#! /bin/false

# Copyright (C) 2019 Guido Flohr <guido.flohr@cantanea.com>,
# all rights reserved.

# This program is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What the Fuck You Want
# to Public License, Version 2, as published by Sam Hocevar. See
# http://www.wtfpl.net/ for more details.

# Make Dist::Zilla happy.
# ABSTRACT: Read chess opening books in polyglot format

package Chess::Opening::ECO;
$Chess::Opening::ECO::VERSION = '0.3';
use common::sense;

use Locale::TextDomain 'com.cantanea.Chess-Opening';

sub positions {
	return {
		'1rbq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP1BPPBP/2R2RK1 b - - 3 12' => {
			eco => 'A39r',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3 a6 11. Bd2 Rb8 12. Rac1

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3 a6 11. Bd2 Rb8 12.Rac1'),
			moves => {
			},
		},
		'1rbq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP1BPPBP/R4RK1 w - - 2 12' => {
			eco => 'A39q',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3 a6 11. Bd2 Rb8

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3 a6 11. Bd2 Rb8'),
			moves => {
				'a1c1' => '1rbq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP1BPPBP/2R2RK1 b - - 3 12',
			},
		},
		'1rbq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/1PN3P1/P1QNPPBP/R1B2RK1 b - - 0 11' => {
			eco => 'E66j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2 Rb8 11. b3

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... a6 10. Qc2 Rb8 11. b3'),
			moves => {
				'b7b5' => '1rbq1rk1/4ppbp/p2p1np1/nppP4/2P5/1PN3P1/P1QNPPBP/R1B2RK1 w - b6 0 12',
			},
		},
		'1rbq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/1RBQ1RK1 w - - 2 11' => {
			eco => 'E66g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Rb1 Rb8

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... a6 10. Rb1 Rb8'),
			moves => {
			},
		},
		'1rbq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PPQNPPBP/R1B2RK1 w - - 2 11' => {
			eco => 'E66i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2 Rb8

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... a6 10. Qc2 Rb8'),
			moves => {
				'b2b3' => '1rbq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/1PN3P1/P1QNPPBP/R1B2RK1 b - - 0 11',
			},
		},
		'1rbq1rk1/1pp1ppb1/p1np1np1/7p/2PPP2P/2N1BP2/PP1QN1P1/2KR1B1R b - - 1 10' => {
			eco => 'E84n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. h4 h5 10. O-O-O

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. h4 h5 10.O-O-O'),
			moves => {
				'b7b5' => '1rbq1rk1/2p1ppb1/p1np1np1/1p5p/2PPP2P/2N1BP2/PP1QN1P1/2KR1B1R w - b6 0 11',
			},
		},
		'1rbq1rk1/1pp1ppb1/p1np1np1/7p/2PPP2P/2N1BP2/PP1QN1P1/R3KB1R w KQ h6 0 10' => {
			eco => 'E84m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. h4 h5

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. h4 h5'),
			moves => {
				'e1c1' => '1rbq1rk1/1pp1ppb1/p1np1np1/7p/2PPP2P/2N1BP2/PP1QN1P1/2KR1B1R b - - 1 10',
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/3N4/2PP4/1P3NP1/P3PPBP/R1BQ1RK1 b - - 2 9' => {
			eco => 'E63g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. b3 Rb8 9. Nd5

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. b3 Rb8 9. Nd5'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E63e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. b3 Rb8

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. b3 Rb8'),
			moves => {
				'c1b2' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 b - - 2 9',
				'c3d5' => '1rbq1rk1/1pp1ppbp/p1np1np1/3N4/2PP4/1P3NP1/P3PPBP/R1BQ1RK1 b - - 2 9',
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 b - - 2 9' => {
			eco => 'E63f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. b3 Rb8 9. Bb2

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. b3 Rb8 9. Bb2'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N1BNPP/PP2PPB1/R2Q1RK1 b - - 2 9' => {
			eco => 'E63j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. h3 Rb8 9. Be3

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. h3 Rb8 9. Be3'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 w - - 1 9' => {
			eco => 'E63i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. h3 Rb8

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. h3 Rb8'),
			moves => {
				'c1e3' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N1BNPP/PP2PPB1/R2Q1RK1 b - - 2 9',
				'e2e4' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 b - e3 0 9',
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP1P1/2N1BP2/PP1QN2P/R3KB1R b KQ g3 0 9' => {
			eco => 'E84d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. g4

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. g4'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP2P/2N1BP2/PP1QN1P1/R3KB1R b KQ h3 0 9' => {
			eco => 'E84k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. h4

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. h4'),
			moves => {
				'b7b5' => '1rbq1rk1/2p1ppbp/p1np1np1/1p6/2PPP2P/2N1BP2/PP1QN1P1/R3KB1R w KQ b6 0 10',
				'h7h5' => '1rbq1rk1/1pp1ppb1/p1np1np1/7p/2PPP2P/2N1BP2/PP1QN1P1/R3KB1R w KQ h6 0 10',
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1Q2PP/R1N1KB1R b KQ - 3 9' => {
			eco => 'E84g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Nc1

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Nc1'),
			moves => {
				'e7e5' => '1rbq1rk1/1pp2pbp/p1np1np1/4p3/2PPP3/2N1BP2/PP1Q2PP/R1N1KB1R w KQ e6 0 10',
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/1R2KB1R b K - 3 9' => {
			eco => 'E84e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Rb1

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Rb1'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/2KR1B1R b - - 3 9' => {
			eco => 'E84c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. O-O-O

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9.O-O-O'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 2 9' => {
			eco => 'E84a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8

			variation => N__('King\'s Indian: Sämisch, Panno Main Line'),
			moves => {
				'a1b1' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/1R2KB1R b K - 3 9',
				'a2a3' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/P1N1BP2/1P1QN1PP/R3KB1R b KQ - 0 9',
				'e1c1' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/2KR1B1R b - - 3 9',
				'e2c1' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1Q2PP/R1N1KB1R b KQ - 3 9',
				'e3h6' => '1rbq1rk1/1pp1ppbp/p1np1npB/8/2PPP3/2N2P2/PP1QN1PP/R3KB1R b KQ - 3 9',
				'g2g4' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP1P1/2N1BP2/PP1QN2P/R3KB1R b KQ g3 0 9',
				'h2h4' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP2P/2N1BP2/PP1QN1P1/R3KB1R b KQ h3 0 9',
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 b - e3 0 9' => {
			eco => 'E63k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. h3 Rb8 9. e4

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. h3 Rb8 9. e4'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/P1N1BP2/1P1QN1PP/R3KB1R b KQ - 0 9' => {
			eco => 'E84b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. a3

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. a3'),
			moves => {
			},
		},
		'1rbq1rk1/1pp1ppbp/p1np1npB/8/2PPP3/2N2P2/PP1QN1PP/R3KB1R b KQ - 3 9' => {
			eco => 'E84f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Bh6

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Bh6'),
			moves => {
			},
		},
		'1rbq1rk1/1pp2pbp/p1np1np1/3Pp3/2P1P3/2N1BP2/PP1Q2PP/R1N1KB1R b KQ - 0 10' => {
			eco => 'E84i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Nc1 e5 10. d5

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Nc1 e5 10. d5'),
			moves => {
				'c6d4' => '1rbq1rk1/1pp2pbp/p2p1np1/3Pp3/2PnP3/2N1BP2/PP1Q2PP/R1N1KB1R w KQ - 1 11',
			},
		},
		'1rbq1rk1/1pp2pbp/p1np1np1/4p3/2PPP3/1NN1BP2/PP1Q2PP/R3KB1R b KQ - 1 10' => {
			eco => 'E84h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Nc1 e5 10. Nb3

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Nc1 e5 10. Nb3'),
			moves => {
			},
		},
		'1rbq1rk1/1pp2pbp/p1np1np1/4p3/2PPP3/2N1BP2/PP1Q2PP/R1N1KB1R w KQ e6 0 10' => {
			eco => 'E84g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Nc1 e5

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Nc1 e5'),
			moves => {
				'c1b3' => '1rbq1rk1/1pp2pbp/p1np1np1/4p3/2PPP3/1NN1BP2/PP1Q2PP/R3KB1R b KQ - 1 10',
				'd4d5' => '1rbq1rk1/1pp2pbp/p1np1np1/3Pp3/2P1P3/2N1BP2/PP1Q2PP/R1N1KB1R b KQ - 0 10',
			},
		},
		'1rbq1rk1/1pp2pbp/p2p1np1/3Pp3/2PnP3/1NN1BP2/PP1Q2PP/R3KB1R b KQ - 2 11' => {
			eco => 'E84j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. Nc1 e5 10. d5 Nd4 11. Nb3

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Nc1 e5 10. d5 Nd4 11. Nb3'),
			moves => {
			},
		},
		'1rbq1rk1/1pp2pbp/p2p1np1/3Pp3/2PnP3/2N1BP2/PP1Q2PP/R1N1KB1R w KQ - 1 11' => {
			eco => 'E84i',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. Nc1 e5 10. d5'),
			moves => {
				'c1b3' => '1rbq1rk1/1pp2pbp/p2p1np1/3Pp3/2PnP3/1NN1BP2/PP1Q2PP/R3KB1R b KQ - 2 11',
			},
		},
		'1rbq1rk1/2p1ppb1/p1np1np1/1p5p/2PPP2P/2N1BP2/PP1QN1P1/2KR1B1R w - b6 0 11' => {
			eco => 'E84n',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. h4 h5 10.O-O-O'),
			moves => {
				'e3h6' => '1rbq1rk1/2p1ppb1/p1np1npB/1p5p/2PPP2P/2N2P2/PP1QN1P1/2KR1B1R b - - 1 11',
			},
		},
		'1rbq1rk1/2p1ppb1/p1np1npB/1p5p/2PPP2P/2N2P2/PP1QN1P1/2KR1B1R b - - 1 11' => {
			eco => 'E84o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. h4 h5 10. O-O-O b5 11. Bh6

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. h4 h5 10.O-O-O b5 11. Bh6'),
			moves => {
			},
		},
		'1rbq1rk1/2p1ppbp/p1np1np1/1p6/2PPP2P/2N1BP2/PP1QN1P1/R3KB1R w KQ b6 0 10' => {
			eco => 'E84l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Rb8 9. h4 b5

			variation => N__('King\'s Indian: Sämisch, Panno Main Line, 9. h4 b5'),
			moves => {
			},
		},
		'1rbq1rk1/4ppbp/p2p1np1/n1pP4/2p5/1PN3P1/PBQNPPBP/R4RK1 w - - 0 13' => {
			eco => 'E66n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2 Rb8 11. b3 b5 12. Bb2 bxc4

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, Main Line, 12. Bb2 bxc4'),
			moves => {
			},
		},
		'1rbq1rk1/4ppbp/p2p1np1/nppP4/2P5/1PN3P1/P1QNPPBP/R1B2RK1 w - b6 0 12' => {
			eco => 'E66k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2 Rb8 11. b3 b5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, Main Line'),
			moves => {
				'c1b2' => '1rbq1rk1/4ppbp/p2p1np1/nppP4/2P5/1PN3P1/PBQNPPBP/R4RK1 b - - 1 12',
			},
		},
		'1rbq1rk1/4ppbp/p2p1np1/nppP4/2P5/1PN3P1/PBQNPPBP/R4RK1 b - - 1 12' => {
			eco => 'E66l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2 Rb8 11. b3 b5 12. Bb2

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, Main Line, 12. Bb2'),
			moves => {
				'b5c4' => '1rbq1rk1/4ppbp/p2p1np1/n1pP4/2p5/1PN3P1/PBQNPPBP/R4RK1 w - - 0 13',
				'e7e5' => '1rbq1rk1/5pbp/p2p1np1/nppPp3/2P5/1PN3P1/PBQNPPBP/R4RK1 w - e6 0 13',
			},
		},
		'1rbq1rk1/5pbp/p2p1np1/nppPp3/2P5/1PN3P1/PBQNPPBP/R4RK1 w - e6 0 13' => {
			eco => 'E66m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2 Rb8 11. b3 b5 12. Bb2 e5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, Main Line, 12. Bb2 e5'),
			moves => {
			},
		},
		'1rbq1rk1/pp2npbp/2npp1p1/2p5/4PP2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 6 10' => {
			eco => 'B25r',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 Rb8

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 Rb8'),
			moves => {
			},
		},
		'1rbq1rk1/pp2ppbp/2np1np1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A36x',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 Nf6 7. O-O O-O 8. d3 Rb8

			variation => N__('English: Symmetrical, Botvinnik System, 5... d6, 8. d3 Rb8'),
			moves => {
			},
		},
		'1rbq1rk1/pp2ppbp/3p1np1/8/2P5/2NQ2P1/PP2PPBP/R1B2RK1 w - - 2 11' => {
			eco => 'A39o',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3 Rb8

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3 Rb8'),
			moves => {
			},
		},
		'1rbq1rk1/ppp1ppbp/2np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 8' => {
			eco => 'E62o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O Rb8

			variation => N__('King\'s Indian: Fianchetto, 6... Nc6 7.O-O Rb8'),
			moves => {
			},
		},
		'1rbq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 6 8' => {
			eco => 'E83e',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Ruban Variation'),
			moves => {
				'f8e8' => '1rbqr1k1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 7 9',
			},
		},
		'1rbq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 5 8' => {
			eco => 'E83e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 Rb8

			variation => N__('King\'s Indian: Sämisch, Ruban Variation'),
			moves => {
				'd1d2' => '1rbq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 6 8',
			},
		},
		'1rbqk1nr/p3ppbp/2np2p1/1pp5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQk b6 0 8' => {
			eco => 'B26i',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 Rb8 7. Qd2 b5

			variation => N__('Sicilian: Closed, 6. Be3 Rb8 7. Qd2 b5'),
			moves => {
				'g1e2' => '1rbqk1nr/p3ppbp/2np2p1/1pp5/4P3/2NPB1P1/PPPQNPBP/R3K2R b KQk - 1 8',
			},
		},
		'1rbqk1nr/p3ppbp/2np2p1/1pp5/4P3/2NPB1P1/PPPQNPBP/R3K2R b KQk - 1 8' => {
			eco => 'B26j',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 Rb8 7. Qd2 b5 8. Nge2

			variation => N__('Sicilian: Closed, 6. Be3 Rb8 7. Qd2 b5 8.Nge2'),
			moves => {
			},
		},
		'1rbqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQk - 2 7' => {
			eco => 'B26g',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 Rb8

			variation => N__('Sicilian: Closed, 6. Be3 Rb8'),
			moves => {
				'd1d2' => '1rbqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR b KQk - 3 7',
			},
		},
		'1rbqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR b KQk - 3 7' => {
			eco => 'B26h',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 Rb8 7. Qd2

			variation => N__('Sicilian: Closed, 6. Be3 Rb8 7. Qd2'),
			moves => {
				'b7b5' => '1rbqk1nr/p3ppbp/2np2p1/1pp5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQk b6 0 8',
			},
		},
		'1rbqk1nr/pp2ppbp/2np2p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQk - 1 7' => {
			eco => 'B25f',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 Rb8

			variation => N__('Sicilian: Closed, 6. f4 Rb8'),
			moves => {
			},
		},
		'1rbqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQk - 2 8' => {
			eco => 'B26p',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6 7. Qd2 Rb8

			variation => N__('Sicilian: Closed, 6. Be3 e6 7. Qd2 Rb8'),
			moves => {
				'g1e2' => '1rbqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQNPBP/R3K2R b KQk - 3 8',
			},
		},
		'1rbqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQNPBP/R3K2R b KQk - 3 8' => {
			eco => 'B26q',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6 7. Qd2 Rb8 8. Nge2

			variation => N__('Sicilian: Closed, 6. Be3 e6 7. Qd2 Rb8 8.Nge2'),
			moves => {
			},
		},
		'1rbqk2r/2pp1ppp/p1n2n2/1pb1p3/P3P3/1B3N2/1PPP1PPP/RNBQ1RK1 w k - 1 8' => {
			eco => 'C78m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. a4 Rb8

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7. a4 Rb8'),
			moves => {
			},
		},
		'1rbqkb1r/p4ppp/2p2n2/nB2p1N1/8/5Q2/PPPP1PPP/RNB1K2R w KQk - 2 9' => {
			eco => 'C58g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Qf3 Rb8

			variation => N__('Two Knights: Morphy, Colman Variation'),
			moves => {
			},
		},
		'1rbqkb1r/ppp2ppp/2n1pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQ1RK1 b k - 5 7' => {
			eco => 'E04u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6 6. O-O Rb8 7. Nc3

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6.O-O Rb8 7. Nc3'),
			moves => {
			},
		},
		'1rbqkb1r/ppp2ppp/2n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w k - 4 7' => {
			eco => 'E04t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6 6. O-O Rb8

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6.O-O Rb8'),
			moves => {
				'b1c3' => '1rbqkb1r/ppp2ppp/2n1pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQ1RK1 b k - 5 7',
			},
		},
		'1rbqr1k1/1p1n1pbp/p2p1np1/2pP4/P1N5/2N3PP/1P2PPB1/R1BQ1RK1 b - - 2 13' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. h3 Rb8 13. Nc4

			variation => N__('Benoni: Fianchetto, 11... Re8 12. h3 Rb8 13. Nc4'),
			moves => {
				'd7b6' => '1rbqr1k1/1p3pbp/pn1p1np1/2pP4/P1N5/2N3PP/1P2PPB1/R1BQ1RK1 w - - 3 14',
				'd7e5' => '1rbqr1k1/1p3pbp/p2p1np1/2pPn3/P1N5/2N3PP/1P2PPB1/R1BQ1RK1 w - - 3 14',
			},
		},
		'1rbqr1k1/1p1n1pbp/p2p1np1/2pP4/P7/2N3PP/1P1NPPB1/R1BQ1RK1 w - - 1 13' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. h3 Rb8

			variation => N__('Benoni: Fianchetto, 11... Re8 12. h3 Rb8'),
			moves => {
				'd2c4' => '1rbqr1k1/1p1n1pbp/p2p1np1/2pP4/P1N5/2N3PP/1P2PPB1/R1BQ1RK1 b - - 2 13',
			},
		},
		'1rbqr1k1/1p3pbp/p2p1np1/2pPn3/P1N5/2N3PP/1P2PPB1/R1BQ1RK1 w - - 3 14' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. h3 Rb8 13. Nc4 Ne5

			variation => N__('Benoni: Fianchetto, 11... Re8 12. h3 Rb8 13. Nc4 Ne5'),
			moves => {
			},
		},
		'1rbqr1k1/1p3pbp/pn1p1np1/2pP4/P1N5/2N3PP/1P2PPB1/R1BQ1RK1 w - - 3 14' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. h3 Rb8 13. Nc4 Nb6

			variation => N__('Benoni: Fianchetto, 11... Re8 12. h3 Rb8 13. Nc4 Nb6'),
			moves => {
			},
		},
		'1rbqr1k1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 7 9' => {
			eco => 'E83e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 Rb8 8. Qd2 Re8

			variation => N__('King\'s Indian: Sämisch, Ruban, 8. Qd2 Re8'),
			moves => {
			},
		},
		'2bqkbnr/r4ppp/p1npp3/1p6/4P3/1BN2N2/PP2QPPP/R1B2RK1 w k - 2 10' => {
			eco => 'B21y',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4 e6 7. O-O a6 8. Qe2 b5 9. Bb3 Ra7

			variation => N__('Sicilian: Smith-Morra Gambit, Chicago Defence'),
			moves => {
			},
		},
		'2kr1b1r/ppp2ppp/2n1N3/q7/3p4/5N2/PPP2PPP/R1BQR1K1 b - - 0 11' => {
			eco => 'C56j',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10.Neg5'),
			moves => {
				'f7e6' => '2kr1b1r/ppp3pp/2n1p3/q7/3p4/5N2/PPP2PPP/R1BQR1K1 w - - 0 12',
			},
		},
		'2kr1b1r/ppp2ppp/2n1b3/q5N1/3p4/5N2/PPP2PPP/R1BQR1K1 w - - 3 11' => {
			eco => 'C56j',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10.Neg5'),
			moves => {
				'g5e6' => '2kr1b1r/ppp2ppp/2n1N3/q7/3p4/5N2/PPP2PPP/R1BQR1K1 b - - 0 11',
			},
		},
		'2kr1b1r/ppp3pp/2n1R3/q7/3p4/5N2/PPP2PPP/R1BQ2K1 b - - 0 12' => {
			eco => 'C56j',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10.Neg5'),
			moves => {
				'f8d6' => '2kr3r/ppp3pp/2nbR3/q7/3p4/5N2/PPP2PPP/R1BQ2K1 w - - 1 13',
			},
		},
		'2kr1b1r/ppp3pp/2n1p3/q7/3p4/5N2/PPP2PPP/R1BQR1K1 w - - 0 12' => {
			eco => 'C56j',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10.Neg5'),
			moves => {
				'e1e6' => '2kr1b1r/ppp3pp/2n1R3/q7/3p4/5N2/PPP2PPP/R1BQ2K1 b - - 0 12',
			},
		},
		'2kr1b1r/ppqn1pp1/2p1p2p/7P/3Pn3/3Q1NP1/PPPB1P2/2KR3R w - - 0 15' => {
			eco => 'B19t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2 e6 12. O-O-O Ngf6 13. Ne4 O-O-O 14. g3 Nxe4

			variation => N__('Caro-Kann: Classical, Spassky, Main Line, 14. g3 Nxe4'),
			moves => {
			},
		},
		'2kr1b1r/ppqn1pp1/2p1pn1p/7P/3PN3/3Q1N2/PPPB1PP1/2KR3R w - - 4 14' => {
			eco => 'B19s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2 e6 12. O-O-O Ngf6 13. Ne4 O-O-O

			variation => N__('Caro-Kann: Classical, Spassky, Main Line, 13. Ne4 O-O-O'),
			moves => {
				'g2g3' => '2kr1b1r/ppqn1pp1/2p1pn1p/7P/3PN3/3Q1NP1/PPPB1P2/2KR3R b - - 0 14',
			},
		},
		'2kr1b1r/ppqn1pp1/2p1pn1p/7P/3PN3/3Q1NP1/PPPB1P2/2KR3R b - - 0 14' => {
			eco => 'B19s',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Classical, Spassky, Main Line, 13. Ne4 O-O-O'),
			moves => {
				'f6e4' => '2kr1b1r/ppqn1pp1/2p1p2p/7P/3Pn3/3Q1NP1/PPPB1P2/2KR3R w - - 0 15',
			},
		},
		'2kr1bnr/pppq1ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPPBP/R1BQK2R w KQ - 5 8' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. Nbd2 Qd7 7. Bg2 O-O-O

			variation => N__('QGD: Albin, 5. g3 Be6 6.Nbd2 Qd7 7. Bg2 O-O-O'),
			moves => {
			},
		},
		'2kr1bnr/pppq1ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQ1RK1 w - - 5 8' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Bg4 6. Bg2 Qd7 7. O-O O-O-O

			variation => N__('QGD: Albin, 5. g3 Bg4 6. Bg2 Qd7 7.O-O O-O-O'),
			moves => {
			},
		},
		'2kr3r/pp1bqppp/2n2n2/2b5/8/2N1PN2/PP2BPPP/R1BQK2R w KQ - 3 12' => {
			eco => 'D32i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qa4+ Bd7 6. Qxd4 exd5 7. Qxd5 Nc6 8. Nf3 Nf6 9. Qd1 Bc5 10. e3 Qe7 11. Be2 O-O-O

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, Main Line (11. Be2 O-O-O)'),
			moves => {
			},
		},
		'2kr3r/pp3ppp/n1p1p3/3q4/P1N5/8/1PK1Q1PP/R4B1R w - - 1 16' => {
			eco => 'D17t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. e4 Bxe4 9. fxe4 Nxe4 10. Bd2 Qxd4 11. Nxe4 Qxe4+ 12. Qe2 Bxd2+ 13. Kxd2 Qd5+ 14. Kc2 Na6 15. Nxc4 O-O-O

			variation => N__('Slav: Central, Main Line, 15.Nxc4 O-O-O'),
			moves => {
			},
		},
		'2kr3r/ppp2ppp/1bn1bPq1/6N1/2ppNPP1/8/PPP4P/R1BQR1K1 w - - 1 14' => {
			eco => 'C55x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 d5 7. exf6 dxc4 8. Re1+ Be6 9. Ng5 Qd5 10. Nc3 Qf5 11. g4 Qg6 12. Nce4 Bb6 13. f4 O-O-O

			variation => N__('Two Knights: Max Lange, Berger Variation'),
			moves => {
			},
		},
		'2kr3r/ppp3pp/2nbR3/q7/3p4/5N2/PPP2PPP/R1BQ2K1 w - - 1 13' => {
			eco => 'C56k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Neg5 O-O-O 11. Nxe6 fxe6 12. Rxe6 Bd6

			variation => N__('Two Knights: Classical, Moller, 10.Neg5 O-O-O 11.Nxe6 fxe6 12.Rxe6 Bd6'),
			moves => {
			},
		},
		'2r1kb1r/pp1bpppp/3p1n2/q5B1/3QP3/2N5/PPP2PPP/2KR1B1R w k - 1 10' => {
			eco => 'B61g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Rc8 8. O-O-O Nxd4 9. Qxd4 Qa5

			variation => N__('Sicilian: Richter-Rauzer, Larsen, Main Line'),
			moves => {
				'f2f4' => '2r1kb1r/pp1bpppp/3p1n2/q5B1/3QPP2/2N5/PPP3PP/2KR1B1R b k f3 0 10',
				'g5d2' => '2r1kb1r/pp1bpppp/3p1n2/q7/3QP3/2N5/PPPB1PPP/2KR1B1R b k - 2 10',
			},
		},
		'2r1kb1r/pp1bpppp/3p1n2/q5B1/3QPP2/2N5/PPP3PP/2KR1B1R b k f3 0 10' => {
			eco => 'B61i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Rc8 8. O-O-O Nxd4 9. Qxd4 Qa5 10. f4

			variation => N__('Sicilian: Richter-Rauzer, Larsen, Main Line, 10. f4'),
			moves => {
			},
		},
		'2r1kb1r/pp1bpppp/3p1n2/q7/3QP3/2N5/PPPB1PPP/2KR1B1R b k - 2 10' => {
			eco => 'B61h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Rc8 8. O-O-O Nxd4 9. Qxd4 Qa5 10. Bd2

			variation => N__('Sicilian: Richter-Rauzer, Larsen, Main Line, 10. Bd2'),
			moves => {
			},
		},
		'2r2rk1/1bq1bppp/p2p1n2/np1Pp3/4P3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 15' => {
			eco => 'C99h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bb7 14. d5 Rac8

			variation => N__('Spanish: Closed, Chigorin, 13... Bb7 14. d5 Rac8'),
			moves => {
			},
		},
		'2r2rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPB1RPP1/R1BQ1NK1 b - - 4 15' => {
			eco => 'C99f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bb7 14. Nf1 Rac8 15. Re2

			variation => N__('Spanish: Closed, Chigorin, 13... Bb7 14. Nf1 Rac8 15. Re2'),
			moves => {
			},
		},
		'2r2rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 w - - 3 15' => {
			eco => 'C99e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bb7 14. Nf1 Rac8

			variation => N__('Spanish: Closed, Chigorin, 13... Bb7 14. Nf1 Rac8'),
			moves => {
				'e1e2' => '2r2rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPB1RPP1/R1BQ1NK1 b - - 4 15',
			},
		},
		'2r2rk1/2qbbppp/p2p1n2/np2p3/3PP3/4NN1P/PPB2PP1/R1BQR1K1 b - - 4 15' => {
			eco => 'C99d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bd7 14. Nf1 Rac8 15. Ne3

			variation => N__('Spanish: Closed, Chigorin, 13... Bd7 14. Nf1 Rac8 15. Ne3'),
			moves => {
			},
		},
		'2r2rk1/2qbbppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 w - - 3 15' => {
			eco => 'C99c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Chigorin, 13... Bd7 14. Nf1'),
			moves => {
				'f1e3' => '2r2rk1/2qbbppp/p2p1n2/np2p3/3PP3/4NN1P/PPB2PP1/R1BQR1K1 b - - 4 15',
			},
		},
		'2r2rk1/pbqnbppp/1pp1pn2/3p4/2PPP3/1P3NP1/PBQN1PBP/R4RK1 w - - 3 12' => {
			eco => 'E09p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Bb7 10. b3 Rc8 11. Bb2 Qc7

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7 10. b3 Rc8 11. Bb2 Qc7'),
			moves => {
			},
		},
		'2r2rk1/pp1bppbp/2np1np1/q7/3NP3/1BN1BP2/PPPQ2PP/2KR3R w - - 9 12' => {
			eco => 'B78s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rac8

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 11. Bb3 Rac8'),
			moves => {
			},
		},
		'2rq1rk1/pb1nbppp/1p2pn2/2pp4/2PPP3/1P3NP1/PBQN1PBP/R4RK1 w - - 0 12' => {
			eco => 'E09q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Bb7 10. b3 Rc8 11. Bb2 c5

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7 10. b3 Rc8 11. Bb2 c5'),
			moves => {
			},
		},
		'2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 w - - 3 11' => {
			eco => 'E09g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. b3 Bb7 10. Bb2 Rc8

			variation => N__('Catalan: Closed, Main Line, 9. b3 Bb7 10. Bb2 Rc8'),
			moves => {
			},
		},
		'2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1PN2NP1/P1Q1PPBP/R1BR2K1 w - - 4 11' => {
			eco => 'E08q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 Rc8

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 Rc8'),
			moves => {
				'e2e4' => '2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1PN2NP1/P1Q2PBP/R1BR2K1 b - e3 0 11',
			},
		},
		'2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1P3NP1/P1QN1PBP/R1B2RK1 w - - 1 11' => {
			eco => 'E09n',
                        # TRANSLATORS:

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7 10. b3'),
			moves => {
				'c1b2' => '2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1P3NP1/PBQN1PBP/R4RK1 b - - 2 11',
			},
		},
		'2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1P3NP1/PBQN1PBP/R4RK1 b - - 2 11' => {
			eco => 'E09o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Bb7 10. b3 Rc8 11. Bb2

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7 10. b3 Rc8 11. Bb2'),
			moves => {
				'c6c5' => '2rq1rk1/pb1nbppp/1p2pn2/2pp4/2PPP3/1P3NP1/PBQN1PBP/R4RK1 w - - 0 12',
				'd8c7' => '2r2rk1/pbqnbppp/1pp1pn2/3p4/2PPP3/1P3NP1/PBQN1PBP/R4RK1 w - - 3 12',
			},
		},
		'2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1PN2NP1/P1Q2PBP/R1BR2K1 b - e3 0 11' => {
			eco => 'E08q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 Rc8 11. e4

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 Rc8 11. e4'),
			moves => {
				'd5e4' => '2rq1rk1/pb1nbppp/1pp1pn2/8/2PPp3/1PN2NP1/P1Q2PBP/R1BR2K1 w - - 0 12',
			},
		},
		'2rq1rk1/pb1nbppp/1pp1pn2/8/2PPp3/1PN2NP1/P1Q2PBP/R1BR2K1 w - - 0 12' => {
			eco => 'E08r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 Rc8 11. e4 dxe4

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 Rc8 11. e4 dxe4'),
			moves => {
			},
		},
		'2rq1rk1/pp1bpp2/3p1npb/4n2p/3NP2P/1BN2P2/PPPQ2P1/2KR3R w - - 0 14' => {
			eco => 'B78m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Bh6 Bxh6

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Bh6 Bxh6'),
			moves => {
			},
		},
		'2rq1rk1/pp1bppb1/3p1np1/4n1Bp/3NP2P/1BN2P2/PPPQ2P1/2KR3R b - - 1 13' => {
			eco => 'B78n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Bg5

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Bg5'),
			moves => {
				'c8c5' => '3q1rk1/pp1bppb1/3p1np1/2r1n1Bp/3NP2P/1BN2P2/PPPQ2P1/2KR3R w - - 2 14',
			},
		},
		'2rq1rk1/pp1bppb1/3p1np1/4n2p/3NP2P/1BN1BP2/PPPQ2P1/1K1R3R b - - 1 13' => {
			eco => 'B78k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Kb1

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Kb1'),
			moves => {
			},
		},
		'2rq1rk1/pp1bppb1/3p1np1/4n2p/3NP2P/1BN1BP2/PPPQ2P1/2KR3R w - h6 0 13' => {
			eco => 'B78j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5'),
			moves => {
				'c1b1' => '2rq1rk1/pp1bppb1/3p1np1/4n2p/3NP2P/1BN1BP2/PPPQ2P1/1K1R3R b - - 1 13',
				'e3g5' => '2rq1rk1/pp1bppb1/3p1np1/4n1Bp/3NP2P/1BN2P2/PPPQ2P1/2KR3R b - - 1 13',
				'e3h6' => '2rq1rk1/pp1bppb1/3p1npB/4n2p/3NP2P/1BN2P2/PPPQ2P1/2KR3R b - - 1 13',
			},
		},
		'2rq1rk1/pp1bppb1/3p1npB/4n2p/3NP2P/1BN2P2/PPPQ2P1/2KR3R b - - 1 13' => {
			eco => 'B78l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Bh6

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Bh6'),
			moves => {
				'g7h6' => '2rq1rk1/pp1bpp2/3p1npb/4n2p/3NP2P/1BN2P2/PPPQ2P1/2KR3R w - - 0 14',
			},
		},
		'2rq1rk1/pp1bppbp/2np1np1/8/2BNP2P/2N1BP2/PPPQ2P1/R3K2R w KQ - 1 11' => {
			eco => 'B77j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. h4 Rc8

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Bd7 10. h4 Rc8'),
			moves => {
			},
		},
		'2rq1rk1/pp1bppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/2KR3R w - - 7 11' => {
			eco => 'B78c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8

			variation => N__('Sicilian: Dragon, Yugoslav, 10.O-O-O Rc8'),
			moves => {
				'c4b3' => '2rq1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1BP2/PPPQ2PP/2KR3R b - - 8 11',
			},
		},
		'2rq1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1BP2/PPPQ2PP/2KR3R b - - 8 11' => {
			eco => 'B78c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3

			variation => N__('Sicilian: Dragon, Yugoslav, 10.O-O-O Rc8 11. Bb3'),
			moves => {
				'c6e5' => '2rq1rk1/pp1bppbp/3p1np1/4n3/3NP3/1BN1BP2/PPPQ2PP/2KR3R w - - 9 12',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/4n3/3NP2P/1BN1BP2/PPPQ2P1/2KR3R b - h3 0 12' => {
			eco => 'B78g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4'),
			moves => {
				'e5c4' => '2rq1rk1/pp1bppbp/3p1np1/8/2nNP2P/1BN1BP2/PPPQ2P1/2KR3R w - - 1 13',
				'h7h5' => '2rq1rk1/pp1bppb1/3p1np1/4n2p/3NP2P/1BN1BP2/PPPQ2P1/2KR3R w - h6 0 13',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/4n3/3NP3/1BN1BP2/PPPQ2PP/1K1R3R b - - 10 12' => {
			eco => 'B78e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. Kb1

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. Kb1'),
			moves => {
				'e5c4' => '2rq1rk1/pp1bppbp/3p1np1/8/2nNP3/1BN1BP2/PPPQ2PP/1K1R3R w - - 11 13',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/4n3/3NP3/1BN1BP2/PPPQ2PP/2KR3R w - - 9 12' => {
			eco => 'B78d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5

			variation => N__('Sicilian: Dragon, Yugoslav, Modern Main Line'),
			moves => {
				'c1b1' => '2rq1rk1/pp1bppbp/3p1np1/4n3/3NP3/1BN1BP2/PPPQ2PP/1K1R3R b - - 10 12',
				'h2h4' => '2rq1rk1/pp1bppbp/3p1np1/4n3/3NP2P/1BN1BP2/PPPQ2P1/2KR3R b - h3 0 12',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/8/2BNP2P/2N1BP2/PPPQ2P1/2KR3R b - - 0 13' => {
			eco => 'B78g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 Nc4'),
			moves => {
				'c8c4' => '3q1rk1/pp1bppbp/3p1np1/8/2rNP2P/2N1BP2/PPPQ2P1/2KR3R w - - 0 14',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/8/2BNP3/2N1BP2/PPPQ2PP/1K1R3R b - - 0 13' => {
			eco => 'B78e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. Kb1'),
			moves => {
				'c8c4' => '3q1rk1/pp1bppbp/3p1np1/8/2rNP3/2N1BP2/PPPQ2PP/1K1R3R w - - 0 14',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/8/2nNP2P/1BN1BP2/PPPQ2P1/2KR3R w - - 1 13' => {
			eco => 'B78g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 Nc4

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 Nc4'),
			moves => {
				'b3c4' => '2rq1rk1/pp1bppbp/3p1np1/8/2BNP2P/2N1BP2/PPPQ2P1/2KR3R b - - 0 13',
			},
		},
		'2rq1rk1/pp1bppbp/3p1np1/8/2nNP3/1BN1BP2/PPPQ2PP/1K1R3R w - - 11 13' => {
			eco => 'B78e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. Kb1'),
			moves => {
				'b3c4' => '2rq1rk1/pp1bppbp/3p1np1/8/2BNP3/2N1BP2/PPPQ2PP/1K1R3R b - - 0 13',
			},
		},
		'2rq1rk1/pp2ppbp/2npbnp1/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 11' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4 Rc8

			variation => N__('Sicilian: Dragon, Classical, 9. Nb3 Be6 10. f4 Rc8'),
			moves => {
			},
		},
		'2rqk2r/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R w k - 5 10' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 Nc6 8. Qd2 Bd7 9. O-O-O Rc8

			variation => N__('Sicilian: Dragon, Yugoslav, 7... Nc6 8. Qd2 Bd7 9.O-O-O Rc8'),
			moves => {
			},
		},
		'2rqkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R w k - 1 10' => {
			eco => 'B67d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f3 Rc8

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f3 Rc8'),
			moves => {
			},
		},
		'2rqkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b k - 8 8' => {
			eco => 'B61f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Rc8 8. O-O-O

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 Rc8 8.O-O-O'),
			moves => {
				'c6d4' => '2rqkb1r/pp1bpppp/3p1n2/6B1/3nP3/2N5/PPPQ1PPP/2KR1B1R w k - 0 9',
			},
		},
		'2rqkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQk - 7 8' => {
			eco => 'B61d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Rc8

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 Rc8'),
			moves => {
				'e1c1' => '2rqkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b k - 8 8',
				'f2f4' => '2rqkb1r/pp1bpppp/2np1n2/6B1/3NPP2/2N5/PPPQ2PP/R3KB1R b KQk f3 0 8',
			},
		},
		'2rqkb1r/pp1bpppp/2np1n2/6B1/3NPP2/2N5/PPPQ2PP/R3KB1R b KQk f3 0 8' => {
			eco => 'B61e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Rc8 8. f4

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 Rc8 8. f4'),
			moves => {
			},
		},
		'2rqkb1r/pp1bpppp/3p1n2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R b k - 0 9' => {
			eco => 'B61f',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 Rc8 8.O-O-O'),
			moves => {
				'd8a5' => '2r1kb1r/pp1bpppp/3p1n2/q5B1/3QP3/2N5/PPP2PPP/2KR1B1R w k - 1 10',
			},
		},
		'2rqkb1r/pp1bpppp/3p1n2/6B1/3nP3/2N5/PPPQ1PPP/2KR1B1R w k - 0 9' => {
			eco => 'B61f',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 Rc8 8.O-O-O'),
			moves => {
				'd2d4' => '2rqkb1r/pp1bpppp/3p1n2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R b k - 0 9',
			},
		},
		'3q1rk1/3nppbp/r2p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQ3R w - - 1 13' => {
			eco => 'A59o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3 Ra6

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3 Ra6'),
			moves => {
				'h1e1' => '3q1rk1/3nppbp/r2p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQR3 b - - 2 13',
			},
		},
		'3q1rk1/3nppbp/r2p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQR3 b - - 2 13' => {
			eco => 'A59o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3 Ra6 13. Re1

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3 Ra6 13. Re1'),
			moves => {
			},
		},
		'3q1rk1/p2bppbp/3p1np1/1p6/2rNP1P1/2N1BP2/PPPQ3P/1K1R3R w - b6 0 15' => {
			eco => 'B78f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. Kb1 Nc4 13. Bxc4 Rxc4 14. g4 b5

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. Kb1: 14. g4 b5'),
			moves => {
			},
		},
		'3q1rk1/pp1bppb1/3p1np1/2r1n1Bp/3NP1PP/1BN2P2/PPPQ4/2KR3R b - g3 0 14' => {
			eco => 'B78p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Bg5 Rc5 14. g4

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Bg5 Rc5 14. g4'),
			moves => {
			},
		},
		'3q1rk1/pp1bppb1/3p1np1/2r1n1Bp/3NP2P/1BN2P2/PPPQ2P1/1K1R3R b - - 3 14' => {
			eco => 'B78q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Bg5 Rc5 14. Kb1

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Bg5 Rc5 14. Kb1'),
			moves => {
			},
		},
		'3q1rk1/pp1bppb1/3p1np1/2r1n1Bp/3NP2P/1BN2P2/PPPQ2P1/2KR3R w - - 2 14' => {
			eco => 'B78o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 h5 13. Bg5 Rc5

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 h5 13. Bg5 Rc5'),
			moves => {
				'c1b1' => '3q1rk1/pp1bppb1/3p1np1/2r1n1Bp/3NP2P/1BN2P2/PPPQ2P1/1K1R3R b - - 3 14',
				'g2g4' => '3q1rk1/pp1bppb1/3p1np1/2r1n1Bp/3NP1PP/1BN2P2/PPPQ4/2KR3R b - g3 0 14',
			},
		},
		'3q1rk1/pp1bppbp/3p1np1/7P/2rNP3/2N1BP2/PPPQ2P1/2KR3R b - - 0 14' => {
			eco => 'B78i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 Nc4 13. Bxc4 Rxc4 14. h5

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 Nc4: 14. h5'),
			moves => {
			},
		},
		'3q1rk1/pp1bppbp/3p1np1/8/2rNP1P1/2N1BP2/PPPQ3P/1K1R3R b - g3 0 14' => {
			eco => 'B78e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. Kb1'),
			moves => {
				'b7b5' => '3q1rk1/p2bppbp/3p1np1/1p6/2rNP1P1/2N1BP2/PPPQ3P/1K1R3R w - b6 0 15',
			},
		},
		'3q1rk1/pp1bppbp/3p1np1/8/2rNP1PP/2N1BP2/PPPQ4/2KR3R b - g3 0 14' => {
			eco => 'B78h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. h4 Nc4 13. Bxc4 Rxc4 14. g4

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 Nc4: 14. g4'),
			moves => {
			},
		},
		'3q1rk1/pp1bppbp/3p1np1/8/2rNP2P/2N1BP2/PPPQ2P1/2KR3R w - - 0 14' => {
			eco => 'B78g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. h4 Nc4'),
			moves => {
				'g2g4' => '3q1rk1/pp1bppbp/3p1np1/8/2rNP1PP/2N1BP2/PPPQ4/2KR3R b - g3 0 14',
				'h4h5' => '3q1rk1/pp1bppbp/3p1np1/7P/2rNP3/2N1BP2/PPPQ2P1/2KR3R b - - 0 14',
			},
		},
		'3q1rk1/pp1bppbp/3p1np1/8/2rNP3/2N1BP2/PPPQ2PP/1K1R3R w - - 0 14' => {
			eco => 'B78e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Rc8 11. Bb3 Ne5 12. Kb1 Nc4 13. Bxc4 Rxc4

			variation => N__('Sicilian: Dragon, Yugoslav, Main Line, 12. Kb1'),
			moves => {
				'g2g4' => '3q1rk1/pp1bppbp/3p1np1/8/2rNP1P1/2N1BP2/PPPQ3P/1K1R3R b - g3 0 14',
			},
		},
		'4rrk1/5ppp/p1pb4/1p1n4/3P2b1/1BPQB1Pq/PP1N1P1P/R3R1K1 b - - 6 17' => {
			eco => 'C89q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2

			variation => N__('Spanish: Marshall, Main Line, 16. Qd3 Rae8 17. Nd2'),
			moves => {
				'e8e6' => '5rk1/5ppp/p1pbr3/1p1n4/3P2b1/1BPQB1Pq/PP1N1P1P/R3R1K1 w - - 7 18',
				'f7f5' => '4rrk1/6pp/p1pb4/1p1n1p2/3P2b1/1BPQB1Pq/PP1N1P1P/R3R1K1 w - f6 0 18',
			},
		},
		'4rrk1/5ppp/p1pb4/1p1n4/3P2b1/1BPQB1Pq/PP3P1P/RN2R1K1 w - - 5 17' => {
			eco => 'C89q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8

			variation => N__('Spanish: Marshall, Main Line, 15. Be3 Bg4 16. Qd3 Rae8'),
			moves => {
				'b1d2' => '4rrk1/5ppp/p1pb4/1p1n4/3P2b1/1BPQB1Pq/PP1N1P1P/R3R1K1 b - - 6 17',
			},
		},
		'4rrk1/6pp/p1pb4/1p1n1p2/3P2b1/1BPQB1Pq/PP1N1P1P/R3R1K1 w - f6 0 18' => {
			eco => 'C89r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 f5

			variation => N__('Spanish: Marshall, Main Line, Pawn Push Variation'),
			moves => {
			},
		},
		'5rk1/5ppp/p1pbr3/1p1B4/3P2b1/2PQB1Pq/PP1N1P1P/R3R1K1 b - - 0 18' => {
			eco => 'C89u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6 18. Bxd5

			variation => N__('Spanish: Marshall, Main Line, Classical, 18.Bxd5'),
			moves => {
			},
		},
		'5rk1/5ppp/p1pbr3/1p1n3q/P2P2b1/1BPQB1P1/1P1N1P1P/R3R1K1 w - - 1 19' => {
			eco => 'C89x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6 18. a4 Qh5

			variation => N__('Spanish: Marshall, Main Line, Classical, Spassky Variation'),
			moves => {
			},
		},
		'5rk1/5ppp/p1pbr3/1p1n4/2PP2b1/1B1QB1Pq/PP1N1P1P/R3R1K1 b - - 0 18' => {
			eco => 'C89t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6 18. c4

			variation => N__('Spanish: Marshall, Main Line, Classical, 18. c4'),
			moves => {
			},
		},
		'5rk1/5ppp/p1pbr3/1p1n4/3P2b1/1BP1B1Pq/PP1N1P1P/R3RQK1 b - - 8 18' => {
			eco => 'C89v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6 18. Qf1

			variation => N__('Spanish: Marshall, Main Line, Classical, 18. Qf1'),
			moves => {
			},
		},
		'5rk1/5ppp/p1pbr3/1p1n4/3P2b1/1BPQB1Pq/PP1N1P1P/R3R1K1 w - - 7 18' => {
			eco => 'C89s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6

			variation => N__('Spanish: Marshall, Main Line, Classical Variation'),
			moves => {
				'a2a4' => '5rk1/5ppp/p1pbr3/1p1n4/P2P2b1/1BPQB1Pq/1P1N1P1P/R3R1K1 b - a3 0 18',
				'b3d5' => '5rk1/5ppp/p1pbr3/1p1B4/3P2b1/2PQB1Pq/PP1N1P1P/R3R1K1 b - - 0 18',
				'c3c4' => '5rk1/5ppp/p1pbr3/1p1n4/2PP2b1/1B1QB1Pq/PP1N1P1P/R3R1K1 b - - 0 18',
				'd3f1' => '5rk1/5ppp/p1pbr3/1p1n4/3P2b1/1BP1B1Pq/PP1N1P1P/R3RQK1 b - - 8 18',
			},
		},
		'5rk1/5ppp/p1pbr3/1p1n4/P2P2b1/1BPQB1Pq/1P1N1P1P/R3R1K1 b - a3 0 18' => {
			eco => 'C89w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6 18. a4

			variation => N__('Spanish: Marshall, Main Line, Classical, 18. a4'),
			moves => {
				'f7f5' => '5rk1/6pp/p1pbr3/1p1n1p2/P2P2b1/1BPQB1Pq/1P1N1P1P/R3R1K1 w - f6 0 19',
				'h3h5' => '5rk1/5ppp/p1pbr3/1p1n3q/P2P2b1/1BPQB1P1/1P1N1P1P/R3R1K1 w - - 1 19',
			},
		},
		'5rk1/6pp/p1pbr3/1p1n1p2/P2P2b1/1BPQB1Pq/1P1N1P1P/R3R1K1 w - f6 0 19' => {
			eco => 'C89y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3 Rae8 17. Nd2 Re6 18. a4 f5

			variation => N__('Spanish: Marshall, Main Line, Classical, 18. a4 f5'),
			moves => {
			},
		},
		'N1bk1b1r/p2pq2p/1pnn2p1/3Qpp2/8/1B1P4/PPP2PPP/R1B1K1NR b KQ - 0 11' => {
			eco => 'C27j',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6 6. Nb5 g6 7. Qf3 f5 8. Qd5 Qe7 9. Nxc7+ Kd8 10. Nxa8 b6 11. d3

			variation => N__('Vienna: Frankenstein-Dracula, 11. d3'),
			moves => {
			},
		},
		'N1bk1b1r/p2pq2p/1pnn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR w KQ - 0 11' => {
			eco => 'C27i',
                        # TRANSLATORS:

			variation => N__('Vienna: Frankenstein-Dracula Variation'),
			moves => {
				'd2d3' => 'N1bk1b1r/p2pq2p/1pnn2p1/3Qpp2/8/1B1P4/PPP2PPP/R1B1K1NR b KQ - 0 11',
			},
		},
		'N1bk1b1r/pp1pq2p/2nn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR b KQ - 0 10' => {
			eco => 'C27i',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6 6. Nb5 g6 7. Qf3 f5 8. Qd5 Qe7 9. Nxc7+ Kd8 10. Nxa8

			variation => N__('Vienna: Frankenstein-Dracula Variation'),
			moves => {
				'b7b6' => 'N1bk1b1r/p2pq2p/1pnn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR w KQ - 0 11',
			},
		},
		'N1bk2nr/pp1p1ppp/2n5/8/1b6/5B1q/PPPN1P1P/R1BQK2R b KQ - 0 10' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'g8f6' => 'N1bk3r/pp1p1ppp/2n2n2/8/1b6/5B1q/PPPN1P1P/R1BQK2R w KQ - 1 11',
			},
		},
		'N1bk3r/pp1p1ppp/2n2n2/8/1b6/5B1q/PPPN1P1P/R1BQK2R w KQ - 1 11' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'a2a3' => 'N1bk3r/pp1p1ppp/2n2n2/8/1b6/P4B1q/1PPN1P1P/R1BQK2R b KQ - 0 11',
			},
		},
		'N1bk3r/pp1p1ppp/2n2n2/8/1b6/P4B1q/1PPN1P1P/R1BQK2R b KQ - 0 11' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Bb4+ 6. Nd2 Qxe4+ 7. Be2 Qxg2 8. Bf3 Qh3 9. Nxc7+ Kd8 10. Nxa8 Nf6 11. a3

			variation => N__('Scotch: Steinitz, Berger Variation'),
			moves => {
			},
		},
		'r1b1k1nr/1p1p1ppp/p1n2Q2/4p3/4P3/8/PPP2PPP/RNB1KB1R b KQkq - 0 8' => {
			eco => 'B32l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 a6 6. Nd6+ Bxd6 7. Qxd6 Qf6 8. Qxf6

			variation => N__('Sicilian: Lowenthal, 5... a6, Queenswap'),
			moves => {
			},
		},
		'r1b1k1nr/1p1p1ppp/p1n2q2/4p3/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 2 8' => {
			eco => 'B32m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 a6 6. Nd6+ Bxd6 7. Qxd6 Qf6 8. Qd1

			variation => N__('Sicilian: Lowenthal, 5... a6, 8. Qd1'),
			moves => {
			},
		},
		'r1b1k1nr/1p1p1ppp/p1nQ1q2/4p3/4P3/8/PPP2PPP/RNB1KB1R w KQkq - 1 8' => {
			eco => 'B32k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, 5... a6'),
			moves => {
				'd6d1' => 'r1b1k1nr/1p1p1ppp/p1n2q2/4p3/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 2 8',
				'd6f6' => 'r1b1k1nr/1p1p1ppp/p1n2Q2/4p3/4P3/8/PPP2PPP/RNB1KB1R b KQkq - 0 8',
			},
		},
		'r1b1k1nr/1ppp1ppp/pbn2q2/8/P3P3/1NN5/1PP2PPP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'C45q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6 7. Nc3 Qf6

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6 7. Nc3 Qf6'),
			moves => {
			},
		},
		'r1b1k1nr/pp3pbp/2npp1p1/q1p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8' => {
			eco => 'B26m',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6 7. Qd2 Qa5

			variation => N__('Sicilian: Closed, 6. Be3 e6 7. Qd2 Qa5'),
			moves => {
			},
		},
		'r1b1k1nr/pp3ppp/n1p1p3/8/1BP1q3/8/PP2BPPP/R2QK1NR w KQkq - 2 9' => {
			eco => 'D31z',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Bb4+ 6. Bd2 Qxd4 7. Bxb4 Qxe4+ 8. Be2 Na6

			variation => N__('Semi-Slav: Marshall Gambit, 8. Be2 Na6'),
			moves => {
			},
		},
		'r1b1k1nr/ppNp1ppp/2n5/8/1b6/5B1q/PPPN1P1P/R1BQK2R b KQkq - 0 9' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'e8d8' => 'r1bk2nr/ppNp1ppp/2n5/8/1b6/5B1q/PPPN1P1P/R1BQK2R w KQ - 1 10',
			},
		},
		'r1b1k1nr/ppp2ppp/2n5/3qp3/1b1P4/2N1P3/PP1B1PPP/R2QKBNR b KQkq - 3 6' => {
			eco => 'D07c',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'b4c3' => 'r1b1k1nr/ppp2ppp/2n5/3qp3/3P4/2b1P3/PP1B1PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'r1b1k1nr/ppp2ppp/2n5/3qp3/1b1P4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 2 6' => {
			eco => 'D07c',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'c1d2' => 'r1b1k1nr/ppp2ppp/2n5/3qp3/1b1P4/2N1P3/PP1B1PPP/R2QKBNR b KQkq - 3 6',
			},
		},
		'r1b1k1nr/ppp2ppp/2n5/3qp3/3P4/2B1P3/PP3PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'D07e',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. cxd5 Qxd5 4. e3 e5 5. Nc3 Bb4 6. Bd2 Bxc3 7. Bxc3

			variation => N__('QGD: Chigorin, 3.cxd5 Main Line, 7.Bxc3'),
			moves => {
			},
		},
		'r1b1k1nr/ppp2ppp/2n5/3qp3/3P4/2P1P3/P2B1PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'D07d',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. cxd5 Qxd5 4. e3 e5 5. Nc3 Bb4 6. Bd2 Bxc3 7. bxc3

			variation => N__('QGD: Chigorin, 3.cxd5, Main Line, 7.bxc3'),
			moves => {
			},
		},
		'r1b1k1nr/ppp2ppp/2n5/3qp3/3P4/2b1P3/PP1B1PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'D07c',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. cxd5 Qxd5 4. e3 e5 5. Nc3 Bb4 6. Bd2 Bxc3

			variation => N__('QGD: Chigorin, 3.cxd5, Main Line'),
			moves => {
				'b2c3' => 'r1b1k1nr/ppp2ppp/2n5/3qp3/3P4/2P1P3/P2B1PPP/R2QKBNR b KQkq - 0 7',
				'd2c3' => 'r1b1k1nr/ppp2ppp/2n5/3qp3/3P4/2B1P3/PP3PPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'r1b1k1nr/ppp2ppp/2p2q2/2b5/4P3/2N5/PPPQ1PPP/R1B1KB1R b KQkq - 1 7' => {
			eco => 'C45t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nxc6 Qf6 6. Qd2 dxc6 7. Nc3

			variation => N__('Scotch: 4.Nxd4 Bc5 5.Nxc6 Qf6 6. Qd2 dxc6 7. Nc3'),
			moves => {
			},
		},
		'r1b1k1nr/ppp2ppp/2p2q2/2b5/4P3/8/PPPQ1PPP/RNB1KB1R w KQkq - 0 7' => {
			eco => 'C45s',
                        # TRANSLATORS:

			variation => N__('Scotch: 4.Nxd4 Bc5 5.Nxc6 Qf6 6. Qd2'),
			moves => {
				'b1c3' => 'r1b1k1nr/ppp2ppp/2p2q2/2b5/4P3/2N5/PPPQ1PPP/R1B1KB1R b KQkq - 1 7',
			},
		},
		'r1b1k1nr/pppp1Bpp/1b6/nP2N1q1/4P3/8/P1PP1PPP/RNBQK2R b KQkq - 0 7' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'e8e7' => 'r1b3nr/ppppkBpp/1b6/nP2N1q1/4P3/8/P1PP1PPP/RNBQK2R w KQ - 1 8',
			},
		},
		'r1b1k1nr/pppp1Qpp/1b6/nP2q3/2B1P3/8/P1PP1PPP/RNB1K2R b KQkq - 0 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'e8d8' => 'r1bk2nr/pppp1Qpp/1b6/nP2q3/2B1P3/8/P1PP1PPP/RNB1K2R w KQ - 1 9',
			},
		},
		'r1b1k1nr/pppp1ppp/1b6/nP2N1q1/2B1P3/5Q2/P1PP1PPP/RNB1K2R b KQkq - 2 7' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'g5e5' => 'r1b1k1nr/pppp1ppp/1b6/nP2q3/2B1P3/5Q2/P1PP1PPP/RNB1K2R w KQkq - 0 8',
			},
		},
		'r1b1k1nr/pppp1ppp/1b6/nP2N1q1/2B1P3/8/P1PP1PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. b5 Na5 6. Nxe5 Qg5

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'c4f7' => 'r1b1k1nr/pppp1Bpp/1b6/nP2N1q1/4P3/8/P1PP1PPP/RNBQK2R b KQkq - 0 7',
				'd1f3' => 'r1b1k1nr/pppp1ppp/1b6/nP2N1q1/2B1P3/5Q2/P1PP1PPP/RNB1K2R b KQkq - 2 7',
			},
		},
		'r1b1k1nr/pppp1ppp/1b6/nP2q3/2B1P3/5Q2/P1PP1PPP/RNB1K2R w KQkq - 0 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'f3f7' => 'r1b1k1nr/pppp1Qpp/1b6/nP2q3/2B1P3/8/P1PP1PPP/RNB1K2R b KQkq - 0 8',
			},
		},
		'r1b1k1nr/pppp1ppp/2N2q2/2b5/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'C45r',
                        # TRANSLATORS:

			variation => N__('Scotch: 4.Nxd4 Bc5 5.Nxc6'),
			moves => {
				'd1d2' => 'r1b1k1nr/pppp1ppp/2N2q2/2b5/4P3/8/PPPQ1PPP/RNB1KB1R b KQkq - 2 6',
			},
		},
		'r1b1k1nr/pppp1ppp/2N2q2/2b5/4P3/8/PPPQ1PPP/RNB1KB1R b KQkq - 2 6' => {
			eco => 'C45s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nxc6 Qf6 6. Qd2

			variation => N__('Scotch: 4.Nxd4 Bc5 5.Nxc6 Qf6 6. Qd2'),
			moves => {
				'd7c6' => 'r1b1k1nr/ppp2ppp/2p2q2/2b5/4P3/8/PPPQ1PPP/RNB1KB1R w KQkq - 0 7',
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C64e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Qf6

			variation => N__('Spanish: Classical, 4. c3 Qf6'),
			moves => {
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5' => {
			eco => 'C64m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Qf6

			variation => N__('Spanish: Classical, 4.O-O Qf6'),
			moves => {
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/1Nb5/4P3/4B3/PPP2PPP/RN1QKB1R b KQkq - 4 6' => {
			eco => 'C45v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. Nb5

			variation => N__('Scotch: Blumenfeld Attack'),
			moves => {
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/2b5/3NP3/2P1B3/PP3PPP/RN1QKB1R b KQkq - 0 6' => {
			eco => 'C45v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Be3 Qf6 6. c3'),
			moves => {
				'g8e7' => 'r1b1k2r/ppppnppp/2n2q2/2b5/3NP3/2P1B3/PP3PPP/RN1QKB1R w KQkq - 1 7',
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/2b5/3NP3/4B3/PPP2PPP/RN1QKB1R w KQkq - 3 6' => {
			eco => 'C45u',
                        # TRANSLATORS:

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Be3'),
			moves => {
				'c2c3' => 'r1b1k1nr/pppp1ppp/2n2q2/2b5/3NP3/2P1B3/PP3PPP/RN1QKB1R b KQkq - 0 6',
				'd4b5' => 'r1b1k1nr/pppp1ppp/2n2q2/1Nb5/4P3/4B3/PPP2PPP/RN1QKB1R b KQkq - 4 6',
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/b3P3/2B5/1Qp2N2/P4PPP/RNB2RK1 b kq - 0 9' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'f6g6' => 'r1b1k1nr/pppp1ppp/2n3q1/b3P3/2B5/1Qp2N2/P4PPP/RNB2RK1 w kq - 1 10',
			},
		},
		'r1b1k1nr/pppp1ppp/2n2q2/b7/2B1P3/1Qp2N2/P4PPP/RNB2RK1 w kq - 2 9' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'e4e5' => 'r1b1k1nr/pppp1ppp/2n2q2/b3P3/2B5/1Qp2N2/P4PPP/RNB2RK1 b kq - 0 9',
			},
		},
		'r1b1k1nr/pppp1ppp/2n3q1/b3P3/2B5/1QN2N2/P4PPP/R1B2RK1 b kq - 0 10' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'g8e7' => 'r1b1k2r/ppppnppp/2n3q1/b3P3/2B5/1QN2N2/P4PPP/R1B2RK1 w kq - 1 11',
			},
		},
		'r1b1k1nr/pppp1ppp/2n3q1/b3P3/2B5/1Qp2N2/P4PPP/RNB2RK1 w kq - 1 10' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'b1c3' => 'r1b1k1nr/pppp1ppp/2n3q1/b3P3/2B5/1QN2N2/P4PPP/R1B2RK1 b kq - 0 10',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2P2q/8/PPP2PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Bb4+

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'b1d2' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2P2q/8/PPPN1PPP/R1BQKB1R b KQkq - 4 6',
				'c1d2' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2P2q/8/PPPB1PPP/RN1QKB1R b KQkq - 4 6',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2P2q/8/PPPB1PPP/RN1QKB1R b KQkq - 4 6' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Bb4+ 6. Bd2

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'h4e4' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPB1PPP/RN1QKB1R w KQkq - 0 7',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2P2q/8/PPPN1PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'h4e4' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPN1PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPB1PPP/RN1QKB1R w KQkq - 0 7' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'f1e2' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPBBPPP/RN1QK2R b KQkq - 1 7',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPBBPPP/RN1QK2R b KQkq - 1 7' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'e8d8' => 'r1bk2nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPBBPPP/RN1QK2R w KQ - 2 8',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPN1PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'f1e2' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPNBPPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPNBPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'e4g2' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b6/8/PPPNBPqP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b6/5B1q/PPPN1P1P/R1BQK2R w KQkq - 2 9' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'b5c7' => 'r1b1k1nr/ppNp1ppp/2n5/8/1b6/5B1q/PPPN1P1P/R1BQK2R b KQkq - 0 9',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b6/5B2/PPPN1PqP/R1BQK2R b KQkq - 1 8' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'g2h3' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b6/5B1q/PPPN1P1P/R1BQK2R w KQkq - 2 9',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1N6/1b6/8/PPPNBPqP/R1BQK2R w KQkq - 0 8' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'e2f3' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b6/5B2/PPPN1PqP/R1BQK2R b KQkq - 1 8',
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1Nb5/4P2q/5Q2/PPP2PPP/RNB1KB1R b KQkq - 4 6' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Bc5 6. Qf3

			variation => N__('Scotch: Steinitz, 5. Nb5 Bc5 6. Qf3'),
			moves => {
			},
		},
		'r1b1k1nr/pppp1ppp/2n5/1Nb5/4P2q/8/PPP2PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Bc5

			variation => N__('Scotch: Steinitz, 5. Nb5 Bc5'),
			moves => {
				'd1f3' => 'r1b1k1nr/pppp1ppp/2n5/1Nb5/4P2q/5Q2/PPP2PPP/RNB1KB1R b KQkq - 4 6',
			},
		},
		'r1b1k1nr/ppppqppp/1bn5/3Pp3/2B1P3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'c6b8' => 'rnb1k1nr/ppppqppp/1b6/3Pp3/2B1P3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1b1k1nr/ppppqppp/1bn5/4p1B1/2BPP3/2P2N2/PP3PPP/RN1QK2R b KQkq - 2 6' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Qe7 5. d4 Bb6 6. Bg5

			variation => N__('Giuoco Piano: Mestel Variation'),
			moves => {
			},
		},
		'r1b1k1nr/ppppqppp/1bn5/4p3/2BPP3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'g8f6' => 'r1b1k2r/ppppqppp/1bn2n2/4p3/2BPP3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 3 7',
			},
		},
		'r1b1k1nr/ppppqppp/1bn5/4p3/2BPP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Qe7 5. d4 Bb6

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'c1g5' => 'r1b1k1nr/ppppqppp/1bn5/4p1B1/2BPP3/2P2N2/PP3PPP/RN1QK2R b KQkq - 2 6',
				'd4d5' => 'r1b1k1nr/ppppqppp/1bn5/3Pp3/2B1P3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6',
				'e1g1' => 'r1b1k1nr/ppppqppp/1bn5/4p3/2BPP3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'r1b1k1nr/ppppqppp/2n5/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C64b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Qe7

			variation => N__('Spanish: Classical, Boden Variation'),
			moves => {
			},
		},
		'r1b1k1nr/ppppqppp/2n5/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Qe7

			variation => N__('Giuoco Piano: Close Variation'),
			moves => {
				'd2d4' => 'r1b1k1nr/ppppqppp/2n5/2b1p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5',
			},
		},
		'r1b1k1nr/ppppqppp/2n5/2b1p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Close Variation'),
			moves => {
				'c5b6' => 'r1b1k1nr/ppppqppp/1bn5/4p3/2BPP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1b1k1nr/pppq1ppp/1b6/n3N3/2B1P3/BQP5/P4PPP/RN3RK1 b kq - 0 11' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6 7. Qb3 Qd7 8. dxe5 dxe5 9. O-O Bb6 10. Ba3 Na5 11. Nxe5

			variation => N__('Evans Gambit: Tartakower, Levenfish Variation'),
			moves => {
			},
		},
		'r1b1k1nr/pppq1ppp/1b6/n3p3/2B1P3/BQP2N2/P4PPP/RN3RK1 w kq - 4 11' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Tartakower Attack, 8.dxe5 dxe5'),
			moves => {
				'f3e5' => 'r1b1k1nr/pppq1ppp/1b6/n3N3/2B1P3/BQP5/P4PPP/RN3RK1 b kq - 0 11',
			},
		},
		'r1b1k1nr/pppq1ppp/1bn5/4p3/2B1P3/1QP2N2/P4PPP/RNB2RK1 w kq - 2 10' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Tartakower Attack, 8.dxe5 dxe5'),
			moves => {
				'c1a3' => 'r1b1k1nr/pppq1ppp/1bn5/4p3/2B1P3/BQP2N2/P4PPP/RN3RK1 b kq - 3 10',
			},
		},
		'r1b1k1nr/pppq1ppp/1bn5/4p3/2B1P3/BQP2N2/P4PPP/RN3RK1 b kq - 3 10' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Tartakower Attack, 8.dxe5 dxe5'),
			moves => {
				'c6a5' => 'r1b1k1nr/pppq1ppp/1b6/n3p3/2B1P3/BQP2N2/P4PPP/RN3RK1 w kq - 4 11',
			},
		},
		'r1b1k1nr/pppq1ppp/1bnp4/4P3/2B1P3/1QP2N2/P4PPP/RNB1K2R w KQkq - 1 9' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6 7. Qb3 Qd7 8. dxe5 Bb6

			variation => N__('Evans Gambit: Tartakower Attack, 8.dxe5 Bb6'),
			moves => {
			},
		},
		'r1b1k1nr/pppq1ppp/2n5/b3p3/2B1P3/1QP2N2/P4PPP/RNB1K2R w KQkq - 0 9' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6 7. Qb3 Qd7 8. dxe5 dxe5

			variation => N__('Evans Gambit: Tartakower Attack, 8.dxe5 dxe5'),
			moves => {
				'e1g1' => 'r1b1k1nr/pppq1ppp/2n5/b3p3/2B1P3/1QP2N2/P4PPP/RNB2RK1 b kq - 1 9',
			},
		},
		'r1b1k1nr/pppq1ppp/2n5/b3p3/2B1P3/1QP2N2/P4PPP/RNB2RK1 b kq - 1 9' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Tartakower Attack, 8.dxe5 dxe5'),
			moves => {
				'a5b6' => 'r1b1k1nr/pppq1ppp/1bn5/4p3/2B1P3/1QP2N2/P4PPP/RNB2RK1 w kq - 2 10',
			},
		},
		'r1b1k1nr/pppq1ppp/2np4/b3P3/2B1P3/1QP2N2/P4PPP/RNB1K2R b KQkq - 0 8' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Tartakower Attack, 7... Qd7'),
			moves => {
				'a5b6' => 'r1b1k1nr/pppq1ppp/1bnp4/4P3/2B1P3/1QP2N2/P4PPP/RNB1K2R w KQkq - 1 9',
				'd6e5' => 'r1b1k1nr/pppq1ppp/2n5/b3p3/2B1P3/1QP2N2/P4PPP/RNB1K2R w KQkq - 0 9',
			},
		},
		'r1b1k1nr/pppq1ppp/2np4/b3p3/2BPP3/1QP2N2/P4PPP/RNB1K2R w KQkq - 2 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6 7. Qb3 Qd7

			variation => N__('Evans Gambit: Tartakower Attack, 7... Qd7'),
			moves => {
				'd4e5' => 'r1b1k1nr/pppq1ppp/2np4/b3P3/2B1P3/1QP2N2/P4PPP/RNB1K2R b KQkq - 0 8',
			},
		},
		'r1b1k1r1/ppq1np1Q/2n1p3/3pP3/3p1P2/P1P5/2P1N1PP/R1B1KB1R b KQq f3 0 11' => {
			eco => 'C18t',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, Konstantinopolsky Variation'),
			moves => {
				'c8d7' => 'r3k1r1/ppqbnp1Q/2n1p3/3pP3/3p1P2/P1P5/2P1N1PP/R1B1KB1R w KQq - 1 12',
			},
		},
		'r1b1k1r1/ppq1np1Q/2n1p3/3pP3/3p4/P1P5/2P1NPPP/R1B1KB1R w KQq - 2 11' => {
			eco => 'C18t',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, Konstantinopolsky Variation'),
			moves => {
				'f2f4' => 'r1b1k1r1/ppq1np1Q/2n1p3/3pP3/3p1P2/P1P5/2P1N1PP/R1B1KB1R b KQq f3 0 11',
			},
		},
		'r1b1k2N/ppp1q1pp/2n2n2/3Pp3/2B5/8/PPPP1bPP/RNBQ1K1R b q - 0 8' => {
			eco => 'C57c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1'),
			moves => {
				'c6d4' => 'r1b1k2N/ppp1q1pp/5n2/3Pp3/2Bn4/8/PPPP1bPP/RNBQ1K1R w q - 1 9',
				'c8g4' => 'r3k2N/ppp1q1pp/2n2n2/3Pp3/2B3b1/8/PPPP1bPP/RNBQ1K1R w q - 1 9',
			},
		},
		'r1b1k2N/ppp1q1pp/2n2n2/3pp3/2B1P3/8/PPPP1bPP/RNBQ1K1R w q d6 0 8' => {
			eco => 'C57c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1'),
			moves => {
				'e4d5' => 'r1b1k2N/ppp1q1pp/2n2n2/3Pp3/2B5/8/PPPP1bPP/RNBQ1K1R b q - 0 8',
			},
		},
		'r1b1k2N/ppp1q1pp/3P1n2/4p3/2Bn4/8/PPPP1bPP/RNBQ1K1R b q - 0 9' => {
			eco => 'C57c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7 Bxf2+ 6. Kf1 Qe7 7. Nxh8 d5 8. exd5 Nd4 9. d6

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1, Menovsky Variation'),
			moves => {
			},
		},
		'r1b1k2N/ppp1q1pp/5n2/3Pp3/2Bn4/7P/PPPP1bP1/RNBQ1K1R b q - 0 9' => {
			eco => 'C57c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7 Bxf2+ 6. Kf1 Qe7 7. Nxh8 d5 8. exd5 Nd4 9. h3

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1, Palkin Variation'),
			moves => {
			},
		},
		'r1b1k2N/ppp1q1pp/5n2/3Pp3/2Bn4/8/PPPP1bPP/RNBQ1K1R w q - 1 9' => {
			eco => 'C57c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7 Bxf2+ 6. Kf1 Qe7 7. Nxh8 d5 8. exd5 Nd4

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1, 8... Nd4'),
			moves => {
				'd5d6' => 'r1b1k2N/ppp1q1pp/3P1n2/4p3/2Bn4/8/PPPP1bPP/RNBQ1K1R b q - 0 9',
				'h2h3' => 'r1b1k2N/ppp1q1pp/5n2/3Pp3/2Bn4/7P/PPPP1bP1/RNBQ1K1R b q - 0 9',
			},
		},
		'r1b1k2N/ppppq1pp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQ1K1R b q - 0 7' => {
			eco => 'C57c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1'),
			moves => {
				'd7d5' => 'r1b1k2N/ppp1q1pp/2n2n2/3pp3/2B1P3/8/PPPP1bPP/RNBQ1K1R w q d6 0 8',
			},
		},
		'r1b1k2r/1p1nbppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R3RK1 w kq - 6 12' => {
			eco => 'B97n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. Be2 Be7 11. O-O Nbd7

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. Be2 Be7 11.O-O Nbd7'),
			moves => {
			},
		},
		'r1b1k2r/1p1nbppp/pq1ppn2/6B1/4PP2/1NN2Q2/PPP3PP/R3KB1R w KQkq - 5 10' => {
			eco => 'B97h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Nb3 Nbd7 9. Qf3 Be7

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Nb3 Nbd7 9. Qf3 Be7'),
			moves => {
			},
		},
		'r1b1k2r/1p2bppp/pqnppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w kq - 2 10' => {
			eco => 'B63g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Qb6 8. Nb3 a6 9. O-O-O Be7

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Qb6 8. Nb3 a6 9.O-O-O Be7'),
			moves => {
			},
		},
		'r1b1k2r/1pp1qppp/2np1n2/p2Pp3/2P1P3/5NP1/PP1N1PBP/R2Q1RK1 b kq - 0 10' => {
			eco => 'E11w',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line, 8... a5 9. e4'),
			moves => {
				'c6b8' => 'rnb1k2r/1pp1qppp/3p1n2/p2Pp3/2P1P3/5NP1/PP1N1PBP/R2Q1RK1 w kq - 1 11',
			},
		},
		'r1b1k2r/1pp1qppp/2np1n2/p3p3/2PPP3/5NP1/PP1N1PBP/R2Q1RK1 w kq - 0 10' => {
			eco => 'E11w',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line, 8... a5 9. e4'),
			moves => {
				'd4d5' => 'r1b1k2r/1pp1qppp/2np1n2/p2Pp3/2P1P3/5NP1/PP1N1PBP/R2Q1RK1 b kq - 0 10',
			},
		},
		'r1b1k2r/1pp1qppp/2nppn2/p7/2PP4/5NP1/PP1NPPBP/R2Q1RK1 w kq a6 0 9' => {
			eco => 'E11w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+ 7. Nbxd2 d6 8. O-O a5

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line, 8... a5'),
			moves => {
				'e2e4' => 'r1b1k2r/1pp1qppp/2nppn2/p7/2PPP3/5NP1/PP1N1PBP/R2Q1RK1 b kq e3 0 9',
			},
		},
		'r1b1k2r/1pp1qppp/2nppn2/p7/2PPP3/5NP1/PP1N1PBP/R2Q1RK1 b kq e3 0 9' => {
			eco => 'E11w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+ 7. Nbxd2 d6 8. O-O a5 9. e4

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line, 8... a5 9. e4'),
			moves => {
				'e6e5' => 'r1b1k2r/1pp1qppp/2np1n2/p3p3/2PPP3/5NP1/PP1N1PBP/R2Q1RK1 w kq - 0 10',
			},
		},
		'r1b1k2r/1pp1qppp/pbnp1n2/4p3/P1BPP3/2P2N1P/1P3PP1/RNBQR1K1 b kq - 0 9' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Qe7 5. d4 Bb6 6. O-O Nf6 7. a4 a6 8. Re1 d6 9. h3

			variation => N__('Giuoco Piano: Tarrasch Variation'),
			moves => {
			},
		},
		'r1b1k2r/1pp1qppp/pbnp1n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQR1K1 w kq - 0 9' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'h2h3' => 'r1b1k2r/1pp1qppp/pbnp1n2/4p3/P1BPP3/2P2N1P/1P3PP1/RNBQR1K1 b kq - 0 9',
			},
		},
		'r1b1k2r/1pppqppp/pbn2n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'f1e1' => 'r1b1k2r/1pppqppp/pbn2n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQR1K1 b kq - 1 8',
			},
		},
		'r1b1k2r/1pppqppp/pbn2n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQR1K1 b kq - 1 8' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'd7d6' => 'r1b1k2r/1pp1qppp/pbnp1n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQR1K1 w kq - 0 9',
			},
		},
		'r1b1k2r/1pq1bppp/p1Nppn2/6B1/4PP2/2N2Q2/PPP3PP/2KR1B1R b kq - 0 10' => {
			eco => 'B98q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nc6 10. Nxc6

			variation => N__('Sicilian: Najdorf, 8... Qc7 9.O-O-O Nc6 10.Nxc6'),
			moves => {
			},
		},
		'r1b1k2r/1pq1bppp/p1nppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq - 5 10' => {
			eco => 'B98p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nc6

			variation => N__('Sicilian: Najdorf, 8... Qc7 9.O-O-O Nc6'),
			moves => {
				'd4c6' => 'r1b1k2r/1pq1bppp/p1Nppn2/6B1/4PP2/2N2Q2/PPP3PP/2KR1B1R b kq - 0 10',
			},
		},
		'r1b1k2r/1pq1bppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 w kq - 2 10' => {
			eco => 'B80u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6 7. Bg2 Qc7 8. O-O a6 9. Re1 Be7

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
			},
		},
		'r1b1k2r/1pq1bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w kq - 3 10' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 Qc7 9. Be3 Nc6

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7 8. f4 Qc7 9. Be3 Nc6'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbp2/p2ppn1p/6P1/3NP2B/2NB1Q2/PPP3PP/2KR3R b kq - 0 12' => {
			eco => 'B99i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 h6 11. Bh4 g5 12. fxg5

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 h6 11. Bh4 g5 12.fxg5'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbp2/p2ppn1p/6p1/3NPP1B/2NB1Q2/PPP3PP/2KR3R w kq g6 0 12' => {
			eco => 'B99h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 h6 11. Bh4 g5

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 h6 11. Bh4 g5'),
			moves => {
				'f4g5' => 'r1b1k2r/1pqnbp2/p2ppn1p/6P1/3NP2B/2NB1Q2/PPP3PP/2KR3R b kq - 0 12',
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/6B1/3NPP2/2NB1Q2/PPP3PP/2KR3R w kq - 0 11' => {
			eco => 'B99f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 h6

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 h6'),
			moves => {
				'f3h3' => 'r1b1k2r/1pqnbpp1/p2ppn1p/6B1/3NPP2/2NB3Q/PPP3PP/2KR3R b kq - 1 11',
				'g5h4' => 'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2NB1Q2/PPP3PP/2KR3R b kq - 1 11',
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/6B1/3NPP2/2NB3Q/PPP3PP/2KR3R b kq - 1 11' => {
			eco => 'B99g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 h6 11. Qh3

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 h6 11. Qh3'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/6B1/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq - 0 11' => {
			eco => 'B99m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 h6

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 h6'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP1B1PP/2KR3R b kq - 5 11' => {
			eco => 'B98j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 Qc7 10. O-O-O Nbd7 11. Be2

			variation => N__('Sicilian: Najdorf, Browne, 10.O-O-O Nbd7 11. Be2'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/2KR1B1R w kq - 4 11' => {
			eco => 'B98h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 Qc7 10. O-O-O Nbd7

			variation => N__('Sicilian: Najdorf, Browne, 10.O-O-O Nbd7'),
			moves => {
				'f1e2' => 'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP1B1PP/2KR3R b kq - 5 11',
				'f3g3' => 'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2N3Q1/PPP3PP/2KR1B1R b kq - 5 11',
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2N3Q1/PPP3PP/2KR1B1R b kq - 5 11' => {
			eco => 'B98i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 Qc7 10. O-O-O Nbd7 11. Qg3

			variation => N__('Sicilian: Najdorf, Browne, 10.O-O-O Nbd7 11. Qg3'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2NB1Q2/PPP3PP/2KR3R b kq - 1 11' => {
			eco => 'B99h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 h6 11. Bh4

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 h6 11. Bh4'),
			moves => {
				'g7g5' => 'r1b1k2r/1pqnbp2/p2ppn1p/6p1/3NPP1B/2NB1Q2/PPP3PP/2KR3R w kq g6 0 12',
			},
		},
		'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP1B1PP/2KR3R b kq - 6 10' => {
			eco => 'B99c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Be2

			variation => N__('Sicilian: Najdorf, Main Line, Keres Variation'),
			moves => {
				'b7b5' => 'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP1B1PP/2KR3R w kq b6 0 11',
			},
		},
		'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq - 5 10' => {
			eco => 'B99a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7

			variation => N__('Sicilian: Najdorf, Main Line'),
			moves => {
				'f1d3' => 'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2NB1Q2/PPP3PP/2KR3R b kq - 6 10',
				'f1e2' => 'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP1B1PP/2KR3R b kq - 6 10',
				'f3g3' => 'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2N3Q1/PPP3PP/2KR1B1R b kq - 6 10',
				'g2g4' => 'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPPP1/2N2Q2/PPP4P/2KR1B1R b kq g3 0 10',
			},
		},
		'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2N3Q1/PPP3PP/2KR1B1R b kq - 6 10' => {
			eco => 'B99b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Qg3

			variation => N__('Sicilian: Najdorf, Main Line, 10. Qg3'),
			moves => {
			},
		},
		'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2NB1Q2/PPP3PP/2KR3R b kq - 6 10' => {
			eco => 'B99f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3'),
			moves => {
				'b7b5' => 'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KR3R w kq b6 0 11',
				'h7h6' => 'r1b1k2r/1pqnbpp1/p2ppn1p/6B1/3NPP2/2NB1Q2/PPP3PP/2KR3R w kq - 0 11',
			},
		},
		'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPPP1/2N2Q2/PPP4P/2KR1B1R b kq g3 0 10' => {
			eco => 'B99m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4'),
			moves => {
				'b7b5' => 'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq b6 0 11',
				'h7h6' => 'r1b1k2r/1pqnbpp1/p2ppn1p/6B1/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq - 0 11',
			},
		},
		'r1b1k2r/1pqp1ppp/p1N1pn2/8/1b2P3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 0 9' => {
			eco => 'B49i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Nxc6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9.Nxc6'),
			moves => {
			},
		},
		'r1b1k2r/1pqp1ppp/p1n1pn2/2b5/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 4 9' => {
			eco => 'B47n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 Nf6 8. O-O Bc5

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 Nf6 8.O-O Bc5'),
			moves => {
			},
		},
		'r1b1k2r/1pqp1ppp/p1n1pn2/8/1b1NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq - 4 9' => {
			eco => 'B49h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4'),
			moves => {
				'c3a4' => 'r1b1k2r/1pqp1ppp/p1n1pn2/8/Nb1NP3/4B3/PPP1BPPP/R2Q1RK1 b kq - 5 9',
				'd4c6' => 'r1b1k2r/1pqp1ppp/p1N1pn2/8/1b2P3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 0 9',
			},
		},
		'r1b1k2r/1pqp1ppp/p1n1pn2/8/Nb1NP3/4B3/PPP1BPPP/R2Q1RK1 b kq - 5 9' => {
			eco => 'B49j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Na4

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9. Na4'),
			moves => {
				'b4e7' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/N2NP3/4B3/PPP1BPPP/R2Q1RK1 w kq - 6 10',
				'e8g8' => 'r1b2rk1/1pqp1ppp/p1n1pn2/8/Nb1NP3/4B3/PPP1BPPP/R2Q1RK1 w - - 6 10',
			},
		},
		'r1b1k2r/1pqp1ppp/p1nbpn2/8/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq - 4 9' => {
			eco => 'B48r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O Bd6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O Bd6'),
			moves => {
			},
		},
		'r1b1k2r/1pqp1ppp/p3pn2/2b1n3/3NP3/2NBB2P/PPP2PP1/R2Q1R1K b kq - 2 10' => {
			eco => 'B48v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O Ne5 9. h3 Bc5 10. Kh1

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O Ne5 9. h3 Bc5 10. Kh1'),
			moves => {
			},
		},
		'r1b1k2r/1pqp1ppp/p3pn2/2b1n3/3NP3/2NBB2P/PPP2PP1/R2Q1RK1 w kq - 1 10' => {
			eco => 'B48u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O Ne5 9. h3 Bc5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O Ne5 9. h3 Bc5'),
			moves => {
				'g1h1' => 'r1b1k2r/1pqp1ppp/p3pn2/2b1n3/3NP3/2NBB2P/PPP2PP1/R2Q1R1K b kq - 2 10',
			},
		},
		'r1b1k2r/1pqp1ppp/p3pn2/2b5/3QP3/2N5/PPP1BPPP/R1B2R1K w kq - 1 10' => {
			eco => 'B47y',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Nxd4'),
			moves => {
				'd4d3' => 'r1b1k2r/1pqp1ppp/p3pn2/2b5/4P3/2NQ4/PPP1BPPP/R1B2R1K b kq - 2 10',
			},
		},
		'r1b1k2r/1pqp1ppp/p3pn2/2b5/4P3/2NQ4/PPP1BPPP/R1B2R1K b kq - 2 10' => {
			eco => 'B47y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. Kh1 Nxd4 9. Qxd4 Bc5 10. Qd3

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Nxd4, 10. Qd3'),
			moves => {
				'b7b5' => 'r1b1k2r/2qp1ppp/p3pn2/1pb5/4P3/2NQ4/PPP1BPPP/R1B2R1K w kq b6 0 11',
			},
		},
		'r1b1k2r/1pqpbppp/p1N1pn2/8/N3P3/4B3/PPP1BPPP/R2Q1RK1 b kq - 0 10' => {
			eco => 'B49m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Na4 Be7 10. Nxc6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9. Na4 Be7 10.Nxc6'),
			moves => {
				'b7c6' => 'r1b1k2r/2qpbppp/p1p1pn2/8/N3P3/4B3/PPP1BPPP/R2Q1RK1 w kq - 0 11',
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq - 4 9' => {
			eco => 'B49g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Be7

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Be7'),
			moves => {
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 4 9' => {
			eco => 'B47o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 Nf6 8. O-O Be7

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 Nf6 8.O-O Be7'),
			moves => {
				'f1e1' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 b kq - 5 9',
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 b kq - 5 9' => {
			eco => 'B47p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 Nf6 8. O-O Be7 9. Re1

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 Nf6 8.O-O Be7 9. Re1'),
			moves => {
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K w kq - 4 9' => {
			eco => 'B47x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. Kh1 Be7

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Be7'),
			moves => {
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/P1N1B3/1PP1BPPP/R2Q1RK1 b kq - 2 9' => {
			eco => 'B49e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. a3 Be7 9. O-O

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8. a3 Be7 9.O-O'),
			moves => {
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/P1N1B3/1PP1BPPP/R2QK2R w KQkq - 1 9' => {
			eco => 'B49e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. a3 Be7

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8. a3 Be7'),
			moves => {
				'e1g1' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/P1N1B3/1PP1BPPP/R2Q1RK1 b kq - 2 9',
			},
		},
		'r1b1k2r/1pqpbppp/p1n1pn2/8/N2NP3/4B3/PPP1BPPP/R2Q1RK1 w kq - 6 10' => {
			eco => 'B49l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Na4 Be7

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9. Na4 Be7'),
			moves => {
				'd4c6' => 'r1b1k2r/1pqpbppp/p1N1pn2/8/N3P3/4B3/PPP1BPPP/R2Q1RK1 b kq - 0 10',
			},
		},
		'r1b1k2r/2q1bp1p/p2ppP2/1pn5/3NP3/2N2Q2/PPP4P/2KR1B1R b kq - 0 15' => {
			eco => 'B99x',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5 14. f6'),
			moves => {
				'e7f8' => 'r1b1kb1r/2q2p1p/p2ppP2/1pn5/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 1 16',
			},
		},
		'r1b1k2r/2q1bp1p/p2ppp2/1pn3P1/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 0 15' => {
			eco => 'B99x',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5 14. f6'),
			moves => {
				'g5f6' => 'r1b1k2r/2q1bp1p/p2ppP2/1pn5/3NP3/2N2Q2/PPP4P/2KR1B1R b kq - 0 15',
			},
		},
		'r1b1k2r/2q1bppp/p1np1n2/1pp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w kq - 1 12' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'd4d5' => 'r1b1k2r/2q1bppp/p1np1n2/1ppPp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 b kq - 0 12',
			},
		},
		'r1b1k2r/2q1bppp/p1np1n2/1ppPp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 b kq - 0 12' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'c6b8' => 'rnb1k2r/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 w kq - 1 13',
			},
		},
		'r1b1k2r/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 b kq - 0 11' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'a5c6' => 'r1b1k2r/2q1bppp/p1np1n2/1pp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w kq - 1 12',
			},
		},
		'r1b1k2r/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N2/PPB2PPP/RNBQR1K1 w kq - 1 11' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'a2a4' => 'r1b1k2r/2q1bppp/p2p1n2/npp1p3/P2PP3/2P2N2/1PB2PPP/RNBQR1K1 b kq a3 0 11',
				'h2h3' => 'r1b1k2r/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 b kq - 0 11',
			},
		},
		'r1b1k2r/2q1bppp/p2p1n2/npp1p3/P2PP3/2P2N2/1PB2PPP/RNBQR1K1 b kq a3 0 11' => {
			eco => 'C88e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. c3 Na5 9. Bc2 c5 10. d4 Qc7 11. a4

			variation => N__('Spanish: Closed, Balla Variation'),
			moves => {
			},
		},
		'r1b1k2r/2q1bppp/p2pp3/1pn2PP1/3NP2P/2N2Q2/PPP5/2KR1B1R b kq h3 0 14' => {
			eco => 'B99w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5 Nc5 14. h4

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5 14. h4'),
			moves => {
			},
		},
		'r1b1k2r/2q1bppp/p2pp3/1pn2PP1/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 1 14' => {
			eco => 'B99v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5 Nc5

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5'),
			moves => {
				'f5f6' => 'r1b1k2r/2q1bppp/p2ppP2/1pn3P1/3NP3/2N2Q2/PPP4P/2KR1B1R b kq - 0 14',
				'h2h4' => 'r1b1k2r/2q1bppp/p2pp3/1pn2PP1/3NP2P/2N2Q2/PPP5/2KR1B1R b kq h3 0 14',
			},
		},
		'r1b1k2r/2q1bppp/p2ppP2/1pn3P1/3NP3/2N2Q2/PPP4P/2KR1B1R b kq - 0 14' => {
			eco => 'B99x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5 Nc5 14. f6

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5 14. f6'),
			moves => {
				'g7f6' => 'r1b1k2r/2q1bp1p/p2ppp2/1pn3P1/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 0 15',
			},
		},
		'r1b1k2r/2q1bppp/p2ppn2/1p2P3/3N1P2/2N2Q2/PPP1B1PP/2KR3R b kq - 0 12' => {
			eco => 'B99c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres Variation'),
			moves => {
				'c8b7' => 'r3k2r/1bq1bppp/p2ppn2/1p2P3/3N1P2/2N2Q2/PPP1B1PP/2KR3R w kq - 1 13',
			},
		},
		'r1b1k2r/2q1bppp/p2ppn2/1p4P1/3NPP2/2N2Q2/PPP4P/2KR1B1R b kq - 0 12' => {
			eco => 'B99r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11.Bxf6 Nxf6 12. g5'),
			moves => {
				'f6d7' => 'r1b1k2r/2qnbppp/p2pp3/1p4P1/3NPP2/2N2Q2/PPP4P/2KR1B1R w kq - 1 13',
			},
		},
		'r1b1k2r/2q1bppp/p2ppn2/1p6/3NPP2/2N2Q2/PPP1B1PP/2KR3R w kq - 0 12' => {
			eco => 'B99c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres Variation'),
			moves => {
				'e4e5' => 'r1b1k2r/2q1bppp/p2ppn2/1p2P3/3N1P2/2N2Q2/PPP1B1PP/2KR3R b kq - 0 12',
			},
		},
		'r1b1k2r/2q1bppp/p2ppn2/1p6/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq - 0 12' => {
			eco => 'B99r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11.Bxf6 Nxf6'),
			moves => {
				'g4g5' => 'r1b1k2r/2q1bppp/p2ppn2/1p4P1/3NPP2/2N2Q2/PPP4P/2KR1B1R b kq - 0 12',
			},
		},
		'r1b1k2r/2q2ppp/p2pp3/1p2nPb1/3NP3/2N2Q2/PPP4P/1K1R1B1R w kq - 2 15' => {
			eco => 'B99u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5 Bxg5+ 14. Kb1 Ne5

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Bxg5+ 14. Kb1 Ne5'),
			moves => {
			},
		},
		'r1b1k2r/2qn1ppp/p2pp3/1p3Pb1/3NP3/2N2Q2/PPP4P/1K1R1B1R b kq - 1 14' => {
			eco => 'B99t',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Bxg5+'),
			moves => {
				'd7e5' => 'r1b1k2r/2q2ppp/p2pp3/1p2nPb1/3NP3/2N2Q2/PPP4P/1K1R1B1R w kq - 2 15',
			},
		},
		'r1b1k2r/2qn1ppp/p2pp3/1p3Pb1/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 0 14' => {
			eco => 'B99t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5 Bxg5+

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Bxg5+'),
			moves => {
				'c1b1' => 'r1b1k2r/2qn1ppp/p2pp3/1p3Pb1/3NP3/2N2Q2/PPP4P/1K1R1B1R b kq - 1 14',
			},
		},
		'r1b1k2r/2qnbp1p/p2ppp2/1p6/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq - 0 12' => {
			eco => 'B99q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 gxf6

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11.Bxf6 gxf6'),
			moves => {
			},
		},
		'r1b1k2r/2qnbppp/p2pp3/1p3PP1/3NP3/2N2Q2/PPP4P/2KR1B1R b kq - 0 13' => {
			eco => 'B99s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5'),
			moves => {
				'd7c5' => 'r1b1k2r/2q1bppp/p2pp3/1pn2PP1/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 1 14',
				'e7g5' => 'r1b1k2r/2qn1ppp/p2pp3/1p3Pb1/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 0 14',
			},
		},
		'r1b1k2r/2qnbppp/p2pp3/1p4P1/3NPP2/2N2Q2/PPP4P/2KR1B1R w kq - 1 13' => {
			eco => 'B99r',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11.Bxf6 Nxf6 12. g5'),
			moves => {
				'a2a3' => 'r1b1k2r/2qnbppp/p2pp3/1p4P1/3NPP2/P1N2Q2/1PP4P/2KR1B1R b kq - 0 13',
				'f4f5' => 'r1b1k2r/2qnbppp/p2pp3/1p3PP1/3NP3/2N2Q2/PPP4P/2KR1B1R b kq - 0 13',
			},
		},
		'r1b1k2r/2qnbppp/p2pp3/1p4P1/3NPP2/P1N2Q2/1PP4P/2KR1B1R b kq - 0 13' => {
			eco => 'B99r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. a3

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11.Bxf6 Nxf6 12. g5 Nd7 13. a3'),
			moves => {
			},
		},
		'r1b1k2r/2qnbppp/p2ppB2/1p6/3NPP2/2N2Q2/PPP1B1PP/2KR3R b kq - 0 11' => {
			eco => 'B99c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres Variation'),
			moves => {
				'd7f6' => 'r1b1k2r/2q1bppp/p2ppn2/1p6/3NPP2/2N2Q2/PPP1B1PP/2KR3R w kq - 0 12',
			},
		},
		'r1b1k2r/2qnbppp/p2ppB2/1p6/3NPPP1/2N2Q2/PPP4P/2KR1B1R b kq - 0 11' => {
			eco => 'B99p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11.Bxf6'),
			moves => {
				'd7f6' => 'r1b1k2r/2q1bppp/p2ppn2/1p6/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq - 0 12',
				'g7f6' => 'r1b1k2r/2qnbp1p/p2ppp2/1p6/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq - 0 12',
			},
		},
		'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP1B1PP/2KR3R w kq b6 0 11' => {
			eco => 'B99c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres Variation'),
			moves => {
				'g5f6' => 'r1b1k2r/2qnbppp/p2ppB2/1p6/3NPP2/2N2Q2/PPP1B1PP/2KR3R b kq - 0 11',
			},
		},
		'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KR3R w kq b6 0 11' => {
			eco => 'B99j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 b5

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 b5'),
			moves => {
				'h1e1' => 'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KRR3 b kq - 1 11',
			},
		},
		'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KRR3 b kq - 1 11' => {
			eco => 'B99j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 b5'),
			moves => {
				'c8b7' => 'r3k2r/1bqnbppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KRR3 w kq - 2 12',
			},
		},
		'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPPP1/2N2Q2/PPP4P/2KR1B1R w kq b6 0 11' => {
			eco => 'B99n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5'),
			moves => {
				'a2a3' => 'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPPP1/P1N2Q2/1PP4P/2KR1B1R b kq - 0 11',
				'f1d3' => 'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPPP1/2NB1Q2/PPP4P/2KR3R b kq - 1 11',
				'g5f6' => 'r1b1k2r/2qnbppp/p2ppB2/1p6/3NPPP1/2N2Q2/PPP4P/2KR1B1R b kq - 0 11',
			},
		},
		'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPPP1/2NB1Q2/PPP4P/2KR3R b kq - 1 11' => {
			eco => 'B99n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bd3

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11. Bd3'),
			moves => {
				'c8b7' => 'r3k2r/1bqnbppp/p2ppn2/1p4B1/3NPPP1/2NB1Q2/PPP4P/2KR3R w kq - 2 12',
			},
		},
		'r1b1k2r/2qnbppp/p2ppn2/1p4B1/3NPPP1/P1N2Q2/1PP4P/2KR1B1R b kq - 0 11' => {
			eco => 'B99o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. a3

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11. a3'),
			moves => {
			},
		},
		'r1b1k2r/2qp1ppp/p3pn2/1pb5/4P3/2NQ4/PPP1BPPP/R1B2R1K w kq b6 0 11' => {
			eco => 'B47y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. Kh1 Nxd4 9. Qxd4 Bc5 10. Qd3 b5

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Nxd4, 10. Qd3 b5'),
			moves => {
				'f2f4' => 'r1b1k2r/2qp1ppp/p3pn2/1pb5/4PP2/2NQ4/PPP1B1PP/R1B2R1K b kq f3 0 11',
			},
		},
		'r1b1k2r/2qp1ppp/p3pn2/1pb5/4PP2/2NQ4/PPP1B1PP/R1B2R1K b kq f3 0 11' => {
			eco => 'B47z',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. Kh1 Nxd4 9. Qxd4 Bc5 10. Qd3 b5 11. f4

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Nxd4, 10. Qd3 b5 11. f4'),
			moves => {
			},
		},
		'r1b1k2r/2qpbppp/p1p1pn2/8/N3P3/4B3/PPP1BPPP/R2Q1RK1 w kq - 0 11' => {
			eco => 'B49m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Na4 Be7 10. Nxc6 bxc6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9. Na4 Be7 10.Nxc6 bxc6'),
			moves => {
				'a4b6' => 'r1b1k2r/2qpbppp/pNp1pn2/8/4P3/4B3/PPP1BPPP/R2Q1RK1 b kq - 1 11',
			},
		},
		'r1b1k2r/2qpbppp/pNp1pn2/8/4P3/4B3/PPP1BPPP/R2Q1RK1 b kq - 1 11' => {
			eco => 'B49n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Na4 Be7 10. Nxc6 bxc6 11. Nb6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9. Na4 Be7 10.Nxc6 bxc6 11. Nb6'),
			moves => {
			},
		},
		'r1b1k2r/4b2p/p1p1pp2/4p3/4N3/q7/P1PQ2PP/1R2KB1R w Kkq - 2 16' => {
			eco => 'B97w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 dxe5 14. Bxf6 gxf6 15. Ne4 Be7

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4 Be7'),
			moves => {
				'f1e2' => 'r1b1k2r/4b2p/p1p1pp2/4p3/4N3/q7/P1PQB1PP/1R2K2R b Kkq - 3 16',
			},
		},
		'r1b1k2r/4b2p/p1p1pp2/4p3/4N3/q7/P1PQB1PP/1R2K2R b Kkq - 3 16' => {
			eco => 'B97w',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4 Be7'),
			moves => {
				'h7h5' => 'r1b1k2r/4b3/p1p1pp2/4p2p/4N3/q7/P1PQB1PP/1R2K2R w Kkq h6 0 17',
			},
		},
		'r1b1k2r/4b3/p1p1pN2/4p2p/q7/1R6/P1PQB1PP/4K2R b Kkq - 0 18' => {
			eco => 'B97y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 dxe5 14. Bxf6 gxf6 15. Ne4 Be7 16. Be2 h5 17. Rb3 Qa4 18. Nxf6+

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5 with 18.Nxf6+'),
			moves => {
			},
		},
		'r1b1k2r/4b3/p1p1pp2/4p2p/4N3/q7/P1PQB1PP/1R2K2R w Kkq h6 0 17' => {
			eco => 'B97w',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4 Be7'),
			moves => {
				'b1b3' => 'r1b1k2r/4b3/p1p1pp2/4p2p/4N3/qR6/P1PQB1PP/4K2R b Kkq - 1 17',
			},
		},
		'r1b1k2r/4b3/p1p1pp2/4p2p/4N3/qR6/P1PQB1PP/4K2R b Kkq - 1 17' => {
			eco => 'B97w',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4 Be7'),
			moves => {
				'a3a4' => 'r1b1k2r/4b3/p1p1pp2/4p2p/q3N3/1R6/P1PQB1PP/4K2R w Kkq - 2 18',
			},
		},
		'r1b1k2r/4b3/p1p1pp2/4p2p/q1P1N3/1R6/P2QB1PP/4K2R b Kkq c3 0 18' => {
			eco => 'B97x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 dxe5 14. Bxf6 gxf6 15. Ne4 Be7 16. Be2 h5 17. Rb3 Qa4 18. c4

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5 with 18. c4'),
			moves => {
			},
		},
		'r1b1k2r/4b3/p1p1pp2/4p2p/q3N3/1R6/P1PQB1PP/4K2R w Kkq - 2 18' => {
			eco => 'B97w',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4 Be7'),
			moves => {
				'c2c4' => 'r1b1k2r/4b3/p1p1pp2/4p2p/q1P1N3/1R6/P2QB1PP/4K2R b Kkq c3 0 18',
				'e4f6' => 'r1b1k2r/4b3/p1p1pN2/4p2p/q7/1R6/P1PQB1PP/4K2R b Kkq - 0 18',
			},
		},
		'r1b1k2r/p1q2pp1/2pb1n1p/n3N3/3Pp3/8/PPP1BPPP/RNBQK2R w KQkq - 1 12' => {
			eco => 'C59j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. d4 Qc7

			variation => N__('Two Knights: Main Line, 11. d4 Qc7'),
			moves => {
				'c1d2' => 'r1b1k2r/p1q2pp1/2pb1n1p/n3N3/3Pp3/8/PPPBBPPP/RN1QK2R b KQkq - 2 12',
			},
		},
		'r1b1k2r/p1q2pp1/2pb1n1p/n3N3/3Pp3/8/PPPBBPPP/RN1QK2R b KQkq - 2 12' => {
			eco => 'C59j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. d4 Qc7 12. Bd2

			variation => N__('Two Knights: Main Line, Knorre Variation'),
			moves => {
			},
		},
		'r1b1k2r/p1q2pp1/2pb1n1p/n7/8/1P1N4/P1P1BPPP/RNBQK2R b KQkq - 0 13' => {
			eco => 'C59l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. d4 exd3 12. Nxd3 Qc7 13. b3

			variation => N__('Two Knights: Main Line, 11. d4, Honfi Variation'),
			moves => {
			},
		},
		'r1b1k2r/p1q2pp1/2pb1n1p/n7/8/3N4/PPP1BPPP/RNBQK2R w KQkq - 1 13' => {
			eco => 'C59k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. d4 exd3 12. Nxd3 Qc7

			variation => N__('Two Knights: Main Line, 11. d4 exd3 12.Nxd3 Qc7'),
			moves => {
				'b2b3' => 'r1b1k2r/p1q2pp1/2pb1n1p/n7/8/1P1N4/P1P1BPPP/RNBQK2R b KQkq - 0 13',
			},
		},
		'r1b1k2r/pp1n1pb1/2p1pqpp/8/1PBP4/2N1PN2/P4PPP/R2Q1RK1 b kq b3 0 11' => {
			eco => 'D43w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3 dxc4 9. Bxc4 g6 10. O-O Bg7 11. b4

			variation => N__('Semi-Slav: Moscow, Main Line, 11. b4'),
			moves => {
			},
		},
		'r1b1k2r/pp1n1pb1/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/2RQ1RK1 b kq - 3 11' => {
			eco => 'D43x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3 dxc4 9. Bxc4 g6 10. O-O Bg7 11. Rc1

			variation => N__('Semi-Slav: Moscow, Main Line, 11. Rc1'),
			moves => {
			},
		},
		'r1b1k2r/pp1n1pb1/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/R2Q1RK1 w kq - 2 11' => {
			eco => 'D43v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3 dxc4 9. Bxc4 g6 10. O-O Bg7

			variation => N__('Semi-Slav: Moscow, Main Line'),
			moves => {
				'a1c1' => 'r1b1k2r/pp1n1pb1/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/2RQ1RK1 b kq - 3 11',
				'b2b4' => 'r1b1k2r/pp1n1pb1/2p1pqpp/8/1PBP4/2N1PN2/P4PPP/R2Q1RK1 b kq b3 0 11',
			},
		},
		'r1b1k2r/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PP1N1PPP/R2QKB1R w KQkq - 4 8' => {
			eco => 'D52i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4

			variation => N__('QGD: Cambridge Springs, Bogoljubow Variation'),
			moves => {
				'd1c2' => 'r1b1k2r/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PPQN1PPP/R3KB1R b KQkq - 5 8',
			},
		},
		'r1b1k2r/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PPQN1PPP/R3KB1R b KQkq - 5 8' => {
			eco => 'D52j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4 8. Qc2

			variation => N__('QGD: Cambridge Springs, Bogoljubow, 8. Qc2'),
			moves => {
				'd5c4' => 'r1b1k2r/pp1n1ppp/2p1pn2/q5B1/1bpP4/2N1P3/PPQN1PPP/R3KB1R w KQkq - 0 9',
				'e8g8' => 'r1b2rk1/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PPQN1PPP/R3KB1R w KQ - 6 9',
			},
		},
		'r1b1k2r/pp1n1ppp/2p1pn2/q5B1/1bpP4/2N1P3/PPQN1PPP/R3KB1R w KQkq - 0 9' => {
			eco => 'D52k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4 8. Qc2 dxc4

			variation => N__('QGD: Cambridge Springs, 8. Qc2 dxc4'),
			moves => {
			},
		},
		'r1b1k2r/pp1pppbp/2n2np1/q7/2BNP3/2N1B3/PPP2PPP/R2Q1RK1 b kq - 7 8' => {
			eco => 'B35b',
                        # TRANSLATORS:

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5'),
			moves => {
				'e8g8' => 'r1b2rk1/pp1pppbp/2n2np1/q7/2BNP3/2N1B3/PPP2PPP/R2Q1RK1 w - - 8 9',
			},
		},
		'r1b1k2r/pp1pppbp/2n2np1/q7/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 6 8' => {
			eco => 'B35b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 Qa5

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5'),
			moves => {
				'e1g1' => 'r1b1k2r/pp1pppbp/2n2np1/q7/2BNP3/2N1B3/PPP2PPP/R2Q1RK1 b kq - 7 8',
			},
		},
		'r1b1k2r/pp1pppbp/4n1p1/q7/2P1P3/2N1B3/PP1Q1PPP/2R1KB1R b Kkq - 5 11' => {
			eco => 'B39l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 Ne6 10. Rc1 Qa5 11. Qd2

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 Ne6 10. Rc1 Qa5 11. Qd2'),
			moves => {
			},
		},
		'r1b1k2r/pp1pppbp/4n1p1/q7/2P1P3/2N1B3/PP3PPP/2RQKB1R w Kkq - 4 11' => {
			eco => 'B39k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 Ne6 10. Rc1 Qa5

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 Ne6 10. Rc1 Qa5'),
			moves => {
				'd1d2' => 'r1b1k2r/pp1pppbp/4n1p1/q7/2P1P3/2N1B3/PP1Q1PPP/2R1KB1R b Kkq - 5 11',
			},
		},
		'r1b1k2r/pp2bppp/1qnppn2/8/2B1P3/1NN1B3/PPP2PPP/R2Q1RK1 b kq - 3 9' => {
			eco => 'B57o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3 e6 8. O-O Be7 9. Be3

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3 e6 8.O-O Be7 9. Be3'),
			moves => {
			},
		},
		'r1b1k2r/pp2bppp/1qnppn2/8/2B1P3/1NN5/PPP2PPP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'B57n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3 e6 8. O-O Be7

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3 e6 8.O-O Be7'),
			moves => {
				'c1e3' => 'r1b1k2r/pp2bppp/1qnppn2/8/2B1P3/1NN1B3/PPP2PPP/R2Q1RK1 b kq - 3 9',
			},
		},
		'r1b1k2r/pp2nppp/2n1p3/q1ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'C19n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Qa5

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Qa5'),
			moves => {
				'c1d2' => 'r1b1k2r/pp2nppp/2n1p3/q1ppP3/P2P4/2P2N2/2PB1PPP/R2QKB1R b KQkq - 2 9',
				'd1d2' => 'r1b1k2r/pp2nppp/2n1p3/q1ppP3/P2P4/2P2N2/2PQ1PPP/R1B1KB1R b KQkq - 2 9',
			},
		},
		'r1b1k2r/pp2nppp/2n1p3/q1ppP3/P2P4/2P2N2/2PB1PPP/R2QKB1R b KQkq - 2 9' => {
			eco => 'C19p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Bd2

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Qa5+ 9. Bd2'),
			moves => {
				'c8d7' => 'r3k2r/pp1bnppp/2n1p3/q1ppP3/P2P4/2P2N2/2PB1PPP/R2QKB1R w KQkq - 3 10',
			},
		},
		'r1b1k2r/pp2nppp/2n1p3/q1ppP3/P2P4/2P2N2/2PQ1PPP/R1B1KB1R b KQkq - 2 9' => {
			eco => 'C19n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Qd2

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Qd2'),
			moves => {
				'c8d7' => 'r3k2r/pp1bnppp/2n1p3/q1ppP3/P2P4/2P2N2/2PQ1PPP/R1B1KB1R w KQkq - 3 10',
			},
		},
		'r1b1k2r/pp2ppbp/2n3p1/q1p5/3PP3/2P1BN2/P2Q1PPP/2R1KB1R b Kkq - 5 10' => {
			eco => 'D85v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 Qa5 9. Qd2 Nc6 10. Rc1

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 Qa5 9. Qd2 Nc6 10. Rc1'),
			moves => {
			},
		},
		'r1b1k2r/pp2ppbp/2n3p1/q1p5/3PP3/2P1BN2/P2Q1PPP/R3KB1R w KQkq - 4 10' => {
			eco => 'D85u',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 Qa5 9. Qd2'),
			moves => {
				'a1c1' => 'r1b1k2r/pp2ppbp/2n3p1/q1p5/3PP3/2P1BN2/P2Q1PPP/2R1KB1R b Kkq - 5 10',
			},
		},
		'r1b1k2r/pp3ppp/2nqpn2/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 9' => {
			eco => 'D13t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 e6 7. e3 Bd6 8. Bxd6 Qxd6

			variation => N__('Slav: Exchange, 6. Bf4 e6 7. e3 Bd6 8.Bxd6 Qxd6'),
			moves => {
			},
		},
		'r1b1k2r/pp4pp/1qnbpn2/3p4/3P4/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 5 12' => {
			eco => 'C06p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3 Qb6 12. Nc3

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6, 11... Qb6 12. Nc3'),
			moves => {
			},
		},
		'r1b1k2r/pp4pp/1qnbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 w kq - 4 12' => {
			eco => 'C06o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3 Qb6

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6, 11... Qb6'),
			moves => {
				'e2c3' => 'r1b1k2r/pp4pp/1qnbpn2/3p4/3P4/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 5 12',
			},
		},
		'r1b1k2r/ppp1qppp/2nppn2/8/2PP4/5NP1/PP1NPPBP/R2Q1RK1 b kq - 1 8' => {
			eco => 'E11v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+ 7. Nbxd2 d6 8. O-O

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line'),
			moves => {
				'a7a5' => 'r1b1k2r/1pp1qppp/2nppn2/p7/2PP4/5NP1/PP1NPPBP/R2Q1RK1 w kq a6 0 9',
				'e8g8' => 'r1b2rk1/ppp1qppp/2nppn2/8/2PP4/5NP1/PP1NPPBP/R2Q1RK1 w - - 2 9',
			},
		},
		'r1b1k2r/ppp1qppp/2nppn2/8/2PP4/5NP1/PP1NPPBP/R2QK2R w KQkq - 0 8' => {
			eco => 'E11u',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Bg2 Bxd2+ 7.Nbxd2'),
			moves => {
				'e1g1' => 'r1b1k2r/ppp1qppp/2nppn2/8/2PP4/5NP1/PP1NPPBP/R2Q1RK1 b kq - 1 8',
			},
		},
		'r1b1k2r/ppp1qppp/3p1n2/4P1B1/2BnP3/8/PPP3PP/RN1Q1RK1 b kq - 0 9' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'd6e5' => 'r1b1k2r/ppp1qppp/5n2/4p1B1/2BnP3/8/PPP3PP/RN1Q1RK1 w kq - 0 10',
			},
		},
		'r1b1k2r/ppp1qppp/3p1n2/4p1B1/2BnPP2/8/PPP3PP/RN1Q1RK1 w kq - 1 9' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'f4e5' => 'r1b1k2r/ppp1qppp/3p1n2/4P1B1/2BnP3/8/PPP3PP/RN1Q1RK1 b kq - 0 9',
			},
		},
		'r1b1k2r/ppp1qppp/5n2/4p1B1/2BnP3/2N5/PPP3PP/R2Q1RK1 b kq - 1 10' => {
			eco => 'C55b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. O-O Bc5 5. d4 Bxd4 6. Nxd4 Nxd4 7. Bg5 d6 8. f4 Qe7 9. fxe5 dxe5 10. Nc3

			variation => N__('Two Knights: 4.O-O Gambit, Holzhausen Attack'),
			moves => {
			},
		},
		'r1b1k2r/ppp1qppp/5n2/4p1B1/2BnP3/8/PPP3PP/RN1Q1RK1 w kq - 0 10' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'b1c3' => 'r1b1k2r/ppp1qppp/5n2/4p1B1/2BnP3/2N5/PPP3PP/R2Q1RK1 b kq - 1 10',
			},
		},
		'r1b1k2r/pppp1ppp/5q2/1Bb5/8/2P5/PPP2PPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'C48l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Nxd4 exd4 6. e5 dxc3 7. exf6 Qxf6 8. dxc3 Bc5

			variation => N__('Four Knights: Rubinstein, Exchange Variation, 8... Bc5'),
			moves => {
			},
		},
		'r1b1k2r/ppppnp1p/2n4b/4q3/2B2p2/2NP1Q2/PPPB2PP/4RRK1 b kq - 6 11' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O gxf3 6. Qxf3 Qf6 7. e5 Qxe5 8. d3 Bh6 9. Nc3 Ne7 10. Bd2 Nbc6 11. Rae1

			variation => N__('KGA: Muzio Gambit, Paulsen Variation'),
			moves => {
			},
		},
		'r1b1k2r/ppppnp1p/2n4b/4q3/2B2p2/2NP1Q2/PPPB2PP/R4RK1 w kq - 5 11' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'a1e1' => 'r1b1k2r/ppppnp1p/2n4b/4q3/2B2p2/2NP1Q2/PPPB2PP/4RRK1 b kq - 6 11',
			},
		},
		'r1b1k2r/ppppnppp/2n2q2/1Bb5/3NP3/2P1B3/PP3PPP/RN1QK2R b KQkq - 2 7' => {
			eco => 'C45w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Bb5

			variation => N__('Scotch: Paulsen Attack'),
			moves => {
				'c6d8' => 'r1bnk2r/ppppnppp/5q2/1Bb5/3NP3/2P1B3/PP3PPP/RN1QK2R w KQkq - 3 8',
			},
		},
		'r1b1k2r/ppppnppp/2n2q2/2b5/2BNP3/2P1B3/PP3PPP/RN1QK2R b KQkq - 2 7' => {
			eco => 'C45x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Bc4

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Bc4'),
			moves => {
				'c6e5' => 'r1b1k2r/ppppnppp/5q2/2b1n3/2BNP3/2P1B3/PP3PPP/RN1QK2R w KQkq - 3 8',
			},
		},
		'r1b1k2r/ppppnppp/2n2q2/2b5/3NP3/2P1B3/PP1Q1PPP/RN2KB1R b KQkq - 2 7' => {
			eco => 'C45w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Qd2

			variation => N__('Scotch: Blackburne Attack'),
			moves => {
			},
		},
		'r1b1k2r/ppppnppp/2n2q2/2b5/3NP3/2P1B3/PP3PPP/RN1QKB1R w KQkq - 1 7' => {
			eco => 'C45w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7'),
			moves => {
				'd1d2' => 'r1b1k2r/ppppnppp/2n2q2/2b5/3NP3/2P1B3/PP1Q1PPP/RN2KB1R b KQkq - 2 7',
				'd4c2' => 'r1b1k2r/ppppnppp/2n2q2/2b5/4P3/2P1B3/PPN2PPP/RN1QKB1R b KQkq - 2 7',
				'f1b5' => 'r1b1k2r/ppppnppp/2n2q2/1Bb5/3NP3/2P1B3/PP3PPP/RN1QK2R b KQkq - 2 7',
				'f1c4' => 'r1b1k2r/ppppnppp/2n2q2/2b5/2BNP3/2P1B3/PP3PPP/RN1QK2R b KQkq - 2 7',
			},
		},
		'r1b1k2r/ppppnppp/2n2q2/2b5/4P3/2P1B3/PPN2PPP/RN1QKB1R b KQkq - 2 7' => {
			eco => 'C45w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Nc2

			variation => N__('Scotch: Meitner Variation'),
			moves => {
			},
		},
		'r1b1k2r/ppppnppp/2n3q1/b3P3/2B5/1QN2N2/P4PPP/R1B2RK1 w kq - 1 11' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'c1a3' => 'r1b1k2r/ppppnppp/2n3q1/b3P3/2B5/BQN2N2/P4PPP/R4RK1 b kq - 2 11',
				'f1d1' => 'r1b1k2r/ppppnppp/2n3q1/b3P3/2B5/1QN2N2/P4PPP/R1BR2K1 b kq - 2 11',
			},
		},
		'r1b1k2r/ppppnppp/2n3q1/b3P3/2B5/1QN2N2/P4PPP/R1BR2K1 b kq - 2 11' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 exd4 7. O-O dxc3 8. Qb3 Qf6 9. e5 Qg6 10. Nxc3 Nge7 11. Rd1

			variation => N__('Evans Gambit: Compromised Defence, Potter Variation'),
			moves => {
			},
		},
		'r1b1k2r/ppppnppp/2n3q1/b3P3/2B5/BQN2N2/P4PPP/R4RK1 b kq - 2 11' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 exd4 7. O-O dxc3 8. Qb3 Qf6 9. e5 Qg6 10. Nxc3 Nge7 11. Ba3

			variation => N__('Evans Gambit: Compromised Defence, Paulsen Variation'),
			moves => {
			},
		},
		'r1b1k2r/ppppnppp/5q2/2b1n3/2BNP3/2P1B3/PP3PPP/RN1QK2R w KQkq - 3 8' => {
			eco => 'C45y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Bc4 Ne5

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Bc4 Ne5'),
			moves => {
			},
		},
		'r1b1k2r/ppppqNpp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQ1K1R w kq - 2 7' => {
			eco => 'C57c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1'),
			moves => {
				'f7h8' => 'r1b1k2N/ppppq1pp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQ1K1R b q - 0 7',
			},
		},
		'r1b1k2r/ppppqppp/1bn2n2/4p3/2BPP3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 3 7' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'a2a4' => 'r1b1k2r/ppppqppp/1bn2n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQ1RK1 b kq a3 0 7',
			},
		},
		'r1b1k2r/ppppqppp/1bn2n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQ1RK1 b kq a3 0 7' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'a7a6' => 'r1b1k2r/1pppqppp/pbn2n2/4p3/P1BPP3/2P2N2/1P3PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1b1k2r/ppppqppp/2n1p3/8/2PPn3/2B2NP1/PP2PP1P/2RQKB1R b Kkq - 2 8' => {
			eco => 'E11s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Nc3 Bxc3 7. Bxc3 Ne4 8. Rc1

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Nc3 Bxc3 7.Bxc3 Ne4 8. Rc1'),
			moves => {
			},
		},
		'r1b1k2r/ppppqppp/2n1p3/8/2PPn3/2B2NP1/PP2PP1P/R2QKB1R w KQkq - 1 8' => {
			eco => 'E11s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Nc3 Bxc3 7. Bxc3 Ne4

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Nc3 Bxc3 7.Bxc3 Ne4'),
			moves => {
				'a1c1' => 'r1b1k2r/ppppqppp/2n1p3/8/2PPn3/2B2NP1/PP2PP1P/2RQKB1R b Kkq - 2 8',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/1bPP4/2N2NP1/PP1BPP1P/R2QKB1R b KQkq - 2 6' => {
			eco => 'E11r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Nc3

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Nc3'),
			moves => {
				'b4c3' => 'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/2b2NP1/PP1BPP1P/R2QKB1R w KQkq - 0 7',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R w KQkq - 1 6' => {
			eco => 'E11r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6'),
			moves => {
				'b1c3' => 'r1b1k2r/ppppqppp/2n1pn2/8/1bPP4/2N2NP1/PP1BPP1P/R2QKB1R b KQkq - 2 6',
				'f1g2' => 'r1b1k2r/ppppqppp/2n1pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R b KQkq - 2 6',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R b KQkq - 2 6' => {
			eco => 'E11t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Bg2'),
			moves => {
				'b4d2' => 'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/5NP1/PP1bPPBP/RN1QK2R w KQkq - 0 7',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/2B2NP1/PP2PP1P/R2QKB1R b KQkq - 0 7' => {
			eco => 'E11s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Nc3 Bxc3 7. Bxc3

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Nc3 Bxc3 7.Bxc3'),
			moves => {
				'f6e4' => 'r1b1k2r/ppppqppp/2n1p3/8/2PPn3/2B2NP1/PP2PP1P/R2QKB1R w KQkq - 1 8',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/2b2NP1/PP1BPP1P/R2QKB1R w KQkq - 0 7' => {
			eco => 'E11s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Nc3 Bxc3

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Nc3 Bxc3'),
			moves => {
				'd2c3' => 'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/2B2NP1/PP2PP1P/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/5NP1/PP1NPPBP/R2QK2R b KQkq - 0 7' => {
			eco => 'E11u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+ 7. Nbxd2

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Bg2 Bxd2+ 7.Nbxd2'),
			moves => {
				'd7d6' => 'r1b1k2r/ppp1qppp/2nppn2/8/2PP4/5NP1/PP1NPPBP/R2QK2R w KQkq - 0 8',
			},
		},
		'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/5NP1/PP1bPPBP/RN1QK2R w KQkq - 0 7' => {
			eco => 'E11t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 6. Bg2 Bxd2+'),
			moves => {
				'b1d2' => 'r1b1k2r/ppppqppp/2n1pn2/8/2PP4/5NP1/PP1NPPBP/R2QK2R b KQkq - 0 7',
			},
		},
		'r1b1k2r/ppppqppp/2n5/4P3/1bP2Bn1/4PN2/PP1N1PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'A52m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Bf4 Nc6 5. Nf3 Bb4+ 6. Nbd2 Qe7 7. e3

			variation => N__('Budapest: Rubinstein, Main Line, 6.Nbd2 Qe7 7. e3'),
			moves => {
			},
		},
		'r1b1k2r/ppppqppp/2n5/4P3/1bP2Bn1/5N2/PP1NPPPP/R2QKB1R w KQkq - 7 7' => {
			eco => 'A52l',
                        # TRANSLATORS:

			variation => N__('Budapest: Rubinstein, Main Line, 6.Nbd2'),
			moves => {
				'e2e3' => 'r1b1k2r/ppppqppp/2n5/4P3/1bP2Bn1/4PN2/PP1N1PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1b1k2r/ppq1bppp/2nppn2/8/3NPP2/2N1BQ2/PPP3PP/R3KB1R w KQkq - 5 9' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6 7. Be3 Be7 8. Qf3 Qc7

			variation => N__('Sicilian: Scheveningen, Tal, 8... Qc7'),
			moves => {
			},
		},
		'r1b1k2r/ppq3pp/2nbpn2/3p2B1/3P4/3B1N2/PP2NPPP/R2Q1RK1 b kq - 5 12' => {
			eco => 'C06r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3 Qc7 12. Bg5

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6, 11... Qc7 12. Bg5'),
			moves => {
			},
		},
		'r1b1k2r/ppq3pp/2nbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 w kq - 4 12' => {
			eco => 'C06q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3 Qc7

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6, 11... Qc7'),
			moves => {
				'c1g5' => 'r1b1k2r/ppq3pp/2nbpn2/3p2B1/3P4/3B1N2/PP2NPPP/R2Q1RK1 b kq - 5 12',
			},
		},
		'r1b1k2r/ppqn1pp1/2pbp2p/8/3PQ3/3B1N2/PPP2PPP/R1B1K2R w KQkq - 1 11' => {
			eco => 'B17n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6 6. Bd3 e6 7. N1f3 Bd6 8. Qe2 h6 9. Ne4 Nxe4 10. Qxe4 Qc7

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3, Main Line 10.Qxe4 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/1p1n1ppp/p2ppn2/q5B1/2BNP3/2N5/PPPQ1PPP/2KR3R b kq - 1 9' => {
			eco => 'B94k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. Bc4 Qa5 8. Qd2 e6 9. O-O-O

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4 Qa5 8. Qd2 e6 9.O-O-O'),
			moves => {
			},
		},
		'r1b1kb1r/1p1n1ppp/p2ppn2/q5B1/2BNP3/2N5/PPPQ1PPP/R3K2R w KQkq - 0 9' => {
			eco => 'B94j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4 Qa5'),
			moves => {
				'e1c1' => 'r1b1kb1r/1p1n1ppp/p2ppn2/q5B1/2BNP3/2N5/PPPQ1PPP/2KR3R b kq - 1 9',
			},
		},
		'r1b1kb1r/1p1n1ppp/pq1ppn2/6B1/4PP2/1NN2Q2/PPP3PP/R3KB1R b KQkq - 4 9' => {
			eco => 'B97g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Nb3 Nbd7 9. Qf3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Nb3 Nbd7 9. Qf3'),
			moves => {
				'f8e7' => 'r1b1k2r/1p1nbppp/pq1ppn2/6B1/4PP2/1NN2Q2/PPP3PP/R3KB1R w KQkq - 5 10',
			},
		},
		'r1b1kb1r/1p1n1ppp/pq1ppn2/6B1/4PP2/1NN5/PPP3PP/R2QKB1R w KQkq - 3 9' => {
			eco => 'B97e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Nb3'),
			moves => {
				'd1f3' => 'r1b1kb1r/1p1n1ppp/pq1ppn2/6B1/4PP2/1NN2Q2/PPP3PP/R3KB1R b KQkq - 4 9',
			},
		},
		'r1b1kb1r/1p1npppp/p2p1n2/q5B1/2BNP3/2N5/PPP2PPP/R2QK2R w KQkq - 4 8' => {
			eco => 'B94j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. Bc4 Qa5

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4 Qa5'),
			moves => {
				'd1d2' => 'r1b1kb1r/1p1npppp/p2p1n2/q5B1/2BNP3/2N5/PPPQ1PPP/R3K2R b KQkq - 5 8',
			},
		},
		'r1b1kb1r/1p1npppp/p2p1n2/q5B1/2BNP3/2N5/PPPQ1PPP/R3K2R b KQkq - 5 8' => {
			eco => 'B94j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4 Qa5'),
			moves => {
				'e7e6' => 'r1b1kb1r/1p1n1ppp/p2ppn2/q5B1/2BNP3/2N5/PPPQ1PPP/R3K2R w KQkq - 0 9',
			},
		},
		'r1b1kb1r/1p3p1p/p1nppp2/8/3NPP2/q1N5/P1PQB1PP/1R2K2R w Kkq - 2 12' => {
			eco => 'B97o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. Bxf6 gxf6 11. Be2 Nc6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10.Bxf6 gxf6 11. Be2 Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/1p3ppp/p1npPn2/6B1/3NP3/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 11' => {
			eco => 'B97q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. f5 Nc6'),
			moves => {
				'f7e6' => 'r1b1kb1r/1p4pp/p1nppn2/6B1/3NP3/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 12',
			},
		},
		'r1b1kb1r/1p3ppp/p1nppn2/5PB1/3NP3/q1N5/P1PQ2PP/1R2KB1R w Kkq - 1 11' => {
			eco => 'B97q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. f5 Nc6'),
			moves => {
				'f5e6' => 'r1b1kb1r/1p3ppp/p1npPn2/6B1/3NP3/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 11',
			},
		},
		'r1b1kb1r/1p3ppp/p1nppn2/6B1/4PP2/1NN5/PqPQ2PP/R3KB1R w KQkq - 2 10' => {
			eco => 'B97j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Nb3 Nc6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 9. Nb3 Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/1p3ppp/p2qpn2/8/2BN4/2P5/PP3PPP/R1BQ1RK1 b kq - 0 11' => {
			eco => 'C07s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6 7. O-O Nf6 8. Nb3 Nc6 9. Nbxd4 Nxd4 10. Nxd4 a6 11. c3

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5, Main Line, 10.Nxd4 a6 11. c3'),
			moves => {
			},
		},
		'r1b1kb1r/1p3ppp/p2qpn2/8/2BN4/8/PPP2PPP/R1BQ1RK1 w kq - 0 11' => {
			eco => 'C07r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6 7. O-O Nf6 8. Nb3 Nc6 9. Nbxd4 Nxd4 10. Nxd4 a6

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5, Main Line, 10.Nxd4 a6'),
			moves => {
				'c2c3' => 'r1b1kb1r/1p3ppp/p2qpn2/8/2BN4/2P5/PP3PPP/R1BQ1RK1 b kq - 0 11',
				'f1e1' => 'r1b1kb1r/1p3ppp/p2qpn2/8/2BN4/8/PPP2PPP/R1BQR1K1 b kq - 1 11',
			},
		},
		'r1b1kb1r/1p3ppp/p2qpn2/8/2BN4/8/PPP2PPP/R1BQR1K1 b kq - 1 11' => {
			eco => 'C07t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6 7. O-O Nf6 8. Nb3 Nc6 9. Nbxd4 Nxd4 10. Nxd4 a6 11. Re1

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5, Main Line, 10.Nxd4 a6 11. Re1'),
			moves => {
			},
		},
		'r1b1kb1r/1p3ppp/pqnppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R b kq - 1 9' => {
			eco => 'B63f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Qb6 8. Nb3 a6 9. O-O-O

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Qb6 8. Nb3 a6 9.O-O-O'),
			moves => {
				'f8e7' => 'r1b1k2r/1p2bppp/pqnppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w kq - 2 10',
			},
		},
		'r1b1kb1r/1p3ppp/pqnppn2/6B1/4P3/1NN5/PPPQ1PPP/R3KB1R w KQkq - 0 9' => {
			eco => 'B63e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Qb6 8. Nb3 a6

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Qb6 8. Nb3 a6'),
			moves => {
				'e1c1' => 'r1b1kb1r/1p3ppp/pqnppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R b kq - 1 9',
			},
		},
		'r1b1kb1r/1p4pp/p1Nppn2/6B1/4P3/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 12' => {
			eco => 'B97q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. f5 Nc6'),
			moves => {
				'b7c6' => 'r1b1kb1r/6pp/p1pppn2/6B1/4P3/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 13',
			},
		},
		'r1b1kb1r/1p4pp/p1nppn2/6B1/3NP3/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 12' => {
			eco => 'B97q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. f5 Nc6'),
			moves => {
				'd4c6' => 'r1b1kb1r/1p4pp/p1Nppn2/6B1/4P3/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 12',
			},
		},
		'r1b1kb1r/1pp2ppp/2p5/p3Pn2/8/2N2N1P/PPP2PP1/R1B2RK1 w - a6 0 11' => {
			eco => 'C67x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+ Kxd8 9. Nc3 Ke8 10. h3 a5

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap, 9. Nc3 Ke8 10. h3 a5'),
			moves => {
			},
		},
		'r1b1kb1r/1pp2ppp/p1p5/4N3/3qN3/8/PPPP1PPP/R1BQ1RK1 b kq - 2 8' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'd4e5' => 'r1b1kb1r/1pp2ppp/p1p5/4q3/4N3/8/PPPP1PPP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r1b1kb1r/1pp2ppp/p1p5/4N3/3qN3/8/PPPP1PPP/R1BQK2R w KQkq - 1 8' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'e1g1' => 'r1b1kb1r/1pp2ppp/p1p5/4N3/3qN3/8/PPPP1PPP/R1BQ1RK1 b kq - 2 8',
			},
		},
		'r1b1kb1r/1pp2ppp/p1p5/4q3/4N3/8/PPPP1PPP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'f1e1' => 'r1b1kb1r/1pp2ppp/p1p5/4q3/4N3/8/PPPP1PPP/R1BQR1K1 b kq - 1 9',
			},
		},
		'r1b1kb1r/1pp2ppp/p1p5/4q3/4N3/8/PPPP1PPP/R1BQR1K1 b kq - 1 9' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'c8e6' => 'r3kb1r/1pp2ppp/p1p1b3/4q3/4N3/8/PPPP1PPP/R1BQR1K1 w kq - 2 10',
			},
		},
		'r1b1kb1r/1pq2pp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R w kq - 2 10' => {
			eco => 'B66j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Qc7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/1pq2ppp/p1n1pn2/2p5/P1BP4/4PN2/1P2QPPP/RNB2RK1 w kq - 3 9' => {
			eco => 'D27u',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Qe2 Qc7

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq - 4 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. Be3 Nc6

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. Be3 Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'B80t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6 7. Bg2 Qc7 8. O-O a6

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'f1e1' => 'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 b kq - 1 9',
			},
		},
		'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 b kq - 1 9' => {
			eco => 'B80t',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'f8e7' => 'r1b1k2r/1pq1bppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 w kq - 2 10',
			},
		},
		'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K w kq - 4 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. Kh1 Nc6

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. Kh1 Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/1pq2ppp/p1nppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 w kq - 1 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. f4 Nc6

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. f4 Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/1pq2ppp/p1nppn2/8/P2NP3/2N5/1PP1BPPP/R1BQ1RK1 w kq - 1 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. a4 Nc6

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. a4 Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/1pqn1ppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'B93q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4 Qc7

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/1pqn1ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 4 9' => {
			eco => 'B96j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qf3 Qc7'),
			moves => {
				'b7b5' => 'r1b1kb1r/2qn1ppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq b6 0 10',
			},
		},
		'r1b1kb1r/1pqn1ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 3 9' => {
			eco => 'B96j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qf3 Qc7

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qf3 Qc7'),
			moves => {
				'e1c1' => 'r1b1kb1r/1pqn1ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 4 9',
			},
		},
		'r1b1kb1r/1pqn1ppp/p2ppn2/6B1/3NPP2/2N5/PPP1Q1PP/R3KB1R w KQkq - 3 9' => {
			eco => 'B96i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qe2 Qc7

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qe2 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/1pqnpppp/p2p1n2/8/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 3 8' => {
			eco => 'B93i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Qc7 7. Nf3 Nbd7

			variation => N__('Sicilian: Najdorf, 6. f4 Qc7 7. Nf3 Nbd7'),
			moves => {
				'f1d3' => 'r1b1kb1r/1pqnpppp/p2p1n2/8/4PP2/2NB1N2/PPP3PP/R1BQK2R b KQkq - 4 8',
			},
		},
		'r1b1kb1r/1pqnpppp/p2p1n2/8/4PP2/2NB1N2/PPP3PP/R1BQK2R b KQkq - 4 8' => {
			eco => 'B93j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Qc7 7. Nf3 Nbd7 8. Bd3

			variation => N__('Sicilian: Najdorf, 6. f4 Qc7 7. Nf3 Nbd7 8. Bd3'),
			moves => {
			},
		},
		'r1b1kb1r/1pqp1pp1/p1n1pn2/7p/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq h6 0 9' => {
			eco => 'B48p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O h5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O h5'),
			moves => {
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 3 8' => {
			eco => 'B49f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O'),
			moves => {
				'b7b5' => 'r1b1kb1r/2qp1ppp/p1n1pn2/1p6/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq b6 0 9',
				'f8b4' => 'r1b1k2r/1pqp1ppp/p1n1pn2/8/1b1NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq - 4 9',
				'f8e7' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq - 4 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq - 2 8' => {
			eco => 'B49c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6'),
			moves => {
				'a2a3' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/P1N1B3/1PP1BPPP/R2QK2R b KQkq - 0 8',
				'e1g1' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 3 8',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B48g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Qd2'),
			moves => {
				'f2f3' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 0 8',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 0 8' => {
			eco => 'B48h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Qd2 Nf6 8. f3

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Qd2 Nf6 8. f3'),
			moves => {
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'B47m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 Nf6 8. O-O

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 Nf6 8.O-O'),
			moves => {
				'f8c5' => 'r1b1k2r/1pqp1ppp/p1n1pn2/2b5/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 4 9',
				'f8e7' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 4 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B47l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 Nf6

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 Nf6'),
			moves => {
				'e1g1' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K b kq - 3 8' => {
			eco => 'B47w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. Kh1

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1'),
			moves => {
				'c6d4' => 'r1b1kb1r/1pqp1ppp/p3pn2/8/3nP3/2N5/PPP1BPPP/R1BQ1R1K w kq - 0 9',
				'f8e7' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K w kq - 4 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'B47v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6'),
			moves => {
				'a2a3' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/P1N5/1PP1BPPP/R1BQ1RK1 b kq - 0 8',
				'g1h1' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K b kq - 3 8',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2NBB3/PPP2PPP/R2Q1RK1 b kq - 3 8' => {
			eco => 'B48n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O'),
			moves => {
				'b7b5' => 'r1b1kb1r/2qp1ppp/p1n1pn2/1p6/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq b6 0 9',
				'c6d4' => 'r1b1kb1r/1pqp1ppp/p3pn2/8/3nP3/2NBB3/PPP2PPP/R2Q1RK1 w kq - 0 9',
				'c6e5' => 'r1b1kb1r/1pqp1ppp/p3pn2/4n3/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq - 4 9',
				'f8d6' => 'r1b1k2r/1pqp1ppp/p1nbpn2/8/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq - 4 9',
				'h7h5' => 'r1b1kb1r/1pqp1pp1/p1n1pn2/7p/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq h6 0 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2NBB3/PPP2PPP/R2QK2R w KQkq - 2 8' => {
			eco => 'B48m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6'),
			moves => {
				'e1g1' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2NBB3/PPP2PPP/R2Q1RK1 b kq - 3 8',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/P1N1B3/1PP1BPPP/R2QK2R b KQkq - 0 8' => {
			eco => 'B49d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. a3

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8. a3'),
			moves => {
				'b7b5' => 'r1b1kb1r/2qp1ppp/p1n1pn2/1p6/3NP3/P1N1B3/1PP1BPPP/R2QK2R w KQkq b6 0 9',
				'f8e7' => 'r1b1k2r/1pqpbppp/p1n1pn2/8/3NP3/P1N1B3/1PP1BPPP/R2QK2R w KQkq - 1 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/P1N5/1PP1BPPP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'B47v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. a3

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. a3'),
			moves => {
			},
		},
		'r1b1kb1r/1pqp1ppp/p3pn2/4n3/3NP3/2NBB2P/PPP2PP1/R2Q1RK1 b kq - 0 9' => {
			eco => 'B48t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O Ne5 9. h3

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O Ne5 9. h3'),
			moves => {
				'f8c5' => 'r1b1k2r/1pqp1ppp/p3pn2/2b1n3/3NP3/2NBB2P/PPP2PP1/R2Q1RK1 w kq - 1 10',
			},
		},
		'r1b1kb1r/1pqp1ppp/p3pn2/4n3/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq - 4 9' => {
			eco => 'B48s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O Ne5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O Ne5'),
			moves => {
				'h2h3' => 'r1b1kb1r/1pqp1ppp/p3pn2/4n3/3NP3/2NBB2P/PPP2PP1/R2Q1RK1 b kq - 0 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p3pn2/8/3QP3/2N5/PPP1BPPP/R1B2R1K b kq - 0 9' => {
			eco => 'B47y',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Nxd4'),
			moves => {
				'f8c5' => 'r1b1k2r/1pqp1ppp/p3pn2/2b5/3QP3/2N5/PPP1BPPP/R1B2R1K w kq - 1 10',
			},
		},
		'r1b1kb1r/1pqp1ppp/p3pn2/8/3nP3/2N5/PPP1BPPP/R1BQ1R1K w kq - 0 9' => {
			eco => 'B47y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O Nf6 8. Kh1 Nxd4

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O Nf6 8. Kh1 Nxd4'),
			moves => {
				'd1d4' => 'r1b1kb1r/1pqp1ppp/p3pn2/8/3QP3/2N5/PPP1BPPP/R1B2R1K b kq - 0 9',
			},
		},
		'r1b1kb1r/1pqp1ppp/p3pn2/8/3nP3/2NBB3/PPP2PPP/R2Q1RK1 w kq - 0 9' => {
			eco => 'B48o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O Nxd4

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O Nxd4'),
			moves => {
			},
		},
		'r1b1kb1r/1pqpnppp/p1n1p3/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq - 2 8' => {
			eco => 'B49a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nge7

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nge7'),
			moves => {
			},
		},
		'r1b1kb1r/2q2p1p/p2ppP2/1pn5/3NP3/2N2Q2/PPP4P/2KR1B1R w kq - 1 16' => {
			eco => 'B99x',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5 14. f6'),
			moves => {
				'h1g1' => 'r1b1kb1r/2q2p1p/p2ppP2/1pn5/3NP3/2N2Q2/PPP4P/2KR1BR1 b kq - 2 16',
			},
		},
		'r1b1kb1r/2q2p1p/p2ppP2/1pn5/3NP3/2N2Q2/PPP4P/2KR1BR1 b kq - 2 16' => {
			eco => 'B99y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bxf6 Nxf6 12. g5 Nd7 13. f5 Nc5 14. f6 gxf6 15. gxf6 Bf8 16. Rg1

			variation => N__('Sicilian: Najdorf, Modern Main Line 13. f5 Nc5 14. f6: Perenyi\'s 16. Rg1'),
			moves => {
			},
		},
		'r1b1kb1r/2qn1ppp/p1n1p3/1p2P1B1/3N4/2N5/PPP1Q1PP/2KR1B1R w kq - 5 12' => {
			eco => 'B96u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Qe2 Nfd7 11. O-O-O Nc6

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2 Nfd7 11.O-O-O Nc6'),
			moves => {
			},
		},
		'r1b1kb1r/2qn1ppp/p2ppn2/1p2P1B1/3N1P2/2N2Q2/PPP3PP/2KR1B1R b kq - 0 10' => {
			eco => 'B96m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qf3 Qc7 9. O-O-O b5 10. e5

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qf3 Qc7 9.O-O-O b5 10. e5'),
			moves => {
			},
		},
		'r1b1kb1r/2qn1ppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq b6 0 10' => {
			eco => 'B96k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qf3 Qc7 9. O-O-O b5

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qf3 Qc7 9.O-O-O b5'),
			moves => {
				'e4e5' => 'r1b1kb1r/2qn1ppp/p2ppn2/1p2P1B1/3N1P2/2N2Q2/PPP3PP/2KR1B1R b kq - 0 10',
				'f1d3' => 'r1b1kb1r/2qn1ppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KR3R b kq - 1 10',
			},
		},
		'r1b1kb1r/2qn1ppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KR3R b kq - 1 10' => {
			eco => 'B96l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qf3 Qc7 9. O-O-O b5 10. Bd3

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qf3 Qc7 9.O-O-O b5 10. Bd3'),
			moves => {
			},
		},
		'r1b1kb1r/2qn1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 11' => {
			eco => 'D48n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 Qc7

			variation => N__('Semi-Slav: Meran, Reynolds, 10... Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/2qn2pp/p3pn2/1p6/2p1P3/2N2N2/PPB2PPP/R1BQ1RK1 b kq - 3 13' => {
			eco => 'D48p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 c4 11. dxe6 fxe6 12. Bc2 Qc7 13. O-O

			variation => N__('Semi-Slav: Meran, Reynolds, 10... c4 11.dxe6 fxe6 12. Bc2 Qc7 13.O-O'),
			moves => {
			},
		},
		'r1b1kb1r/2qn2pp/p3pn2/1p6/2p1P3/2N2N2/PPB2PPP/R1BQK2R w KQkq - 2 13' => {
			eco => 'D48p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 c4 11. dxe6 fxe6 12. Bc2 Qc7

			variation => N__('Semi-Slav: Meran, Reynolds, 10... c4 11.dxe6 fxe6 12. Bc2 Qc7'),
			moves => {
				'e1g1' => 'r1b1kb1r/2qn2pp/p3pn2/1p6/2p1P3/2N2N2/PPB2PPP/R1BQ1RK1 b kq - 3 13',
			},
		},
		'r1b1kb1r/2qp1ppp/p1n1pn2/1p6/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq b6 0 9' => {
			eco => 'B49f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O b5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O b5'),
			moves => {
			},
		},
		'r1b1kb1r/2qp1ppp/p1n1pn2/1p6/3NP3/2NBB3/PPP2PPP/R2Q1RK1 w kq b6 0 9' => {
			eco => 'B48q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 Nf6 8. O-O b5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 Nf6 8.O-O b5'),
			moves => {
			},
		},
		'r1b1kb1r/2qp1ppp/p1n1pn2/1p6/3NP3/P1N1B3/1PP1BPPP/R2QK2R w KQkq b6 0 9' => {
			eco => 'B49d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. a3 b5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8. a3 b5'),
			moves => {
			},
		},
		'r1b1kb1r/3n1ppp/1q2pP2/1p6/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 1 13' => {
			eco => 'D49j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 axb5 12. exf6 Qb6

			variation => N__('Semi-Slav: Meran, Old Main Line, Botvinnik Variation'),
			moves => {
			},
		},
		'r1b1kb1r/6pp/p1p1pB2/4p3/8/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 14' => {
			eco => 'B97u',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5'),
			moves => {
				'g7f6' => 'r1b1kb1r/7p/p1p1pp2/4p3/8/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 15',
			},
		},
		'r1b1kb1r/6pp/p1p1pn2/4p1B1/8/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 14' => {
			eco => 'B97u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 dxe5

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5'),
			moves => {
				'g5f6' => 'r1b1kb1r/6pp/p1p1pB2/4p3/8/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 14',
			},
		},
		'r1b1kb1r/6pp/p1ppp3/3nP1B1/8/q1N5/P1PQ2PP/1R2KB1R w Kkq - 1 14' => {
			eco => 'B97t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 Nd5

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 Nd5'),
			moves => {
			},
		},
		'r1b1kb1r/6pp/p1pppn2/4P1B1/8/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 13' => {
			eco => 'B97t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5'),
			moves => {
				'd6e5' => 'r1b1kb1r/6pp/p1p1pn2/4p1B1/8/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 14',
				'f6d5' => 'r1b1kb1r/6pp/p1ppp3/3nP1B1/8/q1N5/P1PQ2PP/1R2KB1R w Kkq - 1 14',
			},
		},
		'r1b1kb1r/6pp/p1pppn2/6B1/4P3/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 13' => {
			eco => 'B97r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line'),
			moves => {
				'e4e5' => 'r1b1kb1r/6pp/p1pppn2/4P1B1/8/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 13',
				'f1e2' => 'r1b1kb1r/6pp/p1pppn2/6B1/4P3/q1N5/P1PQB1PP/1R2K2R b Kkq - 1 13',
			},
		},
		'r1b1kb1r/6pp/p1pppn2/6B1/4P3/q1N5/P1PQB1PP/1R2K2R b Kkq - 1 13' => {
			eco => 'B97s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. Be2

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, Timman\'s 13. Be2'),
			moves => {
			},
		},
		'r1b1kb1r/7p/p1p1pp2/4p3/4N3/8/q1PQ2PP/1R2KB1R w Kkq - 0 16' => {
			eco => 'B97v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 dxe5 14. Bxf6 gxf6 15. Ne4 Qxa2

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4 Qxa2'),
			moves => {
			},
		},
		'r1b1kb1r/7p/p1p1pp2/4p3/4N3/q7/P1PQ2PP/1R2KB1R b Kkq - 1 15' => {
			eco => 'B97u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5 Nc6 11. fxe6 fxe6 12. Nxc6 bxc6 13. e5 dxe5 14. Bxf6 gxf6 15. Ne4

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5: 15. Ne4'),
			moves => {
				'a3a2' => 'r1b1kb1r/7p/p1p1pp2/4p3/4N3/8/q1PQ2PP/1R2KB1R w Kkq - 0 16',
				'f8e7' => 'r1b1k2r/4b2p/p1p1pp2/4p3/4N3/q7/P1PQ2PP/1R2KB1R w Kkq - 2 16',
			},
		},
		'r1b1kb1r/7p/p1p1pp2/4p3/8/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 15' => {
			eco => 'B97u',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, Main Line, 13. e5 dxe5'),
			moves => {
				'c3e4' => 'r1b1kb1r/7p/p1p1pp2/4p3/4N3/q7/P1PQ2PP/1R2KB1R b Kkq - 1 15',
			},
		},
		'r1b1kb1r/p1p2ppp/8/1p1np3/3n3q/2P3N1/PP1P1PPP/RNBQKB1R b KQkq - 3 9' => {
			eco => 'C57k',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz, Gruber Variation (8. Ne4)'),
			moves => {
				'c8b7' => 'r3kb1r/pbp2ppp/8/1p1np3/3n3q/2P3N1/PP1P1PPP/RNBQKB1R w KQkq - 4 10',
				'c8g4' => 'r3kb1r/p1p2ppp/8/1p1np3/3n2bq/2P3N1/PP1P1PPP/RNBQKB1R w KQkq - 4 10',
			},
		},
		'r1b1kb1r/p1p2ppp/8/1p1np3/3nN2q/2P5/PP1P1PPP/RNBQKB1R w KQkq - 2 9' => {
			eco => 'C57k',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz, Gruber Variation (8. Ne4)'),
			moves => {
				'e4g3' => 'r1b1kb1r/p1p2ppp/8/1p1np3/3n3q/2P3N1/PP1P1PPP/RNBQKB1R b KQkq - 3 9',
			},
		},
		'r1b1kb1r/p1ppqppp/2p2n2/4P3/8/8/PPP1QPPP/RNB1KB1R b KQkq - 2 7' => {
			eco => 'C45j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. e5 Qe7 7. Qe2

			variation => N__('Scotch: Mieses, 6... Qe7 7. Qe2'),
			moves => {
				'f6d5' => 'r1b1kb1r/p1ppqppp/2p5/3nP3/8/8/PPP1QPPP/RNB1KB1R w KQkq - 3 8',
			},
		},
		'r1b1kb1r/p1ppqppp/2p2n2/4P3/8/8/PPP2PPP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'C45j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. e5 Qe7

			variation => N__('Scotch: Mieses, 6... Qe7'),
			moves => {
				'd1e2' => 'r1b1kb1r/p1ppqppp/2p2n2/4P3/8/8/PPP1QPPP/RNB1KB1R b KQkq - 2 7',
			},
		},
		'r1b1kb1r/p1ppqppp/2p5/3nP3/2P5/8/PP2QPPP/RNB1KB1R b KQkq c3 0 8' => {
			eco => 'C45k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. e5 Qe7 7. Qe2 Nd5 8. c4

			variation => N__('Scotch: Mieses, 8. c4'),
			moves => {
				'c8a6' => 'r3kb1r/p1ppqppp/b1p5/3nP3/2P5/8/PP2QPPP/RNB1KB1R w KQkq - 1 9',
			},
		},
		'r1b1kb1r/p1ppqppp/2p5/3nP3/8/8/PPP1QPPP/RNB1KB1R w KQkq - 3 8' => {
			eco => 'C45j',
                        # TRANSLATORS:

			variation => N__('Scotch: Mieses, 6... Qe7 7. Qe2'),
			moves => {
				'c2c4' => 'r1b1kb1r/p1ppqppp/2p5/3nP3/2P5/8/PP2QPPP/RNB1KB1R b KQkq c3 0 8',
			},
		},
		'r1b1kb1r/p1q2pp1/2p2n1p/n3N3/4p3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11' => {
			eco => 'C59e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Qc7

			variation => N__('Two Knights: Morphy, Göring Variation'),
			moves => {
			},
		},
		'r1b1kb1r/p1q2ppp/2p2n2/n3p1N1/8/3B1Q2/PPPP1PPP/RNB1K2R b KQkq - 3 9' => {
			eco => 'C58f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Qf3 Qc7 9. Bd3

			variation => N__('Two Knights: Morphy, Paoli Variation'),
			moves => {
			},
		},
		'r1b1kb1r/p1q2ppp/2p2n2/nB2p1N1/8/5Q2/PPPP1PPP/RNB1K2R w KQkq - 2 9' => {
			eco => 'C58f',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Bogoljubow Variation'),
			moves => {
				'b5d3' => 'r1b1kb1r/p1q2ppp/2p2n2/n3p1N1/8/3B1Q2/PPPP1PPP/RNB1K2R b KQkq - 3 9',
			},
		},
		'r1b1kb1r/p1qp1ppp/2p1p3/3nP3/4N3/8/PPP2PPP/R1BQKB1R w KQkq - 3 9' => {
			eco => 'B45n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Nxc6 bxc6 7. e5 Nd5 8. Ne4 Qc7

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Nxc6 bxc6 7. e5 Nd5 8. Ne4 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/p2n1p2/2p1pn2/qp2P1B1/2pP4/2N3P1/PP3P1P/R2QKB1R w KQkq - 1 12' => {
			eco => 'D44l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. g3 Qa5

			variation => N__('Semi-Slav: Botvinnik, Lilienthal, 11... Qa5'),
			moves => {
			},
		},
		'r1b1kb1r/p3pp1p/1qpp1np1/8/2B1P3/2N5/PPP2PPP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'B57h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nxc6 bxc6 8. O-O g6

			variation => N__('Sicilian: Sozin, Benko, 7.Nxc6 bxc6 8.O-O g6'),
			moves => {
			},
		},
		'r1b1kb1r/p3pppp/1qpp1n2/8/2B1P3/2N5/PPP2PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'B57g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin, Benko, 7.Nxc6'),
			moves => {
				'g7g6' => 'r1b1kb1r/p3pp1p/1qpp1np1/8/2B1P3/2N5/PPP2PPP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r1b1kb1r/p3pppp/1qpp1n2/8/2B1P3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B57g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin, Benko, 7.Nxc6'),
			moves => {
				'e1g1' => 'r1b1kb1r/p3pppp/1qpp1n2/8/2B1P3/2N5/PPP2PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1b1kb1r/p4pp1/2p2n1p/n3N3/3qp3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11' => {
			eco => 'C59f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Qd4

			variation => N__('Two Knights: Morphy, Geller Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1p2/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/R2Q1RK1 b kq - 1 10' => {
			eco => 'D43v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3 dxc4 9. Bxc4 g6 10. O-O

			variation => N__('Semi-Slav: Moscow, Main Line'),
			moves => {
				'f8g7' => 'r1b1k2r/pp1n1pb1/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/R2Q1RK1 w kq - 2 11',
			},
		},
		'r1b1kb1r/pp1n1p2/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/R2QK2R w KQkq - 0 10' => {
			eco => 'D43v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3 dxc4 9. Bxc4 g6

			variation => N__('Semi-Slav: Moscow, Main Line'),
			moves => {
				'e1g1' => 'r1b1kb1r/pp1n1p2/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/R2Q1RK1 b kq - 1 10',
			},
		},
		'r1b1kb1r/pp1n1pp1/2p1pq1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8' => {
			eco => 'D43t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7

			variation => N__('Semi-Slav: Moscow, 7. e3 Nd7'),
			moves => {
				'f1d3' => 'r1b1kb1r/pp1n1pp1/2p1pq1p/3p4/2PP4/2NBPN2/PP3PPP/R2QK2R b KQkq - 2 8',
			},
		},
		'r1b1kb1r/pp1n1pp1/2p1pq1p/3p4/2PP4/2NBPN2/PP3PPP/R2QK2R b KQkq - 2 8' => {
			eco => 'D43u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3

			variation => N__('Semi-Slav: Moscow, 7. e3 Nd7 8. Bd3'),
			moves => {
				'd5c4' => 'r1b1kb1r/pp1n1pp1/2p1pq1p/8/2pP4/2NBPN2/PP3PPP/R2QK2R w KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1pp1/2p1pq1p/8/2BP4/2N1PN2/PP3PPP/R2QK2R b KQkq - 0 9' => {
			eco => 'D43v',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Moscow, 7. e3 Nd7 8. Bd3 dxc4'),
			moves => {
				'g7g6' => 'r1b1kb1r/pp1n1p2/2p1pqpp/8/2BP4/2N1PN2/PP3PPP/R2QK2R w KQkq - 0 10',
			},
		},
		'r1b1kb1r/pp1n1pp1/2p1pq1p/8/2pP4/2NBPN2/PP3PPP/R2QK2R w KQkq - 0 9' => {
			eco => 'D43v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 Nd7 8. Bd3 dxc4

			variation => N__('Semi-Slav: Moscow, 7. e3 Nd7 8. Bd3 dxc4'),
			moves => {
				'd3c4' => 'r1b1kb1r/pp1n1pp1/2p1pq1p/8/2BP4/2N1PN2/PP3PPP/R2QK2R b KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P1P/2P2N2/PP4P1/R1BQKBNR b KQkq h3 0 8' => {
			eco => 'C05j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6 8. h4

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. h4'),
			moves => {
				'c5d4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3p1P1P/2P2N2/PP4P1/R1BQKBNR w KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P2/2P2N2/PP2N1PP/R1BQKB1R b KQkq - 4 8' => {
			eco => 'C05k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6 8. Ne2

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. Ne2'),
			moves => {
				'c5d4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3p1P2/2P2N2/PP2N1PP/R1BQKB1R w KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P2/2P2N2/PP4PP/R1BQKBNR w KQkq - 3 8' => {
			eco => 'C05i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6'),
			moves => {
				'g1e2' => 'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P2/2P2N2/PP2N1PP/R1BQKB1R b KQkq - 4 8',
				'g2g3' => 'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P2/2P2NP1/PP5P/R1BQKBNR b KQkq - 0 8',
				'h2h4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P1P/2P2N2/PP4P1/R1BQKBNR b KQkq h3 0 8',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P2/2P2NP1/PP5P/R1BQKBNR b KQkq - 0 8' => {
			eco => 'C05l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6 8. g3

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. g3'),
			moves => {
				'c5d4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3p1P2/2P2NP1/PP5P/R1BQKBNR w KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQ1RK1 b kq - 4 8' => {
			eco => 'C05u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ngf3 Qb6 8. O-O

			variation => N__('French: Tarrasch, Closed, Nunn-Korchnoi Gambit'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQK2R w KQkq - 3 8' => {
			eco => 'C05r',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 7.Ngf3'),
			moves => {
				'e1g1' => 'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3N1P2/2N1B3/PPP3PP/R2QKB1R w KQkq - 1 9' => {
			eco => 'C11m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3 Nc6 7. Be3 cxd4 8. Nxd4 Qb6

			variation => N__('French: Steinitz, Boleslavsky, 7...cxd4 8.Nxd4 Qb6'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P1P1P/5N2/PP4P1/R1BQKBNR b KQkq - 0 9' => {
			eco => 'C05j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6 8. h4 cxd4 9. cxd4

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. h4'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P1P2/5N2/PP2N1PP/R1BQKB1R b KQkq - 0 9' => {
			eco => 'C05k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6 8. Ne2 cxd4 9. cxd4

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. Ne2'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P1P2/5NP1/PP5P/R1BQKBNR b KQkq - 0 9' => {
			eco => 'C05l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 Qb6 8. g3 cxd4 9. cxd4

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. g3'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P4/3B1N2/PP2NPPP/R1BQK2R b KQkq - 2 9' => {
			eco => 'C06i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Qb6 9. Nf3

			variation => N__('French: Tarrasch, Closed, 8... Qb6 9. Nf3'),
			moves => {
				'f7f6' => 'r1b1kb1r/pp1n2pp/1qn1pp2/3pP3/3P4/3B1N2/PP2NPPP/R1BQK2R w KQkq - 0 10',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'C06h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Qb6 9. O-O

			variation => N__('French: Tarrasch, Closed, 8... Qb6 9.O-O'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'C06g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Qb6

			variation => N__('French: Tarrasch, Closed, 8... Qb6'),
			moves => {
				'd2f3' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P4/3B1N2/PP2NPPP/R1BQK2R b KQkq - 2 9',
				'e1g1' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 b kq - 2 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3p1P1P/2P2N2/PP4P1/R1BQKBNR w KQkq - 0 9' => {
			eco => 'C05j',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. h4'),
			moves => {
				'c3d4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P1P1P/5N2/PP4P1/R1BQKBNR b KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3p1P2/2P2N2/PP2N1PP/R1BQKB1R w KQkq - 0 9' => {
			eco => 'C05k',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. Ne2'),
			moves => {
				'c3d4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P1P2/5N2/PP2N1PP/R1BQKB1R b KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3p1P2/2P2NP1/PP5P/R1BQKBNR w KQkq - 0 9' => {
			eco => 'C05l',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 Qb6 8. g3'),
			moves => {
				'c3d4' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P1P2/5NP1/PP5P/R1BQKBNR b KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp1n1ppp/2n1p3/q1ppP3/3P4/2PB1N2/PP3PPP/R1BQK1NR w KQkq - 3 8' => {
			eco => 'C05q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ndf3 Qa5

			variation => N__('French: Tarrasch, Closed, 7.Ndf3 Qa5'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1p3/q2n2B1/3P4/2N1PN2/PP1Q1PPP/R3KB1R b KQkq - 1 8' => {
			eco => 'D52f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. cxd5 Nxd5 8. Qd2

			variation => N__('QGD: Cambridge Springs, Yugoslav, 8. Qd2'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1p3/q2n2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D52e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. cxd5 Nxd5

			variation => N__('QGD: Cambridge Springs, Yugoslav Variation'),
			moves => {
				'd1d2' => 'r1b1kb1r/pp1n1ppp/2p1p3/q2n2B1/3P4/2N1PN2/PP1Q1PPP/R3KB1R b KQkq - 1 8',
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pB2/q2p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D52c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Bxf6

			variation => N__('QGD: Cambridge Springs, Capablanca Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pn2/q2P2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D52d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. cxd5

			variation => N__('QGD: Cambridge Springs, 7.cxd5'),
			moves => {
				'f6d5' => 'r1b1kb1r/pp1n1ppp/2p1p3/q2n2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pn2/q2p2B1/2PP4/2N1P3/PP1N1PPP/R2QKB1R b KQkq - 3 7' => {
			eco => 'D52g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2

			variation => N__('QGD: Cambridge Springs, 7. Nd2'),
			moves => {
				'd5c4' => 'r1b1kb1r/pp1n1ppp/2p1pn2/q5B1/2pP4/2N1P3/PP1N1PPP/R2QKB1R w KQkq - 0 8',
				'f8b4' => 'r1b1k2r/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PP1N1PPP/R2QKB1R w KQkq - 4 8',
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pn2/q2p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'D52b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5

			variation => N__('QGD: Cambridge Springs Defence'),
			moves => {
				'c4d5' => 'r1b1kb1r/pp1n1ppp/2p1pn2/q2P2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
				'f3d2' => 'r1b1kb1r/pp1n1ppp/2p1pn2/q2p2B1/2PP4/2N1P3/PP1N1PPP/R2QKB1R b KQkq - 3 7',
				'g5f6' => 'r1b1kb1r/pp1n1ppp/2p1pB2/q2p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pn2/q2p2B1/2PP4/2N2N2/PP2PPPP/2RQKB1R w Kkq - 2 7' => {
			eco => 'D51',
                        # TRANSLATORS:

			variation => N__('QGD: 4. Bg5 Nbd7 5. Nf3 c6'),
			moves => {
				'g5d2' => 'r1b1kb1r/pp1n1ppp/2p1pn2/q2p4/2PP4/2N2N2/PP1BPPPP/2RQKB1R b Kkq - 3 7',
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pn2/q2p4/2PP4/2N2N2/PP1BPPPP/2RQKB1R b Kkq - 3 7' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. Nf3 c6 6. Rc1 Qa5 7. Bd2

			variation => N__('QGD: Rochlin Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n1ppp/2p1pn2/q5B1/2pP4/2N1P3/PP1N1PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D52h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 dxc4

			variation => N__('QGD: Cambridge Springs, Rubinstein Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pp1n2pp/1qn1pp2/3pP3/3P4/3B1N2/PP2NPPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'C06j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Qb6 9. Nf3 f6

			variation => N__('French: Tarrasch, Closed, 8... Qb6 9. Nf3 f6'),
			moves => {
			},
		},
		'r1b1kb1r/pp1p1ppp/1q2pn2/4n3/2P5/1NN3P1/PP2PP1P/R1BQKB1R w KQkq - 3 8' => {
			eco => 'A33q',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3 Qb6 7. Nb3 Ne5

			variation => N__('English: Symmetrical, Geller, 7. Nb3 Ne5'),
			moves => {
			},
		},
		'r1b1kb1r/pp1p1ppp/1qn1pn2/8/2P5/1NN3P1/PP2PP1P/R1BQKB1R b KQkq - 2 7' => {
			eco => 'A33p',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3 Qb6 7. Nb3

			variation => N__('English: Symmetrical, Geller, 7. Nb3'),
			moves => {
				'c6e5' => 'r1b1kb1r/pp1p1ppp/1q2pn2/4n3/2P5/1NN3P1/PP2PP1P/R1BQKB1R w KQkq - 3 8',
			},
		},
		'r1b1kb1r/pp1p1ppp/1qn1pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'A33o',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3 Qb6

			variation => N__('English: Symmetrical, Geller Variation'),
			moves => {
				'd4b3' => 'r1b1kb1r/pp1p1ppp/1qn1pn2/8/2P5/1NN3P1/PP2PP1P/R1BQKB1R b KQkq - 2 7',
			},
		},
		'r1b1kb1r/pp1p1ppp/1qn1pn2/8/4P3/1NN5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B33b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Qb6 6. Nb3 e6

			variation => N__('Sicilian: Open, 2... Nc6, 5... Qb6 6. Nb3 e6'),
			moves => {
				'f1d3' => 'r1b1kb1r/pp1p1ppp/1qn1pn2/8/4P3/1NNB4/PPP2PPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1b1kb1r/pp1p1ppp/1qn1pn2/8/4P3/1NNB4/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B33c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Qb6 6. Nb3 e6 7. Bd3

			variation => N__('Sicilian: Open, 2... Nc6, 5... Qb6 6. Nb3 e6 7. Bd3'),
			moves => {
			},
		},
		'r1b1kb1r/pp1ppppp/1qn2n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B33b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Qb6

			variation => N__('Sicilian: Open, 2... Nc6, 5... Qb6'),
			moves => {
				'd4b3' => 'r1b1kb1r/pp1ppppp/1qn2n2/8/4P3/1NN5/PPP2PPP/R1BQKB1R b KQkq - 4 6',
			},
		},
		'r1b1kb1r/pp1ppppp/1qn2n2/8/4P3/1NN5/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B33b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Qb6 6. Nb3

			variation => N__('Sicilian: Open, 2... Nc6, 5... Qb6 6. Nb3'),
			moves => {
				'e7e6' => 'r1b1kb1r/pp1p1ppp/1qn1pn2/8/4P3/1NN5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1b1kb1r/pp2pppp/1qNp1n2/8/2B1P3/2N5/PPP2PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'B57g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nxc6

			variation => N__('Sicilian: Sozin, Benko, 7.Nxc6'),
			moves => {
				'b7c6' => 'r1b1kb1r/p3pppp/1qpp1n2/8/2B1P3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1b1kb1r/pp2pppp/1qn2n2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 5 7' => {
			eco => 'B13v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 Qb6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Spielmann Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pp2pppp/1qnp1n2/1N6/2B1P3/2N5/PPP2PPP/R1BQK2R b KQkq - 6 7' => {
			eco => 'B57f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Ndb5

			variation => N__('Sicilian: Sozin, Benko, 7.Ndb5'),
			moves => {
			},
		},
		'r1b1kb1r/pp2pppp/1qnp1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 5 7' => {
			eco => 'B60d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Qb6

			variation => N__('Sicilian: Richter-Rauzer, 6... Qb6'),
			moves => {
			},
		},
		'r1b1kb1r/pp2pppp/1qnp1n2/8/2B1P3/1NN5/PPP2PPP/R1BQK2R b KQkq - 6 7' => {
			eco => 'B57i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3'),
			moves => {
				'e7e6' => 'r1b1kb1r/pp3ppp/1qnppn2/8/2B1P3/1NN5/PPP2PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1b1kb1r/pp2pppp/1qnp1n2/8/2B1P3/2N5/PPP1NPPP/R1BQK2R b KQkq - 6 7' => {
			eco => 'B57e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nde2

			variation => N__('Sicilian: Sozin, Benko, 7.Nde2'),
			moves => {
			},
		},
		'r1b1kb1r/pp2pppp/1qnp1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 5 7' => {
			eco => 'B57d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6

			variation => N__('Sicilian: Sozin, Benko Variation'),
			moves => {
				'd4b3' => 'r1b1kb1r/pp2pppp/1qnp1n2/8/2B1P3/1NN5/PPP2PPP/R1BQK2R b KQkq - 6 7',
				'd4b5' => 'r1b1kb1r/pp2pppp/1qnp1n2/1N6/2B1P3/2N5/PPP2PPP/R1BQK2R b KQkq - 6 7',
				'd4c6' => 'r1b1kb1r/pp2pppp/1qNp1n2/8/2B1P3/2N5/PPP2PPP/R1BQK2R b KQkq - 0 7',
				'd4e2' => 'r1b1kb1r/pp2pppp/1qnp1n2/8/2B1P3/2N5/PPP1NPPP/R1BQK2R b KQkq - 6 7',
			},
		},
		'r1b1kb1r/pp2pppp/1qnp1n2/8/2PNP3/5P2/PP4PP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'B54k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 Nc6 6. c4 Qb6

			variation => N__('Sicilian: Prins (Moscow), 5... Nc6 6. c4 Qb6'),
			moves => {
			},
		},
		'r1b1kb1r/pp2pppp/2n2n2/q2p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 5 7' => {
			eco => 'B13w',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 Qa5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Czerniak Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pp2pppp/2np1n2/q5B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 5 7' => {
			eco => 'B60c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Qa5

			variation => N__('Sicilian: Richter-Rauzer, 6... Qa5'),
			moves => {
			},
		},
		'r1b1kb1r/pp3ppp/1qn1p2n/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'C02x',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. a3 Nh6

			variation => N__('French: Advance, 5. Nf3 Qb6 6. a3 Nh6'),
			moves => {
			},
		},
		'r1b1kb1r/pp3ppp/1qnppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B63c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Qb6

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Qb6'),
			moves => {
				'd4b3' => 'r1b1kb1r/pp3ppp/1qnppn2/6B1/4P3/1NN5/PPPQ1PPP/R3KB1R b KQkq - 3 8',
			},
		},
		'r1b1kb1r/pp3ppp/1qnppn2/6B1/4P3/1NN5/PPPQ1PPP/R3KB1R b KQkq - 3 8' => {
			eco => 'B63d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Qb6 8. Nb3

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Qb6 8. Nb3'),
			moves => {
				'a7a6' => 'r1b1kb1r/1p3ppp/pqnppn2/6B1/4P3/1NN5/PPPQ1PPP/R3KB1R w KQkq - 0 9',
			},
		},
		'r1b1kb1r/pp3ppp/1qnppn2/8/2B1P3/1NN1B3/PPP2PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'B57k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3 e6 8. Be3

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3 e6 8. Be3'),
			moves => {
			},
		},
		'r1b1kb1r/pp3ppp/1qnppn2/8/2B1P3/1NN5/PPP2PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'B57m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3 e6 8. O-O

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3 e6 8.O-O'),
			moves => {
				'f8e7' => 'r1b1k2r/pp2bppp/1qnppn2/8/2B1P3/1NN5/PPP2PPP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'r1b1kb1r/pp3ppp/1qnppn2/8/2B1P3/1NN5/PPP2PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B57j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3 e6

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3 e6'),
			moves => {
				'c1e3' => 'r1b1kb1r/pp3ppp/1qnppn2/8/2B1P3/1NN1B3/PPP2PPP/R2QK2R b KQkq - 1 8',
				'c1f4' => 'r1b1kb1r/pp3ppp/1qnppn2/8/2B1PB2/1NN5/PPP2PPP/R2QK2R b KQkq - 1 8',
				'e1g1' => 'r1b1kb1r/pp3ppp/1qnppn2/8/2B1P3/1NN5/PPP2PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1b1kb1r/pp3ppp/1qnppn2/8/2B1PB2/1NN5/PPP2PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'B57l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Qb6 7. Nb3 e6 8. Bf4

			variation => N__('Sicilian: Sozin, Benko, 7. Nb3 e6 8. Bf4'),
			moves => {
			},
		},
		'r1b1kb1r/pp3ppp/2nqpn2/8/2BN4/5N2/PPP2PPP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C07o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'c6d4' => 'r1b1kb1r/pp3ppp/3qpn2/8/2Bn4/5N2/PPP2PPP/R1BQ1RK1 w kq - 0 10',
			},
		},
		'r1b1kb1r/pp3ppp/2nqpn2/8/2Bp4/1N3N2/PPP2PPP/R1BQ1RK1 w kq - 6 9' => {
			eco => 'C07o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'b3d4' => 'r1b1kb1r/pp3ppp/2nqpn2/8/2BN4/5N2/PPP2PPP/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r1b1kb1r/pp3ppp/3qpn2/8/2BN4/8/PPP2PPP/R1BQ1RK1 b kq - 0 10' => {
			eco => 'C07q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6 7. O-O Nf6 8. Nb3 Nc6 9. Nbxd4 Nxd4 10. Nxd4

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5, Main Line, 10.Nxd4'),
			moves => {
				'a7a6' => 'r1b1kb1r/1p3ppp/p2qpn2/8/2BN4/8/PPP2PPP/R1BQ1RK1 w kq - 0 11',
			},
		},
		'r1b1kb1r/pp3ppp/3qpn2/8/2BQ4/5N2/PPP2PPP/R1B2RK1 b kq - 0 10' => {
			eco => 'C07p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6 7. O-O Nf6 8. Nb3 Nc6 9. Nbxd4 Nxd4 10. Qxd4

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5, Main Line, 10.Qxd4'),
			moves => {
				'd6d4' => 'r1b1kb1r/pp3ppp/4pn2/8/2Bq4/5N2/PPP2PPP/R1B2RK1 w kq - 0 11',
			},
		},
		'r1b1kb1r/pp3ppp/3qpn2/8/2Bn4/5N2/PPP2PPP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'C07o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'd1d4' => 'r1b1kb1r/pp3ppp/3qpn2/8/2BQ4/5N2/PPP2PPP/R1B2RK1 b kq - 0 10',
				'f3d4' => 'r1b1kb1r/pp3ppp/3qpn2/8/2BN4/8/PPP2PPP/R1BQ1RK1 b kq - 0 10',
			},
		},
		'r1b1kb1r/pp3ppp/4pn2/8/2Bq4/5N2/PPP2PPP/R1B2RK1 w kq - 0 11' => {
			eco => 'C07p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6 7. O-O Nf6 8. Nb3 Nc6 9. Nbxd4 Nxd4 10. Qxd4 Qxd4

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5, Main Line, 10.Qxd4 Qxd4'),
			moves => {
			},
		},
		'r1b1kb1r/ppNpq2p/2nn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR b KQkq - 0 9' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'e8d8' => 'r1bk1b1r/ppNpq2p/2nn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR w KQ - 1 10',
			},
		},
		'r1b1kb1r/ppp2ppp/2n5/3q4/3pn3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 1 8' => {
			eco => 'C56d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3

			variation => N__('Two Knights: Classical, 8. Nc3'),
			moves => {
				'd5a5' => 'r1b1kb1r/ppp2ppp/2n5/q7/3pn3/2N2N2/PPP2PPP/R1BQR1K1 w kq - 2 9',
				'd5d8' => 'r1bqkb1r/ppp2ppp/2n5/8/3pn3/2N2N2/PPP2PPP/R1BQR1K1 w kq - 2 9',
				'd5h5' => 'r1b1kb1r/ppp2ppp/2n5/7q/3pn3/2N2N2/PPP2PPP/R1BQR1K1 w kq - 2 9',
			},
		},
		'r1b1kb1r/ppp2ppp/2n5/3q4/3pn3/5N2/PPP2PPP/RNBQR1K1 w kq - 0 8' => {
			eco => 'C56d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5

			variation => N__('Two Knights: Classical, 7.Bxd5 Qxd5'),
			moves => {
				'b1c3' => 'r1b1kb1r/ppp2ppp/2n5/3q4/3pn3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 1 8',
			},
		},
		'r1b1kb1r/ppp2ppp/2n5/7q/3pN3/5N2/PPP2PPP/R1BQR1K1 b kq - 0 9' => {
			eco => 'C56f',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, 8. Nc3 Qh5'),
			moves => {
				'c8e6' => 'r3kb1r/ppp2ppp/2n1b3/7q/3pN3/5N2/PPP2PPP/R1BQR1K1 w kq - 1 10',
			},
		},
		'r1b1kb1r/ppp2ppp/2n5/7q/3pn3/2N2N2/PPP2PPP/R1BQR1K1 w kq - 2 9' => {
			eco => 'C56f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qh5

			variation => N__('Two Knights: Classical, 8. Nc3 Qh5'),
			moves => {
				'c3e4' => 'r1b1kb1r/ppp2ppp/2n5/7q/3pN3/5N2/PPP2PPP/R1BQR1K1 b kq - 0 9',
			},
		},
		'r1b1kb1r/ppp2ppp/2n5/q7/3pN3/5N2/PPP2PPP/R1BQR1K1 b kq - 0 9' => {
			eco => 'C56h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4

			variation => N__('Two Knights: Classical, Moller, 9.Nxe4'),
			moves => {
				'c8e6' => 'r3kb1r/ppp2ppp/2n1b3/q7/3pN3/5N2/PPP2PPP/R1BQR1K1 w kq - 1 10',
			},
		},
		'r1b1kb1r/ppp2ppp/2n5/q7/3pn3/2N2N2/PPP2PPP/R1BQR1K1 w kq - 2 9' => {
			eco => 'C56h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5

			variation => N__('Two Knights: Classical, Moller Variation'),
			moves => {
				'c3e4' => 'r1b1kb1r/ppp2ppp/2n5/q7/3pN3/5N2/PPP2PPP/R1BQR1K1 b kq - 0 9',
			},
		},
		'r1b1kb1r/ppp2ppp/2p5/4Pn2/8/2N2N1P/PPP2PP1/R1B2RK1 b - - 0 10' => {
			eco => 'C67w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+ Kxd8 9. Nc3 Ke8 10. h3

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap, 9. Nc3 Ke8 10. h3'),
			moves => {
				'a7a5' => 'r1b1kb1r/1pp2ppp/2p5/p3Pn2/8/2N2N1P/PPP2PP1/R1B2RK1 w - a6 0 11',
			},
		},
		'r1b1kb1r/ppp2ppp/2p5/4Pn2/8/2N2N2/PPP2PPP/R1B2RK1 w - - 2 10' => {
			eco => 'C67u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+ Kxd8 9. Nc3 Ke8

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap, 9. Nc3 Ke8'),
			moves => {
				'f1d1' => 'r1b1kb1r/ppp2ppp/2p5/4Pn2/8/2N2N2/PPP2PPP/R1BR2K1 b - - 3 10',
				'h2h3' => 'r1b1kb1r/ppp2ppp/2p5/4Pn2/8/2N2N1P/PPP2PP1/R1B2RK1 b - - 0 10',
			},
		},
		'r1b1kb1r/ppp2ppp/2p5/4Pn2/8/2N2N2/PPP2PPP/R1BR2K1 b - - 3 10' => {
			eco => 'C67v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+ Kxd8 9. Nc3 Ke8 10. Rd1

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap, 9. Nc3 Ke8 10. Rd1'),
			moves => {
			},
		},
		'r1b1kb1r/pppn1pp1/3ppq1p/8/3PP3/2N2N2/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'A46z',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 d6 6. Nc3 Nd7

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 d6 6. Nc3 Nd7'),
			moves => {
				'd1d2' => 'r1b1kb1r/pppn1pp1/3ppq1p/8/3PP3/2N2N2/PPPQ1PPP/R3KB1R b KQkq - 3 7',
			},
		},
		'r1b1kb1r/pppn1pp1/3ppq1p/8/3PP3/2N2N2/PPPQ1PPP/R3KB1R b KQkq - 3 7' => {
			eco => 'A46z',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 d6 6. Nc3 Nd7 7. Qd2

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 d6 6. Nc3 Nd7 7. Qd2'),
			moves => {
			},
		},
		'r1b1kb1r/pppnqppp/3p1n2/6B1/8/3P1N2/PPP1QPPP/RN2KB1R w KQkq - 3 8' => {
			eco => 'C42n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. Qe2 Qe7 6. d3 Nf6 7. Bg5 Nbd7

			variation => N__('Russian Game: Milner-Barry Variation'),
			moves => {
			},
		},
		'r1b1kb1r/pppp1ppp/5q2/1B6/8/2P5/PPP2PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'C48k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'f6e5' => 'r1b1kb1r/pppp1ppp/8/1B2q3/8/2P5/PPP2PPP/R1BQK2R w KQkq - 1 9',
				'f8c5' => 'r1b1k2r/pppp1ppp/5q2/1Bb5/8/2P5/PPP2PPP/R1BQK2R w KQkq - 1 9',
			},
		},
		'r1b1kb1r/pppp1ppp/5q2/1B6/8/2p5/PPPP1PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'C48k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'd2c3' => 'r1b1kb1r/pppp1ppp/5q2/1B6/8/2P5/PPP2PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1b1kb1r/pppp1ppp/8/1B2q3/8/2P5/PPP2PPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'C48m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Nxd4 exd4 6. e5 dxc3 7. exf6 Qxf6 8. dxc3 Qe5+

			variation => N__('Four Knights: Rubinstein, Exchange Variation, 8... Qe5+'),
			moves => {
			},
		},
		'r1b1kb1r/ppppq2p/2nn2p1/1N1Qpp2/8/1B6/PPPP1PPP/R1B1K1NR w KQkq - 2 9' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'b5c7' => 'r1b1kb1r/ppNpq2p/2nn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR b KQkq - 0 9',
			},
		},
		'r1b1kb1r/ppppqppp/5n2/1B2N3/3nP3/2N5/PPPP1PPP/R1BQK2R w KQkq - 1 6' => {
			eco => 'C48h',
                        # TRANSLATORS:

			variation => N__('Four Knights: Rubinstein, 5.Nxe5'),
			moves => {
				'f2f4' => 'r1b1kb1r/ppppqppp/5n2/1B2N3/3nPP2/2N5/PPPP2PP/R1BQK2R b KQkq f3 0 6',
			},
		},
		'r1b1kb1r/ppppqppp/5n2/1B2N3/3nPP2/2N5/PPPP2PP/R1BQK2R b KQkq f3 0 6' => {
			eco => 'C48h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Nxe5 Qe7 6. f4

			variation => N__('Four Knights: Rubinstein, Bogoljubow Variation'),
			moves => {
			},
		},
		'r1b1kb1r/ppq2ppp/2nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 4 8' => {
			eco => 'B80s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'a7a6' => 'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r1b1kb1r/ppq2ppp/2nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 3 8' => {
			eco => 'B80s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6 7. Bg2 Qc7

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'e1g1' => 'r1b1kb1r/ppq2ppp/2nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r1b1kb1r/ppq2ppp/2nppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq - 3 8' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6 7. Be3 Qc7

			variation => N__('Sicilian: Scheveningen, 6. f4 Nc6 7. Be3 Qc7'),
			moves => {
			},
		},
		'r1b1kb1r/ppqp1ppp/2n1pn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 5 7' => {
			eco => 'B48b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 Nf6

			variation => N__('Sicilian: Taimanov, 6. Be3 Nf6'),
			moves => {
			},
		},
		'r1b1kb1r/ppqp1ppp/2n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 5 7' => {
			eco => 'B47q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 Nf6

			variation => N__('Sicilian: Taimanov, 6. Be2 Nf6'),
			moves => {
			},
		},
		'r1b1kbnr/1pp2ppp/p1p5/8/3NP3/8/PPP2PPP/RNB1K2R b KQkq - 0 7' => {
			eco => 'C68e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Lasker Variation'),
			moves => {
				'c8d7' => 'r3kbnr/1ppb1ppp/p1p5/8/3NP3/8/PPP2PPP/RNB1K2R w KQkq - 1 8',
			},
		},
		'r1b1kbnr/1pp2ppp/p1p5/8/3qP3/5N2/PPP2PPP/RNB1K2R w KQkq - 0 7' => {
			eco => 'C68e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Lasker Variation'),
			moves => {
				'f3d4' => 'r1b1kbnr/1pp2ppp/p1p5/8/3NP3/8/PPP2PPP/RNB1K2R b KQkq - 0 7',
			},
		},
		'r1b1kbnr/1pp2ppp/p1pq4/4p3/4P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C68r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Qd6 6. d3

			variation => N__('Spanish: Exchange, Bronstein, 6. d3'),
			moves => {
			},
		},
		'r1b1kbnr/1pp2ppp/p1pq4/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C68q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Qd6

			variation => N__('Spanish: Exchange, Bronstein Variation'),
			moves => {
				'b1a3' => 'r1b1kbnr/1pp2ppp/p1pq4/4p3/4P3/N4N2/PPPP1PPP/R1BQ1RK1 b kq - 3 6',
				'd2d3' => 'r1b1kbnr/1pp2ppp/p1pq4/4p3/4P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1b1kbnr/1pp2ppp/p1pq4/4p3/4P3/N4N2/PPPP1PPP/R1BQ1RK1 b kq - 3 6' => {
			eco => 'C68s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Qd6 6. Na3

			variation => N__('Spanish: Exchange, Bronstein, 6. Na3'),
			moves => {
				'b7b5' => 'r1b1kbnr/2p2ppp/p1pq4/1p2p3/4P3/N4N2/PPPP1PPP/R1BQ1RK1 w kq b6 0 7',
				'c8e6' => 'r3kbnr/1pp2ppp/p1pqb3/4p3/4P3/N4N2/PPPP1PPP/R1BQ1RK1 w kq - 4 7',
			},
		},
		'r1b1kbnr/1pp3pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBR2K1 b kq - 0 9' => {
			eco => 'C69k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5 8. Nb3 Qxd1 9. Rxd1

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3 Queenswap'),
			moves => {
				'c8d7' => 'r3kbnr/1ppb2pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBR2K1 w kq - 1 10',
				'c8g4' => 'r3kbnr/1pp3pp/p4p2/2p5/4P1b1/1N6/PPP2PPP/RNBR2K1 w kq - 1 10',
			},
		},
		'r1b1kbnr/1pp3pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBq1RK1 w kq - 0 9' => {
			eco => 'C69k',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3'),
			moves => {
				'f1d1' => 'r1b1kbnr/1pp3pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBR2K1 b kq - 0 9',
			},
		},
		'r1b1kbnr/1pq2ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'B47i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 d6 8. O-O

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 d6 8.O-O'),
			moves => {
				'c8d7' => 'r3kbnr/1pqb1ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'r1b1kbnr/1pq2ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B47i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 d6

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 d6'),
			moves => {
				'e1g1' => 'r1b1kbnr/1pq2ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1N1p3/8/4PP2/2N5/PPP3PP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B47f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. f4 a6 7. Nxc6

			variation => N__('Sicilian: Taimanov, 6. f4 a6 7.Nxc6'),
			moves => {
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B49a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2'),
			moves => {
				'b7b5' => 'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq b6 0 8',
				'g8e7' => 'r1b1kb1r/1pqpnppp/p1n1p3/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq - 2 8',
				'g8f6' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq - 2 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B48c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6'),
			moves => {
				'a2a3' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/P1N1B3/1PP2PPP/R2QKB1R b KQkq - 0 7',
				'd1d2' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 1 7',
				'f1d3' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2NBB3/PPP2PPP/R2QK2R b KQkq - 1 7',
				'f1e2' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 1 7',
				'f2f4' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NPP2/2N1B3/PPP3PP/R2QKB1R b KQkq f3 0 7',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B48g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Qd2

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Qd2'),
			moves => {
				'g8f6' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R w KQkq - 2 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B47h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6

			variation => N__('Sicilian: Taimanov, 6. g3 a6'),
			moves => {
				'f1g2' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N3P1/PPP2PBP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N3P1/PPP2PBP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B47h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2'),
			moves => {
				'd7d6' => 'r1b1kbnr/1pq2ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 0 8',
				'g8f6' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 2 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'B47t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O'),
			moves => {
				'b7b5' => 'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq b6 0 8',
				'g8f6' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B47r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6

			variation => N__('Sicilian: Taimanov, 6. Be2 a6'),
			moves => {
				'e1g1' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 1 7',
				'f2f4' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NPP2/2N5/PPP1B1PP/R1BQK2R b KQkq f3 0 7',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2NBB3/PPP2PPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B48i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3'),
			moves => {
				'b7b5' => 'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2NBB3/PPP2PPP/R2QK2R w KQkq b6 0 8',
				'g8f6' => 'r1b1kb1r/1pqp1ppp/p1n1pn2/8/3NP3/2NBB3/PPP2PPP/R2QK2R w KQkq - 2 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/P1N1B3/1PP2PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B48d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. a3

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. a3'),
			moves => {
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NPP2/2N1B3/PPP3PP/R2QKB1R b KQkq f3 0 7' => {
			eco => 'B48e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. f4

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. f4'),
			moves => {
				'b7b5' => 'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq b6 0 8',
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NPP2/2N5/PPP1B1PP/R1BQK2R b KQkq f3 0 7' => {
			eco => 'B47s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. f4

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7. f4'),
			moves => {
			},
		},
		'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B47e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. f4 a6

			variation => N__('Sicilian: Taimanov, 6. f4 a6'),
			moves => {
				'd4c6' => 'r1b1kbnr/1pqp1ppp/p1N1p3/8/4PP2/2N5/PPP3PP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1b1kbnr/2p2ppp/p1pq4/1p2p3/4P3/N4N2/PPPP1PPP/R1BQ1RK1 w kq b6 0 7' => {
			eco => 'C68t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Qd6 6. Na3 b5

			variation => N__('Spanish: Exchange, Bronstein, 6. Na3 b5'),
			moves => {
			},
		},
		'r1b1kbnr/2qp1ppp/p1N1p3/1p6/4P3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 0 8' => {
			eco => 'B49b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 b5 8. Nxc6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 b5 8.Nxc6'),
			moves => {
			},
		},
		'r1b1kbnr/2qp1ppp/p1N1p3/1p6/4P3/2NBB3/PPP2PPP/R2QK2R b KQkq - 0 8' => {
			eco => 'B48k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 b5 8. Nxc6

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 b5 8.Nxc6'),
			moves => {
			},
		},
		'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq b6 0 8' => {
			eco => 'B49b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 b5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 b5'),
			moves => {
				'd4c6' => 'r1b1kbnr/2qp1ppp/p1N1p3/1p6/4P3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 0 8',
			},
		},
		'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq b6 0 8' => {
			eco => 'B47u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2 a6 7. O-O b5

			variation => N__('Sicilian: Taimanov, 6. Be2 a6 7.O-O b5'),
			moves => {
			},
		},
		'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2NBB3/PPP2PPP/R2Q1RK1 b kq - 1 8' => {
			eco => 'B48l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 b5 8. O-O

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 b5 8.O-O'),
			moves => {
			},
		},
		'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2NBB3/PPP2PPP/R2QK2R w KQkq b6 0 8' => {
			eco => 'B48j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Bd3 b5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Bd3 b5'),
			moves => {
				'd4c6' => 'r1b1kbnr/2qp1ppp/p1N1p3/1p6/4P3/2NBB3/PPP2PPP/R2QK2R b KQkq - 0 8',
				'e1g1' => 'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NP3/2NBB3/PPP2PPP/R2Q1RK1 b kq - 1 8',
			},
		},
		'r1b1kbnr/2qp1ppp/p1n1p3/1p6/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq b6 0 8' => {
			eco => 'B48f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. f4 b5

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. f4 b5'),
			moves => {
			},
		},
		'r1b1kbnr/p1pp1ppp/1pn5/8/3PPp1q/2N5/PPP1K1PP/R1BQ1BNR w kq - 0 6' => {
			eco => 'C25n',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. d4 Qh4+ 5. Ke2 b6

			variation => N__('Vienna: Steinitz Gambit, Fraser-Minckwitz Variation'),
			moves => {
			},
		},
		'r1b1kbnr/pp1ppppp/1qn5/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'B30l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 Qb6

			variation => N__('Sicilian: Rossolimo, 3... Qb6'),
			moves => {
			},
		},
		'r1b1kbnr/pp1ppppp/1qn5/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B32e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Qb6

			variation => N__('Sicilian: Open, 2... Nc6, 4... Qb6'),
			moves => {
			},
		},
		'r1b1kbnr/pp1qpppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R w KQkq - 3 6' => {
			eco => 'B53m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Qd7

			variation => N__('Sicilian: Chekhover, Zaitsev Variation'),
			moves => {
			},
		},
		'r1b1kbnr/pp2pppp/2n5/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'B22h',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nc6 5. Nf3

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nc6 5. Nf3'),
			moves => {
				'c5d4' => 'r1b1kbnr/pp2pppp/2n5/3q4/3p4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6',
				'c8g4' => 'r3kbnr/pp2pppp/2n5/2pq4/3P2b1/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
			},
		},
		'r1b1kbnr/pp2pppp/2n5/2pq4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B22h',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nc6

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nc6'),
			moves => {
				'g1f3' => 'r1b1kbnr/pp2pppp/2n5/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'r1b1kbnr/pp2pppp/2n5/3q4/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'B22h',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nc6 5. Nf3 cxd4 6. cxd4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nc6 5. Nf3 cxd4 6.cxd4'),
			moves => {
			},
		},
		'r1b1kbnr/pp2pppp/2n5/3q4/3p4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B22h',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nc6 5. Nf3 cxd4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nc6 5. Nf3 cxd4'),
			moves => {
				'c3d4' => 'r1b1kbnr/pp2pppp/2n5/3q4/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 0 6',
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6' => {
			eco => 'C02t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. Be2

			variation => N__('French: Advance, 5. Nf3 Qb6 6. Be2'),
			moves => {
				'c5d4' => 'r1b1kbnr/pp3ppp/1qn1p3/3pP3/3p4/2P2N2/PP2BPPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C02r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6

			variation => N__('French: Advance, 5. Nf3 Qb6'),
			moves => {
				'a2a3' => 'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R b KQkq - 0 6',
				'f1d3' => 'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/2PB1N2/PP3PPP/RNBQK2R b KQkq - 4 6',
				'f1e2' => 'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6',
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/2PB1N2/PP3PPP/RNBQK2R b KQkq - 4 6' => {
			eco => 'C02s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. Bd3

			variation => N__('French: Advance, Milner-Barry Gambit'),
			moves => {
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C02v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. a3

			variation => N__('French: Advance, 5. Nf3 Qb6 6. a3'),
			moves => {
				'c5c4' => 'r1b1kbnr/pp3ppp/1qn1p3/3pP3/2pP4/P1P2N2/1P3PPP/RNBQKB1R w KQkq - 0 7',
				'c8d7' => 'r3kbnr/pp1b1ppp/1qn1p3/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R w KQkq - 1 7',
				'g8h6' => 'r1b1kb1r/pp3ppp/1qn1p2n/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/3pP3/2pP4/P1P2N2/1P1N1PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'C02z',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. a3 c4 7. Nbd2

			variation => N__('French: Advance, 5. Nf3 Qb6 6. a3 c4 7.Nbd2'),
			moves => {
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/3pP3/2pP4/P1P2N2/1P3PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'C02y',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. a3 c4

			variation => N__('French: Advance, 5. Nf3 Qb6 6. a3 c4'),
			moves => {
				'b1d2' => 'r1b1kbnr/pp3ppp/1qn1p3/3pP3/2pP4/P1P2N2/1P1N1PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/3pP3/3P4/5N2/PP2BPPP/RNBQK2R b KQkq - 0 7' => {
			eco => 'C02u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. Be2 cxd4 7. cxd4

			variation => N__('French: Advance, 5. Nf3 Qb6 6. Be2 cxd4 7.cxd4'),
			moves => {
			},
		},
		'r1b1kbnr/pp3ppp/1qn1p3/3pP3/3p4/2P2N2/PP2BPPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C02t',
                        # TRANSLATORS:

			variation => N__('French: Advance, 5. Nf3 Qb6 6. Be2'),
			moves => {
				'c3d4' => 'r1b1kbnr/pp3ppp/1qn1p3/3pP3/3P4/5N2/PP2BPPP/RNBQK2R b KQkq - 0 7',
			},
		},
		'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'B00v',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. exd5 Qxd5 4. Nc3

			variation => N__('Nimzowitsch Defence: Marshall Gambit'),
			moves => {
			},
		},
		'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D07c',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'e7e5' => 'r1b1kbnr/ppp2ppp/2n5/3qp3/3P4/4P3/PP3PPP/RNBQKBNR w KQkq e6 0 5',
			},
		},
		'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'B00v',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. exd5 Qxd5 4. Nf3

			variation => N__('Nimzowitsch Defence: 2. d4 d5 3.exd5 Qxd5 4. Nf3'),
			moves => {
			},
		},
		'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D07c',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'e2e3' => 'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B00v',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. exd5 Qxd5

			variation => N__('Nimzowitsch Defence: 2. d4 d5 3.exd5 Qxd5'),
			moves => {
				'b1c3' => 'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4',
				'g1f3' => 'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'r1b1kbnr/ppp1qppp/2n5/4P3/2Pp4/5N2/PP1NPPPP/R1BQKB1R w KQkq - 4 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. Nbd2 Qe7

			variation => N__('QGD: Albin, Balogh Variation'),
			moves => {
			},
		},
		'r1b1kbnr/ppp2ppp/2n5/3p4/3PPp1q/2N5/PPP1K1PP/R1BQ1BNR w kq d6 0 6' => {
			eco => 'C25o',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. d4 Qh4+ 5. Ke2 d5

			variation => N__('Vienna: Steinitz Gambit, Zukertort Defence'),
			moves => {
			},
		},
		'r1b1kbnr/ppp2ppp/2n5/3qp3/3P4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 1 5' => {
			eco => 'D07c',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'f8b4' => 'r1b1k1nr/ppp2ppp/2n5/3qp3/1b1P4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 2 6',
			},
		},
		'r1b1kbnr/ppp2ppp/2n5/3qp3/3P4/4P3/PP3PPP/RNBQKBNR w KQkq e6 0 5' => {
			eco => 'D07c',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'b1c3' => 'r1b1kbnr/ppp2ppp/2n5/3qp3/3P4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 1 5',
			},
		},
		'r1b1kbnr/ppp3pp/2N5/1B1q4/4p3/8/PPPP1PPP/R1BQK2R w KQkq - 1 8' => {
			eco => 'C63t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 d5 6. Nxe5 dxe4 7. Nxc6 Qd5

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 d5 6.Nxe5 dxe4 7.Nxc6 Qd5'),
			moves => {
			},
		},
		'r1b1kbnr/ppp3pp/2N5/1B4q1/4p3/8/PPPP1PPP/R1BQK2R w KQkq - 1 8' => {
			eco => 'C63u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 d5 6. Nxe5 dxe4 7. Nxc6 Qg5

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 d5 6.Nxe5 dxe4 7.Nxc6 Qg5'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1p1p/2n5/4N3/2B1Pppq/8/PPPP2PP/RNBQ1K1R w kq - 4 7' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Ne5 Qh4+ 6. Kf1 Nc6

			variation => N__('KGA: Salvio, Herzfeld Gambit'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1p1p/5q2/4n3/2B1PQ2/2N5/PPPP2PP/R1B2RK1 w kq - 1 9' => {
			eco => 'C25p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5 5. Bc4 g4 6. O-O gxf3 7. Qxf3 Ne5 8. Qxf4 Qf6

			variation => N__('Vienna: Hamppe-Muzio, Dubois Variation'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1ppp/2n2q2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C45d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qf6

			variation => N__('Scotch: 4.Nxd4 Qf6'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/1N6/4P2q/8/PPP2PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5

			variation => N__('Scotch: Steinitz, 5. Nb5'),
			moves => {
				'f8b4' => 'r1b1k1nr/pppp1ppp/2n5/1N6/1b2P2q/8/PPP2PPP/RNBQKB1R w KQkq - 3 6',
				'f8c5' => 'r1b1k1nr/pppp1ppp/2n5/1Nb5/4P2q/8/PPP2PPP/RNBQKB1R w KQkq - 3 6',
				'h4e4' => 'r1b1kbnr/pppp1ppp/2n5/1N6/4q3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/1N6/4q3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Qxe4+

			variation => N__('Scotch: Steinitz, 5. Nb5 Qxe4+'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq - 4 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 Nc6

			variation => N__('KGA: Bishop\'s Gambit, Boden Defence'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/3NP2q/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'C45f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nc3

			variation => N__('Scotch: Steinitz, 5. Nc3'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/3NP2q/3Q4/PPP2PPP/RNB1KB1R b KQkq - 2 5' => {
			eco => 'C45e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Qd3

			variation => N__('Scotch: Steinitz, 5. Qd3'),
			moves => {
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/3NP2q/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C45e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4

			variation => N__('Scotch: Steinitz Variation'),
			moves => {
				'b1c3' => 'r1b1kbnr/pppp1ppp/2n5/8/3NP2q/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5',
				'd1d3' => 'r1b1kbnr/pppp1ppp/2n5/8/3NP2q/3Q4/PPP2PPP/RNB1KB1R b KQkq - 2 5',
				'd4b5' => 'r1b1kbnr/pppp1ppp/2n5/1N6/4P2q/8/PPP2PPP/RNBQKB1R b KQkq - 2 5',
				'd4f3' => 'r1b1kbnr/pppp1ppp/2n5/8/4P2q/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/3PPp1q/2N5/PPP1K1PP/R1BQ1BNR b kq - 2 5' => {
			eco => 'C25n',
                        # TRANSLATORS:

			variation => N__('Vienna: Steinitz Gambit'),
			moves => {
				'b7b6' => 'r1b1kbnr/p1pp1ppp/1pn5/8/3PPp1q/2N5/PPP1K1PP/R1BQ1BNR w kq - 0 6',
				'd7d5' => 'r1b1kbnr/ppp2ppp/2n5/3p4/3PPp1q/2N5/PPP1K1PP/R1BQ1BNR w kq d6 0 6',
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/3PPp1q/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C25n',
                        # TRANSLATORS:

			variation => N__('Vienna: Steinitz Gambit'),
			moves => {
				'e1e2' => 'r1b1kbnr/pppp1ppp/2n5/8/3PPp1q/2N5/PPP1K1PP/R1BQ1BNR b kq - 2 5',
			},
		},
		'r1b1kbnr/pppp1ppp/2n5/8/4P2q/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'C45e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nf3

			variation => N__('Scotch: Steinitz, Fraser Attack'),
			moves => {
			},
		},
		'r1b1kbnr/ppppqppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C60f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Qe7

			variation => N__('Spanish: Vinogradov Variation'),
			moves => {
			},
		},
		'r1b1kbnr/ppppqppp/2n5/4P3/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 3 4' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 Nc6 3. Nf3 Qe7

			variation => N__('Englund Gambit: 2.dxe5 Nc6 3. Nf3 Qe7'),
			moves => {
			},
		},
		'r1b1kbnr/ppqp1ppp/2N1p3/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B47b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Nxc6

			variation => N__('Sicilian: Taimanov, 6.Nxc6'),
			moves => {
			},
		},
		'r1b1kbnr/ppqp1ppp/2n1p3/1N6/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B47c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Ndb5

			variation => N__('Sicilian: Taimanov, 6.Ndb5'),
			moves => {
			},
		},
		'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 4 6' => {
			eco => 'B48a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3

			variation => N__('Sicilian: Taimanov, 6. Be3'),
			moves => {
				'a7a6' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7',
				'g8f6' => 'r1b1kb1r/ppqp1ppp/2n1pn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 5 7',
			},
		},
		'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B47g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3

			variation => N__('Sicilian: Taimanov, 6. g3'),
			moves => {
				'a7a6' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B47q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be2

			variation => N__('Sicilian: Taimanov, 6. Be2'),
			moves => {
				'a7a6' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7',
				'g8f6' => 'r1b1kb1r/ppqp1ppp/2n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 5 7',
			},
		},
		'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B47a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7

			variation => N__('Sicilian: Taimanov, Bastrikov Variation'),
			moves => {
				'c1e3' => 'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 4 6',
				'd4b5' => 'r1b1kbnr/ppqp1ppp/2n1p3/1N6/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 4 6',
				'd4c6' => 'r1b1kbnr/ppqp1ppp/2N1p3/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'f1e2' => 'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6',
				'f2f4' => 'r1b1kbnr/ppqp1ppp/2n1p3/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6',
				'g2g3' => 'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1b1kbnr/ppqp1ppp/2n1p3/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6' => {
			eco => 'B47d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. f4

			variation => N__('Sicilian: Taimanov, 6. f4'),
			moves => {
				'a7a6' => 'r1b1kbnr/1pqp1ppp/p1n1p3/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1b1kbnr/ppqppppp/2n5/1N6/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'B32g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Qc7 5. Nb5

			variation => N__('Sicilian: Flohr, 5. Nb5'),
			moves => {
			},
		},
		'r1b1kbnr/ppqppppp/2n5/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B32f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Qc7

			variation => N__('Sicilian: Flohr Variation'),
			moves => {
				'd4b5' => 'r1b1kbnr/ppqppppp/2n5/1N6/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'r1b1qrk1/1pp1b1pp/n2ppn2/p4p2/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 w - - 2 10' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Bb2 Na6

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Bb2 Na6'),
			moves => {
			},
		},
		'r1b1qrk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2Q1RK1 b - - 5 11' => {
			eco => 'E92w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6 9. Bh4 Na6 10. Nd2 Qe8 11. O-O

			variation => N__('King\'s Indian: Petrosian, Stein, Main Line, 10... Qe8 11.O-O'),
			moves => {
				'c8d7' => 'r3qrk1/1ppb1pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2Q1RK1 w - - 6 12',
				'f6h7' => 'r1b1qrk1/1pp2pbn/n2p2pp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2Q1RK1 w - - 6 12',
			},
		},
		'r1b1qrk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2QK2R w KQ - 4 11' => {
			eco => 'E92v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6 9. Bh4 Na6 10. Nd2 Qe8

			variation => N__('King\'s Indian: Petrosian, Stein, Main Line, 10... Qe8'),
			moves => {
				'e1g1' => 'r1b1qrk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2Q1RK1 b - - 5 11',
			},
		},
		'r1b1qrk1/1pp2pbn/n2p2pp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2Q1RK1 w - - 6 12' => {
			eco => 'E92x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6 9. Bh4 Na6 10. Nd2 Qe8 11. O-O Nh7

			variation => N__('King\'s Indian: Petrosian, Stein, Main Line, 10... Qe8 11.O-O Nh7'),
			moves => {
			},
		},
		'r1b1qrk1/ppp1b1pp/2nppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 Nc6

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 Nc6'),
			moves => {
			},
		},
		'r1b1qrk1/ppp1b1pp/2nppn2/5p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 4 9' => {
			eco => 'A98',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Qc2 Nc6

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. Qc2 Nc6'),
			moves => {
			},
		},
		'r1b1qrk1/ppp1p1bp/n2p1np1/3P1p2/2P5/2N2NP1/PP2PPBP/1RBQ1RK1 b - - 2 9' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. d5 Na6 9. Rb1

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. d5 Na6 9. Rb1'),
			moves => {
			},
		},
		'r1b1qrk1/ppp1p1bp/n2p1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. d5 Na6

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. d5 Na6'),
			moves => {
				'a1b1' => 'r1b1qrk1/ppp1p1bp/n2p1np1/3P1p2/2P5/2N2NP1/PP2PPBP/1RBQ1RK1 b - - 2 9',
			},
		},
		'r1b1qrk1/ppp2pb1/n5pp/4p1B1/2P1P1n1/2N2N1P/PP2BPP1/R2Q1RK1 w - - 0 12' => {
			eco => 'E94o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Be3 Ng4 9. Bg5 Qe8 10. dxe5 dxe5 11. h3 h6

			variation => N__('King\'s Indian: Glek, Main Line, 11. h3 h6'),
			moves => {
			},
		},
		'r1b1qrk1/ppp2pbp/n2p1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 1 9' => {
			eco => 'E73u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6 7. Qd2 e5 8. d5 Qe8

			variation => N__('King\'s Indian: Averbakh, 6... Na6 7. Qd2 e5 8. d5 Qe8'),
			moves => {
			},
		},
		'r1b1qrk1/ppp2pbp/n2p2p1/4P1B1/2P1P1n1/2N2N2/PP2BPPP/R2Q1RK1 b - - 0 10' => {
			eco => 'E94n',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Glek, 8. Be3 Ng4 9. Ng5 Qe8'),
			moves => {
				'd6e5' => 'r1b1qrk1/ppp2pbp/n5p1/4p1B1/2P1P1n1/2N2N2/PP2BPPP/R2Q1RK1 w - - 0 11',
			},
		},
		'r1b1qrk1/ppp2pbp/n2p2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 w - - 6 10' => {
			eco => 'E94n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Be3 Ng4 9. Bg5 Qe8

			variation => N__('King\'s Indian: Glek, 8. Be3 Ng4 9. Ng5 Qe8'),
			moves => {
				'd4e5' => 'r1b1qrk1/ppp2pbp/n2p2p1/4P1B1/2P1P1n1/2N2N2/PP2BPPP/R2Q1RK1 b - - 0 10',
			},
		},
		'r1b1qrk1/ppp2pbp/n5p1/4p1B1/2P1P1n1/2N2N1P/PP2BPP1/R2Q1RK1 b - - 0 11' => {
			eco => 'E94o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Be3 Ng4 9. Bg5 Qe8 10. dxe5 dxe5 11. h3

			variation => N__('King\'s Indian: Glek, Main Line, 11. h3'),
			moves => {
				'h7h6' => 'r1b1qrk1/ppp2pb1/n5pp/4p1B1/2P1P1n1/2N2N1P/PP2BPP1/R2Q1RK1 w - - 0 12',
			},
		},
		'r1b1qrk1/ppp2pbp/n5p1/4p1B1/2P1P1n1/2N2N2/PP2BPPP/R2Q1RK1 w - - 0 11' => {
			eco => 'E94o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Be3 Ng4 9. Bg5 Qe8 10. dxe5 dxe5

			variation => N__('King\'s Indian: Glek, Main Line'),
			moves => {
				'h2h3' => 'r1b1qrk1/ppp2pbp/n5p1/4p1B1/2P1P1n1/2N2N1P/PP2BPP1/R2Q1RK1 b - - 0 11',
			},
		},
		'r1b1qrk1/pppnb1pp/3ppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 Nbd7

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 Nbd7'),
			moves => {
			},
		},
		'r1b1r1k1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13' => {
			eco => 'C97f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Re8

			variation => N__('Spanish: Closed, Chigorin, 12... Re8'),
			moves => {
			},
		},
		'r1b1r1k1/pp1n1pbp/1qpp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 3 11' => {
			eco => 'E69l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. Re1 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. Re1 Re8'),
			moves => {
			},
		},
		'r1b1r1k1/pp1n1pbp/1qpp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - - 1 12' => {
			eco => 'E69n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. Re1 exd4 11. Nxd4 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. Re1 exd4 11.Nxd4 Re8'),
			moves => {
			},
		},
		'r1b1r1k1/ppq2ppp/2n2n2/2p1P3/8/P1PBPN2/2Q2PPP/R1B2RK1 b - - 0 13' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Bd3 e5 12. Qc2 Re8 13. dxe5

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bd3 e5 12. Qc2 Re8 13.de5'),
			moves => {
			},
		},
		'r1b1r1k1/ppq2ppp/2n2n2/2p1p3/3P4/P1PBPN2/2Q2PPP/R1B2RK1 w - - 2 13' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Bd3 e5 12. Qc2 Re8

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bd3 e5 12. Qc2 Re8'),
			moves => {
				'd4e5' => 'r1b1r1k1/ppq2ppp/2n2n2/2p1P3/8/P1PBPN2/2Q2PPP/R1B2RK1 b - - 0 13',
			},
		},
		'r1b1r1k1/ppq2ppp/2n2n2/2pp4/P2P4/2PBPN2/5PPP/R1BQ1RK1 w - - 1 12' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. cxd5 exd5 11. a4 Re8

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10.cxd5 exd5 11. a4 Re8'),
			moves => {
			},
		},
		'r1b1r1k1/ppqn1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 3 11' => {
			eco => 'E69t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 Qc7

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 Qc7'),
			moves => {
			},
		},
		'r1b2rk1/1p1nqpbp/p2p1np1/2pP4/P3PB2/2N2N2/1P2BPPP/R2Q1RK1 w - - 3 12' => {
			eco => 'A74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Nbd7 11. Bf4 Qe7

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Nbd7 11. Bf4 Qe7'),
			moves => {
			},
		},
		'r1b2rk1/1pq1bppp/p1nppn2/8/3NP3/1BN1B3/PPP1QPPP/2KR3R w - - 0 11' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 Be7 8. Qe2 O-O 9. O-O-O Qc7 10. Bb3 a6

			variation => N__('Sicilian: Velimirovic, 9.O-O-O Qc7 10. Bb3 a6'),
			moves => {
			},
		},
		'r1b2rk1/1pq1bppp/p1nppn2/8/3NPP2/2N1B1Q1/PPP1B1PP/R4RK1 b - - 6 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. Qe1 Qc7 11. Qg3

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. Qe1 Qc7 11. Qg3'),
			moves => {
			},
		},
		'r1b2rk1/1pq1bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K w - - 5 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. Kh1 Qc7

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. Kh1 Qc7'),
			moves => {
			},
		},
		'r1b2rk1/1pq1bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R3QRK1 w - - 5 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. Qe1 Qc7

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. Qe1 Qc7'),
			moves => {
				'e1g3' => 'r1b2rk1/1pq1bppp/p1nppn2/8/3NPP2/2N1B1Q1/PPP1B1PP/R4RK1 b - - 6 11',
			},
		},
		'r1b2rk1/1pq1bppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1R1K b - - 2 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. a4 Qc7 11. Kh1

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. a4 Qc7'),
			moves => {
			},
		},
		'r1b2rk1/1pq1bppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1RK1 w - - 1 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. a4 Qc7

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. a4 Qc7'),
			moves => {
				'g1h1' => 'r1b2rk1/1pq1bppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1R1K b - - 2 11',
			},
		},
		'r1b2rk1/1pqp1ppp/p1n1pn2/8/Nb1NP3/4B3/PPP1BPPP/R2Q1RK1 w - - 6 10' => {
			eco => 'B49k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. Be3 a6 7. Be2 Nf6 8. O-O Bb4 9. Na4 O-O

			variation => N__('Sicilian: Taimanov, 6. Be3 a6 7. Be2 Nf6 8.O-O Bb4 9. Na4 O-O'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/2np1n2/1p2p3/p2PP3/1N2BN1P/PPB2PP1/R2QR1K1 w - - 0 16' => {
			eco => 'C99m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. Nb3 a5 15. Be3 a4

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3 a5 15. Be3 a4'),
			moves => {
				'b3d2' => 'r1b2rk1/2q1bppp/2np1n2/1p2p3/p2PP3/4BN1P/PPBN1PP1/R2QR1K1 b - - 1 16',
			},
		},
		'r1b2rk1/2q1bppp/2np1n2/1p2p3/p2PP3/4BN1P/PPBN1PP1/R2QR1K1 b - - 1 16' => {
			eco => 'C99m',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3 a5 15. Be3 a4'),
			moves => {
				'c6b4' => 'r1b2rk1/2q1bppp/3p1n2/1p2p3/pn1PP3/4BN1P/PPBN1PP1/R2QR1K1 w - - 2 17',
				'c8d7' => 'r4rk1/2qbbppp/2np1n2/1p2p3/p2PP3/4BN1P/PPBN1PP1/R2QR1K1 w - - 2 17',
			},
		},
		'r1b2rk1/2q1bppp/2np1n2/pp2p3/3PP3/1N2BN1P/PPB2PP1/R2QR1K1 b - - 1 15' => {
			eco => 'C99l',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3 a5'),
			moves => {
				'a5a4' => 'r1b2rk1/2q1bppp/2np1n2/1p2p3/p2PP3/1N2BN1P/PPB2PP1/R2QR1K1 w - - 0 16',
			},
		},
		'r1b2rk1/2q1bppp/2np1n2/pp2p3/3PP3/1N3N1P/PPB2PP1/R1BQR1K1 w - - 0 15' => {
			eco => 'C99l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. Nb3 a5

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3 a5'),
			moves => {
				'c1e3' => 'r1b2rk1/2q1bppp/2np1n2/pp2p3/3PP3/1N2BN1P/PPB2PP1/R2QR1K1 b - - 1 15',
			},
		},
		'r1b2rk1/2q1bppp/3p1n2/1p2p3/pn1PP3/4BN1P/PPBN1PP1/R2QR1K1 w - - 2 17' => {
			eco => 'C99n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. Nb3 a5 15. Be3 a4 16. Nbd2 Nb4

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3 a5 15. Be3 a4 16.Nbd2 Nb4'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p1n2n2/1pp1p3/4P3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 1 14' => {
			eco => 'C98c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. dxc5 dxc5 14. Nf1

			variation => N__('Spanish: Closed, Chigorin, Rauzer, 14. Nf1'),
			moves => {
				'c8e6' => 'r4rk1/2q1bppp/p1n1bn2/1pp1p3/4P3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 2 15',
			},
		},
		'r1b2rk1/2q1bppp/p1n2n2/1pp1p3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 0 14' => {
			eco => 'C98b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. dxc5 dxc5

			variation => N__('Spanish: Closed, Chigorin, Rauzer Attack'),
			moves => {
				'd2f1' => 'r1b2rk1/2q1bppp/p1n2n2/1pp1p3/4P3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 1 14',
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1p1Pp3/4P3/5N1P/PPBN1PP1/R1BQR1K1 b - - 0 14' => {
			eco => 'C99k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. d5

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. d5'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/1N3N1P/PPB2PP1/R1BQR1K1 b - - 2 14' => {
			eco => 'C99l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. Nb3

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3'),
			moves => {
				'a6a5' => 'r1b2rk1/2q1bppp/2np1n2/pp2p3/3PP3/1N3N1P/PPB2PP1/R1BQR1K1 w - - 0 15',
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 b - - 2 14' => {
			eco => 'C99j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. Nf1

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nf1'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14' => {
			eco => 'C99i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6'),
			moves => {
				'a2a3' => 'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/P4N1P/1PBN1PP1/R1BQR1K1 b - - 0 14',
				'd2b3' => 'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/1N3N1P/PPB2PP1/R1BQR1K1 b - - 2 14',
				'd2f1' => 'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 b - - 2 14',
				'd4d5' => 'r1b2rk1/2q1bppp/p1np1n2/1p1Pp3/4P3/5N1P/PPBN1PP1/R1BQR1K1 b - - 0 14',
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/P4N1P/1PBN1PP1/R1BQR1K1 b - - 0 14' => {
			eco => 'C99i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. a3

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. a3'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1pP1p3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 0 13' => {
			eco => 'C98b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. dxc5

			variation => N__('Spanish: Closed, Chigorin, Rauzer Attack'),
			moves => {
				'd6c5' => 'r1b2rk1/2q1bppp/p1n2n2/1pp1p3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 0 14',
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1pp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13' => {
			eco => 'C98a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6

			variation => N__('Spanish: Closed, Chigorin, 12... Nc6'),
			moves => {
				'd4c5' => 'r1b2rk1/2q1bppp/p1np1n2/1pP1p3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 0 13',
				'd4d5' => 'r1b2rk1/2q1bppp/p1np1n2/1ppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 0 13',
			},
		},
		'r1b2rk1/2q1bppp/p1np1n2/1ppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 0 13' => {
			eco => 'C98e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. d5

			variation => N__('Spanish: Closed, Chigorin, 12... Nc6 13. d5'),
			moves => {
				'c6a5' => 'r1b2rk1/2q1bppp/p2p1n2/nppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 1 14',
				'c6d8' => 'r1bn1rk1/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 1 14',
			},
		},
		'r1b2rk1/2q1bppp/p1pp1n2/np2p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C96d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c6 11. d4 Qc7

			variation => N__('Spanish: Closed, Chigorin, Rossolimo Variation'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/np1Pp3/2p1P3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 0 13' => {
			eco => 'C97c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. d5 c4

			variation => N__('Spanish: Closed, Chigorin, 11. d4 Qc7 12. d5 c4'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 b - - 0 13' => {
			eco => 'C99a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4

			variation => N__('Spanish: Closed, Chigorin, 12...cxd4 13.cxd4'),
			moves => {
				'a5c6' => 'r1b2rk1/2q1bppp/p1np1n2/1p2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14',
				'c8b7' => 'r4rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14',
				'c8d7' => 'r4rk1/2qbbppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14',
				'f8d8' => 'r1br2k1/2q1bppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14',
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/np2p3/3pP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 0 13' => {
			eco => 'C99a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4

			variation => N__('Spanish: Closed, Chigorin, 12...cxd4'),
			moves => {
				'c3d4' => 'r1b2rk1/2q1bppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 b - - 0 13',
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C97a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7

			variation => N__('Spanish: Closed, Chigorin, 11. d4 Qc7'),
			moves => {
				'b1d2' => 'r1b2rk1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 2 12',
				'd4d5' => 'r1b2rk1/2q1bppp/p2p1n2/nppPp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 b - - 0 12',
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 2 12' => {
			eco => 'C97d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2

			variation => N__('Spanish: Closed, Chigorin, 11. d4 Qc7 12.Nbd2'),
			moves => {
				'a5c6' => 'r1b2rk1/2q1bppp/p1np1n2/1pp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13',
				'c5d4' => 'r1b2rk1/2q1bppp/p2p1n2/np2p3/3pP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 0 13',
				'c8b7' => 'r4rk1/1bq1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13',
				'c8d7' => 'r4rk1/2qbbppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13',
				'f8d8' => 'r1br2k1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13',
				'f8e8' => 'r1b1r1k1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13',
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 w - - 2 12' => {
			eco => 'C90i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Qc7

			variation => N__('Spanish: Closed, Pilnik, 11.Nbd2 Qc7'),
			moves => {
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/nppPp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 b - - 0 12' => {
			eco => 'C97b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. d5

			variation => N__('Spanish: Closed, Chigorin, 11. d4 Qc7 12. d5'),
			moves => {
				'c5c4' => 'r1b2rk1/2q1bppp/p2p1n2/np1Pp3/2p1P3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 0 13',
			},
		},
		'r1b2rk1/2q1bppp/p2p1n2/nppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 1 14' => {
			eco => 'C98f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. d5 Na5

			variation => N__('Spanish: Closed, Chigorin, 12... Nc6 13. d5 Na5'),
			moves => {
			},
		},
		'r1b2rk1/5ppp/p1pb4/1p1n4/3P3q/1BP3P1/PP3P1P/RNBQR1K1 b - - 0 14' => {
			eco => 'C89m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3

			variation => N__('Spanish: Marshall, Main Line, 13. Re1 Qh4 14. g3'),
			moves => {
				'h4h3' => 'r1b2rk1/5ppp/p1pb4/1p1n4/3P4/1BP3Pq/PP3P1P/RNBQR1K1 w - - 1 15',
			},
		},
		'r1b2rk1/5ppp/p1pb4/1p1n4/3P3q/1BP5/PP3PPP/RNBQR1K1 w - - 3 14' => {
			eco => 'C89m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4

			variation => N__('Spanish: Marshall, Main Line, 13. Re1 Qh4'),
			moves => {
				'g2g3' => 'r1b2rk1/5ppp/p1pb4/1p1n4/3P3q/1BP3P1/PP3P1P/RNBQR1K1 b - - 0 14',
			},
		},
		'r1b2rk1/5ppp/p1pb4/1p1n4/3P4/1BP1B1Pq/PP3P1P/RN1QR1K1 b - - 2 15' => {
			eco => 'C89p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3

			variation => N__('Spanish: Marshall, Main Line, 15. Be3'),
			moves => {
				'c8g4' => 'r4rk1/5ppp/p1pb4/1p1n4/3P2b1/1BP1B1Pq/PP3P1P/RN1QR1K1 w - - 3 16',
			},
		},
		'r1b2rk1/5ppp/p1pb4/1p1n4/3P4/1BP3Pq/PP3P1P/RNBQR1K1 w - - 1 15' => {
			eco => 'C89n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3

			variation => N__('Spanish: Marshall, Main Line, 14. g3 Qh3'),
			moves => {
				'c1e3' => 'r1b2rk1/5ppp/p1pb4/1p1n4/3P4/1BP1B1Pq/PP3P1P/RN1QR1K1 b - - 2 15',
				'e1e4' => 'r1b2rk1/5ppp/p1pb4/1p1n4/3PR3/1BP3Pq/PP3P1P/RNBQ2K1 b - - 2 15',
			},
		},
		'r1b2rk1/5ppp/p1pb4/1p1n4/3PR3/1BP3Pq/PP3P1P/RNBQ2K1 b - - 2 15' => {
			eco => 'C89o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Re4

			variation => N__('Spanish: Marshall, Main Line, 15. Re4'),
			moves => {
			},
		},
		'r1b2rk1/5ppp/p1pb4/1p1n4/7q/1BPP4/PP3PPP/RNBQR1K1 w - - 3 14' => {
			eco => 'C89j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d3 Bd6 13. Re1 Qh4

			variation => N__('Spanish: Marshall, 12. d3 Bd6 13. Re1 Qh4'),
			moves => {
			},
		},
		'r1b2rk1/pp1n1pbp/1qpp1np1/2P1p3/3PP3/2N2NPP/PP3PB1/R1BQ1RK1 b - - 0 10' => {
			eco => 'E69j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. c5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. c5'),
			moves => {
			},
		},
		'r1b2rk1/pp1n1pbp/1qpp1np1/3Pp3/2P1P3/2N2NPP/PP3PB1/R1BQ1RK1 b - - 0 10' => {
			eco => 'E69i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. d5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. d5'),
			moves => {
			},
		},
		'r1b2rk1/pp1n1pbp/1qpp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10' => {
			eco => 'E69i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6'),
			moves => {
				'c4c5' => 'r1b2rk1/pp1n1pbp/1qpp1np1/2P1p3/3PP3/2N2NPP/PP3PB1/R1BQ1RK1 b - - 0 10',
				'd4d5' => 'r1b2rk1/pp1n1pbp/1qpp1np1/3Pp3/2P1P3/2N2NPP/PP3PB1/R1BQ1RK1 b - - 0 10',
				'f1e1' => 'r1b2rk1/pp1n1pbp/1qpp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 b - - 2 10',
			},
		},
		'r1b2rk1/pp1n1pbp/1qpp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 b - - 2 10' => {
			eco => 'E69k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. Re1

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. Re1'),
			moves => {
				'e5d4' => 'r1b2rk1/pp1n1pbp/1qpp1np1/8/2PpP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 0 11',
				'f8e8' => 'r1b1r1k1/pp1n1pbp/1qpp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 3 11',
			},
		},
		'r1b2rk1/pp1n1pbp/1qpp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 0 11' => {
			eco => 'E69m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. Re1 exd4 11. Nxd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. Re1 exd4'),
			moves => {
				'f8e8' => 'r1b1r1k1/pp1n1pbp/1qpp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - - 1 12',
			},
		},
		'r1b2rk1/pp1n1pbp/1qpp1np1/8/2PpP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 0 11' => {
			eco => 'E69m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qb6 10. Re1 exd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qb6 10. Re1 exd4'),
			moves => {
				'f3d4' => 'r1b2rk1/pp1n1pbp/1qpp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 0 11',
			},
		},
		'r1b2rk1/pp1n1pbp/2pp1np1/q3p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10' => {
			eco => 'E69g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qa5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qa5'),
			moves => {
				'f1e1' => 'r1b2rk1/pp1n1pbp/2pp1np1/q3p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 b - - 2 10',
			},
		},
		'r1b2rk1/pp1n1pbp/2pp1np1/q3p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 b - - 2 10' => {
			eco => 'E69h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qa5 10. Re1

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qa5 10. Re1'),
			moves => {
				'e5d4' => 'r1b2rk1/pp1n1pbp/2pp1np1/q7/2PpP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 0 11',
			},
		},
		'r1b2rk1/pp1n1pbp/2pp1np1/q7/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 0 11' => {
			eco => 'E69h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qa5 10. Re1 exd4 11. Nxd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qa5 10. Re1 exd4'),
			moves => {
			},
		},
		'r1b2rk1/pp1n1pbp/2pp1np1/q7/2PpP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 0 11' => {
			eco => 'E69h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qa5 10. Re1 exd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qa5 10. Re1 exd4'),
			moves => {
				'f3d4' => 'r1b2rk1/pp1n1pbp/2pp1np1/q7/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 0 11',
			},
		},
		'r1b2rk1/pp1n1ppp/2n1p3/2qpP3/5P2/2N2N2/PPPQ2PP/R3KB1R w KQ - 0 11' => {
			eco => 'C14o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O 8. Nf3 c5 9. Qd2 Nc6 10. dxc5 Qxc5

			variation => N__('French: Classical, Steinitz, 9. Qd2 Nc6 10.dxc5 Qxc5'),
			moves => {
			},
		},
		'r1b2rk1/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PPQN1PPP/R3KB1R w KQ - 6 9' => {
			eco => 'D52l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4 8. Qc2 O-O

			variation => N__('QGD: Cambridge Springs, 8. Qc2 O-O'),
			moves => {
				'f1e2' => 'r1b2rk1/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PPQNBPPP/R3K2R b KQ - 7 9',
				'g5h4' => 'r1b2rk1/pp1n1ppp/2p1pn2/q2p4/1bPP3B/2N1P3/PPQN1PPP/R3KB1R b KQ - 7 9',
			},
		},
		'r1b2rk1/pp1n1ppp/2p1pn2/q2p2B1/1bPP4/2N1P3/PPQNBPPP/R3K2R b KQ - 7 9' => {
			eco => 'D52n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4 8. Qc2 O-O 9. Be2

			variation => N__('QGD: Cambridge Springs, 8. Qc2 O-O 9. Be2'),
			moves => {
				'e6e5' => 'r1b2rk1/pp1n1ppp/2p2n2/q2pp1B1/1bPP4/2N1P3/PPQNBPPP/R3K2R w KQ - 0 10',
			},
		},
		'r1b2rk1/pp1n1ppp/2p1pn2/q2p4/1bPP3B/2N1P3/PPQN1PPP/R3KB1R b KQ - 7 9' => {
			eco => 'D52m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4 8. Qc2 O-O 9. Bh4

			variation => N__('QGD: Cambridge Springs, Argentine Variation'),
			moves => {
			},
		},
		'r1b2rk1/pp1n1ppp/2p2n2/q2pp1B1/1bPP4/2N1P3/PPQNBPPP/R3K2R w KQ - 0 10' => {
			eco => 'D52o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3 Qa5 7. Nd2 Bb4 8. Qc2 O-O 9. Be2 e5

			variation => N__('QGD: Cambridge Springs, 8. Qc2 O-O 9. Be2 e5'),
			moves => {
			},
		},
		'r1b2rk1/pp1nppbp/6p1/q1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11' => {
			eco => 'D87c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. Be3 Qa5 10. O-O Nd7

			variation => N__('Grünfeld: Classical Exchange, 8... c5 9. Be3 Qa5'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqp1p/6p1/2pp4/3P4/3BPN2/PPQ2PPP/R3K2R w KQ - 0 12' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Bd3 g6

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5, Main Line, 11. Bd3 g6'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqpbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10' => {
			eco => 'E69b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Qe7

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Qe7'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2n1p3/2PpP3/5P2/2N2N2/PPPQ2PP/R3KB1R b KQ - 0 10' => {
			eco => 'C14n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O 8. Nf3 c5 9. Qd2 Nc6 10. dxc5

			variation => N__('French: Classical, Steinitz, 9. Qd2 Nc6 10.dxc5'),
			moves => {
				'e7c5' => 'r1b2rk1/pp1n1ppp/2n1p3/2qpP3/5P2/2N2N2/PPPQ2PP/R3KB1R w KQ - 0 11',
			},
		},
		'r1b2rk1/pp1nqppp/2n1p3/2ppP3/3P1P2/2N2N2/PPPQ2PP/2KR1B1R b - - 3 10' => {
			eco => 'C14m',
                        # TRANSLATORS:

			variation => N__('French: Classical, Steinitz, 9. Qd2'),
			moves => {
				'c5c4' => 'r1b2rk1/pp1nqppp/2n1p3/3pP3/2pP1P2/2N2N2/PPPQ2PP/2KR1B1R w - - 0 11',
			},
		},
		'r1b2rk1/pp1nqppp/2n1p3/2ppP3/3P1P2/2N2N2/PPPQ2PP/R3KB1R w KQ - 2 10' => {
			eco => 'C14m',
                        # TRANSLATORS:

			variation => N__('French: Classical, Steinitz, 9. Qd2'),
			moves => {
				'd4c5' => 'r1b2rk1/pp1nqppp/2n1p3/2PpP3/5P2/2N2N2/PPPQ2PP/R3KB1R b KQ - 0 10',
				'e1c1' => 'r1b2rk1/pp1nqppp/2n1p3/2ppP3/3P1P2/2N2N2/PPPQ2PP/2KR1B1R b - - 3 10',
			},
		},
		'r1b2rk1/pp1nqppp/2n1p3/3pP3/2pP1P2/2N2N2/PPPQ2PP/2KR1B1R w - - 0 11' => {
			eco => 'C14m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O 8. Nf3 c5 9. Qd2 Nc6 10. O-O-O c4

			variation => N__('French: Classical, Stahlberg Variation'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2p1p3/3n4/2BP4/2N1PN2/PP3PPP/2RQ1RK1 b - - 1 11' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O

			variation => N__('QGD: Orthodox, Main Line, Capablanca, 11.O-O'),
			moves => {
				'd5c3' => 'r1b2rk1/pp1nqppp/2p1p3/8/2BP4/2n1PN2/PP3PPP/2RQ1RK1 w - - 0 12',
			},
		},
		'r1b2rk1/pp1nqppp/2p1p3/3n4/2BP4/2N1PN2/PP3PPP/2RQK2R w K - 0 11' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7

			variation => N__('QGD: Orthodox, Main Line, Capablanca Variation'),
			moves => {
				'c3e4' => 'r1b2rk1/pp1nqppp/2p1p3/3n4/2BPN3/4PN2/PP3PPP/2RQK2R b K - 1 11',
				'e1g1' => 'r1b2rk1/pp1nqppp/2p1p3/3n4/2BP4/2N1PN2/PP3PPP/2RQ1RK1 b - - 1 11',
			},
		},
		'r1b2rk1/pp1nqppp/2p1p3/3n4/2BPN3/4PN2/PP3PPP/2RQK2R b K - 1 11' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. Ne4

			variation => N__('QGD: Orthodox, Main Line, Alekhine Variation'),
			moves => {
				'd5f6' => 'r1b2rk1/pp1nqppp/2p1pn2/8/2BPN3/4PN2/PP3PPP/2RQK2R w K - 2 12',
				'e6e5' => 'r1b2rk1/pp1nqppp/2p5/3np3/2BPN3/4PN2/PP3PPP/2RQK2R w K - 0 12',
			},
		},
		'r1b2rk1/pp1nqppp/2p1p3/8/2BP4/2R1PN2/PP3PPP/3Q1RK1 b - - 0 12' => {
			eco => 'D67',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Main Line, Capablanca, 11.O-O Nxc3'),
			moves => {
				'e6e5' => 'r1b2rk1/pp1nqppp/2p5/4p3/2BP4/2R1PN2/PP3PPP/3Q1RK1 w - - 0 13',
			},
		},
		'r1b2rk1/pp1nqppp/2p1p3/8/2BP4/2n1PN2/PP3PPP/2RQ1RK1 w - - 0 12' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3

			variation => N__('QGD: Orthodox, Main Line, Capablanca, 11.O-O Nxc3'),
			moves => {
				'c1c3' => 'r1b2rk1/pp1nqppp/2p1p3/8/2BP4/2R1PN2/PP3PPP/3Q1RK1 b - - 0 12',
			},
		},
		'r1b2rk1/pp1nqppp/2p1pn2/8/2BPN3/4PN2/PP3PPP/2RQK2R w K - 2 12' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. Ne4 N5f6

			variation => N__('QGD: Orthodox, Main Line, Alekhine, 11...N5f6'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2p5/3np3/2BPN3/4PN2/PP3PPP/2RQK2R w K - 0 12' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. Ne4 e5

			variation => N__('QGD: Orthodox, Main Line, Alekhine, 11... e5 (Lasker)'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2p5/4P3/2B5/2R1PN2/PP3PPP/3Q1RK1 b - - 0 13' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5

			variation => N__('QGD: Orthodox, Classical, 13.dxe5'),
			moves => {
				'd7e5' => 'r1b2rk1/pp2qppp/2p5/4n3/2B5/2R1PN2/PP3PPP/3Q1RK1 w - - 0 14',
			},
		},
		'r1b2rk1/pp1nqppp/2p5/4p3/2BP4/2R1PN2/PP3PPP/1Q3RK1 b - - 1 13' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Qb1

			variation => N__('QGD: Orthodox, Classical, Maroczy Variation'),
			moves => {
				'e5d4' => 'r1b2rk1/pp1nqppp/2p5/8/2Bp4/2R1PN2/PP3PPP/1Q3RK1 w - - 0 14',
			},
		},
		'r1b2rk1/pp1nqppp/2p5/4p3/2BP4/2R1PN2/PP3PPP/3Q1RK1 w - - 0 13' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5

			variation => N__('QGD: Orthodox, Classical Variation'),
			moves => {
				'c4b3' => 'r1b2rk1/pp1nqppp/2p5/4p3/3P4/1BR1PN2/PP3PPP/3Q1RK1 b - - 1 13',
				'd1b1' => 'r1b2rk1/pp1nqppp/2p5/4p3/2BP4/2R1PN2/PP3PPP/1Q3RK1 b - - 1 13',
				'd1c2' => 'r1b2rk1/pp1nqppp/2p5/4p3/2BP4/2R1PN2/PPQ2PPP/5RK1 b - - 1 13',
				'd4e5' => 'r1b2rk1/pp1nqppp/2p5/4P3/2B5/2R1PN2/PP3PPP/3Q1RK1 b - - 0 13',
			},
		},
		'r1b2rk1/pp1nqppp/2p5/4p3/2BP4/2R1PN2/PPQ2PPP/5RK1 b - - 1 13' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Qc2

			variation => N__('QGD: Orthodox, Classical, Vidmar Variation'),
			moves => {
				'e5d4' => 'r1b2rk1/pp1nqppp/2p5/8/2Bp4/2R1PN2/PPQ2PPP/5RK1 w - - 0 14',
				'e5e4' => 'r1b2rk1/pp1nqppp/2p5/8/2BPp3/2R1PN2/PPQ2PPP/5RK1 w - - 0 14',
			},
		},
		'r1b2rk1/pp1nqppp/2p5/4p3/3P4/1BR1PN2/PP3PPP/3Q1RK1 b - - 1 13' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Bb3

			variation => N__('QGD: Orthodox, Classical, 13. Bb3'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2p5/8/2BP4/2R2N2/PPQ2PPP/5RK1 b - - 0 14' => {
			eco => 'D68',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Classical, Vidmar, 13...exd4'),
			moves => {
				'd7b6' => 'r1b2rk1/pp2qppp/1np5/8/2BP4/2R2N2/PPQ2PPP/5RK1 w - - 1 15',
			},
		},
		'r1b2rk1/pp1nqppp/2p5/8/2BPp3/2R1PN2/PPQ2PPP/5RK1 w - - 0 14' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Qc2 e4

			variation => N__('QGD: Orthodox, Classical, Vidmar, 13... e4'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2p5/8/2Bp4/2R1PN2/PP3PPP/1Q3RK1 w - - 0 14' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Qb1 exd4

			variation => N__('QGD: Orthodox, Classical, Maroczy, 13...exd4'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2p5/8/2Bp4/2R1PN2/PPQ2PPP/5RK1 w - - 0 14' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Qc2 exd4

			variation => N__('QGD: Orthodox, Classical, Vidmar, 13...exd4'),
			moves => {
				'e3d4' => 'r1b2rk1/pp1nqppp/2p5/8/2BP4/2R2N2/PPQ2PPP/5RK1 b - - 0 14',
			},
		},
		'r1b2rk1/pp1nqppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 7 9' => {
			eco => 'D45x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. Be2 O-O 8. O-O Qe7

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Be2 O-O 8.O-O Qe7'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/2N1PN1P/PPQ2PP1/R1B2RK1 b - - 0 10' => {
			eco => 'D46j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 Qe7 10. h3

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... Qe7 10. h3'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - - 1 10' => {
			eco => 'D46j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 Qe7

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... Qe7'),
			moves => {
				'a2a3' => 'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/P1N1PN2/1PQ2PPP/R1B2RK1 b - - 0 10',
				'c1d2' => 'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/2N1PN2/PPQB1PPP/R4RK1 b - - 2 10',
				'h2h3' => 'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/2N1PN1P/PPQ2PP1/R1B2RK1 b - - 0 10',
			},
		},
		'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/2N1PN2/PPQB1PPP/R4RK1 b - - 2 10' => {
			eco => 'D46j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 Qe7 10. Bd2

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... Qe7 10. Bd2'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/P1N1PN2/1PQ2PPP/R1B2RK1 b - - 0 10' => {
			eco => 'D46j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 Qe7 10. a3

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... Qe7 10. a3'),
			moves => {
			},
		},
		'r1b2rk1/pp1nqppp/4p3/2pN4/3P4/4PN2/PPQ2PPP/R3KB1R b KQ - 0 10' => {
			eco => 'D62',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 Nxd5 9.Bxe7 Qxe7'),
			moves => {
				'e6d5' => 'r1b2rk1/pp1nqppp/8/2pp4/3P4/4PN2/PPQ2PPP/R3KB1R w KQ - 0 11',
			},
		},
		'r1b2rk1/pp1nqppp/4p3/2pn4/3P4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 10' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 Nxd5 9. Bxe7 Qxe7

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 Nxd5 9.Bxe7 Qxe7'),
			moves => {
				'c3d5' => 'r1b2rk1/pp1nqppp/4p3/2pN4/3P4/4PN2/PPQ2PPP/R3KB1R b KQ - 0 10',
			},
		},
		'r1b2rk1/pp1nqppp/8/2pp4/3P4/3BPN2/PPQ2PPP/R3K2R b KQ - 1 11' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Bd3

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5, Main Line, 11. Bd3'),
			moves => {
				'g7g6' => 'r1b2rk1/pp1nqp1p/6p1/2pp4/3P4/3BPN2/PPQ2PPP/R3K2R w KQ - 0 12',
			},
		},
		'r1b2rk1/pp1nqppp/8/2pp4/3P4/4PN2/PPQ2PPP/R3KB1R w KQ - 0 11' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5, Main Line'),
			moves => {
				'f1d3' => 'r1b2rk1/pp1nqppp/8/2pp4/3P4/3BPN2/PPQ2PPP/R3K2R b KQ - 1 11',
			},
		},
		'r1b2rk1/pp1pppbp/1qn2np1/8/2P5/2N3P1/PPN1PPBP/R1BQ1RK1 b - - 2 9' => {
			eco => 'A39e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Qb6 9. Nc2

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... Qb6'),
			moves => {
			},
		},
		'r1b2rk1/pp1pppbp/1qn2np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A39e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Qb6

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... Qb6'),
			moves => {
				'd4c2' => 'r1b2rk1/pp1pppbp/1qn2np1/8/2P5/2N3P1/PPN1PPBP/R1BQ1RK1 b - - 2 9',
			},
		},
		'r1b2rk1/pp1pppbp/2n2np1/q7/2BNP3/2N1B3/PPP2PPP/R2Q1RK1 w - - 8 9' => {
			eco => 'B35c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 Qa5 8. O-O O-O

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5 8.O-O O-O'),
			moves => {
				'c4b3' => 'r1b2rk1/pp1pppbp/2n2np1/q7/3NP3/1BN1B3/PPP2PPP/R2Q1RK1 b - - 9 9',
			},
		},
		'r1b2rk1/pp1pppbp/2n2np1/q7/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A39d',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Qa5

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... Qa5'),
			moves => {
			},
		},
		'r1b2rk1/pp1pppbp/2n2np1/q7/3NP3/1BN1B3/PPP2PPP/R2Q1RK1 b - - 9 9' => {
			eco => 'B35d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 Qa5 8. O-O O-O 9. Bb3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5 8.O-O O-O 9. Bb3'),
			moves => {
				'd7d6' => 'r1b2rk1/pp2ppbp/2np1np1/q7/3NP3/1BN1B3/PPP2PPP/R2Q1RK1 w - - 0 10',
			},
		},
		'r1b2rk1/pp1pppbp/2n2np1/q7/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ - 8 9' => {
			eco => 'B35k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 Qa5

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 Qa5'),
			moves => {
			},
		},
		'r1b2rk1/pp2bppp/1qnppn2/6B1/4P3/1NN2P2/PPPQ2PP/2KR1B1R b - - 0 10' => {
			eco => 'B63u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3 Qb6 10. f3

			variation => N__('Sicilian: Richter-Rauzer, Podebrad, 10. f3'),
			moves => {
				'f8d8' => 'r1br2k1/pp2bppp/1qnppn2/6B1/4P3/1NN2P2/PPPQ2PP/2KR1B1R w - - 1 11',
			},
		},
		'r1b2rk1/pp2bppp/1qnppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w - - 6 10' => {
			eco => 'B63t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3 Qb6

			variation => N__('Sicilian: Richter-Rauzer, Podebrad Variation'),
			moves => {
				'f2f3' => 'r1b2rk1/pp2bppp/1qnppn2/6B1/4P3/1NN2P2/PPPQ2PP/2KR1B1R b - - 0 10',
			},
		},
		'r1b2rk1/pp2bppp/3ppn2/q5B1/2BQPP2/2N5/PPP3PP/2KR3R b - - 2 11' => {
			eco => 'B65e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4 Qa5 11. Bc4

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4 10.Qxd4 Qa5 11. Bc4'),
			moves => {
				'c8d7' => 'r4rk1/pp1bbppp/3ppn2/q5B1/2BQPP2/2N5/PPP3PP/2KR3R w - - 3 12',
			},
		},
		'r1b2rk1/pp2bppp/3ppn2/q5B1/3QPP2/2N5/PPP3PP/1K1R1B1R b - - 2 11' => {
			eco => 'B65d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4 Qa5 11. Kb1

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4 10.Qxd4 Qa5 11. Kb1'),
			moves => {
			},
		},
		'r1b2rk1/pp2bppp/3ppn2/q5B1/3QPP2/2N5/PPP3PP/2KR1B1R w - - 1 11' => {
			eco => 'B65c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4 Qa5

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4 10.Qxd4 Qa5'),
			moves => {
				'c1b1' => 'r1b2rk1/pp2bppp/3ppn2/q5B1/3QPP2/2N5/PPP3PP/1K1R1B1R b - - 2 11',
				'f1c4' => 'r1b2rk1/pp2bppp/3ppn2/q5B1/2BQPP2/2N5/PPP3PP/2KR3R b - - 2 11',
			},
		},
		'r1b2rk1/pp2nppp/2pq1n2/3pp1B1/1b2P2N/1BNP4/PPP2PPP/R2Q1RK1 w - - 2 11' => {
			eco => 'C49k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Ne7 8. Nh4 c6 9. Bc4 d5 10. Bb3 Qd6

			variation => N__('Four Knights: Symmetrical, Blake Variation'),
			moves => {
			},
		},
		'r1b2rk1/pp2ppbp/1qnp1np1/4P3/3N1P2/2N1B3/PPP1B1PP/R2Q1RK1 b - - 0 10' => {
			eco => 'B73',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. f4 Qb6 10. e5

			variation => N__('Sicilian: Dragon, Classical, Zollner Gambit'),
			moves => {
			},
		},
		'r1b2rk1/pp2ppbp/1qnp1np1/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 1 10' => {
			eco => 'B73',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, 9. f4'),
			moves => {
				'e4e5' => 'r1b2rk1/pp2ppbp/1qnp1np1/4P3/3N1P2/2N1B3/PPP1B1PP/R2Q1RK1 b - - 0 10',
			},
		},
		'r1b2rk1/pp2ppbp/2n3p1/q1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11' => {
			eco => 'D87h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Qa5

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Qa5'),
			moves => {
			},
		},
		'r1b2rk1/pp2ppbp/2np1np1/2q5/2P1PP2/2NB1N2/PP2Q1PP/R1B1K2R w KQ - 2 10' => {
			eco => 'E76m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. dxc5 Qa5 8. Bd3 Qxc5 9. Qe2 Nc6

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7.dxc5 Qa5 8. Bd3 Qxc5 9. Qe2 Nc6'),
			moves => {
			},
		},
		'r1b2rk1/pp2ppbp/2np1np1/q7/3NP3/1BN1B2P/PPP2PP1/R2Q1RK1 b - - 0 10' => {
			eco => 'B35d',
                        # TRANSLATORS:

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5 8.O-O O-O 9. Bb3'),
			moves => {
				'c8d7' => 'r4rk1/pp1bppbp/2np1np1/q7/3NP3/1BN1B2P/PPP2PP1/R2Q1RK1 w - - 1 11',
			},
		},
		'r1b2rk1/pp2ppbp/2np1np1/q7/3NP3/1BN1B3/PPP2PPP/R2Q1RK1 w - - 0 10' => {
			eco => 'B35d',
                        # TRANSLATORS:

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5 8.O-O O-O 9. Bb3'),
			moves => {
				'h2h3' => 'r1b2rk1/pp2ppbp/2np1np1/q7/3NP3/1BN1B2P/PPP2PP1/R2Q1RK1 b - - 0 10',
			},
		},
		'r1b2rk1/pp2qppp/1np5/8/2BP4/2R2N2/PPQ2PPP/5RK1 w - - 1 15' => {
			eco => 'D68',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. Qc2 exd4 14. exd4 Nb6

			variation => N__('QGD: Orthodox, Classical, Vidmar, 13...exd4 14.exd4 Nb6'),
			moves => {
			},
		},
		'r1b2rk1/pp2qppp/2p5/4N3/2B5/2R1P3/PP3PPP/3Q1RK1 b - - 0 14' => {
			eco => 'D69',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Classical, 13.dxe5'),
			moves => {
				'e7e5' => 'r1b2rk1/pp3ppp/2p5/4q3/2B5/2R1P3/PP3PPP/3Q1RK1 w - - 0 15',
			},
		},
		'r1b2rk1/pp2qppp/2p5/4n3/2B5/2R1PN2/PP3PPP/3Q1RK1 w - - 0 14' => {
			eco => 'D69',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Classical, 13.dxe5'),
			moves => {
				'f3e5' => 'r1b2rk1/pp2qppp/2p5/4N3/2B5/2R1P3/PP3PPP/3Q1RK1 b - - 0 14',
			},
		},
		'r1b2rk1/pp2qppp/3pnn2/1Bp1p3/3PP3/2P2N2/P1P2PPP/R1BQR1K1 w - c6 0 12' => {
			eco => 'C49r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 Qe7 9. Re1 Nd8 10. d4 Ne6 11. Bc1 c5

			variation => N__('Four Knights: Symmetrical, Metger, 10. d4 Ne6 11. Bc1 c5'),
			moves => {
			},
		},
		'r1b2rk1/pp3ppp/2n1pn2/q1bp4/2P2B2/P1N1PN2/1PQ2PPP/2KR1B1R b - - 2 10' => {
			eco => 'D37v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. Qc2 Nc6 9. a3 Qa5 10. O-O-O

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. Qc2 Nc6 9. a3 Qa5 10.O-O-O'),
			moves => {
			},
		},
		'r1b2rk1/pp3ppp/2n1pn2/q1bp4/2P2B2/P1N1PN2/1PQ2PPP/3RKB1R b K - 2 10' => {
			eco => 'D37u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. Qc2 Nc6 9. a3 Qa5 10. Rd1

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. Qc2 Nc6 9. a3 Qa5 10. Rd1'),
			moves => {
			},
		},
		'r1b2rk1/pp3ppp/2n1pn2/q1bp4/2P2B2/P1N1PN2/1PQ2PPP/R3KB1R w KQ - 1 10' => {
			eco => 'D37t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. Qc2 Nc6 9. a3 Qa5

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. Qc2 Nc6 9. a3 Qa5'),
			moves => {
				'a1d1' => 'r1b2rk1/pp3ppp/2n1pn2/q1bp4/2P2B2/P1N1PN2/1PQ2PPP/3RKB1R b K - 2 10',
				'e1c1' => 'r1b2rk1/pp3ppp/2n1pn2/q1bp4/2P2B2/P1N1PN2/1PQ2PPP/2KR1B1R b - - 2 10',
			},
		},
		'r1b2rk1/pp3ppp/2n2q2/2b5/3p4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 12' => {
			eco => 'D34l',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4'),
			moves => {
				'c3d5' => 'r1b2rk1/pp3ppp/2n2q2/2bN4/3p4/5NP1/PP2PPBP/R2Q1RK1 b - - 1 12',
			},
		},
		'r1b2rk1/pp3ppp/2n2q2/2bN4/3p4/5NP1/PP2PPBP/R2Q1RK1 b - - 1 12' => {
			eco => 'D34m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Bg5 d4 11. Bxf6 Qxf6 12. Nd5

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4 11.Bxf6 Qxf6 12. Nd5'),
			moves => {
				'f6d8' => 'r1bq1rk1/pp3ppp/2n5/2bN4/3p4/5NP1/PP2PPBP/R2Q1RK1 w - - 2 13',
			},
		},
		'r1b2rk1/pp3ppp/2p2q2/5P2/2B5/2R1P3/PP4PP/3Q1RK1 b - - 0 16' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5 Nxe5 14. Nxe5 Qxe5 15. f4 Qf6 16. f5

			variation => N__('QGD: Orthodox, Classical, 13.dxe5: 15. f4 Qf6 16. f5'),
			moves => {
			},
		},
		'r1b2rk1/pp3ppp/2p2q2/8/2B1PP2/2R5/PP4PP/3Q1RK1 b - - 0 16' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5 Nxe5 14. Nxe5 Qxe5 15. f4 Qf6 16. e4

			variation => N__('QGD: Orthodox, Classical, 13.dxe5: 15. f4 Qf6 16. e4'),
			moves => {
			},
		},
		'r1b2rk1/pp3ppp/2p2q2/8/2B2P2/2R1P3/PP4PP/3Q1RK1 w - - 1 16' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5 Nxe5 14. Nxe5 Qxe5 15. f4 Qf6

			variation => N__('QGD: Orthodox, Classical, 13.dxe5: 15. f4 Qf6'),
			moves => {
				'e3e4' => 'r1b2rk1/pp3ppp/2p2q2/8/2B1PP2/2R5/PP4PP/3Q1RK1 b - - 0 16',
				'f4f5' => 'r1b2rk1/pp3ppp/2p2q2/5P2/2B5/2R1P3/PP4PP/3Q1RK1 b - - 0 16',
			},
		},
		'r1b2rk1/pp3ppp/2p5/4q3/2B2P2/2R1P3/PP4PP/3Q1RK1 b - f3 0 15' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5 Nxe5 14. Nxe5 Qxe5 15. f4

			variation => N__('QGD: Orthodox, Classical, 13.dxe5: 15. f4'),
			moves => {
				'e5e4' => 'r1b2rk1/pp3ppp/2p5/8/2B1qP2/2R1P3/PP4PP/3Q1RK1 w - - 1 16',
				'e5f6' => 'r1b2rk1/pp3ppp/2p2q2/8/2B2P2/2R1P3/PP4PP/3Q1RK1 w - - 1 16',
			},
		},
		'r1b2rk1/pp3ppp/2p5/4q3/2B5/2R1P3/PP3PPP/3Q1RK1 w - - 0 15' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5 Nxe5 14. Nxe5 Qxe5

			variation => N__('QGD: Orthodox, Classical, 13.dxe5'),
			moves => {
				'f2f4' => 'r1b2rk1/pp3ppp/2p5/4q3/2B2P2/2R1P3/PP4PP/3Q1RK1 b - f3 0 15',
			},
		},
		'r1b2rk1/pp3ppp/2p5/8/2B1qP2/2R1P3/PP4PP/3Q1RK1 w - - 1 16' => {
			eco => 'D69',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. Bxe7 Qxe7 11. O-O Nxc3 12. Rxc3 e5 13. dxe5 Nxe5 14. Nxe5 Qxe5 15. f4 Qe4

			variation => N__('QGD: Orthodox, Classical, 13.dxe5: 15. f4 Qe4'),
			moves => {
			},
		},
		'r1b2rk1/ppB1ppbp/2n3p1/3q4/3P4/4P3/PP2NPPP/R2QKB1R b KQ - 2 9' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 Nc6 9. Ne2

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Nc6 9. Ne2'),
			moves => {
				'c8g4' => 'r4rk1/ppB1ppbp/2n3p1/3q4/3P2b1/4P3/PP2NPPP/R2QKB1R w KQ - 3 10',
			},
		},
		'r1b2rk1/ppB1ppbp/2n3p1/3q4/3P4/4P3/PP3PPP/R2QKBNR w KQ - 1 9' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 Nc6

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Nc6'),
			moves => {
				'g1e2' => 'r1b2rk1/ppB1ppbp/2n3p1/3q4/3P4/4P3/PP2NPPP/R2QKB1R b KQ - 2 9',
			},
		},
		'r1b2rk1/ppB1ppbp/B5p1/3q4/3P4/4P3/PP3PPP/R2QK1NR b KQ - 0 9' => {
			eco => 'D84',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Na6'),
			moves => {
				'd5g2' => 'r1b2rk1/ppB1ppbp/B5p1/8/3P4/4P3/PP3PqP/R2QK1NR w KQ - 0 10',
			},
		},
		'r1b2rk1/ppB1ppbp/B5p1/8/3P4/4P3/PP3PqP/R2QK1NR w KQ - 0 10' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 Na6 9. Bxa6 Qxg2

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Na6 9.Bxa6 Qxg2'),
			moves => {
			},
		},
		'r1b2rk1/ppB1ppbp/n5p1/3q4/3P4/4P3/PP3PPP/R2QKBNR w KQ - 1 9' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 Na6

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Na6'),
			moves => {
				'f1a6' => 'r1b2rk1/ppB1ppbp/B5p1/3q4/3P4/4P3/PP3PPP/R2QK1NR b KQ - 0 9',
			},
		},
		'r1b2rk1/ppp1qppp/2np1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2Q1RK1 w - - 1 9' => {
			eco => 'C49n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 Qe7

			variation => N__('Four Knights: Symmetrical, Metger unpin'),
			moves => {
				'f1e1' => 'r1b2rk1/ppp1qppp/2np1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2QR1K1 b - - 2 9',
			},
		},
		'r1b2rk1/ppp1qppp/2np1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2QR1K1 b - - 2 9' => {
			eco => 'C49n',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Metger unpin'),
			moves => {
				'c6d8' => 'r1bn1rk1/ppp1qppp/3p1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2QR1K1 w - - 3 10',
			},
		},
		'r1b2rk1/ppp1qppp/2nppn2/8/2PP4/5NP1/PP1NPPBP/R2Q1RK1 w - - 2 9' => {
			eco => 'E11v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+ 7. Nbxd2 d6 8. O-O O-O

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line, 8...O-O'),
			moves => {
			},
		},
		'r1b2rk1/ppp1qppp/3pnn2/1B2p1B1/3PP3/2P2N2/P1P2PPP/R2QR1K1 w - - 1 11' => {
			eco => 'C49p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 Qe7 9. Re1 Nd8 10. d4 Ne6

			variation => N__('Four Knights: Symmetrical, Metger, 10. d4 Ne6'),
			moves => {
				'g5c1' => 'r1b2rk1/ppp1qppp/3pnn2/1B2p3/3PP3/2P2N2/P1P2PPP/R1BQR1K1 b - - 2 11',
			},
		},
		'r1b2rk1/ppp1qppp/3pnn2/1B2p3/3PP3/2P2N2/P1P2PPP/R1BQR1K1 b - - 2 11' => {
			eco => 'C49p',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Metger, 10. d4 Ne6'),
			moves => {
				'c7c5' => 'r1b2rk1/pp2qppp/3pnn2/1Bp1p3/3PP3/2P2N2/P1P2PPP/R1BQR1K1 w - c6 0 12',
				'f8d8' => 'r1br2k1/ppp1qppp/3pnn2/1B2p3/3PP3/2P2N2/P1P2PPP/R1BQR1K1 w - - 3 12',
			},
		},
		'r1b2rk1/ppq1bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/2KR3R w - - 8 10' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 Be7 8. Qe2 O-O 9. O-O-O Qc7

			variation => N__('Sicilian: Velimirovic, 9.O-O-O Qc7'),
			moves => {
				'c4b3' => 'r1b2rk1/ppq1bppp/2nppn2/8/3NP3/1BN1B3/PPP1QPPP/2KR3R b - - 9 10',
			},
		},
		'r1b2rk1/ppq1bppp/2nppn2/8/3NP3/1BN1B3/PPP1QPPP/2KR3R b - - 9 10' => {
			eco => 'B89',
                        # TRANSLATORS:

			variation => N__('Sicilian: Velimirovic, 9.O-O-O Qc7'),
			moves => {
				'a7a6' => 'r1b2rk1/1pq1bppp/p1nppn2/8/3NP3/1BN1B3/PPP1QPPP/2KR3R w - - 0 11',
			},
		},
		'r1b2rk1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/2RQ1RK1 b - - 5 11' => {
			eco => 'D87j',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Qc7'),
			moves => {
				'f8d8' => 'r1br2k1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/2RQ1RK1 w - - 6 12',
			},
		},
		'r1b2rk1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11' => {
			eco => 'D87j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Qc7

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Qc7'),
			moves => {
				'a1c1' => 'r1b2rk1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/2RQ1RK1 b - - 5 11',
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/1Bp5/3P4/P1P1PN2/5PPP/R1BQ1RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Bb5

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bb5'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/1B3PPP/R2Q1RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Bb2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bb2'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/2Q2PPP/R1B2RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Qc2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Qc2'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/4QPPP/R1B2RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Qe2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Qe2'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/5PPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7'),
			moves => {
				'a3a4' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/P1BP4/2P1PN2/5PPP/R1BQ1RK1 b - - 0 11',
				'c1b2' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/1B3PPP/R2Q1RK1 b - - 2 11',
				'c4a2' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/3P4/P1P1PN2/B4PPP/R1BQ1RK1 b - - 2 11',
				'c4b5' => 'r1b2rk1/ppq2ppp/2n1pn2/1Bp5/3P4/P1P1PN2/5PPP/R1BQ1RK1 b - - 2 11',
				'c4d3' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/3P4/P1PBPN2/5PPP/R1BQ1RK1 b - - 2 11',
				'c4e2' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/3P4/P1P1PN2/4BPPP/R1BQ1RK1 b - - 2 11',
				'd1c2' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/2Q2PPP/R1B2RK1 b - - 2 11',
				'd1e2' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/4QPPP/R1B2RK1 b - - 2 11',
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/3P4/P1P1PN2/4BPPP/R1BQ1RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Be2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Be2'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/3P4/P1P1PN2/B4PPP/R1BQ1RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Ba2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Ba2'),
			moves => {
				'e6e5' => 'r1b2rk1/ppq2ppp/2n2n2/2p1p3/3P4/P1P1PN2/B4PPP/R1BQ1RK1 w - - 0 12',
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/3P4/P1PBPN2/5PPP/R1BQ1RK1 b - - 2 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Bd3

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bd3'),
			moves => {
				'e6e5' => 'r1b2rk1/ppq2ppp/2n2n2/2p1p3/3P4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 12',
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2p5/P1BP4/2P1PN2/5PPP/R1BQ1RK1 b - - 0 11' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. a4

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. a4'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2pP4/3P4/P1PBPN2/5PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. cxd5

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10.cxd5'),
			moves => {
				'e6d5' => 'r1b2rk1/ppq2ppp/2n2n2/2pp4/3P4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN1P/5PP1/R1BQ1RK1 b - - 0 10' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. h3

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10. h3'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN2/1B3PPP/R2Q1RK1 b - - 2 10' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. Bb2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10. Bb2'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN2/2Q2PPP/R1B2RK1 b - - 2 10' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. Qc2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10. Qc2'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN2/5PPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7'),
			moves => {
				'c1b2' => 'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN2/1B3PPP/R2Q1RK1 b - - 2 10',
				'c4d5' => 'r1b2rk1/ppq2ppp/2n1pn2/2pP4/3P4/P1PBPN2/5PPP/R1BQ1RK1 b - - 0 10',
				'd1c2' => 'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN2/2Q2PPP/R1B2RK1 b - - 2 10',
				'h2h3' => 'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN1P/5PP1/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1b2rk1/ppq2ppp/2n2n2/2p1p3/3P4/P1P1PN2/B4PPP/R1BQ1RK1 w - - 0 12' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Ba2 e5

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Ba2 e5'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n2n2/2p1p3/3P4/P1PBPN2/2Q2PPP/R1B2RK1 b - - 1 12' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4 Qc7 11. Bd3 e5 12. Qc2

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bd3 e5 12. Qc2'),
			moves => {
				'f8e8' => 'r1b1r1k1/ppq2ppp/2n2n2/2p1p3/3P4/P1PBPN2/2Q2PPP/R1B2RK1 w - - 2 13',
			},
		},
		'r1b2rk1/ppq2ppp/2n2n2/2p1p3/3P4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 12' => {
			eco => 'E59',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4 Qc7 11. Bd3'),
			moves => {
				'd1c2' => 'r1b2rk1/ppq2ppp/2n2n2/2p1p3/3P4/P1PBPN2/2Q2PPP/R1B2RK1 b - - 1 12',
			},
		},
		'r1b2rk1/ppq2ppp/2n2n2/2pp4/3P3N/P1PBP3/5PPP/R1BQ1RK1 b - - 1 11' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. cxd5 exd5 11. Nh4

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10.cxd5 exd5 11. Nh4'),
			moves => {
			},
		},
		'r1b2rk1/ppq2ppp/2n2n2/2pp4/3P4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 11' => {
			eco => 'E58',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10.cxd5'),
			moves => {
				'a3a4' => 'r1b2rk1/ppq2ppp/2n2n2/2pp4/P2P4/2PBPN2/5PPP/R1BQ1RK1 b - - 0 11',
				'f3h4' => 'r1b2rk1/ppq2ppp/2n2n2/2pp4/3P3N/P1PBP3/5PPP/R1BQ1RK1 b - - 1 11',
			},
		},
		'r1b2rk1/ppq2ppp/2n2n2/2pp4/P2P4/2PBPN2/5PPP/R1BQ1RK1 b - - 0 11' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 Qc7 10. cxd5 exd5 11. a4

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 Qc7 10.cxd5 exd5 11. a4'),
			moves => {
				'f8e8' => 'r1b1r1k1/ppq2ppp/2n2n2/2pp4/P2P4/2PBPN2/5PPP/R1BQ1RK1 w - - 1 12',
			},
		},
		'r1b2rk1/ppqnbppp/2pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 4 9' => {
			eco => 'A55o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Qc2 Qc7

			variation => N__('Old Indian: Main Line, 8. Qc2 Qc7'),
			moves => {
			},
		},
		'r1b2rk1/ppqnbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 4 9' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8. O-O Qc7

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8.O-O Qc7'),
			moves => {
			},
		},
		'r1b3nr/ppppkBpp/1b6/nP2N1q1/4P3/8/P1PP1PPP/RNBQK2R w KQ - 1 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'd1h5' => 'r1b3nr/ppppkBpp/1b6/nP2N1qQ/4P3/8/P1PP1PPP/RNB1K2R b KQ - 2 8',
			},
		},
		'r1b3nr/ppppkBpp/1b6/nP2N1qQ/4P3/8/P1PP1PPP/RNB1K2R b KQ - 2 8' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. b5 Na5 6. Nxe5 Qg5 7. Bxf7+ Ke7 8. Qh5

			variation => N__('Evans Gambit Declined, Vasquez Variation'),
			moves => {
			},
		},
		'r1b3r1/ppp2k1p/1b6/nP2B1q1/3PP3/8/P1P2PPP/RN1QK2R w KQ - 1 12' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'b1d2' => 'r1b3r1/ppp2k1p/1b6/nP2B1q1/3PP3/8/P1PN1PPP/R2QK2R b KQ - 2 12',
			},
		},
		'r1b3r1/ppp2k1p/1b6/nP2B1q1/3PP3/8/P1PN1PPP/R2QK2R b KQ - 2 12' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. b5 Na5 6. Nxe5 Nh6 7. d4 d6 8. Bxh6 dxe5 9. Bxg7 Rg8 10. Bxf7+ Kxf7 11. Bxe5 Qg5 12. Nd2

			variation => N__('Evans Gambit Declined, Pavlov Variation'),
			moves => {
			},
		},
		'r1bQkb1r/p4ppp/2p2n2/1N6/5p2/2N5/PP2PPPP/R3KB1R b KQkq - 0 10' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'e8d8' => 'r1bk1b1r/p4ppp/2p2n2/1N6/5p2/2N5/PP2PPPP/R3KB1R w KQ - 0 11',
			},
		},
		'r1bQkb1r/ppp2ppp/2p5/4Pn2/8/5N2/PPP2PPP/RNB2RK1 b kq - 0 8' => {
			eco => 'C67r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap'),
			moves => {
				'e8d8' => 'r1bk1b1r/ppp2ppp/2p5/4Pn2/8/5N2/PPP2PPP/RNB2RK1 w - - 0 9',
			},
		},
		'r1bk1b1r/p4ppp/2p2n2/1N6/5p2/2N5/PP2PPPP/R3KB1R w KQ - 0 11' => {
			eco => 'A33j',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. Ndb5 d5 7. Bf4 e5 8. cxd5 exf4 9. dxc6 bxc6 10. Qxd8+ Kxd8

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5 Queenswap'),
			moves => {
			},
		},
		'r1bk1b1r/ppNpq2p/2nn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR w KQ - 1 10' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'c7a8' => 'N1bk1b1r/pp1pq2p/2nn2p1/3Qpp2/8/1B6/PPPP1PPP/R1B1K1NR b KQ - 0 10',
			},
		},
		'r1bk1b1r/ppp2pp1/2p4p/4Pn2/8/2N2N2/PPP2PPP/R1B2RK1 w - - 0 10' => {
			eco => 'C67t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+ Kxd8 9. Nc3 h6

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap, 9. Nc3 h6'),
			moves => {
			},
		},
		'r1bk1b1r/ppp2ppp/2p5/4Pn2/8/2N2N2/PPP2PPP/R1B2RK1 b - - 1 9' => {
			eco => 'C67s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5 8. Qxd8+ Kxd8 9. Nc3

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap, 9. Nc3'),
			moves => {
				'd8e8' => 'r1b1kb1r/ppp2ppp/2p5/4Pn2/8/2N2N2/PPP2PPP/R1B2RK1 w - - 2 10',
				'h7h6' => 'r1bk1b1r/ppp2pp1/2p4p/4Pn2/8/2N2N2/PPP2PPP/R1B2RK1 w - - 0 10',
			},
		},
		'r1bk1b1r/ppp2ppp/2p5/4Pn2/8/5N2/PPP2PPP/RNB2RK1 w - - 0 9' => {
			eco => 'C67r',
                        # TRANSLATORS:

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 Queenswap'),
			moves => {
				'b1c3' => 'r1bk1b1r/ppp2ppp/2p5/4Pn2/8/2N2N2/PPP2PPP/R1B2RK1 b - - 1 9',
			},
		},
		'r1bk2nr/ppNp1ppp/2n5/8/1b6/5B1q/PPPN1P1P/R1BQK2R w KQ - 1 10' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+'),
			moves => {
				'c7a8' => 'N1bk2nr/pp1p1ppp/2n5/8/1b6/5B1q/PPPN1P1P/R1BQK2R b KQ - 0 10',
			},
		},
		'r1bk2nr/pppp1Qpp/1b6/nP2q3/2B1P3/8/P1PP1PPP/RNB1K2R w KQ - 1 9' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Hirschbach Variation'),
			moves => {
				'c1b2' => 'r1bk2nr/pppp1Qpp/1b6/nP2q3/2B1P3/8/PBPP1PPP/RN2K2R b KQ - 2 9',
			},
		},
		'r1bk2nr/pppp1Qpp/1b6/nP2q3/2B1P3/8/PBPP1PPP/RN2K2R b KQ - 2 9' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. b5 Na5 6. Nxe5 Qg5 7. Qf3 Qxe5 8. Qxf7+ Kd8 9. Bb2

			variation => N__('Evans Gambit Declined, Hicken Variation'),
			moves => {
			},
		},
		'r1bk2nr/pppp1ppp/2n3q1/1N6/8/8/PPPNBPPP/R2Q1RK1 w - - 1 10' => {
			eco => 'C45g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Qh4 5. Nb5 Bb4+ 6. Bd2 Qxe4+ 7. Be2 Kd8 8. O-O Bxd2 9. Nxd2 Qg6

			variation => N__('Scotch: Steinitz, Rosenthal Variation'),
			moves => {
			},
		},
		'r1bk2nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPBBPPP/RN1Q1RK1 b - - 3 8' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'b4d2' => 'r1bk2nr/pppp1ppp/2n5/1N6/4q3/8/PPPbBPPP/RN1Q1RK1 w - - 0 9',
			},
		},
		'r1bk2nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPBBPPP/RN1QK2R w KQ - 2 8' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'e1g1' => 'r1bk2nr/pppp1ppp/2n5/1N6/1b2q3/8/PPPBBPPP/RN1Q1RK1 b - - 3 8',
			},
		},
		'r1bk2nr/pppp1ppp/2n5/1N6/4q3/8/PPPNBPPP/R2Q1RK1 b - - 0 9' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'e4g6' => 'r1bk2nr/pppp1ppp/2n3q1/1N6/8/8/PPPNBPPP/R2Q1RK1 w - - 1 10',
			},
		},
		'r1bk2nr/pppp1ppp/2n5/1N6/4q3/8/PPPbBPPP/RN1Q1RK1 w - - 0 9' => {
			eco => 'C45g',
                        # TRANSLATORS:

			variation => N__('Scotch: Steinitz, 5. Nb5 Bb4+ 6. Bd2'),
			moves => {
				'b1d2' => 'r1bk2nr/pppp1ppp/2n5/1N6/4q3/8/PPPNBPPP/R2Q1RK1 b - - 0 9',
			},
		},
		'r1bn1rk1/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 2 14' => {
			eco => 'C98h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. d5 Nd8 14. Nf1

			variation => N__('Spanish: Closed, Chigorin, 12... Nc6 13. d5 Nd8 14. Nf1'),
			moves => {
			},
		},
		'r1bn1rk1/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 1 14' => {
			eco => 'C98g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. d5 Nd8

			variation => N__('Spanish: Closed, Chigorin, 12... Nc6 13. d5 Nd8'),
			moves => {
				'a2a4' => 'r1bn1rk1/2q1bppp/p2p1n2/1ppPp3/P3P3/2P2N1P/1PBN1PP1/R1BQR1K1 b - a3 0 14',
				'd2f1' => 'r1bn1rk1/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 2 14',
			},
		},
		'r1bn1rk1/2q1bppp/p2p1n2/1ppPp3/P3P3/2P2N1P/1PBN1PP1/R1BQR1K1 b - a3 0 14' => {
			eco => 'C98i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. d5 Nd8 14. a4

			variation => N__('Spanish: Closed, Chigorin, 12... Nc6 13. d5 Nd8 14. a4'),
			moves => {
			},
		},
		'r1bn1rk1/ppp1qppp/3p1n2/1B2p1B1/3PP3/2P2N2/P1P2PPP/R2QR1K1 b - - 0 10' => {
			eco => 'C49n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 Qe7 9. Re1 Nd8 10. d4

			variation => N__('Four Knights: Symmetrical, Metger, 10. d4'),
			moves => {
				'c8g4' => 'r2n1rk1/ppp1qppp/3p1n2/1B2p1B1/3PP1b1/2P2N2/P1P2PPP/R2QR1K1 w - - 1 11',
				'd8e6' => 'r1b2rk1/ppp1qppp/3pnn2/1B2p1B1/3PP3/2P2N2/P1P2PPP/R2QR1K1 w - - 1 11',
			},
		},
		'r1bn1rk1/ppp1qppp/3p1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2QR1K1 w - - 3 10' => {
			eco => 'C49n',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Metger unpin'),
			moves => {
				'd3d4' => 'r1bn1rk1/ppp1qppp/3p1n2/1B2p1B1/3PP3/2P2N2/P1P2PPP/R2QR1K1 b - - 0 10',
			},
		},
		'r1bnk2r/ppppnppp/5q2/1Bb5/3NP3/2P1B3/PP3PPP/RN1QK2R w KQkq - 3 8' => {
			eco => 'C45w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3 Qf6 6. c3 Nge7 7. Bb5 Nd8

			variation => N__('Scotch: Paulsen, Gunsberg Defence'),
			moves => {
			},
		},
		'r1bq1b1r/pp4pp/2p1k3/3np3/1nB1Q3/2N5/PPPP1PPP/R1B1K2R w KQ - 0 10' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'a2a3' => 'r1bq1b1r/pp4pp/2p1k3/3np3/1nB1Q3/P1N5/1PPP1PPP/R1B1K2R b KQ - 0 10',
			},
		},
		'r1bq1b1r/pp4pp/2p1k3/3np3/1nB1Q3/P1N5/1PPP1PPP/R1B1K2R b KQ - 0 10' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'b4a6' => 'r1bq1b1r/pp4pp/n1p1k3/3np3/2B1Q3/P1N5/1PPP1PPP/R1B1K2R w KQ - 1 11',
			},
		},
		'r1bq1b1r/pp4pp/n1p1k3/3np3/2B1Q3/P1N5/1PPP1PPP/R1B1K2R w KQ - 1 11' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'd2d4' => 'r1bq1b1r/pp4pp/n1p1k3/3np3/2BPQ3/P1N5/1PP2PPP/R1B1K2R b KQ d3 0 11',
			},
		},
		'r1bq1b1r/pp4pp/n1p1k3/3np3/2BPQ3/P1N5/1PP2PPP/R1B1K2R b KQ d3 0 11' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'a6c7' => 'r1bq1b1r/ppn3pp/2p1k3/3np3/2BPQ3/P1N5/1PP2PPP/R1B1K2R w KQ - 1 12',
			},
		},
		'r1bq1b1r/ppn3pp/2p1k3/3np3/2BPQ3/P1N5/1PP2PPP/R1B1K2R w KQ - 1 12' => {
			eco => 'C57g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nxd5 6. Nxf7 Kxf7 7. Qf3+ Ke6 8. Nc3 Nb4 9. Qe4 c6 10. a3 Na6 11. d4 Nc7

			variation => N__('Two Knights: Fegatello, Leonhardt Variation'),
			moves => {
			},
		},
		'r1bq1b1r/ppp1n1pp/4k3/3np3/2B5/2N2Q2/PPPP1PPP/R1B1K2R w KQ - 4 9' => {
			eco => 'C57g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nxd5 6. Nxf7 Kxf7 7. Qf3+ Ke6 8. Nc3 Ne7

			variation => N__('Two Knights: Fegatello, Polerio Defence'),
			moves => {
			},
		},
		'r1bq1b1r/ppp2kpp/2n5/3np3/2B5/5Q2/PPPP1PPP/RNB1K2R b KQ - 1 7' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'f7e6' => 'r1bq1b1r/ppp3pp/2n1k3/3np3/2B5/5Q2/PPPP1PPP/RNB1K2R w KQ - 2 8',
			},
		},
		'r1bq1b1r/ppp2kpp/2n5/3np3/2B5/8/PPPP1PPP/RNBQK2R w KQ - 0 7' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'd1f3' => 'r1bq1b1r/ppp2kpp/2n5/3np3/2B5/5Q2/PPPP1PPP/RNB1K2R b KQ - 1 7',
			},
		},
		'r1bq1b1r/ppp3pp/2n1k3/3np3/2B5/2N2Q2/PPPP1PPP/R1B1K2R b KQ - 3 8' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'c6b4' => 'r1bq1b1r/ppp3pp/4k3/3np3/1nB5/2N2Q2/PPPP1PPP/R1B1K2R w KQ - 4 9',
				'c6e7' => 'r1bq1b1r/ppp1n1pp/4k3/3np3/2B5/2N2Q2/PPPP1PPP/R1B1K2R w KQ - 4 9',
			},
		},
		'r1bq1b1r/ppp3pp/2n1k3/3np3/2B5/5Q2/PPPP1PPP/RNB1K2R w KQ - 2 8' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'b1c3' => 'r1bq1b1r/ppp3pp/2n1k3/3np3/2B5/2N2Q2/PPPP1PPP/R1B1K2R b KQ - 3 8',
			},
		},
		'r1bq1b1r/ppp3pp/4k3/3np3/1nB1Q3/2N5/PPPP1PPP/R1B1K2R b KQ - 5 9' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'c7c6' => 'r1bq1b1r/pp4pp/2p1k3/3np3/1nB1Q3/2N5/PPPP1PPP/R1B1K2R w KQ - 0 10',
			},
		},
		'r1bq1b1r/ppp3pp/4k3/3np3/1nB5/2N2Q2/PPPP1PPP/R1B1K2R w KQ - 4 9' => {
			eco => 'C57g',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'f3e4' => 'r1bq1b1r/ppp3pp/4k3/3np3/1nB1Q3/2N5/PPPP1PPP/R1B1K2R b KQ - 5 9',
			},
		},
		'r1bq1k1r/2p2ppp/p4n2/1pbPR1N1/3n4/1B6/PPPP1PPP/RNBQ2K1 w - - 1 11' => {
			eco => 'C78f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 d6 7. Ng5 d5 8. exd5 Nd4 9. Re1 Bc5 10. Rxe5+ Kf8

			variation => N__('Spanish: Rabinovich Variation'),
			moves => {
			},
		},
		'r1bq1r1k/ppp1npbp/3p1np1/3Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E97g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Kh8

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Kh8'),
			moves => {
			},
		},
		'r1bq1r1k/pppnn1bp/3p2p1/3Ppp2/2P1P1P1/2N2P2/PP2B2P/R1BQNRK1 w - - 1 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. g4 Kh8

			variation => N__('King\'s Indian: Mar del Plata, Benko Attack, 11... Kh8'),
			moves => {
			},
		},
		'r1bq1r1k/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 w - - 2 12' => {
			eco => 'E98p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Nd3 f5 11. Bd2 Kh8

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Nd3 f5 11. Bd2 Kh8'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pb1/p2p1np1/3Pp2p/4P1P1/2N1BP1P/PP1QN3/R3KB1R b KQ - 0 12' => {
			eco => 'E89i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Nbd7 10. Qd2 a6 11. g4 h5 12. h3

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 10. Qd2 a6 11. g4 h5 12. h3'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pb1/p2p1np1/3Pp2p/4P1P1/2N1BP2/PP1QN2P/R3KB1R w KQ h6 0 12' => {
			eco => 'E89h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Nbd7 10. Qd2 a6 11. g4 h5

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 10. Qd2 a6 11. g4 h5'),
			moves => {
				'h2h3' => 'r1bq1rk1/1p1n1pb1/p2p1np1/3Pp2p/4P1P1/2N1BP1P/PP1QN3/R3KB1R b KQ - 0 12',
			},
		},
		'r1bq1rk1/1p1n1pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1Q2PP/R3KBNR w KQ - 1 12' => {
			eco => 'A65x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Nbd7

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/2p2np1/p3p3/2P1P3/2N2N2/PP3PPP/R1BQRBK1 w - - 0 11' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. dxe5 dxe5

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10.dxe5'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/2pp1np1/p3P3/2P1P3/2N2N2/PP3PPP/R1BQRBK1 b - - 0 10' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. dxe5

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10.dxe5'),
			moves => {
				'd6e5' => 'r1bq1rk1/1p1n1pbp/2p2np1/p3p3/2P1P3/2N2N2/PP3PPP/R1BQRBK1 w - - 0 11',
			},
		},
		'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N1P/PP3PP1/R1BQRBK1 b - - 0 10' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. h3

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10. h3'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N2/PP3PPP/1RBQRBK1 b - - 1 10' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. Rb1

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10. Rb1'),
			moves => {
				'f8e8' => 'r1bqr1k1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N2/PP3PPP/1RBQRBK1 w - - 2 11',
			},
		},
		'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 w - a6 0 10' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line'),
			moves => {
				'a1b1' => 'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N2/PP3PPP/1RBQRBK1 b - - 1 10',
				'd4e5' => 'r1bq1rk1/1p1n1pbp/2pp1np1/p3P3/2P1P3/2N2N2/PP3PPP/R1BQRBK1 b - - 0 10',
				'h2h3' => 'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N1P/PP3PP1/R1BQRBK1 b - - 0 10',
			},
		},
		'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - a6 0 10' => {
			eco => 'E69d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 a5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/p1pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/1K1R1B1R b - - 1 10' => {
			eco => 'E86i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qd2 Nbd7 9. O-O-O a6 10. Kb1

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qd2 Nbd7 9.O-O-O a6 10. Kb1'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/p1pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/2KR1B1R w - - 0 10' => {
			eco => 'E86h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qd2 Nbd7 9. O-O-O a6

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qd2 Nbd7 9.O-O-O a6'),
			moves => {
				'c1b1' => 'r1bq1rk1/1p1n1pbp/p1pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/1K1R1B1R b - - 1 10',
			},
		},
		'r1bq1rk1/1p1n1pbp/p1pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 0 10' => {
			eco => 'E69c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 a6

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP2B1/P3P3/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11' => {
			eco => 'A74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Nbd7 11. Bg5

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Nbd7 11. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 0 11' => {
			eco => 'A63',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6

			variation => N__('Benoni: Fianchetto, 9...Nbd7 10. Nd2 a6'),
			moves => {
				'a2a4' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P7/2N3P1/1P1NPPBP/R1BQ1RK1 b - a3 0 11',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N2N2/1P2BPPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'A74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Nbd7

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Nbd7'),
			moves => {
				'c1f4' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3PB2/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11',
				'c1g5' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP2B1/P3P3/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11',
				'f3d2' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N5/1P1NBPPP/R1BQ1RK1 b - - 2 11',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N5/1P1NBPPP/R1BQ1RK1 b - - 2 11' => {
			eco => 'A74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Nbd7 11. Nd2

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Nbd7 11. Nd2'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQ1RK1 b - - 2 11' => {
			eco => 'A70s',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6 10. a4 Nbd7'),
			moves => {
				'f8e8' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQ1RK1 w - - 3 12',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQK2R w KQ - 1 11' => {
			eco => 'A70s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 a6 10. a4 Nbd7

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6 10. a4 Nbd7'),
			moves => {
				'e1g1' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQ1RK1 b - - 2 11',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3PB2/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11' => {
			eco => 'A74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Nbd7 11. Bf4

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Nbd7 11. Bf4'),
			moves => {
				'd8e7' => 'r1b2rk1/1p1nqpbp/p2p1np1/2pP4/P3PB2/2N2N2/1P2BPPP/R2Q1RK1 w - - 3 12',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P7/2N2NP1/1P2PPBP/R1BQ1RK1 w - - 1 11' => {
			eco => 'A63',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O a6 10. a4 Nbd7

			variation => N__('Benoni: Fianchetto, 9... a6 10. a4 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P7/2N3P1/1P1NPPBP/R1BQ1RK1 b - a3 0 11' => {
			eco => 'A63',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4

			variation => N__('Benoni: Fianchetto, 9...Nbd7 10. Nd2 a6 11. a4'),
			moves => {
				'f8e8' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P7/2N3P1/1P1NPPBP/R1BQ1RK1 w - - 1 12',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/3Pp3/4P1P1/2N1BP2/PP1QN2P/R3KB1R b KQ g3 0 11' => {
			eco => 'E89g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Nbd7 10. Qd2 a6 11. g4

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 10. Qd2 a6 11. g4'),
			moves => {
				'h7h5' => 'r1bq1rk1/1p1n1pb1/p2p1np1/3Pp2p/4P1P1/2N1BP2/PP1QN2P/R3KB1R w KQ h6 0 12',
			},
		},
		'r1bq1rk1/1p1n1pbp/p2p1np1/3Pp3/4P3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 0 11' => {
			eco => 'E89f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Nbd7 10. Qd2 a6

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 10. Qd2 a6'),
			moves => {
				'g2g4' => 'r1bq1rk1/1p1n1pbp/p2p1np1/3Pp3/4P1P1/2N1BP2/PP1QN2P/R3KB1R b KQ g3 0 11',
			},
		},
		'r1bq1rk1/1p1n1ppp/p1pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - - 0 10' => {
			eco => 'D46k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 a6

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... a6'),
			moves => {
				'f1d1' => 'r1bq1rk1/1p1n1ppp/p1pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1BR2K1 b - - 1 10',
			},
		},
		'r1bq1rk1/1p1n1ppp/p1pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1BR2K1 b - - 1 10' => {
			eco => 'D46k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 a6 10. Rd1

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... a6 10. Rd1'),
			moves => {
				'b7b5' => 'r1bq1rk1/3n1ppp/p1pbpn2/1p6/2BP4/2N1PN2/PPQ2PPP/R1BR2K1 w - b6 0 11',
			},
		},
		'r1bq1rk1/1p1n1ppp/p3pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1B2RK1 w - - 0 10' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. Qe2 a6

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. Qe2 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbpp1/p1p1pn1p/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R w K - 0 10' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. a3 h6

			variation => N__('QGD: Orthodox, Rubinstein Attack, Grünfeld Variation, 9... h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/1p1nbpp1/p1p1pn1p/3p4/2PP3B/P1N1PN2/1PQ2PPP/2R1KB1R b K - 1 10',
			},
		},
		'r1bq1rk1/1p1nbpp1/p1p1pn1p/3p4/2PP3B/P1N1PN2/1PQ2PPP/2R1KB1R b K - 1 10' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. a3 h6 10. Bh4

			variation => N__('QGD: Orthodox, Rubinstein Attack, Grünfeld Variation, 9... h6 10. Bh4'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbpp1/p1p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/2R1K2R w K - 0 11' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5 10. Bd3 h6

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/1p1nbpp1/p1p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/2R1K2R b K - 1 11',
			},
		},
		'r1bq1rk1/1p1nbpp1/p1p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/2R1K2R b K - 1 11' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5 10. Bd3 h6 11. Bh4

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 h6 11. Bh4'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/2pp1n2/p3p3/P1BPP3/2N2N2/1PP2PPP/R1BQR1K1 w - a6 0 9' => {
			eco => 'C41y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. Re1 c6 8. a4 a5

			variation => N__('Philidor: Improved Hanham, 7. Re1 c6 8. a4 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1p1p3/3n2B1/3P4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 10' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 Nxd5

			variation => N__('QGD: Orthodox, Rubinstein Attack, 9.cxd5 Nxd5'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1p1pn2/2Pp2B1/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. c5 c6

			variation => N__('QGD: Orthodox, Swiss, 8. c5 c6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1p1pn2/3P2B1/3P4/2N1PN2/PPQ2PPP/2R1KB1R b K - 0 9' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5

			variation => N__('QGD: Orthodox, Rubinstein Attack, 9.cxd5'),
			moves => {
				'e6d5' => 'r1bq1rk1/1p1nbppp/p1p2n2/3p2B1/3P4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 10',
				'f6d5' => 'r1bq1rk1/1p1nbppp/p1p1p3/3n2B1/3P4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 10',
			},
		},
		'r1bq1rk1/1p1nbppp/p1p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6

			variation => N__('QGD: Orthodox, Rubinstein Attack, Carlsbad Variation'),
			moves => {
				'a2a3' => 'r1bq1rk1/1p1nbppp/p1p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R b K - 0 9',
				'c4d5' => 'r1bq1rk1/1p1nbppp/p1p1pn2/3P2B1/3P4/2N1PN2/PPQ2PPP/2R1KB1R b K - 0 9',
			},
		},
		'r1bq1rk1/1p1nbppp/p1p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 0 9' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 a6

			variation => N__('QGD: Orthodox, Main Line, 8... a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R b K - 0 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. a3

			variation => N__('QGD: Orthodox, Rubinstein Attack, Grünfeld Variation'),
			moves => {
				'h7h6' => 'r1bq1rk1/1p1nbpp1/p1p1pn1p/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R w K - 0 10',
			},
		},
		'r1bq1rk1/1p1nbppp/p1p2n2/3p2B1/3P4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 10' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5

			variation => N__('QGD: Orthodox, Rubinstein Attack, 9.cxd5 exd5'),
			moves => {
				'f1d3' => 'r1bq1rk1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R1K2R b K - 1 10',
			},
		},
		'r1bq1rk1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R w K - 0 10' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. cxd5 exd5 9. Bd3 c6

			variation => N__('QGD: Orthodox, Swiss, Karlsbad, 9. Bd3 c6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R1K2R b K - 1 10' => {
			eco => 'D65',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Rubinstein Attack, 9.cxd5 exd5'),
			moves => {
				'f8e8' => 'r1bqr1k1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R1K2R w K - 2 11',
				'h7h6' => 'r1bq1rk1/1p1nbpp1/p1p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/2R1K2R w K - 0 11',
			},
		},
		'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 9' => {
			eco => 'A55n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Be3 a6

			variation => N__('Old Indian: Main Line, 8. Be3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9' => {
			eco => 'A55s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Re1 a6

			variation => N__('Old Indian: Main Line, 8. Re1 a6'),
			moves => {
				'e2f1' => 'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 1 9',
			},
		},
		'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 1 9' => {
			eco => 'A55t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Re1 a6 9. Bf1

			variation => N__('Old Indian: Main Line, 8. Re1 a6 9. Bf1'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 0 9' => {
			eco => 'A55p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Qc2 a6

			variation => N__('Old Indian: Main Line, 8. Qc2 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A55g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8. O-O a6

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8.O-O a6'),
			moves => {
				'a2a4' => 'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/P1PPP3/2N2NP1/1P3PBP/R1BQ1RK1 b - a3 0 9',
			},
		},
		'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/P1PPP3/2N2NP1/1P3PBP/R1BQ1RK1 b - a3 0 9' => {
			eco => 'A55h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8. O-O a6 9. a4

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8.O-O a6 9. a4'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p2p1n2/4p3/P3PP2/2NB1N2/1PP3PP/R1BQ1RK1 b - - 4 10' => {
			eco => 'B93s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3'),
			moves => {
				'd7c5' => 'r1bq1rk1/1p2bppp/p2p1n2/2n1p3/P3PP2/2NB1N2/1PP3PP/R1BQ1RK1 w - - 5 11',
				'e5f4' => 'r1bq1rk1/1p1nbppp/p2p1n2/8/P3Pp2/2NB1N2/1PP3PP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/1p1nbppp/p2p1n2/4p3/P3PP2/2NB1N2/1PP3PP/R1BQK2R w KQ - 3 10' => {
			eco => 'B93s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3'),
			moves => {
				'e1g1' => 'r1bq1rk1/1p1nbppp/p2p1n2/4p3/P3PP2/2NB1N2/1PP3PP/R1BQ1RK1 b - - 4 10',
			},
		},
		'r1bq1rk1/1p1nbppp/p2p1n2/8/P3Pp2/2NB1N2/1PP3PP/R1BQ1RK1 w - - 0 11' => {
			eco => 'B93u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3 O-O 10. O-O exf4

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3 O-O 10.O-O exf4'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nbppp/p3pn2/2p3B1/2BP4/2N1PN2/PP3PPP/R2Q1RK1 w - - 0 10' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 dxc4 8. Bxc4 c5 9. O-O a6

			variation => N__('QGD: Orthodox, Botvinnik, 8... c5 9.O-O a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nppbp/2np2p1/p7/2P1P3/2N5/PPNBBPPP/R2Q1RK1 w - a6 0 11' => {
			eco => 'B37o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O Nd7 10. Bd2 a5

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O Nd7 10. Bd2 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1p1nppbp/p2p1np1/2p5/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 0 9' => {
			eco => 'E81t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nbd7 7. Qd2 c5 8. Nge2 a6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 Nbd7 7. Qd2 c5 8.Nge2 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1pppbp/2n2np1/p7/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ a6 0 9' => {
			eco => 'B35l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 a5

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 a5'),
			moves => {
				'a2a4' => 'r1bq1rk1/1p1pppbp/2n2np1/p7/P2NP3/1BN1B3/1PP2PPP/R2QK2R b KQ a3 0 9',
				'f2f3' => 'r1bq1rk1/1p1pppbp/2n2np1/p7/3NP3/1BN1BP2/PPP3PP/R2QK2R b KQ - 0 9',
			},
		},
		'r1bq1rk1/1p1pppbp/2n2np1/p7/3NP3/1BN1BP2/PPP3PP/R2QK2R b KQ - 0 9' => {
			eco => 'B35n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 a5 9. f3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 a5 9. f3'),
			moves => {
			},
		},
		'r1bq1rk1/1p1pppbp/2n2np1/p7/P2NP3/1BN1B3/1PP2PPP/R2QK2R b KQ a3 0 9' => {
			eco => 'B35m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 a5 9. a4

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 a5 9. a4'),
			moves => {
			},
		},
		'r1bq1rk1/1p1pppbp/p1n2np1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A38l',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 a6

			variation => N__('English: Symmetrical, Main Line, 7. d3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p1pppbp/p1n2np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A39f',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 a6

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bpp1/p1np1n1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQR1K1 w - - 0 11' => {
			eco => 'B58o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 a6

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/2np1n2/p3p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - a6 0 10' => {
			eco => 'B59n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 a5

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Be3 a5'),
			moves => {
				'a2a4' => 'r1bq1rk1/1p2bppp/2np1n2/p3p3/P3P3/1NN1B3/1PP1BPPP/R2Q1RK1 b - a3 0 10',
			},
		},
		'r1bq1rk1/1p2bppp/2np1n2/p3p3/P3P3/1NN1B3/1PP1BPPP/R2Q1RK1 b - a3 0 10' => {
			eco => 'B59o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 a5 10. a4

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Be3 a5 10. a4'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/2nppn2/p5B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w - a6 0 10' => {
			eco => 'B63r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3 a5

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. Nb3 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/p1n1p3/3n4/P1BP4/2N2N2/1P2QPPP/R1BR2K1 w - - 3 12' => {
			eco => 'D27w',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Qe2 cxd4 9. Rd1 Be7 10. exd4 O-O 11. Nc3 Nd5

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4, 11. Nc3 Nd5'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/p1n1pn2/8/2BP4/P1N2N2/1P3PPP/R1BQR1K1 w - - 0 12' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7 11. Re1 a6

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7 11. Re1 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/p1n1pn2/8/P1BP4/2N2N2/1P2QPPP/R1BR2K1 b - - 2 11' => {
			eco => 'D27v',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Qe2 cxd4 9. Rd1 Be7 10. exd4 O-O 11. Nc3

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4, 11. Nc3'),
			moves => {
				'f6d5' => 'r1bq1rk1/1p2bppp/p1n1p3/3n4/P1BP4/2N2N2/1P2QPPP/R1BR2K1 w - - 3 12',
			},
		},
		'r1bq1rk1/1p2bppp/p1n1pn2/8/P1BP4/5N2/1P2QPPP/RNBR2K1 w - - 1 11' => {
			eco => 'D27v',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4'),
			moves => {
				'b1c3' => 'r1bq1rk1/1p2bppp/p1n1pn2/8/P1BP4/2N2N2/1P2QPPP/R1BR2K1 b - - 2 11',
			},
		},
		'r1bq1rk1/1p2bppp/p1np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K w - - 6 10' => {
			eco => 'B92q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Kh1 Nc6

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Kh1 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w - - 0 10' => {
			eco => 'B63s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3 a6

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. Nb3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP2BPPP/R1BQ1RK1 b - - 5 10' => {
			eco => 'B44v',
                        # TRANSLATORS:

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3 Be7 9. Be2 O-O'),
			moves => {
				'b7b6' => 'r1bq1rk1/4bppp/ppnppn2/8/2P1P3/N1N5/PP2BPPP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP2BPPP/R1BQK2R w KQ - 4 10' => {
			eco => 'B44v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3 Be7 9. Be2 O-O'),
			moves => {
				'e1g1' => 'r1bq1rk1/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP2BPPP/R1BQ1RK1 b - - 5 10',
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K b - - 4 10' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. Kh1

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. Kh1'),
			moves => {
				'c8d7' => 'r2q1rk1/1p1bbppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K w - - 5 11',
				'd8c7' => 'r1b2rk1/1pq1bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K w - - 5 11',
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 3 10' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6'),
			moves => {
				'a2a4' => 'r1bq1rk1/1p2bppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1RK1 b - a3 0 10',
				'd1e1' => 'r1bq1rk1/1p2bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R3QRK1 b - - 4 10',
				'g1h1' => 'r1bq1rk1/1p2bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K b - - 4 10',
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R3QRK1 b - - 4 10' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. Qe1

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. Qe1'),
			moves => {
				'd8c7' => 'r1b2rk1/1pq1bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R3QRK1 w - - 5 11',
			},
		},
		'r1bq1rk1/1p2bppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1RK1 b - a3 0 10' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. a4

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. a4'),
			moves => {
				'c8d7' => 'r2q1rk1/1p1bbppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1RK1 w - - 1 11',
				'd8c7' => 'r1b2rk1/1pq1bppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/1p2bppp/p2p1n2/2n1p3/P3PP2/2NB1N2/1PP3PP/R1BQ1RK1 w - - 5 11' => {
			eco => 'B93t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3 O-O 10. O-O Nc5

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3 O-O 10.O-O Nc5'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/2np1np1/p7/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ a6 0 10' => {
			eco => 'B77b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 a5

			variation => N__('Sicilian: Dragon, Yugoslav, Byrne Variation'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/2np1np1/p7/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - a6 0 10' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 a5

			variation => N__('Sicilian: Dragon, Classical, Alekhine Variation'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/p1np1np1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A36z',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 Nf6 7. O-O O-O 8. d3 a6

			variation => N__('English: Symmetrical, Botvinnik System, 5... d6, 8. d3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/p1np1np1/2p5/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A38r',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 d6 8. a3 a6

			variation => N__('English: Symmetrical, Main Line, 7. d3 d6 8. a3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/p1np1np1/6B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'B70u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Bg5 Nc6 9. Nb3 a6

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Bg5 Nc6 9. Nb3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/p1np1np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'B38p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O a6

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/p1np1np1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 a6

			variation => N__('Sicilian: Dragon, Classical, 9. Nb3 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP1BPPBP/R4RK1 b - - 1 11' => {
			eco => 'A39q',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3 a6 11. Bd2

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3 a6 11. Bd2'),
			moves => {
				'a8b8' => '1rbq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP1BPPBP/R4RK1 w - - 2 12',
			},
		},
		'r1bq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP2PPBP/R1B2RK1 w - - 0 11' => {
			eco => 'A39p',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3 a6

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3 a6'),
			moves => {
				'c1d2' => 'r1bq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP1BPPBP/R4RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/1RBQ1RK1 b - - 1 10' => {
			eco => 'E66f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Rb1

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... a6 10. Rb1'),
			moves => {
				'a8b8' => '1rbq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/1RBQ1RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E66e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... a6'),
			moves => {
				'a1b1' => 'r1bq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/1RBQ1RK1 b - - 1 10',
				'd1c2' => 'r1bq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PPQNPPBP/R1B2RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PPQNPPBP/R1B2RK1 b - - 1 10' => {
			eco => 'E66h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 a6 10. Qc2

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... a6 10. Qc2'),
			moves => {
				'a8b8' => '1rbq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PPQNPPBP/R1B2RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/1pp1b1pp/p1p2p2/2n1p3/2N1P2N/3P4/PPP2PPP/R1BQ1RK1 w - - 2 11' => {
			eco => 'C85q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7 8. Nbd2 O-O 9. Nc4 f6 10. Nh4 Nc5

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2 O-O 9. Nc4 f6 10. Nh4 Nc5'),
			moves => {
				'h4f5' => 'r1bq1rk1/1pp1b1pp/p1p2p2/2n1pN2/2N1P3/3P4/PPP2PPP/R1BQ1RK1 b - - 3 11',
			},
		},
		'r1bq1rk1/1pp1b1pp/p1p2p2/2n1pN2/2N1P3/3P4/PPP2PPP/R1BQ1RK1 b - - 3 11' => {
			eco => 'C85r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7 8. Nbd2 O-O 9. Nc4 f6 10. Nh4 Nc5 11. Nf5

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2 O-O 9. Nc4 f6 10. Nh4 Nc5 11. Nf5'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1bppp/p1np1n2/4p3/B2PP3/2P2N2/PP3PPP/RNBQR1K1 b - d3 0 8' => {
			eco => 'C87j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O 8. d4

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O 8. d4'),
			moves => {
				'c8d7' => 'r2q1rk1/1ppbbppp/p1np1n2/4p3/B2PP3/2P2N2/PP3PPP/RNBQR1K1 w - - 1 9',
			},
		},
		'r1bq1rk1/1pp1bppp/p1np1n2/4p3/B3P3/2P2N1P/PP1P1PP1/RNBQR1K1 b - - 0 8' => {
			eco => 'C87n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O 8. h3

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O 8. h3'),
			moves => {
				'c8d7' => 'r2q1rk1/1ppbbppp/p1np1n2/4p3/B3P3/2P2N1P/PP1P1PP1/RNBQR1K1 w - - 1 9',
			},
		},
		'r1bq1rk1/1pp1bppp/p1np1n2/4p3/B3P3/2P2N2/PP1P1PPP/RNBQR1K1 w - - 1 8' => {
			eco => 'C87i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O'),
			moves => {
				'd2d4' => 'r1bq1rk1/1pp1bppp/p1np1n2/4p3/B2PP3/2P2N2/PP3PPP/RNBQR1K1 b - d3 0 8',
				'h2h3' => 'r1bq1rk1/1pp1bppp/p1np1n2/4p3/B3P3/2P2N1P/PP1P1PP1/RNBQR1K1 b - - 0 8',
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/3Pp3/1BP1P3/2N2N2/P3BPPP/R2Q1RK1 b - - 0 11' => {
			eco => 'E97k',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... a5 10. Ba3 axb4'),
			moves => {
				'f6d7' => 'r1bq1rk1/1ppnnpbp/3p2p1/3Pp3/1BP1P3/2N2N2/P3BPPP/R2Q1RK1 w - - 1 12',
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/3Pp3/1pP1P3/B1N2N2/P3BPPP/R2Q1RK1 w - - 0 11' => {
			eco => 'E97k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 a5 10. Ba3 axb4

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... a5 10. Ba3 axb4'),
			moves => {
				'a3b4' => 'r1bq1rk1/1pp1npbp/3p1np1/3Pp3/1BP1P3/2N2N2/P3BPPP/R2Q1RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/P2Pp3/2P1P3/2N2N2/P3BPPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E97i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 a5 10. bxa5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... a5 10.bxa5'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - a6 0 10' => {
			eco => 'E97i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 a5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... a5'),
			moves => {
				'b4a5' => 'r1bq1rk1/1pp1npbp/3p1np1/P2Pp3/2P1P3/2N2N2/P3BPPP/R1BQ1RK1 b - - 0 10',
				'c1a3' => 'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/1PP1P3/B1N2N2/P3BPPP/R2Q1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/1PP1P3/B1N2N2/P3BPPP/R2Q1RK1 b - - 1 10' => {
			eco => 'E97j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 a5 10. Ba3

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... a5 10. Ba3'),
			moves => {
				'a5b4' => 'r1bq1rk1/1pp1npbp/3p1np1/3Pp3/1pP1P3/B1N2N2/P3BPPP/R2Q1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/2P1P3/2N5/PP1NBPPP/1RBQ1RK1 b - - 1 10' => {
			eco => 'E97x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5 10. Rb1

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. Rb1'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - a6 0 10' => {
			eco => 'E97x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5'),
			moves => {
				'a1b1' => 'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/2P1P3/2N5/PP1NBPPP/1RBQ1RK1 b - - 1 10',
				'a2a3' => 'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/2P1P3/P1N5/1P1NBPPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/2P1P3/P1N5/1P1NBPPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E97y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5 10. a3

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. a3'),
			moves => {
				'c8d7' => 'r2q1rk1/1ppbnpbp/3p1np1/p2Pp3/2P1P3/P1N5/1P1NBPPP/R1BQ1RK1 w - - 1 11',
				'f6d7' => 'r1bq1rk1/1ppnnpbp/3p2p1/p2Pp3/2P1P3/P1N5/1P1NBPPP/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/1pp1npbp/3p2p1/P2Pp3/2P1Pn2/2N2N2/P4PPP/R1BQRBK1 b - - 0 12' => {
			eco => 'E97q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 Nf4 11. Bf1 a5 12. bxa5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 Nf4 11. Bf1 a5 12.bxa5'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1npbp/3p2p1/p2Pp2n/1PP1P3/2N2N2/P3BPPP/R1BQR1K1 w - a6 0 11' => {
			eco => 'E97p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 a5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1npbp/3p2p1/p2Pp3/1PP1Pn2/2N2N2/P4PPP/R1BQRBK1 w - a6 0 12' => {
			eco => 'E97q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 Nf4 11. Bf1 a5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 Nf4 11. Bf1 a5'),
			moves => {
				'b4a5' => 'r1bq1rk1/1pp1npbp/3p2p1/P2Pp3/2P1Pn2/2N2N2/P4PPP/R1BQRBK1 b - - 0 12',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/3P4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E63l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. d5

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. d5'),
			moves => {
				'c6a5' => 'r1bq1rk1/1pp1ppbp/p2p1np1/n2P4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E63d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. b3

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. b3'),
			moves => {
				'a8b8' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E63a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6

			variation => N__('King\'s Indian: Fianchetto, Panno Variation'),
			moves => {
				'b2b3' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'd4d5' => 'r1bq1rk1/1pp1ppbp/p1np1np1/3P4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'e2e4' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8',
				'f1e1' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 1 8',
				'h2h3' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 1 8' => {
			eco => 'E63b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. Re1

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. Re1'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8' => {
			eco => 'E63h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. h3

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. h3'),
			moves => {
				'a8b8' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1Q2PP/2KR1BNR b - - 1 8' => {
			eco => 'E83c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Qd2 a6 8. O-O-O

			variation => N__('King\'s Indian: Sämisch, 6... Nc6 7. Qd2 a6 8.O-O-O'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 8' => {
			eco => 'E83c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Qd2 a6

			variation => N__('King\'s Indian: Sämisch, 6... Nc6 7. Qd2 a6'),
			moves => {
				'e1c1' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1Q2PP/2KR1BNR b - - 1 8',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 1 8' => {
			eco => 'E83i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2

			variation => N__('King\'s Indian: Sämisch, Panno, 8. Qd2'),
			moves => {
				'a8b8' => '1rbq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 2 9',
				'f8e8' => 'r1bqr1k1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 2 9',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 8' => {
			eco => 'E83f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6

			variation => N__('King\'s Indian: Sämisch, Panno'),
			moves => {
				'a2a3' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/P1N1BP2/1P2N1PP/R2QKB1R b KQ - 0 8',
				'd1d2' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 1 8',
				'e2c1' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP4PP/R1NQKB1R b KQ - 1 8',
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP4PP/R1NQKB1R b KQ - 1 8' => {
			eco => 'E83h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Nc1

			variation => N__('King\'s Indian: Sämisch, Panno, 8. Nc1'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8' => {
			eco => 'E63c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. e4

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. e4'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/P1N1BP2/1P2N1PP/R2QKB1R b KQ - 0 8' => {
			eco => 'E83g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. a3

			variation => N__('King\'s Indian: Sämisch, Panno, 8. a3'),
			moves => {
			},
		},
		'r1bq1rk1/1pp1ppbp/p2p1np1/n2P4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E63m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O a6 8. d5 Na5

			variation => N__('King\'s Indian: Fianchetto, Panno, 8. d5 Na5'),
			moves => {
			},
		},
		'r1bq1rk1/1pp2pb1/2np1npp/p3p3/2P5/P1NP1NP1/1P2PPBP/1RBQ1RK1 w - - 0 10' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6 7. O-O O-O 8. Rb1 a5 9. a3 h6

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6 7.O-O O-O 8. Rb1 a5 9. a3 h6'),
			moves => {
			},
		},
		'r1bq1rk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 2 10' => {
			eco => 'E92t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6 9. Bh4 Na6

			variation => N__('King\'s Indian: Petrosian, Stein, 8. Bg5 h6 9. Bh4 Na6'),
			moves => {
				'f3d2' => 'r1bq1rk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2QK2R b KQ - 3 10',
			},
		},
		'r1bq1rk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2QK2R b KQ - 3 10' => {
			eco => 'E92u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6 9. Bh4 Na6 10. Nd2

			variation => N__('King\'s Indian: Petrosian, Stein, Main Line'),
			moves => {
				'd8e8' => 'r1b1qrk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2QK2R w KQ - 4 11',
			},
		},
		'r1bq1rk1/1pp2pbp/2np1np1/p3p3/2P5/2NP1NP1/PP2PPBP/1RBQ1RK1 w - a6 0 9' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6 7. O-O O-O 8. Rb1 a5

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6 7.O-O O-O 8. Rb1 a5'),
			moves => {
				'a2a3' => 'r1bq1rk1/1pp2pbp/2np1np1/p3p3/2P5/P1NP1NP1/1P2PPBP/1RBQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/1pp2pbp/2np1np1/p3p3/2P5/P1NP1NP1/1P2PPBP/1RBQ1RK1 b - - 0 9' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6 7. O-O O-O 8. Rb1 a5 9. a3

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6 7.O-O O-O 8. Rb1 a5 9. a3'),
			moves => {
				'h7h6' => 'r1bq1rk1/1pp2pb1/2np1npp/p3p3/2P5/P1NP1NP1/1P2PPBP/1RBQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/1pp2pbp/3p1np1/p1nPp3/2P1P3/2N2N2/PPQ1BPPP/R1B2RK1 w - a6 0 10' => {
			eco => 'E94r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. d5 Nc5 9. Qc2 a5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. d5 Nc5 9. Qc2 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1pp2ppp/p1n1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6 7. O-O a6

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6 7.O-O a6'),
			moves => {
			},
		},
		'r1bq1rk1/1ppn1pb1/3p1npp/p2Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ a6 0 10' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5 h6 9. Bh4 a5

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6 9. Bh4 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1ppn1pb1/p2p1npp/3Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5 h6 9. Bh4 a6

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6 9. Bh4 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1ppn1pbp/3p1np1/p3p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - a6 0 9' => {
			eco => 'E95b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 a5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 a5'),
			moves => {
			},
		},
		'r1bq1rk1/1ppn1pbp/p2p1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E68i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 a6

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnb1pp/p1p2p2/4p3/2N1P2N/3P4/PPP2PPP/R1BQ1RK1 b - - 1 10' => {
			eco => 'C85p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7 8. Nbd2 O-O 9. Nc4 f6 10. Nh4

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2 O-O 9. Nc4 f6 10. Nh4'),
			moves => {
				'd7c5' => 'r1bq1rk1/1pp1b1pp/p1p2p2/2n1p3/2N1P2N/3P4/PPP2PPP/R1BQ1RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/1ppnb1pp/p1p2p2/4p3/2N1P3/3P1N2/PPP2PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'C85o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7 8. Nbd2 O-O 9. Nc4 f6

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2 O-O 9. Nc4 f6'),
			moves => {
				'f3h4' => 'r1bq1rk1/1ppnb1pp/p1p2p2/4p3/2N1P2N/3P4/PPP2PPP/R1BQ1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/1ppnbppp/p1p5/4p3/2N1P3/3P1N2/PPP2PPP/R1BQ1RK1 b - - 4 9' => {
			eco => 'C85n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7 8. Nbd2 O-O 9. Nc4

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2 O-O 9. Nc4'),
			moves => {
				'f7f6' => 'r1bq1rk1/1ppnb1pp/p1p2p2/4p3/2N1P3/3P1N2/PPP2PPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/1ppnbppp/p1p5/4p3/4P3/3P1N2/PPPN1PPP/R1BQ1RK1 w - - 3 9' => {
			eco => 'C85m',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2'),
			moves => {
				'd2c4' => 'r1bq1rk1/1ppnbppp/p1p5/4p3/2N1P3/3P1N2/PPP2PPP/R1BQ1RK1 b - - 4 9',
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/2Pp2B1/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. c5

			variation => N__('QGD: Orthodox, Swiss, 8. c5'),
			moves => {
				'c7c6' => 'r1bq1rk1/1p1nbppp/p1p1pn2/2Pp2B1/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9',
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/3P2B1/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. cxd5

			variation => N__('QGD: Orthodox, Swiss, Karlsbad Variation'),
			moves => {
				'e6d5' => 'r1bq1rk1/1ppnbppp/p4n2/3p2B1/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9',
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6

			variation => N__('QGD: Orthodox, Swiss Variation'),
			moves => {
				'a2a3' => 'r1bq1rk1/1ppnbppp/p3pn2/3p2B1/2PP4/P1N1PN2/1P3PPP/2RQKB1R b K - 0 8',
				'c4c5' => 'r1bq1rk1/1ppnbppp/p3pn2/2Pp2B1/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 8',
				'c4d5' => 'r1bq1rk1/1ppnbppp/p3pn2/3P2B1/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 8',
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 a6

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/3p2B1/2PP4/P1N1PN2/1P3PPP/2RQKB1R b K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. a3

			variation => N__('QGD: Orthodox, Swiss, 8. a3'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/6B1/2BP4/2N1PN2/PP3PPP/2RQK2R w K - 0 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 dxc4 8. Bxc4 a6

			variation => N__('QGD: Orthodox, 7. Rc1 dxc4 8.Bxc4 a6'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnbppp/p3pn2/6B1/2BP4/2N1PN2/PP3PPP/R2QK2R w KQ - 0 9' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 dxc4 8. Bxc4 a6

			variation => N__('QGD: Orthodox, Botvinnik, 8... a6'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnbppp/p4n2/3p2B1/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. cxd5 exd5

			variation => N__('QGD: Orthodox, Swiss, Karlsbad Variation'),
			moves => {
				'f1d3' => 'r1bq1rk1/1ppnbppp/p4n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R b K - 1 9',
			},
		},
		'r1bq1rk1/1ppnbppp/p4n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R b K - 1 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. cxd5 exd5 9. Bd3

			variation => N__('QGD: Orthodox, Swiss, Karlsbad, 9. Bd3'),
			moves => {
				'c7c6' => 'r1bq1rk1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R w K - 0 10',
				'f8e8' => 'r1bqr1k1/1ppnbppp/p4n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R w K - 2 10',
			},
		},
		'r1bq1rk1/1ppnn1bp/3p2p1/p2Ppp2/1PP1P3/P1N5/3NBPPP/1RBQ1RK1 b - b3 0 12' => {
			eco => 'E97z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5 10. a3 Nd7 11. Rb1 f5 12. b4

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. a3 Nd7 11. Rb1 f5 12. b4'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnn1bp/3p2p1/p2Ppp2/2P1P3/P1N5/1P1NBPPP/1RBQ1RK1 w - f6 0 12' => {
			eco => 'E97z',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. a3 Nd7 11. Rb1'),
			moves => {
				'b2b4' => 'r1bq1rk1/1ppnn1bp/3p2p1/p2Ppp2/1PP1P3/P1N5/3NBPPP/1RBQ1RK1 b - b3 0 12',
			},
		},
		'r1bq1rk1/1ppnnpbp/3p2p1/3Pp3/1BP1P3/2N2N2/P3BPPP/R2Q1RK1 w - - 1 12' => {
			eco => 'E97k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 a5 10. Ba3 axb4 11. Bxb4 Nd7

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... a5 10. Ba3 axb4 11.Bxb4 Nd7'),
			moves => {
			},
		},
		'r1bq1rk1/1ppnnpbp/3p2p1/p2Pp3/2P1P3/P1N5/1P1NBPPP/1RBQ1RK1 b - - 2 11' => {
			eco => 'E97z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5 10. a3 Nd7 11. Rb1

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. a3 Nd7 11. Rb1'),
			moves => {
				'f7f5' => 'r1bq1rk1/1ppnn1bp/3p2p1/p2Ppp2/2P1P3/P1N5/1P1NBPPP/1RBQ1RK1 w - f6 0 12',
			},
		},
		'r1bq1rk1/1ppnnpbp/3p2p1/p2Pp3/2P1P3/P1N5/1P1NBPPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'E97z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5 10. a3 Nd7

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. a3 Nd7'),
			moves => {
				'a1b1' => 'r1bq1rk1/1ppnnpbp/3p2p1/p2Pp3/2P1P3/P1N5/1P1NBPPP/1RBQ1RK1 b - - 2 11',
			},
		},
		'r1bq1rk1/1pppbppp/p1n2n2/4P3/B2p4/5N2/PPP2PPP/RNBQR1K1 b - - 0 8' => {
			eco => 'C84o',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 O-O'),
			moves => {
				'f6e8' => 'r1bqnrk1/1pppbppp/p1n5/4P3/B2p4/5N2/PPP2PPP/RNBQR1K1 w - - 1 9',
			},
		},
		'r1bq1rk1/1pppbppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQR1K1 w - - 2 8' => {
			eco => 'C84o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 O-O

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 O-O'),
			moves => {
				'e4e5' => 'r1bq1rk1/1pppbppp/p1n2n2/4P3/B2p4/5N2/PPP2PPP/RNBQR1K1 b - - 0 8',
			},
		},
		'r1bq1rk1/1pppbppp/p1n5/4P3/B2Nn3/8/PPP2PPP/RNBQ1RK1 w - - 1 9' => {
			eco => 'C84k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4 8. Nxd4 O-O

			variation => N__('Spanish: Closed, Centre Attack, 7. e5 Ne4 8.Nxd4 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4

			variation => N__('Spanish: Closed, Smyslov, 10. d4'),
			moves => {
				'f8e8' => 'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11',
			},
		},
		'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 0 10' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6

			variation => N__('Spanish: Closed, Smyslov Defence'),
			moves => {
				'd2d3' => 'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/4P3/1BPP1N1P/PP3PP1/RNBQR1K1 b - - 0 10',
				'd2d4' => 'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10',
			},
		},
		'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/4P3/1BPP1N1P/PP3PP1/RNBQR1K1 b - - 0 10' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d3

			variation => N__('Spanish: Closed, Smyslov, 10. d3'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/4P3/1BPP1N2/PP3PPP/RNBQR1K1 w - - 0 10' => {
			eco => 'C90f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 h6

			variation => N__('Spanish: Closed, Pilnik, 9... h6'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/2np1n2/pp2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 0 10' => {
			eco => 'C92b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 a5

			variation => N__('Spanish: Closed, Keres Variation'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1n2n2/1p1P4/4p3/1BP2N2/PP1P1PPP/RNBQR1K1 w - - 0 10' => {
			eco => 'C89b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 e4

			variation => N__('Spanish: Marshall, Herman Steiner Variation'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1n2n2/1p1Pp3/8/1BP2N2/PP1P1PPP/RNBQR1K1 b - - 0 9' => {
			eco => 'C89b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5

			variation => N__('Spanish: Marshall, 9.exd5'),
			moves => {
				'e5e4' => 'r1bq1rk1/2p1bppp/p1n2n2/1p1P4/4p3/1BP2N2/PP1P1PPP/RNBQR1K1 w - - 0 10',
				'f6d5' => 'r1bq1rk1/2p1bppp/p1n5/1p1np3/8/1BP2N2/PP1P1PPP/RNBQR1K1 w - - 0 10',
			},
		},
		'r1bq1rk1/2p1bppp/p1n2n2/1p1Pp3/8/1BP2N2/PP1PQPPP/RNB2RK1 b - - 0 9' => {
			eco => 'C86j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d5 9. exd5

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d5 9.exd5'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1n2n2/1p1pp3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w - d6 0 9' => {
			eco => 'C89a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5

			variation => N__('Spanish: Marshall Counterattack'),
			moves => {
				'e4d5' => 'r1bq1rk1/2p1bppp/p1n2n2/1p1Pp3/8/1BP2N2/PP1P1PPP/RNBQR1K1 b - - 0 9',
			},
		},
		'r1bq1rk1/2p1bppp/p1n2n2/1p1pp3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 w - d6 0 9' => {
			eco => 'C86j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d5

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d5'),
			moves => {
				'd2d3' => 'r1bq1rk1/2p1bppp/p1n2n2/1p1pp3/4P3/1BPP1N2/PP2QPPP/RNB2RK1 b - - 0 9',
				'e4d5' => 'r1bq1rk1/2p1bppp/p1n2n2/1p1Pp3/8/1BP2N2/PP1PQPPP/RNB2RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/2p1bppp/p1n2n2/1p1pp3/4P3/1BPP1N2/PP2QPPP/RNB2RK1 b - - 0 9' => {
			eco => 'C86k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d5 9. d3

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d5 9. d3'),
			moves => {
				'c8b7' => 'r2q1rk1/1bp1bppp/p1n2n2/1p1pp3/4P3/1BPP1N2/PP2QPPP/RNB2RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/2p1bppp/p1n5/1p1nN3/8/1BP5/PP1P1PPP/RNBQR1K1 b - - 0 10' => {
			eco => 'C89d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5

			variation => N__('Spanish: Marshall, 9.exd5 Nxd5 10.Nxe5'),
			moves => {
				'c6e5' => 'r1bq1rk1/2p1bppp/p7/1p1nn3/8/1BP5/PP1P1PPP/RNBQR1K1 w - - 0 11',
			},
		},
		'r1bq1rk1/2p1bppp/p1n5/1p1np3/8/1BP2N2/PP1P1PPP/RNBQR1K1 w - - 0 10' => {
			eco => 'C89c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5

			variation => N__('Spanish: Marshall, 9.exd5 Nxd5'),
			moves => {
				'f3e5' => 'r1bq1rk1/2p1bppp/p1n5/1p1nN3/8/1BP5/PP1P1PPP/RNBQR1K1 b - - 0 10',
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1BP2N2/PP2QPPP/RNB2RK1 b - d3 0 9' => {
			eco => 'C86h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d6 9. d4

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d6 9. d4'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1BP2N2/PP3PPP/RNBQR1K1 b - d3 0 9' => {
			eco => 'C91a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4

			variation => N__('Spanish: Closed, 9. d4'),
			moves => {
				'c8g4' => 'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP1b1/1BP2N2/PP3PPP/RNBQR1K1 w - - 1 10',
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQR1K1 w - - 0 9' => {
			eco => 'C88g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. d3 d6

			variation => N__('Spanish: Closed, 8. d3 d6'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 b - - 0 9' => {
			eco => 'C92a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3

			variation => N__('Spanish: Closed, 9. h3'),
			moves => {
				'a6a5' => 'r1bq1rk1/2p1bppp/2np1n2/pp2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 0 10',
				'c6a5' => 'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10',
				'c6b8' => 'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10',
				'c8b7' => 'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10',
				'c8e6' => 'r2q1rk1/2p1bppp/p1npbn2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10',
				'f6d7' => 'r1bq1rk1/2pnbppp/p1np4/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10',
				'h7h6' => 'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 0 10',
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w - - 0 9' => {
			eco => 'C90a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6

			variation => N__('Spanish: Closed, 8... d6'),
			moves => {
				'a2a3' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/PBP2N2/1P1P1PPP/RNBQR1K1 b - - 0 9',
				'a2a4' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/P3P3/1BP2N2/1P1P1PPP/RNBQR1K1 b - a3 0 9',
				'b3c2' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/2P2N2/PPBP1PPP/RNBQR1K1 b - - 1 9',
				'd2d3' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BPP1N2/PP3PPP/RNBQR1K1 b - - 0 9',
				'd2d4' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1BP2N2/PP3PPP/RNBQR1K1 b - d3 0 9',
				'h2h3' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 b - - 0 9',
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 w - - 0 9' => {
			eco => 'C86g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d6

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d6'),
			moves => {
				'd2d4' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1BP2N2/PP2QPPP/RNB2RK1 b - d3 0 9',
				'f1d1' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNBR2K1 b - - 1 9',
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNBR2K1 b - - 1 9' => {
			eco => 'C86i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d6 9. Rd1

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d6 9. Rd1'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BPP1N2/PP3PPP/RNBQR1K1 b - - 0 9' => {
			eco => 'C90e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3

			variation => N__('Spanish: Closed, Pilnik Variation'),
			moves => {
				'c6a5' => 'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/1BPP1N2/PP3PPP/RNBQR1K1 w - - 1 10',
				'h7h6' => 'r1bq1rk1/2p1bpp1/p1np1n1p/1p2p3/4P3/1BPP1N2/PP3PPP/RNBQR1K1 w - - 0 10',
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/2P2N2/PPBP1PPP/RNBQR1K1 b - - 1 9' => {
			eco => 'C90b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. Bc2

			variation => N__('Spanish: Closed, Lutikov Variation'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/PBP2N2/1P1P1PPP/RNBQR1K1 b - - 0 9' => {
			eco => 'C90c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. a3

			variation => N__('Spanish: Closed, Suetin Variation'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/P3P3/1BP2N2/1P1P1PPP/RNBQR1K1 b - a3 0 9' => {
			eco => 'C90d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. a4

			variation => N__('Spanish: Closed, 8... d6 9. a4'),
			moves => {
				'c8g4' => 'r2q1rk1/2p1bppp/p1np1n2/1p2p3/P3P1b1/1BP2N2/1P1P1PPP/RNBQR1K1 w - - 1 10',
			},
		},
		'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10' => {
			eco => 'C96a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5

			variation => N__('Spanish: Closed, Chigorin'),
			moves => {
				'b3c2' => 'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 b - - 2 10',
			},
		},
		'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/1BPP1N2/PP3PPP/RNBQR1K1 w - - 1 10' => {
			eco => 'C90g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5

			variation => N__('Spanish: Closed, Pilnik, 9... Na5'),
			moves => {
				'b3c2' => 'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/2PP1N2/PPB2PPP/RNBQR1K1 b - - 2 10',
			},
		},
		'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 b - - 2 10' => {
			eco => 'C96b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2

			variation => N__('Spanish: Closed, Chigorin, 10. Bc2'),
			moves => {
				'c7c5' => 'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 w - c6 0 11',
				'c7c6' => 'r1bq1rk1/4bppp/p1pp1n2/np2p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 w - - 0 11',
				'c8b7' => 'r2q1rk1/1bp1bppp/p2p1n2/np2p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 w - - 3 11',
			},
		},
		'r1bq1rk1/2p1bppp/p2p1n2/np2p3/4P3/2PP1N2/PPB2PPP/RNBQR1K1 b - - 2 10' => {
			eco => 'C90g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2

			variation => N__('Spanish: Closed, Pilnik, 9... Na5'),
			moves => {
				'c7c5' => 'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPB2PPP/RNBQR1K1 w - c6 0 11',
			},
		},
		'r1bq1rk1/2p1bppp/p4n2/1p2R3/8/1BP5/PP1P1PPP/RNBQ2K1 w - - 1 12' => {
			eco => 'C89e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 Nf6

			variation => N__('Spanish: Marshall, 11.Rxe5 Nf6'),
			moves => {
			},
		},
		'r1bq1rk1/2p1bppp/p7/1p1nR3/8/1BP5/PP1P1PPP/RNBQ2K1 b - - 0 11' => {
			eco => 'C89d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5

			variation => N__('Spanish: Marshall, 11.Rxe5'),
			moves => {
				'c7c6' => 'r1bq1rk1/4bppp/p1p5/1p1nR3/8/1BP5/PP1P1PPP/RNBQ2K1 w - - 0 12',
				'd5f6' => 'r1bq1rk1/2p1bppp/p4n2/1p2R3/8/1BP5/PP1P1PPP/RNBQ2K1 w - - 1 12',
			},
		},
		'r1bq1rk1/2p1bppp/p7/1p1nn3/8/1BP5/PP1P1PPP/RNBQR1K1 w - - 0 11' => {
			eco => 'C89d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5

			variation => N__('Spanish: Marshall, 9.exd5 Nxd5 10.Nxe5 Nxe5'),
			moves => {
				'e1e5' => 'r1bq1rk1/2p1bppp/p7/1p1nR3/8/1BP5/PP1P1PPP/RNBQ2K1 b - - 0 11',
			},
		},
		'r1bq1rk1/2pn1ppp/p1np1b2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11' => {
			eco => 'C92f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nd7 10. d4 Bf6

			variation => N__('Spanish: Closed, Karpov Variation, 10. d4 Bf6'),
			moves => {
				'a2a4' => 'r1bq1rk1/2pn1ppp/p1np1b2/1p2p3/P2PP3/1BP2N1P/1P3PP1/RNBQR1K1 b - a3 0 11',
			},
		},
		'r1bq1rk1/2pn1ppp/p1np1b2/1p2p3/P2PP3/1BP2N1P/1P3PP1/RNBQR1K1 b - a3 0 11' => {
			eco => 'C92g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nd7 10. d4 Bf6 11. a4

			variation => N__('Spanish: Closed, Karpov Variation, 10. d4 Bf6 11. a4'),
			moves => {
				'c8b7' => 'r2q1rk1/1bpn1ppp/p1np1b2/1p2p3/P2PP3/1BP2N1P/1P3PP1/RNBQR1K1 w - - 1 12',
			},
		},
		'r1bq1rk1/2pnbppp/p1np4/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10' => {
			eco => 'C92e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Karpov Variation'),
			moves => {
				'e7f6' => 'r1bq1rk1/2pn1ppp/p1np1b2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11',
			},
		},
		'r1bq1rk1/2pnbppp/p1np4/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10' => {
			eco => 'C92e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nd7

			variation => N__('Spanish: Closed, Karpov Variation'),
			moves => {
				'd2d4' => 'r1bq1rk1/2pnbppp/p1np4/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10',
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p1B1/3PP3/1BP2N1P/PP3PP1/RN1QR1K1 b - - 2 11' => {
			eco => 'C95d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Bg5

			variation => N__('Spanish: Closed, Breyer, 10. d4 Nbd7 11. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/2PPP3/1B3N1P/PP3PP1/RNBQR1K1 b - - 0 11' => {
			eco => 'C95e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. c4

			variation => N__('Spanish: Closed, Breyer, Arseniev Variation'),
			moves => {
				'c7c6' => 'r1bq1rk1/3nbppp/p1pp1n2/1p2p3/2PPP3/1B3N1P/PP3PP1/RNBQR1K1 w - - 0 12',
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/3PP2N/1BP4P/PP3PP1/RNBQR1K1 b - - 2 11' => {
			eco => 'C95c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nh4

			variation => N__('Spanish: Closed, Breyer, Simagin Variation'),
			moves => {
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 2 11' => {
			eco => 'C95f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2

			variation => N__('Spanish: Closed, Breyer, 10. d4 Nbd7 11.Nbd2'),
			moves => {
				'c8b7' => 'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 3 12',
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11' => {
			eco => 'C95b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7

			variation => N__('Spanish: Closed, Breyer, 10. d4 Nbd7'),
			moves => {
				'b1d2' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 2 11',
				'c1g5' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p1B1/3PP3/1BP2N1P/PP3PP1/RN1QR1K1 b - - 2 11',
				'c3c4' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/2PPP3/1B3N1P/PP3PP1/RNBQR1K1 b - - 0 11',
				'f3h4' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/3PP2N/1BP4P/PP3PP1/RNBQR1K1 b - - 2 11',
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP1N1PP1/R1BQR1K1 b - - 2 11' => {
			eco => 'C94e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d3 Nbd7 11. Nbd2

			variation => N__('Spanish: Closed, Breyer, 10. d3 Nbd7 11.Nbd2'),
			moves => {
				'c8b7' => 'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP1N1PP1/R1BQR1K1 w - - 3 12',
			},
		},
		'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP3PP1/RNBQR1K1 w - - 1 11' => {
			eco => 'C94d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d3 Nbd7

			variation => N__('Spanish: Closed, Breyer, 10. d3 Nbd7'),
			moves => {
				'b1d2' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP1N1PP1/R1BQR1K1 b - - 2 11',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQR1K1 b - - 0 8' => {
			eco => 'C88g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. d3

			variation => N__('Spanish: Closed, 8. d3'),
			moves => {
				'd7d6' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQR1K1 w - - 0 9',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B3N1P/PPPP1PP1/RNBQR1K1 b - - 0 8' => {
			eco => 'C88h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. h3

			variation => N__('Spanish: Closed, 8. h3'),
			moves => {
				'c8b7' => 'r2q1rk1/1bppbppp/p1n2n2/1p2p3/4P3/1B3N1P/PPPP1PP1/RNBQR1K1 w - - 1 9',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w - - 2 8' => {
			eco => 'C88f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O

			variation => N__('Spanish: Closed, 7...O-O'),
			moves => {
				'a2a4' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/P3P3/1B3N2/1PPP1PPP/RNBQR1K1 b - a3 0 8',
				'c2c3' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 b - - 0 8',
				'd2d3' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQR1K1 b - - 0 8',
				'h2h3' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B3N1P/PPPP1PP1/RNBQR1K1 b - - 0 8',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB2RK1 w - - 2 8' => {
			eco => 'C86e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O

			variation => N__('Spanish: Worrall Attack, 7...O-O'),
			moves => {
				'a2a4' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/P3P3/1B3N2/1PPPQPPP/RNB2RK1 b - a3 0 8',
				'c2c3' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 b - - 0 8' => {
			eco => 'C88q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3

			variation => N__('Spanish: Closed, 8. c3'),
			moves => {
				'd7d5' => 'r1bq1rk1/2p1bppp/p1n2n2/1p1pp3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w - d6 0 9',
				'd7d6' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w - - 0 9',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 b - - 0 8' => {
			eco => 'C86f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3'),
			moves => {
				'd7d5' => 'r1bq1rk1/2p1bppp/p1n2n2/1p1pp3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 w - d6 0 9',
				'd7d6' => 'r1bq1rk1/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/P3P3/1B3N2/1PPP1PPP/RNBQR1K1 b - a3 0 8' => {
			eco => 'C88j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4'),
			moves => {
				'b5b4' => 'r1bq1rk1/2ppbppp/p1n2n2/4p3/Pp2P3/1B3N2/1PPP1PPP/RNBQR1K1 w - - 0 9',
				'c8b7' => 'r2q1rk1/1bppbppp/p1n2n2/1p2p3/P3P3/1B3N2/1PPP1PPP/RNBQR1K1 w - - 1 9',
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/P3P3/1B3N2/1PPPQPPP/RNB2RK1 b - a3 0 8' => {
			eco => 'C86e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. a4

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. a4'),
			moves => {
			},
		},
		'r1bq1rk1/2ppbppp/p1n2n2/4p3/Pp2P3/1B3N2/1PPP1PPP/RNBQR1K1 w - - 0 9' => {
			eco => 'C88k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4 b4

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4 b4'),
			moves => {
			},
		},
		'r1bq1rk1/3n1ppp/p1pbpn2/1p6/2BP4/2N1PN2/PPQ2PPP/R1BR2K1 w - b6 0 11' => {
			eco => 'D46k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 a6 10. Rd1 b5

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... a6 10. Rd1 b5'),
			moves => {
			},
		},
		'r1bq1rk1/3nbppp/1pp1pn2/p2p4/2PP4/1P3NP1/P1QNPPBP/R1B2RK1 w - a6 0 10' => {
			eco => 'E09e',
                        # TRANSLATORS:

			variation => N__('Catalan: Closed, Main Line, 9. b3'),
			moves => {
				'c1b2' => 'r1bq1rk1/3nbppp/1pp1pn2/p2p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/3nbppp/1pp1pn2/p2p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 b - - 1 10' => {
			eco => 'E09e',
                        # TRANSLATORS:

			variation => N__('Catalan: Closed, Main Line, 9. b3'),
			moves => {
				'c8a6' => 'r2q1rk1/3nbppp/bpp1pn2/p2p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/3nbppp/p1p1pn2/1p4B1/3P4/2NBPN2/PP3PPP/2RQ1RK1 b - - 1 11' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 b5 10. Bd3 a6 11. O-O

			variation => N__('QGD: Orthodox, Main Line, Fianchetto, 11.O-O'),
			moves => {
			},
		},
		'r1bq1rk1/3nbppp/p1p1pn2/1p4B1/3P4/2NBPN2/PP3PPP/2RQK2R w K - 0 11' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 b5 10. Bd3 a6

			variation => N__('QGD: Orthodox, Main Line, Fianchetto Variation'),
			moves => {
				'e1g1' => 'r1bq1rk1/3nbppp/p1p1pn2/1p4B1/3P4/2NBPN2/PP3PPP/2RQ1RK1 b - - 1 11',
				'e3e4' => 'r1bq1rk1/3nbppp/p1p1pn2/1p4B1/3PP3/2NB1N2/PP3PPP/2RQK2R b K - 0 11',
			},
		},
		'r1bq1rk1/3nbppp/p1p1pn2/1p4B1/3PP3/2NB1N2/PP3PPP/2RQK2R b K - 0 11' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 b5 10. Bd3 a6 11. e4

			variation => N__('QGD: Orthodox, Main Line, Fianchetto, 11. e4'),
			moves => {
			},
		},
		'r1bq1rk1/3nbppp/p1pp1n2/1p2p3/2PPP3/1B3N1P/PP3PP1/RNBQR1K1 w - - 0 12' => {
			eco => 'C95e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. c4 c6

			variation => N__('Spanish: Closed, Breyer, Arseniev, 11... c6'),
			moves => {
			},
		},
		'r1bq1rk1/3nbppp/p2p4/np2p3/3pP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 0 13' => {
			eco => 'C96o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Nd7 12. Nbd2 cxd4

			variation => N__('Spanish: Closed, Chigorin, Keres, 11.Nbd2 cxd4'),
			moves => {
			},
		},
		'r1bq1rk1/3nbppp/p2p4/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C96n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Nd7

			variation => N__('Spanish: Closed, Chigorin, Keres Variation'),
			moves => {
				'b1d2' => 'r1bq1rk1/3nbppp/p2p4/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 2 12',
			},
		},
		'r1bq1rk1/3nbppp/p2p4/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 2 12' => {
			eco => 'C96n',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Chigorin, Keres Variation'),
			moves => {
				'c5d4' => 'r1bq1rk1/3nbppp/p2p4/np2p3/3pP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 0 13',
			},
		},
		'r1bq1rk1/4bppp/p1np1n2/1pp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C96m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Nc6

			variation => N__('Spanish: Closed, Chigorin, Borisenko Variation'),
			moves => {
			},
		},
		'r1bq1rk1/4bppp/p1np1n2/1pp1p3/4P3/2PP1N1P/PPB2PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C96h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d3 Nc6

			variation => N__('Spanish: Closed, Chigorin, 11. d3 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/4bppp/p1np1n2/1pp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 w - - 2 12' => {
			eco => 'C90j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Nc6

			variation => N__('Spanish: Closed, Pilnik, 11.Nbd2 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/4bppp/p1p5/1p1BR3/8/2P5/PP1P1PPP/RNBQ2K1 b - - 0 12' => {
			eco => 'C89g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. Bxd5

			variation => N__('Spanish: Marshall, 12.Bxd5'),
			moves => {
				'c6d5' => 'r1bq1rk1/4bppp/p7/1p1pR3/8/2P5/PP1P1PPP/RNBQ2K1 w - - 0 13',
			},
		},
		'r1bq1rk1/4bppp/p1p5/1p1nR3/3P4/1BP5/PP3PPP/RNBQ2K1 b - d3 0 12' => {
			eco => 'C89k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4

			variation => N__('Spanish: Marshall, Main Line (12. d4)'),
			moves => {
				'e7d6' => 'r1bq1rk1/5ppp/p1pb4/1p1nR3/3P4/1BP5/PP3PPP/RNBQ2K1 w - - 1 13',
			},
		},
		'r1bq1rk1/4bppp/p1p5/1p1nR3/8/1BP5/PP1P1PPP/RNBQ2K1 w - - 0 12' => {
			eco => 'C89f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6

			variation => N__('Spanish: Marshall, 11.Rxe5 c6'),
			moves => {
				'b3d5' => 'r1bq1rk1/4bppp/p1p5/1p1BR3/8/2P5/PP1P1PPP/RNBQ2K1 b - - 0 12',
				'd2d3' => 'r1bq1rk1/4bppp/p1p5/1p1nR3/8/1BPP4/PP3PPP/RNBQ2K1 b - - 0 12',
				'd2d4' => 'r1bq1rk1/4bppp/p1p5/1p1nR3/3P4/1BP5/PP3PPP/RNBQ2K1 b - d3 0 12',
			},
		},
		'r1bq1rk1/4bppp/p1p5/1p1nR3/8/1BPP4/PP3PPP/RNBQ2K1 b - - 0 12' => {
			eco => 'C89i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d3

			variation => N__('Spanish: Marshall, 12. d3'),
			moves => {
				'e7d6' => 'r1bq1rk1/5ppp/p1pb4/1p1nR3/8/1BPP4/PP3PPP/RNBQ2K1 w - - 1 13',
			},
		},
		'r1bq1rk1/4bppp/p1pp1n2/np1Pp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 13' => {
			eco => 'C91i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. d5 Na5 11. Bc2 c6 12. h3 Bc8

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5 Na5 11. Bc2 c6 12. h3 Bc8'),
			moves => {
			},
		},
		'r1bq1rk1/4bppp/p1pp1n2/np2p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 b - d3 0 11' => {
			eco => 'C96c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Chigorin, 10... c6'),
			moves => {
				'd8c7' => 'r1b2rk1/2q1bppp/p1pp1n2/np2p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12',
			},
		},
		'r1bq1rk1/4bppp/p1pp1n2/np2p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 w - - 0 11' => {
			eco => 'C96c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c6

			variation => N__('Spanish: Closed, Chigorin, 10... c6'),
			moves => {
				'd2d4' => 'r1bq1rk1/4bppp/p1pp1n2/np2p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 b - d3 0 11',
			},
		},
		'r1bq1rk1/4bppp/p2p1n2/np2p3/3pP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 0 12' => {
			eco => 'C96j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 cxd4

			variation => N__('Spanish: Closed, Chigorin, 11. d4 cxd4'),
			moves => {
			},
		},
		'r1bq1rk1/4bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 b - d3 0 11' => {
			eco => 'C96i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4

			variation => N__('Spanish: Closed, Chigorin, 11. d4'),
			moves => {
				'a5c6' => 'r1bq1rk1/4bppp/p1np1n2/1pp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12',
				'c5d4' => 'r1bq1rk1/4bppp/p2p1n2/np2p3/3pP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 0 12',
				'c8b7' => 'r2q1rk1/1b2bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12',
				'd8c7' => 'r1b2rk1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12',
				'f6d7' => 'r1bq1rk1/3nbppp/p2p4/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12',
			},
		},
		'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 w - c6 0 11' => {
			eco => 'C96f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5

			variation => N__('Spanish: Closed, Chigorin, 10... c5'),
			moves => {
				'd2d3' => 'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2PP1N1P/PPB2PP1/RNBQR1K1 b - - 0 11',
				'd2d4' => 'r1bq1rk1/4bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 b - d3 0 11',
			},
		},
		'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2PP1N1P/PPB2PP1/RNBQR1K1 b - - 0 11' => {
			eco => 'C96g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d3

			variation => N__('Spanish: Closed, Chigorin, 11. d3'),
			moves => {
				'a5c6' => 'r1bq1rk1/4bppp/p1np1n2/1pp1p3/4P3/2PP1N1P/PPB2PP1/RNBQR1K1 w - - 1 12',
			},
		},
		'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPB2PPP/RNBQR1K1 w - c6 0 11' => {
			eco => 'C90g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5

			variation => N__('Spanish: Closed, Pilnik, 9... Na5'),
			moves => {
				'b1d2' => 'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 b - - 1 11',
			},
		},
		'r1bq1rk1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 b - - 1 11' => {
			eco => 'C90h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2

			variation => N__('Spanish: Closed, Pilnik, 11.Nbd2'),
			moves => {
				'a5c6' => 'r1bq1rk1/4bppp/p1np1n2/1pp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 w - - 2 12',
				'd8c7' => 'r1b2rk1/2q1bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 w - - 2 12',
				'f8e8' => 'r1bqr1k1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 w - - 2 12',
			},
		},
		'r1bq1rk1/4bppp/p7/1p1pR3/3P4/2P5/PP3PPP/RNBQ2K1 b - d3 0 13' => {
			eco => 'C89g',
                        # TRANSLATORS:

			variation => N__('Spanish: Marshall, 12.Bxd5'),
			moves => {
				'e7d6' => 'r1bq1rk1/5ppp/p2b4/1p1pR3/3P4/2P5/PP3PPP/RNBQ2K1 w - - 1 14',
			},
		},
		'r1bq1rk1/4bppp/p7/1p1pR3/8/2P5/PP1P1PPP/RNBQ2K1 w - - 0 13' => {
			eco => 'C89g',
                        # TRANSLATORS:

			variation => N__('Spanish: Marshall, 12.Bxd5'),
			moves => {
				'd2d4' => 'r1bq1rk1/4bppp/p7/1p1pR3/3P4/2P5/PP3PPP/RNBQ2K1 b - d3 0 13',
			},
		},
		'r1bq1rk1/4bppp/pp1ppn2/4n3/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 w - - 2 12' => {
			eco => 'B44x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O 10. O-O b6 11. Be3 Ne5

			variation => N__('Sicilian: Taimanov, Szen, Hedgehog, 11. Be3 Ne5'),
			moves => {
			},
		},
		'r1bq1rk1/4bppp/ppnppn2/8/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 b - - 1 11' => {
			eco => 'B44x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O 10. O-O b6 11. Be3

			variation => N__('Sicilian: Taimanov, Szen, Hedgehog, 11. Be3'),
			moves => {
				'c6e5' => 'r1bq1rk1/4bppp/pp1ppn2/4n3/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 w - - 2 12',
				'c8b7' => 'r2q1rk1/1b2bppp/ppnppn2/8/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 w - - 2 12',
				'c8d7' => 'r2q1rk1/3bbppp/ppnppn2/8/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 w - - 2 12',
			},
		},
		'r1bq1rk1/4bppp/ppnppn2/8/2P1P3/N1N5/PP2BPPP/R1BQ1RK1 w - - 0 11' => {
			eco => 'B44w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O 10. O-O b6

			variation => N__('Sicilian: Taimanov, Szen, Hedgehog Variation'),
			moves => {
				'c1e3' => 'r1bq1rk1/4bppp/ppnppn2/8/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/5ppp/p1np1b2/1p1Np3/4P3/2P5/PPN2PPP/R2QKB1R b KQ - 2 12' => {
			eco => 'B33o',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7, 11. c3 O-O'),
			moves => {
				'f6g5' => 'r1bq1rk1/5ppp/p1np4/1p1Np1b1/4P3/2P5/PPN2PPP/R2QKB1R w KQ - 3 13',
			},
		},
		'r1bq1rk1/5ppp/p1np1b2/1p1Np3/4P3/N1P5/PP3PPP/R2QKB1R w KQ - 1 12' => {
			eco => 'B33o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5 Be7 10. Bxf6 Bxf6 11. c3 O-O

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7, 11. c3 O-O'),
			moves => {
				'a3c2' => 'r1bq1rk1/5ppp/p1np1b2/1p1Np3/4P3/2P5/PPN2PPP/R2QKB1R b KQ - 2 12',
			},
		},
		'r1bq1rk1/5ppp/p1np4/1p1Np1b1/4P3/2P5/PPN2PPP/R2QKB1R w KQ - 3 13' => {
			eco => 'B33q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5 Be7 10. Bxf6 Bxf6 11. c3 O-O 12. Nc2 Bg5

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7, 11. c3 O-O 12. Nc2 Bg5'),
			moves => {
				'a2a4' => 'r1bq1rk1/5ppp/p1np4/1p1Np1b1/P3P3/2P5/1PN2PPP/R2QKB1R b KQ a3 0 13',
			},
		},
		'r1bq1rk1/5ppp/p1np4/1p1Np1b1/P3P3/2P5/1PN2PPP/R2QKB1R b KQ a3 0 13' => {
			eco => 'B33r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5 Be7 10. Bxf6 Bxf6 11. c3 O-O 12. Nc2 Bg5 13. a4

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7, 11. c3 O-O 12. Nc2 Bg5 13. a4'),
			moves => {
			},
		},
		'r1bq1rk1/5ppp/p1pb4/1p1n4/3P4/1BP5/PP2RPPP/RNBQ2K1 b - - 2 13' => {
			eco => 'C89l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re2

			variation => N__('Spanish: Marshall, Main Line, 13. Re2'),
			moves => {
			},
		},
		'r1bq1rk1/5ppp/p1pb4/1p1n4/3P4/1BP5/PP3PPP/RNBQR1K1 b - - 2 13' => {
			eco => 'C89m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1

			variation => N__('Spanish: Marshall, Main Line, 13. Re1'),
			moves => {
				'd8h4' => 'r1b2rk1/5ppp/p1pb4/1p1n4/3P3q/1BP5/PP3PPP/RNBQR1K1 w - - 3 14',
			},
		},
		'r1bq1rk1/5ppp/p1pb4/1p1n4/8/1BPP4/PP3PPP/RNBQR1K1 b - - 2 13' => {
			eco => 'C89i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d3 Bd6 13. Re1

			variation => N__('Spanish: Marshall, 12. d3 Bd6 13. Re1'),
			moves => {
				'd8h4' => 'r1b2rk1/5ppp/p1pb4/1p1n4/7q/1BPP4/PP3PPP/RNBQR1K1 w - - 3 14',
			},
		},
		'r1bq1rk1/5ppp/p1pb4/1p1nR3/3P4/1BP5/PP3PPP/RNBQ2K1 w - - 1 13' => {
			eco => 'C89k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6

			variation => N__('Spanish: Marshall, Main Line (12. d4 Bd6)'),
			moves => {
				'e5e1' => 'r1bq1rk1/5ppp/p1pb4/1p1n4/3P4/1BP5/PP3PPP/RNBQR1K1 b - - 2 13',
				'e5e2' => 'r1bq1rk1/5ppp/p1pb4/1p1n4/3P4/1BP5/PP2RPPP/RNBQ2K1 b - - 2 13',
			},
		},
		'r1bq1rk1/5ppp/p1pb4/1p1nR3/8/1BPP4/PP3PPP/RNBQ2K1 w - - 1 13' => {
			eco => 'C89i',
                        # TRANSLATORS:

			variation => N__('Spanish: Marshall, 12. d3'),
			moves => {
				'e5e1' => 'r1bq1rk1/5ppp/p1pb4/1p1n4/8/1BPP4/PP3PPP/RNBQR1K1 b - - 2 13',
			},
		},
		'r1bq1rk1/5ppp/p2b4/1p1p4/3P4/2P1R3/PP3PPP/RNBQ2K1 b - - 2 14' => {
			eco => 'C89h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. Bxd5 cxd5 13. d4 Bd6 14. Re3

			variation => N__('Spanish: Marshall, Kevitz Variation'),
			moves => {
			},
		},
		'r1bq1rk1/5ppp/p2b4/1p1pR3/3P4/2P5/PP3PPP/RNBQ2K1 w - - 1 14' => {
			eco => 'C89g',
                        # TRANSLATORS:

			variation => N__('Spanish: Marshall, 12.Bxd5'),
			moves => {
				'e5e3' => 'r1bq1rk1/5ppp/p2b4/1p1p4/3P4/2P1R3/PP3PPP/RNBQ2K1 b - - 2 14',
			},
		},
		'r1bq1rk1/p1p1ppbp/1pn3p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 Nc6 9. O-O b6

			variation => N__('Grünfeld: Classical Exchange, Simagin Improved, 9.O-O b6'),
			moves => {
			},
		},
		'r1bq1rk1/p1p2ppp/5n2/3p2B1/1b6/2NB4/PPP2PPP/R2Q1RK1 b - - 3 10' => {
			eco => 'C47u',
                        # TRANSLATORS:

			variation => N__('Four Knights: Scotch, Main Line, 8.exd5 cxd5 9.O-O O-O'),
			moves => {
				'c7c6' => 'r1bq1rk1/p4ppp/2p2n2/3p2B1/1b6/2NB4/PPP2PPP/R2Q1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/p1p2ppp/5n2/3p4/1b6/2NB4/PPP2PPP/R1BQ1RK1 w - - 2 10' => {
			eco => 'C47u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5 8. exd5 cxd5 9. O-O O-O

			variation => N__('Four Knights: Scotch, Main Line, 8.exd5 cxd5 9.O-O O-O'),
			moves => {
				'c1g5' => 'r1bq1rk1/p1p2ppp/5n2/3p2B1/1b6/2NB4/PPP2PPP/R2Q1RK1 b - - 3 10',
			},
		},
		'r1bq1rk1/p1pnbppp/1p2pn2/3P2B1/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, 7. Rc1 b6'),
			moves => {
				'e6d5' => 'r1bq1rk1/p1pnbppp/1p3n2/3p2B1/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9',
			},
		},
		'r1bq1rk1/p1pnbppp/1p2pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 b6

			variation => N__('QGD: Orthodox, 7. Rc1 b6'),
			moves => {
				'c4d5' => 'r1bq1rk1/p1pnbppp/1p2pn2/3P2B1/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 8',
			},
		},
		'r1bq1rk1/p1pnbppp/1p2pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 8' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 b6

			variation => N__('QGD: Orthodox, Botvinnik, 7... b6'),
			moves => {
			},
		},
		'r1bq1rk1/p1pnbppp/1p2pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8' => {
			eco => 'E08c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 b6

			variation => N__('Catalan: Closed, 7. Qc2 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p1pnbppp/1p3n2/1B1p2B1/3P4/2N1PN2/PP3PPP/2RQK2R b K - 1 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 b6 8. cxd5 exd5 9. Bb5

			variation => N__('QGD: Orthodox, 7. Rc1 b6, Capablanca Variation'),
			moves => {
			},
		},
		'r1bq1rk1/p1pnbppp/1p3n2/3p2B1/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 b6 8. cxd5 exd5

			variation => N__('QGD: Orthodox, 7. Rc1 b6'),
			moves => {
				'd1a4' => 'r1bq1rk1/p1pnbppp/1p3n2/3p2B1/Q2P4/2N1PN2/PP3PPP/2R1KB1R b K - 1 9',
				'f1b5' => 'r1bq1rk1/p1pnbppp/1p3n2/1B1p2B1/3P4/2N1PN2/PP3PPP/2RQK2R b K - 1 9',
				'f1d3' => 'r1bq1rk1/p1pnbppp/1p3n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R b K - 1 9',
			},
		},
		'r1bq1rk1/p1pnbppp/1p3n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R b K - 1 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 b6 8. cxd5 exd5 9. Bd3

			variation => N__('QGD: Orthodox, 7. Rc1 b6, Pillsbury Attack'),
			moves => {
			},
		},
		'r1bq1rk1/p1pnbppp/1p3n2/3p2B1/Q2P4/2N1PN2/PP3PPP/2R1KB1R b K - 1 9' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 b6 8. cxd5 exd5 9. Qa4

			variation => N__('QGD: Orthodox, 7. Rc1 b6, Duras Variation'),
			moves => {
			},
		},
		'r1bq1rk1/p2n1ppp/1p2pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1B2RK1 w - - 0 10' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. Qe2 b6

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. Qe2 b6'),
			moves => {
				'f1d1' => 'r1bq1rk1/p2n1ppp/1p2pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1BR2K1 b - - 1 10',
			},
		},
		'r1bq1rk1/p2n1ppp/1p2pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1BR2K1 b - - 1 10' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. Qe2 b6 10. Rd1

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. Qe2 b6 10. Rd1'),
			moves => {
			},
		},
		'r1bq1rk1/p2n1ppp/1ppbpn2/3p4/2PP4/1PN1PN2/P1Q1BPPP/R1B1K2R w KQ - 0 9' => {
			eco => 'D45s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. b3 O-O 8. Be2 b6

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. b3 O-O 8. Be2 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p2n1ppp/2pbpn2/1p6/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - b6 0 10' => {
			eco => 'D46l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 b5

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... b5'),
			moves => {
				'c4e2' => 'r1bq1rk1/p2n1ppp/2pbpn2/1p6/3P4/2N1PN2/PPQ1BPPP/R1B2RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/p2n1ppp/2pbpn2/1p6/3P4/2N1PN2/PPQ1BPPP/R1B2RK1 b - - 1 10' => {
			eco => 'D46m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 b5 10. Be2

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... b5 10. Be2'),
			moves => {
				'c8b7' => 'r2q1rk1/pb1n1ppp/2pbpn2/1p6/3P4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/p2nbpp1/2p1pn1p/1p6/2BP3B/2N1PN2/PP3PPP/2RQK2R w K b6 0 11' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 h6 9. Bh4 dxc4 10. Bxc4 b5

			variation => N__('QGD: Orthodox, Main Line, 8... h6 9. Bh4 dxc4 10.Bxc4 b5'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/1pp1p3/8/2PPQ3/5NP1/PP3PBP/R1B2RK1 b - - 0 11' => {
			eco => 'E09k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 dxe4 10. Nxe4 Nxe4 11. Qxe4

			variation => N__('Catalan: Closed, Main Line, 9. e4 dxe4 10.Nxe4 Nxe4 11.Qxe4'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/1pp1p3/8/2PPn3/5NP1/PPQ2PBP/R1B2RK1 w - - 0 11' => {
			eco => 'E09j',
                        # TRANSLATORS:

			variation => N__('Catalan: Closed, Main Line, 9. e4 dxe4'),
			moves => {
				'c2e4' => 'r1bq1rk1/p2nbppp/1pp1p3/8/2PPQ3/5NP1/PP3PBP/R1B2RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 w - - 0 9' => {
			eco => 'E08j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Bf4 b6

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. Bf4 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 w - - 0 9' => {
			eco => 'E08m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6'),
			moves => {
				'f1d1' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNBR2K1 b - - 1 9',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNBR2K1 b - - 1 9' => {
			eco => 'E08n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1'),
			moves => {
				'c8a6' => 'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNBR2K1 w - - 2 10',
				'c8b7' => 'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNBR2K1 w - - 2 10',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1QNPPBP/R1B2RK1 b - - 0 9' => {
			eco => 'E09e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. b3

			variation => N__('Catalan: Closed, Main Line, 9. b3'),
			moves => {
				'a7a5' => 'r1bq1rk1/3nbppp/1pp1pn2/p2p4/2PP4/1P3NP1/P1QNPPBP/R1B2RK1 w - a6 0 10',
				'c8b7' => 'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1QNPPBP/R1B2RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNBR2K1 w - - 0 9' => {
			eco => 'E08f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Rd1 b6

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. Rd1 b6'),
			moves => {
				'a2a4' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/P1PP4/5NP1/1PQ1PPBP/RNBR2K1 b - a3 0 9',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 w - - 0 9' => {
			eco => 'E09d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6

			variation => N__('Catalan: Closed, Main Line, 8... b6'),
			moves => {
				'b2b3' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1QNPPBP/R1B2RK1 b - - 0 9',
				'e2e4' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PPP3/5NP1/PPQN1PBP/R1B2RK1 b - e3 0 9',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PPP3/5NP1/PPQN1PBP/R1B2RK1 b - e3 0 9' => {
			eco => 'E09h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4

			variation => N__('Catalan: Closed, Main Line, 9. e4'),
			moves => {
				'c8a6' => 'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PPP3/5NP1/PPQN1PBP/R1B2RK1 w - - 1 10',
				'c8b7' => 'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/5NP1/PPQN1PBP/R1B2RK1 w - - 1 10',
				'd5e4' => 'r1bq1rk1/p2nbppp/1pp1pn2/8/2PPp3/5NP1/PPQN1PBP/R1B2RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/3p4/P1PP4/5NP1/1PQ1PPBP/RNBR2K1 b - a3 0 9' => {
			eco => 'E08f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Rd1 b6 9. a4

			variation => N__('Catalan: Closed, Zagoryansky Variation'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/8/2PPN3/5NP1/PPQ2PBP/R1B2RK1 b - - 0 10' => {
			eco => 'E09j',
                        # TRANSLATORS:

			variation => N__('Catalan: Closed, Main Line, 9. e4 dxe4'),
			moves => {
				'f6e4' => 'r1bq1rk1/p2nbppp/1pp1p3/8/2PPn3/5NP1/PPQ2PBP/R1B2RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/p2nbppp/1pp1pn2/8/2PPp3/5NP1/PPQN1PBP/R1B2RK1 w - - 0 10' => {
			eco => 'E09j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 dxe4

			variation => N__('Catalan: Closed, Main Line, 9. e4 dxe4'),
			moves => {
				'd2e4' => 'r1bq1rk1/p2nbppp/1pp1pn2/8/2PPN3/5NP1/PPQ2PBP/R1B2RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/p2nbppp/1ppp1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQR1K1 w - - 0 9' => {
			eco => 'C41z',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. Re1 c6 8. a4 b6

			variation => N__('Philidor: Improved Hanham, 7. Re1 c6 8. a4 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/2p1pn2/1p1p4/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 w - b6 0 9' => {
			eco => 'E08l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b5

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b5'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/2p1pn2/1p1p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 w - b6 0 9' => {
			eco => 'E09b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b5

			variation => N__('Catalan: Closed, Main Line, 8... b5'),
			moves => {
			},
		},
		'r1bq1rk1/p2nbppp/2p1pn2/1p4B1/2BP4/2N1PN2/PP3PPP/2RQK2R w K b6 0 10' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 b5

			variation => N__('QGD: Orthodox, Main Line, Fianchetto Variation'),
			moves => {
				'c4d3' => 'r1bq1rk1/p2nbppp/2p1pn2/1p4B1/3P4/2NBPN2/PP3PPP/2RQK2R b K - 1 10',
			},
		},
		'r1bq1rk1/p2nbppp/2p1pn2/1p4B1/3P4/2NBPN2/PP3PPP/2RQK2R b K - 1 10' => {
			eco => 'D66',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Main Line, Fianchetto Variation'),
			moves => {
				'a7a6' => 'r1bq1rk1/3nbppp/p1p1pn2/1p4B1/3P4/2NBPN2/PP3PPP/2RQK2R w K - 0 11',
			},
		},
		'r1bq1rk1/p2p1ppp/1pn1pn2/2p5/2PP4/P1PBP3/4NPPP/R1BQK2R w KQ - 0 9' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 b6

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 b6'),
			moves => {
				'e3e4' => 'r1bq1rk1/p2p1ppp/1pn1pn2/2p5/2PPP3/P1PB4/4NPPP/R1BQK2R b KQ - 0 9',
			},
		},
		'r1bq1rk1/p2p1ppp/1pn1pn2/2p5/2PPP3/P1PB4/4NPPP/R1BQK2R b KQ - 0 9' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 b6 9. e4

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 b6 9. e4'),
			moves => {
				'f6e8' => 'r1bqnrk1/p2p1ppp/1pn1p3/2p5/2PPP3/P1PB4/4NPPP/R1BQK2R w KQ - 1 10',
			},
		},
		'r1bq1rk1/p2pppbp/1pn2np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 b - - 1 9' => {
			eco => 'B38i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 b6 9. O-O

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 b6 9.O-O'),
			moves => {
				'c8b7' => 'r2q1rk1/pb1pppbp/1pn2np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 2 10',
			},
		},
		'r1bq1rk1/p2pppbp/1pn2np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'B38h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 b6

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 b6'),
			moves => {
				'e1g1' => 'r1bq1rk1/p2pppbp/1pn2np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 b - - 1 9',
			},
		},
		'r1bq1rk1/p3bppp/1pn1p3/8/2BP4/2P2N2/P4PPP/R1BQR1K1 w - - 0 12' => {
			eco => 'D41t',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 7. Bc4 Main Line'),
			moves => {
				'c4d3' => 'r1bq1rk1/p3bppp/1pn1p3/8/3P4/2PB1N2/P4PPP/R1BQR1K1 b - - 1 12',
			},
		},
		'r1bq1rk1/p3bppp/1pn1p3/8/3P4/2PB1N2/P4PPP/R1BQR1K1 b - - 1 12' => {
			eco => 'D41t',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 7. Bc4 Main Line'),
			moves => {
				'c8b7' => 'r2q1rk1/pb2bppp/1pn1p3/8/3P4/2PB1N2/P4PPP/R1BQR1K1 w - - 2 13',
			},
		},
		'r1bq1rk1/p3bppp/1pn1pn2/6B1/3P4/P1NB1N2/1P3PPP/R2QR1K1 b - - 1 12' => {
			eco => 'D42n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6 11. a3 b6 12. Bg5

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6 11. a3 b6 12. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/p3bppp/1pn1pn2/8/3P4/P1N2N2/1PB2PPP/R1BQR1K1 b - - 1 12' => {
			eco => 'D42o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6 11. a3 b6 12. Bc2

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6 11. a3 b6 12. Bc2'),
			moves => {
			},
		},
		'r1bq1rk1/p3bppp/1pn1pn2/8/3P4/P1NB1N2/1P3PPP/R1BQR1K1 w - - 0 12' => {
			eco => 'D42m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6 11. a3 b6

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6 11. a3 b6'),
			moves => {
				'c1e3' => 'r1bq1rk1/p3bppp/1pn1pn2/8/3P4/P1NBBN2/1P3PPP/R2QR1K1 b - - 1 12',
				'c1g5' => 'r1bq1rk1/p3bppp/1pn1pn2/6B1/3P4/P1NB1N2/1P3PPP/R2QR1K1 b - - 1 12',
				'd3c2' => 'r1bq1rk1/p3bppp/1pn1pn2/8/3P4/P1N2N2/1PB2PPP/R1BQR1K1 b - - 1 12',
			},
		},
		'r1bq1rk1/p3bppp/1pn1pn2/8/3P4/P1NBBN2/1P3PPP/R2QR1K1 b - - 1 12' => {
			eco => 'D42m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6 11. a3 b6 12. Be3

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6 11. a3 b6 12. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/p3bppp/2p2n2/3p2B1/8/2NB1Q2/PPP2PPP/R4RK1 w - - 2 12' => {
			eco => 'C47y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5 8. exd5 cxd5 9. O-O O-O 10. Bg5 c6 11. Qf3 Be7

			variation => N__('Four Knights: Scotch, Main Line, 10. Bg5 c6 11. Qf3 Be7'),
			moves => {
			},
		},
		'r1bq1rk1/p3npbp/1pnpp1p1/2p5/4PP2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 0 10' => {
			eco => 'B25s',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 b6

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p3ppbp/1pnp1np1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 b6

			variation => N__('Sicilian: Dragon, Classical, 9. Nb3 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p3ppbp/2p3p1/3N4/8/4BP2/PPPQ2PP/2KR1B1R b - - 0 12' => {
			eco => 'B76r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. exd5 Nxd5 11. Nxc6 bxc6 12. Nxd5

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5: 12.Nxd5'),
			moves => {
			},
		},
		'r1bq1rk1/p3ppbp/2p3p1/3n4/3B4/2N2P2/PPPQ2PP/2KR1B1R b - - 1 12' => {
			eco => 'B76s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. exd5 Nxd5 11. Nxc6 bxc6 12. Bd4

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5: 12. Bd4'),
			moves => {
				'e7e5' => 'r1bq1rk1/p4pbp/2p3p1/3np3/3B4/2N2P2/PPPQ2PP/2KR1B1R w - e6 0 13',
			},
		},
		'r1bq1rk1/p3ppbp/2p3p1/3n4/8/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 12' => {
			eco => 'B76q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5 10.exd5 Nxd5 11.Nxc6'),
			moves => {
				'c3d5' => 'r1bq1rk1/p3ppbp/2p3p1/3N4/8/4BP2/PPPQ2PP/2KR1B1R b - - 0 12',
				'e3d4' => 'r1bq1rk1/p3ppbp/2p3p1/3n4/3B4/2N2P2/PPPQ2PP/2KR1B1R b - - 1 12',
			},
		},
		'r1bq1rk1/p4pbp/2p3p1/2Bnp3/8/2N2P2/PPPQ2PP/2KR1B1R b - - 1 13' => {
			eco => 'B76t',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5: 12. Bd4 e5'),
			moves => {
				'c8e6' => 'r2q1rk1/p4pbp/2p1b1p1/2Bnp3/8/2N2P2/PPPQ2PP/2KR1B1R w - - 2 14',
			},
		},
		'r1bq1rk1/p4pbp/2p3p1/3np3/3B4/2N2P2/PPPQ2PP/2KR1B1R w - e6 0 13' => {
			eco => 'B76t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. exd5 Nxd5 11. Nxc6 bxc6 12. Bd4 e5

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5: 12. Bd4 e5'),
			moves => {
				'd4c5' => 'r1bq1rk1/p4pbp/2p3p1/2Bnp3/8/2N2P2/PPPQ2PP/2KR1B1R b - - 1 13',
			},
		},
		'r1bq1rk1/p4ppp/1pn1p3/8/2BPP3/5N2/P2Q1PPP/3R1RK1 b - - 1 13' => {
			eco => 'D41o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+ 9. Bd2 Bxd2+ 10. Qxd2 O-O 11. Bc4 Nc6 12. O-O b6 13. Rad1

			variation => N__('QGD: Semi-Tarrasch, 6. e4, Main Line, 12... b6'),
			moves => {
			},
		},
		'r1bq1rk1/p4ppp/1pn1p3/8/2BPP3/5N2/P2Q1PPP/R4RK1 w - - 0 13' => {
			eco => 'D41o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+ 9. Bd2 Bxd2+ 10. Qxd2 O-O 11. Bc4 Nc6 12. O-O b6

			variation => N__('QGD: Semi-Tarrasch, 6. e4, Main Line, 12... b6'),
			moves => {
				'a1d1' => 'r1bq1rk1/p4ppp/1pn1p3/8/2BPP3/5N2/P2Q1PPP/3R1RK1 b - - 1 13',
			},
		},
		'r1bq1rk1/p4ppp/1pn1pn2/2pp4/2PP4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 b6

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 b6'),
			moves => {
			},
		},
		'r1bq1rk1/p4ppp/2p2n2/3p2B1/1b6/2NB1Q2/PPP2PPP/R4RK1 b - - 1 11' => {
			eco => 'C47x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5 8. exd5 cxd5 9. O-O O-O 10. Bg5 c6 11. Qf3

			variation => N__('Four Knights: Scotch, Main Line, 10. Bg5 c6 11. Qf3'),
			moves => {
				'b4e7' => 'r1bq1rk1/p3bppp/2p2n2/3p2B1/8/2NB1Q2/PPP2PPP/R4RK1 w - - 2 12',
			},
		},
		'r1bq1rk1/p4ppp/2p2n2/3p2B1/1b6/2NB4/PPP2PPP/R2Q1RK1 w - - 0 11' => {
			eco => 'C47v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5 8. exd5 cxd5 9. O-O O-O 10. Bg5 c6

			variation => N__('Four Knights: Scotch, Main Line, 9.O-O O-O 10. Bg5 c6'),
			moves => {
				'c3a4' => 'r1bq1rk1/p4ppp/2p2n2/3p2B1/Nb6/3B4/PPP2PPP/R2Q1RK1 b - - 1 11',
				'd1f3' => 'r1bq1rk1/p4ppp/2p2n2/3p2B1/1b6/2NB1Q2/PPP2PPP/R4RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/p4ppp/2p2n2/3p2B1/Nb6/3B4/PPP2PPP/R2Q1RK1 b - - 1 11' => {
			eco => 'C47w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5 8. exd5 cxd5 9. O-O O-O 10. Bg5 c6 11. Na4

			variation => N__('Four Knights: Scotch, Main Line, 10. Bg5 c6 11. Na4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2n1p1p1/3pN3/3P1P2/2N1B1P1/PP2P1BP/R2Q1RK1 b - - 2 11' => {
			eco => 'D79m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3 Nfd7 10. f4 Nc6 11. Be3

			variation => N__('Neo-Grünfeld, Main Line, 10. f4 Nc6 11. Be3'),
			moves => {
				'd7b6' => 'r1bq1rk1/pp3pbp/1nn1p1p1/3pN3/3P1P2/2N1B1P1/PP2P1BP/R2Q1RK1 w - - 3 12',
				'f7f6' => 'r1bq1rk1/pp1n2bp/2n1ppp1/3pN3/3P1P2/2N1B1P1/PP2P1BP/R2Q1RK1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp1n1pbp/2n1p1p1/3pN3/3P1P2/2N3P1/PP2P1BP/R1BQ1RK1 w - - 1 11' => {
			eco => 'D79l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3 Nfd7 10. f4 Nc6

			variation => N__('Neo-Grünfeld, Main Line, 10. f4 Nc6'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp1n1pbp/2n1p1p1/3pN3/3P1P2/2N1B1P1/PP2P1BP/R2Q1RK1 b - - 2 11',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/3Pp3/2P1P3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 0 9' => {
			eco => 'E94y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Be3 c6 9. d5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Be3 c6 9. d5'),
			moves => {
				'c6c5' => 'r1bq1rk1/pp1n1pbp/3p1np1/2pPp3/2P1P3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 0 9' => {
			eco => 'E86f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qd2 Nbd7 9. d5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qd2 Nbd7 9. d5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 0 9' => {
			eco => 'E67i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. Qc2 c6

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. Qc2 c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 w - - 0 9' => {
			eco => 'E67l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. h3 c6

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. h3 c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/1PN2NP1/P4PBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E68l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. b3

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 c6 9. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 9' => {
			eco => 'E94x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Be3 c6

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Be3 c6'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp1n1pbp/2pp1np1/3Pp3/2P1P3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/2KR1B1R b - - 3 9' => {
			eco => 'E86g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qd2 Nbd7 9. O-O-O

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qd2 Nbd7 9.O-O-O'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1n1pbp/p1pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/2KR1B1R w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 2 9' => {
			eco => 'E86e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qd2 Nbd7

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qd2 Nbd7'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp1n1pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 0 9',
				'e1c1' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/2KR1B1R b - - 3 9',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/1RBQR1K1 b - - 1 9' => {
			eco => 'E95i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Rb1

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Rb1'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9' => {
			eco => 'E95h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6'),
			moves => {
				'a1b1' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/1RBQR1K1 b - - 1 9',
				'e2f1' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 1 9',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 1 9' => {
			eco => 'E95j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Bf1'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 w - a6 0 10',
				'e5d4' => 'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PpP3/2N2N2/PP3PPP/R1BQRBK1 w - - 0 10',
				'f8e8' => 'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 w - - 2 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 0 9' => {
			eco => 'E94t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Qc2 c6

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Qc2 c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NP1/PP3PBP/1RBQ1RK1 b - - 1 9' => {
			eco => 'E68k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. Rb1

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 c6 9. Rb1'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E68j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 c6'),
			moves => {
				'a1b1' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NP1/PP3PBP/1RBQ1RK1 b - - 1 9',
				'b2b3' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/1PN2NP1/P4PBP/R1BQ1RK1 b - - 0 9',
				'h2h3' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 b - - 0 9' => {
			eco => 'E69a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - a6 0 10',
				'a7a6' => 'r1bq1rk1/1p1n1pbp/p1pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 0 10',
				'd8a5' => 'r1b2rk1/pp1n1pbp/2pp1np1/q3p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10',
				'd8b6' => 'r1b2rk1/pp1n1pbp/1qpp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10',
				'd8e7' => 'r1b2rk1/pp1nqpbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10',
				'e5d4' => 'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PpP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 0 10',
				'f8e8' => 'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 b - - 0 10' => {
			eco => 'E69e',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 exd4'),
			moves => {
				'f8e8' => 'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PNP3/2N5/PP3PPP/R1BQRBK1 b - - 0 10' => {
			eco => 'E95l',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Bf1 exd4'),
			moves => {
				'f6g4' => 'r1bq1rk1/pp1n1pbp/2pp2p1/8/2PNP1n1/2N5/PP3PPP/R1BQRBK1 w - - 1 11',
				'f8e8' => 'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N5/PP3PPP/R1BQRBK1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PpP3/2N2N2/PP3PPP/R1BQRBK1 w - - 0 10' => {
			eco => 'E95l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 exd4

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Bf1 exd4'),
			moves => {
				'f3d4' => 'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PNP3/2N5/PP3PPP/R1BQRBK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PpP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 0 10' => {
			eco => 'E69e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 exd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 exd4'),
			moves => {
				'f3d4' => 'r1bq1rk1/pp1n1pbp/2pp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/2pp2p1/8/2PNP1n1/2N5/PP3PPP/R1BQRBK1 w - - 1 11' => {
			eco => 'E95m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 exd4 10. Nxd4 Ng4

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Bf1 exd4 10.Nxd4 Ng4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Nbd7

			variation => N__('Benoni: Classical, 9.O-O Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 10' => {
			eco => 'A63',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7

			variation => N__('Benoni: Fianchetto, 9...Nbd7'),
			moves => {
				'f3d2' => 'r1bq1rk1/pp1n1pbp/3p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 6 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/3p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 6 10' => {
			eco => 'A63',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2

			variation => N__('Benoni: Fianchetto, 9...Nbd7 10. Nd2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 0 11',
				'f8e8' => 'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 7 11',
			},
		},
		'r1bq1rk1/pp1n1pbp/3p1np1/2pPp3/2P1P3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'E94y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Be3 c6 9. d5 c5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Be3 c6 9. d5 c5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pbp/3p1np1/3Pp3/4P3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 2 10' => {
			eco => 'E89e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Nbd7 10. Qd2

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 10. Qd2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1n1pbp/p2p1np1/3Pp3/4P3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 0 11',
			},
		},
		'r1bq1rk1/pp1n1pbp/3p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 1 10' => {
			eco => 'E89d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Nbd7

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 9...Nbd7'),
			moves => {
				'd1d2' => 'r1bq1rk1/pp1n1pbp/3p1np1/3Pp3/4P3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 2 10',
			},
		},
		'r1bq1rk1/pp1n1pbp/n2p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQ1RK1 b - - 4 11' => {
			eco => 'A67r',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 O-O 10. Nf3 Na6'),
			moves => {
				'a6c7' => 'r1bq1rk1/ppnn1pbp/3p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQ1RK1 w - - 5 12',
			},
		},
		'r1bq1rk1/pp1n1pbp/n2p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQK2R w KQ - 3 11' => {
			eco => 'A67r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 O-O 10. Nf3 Na6

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 O-O 10. Nf3 Na6'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1n1pbp/n2p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQ1RK1 b - - 4 11',
			},
		},
		'r1bq1rk1/pp1n1pbp/n2p2p1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R w KQ - 7 11' => {
			eco => 'A67j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. Bd3 O-O 10. Nf3 Na6

			variation => N__('Benoni: Four Pawns, Taimanov, 9. Bd3 O-O 10. Nf3 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pp1/2p1pb1p/3p4/2PP4/2NBPN2/PP3PPP/2RQ1RK1 b - - 3 10' => {
			eco => 'D55j',
                        # TRANSLATORS:

			variation => N__('QGD: Neo-Orthodox, Gligoric Variation'),
			moves => {
				'd5c4' => 'r1bq1rk1/pp1n1pp1/2p1pb1p/8/2pP4/2NBPN2/PP3PPP/2RQ1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp1n1pp1/2p1pb1p/3p4/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 2 10' => {
			eco => 'D55j',
                        # TRANSLATORS:

			variation => N__('QGD: Neo-Orthodox, Gligoric Variation'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1n1pp1/2p1pb1p/3p4/2PP4/2NBPN2/PP3PPP/2RQ1RK1 b - - 3 10',
			},
		},
		'r1bq1rk1/pp1n1pp1/2p1pb1p/8/2BP4/2N1PN2/PP3PPP/2RQ1RK1 b - - 0 11' => {
			eco => 'D55k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bxf6 Bxf6 8. Rc1 c6 9. Bd3 Nd7 10. O-O dxc4 11. Bxc4

			variation => N__('QGD: Neo-Orthodox, Petrosian Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pp1/2p1pb1p/8/2pP4/2NBPN2/PP3PPP/2RQ1RK1 w - - 0 11' => {
			eco => 'D55j',
                        # TRANSLATORS:

			variation => N__('QGD: Neo-Orthodox, Gligoric Variation'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp1n1pp1/2p1pb1p/8/2BP4/2N1PN2/PP3PPP/2RQ1RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/pp1n1pp1/2pb3p/4p3/2PP4/5N2/PPB2PPP/R1BQ1RK1 w - - 0 12' => {
			eco => 'D46r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxe4 9. Nxe4 Nxe4 10. Bxe4 h6 11. Bc2 e5

			variation => N__('Semi-Slav: Chigorin, Main Line, 10... h6 11. Bc2 e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1pp1/2pbp2p/8/2PP4/5N2/PPB2PPP/R1BQ1RK1 b - - 1 11' => {
			eco => 'D46r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxe4 9. Nxe4 Nxe4 10. Bxe4 h6 11. Bc2

			variation => N__('Semi-Slav: Chigorin, Main Line, 10... h6 11. Bc2'),
			moves => {
				'e6e5' => 'r1bq1rk1/pp1n1pp1/2pb3p/4p3/2PP4/5N2/PPB2PPP/R1BQ1RK1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp1n1pp1/2pbp2p/8/2PPB3/5N2/PP3PPP/R1BQ1RK1 w - - 0 11' => {
			eco => 'D46r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxe4 9. Nxe4 Nxe4 10. Bxe4 h6

			variation => N__('Semi-Slav: Chigorin, Main Line, 10... h6'),
			moves => {
				'e4c2' => 'r1bq1rk1/pp1n1pp1/2pbp2p/8/2PP4/5N2/PPB2PPP/R1BQ1RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/pp1n1ppp/2n1p3/2bpP3/5PQ1/P1N2N2/1PP3PP/R1B1KB1R b KQ - 3 9' => {
			eco => 'C11g',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6.dxc5 Nc6'),
			moves => {
				'f7f6' => 'r1bq1rk1/pp1n2pp/2n1pp2/2bpP3/5PQ1/P1N2N2/1PP3PP/R1B1KB1R w KQ - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/2n1p3/2bpP3/5PQ1/P1N5/1PP3PP/R1B1KBNR w KQ - 2 9' => {
			eco => 'C11g',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6.dxc5 Nc6'),
			moves => {
				'g1f3' => 'r1bq1rk1/pp1n1ppp/2n1p3/2bpP3/5PQ1/P1N2N2/1PP3PP/R1B1KB1R b KQ - 3 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2p1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8' => {
			eco => 'D46b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bb4 7. O-O O-O

			variation => N__('Semi-Slav: Romih, 7.O-O O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/2pb1n2/3pp3/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 0 9' => {
			eco => 'D45w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. Be2 O-O 8. O-O e5

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Be2 O-O 8.O-O e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/2pb1n2/4p3/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - - 0 10' => {
			eco => 'D46i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 e5

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/2pb1n2/4p3/2BPP3/2N2N2/PP3PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D46p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxc4 9. Bxc4 e5

			variation => N__('Semi-Slav: Chigorin, 7.O-O O-O 8. e4 dxc4 9.Bxc4 e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbp3/8/2PPB3/5N2/PP3PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D46q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxe4 9. Nxe4 Nxe4 10. Bxe4

			variation => N__('Semi-Slav: Chigorin, Main Line'),
			moves => {
				'h7h6' => 'r1bq1rk1/pp1n1pp1/2pbp2p/8/2PPB3/5N2/PP3PPP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbp3/8/2PPn3/3B1N2/PP3PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D46q',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Chigorin, Main Line'),
			moves => {
				'd3e4' => 'r1bq1rk1/pp1n1ppp/2pbp3/8/2PPB3/5N2/PP3PPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q1BPPP/R1B1K2R b KQ - 2 8' => {
			eco => 'D45r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. b3 O-O 8. Be2

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. b3 O-O 8. Be2'),
			moves => {
				'b7b6' => 'r1bq1rk1/p2n1ppp/1ppbpn2/3p4/2PP4/1PN1PN2/P1Q1BPPP/R1B1K2R w KQ - 0 9',
				'f8e8' => 'r1bqr1k1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q1BPPP/R1B1K2R w KQ - 3 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q2PPP/R1B1KB1R w KQ - 1 8' => {
			eco => 'D45p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. b3 O-O

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. b3 O-O'),
			moves => {
				'c1b2' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/PBQ2PPP/R3KB1R b KQ - 2 8',
				'f1e2' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q1BPPP/R1B1K2R b KQ - 2 8',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/PBQ2PPP/R3KB1R b KQ - 2 8' => {
			eco => 'D45q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. b3 O-O 8. Bb2

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. b3 O-O 8. Bb2'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B1K2R w KQ - 5 8' => {
			eco => 'D45v',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Be2'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 b - - 6 8',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 b - - 6 8' => {
			eco => 'D45v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. Be2 O-O 8. O-O

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Be2 O-O 8.O-O'),
			moves => {
				'd8e7' => 'r1b2rk1/pp1nqppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 7 9',
				'e6e5' => 'r1bq1rk1/pp1n1ppp/2pb1n2/3pp3/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 7 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8' => {
			eco => 'D46o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O

			variation => N__('Semi-Slav: Chigorin, 7.O-O O-O'),
			moves => {
				'e3e4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PPP3/2NB1N2/PP3PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PPQ2PPP/R1B1K2R w KQ - 5 8' => {
			eco => 'D46h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 O-O'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PPQ2PPP/R1B2RK1 b - - 6 8',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PPQ2PPP/R1B2RK1 b - - 6 8' => {
			eco => 'D46h',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 O-O'),
			moves => {
				'd5c4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/8/2pP4/2NBPN2/PPQ2PPP/R1B2RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PPP3/2NB1N2/PP3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'D46p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4

			variation => N__('Semi-Slav: Chigorin, 7.O-O O-O 8. e4'),
			moves => {
				'd5c4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/8/2pPP3/2NB1N2/PP3PPP/R1BQ1RK1 w - - 0 9',
				'd5e4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/8/2PPp3/2NB1N2/PP3PPP/R1BQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 b - - 0 9' => {
			eco => 'D46i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1n1ppp/p1pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - - 0 10',
				'b7b5' => 'r1bq1rk1/p2n1ppp/2pbpn2/1p6/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - b6 0 10',
				'd8e7' => 'r1b2rk1/pp1nqppp/2pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - - 1 10',
				'e6e5' => 'r1bq1rk1/pp1n1ppp/2pb1n2/4p3/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/8/2BPP3/2N2N2/PP3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D46p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxc4 9. Bxc4

			variation => N__('Semi-Slav: Chigorin, 7.O-O O-O 8. e4 dxc4 9.Bxc4'),
			moves => {
				'e6e5' => 'r1bq1rk1/pp1n1ppp/2pb1n2/4p3/2BPP3/2N2N2/PP3PPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/8/2PPN3/3B1N2/PP3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D46q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxe4 9. Nxe4

			variation => N__('Semi-Slav: Chigorin, Main Line'),
			moves => {
				'f6e4' => 'r1bq1rk1/pp1n1ppp/2pbp3/8/2PPn3/3B1N2/PP3PPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/8/2PPp3/2NB1N2/PP3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'D46q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxe4

			variation => N__('Semi-Slav: Chigorin, Main Line'),
			moves => {
				'c3e4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/8/2PPN3/3B1N2/PP3PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/8/2pP4/2NBPN2/PPQ2PPP/R1B2RK1 w - - 0 9' => {
			eco => 'D46h',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 O-O'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B2RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/2pbpn2/8/2pPP3/2NB1N2/PP3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'D46p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O O-O 8. e4 dxc4

			variation => N__('Semi-Slav: Chigorin, 7.O-O O-O 8. e4 dxc4'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp1n1ppp/2pbpn2/8/2BPP3/2N2N2/PP3PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1B2RK1 b - - 2 9' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. Qe2

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. Qe2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1n1ppp/p3pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1B2RK1 w - - 0 10',
				'b7b6' => 'r1bq1rk1/p2n1ppp/1p2pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1B2RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7

			variation => N__('Nimzo-Indian: Main Line, Bronstein Variation'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/P1N1PN2/1P3PPP/R1BQ1RK1 b - - 0 9',
				'c3e2' => 'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/4PN2/PP2NPPP/R1BQ1RK1 b - - 2 9',
				'd1e2' => 'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/2N1PN2/PP2QPPP/R1B2RK1 b - - 2 9',
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/4PN2/PP2NPPP/R1BQ1RK1 b - - 2 9' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. Ne2

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. Ne2'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/P1N1PN2/1P3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. a3

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. a3'),
			moves => {
				'c5d4' => 'r1bq1rk1/pp1n1ppp/4pn2/8/1bBp4/P1N1PN2/1P3PPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nbd7

			variation => N__('Nimzo-Indian: Main Line, 7...Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/8/1bBP4/P1N2N2/1P3PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E55',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nbd7 9. a3 cxd4 10. exd4

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. a3 cd4 10.ed4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n1ppp/4pn2/8/1bBp4/P1N1PN2/1P3PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E55',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Main Line, Bronstein, 9. a3'),
			moves => {
				'e3d4' => 'r1bq1rk1/pp1n1ppp/4pn2/8/1bBP4/P1N2N2/1P3PPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp1n2bp/2n1ppp1/3pN3/3P1P2/2N1B1P1/PP2P1BP/R2Q1RK1 w - - 0 12' => {
			eco => 'D79m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3 Nfd7 10. f4 Nc6 11. Be3 f6

			variation => N__('Neo-Grünfeld, Main Line, 10. f4 Nc6 11. Be3 f6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n2pp/2n1pp2/2bpP3/5PQ1/P1N2N2/1PP3PP/R1B1KB1R w KQ - 0 10' => {
			eco => 'C11g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. dxc5 Nc6 7. a3 Bxc5 8. Qg4 O-O 9. Nf3 f6

			variation => N__('French: Steinitz, Brodsky-Jones Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1n2pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R2Q1RK1 w - - 1 10' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Bxa3 9. Nxa3 Nbd7

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Bxa3 9.Nxa3 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nB1pp/2p1pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RN1Q1RK1 b - - 0 9' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Nbd7 9. Bxe7

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Nbd7 9.Bxe7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nBppp/2p1p3/3n4/2BP4/2N1PN2/PP3PPP/2RQK2R b K - 0 10' => {
			eco => 'D67',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Main Line, Capablanca Freeing Manoevure'),
			moves => {
				'd8e7' => 'r1b2rk1/pp1nqppp/2p1p3/3n4/2BP4/2N1PN2/PP3PPP/2RQK2R w K - 0 11',
			},
		},
		'r1bq1rk1/pp1nBppp/4p3/2pn4/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 0 9' => {
			eco => 'D62',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 Nxd5'),
			moves => {
				'd8e7' => 'r1b2rk1/pp1nqppp/4p3/2pn4/3P4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 10',
			},
		},
		'r1bq1rk1/pp1nb1pp/2p1pn2/3p1p2/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 w - - 2 9' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Nbd7

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Nbd7'),
			moves => {
				'a3e7' => 'r1bq1rk1/pp1nB1pp/2p1pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RN1Q1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p1pn1p/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 h6

			variation => N__('QGD: Orthodox, Rubinstein Attack, 8... h6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbpp1/2p1pn1p/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 0 9' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 h6

			variation => N__('QGD: Orthodox, Main Line, 8... h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pp1nbpp1/2p1pn1p/3p4/2PP3B/2NBPN2/PP3PPP/2RQK2R b K - 1 9',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p1pn1p/3p4/2PP3B/2NBPN2/PP3PPP/2RQK2R b K - 1 9' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 h6 9. Bh4

			variation => N__('QGD: Orthodox, Main Line, 8... h6 9. Bh4'),
			moves => {
				'd5c4' => 'r1bq1rk1/pp1nbpp1/2p1pn1p/8/2pP3B/2NBPN2/PP3PPP/2RQK2R w K - 0 10',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p1pn1p/8/2BP3B/2N1PN2/PP3PPP/2RQK2R b K - 0 10' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 h6 9. Bh4 dxc4 10. Bxc4

			variation => N__('QGD: Orthodox, Main Line, 8... h6 9. Bh4 dxc4 10.Bxc4'),
			moves => {
				'b7b5' => 'r1bq1rk1/p2nbpp1/2p1pn1p/1p6/2BP3B/2N1PN2/PP3PPP/2RQK2R w K b6 0 11',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p1pn1p/8/2pP3B/2NBPN2/PP3PPP/2RQK2R w K - 0 10' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 h6 9. Bh4 dxc4

			variation => N__('QGD: Orthodox, Main Line, 8... h6 9. Bh4 dxc4'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp1nbpp1/2p1pn1p/8/2BP3B/2N1PN2/PP3PPP/2RQK2R b K - 0 10',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBP3/PPQ1NPPP/R3K2R w KQ - 0 10' => {
			eco => 'D36k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2 h6

			variation => N__('QGD: Exchange, Main Line, 9.Nge2 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBP3/PPQ1NPPP/R3K2R b KQ - 1 10',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/R3K2R w KQ - 0 10' => {
			eco => 'D36o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 h6

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/R3K2R b KQ - 1 10',
			},
		},
		'r1bq1rk1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBP3/PPQ1NPPP/R3K2R b KQ - 1 10' => {
			eco => 'D36k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2 h6 10. Bh4

			variation => N__('QGD: Exchange, Main Line, 9.Nge2 h6 10. Bh4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/R3K2R b KQ - 1 10' => {
			eco => 'D36o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 h6 10. Bh4

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 h6 10. Bh4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbpp1/4pn1p/2pp4/2PP3B/2N1PN2/PPQ2PPP/R3KB1R w KQ c6 0 9' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 h6 8. Bh4 c5

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 h6 8. Bh4 c5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1p3/3n2B1/2BP3P/2N1PN2/PP3PP1/2RQK2R b K h3 0 10' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5 10. h4

			variation => N__('QGD: Orthodox, Main Line, Janowski Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1p3/3n2B1/2BP4/2N1PN2/PP3PPP/2RQK2R w K - 1 10' => {
			eco => 'D67',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4 Nd5

			variation => N__('QGD: Orthodox, Main Line, Capablanca Freeing Manoevure'),
			moves => {
				'g5e7' => 'r1bq1rk1/pp1nBppp/2p1p3/3n4/2BP4/2N1PN2/PP3PPP/2RQK2R b K - 0 10',
				'h2h4' => 'r1bq1rk1/pp1nbppp/2p1p3/3n2B1/2BP3P/2N1PN2/PP3PP1/2RQK2R b K h3 0 10',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1p3/3p2B1/2PPn3/2N1PN2/PPQ2PPP/2R1KB1R w K - 2 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 Ne4

			variation => N__('QGD: Orthodox, Rubinstein Attack, Wolf Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1p3/3p3n/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 w - - 2 9' => {
			eco => 'E08h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Bf4 Nh5

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. Bf4 Nh5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1p3/3p4/2PPnB2/5NP1/PPQ1PPBP/RN3RK1 w - - 2 9' => {
			eco => 'E08i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Bf4 Ne4

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. Bf4 Ne4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/1QN1PN2/PP3PPP/R3KB1R w KQ - 0 8' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qb3 c6

			variation => N__('QGD: Orthodox, Rauzer Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6

			variation => N__('QGD: Orthodox, 7. Rc1 c6'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/P1N1PN2/1P3PPP/2RQKB1R b K - 0 8',
				'd1c2' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R b K - 1 8',
				'f1d3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R b K - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R b K - 1 8' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2

			variation => N__('QGD: Orthodox, Rubinstein Attack'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p1p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 9',
				'd5c4' => 'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2pP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 9',
				'f6e4' => 'r1bq1rk1/pp1nbppp/2p1p3/3p2B1/2PPn3/2N1PN2/PPQ2PPP/2R1KB1R w K - 2 9',
				'f8e8' => 'r1bqr1k1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 2 9',
				'h7h6' => 'r1bq1rk1/pp1nbpp1/2p1pn1p/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/3RKB1R b K - 1 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c6 8. Rd1

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c6 8. Rd1'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c6

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c6'),
			moves => {
				'a1d1' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/3RKB1R b K - 1 8',
				'a2a3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/R3KB1R b KQ - 0 8',
				'f1d3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PPQ2PPP/R3K2R b KQ - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R b K - 1 8' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3

			variation => N__('QGD: Orthodox, Main Line'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p1p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 0 9',
				'd5c4' => 'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2pP4/2NBPN2/PP3PPP/2RQK2R w K - 0 9',
				'f8e8' => 'r1bqr1k1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 2 9',
				'h7h6' => 'r1bq1rk1/pp1nbpp1/2p1pn1p/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 8' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 c6

			variation => N__('QGD: Orthodox, Botvinnik, 7... c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PPQ2PPP/R3K2R b KQ - 1 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c6 8. Bd3

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c6 8. Bd3'),
			moves => {
				'd5c4' => 'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2pP4/2NBPN2/PPQ2PPP/R3K2R w KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/P1N1PN2/1P3PPP/2RQKB1R b K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. a3

			variation => N__('QGD: Orthodox, 7. Rc1 c6 8. a3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/R3KB1R b KQ - 0 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c6 8. a3

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c6 8. a3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 b - - 1 8' => {
			eco => 'E08g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Bf4

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. Bf4'),
			moves => {
				'b7b6' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 w - - 0 9',
				'f6e4' => 'r1bq1rk1/pp1nbppp/2p1p3/3p4/2PPnB2/5NP1/PPQ1PPBP/RN3RK1 w - - 2 9',
				'f6h5' => 'r1bq1rk1/pp1nbppp/2p1p3/3p3n/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 w - - 2 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 b - - 0 8' => {
			eco => 'E08k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3'),
			moves => {
				'b7b5' => 'r1bq1rk1/p2nbppp/2p1pn2/1p1p4/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 w - b6 0 9',
				'b7b6' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. b3 c6

			variation => N__('Catalan: Closed, 6...Nbd7 7. b3'),
			moves => {
				'c1a3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 b - - 1 8',
				'c1b2' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 1 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. b3 c6 8. Bb2

			variation => N__('Catalan: Closed, 6...Nbd7 7. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3 c6 8. b3

			variation => N__('Catalan: Closed, 6...Nbd7 7. Nc3 c6 8. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1PNBPN2/P4PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'D46d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Be7 7. O-O O-O 8. b3

			variation => N__('Semi-Slav: Bogoljubow, 7.O-O O-O 8. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1QN2NP1/PP2PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3 c6 8. Qb3

			variation => N__('Catalan: Closed, 6...Nbd7 7. Nc3 c6 8. Qb3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3 c6

			variation => N__('Catalan: Closed, 6...Nbd7 7. Nc3 c6'),
			moves => {
				'b2b3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'd1b3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1QN2NP1/PP2PPBP/R1B2RK1 b - - 1 8',
				'd1d3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/2NQ1NP1/PP2PPBP/R1B2RK1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8' => {
			eco => 'D46c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Be7 7. O-O O-O

			variation => N__('Semi-Slav: Bogoljubow, 7.O-O O-O'),
			moves => {
				'b2b3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1PNBPN2/P4PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/2NQ1NP1/PP2PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3 c6 8. Qd3

			variation => N__('Catalan: Closed, Botvinnik Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8' => {
			eco => 'E08d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6

			variation => N__('Catalan: Closed, 7. Qc2 c6'),
			moves => {
				'b1d2' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 b - - 1 8',
				'b2b3' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 b - - 0 8',
				'c1f4' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 b - - 1 8',
				'f1d1' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNBR2K1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNBR2K1 b - - 1 8' => {
			eco => 'E08e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Rd1

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. Rd1'),
			moves => {
				'b7b6' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNBR2K1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 b - - 1 8' => {
			eco => 'E09a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2

			variation => N__('Catalan: Closed, Main Line'),
			moves => {
				'b7b5' => 'r1bq1rk1/p2nbppp/2p1pn2/1p1p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 w - b6 0 9',
				'b7b6' => 'r1bq1rk1/p2nbppp/1pp1pn2/3p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 w - - 2 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 b - - 1 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. b3 c6 8. Ba3

			variation => N__('Catalan: Closed, 6...Nbd7 7. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2BP4/2N1PN2/PP3PPP/2RQK2R b K - 0 9' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4 9. Bxc4

			variation => N__('QGD: Orthodox, Main Line, 8...dxc4'),
			moves => {
				'b7b5' => 'r1bq1rk1/p2nbppp/2p1pn2/1p4B1/2BP4/2N1PN2/PP3PPP/2RQK2R w K b6 0 10',
				'f6d5' => 'r1bq1rk1/pp1nbppp/2p1p3/3n2B1/2BP4/2N1PN2/PP3PPP/2RQK2R w K - 1 10',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2BP4/2N1PN2/PPQ2PPP/R3K2R b KQ - 0 9' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c6 8. Bd3 dxc4 9. Bxc4

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c6 8. Bd3 dxc4 9.Bxc4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2pP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 0 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 dxc4

			variation => N__('QGD: Orthodox, Rubinstein Attack, 8...dxc4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2pP4/2NBPN2/PP3PPP/2RQK2R w K - 0 9' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 dxc4

			variation => N__('QGD: Orthodox, Main Line, 8...dxc4'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2BP4/2N1PN2/PP3PPP/2RQK2R b K - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2pP4/2NBPN2/PPQ2PPP/R3K2R w KQ - 0 9' => {
			eco => 'D61',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c6 8. Bd3'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp1nbppp/2p1pn2/6B1/2BP4/2N1PN2/PPQ2PPP/R3K2R b KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R3K2R b KQ - 4 9' => {
			eco => 'D36k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2

			variation => N__('QGD: Exchange, Main Line, 9.Nge2'),
			moves => {
				'f8e8' => 'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R3K2R w KQ - 5 10',
				'h7h6' => 'r1bq1rk1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBP3/PPQ1NPPP/R3K2R w KQ - 0 10',
			},
		},
		'r1bq1rk1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR w KQ - 3 9' => {
			eco => 'D36k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O

			variation => N__('QGD: Exchange, Main Line'),
			moves => {
				'g1e2' => 'r1bq1rk1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R3K2R b KQ - 4 9',
				'g1f3' => 'r1bq1rk1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R3K2R b KQ - 4 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R3K2R b KQ - 4 9' => {
			eco => 'D36o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3

			variation => N__('QGD: Exchange, Main Line, 9. Nf3'),
			moves => {
				'f8e8' => 'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R3K2R w KQ - 5 10',
				'h7h6' => 'r1bq1rk1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/R3K2R w KQ - 0 10',
			},
		},
		'r1bq1rk1/pp1nbppp/2p4n/3pp1N1/2BPPP2/2P5/PP4PP/RNBQ1RK1 w - - 0 9' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6 5. Ng5 Nh6 6. f4 Be7 7. O-O O-O 8. c3 d5

			variation => N__('Philidor: Hanham, Berger Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p1B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 Be7 6. e3 O-O 7. Qc2 c6

			variation => N__('Old Indian: 5. Bg5 Be7 6. e3 O-O 7. Qc2 c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2BPP3/2N2N2/PPP1QPPP/R1B2RK1 w - - 0 8' => {
			eco => 'C41w',
                        # TRANSLATORS:

			variation => N__('Philidor: Improved Hanham, 7. Qe2'),
			moves => {
				'a2a4' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP1QPPP/R1B2RK1 b - a3 0 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQR1K1 w - - 0 8' => {
			eco => 'C41x',
                        # TRANSLATORS:

			variation => N__('Philidor: Improved Hanham, 7. Re1'),
			moves => {
				'a2a4' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQR1K1 b - a3 0 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 Be7 6. Bg2 O-O 7. O-O c6

			variation => N__('Old Indian: 5. g3 Be7, Main Line'),
			moves => {
				'd1c2' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 Be7 6. Bg2 O-O 7. O-O c6 8. Qc2

			variation => N__('Old Indian: 5. g3 Be7, Main Line, 8. Qc2'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8' => {
			eco => 'A55m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Be3

			variation => N__('Old Indian: Main Line, 8. Be3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N1P/PP2BPP1/R1BQ1RK1 b - - 0 8' => {
			eco => 'A55l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. h3

			variation => N__('Old Indian: Main Line, 8. h3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'A55k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O

			variation => N__('Old Indian: Main Line'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8',
				'd1c2' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 3 8',
				'f1e1' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 b - - 3 8',
				'h2h3' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N1P/PP2BPP1/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 b - - 3 8' => {
			eco => 'A55q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Re1

			variation => N__('Old Indian: Main Line, 8. Re1'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 4 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 3 8' => {
			eco => 'A55o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Qc2

			variation => N__('Old Indian: Main Line, 8. Qc2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 0 9',
				'd8c7' => 'r1b2rk1/ppqnbppp/2pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 4 9',
				'f8e8' => 'r1bqr1k1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 4 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - - 3 8' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8. O-O

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8.O-O'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9',
				'd8c7' => 'r1b2rk1/ppqnbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 4 9',
				'f8e8' => 'r1bqr1k1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 4 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQK2R w KQ - 2 8' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2 O-O

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2 O-O'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - - 3 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP1QPPP/R1B2RK1 b - - 1 8' => {
			eco => 'C41w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. a4 c6 8. Qe2

			variation => N__('Philidor: Improved Hanham, 7. Qe2 c6 8. a4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP1QPPP/R1B2RK1 b - a3 0 8' => {
			eco => 'C41w',
                        # TRANSLATORS:

			variation => N__('Philidor: Improved Hanham, 7. Qe2'),
			moves => {
				'e5d4' => 'r1bq1rk1/pp1nbppp/2pp1n2/8/P1BpP3/2N2N2/1PP1QPPP/R1B2RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'C41v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. a4 c6

			variation => N__('Philidor: Improved Hanham, 7. a4 c6'),
			moves => {
				'd1e2' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP1QPPP/R1B2RK1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQR1K1 b - a3 0 8' => {
			eco => 'C41x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. Re1 c6 8. a4

			variation => N__('Philidor: Improved Hanham, 7. Re1 c6 8. a4'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p1nbppp/2pp1n2/p3p3/P1BPP3/2N2N2/1PP2PPP/R1BQR1K1 w - a6 0 9',
				'b7b6' => 'r1bq1rk1/p2nbppp/1ppp1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQR1K1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp1n2/8/P1BpP3/2N2N2/1PP1QPPP/R1B2RK1 w - - 0 9' => {
			eco => 'C41w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. Qe2 c6 8. a4 exd4

			variation => N__('Philidor: Improved Hanham, Sozin Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/2pp3n/4p1N1/2BPPP2/2P5/PP4PP/RNBQ1RK1 b - - 0 8' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'd6d5' => 'r1bq1rk1/pp1nbppp/2p4n/3pp1N1/2BPPP2/2P5/PP4PP/RNBQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQ1RK1 w - - 3 8' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'c2c3' => 'r1bq1rk1/pp1nbppp/2pp3n/4p1N1/2BPPP2/2P5/PP4PP/RNBQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1nbppp/4p3/2pn2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 9' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 Nxd5

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 Nxd5'),
			moves => {
				'g5e7' => 'r1bq1rk1/pp1nBppp/4p3/2pn4/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2p3B1/2BP4/2N1PN2/PP3PPP/R2Q1RK1 b - - 1 9' => {
			eco => 'D60',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Botvinnik, 8... c5'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nbppp/p3pn2/2p3B1/2BP4/2N1PN2/PP3PPP/R2Q1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2p3B1/2BP4/2N1PN2/PP3PPP/R2QK2R w KQ c6 0 9' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 dxc4 8. Bxc4 c5

			variation => N__('QGD: Orthodox, Botvinnik, 8... c5'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1nbppp/4pn2/2p3B1/2BP4/2N1PN2/PP3PPP/R2Q1RK1 b - - 1 9',
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2pP2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 0 8' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5'),
			moves => {
				'c5d4' => 'r1bq1rk1/pp1nbppp/4pn2/3P2B1/3p4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 9',
				'e6d5' => 'r1bq1rk1/pp1nbppp/5n2/2pp2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 9',
				'f6d5' => 'r1bq1rk1/pp1nbppp/4p3/2pn2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2pp2B1/2PP4/2N1PN2/PPQ2PPP/2KR1B1R b - - 1 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. O-O-O

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.O-O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2pp2B1/2PP4/2N1PN2/PPQ2PPP/3RKB1R b K - 1 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. Rd1

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8. Rd1'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2pp2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ c6 0 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5'),
			moves => {
				'a1d1' => 'r1bq1rk1/pp1nbppp/4pn2/2pp2B1/2PP4/2N1PN2/PPQ2PPP/3RKB1R b K - 1 8',
				'c4d5' => 'r1bq1rk1/pp1nbppp/4pn2/2pP2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 0 8',
				'e1c1' => 'r1bq1rk1/pp1nbppp/4pn2/2pp2B1/2PP4/2N1PN2/PPQ2PPP/2KR1B1R b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/2pp4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - c6 0 8' => {
			eco => 'E08b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c5

			variation => N__('Catalan: Closed, 7. Qc2 c5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/4pn2/3P2B1/3p4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 9' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 cxd4

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 cxd4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nbppp/5n2/2pp2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 9' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 exd5

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 exd5'),
			moves => {
				'f1d3' => 'r1bq1rk1/pp1nbppp/5n2/2pp2B1/3P4/2NBPN2/PPQ2PPP/R3K2R b KQ - 1 9',
			},
		},
		'r1bq1rk1/pp1nbppp/5n2/2pp2B1/3P4/2NBPN2/PPQ2PPP/R3K2R b KQ - 1 9' => {
			eco => 'D62',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 c5 8. cxd5 exd5 9. Bd3

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 c5 8.cxd5 exd5 9. Bd3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nn1bp/3p2p1/2pPpp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 w - c6 0 12' => {
			eco => 'E98o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Nd3 f5 11. Bd2 c5

			variation => N__('King\'s Indian: Mar del Plata, Fischer Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2np2p1/8/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 5 10' => {
			eco => 'B77d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Nd7

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Nd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2np2p1/8/2P1P3/2N5/PPN1BPPP/R1BQ1RK1 w - - 4 10' => {
			eco => 'B37n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O Nd7

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O Nd7'),
			moves => {
				'c1d2' => 'r1bq1rk1/pp1nppbp/2np2p1/8/2P1P3/2N5/PPNBBPPP/R2Q1RK1 b - - 5 10',
			},
		},
		'r1bq1rk1/pp1nppbp/2np2p1/8/2P1P3/2N5/PPNBBPPP/R2Q1RK1 b - - 5 10' => {
			eco => 'B37o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O Nd7 10. Bd2

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O Nd7 10. Bd2'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p1nppbp/2np2p1/p7/2P1P3/2N5/PPNBBPPP/R2Q1RK1 w - a6 0 11',
				'd7c5' => 'r1bq1rk1/pp2ppbp/2np2p1/2n5/2P1P3/2N5/PPNBBPPP/R2Q1RK1 w - - 6 11',
			},
		},
		'r1bq1rk1/pp1nppbp/2np2p1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 2 10' => {
			eco => 'B38o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Nd7

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Nd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2p2np1/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D78l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Nbd2 Nbd7

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.Nbd2 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2pp1np1/6B1/2PP4/2N1PN2/PP2BPPP/R2Q1RK1 b - - 1 8' => {
			eco => 'E61z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3 Nbd7 7. Be2 c6 8. O-O

			variation => N__('King\'s Indian: Smyslov System, 5...O-O 6. e3 Nbd7 7. Be2 c6 8.O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2pp1np1/6B1/2PP4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 8' => {
			eco => 'E61z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3 Nbd7 7. Be2 c6

			variation => N__('King\'s Indian: Smyslov System, 5...O-O 6. e3 Nbd7 7. Be2 c6'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1nppbp/2pp1np1/6B1/2PP4/2N1PN2/PP2BPPP/R2Q1RK1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp1nppbp/2pp1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR w KQ - 2 8' => {
			eco => 'E73l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c6 7. Qd2 Nbd7

			variation => N__('King\'s Indian: Averbakh, 6... c6 7. Qd2 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E67c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O c6

			variation => N__('King\'s Indian: Fianchetto with 6... Nd7 7.O-O c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E91t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nbd7 7. O-O c6

			variation => N__('King\'s Indian: 6. Be2 Nbd7 7.O-O c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/2pp1np1/8/P2PP3/2N2N2/1PP1BPPP/R1BQ1RK1 w - - 1 8' => {
			eco => 'B08s',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O c6 7. a4 Nbd7

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O c6 7. a4 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/3p1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E65c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nbd7

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7.O-O Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/3p1np1/2p5/2PPP3/2N1BP2/PP1Q2PP/R3KBNR w KQ c6 0 8' => {
			eco => 'E81s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nbd7 7. Qd2 c5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 Nbd7 7. Qd2 c5'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp1nppbp/3p1np1/2pP4/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 0 8',
				'g1e2' => 'r1bq1rk1/pp1nppbp/3p1np1/2p5/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 1 8',
			},
		},
		'r1bq1rk1/pp1nppbp/3p1np1/2p5/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 1 8' => {
			eco => 'E81t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nbd7 7. Qd2 c5 8. Nge2

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 Nbd7 7. Qd2 c5 8.Nge2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1nppbp/p2p1np1/2p5/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1nppbp/3p1np1/2pP4/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 0 8' => {
			eco => 'E81s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nbd7 7. Qd2 c5 8. d5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 Nbd7 7. Qd2 c5 8. d5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/5np1/2pp2B1/3P4/2PBPN2/PP1N1PPP/R2QK2R w KQ - 1 8' => {
			eco => 'D03u',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7 5. e3 O-O 6. Bd3 c5 7. c3 Nbd7

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. Bd3 c5 7. c3 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1nppbp/5np1/3P4/3P4/1QN5/PP2BPPP/R1B1K1NR w KQ - 3 9' => {
			eco => 'B14h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. Qb3 Bg7 7. cxd5 O-O 8. Be2 Nbd7

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6, Main Line, 8. Be2 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/2n1pn2/2b5/2P5/P1N2N2/1PQ1PPPP/R1B1KB1R w KQ - 2 8' => {
			eco => 'E39k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. a3 Bxc5 7. Nf3 Nc6

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. a3 Bxc5 7. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 3 7' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 c5 6. Bd3 Nc6 7. O-O

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 c5 6. Bd3 Nc6 7.O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 2 7' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 c5 6. Bd3 Nc6

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 c5 6. Bd3 Nc6'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 3 7',
			},
		},
		'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBP3/4NPPP/R1BQK2R b KQ - 3 8' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2'),
			moves => {
				'b7b6' => 'r1bq1rk1/p2p1ppp/1pn1pn2/2p5/2PP4/P1PBP3/4NPPP/R1BQK2R w KQ - 0 9',
				'd7d6' => 'r1bq1rk1/pp3ppp/2nppn2/2p5/2PP4/P1PBP3/4NPPP/R1BQK2R w KQ - 0 9',
				'e6e5' => 'r1bq1rk1/pp1p1ppp/2n2n2/2p1p3/2PP4/P1PBP3/4NPPP/R1BQK2R w KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR w KQ - 2 8' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6'),
			moves => {
				'g1e2' => 'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBP3/4NPPP/R1BQK2R b KQ - 3 8',
				'g1f3' => 'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBPN2/5PPP/R1BQK2R b KQ - 3 8',
			},
		},
		'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBPN2/5PPP/R1BQK2R b KQ - 3 8' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Nf3

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6 8. Nf3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/2n2n2/2p1p3/2PP4/P1PBP3/4NPPP/R1BQK2R w KQ - 0 9' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 e5

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/2N2N2/PPQ1PPPP/R1B1KB1R w KQ - 3 7' => {
			eco => 'E39e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Nf3 Na6

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Nf3 Na6'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/P1N2N2/1PQ1PPPP/R1B1KB1R b KQ - 0 7',
				'c1d2' => 'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/2N2N2/PPQBPPPP/R3KB1R b KQ - 4 7',
				'g2g3' => 'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/2N2NP1/PPQ1PP1P/R1B1KB1R b KQ - 0 7',
			},
		},
		'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/2N2N2/PPQBPPPP/R3KB1R b KQ - 4 7' => {
			eco => 'E39f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Nf3 Na6 7. Bd2

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Nf3 Na6 7. Bd2'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/2N2NP1/PPQ1PP1P/R1B1KB1R b KQ - 0 7' => {
			eco => 'E39h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Nf3 Na6 7. g3

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Nf3 Na6 7. g3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/P1N2N2/1PQ1PPPP/R1B1KB1R b KQ - 0 7' => {
			eco => 'E39g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Nf3 Na6 7. a3

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Nf3 Na6 7. a3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 w - - 4 8' => {
			eco => 'A36p',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Botvinnik, 5... e6'),
			moves => {
				'd2d3' => 'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A36q',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 e6 6. Nge2 Nge7 7. O-O O-O 8. d3

			variation => N__('English: Symmetrical, Botvinnik, 5... e6, 8. d3'),
			moves => {
				'd7d6' => 'r1bq1rk1/pp2npbp/2npp1p1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P5/2NP1NP1/PP1BPPBP/R2Q1RK1 b - - 2 8' => {
			eco => 'A37p',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. O-O Nge7 7. d3 O-O 8. Bd2

			variation => N__('English: Symmetrical, 5. Nf3 e6 6.O-O Nge7 7. d3 O-O 8. Bd2'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 1 8' => {
			eco => 'A37o',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. O-O Nge7 7. d3 O-O

			variation => N__('English: Symmetrical, 5. Nf3 e6 6.O-O Nge7 7. d3 O-O'),
			moves => {
				'c1d2' => 'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P5/2NP1NP1/PP1BPPBP/R2Q1RK1 b - - 2 8',
			},
		},
		'r1bq1rk1/pp1pnpbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 1 8' => {
			eco => 'A37y',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O Nge7 7. d3 O-O

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O Nge7 7. d3 O-O'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp1pnpbp/2n3p1/2p1p3/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1pnpbp/2n3p1/2p1p3/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A37y',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O Nge7 7. d3 O-O 8. a3

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O Nge7 7. d3 O-O 8. a3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/1Bp5/3PP3/2P2N2/PP3PPP/RNBQR1K1 b - d3 0 7' => {
			eco => 'B31n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 Nf6 6. c3 O-O 7. d4

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 Nf6 6. c3 O-O 7. d4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/1Bp5/4P3/2P2N1P/PP1P1PP1/RNBQR1K1 b - - 0 7' => {
			eco => 'B31m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 Nf6 6. c3 O-O 7. h3

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 Nf6 6. c3 O-O 7. h3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 w - - 1 7' => {
			eco => 'B31l',
                        # TRANSLATORS:

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 Nf6 6. c3'),
			moves => {
				'd2d4' => 'r1bq1rk1/pp1pppbp/2n2np1/1Bp5/3PP3/2P2N2/PP3PPP/RNBQR1K1 b - d3 0 7',
				'h2h3' => 'r1bq1rk1/pp1pppbp/2n2np1/1Bp5/4P3/2P2N1P/PP1P1PP1/RNBQR1K1 b - - 0 7',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 b - - 0 7' => {
			eco => 'A38h',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. b3

			variation => N__('English: Symmetrical, Main Line, 7. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/1RBQ1RK1 b - - 7 7' => {
			eco => 'A38g',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. Rb1

			variation => N__('English: Symmetrical, Main Line, 7. Rb1'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 6 7' => {
			eco => 'A38f',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O

			variation => N__('English: Symmetrical, Main Line, 6.O-O O-O'),
			moves => {
				'a1b1' => 'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/1RBQ1RK1 b - - 7 7',
				'a2a3' => 'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/P1N2NP1/1P1PPPBP/R1BQ1RK1 b - - 0 7',
				'b2b3' => 'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 b - - 0 7',
				'd2d3' => 'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b - - 0 7',
				'd2d4' => 'r1bq1rk1/pp1pppbp/2n2np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - d3 0 7',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b - - 0 7' => {
			eco => 'A38k',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3

			variation => N__('English: Symmetrical, Main Line, 7. d3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1pppbp/p1n2np1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'd7d5' => 'r1bq1rk1/pp2ppbp/2n2np1/2pp4/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - d6 0 8',
				'd7d6' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/P1N2NP1/1P1PPPBP/R1BQ1RK1 b - - 0 7' => {
			eco => 'A38i',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. a3

			variation => N__('English: Symmetrical, Main Line, 7. a3'),
			moves => {
				'd7d6' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/P1N2NP1/1P1PPPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - d3 0 7' => {
			eco => 'A39a',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4

			variation => N__('English: Symmetrical, Main Line 7. d4'),
			moves => {
				'c5d4' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2Pp4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2BNP3/2N1B3/PPP2PPP/R2Q1RK1 b - - 7 8' => {
			eco => 'B35h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. O-O

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8.O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQ - 6 8' => {
			eco => 'B35f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O'),
			moves => {
				'c4b3' => 'r1bq1rk1/pp1pppbp/2n2np1/8/3NP3/1BN1B3/PPP2PPP/R2QK2R b KQ - 7 8',
				'e1g1' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2BNP3/2N1B3/PPP2PPP/R2Q1RK1 b - - 7 8',
				'f2f3' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2BNP3/2N1BP2/PPP3PP/R2QK2R b KQ - 0 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2BNP3/2N1BP2/PPP3PP/R2QK2R b KQ - 0 8' => {
			eco => 'B35g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. f3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. f3'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2P1P3/2N5/PPN2PPP/R1BQKB1R w KQ - 5 8' => {
			eco => 'B37h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 O-O

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A39c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4

			variation => N__('English: Symmetrical, Main Line 7. d4'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p1pppbp/p1n2np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9',
				'c6d4' => 'r1bq1rk1/pp1pppbp/5np1/8/2Pn4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9',
				'd7d6' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9',
				'd8a5' => 'r1b2rk1/pp1pppbp/2n2np1/q7/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 1 9',
				'd8b6' => 'r1b2rk1/pp1pppbp/1qn2np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 1 9',
				'f6g4' => 'r1bq1rk1/pp1pppbp/2n3p1/8/2PN2n1/2N3P1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R b KQ - 6 8' => {
			eco => 'B38g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2'),
			moves => {
				'b7b6' => 'r1bq1rk1/p2pppbp/1pn2np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R w KQ - 0 9',
				'd7d6' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R w KQ - 0 9',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R w KQ - 5 8' => {
			eco => 'B38g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O'),
			moves => {
				'f1e2' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R b KQ - 6 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/2Pp4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A39b',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4

			variation => N__('English: Symmetrical, Main Line 7. d4'),
			moves => {
				'f3d4' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/3NP3/1BN1B3/PPP2PPP/R2QK2R b KQ - 7 8' => {
			eco => 'B35i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p1pppbp/2n2np1/p7/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ a6 0 9',
				'd7d6' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ - 0 9',
				'd8a5' => 'r1b2rk1/pp1pppbp/2n2np1/q7/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ - 8 9',
				'f6g4' => 'r1bq1rk1/pp1pppbp/2n3p1/8/3NP1n1/1BN1B3/PPP2PPP/R2QK2R w KQ - 8 9',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b - - 7 8' => {
			eco => 'B34v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Be2 O-O 8. O-O

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7. Be2 O-O 8.O-O'),
			moves => {
				'd7d5' => 'r1bq1rk1/pp2ppbp/2n2np1/3p4/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w - d6 0 9',
			},
		},
		'r1bq1rk1/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQ - 6 8' => {
			eco => 'B34u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Be2 O-O

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7. Be2 O-O'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b - - 7 8',
			},
		},
		'r1bq1rk1/pp1pppbp/2n3p1/8/2PN2n1/2N1P1P1/PP3PBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'A39h',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... Ng4'),
			moves => {
				'd7d6' => 'r1bq1rk1/pp2ppbp/2np2p1/8/2PN2n1/2N1P1P1/PP3PBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1pppbp/2n3p1/8/2PN2n1/2N3P1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A39h',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Ng4

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... Ng4'),
			moves => {
				'e2e3' => 'r1bq1rk1/pp1pppbp/2n3p1/8/2PN2n1/2N1P1P1/PP3PBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp1pppbp/2n3p1/8/3NP1n1/1BN1B3/PPP2PPP/R2QK2R w KQ - 8 9' => {
			eco => 'B35j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 Ng4

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 Ng4'),
			moves => {
			},
		},
		'r1bq1rk1/pp1pppbp/5np1/8/2PQ4/2N3P1/PP2PPBP/R1B2RK1 b - - 0 9' => {
			eco => 'A39j',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4

			variation => N__('English: Symmetrical, Main Line 7. d4, 9.Qxd4'),
			moves => {
				'd7d6' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2PQ4/2N3P1/PP2PPBP/R1B2RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp1pppbp/5np1/8/2Pn4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A39j',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4

			variation => N__('English: Symmetrical, Main Line 7. d4, 8...Nxd4'),
			moves => {
				'd1d4' => 'r1bq1rk1/pp1pppbp/5np1/8/2PQ4/2N3P1/PP2PPBP/R1B2RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp2bpp1/2n2n1p/3p2B1/3N4/2N3P1/PP2PPBP/R2Q1RK1 w - - 0 11' => {
			eco => 'D34s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6'),
			moves => {
				'g5e3' => 'r1bq1rk1/pp2bpp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/R2Q1RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/pp2bpp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/R2Q1RK1 b - - 1 11' => {
			eco => 'D34t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6 11. Be3

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6 11. Be3'),
			moves => {
				'f8e8' => 'r1bqr1k1/pp2bpp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/R2Q1RK1 w - - 2 12',
			},
		},
		'r1bq1rk1/pp2bpp1/2np1n1p/4p3/3NPP1B/2N5/PPPQ2PP/2KR1B1R w - - 0 11' => {
			eco => 'B64f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 h6 10. Bh4 e5

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. f4 h6 10. Bh4 e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bpp1/2np1n1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQR1K1 b - - 0 10' => {
			eco => 'B58n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2bpp1/p1np1n1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQR1K1 w - - 0 11',
				'c8e6' => 'r2q1rk1/pp2bpp1/2npbn1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQR1K1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp2bpp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQR1K1 w - - 4 10' => {
			eco => 'B58m',
                        # TRANSLATORS:

			variation => N__('Sicilian: Boleslavsky, 7. Nf3 h6 8.O-O Be7 9. Re1'),
			moves => {
				'h2h3' => 'r1bq1rk1/pp2bpp1/2np1n1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQR1K1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp2bpp1/2nppn1p/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w - - 0 10' => {
			eco => 'B64c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 h6

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. f4 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pp2bpp1/2nppn1p/8/3NPP1B/2N5/PPPQ2PP/2KR1B1R b - - 1 10',
			},
		},
		'r1bq1rk1/pp2bpp1/2nppn1p/8/3NPP1B/2N5/PPPQ2PP/2KR1B1R b - - 1 10' => {
			eco => 'B64d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 h6 10. Bh4

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. f4 h6 10. Bh4'),
			moves => {
				'c8d7' => 'r2q1rk1/pp1bbpp1/2nppn1p/8/3NPP1B/2N5/PPPQ2PP/2KR1B1R w - - 2 11',
				'e6e5' => 'r1bq1rk1/pp2bpp1/2np1n1p/4p3/3NPP1B/2N5/PPPQ2PP/2KR1B1R w - - 0 11',
			},
		},
		'r1bq1rk1/pp2bpp1/3ppn1p/6B1/3QPP2/2N5/PPP3PP/2KR1B1R w - - 0 11' => {
			eco => 'B65b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4 h6

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4 10.Qxd4 h6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/1nn1p3/2p5/3PP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'D41f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O 9. e4 Nb6

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9. e4 Nb6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/2p5/1n1PP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'D41g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O 9. e4 Ndb4

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9. e4 Ndb4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/2pN4/3P4/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D41h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O 9. Nxd5

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9.Nxd5'),
			moves => {
				'e6d5' => 'r1bq1rk1/pp2bppp/2n5/2pp4/3P4/5NP1/PP2PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 9' => {
			eco => 'D41d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line'),
			moves => {
				'c3d5' => 'r1bq1rk1/pp2bppp/2n1p3/2pN4/3P4/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 9',
				'e2e4' => 'r1bq1rk1/pp2bppp/2n1p3/2pn4/3PP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 9',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 9' => {
			eco => 'D42c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 Be7 8. O-O O-O

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bd3 Be7 8.O-O O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/2pn4/3PP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 9' => {
			eco => 'D41e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O 9. e4

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9. e4'),
			moves => {
				'd5b4' => 'r1bq1rk1/pp2bppp/2n1p3/2p5/1n1PP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 10',
				'd5b6' => 'r1bq1rk1/pp2bppp/1nn1p3/2p5/3PP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQ1RK1 w - - 3 10' => {
			eco => 'D41s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4 Be7 9. O-O O-O

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'f1e1' => 'r1bq1rk1/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQR1K1 b - - 4 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQR1K1 b - - 4 10' => {
			eco => 'D41s',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'd5c3' => 'r1bq1rk1/pp2bppp/2n1p3/8/2BP4/2n2N2/PP3PPP/R1BQR1K1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQ1RK1 w - - 3 10' => {
			eco => 'D42f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 9.O-O O-O'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp2bppp/2n1p3/3n4/3P4/P1NB1N2/1P3PPP/R1BQ1RK1 b - - 0 10',
				'f1e1' => 'r1bq1rk1/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQR1K1 b - - 4 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQR1K1 b - - 4 10' => {
			eco => 'D42j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1'),
			moves => {
				'c6b4' => 'r1bq1rk1/pp2bppp/4p3/3n4/1n1P4/2NB1N2/PP3PPP/R1BQR1K1 w - - 5 11',
				'd5f6' => 'r1bq1rk1/pp2bppp/2n1pn2/8/3P4/2NB1N2/PP3PPP/R1BQR1K1 w - - 5 11',
				'e7f6' => 'r1bq1rk1/pp3ppp/2n1pb2/3n4/3P4/2NB1N2/PP3PPP/R1BQR1K1 w - - 5 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/3n4/3P4/P1NB1N2/1P3PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D42g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. a3

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. a3'),
			moves => {
				'd5f6' => 'r1bq1rk1/pp2bppp/2n1pn2/8/3P4/P1NB1N2/1P3PPP/R1BQ1RK1 w - - 1 11',
				'e7f6' => 'r1bq1rk1/pp3ppp/2n1pb2/3n4/3P4/P1NB1N2/1P3PPP/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/8/2BP4/2P2N2/P4PPP/R1BQR1K1 b - - 0 11' => {
			eco => 'D41t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nxc3 11. bxc3

			variation => N__('QGD: Semi-Tarrasch, 7. Bc4 Main Line'),
			moves => {
				'b7b6' => 'r1bq1rk1/p3bppp/1pn1p3/8/2BP4/2P2N2/P4PPP/R1BQR1K1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp2bppp/2n1p3/8/2BP4/2n2N2/PP3PPP/R1BQR1K1 w - - 0 11' => {
			eco => 'D41s',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'b2c3' => 'r1bq1rk1/pp2bppp/2n1p3/8/2BP4/2P2N2/P4PPP/R1BQR1K1 b - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/1BPp2B1/3P4/2N2N2/PP3PPP/R2QK2R b KQ - 2 9' => {
			eco => 'B13z',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nf3 Be7 8. c5 O-O 9. Bb5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Normal Variation, 8. c5 O-O 9. Bb5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/2Pp2B1/3P4/2N2N2/PP3PPP/R2QKB1R w KQ - 1 9' => {
			eco => 'B13z',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nf3 Be7 8. c5 O-O

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Normal Variation, 8. c5 O-O'),
			moves => {
				'f1b5' => 'r1bq1rk1/pp2bppp/2n1pn2/1BPp2B1/3P4/2N2N2/PP3PPP/R2QK2R b KQ - 2 9',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/2pp4/2P5/1P2PNP1/PB1P1PBP/RN1Q1RK1 b - - 0 8' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 c5 7. Bb2 Nc6 8. e3

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 c5 7. Bb2 Nc6 8. e3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/2pp4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 w - - 2 8' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 c5 7. Bb2 Nc6

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 c5 7. Bb2 Nc6'),
			moves => {
				'e2e3' => 'r1bq1rk1/pp2bppp/2n1pn2/2pp4/2P5/1P2PNP1/PB1P1PBP/RN1Q1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w - - 1 8' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, French Variation'),
			moves => {
				'f1e1' => 'r1bq1rk1/pp2bppp/2n1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQR1K1 b - - 2 8',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQR1K1 b - - 2 8' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Nf6 6. Nbd2 Be7 7. e4 O-O 8. Re1

			variation => N__('Reti: KIA, French Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/6B1/2BP4/P1N2N2/1P3PPP/R2Q1RK1 b - - 2 11' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7 11. Bg5

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7 11. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/6B1/3P4/2NB1N2/PP3PPP/R2QR1K1 b - - 6 11' => {
			eco => 'D42l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6 11. Bg5

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6 11. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/2BP1B2/P1N2N2/1P3PPP/R2Q1RK1 b - - 2 11' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7 11. Bf4

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7 11. Bf4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1N1BN2/1P3PPP/R2Q1RK1 b - - 2 11' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7 11. Be3

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7 11. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1N2N2/1P3PPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1N1BN2/1P3PPP/R2Q1RK1 b - - 2 11',
				'c1f4' => 'r1bq1rk1/pp2bppp/2n1pn2/8/2BP1B2/P1N2N2/1P3PPP/R2Q1RK1 b - - 2 11',
				'c1g5' => 'r1bq1rk1/pp2bppp/2n1pn2/6B1/2BP4/P1N2N2/1P3PPP/R2Q1RK1 b - - 2 11',
				'd1d3' => 'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1NQ1N2/1P3PPP/R1B2RK1 b - - 2 11',
				'f1e1' => 'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1N2N2/1P3PPP/R1BQR1K1 b - - 2 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1N2N2/1P3PPP/R1BQR1K1 b - - 2 11' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7 11. Re1

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7 11. Re1'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2bppp/p1n1pn2/8/2BP4/P1N2N2/1P3PPP/R1BQR1K1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1NQ1N2/1P3PPP/R1B2RK1 b - - 2 11' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4 Be7 11. Qd3

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4 Be7 11. Qd3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/3P4/2NB1N2/PP3PPP/R1BQR1K1 w - - 5 11' => {
			eco => 'D42l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp2bppp/2n1pn2/8/3P4/P1NB1N2/1P3PPP/R1BQR1K1 b - - 0 11',
				'c1g5' => 'r1bq1rk1/pp2bppp/2n1pn2/6B1/3P4/2NB1N2/PP3PPP/R2QR1K1 b - - 6 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/3P4/P1NB1N2/1P3PPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'D42h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. a3 Nf6

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. a3 Nf6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n1pn2/8/3P4/P1NB1N2/1P3PPP/R1BQR1K1 b - - 0 11' => {
			eco => 'D42m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nf6 11. a3

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Nf6 11. a3'),
			moves => {
				'b7b6' => 'r1bq1rk1/p3bppp/1pn1pn2/8/3P4/P1NB1N2/1P3PPP/R1BQR1K1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2P5/3p4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D34h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 d4

			variation => N__('QGD Tarrasch: Tarrasch\'s Gambit'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2Pp4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D34h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5

			variation => N__('QGD Tarrasch: 9.dxc5'),
			moves => {
				'd5d4' => 'r1bq1rk1/pp2bppp/2n2n2/2P5/3p4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10',
				'e7c5' => 'r1bq1rk1/pp3ppp/2n2n2/2bp4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2pp2B1/3P4/2N2NP1/PP2PPBP/R2Q1RK1 b - - 6 9' => {
			eco => 'D34o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5

			variation => N__('QGD Tarrasch: 9. Bg5'),
			moves => {
				'c5c4' => 'r1bq1rk1/pp2bppp/2n2n2/3p2B1/2pP4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 10',
				'c5d4' => 'r1bq1rk1/pp2bppp/2n2n2/3p2B1/3p4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 10',
				'c8e6' => 'r2q1rk1/pp2bppp/2n1bn2/2pp2B1/3P4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 7 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P1B2/2N2NP1/PP2PPBP/R2Q1RK1 b - - 6 9' => {
			eco => 'D34f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bf4

			variation => N__('QGD Tarrasch: 9. Bf4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D34d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. b3

			variation => N__('QGD Tarrasch: 9. b3'),
			moves => {
				'f6e4' => 'r1bq1rk1/pp2bppp/2n5/2pp4/3Pn3/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P4/2N1BNP1/PP2PPBP/R2Q1RK1 b - - 6 9' => {
			eco => 'D34g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Be3

			variation => N__('QGD Tarrasch: 9. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 9' => {
			eco => 'D34c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O

			variation => N__('QGD Tarrasch: 8.O-O O-O (Main Line)'),
			moves => {
				'b2b3' => 'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 9',
				'c1e3' => 'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P4/2N1BNP1/PP2PPBP/R2Q1RK1 b - - 6 9',
				'c1f4' => 'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P1B2/2N2NP1/PP2PPBP/R2Q1RK1 b - - 6 9',
				'c1g5' => 'r1bq1rk1/pp2bppp/2n2n2/2pp2B1/3P4/2N2NP1/PP2PPBP/R2Q1RK1 b - - 6 9',
				'd4c5' => 'r1bq1rk1/pp2bppp/2n2n2/2Pp4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/3p2B1/2pP4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 10' => {
			eco => 'D34p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 c4

			variation => N__('QGD Tarrasch: 9. Bg5 c4'),
			moves => {
				'f3e5' => 'r1bq1rk1/pp2bppp/2n2n2/3pN1B1/2pP4/2N3P1/PP2PPBP/R2Q1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/3p2B1/3N4/2N3P1/PP2PPBP/R2Q1RK1 b - - 0 10' => {
			eco => 'D34r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4'),
			moves => {
				'h7h6' => 'r1bq1rk1/pp2bpp1/2n2n1p/3p2B1/3N4/2N3P1/PP2PPBP/R2Q1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/3p2B1/3p4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 10' => {
			eco => 'D34r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4'),
			moves => {
				'f3d4' => 'r1bq1rk1/pp2bppp/2n2n2/3p2B1/3N4/2N3P1/PP2PPBP/R2Q1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/3p4/3N4/2N1B1P1/PP2PPBP/R2Q1RK1 b - - 4 10' => {
			eco => 'D33q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 cxd4 8. Nxd4 Be7 9. O-O O-O 10. Be3

			variation => N__('QGD Tarrasch: 7. Bg2 cxd4 8.Nxd4 Be7 9.O-O O-O 10. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 3 10' => {
			eco => 'D33p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 cxd4 8. Nxd4 Be7 9. O-O O-O

			variation => N__('QGD Tarrasch: 7. Bg2 cxd4 8.Nxd4 Be7'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2bppp/2n2n2/3p4/3N4/2N1B1P1/PP2PPBP/R2Q1RK1 b - - 4 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n2n2/3pN1B1/2pP4/2N3P1/PP2PPBP/R2Q1RK1 b - - 1 10' => {
			eco => 'D34p',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9. Bg5 c4'),
			moves => {
				'c8e6' => 'r2q1rk1/pp2bppp/2n1bn2/3pN1B1/2pP4/2N3P1/PP2PPBP/R2Q1RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n5/2Pp4/8/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D41h',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9.Nxd5'),
			moves => {
				'e7c5' => 'r1bq1rk1/pp3ppp/2n5/2bp4/8/5NP1/PP2PPBP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/2n5/2pp4/3P4/5NP1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D41h',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9.Nxd5'),
			moves => {
				'd4c5' => 'r1bq1rk1/pp2bppp/2n5/2Pp4/8/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/2n5/2pp4/3Pn3/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'D34e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. b3 Ne4

			variation => N__('QGD Tarrasch: 9. b3 Ne4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/2p3B1/4P3/2N2N2/PPP1BPPP/R2Q1RK1 b - - 3 9' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'c8e6' => 'r2q1rk1/pp2bppp/2npbn2/2p3B1/4P3/2N2N2/PPP1BPPP/R2Q1RK1 w - - 4 10',
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/2p5/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 2 9' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'c1g5' => 'r1bq1rk1/pp2bppp/2np1n2/2p3B1/4P3/2N2N2/PPP1BPPP/R2Q1RK1 b - - 3 9',
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p1B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w - - 0 10' => {
			eco => 'B64b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 e5

			variation => N__('Sicilian: Richter-Rauzer, Geller Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p1B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 b - - 5 9' => {
			eco => 'B59h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Bg5

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p3/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 0 10' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 e5

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 e5'),
			moves => {
				'd4b3' => 'r1bq1rk1/pp2bppp/2np1n2/4p3/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 b - - 5 9' => {
			eco => 'B59k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Be3'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p2bppp/2np1n2/p3p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - a6 0 10',
				'c8e6' => 'r2q1rk1/pp2bppp/2npbn2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 6 10',
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K b - - 5 9' => {
			eco => 'B59i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Kh1

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Kh1'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w - - 4 9' => {
			eco => 'B59g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2bppp/2np1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 b - - 5 9',
				'c1g5' => 'r1bq1rk1/pp2bppp/2np1n2/4p1B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 b - - 5 9',
				'f2f4' => 'r1bq1rk1/pp2bppp/2np1n2/4p3/4PP2/1NN5/PPP1B1PP/R1BQ1RK1 b - f3 0 9',
				'g1h1' => 'r1bq1rk1/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K b - - 5 9',
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p3/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 b - - 1 10' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 e5 10. Nb3

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 e5 10. Nb3'),
			moves => {
				'e5f4' => 'r1bq1rk1/pp2bppp/2np1n2/8/4Pp2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/4p3/4PP2/1NN5/PPP1B1PP/R1BQ1RK1 b - f3 0 9' => {
			eco => 'B59j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. f4

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. f4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/8/4PB2/1NN5/PPP1B1PP/R2Q1RK1 b - - 0 11' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 e5 10. Nb3 exf4 11. Bxf4

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 e5 10. Nb3 exf4 11.Bxf4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2np1n2/8/4Pp2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 0 11' => {
			eco => 'B83',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 e5 10. Nb3'),
			moves => {
				'e3f4' => 'r1bq1rk1/pp2bppp/2np1n2/8/4PB2/1NN5/PPP1B1PP/R2Q1RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R b - - 0 9' => {
			eco => 'B63p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f3

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. f3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w - - 4 9' => {
			eco => 'B63o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7 8.O-O-O O-O'),
			moves => {
				'd4b3' => 'r1bq1rk1/pp2bppp/2nppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R b - - 5 9',
				'f2f3' => 'r1bq1rk1/pp2bppp/2nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R b - - 0 9',
				'f2f4' => 'r1bq1rk1/pp2bppp/2nppn2/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R b - f3 0 9',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R b - f3 0 9' => {
			eco => 'B64a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. f4'),
			moves => {
				'c6d4' => 'r1bq1rk1/pp2bppp/3ppn2/6B1/3nPP2/2N5/PPPQ2PP/2KR1B1R w - - 0 10',
				'e6e5' => 'r1bq1rk1/pp2bppp/2np1n2/4p1B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w - - 0 10',
				'h7h6' => 'r1bq1rk1/pp2bpp1/2nppn1p/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R b - - 5 9' => {
			eco => 'B63q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. Nb3'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p2bppp/2nppn2/p5B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w - a6 0 10',
				'a7a6' => 'r1bq1rk1/1p2bppp/p1nppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w - - 0 10',
				'd8b6' => 'r1b2rk1/pp2bppp/1qnppn2/6B1/4P3/1NN5/PPPQ1PPP/2KR1B1R w - - 6 10',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/2KR3R b - - 7 9' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 Be7 8. Qe2 O-O 9. O-O-O

			variation => N__('Sicilian: Velimirovic, 9.O-O-O'),
			moves => {
				'd8c7' => 'r1b2rk1/ppq1bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/2KR3R w - - 8 10',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/R3K2R w KQ - 6 9' => {
			eco => 'B89',
                        # TRANSLATORS:

			variation => N__('Sicilian: Velimirovic'),
			moves => {
				'e1c1' => 'r1bq1rk1/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/2KR3R b - - 7 9',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/8/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ - 6 9' => {
			eco => 'B88',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin-Scheveningen, 7. Bb3'),
			moves => {
				'f2f4' => 'r1bq1rk1/pp2bppp/2nppn2/8/3NPP2/1BN1B3/PPP3PP/R2QK2R b KQ f3 0 9',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w - - 6 9' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O

			variation => N__('Sicilian: Scheveningen, Modern'),
			moves => {
				'f2f4' => 'r1bq1rk1/pp2bppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b - f3 0 9',
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/8/3NPP2/1BN1B3/PPP3PP/R2QK2R b KQ f3 0 9' => {
			eco => 'B88',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Bb3 Be7 8. Be3 O-O 9. f4

			variation => N__('Sicilian: Sozin, Fischer Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b - f3 0 9' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4'),
			moves => {
				'c8d7' => 'r2q1rk1/pp1bbppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 1 10',
				'e6e5' => 'r1bq1rk1/pp2bppp/2np1n2/4p3/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/3ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R w - - 1 10' => {
			eco => 'B63n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O Nxd4 9. Qxd4 O-O

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7 8.O-O-O Nxd4 9.Qxd4 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp2bppp/3ppn2/6B1/3QPP2/2N5/PPP3PP/2KR1B1R b - - 0 10' => {
			eco => 'B65a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4'),
			moves => {
				'd8a5' => 'r1b2rk1/pp2bppp/3ppn2/q5B1/3QPP2/2N5/PPP3PP/2KR1B1R w - - 1 11',
				'h7h6' => 'r1bq1rk1/pp2bpp1/3ppn1p/6B1/3QPP2/2N5/PPP3PP/2KR1B1R w - - 0 11',
			},
		},
		'r1bq1rk1/pp2bppp/3ppn2/6B1/3nPP2/2N5/PPPQ2PP/2KR1B1R w - - 0 10' => {
			eco => 'B65a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4'),
			moves => {
				'd2d4' => 'r1bq1rk1/pp2bppp/3ppn2/6B1/3QPP2/2N5/PPP3PP/2KR1B1R b - - 0 10',
			},
		},
		'r1bq1rk1/pp2bppp/4p3/3n4/1n1P4/2NB1N2/PP3PPP/R1BQR1K1 w - - 5 11' => {
			eco => 'D42k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Ncb4

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Ncb4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2n1bp/2pp1np1/3PppN1/1PP1P3/2N2B2/P4PPP/R1BQR1K1 w - - 0 13' => {
			eco => 'E97s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 f5 11. Ng5 Nf6 12. Bf3 c6

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 f5 11. Ng5 Nf6 12. Bf3 c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2npbp/2npp1p1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A36r',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 e6 6. Nge2 Nge7 7. O-O O-O 8. d3 d6

			variation => N__('English: Symmetrical, Botvinnik, 5... e6, 8. d3 d6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2npbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQ1RK1 w - - 4 9' => {
			eco => 'B25p',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O

			variation => N__('Sicilian: Closed, 6. f4 e6'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2npbp/2npp1p1/2p5/4PP2/2NPBNP1/PPP3BP/R2Q1RK1 b - - 5 9',
			},
		},
		'r1bq1rk1/pp2npbp/2npp1p1/2p5/4PP2/2NPBNP1/PPP3BP/R2Q1RK1 b - - 5 9' => {
			eco => 'B25q',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3'),
			moves => {
				'a8b8' => '1rbq1rk1/pp2npbp/2npp1p1/2p5/4PP2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 6 10',
				'b7b6' => 'r1bq1rk1/p3npbp/1pnpp1p1/2p5/4PP2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 0 10',
				'c6d4' => 'r1bq1rk1/pp2npbp/3pp1p1/2p5/3nPP2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 6 10',
			},
		},
		'r1bq1rk1/pp2npbp/3p1np1/2pPp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - c6 0 10' => {
			eco => 'E97w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 c5

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 c5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2npbp/3p1np1/2pPp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 w - c6 0 10' => {
			eco => 'E98b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 c5

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 c5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2npbp/3pp1p1/2p1P3/3n1P2/2NPBNP1/PPP3BP/R2Q1RK1 b - - 0 10' => {
			eco => 'B25w',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 Nd4 10. e5

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 Nd4 10. e5'),
			moves => {
				'e7f5' => 'r1bq1rk1/pp3pbp/3pp1p1/2p1Pn2/3n1P2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp2npbp/3pp1p1/2p5/3nPP2/2NPBNP1/PPP3BP/1R1Q1RK1 b - - 7 10' => {
			eco => 'B25u',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 Nd4 10. Rb1

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 Nd4 10. Rb1'),
			moves => {
			},
		},
		'r1bq1rk1/pp2npbp/3pp1p1/2p5/3nPP2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 6 10' => {
			eco => 'B25t',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 Nd4

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 Nd4'),
			moves => {
				'a1b1' => 'r1bq1rk1/pp2npbp/3pp1p1/2p5/3nPP2/2NPBNP1/PPP3BP/1R1Q1RK1 b - - 7 10',
				'd1d2' => 'r1bq1rk1/pp2npbp/3pp1p1/2p5/3nPP2/2NPBNP1/PPPQ2BP/R4RK1 b - - 7 10',
				'e4e5' => 'r1bq1rk1/pp2npbp/3pp1p1/2p1P3/3n1P2/2NPBNP1/PPP3BP/R2Q1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp2npbp/3pp1p1/2p5/3nPP2/2NPBNP1/PPPQ2BP/R4RK1 b - - 7 10' => {
			eco => 'B25v',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 Nd4 10. Qd2

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 Nd4 10. Qd2'),
			moves => {
			},
		},
		'r1bq1rk1/pp2nppp/1bn5/1B1p4/8/1NP2N2/PP3PPP/R1BQ1RK1 w - - 3 11' => {
			eco => 'C09i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. c3 O-O 10. Nb3 Bb6

			variation => N__('French: Tarrasch, Open, Main Line, 9. c3 O-O 10. Nb3 Bb6'),
			moves => {
				'f1e1' => 'r1bq1rk1/pp2nppp/1bn5/1B1p4/8/1NP2N2/PP3PPP/R1BQR1K1 b - - 4 11',
			},
		},
		'r1bq1rk1/pp2nppp/1bn5/1B1p4/8/1NP2N2/PP3PPP/R1BQR1K1 b - - 4 11' => {
			eco => 'C09j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. c3 O-O 10. Nb3 Bb6 11. Re1

			variation => N__('French: Tarrasch, Open, Main Line, 9. c3 O-O 10. Nb3 Bb6 11. Re1'),
			moves => {
			},
		},
		'r1bq1rk1/pp2nppp/2n1p3/2ppP3/3P2Q1/P1P2N2/2P2PPP/R1B1KB1R w KQ - 5 9' => {
			eco => 'C18q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 O-O 8. Nf3 Nbc6

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 O-O 8. Nf3 Nbc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2nppp/2n1p3/2ppP3/3P2Q1/P1PB4/2P2PPP/R1B1K1NR w KQ - 5 9' => {
			eco => 'C18p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 O-O 8. Bd3 Nbc6

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 O-O 8. Bd3 Nbc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2nppp/2n5/1Bbp4/8/1NP2N2/PP3PPP/R1BQ1RK1 b - - 2 10' => {
			eco => 'C09g',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, Main Line, 9. c3'),
			moves => {
				'c5b6' => 'r1bq1rk1/pp2nppp/1bn5/1B1p4/8/1NP2N2/PP3PPP/R1BQ1RK1 w - - 3 11',
				'c5d6' => 'r1bq1rk1/pp2nppp/2nb4/1B1p4/8/1NP2N2/PP3PPP/R1BQ1RK1 w - - 3 11',
			},
		},
		'r1bq1rk1/pp2nppp/2n5/1Bbp4/8/2P2N2/PP1N1PPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'C09g',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, Main Line, 9. c3'),
			moves => {
				'd2b3' => 'r1bq1rk1/pp2nppp/2n5/1Bbp4/8/1NP2N2/PP3PPP/R1BQ1RK1 b - - 2 10',
			},
		},
		'r1bq1rk1/pp2nppp/2nb4/1B1p2B1/8/1N3N2/PPP2PPP/R2QR1K1 b - - 5 11' => {
			eco => 'C09p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bd6 10. Re1 O-O 11. Bg5

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6 10. Re1 O-O 11. Bg5'),
			moves => {
				'c8g4' => 'r2q1rk1/pp2nppp/2nb4/1B1p2B1/6b1/1N3N2/PPP2PPP/R2QR1K1 w - - 6 12',
			},
		},
		'r1bq1rk1/pp2nppp/2nb4/1B1p4/8/1N3N2/PPP2PPP/R1BQR1K1 w - - 4 11' => {
			eco => 'C09o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6 10. Re1'),
			moves => {
				'c1g5' => 'r1bq1rk1/pp2nppp/2nb4/1B1p2B1/8/1N3N2/PPP2PPP/R2QR1K1 b - - 5 11',
			},
		},
		'r1bq1rk1/pp2nppp/2nb4/1B1p4/8/1NP2N2/PP3PPP/R1BQ1RK1 w - - 3 11' => {
			eco => 'C09h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. c3 O-O 10. Nb3 Bd6

			variation => N__('French: Tarrasch, Open, Main Line, 9. c3 O-O 10. Nb3 Bd6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2nppp/2nbp3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w - - 1 8' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, French Variation'),
			moves => {
				'f1e1' => 'r1bq1rk1/pp2nppp/2nbp3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQR1K1 b - - 2 8',
			},
		},
		'r1bq1rk1/pp2nppp/2nbp3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQR1K1 b - - 2 8' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Bd6 6. Nbd2 Nge7 7. e4 O-O 8. Re1

			variation => N__('Reti: KIA, French Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp2nppp/2p2n2/3pp1B1/1b2P2N/1BNP4/PPP2PPP/R2Q1RK1 b - - 1 10' => {
			eco => 'C49k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Pillsbury Variation'),
			moves => {
				'd8d6' => 'r1b2rk1/pp2nppp/2pq1n2/3pp1B1/1b2P2N/1BNP4/PPP2PPP/R2Q1RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/pp2nppp/2p2n2/3pp1B1/1bB1P2N/2NP4/PPP2PPP/R2Q1RK1 w - - 0 10' => {
			eco => 'C49k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Pillsbury Variation'),
			moves => {
				'c4b3' => 'r1bq1rk1/pp2nppp/2p2n2/3pp1B1/1b2P2N/1BNP4/PPP2PPP/R2Q1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/pp2nppp/2pp1n2/1B2p1B1/1b2P2N/2NP4/PPP2PPP/R2Q1RK1 w - - 0 9' => {
			eco => 'C49k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Pillsbury Variation'),
			moves => {
				'b5c4' => 'r1bq1rk1/pp2nppp/2pp1n2/4p1B1/1bB1P2N/2NP4/PPP2PPP/R2Q1RK1 b - - 1 9',
			},
		},
		'r1bq1rk1/pp2nppp/2pp1n2/4p1B1/1bB1P2N/2NP4/PPP2PPP/R2Q1RK1 b - - 1 9' => {
			eco => 'C49k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Pillsbury Variation'),
			moves => {
				'd6d5' => 'r1bq1rk1/pp2nppp/2p2n2/3pp1B1/1bB1P2N/2NP4/PPP2PPP/R2Q1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp2nppp/4pb2/3n4/3PB3/2N2N2/PP3PPP/R1BQR1K1 w - - 7 12' => {
			eco => 'D42q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Bf6 11. Be4 Nce7

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Bf6 11. Be4 Nce7'),
			moves => {
				'f3e5' => 'r1bq1rk1/pp2nppp/4pb2/3nN3/3PB3/2N5/PP3PPP/R1BQR1K1 b - - 8 12',
			},
		},
		'r1bq1rk1/pp2nppp/4pb2/3nN3/3PB3/2N5/PP3PPP/R1BQR1K1 b - - 8 12' => {
			eco => 'D42r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Bf6 11. Be4 Nce7 12. Ne5

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Bf6 11. Be4 Nce7 12. Ne5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2p1bp/n1pp1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. b3 Na6

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. b3 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2pp1p/6p1/2pPb3/4P3/2P5/P2QBPPP/1RB1K2R b K - 1 12' => {
			eco => 'D85p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 Nc6 10. d5 Ne5 11. Nxe5 Bxe5 12. Qd2

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6, Main Line'),
			moves => {
				'e7e6' => 'r1bq1rk1/pp3p1p/4p1p1/2pPb3/4P3/2P5/P2QBPPP/1RB1K2R w K - 0 13',
			},
		},
		'r1bq1rk1/pp2pp1p/6p1/2pPb3/4P3/2P5/P3BPPP/1RBQK2R w K - 0 12' => {
			eco => 'D85o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6'),
			moves => {
				'd1d2' => 'r1bq1rk1/pp2pp1p/6p1/2pPb3/4P3/2P5/P2QBPPP/1RB1K2R b K - 1 12',
			},
		},
		'r1bq1rk1/pp2ppbp/1np3p1/n2P2B1/4P3/2N2NP1/PP3PBP/R2Q1RK1 b - - 1 11' => {
			eco => 'D76l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. d5 Na5 10. e4 c6 11. Bg5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. d5 Na5 10. e4 c6 11. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/1np3p1/n2P4/4P3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 11' => {
			eco => 'D76k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. d5 Na5 10. e4 c6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. d5 Na5 10. e4 c6'),
			moves => {
				'c1g5' => 'r1bq1rk1/pp2ppbp/1np3p1/n2P2B1/4P3/2N2NP1/PP3PBP/R2Q1RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2N3p1/3n4/8/2N1BP2/PPPQ2PP/2KR1B1R b - - 0 11' => {
			eco => 'B76q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. exd5 Nxd5 11. Nxc6

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5 10.exd5 Nxd5 11.Nxc6'),
			moves => {
				'b7c6' => 'r1bq1rk1/p3ppbp/2p3p1/3n4/8/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 12',
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/2p5/2P2B2/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 9' => {
			eco => 'E65h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. dxc5 dxc5 9. Bf4

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, Exchange, 9. Bf4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/2p5/2P5/2N1BNP1/PP2PPBP/R2Q1RK1 b - - 1 9' => {
			eco => 'E65g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. dxc5 dxc5 9. Be3

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, Exchange, 9. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/2p5/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E65f',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, Exchange'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2ppbp/2n2np1/2p5/2P5/2N1BNP1/PP2PPBP/R2Q1RK1 b - - 1 9',
				'c1f4' => 'r1bq1rk1/pp2ppbp/2n2np1/2p5/2P2B2/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/2pp4/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - d6 0 8' => {
			eco => 'A38m',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 d5

			variation => N__('English: Symmetrical, Main Line, 7. d3 d5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3P4/3N4/2N1B3/PPP1BPPP/R2Q1RK1 b - - 0 9' => {
			eco => 'B34w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Be2 O-O 8. O-O d5 9. exd5

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7. Be2 O-O 8.O-O d5 9.exd5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3P4/3N4/2N1BP2/PPPQ2PP/2KR1B1R b - - 0 10' => {
			eco => 'B76q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. exd5

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5 10.exd5'),
			moves => {
				'f6d5' => 'r1bq1rk1/pp2ppbp/2n3p1/3n4/3N4/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3p4/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w - d6 0 9' => {
			eco => 'B34v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Be2 O-O 8. O-O d5

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7. Be2 O-O 8.O-O d5'),
			moves => {
				'e4d5' => 'r1bq1rk1/pp2ppbp/2n2np1/3P4/3N4/2N1B3/PPP1BPPP/R2Q1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3p4/3NP3/2N1BP2/PPP3PP/2KRQB1R b - - 1 10' => {
			eco => 'B76p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. Qe1

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5 10. Qe1'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3p4/3NP3/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 10' => {
			eco => 'B76o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5'),
			moves => {
				'd2e1' => 'r1bq1rk1/pp2ppbp/2n2np1/3p4/3NP3/2N1BP2/PPP3PP/2KRQB1R b - - 1 10',
				'e4d5' => 'r1bq1rk1/pp2ppbp/2n2np1/3P4/3N4/2N1BP2/PPPQ2PP/2KR1B1R b - - 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 9' => {
			eco => 'D79f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 Nc6

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 Nc6'),
			moves => {
				'f3e5' => 'r1bq1rk1/pp2ppbp/2n2np1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 3 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2n2np1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 3 9' => {
			eco => 'D79g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 Nc6 9. Ne5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 Nc6 9. Ne5'),
			moves => {
				'e7e6' => 'r1bq1rk1/pp3pbp/2n1pnp1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 b - - 3 10' => {
			eco => 'D87g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3

			variation => N__('Grünfeld: Classical Exchange, 9.O-O Nc6 10. Be3'),
			moves => {
				'c5d4' => 'r1bq1rk1/pp2ppbp/2n3p1/8/2BpP3/2P1B3/P3NPPP/R2Q1RK1 w - - 0 11',
				'c6a5' => 'r1bq1rk1/pp2ppbp/6p1/n1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11',
				'c8g4' => 'r2q1rk1/pp2ppbp/2n3p1/2p5/2BPP1b1/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11',
				'd8a5' => 'r1b2rk1/pp2ppbp/2n3p1/q1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11',
				'd8c7' => 'r1b2rk1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/R2QK2R w KQ - 2 10' => {
			eco => 'D87d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. Be3 Nc6

			variation => N__('Grünfeld: Classical Exchange, 8... c5 9. Be3 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/2p5/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - - 2 10' => {
			eco => 'D87f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6

			variation => N__('Grünfeld: Classical Exchange, 9.O-O Nc6'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 b - - 3 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/2p5/3PP3/2P2N2/P3BPPP/1RBQK2R w K - 4 10' => {
			eco => 'D85o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 Nc6

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp2ppbp/2n3p1/2pP4/4P3/2P2N2/P3BPPP/1RBQK2R b K - 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/2p5/3PP3/2P2N2/P3BPPP/R1BQ1RK1 w - - 4 10' => {
			eco => 'D85l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be2 O-O 9. O-O Nc6

			variation => N__('Grünfeld: Modern Exchange, 8. Be2 O-O 9.O-O Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/2pP4/4P3/2P2N2/P3BPPP/1RBQK2R b K - 0 10' => {
			eco => 'D85o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6'),
			moves => {
				'c6e5' => 'r1bq1rk1/pp2ppbp/6p1/2pPn3/4P3/2P2N2/P3BPPP/1RBQK2R w K - 1 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/3n4/3N4/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 11' => {
			eco => 'B76q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5 10.exd5'),
			moves => {
				'd4c6' => 'r1bq1rk1/pp2ppbp/2N3p1/3n4/8/2N1BP2/PPPQ2PP/2KR1B1R b - - 0 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/8/2BPP3/4B3/P3NPPP/R2Q1RK1 b - - 0 11' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4'),
			moves => {
				'c6a5' => 'r1bq1rk1/pp2ppbp/6p1/n7/2BPP3/4B3/P3NPPP/R2Q1RK1 w - - 1 12',
				'c8g4' => 'r2q1rk1/pp2ppbp/2n3p1/8/2BPP1b1/4B3/P3NPPP/R2Q1RK1 w - - 1 12',
			},
		},
		'r1bq1rk1/pp2ppbp/2n3p1/8/2BpP3/2P1B3/P3NPPP/R2Q1RK1 w - - 0 11' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4

			variation => N__('Grünfeld: Classical Exchange, Main Line'),
			moves => {
				'c3d4' => 'r1bq1rk1/pp2ppbp/2n3p1/8/2BPP3/4B3/P3NPPP/R2Q1RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2P5/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E65f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. dxc5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, Exchange'),
			moves => {
				'd6c5' => 'r1bq1rk1/pp2ppbp/2n2np1/2p5/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 w - - 4 8' => {
			eco => 'A36v',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 Nf6 7. O-O O-O

			variation => N__('English: Symmetrical, Botvinnik System, 5... d6, 7.O-O O-O'),
			moves => {
				'd2d3' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A36w',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 Nf6 7. O-O O-O 8. d3

			variation => N__('English: Symmetrical, Botvinnik System, 5... d6, 8. d3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2ppbp/p1np1np1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 0 9',
				'a8b8' => '1rbq1rk1/pp2ppbp/2np1np1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 1 9',
				'f6e8' => 'r1bqnrk1/pp2ppbp/2np2p1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/2NP1NP1/PP1BPPBP/R2Q1RK1 b - - 1 8' => {
			eco => 'A38p',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 d6 8. Bd2

			variation => N__('English: Symmetrical, Main Line, 7. d3 d6 8. Bd2'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/2NP1NP1/PP2PPBP/1RBQ1RK1 b - - 1 8' => {
			eco => 'A38o',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 d6 8. Rb1

			variation => N__('English: Symmetrical, Main Line, 7. d3 d6 8. Rb1'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A38n',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 d6

			variation => N__('English: Symmetrical, Main Line, 7. d3 d6'),
			moves => {
				'a1b1' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/2NP1NP1/PP2PPBP/1RBQ1RK1 b - - 1 8',
				'a2a3' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 b - - 0 8',
				'c1d2' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/2NP1NP1/PP1BPPBP/R2Q1RK1 b - - 1 8',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/P1N2NP1/1P1PPPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A38j',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. a3 d6

			variation => N__('English: Symmetrical, Main Line, 7. a3 d6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A38q',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d3 d6 8. a3

			variation => N__('English: Symmetrical, Main Line, 7. d3 d6 8. a3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2ppbp/p1np1np1/2p5/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 3 8' => {
			eco => 'E60i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3 d6 5. Bb2 O-O 6. g3 c5 7. Bg2 Nc6 8. O-O

			variation => N__('King\'s Indian: b3+g3 System'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/1P3NP1/PB2PPBP/RN1QK2R w KQ - 2 8' => {
			eco => 'E60i',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: b3+g3 System'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 3 8',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E65d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7.O-O Nc6'),
			moves => {
				'd4c5' => 'r1bq1rk1/pp2ppbp/2np1np1/2P5/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'd4d5' => 'r1bq1rk1/pp2ppbp/2np1np1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'h2h3' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8' => {
			eco => 'E65e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. h3

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7.O-O Nc6 8. h3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 2 8' => {
			eco => 'E81o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5 7. Nge2 Nc6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 7.Nge2 Nc6'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp2ppbp/2np1np1/2pP4/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 8',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E91k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. O-O Nc6

			variation => N__('King\'s Indian: 6. Be2 c5 7.O-O Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2pP4/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 8' => {
			eco => 'E81p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5 7. Nge2 Nc6 8. d5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 7.Nge2 Nc6 8. d5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E66a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno'),
			moves => {
				'c6a5' => 'r1bq1rk1/pp2ppbp/3p1np1/n1pP4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/6B1/3NP3/2N5/PPP1BPPP/R2Q1RK1 w - - 6 9' => {
			eco => 'B70s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Bg5 Nc6

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Bg5'),
			moves => {
				'd4b3' => 'r1bq1rk1/pp2ppbp/2np1np1/6B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 b - - 7 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/6B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 b - - 7 9' => {
			eco => 'B70t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Bg5 Nc6 9. Nb3

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Bg5 Nc6 9. Nb3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2ppbp/p1np1np1/6B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 w - - 0 10',
				'c8e6' => 'r2q1rk1/pp2ppbp/2npbnp1/6B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 w - - 8 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/R3K2R b KQ - 4 9' => {
			eco => 'B77a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p2ppbp/2np1np1/p7/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ a6 0 10',
				'c6d4' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2BnP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 0 10',
				'c6e5' => 'r1bq1rk1/pp2ppbp/3p1np1/4n3/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 5 10',
				'c8d7' => 'r2q1rk1/pp1bppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 5 10',
				'f6d7' => 'r1bq1rk1/pp1nppbp/2np2p1/8/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 5 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN1BPPP/R1BQ1RK1 b - - 3 9' => {
			eco => 'B37l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O'),
			moves => {
				'c8e6' => 'r2q1rk1/pp2ppbp/2npbnp1/8/2P1P3/2N5/PPN1BPPP/R1BQ1RK1 w - - 4 10',
				'f6d7' => 'r1bq1rk1/pp1nppbp/2np2p1/8/2P1P3/2N5/PPN1BPPP/R1BQ1RK1 w - - 4 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN1BPPP/R1BQK2R w KQ - 2 9' => {
			eco => 'B37k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN1BPPP/R1BQ1RK1 b - - 3 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2PN4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A39g',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 d6

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... d6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 b - - 1 9' => {
			eco => 'B38n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2ppbp/p1np1np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 0 10',
				'c6d4' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2PnP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 0 10',
				'c8d7' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 2 10',
				'f6d7' => 'r1bq1rk1/pp1nppbp/2np2p1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 2 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'B38l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 b - - 1 9',
				'f2f3' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2PNP3/2N1BP2/PP2B1PP/R2QK2R b KQ - 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2PNP3/2N1BP2/PP2B1PP/R2QK2R b KQ - 0 9' => {
			eco => 'B38m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. f3

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9. f3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2PNPP2/2N1B3/PP2B1PP/R2QK2R b KQ - 2 9' => {
			eco => 'E79a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line'),
			moves => {
				'c6d4' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2PnPP2/2N1B3/PP2B1PP/R2QK2R w KQ - 0 10',
				'c8d7' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNPP2/2N1B3/PP2B1PP/R2QK2R w KQ - 3 10',
				'c8g4' => 'r2q1rk1/pp2ppbp/2np1np1/8/2PNPPb1/2N1B3/PP2B1PP/R2QK2R w KQ - 3 10',
				'f6g4' => 'r1bq1rk1/pp2ppbp/2np2p1/8/2PNPPn1/2N1B3/PP2B1PP/R2QK2R w KQ - 3 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/2PNPP2/2N5/PP2B1PP/R1BQK2R w KQ - 1 9' => {
			eco => 'E78',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 7. Nf3 cxd4'),
			moves => {
				'c1e3' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2PNPP2/2N1B3/PP2B1PP/R2QK2R b KQ - 2 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP1P1/2N1BP2/PPPQ3P/R3KB1R b KQ g3 0 9' => {
			eco => 'B76d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. g4

			variation => N__('Sicilian: Dragon, Yugoslav, 9. g4'),
			moves => {
				'c6d4' => 'r1bq1rk1/pp2ppbp/3p1np1/8/3nP1P1/2N1BP2/PPPQ3P/R3KB1R w KQ - 0 10',
				'c8e6' => 'r2q1rk1/pp2ppbp/2npbnp1/8/3NP1P1/2N1BP2/PPPQ3P/R3KB1R w KQ - 1 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/1BN1B2P/PPP2PP1/R2QK2R b KQ - 0 9' => {
			eco => 'B35o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 d6 9. h3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 d6 9. h3'),
			moves => {
				'c8d7' => 'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1B2P/PPP2PP1/R2QK2R w KQ - 1 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ - 0 9' => {
			eco => 'B35o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 d6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 d6'),
			moves => {
				'f2f3' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/1BN1BP2/PPP3PP/R2QK2R b KQ - 0 9',
				'h2h3' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/1BN1B2P/PPP2PP1/R2QK2R b KQ - 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/1BN1BP2/PPP3PP/R2QK2R b KQ - 0 9' => {
			eco => 'B35q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 d6 9. f3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 d6 9. f3'),
			moves => {
				'c8d7' => 'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1BP2/PPP3PP/R2QK2R w KQ - 1 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1R1K b - - 7 9' => {
			eco => 'B73',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Kh1

			variation => N__('Sicilian: Dragon, Classical, 9. Kh1'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w - - 6 9' => {
			eco => 'B73',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O

			variation => N__('Sicilian: Dragon, Classical, 8.O-O O-O'),
			moves => {
				'd1d2' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/R4RK1 b - - 7 9',
				'd4b3' => 'r1bq1rk1/pp2ppbp/2np1np1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 b - - 7 9',
				'f2f4' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b - f3 0 9',
				'g1h1' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1R1K b - - 7 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/2KR3R b - - 7 9' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. Qd2 O-O 9. O-O-O

			variation => N__('Sicilian: Dragon, Classical, Grigoriev Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/R3K2R w KQ - 6 9' => {
			eco => 'B72',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Amsterdam Variation'),
			moves => {
				'e1c1' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/2KR3R b - - 7 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/R4RK1 b - - 7 9' => {
			eco => 'B73',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Qd2

			variation => N__('Sicilian: Dragon, Classical, Richter Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b - - 4 9' => {
			eco => 'B76g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O'),
			moves => {
				'c6d4' => 'r1bq1rk1/pp2ppbp/3p1np1/8/3nP3/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 10',
				'c8d7' => 'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R w - - 5 10',
				'd6d5' => 'r1bq1rk1/pp2ppbp/2n2np1/3p4/3NP3/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQ - 3 9' => {
			eco => 'B76c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6

			variation => N__('Sicilian: Dragon, Yugoslav, 8. Qd2 Nc6'),
			moves => {
				'e1c1' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b - - 4 9',
				'f1c4' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/R3K2R b KQ - 4 9',
				'g2g4' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP1P1/2N1BP2/PPPQ3P/R3KB1R b KQ g3 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N5/PPP1BPPP/R1BQR1K1 w - - 6 9' => {
			eco => 'B70p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Re1 Nc6

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Re1 Nc6'),
			moves => {
				'd4b3' => 'r1bq1rk1/pp2ppbp/2np1np1/8/4P3/1NN5/PPP1BPPP/R1BQR1K1 b - - 7 9',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b - f3 0 9' => {
			eco => 'B73',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. f4

			variation => N__('Sicilian: Dragon, Classical, 9. f4'),
			moves => {
				'd8b6' => 'r1b2rk1/pp2ppbp/1qnp1np1/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 b - - 7 9' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3

			variation => N__('Sicilian: Dragon, Classical, 9. Nb3'),
			moves => {
				'a7a5' => 'r1bq1rk1/1p2ppbp/2np1np1/p7/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - a6 0 10',
				'a7a6' => 'r1bq1rk1/1p2ppbp/p1np1np1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 0 10',
				'b7b6' => 'r1bq1rk1/p3ppbp/1pnp1np1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 0 10',
				'c8e6' => 'r2q1rk1/pp2ppbp/2npbnp1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 8 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w - - 6 9' => {
			eco => 'B70n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O Nc6 8. Nb3 O-O

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O Nc6 8. Nb3 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np1np1/8/4P3/1NN5/PPP1BPPP/R1BQR1K1 b - - 7 9' => {
			eco => 'B70q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Re1 Nc6 9. Nb3

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Re1 Nc6 9. Nb3'),
			moves => {
				'c8e6' => 'r2q1rk1/pp2ppbp/2npbnp1/8/4P3/1NN5/PPP1BPPP/R1BQR1K1 w - - 8 10',
			},
		},
		'r1bq1rk1/pp2ppbp/2np2p1/2n5/1PP1P3/2N5/P1NBBPPP/R2Q1RK1 b - b3 0 11' => {
			eco => 'B37q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O Nd7 10. Bd2 Nc5 11. b4

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O Nd7 10. Bd2 Nc5 11. b4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np2p1/2n5/2P1P3/2N5/PPNBBPPP/R2Q1RK1 w - - 6 11' => {
			eco => 'B37p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O Nd7 10. Bd2 Nc5

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O Nd7 10. Bd2 Nc5'),
			moves => {
				'b2b4' => 'r1bq1rk1/pp2ppbp/2np2p1/2n5/1PP1P3/2N5/P1NBBPPP/R2Q1RK1 b - b3 0 11',
			},
		},
		'r1bq1rk1/pp2ppbp/2np2p1/8/2PN2n1/2N1P1P1/PP3PBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A39i',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Ng4 9. e3 d6

			variation => N__('English: Symmetrical, Main Line 7. d4, 8... Ng4 9. e3 d6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/2np2p1/8/2PNPPn1/2N1B3/PP2B1PP/R2QK2R w KQ - 3 10' => {
			eco => 'E79c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3 Ng4

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line, 9... Ng4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/4n3/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 5 10' => {
			eco => 'B77c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Ne5

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Ne5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/6B1/2P1P3/2N5/PP1QBPPP/R3K2R b KQ - 4 10' => {
			eco => 'B36m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Bg5 O-O 10. Qd2

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Bg5 O-O 10. Qd2'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/6B1/2PQP3/2N5/PP2BPPP/R3K2R w KQ - 3 10' => {
			eco => 'B36l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Bg5 O-O

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Bg5 O-O'),
			moves => {
				'd4d2' => 'r1bq1rk1/pp2ppbp/3p1np1/6B1/2P1P3/2N5/PP1QBPPP/R3K2R b KQ - 4 10',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2BnP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 0 10' => {
			eco => 'B77e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Nxd4

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Nxd4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2P1P3/2N1B3/PP1QBPPP/R3K2R b KQ - 4 10' => {
			eco => 'B36o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Be3 O-O 10. Qd2

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Be3 O-O 10. Qd2'),
			moves => {
				'c8e6' => 'r2q1rk1/pp2ppbp/3pbnp1/8/2P1P3/2N1B3/PP1QBPPP/R3K2R w KQ - 5 11',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2P5/2N3P1/PP1QPPBP/R1B2RK1 b - - 1 10' => {
			eco => 'A39l',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd2

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd2'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2P5/2NQ2P1/PP2PPBP/R1B2RK1 b - - 1 10' => {
			eco => 'A39m',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2ppbp/p2p1np1/8/2P5/2NQ2P1/PP2PPBP/R1B2RK1 w - - 0 11',
				'a8b8' => '1rbq1rk1/pp2ppbp/3p1np1/8/2P5/2NQ2P1/PP2PPBP/R1B2RK1 w - - 2 11',
				'c8f5' => 'r2q1rk1/pp2ppbp/3p1np1/5b2/2P5/2NQ2P1/PP2PPBP/R1B2RK1 w - - 2 11',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2PQ4/2N3P1/PP2PPBP/R1B2RK1 w - - 0 10' => {
			eco => 'A39k',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6

			variation => N__('English: Symmetrical, Main Line 7. d4, 9... d6'),
			moves => {
				'd4d2' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2P5/2N3P1/PP1QPPBP/R1B2RK1 b - - 1 10',
				'd4d3' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2P5/2NQ2P1/PP2PPBP/R1B2RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2PQP3/2N1B3/PP2BPPP/R3K2R w KQ - 3 10' => {
			eco => 'B36n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Be3 O-O

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Be3 O-O'),
			moves => {
				'd4d2' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2P1P3/2N1B3/PP1QBPPP/R3K2R b KQ - 4 10',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2PnP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'B38q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Nxd4

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Nxd4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/2PnPP2/2N1B3/PP2B1PP/R2QK2R w KQ - 0 10' => {
			eco => 'E79b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3 Nxd4

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line, 9...Nxd4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/3BP3/2N2P2/PPPQ2PP/2KR1B1R b - - 0 10' => {
			eco => 'B76j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Nxd4'),
			moves => {
				'c8e6' => 'r2q1rk1/pp2ppbp/3pbnp1/8/3BP3/2N2P2/PPPQ2PP/2KR1B1R w - - 1 11',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/3nP1P1/2N1BP2/PPPQ3P/R3KB1R w KQ - 0 10' => {
			eco => 'B76f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. g4 Nxd4

			variation => N__('Sicilian: Dragon, Yugoslav, 9. g4 Nxd4'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/8/3nP3/2N1BP2/PPPQ2PP/2KR1B1R w - - 0 10' => {
			eco => 'B76j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Nxd4

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Nxd4'),
			moves => {
				'e3d4' => 'r1bq1rk1/pp2ppbp/3p1np1/8/3BP3/2N2P2/PPPQ2PP/2KR1B1R b - - 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/n1pP4/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E66c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno'),
			moves => {
				'f3d2' => 'r1bq1rk1/pp2ppbp/3p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 2 9',
			},
		},
		'r1bq1rk1/pp2ppbp/3p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 2 9' => {
			eco => 'E66b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno'),
			moves => {
				'a7a6' => 'r1bq1rk1/1p2ppbp/p2p1np1/n1pP4/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 0 10',
				'e7e5' => 'r1bq1rk1/pp3pbp/3p1np1/n1pPp3/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 w - e6 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/6p1/2pPN3/4P3/2P5/P3BPPP/1RBQK2R b K - 0 11' => {
			eco => 'D85o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6'),
			moves => {
				'g7e5' => 'r1bq1rk1/pp2pp1p/6p1/2pPb3/4P3/2P5/P3BPPP/1RBQK2R w K - 0 12',
			},
		},
		'r1bq1rk1/pp2ppbp/6p1/2pPn3/4P3/2P2N2/P3BPPP/1RBQK2R w K - 1 11' => {
			eco => 'D85o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6'),
			moves => {
				'f3e5' => 'r1bq1rk1/pp2ppbp/6p1/2pPN3/4P3/2P5/P3BPPP/1RBQK2R b K - 0 11',
			},
		},
		'r1bq1rk1/pp2ppbp/6p1/n1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11' => {
			eco => 'D87i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Na5

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Na5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/6p1/n7/2BPP3/4B3/P3NPPP/R2Q1RK1 w - - 1 12' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Na5

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4 Na5'),
			moves => {
				'c4d3' => 'r1bq1rk1/pp2ppbp/6p1/n7/3PP3/3BB3/P3NPPP/R2Q1RK1 b - - 2 12',
			},
		},
		'r1bq1rk1/pp2ppbp/6p1/n7/3PP3/3BB3/P3NPPP/R2Q1RK1 b - - 2 12' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Na5 12. Bd3

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4 Na5 12. Bd3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/n1pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E91n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Na6 7. O-O c6

			variation => N__('King\'s Indian: Kazakh Variation, 7.O-O c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/n2p1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - c6 0 8' => {
			eco => 'E91m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Na6 7. O-O c5

			variation => N__('King\'s Indian: Kazakh Variation, 7.O-O c5'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/n2p1np1/2p5/3PPP2/2NB1N2/PPP3PP/R1BQ1RK1 w - c6 0 8' => {
			eco => 'B09m',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3 Na6 7. O-O c5

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Na6 7.O-O c5'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/4PP2/2NB1N2/PPP3PP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQK2R w KQ - 1 8' => {
			eco => 'E64e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. d5 Na6

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7. d5 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/4P3/2N2N1P/PPP1BPP1/R1BQ1RK1 b - - 0 8' => {
			eco => 'A43z',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Be2 O-O 7. O-O Na6 8. h3

			variation => N__('Old Benoni: Schmidt, 6. Be2 O-O 7.O-O Na6 8. h3'),
			moves => {
			},
		},
		'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 6 8' => {
			eco => 'A43y',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Be2 O-O 7. O-O Na6

			variation => N__('Old Benoni: Schmidt, 6. Be2 O-O 7.O-O Na6'),
			moves => {
				'h2h3' => 'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/4P3/2N2N1P/PPP1BPP1/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/4PP2/2NB1N2/PPP3PP/R1BQ1RK1 b - - 0 8' => {
			eco => 'B09m',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3 Na6 7. O-O c5 8. d5

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Na6 7.O-O c5 8. d5'),
			moves => {
				'c8g4' => 'r2q1rk1/pp2ppbp/n2p1np1/2pP4/4PPb1/2NB1N2/PPP3PP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pp2ppbp/n4np1/2p5/2QPP3/2N2N2/PP2BPPP/R1B1K2R w KQ c6 0 9' => {
			eco => 'D97o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'd4d5' => 'r1bq1rk1/pp2ppbp/n4np1/2pP4/2Q1P3/2N2N2/PP2BPPP/R1B1K2R b KQ - 0 9',
			},
		},
		'r1bq1rk1/pp2ppbp/n4np1/2pP4/2Q1P3/2N2N2/PP2BPPP/R1B1K2R b KQ - 0 9' => {
			eco => 'D97o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'e7e6' => 'r1bq1rk1/pp3pbp/n3pnp1/2pP4/2Q1P3/2N2N2/PP2BPPP/R1B1K2R w KQ - 0 10',
			},
		},
		'r1bq1rk1/pp2ppbp/n5p1/2Pn4/8/5NP1/PP2PPBP/RNBQ1RK1 w - - 1 9' => {
			eco => 'D75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. dxc5 Na6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8.dxc5 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3p1p/4p1p1/2pPb3/4P3/2P5/P2QBPPP/1RB1K2R w K - 0 13' => {
			eco => 'D85p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 Nc6 10. d5 Ne5 11. Nxe5 Bxe5 12. Qd2 e6

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Nc6, Main Line, 12... e6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/1nn1p1p1/3pN3/3P1P2/2N1B1P1/PP2P1BP/R2Q1RK1 w - - 3 12' => {
			eco => 'D79n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3 Nfd7 10. f4 Nc6 11. Be3 Nb6

			variation => N__('Neo-Grünfeld, Main Line, 10. f4 Nc6 11. Be3 Nb6'),
			moves => {
				'e3f2' => 'r1bq1rk1/pp3pbp/1nn1p1p1/3pN3/3P1P2/2N3P1/PP2PBBP/R2Q1RK1 b - - 4 12',
			},
		},
		'r1bq1rk1/pp3pbp/1nn1p1p1/3pN3/3P1P2/2N3P1/PP2PBBP/R2Q1RK1 b - - 4 12' => {
			eco => 'D79n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3 Nfd7 10. f4 Nc6 11. Be3 Nb6 12. Bf2

			variation => N__('Neo-Grünfeld, Main Line, 10. f4 Nc6 11. Be3 Nb6 12. Bf2'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/2n1pnp1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D79h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 Nc6 9. Ne5 e6

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 Nc6 9. Ne5 e6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/3p1np1/n1pPp3/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 w - e6 0 10' => {
			eco => 'E66d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O Nc6 8. d5 Na5 9. Nd2 e5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav Panno, 9... e5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/3pp1p1/2p1Pn2/3n1P2/2NPBNP1/PPP3BP/R2Q1RK1 w - - 1 11' => {
			eco => 'B25x',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O O-O 9. Be3 Nd4 10. e5 Nef5

			variation => N__('Sicilian: Closed, 6. f4 e6: 9. Be3 Nd4 10. e5 Nef5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/n1pp1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 0 9' => {
			eco => 'E73t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6 7. Qd2 e5 8. d5 c6

			variation => N__('King\'s Indian: Averbakh, 6... Na6 7. Qd2 e5 8. d5 c6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/n1pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9' => {
			eco => 'E94k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Re1 c6

			variation => N__('King\'s Indian: Glek, 8. Re1 c6'),
			moves => {
				'e2f1' => 'r1bq1rk1/pp3pbp/n1pp1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 1 9',
			},
		},
		'r1bq1rk1/pp3pbp/n1pp1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 1 9' => {
			eco => 'E94k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Re1 c6 9. Bf1

			variation => N__('King\'s Indian: Glek, 8. Re1 c6 9. Bf1'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Na6

			variation => N__('Benoni: Classical, 9.O-O Na6'),
			moves => {
				'f3d2' => 'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 7 10',
			},
		},
		'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 7 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Na6 10. Nd2

			variation => N__('Benoni: Classical, 9.O-O Na6 10. Nd2'),
			moves => {
				'a6c7' => 'r1bq1rk1/ppn2pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 8 11',
			},
		},
		'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 3 10' => {
			eco => 'A70m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 Na6

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 w - - 6 10' => {
			eco => 'A65i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. Nge2 O-O 9. O-O Na6

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8.Nge2 O-O 9.O-O Na6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3pbp/n2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 10' => {
			eco => 'A62',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Na6

			variation => N__('Benoni: Fianchetto, 9.O-O Na6'),
			moves => {
				'f3d2' => 'r1bq1rk1/pp3pbp/n2p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 6 10',
			},
		},
		'r1bq1rk1/pp3pbp/n2p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 6 10' => {
			eco => 'A62',
                        # TRANSLATORS:

			variation => N__('Benoni: Fianchetto, 9.O-O Na6'),
			moves => {
				'a6c7' => 'r1bq1rk1/ppn2pbp/3p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 7 11',
			},
		},
		'r1bq1rk1/pp3pbp/n2ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQ1RK1 b - - 3 9' => {
			eco => 'E90w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 Na6 9. O-O

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 Na6 9.O-O'),
			moves => {
				'a6c7' => 'r1bq1rk1/ppn2pbp/3ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQ1RK1 w - - 4 10',
			},
		},
		'r1bq1rk1/pp3pbp/n2ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 2 9' => {
			eco => 'E90w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 Na6

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 Na6'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp3pbp/n2ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQ1RK1 b - - 3 9',
			},
		},
		'r1bq1rk1/pp3pbp/n3pnp1/2pP4/2Q1P3/2N2N2/PP2BPPP/R1B1K2R w KQ - 0 10' => {
			eco => 'D97o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp3pbp/n3pnp1/2pP4/2Q1P3/2N2N2/PP2BPPP/R1B2RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/pp3pbp/n3pnp1/2pP4/2Q1P3/2N2N2/PP2BPPP/R1B2RK1 b - - 1 10' => {
			eco => 'D97o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'e6d5' => 'r1bq1rk1/pp3pbp/n4np1/2pp4/2Q1P3/2N2N2/PP2BPPP/R1B2RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp3pbp/n4np1/2pP4/2Q5/2N2N2/PP2BPPP/R1B2RK1 b - - 0 11' => {
			eco => 'D97o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'c8f5' => 'r2q1rk1/pp3pbp/n4np1/2pP1b2/2Q5/2N2N2/PP2BPPP/R1B2RK1 w - - 1 12',
			},
		},
		'r1bq1rk1/pp3pbp/n4np1/2pp4/2Q1P3/2N2N2/PP2BPPP/R1B2RK1 w - - 0 11' => {
			eco => 'D97o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'e4d5' => 'r1bq1rk1/pp3pbp/n4np1/2pP4/2Q5/2N2N2/PP2BPPP/R1B2RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n1p3/3n4/1b1P4/2NB1N2/PP1B1PPP/R2QK2R w KQ - 4 10' => {
			eco => 'B14x',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5 8. Bd2 Nc6 9. Bd3 O-O

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Bd2 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1p3/8/2BPP3/5N2/P2Q1PPP/R3K2R w KQ - 3 12' => {
			eco => 'D41n',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4, Main Line'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp3ppp/2n1p3/8/2BPP3/5N2/P2Q1PPP/R4RK1 b - - 4 12',
			},
		},
		'r1bq1rk1/pp3ppp/2n1p3/8/2BPP3/5N2/P2Q1PPP/R4RK1 b - - 4 12' => {
			eco => 'D41n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+ 9. Bd2 Bxd2+ 10. Qxd2 O-O 11. Bc4 Nc6 12. O-O

			variation => N__('QGD: Semi-Tarrasch, 6. e4, Main Line'),
			moves => {
				'b7b6' => 'r1bq1rk1/p4ppp/1pn1p3/8/2BPP3/5N2/P2Q1PPP/R4RK1 w - - 0 13',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pb2/3n4/3P4/2NB1N2/PP3PPP/R1BQR1K1 w - - 5 11' => {
			eco => 'D42p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Bf6

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Bf6'),
			moves => {
				'd3e4' => 'r1bq1rk1/pp3ppp/2n1pb2/3n4/3PB3/2N2N2/PP3PPP/R1BQR1K1 b - - 6 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pb2/3n4/3P4/P1NB1N2/1P3PPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'D42i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. a3 Bf6

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. a3 Bf6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pb2/3n4/3PB3/2N2N2/PP3PPP/R1BQR1K1 b - - 6 11' => {
			eco => 'D42p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Bf6 11. Be4

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line, 10. Re1 Bf6 11. Be4'),
			moves => {
				'c6e7' => 'r1bq1rk1/pp2nppp/4pb2/3n4/3PB3/2N2N2/PP3PPP/R1BQR1K1 w - - 7 12',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2bp4/2P2B2/2N1PN2/PPQ2PPP/R3KB1R w KQ - 2 9' => {
			eco => 'D37r',
                        # TRANSLATORS:

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. Qc2'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp3ppp/2n1pn2/2bp4/2P2B2/P1N1PN2/1PQ2PPP/R3KB1R b KQ - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2bp4/2P2B2/P1N1PN2/1PQ2PPP/R3KB1R b KQ - 0 9' => {
			eco => 'D37s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. Qc2 Nc6 9. a3

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. Qc2 Nc6 9. a3'),
			moves => {
				'd8a5' => 'r1b2rk1/pp3ppp/2n1pn2/q1bp4/2P2B2/P1N1PN2/1PQ2PPP/R3KB1R w KQ - 1 10',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2bp4/8/2PBPN2/PP1N1PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'D05n',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3 Nc6 6. Nbd2 Bd6 7. O-O O-O 8. dxc5 Bxc5

			variation => N__('Colle: 5. c3 Nc6 6.Nbd2 Bd6 7.O-O O-O 8.dxc5 Bxc5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Nc6

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4 8.Bxc4 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2p5/1bBP4/P1N1PN2/1P3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 dxc4 9.Bxc4'),
			moves => {
				'b4a5' => 'r1bq1rk1/pp3ppp/2n1pn2/b1p5/2BP4/P1N1PN2/1P3PPP/R1BQ1RK1 w - - 1 10',
				'c5d4' => 'r1bq1rk1/pp3ppp/2n1pn2/8/1bBp4/P1N1PN2/1P3PPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2p5/1bpP4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 dxc4'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp3ppp/2n1pn2/2p5/1bBP4/P1N1PN2/1P3PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2p5/2BP4/P1P1PN2/5PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4 10. Bxc4

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4 10.Bxc4'),
			moves => {
				'd8c7' => 'r1b2rk1/ppq2ppp/2n1pn2/2p5/2BP4/P1P1PN2/5PPP/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2p5/2pP4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E59',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3 dxc4

			variation => N__('Nimzo-Indian: Main Line, 9.bxc3 dxc4'),
			moves => {
				'd3c4' => 'r1bq1rk1/pp3ppp/2n1pn2/2p5/2BP4/P1P1PN2/5PPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2pP4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. cxd5

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8.cxd5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6'),
			moves => {
				'a2a3' => 'r1bq1rk1/pp3ppp/2n1pn2/2pp4/1bPP4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 8',
				'c4d5' => 'r1bq1rk1/pp3ppp/2n1pn2/2pP4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2pp4/1bPP4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3'),
			moves => {
				'b4a5' => 'r1bq1rk1/pp3ppp/2n1pn2/b1pp4/2PP4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 1 9',
				'b4c3' => 'r1bq1rk1/pp3ppp/2n1pn2/2pp4/2PP4/P1bBPN2/1P3PPP/R1BQ1RK1 w - - 0 9',
				'c5d4' => 'r1bq1rk1/pp3ppp/2n1pn2/3p4/1bPp4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 0 9',
				'd5c4' => 'r1bq1rk1/pp3ppp/2n1pn2/2p5/1bpP4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2pp4/2PP4/P1PBPN2/5PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3 9. bxc3

			variation => N__('Nimzo-Indian: Main Line, 8...Bxc3'),
			moves => {
				'b7b6' => 'r1bq1rk1/p4ppp/1pn1pn2/2pp4/2PP4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 10',
				'd5c4' => 'r1bq1rk1/pp3ppp/2n1pn2/2p5/2pP4/P1PBPN2/5PPP/R1BQ1RK1 w - - 0 10',
				'd8c7' => 'r1b2rk1/ppq2ppp/2n1pn2/2pp4/2PP4/P1PBPN2/5PPP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/2pp4/2PP4/P1bBPN2/1P3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E58',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Bxc3

			variation => N__('Nimzo-Indian: Main Line, 8...Bxc3'),
			moves => {
				'b2c3' => 'r1bq1rk1/pp3ppp/2n1pn2/2pp4/2PP4/P1PBPN2/5PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/3p4/1bPP4/P1NB1N2/1P3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 cxd4 9. exd4

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 cxd4 9.exd4'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/3p4/1bPp4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 cxd4

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 cxd4'),
			moves => {
				'e3d4' => 'r1bq1rk1/pp3ppp/2n1pn2/3p4/1bPP4/P1NB1N2/1P3PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/8/1bBP4/P1N2N2/1P3PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4 10. exd4

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4 10.exd4'),
			moves => {
				'b4e7' => 'r1bq1rk1/pp2bppp/2n1pn2/8/2BP4/P1N2N2/1P3PPP/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/8/1bBp4/P1N1PN2/1P3PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E57',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 cxd4

			variation => N__('Nimzo-Indian: Main Line, 8...dxc4 9.Bxc4 cxd4'),
			moves => {
				'e3d4' => 'r1bq1rk1/pp3ppp/2n1pn2/8/1bBP4/P1N2N2/1P3PPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/b1p5/2BP4/P1N1PN2/1P3PPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 dxc4 9. Bxc4 Ba5

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 dxc4 9.Bxc4 Ba5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/b1pP4/3P4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Ba5 9. cxd5

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 Ba5 9.cxd5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n1pn2/b1pp4/2PP4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E56',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Nc6 8. a3 Ba5

			variation => N__('Nimzo-Indian: Main Line, 7... Nc6 8. a3 Ba5'),
			moves => {
				'c4d5' => 'r1bq1rk1/pp3ppp/2n1pn2/b1pP4/3P4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2n2B2/2b5/3p4/2N2NP1/PP2PPBP/R2Q1RK1 b - - 0 11' => {
			eco => 'D34l',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4'),
			moves => {
				'd8f6' => 'r1b2rk1/pp3ppp/2n2q2/2b5/3p4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/2b3B1/3p4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 11' => {
			eco => 'D34l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Bg5 d4

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4'),
			moves => {
				'g5f6' => 'r1bq1rk1/pp3ppp/2n2B2/2b5/3p4/2N2NP1/PP2PPBP/R2Q1RK1 b - - 0 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/2bp2B1/8/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 10' => {
			eco => 'D34k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Bg5

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5'),
			moves => {
				'c8e6' => 'r2q1rk1/pp3ppp/2n1bn2/2bp2B1/8/2N2NP1/PP2PPBP/R2Q1RK1 w - - 2 11',
				'd5d4' => 'r1bq1rk1/pp3ppp/2n2n2/2b3B1/3p4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/2bp4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D34i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5'),
			moves => {
				'c1g5' => 'r1bq1rk1/pp3ppp/2n2n2/2bp2B1/8/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 10',
				'c3a4' => 'r1bq1rk1/pp3ppp/2n2n2/2bp4/N7/5NP1/PP2PPBP/R1BQ1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/2bp4/N7/5NP1/PP2PPBP/R1BQ1RK1 b - - 1 10' => {
			eco => 'D34j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Na4

			variation => N__('QGD Tarrasch: Reti Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/3p4/8/P1NB4/1PP2PPP/R1BQ1RK1 b - - 3 11' => {
			eco => 'B45u',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3, 9.ed5 ed5 10. Bd3 O-O'),
			moves => {
				'd5d4' => 'r1bq1rk1/pp3ppp/2n2n2/8/3p4/P1NB4/1PP2PPP/R1BQ1RK1 w - - 0 12',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/3p4/8/P1NB4/1PP2PPP/R1BQK2R w KQ - 2 11' => {
			eco => 'B45u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. a3 Bxc3+ 8. Nxc3 d5 9. exd5 exd5 10. Bd3 O-O

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3, 9.ed5 ed5 10. Bd3 O-O'),
			moves => {
				'e1g1' => 'r1bq1rk1/pp3ppp/2n2n2/3p4/8/P1NB4/1PP2PPP/R1BQ1RK1 b - - 3 11',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/8/3p4/P1NB4/1PP2PPP/R1BQ1RK1 w - - 0 12' => {
			eco => 'B45v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. a3 Bxc3+ 8. Nxc3 d5 9. exd5 exd5 10. Bd3 O-O 11. O-O d4

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3, 10. Bd3 O-O 11.O-O d4'),
			moves => {
				'c3e2' => 'r1bq1rk1/pp3ppp/2n2n2/8/3p4/P2B4/1PP1NPPP/R1BQ1RK1 b - - 1 12',
			},
		},
		'r1bq1rk1/pp3ppp/2n2n2/8/3p4/P2B4/1PP1NPPP/R1BQ1RK1 b - - 1 12' => {
			eco => 'B45w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. a3 Bxc3+ 8. Nxc3 d5 9. exd5 exd5 10. Bd3 O-O 11. O-O d4 12. Ne2

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3, 10. Bd3 O-O 11.O-O d4 12. Ne2'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n5/2bN4/3p4/5NP1/PP2PPBP/R2Q1RK1 w - - 2 13' => {
			eco => 'D34m',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4 11.Bxf6 Qxf6 12. Nd5'),
			moves => {
				'f3d2' => 'r1bq1rk1/pp3ppp/2n5/2bN4/3p4/6P1/PP1NPPBP/R2Q1RK1 b - - 3 13',
			},
		},
		'r1bq1rk1/pp3ppp/2n5/2bN4/3p4/6P1/PP1NPPBP/R2Q1RK1 b - - 3 13' => {
			eco => 'D34m',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4 11.Bxf6 Qxf6 12. Nd5'),
			moves => {
				'f8e8' => 'r1bqr1k1/pp3ppp/2n5/2bN4/3p4/6P1/PP1NPPBP/R2Q1RK1 w - - 4 14',
			},
		},
		'r1bq1rk1/pp3ppp/2n5/2bp2B1/8/5NP1/PP2PPBP/R2Q1RK1 b - - 1 11' => {
			eco => 'D41j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O 9. Nxd5 exd5 10. dxc5 Bxc5 11. Bg5

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9.Nxd5 exd5 10.dxc5 Bxc5 11. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2n5/2bp4/8/5NP1/PP2PPBP/R1BQ1RK1 w - - 0 11' => {
			eco => 'D41i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7 8. O-O O-O 9. Nxd5 exd5 10. dxc5 Bxc5

			variation => N__('QGD: Semi-Tarrasch, 6. g3, Main Line, 9.Nxd5 exd5 10.dxc5 Bxc5'),
			moves => {
				'c1g5' => 'r1bq1rk1/pp3ppp/2n5/2bp2B1/8/5NP1/PP2PPBP/R2Q1RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/pp3ppp/2nb1n2/3p4/3N4/1PN1P3/P3BPPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D32x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8. Nxd4 Bd6 9. O-O O-O 10. b3

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8.Nxd4 Bd6 9.O-O O-O 10. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2nb1n2/3p4/3N4/2N1P3/PP2BPPP/R1BQ1RK1 w - - 3 10' => {
			eco => 'D32w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8. Nxd4 Bd6 9. O-O O-O

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8.Nxd4 Bd6 9.O-O O-O'),
			moves => {
				'b2b3' => 'r1bq1rk1/pp3ppp/2nb1n2/3p4/3N4/1PN1P3/P3BPPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/pp3ppp/2nbpn2/2Pp4/8/2PBPN2/PP1N1PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'D05m',
                        # TRANSLATORS:

			variation => N__('Colle: 5. c3 Nc6 6.Nbd2 Bd6 7.O-O O-O'),
			moves => {
				'd6c5' => 'r1bq1rk1/pp3ppp/2n1pn2/2bp4/8/2PBPN2/PP1N1PPP/R1BQ1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pp3ppp/2nbpn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8' => {
			eco => 'D40h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. Bd3 Bd6 7. O-O O-O

			variation => N__('QGD: Semi-Tarrasch, Symmetrical Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pp3ppp/2nbpn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQ1RK1 w - - 5 8' => {
			eco => 'D05m',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3 Nc6 6. Nbd2 Bd6 7. O-O O-O

			variation => N__('Colle: 5. c3 Nc6 6.Nbd2 Bd6 7.O-O O-O'),
			moves => {
				'd4c5' => 'r1bq1rk1/pp3ppp/2nbpn2/2Pp4/8/2PBPN2/PP1N1PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pp3ppp/2nppn2/2p5/2PP4/P1PBP3/4NPPP/R1BQK2R w KQ - 0 9' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 d6

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 d6'),
			moves => {
			},
		},
		'r1bq1rk1/pp4bp/2nppnp1/2p5/2P1PPP1/2N5/PP2B2P/R1BQK1NR w KQ - 1 10' => {
			eco => 'E77',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5'),
			moves => {
				'h2h4' => 'r1bq1rk1/pp4bp/2nppnp1/2p5/2P1PPPP/2N5/PP2B3/R1BQK1NR b KQ h3 0 10',
			},
		},
		'r1bq1rk1/pp4bp/2nppnp1/2p5/2P1PPPP/2N5/PP2B3/R1BQK1NR b KQ h3 0 10' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. d5 e6 8. dxe6 fxe6 9. g4 Nc6 10. h4

			variation => N__('King\'s Indian: Four Pawns Attack, Six Pawns line'),
			moves => {
			},
		},
		'r1bq1rk1/pp4pp/2nbpn2/3p4/3P1B2/3B1N2/PP2NPPP/R2Q1RK1 b - - 5 12' => {
			eco => 'C06t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3 O-O 12. Bf4

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6, 11...O-O 12. Bf4'),
			moves => {
			},
		},
		'r1bq1rk1/pp4pp/2nbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 w - - 4 12' => {
			eco => 'C06s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3 O-O

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6, 11...O-O'),
			moves => {
				'c1f4' => 'r1bq1rk1/pp4pp/2nbpn2/3p4/3P1B2/3B1N2/PP2NPPP/R2Q1RK1 b - - 5 12',
			},
		},
		'r1bq1rk1/ppn2pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 8 11' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Na6 10. Nd2 Nc7

			variation => N__('Benoni: Classical, 9.O-O Na6 10. Nd2 Nc7'),
			moves => {
			},
		},
		'r1bq1rk1/ppn2pbp/3p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 7 11' => {
			eco => 'A62',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Na6 10. Nd2 Nc7

			variation => N__('Benoni: Fianchetto, 9.O-O Na6 10. Nd2 Nc7'),
			moves => {
			},
		},
		'r1bq1rk1/ppn2pbp/3ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQ1RK1 w - - 4 10' => {
			eco => 'E90w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 Na6 9. O-O Nc7

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 Na6 9.O-O Nc7'),
			moves => {
			},
		},
		'r1bq1rk1/ppnn1pbp/3p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQ1RK1 w - - 5 12' => {
			eco => 'A67s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 O-O 10. Nf3 Na6 11. O-O Nc7

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 O-O 10. Nf3 Na6 11.O-O Nc7'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1b1pp/2n1pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 Nc6

			variation => N__('Dutch: Stonewall, Botvinnik, 7... Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1bppp/1nn5/4p3/8/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6 7. O-O Be7 8. d3 O-O

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6 7.O-O Be7 8. d3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1bppp/2N1pn2/8/2pP4/6P1/PP2PPBP/RNBQ1RK1 b - - 0 8' => {
			eco => 'E05h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Ne5 Nc6 8. Nxc6

			variation => N__('Catalan: Open, Classical, 7. Ne5 Nc6 8.Nxc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1bppp/2n1pn2/4N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 w - - 6 8' => {
			eco => 'E05g',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, Classical, 7. Ne5'),
			moves => {
				'e5c6' => 'r1bq1rk1/ppp1bppp/2N1pn2/8/2pP4/6P1/PP2PPBP/RNBQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/ppp1bppp/2n1pn2/8/3PN3/P5N1/1PP1BPPP/R1BQK2R w KQ - 5 9' => {
			eco => 'C15u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Be7 6. Nxe4 Nf6 7. N2g3 O-O 8. Be2 Nc6

			variation => N__('French: Winawer, Alekhine Gambit, Alatortsev Variation'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1bppp/2np1n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 w - - 0 7' => {
			eco => 'C55i',
                        # TRANSLATORS:

			variation => N__('Two Knights: 4. d3 Be7 5.O-O O-O 6. Bb3'),
			moves => {
				'c2c3' => 'r1bq1rk1/ppp1bppp/2np1n2/4p3/4P3/1BPP1N2/PP3PPP/RNBQ1RK1 b - - 0 7',
			},
		},
		'r1bq1rk1/ppp1bppp/2np1n2/4p3/4P3/1BPP1N2/PP3PPP/RNBQ1RK1 b - - 0 7' => {
			eco => 'C55j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. O-O O-O 6. Bb3 d6 7. c3

			variation => N__('Two Knights: 4. d3 Be7 5.O-O O-O 6. Bb3 d6 7. c3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1bppp/8/3p4/1nPPn3/5N2/PP2BPPP/RNBQ1RK1 w - - 3 10' => {
			eco => 'C42u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. c4 Nb4 9. Be2 O-O

			variation => N__('Russian Game: Classical, Jänisch, Main Line'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/2PPp3/4Pp2/2NN1P2/PP1BB1PP/R2Q1RK1 b - - 0 13' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Nd3 Nf6 12. Bd2 f4 13. c5

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Nd3 Nf6 12. Bd2 f4 13. c5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3Pp3/2P1Pp2/2NN1P2/PP1BB1PP/R2Q1RK1 w - - 0 13' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Nd3 Nf6 12. Bd2 f4

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Nd3 Nf6 12. Bd2 f4'),
			moves => {
				'c4c5' => 'r1bq1rk1/ppp1n1bp/3p1np1/2PPp3/4Pp2/2NN1P2/PP1BB1PP/R2Q1RK1 b - - 0 13',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P1P1/2N2P2/PP2B2P/R1BQNRK1 w - - 1 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. g4 Nf6

			variation => N__('King\'s Indian: Mar del Plata, Benko Attack, 11... Nf6'),
			moves => {
				'e1d3' => 'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P1P1/2NN1P2/PP2B2P/R1BQ1RK1 b - - 2 12',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P1P1/2NN1P2/PP2B2P/R1BQ1RK1 b - - 2 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. g4 Nf6 12. Nd3

			variation => N__('King\'s Indian: Mar del Plata, Benko Attack, 11... Nf6 12. Nd3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P3/2NN1P2/PP1BB1PP/R2Q1RK1 b - - 3 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Nd3 Nf6 12. Bd2

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Nd3 Nf6 12. Bd2'),
			moves => {
				'f5f4' => 'r1bq1rk1/ppp1n1bp/3p1np1/3Pp3/2P1Pp2/2NN1P2/PP1BB1PP/R2Q1RK1 w - - 0 13',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P3/2NN1P2/PP2B1PP/R1BQ1RK1 w - - 2 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Nd3 Nf6

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Nd3 Nf6'),
			moves => {
				'c1d2' => 'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P3/2NN1P2/PP1BB1PP/R2Q1RK1 b - - 3 12',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 w - - 2 12' => {
			eco => 'E98q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Nd3 f5 11. Bd2 Nf6

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Nd3 f5 11. Bd2 Nf6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3PppN1/1PP1P3/2N2B2/P4PPP/R1BQR1K1 b - - 3 12' => {
			eco => 'E97s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 f5 11. Ng5 Nf6 12. Bf3

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 f5 11. Ng5 Nf6 12. Bf3'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp2n1bp/2pp1np1/3PppN1/1PP1P3/2N2B2/P4PPP/R1BQR1K1 w - - 0 13',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p1np1/3PppN1/1PP1P3/2N5/P3BPPP/R1BQR1K1 w - - 2 12' => {
			eco => 'E97s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 f5 11. Ng5 Nf6

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 f5 11. Ng5 Nf6'),
			moves => {
				'e2f3' => 'r1bq1rk1/ppp1n1bp/3p1np1/3PppN1/1PP1P3/2N2B2/P4PPP/R1BQR1K1 b - - 3 12',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p2p1/3Ppp1n/1PP1P3/2N2N2/P3BPPP/R1BQR1K1 w - f6 0 11' => {
			eco => 'E97r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 f5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 f5'),
			moves => {
				'f3g5' => 'r1bq1rk1/ppp1n1bp/3p2p1/3PppNn/1PP1P3/2N5/P3BPPP/R1BQR1K1 b - - 1 11',
			},
		},
		'r1bq1rk1/ppp1n1bp/3p2p1/3PppNn/1PP1P3/2N5/P3BPPP/R1BQR1K1 b - - 1 11' => {
			eco => 'E97r',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 f5'),
			moves => {
				'h5f6' => 'r1bq1rk1/ppp1n1bp/3p1np1/3PppN1/1PP1P3/2N5/P3BPPP/R1BQR1K1 w - - 2 12',
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp1B1/2P1P3/2N2N2/PP2BPPP/R2Q1RK1 b - - 2 9' => {
			eco => 'E97d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Bg5

			variation => N__('King\'s Indian: Mar del Plata, Odessa Variation'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 b - b3 0 9' => {
			eco => 'E97f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack'),
			moves => {
				'a7a5' => 'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - a6 0 10',
				'f6e8' => 'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - - 1 10',
				'f6h5' => 'r1bq1rk1/ppp1npbp/3p2p1/3Pp2n/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - - 1 10',
				'g8h8' => 'r1bq1r1k/ppp1npbp/3p1np1/3Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 2 9' => {
			eco => 'E85j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 Nc6 8. d5 Ne7 9. Qd2

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 Nc6 8. d5 Ne7 9. Qd2'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 1 9' => {
			eco => 'E85i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 Nc6 8. d5 Ne7

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 Nc6 8. d5 Ne7'),
			moves => {
				'd1d2' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 2 9',
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N2N2/PP1BBPPP/R2Q1RK1 b - - 2 9' => {
			eco => 'E97e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Bd2

			variation => N__('King\'s Indian: Mar del Plata, 9. Bd2'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E97d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7

			variation => N__('King\'s Indian: Mar del Plata, 8. d5 Ne7'),
			moves => {
				'b2b4' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 b - b3 0 9',
				'c1d2' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N2N2/PP1BBPPP/R2Q1RK1 b - - 2 9',
				'c1g5' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp1B1/2P1P3/2N2N2/PP2BPPP/R2Q1RK1 b - - 2 9',
				'f3d2' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 2 9',
				'f3e1' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 b - - 2 9',
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 9' => {
			eco => 'E62v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O e5 8. d5 Ne7 9. e4

			variation => N__('King\'s Indian: Fianchetto, Uhlmann/Szabo, 9. e4'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 2 9' => {
			eco => 'E97t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2'),
			moves => {
				'a7a5' => 'r1bq1rk1/1pp1npbp/3p1np1/p2Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - a6 0 10',
				'c7c5' => 'r1bq1rk1/pp2npbp/3p1np1/2pPp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - c6 0 10',
				'f6d7' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 3 10',
				'f6e8' => 'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 3 10',
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 b - - 2 9' => {
			eco => 'E98a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp2npbp/3p1np1/2pPp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 w - c6 0 10',
				'f6d7' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 w - - 3 10',
				'f6e8' => 'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 w - - 3 10',
			},
		},
		'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E62u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O e5 8. d5 Ne7

			variation => N__('King\'s Indian: Fianchetto, Uhlmann/Szabo Variation'),
			moves => {
				'e2e4' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 9',
			},
		},
		'r1bq1rk1/ppp1npbp/3p2p1/2PPp2n/1P2P3/2N2N2/P3BPPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E97m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. c5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Nh5 10. c5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1npbp/3p2p1/3Pp2n/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E97l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Nh5'),
			moves => {
				'c4c5' => 'r1bq1rk1/ppp1npbp/3p2p1/2PPp2n/1P2P3/2N2N2/P3BPPP/R1BQ1RK1 b - - 0 10',
				'f1e1' => 'r1bq1rk1/ppp1npbp/3p2p1/3Pp2n/1PP1P3/2N2N2/P3BPPP/R1BQR1K1 b - - 2 10',
				'g2g3' => 'r1bq1rk1/ppp1npbp/3p2p1/3Pp2n/1PP1P3/2N2NP1/P3BP1P/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/ppp1npbp/3p2p1/3Pp2n/1PP1P3/2N2N2/P3BPPP/R1BQR1K1 b - - 2 10' => {
			eco => 'E97o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Nh5 10. Re1'),
			moves => {
				'a7a5' => 'r1bq1rk1/1pp1npbp/3p2p1/p2Pp2n/1PP1P3/2N2N2/P3BPPP/R1BQR1K1 w - a6 0 11',
				'f7f5' => 'r1bq1rk1/ppp1n1bp/3p2p1/3Ppp1n/1PP1P3/2N2N2/P3BPPP/R1BQR1K1 w - f6 0 11',
				'h5f4' => 'r1bq1rk1/ppp1npbp/3p2p1/3Pp3/1PP1Pn2/2N2N2/P3BPPP/R1BQR1K1 w - - 3 11',
			},
		},
		'r1bq1rk1/ppp1npbp/3p2p1/3Pp2n/1PP1P3/2N2NP1/P3BP1P/R1BQ1RK1 b - - 0 10' => {
			eco => 'E97n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. g3

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Nh5 10. g3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1npbp/3p2p1/3Pp3/1PP1Pn2/2N2N2/P3BPPP/R1BQR1K1 w - - 3 11' => {
			eco => 'E97q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Nh5 10. Re1 Nf4

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 Nf4'),
			moves => {
				'e2f1' => 'r1bq1rk1/ppp1npbp/3p2p1/3Pp3/1PP1Pn2/2N2N2/P4PPP/R1BQRBK1 b - - 4 11',
			},
		},
		'r1bq1rk1/ppp1npbp/3p2p1/3Pp3/1PP1Pn2/2N2N2/P4PPP/R1BQRBK1 b - - 4 11' => {
			eco => 'E97q',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, Bayonet, 9... Nh5 10. Re1 Nf4'),
			moves => {
				'a7a5' => 'r1bq1rk1/1pp1npbp/3p2p1/p2Pp3/1PP1Pn2/2N2N2/P4PPP/R1BQRBK1 w - a6 0 12',
			},
		},
		'r1bq1rk1/ppp1nppN/3p4/3P4/2B1R3/8/PP3PPP/R2Q2K1 b - - 0 14' => {
			eco => 'C54o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Bxc3 9. d5 Bf6 10. Re1 Ne7 11. Rxe4 d6 12. Bg5 Bxg5 13. Nxg5 O-O 14. Nxh7

			variation => N__('Giuoco Piano: Möller-Herzog Variation'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1nppp/3p1n2/1B2p1B1/1b2P2N/2NP4/PPP2PPP/R2Q1RK1 b - - 3 8' => {
			eco => 'C49k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Symmetrical, Pillsbury Variation'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp2nppp/2pp1n2/1B2p1B1/1b2P2N/2NP4/PPP2PPP/R2Q1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/ppp1nppp/3p1n2/1B2p1B1/1b2P3/2NP1N2/PPP2PPP/R2Q1RK1 w - - 2 8' => {
			eco => 'C49k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Ne7

			variation => N__('Four Knights: Symmetrical, Pillsbury Variation'),
			moves => {
				'f3h4' => 'r1bq1rk1/ppp1nppp/3p1n2/1B2p1B1/1b2P2N/2NP4/PPP2PPP/R2Q1RK1 b - - 3 8',
			},
		},
		'r1bq1rk1/ppp1nppp/3p4/3P2N1/2B1R3/8/PP3PPP/R2Q2K1 w - - 1 14' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'g5h7' => 'r1bq1rk1/ppp1nppN/3p4/3P4/2B1R3/8/PP3PPP/R2Q2K1 b - - 0 14',
			},
		},
		'r1bq1rk1/ppp1p1bp/2np1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5'),
			moves => {
				'c6a5' => 'r1bq1rk1/ppp1p1bp/3p1np1/n2P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
				'c6e5' => 'r1bq1rk1/ppp1p1bp/3p1np1/3Pnp2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/ppp1p1bp/2np1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. b3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. b3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1p1bp/2np1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6'),
			moves => {
				'b2b3' => 'r1bq1rk1/ppp1p1bp/2np1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'd4d5' => 'r1bq1rk1/ppp1p1bp/2np1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/ppp1p1bp/3p1np1/3PNp2/2P5/2N3P1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Ne5 9. Nxe5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Ne5 9.Nxe5'),
			moves => {
				'd6e5' => 'r1bq1rk1/ppp1p1bp/5np1/3Ppp2/2P5/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/ppp1p1bp/3p1np1/3Pnp2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Ne5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Ne5'),
			moves => {
				'f3e5' => 'r1bq1rk1/ppp1p1bp/3p1np1/3PNp2/2P5/2N3P1/PP2PPBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/ppp1p1bp/3p1np1/n2P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Na5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Na5'),
			moves => {
				'd1d3' => 'r1bq1rk1/ppp1p1bp/3p1np1/n2P1p2/2P5/2NQ1NP1/PP2PPBP/R1B2RK1 b - - 2 9',
				'f3d2' => 'r1bq1rk1/ppp1p1bp/3p1np1/n2P1p2/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 2 9',
			},
		},
		'r1bq1rk1/ppp1p1bp/3p1np1/n2P1p2/2P5/2N3P1/PP1NPPBP/R1BQ1RK1 b - - 2 9' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Na5 9. Nd2

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Na5 9. Nd2'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1p1bp/3p1np1/n2P1p2/2P5/2NQ1NP1/PP2PPBP/R1B2RK1 b - - 2 9' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Na5 9. Qd3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Na5 9. Qd3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1p1bp/5np1/3Ppp2/2P1P3/2N3P1/PP3PBP/R1BQ1RK1 b - e3 0 10' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Ne5 9. Nxe5 dxe5 10. e4

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Ne5 9.Nxe5 dxe5 10. e4'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1p1bp/5np1/3Ppp2/2P5/1QN3P1/PP2PPBP/R1B2RK1 b - - 1 10' => {
			eco => 'A89',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Nc6 8. d5 Ne5 9. Nxe5 dxe5 10. Qb3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Ne5 9.Nxe5 dxe5 10. Qb3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1p1bp/5np1/3Ppp2/2P5/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A89',
                        # TRANSLATORS:

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Nc6 8. d5 Ne5 9.Nxe5'),
			moves => {
				'd1b3' => 'r1bq1rk1/ppp1p1bp/5np1/3Ppp2/2P5/1QN3P1/PP2PPBP/R1B2RK1 b - - 1 10',
				'e2e4' => 'r1bq1rk1/ppp1p1bp/5np1/3Ppp2/2P1P3/2N3P1/PP3PBP/R1BQ1RK1 b - e3 0 10',
			},
		},
		'r1bq1rk1/ppp1p1bp/n2p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Na6

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/1n4p1/n2P4/4P3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 10' => {
			eco => 'D76k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. d5 Na5 10. e4

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. d5 Na5 10. e4'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp2ppbp/1np3p1/n2P4/4P3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r1bq1rk1/ppp1ppbp/1n4p1/n2P4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'D76j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. d5 Na5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. d5 Na5'),
			moves => {
				'e2e4' => 'r1bq1rk1/ppp1ppbp/1n4p1/n2P4/4P3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 10',
			},
		},
		'r1bq1rk1/ppp1ppbp/1nn3p1/3P4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D76i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. d5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. d5'),
			moves => {
				'c6a5' => 'r1bq1rk1/ppp1ppbp/1n4p1/n2P4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/ppp1ppbp/1nn3p1/8/3P4/2N1PNP1/PP3PBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D76d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. e3

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. e3'),
			moves => {
				'e7e5' => 'r1bq1rk1/ppp2pbp/1nn3p1/4p3/3P4/2N1PNP1/PP3PBP/R1BQ1RK1 w - e6 0 10',
				'f8e8' => 'r1bqr1k1/ppp1ppbp/1nn3p1/8/3P4/2N1PNP1/PP3PBP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/ppp1ppbp/1nn3p1/8/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 4 9' => {
			eco => 'D76c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp1ppbp/1nn3p1/3P4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 9',
				'e2e3' => 'r1bq1rk1/ppp1ppbp/1nn3p1/8/3P4/2N1PNP1/PP3PBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/ppp1ppbp/2n2np1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 6 7' => {
			eco => 'D77d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O Nc6

			variation => N__('Neo-Grünfeld, 6.O-O Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2n2np1/8/2BP4/1QN1PN2/PP3PPP/R1B1K2R w KQ - 1 8' => {
			eco => 'D95j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 dxc4 7. Bxc4 Nc6

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 dxc4 7.Bxc4 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2n2np1/8/2NP4/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'D77j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 Nc6 8. Nxc4

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 Nc6 8.Nxc4'),
			moves => {
				'c8e6' => 'r2q1rk1/ppp1ppbp/2n1bnp1/8/2NP4/5NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/ppp1ppbp/2n2np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 2 8' => {
			eco => 'D97l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Nc6 8. Be2

			variation => N__('Grünfeld: Russian, Simagin, 8. Be2'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2n2np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8' => {
			eco => 'D97l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Nc6

			variation => N__('Grünfeld: Russian, Simagin Variation'),
			moves => {
				'f1e2' => 'r1bq1rk1/ppp1ppbp/2n2np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 2 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/2n2np1/8/2pP4/N4NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D77j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 Nc6

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 Nc6'),
			moves => {
				'a3c4' => 'r1bq1rk1/ppp1ppbp/2n2np1/8/2NP4/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/2n3p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 8' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nc6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2n3p1/8/2BPP3/2P1B3/P3NPPP/R2QK2R b KQ - 6 9' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 Nc6 9. Be3

			variation => N__('Grünfeld: Classical Exchange, Simagin Improved, 9. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2n3p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 6 9' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 Nc6 9. O-O

			variation => N__('Grünfeld: Classical Exchange, Simagin Improved, 9.O-O'),
			moves => {
				'b7b6' => 'r1bq1rk1/p1p1ppbp/1pn3p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - - 0 10',
				'e7e5' => 'r1bq1rk1/ppp2pbp/2n3p1/4p3/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - e6 0 10',
			},
		},
		'r1bq1rk1/ppp1ppbp/2n3p1/8/2BPP3/2P5/P3NPPP/R1BQK2R w KQ - 5 9' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 Nc6

			variation => N__('Grünfeld: Classical Exchange, Simagin Improved'),
			moves => {
				'c1e3' => 'r1bq1rk1/ppp1ppbp/2n3p1/8/2BPP3/2P1B3/P3NPPP/R2QK2R b KQ - 6 9',
				'e1g1' => 'r1bq1rk1/ppp1ppbp/2n3p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 6 9',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/3P4/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E91p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nc6 7. d5

			variation => N__('King\'s Indian: 6. Be2 Nc6 7. d5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/3P4/2P5/2N2NP1/PP2PPBP/R1BQK2R b KQ - 0 7' => {
			eco => 'E62m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. d5

			variation => N__('King\'s Indian: Fianchetto, 6... Nc6 7. d5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/4P3/3P1P2/2NB1N2/PPP3PP/R1BQK2R b KQ - 0 7' => {
			eco => 'B09k',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3 Nc6 7. e5

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Nc6 7. e5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/6B1/2PPP3/2N2P2/PP4PP/R2QKBNR w KQ - 3 7' => {
			eco => 'E81f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 Nc6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 7' => {
			eco => 'E62n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O

			variation => N__('King\'s Indian: Fianchetto, 6... Nc6 7.O-O'),
			moves => {
				'a7a6' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'a8b8' => '1rbq1rk1/ppp1ppbp/2np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 8',
				'c8f5' => 'r2q1rk1/ppp1ppbp/2np1np1/5b2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 8',
				'c8g4' => 'r2q1rk1/ppp1ppbp/2np1np1/8/2PP2b1/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 8',
				'e7e5' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - e6 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 7' => {
			eco => 'E62l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6

			variation => N__('King\'s Indian: Fianchetto, 6... Nc6'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp1ppbp/2np1np1/3P4/2P5/2N2NP1/PP2PPBP/R1BQK2R b KQ - 0 7',
				'e1g1' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 7' => {
			eco => 'E60n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O Nc6

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6... Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 4 7' => {
			eco => 'E83b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Qd2

			variation => N__('King\'s Indian: Sämisch, 6... Nc6 7. Qd2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 4 7' => {
			eco => 'E83d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2

			variation => N__('King\'s Indian: Sämisch, 6... Nc6 7.Nge2'),
			moves => {
				'a7a6' => 'r1bq1rk1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 8',
				'a8b8' => '1rbq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 5 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 3 7' => {
			eco => 'E83a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6

			variation => N__('King\'s Indian: Sämisch, 6... Nc6'),
			moves => {
				'd1d2' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 4 7',
				'g1e2' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 4 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7' => {
			eco => 'E91o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nc6

			variation => N__('King\'s Indian: 6. Be2 Nc6'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp1ppbp/2np1np1/3P4/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ - 3 7' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2 Nc6

			variation => N__('King\'s Indian: 4. e4 d6 5. g3 O-O 6. Bg2 Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2NB4/PP2NPPP/R1BQ1RK1 b - - 5 7' => {
			eco => 'E70n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 Nc6 7. O-O

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 Nc6 7.O-O'),
			moves => {
				'e7e5' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2NB4/PP2NPPP/R1BQ1RK1 w - e6 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2NB4/PP2NPPP/R1BQK2R w KQ - 4 7' => {
			eco => 'E70n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 Nc6

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 Nc6'),
			moves => {
				'e1g1' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2NB4/PP2NPPP/R1BQ1RK1 b - - 5 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 6 7' => {
			eco => 'B08o',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O Nc6

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O Nc6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/2np1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQK2R w KQ - 5 7' => {
			eco => 'B09j',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3 Nc6

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Nc6'),
			moves => {
				'e4e5' => 'r1bq1rk1/ppp1ppbp/2np1np1/4P3/3P1P2/2NB1N2/PPP3PP/R1BQK2R b KQ - 0 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/4P3/2PP1P2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E76g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 Na6 7. e5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 Na6 7. e5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP2P/2N5/PP2BPP1/R2QK1NR b KQ h3 0 7' => {
			eco => 'E73q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6 7. h4

			variation => N__('King\'s Indian: Averbakh, 6... Na6 7. h4'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 7' => {
			eco => 'E90m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 Na6 7. Bg5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 Na6 7. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N2N2/PP2BPPP/R2QK2R b KQ - 5 7' => {
			eco => 'E91l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Na6 7. Bg5

			variation => N__('King\'s Indian: Kazakh Variation, 7. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 3 7' => {
			eco => 'E71k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 Na6

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 Na6'),
			moves => {
				'f1d3' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2NB3P/PP3PP1/R2QK1NR b KQ - 4 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR b KQ - 5 7' => {
			eco => 'E73r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6 7. Qd2

			variation => N__('King\'s Indian: Averbakh, 6... Na6 7. Qd2'),
			moves => {
				'e7e5' => 'r1bq1rk1/ppp2pbp/n2p1np1/4p1B1/2PPP3/2N5/PP1QBPPP/R3K1NR w KQ e6 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 4 7' => {
			eco => 'E73p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6

			variation => N__('King\'s Indian: Averbakh, 6... Na6'),
			moves => {
				'd1d2' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR b KQ - 5 7',
				'h2h4' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP2P/2N5/PP2BPP1/R2QK1NR b KQ h3 0 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2NB3P/PP3PP1/R2QK1NR b KQ - 4 7' => {
			eco => 'E71l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 Na6 7. Bd3

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 Na6 7. Bd3'),
			moves => {
				'e7e5' => 'r1bq1rk1/ppp2pbp/n2p1np1/4p1B1/2PPP3/2NB3P/PP3PP1/R2QK1NR w KQ e6 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 7' => {
			eco => 'E90m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 Na6

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 Na6'),
			moves => {
				'c1g5' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 5 7' => {
			eco => 'E91m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Na6 7. O-O

			variation => N__('King\'s Indian: Kazakh Variation, 7.O-O'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp2ppbp/n2p1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - c6 0 8',
				'c7c6' => 'r1bq1rk1/pp2ppbp/n1pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7' => {
			eco => 'E91l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Na6

			variation => N__('King\'s Indian: Kazakh Variation'),
			moves => {
				'c1g5' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N2N2/PP2BPPP/R2QK2R b KQ - 5 7',
				'e1g1' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 5 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPPP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 4 7' => {
			eco => 'E76h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 Na6 7. Be2

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 Na6 7. Be2'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPPP2/2N2N2/PP4PP/R1BQKB1R w KQ - 3 7' => {
			eco => 'E76f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 Na6

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 Na6'),
			moves => {
				'e4e5' => 'r1bq1rk1/ppp1ppbp/n2p1np1/4P3/2PP1P2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 7',
				'f1d3' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPPP2/2NB1N2/PP4PP/R1BQK2R b KQ - 4 7',
				'f1e2' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPPP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 4 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPPP2/2NB1N2/PP4PP/R1BQK2R b KQ - 4 7' => {
			eco => 'E76i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 Na6 7. Bd3

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 Na6 7. Bd3'),
			moves => {
				'e7e5' => 'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPPP2/2NB1N2/PP4PP/R1BQK2R w KQ e6 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQ1RK1 b - - 6 7' => {
			eco => 'B09l',
                        # TRANSLATORS:

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Na6'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp2ppbp/n2p1np1/2p5/3PPP2/2NB1N2/PPP3PP/R1BQ1RK1 w - c6 0 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/n2p1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQK2R w KQ - 5 7' => {
			eco => 'B09l',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3 Na6

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Na6'),
			moves => {
				'e1g1' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQ1RK1 b - - 6 7',
			},
		},
		'r1bq1rk1/ppp1ppbp/n4np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 2 8' => {
			eco => 'D97o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Na6 8. Be2

			variation => N__('Grünfeld: Russian, Prins, 8. Be2'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp2ppbp/n4np1/2p5/2QPP3/2N2N2/PP2BPPP/R1B1K2R w KQ c6 0 9',
			},
		},
		'r1bq1rk1/ppp1ppbp/n4np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8' => {
			eco => 'D97m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Na6

			variation => N__('Grünfeld: Russian, Prins Variation'),
			moves => {
				'c1f4' => 'r1bq1rk1/ppp1ppbp/n4np1/8/2QPPB2/2N2N2/PP3PPP/R3KB1R b KQ - 2 8',
				'f1e2' => 'r1bq1rk1/ppp1ppbp/n4np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 2 8',
			},
		},
		'r1bq1rk1/ppp1ppbp/n4np1/8/2QPPB2/2N2N2/PP3PPP/R3KB1R b KQ - 2 8' => {
			eco => 'D97n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Na6 8. Bf4

			variation => N__('Grünfeld: Russian, Prins, 8. Bf4'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n4np1/8/2pP4/N4NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D77g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 Na6

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 Na6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp1ppbp/n5p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 8' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Na6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Na6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/1n4p1/n2Pp3/8/2N1PNP1/PP3PBP/R1BQ1RK1 w - - 1 11' => {
			eco => 'D76f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. e3 e5 10. d5 Na5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. e3 e5 10. d5 Na5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/1nn3p1/3Pp3/8/2N1PNP1/PP3PBP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D76e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. e3 e5 10. d5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. e3 e5 10. d5'),
			moves => {
				'c6a5' => 'r1bq1rk1/ppp2pbp/1n4p1/n2Pp3/8/2N1PNP1/PP3PBP/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bq1rk1/ppp2pbp/1nn3p1/4p3/3P4/2N1PNP1/PP3PBP/R1BQ1RK1 w - e6 0 10' => {
			eco => 'D76e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. e3 e5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. e3 e5'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp2pbp/1nn3p1/3Pp3/8/2N1PNP1/PP3PBP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bq1rk1/ppp2pbp/2n3p1/4p3/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - e6 0 10' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 Nc6 9. O-O e5

			variation => N__('Grünfeld: Classical Exchange, Simagin Improved, 9.O-O e5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 8' => {
			eco => 'E85h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 Nc6 8. d5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 Nc6 8. d5'),
			moves => {
				'c6e7' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 1 9',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E97d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5

			variation => N__('King\'s Indian: Mar del Plata, 8. d5'),
			moves => {
				'c6e7' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/3Pp3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E62t',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Fianchetto, Uhlmann/Szabo Variation'),
			moves => {
				'c6e7' => 'r1bq1rk1/ppp1npbp/3p1np1/3Pp3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4P3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E97b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. dxe5

			variation => N__('King\'s Indian: Mar del Plata, 8.dxe5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP1NP1/PP3PBP/R1BQ1RK1 b - - 4 8' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nf6 7. Nf3 O-O 8. O-O

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP1NP1/PP3PBP/R1BQK2R w KQ - 3 8' => {
			eco => 'A26',
                        # TRANSLATORS:

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
				'e1g1' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP1NP1/PP3PBP/R1BQ1RK1 b - - 4 8',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 b - - 4 8' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nf6 7. Nge2 O-O 8. O-O

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R w KQ - 3 8' => {
			eco => 'A26',
                        # TRANSLATORS:

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
				'e1g1' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 b - - 4 8',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/1RBQ1RK1 b - - 5 8' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6 7. O-O O-O 8. Rb1

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6 7.O-O O-O 8. Rb1'),
			moves => {
				'a7a5' => 'r1bq1rk1/1pp2pbp/2np1np1/p3p3/2P5/2NP1NP1/PP2PPBP/1RBQ1RK1 w - a6 0 9',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 4 8' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6 7. O-O O-O

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6 7.O-O O-O'),
			moves => {
				'a1b1' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/1RBQ1RK1 b - - 5 8',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - e6 0 8' => {
			eco => 'E62t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O e5

			variation => N__('King\'s Indian: Fianchetto, Uhlmann/Szabo Variation'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp2pbp/2np1np1/3Pp3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8' => {
			eco => 'E97c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. Be3

			variation => N__('King\'s Indian: Mar del Plata, 8. Be3'),
			moves => {
				'f6g4' => 'r1bq1rk1/ppp2pbp/2np2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 2 8' => {
			eco => 'E85g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 Nc6

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 Nc6'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp2pbp/2np1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 8',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E97a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6

			variation => N__('King\'s Indian: Mar del Plata'),
			moves => {
				'c1e3' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8',
				'd4d5' => 'r1bq1rk1/ppp2pbp/2np1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8',
				'd4e5' => 'r1bq1rk1/ppp2pbp/2np1np1/4P3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N3P1/PP2NPBP/R1BQK2R w KQ - 2 8' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2 e5 7. Nge2 Nc6

			variation => N__('King\'s Indian: Pomar System'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2NB4/PP2NPPP/R1BQ1RK1 w - e6 0 8' => {
			eco => 'E70n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 Nc6 7. O-O e5

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 Nc6 7.O-O e5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/2np2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 b - - 5 9' => {
			eco => 'E97c',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, 8. Be3 Ng4'),
			moves => {
				'f7f6' => 'r1bq1rk1/ppp3bp/2np1pp1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 w - - 0 10',
			},
		},
		'r1bq1rk1/ppp2pbp/2np2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9' => {
			eco => 'E97c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. Be3 Ng4

			variation => N__('King\'s Indian: Mar del Plata, 8. Be3 Ng4'),
			moves => {
				'e3g5' => 'r1bq1rk1/ppp2pbp/2np2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 b - - 5 9',
			},
		},
		'r1bq1rk1/ppp2pbp/3p1np1/2n5/2PNP3/2N3P1/PP3PBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E68d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 exd4 9. Nxd4 Nc5

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4 9.Nxd4 Nc5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/3p1np1/2n5/2PNP3/2N5/PP2BPPP/R1BQR1K1 w - - 1 10' => {
			eco => 'E95g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 exd4 9. Nxd4 Nc5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 exd4 9.Nxd4 Nc5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/3p1np1/2nPp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 1 9' => {
			eco => 'E73o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Nbd7 7. Qd2 e5 8. d5 Nc5

			variation => N__('King\'s Indian: Averbakh, 6...Nbd7 7. Qd2 e5 8. d5 Nc5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/3p1np1/2nPp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E94q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. d5 Nc5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. d5 Nc5'),
			moves => {
				'd1c2' => 'r1bq1rk1/ppp2pbp/3p1np1/2nPp3/2P1P3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 2 9',
			},
		},
		'r1bq1rk1/ppp2pbp/3p1np1/2nPp3/2P1P3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 2 9' => {
			eco => 'E94r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. d5 Nc5 9. Qc2

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. d5 Nc5 9. Qc2'),
			moves => {
				'a7a5' => 'r1bq1rk1/1pp2pbp/3p1np1/p1nPp3/2P1P3/2N2N2/PPQ1BPPP/R1B2RK1 w - a6 0 10',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/3Pp1B1/2P1P3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 8' => {
			eco => 'E90s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 Na6 8. Bg5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 Na6 8. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 0 8' => {
			eco => 'E73s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6 7. Qd2 e5 8. d5

			variation => N__('King\'s Indian: Averbakh, 6... Na6 7. Qd2 e5 8. d5'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp3pbp/n1pp1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 0 9',
				'd8e8' => 'r1b1qrk1/ppp2pbp/n2p1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 1 9',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/3Pp3/2P1P3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 2 8' => {
			eco => 'E90s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 Na6 8. Be3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 Na6 8. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/3Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 8' => {
			eco => 'E90s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 Na6

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 Na6'),
			moves => {
				'c1e3' => 'r1bq1rk1/ppp2pbp/n2p1np1/3Pp3/2P1P3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 2 8',
				'c1g5' => 'r1bq1rk1/ppp2pbp/n2p1np1/3Pp1B1/2P1P3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 8',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/4p1B1/2PPP3/2N5/PP1QBPPP/R3K1NR w KQ e6 0 8' => {
			eco => 'E73s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Na6 7. Qd2 e5

			variation => N__('King\'s Indian: Averbakh, 6... Na6 7. Qd2 e5'),
			moves => {
				'd4d5' => 'r1bq1rk1/ppp2pbp/n2p1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 0 8',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/4p1B1/2PPP3/2NB3P/PP3PP1/R2QK1NR w KQ e6 0 8' => {
			eco => 'E71l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 Na6 7. Bd3 e5

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 Na6 7. Bd3 e5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8' => {
			eco => 'E94l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Be3

			variation => N__('King\'s Indian: Glek, 8. Be3'),
			moves => {
				'f6g4' => 'r1bq1rk1/ppp2pbp/n2p2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E94i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6

			variation => N__('King\'s Indian: Glek Variation'),
			moves => {
				'c1e3' => 'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8',
				'f1e1' => 'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 b - - 3 8',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 b - - 3 8' => {
			eco => 'E94j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Re1

			variation => N__('King\'s Indian: Glek, 8. Re1'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp3pbp/n1pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPPP2/2NB1N2/PP4PP/R1BQK2R w KQ e6 0 8' => {
			eco => 'E76j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 Na6 7. Bd3 e5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 Na6 7. Bd3 e5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2pbp/n2p2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 b - - 5 9' => {
			eco => 'E94m',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Glek, 8. Be3 Ng4'),
			moves => {
				'd8e8' => 'r1b1qrk1/ppp2pbp/n2p2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 w - - 6 10',
			},
		},
		'r1bq1rk1/ppp2pbp/n2p2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9' => {
			eco => 'E94m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Na6 8. Be3 Ng4

			variation => N__('King\'s Indian: Glek, 8. Be3 Ng4'),
			moves => {
				'e3g5' => 'r1bq1rk1/ppp2pbp/n2p2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 b - - 5 9',
			},
		},
		'r1bq1rk1/ppp2pp1/2np1n1p/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2Q1RK1 w - - 0 9' => {
			eco => 'C49m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 h6

			variation => N__('Four Knights: Symmetrical, 7. Bg5 Bxc3 8.bxc3 h6'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2n1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 3 7' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6 7. O-O

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6 7.O-O'),
			moves => {
				'a7a6' => 'r1bq1rk1/1pp2ppp/p1n1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8',
				'd5c4' => 'r1bq1rk1/ppp2ppp/2n1pn2/8/1bpP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/ppp2ppp/2n1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 2 7' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6'),
			moves => {
				'e1g1' => 'r1bq1rk1/ppp2ppp/2n1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 3 7',
			},
		},
		'r1bq1rk1/ppp2ppp/2n1pn2/8/1bpP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6 7. O-O dxc4

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. Bd3 Nc6 7.O-O dxc4'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2n2n2/1B1pp3/4P3/2PP1N2/P1P2PPP/R1BQ1RK1 w - d6 0 8' => {
			eco => 'C49f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 Bxc3 7. bxc3 d5

			variation => N__('Four Knights: Svenonius Variation'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p1B1/1b2P3/2NP1N2/PPP2PPP/R2Q1RK1 b - - 1 7' => {
			eco => 'C49i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5

			variation => N__('Four Knights: Symmetrical, 7. Bg5'),
			moves => {
				'b4c3' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p1B1/4P3/2bP1N2/PPP2PPP/R2Q1RK1 w - - 0 8',
				'c6e7' => 'r1bq1rk1/ppp1nppp/3p1n2/1B2p1B1/1b2P3/2NP1N2/PPP2PPP/R2Q1RK1 w - - 2 8',
				'c8e6' => 'r2q1rk1/ppp2ppp/2npbn2/1B2p1B1/1b2P3/2NP1N2/PPP2PPP/R2Q1RK1 w - - 2 8',
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2Q1RK1 b - - 0 8' => {
			eco => 'C49l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3

			variation => N__('Four Knights: Symmetrical, 7. Bg5 Bxc3'),
			moves => {
				'd8e7' => 'r1b2rk1/ppp1qppp/2np1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2Q1RK1 w - - 1 9',
				'h7h6' => 'r1bq1rk1/ppp2pp1/2np1n1p/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2Q1RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p1B1/4P3/2bP1N2/PPP2PPP/R2Q1RK1 w - - 0 8' => {
			eco => 'C49l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3

			variation => N__('Four Knights: Symmetrical, 7. Bg5 Bxc3'),
			moves => {
				'b2c3' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p1B1/4P3/2PP1N2/P1P2PPP/R2Q1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/1b2P3/2NP1N2/PPP2PPP/R1BQ1RK1 w - - 0 7' => {
			eco => 'C49g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6

			variation => N__('Four Knights: Symmetrical Variation'),
			moves => {
				'c1g5' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p1B1/1b2P3/2NP1N2/PPP2PPP/R2Q1RK1 b - - 1 7',
				'c3e2' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/1b2P3/3P1N2/PPP1NPPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/1b2P3/3P1N2/PPP1NPPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'C49h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Ne2

			variation => N__('Four Knights: Symmetrical, Maroczy System'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/4P3/2PP1N2/P1P2PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'C49f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Double Bishops, 6. d3 Bxc3'),
			moves => {
				'f1e1' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/4P3/2PP1N2/P1P2PPP/R1BQR1K1 b - - 1 8',
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/4P3/2PP1N2/P1P2PPP/R1BQR1K1 b - - 1 8' => {
			eco => 'C49f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 Bxc3 7. bxc3 d6 8. Re1

			variation => N__('Four Knights: Janowski Variation'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQ1RK1 w - - 2 7' => {
			eco => 'C54i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d3 d6 6. O-O O-O

			variation => N__('Giuoco Pianissimo: 5. d3 d6 6.O-O O-O'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 2 7' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bc5 5. Bg2 d6 6. O-O O-O

			variation => N__('English: Four Knights, 4. g3 Bc5'),
			moves => {
				'd2d3' => 'r1bq1rk1/ppp2ppp/2np1n2/2b1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b - - 0 7',
			},
		},
		'r1bq1rk1/ppp2ppp/2np1n2/2b1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b - - 0 7' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bc5 5. Bg2 d6 6. O-O O-O 7. d3

			variation => N__('English: Four Knights, 4. g3 Bc5'),
			moves => {
			},
		},
		'r1bq1rk1/ppp2ppp/2nppn2/8/1bPP4/2N2N2/PPQBPPPP/R3KB1R w KQ - 2 7' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3 d6 6. Bd2 O-O

			variation => N__('Nimzo-Indian: Classical, Milner-Barry (Zurich), 6. Bd2 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/ppp3bp/2np1pp1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2Q1RK1 w - - 0 10' => {
			eco => 'E97c',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, 8. Be3 Ng4'),
			moves => {
				'g5c1' => 'r1bq1rk1/ppp3bp/2np1pp1/4p3/2PPP1n1/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 10',
			},
		},
		'r1bq1rk1/ppp3bp/2np1pp1/4p3/2PPP1n1/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 10' => {
			eco => 'E97c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. Be3 Ng4 9. Bg5 f6 10. Bc1

			variation => N__('King\'s Indian: Mar del Plata, 8. Be3 Ng4'),
			moves => {
			},
		},
		'r1bq1rk1/ppp3bp/2np1pp1/4p3/2PPP1nB/2N2N2/PP2BPPP/R2QK2R w KQ - 2 10' => {
			eco => 'E92m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 Ng4 8. Bg5 f6 9. Bh4 Nc6

			variation => N__('King\'s Indian: Gligoric-Taimanov, Main Line'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pb1/3p1n1p/3Pp1p1/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5 h6 9. Bh4 g5

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6 9. Bh4 g5'),
			moves => {
				'h4g3' => 'r1bq1rk1/pppn1pb1/3p1n1p/3Pp1p1/2P1P3/2N2NB1/PP2BPPP/R2QK2R b KQ - 1 10',
			},
		},
		'r1bq1rk1/pppn1pb1/3p1n1p/3Pp1p1/2P1P3/2N2NB1/PP2BPPP/R2QK2R b KQ - 1 10' => {
			eco => 'E93',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6 9. Bh4 g5'),
			moves => {
				'f6h5' => 'r1bq1rk1/pppn1pb1/3p3p/3Pp1pn/2P1P3/2N2NB1/PP2BPPP/R2QK2R w KQ - 2 11',
			},
		},
		'r1bq1rk1/pppn1pb1/3p1npp/3Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5 h6

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pppn1pb1/3p1npp/3Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9',
			},
		},
		'r1bq1rk1/pppn1pb1/3p1npp/3Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5 h6 9. Bh4

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6 9. Bh4'),
			moves => {
				'a7a5' => 'r1bq1rk1/1ppn1pb1/3p1npp/p2Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ a6 0 10',
				'a7a6' => 'r1bq1rk1/1ppn1pb1/p2p1npp/3Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10',
				'g6g5' => 'r1bq1rk1/pppn1pb1/3p1n1p/3Pp1p1/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10',
			},
		},
		'r1bq1rk1/pppn1pb1/3p1npp/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QK1NR w KQ - 1 9' => {
			eco => 'E73z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6 7. Be3 e5 8. d5 Nbd7

			variation => N__('King\'s Indian: Averbakh, 6... h6 7. Be3 e5 8. d5 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pb1/3p1npp/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9' => {
			eco => 'E95c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 h6

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 h6'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pb1/3p3p/3Pp1pn/2P1P2P/2N2NB1/PP2BPP1/R2QK2R b KQ h3 0 11' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5 h6 9. Bh4 g5 10. Bg3 Nh5 11. h4

			variation => N__('King\'s Indian: Petrosian, Keres Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pb1/3p3p/3Pp1pn/2P1P3/2N2NB1/PP2BPPP/R2QK2R w KQ - 2 11' => {
			eco => 'E93',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5 h6 9. Bh4 g5'),
			moves => {
				'h2h4' => 'r1bq1rk1/pppn1pb1/3p3p/3Pp1pn/2P1P2P/2N2NB1/PP2BPP1/R2QK2R b KQ h3 0 11',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp1B1/2P1P3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 8' => {
			eco => 'E90r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 Nbd7 8. Bg5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 Nbd7 8. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 2 8' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Bg5

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Bg5'),
			moves => {
				'h7h6' => 'r1bq1rk1/pppn1pb1/3p1npp/3Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 0 8' => {
			eco => 'E73o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Nbd7 7. Qd2 e5 8. d5

			variation => N__('King\'s Indian: Averbakh, 6...Nbd7 7. Qd2 e5 8. d5'),
			moves => {
				'd7c5' => 'r1bq1rk1/ppp2pbp/3p1np1/2nPp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 1 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 2 8' => {
			eco => 'E90r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 Nbd7 8. Be3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 Nbd7 8. Be3'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N1BN2/PP2BPPP/R2QK2R b KQ - 2 8' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Be3

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Be3'),
			moves => {
				'f6g4' => 'r1bq1rk1/pppn1pbp/3p2p1/3Pp3/2P1P1n1/2N1BN2/PP2BPPP/R2QK2R w KQ - 3 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 1 8' => {
			eco => 'E87d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nbd7

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 8' => {
			eco => 'E90r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 Nbd7

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 Nbd7'),
			moves => {
				'c1e3' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 2 8',
				'c1g5' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp1B1/2P1P3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 8',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E94q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. d5

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. d5'),
			moves => {
				'd7c5' => 'r1bq1rk1/ppp2pbp/3p1np1/2nPp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 1 8' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7

			variation => N__('King\'s Indian: Petrosian, Main Line'),
			moves => {
				'c1e3' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N1BN2/PP2BPPP/R2QK2R b KQ - 2 8',
				'c1g5' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 2 8',
				'd1c2' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N2/PPQ1BPPP/R1B1K2R b KQ - 2 8',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N2/PPQ1BPPP/R1B1K2R b KQ - 2 8' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Qc2

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Qc2'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E67e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. d5

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. d5'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4P3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E67f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. dxe5

			variation => N__('King\'s Indian: Fianchetto, Classical, 8.dxe5'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p1B1/2PPP3/2N5/PP1QBPPP/R3K1NR w KQ e6 0 8' => {
			eco => 'E73o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Nbd7 7. Qd2 e5

			variation => N__('King\'s Indian: Averbakh, 6...Nbd7 7. Qd2 e5'),
			moves => {
				'd4d5' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp1B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 0 8',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E67g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. b3

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. b3'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - e6 0 8' => {
			eco => 'E67d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5

			variation => N__('King\'s Indian: Fianchetto, Classical Variation'),
			moves => {
				'b2b3' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'd1c2' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8',
				'd4d5' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'd4e5' => 'r1bq1rk1/pppn1pbp/3p1np1/4P3/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'e2e4' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8',
				'h2h3' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'E67h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. Qc2

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. Qc2'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 0 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8' => {
			eco => 'E67j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. h3

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. h3'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - e6 0 8' => {
			eco => 'E60q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O Nbd7 7. Qc2 e5

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6...Nbd7 7. Qc2 e5'),
			moves => {
				'f1d1' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/5NP1/PPQ1PPBP/RNBR2K1 b - - 1 8',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/5NP1/PPQ1PPBP/RNBR2K1 b - - 1 8' => {
			eco => 'E60q',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6...Nbd7 7. Qc2 e5'),
			moves => {
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PP4/5NP1/PPQ1PPBP/RNBR2K1 w - - 2 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8' => {
			eco => 'E94u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Be3

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Be3'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 9',
				'f6g4' => 'r1bq1rk1/pppn1pbp/3p2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9',
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 2 8' => {
			eco => 'E85f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 Nbd7

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E94p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7

			variation => N__('King\'s Indian: 7.O-O Nbd7'),
			moves => {
				'c1e3' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 3 8',
				'd1c2' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 3 8',
				'd4d5' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8',
				'f1e1' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 b - - 3 8',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 b - - 3 8' => {
			eco => 'E95a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1'),
			moves => {
				'a7a5' => 'r1bq1rk1/1ppn1pbp/3p1np1/p3p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - a6 0 9',
				'c7c6' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9',
				'e5d4' => 'r1bq1rk1/pppn1pbp/3p1np1/8/2PpP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 4 9',
				'h7h6' => 'r1bq1rk1/pppn1pb1/3p1npp/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 3 8' => {
			eco => 'E94s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Qc2

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Qc2'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 4 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8' => {
			eco => 'E68a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4'),
			moves => {
				'a7a6' => 'r1bq1rk1/1ppn1pbp/p2p1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9',
				'c7c6' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9',
				'e5d4' => 'r1bq1rk1/pppn1pbp/3p1np1/8/2PpP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9',
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/8/2PNP3/2N3P1/PP3PBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E68c',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4'),
			moves => {
				'd7c5' => 'r1bq1rk1/ppp2pbp/3p1np1/2n5/2PNP3/2N3P1/PP3PBP/R1BQ1RK1 w - - 1 10',
				'f8e8' => 'r1bqr1k1/pppn1pbp/3p1np1/8/2PNP3/2N3P1/PP3PBP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/8/2PNP3/2N5/PP2BPPP/R1BQR1K1 b - - 0 9' => {
			eco => 'E95f',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 exd4'),
			moves => {
				'd7c5' => 'r1bq1rk1/ppp2pbp/3p1np1/2n5/2PNP3/2N5/PP2BPPP/R1BQR1K1 w - - 1 10',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/8/2PpP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 0 9' => {
			eco => 'E95f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 exd4

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 exd4'),
			moves => {
				'f3d4' => 'r1bq1rk1/pppn1pbp/3p1np1/8/2PNP3/2N5/PP2BPPP/R1BQR1K1 b - - 0 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p1np1/8/2PpP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E68c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 exd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4'),
			moves => {
				'f3d4' => 'r1bq1rk1/pppn1pbp/3p1np1/8/2PNP3/2N3P1/PP3PBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pppn1pbp/3p2p1/3Pp3/2P1P1n1/2N1BN2/PP2BPPP/R2QK2R w KQ - 3 9' => {
			eco => 'E93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 Nbd7 8. Be3 Ng4

			variation => N__('King\'s Indian: Petrosian, Main Line, 8. Be3 Ng4'),
			moves => {
			},
		},
		'r1bq1rk1/pppn1pbp/3p2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9' => {
			eco => 'E94v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Be3 Ng4

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Be3 Ng4'),
			moves => {
			},
		},
		'r1bq1rk1/pppn2bp/3p2n1/3Pp1p1/P1P1Pp2/2N2P2/1P2BBPP/R2QNRK1 w - - 1 14' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Be3 f4 12. Bf2 g5 13. a4 Ng6

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Be3 f4 12. Bf2 g5 13. a4 Ng6'),
			moves => {
			},
		},
		'r1bq1rk1/pppn2bp/3p2p1/3Ppp1n/2P1P3/2N1BP2/PP1Q2PP/2KR1BNR w - - 2 10' => {
			eco => 'E87m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 f5 9. O-O-O Nd7

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5 8. Qd2 f5 9.O-O-O Nd7'),
			moves => {
				'f1d3' => 'r1bq1rk1/pppn2bp/3p2p1/3Ppp1n/2P1P3/2NBBP2/PP1Q2PP/2KR2NR b - - 3 10',
			},
		},
		'r1bq1rk1/pppn2bp/3p2p1/3Ppp1n/2P1P3/2NBBP2/PP1Q2PP/2KR2NR b - - 3 10' => {
			eco => 'E87n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 f5 9. O-O-O Nd7 10. Bd3

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5 8. Qd2 f5 9.O-O-O Nd7 10. Bd3'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbBpp/3p1n2/4p1N1/3PP3/2N5/PPP2PPP/R1BQK2R b KQ - 0 7' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. Ng5 O-O 7. Bxf7+

			variation => N__('Philidor: Improved Hanham, Larobok Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbpp1/4pn1p/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 h6

			variation => N__('QGD: Orthodox, 7. Rc1 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pppnbpp1/4pn1p/3p4/2PP3B/2N1PN2/PP3PPP/2RQKB1R b K - 1 8',
			},
		},
		'r1bq1rk1/pppnbpp1/4pn1p/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 h6

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 h6'),
			moves => {
				'g5h4' => 'r1bq1rk1/pppnbpp1/4pn1p/3p4/2PP3B/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 8',
			},
		},
		'r1bq1rk1/pppnbpp1/4pn1p/3p4/2PP3B/2N1PN2/PP3PPP/2RQKB1R b K - 1 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 h6 8. Bh4

			variation => N__('QGD: Orthodox, 7. Rc1 h6 8. Bh4'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbpp1/4pn1p/3p4/2PP3B/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 8' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2 h6 8. Bh4

			variation => N__('QGD: Orthodox, Rubinstein, 7. Qc2 h6 8. Bh4'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp1nbpp1/4pn1p/2pp4/2PP3B/2N1PN2/PPQ2PPP/R3KB1R w KQ c6 0 9',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p1B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 Be7 6. e3 O-O

			variation => N__('Old Indian: 5. Bg5 Be7 6. e3 O-O'),
			moves => {
				'd1c2' => 'r1bq1rk1/pppnbppp/3p1n2/4p1B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 2 7',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p1B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 2 7' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 Be7 6. e3 O-O 7. Qc2

			variation => N__('Old Indian: 5. Bg5 Be7 6. e3 O-O 7. Qc2'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p1B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p1N1/2BPP3/2N5/PPP2PPP/R1BQK2R w KQ - 7 7' => {
			eco => 'C41s',
                        # TRANSLATORS:

			variation => N__('Philidor: Improved Hanham, 6. Ng5'),
			moves => {
				'c4f7' => 'r1bq1rk1/pppnbBpp/3p1n2/4p1N1/3PP3/2N5/PPP2PPP/R1BQK2R b KQ - 0 7',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP1QPPP/R1B2RK1 b - - 8 7' => {
			eco => 'C41w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. Qe2

			variation => N__('Philidor: Improved Hanham, 7. Qe2'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2BPP3/2N2N2/PPP1QPPP/R1B2RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQ1RK1 w - - 7 7' => {
			eco => 'C41u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O

			variation => N__('Philidor: Improved Hanham, Main Line'),
			moves => {
				'a2a4' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQ1RK1 b - a3 0 7',
				'd1e2' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP1QPPP/R1B2RK1 b - - 8 7',
				'f1e1' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQR1K1 b - - 8 7',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQR1K1 b - - 8 7' => {
			eco => 'C41x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. Re1

			variation => N__('Philidor: Improved Hanham, 7. Re1'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQR1K1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 7' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 Be7 6. Bg2 O-O 7. O-O

			variation => N__('Old Indian: 5. g3 Be7'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 7' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 Be7 6. Bg2 O-O

			variation => N__('Old Indian: 5. g3 Be7'),
			moves => {
				'e1g1' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 7',
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 3 7' => {
			eco => 'A55i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 O-O

			variation => N__('Old Indian: 5. e4 Be7 6. Be2 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/2PPP3/2N2NP1/PP3P1P/R1BQKB1R w KQ - 1 7' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 O-O

			variation => N__('Old Indian: 5. e4 Be7 6. g3 O-O'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/3p1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQ1RK1 b - a3 0 7' => {
			eco => 'C41v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O O-O 7. a4

			variation => N__('Philidor: Improved Hanham, 7. a4'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/P1BPP3/2N2N2/1PP2PPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4p3/3n2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. cxd5 Nxd5

			variation => N__('QGD: Orthodox, 7.cxd5 Nxd5'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3P2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7' => {
			eco => 'D60',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox Defence'),
			moves => {
				'f6d5' => 'r1bq1rk1/pppnbppp/4p3/3n2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/1QN1PN2/PP3PPP/R3KB1R b KQ - 4 7' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qb3

			variation => N__('QGD: Orthodox, Rauzer Variation'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/1QN1PN2/PP3PPP/R3KB1R w KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R b K - 4 7' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1

			variation => N__('QGD: Orthodox, 7. Rc1'),
			moves => {
				'a7a6' => 'r1bq1rk1/1ppnbppp/p3pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8',
				'b7b6' => 'r1bq1rk1/p1pnbppp/1p2pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8',
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8',
				'd5c4' => 'r1bq1rk1/pppnbppp/4pn2/6B1/2pP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8',
				'f8e8' => 'r1bqr1k1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 5 8',
				'h7h6' => 'r1bq1rk1/pppnbpp1/4pn1p/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 3 7' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7

			variation => N__('QGD: Orthodox Defence'),
			moves => {
				'a1c1' => 'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R b K - 4 7',
				'c4d5' => 'r1bq1rk1/pppnbppp/4pn2/3P2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7',
				'd1b3' => 'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/1QN1PN2/PP3PPP/R3KB1R b KQ - 4 7',
				'd1c2' => 'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 4 7',
				'f1d3' => 'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R b KQ - 4 7',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R b KQ - 4 7' => {
			eco => 'D61',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Qc2

			variation => N__('QGD: Orthodox, Rubinstein Variation'),
			moves => {
				'a7a6' => 'r1bq1rk1/1ppnbppp/p3pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8',
				'c7c5' => 'r1bq1rk1/pp1nbppp/4pn2/2pp2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ c6 0 8',
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8',
				'h7h6' => 'r1bq1rk1/pppnbpp1/4pn1p/3p2B1/2PP4/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N2N2/PPQ1PPPP/R3KB1R w KQ - 8 7' => {
			eco => 'D53n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 O-O 6. Qc2 Nbd7

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 O-O 6. Qc2 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R b KQ - 4 7' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3

			variation => N__('QGD: Orthodox, Botvinnik Variation'),
			moves => {
				'b7b6' => 'r1bq1rk1/p1pnbppp/1p2pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 8',
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 8',
				'd5c4' => 'r1bq1rk1/pppnbppp/4pn2/6B1/2pP4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7' => {
			eco => 'D37j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 Nbd7

			variation => N__('QGD: Classical, 5...O-O 6. e3 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. b3

			variation => N__('Catalan: Closed, 6...Nbd7 7. b3'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 7 7' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3

			variation => N__('Catalan: Closed, 6...Nbd7 7. Nc3'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'd5c4' => 'r1bq1rk1/pppnbppp/4pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/3Q1NP1/PP2PPBP/RNB2RK1 b - - 7 7' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qd3

			variation => N__('Catalan: Closed, 6...Nbd7 7. Qd3'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 7 7' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nbd2

			variation => N__('Catalan: Closed, 6...Nbd7 7.Nbd2'),
			moves => {
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 6 7' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7

			variation => N__('Catalan: Closed, 6...Nbd7'),
			moves => {
				'b1c3' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 7 7',
				'b1d2' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 7 7',
				'b2b3' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
				'd1c2' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 7 7',
				'd1d3' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/3Q1NP1/PP2PPBP/RNB2RK1 b - - 7 7',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 7 7' => {
			eco => 'E08a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2

			variation => N__('Catalan: Closed, 7. Qc2'),
			moves => {
				'b7b6' => 'r1bq1rk1/p1pnbppp/1p2pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8',
				'c7c5' => 'r1bq1rk1/pp1nbppp/4pn2/2pp4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - c6 0 8',
				'c7c6' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/6B1/2BP4/2N1PN2/PP3PPP/2RQK2R b K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, 7. Rc1 dxc4'),
			moves => {
				'a7a6' => 'r1bq1rk1/1ppnbppp/p3pn2/6B1/2BP4/2N1PN2/PP3PPP/2RQK2R w K - 0 9',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/6B1/2BP4/2N1PN2/PP3PPP/R2QK2R b KQ - 0 8' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 dxc4 8. Bxc4

			variation => N__('QGD: Orthodox, Botvinnik, 7...dxc4 8.Bxc4'),
			moves => {
				'a7a6' => 'r1bq1rk1/1ppnbppp/p3pn2/6B1/2BP4/2N1PN2/PP3PPP/R2QK2R w KQ - 0 9',
				'c7c5' => 'r1bq1rk1/pp1nbppp/4pn2/2p3B1/2BP4/2N1PN2/PP3PPP/R2QK2R w KQ c6 0 9',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/6B1/2pP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 dxc4

			variation => N__('QGD: Orthodox, 7. Rc1 dxc4'),
			moves => {
				'f1c4' => 'r1bq1rk1/pppnbppp/4pn2/6B1/2BP4/2N1PN2/PP3PPP/2RQK2R b K - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/6B1/2pP4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 8' => {
			eco => 'D60',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Bd3 dxc4

			variation => N__('QGD: Orthodox, Botvinnik, 7...dxc4'),
			moves => {
				'd3c4' => 'r1bq1rk1/pppnbppp/4pn2/6B1/2BP4/2N1PN2/PP3PPP/R2QK2R b KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3 dxc4

			variation => N__('Catalan: Closed, 6...Nbd7 7. Nc3 dxc4'),
			moves => {
				'e2e4' => 'r1bq1rk1/pppnbppp/4pn2/8/2pPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8',
			},
		},
		'r1bq1rk1/pppnbppp/4pn2/8/2pPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8' => {
			eco => 'E07',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Nc3 dxc4 8. e4

			variation => N__('Catalan: Closed, 6...Nbd7 7. Nc3 dxc4'),
			moves => {
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Pp3/2P1Pp2/2N1BP2/PP2B1PP/R2QNRK1 w - - 0 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Be3 f4

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Be3 f4'),
			moves => {
				'e3f2' => 'r1bq1rk1/pppnn1bp/3p2p1/3Pp3/2P1Pp2/2N2P2/PP2BBPP/R2QNRK1 b - - 1 12',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Pp3/2P1Pp2/2N2P2/PP2BBPP/R2QNRK1 b - - 1 12' => {
			eco => 'E99',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Be3 f4'),
			moves => {
				'g6g5' => 'r1bq1rk1/pppnn1bp/3p4/3Pp1p1/2P1Pp2/2N2P2/PP2BBPP/R2QNRK1 w - - 0 13',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Pp3/2P1Pp2/2NN1P2/PP2B1PP/R1BQ1RK1 w - - 0 12' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Nd3 f4

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Nd3 f4'),
			moves => {
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P1P1/2N2P2/PP2B2P/R1BQNRK1 b - g3 0 11' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. g4

			variation => N__('King\'s Indian: Mar del Plata, Benko Attack'),
			moves => {
				'd7f6' => 'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P1P1/2N2P2/PP2B2P/R1BQNRK1 w - - 1 12',
				'g8h8' => 'r1bq1r1k/pppnn1bp/3p2p1/3Ppp2/2P1P1P1/2N2P2/PP2B2P/R1BQNRK1 w - - 1 12',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2N1B3/PP2BPPP/R2QNRK1 w - f6 0 11' => {
			eco => 'E98k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Be3 f5

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Be3 f5'),
			moves => {
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2N1BP2/PP2B1PP/R2QNRK1 b - - 1 11' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Be3

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Be3'),
			moves => {
				'f5f4' => 'r1bq1rk1/pppnn1bp/3p2p1/3Pp3/2P1Pp2/2N1BP2/PP2B1PP/R2QNRK1 w - - 0 12',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2N2P2/PP2B1PP/R1BQNRK1 w - f6 0 11' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5'),
			moves => {
				'c1e3' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2N1BP2/PP2B1PP/R2QNRK1 b - - 1 11',
				'e1d3' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN1P2/PP2B1PP/R1BQ1RK1 b - - 1 11',
				'g2g4' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P1P1/2N2P2/PP2B2P/R1BQNRK1 b - g3 0 11',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN1P2/PP2B1PP/R1BQ1RK1 b - - 1 11' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Nd3

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Nd3'),
			moves => {
				'd7f6' => 'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P3/2NN1P2/PP2B1PP/R1BQ1RK1 w - - 2 12',
				'f5f4' => 'r1bq1rk1/pppnn1bp/3p2p1/3Pp3/2P1Pp2/2NN1P2/PP2B1PP/R1BQ1RK1 w - - 0 12',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 b - - 1 11' => {
			eco => 'E98n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Nd3 f5 11. Bd2

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Nd3 f5 11. Bd2'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp1nn1bp/3p2p1/2pPpp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 w - c6 0 12',
				'd7f6' => 'r1bq1rk1/ppp1n1bp/3p1np1/3Ppp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 w - - 2 12',
				'g8h8' => 'r1bq1r1k/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 w - - 2 12',
			},
		},
		'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 w - f6 0 11' => {
			eco => 'E98m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Nd3 f5

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Nd3 f5'),
			moves => {
				'c1d2' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP1BBPPP/R2Q1RK1 b - - 1 11',
			},
		},
		'r1bq1rk1/pppnn1bp/3p4/3Pp1p1/2P1Pp2/2N2P2/PP2BBPP/R2QNRK1 w - - 0 13' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Be3 f4 12. Bf2 g5

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Be3 f4 12. Bf2 g5'),
			moves => {
				'a2a4' => 'r1bq1rk1/pppnn1bp/3p4/3Pp1p1/P1P1Pp2/2N2P2/1P2BBPP/R2QNRK1 b - a3 0 13',
			},
		},
		'r1bq1rk1/pppnn1bp/3p4/3Pp1p1/P1P1Pp2/2N2P2/1P2BBPP/R2QNRK1 b - a3 0 13' => {
			eco => 'E99',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3 f5 11. Be3 f4 12. Bf2 g5 13. a4

			variation => N__('King\'s Indian: Mar del Plata, 10. f3 f5 11. Be3 f4 12. Bf2 g5 13. a4'),
			moves => {
				'e7g6' => 'r1bq1rk1/pppn2bp/3p2n1/3Pp1p1/P1P1Pp2/2N2P2/1P2BBPP/R2QNRK1 w - - 1 14',
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/1PP1P3/2N5/P2NBPPP/R1BQ1RK1 b - b3 0 10' => {
			eco => 'E97v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 Nd7 10. b4

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 Nd7 10. b4'),
			moves => {
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QNRK1 b - - 4 10' => {
			eco => 'E98j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Be3

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Be3'),
			moves => {
				'f7f5' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2N1B3/PP2BPPP/R2QNRK1 w - f6 0 11',
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N2P2/PP2B1PP/R1BQNRK1 b - - 0 10' => {
			eco => 'E98r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. f3

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. f3'),
			moves => {
				'f7f5' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2N2P2/PP2B1PP/R1BQNRK1 w - f6 0 11',
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 3 10' => {
			eco => 'E97v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 Nd7

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 Nd7'),
			moves => {
				'b2b4' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/1PP1P3/2N5/P2NBPPP/R1BQ1RK1 b - b3 0 10',
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNR1K b - - 4 10' => {
			eco => 'E98i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Kh1

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Kh1'),
			moves => {
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 w - - 3 10' => {
			eco => 'E98h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7'),
			moves => {
				'c1e3' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QNRK1 b - - 4 10',
				'e1d3' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 b - - 4 10',
				'f2f3' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N2P2/PP2B1PP/R1BQNRK1 b - - 0 10',
				'g1h1' => 'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNR1K b - - 4 10',
			},
		},
		'r1bq1rk1/pppnnpbp/3p2p1/3Pp3/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 b - - 4 10' => {
			eco => 'E98l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Nd7 10. Nd3

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Nd7 10. Nd3'),
			moves => {
				'f7f5' => 'r1bq1rk1/pppnn1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 w - f6 0 11',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/4P3/2PP4/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E91r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nbd7 7. e5

			variation => N__('King\'s Indian: 6. Be2 Nbd7 7. e5'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PP4/2N1PN2/PP2BPPP/R2QK2R b KQ - 2 7' => {
			eco => 'E61y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3 Nbd7 7. Be2

			variation => N__('King\'s Indian: Smyslov System, 5...O-O 6. e3 Nbd7 7. Be2'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nppbp/2pp1np1/6B1/2PP4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 8',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7' => {
			eco => 'E61y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3 Nbd7

			variation => N__('King\'s Indian: Smyslov System, 5...O-O 6. e3 Nbd7'),
			moves => {
				'f1e2' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PP4/2N1PN2/PP2BPPP/R2QK2R b KQ - 2 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 7' => {
			eco => 'E90l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 Nbd7 7. Bg5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 Nbd7 7. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N2N2/PP2BPPP/R2QK2R b KQ - 5 7' => {
			eco => 'E91q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nbd7 7. Bg5

			variation => N__('King\'s Indian: 6. Be2 Nbd7 7. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 3 7' => {
			eco => 'E71j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 Nbd7

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR b KQ - 5 7' => {
			eco => 'E73n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Nbd7 7. Qd2

			variation => N__('King\'s Indian: Averbakh, 6...Nbd7 7. Qd2'),
			moves => {
				'e7e5' => 'r1bq1rk1/pppn1pbp/3p1np1/4p1B1/2PPP3/2N5/PP1QBPPP/R3K1NR w KQ e6 0 8',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 4 7' => {
			eco => 'E73m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 Nbd7

			variation => N__('King\'s Indian: Averbakh, 6...Nbd7'),
			moves => {
				'd1d2' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR b KQ - 5 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 7' => {
			eco => 'E67b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O

			variation => N__('King\'s Indian: Fianchetto with 6... Nd7 7.O-O'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nppbp/2pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'e7e5' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - e6 0 8',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 7' => {
			eco => 'E67a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7

			variation => N__('King\'s Indian: Fianchetto with 6... Nd7'),
			moves => {
				'e1g1' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 7' => {
			eco => 'E60o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O Nbd7

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6...Nbd7'),
			moves => {
				'd1c2' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 3 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 3 7' => {
			eco => 'E60p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O Nbd7 7. Qc2

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6...Nbd7 7. Qc2'),
			moves => {
				'e7e5' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - e6 0 8',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 4 7' => {
			eco => 'E81r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nbd7 7. Qd2

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 Nbd7 7. Qd2'),
			moves => {
				'c7c5' => 'r1bq1rk1/pp1nppbp/3p1np1/2p5/2PPP3/2N1BP2/PP1Q2PP/R3KBNR w KQ c6 0 8',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 3 7' => {
			eco => 'E81q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nbd7

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 Nbd7'),
			moves => {
				'd1d2' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 4 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 7' => {
			eco => 'E90l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 Nbd7

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 Nbd7'),
			moves => {
				'c1g5' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N2N1P/PP3PP1/R2QKB1R b KQ - 2 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 5 7' => {
			eco => 'E91s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nbd7 7. O-O

			variation => N__('King\'s Indian: 6. Be2 Nbd7 7.O-O'),
			moves => {
				'c7c6' => 'r1bq1rk1/pp1nppbp/2pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7' => {
			eco => 'E91q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Nbd7

			variation => N__('King\'s Indian: 6. Be2 Nbd7'),
			moves => {
				'c1g5' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N2N2/PP2BPPP/R2QK2R b KQ - 5 7',
				'e1g1' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 5 7',
				'e4e5' => 'r1bq1rk1/pppnppbp/3p1np1/4P3/2PP4/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7',
			},
		},
		'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ - 3 7' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2 Nbd7

			variation => N__('King\'s Indian: 4. e4 d6 5. g3 O-O 6. Bg2 Nbd7'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/5np1/3p2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R w KQ - 1 7' => {
			eco => 'D03q',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7 5. e3 O-O 6. c3 Nbd7

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. c3 Nbd7'),
			moves => {
				'f1e2' => 'r1bq1rk1/pppnppbp/5np1/3p2B1/3P4/2P1PN2/PP1NBPPP/R2QK2R b KQ - 2 7',
			},
		},
		'r1bq1rk1/pppnppbp/5np1/3p2B1/3P4/2P1PN2/PP1NBPPP/R2QK2R b KQ - 2 7' => {
			eco => 'D03r',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7 5. e3 O-O 6. c3 Nbd7 7. Be2

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. c3 Nbd7 7. Be2'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/5np1/6N1/2BP4/1QN1P3/PP3PPP/R1B1K2R b KQ - 2 8' => {
			eco => 'D95i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 dxc4 7. Bxc4 Nbd7 8. Ng5

			variation => N__('Grünfeld: Pachman Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppnppbp/5np1/8/2BP4/1QN1PN2/PP3PPP/R1B1K2R w KQ - 1 8' => {
			eco => 'D95i',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 dxc4 7.Bxc4'),
			moves => {
				'f3g5' => 'r1bq1rk1/pppnppbp/5np1/6N1/2BP4/1QN1P3/PP3PPP/R1B1K2R b KQ - 2 8',
			},
		},
		'r1bq1rk1/pppp1ppp/1bn2n2/1B2p1B1/3PP3/2P2N2/PP3PPP/RN1Q1RK1 b - - 2 7' => {
			eco => 'C65p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Bc5 5. c3 O-O 6. d4 Bb6 7. Bg5

			variation => N__('Spanish: Berlin, Benelux Variation, 7. Bg5'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/1bn2n2/1B2p3/3PP3/2P2N2/PP3PPP/RNBQ1RK1 w - - 1 7' => {
			eco => 'C65o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Bc5 5. c3 O-O 6. d4 Bb6

			variation => N__('Spanish: Berlin, Benelux Variation'),
			moves => {
				'c1g5' => 'r1bq1rk1/pppp1ppp/1bn2n2/1B2p1B1/3PP3/2P2N2/PP3PPP/RN1Q1RK1 b - - 2 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2B2n2/4p3/1b2P3/2N2N2/PPPP1PPP/R1BQ1RK1 b - - 0 6' => {
			eco => 'C49d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. Bxc6

			variation => N__('Four Knights: Nimzowitsch (Paulsen) Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1B1Np3/1b2P3/5N2/PPPP1PPP/R1BQ1RK1 b - - 9 6' => {
			eco => 'C49c',
                        # TRANSLATORS:

			variation => N__('Four Knights: 4. Bb5 Bb4 5.O-O O-O'),
			moves => {
				'f6d5' => 'r1bq1rk1/pppp1ppp/2n5/1B1np3/1b2P3/5N2/PPPP1PPP/R1BQ1RK1 w - - 0 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1B2P3/1b1P1P2/2N5/PPP3PP/R1BQ1RK1 w - - 1 10' => {
			eco => 'C48f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bc5 5. O-O O-O 6. Nxe5 Nxe5 7. d4 Bd6 8. f4 Nc6 9. e5 Bb4

			variation => N__('Four Knights: Spanish, Classical, Bardeleben Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/1b2P3/2N2N2/PPPP1PPP/R1BQ1RK1 w - - 8 6' => {
			eco => 'C49c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O

			variation => N__('Four Knights: 4. Bb5 Bb4 5.O-O O-O'),
			moves => {
				'b5c6' => 'r1bq1rk1/pppp1ppp/2B2n2/4p3/1b2P3/2N2N2/PPPP1PPP/R1BQ1RK1 b - - 0 6',
				'c3d5' => 'r1bq1rk1/pppp1ppp/2n2n2/1B1Np3/1b2P3/5N2/PPPP1PPP/R1BQ1RK1 b - - 9 6',
				'd2d3' => 'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/1b2P3/2NP1N2/PPP2PPP/R1BQ1RK1 b - - 0 6',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/1b2P3/2NP1N2/PPP2PPP/R1BQ1RK1 b - - 0 6' => {
			eco => 'C49e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3

			variation => N__('Four Knights: Double Bishops, 6. d3'),
			moves => {
				'b4c3' => 'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/4P3/2bP1N2/PPP2PPP/R1BQ1RK1 w - - 0 7',
				'd7d6' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/1b2P3/2NP1N2/PPP2PPP/R1BQ1RK1 w - - 0 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/4P3/2PP1N2/P1P2PPP/R1BQ1RK1 b - - 0 7' => {
			eco => 'C49f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Double Bishops, 6. d3 Bxc3'),
			moves => {
				'd7d5' => 'r1bq1rk1/ppp2ppp/2n2n2/1B1pp3/4P3/2PP1N2/P1P2PPP/R1BQ1RK1 w - d6 0 8',
				'd7d6' => 'r1bq1rk1/ppp2ppp/2np1n2/1B2p3/4P3/2PP1N2/P1P2PPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/4P3/2bP1N2/PPP2PPP/R1BQ1RK1 w - - 0 7' => {
			eco => 'C49f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 Bxc3

			variation => N__('Four Knights: Double Bishops, 6. d3 Bxc3'),
			moves => {
				'b2c3' => 'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/4P3/2PP1N2/P1P2PPP/R1BQ1RK1 b - - 0 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1Bb1N3/4P3/2N5/PPPP1PPP/R1BQ1RK1 b - - 0 6' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'c6d4' => 'r1bq1rk1/pppp1ppp/5n2/1Bb1N3/3nP3/2N5/PPPP1PPP/R1BQ1RK1 w - - 1 7',
				'c6e5' => 'r1bq1rk1/pppp1ppp/5n2/1Bb1n3/4P3/2N5/PPPP1PPP/R1BQ1RK1 w - - 0 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1Bb1p3/3PP3/2P2N2/PP3PPP/RNBQ1RK1 b - d3 0 6' => {
			eco => 'C65n',
                        # TRANSLATORS:

			variation => N__('Spanish: Berlin, Beverwijk, 5. c3'),
			moves => {
				'c5b6' => 'r1bq1rk1/pppp1ppp/1bn2n2/1B2p3/3PP3/2P2N2/PP3PPP/RNBQ1RK1 w - - 1 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 w - - 8 6' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'f3e5' => 'r1bq1rk1/pppp1ppp/2n2n2/1Bb1N3/4P3/2N5/PPPP1PPP/R1BQ1RK1 b - - 0 6',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w - - 1 6' => {
			eco => 'C65n',
                        # TRANSLATORS:

			variation => N__('Spanish: Berlin, Beverwijk, 5. c3'),
			moves => {
				'd2d4' => 'r1bq1rk1/pppp1ppp/2n2n2/1Bb1p3/3PP3/2P2N2/PP3PPP/RNBQ1RK1 b - d3 0 6',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/3Np3/1bP5/4PN2/PPQP1PPP/R1B1KB1R b KQ - 4 6' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. e3 Bb4 5. Qc2'),
			moves => {
				'f8e8' => 'r1bqr1k1/pppp1ppp/2n2n2/3Np3/1bP5/4PN2/PPQP1PPP/R1B1KB1R w KQ - 5 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/4p3/1bP5/2N1PN2/PPQP1PPP/R1B1KB1R w KQ - 3 6' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. e3 Bb4 5. Qc2'),
			moves => {
				'c3d5' => 'r1bq1rk1/pppp1ppp/2n2n2/3Np3/1bP5/4PN2/PPQP1PPP/R1B1KB1R b KQ - 4 6',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPPBP/R1BQ1RK1 b - - 4 6' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bb4 5. Bg2 O-O 6. O-O

			variation => N__('English: Four Knights, Main Line 6.O-O'),
			moves => {
				'e5e4' => 'r1bq1rk1/pppp1ppp/2n2n2/8/1bP1p3/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 0 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPPBP/R1BQK2R w KQ - 3 6' => {
			eco => 'A29',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. g3 Bb4 5. Bg2'),
			moves => {
				'e1g1' => 'r1bq1rk1/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPPBP/R1BQ1RK1 b - - 4 6',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/2KR1BNR b - - 8 7' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'f8e8' => 'r1bqr1k1/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/2KR1BNR w - - 9 8',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/R3KBNR w KQ - 7 7' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'e1c1' => 'r1bq1rk1/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/2KR1BNR b - - 8 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/8/1bP1p3/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 0 7' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bb4 5. Bg2 O-O 6. O-O e4

			variation => N__('English: Four Knights, Main Line 6.O-O e4'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/b3N3/2BPP3/2P5/P4PPP/RNBQ1RK1 b - - 0 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O Nf6 7. d4 O-O 8. Nxe5

			variation => N__('Evans Gambit: Richardson Attack'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/2n2n2/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 w - - 1 8' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: 5... Ba5 6.O-O Nf6'),
			moves => {
				'f3e5' => 'r1bq1rk1/pppp1ppp/2n2n2/b3N3/2BPP3/2P5/P4PPP/RNBQ1RK1 b - - 0 8',
			},
		},
		'r1bq1rk1/pppp1ppp/2n5/1B1P4/1b2p3/5N2/PPPP1PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'C49c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. Nd5 Nxd5 7. exd5 e4

			variation => N__('Four Knights: Gunsberg Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/2n5/1B1Pp3/1b6/5N2/PPPP1PPP/R1BQ1RK1 b - - 0 7' => {
			eco => 'C49c',
                        # TRANSLATORS:

			variation => N__('Four Knights: 4. Bb5 Bb4 5.O-O O-O'),
			moves => {
				'e5e4' => 'r1bq1rk1/pppp1ppp/2n5/1B1P4/1b2p3/5N2/PPPP1PPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'r1bq1rk1/pppp1ppp/2n5/1B1np3/1b2P3/5N2/PPPP1PPP/R1BQ1RK1 w - - 0 7' => {
			eco => 'C49c',
                        # TRANSLATORS:

			variation => N__('Four Knights: 4. Bb5 Bb4 5.O-O O-O'),
			moves => {
				'e4d5' => 'r1bq1rk1/pppp1ppp/2n5/1B1Pp3/1b6/5N2/PPPP1PPP/R1BQ1RK1 b - - 0 7',
			},
		},
		'r1bq1rk1/pppp1ppp/2nb1n2/1B2P3/3P1P2/2N5/PPP3PP/R1BQ1RK1 b - - 0 9' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'd6b4' => 'r1bq1rk1/pppp1ppp/2n2n2/1B2P3/1b1P1P2/2N5/PPP3PP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bq1rk1/pppp1ppp/2nb1n2/1B6/3PPP2/2N5/PPP3PP/R1BQ1RK1 w - - 1 9' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'e4e5' => 'r1bq1rk1/pppp1ppp/2nb1n2/1B2P3/3P1P2/2N5/PPP3PP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r1bq1rk1/pppp1ppp/3b1n2/1B2n3/3PP3/2N5/PPP2PPP/R1BQ1RK1 w - - 1 8' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'f2f4' => 'r1bq1rk1/pppp1ppp/3b1n2/1B2n3/3PPP2/2N5/PPP3PP/R1BQ1RK1 b - f3 0 8',
			},
		},
		'r1bq1rk1/pppp1ppp/3b1n2/1B2n3/3PPP2/2N5/PPP3PP/R1BQ1RK1 b - f3 0 8' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'e5c6' => 'r1bq1rk1/pppp1ppp/2nb1n2/1B6/3PPP2/2N5/PPP3PP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bq1rk1/pppp1ppp/5n2/1Bb1N3/3nP3/2N5/PPPP1PPP/R1BQ1RK1 w - - 1 7' => {
			eco => 'C48f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bc5 5. O-O O-O 6. Nxe5 Nd4

			variation => N__('Four Knights: Spanish, Classical, Marshall Variation'),
			moves => {
			},
		},
		'r1bq1rk1/pppp1ppp/5n2/1Bb1n3/3PP3/2N5/PPP2PPP/R1BQ1RK1 b - d3 0 7' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'c5d6' => 'r1bq1rk1/pppp1ppp/3b1n2/1B2n3/3PP3/2N5/PPP2PPP/R1BQ1RK1 w - - 1 8',
			},
		},
		'r1bq1rk1/pppp1ppp/5n2/1Bb1n3/4P3/2N5/PPPP1PPP/R1BQ1RK1 w - - 0 7' => {
			eco => 'C48f',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'd2d4' => 'r1bq1rk1/pppp1ppp/5n2/1Bb1n3/3PP3/2N5/PPP2PPP/R1BQ1RK1 b - d3 0 7',
			},
		},
		'r1bq1rk1/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 w - - 3 6' => {
			eco => 'C55h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. O-O O-O

			variation => N__('Two Knights: 4. d3 Be7 5.O-O O-O'),
			moves => {
				'c4b3' => 'r1bq1rk1/ppppbppp/2n2n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 b - - 4 6',
				'f1e1' => 'r1bq1rk1/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQR1K1 b - - 4 6',
			},
		},
		'r1bq1rk1/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQR1K1 b - - 4 6' => {
			eco => 'C55k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. O-O O-O 6. Re1

			variation => N__('Two Knights: 4. d3 Be7 5.O-O O-O 6. Re1'),
			moves => {
			},
		},
		'r1bq1rk1/ppppbppp/2n2n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 b - - 4 6' => {
			eco => 'C55i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. O-O O-O 6. Bb3

			variation => N__('Two Knights: 4. d3 Be7 5.O-O O-O 6. Bb3'),
			moves => {
				'd7d6' => 'r1bq1rk1/ppp1bppp/2np1n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 w - - 0 7',
			},
		},
		'r1bq1rk1/ppppbppp/2n2n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQK2R w KQ - 3 6' => {
			eco => 'C55e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. Bb3 O-O

			variation => N__('Two Knights: 4. d3 Be7 5. Bb3 O-O'),
			moves => {
			},
		},
		'r1bq2r1/ppp2k1p/1b6/nP2B3/3PP3/8/P1P2PPP/RN1QK2R b KQ - 0 11' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'd8g5' => 'r1b3r1/ppp2k1p/1b6/nP2B1q1/3PP3/8/P1P2PPP/RN1QK2R w KQ - 1 12',
			},
		},
		'r1bq2r1/ppp2kBp/1b6/nP2p3/3PP3/8/P1P2PPP/RN1QK2R w KQ - 0 11' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'g7e5' => 'r1bq2r1/ppp2k1p/1b6/nP2B3/3PP3/8/P1P2PPP/RN1QK2R b KQ - 0 11',
			},
		},
		'r1bq3r/ppp2kpp/8/3p4/1n1Pn3/1Q3N2/PP3PPP/RN2K2R w KQ d6 0 11' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'f3e5' => 'r1bq3r/ppp2kpp/8/3pN3/1n1Pn3/1Q6/PP3PPP/RN2K2R b KQ - 1 11',
			},
		},
		'r1bq3r/ppp2kpp/8/3pN3/1n1Pn3/1Q6/PP3PPP/RN2K2R b KQ - 1 11' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'f7f6' => 'r1bq3r/ppp3pp/5k2/3pN3/1n1Pn3/1Q6/PP3PPP/RN2K2R w KQ - 2 12',
			},
		},
		'r1bq3r/ppp3pp/5k2/3pN3/1n1Pn3/1Q3P2/PP4PP/RN2K2R b KQ - 0 12' => {
			eco => 'C54m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Bd2 Nxe4 8. Bxb4 Nxb4 9. Bxf7+ Kxf7 10. Qb3+ d5 11. Ne5+ Kf6 12. f3

			variation => N__('Giuoco Piano: Krause Variation'),
			moves => {
			},
		},
		'r1bq3r/ppp3pp/5k2/3pN3/1n1Pn3/1Q6/PP3PPP/RN2K2R w KQ - 2 12' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'f2f3' => 'r1bq3r/ppp3pp/5k2/3pN3/1n1Pn3/1Q3P2/PP4PP/RN2K2R b KQ - 0 12',
			},
		},
		'r1bq3r/pppp1kpp/8/8/1n1Pn3/1Q3N2/PP3PPP/RN2K2R b KQ - 1 10' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'd7d5' => 'r1bq3r/ppp2kpp/8/3p4/1n1Pn3/1Q3N2/PP3PPP/RN2K2R w KQ d6 0 11',
			},
		},
		'r1bq3r/pppp1kpp/8/8/1n1Pn3/5N2/PP3PPP/RN1QK2R w KQ - 0 10' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'd1b3' => 'r1bq3r/pppp1kpp/8/8/1n1Pn3/1Q3N2/PP3PPP/RN2K2R b KQ - 1 10',
			},
		},
		'r1bq3r/ppppk1pp/2n2n2/2b1p1N1/4P3/1B6/PPPP1PPP/RNBQK2R b KQ - 2 6' => {
			eco => 'C57d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Bxf7+ Ke7 6. Bb3

			variation => N__('Two Knights: Traxler, 5.Bxf7+ Ke7 6. Bb3'),
			moves => {
			},
		},
		'r1bq3r/ppppk1pp/2n2n2/2bBp1N1/4P3/8/PPPP1PPP/RNBQK2R b KQ - 2 6' => {
			eco => 'C57d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Bxf7+ Ke7 6. Bd5

			variation => N__('Two Knights: Traxler, Chigorin/Pithart Variation'),
			moves => {
			},
		},
		'r1bq3r/ppppkBpp/2n2n2/2b1p1N1/4P3/8/PPPP1PPP/RNBQK2R w KQ - 1 6' => {
			eco => 'C57d',
                        # TRANSLATORS:

			variation => N__('Two Knights: Traxler, 5.Bxf7+'),
			moves => {
				'f7b3' => 'r1bq3r/ppppk1pp/2n2n2/2b1p1N1/4P3/1B6/PPPP1PPP/RNBQK2R b KQ - 2 6',
				'f7d5' => 'r1bq3r/ppppk1pp/2n2n2/2bBp1N1/4P3/8/PPPP1PPP/RNBQK2R b KQ - 2 6',
			},
		},
		'r1bqk1nr/1p1p1ppp/p1nQ4/4p3/4P3/8/PPP2PPP/RNB1KB1R b KQkq - 0 7' => {
			eco => 'B32k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, 5... a6'),
			moves => {
				'd8f6' => 'r1b1k1nr/1p1p1ppp/p1nQ1q2/4p3/4P3/8/PPP2PPP/RNB1KB1R w KQkq - 1 8',
			},
		},
		'r1bqk1nr/1p1p1ppp/p1nb4/4p3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B32k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, 5... a6'),
			moves => {
				'd1d6' => 'r1bqk1nr/1p1p1ppp/p1nQ4/4p3/4P3/8/PPP2PPP/RNB1KB1R b KQkq - 0 7',
			},
		},
		'r1bqk1nr/1p1pppbp/p1n3p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6' => {
			eco => 'A37b',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 a6

			variation => N__('English: Symmetrical, 5. Nf3 a6'),
			moves => {
			},
		},
		'r1bqk1nr/1p2bppp/p1np4/1N2p3/2P1P3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B32v',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. c4 Be7 7.N1c3'),
			moves => {
				'b5a3' => 'r1bqk1nr/1p2bppp/p1np4/4p3/2P1P3/N1N5/PP3PPP/R1BQKB1R b KQkq - 1 8',
			},
		},
		'r1bqk1nr/1p2bppp/p1np4/4p3/2P1P3/N1N5/PP3PPP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'B32v',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. c4 Be7 7.N1c3'),
			moves => {
				'c8e6' => 'r2qk1nr/1p2bppp/p1npb3/4p3/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 2 9',
			},
		},
		'r1bqk1nr/1pp2pbp/2np2p1/p3p3/2P5/2NP2P1/PP2PPBP/1RBQK1NR w Kkq a6 0 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Rb1 a5

			variation => N__('English: Closed, 5. d3 d6 6. Rb1 a5'),
			moves => {
			},
		},
		'r1bqk1nr/1pp2ppp/p1pb4/4p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6' => {
			eco => 'C68l',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, 5.O-O Bd6'),
			moves => {
				'e5d4' => 'r1bqk1nr/1pp2ppp/p1pb4/8/3pP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqk1nr/1pp2ppp/p1pb4/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C68l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Bd6

			variation => N__('Spanish: Exchange, 5.O-O Bd6'),
			moves => {
				'd2d4' => 'r1bqk1nr/1pp2ppp/p1pb4/4p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6',
			},
		},
		'r1bqk1nr/1pp2ppp/p1pb4/8/3pP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C68m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Bd6 6. d4 exd4

			variation => N__('Spanish: Exchange, 5.O-O Bd6 6. d4 exd4'),
			moves => {
			},
		},
		'r1bqk1nr/1ppp1pbp/2n3p1/p3p3/2P5/2N3P1/PP1PPPBP/1RBQK1NR w Kkq a6 0 6' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Rb1 a5

			variation => N__('English: Closed, 5. Rb1'),
			moves => {
			},
		},
		'r1bqk1nr/1ppp1ppp/p1n5/2b1p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Bc5

			variation => N__('Spanish: Classical Deferred'),
			moves => {
			},
		},
		'r1bqk1nr/1ppp1ppp/p1n5/4p3/Bb2P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Bb4

			variation => N__('Spanish: Alapin Deferred'),
			moves => {
			},
		},
		'r1bqk1nr/1ppp1ppp/pbn5/4p3/PPB1P3/2N2N2/2PP1PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. a4 a6 6. Nc3

			variation => N__('Evans Gambit Declined, Showalter Variation'),
			moves => {
			},
		},
		'r1bqk1nr/1ppp1ppp/pbn5/4p3/PPB1P3/5N2/2PP1PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, 5. a4'),
			moves => {
				'b1c3' => 'r1bqk1nr/1ppp1ppp/pbn5/4p3/PPB1P3/2N2N2/2PP1PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/1ppp1ppp/pbn5/8/P3P3/1N6/1PP2PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'C45q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6'),
			moves => {
				'b1c3' => 'r1bqk1nr/1ppp1ppp/pbn5/8/P3P3/1NN5/1PP2PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/1ppp1ppp/pbn5/8/P3P3/1NN5/1PP2PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'C45q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6 7. Nc3

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6 7. Nc3'),
			moves => {
				'd8f6' => 'r1b1k1nr/1ppp1ppp/pbn2q2/8/P3P3/1NN5/1PP2PPP/R1BQKB1R w KQkq - 2 8',
				'g8f6' => 'r1bqk2r/1ppp1ppp/pbn2n2/8/P3P3/1NN5/1PP2PPP/R1BQKB1R w KQkq - 2 8',
			},
		},
		'r1bqk1nr/1pppbppp/p1n5/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Be7

			variation => N__('Spanish: 4. Ba4 Be7'),
			moves => {
			},
		},
		'r1bqk1nr/2pp1ppp/p1n5/1pb1p3/4P3/1B3N2/PPPP1PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 b5 5. Bb3 Bc5

			variation => N__('Spanish: Graz Variation'),
			moves => {
			},
		},
		'r1bqk1nr/p1pp1ppp/2n5/bp2p3/2BPP3/2P2N2/P4PPP/RNBQK2R w KQkq b6 0 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 b5

			variation => N__('Evans Gambit: Leonhardt Variation'),
			moves => {
			},
		},
		'r1bqk1nr/p2pppbp/2p3p1/4P3/8/2N1B3/PPP2PPP/R2QKB1R w KQkq - 1 9' => {
			eco => 'B34s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Nxc6 bxc6 8. e5 Ng8

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7.Nxc6 bxc6 8. e5 Ng8'),
			moves => {
			},
		},
		'r1bqk1nr/pp1nbppp/2pp4/4P3/2B1P3/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6 5. O-O Be7 6. dxe5

			variation => N__('Philidor: Hanham, Steiner Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pp1nbppp/2pp4/4p3/2BPP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Krause Variation'),
			moves => {
				'd4e5' => 'r1bqk1nr/pp1nbppp/2pp4/4P3/2B1P3/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p2P2/2B1P3/2N2N2/PPPP2PP/R1BQK2R b KQkq - 0 6' => {
			eco => 'B23v',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bc4 e6 6. f5

			variation => N__('Sicilian: Closed, Grand Prix, Schofman Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2B1PP2/2N2N2/PPPP2PP/R1BQK2R w KQkq - 0 6' => {
			eco => 'B23u',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bc4 e6

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6 4. Nf3 Bg7 5. Bc4 e6'),
			moves => {
				'f4f5' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p2P2/2B1P3/2N2N2/PPPP2PP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A36p',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 e6

			variation => N__('English: Symmetrical, Botvinnik, 5... e6'),
			moves => {
				'g1e2' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A36p',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Botvinnik, 5... e6'),
			moves => {
				'g8e7' => 'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A36m',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 e6

			variation => N__('English: Symmetrical, 5. e3 e6'),
			moves => {
				'g1e2' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N1P1P1/PP1PNPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N1P1P1/PP1PNPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A36m',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 e6 6. Nge2

			variation => N__('English: Symmetrical, 5. e3 e6'),
			moves => {
				'g8e7' => 'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N1PNP1/PP1P1PBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'A37k',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. e3

			variation => N__('English: Symmetrical, 5. Nf3 e6 6. e3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A37l',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. O-O

			variation => N__('English: Symmetrical, 5. Nf3 e6 6.O-O'),
			moves => {
				'g8e7' => 'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6' => {
			eco => 'A37i',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6

			variation => N__('English: Symmetrical, 5. Nf3 e6'),
			moves => {
				'd2d3' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6',
				'e1g1' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
				'e2e3' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N1PNP1/PP1P1PBP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'A37j',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. d3

			variation => N__('English: Symmetrical, 5. Nf3 e6 6. d3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/P1N3P1/1P1PPPBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A36h',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. a3 e6

			variation => N__('English: Symmetrical, 5. a3 e6'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4P3/2NP2P1/PPP2PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 e6

			variation => N__('Sicilian: Closed, 3. g3 g6, 5. d3 e6'),
			moves => {
				'c1e3' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR b KQkq - 1 6',
				'f2f4' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR b KQkq f3 0 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR b KQkq - 1 6' => {
			eco => 'B24',
                        # TRANSLATORS:

			variation => N__('Sicilian: Closed, 3. g3 g6, 5. d3 e6'),
			moves => {
				'c6d4' => 'r1bqk1nr/pp1p1pbp/4p1p1/2p5/3nP3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq - 2 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4P3/3P1NP1/PPP2PBP/RNBQ1RK1 b kq e3 0 6' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7 4. O-O Nc6 5. d3 e6 6. e4

			variation => N__('Reti: 1... c5'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR b KQkq f3 0 6' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 e6 6. f4

			variation => N__('Sicilian: Closed, 3. g3 g6, 5. d3 e6 6. f4'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: 1... c5'),
			moves => {
				'e2e4' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4P3/3P1NP1/PPP2PBP/RNBQ1RK1 b kq e3 0 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/1P2P3/5N2/P1PP1PPP/RNBQR1K1 b kq b3 0 6' => {
			eco => 'B31p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 e5 6. b4

			variation => N__('Sicilian: Rossolimo, Gurgenidze Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w kq e6 0 6' => {
			eco => 'B31i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. c3 e5

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. c3 e5'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/4P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 6' => {
			eco => 'B31q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 e5 6. c3

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 e5'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq e6 0 6' => {
			eco => 'B31o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 e5

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 e5'),
			moves => {
				'b2b4' => 'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/1P2P3/5N2/P1PP1PPP/RNBQR1K1 b kq b3 0 6',
				'c2c3' => 'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/4P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq e6 0 6' => {
			eco => 'A36l',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 e5

			variation => N__('English: Symmetrical, 5. e3 e5 (Botvinnik Reversed)'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A37u',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7',
				'g8e7' => 'r1bqk2r/pp1pnpbp/2n3p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq e6 0 6' => {
			eco => 'A37q',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5

			variation => N__('English: Symmetrical, 5. Nf3 e5'),
			moves => {
				'a2a3' => 'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/P1N2NP1/1P1PPPBP/R1BQK2R b KQkq - 0 6',
				'd2d3' => 'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6',
				'e1g1' => 'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'A37s',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. d3

			variation => N__('English: Symmetrical, 5. Nf3 e5 6. d3'),
			moves => {
				'g8e7' => 'r1bqk2r/pp1pnpbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R w KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/P1N2NP1/1P1PPPBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'A37r',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. a3

			variation => N__('English: Symmetrical, 5. Nf3 e5 6. a3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1pbp/4p1p1/2p5/3nP3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq - 2 7' => {
			eco => 'B24',
                        # TRANSLATORS:

			variation => N__('Sicilian: Closed, 3. g3 g6, 5. d3 e6'),
			moves => {
				'c3e2' => 'r1bqk1nr/pp1p1pbp/4p1p1/2p5/3nP3/3PB1P1/PPP1NPBP/R2QK1NR b KQkq - 3 7',
			},
		},
		'r1bqk1nr/pp1p1pbp/4p1p1/2p5/3nP3/3PB1P1/PPP1NPBP/R2QK1NR b KQkq - 3 7' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 e6 6. Be3 Nd4 7. Nce2

			variation => N__('Sicilian: Closed, Smyslov Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1ppp/1bp5/1B2p3/3PP3/2P5/PP3PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C64t',
                        # TRANSLATORS:

			variation => N__('Spanish: Classical, 4.O-O Nd4 5.Nxd4 Bxd4 6. c3 Bb6 7. d4'),
			moves => {
				'b5a4' => 'r1bqk1nr/pp1p1ppp/1bp5/4p3/B2PP3/2P5/PP3PPP/RNBQ1RK1 b kq - 1 8',
			},
		},
		'r1bqk1nr/pp1p1ppp/1bp5/4p3/B2PP3/2P5/PP3PPP/RNBQ1RK1 b kq - 1 8' => {
			eco => 'C64u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4 Bxd4 6. c3 Bb6 7. d4 c6 8. Ba4

			variation => N__('Spanish: Classical, 6. c3 Bb6 7. d4 c6 8. Ba4'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp3ppp/1bpp4/4p3/B2PP3/2P5/PP3PPP/RNBQ1RK1 w kq - 0 9',
			},
		},
		'r1bqk1nr/pp1p1ppp/2n1p3/8/1b1NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B45b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Bb4

			variation => N__('Sicilian: Taimanov, 5. Nc3 Bb4'),
			moves => {
			},
		},
		'r1bqk1nr/pp1p1ppp/2p5/1Bb5/3pP3/3P4/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C61k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O Bc5 6. d3 c6

			variation => N__('Spanish: Bird\'s, 5.O-O Bc5 6. d3 c6'),
			moves => {
				'b5c4' => 'r1bqk1nr/pp1p1ppp/2p5/2b5/2BpP3/3P4/PPP2PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk1nr/pp1p1ppp/2p5/2b5/2BpP3/3P4/PPP2PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C61l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O Bc5 6. d3 c6 7. Bc4

			variation => N__('Spanish: Bird\'s, 5.O-O Bc5 6. d3 c6 7. Bc4'),
			moves => {
				'd7d5' => 'r1bqk1nr/pp3ppp/2p5/2bp4/2BpP3/3P4/PPP2PPP/RNBQ1RK1 w kq d6 0 8',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'B31h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. c3

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. c3'),
			moves => {
				'e7e5' => 'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w kq e6 0 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 5' => {
			eco => 'B31g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O'),
			moves => {
				'c2c3' => 'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5',
				'f1e1' => 'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 3 5',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 3 5' => {
			eco => 'B31j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1'),
			moves => {
				'e7e5' => 'r1bqk1nr/pp1p1pbp/2n3p1/1Bp1p3/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq e6 0 6',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 4 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4PP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5' => {
			eco => 'B23w',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bb5

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6 4. Nf3 Bg7 5. Bb5'),
			moves => {
				'c6d4' => 'r1bqk1nr/pp1pppbp/6p1/1Bp5/3nPP2/2N2N2/PPPP2PP/R1BQK2R w KQkq - 4 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2B1PP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5' => {
			eco => 'B23t',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bc4

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6 4. Nf3 Bg7 5. Bc4'),
			moves => {
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2B1PP2/2N2N2/PPPP2PP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQkq e3 0 5' => {
			eco => 'A36n',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4

			variation => N__('English: Symmetrical, Botvinnik System'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 1 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/1PN3P1/P2PPPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A36j',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. b3

			variation => N__('English: Symmetrical, 5. b3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A36k',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3

			variation => N__('English: Symmetrical, 5. e3'),
			moves => {
				'e7e5' => 'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq e6 0 6',
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N1PN2/PP1P1PPP/R1BQKB1R w KQkq - 1 5' => {
			eco => 'A35f',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 g6 4. e3 Bg7

			variation => N__('English: Symmetrical, 2. Nc3 Nc6 3. Nf3 g6 4. e3 Bg7'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5' => {
			eco => 'A37a',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3

			variation => N__('English: Symmetrical, 5. Nf3'),
			moves => {
				'a7a6' => 'r1bqk1nr/1p1pppbp/p1n3p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6',
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6',
				'e7e5' => 'r1bqk1nr/pp1p1pbp/2n3p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq e6 0 6',
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 4 6',
				'g8h6' => 'r1bqk2r/pp1pppbp/2n3pn/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 4 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'A36e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7

			variation => N__('English: Symmetrical, 3. g3 g6'),
			moves => {
				'a2a3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/P1N3P1/1P1PPPBP/R1BQK1NR b KQkq - 0 5',
				'b2b3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/1PN3P1/P2PPPBP/R1BQK1NR b KQkq - 0 5',
				'd2d3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5',
				'e2e3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5',
				'e2e4' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQkq e3 0 5',
				'g1f3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A36f',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3

			variation => N__('English: Symmetrical, 5. d3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/P1N3P1/1P1PPPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A36g',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. a3

			variation => N__('English: Symmetrical, 5. a3'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/P1N3P1/1P1PPPBP/R1BQK1NR w KQkq - 0 6',
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/2P5/P1N3P1/1P1PPPBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7

			variation => N__('Sicilian: Closed, 3. g3 g6'),
			moves => {
				'd2d3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/4P3/2NP2P1/PPP2PBP/R1BQK1NR b KQkq - 0 5',
				'f2f4' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/4PP2/2N3P1/PPPP2BP/R1BQK1NR b KQkq f3 0 5',
				'g1e2' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/4P3/2N3P1/PPPPNPBP/R1BQK2R b KQkq - 3 5',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/4P3/2N3P1/PPPPNPBP/R1BQK2R b KQkq - 3 5' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nge2

			variation => N__('Sicilian: Closed, 3. g3 g6, 5.Nge2'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/4P3/2NP2P1/PPP2PBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3

			variation => N__('Sicilian: Closed, 3. g3 g6, 5. d3'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NP2P1/PPP2PBP/R1BQK1NR w KQkq - 0 6',
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/4P3/2NP2P1/PPP2PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B23s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6'),
			moves => {
				'f1b5' => 'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4PP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5',
				'f1c4' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2B1PP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/4PP2/2N3P1/PPPP2BP/R1BQK1NR b KQkq f3 0 5' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. f4

			variation => N__('Sicilian: Closed, 3. g3 g6, 5. f4'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7 4. O-O Nc6 5. d3

			variation => N__('Reti: 1... c5'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 0 6',
				'e7e6' => 'r1bqk1nr/pp1p1pbp/2n1p1p1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 0 6',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/2p5/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 4 5' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7 4. O-O Nc6

			variation => N__('Reti: 1... c5'),
			moves => {
				'd2d3' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/2P1P3/1N6/PP3PPP/RNBQKB1R b KQkq - 2 6' => {
			eco => 'B37b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nb3

			variation => N__('Sicilian: Maroczy Bind, 6. Nb3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/2P1P3/8/PPN2PPP/RNBQKB1R b KQkq - 2 6' => {
			eco => 'B37c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/8/2P1P3/8/PPN2PPP/RNBQKB1R w KQkq - 0 7',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/8/2P1P3/8/PPN2PPP/RNBQKB1R w KQkq - 3 7',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/2PNP3/4B3/PP3PPP/RN1QKB1R b KQkq - 2 6' => {
			eco => 'B38a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3

			variation => N__('Sicilian: Maroczy Bind, 6. Be3'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/8/2PNP3/4B3/PP3PPP/RN1QKB1R w KQkq - 0 7',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/8/2PNP3/4B3/PP3PPP/RN1QKB1R w KQkq - 3 7',
				'g8h6' => 'r1bqk2r/pp1pppbp/2n3pn/8/2PNP3/4B3/PP3PPP/RN1QKB1R w KQkq - 3 7',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B37a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7

			variation => N__('Sicilian: Maroczy Bind, 5... Bg7'),
			moves => {
				'c1e3' => 'r1bqk1nr/pp1pppbp/2n3p1/8/2PNP3/4B3/PP3PPP/RN1QKB1R b KQkq - 2 6',
				'd4b3' => 'r1bqk1nr/pp1pppbp/2n3p1/8/2P1P3/1N6/PP3PPP/RNBQKB1R b KQkq - 2 6',
				'd4c2' => 'r1bqk1nr/pp1pppbp/2n3p1/8/2P1P3/8/PPN2PPP/RNBQKB1R b KQkq - 2 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'B34m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3'),
			moves => {
				'd7d6' => 'r1bqk1nr/pp2ppbp/2np2p1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7',
				'g8f6' => 'r1bqk2r/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 4 7',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'B34j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 5... Bg7'),
			moves => {
				'c1e3' => 'r1bqk1nr/pp1pppbp/2n3p1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 3 6',
				'd4b3' => 'r1bqk1nr/pp1pppbp/2n3p1/8/4P3/1NN5/PPP2PPP/R1BQKB1R b KQkq - 3 6',
				'd4e2' => 'r1bqk1nr/pp1pppbp/2n3p1/8/4P3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 3 6',
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/4P3/1NN5/PPP2PPP/R1BQKB1R b KQkq - 3 6' => {
			eco => 'B34l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Nb3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Nb3'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/2n3p1/8/4P3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 3 6' => {
			eco => 'B34k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Nde2

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6.Nde2'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/6p1/1Bp5/3nPP2/2N2N2/PPPP2PP/R1BQ1RK1 b kq - 5 6' => {
			eco => 'B23y',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bb5 Nd4 6. O-O

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6 4. Nf3 Bg7 5. Bb5 Nd4 6.O-O'),
			moves => {
			},
		},
		'r1bqk1nr/pp1pppbp/6p1/1Bp5/3nPP2/2N2N2/PPPP2PP/R1BQK2R w KQkq - 4 6' => {
			eco => 'B23w',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bb5 Nd4

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6 4. Nf3 Bg7 5. Bb5 Nd4'),
			moves => {
				'b5d3' => 'r1bqk1nr/pp1pppbp/6p1/2p5/3nPP2/2NB1N2/PPPP2PP/R1BQK2R b KQkq - 5 6',
				'e1g1' => 'r1bqk1nr/pp1pppbp/6p1/1Bp5/3nPP2/2N2N2/PPPP2PP/R1BQ1RK1 b kq - 5 6',
			},
		},
		'r1bqk1nr/pp1pppbp/6p1/2p5/3nPP2/2NB1N2/PPPP2PP/R1BQK2R b KQkq - 5 6' => {
			eco => 'B23x',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6 4. Nf3 Bg7 5. Bb5 Nd4 6. Bd3

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6 4. Nf3 Bg7 5. Bb5 Nd4 6. Bd3'),
			moves => {
			},
		},
		'r1bqk1nr/pp2bppp/2n5/2pp2B1/3P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 7' => {
			eco => 'D32q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. Bg5 Be7

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. Bg5 Be7'),
			moves => {
			},
		},
		'r1bqk1nr/pp2bppp/2np4/1N2p3/2P1P3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B32v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4 Be7 7. N1c3

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. c4 Be7 7.N1c3'),
			moves => {
				'a7a6' => 'r1bqk1nr/1p2bppp/p1np4/1N2p3/2P1P3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqk1nr/pp2bppp/2np4/1N2p3/2P1P3/8/PP2BPPP/RNBQK2R b KQkq - 2 7' => {
			eco => 'B32u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4 Be7 7. Be2

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. c4 Be7 7. Be2'),
			moves => {
			},
		},
		'r1bqk1nr/pp2bppp/2np4/1N2p3/2P1P3/8/PP3PPP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'B32t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4 Be7

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. c4 Be7'),
			moves => {
				'b1c3' => 'r1bqk1nr/pp2bppp/2np4/1N2p3/2P1P3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 7',
				'f1e2' => 'r1bqk1nr/pp2bppp/2np4/1N2p3/2P1P3/8/PP2BPPP/RNBQK2R b KQkq - 2 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A36s',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6

			variation => N__('English: Symmetrical, Botvinnik, 5... d6'),
			moves => {
				'g1e2' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A36t',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2

			variation => N__('English: Symmetrical, Botvinnik, 5... d6 6.Nge2'),
			moves => {
				'g8f6' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A37g',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. O-O

			variation => N__('English: Symmetrical, 5. Nf3 d6 6.O-O'),
			moves => {
				'g8h6' => 'r1bqk2r/pp2ppbp/2np2pn/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6' => {
			eco => 'A37e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6

			variation => N__('English: Symmetrical, 5. Nf3 d6'),
			moves => {
				'd2d3' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6',
				'e1g1' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'A37f',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. d3

			variation => N__('English: Symmetrical, 5. Nf3 d6 6. d3'),
			moves => {
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/2P5/P1N3P1/1P1PPPBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A36i',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. a3 d6

			variation => N__('English: Symmetrical, 5. a3 d6'),
			moves => {
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NP2P1/PPP1NPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B25b',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nge2

			variation => N__('Sicilian: Closed, 3. g3, 5. d3 d6 6.Nge2'),
			moves => {
				'e7e5' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4P3/2NP2P1/PPP1NPBP/R1BQK2R w KQkq e6 0 7',
				'e7e6' => 'r1bqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NP2P1/PPP1NPBP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NP2P1/PPP2PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'B25a',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6

			variation => N__('Sicilian: Closed, 3. g3, 5. d3 d6'),
			moves => {
				'c1e3' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR b KQkq - 1 6',
				'f2f4' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR b KQkq f3 0 6',
				'g1e2' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NP2P1/PPP1NPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR b KQkq - 1 6' => {
			eco => 'B26a',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3

			variation => N__('Sicilian: Closed, 6. Be3'),
			moves => {
				'a8b8' => '1rbqk1nr/pp2ppbp/2np2p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQk - 2 7',
				'e7e5' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq e6 0 7',
				'e7e6' => 'r1bqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq - 0 7',
				'g8f6' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq - 2 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/3P1NP1/PPP2PBP/RNBQ1RK1 b kq e3 0 6' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7 4. O-O Nc6 5. d3 d6 6. e4

			variation => N__('Reti: 1... c5'),
			moves => {
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR b KQkq f3 0 6' => {
			eco => 'B25e',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4

			variation => N__('Sicilian: Closed, 6. f4'),
			moves => {
				'a8b8' => '1rbqk1nr/pp2ppbp/2np2p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQk - 1 7',
				'e7e5' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQkq e6 0 7',
				'e7e6' => 'r1bqk1nr/pp3pbp/2npp1p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQkq - 0 7',
				'g8f6' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: 1... c5'),
			moves => {
				'e2e4' => 'r1bqk1nr/pp2ppbp/2np2p1/2p5/4P3/3P1NP1/PPP2PBP/RNBQ1RK1 b kq e3 0 6',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/2P1P3/8/PPN1BPPP/RNBQK2R b KQkq - 1 7' => {
			eco => 'B37e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 d6 7. Be2

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 d6 7. Be2'),
			moves => {
				'g8h6' => 'r1bqk2r/pp2ppbp/2np2pn/8/2P1P3/8/PPN1BPPP/RNBQK2R w KQkq - 2 8',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/2P1P3/8/PPN2PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B37d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 d6

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 d6'),
			moves => {
				'f1e2' => 'r1bqk1nr/pp2ppbp/2np2p1/8/2P1P3/8/PPN1BPPP/RNBQK2R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B38c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 d6'),
			moves => {
				'g8h6' => 'r1bqk2r/pp2ppbp/2np2pn/8/2PNP3/2N1B3/PP3PPP/R2QKB1R w KQkq - 2 8',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B36b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 d6 6. Nc3 Bg7

			variation => N__('Sicilian: Maroczy Bind, 5... d6 6. Nc3 Bg7'),
			moves => {
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/2PNP3/4B3/PP3PPP/RN1QKB1R w KQkq - 0 7' => {
			eco => 'B38c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 d6

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 d6'),
			moves => {
				'b1c3' => 'r1bqk1nr/pp2ppbp/2np2p1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B34n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 d6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 d6'),
			moves => {
				'd1d2' => 'r1bqk1nr/pp2ppbp/2np2p1/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp2ppbp/2np2p1/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B34o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 d6 7. Qd2

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 d6 7. Qd2'),
			moves => {
			},
		},
		'r1bqk1nr/pp2ppbp/2p3p1/2p5/4P3/3P1N1P/PPP2PP1/RNBQK2R b KQkq - 0 6' => {
			eco => 'B31f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. Bxc6 dxc6 5. d3 Bg7 6. h3

			variation => N__('Sicilian: Rossolimo, 3... g6 4.Bxc6 dxc6 5. d3 Bg7 6. h3'),
			moves => {
			},
		},
		'r1bqk1nr/pp2ppbp/2p3p1/2p5/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'B31e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Rossolimo, 3... g6 4.Bxc6 dxc6 5. d3'),
			moves => {
				'h2h3' => 'r1bqk1nr/pp2ppbp/2p3p1/2p5/4P3/3P1N1P/PPP2PP1/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A37u',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O d6

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O d6'),
			moves => {
				'd2d3' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A37v',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O d6 7. d3

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O d6 7. d3'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2npbp/2np2p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w kq - 1 8',
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4P3/2NP2P1/PPP1NPBP/R1BQK2R w KQkq e6 0 7' => {
			eco => 'B25c',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nge2 e5

			variation => N__('Sicilian: Closed, Botvinnik'),
			moves => {
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq e6 0 7' => {
			eco => 'B26d',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e5

			variation => N__('Sicilian: Closed, 6. Be3 e5'),
			moves => {
				'd1d2' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4P3/2NPB1P1/PPPQ1PBP/R3K1NR b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4P3/2NPB1P1/PPPQ1PBP/R3K1NR b KQkq - 1 7' => {
			eco => 'B26e',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e5 7. Qd2

			variation => N__('Sicilian: Closed, 6. Be3 e5 7. Qd2'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2npbp/2np2p1/2p1p3/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8',
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4PP2/2NP1NP1/PPP3BP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B25j',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e5 7. Nf3

			variation => N__('Sicilian: Closed, 6. f4 e5 7. Nf3'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2npbp/2np2p1/2p1p3/4PP2/2NP1NP1/PPP3BP/R1BQK2R w KQkq - 2 8',
			},
		},
		'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQkq e6 0 7' => {
			eco => 'B25i',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e5

			variation => N__('Sicilian: Closed, 6. f4 e5'),
			moves => {
				'g1f3' => 'r1bqk1nr/pp3pbp/2np2p1/2p1p3/4PP2/2NP1NP1/PPP3BP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NP2P1/PPP1NPBP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B25d',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nge2 e6

			variation => N__('Sicilian: Closed, 3. g3, 5. d3 d6 6.Nge2 e6'),
			moves => {
			},
		},
		'r1bqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq - 0 7' => {
			eco => 'B26k',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6

			variation => N__('Sicilian: Closed, 6. Be3 e6'),
			moves => {
				'd1d2' => 'r1bqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR b KQkq - 1 7' => {
			eco => 'B26l',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6 7. Qd2

			variation => N__('Sicilian: Closed, 6. Be3 e6 7. Qd2'),
			moves => {
				'a8b8' => '1rbqk1nr/pp3pbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQk - 2 8',
				'c6d4' => 'r1bqk1nr/pp3pbp/3pp1p1/2p5/3nP3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8',
				'd8a5' => 'r1b1k1nr/pp3pbp/2npp1p1/q1p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8',
				'g8e7' => 'r1bqk2r/pp2npbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8',
			},
		},
		'r1bqk1nr/pp3pbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B25m',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3

			variation => N__('Sicilian: Closed, 6. f4 e6'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2npbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQK2R w KQkq - 2 8',
			},
		},
		'r1bqk1nr/pp3pbp/2npp1p1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'B25l',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6

			variation => N__('Sicilian: Closed, 6. f4 e6'),
			moves => {
				'g1f3' => 'r1bqk1nr/pp3pbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/pp3pbp/3pp1p1/2p5/3nP3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8' => {
			eco => 'B26n',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6 7. Qd2 Nd4

			variation => N__('Sicilian: Closed, 6. Be3 e6 7. Qd2 Nd4'),
			moves => {
			},
		},
		'r1bqk1nr/pp3ppp/1bpp4/4p3/B2PP3/2P5/PP3PPP/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C64u',
                        # TRANSLATORS:

			variation => N__('Spanish: Classical, 6. c3 Bb6 7. d4 c6 8. Ba4'),
			moves => {
				'b1a3' => 'r1bqk1nr/pp3ppp/1bpp4/4p3/B2PP3/N1P5/PP3PPP/R1BQ1RK1 b kq - 1 9',
			},
		},
		'r1bqk1nr/pp3ppp/1bpp4/4p3/B2PP3/N1P5/PP3PPP/R1BQ1RK1 b kq - 1 9' => {
			eco => 'C64v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4 Bxd4 6. c3 Bb6 7. d4 c6 8. Ba4 d6 9. Na3

			variation => N__('Spanish: Classical, 6. c3 Bb6 7. d4 c6 8. Ba4 d6 9. Na3'),
			moves => {
			},
		},
		'r1bqk1nr/pp3ppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'C09e',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 7.dxc5'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2nppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'r1bqk1nr/pp3ppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'C09e',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 7.dxc5'),
			moves => {
				'e1g1' => 'r1bqk1nr/pp3ppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1bqk1nr/pp3ppp/2n5/3p4/1bpP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'D33e',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: Swedish, 7. Bg2'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2nppp/2n5/3p4/1bpP4/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 4 9',
			},
		},
		'r1bqk1nr/pp3ppp/2n5/3p4/1bpP4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 2 8' => {
			eco => 'D33e',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: Swedish, 7. Bg2'),
			moves => {
				'e1g1' => 'r1bqk1nr/pp3ppp/2n5/3p4/1bpP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'r1bqk1nr/pp3ppp/2nb4/1BPp4/8/5N2/PPPN1PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'C09e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. dxc5

			variation => N__('French: Tarrasch, Open, 7.dxc5'),
			moves => {
				'd6c5' => 'r1bqk1nr/pp3ppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqk1nr/pp3ppp/2nb4/1Bpp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 b kq - 5 7' => {
			eco => 'C09d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O

			variation => N__('French: Tarrasch, Open, 7.O-O'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2nppp/2nb4/1Bpp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 w kq - 6 8',
			},
		},
		'r1bqk1nr/pp3ppp/2nb4/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R w KQkq - 4 7' => {
			eco => 'C09d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6

			variation => N__('French: Tarrasch, Open, 5.Ngf3 Nc6 6. Bb5 Bd6'),
			moves => {
				'd4c5' => 'r1bqk1nr/pp3ppp/2nb4/1BPp4/8/5N2/PPPN1PPP/R1BQK2R b KQkq - 0 7',
				'e1g1' => 'r1bqk1nr/pp3ppp/2nb4/1Bpp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 b kq - 5 7',
			},
		},
		'r1bqk1nr/pp3ppp/2nbp3/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'b1d2' => 'r1bqk1nr/pp3ppp/2nbp3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6',
			},
		},
		'r1bqk1nr/pp3ppp/2nbp3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'g8e7' => 'r1bqk2r/pp2nppp/2nbp3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 3 7',
			},
		},
		'r1bqk1nr/pp3ppp/2p5/2bp4/2BpP3/3P4/PPP2PPP/RNBQ1RK1 w kq d6 0 8' => {
			eco => 'C61m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O Bc5 6. d3 c6 7. Bc4 d5

			variation => N__('Spanish: Bird\'s, 5.O-O Bc5 6. d3 c6 7. Bc4 d5'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1bppp/2np4/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C50g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Be7 4. d4 d6

			variation => N__('Italian: Hungarian Defence, 4. d4 d6'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1npbp/3p2p1/2PPp3/4P3/2N1B3/PP3PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'A42l',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3 e5 6. d5 Nce7 7. c5

			variation => N__('Modern: Averbakh, Kotov, 5. Be3 e5 6. d5 Nce7 7. c5'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1npbp/3p2p1/3Pp3/2P1P1P1/2N1B3/PP3P1P/R2QKBNR b KQkq g3 0 7' => {
			eco => 'A42m',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3 e5 6. d5 Nce7 7. g4

			variation => N__('Modern: Averbakh, Kotov, 5. Be3 e5 6. d5 Nce7 7. g4'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1npbp/3p2p1/3Pp3/2P1P3/2N1B3/PP3PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'A42l',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3 e5 6. d5 Nce7

			variation => N__('Modern: Averbakh, Kotov, 5. Be3 e5 6. d5 Nce7'),
			moves => {
				'c4c5' => 'r1bqk1nr/ppp1npbp/3p2p1/2PPp3/4P3/2N1B3/PP3PPP/R2QKBNR b KQkq - 0 7',
				'g2g4' => 'r1bqk1nr/ppp1npbp/3p2p1/3Pp3/2P1P1P1/2N1B3/PP3P1P/R2QKBNR b KQkq g3 0 7',
			},
		},
		'r1bqk1nr/ppp1ppbp/2np2p1/3P4/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A42h',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. d5

			variation => N__('Modern: Averbakh, Kotov, 5. d5'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N1B3/PP3PPP/R2QKBNR b KQkq - 2 5' => {
			eco => 'A42i',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3

			variation => N__('Modern: Averbakh, Kotov, 5. Be3'),
			moves => {
				'e7e5' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2PPP3/2N1B3/PP3PPP/R2QKBNR w KQkq e6 0 6',
			},
		},
		'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'A42g',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Nf3

			variation => N__('Modern: Averbakh, Kotov, 5. Nf3'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'A42g',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Nge2

			variation => N__('Modern: Averbakh, Kotov, 5.Nge2'),
			moves => {
			},
		},
		'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A42g',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6

			variation => N__('Modern: Averbakh, Kotov Variation'),
			moves => {
				'c1e3' => 'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N1B3/PP3PPP/R2QKBNR b KQkq - 2 5',
				'd4d5' => 'r1bqk1nr/ppp1ppbp/2np2p1/3P4/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5',
				'g1e2' => 'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 2 5',
				'g1f3' => 'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqk1nr/ppp1ppbp/2np2p1/8/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B06t',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. f4 Nc6

			variation => N__('Modern: Pseudo-Austrian Attack'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/3Pp3/2P1P3/2N1B3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'A42k',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3 e5 6. d5

			variation => N__('Modern: Averbakh, Kotov, 5. Be3 e5 6. d5'),
			moves => {
				'c6e7' => 'r1bqk1nr/ppp1npbp/3p2p1/3Pp3/2P1P3/2N1B3/PP3PPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P1P3/2NP2P1/PP3PBP/R1BQK1NR b KQkq e3 0 6' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4

			variation => N__('English: Botvinnik System'),
			moves => {
				'g8e7' => 'r1bqk2r/ppp1npbp/2np2p1/4p3/2P1P3/2NP2P1/PP3PBP/R1BQK1NR w KQkq - 1 7',
				'g8f6' => 'r1bqk2r/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP3PBP/R1BQK1NR w KQkq - 1 7',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A25',
                        # TRANSLATORS:

			variation => N__('English: Closed, 5. e3'),
			moves => {
				'g1e2' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A25',
                        # TRANSLATORS:

			variation => N__('English: Closed, 5. e3'),
			moves => {
				'c8e6' => 'r2qk1nr/ppp2pbp/2npb1p1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQkq - 2 7',
				'g8h6' => 'r1bqk2r/ppp2pbp/2np2pn/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3

			variation => N__('English: Closed, 5. d3 d6 6. Nf3'),
			moves => {
				'g8f6' => 'r1bqk2r/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NP2P1/PP2PPBP/1RBQK1NR b Kkq - 1 6' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Rb1

			variation => N__('English: Closed, 5. d3 d6 6. Rb1'),
			moves => {
				'a7a5' => 'r1bqk1nr/1pp2pbp/2np2p1/p3p3/2P5/2NP2P1/PP2PPBP/1RBQK1NR w Kkq a6 0 7',
				'f7f5' => 'r1bqk1nr/ppp3bp/2np2p1/4pp2/2P5/2NP2P1/PP2PPBP/1RBQK1NR w Kkq f6 0 7',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6

			variation => N__('English: Closed, 5. d3 d6'),
			moves => {
				'a1b1' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NP2P1/PP2PPBP/1RBQK1NR b Kkq - 1 6',
				'e2e3' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NPP1P1/PP3PBP/R1BQK1NR b KQkq - 0 6',
				'e2e4' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P1P3/2NP2P1/PP3PBP/R1BQK1NR b KQkq e3 0 6',
				'g1f3' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NPP1P1/PP3PBP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e3

			variation => N__('English: Closed, 5. d3 d6 6. e3'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2PPP3/2N1B3/PP2NPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'A42j',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3 e5 6. Nge2

			variation => N__('Modern: Averbakh, Kotov, 5. Be3 e5 6.Nge2'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2pbp/2np2p1/4p3/2PPP3/2N1B3/PP3PPP/R2QKBNR w KQkq e6 0 6' => {
			eco => 'A42j',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nc6 5. Be3 e5

			variation => N__('Modern: Averbakh, Kotov, 5. Be3 e5'),
			moves => {
				'd4d5' => 'r1bqk1nr/ppp2pbp/2np2p1/3Pp3/2P1P3/2N1B3/PP3PPP/R2QKBNR b KQkq - 0 6',
				'g1e2' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2PPP3/2N1B3/PP2NPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'r1bqk1nr/ppp2ppp/1b1p4/n2P4/2B1P3/5N2/P4PPP/RNBQ1RK1 w kq - 1 10' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Anderssen Variation'),
			moves => {
				'c1b2' => 'r1bqk1nr/ppp2ppp/1b1p4/n2P4/2B1P3/5N2/PB3PPP/RN1Q1RK1 b kq - 2 10',
			},
		},
		'r1bqk1nr/ppp2ppp/1b1p4/n2P4/2B1P3/5N2/PB3PPP/RN1Q1RK1 b kq - 2 10' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. d5 Na5 10. Bb2

			variation => N__('Evans Gambit: Ulvestad Variation'),
			moves => {
				'g8e7' => 'r1bqk2r/ppp1nppp/1b1p4/n2P4/2B1P3/5N2/PB3PPP/RN1Q1RK1 w kq - 3 11',
			},
		},
		'r1bqk1nr/ppp2ppp/1b1p4/n5B1/2BPP3/2N2N2/P4PPP/R2Q1RK1 b kq - 4 10' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. Nc3 Na5 10. Bg5

			variation => N__('Evans Gambit: Göring Attack'),
			moves => {
				'f7f6' => 'r1bqk1nr/ppp3pp/1b1p1p2/n5B1/2BPP3/2N2N2/P4PPP/R2Q1RK1 w kq - 0 11',
			},
		},
		'r1bqk1nr/ppp2ppp/1b1p4/n7/2BPP3/2N2N2/P4PPP/R1BQ1RK1 w kq - 3 10' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Morphy Attack'),
			moves => {
				'c1g5' => 'r1bqk1nr/ppp2ppp/1b1p4/n5B1/2BPP3/2N2N2/P4PPP/R2Q1RK1 b kq - 4 10',
			},
		},
		'r1bqk1nr/ppp2ppp/1bnp4/3P4/2B1P3/5N2/P4PPP/RNBQ1RK1 b kq - 0 9' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. d5

			variation => N__('Evans Gambit: Anderssen Variation'),
			moves => {
				'c6a5' => 'r1bqk1nr/ppp2ppp/1b1p4/n2P4/2B1P3/5N2/P4PPP/RNBQ1RK1 w kq - 1 10',
			},
		},
		'r1bqk1nr/ppp2ppp/1bnp4/4p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O d6 7. d4 Bb6

			variation => N__('Evans Gambit: Lasker\'s Defence'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/1bnp4/8/2BPP3/2N2N2/P4PPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. Nc3

			variation => N__('Evans Gambit: Morphy Attack'),
			moves => {
				'c6a5' => 'r1bqk1nr/ppp2ppp/1b1p4/n7/2BPP3/2N2N2/P4PPP/R1BQ1RK1 w kq - 3 10',
				'c8g4' => 'r2qk1nr/ppp2ppp/1bnp4/8/2BPP1b1/2N2N2/P4PPP/R1BQ1RK1 w kq - 3 10',
			},
		},
		'r1bqk1nr/ppp2ppp/1bnp4/8/2BPP3/5N2/P4PPP/RNBQ1RK1 w kq - 1 9' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6

			variation => N__('Evans Gambit: Normal Variation'),
			moves => {
				'b1c3' => 'r1bqk1nr/ppp2ppp/1bnp4/8/2BPP3/2N2N2/P4PPP/R1BQ1RK1 b kq - 2 9',
				'd4d5' => 'r1bqk1nr/ppp2ppp/1bnp4/3P4/2B1P3/5N2/P4PPP/RNBQ1RK1 b kq - 0 9',
			},
		},
		'r1bqk1nr/ppp2ppp/1bnp4/8/2BPP3/5N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 d6 5. d4 exd4 6. cxd4 Bb6

			variation => N__('Giuoco Piano: LaBourdonnais Variation'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2n1p3/1B6/3Pp3/P1N5/1PP2PPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'C15s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Bxc3+ 6. Nxc3 Nc6 7. Bb5

			variation => N__('French: Winawer, Alekhine Gambit, Kan, 7. Bb5'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2n1p3/3p4/1b1PP3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'C15p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 Nc6

			variation => N__('French: Winawer, Alekhine Gambit, 4... Nc6'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2n1p3/8/3Pp3/P1N5/1PP2PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'C15r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Bxc3+ 6. Nxc3 Nc6

			variation => N__('French: Winawer, Alekhine Gambit, Kan Variation'),
			moves => {
				'f1b5' => 'r1bqk1nr/ppp2ppp/2n1p3/1B6/3Pp3/P1N5/1PP2PPP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqk1nr/ppp2ppp/2n5/2bpp3/1PB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq d6 0 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 d5

			variation => N__('Evans Gambit: Hein Countergambit'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2n5/3p4/1b1P4/2NB4/PPP2PPP/R1BQK1NR w KQkq - 4 6' => {
			eco => 'C01w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Bb4 5. Bd3 Nc6

			variation => N__('French: Exchange Winawer, 5. Bd3 Nc6'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'C64o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O d6 5. c3

			variation => N__('Spanish: Classical, 4.O-O d6 5. c3'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5' => {
			eco => 'C64n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O d6

			variation => N__('Spanish: Classical, 4.O-O d6'),
			moves => {
				'c2c3' => 'r1bqk1nr/ppp2ppp/2np4/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 4. c3'),
			moves => {
				'd2d4' => 'r1bqk1nr/ppp2ppp/2np4/2b1p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b1p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 4. c3'),
			moves => {
				'e5d4' => 'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b5/2BPP3/5N2/P4PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'c5b6' => 'r1bqk1nr/ppp2ppp/1bnp4/8/2BPP3/5N2/P4PPP/RNBQ1RK1 w kq - 1 9',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b5/2BPP3/5N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 4. c3'),
			moves => {
				'c5b6' => 'r1bqk1nr/ppp2ppp/1bnp4/8/2BPP3/5N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'c3d4' => 'r1bqk1nr/ppp2ppp/2np4/2b5/2BPP3/5N2/P4PPP/RNBQ1RK1 b kq - 0 8',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C44y',
                        # TRANSLATORS:

			variation => N__('Scotch Gambit: 5.O-O'),
			moves => {
				'c8g4' => 'r2qk1nr/ppp2ppp/2np4/2b5/2BpP1b1/2P2N2/PP3PPP/RNBQ1RK1 w kq - 1 7',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 4. c3'),
			moves => {
				'c3d4' => 'r1bqk1nr/ppp2ppp/2np4/2b5/2BPP3/5N2/PP3PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C44y',
                        # TRANSLATORS:

			variation => N__('Scotch Gambit: 5.O-O'),
			moves => {
				'c2c3' => 'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/8/1bB1P3/2N2N2/PP3PPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Nxc3 Bb4 6. Bc4 d6 7. O-O

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5.Nxc3 Bb4 6. Bc4 d6 7.O-O'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/8/1bB1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Nxc3 Bb4 6. Bc4 d6

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5.Nxc3 Bb4 6. Bc4 d6'),
			moves => {
				'e1g1' => 'r1bqk1nr/ppp2ppp/2np4/8/1bB1P3/2N2N2/PP3PPP/R1BQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b3p1B1/2BPP3/2P2N2/P4PPP/RN1QK2R b KQkq - 1 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6 7. Bg5

			variation => N__('Evans Gambit: Sokolsky Variation'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O d6

			variation => N__('Evans Gambit: 5... Ba5 6.O-O d6'),
			moves => {
				'd2d4' => 'r1bqk1nr/ppp2ppp/2np4/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 b kq d3 0 7',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b3p3/2BPP3/1QP2N2/P4PPP/RNB1K2R b KQkq - 1 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6 7. Qb3

			variation => N__('Evans Gambit: Tartakower Attack'),
			moves => {
				'd8d7' => 'r1b1k1nr/pppq1ppp/2np4/b3p3/2BPP3/1QP2N2/P4PPP/RNB1K2R w KQkq - 2 8',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O d6 7. d4

			variation => N__('Evans Gambit: 5... Ba5 6.O-O d6 7. d4'),
			moves => {
				'a5b6' => 'r1bqk1nr/ppp2ppp/1bnp4/4p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 1 8',
				'c8d7' => 'r2qk1nr/pppb1ppp/2np4/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 1 8',
				'c8g4' => 'r2qk1nr/ppp2ppp/2np4/b3p3/2BPP1b1/2P2N2/P4PPP/RNBQ1RK1 w kq - 1 8',
				'e5d4' => 'r1bqk1nr/ppp2ppp/2np4/b7/2BpP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b3p3/2BPP3/2P2N2/P4PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 d6

			variation => N__('Evans Gambit: 5... Ba5 6. d4 d6'),
			moves => {
				'c1g5' => 'r1bqk1nr/ppp2ppp/2np4/b3p1B1/2BPP3/2P2N2/P4PPP/RN1QK2R b KQkq - 1 7',
				'd1b3' => 'r1bqk1nr/ppp2ppp/2np4/b3p3/2BPP3/1QP2N2/P4PPP/RNB1K2R b KQkq - 1 7',
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b7/2BpP3/1QP2N2/P4PPP/RNB2RK1 b kq - 1 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O d6 7. d4 exd4 8. Qb3

			variation => N__('Evans Gambit: Waller Attack'),
			moves => {
			},
		},
		'r1bqk1nr/ppp2ppp/2np4/b7/2BpP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: 5... Ba5 6.O-O d6 7. d4'),
			moves => {
				'd1b3' => 'r1bqk1nr/ppp2ppp/2np4/b7/2BpP3/1QP2N2/P4PPP/RNB2RK1 b kq - 1 8',
			},
		},
		'r1bqk1nr/ppp3bp/2np2p1/4pp2/2P5/2NP2P1/PP2PPBP/1RBQK1NR w Kkq f6 0 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Rb1 f5

			variation => N__('English: Closed, 5. d3 d6 6. Rb1 f5'),
			moves => {
			},
		},
		'r1bqk1nr/ppp3pp/1b1p1p2/n5B1/2BPP3/2N2N2/P4PPP/R2Q1RK1 w kq - 0 11' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Göring Attack'),
			moves => {
				'g5e3' => 'r1bqk1nr/ppp3pp/1b1p1p2/n7/2BPP3/2N1BN2/P4PPP/R2Q1RK1 b kq - 1 11',
			},
		},
		'r1bqk1nr/ppp3pp/1b1p1p2/n7/2BPP3/2N1BN2/P4PPP/R2Q1RK1 b kq - 1 11' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. Nc3 Na5 10. Bg5 f6 11. Be3

			variation => N__('Evans Gambit: Steinitz Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppn1pbp/3p2p1/3Pp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'A42u',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. d5 Nd7

			variation => N__('Modern: Averbakh, 4... e5 5. d5 Nd7'),
			moves => {
			},
		},
		'r1bqk1nr/pppn1pbp/3p2p1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'A42r',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. Nf3 Nd7 6. Be2

			variation => N__('Modern: Averbakh, 4... e5 5. Nf3 Nd7 6. Be2'),
			moves => {
				'g8e7' => 'r1bqk2r/pppnnpbp/3p2p1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQkq - 4 7',
			},
		},
		'r1bqk1nr/pppn1pbp/3p2p1/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A42q',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. Nf3 Nd7

			variation => N__('Modern: Averbakh, 4... e5 5. Nf3 Nd7'),
			moves => {
				'f1e2' => 'r1bqk1nr/pppn1pbp/3p2p1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'r1bqk1nr/pppn1ppp/3p4/2b1p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'C30l',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. Nc3 Nd7

			variation => N__('KGD: Classical, Hanham Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppnbppp/4p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'C10i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Be7

			variation => N__('French: Rubinstein, 5. Nf3 Be7'),
			moves => {
			},
		},
		'r1bqk1nr/pppnppbp/3p2p1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'A42f',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nd7 5. Nf3

			variation => N__('Modern: Averbakh, 4... Nd7 5. Nf3'),
			moves => {
			},
		},
		'r1bqk1nr/pppnppbp/3p2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A42f',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 Nd7

			variation => N__('Modern: Averbakh, 4... Nd7'),
			moves => {
				'g1f3' => 'r1bqk1nr/pppnppbp/3p2p1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqk1nr/pppp1Bpp/2n5/2b1p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C50h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. Bxf7+

			variation => N__('Giuoco Piano: Jerome Gambit'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQkq e3 0 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4

			variation => N__('English: Closed, 5. e4'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3

			variation => N__('English: Closed, 5. e3'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2N3P1/PP1PPPBP/1RBQK1NR b Kkq - 3 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Rb1

			variation => N__('English: Closed, 5. Rb1'),
			moves => {
				'a7a5' => 'r1bqk1nr/1ppp1pbp/2n3p1/p3p3/2P5/2N3P1/PP1PPPBP/1RBQK1NR w Kkq a6 0 6',
				'g8h6' => 'r1bqk2r/pppp1pbp/2n3pn/4p3/2P5/2N3P1/PP1PPPBP/1RBQK1NR w Kkq - 4 6',
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7

			variation => N__('English: Closed'),
			moves => {
				'a1b1' => 'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2N3P1/PP1PPPBP/1RBQK1NR b Kkq - 3 5',
				'd2d3' => 'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5',
				'e2e3' => 'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5',
				'e2e4' => 'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQkq e3 0 5',
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3

			variation => N__('English: Closed, 5. d3'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2pbp/2np2p1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR w KQkq - 0 6',
				'g8e7' => 'r1bqk2r/ppppnpbp/2n3p1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR w KQkq - 1 6',
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 2 6' => {
			eco => 'C46l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6 4. d4 exd4 5. Nxd4 Bg7 6. Be3

			variation => N__('Three Knights: Steinitz, 4. d4 exd4 5.Nxd4 Bg7 6. Be3'),
			moves => {
				'g8f6' => 'r1bqk2r/pppp1pbp/2n2np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 3 7',
			},
		},
		'r1bqk1nr/pppp1pbp/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'C46l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6 4. d4 exd4 5. Nxd4 Bg7

			variation => N__('Three Knights: Steinitz, 4. d4 exd4 5.Nxd4 Bg7'),
			moves => {
				'c1e3' => 'r1bqk1nr/pppp1pbp/2n3p1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 2 6',
			},
		},
		'r1bqk1nr/pppp1ppp/1b6/1B2p3/3PP3/2P5/PP3PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'C64t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4 Bxd4 6. c3 Bb6 7. d4

			variation => N__('Spanish: Classical, 4.O-O Nd4 5.Nxd4 Bxd4 6. c3 Bb6 7. d4'),
			moves => {
				'c7c6' => 'r1bqk1nr/pp1p1ppp/1bp5/1B2p3/3PP3/2P5/PP3PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk1nr/pppp1ppp/1b6/1B2p3/4P3/2P5/PP1P1PPP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'C64s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4 Bxd4 6. c3 Bb6

			variation => N__('Spanish: Classical, 4.O-O Nd4 5.Nxd4 Bxd4 6. c3 Bb6'),
			moves => {
				'd2d4' => 'r1bqk1nr/pppp1ppp/1b6/1B2p3/3PP3/2P5/PP3PPP/RNBQ1RK1 b kq d3 0 7',
			},
		},
		'r1bqk1nr/pppp1ppp/1b6/nP2N3/2B1P3/8/P1PP1PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, 5. b5'),
			moves => {
				'd8g5' => 'r1b1k1nr/pppp1ppp/1b6/nP2N1q1/2B1P3/8/P1PP1PPP/RNBQK2R w KQkq - 1 7',
				'g8h6' => 'r1bqk2r/pppp1ppp/1b5n/nP2N3/2B1P3/8/P1PP1PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqk1nr/pppp1ppp/1b6/nP2p3/2B1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, 5. b5'),
			moves => {
				'f3e5' => 'r1bqk1nr/pppp1ppp/1b6/nP2N3/2B1P3/8/P1PP1PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C64c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Bb6

			variation => N__('Spanish: Classical, Charousek Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/1P2p3/2B1P3/5N2/P1PP1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. b5

			variation => N__('Evans Gambit Declined, 5. b5'),
			moves => {
				'c6a5' => 'r1bqk1nr/pppp1ppp/1b6/nP2p3/2B1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/4p3/1PB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6

			variation => N__('Evans Gambit Declined'),
			moves => {
				'a2a4' => 'r1bqk1nr/pppp1ppp/1bn5/4p3/PPB1P3/5N2/2PP1PPP/RNBQK2R b KQkq a3 0 5',
				'b4b5' => 'r1bqk1nr/pppp1ppp/1bn5/1P2p3/2B1P3/5N2/P1PP1PPP/RNBQK2R b KQkq - 0 5',
				'c1b2' => 'r1bqk1nr/pppp1ppp/1bn5/4p3/1PB1P3/5N2/PBPP1PPP/RN1QK2R b KQkq - 2 5',
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/4p3/1PB1P3/5N2/PBPP1PPP/RN1QK2R b KQkq - 2 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. Bb2

			variation => N__('Evans Gambit Declined, Cordel Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/4p3/PPB1P3/5N2/2PP1PPP/RNBQK2R b KQkq a3 0 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. a4

			variation => N__('Evans Gambit Declined, 5. a4'),
			moves => {
				'a7a6' => 'r1bqk1nr/1ppp1ppp/pbn5/4p3/PPB1P3/5N2/2PP1PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/8/4P3/1N6/PPP2PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C45p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb6

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Nb3 Bb6'),
			moves => {
				'a2a4' => 'r1bqk1nr/pppp1ppp/1bn5/8/P3P3/1N6/1PP2PPP/RNBQKB1R b KQkq a3 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/1bn5/8/P3P3/1N6/1PP2PPP/RNBQKB1R b KQkq a3 0 6' => {
			eco => 'C45p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb6 6. a4

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Nb3 Bb6 6. a4'),
			moves => {
				'a7a6' => 'r1bqk1nr/1ppp1ppp/pbn5/8/P3P3/1N6/1PP2PPP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2B5/2b1p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C64a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. Bxc6

			variation => N__('Spanish: Classical, Exchange'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2N5/2b5/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C45r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nxc6

			variation => N__('Scotch: 4.Nxd4 Bc5 5.Nxc6'),
			moves => {
				'd8f6' => 'r1b1k1nr/pppp1ppp/2N2q2/2b5/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/1B2p3/1b2P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C60i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bb4 4. c3

			variation => N__('Spanish: Alapin, 4. c3'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/1B2p3/1b2P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C60h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bb4

			variation => N__('Spanish: Alapin Variation'),
			moves => {
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/1B2p3/1b2P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C46e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Bc5 4. Bb5

			variation => N__('Three Knights: 3... Bc5 4. Bb5'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C64b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3

			variation => N__('Spanish: Classical, 4. c3'),
			moves => {
				'c5b6' => 'r1bqk1nr/pppp1ppp/1bn5/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
				'd8e7' => 'r1b1k1nr/ppppqppp/2n5/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
				'd8f6' => 'r1b1k1nr/pppp1ppp/2n2q2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
				'f7f5' => 'r1bqk1nr/pppp2pp/2n5/1Bb1pp2/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq f6 0 5',
				'g8e7' => 'r1bqk2r/ppppnppp/2n5/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4' => {
			eco => 'C64k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O

			variation => N__('Spanish: Classical, 4.O-O'),
			moves => {
				'c6d4' => 'r1bqk1nr/pppp1ppp/8/1Bb1p3/3nP3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5',
				'd8f6' => 'r1b1k1nr/pppp1ppp/2n2q2/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
				'g8e7' => 'r1bqk2r/ppppnppp/2n5/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C64a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5

			variation => N__('Spanish: Classical Defence'),
			moves => {
				'b5c6' => 'r1bqk1nr/pppp1ppp/2B5/2b1p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
				'e1g1' => 'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1N3/4P3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'C46f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Bc5 4. Nxe5

			variation => N__('Three Knights: 3... Bc5 4.Nxe5'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1P3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B00u',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 e5 3. dxe5 Bc5

			variation => N__('Nimzowitsch Defence: Bielefelder Gambit'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/1PB1P3/5N2/P1PP1PPP/RNBQK2R b KQkq b3 0 4' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4

			variation => N__('Evans Gambit'),
			moves => {
				'c5b4' => 'r1bqk1nr/pppp1ppp/2n5/4p3/1bB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 0 5',
				'c5b6' => 'r1bqk1nr/pppp1ppp/1bn5/4p3/1PB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 1 5',
				'c5e7' => 'r1bqk1nr/ppppbppp/2n5/4p3/1PB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 1 5',
				'd7d5' => 'r1bqk1nr/ppp2ppp/2n5/2bpp3/1PB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq d6 0 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P1Q1/2N5/PPPP1PPP/R1B1K1NR b KQkq - 5 4' => {
			eco => 'C25l',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. Bc4 Bc5 4. Qg4

			variation => N__('Vienna: 2... Nc6 3. Bc4 Bc5 4. Qg4'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C50i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. Nc3

			variation => N__('Giuoco Piano: Three Knights Variation'),
			moves => {
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4' => {
			eco => 'C25j',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. Bc4 Bc5

			variation => N__('Vienna: 2... Nc6 3. Bc4 Bc5'),
			moves => {
				'd1g4' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P1Q1/2N5/PPPP1PPP/R1B1K1NR b KQkq - 5 4',
				'd2d3' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR b KQkq - 0 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR b KQkq - 0 4' => {
			eco => 'C25k',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. Bc4 Bc5 4. d3

			variation => N__('Vienna: 2... Nc6 3. Bc4 Bc5 4. d3'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'd2d4' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2BPP3/2P2N2/P4PPP/RNBQK2R b KQkq d3 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3

			variation => N__('Giuoco Piano: 4. c3'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 0 5',
				'd8e7' => 'r1b1k1nr/ppppqppp/2n5/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2P5/PP1PQPPP/RNB1K1NR b KQkq - 0 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. Qe2'),
			moves => {
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2P5/PP1PQPPP/RNB1K1NR w KQkq - 1 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C50n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3

			variation => N__('Giuoco Pianissimo: 4. d3'),
			moves => {
				'f7f5' => 'r1bqk1nr/pppp2pp/2n5/2b1pp2/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq f6 0 5',
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4' => {
			eco => 'C50k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. O-O

			variation => N__('Giuoco Piano: 4.O-O'),
			moves => {
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C50h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5

			variation => N__('Giuoco Piano'),
			moves => {
				'b1c3' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
				'b2b4' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/1PB1P3/5N2/P1PP1PPP/RNBQK2R b KQkq b3 0 4',
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
				'c4f7' => 'r1bqk1nr/pppp1Bpp/2n5/2b1p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'd2d3' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/8/PPPPQPPP/RNB1K1NR w KQkq - 4 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. Qe2'),
			moves => {
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2P5/PP1PQPPP/RNB1K1NR b KQkq - 0 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2BPP3/2P2N2/P4PPP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'e5d4' => 'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/2P2N2/P4PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C50h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d4

			variation => N__('Giuoco Piano: Rosentreter Gambit'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'C46e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Bc5

			variation => N__('Three Knights: 3... Bc5'),
			moves => {
				'f1b5' => 'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
				'f3e5' => 'r1bqk1nr/pppp1ppp/2n5/2b1N3/4P3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 0 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b1p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'C25i',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. g3 Bc5

			variation => N__('Vienna: Paulsen, 3... Bc5'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b3N1/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 3 5' => {
			eco => 'C44y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5 5. Ng5

			variation => N__('Scotch Gambit: 5. Ng5'),
			moves => {
				'g8h6' => 'r1bqk2r/pppp1ppp/2n4n/2b3N1/2BpP3/8/PPP2PPP/RNBQK2R w KQkq - 4 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/2B1P3/2Pp1N2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C44z',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5 5. c3 d3

			variation => N__('Scotch Gambit: 5. c3 d3'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/2P2N2/P4PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/2P2N2/P4PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'e1g1' => 'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/2P2N2/P4PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C44z',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5 5. c3

			variation => N__('Scotch Gambit: 5. c3'),
			moves => {
				'd4d3' => 'r1bqk1nr/pppp1ppp/2n5/2b5/2B1P3/2Pp1N2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'C44y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5 5. O-O

			variation => N__('Scotch Gambit: 5.O-O'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/2b5/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C44y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5

			variation => N__('Scotch Gambit: 4... Bc5'),
			moves => {
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 5',
				'e1g1' => 'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 3 5',
				'f3g5' => 'r1bqk1nr/pppp1ppp/2n5/2b3N1/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 3 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 2 5' => {
			eco => 'C45u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Be3

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Be3'),
			moves => {
				'd8f6' => 'r1b1k1nr/pppp1ppp/2n2q2/2b5/3NP3/4B3/PPP2PPP/RN1QKB1R w KQkq - 3 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C45n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5

			variation => N__('Scotch: 4.Nxd4 Bc5'),
			moves => {
				'c1e3' => 'r1bqk1nr/pppp1ppp/2n5/2b5/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 2 5',
				'd4b3' => 'r1bqk1nr/pppp1ppp/2n5/2b5/4P3/1N6/PPP2PPP/RNBQKB1R b KQkq - 2 5',
				'd4c6' => 'r1bqk1nr/pppp1ppp/2N5/2b5/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/2b5/4P3/1N6/PPP2PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'C45o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3

			variation => N__('Scotch: 4.Nxd4 Bc5 5. Nb3'),
			moves => {
				'c5b4' => 'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/1N6/PPP2PPP/RNBQKB1R w KQkq - 3 6',
				'c5b6' => 'r1bqk1nr/pppp1ppp/1bn5/8/4P3/1N6/PPP2PPP/RNBQKB1R w KQkq - 3 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/3Np3/1b2P3/5N2/PPPP1PPP/R1BQKB1R b KQkq - 5 4' => {
			eco => 'C46h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Bb4 4. Nd5

			variation => N__('Three Knights: 3... Bb4 4. Nd5'),
			moves => {
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/3Np3/1b2P3/5N2/PPPP1PPP/R1BQKB1R w KQkq - 6 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/4p3/1b2P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'C46g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Bb4

			variation => N__('Three Knights: 3... Bb4'),
			moves => {
				'c3d5' => 'r1bqk1nr/pppp1ppp/2n5/3Np3/1b2P3/5N2/PPPP1PPP/R1BQKB1R b KQkq - 5 4',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/4p3/1bB1P3/2P2N2/P2P1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3

			variation => N__('Evans Gambit Accepted, 5. c3'),
			moves => {
				'b4a5' => 'r1bqk1nr/pppp1ppp/2n5/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6',
				'b4c5' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6',
				'b4d6' => 'r1bqk1nr/pppp1ppp/2nb4/4p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6',
				'b4e7' => 'r1bqk1nr/ppppbppp/2n5/4p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6',
				'b4f8' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/4p3/1bB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4

			variation => N__('Evans Gambit Accepted'),
			moves => {
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/4p3/1bB1P3/2P2N2/P2P1PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/4p3/1bP5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Bb4

			variation => N__('English: Three Knights, 3... Bb4'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1b1NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C45c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bb4+

			variation => N__('Scotch: 4.Nxd4 Bb4+'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/1N6/PPP2PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C45o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb4+

			variation => N__('Scotch: Romanishin Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Nxc3 Bb4

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5.Nxc3 Bb4'),
			moves => {
				'f1c4' => 'r1bqk1nr/pppp1ppp/2n5/8/1bB1P3/2N2N2/PP3PPP/R1BQK2R b KQkq - 2 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/2P1Q3/PP3PPP/RNB1KBNR b KQkq - 0 5' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Paulsen Attack'),
			moves => {
				'b4e7' => 'r1bqk1nr/ppppbppp/2n5/8/4P3/2P1Q3/PP3PPP/RNB1KBNR w KQkq - 1 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/4Q3/PPP2PPP/RNB1KBNR w KQkq - 3 5' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Paulsen Attack'),
			moves => {
				'c2c3' => 'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/2P1Q3/PP3PPP/RNB1KBNR b KQkq - 0 5',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1bB1P3/2N2N2/PP3PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'C44w',
                        # TRANSLATORS:

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5.Nxc3 Bb4'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/8/1bB1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7',
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/8/1bB1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 3 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/8/1bBpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C44x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bb4+

			variation => N__('Scotch Gambit: London Defence'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b3p3/2B1P3/1QP2N2/P2P1PPP/RNB1K2R b KQkq - 2 6' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. Qb3

			variation => N__('Evans Gambit: 5... Ba5 6. Qb3'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O

			variation => N__('Evans Gambit: 5... Ba5 6.O-O'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQ1RK1 w kq - 0 7',
				'g8f6' => 'r1bqk2r/pppp1ppp/2n2n2/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQ1RK1 w kq - 3 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5

			variation => N__('Evans Gambit: 5... Ba5'),
			moves => {
				'd1b3' => 'r1bqk1nr/pppp1ppp/2n5/b3p3/2B1P3/1QP2N2/P2P1PPP/RNB1K2R b KQkq - 2 6',
				'd2d4' => 'r1bqk1nr/pppp1ppp/2n5/b3p3/2BPP3/2P2N2/P4PPP/RNBQK2R b KQkq d3 0 6',
				'e1g1' => 'r1bqk1nr/pppp1ppp/2n5/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b3p3/2BPP3/2P2N2/P4PPP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4

			variation => N__('Evans Gambit: 5... Ba5 6. d4'),
			moves => {
				'b7b5' => 'r1bqk1nr/p1pp1ppp/2n5/bp2p3/2BPP3/2P2N2/P4PPP/RNBQK2R w KQkq b6 0 7',
				'd7d6' => 'r1bqk1nr/ppp2ppp/2np4/b3p3/2BPP3/2P2N2/P4PPP/RNBQK2R w KQkq - 0 7',
				'e5d4' => 'r1bqk1nr/pppp1ppp/2n5/b7/2BpP3/2P2N2/P4PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b7/2B1P3/1Qp2N2/P4PPP/RNB2RK1 b kq - 1 8' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'd8f6' => 'r1b1k1nr/pppp1ppp/2n2q2/b7/2B1P3/1Qp2N2/P4PPP/RNB2RK1 w kq - 2 9',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b7/2B1P3/2p2N2/P4PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 exd4 7. O-O dxc3

			variation => N__('Evans Gambit: Compromised Defence'),
			moves => {
				'd1b3' => 'r1bqk1nr/pppp1ppp/2n5/b7/2B1P3/1Qp2N2/P4PPP/RNB2RK1 b kq - 1 8',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b7/2BpP3/2P2N2/P4PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: 5... Ba5 6. d4 exd4'),
			moves => {
				'd4c3' => 'r1bqk1nr/pppp1ppp/2n5/b7/2B1P3/2p2N2/P4PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk1nr/pppp1ppp/2n5/b7/2BpP3/2P2N2/P4PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. d4 exd4

			variation => N__('Evans Gambit: 5... Ba5 6. d4 exd4'),
			moves => {
				'e1g1' => 'r1bqk1nr/pppp1ppp/2n5/b7/2BpP3/2P2N2/P4PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk1nr/pppp1ppp/2nb4/4p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bd6

			variation => N__('Evans Gambit: Stone-Ware Defence'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/8/1B2p3/3bP3/2P5/PP1P1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C64s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4 Bxd4 6. c3

			variation => N__('Spanish: Classical, 4.O-O Nd4 5.Nxd4 Bxd4 6. c3'),
			moves => {
				'd4b6' => 'r1bqk1nr/pppp1ppp/1b6/1B2p3/4P3/2P5/PP1P1PPP/RNBQ1RK1 w kq - 1 7',
			},
		},
		'r1bqk1nr/pppp1ppp/8/1B2p3/3bP3/8/PPPP1PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C64s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4 Bxd4

			variation => N__('Spanish: Classical, 4.O-O Nd4 5.Nxd4 Bxd4'),
			moves => {
				'c2c3' => 'r1bqk1nr/pppp1ppp/8/1B2p3/3bP3/2P5/PP1P1PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/8/1Bb1p3/1P1nP3/5N2/P1PP1PPP/RNBQ1RK1 b kq b3 0 5' => {
			eco => 'C64q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. b4

			variation => N__('Spanish: Classical, Zaitsev Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppp1ppp/8/1Bb1p3/3NP3/8/PPPP1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'C64s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Nxd4

			variation => N__('Spanish: Classical, 4.O-O Nd4 5.Nxd4'),
			moves => {
				'c5d4' => 'r1bqk1nr/pppp1ppp/8/1B2p3/3bP3/8/PPPP1PPP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/8/1Bb1p3/3nP3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5' => {
			eco => 'C64p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4

			variation => N__('Spanish: Classical, 4.O-O Nd4'),
			moves => {
				'b2b4' => 'r1bqk1nr/pppp1ppp/8/1Bb1p3/1P1nP3/5N2/P1PP1PPP/RNBQ1RK1 b kq b3 0 5',
				'b5c4' => 'r1bqk1nr/pppp1ppp/8/2b1p3/2BnP3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 7 5',
				'f3d4' => 'r1bqk1nr/pppp1ppp/8/1Bb1p3/3NP3/8/PPPP1PPP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqk1nr/pppp1ppp/8/1Bb5/3pP3/3P4/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C61j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O Bc5 6. d3

			variation => N__('Spanish: Bird\'s, 5.O-O Bc5 6. d3'),
			moves => {
				'c7c6' => 'r1bqk1nr/pp1p1ppp/2p5/1Bb5/3pP3/3P4/PPP2PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqk1nr/pppp1ppp/8/1Bb5/3pP3/8/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C61i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O Bc5

			variation => N__('Spanish: Bird\'s, 5.O-O Bc5'),
			moves => {
				'd2d3' => 'r1bqk1nr/pppp1ppp/8/1Bb5/3pP3/3P4/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk1nr/pppp1ppp/8/2b1p3/2BnP3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 7 5' => {
			eco => 'C64r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nd4 5. Bc4

			variation => N__('Spanish: Classical, 4.O-O Nd4 5. Bc4'),
			moves => {
			},
		},
		'r1bqk1nr/pppp2pp/2n5/1Bb1pp2/3PP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C64g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 f5 5. d4

			variation => N__('Spanish: Classical, Cordel Gambit, 5. d4'),
			moves => {
			},
		},
		'r1bqk1nr/pppp2pp/2n5/1Bb1pp2/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq f6 0 5' => {
			eco => 'C64f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 f5

			variation => N__('Spanish: Classical, Cordel Gambit'),
			moves => {
				'd2d4' => 'r1bqk1nr/pppp2pp/2n5/1Bb1pp2/3PP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5',
			},
		},
		'r1bqk1nr/pppp2pp/2n5/2b1p1N1/2B1Pp2/3P4/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C50n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 f5 5. Ng5 f4

			variation => N__('Giuoco Pianissimo: Dubois Variation'),
			moves => {
			},
		},
		'r1bqk1nr/pppp2pp/2n5/2b1pp2/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq f6 0 5' => {
			eco => 'C50n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 f5

			variation => N__('Giuoco Pianissimo: Lucchini Gambit'),
			moves => {
				'f3g5' => 'r1bqk1nr/pppp2pp/2n5/2b1ppN1/2B1P3/3P4/PPP2PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'r1bqk1nr/pppp2pp/2n5/2b1ppN1/2B1P3/3P4/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C50n',
                        # TRANSLATORS:

			variation => N__('Giuoco Pianissimo: Lucchini Gambit'),
			moves => {
				'f5f4' => 'r1bqk1nr/pppp2pp/2n5/2b1p1N1/2B1Pp2/3P4/PPP2PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C60e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Be7

			variation => N__('Spanish: Lucena Defence'),
			moves => {
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/1PB1P3/5N2/P1PP1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Be7

			variation => N__('Evans Gambit Declined: 4... Be7'),
			moves => {
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/2B1P3/1QP2N2/P2P1PPP/RNB1K2R b KQkq - 2 6' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Be7 6. Qb3

			variation => N__('Evans Gambit: Anderssen, 6. Qb3'),
			moves => {
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Be7

			variation => N__('Evans Gambit: Anderssen Variation'),
			moves => {
				'd1b3' => 'r1bqk1nr/ppppbppp/2n5/4p3/2B1P3/1QP2N2/P2P1PPP/RNB1K2R b KQkq - 2 6',
				'd2d4' => 'r1bqk1nr/ppppbppp/2n5/4p3/2BPP3/2P2N2/P4PPP/RNBQK2R b KQkq d3 0 6',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C50e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Be7

			variation => N__('Italian: Hungarian Defence'),
			moves => {
				'd2d4' => 'r1bqk1nr/ppppbppp/2n5/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/2BPP3/2P2N2/P4PPP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Be7 6. d4

			variation => N__('Evans Gambit: Anderssen, 6. d4'),
			moves => {
				'c6a5' => 'r1bqk1nr/ppppbppp/8/n3p3/2BPP3/2P2N2/P4PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C50e',
                        # TRANSLATORS:

			variation => N__('Italian: Hungarian Defence'),
			moves => {
				'd7d6' => 'r1bqk1nr/ppp1bppp/2np4/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
				'e5d4' => 'r1bqk1nr/ppppbppp/2n5/8/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C44i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Be7

			variation => N__('Ponziani: Romanishin Variation'),
			moves => {
			},
		},
		'r1bqk1nr/ppppbppp/2n5/8/2BpP3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C50f',
                        # TRANSLATORS:

			variation => N__('Italian: Hungarian Defence, 4. d4 exd4'),
			moves => {
				'g8f6' => 'r1bqk2r/ppppbppp/2n2n2/8/2BpP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/8/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C50f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Be7 4. d4 exd4

			variation => N__('Italian: Hungarian Defence, 4. d4 exd4'),
			moves => {
				'c2c3' => 'r1bqk1nr/ppppbppp/2n5/8/2BpP3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqk1nr/ppppbppp/2n5/8/4P3/2P1Q3/PP3PPP/RNB1KBNR w KQkq - 1 6' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4 Nc6 4. Qe3 Bb4+ 5. c3 Be7

			variation => N__('Centre Game: Charousek Variation'),
			moves => {
			},
		},
		'r1bqk1nr/ppppbppp/8/n3p3/2BPP3/2P2N2/P4PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Be7 6. d4 Na5

			variation => N__('Evans Gambit: Cordel Variation'),
			moves => {
			},
		},
		'r1bqk1r1/ppp2BBp/1b6/nP2p3/3PP3/8/P1P2PPP/RN1QK2R b KQq - 0 10' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'e8f7' => 'r1bq2r1/ppp2kBp/1b6/nP2p3/3PP3/8/P1P2PPP/RN1QK2R w KQ - 0 11',
			},
		},
		'r1bqk1r1/ppp2pBp/1b6/nP2p3/2BPP3/8/P1P2PPP/RN1QK2R w KQq - 1 10' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'c4f7' => 'r1bqk1r1/ppp2BBp/1b6/nP2p3/3PP3/8/P1P2PPP/RN1QK2R b KQq - 0 10',
			},
		},
		'r1bqk2r/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w kq - 0 8' => {
			eco => 'A55j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O a6

			variation => N__('Old Indian: 5. e4 Be7 6. Be2 c6 7.O-O a6'),
			moves => {
			},
		},
		'r1bqk2r/1p1nbppp/p2p1n2/4p3/4P3/2N3P1/PPP1NPBP/R1BQK2R w KQkq - 4 9' => {
			eco => 'B91m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nde2 Be7 8. Bg2 Nbd7

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2 Be7 8. Bg2 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/1p1nbppp/p2p1n2/4p3/P1B1PP2/2N2N2/1PP3PP/R1BQK2R b KQkq - 2 9' => {
			eco => 'B93r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bc4

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bc4'),
			moves => {
			},
		},
		'r1bqk2r/1p1nbppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'B93r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7'),
			moves => {
				'f1c4' => 'r1bqk2r/1p1nbppp/p2p1n2/4p3/P1B1PP2/2N2N2/1PP3PP/R1BQK2R b KQkq - 2 9',
				'f1d3' => 'r1bqk2r/1p1nbppp/p2p1n2/4p3/P3PP2/2NB1N2/1PP3PP/R1BQK2R b KQkq - 2 9',
			},
		},
		'r1bqk2r/1p1nbppp/p2p1n2/4p3/P3PP2/2NB1N2/1PP3PP/R1BQK2R b KQkq - 2 9' => {
			eco => 'B93s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4 Be7 9. Bd3'),
			moves => {
				'e8g8' => 'r1bq1rk1/1p1nbppp/p2p1n2/4p3/P3PP2/2NB1N2/1PP3PP/R1BQK2R w KQ - 3 10',
			},
		},
		'r1bqk2r/1p1nbppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/2KR1B1R w kq - 4 9' => {
			eco => 'B95m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qf3 Be7 8. O-O-O Nbd7

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qf3 Be7 8.O-O-O Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/1p1p1ppp/p1n1pn2/8/1bPNP3/2N5/PPB2PPP/R1BQK2R b KQkq - 6 8' => {
			eco => 'B41k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6 6. Nc3 Bb4 7. Bd3 Nc6 8. Bc2

			variation => N__('Sicilian: Kan, 5. c4 Nf6, Bronstein Variation'),
			moves => {
			},
		},
		'r1bqk2r/1p1p1ppp/p1n1pn2/8/1bPNP3/2NB4/PP3PPP/R1BQK2R w KQkq - 5 8' => {
			eco => 'B41k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3 Bb4'),
			moves => {
				'd3c2' => 'r1bqk2r/1p1p1ppp/p1n1pn2/8/1bPNP3/2N5/PPB2PPP/R1BQK2R b KQkq - 6 8',
			},
		},
		'r1bqk2r/1p2bpp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R w kq - 2 10' => {
			eco => 'B66m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Be7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Be7'),
			moves => {
				'f2f3' => 'r1bqk2r/1p2bpp1/p1nppn1p/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b kq - 0 10',
				'f2f4' => 'r1bqk2r/1p2bpp1/p1nppn1p/8/3NPP2/2N1B3/PPPQ2PP/2KR1B1R b kq f3 0 10',
			},
		},
		'r1bqk2r/1p2bpp1/p1nppn1p/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b kq - 0 10' => {
			eco => 'B66m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Be7 10. f3

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Be7 10. f3'),
			moves => {
				'c6d4' => 'r1bqk2r/1p2bpp1/p2ppn1p/8/3nP3/2N1BP2/PPPQ2PP/2KR1B1R w kq - 0 11',
			},
		},
		'r1bqk2r/1p2bpp1/p1nppn1p/8/3NPP2/2N1B3/PPPQ2PP/2KR1B1R b kq f3 0 10' => {
			eco => 'B66o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Be7 10. f4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Be7 10. f4'),
			moves => {
				'c6d4' => 'r1bqk2r/1p2bpp1/p2ppn1p/8/3nPP2/2N1B3/PPPQ2PP/2KR1B1R w kq - 0 11',
			},
		},
		'r1bqk2r/1p2bpp1/p2ppn1p/8/3BP3/2N2P2/PPPQ2PP/2KR1B1R b kq - 0 11' => {
			eco => 'B66n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Be7 10. f3 Nxd4 11. Bxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Be7 10. f3 Nxd4 11.Bxd4'),
			moves => {
			},
		},
		'r1bqk2r/1p2bpp1/p2ppn1p/8/3nP3/2N1BP2/PPPQ2PP/2KR1B1R w kq - 0 11' => {
			eco => 'B66m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Be7 10. f3 Nxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Be7 10. f3 Nxd4'),
			moves => {
				'e3d4' => 'r1bqk2r/1p2bpp1/p2ppn1p/8/3BP3/2N2P2/PPPQ2PP/2KR1B1R b kq - 0 11',
			},
		},
		'r1bqk2r/1p2bpp1/p2ppn1p/8/3nPP2/2N1B3/PPPQ2PP/2KR1B1R w kq - 0 11' => {
			eco => 'B66p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Be7 10. f4 Nxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Be7 10. f4 Nxd4'),
			moves => {
			},
		},
		'r1bqk2r/1p2bppp/p1n1pn2/2p5/P1BP4/2N1PN2/1P2QPPP/R1B2RK1 b kq - 4 9' => {
			eco => 'D27s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Nc3 Be7 9. Qe2

			variation => N__('QGA: Classical, Rubinstein, 8. Nc3 Be7 9. Qe2'),
			moves => {
			},
		},
		'r1bqk2r/1p2bppp/p1n1pn2/2p5/P1BP4/2N1PN2/1P3PPP/R1BQ1RK1 w kq - 3 9' => {
			eco => 'D27r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Nc3 Be7

			variation => N__('QGA: Classical, Rubinstein, 8. Nc3 Be7'),
			moves => {
				'd1e2' => 'r1bqk2r/1p2bppp/p1n1pn2/2p5/P1BP4/2N1PN2/1P2QPPP/R1B2RK1 b kq - 4 9',
			},
		},
		'r1bqk2r/1p2bppp/p1n1pn2/8/P1BP4/5N2/1P2QPPP/RNBR2K1 b kq - 0 10' => {
			eco => 'D27v',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4'),
			moves => {
				'e8g8' => 'r1bq1rk1/1p2bppp/p1n1pn2/8/P1BP4/5N2/1P2QPPP/RNBR2K1 w - - 1 11',
			},
		},
		'r1bqk2r/1p2bppp/p1n1pn2/8/P1Bp4/4PN2/1P2QPPP/RNBR2K1 w kq - 2 10' => {
			eco => 'D27v',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4'),
			moves => {
				'e3d4' => 'r1bqk2r/1p2bppp/p1n1pn2/8/P1BP4/5N2/1P2QPPP/RNBR2K1 b kq - 0 10',
			},
		},
		'r1bqk2r/1p2bppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 2 9' => {
			eco => 'B66d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Be7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O Be7'),
			moves => {
			},
		},
		'r1bqk2r/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP2BPPP/R1BQK2R b KQkq - 3 9' => {
			eco => 'B44u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3 Be7 9. Be2'),
			moves => {
				'b7b6' => 'r1bqk2r/4bppp/ppnppn2/8/2P1P3/N1N5/PP2BPPP/R1BQK2R w KQkq - 0 10',
				'e8g8' => 'r1bq1rk1/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP2BPPP/R1BQK2R w KQ - 4 10',
			},
		},
		'r1bqk2r/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'B44t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3 Be7'),
			moves => {
				'f1e2' => 'r1bqk2r/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP2BPPP/R1BQK2R b KQkq - 3 9',
			},
		},
		'r1bqk2r/1p2bppp/p1nppn2/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 3 9' => {
			eco => 'B80i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 Nc6 8. Qd2 Be7

			variation => N__('Sicilian: Scheveningen, English, 7... Nc6 8. Qd2'),
			moves => {
			},
		},
		'r1bqk2r/1p2bppp/p2ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R w kq - 1 10' => {
			eco => 'B66e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Nxd4 9. Qxd4 Be7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O Nxd4 9.Qxd4 Be7'),
			moves => {
			},
		},
		'r1bqk2r/1p3ppp/p1n1pn2/2b5/2B5/4PN2/PP2QPPP/RNB2RK1 w kq - 2 9' => {
			eco => 'D26n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Qe2 a6 7. dxc5 Bxc5 8. O-O Nc6

			variation => N__('QGA: Classical, Furman, 6... a6 7.dxc5 Bxc5 8.O-O Nc6'),
			moves => {
			},
		},
		'r1bqk2r/1p3ppp/p1n1pn2/2bp4/1PP5/P1N1PN2/5PPP/R1BQKB1R b KQkq b3 0 8' => {
			eco => 'D40q',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 a6 7.dxc5 Bxc5'),
			moves => {
				'c5a7' => 'r1bqk2r/bp3ppp/p1n1pn2/3p4/1PP5/P1N1PN2/5PPP/R1BQKB1R w KQkq - 1 9',
			},
		},
		'r1bqk2r/1p3ppp/p1n1pn2/2bp4/2P5/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D40q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 a6 7. dxc5 Bxc5

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 a6 7.dxc5 Bxc5'),
			moves => {
				'b2b4' => 'r1bqk2r/1p3ppp/p1n1pn2/2bp4/1PP5/P1N1PN2/5PPP/R1BQKB1R b KQkq b3 0 8',
			},
		},
		'r1bqk2r/1pp1bppp/p1Bp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 0 7' => {
			eco => 'C87c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. Bxc6+

			variation => N__('Spanish: Closed, Averbakh, 7.Bxc6+'),
			moves => {
				'b7c6' => 'r1bqk2r/2p1bppp/p1pp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 8',
			},
		},
		'r1bqk2r/1pp1bppp/p1np1n2/4p3/B3P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 7' => {
			eco => 'C87e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3

			variation => N__('Spanish: Closed, Averbakh, 7. c3'),
			moves => {
				'c8g4' => 'r2qk2r/1pp1bppp/p1np1n2/4p3/B3P1b1/2P2N2/PP1P1PPP/RNBQR1K1 w kq - 1 8',
				'e8g8' => 'r1bq1rk1/1pp1bppp/p1np1n2/4p3/B3P3/2P2N2/PP1P1PPP/RNBQR1K1 w - - 1 8',
			},
		},
		'r1bqk2r/1pp1bppp/p1np1n2/4p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 7' => {
			eco => 'C87b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6

			variation => N__('Spanish: Closed, Averbakh (Russian) Variation'),
			moves => {
				'a4c6' => 'r1bqk2r/1pp1bppp/p1Bp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 0 7',
				'c2c3' => 'r1bqk2r/1pp1bppp/p1np1n2/4p3/B3P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 7',
			},
		},
		'r1bqk2r/1pp1bppp/p1np1n2/4p3/B3P3/5N2/PPPPQPPP/RNB2RK1 w kq - 0 7' => {
			eco => 'C86a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 d6

			variation => N__('Spanish: Worrall Attack, 6... d6'),
			moves => {
			},
		},
		'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'C85d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Nc3

			variation => N__('Spanish: Closed, Exchange, 7. Nc3'),
			moves => {
				'c8g4' => 'r2qk2r/1pp1bppp/p1p2n2/4p3/4P1b1/2N2N2/PPPP1PPP/R1BQ1RK1 w kq - 2 8',
			},
		},
		'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C85j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3

			variation => N__('Spanish: Closed, Exchange, 7. d3'),
			moves => {
				'c8g4' => 'r2qk2r/1pp1bppp/p1p2n2/4p3/4P1b1/3P1N2/PPP2PPP/RNBQ1RK1 w kq - 1 8',
				'f6d7' => 'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/3P1N2/PPP2PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/5N2/PPPP1PPP/RNB1QRK1 b kq - 1 7' => {
			eco => 'C85f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Qe1

			variation => N__('Spanish: Closed, Exchange, 7. Qe1'),
			moves => {
				'c6c5' => 'r1bqk2r/1pp1bppp/p4n2/2p1p3/4P3/5N2/PPPP1PPP/RNB1QRK1 w kq - 0 8',
				'f6d7' => 'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNB1QRK1 w kq - 2 8',
			},
		},
		'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C85b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6

			variation => N__('Spanish: Closed, Exchange'),
			moves => {
				'b1c3' => 'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 1 7',
				'd1e1' => 'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/5N2/PPPP1PPP/RNB1QRK1 b kq - 1 7',
				'd1e2' => 'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/5N2/PPPPQPPP/RNB2RK1 b kq - 1 7',
				'd2d3' => 'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/5N2/PPPPQPPP/RNB2RK1 b kq - 1 7' => {
			eco => 'C85c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Qe2

			variation => N__('Spanish: Closed, Exchange, 7. Qe2'),
			moves => {
			},
		},
		'r1bqk2r/1pp1bppp/p4n2/2p1p3/4P3/5N2/PPPP1PPP/RNB1QRK1 w kq - 0 8' => {
			eco => 'C85g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Qe1 c5

			variation => N__('Spanish: Closed, Exchange, 7. Qe1 c5'),
			moves => {
			},
		},
		'r1bqk2r/1pp2ppp/p1np1n2/2b1p3/2B1PP2/2NP1N2/PPP3PP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C28l',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bc5 5. f4 d6 6. Nf3 a6

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5 5. f4 d6 6. Nf3 a6'),
			moves => {
			},
		},
		'r1bqk2r/1pp2ppp/p1np1n2/2b1p3/4P3/2NP1NP1/PPP2PBP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C47h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. g3 Bc5 5. Bg2 d6 6. d3 a6

			variation => N__('Four Knights: Glek, Main Line'),
			moves => {
			},
		},
		'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/1P3N2/P1PP1PPP/RNB1QRK1 b kq - 0 8' => {
			eco => 'C85i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Qe1 Nd7 8. b3

			variation => N__('Spanish: Closed, Exchange, 7. Qe1 Nd7 8. b3'),
			moves => {
			},
		},
		'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/3P1N2/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C85l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7'),
			moves => {
				'b1d2' => 'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/3P1N2/PPPN1PPP/R1BQ1RK1 b kq - 2 8',
			},
		},
		'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/3P1N2/PPPN1PPP/R1BQ1RK1 b kq - 2 8' => {
			eco => 'C85m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Nd7 8. Nbd2

			variation => N__('Spanish: Closed, Exchange, 7. d3 Nd7 8.Nbd2'),
			moves => {
				'e8g8' => 'r1bq1rk1/1ppnbppp/p1p5/4p3/4P3/3P1N2/PPPN1PPP/R1BQ1RK1 w - - 3 9',
			},
		},
		'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNB1QRK1 w kq - 2 8' => {
			eco => 'C85h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Qe1 Nd7

			variation => N__('Spanish: Closed, Exchange, 7. Qe1 Nd7'),
			moves => {
				'b2b3' => 'r1bqk2r/1ppnbppp/p1p5/4p3/4P3/1P3N2/P1PP1PPP/RNB1QRK1 b kq - 0 8',
			},
		},
		'r1bqk2r/1ppp1ppp/p1n2n2/2b1N3/B3P3/8/PPPP1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C78c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Bc5 6. Nxe5

			variation => N__('Spanish: Möller Defence, 6.Nxe5'),
			moves => {
			},
		},
		'r1bqk2r/1ppp1ppp/p1n2n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C54e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d3 a6

			variation => N__('Giuoco Pianissimo: 5. d3 a6'),
			moves => {
			},
		},
		'r1bqk2r/1ppp1ppp/p1n2n2/2b1p3/B3P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C78d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Bc5 6. c3

			variation => N__('Spanish: Möller Defence, 6. c3'),
			moves => {
			},
		},
		'r1bqk2r/1ppp1ppp/p1n2n2/2b1p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 6' => {
			eco => 'C78b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Bc5

			variation => N__('Spanish: Möller Defence'),
			moves => {
				'c2c3' => 'r1bqk2r/1ppp1ppp/p1n2n2/2b1p3/B3P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 6',
				'f3e5' => 'r1bqk2r/1ppp1ppp/p1n2n2/2b1N3/B3P3/8/PPPP1PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk2r/1ppp1ppp/pbn2n2/8/P3P3/1NN5/1PP2PPP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'C45q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Bc5 5. Nb3 Bb6 6. a4 a6 7. Nc3 Nf6

			variation => N__('Scotch: Geller Variation'),
			moves => {
			},
		},
		'r1bqk2r/1pppbppp/p1B2n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C85a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6

			variation => N__('Spanish: Closed, Exchange'),
			moves => {
				'd7c6' => 'r1bqk2r/1pp1bppp/p1p2n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4P3/B2p4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C84h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5

			variation => N__('Spanish: Closed, Centre Attack, 7. e5'),
			moves => {
				'f6e4' => 'r1bqk2r/1pppbppp/p1n5/4P3/B2pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B2PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6' => {
			eco => 'C84g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4

			variation => N__('Spanish: Closed, Centre Attack'),
			moves => {
				'e5d4' => 'r1bqk2r/1pppbppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B2PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C77n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d4 Be7

			variation => N__('Spanish: Centre Attack, 5... Be7'),
			moves => {
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 5 6' => {
			eco => 'C84b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Nc3

			variation => N__('Spanish: Closed, 6. Nc3'),
			moves => {
				'b7b5' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/2N2N2/PPPP1PPP/R1BQ1RK1 w kq b6 0 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C84e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d3

			variation => N__('Spanish: Closed, 6. d3'),
			moves => {
				'b7b5' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/3P1N2/PPP2PPP/RNBQ1RK1 w kq b6 0 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 6' => {
			eco => 'C84a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7

			variation => N__('Spanish: Closed System'),
			moves => {
				'a4c6' => 'r1bqk2r/1pppbppp/p1B2n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 0 6',
				'b1c3' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 5 6',
				'd1e2' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPPQPPP/RNB2RK1 b kq - 5 6',
				'd2d3' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
				'd2d4' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B2PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6',
				'f1e1' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 5 6',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 5 6' => {
			eco => 'C87a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1

			variation => N__('Spanish: Closed, 6. Re1'),
			moves => {
				'b7b5' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 w kq b6 0 7',
				'd7d6' => 'r1bqk2r/1pp1bppp/p1np1n2/4p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPPQPPP/RNB1K2R w KQkq - 4 6' => {
			eco => 'C77q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2 Be7

			variation => N__('Spanish: Wormald Attack, 5... Be7'),
			moves => {
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPPQPPP/RNB2RK1 b kq - 5 6' => {
			eco => 'C86a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2

			variation => N__('Spanish: Worrall Attack'),
			moves => {
				'b7b5' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/5N2/PPPPQPPP/RNB2RK1 w kq b6 0 7',
				'd7d6' => 'r1bqk2r/1pp1bppp/p1np1n2/4p3/B3P3/5N2/PPPPQPPP/RNB2RK1 w kq - 0 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C84g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4

			variation => N__('Spanish: Closed, Centre Attack'),
			moves => {
				'e4e5' => 'r1bqk2r/1pppbppp/p1n2n2/4P3/B2p4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
				'f1e1' => 'r1bqk2r/1pppbppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQR1K1 b kq - 1 7',
			},
		},
		'r1bqk2r/1pppbppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQR1K1 b kq - 1 7' => {
			eco => 'C84l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1'),
			moves => {
				'b7b5' => 'r1bqk2r/2ppbppp/p1n2n2/1p6/B2pP3/5N2/PPP2PPP/RNBQR1K1 w kq b6 0 8',
				'e8g8' => 'r1bq1rk1/1pppbppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQR1K1 w - - 2 8',
			},
		},
		'r1bqk2r/1pppbppp/p1n5/4P3/B2Nn3/8/PPP2PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C84i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4 8. Nxd4

			variation => N__('Spanish: Closed, Centre Attack, 7. e5 Ne4 8.Nxd4'),
			moves => {
				'c6d4' => 'r1bqk2r/1pppbppp/p7/4P3/B2nn3/8/PPP2PPP/RNBQ1RK1 w kq - 0 9',
				'e8g8' => 'r1bq1rk1/1pppbppp/p1n5/4P3/B2Nn3/8/PPP2PPP/RNBQ1RK1 w - - 1 9',
			},
		},
		'r1bqk2r/1pppbppp/p1n5/4P3/B2pn3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C84h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4 8. c3

			variation => N__('Spanish: Closed, Centre Attack, Basque Gambit'),
			moves => {
				'd4c3' => 'r1bqk2r/1pppbppp/p1n5/4P3/B3n3/2p2N2/PP3PPP/RNBQ1RK1 w kq - 0 9',
			},
		},
		'r1bqk2r/1pppbppp/p1n5/4P3/B2pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C84h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4

			variation => N__('Spanish: Closed, Centre Attack, 7. e5 Ne4'),
			moves => {
				'b2b4' => 'r1bqk2r/1pppbppp/p1n5/4P3/BP1pn3/5N2/P1P2PPP/RNBQ1RK1 b kq b3 0 8',
				'c2c3' => 'r1bqk2r/1pppbppp/p1n5/4P3/B2pn3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 8',
				'f3d4' => 'r1bqk2r/1pppbppp/p1n5/4P3/B2Nn3/8/PPP2PPP/RNBQ1RK1 b kq - 0 8',
			},
		},
		'r1bqk2r/1pppbppp/p1n5/4P3/B3n3/2p2N2/PP3PPP/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C84h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4 8. c3 dxc3

			variation => N__('Spanish: Closed, Centre Attack, Basque Gambit Accepted'),
			moves => {
			},
		},
		'r1bqk2r/1pppbppp/p1n5/4P3/BP1pn3/5N2/P1P2PPP/RNBQ1RK1 b kq b3 0 8' => {
			eco => 'C84h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4 8. b4

			variation => N__('Spanish: Closed, Centre Attack, 7. e5 Ne4 8. b4'),
			moves => {
			},
		},
		'r1bqk2r/1pppbppp/p1n5/4p3/B2Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'C80h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 Be7

			variation => N__('Spanish: Open, 6. d4 Be7'),
			moves => {
			},
		},
		'r1bqk2r/1pppbppp/p7/4P3/B2nn3/8/PPP2PPP/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C84j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. e5 Ne4 8. Nxd4 Nxd4

			variation => N__('Spanish: Closed, Centre Attack, 7. e5 Ne4 8.Nxd4 Nxd4'),
			moves => {
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p1Np3/4P3/1B3N2/PPPP1PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'C84d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Nc3 b5 7. Bb3 d6 8. Nd5

			variation => N__('Spanish: Closed, 6. Nc3 b5 7. Bb3 d6 8. Nd5'),
			moves => {
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/3PP3/1B3N2/PPP1QPPP/RNB1K2R w KQkq - 0 8' => {
			eco => 'C77u',
                        # TRANSLATORS:

			variation => N__('Spanish: Wormald Attack, 5... b5 6. Bb3 Be7'),
			moves => {
				'c2c3' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/3PP3/1BP2N2/PP2QPPP/RNB1K2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQR1K1 b kq d3 0 8' => {
			eco => 'C88c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. d4

			variation => N__('Spanish: Closed, 7... d6 8. d4'),
			moves => {
				'c6d4' => 'r1bqk2r/2p1bppp/p2p1n2/1p2p3/3nP3/1B3N2/PPP2PPP/RNBQR1K1 w kq - 0 9',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/3PP3/1BP2N2/PP2QPPP/RNB1K2R b KQkq - 0 8' => {
			eco => 'C77u',
                        # TRANSLATORS:

			variation => N__('Spanish: Wormald Attack, 5... b5 6. Bb3 Be7'),
			moves => {
				'c8g4' => 'r2qk2r/2p1bppp/p1np1n2/1p2p3/3PP1b1/1BP2N2/PP2QPPP/RNB1K2R w KQkq - 1 9',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w kq - 0 8' => {
			eco => 'C88c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6

			variation => N__('Spanish: Closed 7... d6'),
			moves => {
				'c2c3' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 b kq - 0 8',
				'd2d4' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQR1K1 b kq d3 0 8',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB2RK1 w kq - 0 8' => {
			eco => 'C86c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 d6

			variation => N__('Spanish: Worrall Attack, 7... d6'),
			moves => {
				'c2c3' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 b kq - 0 8',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1BN2N2/PPPP1PPP/R1BQ1RK1 w kq - 0 8' => {
			eco => 'C84c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Nc3 b5 7. Bb3 d6

			variation => N__('Spanish: Closed, 6. Nc3 b5 7. Bb3 d6'),
			moves => {
				'c3d5' => 'r1bqk2r/2p1bppp/p1np1n2/1p1Np3/4P3/1B3N2/PPPP1PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 b kq - 0 8' => {
			eco => 'C88d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. c3

			variation => N__('Spanish: Closed 7... d6 8. c3'),
			moves => {
				'c6a5' => 'r1bqk2r/2p1bppp/p2p1n2/np2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w kq - 1 9',
				'c8g4' => 'r2qk2r/2p1bppp/p1np1n2/1p2p3/4P1b1/1BP2N2/PP1P1PPP/RNBQR1K1 w kq - 1 9',
			},
		},
		'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1BP2N2/PP1PQPPP/RNB2RK1 b kq - 0 8' => {
			eco => 'C86d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 d6 8. c3

			variation => N__('Spanish: Worrall Attack, 7... d6 8. c3'),
			moves => {
			},
		},
		'r1bqk2r/2p1bppp/p1pp1n2/4p3/3PP3/5N2/PPP2PPP/RNBQR1K1 b kq d3 0 8' => {
			eco => 'C87c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. Bxc6+ bxc6 8. d4

			variation => N__('Spanish: Closed, Averbakh, 7.Bxc6+ bxc6 8. d4'),
			moves => {
				'e5d4' => 'r1bqk2r/2p1bppp/p1pp1n2/8/3pP3/5N2/PPP2PPP/RNBQR1K1 w kq - 0 9',
			},
		},
		'r1bqk2r/2p1bppp/p1pp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 8' => {
			eco => 'C87c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Averbakh, 7.Bxc6+'),
			moves => {
				'd2d4' => 'r1bqk2r/2p1bppp/p1pp1n2/4p3/3PP3/5N2/PPP2PPP/RNBQR1K1 b kq d3 0 8',
			},
		},
		'r1bqk2r/2p1bppp/p1pp1n2/8/3NP3/8/PPP2PPP/RNBQR1K1 b kq - 0 9' => {
			eco => 'C87d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. Bxc6+ bxc6 8. d4 exd4 9. Nxd4

			variation => N__('Spanish: Closed, Averbakh, 7.Bxc6+ bxc6 8. d4 exd4'),
			moves => {
			},
		},
		'r1bqk2r/2p1bppp/p1pp1n2/8/3pP3/5N2/PPP2PPP/RNBQR1K1 w kq - 0 9' => {
			eco => 'C87d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. Bxc6+ bxc6 8. d4 exd4

			variation => N__('Spanish: Closed, Averbakh, 7.Bxc6+ bxc6 8. d4 exd4'),
			moves => {
				'f3d4' => 'r1bqk2r/2p1bppp/p1pp1n2/8/3NP3/8/PPP2PPP/RNBQR1K1 b kq - 0 9',
			},
		},
		'r1bqk2r/2p1bppp/p2p1n2/1p2p3/3NP3/1B6/PPP2PPP/RNBQR1K1 b kq - 0 9' => {
			eco => 'C88c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, 7... d6 8. d4'),
			moves => {
				'e5d4' => 'r1bqk2r/2p1bppp/p2p1n2/1p6/3pP3/1B6/PPP2PPP/RNBQR1K1 w kq - 0 10',
			},
		},
		'r1bqk2r/2p1bppp/p2p1n2/1p2p3/3nP3/1B3N2/PPP2PPP/RNBQR1K1 w kq - 0 9' => {
			eco => 'C88c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, 7... d6 8. d4'),
			moves => {
				'f3d4' => 'r1bqk2r/2p1bppp/p2p1n2/1p2p3/3NP3/1B6/PPP2PPP/RNBQR1K1 b kq - 0 9',
			},
		},
		'r1bqk2r/2p1bppp/p2p1n2/1p6/3QP3/1B6/PPP2PPP/RNB1R1K1 b kq - 0 10' => {
			eco => 'C88c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, 7... d6 8. d4'),
			moves => {
				'c7c5' => 'r1bqk2r/4bppp/p2p1n2/1pp5/3QP3/1B6/PPP2PPP/RNB1R1K1 w kq c6 0 11',
			},
		},
		'r1bqk2r/2p1bppp/p2p1n2/1p6/3pP3/1B6/PPP2PPP/RNBQR1K1 w kq - 0 10' => {
			eco => 'C88c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, 7... d6 8. d4'),
			moves => {
				'd1d4' => 'r1bqk2r/2p1bppp/p2p1n2/1p6/3QP3/1B6/PPP2PPP/RNB1R1K1 b kq - 0 10',
			},
		},
		'r1bqk2r/2p1bppp/p2p1n2/np2p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w kq - 1 9' => {
			eco => 'C88e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. c3 Na5

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'b3c2' => 'r1bqk2r/2p1bppp/p2p1n2/np2p3/4P3/2P2N2/PPBP1PPP/RNBQR1K1 b kq - 2 9',
			},
		},
		'r1bqk2r/2p1bppp/p2p1n2/np2p3/4P3/2P2N2/PPBP1PPP/RNBQR1K1 b kq - 2 9' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'c7c5' => 'r1bqk2r/4bppp/p2p1n2/npp1p3/4P3/2P2N2/PPBP1PPP/RNBQR1K1 w kq c6 0 10',
			},
		},
		'r1bqk2r/2p2ppp/p1np1n2/1pb1p3/3PP3/1BP2N2/PP3PPP/RNBQ1RK1 b kq d3 0 8' => {
			eco => 'C78k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. c3 d6 8. d4

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7. c3 d6 8. d4'),
			moves => {
			},
		},
		'r1bqk2r/2p2ppp/p1np1n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C78j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. c3 d6

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7. c3 d6'),
			moves => {
				'd2d4' => 'r1bqk2r/2p2ppp/p1np1n2/1pb1p3/3PP3/1BP2N2/PP3PPP/RNBQ1RK1 b kq d3 0 8',
			},
		},
		'r1bqk2r/2p2ppp/p4n2/1pbPR1N1/3n4/1B6/PPPP1PPP/RNBQ2K1 b kq - 0 10' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'e8f8' => 'r1bq1k1r/2p2ppp/p4n2/1pbPR1N1/3n4/1B6/PPPP1PPP/RNBQ2K1 w - - 1 11',
			},
		},
		'r1bqk2r/2p2ppp/p4n2/1pbPp1N1/3n4/1B6/PPPP1PPP/RNBQR1K1 w kq - 3 10' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'e1e5' => 'r1bqk2r/2p2ppp/p4n2/1pbPR1N1/3n4/1B6/PPPP1PPP/RNBQ2K1 b kq - 0 10',
			},
		},
		'r1bqk2r/2pp1ppp/p1n2n2/1pb1N3/4P3/1B6/PPPP1PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C78i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. Nxe5

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7.Nxe5'),
			moves => {
				'c6e5' => 'r1bqk2r/2pp1ppp/p4n2/1pb1n3/4P3/1B6/PPPP1PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'C78h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5'),
			moves => {
				'a2a4' => 'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/P3P3/1B3N2/1PPP1PPP/RNBQ1RK1 b kq a3 0 7',
				'c2c3' => 'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 7',
				'f3e5' => 'r1bqk2r/2pp1ppp/p1n2n2/1pb1N3/4P3/1B6/PPPP1PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1B3N2/PPPPQPPP/RNB1K2R w KQkq - 2 7' => {
			eco => 'C77s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2 b5 6. Bb3 Bc5

			variation => N__('Spanish: Wormald Attack, 5... b5 6. Bb3 Bc5'),
			moves => {
				'c2c3' => 'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1BP2N2/PP1PQPPP/RNB1K2R b KQkq - 0 7',
			},
		},
		'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C78j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. c3

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7. c3'),
			moves => {
				'd7d6' => 'r1bqk2r/2p2ppp/p1np1n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1BP2N2/PP1PQPPP/RNB1K2R b KQkq - 0 7' => {
			eco => 'C77t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2 b5 6. Bb3 Bc5 7. c3

			variation => N__('Spanish: Wormald Attack, 5... b5 6. Bb3 Bc5 7. c3'),
			moves => {
			},
		},
		'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/P3P3/1B3N2/1PPP1PPP/RNBQ1RK1 b kq a3 0 7' => {
			eco => 'C78l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. a4

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7. a4'),
			moves => {
				'a8b8' => '1rbqk2r/2pp1ppp/p1n2n2/1pb1p3/P3P3/1B3N2/1PPP1PPP/RNBQ1RK1 w k - 1 8',
			},
		},
		'r1bqk2r/2pp1ppp/p4n2/1pb1n3/3PP3/1B6/PPP2PPP/RNBQ1RK1 b kq d3 0 8' => {
			eco => 'C78i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bc5 7. Nxe5 Nxe5 8. d4

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7.Nxe5'),
			moves => {
			},
		},
		'r1bqk2r/2pp1ppp/p4n2/1pb1n3/4P3/1B6/PPPP1PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C78i',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Bc5 7.Nxe5'),
			moves => {
				'd2d4' => 'r1bqk2r/2pp1ppp/p4n2/1pb1n3/3PP3/1B6/PPP2PPP/RNBQ1RK1 b kq d3 0 8',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2P3/B2p4/5N2/PPP2PPP/RNBQR1K1 b kq - 0 8' => {
			eco => 'C84n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 b5 8. e5

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 b5 8. e5'),
			moves => {
				'c6e5' => 'r1bqk2r/2ppbppp/p4n2/1p2n3/B2p4/5N2/PPP2PPP/RNBQR1K1 w kq - 0 9',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/3PP3/1B3N2/PPP1QPPP/RNB1K2R b KQkq d3 0 7' => {
			eco => 'C77u',
                        # TRANSLATORS:

			variation => N__('Spanish: Wormald Attack, 5... b5 6. Bb3 Be7'),
			moves => {
				'd7d6' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/3PP3/1B3N2/PPP1QPPP/RNB1K2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'C78g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Be7 7. d4

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C84f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d3 b5 7. Bb3

			variation => N__('Spanish: Closed, 6. d3 b5'),
			moves => {
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'C78g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Be7

			variation => N__('Spanish: 5.O-O b5 6. Bb3 Be7'),
			moves => {
				'd2d4' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQ1RK1 b kq d3 0 7',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 b kq - 1 7' => {
			eco => 'C88a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3

			variation => N__('Spanish: Closed, 6... b5 7. Bb3'),
			moves => {
				'c8b7' => 'r2qk2r/1bppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w kq - 2 8',
				'd7d6' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w kq - 0 8',
				'e8g8' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w - - 2 8',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB1K2R w KQkq - 2 7' => {
			eco => 'C77u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2 b5 6. Bb3 Be7

			variation => N__('Spanish: Wormald Attack, 5... b5 6. Bb3 Be7'),
			moves => {
				'd2d4' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/3PP3/1B3N2/PPP1QPPP/RNB1K2R b KQkq d3 0 7',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB2RK1 b kq - 1 7' => {
			eco => 'C86b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3

			variation => N__('Spanish: Worrall Attack, 6... b5 7. Bb3'),
			moves => {
				'd7d6' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB2RK1 w kq - 0 8',
				'e8g8' => 'r1bq1rk1/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB2RK1 w - - 2 8',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1BN2N2/PPPP1PPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'C84c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Nc3 b5 7. Bb3

			variation => N__('Spanish: Closed, 6. Nc3 b5 7. Bb3'),
			moves => {
				'd7d6' => 'r1bqk2r/2p1bppp/p1np1n2/1p2p3/4P3/1BN2N2/PPPP1PPP/R1BQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/2N2N2/PPPP1PPP/R1BQ1RK1 w kq b6 0 7' => {
			eco => 'C84c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Nc3 b5

			variation => N__('Spanish: Closed, 6. Nc3 b5'),
			moves => {
				'a4b3' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1BN2N2/PPPP1PPP/R1BQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/3P1N2/PPP2PPP/RNBQ1RK1 w kq b6 0 7' => {
			eco => 'C84f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d3 b5

			variation => N__('Spanish: Closed, 6. d3 b5'),
			moves => {
				'a4b3' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 w kq b6 0 7' => {
			eco => 'C88a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5

			variation => N__('Spanish: Closed, 6... b5'),
			moves => {
				'a4b3' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 b kq - 1 7',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p2p3/B3P3/5N2/PPPPQPPP/RNB2RK1 w kq b6 0 7' => {
			eco => 'C86b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5

			variation => N__('Spanish: Worrall Attack, 6... b5'),
			moves => {
				'a4b3' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB2RK1 b kq - 1 7',
			},
		},
		'r1bqk2r/2ppbppp/p1n2n2/1p6/B2pP3/5N2/PPP2PPP/RNBQR1K1 w kq b6 0 8' => {
			eco => 'C84m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 b5

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 b5'),
			moves => {
				'e4e5' => 'r1bqk2r/2ppbppp/p1n2n2/1p2P3/B2p4/5N2/PPP2PPP/RNBQR1K1 b kq - 0 8',
			},
		},
		'r1bqk2r/2ppbppp/p4n2/1p2R3/B2p4/5N2/PPP2PPP/RNBQ2K1 b kq - 0 9' => {
			eco => 'C84n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 b5 8. e5 Nxe5 9. Rxe5

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 b5 8. e5 Nxe5 9.Rxe5'),
			moves => {
			},
		},
		'r1bqk2r/2ppbppp/p4n2/1p2n3/B2p4/5N2/PPP2PPP/RNBQR1K1 w kq - 0 9' => {
			eco => 'C84n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 b5 8. e5 Nxe5

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 b5 8. e5 Nxe5'),
			moves => {
				'e1e5' => 'r1bqk2r/2ppbppp/p4n2/1p2R3/B2p4/5N2/PPP2PPP/RNBQ2K1 b kq - 0 9',
			},
		},
		'r1bqk2r/4bppp/p1n1pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 w kq - 4 10' => {
			eco => 'D28n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Nc6 9. Nc3 Be7

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3 Nc6 9. Nc3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/4bppp/p1np1B2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R b KQkq - 0 10' => {
			eco => 'B33m',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7'),
			moves => {
				'e7f6' => 'r1bqk2r/5ppp/p1np1b2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 0 11',
			},
		},
		'r1bqk2r/4bppp/p1np1n2/1p1Np1B1/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 2 10' => {
			eco => 'B33m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5 Be7

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7'),
			moves => {
				'g5f6' => 'r1bqk2r/4bppp/p1np1B2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R b KQkq - 0 10',
			},
		},
		'r1bqk2r/4bppp/p2p1n2/1pp5/3QP3/1B6/PPP2PPP/RNB1R1K1 w kq c6 0 11' => {
			eco => 'C88c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. d4 Nxd4 9. Nxd4 exd4 10. Qxd4 c5

			variation => N__('Spanish: Closed, Noah\'s Ark Trap'),
			moves => {
			},
		},
		'r1bqk2r/4bppp/p2p1n2/npp1p3/3PP3/2P2N2/PPB2PPP/RNBQR1K1 b kq d3 0 10' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'd8c7' => 'r1b1k2r/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N2/PPB2PPP/RNBQR1K1 w kq - 1 11',
			},
		},
		'r1bqk2r/4bppp/p2p1n2/npp1p3/4P3/2P2N2/PPBP1PPP/RNBQR1K1 w kq c6 0 10' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'd2d4' => 'r1bqk2r/4bppp/p2p1n2/npp1p3/3PP3/2P2N2/PPB2PPP/RNBQR1K1 b kq d3 0 10',
			},
		},
		'r1bqk2r/4bppp/ppnppn2/8/2P1P3/N1N5/PP2BPPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'B44u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 b6

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3 Be7 9. Be2 b6'),
			moves => {
			},
		},
		'r1bqk2r/5pbp/p1np1p2/1p1Np3/4P3/N2B4/PPP2PPP/R2QK2R b KQkq - 3 11' => {
			eco => 'B33t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 Bg7 11. Bd3

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9.Bxf6 gxf6 10. Nd5 Bg7 11. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/5pbp/p1np1p2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 2 11' => {
			eco => 'B33s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 Bg7

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9.Bxf6 gxf6 10. Nd5 Bg7'),
			moves => {
				'f1d3' => 'r1bqk2r/5pbp/p1np1p2/1p1Np3/4P3/N2B4/PPP2PPP/R2QK2R b KQkq - 3 11',
			},
		},
		'r1bqk2r/5pbp/p1np4/1p1NpP2/8/N1P5/PP3PPP/R2QKB1R b KQkq - 0 12' => {
			eco => 'B33v',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. c3 Bg7'),
			moves => {
				'c8f5' => 'r2qk2r/5pbp/p1np4/1p1Npb2/8/N1P5/PP3PPP/R2QKB1R w KQkq - 0 13',
			},
		},
		'r1bqk2r/5pbp/p1np4/1p1Npp2/4P3/N1P5/PP3PPP/R2QKB1R w KQkq - 1 12' => {
			eco => 'B33v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. c3 Bg7

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. c3 Bg7'),
			moves => {
				'e4f5' => 'r1bqk2r/5pbp/p1np4/1p1NpP2/8/N1P5/PP3PPP/R2QKB1R b KQkq - 0 12',
			},
		},
		'r1bqk2r/5ppp/4pn2/1p2N3/1b1p4/3B1Q2/PP3PPP/R1B1K2R w KQkq - 2 14' => {
			eco => 'D49g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Nxe5 12. Nxe5 axb5 13. Qf3 Bb4+

			variation => N__('Semi-Slav: Meran, Sozin, Stahlberg Attack, 13... Bb4'),
			moves => {
			},
		},
		'r1bqk2r/5ppp/p1np1b2/1p1Np3/4P3/N1P5/PP3PPP/R2QKB1R b KQkq - 0 11' => {
			eco => 'B33n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5 Be7 10. Bxf6 Bxf6 11. c3

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7, 11. c3'),
			moves => {
				'e8g8' => 'r1bq1rk1/5ppp/p1np1b2/1p1Np3/4P3/N1P5/PP3PPP/R2QKB1R w KQ - 1 12',
				'f6g5' => 'r1bqk2r/5ppp/p1np4/1p1Np1b1/4P3/N1P5/PP3PPP/R2QKB1R w KQkq - 1 12',
			},
		},
		'r1bqk2r/5ppp/p1np1b2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 0 11' => {
			eco => 'B33m',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7'),
			moves => {
				'c2c3' => 'r1bqk2r/5ppp/p1np1b2/1p1Np3/4P3/N1P5/PP3PPP/R2QKB1R b KQkq - 0 11',
			},
		},
		'r1bqk2r/5ppp/p1np4/1p1Np1b1/4P3/N1P5/PP3PPP/R2QKB1R w KQkq - 1 12' => {
			eco => 'B33p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5 Be7 10. Bxf6 Bxf6 11. c3 Bg5

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5 Be7, 11. c3 Bg5'),
			moves => {
			},
		},
		'r1bqk2r/bp3ppp/p1n1pn2/3p4/1PP5/P1N1PN2/5PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'D40r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 a6 7. dxc5 Bxc5 8. b4 Ba7

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 a6 7.dxc5 Bxc5 8. b4 Ba7'),
			moves => {
			},
		},
		'r1bqk2r/p1p2ppp/2p2n2/3P4/1b6/2NB4/PPP2PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'C47t',
                        # TRANSLATORS:

			variation => N__('Four Knights: Scotch, Main Line, 7... d5'),
			moves => {
				'c6d5' => 'r1bqk2r/p1p2ppp/5n2/3p4/1b6/2NB4/PPP2PPP/R1BQK2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/p1p2ppp/2p2n2/3p4/1b2P3/2NB4/PPP2PPP/R1BQK2R w KQkq d6 0 8' => {
			eco => 'C47t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5

			variation => N__('Four Knights: Scotch, Main Line, 7... d5'),
			moves => {
				'e4d5' => 'r1bqk2r/p1p2ppp/2p2n2/3P4/1b6/2NB4/PPP2PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/p1p2ppp/5n2/3p4/1b6/2NB4/PPP2PPP/R1BQ1RK1 b kq - 1 9' => {
			eco => 'C47t',
                        # TRANSLATORS:

			variation => N__('Four Knights: Scotch, Main Line, 7... d5 8.exd5 cxd5'),
			moves => {
				'e8g8' => 'r1bq1rk1/p1p2ppp/5n2/3p4/1b6/2NB4/PPP2PPP/R1BQ1RK1 w - - 2 10',
			},
		},
		'r1bqk2r/p1p2ppp/5n2/3p4/1b6/2NB4/PPP2PPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'C47t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3 d5 8. exd5 cxd5

			variation => N__('Four Knights: Scotch, Main Line, 7... d5 8.exd5 cxd5'),
			moves => {
				'e1g1' => 'r1bqk2r/p1p2ppp/5n2/3p4/1b6/2NB4/PPP2PPP/R1BQ1RK1 b kq - 1 9',
			},
		},
		'r1bqk2r/p1pp1ppp/2p2n2/8/1b2P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'C47s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6

			variation => N__('Four Knights: Scotch, 4...exd4 5.Nxd4 Bb4 6.Nxc6 bxc6'),
			moves => {
				'f1d3' => 'r1bqk2r/p1pp1ppp/2p2n2/8/1b2P3/2NB4/PPP2PPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqk2r/p1pp1ppp/2p2n2/8/1b2P3/2NB4/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'C47s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Nxc6 bxc6 7. Bd3

			variation => N__('Four Knights: Scotch, Main Line'),
			moves => {
				'd7d5' => 'r1bqk2r/p1p2ppp/2p2n2/3p4/1b2P3/2NB4/PPP2PPP/R1BQK2R w KQkq d6 0 8',
			},
		},
		'r1bqk2r/p1ppbppp/2p5/4Pn2/8/5N2/PPP1QPPP/RNB2RK1 w kq - 1 9' => {
			eco => 'C67i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6 7. Bxc6 bxc6 8. dxe5 Nf5

			variation => N__('Spanish: Open Berlin, Cordel Variation'),
			moves => {
			},
		},
		'r1bqk2r/p1ppbppp/2pn4/4P3/8/5N2/PPP1QPPP/RNB2RK1 b kq - 0 8' => {
			eco => 'C67i',
                        # TRANSLATORS:

			variation => N__('Spanish: Open Berlin, 5... Be7 6. Qd2 Nd6'),
			moves => {
				'd6b7' => 'r1bqk2r/pnppbppp/2p5/4P3/8/5N2/PPP1QPPP/RNB2RK1 w kq - 1 9',
				'd6f5' => 'r1bqk2r/p1ppbppp/2p5/4Pn2/8/5N2/PPP1QPPP/RNB2RK1 w kq - 1 9',
			},
		},
		'r1bqk2r/p1ppbppp/2pn4/4p3/3P4/5N2/PPP1QPPP/RNB2RK1 w kq - 0 8' => {
			eco => 'C67i',
                        # TRANSLATORS:

			variation => N__('Spanish: Open Berlin, 5... Be7 6. Qd2 Nd6'),
			moves => {
				'd4e5' => 'r1bqk2r/p1ppbppp/2pn4/4P3/8/5N2/PPP1QPPP/RNB2RK1 b kq - 0 8',
			},
		},
		'r1bqk2r/p2n1ppp/2pbpn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 2 9' => {
			eco => 'D47h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bd6

			variation => N__('Semi-Slav: Meran, 8. Bd3 Bd6'),
			moves => {
			},
		},
		'r1bqk2r/p2nbppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQ1RK1 b kq - 3 10' => {
			eco => 'D47l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 b4 9. Ne4 Be7 10. O-O

			variation => N__('Semi-Slav: Meran, Lundin, 9. Ne4 Be7 10.O-O'),
			moves => {
			},
		},
		'r1bqk2r/p2nbppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQK2R w KQkq - 2 10' => {
			eco => 'D47l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 b4 9. Ne4 Be7

			variation => N__('Semi-Slav: Meran, Lundin, 9. Ne4 Be7'),
			moves => {
				'e1g1' => 'r1bqk2r/p2nbppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQ1RK1 b kq - 3 10',
			},
		},
		'r1bqk2r/p2p1ppp/2p1pn2/8/1bP1P3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B44f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. c4 Nf6 6. Nc3 Bb4 7. Nxc6 bxc6

			variation => N__('Sicilian: Taimanov, 5. c4 Nf6 6. Nc3 Bb4 7.Nxc6 bxc6'),
			moves => {
			},
		},
		'r1bqk2r/p2pppbp/2p2np1/4P3/8/2N1B3/PPP2PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B34r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Nxc6 bxc6 8. e5

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7.Nxc6 bxc6 8. e5'),
			moves => {
				'f6g8' => 'r1bqk1nr/p2pppbp/2p3p1/4P3/8/2N1B3/PPP2PPP/R2QKB1R w KQkq - 1 9',
			},
		},
		'r1bqk2r/p2pppbp/2p2np1/8/4P3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'B34r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Nxc6 bxc6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7.Nxc6 bxc6'),
			moves => {
				'e4e5' => 'r1bqk2r/p2pppbp/2p2np1/4P3/8/2N1B3/PPP2PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/p4pp1/2p2n1p/n1b1N3/4p3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11' => {
			eco => 'C59d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bc5

			variation => N__('Two Knights: Morphy, Steiner Variation'),
			moves => {
			},
		},
		'r1bqk2r/p4pp1/2pb1n1p/n3N3/3Pp3/8/PPP1BPPP/RNBQK2R b KQkq d3 0 11' => {
			eco => 'C59j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. d4

			variation => N__('Two Knights: Main Line, 11. d4'),
			moves => {
				'd8c7' => 'r1b1k2r/p1q2pp1/2pb1n1p/n3N3/3Pp3/8/PPP1BPPP/RNBQK2R w KQkq - 1 12',
				'e4d3' => 'r1bqk2r/p4pp1/2pb1n1p/n3N3/8/3p4/PPP1BPPP/RNBQK2R w KQkq - 0 12',
			},
		},
		'r1bqk2r/p4pp1/2pb1n1p/n3N3/4p3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11' => {
			eco => 'C59g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6

			variation => N__('Two Knights: Main Line 10... Bd6'),
			moves => {
				'd2d4' => 'r1bqk2r/p4pp1/2pb1n1p/n3N3/3Pp3/8/PPP1BPPP/RNBQK2R b KQkq d3 0 11',
				'f2f4' => 'r1bqk2r/p4pp1/2pb1n1p/n3N3/4pP2/8/PPPPB1PP/RNBQK2R b KQkq f3 0 11',
			},
		},
		'r1bqk2r/p4pp1/2pb1n1p/n3N3/4pP2/8/PPPPB1PP/RNBQK2R b KQkq f3 0 11' => {
			eco => 'C59h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. f4

			variation => N__('Two Knights: Main Line, 11. f4'),
			moves => {
				'e4f3' => 'r1bqk2r/p4pp1/2pb1n1p/n3N3/8/5p2/PPPPB1PP/RNBQK2R w KQkq - 0 12',
			},
		},
		'r1bqk2r/p4pp1/2pb1n1p/n3N3/8/3p4/PPP1BPPP/RNBQK2R w KQkq - 0 12' => {
			eco => 'C59k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. d4 exd3

			variation => N__('Two Knights: Main Line, 11. d4 exd3'),
			moves => {
				'e5d3' => 'r1bqk2r/p4pp1/2pb1n1p/n7/8/3N4/PPP1BPPP/RNBQK2R b KQkq - 0 12',
			},
		},
		'r1bqk2r/p4pp1/2pb1n1p/n3N3/8/5p2/PPPPB1PP/RNBQK2R w KQkq - 0 12' => {
			eco => 'C59i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5 Bd6 11. f4 exf3

			variation => N__('Two Knights: Main Line, 11. f4 exf3'),
			moves => {
			},
		},
		'r1bqk2r/p4pp1/2pb1n1p/n7/8/3N4/PPP1BPPP/RNBQK2R b KQkq - 0 12' => {
			eco => 'C59k',
                        # TRANSLATORS:

			variation => N__('Two Knights: Main Line, 11. d4 exd3'),
			moves => {
				'd8c7' => 'r1b1k2r/p1q2pp1/2pb1n1p/n7/8/3N4/PPP1BPPP/RNBQK2R w KQkq - 1 13',
			},
		},
		'r1bqk2r/pnppbppp/2p5/4P3/2P5/5N2/PP2QPPP/RNB2RK1 b kq c3 0 9' => {
			eco => 'C67j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6 7. Bxc6 bxc6 8. dxe5 Nb7 9. c4

			variation => N__('Spanish: Open Berlin, Zukertort Variation'),
			moves => {
			},
		},
		'r1bqk2r/pnppbppp/2p5/4P3/3N4/8/PPP1QPPP/RNB2RK1 b kq - 2 9' => {
			eco => 'C67k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6 7. Bxc6 bxc6 8. dxe5 Nb7 9. Nd4

			variation => N__('Spanish: Open Berlin, Winawer Attack'),
			moves => {
			},
		},
		'r1bqk2r/pnppbppp/2p5/4P3/8/1P3N2/P1P1QPPP/RNB2RK1 b kq - 0 9' => {
			eco => 'C67j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6 7. Bxc6 bxc6 8. dxe5 Nb7 9. b3

			variation => N__('Spanish: Open Berlin, Pillsbury Variation'),
			moves => {
			},
		},
		'r1bqk2r/pnppbppp/2p5/4P3/8/2N2N2/PPP1QPPP/R1B2RK1 b kq - 2 9' => {
			eco => 'C67l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6 7. Bxc6 bxc6 8. dxe5 Nb7 9. Nc3

			variation => N__('Spanish: Open Berlin, Main Line 9. Nc3 (Rio de Janerio)'),
			moves => {
			},
		},
		'r1bqk2r/pnppbppp/2p5/4P3/8/5N2/PPP1QPPP/RNB2RK1 w kq - 1 9' => {
			eco => 'C67j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6 7. Bxc6 bxc6 8. dxe5 Nb7

			variation => N__('Spanish: Open Berlin, 5... Be7 6. Qd2 Nd6 7.Bxc6 bxc6 8.dxe5 Nb7'),
			moves => {
				'b1c3' => 'r1bqk2r/pnppbppp/2p5/4P3/8/2N2N2/PPP1QPPP/R1B2RK1 b kq - 2 9',
				'b2b3' => 'r1bqk2r/pnppbppp/2p5/4P3/8/1P3N2/P1P1QPPP/RNB2RK1 b kq - 0 9',
				'c2c4' => 'r1bqk2r/pnppbppp/2p5/4P3/2P5/5N2/PP2QPPP/RNB2RK1 b kq c3 0 9',
				'f3d4' => 'r1bqk2r/pnppbppp/2p5/4P3/3N4/8/PPP1QPPP/RNB2RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/pp1n1pbp/3p1np1/1BpP4/4PP2/2N5/PP4PP/R1BQK1NR w KQkq - 3 9' => {
			eco => 'A67b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nbd7

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nbd7'),
			moves => {
				'e4e5' => 'r1bqk2r/pp1n1pbp/3p1np1/1BpPP3/5P2/2N5/PP4PP/R1BQK1NR b KQkq - 0 9',
			},
		},
		'r1bqk2r/pp1n1pbp/3p1np1/1BpPP3/5P2/2N5/PP4PP/R1BQK1NR b KQkq - 0 9' => {
			eco => 'A67b',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nbd7'),
			moves => {
				'd6e5' => 'r1bqk2r/pp1n1pbp/5np1/1BpPp3/5P2/2N5/PP4PP/R1BQK1NR w KQkq - 0 10',
			},
		},
		'r1bqk2r/pp1n1pbp/4P1p1/1BpP3n/8/2N5/PP4PP/R1BQK1NR b KQkq - 0 11' => {
			eco => 'A67c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nbd7 9. e5 dxe5 10. fxe5 Nh5 11. e6

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nbd7 9. e5 dxe5 10.fxe5 Nh5 11. e6'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1pbp/5np1/1BpPP3/8/2N5/PP4PP/R1BQK1NR b KQkq - 0 10' => {
			eco => 'A67b',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nbd7'),
			moves => {
				'f6h5' => 'r1bqk2r/pp1n1pbp/6p1/1BpPP2n/8/2N5/PP4PP/R1BQK1NR w KQkq - 1 11',
			},
		},
		'r1bqk2r/pp1n1pbp/5np1/1BpPp3/5P2/2N5/PP4PP/R1BQK1NR w KQkq - 0 10' => {
			eco => 'A67b',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nbd7'),
			moves => {
				'f4e5' => 'r1bqk2r/pp1n1pbp/5np1/1BpPP3/8/2N5/PP4PP/R1BQK1NR b KQkq - 0 10',
			},
		},
		'r1bqk2r/pp1n1pbp/6p1/1BpPP2n/8/2N5/PP4PP/R1BQK1NR w KQkq - 1 11' => {
			eco => 'A67b',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nbd7'),
			moves => {
				'e5e6' => 'r1bqk2r/pp1n1pbp/4P1p1/1BpP3n/8/2N5/PP4PP/R1BQK1NR b KQkq - 0 11',
			},
		},
		'r1bqk2r/pp1n1pp1/2pbp2p/8/3PQ3/3B1N2/PPP2PPP/R1B1K2R b KQkq - 0 10' => {
			eco => 'B17m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6 6. Bd3 e6 7. N1f3 Bd6 8. Qe2 h6 9. Ne4 Nxe4 10. Qxe4

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3, Main Line 10.Qxe4'),
			moves => {
				'd7f6' => 'r1bqk2r/pp3pp1/2pbpn1p/8/3PQ3/3B1N2/PPP2PPP/R1B1K2R w KQkq - 1 11',
				'd8c7' => 'r1b1k2r/ppqn1pp1/2pbp2p/8/3PQ3/3B1N2/PPP2PPP/R1B1K2R w KQkq - 1 11',
			},
		},
		'r1bqk2r/pp1n1pp1/2pbp2p/8/3Pn3/3B1N2/PPP1QPPP/R1B1K2R w KQkq - 0 10' => {
			eco => 'B17l',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6 7.N1f3 Bd6 8. Qe2'),
			moves => {
				'e2e4' => 'r1bqk2r/pp1n1pp1/2pbp2p/8/3PQ3/3B1N2/PPP2PPP/R1B1K2R b KQkq - 0 10',
			},
		},
		'r1bqk2r/pp1n1pp1/2pbpn1p/3p4/2PP2P1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 0 8' => {
			eco => 'D45l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. g4 h6

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. g4 h6'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1pp1/2pbpn1p/6N1/3P4/3B1N2/PPP1QPPP/R1B1K2R w KQkq - 0 9' => {
			eco => 'B17l',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6 7.N1f3 Bd6 8. Qe2'),
			moves => {
				'g5e4' => 'r1bqk2r/pp1n1pp1/2pbpn1p/8/3PN3/3B1N2/PPP1QPPP/R1B1K2R b KQkq - 1 9',
			},
		},
		'r1bqk2r/pp1n1pp1/2pbpn1p/8/3PN3/3B1N2/PPP1QPPP/R1B1K2R b KQkq - 1 9' => {
			eco => 'B17l',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6 7.N1f3 Bd6 8. Qe2'),
			moves => {
				'f6e4' => 'r1bqk2r/pp1n1pp1/2pbp2p/8/3Pn3/3B1N2/PPP1QPPP/R1B1K2R w KQkq - 0 10',
			},
		},
		'r1bqk2r/pp1n1pp1/3p1n1p/2pPp3/2P4B/2P1PP2/P5PP/R2QKBNR w KQkq - 1 10' => {
			eco => 'E31o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 e5 9. f3 Nbd7

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... e5 9. f3 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2n1p3/2bpP3/3N1P2/2N1B3/PPP3PP/R2QKB1R w KQkq - 1 9' => {
			eco => 'C11n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3 Nc6 7. Be3 cxd4 8. Nxd4 Bc5

			variation => N__('French: Steinitz, Boleslavsky, 7...cxd4 8.Nxd4 Bc5'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2n1p3/2bpP3/5P2/P1N5/1PP3PP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'C11g',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6.dxc5 Nc6'),
			moves => {
				'd1g4' => 'r1bqk2r/pp1n1ppp/2n1p3/2bpP3/5PQ1/P1N5/1PP3PP/R1B1KBNR b KQkq - 1 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2n1p3/2bpP3/5PQ1/P1N5/1PP3PP/R1B1KBNR b KQkq - 1 8' => {
			eco => 'C11g',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6.dxc5 Nc6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1n1ppp/2n1p3/2bpP3/5PQ1/P1N5/1PP3PP/R1B1KBNR w KQ - 2 9',
			},
		},
		'r1bqk2r/pp1n1ppp/2p1pn2/3p4/1bPP2P1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 1 8' => {
			eco => 'D45n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. g4 Bb4

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. g4 Bb4'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2p1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7' => {
			eco => 'D46b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bb4 7. O-O

			variation => N__('Semi-Slav: Romih, 7.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1n1ppp/2p1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2p1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'D46b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bb4

			variation => N__('Semi-Slav: Romih Variation'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1n1ppp/2p1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbp3/3p4/2PP2n1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 0 8' => {
			eco => 'D45o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. g4 Nxg4

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. g4 Nxg4'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP2P1/2N1PN2/PPQ2P1P/R1B1KB1R b KQkq g3 0 7' => {
			eco => 'D45l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. g4

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. g4'),
			moves => {
				'd5c4' => 'r1bqk2r/pp1n1ppp/2pbpn2/8/2pP2P1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 0 8',
				'd6b4' => 'r1bqk2r/pp1n1ppp/2p1pn2/3p4/1bPP2P1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 1 8',
				'f6g4' => 'r1bqk2r/pp1n1ppp/2pbp3/3p4/2PP2n1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 0 8',
				'h7h6' => 'r1bqk2r/pp1n1pp1/2pbpn1p/3p4/2PP2P1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q2PPP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'D45p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. b3

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. b3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q2PPP/R1B1KB1R w KQ - 1 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B1K2R b KQkq - 4 7' => {
			eco => 'D45v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. Be2

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Be2'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B1K2R w KQ - 5 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R w KQkq - 3 7' => {
			eco => 'D45k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6

			variation => N__('Semi-Slav: 6. Qc2 Bd6'),
			moves => {
				'b2b3' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q2PPP/R1B1KB1R b KQkq - 0 7',
				'c1d2' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQB1PPP/R3KB1R b KQkq - 4 7',
				'e3e4' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PPP3/2N2N2/PPQ2PPP/R1B1KB1R b KQkq - 0 7',
				'f1e2' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B1K2R b KQkq - 4 7',
				'g2g4' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP2P1/2N1PN2/PPQ2P1P/R1B1KB1R b KQkq g3 0 7',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQB1PPP/R3KB1R b KQkq - 4 7' => {
			eco => 'D45u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. Bd2

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Bd2'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7' => {
			eco => 'D46n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. O-O

			variation => N__('Semi-Slav: Chigorin, 7.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'D46e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6

			variation => N__('Semi-Slav: Chigorin Defence'),
			moves => {
				'd1c2' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PPQ2PPP/R1B1K2R b KQkq - 4 7',
				'e1g1' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7',
				'e3e4' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PPP3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PPQ2PPP/R1B1K2R b KQkq - 4 7' => {
			eco => 'D46f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2

			variation => N__('Semi-Slav: Chigorin, 7. Qc2'),
			moves => {
				'd5c4' => 'r1bqk2r/pp1n1ppp/2pbpn2/8/2pP4/2NBPN2/PPQ2PPP/R1B1K2R w KQkq - 0 8',
				'e8g8' => 'r1bq1rk1/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PPQ2PPP/R1B1K2R w KQ - 5 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PPP3/2N2N2/PPQ2PPP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'D45k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. e4

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. e4'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PPP3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'D46e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. e4

			variation => N__('Semi-Slav: Chigorin, 7. e4'),
			moves => {
				'd5e4' => 'r1bqk2r/pp1n1ppp/2pbpn2/8/2PPp3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/6N1/3P4/3B1N2/PPP1QPPP/R1B1K2R b KQkq - 3 8' => {
			eco => 'B17l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6 6. Bd3 e6 7. N1f3 Bd6 8. Qe2

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6 7.N1f3 Bd6 8. Qe2'),
			moves => {
				'h7h6' => 'r1bqk2r/pp1n1pp1/2pbpn1p/6N1/3P4/3B1N2/PPP1QPPP/R1B1K2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/6N1/3P4/3B1N2/PPP2PPP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B17k',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6'),
			moves => {
				'd1e2' => 'r1bqk2r/pp1n1ppp/2pbpn2/6N1/3P4/3B1N2/PPP1QPPP/R1B1K2R b KQkq - 3 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B1K2R b KQkq - 0 8' => {
			eco => 'D46g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 dxc4 8. Bxc4

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 dxc4'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/8/2PPN3/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'D46e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. e4 dxe4 8. Nxe4

			variation => N__('Semi-Slav: Chigorin, 7. e4 dxe4 8.Nxe4'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/8/2PPp3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'D46e',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Chigorin, 7. e4'),
			moves => {
				'c3e4' => 'r1bqk2r/pp1n1ppp/2pbpn2/8/2PPN3/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/8/2pP2P1/2N1PN2/PPQ2P1P/R1B1KB1R w KQkq - 0 8' => {
			eco => 'D45m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. g4 dxc4

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. g4 dxc4'),
			moves => {
			},
		},
		'r1bqk2r/pp1n1ppp/2pbpn2/8/2pP4/2NBPN2/PPQ2PPP/R1B1K2R w KQkq - 0 8' => {
			eco => 'D46g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 dxc4

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 dxc4'),
			moves => {
				'd3c4' => 'r1bqk2r/pp1n1ppp/2pbpn2/8/2BP4/2N1PN2/PPQ2PPP/R1B1K2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1nbp1p/2n1p3/2ppP1p1/3P4/2PB1N2/PP1N1PPP/R1BQ1RK1 w kq g6 0 9' => {
			eco => 'C05t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ngf3 Be7 8. O-O g5

			variation => N__('French: Tarrasch, Closed, 7.Ngf3 Be7 8.O-O g5'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2n1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQ1RK1 b kq - 4 8' => {
			eco => 'C05s',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 7.Ngf3 Be7'),
			moves => {
				'g7g5' => 'r1bqk2r/pp1nbp1p/2n1p3/2ppP1p1/3P4/2PB1N2/PP1N1PPP/R1BQ1RK1 w kq g6 0 9',
			},
		},
		'r1bqk2r/pp1nbppp/2n1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQK2R w KQkq - 3 8' => {
			eco => 'C05s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ngf3 Be7

			variation => N__('French: Tarrasch, Closed, 7.Ngf3 Be7'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1nbppp/2n1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r1bqk2r/pp1nbppp/2npp3/6P1/3NP3/2N1B3/PPP2P1P/R2QKB1R w KQkq - 3 9' => {
			eco => 'B81t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 Nc6 7. g5 Nd7 8. Be3 Be7

			variation => N__('Sicilian: Scheveningen, Keres, 6... Nc6 7. g5 Nd7 8. Be3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 7' => {
			eco => 'D43i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 Be7 6. e3 Nbd7

			variation => N__('Semi-Slav: Botvinnik, 5... Be7 6. e3 Nbd7'),
			moves => {
				'f1d3' => 'r1bqk2r/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R b KQkq - 2 7',
			},
		},
		'r1bqk2r/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/R2QK2R b KQkq - 2 7' => {
			eco => 'D43j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 Be7 6. e3 Nbd7 7. Bd3

			variation => N__('Semi-Slav: Botvinnik, 5... Be7 6. e3 Nbd7 7. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p1pn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R w KQkq - 3 7' => {
			eco => 'D45j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Be7

			variation => N__('Semi-Slav: 6. Qc2 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7' => {
			eco => 'D46c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Be7 7. O-O

			variation => N__('Semi-Slav: Bogoljubow, 7.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1nbppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8',
			},
		},
		'r1bqk2r/pp1nbppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'D46c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Be7

			variation => N__('Semi-Slav: Bogoljubow Variation'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1nbppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7',
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR w KQkq - 1 8' => {
			eco => 'D36h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 Nbd7'),
			moves => {
				'f1d3' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR b KQkq - 2 8',
				'g1f3' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQkq - 2 8',
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQkq - 2 8' => {
			eco => 'D36h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Nf3

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 Nbd7 8. Nf3'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R w KQkq - 4 8' => {
			eco => 'D36c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. Nf3 Nbd7

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. Nf3 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PP2NPPP/R2QK2R b KQkq - 4 8' => {
			eco => 'D35t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 Nbd7 8. Nge2

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 Nbd7 8.Nge2'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PP3PPP/R2QK1NR w KQkq - 3 8' => {
			eco => 'D35t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 Nbd7

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 Nbd7'),
			moves => {
				'g1e2' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PP2NPPP/R2QK2R b KQkq - 4 8',
				'g1f3' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PP3PPP/R2QK2R b KQkq - 4 8',
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR b KQkq - 2 8' => {
			eco => 'D36i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 Nbd7 8. Bd3'),
			moves => {
				'd7f8' => 'r1bqkn1r/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR w KQkq - 3 9',
				'e8g8' => 'r1bq1rk1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR w KQ - 3 9',
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PP3PPP/R2QK2R b KQkq - 4 8' => {
			eco => 'D35u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 Nbd7 8. Nf3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 Nbd7 8. Nf3'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2p2n2/3p4/3P1B2/2N2N2/PPQ1PPPP/R3KB1R w KQkq - 2 8' => {
			eco => 'D35h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Be7 6. Bf4 c6 7. Qc2 Nbd7

			variation => N__('QGD: Exchange, 5. Nf3 Be7 6. Bf4 c6 7. Qc2 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2pp1n2/4p1B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 Be7 6. e3 c6

			variation => N__('Old Indian: 5. Bg5 Be7 6. e3 c6'),
			moves => {
			},
		},
		'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'A55j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O

			variation => N__('Old Indian: 5. e4 Be7 6. Be2 c6 7.O-O'),
			moves => {
				'a7a6' => 'r1bqk2r/1p1nbppp/p1pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w kq - 0 8',
				'e8g8' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8',
			},
		},
		'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'A55j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6

			variation => N__('Old Indian: 5. e4 Be7 6. Be2 c6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b kq - 1 7',
			},
		},
		'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6'),
			moves => {
				'f1g2' => 'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQK2R b KQkq - 1 7' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQK2R w KQ - 2 8',
			},
		},
		'r1bqk2r/pp1nbppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQ1RK1 b kq - 2 7' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1nbppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQ1RK1 w - - 3 8',
			},
		},
		'r1bqk2r/pp1nbppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1nbppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQ1RK1 b kq - 2 7',
			},
		},
		'r1bqk2r/pp1nbppp/4pn2/2pp2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R w KQkq - 1 7' => {
			eco => 'D03j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3 c5 5. Nbd2 Be7 6. c3 Nbd7

			variation => N__('Torre Attack: 3... e6 4. e3 c5'),
			moves => {
			},
		},
		'r1bqk2r/pp1nppbp/2np2p1/8/2P1P3/2N5/PPN1BPPP/R1BQK2R w KQkq - 2 9' => {
			eco => 'B37j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 Nd7

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 Nd7'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1pbp/6p1/1N2p3/2PnP3/4B3/PP3PPP/R2QKB1R b KQkq - 1 10' => {
			eco => 'B39f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 e5 10. Nb5

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 e5 10. Nb5'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1pbp/6p1/4p3/2PnP3/2N1B3/PP3PPP/R2QKB1R w KQkq e6 0 10' => {
			eco => 'B39e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 e5

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 e5'),
			moves => {
				'c3b5' => 'r1bqk2r/pp1p1pbp/6p1/1N2p3/2PnP3/4B3/PP3PPP/R2QKB1R b KQkq - 1 10',
				'f1d3' => 'r1bqk2r/pp1p1pbp/6p1/4p3/2PnP3/2NBB3/PP3PPP/R2QK2R b KQkq - 1 10',
			},
		},
		'r1bqk2r/pp1p1pbp/6p1/4p3/2PnP3/2NBB3/PP3PPP/R2QK2R b KQkq - 1 10' => {
			eco => 'B39g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 e5 10. Bd3

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 e5 10. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2N1pn2/8/1bP1P3/2N5/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B44e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. c4 Nf6 6. Nc3 Bb4 7. Nxc6

			variation => N__('Sicilian: Taimanov, 5. c4 Nf6 6. Nc3 Bb4 7.Nxc6'),
			moves => {
				'b7c6' => 'r1bqk2r/p2p1ppp/2p1pn2/8/1bP1P3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1p3/2P5/1bP1n3/1QN2N2/PP1BPPPP/R3KB1R b KQkq - 4 7' => {
			eco => 'E23',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5 Nc6 6. Nf3 Ne4'),
			moves => {
				'e4c5' => 'r1bqk2r/pp1p1ppp/2n1p3/2n5/1bP5/1QN2N2/PP1BPPPP/R3KB1R w KQkq - 0 8',
				'e4d2' => 'r1bqk2r/pp1p1ppp/2n1p3/2P5/1bP5/1QN2N2/PP1nPPPP/R3KB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1p3/2P5/1bP1n3/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 3 7' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6 6. Nf3 Ne4

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5 Nc6 6. Nf3 Ne4'),
			moves => {
				'c1d2' => 'r1bqk2r/pp1p1ppp/2n1p3/2P5/1bP1n3/1QN2N2/PP1BPPPP/R3KB1R b KQkq - 4 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1p3/2P5/1bP5/1QN2N2/PP1nPPPP/R3KB1R w KQkq - 0 8' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6 6. Nf3 Ne4 7. Bd2 Nxd2

			variation => N__('Nimzo-Indian: Spielmann, Carlsbad Variation'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1p3/2n5/1bP5/1QN2N2/PP1BPPPP/R3KB1R w KQkq - 0 8' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6 6. Nf3 Ne4 7. Bd2 Nxc5

			variation => N__('Nimzo-Indian: Spielmann, San Remo Variation'),
			moves => {
				'b3c2' => 'r1bqk2r/pp1p1ppp/2n1p3/2n5/1bP5/2N2N2/PPQBPPPP/R3KB1R b KQkq - 1 8',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1p3/2n5/1bP5/2N2N2/PPQBPPPP/R3KB1R b KQkq - 1 8' => {
			eco => 'E23',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Spielmann, San Remo Variation'),
			moves => {
				'f7f5' => 'r1bqk2r/pp1p2pp/2n1p3/2n2p2/1bP5/2N2N2/PPQBPPPP/R3KB1R w KQkq f6 0 9',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/1N6/1b2P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 5 7' => {
			eco => 'B45p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4'),
			moves => {
				'a2a3' => 'r1bqk2r/pp1p1ppp/2n1pn2/1N6/1b2P3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 7',
				'b5d6' => 'r1bqk2r/pp1p1ppp/2nNpn2/8/1b2P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 6 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/1N6/1b2P3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B45r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. a3

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3'),
			moves => {
				'b4c3' => 'r1bqk2r/pp1p1ppp/2n1pn2/1N6/4P3/P1b5/1PP2PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/1N6/1bP5/2N5/PP2PPPP/R1BQKB1R w KQkq - 4 7' => {
			eco => 'A33h',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. Ndb5 Bb4

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 Bb4'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/1N6/4P3/P1b5/1PP2PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B45r',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3'),
			moves => {
				'b5c3' => 'r1bqk2r/pp1p1ppp/2n1pn2/8/4P3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2P3B1/1bP5/1QN5/PP2PPPP/R3KBNR b KQkq - 2 6' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6 6. Bg5

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5 Nc6 6. Bg5'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2P5/1bP5/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 2 6' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6 6. Nf3

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5 Nc6 6. Nf3'),
			moves => {
				'f6e4' => 'r1bqk2r/pp1p1ppp/2n1p3/2P5/1bP1n3/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 3 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2P5/1bP5/1QN5/PP2PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5 Nc6'),
			moves => {
				'c1g5' => 'r1bqk2r/pp1p1ppp/2n1pn2/2P3B1/1bP5/1QN5/PP2PPPP/R3KBNR b KQkq - 2 6',
				'g1f3' => 'r1bqk2r/pp1p1ppp/2n1pn2/2P5/1bP5/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 2 6',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'E38h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Nc6

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2b5/2PN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'A33n',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3 Bc5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. g3 Bc5'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2b5/2PN4/P1N5/1P2PPPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'A33f',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. a3 Bc5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. a3 Bc5'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 Nc6

			variation => N__('Nimzo-Indian: Romanishin, 4... c5 5. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBP3/PP2NPPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Ne2

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Bd3 Nc6 6. Ne2'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Bd3 Nc6'),
			moves => {
				'g1e2' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBP3/PP2NPPP/R1BQK2R b KQkq - 3 6',
				'g1f3' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Bd3 Nc6 6. Nf3'),
			moves => {
				'b4c3' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/2bBPN2/PP3PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/2PBPN2/P4PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'E41',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Bd3 Nc6 6. Nf3'),
			moves => {
				'd7d6' => 'r1bqk2r/pp3ppp/2nppn2/2p5/2PP4/2PBPN2/P4PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/2bBPN2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'E41',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Bd3 Nc6 6. Nf3'),
			moves => {
				'b2c3' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/2PBPN2/P4PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'E26',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. e3 Nc6

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. e3 Nc6'),
			moves => {
				'f1d3' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR b KQkq - 2 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR b KQkq - 2 7' => {
			eco => 'E26',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. e3 Nc6 7. Bd3

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. e3 Nc6 7. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/8/1b1NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 5 7' => {
			eco => 'B45i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Be3 Bb4

			variation => N__('Sicilian: Taimanov, Four Knights, 6. Be3 Bb4'),
			moves => {
				'f1d3' => 'r1bqk2r/pp1p1ppp/2n1pn2/8/1b1NP3/2NBB3/PPP2PPP/R2QK2R b KQkq - 6 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/8/1b1NP3/2NBB3/PPP2PPP/R2QK2R b KQkq - 6 7' => {
			eco => 'B45j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Be3 Bb4 7. Bd3

			variation => N__('Sicilian: Taimanov, Four Knights, 6. Be3 Bb4 7. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/8/1bPN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'A33m',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3 Bb4

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. g3 Bb4'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/8/1bPN4/2N5/PP1BPPPP/R2QKB1R w KQkq - 4 7' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Bb4 6. Bd2 Nc6

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Bb4 6. Bd2 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/8/1bPNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'B44e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. c4 Nf6 6. Nc3 Bb4

			variation => N__('Sicilian: Taimanov, 5. c4 Nf6 6. Nc3 Bb4'),
			moves => {
				'd4c6' => 'r1bqk2r/pp1p1ppp/2N1pn2/8/1bP1P3/2N5/PP3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1p1ppp/2n1pn2/8/4P3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'B45r',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3'),
			moves => {
				'd7d5' => 'r1bqk2r/pp3ppp/2n1pn2/3p4/4P3/P1N5/1PP2PPP/R1BQKB1R w KQkq d6 0 9',
			},
		},
		'r1bqk2r/pp1p1ppp/2nNpn2/8/1b2P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 6 7' => {
			eco => 'B45q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. Nd6+

			variation => N__('Sicilian: Taimanov, Four Knights, American Attack'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/4pn2/2n5/1PP5/P1Q5/4PPPP/R1B1KBNR b KQkq b3 0 8' => {
			eco => 'E38k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Na6 6. a3 Bxc3+ 7. Qxc3 Nxc5 8. b4

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6: 8. b4'),
			moves => {
				'c5e4' => 'r1bqk2r/pp1p1ppp/4pn2/8/1PP1n3/P1Q5/4PPPP/R1B1KBNR w KQkq - 1 9',
			},
		},
		'r1bqk2r/pp1p1ppp/4pn2/2n5/2P5/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 0 8' => {
			eco => 'E38j',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6 6. a3 Bxc3+ 7.Qxc3'),
			moves => {
				'b2b4' => 'r1bqk2r/pp1p1ppp/4pn2/2n5/1PP5/P1Q5/4PPPP/R1B1KBNR b KQkq b3 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/4pn2/8/1PP1n3/P1Q5/4PPPP/R1B1KBNR w KQkq - 1 9' => {
			eco => 'E38k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Na6 6. a3 Bxc3+ 7. Qxc3 Nxc5 8. b4 Nce4

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6: 8. b4 Nce4'),
			moves => {
				'c3d4' => 'r1bqk2r/pp1p1ppp/4pn2/8/1PPQn3/P7/4PPPP/R1B1KBNR b KQkq - 2 9',
			},
		},
		'r1bqk2r/pp1p1ppp/4pn2/8/1PPQn3/P7/4PPPP/R1B1KBNR b KQkq - 2 9' => {
			eco => 'E38l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Na6 6. a3 Bxc3+ 7. Qxc3 Nxc5 8. b4 Nce4 9. Qd4

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6: 8. b4 Nce4 9. Qd4'),
			moves => {
				'd7d5' => 'r1bqk2r/pp3ppp/4pn2/3p4/1PPQn3/P7/4PPPP/R1B1KBNR w KQkq d6 0 10',
			},
		},
		'r1bqk2r/pp1p1ppp/n3pn2/2P5/1bP5/1QN5/PP2PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Na6

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5 Na6'),
			moves => {
			},
		},
		'r1bqk2r/pp1p1ppp/n3pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'E38i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Na6

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6'),
			moves => {
				'a2a3' => 'r1bqk2r/pp1p1ppp/n3pn2/2P5/1bP5/P1N5/1PQ1PPPP/R1B1KBNR b KQkq - 0 6',
			},
		},
		'r1bqk2r/pp1p1ppp/n3pn2/2P5/1bP5/P1N5/1PQ1PPPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'E38i',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6'),
			moves => {
				'b4c3' => 'r1bqk2r/pp1p1ppp/n3pn2/2P5/2P5/P1b5/1PQ1PPPP/R1B1KBNR w KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1p1ppp/n3pn2/2P5/2P5/P1Q5/1P2PPPP/R1B1KBNR b KQkq - 0 7' => {
			eco => 'E38j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Na6 6. a3 Bxc3+ 7. Qxc3

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6 6. a3 Bxc3+ 7.Qxc3'),
			moves => {
				'a6c5' => 'r1bqk2r/pp1p1ppp/4pn2/2n5/2P5/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1p1ppp/n3pn2/2P5/2P5/P1b5/1PQ1PPPP/R1B1KBNR w KQkq - 0 7' => {
			eco => 'E38i',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6'),
			moves => {
				'c2c3' => 'r1bqk2r/pp1p1ppp/n3pn2/2P5/2P5/P1Q5/1P2PPPP/R1B1KBNR b KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1p2pp/2n1p3/2n2p2/1bP5/2N2N2/PPQBPPPP/R3KB1R w KQkq f6 0 9' => {
			eco => 'E23',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Spielmann, San Remo Variation'),
			moves => {
				'g2g3' => 'r1bqk2r/pp1p2pp/2n1p3/2n2p2/1bP5/2N2NP1/PPQBPP1P/R3KB1R b KQkq - 0 9',
			},
		},
		'r1bqk2r/pp1p2pp/2n1p3/2n2p2/1bP5/2N2NP1/PPQBPP1P/R3KB1R b KQkq - 0 9' => {
			eco => 'E23',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5 Nc6 6. Nf3 Ne4 7. Bd2 Nxc5 8. Qc2 f5 9. g3

			variation => N__('Nimzo-Indian: Spielmann, Stahlberg Variation'),
			moves => {
			},
		},
		'r1bqk2r/pp1pbppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 5 6' => {
			eco => 'A36c',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Keres-Parma System'),
			moves => {
				'd7d5' => 'r1bqk2r/pp2bppp/2n1pn2/2pp4/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq d6 0 7',
			},
		},
		'r1bqk2r/pp1pbppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 4 6' => {
			eco => 'A36c',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Keres-Parma System'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1pbppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 5 6',
			},
		},
		'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'A36p',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Botvinnik, 5... e6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 w - - 4 8',
			},
		},
		'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A36p',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 e6 6. Nge2 Nge7

			variation => N__('English: Symmetrical, Botvinnik, 5... e6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A36m',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 e6 6. Nge2 Nge7

			variation => N__('English: Symmetrical, 5. e3 e6'),
			moves => {
			},
		},
		'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2N1PNP1/PP1P1PBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A37m',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. O-O Nge7 7. e3

			variation => N__('English: Symmetrical, 5. Nf3 e6 6.O-O Nge7 7. e3'),
			moves => {
			},
		},
		'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7' => {
			eco => 'A37l',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. O-O Nge7

			variation => N__('English: Symmetrical, 5. Nf3 e6 6.O-O Nge7'),
			moves => {
				'd2d3' => 'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 7',
				'e2e3' => 'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2N1PNP1/PP1P1PBP/R1BQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/pp1pnpbp/2n1p1p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A37n',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e6 6. O-O Nge7 7. d3

			variation => N__('English: Symmetrical, 5. Nf3 e6 6.O-O Nge7 7. d3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1pnpbp/2n1p1p1/2p5/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 1 8',
			},
		},
		'r1bqk2r/pp1pnpbp/2n3p1/2p1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7' => {
			eco => 'A37x',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O Nge7

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O Nge7'),
			moves => {
				'd2d3' => 'r1bqk2r/pp1pnpbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/pp1pnpbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A37y',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O Nge7 7. d3

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O Nge7 7. d3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1pnpbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 1 8',
			},
		},
		'r1bqk2r/pp1pnpbp/2n3p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R w KQkq - 1 7' => {
			eco => 'A37t',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. d3 Nge7

			variation => N__('English: Symmetrical, 5. Nf3 e5 6. d3 Nge7'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/2N2np1/8/4P3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B34r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Nxc6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7.Nxc6'),
			moves => {
				'b7c6' => 'r1bqk2r/p2pppbp/2p2np1/8/4P3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 6' => {
			eco => 'B31l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 Nf6 6. c3

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 Nf6 6. c3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1pppbp/2n2np1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 w - - 1 7',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 4 6' => {
			eco => 'B31k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O Bg7 5. Re1 Nf6

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O Bg7 5. Re1 Nf6'),
			moves => {
				'c2c3' => 'r1bqk2r/pp1pppbp/2n2np1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 6',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 1 6' => {
			eco => 'A36o',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 Nf6

			variation => N__('English: Symmetrical, Botvinnik, 5... Nf6'),
			moves => {
				'g1e2' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 2 6',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 2 6' => {
			eco => 'A36o',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 Nf6 6. Nge2

			variation => N__('English: Symmetrical, Botvinnik, 5... Nf6 6.Nge2'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 5 6' => {
			eco => 'A38c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O

			variation => N__('English: Symmetrical, Main Line, 6.O-O'),
			moves => {
				'd7d5' => 'r1bqk2r/pp2ppbp/2n2np1/2pp4/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq d6 0 7',
				'd7d6' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7',
				'e8g8' => 'r1bq1rk1/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 6 7',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 4 6' => {
			eco => 'A38a',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6

			variation => N__('English: Symmetrical, Main Line'),
			moves => {
				'd2d4' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq d3 0 6',
				'e1g1' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 5 6',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq d3 0 6' => {
			eco => 'A38b',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. d4

			variation => N__('English: Symmetrical, Main Line, 6. d4'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/4P3/3P1NP1/PPP2PBP/RNBQ1RK1 b kq e3 0 6' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7 4. O-O Nc6 5. d3 Nf6 6. e4

			variation => N__('Reti: 1... c5'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7 4. O-O Nc6 5. d3 Nf6

			variation => N__('Reti: 1... c5'),
			moves => {
				'e2e4' => 'r1bqk2r/pp1pppbp/2n2np1/2p5/4P3/3P1NP1/PPP2PBP/RNBQ1RK1 b kq e3 0 6',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R b KQkq - 5 7' => {
			eco => 'B35a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4'),
			moves => {
				'd8a5' => 'r1b1k2r/pp1pppbp/2n2np1/q7/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 6 8',
				'e8g8' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQ - 6 8',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/2P1P3/2N5/PPN2PPP/R1BQKB1R b KQkq - 4 7' => {
			eco => 'B37g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3'),
			moves => {
				'd7d6' => 'r1bqk2r/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN2PPP/R1BQKB1R w KQkq - 0 8',
				'e8g8' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2P1P3/2N5/PPN2PPP/R1BQKB1R w KQ - 5 8',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/2P1P3/8/PPN2PPP/RNBQKB1R w KQkq - 3 7' => {
			eco => 'B37g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6'),
			moves => {
				'b1c3' => 'r1bqk2r/pp1pppbp/2n2np1/8/2P1P3/2N5/PPN2PPP/R1BQKB1R b KQkq - 4 7',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R b KQkq - 4 7' => {
			eco => 'B38d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 Nf6 7. Nc3'),
			moves => {
				'd7d6' => 'r1bqk2r/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R w KQkq - 0 8',
				'e8g8' => 'r1bq1rk1/pp1pppbp/2n2np1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R w KQ - 5 8',
				'f6g4' => 'r1bqk2r/pp1pppbp/2n3p1/8/2PNP1n1/2N1B3/PP3PPP/R2QKB1R w KQkq - 5 8',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/2PNP3/4B3/PP3PPP/RN1QKB1R w KQkq - 3 7' => {
			eco => 'B38d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 Nf6'),
			moves => {
				'b1c3' => 'r1bqk2r/pp1pppbp/2n2np1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R b KQkq - 4 7',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 5 7' => {
			eco => 'B34t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Be2

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7. Be2'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQ - 6 8',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 4 7' => {
			eco => 'B34p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6'),
			moves => {
				'd4c6' => 'r1bqk2r/pp1pppbp/2N2np1/8/4P3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 0 7',
				'f1c4' => 'r1bqk2r/pp1pppbp/2n2np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R b KQkq - 5 7',
				'f1e2' => 'r1bqk2r/pp1pppbp/2n2np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 5 7',
				'f2f3' => 'r1bqk2r/pp1pppbp/2n2np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1bqk2r/pp1pppbp/2n2np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B34q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. f3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 6. Be3 Nf6 7. f3'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/2n3p1/8/2PNP1Q1/2N1B3/PP3PPP/R3KB1R b KQkq - 0 8' => {
			eco => 'B39b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4'),
			moves => {
				'c6d4' => 'r1bqk2r/pp1pppbp/6p1/8/2PnP1Q1/2N1B3/PP3PPP/R3KB1R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pp1pppbp/2n3p1/8/2PNP1n1/2N1B3/PP3PPP/R2QKB1R w KQkq - 5 8' => {
			eco => 'B39a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4

			variation => N__('Sicilian: Maroczy Bind, Breyer Variation'),
			moves => {
				'd1g4' => 'r1bqk2r/pp1pppbp/2n3p1/8/2PNP1Q1/2N1B3/PP3PPP/R3KB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp1pppbp/2n3pn/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 5 6' => {
			eco => 'A37d',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nh6 6. O-O

			variation => N__('English: Symmetrical, 5. Nf3 Nh6 6.O-O'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/2n3pn/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 4 6' => {
			eco => 'A37c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nh6

			variation => N__('English: Symmetrical, 5. Nf3 Nh6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp1pppbp/2n3pn/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 5 6',
			},
		},
		'r1bqk2r/pp1pppbp/2n3pn/8/2PNP3/4B3/PP3PPP/RN1QKB1R w KQkq - 3 7' => {
			eco => 'B38b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nh6

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 Nh6'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/4n1p1/8/2P1P3/2N1B3/PP1Q1PPP/R3KB1R b KQkq - 3 10' => {
			eco => 'B39i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 Ne6 10. Qd2

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 Ne6 10. Qd2'),
			moves => {
			},
		},
		'r1bqk2r/pp1pppbp/4n1p1/8/2P1P3/2N1B3/PP3PPP/2RQKB1R b Kkq - 3 10' => {
			eco => 'B39j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 Ne6 10. Rc1

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 Ne6 10. Rc1'),
			moves => {
				'd8a5' => 'r1b1k2r/pp1pppbp/4n1p1/q7/2P1P3/2N1B3/PP3PPP/2RQKB1R w Kkq - 4 11',
			},
		},
		'r1bqk2r/pp1pppbp/4n1p1/8/2P1P3/2N1B3/PP3PPP/R2QKB1R w KQkq - 2 10' => {
			eco => 'B39h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1 Ne6

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1 Ne6'),
			moves => {
				'a1c1' => 'r1bqk2r/pp1pppbp/4n1p1/8/2P1P3/2N1B3/PP3PPP/2RQKB1R b Kkq - 3 10',
				'd1d2' => 'r1bqk2r/pp1pppbp/4n1p1/8/2P1P3/2N1B3/PP1Q1PPP/R3KB1R b KQkq - 3 10',
			},
		},
		'r1bqk2r/pp1pppbp/6p1/8/2PnP1Q1/2N1B3/PP3PPP/R3KB1R w KQkq - 0 9' => {
			eco => 'B39c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4'),
			moves => {
				'g4d1' => 'r1bqk2r/pp1pppbp/6p1/8/2PnP3/2N1B3/PP3PPP/R2QKB1R b KQkq - 1 9',
			},
		},
		'r1bqk2r/pp1pppbp/6p1/8/2PnP3/2N1B3/PP3PPP/R2QKB1R b KQkq - 1 9' => {
			eco => 'B39d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 Ng4 8. Qxg4 Nxd4 9. Qd1

			variation => N__('Sicilian: Maroczy Bind, Breyer, 8.Qxg4 Nxd4 9. Qd1'),
			moves => {
				'd4e6' => 'r1bqk2r/pp1pppbp/4n1p1/8/2P1P3/2N1B3/PP3PPP/R2QKB1R w KQkq - 2 10',
				'e7e5' => 'r1bqk2r/pp1p1pbp/6p1/4p3/2PnP3/2N1B3/PP3PPP/R2QKB1R w KQkq e6 0 10',
			},
		},
		'r1bqk2r/pp2bpp1/2np1n1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQ1RK1 b kq - 0 9' => {
			eco => 'B58l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. h3

			variation => N__('Sicilian: Boleslavsky, 7. Nf3 h6 8.O-O Be7 9. h3'),
			moves => {
			},
		},
		'r1bqk2r/pp2bpp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'B58k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7

			variation => N__('Sicilian: Boleslavsky, 7. Nf3 h6 8.O-O Be7'),
			moves => {
				'f1e1' => 'r1bqk2r/pp2bpp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQR1K1 b kq - 3 9',
				'h2h3' => 'r1bqk2r/pp2bpp1/2np1n1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r1bqk2r/pp2bpp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQR1K1 b kq - 3 9' => {
			eco => 'B58m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1

			variation => N__('Sicilian: Boleslavsky, 7. Nf3 h6 8.O-O Be7 9. Re1'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bpp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQR1K1 w - - 4 10',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 4 8' => {
			eco => 'D41c',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. g3 Nc6 7. Bg2 Be7'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 9',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8' => {
			eco => 'D41c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6 7. Bg2 Be7

			variation => N__('QGD: Semi-Tarrasch, 6. g3 Nc6 7. Bg2 Be7'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 8' => {
			eco => 'D42b',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bd3 Be7'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 9',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 8' => {
			eco => 'D42b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 Be7

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bd3 Be7'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/2pn4/8/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 8' => {
			eco => 'A36d',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 e6 4. Nf3 Nf6 5. Bg2 Be7 6. O-O d5 7. cxd5 Nxd5

			variation => N__('English: Symmetrical, Keres-Parma, Main Line Exchange'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'D41r',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQ1RK1 w - - 3 10',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'D41r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4 Be7

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQ1RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'D42e',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQ1RK1 w - - 3 10',
			},
		},
		'r1bqk2r/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'D42e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4 Be7

			variation => N__('QGD: Semi-Tarrasch, 7. Bd3 Main Line'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/2Pp2B1/3P4/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B13z',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nf3 Be7 8. c5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Normal Variaiton, 8. c5'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n1pn2/2Pp2B1/3P4/2N2N2/PP3PPP/R2QKB1R w KQ - 1 9',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/2Pp4/3P4/P1N2N2/1P3PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D40m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 cxd4 7. exd4 Be7 8. c5

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 cxd4 7.exd4 Be7 8. c5'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/2pP4/8/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A36c',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Keres-Parma System'),
			moves => {
				'f6d5' => 'r1bqk2r/pp2bppp/2n1p3/2pn4/8/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/2pp4/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq d6 0 7' => {
			eco => 'A36c',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Keres-Parma System'),
			moves => {
				'c4d5' => 'r1bqk2r/pp2bppp/2n1pn2/2pP4/8/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Nf6 6. Nbd2 Be7 7. e4

			variation => N__('Reti: KIA, French Variation'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w - - 1 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 3 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e2e4' => 'r1bqk2r/pp2bppp/2n1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/3P2B1/3QP3/2N5/PP3PPP/R3KBNR w KQkq - 1 8' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. cxd5 cxd4 6. Qxd4 Be7 7. e4 Nc6

			variation => N__('QGD: Dutch-Peruvian, Prins, 6.Qxd4 Be7 7. e4 Nc6'),
			moves => {
				'd4d2' => 'r1bqk2r/pp2bppp/2n1pn2/3P2B1/4P3/2N5/PP1Q1PPP/R3KBNR b KQkq - 2 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/3P2B1/4P3/2N5/PP1Q1PPP/R3KBNR b KQkq - 2 8' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. cxd5 cxd4 6. Qxd4 Be7 7. e4 Nc6 8. Qd2

			variation => N__('QGD: Dutch-Peruvian, Prins, 6.Qxd4 Be7 7. e4 Nc6 8. Qd2'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/3p2B1/2PP4/2N2N2/PP3PPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'B13y',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nf3 Be7

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Normal Variation'),
			moves => {
				'c4c5' => 'r1bqk2r/pp2bppp/2n1pn2/2Pp2B1/3P4/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/3p4/2PP4/P1N2N2/1P3PPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D40l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 cxd4 7. exd4 Be7

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 cxd4 7.exd4 Be7'),
			moves => {
				'c4c5' => 'r1bqk2r/pp2bppp/2n1pn2/2Pp4/3P4/P1N2N2/1P3PPP/R1BQKB1R b KQkq - 0 8',
				'f1d3' => 'r1bqk2r/pp2bppp/2n1pn2/3p4/2PP4/P1NB1N2/1P3PPP/R1BQK2R b KQkq - 2 8',
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/3p4/2PP4/P1NB1N2/1P3PPP/R1BQK2R b KQkq - 2 8' => {
			eco => 'D40n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 cxd4 7. exd4 Be7 8. Bd3

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 cxd4 7.exd4 Be7 8. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2n1pn2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8' => {
			eco => 'D13r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 e6 7. e3 Be7

			variation => N__('Slav: Exchange, 6. Bf4 e6 7. e3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/2Pp4/8/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'D32u',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 Be7'),
			moves => {
				'e7c5' => 'r1bqk2r/pp3ppp/2n2n2/2bp4/8/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 3 8' => {
			eco => 'D32u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 Be7

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 Be7'),
			moves => {
				'd4c5' => 'r1bqk2r/pp2bppp/2n2n2/2Pp4/8/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 4 8' => {
			eco => 'D34a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O

			variation => N__('QGD Tarrasch: 7. Bg2 Be7 8.O-O'),
			moves => {
				'c8e6' => 'r2qk2r/pp2bppp/2n1bn2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 5 9',
				'e8g8' => 'r1bq1rk1/pp2bppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 9',
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8' => {
			eco => 'D34a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7

			variation => N__('QGD Tarrasch: 7. Bg2 Be7'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 5 8' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nc6 6. Bg2 Nf6 7. O-O Be7

			variation => N__('QGD: Tarrasch without Nc3: Main Line'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'D33p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 cxd4 8. Nxd4 Be7 9. O-O

			variation => N__('QGD Tarrasch: 7. Bg2 cxd4 8.Nxd4 Be7'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 3 10',
			},
		},
		'r1bqk2r/pp2bppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQK2R w KQkq - 1 9' => {
			eco => 'D33p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 cxd4 8. Nxd4 Be7

			variation => N__('QGD Tarrasch: 7. Bg2 cxd4 8.Nxd4 Be7'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2bppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQ1RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/pp2bppp/2np1n2/4p1B1/4P3/1NN5/PPP1BPPP/R2QK2R b KQkq - 3 8' => {
			eco => 'B59d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. Bg5

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8. Bg5'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2np1n2/4p3/3NPP2/2N1BQ2/PPP3PP/R3KB1R w KQkq - 0 9' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6 7. Be3 Be7 8. Qf3 e5

			variation => N__('Sicilian: Scheveningen, Tal, 8... e5'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2QK2R b KQkq - 3 8' => {
			eco => 'B59e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. Be3

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8. Be3'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN2P2/PPP3PP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B56n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. f3 e5 7. Nb3 Be7

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. f3 e5 7. Nb3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'B59f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w - - 4 9',
			},
		},
		'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B59c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7'),
			moves => {
				'c1e3' => 'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2QK2R b KQkq - 3 8',
				'c1g5' => 'r1bqk2r/pp2bppp/2np1n2/4p1B1/4P3/1NN5/PPP1BPPP/R2QK2R b KQkq - 3 8',
				'e1g1' => 'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R w KQkq - 2 8' => {
			eco => 'B62c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Be2 Be7

			variation => N__('Sicilian: Richter-Rauzer, 6... e6 7. Be2 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 3 8' => {
			eco => 'B63l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7 8.O-O-O'),
			moves => {
				'c6d4' => 'r1bqk2r/pp2bppp/3ppn2/6B1/3nP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 9',
				'e8g8' => 'r1bq1rk1/pp2bppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w - - 4 9',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B63k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7'),
			moves => {
				'e1c1' => 'r1bqk2r/pp2bppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 3 8',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/2B1P3/2N2N2/PP2QPPP/R1B2RK1 w kq - 4 9' => {
			eco => 'B21z',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4 e6 7. O-O Nf6 8. Qe2 Be7

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O Nf6 8. Qe2 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/R3K2R b KQkq - 5 8' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 Be7 8. Qe2

			variation => N__('Sicilian: Velimirovic'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/R3K2R w KQ - 6 9',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 4 8' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 Be7

			variation => N__('Sicilian: Sozin, 7. Be3 Be7'),
			moves => {
				'd1e2' => 'r1bqk2r/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP1QPPP/R3K2R b KQkq - 5 8',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/3NP3/1BN1B3/PPP2PPP/R2QK2R b KQkq - 5 8' => {
			eco => 'B88',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Bb3 Be7 8. Be3

			variation => N__('Sicilian: Sozin-Scheveningen, 7. Bb3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2nppn2/8/3NP3/1BN1B3/PPP2PPP/R2QK2R w KQ - 6 9',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R w KQkq - 4 8' => {
			eco => 'B88',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin-Scheveningen, 7. Bb3'),
			moves => {
				'c1e3' => 'r1bqk2r/pp2bppp/2nppn2/8/3NP3/1BN1B3/PPP2PPP/R2QK2R b KQkq - 5 8',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 5 8' => {
			eco => 'B83',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Modern'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/2nppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w - - 6 9',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 4 8' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7

			variation => N__('Sicilian: Scheveningen, Modern'),
			moves => {
				'c1e3' => 'r1bqk2r/pp2bppp/2nppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 5 8',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq - 3 8' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6 7. Be3 Be7

			variation => N__('Sicilian: Scheveningen, 6. f4 Nc6 7. Be3 Be7'),
			moves => {
				'd1f3' => 'r1bqk2r/pp2bppp/2nppn2/8/3NPP2/2N1BQ2/PPP3PP/R3KB1R b KQkq - 4 8',
			},
		},
		'r1bqk2r/pp2bppp/2nppn2/8/3NPP2/2N1BQ2/PPP3PP/R3KB1R b KQkq - 4 8' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6 7. Be3 Be7 8. Qf3

			variation => N__('Sicilian: Scheveningen, Tal Variation'),
			moves => {
				'd8c7' => 'r1b1k2r/ppq1bppp/2nppn2/8/3NPP2/2N1BQ2/PPP3PP/R3KB1R w KQkq - 5 9',
				'e6e5' => 'r1bqk2r/pp2bppp/2np1n2/4p3/3NPP2/2N1BQ2/PPP3PP/R3KB1R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pp2bppp/3ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R b kq - 0 9' => {
			eco => 'B63m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O Nxd4 9. Qxd4

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7 8.O-O-O Nxd4'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2bppp/3ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R w - - 1 10',
			},
		},
		'r1bqk2r/pp2bppp/3ppn2/6B1/3nP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 9' => {
			eco => 'B63m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O Nxd4

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7 8.O-O-O Nxd4'),
			moves => {
				'd2d4' => 'r1bqk2r/pp2bppp/3ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R b kq - 0 9',
			},
		},
		'r1bqk2r/pp2npbp/2n1p1p1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w kq - 1 8' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Nge7 6. Nbd2 g6 7. e4 Bg7

			variation => N__('Reti: KIA, French Variation'),
			moves => {
			},
		},
		'r1bqk2r/pp2npbp/2np2p1/2p1p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w kq - 1 8' => {
			eco => 'A37v',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O d6 7. d3 Nge7

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O d6 7. d3 Nge7'),
			moves => {
				'a2a3' => 'r1bqk2r/pp2npbp/2np2p1/2p1p3/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 b kq - 0 8',
			},
		},
		'r1bqk2r/pp2npbp/2np2p1/2p1p3/2P5/P1NP1NP1/1P2PPBP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'A37w',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 e5 6. O-O d6 7. d3 Nge7 8. a3

			variation => N__('English: Symmetrical, 5. Nf3 e5 6.O-O d6 7. d3 Nge7 8. a3'),
			moves => {
			},
		},
		'r1bqk2r/pp2npbp/2np2p1/2p1p3/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8' => {
			eco => 'B26f',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e5 7. Qd2 Nge7

			variation => N__('Sicilian: Closed, 6. Be3 e5 7. Qd2 Nge7'),
			moves => {
			},
		},
		'r1bqk2r/pp2npbp/2np2p1/2p1p3/4PP2/2NP1NP1/PPP3BP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'B25k',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e5 7. Nf3 Nge7 8. O-O

			variation => N__('Sicilian: Closed, 6. f4 e5 7. Nf3 Nge7 8.O-O'),
			moves => {
			},
		},
		'r1bqk2r/pp2npbp/2np2p1/2p1p3/4PP2/2NP1NP1/PPP3BP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B25j',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e5 7. Nf3 Nge7

			variation => N__('Sicilian: Closed, 6. f4 e5 7. Nf3 Nge7'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2npbp/2np2p1/2p1p3/4PP2/2NP1NP1/PPP3BP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'r1bqk2r/pp2npbp/2npp1p1/2p5/4P3/2NPB1P1/PPPQ1PBP/R3K1NR w KQkq - 2 8' => {
			eco => 'B26o',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 e6 7. Qd2 Nge7

			variation => N__('Sicilian: Closed, 6. Be3 e6 7. Qd2 Nge7'),
			moves => {
			},
		},
		'r1bqk2r/pp2npbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'B25o',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7 8. O-O

			variation => N__('Sicilian: Closed, 6. f4 e6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2npbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQ1RK1 w - - 4 9',
			},
		},
		'r1bqk2r/pp2npbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B25n',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 e6 7. Nf3 Nge7

			variation => N__('Sicilian: Closed, 6. f4 e6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2npbp/2npp1p1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'r1bqk2r/pp2nppp/1bn5/1B1p4/8/1N3N2/PPP2PPP/R1BQ1RK1 w kq - 2 10' => {
			eco => 'C09k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bb6

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bb6'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 3 8' => {
			eco => 'C18k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Nbc6

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 Nbc6'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P3P/P1P2N2/2P2PP1/R1BQKB1R b KQkq - 2 8' => {
			eco => 'C18i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. h4 Nbc6 8. Nf3

			variation => N__('French: Winawer, 6... Ne7 7. h4 Nbc6'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P3P/P1P5/2P2PP1/R1BQKBNR w KQkq - 1 8' => {
			eco => 'C18i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. h4 Nbc6

			variation => N__('French: Winawer, 6... Ne7 7. h4 Nbc6'),
			moves => {
				'g1f3' => 'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P3P/P1P2N2/2P2PP1/R1BQKB1R b KQkq - 2 8',
			},
		},
		'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8' => {
			eco => 'C19l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6'),
			moves => {
				'a3a4' => 'r1bqk2r/pp2nppp/2n1p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp2nppp/2n1p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'C19m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4'),
			moves => {
				'c8d7' => 'r2qk2r/pp1bnppp/2n1p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R w KQkq - 1 9',
				'd8a5' => 'r1b1k2r/pp2nppp/2n1p3/q1ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R w KQkq - 1 9',
			},
		},
		'r1bqk2r/pp2nppp/2n5/1Bbp4/8/1N3N2/PPP2PPP/R1BQ1RK1 b kq - 1 9' => {
			eco => 'C09k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3'),
			moves => {
				'c5b6' => 'r1bqk2r/pp2nppp/1bn5/1B1p4/8/1N3N2/PPP2PPP/R1BQ1RK1 w kq - 2 10',
				'c5d6' => 'r1bqk2r/pp2nppp/2nb4/1B1p4/8/1N3N2/PPP2PPP/R1BQ1RK1 w kq - 2 10',
			},
		},
		'r1bqk2r/pp2nppp/2n5/1Bbp4/8/2P2N2/PP1N1PPP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C09g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. c3

			variation => N__('French: Tarrasch, Open, Main Line, 9. c3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2nppp/2n5/1Bbp4/8/2P2N2/PP1N1PPP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r1bqk2r/pp2nppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'C09d',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 7.O-O'),
			moves => {
				'c2c3' => 'r1bqk2r/pp2nppp/2n5/1Bbp4/8/2P2N2/PP1N1PPP/R1BQ1RK1 b kq - 0 9',
				'd2b3' => 'r1bqk2r/pp2nppp/2n5/1Bbp4/8/1N3N2/PPP2PPP/R1BQ1RK1 b kq - 1 9',
			},
		},
		'r1bqk2r/pp2nppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'C09f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. dxc5 Bxc5 8. O-O Ne7

			variation => N__('French: Tarrasch, Open, Main Line'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n5/3p4/1bpP4/2N2NP1/PP1BPPBP/R2Q1RK1 b kq - 5 9' => {
			eco => 'D33h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2 Bb4 8. O-O Nge7 9. Bd2

			variation => N__('QGD Tarrasch: Swedish, 9. Bd2'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n5/3p4/1bpP4/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 4 9' => {
			eco => 'D33f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2 Bb4 8. O-O Nge7

			variation => N__('QGD Tarrasch: Swedish, 7. Bg2 Bb4 8.O-O Nge7'),
			moves => {
				'a2a3' => 'r1bqk2r/pp2nppp/2n5/3p4/1bpP4/P1N2NP1/1P2PPBP/R1BQ1RK1 b kq - 0 9',
				'c1d2' => 'r1bqk2r/pp2nppp/2n5/3p4/1bpP4/2N2NP1/PP1BPPBP/R2Q1RK1 b kq - 5 9',
				'e2e4' => 'r1bqk2r/pp2nppp/2n5/3p4/1bpPP3/2N2NP1/PP3PBP/R1BQ1RK1 b kq e3 0 9',
				'f3e5' => 'r1bqk2r/pp2nppp/2n5/3pN3/1bpP4/2N3P1/PP2PPBP/R1BQ1RK1 b kq - 5 9',
			},
		},
		'r1bqk2r/pp2nppp/2n5/3p4/1bpP4/P1N2NP1/1P2PPBP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'D33g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2 Bb4 8. O-O Nge7 9. a3

			variation => N__('QGD Tarrasch: Swedish, 9. a3'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n5/3p4/1bpPP3/2N2NP1/PP3PBP/R1BQ1RK1 b kq e3 0 9' => {
			eco => 'D33j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2 Bb4 8. O-O Nge7 9. e4

			variation => N__('QGD Tarrasch: Swedish, 9. e4'),
			moves => {
				'd5e4' => 'r1bqk2r/pp2nppp/2n5/8/1bpPp3/2N2NP1/PP3PBP/R1BQ1RK1 w kq - 0 10',
			},
		},
		'r1bqk2r/pp2nppp/2n5/3pN3/1bpP4/2N3P1/PP2PPBP/R1BQ1RK1 b kq - 5 9' => {
			eco => 'D33i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2 Bb4 8. O-O Nge7 9. Ne5

			variation => N__('QGD Tarrasch: Swedish, 9. Ne5'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2n5/8/1bpPp3/2N2NP1/PP3PBP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'D33k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2 Bb4 8. O-O Nge7 9. e4 dxe4

			variation => N__('QGD Tarrasch: Swedish, 9. e4 dxe4'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2nb4/1B1p2B1/8/1N3N2/PPP2PPP/R2Q1RK1 b kq - 3 10' => {
			eco => 'C09n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bd6 10. Bg5

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6 10. Bg5'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2nb4/1B1p4/3N4/5N2/PPP2PPP/R1BQ1RK1 b kq - 3 10' => {
			eco => 'C09m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bd6 10. Nbd4

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6 10.Nbd4'),
			moves => {
			},
		},
		'r1bqk2r/pp2nppp/2nb4/1B1p4/8/1N3N2/PPP2PPP/R1BQ1RK1 w kq - 2 10' => {
			eco => 'C09l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bd6

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6'),
			moves => {
				'b3d4' => 'r1bqk2r/pp2nppp/2nb4/1B1p4/3N4/5N2/PPP2PPP/R1BQ1RK1 b kq - 3 10',
				'c1g5' => 'r1bqk2r/pp2nppp/2nb4/1B1p2B1/8/1N3N2/PPP2PPP/R2Q1RK1 b kq - 3 10',
				'f1e1' => 'r1bqk2r/pp2nppp/2nb4/1B1p4/8/1N3N2/PPP2PPP/R1BQR1K1 b kq - 3 10',
			},
		},
		'r1bqk2r/pp2nppp/2nb4/1B1p4/8/1N3N2/PPP2PPP/R1BQR1K1 b kq - 3 10' => {
			eco => 'C09o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bd6 10. Re1

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6 10. Re1'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2nppp/2nb4/1B1p4/8/1N3N2/PPP2PPP/R1BQR1K1 w - - 4 11',
			},
		},
		'r1bqk2r/pp2nppp/2nb4/1BPp4/8/5N2/PPPN1PPP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'C09d',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 7.O-O'),
			moves => {
				'd6c5' => 'r1bqk2r/pp2nppp/2n5/1Bbp4/8/5N2/PPPN1PPP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r1bqk2r/pp2nppp/2nb4/1Bpp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 w kq - 6 8' => {
			eco => 'C09d',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 7.O-O'),
			moves => {
				'd4c5' => 'r1bqk2r/pp2nppp/2nb4/1BPp4/8/5N2/PPPN1PPP/R1BQ1RK1 b kq - 0 8',
			},
		},
		'r1bqk2r/pp2nppp/2nbp3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Bd6 6. Nbd2 Nge7 7. e4

			variation => N__('Reti: KIA, French Variation'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2nppp/2nbp3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w - - 1 8',
			},
		},
		'r1bqk2r/pp2nppp/2nbp3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 3 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e2e4' => 'r1bqk2r/pp2nppp/2nbp3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7',
			},
		},
		'r1bqk2r/pp2nppp/3p1n2/2pPp3/2P1P3/2PB1N2/P4PPP/R1BQK2R w KQkq - 1 10' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3 Bxc3+ 7. bxc3 d6 8. e4 e5 9. d5 Ne7

			variation => N__('Nimzo-Indian: Hübner, 8. e4 e5 9. d5 Ne7'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2n2np1/2pp4/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq d6 0 7' => {
			eco => 'A38d',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O d5

			variation => N__('English: Symmetrical, Main Line, 6.O-O d5'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2n3p1/2p5/3PP3/2P1BN2/P4PPP/R2QKB1R w KQkq - 2 9' => {
			eco => 'D85t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 Nc6

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'A36u',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Botvinnik, 5... d6 6.Nge2 Nf6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 w - - 4 8',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A36u',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 Nf6

			variation => N__('English: Symmetrical, Botvinnik, 5... d6 6.Nge2 Nf6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/2P1P3/2N3P1/PP1PNPBP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A38e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O d6

			variation => N__('English: Symmetrical, Main Line, 6.O-O d6'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/4P3/2NPB1P1/PPP2PBP/R2QK1NR w KQkq - 2 7' => {
			eco => 'B26b',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 Nf6

			variation => N__('Sicilian: Closed, 6. Be3 Nf6'),
			moves => {
				'h2h3' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/4P3/2NPB1PP/PPP2PB1/R2QK1NR b KQkq - 0 7',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/4P3/2NPB1PP/PPP2PB1/R2QK1NR b KQkq - 0 7' => {
			eco => 'B26c',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Be3 Nf6 7. h3

			variation => N__('Sicilian: Closed, 6. Be3 Nf6 7. h3'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B25h',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 Nf6 7. Nf3

			variation => N__('Sicilian: Closed, 6. f4 Nf6 7. Nf3'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/2p5/4PP2/2NP2P1/PPP3BP/R1BQK1NR w KQkq - 1 7' => {
			eco => 'B25g',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. f4 Nf6

			variation => N__('Sicilian: Closed, 6. f4 Nf6'),
			moves => {
				'g1f3' => 'r1bqk2r/pp2ppbp/2np1np1/2p5/4PP2/2NP1NP1/PPP3BP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 4 8' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Bc4 Nc6

			variation => N__('Sicilian: Dragon, 6. Be3 Bg7 7. Bc4 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/2BNP3/2N1BP2/PPP3PP/R2QK2R b KQkq - 2 8' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 Nc6 8. Bc4

			variation => N__('Sicilian: Dragon, Yugoslav, 7... Nc6 8. Bc4'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN1BPPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'B37i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN1BPPP/R1BQK2R w KQ - 2 9',
				'f6d7' => 'r1bqk2r/pp1nppbp/2np2p1/8/2P1P3/2N5/PPN1BPPP/R1BQK2R w KQkq - 2 9',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN2PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B37i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6'),
			moves => {
				'f1e2' => 'r1bqk2r/pp2ppbp/2np1np1/8/2P1P3/2N5/PPN1BPPP/R1BQK2R b KQkq - 1 8',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R b KQkq - 1 8' => {
			eco => 'B38f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 d6 8. Be2

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 Nf6 7. Nc3 d6 8. Be2'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'B38e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 d6

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 Nf6 7. Nc3 d6'),
			moves => {
				'f1e2' => 'r1bqk2r/pp2ppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2QK2R b KQkq - 1 8',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 5 8' => {
			eco => 'B73',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O

			variation => N__('Sicilian: Dragon, Classical, 8.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w - - 6 9',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq - 4 8' => {
			eco => 'B72',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical Attack'),
			moves => {
				'd1d2' => 'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/R3K2R b KQkq - 5 8',
				'd4b3' => 'r1bqk2r/pp2ppbp/2np1np1/8/4P3/1NN1B3/PPP1BPPP/R2QK2R b KQkq - 5 8',
				'e1g1' => 'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 5 8',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/R3K2R b KQkq - 5 8' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. Qd2

			variation => N__('Sicilian: Dragon, Classical, Amsterdam Variation'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPPQBPPP/R3K2R w KQ - 6 9',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 Nc6

			variation => N__('Sicilian: Dragon, Yugoslav, 7... Nc6'),
			moves => {
				'd1d2' => 'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 2 8',
				'f1c4' => 'r1bqk2r/pp2ppbp/2np1np1/8/2BNP3/2N1BP2/PPP3PP/R2QK2R b KQkq - 2 8',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 Nc6 8. Qd2

			variation => N__('Sicilian: Dragon, Yugoslav, 7... Nc6 8. Qd2'),
			moves => {
				'c8d7' => 'r2qk2r/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 3 9',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 4 8' => {
			eco => 'B70l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O Nc6

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O Nc6'),
			moves => {
				'd4b3' => 'r1bqk2r/pp2ppbp/2np1np1/8/4P3/1NN5/PPP1BPPP/R1BQ1RK1 b kq - 5 8',
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/4P3/1NN1B3/PPP1BPPP/R2QK2R b KQkq - 5 8' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. Nb3

			variation => N__('Sicilian: Dragon, Classical, Nottingham Variation'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np1np1/8/4P3/1NN5/PPP1BPPP/R1BQ1RK1 b kq - 5 8' => {
			eco => 'B70m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O Nc6 8. Nb3

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O Nc6 8. Nb3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/2np1np1/8/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w - - 6 9',
			},
		},
		'r1bqk2r/pp2ppbp/2np2pn/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7' => {
			eco => 'A37h',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. O-O Nh6

			variation => N__('English: Symmetrical, 5. Nf3 d6 6.O-O Nh6'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np2pn/8/2P1P3/8/PPN1BPPP/RNBQK2R w KQkq - 2 8' => {
			eco => 'B37f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 d6 7. Be2 Nh6

			variation => N__('Sicilian: Maroczy Bind, Simagin Variation'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/2np2pn/8/2PNP3/2N1B3/PP3PPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'B38c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 d6 7. Nc3 Nh6

			variation => N__('Sicilian: Maroczy Bind, 6. Be3 d6 7. Nc3 Nh6'),
			moves => {
			},
		},
		'r1bqk2r/pp2ppbp/3p1np1/6B1/2PQP3/2N5/PP2BPPP/R3K2R b KQkq - 2 9' => {
			eco => 'B36k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Bg5

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Bg5'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/3p1np1/6B1/2PQP3/2N5/PP2BPPP/R3K2R w KQ - 3 10',
			},
		},
		'r1bqk2r/pp2ppbp/3p1np1/8/2PQP3/2N1B3/PP2BPPP/R3K2R b KQkq - 2 9' => {
			eco => 'B36n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Be3

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Be3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp2ppbp/3p1np1/8/2PQP3/2N1B3/PP2BPPP/R3K2R w KQ - 3 10',
			},
		},
		'r1bqk2r/pp2ppbp/3p1np1/8/2PQP3/2N5/PP2BPPP/R1B1K2R w KQkq - 1 9' => {
			eco => 'B36i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7'),
			moves => {
				'c1e3' => 'r1bqk2r/pp2ppbp/3p1np1/8/2PQP3/2N1B3/PP2BPPP/R3K2R b KQkq - 2 9',
				'c1g5' => 'r1bqk2r/pp2ppbp/3p1np1/6B1/2PQP3/2N5/PP2BPPP/R3K2R b KQkq - 2 9',
				'e1g1' => 'r1bqk2r/pp2ppbp/3p1np1/8/2PQP3/2N5/PP2BPPP/R1B2RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/pp2ppbp/3p1np1/8/2PQP3/2N5/PP2BPPP/R1B2RK1 b kq - 2 9' => {
			eco => 'B36j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. O-O

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9.O-O'),
			moves => {
			},
		},
		'r1bqk2r/pp3p2/2n1p1pp/2ppP3/3P2Q1/2PB1N2/P1PK1PPP/R6R w kq - 2 12' => {
			eco => 'C12r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 g6 9. Bd3 Nxd2 10. Kxd2 c5 11. Nf3 Nc6

			variation => N__('French: MacCutcheon, Main Line, 11. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp3pbp/2n1pnp1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w kq - 1 8' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Nf6 6. Nbd2 g6 7. e4 Bg7

			variation => N__('Reti: KIA, French Variation'),
			moves => {
			},
		},
		'r1bqk2r/pp3pp1/2pbpn1p/8/3PQ3/3B1N2/PPP2PPP/R1B1K2R w KQkq - 1 11' => {
			eco => 'B17o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6 6. Bd3 e6 7. N1f3 Bd6 8. Qe2 h6 9. Ne4 Nxe4 10. Qxe4 Nf6

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3, Main Line 10.Qxe4 Nf6'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2n1p3/2Pp4/2P1n3/P7/1PQ1PPPP/R1B1KBNR w KQkq - 1 9' => {
			eco => 'E37',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Ne4 7. Qc2 c5 8. dxc5 Nc6

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line, 7. Qc2 c5 8.dxc5 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2N2N2/PP1B1PPP/R2QKB1R w KQkq - 2 9' => {
			eco => 'B14w',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5 8. Bd2 Nc6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Bd2 Nc6'),
			moves => {
				'f1d3' => 'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2NB1N2/PP1B1PPP/R2QK2R b KQkq - 3 9',
			},
		},
		'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2N2N2/PPQ2PPP/R1B1KB1R w KQkq - 2 9' => {
			eco => 'B14t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5 8. Qc2 Nc6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Qc2 Nc6'),
			moves => {
				'f1d3' => 'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2NB1N2/PPQ2PPP/R1B1K2R b KQkq - 3 9',
			},
		},
		'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2NB1N2/PP1B1PPP/R2QK2R b KQkq - 3 9' => {
			eco => 'B14w',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Bd2 Nc6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp3ppp/2n1p3/3n4/1b1P4/2NB1N2/PP1B1PPP/R2QK2R w KQ - 4 10',
			},
		},
		'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2NB1N2/PPQ2PPP/R1B1K2R b KQkq - 3 9' => {
			eco => 'B14u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5 8. Qc2 Nc6 9. Bd3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Qc2 Nc6 9. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2n1pn2/3P4/8/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 9' => {
			eco => 'B45s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3 Bxc3+ 8.Nxc3 d5'),
			moves => {
				'e6d5' => 'r1bqk2r/pp3ppp/2n2n2/3p4/8/P1N5/1PP2PPP/R1BQKB1R w KQkq - 0 10',
			},
		},
		'r1bqk2r/pp3ppp/2n1pn2/3p4/4P3/P1N5/1PP2PPP/R1BQKB1R w KQkq d6 0 9' => {
			eco => 'B45s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. a3 Bxc3+ 8. Nxc3 d5

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3 Bxc3+ 8.Nxc3 d5'),
			moves => {
				'e4d5' => 'r1bqk2r/pp3ppp/2n1pn2/3P4/8/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 9',
			},
		},
		'r1bqk2r/pp3ppp/2n1pn2/4P1B1/Qbpp4/2N2N2/PP3PPP/2KR1B1R b kq - 3 9' => {
			eco => 'D39g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. e5 cxd4 8. Qa4+ Nc6 9. O-O-O

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7. e5 cxd4 8. Qa4+ Nc6 9.O-O-O'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2n1pn2/4P1B1/Qbpp4/2N2N2/PP3PPP/R3KB1R w KQkq - 2 9' => {
			eco => 'D39f',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7. e5 cxd4'),
			moves => {
				'e1c1' => 'r1bqk2r/pp3ppp/2n1pn2/4P1B1/Qbpp4/2N2N2/PP3PPP/2KR1B1R b kq - 3 9',
			},
		},
		'r1bqk2r/pp3ppp/2n2n2/2bp4/8/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'D32u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 Be7 8. dxc5 Bxc5

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 Be7 8.dxc5 Bxc5'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2n2n2/3p4/8/P1N5/1PP2PPP/R1BQKB1R w KQkq - 0 10' => {
			eco => 'B45t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 Bb4 7. a3 Bxc3+ 8. Nxc3 d5 9. exd5 exd5

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3, 9.ed5 ed5'),
			moves => {
				'f1d3' => 'r1bqk2r/pp3ppp/2n2n2/3p4/8/P1NB4/1PP2PPP/R1BQK2R b KQkq - 1 10',
			},
		},
		'r1bqk2r/pp3ppp/2n2n2/3p4/8/P1NB4/1PP2PPP/R1BQK2R b KQkq - 1 10' => {
			eco => 'B45t',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 Bb4 7. a3, 9.ed5 ed5'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp3ppp/2n2n2/3p4/8/P1NB4/1PP2PPP/R1BQK2R w KQ - 2 11',
			},
		},
		'r1bqk2r/pp3ppp/2nBpn2/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'D13s',
                        # TRANSLATORS:

			variation => N__('Slav: Exchange, 6. Bf4 e6 7. e3 Bd6'),
			moves => {
				'd8d6' => 'r1b1k2r/pp3ppp/2nqpn2/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pp3ppp/2nb1n2/3p4/3N4/2N1P3/PP2BPPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'D32v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8. Nxd4 Bd6 9. O-O

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8.Nxd4 Bd6 9.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp3ppp/2nb1n2/3p4/3N4/2N1P3/PP2BPPP/R1BQ1RK1 w - - 3 10',
			},
		},
		'r1bqk2r/pp3ppp/2nb1n2/3p4/3N4/2N1P3/PP2BPPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'D32v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8. Nxd4 Bd6

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4 8.Nxd4 Bd6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp3ppp/2nb1n2/3p4/3N4/2N1P3/PP2BPPP/R1BQ1RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/pp3ppp/2nbpn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7' => {
			eco => 'D40g',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. Bd3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp3ppp/2nbpn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 5 8',
			},
		},
		'r1bqk2r/pp3ppp/2nbpn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'D40g',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. Bd3'),
			moves => {
				'e1g1' => 'r1bqk2r/pp3ppp/2nbpn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 4 7',
			},
		},
		'r1bqk2r/pp3ppp/2nbpn2/2pp4/3P1B2/2PBPN2/PP3PPP/RN1QK2R w KQkq - 3 7' => {
			eco => 'D02q',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4 e6 4. e3 c5 5. c3 Nc6 6. Bd3 Bd6

			variation => N__('Queen\'s Pawn: London'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2nbpn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQ1RK1 b kq - 4 7' => {
			eco => 'D05l',
                        # TRANSLATORS:

			variation => N__('Colle: 5. c3 Nc6 6.Nbd2 Bd6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pp3ppp/2nbpn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQ1RK1 w - - 5 8',
			},
		},
		'r1bqk2r/pp3ppp/2nbpn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'D05l',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3 Nc6 6. Nbd2 Bd6

			variation => N__('Colle: 5. c3 Nc6 6.Nbd2 Bd6'),
			moves => {
				'e1g1' => 'r1bqk2r/pp3ppp/2nbpn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQ1RK1 b kq - 4 7',
			},
		},
		'r1bqk2r/pp3ppp/2nbpn2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8' => {
			eco => 'D13s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 e6 7. e3 Bd6

			variation => N__('Slav: Exchange, 6. Bf4 e6 7. e3 Bd6'),
			moves => {
				'f4d6' => 'r1bqk2r/pp3ppp/2nBpn2/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp3ppp/2np1n2/2p1p3/2PP4/2PBP3/P2N1PPP/R1BQ1RK1 b kq - 1 9' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3 Bxc3+ 7. bxc3 d6 8. O-O e5 9. Nd2

			variation => N__('Nimzo-Indian: Hübner, 8.O-O e5 9. Nd2'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/2np1n2/2p1p3/2PP4/2PBPN2/P4PPP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3 Bxc3+ 7. bxc3 d6 8. O-O e5

			variation => N__('Nimzo-Indian: Hübner, 8.O-O e5'),
			moves => {
				'f3d2' => 'r1bqk2r/pp3ppp/2np1n2/2p1p3/2PP4/2PBP3/P2N1PPP/R1BQ1RK1 b kq - 1 9',
			},
		},
		'r1bqk2r/pp3ppp/2np1n2/2p1p3/2PPP3/2PB1N2/P4PPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'E41',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Hübner, 8. e4'),
			moves => {
				'd4d5' => 'r1bqk2r/pp3ppp/2np1n2/2pPp3/2P1P3/2PB1N2/P4PPP/R1BQK2R b KQkq - 0 9',
			},
		},
		'r1bqk2r/pp3ppp/2np1n2/2pPp3/2P1P3/2PB1N2/P4PPP/R1BQK2R b KQkq - 0 9' => {
			eco => 'E41',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Hübner, 8. e4'),
			moves => {
				'c6e7' => 'r1bqk2r/pp2nppp/3p1n2/2pPp3/2P1P3/2PB1N2/P4PPP/R1BQK2R w KQkq - 1 10',
			},
		},
		'r1bqk2r/pp3ppp/2nppn2/2p5/2PP4/2PBPN2/P4PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3 Bxc3+ 7. bxc3 d6 8. O-O

			variation => N__('Nimzo-Indian: Hübner, 8.O-O'),
			moves => {
				'e6e5' => 'r1bqk2r/pp3ppp/2np1n2/2p1p3/2PP4/2PBPN2/P4PPP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r1bqk2r/pp3ppp/2nppn2/2p5/2PP4/2PBPN2/P4PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3 Bxc3+ 7. bxc3 d6

			variation => N__('Nimzo-Indian: 4. e3 c5, Hübner Variation'),
			moves => {
				'e1g1' => 'r1bqk2r/pp3ppp/2nppn2/2p5/2PP4/2PBPN2/P4PPP/R1BQ1RK1 b kq - 1 8',
				'e3e4' => 'r1bqk2r/pp3ppp/2nppn2/2p5/2PPP3/2PB1N2/P4PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pp3ppp/2nppn2/2p5/2PPP3/2PB1N2/P4PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3 Nc6 6. Nf3 Bxc3+ 7. bxc3 d6 8. e4

			variation => N__('Nimzo-Indian: Hübner, 8. e4'),
			moves => {
				'e6e5' => 'r1bqk2r/pp3ppp/2np1n2/2p1p3/2PPP3/2PB1N2/P4PPP/R1BQK2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pp3ppp/4pn2/2Pp4/1P1Qn3/P7/4PPPP/R1B1KBNR b KQkq - 0 10' => {
			eco => 'E38l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Na6 6. a3 Bxc3+ 7. Qxc3 Nxc5 8. b4 Nce4 9. Qd4 d5 10. c5

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6: 8. b4 Nce4 9. Qd4 d5 10. c5'),
			moves => {
			},
		},
		'r1bqk2r/pp3ppp/4pn2/3p4/1PPQn3/P7/4PPPP/R1B1KBNR w KQkq d6 0 10' => {
			eco => 'E38l',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Na6: 8. b4 Nce4 9. Qd4'),
			moves => {
				'c4c5' => 'r1bqk2r/pp3ppp/4pn2/2Pp4/1P1Qn3/P7/4PPPP/R1B1KBNR b KQkq - 0 10',
			},
		},
		'r1bqk2r/pp4pp/2nbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 b kq - 3 11' => {
			eco => 'C06o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6 10. O-O Bd6 11. Nf3

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6 Nxf6 10.O-O Bd6 11. Nf3'),
			moves => {
				'd8b6' => 'r1b1k2r/pp4pp/1qnbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 w kq - 4 12',
				'd8c7' => 'r1b1k2r/ppq3pp/2nbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 w kq - 4 12',
				'e8g8' => 'r1bq1rk1/pp4pp/2nbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 w - - 4 12',
			},
		},
		'r1bqk2r/pp4pp/2nbpn2/3p4/3P4/3B4/PP1NNPPP/R1BQ1RK1 w kq - 2 11' => {
			eco => 'C06n',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6 Nxf6'),
			moves => {
				'd2f3' => 'r1bqk2r/pp4pp/2nbpn2/3p4/3P4/3B1N2/PP2NPPP/R1BQ1RK1 b kq - 3 11',
			},
		},
		'r1bqk2r/ppp1bpp1/5n1p/3P4/2P1p3/5N2/PPP1QPPP/RNB1K2R w KQkq - 1 10' => {
			eco => 'C58c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. d3 h6 7. Nf3 e4 8. Qe2 Nxc4 9. dxc4 Be7

			variation => N__('Two Knights: Morphy, Maroczy Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/1RBQ1RK1 b kq - 5 8' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6 7. O-O Be7 8. Rb1

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6 7.O-O Be7 8. Rb1'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 4 8' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6 7. O-O Be7

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6 7.O-O Be7'),
			moves => {
				'a1b1' => 'r1bqk2r/ppp1bppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/1RBQ1RK1 b kq - 5 8',
				'a2a3' => 'r1bqk2r/ppp1bppp/1nn5/4p3/8/P1N2NP1/1P1PPPBP/R1BQ1RK1 b kq - 0 8',
				'd2d3' => 'r1bqk2r/ppp1bppp/1nn5/4p3/8/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 8',
			},
		},
		'r1bqk2r/ppp1bppp/1nn5/4p3/8/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6 7. O-O Be7 8. d3

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6 7.O-O Be7 8. d3'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp1bppp/1nn5/4p3/8/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'r1bqk2r/ppp1bppp/1nn5/4p3/8/P1N2NP1/1P1PPPBP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6 7. O-O Be7 8. a3

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6 7.O-O Be7 8. a3'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/2n2n2/3p2B1/2B4Q/2N2N2/PPP2PPP/R3K2R w KQkq d6 0 9' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 Nc6 8. Qh4 d5

			variation => N__('Bishop\'s Opening: Urusov Gambit, Estrin Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/2n2n2/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 3 6' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nc6 3. Bg2 e5 4. d3 Nf6 5. O-O Be7

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/2n5/1B1pp3/3Pn3/5N2/PPP1QPPP/RNB2RK1 w kq d6 0 7' => {
			eco => 'C67h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 d5

			variation => N__('Spanish: Open Berlin, Trifunovic Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/2n5/3p4/2PPn3/3B1N2/PP3PPP/RNBQ1RK1 b kq c3 0 8' => {
			eco => 'C42t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. c4

			variation => N__('Russian Game: Classical, Jänisch Variation'),
			moves => {
				'c6b4' => 'r1bqk2r/ppp1bppp/8/3p4/1nPPn3/3B1N2/PP3PPP/RNBQ1RK1 w kq - 1 9',
			},
		},
		'r1bqk2r/ppp1bppp/2n5/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 w kq - 4 8' => {
			eco => 'C42s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6

			variation => N__('Russian Game: Classical, 6. Bd3 Be7 7.O-O Nc6'),
			moves => {
				'c2c4' => 'r1bqk2r/ppp1bppp/2n5/3p4/2PPn3/3B1N2/PP3PPP/RNBQ1RK1 b kq c3 0 8',
				'f1e1' => 'r1bqk2r/ppp1bppp/2n5/3p4/3Pn3/3B1N2/PPP2PPP/RNBQR1K1 b kq - 5 8',
			},
		},
		'r1bqk2r/ppp1bppp/2n5/3p4/3Pn3/3B1N2/PPP2PPP/RNBQR1K1 b kq - 5 8' => {
			eco => 'C42v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1

			variation => N__('Russian Game: Classical, Chigorin Variation'),
			moves => {
				'c8f5' => 'r2qk2r/ppp1bppp/2n5/3p1b2/3Pn3/3B1N2/PPP2PPP/RNBQR1K1 w kq - 6 9',
				'c8g4' => 'r2qk2r/ppp1bppp/2n5/3p4/3Pn1b1/3B1N2/PPP2PPP/RNBQR1K1 w kq - 6 9',
			},
		},
		'r1bqk2r/ppp1bppp/2np1n2/6B1/2B4Q/2N2N2/PPP2PPP/R3K2R w KQkq - 0 9' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 Nc6 8. Qh4 d6

			variation => N__('Bishop\'s Opening: Urusov Gambit, Keres Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/3p1n2/4n3/2B1P3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. dxe5 Nxe5

			variation => N__('Philidor: Improved Hanham, 6.dxe5 Nxe5'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/4pn2/6B1/3P4/5N2/PPP2PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'C11r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Nbd7 6. Nf3 Be7 7. Nxf6+ Nxf6

			variation => N__('French: Burn, 5.Nxe4 Nbd7 6. Nf3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/ppp1bppp/8/3p4/1nPPn3/3B1N2/PP3PPP/RNBQ1RK1 w kq - 1 9' => {
			eco => 'C42t',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Jänisch Variation'),
			moves => {
				'd3e2' => 'r1bqk2r/ppp1bppp/8/3p4/1nPPn3/5N2/PP2BPPP/RNBQ1RK1 b kq - 2 9',
			},
		},
		'r1bqk2r/ppp1bppp/8/3p4/1nPPn3/5N2/PP2BPPP/RNBQ1RK1 b kq - 2 9' => {
			eco => 'C42t',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Jänisch Variation'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp1bppp/8/3p4/1nPPn3/5N2/PP2BPPP/RNBQ1RK1 w - - 3 10',
			},
		},
		'r1bqk2r/ppp1npbp/2np2p1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nge7 7. Nge2

			variation => N__('English: Botvinnik System, 6...Nge7 7.Nge2'),
			moves => {
			},
		},
		'r1bqk2r/ppp1npbp/2np2p1/4p3/2P1P3/2NP2P1/PP3PBP/R1BQK1NR w KQkq - 1 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nge7

			variation => N__('English: Botvinnik System, 6...Nge7'),
			moves => {
				'g1e2' => 'r1bqk2r/ppp1npbp/2np2p1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqk2r/ppp1nppp/1b1p4/n2P4/2B1P3/5N2/PB3PPP/RN1Q1RK1 w kq - 3 11' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. d5 Na5 10. Bb2 Ne7

			variation => N__('Evans Gambit: Paulsen Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp1nppp/3p1b2/3P2B1/2B1R3/5N2/PP3PPP/R2Q2K1 b kq - 1 12' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'f6g5' => 'r1bqk2r/ppp1nppp/3p4/3P2b1/2B1R3/5N2/PP3PPP/R2Q2K1 w kq - 0 13',
			},
		},
		'r1bqk2r/ppp1nppp/3p1b2/3P4/2B1R1P1/5N2/PP3P1P/R1BQ2K1 b kq g3 0 12' => {
			eco => 'C54o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Bxc3 9. d5 Bf6 10. Re1 Ne7 11. Rxe4 d6 12. g4

			variation => N__('Giuoco Piano: Möller, Bayonet Attack'),
			moves => {
			},
		},
		'r1bqk2r/ppp1nppp/3p1b2/3P4/2B1R3/5N2/PP3PPP/R1BQ2K1 w kq - 0 12' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'c1g5' => 'r1bqk2r/ppp1nppp/3p1b2/3P2B1/2B1R3/5N2/PP3PPP/R2Q2K1 b kq - 1 12',
				'g2g4' => 'r1bqk2r/ppp1nppp/3p1b2/3P4/2B1R1P1/5N2/PP3P1P/R1BQ2K1 b kq g3 0 12',
			},
		},
		'r1bqk2r/ppp1nppp/3p4/3P2N1/2B1R3/8/PP3PPP/R2Q2K1 b kq - 0 13' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp1nppp/3p4/3P2N1/2B1R3/8/PP3PPP/R2Q2K1 w - - 1 14',
			},
		},
		'r1bqk2r/ppp1nppp/3p4/3P2b1/2B1R3/5N2/PP3PPP/R2Q2K1 w kq - 0 13' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'f3g5' => 'r1bqk2r/ppp1nppp/3p4/3P2N1/2B1R3/8/PP3PPP/R2Q2K1 b kq - 0 13',
			},
		},
		'r1bqk2r/ppp1ppbp/1nn3p1/8/3PP3/6P1/PP2NPBP/RNBQK2R w KQkq - 3 8' => {
			eco => 'D72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6 7. Ne2 Nc6

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6 7. Ne2 Nc6'),
			moves => {
			},
		},
		'r1bqk2r/ppp1ppbp/2np1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'E80c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 Nc6

			variation => N__('King\'s Indian: Sämisch, 5... Nc6'),
			moves => {
			},
		},
		'r1bqk2r/ppp1ppbp/n2p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'E80b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 Na6

			variation => N__('King\'s Indian: Sämisch, 5... Na6'),
			moves => {
			},
		},
		'r1bqk2r/ppp2pBp/1b6/nP2p3/2BPP3/8/P1P2PPP/RN1QK2R b KQkq - 0 9' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'h8g8' => 'r1bqk1r1/ppp2pBp/1b6/nP2p3/2BPP3/8/P1P2PPP/RN1QK2R w KQq - 1 10',
			},
		},
		'r1bqk2r/ppp2pbp/2np1np1/4p3/2P1P3/2NP1NP1/PP3PBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nf6 7. Nf3

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP1NP1/PP3PBP/R1BQK2R w KQ - 3 8',
			},
		},
		'r1bqk2r/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nf6 7. Nge2

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R w KQ - 3 8',
			},
		},
		'r1bqk2r/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP3PBP/R1BQK1NR w KQkq - 1 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. e4 Nf6

			variation => N__('English: Botvinnik System, 6... Nf6'),
			moves => {
				'g1e2' => 'r1bqk2r/ppp2pbp/2np1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R b KQkq - 2 7',
				'g1f3' => 'r1bqk2r/ppp2pbp/2np1np1/4p3/2P1P3/2NP1NP1/PP3PBP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqk2r/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6 7. O-O

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6 7.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 4 8',
			},
		},
		'r1bqk2r/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A26',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 d6 6. Nf3 Nf6

			variation => N__('English: Closed, 5. d3 d6 6. Nf3 Nf6'),
			moves => {
				'e1g1' => 'r1bqk2r/ppp2pbp/2np1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqk2r/ppp2pbp/2np2pn/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 d6 6. Nge2 Nh6

			variation => N__('English: Closed, Taimanov Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2pp1/2np1n1p/2b1p1B1/2B1P3/2NP1N2/PPP2PPP/R2QK2R w KQkq - 0 7' => {
			eco => 'C50v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. Nc3 d6 6. Bg5 h6

			variation => N__('Giuoco Pianissimo: Canal, 6... h6'),
			moves => {
			},
		},
		'r1bqk2r/ppp2pp1/5n1p/2bP4/2P1p3/5N2/PPP1QPPP/RNB1K2R w KQkq - 1 10' => {
			eco => 'C58c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Kieseritsky Variation, 6... h6 7. Nf3 e4 8. Qe2'),
			moves => {
				'f3d2' => 'r1bqk2r/ppp2pp1/5n1p/2bP4/2P1p3/8/PPPNQPPP/RNB1K2R b KQkq - 2 10',
			},
		},
		'r1bqk2r/ppp2pp1/5n1p/2bP4/2P1p3/8/PPPNQPPP/RNB1K2R b KQkq - 2 10' => {
			eco => 'C58c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. d3 h6 7. Nf3 e4 8. Qe2 Nxc4 9. dxc4 Bc5 10. Nfd2

			variation => N__('Two Knights: Morphy, Yankovich Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/1b1p3B/nP2N3/2BPP3/8/P1P2PPP/RN1QK2R b KQkq - 0 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'd6e5' => 'r1bqk2r/ppp2ppp/1b5B/nP2p3/2BPP3/8/P1P2PPP/RN1QK2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/ppp2ppp/1b1p3n/nP2N3/2BPP3/8/P1P2PPP/RNBQK2R w KQkq - 0 8' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'c1h6' => 'r1bqk2r/ppp2ppp/1b1p3B/nP2N3/2BPP3/8/P1P2PPP/RN1QK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/ppp2ppp/1b5B/nP2p3/2BPP3/8/P1P2PPP/RN1QK2R w KQkq - 0 9' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'h6g7' => 'r1bqk2r/ppp2pBp/1b6/nP2p3/2BPP3/8/P1P2PPP/RN1QK2R b KQkq - 0 9',
			},
		},
		'r1bqk2r/ppp2ppp/2n1p3/3n4/QbpP4/5NP1/PP1BPPBP/RN2K2R w KQkq - 6 8' => {
			eco => 'E04x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6 6. Qa4 Bb4+ 7. Bd2 Nd5

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6. Qa4 Bb4+ 7. Bd2 Nd5'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n1p3/3p4/2PPn3/P3P3/1PQ2PPP/R1B1KBNR b KQkq - 0 8' => {
			eco => 'E37',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line, 7. Qc2 Nc6'),
			moves => {
				'e6e5' => 'r1bqk2r/ppp2ppp/2n5/3pp3/2PPn3/P3P3/1PQ2PPP/R1B1KBNR w KQkq - 0 9',
			},
		},
		'r1bqk2r/ppp2ppp/2n1p3/3p4/2PPn3/P7/1PQ1PPPP/R1B1KBNR w KQkq - 3 8' => {
			eco => 'E37',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Ne4 7. Qc2 Nc6

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line, 7. Qc2 Nc6'),
			moves => {
				'e2e3' => 'r1bqk2r/ppp2ppp/2n1p3/3p4/2PPn3/P3P3/1PQ2PPP/R1B1KBNR b KQkq - 0 8',
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/3P4/Qb1P4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6' => {
			eco => 'D38c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Qa4+ Nc6 6. cxd5

			variation => N__('QGD: Ragozin, 5. Qa4+ Nc6 6.cxd5'),
			moves => {
				'e6d5' => 'r1bqk2r/ppp2ppp/2n2n2/3p4/Qb1P4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7',
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/3p4/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq d6 0 6' => {
			eco => 'E40',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 Nc6 5. Ne2 d5

			variation => N__('Nimzo-Indian: Taimanov, 5. Ne2 d5'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/3p4/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq d6 0 6' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3 d5

			variation => N__('Nimzo-Indian: Classical, 4... Nc6 5. Nf3 d5'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/3p4/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 1 7' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Nc6

			variation => N__('Nimzo-Indian: Classical, Noa, Botvinnik Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/3p4/QbPP4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 6 6' => {
			eco => 'D38b',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, 5. Qa4+'),
			moves => {
				'c4d5' => 'r1bqk2r/ppp2ppp/2n1pn2/3P4/Qb1P4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6',
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/8/QbpP4/5NP1/PP1BPPBP/RN2K2R b KQkq - 5 7' => {
			eco => 'E04w',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6. Qa4 Bb4+'),
			moves => {
				'f6d5' => 'r1bqk2r/ppp2ppp/2n1p3/3n4/QbpP4/5NP1/PP1BPPBP/RN2K2R w KQkq - 6 8',
			},
		},
		'r1bqk2r/ppp2ppp/2n1pn2/8/QbpP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 4 7' => {
			eco => 'E04w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6 6. Qa4 Bb4+

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6. Qa4 Bb4+'),
			moves => {
				'c1d2' => 'r1bqk2r/ppp2ppp/2n1pn2/8/QbpP4/5NP1/PP1BPPBP/RN2K2R b KQkq - 5 7',
			},
		},
		'r1bqk2r/ppp2ppp/2n2P2/2b5/2pp4/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'f1e1' => 'r1bqk2r/ppp2ppp/2n2P2/2b5/2pp4/5N2/PPP2PPP/RNBQR1K1 b kq - 1 8',
			},
		},
		'r1bqk2r/ppp2ppp/2n2P2/2b5/2pp4/5N2/PPP2PPP/RNBQR1K1 b kq - 1 8' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'c8e6' => 'r2qk2r/ppp2ppp/2n1bP2/2b5/2pp4/5N2/PPP2PPP/RNBQR1K1 w kq - 2 9',
			},
		},
		'r1bqk2r/ppp2ppp/2n2P2/2bp4/2Bp4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'd5c4' => 'r1bqk2r/ppp2ppp/2n2P2/2b5/2pp4/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/ppp2ppp/2n2n2/1BbpP3/3p4/2P2N2/PP3PPP/RNBQK2R b KQkq - 1 7' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5 d5'),
			moves => {
				'f6e4' => 'r1bqk2r/ppp2ppp/2n5/1BbpP3/3pn3/2P2N2/PP3PPP/RNBQK2R w KQkq - 2 8',
			},
		},
		'r1bqk2r/ppp2ppp/2n2n2/2bpP3/2Bp4/2P2N2/PP3PPP/RNBQK2R w KQkq d6 0 7' => {
			eco => 'C54k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. e5 d5

			variation => N__('Giuoco Piano: 6. e5 d5'),
			moves => {
				'c4b5' => 'r1bqk2r/ppp2ppp/2n2n2/1BbpP3/3p4/2P2N2/PP3PPP/RNBQK2R b KQkq - 1 7',
			},
		},
		'r1bqk2r/ppp2ppp/2n2n2/2bpP3/2Bp4/5N2/PPP2PPP/RNBQ1RK1 w kq d6 0 7' => {
			eco => 'C55w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 d5

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'e5f6' => 'r1bqk2r/ppp2ppp/2n2P2/2bp4/2Bp4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/ppp2ppp/2n2n2/3p2B1/Qb1P4/2N2N2/PP2PPPP/R3KB1R b KQkq - 1 7' => {
			eco => 'D38d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Qa4+ Nc6 6. cxd5 exd5 7. Bg5

			variation => N__('QGD: Ragozin, 5. Qa4+ Nc6 6.cxd5 exd5 7. Bg5'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n2n2/3p4/Qb1P4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'D38c',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, 5. Qa4+ Nc6 6.cxd5'),
			moves => {
				'c1g5' => 'r1bqk2r/ppp2ppp/2n2n2/3p2B1/Qb1P4/2N2N2/PP2PPPP/R3KB1R b KQkq - 1 7',
			},
		},
		'r1bqk2r/ppp2ppp/2n5/1B1pP3/1b1Pn3/5N2/PP3PPP/RNBQK2R w KQkq - 1 9' => {
			eco => 'C54k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. e5 d5 7. Bb5 Ne4 8. cxd4 Bb4+

			variation => N__('Giuoco Piano: Anderssen Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n5/1BbpP3/3Pn3/5N2/PP3PPP/RNBQK2R b KQkq - 0 8' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5 d5'),
			moves => {
				'c5b4' => 'r1bqk2r/ppp2ppp/2n5/1B1pP3/1b1Pn3/5N2/PP3PPP/RNBQK2R w KQkq - 1 9',
			},
		},
		'r1bqk2r/ppp2ppp/2n5/1BbpP3/3pn3/2P2N2/PP3PPP/RNBQK2R w KQkq - 2 8' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5 d5'),
			moves => {
				'c3d4' => 'r1bqk2r/ppp2ppp/2n5/1BbpP3/3Pn3/5N2/PP3PPP/RNBQK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/ppp2ppp/2n5/3np1N1/1bBP4/8/PPP2PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C57f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nxd5 6. d4 Bb4+

			variation => N__('Two Knights: Lolli Attack, Pinkus Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n5/3p4/2BP4/1Qb2N2/P4PPP/R1B2RK1 w kq d6 0 11' => {
			eco => 'C54n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Nxc3 9. bxc3 Bxc3 10. Qb3 d5

			variation => N__('Giuoco Piano: Bernstein Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n5/3p4/2BPn3/2P2N2/P4PPP/R1BQ1RK1 w kq d6 0 10' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'c1a3' => 'r1bqk2r/ppp2ppp/2n5/3p4/2BPn3/B1P2N2/P4PPP/R2Q1RK1 b kq - 1 10',
			},
		},
		'r1bqk2r/ppp2ppp/2n5/3p4/2BPn3/B1P2N2/P4PPP/R2Q1RK1 b kq - 1 10' => {
			eco => 'C54o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Bxc3 9. bxc3 d5 10. Ba3

			variation => N__('Giuoco Piano: Steinitz Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2n5/3pp3/2PPn3/P3P3/1PQ2PPP/R1B1KBNR w KQkq - 0 9' => {
			eco => 'E37',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Ne4 7. Qc2 Nc6 8. e3 e5

			variation => N__('Nimzo-Indian: Classical, Noa, San Remo Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p1B1/2B1P3/2NP1N2/PPP2PPP/R2QK2R b KQkq - 1 6' => {
			eco => 'C50u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. Nc3 d6 6. Bg5

			variation => N__('Giuoco Pianissimo: Canal Variation'),
			moves => {
				'h7h6' => 'r1bqk2r/ppp2pp1/2np1n1p/2b1p1B1/2B1P3/2NP1N2/PPP2PPP/R2QK2R w KQkq - 0 7',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2NP1N2/PPP2PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C50s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. Nc3 d6

			variation => N__('Giuoco Pianissimo: Italian Four Knights, 5... d6'),
			moves => {
				'c1e3' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2NPBN2/PPP2PPP/R2QK2R b KQkq - 1 6',
				'c1g5' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p1B1/2B1P3/2NP1N2/PPP2PPP/R2QK2R b KQkq - 1 6',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2NPBN2/PPP2PPP/R2QK2R b KQkq - 1 6' => {
			eco => 'C50t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. Nc3 d6 6. Be3

			variation => N__('Giuoco Pianissimo: Italian Four Knights, 5... d6 6. Be3'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP1N1PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C54g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d3 d6 6. Nbd2

			variation => N__('Giuoco Pianissimo: 5. d3 d6 6.Nbd2'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'C54h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d3 d6 6. O-O

			variation => N__('Giuoco Pianissimo: 5. d3 d6 6.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQ1RK1 w - - 2 7',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C54f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d3 d6

			variation => N__('Giuoco Pianissimo: 5. d3 d6'),
			moves => {
				'b1d2' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP1N1PPP/R1BQK2R b KQkq - 1 6',
				'e1g1' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C50q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. O-O d6

			variation => N__('Giuoco Pianissimo: 4. d3 Nf6 5.O-O d6'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2N2N2/PPPP2PP/R1BQK2R w KQkq - 4 6' => {
			eco => 'C30m',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'd2d3' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1N2/PPP3PP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1N2/PPP3PP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C30m',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'c8g4' => 'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PPb1/2NP1N2/PPP3PP/R1BQK2R w KQkq - 1 7',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1N2/PPP3PP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C28j',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bc5 5. f4 d6 6. Nf3

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5 5. f4 d6 6. Nf3'),
			moves => {
				'a7a6' => 'r1bqk2r/1pp2ppp/p1np1n2/2b1p3/2B1PP2/2NP1N2/PPP3PP/R1BQK2R w KQkq - 0 7',
				'c8g4' => 'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PPb1/2NP1N2/PPP3PP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP4/PPP3PP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'C28i',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5 5. f4'),
			moves => {
				'g1f3' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1N2/PPP3PP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bc5 5. Bg2 d6 6. O-O

			variation => N__('English: Four Knights, 4. g3 Bc5'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp2ppp/2np1n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 2 7',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6' => {
			eco => 'A29',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. g3 Bc5'),
			moves => {
				'e1g1' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/4P3/2N2NP1/PPPP1PBP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C47g',
                        # TRANSLATORS:

			variation => N__('Four Knights: Glek, 4... Bc5'),
			moves => {
				'd2d3' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/4P3/2NP1NP1/PPP2PBP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqk2r/ppp2ppp/2np1n2/2b1p3/4P3/2NP1NP1/PPP2PBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C47g',
                        # TRANSLATORS:

			variation => N__('Four Knights: Glek, 4... Bc5'),
			moves => {
				'a7a6' => 'r1bqk2r/1pp2ppp/p1np1n2/2b1p3/4P3/2NP1NP1/PPP2PBP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqk2r/ppp2ppp/2nppn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 0 6' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3 d6

			variation => N__('Nimzo-Indian: Classical, Milner-Barry (Zurich) Variation'),
			moves => {
				'a2a3' => 'r1bqk2r/ppp2ppp/2nppn2/8/1bPP4/P1N2N2/1PQ1PPPP/R1B1KB1R b KQkq - 0 6',
				'c1d2' => 'r1bqk2r/ppp2ppp/2nppn2/8/1bPP4/2N2N2/PPQBPPPP/R3KB1R b KQkq - 1 6',
			},
		},
		'r1bqk2r/ppp2ppp/2nppn2/8/1bPP4/2N2N2/PPQBPPPP/R3KB1R b KQkq - 1 6' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3 d6 6. Bd2

			variation => N__('Nimzo-Indian: Classical, Milner-Barry (Zurich), 6. Bd2'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppp2ppp/2nppn2/8/1bPP4/2N2N2/PPQBPPPP/R3KB1R w KQ - 2 7',
			},
		},
		'r1bqk2r/ppp2ppp/2nppn2/8/1bPP4/P1N2N2/1PQ1PPPP/R1B1KB1R b KQkq - 0 6' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3 d6 6. a3

			variation => N__('Nimzo-Indian: Classical, Milner-Barry (Zurich), 6. a3'),
			moves => {
				'b4c3' => 'r1bqk2r/ppp2ppp/2nppn2/8/2PP4/P1b2N2/1PQ1PPPP/R1B1KB1R w KQkq - 0 7',
			},
		},
		'r1bqk2r/ppp2ppp/2nppn2/8/2PP4/P1Q2N2/1P2PPPP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3 d6 6. a3 Bxc3+ 7. Qxc3

			variation => N__('Nimzo-Indian: Classical, Milner-Barry (Zurich), 6. a3 Bxc3+ 7.Qxc3'),
			moves => {
			},
		},
		'r1bqk2r/ppp2ppp/2nppn2/8/2PP4/P1b2N2/1PQ1PPPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'E33',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, Milner-Barry (Zurich), 6. a3'),
			moves => {
				'c2c3' => 'r1bqk2r/ppp2ppp/2nppn2/8/2PP4/P1Q2N2/1P2PPPP/R1B1KB1R b KQkq - 0 7',
			},
		},
		'r1bqk2r/ppp2ppp/3p1n2/4p1B1/2BnP3/8/PPP2PPP/RN1Q1RK1 w kq - 0 8' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'f2f4' => 'r1bqk2r/ppp2ppp/3p1n2/4p1B1/2BnPP2/8/PPP3PP/RN1Q1RK1 b kq f3 0 8',
			},
		},
		'r1bqk2r/ppp2ppp/3p1n2/4p1B1/2BnPP2/8/PPP3PP/RN1Q1RK1 b kq f3 0 8' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'd8e7' => 'r1b1k2r/ppp1qppp/3p1n2/4p1B1/2BnPP2/8/PPP3PP/RN1Q1RK1 w kq - 1 9',
			},
		},
		'r1bqk2r/pppn1pbp/3p1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 g6 6. Bg2 Bg7

			variation => N__('Old Indian: 5. g3 g6'),
			moves => {
			},
		},
		'r1bqk2r/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQkq e6 0 7' => {
			eco => 'E90a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 Nbd7 6. Be2 e5

			variation => N__('King\'s Indian: 5. Nf3 Nbd7 6. Be2 e5'),
			moves => {
			},
		},
		'r1bqk2r/pppn1ppp/4pn2/3P2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D38i',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, Westphalia Variation'),
			moves => {
				'e6d5' => 'r1bqk2r/pppn1ppp/5n2/3p2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'r1bqk2r/pppn1ppp/4pn2/3p2B1/1bPP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 Bb4

			variation => N__('QGD: Manhattan Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppn1ppp/4pn2/3p2B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 6 6' => {
			eco => 'D38i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 Nbd7

			variation => N__('QGD: Ragozin, Westphalia Variation'),
			moves => {
				'c4d5' => 'r1bqk2r/pppn1ppp/4pn2/3P2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'r1bqk2r/pppn1ppp/5n2/3p2B1/1b1P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D38k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 Nbd7 6. cxd5 exd5 7. e3

			variation => N__('QGD: Ragozin, Westphalia, 7. e3'),
			moves => {
			},
		},
		'r1bqk2r/pppn1ppp/5n2/3p2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D38i',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, Westphalia Variation'),
			moves => {
				'd1c2' => 'r1bqk2r/pppn1ppp/5n2/3p2B1/1b1P4/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 1 7',
				'e2e3' => 'r1bqk2r/pppn1ppp/5n2/3p2B1/1b1P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1bqk2r/pppn1ppp/5n2/3p2B1/1b1P4/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 1 7' => {
			eco => 'D38j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 Nbd7 6. cxd5 exd5 7. Qc2

			variation => N__('QGD: Ragozin, Westphalia, 7. Qc2'),
			moves => {
			},
		},
		'r1bqk2r/pppnbBpp/3p1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. Bxf7+

			variation => N__('Philidor: Improved Hanham, 6.Bxf7+'),
			moves => {
			},
		},
		'r1bqk2r/pppnbppp/2n1p3/3pP3/3P4/1N3N2/PPP2PPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'C04j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. Nb3 Be7

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. Nb3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4P3/2B1P3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. dxe5

			variation => N__('Philidor: Improved Hanham, 6.dxe5'),
			moves => {
				'd6e5' => 'r1bqk2r/pppnbppp/5n2/4p3/2B1P3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 7',
				'd7e5' => 'r1bqk2r/ppp1bppp/3p1n2/4n3/2B1P3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p1B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 Be7 6. e3

			variation => N__('Old Indian: 5. Bg5 Be7 6. e3'),
			moves => {
				'c7c6' => 'r1bqk2r/pp1nbppp/2pp1n2/4p1B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 7',
				'e8g8' => 'r1bq1rk1/pppnbppp/3p1n2/4p1B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p1B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 Be7

			variation => N__('Old Indian: 5. Bg5 Be7'),
			moves => {
				'e2e3' => 'r1bqk2r/pppnbppp/3p1n2/4p1B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p1N1/2BPP3/2N5/PPP2PPP/R1BQK2R b KQkq - 6 6' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. Ng5

			variation => N__('Philidor: Improved Hanham, 6. Ng5'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppnbppp/3p1n2/4p1N1/2BPP3/2N5/PPP2PPP/R1BQK2R w KQ - 7 7',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 6 6' => {
			eco => 'C41t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. O-O

			variation => N__('Philidor: Improved Hanham 6.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQ1RK1 w - - 7 7',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 5 6' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7

			variation => N__('Philidor: Improved Hanham, 5. Bc4 Be7'),
			moves => {
				'c4f7' => 'r1bqk2r/pppnbBpp/3p1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 0 6',
				'd4e5' => 'r1bqk2r/pppnbppp/3p1n2/4P3/2B1P3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 0 6',
				'e1g1' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 6 6',
				'f3g5' => 'r1bqk2r/pppnbppp/3p1n2/4p1N1/2BPP3/2N5/PPP2PPP/R1BQK2R b KQkq - 6 6',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 Be7

			variation => N__('Old Indian: 5. g3 Be7'),
			moves => {
				'f1g2' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 6',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 Be7 6. Bg2

			variation => N__('Old Indian: 5. g3 Be7'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 7',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'A55i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2

			variation => N__('Old Indian: 5. e4 Be7 6. Be2'),
			moves => {
				'c7c6' => 'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQkq - 0 7',
				'e8g8' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 3 7',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'A55d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7

			variation => N__('Old Indian: 5. e4 Be7'),
			moves => {
				'f1e2' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 2 6',
				'g2g3' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2PPP3/2N2NP1/PP3P1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqk2r/pppnbppp/3p1n2/4p3/2PPP3/2N2NP1/PP3P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A55e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3

			variation => N__('Old Indian: 5. e4 Be7 6. g3'),
			moves => {
				'c7c6' => 'r1bqk2r/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3P1P/R1BQKB1R w KQkq - 0 7',
				'e8g8' => 'r1bq1rk1/pppnbppp/3p1n2/4p3/2PPP3/2N2NP1/PP3P1P/R1BQKB1R w KQ - 1 7',
			},
		},
		'r1bqk2r/pppnbppp/4pN2/6B1/3P4/5N2/PPP2PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'C11r',
                        # TRANSLATORS:

			variation => N__('French: Burn, 5.Nxe4 Nbd7 6. Nf3 Be7'),
			moves => {
				'd7f6' => 'r1bqk2r/ppp1bppp/4pn2/6B1/3P4/5N2/PPP2PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pppnbppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'D53r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 Nbd7

			variation => N__('QGD: 4. Bg5 Be7 5. e3 Nbd7'),
			moves => {
				'g1f3' => 'r1bqk2r/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 2 6',
			},
		},
		'r1bqk2r/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 2 6' => {
			eco => 'D53s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 Nbd7 6. Nf3

			variation => N__('QGD: 4. Bg5 Be7 5. e3 Nbd7 6. Nf3'),
			moves => {
			},
		},
		'r1bqk2r/pppnbppp/4pn2/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQkq - 3 6' => {
			eco => 'D03f',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3 Nbd7 5. Nbd2 Be7

			variation => N__('Torre Attack: 3... e6 4. e3 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pppnbppp/4pn2/6B1/3PN3/5N2/PPP2PPP/R2QKB1R w KQkq - 3 7' => {
			eco => 'C11r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Nbd7 6. Nf3 Be7

			variation => N__('French: Burn, 5.Nxe4 Nbd7 6. Nf3 Be7'),
			moves => {
				'e4f6' => 'r1bqk2r/pppnbppp/4pN2/6B1/3P4/5N2/PPP2PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1bqk2r/pppnbppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D35j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Be7 6. e3 Nbd7

			variation => N__('QGD: Exchange, 5. Bg5 Be7 6. e3 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pppnbppp/5n2/4p3/2B1P3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4 Be7 6. dxe5 dxe5

			variation => N__('Philidor: Improved Hanham, 6.dxe5 dxe5'),
			moves => {
			},
		},
		'r1bqk2r/pppnnpbp/3p2p1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQkq - 4 7' => {
			eco => 'A42s',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. Nf3 Nd7 6. Be2 Ne7

			variation => N__('Modern: Averbakh, 4... e5 5. Nf3 Nd7 6. Be2 Ne7'),
			moves => {
			},
		},
		'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'E90a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 Nbd7 6. Be2

			variation => N__('King\'s Indian: 5. Nf3 Nbd7 6. Be2'),
			moves => {
				'e7e5' => 'r1bqk2r/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQkq e6 0 7',
			},
		},
		'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E90a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 Nbd7

			variation => N__('King\'s Indian: 5. Nf3 Nbd7'),
			moves => {
				'f1e2' => 'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'E80d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 Nbd7

			variation => N__('King\'s Indian: Sämisch, 5...Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N4P/PP3PP1/R1BQKBNR w KQkq - 1 6' => {
			eco => 'E71c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 Nbd7

			variation => N__('King\'s Indian: Makagonov, 5...Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'E73e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 Nbd7

			variation => N__('King\'s Indian: 5. Be2 Nbd7'),
			moves => {
			},
		},
		'r1bqk2r/pppp1Bpp/2n2n2/2b1p1N1/4P3/8/PPPP1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C57d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Bxf7+

			variation => N__('Two Knights: Traxler, 5.Bxf7+'),
			moves => {
				'e8e7' => 'r1bq3r/ppppkBpp/2n2n2/2b1p1N1/4P3/8/PPPP1PPP/RNBQK2R w KQ - 1 6',
			},
		},
		'r1bqk2r/pppp1Bpp/8/8/1n1Pn3/5N2/PP3PPP/RN1QK2R b KQkq - 0 9' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'e8f7' => 'r1bq3r/pppp1kpp/8/8/1n1Pn3/5N2/PP3PPP/RN1QK2R w KQ - 0 10',
			},
		},
		'r1bqk2r/pppp1Npp/2n2n2/2b1p3/2B1P3/8/PPPP1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C57b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7

			variation => N__('Two Knights: Traxler, 5.Nxf7'),
			moves => {
				'c5f2' => 'r1bqk2r/pppp1Npp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1Npp/2n2n2/4p3/2B1P3/8/PPPP1KPP/RNBQ3R b kq - 0 6' => {
			eco => 'C57b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7 Bxf2+ 6. Kxf2

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6.Kxf2'),
			moves => {
			},
		},
		'r1bqk2r/pppp1Npp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQ1K1R b kq - 1 6' => {
			eco => 'C57c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7 Bxf2+ 6. Kf1

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1'),
			moves => {
				'd8e7' => 'r1b1k2r/ppppqNpp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQ1K1R w kq - 2 7',
			},
		},
		'r1bqk2r/pppp1Npp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C57b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Traxler, 5.Nxf7'),
			moves => {
				'e1f1' => 'r1bqk2r/pppp1Npp/2n2n2/4p3/2B1P3/8/PPPP1bPP/RNBQ1K1R b kq - 1 6',
				'e1f2' => 'r1bqk2r/pppp1Npp/2n2n2/4p3/2B1P3/8/PPPP1KPP/RNBQ3R b kq - 0 6',
			},
		},
		'r1bqk2r/pppp1p2/5n1p/4p1p1/2BnP2B/8/PPP2PPP/RN1Q1RK1 w kq g6 0 9' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'f2f4' => 'r1bqk2r/pppp1p2/5n1p/4p1p1/2BnPP1B/8/PPP3PP/RN1Q1RK1 b kq f3 0 9',
			},
		},
		'r1bqk2r/pppp1p2/5n1p/4p1p1/2BnPP1B/8/PPP3PP/RN1Q1RK1 b kq f3 0 9' => {
			eco => 'C55b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. O-O Bc5 5. d4 Bxd4 6. Nxd4 Nxd4 7. Bg5 h6 8. Bh4 g5 9. f4

			variation => N__('Two Knights: 4.O-O Gambit, Rosentreter Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1pbp/2n2np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 3 7' => {
			eco => 'C46m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6 4. d4 exd4 5. Nxd4 Bg7 6. Be3 Nf6

			variation => N__('Three Knights: Steinitz, 4. d4 exd4 5.Nxd4 Bg7 6. Be3 Nf6'),
			moves => {
			},
		},
		'r1bqk2r/pppp1pbp/2n3pn/4p3/2P5/2N3P1/PP1PPPBP/1RBQK1NR w Kkq - 4 6' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Rb1 Nh6

			variation => N__('English: Closed, 5. Rb1, Taimanov Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1pp1/2n2n1p/6B1/1bPN4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'g5h4' => 'r1bqk2r/pppp1pp1/2n2n1p/8/1bPN3B/2N5/PP2PPPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'r1bqk2r/pppp1pp1/2n2n1p/8/1bPN3B/2N5/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'b4c3' => 'r1bqk2r/pppp1pp1/2n2n1p/8/2PN3B/2b5/PP2PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1pp1/2n2n1p/8/2PN3B/2P5/P3PPPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'c6e5' => 'r1bqk2r/pppp1pp1/5n1p/4n3/2PN3B/2P5/P3PPPP/R2QKB1R w KQkq - 1 9',
			},
		},
		'r1bqk2r/pppp1pp1/2n2n1p/8/2PN3B/2b5/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'b2c3' => 'r1bqk2r/pppp1pp1/2n2n1p/8/2PN3B/2P5/P3PPPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1pp1/5n1p/4n3/2PN3B/2P5/P3PPPP/R2QKB1R w KQkq - 1 9' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. d4 exd4 5. Nxd4 Bb4 6. Bg5 h6 7. Bh4 Bxc3+ 8. bxc3 Ne5

			variation => N__('English: Four Knights, Nenarokov Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1pp1/5n1p/4p1B1/2BnP3/8/PPP2PPP/RN1Q1RK1 w kq - 0 8' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'g5h4' => 'r1bqk2r/pppp1pp1/5n1p/4p3/2BnP2B/8/PPP2PPP/RN1Q1RK1 b kq - 1 8',
			},
		},
		'r1bqk2r/pppp1pp1/5n1p/4p3/2BnP2B/8/PPP2PPP/RN1Q1RK1 b kq - 1 8' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'g7g5' => 'r1bqk2r/pppp1p2/5n1p/4p1p1/2BnP2B/8/PPP2PPP/RN1Q1RK1 w kq g6 0 9',
			},
		},
		'r1bqk2r/pppp1ppp/1b5n/nP2N3/2B1P3/8/P1PP1PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bb6 5. b5 Na5 6. Nxe5 Nh6

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'd2d4' => 'r1bqk2r/pppp1ppp/1b5n/nP2N3/2BPP3/8/P1P2PPP/RNBQK2R b KQkq d3 0 7',
			},
		},
		'r1bqk2r/pppp1ppp/1b5n/nP2N3/2BPP3/8/P1P2PPP/RNBQK2R b KQkq d3 0 7' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit Declined, Lange Variation'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/1b1p3n/nP2N3/2BPP3/8/P1P2PPP/RNBQK2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/1bn2n2/1B2p3/3PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C64j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Nf6 5. d4 Bb6

			variation => N__('Spanish: Classical, 4. c3 Nf6 5. d4 Bb6'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/1bn2n2/4p3/1PB1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C54c',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Bird\'s Attack'),
			moves => {
				'a2a4' => 'r1bqk2r/pppp1ppp/1bn2n2/4p3/PPB1P3/2P2N2/3P1PPP/RNBQK2R b KQkq a3 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/1bn2n2/4p3/PPB1P3/2P2N2/3P1PPP/RNBQK2R b KQkq a3 0 6' => {
			eco => 'C54c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. b4 Bb6 6. a4

			variation => N__('Giuoco Piano: Bird\'s Attack'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2N2n2/8/1b2P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'C47r',
                        # TRANSLATORS:

			variation => N__('Four Knights: Scotch, 4...exd4 5.Nxd4 Bb4'),
			moves => {
				'b7c6' => 'r1bqk2r/p1pp1ppp/2p2n2/8/1b2P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/1QN5/PP2PPPP/R1B1KBNR w KQkq - 4 5' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 Nc6

			variation => N__('Nimzo-Indian: Spielmann, 4... Nc6'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'E40',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 Nc6 5. Ne2

			variation => N__('Nimzo-Indian: Taimanov, 5. Ne2'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp2ppp/2n1pn2/3p4/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq d6 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'E40',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 Nc6

			variation => N__('Nimzo-Indian: Taimanov'),
			moves => {
				'f1d3' => 'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 2 5',
				'g1e2' => 'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 5 5' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6 5. Nf3

			variation => N__('Nimzo-Indian: Classical, 4... Nc6 5. Nf3'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp2ppp/2n1pn2/3p4/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq d6 0 6',
				'd7d6' => 'r1bqk2r/ppp2ppp/2nppn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 4 5' => {
			eco => 'E33',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 Nc6

			variation => N__('Nimzo-Indian: Classical, 4... Nc6'),
			moves => {
				'g1f3' => 'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 5 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'E40',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 Nc6 5. Bd3

			variation => N__('Nimzo-Indian: Taimanov, 5. Bd3'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2b2/3P4/2B1n3/5N2/PP3PPP/R1BQ1RK1 w kq - 1 10' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'f1e1' => 'r1bqk2r/pppp1ppp/2n2b2/3P4/2B1n3/5N2/PP3PPP/R1BQR1K1 b kq - 2 10',
			},
		},
		'r1bqk2r/pppp1ppp/2n2b2/3P4/2B1n3/5N2/PP3PPP/R1BQR1K1 b kq - 2 10' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'c6e7' => 'r1bqk2r/ppppnppp/5b2/3P4/2B1n3/5N2/PP3PPP/R1BQR1K1 w kq - 3 11',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1B2p3/1b2P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5' => {
			eco => 'C49b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O

			variation => N__('Four Knights: 4. Bb5 Bb4 5.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/1B2p3/1b2P3/2N2N2/PPPP1PPP/R1BQ1RK1 w - - 8 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1B2p3/1b2P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5' => {
			eco => 'C49a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4

			variation => N__('Four Knights: 4. Bb5 Bb4'),
			moves => {
				'e1g1' => 'r1bqk2r/pppp1ppp/2n2n2/1B2p3/1b2P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1N3/4P3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 5' => {
			eco => 'C48e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bc5 5. Nxe5

			variation => N__('Four Knights: Spanish, Classical Defence, 5.Nxe5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1N3/4P3/8/PPPP1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'C65q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Bc5 5. Nxe5

			variation => N__('Spanish: Berlin, Beverwijk, 5.Nxe5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/3PP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C64i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Nf6 5. d4

			variation => N__('Spanish: Classical, 4. c3 Nf6 5. d4'),
			moves => {
				'c5b6' => 'r1bqk2r/pppp1ppp/1bn2n2/1B2p3/3PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5' => {
			eco => 'C48f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bc5 5. O-O

			variation => N__('Four Knights: Spanish, Classical Defence, 5.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 w - - 8 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5' => {
			eco => 'C48d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bc5

			variation => N__('Four Knights: Spanish, Classical Defence'),
			moves => {
				'e1g1' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5',
				'f3e5' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1N3/4P3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'C65n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Bc5 5. c3

			variation => N__('Spanish: Berlin, Beverwijk, 5. c3'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w - - 1 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C64h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Nf6

			variation => N__('Spanish: Classical, 4. c3 Nf6'),
			moves => {
				'd2d4' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/3PP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C65j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 Bc5

			variation => N__('Spanish: Berlin, 4. d3 Bc5'),
			moves => {
				'c1e3' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/3PBN2/PPP2PPP/RN1QK2R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/3PBN2/PPP2PPP/RN1QK2R b KQkq - 2 5' => {
			eco => 'C65j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 Bc5 5. Be3

			variation => N__('Spanish: Berlin, Kaufmann Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5' => {
			eco => 'C65m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Bc5

			variation => N__('Spanish: Berlin, Beverwijk Variation'),
			moves => {
				'c2c3' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5',
				'f3e5' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1N3/4P3/8/PPPP1PPP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1P3/2Bp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C54k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. e5

			variation => N__('Giuoco Piano: 6. e5'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp2ppp/2n2n2/2bpP3/2Bp4/2P2N2/PP3PPP/RNBQK2R w KQkq d6 0 7',
				'f6e4' => 'r1bqk2r/pppp1ppp/2n5/2b1P3/2Bpn3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1P3/2Bp4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C55u',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange Attack'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp2ppp/2n2n2/2bpP3/2Bp4/5N2/PPP2PPP/RNBQ1RK1 w kq d6 0 7',
				'f6g4' => 'r1bqk2r/pppp1ppp/2n5/2b1P3/2Bp2n1/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p1B1/2B1P3/2NP4/PPP2PPP/R2QK1NR b KQkq - 2 5' => {
			eco => 'C28h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bc5 5. Bg5

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5 5. Bg5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p1N1/2B1P3/8/PPPP1PPP/RNBQK2R w KQkq - 6 5' => {
			eco => 'C57a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5

			variation => N__('Two Knights: Traxler (Wilkes-Barre) Variation'),
			moves => {
				'c4f7' => 'r1bqk2r/pppp1Bpp/2n2n2/2b1p1N1/4P3/8/PPPP1PPP/RNBQK2R b KQkq - 0 5',
				'd2d4' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p1N1/2BPP3/8/PPP2PPP/RNBQK2R b KQkq d3 0 5',
				'g5f7' => 'r1bqk2r/pppp1Npp/2n2n2/2b1p3/2B1P3/8/PPPP1PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p1N1/2BPP3/8/PPP2PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C57a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. d4

			variation => N__('Two Knights: Traxler, 5. d4'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/1PB1P3/2P2N2/P2P1PPP/RNBQK2R b KQkq b3 0 5' => {
			eco => 'C54c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. b4

			variation => N__('Giuoco Piano: Bird\'s Attack'),
			moves => {
				'c5b6' => 'r1bqk2r/pppp1ppp/1bn2n2/4p3/1PB1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5' => {
			eco => 'C50m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. O-O Nf6 5. Nc3

			variation => N__('Giuoco Piano: 4.O-O Nf6 5. Nc3'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5' => {
			eco => 'C50j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. Nc3 Nf6

			variation => N__('Giuoco Piano: Four Knights Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2NP1N2/PPP2PPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'C50r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. Nc3

			variation => N__('Giuoco Pianissimo: Italian Four Knights'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2NP1N2/PPP2PPP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5' => {
			eco => 'C28g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bc5

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5'),
			moves => {
				'c1g5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p1B1/2B1P3/2NP4/PPP2PPP/R2QK1NR b KQkq - 2 5',
				'f2f4' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1PP2/2NP4/PPP3PP/R1BQK1NR b KQkq f3 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 2 5' => {
			eco => 'C54b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. O-O

			variation => N__('Giuoco Piano: Albin Gambit'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C54a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6

			variation => N__('Giuoco Piano: 4. c3 Nf6'),
			moves => {
				'b2b4' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/1PB1P3/2P2N2/P2P1PPP/RNBQK2R b KQkq b3 0 5',
				'd2d3' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 5',
				'd2d4' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5',
				'e1g1' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2P5/PP1PQPPP/RNB1K1NR w KQkq - 1 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. Qe2'),
			moves => {
				'f2f4' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1PP2/2P5/PP1PQ1PP/RNB1K1NR b KQkq f3 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C54d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d3

			variation => N__('Giuoco Pianissimo: 5. d3'),
			moves => {
				'a7a6' => 'r1bqk2r/1ppp1ppp/p1n2n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R w KQkq - 0 6',
				'd7d6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 2 5' => {
			eco => 'C50p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6 5. O-O

			variation => N__('Giuoco Pianissimo: 4. d3 Nf6 5.O-O'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C50o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. d3 Nf6

			variation => N__('Giuoco Pianissimo: 4. d3 Nf6'),
			moves => {
				'b1c3' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2NP1N2/PPP2PPP/R1BQK2R b KQkq - 2 5',
				'e1g1' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5' => {
			eco => 'C50l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. O-O Nf6

			variation => N__('Giuoco Piano: 4.O-O Nf6'),
			moves => {
				'b1c3' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5',
				'd2d4' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2BPP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1PP2/2NP4/PPP3PP/R1BQK1NR b KQkq f3 0 5' => {
			eco => 'C28i',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bc5 5. f4

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5 5. f4'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP4/PPP3PP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1PP2/2P5/PP1PQ1PP/RNB1K1NR b KQkq f3 0 5' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. Qe2 Nc6 4. c3 Nf6 5. f4

			variation => N__('Bishop\'s Opening: Lopez Gambit'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C54j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4

			variation => N__('Giuoco Piano: 5. d4'),
			moves => {
				'e5d4' => 'r1bqk2r/pppp1ppp/2n2n2/2b5/2BpP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2BPP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 5' => {
			eco => 'C55b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. O-O Bc5 5. d4

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'c5d4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/2BbP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 Nf6 4. Bg2 Bc5 5. e3

			variation => N__('English: Closed'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bc5

			variation => N__('English: Four Knights, 4. g3 Bc5'),
			moves => {
				'f1g2' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bc5 5. Bg2

			variation => N__('English: Four Knights, 4. g3 Bc5'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 3 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 Nf6 4. Bg2 Bc5

			variation => N__('English: Closed'),
			moves => {
				'e2e3' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N2NP1/PPPP1P1P/R1BQKB1R w KQkq - 1 5' => {
			eco => 'C47g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. g3 Bc5

			variation => N__('Four Knights: Glek, 4... Bc5'),
			moves => {
				'f1g2' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N2NP1/PPPP1PBP/R1BQK2R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N2NP1/PPPP1PBP/R1BQK2R b KQkq - 2 5' => {
			eco => 'C47g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. g3 Bc5 5. Bg2

			variation => N__('Four Knights: Glek, 4... Bc5'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/4P3/2N2NP1/PPPP1PBP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQkq - 3 5' => {
			eco => 'C26h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Bc5 4. Bg2 Nc6

			variation => N__('Vienna: Smyslov, 3... Bc5 4. Bg2 Nc6'),
			moves => {
				'g1e2' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N3P1/PPPPNPBP/R1BQK2R b KQkq - 4 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N3P1/PPPPNPBP/R1BQK2R b KQkq - 4 5' => {
			eco => 'C26h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Bc5 4. Bg2 Nc6 5. Nge2

			variation => N__('Vienna: Smyslov, 3... Bc5 4. Bg2 Nc6 5.Nge2'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b5/2BPP3/5N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C54l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4

			variation => N__('Giuoco Piano: 6.cxd4'),
			moves => {
				'c5b4' => 'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/5N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b5/2BpP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C54j',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 5. d4'),
			moves => {
				'c3d4' => 'r1bqk2r/pppp1ppp/2n2n2/2b5/2BPP3/5N2/PP3PPP/RNBQK2R b KQkq - 0 6',
				'e4e5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1P3/2Bp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b5/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C55u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5

			variation => N__('Two Knights: Max Lange Attack'),
			moves => {
				'e4e5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1P3/2Bp4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/2b5/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'C47q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bc5

			variation => N__('Four Knights: Scotch, 4...exd4 5.Nxd4 Bc5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/3Np3/1b2P3/5N2/PPPP1PPP/R1BQKB1R w KQkq - 6 5' => {
			eco => 'C46h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Bb4 4. Nd5 Nf6

			variation => N__('Three Knights: Schlechter Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/3Np3/1bP5/5NP1/PP1PPP1P/R1BQKB1R b KQkq - 2 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bb4 5. Nd5

			variation => N__('English: Four Knights, 4. g3 Bb4 5. Nd5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/3Np3/1bP5/6P1/PP1PPPBP/R1BQK1NR b KQkq - 4 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 Nf6 4. Bg2 Bb4 5. Nd5

			variation => N__('English: Closed'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/3Pp3/1b2P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C47j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 Bb4 5. d5

			variation => N__('Four Knights: Scotch, Bogoljubow, 5. d5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4N3/1b1PP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C47k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 Bb4 5. Nxe5

			variation => N__('Four Knights: Scotch, Krause Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p1B1/1bB1P3/2NP4/PPP2PPP/R2QK1NR b KQkq - 2 5' => {
			eco => 'C28o',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bb4 5. Bg5

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bb4 5. Bg5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1b1PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 1 5' => {
			eco => 'C47j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 Bb4

			variation => N__('Four Knights: Scotch, Bogoljubow'),
			moves => {
				'd4d5' => 'r1bqk2r/pppp1ppp/2n2n2/3Pp3/1b2P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5',
				'f3e5' => 'r1bqk2r/pppp1ppp/2n2n2/4N3/1b1PP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bB1P3/2NP1N2/PPP2PPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'C28n',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bb4 5. Nf3

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bb4 5. Nf3'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bB1P3/2NP4/PPP1NPPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'C28p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bb4 5. Ne2

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bb4 5. Ne2'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bB1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5' => {
			eco => 'C28m',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bb4

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bb4'),
			moves => {
				'c1g5' => 'r1bqk2r/pppp1ppp/2n2n2/4p1B1/1bB1P3/2NP4/PPP2PPP/R2QK1NR b KQkq - 2 5',
				'g1e2' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bB1P3/2NP4/PPP1NPPP/R1BQK2R b KQkq - 2 5',
				'g1f3' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bB1P3/2NP1N2/PPP2PPP/R1BQK2R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N1PN2/PP1P1PPP/R1BQKB1R w KQkq - 1 5' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3 Bb4

			variation => N__('English: Four Knights, 4. e3 Bb4'),
			moves => {
				'd1c2' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N1PN2/PPQP1PPP/R1B1KB1R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N1PN2/PPQP1PPP/R1B1KB1R b KQkq - 2 5' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3 Bb4 5. Qc2

			variation => N__('English: Four Knights, 4. e3 Bb4 5. Qc2'),
			moves => {
				'b4c3' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/2P5/2b1PN2/PPQP1PPP/R1B1KB1R w KQkq - 0 6',
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/4p3/1bP5/2N1PN2/PPQP1PPP/R1B1KB1R w KQ - 3 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bb4

			variation => N__('English: Four Knights, 4. g3 Bb4'),
			moves => {
				'c3d5' => 'r1bqk2r/pppp1ppp/2n2n2/3Np3/1bP5/5NP1/PP1PPP1P/R1BQKB1R b KQkq - 2 5',
				'f1g2' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 Bb4 5. Bg2

			variation => N__('English: Four Knights, 4. g3 Bb4 5. Bg2'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPPBP/R1BQK2R w KQ - 3 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 3 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 Nf6 4. Bg2 Bb4

			variation => N__('English: Closed'),
			moves => {
				'c3d5' => 'r1bqk2r/pppp1ppp/2n2n2/3Np3/1bP5/6P1/PP1PPPBP/R1BQK1NR b KQkq - 4 5',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/2BNP3/8/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'c6d4' => 'r1bqk2r/pppp1ppp/5n2/4p3/2BnP3/8/PPP2PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/2BbP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'f3d4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/2BNP3/8/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/4p3/2P5/2b1PN2/PPQP1PPP/R1B1KB1R w KQkq - 0 6' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3 Bb4 5. Qc2 Bxc3

			variation => N__('English: Four Knights, Romanishin Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/6B1/1bPN4/2N5/PP2PPPP/R2QKB1R b KQkq - 2 6' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'h7h6' => 'r1bqk2r/pppp1pp1/2n2n1p/6B1/1bPN4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1b1NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'C47r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 Bb4

			variation => N__('Four Knights: Scotch, 4...exd4 5.Nxd4 Bb4'),
			moves => {
				'd4c6' => 'r1bqk2r/pppp1ppp/2N2n2/8/1b2P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPP2PPP/R1B1KBNR w KQkq - 5 6' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'c1d2' => 'r1bqk2r/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/R3KBNR b KQkq - 6 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/R3KBNR b KQkq - 6 6' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/R3KBNR w KQ - 7 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1bB1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Nxc3 Bb4 6. Bc4 Nf6

			variation => N__('Scotch: Goring Gambit, Bardeleben Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/2N2N2/PP3PPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'C54n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'f6e4' => 'r1bqk2r/pppp1ppp/2n5/8/1bBPn3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/5N2/PP1B1PPP/RN1QK2R b KQkq - 2 7' => {
			eco => 'C54m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Bd2

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'f6e4' => 'r1bqk2r/pppp1ppp/2n5/8/1bBPn3/5N2/PP1B1PPP/RN1QK2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/5N2/PP3PPP/RNBQ1K1R b kq - 2 7' => {
			eco => 'C54l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Kf1

			variation => N__('Giuoco Piano: Krakow Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/5N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C54l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+'),
			moves => {
				'b1c3' => 'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/2N2N2/PP3PPP/R1BQK2R b KQkq - 2 7',
				'c1d2' => 'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/5N2/PP1B1PPP/RN1QK2R b KQkq - 2 7',
				'e1f1' => 'r1bqk2r/pppp1ppp/2n2n2/8/1bBPP3/5N2/PP3PPP/RNBQ1K1R b kq - 2 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/8/1bPN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'c1g5' => 'r1bqk2r/pppp1ppp/2n2n2/6B1/1bPN4/2N5/PP2PPPP/R2QKB1R b KQkq - 2 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/b3p3/2B1P3/2P2N2/P2P1PPP/RNBQ1RK1 w kq - 3 7' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O Nf6

			variation => N__('Evans Gambit: 5... Ba5 6.O-O Nf6'),
			moves => {
				'd2d4' => 'r1bqk2r/pppp1ppp/2n2n2/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 b kq d3 0 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n2n2/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'C52',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: 5... Ba5 6.O-O Nf6'),
			moves => {
				'e8g8' => 'r1bq1rk1/pppp1ppp/2n2n2/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 w - - 1 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n4n/2b3N1/2BpP3/8/PPP2PPP/RNBQK2R w KQkq - 4 6' => {
			eco => 'C44y',
                        # TRANSLATORS:

			variation => N__('Scotch Gambit: 5. Ng5'),
			moves => {
				'd1h5' => 'r1bqk2r/pppp1ppp/2n4n/2b3NQ/2BpP3/8/PPP2PPP/RNB1K2R b KQkq - 5 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n4n/2b3NQ/2BpP3/8/PPP2PPP/RNB1K2R b KQkq - 5 6' => {
			eco => 'C44y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5 5. Ng5 Nh6 6. Qh5

			variation => N__('Scotch Gambit: Vitzhum Attack'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2b1P3/2Bp2n1/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C55v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 Ng4 7. c3

			variation => N__('Two Knights: Max Lange, Krause Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2b1P3/2Bp2n1/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'C55v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 Ng4

			variation => N__('Two Knights: Max Lange, Steinitz Variation'),
			moves => {
				'c2c3' => 'r1bqk2r/pppp1ppp/2n5/2b1P3/2Bp2n1/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2b1P3/2Bpn3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5'),
			moves => {
				'c4d5' => 'r1bqk2r/pppp1ppp/2n5/2bBP3/3pn3/2P2N2/PP3PPP/RNBQK2R b KQkq - 2 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2b1P3/2P3n1/4PN2/PP2BPPP/RNBQK2R b KQkq - 2 6' => {
			eco => 'A52g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Nf3 Bc5 5. e3 Nc6 6. Be2

			variation => N__('Budapest: Adler, 4... Bc5 5. e3 Nc6 6. Be2'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2b1P3/2P3n1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'A52f',
                        # TRANSLATORS:

			variation => N__('Budapest: Adler, 4... Bc5'),
			moves => {
				'f1e2' => 'r1bqk2r/pppp1ppp/2n5/2b1P3/2P3n1/4PN2/PP2BPPP/RNBQK2R b KQkq - 2 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2bBP3/3p4/2P2N2/PP3KPP/RNBQ3R b kq - 0 8' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5'),
			moves => {
				'd4c3' => 'r1bqk2r/pppp1ppp/2n5/2bBP3/8/2p2N2/PP3KPP/RNBQ3R w kq - 0 9',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2bBP3/3p4/2P2N2/PP3nPP/RNBQK2R w KQkq - 0 8' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5'),
			moves => {
				'e1f2' => 'r1bqk2r/pppp1ppp/2n5/2bBP3/3p4/2P2N2/PP3KPP/RNBQ3R b kq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2bBP3/3pn3/2P2N2/PP3PPP/RNBQK2R b KQkq - 2 7' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5'),
			moves => {
				'e4f2' => 'r1bqk2r/pppp1ppp/2n5/2bBP3/3p4/2P2N2/PP3nPP/RNBQK2R w KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2bBP3/8/2p2N2/PP3KPP/RNBQ3R w kq - 0 9' => {
			eco => 'C54k',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6. e5'),
			moves => {
				'f2g3' => 'r1bqk2r/pppp1ppp/2n5/2bBP3/8/2p2NK1/PP4PP/RNBQ3R b kq - 1 9',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2bBP3/8/2p2NK1/PP4PP/RNBQ3R b kq - 1 9' => {
			eco => 'C54k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. e5 Ne4 7. Bd5 Nxf2 8. Kxf2 dxc3+ 9. Kg3

			variation => N__('Giuoco Piano: Ghulam Kassim Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n5/2bPp3/4n3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'C44p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nf6 4. d4 Nxe4 5. d5 Bc5

			variation => N__('Ponziani: Fraser Defence'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n5/3P4/2B1n3/2b2N2/PP3PPP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C54o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Bxc3 9. d5

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'c3f6' => 'r1bqk2r/pppp1ppp/2n2b2/3P4/2B1n3/5N2/PP3PPP/R1BQ1RK1 w kq - 1 10',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/4P3/1bP2Bn1/5N2/PP1NPPPP/R2QKB1R b KQkq - 6 6' => {
			eco => 'A52l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Bf4 Nc6 5. Nf3 Bb4+ 6. Nbd2

			variation => N__('Budapest: Rubinstein, Main Line, 6.Nbd2'),
			moves => {
				'd8e7' => 'r1b1k2r/ppppqppp/2n5/4P3/1bP2Bn1/5N2/PP1NPPPP/R2QKB1R w KQkq - 7 7',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/4P3/1bP2Bn1/5N2/PP2PPPP/RN1QKB1R w KQkq - 5 6' => {
			eco => 'A52k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Bf4 Nc6 5. Nf3 Bb4+

			variation => N__('Budapest: Rubinstein, Main Line'),
			moves => {
				'b1d2' => 'r1bqk2r/pppp1ppp/2n5/4P3/1bP2Bn1/5N2/PP1NPPPP/R2QKB1R b KQkq - 6 6',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/1BBPn3/5N2/PP3PPP/RN1QK2R b KQkq - 0 8' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'c6b4' => 'r1bqk2r/pppp1ppp/8/8/1nBPn3/5N2/PP3PPP/RN1QK2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/1bBP4/2P2N2/P4PPP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C54n',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'b4c3' => 'r1bqk2r/pppp1ppp/2n5/8/2BP4/2b2N2/P4PPP/R1BQ1RK1 w kq - 0 10',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/1bBP4/2n2N2/PP3PPP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'C54n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Nxc3

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'b2c3' => 'r1bqk2r/pppp1ppp/2n5/8/1bBP4/2P2N2/P4PPP/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/1bBPn3/2N2N2/PP3PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'C54n',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'b4c3' => 'r1bqk2r/pppp1ppp/2n5/8/2BPn3/2b2N2/PP3PPP/R1BQ1RK1 w kq - 0 9',
				'e4c3' => 'r1bqk2r/pppp1ppp/2n5/8/1bBP4/2n2N2/PP3PPP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/1bBPn3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'C54n',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'e1g1' => 'r1bqk2r/pppp1ppp/2n5/8/1bBPn3/2N2N2/PP3PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/1bBPn3/5N2/PP1B1PPP/RN1QK2R w KQkq - 0 8' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'd2b4' => 'r1bqk2r/pppp1ppp/2n5/8/1BBPn3/5N2/PP3PPP/RN1QK2R b KQkq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/2BP4/1Qb2N2/P4PPP/R1B2RK1 b kq - 1 10' => {
			eco => 'C54n',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp2ppp/2n5/3p4/2BP4/1Qb2N2/P4PPP/R1B2RK1 w kq d6 0 11',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/2BP4/2b2N2/P4PPP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'C54n',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'c1a3' => 'r1bqk2r/pppp1ppp/2n5/8/2BP4/B1b2N2/P4PPP/R2Q1RK1 b kq - 1 10',
				'd1b3' => 'r1bqk2r/pppp1ppp/2n5/8/2BP4/1Qb2N2/P4PPP/R1B2RK1 b kq - 1 10',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/2BP4/B1b2N2/P4PPP/R2Q1RK1 b kq - 1 10' => {
			eco => 'C54n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Nxc3 9. bxc3 Bxc3 10. Ba3

			variation => N__('Giuoco Piano: Aitken Variation'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/2BPn3/2P2N2/P4PPP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp2ppp/2n5/3p4/2BPn3/2P2N2/P4PPP/R1BQ1RK1 w kq d6 0 10',
			},
		},
		'r1bqk2r/pppp1ppp/2n5/8/2BPn3/2b2N2/PP3PPP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'C54o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb4+ 7. Nc3 Nxe4 8. O-O Bxc3

			variation => N__('Giuoco Piano: Greco Attack'),
			moves => {
				'b2c3' => 'r1bqk2r/pppp1ppp/2n5/8/2BPn3/2P2N2/P4PPP/R1BQ1RK1 b kq - 0 9',
				'd4d5' => 'r1bqk2r/pppp1ppp/2n5/3P4/2B1n3/2b2N2/PP3PPP/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r1bqk2r/pppp1ppp/5n2/2b1N3/B2nP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C48r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Ba4 Bc5 6. Nxe5

			variation => N__('Four Knights: Rubinstein, 5. Ba4 Bc5 6.Nxe5'),
			moves => {
			},
		},
		'r1bqk2r/pppp1ppp/5n2/2b1p3/B2nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 8 6' => {
			eco => 'C48q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Ba4 Bc5

			variation => N__('Four Knights: Rubinstein, 5. Ba4 Bc5'),
			moves => {
				'f3e5' => 'r1bqk2r/pppp1ppp/5n2/2b1N3/B2nP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqk2r/pppp1ppp/5n2/4p1B1/2BnP3/8/PPP2PPP/RN1Q1RK1 b kq - 1 7' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp2ppp/3p1n2/4p1B1/2BnP3/8/PPP2PPP/RN1Q1RK1 w kq - 0 8',
				'h7h6' => 'r1bqk2r/pppp1pp1/5n1p/4p1B1/2BnP3/8/PPP2PPP/RN1Q1RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/pppp1ppp/5n2/4p3/2BnP3/8/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C55b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Deutz Gambit'),
			moves => {
				'c1g5' => 'r1bqk2r/pppp1ppp/5n2/4p1B1/2BnP3/8/PPP2PPP/RN1Q1RK1 b kq - 1 7',
			},
		},
		'r1bqk2r/pppp1ppp/8/8/1nBPn3/5N2/PP3PPP/RN1QK2R w KQkq - 0 9' => {
			eco => 'C54m',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: 6.cxd4 Bb4+ 7. Bd2'),
			moves => {
				'c4f7' => 'r1bqk2r/pppp1Bpp/8/8/1n1Pn3/5N2/PP3PPP/RN1QK2R b KQkq - 0 9',
			},
		},
		'r1bqk2r/ppppbppp/2Bn4/4p3/3P4/5N2/PPP1QPPP/RNB2RK1 b kq - 0 7' => {
			eco => 'C67i',
                        # TRANSLATORS:

			variation => N__('Spanish: Open Berlin, 5... Be7 6. Qd2 Nd6'),
			moves => {
				'b7c6' => 'r1bqk2r/p1ppbppp/2pn4/4p3/3P4/5N2/PPP1QPPP/RNB2RK1 w kq - 0 8',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5' => {
			eco => 'C65l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Be7

			variation => N__('Spanish: Berlin, 4.O-O Be7'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/3N4/3pP3/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C47n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5 Be7

			variation => N__('Four Knights: Belgrade Gambit, 5... Be7'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/4P3/2Bp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C50f',
                        # TRANSLATORS:

			variation => N__('Italian: Hungarian Defence, 4. d4 exd4'),
			moves => {
				'f6e4' => 'r1bqk2r/ppppbppp/2n5/4P3/2Bpn3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5' => {
			eco => 'C28d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Be7

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Be7'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C55f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. c3

			variation => N__('Two Knights: 4. d3 Be7 5. c3'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 2 5' => {
			eco => 'C55g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7 5. O-O

			variation => N__('Two Knights: 4. d3 Be7 5.O-O'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 w - - 3 6',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C55e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 Be7

			variation => N__('Two Knights: 4. d3 Be7'),
			moves => {
				'c2c3' => 'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 5',
				'c4b3' => 'r1bqk2r/ppppbppp/2n2n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQK2R b KQkq - 2 5',
				'e1g1' => 'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQ1RK1 b kq - 2 5',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'C55e',
                        # TRANSLATORS:

			variation => N__('Two Knights: 4. d3 Be7'),
			moves => {
				'e8g8' => 'r1bq1rk1/ppppbppp/2n2n2/4p3/4P3/1B1P1N2/PPP2PPP/RNBQK2R w KQ - 3 6',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/6B1/2B4Q/2N2N2/PPP2PPP/R3K2R b KQkq - 6 8' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... Nc6 Defence'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp1bppp/2n2n2/3p2B1/2B4Q/2N2N2/PPP2PPP/R3K2R w KQkq d6 0 9',
				'd7d6' => 'r1bqk2r/ppp1bppp/2np1n2/6B1/2B4Q/2N2N2/PPP2PPP/R3K2R w KQkq - 0 9',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/6B1/2BQ4/2N2N2/PPP2PPP/R3K2R w KQkq - 5 8' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 Nc6

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... Nc6 Defence'),
			moves => {
				'd4h4' => 'r1bqk2r/ppppbppp/2n2n2/6B1/2B4Q/2N2N2/PPP2PPP/R3K2R b KQkq - 6 8',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/8/2BpP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C50f',
                        # TRANSLATORS:

			variation => N__('Italian: Hungarian Defence, 4. d4 exd4'),
			moves => {
				'e4e5' => 'r1bqk2r/ppppbppp/2n2n2/4P3/2Bp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqk2r/ppppbppp/2n2n2/8/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C55s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Be7

			variation => N__('Two Knights: 5.O-O Be7'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2n5/1B2P3/4n3/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C67g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. dxe5

			variation => N__('Spanish: Berlin, Minckwitz Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2n5/1B2p3/3Pn3/5N2/PPP1QPPP/RNB2RK1 b kq - 2 6' => {
			eco => 'C67h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2

			variation => N__('Spanish: Open Berlin, 5... Be7 6. Qe2'),
			moves => {
				'd7d5' => 'r1bqk2r/ppp1bppp/2n5/1B1pp3/3Pn3/5N2/PPP1QPPP/RNB2RK1 w kq d6 0 7',
				'e4d6' => 'r1bqk2r/ppppbppp/2nn4/1B2p3/3P4/5N2/PPP1QPPP/RNB2RK1 w kq - 3 7',
			},
		},
		'r1bqk2r/ppppbppp/2n5/1B2p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'C67f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7

			variation => N__('Spanish: Open Berlin, 5... Be7'),
			moves => {
				'd1e2' => 'r1bqk2r/ppppbppp/2n5/1B2p3/3Pn3/5N2/PPP1QPPP/RNB2RK1 b kq - 2 6',
				'd4e5' => 'r1bqk2r/ppppbppp/2n5/1B2P3/4n3/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqk2r/ppppbppp/2n5/4P3/2Bpn3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C50f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Be7 4. d4 exd4 5. c3 Nf6 6. e5 Ne4

			variation => N__('Italian: Hungarian Defence, Tartakower Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2nn4/1B2p3/3P4/5N2/PPP1QPPP/RNB2RK1 w kq - 3 7' => {
			eco => 'C67i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Be7 6. Qe2 Nd6

			variation => N__('Spanish: Open Berlin, 5... Be7 6. Qd2 Nd6'),
			moves => {
				'b5c6' => 'r1bqk2r/ppppbppp/2Bn4/4p3/3P4/5N2/PPP1QPPP/RNB2RK1 b kq - 0 7',
			},
		},
		'r1bqk2r/ppppbppp/2nn4/4N2Q/8/1BN5/PPPP1PPP/R1B1K2R b KQkq - 0 7' => {
			eco => 'C27g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Be7 6. Nf3 Nc6 7. Nxe5

			variation => N__('Vienna: Alekhine Variation'),
			moves => {
			},
		},
		'r1bqk2r/ppppbppp/2nn4/4p2Q/8/1BN2N2/PPPP1PPP/R1B1K2R w KQkq - 6 7' => {
			eco => 'C27f',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Be7'),
			moves => {
				'f3e5' => 'r1bqk2r/ppppbppp/2nn4/4N2Q/8/1BN5/PPPP1PPP/R1B1K2R b KQkq - 0 7',
			},
		},
		'r1bqk2r/ppppnpbp/2n3p1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR w KQkq - 1 6' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. d3 Nge7

			variation => N__('English: Closed, 5. d3 Nge7'),
			moves => {
			},
		},
		'r1bqk2r/ppppnppp/2n5/1Bb1p3/4P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C64d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. c3 Nge7

			variation => N__('Spanish: Classical, 4. c3 Nge7'),
			moves => {
			},
		},
		'r1bqk2r/ppppnppp/2n5/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5' => {
			eco => 'C64l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Bc5 4. O-O Nge7

			variation => N__('Spanish: Classical, 4.O-O Nge7'),
			moves => {
			},
		},
		'r1bqk2r/ppppnppp/5b2/3P4/2B1R3/5N2/PP3PPP/R1BQ2K1 b kq - 0 11' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'd7d6' => 'r1bqk2r/ppp1nppp/3p1b2/3P4/2B1R3/5N2/PP3PPP/R1BQ2K1 w kq - 0 12',
			},
		},
		'r1bqk2r/ppppnppp/5b2/3P4/2B1n3/5N2/PP3PPP/R1BQR1K1 w kq - 3 11' => {
			eco => 'C54o',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Möller (Therkatz) Attack'),
			moves => {
				'e1e4' => 'r1bqk2r/ppppnppp/5b2/3P4/2B1R3/5N2/PP3PPP/R1BQ2K1 b kq - 0 11',
			},
		},
		'r1bqkb1r/1p1n1ppp/2n1p3/p1ppP3/3P4/2PB4/PP1NNPPP/R1BQK2R w KQkq a6 0 8' => {
			eco => 'C05w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 a5

			variation => N__('French: Tarrasch, Closed, 7. Ne2 a5'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p1n1p3/2ppP3/3P1P2/2N1BN2/PPP3PP/R2QKB1R w KQkq - 0 8' => {
			eco => 'C11j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3 Nc6 7. Be3 a6

			variation => N__('French: Steinitz, Boleslavsky, 7... a6'),
			moves => {
				'd1d2' => 'r1bqkb1r/1p1n1ppp/p1n1p3/2ppP3/3P1P2/2N1BN2/PPPQ2PP/R3KB1R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/1p1n1ppp/p1n1p3/2ppP3/3P1P2/2N1BN2/PPPQ2PP/R3KB1R b KQkq - 1 8' => {
			eco => 'C11j',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, Boleslavsky, 7... a6'),
			moves => {
				'b7b5' => 'r1bqkb1r/3n1ppp/p1n1p3/1pppP3/3P1P2/2N1BN2/PPPQ2PP/R3KB1R w KQkq b6 0 9',
			},
		},
		'r1bqkb1r/1p1n1ppp/p1p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'D46a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 a6

			variation => N__('Semi-Slav: 6. Bd3 a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B91k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nde2 Nbd7

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B93n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7'),
			moves => {
				'a2a4' => 'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R b KQkq a3 0 8',
				'f1d3' => 'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/4PP2/2NB1N2/PPP3PP/R1BQK2R b KQkq - 3 8',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/4PP2/2NB1N2/PPP3PP/R1BQK2R b KQkq - 3 8' => {
			eco => 'B93o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. Bd3

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. Bd3'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R b KQkq a3 0 8' => {
			eco => 'B93p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Nbd7 8. a4

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Nbd7 8. a4'),
			moves => {
				'd8c7' => 'r1b1kb1r/1pqn1ppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R w KQkq - 1 9',
				'f8e7' => 'r1bqk2r/1p1nbppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/2BNP3/2N5/PPP2PPP/R2Q1RK1 b kq - 1 8' => {
			eco => 'B94i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. Bc4 e6 8. O-O

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4 e6 8.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/2BNP3/2N5/PPP2PPP/R2QK2R w KQkq - 0 8' => {
			eco => 'B94h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. Bc4 e6

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4 e6'),
			moves => {
				'e1g1' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/2BNP3/2N5/PPP2PPP/R2Q1RK1 b kq - 1 8',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/2BNPP2/2N5/PPP3PP/R2QK2R b KQkq - 2 8' => {
			eco => 'B96i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Bc4

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Bc4'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B95k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qf3 Nbd7

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qf3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B96j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qf3

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qf3'),
			moves => {
				'd8c7' => 'r1b1kb1r/1pqn1ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 3 9',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NPP2/2N5/PPP1Q1PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B96i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7 8. Qe2

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7 8. Qe2'),
			moves => {
				'd8c7' => 'r1b1kb1r/1pqn1ppp/p2ppn2/6B1/3NPP2/2N5/PPP1Q1PP/R3KB1R w KQkq - 3 9',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B96i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nbd7

			variation => N__('Sicilian: Najdorf, 7. f4 Nbd7'),
			moves => {
				'd1e2' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NPP2/2N5/PPP1Q1PP/R3KB1R b KQkq - 2 8',
				'd1f3' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 2 8',
				'f1c4' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/2BNPP2/2N5/PPP3PP/R2QK2R b KQkq - 2 8',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Nbd7

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Nbd7'),
			moves => {
				'a2a4' => 'r1bqkb1r/1p1n1ppp/p2ppn2/8/P2NP3/2N5/1PP1BPPP/R1BQ1RK1 b kq a3 0 8',
				'f2f4' => 'r1bqkb1r/1p1n1ppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 b kq f3 0 8',
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 b kq f3 0 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Nbd7 8. f4

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Nbd7 8. f4'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p2ppn2/8/P2NP3/2N5/1PP1BPPP/R1BQ1RK1 b kq a3 0 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Nbd7 8. a4

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Nbd7 8. a4'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p3pn2/2p5/2QP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 0 8' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5 7. Nf3 a6

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 c5 7. Nf3 a6'),
			moves => {
				'c4c2' => 'r1bqkb1r/1p1n1ppp/p3pn2/2p5/3P4/5NP1/PPQ1PPBP/RNB1K2R b KQkq - 1 8',
				'e1g1' => 'r1bqkb1r/1p1n1ppp/p3pn2/2p5/2QP4/5NP1/PP2PPBP/RNB2RK1 b kq - 1 8',
			},
		},
		'r1bqkb1r/1p1n1ppp/p3pn2/2p5/2QP4/5NP1/PP2PPBP/RNB2RK1 b kq - 1 8' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5 7. Nf3 a6 8. O-O

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 c5 7. Nf3 a6 8.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p3pn2/2p5/3P4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'D27e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bd3 Nbd7

			variation => N__('QGA: Classical, 6... a6 7. Bd3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1n1ppp/p3pn2/2p5/3P4/5NP1/PPQ1PPBP/RNB1K2R b KQkq - 1 8' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5 7. Nf3 a6 8. Qc2

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 c5 7. Nf3 a6 8. Qc2'),
			moves => {
				'b7b6' => 'r1bqkb1r/3n1ppp/pp2pn2/2p5/3P4/5NP1/PPQ1PPBP/RNB1K2R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/1p1npppp/p1p2n2/2Pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D15k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. c5 Nbd7

			variation => N__('Slav: Chameleon, 5. c5 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 6' => {
			eco => 'B51d',
                        # TRANSLATORS:

			variation => N__('Sicilian: 3. Bb5+ Nd7 4.O-O Nf6'),
			moves => {
				'b5f1' => 'r1bqkb1r/1p1npppp/p2p1n2/2p5/4P3/5N2/PPPP1PPP/RNBQRBK1 b kq - 1 6',
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/2p5/4P3/5N2/PPPP1PPP/RNBQRBK1 b kq - 1 6' => {
			eco => 'B51e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. O-O Nf6 5. Re1 a6 6. Bf1

			variation => N__('Sicilian: 3. Bb5+ Nd7 4.O-O Nf6 5. Re1 a6 6. Bf1'),
			moves => {
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/6B1/2BNP3/2N5/PPP2PPP/R2QK2R b KQkq - 3 7' => {
			eco => 'B94g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. Bc4

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. Bc4'),
			moves => {
				'd8a5' => 'r1b1kb1r/1p1npppp/p2p1n2/q5B1/2BNP3/2N5/PPP2PPP/R2QK2R w KQkq - 4 8',
				'e7e6' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/2BNP3/2N5/PPP2PPP/R2QK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B94d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7'),
			moves => {
				'f1c4' => 'r1bqkb1r/1p1npppp/p2p1n2/6B1/2BNP3/2N5/PPP2PPP/R2QK2R b KQkq - 3 7',
				'f2f4' => 'r1bqkb1r/1p1npppp/p2p1n2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq f3 0 7',
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq f3 0 7' => {
			eco => 'B94e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. f4

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. f4'),
			moves => {
				'b7b5' => 'r1bqkb1r/3npppp/p2p1n2/1p4B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq b6 0 8',
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B92b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 Nbd7

			variation => N__('Sicilian: Najdorf, 6. Be2 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/8/3NPP2/2N5/PPP1B1PP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B93d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Nbd7 7. Be2

			variation => N__('Sicilian: Najdorf, 6. f4 Nbd7 7. Be2'),
			moves => {
			},
		},
		'r1bqkb1r/1p1npppp/p2p1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B93c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Nbd7

			variation => N__('Sicilian: Najdorf, 6. f4 Nbd7'),
			moves => {
				'f1e2' => 'r1bqkb1r/1p1npppp/p2p1n2/8/3NPP2/2N5/PPP1B1PP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/1p1p1ppp/p1n1pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'A33l',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3 a6

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. g3 a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p1p1ppp/p1n1pn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B46l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be3 Nf6

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/1p1p1ppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B46n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be2 Nf6

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be2 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/1p1pnppp/p1n1p3/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'B30w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O Nge7 5. c3 a6

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O Nge7 5. c3 a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 3 7' => {
			eco => 'B46q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be2 Nge7 7. Be3

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be2 Nge7 7. Be3'),
			moves => {
			},
		},
		'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B46k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be3 Nge7

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be3 Nge7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B46g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. g3 Nge7

			variation => N__('Sicilian: Taimanov, 5... a6 6. g3 Nge7'),
			moves => {
			},
		},
		'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'B46p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be2 Nge7 7. O-O

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be2 Nge7 7.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B46o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be2 Nge7

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be2 Nge7'),
			moves => {
				'c1e3' => 'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 3 7',
				'e1g1' => 'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqkb1r/1p2pppp/p1n2n2/3p4/3P1B2/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'D10g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nc3 Nf6 5. Bf4 Nc6 6. e3 a6

			variation => N__('Slav: Exchange, 4. Nc3 Nf6 5. Bf4 Nc6 6. e3 a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p2pppp/p1n2n2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D13m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 a6 7. e3

			variation => N__('Slav: Exchange, 6. Bf4 a6 7. e3'),
			moves => {
				'c8g4' => 'r2qkb1r/1p2pppp/p1n2n2/3p4/3P1Bb1/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/1p2pppp/p1n2n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D13l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 a6

			variation => N__('Slav: Exchange, 6. Bf4 a6'),
			moves => {
				'e2e3' => 'r1bqkb1r/1p2pppp/p1n2n2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/1p2pppp/p1np1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B60e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 a6

			variation => N__('Sicilian: Richter-Rauzer, 6... a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p2pppp/p1np1n2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B91d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 Nc6

			variation => N__('Sicilian: Najdorf, 6. g3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/1p2pppp/p1np1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B58c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 a6

			variation => N__('Sicilian: Boleslavsky, 6... a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p2pppp/p1np1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B93b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Nc6

			variation => N__('Sicilian: Najdorf, 6. f4 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3p2/p1nppp1p/8/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 10' => {
			eco => 'B63i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 h6 8. Bxf6 gxf6 9. O-O-O a6

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 h6 8.Bxf6 gxf6 9.O-O-O a6'),
			moves => {
				'f2f4' => 'r1bqkb1r/1p3p2/p1nppp1p/8/3NPP2/2N5/PPPQ2PP/2KR1B1R b kq f3 0 10',
			},
		},
		'r1bqkb1r/1p3p2/p1nppp1p/8/3NPP2/2N5/PPPQ2PP/2KR1B1R b kq f3 0 10' => {
			eco => 'B63j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 h6 8. Bxf6 gxf6 9. O-O-O a6 10. f4

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 h6 8.Bxf6 gxf6 9.O-O-O a6 10. f4'),
			moves => {
			},
		},
		'r1bqkb1r/1p3pp1/p1nppn1p/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 9' => {
			eco => 'B66f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6'),
			moves => {
				'g5e3' => 'r1bqkb1r/1p3pp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R b kq - 1 9',
				'g5f4' => 'r1bqkb1r/1p3pp1/p1nppn1p/8/3NPB2/2N5/PPPQ1PPP/2KR1B1R b kq - 1 9',
			},
		},
		'r1bqkb1r/1p3pp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R b kq - 1 9' => {
			eco => 'B66i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3'),
			moves => {
				'c8d7' => 'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R w kq - 2 10',
				'd8c7' => 'r1b1kb1r/1pq2pp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R w kq - 2 10',
				'f8e7' => 'r1bqk2r/1p2bpp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R w kq - 2 10',
			},
		},
		'r1bqkb1r/1p3pp1/p1nppn1p/8/3NPB2/2N5/PPPQ1PPP/2KR1B1R b kq - 1 9' => {
			eco => 'B66g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Bf4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Bf4'),
			moves => {
				'c8d7' => 'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NPB2/2N5/PPPQ1PPP/2KR1B1R w kq - 2 10',
			},
		},
		'r1bqkb1r/1p3ppp/1nn1p3/p2pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 w kq a6 0 10' => {
			eco => 'C06e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Nb6 9. O-O a5

			variation => N__('French: Tarrasch, Closed, Leningrad, 9.O-O a5'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1Nppn2/6B1/4PP2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B96d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nc6 8. Nxc6

			variation => N__('Sicilian: Najdorf, 7. f4 Nc6 8.Nxc6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2Pp4/2P5/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D40q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 a6 7. dxc5

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 a6 7.dxc5'),
			moves => {
				'f8c5' => 'r1bqk2r/1p3ppp/p1n1pn2/2bp4/2P5/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2p5/2BP4/4PN2/PP2QPPP/RNB2RK1 w kq - 2 8' => {
			eco => 'D28c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 Nc6

			variation => N__('QGA: Classical, 7. Qe2 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2p5/3P4/1B2PN2/PP3PPP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'D27i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bb3 Nc6

			variation => N__('QGA: Classical, 6... a6 7. Bb3 Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/1p3ppp/p1n1pn2/2p5/3P4/1BN1PN2/PP3PPP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2p5/3P4/1BN1PN2/PP3PPP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'D27j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bb3 Nc6 8. Nc3

			variation => N__('QGA: Classical, 6... a6 7. Bb3 Nc6 8. Nc3'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2p5/P1BP4/2N1PN2/1P3PPP/R1BQ1RK1 b kq - 2 8' => {
			eco => 'D27q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Nc3

			variation => N__('QGA: Classical, Rubinstein, 8. Nc3'),
			moves => {
				'f8e7' => 'r1bqk2r/1p2bppp/p1n1pn2/2p5/P1BP4/2N1PN2/1P3PPP/R1BQ1RK1 w kq - 3 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2p5/P1BP4/4PN2/1P2QPPP/RNB2RK1 b kq - 2 8' => {
			eco => 'D27t',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Qe2

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2'),
			moves => {
				'c5d4' => 'r1bqkb1r/1p3ppp/p1n1pn2/8/P1Bp4/4PN2/1P2QPPP/RNB2RK1 w kq - 0 9',
				'd8c7' => 'r1b1kb1r/1pq2ppp/p1n1pn2/2p5/P1BP4/4PN2/1P2QPPP/RNB2RK1 w kq - 3 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2p5/P1BP4/4PN2/1P3PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'D27p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6

			variation => N__('QGA: Classical, Rubinstein, 7... Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/1p3ppp/p1n1pn2/2p5/P1BP4/2N1PN2/1P3PPP/R1BQ1RK1 b kq - 2 8',
				'd1e2' => 'r1bqkb1r/1p3ppp/p1n1pn2/2p5/P1BP4/4PN2/1P2QPPP/RNB2RK1 b kq - 2 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2pp4/2PP4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D40o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 a6

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 a6'),
			moves => {
				'b2b3' => 'r1bqkb1r/1p3ppp/p1n1pn2/2pp4/2PP4/PPN1PN2/5PPP/R1BQKB1R b KQkq - 0 7',
				'd4c5' => 'r1bqkb1r/1p3ppp/p1n1pn2/2Pp4/2P5/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/2pp4/2PP4/PPN1PN2/5PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D40p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 a6 7. b3

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 a6 7. b3'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/3p4/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 0 9' => {
			eco => 'B44r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 d5

			variation => N__('Sicilian: Taimanov, Szen, Kasparov Gambit'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/8/P1Bp4/4PN2/1P2QPPP/RNB2RK1 w kq - 0 9' => {
			eco => 'D27v',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4 Nc6 8. Qe2 cxd4

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4'),
			moves => {
				'f1d1' => 'r1bqkb1r/1p3ppp/p1n1pn2/8/P1Bp4/4PN2/1P2QPPP/RNBR2K1 b kq - 1 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1n1pn2/8/P1Bp4/4PN2/1P2QPPP/RNBR2K1 b kq - 1 9' => {
			eco => 'D27v',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Rubinstein, 8. Qe2 cxd4'),
			moves => {
				'f8e7' => 'r1bqk2r/1p2bppp/p1n1pn2/8/P1Bp4/4PN2/1P2QPPP/RNBR2K1 w kq - 2 10',
			},
		},
		'r1bqkb1r/1p3ppp/p1np1B2/1N2p3/4P3/2N5/PPP2PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B33j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Bxf6

			variation => N__('Sicilian: Pelikan, 7. Bg5 a6 8.Bxf6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1np1n2/1N2p1B1/4P3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'B33i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6

			variation => N__('Sicilian: Pelikan, 7. Bg5 a6'),
			moves => {
				'b5a3' => 'r1bqkb1r/1p3ppp/p1np1n2/4p1B1/4P3/N1N5/PPP2PPP/R2QKB1R b KQkq - 1 8',
				'g5f6' => 'r1bqkb1r/1p3ppp/p1np1B2/1N2p3/4P3/2N5/PPP2PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1np1n2/1N2p3/P3P3/2N5/1PP2PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B33g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pelikan, 7. a4'),
			moves => {
				'b5a3' => 'r1bqkb1r/1p3ppp/p1np1n2/4p3/P3P3/N1N5/1PP2PPP/R1BQKB1R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1np1n2/4p1B1/4P3/N1N5/PPP2PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'B33k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3

			variation => N__('Sicilian: Pelikan, 7. Bg5 a6 8. Na3'),
			moves => {
				'b7b5' => 'r1bqkb1r/5ppp/p1np1n2/1p2p1B1/4P3/N1N5/PPP2PPP/R2QKB1R w KQkq b6 0 9',
				'c8e6' => 'r2qkb1r/1p3ppp/p1npbn2/4p1B1/4P3/N1N5/PPP2PPP/R2QKB1R w KQkq - 2 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1np1n2/4p3/P3P3/N1N5/1PP2PPP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'B33g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. a4 a6 8. Na3

			variation => N__('Sicilian: Pelikan, 7. a4'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/1N6/2P1P3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B44q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6'),
			moves => {
				'b5a3' => 'r1bqkb1r/1p3ppp/p1nppn2/8/2P1P3/N1N5/PP3PPP/R1BQKB1R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/4P1B1/3N1P2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B96c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nc6 8. e5

			variation => N__('Sicilian: Najdorf, 7. f4 Nc6 8. e5'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 1 8' => {
			eco => 'B66c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O'),
			moves => {
				'c6d4' => 'r1bqkb1r/1p3ppp/p2ppn2/6B1/3nP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 9',
				'c8d7' => 'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 2 9',
				'f8e7' => 'r1bqk2r/1p2bppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 2 9',
				'h7h6' => 'r1bqkb1r/1p3pp1/p1nppn1p/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8' => {
			eco => 'B66a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6'),
			moves => {
				'e1c1' => 'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 1 8',
				'f1e2' => 'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2N5/PPPQBPPP/R3K2R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2N5/PPPQBPPP/R3K2R b KQkq - 1 8' => {
			eco => 'B66b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. Be2

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8. Be2'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2NQ4/PPP2PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B95i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qd3 Nc6

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qd3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B96c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Nc6

			variation => N__('Sicilian: Najdorf, 7. f4 Nc6'),
			moves => {
				'd4c6' => 'r1bqkb1r/1p3ppp/p1Nppn2/6B1/4PP2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8',
				'e4e5' => 'r1bqkb1r/1p3ppp/p1nppn2/4P1B1/3N1P2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/2BNP3/2N1B3/PPP1QPPP/R3K2R b KQkq - 1 8' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 a6 8. Qe2

			variation => N__('Sicilian: Velimirovic vs. 7... a6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 0 8' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3 a6

			variation => N__('Sicilian: Sozin, 7. Be3 a6'),
			moves => {
				'd1e2' => 'r1bqkb1r/1p3ppp/p1nppn2/8/2BNP3/2N1B3/PPP1QPPP/R3K2R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/2P1P3/N1N5/PP3PPP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'B44r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3'),
			moves => {
				'b7b6' => 'r1bqkb1r/5ppp/ppnppn2/8/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 0 9',
				'd6d5' => 'r1bqkb1r/1p3ppp/p1n1pn2/3p4/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 0 9',
				'f8e7' => 'r1bqk2r/1p2bppp/p1nppn2/8/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 2 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B80h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 Nc6

			variation => N__('Sicilian: Scheveningen, English Attack, 7... Nc6'),
			moves => {
				'd1d2' => 'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 2 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B80h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 Nc6 8. Qd2

			variation => N__('Sicilian: Scheveningen, English, 7... Nc6 8. Qd2'),
			moves => {
				'f8e7' => 'r1bqk2r/1p2bppp/p1nppn2/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 3 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'B80r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6 7. Bg2 a6 8. O-O

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'c8d7' => 'r2qkb1r/1p1b1ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B80r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6 7. Bg2 a6

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'e1g1' => 'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Nc6

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/1p3ppp/p2ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R b kq - 0 9' => {
			eco => 'B66e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Nxd4 9. Qxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O Nxd4 9.Qxd4'),
			moves => {
				'f8e7' => 'r1bqk2r/1p2bppp/p2ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R w kq - 1 10',
			},
		},
		'r1bqkb1r/1p3ppp/p2ppn2/6B1/3nP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 9' => {
			eco => 'B66e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Nxd4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O Nxd4'),
			moves => {
				'd2d4' => 'r1bqkb1r/1p3ppp/p2ppn2/6B1/3QP3/2N5/PPP2PPP/2KR1B1R b kq - 0 9',
			},
		},
		'r1bqkb1r/1pp1n1pp/p1p2p2/8/3NP3/8/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C69i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 Ne7

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 exd4 7.Nxd4 Ne7'),
			moves => {
			},
		},
		'r1bqkb1r/1pp1nppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C72',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. O-O Ne7

			variation => N__('Spanish: Modern Steinitz, 5.O-O Ne7'),
			moves => {
			},
		},
		'r1bqkb1r/1pp1nppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C68k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Ne7

			variation => N__('Spanish: Exchange, 5.O-O Ne7'),
			moves => {
			},
		},
		'r1bqkb1r/1pp1pppp/p1n2n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D24j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. a4 Nc6

			variation => N__('QGA: 4. Nc3 a6 5. a4 Nc6'),
			moves => {
				'e2e4' => 'r1bqkb1r/1pp1pppp/p1n2n2/8/P1pPP3/2N2N2/1P3PPP/R1BQKB1R b KQkq e3 0 6',
			},
		},
		'r1bqkb1r/1pp1pppp/p1n2n2/8/P1pPP3/2N2N2/1P3PPP/R1BQKB1R b KQkq e3 0 6' => {
			eco => 'D24j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. a4 Nc6 6. e4

			variation => N__('QGA: 4. Nc3 a6 5. a4 Nc6 5. e4'),
			moves => {
			},
		},
		'r1bqkb1r/1pp2p1p/p1np1np1/4p3/B3P3/2PP1N2/PP3PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C77g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d3 d6 6. c3 g6

			variation => N__('Spanish: Anderssen, 5... d6 6. c3 g6'),
			moves => {
			},
		},
		'r1bqkb1r/1pp2ppp/p1Bp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6 6. Bxc6+

			variation => N__('Spanish: Steinitz Deferred, Exchange'),
			moves => {
				'b7c6' => 'r1bqkb1r/2p2ppp/p1pp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqkb1r/1pp2ppp/p1n1pn2/8/2pP4/4PNP1/PP3PBP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'E04j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6 6. O-O Nc6 7. e3

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O Nc6 7. e3'),
			moves => {
			},
		},
		'r1bqkb1r/1pp2ppp/p1n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'E04i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6 6. O-O Nc6

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O Nc6'),
			moves => {
				'e2e3' => 'r1bqkb1r/1pp2ppp/p1n1pn2/8/2pP4/4PNP1/PP3PBP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B1P1P3/3P1N2/PP3PPP/RNBQK2R b KQkq c3 0 6' => {
			eco => 'C77e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d3 d6 6. c4

			variation => N__('Spanish: Duras Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6 6. c3

			variation => N__('Spanish: Steinitz Deferred, 6. c3'),
			moves => {
			},
		},
		'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C77f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d3 d6 6. c3

			variation => N__('Spanish: Anderssen, 5... d6 6. c3'),
			moves => {
				'g7g6' => 'r1bqkb1r/1pp2p1p/p1np1np1/4p3/B3P3/2PP1N2/PP3PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C77d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d3 d6

			variation => N__('Spanish: Anderssen, 5... d6'),
			moves => {
				'c2c3' => 'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 6',
				'c2c4' => 'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B1P1P3/3P1N2/PP3PPP/RNBQK2R b KQkq c3 0 6',
			},
		},
		'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6

			variation => N__('Spanish: Steinitz Deferred'),
			moves => {
				'a4c6' => 'r1bqkb1r/1pp2ppp/p1Bp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 0 6',
				'c2c3' => 'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 6',
				'f1e1' => 'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 6',
			},
		},
		'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 6' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6 6. Re1

			variation => N__('Spanish: Steinitz Deferred, 6. Re1'),
			moves => {
			},
		},
		'r1bqkb1r/1pp2ppp/p1p2n2/4N3/4P3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'f6e4' => 'r1bqkb1r/1pp2ppp/p1p5/4N3/4n3/2N5/PPPP1PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/1pp2ppp/p1p2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'f3e5' => 'r1bqkb1r/1pp2ppp/p1p2n2/4N3/4P3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/1pp2ppp/p1p5/4N3/4N3/8/PPPP1PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'd8d4' => 'r1b1kb1r/1pp2ppp/p1p5/4N3/3qN3/8/PPPP1PPP/R1BQK2R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/1pp2ppp/p1p5/4N3/4n3/2N5/PPPP1PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'c3e4' => 'r1bqkb1r/1pp2ppp/p1p5/4N3/4N3/8/PPPP1PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/1ppn1ppp/2n1p3/p2pP3/3P4/1N3N2/PPP2PPP/R1BQKB1R w KQkq a6 0 7' => {
			eco => 'C04i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. Nb3 a5

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. Nb3 a5'),
			moves => {
			},
		},
		'r1bqkb1r/1ppn1ppp/p3pn2/8/2Q5/5NP1/PP1PPPBP/RNB1K2R w KQkq - 0 7' => {
			eco => 'A13w',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6

			variation => N__('English: Neo-Catalan Accepted, 5. Qa4+ Nbd7 6.Qxc4 a6'),
			moves => {
			},
		},
		'r1bqkb1r/1ppn1ppp/p3pn2/8/2QP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 7' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6 7. Nf3

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 a6 7. Nf3'),
			moves => {
				'b7b5' => 'r1bqkb1r/2pn1ppp/p3pn2/1p6/2QP4/5NP1/PP2PPBP/RNB1K2R w KQkq b6 0 8',
			},
		},
		'r1bqkb1r/1ppn1ppp/p3pn2/8/2QP4/6P1/PP2PPBP/RNB1K1NR w KQkq - 0 7' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 a6'),
			moves => {
				'c4c2' => 'r1bqkb1r/1ppn1ppp/p3pn2/8/3P4/6P1/PPQ1PPBP/RNB1K1NR b KQkq - 1 7',
				'g1f3' => 'r1bqkb1r/1ppn1ppp/p3pn2/8/2QP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/1ppn1ppp/p3pn2/8/3P4/6P1/PPQ1PPBP/RNB1K1NR b KQkq - 1 7' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6 7. Qc2

			variation => N__('Catalan: Open, Alekhine Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1ppn1ppp/p3pn2/8/Q1pP4/2N2NP1/PP2PPBP/R1B1K2R b KQkq - 1 7' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Nf3 a6 7. Nc3

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6. Nf3 a6 7. Nc3'),
			moves => {
			},
		},
		'r1bqkb1r/1ppn1ppp/p3pn2/8/Q1pP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 0 7' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Nf3 a6

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6. Nf3 a6'),
			moves => {
				'b1c3' => 'r1bqkb1r/1ppn1ppp/p3pn2/8/Q1pP4/2N2NP1/PP2PPBP/R1B1K2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1B2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 0 5' => {
			eco => 'C48a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 a6 5. Bxc6

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'd7c6' => 'r1bqkb1r/1pp2ppp/p1p2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1B2n2/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C77i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Bxc6

			variation => N__('Spanish: Exchange Deferred'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1B5/2n1p3/8/5N2/PPPP1PPP/RNBQR1K1 b kq - 0 7' => {
			eco => 'C80e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. Re1 Nc5 7. Bxc6

			variation => N__('Spanish: Open, 6. Re1 Nc5 7.Bxc6'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish Variation'),
			moves => {
				'b5c6' => 'r1bqkb1r/1ppp1ppp/p1B2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B2PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C77j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d4

			variation => N__('Spanish: Centre Attack'),
			moves => {
				'b7b5' => 'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B2PP3/5N2/PPP2PPP/RNBQK2R w KQkq b6 0 6',
				'c6d4' => 'r1bqkb1r/1ppp1ppp/p4n2/4p3/B2nP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6',
				'e5d4' => 'r1bqkb1r/1ppp1ppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6',
				'f6e4' => 'r1bqkb1r/1ppp1ppp/p1n5/4p3/B2Pn3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6',
				'f8e7' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B2PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 3 5' => {
			eco => 'C77h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Nc3

			variation => N__('Spanish: Four Knights (Tarrasch) Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C77b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d3

			variation => N__('Spanish: Anderssen Variation'),
			moves => {
				'b7b5' => 'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B3P3/3P1N2/PPP2PPP/RNBQK2R w KQkq b6 0 6',
				'd7d6' => 'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'C78a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O

			variation => N__('Spanish: 5.O-O'),
			moves => {
				'b7b5' => 'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq b6 0 6',
				'd7d6' => 'r1bqkb1r/1pp2ppp/p1np1n2/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 6',
				'f6e4' => 'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 6',
				'f8c5' => 'r1bqk2r/1ppp1ppp/p1n2n2/2b1p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 6',
				'f8e7' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 6',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C77a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6

			variation => N__('Spanish: Morphy Defence'),
			moves => {
				'a4c6' => 'r1bqkb1r/1ppp1ppp/p1B2n2/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 5',
				'b1c3' => 'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 3 5',
				'd1e2' => 'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/5N2/PPPPQPPP/RNB1K2R b KQkq - 3 5',
				'd2d3' => 'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'd2d4' => 'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B2PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5',
				'e1g1' => 'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/5N2/PPPPQPPP/RNB1K2R b KQkq - 3 5' => {
			eco => 'C77p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2

			variation => N__('Spanish: Wormald Attack'),
			moves => {
				'b7b5' => 'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B3P3/5N2/PPPPQPPP/RNB1K2R w KQkq b6 0 6',
				'f8e7' => 'r1bqk2r/1pppbppp/p1n2n2/4p3/B3P3/5N2/PPPPQPPP/RNB1K2R w KQkq - 4 6',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n2n2/8/B2pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C77o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d4 exd4

			variation => N__('Spanish: Centre Attack, 5...exd4'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/1B2p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C67e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 a6

			variation => N__('Spanish: Open Berlin, Rosenthal Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/2n1p3/B7/5N2/PPPP1PPP/RNBQR1K1 w kq - 2 7' => {
			eco => 'C80d',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, 6. Re1'),
			moves => {
				'a4c6' => 'r1bqkb1r/1ppp1ppp/p1B5/2n1p3/8/5N2/PPPP1PPP/RNBQR1K1 b kq - 0 7',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/4p3/B2Pn3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6' => {
			eco => 'C80f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4

			variation => N__('Spanish: Open, 6. d4'),
			moves => {
				'b7b5' => 'r1bqkb1r/2pp1ppp/p1n5/1p2p3/B2Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq b6 0 7',
				'e5d4' => 'r1bqkb1r/1ppp1ppp/p1n5/8/B2pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7',
				'f8e7' => 'r1bqk2r/1pppbppp/p1n5/4p3/B2Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 7',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/4p3/B2Pn3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C77m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d4 Nxe4

			variation => N__('Spanish: Centre Attack, 5...Nxe4'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'C80b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. Nc3

			variation => N__('Spanish: Open, Knorre Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C80a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4

			variation => N__('Spanish: Open'),
			moves => {
				'b1c3' => 'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 1 6',
				'd1e2' => 'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/5N2/PPPPQPPP/RNB2RK1 b kq - 1 6',
				'd2d4' => 'r1bqkb1r/1ppp1ppp/p1n5/4p3/B2Pn3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6',
				'f1e1' => 'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 6',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 6' => {
			eco => 'C80d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. Re1

			variation => N__('Spanish: Open, 6. Re1'),
			moves => {
				'e4c5' => 'r1bqkb1r/1ppp1ppp/p1n5/2n1p3/B7/5N2/PPPP1PPP/RNBQR1K1 w kq - 2 7',
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/4p3/B3n3/5N2/PPPPQPPP/RNB2RK1 b kq - 1 6' => {
			eco => 'C80c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. Qe2

			variation => N__('Spanish: Open, Tartakower Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p1n5/8/B2pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C80g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 exd4

			variation => N__('Spanish: Open, Riga Variation'),
			moves => {
			},
		},
		'r1bqkb1r/1ppp1ppp/p4n2/4p3/B2nP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C77l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d4 Nxd4

			variation => N__('Spanish: Centre Attack, 5...Nxd4'),
			moves => {
			},
		},
		'r1bqkb1r/1pppnppp/p1n5/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nge7

			variation => N__('Spanish: Cozio Deferred'),
			moves => {
			},
		},
		'r1bqkb1r/2p1nppp/p7/1p1pP3/4n3/1B3N2/PPP2PPP/RNBQ1RK1 w kq - 1 9' => {
			eco => 'C80m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Ne7

			variation => N__('Spanish: Open, Zukertort Variation'),
			moves => {
			},
		},
		'r1bqkb1r/2p2ppp/p1n2n2/1p1Pp1N1/8/1B6/PPPP1PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'c6d4' => 'r1bqkb1r/2p2ppp/p4n2/1p1Pp1N1/3n4/1B6/PPPP1PPP/RNBQ1RK1 w kq - 1 9',
			},
		},
		'r1bqkb1r/2p2ppp/p1n2n2/1p1pp1N1/4P3/1B6/PPPP1PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'e4d5' => 'r1bqkb1r/2p2ppp/p1n2n2/1p1Pp1N1/8/1B6/PPPP1PPP/RNBQ1RK1 b kq - 0 8',
			},
		},
		'r1bqkb1r/2p2ppp/p1n5/1p1pN3/3Pn3/1B6/PPP2PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C80l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. Nxe5

			variation => N__('Spanish: Open, 8.Nxe5'),
			moves => {
			},
		},
		'r1bqkb1r/2p2ppp/p1n5/1p1pP3/4n3/1B3N2/PPP2PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C80m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5

			variation => N__('Spanish: Open, 8.dxe5'),
			moves => {
				'c6e7' => 'r1bqkb1r/2p1nppp/p7/1p1pP3/4n3/1B3N2/PPP2PPP/RNBQ1RK1 w kq - 1 9',
				'c8e6' => 'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP2PPP/RNBQ1RK1 w kq - 1 9',
			},
		},
		'r1bqkb1r/2p2ppp/p1n5/1p1pp3/2PPn3/1B3N2/PP3PPP/RNBQ1RK1 b kq c3 0 8' => {
			eco => 'C80k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. c4

			variation => N__('Spanish: Open, Harksen Gambit'),
			moves => {
			},
		},
		'r1bqkb1r/2p2ppp/p1n5/1p1pp3/3Pn3/1B3N2/PPP2PPP/RNBQ1RK1 w kq d6 0 8' => {
			eco => 'C80k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5

			variation => N__('Spanish: Open, 7. Bb3 d5'),
			moves => {
				'a2a4' => 'r1bqkb1r/2p2ppp/p1n5/1p1pp3/P2Pn3/1B3N2/1PP2PPP/RNBQ1RK1 b kq a3 0 8',
				'c2c4' => 'r1bqkb1r/2p2ppp/p1n5/1p1pp3/2PPn3/1B3N2/PP3PPP/RNBQ1RK1 b kq c3 0 8',
				'd4e5' => 'r1bqkb1r/2p2ppp/p1n5/1p1pP3/4n3/1B3N2/PPP2PPP/RNBQ1RK1 b kq - 0 8',
				'f3e5' => 'r1bqkb1r/2p2ppp/p1n5/1p1pN3/3Pn3/1B6/PPP2PPP/RNBQ1RK1 b kq - 0 8',
			},
		},
		'r1bqkb1r/2p2ppp/p1n5/1p1pp3/P2Pn3/1B3N2/1PP2PPP/RNBQ1RK1 b kq a3 0 8' => {
			eco => 'C80k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. a4

			variation => N__('Spanish: Open, 7. Bb3 d5 8. a4'),
			moves => {
				'c6d4' => 'r1bqkb1r/2p2ppp/p7/1p1pp3/P2nn3/1B3N2/1PP2PPP/RNBQ1RK1 w kq - 0 9',
			},
		},
		'r1bqkb1r/2p2ppp/p1np1n2/1p2p1N1/4P3/1B6/PPPP1PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'd6d5' => 'r1bqkb1r/2p2ppp/p1n2n2/1p1pp1N1/4P3/1B6/PPPP1PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqkb1r/2p2ppp/p1np1n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C78f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 d6

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'f3g5' => 'r1bqkb1r/2p2ppp/p1np1n2/1p2p1N1/4P3/1B6/PPPP1PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r1bqkb1r/2p2ppp/p1pp1n2/4p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'C79',
                        # TRANSLATORS:

			variation => N__('Spanish: Steinitz Deferred, Exchange'),
			moves => {
				'c8g4' => 'r2qkb1r/2p2ppp/p1pp1n2/4p3/3PP1b1/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 8',
				'f6e4' => 'r1bqkb1r/2p2ppp/p1pp4/4p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r1bqkb1r/2p2ppp/p1pp1n2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C79',
                        # TRANSLATORS:

			variation => N__('Spanish: Steinitz Deferred, Exchange'),
			moves => {
				'd2d4' => 'r1bqkb1r/2p2ppp/p1pp1n2/4p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 7',
			},
		},
		'r1bqkb1r/2p2ppp/p1pp4/4p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6 6. Bxc6+ bxc6 7. d4 Nxe4

			variation => N__('Spanish: Steinitz Deferred, Rubinstein Variation'),
			moves => {
				'f1e1' => 'r1bqkb1r/2p2ppp/p1pp4/4p3/3Pn3/5N2/PPP2PPP/RNBQR1K1 b kq - 1 8',
			},
		},
		'r1bqkb1r/2p2ppp/p1pp4/4p3/3Pn3/5N2/PPP2PPP/RNBQR1K1 b kq - 1 8' => {
			eco => 'C79',
                        # TRANSLATORS:

			variation => N__('Spanish: Steinitz Deferred, Rubinstein Variation'),
			moves => {
				'f7f5' => 'r1bqkb1r/2p3pp/p1pp4/4pp2/3Pn3/5N2/PPP2PPP/RNBQR1K1 w kq f6 0 9',
			},
		},
		'r1bqkb1r/2p2ppp/p4n2/1p1Pp1N1/3n4/1B6/PPPP1PPP/RNBQ1RK1 w kq - 1 9' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'f1e1' => 'r1bqkb1r/2p2ppp/p4n2/1p1Pp1N1/3n4/1B6/PPPP1PPP/RNBQR1K1 b kq - 2 9',
			},
		},
		'r1bqkb1r/2p2ppp/p4n2/1p1Pp1N1/3n4/1B6/PPPP1PPP/RNBQR1K1 b kq - 2 9' => {
			eco => 'C78f',
                        # TRANSLATORS:

			variation => N__('Spanish: 5.O-O b5 6. Bb3 d6'),
			moves => {
				'f8c5' => 'r1bqk2r/2p2ppp/p4n2/1pbPp1N1/3n4/1B6/PPPP1PPP/RNBQR1K1 w kq - 3 10',
			},
		},
		'r1bqkb1r/2p2ppp/p7/1p1p4/P2pn3/1B6/1PP2PPP/RNBQ1RK1 w kq - 0 10' => {
			eco => 'C80k',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Schlechter Defence'),
			moves => {
				'b1c3' => 'r1bqkb1r/2p2ppp/p7/1p1p4/P2pn3/1BN5/1PP2PPP/R1BQ1RK1 b kq - 1 10',
			},
		},
		'r1bqkb1r/2p2ppp/p7/1p1p4/P2pn3/1BN5/1PP2PPP/R1BQ1RK1 b kq - 1 10' => {
			eco => 'C80k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. a4 Nxd4 9. Nxd4 exd4 10. Nc3

			variation => N__('Spanish: Open, Schlecter, Berger Variation'),
			moves => {
			},
		},
		'r1bqkb1r/2p2ppp/p7/1p1pp3/P2Nn3/1B6/1PP2PPP/RNBQ1RK1 b kq - 0 9' => {
			eco => 'C80k',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Schlechter Defence'),
			moves => {
				'e5d4' => 'r1bqkb1r/2p2ppp/p7/1p1p4/P2pn3/1B6/1PP2PPP/RNBQ1RK1 w kq - 0 10',
			},
		},
		'r1bqkb1r/2p2ppp/p7/1p1pp3/P2nn3/1B3N2/1PP2PPP/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C80k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. a4 Nxd4

			variation => N__('Spanish: Open, Schlechter Defence'),
			moves => {
				'f3d4' => 'r1bqkb1r/2p2ppp/p7/1p1pp3/P2Nn3/1B6/1PP2PPP/RNBQ1RK1 b kq - 0 9',
			},
		},
		'r1bqkb1r/2p3pp/p1p5/3pPp2/4n3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 1 10' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6 6. Bxc6+ bxc6 7. d4 Nxe4 8. Re1 f5 9. dxe5 d5 10. Nc3

			variation => N__('Spanish: Steinitz Deferred, Boleslavsky Variation'),
			moves => {
			},
		},
		'r1bqkb1r/2p3pp/p1p5/3pPp2/4n3/5N2/PPP2PPP/RNBQR1K1 w kq - 0 10' => {
			eco => 'C79',
                        # TRANSLATORS:

			variation => N__('Spanish: Steinitz Deferred, Rubinstein Variation'),
			moves => {
				'b1c3' => 'r1bqkb1r/2p3pp/p1p5/3pPp2/4n3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 1 10',
			},
		},
		'r1bqkb1r/2p3pp/p1pp4/4Pp2/4n3/5N2/PPP2PPP/RNBQR1K1 b kq - 0 9' => {
			eco => 'C79',
                        # TRANSLATORS:

			variation => N__('Spanish: Steinitz Deferred, Rubinstein Variation'),
			moves => {
				'd6d5' => 'r1bqkb1r/2p3pp/p1p5/3pPp2/4n3/5N2/PPP2PPP/RNBQR1K1 w kq - 0 10',
			},
		},
		'r1bqkb1r/2p3pp/p1pp4/4pp2/3Pn3/5N2/PPP2PPP/RNBQR1K1 w kq f6 0 9' => {
			eco => 'C79',
                        # TRANSLATORS:

			variation => N__('Spanish: Steinitz Deferred, Rubinstein Variation'),
			moves => {
				'd4e5' => 'r1bqkb1r/2p3pp/p1pp4/4Pp2/4n3/5N2/PPP2PPP/RNBQR1K1 b kq - 0 9',
			},
		},
		'r1bqkb1r/2pn1ppp/p1Q1pn2/1p6/3P4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 8' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6 7. Nf3 b5 8. Qc6

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 a6 7. Nf3 b5 8. Qc6'),
			moves => {
			},
		},
		'r1bqkb1r/2pn1ppp/p3pn2/1p6/2QP4/5NP1/PP2PPBP/RNB1K2R w KQkq b6 0 8' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6 7. Nf3 b5

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 a6 7. Nf3 b5'),
			moves => {
				'c4c2' => 'r1bqkb1r/2pn1ppp/p3pn2/1p6/3P4/5NP1/PPQ1PPBP/RNB1K2R b KQkq - 1 8',
				'c4c6' => 'r1bqkb1r/2pn1ppp/p1Q1pn2/1p6/3P4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/2pn1ppp/p3pn2/1p6/3P4/5NP1/PPQ1PPBP/RNB1K2R b KQkq - 1 8' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 a6 7. Nf3 b5 8. Qc2

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 a6 7. Nf3 b5 8. Qc2'),
			moves => {
			},
		},
		'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'C78e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3

			variation => N__('Spanish: 5.O-O b5 6. Bb3'),
			moves => {
				'c8b7' => 'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 2 7',
				'd7d6' => 'r1bqkb1r/2p2ppp/p1np1n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 0 7',
				'f8c5' => 'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 2 7',
				'f8e7' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB1K2R b KQkq - 1 6' => {
			eco => 'C77r',
                        # TRANSLATORS:

			variation => N__('Spanish: Wormald Attack, 5... b5'),
			moves => {
				'f8c5' => 'r1bqk2r/2pp1ppp/p1n2n2/1pb1p3/4P3/1B3N2/PPPPQPPP/RNB1K2R w KQkq - 2 7',
				'f8e7' => 'r1bqk2r/2ppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB1K2R w KQkq - 2 7',
			},
		},
		'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B2PP3/5N2/PPP2PPP/RNBQK2R w KQkq b6 0 6' => {
			eco => 'C77k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d4 b5

			variation => N__('Spanish: Centre Attack, 5... b5'),
			moves => {
			},
		},
		'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B3P3/3P1N2/PPP2PPP/RNBQK2R w KQkq b6 0 6' => {
			eco => 'C77c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. d3 b5

			variation => N__('Spanish: Anderssen, 5... b5'),
			moves => {
			},
		},
		'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq b6 0 6' => {
			eco => 'C78e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5

			variation => N__('Spanish: 5.O-O b5'),
			moves => {
				'a4b3' => 'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/B3P3/5N2/PPPPQPPP/RNB1K2R w KQkq b6 0 6' => {
			eco => 'C77r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2 b5

			variation => N__('Spanish: Wormald Attack, 5... b5'),
			moves => {
				'a4b3' => 'r1bqkb1r/2pp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPPQPPP/RNB1K2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/2pp1ppp/p1n5/1p1Pp3/B3n3/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C80i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. d5

			variation => N__('Spanish: Open, Richter Variation'),
			moves => {
			},
		},
		'r1bqkb1r/2pp1ppp/p1n5/1p2N3/B2Pn3/8/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C80i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Nxe5

			variation => N__('Spanish: Open, Friess Attack'),
			moves => {
			},
		},
		'r1bqkb1r/2pp1ppp/p1n5/1p2p3/3Pn3/1B3N2/PPP2PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C80j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3

			variation => N__('Spanish: Open, 7. Bb3'),
			moves => {
				'd7d5' => 'r1bqkb1r/2p2ppp/p1n5/1p1pp3/3Pn3/1B3N2/PPP2PPP/RNBQ1RK1 w kq d6 0 8',
			},
		},
		'r1bqkb1r/2pp1ppp/p1n5/1p2p3/B2Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq b6 0 7' => {
			eco => 'C80i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5

			variation => N__('Spanish: Open, 6. d4 b5'),
			moves => {
				'a4b3' => 'r1bqkb1r/2pp1ppp/p1n5/1p2p3/3Pn3/1B3N2/PPP2PPP/RNBQ1RK1 b kq - 1 7',
				'd4d5' => 'r1bqkb1r/2pp1ppp/p1n5/1p1Pp3/B3n3/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
				'f3e5' => 'r1bqkb1r/2pp1ppp/p1n5/1p2N3/B2Pn3/8/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqkb1r/3n1p1p/4pp2/1p6/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 13' => {
			eco => 'D49k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 axb5 12. exf6 gxf6

			variation => N__('Semi-Slav: Meran, Old Main Line, 11.Nxb5 axb5 12.exf6 gxf6'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/4pP2/1p6/3p4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 12' => {
			eco => 'D49i',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Old Main Line, 11.Nxb5 axb5'),
			moves => {
				'd8b6' => 'r1b1kb1r/3n1ppp/1q2pP2/1p6/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 1 13',
				'g7f6' => 'r1bqkb1r/3n1p1p/4pp2/1p6/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 13',
			},
		},
		'r1bqkb1r/3n1ppp/4pn2/1p2P3/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 12' => {
			eco => 'D49i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 axb5

			variation => N__('Semi-Slav: Meran, Old Main Line, 11.Nxb5 axb5'),
			moves => {
				'e5f6' => 'r1bqkb1r/3n1ppp/4pP2/1p6/3p4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 12',
			},
		},
		'r1bqkb1r/3n1ppp/p1n1p3/1pppP3/3P1P2/2N1BN2/PPPQ2PP/R3KB1R w KQkq b6 0 9' => {
			eco => 'C11k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3 Nc6 7. Be3 a6 8. Qd2 b5

			variation => N__('French: Steinitz, Boleslavsky, 7... a6 8. Qd2 b5'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'D47e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Be2 a6

			variation => N__('Semi-Slav: Meran, Capablanca, 8... a6'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 1 9' => {
			eco => 'D48c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O'),
			moves => {
				'c6c5' => 'r1bqkb1r/3n1ppp/p3pn2/1pp5/3P4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 0 10',
				'c8b7' => 'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 2 10',
			},
		},
		'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'D48a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6

			variation => N__('Semi-Slav: Meran, 8... a6'),
			moves => {
				'a2a4' => 'r1bqkb1r/3n1ppp/p1p1pn2/1p6/P2P4/2NBPN2/1P3PPP/R1BQK2R b KQkq a3 0 9',
				'e1g1' => 'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 1 9',
				'e3e4' => 'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 9',
			},
		},
		'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 9' => {
			eco => 'D48h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4

			variation => N__('Semi-Slav: Meran, 8... a6 9. e4'),
			moves => {
				'b5b4' => 'r1bqkb1r/3n1ppp/p1p1pn2/8/1p1PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 10',
				'c6c5' => 'r1bqkb1r/3n1ppp/p3pn2/1pp5/3PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 10',
				'c8b7' => 'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 10',
			},
		},
		'r1bqkb1r/3n1ppp/p1p1pn2/1p6/P2P4/2NBPN2/1P3PPP/R1BQK2R b KQkq a3 0 9' => {
			eco => 'D48b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. a4

			variation => N__('Semi-Slav: Meran, 8... a6 9. a4'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p1p1pn2/8/1p1PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'D48j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 b4

			variation => N__('Semi-Slav: Meran, Pirc Variation'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p2ppn2/1p6/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 2 9' => {
			eco => 'B80m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 b5 8. Qd2 Nbd7

			variation => N__('Sicilian: Scheveningen, English, 7... b5 8. Qd2 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p3Pn2/1p6/2p1P3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 11' => {
			eco => 'D48o',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Reynolds, 10... c4'),
			moves => {
				'f7e6' => 'r1bqkb1r/3n2pp/p3pn2/1p6/2p1P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 12',
			},
		},
		'r1bqkb1r/3n1ppp/p3p3/1N2P3/3p2n1/3B1N2/PP3PPP/R1BQK2R w KQkq - 1 12' => {
			eco => 'D49b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Ng4

			variation => N__('Semi-Slav: Meran, Old Main Line, Gligoric Variation'),
			moves => {
				'd1a4' => 'r1bqkb1r/3n1ppp/p3p3/1N2P3/Q2p2n1/3B1N2/PP3PPP/R1B1K2R b KQkq - 2 12',
			},
		},
		'r1bqkb1r/3n1ppp/p3p3/1N2P3/Q2p2n1/3B1N2/PP3PPP/R1B1K2R b KQkq - 2 12' => {
			eco => 'D49c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Ng4 12. Qa4

			variation => N__('Semi-Slav: Meran, Old Main Line, Gligoric, 12. Qa4'),
			moves => {
				'g4e5' => 'r1bqkb1r/3n1ppp/p3p3/1N2n3/Q2p4/3B1N2/PP3PPP/R1B1K2R w KQkq - 0 13',
			},
		},
		'r1bqkb1r/3n1ppp/p3p3/1N2n3/Q2p4/3B1N2/PP3PPP/R1B1K2R w KQkq - 0 13' => {
			eco => 'D49d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Ng4 12. Qa4 Ngxe5

			variation => N__('Semi-Slav: Meran, Trifunovic Variation'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p3p3/1pp1P3/3P2n1/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 11' => {
			eco => 'D48r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 Ng4

			variation => N__('Semi-Slav: Meran, Rabinovich Variation'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1N2P3/3p4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 11' => {
			eco => 'D49a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5

			variation => N__('Semi-Slav: Meran, Blumenfeld Variation'),
			moves => {
				'a6b5' => 'r1bqkb1r/3n1ppp/4pn2/1p2P3/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 12',
				'd7e5' => 'r1bqkb1r/5ppp/p3pn2/1N2n3/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 12',
				'f6g4' => 'r1bqkb1r/3n1ppp/p3p3/1N2P3/3p2n1/3B1N2/PP3PPP/R1BQK2R w KQkq - 1 12',
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1p1P4/2p1P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 11' => {
			eco => 'D48o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 c4

			variation => N__('Semi-Slav: Meran, Reynolds, 10... c4'),
			moves => {
				'd5e6' => 'r1bqkb1r/3n1ppp/p3Pn2/1p6/2p1P3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 11',
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1p2P3/3p4/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 11' => {
			eco => 'D48s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4

			variation => N__('Semi-Slav: Meran, Old Main Line, 10...cxd4'),
			moves => {
				'c3b5' => 'r1bqkb1r/3n1ppp/p3pn2/1N2P3/3p4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 11',
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1pp1P3/3P4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 10' => {
			eco => 'D48q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5

			variation => N__('Semi-Slav: Meran, Old Main Line'),
			moves => {
				'c5d4' => 'r1bqkb1r/3n1ppp/p3pn2/1p2P3/3p4/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 11',
				'f6g4' => 'r1bqkb1r/3n1ppp/p3p3/1pp1P3/3P2n1/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 11',
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1pp5/3P4/2NBPN2/PP2QPPP/R1B2RK1 b kq - 1 10' => {
			eco => 'D48d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O c5 10. Qe2

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O c5 10. Qe2'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1pp5/3P4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'D48c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O c5

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O c5'),
			moves => {
				'd1e2' => 'r1bqkb1r/3n1ppp/p3pn2/1pp5/3P4/2NBPN2/PP2QPPP/R1B2RK1 b kq - 1 10',
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1pp5/3PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'D48k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5

			variation => N__('Semi-Slav: Meran, 8... a6 9. e4 c5'),
			moves => {
				'd4d5' => 'r1bqkb1r/3n1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 10',
				'e4e5' => 'r1bqkb1r/3n1ppp/p3pn2/1pp1P3/3P4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 10',
			},
		},
		'r1bqkb1r/3n1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 10' => {
			eco => 'D48l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5

			variation => N__('Semi-Slav: Meran, Reynolds Variation'),
			moves => {
				'c5c4' => 'r1bqkb1r/3n1ppp/p3pn2/1p1P4/2p1P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 11',
				'd8c7' => 'r1b1kb1r/2qn1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 11',
				'e6e5' => 'r1bqkb1r/3n1ppp/p4n2/1ppPp3/4P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 11',
			},
		},
		'r1bqkb1r/3n1ppp/p4n2/1ppPp3/4P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 11' => {
			eco => 'D48m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 e5

			variation => N__('Semi-Slav: Meran, Reynolds, 10... e5'),
			moves => {
			},
		},
		'r1bqkb1r/3n1ppp/pp2pn2/2p5/3P4/5NP1/PPQ1PPBP/RNB1K2R w KQkq - 0 9' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5 7. Nf3 a6 8. Qc2 b6

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 c5 7. Nf3 a6 8. Qc2 b6'),
			moves => {
			},
		},
		'r1bqkb1r/3n2pp/p3pn2/1p6/2p1P3/2N2N2/PPB2PPP/R1BQK2R b KQkq - 1 12' => {
			eco => 'D48p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 c4 11. dxe6 fxe6 12. Bc2

			variation => N__('Semi-Slav: Meran, Reynolds, 10... c4 11.dxe6 fxe6 12. Bc2'),
			moves => {
				'd8c7' => 'r1b1kb1r/2qn2pp/p3pn2/1p6/2p1P3/2N2N2/PPB2PPP/R1BQK2R w KQkq - 2 13',
			},
		},
		'r1bqkb1r/3n2pp/p3pn2/1p6/2p1P3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 12' => {
			eco => 'D48p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. d5 c4 11. dxe6 fxe6

			variation => N__('Semi-Slav: Meran, Reynolds, 10... c4 11.dxe6 fxe6'),
			moves => {
				'd3c2' => 'r1bqkb1r/3n2pp/p3pn2/1p6/2p1P3/2N2N2/PPB2PPP/R1BQK2R b KQkq - 1 12',
			},
		},
		'r1bqkb1r/3npppp/p2p1n2/1p4B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq b6 0 8' => {
			eco => 'B94f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 Nbd7 7. f4 b5

			variation => N__('Sicilian: Najdorf, 6. Bg5 Nbd7 7. f4 b5'),
			moves => {
			},
		},
		'r1bqkb1r/4nppp/p1np4/1p1Np3/4P3/N7/PPP2PPP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'B32r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. N1c3 a6 7. Na3 b5 8. Nd5 Nge7

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6.N1c3 a6 7. Na3 b5 8. Nd5 Nge7'),
			moves => {
			},
		},
		'r1bqkb1r/5p1p/p1np1p2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R b KQkq - 1 10' => {
			eco => 'B33s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9.Bxf6 gxf6 10. Nd5'),
			moves => {
				'f6f5' => 'r1bqkb1r/5p1p/p1np4/1p1Npp2/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 0 11',
				'f8g7' => 'r1bqk2r/5pbp/p1np1p2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 2 11',
			},
		},
		'r1bqkb1r/5p1p/p1np1p2/1p2p3/4P3/N1N5/PPP2PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'B33s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9.Bxf6 gxf6'),
			moves => {
				'c3d5' => 'r1bqkb1r/5p1p/p1np1p2/1p1Np3/4P3/N7/PPP2PPP/R2QKB1R b KQkq - 1 10',
			},
		},
		'r1bqkb1r/5p1p/p1np4/1p1Npp2/4P3/N1P5/PP3PPP/R2QKB1R b KQkq - 0 11' => {
			eco => 'B33v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. c3

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. c3'),
			moves => {
				'f8g7' => 'r1bqk2r/5pbp/p1np4/1p1Npp2/4P3/N1P5/PP3PPP/R2QKB1R w KQkq - 1 12',
			},
		},
		'r1bqkb1r/5p1p/p1np4/1p1Npp2/4P3/N2B4/PPP2PPP/R2QK2R b KQkq - 1 11' => {
			eco => 'B33x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. Bd3

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. Bd3'),
			moves => {
				'c8e6' => 'r2qkb1r/5p1p/p1npb3/1p1Npp2/4P3/N2B4/PPP2PPP/R2QK2R w KQkq - 2 12',
			},
		},
		'r1bqkb1r/5p1p/p1np4/1p1Npp2/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 0 11' => {
			eco => 'B33u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5

			variation => N__('Sicilian: Pelikan, Sveshnikov Variation'),
			moves => {
				'c2c3' => 'r1bqkb1r/5p1p/p1np4/1p1Npp2/4P3/N1P5/PP3PPP/R2QKB1R b KQkq - 0 11',
				'f1d3' => 'r1bqkb1r/5p1p/p1np4/1p1Npp2/4P3/N2B4/PPP2PPP/R2QK2R b KQkq - 1 11',
			},
		},
		'r1bqkb1r/5ppp/4pn2/1B2N3/3p4/8/PP3PPP/R1BQK2R b KQkq - 0 13' => {
			eco => 'D49h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Nxe5 12. Nxe5 axb5 13. Bxb5+

			variation => N__('Semi-Slav: Meran, Sozin, 13.Bxb5+'),
			moves => {
			},
		},
		'r1bqkb1r/5ppp/4pn2/1p2N3/3p4/3B1Q2/PP3PPP/R1B1K2R b KQkq - 1 13' => {
			eco => 'D49g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Nxe5 12. Nxe5 axb5 13. Qf3

			variation => N__('Semi-Slav: Meran, Sozin, Stahlberg Attack'),
			moves => {
				'f8b4' => 'r1bqk2r/5ppp/4pn2/1p2N3/1b1p4/3B1Q2/PP3PPP/R1B1K2R w KQkq - 2 14',
			},
		},
		'r1bqkb1r/5ppp/4pn2/1p2N3/3p4/3B4/PP3PPP/R1BQ1RK1 b kq - 1 13' => {
			eco => 'D49f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Nxe5 12. Nxe5 axb5 13. O-O

			variation => N__('Semi-Slav: Meran, Sozin, Rellstab Attack'),
			moves => {
			},
		},
		'r1bqkb1r/5ppp/4pn2/1p2N3/3p4/3B4/PP3PPP/R1BQK2R w KQkq - 0 13' => {
			eco => 'D49e',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Sozin Variation'),
			moves => {
				'd1f3' => 'r1bqkb1r/5ppp/4pn2/1p2N3/3p4/3B1Q2/PP3PPP/R1B1K2R b KQkq - 1 13',
				'd3b5' => 'r1bqkb1r/5ppp/4pn2/1B2N3/3p4/8/PP3PPP/R1BQK2R b KQkq - 0 13',
				'e1g1' => 'r1bqkb1r/5ppp/4pn2/1p2N3/3p4/3B4/PP3PPP/R1BQ1RK1 b kq - 1 13',
			},
		},
		'r1bqkb1r/5ppp/p1n1pn2/1p6/2pP4/1B2PN2/PP2QPPP/RNBR2K1 w kq - 0 10' => {
			eco => 'D28k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Nc6 9. Rd1 c4

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3 Nc6 9. Rd1 c4'),
			moves => {
			},
		},
		'r1bqkb1r/5ppp/p1n1pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNB2RK1 w kq - 2 9' => {
			eco => 'D28i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Nc6

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3 Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/5ppp/p1n1pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 b kq - 3 9',
				'f1d1' => 'r1bqkb1r/5ppp/p1n1pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNBR2K1 b kq - 3 9',
			},
		},
		'r1bqkb1r/5ppp/p1n1pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNBR2K1 b kq - 3 9' => {
			eco => 'D28j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Nc6 9. Rd1

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3 Nc6 9. Rd1'),
			moves => {
				'c5c4' => 'r1bqkb1r/5ppp/p1n1pn2/1p6/2pP4/1B2PN2/PP2QPPP/RNBR2K1 w kq - 0 10',
			},
		},
		'r1bqkb1r/5ppp/p1n1pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 b kq - 3 9' => {
			eco => 'D28l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Nc6 9. Nc3

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3 Nc6 9. Nc3'),
			moves => {
				'c8b7' => 'r2qkb1r/1b3ppp/p1n1pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 w kq - 4 10',
				'f8e7' => 'r1bqk2r/4bppp/p1n1pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 w kq - 4 10',
			},
		},
		'r1bqkb1r/5ppp/p1np1B2/1p2p3/4P3/N1N5/PPP2PPP/R2QKB1R b KQkq - 0 9' => {
			eco => 'B33s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9.Bxf6'),
			moves => {
				'g7f6' => 'r1bqkb1r/5p1p/p1np1p2/1p2p3/4P3/N1N5/PPP2PPP/R2QKB1R w KQkq - 0 10',
			},
		},
		'r1bqkb1r/5ppp/p1np1n2/1p1Np1B1/4P3/N7/PPP2PPP/R2QKB1R b KQkq - 1 9' => {
			eco => 'B33l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Nd5

			variation => N__('Sicilian: Pelikan, Chelyabinsk, 9. Nd5'),
			moves => {
				'f8e7' => 'r1bqk2r/4bppp/p1np1n2/1p1Np1B1/4P3/N7/PPP2PPP/R2QKB1R w KQkq - 2 10',
			},
		},
		'r1bqkb1r/5ppp/p1np1n2/1p2p1B1/4P3/N1N5/PPP2PPP/R2QKB1R w KQkq b6 0 9' => {
			eco => 'B33l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5

			variation => N__('Sicilian: Pelikan, Chelyabinsk Variation'),
			moves => {
				'c3d5' => 'r1bqkb1r/5ppp/p1np1n2/1p1Np1B1/4P3/N7/PPP2PPP/R2QKB1R b KQkq - 1 9',
				'g5f6' => 'r1bqkb1r/5ppp/p1np1B2/1p2p3/4P3/N1N5/PPP2PPP/R2QKB1R b KQkq - 0 9',
			},
		},
		'r1bqkb1r/5ppp/p3pn2/1N2N3/3p4/3B4/PP3PPP/R1BQK2R b KQkq - 0 12' => {
			eco => 'D49e',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Sozin Variation'),
			moves => {
				'a6b5' => 'r1bqkb1r/5ppp/4pn2/1p2N3/3p4/3B4/PP3PPP/R1BQK2R w KQkq - 0 13',
			},
		},
		'r1bqkb1r/5ppp/p3pn2/1N2n3/3p4/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 12' => {
			eco => 'D49e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 c5 10. e5 cxd4 11. Nxb5 Nxe5

			variation => N__('Semi-Slav: Meran, Sozin Variation'),
			moves => {
				'f3e5' => 'r1bqkb1r/5ppp/p3pn2/1N2N3/3p4/3B4/PP3PPP/R1BQK2R b KQkq - 0 12',
			},
		},
		'r1bqkb1r/5ppp/ppnppn2/8/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 0 9' => {
			eco => 'B44s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 b6

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3 a6 8. Na3 b6'),
			moves => {
			},
		},
		'r1bqkb1r/p1p2Npp/8/1p1np3/3n4/2P5/PP1P1PPP/RNBQKB1R b KQkq - 0 8' => {
			eco => 'C57j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5 8. Nxf7

			variation => N__('Two Knights: Fritz, 8.Nxf7'),
			moves => {
			},
		},
		'r1bqkb1r/p1p2ppp/2n2n2/1p1Pp1N1/2B5/8/PPPP1PPP/RNBQK2R w KQkq b6 0 6' => {
			eco => 'C57h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 b5

			variation => N__('Two Knights: Ulvestad Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p1p2ppp/5n2/1p1Pp1N1/2Bn4/2P5/PP1P1PPP/RNBQK2R w KQkq b6 0 7' => {
			eco => 'C57i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz Variation'),
			moves => {
				'c4f1' => 'r1bqkb1r/p1p2ppp/5n2/1p1Pp1N1/3n4/2P5/PP1P1PPP/RNBQKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/p1p2ppp/5n2/1p1Pp1N1/3n4/2P5/PP1P1PPP/RNBQKB1R b KQkq - 1 7' => {
			eco => 'C57i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz Variation'),
			moves => {
				'f6d5' => 'r1bqkb1r/p1p2ppp/8/1p1np1N1/3n4/2P5/PP1P1PPP/RNBQKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/p1p2ppp/8/1p1np1N1/3P4/8/PP1P1PPP/RNBQKB1R b KQkq - 0 8' => {
			eco => 'C57j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5 8. cxd4

			variation => N__('Two Knights: Fritz, 8.cxd5'),
			moves => {
			},
		},
		'r1bqkb1r/p1p2ppp/8/1p1np1N1/3n3P/2P5/PP1P1PP1/RNBQKB1R b KQkq h3 0 8' => {
			eco => 'C57j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5 8. h4

			variation => N__('Two Knights: Fritz, Paoli\'s 8. h4'),
			moves => {
			},
		},
		'r1bqkb1r/p1p2ppp/8/1p1np1N1/3n4/2P5/PP1P1PPP/RNBQKB1R w KQkq - 0 8' => {
			eco => 'C57i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5

			variation => N__('Two Knights: Fritz Variation, Main Line'),
			moves => {
				'c3d4' => 'r1bqkb1r/p1p2ppp/8/1p1np1N1/3P4/8/PP1P1PPP/RNBQKB1R b KQkq - 0 8',
				'g5e4' => 'r1bqkb1r/p1p2ppp/8/1p1np3/3nN3/2P5/PP1P1PPP/RNBQKB1R b KQkq - 1 8',
				'g5f7' => 'r1bqkb1r/p1p2Npp/8/1p1np3/3n4/2P5/PP1P1PPP/RNBQKB1R b KQkq - 0 8',
				'h2h4' => 'r1bqkb1r/p1p2ppp/8/1p1np1N1/3n3P/2P5/PP1P1PP1/RNBQKB1R b KQkq h3 0 8',
			},
		},
		'r1bqkb1r/p1p2ppp/8/1p1np3/3nN3/2P5/PP1P1PPP/RNBQKB1R b KQkq - 1 8' => {
			eco => 'C57k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5 8. Ne4

			variation => N__('Two Knights: Fritz, Gruber Variation (8. Ne4)'),
			moves => {
				'd8h4' => 'r1b1kb1r/p1p2ppp/8/1p1np3/3nN2q/2P5/PP1P1PPP/RNBQKB1R w KQkq - 2 9',
			},
		},
		'r1bqkb1r/p1pp1ppp/2p2n2/4P3/8/8/PPP2PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C45j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. e5

			variation => N__('Scotch: Mieses Variation'),
			moves => {
				'd8e7' => 'r1b1kb1r/p1ppqppp/2p2n2/4P3/8/8/PPP2PPP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/p1pp1ppp/2p2n2/8/4P3/3B4/PPP2PPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'C45i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. Bd3

			variation => N__('Scotch: 4.Nxd4 Nf6 5.Nxc6 bxc6 6. Bd3'),
			moves => {
			},
		},
		'r1bqkb1r/p1pp1ppp/2p2n2/8/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C45h',
                        # TRANSLATORS:

			variation => N__('Scotch: 4.Nxd4 Nf6 5.Nxc6'),
			moves => {
				'b1d2' => 'r1bqkb1r/p1pp1ppp/2p2n2/8/4P3/8/PPPN1PPP/R1BQKB1R b KQkq - 1 6',
				'e4e5' => 'r1bqkb1r/p1pp1ppp/2p2n2/4P3/8/8/PPP2PPP/RNBQKB1R b KQkq - 0 6',
				'f1d3' => 'r1bqkb1r/p1pp1ppp/2p2n2/8/4P3/3B4/PPP2PPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/p1pp1ppp/2p2n2/8/4P3/8/PPPN1PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'C45h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. Nd2

			variation => N__('Scotch: Tartakower Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p2n1p2/2p1pP2/1p4B1/2pP4/2N5/PP3PPP/R2QKB1R b KQkq - 0 11' => {
			eco => 'D44o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. exf6

			variation => N__('Semi-Slav: Botvinnik, Main Line'),
			moves => {
				'c8b7' => 'r2qkb1r/pb1n1p2/2p1pP2/1p4B1/2pP4/2N5/PP3PPP/R2QKB1R w KQkq - 1 12',
			},
		},
		'r1bqkb1r/p2n1p2/2p1pn2/1p2P1B1/2pP4/2N2Q2/PP3PPP/R3KB1R b KQkq - 2 11' => {
			eco => 'D44n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. Qf3

			variation => N__('Semi-Slav: Botvinnik, Szabo Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p2n1p2/2p1pn2/1p2P1B1/2pP4/2N3P1/PP3P1P/R2QKB1R b KQkq - 0 11' => {
			eco => 'D44k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. g3

			variation => N__('Semi-Slav: Botvinnik, Lilienthal Variation'),
			moves => {
				'c8b7' => 'r2qkb1r/pb1n1p2/2p1pn2/1p2P1B1/2pP4/2N3P1/PP3P1P/R2QKB1R w KQkq - 1 12',
				'd8a5' => 'r1b1kb1r/p2n1p2/2p1pn2/qp2P1B1/2pP4/2N3P1/PP3P1P/R2QKB1R w KQkq - 1 12',
			},
		},
		'r1bqkb1r/p2n1p2/2p1pn2/1p2P1B1/2pP4/2N5/PP3PPP/R2QKB1R w KQkq - 1 11' => {
			eco => 'D44j',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, 9.Nxg5'),
			moves => {
				'd1f3' => 'r1bqkb1r/p2n1p2/2p1pn2/1p2P1B1/2pP4/2N2Q2/PP3PPP/R3KB1R b KQkq - 2 11',
				'e5f6' => 'r1bqkb1r/p2n1p2/2p1pP2/1p4B1/2pP4/2N5/PP3PPP/R2QKB1R b KQkq - 0 11',
				'g2g3' => 'r1bqkb1r/p2n1p2/2p1pn2/1p2P1B1/2pP4/2N3P1/PP3P1P/R2QKB1R b KQkq - 0 11',
			},
		},
		'r1bqkb1r/p2n1ppp/1pp1pn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'D45i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 b6

			variation => N__('Semi-Slav: 6. Qc2 b6'),
			moves => {
			},
		},
		'r1bqkb1r/p2n1ppp/2p1p3/8/1p1Pn3/3BPN2/PP3PPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'D47k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 b4 9. Ne4 Nxe4

			variation => N__('Semi-Slav: Meran, Lundin, 9. Ne4 Nxe4'),
			moves => {
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/1p6/2BP4/2N1PN2/PP3PPP/R1BQK2R w KQkq b6 0 8' => {
			eco => 'D47a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5

			variation => N__('Semi-Slav: Meran, 7... b5'),
			moves => {
				'c4b3' => 'r1bqkb1r/p2n1ppp/2p1pn2/1p6/3P4/1BN1PN2/PP3PPP/R1BQK2R b KQkq - 1 8',
				'c4d3' => 'r1bqkb1r/p2n1ppp/2p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 8',
				'c4e2' => 'r1bqkb1r/p2n1ppp/2p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/1p6/3P4/1BN1PN2/PP3PPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'D47b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bb3

			variation => N__('Semi-Slav: Meran, Freymann Variation'),
			moves => {
				'b5b4' => 'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1P4/1BN1PN2/PP3PPP/R1BQK2R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'D47d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Be2

			variation => N__('Semi-Slav: Meran, Capablanca Variation'),
			moves => {
				'a7a6' => 'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 9',
				'b5b4' => 'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 9',
				'c8b7' => 'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 2 9',
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'D47h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3

			variation => N__('Semi-Slav: Meran, 8. Bd3'),
			moves => {
				'a7a6' => 'r1bqkb1r/3n1ppp/p1p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 9',
				'b5b4' => 'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 9',
				'c8b7' => 'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 2 9',
				'f8d6' => 'r1bqk2r/p2n1ppp/2pbpn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 2 9',
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1P4/1BN1PN2/PP3PPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'D47c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bb3 b4

			variation => N__('Semi-Slav: Meran, Freymann, 8... b4'),
			moves => {
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'D47d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Be2 b4

			variation => N__('Semi-Slav: Meran, Capablanca, 8... b4'),
			moves => {
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'D47i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 b4

			variation => N__('Semi-Slav: Meran, Lundin Variation'),
			moves => {
				'c3a4' => 'r1bqkb1r/p2n1ppp/2p1pn2/8/Np1P4/3BPN2/PP3PPP/R1BQK2R b KQkq - 1 9',
				'c3e4' => 'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQK2R b KQkq - 1 9',
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQK2R b KQkq - 1 9' => {
			eco => 'D47j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 b4 9. Ne4

			variation => N__('Semi-Slav: Meran, Lundin, 9. Ne4'),
			moves => {
				'f6e4' => 'r1bqkb1r/p2n1ppp/2p1p3/8/1p1Pn3/3BPN2/PP3PPP/R1BQK2R w KQkq - 0 10',
				'f8e7' => 'r1bqk2r/p2nbppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQK2R w KQkq - 2 10',
			},
		},
		'r1bqkb1r/p2n1ppp/2p1pn2/8/Np1P4/3BPN2/PP3PPP/R1BQK2R b KQkq - 1 9' => {
			eco => 'D47i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 b4 9. Na4

			variation => N__('Semi-Slav: Meran, Lundin, 9. Na4'),
			moves => {
			},
		},
		'r1bqkb1r/p2p1ppp/2p1p3/3nP3/4N3/8/PPP2PPP/R1BQKB1R b KQkq - 2 8' => {
			eco => 'B45m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Nxc6 bxc6 7. e5 Nd5 8. Ne4

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Nxc6 bxc6 7. e5 Nd5 8. Ne4'),
			moves => {
				'd8c7' => 'r1b1kb1r/p1qp1ppp/2p1p3/3nP3/4N3/8/PPP2PPP/R1BQKB1R w KQkq - 3 9',
			},
		},
		'r1bqkb1r/p2p1ppp/2p1p3/3nP3/8/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'B45l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Nxc6 bxc6 7. e5 Nd5

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Nxc6 bxc6 7. e5 Nd5'),
			moves => {
				'c3e4' => 'r1bqkb1r/p2p1ppp/2p1p3/3nP3/4N3/8/PPP2PPP/R1BQKB1R b KQkq - 2 8',
			},
		},
		'r1bqkb1r/p2p1ppp/2p1pn2/4P3/8/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B45l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Nxc6 bxc6 7. e5

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Nxc6 bxc6 7. e5'),
			moves => {
				'f6d5' => 'r1bqkb1r/p2p1ppp/2p1p3/3nP3/8/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/p2p1ppp/2p1pn2/8/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B45k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Nxc6'),
			moves => {
				'e4e5' => 'r1bqkb1r/p2p1ppp/2p1pn2/4P3/8/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/p2ppp1p/2p2np1/8/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B34i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Nf6 6. Nxc6 bxc6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 5... Nf6 6.Nxc6 bxc6'),
			moves => {
			},
		},
		'r1bqkb1r/p3nppp/1pn1p3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Nge7 6. Nbd2 b6 7. e4

			variation => N__('Reti: KIA, French Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p3nppp/1pn1p3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e2e4' => 'r1bqkb1r/p3nppp/1pn1p3/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7',
			},
		},
		'r1bqkb1r/p3pp1p/2pp1np1/4P3/2B5/2N5/PPP2PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'B57a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 g6 7. Nxc6 bxc6 8. e5

			variation => N__('Sicilian: Sozin, Magnus Smith Trap'),
			moves => {
			},
		},
		'r1bqkb1r/p3pp1p/2pp1np1/8/2B1P3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B57a',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin'),
			moves => {
				'e4e5' => 'r1bqkb1r/p3pp1p/2pp1np1/4P3/2B5/2N5/PPP2PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/p4pp1/2p2n1p/n3N3/4p3/8/PPPPBPPP/RNBQK2R b KQkq - 1 10' => {
			eco => 'C59c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4 10. Ne5

			variation => N__('Two Knights: Morphy, Main Line 9. Nf3 e4 10. Ne5'),
			moves => {
				'd8c7' => 'r1b1kb1r/p1q2pp1/2p2n1p/n3N3/4p3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11',
				'd8d4' => 'r1b1kb1r/p4pp1/2p2n1p/n3N3/3qp3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11',
				'f8c5' => 'r1bqk2r/p4pp1/2p2n1p/n1b1N3/4p3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11',
				'f8d6' => 'r1bqk2r/p4pp1/2pb1n1p/n3N3/4p3/8/PPPPBPPP/RNBQK2R w KQkq - 2 11',
			},
		},
		'r1bqkb1r/p4pp1/2p2n1p/n3p1N1/8/8/PPPPBPPP/RNBQK2R w KQkq - 0 9' => {
			eco => 'C59a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6

			variation => N__('Two Knights: Morphy, 8. Be2 h6'),
			moves => {
				'g5f3' => 'r1bqkb1r/p4pp1/2p2n1p/n3p3/8/5N2/PPPPBPPP/RNBQK2R b KQkq - 1 9',
				'g5h3' => 'r1bqkb1r/p4pp1/2p2n1p/n3p3/8/7N/PPPPBPPP/RNBQK2R b KQkq - 1 9',
			},
		},
		'r1bqkb1r/p4pp1/2p2n1p/n3p3/8/5N2/PPPPBPPP/RNBQK2R b KQkq - 1 9' => {
			eco => 'C59c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3

			variation => N__('Two Knights: Morphy, Main Line 9. Nf3'),
			moves => {
				'e5e4' => 'r1bqkb1r/p4pp1/2p2n1p/n7/4p3/5N2/PPPPBPPP/RNBQK2R w KQkq - 0 10',
			},
		},
		'r1bqkb1r/p4pp1/2p2n1p/n3p3/8/7N/PPPPBPPP/RNBQK2R b KQkq - 1 9' => {
			eco => 'C59b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nh3

			variation => N__('Two Knights: Morphy, Steinitz Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p4pp1/2p2n1p/n7/4p3/5N2/PPPPBPPP/RNBQK2R w KQkq - 0 10' => {
			eco => 'C59c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2 h6 9. Nf3 e4

			variation => N__('Two Knights: Morphy, Main Line 9. Nf3 e4'),
			moves => {
				'f3e5' => 'r1bqkb1r/p4pp1/2p2n1p/n3N3/4p3/8/PPPPBPPP/RNBQK2R b KQkq - 1 10',
			},
		},
		'r1bqkb1r/p4ppp/1pn1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O e6 5. d3 Nf6 6. Nbd2 b6 7. e4

			variation => N__('Reti: KIA, French Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p4ppp/1pn1pn2/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e2e4' => 'r1bqkb1r/p4ppp/1pn1pn2/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7',
			},
		},
		'r1bqkb1r/p4ppp/2p2n2/1N6/5p2/2N5/PP2PPPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'd1d8' => 'r1bQkb1r/p4ppp/2p2n2/1N6/5p2/2N5/PP2PPPP/R3KB1R b KQkq - 0 10',
			},
		},
		'r1bqkb1r/p4ppp/2p2n2/n3p1N1/8/8/PPPPBPPP/RNBQK2R b KQkq - 1 8' => {
			eco => 'C58h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Be2

			variation => N__('Two Knights: Morphy, 8. Be2'),
			moves => {
				'h7h6' => 'r1bqkb1r/p4pp1/2p2n1p/n3p1N1/8/8/PPPPBPPP/RNBQK2R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/p4ppp/2p2n2/nB2p1N1/8/5Q2/PPPP1PPP/RNB1K2R b KQkq - 1 8' => {
			eco => 'C58f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Qf3

			variation => N__('Two Knights: Morphy, Bogoljubow Variation'),
			moves => {
				'a8b8' => '1rbqkb1r/p4ppp/2p2n2/nB2p1N1/8/5Q2/PPPP1PPP/RNB1K2R w KQk - 2 9',
				'c6b5' => 'r1bqkb1r/p4ppp/5n2/np2p1N1/8/5Q2/PPPP1PPP/RNB1K2R w KQkq - 0 9',
				'd8c7' => 'r1b1kb1r/p1q2ppp/2p2n2/nB2p1N1/8/5Q2/PPPP1PPP/RNB1K2R w KQkq - 2 9',
			},
		},
		'r1bqkb1r/p4ppp/2p2n2/nB2p1N1/8/8/PPPP1PPP/RNBQK2R w KQkq - 0 8' => {
			eco => 'C58e',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Polerio, 6... c6'),
			moves => {
				'b5e2' => 'r1bqkb1r/p4ppp/2p2n2/n3p1N1/8/8/PPPPBPPP/RNBQK2R b KQkq - 1 8',
				'd1f3' => 'r1bqkb1r/p4ppp/2p2n2/nB2p1N1/8/5Q2/PPPP1PPP/RNB1K2R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/p4ppp/2pp1n2/4p3/4P3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B58g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nxc6 bxc6

			variation => N__('Sicilian: Boleslavsky, Louma Variation'),
			moves => {
			},
		},
		'r1bqkb1r/p4ppp/5n2/np2p1N1/8/5Q2/PPPP1PPP/RNB1K2R w KQkq - 0 9' => {
			eco => 'C58f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6 7. dxc6 bxc6 8. Qf3 cxb5

			variation => N__('Two Knights: Morphy, Blackburne Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1p1p/3p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Nd2 Nbd7

			variation => N__('Benoni: Nimzowitsch, 7...Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2PpP3/5P2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'C11g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. dxc5 Nc6

			variation => N__('French: Steinitz, 6.dxc5 Nc6'),
			moves => {
				'a2a3' => 'r1bqkb1r/pp1n1ppp/2n1p3/2PpP3/5P2/P1N5/1PP3PP/R1BQKBNR b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2PpP3/5P2/P1N5/1PP3PP/R1BQKBNR b KQkq - 0 7' => {
			eco => 'C11g',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6.dxc5 Nc6'),
			moves => {
				'f8c5' => 'r1bqk2r/pp1n1ppp/2n1p3/2bpP3/5P2/P1N5/1PP3PP/R1BQKBNR w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2N1BN2/PPP3PP/R2QKB1R b KQkq - 3 7' => {
			eco => 'C11i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3 Nc6 7. Be3

			variation => N__('French: Steinitz, Boleslavsky Variation'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p1n1ppp/p1n1p3/2ppP3/3P1P2/2N1BN2/PPP3PP/R2QKB1R w KQkq - 0 8',
				'c5d4' => 'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3p1P2/2N1BN2/PPP3PP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'C11h',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6. Nf3'),
			moves => {
				'c1e3' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2N1BN2/PPP3PP/R2QKB1R b KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2P2N2/PP4PP/R1BQKBNR b KQkq - 2 7' => {
			eco => 'C05g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3p1P2/2P2N2/PP4PP/R1BQKBNR w KQkq - 0 8',
				'd8b6' => 'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P1P2/2P2N2/PP4PP/R1BQKBNR w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2P5/PP1N2PP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'C05f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6'),
			moves => {
				'd2f3' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2P2N2/PP4PP/R1BQKBNR b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'C05r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ngf3

			variation => N__('French: Tarrasch, Closed, 7.Ngf3'),
			moves => {
				'd8b6' => 'r1b1kb1r/pp1n1ppp/1qn1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQK2R w KQkq - 3 8',
				'f8e7' => 'r1bqk2r/pp1nbppp/2n1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQK2R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB1N2/PP3PPP/R1BQK1NR b KQkq - 2 7' => {
			eco => 'C05p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ndf3

			variation => N__('French: Tarrasch, Closed, 7.Ndf3'),
			moves => {
				'd8a5' => 'r1b1kb1r/pp1n1ppp/2n1p3/q1ppP3/3P4/2PB1N2/PP3PPP/R1BQK1NR w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB4/PP1N1PPP/R1BQK1NR w KQkq - 1 7' => {
			eco => 'C05p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6

			variation => N__('French: Tarrasch, Closed, 6... Nc6'),
			moves => {
				'd2f3' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB1N2/PP3PPP/R1BQK1NR b KQkq - 2 7',
				'g1e2' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB4/PP1NNPPP/R1BQK2R b KQkq - 2 7',
				'g1f3' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB1N2/PP1N1PPP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB4/PP1NNPPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'C05v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2

			variation => N__('French: Tarrasch, Closed, 7. Ne2'),
			moves => {
				'a7a5' => 'r1bqkb1r/1p1n1ppp/2n1p3/p1ppP3/3P4/2PB4/PP1NNPPP/R1BQK2R w KQkq a6 0 8',
				'c5d4' => 'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3p4/2PB4/PP1NNPPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3N1P2/2N1B3/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'C11l',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, Boleslavsky, 7...cxd4'),
			moves => {
				'd8b6' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3N1P2/2N1B3/PPP3PP/R2QKB1R w KQkq - 1 9',
				'f8c5' => 'r1bqk2r/pp1n1ppp/2n1p3/2bpP3/3N1P2/2N1B3/PPP3PP/R2QKB1R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'C06a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4

			variation => N__('French: Tarrasch, Closed, Main Line'),
			moves => {
				'd7b6' => 'r1bqkb1r/pp3ppp/1nn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 1 9',
				'd8b6' => 'r1b1kb1r/pp1n1ppp/1qn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 1 9',
				'f7f6' => 'r1bqkb1r/pp1n2pp/2n1pp2/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3p1P2/2N1BN2/PPP3PP/R2QKB1R w KQkq - 0 8' => {
			eco => 'C11l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3 Nc6 7. Be3 cxd4

			variation => N__('French: Steinitz, Boleslavsky, 7...cxd4'),
			moves => {
				'f3d4' => 'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3N1P2/2N1B3/PPP3PP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3p1P2/2P2N2/PP4PP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'C05h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4 c5 6. c3 Nc6 7. Ndf3 cxd4

			variation => N__('French: Tarrasch, Closed, 5. f4 c5 6. c3 Nc6 7.Ndf3 cxd4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3p4/2PB4/PP1NNPPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'C05v',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 7. Ne2'),
			moves => {
				'c3d4' => 'r1bqkb1r/pp1n1ppp/2n1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2npp3/6P1/3NP3/2N1B3/PPP2P1P/R2QKB1R b KQkq - 2 8' => {
			eco => 'B81s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 Nc6 7. g5 Nd7 8. Be3

			variation => N__('Sicilian: Scheveningen, Keres, 6... Nc6 7. g5 Nd7 8. Be3'),
			moves => {
				'f8e7' => 'r1bqk2r/pp1nbppp/2npp3/6P1/3NP3/2N1B3/PPP2P1P/R2QKB1R w KQkq - 3 9',
			},
		},
		'r1bqkb1r/pp1n1ppp/2npp3/6P1/3NP3/2N5/PPP2P1P/R1BQKB1R w KQkq - 1 8' => {
			eco => 'B81r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 Nc6 7. g5 Nd7

			variation => N__('Sicilian: Scheveningen, Keres, 6... Nc6 7. g5 Nd7'),
			moves => {
				'c1e3' => 'r1bqkb1r/pp1n1ppp/2npp3/6P1/3NP3/2N1B3/PPP2P1P/R2QKB1R b KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3P2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D51',
                        # TRANSLATORS:

			variation => N__('QGD: 4. Bg5 Nbd7 5. e3 c6'),
			moves => {
				'c6d5' => 'r1bqkb1r/pp1n1ppp/4pn2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6

			variation => N__('QGD: 4. Bg5 Nbd7 5. e3 c6'),
			moves => {
				'a2a3' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/P1N1P3/1P3PPP/R2QKBNR b KQkq - 0 6',
				'c4d5' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3P2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
				'd1c2' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N1P3/PPQ2PPP/R3KBNR b KQkq - 1 6',
				'g1f3' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N1P3/PPQ2PPP/R3KBNR b KQkq - 1 6' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Qc2

			variation => N__('QGD: 4. Bg5 Nbd7 5. e3 c6 6. Qc2'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D52a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nf3

			variation => N__('QGD: 4. Bg5 Nbd7 5. e3 c6 6. Nf3'),
			moves => {
				'd8a5' => 'r1b1kb1r/pp1n1ppp/2p1pn2/q2p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/2RQKB1R b Kkq - 1 6' => {
			eco => 'D51',
                        # TRANSLATORS:

			variation => N__('QGD: 4. Bg5 Nbd7 5. Nf3 c6'),
			moves => {
				'd8a5' => 'r1b1kb1r/pp1n1ppp/2p1pn2/q2p2B1/2PP4/2N2N2/PP2PPPP/2RQKB1R w Kkq - 2 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. Nf3 c6

			variation => N__('QGD: 4. Bg5 Nbd7 5. Nf3 c6'),
			moves => {
				'a1c1' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/2RQKB1R b Kkq - 1 6',
				'e2e4' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/4PN2/PP1N1PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D30v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. Nbd2

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5, Capablanca Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/4PN2/PP3PPP/RN1QKB1R w KQkq - 0 6' => {
			eco => 'D30v',
                        # TRANSLATORS:

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5 Nbd7'),
			moves => {
				'b1d2' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/4PN2/PP1N1PPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/P1N1P3/1P3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. a3

			variation => N__('QGD: Capablanca\'s Anti-Cambridge Springs'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. Nf3 c6 6. e4

			variation => N__('QGD: Alekhine 4. Bg5 Nbd7 5. Nf3 c6 6. e4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D45f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7

			variation => N__('Semi-Slav: 5. e3 Nbd7'),
			moves => {
				'a2a3' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6',
				'd1c2' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R b KQkq - 2 6',
				'f1d3' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 6',
				'f3e5' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3pN3/2PP4/2N1P3/PP3PPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R b KQkq - 2 6' => {
			eco => 'D45h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2

			variation => N__('Semi-Slav: 6. Qc2'),
			moves => {
				'b7b6' => 'r1bqkb1r/p2n1ppp/1pp1pn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R w KQkq - 0 7',
				'f8d6' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R w KQkq - 3 7',
				'f8e7' => 'r1bqk2r/pp1nbppp/2p1pn2/3p4/2PP4/2N1PN2/PPQ2PPP/R1B1KB1R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'D46a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3

			variation => N__('Semi-Slav: 6. Bd3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p1n1ppp/p1p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 7',
				'd5c4' => 'r1bqkb1r/pp1n1ppp/2p1pn2/8/2pP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 7',
				'f8b4' => 'r1bqk2r/pp1n1ppp/2p1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7',
				'f8d6' => 'r1bqk2r/pp1n1ppp/2pbpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7',
				'f8e7' => 'r1bqk2r/pp1nbppp/2p1pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/3BPN2/PP1N1PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'D30u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6 5. Nbd2 Nbd7 6. Bd3

			variation => N__('QGD: 3. Nf3 Nf6 4. e3 c6 5.Nbd2 Nbd7 6. Bd3'),
			moves => {
				'c6c5' => 'r1bqkb1r/pp1n1ppp/4pn2/2pp4/2PP4/3BPN2/PP1N1PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/4PN2/PP1N1PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D30t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6 5. Nbd2 Nbd7

			variation => N__('QGD: 3. Nf3 Nf6 4. e3 c6 5.Nbd2 Nbd7'),
			moves => {
				'f1d3' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/3BPN2/PP1N1PPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 c6 5. Nf3 Nbd7

			variation => N__('Catalan: 4... c6 5. Nf3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D45g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. a3

			variation => N__('Semi-Slav: 6. a3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/3pN3/2PP4/2N1P3/PP3PPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D45f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Ne5

			variation => N__('Semi-Slav: Rubinstein System'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/2BP4/8/PPP1QPPP/R1B1K1NR b KQkq - 1 7' => {
			eco => 'B17r',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5'),
			moves => {
				'd7b6' => 'r1bqkb1r/pp3ppp/1np1pn2/6N1/2BP4/8/PPP1QPPP/R1B1K1NR w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/2BP4/8/PPP2PPP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'B17r',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5'),
			moves => {
				'd1e2' => 'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/2BP4/8/PPP1QPPP/R1B1K1NR b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/3P4/3B1N2/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B17k',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6'),
			moves => {
				'f8d6' => 'r1bqk2r/pp1n1ppp/2pbpn2/6N1/3P4/3B1N2/PPP2PPP/R1BQK2R w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/3P4/3B4/PPP2PPP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'B17k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6 6. Bd3 e6

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3 e6'),
			moves => {
				'g1f3' => 'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/3P4/3B1N2/PPP2PPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/8/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'D47a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4

			variation => N__('Semi-Slav: Meran Variation'),
			moves => {
				'b7b5' => 'r1bqkb1r/p2n1ppp/2p1pn2/1p6/2BP4/2N1PN2/PP3PPP/R1BQK2R w KQkq b6 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/8/2pP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'D47a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4

			variation => N__('Semi-Slav: Meran Variation'),
			moves => {
				'd3c4' => 'r1bqkb1r/pp1n1ppp/2p1pn2/8/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/2p1pn2/8/3P4/5NN1/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B17g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Ng3 e6

			variation => N__('Caro-Kann: Steinitz, 5. Nf3 Ngf6 6. Ng3 e6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p2n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D35n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Nbd7

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2p2n2/3p4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D35e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Nbd7 6. e3 c6

			variation => N__('QGD: Exchange, 5. Nf3 Nbd7 6. e3 c6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp1n2/4p1B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5 c6

			variation => N__('Old Indian: 5. Bg5 c6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 c6

			variation => N__('Old Indian: 5. g3 c6'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 c6 6. Bg2

			variation => N__('Old Indian: 5. g3 c6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A55c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 c6 6. Be2

			variation => N__('Old Indian: 5. e4 c6 6. Be2'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A55c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 c6

			variation => N__('Old Indian: 5. e4 c6'),
			moves => {
				'f1e2' => 'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp3n/4p1N1/2BPP3/8/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'f2f4' => 'r1bqkb1r/pp1n1ppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQK2R b KQkq f3 0 6',
			},
		},
		'r1bqkb1r/pp1n1ppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQK2R b KQkq f3 0 6' => {
			eco => 'C41g',
                        # TRANSLATORS:

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'f8e7' => 'r1bqk2r/pp1nbppp/2pp3n/4p1N1/2BPPP2/8/PPP3PP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/3p1n2/1B2p3/3NP3/5P2/PPP3PP/RNBQK2R w KQkq - 2 7' => {
			eco => 'B55',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Bb5+ Nbd7

			variation => N__('Sicilian: Prins, Venice Attack, 6...Nbd7'),
			moves => {
				'd4f5' => 'r1bqkb1r/pp1n1ppp/3p1n2/1B2pN2/4P3/5P2/PPP3PP/RNBQK2R b KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/3p1n2/1B2p3/3QP3/2N2N2/PPP2PPP/R1B1K2R w KQkq e6 0 7' => {
			eco => 'B51j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3 cxd4 6. Qxd4 e5

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3 cxd4 6.Qxd4 e5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/3p1n2/1B2pN2/4P3/5P2/PPP3PP/RNBQK2R b KQkq - 3 7' => {
			eco => 'B55',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Bb5+ Nbd7 7. Nf5

			variation => N__('Sicilian: Prins, Venice Attack, 6...Nbd7 7. Nf5'),
			moves => {
				'd6d5' => 'r1bqkb1r/pp1n1ppp/5n2/1B1ppN2/4P3/5P2/PPP3PP/RNBQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/2p5/2Q5/5NP1/PP1PPPBP/RNB1K2R w KQkq c6 0 7' => {
			eco => 'A13w',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5

			variation => N__('English: Neo-Catalan Accepted, 5. Qa4+ Nbd7 6.Qxc4 c5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/2p5/2QP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 7' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5 7. Nf3

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 c5 7. Nf3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p1n1ppp/p3pn2/2p5/2QP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/2p5/2QP4/6P1/PP2PPBP/RNB1K1NR w KQkq c6 0 7' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4 c5

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4 c5'),
			moves => {
				'g1f3' => 'r1bqkb1r/pp1n1ppp/4pn2/2p5/2QP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/2pp2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQkq - 2 6' => {
			eco => 'D03h',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3 c5 5. Nbd2 Nbd7

			variation => N__('Torre Attack: 3... e6 4. e3 c5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/2pp4/2PP4/3BPN2/PP1N1PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'D30u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6 5. Nbd2 Nbd7 6. Bd3 c5

			variation => N__('QGD: Semmering Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/2pp4/3P4/2PBPN2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'D05i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3 Nbd7

			variation => N__('Colle: 5. c3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/4pn2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3 c6 6. cxd5 cxd5

			variation => N__('QGD: 4. Bg5 Nbd7 5. e3 c6 6.cxd5 cxd5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n1ppp/5n2/1B1ppN2/4P3/5P2/PPP3PP/RNBQK2R w KQkq - 0 8' => {
			eco => 'B55',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Bb5+ Nbd7 7. Nf5 d5

			variation => N__('Sicilian: Prins, Venice Attack, 6...Nbd7 7. Nf5 d5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n2pp/2n1pP2/3p4/3P4/3B4/PP1NNPPP/R1BQK2R b KQkq - 0 9' => {
			eco => 'C06m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6'),
			moves => {
				'd7f6' => 'r1bqkb1r/pp4pp/2n1pn2/3p4/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 0 10',
			},
		},
		'r1bqkb1r/pp1n2pp/2n1pp2/3pP3/3P1N2/3B4/PP1N1PPP/R1BQK2R b KQkq - 1 9' => {
			eco => 'C06l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. Nf4

			variation => N__('French: Tarrasch, Closed, 8... f6 9. Nf4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1n2pp/2n1pp2/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'C06k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6

			variation => N__('French: Tarrasch, Closed, 8... f6'),
			moves => {
				'e2f4' => 'r1bqkb1r/pp1n2pp/2n1pp2/3pP3/3P1N2/3B4/PP1N1PPP/R1BQK2R b KQkq - 1 9',
				'e5f6' => 'r1bqkb1r/pp1n2pp/2n1pP2/3p4/3P4/3B4/PP1NNPPP/R1BQK2R b KQkq - 0 9',
			},
		},
		'r1bqkb1r/pp1npp1p/3p1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Nbd7

			variation => N__('Sicilian: Dragon, Levenfish, 6...Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1npppp/2p2N2/8/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B17d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+

			variation => N__('Caro-Kann: Steinitz, 5. Nf3 Ngf6 6.Nxf6+'),
			moves => {
				'd7f6' => 'r1bqkb1r/pp2pppp/2p2n2/8/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1npppp/2p2n2/6N1/2BP4/8/PPP2PPP/R1BQK1NR b KQkq - 4 6' => {
			eco => 'B17r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Bc4 Ngf6 6. Ng5

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5'),
			moves => {
				'e7e6' => 'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/2BP4/8/PPP2PPP/R1BQK1NR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1npppp/2p2n2/6N1/3P4/3B4/PPP2PPP/R1BQK1NR b KQkq - 4 6' => {
			eco => 'B17j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6 6. Bd3

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6 6. Bd3'),
			moves => {
				'e7e6' => 'r1bqkb1r/pp1n1ppp/2p1pn2/6N1/3P4/3B4/PPP2PPP/R1BQK1NR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1npppp/2p2n2/6N1/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 3 6' => {
			eco => 'B17i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5 Ngf6

			variation => N__('Caro-Kann: Steinitz, 5. Ng5 Ngf6'),
			moves => {
				'f1d3' => 'r1bqkb1r/pp1npppp/2p2n2/6N1/3P4/3B4/PPP2PPP/R1BQK1NR b KQkq - 4 6',
			},
		},
		'r1bqkb1r/pp1npppp/2p2n2/8/2BPN3/8/PPP2PPP/R1BQK1NR w KQkq - 3 6' => {
			eco => 'B17q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Bc4 Ngf6

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6'),
			moves => {
				'e4g5' => 'r1bqkb1r/pp1npppp/2p2n2/6N1/2BP4/8/PPP2PPP/R1BQK1NR b KQkq - 4 6',
			},
		},
		'r1bqkb1r/pp1npppp/2p2n2/8/3P4/5NN1/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B17f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Ng3

			variation => N__('Caro-Kann: Steinitz, 5. Nf3 Ngf6 6. Ng3'),
			moves => {
				'e7e6' => 'r1bqkb1r/pp1n1ppp/2p1pn2/8/3P4/5NN1/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1npppp/2p2n2/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B17c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6

			variation => N__('Caro-Kann: Steinitz, 5. Nf3 Ngf6'),
			moves => {
				'e4f6' => 'r1bqkb1r/pp1npppp/2p2N2/8/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'e4g3' => 'r1bqkb1r/pp1npppp/2p2n2/8/3P4/5NN1/PPP2PPP/R1BQKB1R b KQkq - 4 6',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/1B6/3QP3/2N2N2/PPP2PPP/R1B1K2R b KQkq - 0 6' => {
			eco => 'B51i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3 cxd4 6. Qxd4

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3 cxd4 6.Qxd4'),
			moves => {
				'e7e5' => 'r1bqkb1r/pp1n1ppp/3p1n2/1B2p3/3QP3/2N2N2/PPP2PPP/R1B1K2R w KQkq e6 0 7',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/1B6/3pP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'B51i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3 cxd4

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3 cxd4'),
			moves => {
				'd1d4' => 'r1bqkb1r/pp1npppp/3p1n2/1B6/3QP3/2N2N2/PPP2PPP/R1B1K2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/1Bp5/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'B51h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4 Nf6 5. Nc3'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp1npppp/3p1n2/1B6/3pP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/1Bp5/3PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'B51g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4 Nf6

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4 Nf6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp1npppp/3p1n2/1Bp5/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 2 5',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 5' => {
			eco => 'B51d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. O-O Nf6

			variation => N__('Sicilian: 3. Bb5+ Nd7 4.O-O Nf6'),
			moves => {
				'f1e1' => 'r1bqkb1r/pp1npppp/3p1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 5 5',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 5 5' => {
			eco => 'B51d',
                        # TRANSLATORS:

			variation => N__('Sicilian: 3. Bb5+ Nd7 4.O-O Nf6'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p1npppp/p2p1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 6',
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R w KQkq - 3 5' => {
			eco => 'B50u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Be2 Nbd7

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Be2 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B56e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nbd7 6. Bc4

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nbd7 6. Bc4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1npppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B56d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nbd7

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nbd7'),
			moves => {
				'f1c4' => 'r1bqkb1r/pp1npppp/3p1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 4 6',
			},
		},
		'r1bqkb1r/pp1npppp/5n2/2pp4/3P4/2P1PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D04j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 c5 4. c3 Nbd7

			variation => N__('Colle: 3... c5 4. c3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1Npp/2p1nn2/8/2B1P3/8/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C44r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 Nxd4 4. Nxe5 Ne6 5. Bc4 c6 6. O-O Nf6 7. Nxf7

			variation => N__('Scotch: Cochrane Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1pp1/2n2n1p/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B33e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 h6

			variation => N__('Sicilian: Pelikan, Haberditz Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2N1pn2/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B45k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Nxc6

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Nxc6'),
			moves => {
				'b7c6' => 'r1bqkb1r/p2p1ppp/2p1pn2/8/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2N1pn2/8/4P3/3B4/PPP2PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'B40q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Bd3 Nc6 6. Nxc6

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 Nf6 5. Bd3 Nc6 6.Nxc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2N2n2/4p3/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B33d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Nxc6

			variation => N__('Sicilian: Pelikan/Sveshnikov, 6.Nxc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1p3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 2 7' => {
			eco => 'B22v',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 e6 6. Nf3 Nc6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 e6 6. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/1N6/2P5/2N5/PP2PPPP/R1BQKB1R b KQkq - 3 6' => {
			eco => 'A33g',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. Ndb5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5'),
			moves => {
				'd7d5' => 'r1bqkb1r/pp3ppp/2n1pn2/1N1p4/2P5/2N5/PP2PPPP/R1BQKB1R w KQkq d6 0 7',
				'f8b4' => 'r1bqk2r/pp1p1ppp/2n1pn2/1N6/1bP5/2N5/PP2PPPP/R1BQKB1R w KQkq - 4 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/1N6/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B45o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5'),
			moves => {
				'd7d6' => 'r1bqkb1r/pp3ppp/2nppn2/1N6/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
				'f8b4' => 'r1bqk2r/pp1p1ppp/2n1pn2/1N6/1b2P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 5 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 2 5' => {
			eco => 'A36c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 e6 4. Nf3 Nf6

			variation => N__('English: Symmetrical, Keres-Parma System'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp1p1ppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5' => {
			eco => 'A36c',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Keres-Parma System'),
			moves => {
				'f8e7' => 'r1bqk2r/pp1pbppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 4 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A46o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 Nc6

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/6B1/2PN4/2N5/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'A33c',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. Bg5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. Bg5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 4 6' => {
			eco => 'B45e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Bg5

			variation => N__('Sicilian: Taimanov, Four Knights, 6. Bg5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN1B2/2N5/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'A33b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. Bf4

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. Bf4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/2N1P3/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A33d',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. e3

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. e3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A33k',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. g3

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. g3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p1p1ppp/p1n1pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 0 7',
				'd8b6' => 'r1b1kb1r/pp1p1ppp/1qn1pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 1 7',
				'f8b4' => 'r1bqk2r/pp1p1ppp/2n1pn2/8/1bPN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 1 7',
				'f8c5' => 'r1bqk2r/pp1p1ppp/2n1pn2/2b5/2PN4/2N3P1/PP2PP1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A33a',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6'),
			moves => {
				'a2a3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/P1N5/1P2PPPP/R1BQKB1R b KQkq - 0 6',
				'c1f4' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN1B2/2N5/PP2PPPP/R2QKB1R b KQkq - 3 6',
				'c1g5' => 'r1bqkb1r/pp1p1ppp/2n1pn2/6B1/2PN4/2N5/PP2PPPP/R2QKB1R b KQkq - 3 6',
				'd4b5' => 'r1bqkb1r/pp1p1ppp/2n1pn2/1N6/2P5/2N5/PP2PPPP/R1BQKB1R b KQkq - 3 6',
				'e2e3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/2N1P3/PP3PPP/R1BQKB1R b KQkq - 0 6',
				'g2g3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 Nc6

			variation => N__('English: Symmetrical, Two Knights, 5. g3 Nc6'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 Nc6 6. Bg2

			variation => N__('English: Symmetrical, Two Knights, 5. g3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/P1N5/1P2PPPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A33e',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. a3

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6. a3'),
			moves => {
				'f8c5' => 'r1bqk2r/pp1p1ppp/2n1pn2/2b5/2PN4/P1N5/1P2PPPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'B44e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. c4 Nf6 6. Nc3

			variation => N__('Sicilian: Taimanov, 5. c4 Nf6 6. Nc3'),
			moves => {
				'f8b4' => 'r1bqk2r/pp1p1ppp/2n1pn2/8/1bPNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B44d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. c4 Nf6

			variation => N__('Sicilian: Taimanov, 5. c4 Nf6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 4 6' => {
			eco => 'B45i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Be3

			variation => N__('Sicilian: Taimanov, Four Knights, 6. Be3'),
			moves => {
				'f8b4' => 'r1bqk2r/pp1p1ppp/2n1pn2/8/1b1NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 5 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B45g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. g3

			variation => N__('Sicilian: Taimanov, Four Knights, 6. g3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B45h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Be2

			variation => N__('Sicilian: Taimanov, Four Knights, 6. Be2'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B45d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6

			variation => N__('Sicilian: Taimanov, Four Knights'),
			moves => {
				'a2a3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 6',
				'c1e3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 4 6',
				'c1g5' => 'r1bqkb1r/pp1p1ppp/2n1pn2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 4 6',
				'd4b5' => 'r1bqkb1r/pp1p1ppp/2n1pn2/1N6/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 4 6',
				'd4c6' => 'r1bqkb1r/pp1p1ppp/2N1pn2/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'f1e2' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6',
				'g2g3' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 3 6' => {
			eco => 'B40p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Bd3 Nc6

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 Nf6 5. Bd3 Nc6'),
			moves => {
				'd4c6' => 'r1bqkb1r/pp1p1ppp/2N1pn2/8/4P3/3B4/PPP2PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/4B3/PPP2PPP/RN1QKB1R w KQkq - 3 6' => {
			eco => 'B44h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Be3 Nf6

			variation => N__('Sicilian: Taimanov, 5. Be3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B45f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. a3

			variation => N__('Sicilian: Taimanov, Four Knights, 6. a3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n2n2/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B33e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5

			variation => N__('Sicilian: Pelikan/Sveshnikov, 6.Nbd5'),
			moves => {
				'd7d6' => 'r1bqkb1r/pp3ppp/2np1n2/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
				'h7h6' => 'r1bqkb1r/pp1p1pp1/2n2n1p/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n2n2/2p1p3/2P5/2N1PN2/PP1P1PPP/R1BQKB1R w KQkq e6 0 5' => {
			eco => 'A35i',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3 e5

			variation => N__('English: Symmetrical, Four Knights, 4. e3 e5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n2n2/4p3/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq e6 0 6' => {
			eco => 'B33d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5

			variation => N__('Sicilian: Pelikan/Sveshnikov'),
			moves => {
				'd4b3' => 'r1bqkb1r/pp1p1ppp/2n2n2/4p3/4P3/1NN5/PPP2PPP/R1BQKB1R b KQkq - 1 6',
				'd4b5' => 'r1bqkb1r/pp1p1ppp/2n2n2/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6',
				'd4c6' => 'r1bqkb1r/pp1p1ppp/2N2n2/4p3/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'd4f3' => 'r1bqkb1r/pp1p1ppp/2n2n2/4p3/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 6',
				'd4f5' => 'r1bqkb1r/pp1p1ppp/2n2n2/4pN2/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pp1p1ppp/2n2n2/4p3/4P3/1NN5/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B33d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Nb3

			variation => N__('Sicilian: Pelikan/Sveshnikov, 6. Nb3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n2n2/4p3/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B33d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Nf3

			variation => N__('Sicilian: Pelikan/Sveshnikov, 6. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n2n2/4pN2/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B33d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Nf5

			variation => N__('Sicilian: Pelikan/Sveshnikov, 6. Nf5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n5/2PpP3/8/5N2/PPP2PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B29n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. Nc3 e6 5. Nxd5 exd5 6. d4 Nc6 7. dxc5

			variation => N__('Sicilian: Nimzowitsch, Rubinstein Countergambit, 7.dxc5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2n5/2ppP3/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B29m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. Nc3 e6 5. Nxd5 exd5 6. d4 Nc6

			variation => N__('Sicilian: Nimzowitsch, Rubinstein Countergambit'),
			moves => {
				'd4c5' => 'r1bqkb1r/pp1p1ppp/2n5/2PpP3/8/5N2/PPP2PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2p1nn2/4N3/2B1P3/8/PPP2PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'C44r',
                        # TRANSLATORS:

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'e5f7' => 'r1bqkb1r/pp1p1Npp/2p1nn2/8/2B1P3/8/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqkb1r/pp1p1ppp/2p2n2/4N3/B2nP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C48p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Ba4 c6 6. Nxe5

			variation => N__('Four Knights: Rubinstein, 5. Ba4 c6 6.Nxe5'),
			moves => {
			},
		},
		'r1bqkb1r/pp1p1ppp/2p2n2/4p3/B2nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C48o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Ba4 c6

			variation => N__('Four Knights: Rubinstein, 5. Ba4 c6'),
			moves => {
				'f3e5' => 'r1bqkb1r/pp1p1ppp/2p2n2/4N3/B2nP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/1P3N2/P1PP1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'B30s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O Nge7 5. b3

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O Nge7 5. b3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 3 5' => {
			eco => 'B30u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O Nge7 5. Nc3

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O Nge7 5. Nc3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'B30v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O Nge7 5. c3

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O Nge7 5. c3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p1pnppp/p1n1p3/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 5' => {
			eco => 'B30s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O Nge7

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O Nge7'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 3 5',
				'b2b3' => 'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/1P3N2/P1PP1PPP/RNBQ1RK1 b kq - 0 5',
				'c2c3' => 'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5',
				'f1e1' => 'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 3 5',
			},
		},
		'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 3 5' => {
			eco => 'B30t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O Nge7 5. Re1

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O Nge7 5. Re1'),
			moves => {
			},
		},
		'r1bqkb1r/pp1pnppp/2p2n2/1B2N3/4P3/3P4/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C65g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 Ne7 5. Nxe5 c6

			variation => N__('Spanish: Berlin, Mortimer Trap'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppp1p/2N2np1/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B34h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Nf6 6. Nxc6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 5... Nf6 6.Nxc6'),
			moves => {
				'b7c6' => 'r1bqkb1r/p2ppp1p/2p2np1/8/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1ppp1p/2n2np1/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 g6

			variation => N__('English: Symmetrical, Two Knights, 4... Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppp1p/2n2np1/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'B36d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3'),
			moves => {
				'c6d4' => 'r1bqkb1r/pp1ppp1p/5np1/8/2PnP3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 7',
				'd7d6' => 'r1bqkb1r/pp2pp1p/2np1np1/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1ppp1p/2n2np1/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B36c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp1ppp1p/2n2np1/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp1ppp1p/2n2np1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'B34g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Nf6

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 5... Nf6'),
			moves => {
				'd4c6' => 'r1bqkb1r/pp1ppp1p/2N2np1/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1ppp1p/5np1/8/2PQP3/2N5/PP3PPP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'B36e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 Nxd4'),
			moves => {
				'd7d6' => 'r1bqkb1r/pp2pp1p/3p1np1/8/2PQP3/2N5/PP3PPP/R1B1KB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp1ppp1p/5np1/8/2PnP3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B36e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 Nxd4

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 Nxd4'),
			moves => {
				'd1d4' => 'r1bqkb1r/pp1ppp1p/5np1/8/2PQP3/2N5/PP3PPP/R1B1KB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'B30m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 Nf6

			variation => N__('Sicilian: Rossolimo, 3... Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A35h',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3

			variation => N__('English: Symmetrical, Four Knights, 4. e3'),
			moves => {
				'e7e5' => 'r1bqkb1r/pp1p1ppp/2n2n2/2p1p3/2P5/2N1PN2/PP1P1PPP/R1BQKB1R w KQkq e6 0 5',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'A35g',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6

			variation => N__('English: Symmetrical, Four Knights'),
			moves => {
				'd2d4' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4',
				'e2e3' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 0 4',
				'g2g3' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A35j',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3

			variation => N__('English: Symmetrical, Four Knights, 4. g3'),
			moves => {
				'd7d5' => 'r1bqkb1r/pp2pppp/2n2n2/2pp4/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq d6 0 5',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A36b',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 Nf6

			variation => N__('English: Symmetrical, 3. g3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'A35g',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6 4. d4

			variation => N__('English: Symmetrical, Four Knights, 4. d4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'B30h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Nc3 Nf6

			variation => N__('Sicilian: 2... Nc6 3. Nc3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B30c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d3 Nf6

			variation => N__('Sicilian: 2... Nc6 3. d3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/2p5/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 c5 3. Bg2 Nc6

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3

			variation => N__('English: Symmetrical, Two Knights, 4... Nc6'),
			moves => {
				'g7g6' => 'r1bqkb1r/pp1ppp1p/2n2np1/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 Nc6

			variation => N__('English: Symmetrical, Two Knights, 4... Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp1ppppp/2n2n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B33a',
                        # TRANSLATORS:

			variation => N__('Sicilian: Open, 2... Nc6'),
			moves => {
				'd8b6' => 'r1b1kb1r/pp1ppppp/1qn2n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'e7e5' => 'r1bqkb1r/pp1p1ppp/2n2n2/4p3/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq e6 0 6',
			},
		},
		'r1bqkb1r/pp1ppppp/2n2n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B33a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6

			variation => N__('Sicilian: Open, 2... Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp1ppppp/2n2n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqkb1r/pp1ppppp/2n5/2pnP3/8/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 3 5' => {
			eco => 'B22p',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. Nf3 Nc6

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. Nf3 Nc6'),
			moves => {
				'b1a3' => 'r1bqkb1r/pp1ppppp/2n5/2pnP3/8/N1P2N2/PP1P1PPP/R1BQKB1R b KQkq - 4 5',
			},
		},
		'r1bqkb1r/pp1ppppp/2n5/2pnP3/8/N1P2N2/PP1P1PPP/R1BQKB1R b KQkq - 4 5' => {
			eco => 'B22p',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. Nf3 Nc6 5. Na3

			variation => N__('Sicilian: Alapin, Heidenfeld Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n5/3nP3/2Bp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 3 6' => {
			eco => 'B22s',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3 Nc6 6. Bc4

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3 Nc6 6. Bc4'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n5/3nP3/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 6' => {
			eco => 'B22t',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 Nc6 6. Nf3

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 Nc6 6. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pp1ppppp/2n5/3nP3/3P4/8/PP3PPP/RNBQKBNR w KQkq - 1 6' => {
			eco => 'B22t',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 Nc6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 Nc6'),
			moves => {
				'g1f3' => 'r1bqkb1r/pp1ppppp/2n5/3nP3/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp1ppppp/2n5/3nP3/3p4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 2 6' => {
			eco => 'B22r',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3 Nc6

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3 Nc6'),
			moves => {
				'f1c4' => 'r1bqkb1r/pp1ppppp/2n5/3nP3/2Bp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 3 6',
			},
		},
		'r1bqkb1r/pp2np1p/2n1p1p1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'f8g7' => 'r1bqk2r/pp2npbp/2n1p1p1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w kq - 1 8',
			},
		},
		'r1bqkb1r/pp2np1p/2n1p1p1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e2e4' => 'r1bqkb1r/pp2np1p/2n1p1p1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7',
			},
		},
		'r1bqkb1r/pp2nppp/2n1p3/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'b1d2' => 'r1bqkb1r/pp2nppp/2n1p3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6',
			},
		},
		'r1bqkb1r/pp2nppp/2n1p3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'b7b6' => 'r1bqkb1r/p3nppp/1pn1p3/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7',
				'g7g6' => 'r1bqkb1r/pp2np1p/2n1p1p1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7',
			},
		},
		'r1bqkb1r/pp2nppp/2n1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C02m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Nge7

			variation => N__('French: Advance, 5. Nf3 Nge7'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2Np1np1/8/2B1P3/2N5/PPP2PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'B57a',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin'),
			moves => {
				'b7c6' => 'r1bqkb1r/p3pp1p/2pp1np1/8/2B1P3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp2pp1p/2Np1np1/8/4PP2/2N5/PPP3PP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Nc6 7. Nxc6

			variation => N__('Sicilian: Dragon, Levenfish, 6... Nc6 7.Nxc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2n2np1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 g6 4. O-O Nc6 5. d3 Nf6

			variation => N__('Reti: KIA, 2... c5 + 3... g6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/1B6/3NPP2/2N5/PPP3PP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Nc6 7. Bb5

			variation => N__('Sicilian: Dragon, Levenfish, 6... Nc6 7. Bb5'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B60b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 g6

			variation => N__('Sicilian: Richter-Rauzer, Bondarevsky Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B57a',
                        # TRANSLATORS:

			variation => N__('Sicilian: Sozin'),
			moves => {
				'd4c6' => 'r1bqkb1r/pp2pp1p/2Np1np1/8/2B1P3/2N5/PPP2PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/2PNP3/2N5/PP2BPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B36h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2'),
			moves => {
				'c6d4' => 'r1bqkb1r/pp2pp1p/3p1np1/8/2PnP3/2N5/PP2BPPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B36g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6'),
			moves => {
				'f1e2' => 'r1bqkb1r/pp2pp1p/2np1np1/8/2PNP3/2N5/PP2BPPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B70c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. g3 Nc6

			variation => N__('Sicilian: Dragon, 6. g3 Nc6'),
			moves => {
				'd4e2' => 'r1bqkb1r/pp2pp1p/2np1np1/8/4P3/2N3P1/PPP1NP1P/R1BQKB1R b KQkq - 2 7',
				'f1g2' => 'r1bqkb1r/pp2pp1p/2np1np1/8/3NP3/2N3P1/PPP2PBP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/3NP3/2N3P1/PPP2PBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B70c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. g3 Nc6 7. Bg2

			variation => N__('Sicilian: Dragon, 6. g3 Nc6 7. Bg2'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B70i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Nc6

			variation => N__('Sicilian: Dragon, 6. Be2 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Nc6

			variation => N__('Sicilian: Dragon, Levenfish, 6... Nc6'),
			moves => {
				'd4c6' => 'r1bqkb1r/pp2pp1p/2Np1np1/8/4PP2/2N5/PPP3PP/R1BQKB1R b KQkq - 0 7',
				'd4f3' => 'r1bqkb1r/pp2pp1p/2np1np1/8/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 2 7',
				'f1b5' => 'r1bqkb1r/pp2pp1p/2np1np1/1B6/3NPP2/2N5/PPP3PP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/4P3/2N3P1/PPP1NP1P/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B70d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. g3 Nc6 7. Nde2

			variation => N__('Sicilian: Dragon, 6. g3 Nc6 7.Nde2'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/2np1np1/8/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Nc6 7. Nf3

			variation => N__('Sicilian: Dragon, Levenfish, 6... Nc6 7. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/3p1np1/8/2PQP3/2N5/PP2BPPP/R1B1K2R b KQkq - 0 8' => {
			eco => 'B36i',
                        # TRANSLATORS:

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4'),
			moves => {
				'f8g7' => 'r1bqk2r/pp2ppbp/3p1np1/8/2PQP3/2N5/PP2BPPP/R1B1K2R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/pp2pp1p/3p1np1/8/2PQP3/2N5/PP3PPP/R1B1KB1R w KQkq - 0 8' => {
			eco => 'B36f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 Nxd4 7. Qxd4 d6

			variation => N__('Sicilian: Maroczy Bind, Gurgenidze Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pp1p/3p1np1/8/2PnP3/2N5/PP2BPPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B36i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4'),
			moves => {
				'd1d4' => 'r1bqkb1r/pp2pp1p/3p1np1/8/2PQP3/2N5/PP2BPPP/R1B1K2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp2pppp/1nnp4/1B2P3/3P4/5N2/PP3PPP/RNBQK2R b KQkq - 5 8' => {
			eco => 'B22u',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 d6 6. Nf3 Nc6 7. Bc4 Nb6 8. Bb5

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 d6 6. Nf3 Nc6 7. Bc4 Nb6 8. Bb5'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/1nnp4/4P3/2BP4/5N2/PP3PPP/RNBQK2R w KQkq - 4 8' => {
			eco => 'B22u',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 d6 6. Nf3 Nc6 7. Bc4 Nb6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 d6 6. Nf3 Nc6 7. Bc4 Nb6'),
			moves => {
				'c4b5' => 'r1bqkb1r/pp2pppp/1nnp4/1B2P3/3P4/5N2/PP3PPP/RNBQK2R b KQkq - 5 8',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/2pP4/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A35l',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5

			variation => N__('English: Symmetrical, Four Knights, 4. g3 d5 5.cxd5'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/2pp4/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq d6 0 5' => {
			eco => 'A35k',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5

			variation => N__('English: Symmetrical, Four Knights, 4. g3 d5'),
			moves => {
				'c4d5' => 'r1bqkb1r/pp2pppp/2n2n2/2pP4/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/2pp4/3P4/2P1PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D04i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 c5 4. c3 Nc6

			variation => N__('Colle: 3... c5 4. c3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 4 5' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nf6 4. O-O Nc6

			variation => N__('Reti: KIA, 2... c5 3. Bg2 Nf6 4.O-O Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3P2B1/2p5/2N5/PP3PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'B13v',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Bg5 dxc4'),
			moves => {
				'c6a5' => 'r1bqkb1r/pp2pppp/5n2/n2P2B1/2p5/2N5/PP3PPP/R2QKBNR w KQkq - 1 8',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3P4/3P4/2N5/PP3PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'B13q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. cxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6.cxd5'),
			moves => {
				'f6d5' => 'r1bqkb1r/pp2pppp/2n5/3n4/3P4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR b KQkq - 4 6' => {
			eco => 'B13v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Bg5'),
			moves => {
				'c8e6' => 'r2qkb1r/pp2pppp/2n1bn2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 5 7',
				'd5c4' => 'r1bqkb1r/pp2pppp/2n2n2/6B1/2pP4/2N5/PP3PPP/R2QKBNR w KQkq - 0 7',
				'd8a5' => 'r1b1kb1r/pp2pppp/2n2n2/q2p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 5 7',
				'd8b6' => 'r1b1kb1r/pp2pppp/1qn2n2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 5 7',
				'e7e6' => 'r1bqkb1r/pp3ppp/2n1pn2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B13r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3'),
			moves => {
				'c8g4' => 'r2qkb1r/pp2pppp/2n2n2/3p4/2PP2b1/2N2N2/PP3PPP/R1BQKB1R w KQkq - 5 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 3 6' => {
			eco => 'B13q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6'),
			moves => {
				'c1g5' => 'r1bqkb1r/pp2pppp/2n2n2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR b KQkq - 4 6',
				'c4d5' => 'r1bqkb1r/pp2pppp/2n2n2/3P4/3P4/2N5/PP3PPP/R1BQKBNR b KQkq - 0 6',
				'g1f3' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 4 6',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D10f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nc3 Nf6 5. Bf4 Nc6 6. e3

			variation => N__('Slav: Exchange, 4. Nc3 Nf6 5. Bf4 Nc6 6. e3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p2pppp/p1n2n2/3p4/3P1B2/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D13k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4

			variation => N__('Slav: Exchange, 6. Bf4'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p2pppp/p1n2n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
				'c8f5' => 'r2qkb1r/pp2pppp/2n2n2/3p1b2/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 7',
				'e7e6' => 'r1bqkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR w KQkq - 4 6' => {
			eco => 'D10f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nc3 Nf6 5. Bf4 Nc6

			variation => N__('Slav: Exchange, 4. Nc3 Nf6 5. Bf4 Nc6'),
			moves => {
				'e2e3' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2PB4/PP3PPP/RN1QK1NR b KQkq - 2 6' => {
			eco => 'B13j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 Nf6 6. Bf4

			variation => N__('Caro-Kann: Exchange, Rubinstein Variation'),
			moves => {
				'c8g4' => 'r2qkb1r/pp2pppp/2n2n2/3p4/3P1Bb1/2PB4/PP3PPP/RN1QK1NR w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D13j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6

			variation => N__('Slav: Exchange, 5. Nc3 Nc6'),
			moves => {
				'c1f4' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P4/2PB1N2/PP3PPP/RNBQK2R b KQkq - 2 6' => {
			eco => 'B13i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 Nf6 6. Nf3

			variation => N__('Caro-Kann: Exchange, 4. Bd3 Nc6 5. c3 Nf6 6. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/3p4/3P4/2PB4/PP3PPP/RNBQK1NR w KQkq - 1 6' => {
			eco => 'B13h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 Nf6

			variation => N__('Caro-Kann: Exchange, 4. Bd3 Nc6 5. c3 Nf6'),
			moves => {
				'c1f4' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2PB4/PP3PPP/RN1QK1NR b KQkq - 2 6',
				'g1f3' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P4/2PB1N2/PP3PPP/RNBQK2R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp2pppp/2n2n2/6B1/2pP4/2N5/PP3PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'B13v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 dxc4

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Bg5 dxc4'),
			moves => {
				'd4d5' => 'r1bqkb1r/pp2pppp/2n2n2/3P2B1/2p5/2N5/PP3PPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2n5/3n4/3P4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'B13q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. cxd5 Nxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6.cxd5 Nxd5'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/2p5/4P3/2P2N1P/PP1P1PP1/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B50o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. h3 Nc6

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. h3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R w KQkq - 3 5' => {
			eco => 'B50t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Be2 Nc6

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Be2 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/2p5/4P3/2PB1N2/PP1P1PPP/RNBQK2R w KQkq - 3 5' => {
			eco => 'B50r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Bd3 Nc6

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Bd3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 4 6' => {
			eco => 'B60a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5

			variation => N__('Sicilian: Richter-Rauzer'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p2pppp/p1np1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7',
				'c8d7' => 'r2qkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 5 7',
				'd8a5' => 'r1b1kb1r/pp2pppp/2np1n2/q5B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 5 7',
				'd8b6' => 'r1b1kb1r/pp2pppp/1qnp1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 5 7',
				'e7e6' => 'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7',
				'g7g6' => 'r1bqkb1r/pp2pp1p/2np1np1/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B57a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4

			variation => N__('Sicilian: Sozin'),
			moves => {
				'c8d7' => 'r2qkb1r/pp1bpppp/2np1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 5 7',
				'd8b6' => 'r1b1kb1r/pp2pppp/1qnp1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 5 7',
				'g7g6' => 'r1bqkb1r/pp2pp1p/2np1np1/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/2PNP3/5P2/PP4PP/RNBQKB1R b KQkq c3 0 6' => {
			eco => 'B54i',
                        # TRANSLATORS:

			variation => N__('Sicilian: Prins (Moscow), 5... Nc6'),
			moves => {
				'c6d4' => 'r1bqkb1r/pp2pppp/3p1n2/8/2PnP3/5P2/PP4PP/RNBQKB1R w KQkq - 0 7',
				'd8b6' => 'r1b1kb1r/pp2pppp/1qnp1n2/8/2PNP3/5P2/PP4PP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 4 6' => {
			eco => 'B56o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be3

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. Be3'),
			moves => {
				'e7e5' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq e6 0 7',
				'f6g4' => 'r1bqkb1r/pp2pppp/2np4/8/3NP1n1/2N1B3/PPP2PPP/R2QKB1R w KQkq - 5 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N2P2/PPP3PP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B56l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. f3

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. f3'),
			moves => {
				'e7e5' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/3NP3/2N2P2/PPP3PP/R1BQKB1R w KQkq e6 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B56j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. g3

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. g3'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N4P/PPP2PP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B56i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. h3

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. h3'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B58a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2

			variation => N__('Sicilian: Boleslavsky Variation'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p2pppp/p1np1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7',
				'c6d4' => 'r1bqkb1r/pp2pppp/3p1n2/8/3nP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7',
				'c8d7' => 'r2qkb1r/pp1bpppp/2np1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 5 7',
				'e7e5' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq e6 0 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B56h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6

			variation => N__('Sicilian: Open, 2... d6 5. Nc3 Nc6'),
			moves => {
				'c1e3' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 4 6',
				'c1g5' => 'r1bqkb1r/pp2pppp/2np1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 4 6',
				'f1c4' => 'r1bqkb1r/pp2pppp/2np1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 4 6',
				'f1e2' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6',
				'f2f3' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N2P2/PPP3PP/R1BQKB1R b KQkq - 0 6',
				'f2f4' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6',
				'g2g3' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
				'h2h3' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N4P/PPP2PP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B54i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 Nc6

			variation => N__('Sicilian: Prins (Moscow), 5... Nc6'),
			moves => {
				'c2c4' => 'r1bqkb1r/pp2pppp/2np1n2/8/2PNP3/5P2/PP4PP/RNBQKB1R b KQkq c3 0 6',
			},
		},
		'r1bqkb1r/pp2pppp/2np1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6' => {
			eco => 'B56k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. f4

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. f4'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np4/1B6/3NP1n1/2N1B3/PPP2PPP/R2QK2R b KQkq - 6 7' => {
			eco => 'B56r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be3 Ng4 7. Bb5

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. Be3 Ng4 7. Bb5'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2np4/3nP3/2BP4/5N2/PP3PPP/RNBQK2R b KQkq - 3 7' => {
			eco => 'B22u',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 d6 6. Nf3 Nc6 7. Bc4

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 d6 6. Nf3 Nc6 7. Bc4'),
			moves => {
				'd5b6' => 'r1bqkb1r/pp2pppp/1nnp4/4P3/2BP4/5N2/PP3PPP/RNBQK2R w KQkq - 4 8',
			},
		},
		'r1bqkb1r/pp2pppp/2np4/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 2 7' => {
			eco => 'B22t',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 d6 6. Nf3 Nc6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 d6 6. Nf3 Nc6'),
			moves => {
				'f1c4' => 'r1bqkb1r/pp2pppp/2np4/3nP3/2BP4/5N2/PP3PPP/RNBQK2R b KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp2pppp/2np4/8/3NP1n1/2N1B3/PPP2PPP/R2QKB1R w KQkq - 5 7' => {
			eco => 'B56q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be3 Ng4

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. Be3 Ng4'),
			moves => {
				'f1b5' => 'r1bqkb1r/pp2pppp/2np4/1B6/3NP1n1/2N1B3/PPP2PPP/R2QK2R b KQkq - 6 7',
			},
		},
		'r1bqkb1r/pp2pppp/2p2n2/8/2BP4/5N2/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B17e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+ Nxf6 7. Bc4

			variation => N__('Caro-Kann: Steinitz, 5. Nf3 Ngf6 6.Nxf6+ Nxf6 7. Bc4'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/2p2n2/8/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B17d',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Nf3 Ngf6 6.Nxf6+'),
			moves => {
				'f1c4' => 'r1bqkb1r/pp2pppp/2p2n2/8/2BP4/5N2/PPP2PPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp2pppp/3p1n2/8/2PnP3/5P2/PP4PP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B54j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 Nc6 6. c4 Nxd4

			variation => N__('Sicilian: Prins (Moscow), 5... Nc6 6. c4 Nxd4'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/3p1n2/8/3nP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B58d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 Nxd4

			variation => N__('Sicilian: Boleslavsky, 6...Nxd4'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/5n2/n2P2B1/2p5/2N5/PP3PPP/R2QKBNR w KQkq - 1 8' => {
			eco => 'B13v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 dxc4 7. d5 Na5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Herzog Defence'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/n1p2n2/4N3/P1pP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D16f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6 6. Ne5

			variation => N__('Slav: Smyslov, 6. Ne5'),
			moves => {
			},
		},
		'r1bqkb1r/pp2pppp/n1p2n2/8/P1pP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D16g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6 6. e3

			variation => N__('Slav: Smyslov, 6. e3'),
			moves => {
				'c8g4' => 'r2qkb1r/pp2pppp/n1p2n2/8/P1pP2b1/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp2pppp/n1p2n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D16e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6

			variation => N__('Slav: Smyslov Variation'),
			moves => {
				'e2e3' => 'r1bqkb1r/pp2pppp/n1p2n2/8/P1pP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6',
				'e2e4' => 'r1bqkb1r/pp2pppp/n1p2n2/8/P1pPP3/2N2N2/1P3PPP/R1BQKB1R b KQkq e3 0 6',
				'f3e5' => 'r1bqkb1r/pp2pppp/n1p2n2/4N3/P1pP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp2pppp/n1p2n2/8/P1pPP3/2N2N2/1P3PPP/R1BQKB1R b KQkq e3 0 6' => {
			eco => 'D16i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6 6. e4

			variation => N__('Slav: Smyslov, 6. e4'),
			moves => {
				'c8g4' => 'r2qkb1r/pp2pppp/n1p2n2/8/P1pPP1b1/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3p1p/2n1pnp1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'f8g7' => 'r1bqk2r/pp3pbp/2n1pnp1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 w kq - 1 8',
			},
		},
		'r1bqkb1r/pp3p1p/2n1pnp1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e2e4' => 'r1bqkb1r/pp3p1p/2n1pnp1/2pp4/4P3/3P1NP1/PPPN1PBP/R1BQ1RK1 b kq e3 0 7',
			},
		},
		'r1bqkb1r/pp3p2/2nppp1p/8/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 1 9' => {
			eco => 'B63h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 h6'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3p2/p1nppp1p/8/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 0 10',
			},
		},
		'r1bqkb1r/pp3p2/2nppp1p/8/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 9' => {
			eco => 'B63h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 h6'),
			moves => {
				'e1c1' => 'r1bqkb1r/pp3p2/2nppp1p/8/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 1 9',
			},
		},
		'r1bqkb1r/pp3pp1/1np1pn1p/6N1/3P4/3B4/PPP1QPPP/R1B1K1NR w KQkq - 0 9' => {
			eco => 'B17u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Bc4 Ngf6 6. Ng5 e6 7. Qe2 Nb6 8. Bd3 h6

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5, 8. Bd3 h6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3pp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'B58j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O

			variation => N__('Sicilian: Boleslavsky, 7. Nf3 h6 8.O-O'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bpp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'r1bqkb1r/pp3pp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B58i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6

			variation => N__('Sicilian: Boleslavsky, 7. Nf3 h6'),
			moves => {
				'e1g1' => 'r1bqkb1r/pp3pp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r1bqkb1r/pp3pp1/2nppB1p/8/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 0 8' => {
			eco => 'B63h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 h6'),
			moves => {
				'g7f6' => 'r1bqkb1r/pp3p2/2nppp1p/8/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/pp3pp1/2nppn1p/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8' => {
			eco => 'B63h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 h6

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 h6'),
			moves => {
				'g5f6' => 'r1bqkb1r/pp3pp1/2nppB1p/8/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1P1/2N4P/PPP2P2/R1BQKB1R w KQkq - 1 8' => {
			eco => 'B81i',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h3'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1P1/2N4P/PPP2PB1/R1BQK2R b KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1P1/2N4P/PPP2PB1/R1BQK2R b KQkq - 2 8' => {
			eco => 'B81i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h3 Nc6 8. Bg2

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h3'),
			moves => {
			},
		},
		'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKB1R w KQkq - 1 8' => {
			eco => 'B81l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h4 Nc6

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h4 Nc6'),
			moves => {
				'h1g1' => 'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKBR1 b Qkq - 2 8',
			},
		},
		'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKBR1 b Qkq - 2 8' => {
			eco => 'B81m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h4 Nc6 8. Rg1

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h4 Nc6 8. Rg1'),
			moves => {
				'h6h5' => 'r1bqkb1r/pp3pp1/2nppn2/7p/3NP1PP/2N5/PPP2P2/R1BQKBR1 w Qkq - 0 9',
			},
		},
		'r1bqkb1r/pp3pp1/2nppn2/7p/3NP1PP/2N5/PPP2P2/R1BQKBR1 w Qkq - 0 9' => {
			eco => 'B81n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h4 Nc6 8. Rg1 h5

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h4 Nc6 8. Rg1 h5'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/1nn1p3/3pP3/3P4/3B1N2/PP2NPPP/R1BQK2R b KQkq - 2 9' => {
			eco => 'C06c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Nb6 9. Nf3

			variation => N__('French: Tarrasch, Closed, Leningrad, 9. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/1nn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'C06d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Nb6 9. O-O

			variation => N__('French: Tarrasch, Closed, Leningrad, 9.O-O'),
			moves => {
				'a7a5' => 'r1bqkb1r/1p3ppp/1nn1p3/p2pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 w kq a6 0 10',
				'c8d7' => 'r2qkb1r/pp1b1ppp/1nn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 w kq - 3 10',
			},
		},
		'r1bqkb1r/pp3ppp/1nn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 1 9' => {
			eco => 'C06b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Nb6

			variation => N__('French: Tarrasch, Closed, Leningrad'),
			moves => {
				'd2f3' => 'r1bqkb1r/pp3ppp/1nn1p3/3pP3/3P4/3B1N2/PP2NPPP/R1BQK2R b KQkq - 2 9',
				'e1g1' => 'r1bqkb1r/pp3ppp/1nn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 b kq - 2 9',
			},
		},
		'r1bqkb1r/pp3ppp/1np1pn2/6N1/2BP4/8/PPP1QPPP/R1B1K1NR w KQkq - 2 8' => {
			eco => 'B17r',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5'),
			moves => {
				'c4b3' => 'r1bqkb1r/pp3ppp/1np1pn2/6N1/3P4/1B6/PPP1QPPP/R1B1K1NR b KQkq - 3 8',
				'c4d3' => 'r1bqkb1r/pp3ppp/1np1pn2/6N1/3P4/3B4/PPP1QPPP/R1B1K1NR b KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/1np1pn2/6N1/3P4/1B6/PPP1QPPP/R1B1K1NR b KQkq - 3 8' => {
			eco => 'B17s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Bc4 Ngf6 6. Ng5 e6 7. Qe2 Nb6 8. Bb3

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5, 8. Bb3'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/1np1pn2/6N1/3P4/3B4/PPP1QPPP/R1B1K1NR b KQkq - 3 8' => {
			eco => 'B17t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Bc4 Ngf6 6. Ng5 e6 7. Qe2 Nb6 8. Bd3

			variation => N__('Caro-Kann: Steinitz, 5. Bc4 Ngf6 6. Ng5, 8. Bd3'),
			moves => {
				'h7h6' => 'r1bqkb1r/pp3pp1/1np1pn1p/6N1/3P4/3B4/PPP1QPPP/R1B1K1NR w KQkq - 0 9',
			},
		},
		'r1bqkb1r/pp3ppp/2Np1n2/4p3/4P3/2N5/PPP1BPPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'B58g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nxc6

			variation => N__('Sicilian: Boleslavsky, Louma Variation'),
			moves => {
				'b7c6' => 'r1bqkb1r/p4ppp/2pp1n2/4p3/4P3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2Nppn2/6B1/4P3/2N5/PPP2PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B62e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nxc6

			variation => N__('Sicilian: Richter-Rauzer, Richter Attack'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2P2n2/1N6/5p2/2N5/PP2PPPP/R2QKB1R b KQkq - 0 9' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'b7c6' => 'r1bqkb1r/p4ppp/2p2n2/1N6/5p2/2N5/PP2PPPP/R2QKB1R w KQkq - 0 10',
			},
		},
		'r1bqkb1r/pp3ppp/2P2n2/nB2p1N1/8/8/PPPP1PPP/RNBQK2R b KQkq - 0 7' => {
			eco => 'C58e',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Polerio, 6... c6'),
			moves => {
				'b7c6' => 'r1bqkb1r/p4ppp/2p2n2/nB2p1N1/8/8/PPPP1PPP/RNBQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/2pn4/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'D41r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp3ppp/2n1p3/3n4/2Bp4/2N1PN2/PP3PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D41q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6'),
			moves => {
				'f1c4' => 'r1bqkb1r/pp3ppp/2n1p3/2pn4/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 2 7',
				'f1d3' => 'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D41c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3 Nc6

			variation => N__('QGD: Semi-Tarrasch, 6. g3 Nc6'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'D41c',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. g3 Nc6'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'D42a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bd3'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp3ppp/2n1p3/3n4/3p4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2n1p3/2pn4/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/2pp4/2PPn3/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D40j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 Ne4

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 Ne4'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'D41r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQK2R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/3n4/2Bp4/2N1PN2/PP3PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'D41r',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bc4'),
			moves => {
				'e3d4' => 'r1bqkb1r/pp3ppp/2n1p3/3n4/2BP4/2N2N2/PP3PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'D42d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4 8. exd4

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bd3 cxd4'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/3n4/3p4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'D42d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bd3 cxd4

			variation => N__('QGD: Semi-Tarrasch, 6. e3 Nc6 7. Bd3 cxd4'),
			moves => {
				'e3d4' => 'r1bqkb1r/pp3ppp/2n1p3/3n4/3P4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1p3/8/3PP3/5N2/P4PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'D41l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Nc6

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/1N1p4/2P2B2/2N5/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'e6e5' => 'r1bqkb1r/pp3ppp/2n2n2/1N1pp3/2P2B2/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/1N1p4/2P5/2N5/PP2PPPP/R1BQKB1R w KQkq d6 0 7' => {
			eco => 'A33i',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Nc6 6. Ndb5 d5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'c1f4' => 'r1bqkb1r/pp3ppp/2n1pn2/1N1p4/2P2B2/2N5/PP2PPPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p1N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 b kq - 3 7' => {
			eco => 'E04m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Ne5

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Ne5'),
			moves => {
				'c8d7' => 'r2qkb1r/pp1b1ppp/2n1pn2/2p1N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 w kq - 4 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'D26s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O Nc6 7. Nc3

			variation => N__('QGA: Classical, 6.O-O Nc6 7. Nc3'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p5/2BP4/4PN2/PP2QPPP/RNB2RK1 b kq - 3 7' => {
			eco => 'D26t',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O Nc6 7. Qe2

			variation => N__('QGA: Classical, 6.O-O Nc6 7. Qe2'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p5/2BP4/4PN2/PP3PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'D26r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O Nc6

			variation => N__('QGA: Classical, 6.O-O Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp3ppp/2n1pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQ1RK1 b kq - 3 7',
				'd1e2' => 'r1bqkb1r/pp3ppp/2n1pn2/2p5/2BP4/4PN2/PP2QPPP/RNB2RK1 b kq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p5/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'E04l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6'),
			moves => {
				'd1a4' => 'r1bqkb1r/pp3ppp/2n1pn2/2p5/Q1pP4/5NP1/PP2PPBP/RNB2RK1 b kq - 3 7',
				'f3e5' => 'r1bqkb1r/pp3ppp/2n1pn2/2p1N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 b kq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p5/2pP4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D40k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 dxc4

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 dxc4'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2p5/Q1pP4/5NP1/PP2PPBP/RNB2RK1 b kq - 3 7' => {
			eco => 'E04o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Qa4

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Qa4'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp3ppp/2n1pn2/8/Q1pp4/5NP1/PP2PPBP/RNB2RK1 w kq - 0 8',
				'c8d7' => 'r2qkb1r/pp1b1ppp/2n1pn2/2p5/Q1pP4/5NP1/PP2PPBP/RNB2RK1 w kq - 4 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R w KQkq - 1 6' => {
			eco => 'A13n',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. b3 Nf6 4. Bb2 c5 5. e3 Nc6

			variation => N__('English: Wimpey System'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D40f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6'),
			moves => {
				'a2a3' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2PP4/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6',
				'f1d3' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'D40g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. Bd3

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. Bd3'),
			moves => {
				'f8d6' => 'r1bqk2r/pp3ppp/2nbpn2/2pp4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2PP4/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D40i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1n1pn2/2pp4/2PP4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7',
				'c5d4' => 'r1bqkb1r/pp3ppp/2n1pn2/3p4/2Pp4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7',
				'd5c4' => 'r1bqkb1r/pp3ppp/2n1pn2/2p5/2pP4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7',
				'f6e4' => 'r1bqkb1r/pp3ppp/2n1p3/2pp4/2PPn3/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P1B2/2P1PN2/PP3PPP/RN1QKB1R w KQkq - 1 6' => {
			eco => 'D02p',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4 c5 4. e3 e6 5. c3 Nc6

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'f1d3' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P1B2/2PBPN2/PP3PPP/RN1QK2R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P1B2/2PBPN2/PP3PPP/RN1QK2R b KQkq - 2 6' => {
			eco => 'D02p',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'f8d6' => 'r1bqk2r/pp3ppp/2nbpn2/2pp4/3P1B2/2PBPN2/PP3PPP/RN1QK2R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P4/1P1BPN2/P1P2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'D05g',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. b3 Nc6

			variation => N__('Colle: Rubinstein\'s Attack, 5... Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'D05k',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3 Nc6 6. Nbd2

			variation => N__('Colle: 5. c3 Nc6 6.Nbd2'),
			moves => {
				'f8d6' => 'r1bqk2r/pp3ppp/2nbpn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQK2R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P4/2PBPN2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'D05j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3 Nc6

			variation => N__('Colle: 5. c3 Nc6'),
			moves => {
				'b1d2' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P4/2PBPN2/PP1N1PPP/R1BQK2R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'b1d2' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'b7b6' => 'r1bqkb1r/p4ppp/1pn1pn2/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7',
				'f8e7' => 'r1bqk2r/pp2bppp/2n1pn2/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 3 7',
				'g7g6' => 'r1bqkb1r/pp3p1p/2n1pnp1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p2B1/2PP4/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B13y',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nf3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Normal Variation'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n1pn2/3p2B1/2PP4/2N2N2/PP3PPP/R2QKB1R w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'B13y',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 e6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6, Normal Variation'),
			moves => {
				'g1f3' => 'r1bqkb1r/pp3ppp/2n1pn2/3p2B1/2PP4/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B14j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Nc6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p4/2PP4/P1N2N2/1P3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D40l',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 cxd4'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n1pn2/3p4/2PP4/P1N2N2/1P3PPP/R1BQKB1R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p4/2Pp4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D40l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Nc6 6. a3 cxd4

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Nc6 6. a3 cxd4'),
			moves => {
				'e3d4' => 'r1bqkb1r/pp3ppp/2n1pn2/3p4/2PP4/P1N2N2/1P3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D13q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 e6 7. e3

			variation => N__('Slav: Exchange, 6. Bf4 e6 7. e3'),
			moves => {
				'f8d6' => 'r1bqk2r/pp3ppp/2nbpn2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2n1pn2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D13p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 e6

			variation => N__('Slav: Exchange, 6. Bf4 e6'),
			moves => {
				'e2e3' => 'r1bqkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n1pn2/8/Q1pp4/5NP1/PP2PPBP/RNB2RK1 w kq - 0 8' => {
			eco => 'E04o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Qa4 cxd4

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Qa4 cxd4'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/1B6/5p2/5N2/PPPP2PP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Bb5+ c6 6. dxc6 Nxc6

			variation => N__('KGA: Scandinavian, Modern, 5. Bb5+ c6 6.dxc6 Nxc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/1Bpp4/3P4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'D32s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Bb5

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Bb5'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/1N1P4/5p2/2N5/PP2PPPP/R2QKB1R w KQkq - 0 9' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'd5c6' => 'r1bqkb1r/pp3ppp/2P2n2/1N6/5p2/2N5/PP2PPPP/R2QKB1R b KQkq - 0 9',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/1N1Pp3/5B2/2N5/PP2PPPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'e5f4' => 'r1bqkb1r/pp3ppp/2n2n2/1N1P4/5p2/2N5/PP2PPPP/R2QKB1R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/1N1pp3/2P2B2/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'A33i',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Nc6 6.Ndb5 d5'),
			moves => {
				'c4d5' => 'r1bqkb1r/pp3ppp/2n2n2/1N1Pp3/5B2/2N5/PP2PPPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 7' => {
			eco => 'D32o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. Bf4 Nf6

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. Bf4 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 2 7' => {
			eco => 'D32t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp3ppp/2n2n2/3p4/3p4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D32r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6'),
			moves => {
				'f1b5' => 'r1bqkb1r/pp3ppp/2n2n2/1Bpp4/3P4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 2 7',
				'f1e2' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D33l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6

			variation => N__('QGD Tarrasch: 6. g3 Nf6 (Prague)'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'D33l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2

			variation => N__('QGD Tarrasch: 6. g3 Nf6 (Prague)'),
			moves => {
				'c5d4' => 'r1bqkb1r/pp3ppp/2n2n2/3p4/3p4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 0 8',
				'c8e6' => 'r2qkb1r/pp3ppp/2n1bn2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8',
				'c8g4' => 'r2qkb1r/pp3ppp/2n2n2/2pp4/3P2b1/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 7' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nc6 6. Bg2 Nf6 7. O-O

			variation => N__('QGD: Tarrasch without Nc3: 5. g3 Nc6 6. Bg2 Nf6 7.O-O'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n2n2/2pp4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 5 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 7' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nc6 6. Bg2 Nf6

			variation => N__('QGD: Tarrasch without Nc3: 5. g3 Nc6 6. Bg2 Nf6'),
			moves => {
				'e1g1' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 7',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/3p4/3N4/2N1P3/PP2BPPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'D32v',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4'),
			moves => {
				'f8d6' => 'r1bqk2r/pp3ppp/2nb1n2/3p4/3N4/2N1P3/PP2BPPP/R1BQK2R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQK2R b KQkq - 0 8' => {
			eco => 'D33o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 cxd4 8. Nxd4

			variation => N__('QGD Tarrasch: 7. Bg2 cxd4 8.Nxd4'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQK2R w KQkq - 1 9',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/3p4/3p4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'D32v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3 Nf6 7. Be2 cxd4'),
			moves => {
				'f3d4' => 'r1bqkb1r/pp3ppp/2n2n2/3p4/3N4/2N1P3/PP2BPPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2n2n2/3p4/3p4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 0 8' => {
			eco => 'D33o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 cxd4

			variation => N__('QGD Tarrasch: 7. Bg2 cxd4'),
			moves => {
				'f3d4' => 'r1bqkb1r/pp3ppp/2n2n2/3p4/3N4/2N3P1/PP2PPBP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/1N1Np3/4P3/8/PPP2PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B33h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Nd5

			variation => N__('Sicilian: Pelikan, 7. Nd5'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/1N2p1B1/4P3/2N5/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B33i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5

			variation => N__('Sicilian: Pelikan, 7. Bg5'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1np1n2/1N2p1B1/4P3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/1N2p3/4P3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B58f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Ndb5

			variation => N__('Sicilian: Boleslavsky, 7.Ndb5'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B33f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6

			variation => N__('Sicilian: Pelikan/Sveshnikov, Pelikan Variation'),
			moves => {
				'a2a4' => 'r1bqkb1r/pp3ppp/2np1n2/1N2p3/P3P3/2N5/1PP2PPP/R1BQKB1R b KQkq a3 0 7',
				'b5a3' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/N1N5/PPP2PPP/R1BQKB1R b KQkq - 1 7',
				'c1g5' => 'r1bqkb1r/pp3ppp/2np1n2/1N2p1B1/4P3/2N5/PPP2PPP/R2QKB1R b KQkq - 1 7',
				'c3d5' => 'r1bqkb1r/pp3ppp/2np1n2/1N1Np3/4P3/8/PPP2PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/1N2p3/4P3/4B3/PPP2PPP/RN1QKB1R w KQkq - 2 8' => {
			eco => 'B44n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. Bf4 e5 7. Be3 Nf6

			variation => N__('Sicilian, Taimanov, Szen, 6. Bf4 e5 7. Be3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/1N2p3/P3P3/2N5/1PP2PPP/R1BQKB1R b KQkq a3 0 7' => {
			eco => 'B33g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. a4

			variation => N__('Sicilian: Pelikan, 7. a4'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1np1n2/1N2p3/P3P3/2N5/1PP2PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq e6 0 7' => {
			eco => 'B56p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be3 e5

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. Be3 e5'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/3NP3/2N2P2/PPP3PP/R1BQKB1R w KQkq e6 0 7' => {
			eco => 'B56m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. f3 e5

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. f3 e5'),
			moves => {
				'd4b3' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/1NN2P2/PPP3PP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq e6 0 7' => {
			eco => 'B58e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5

			variation => N__('Sicilian: Boleslavsky, 6... e5'),
			moves => {
				'd4b3' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R b KQkq - 1 7',
				'd4b5' => 'r1bqkb1r/pp3ppp/2np1n2/1N2p3/4P3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 7',
				'd4c6' => 'r1bqkb1r/pp3ppp/2Np1n2/4p3/4P3/2N5/PPP1BPPP/R1BQK2R b KQkq - 0 7',
				'd4f3' => 'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/1NN2P2/PPP3PP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B56m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. f3 e5 7. Nb3

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Nc6 6. f3 e5 7. Nb3'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN2P2/PPP3PP/R1BQKB1R w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B59a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3

			variation => N__('Sicilian: Boleslavsky, 7. Nb3'),
			moves => {
				'c8e6' => 'r2qkb1r/pp3ppp/2npbn2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R w KQkq - 2 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B58h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3

			variation => N__('Sicilian: Boleslavsky, 7. Nf3'),
			moves => {
				'h7h6' => 'r1bqkb1r/pp3pp1/2np1n1p/4p3/4P3/2N2N2/PPP1BPPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2np1n2/4p3/4P3/N1N5/PPP2PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B33f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Na3

			variation => N__('Sicilian: Pelikan, 7. Na3'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/1B4B1/3NP3/2N5/PPP2PPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B62g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Bb5

			variation => N__('Sicilian: Richter-Rauzer, Margate (Alekhine) Variation'),
			moves => {
				'c8d7' => 'r2qkb1r/pp1b1ppp/2nppn2/1B4B1/3NP3/2N5/PPP2PPP/R2QK2R w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/1N6/2P1P3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B44q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3

			variation => N__('Sicilian, Taimanov, Szen, 7.N1c3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1nppn2/1N6/2P1P3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/1N6/2P1P3/8/PP3PPP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'B44o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6

			variation => N__('Sicilian, Taimanov, Szen, 6. c4 Nf6'),
			moves => {
				'b1c3' => 'r1bqkb1r/pp3ppp/2nppn2/1N6/2P1P3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 7',
				'b5c3' => 'r1bqkb1r/pp3ppp/2nppn2/8/2P1P3/2N5/PP3PPP/RNBQKB1R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/1N6/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B45o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 d6

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 d6'),
			moves => {
				'c1f4' => 'r1bqkb1r/pp3ppp/2nppn2/1N6/4PB2/2N5/PPP2PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/1N6/4PB2/2N5/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B45o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Nf6 6. Ndb5 d6 7. Bf4

			variation => N__('Sicilian: Taimanov, Four Knights, 6.Ndb5 d6 7. Bf4'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B62b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Be2

			variation => N__('Sicilian: Richter-Rauzer, 6... e6 7. Be2'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R w KQkq - 2 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B62a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6

			variation => N__('Sicilian: Richter-Rauzer, 6... e6'),
			moves => {
				'd1d2' => 'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 1 7',
				'd1d3' => 'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2NQ4/PPP2PPP/R3KB1R b KQkq - 1 7',
				'd4b3' => 'r1bqkb1r/pp3ppp/2nppn2/6B1/4P3/1NN5/PPP2PPP/R2QKB1R b KQkq - 1 7',
				'd4c6' => 'r1bqkb1r/pp3ppp/2Nppn2/6B1/4P3/2N5/PPP2PPP/R2QKB1R b KQkq - 0 7',
				'f1b5' => 'r1bqkb1r/pp3ppp/2nppn2/1B4B1/3NP3/2N5/PPP2PPP/R2QK2R b KQkq - 1 7',
				'f1e2' => 'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B63a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2

			variation => N__('Sicilian: Richter-Rauzer, Rauzer Attack (7. Qd2)'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8',
				'c6d4' => 'r1bqkb1r/pp3ppp/3ppn2/6B1/3nP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8',
				'd8b6' => 'r1b1kb1r/pp3ppp/1qnppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 2 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 2 8',
				'h7h6' => 'r1bqkb1r/pp3pp1/2nppn1p/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/6B1/3NP3/2NQ4/PPP2PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B62f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd3

			variation => N__('Sicilian: Richter-Rauzer, Keres Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/6B1/4P3/1NN5/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B62d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Nb3

			variation => N__('Sicilian: Richter-Rauzer, Podebrady Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/6P1/3NP3/2N5/PPP2P1P/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B81q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Keres, 6... Nc6'),
			moves => {
				'f6d7' => 'r1bqkb1r/pp1n1ppp/2npp3/6P1/3NP3/2N5/PPP2P1P/R1BQKB1R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/2B1P3/2N2N2/PP2QPPP/R1B2RK1 b kq - 3 8' => {
			eco => 'B21z',
                        # TRANSLATORS:

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O Nf6'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/8/2B1P3/2N2N2/PP2QPPP/R1B2RK1 w kq - 4 9',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/2B1P3/2N2N2/PP3PPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'B21z',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4 e6 7. O-O Nf6

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O Nf6'),
			moves => {
				'd1e2' => 'r1bqkb1r/pp3ppp/2nppn2/8/2B1P3/2N2N2/PP2QPPP/R1B2RK1 b kq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/2BNP3/2N1B3/PPP2PPP/R2QK2R b KQkq - 3 7' => {
			eco => 'B89',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Be3

			variation => N__('Sicilian: Sozin, 7. Be3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1nppn2/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 0 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 4 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/2BNP3/2N5/PPP2PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'B88',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. O-O

			variation => N__('Sicilian: Sozin-Scheveningen, 7.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B88',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6

			variation => N__('Sicilian: Sozin-Scheveningen'),
			moves => {
				'c1e3' => 'r1bqkb1r/pp3ppp/2nppn2/8/2BNP3/2N1B3/PPP2PPP/R2QK2R b KQkq - 3 7',
				'c4b3' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R b KQkq - 3 7',
				'e1g1' => 'r1bqkb1r/pp3ppp/2nppn2/8/2BNP3/2N5/PPP2PPP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/2P1P3/2N5/PP3PPP/RNBQKB1R b KQkq - 2 7' => {
			eco => 'B44p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N5c3

			variation => N__('Sicilian, Taimanov, Szen, 7.N5c3'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B81q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 Nc6

			variation => N__('Sicilian: Scheveningen, Keres, 6... Nc6'),
			moves => {
				'g4g5' => 'r1bqkb1r/pp3ppp/2nppn2/6P1/3NP3/2N5/PPP2P1P/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R b KQkq - 3 7' => {
			eco => 'B88',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Nc6 7. Bb3

			variation => N__('Sicilian: Sozin-Scheveningen, 7. Bb3'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R w KQkq - 4 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B80c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 Nc6

			variation => N__('Sicilian: Scheveningen, 6. Be3 Nc6'),
			moves => {
				'f2f3' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B80d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 Nc6 7. f3

			variation => N__('Sicilian: Scheveningen, 6. Be3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B80q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'f1g2' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B80q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'a7a6' => 'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 0 8',
				'd8c7' => 'r1b1kb1r/ppq2ppp/2nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQK2R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'B83',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Modern'),
			moves => {
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 4 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6

			variation => N__('Sicilian: Scheveningen, Modern'),
			moves => {
				'e1g1' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R b KQkq - 2 7' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6 7. Be3

			variation => N__('Sicilian: Scheveningen, 6. f4 Nc6 7. Be3'),
			moves => {
				'd8c7' => 'r1b1kb1r/ppq2ppp/2nppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq - 3 8',
				'f8e7' => 'r1bqk2r/pp2bppp/2nppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/pp3ppp/2nppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Nc6

			variation => N__('Sicilian: Scheveningen, 6. f4 Nc6'),
			moves => {
				'c1e3' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/pp3ppp/2p2n2/nB1Pp1N1/8/8/PPPP1PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C58e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ c6

			variation => N__('Two Knights: Morphy, Polerio, 6... c6'),
			moves => {
				'd5c6' => 'r1bqkb1r/pp3ppp/2P2n2/nB2p1N1/8/8/PPPP1PPP/RNBQK2R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pp3ppp/3ppn2/6B1/3nP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8' => {
			eco => 'B63b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Nxd4

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Nxd4'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/4pn2/2P5/8/3B1N2/PPP2PPP/R1BQK2R b KQkq - 0 8' => {
			eco => 'C10p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+ Nxf6 7. Bd3 c5 8. dxc5

			variation => N__('French: Rubinstein, 7. Bd3 c5 8.dxc5'),
			moves => {
			},
		},
		'r1bqkb1r/pp3ppp/4pn2/2p5/3P4/3B1N2/PPP2PPP/R1BQK2R w KQkq c6 0 8' => {
			eco => 'C10o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+ Nxf6 7. Bd3 c5

			variation => N__('French: Rubinstein, 7. Bd3 c5'),
			moves => {
				'd4c5' => 'r1bqkb1r/pp3ppp/4pn2/2P5/8/3B1N2/PPP2PPP/R1BQK2R b KQkq - 0 8',
			},
		},
		'r1bqkb1r/pp3ppp/n1p2n2/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 2 7' => {
			eco => 'D36b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Na6

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. Qc2 Na6'),
			moves => {
			},
		},
		'r1bqkb1r/pp4pp/2n1pn2/3p4/3P4/3B4/PP1NNPPP/R1BQ1RK1 b kq - 1 10' => {
			eco => 'C06n',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6 Nxf6'),
			moves => {
				'f8d6' => 'r1bqk2r/pp4pp/2nbpn2/3p4/3P4/3B4/PP1NNPPP/R1BQ1RK1 w kq - 2 11',
			},
		},
		'r1bqkb1r/pp4pp/2n1pn2/3p4/3P4/3B4/PP1NNPPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'C06n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 f6 9. exf6 Nxf6

			variation => N__('French: Tarrasch, Closed, 8... f6 9.exf6 Nxf6'),
			moves => {
				'e1g1' => 'r1bqkb1r/pp4pp/2n1pn2/3p4/3P4/3B4/PP1NNPPP/R1BQ1RK1 b kq - 1 10',
			},
		},
		'r1bqkb1r/ppp1nppp/2n5/4P3/2Pp4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Nge7

			variation => N__('QGD: Albin, 5. g3 Nge7'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pp1p/2np1np1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'B00q',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3 g6

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3 g6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/1nn5/4P3/2PP4/4B3/PP4PP/RN1QKBNR b KQkq - 2 7' => {
			eco => 'B03n',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 Nc6 7. Be3

			variation => N__('Alekhine: Four Pawns Attack, 7. Be3'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/1nn5/4P3/2PP4/5N2/PP4PP/RNBQKB1R b KQkq - 2 7' => {
			eco => 'B03m',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Nc6'),
			moves => {
				'c8g4' => 'r2qkb1r/ppp1pppp/1nn5/4P3/2PP2b1/5N2/PP4PP/RNBQKB1R w KQkq - 3 8',
			},
		},
		'r1bqkb1r/ppp1pppp/1nn5/4P3/2PP4/8/PP4PP/RNBQKBNR w KQkq - 1 7' => {
			eco => 'B03m',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 Nc6

			variation => N__('Alekhine: Four Pawns Attack, 6... Nc6'),
			moves => {
				'c1e3' => 'r1bqkb1r/ppp1pppp/1nn5/4P3/2PP4/4B3/PP4PP/RN1QKBNR b KQkq - 2 7',
				'g1f3' => 'r1bqkb1r/ppp1pppp/1nn5/4P3/2PP4/5N2/PP4PP/RNBQKB1R b KQkq - 2 7',
			},
		},
		'r1bqkb1r/ppp1pppp/1nn5/4P3/3P4/1B6/PP2NPPP/RNBQK2R b KQkq - 4 7' => {
			eco => 'D20r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5 5. Bxc4 Nb6 6. Bb3 Nc6 7. Ne2

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5 5.Bxc4 Nb6 6. Bb3 Nc6 7. Ne2'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/1nn5/4P3/3P4/1B6/PP3PPP/RNBQK1NR w KQkq - 3 7' => {
			eco => 'D20q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5 5. Bxc4 Nb6 6. Bb3 Nc6

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5 5.Bxc4 Nb6 6. Bb3'),
			moves => {
				'g1e2' => 'r1bqkb1r/ppp1pppp/1nn5/4P3/3P4/1B6/PP2NPPP/RNBQK2R b KQkq - 4 7',
			},
		},
		'r1bqkb1r/ppp1pppp/1nn5/4P3/3P4/3B4/PP3PPP/RNBQK1NR w KQkq - 3 7' => {
			eco => 'D20p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5 5. Bxc4 Nb6 6. Bd3 Nc6

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5 5.Bxc4 Nb6 6. Bd3'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 4 4' => {
			eco => 'D07q',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 Nf6 4. Nf3

			variation => N__('QGD: Chigorin, 3. Nc3 Nf6 4. Nf3'),
			moves => {
				'c8g4' => 'r2qkb1r/ppp1pppp/2n2n2/3p4/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 5 5',
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 3 4' => {
			eco => 'D07p',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 Nf6

			variation => N__('QGD: Chigorin, 3. Nc3 Nf6'),
			moves => {
				'g1f3' => 'r1bqkb1r/ppp1pppp/2n2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 4 4',
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'B00w',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. Nc3 Nf6

			variation => N__('Nimzowitsch Defence: Bogoljubow, 3... Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'D07v',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 dxc4 4. Nf3 Nf6

			variation => N__('QGD: Chigorin, 3. Nc3 dxc4 4. Nf3 Nf6'),
			moves => {
				'e2e4' => 'r1bqkb1r/ppp1pppp/2n2n2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5',
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'D07w',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 dxc4 4. Nf3 Nf6 5. e4

			variation => N__('QGD: Chigorin, 3. Nc3 dxc4 4. Nf3 Nf6 5. e4'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 Nc6

			variation => N__('Blackmar-Diemer: Lamb Defence'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/8/Q1pP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5' => {
			eco => 'D23e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ Nc6 5. Nc3

			variation => N__('QGA: Mannheim, 4... Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/2n2n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 4 5' => {
			eco => 'D23d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ Nc6

			variation => N__('QGA: Mannheim, 4... Nc6'),
			moves => {
				'b1c3' => 'r1bqkb1r/ppp1pppp/2n2n2/8/Q1pP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5',
			},
		},
		'r1bqkb1r/ppp1pppp/2np1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A50d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6 3. Nf3 d6

			variation => N__('Indian: Mexican Defence, 3. Nf3 d6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp1pppp/2np1n2/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 4' => {
			eco => 'B00q',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3'),
			moves => {
				'c8g4' => 'r2qkb1r/ppp1pppp/2np1n2/8/3PP1b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 5',
				'g7g6' => 'r1bqkb1r/ppp1pp1p/2np1np1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppp1pppp/2np1n2/8/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B00p',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6'),
			moves => {
				'b1c3' => 'r1bqkb1r/ppp1pppp/2np1n2/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 4',
			},
		},
		'r1bqkb1r/ppp1pppp/2np4/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'B04b',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Nc6

			variation => N__('Alekhine: Modern, 4... Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2Npp/2n5/3np3/2B5/8/PPPP1PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C57g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nxd5 6. Nxf7

			variation => N__('Two Knights: Fegatello (Fried Liver) Attack'),
			moves => {
				'e8f7' => 'r1bq1b1r/ppp2kpp/2n5/3np3/2B5/8/PPPP1PPP/RNBQK2R w KQ - 0 7',
			},
		},
		'r1bqkb1r/ppp2pp1/5n1p/3P4/2P1p3/5N2/PPP1QPPP/RNB1K2R b KQkq - 0 9' => {
			eco => 'C58c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Kieseritsky Variation, 6... h6 7. Nf3 e4 8. Qe2'),
			moves => {
				'f8c5' => 'r1bqk2r/ppp2pp1/5n1p/2bP4/2P1p3/5N2/PPP1QPPP/RNB1K2R w KQkq - 1 10',
				'f8e7' => 'r1bqk2r/ppp1bpp1/5n1p/3P4/2P1p3/5N2/PPP1QPPP/RNB1K2R w KQkq - 1 10',
			},
		},
		'r1bqkb1r/ppp2pp1/5n1p/3P4/2n1p3/3P1N2/PPP1QPPP/RNB1K2R w KQkq - 0 9' => {
			eco => 'C58c',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Kieseritsky Variation, 6... h6 7. Nf3 e4 8. Qe2'),
			moves => {
				'd3c4' => 'r1bqkb1r/ppp2pp1/5n1p/3P4/2P1p3/5N2/PPP1QPPP/RNB1K2R b KQkq - 0 9',
			},
		},
		'r1bqkb1r/ppp2pp1/5n1p/n2P4/2B1p3/3P1N2/PPP1QPPP/RNB1K2R b KQkq - 1 8' => {
			eco => 'C58c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. d3 h6 7. Nf3 e4 8. Qe2

			variation => N__('Two Knights: Morphy, Kieseritsky Variation, 6... h6 7. Nf3 e4 8. Qe2'),
			moves => {
				'a5c4' => 'r1bqkb1r/ppp2pp1/5n1p/3P4/2n1p3/3P1N2/PPP1QPPP/RNB1K2R w KQkq - 0 9',
			},
		},
		'r1bqkb1r/ppp2pp1/5n1p/n2P4/2B1p3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 8' => {
			eco => 'C58b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Kieseritsky Variation'),
			moves => {
				'd1e2' => 'r1bqkb1r/ppp2pp1/5n1p/n2P4/2B1p3/3P1N2/PPP1QPPP/RNB1K2R b KQkq - 1 8',
			},
		},
		'r1bqkb1r/ppp2pp1/5n1p/n2Pp1N1/2B5/3P4/PPP2PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C58b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Kieseritsky Variation'),
			moves => {
				'g5f3' => 'r1bqkb1r/ppp2pp1/5n1p/n2Pp3/2B5/3P1N2/PPP2PPP/RNBQK2R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/ppp2pp1/5n1p/n2Pp3/2B5/3P1N2/PPP2PPP/RNBQK2R b KQkq - 1 7' => {
			eco => 'C58b',
                        # TRANSLATORS:

			variation => N__('Two Knights: Morphy, Kieseritsky Variation'),
			moves => {
				'e5e4' => 'r1bqkb1r/ppp2pp1/5n1p/n2P4/2B1p3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/ppp2ppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6 7. O-O

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6 7.O-O'),
			moves => {
				'f8e7' => 'r1bqk2r/ppp1bppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 4 8',
			},
		},
		'r1bqkb1r/ppp2ppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Nb6

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Nb6'),
			moves => {
				'e1g1' => 'r1bqkb1r/ppp2ppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2Bp1n2/4p3/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C65h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 d6 5. Bxc6+

			variation => N__('Spanish: Berlin, Anderssen Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'C10d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nc6 4. Nf3 Nf6

			variation => N__('French: 3. Nc3 Nc6 4. Nf3 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'C04a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6

			variation => N__('French: Tarrasch, Guimard, 4.Ngf3 Nf6'),
			moves => {
				'e4e5' => 'r1bqkb1r/ppp2ppp/2n1pn2/3pP3/3P4/5N2/PPPN1PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/3p4/4P3/2PP1N2/PP1N1PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C00v',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2 Nf6 4. Ngf3 Nc6 5. c3

			variation => N__('French: KIA, Reversed Philidor, 4... Nc6 5. c3'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/3p4/4P3/3P1N2/PPPN1PPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'C00u',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2 Nf6 4. Ngf3 Nc6

			variation => N__('French: KIA, Reversed Philidor, 4... Nc6'),
			moves => {
				'c2c3' => 'r1bqkb1r/ppp2ppp/2n1pn2/3p4/4P3/2PP1N2/PP1N1PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/3pP3/3P4/5N2/PPPN1PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C04b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5

			variation => N__('French: Tarrasch, Guimard, 4.Ngf3 Nf6 5. e5'),
			moves => {
				'f6d7' => 'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 3 6' => {
			eco => 'E04s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6 6. O-O

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6.O-O'),
			moves => {
				'a8b8' => '1rbqkb1r/ppp2ppp/2n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w k - 4 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6' => {
			eco => 'E04s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6

			variation => N__('Catalan: Open, 5. Nf3 Nc6'),
			moves => {
				'd1a4' => 'r1bqkb1r/ppp2ppp/2n1pn2/8/Q1pP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 3 6',
				'e1g1' => 'r1bqkb1r/ppp2ppp/2n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 3 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n1pn2/8/Q1pP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 3 6' => {
			eco => 'E04v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nc6 6. Qa4

			variation => N__('Catalan: Open, 5. Nf3 Nc6 6. Qa4'),
			moves => {
				'f8b4' => 'r1bqk2r/ppp2ppp/2n1pn2/8/QbpP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 4 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/1B1pP3/3p4/5N2/PPP2PPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'C55p',
                        # TRANSLATORS:

			variation => N__('Two Knights: Modern, 5. e5 d5'),
			moves => {
				'f6e4' => 'r1bqkb1r/ppp2ppp/2n5/1B1pP3/3pn3/5N2/PPP2PPP/RNBQK2R w KQkq - 2 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3Pp1N1/2B5/8/PPPP1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C57e',
                        # TRANSLATORS:

			variation => N__('Two Knights: 4. Ng5 d5'),
			moves => {
				'b7b5' => 'r1bqkb1r/p1p2ppp/2n2n2/1p1Pp1N1/2B5/8/PPPP1PPP/RNBQK2R w KQkq b6 0 6',
				'c6a5' => 'r1bqkb1r/ppp2ppp/5n2/n2Pp1N1/2B5/8/PPPP1PPP/RNBQK2R w KQkq - 1 6',
				'c6b4' => 'r1bqkb1r/ppp2ppp/5n2/3Pp1N1/1nB5/8/PPPP1PPP/RNBQK2R w KQkq - 1 6',
				'c6d4' => 'r1bqkb1r/ppp2ppp/5n2/3Pp1N1/2Bn4/8/PPPP1PPP/RNBQK2R w KQkq - 1 6',
				'f6d5' => 'r1bqkb1r/ppp2ppp/2n5/3np1N1/2B5/8/PPPP1PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3Pp3/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5'),
			moves => {
				'f6d5' => 'r1bqkb1r/ppp2ppp/2n5/3np3/8/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3p2B1/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 4 6' => {
			eco => 'C01t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Nf6 5. Bg5 Nc6

			variation => N__('French: Exchange, Bogoljubow Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pP3/2Bp4/5N2/PPP2PPP/RNBQK2R w KQkq d6 0 6' => {
			eco => 'C55p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. e5 d5

			variation => N__('Two Knights: Modern, 5. e5 d5'),
			moves => {
				'c4b5' => 'r1bqkb1r/ppp2ppp/2n2n2/1B1pP3/3p4/5N2/PPP2PPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp1N1/2B1P3/8/PPPP1PPP/RNBQK2R w KQkq d6 0 5' => {
			eco => 'C57e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5

			variation => N__('Two Knights: 4. Ng5 d5'),
			moves => {
				'e4d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3Pp1N1/2B5/8/PPPP1PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq d6 0 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5

			variation => N__('English: Four Knights, 4. g3 d5'),
			moves => {
				'c4d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3Pp3/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/4P3/2N2NP1/PPPP1P1P/R1BQKB1R w KQkq d6 0 5' => {
			eco => 'C47f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. g3 d5

			variation => N__('Four Knights: Glek, 4... d5'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/4P3/3P1N2/PPP1BPPP/RNBQK2R w KQkq d6 0 5' => {
			eco => 'C44c',
                        # TRANSLATORS:

			variation => N__('Open Game: Inverted Hungarian'),
			moves => {
				'b1d2' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/4P3/3P1N2/PPPNBPPP/R1BQK2R b KQkq - 1 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/4P3/3P1N2/PPPNBPPP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C44h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Be2 Nf6 4. d3 d5 5. Nbd2

			variation => N__('Open Game: Inverted Hanham'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 2 5' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nc6 3. Bg2 e5 4. d3 Nf6 5. O-O

			variation => N__('Reti: KIA'),
			moves => {
				'f8e7' => 'r1bqk2r/ppp1bppp/2n2n2/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 3 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/8/3P1NP1/PPP1PPBP/RNBQK2R w KQkq - 1 5' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nc6 3. Bg2 e5 4. d3 Nf6

			variation => N__('Reti: KIA'),
			moves => {
				'e1g1' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 2 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2n2n2/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R w KQkq - 2 5' => {
			eco => 'C44l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 d5 4. Qa4 Nf6

			variation => N__('Ponziani: Leonhardt Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/1B1pP3/3Nn3/8/PPP2PPP/RNBQK2R b KQkq - 0 7' => {
			eco => 'C55p',
                        # TRANSLATORS:

			variation => N__('Two Knights: Modern, 5. e5 d5'),
			moves => {
				'c8d7' => 'r2qkb1r/pppb1ppp/2n5/1B1pP3/3Nn3/8/PPP2PPP/RNBQK2R w KQkq - 1 8',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/1B1pP3/3pn3/5N2/PPP2PPP/RNBQK2R w KQkq - 2 7' => {
			eco => 'C55p',
                        # TRANSLATORS:

			variation => N__('Two Knights: Modern, 5. e5 d5'),
			moves => {
				'f3d4' => 'r1bqkb1r/ppp2ppp/2n5/1B1pP3/3Nn3/8/PPP2PPP/RNBQK2R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3B4/3pn3/5N2/PPP2PPP/RNBQR1K1 b kq - 0 7' => {
			eco => 'C56d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5

			variation => N__('Two Knights: Classical, 7.Bxd5'),
			moves => {
				'd8d5' => 'r1b1kb1r/ppp2ppp/2n5/3q4/3pn3/5N2/PPP2PPP/RNBQR1K1 w kq - 0 8',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3np1N1/2B5/8/PPPP1PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C57e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nxd5

			variation => N__('Two Knights: 4. Ng5 d5 5.exd5 Nxd5'),
			moves => {
				'd2d4' => 'r1bqkb1r/ppp2ppp/2n5/3np1N1/2BP4/8/PPP2PPP/RNBQK2R b KQkq d3 0 6',
				'g5f7' => 'r1bqkb1r/ppp2Npp/2n5/3np3/2B5/8/PPPP1PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3np1N1/2BP4/8/PPP2PPP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C57f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nxd5 6. d4

			variation => N__('Two Knights: Lolli Attack'),
			moves => {
				'f8b4' => 'r1bqk2r/ppp2ppp/2n5/3np1N1/1bBP4/8/PPP2PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3np3/8/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5'),
			moves => {
				'f1g2' => 'r1bqkb1r/ppp2ppp/2n5/3np3/8/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3np3/8/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2'),
			moves => {
				'c8e6' => 'r2qkb1r/ppp2ppp/2n1b3/3np3/8/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 2 7',
				'd5b6' => 'r1bqkb1r/ppp2ppp/1nn5/4p3/8/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3p4/2Bpn3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 1 7' => {
			eco => 'C56c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Nc3

			variation => N__('Two Knights: Classical, Canal Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3p4/2Bpn3/5N2/PPP2PPP/RNBQR1K1 w kq d6 0 7' => {
			eco => 'C56b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5

			variation => N__('Two Knights: Classical, 6. Re1 d5'),
			moves => {
				'b1c3' => 'r1bqkb1r/ppp2ppp/2n5/3p4/2Bpn3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 1 7',
				'c4d5' => 'r1bqkb1r/ppp2ppp/2n5/3B4/3pn3/5N2/PPP2PPP/RNBQR1K1 b kq - 0 7',
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C43k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nc6

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3pP3/4n3/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C29q',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Nc6

			variation => N__('Vienna Gambit: 5. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/3pP3/4n3/2N2Q2/PPPP2PP/R1B1KBNR w KQkq - 2 6' => {
			eco => 'C29k',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Qf3 Nc6

			variation => N__('Vienna Gambit: Paulsen Attack, 5... Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2n5/8/3pn3/2N2N2/PPP2PPP/R1BQR1K1 w kq - 2 9' => {
			eco => 'C56e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qd8

			variation => N__('Two Knights: Classical, 8. Nc3 Qd8'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2nn4/8/3Q4/2N2N2/PPP2PPP/R1B1KB1R w KQkq - 2 8' => {
			eco => 'C43e',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line, 7. Nc3'),
			moves => {
				'd4f4' => 'r1bqkb1r/ppp2ppp/2nn4/8/5Q2/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 3 8',
			},
		},
		'r1bqkb1r/ppp2ppp/2nn4/8/5Q2/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 3 8' => {
			eco => 'C43e',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line, 7. Nc3'),
			moves => {
				'c8f5' => 'r2qkb1r/ppp2ppp/2nn4/5b2/5Q2/2N2N2/PPP2PPP/R1B1KB1R w KQkq - 4 9',
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/2P1P3/3P1N2/PP3PPP/RNBQK2R b KQkq c3 0 5' => {
			eco => 'C65h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 d6 5. c4

			variation => N__('Spanish: Berlin, Duras Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq d3 0 5' => {
			eco => 'C48c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 d6 5. d4

			variation => N__('Four Knights: Spanish, 4... d6 5. d4'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 5' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4

			variation => N__('Spanish: Closed Berlin, 5. d4'),
			moves => {
				'c8d7' => 'r2qkb1r/pppb1ppp/2np1n2/1B2p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6',
				'f6d7' => 'r1bqkb1r/pppn1ppp/2np4/1B2p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5' => {
			eco => 'C48b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 d6

			variation => N__('Four Knights: Spanish, 4... d6'),
			moves => {
				'd2d4' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq d3 0 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C65i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 d6 5. c3

			variation => N__('Spanish: Berlin, 4. d3 d6 5. c3'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C65h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 d6

			variation => N__('Spanish: Berlin, 4. d3 d6'),
			moves => {
				'b5c6' => 'r1bqkb1r/ppp2ppp/2Bp1n2/4p3/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'c2c3' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/2PP1N2/PP3PPP/RNBQK2R b KQkq - 0 5',
				'c2c4' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/2P1P3/3P1N2/PP3PPP/RNBQK2R b KQkq c3 0 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6

			variation => N__('Spanish: Closed Berlin'),
			moves => {
				'd2d4' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 5',
				'f1e1' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 5',
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 5' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. Re1

			variation => N__('Spanish: Closed Berlin, 5. Re1'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nc6

			variation => N__('Old Indian: 4. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C47i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 d6

			variation => N__('Four Knights: Scotch, 4... d6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/8/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C55t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O d6

			variation => N__('Two Knights: 5.O-O d6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2np1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C47p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4 d6

			variation => N__('Four Knights: Scotch, 4...exd4 5.Nxd4 d6'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/2p5/4Pn2/8/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C67q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6 7. dxe5 Nf5

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 6.Bxc6 dxc6 7.dxe5 Nf5'),
			moves => {
				'd1d8' => 'r1bQkb1r/ppp2ppp/2p5/4Pn2/8/5N2/PPP2PPP/RNB2RK1 b kq - 0 8',
			},
		},
		'r1bqkb1r/ppp2ppp/2pn4/4P3/8/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C67p',
                        # TRANSLATORS:

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 6.Bxc6 dxc6'),
			moves => {
				'd6f5' => 'r1bqkb1r/ppp2ppp/2p5/4Pn2/8/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r1bqkb1r/ppp2ppp/2pn4/4p3/3P4/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C67p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6 dxc6

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 6.Bxc6 dxc6'),
			moves => {
				'd4e5' => 'r1bqkb1r/ppp2ppp/2pn4/4P3/8/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqkb1r/ppp2ppp/4pn2/4N3/3P4/8/PPP2PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'C10m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+ Nxf6 7. Ne5

			variation => N__('French: Rubinstein, Capablanca'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/4pn2/6B1/3P4/5N2/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'C10q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+ Nxf6 7. Bg5

			variation => N__('French: Rubinstein, 7. Bg5'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/4pn2/8/3P4/3B1N2/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'C10n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+ Nxf6 7. Bd3

			variation => N__('French: Rubinstein, 7. Bd3'),
			moves => {
				'c7c5' => 'r1bqkb1r/pp3ppp/4pn2/2p5/3P4/3B1N2/PPP2PPP/R1BQK2R w KQkq c6 0 8',
			},
		},
		'r1bqkb1r/ppp2ppp/4pn2/8/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'C10l',
                        # TRANSLATORS:

			variation => N__('French: Rubinstein, 5. Nf3 Ngf6 6.Nxf6+'),
			moves => {
				'c1g5' => 'r1bqkb1r/ppp2ppp/4pn2/6B1/3P4/5N2/PPP2PPP/R2QKB1R b KQkq - 1 7',
				'f1d3' => 'r1bqkb1r/ppp2ppp/4pn2/8/3P4/3B1N2/PPP2PPP/R1BQK2R b KQkq - 1 7',
				'f3e5' => 'r1bqkb1r/ppp2ppp/4pn2/4N3/3P4/8/PPP2PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'r1bqkb1r/ppp2ppp/5n2/3Pp1N1/1nB5/8/PPPP1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C57e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nb4

			variation => N__('Two Knights: Kloss Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppp2ppp/5n2/3Pp1N1/2Bn4/2P5/PP1P1PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C57i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz Variation'),
			moves => {
				'b7b5' => 'r1bqkb1r/p1p2ppp/5n2/1p1Pp1N1/2Bn4/2P5/PP1P1PPP/RNBQK2R w KQkq b6 0 7',
			},
		},
		'r1bqkb1r/ppp2ppp/5n2/3Pp1N1/2Bn4/8/PPPP1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C57i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4

			variation => N__('Two Knights: Fritz Variation'),
			moves => {
				'c2c3' => 'r1bqkb1r/ppp2ppp/5n2/3Pp1N1/2Bn4/2P5/PP1P1PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/ppp2ppp/5n2/n2Pp1N1/2B5/3P4/PPP2PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C58b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. d3

			variation => N__('Two Knights: Morphy, Kieseritsky Variation'),
			moves => {
				'h7h6' => 'r1bqkb1r/ppp2pp1/5n1p/n2Pp1N1/2B5/3P4/PPP2PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/ppp2ppp/5n2/n2Pp1N1/2B5/8/PPPP1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C58a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5

			variation => N__('Two Knights: Morphy Variation (5... Na5)'),
			moves => {
				'c4b5' => 'r1bqkb1r/ppp2ppp/5n2/nB1Pp1N1/8/8/PPPP1PPP/RNBQK2R b KQkq - 2 6',
				'd2d3' => 'r1bqkb1r/ppp2ppp/5n2/n2Pp1N1/2B5/3P4/PPP2PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/ppp2ppp/5n2/nB1Pp1N1/8/8/PPPP1PPP/RNBQK2R b KQkq - 2 6' => {
			eco => 'C58d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+

			variation => N__('Two Knights: Morphy, Polerio (6. Bb5+)'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp3ppp/2p2n2/nB1Pp1N1/8/8/PPPP1PPP/RNBQK2R w KQkq - 0 7',
				'c8d7' => 'r2qkb1r/pppb1ppp/5n2/nB1Pp1N1/8/8/PPPP1PPP/RNBQK2R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pppN1ppp/8/3p4/3Pn3/3B4/PPP2PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C43q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7 6. Nxd7

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Nd7 6.Nxd7'),
			moves => {
				'c8d7' => 'r2qkb1r/pppb1ppp/8/3p4/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppn1p1p/3p1np1/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 g6

			variation => N__('Old Indian: 5. g3 g6'),
			moves => {
				'f1g2' => 'r1bqkb1r/pppn1p1p/3p1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppn1p1p/3p1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3 g6 6. Bg2

			variation => N__('Old Indian: 5. g3 g6'),
			moves => {
				'f8g7' => 'r1bqk2r/pppn1pbp/3p1np1/4p3/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqkb1r/pppn1p1p/3p1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A55b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 g6

			variation => N__('Old Indian: 5. e4 g6'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/2n1p3/1B1pP3/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'C04e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. Bb5

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. Bb5'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/1N3N2/PPP2PPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'C04h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. Nb3

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. Nb3'),
			moves => {
				'a7a5' => 'r1bqkb1r/1ppn1ppp/2n1p3/p2pP3/3P4/1N3N2/PPP2PPP/R1BQKB1R w KQkq a6 0 7',
				'f8e7' => 'r1bqk2r/pppnbppp/2n1p3/3pP3/3P4/1N3N2/PPP2PPP/R1BQKB1R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/2P2N2/PP1N1PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'C04d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. c3

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. c3'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'C04c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7

			variation => N__('French: Tarrasch, Guimard, Main Line'),
			moves => {
				'c2c3' => 'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/2P2N2/PP1N1PPP/R1BQKB1R b KQkq - 0 6',
				'd2b3' => 'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/1N3N2/PPP2PPP/R1BQKB1R b KQkq - 2 6',
				'f1b5' => 'r1bqkb1r/pppn1ppp/2n1p3/1B1pP3/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 2 6',
				'f1e2' => 'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/5N2/PPPNBPPP/R1BQK2R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pppn1ppp/2n1p3/3pP3/3P4/5N2/PPPNBPPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'C04f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. Be2

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. Be2'),
			moves => {
				'f7f6' => 'r1bqkb1r/pppn2pp/2n1pp2/3pP3/3P4/5N2/PPPNBPPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppn1ppp/2np4/1B2p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Nd7

			variation => N__('Spanish: Closed Berlin, Chigorin Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/3Pp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Nbd7 4. e4 e5 5. d5

			variation => N__('Old Indian: 3. Nc3 Nbd7 4. e4 e5 5. d5'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p1B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. Bg5

			variation => N__('Old Indian: 5. Bg5'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2pp1n2/4p1B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
				'f8e7' => 'r1bqk2r/pppnbppp/3p1n2/4p1B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 6',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 4 5' => {
			eco => 'C41s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7 5. Bc4

			variation => N__('Philidor: Improved Hanham, 5. Bc4'),
			moves => {
				'f8e7' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 5 6',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A53',
                        # TRANSLATORS:

			variation => N__('Old Indian: Ukranian, 4. e3'),
			moves => {
				'f1d3' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 2 5',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e3

			variation => N__('Old Indian: 5. e3'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7

			variation => N__('Old Indian: 4. Nf3 Nbd7'),
			moves => {
				'c1g5' => 'r1bqkb1r/pppn1ppp/3p1n2/4p1B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5',
				'e2e3' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'e2e4' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5',
				'g2g3' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. g3

			variation => N__('Old Indian: 5. g3'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 6',
				'f8e7' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 6',
				'g7g6' => 'r1bqkb1r/pppn1p1p/3p1np1/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. e3 Nbd7 5. Bd3

			variation => N__('Old Indian: Dus-Khotimirsky'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'A55a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4

			variation => N__('Old Indian: 5. e4'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2pp1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6',
				'f8e7' => 'r1bqk2r/pppnbppp/3p1n2/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 6',
				'g7g6' => 'r1bqkb1r/pppn1p1p/3p1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Nbd7 4. e4 e5 5. Nge2

			variation => N__('Old Indian: 3. Nc3 Nbd7 4. e4 e5 5.Nge2'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq e6 0 5' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Nbd7 4. e4 e5

			variation => N__('Old Indian: 3. Nc3 Nbd7 4. e4 e5'),
			moves => {
				'd4d5' => 'r1bqkb1r/pppn1ppp/3p1n2/3Pp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5',
				'g1e2' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppn1ppp/3p1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 5' => {
			eco => 'C41r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3 Nbd7

			variation => N__('Philidor: Improved Hanham'),
			moves => {
				'f1c4' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2BPP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 4 5',
			},
		},
		'r1bqkb1r/pppn1ppp/4pN2/8/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'C10l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Nxf6+

			variation => N__('French: Rubinstein, 5. Nf3 Ngf6 6.Nxf6+'),
			moves => {
				'd7f6' => 'r1bqkb1r/ppp2ppp/4pn2/8/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. e3

			variation => N__('QGD: 4. Bg5 Nbd7 5. e3'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 6',
				'f8b4' => 'r1bqk2r/pppn1ppp/4pn2/3p2B1/1bPP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7 5. Nf3

			variation => N__('QGD: 4. Bg5 Nbd7 5. Nf3'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq - 4 5' => {
			eco => 'D51',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Nbd7

			variation => N__('QGD: 4. Bg5 Nbd7'),
			moves => {
				'e2e3' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5',
				'g1f3' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/4PN2/PP3PPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'D30v',
                        # TRANSLATORS:

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5 Nbd7'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p2B1/2PP4/4PN2/PP3PPP/RN1QKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 4 5' => {
			eco => 'D30v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 Nbd7

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5 Nbd7'),
			moves => {
				'e2e3' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/4PN2/PP3PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'D03f',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3 Nbd7

			variation => N__('Torre Attack: 3... e6 4. e3 Nbd7'),
			moves => {
				'b1d2' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 2 5',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 2 5' => {
			eco => 'D03f',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... e6 4. e3 Nbd7'),
			moves => {
				'f8e7' => 'r1bqk2r/pppnbppp/4pn2/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQkq - 3 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5' => {
			eco => 'D37c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Nbd7 5. Bf4

			variation => N__('QGD: Westphalia: 4. Nf3 Nbd7 5. Bf4'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'D37b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Nbd7

			variation => N__('QGD: Westphalia without Bg5'),
			moves => {
				'c1f4' => 'r1bqkb1r/pppn1ppp/4pn2/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/6B1/3PN3/5N2/PPP2PPP/R2QKB1R b KQkq - 2 6' => {
			eco => 'C11q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Nbd7 6. Nf3

			variation => N__('French: Burn, 5.Nxe4 Nbd7 6. Nf3'),
			moves => {
				'f8e7' => 'r1bqk2r/pppnbppp/4pn2/6B1/3PN3/5N2/PPP2PPP/R2QKB1R w KQkq - 3 7',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'C11p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Nbd7

			variation => N__('French: Burn, 5.Nxe4 Nbd7'),
			moves => {
				'g1f3' => 'r1bqkb1r/pppn1ppp/4pn2/6B1/3PN3/5N2/PPP2PPP/R2QKB1R b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/2Q5/5NP1/PP1PPPBP/RNB1K2R b KQkq - 0 6' => {
			eco => 'A13w',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4

			variation => N__('English: Neo-Catalan Accepted, 5. Qa4+ Nbd7 6.Qxc4'),
			moves => {
				'a7a6' => 'r1bqkb1r/1ppn1ppp/p3pn2/8/2Q5/5NP1/PP1PPPBP/RNB1K2R w KQkq - 0 7',
				'c7c5' => 'r1bqkb1r/pp1n1ppp/4pn2/2p5/2Q5/5NP1/PP1PPPBP/RNB1K2R w KQkq c6 0 7',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/2QP4/6P1/PP2PPBP/RNB1K1NR b KQkq - 0 6' => {
			eco => 'E03',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Qxc4

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6.Qxc4'),
			moves => {
				'a7a6' => 'r1bqkb1r/1ppn1ppp/p3pn2/8/2QP4/6P1/PP2PPBP/RNB1K1NR w KQkq - 0 7',
				'c7c5' => 'r1bqkb1r/pp1n1ppp/4pn2/2p5/2QP4/6P1/PP2PPBP/RNB1K1NR w KQkq c6 0 7',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6' => {
			eco => 'E04c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Nbd7

			variation => N__('Catalan: Open, 5. Nf3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/3PN3/3B1N2/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'C10k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6 6. Bd3

			variation => N__('French: Rubinstein, 5. Nf3 Ngf6 6. Bd3'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'C10j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3 Ngf6

			variation => N__('French: Rubinstein, 5. Nf3 Ngf6'),
			moves => {
				'e4f6' => 'r1bqkb1r/pppn1ppp/4pN2/8/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'f1d3' => 'r1bqkb1r/pppn1ppp/4pn2/8/3PN3/3B1N2/PPP2PPP/R1BQK2R b KQkq - 4 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1p5/5NP1/PP1PPPBP/RNB1K2R w KQkq - 2 6' => {
			eco => 'A13v',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4 5. Qa4+ Nbd7

			variation => N__('English: Neo-Catalan Accepted, 5. Qa4+ Nbd7'),
			moves => {
				'a4c4' => 'r1bqkb1r/pppn1ppp/4pn2/8/2Q5/5NP1/PP1PPPBP/RNB1K2R b KQkq - 0 6',
				'e1g1' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1p5/5NP1/PP1PPPBP/RNB2RK1 b kq - 3 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1p5/5NP1/PP1PPPBP/RNB2RK1 b kq - 3 6' => {
			eco => 'A13v',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4 5. Qa4+ Nbd7 6. O-O

			variation => N__('English: Neo-Catalan Accepted, 5. Qa4+ Nbd7 6.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6' => {
			eco => 'D23g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ Nbd7 5. Nc3 e6

			variation => N__('QGA: Mannheim, 4...Nbd7 5. Nc3 e6'),
			moves => {
				'e2e4' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1pPP3/2N2N2/PP3PPP/R1B1KB1R b KQkq e3 0 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 2 6' => {
			eco => 'E00v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 dxc4 5. Qa4+ Nbd7

			variation => N__('Catalan: 4. Nf3 dxc4 5. Qa4+ Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 3 6' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7 6. Nf3

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7 6. Nf3'),
			moves => {
				'a7a6' => 'r1bqkb1r/1ppn1ppp/p3pn2/8/Q1pP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/6P1/PP2PPBP/RNB1K1NR w KQkq - 2 6' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Nbd7

			variation => N__('Catalan: Open, 5. Qa4+ Nbd7'),
			moves => {
				'a4c4' => 'r1bqkb1r/pppn1ppp/4pn2/8/2QP4/6P1/PP2PPBP/RNB1K1NR b KQkq - 0 6',
				'g1f3' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 3 6',
			},
		},
		'r1bqkb1r/pppn1ppp/4pn2/8/Q1pPP3/2N2N2/PP3PPP/R1B1KB1R b KQkq e3 0 6' => {
			eco => 'D23h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ Nbd7 5. Nc3 e6 6. e4

			variation => N__('QGA: Mannheim, 4...Nbd7 5. Nc3 e6 6. e4'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D35i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Nbd7 6. e3

			variation => N__('QGD: Exchange, 5. Bg5 Nbd7 6. e3'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/5n2/3p2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D35i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Nbd7 6. Nf3

			variation => N__('QGD: Exchange, 5. Bg5 Nbd7 6. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/5n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 2 6' => {
			eco => 'D35i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Nbd7

			variation => N__('QGD: Exchange, 5. Bg5 Nbd7'),
			moves => {
				'e2e3' => 'r1bqkb1r/pppn1ppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
				'g1f3' => 'r1bqkb1r/pppn1ppp/5n2/3p2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
			},
		},
		'r1bqkb1r/pppn1ppp/5n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D35f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Nbd7 6. Bf4

			variation => N__('QGD: Exchange, 5. Nf3 Nbd7 6. Bf4'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/5n2/3p4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D35e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Nbd7 6. e3

			variation => N__('QGD: Exchange, 5. Nf3 Nbd7 6. e3'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1n1ppp/2p2n2/3p4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppn1ppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D35e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Nbd7

			variation => N__('QGD: Exchange, 5. Nf3 Nbd7'),
			moves => {
				'c1f4' => 'r1bqkb1r/pppn1ppp/5n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
				'e2e3' => 'r1bqkb1r/pppn1ppp/5n2/3p4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppn1ppp/8/3pN3/3Pn3/3B4/PPP1QPPP/RNB1K2R b KQkq - 2 6' => {
			eco => 'C43p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7 6. Qe2

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Nd7 6. Qe2'),
			moves => {
			},
		},
		'r1bqkb1r/pppn1ppp/8/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C43o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Nd7'),
			moves => {
				'd1e2' => 'r1bqkb1r/pppn1ppp/8/3pN3/3Pn3/3B4/PPP1QPPP/RNB1K2R b KQkq - 2 6',
				'e5d7' => 'r1bqkb1r/pppN1ppp/8/3p4/3Pn3/3B4/PPP2PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppn2pp/2n1pp2/3pP3/3P4/5N2/PPPNBPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C04g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3 Nf6 5. e5 Nd7 6. Be2 f6

			variation => N__('French: Tarrasch, Guimard, Main Line, 6. Be2 f6'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpp1p/5np1/3p2B1/3P4/2N2N2/PPP1PPPP/R2QKB1R w KQkq - 0 5' => {
			eco => 'D01p',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Nbd7 4. Nf3 g6

			variation => N__('Richter-Veresov: 3...Nbd7 4. Nf3 g6'),
			moves => {
			},
		},
		'r1bqkb1r/pppnppp1/5n1p/3p2B1/3P4/2N2N2/PPP1PPPP/R2QKB1R w KQkq - 0 5' => {
			eco => 'D01o',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Nbd7 4. Nf3 h6

			variation => N__('Richter-Veresov: 3...Nbd7 4. Nf3 h6'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/3p1n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A46i',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 d6 3. Bg5 Nbd7

			variation => N__('Indian: 2. Nf3 d6 3. Bg5 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/3p1n2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Nbd7 4. Nf3

			variation => N__('Old Indian: 3. Nc3 Nbd7 4. Nf3'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/3p1n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Nbd7

			variation => N__('Old Indian: 3. Nc3 Nbd7'),
			moves => {
				'e2e4' => 'r1bqkb1r/pppnpppp/3p1n2/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
				'g1f3' => 'r1bqkb1r/pppnpppp/3p1n2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'r1bqkb1r/pppnpppp/3p1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3 Nbd7

			variation => N__('Old Indian: 3. Nf3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/3p1n2/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Nbd7 4. e4

			variation => N__('Old Indian: 3. Nc3 Nbd7 4. e4'),
			moves => {
				'e7e5' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq e6 0 5',
			},
		},
		'r1bqkb1r/pppnpppp/3p1n2/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 3 4' => {
			eco => 'B07l',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 Nbd7

			variation => N__('Pirc: 3. Nc3 Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/5n2/1B1P4/8/8/PPPP1PPP/RNBQK1NR w KQkq - 3 4' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+ Nbd7

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Nbd7'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/5n2/3p2B1/3P4/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 5 4' => {
			eco => 'D01n',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Nbd7 4. Nf3

			variation => N__('Richter-Veresov: 3...Nbd7 4. Nf3'),
			moves => {
				'g7g6' => 'r1bqkb1r/pppnpp1p/5np1/3p2B1/3P4/2N2N2/PPP1PPPP/R2QKB1R w KQkq - 0 5',
				'h7h6' => 'r1bqkb1r/pppnppp1/5n1p/3p2B1/3P4/2N2N2/PPP1PPPP/R2QKB1R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppnpppp/5n2/3p2B1/3P4/2N2P2/PPP1P1PP/R2QKBNR b KQkq - 0 4' => {
			eco => 'D01m',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Nbd7 4. f3

			variation => N__('Richter-Veresov: 3...Nbd7 4. f3'),
			moves => {
			},
		},
		'r1bqkb1r/pppnpppp/5n2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 4 4' => {
			eco => 'D01l',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Nbd7

			variation => N__('Richter-Veresov: 3...Nbd7'),
			moves => {
				'f2f3' => 'r1bqkb1r/pppnpppp/5n2/3p2B1/3P4/2N2P2/PPP1P1PP/R2QKBNR b KQkq - 0 4',
				'g1f3' => 'r1bqkb1r/pppnpppp/5n2/3p2B1/3P4/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 5 4',
			},
		},
		'r1bqkb1r/pppnpppp/5n2/8/Q1pP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5' => {
			eco => 'D23f',
                        # TRANSLATORS:

			variation => N__('QGA: Mannheim, 4...Nbd7'),
			moves => {
				'e7e6' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppnpppp/5n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 4 5' => {
			eco => 'D23f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ Nbd7

			variation => N__('QGA: Mannheim, 4...Nbd7'),
			moves => {
				'b1c3' => 'r1bqkb1r/pppnpppp/5n2/8/Q1pP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5',
			},
		},
		'r1bqkb1r/pppp1p1p/2n2np1/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 g6

			variation => N__('English: Four Knights, 4. g3 g6'),
			moves => {
				'd2d4' => 'r1bqkb1r/pppp1p1p/2n2np1/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq d3 0 5',
			},
		},
		'r1bqkb1r/pppp1p1p/2n2np1/4p3/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq d3 0 5' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 g6 5. d4

			variation => N__('English: Four Knights, 4. g3 g6 5. d4'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1p1p/2nn2p1/1N2p2Q/8/1B6/PPPP1PPP/R1B1K1NR w KQkq - 0 7' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'h5f3' => 'r1bqkb1r/pppp1p1p/2nn2p1/1N2p3/8/1B3Q2/PPPP1PPP/R1B1K1NR b KQkq - 1 7',
			},
		},
		'r1bqkb1r/pppp1p1p/2nn2p1/1N2p3/8/1B3Q2/PPPP1PPP/R1B1K1NR b KQkq - 1 7' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'f7f5' => 'r1bqkb1r/pppp3p/2nn2p1/1N2pp2/8/1B3Q2/PPPP1PPP/R1B1K1NR w KQkq f6 0 8',
			},
		},
		'r1bqkb1r/pppp1pp1/2n2n1p/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C55d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3 h6

			variation => N__('Two Knights: 4. d3 h6'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2Bn4/4p3/3P4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C67o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Bxc6

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 6.Bxc6'),
			moves => {
				'd7c6' => 'r1bqkb1r/ppp2ppp/2pn4/4p3/3P4/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqkb1r/pppp1ppp/2N2n2/8/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C45h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6

			variation => N__('Scotch: 4.Nxd4 Nf6 5.Nxc6'),
			moves => {
				'b7c6' => 'r1bqkb1r/p1pp1ppp/2p2n2/8/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n1pn2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 Nc6

			variation => N__('English: Mikenas, Kevitz Defence'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n1pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A50g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6 3. Nf3 e6 4. Nc3

			variation => N__('Indian: Mexican Defence, 3. Nf3 e6 4. Nc3'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n1pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A50e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6 3. Nf3 e6

			variation => N__('Indian: Mexican Defence, 3. Nf3 e6'),
			moves => {
				'a2a3' => 'r1bqkb1r/pppp1ppp/2n1pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 4',
				'b1c3' => 'r1bqkb1r/pppp1ppp/2n1pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n1pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A50f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6 3. Nf3 e6 4. a3

			variation => N__('Indian: Mexican Defence, 3. Nf3 e6 4. a3'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C65c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d4

			variation => N__('Spanish: Berlin, 4. d4'),
			moves => {
				'e5d4' => 'r1bqkb1r/pppp1ppp/2n2n2/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C48a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5

			variation => N__('Four Knights: Spanish Variation'),
			moves => {
				'a7a6' => 'r1bqkb1r/1ppp1ppp/p1n2n2/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5',
				'c6d4' => 'r1bqkb1r/pppp1ppp/5n2/1B2p3/3nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5',
				'd7d6' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5',
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/1B2p3/1b2P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C65f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3

			variation => N__('Spanish: Berlin, 4. d3'),
			moves => {
				'c6e7' => 'r1bqkb1r/ppppnppp/5n2/1B2p3/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5',
				'd7d6' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4' => {
			eco => 'C65k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O

			variation => N__('Spanish: Berlin, 4.O-O'),
			moves => {
				'd7d6' => 'r1bqkb1r/ppp2ppp/2np1n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5',
				'f6e4' => 'r1bqkb1r/pppp1ppp/2n5/1B2p3/4n3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
				'f8e7' => 'r1bqk2r/ppppbppp/2n2n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C65a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6

			variation => N__('Spanish: Berlin Defence'),
			moves => {
				'd1e2' => 'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/5N2/PPPPQPPP/RNB1K2R b KQkq - 5 4',
				'd2d3' => 'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/5N2/PPPPQPPP/RNB1K2R b KQkq - 5 4' => {
			eco => 'C65b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. Qe2

			variation => N__('Spanish: Berlin, 4. Qe2'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B6/3pP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C65e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d4 exd4 5. O-O

			variation => N__('Spanish: Berlin, Nyholm Attack'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C65d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d4 exd4

			variation => N__('Spanish: Berlin, 4. d4 exd4'),
			moves => {
				'e1g1' => 'r1bqkb1r/pppp1ppp/2n2n2/1B6/3pP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/3N4/3pP3/5N2/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C47l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5

			variation => N__('Four Knights: Belgrade Gambit'),
			moves => {
				'c6b4' => 'r1bqkb1r/pppp1ppp/5n2/3N4/1n1pP3/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 6',
				'f6d5' => 'r1bqkb1r/pppp1ppp/2n5/3n4/3pP3/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'f6e4' => 'r1bqkb1r/pppp1ppp/2n5/3N4/3pn3/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'f8e7' => 'r1bqk2r/ppppbppp/2n2n2/3N4/3pP3/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4N3/4P3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'C47a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Nxe5

			variation => N__('Four Knights: Halloween Gambit'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4P3/2Bp4/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C55o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. e5

			variation => N__('Two Knights: Modern Variation'),
			moves => {
				'd7d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3pP3/2Bp4/5N2/PPP2PPP/RNBQK2R w KQkq d6 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p1N1/2B1P3/8/PPPP1PPP/RNBQK2R b KQkq - 5 4' => {
			eco => 'C57a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5

			variation => N__('Two Knights: 4. Ng5'),
			moves => {
				'd7d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp1N1/2B1P3/8/PPPP1PPP/RNBQK2R w KQkq d6 0 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p1N1/2B1P3/8/PPPP1PPP/RNBQK2R w KQkq - 6 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C47d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bc4

			variation => N__('Four Knights: Italian Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4' => {
			eco => 'C28a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6

			variation => N__('Vienna: 3. Bc4 Nc6'),
			moves => {
				'd2d3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR b KQkq - 0 4',
				'f2f4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1PP2/2N5/PPPP2PP/R1BQK1NR b KQkq f3 0 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR b KQkq - 0 4' => {
			eco => 'C28c',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3'),
			moves => {
				'c6a5' => 'r1bqkb1r/pppp1ppp/5n2/n3p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5',
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bB1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5',
				'f8e7' => 'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C55c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d3

			variation => N__('Two Knights: 4. d3'),
			moves => {
				'f8e7' => 'r1bqk2r/ppppbppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5',
				'h7h6' => 'r1bqkb1r/pppp1pp1/2n2n1p/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4' => {
			eco => 'C55a',
                        # TRANSLATORS:

			variation => N__('Two Knights Defence'),
			moves => {
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 6 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C55a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6

			variation => N__('Two Knights Defence'),
			moves => {
				'd2d3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4',
				'f3g5' => 'r1bqkb1r/pppp1ppp/2n2n2/4p1N1/2B1P3/8/PPPP1PPP/RNBQK2R b KQkq - 5 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1PP2/2N5/PPPP2PP/R1BQK1NR b KQkq f3 0 4' => {
			eco => 'C28b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. f4

			variation => N__('Vienna: 3. Bc4 Nc6 4. f4'),
			moves => {
				'f6e4' => 'r1bqkb1r/pppp1ppp/2n5/4p3/2B1nP2/2N5/PPPP2PP/R1BQK1NR w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C55l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4

			variation => N__('Two Knights: 4. d4'),
			moves => {
				'e5d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P1P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 4' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. e4

			variation => N__('English: Four Knights, Nimzowitsch Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N1P3/PP1P1PPP/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. e3 Nf6

			variation => N__('English: Closed'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3

			variation => N__('English: Four Knights, 4. e3'),
			moves => {
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N1PN2/PP1P1PPP/R1BQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6

			variation => N__('English: Four Knights'),
			moves => {
				'a2a3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/P1N2N2/1P1PPPPP/R1BQKB1R b KQkq - 0 4',
				'd2d3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2NP1N2/PP2PPPP/R1BQKB1R b KQkq - 0 4',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4',
				'e2e3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 0 4',
				'e2e4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P1P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 4',
				'g2g3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3

			variation => N__('English: Four Knights, 4. g3'),
			moves => {
				'd7d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq d6 0 5',
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 5',
				'g7g6' => 'r1bqkb1r/pppp1p1p/2n2np1/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 Nf6

			variation => N__('English: Closed'),
			moves => {
				'f1g2' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 Nf6 4. Bg2

			variation => N__('English: Closed'),
			moves => {
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1bP5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 3 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 3 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2NP1N2/PP2PPPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. d3

			variation => N__('English: Four Knights, Capablanca Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 3 4' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nf6 3. Bg2 Nc6

			variation => N__('English: King\'s, 2. g3 Nf6 3. Bg2 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/P1N2N2/1P1PPPPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. a3

			variation => N__('English: Four Knights, Marini Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. d4

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'e5d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'e5e4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'C47i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4

			variation => N__('Four Knights: Scotch Variation'),
			moves => {
				'd7d6' => 'r1bqkb1r/ppp2ppp/2np1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5',
				'e5d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/3pP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5',
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/4p3/1b1PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'C44n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nf6 4. d4

			variation => N__('Ponziani: 3... Nf6'),
			moves => {
				'e5d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/3pP3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'f6e4' => 'r1bqkb1r/pppp1ppp/2n5/4p3/3Pn3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/3PP3/5N2/PPP1BPPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C44c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Be2 Nf6 4. d4

			variation => N__('Open Game: Tayler Opening'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'C47a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6

			variation => N__('Four Knights Game'),
			moves => {
				'a2a3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/P1N2N2/1PPP1PPP/R1BQKB1R b KQkq - 0 4',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq d3 0 4',
				'f1b5' => 'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
				'f1c4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
				'f1e2' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 5 4',
				'f3e5' => 'r1bqkb1r/pppp1ppp/2n2n2/4N3/4P3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 0 4',
				'g2g3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N2NP1/PPPP1P1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C47c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Be2

			variation => N__('Four Knights: Van der Wiel Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N2NP1/PPPP1P1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'C47e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. g3

			variation => N__('Four Knights: Glek Variation'),
			moves => {
				'd7d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/4P3/2N2NP1/PPPP1P1P/R1BQKB1R w KQkq d6 0 5',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N2NP1/PPPP1P1P/R1BQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'C26d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Nc6

			variation => N__('Vienna: Smyslov, 3... Nc6'),
			moves => {
				'f1g2' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'C26d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Nc6 4. Bg2

			variation => N__('Vienna: Smyslov, 3... Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C44n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nf6

			variation => N__('Ponziani: 3... Nf6'),
			moves => {
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq d3 0 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1N2/PPP1BPPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C44c',
                        # TRANSLATORS:

			variation => N__('Open Game: Inverted Hungarian'),
			moves => {
				'd7d5' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/4P3/3P1N2/PPP1BPPP/RNBQK2R w KQkq d6 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1N2/PPP1BPPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'C44g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d3 Nf6 4. Be2

			variation => N__('Open Game: Inverted Philidor, 4. Be2'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C44e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d3 Nf6

			variation => N__('Open Game: Inverted Philidor'),
			moves => {
				'f1e2' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1N2/PPP1BPPP/RNBQK2R b KQkq - 2 4',
				'g2g3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1NP1/PPP2P1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1NP1/PPP2P1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C44f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d3 Nf6 4. g3

			variation => N__('Open Game: Inverted Philidor, 4. g3'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/5N2/PPPPBPPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C44c',
                        # TRANSLATORS:

			variation => N__('Open Game: Inverted Hungarian'),
			moves => {
				'd2d3' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1N2/PPP1BPPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/3PP3/5N2/PPP1BPPP/RNBQK2R b KQkq d3 0 4',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/P1N2N2/1PPP1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'C47b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. a3

			variation => N__('Four Knights: Gunsberg Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/6B1/2BQ4/5N2/PPP2PPP/RN2K2R w KQkq - 3 7' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Nc6

			variation => N__('Bishop\'s Opening: Urusov Gambit, 6. Bg5 Nc6 Defence'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/6N1/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C55n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. Ng5

			variation => N__('Two Knights: Perreux Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2B4Q/2N2N2/PPP2PPP/R1B1K2R b KQkq - 4 7' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Nc3 Nc6 7. Qh4

			variation => N__('Bishop\'s Opening: Urusov Gambit, Marshall Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2BNP3/8/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C55m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. Nxd4

			variation => N__('Two Knights: 4. d4 exd4 5.Nxd4'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2BQ4/2N2N2/PPP2PPP/R1B1K2R w KQkq - 3 7' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'd4h4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2B4Q/2N2N2/PPP2PPP/R1B1K2R b KQkq - 4 7',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2BpP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C55s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O

			variation => N__('Two Knights: 5.O-O'),
			moves => {
				'd7d6' => 'r1bqkb1r/ppp2ppp/2np1n2/8/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6',
				'f6e4' => 'r1bqkb1r/pppp1ppp/2n5/8/2Bpn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b5/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 2 6',
				'f8e7' => 'r1bqk2r/ppppbppp/2n2n2/8/2BpP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C55l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4

			variation => N__('Two Knights: 4. d4 exd4'),
			moves => {
				'e1g1' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2BpP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 1 5',
				'e4e5' => 'r1bqkb1r/pppp1ppp/2n2n2/4P3/2Bp4/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'f3d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2BNP3/8/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'f3g5' => 'r1bqkb1r/pppp1ppp/2n2n2/6N1/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/8/1bPN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. d4 e4

			variation => N__('English: Four Knights, Bradley Beach Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. d4'),
			moves => {
				'f3d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C47o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nxd4

			variation => N__('Four Knights: Scotch, 4...exd4 5.Nxd4'),
			moves => {
				'd7d6' => 'r1bqkb1r/ppp2ppp/2np1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/8/1b1NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 6',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n2n2/2b5/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C45h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6

			variation => N__('Scotch: 4.Nxd4 Nf6'),
			moves => {
				'd4c6' => 'r1bqkb1r/pppp1ppp/2N2n2/8/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/3pP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C47l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4

			variation => N__('Four Knights: Scotch, 4...exd4'),
			moves => {
				'c3d5' => 'r1bqkb1r/pppp1ppp/2n2n2/3N4/3pP3/5N2/PPP2PPP/R1BQKB1R b KQkq - 1 5',
				'f3d4' => 'r1bqkb1r/pppp1ppp/2n2n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/3pP3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C44o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nf6 4. d4 exd4

			variation => N__('Ponziani: 3... Nf6 4. d4 exd4'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/4P3/2N1Q3/PPP2PPP/R1B1KBNR b KQkq - 4 5' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'f8b4' => 'r1bqk2r/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPP2PPP/R1B1KBNR w KQkq - 5 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n2n2/8/4P3/4Q3/PPP2PPP/RNB1KBNR w KQkq - 3 5' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4 Nc6 4. Qe3 Nf6

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'b1c3' => 'r1bqkb1r/pppp1ppp/2n2n2/8/4P3/2N1Q3/PPP2PPP/R1B1KBNR b KQkq - 4 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/1B2p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 5' => {
			eco => 'C67e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4

			variation => N__('Spanish: Open Berlin, 5. d4'),
			moves => {
				'a7a6' => 'r1bqkb1r/1ppp1ppp/p1n5/1B2p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6',
				'e4d6' => 'r1bqkb1r/pppp1ppp/2nn4/1B2p3/3P4/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6',
				'f8e7' => 'r1bqk2r/ppppbppp/2n5/1B2p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/1B2p3/4n3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5' => {
			eco => 'C67a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4

			variation => N__('Spanish: Open Berlin'),
			moves => {
				'd1e2' => 'r1bqkb1r/pppp1ppp/2n5/1B2p3/4n3/5N2/PPPPQPPP/RNB2RK1 b kq - 1 5',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2n5/1B2p3/3Pn3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 5',
				'f1e1' => 'r1bqkb1r/pppp1ppp/2n5/1B2p3/4n3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/1B2p3/4n3/5N2/PPPP1PPP/RNBQR1K1 b kq - 1 5' => {
			eco => 'C67c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. Re1

			variation => N__('Spanish: Open Berlin, 5. Re1'),
			moves => {
				'e4d6' => 'r1bqkb1r/pppp1ppp/2nn4/1B2p3/8/5N2/PPPP1PPP/RNBQR1K1 w kq - 2 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/1B2p3/4n3/5N2/PPPPQPPP/RNB2RK1 b kq - 1 5' => {
			eco => 'C67b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. Qe2

			variation => N__('Spanish: Open Berlin, 5. Qe2'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/2n1P3/3N4/8/PPP1QPPP/RNB1KB1R w KQkq - 1 7' => {
			eco => 'C43c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4 5. Qe2 Nc5 6. Nxd4 Nc6

			variation => N__('Russian Game: Modern Attack, Bardeleben Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/3N4/2Bpn3/5N2/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C47m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5 Nxe4 6. Bc4

			variation => N__('Four Knights: Belgrade Gambit, 5...Nxe4 6. Bc4 (Modern Line)'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/3N4/3pn3/5N2/PPP1QPPP/R1B1KB1R b KQkq - 1 6' => {
			eco => 'C47m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5 Nxe4 6. Qe2

			variation => N__('Four Knights: Belgrade Gambit, 5...Nxe4 6. Qe2 (Old Line)'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/3N4/3pn3/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C47m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5 Nxe4

			variation => N__('Four Knights: Belgrade Gambit, 5...Nxe4'),
			moves => {
				'd1e2' => 'r1bqkb1r/pppp1ppp/2n5/3N4/3pn3/5N2/PPP1QPPP/R1B1KB1R b KQkq - 1 6',
				'f1c4' => 'r1bqkb1r/pppp1ppp/2n5/3N4/2Bpn3/5N2/PPP2PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/3Pp3/4n3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C44p',
                        # TRANSLATORS:

			variation => N__('Ponziani: 3... Nf6 4. d4 Nxe4'),
			moves => {
				'c6e7' => 'r1bqkb1r/ppppnppp/8/3Pp3/4n3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 1 6',
				'f8c5' => 'r1bqk2r/pppp1ppp/2n5/2bPp3/4n3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/3Pp3/4n3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C43g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 Nc6 5. d5

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 Nc6 5. d5'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/3n4/3pP3/5N2/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C47l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5 Nxd5

			variation => N__('Four Knights: Belgrade Gambit, 5...Nxd5'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4P3/2P1n3/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 5' => {
			eco => 'A51j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. Nf3 Nc6

			variation => N__('Budapest: Fajarowicz, 4. Nf3 Nc6'),
			moves => {
				'a2a3' => 'r1bqkb1r/pppp1ppp/2n5/4P3/2P1n3/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4P3/2P1n3/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A51k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. Nf3 Nc6 5. a3

			variation => N__('Budapest: Fajarowicz, 4. Nf3 Nc6 5. a3'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4P3/2P2Bn1/5N2/PP2PPPP/RN1QKB1R b KQkq - 4 5' => {
			eco => 'A52j',
                        # TRANSLATORS:

			variation => N__('Budapest: Rubinstein, 4... Nc6'),
			moves => {
				'f8b4' => 'r1bqk2r/pppp1ppp/2n5/4P3/1bP2Bn1/5N2/PP2PPPP/RN1QKB1R w KQkq - 5 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4P3/2P2Bn1/8/PP2PPPP/RN1QKBNR w KQkq - 3 5' => {
			eco => 'A52j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Bf4 Nc6

			variation => N__('Budapest: Rubinstein, 4... Nc6'),
			moves => {
				'g1f3' => 'r1bqkb1r/pppp1ppp/2n5/4P3/2P2Bn1/5N2/PP2PPPP/RN1QKB1R b KQkq - 4 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4p3/2B1nP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C28b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. f4 Nxe4 5. Nf3

			variation => N__('Vienna: Bronstein Gambit'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4p3/2B1nP2/2N5/PPPP2PP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'C28b',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nc6 4. f4'),
			moves => {
				'g1f3' => 'r1bqkb1r/pppp1ppp/2n5/4p3/2B1nP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 1 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4p3/3PB3/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C43h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 Nc6 5. Bxe4

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 Nc6 5.Bxe4'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4p3/3Pn3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C44p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nf6 4. d4 Nxe4

			variation => N__('Ponziani: 3... Nf6 4. d4 Nxe4'),
			moves => {
				'd4d5' => 'r1bqkb1r/pppp1ppp/2n5/3Pp3/4n3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/4p3/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C43g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 Nc6

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 Nc6'),
			moves => {
				'd3e4' => 'r1bqkb1r/pppp1ppp/2n5/4p3/3PB3/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'd4d5' => 'r1bqkb1r/pppp1ppp/2n5/3Pp3/4n3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/8/2Bpn3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'C56a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Nc3

			variation => N__('Two Knights: Classical, Nakhmanson Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/8/2Bpn3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C56a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4

			variation => N__('Two Knights: Classical'),
			moves => {
				'b1c3' => 'r1bqkb1r/pppp1ppp/2n5/8/2Bpn3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 1 6',
				'f1e1' => 'r1bqkb1r/pppp1ppp/2n5/8/2Bpn3/5N2/PPP2PPP/RNBQR1K1 b kq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2n5/8/2Bpn3/5N2/PPP2PPP/RNBQR1K1 b kq - 1 6' => {
			eco => 'C56b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1

			variation => N__('Two Knights: Classical, 6. Re1'),
			moves => {
				'd7d5' => 'r1bqkb1r/ppp2ppp/2n5/3p4/2Bpn3/5N2/PPP2PPP/RNBQR1K1 w kq d6 0 7',
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/1B2N3/8/8/PPPP1PPP/RNBQR1K1 b kq - 0 6' => {
			eco => 'C67d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. Re1 Nd6 6. Nxe5

			variation => N__('Spanish: Open Berlin, 5. Re1 Nd6 6.Nxe5'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/1B2P3/8/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C67n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. dxe5

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6 6.dxe5'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/1B2p3/3P4/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'C67m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6

			variation => N__('Spanish: Open Berlin, 5. d4 Nd6'),
			moves => {
				'b5a4' => 'r1bqkb1r/pppp1ppp/2nn4/4p3/B2P4/5N2/PPP2PPP/RNBQ1RK1 b kq - 2 6',
				'b5c6' => 'r1bqkb1r/pppp1ppp/2Bn4/4p3/3P4/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
				'd4e5' => 'r1bqkb1r/pppp1ppp/2nn4/1B2P3/8/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/1B2p3/8/5N2/PPPP1PPP/RNBQR1K1 w kq - 2 6' => {
			eco => 'C67c',
                        # TRANSLATORS:

			variation => N__('Spanish: Open Berlin, 5. Re1'),
			moves => {
				'f3e5' => 'r1bqkb1r/pppp1ppp/2nn4/1B2N3/8/8/PPPP1PPP/RNBQR1K1 b kq - 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/1N2p2Q/8/1B6/PPPP1PPP/R1B1K1NR b KQkq - 5 6' => {
			eco => 'C27h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6 6. Nb5

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'g7g6' => 'r1bqkb1r/pppp1p1p/2nn2p1/1N2p2Q/8/1B6/PPPP1PPP/R1B1K1NR w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/4p2Q/3P4/1BN5/PPP2PPP/R1B1K1NR b KQkq d3 0 6' => {
			eco => 'C27h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6 6. d4

			variation => N__('Vienna: Adams\' Gambit'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/4p2Q/8/1BN5/PPPP1PPP/R1B1K1NR w KQkq - 4 6' => {
			eco => 'C27h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'c3b5' => 'r1bqkb1r/pppp1ppp/2nn4/1N2p2Q/8/1B6/PPPP1PPP/R1B1K1NR b KQkq - 5 6',
				'd2d4' => 'r1bqkb1r/pppp1ppp/2nn4/4p2Q/3P4/1BN5/PPP2PPP/R1B1K1NR b KQkq d3 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/2nn4/4p3/B2P4/5N2/PPP2PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'C67m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O Nxe4 5. d4 Nd6 6. Ba4

			variation => N__('Spanish: Open Berlin, Showalter Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5P2/1B6/8/2p5/PPPP1PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'C48k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Nxd4 exd4 6. e5 dxc3 7. exf6

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'd8f6' => 'r1b1kb1r/pppp1ppp/5q2/1B6/8/2p5/PPPP1PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B2N3/3nP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 5' => {
			eco => 'C48h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Nxe5

			variation => N__('Four Knights: Rubinstein, 5.Nxe5'),
			moves => {
				'd8e7' => 'r1b1kb1r/ppppqppp/5n2/1B2N3/3nP3/2N5/PPPP1PPP/R1BQK2R w KQkq - 1 6',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B2P3/3p4/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C48k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'd4c3' => 'r1bqkb1r/pppp1ppp/5n2/1B2P3/8/2p5/PPPP1PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B2P3/8/2p5/PPPP1PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C48k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'e5f6' => 'r1bqkb1r/pppp1ppp/5P2/1B6/8/2p5/PPPP1PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B2p3/3NP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 5' => {
			eco => 'C48k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Nxd4

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'e5d4' => 'r1bqkb1r/pppp1ppp/5n2/1B6/3pP3/2N5/PPPP1PPP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B2p3/3nP3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5' => {
			eco => 'C48i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. O-O

			variation => N__('Four Knights: Rubinstein, Henneberger Variation'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B2p3/3nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 6 5' => {
			eco => 'C48g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4

			variation => N__('Four Knights: Rubinstein Countergambit'),
			moves => {
				'b5a4' => 'r1bqkb1r/pppp1ppp/5n2/4p3/B2nP3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 7 5',
				'b5c4' => 'r1bqkb1r/pppp1ppp/5n2/4p3/2BnP3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 7 5',
				'b5e2' => 'r1bqkb1r/pppp1ppp/5n2/4p3/3nP3/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 7 5',
				'e1g1' => 'r1bqkb1r/pppp1ppp/5n2/1B2p3/3nP3/2N2N2/PPPP1PPP/R1BQ1RK1 b kq - 7 5',
				'f3d4' => 'r1bqkb1r/pppp1ppp/5n2/1B2p3/3NP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 5',
				'f3e5' => 'r1bqkb1r/pppp1ppp/5n2/1B2N3/3nP3/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/1B6/3pP3/2N5/PPPP1PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C48k',
                        # TRANSLATORS:

			variation => N__('Four Knights: Rubinstein, Exchange Variation'),
			moves => {
				'e4e5' => 'r1bqkb1r/pppp1ppp/5n2/1B2P3/3p4/2N5/PPPP1PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/3N4/1n1pP3/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C47l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. d4 exd4 5. Nd5 Nb4

			variation => N__('Four Knights: Belgrade Gambit, 5... Nb4'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/4p3/2BnP3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 7 5' => {
			eco => 'C48j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Bc4

			variation => N__('Four Knights: Rubinstein, 5. Bc4'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/4p3/3nP3/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 7 5' => {
			eco => 'C48g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Be2

			variation => N__('Four Knights: Rubinstein, 5. Be2'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/4p3/B2nP3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 7 5' => {
			eco => 'C48n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Ba4

			variation => N__('Four Knights: Rubinstein, 5. Ba4'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1p1ppp/2p2n2/4p3/B2nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 6',
				'd4f3' => 'r1bqkb1r/pppp1ppp/5n2/4p3/B3P3/2N2n2/PPPP1PPP/R1BQK2R w KQkq - 0 6',
				'f8c5' => 'r1bqk2r/pppp1ppp/5n2/2b1p3/B2nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 8 6',
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/4p3/B3P3/2N2n2/PPPP1PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C48n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Nd4 5. Ba4 Nxf3+

			variation => N__('Four Knights: Rubinstein, 5. Ba4 Nxf3+'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/n3p3/2B1P3/2NP4/PPP1NPPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'C28f',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Na5 5. Nge2

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Na5 5.Nge2'),
			moves => {
			},
		},
		'r1bqkb1r/pppp1ppp/5n2/n3p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 1 5' => {
			eco => 'C28e',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Na5

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Na5'),
			moves => {
				'g1e2' => 'r1bqkb1r/pppp1ppp/5n2/n3p3/2B1P3/2NP4/PPP1NPPP/R1BQK2R b KQkq - 2 5',
			},
		},
		'r1bqkb1r/pppp2pp/2n2N2/1B2p3/8/5N2/PPPP1PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C63q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 Nf6 6. Nxf6+

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 Nf6 6.Nxf6+'),
			moves => {
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4N3/5N2/PPPP1PPP/R1BQK2R w KQkq - 1 6' => {
			eco => 'C63o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 Nf6

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 Nf6'),
			moves => {
				'd1e2' => 'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4N3/5N2/PPPPQPPP/R1B1K2R b KQkq - 2 6',
				'e4f6' => 'r1bqkb1r/pppp2pp/2n2N2/1B2p3/8/5N2/PPPP1PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4N3/5N2/PPPPQPPP/R1B1K2R b KQkq - 2 6' => {
			eco => 'C63p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 Nf6 6. Qe2

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 Nf6 6. Qe2'),
			moves => {
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4P3/5N2/PPP2PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'C63f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. d3 fxe4 5. dxe4 Nf6 6. O-O

			variation => N__('Spanish: Schliemann, 4. d3 fxe4 5.dxe4 Nf6 6.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4P3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C63e',
                        # TRANSLATORS:

			variation => N__('Spanish: Schliemann, 4. d3 fxe4'),
			moves => {
				'e1g1' => 'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4P3/5N2/PPP2PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/1B2pP2/8/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 0 5' => {
			eco => 'C63m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 Nf6 5. exf5

			variation => N__('Spanish: Schliemann, 4. Nc3 Nf6 5.exf5'),
			moves => {
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/1B2pp2/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 2 5' => {
			eco => 'C63l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 Nf6

			variation => N__('Spanish: Schliemann, 4. Nc3 Nf6'),
			moves => {
				'e4f5' => 'r1bqkb1r/pppp2pp/2n2n2/1B2pP2/8/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/4NP2/4pP2/8/PPPP2PP/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C30i',
                        # TRANSLATORS: 1. e4 e5 2. f4 Nc6 3. Nf3 f5 4. exf5 e4 5. Ne5 Nf6

			variation => N__('King\'s Gambit: Adelaide-Wahls, 4.exf5 e4 5. Ne5 Nf6'),
			moves => {
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/4pp2/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A25',
                        # TRANSLATORS:

			variation => N__('English: Closed'),
			moves => {
				'g7g6' => 'r1bqkb1r/pppp3p/2n2np1/4pp2/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/4pp2/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 f5 4. Bg2 Nf6

			variation => N__('English: Closed'),
			moves => {
				'd2d3' => 'r1bqkb1r/pppp2pp/2n2n2/4pp2/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5',
				'e2e3' => 'r1bqkb1r/pppp2pp/2n2n2/4pp2/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'r1bqkb1r/pppp2pp/2n2n2/4pp2/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 f5 4. Bg2 Nf6 5. d3

			variation => N__('English: Closed'),
			moves => {
			},
		},
		'r1bqkb1r/pppp3p/2n2np1/4pp2/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 f5 4. Bg2 Nf6 5. e3 g6

			variation => N__('English: Closed'),
			moves => {
			},
		},
		'r1bqkb1r/pppp3p/2nn2p1/1N1Qpp2/8/1B6/PPPP1PPP/R1B1K1NR b KQkq - 1 8' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'd8e7' => 'r1b1kb1r/ppppq2p/2nn2p1/1N1Qpp2/8/1B6/PPPP1PPP/R1B1K1NR w KQkq - 2 9',
			},
		},
		'r1bqkb1r/pppp3p/2nn2p1/1N2pp2/8/1B3Q2/PPPP1PPP/R1B1K1NR w KQkq f6 0 8' => {
			eco => 'C27h',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Nc6'),
			moves => {
				'f3d5' => 'r1bqkb1r/pppp3p/2nn2p1/1N1Qpp2/8/1B6/PPPP1PPP/R1B1K1NR b KQkq - 1 8',
			},
		},
		'r1bqkb1r/ppppnp1p/2n3p1/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5' => {
			eco => 'C60q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7 4. Nc3 g6

			variation => N__('Spanish: Cozio, Paulsen Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppppnp1p/2n3p1/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'C60u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7 4. O-O g6 5. c3

			variation => N__('Spanish: Cozio, 4.O-O g6'),
			moves => {
			},
		},
		'r1bqkb1r/ppppnp1p/2n3p1/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5' => {
			eco => 'C60t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7 4. O-O g6

			variation => N__('Spanish: Cozio, 4.O-O g6'),
			moves => {
				'c2c3' => 'r1bqkb1r/ppppnp1p/2n3p1/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C60p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7 4. Nc3

			variation => N__('Spanish: Cozio, 4. Nc3'),
			moves => {
				'g7g6' => 'r1bqkb1r/ppppnp1p/2n3p1/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C60r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7 4. c3

			variation => N__('Spanish: Cozio, 4.O-O'),
			moves => {
			},
		},
		'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4' => {
			eco => 'C60s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7 4. O-O

			variation => N__('Spanish: Cozio, 4.O-O'),
			moves => {
				'g7g6' => 'r1bqkb1r/ppppnp1p/2n3p1/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 5',
			},
		},
		'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C60o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nge7

			variation => N__('Spanish: Cozio Defence'),
			moves => {
				'b1c3' => 'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
				'c2c3' => 'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
				'e1g1' => 'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 5 4',
			},
		},
		'r1bqkb1r/ppppnppp/2n5/4P3/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 3 4' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 Nc6 3. Nf3 Nge7

			variation => N__('Englund Gambit: Zilbermints'),
			moves => {
			},
		},
		'r1bqkb1r/ppppnppp/2n5/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C44i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nge7

			variation => N__('Ponziani: Reti Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppppnppp/5n2/1B2N3/4P3/3P4/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C65g',
                        # TRANSLATORS:

			variation => N__('Spanish: Berlin, Mortimer Variation'),
			moves => {
				'c7c6' => 'r1bqkb1r/pp1pnppp/2p2n2/1B2N3/4P3/3P4/PPP2PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkb1r/ppppnppp/5n2/1B2p3/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C65g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. d3 Ne7

			variation => N__('Spanish: Berlin, Mortimer Variation'),
			moves => {
				'f3e5' => 'r1bqkb1r/ppppnppp/5n2/1B2N3/4P3/3P4/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppppnppp/8/1B6/3pP3/8/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C61f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O Ne7

			variation => N__('Spanish: Bird\'s, Paulsen Variation'),
			moves => {
			},
		},
		'r1bqkb1r/ppppnppp/8/3Pp3/4n3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'C44q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 Nf6 4. d4 Nxe4 5. d5 Ne7

			variation => N__('Ponziani: 3... Nf6 4. d4 Nxe4 5. d5 Ne7'),
			moves => {
			},
		},
		'r1bqkb1r/ppppp1pp/2n2n2/3P2B1/4p3/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'A83l',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 Nc6 5. d5

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 Nc6 5. d5'),
			moves => {
				'c6e5' => 'r1bqkb1r/ppppp1pp/5n2/3Pn1B1/4p3/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6',
			},
		},
		'r1bqkb1r/ppppp1pp/2n2n2/6B1/3Pp3/2N2P2/PPP3PP/R2QKBNR b KQkq - 0 5' => {
			eco => 'A83k',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 Nc6 5. f3

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 Nc6 5. f3'),
			moves => {
			},
		},
		'r1bqkb1r/ppppp1pp/2n2n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 4 5' => {
			eco => 'A83j',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 Nc6

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 Nc6'),
			moves => {
				'd4d5' => 'r1bqkb1r/ppppp1pp/2n2n2/3P2B1/4p3/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5',
				'f2f3' => 'r1bqkb1r/ppppp1pp/2n2n2/6B1/3Pp3/2N2P2/PPP3PP/R2QKBNR b KQkq - 0 5',
			},
		},
		'r1bqkb1r/ppppp1pp/2n2n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3 Nc6

			variation => N__('Dutch: Staunton Gambit, 4. f3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/ppppp1pp/5n2/3Pn1B1/3Qp3/2N5/PPP2PPP/R3KBNR b KQkq - 2 6' => {
			eco => 'A83m',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 Nc6 5. d5 Ne5 6. Qd4

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 Nc6 5. d5 Ne5 6. Qd4'),
			moves => {
			},
		},
		'r1bqkb1r/ppppp1pp/5n2/3Pn1B1/4p3/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'A83l',
                        # TRANSLATORS:

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 Nc6 5. d5'),
			moves => {
				'd1d4' => 'r1bqkb1r/ppppp1pp/5n2/3Pn1B1/3Qp3/2N5/PPP2PPP/R3KBNR b KQkq - 2 6',
			},
		},
		'r1bqkb1r/pppppppp/2n2n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 3 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 Nc6

			variation => N__('English: Anglo-Indian, 2. Nc3 Nc6'),
			moves => {
			},
		},
		'r1bqkb1r/pppppppp/2n2n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'A50c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6 3. Nc3

			variation => N__('Indian: Mexican Defence, 3. Nc3'),
			moves => {
			},
		},
		'r1bqkb1r/pppppppp/2n2n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'A50d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6 3. Nf3

			variation => N__('Indian: Mexican Defence, 3. Nf3'),
			moves => {
				'd7d6' => 'r1bqkb1r/ppp1pppp/2np1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'e7e6' => 'r1bqkb1r/pppp1ppp/2n1pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'r1bqkb1r/pppppppp/2n2n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A50b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 Nc6

			variation => N__('Indian: Mexican Defence (Two Knights Tango)'),
			moves => {
				'b1c3' => 'r1bqkb1r/pppppppp/2n2n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
				'g1f3' => 'r1bqkb1r/pppppppp/2n2n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkb1r/pppppppp/2n2n2/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 3 3' => {
			eco => 'B00m',
                        # TRANSLATORS: 1. e4 Nc6 2. Nc3 Nf6

			variation => N__('Nimzowitsch Defence: 2. Nc3 Nf6'),
			moves => {
			},
		},
		'r1bqkbnr/1p1p1ppp/p1N1p3/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B46c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Nxc6

			variation => N__('Sicilian: Taimanov, 5... a6 6.Nxc6'),
			moves => {
				'b7c6' => 'r1bqkbnr/3p1ppp/p1p1p3/8/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/2B1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B21x',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 e6 6. Bc4 a6

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 Nc6 5. Nf3 e6 6. Bc4 a6'),
			moves => {
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'B46i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be3

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be3'),
			moves => {
				'd7d6' => 'r1bqkbnr/1p3ppp/p1npp3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7',
				'g8e7' => 'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7',
				'g8f6' => 'r1bqkb1r/1p1p1ppp/p1n1pn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B46f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. g3

			variation => N__('Sicilian: Taimanov, 5... a6 6. g3'),
			moves => {
				'd7d6' => 'r1bqkbnr/1p3ppp/p1npp3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7',
				'g8e7' => 'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B46m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be2

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be2'),
			moves => {
				'd7d6' => 'r1bqkbnr/1p3ppp/p1npp3/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7',
				'g8e7' => 'r1bqkb1r/1p1pnppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
				'g8f6' => 'r1bqkb1r/1p1p1ppp/p1n1pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B46a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6

			variation => N__('Sicilian: Taimanov, 5... a6'),
			moves => {
				'c1e3' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6',
				'd4c6' => 'r1bqkbnr/1p1p1ppp/p1N1p3/8/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'f1e2' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6',
				'f2f4' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6',
				'g2g3' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B42e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nc6

			variation => N__('Sicilian: Kan, 5. Bd3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6' => {
			eco => 'B46b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. f4

			variation => N__('Sicilian: Taimanov, 5... a6 6. f4'),
			moves => {
			},
		},
		'r1bqkbnr/1p1p1ppp/p1n5/1N2p3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B32k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 a6

			variation => N__('Sicilian: Lowenthal, 5... a6'),
			moves => {
				'b5d6' => 'r1bqkbnr/1p1p1ppp/p1nN4/4p3/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/1p1p1ppp/p1nN4/4p3/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'B32k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, 5... a6'),
			moves => {
				'f8d6' => 'r1bqk1nr/1p1p1ppp/p1nb4/4p3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/1p2pp1p/p1np2p1/8/2P1P3/5N2/PP1Q1PPP/RNB1KB1R w KQkq - 0 7' => {
			eco => 'B53j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 a6 5. c4 Nc6 6. Qd2 g6

			variation => N__('Sicilian, Chekhover, 4... a6 5. c4 Nc6 6. Qd2 g6'),
			moves => {
			},
		},
		'r1bqkbnr/1p2pppp/p1np4/8/2B1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B21y',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4 a6

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 a6'),
			moves => {
			},
		},
		'r1bqkbnr/1p2pppp/p1np4/8/2P1P3/5N2/PP1Q1PPP/RNB1KB1R b KQkq - 2 6' => {
			eco => 'B53i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 a6 5. c4 Nc6 6. Qd2

			variation => N__('Sicilian, Chekhover, 4... a6 5. c4 Nc6 6. Qd2'),
			moves => {
				'g7g6' => 'r1bqkbnr/1p2pp1p/p1np2p1/8/2P1P3/5N2/PP1Q1PPP/RNB1KB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/1p2pppp/p1np4/8/2PQP3/5N2/PP3PPP/RNB1KB1R w KQkq - 1 6' => {
			eco => 'B53h',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 4... a6 5. c4'),
			moves => {
				'd4d2' => 'r1bqkbnr/1p2pppp/p1np4/8/2P1P3/5N2/PP1Q1PPP/RNB1KB1R b KQkq - 2 6',
			},
		},
		'r1bqkbnr/1p3ppp/p1np4/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B32p',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6.N1c3'),
			moves => {
				'b5a3' => 'r1bqkbnr/1p3ppp/p1np4/4p3/4P3/N1N5/PPP2PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'r1bqkbnr/1p3ppp/p1np4/1N2p3/4P3/4B3/PPP2PPP/RN1QKB1R w KQkq - 0 8' => {
			eco => 'B44m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. Bf4 e5 7. Be3 a6

			variation => N__('Sicilian, Taimanov, Szen, 6. Bf4 e5 7. Be3 a6'),
			moves => {
			},
		},
		'r1bqkbnr/1p3ppp/p1np4/4p3/4P3/N1N5/PPP2PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B32p',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6.N1c3'),
			moves => {
				'b7b5' => 'r1bqkbnr/5ppp/p1np4/1p2p3/4P3/N1N5/PPP2PPP/R1BQKB1R w KQkq b6 0 8',
			},
		},
		'r1bqkbnr/1p3ppp/p1npp3/1N6/2P1P3/8/PP3PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B44o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 a6

			variation => N__('Sicilian, Taimanov, Szen, 6. c4 a6'),
			moves => {
			},
		},
		'r1bqkbnr/1p3ppp/p1npp3/8/2B1P3/2N2N2/PP2QPPP/R1B2RK1 b kq - 1 8' => {
			eco => 'B21y',
                        # TRANSLATORS:

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O'),
			moves => {
				'b7b5' => 'r1bqkbnr/5ppp/p1npp3/1p6/2B1P3/2N2N2/PP2QPPP/R1B2RK1 w kq b6 0 9',
			},
		},
		'r1bqkbnr/1p3ppp/p1npp3/8/2B1P3/2N2N2/PP3PPP/R1BQ1RK1 w kq - 0 8' => {
			eco => 'B21y',
                        # TRANSLATORS:

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O'),
			moves => {
				'd1e2' => 'r1bqkbnr/1p3ppp/p1npp3/8/2B1P3/2N2N2/PP2QPPP/R1B2RK1 b kq - 1 8',
			},
		},
		'r1bqkbnr/1p3ppp/p1npp3/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B46j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be3 d6

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be3 d6'),
			moves => {
			},
		},
		'r1bqkbnr/1p3ppp/p1npp3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B46h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. g3 d6

			variation => N__('Sicilian: Taimanov, 5... a6 6. g3 d6'),
			moves => {
			},
		},
		'r1bqkbnr/1p3ppp/p1npp3/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B46r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Be2 d6

			variation => N__('Sicilian: Taimanov, 5... a6 6. Be2 d6'),
			moves => {
			},
		},
		'r1bqkbnr/1pp2p1p/p1np2p1/4p3/B3P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C74',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 g6

			variation => N__('Spanish: Modern Steinitz, 5. c3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/1pp2ppp/2n5/p3P3/2Pp4/P4N2/1P2PPPP/RNBQKB1R w KQkq a6 0 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. a3 a5

			variation => N__('QGD: Albin, 4. Nf3 Nc6 5. a3 a5'),
			moves => {
			},
		},
		'r1bqkbnr/1pp2ppp/p1Bp4/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C73',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. Bxc6+

			variation => N__('Spanish: Modern Steinitz, Richter Variation'),
			moves => {
				'b7c6' => 'r1bqkbnr/2p2ppp/p1pp4/4p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1np4/4p3/B1P1P3/5N2/PP1P1PPP/RNBQK2R b KQkq c3 0 5' => {
			eco => 'C71',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c4

			variation => N__('Spanish: Modern Steinitz, Keres Variation'),
			moves => {
			},
		},
		'r1bqkbnr/1pp2ppp/p1np4/4p3/B2PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C71',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. d4

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'b7b5' => 'r1bqkbnr/2p2ppp/p1np4/1p2p3/B2PP3/5N2/PPP2PPP/RNBQK2R w KQkq b6 0 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C71',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. Nc3

			variation => N__('Spanish: Modern Steinitz, Three Knights Variation'),
			moves => {
			},
		},
		'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C74',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3

			variation => N__('Spanish: Modern Steinitz, 5. c3'),
			moves => {
				'c8d7' => 'r2qkbnr/1ppb1ppp/p1np4/4p3/B3P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 6',
				'f7f5' => 'r1bqkbnr/1pp3pp/p1np4/4pp2/B3P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq f6 0 6',
				'g7g6' => 'r1bqkbnr/1pp2p1p/p1np2p1/4p3/B3P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C72',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. O-O

			variation => N__('Spanish: Modern Steinitz, 5.O-O'),
			moves => {
				'c8d7' => 'r2qkbnr/1ppb1ppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
				'c8g4' => 'r2qkbnr/1pp2ppp/p1np4/4p3/B3P1b1/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
				'g8e7' => 'r1bqkb1r/1pp1nppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C71',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6

			variation => N__('Spanish: Modern Steinitz Defence'),
			moves => {
				'a4c6' => 'r1bqkbnr/1pp2ppp/p1Bp4/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 5',
				'b1c3' => 'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 5',
				'c2c3' => 'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 5',
				'c2c4' => 'r1bqkbnr/1pp2ppp/p1np4/4p3/B1P1P3/5N2/PP1P1PPP/RNBQK2R b KQkq c3 0 5',
				'd2d4' => 'r1bqkbnr/1pp2ppp/p1np4/4p3/B2PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5',
				'e1g1' => 'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'r1bqkbnr/1pp2ppp/p1p5/4p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C68e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. d4

			variation => N__('Spanish: Exchange, Lasker Variation'),
			moves => {
				'e5d4' => 'r1bqkbnr/1pp2ppp/p1p5/8/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1p5/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C68g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. Nc3

			variation => N__('Spanish: Exchange, Keres Variation'),
			moves => {
				'f7f6' => 'r1bqkbnr/1pp3pp/p1p2p2/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C68j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O

			variation => N__('Spanish: Exchange, 5.O-O'),
			moves => {
				'c8g4' => 'r2qkbnr/1pp2ppp/p1p5/4p3/4P1b1/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
				'd8d6' => 'r1b1kbnr/1pp2ppp/p1pq4/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
				'f7f6' => 'r1bqkbnr/1pp3pp/p1p2p2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 6',
				'f8d6' => 'r1bqk1nr/1pp2ppp/p1pb4/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
				'g8e7' => 'r1bqkb1r/1pp1nppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'r1bqkbnr/1pp2ppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C68d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6

			variation => N__('Spanish: Exchange, 4...dxc6'),
			moves => {
				'b1c3' => 'r1bqkbnr/1pp2ppp/p1p5/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 5',
				'd2d4' => 'r1bqkbnr/1pp2ppp/p1p5/4p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5',
				'e1g1' => 'r1bqkbnr/1pp2ppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'r1bqkbnr/1pp2ppp/p1p5/8/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 6' => {
			eco => 'C68e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Lasker Variation'),
			moves => {
				'd8d4' => 'r1b1kbnr/1pp2ppp/p1p5/8/3qP3/5N2/PPP2PPP/RNB1K2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/1pp2ppp/p1p5/8/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C68e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Lasker Variation'),
			moves => {
				'd1d4' => 'r1bqkbnr/1pp2ppp/p1p5/8/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/1pp3pp/p1np4/4pP2/B7/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C74',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 f5 6. exf5

			variation => N__('Spanish: Modern Steinitz, Siesta, 6.exf5'),
			moves => {
				'c8f5' => 'r2qkbnr/1pp3pp/p1np4/4pb2/B7/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/1pp3pp/p1np4/4pp2/B3P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq f6 0 6' => {
			eco => 'C74',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 f5

			variation => N__('Spanish: Modern Steinitz, Siesta Variation'),
			moves => {
				'e4f5' => 'r1bqkbnr/1pp3pp/p1np4/4pP2/B7/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/4p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq d3 0 6' => {
			eco => 'C68i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. Nc3 f6 6. d4

			variation => N__('Spanish: Exchange, 5. Nc3 f6 6. d4'),
			moves => {
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/4p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6' => {
			eco => 'C69b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4

			variation => N__('Spanish: Exchange, Gligoric, 6. d4'),
			moves => {
				'c8g4' => 'r2qkbnr/1pp3pp/p1p2p2/4p3/3PP1b1/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 7',
				'e5d4' => 'r1bqkbnr/1pp3pp/p1p2p2/8/3pP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/4p3/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C68h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. Nc3 f6

			variation => N__('Spanish: Exchange, Keres, 5... f6'),
			moves => {
				'd2d3' => 'r1bqkbnr/1pp3pp/p1p2p2/4p3/4P3/2NP1N2/PPP2PPP/R1BQK2R b KQkq - 0 6',
				'd2d4' => 'r1bqkbnr/1pp3pp/p1p2p2/4p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq d3 0 6',
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/4p3/4P3/2NP1N2/PPP2PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C68h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. Nc3 f6 6. d3

			variation => N__('Spanish: Exchange, Romanovsky Variation'),
			moves => {
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/4p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C69a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6

			variation => N__('Spanish: Exchange, Gligoric Variation'),
			moves => {
				'd2d4' => 'r1bqkbnr/1pp3pp/p1p2p2/4p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6',
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/8/3NP3/8/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C69h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 exd4 7.Qxd4'),
			moves => {
				'c6c5' => 'r1bqkbnr/1pp3pp/p4p2/2p5/3NP3/8/PPP2PPP/RNBQ1RK1 w kq - 0 8',
				'g8e7' => 'r1bqkb1r/1pp1n1pp/p1p2p2/8/3NP3/8/PPP2PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/8/3QP3/5N2/PPP2PPP/RNB2RK1 b kq - 0 7' => {
			eco => 'C69i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Qxd4

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 exd4 7.Nxd4'),
			moves => {
			},
		},
		'r1bqkbnr/1pp3pp/p1p2p2/8/3pP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C69h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 exd4'),
			moves => {
				'd1d4' => 'r1bqkbnr/1pp3pp/p1p2p2/8/3QP3/5N2/PPP2PPP/RNB2RK1 b kq - 0 7',
				'f3d4' => 'r1bqkbnr/1pp3pp/p1p2p2/8/3NP3/8/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r1bqkbnr/1pp3pp/p4p2/2p5/3NP3/8/PPP2PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C69j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 exd4 7.Nxd4 c5'),
			moves => {
				'd4b3' => 'r1bqkbnr/1pp3pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBQ1RK1 b kq - 1 8',
				'd4e2' => 'r1bqkbnr/1pp3pp/p4p2/2p5/4P3/8/PPP1NPPP/RNBQ1RK1 b kq - 1 8',
			},
		},
		'r1bqkbnr/1pp3pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBQ1RK1 b kq - 1 8' => {
			eco => 'C69k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5 8. Nb3

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3'),
			moves => {
				'd8d1' => 'r1b1kbnr/1pp3pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBq1RK1 w kq - 0 9',
			},
		},
		'r1bqkbnr/1pp3pp/p4p2/2p5/4P3/8/PPP1NPPP/RNBQ1RK1 b kq - 1 8' => {
			eco => 'C69j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5 8. Ne2

			variation => N__('Spanish: Exchange, Gligoric, 8. Ne2'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp1p1p/p1n3p1/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 g6

			variation => N__('Spanish: Fianchetto Deferred'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp1p1p/p1n5/4p1p1/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq g6 0 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 g5

			variation => N__('Spanish: Brentano Variation'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp1ppp/2n5/pB2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq a6 0 4' => {
			eco => 'C60b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a5

			variation => N__('Spanish: 3... a5'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp1ppp/p1B5/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C68c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6

			variation => N__('Spanish: Exchange Variation'),
			moves => {
				'b7c6' => 'r1bqkbnr/2pp1ppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
				'd7c6' => 'r1bqkbnr/1pp2ppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/1ppp1ppp/p1n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C68a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6

			variation => N__('Spanish: 3... a6'),
			moves => {
				'b5a4' => 'r1bqkbnr/1ppp1ppp/p1n5/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 4',
				'b5c4' => 'r1bqkbnr/1ppp1ppp/p1n5/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 4',
				'b5c6' => 'r1bqkbnr/1ppp1ppp/p1B5/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/1ppp1ppp/p1n5/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C68b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bc4

			variation => N__('Spanish: 3... a6 4. Bc4'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp1ppp/p1n5/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4

			variation => N__('Spanish: 4. Ba4'),
			moves => {
				'b7b5' => 'r1bqkbnr/2pp1ppp/p1n5/1p2p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq b6 0 5',
				'c6d4' => 'r1bqkbnr/1ppp1ppp/p7/4p3/B2nP3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5',
				'd7d6' => 'r1bqkbnr/1pp2ppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
				'f7f5' => 'r1bqkbnr/1ppp2pp/p1n5/4pp2/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 5',
				'f8b4' => 'r1bqk1nr/1ppp1ppp/p1n5/4p3/Bb2P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5',
				'f8c5' => 'r1bqk1nr/1ppp1ppp/p1n5/2b1p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5',
				'f8e7' => 'r1bqk1nr/1pppbppp/p1n5/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5',
				'g7g5' => 'r1bqkbnr/1ppp1p1p/p1n5/4p1p1/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq g6 0 5',
				'g7g6' => 'r1bqkbnr/1ppp1p1p/p1n3p1/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
				'g8e7' => 'r1bqkb1r/1pppnppp/p1n5/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5',
				'g8f6' => 'r1bqkb1r/1ppp1ppp/p1n2n2/4p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/1ppp1ppp/p7/4p3/B2nP3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nd4

			variation => N__('Spanish: Bird\'s Deferred'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp2pp/p1n5/4Pp2/B2p4/5N2/PPP2PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 f5 5. d4 exd4 6. e5

			variation => N__('Spanish: Schliemann Deferred, 5. d4 exd4 6. e5'),
			moves => {
			},
		},
		'r1bqkbnr/1ppp2pp/p1n5/4pp2/B2PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 f5 5. d4

			variation => N__('Spanish: Schliemann Deferred, 5. d4'),
			moves => {
				'e5d4' => 'r1bqkbnr/1ppp2pp/p1n5/5p2/B2pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/1ppp2pp/p1n5/4pp2/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 f5

			variation => N__('Spanish: Schliemann Deferred'),
			moves => {
				'd2d4' => 'r1bqkbnr/1ppp2pp/p1n5/4pp2/B2PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 5',
			},
		},
		'r1bqkbnr/1ppp2pp/p1n5/5p2/B2pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C70',
                        # TRANSLATORS:

			variation => N__('Spanish: Schliemann Deferred, 5. d4'),
			moves => {
				'e4e5' => 'r1bqkbnr/1ppp2pp/p1n5/4Pp2/B2p4/5N2/PPP2PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/2p2ppp/p1np4/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'C71',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'c6d4' => 'r1bqkbnr/2p2ppp/p2p4/1p2p3/3nP3/1B3N2/PPP2PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/2p2ppp/p1np4/1p2p3/B2PP3/5N2/PPP2PPP/RNBQK2R w KQkq b6 0 6' => {
			eco => 'C71',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'a4b3' => 'r1bqkbnr/2p2ppp/p1np4/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/2p2ppp/p1pp4/4p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C73',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. Bxc6+ bxc6 6. d4

			variation => N__('Spanish: Modern Steinitz, Richter Variation'),
			moves => {
				'f7f6' => 'r1bqkbnr/2p3pp/p1pp1p2/4p3/3PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/2p2ppp/p1pp4/4p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C73',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. Bxc6+ bxc6

			variation => N__('Spanish: Modern Steinitz, Richter Variation'),
			moves => {
				'd2d4' => 'r1bqkbnr/2p2ppp/p1pp4/4p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 6',
			},
		},
		'r1bqkbnr/2p2ppp/p2p4/1p2p3/3NP3/1B6/PPP2PPP/RNBQK2R b KQkq - 0 7' => {
			eco => 'C71',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'e5d4' => 'r1bqkbnr/2p2ppp/p2p4/1p6/3pP3/1B6/PPP2PPP/RNBQK2R w KQkq - 0 8',
			},
		},
		'r1bqkbnr/2p2ppp/p2p4/1p2p3/3nP3/1B3N2/PPP2PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C71',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'f3d4' => 'r1bqkbnr/2p2ppp/p2p4/1p2p3/3NP3/1B6/PPP2PPP/RNBQK2R b KQkq - 0 7',
			},
		},
		'r1bqkbnr/2p2ppp/p2p4/1p6/3QP3/1B6/PPP2PPP/RNB1K2R b KQkq - 0 8' => {
			eco => 'C71',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'c7c5' => 'r1bqkbnr/5ppp/p2p4/1pp5/3QP3/1B6/PPP2PPP/RNB1K2R w KQkq c6 0 9',
			},
		},
		'r1bqkbnr/2p2ppp/p2p4/1p6/3pP3/1B6/PPP2PPP/RNBQK2R w KQkq - 0 8' => {
			eco => 'C71',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, 5. d4'),
			moves => {
				'd1d4' => 'r1bqkbnr/2p2ppp/p2p4/1p6/3QP3/1B6/PPP2PPP/RNB1K2R b KQkq - 0 8',
			},
		},
		'r1bqkbnr/2p3pp/p1pp1p2/4p3/3PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C73',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. Bxc6+ bxc6 6. d4 f6

			variation => N__('Spanish: Modern Steinitz, Alapin Variation'),
			moves => {
			},
		},
		'r1bqkbnr/2pp1ppp/p1n5/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C70',
                        # TRANSLATORS:

			variation => N__('Spanish: Caro Variation'),
			moves => {
				'c6a5' => 'r1bqkbnr/2pp1ppp/p7/np2p3/4P3/1B3N2/PPPP1PPP/RNBQK2R w KQkq - 2 6',
				'f8c5' => 'r1bqk1nr/2pp1ppp/p1n5/1pb1p3/4P3/1B3N2/PPPP1PPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'r1bqkbnr/2pp1ppp/p1n5/1p2p3/B3P3/5N2/PPPP1PPP/RNBQK2R w KQkq b6 0 5' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 b5

			variation => N__('Spanish: Caro Variation'),
			moves => {
				'a4b3' => 'r1bqkbnr/2pp1ppp/p1n5/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/2pp1ppp/p1p5/4p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C68c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 bxc6

			variation => N__('Spanish: Exchange, 4...bxc6'),
			moves => {
			},
		},
		'r1bqkbnr/2pp1ppp/p7/np2p3/4P3/1B3N2/PPPP1PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'C70',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 b5 5. Bb3 Na5

			variation => N__('Spanish: Taimanov (Wing) Variation'),
			moves => {
			},
		},
		'r1bqkbnr/3p1ppp/p1p1p3/8/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B46c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, 5... a6 6.Nxc6'),
			moves => {
				'f1d3' => 'r1bqkbnr/3p1ppp/p1p1p3/8/4P3/2NB4/PPP2PPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqkbnr/3p1ppp/p1p1p3/8/4P3/2NB4/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B46d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Nxc6 bxc6 7. Bd3

			variation => N__('Sicilian: Taimanov, 5... a6 6.Nxc6 bxc6 7. Bd3'),
			moves => {
				'd7d5' => 'r1bqkbnr/5ppp/p1p1p3/3p4/4P3/2NB4/PPP2PPP/R1BQK2R w KQkq d6 0 8',
			},
		},
		'r1bqkbnr/5ppp/p1np4/1p1Np3/4P3/N7/PPP2PPP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'B32q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6.N1c3 a6 7. Na3 b5'),
			moves => {
				'g8e7' => 'r1bqkb1r/4nppp/p1np4/1p1Np3/4P3/N7/PPP2PPP/R1BQKB1R w KQkq - 2 9',
			},
		},
		'r1bqkbnr/5ppp/p1np4/1p2p3/4P3/N1N5/PPP2PPP/R1BQKB1R w KQkq b6 0 8' => {
			eco => 'B32q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. N1c3 a6 7. Na3 b5

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6.N1c3 a6 7. Na3 b5'),
			moves => {
				'c3d5' => 'r1bqkbnr/5ppp/p1np4/1p1Np3/4P3/N7/PPP2PPP/R1BQKB1R b KQkq - 1 8',
			},
		},
		'r1bqkbnr/5ppp/p1npp3/1p6/2B1P3/2N2N2/PP2QPPP/R1B2RK1 w kq b6 0 9' => {
			eco => 'B21y',
                        # TRANSLATORS:

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O'),
			moves => {
				'c4b3' => 'r1bqkbnr/5ppp/p1npp3/1p6/4P3/1BN2N2/PP2QPPP/R1B2RK1 b kq - 1 9',
			},
		},
		'r1bqkbnr/5ppp/p1npp3/1p6/4P3/1BN2N2/PP2QPPP/R1B2RK1 b kq - 1 9' => {
			eco => 'B21y',
                        # TRANSLATORS:

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O'),
			moves => {
				'a8a7' => '2bqkbnr/r4ppp/p1npp3/1p6/4P3/1BN2N2/PP2QPPP/R1B2RK1 w k - 2 10',
			},
		},
		'r1bqkbnr/5ppp/p1p1p3/3p4/4P3/2NB4/PPP2PPP/R1BQK2R w KQkq d6 0 8' => {
			eco => 'B46e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 a6 6. Nxc6 bxc6 7. Bd3 d5

			variation => N__('Sicilian: Taimanov, 5... a6 6.Nxc6 bxc6 7. Bd3 d5'),
			moves => {
			},
		},
		'r1bqkbnr/5ppp/p2p4/1pp5/3QP3/1B6/PPP2PPP/RNB1K2R w KQkq c6 0 9' => {
			eco => 'C71',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. d4 b5 6. Bb3 Nxd4 7. Nxd4 exd4 8. Qxd4 c5

			variation => N__('Spanish: Modern Steinitz, Noah\'s Ark Trap'),
			moves => {
			},
		},
		'r1bqkbnr/p2p1ppp/2p1p3/8/4P3/3B4/PPP2PPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'B44j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nxc6 bxc6 6. Bd3

			variation => N__('Sicilian: Taimanov, 5.Nxc6 bxc6 6. Bd3'),
			moves => {
			},
		},
		'r1bqkbnr/p2p1ppp/2p1p3/8/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B44i',
                        # TRANSLATORS:

			variation => N__('Sicilian: Taimanov, 5.Nxc6'),
			moves => {
				'f1d3' => 'r1bqkbnr/p2p1ppp/2p1p3/8/4P3/3B4/PPP2PPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp1n1ppp/2pp4/4p1N1/2BPP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6 5. Ng5

			variation => N__('Philidor: Hanham, Kmoch Variation'),
			moves => {
				'g8h6' => 'r1bqkb1r/pp1n1ppp/2pp3n/4p1N1/2BPP3/8/PPP2PPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6 5. Nc3

			variation => N__('Philidor: Hanham, Schlechter Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6 5. c3

			variation => N__('Philidor: Hanham, Delmar Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6 5. O-O

			variation => N__('Philidor: Hanham, Krause Variation'),
			moves => {
				'f8e7' => 'r1bqk1nr/pp1nbppp/2pp4/4p3/2BPP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C41g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4 c6

			variation => N__('Philidor: Hanham, 4. Bc4 c6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 1 5',
				'c2c3' => 'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 5',
				'e1g1' => 'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/5N2/PPP2PPP/RNBQ1RK1 b kq - 1 5',
				'f3g5' => 'r1bqkbnr/pp1n1ppp/2pp4/4p1N1/2BPP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/pp1npppp/2p5/6N1/3P4/8/PPP2PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B17h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Ng5

			variation => N__('Caro-Kann: Steinitz, 5. Ng5'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1npppp/2p2n2/6N1/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp1npppp/2p5/8/2BPN3/8/PPP2PPP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'B17p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Bc4

			variation => N__('Caro-Kann: Steinitz, 5. Bc4'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1npppp/2p2n2/8/2BPN3/8/PPP2PPP/R1BQK1NR w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp1npppp/2p5/8/3PN3/5N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B17b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7 5. Nf3

			variation => N__('Caro-Kann: Steinitz, 5. Nf3'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1npppp/2p2n2/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp1npppp/2p5/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B17a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7

			variation => N__('Caro-Kann: Steinitz Variation'),
			moves => {
				'e4g5' => 'r1bqkbnr/pp1npppp/2p5/6N1/3P4/8/PPP2PPP/R1BQKBNR b KQkq - 2 5',
				'f1c4' => 'r1bqkbnr/pp1npppp/2p5/8/2BPN3/8/PPP2PPP/R1BQK1NR b KQkq - 2 5',
				'g1f3' => 'r1bqkbnr/pp1npppp/2p5/8/3PN3/5N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp1npppp/3p4/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'B51f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4 cxd4

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4 cxd4'),
			moves => {
			},
		},
		'r1bqkbnr/pp1npppp/3p4/1Bp5/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'B51f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. d4

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. d4'),
			moves => {
				'c5d4' => 'r1bqkbnr/pp1npppp/3p4/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
				'g8f6' => 'r1bqkb1r/pp1npppp/3p1n2/1Bp5/3PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 5',
			},
		},
		'r1bqkbnr/pp1npppp/3p4/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B51b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. c3

			variation => N__('Sicilian: 3. Bb5+ Nd7 4. c3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1npppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'B51c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7 4. O-O

			variation => N__('Sicilian: 3. Bb5+ Nd7 4.O-O'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1npppp/3p1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 5',
			},
		},
		'r1bqkbnr/pp1npppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4' => {
			eco => 'B51a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nd7

			variation => N__('Sicilian: 3. Bb5+ Nd7'),
			moves => {
				'c2c3' => 'r1bqkbnr/pp1npppp/3p4/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkbnr/pp1npppp/3p4/1Bp5/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqkbnr/pp1npppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'r1bqkbnr/pp1p1ppp/2B1p3/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B30q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. Bxc6

			variation => N__('Sicilian: Rossolimo, 3... e6 4.Bxc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2N1p3/8/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B44i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nxc6

			variation => N__('Sicilian: Taimanov, 5.Nxc6'),
			moves => {
				'b7c6' => 'r1bqkbnr/p2p1ppp/2p1p3/8/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/1P3N2/P1PP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B30o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. b3

			variation => N__('Sicilian: Rossolimo, 3... e6 4. b3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4' => {
			eco => 'B30p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. Nc3

			variation => N__('Sicilian: Rossolimo, 3... e6 4. Nc3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4' => {
			eco => 'B30r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6 4. O-O

			variation => N__('Sicilian: Rossolimo, 3... e6 4.O-O'),
			moves => {
				'g8e7' => 'r1bqkb1r/pp1pnppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 5',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'B30n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 e6

			variation => N__('Sicilian: Rossolimo, 3... e6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4',
				'b2b3' => 'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/1P3N2/P1PP1PPP/RNBQK2R b KQkq - 0 4',
				'b5c6' => 'r1bqkbnr/pp1p1ppp/2B1p3/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'e1g1' => 'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/1N6/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'B44k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5

			variation => N__('Sicilian, Taimanov, Szen Variation'),
			moves => {
				'd7d6' => 'r1bqkbnr/pp3ppp/2npp3/1N6/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p1P3/2P5/2N2N2/PP1P1PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5 Ng8 5. Nf3 Nc6

			variation => N__('English: Mikenas, 5. Nf3 Nc6'),
			moves => {
				'd2d4' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p1P3/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq d3 0 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p1P3/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq d3 0 6' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5 Ng8 5. Nf3 Nc6 6. d4

			variation => N__('English: Mikenas, 5. Nf3 Nc6 6. d4'),
			moves => {
				'c5d4' => 'r1bqkbnr/pp1p1ppp/2n1p3/4P3/2Pp4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A36c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 e6 4. Nf3

			variation => N__('English: Symmetrical, 3. g3 e6 4. Nf3'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A36c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 e6

			variation => N__('English: Symmetrical, 3. g3 e6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B40d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. b3 Nc6

			variation => N__('Sicilian: 2... e6 3. b3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 2 4' => {
			eco => 'B40h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. Nc3 Nc6

			variation => N__('Sicilian: 2... e6 3. Nc3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 e6

			variation => N__('Sicilian: Closed, 3. g3 e6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/3P1NP1/PPP2P1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B30e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. g3 e6 4. d3

			variation => N__('Sicilian: 2... Nc6 3. g3 e6 4. d3'),
			moves => {
				'd7d5' => 'r1bqkbnr/pp3ppp/2n1p3/2pp4/4P3/3P1NP1/PPP2P1P/RNBQKB1R w KQkq d6 0 5',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B30d',
                        # TRANSLATORS:

			variation => N__('Sicilian: 2... Nc6 3. g3'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/3P1NP1/PPP2P1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B23q',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 e6 4. Nf3

			variation => N__('Sicilian: Closed, Grand Prix, 3... e6 4. Nf3'),
			moves => {
				'd7d5' => 'r1bqkbnr/pp3ppp/2n1p3/2pp4/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq d6 0 5',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B23q',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 e6

			variation => N__('Sicilian: Closed, Grand Prix, 3... e6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B21g',
                        # TRANSLATORS: 1. e4 c5 2. f4 Nc6 3. Nf3 e6

			variation => N__('Sicilian: Grand Prix, 2... Nc6 3. Nf3 e6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/4P3/2PN4/2N5/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'A19',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, 5. Nf3 Nc6 6. d4'),
			moves => {
				'c6e5' => 'r1bqkbnr/pp1p1ppp/4p3/4n3/2PN4/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/4P3/2Pp4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'A19',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, 5. Nf3 Nc6 6. d4'),
			moves => {
				'f3d4' => 'r1bqkbnr/pp1p1ppp/2n1p3/4P3/2PN4/2N5/PP3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/2B1P3/2N2N2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B21x',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 e6 6. Bc4

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 Nc6 5. Nf3 e6 6. Bc4'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/2B1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/2PNP3/8/PP3PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'B44d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. c4

			variation => N__('Sicilian: Taimanov, 5. c4'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B45a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3

			variation => N__('Sicilian: Taimanov, 5. Nc3'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'd7d6' => 'r1bqkbnr/pp3ppp/2npp3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'd8c7' => 'r1b1kbnr/ppqp1ppp/2n1p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'f8b4' => 'r1bqk1nr/pp1p1ppp/2n1p3/8/1b1NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'g8f6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 2 5' => {
			eco => 'B44g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Be3

			variation => N__('Sicilian: Taimanov, 5. Be3'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/4B3/PPP2PPP/RN1QKB1R w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/6P1/PPP2P1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B44b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. g3

			variation => N__('Sicilian: Taimanov, 5. g3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/8/PPP1BPPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'B44c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Be2

			variation => N__('Sicilian: Taimanov, 5. Be2'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B44a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6

			variation => N__('Sicilian: Taimanov'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5',
				'c1e3' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 2 5',
				'c2c4' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/2PNP3/8/PP3PPP/RNBQKB1R b KQkq c3 0 5',
				'd4b5' => 'r1bqkbnr/pp1p1ppp/2n1p3/1N6/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 2 5',
				'd4c6' => 'r1bqkbnr/pp1p1ppp/2N1p3/8/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5',
				'f1e2' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/8/PPP1BPPP/RNBQK2R b KQkq - 2 5',
				'g2g3' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/6P1/PPP2P1P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n1p3/8/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B21x',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 e6

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 Nc6 5. Nf3 e6'),
			moves => {
				'f1c4' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/2B1P3/2N2N2/PP3PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n5/1N2p3/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'B32j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5

			variation => N__('Sicilian: Lowenthal, 5. Nb5'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p1p1ppp/p1n5/1N2p3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
				'd7d6' => 'r1bqkbnr/pp3ppp/2np4/1N2p3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2n5/2p1p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq e6 0 4' => {
			eco => 'B30j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Nc3 e5

			variation => N__('Sicilian: 2... Nc6 3. Nc3 e5'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n5/2p1p3/4P3/2N5/PPPPNPPP/R1BQKB1R w KQkq e6 0 4' => {
			eco => 'B23n',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. Nge2 e5

			variation => N__('Sicilian: Chameleon, 3... e5'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/2n5/4p3/3NP3/8/PPP2PPP/RNBQKB1R w KQkq e6 0 5' => {
			eco => 'B32i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5

			variation => N__('Sicilian: Lowenthal'),
			moves => {
				'd4b5' => 'r1bqkbnr/pp1p1ppp/2n5/1N2p3/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/pp1p1ppp/2p1n3/4N3/2B1P3/8/PPP2PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'C44r',
                        # TRANSLATORS:

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1p1ppp/2p1nn2/4N3/2B1P3/8/PPP2PPP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'r1bqkbnr/pp1p1ppp/2p1n3/4N3/2B1P3/8/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C44r',
                        # TRANSLATORS:

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'e1g1' => 'r1bqkbnr/pp1p1ppp/2p1n3/4N3/2B1P3/8/PPP2PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2p5/1B6/3pP3/8/PPPP1PPP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C61g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O c6

			variation => N__('Spanish: Bird\'s, 5.O-O c6'),
			moves => {
				'b5c4' => 'r1bqkbnr/pp1p1ppp/2p5/8/2BpP3/8/PPPP1PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'r1bqkbnr/pp1p1ppp/2p5/8/2BpP3/8/PPPP1PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'C61h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O c6 6. Bc4

			variation => N__('Spanish: Bird\'s, 5.O-O c6 6. Bc4'),
			moves => {
			},
		},
		'r1bqkbnr/pp1p1ppp/4p3/4n3/2PN4/2N5/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5 Ng8 5. Nf3 Nc6 6. d4 cxd4 7. Nxd4 Nxe5

			variation => N__('English: Mikenas, 6. d4 cxd4 7.Nxd4 Nxe5'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2B3p1/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B31c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. Bxc6

			variation => N__('Sicilian: Rossolimo, 3... g6 4.Bxc6'),
			moves => {
				'd7c6' => 'r1bqkbnr/pp2pp1p/2p3p1/2p5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pp1ppp1p/2N3p1/8/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B34c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nxc6

			variation => N__('Sicilian: Accelerated Fianchetto, Exchange Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/1Bp5/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4' => {
			eco => 'B31b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. Nc3

			variation => N__('Sicilian: Rossolimo, 3... g6 4. Nc3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4' => {
			eco => 'B31g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. O-O

			variation => N__('Sicilian: Rossolimo, 3... g6 4.O-O'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 5',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'B31a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6

			variation => N__('Sicilian: Rossolimo, 3... g6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1ppp1p/2n3p1/1Bp5/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4',
				'b5c6' => 'r1bqkbnr/pp1ppp1p/2B3p1/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'e1g1' => 'r1bqkbnr/pp1ppp1p/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 Nc6 3. Nc3 g6

			variation => N__('Sicilian: English, 2... Nc6 3. Nc3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A35e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 g6 4. e3

			variation => N__('English: Symmetrical, 2. Nc3 Nc6 3. Nf3 g6 4. e3'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N1PN2/PP1P1PPP/R1BQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A35d',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3 g6

			variation => N__('English: Symmetrical, 2. Nc3 Nc6 3. Nf3 g6'),
			moves => {
				'e2e3' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A36e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6

			variation => N__('English: Symmetrical, 3. g3 g6'),
			moves => {
				'f1g2' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A36e',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2

			variation => N__('English: Symmetrical, 3. g3 g6'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'B30i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Nc3 g6

			variation => N__('Sicilian: 2... Nc6 3. Nc3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6

			variation => N__('Sicilian: Closed, 3. g3 g6'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2NP2P1/PPP2P1P/R1BQKBNR b KQkq - 0 4',
				'f1g2' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2

			variation => N__('Sicilian: Closed, 3. g3 g6'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'B23n',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. Nge2 g6

			variation => N__('Sicilian: Chameleon, 3... g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2NP2P1/PPP2P1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3 g6 4. d3

			variation => N__('Sicilian: Closed, 3. g3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B23s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B23s',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 g6

			variation => N__('Sicilian: Closed, Grand Prix, 3... g6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B21h',
                        # TRANSLATORS: 1. e4 c5 2. f4 Nc6 3. Nf3 g6

			variation => N__('Sicilian: Grand Prix, 2... Nc6 3. Nf3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/8/2BNP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B34b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Bc4

			variation => N__('Sicilian: Accelerated Fianchetto, 5. Bc4'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/8/2PNP3/8/PP3PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'B36a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4

			variation => N__('Sicilian: Maroczy Bind'),
			moves => {
				'd7d6' => 'r1bqkbnr/pp2pp1p/2np2p1/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 0 6',
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6',
				'g8f6' => 'r1bqkb1r/pp1ppp1p/2n2np1/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'B34f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3

			variation => N__('Sicilian: Accelerated Fianchetto, Modern Variation'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp1pppbp/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 2 6',
				'g8f6' => 'r1bqkb1r/pp1ppp1p/2n2np1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 1 5' => {
			eco => 'B34e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Be3

			variation => N__('Sicilian: Accelerated Fianchetto, 5. Be3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/8/PPP1BPPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B34d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Be2

			variation => N__('Sicilian: Accelerated Fianchetto, 5. Be2'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B34a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6

			variation => N__('Sicilian: Accelerated Fianchetto'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 5',
				'c1e3' => 'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 1 5',
				'c2c4' => 'r1bqkbnr/pp1ppp1p/2n3p1/8/2PNP3/8/PP3PPP/RNBQKB1R b KQkq c3 0 5',
				'd4c6' => 'r1bqkbnr/pp1ppp1p/2N3p1/8/4P3/8/PPP2PPP/RNBQKB1R b KQkq - 0 5',
				'f1c4' => 'r1bqkbnr/pp1ppp1p/2n3p1/8/2BNP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5',
				'f1e2' => 'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/8/PPP1BPPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/1Bp5/4P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3' => {
			eco => 'B23l',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. Bb5

			variation => N__('Sicilian: Closed, 2... Nc6 3. Bb5'),
			moves => {
				'c6d4' => 'r1bqkbnr/pp1ppppp/8/1Bp5/3nP3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3' => {
			eco => 'B30k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5

			variation => N__('Sicilian: Rossolimo'),
			moves => {
				'd8b6' => 'r1b1kbnr/pp1ppppp/1qn5/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pp1ppppp/2n2n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3' => {
			eco => 'B30g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bc4

			variation => N__('Sicilian: 2... Nc6 3. Bc4'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 Nc6 3. Nc3

			variation => N__('Sicilian: English, 2... Nc6 3. Nc3'),
			moves => {
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 Nc6 3. Nf3

			variation => N__('Sicilian: English, 2... Nc6 3. Nf3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 Nc6

			variation => N__('Sicilian: English, 2... Nc6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq - 2 3',
				'g1e2' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/8/PP1PNPPP/RNBQKB1R b KQkq - 2 3',
				'g1f3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/8/PP1PNPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 Nc6 3. Ne2

			variation => N__('Sicilian: English, 2... Nc6 3. Ne2'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N1P3/PP1P1PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A35b',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. e3

			variation => N__('English: Symmetrical, 2. Nc3 Nc6 3. e3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'A35c',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. Nf3

			variation => N__('English: Symmetrical, 2. Nc3 Nc6 3. Nf3'),
			moves => {
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A36a',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3

			variation => N__('English: Symmetrical, 3. g3'),
			moves => {
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A35a',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6

			variation => N__('English: Symmetrical'),
			moves => {
				'e2e3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N1P3/PP1P1PPP/R1BQKBNR b KQkq - 0 3',
				'g1f3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3',
				'g2g3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A30b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nc6

			variation => N__('English: Symmetrical, 2. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'B22b',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nc6 3. d4

			variation => N__('Sicilian: Alapin, 2... Nc6 3. d4'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'B32a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4

			variation => N__('Sicilian: 2... Nc6 3. d4'),
			moves => {
				'c5d4' => 'r1bqkbnr/pp1ppppp/2n5/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B30b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. b3

			variation => N__('Sicilian: 2... Nc6 3. b3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b3 Nc6

			variation => N__('Sicilian: Snyder, 2... Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'B30h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Nc3

			variation => N__('Sicilian: 2... Nc6 3. Nc3'),
			moves => {
				'e7e5' => 'r1bqkbnr/pp1p1ppp/2n5/2p1p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq e6 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'B24',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. g3

			variation => N__('Sicilian: Closed, 3. g3'),
			moves => {
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'B23k',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6

			variation => N__('Sicilian: Closed, 2... Nc6'),
			moves => {
				'f1b5' => 'r1bqkbnr/pp1ppppp/2n5/1Bp5/4P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3',
				'f2f4' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
				'g1e2' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R b KQkq - 3 3',
				'g2g3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'B23n',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. Nge2

			variation => N__('Sicilian: Chameleon Variation'),
			moves => {
				'e7e5' => 'r1bqkbnr/pp1p1ppp/2n5/2p1p3/4P3/2N5/PPPPNPPP/R1BQKB1R w KQkq e6 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B22b',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nc6 3. Nf3

			variation => N__('Sicilian: Alapin, 2... Nc6 3. Nf3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B22b',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nc6

			variation => N__('Sicilian: Alapin, 2... Nc6'),
			moves => {
				'd2d4' => 'r1bqkbnr/pp1ppppp/2n5/2p5/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 3',
				'g1f3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B30c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d3

			variation => N__('Sicilian: 2... Nc6 3. d3'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/3P2P1/PPP2P1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. d3 Nc6 3. g3

			variation => N__('Sicilian: 2. d3 Nc6 3. g3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. d3 Nc6

			variation => N__('Sicilian: 2. d3 Nc6'),
			moves => {
				'g2g3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/3P2P1/PPP2P1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'B30a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6

			variation => N__('Sicilian: 2... Nc6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
				'b2b3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R b KQkq - 0 3',
				'd2d3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'd2d4' => 'r1bqkbnr/pp1ppppp/2n5/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'f1b5' => 'r1bqkbnr/pp1ppppp/2n5/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3',
				'f1c4' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3',
				'g2g3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B30d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. g3

			variation => N__('Sicilian: 2... Nc6 3. g3'),
			moves => {
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'B23o',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4

			variation => N__('Sicilian: Closed, Grand Prix'),
			moves => {
				'd7d6' => 'r1bqkbnr/pp2pppp/2np4/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/3P4/PPP3PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B21e',
                        # TRANSLATORS: 1. e4 c5 2. f4 Nc6 3. d3

			variation => N__('Sicilian: Grand Prix, 2... Nc6 3. d3'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B21f',
                        # TRANSLATORS: 1. e4 c5 2. f4 Nc6 3. Nf3

			variation => N__('Sicilian: Grand Prix, 2... Nc6 3. Nf3'),
			moves => {
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B21e',
                        # TRANSLATORS: 1. e4 c5 2. f4 Nc6

			variation => N__('Sicilian: Grand Prix, 2... Nc6'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/3P4/PPP3PP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 c5 2. Nf3 Nc6

			variation => N__('Bird: 1..c5 2. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/8/2N2N2/PPPPPPPP/R1BQKB1R w KQkq - 2 3' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 c5 2. Nf3 Nc6

			variation => N__('Van Geet: Sicilian Variation, 2. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 Nc6

			variation => N__('Reti: 1... c5'),
			moves => {
				'f1g2' => 'r1bqkbnr/pp1ppppp/2n5/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 Nc6 3. Bg2

			variation => N__('Reti: 1... c5'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B32c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4

			variation => N__('Sicilian: Open, 2... Nc6'),
			moves => {
				'd7d5' => 'r1bqkbnr/pp2pppp/2n5/3p4/3NP3/8/PPP2PPP/RNBQKB1R w KQkq d6 0 5',
				'd7d6' => 'r1bqkbnr/pp2pppp/2np4/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'd8b6' => 'r1b1kbnr/pp1ppppp/1qn5/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'd8c7' => 'r1b1kbnr/ppqppppp/2n5/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'e7e5' => 'r1bqkbnr/pp1p1ppp/2n5/4p3/3NP3/8/PPP2PPP/RNBQKB1R w KQkq e6 0 5',
				'g7g6' => 'r1bqkbnr/pp1ppp1p/2n3p1/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'g8f6' => 'r1bqkb1r/pp1ppppp/2n2n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B32b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4

			variation => N__('Sicilian: 2... Nc6 3. d4 cxd4'),
			moves => {
				'f3d4' => 'r1bqkbnr/pp1ppppp/2n5/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/8/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B21w',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 Nc6 5. Nf3'),
			moves => {
				'd7d6' => 'r1bqkbnr/pp2pppp/2np4/8/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6',
				'e7e6' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pp1ppppp/2n5/8/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B21w',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 Nc6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pp1ppppp/2n5/8/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp1ppppp/8/1Bp5/3nP3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4' => {
			eco => 'B23m',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. Bb5 Nd4

			variation => N__('Sicilian: Closed, 2... Nc6 3. Bb5 Nd4'),
			moves => {
			},
		},
		'r1bqkbnr/pp1ppppp/n1p5/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B12a',
                        # TRANSLATORS: 1. e4 c6 2. d4 Na6

			variation => N__('Caro-Kann: de Bruycker Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pp1p/2n3p1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp2pp1p/2n2np1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'r1bqkbnr/pp2pp1p/2n3p1/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp2pp1p/2n3p1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqkbnr/pp2pp1p/2n3p1/3p4/3P4/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 6' => {
			eco => 'B13g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 g6

			variation => N__('Caro-Kann: Exchange, 4. Bd3 Nc6 5. c3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pp1p/2np2p1/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'B23p',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 d6 4. Nf3 g6

			variation => N__('Sicilian: Closed, Grand Prix, 3... d6 4. Nf3 g6'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pp1p/2np2p1/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B36b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 d6 6. Nc3

			variation => N__('Sicilian: Maroczy Bind, 5... d6 6. Nc3'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp2ppbp/2np2p1/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 2 7',
			},
		},
		'r1bqkbnr/pp2pp1p/2np2p1/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B36b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 d6

			variation => N__('Sicilian: Maroczy Bind, 5... d6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pp2pp1p/2np2p1/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp2pp1p/2p3p1/2p5/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'B31e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. Bxc6 dxc6 5. d3

			variation => N__('Sicilian: Rossolimo, 3... g6 4.Bxc6 dxc6 5. d3'),
			moves => {
				'f8g7' => 'r1bqk1nr/pp2ppbp/2p3p1/2p5/4P3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp2pp1p/2p3p1/2p5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'B31d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. Bb5 g6 4. Bxc6 dxc6

			variation => N__('Sicilian: Rossolimo, 3... g6 4.Bxc6 dxc6'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp2pp1p/2p3p1/2p5/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pp2pppp/2Bp4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B51l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. Bxc6+

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.Bxc6+'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pppp/2n5/2PpP3/8/8/PPP2PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B12k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 c5 4. dxc5 Nc6

			variation => N__('Caro-Kann: Advance, 3... c5 4.dxc5 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pppp/2n5/2p5/2Pp4/3P1NP1/PP2PPBP/RNBQK2R b KQkq - 0 5' => {
			eco => 'A09j',
                        # TRANSLATORS:

			variation => N__('Reti: Advance, 3. g3 c5 4. Bg2 Nc6'),
			moves => {
				'e7e5' => 'r1bqkbnr/pp3ppp/2n5/2p1p3/2Pp4/3P1NP1/PP2PPBP/RNBQK2R w KQkq e6 0 6',
			},
		},
		'r1bqkbnr/pp2pppp/2n5/2p5/2Pp4/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A09j',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 c5 4. Bg2 Nc6

			variation => N__('Reti: Advance, 3. g3 c5 4. Bg2 Nc6'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp2pppp/2n5/2p5/2Pp4/3P1NP1/PP2PPBP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pp2pppp/2n5/2pp4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. d4

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4. d4'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pppp/2n5/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6 4. O-O

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'e7e6' => 'r1bqkbnr/pp3ppp/2n1p3/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 0 5',
			},
		},
		'r1bqkbnr/pp2pppp/2n5/2pp4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nc6

			variation => N__('Reti: KIA, 2... c5  3. Bg2 Nc6'),
			moves => {
				'd2d4' => 'r1bqkbnr/pp2pppp/2n5/2pp4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqkbnr/pp2pppp/2n5/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'r1bqkbnr/pp2pppp/2n5/3p4/3NP3/8/PPP2PPP/RNBQKB1R w KQkq d6 0 5' => {
			eco => 'B32h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 d5

			variation => N__('Sicilian: 2..Nc6, Nimzowitsch Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pppp/2n5/3p4/3P4/2PB4/PP3PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'B13f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3

			variation => N__('Caro-Kann: Exchange, 4. Bd3 Nc6 5. c3'),
			moves => {
				'g7g6' => 'r1bqkbnr/pp2pp1p/2n3p1/3p4/3P4/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 6',
				'g8f6' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P4/2PB4/PP3PPP/RNBQK1NR w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp2pppp/2n5/3p4/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'B13f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6

			variation => N__('Caro-Kann: Exchange, 4. Bd3 Nc6'),
			moves => {
				'c2c3' => 'r1bqkbnr/pp2pppp/2n5/3p4/3P4/2PB4/PP3PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 2 5' => {
			eco => 'B53l',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 4... Nc6'),
			moves => {
				'c8d7' => 'r2qkbnr/pp1bpppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R w KQkq - 3 6',
				'd8d7' => 'r1b1kbnr/pp1qpppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'B51m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O'),
			moves => {
				'c8d7' => 'r2qkbnr/pp1bpppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 5',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4' => {
			eco => 'B51k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6

			variation => N__('Sicilian: 3. Bb5+ Nc6'),
			moves => {
				'b5c6' => 'r1bqkbnr/pp2pppp/2Bp4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'e1g1' => 'r1bqkbnr/pp2pppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B23o',
                        # TRANSLATORS:

			variation => N__('Sicilian: Closed, Grand Prix, 3... d6'),
			moves => {
				'g7g6' => 'r1bqkbnr/pp2pp1p/2np2p1/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B23o',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 d6

			variation => N__('Sicilian: Closed, Grand Prix, 3... d6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pp2pppp/2np4/2p5/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/8/2B1P3/2N2N2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B21y',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p2pppp/p1np4/8/2B1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7',
				'e7e6' => 'r1bqkbnr/pp3ppp/2npp3/8/2B1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B32d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 d6

			variation => N__('Sicilian: Open, 2... Nc6, 4... d6'),
			moves => {
			},
		},
		'r1bqkbnr/pp2pppp/2np4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5' => {
			eco => 'B53l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6

			variation => N__('Sicilian, Chekhover, 4... Nc6'),
			moves => {
				'f1b5' => 'r1bqkbnr/pp2pppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp2pppp/2np4/8/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B21y',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 Nc6 5. Nf3 d6'),
			moves => {
				'f1c4' => 'r1bqkbnr/pp2pppp/2np4/8/2B1P3/2N2N2/PP3PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'C07g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. Ngf3 Nc6

			variation => N__('French: Tarrasch, Open, 4.Ngf3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2pp4/4P3/3P1NP1/PPP2P1P/RNBQKB1R w KQkq d6 0 5' => {
			eco => 'B30f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. g3 e6 4. d3 d5

			variation => N__('Sicilian: 2... Nc6 3. g3 e6 4. d3 d5'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2pp4/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq d6 0 5' => {
			eco => 'B23r',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 Nc6 3. f4 e6 4. Nf3 d5

			variation => N__('Sicilian: Closed, Grand Prix, 3... e6 4. Nf3 d5'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'f8d6' => 'r1bqk1nr/pp3ppp/2nbp3/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
				'g8e7' => 'r1bqkb1r/pp2nppp/2n1p3/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
				'g8f6' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 0 5' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5, 3. Bg2 Nc6 4.O-O'),
			moves => {
				'd2d3' => 'r1bqkbnr/pp3ppp/2n1p3/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'C02l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3

			variation => N__('French: Advance, Paulsen'),
			moves => {
				'c8d7' => 'r2qkbnr/pp1b1ppp/2n1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
				'd8b6' => 'r1b1kbnr/pp3ppp/1qn1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
				'g8e7' => 'r1bqkb1r/pp2nppp/2n1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/2ppP3/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'C02k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6

			variation => N__('French: Advance, 4... Nc6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pp3ppp/2n1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/3P4/3Q4/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'D32d',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5.Qxd4'),
			moves => {
				'd4d1' => 'r1bqkbnr/pp3ppp/2n1p3/3P4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n1p3/3P4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 6' => {
			eco => 'D32d',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5.Qxd4'),
			moves => {
				'e6d5' => 'r1bqkbnr/pp3ppp/2n5/3p4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/1B1p4/3p4/5N2/PPPN1PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C09c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 cxd4

			variation => N__('French: Tarrasch, Open, 5.Ngf3 Nc6 6. Bb5 cxd4'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'C09b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5

			variation => N__('French: Tarrasch, Open, 5.Ngf3 Nc6 6. Bb5'),
			moves => {
				'c5d4' => 'r1bqkbnr/pp3ppp/2n5/1B1p4/3p4/5N2/PPPN1PPP/R1BQK2R w KQkq - 0 7',
				'f8d6' => 'r1bqk1nr/pp3ppp/2nb4/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R w KQkq - 4 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/1Bpp4/3P4/8/PPPN1PPP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'C08b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Bb5+ Nc6

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5. Bb5+ Nc6'),
			moves => {
				'd1e2' => 'r1bqkbnr/pp3ppp/2n5/1Bpp4/3P4/8/PPPNQPPP/R1B1K1NR b KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/1Bpp4/3P4/8/PPPNQPPP/R1B1K1NR b KQkq - 3 6' => {
			eco => 'C08c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Bb5+ Nc6 6. Qe2+

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5. Bb5+ Nc6 6. Qe2+'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2p1p3/2Pp4/3P1NP1/PP2PPBP/RNBQK2R w KQkq e6 0 6' => {
			eco => 'A09j',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 c5 4. Bg2 Nc6 5. d3 e5

			variation => N__('Reti: Advance, 3. g3 c5 4. Bg2 Nc6 5. d3 e5'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D32p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. Bg5

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. Bg5'),
			moves => {
				'f8e7' => 'r1bqk1nr/pp2bppp/2n5/2pp2B1/3P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D32o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. Bf4

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. Bf4'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D32r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. e3

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6 6. e3'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D32n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6

			variation => N__('QGD Tarrasch: 5. Nf3 Nc6'),
			moves => {
				'c1f4' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
				'c1g5' => 'r1bqkbnr/pp3ppp/2n5/2pp2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
				'e2e3' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6',
				'g2g3' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D33a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3

			variation => N__('QGD Tarrasch: 6. g3 (Schlecter/Rubinstein)'),
			moves => {
				'c5c4' => 'r1bqkbnr/pp3ppp/2n5/3p4/2pP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 7',
				'c5d4' => 'r1bqkbnr/pp3ppp/2n5/3p4/3p4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 7',
				'g8f6' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 2 6' => {
			eco => 'B22d',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. d4 d5 4. exd5 exd5 5. Nf3 Nc6

			variation => N__('Sicilian: Alapin, 2... e6 3. d4 d5 4.exd5 exd5 5. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C09a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6

			variation => N__('French: Tarrasch, Open, 5.Ngf3 Nc6'),
			moves => {
				'f1b5' => 'r1bqkbnr/pp3ppp/2n5/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nc6

			variation => N__('QGD: Tarrasch without Nc3: 5. g3 Nc6'),
			moves => {
				'f1g2' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nc6 6. Bg2

			variation => N__('QGD: Tarrasch without Nc3: 5. g3 Nc6 6. Bg2'),
			moves => {
				'g8f6' => 'r1bqkb1r/pp3ppp/2n2n2/2pp4/3P4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/3p4/2pP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D33c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4

			variation => N__('QGD Tarrasch: Swedish Variation'),
			moves => {
				'e2e4' => 'r1bqkbnr/pp3ppp/2n5/3p4/2pPP3/2N2NP1/PP3P1P/R1BQKB1R b KQkq e3 0 7',
				'f1g2' => 'r1bqkbnr/pp3ppp/2n5/3p4/2pP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/3p4/2pP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 7' => {
			eco => 'D33e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. Bg2

			variation => N__('QGD Tarrasch: Swedish, 7. Bg2'),
			moves => {
				'f8b4' => 'r1bqk1nr/pp3ppp/2n5/3p4/1bpP4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 2 8',
			},
		},
		'r1bqkbnr/pp3ppp/2n5/3p4/2pPP3/2N2NP1/PP3P1P/R1BQKB1R b KQkq e3 0 7' => {
			eco => 'D33d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 c4 7. e4

			variation => N__('QGD Tarrasch: Swedish, Rey Ardid Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/3p4/3p4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D33b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 cxd4

			variation => N__('QGD Tarrasch: 6. g3 cxd4'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/3p4/8/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 7' => {
			eco => 'D32e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qxd4 Nc6 6. Qd1 exd5 7. e3

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5.Qxd4 Nc6 6. Qd1 exd5 7. e3'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2n5/3p4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'D32d',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5.Qxd4'),
			moves => {
				'e2e3' => 'r1bqkbnr/pp3ppp/2n5/3p4/8/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp3ppp/2np4/1N2p3/2P1P3/8/PP3PPP/RNBQKB1R b KQkq c3 0 6' => {
			eco => 'B32s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. c4'),
			moves => {
				'f8e7' => 'r1bqk1nr/pp2bppp/2np4/1N2p3/2P1P3/8/PP3PPP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2np4/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B32p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. N1c3

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6.N1c3'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p3ppp/p1np4/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp3ppp/2np4/1N2p3/4P3/4B3/PPP2PPP/RN1QKB1R b KQkq - 1 7' => {
			eco => 'B44l',
                        # TRANSLATORS:

			variation => N__('Sicilian, Taimanov, Szen, 6. Bf4'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p3ppp/p1np4/1N2p3/4P3/4B3/PPP2PPP/RN1QKB1R w KQkq - 0 8',
				'g8f6' => 'r1bqkb1r/pp3ppp/2np1n2/1N2p3/4P3/4B3/PPP2PPP/RN1QKB1R w KQkq - 2 8',
			},
		},
		'r1bqkbnr/pp3ppp/2np4/1N2p3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B32n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6

			variation => N__('Sicilian: Lowenthal, Kalashnikov'),
			moves => {
				'a2a4' => 'r1bqkbnr/pp3ppp/2np4/1N2p3/P3P3/8/1PP2PPP/RNBQKB1R b KQkq a3 0 6',
				'b1c3' => 'r1bqkbnr/pp3ppp/2np4/1N2p3/4P3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6',
				'c2c4' => 'r1bqkbnr/pp3ppp/2np4/1N2p3/2P1P3/8/PP3PPP/RNBQKB1R b KQkq c3 0 6',
			},
		},
		'r1bqkbnr/pp3ppp/2np4/1N2p3/4PB2/8/PPP2PPP/RN1QKB1R w KQkq - 0 7' => {
			eco => 'B44l',
                        # TRANSLATORS:

			variation => N__('Sicilian, Taimanov, Szen, 6. Bf4'),
			moves => {
				'f4e3' => 'r1bqkbnr/pp3ppp/2np4/1N2p3/4P3/4B3/PPP2PPP/RN1QKB1R b KQkq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2np4/1N2p3/P3P3/8/1PP2PPP/RNBQKB1R b KQkq a3 0 6' => {
			eco => 'B32o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. a4

			variation => N__('Sicilian: Lowenthal, Kalashnikov, 6. a4'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/1N6/2P1P3/8/PP3PPP/RNBQKB1R b KQkq c3 0 6' => {
			eco => 'B44o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4

			variation => N__('Sicilian, Taimanov, Szen, 6. c4'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p3ppp/p1npp3/1N6/2P1P3/8/PP3PPP/RNBQKB1R w KQkq - 0 7',
				'g8f6' => 'r1bqkb1r/pp3ppp/2nppn2/1N6/2P1P3/8/PP3PPP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/1N6/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B44k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6

			variation => N__('Sicilian, Taimanov, Szen, 5... d6'),
			moves => {
				'c1f4' => 'r1bqkbnr/pp3ppp/2npp3/1N6/4PB2/8/PPP2PPP/RN1QKB1R b KQkq - 1 6',
				'c2c4' => 'r1bqkbnr/pp3ppp/2npp3/1N6/2P1P3/8/PP3PPP/RNBQKB1R b KQkq c3 0 6',
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/1N6/4PB2/8/PPP2PPP/RN1QKB1R b KQkq - 1 6' => {
			eco => 'B44l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. Bf4

			variation => N__('Sicilian, Taimanov, Szen, 6. Bf4'),
			moves => {
				'e6e5' => 'r1bqkbnr/pp3ppp/2np4/1N2p3/4PB2/8/PPP2PPP/RN1QKB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/8/2B1P3/2N2N2/PP3PPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'B21y',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4 e6 7. O-O

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6 7.O-O'),
			moves => {
				'a7a6' => 'r1bqkbnr/1p3ppp/p1npp3/8/2B1P3/2N2N2/PP3PPP/R1BQ1RK1 w kq - 0 8',
				'g8f6' => 'r1bqkb1r/pp3ppp/2nppn2/8/2B1P3/2N2N2/PP3PPP/R1BQ1RK1 w kq - 2 8',
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/8/2B1P3/2N2N2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B21y',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 Nc6 5. Nf3 d6 6. Bc4 e6

			variation => N__('Sicilian: Smith-Morra, 5. Nf3 d6 6. Bc4 e6'),
			moves => {
				'e1g1' => 'r1bqkbnr/pp3ppp/2npp3/8/2B1P3/2N2N2/PP3PPP/R1BQ1RK1 b kq - 1 7',
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R b KQkq g3 0 6' => {
			eco => 'B45c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 d6 6. g4

			variation => N__('Sicilian: Taimanov, 5. Nc3 d6 6. g4'),
			moves => {
			},
		},
		'r1bqkbnr/pp3ppp/2npp3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B45c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 d6

			variation => N__('Sicilian: Taimanov, 5. Nc3 d6'),
			moves => {
				'g2g4' => 'r1bqkbnr/pp3ppp/2npp3/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R b KQkq g3 0 6',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3P4/2p5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D07t',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 dxc4 4. d5

			variation => N__('QGD: Chigorin, 3. Nc3 dxc4 4. d5'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D07c',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. cxd5

			variation => N__('QGD: Chigorin, 3.cxd5'),
			moves => {
				'd8d5' => 'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B00v',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. exd5

			variation => N__('Nimzowitsch Defence: 2. d4 d5 3.exd5'),
			moves => {
				'c6b4' => 'r1bqkbnr/ppp1pppp/8/3P4/1n1P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
				'd8d5' => 'r1b1kbnr/ppp1pppp/2n5/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'D07n',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3

			variation => N__('QGD: Chigorin, 3. Nc3'),
			moves => {
				'd5c4' => 'r1bqkbnr/ppp1pppp/2n5/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'e7e5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4',
				'g8f6' => 'r1bqkb1r/ppp1pppp/2n2n2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 3 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D07b',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. e3

			variation => N__('QGD: Chigorin, 3. e3'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'D07f',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3

			variation => N__('QGD: Chigorin, 3. Nf3'),
			moves => {
				'c8g4' => 'r2qkbnr/ppp1pppp/2n5/3p4/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 4',
				'e7e5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq e6 0 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'D07a',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6

			variation => N__('QGD: Chigorin Defence'),
			moves => {
				'b1c3' => 'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
				'c4d5' => 'r1bqkbnr/ppp1pppp/2n5/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e3' => 'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3' => {
			eco => 'D02f',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nc6 3. Bf4

			variation => N__('Queen\'s Pawn: 2. Nf3 Nc6 3. Bf4'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'D02e',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nc6

			variation => N__('Queen\'s Pawn: 2. Nf3 Nc6'),
			moves => {
				'c1f4' => 'r1bqkbnr/ppp1pppp/2n5/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3',
				'g2g3' => 'r1bqkbnr/ppp1pppp/2n5/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02g',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nc6 3. g3

			variation => N__('Queen\'s Pawn: 2. Nf3 Nc6 3. g3'),
			moves => {
				'c8g4' => 'r2qkbnr/ppp1pppp/2n5/3p4/3P2b1/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B00w',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. Nc3

			variation => N__('Nimzowitsch Defence: Bogoljubow Variation'),
			moves => {
				'd5e4' => 'r1bqkbnr/ppp1pppp/2n5/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'e7e5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 4',
				'g8f6' => 'r1bqkb1r/ppp1pppp/2n2n2/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B00v',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5

			variation => N__('Nimzowitsch Defence: 2. d4 d5'),
			moves => {
				'b1c3' => 'r1bqkbnr/ppp1pppp/2n5/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3',
				'e4d5' => 'r1bqkbnr/ppp1pppp/2n5/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
				'e4e5' => 'r1bqkbnr/ppp1pppp/2n5/3pP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nc6

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'r1bqkbnr/ppp1pppp/2n5/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'e7e5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/8/5NP1/PPPPPPBP/RNBQK2R w KQkq e6 0 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/3pP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B00x',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. e5

			variation => N__('Nimzowitsch Defence: 2. d4 d5 3. e5'),
			moves => {
				'c8f5' => 'r2qkbnr/ppp1pppp/2n5/3pPb2/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2PP4/5N2/PP1P1PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A09f',
                        # TRANSLATORS:

			variation => N__('Reti: Advance, 3. e3 Nc6'),
			moves => {
				'c6d4' => 'r1bqkbnr/ppp1pppp/8/8/2Pn4/5N2/PP1P1PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2Pp4/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A09f',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. e3 Nc6

			variation => N__('Reti: Advance, 3. e3 Nc6'),
			moves => {
				'e3d4' => 'r1bqkbnr/ppp1pppp/2n5/8/2PP4/5N2/PP1P1PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2Pp4/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A09h',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 Nc6

			variation => N__('Reti: Advance, 3. g3 Nc6'),
			moves => {
				'f1g2' => 'r1bqkbnr/ppp1pppp/2n5/8/2Pp4/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2Pp4/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A09h',
                        # TRANSLATORS:

			variation => N__('Reti: Advance, 3. g3 Nc6'),
			moves => {
				'e7e5' => 'r1bqkbnr/ppp2ppp/2n5/4p3/2Pp4/5NP1/PP1PPPBP/RNBQK2R w KQkq e6 0 5',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D07u',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 dxc4 4. Nf3

			variation => N__('QGD: Chigorin, 3. Nc3 dxc4 4. Nf3'),
			moves => {
				'g8f6' => 'r1bqkb1r/ppp1pppp/2n2n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D07s',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 dxc4

			variation => N__('QGD: Chigorin, 3. Nc3 dxc4'),
			moves => {
				'd4d5' => 'r1bqkbnr/ppp1pppp/2n5/3P4/2p5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'r1bqkbnr/ppp1pppp/2n5/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2pPP3/4B3/PP3PPP/RN1QKBNR b KQkq - 2 4' => {
			eco => 'D20n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nc6 4. Be3

			variation => N__('QGA: 3. e4 Nc6 4. Be3'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'D20m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nc6 4. Nf3

			variation => N__('QGA: 3. e4 Nc6 4. Nf3'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/2pPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'D20m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nc6

			variation => N__('QGA: 3. e4 Nc6'),
			moves => {
				'c1e3' => 'r1bqkbnr/ppp1pppp/2n5/8/2pPP3/4B3/PP3PPP/RN1QKBNR b KQkq - 2 4',
				'g1f3' => 'r1bqkbnr/ppp1pppp/2n5/8/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2n5/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B00w',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. Nc3 dxe4

			variation => N__('Nimzowitsch Defence: Bogoljubow, 3...dxe4'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2np4/8/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B00t',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d6 3. Nc3

			variation => N__('Nimzowitsch Defence: 2. d4 d6 3. Nc3'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/2np4/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'B00o',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4'),
			moves => {
				'c8g4' => 'r2qkbnr/ppp1pppp/2np4/8/3PP1b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
				'g8f6' => 'r1bqkb1r/ppp1pppp/2np1n2/8/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'r1bqkbnr/ppp1pppp/2np4/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B00t',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d6

			variation => N__('Nimzowitsch Defence: 2. d4 d6'),
			moves => {
				'b1c3' => 'r1bqkbnr/ppp1pppp/2np4/8/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3',
			},
		},
		'r1bqkbnr/ppp1pppp/2np4/8/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'B00o',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6'),
			moves => {
				'd2d4' => 'r1bqkbnr/ppp1pppp/2np4/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
			},
		},
		'r1bqkbnr/ppp1pppp/8/3P4/1n1P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B00v',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. exd5 Nb4

			variation => N__('Nimzowitsch Defence: Aachen Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/ppp1pppp/8/8/2Pn4/5N2/PP1P1PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A09f',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. e3 Nc6 4. exd4 Nxd4

			variation => N__('Reti: Advance, 3. e3 Nc6 4.exd4 Nxd4'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2p1p/2n5/3P4/2BP1p2/2N2p2/PPP3PP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'c8g4' => 'r2qkbnr/ppp2p1p/2n5/3P4/2BP1pb1/2N2p2/PPP3PP/R1BQ1RK1 w kq - 1 9',
			},
		},
		'r1bqkbnr/ppp2p1p/2n5/3p4/2BPPp2/2N2p2/PPP3PP/R1BQ1RK1 w kq d6 0 8' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'e4d5' => 'r1bqkbnr/ppp2p1p/2n5/3P4/2BP1p2/2N2p2/PPP3PP/R1BQ1RK1 b kq - 0 8',
			},
		},
		'r1bqkbnr/ppp2p1p/2np4/6N1/4PppP/2N5/PPPP2P1/R1BQKB1R w KQkq - 0 7' => {
			eco => 'C25q',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5 5. h4 g4 6. Ng5 d6

			variation => N__('Vienna: Hamppe-Allgaier Gambit, Alapin Variation'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2Bp4/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. Bxc6+

			variation => N__('Spanish: Old Steinitz, 4.Bxc6+'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n1p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D31a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nc6

			variation => N__('QGD: 3. Nc3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'C10c',
                        # TRANSLATORS:

			variation => N__('French: 3. Nc3 Nc6'),
			moves => {
				'g8f6' => 'r1bqkb1r/ppp2ppp/2n1pn2/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C10c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nc6

			variation => N__('French: 3. Nc3 Nc6'),
			moves => {
				'g1f3' => 'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C03m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. c3

			variation => N__('French: Tarrasch, Guimard, 4. c3'),
			moves => {
				'e6e5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/3PP3/2P5/PP1N1PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'C03o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. Ngf3

			variation => N__('French: Tarrasch, Guimard, 4.Ngf3'),
			moves => {
				'g8f6' => 'r1bqkb1r/ppp2ppp/2n1pn2/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C03l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6

			variation => N__('French: Tarrasch, Guimard Variation'),
			moves => {
				'c2c3' => 'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/1B1pp3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq d6 0 4' => {
			eco => 'C60a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d5

			variation => N__('Spanish: Spanish Countergambit'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3p4/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'C01i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Bd3 Nc6

			variation => N__('French: Exchange, 4. Bd3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3p4/3pP3/2P2N2/PP3PPP/RNBQKB1R w KQkq d6 0 5' => {
			eco => 'C44u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 d5

			variation => N__('Scotch: Goring Gambit, 4... d5'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'D07o',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 e5

			variation => N__('QGD: Chigorin, Tartakower Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq e6 0 4' => {
			eco => 'D07g',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 e5

			variation => N__('QGD: Chigorin, Lazard Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'B00w',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. Nc3 e5

			variation => N__('Nimzowitsch Defence: Bogoljubow, 3... e5'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/3PP3/2P5/PP1N1PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C03n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nc6 4. c3 e5

			variation => N__('French: Tarrasch, Guimard, 4. c3 e5'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'C44k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 d5

			variation => N__('Ponziani: 3... d5'),
			moves => {
				'd1a4' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/4P3/3P1N2/PPP2PPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'C44d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d3 d5

			variation => N__('Open Game: Inverted Philidor'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/8/3P1NP1/PPP1PPBP/RNBQK2R b KQkq - 0 4' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'g8f6' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/8/3P1NP1/PPP1PPBP/RNBQK2R w KQkq - 1 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/8/5NP1/PPPPPPBP/RNBQK2R w KQkq e6 0 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nc6 3. Bg2 e5

			variation => N__('Reti: KIA'),
			moves => {
				'd2d3' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/8/3P1NP1/PPP1PPBP/RNBQK2R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R b KQkq - 1 4' => {
			eco => 'C44l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 d5 4. Qa4

			variation => N__('Ponziani: 3... d5 4. Qa4'),
			moves => {
				'c8d7' => 'r2qkbnr/pppb1ppp/2n5/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R w KQkq - 2 5',
				'f7f6' => 'r1bqkbnr/ppp3pp/2n2p2/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R w KQkq - 0 5',
				'g8f6' => 'r1bqkb1r/ppp2ppp/2n2n2/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 3 5' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. Nbd2

			variation => N__('QGD: Albin, Alapin Variation'),
			moves => {
				'c8g4' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5N2/PP1NPPPP/R1BQKB1R w KQkq - 4 6',
				'd8e7' => 'r1b1kbnr/ppp1qppp/2n5/4P3/2Pp4/5N2/PP1NPPPP/R1BQKB1R w KQkq - 4 6',
				'f7f6' => 'r1bqkbnr/ppp3pp/2n2p2/4P3/2Pp4/5N2/PP1NPPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6

			variation => N__('QGD: Albin, 4. Nf3 Nc6'),
			moves => {
				'a2a3' => 'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 5',
				'b1d2' => 'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 3 5',
				'g2g3' => 'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3

			variation => N__('QGD: Albin, 5. g3'),
			moves => {
				'c8e6' => 'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'c8f5' => 'r2qkbnr/ppp2ppp/2n5/4Pb2/2Pp4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'c8g4' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'g8e7' => 'r1bqkb1r/ppp1nppp/2n5/4P3/2Pp4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. a3

			variation => N__('QGD: Albin, 4. Nf3 Nc6 5. a3'),
			moves => {
				'a7a5' => 'r1bqkbnr/1pp2ppp/2n5/p3P3/2Pp4/P4N2/1P2PPPP/RNBQKB1R w KQkq a6 0 6',
				'c8e6' => 'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 6',
				'c8g4' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/4p3/2Pp4/5NP1/PP1PPPBP/RNBQK2R w KQkq e6 0 5' => {
			eco => 'A09h',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 Nc6 4. Bg2 e5

			variation => N__('Reti: Advance, 3. g3 Nc6 4. Bg2 e5'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2n5/8/2BpP3/5N2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'D20y',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 e5 4. Nf3 exd4 5. Bxc4 Nc6

			variation => N__('QGA: 3. e4 e5 4. Nf3 exd4 5.Bxc4 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4

			variation => N__('Spanish: Old Steinitz, 4. d4'),
			moves => {
				'c8d7' => 'r2qkbnr/pppb1ppp/2np4/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 5',
				'e5d4' => 'r1bqkbnr/ppp2ppp/2np4/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. c3

			variation => N__('Spanish: Old Steinitz, 4. c3'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. O-O

			variation => N__('Spanish: Old Steinitz, 4.O-O'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6

			variation => N__('Spanish: Old Steinitz'),
			moves => {
				'b5c6' => 'r1bqkbnr/ppp2ppp/2Bp4/4p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'c2c3' => 'r1bqkbnr/ppp2ppp/2np4/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkbnr/ppp2ppp/2np4/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqkbnr/ppp2ppp/2np4/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B6/3NP3/8/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 exd4 5. Nxd4

			variation => N__('Spanish: Old Steinitz, 4. d4 exd4 5.Nxd4'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 5' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 exd4 5. Qxd4

			variation => N__('Spanish: Old Steinitz, 4. d4 exd4 5.Qxd4'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 exd4

			variation => N__('Spanish: Old Steinitz, 4. d4 exd4'),
			moves => {
				'd1d4' => 'r1bqkbnr/ppp2ppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 5',
				'f3d4' => 'r1bqkbnr/ppp2ppp/2np4/1B6/3NP3/8/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C50d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 d6

			variation => N__('Italian: 3... d6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 d6

			variation => N__('English: Three Knights, 3... d6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A25',
                        # TRANSLATORS:

			variation => N__('English: Closed'),
			moves => {
				'f1g2' => 'r1bqkbnr/ppp2ppp/2np4/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 d6 4. Bg2

			variation => N__('English: Closed'),
			moves => {
				'c8e6' => 'r2qkbnr/ppp2ppp/2npb3/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'C46d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 d6 4. d4

			variation => N__('Three Knights: 3... d6 4. d4'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C44s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 d6

			variation => N__('Scotch: 3... d6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'C46c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 d6

			variation => N__('Three Knights: 3... d6'),
			moves => {
				'd2d4' => 'r1bqkbnr/ppp2ppp/2np4/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq d3 0 4',
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'C30c',
                        # TRANSLATORS: 1. e4 e5 2. f4 d6 3. Nf3 Nc6

			variation => N__('King\'s Gambit: 2... d6 3. Nf3 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/2np4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5' => {
			eco => 'C41i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Qxd4 Nc6

			variation => N__('Philidor: Morphy, 4... Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/3p4/8/3QP3/3B4/PPP2PPP/RNB1K2R b KQkq - 1 6' => {
			eco => 'C45b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nxd4 5. Qxd4 d6 6. Bd3

			variation => N__('Scotch: Ghulam Kassim Variation'),
			moves => {
			},
		},
		'r1bqkbnr/ppp2ppp/3p4/8/3QP3/8/PPP2PPP/RNB1KB1R w KQkq - 0 6' => {
			eco => 'C45b',
                        # TRANSLATORS:

			variation => N__('Scotch: 4.Nxd4 Nxd4'),
			moves => {
				'f1d3' => 'r1bqkbnr/ppp2ppp/3p4/8/3QP3/3B4/PPP2PPP/RNB1K2R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/ppp3pp/2N5/1B6/4p3/8/PPPP1PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'C63s',
                        # TRANSLATORS:

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 d5 6.Nxe5'),
			moves => {
				'd8d5' => 'r1b1kbnr/ppp3pp/2N5/1B1q4/4p3/8/PPPP1PPP/R1BQK2R w KQkq - 1 8',
				'd8g5' => 'r1b1kbnr/ppp3pp/2N5/1B4q1/4p3/8/PPPP1PPP/R1BQK2R w KQkq - 1 8',
			},
		},
		'r1bqkbnr/ppp3pp/2n2p2/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R w KQkq - 0 5' => {
			eco => 'C44m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 d5 4. Qa4 f6

			variation => N__('Ponziani: Steinitz Variation'),
			moves => {
			},
		},
		'r1bqkbnr/ppp3pp/2n2p2/4P3/2Pp4/5N2/PP1NPPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. Nbd2 f6

			variation => N__('QGD: Albin, Janowski Variation'),
			moves => {
			},
		},
		'r1bqkbnr/ppp3pp/2n5/1B1pN3/4N3/8/PPPP1PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C63s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 d5 6. Nxe5

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 d5 6.Nxe5'),
			moves => {
				'd5e4' => 'r1bqkbnr/ppp3pp/2n5/1B2N3/4p3/8/PPPP1PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/ppp3pp/2n5/1B1pp3/4N3/5N2/PPPP1PPP/R1BQK2R w KQkq d6 0 6' => {
			eco => 'C63r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4 5. Nxe4 d5

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 d5'),
			moves => {
				'f3e5' => 'r1bqkbnr/ppp3pp/2n5/1B1pN3/4N3/8/PPPP1PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/ppp3pp/2n5/1B2N3/4p3/8/PPPP1PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C63s',
                        # TRANSLATORS:

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4 5.Nxe4 d5 6.Nxe5'),
			moves => {
				'e5c6' => 'r1bqkbnr/ppp3pp/2N5/1B6/4p3/8/PPPP1PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r1bqkbnr/ppp3pp/2np4/3Pp1N1/4p3/2P5/PP3PPP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'c6b8' => 'rnbqkbnr/ppp3pp/3p4/3Pp1N1/4p3/2P5/PP3PPP/RNBQKB1R w KQkq - 2 7',
			},
		},
		'r1bqkbnr/ppp3pp/2np4/3Pp3/4p3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'f3g5' => 'r1bqkbnr/ppp3pp/2np4/3Pp1N1/4p3/2P5/PP3PPP/RNBQKB1R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/ppp3pp/2np4/3Ppp2/4P3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C44j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 f5 4. d4 d6 5. d5

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'f5e4' => 'r1bqkbnr/ppp3pp/2np4/3Pp3/4p3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/ppp3pp/2np4/4pp2/3PP3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit'),
			moves => {
				'd4d5' => 'r1bqkbnr/ppp3pp/2np4/3Ppp2/4P3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/ppp3pp/2p5/4pp2/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C63h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Bxc6 dxc6

			variation => N__('Spanish: Schliemann, 4.Bxc6 dxc6'),
			moves => {
			},
		},
		'r1bqkbnr/pppn1ppp/3p4/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'C41f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7 4. Bc4

			variation => N__('Philidor: Hanham, 4. Bc4'),
			moves => {
				'c7c6' => 'r1bqkbnr/pp1n1ppp/2pp4/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppn1ppp/3p4/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C41f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nd7

			variation => N__('Philidor: Hanham Variation'),
			moves => {
				'f1c4' => 'r1bqkbnr/pppn1ppp/3p4/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq - 2 4',
			},
		},
		'r1bqkbnr/pppn1ppp/4p3/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D30g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nd7

			variation => N__('QGD: 3. Nf3 Nbd7 (Westphalia)'),
			moves => {
			},
		},
		'r1bqkbnr/pppn1ppp/4p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'C10h',
                        # TRANSLATORS:

			variation => N__('French: Rubinstein, 4... Nd7'),
			moves => {
				'f8e7' => 'r1bqk1nr/pppnbppp/4p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'g8f6' => 'r1bqkb1r/pppn1ppp/4pn2/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'r1bqkbnr/pppn1ppp/4p3/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C10h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nd7

			variation => N__('French: Rubinstein, 4... Nd7'),
			moves => {
				'g1f3' => 'r1bqkbnr/pppn1ppp/4p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'r1bqkbnr/pppnpppp/3p4/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nd7

			variation => N__('Pirc: 2. d4 Nd7'),
			moves => {
			},
		},
		'r1bqkbnr/pppnpppp/8/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D21b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nd7

			variation => N__('QGA: 3. Nf3 Nd7'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C60l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 g6 4. d4

			variation => N__('Spanish: Fianchetto, 4. d4'),
			moves => {
				'e5d4' => 'r1bqkbnr/pppp1p1p/2n3p1/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C60n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 g6 4. c3

			variation => N__('Spanish: Fianchetto, 4. c3'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4' => {
			eco => 'C60k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 g6 4. O-O

			variation => N__('Spanish: Fianchetto, 4.O-O'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C60j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 g6

			variation => N__('Spanish: Fianchetto (Smyslov) Defence'),
			moves => {
				'c2c3' => 'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/4P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e1g1' => 'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 4',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/1B4B1/3pP3/5N2/PPP2PPP/RN1QK2R b KQkq - 1 5' => {
			eco => 'C60m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 g6 4. d4 exd4 5. Bg5

			variation => N__('Spanish: Fianchetto, 4. d4 exd4 5. Bg5'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/1B6/3pP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C60l',
                        # TRANSLATORS:

			variation => N__('Spanish: Fianchetto, 4. d4'),
			moves => {
				'c1g5' => 'r1bqkbnr/pppp1p1p/2n3p1/1B4B1/3pP3/5N2/PPP2PPP/RN1QK2R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/3N4/3pP3/5N2/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C46k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6 4. d4 exd4 5. Nd5

			variation => N__('Three Knights: Steinitz, Rosenthal Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 g6

			variation => N__('English: Three Knights, 3... g6'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp1p1p/2n3p1/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6

			variation => N__('English: Closed'),
			moves => {
				'f1g2' => 'r1bqkbnr/pppp1p1p/2n3p1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2

			variation => N__('English: Closed'),
			moves => {
				'f8g7' => 'r1bqk1nr/pppp1pbp/2n3p1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 g6 4. d4

			variation => N__('English: Three Knights, 3... g6 4. d4'),
			moves => {
				'e5d4' => 'r1bqkbnr/pppp1p1p/2n3p1/8/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'C46j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6 4. d4

			variation => N__('Three Knights: Steinitz, 4. d4'),
			moves => {
				'e5d4' => 'r1bqkbnr/pppp1p1p/2n3p1/8/3pP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'C46i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6

			variation => N__('Three Knights: Steinitz Variation'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp1p1p/2n3p1/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq d3 0 4',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 g6 4. d4 exd4 5. Nxd4

			variation => N__('English: Three Knights, 3... g6 4. d4 exd4 5.Nxd4'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/8/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 g6 4. d4 exd4

			variation => N__('English: Three Knights, 3... g6 4. d4 exd4'),
			moves => {
				'f3d4' => 'r1bqkbnr/pppp1p1p/2n3p1/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C46l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 g6 4. d4 exd4 5. Nxd4

			variation => N__('Three Knights: Steinitz, 4. d4 exd4 5.Nxd4'),
			moves => {
				'f8g7' => 'r1bqk1nr/pppp1pbp/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pppp1p1p/2n3p1/8/3pP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C46j',
                        # TRANSLATORS:

			variation => N__('Three Knights: Steinitz, 4. d4'),
			moves => {
				'c3d5' => 'r1bqkbnr/pppp1p1p/2n3p1/3N4/3pP3/5N2/PPP2PPP/R1BQKB1R b KQkq - 1 5',
				'f3d4' => 'r1bqkbnr/pppp1p1p/2n3p1/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/1B2p1p1/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq g6 0 4' => {
			eco => 'C60g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 g5

			variation => N__('Spanish: Brentano Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Nc6

			variation => N__('KGA: Kieseritsky, Neumann Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/6N1/4PppP/2N5/PPPP2P1/R1BQKB1R b KQkq - 1 6' => {
			eco => 'C25q',
                        # TRANSLATORS:

			variation => N__('Vienna: Hamppe-Allgaier Gambit'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp2p1p/2np4/6N1/4PppP/2N5/PPPP2P1/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/6p1/2B1Pp2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C25p',
                        # TRANSLATORS:

			variation => N__('Vienna: 2... Nc6 3. f4 exf4 4. Nf3 g5'),
			moves => {
				'g5g4' => 'r1bqkbnr/pppp1p1p/2n5/8/2B1Ppp1/2N2N2/PPPP2PP/R1BQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/6p1/2B1Pp2/5N2/PPPP2PP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Nc6

			variation => N__('KGA: Blachly Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/6p1/3PPp2/2N2N2/PPP3PP/R1BQKB1R b KQkq d3 0 5' => {
			eco => 'C25r',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5 5. d4

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'g5g4' => 'r1bqkbnr/pppp1p1p/2n5/8/3PPpp1/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/6p1/4Pp1P/2N2N2/PPPP2P1/R1BQKB1R b KQkq h3 0 5' => {
			eco => 'C25q',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5 5. h4

			variation => N__('Vienna: Hamppe-Allgaier Gambit'),
			moves => {
				'g5g4' => 'r1bqkbnr/pppp1p1p/2n5/8/4PppP/2N2N2/PPPP2P1/R1BQKB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/6p1/4Pp2/2N2N2/PPPP2PP/R1BQKB1R w KQkq g6 0 5' => {
			eco => 'C25p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5

			variation => N__('Vienna: 2... Nc6 3. f4 exf4 4. Nf3 g5'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp1p1p/2n5/6p1/3PPp2/2N2N2/PPP3PP/R1BQKB1R b KQkq d3 0 5',
				'f1c4' => 'r1bqkbnr/pppp1p1p/2n5/6p1/2B1Pp2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 1 5',
				'h2h4' => 'r1bqkbnr/pppp1p1p/2n5/6p1/4Pp1P/2N2N2/PPPP2P1/R1BQKB1R b KQkq h3 0 5',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2B1Pp2/2N2Q2/PPPP2PP/R1B2RK1 b kq - 0 7' => {
			eco => 'C25p',
                        # TRANSLATORS:

			variation => N__('Vienna: Hamppe-Muzio Gambit'),
			moves => {
				'c6e5' => 'r1bqkbnr/pppp1p1p/8/4n3/2B1Pp2/2N2Q2/PPPP2PP/R1B2RK1 w kq - 1 8',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2B1Pp2/2N2p2/PPPP2PP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'C25p',
                        # TRANSLATORS:

			variation => N__('Vienna: Hamppe-Muzio Gambit'),
			moves => {
				'd1f3' => 'r1bqkbnr/pppp1p1p/2n5/8/2B1Pp2/2N2Q2/PPPP2PP/R1B2RK1 b kq - 0 7',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 w kq - 1 7' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O gxf3 6. Qxf3 Nc6

			variation => N__('KGA: Muzio Gambit, Holloway Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2B1Ppp1/2N2N2/PPPP2PP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'C25p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5 5. Bc4 g4 6. O-O

			variation => N__('Vienna: Hamppe-Muzio Gambit'),
			moves => {
				'g4f3' => 'r1bqkbnr/pppp1p1p/2n5/8/2B1Pp2/2N2p2/PPPP2PP/R1BQ1RK1 w kq - 0 7',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2B1Ppp1/2N2N2/PPPP2PP/R1BQK2R w KQkq - 0 6' => {
			eco => 'C25p',
                        # TRANSLATORS:

			variation => N__('Vienna: 2... Nc6 3. f4 exf4 4. Nf3 g5'),
			moves => {
				'e1g1' => 'r1bqkbnr/pppp1p1p/2n5/8/2B1Ppp1/2N2N2/PPPP2PP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2BPPp2/2N2p2/PPP3PP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'd7d5' => 'r1bqkbnr/ppp2p1p/2n5/3p4/2BPPp2/2N2p2/PPP3PP/R1BQ1RK1 w kq d6 0 8',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2BPPp2/2N2p2/PPP3PP/R1BQK2R w KQkq - 0 7' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'e1g1' => 'r1bqkbnr/pppp1p1p/2n5/8/2BPPp2/2N2p2/PPP3PP/R1BQ1RK1 b kq - 1 7',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/2BPPpp1/2N2N2/PPP3PP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'g4f3' => 'r1bqkbnr/pppp1p1p/2n5/8/2BPPp2/2N2p2/PPP3PP/R1BQK2R w KQkq - 0 7',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/3PPpp1/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'f1c4' => 'r1bqkbnr/pppp1p1p/2n5/8/2BPPpp1/2N2N2/PPP3PP/R1BQK2R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/pppp1p1p/2n5/8/4PppP/2N2N2/PPPP2P1/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C25q',
                        # TRANSLATORS:

			variation => N__('Vienna: Hamppe-Allgaier Gambit'),
			moves => {
				'f3g5' => 'r1bqkbnr/pppp1p1p/2n5/6N1/4PppP/2N5/PPPP2P1/R1BQKB1R b KQkq - 1 6',
			},
		},
		'r1bqkbnr/pppp1p1p/8/4n3/2B1PQ2/2N5/PPPP2PP/R1B2RK1 b kq - 0 8' => {
			eco => 'C25p',
                        # TRANSLATORS:

			variation => N__('Vienna: Hamppe-Muzio Gambit'),
			moves => {
				'd8f6' => 'r1b1kbnr/pppp1p1p/5q2/4n3/2B1PQ2/2N5/PPPP2PP/R1B2RK1 w kq - 1 9',
			},
		},
		'r1bqkbnr/pppp1p1p/8/4n3/2B1Pp2/2N2Q2/PPPP2PP/R1B2RK1 w kq - 1 8' => {
			eco => 'C25p',
                        # TRANSLATORS:

			variation => N__('Vienna: Hamppe-Muzio Gambit'),
			moves => {
				'f3f4' => 'r1bqkbnr/pppp1p1p/8/4n3/2B1PQ2/2N5/PPPP2PP/R1B2RK1 b kq - 0 8',
			},
		},
		'r1bqkbnr/pppp1ppp/2n1p3/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'B00m',
                        # TRANSLATORS: 1. e4 Nc6 2. Nc3 e6

			variation => N__('Nimzowitsch Defence: 2. Nc3 e6'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3' => {
			eco => 'C60a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5

			variation => N__('Spanish (Ruy Lopez)'),
			moves => {
				'a7a5' => 'r1bqkbnr/1ppp1ppp/2n5/pB2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq a6 0 4',
				'a7a6' => 'r1bqkbnr/1ppp1ppp/p1n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'c6a5' => 'r1bqkbnr/pppp1ppp/8/nB2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'c6d4' => 'r1bqkbnr/pppp1ppp/8/1B2p3/3nP3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'd7d5' => 'r1bqkbnr/ppp2ppp/2n5/1B1pp3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq d6 0 4',
				'd7d6' => 'r1bqkbnr/ppp2ppp/2np4/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'd8e7' => 'r1b1kbnr/ppppqppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 4',
				'f7f6' => 'r1bqkbnr/pppp2pp/2n2p2/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/1B2p3/1b2P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/1Bb1p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'f8e7' => 'r1bqk1nr/ppppbppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'g7g5' => 'r1bqkbnr/pppp1p1p/2n5/1B2p1p1/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq g6 0 4',
				'g7g6' => 'r1bqkbnr/pppp1p1p/2n3p1/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'g8e7' => 'r1bqkb1r/ppppnppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Bb5 Nc6

			variation => N__('Open Game: Portuguese, 2... Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/1B6/3pP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C44t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bb5

			variation => N__('Scotch: Relfsson Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/3Pp3/4P3/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B00u',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 e5 3. d5

			variation => N__('Nimzowitsch Defence: 2. d4 e5 3. d5'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4N3/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C44a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nxe5

			variation => N__('Open Game: Irish (Chicago) Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4P3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A40k',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. c4'),
			moves => {
				'c6e5' => 'r1bqkbnr/pppp1ppp/8/4n3/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4P3/4P3/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B00u',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 e5 3. dxe5

			variation => N__('Nimzowitsch Defence: 2. d4 e5 3.dxe5'),
			moves => {
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b1P3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4P3/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 Nc6 3. Nf3

			variation => N__('Englund Gambit: 2.dxe5 Nc6 3. Nf3'),
			moves => {
				'd8e7' => 'r1b1kbnr/ppppqppp/2n5/4P3/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 3 4',
				'f7f6' => 'r1bqkbnr/pppp2pp/2n2p2/4P3/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4',
				'g8e7' => 'r1bqkb1r/ppppnppp/2n5/4P3/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 3 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4P3/8/8/PPP1PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 Nc6

			variation => N__('Englund Gambit: 2.dxe5 Nc6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pppp1ppp/2n5/4P3/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4P3/8/8/PPPPP1PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 Nc6

			variation => N__('Bird: From Gambit, Schlecter'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/1P2P3/5N2/P1PP1PPP/RNBQKB1R b KQkq b3 0 3' => {
			eco => 'C44a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. b4

			variation => N__('Open Game: Paschman Wing Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3' => {
			eco => 'C25j',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. Bc4

			variation => N__('Vienna: 2... Nc6 3. Bc4'),
			moves => {
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2B1P3/2P2N2/P2P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bf8

			variation => N__('Evans Gambit: Mayet Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3' => {
			eco => 'C50a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4

			variation => N__('Italian Game'),
			moves => {
				'c6d4' => 'r1bqkbnr/pppp1ppp/8/4p3/2BnP3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'd7d6' => 'r1bqkbnr/ppp2ppp/2np4/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 4',
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'f8e7' => 'r1bqk1nr/ppppbppp/2n5/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3' => {
			eco => 'C44a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c4

			variation => N__('Open Game: Dresden Opening'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N1P3/PP1P1PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. e3

			variation => N__('English: Closed'),
			moves => {
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N1P3/PP1P1PPP/R1BQKBNR w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3

			variation => N__('English: Three Knights'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp2ppp/2np4/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/4pp2/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq f6 0 4',
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/4p3/1bP5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 4 4',
				'g7g6' => 'r1bqkbnr/pppp1p1p/2n3p1/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3

			variation => N__('English: Closed'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp2ppp/2np4/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/4pp2/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq f6 0 4',
				'g7g6' => 'r1bqkbnr/pppp1p1p/2n3p1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6

			variation => N__('English: Closed'),
			moves => {
				'e2e3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N1P3/PP1P1PPP/R1BQKBNR b KQkq - 0 3',
				'g1f3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3',
				'g2g3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. Nf3 Nc6

			variation => N__('English: King\'s, Nimzowitsch, 2... Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nc6

			variation => N__('English: King\'s, 2. g3 Nc6'),
			moves => {
				'f1g2' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nc6 3. Bg2

			variation => N__('English: King\'s, 2. g3 Nc6 3. Bg2'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'A40k',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. c4'),
			moves => {
				'd4e5' => 'r1bqkbnr/pppp1ppp/2n5/4P3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq d3 0 3' => {
			eco => 'C25g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. d4

			variation => N__('Vienna: Fyfe Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'C44r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4

			variation => N__('Scotch Opening'),
			moves => {
				'c6d4' => 'r1bqkbnr/pppp1ppp/8/4p3/3nP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'd7d6' => 'r1bqkbnr/ppp2ppp/2np4/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'e5d4' => 'r1bqkbnr/pppp1ppp/2n5/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/3PP3/8/PPP2PPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'B00u',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 e5

			variation => N__('Nimzowitsch Defence: 2. d4 e5'),
			moves => {
				'd4d5' => 'r1bqkbnr/pppp1ppp/2n5/3Pp3/4P3/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
				'd4e5' => 'r1bqkbnr/pppp1ppp/2n5/4P3/4P3/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'C46a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3

			variation => N__('Three Knights Game'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp2ppp/2np4/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4',
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/4pp2/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq f6 0 4',
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/4p3/1b2P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4',
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4',
				'g7g6' => 'r1bqkbnr/pppp1p1p/2n3p1/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'C25h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. g3

			variation => N__('Vienna: Paulsen Variation'),
			moves => {
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'C25g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6

			variation => N__('Vienna: 2... Nc6'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq d3 0 3',
				'f1c4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3',
				'f2f4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
				'g2g3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C44i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3

			variation => N__('Ponziani Opening'),
			moves => {
				'd7d5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq d6 0 4',
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/4pp2/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq f6 0 4',
				'f8e7' => 'r1bqk1nr/ppppbppp/2n5/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
				'g8e7' => 'r1bqkb1r/ppppnppp/2n5/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C44d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d3

			variation => N__('Open Game: Inverted Philidor'),
			moves => {
				'd7d5' => 'r1bqkbnr/ppp2ppp/2n5/3pp3/4P3/3P1N2/PPP2PPP/RNBQKB1R w KQkq d6 0 4',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/3P1N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'C44a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6

			variation => N__('Open Game'),
			moves => {
				'b1c3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
				'b2b4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/1P2P3/5N2/P1PP1PPP/RNBQKB1R b KQkq b3 0 3',
				'c2c3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'c2c4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3',
				'd2d3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'd2d4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'f1b5' => 'r1bqkbnr/pppp1ppp/2n5/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3',
				'f1c4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3',
				'f1e2' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/5N2/PPPPBPPP/RNBQK2R b KQkq - 3 3',
				'f3e5' => 'r1bqkbnr/pppp1ppp/2n5/4N3/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/5N2/PPPPBPPP/RNBQK2R b KQkq - 3 3' => {
			eco => 'C44c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Be2

			variation => N__('Open Game: Inverted Hungarian'),
			moves => {
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/5N2/PPPPBPPP/RNBQK2R w KQkq - 4 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C44b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. g3

			variation => N__('Open Game: Konstantinopolsky'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'C25m',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4

			variation => N__('Vienna: 2... Nc6 3. f4'),
			moves => {
				'e5f4' => 'r1bqkbnr/pppp1ppp/2n5/8/4Pp2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'C30g',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: 2... Nc6'),
			moves => {
				'f7f5' => 'r1bqkbnr/pppp2pp/2n5/4pp2/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq f6 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'C30g',
                        # TRANSLATORS: 1. e4 e5 2. f4 Nc6

			variation => N__('King\'s Gambit: 2... Nc6'),
			moves => {
				'g1f3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/5P2/1P6/PBPPP1PP/RN1QKBNR b KQkq f3 0 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 e5 2. Bb2 Nc6 3. f4

			variation => N__('Nimzowitsch-Larsen: Paschmann Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/8/1P2P3/PBPP1PPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 e5 2. Bb2 Nc6 3. e3

			variation => N__('Nimzowitsch-Larsen: 1... e5 2. Bb2 Nc6 3. e3'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/4p3/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 2 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 e5 2. Bb2 Nc6

			variation => N__('Nimzowitsch-Larsen: 1... e5 2. Bb2 Nc6'),
			moves => {
				'e2e3' => 'r1bqkbnr/pppp1ppp/2n5/4p3/8/1P2P3/PBPP1PPP/RN1QKBNR b KQkq - 0 3',
				'f2f4' => 'r1bqkbnr/pppp1ppp/2n5/4p3/5P2/1P6/PBPPP1PP/RN1QKBNR b KQkq f3 0 3',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/2B1P3/2p2N2/PP3PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Bc4

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5. Bc4'),
			moves => {
				'c3b2' => 'r1bqkbnr/pppp1ppp/2n5/8/2B1P3/5N2/Pp3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/2B1P3/5N2/PB3PPP/RN1QK2R b KQkq - 0 6' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Bc4 cxb2 6. Bxb2

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5. Bc4 cxb2 6.Bxb2'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/2B1P3/5N2/Pp3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C44w',
                        # TRANSLATORS:

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5. Bc4'),
			moves => {
				'c1b2' => 'r1bqkbnr/pppp1ppp/2n5/8/2B1P3/5N2/PB3PPP/RN1QK2R b KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Nc6

			variation => N__('KGA: Bishop\'s Gambit, Maurian Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/2BpP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C44x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4

			variation => N__('Scotch Gambit'),
			moves => {
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/8/1bBpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 2 5',
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b5/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/2Q1P3/8/PPP2PPP/RNB1KBNR b KQkq - 2 4' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4 Nc6 4. Qc4

			variation => N__('Centre Game: Hall Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C45a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4

			variation => N__('Scotch: 4.Nxd4'),
			moves => {
				'c6d4' => 'r1bqkbnr/pppp1ppp/8/8/3nP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'd8f6' => 'r1b1kbnr/pppp1ppp/2n2q2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'd8h4' => 'r1b1kbnr/pppp1ppp/2n5/8/3NP2q/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/8/1b1NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'f8c5' => 'r1bqk1nr/pppp1ppp/2n5/2b5/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/3PPp2/2N5/PPP3PP/R1BQKBNR b KQkq d3 0 4' => {
			eco => 'C25n',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. d4

			variation => N__('Vienna: Steinitz Gambit'),
			moves => {
				'd8h4' => 'r1b1kbnr/pppp1ppp/2n5/8/3PPp1q/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/3QP3/8/PPP2PPP/RNB1KBNR w KQkq - 1 4' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4 Nc6

			variation => N__('Centre Game'),
			moves => {
				'd4c4' => 'r1bqkbnr/pppp1ppp/2n5/8/2Q1P3/8/PPP2PPP/RNB1KBNR b KQkq - 2 4',
				'd4e3' => 'r1bqkbnr/pppp1ppp/2n5/8/4P3/4Q3/PPP2PPP/RNB1KBNR b KQkq - 2 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/3pP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C44t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3

			variation => N__('Scotch: Goring Gambit'),
			moves => {
				'd4c3' => 'r1bqkbnr/pppp1ppp/2n5/8/4P3/2p2N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'd4d3' => 'r1bqkbnr/pppp1ppp/2n5/8/4P3/2Pp1N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'd7d5' => 'r1bqkbnr/ppp2ppp/2n5/3p4/3pP3/2P2N2/PP3PPP/RNBQKB1R w KQkq d6 0 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C44t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4

			variation => N__('Scotch: 3...exd4'),
			moves => {
				'c2c3' => 'r1bqkbnr/pppp1ppp/2n5/8/3pP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'f1b5' => 'r1bqkbnr/pppp1ppp/2n5/1B6/3pP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
				'f1c4' => 'r1bqkbnr/pppp1ppp/2n5/8/2BpP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
				'f3d4' => 'r1bqkbnr/pppp1ppp/2n5/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3 5. Nxc3

			variation => N__('Scotch: Goring Gambit, 4...dxc3 5.Nxc3'),
			moves => {
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/4P3/2Pp1N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C44v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 d3

			variation => N__('Scotch: Goring Gambit, 4... d3'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/4P3/2p2N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C44w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. c3 dxc3

			variation => N__('Scotch: Goring Gambit, 4...dxc3'),
			moves => {
				'b1c3' => 'r1bqkbnr/pppp1ppp/2n5/8/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'f1c4' => 'r1bqkbnr/pppp1ppp/2n5/8/2B1P3/2p2N2/PP3PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/4P3/4Q3/PPP2PPP/RNB1KBNR b KQkq - 2 4' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4 Nc6 4. Qe3

			variation => N__('Centre Game: Paulsen Attack'),
			moves => {
				'f8b4' => 'r1bqk1nr/pppp1ppp/2n5/8/1b2P3/4Q3/PPP2PPP/RNB1KBNR w KQkq - 3 5',
				'g8f6' => 'r1bqkb1r/pppp1ppp/2n2n2/8/4P3/4Q3/PPP2PPP/RNB1KBNR w KQkq - 3 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/4Pp2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C25p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3

			variation => N__('Vienna: 2... Nc6 3. f4 exf4 4. Nf3'),
			moves => {
				'g7g5' => 'r1bqkbnr/pppp1p1p/2n5/6p1/4Pp2/2N2N2/PPPP2PP/R1BQKB1R w KQkq g6 0 5',
			},
		},
		'r1bqkbnr/pppp1ppp/2n5/8/4Pp2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C25m',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4

			variation => N__('Vienna: 2... Nc6 3. f4'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp1ppp/2n5/8/3PPp2/2N5/PPP3PP/R1BQKBNR b KQkq d3 0 4',
				'g1f3' => 'r1bqkbnr/pppp1ppp/2n5/8/4Pp2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/4n3/4N3/2B1P3/8/PPP2PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'C44r',
                        # TRANSLATORS:

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'c7c6' => 'r1bqkbnr/pp1p1ppp/2p1n3/4N3/2B1P3/8/PPP2PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1ppp/4n3/4N3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C44r',
                        # TRANSLATORS:

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'f1c4' => 'r1bqkbnr/pppp1ppp/4n3/4N3/2B1P3/8/PPP2PPP/RNBQK2R b KQkq - 2 5',
			},
		},
		'r1bqkbnr/pppp1ppp/8/1B2p3/3NP3/8/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C61c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4

			variation => N__('Spanish: Bird\'s, 4.Nxd4'),
			moves => {
				'e5d4' => 'r1bqkbnr/pppp1ppp/8/1B6/3pP3/8/PPPP1PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1ppp/8/1B2p3/3nP3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C61a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4

			variation => N__('Spanish: Bird\'s Defence'),
			moves => {
				'b5c4' => 'r1bqkbnr/pppp1ppp/8/4p3/2BnP3/5N2/PPPP1PPP/RNBQK2R b KQkq - 5 4',
				'f3d4' => 'r1bqkbnr/pppp1ppp/8/1B2p3/3NP3/8/PPPP1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/8/1B6/3pP3/3P4/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C61d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. d3

			variation => N__('Spanish: Bird\'s, 5. d3'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/8/1B6/3pP3/8/PPPP1PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C61f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. O-O

			variation => N__('Spanish: Bird\'s, 5.O-O'),
			moves => {
				'c7c6' => 'r1bqkbnr/pp1p1ppp/2p5/1B6/3pP3/8/PPPP1PPP/RNBQ1RK1 w kq - 0 6',
				'f8c5' => 'r1bqk1nr/pppp1ppp/8/1Bb5/3pP3/8/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
				'g8e7' => 'r1bqkb1r/ppppnppp/8/1B6/3pP3/8/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'r1bqkbnr/pppp1ppp/8/1B6/3pP3/8/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C61c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4

			variation => N__('Spanish: Bird\'s, 4.Nxd4 exd4'),
			moves => {
				'b5c4' => 'r1bqkbnr/pppp1ppp/8/8/2BpP3/8/PPPP1PPP/RNBQK2R b KQkq - 1 5',
				'd2d3' => 'r1bqkbnr/pppp1ppp/8/1B6/3pP3/3P4/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'e1g1' => 'r1bqkbnr/pppp1ppp/8/1B6/3pP3/8/PPPP1PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'r1bqkbnr/pppp1ppp/8/4N3/3nP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C44r',
                        # TRANSLATORS:

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'd4e6' => 'r1bqkbnr/pppp1ppp/4n3/4N3/4P3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'r1bqkbnr/pppp1ppp/8/4n3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A40k',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. c4'),
			moves => {
				'e5c4' => 'r1bqkbnr/pppp1ppp/8/8/2n5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1ppp/8/4n3/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A40k',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. c4'),
			moves => {
				'b1c3' => 'r1bqkbnr/pppp1ppp/8/4n3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp1ppp/8/4p3/2BnP3/5N2/PPPP1PPP/RNBQK2R b KQkq - 5 4' => {
			eco => 'C61b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Bc4

			variation => N__('Spanish: Bird\'s, 4. Bc4'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/8/4p3/2BnP3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C50c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nd4

			variation => N__('Italian: Blackburne-Shilling Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/8/4p3/3nP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C44r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 Nxd4

			variation => N__('Scotch: Lolli Variation'),
			moves => {
				'f3e5' => 'r1bqkbnr/pppp1ppp/8/4N3/3nP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp1ppp/8/8/2BpP3/8/PPPP1PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C61e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nd4 4. Nxd4 exd4 5. Bc4

			variation => N__('Spanish: Bird\'s, 5. Bc4'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/8/8/2n5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A40k',
                        # TRANSLATORS: 1. d4 Nc6 2. c4 e5 3. dxe5 Nxe5 4. Nc3 Nxc4

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, Pozarek Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp1ppp/8/8/3QP3/8/PPP2PPP/RNB1KB1R b KQkq - 0 5' => {
			eco => 'C45b',
                        # TRANSLATORS:

			variation => N__('Scotch: 4.Nxd4 Nxd4'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp2ppp/3p4/8/3QP3/8/PPP2PPP/RNB1KB1R w KQkq - 0 6',
			},
		},
		'r1bqkbnr/pppp1ppp/8/8/3nP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C45b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nxd4

			variation => N__('Scotch: 4.Nxd4 Nxd4'),
			moves => {
				'd1d4' => 'r1bqkbnr/pppp1ppp/8/8/3QP3/8/PPP2PPP/RNB1KB1R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp1ppp/8/nB2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 4 4' => {
			eco => 'C60d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Na5

			variation => N__('Spanish: Pollock Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2B5/4pp2/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C63g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Bxc6

			variation => N__('Spanish: Schliemann, 4.Bxc6'),
			moves => {
				'd7c6' => 'r1bqkbnr/ppp3pp/2p5/4pp2/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n2p2/1B2p3/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C60c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f6

			variation => N__('Spanish: Nürnberg Variation'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n2p2/4P3/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 Nc6 3. Nf3 f6

			variation => N__('Englund Gambit: Soller Deferred'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2p3/4N3/5N2/PPPP1PPP/R1BQK2R b KQkq - 0 5' => {
			eco => 'C63n',
                        # TRANSLATORS:

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4'),
			moves => {
				'd7d5' => 'r1bqkbnr/ppp3pp/2n5/1B1pp3/4N3/5N2/PPPP1PPP/R1BQK2R w KQkq d6 0 6',
				'g8f6' => 'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4N3/5N2/PPPP1PPP/R1BQK2R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2p3/4P3/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C63e',
                        # TRANSLATORS:

			variation => N__('Spanish: Schliemann, 4. d3 fxe4'),
			moves => {
				'g8f6' => 'r1bqkb1r/pppp2pp/2n2n2/1B2p3/4P3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2p3/4p3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5' => {
			eco => 'C63n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 fxe4

			variation => N__('Spanish: Schliemann, 4. Nc3 fxe4'),
			moves => {
				'c3e4' => 'r1bqkbnr/pppp2pp/2n5/1B2p3/4N3/5N2/PPPP1PPP/R1BQK2R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2p3/4p3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C63e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. d3 fxe4

			variation => N__('Spanish: Schliemann, 4. d3 fxe4'),
			moves => {
				'd3e4' => 'r1bqkbnr/pppp2pp/2n5/1B2p3/4P3/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2pP2/8/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C63b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. exf5

			variation => N__('Spanish: Schliemann, 4.exf5'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2pp2/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C63c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. d4

			variation => N__('Spanish: Schliemann, 4. d4'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4' => {
			eco => 'C63i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3

			variation => N__('Spanish: Schliemann, 4. Nc3'),
			moves => {
				'c6d4' => 'r1bqkbnr/pppp2pp/8/1B2pp2/3nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 2 5',
				'f5e4' => 'r1bqkbnr/pppp2pp/2n5/1B2p3/4p3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5',
				'g8f6' => 'r1bqkb1r/pppp2pp/2n2n2/1B2pp2/4P3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C63d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. d3

			variation => N__('Spanish: Schliemann, 4. d3'),
			moves => {
				'f5e4' => 'r1bqkbnr/pppp2pp/2n5/1B2p3/4p3/3P1N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 4' => {
			eco => 'C63a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5

			variation => N__('Spanish: Schliemann (Jänisch)'),
			moves => {
				'b1c3' => 'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4',
				'b5c6' => 'r1bqkbnr/pppp2pp/2B5/4pp2/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
				'd1e2' => 'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/5N2/PPPPQPPP/RNB1K2R b KQkq - 1 4',
				'd2d3' => 'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4',
				'd2d4' => 'r1bqkbnr/pppp2pp/2n5/1B2pp2/3PP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
				'e4f5' => 'r1bqkbnr/pppp2pp/2n5/1B2pP2/8/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/1B2pp2/4P3/5N2/PPPPQPPP/RNB1K2R b KQkq - 1 4' => {
			eco => 'C63a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Qe2

			variation => N__('Spanish: Schliemann, 4. Qe2'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4NP2/4pP2/8/PPPP2PP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'C30i',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Adelaide-Wahls, 4.exf5 e4'),
			moves => {
				'g8f6' => 'r1bqkb1r/pppp2pp/2n2n2/4NP2/4pP2/8/PPPP2PP/RNBQKB1R w KQkq - 2 6',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4Np2/4P3/8/PPPP1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C40s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Nc6

			variation => N__('Latvian Gambit: 3.Nxe5 Nc6'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pP2/5P2/5N2/PPPP2PP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C30i',
                        # TRANSLATORS: 1. e4 e5 2. f4 Nc6 3. Nf3 f5 4. exf5

			variation => N__('King\'s Gambit: Adelaide-Wahls, 4.exf5'),
			moves => {
				'e5e4' => 'r1bqkbnr/pppp2pp/2n5/5P2/4pP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 4' => {
			eco => 'C50b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 f5

			variation => N__('Italian: Rousseau Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq f6 0 4' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 f5

			variation => N__('English: Three Knights, 3... f5'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp2pp/2n5/4pp2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq f6 0 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 f5

			variation => N__('English: Closed'),
			moves => {
				'f1g2' => 'r1bqkbnr/pppp2pp/2n5/4pp2/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 f5 4. Bg2

			variation => N__('English: Closed'),
			moves => {
				'g8f6' => 'r1bqkb1r/pppp2pp/2n2n2/4pp2/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 f5 4. d4

			variation => N__('English: Three Knights, 3... f5 4. d4'),
			moves => {
				'e5e4' => 'r1bqkbnr/pppp2pp/2n5/5p2/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp3pp/2np4/4pp2/3PP3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq f6 0 4' => {
			eco => 'C46b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 f5

			variation => N__('Three Knights: Winawer Defence'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq f6 0 4' => {
			eco => 'C44j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 f5

			variation => N__('Ponziani Countergambit'),
			moves => {
				'd2d4' => 'r1bqkbnr/pppp2pp/2n5/4pp2/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq d3 0 4',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/4pp2/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq f6 0 4' => {
			eco => 'C30h',
                        # TRANSLATORS: 1. e4 e5 2. f4 Nc6 3. Nf3 f5

			variation => N__('King\'s Gambit: Adelaide-Wahls Variation'),
			moves => {
				'e4f5' => 'r1bqkbnr/pppp2pp/2n5/4pP2/5P2/5N2/PPPP2PP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/5P2/4pP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C30i',
                        # TRANSLATORS: 1. e4 e5 2. f4 Nc6 3. Nf3 f5 4. exf5 e4

			variation => N__('King\'s Gambit: Adelaide-Wahls, 4.exf5 e4'),
			moves => {
				'f3e5' => 'r1bqkbnr/pppp2pp/2n5/4NP2/4pP2/8/PPPP2PP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'r1bqkbnr/pppp2pp/2n5/5p2/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A27',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 f5 4. d4 e4

			variation => N__('English: Three Knights, 3... f5 4. d4 e4'),
			moves => {
			},
		},
		'r1bqkbnr/pppp2pp/8/1B2pp2/3nP3/2N2N2/PPPP1PPP/R1BQK2R w KQkq - 2 5' => {
			eco => 'C63j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 Nd4

			variation => N__('Spanish: Schliemann, 4. Nc3 Nd4'),
			moves => {
				'b5a4' => 'r1bqkbnr/pppp2pp/8/4pp2/B2nP3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 3 5',
			},
		},
		'r1bqkbnr/pppp2pp/8/4pp2/B2nP3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 3 5' => {
			eco => 'C63k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 f5 4. Nc3 Nd4 5. Ba4

			variation => N__('Spanish: Schliemann, 4. Nc3 Nd4 5. Ba4'),
			moves => {
			},
		},
		'r1bqkbnr/ppppp1pp/2n5/5p2/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq f6 0 3' => {
			eco => 'B00n',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 f5

			variation => N__('Nimzowitsch Defence: Colorado Counter'),
			moves => {
			},
		},
		'r1bqkbnr/pppppppp/2n5/3P4/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A40j',
                        # TRANSLATORS: 1. d4 Nc6 2. d5

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. d5'),
			moves => {
			},
		},
		'r1bqkbnr/pppppppp/2n5/6B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 2 2' => {
			eco => 'A40j',
                        # TRANSLATORS: 1. d4 Nc6 2. Bg5

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. Bg5'),
			moves => {
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/1P2P3/8/P1PP1PPP/RNBQKBNR b KQkq b3 0 2' => {
			eco => 'B00l',
                        # TRANSLATORS: 1. e4 Nc6 2. b4

			variation => N__('Nimzowitsch Defence: Wheeler Gambit'),
			moves => {
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 2 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 Nc6 2. Nc3

			variation => N__('English: 1... Nc6 2. Nc3'),
			moves => {
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 Nc6

			variation => N__('English: 1... Nc6'),
			moves => {
				'b1c3' => 'r1bqkbnr/pppppppp/2n5/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 2 2',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A40k',
                        # TRANSLATORS: 1. d4 Nc6 2. c4

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. c4'),
			moves => {
				'e7e5' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 2' => {
			eco => 'A40l',
                        # TRANSLATORS: 1. d4 Nc6 2. Nf3

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles, 2. Nf3'),
			moves => {
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A40j',
                        # TRANSLATORS: 1. d4 Nc6

			variation => N__('Queen\'s Pawn: Bogoljubow-Miles Defence'),
			moves => {
				'c1g5' => 'r1bqkbnr/pppppppp/2n5/6B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 2 2',
				'c2c4' => 'r1bqkbnr/pppppppp/2n5/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'd4d5' => 'r1bqkbnr/pppppppp/2n5/3P4/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2',
				'g1f3' => 'r1bqkbnr/pppppppp/2n5/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 2',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B00t',
                        # TRANSLATORS: 1. e4 Nc6 2. d4

			variation => N__('Nimzowitsch Defence: 2. d4'),
			moves => {
				'd7d5' => 'r1bqkbnr/ppp1pppp/2n5/3p4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
				'd7d6' => 'r1bqkbnr/ppp1pppp/2np4/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'r1bqkbnr/pppp1ppp/2n5/4p3/3PP3/8/PPP2PPP/RNBQKBNR w KQkq e6 0 3',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 2 2' => {
			eco => 'B00m',
                        # TRANSLATORS: 1. e4 Nc6 2. Nc3

			variation => N__('Nimzowitsch Defence: 2. Nc3'),
			moves => {
				'e7e6' => 'r1bqkbnr/pppp1ppp/2n1p3/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
				'g8f6' => 'r1bqkb1r/pppppppp/2n2n2/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 3 3',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 2 2' => {
			eco => 'B00n',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3

			variation => N__('Nimzowitsch Defence: 2. Nf3'),
			moves => {
				'd7d6' => 'r1bqkbnr/ppp1pppp/2np4/8/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'f7f5' => 'r1bqkbnr/ppppp1pp/2n5/5p2/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq f6 0 3',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'B00l',
                        # TRANSLATORS: 1. e4 Nc6

			variation => N__('Nimzowitsch Defence'),
			moves => {
				'b1c3' => 'r1bqkbnr/pppppppp/2n5/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 2 2',
				'b2b4' => 'r1bqkbnr/pppppppp/2n5/8/1P2P3/8/P1PP1PPP/RNBQKBNR b KQkq b3 0 2',
				'd2d4' => 'r1bqkbnr/pppppppp/2n5/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'g1f3' => 'r1bqkbnr/pppppppp/2n5/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 2 2',
			},
		},
		'r1bqkbnr/pppppppp/2n5/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 2 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 Nc6

			variation => N__('Reti: 1... Nc6'),
			moves => {
			},
		},
		'r1bqkn1r/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR w KQkq - 3 9' => {
			eco => 'D36j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 Nf8

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 Nf8'),
			moves => {
			},
		},
		'r1bqnrk1/1pppbppp/p1n5/4P3/B2p1B2/5N2/PPP2PPP/RN1QR1K1 b - - 2 9' => {
			eco => 'C84q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 O-O 8. e5 Ne8 9. Bf4

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 O-O 8. e5 Ne8 9. Bf4'),
			moves => {
			},
		},
		'r1bqnrk1/1pppbppp/p1n5/4P3/B2p4/2P2N2/PP3PPP/RNBQR1K1 b - - 0 9' => {
			eco => 'C84p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 O-O 8. e5 Ne8 9. c3

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 O-O 8. e5 Ne8 9. c3'),
			moves => {
			},
		},
		'r1bqnrk1/1pppbppp/p1n5/4P3/B2p4/5N2/PPP2PPP/RNBQR1K1 w - - 1 9' => {
			eco => 'C84p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. d4 exd4 7. Re1 O-O 8. e5 Ne8

			variation => N__('Spanish: Closed, Centre Attack, 7. Re1 O-O 8. e5 Ne8'),
			moves => {
				'c1f4' => 'r1bqnrk1/1pppbppp/p1n5/4P3/B2p1B2/5N2/PPP2PPP/RN1QR1K1 b - - 2 9',
				'c2c3' => 'r1bqnrk1/1pppbppp/p1n5/4P3/B2p4/2P2N2/PP3PPP/RNBQR1K1 b - - 0 9',
			},
		},
		'r1bqnrk1/p2p1ppp/1pn1p3/2p5/2PPP3/P1PB4/4NPPP/R1BQ1RK1 b - - 2 10' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 b6 9. e4 Ne8 10. O-O

			variation => N__('Nimzo-Indian: Sämisch, Capablanca, 10.O-O'),
			moves => {
			},
		},
		'r1bqnrk1/p2p1ppp/1pn1p3/2p5/2PPP3/P1PB4/4NPPP/R1BQK2R w KQ - 1 10' => {
			eco => 'E29',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 Nc6 8. Ne2 b6 9. e4 Ne8

			variation => N__('Nimzo-Indian: Sämisch, Capablanca Variation'),
			moves => {
				'e1g1' => 'r1bqnrk1/p2p1ppp/1pn1p3/2p5/2PPP3/P1PB4/4NPPP/R1BQ1RK1 b - - 2 10',
			},
		},
		'r1bqnrk1/pp2ppbp/2np2p1/2p5/2P1P3/2NP2P1/PP2NPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A36y',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 Nf6 7. O-O O-O 8. d3 Ne8

			variation => N__('English: Symmetrical, Botvinnik System, 5... d6, 8. d3 Ne8'),
			moves => {
			},
		},
		'r1bqnrk1/ppp1n1bp/3p2p1/3Pp3/2P1Pp2/2N1BP2/PP2B1PP/R2QNRK1 w - - 0 12' => {
			eco => 'E98g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Ne8 10. Be3 f5 11. f3 f4

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8 10. Be3 f5 11. f3 f4'),
			moves => {
			},
		},
		'r1bqnrk1/ppp1n1bp/3p2p1/3Ppp2/2P1P3/2N1B3/PP2BPPP/R2QNRK1 w - f6 0 11' => {
			eco => 'E98e',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8 10. Be3'),
			moves => {
				'f2f3' => 'r1bqnrk1/ppp1n1bp/3p2p1/3Ppp2/2P1P3/2N1BP2/PP2B1PP/R2QNRK1 b - - 0 11',
			},
		},
		'r1bqnrk1/ppp1n1bp/3p2p1/3Ppp2/2P1P3/2N1BP2/PP2B1PP/R2QNRK1 b - - 0 11' => {
			eco => 'E98f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Ne8 10. Be3 f5 11. f3

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8 10. Be3 f5 11. f3'),
			moves => {
				'f5f4' => 'r1bqnrk1/ppp1n1bp/3p2p1/3Pp3/2P1Pp2/2N1BP2/PP2B1PP/R2QNRK1 w - - 0 12',
			},
		},
		'r1bqnrk1/ppp1n1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 w - f6 0 11' => {
			eco => 'E98d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Ne8 10. Nd3 f5

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8 10. Nd3 f5'),
			moves => {
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/2PPp3/1P2P3/2N2N2/P3BPPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E97h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Ne8 10. c5

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Ne8 10. c5'),
			moves => {
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/1PP1P3/2N2N2/P3BPPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E97h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. b4 Ne8

			variation => N__('King\'s Indian: Mar del Plata, Bayonet Attack, 9... Ne8'),
			moves => {
				'c4c5' => 'r1bqnrk1/ppp1npbp/3p2p1/2PPp3/1P2P3/2N2N2/P3BPPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/1PP1P3/2N5/P2NBPPP/R1BQ1RK1 b - b3 0 10' => {
			eco => 'E97u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 Ne8 10. b4

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 Ne8 10. b4'),
			moves => {
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QNRK1 b - - 4 10' => {
			eco => 'E98e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Ne8 10. Be3

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8 10. Be3'),
			moves => {
				'f7f5' => 'r1bqnrk1/ppp1n1bp/3p2p1/3Ppp2/2P1P3/2N1B3/PP2BPPP/R2QNRK1 w - f6 0 11',
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 3 10' => {
			eco => 'E97u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 Ne8

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 Ne8'),
			moves => {
				'b2b4' => 'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/1PP1P3/2N5/P2NBPPP/R1BQ1RK1 b - b3 0 10',
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQNRK1 w - - 3 10' => {
			eco => 'E98c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Ne8

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8'),
			moves => {
				'c1e3' => 'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QNRK1 b - - 4 10',
				'e1d3' => 'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 b - - 4 10',
			},
		},
		'r1bqnrk1/ppp1npbp/3p2p1/3Pp3/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 b - - 4 10' => {
			eco => 'E98d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Ne1 Ne8 10. Nd3

			variation => N__('King\'s Indian: Mar del Plata, 9. Ne1 Ne8 10. Nd3'),
			moves => {
				'f7f5' => 'r1bqnrk1/ppp1n1bp/3p2p1/3Ppp2/2P1P3/2NN4/PP2BPPP/R1BQ1RK1 w - f6 0 11',
			},
		},
		'r1bqr1k1/1p1n1pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1QN1PP/R3KB1R w KQ - 3 13' => {
			eco => 'A65y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Re8 12. Nge2 Nbd7

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Re8 12.Nge2 Nbd7'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/2pp1np1/p2Pp3/2P1P3/2N2N2/PP3PPP/1RBQRBK1 b - - 0 11' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. Rb1 Re8 11. d5

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10. Rb1 Re8 11. d5'),
			moves => {
				'd7c5' => 'r1bqr1k1/1p3pbp/2pp1np1/p1nPp3/2P1P3/2N2N2/PP3PPP/1RBQRBK1 w - - 1 12',
			},
		},
		'r1bqr1k1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N1BNPP/PP3PB1/R2QR1K1 b - - 1 11' => {
			eco => 'E69u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 a5 11. Be3

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 a5 11. Be3'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2N2/PP3PPP/1RBQRBK1 w - - 2 11' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. Rb1 Re8

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10. Rb1 Re8'),
			moves => {
				'd4d5' => 'r1bqr1k1/1p1n1pbp/2pp1np1/p2Pp3/2P1P3/2N2N2/PP3PPP/1RBQRBK1 b - - 0 11',
			},
		},
		'r1bqr1k1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 w - a6 0 11' => {
			eco => 'E69u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 a5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 a5'),
			moves => {
				'c1e3' => 'r1bqr1k1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N1BNPP/PP3PB1/R2QR1K1 b - - 1 11',
			},
		},
		'r1bqr1k1/1p1n1pbp/2pp1np1/p7/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - a6 0 12' => {
			eco => 'E69w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 exd4 11. Nxd4 a5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 exd4, 11... a5'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P1N5/2N3P1/1P2PPBP/R1BQ1RK1 b - - 2 12' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. Nc4

			variation => N__('Benoni: Fianchetto, 11... Re8 12. Nc4'),
			moves => {
				'd7e5' => 'r1bqr1k1/1p3pbp/p2p1np1/2pPn3/P1N5/2N3P1/1P2PPBP/R1BQ1RK1 w - - 3 13',
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N4P/1P1NBPP1/R1BQ1RK1 b - - 0 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 a6 12. h3

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 a6 12. h3'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N5/1P1NBPPP/R1BQ1RK1 w - - 0 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 a6

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 a6'),
			moves => {
				'a1a3' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/R1N5/1P1NBPPP/2BQ1RK1 b - - 1 12',
				'd1c2' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N5/1PQNBPPP/R1B2RK1 b - - 1 12',
				'h2h3' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N4P/1P1NBPP1/R1BQ1RK1 b - - 0 12',
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N5/1PQNBPPP/R1B2RK1 b - - 1 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 a6 12. Qc2

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 a6 12. Qc2'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQ1RK1 w - - 3 12' => {
			eco => 'A70t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 a6 10. a4 Nbd7 11. O-O Re8

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6 10. a4 Nbd7 11.O-O Re8'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/R1N5/1P1NBPPP/2BQ1RK1 b - - 1 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 a6 12. Ra3

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 a6 12. Ra3'),
			moves => {
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P7/2N3P1/1P1NPPBP/R1BQ1RK1 w - - 1 12' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8

			variation => N__('Benoni: Fianchetto, 11... Re8'),
			moves => {
				'a4a5' => 'r1bqr1k1/1p1n1pbp/p2p1np1/P1pP4/8/2N3P1/1P1NPPBP/R1BQ1RK1 b - - 0 12',
				'd2c4' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P1N5/2N3P1/1P2PPBP/R1BQ1RK1 b - - 2 12',
				'h2h3' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P7/2N3PP/1P1NPPB1/R1BQ1RK1 b - - 0 12',
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P7/2N3PP/1P1NPPB1/R1BQ1RK1 b - - 0 12' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. h3

			variation => N__('Benoni: Fianchetto, 11... Re8 12. h3'),
			moves => {
				'a8b8' => '1rbqr1k1/1p1n1pbp/p2p1np1/2pP4/P7/2N3PP/1P1NPPB1/R1BQ1RK1 w - - 1 13',
			},
		},
		'r1bqr1k1/1p1n1pbp/p2p1np1/P1pP4/8/2N3P1/1P1NPPBP/R1BQ1RK1 b - - 0 12' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. a5

			variation => N__('Benoni: Fianchetto, 11... Re8 12. a5'),
			moves => {
			},
		},
		'r1bqr1k1/1p1nbppp/p1p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R w K - 0 10' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 Re8 9. a3 a6

			variation => N__('QGD: Orthodox, Rubinstein Attack, 8... Re8 9. a3 a6'),
			moves => {
			},
		},
		'r1bqr1k1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R1K2R w K - 2 11' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5 10. Bd3 Re8

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 Re8'),
			moves => {
				'e1g1' => 'r1bqr1k1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R2RK1 b - - 3 11',
			},
		},
		'r1bqr1k1/1p1nbppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R2RK1 b - - 3 11' => {
			eco => 'D65',
                        # TRANSLATORS:

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 Re8'),
			moves => {
				'd7f8' => 'r1bqrnk1/1p2bppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R2RK1 w - - 4 12',
			},
		},
		'r1bqr1k1/1p3pbp/2pp1np1/p1nPp3/2P1P3/1PN2N2/P4PPP/1RBQRBK1 b - - 0 12' => {
			eco => 'E96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 a5 10. Rb1 Re8 11. d5 Nc5 12. b3

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10. Rb1 Re8 11. d5 Nc5 12. b3'),
			moves => {
			},
		},
		'r1bqr1k1/1p3pbp/2pp1np1/p1nPp3/2P1P3/2N2N2/PP3PPP/1RBQRBK1 w - - 1 12' => {
			eco => 'E96',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 7.O-O Nbd7, Old Main Line, 10. Rb1 Re8 11. d5'),
			moves => {
				'b2b3' => 'r1bqr1k1/1p3pbp/2pp1np1/p1nPp3/2P1P3/1PN2N2/P4PPP/1RBQRBK1 b - - 0 12',
			},
		},
		'r1bqr1k1/1p3pbp/p2p1np1/2pPn3/P1N5/2N3P1/1P2PPBP/R1BQ1RK1 w - - 3 13' => {
			eco => 'A64',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 a6 11. a4 Re8 12. Nc4 Ne5

			variation => N__('Benoni: Fianchetto, 11... Re8 12. Nc4 Ne5'),
			moves => {
			},
		},
		'r1bqr1k1/1pp1ppbp/p1np1np1/8/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 2 9' => {
			eco => 'E83j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 Nc6 7. Nge2 a6 8. Qd2 Re8

			variation => N__('King\'s Indian: Sämisch, Panno, 8. Qd2 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/1pp2pbp/3p1np1/p1n5/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - a6 0 12' => {
			eco => 'E68h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 exd4 9. Nxd4 Re8 10. h3 Nc5 11. Re1 a5

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4 9.Nxd4 Re8 10. h3 Nc5 11. Re1 a5'),
			moves => {
			},
		},
		'r1bqr1k1/1ppnbppp/p4n2/3p2B1/3P4/2NBPN2/PP3PPP/2RQK2R w K - 2 10' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 a6 8. cxd5 exd5 9. Bd3 Re8

			variation => N__('QGD: Orthodox, Swiss, Karlsbad, 9. Bd3 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP1BN1P/PP3PP1/RN1QR1K1 b - - 2 11' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Be3

			variation => N__('Spanish: Closed, Smyslov, 10. d4 Re8 11. Be3'),
			moves => {
			},
		},
		'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 2 11' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2

			variation => N__('Spanish: Closed, Smyslov, 10. d4 Re8 11.Nbd2'),
			moves => {
				'e7f8' => 'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 3 12',
			},
		},
		'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8

			variation => N__('Spanish: Closed, Smyslov, 10. d4 Re8'),
			moves => {
				'a2a4' => 'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/P2PP3/1BP2N1P/1P3PP1/RNBQR1K1 b - a3 0 11',
				'b1d2' => 'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 2 11',
				'c1e3' => 'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/3PP3/1BP1BN1P/PP3PP1/RN1QR1K1 b - - 2 11',
			},
		},
		'r1bqr1k1/2p1bpp1/p1np1n1p/1p2p3/P2PP3/1BP2N1P/1P3PP1/RNBQR1K1 b - a3 0 11' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. a4

			variation => N__('Spanish: Closed, Smyslov, 10. d4 Re8 11. a4'),
			moves => {
			},
		},
		'r1bqr1k1/4bpp1/p2p1n1p/npp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 w - - 0 13' => {
			eco => 'C90n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Re8 12. Nf1 h6

			variation => N__('Spanish: Closed, Pilnik, 12. Nf1 h6'),
			moves => {
			},
		},
		'r1bqr1k1/4bppp/p1np1n2/1pp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 w - - 4 13' => {
			eco => 'C90l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Re8 12. Nf1 Nc6

			variation => N__('Spanish: Closed, Pilnik, 12. Nf1 Nc6'),
			moves => {
			},
		},
		'r1bqr1k1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 b - - 3 12' => {
			eco => 'C90k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Re8 12. Nf1

			variation => N__('Spanish: Closed, Pilnik, 12. Nf1'),
			moves => {
				'a5c6' => 'r1bqr1k1/4bppp/p1np1n2/1pp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 w - - 4 13',
				'e7f8' => 'r1bqrbk1/5ppp/p2p1n2/npp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 w - - 4 13',
				'h7h6' => 'r1bqr1k1/4bpp1/p2p1n1p/npp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 w - - 0 13',
			},
		},
		'r1bqr1k1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPBN1PPP/R1BQR1K1 w - - 2 12' => {
			eco => 'C90k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Re8

			variation => N__('Spanish: Closed, Pilnik, 11.Nbd2 Re8'),
			moves => {
				'd2f1' => 'r1bqr1k1/4bppp/p2p1n2/npp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 b - - 3 12',
			},
		},
		'r1bqr1k1/p1n2pbp/1p1p1np1/2pP4/P1N1P3/2N2P2/1P2B1PP/R1BQ1RK1 b - - 1 13' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3 Nc7 12. a4 b6 13. Nc4

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3 Nc7 12. a4 b6 13. Nc4'),
			moves => {
			},
		},
		'r1bqr1k1/p1n2pbp/1p1p1np1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1R1K b - - 1 13' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3 Nc7 12. a4 b6 13. Kh1

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3 Nc7 12. a4 b6 13. Kh1'),
			moves => {
			},
		},
		'r1bqr1k1/p1n2pbp/1p1p1np1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1RK1 w - - 0 13' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3 Nc7 12. a4 b6

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3 Nc7 12. a4 b6'),
			moves => {
				'd2c4' => 'r1bqr1k1/p1n2pbp/1p1p1np1/2pP4/P1N1P3/2N2P2/1P2B1PP/R1BQ1RK1 b - - 1 13',
				'g1h1' => 'r1bqr1k1/p1n2pbp/1p1p1np1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1R1K b - - 1 13',
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/1PN2NPP/P4PB1/R1BQ1RK1 b - - 0 10' => {
			eco => 'E69p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. b3

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. b3'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BNPP/PP3PB1/R2Q1RK1 b - - 2 10' => {
			eco => 'E69q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Be3

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Be3'),
			moves => {
				'e5d4' => 'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PpP3/2N1BNPP/PP3PB1/R2Q1RK1 w - - 0 11',
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 w - - 2 10' => {
			eco => 'E95k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 Re8

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Bf1 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQ1RK1 w - - 1 10' => {
			eco => 'E69o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8'),
			moves => {
				'b2b3' => 'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/1PN2NPP/P4PB1/R1BQ1RK1 b - - 0 10',
				'c1e3' => 'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BNPP/PP3PB1/R2Q1RK1 b - - 2 10',
				'f1e1' => 'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 b - - 2 10',
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 b - - 2 10' => {
			eco => 'E69s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1'),
			moves => {
				'a7a5' => 'r1bqr1k1/1p1n1pbp/2pp1np1/p3p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 w - a6 0 11',
				'd8c7' => 'r1b1r1k1/ppqn1pbp/2pp1np1/4p3/2PPP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 3 11',
				'e5d4' => 'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PpP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 0 11',
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N1B1PP/PP3PB1/R2Q1RK1 b - - 0 11' => {
			eco => 'E69r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Be3 exd4 11. Nxd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Be3 exd4 11.Nxd4'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 w - - 1 11' => {
			eco => 'E69f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 exd4 10. Nxd4 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 exd4 10.Nxd4 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 0 11' => {
			eco => 'E69v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 exd4 11. Nxd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 exd4'),
			moves => {
				'a7a5' => 'r1bqr1k1/1p1n1pbp/2pp1np1/p7/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - a6 0 12',
				'd7c5' => 'r1bqr1k1/pp3pbp/2pp1np1/2n5/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - - 1 12',
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N5/PP3PPP/R1BQRBK1 w - - 1 11' => {
			eco => 'E95n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 c6 9. Bf1 exd4 10. Nxd4 Re8

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 c6 9. Bf1 exd4 10.Nxd4 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PpP3/2N1BNPP/PP3PB1/R2Q1RK1 w - - 0 11' => {
			eco => 'E69r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Be3 exd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Be3 exd4'),
			moves => {
				'f3d4' => 'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N1B1PP/PP3PB1/R2Q1RK1 b - - 0 11',
			},
		},
		'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PpP3/2N2NPP/PP3PB1/R1BQR1K1 w - - 0 11' => {
			eco => 'E69v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 exd4

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 exd4'),
			moves => {
				'f3d4' => 'r1bqr1k1/pp1n1pbp/2pp1np1/8/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 0 11',
			},
		},
		'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 8 11' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7'),
			moves => {
				'a2a4' => 'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/P3P3/2N5/1P1NBPPP/R1BQ1RK1 b - a3 0 11',
			},
		},
		'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQK2R w KQ - 7 11' => {
			eco => 'A69e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. Nd2 Nbd7

			variation => N__('Benoni: Four Pawns, Main Line, 10. Nd2 Nbd7'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/8/2N3P1/PP1NPPBP/R1BQ1RK1 w - - 7 11' => {
			eco => 'A63',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Nbd7 10. Nd2 Re8

			variation => N__('Benoni: Fianchetto, 9...Nbd7 10. Nd2 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/P3P3/2N5/1P1NBPPP/R1BQ1RK1 b - a3 0 11' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4'),
			moves => {
				'a7a6' => 'r1bqr1k1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N5/1P1NBPPP/R1BQ1RK1 w - - 0 12',
				'd7e5' => 'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1P1NBPPP/R1BQ1RK1 w - - 1 12',
			},
		},
		'r1bqr1k1/pp1n1ppp/2pbpn2/3p4/2PP4/1PN1PN2/P1Q1BPPP/R1B1K2R w KQ - 3 9' => {
			eco => 'D45t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. b3 O-O 8. Be2 Re8

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. b3 O-O 8. Be2 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1n1ppp/2pbpn2/3p4/2PP4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 7 9' => {
			eco => 'D45y',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Qc2 Bd6 7. Be2 O-O 8. O-O Re8

			variation => N__('Semi-Slav: 6. Qc2 Bd6 7. Be2 O-O 8.O-O Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/2KR3R w - - 0 11' => {
			eco => 'D36r',
                        # TRANSLATORS:

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O-O'),
			moves => {
				'g5h4' => 'r1bqr1k1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/2KR3R b - - 1 11',
			},
		},
		'r1bqr1k1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/R4RK1 w - - 0 11' => {
			eco => 'D36t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O h6

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O h6'),
			moves => {
				'g5f4' => 'r1bqr1k1/pp1nbpp1/2p2n1p/3p4/3P1B2/2NBPN2/PPQ2PPP/R4RK1 b - - 1 11',
				'g5h4' => 'r1bqr1k1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/R4RK1 b - - 1 11',
			},
		},
		'r1bqr1k1/pp1nbpp1/2p2n1p/3p4/3P1B2/2NBPN2/PPQ2PPP/R4RK1 b - - 1 11' => {
			eco => 'D36t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O h6 11. Bf4

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O h6'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/2KR3R b - - 1 11' => {
			eco => 'D36r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O-O h6 11. Bh4

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O-O h6'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbpp1/2p2n1p/3p4/3P3B/2NBPN2/PPQ2PPP/R4RK1 b - - 1 11' => {
			eco => 'D36t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O h6 11. Bh4

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O h6'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PPQ2PPP/2R1KB1R w K - 2 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 Re8

			variation => N__('QGD: Orthodox, Rubinstein Attack, 8... Re8'),
			moves => {
				'a2a3' => 'r1bqr1k1/pp1nbppp/2p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R b K - 0 9',
			},
		},
		'r1bqr1k1/pp1nbppp/2p1pn2/3p2B1/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 2 9' => {
			eco => 'D66',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Bd3 Re8

			variation => N__('QGD: Orthodox, Main Line, 8... Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbppp/2p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R b K - 0 9' => {
			eco => 'D64',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 Re8 9. a3

			variation => N__('QGD: Orthodox, Rubinstein Attack, 8... Re8 9. a3'),
			moves => {
				'a7a6' => 'r1bqr1k1/1p1nbppp/p1p1pn2/3p2B1/2PP4/P1N1PN2/1PQ2PPP/2R1KB1R w K - 0 10',
			},
		},
		'r1bqr1k1/pp1nbppp/2p1pn2/3p4/2PP4/5NP1/PPQNPPBP/R1B2RK1 w - - 2 9' => {
			eco => 'E09c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 Re8

			variation => N__('Catalan: Closed, Main Line, 8... Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R3K2R w KQ - 5 10' => {
			eco => 'D36l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2 Re8

			variation => N__('QGD: Exchange, Main Line, 9.Nge2 Re8'),
			moves => {
				'e1g1' => 'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R4RK1 b - - 6 10',
			},
		},
		'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R4RK1 b - - 6 10' => {
			eco => 'D36m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2 Re8 10. O-O

			variation => N__('QGD: Exchange, Main Line, 9.Nge2 Re8 10.O-O'),
			moves => {
				'd7f8' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R4RK1 w - - 7 11',
			},
		},
		'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/R3K2R b KQ - 0 10' => {
			eco => 'D36p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. h3

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10. h3'),
			moves => {
				'd7f8' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/R3K2R w KQ - 1 11',
			},
		},
		'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2KR3R b - - 6 10' => {
			eco => 'D36r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O-O

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O-O'),
			moves => {
				'd7f8' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2KR3R w - - 7 11',
				'h7h6' => 'r1bqr1k1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/2KR3R w - - 0 11',
			},
		},
		'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R3K2R w KQ - 5 10' => {
			eco => 'D36p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8'),
			moves => {
				'e1c1' => 'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2KR3R b - - 6 10',
				'e1g1' => 'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R4RK1 b - - 6 10',
				'h2h3' => 'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/R3K2R b KQ - 0 10',
			},
		},
		'r1bqr1k1/pp1nbppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R4RK1 b - - 6 10' => {
			eco => 'D36s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O'),
			moves => {
				'd7f8' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R4RK1 w - - 7 11',
				'h7h6' => 'r1bqr1k1/pp1nbpp1/2p2n1p/3p2B1/3P4/2NBPN2/PPQ2PPP/R4RK1 w - - 0 11',
			},
		},
		'r1bqr1k1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 4 9' => {
			eco => 'A55r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Re1 Re8

			variation => N__('Old Indian: Main Line, 8. Re1 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 4 9' => {
			eco => 'A55o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. Be2 c6 7. O-O O-O 8. Qc2 Re8

			variation => N__('Old Indian: Main Line, 8. Qc2 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp1nbppp/2pp1n2/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 4 9' => {
			eco => 'A55f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 Nbd7 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8. O-O Re8

			variation => N__('Old Indian: 5. e4 Be7 6. g3 c6 7. Bg2 O-O 8.O-O Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pp2bpp1/2n2n1p/3p4/3N4/1QN1B1P1/PP2PPBP/R4RK1 b - - 3 12' => {
			eco => 'D34v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6 11. Be3 Re8 12. Qb3

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6 11. Be3 Re8 12. Qb3'),
			moves => {
			},
		},
		'r1bqr1k1/pp2bpp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/2RQ1RK1 b - - 3 12' => {
			eco => 'D34w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6 11. Be3 Re8 12. Rc1

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6 11. Be3 Re8 12. Rc1'),
			moves => {
				'e7f8' => 'r1bqrbk1/pp3pp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/2RQ1RK1 w - - 4 13',
			},
		},
		'r1bqr1k1/pp2bpp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/R2Q1RK1 w - - 2 12' => {
			eco => 'D34u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6 11. Be3 Re8

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6 11. Be3 Re8'),
			moves => {
				'a1c1' => 'r1bqr1k1/pp2bpp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/2RQ1RK1 b - - 3 12',
				'd1b3' => 'r1bqr1k1/pp2bpp1/2n2n1p/3p4/3N4/1QN1B1P1/PP2PPBP/R4RK1 b - - 3 12',
			},
		},
		'r1bqr1k1/pp3pbp/2pp1np1/2n5/2PNP3/1PN3PP/P4PB1/R1BQR1K1 b - - 0 12' => {
			eco => 'E69z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 exd4 11. Nxd4 Nc5 12. b3

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 exd4, 11... Nc5 12. b3'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/2pp1np1/2n5/2PNP3/2N3PP/PP3PB1/1RBQR1K1 b - - 2 12' => {
			eco => 'E69y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 exd4 11. Nxd4 Nc5 12. Rb1

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 exd4, 11... Nc5 12. Rb1'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/2pp1np1/2n5/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - - 1 12' => {
			eco => 'E69x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 c6 9. h3 Re8 10. Re1 exd4 11. Nxd4 Nc5

			variation => N__('King\'s Indian: Fianchetto, Classical, 9. h3 Re8 10. Re1 exd4, 11... Nc5'),
			moves => {
				'a1b1' => 'r1bqr1k1/pp3pbp/2pp1np1/2n5/2PNP3/2N3PP/PP3PB1/1RBQR1K1 b - - 2 12',
				'b2b3' => 'r1bqr1k1/pp3pbp/2pp1np1/2n5/2PNP3/1PN3PP/P4PB1/R1BQR1K1 b - - 0 12',
			},
		},
		'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1P1NBPPP/R1BQ1RK1 w - - 1 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 Ne5

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 Ne5'),
			moves => {
				'a1a3' => 'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/R1N5/1P1NBPPP/2BQ1RK1 b - - 2 12',
				'd1c2' => 'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1PQNBPPP/R1B2RK1 b - - 2 12',
				'd2b1' => 'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1P2BPPP/RNBQ1RK1 b - - 2 12',
				'f1e1' => 'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1P1NBPPP/R1BQR1K1 b - - 2 12',
			},
		},
		'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1P1NBPPP/R1BQR1K1 b - - 2 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 Ne5 12. Re1

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 Ne5 12. Re1'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1P2BPPP/RNBQ1RK1 b - - 2 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 Ne5 12. Ndb1

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 Ne5 12.Ndb1'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/2N5/1PQNBPPP/R1B2RK1 b - - 2 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 Ne5 12. Qc2

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 Ne5 12. Qc2'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/3p1np1/2pPn3/P3P3/R1N5/1P1NBPPP/2BQ1RK1 b - - 2 12' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Nbd7 11. a4 Ne5 12. Ra3

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Nbd7 11. a4 Ne5 12. Ra3'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N2N2/PPQ1BPPP/R1B1R1K1 b - - 9 11' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Qc2 Na6 11. Re1

			variation => N__('Benoni: Classical, Main Line, 10. Qc2 Na6 11. Re1'),
			moves => {
				'c8g4' => 'r2qr1k1/pp3pbp/n2p1np1/2pP4/4P1b1/2N2N2/PPQ1BPPP/R1B1R1K1 w - - 10 12',
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 8 11' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Qc2 Na6

			variation => N__('Benoni: Classical, Main Line, 10. Qc2 Na6'),
			moves => {
				'c1f4' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4PB2/2N2N2/PPQ1BPPP/R4RK1 b - - 9 11',
				'f1e1' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N2N2/PPQ1BPPP/R1B1R1K1 b - - 9 11',
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N2P2/PP1NB1PP/R1BQ1RK1 b - - 0 11' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3'),
			moves => {
				'a6c7' => 'r1bqr1k1/ppn2pbp/3p1np1/2pP4/4P3/2N2P2/PP1NB1PP/R1BQ1RK1 w - - 1 12',
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/1RBQ1RK1 b - - 9 11' => {
			eco => 'A78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. Rb1

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. Rb1'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1R1K b - - 9 11' => {
			eco => 'A78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. Kh1

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. Kh1'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 8 11' => {
			eco => 'A78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6'),
			moves => {
				'a1b1' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/1RBQ1RK1 b - - 9 11',
				'f1e1' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQR1K1 b - - 9 11',
				'f2f3' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N2P2/PP1NB1PP/R1BQ1RK1 b - - 0 11',
				'f2f4' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQ1RK1 b - f3 0 11',
				'g1h1' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1R1K b - - 9 11',
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQR1K1 b - - 9 11' => {
			eco => 'A78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. Re1

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. Re1'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4PB2/2N2N2/PPQ1BPPP/R4RK1 b - - 9 11' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Qc2 Na6 11. Bf4

			variation => N__('Benoni: Classical, Main Line, 10. Qc2 Na6 11. Bf4'),
			moves => {
			},
		},
		'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQ1RK1 b - f3 0 11' => {
			eco => 'A78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f4

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f4'),
			moves => {
				'a6c7' => 'r1bqr1k1/ppn2pbp/3p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQ1RK1 w - - 1 12',
			},
		},
		'r1bqr1k1/pp3ppp/2n5/2bN4/3p4/6P1/PP1NPPBP/R2Q1RK1 w - - 4 14' => {
			eco => 'D34n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Bg5 d4 11. Bxf6 Qxf6 12. Nd5 Qd8 13. Nd2 Re8

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 d4 11.Bxf6: Main Line'),
			moves => {
			},
		},
		'r1bqr1k1/ppn2pbp/3p1np1/2pP4/4P3/2N2P2/PP1NB1PP/R1BQ1RK1 w - - 1 12' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3 Nc7

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3 Nc7'),
			moves => {
				'a2a4' => 'r1bqr1k1/ppn2pbp/3p1np1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1RK1 b - a3 0 12',
			},
		},
		'r1bqr1k1/ppn2pbp/3p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQ1RK1 w - - 1 12' => {
			eco => 'A78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f4 Nc7

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f4 Nc7'),
			moves => {
			},
		},
		'r1bqr1k1/ppn2pbp/3p1np1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1RK1 b - a3 0 12' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3 Nc7 12. a4

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3 Nc7 12. a4'),
			moves => {
				'b7b6' => 'r1bqr1k1/p1n2pbp/1p1p1np1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1RK1 w - - 0 13',
				'f6d7' => 'r1bqr1k1/ppnn1pbp/3p2p1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1RK1 w - - 1 13',
			},
		},
		'r1bqr1k1/ppnn1pbp/3p2p1/2pP4/P3P3/2N2P2/1P1NB1PP/R1BQ1RK1 w - - 1 13' => {
			eco => 'A79',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 Na6 11. f3 Nc7 12. a4 Nd7

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 Na6 11. f3 Nc7 12. a4 Nd7'),
			moves => {
			},
		},
		'r1bqr1k1/ppp1ppbp/1nn3p1/3P4/8/2N1PNP1/PP3PBP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D76h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. e3 Re8 10. d5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. e3 Re8 10. d5'),
			moves => {
			},
		},
		'r1bqr1k1/ppp1ppbp/1nn3p1/8/3P4/2N1PNP1/PP3PBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'D76g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3 Nc6 9. e3 Re8

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3 Nc6 9. e3 Re8'),
			moves => {
				'd4d5' => 'r1bqr1k1/ppp1ppbp/1nn3p1/3P4/8/2N1PNP1/PP3PBP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bqr1k1/ppp2pbp/2np1np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 b - - 2 10' => {
			eco => 'E94d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4 8. Nxd4 Re8 9. f3 Nc6 10. Be3

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3 Nc6 10. Be3'),
			moves => {
				'f6h5' => 'r1bqr1k1/ppp2pbp/2np2p1/7n/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 w - - 3 11',
			},
		},
		'r1bqr1k1/ppp2pbp/2np1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E94d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4 8. Nxd4 Re8 9. f3 Nc6

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3 Nc6'),
			moves => {
				'c1e3' => 'r1bqr1k1/ppp2pbp/2np1np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 b - - 2 10',
			},
		},
		'r1bqr1k1/ppp2pbp/2np2p1/7n/2PNP3/2N1BP2/PP1QB1PP/R4RK1 b - - 4 11' => {
			eco => 'E94e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4 8. Nxd4 Re8 9. f3 Nc6 10. Be3 Nh5 11. Qd2

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3 Nc6 10. Be3 Nh5 11. Qd2'),
			moves => {
			},
		},
		'r1bqr1k1/ppp2pbp/2np2p1/7n/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 w - - 3 11' => {
			eco => 'E94d',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3 Nc6 10. Be3'),
			moves => {
				'd1d2' => 'r1bqr1k1/ppp2pbp/2np2p1/7n/2PNP3/2N1BP2/PP1QB1PP/R4RK1 b - - 4 11',
			},
		},
		'r1bqr1k1/ppp2pbp/3p1np1/2n5/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 w - - 1 11' => {
			eco => 'E68g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 exd4 9. Nxd4 Re8 10. h3 Nc5

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4 9.Nxd4 Re8 10. h3 Nc5'),
			moves => {
				'f1e1' => 'r1bqr1k1/ppp2pbp/3p1np1/2n5/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 2 11',
			},
		},
		'r1bqr1k1/ppp2pbp/3p1np1/2n5/2PNP3/2N3PP/PP3PB1/R1BQR1K1 b - - 2 11' => {
			eco => 'E68g',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4 9.Nxd4 Re8 10. h3 Nc5'),
			moves => {
				'a7a5' => 'r1bqr1k1/1pp2pbp/3p1np1/p1n5/2PNP3/2N3PP/PP3PB1/R1BQR1K1 w - a6 0 12',
			},
		},
		'r1bqr1k1/ppp2ppp/2np1n2/8/1bB1P3/2N1Q2N/PPPB1PPP/2KR3R b - - 1 9' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4 Nc6 4. Qe3 Nf6 5. Nc3 Bb4 6. Bd2 O-O 7. O-O-O Re8 8. Bc4 d6 9. Nh3

			variation => N__('Centre Game: Kupreichik Variation'),
			moves => {
			},
		},
		'r1bqr1k1/ppp2ppp/2np1n2/8/1bB1P3/2N1Q3/PPPB1PPP/2KR2NR w - - 0 9' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'g1h3' => 'r1bqr1k1/ppp2ppp/2np1n2/8/1bB1P3/2N1Q2N/PPPB1PPP/2KR3R b - - 1 9',
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 w - - 1 9' => {
			eco => 'E67k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. h3 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. h3 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PP4/5NP1/PPQ1PPBP/RNBR2K1 w - - 2 9' => {
			eco => 'E60r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O Nbd7 7. Qc2 e5 8. Rd1 Re8

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6...Nbd7 7. Qc2 e5 8. Rd1 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 w - - 4 9' => {
			eco => 'E94w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Be3 Re8

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Be3 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQR1K1 w - - 4 9' => {
			eco => 'E95d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 Re8

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 Re8'),
			moves => {
				'e2f1' => 'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 5 9',
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP3PPP/R1BQRBK1 b - - 5 9' => {
			eco => 'E95e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Re1 Re8 9. Bf1

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Re1 Re8 9. Bf1'),
			moves => {
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 4 9' => {
			eco => 'E94s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nbd7 8. Qc2 Re8

			variation => N__('King\'s Indian: 7.O-O Nbd7 8. Qc2 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/4p3/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E68b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/8/2PNP3/2N3P1/PP3PBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E68e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 exd4 9. Nxd4 Re8

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4 9.Nxd4 Re8'),
			moves => {
				'h2h3' => 'r1bqr1k1/pppn1pbp/3p1np1/8/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 b - - 0 10',
			},
		},
		'r1bqr1k1/pppn1pbp/3p1np1/8/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 b - - 0 10' => {
			eco => 'E68f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nbd7 7. O-O e5 8. e4 exd4 9. Nxd4 Re8 10. h3

			variation => N__('King\'s Indian: Fianchetto, Classical, 8. e4 exd4 9.Nxd4 Re8 10. h3'),
			moves => {
				'd7c5' => 'r1bqr1k1/ppp2pbp/3p1np1/2n5/2PNP3/2N3PP/PP3PB1/R1BQ1RK1 w - - 1 11',
			},
		},
		'r1bqr1k1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 5 8' => {
			eco => 'D63',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 Re8

			variation => N__('QGD: Orthodox, 7. Rc1 Re8'),
			moves => {
			},
		},
		'r1bqr1k1/pppp1ppp/2n2n2/3Np3/1bP5/4PN2/PPQP1PPP/R1B1KB1R w KQ - 5 7' => {
			eco => 'A28',
                        # TRANSLATORS:

			variation => N__('English: Four Knights, 4. e3 Bb4 5. Qc2'),
			moves => {
				'c2f5' => 'r1bqr1k1/pppp1ppp/2n2n2/3NpQ2/1bP5/4PN2/PP1P1PPP/R1B1KB1R b KQ - 6 7',
			},
		},
		'r1bqr1k1/pppp1ppp/2n2n2/3NpQ2/1bP5/4PN2/PP1P1PPP/R1B1KB1R b KQ - 6 7' => {
			eco => 'A28',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. e3 Bb4 5. Qc2 O-O 6. Nd5 Re8 7. Qf5

			variation => N__('English: Four Knights, Stean Variation'),
			moves => {
			},
		},
		'r1bqr1k1/pppp1ppp/2n2n2/8/1b2P3/2N1Q3/PPPB1PPP/2KR1BNR w - - 9 8' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'f1c4' => 'r1bqr1k1/pppp1ppp/2n2n2/8/1bB1P3/2N1Q3/PPPB1PPP/2KR2NR b - - 10 8',
			},
		},
		'r1bqr1k1/pppp1ppp/2n2n2/8/1bB1P3/2N1Q3/PPPB1PPP/2KR2NR b - - 10 8' => {
			eco => 'C22',
                        # TRANSLATORS:

			variation => N__('Centre Game: Berger Variation'),
			moves => {
				'd7d6' => 'r1bqr1k1/ppp2ppp/2np1n2/8/1bB1P3/2N1Q3/PPPB1PPP/2KR2NR w - - 0 9',
			},
		},
		'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 3 12' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8

			variation => N__('Spanish: Closed, Smyslov, 10. d4 Re8 11.Nbd2 Bf8'),
			moves => {
				'a2a3' => 'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/PBP2N1P/1P1N1PP1/R1BQR1K1 b - - 0 12',
				'b3c2' => 'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 4 12',
				'd2f1' => 'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/R1BQRNK1 b - - 4 12',
			},
		},
		'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/R1BQRNK1 b - - 4 12' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. Nf1

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1'),
			moves => {
				'c8b7' => 'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/R1BQRNK1 w - - 5 13',
				'c8d7' => 'r2qrbk1/2pb1pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/R1BQRNK1 w - - 5 13',
			},
		},
		'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 4 12' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. Bc2

			variation => N__('Spanish: Closed, Smyslov, 12. Bc2'),
			moves => {
			},
		},
		'r1bqrbk1/2p2pp1/p1np1n1p/1p2p3/3PP3/PBP2N1P/1P1N1PP1/R1BQR1K1 b - - 0 12' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. a3

			variation => N__('Spanish: Closed, Smyslov, 12. a3'),
			moves => {
			},
		},
		'r1bqrbk1/5ppp/p2p1n2/npp1p3/4P3/2PP1N2/PPB2PPP/R1BQRNK1 w - - 4 13' => {
			eco => 'C90m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d3 Na5 10. Bc2 c5 11. Nbd2 Re8 12. Nf1 Bf8

			variation => N__('Spanish: Closed, Pilnik, 12. Nf1 Bf8'),
			moves => {
			},
		},
		'r1bqrbk1/pp3pp1/2N2n1p/3p4/8/2N1B1P1/PP2PPBP/2RQ1RK1 b - - 0 13' => {
			eco => 'D34y',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6 11. Be3 Re8 12. Rc1 Bf8 13. Nxc6

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6 11. Be3 Re8 12. Rc1 Bf8 13.Nxc6'),
			moves => {
			},
		},
		'r1bqrbk1/pp3pp1/2n2n1p/3p4/3N4/2N1B1P1/PP2PPBP/2RQ1RK1 w - - 4 13' => {
			eco => 'D34x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 cxd4 10. Nxd4 h6 11. Be3 Re8 12. Rc1 Bf8

			variation => N__('QGD Tarrasch: 9. Bg5 cxd4 10.Nxd4 h6 11. Be3 Re8 12. Rc1 Bf8'),
			moves => {
				'd4c6' => 'r1bqrbk1/pp3pp1/2N2n1p/3p4/8/2N1B1P1/PP2PPBP/2RQ1RK1 b - - 0 13',
			},
		},
		'r1bqrnk1/1p2bppp/2p2n2/p2p2B1/3P4/2NBPN2/PPQ2PPP/1R3RK1 w - a6 0 12' => {
			eco => 'D36z',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O Nf8 11. Rab1 a5

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O Nf8 11.Rab1 a5'),
			moves => {
			},
		},
		'r1bqrnk1/1p2bppp/p1p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/2R2RK1 b - - 0 12' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5 10. Bd3 Re8 11. O-O Nf8 12. h3

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 Re8 11.O-O Nf8 12. h3'),
			moves => {
			},
		},
		'r1bqrnk1/1p2bppp/p1p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2R2RK1 w - - 4 12' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5 10. Bd3 Re8 11. O-O Nf8

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 Re8 11.O-O Nf8'),
			moves => {
				'f3e5' => 'r1bqrnk1/1p2bppp/p1p2n2/3pN1B1/3P4/2NBP3/PPQ2PPP/2R2RK1 b - - 5 12',
				'h2h3' => 'r1bqrnk1/1p2bppp/p1p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/2R2RK1 b - - 0 12',
			},
		},
		'r1bqrnk1/1p2bppp/p1p2n2/3pN1B1/3P4/2NBP3/PPQ2PPP/2R2RK1 b - - 5 12' => {
			eco => 'D65',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Nbd7 7. Rc1 c6 8. Qc2 a6 9. cxd5 exd5 10. Bd3 Re8 11. O-O Nf8 12. Ne5

			variation => N__('QGD: Orthodox, Rubinstein, 9.cxd5 exd5 10. Bd3 Re8 11.O-O Nf8 12. Ne5'),
			moves => {
			},
		},
		'r1bqrnk1/pp2bp1p/2p2np1/3p2B1/3P4/2NBPN1P/PPQ2PP1/R4RK1 w - - 0 12' => {
			eco => 'D36x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O Nf8 11. h3 g6

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O Nf8 11. h3 g6'),
			moves => {
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PPQ1NPPP/R4RK1 w - - 7 11' => {
			eco => 'D36m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2 Re8 10. O-O Nf8

			variation => N__('QGD: Exchange, Main Line, 9.Nge2 Re8 10.O-O Nf8'),
			moves => {
				'f2f3' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPP2/PPQ1N1PP/R4RK1 b - - 0 11',
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/2KR3R b - - 0 11' => {
			eco => 'D36r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O-O Nf8 11. h3

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O-O Nf8 11. h3'),
			moves => {
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/R3K2R w KQ - 1 11' => {
			eco => 'D36p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. h3 Nf8

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10. h3 Nf8'),
			moves => {
				'g5f4' => 'r1bqrnk1/pp2bppp/2p2n2/3p4/3P1B2/2NBPN1P/PPQ2PP1/R3K2R b KQ - 2 11',
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/R4RK1 b - - 0 11' => {
			eco => 'D36w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O Nf8 11. h3

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O Nf8 11. h3'),
			moves => {
				'g7g6' => 'r1bqrnk1/pp2bp1p/2p2np1/3p2B1/3P4/2NBPN1P/PPQ2PP1/R4RK1 w - - 0 12',
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/1R3RK1 b - - 8 11' => {
			eco => 'D36y',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O Nf8 11. Rab1

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O Nf8 11.Rab1'),
			moves => {
				'a7a5' => 'r1bqrnk1/1p2bppp/2p2n2/p2p2B1/3P4/2NBPN2/PPQ2PPP/1R3RK1 w - a6 0 12',
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/2KR3R w - - 7 11' => {
			eco => 'D36r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O-O Nf8

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O-O Nf8'),
			moves => {
				'h2h3' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/2KR3R b - - 0 11',
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/4RRK1 b - - 8 11' => {
			eco => 'D36v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O Nf8 11. Rae1

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O Nf8 11.Rae1'),
			moves => {
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/R4RK1 w - - 7 11' => {
			eco => 'D36u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. O-O Nf8

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10.O-O Nf8'),
			moves => {
				'a1b1' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/1R3RK1 b - - 8 11',
				'a1e1' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PPQ2PPP/4RRK1 b - - 8 11',
				'h2h3' => 'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN1P/PPQ2PP1/R4RK1 b - - 0 11',
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPP2/PPQ1N1PP/R4RK1 b - - 0 11' => {
			eco => 'D36n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nge2 Re8 10. O-O Nf8 11. f3

			variation => N__('QGD: Exchange, Main Line, 9.Nge2 Re8 10.O-O Nf8 11. f3'),
			moves => {
			},
		},
		'r1bqrnk1/pp2bppp/2p2n2/3p4/3P1B2/2NBPN1P/PPQ2PP1/R3K2R b KQ - 2 11' => {
			eco => 'D36q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 Nbd7 8. Bd3 O-O 9. Nf3 Re8 10. h3 Nf8 11. Bf4

			variation => N__('QGD: Exchange, Main Line, 9. Nf3 Re8 10. h3 Nf8 11. Bf4'),
			moves => {
			},
		},
		'r1br2k1/2q1bppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14' => {
			eco => 'C99b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Rd8

			variation => N__('Spanish: Closed, Chigorin, 13... Rd8'),
			moves => {
			},
		},
		'r1br2k1/2q1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13' => {
			eco => 'C97g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Rd8

			variation => N__('Spanish: Closed, Chigorin, 12... Rd8'),
			moves => {
			},
		},
		'r1br2k1/pp2bppp/1qnppn2/6B1/4P3/1NN2P2/PPPQ2PP/1K1R1B1R b - - 2 11' => {
			eco => 'B63w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3 Qb6 10. f3 Rd8 11. Kb1

			variation => N__('Sicilian: Richter-Rauzer, Podebrad, 10. f3 Rd8 11. Kb1'),
			moves => {
			},
		},
		'r1br2k1/pp2bppp/1qnppn2/6B1/4P3/1NN2P2/PPPQ2PP/2KR1B1R w - - 1 11' => {
			eco => 'B63v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. Nb3 Qb6 10. f3 Rd8

			variation => N__('Sicilian: Richter-Rauzer, Podebrad, 10. f3 Rd8'),
			moves => {
				'c1b1' => 'r1br2k1/pp2bppp/1qnppn2/6B1/4P3/1NN2P2/PPPQ2PP/1K1R1B1R b - - 2 11',
			},
		},
		'r1br2k1/ppp1qppp/3pnn2/1B2p3/3PP3/2P2N2/P1P2PPP/R1BQR1K1 w - - 3 12' => {
			eco => 'C49q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 Qe7 9. Re1 Nd8 10. d4 Ne6 11. Bc1 Rd8

			variation => N__('Four Knights: Symmetrical, Metger, 10. d4 Ne6 11. Bc1 Rd8'),
			moves => {
			},
		},
		'r1br2k1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P2QNPPP/2R2RK1 b - - 7 12' => {
			eco => 'D87l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Qc7 11. Rc1 Rd8 12. Qd2

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Qc7 11. Rc1 Rd8 12. Qd2'),
			moves => {
			},
		},
		'r1br2k1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/2RQ1RK1 w - - 6 12' => {
			eco => 'D87k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Qc7 11. Rc1 Rd8

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Qc7 11. Rc1 Rd8'),
			moves => {
				'd1d2' => 'r1br2k1/ppq1ppbp/2n3p1/2p5/2BPP3/2P1B3/P2QNPPP/2R2RK1 b - - 7 12',
				'e3f4' => 'r1br2k1/ppq1ppbp/2n3p1/2p5/2BPPB2/2P5/P3NPPP/2RQ1RK1 b - - 7 12',
			},
		},
		'r1br2k1/ppq1ppbp/2n3p1/2p5/2BPPB2/2P5/P3NPPP/2RQ1RK1 b - - 7 12' => {
			eco => 'D87m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Qc7 11. Rc1 Rd8 12. Bf4

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Qc7 11. Rc1 Rd8 12. Bf4'),
			moves => {
			},
		},
		'r1q1kb1r/pp2pppp/2n2n2/3p4/3P1Bb1/1QPB4/PP3PPP/RN2K1NR w KQkq - 5 8' => {
			eco => 'B13l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 Nf6 6. Bf4 Bg4 7. Qb3 Qc8

			variation => N__('Caro-Kann: Exchange, Rubinstein, 7. Qb3 Qc8'),
			moves => {
			},
		},
		'r1q2rk1/2p1bppp/p2p1n2/np1Pp3/4P1b1/2P2N2/PPB2PPP/RNBQR1K1 w - - 3 12' => {
			eco => 'C91g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. d5 Na5 11. Bc2 Qc8

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5 Na5 11. Bc2 Qc8'),
			moves => {
			},
		},
		'r1q2rk1/pp2ppbp/2npbnp1/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 11' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4 Qc8

			variation => N__('Sicilian: Dragon, Classical, Tartakower Variation'),
			moves => {
			},
		},
		'r1r3k1/pp1bppb1/2np1np1/q6p/3NP2P/1BN1BP2/PPPQ2P1/2KR3R w - h6 0 13' => {
			eco => 'B79a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4 h5

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4 h5'),
			moves => {
			},
		},
		'r1r3k1/pp1bppbp/2np1np1/q7/3NP2P/1BN1BP2/PPPQ2P1/2KR3R b - h3 0 12' => {
			eco => 'B79a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4'),
			moves => {
				'c6e5' => 'r1r3k1/pp1bppbp/3p1np1/q3n3/3NP2P/1BN1BP2/PPPQ2P1/2KR3R w - - 1 13',
				'h7h5' => 'r1r3k1/pp1bppb1/2np1np1/q6p/3NP2P/1BN1BP2/PPPQ2P1/2KR3R w - h6 0 13',
			},
		},
		'r1r3k1/pp1bppbp/2np1np1/q7/3NP3/1BN1BP2/PPPQ2PP/2KR3R w - - 9 12' => {
			eco => 'B78t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 11. Bb3 Rfc8'),
			moves => {
				'h2h4' => 'r1r3k1/pp1bppbp/2np1np1/q7/3NP2P/1BN1BP2/PPPQ2P1/2KR3R b - h3 0 12',
			},
		},
		'r1r3k1/pp1bppbp/3p1np1/q3n2P/3NP3/1BN1BP2/PPPQ2P1/2KR3R b - - 0 13' => {
			eco => 'B79f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4 Ne5 13. h5

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4 Ne5 13. h5'),
			moves => {
			},
		},
		'r1r3k1/pp1bppbp/3p1np1/q3n3/3NP1PP/1BN1BP2/PPPQ4/2KR3R b - g3 0 13' => {
			eco => 'B79e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4 Ne5 13. g4

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4 Ne5 13. g4'),
			moves => {
			},
		},
		'r1r3k1/pp1bppbp/3p1np1/q3n3/3NP2P/1BN1BP2/PPPQ2P1/1K1R3R b - - 2 13' => {
			eco => 'B79c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4 Ne5 13. Kb1

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4 Ne5 13. Kb1'),
			moves => {
				'e5c4' => 'r1r3k1/pp1bppbp/3p1np1/q7/2nNP2P/1BN1BP2/PPPQ2P1/1K1R3R w - - 3 14',
			},
		},
		'r1r3k1/pp1bppbp/3p1np1/q3n3/3NP2P/1BN1BP2/PPPQ2P1/2KR3R w - - 1 13' => {
			eco => 'B79b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4 Ne5

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4 Ne5'),
			moves => {
				'c1b1' => 'r1r3k1/pp1bppbp/3p1np1/q3n3/3NP2P/1BN1BP2/PPPQ2P1/1K1R3R b - - 2 13',
				'g2g4' => 'r1r3k1/pp1bppbp/3p1np1/q3n3/3NP1PP/1BN1BP2/PPPQ4/2KR3R b - g3 0 13',
				'h4h5' => 'r1r3k1/pp1bppbp/3p1np1/q3n2P/3NP3/1BN1BP2/PPPQ2P1/2KR3R b - - 0 13',
			},
		},
		'r1r3k1/pp1bppbp/3p1np1/q7/2nNP2P/1BN1BP2/PPPQ2P1/1K1R3R w - - 3 14' => {
			eco => 'B79d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5 11. Bb3 Rfc8 12. h4 Ne5 13. Kb1 Nc4

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line, 12. h4 Ne5 13. Kb1 Nc4'),
			moves => {
			},
		},
		'r1r3k1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R1R3K1 w - - 4 13' => {
			eco => 'D14n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3 Bxd3 9. Qxd3 Bd6 10. Bxd6 Qxd6 11. O-O O-O 12. Rfc1 Rfc8

			variation => N__('Slav: Exchange, 8. Bd3 Main Line, 12.Rfc1 Rfc8'),
			moves => {
			},
		},
		'r2n1rk1/ppp1qppp/3p1n2/1B2p1B1/3PP1b1/2P2N2/P1P2PPP/R2QR1K1 w - - 1 11' => {
			eco => 'C49o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Bxc3 8. bxc3 Qe7 9. Re1 Nd8 10. d4 Bg4

			variation => N__('Four Knights: Symmetrical, Metger, Capablanca Variation'),
			moves => {
			},
		},
		'r2q1knr/pppb1Bpp/1b1p4/n7/3PP3/1QN2N2/P4PPP/R1B2RK1 w - - 1 13' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Fraser Attack'),
			moves => {
				'b3c2' => 'r2q1knr/pppb1Bpp/1b1p4/n7/3PP3/2N2N2/P1Q2PPP/R1B2RK1 b - - 2 13',
			},
		},
		'r2q1knr/pppb1Bpp/1b1p4/n7/3PP3/2N2N2/P1Q2PPP/R1B2RK1 b - - 2 13' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. Nc3 Bg4 10. Qa4 Bd7 11. Qb3 Na5 12. Bxf7+ Kf8 13. Qc2

			variation => N__('Evans Gambit: Fraser-Mortimer Attack'),
			moves => {
			},
		},
		'r2q1rk1/1b1n1ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2Q1RK1 w - - 6 15' => {
			eco => 'D31v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4 Bb4 6. e3 b5 7. Bd2 a5 8. axb5 Bxc3 9. Bxc3 cxb5 10. b3 Bb7 11. bxc4 b4 12. Bb2 Nf6 13. Bd3 Nbd7 14. O-O O-O

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line, 14.O-O O-O'),
			moves => {
			},
		},
		'r2q1rk1/1b1nbppp/p2p1n2/1pp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - c6 0 13' => {
			eco => 'C95g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 c5

			variation => N__('Spanish: Closed, Breyer, Gligoric Variation'),
			moves => {
			},
		},
		'r2q1rk1/1b1nbppp/pp1ppn2/8/2PQP3/1PN2NP1/P4PBP/R1BR2K1 w - - 1 12' => {
			eco => 'A30y',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. Rd1 a6 10. b3 Nbd7 11. e4 O-O

			variation => N__('English: Symmetrical, Hedgehog, Flexible Formation, 11. e4 O-O'),
			moves => {
			},
		},
		'r2q1rk1/1b2bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C96k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Bb7

			variation => N__('Spanish: Closed, Chigorin, 11. d4 Bb7'),
			moves => {
				'b1d2' => 'r2q1rk1/1b2bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 2 12',
			},
		},
		'r2q1rk1/1b2bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 2 12' => {
			eco => 'C96l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Bb7 12. Nbd2

			variation => N__('Spanish: Closed, Chigorin, 11. d4 Bb7 12.Nbd2'),
			moves => {
			},
		},
		'r2q1rk1/1b2bppp/ppnppn2/8/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 w - - 2 12' => {
			eco => 'B44y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O 10. O-O b6 11. Be3 Bb7

			variation => N__('Sicilian: Taimanov, Szen, Hedgehog, 11. Be3 Bb7'),
			moves => {
				'd1b3' => 'r2q1rk1/1b2bppp/ppnppn2/8/2P1P3/NQN1B3/PP2BPPP/R4RK1 b - - 3 12',
			},
		},
		'r2q1rk1/1b2bppp/ppnppn2/8/2P1P3/NQN1B3/PP2BPPP/R4RK1 b - - 3 12' => {
			eco => 'B44z',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O 10. O-O b6 11. Be3 Bb7 12. Qb3

			variation => N__('Sicilian: Taimanov, Szen, Hedgehog, 11. Be3 Bb7 12. Qb3'),
			moves => {
			},
		},
		'r2q1rk1/1bp1bppp/p1n1pn2/1p6/3P1B2/5NP1/PPQ1PPBP/RN1R2K1 b - - 5 11' => {
			eco => 'E05v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7 10. Bf4 Nc6 11. Rd1

			variation => N__('Catalan: Open, Classical, 8.Qxc4, 10. Bf4 Nc6 11. Rd1'),
			moves => {
			},
		},
		'r2q1rk1/1bp1bppp/p1n1pn2/1p6/3P1B2/5NP1/PPQ1PPBP/RN3RK1 w - - 4 11' => {
			eco => 'E05v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7 10. Bf4 Nc6

			variation => N__('Catalan: Open, Classical, 8.Qxc4, 10. Bf4 Nc6'),
			moves => {
				'f1d1' => 'r2q1rk1/1bp1bppp/p1n1pn2/1p6/3P1B2/5NP1/PPQ1PPBP/RN1R2K1 b - - 5 11',
			},
		},
		'r2q1rk1/1bp1bppp/p1n2n2/1p1pp3/4P3/1BPP1N2/PP2QPPP/RNB2RK1 w - - 1 10' => {
			eco => 'C86l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Qe2 b5 7. Bb3 O-O 8. c3 d5 9. d3 Bb7

			variation => N__('Spanish: Worrall Attack, 7...O-O 8. c3 d5 9. d3 Bb7'),
			moves => {
			},
		},
		'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10' => {
			eco => 'C92j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4

			variation => N__('Spanish: Closed, Zaitsev, 10. d4'),
			moves => {
				'f8e8' => 'r2qr1k1/1bp1bppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11',
			},
		},
		'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/4P3/1B1P1N1P/PPP2PP1/RNBQR1K1 w - - 0 10' => {
			eco => 'C88i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. h3 Bb7 9. d3 d6

			variation => N__('Spanish: Closed, 8. h3 Bb7 9. d3 d6'),
			moves => {
			},
		},
		'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10' => {
			eco => 'C92i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7

			variation => N__('Spanish: Closed, Zaitsev (Flohr) Variation'),
			moves => {
				'd2d4' => 'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10',
			},
		},
		'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/P3P3/1B1P1N2/1PP2PPP/RNBQR1K1 w - - 0 10' => {
			eco => 'C88n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4 Bb7 9. d3 d6

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4 Bb7 9. d3 d6'),
			moves => {
				'b1c3' => 'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/P3P3/1BNP1N2/1PP2PPP/R1BQR1K1 b - - 1 10',
				'c2c3' => 'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/P3P3/1BPP1N2/1P3PPP/RNBQR1K1 b - - 0 10',
			},
		},
		'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/P3P3/1BNP1N2/1PP2PPP/R1BQR1K1 b - - 1 10' => {
			eco => 'C88o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4 Bb7 9. d3 d6 10. Nc3

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4 Bb7 9. d3 d6 10. Nc3'),
			moves => {
			},
		},
		'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/P3P3/1BPP1N2/1P3PPP/RNBQR1K1 b - - 0 10' => {
			eco => 'C88p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4 Bb7 9. d3 d6 10. c3

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4 Bb7 9. d3 d6 10. c3'),
			moves => {
			},
		},
		'r2q1rk1/1bp1bppp/p2p1n2/np2p3/4P3/2P2N1P/PPBP1PP1/RNBQR1K1 w - - 3 11' => {
			eco => 'C96e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 Bb7

			variation => N__('Spanish: Closed, Chigorin, 10... Bb7'),
			moves => {
			},
		},
		'r2q1rk1/1bpn1ppp/p1np1b2/1p2p3/P2PP3/1BP2N1P/1P3PP1/RNBQR1K1 w - - 1 12' => {
			eco => 'C92g',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Karpov Variation, 10. d4 Bf6 11. a4'),
			moves => {
				'b1a3' => 'r2q1rk1/1bpn1ppp/p1np1b2/1p2p3/P2PP3/NBP2N1P/1P3PP1/R1BQR1K1 b - - 2 12',
			},
		},
		'r2q1rk1/1bpn1ppp/p1np1b2/1p2p3/P2PP3/NBP2N1P/1P3PP1/R1BQR1K1 b - - 2 12' => {
			eco => 'C92h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nd7 10. d4 Bf6 11. a4 Bb7 12. Na3

			variation => N__('Spanish: Closed, Karpov Variation, 10. d4 Bf6 11. a4 Bb7 12. Na3'),
			moves => {
			},
		},
		'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 3 12' => {
			eco => 'C95f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7

			variation => N__('Spanish: Closed, Breyer, 10. d4 Nbd7 11.Nbd2 Bb7'),
			moves => {
				'a2a4' => 'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/P2PP3/1BP2N1P/1P1N1PP1/R1BQR1K1 b - a3 0 12',
				'b3c2' => 'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 4 12',
			},
		},
		'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 4 12' => {
			eco => 'C95g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2

			variation => N__('Spanish: Closed, Breyer, 10. d4 Nbd7 11.Nbd2 Bb7 12. Bc2'),
			moves => {
				'c7c5' => 'r2q1rk1/1b1nbppp/p2p1n2/1pp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - c6 0 13',
				'f8e8' => 'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 5 13',
			},
		},
		'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP1N1PP1/R1BQR1K1 w - - 3 12' => {
			eco => 'C94f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d3 Nbd7 11. Nbd2 Bb7

			variation => N__('Spanish: Closed, Breyer, 10. d3 Nbd7 11.Nbd2 Bb7'),
			moves => {
			},
		},
		'r2q1rk1/1bpnbppp/p2p1n2/1p2p3/P2PP3/1BP2N1P/1P1N1PP1/R1BQR1K1 b - a3 0 12' => {
			eco => 'C95f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. a4

			variation => N__('Spanish: Closed, Breyer, 10. d4 Nbd7 11.Nbd2 Bb7 12. a4'),
			moves => {
			},
		},
		'r2q1rk1/1bppbppp/p1n2n2/1p2p3/4P3/1B1P1N1P/PPP2PP1/RNBQR1K1 b - - 0 9' => {
			eco => 'C88h',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, 8. h3'),
			moves => {
				'd7d6' => 'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/4P3/1B1P1N1P/PPP2PP1/RNBQR1K1 w - - 0 10',
			},
		},
		'r2q1rk1/1bppbppp/p1n2n2/1p2p3/4P3/1B3N1P/PPPP1PP1/RNBQR1K1 w - - 1 9' => {
			eco => 'C88h',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, 8. h3'),
			moves => {
				'd2d3' => 'r2q1rk1/1bppbppp/p1n2n2/1p2p3/4P3/1B1P1N1P/PPP2PP1/RNBQR1K1 b - - 0 9',
			},
		},
		'r2q1rk1/1bppbppp/p1n2n2/1p2p3/P3P3/1B1P1N2/1PP2PPP/RNBQR1K1 b - - 0 9' => {
			eco => 'C88m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4 Bb7 9. d3

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4 Bb7 9. d3'),
			moves => {
				'd7d6' => 'r2q1rk1/1bp1bppp/p1np1n2/1p2p3/P3P3/1B1P1N2/1PP2PPP/RNBQR1K1 w - - 0 10',
			},
		},
		'r2q1rk1/1bppbppp/p1n2n2/1p2p3/P3P3/1B3N2/1PPP1PPP/RNBQR1K1 w - - 1 9' => {
			eco => 'C88l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. a4 Bb7

			variation => N__('Spanish: Closed, Anti-Marshall 8. a4 Bb7'),
			moves => {
				'd2d3' => 'r2q1rk1/1bppbppp/p1n2n2/1p2p3/P3P3/1B1P1N2/1PP2PPP/RNBQR1K1 b - - 0 9',
			},
		},
		'r2q1rk1/1p1bbppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K w - - 5 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. Kh1 Bd7

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. Kh1 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/1p1bbppp/p1nppn2/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 0 11' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 Bd7 10. Nb3 a6

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 Bd7 10. Nb3 a6'),
			moves => {
				'a2a4' => 'r2q1rk1/1p1bbppp/p1nppn2/8/P3PP2/1NN1B3/1PP1B1PP/R2Q1RK1 b - a3 0 11',
			},
		},
		'r2q1rk1/1p1bbppp/p1nppn2/8/P2NPP2/2N1B3/1PP1B1PP/R2Q1RK1 w - - 1 11' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Nc6 10. a4 Bd7

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Nc6 10. a4 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/1p1bbppp/p1nppn2/8/P3PP2/1NN1B3/1PP1B1PP/R2Q1RK1 b - a3 0 11' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 Bd7 10. Nb3 a6 11. a4

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 Bd7 10. Nb3 a6 11. a4'),
			moves => {
			},
		},
		'r2q1rk1/1p1nbppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/2KR1B1R w - - 5 11' => {
			eco => 'B90r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 O-O 10. O-O-O Nbd7

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 O-O 10.O-O-O Nbd7'),
			moves => {
			},
		},
		'r2q1rk1/1p2ppbp/2bp1np1/p7/2PBP3/1PN2P2/P2QB1PP/R4RK1 b - - 0 13' => {
			eco => 'B38z',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Qd2 Nxd4 11. Bxd4 Bc6 12. f3 a5 13. b3

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2 Nxd4, 12. f3 a5 13. b3'),
			moves => {
			},
		},
		'r2q1rk1/1p2ppbp/2bp1np1/p7/2PBP3/2N2P2/PP1QB1PP/R4RK1 w - a6 0 13' => {
			eco => 'B38y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Qd2 Nxd4 11. Bxd4 Bc6 12. f3 a5

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2 Nxd4, 12. f3 a5'),
			moves => {
				'b2b3' => 'r2q1rk1/1p2ppbp/2bp1np1/p7/2PBP3/1PN2P2/P2QB1PP/R4RK1 b - - 0 13',
			},
		},
		'r2q1rk1/1ppbbppp/p1np1n2/4p3/B2PP3/2P2N2/PP1N1PPP/R1BQR1K1 b - - 2 9' => {
			eco => 'C87l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O 8. d4 Bd7 9. Nbd2

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O 8. d4 Bd7 9.Nbd2'),
			moves => {
				'f8e8' => 'r2qr1k1/1ppbbppp/p1np1n2/4p3/B2PP3/2P2N2/PP1N1PPP/R1BQR1K1 w - - 3 10',
			},
		},
		'r2q1rk1/1ppbbppp/p1np1n2/4p3/B2PP3/2P2N2/PP3PPP/RNBQR1K1 w - - 1 9' => {
			eco => 'C87k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O 8. d4 Bd7

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O 8. d4 Bd7'),
			moves => {
				'b1d2' => 'r2q1rk1/1ppbbppp/p1np1n2/4p3/B2PP3/2P2N2/PP1N1PPP/R1BQR1K1 b - - 2 9',
			},
		},
		'r2q1rk1/1ppbbppp/p1np1n2/4p3/B3P3/2P2N1P/PP1P1PP1/RNBQR1K1 w - - 1 9' => {
			eco => 'C87o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O 8. h3 Bd7

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O 8. h3 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/1ppbnpbp/3p1np1/p2Pp3/2P1P3/P1N5/1P1NBPPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'E97y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O Nc6 8. d5 Ne7 9. Nd2 a5 10. a3 Bd7

			variation => N__('King\'s Indian: Mar del Plata, 9. Nd2 a5 10. a3 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/2p1b1pp/p1n1b3/1p1pPp2/4n3/1NP2N2/PPB2PPP/R1BQ1RK1 b - - 1 12' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2 O-O 11. Bc2 f5 12. Nb3

			variation => N__('Spanish: Open, Classical, 10.Nbd2 O-O 11. Bc2 f5 12. Nb3'),
			moves => {
			},
		},
		'r2q1rk1/2p1b1pp/p1n1b3/1p1pPp2/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 w - f6 0 12' => {
			eco => 'C83',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Classical, 10.Nbd2 O-O 11. Bc2'),
			moves => {
				'd2b3' => 'r2q1rk1/2p1b1pp/p1n1b3/1p1pPp2/4n3/1NP2N2/PPB2PPP/R1BQ1RK1 b - - 1 12',
				'e5f6' => 'r2q1rk1/2p1b1pp/p1n1bP2/1p1p4/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 b - - 0 12',
			},
		},
		'r2q1rk1/2p1b1pp/p1n1bP2/1p1p4/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 b - - 0 12' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2 O-O 11. Bc2 f5 12. exf6

			variation => N__('Spanish: Open, Classical, 10.Nbd2 O-O 11. Bc2 f5 12.exf6'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/2P1n3/1B3N2/PP2QPPP/RNBR2K1 b - c3 0 11' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. Rd1 O-O 11. c4

			variation => N__('Spanish: Open, Keres, 10. Rd1 O-O 11. c4'),
			moves => {
				'b5c4' => 'r2q1rk1/2p1bppp/p1n1b3/3pP3/2p1n3/1B3N2/PP2QPPP/RNBR2K1 w - - 0 12',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/3Nn3/1BP5/PP3PPP/RNBQR1K1 b - - 4 11' => {
			eco => 'C83',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Classical, 10. Re1'),
			moves => {
				'c6e5' => 'r2q1rk1/2p1bppp/p3b3/1p1pn3/3Nn3/1BP5/PP3PPP/RNBQR1K1 w - - 0 12',
				'd8d7' => 'r4rk1/2pqbppp/p1n1b3/1p1pP3/3Nn3/1BP5/PP3PPP/RNBQR1K1 w - - 5 12',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNBR2K1 w - - 5 11' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. Rd1 O-O

			variation => N__('Spanish: Open, Keres, 10. Rd1 O-O'),
			moves => {
				'c2c3' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP2QPPP/RNBR2K1 b - - 0 11',
				'c2c4' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/2P1n3/1B3N2/PP2QPPP/RNBR2K1 b - c3 0 11',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 w - - 3 11' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2 O-O

			variation => N__('Spanish: Open, Classical, 10.Nbd2 O-O'),
			moves => {
				'b3c2' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 b - - 4 11',
				'd1e2' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP1NQPPP/R1B2RK1 b - - 4 11',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP1NQPPP/R1B2RK1 b - - 4 11' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2 O-O 11. Qe2

			variation => N__('Spanish: Open, Classical, Malkin Variation'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP2QPPP/RNBR2K1 b - - 0 11' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. Rd1 O-O 11. c3

			variation => N__('Spanish: Open, Keres, 10. Rd1 O-O 11. c3'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQR1K1 w - - 3 11' => {
			eco => 'C83',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Classical, 10. Re1'),
			moves => {
				'f3d4' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/3Nn3/1BP5/PP3PPP/RNBQR1K1 b - - 4 11',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 b - - 4 11' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2 O-O 11. Bc2

			variation => N__('Spanish: Open, Classical, 10.Nbd2 O-O 11. Bc2'),
			moves => {
				'f7f5' => 'r2q1rk1/2p1b1pp/p1n1b3/1p1pPp2/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 w - f6 0 12',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/3pP3/2B1n3/5N2/PP2QPPP/RNBR2K1 b - - 0 12' => {
			eco => 'C81',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Keres, 10. Rd1 O-O 11. c4'),
			moves => {
				'd8d7' => 'r4rk1/2pqbppp/p1n1b3/3pP3/2B1n3/5N2/PP2QPPP/RNBR2K1 w - - 1 13',
				'e7c5' => 'r2q1rk1/2p2ppp/p1n1b3/2bpP3/2B1n3/5N2/PP2QPPP/RNBR2K1 w - - 1 13',
			},
		},
		'r2q1rk1/2p1bppp/p1n1b3/3pP3/2p1n3/1B3N2/PP2QPPP/RNBR2K1 w - - 0 12' => {
			eco => 'C81',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Keres, 10. Rd1 O-O 11. c4'),
			moves => {
				'b3c4' => 'r2q1rk1/2p1bppp/p1n1b3/3pP3/2B1n3/5N2/PP2QPPP/RNBR2K1 b - - 0 12',
			},
		},
		'r2q1rk1/2p1bppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQR1K1 w - - 7 13' => {
			eco => 'C82h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4 11. Nbd2 Be7 12. Re1 O-O

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4 11.Nbd2 Be7 12. Re1 O-O'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p1Pp3/4P1b1/1BP2N2/PP3PPP/RNBQR1K1 b - - 0 10' => {
			eco => 'C91e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. d5

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5'),
			moves => {
				'c6a5' => 'r2q1rk1/2p1bppp/p2p1n2/np1Pp3/4P1b1/1BP2N2/PP3PPP/RNBQR1K1 w - - 1 11',
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP1b1/1BP1BN2/PP3PPP/RN1QR1K1 b - - 2 10' => {
			eco => 'C91c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. Be3

			variation => N__('Spanish: Closed, Bogoljubow, 10. Be3'),
			moves => {
				'e5d4' => 'r2q1rk1/2p1bppp/p1np1n2/1p6/3pP1b1/1BP1BN2/PP3PPP/RN1QR1K1 w - - 0 11',
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP1b1/1BP2N2/PP3PPP/RNBQR1K1 w - - 1 10' => {
			eco => 'C91b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4

			variation => N__('Spanish: Closed, Bogoljubow Variation'),
			moves => {
				'c1e3' => 'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP1b1/1BP1BN2/PP3PPP/RN1QR1K1 b - - 2 10',
				'd4d5' => 'r2q1rk1/2p1bppp/p1np1n2/1p1Pp3/4P1b1/1BP2N2/PP3PPP/RNBQR1K1 b - - 0 10',
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1QP2N1P/PP3PP1/RNB1R1K1 b - - 0 11' => {
			eco => 'C92d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Be6 10. d4 Bxb3 11. Qxb3

			variation => N__('Spanish: Closed, Kholmov, 11.Qxb3'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1bP2N1P/PP3PP1/RNBQR1K1 w - - 0 11' => {
			eco => 'C92c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Kholmov Variation'),
			moves => {
				'd1b3' => 'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1QP2N1P/PP3PP1/RNB1R1K1 b - - 0 11',
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p2p3/P3P1b1/1BP2N2/1P1P1PPP/RNBQR1K1 w - - 1 10' => {
			eco => 'C90d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. a4 Bg4

			variation => N__('Spanish: Closed, 8... d6 9. a4 Bg4'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1np1n2/1p6/3pP1b1/1BP1BN2/PP3PPP/RN1QR1K1 w - - 0 11' => {
			eco => 'C91d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. Be3 exd4

			variation => N__('Spanish: Closed, Bogoljubow, 10. Be3 exd4'),
			moves => {
			},
		},
		'r2q1rk1/2p1bppp/p1npbn2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10' => {
			eco => 'C92c',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Kholmov Variation'),
			moves => {
				'e6b3' => 'r2q1rk1/2p1bppp/p1np1n2/1p2p3/3PP3/1bP2N1P/PP3PP1/RNBQR1K1 w - - 0 11',
			},
		},
		'r2q1rk1/2p1bppp/p1npbn2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10' => {
			eco => 'C92c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Be6

			variation => N__('Spanish: Closed, Kholmov Variation'),
			moves => {
				'd2d4' => 'r2q1rk1/2p1bppp/p1npbn2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10',
			},
		},
		'r2q1rk1/2p1bppp/p2p1n2/np1Pp3/4P1b1/1BP2N2/PP3PPP/RNBQR1K1 w - - 1 11' => {
			eco => 'C91f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. d5 Na5

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5 Na5'),
			moves => {
				'b3c2' => 'r2q1rk1/2p1bppp/p2p1n2/np1Pp3/4P1b1/2P2N2/PPB2PPP/RNBQR1K1 b - - 2 11',
			},
		},
		'r2q1rk1/2p1bppp/p2p1n2/np1Pp3/4P1b1/2P2N2/PPB2PPP/RNBQR1K1 b - - 2 11' => {
			eco => 'C91f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. d5 Na5 11. Bc2

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5 Na5'),
			moves => {
				'c7c6' => 'r2q1rk1/4bppp/p1pp1n2/np1Pp3/4P1b1/2P2N2/PPB2PPP/RNBQR1K1 w - - 0 12',
				'd8c8' => 'r1q2rk1/2p1bppp/p2p1n2/np1Pp3/4P1b1/2P2N2/PPB2PPP/RNBQR1K1 w - - 3 12',
			},
		},
		'r2q1rk1/2p1bppp/p3b3/1p1pn3/3Nn3/1BP5/PP3PPP/RNBQR1K1 w - - 0 12' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Re1 O-O 11. Nd4 Nxe5

			variation => N__('Spanish: Open, Classical, Breslau Variation'),
			moves => {
			},
		},
		'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 w - - 3 11' => {
			eco => 'C82k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O

			variation => N__('Spanish: Open, St. Petersburg Variation'),
			moves => {
				'b3c2' => 'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 b - - 4 11',
			},
		},
		'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 b - - 4 11' => {
			eco => 'C82l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2

			variation => N__('Spanish: Open, St. Petersburg, 11. Bc2'),
			moves => {
				'e4f2' => 'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/8/2P2N2/PPBN1nPP/R1BQ1RK1 w - - 0 12',
				'e6f5' => 'r2q1rk1/2p2ppp/p1n5/1pbpPb2/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 w - - 5 12',
				'f7f5' => 'r2q1rk1/2p3pp/p1n1b3/1pbpPp2/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 w - f6 0 12',
			},
		},
		'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/8/2P2N2/PPBN1RPP/R1BQ2K1 b - - 0 12' => {
			eco => 'C82o',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Dilworth Variation'),
			moves => {
				'f7f6' => 'r2q1rk1/2p3pp/p1n1bp2/1pbpP3/8/2P2N2/PPBN1RPP/R1BQ2K1 w - - 0 13',
			},
		},
		'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/8/2P2N2/PPBN1nPP/R1BQ1RK1 w - - 0 12' => {
			eco => 'C82o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Nxf2

			variation => N__('Spanish: Open, Dilworth Variation'),
			moves => {
				'f1f2' => 'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/8/2P2N2/PPBN1RPP/R1BQ2K1 b - - 0 12',
			},
		},
		'r2q1rk1/2p2ppp/p1n1b3/2bpP3/2B1n3/5N2/PP2QPPP/RNBR2K1 w - - 1 13' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. Rd1 O-O 11. c4 bxc4 12. Bxc4 Bc5

			variation => N__('Spanish: Open, Keres, 10. Rd1 O-O 11. c4 bxc4 12.Bxc4 Bc5'),
			moves => {
			},
		},
		'r2q1rk1/2p2ppp/p1n5/1pbpPb2/4n3/1NP2N2/PPB2PPP/R1BQ1RK1 b - - 6 12' => {
			eco => 'C82n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Bf5 12. Nb3

			variation => N__('Spanish: Open, Baguio, 12. Nb3'),
			moves => {
			},
		},
		'r2q1rk1/2p2ppp/p1n5/1pbpPb2/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 w - - 5 12' => {
			eco => 'C82n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Bf5

			variation => N__('Spanish: Open, Baguio Variation'),
			moves => {
				'd2b3' => 'r2q1rk1/2p2ppp/p1n5/1pbpPb2/4n3/1NP2N2/PPB2PPP/R1BQ1RK1 b - - 6 12',
			},
		},
		'r2q1rk1/2p3pp/p1n1b3/1pbpPp2/4n3/2P2N2/PPBN1PPP/R1BQ1RK1 w - f6 0 12' => {
			eco => 'C82m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 f5

			variation => N__('Spanish: Open, St. Petersburg, 11. Bc2 f5'),
			moves => {
			},
		},
		'r2q1rk1/2p3pp/p1n1bP2/1p1p4/8/2P2N2/PPBN1KPP/R1BQ4 b - - 0 14' => {
			eco => 'C82p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Nxf2 12. Rxf2 f6 13. exf6 Bxf2+ 14. Kxf2

			variation => N__('Spanish: Open, Dilworth, 14.Kxf2'),
			moves => {
				'd8f6' => 'r4rk1/2p3pp/p1n1bq2/1p1p4/8/2P2N2/PPBN1KPP/R1BQ4 w - - 0 15',
			},
		},
		'r2q1rk1/2p3pp/p1n1bP2/1p1p4/8/2P2N2/PPBN1bPP/R1BQ2K1 w - - 0 14' => {
			eco => 'C82o',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Dilworth, 12.Rxf2 f6 13.exf6'),
			moves => {
				'g1f2' => 'r2q1rk1/2p3pp/p1n1bP2/1p1p4/8/2P2N2/PPBN1KPP/R1BQ4 b - - 0 14',
			},
		},
		'r2q1rk1/2p3pp/p1n1bP2/1pbp4/8/2P2N2/PPBN1RPP/R1BQ2K1 b - - 0 13' => {
			eco => 'C82o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Nxf2 12. Rxf2 f6 13. exf6

			variation => N__('Spanish: Open, Dilworth, 12.Rxf2 f6 13.exf6'),
			moves => {
				'c5f2' => 'r2q1rk1/2p3pp/p1n1bP2/1p1p4/8/2P2N2/PPBN1bPP/R1BQ2K1 w - - 0 14',
			},
		},
		'r2q1rk1/2p3pp/p1n1bp2/1pbpP3/8/2P2N2/PPBN1RPP/R1BQ2K1 w - - 0 13' => {
			eco => 'C82o',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Dilworth Variation'),
			moves => {
				'e5f6' => 'r2q1rk1/2p3pp/p1n1bP2/1pbp4/8/2P2N2/PPBN1RPP/R1BQ2K1 b - - 0 13',
			},
		},
		'r2q1rk1/3bbppp/ppnppn2/8/2P1P3/N1N1B3/PP2BPPP/R2Q1RK1 w - - 2 12' => {
			eco => 'B44x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nb5 d6 6. c4 Nf6 7. N1c3 a6 8. Na3 Be7 9. Be2 O-O 10. O-O b6 11. Be3 Bd7

			variation => N__('Sicilian: Taimanov, Szen, Hedgehog, 11. Be3 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/3nbppp/bpp1pn2/p2p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 w - - 2 11' => {
			eco => 'E09e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. b3 a5 10. Bb2 Ba6

			variation => N__('Catalan: Closed, Main Line, Sokolsky Variation'),
			moves => {
			},
		},
		'r2q1rk1/3nppbp/3p1np1/2pP4/4P3/2N2NP1/PP3PKP/R1BQ3R w - - 5 12' => {
			eco => 'A59j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7'),
			moves => {
				'h1e1' => 'r2q1rk1/3nppbp/3p1np1/2pP4/4P3/2N2NP1/PP3PKP/R1BQR3 b - - 6 12',
				'h2h3' => 'r2q1rk1/3nppbp/3p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQ3R b - - 0 12',
			},
		},
		'r2q1rk1/3nppbp/3p1np1/2pP4/4P3/2N2NP1/PP3PKP/R1BQR3 b - - 6 12' => {
			eco => 'A59k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. Re1

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. Re1'),
			moves => {
				'd8a5' => 'r4rk1/3nppbp/3p1np1/q1pP4/4P3/2N2NP1/PP3PKP/R1BQR3 w - - 7 13',
				'f6g4' => 'r2q1rk1/3nppbp/3p2p1/2pP4/4P1n1/2N2NP1/PP3PKP/R1BQR3 w - - 7 13',
			},
		},
		'r2q1rk1/3nppbp/3p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQ3R b - - 0 12' => {
			eco => 'A59n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3'),
			moves => {
				'a8a6' => '3q1rk1/3nppbp/r2p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQ3R w - - 1 13',
				'd8a5' => 'r4rk1/3nppbp/3p1np1/q1pP4/4P3/2N2NPP/PP3PK1/R1BQ3R w - - 1 13',
				'd8b6' => 'r4rk1/3nppbp/1q1p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQ3R w - - 1 13',
			},
		},
		'r2q1rk1/3nppbp/3p2p1/2pP4/4P1n1/2N2NP1/PP3PKP/R1BQR3 w - - 7 13' => {
			eco => 'A59m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. Re1 Ng4

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. Re1 Ng4'),
			moves => {
			},
		},
		'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/1RBQ1RK1 b - - 6 11' => {
			eco => 'A58q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 O-O 10. O-O Nbd7 11. Rb1

			variation => N__('Benko Gambit: Fianchetto, Main Line, 11. Rb1'),
			moves => {
			},
		},
		'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 11' => {
			eco => 'A58p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 O-O 10. O-O Nbd7

			variation => N__('Benko Gambit: Fianchetto, Main Line'),
			moves => {
				'a1b1' => 'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/1RBQ1RK1 b - - 6 11',
				'd1c2' => 'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 6 11',
				'f1e1' => 'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQR1K1 b - - 6 11',
			},
		},
		'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQR1K1 b - - 6 11' => {
			eco => 'A58r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 O-O 10. O-O Nbd7 11. Re1

			variation => N__('Benko Gambit: Fianchetto, Main Line, 11. Re1'),
			moves => {
			},
		},
		'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 6 11' => {
			eco => 'A58s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 O-O 10. O-O Nbd7 11. Qc2

			variation => N__('Benko Gambit: Fianchetto, Main Line, 11. Qc2'),
			moves => {
				'd8b6' => 'r4rk1/3nppbp/bq1p1np1/2pP4/8/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 7 12',
			},
		},
		'r2q1rk1/4bppp/p1pp1n2/np1Pp3/4P1b1/2P2N1P/PPB2PP1/RNBQR1K1 b - - 0 12' => {
			eco => 'C91h',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5 Na5 11. Bc2 c6'),
			moves => {
				'g4c8' => 'r1bq1rk1/4bppp/p1pp1n2/np1Pp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 w - - 1 13',
			},
		},
		'r2q1rk1/4bppp/p1pp1n2/np1Pp3/4P1b1/2P2N2/PPB2PPP/RNBQR1K1 w - - 0 12' => {
			eco => 'C91h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. d4 Bg4 10. d5 Na5 11. Bc2 c6

			variation => N__('Spanish: Closed, Bogoljubow, 10. d5 Na5 11. Bc2 c6'),
			moves => {
				'h2h3' => 'r2q1rk1/4bppp/p1pp1n2/np1Pp3/4P1b1/2P2N1P/PPB2PP1/RNBQR1K1 b - - 0 12',
			},
		},
		'r2q1rk1/5pbp/p1np4/1p1Npb2/8/2P5/PPN2PPP/R2QKB1R w KQ - 2 14' => {
			eco => 'B33w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. c3 Bg7 12. exf5 Bxf5 13. Nc2 O-O

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. c3 Bg7 12.exf5 Bxf5 13. Nc2 O-O'),
			moves => {
			},
		},
		'r2q1rk1/p2bppbp/1p4p1/n2P4/1Q2P3/3BBP2/4N1PP/2R2RK1 w - - 0 18' => {
			eco => 'D89e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. Rc1 Bxa2 15. Qa4 Be6 16. d5 Bd7 17. Qb4 b6

			variation => N__('Grünfeld: Classical Exchange, 14. Rc1 Bxa2 15. Qa4 with 17. Qb4 b6'),
			moves => {
			},
		},
		'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNBR2K1 w - - 2 10' => {
			eco => 'E08n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Ba6

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Ba6'),
			moves => {
				'b1d2' => 'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PP4/1P3NP1/P1QNPPBP/R1BR2K1 b - - 3 10',
			},
		},
		'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PP4/1P3NP1/P1QNPPBP/R1BR2K1 b - - 3 10' => {
			eco => 'E08o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Ba6 10. Nbd2

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Ba6 10.Nbd2'),
			moves => {
			},
		},
		'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PPP3/1P3NP1/P1QN1PBP/R1B2RK1 b - - 0 10' => {
			eco => 'E09i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Ba6 10. b3

			variation => N__('Catalan: Closed, Main Line, 9. e4 Ba6 10. b3'),
			moves => {
			},
		},
		'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PPP3/5NP1/PPQN1PBP/R1B2RK1 w - - 1 10' => {
			eco => 'E09i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Ba6

			variation => N__('Catalan: Closed, Main Line, 9. e4 Ba6'),
			moves => {
				'b2b3' => 'r2q1rk1/p2nbppp/bpp1pn2/3p4/2PPP3/1P3NP1/P1QN1PBP/R1B2RK1 b - - 0 10',
			},
		},
		'r2q1rk1/p3bppp/1pn1bn2/2pp2B1/3P4/2N2NP1/PP2PPBP/2RQ1RK1 w - - 0 11' => {
			eco => 'D34o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 Be6 10. Rc1 b6

			variation => N__('QGD Tarrasch: Stoltz Variation'),
			moves => {
			},
		},
		'r2q1rk1/p4pbp/2p1b1p1/2Bnp3/8/2N2P2/PPPQ2PP/2KR1B1R w - - 2 14' => {
			eco => 'B76u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O d5 10. exd5 Nxd5 11. Nxc6 bxc6 12. Bd4 e5 13. Bc5 Be6

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O d5: 12. Bd4 e5 13. Bc5 Be6'),
			moves => {
			},
		},
		'r2q1rk1/pb1n1ppp/2pbpn2/1p6/3P4/2N1PN2/PPQ1BPPP/R1B2RK1 w - - 2 11' => {
			eco => 'D46m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 b5 10. Be2 Bb7

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... b5 10. Be2 Bb7'),
			moves => {
				'f1d1' => 'r2q1rk1/pb1n1ppp/2pbpn2/1p6/3P4/2N1PN2/PPQ1BPPP/R1BR2K1 b - - 3 11',
			},
		},
		'r2q1rk1/pb1n1ppp/2pbpn2/1p6/3P4/2N1PN2/PPQ1BPPP/R1BR2K1 b - - 3 11' => {
			eco => 'D46m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 Bd6 7. Qc2 O-O 8. O-O dxc4 9. Bxc4 b5 10. Be2 Bb7 11. Rd1

			variation => N__('Semi-Slav: Chigorin, 7. Qc2 Main Line, 9... b5 10. Be2 Bb7 11. Rd1'),
			moves => {
			},
		},
		'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP3B/2NBPN2/PP2QPPP/R2R2K1 b - - 1 11' => {
			eco => 'D58j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Qe2 c5 11. Rfd1

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Qe2 c5 11.Rfd1'),
			moves => {
			},
		},
		'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP3B/2NBPN2/PP2QPPP/R4RK1 w - c6 0 11' => {
			eco => 'D58j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Qe2 c5

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Qe2 c5'),
			moves => {
				'f1d1' => 'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP3B/2NBPN2/PP2QPPP/R2R2K1 b - - 1 11',
				'h4g3' => 'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP4/2NBPNB1/PP2QPPP/R4RK1 b - - 1 11',
			},
		},
		'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP3B/2NBPN2/PP3PPP/2RQ1RK1 w - c6 0 11' => {
			eco => 'D58h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Rc1 c5

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Rc1 c5'),
			moves => {
			},
		},
		'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP4/2NBPNB1/PP2QPPP/R4RK1 b - - 1 11' => {
			eco => 'D58j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Qe2 c5 11. Bg3

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Qe2 c5 11. Bg3'),
			moves => {
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1Q1PPBP/RNBR2K1 w - - 2 10' => {
			eco => 'E08p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7'),
			moves => {
				'b1c3' => 'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1PN2NP1/P1Q1PPBP/R1BR2K1 b - - 3 10',
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/P1QNPPBP/R1B2RK1 w - - 1 10' => {
			eco => 'E09e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. b3 Bb7

			variation => N__('Catalan: Closed, Main Line, 9. b3 Bb7'),
			moves => {
				'c1b2' => 'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 b - - 2 10',
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 b - - 2 10' => {
			eco => 'E09f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. b3 Bb7 10. Bb2

			variation => N__('Catalan: Closed, Main Line, 9. b3 Bb7 10. Bb2'),
			moves => {
				'a8c8' => '2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1P3NP1/PBQNPPBP/R4RK1 w - - 3 11',
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1PN2NP1/P1Q1PPBP/R1BR2K1 b - - 3 10' => {
			eco => 'E08p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3

			variation => N__('Catalan: Closed, 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3'),
			moves => {
				'a8c8' => '2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PP4/1PN2NP1/P1Q1PPBP/R1BR2K1 w - - 4 11',
				'b6b5' => 'r2q1rk1/pb1nbppp/2p1pn2/1p1p4/2PP4/1PN2NP1/P1Q1PPBP/R1BR2K1 w - - 0 11',
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1P3NP1/P1QN1PBP/R1B2RK1 b - - 0 10' => {
			eco => 'E09n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Bb7 10. b3

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7 10. b3'),
			moves => {
				'a8c8' => '2rq1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1P3NP1/P1QN1PBP/R1B2RK1 w - - 1 11',
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/5NP1/PPQN1PBP/R1B2RK1 w - - 1 10' => {
			eco => 'E09l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Bb7

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7'),
			moves => {
				'b2b3' => 'r2q1rk1/pb1nbppp/1pp1pn2/3p4/2PPP3/1P3NP1/P1QN1PBP/R1B2RK1 b - - 0 10',
				'e4e5' => 'r2q1rk1/pb1nbppp/1pp1pn2/3pP3/2PP4/5NP1/PPQN1PBP/R1B2RK1 b - - 0 10',
			},
		},
		'r2q1rk1/pb1nbppp/1pp1pn2/3pP3/2PP4/5NP1/PPQN1PBP/R1B2RK1 b - - 0 10' => {
			eco => 'E09m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. Nbd2 b6 9. e4 Bb7 10. e5

			variation => N__('Catalan: Closed, Main Line, 9. e4 Bb7 10. e5'),
			moves => {
			},
		},
		'r2q1rk1/pb1nbppp/2p1pn2/1p1p4/2PP4/1PN2NP1/P1Q1PPBP/R1BR2K1 w - - 0 11' => {
			eco => 'E08p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O Nbd7 7. Qc2 c6 8. b3 b6 9. Rd1 Bb7 10. Nc3 b5

			variation => N__('Catalan: Closed, Spassky Gambit'),
			moves => {
			},
		},
		'r2q1rk1/pb1pbppp/1pn1pn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 w - - 3 10' => {
			eco => 'A30s',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 O-O 9. Rd1 Nc6

			variation => N__('English: Symmetrical, Hedgehog, 8.Qxd4 O-O'),
			moves => {
			},
		},
		'r2q1rk1/pb1pppbp/1pn2np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 2 10' => {
			eco => 'B38i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 b6 9. O-O Bb7

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 b6 9.O-O Bb7'),
			moves => {
				'f2f3' => 'r2q1rk1/pb1pppbp/1pn2np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 b - - 0 10',
			},
		},
		'r2q1rk1/pb1pppbp/1pn2np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 b - - 0 10' => {
			eco => 'B38j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 b6 9. O-O Bb7 10. f3

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 b6 9.O-O Bb7 10. f3'),
			moves => {
				'd8b8' => 'rq3rk1/pb1pppbp/1pn2np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 w - - 1 11',
			},
		},
		'r2q1rk1/pb2bppp/1pn1p3/8/3P3P/2PB1N2/P4PP1/R1BQR1K1 b - h3 0 13' => {
			eco => 'D41v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nxc3 11. bxc3 b6 12. Bd3 Bb7 13. h4

			variation => N__('QGD: Semi-Tarrasch, 7. Bc4 Main Line, 13. h4'),
			moves => {
			},
		},
		'r2q1rk1/pb2bppp/1pn1p3/8/3P4/2PB1N2/P1Q2PPP/R1B1R1K1 b - - 3 13' => {
			eco => 'D41u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nxc3 11. bxc3 b6 12. Bd3 Bb7 13. Qc2

			variation => N__('QGD: Semi-Tarrasch, 7. Bc4 Main Line, 13. Qc2'),
			moves => {
			},
		},
		'r2q1rk1/pb2bppp/1pn1p3/8/3P4/2PB1N2/P4PPP/R1BQR1K1 w - - 2 13' => {
			eco => 'D41t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 Nc6 7. Bc4 cxd4 8. exd4 Be7 9. O-O O-O 10. Re1 Nxc3 11. bxc3 b6 12. Bd3 Bb7

			variation => N__('QGD: Semi-Tarrasch, 7. Bc4 Main Line'),
			moves => {
				'd1c2' => 'r2q1rk1/pb2bppp/1pn1p3/8/3P4/2PB1N2/P1Q2PPP/R1B1R1K1 b - - 3 13',
				'h2h4' => 'r2q1rk1/pb2bppp/1pn1p3/8/3P3P/2PB1N2/P4PP1/R1BQR1K1 b - h3 0 13',
			},
		},
		'r2q1rk1/pbp1bppp/np2pn2/3PN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E18j',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8. Ne5 Na6'),
			moves => {
				'e6d5' => 'r2q1rk1/pbp1bppp/np3n2/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'r2q1rk1/pbp1bppp/np2pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 2 9' => {
			eco => 'E18j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. Ne5 Na6

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8. Ne5 Na6'),
			moves => {
				'c4d5' => 'r2q1rk1/pbp1bppp/np2pn2/3PN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r2q1rk1/pbp1bppp/np3n2/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E18k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. Ne5 Na6 9. cxd5 exd5

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8. Ne5 Na6 9.cxd5 exd5'),
			moves => {
			},
		},
		'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP2QPPP/R4RK1 b - - 5 10' => {
			eco => 'D58i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Qe2

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Qe2'),
			moves => {
				'c7c5' => 'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP3B/2NBPN2/PP2QPPP/R4RK1 w - c6 0 11',
			},
		},
		'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/2RQ1RK1 b - - 5 10' => {
			eco => 'D58h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Rc1

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Rc1'),
			moves => {
				'c7c5' => 'r2q1rk1/pb1nbpp1/1p2pn1p/2pp4/2PP3B/2NBPN2/PP3PPP/2RQ1RK1 w - c6 0 11',
			},
		},
		'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2Q1RK1 w - - 4 10' => {
			eco => 'D58h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7'),
			moves => {
				'a1c1' => 'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/2RQ1RK1 b - - 5 10',
				'd1e2' => 'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP2QPPP/R4RK1 b - - 5 10',
				'h4g3' => 'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP4/2NBPNB1/PP3PPP/R2Q1RK1 b - - 5 10',
			},
		},
		'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP4/2NBPNB1/PP3PPP/R2Q1RK1 b - - 5 10' => {
			eco => 'D58h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O Nbd7 10. Bg3

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O Nbd7 10. Bg3'),
			moves => {
			},
		},
		'r2q1rk1/pbpnbppp/1p2pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 2 9' => {
			eco => 'E18i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. Ne5 Nbd7

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8. Ne5 Nbd7'),
			moves => {
			},
		},
		'r2q1rk1/pbppbppp/np2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 7 8' => {
			eco => 'E18d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Na6

			variation => N__('Queen\'s Indian: 7. Nc3 Na6'),
			moves => {
			},
		},
		'r2q1rk1/pp1b1pbp/4p1p1/n2P4/1Q2P3/3BBP2/4N1PP/2R2RK1 w - - 0 18' => {
			eco => 'D89f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. Rc1 Bxa2 15. Qa4 Be6 16. d5 Bd7 17. Qb4 e6

			variation => N__('Grünfeld: Classical Exchange, 14. Rc1 Bxa2 15. Qa4 with 17. Qb4 e6'),
			moves => {
			},
		},
		'r2q1rk1/pp1bbpp1/2nppn1p/8/3NPP1B/2N5/PPPQ2PP/2KR1B1R w - - 2 11' => {
			eco => 'B64e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 h6 10. Bh4 Bd7

			variation => N__('Sicilian: Richter-Rauzer, 7. Qd2 Be7, 9. f4 h6 10. Bh4 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/pp1bbppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K b - - 2 10' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 Bd7 10. Kh1

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 Bd7 10. Kh1'),
			moves => {
			},
		},
		'r2q1rk1/pp1bbppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 1 10' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 Bd7

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 Bd7'),
			moves => {
				'd1e1' => 'r2q1rk1/pp1bbppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R3QRK1 b - - 2 10',
				'd4b3' => 'r2q1rk1/pp1bbppp/2nppn2/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 b - - 2 10',
				'g1h1' => 'r2q1rk1/pp1bbppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1R1K b - - 2 10',
			},
		},
		'r2q1rk1/pp1bbppp/2nppn2/8/3NPP2/2N1B3/PPP1B1PP/R3QRK1 b - - 2 10' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 Bd7 10. Qe1

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 Bd7 10. Qe1'),
			moves => {
			},
		},
		'r2q1rk1/pp1bbppp/2nppn2/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 b - - 2 10' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Nc6 7. O-O Be7 8. Be3 O-O 9. f4 Bd7 10. Nb3

			variation => N__('Sicilian: Scheveningen, Modern, 9. f4 Bd7 10. Nb3'),
			moves => {
				'a7a6' => 'r2q1rk1/1p1bbppp/p1nppn2/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 0 11',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2BNP1P1/2N1BP2/PPPQ3P/R3K2R b KQ g3 0 10' => {
			eco => 'B77h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. g4

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Bd7 10. g4'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2BNP2P/2N1BP2/PPPQ2P1/R3K2R b KQ h3 0 10' => {
			eco => 'B77i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. h4

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Bd7 10. h4'),
			moves => {
				'a8c8' => '2rq1rk1/pp1bppbp/2np1np1/8/2BNP2P/2N1BP2/PPPQ2P1/R3K2R w KQ - 1 11',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/2KR3R b - - 6 10' => {
			eco => 'B78a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O

			variation => N__('Sicilian: Dragon, Yugoslav, 10.O-O-O'),
			moves => {
				'a8c8' => '2rq1rk1/pp1bppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/2KR3R w - - 7 11',
				'c6e5' => 'r2q1rk1/pp1bppbp/3p1np1/4n3/2BNP3/2N1BP2/PPPQ2PP/2KR3R w - - 7 11',
				'd8a5' => 'r4rk1/pp1bppbp/2np1np1/q7/2BNP3/2N1BP2/PPPQ2PP/2KR3R w - - 7 11',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/R3K2R w KQ - 5 10' => {
			eco => 'B77f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Bd7'),
			moves => {
				'c4b3' => 'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1BP2/PPPQ2PP/R3K2R b KQ - 6 10',
				'e1c1' => 'r2q1rk1/pp1bppbp/2np1np1/8/2BNP3/2N1BP2/PPPQ2PP/2KR3R b - - 6 10',
				'g2g4' => 'r2q1rk1/pp1bppbp/2np1np1/8/2BNP1P1/2N1BP2/PPPQ3P/R3K2R b KQ g3 0 10',
				'h2h4' => 'r2q1rk1/pp1bppbp/2np1np1/8/2BNP2P/2N1BP2/PPPQ2P1/R3K2R b KQ h3 0 10',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2P1P3/2N1B3/PPN1BPPP/R2Q1RK1 b - - 3 10' => {
			eco => 'B38s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Nc2

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Nc2'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1B3/PP1QBPPP/R4RK1 b - - 3 10' => {
			eco => 'B38v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Qd2

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2'),
			moves => {
				'c6d4' => 'r2q1rk1/pp1bppbp/3p1np1/8/2PnP3/2N1B3/PP1QBPPP/R4RK1 w - - 0 11',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/2RQ1RK1 b - - 3 10' => {
			eco => 'B38u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Rc1

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Rc1'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/R2Q1RK1 w - - 2 10' => {
			eco => 'B38r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7'),
			moves => {
				'a1c1' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1B3/PP2BPPP/2RQ1RK1 b - - 3 10',
				'd1d2' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1B3/PP1QBPPP/R4RK1 b - - 3 10',
				'd4c2' => 'r2q1rk1/pp1bppbp/2np1np1/8/2P1P3/2N1B3/PPN1BPPP/R2Q1RK1 b - - 3 10',
				'f2f3' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 b - - 0 10',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 b - - 0 10' => {
			eco => 'B38t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. f3

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. f3'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNPP2/2N1B3/PP1QB1PP/R3K2R b KQ - 4 10' => {
			eco => 'E79e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3 Bd7 10. Qd2

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line, 9... Bd7 10. Qd2'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNPP2/2N1B3/PP2B1PP/R2Q1RK1 b - - 4 10' => {
			eco => 'E79f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3 Bd7 10. O-O

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line, 9... Bd7 10.O-O'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/2PNPP2/2N1B3/PP2B1PP/R2QK2R w KQ - 3 10' => {
			eco => 'E79e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3 Bd7

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line, 9... Bd7'),
			moves => {
				'd1d2' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNPP2/2N1B3/PP1QB1PP/R3K2R b KQ - 4 10',
				'e1g1' => 'r2q1rk1/pp1bppbp/2np1np1/8/2PNPP2/2N1B3/PP2B1PP/R2Q1RK1 b - - 4 10',
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/3NP1P1/2N1BP2/PPPQ3P/2KR1B1R b - g3 0 10' => {
			eco => 'B76i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Bd7 10. g4

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Bd7 10. g4'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1B2P/PPP2PP1/R2QK2R w KQ - 1 10' => {
			eco => 'B35p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 d6 9. h3 Bd7

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 d6 9. h3 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1BP2/PPP3PP/R2QK2R w KQ - 1 10' => {
			eco => 'B35r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 O-O 8. Bb3 d6 9. f3 Bd7

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 O-O 8. Bb3 d6 9. f3 Bd7'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/1BN1BP2/PPPQ2PP/R3K2R b KQ - 6 10' => {
			eco => 'B77g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. Bb3

			variation => N__('Sicilian: Dragon, Yugoslav, 9. Bc4 Bd7 10. Bb3'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R w - - 5 10' => {
			eco => 'B76h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Bd7

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Bd7'),
			moves => {
				'g2g4' => 'r2q1rk1/pp1bppbp/2np1np1/8/3NP1P1/2N1BP2/PPPQ3P/2KR1B1R b - g3 0 10',
			},
		},
		'r2q1rk1/pp1bppbp/3p1np1/4n3/2BNP3/2N1BP2/PPPQ2PP/2KR3R w - - 7 11' => {
			eco => 'B78b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Ne5

			variation => N__('Sicilian: Dragon, Yugoslav, 10.O-O-O Ne5'),
			moves => {
			},
		},
		'r2q1rk1/pp1bppbp/3p1np1/8/2PBP3/2N5/PP1QBPPP/R4RK1 b - - 0 11' => {
			eco => 'B38w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Qd2 Nxd4 11. Bxd4

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2 Nxd4 11.Bxd4'),
			moves => {
				'd7c6' => 'r2q1rk1/pp2ppbp/2bp1np1/8/2PBP3/2N5/PP1QBPPP/R4RK1 w - - 1 12',
			},
		},
		'r2q1rk1/pp1bppbp/3p1np1/8/2PnP3/2N1B3/PP1QBPPP/R4RK1 w - - 0 11' => {
			eco => 'B38w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Qd2 Nxd4

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2 Nxd4'),
			moves => {
				'e3d4' => 'r2q1rk1/pp1bppbp/3p1np1/8/2PBP3/2N5/PP1QBPPP/R4RK1 b - - 0 11',
			},
		},
		'r2q1rk1/pp1bppbp/6p1/n2P4/1Q2P3/3BBP2/4N1PP/2R2RK1 b - - 2 17' => {
			eco => 'D89d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. Rc1 Bxa2 15. Qa4 Be6 16. d5 Bd7 17. Qb4

			variation => N__('Grünfeld: Classical Exchange, 14. Rc1 Bxa2 15. Qa4 Be6 16. d5 Be7 17. Qb4'),
			moves => {
				'b7b6' => 'r2q1rk1/p2bppbp/1p4p1/n2P4/1Q2P3/3BBP2/4N1PP/2R2RK1 w - - 0 18',
				'e7e6' => 'r2q1rk1/pp1b1pbp/4p1p1/n2P4/1Q2P3/3BBP2/4N1PP/2R2RK1 w - - 0 18',
			},
		},
		'r2q1rk1/pp1bppbp/6p1/n2P4/Q3P3/3BBP2/4N1PP/2R2RK1 w - - 1 17' => {
			eco => 'D89c',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6 14. Rc1'),
			moves => {
				'a4b4' => 'r2q1rk1/pp1bppbp/6p1/n2P4/1Q2P3/3BBP2/4N1PP/2R2RK1 b - - 2 17',
			},
		},
		'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2B1P/PP4P1/R1BQ1RK1 b - - 0 12' => {
			eco => 'A68l',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O Nbd7 11. h3'),
			moves => {
				'f8e8' => 'r2qr1k1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2B1P/PP4P1/R1BQ1RK1 w - - 1 13',
			},
		},
		'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2b1P/PP2B1P1/R1BQ1RK1 w - - 0 12' => {
			eco => 'A68l',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O Nbd7 11. h3'),
			moves => {
				'e2f3' => 'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2B1P/PP4P1/R1BQ1RK1 b - - 0 12',
			},
		},
		'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PPb1/2N2N1P/PP2B1P1/R1BQ1RK1 b - - 0 11' => {
			eco => 'A68l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4 10. O-O Nbd7 11. h3

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O Nbd7 11. h3'),
			moves => {
				'g4f3' => 'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2b1P/PP2B1P1/R1BQ1RK1 w - - 0 12',
			},
		},
		'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PPb1/2N2N2/PP2B1PP/R1BQ1RK1 w - - 7 11' => {
			eco => 'A68k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4 10. O-O Nbd7

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O Nbd7'),
			moves => {
				'h2h3' => 'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PPb1/2N2N1P/PP2B1P1/R1BQ1RK1 b - - 0 11',
			},
		},
		'r2q1rk1/pp1n1pp1/2p1pnbp/8/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 w - - 0 12' => {
			eco => 'D19l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 h6

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 h6'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1pp1/2p1pnp1/8/PbBP4/2N1P3/1P2QPPP/R1B2RK1 w - - 0 12' => {
			eco => 'D19e',
                        # TRANSLATORS:

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Ne5 Nbd7, 12. Rd1'),
			moves => {
				'f1d1' => 'r2q1rk1/pp1n1pp1/2p1pnp1/8/PbBP4/2N1P3/1P2QPPP/R1BR2K1 b - - 1 12',
			},
		},
		'r2q1rk1/pp1n1pp1/2p1pnp1/8/PbBP4/2N1P3/1P2QPPP/R1BR2K1 b - - 1 12' => {
			eco => 'D19f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg6 10. Ne5 Nbd7 11. Nxg6 hxg6 12. Rd1

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Ne5 Nbd7, 12. Rd1'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/4P2b/Pb1P4/2NB1N2/1P2QPPP/R1B2RK1 b - - 0 12' => {
			eco => 'D19o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 Bh5 12. e5

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 Bh5 12. e5'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/5N2/PbBP4/2N1P3/1P3PPP/R1BQ1RK1 b - - 0 10' => {
			eco => 'D18s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4 Nbd7 10. Nxf5

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4 Nbd7 10.Nxf5'),
			moves => {
				'e6f5' => 'r2q1rk1/pp1n1ppp/2p2n2/5p2/PbBP4/2N1P3/1P3PPP/R1BQ1RK1 w - - 0 11',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/5b2/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 w - - 5 10' => {
			eco => 'D18r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4 Nbd7

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4 Nbd7'),
			moves => {
				'h4f5' => 'r2q1rk1/pp1n1ppp/2p1pn2/5N2/PbBP4/2N1P3/1P3PPP/R1BQ1RK1 b - - 0 10',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10' => {
			eco => 'D19h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7'),
			moves => {
				'e3e4' => 'r2q1rk1/pp1n1ppp/2p1pn2/5b2/PbBPP3/2N2N2/1P2QPPP/R1B2RK1 b - - 0 10',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/5b2/PbBPP3/2N2N2/1P2QPPP/R1B2RK1 b - - 0 10' => {
			eco => 'D19i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4'),
			moves => {
				'f5g4' => 'r2q1rk1/pp1n1ppp/2p1pn2/8/PbBPP1b1/2N2N2/1P2QPPP/R1B2RK1 w - - 1 11',
				'f5g6' => 'r2q1rk1/pp1n1ppp/2p1pnb1/8/PbBPP3/2N2N2/1P2QPPP/R1B2RK1 w - - 1 11',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/7b/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 w - - 3 12' => {
			eco => 'D19n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 Bh5

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 Bh5'),
			moves => {
				'e4e5' => 'r2q1rk1/pp1n1ppp/2p1pn2/4P2b/Pb1P4/2NB1N2/1P2QPPP/R1B2RK1 b - - 0 12',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pn2/8/PbBPP1b1/2N2N2/1P2QPPP/R1B2RK1 w - - 1 11' => {
			eco => 'D19i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg4

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg4'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pnN1/8/PbBP4/2N1P3/1P2QPPP/R1B2RK1 b - - 0 11' => {
			eco => 'D19e',
                        # TRANSLATORS:

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Ne5 Nbd7, 12. Rd1'),
			moves => {
				'h7g6' => 'r2q1rk1/pp1n1pp1/2p1pnp1/8/PbBP4/2N1P3/1P2QPPP/R1B2RK1 w - - 0 12',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pnb1/4N3/PbBP4/2N1P3/1P2QPPP/R1B2RK1 w - - 7 11' => {
			eco => 'D19e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg6 10. Ne5 Nbd7

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Ne5 Nbd7, 12. Rd1'),
			moves => {
				'e5g6' => 'r2q1rk1/pp1n1ppp/2p1pnN1/8/PbBP4/2N1P3/1P2QPPP/R1B2RK1 b - - 0 11',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pnb1/8/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 b - - 2 11' => {
			eco => 'D19k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3'),
			moves => {
				'f8e8' => 'r2qr1k1/pp1n1ppp/2p1pnb1/8/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 w - - 3 12',
				'g6h5' => 'r2q1rk1/pp1n1ppp/2p1pn2/7b/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 w - - 3 12',
				'h7h6' => 'r2q1rk1/pp1n1pp1/2p1pnbp/8/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 w - - 0 12',
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1BR2K1 w - - 7 11' => {
			eco => 'D19d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg6 10. Rd1 Nbd7

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Rd1 Nbd7'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1ppp/2p1pnb1/8/PbBPP3/2N2N2/1P2QPPP/R1B2RK1 w - - 1 11' => {
			eco => 'D19j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg6

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg6'),
			moves => {
				'c4d3' => 'r2q1rk1/pp1n1ppp/2p1pnb1/8/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 b - - 2 11',
			},
		},
		'r2q1rk1/pp1n1ppp/2p2n2/5p2/PbBP4/2N1P3/1P3PPP/R1BQ1RK1 w - - 0 11' => {
			eco => 'D18s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4 Nbd7 10. Nxf5 exf5

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4 Nbd7 10.Nxf5 exf5'),
			moves => {
				'd1c2' => 'r2q1rk1/pp1n1ppp/2p2n2/5p2/PbBP4/2N1P3/1PQ2PPP/R1B2RK1 b - - 1 11',
			},
		},
		'r2q1rk1/pp1n1ppp/2p2n2/5p2/PbBP4/2N1P3/1PQ2PPP/R1B2RK1 b - - 1 11' => {
			eco => 'D18t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4 Nbd7 10. Nxf5 exf5 11. Qc2

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4 Nbd7 10.Nxf5 exf5 11. Qc2'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/PB1NPPBP/R2Q1RK1 b - - 4 9' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3 Bd6 8. Bb2 O-O 9. Nbd2

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
			},
		},
		'r2q1rk1/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 w - - 3 9' => {
			eco => 'D11n',
                        # TRANSLATORS:

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'b1d2' => 'r2q1rk1/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/PB1NPPBP/R2Q1RK1 b - - 4 9',
			},
		},
		'r2q1rk1/pp1nbppp/2p1p3/5b2/2QPn3/2N1PNP1/PP3PBP/R1B2RK1 w - - 1 11' => {
			eco => 'D23s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7 8. O-O Be7 9. Nc3 O-O 10. e3 Ne4

			variation => N__('QGA: Mannheim, Main Line, 10. e3 Ne4'),
			moves => {
				'c4e2' => 'r2q1rk1/pp1nbppp/2p1p3/5b2/3Pn3/2N1PNP1/PP2QPBP/R1B2RK1 b - - 2 11',
			},
		},
		'r2q1rk1/pp1nbppp/2p1p3/5b2/3Pn3/2N1PNP1/PP2QPBP/R1B2RK1 b - - 2 11' => {
			eco => 'D23t',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7 8. O-O Be7 9. Nc3 O-O 10. e3 Ne4 11. Qe2

			variation => N__('QGA: Mannheim, Main Line, 10. e3 Ne4 11. Qe2'),
			moves => {
			},
		},
		'r2q1rk1/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/PB1NPPBP/R2Q1RK1 b - - 4 9' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3 Be7 8. Bb2 O-O 9. Nbd2

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
			},
		},
		'r2q1rk1/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 w - - 3 9' => {
			eco => 'D11n',
                        # TRANSLATORS:

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'b1d2' => 'r2q1rk1/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/PB1NPPBP/R2Q1RK1 b - - 4 9',
			},
		},
		'r2q1rk1/pp1nbppp/2p1pn2/5b2/2QP4/2N1PNP1/PP3PBP/R1B2RK1 b - - 0 10' => {
			eco => 'D23r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7 8. O-O Be7 9. Nc3 O-O 10. e3

			variation => N__('QGA: Mannheim, Main Line, 10. e3'),
			moves => {
				'f6e4' => 'r2q1rk1/pp1nbppp/2p1p3/5b2/2QPn3/2N1PNP1/PP3PBP/R1B2RK1 w - - 1 11',
			},
		},
		'r2q1rk1/pp1nbppp/2p1pn2/5b2/2QP4/2N2NP1/PP2PPBP/R1B2RK1 w - - 6 10' => {
			eco => 'D23p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7 8. O-O Be7 9. Nc3 O-O

			variation => N__('QGA: Mannheim, Main Line, 9...O-O'),
			moves => {
				'c1g5' => 'r2q1rk1/pp1nbppp/2p1pn2/5bB1/2QP4/2N2NP1/PP2PPBP/R4RK1 b - - 7 10',
				'e2e3' => 'r2q1rk1/pp1nbppp/2p1pn2/5b2/2QP4/2N1PNP1/PP3PBP/R1B2RK1 b - - 0 10',
			},
		},
		'r2q1rk1/pp1nbppp/2p1pn2/5bB1/2QP4/2N2NP1/PP2PPBP/R4RK1 b - - 7 10' => {
			eco => 'D23q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7 8. O-O Be7 9. Nc3 O-O 10. Bg5

			variation => N__('QGA: Mannheim, Main Line, 10. Bg5'),
			moves => {
			},
		},
		'r2q1rk1/pp2bpp1/2npbn1p/4p3/4P3/2N2N1P/PPP1BPP1/R1BQR1K1 w - - 1 11' => {
			eco => 'B58p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6'),
			moves => {
				'e2f1' => 'r2q1rk1/pp2bpp1/2npbn1p/4p3/4P3/2N2N1P/PPP2PP1/R1BQRBK1 b - - 2 11',
			},
		},
		'r2q1rk1/pp2bpp1/2npbn1p/4p3/4P3/2N2N1P/PPP2PP1/R1BQRBK1 b - - 2 11' => {
			eco => 'B58p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6 11. Bf1

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6 11. Bf1'),
			moves => {
				'c6b8' => 'rn1q1rk1/pp2bpp1/3pbn1p/4p3/4P3/2N2N1P/PPP2PP1/R1BQRBK1 w - - 3 12',
				'd8a5' => 'r4rk1/pp2bpp1/2npbn1p/q3p3/4P3/2N2N1P/PPP2PP1/R1BQRBK1 w - - 3 12',
			},
		},
		'r2q1rk1/pp2bppp/1nnpp3/8/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 11' => {
			eco => 'B05u',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. Nc3 O-O 9. Be3 Nc6 10. exd6 cxd6

			variation => N__('Alekhine: Modern, Main Line, 8. Nc3 O-O 9. Be3 Nc6 10.exd6 cxd6'),
			moves => {
			},
		},
		'r2q1rk1/pp2bppp/2N1bn2/3p2B1/2pP4/2N3P1/PP2PPBP/R2Q1RK1 b - - 0 11' => {
			eco => 'D34q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 c4 10. Ne5 Be6 11. Nxc6

			variation => N__('QGD Tarrasch: 9. Bg5 c4 10. Ne5 Be6 11.Nxc6'),
			moves => {
			},
		},
		'r2q1rk1/pp2bppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KRR3 w - - 4 11' => {
			eco => 'B53w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3 Nf6 8. Bg5 e6 9. O-O-O Be7 10. Rhe1 O-O

			variation => N__('Sicilian, Chekhover, Main Line, 10.Rhe1 O-O'),
			moves => {
				'd4d2' => 'r2q1rk1/pp2bppp/2bppn2/6B1/4P3/2N2N2/PPPQ1PPP/2KRR3 b - - 5 11',
			},
		},
		'r2q1rk1/pp2bppp/2bppn2/6B1/4P3/2N2N2/PPPQ1PPP/2KRR3 b - - 5 11' => {
			eco => 'B53x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3 Nf6 8. Bg5 e6 9. O-O-O Be7 10. Rhe1 O-O 11. Qd2

			variation => N__('Sicilian, Chekhover, Main Line, 11. Qd2'),
			moves => {
			},
		},
		'r2q1rk1/pp2bppp/2n1bn2/2pp2B1/3P4/2N2NP1/PP2PPBP/2RQ1RK1 b - - 8 10' => {
			eco => 'D34o',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9. Bg5 Be6'),
			moves => {
				'b7b6' => 'r2q1rk1/p3bppp/1pn1bn2/2pp2B1/3P4/2N2NP1/PP2PPBP/2RQ1RK1 w - - 0 11',
				'c5c4' => 'r2q1rk1/pp2bppp/2n1bn2/3p2B1/2pP4/2N2NP1/PP2PPBP/2RQ1RK1 w - - 0 11',
			},
		},
		'r2q1rk1/pp2bppp/2n1bn2/2pp2B1/3P4/2N2NP1/PP2PPBP/R2Q1RK1 w - - 7 10' => {
			eco => 'D34o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 Be6

			variation => N__('QGD Tarrasch: 9. Bg5 Be6'),
			moves => {
				'a1c1' => 'r2q1rk1/pp2bppp/2n1bn2/2pp2B1/3P4/2N2NP1/PP2PPBP/2RQ1RK1 b - - 8 10',
			},
		},
		'r2q1rk1/pp2bppp/2n1bn2/3p2B1/2pP4/2N2NP1/PP2PPBP/2RQ1RK1 w - - 0 11' => {
			eco => 'D34o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. Bg5 Be6 10. Rc1 c4

			variation => N__('QGD Tarrasch: Bogoljubow Variation'),
			moves => {
			},
		},
		'r2q1rk1/pp2bppp/2n1bn2/3pN1B1/2pP4/2N3P1/PP2PPBP/R2Q1RK1 w - - 2 11' => {
			eco => 'D34p',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 9. Bg5 c4'),
			moves => {
				'e5c6' => 'r2q1rk1/pp2bppp/2N1bn2/3p2B1/2pP4/2N3P1/PP2PPBP/R2Q1RK1 b - - 0 11',
			},
		},
		'r2q1rk1/pp2bppp/2npbn2/2p3B1/4P3/2N2N2/PPP1BPPP/R2Q1RK1 w - - 4 10' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'f1e1' => 'r2q1rk1/pp2bppp/2npbn2/2p3B1/4P3/2N2N2/PPP1BPPP/R2QR1K1 b - - 5 10',
			},
		},
		'r2q1rk1/pp2bppp/2npbn2/2p3B1/4P3/2N2N2/PPP1BPPP/R2QR1K1 b - - 5 10' => {
			eco => 'C41m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 Nf6 5. Nc3 Be7 6. Be2 O-O 7. O-O c5 8. Nf3 Nc6 9. Bg5 Be6 10. Re1

			variation => N__('Philidor: Berger Variation'),
			moves => {
			},
		},
		'r2q1rk1/pp2bppp/2npbn2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 6 10' => {
			eco => 'B59l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 Be6

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Be3 Be6'),
			moves => {
				'e2f3' => 'r2q1rk1/pp2bppp/2npbn2/4p3/4P3/1NN1BB2/PPP2PPP/R2Q1RK1 b - - 7 10',
			},
		},
		'r2q1rk1/pp2bppp/2npbn2/4p3/4P3/1NN1BB2/PPP2PPP/R2Q1RK1 b - - 7 10' => {
			eco => 'B59m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 Be6 10. Bf3

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be7 8.O-O O-O 9. Be3 Be6 10. Bf3'),
			moves => {
			},
		},
		'r2q1rk1/pp2nppp/2nb4/1B1p2B1/6b1/1N3N2/PPP2PPP/R2QR1K1 w - - 6 12' => {
			eco => 'C09q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nc6 6. Bb5 Bd6 7. O-O Ne7 8. dxc5 Bxc5 9. Nb3 Bd6 10. Re1 O-O 11. Bg5 Bg4

			variation => N__('French: Tarrasch, Open, Main Line, 9. Nb3 Bd6 10. Re1 O-O 11. Bg5 Bg4'),
			moves => {
			},
		},
		'r2q1rk1/pp2p2p/4bpp1/n2P4/4P3/3BBP2/P3N1PP/Q4RK1 w - - 0 16' => {
			eco => 'D89g',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, Sokolsky Variation'),
			moves => {
				'e3h6' => 'r2q1rk1/pp2p2p/4bppB/n2P4/4P3/3B1P2/P3N1PP/Q4RK1 b - - 1 16',
				'f1b1' => 'r2q1rk1/pp2p2p/4bpp1/n2P4/4P3/3BBP2/P3N1PP/QR4K1 b - - 1 16',
			},
		},
		'r2q1rk1/pp2p2p/4bpp1/n2P4/4P3/3BBP2/P3N1PP/QR4K1 b - - 1 16' => {
			eco => 'D89h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. d5 Bxa1 15. Qxa1 f6 16. Rb1

			variation => N__('Grünfeld: Classical Exchange, Main Line, Sokolsky, 16. Rb1'),
			moves => {
			},
		},
		'r2q1rk1/pp2p2p/4bppB/n2P4/4P3/3B1P2/P3N1PP/Q4RK1 b - - 1 16' => {
			eco => 'D89i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. d5 Bxa1 15. Qxa1 f6 16. Bh6

			variation => N__('Grünfeld: Classical Exchange, Main Line, Sokolsky, 16. Bh6'),
			moves => {
			},
		},
		'r2q1rk1/pp2pBbp/6p1/n1p5/3PP1b1/2P1BP2/P3N1PP/R2Q1RK1 b - - 0 12' => {
			eco => 'D87p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Bg4 11. f3 Na5 12. Bxf7+

			variation => N__('Grünfeld: Classical Exchange, Seville Variation'),
			moves => {
			},
		},
		'r2q1rk1/pp2pBbp/6p1/n7/3PP1b1/4BP2/P3N1PP/R2Q1RK1 b - - 0 13' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bxf7+

			variation => N__('Grünfeld: Classical Exchange, Main Line, Neo-Seville'),
			moves => {
			},
		},
		'r2q1rk1/pp2pp1p/4b1p1/n2P4/4P3/3BBP2/P3N1PP/Q4RK1 b - - 0 15' => {
			eco => 'D89g',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, Sokolsky Variation'),
			moves => {
				'f7f6' => 'r2q1rk1/pp2p2p/4bpp1/n2P4/4P3/3BBP2/P3N1PP/Q4RK1 w - - 0 16',
			},
		},
		'r2q1rk1/pp2pp1p/4b1p1/n2P4/4P3/3BBP2/P3N1PP/b2Q1RK1 w - - 0 15' => {
			eco => 'D89g',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, Sokolsky Variation'),
			moves => {
				'd1a1' => 'r2q1rk1/pp2pp1p/4b1p1/n2P4/4P3/3BBP2/P3N1PP/Q4RK1 b - - 0 15',
			},
		},
		'r2q1rk1/pp2ppbp/2bp1np1/8/2PBP3/2N2P2/PP1QB1PP/R4RK1 b - - 0 12' => {
			eco => 'B38x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 d6 9. O-O Bd7 10. Qd2 Nxd4 11. Bxd4 Bc6 12. f3

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2 Nxd4, 12. f3'),
			moves => {
				'a7a5' => 'r2q1rk1/1p2ppbp/2bp1np1/p7/2PBP3/2N2P2/PP1QB1PP/R4RK1 w - a6 0 13',
			},
		},
		'r2q1rk1/pp2ppbp/2bp1np1/8/2PBP3/2N5/PP1QBPPP/R4RK1 w - - 1 12' => {
			eco => 'B38w',
                        # TRANSLATORS:

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 d6 9.O-O Bd7 10. Qd2 Nxd4 11.Bxd4'),
			moves => {
				'f2f3' => 'r2q1rk1/pp2ppbp/2bp1np1/8/2PBP3/2N2P2/PP1QB1PP/R4RK1 b - - 0 12',
			},
		},
		'r2q1rk1/pp2ppbp/2n3p1/2p5/2BPP1b1/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11' => {
			eco => 'D87n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Bg4

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Bg4'),
			moves => {
				'f2f3' => 'r2q1rk1/pp2ppbp/2n3p1/2p5/2BPP1b1/2P1BP2/P3N1PP/R2Q1RK1 b - - 0 11',
			},
		},
		'r2q1rk1/pp2ppbp/2n3p1/2p5/2BPP1b1/2P1BP2/P3N1PP/R2Q1RK1 b - - 0 11' => {
			eco => 'D87n',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Bg4'),
			moves => {
				'c6a5' => 'r2q1rk1/pp2ppbp/6p1/n1p5/2BPP1b1/2P1BP2/P3N1PP/R2Q1RK1 w - - 1 12',
			},
		},
		'r2q1rk1/pp2ppbp/2n3p1/8/2BPP1b1/4B3/P3NPPP/R2Q1RK1 w - - 1 12' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4 Bg4'),
			moves => {
				'f2f3' => 'r2q1rk1/pp2ppbp/2n3p1/8/2BPP1b1/4BP2/P3N1PP/R2Q1RK1 b - - 0 12',
			},
		},
		'r2q1rk1/pp2ppbp/2n3p1/8/2BPP1b1/4BP2/P3N1PP/R2Q1RK1 b - - 0 12' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4 Bg4 12. f3'),
			moves => {
				'c6a5' => 'r2q1rk1/pp2ppbp/6p1/n7/2BPP1b1/4BP2/P3N1PP/R2Q1RK1 w - - 1 13',
			},
		},
		'r2q1rk1/pp2ppbp/2np1np1/8/2PNPPb1/2N1B3/PP2B1PP/R2QK2R w KQ - 3 10' => {
			eco => 'E79d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4 Nc6 9. Be3 Bg4

			variation => N__('King\'s Indian: Four Pawns Attack, Main Line, 9... Bg4'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/6B1/4P3/1NN5/PPP1BPPP/R2Q1R1K b - - 9 10' => {
			eco => 'B70w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Bg5 Nc6 9. Nb3 Be6 10. Kh1

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Bg5 Nc6 9. Nb3 Be6 10. Kh1'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/6B1/4P3/1NN5/PPP1BPPP/R2Q1RK1 w - - 8 10' => {
			eco => 'B70v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Bg5 Nc6 9. Nb3 Be6

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Bg5 Nc6 9. Nb3 Be6'),
			moves => {
				'g1h1' => 'r2q1rk1/pp2ppbp/2npbnp1/6B1/4P3/1NN5/PPP1BPPP/R2Q1R1K b - - 9 10',
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/8/2P1P3/2N5/PPN1BPPP/R1BQ1RK1 w - - 4 10' => {
			eco => 'B37m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9. O-O Be6

			variation => N__('Sicilian: Maroczy Bind, 6. Nc2 Nf6 7. Nc3 d6 8. Be2 O-O 9.O-O Be6'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/8/3NP1P1/2N1BP2/PPPQ3P/R3KB1R w KQ - 1 10' => {
			eco => 'B76e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. g4 Be6

			variation => N__('Sicilian: Dragon, Yugoslav, 9. g4 Be6'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/8/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 8 10' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6

			variation => N__('Sicilian: Dragon, Classical, 9. Nb3 Be6'),
			moves => {
				'f2f4' => 'r2q1rk1/pp2ppbp/2npbnp1/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 b - f3 0 10',
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/8/4P3/1NN5/PPP1BPPP/R1BQR1K1 w - - 8 10' => {
			eco => 'B70r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Re1 Nc6 9. Nb3 Be6

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Re1 Nc6 9. Nb3 Be6'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/2npbnp1/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 b - f3 0 10' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4

			variation => N__('Sicilian: Dragon, Classical, 9. Nb3 Be6 10. f4'),
			moves => {
				'a8c8' => '2rq1rk1/pp2ppbp/2npbnp1/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 11',
				'c6a5' => 'r2q1rk1/pp2ppbp/3pbnp1/n7/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 11',
				'd8c8' => 'r1q2rk1/pp2ppbp/2npbnp1/8/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 11',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/5b2/2P5/2NQ2P1/PP2PPBP/R1B2RK1 w - - 2 11' => {
			eco => 'A39n',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. Nf3 Nf6 6. O-O O-O 7. d4 cxd4 8. Nxd4 Nxd4 9. Qxd4 d6 10. Qd3 Bf5

			variation => N__('English: Symmetrical, Main Line 7. d4, 10. Qd3 Bf5'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/N4P2/2b1P3/2N1B3/PPP1B1PP/R2Q1RK1 b - - 0 12' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'c4e2' => 'r2q1rk1/pp2ppbp/3p1np1/N4P2/4P3/2N1B3/PPP1b1PP/R2Q1RK1 w - - 0 13',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/N4P2/4P3/2N1B3/PPP1Q1PP/R4RK1 b - - 0 13' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'd8a5' => 'r4rk1/pp2ppbp/3p1np1/q4P2/4P3/2N1B3/PPP1Q1PP/R4RK1 w - - 0 14',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/N4P2/4P3/2N1B3/PPP1b1PP/R2Q1RK1 w - - 0 13' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'd1e2' => 'r2q1rk1/pp2ppbp/3p1np1/N4P2/4P3/2N1B3/PPP1Q1PP/R4RK1 b - - 0 13',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/n4P2/2b1P3/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 12' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'b3a5' => 'r2q1rk1/pp2ppbp/3p1np1/N4P2/2b1P3/2N1B3/PPP1B1PP/R2Q1RK1 b - - 0 12',
				'e2d3' => 'r2q1rk1/pp2ppbp/3p1np1/n4P2/2b1P3/1NNBB3/PPP3PP/R2Q1RK1 b - - 2 12',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/n4P2/2b1P3/1NNBB3/PPP3PP/R2Q1RK1 b - - 2 12' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4 Na5 11. f5 Bc4 12. Bd3

			variation => N__('Sicilian: Dragon, Classical, Spielmann Variation'),
			moves => {
				'c4d3' => 'r2q1rk1/pp2ppbp/3p1np1/n4P2/4P3/1NNbB3/PPP3PP/R2Q1RK1 w - - 0 13',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/n4P2/4P3/1NNPB3/PP4PP/R2Q1RK1 b - - 0 13' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Spielmann Variation'),
			moves => {
				'd6d5' => 'r2q1rk1/pp2ppbp/5np1/n2p1P2/4P3/1NNPB3/PP4PP/R2Q1RK1 w - - 0 14',
			},
		},
		'r2q1rk1/pp2ppbp/3p1np1/n4P2/4P3/1NNbB3/PPP3PP/R2Q1RK1 w - - 0 13' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Spielmann Variation'),
			moves => {
				'c2d3' => 'r2q1rk1/pp2ppbp/3p1np1/n4P2/4P3/1NNPB3/PP4PP/R2Q1RK1 b - - 0 13',
			},
		},
		'r2q1rk1/pp2ppbp/3pbnp1/8/2P1P3/2N1B3/PP1QBPPP/R3K2R w KQ - 5 11' => {
			eco => 'B36p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Be3 O-O 10. Qd2 Be6

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Be3 O-O 10. Qd2 Be6'),
			moves => {
				'e1g1' => 'r2q1rk1/pp2ppbp/3pbnp1/8/2P1P3/2N1B3/PP1QBPPP/R4RK1 b - - 6 11',
			},
		},
		'r2q1rk1/pp2ppbp/3pbnp1/8/2P1P3/2N1B3/PP1QBPPP/R4RK1 b - - 6 11' => {
			eco => 'B36q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Nf6 6. Nc3 d6 7. Be2 Nxd4 8. Qxd4 Bg7 9. Be3 O-O 10. Qd2 Be6 11. O-O

			variation => N__('Sicilian: Maroczy Bind, 5... Nf6 6. Nc3 d6 7. Be2 Nxd4 8.Qxd4 Bg7 9. Be3 O-O 10. Qd2 Be6 11.O-O'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/3pbnp1/8/3BP3/2N2P2/PPPQ2PP/1K1R1B1R b - - 2 11' => {
			eco => 'B76l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Nxd4 10. Bxd4 Be6 11. Kb1

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Nxd4: 11. Kb1'),
			moves => {
				'd8c7' => 'r4rk1/ppq1ppbp/3pbnp1/8/3BP3/2N2P2/PPPQ2PP/1K1R1B1R w - - 3 12',
			},
		},
		'r2q1rk1/pp2ppbp/3pbnp1/8/3BP3/2N2P2/PPPQ2PP/2KR1B1R w - - 1 11' => {
			eco => 'B76k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Nxd4 10. Bxd4 Be6

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Nxd4 10.Bxd4 Be6'),
			moves => {
				'c1b1' => 'r2q1rk1/pp2ppbp/3pbnp1/8/3BP3/2N2P2/PPPQ2PP/1K1R1B1R b - - 2 11',
			},
		},
		'r2q1rk1/pp2ppbp/3pbnp1/n4P2/4P3/1NN1B3/PPP1B1PP/R2Q1RK1 b - - 0 11' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'e6c4' => 'r2q1rk1/pp2ppbp/3p1np1/n4P2/2b1P3/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 12',
			},
		},
		'r2q1rk1/pp2ppbp/3pbnp1/n7/4PP2/1NN1B3/PPP1B1PP/R2Q1RK1 w - - 1 11' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4 Na5

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'f4f5' => 'r2q1rk1/pp2ppbp/3pbnp1/n4P2/4P3/1NN1B3/PPP1B1PP/R2Q1RK1 b - - 0 11',
			},
		},
		'r2q1rk1/pp2ppbp/4b1p1/n2P4/4P3/3BBP2/P3N1PP/R2Q1RK1 b - - 0 14' => {
			eco => 'D89g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. d5

			variation => N__('Grünfeld: Classical Exchange, Main Line, Sokolsky Variation'),
			moves => {
				'g7a1' => 'r2q1rk1/pp2pp1p/4b1p1/n2P4/4P3/3BBP2/P3N1PP/b2Q1RK1 w - - 0 15',
			},
		},
		'r2q1rk1/pp2ppbp/4b1p1/n2P4/Q3P3/3BBP2/4N1PP/2R2RK1 b - - 0 16' => {
			eco => 'D89c',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6 14. Rc1'),
			moves => {
				'e6d7' => 'r2q1rk1/pp1bppbp/6p1/n2P4/Q3P3/3BBP2/4N1PP/2R2RK1 w - - 1 17',
			},
		},
		'r2q1rk1/pp2ppbp/4b1p1/n7/3PP3/3BBP2/P3N1PP/2RQ1RK1 b - - 4 14' => {
			eco => 'D89c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6 14. Rc1

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6 14. Rc1'),
			moves => {
				'e6a2' => 'r2q1rk1/pp2ppbp/6p1/n7/3PP3/3BBP2/b3N1PP/2RQ1RK1 w - - 0 15',
			},
		},
		'r2q1rk1/pp2ppbp/4b1p1/n7/3PP3/3BBP2/P3N1PP/R2Q1RK1 w - - 3 14' => {
			eco => 'D89b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3 Be6

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6'),
			moves => {
				'a1c1' => 'r2q1rk1/pp2ppbp/4b1p1/n7/3PP3/3BBP2/P3N1PP/2RQ1RK1 b - - 4 14',
				'd4d5' => 'r2q1rk1/pp2ppbp/4b1p1/n2P4/4P3/3BBP2/P3N1PP/R2Q1RK1 b - - 0 14',
			},
		},
		'r2q1rk1/pp2ppbp/4b1p1/n7/Q2PP3/3BBP2/4N1PP/2R2RK1 w - - 2 16' => {
			eco => 'D89c',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6 14. Rc1'),
			moves => {
				'd4d5' => 'r2q1rk1/pp2ppbp/4b1p1/n2P4/Q3P3/3BBP2/4N1PP/2R2RK1 b - - 0 16',
			},
		},
		'r2q1rk1/pp2ppbp/5np1/n2p1P2/4P3/1NNPB3/PP4PP/R2Q1RK1 w - - 0 14' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4 Na5 11. f5 Bc4 12. Bd3 Bxd3 13. cxd3 d5

			variation => N__('Sicilian: Dragon, Classical, Bernard Defence'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n1p5/2BPP1b1/2P1BP2/P3N1PP/R2Q1RK1 w - - 1 12' => {
			eco => 'D87n',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Bg4'),
			moves => {
				'c4d5' => 'r2q1rk1/pp2ppbp/6p1/n1pB4/3PP1b1/2P1BP2/P3N1PP/R2Q1RK1 b - - 2 12',
				'c4f7' => 'r2q1rk1/pp2pBbp/6p1/n1p5/3PP1b1/2P1BP2/P3N1PP/R2Q1RK1 b - - 0 12',
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n1pB4/3PP1b1/2P1BP2/P3N1PP/R2Q1RK1 b - - 2 12' => {
			eco => 'D87o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 Bg4 11. f3 Na5 12. Bd5

			variation => N__('Grünfeld: Classical Exchange, 10. Be3 Bg4 11. f3 Na5 12. Bd5'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n2B4/3PP1b1/4BP2/P3N1PP/R2Q1RK1 b - - 2 13' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd5

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4 Bg4 12. f3 Na5 13. Bd5'),
			moves => {
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n7/2BPP1b1/4BP2/P3N1PP/R2Q1RK1 w - - 1 13' => {
			eco => 'D88',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5

			variation => N__('Grünfeld: Classical Exchange, Main Line, 11.cxd4 Bg4 12. f3 Na5'),
			moves => {
				'c4d3' => 'r2q1rk1/pp2ppbp/6p1/n7/3PP1b1/3BBP2/P3N1PP/R2Q1RK1 b - - 2 13',
				'c4d5' => 'r2q1rk1/pp2ppbp/6p1/n2B4/3PP1b1/4BP2/P3N1PP/R2Q1RK1 b - - 2 13',
				'c4f7' => 'r2q1rk1/pp2pBbp/6p1/n7/3PP1b1/4BP2/P3N1PP/R2Q1RK1 b - - 0 13',
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n7/3PP1b1/3BBP2/P3N1PP/R2Q1RK1 b - - 2 13' => {
			eco => 'D89a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O Nc6 10. Be3 cxd4 11. cxd4 Bg4 12. f3 Na5 13. Bd3

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3'),
			moves => {
				'g4e6' => 'r2q1rk1/pp2ppbp/4b1p1/n7/3PP3/3BBP2/P3N1PP/R2Q1RK1 w - - 3 14',
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n7/3PP3/3BBP2/b3N1PP/2RQ1RK1 w - - 0 15' => {
			eco => 'D89c',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6 14. Rc1'),
			moves => {
				'd1a4' => 'r2q1rk1/pp2ppbp/6p1/n7/Q2PP3/3BBP2/b3N1PP/2R2RK1 b - - 1 15',
			},
		},
		'r2q1rk1/pp2ppbp/6p1/n7/Q2PP3/3BBP2/b3N1PP/2R2RK1 b - - 1 15' => {
			eco => 'D89c',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, Main Line, 13. Bd3 Be6 14. Rc1'),
			moves => {
				'a2e6' => 'r2q1rk1/pp2ppbp/4b1p1/n7/Q2PP3/3BBP2/4N1PP/2R2RK1 w - - 2 16',
			},
		},
		'r2q1rk1/pp2ppbp/n2p1np1/2pP4/4PPb1/2NB1N2/PPP3PP/R1BQ1RK1 w - - 1 9' => {
			eco => 'B09n',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3 Na6 7. O-O c5 8. d5 Bg4

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3 Na6 7.O-O c5 8. d5 Bg4'),
			moves => {
			},
		},
		'r2q1rk1/pp3pbp/n4np1/2pP1b2/2Q5/2N2N2/PP2BPPP/R1B2RK1 w - - 1 12' => {
			eco => 'D97p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Na6 8. Be2 c5 9. d5 e6 10. O-O exd5 11. exd5 Bf5

			variation => N__('Grünfeld: Russian, Prins, 8. Be2, Main Line'),
			moves => {
			},
		},
		'r2q1rk1/pp3ppp/2n1bB2/2bp4/8/2N2NP1/PP2PPBP/R2Q1RK1 b - - 0 11' => {
			eco => 'D34k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Bg5 Be6 11. Bxf6

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 Be6 11.Bxf6'),
			moves => {
			},
		},
		'r2q1rk1/pp3ppp/2n1bn2/2bp2B1/8/2N2NP1/PP2PPBP/R2Q1RK1 w - - 2 11' => {
			eco => 'D34k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O O-O 9. dxc5 Bxc5 10. Bg5 Be6

			variation => N__('QGD Tarrasch: 9.dxc5 Bxc5 10. Bg5 Be6'),
			moves => {
				'g5f6' => 'r2q1rk1/pp3ppp/2n1bB2/2bp4/8/2N2NP1/PP2PPBP/R2Q1RK1 b - - 0 11',
			},
		},
		'r2q1rk1/ppp1bppp/1nnPp3/8/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 b - - 0 10' => {
			eco => 'B05t',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, Main Line, 8. Nc3 O-O 9. Be3 Nc6'),
			moves => {
				'c7d6' => 'r2q1rk1/pp2bppp/1nnpp3/8/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 0 11',
			},
		},
		'r2q1rk1/ppp1bppp/1nnpp3/4P3/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 5 10' => {
			eco => 'B05t',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. Nc3 O-O 9. Be3 Nc6

			variation => N__('Alekhine: Modern, Main Line, 8. Nc3 O-O 9. Be3 Nc6'),
			moves => {
				'e5d6' => 'r2q1rk1/ppp1bppp/1nnPp3/8/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 b - - 0 10',
			},
		},
		'r2q1rk1/ppp1ppbp/1n4p1/3Pn3/4P1b1/1QN1BN2/PP3PPP/3RKB1R w K - 1 12' => {
			eco => 'D99g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3 Nb6 10. Rd1 Nc6 11. d5 Ne5

			variation => N__('Grünfeld: Russian, Smyslov, Main Line, 9... Nb6 10. Rd1 Nc6 11. d5 Ne5'),
			moves => {
			},
		},
		'r2q1rk1/ppp1ppbp/1nn3p1/3P4/4P1b1/1QN1BN2/PP3PPP/3RKB1R b K - 0 11' => {
			eco => 'D99f',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Smyslov, Main Line, 9... Nb6 10. Rd1 Nc6'),
			moves => {
				'c6e5' => 'r2q1rk1/ppp1ppbp/1n4p1/3Pn3/4P1b1/1QN1BN2/PP3PPP/3RKB1R w K - 1 12',
			},
		},
		'r2q1rk1/ppp1ppbp/1nn3p1/8/3PP1b1/1QN1BN2/PP3PPP/3RKB1R w K - 7 11' => {
			eco => 'D99f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3 Nb6 10. Rd1 Nc6

			variation => N__('Grünfeld: Russian, Smyslov, Main Line, 9... Nb6 10. Rd1 Nc6'),
			moves => {
				'd4d5' => 'r2q1rk1/ppp1ppbp/1nn3p1/3P4/4P1b1/1QN1BN2/PP3PPP/3RKB1R b K - 0 11',
			},
		},
		'r2q1rk1/ppp1ppbp/1nn3p1/8/3PP1b1/2NQBN2/PP2BPPP/2KR3R b - - 8 11' => {
			eco => 'D98h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Be2 Nb6 10. Qd3 Nc6 11. O-O-O

			variation => N__('Grünfeld: Russian, Smyslov, Keres Variation'),
			moves => {
			},
		},
		'r2q1rk1/ppp1ppbp/1nn3p1/8/3PP1b1/2NQBN2/PP2BPPP/R3K2R w KQ - 7 11' => {
			eco => 'D98g',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9. Be2'),
			moves => {
				'e1c1' => 'r2q1rk1/ppp1ppbp/1nn3p1/8/3PP1b1/2NQBN2/PP2BPPP/2KR3R b - - 8 11',
			},
		},
		'r2q1rk1/ppp1ppbp/2n1bnp1/8/2NP4/1P3NP1/P3PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D77k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 Nc6 8. Nxc4 Be6 9. b3

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 Nc6 8.Nxc4 Be6 9. b3'),
			moves => {
				'e6d5' => 'r2q1rk1/ppp1ppbp/2n2np1/3b4/2NP4/1P3NP1/P3PPBP/R1BQ1RK1 w - - 1 10',
			},
		},
		'r2q1rk1/ppp1ppbp/2n1bnp1/8/2NP4/5NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'D77j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 Nc6 8. Nxc4 Be6

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 Nc6 8.Nxc4 Be6'),
			moves => {
				'b2b3' => 'r2q1rk1/ppp1ppbp/2n1bnp1/8/2NP4/1P3NP1/P3PPBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'r2q1rk1/ppp1ppbp/2n2np1/3P4/2Q1P1b1/2N2N2/PP2BPPP/R1B1K2R b KQ - 0 9' => {
			eco => 'D98d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be2 Nc6 9. d5

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be2 Nc6 9. d5'),
			moves => {
			},
		},
		'r2q1rk1/ppp1ppbp/2n2np1/3b4/2NP4/1P3NP1/P3PPBP/R1BQ1RK1 w - - 1 10' => {
			eco => 'D77l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 Nc6 8. Nxc4 Be6 9. b3 Bd5

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 Nc6 8.Nxc4 Be6 9. b3 Bd5'),
			moves => {
			},
		},
		'r2q1rk1/ppp1ppbp/2n2np1/8/2QPP1b1/2N2N2/PP2BPPP/R1B1K2R w KQ - 3 9' => {
			eco => 'D98c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be2 Nc6

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be2 Nc6'),
			moves => {
				'd4d5' => 'r2q1rk1/ppp1ppbp/2n2np1/3P4/2Q1P1b1/2N2N2/PP2BPPP/R1B1K2R b KQ - 0 9',
			},
		},
		'r2q1rk1/ppp1ppbp/2np1np1/3P1b2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E62q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O Bf5 8. d5

			variation => N__('King\'s Indian: Fianchetto, Spassky, 8. d5'),
			moves => {
			},
		},
		'r2q1rk1/ppp1ppbp/2np1np1/3P4/2P3b1/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E62s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O Bg4 8. d5

			variation => N__('King\'s Indian: Fianchetto, Simagin, 8. d5'),
			moves => {
			},
		},
		'r2q1rk1/ppp1ppbp/2np1np1/5b2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 8' => {
			eco => 'E62p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O Bf5

			variation => N__('King\'s Indian: Fianchetto, Spassky Variation'),
			moves => {
				'd4d5' => 'r2q1rk1/ppp1ppbp/2np1np1/3P1b2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r2q1rk1/ppp1ppbp/2np1np1/8/2PP2b1/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 8' => {
			eco => 'E62r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 Nc6 7. O-O Bg4

			variation => N__('King\'s Indian: Fianchetto, Simagin Variation'),
			moves => {
				'd4d5' => 'r2q1rk1/ppp1ppbp/2np1np1/3P4/2P3b1/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'r2q1rk1/ppp1ppbp/2np1np1/8/3PP1b1/2N1BN2/PPP1BPPP/R2Q1RK1 w - - 8 8' => {
			eco => 'B08t',
                        # TRANSLATORS:

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O Bg4'),
			moves => {
				'd1d2' => 'r2q1rk1/ppp1ppbp/2np1np1/8/3PP1b1/2N1BN2/PPPQBPPP/R4RK1 b - - 9 8',
			},
		},
		'r2q1rk1/ppp1ppbp/2np1np1/8/3PP1b1/2N1BN2/PPPQBPPP/R4RK1 b - - 9 8' => {
			eco => 'B08u',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O Bg4 7. Be3 Nc6 8. Qd2

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O Bg4 7. Be3 Nc6 8. Qd2'),
			moves => {
				'e7e5' => 'r2q1rk1/ppp2pbp/2np1np1/4p3/3PP1b1/2N1BN2/PPPQBPPP/R4RK1 w - e6 0 9',
			},
		},
		'r2q1rk1/ppp2pbp/2np1np1/4p3/3PP1b1/2N1BN2/PPPQBPPP/R4RK1 w - e6 0 9' => {
			eco => 'B08v',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O Bg4 7. Be3 Nc6 8. Qd2 e5

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O Bg4 7. Be3 Nc6 8. Qd2 e5'),
			moves => {
			},
		},
		'r2q1rk1/ppp2ppp/2npbn2/1B2p1B1/1b2P3/2NP1N2/PPP2PPP/R2Q1RK1 w - - 2 8' => {
			eco => 'C49j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 Bb4 5. O-O O-O 6. d3 d6 7. Bg5 Be6

			variation => N__('Four Knights: Symmetrical, Tarrasch Variation'),
			moves => {
			},
		},
		'r2q1rk1/pppbbppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQR1K1 w - - 5 8' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Bd7 6. Nc3 Be7 7. Re1 O-O

			variation => N__('Spanish: Closed Berlin, Tarrasch Trap'),
			moves => {
			},
		},
		'r2q1rk1/pppnppbp/2n3p1/8/2QPP1b1/2N1BN2/PP3PPP/3RKB1R w K - 5 10' => {
			eco => 'D98k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Rd1 Nc6

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9. Rd1 Nc6'),
			moves => {
			},
		},
		'r2qk1nr/1p2bppp/p1npb3/4p3/2P1P3/2N5/PPN2PPP/R1BQKB1R b KQkq - 3 9' => {
			eco => 'B32x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4 Be7 7. N1c3 a6 8. Na3 Be6 9. Nc2

			variation => N__('Sicilian: Lowenthal, Kalashnikov, Main Line, 9. Nc2'),
			moves => {
			},
		},
		'r2qk1nr/1p2bppp/p1npb3/4p3/2P1P3/N1N5/PP2BPPP/R1BQK2R b KQkq - 3 9' => {
			eco => 'B32y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4 Be7 7. N1c3 a6 8. Na3 Be6 9. Be2

			variation => N__('Sicilian: Lowenthal, Kalashnikov, Main Line, 9. Be2'),
			moves => {
			},
		},
		'r2qk1nr/1p2bppp/p1npb3/4p3/2P1P3/N1N5/PP3PPP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'B32w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 e5 5. Nb5 d6 6. c4 Be7 7. N1c3 a6 8. Na3 Be6

			variation => N__('Sicilian: Lowenthal, Kalashnikov, Main Line'),
			moves => {
				'a3c2' => 'r2qk1nr/1p2bppp/p1npb3/4p3/2P1P3/2N5/PPN2PPP/R1BQKB1R b KQkq - 3 9',
				'f1e2' => 'r2qk1nr/1p2bppp/p1npb3/4p3/2P1P3/N1N5/PP2BPPP/R1BQK2R b KQkq - 3 9',
			},
		},
		'r2qk1nr/1pp1b1pp/p1np4/4p3/8/2P2N2/PPQP1PPP/RNB1R1K1 b kq - 0 10' => {
			eco => 'C74',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 f5 6. exf5 Bxf5 7. O-O Bd3 8. Re1 Be7 9. Bc2 Bxc2 10. Qxc2

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev, Main Line'),
			moves => {
			},
		},
		'r2qk1nr/1pp1b1pp/p1np4/4p3/8/2P2N2/PPbP1PPP/RNBQR1K1 w kq - 0 10' => {
			eco => 'C74',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev Variation'),
			moves => {
				'd1c2' => 'r2qk1nr/1pp1b1pp/p1np4/4p3/8/2P2N2/PPQP1PPP/RNB1R1K1 b kq - 0 10',
			},
		},
		'r2qk1nr/1pp1b1pp/p1np4/4p3/8/2Pb1N2/PPBP1PPP/RNBQR1K1 b kq - 5 9' => {
			eco => 'C74',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev Variation'),
			moves => {
				'd3c2' => 'r2qk1nr/1pp1b1pp/p1np4/4p3/8/2P2N2/PPbP1PPP/RNBQR1K1 w kq - 0 10',
			},
		},
		'r2qk1nr/1pp1b1pp/p1np4/4p3/B7/2Pb1N2/PP1P1PPP/RNBQR1K1 w kq - 4 9' => {
			eco => 'C74',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev Variation'),
			moves => {
				'a4c2' => 'r2qk1nr/1pp1b1pp/p1np4/4p3/8/2Pb1N2/PPBP1PPP/RNBQR1K1 b kq - 5 9',
			},
		},
		'r2qk1nr/1pp3pp/p1pb1p2/4p3/3PP1b1/2P2N2/PP3PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C69d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 Bg4 7. c3 Bd6

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 7. c3 Bd6'),
			moves => {
			},
		},
		'r2qk1nr/1ppb1pbp/p1n3p1/4p3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6 7. O-O Bg7 8. dxe5 dxe5

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O Bg7 8.dxe5 dxe5'),
			moves => {
			},
		},
		'r2qk1nr/1ppb1pbp/p1np2p1/3Pp3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6 7. O-O Bg7 8. d5

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O Bg7 8. d5'),
			moves => {
			},
		},
		'r2qk1nr/1ppb1pbp/p1np2p1/4P3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6 7. O-O Bg7 8. dxe5

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O Bg7 8.dxe5'),
			moves => {
				'c6e5' => 'r2qk1nr/1ppb1pbp/p2p2p1/4n3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 0 9',
				'd6e5' => 'r2qk1nr/1ppb1pbp/p1n3p1/4p3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 0 9',
			},
		},
		'r2qk1nr/1ppb1pbp/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'C76',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O'),
			moves => {
				'd4d5' => 'r2qk1nr/1ppb1pbp/p1np2p1/3Pp3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 8',
				'd4e5' => 'r2qk1nr/1ppb1pbp/p1np2p1/4P3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 8',
				'f1e1' => 'r2qk1nr/1ppb1pbp/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQR1K1 b kq - 3 8',
			},
		},
		'r2qk1nr/1ppb1pbp/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQR1K1 b kq - 3 8' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6 7. O-O Bg7 8. Re1

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O Bg7 8. Re1'),
			moves => {
			},
		},
		'r2qk1nr/1ppb1pbp/p2p2p1/4n3/B3P3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6 7. O-O Bg7 8. dxe5 Nxe5

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O Bg7 8.dxe5 Nxe5'),
			moves => {
			},
		},
		'r2qk1nr/pp2bppp/2p1b3/2np2B1/3N3Q/2NB4/PPP2PPP/2KRR3 w kq - 8 13' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 c6 8. O-O-O d5 9. Rhe1 Be6 10. Bd3 Nbd7 11. Qh4 Nc5 12. Nd4 Ng8

			variation => N__('Bishop\'s Opening: Urusov Gambit, Forintos/Haag Variation'),
			moves => {
			},
		},
		'r2qk1nr/ppp1ppbp/2np2p1/8/2PP2b1/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'A41w',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. Nf3 Bg4 5. e3 Nc6

			variation => N__('Neo-Old Indian: Modern, 3. Nc3 Bg7 4. Nf3 Bf4 5. e3 Nc6'),
			moves => {
			},
		},
		'r2qk1nr/ppp2pbp/2npb1p1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 d6 6. Nge2 Be6

			variation => N__('English: Closed, Hort Variation'),
			moves => {
				'd2d3' => 'r2qk1nr/ppp2pbp/2npb1p1/4p3/2P5/2NPP1P1/PP2NPBP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r2qk1nr/ppp2pbp/2npb1p1/4p3/2P5/2NPP1P1/PP2NPBP/R1BQK2R b KQkq - 0 7' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 g6 4. Bg2 Bg7 5. e3 d6 6. Nge2 Be6 7. d3

			variation => N__('English: Closed, Hort, 7. d3'),
			moves => {
			},
		},
		'r2qk1nr/ppp2ppp/1bnp4/8/2BPP1b1/2N2N2/P4PPP/R1BQ1RK1 w kq - 3 10' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. Nc3 Bg4

			variation => N__('Evans Gambit: Normal Variation'),
			moves => {
				'd1a4' => 'r2qk1nr/ppp2ppp/1bnp4/8/Q1BPP1b1/2N2N2/P4PPP/R1B2RK1 b kq - 4 10',
			},
		},
		'r2qk1nr/ppp2ppp/1bnp4/8/Q1BPP1b1/2N2N2/P4PPP/R1B2RK1 b kq - 4 10' => {
			eco => 'C51',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Bc5 6. d4 exd4 7. O-O d6 8. cxd4 Bb6 9. Nc3 Bg4 10. Qa4

			variation => N__('Evans Gambit: Fraser Attack'),
			moves => {
				'g4d7' => 'r2qk1nr/pppb1ppp/1bnp4/8/Q1BPP3/2N2N2/P4PPP/R1B2RK1 w kq - 5 11',
			},
		},
		'r2qk1nr/ppp2ppp/2n5/4P3/1bPp4/5N1P/PP1BPPP1/R2QKB1R b KQkq - 2 8' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, Alapin, 5... Bg4'),
			moves => {
				'd8e7' => 'r3k1nr/ppp1qppp/2n5/4P3/1bPp4/5N1P/PP1BPPP1/R2QKB1R w KQkq - 3 9',
			},
		},
		'r2qk1nr/ppp2ppp/2n5/4P3/1bPp4/5N1P/PP2PPP1/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, Alapin, 5... Bg4'),
			moves => {
				'c1d2' => 'r2qk1nr/ppp2ppp/2n5/4P3/1bPp4/5N1P/PP1BPPP1/R2QKB1R b KQkq - 2 8',
			},
		},
		'r2qk1nr/ppp2ppp/2np4/2b5/2BpP1b1/2P2N2/PP3PPP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'C44y',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Bc4 Bc5 5. O-O d6 6. c3 Bg4

			variation => N__('Scotch Gambit: Anderssen (Paulsen) Counterattack'),
			moves => {
			},
		},
		'r2qk1nr/ppp2ppp/2np4/b3p3/2BPP1b1/2P2N2/P4PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O d6 7. d4 Bg4

			variation => N__('Evans Gambit: Alapin-Steinitz Variation'),
			moves => {
			},
		},
		'r2qk1nr/pppb1Bpp/1b1p4/n7/3PP3/1QN2N2/P4PPP/R1B2RK1 b kq - 0 12' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Fraser Attack'),
			moves => {
				'e8f8' => 'r2q1knr/pppb1Bpp/1b1p4/n7/3PP3/1QN2N2/P4PPP/R1B2RK1 w - - 1 13',
			},
		},
		'r2qk1nr/pppb1ppp/1b1p4/n7/2BPP3/1QN2N2/P4PPP/R1B2RK1 w kq - 7 12' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Fraser Attack'),
			moves => {
				'c4f7' => 'r2qk1nr/pppb1Bpp/1b1p4/n7/3PP3/1QN2N2/P4PPP/R1B2RK1 b kq - 0 12',
			},
		},
		'r2qk1nr/pppb1ppp/1bnp4/8/2BPP3/1QN2N2/P4PPP/R1B2RK1 b kq - 6 11' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Fraser Attack'),
			moves => {
				'c6a5' => 'r2qk1nr/pppb1ppp/1b1p4/n7/2BPP3/1QN2N2/P4PPP/R1B2RK1 w kq - 7 12',
			},
		},
		'r2qk1nr/pppb1ppp/1bnp4/8/Q1BPP3/2N2N2/P4PPP/R1B2RK1 w kq - 5 11' => {
			eco => 'C51',
                        # TRANSLATORS:

			variation => N__('Evans Gambit: Fraser Attack'),
			moves => {
				'a4b3' => 'r2qk1nr/pppb1ppp/1bnp4/8/2BPP3/1QN2N2/P4PPP/R1B2RK1 b kq - 6 11',
			},
		},
		'r2qk1nr/pppb1ppp/2np4/b3p3/2BPP3/2P2N2/P4PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C52',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. b4 Bxb4 5. c3 Ba5 6. O-O d6 7. d4 Bd7

			variation => N__('Evans Gambit: Sanders-Alapin Variation'),
			moves => {
			},
		},
		'r2qk2r/1b1n1ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2Q1RK1 b kq - 5 14' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'e8g8' => 'r2q1rk1/1b1n1ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2Q1RK1 w - - 6 15',
			},
		},
		'r2qk2r/1b1n1ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2QK2R w KQkq - 4 14' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'e1g1' => 'r2qk2r/1b1n1ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2Q1RK1 b kq - 5 14',
			},
		},
		'r2qk2r/1b1n1ppp/p2bpn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11' => {
			eco => 'D29k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. Nc3 Bd6

			variation => N__('QGA: Classical, Smyslov Variation'),
			moves => {
			},
		},
		'r2qk2r/1b1nbppp/pp1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1BR2K1 w kq - 1 11' => {
			eco => 'A30x',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. Rd1 a6 10. b3 Nbd7

			variation => N__('English: Symmetrical, Hedgehog, Flexible Formation'),
			moves => {
				'e2e4' => 'r2qk2r/1b1nbppp/pp1ppn2/8/2PQP3/1PN2NP1/P4PBP/R1BR2K1 b kq e3 0 11',
			},
		},
		'r2qk2r/1b1nbppp/pp1ppn2/8/2PQP3/1PN2NP1/P4PBP/R1BR2K1 b kq e3 0 11' => {
			eco => 'A30y',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. Rd1 a6 10. b3 Nbd7 11. e4

			variation => N__('English: Symmetrical, Hedgehog, Flexible Formation'),
			moves => {
				'd8c7' => 'r3k2r/1bqnbppp/pp1ppn2/8/2PQP3/1PN2NP1/P4PBP/R1BR2K1 w kq - 1 12',
				'e8g8' => 'r2q1rk1/1b1nbppp/pp1ppn2/8/2PQP3/1PN2NP1/P4PBP/R1BR2K1 w - - 1 12',
			},
		},
		'r2qk2r/1bp2ppp/p1np1n2/1pb1p3/3PP3/1BP2N2/PP3PPP/RNBQR1K1 b kq d3 0 9' => {
			eco => 'C78u',
                        # TRANSLATORS:

			variation => N__('Spanish: Archangelsk, 7. Re1 Bc5 8. c3 d6'),
			moves => {
				'c5b6' => 'r2qk2r/1bp2ppp/pbnp1n2/1p2p3/3PP3/1BP2N2/PP3PPP/RNBQR1K1 w kq - 1 10',
			},
		},
		'r2qk2r/1bp2ppp/p1np1n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w kq - 0 9' => {
			eco => 'C78u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1 Bc5 8. c3 d6

			variation => N__('Spanish: Archangelsk, 7. Re1 Bc5 8. c3 d6'),
			moves => {
				'd2d4' => 'r2qk2r/1bp2ppp/p1np1n2/1pb1p3/3PP3/1BP2N2/PP3PPP/RNBQR1K1 b kq d3 0 9',
			},
		},
		'r2qk2r/1bp2ppp/pbnp1n2/1p2p1B1/3PP3/1BP2N2/PP3PPP/RN1QR1K1 b kq - 2 10' => {
			eco => 'C78w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1 Bc5 8. c3 d6 9. d4 Bb6 10. Bg5

			variation => N__('Spanish: Archangelsk, Main Line, 10. Bg5'),
			moves => {
			},
		},
		'r2qk2r/1bp2ppp/pbnp1n2/1p2p3/3PP3/1BP1BN2/PP3PPP/RN1QR1K1 b kq - 2 10' => {
			eco => 'C78x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1 Bc5 8. c3 d6 9. d4 Bb6 10. Be3

			variation => N__('Spanish: Archangelsk, Main Line, 10. Be3'),
			moves => {
			},
		},
		'r2qk2r/1bp2ppp/pbnp1n2/1p2p3/3PP3/1BP2N2/PP3PPP/RNBQR1K1 w kq - 1 10' => {
			eco => 'C78u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1 Bc5 8. c3 d6 9. d4 Bb6

			variation => N__('Spanish: Archangelsk, Main Line'),
			moves => {
				'a2a4' => 'r2qk2r/1bp2ppp/pbnp1n2/1p2p3/P2PP3/1BP2N2/1P3PPP/RNBQR1K1 b kq a3 0 10',
				'c1e3' => 'r2qk2r/1bp2ppp/pbnp1n2/1p2p3/3PP3/1BP1BN2/PP3PPP/RN1QR1K1 b kq - 2 10',
				'c1g5' => 'r2qk2r/1bp2ppp/pbnp1n2/1p2p1B1/3PP3/1BP2N2/PP3PPP/RN1QR1K1 b kq - 2 10',
			},
		},
		'r2qk2r/1bp2ppp/pbnp1n2/1p2p3/P2PP3/1BP2N2/1P3PPP/RNBQR1K1 b kq a3 0 10' => {
			eco => 'C78v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1 Bc5 8. c3 d6 9. d4 Bb6 10. a4

			variation => N__('Spanish: Archangelsk, Main Line, 10. a4'),
			moves => {
			},
		},
		'r2qk2r/1bpp1ppp/p1n2n2/1pb1p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w kq - 4 8' => {
			eco => 'C78t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1 Bc5

			variation => N__('Spanish: Archangelsk, 7. Re1 Bc5'),
			moves => {
				'c2c3' => 'r2qk2r/1bpp1ppp/p1n2n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 b kq - 0 8',
			},
		},
		'r2qk2r/1bpp1ppp/p1n2n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 b kq - 0 8' => {
			eco => 'C78t',
                        # TRANSLATORS:

			variation => N__('Spanish: Archangelsk, 7. Re1 Bc5'),
			moves => {
				'd7d6' => 'r2qk2r/1bp2ppp/p1np1n2/1pb1p3/4P3/1BP2N2/PP1P1PPP/RNBQR1K1 w kq - 0 9',
			},
		},
		'r2qk2r/1bppbppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C78q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. d3 Be7

			variation => N__('Spanish: Archangelsk, 7. d3 Be7'),
			moves => {
			},
		},
		'r2qk2r/1bppbppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w kq - 2 8' => {
			eco => 'C88b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 Bb7

			variation => N__('Spanish: Closed, Trajkovic Counterattack'),
			moves => {
			},
		},
		'r2qk2r/1p1bbppp/p1nppn2/6B1/3NP2P/2N2P2/PPPQ2P1/2KR1B1R b kq h3 0 10' => {
			eco => 'B67f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f3 Be7 10. h4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f3 Be7 10. h4'),
			moves => {
			},
		},
		'r2qk2r/1p1bbppp/p1nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R w kq - 1 10' => {
			eco => 'B67e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f3 Be7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f3 Be7'),
			moves => {
				'h2h4' => 'r2qk2r/1p1bbppp/p1nppn2/6B1/3NP2P/2N2P2/PPPQ2P1/2KR1B1R b kq h3 0 10',
			},
		},
		'r2qk2r/1p1bbppp/p1nppn2/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq - 1 10' => {
			eco => 'B68',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7'),
			moves => {
				'd4f3' => 'r2qk2r/1p1bbppp/p1nppn2/6B1/4PP2/2N2N2/PPPQ2PP/2KR1B1R b kq - 2 10',
				'f1e2' => 'r2qk2r/1p1bbppp/p1nppn2/6B1/3NPP2/2N5/PPPQB1PP/2KR3R b kq - 2 10',
			},
		},
		'r2qk2r/1p1bbppp/p1nppn2/6B1/3NPP2/2N5/PPPQB1PP/2KR3R b kq - 2 10' => {
			eco => 'B68',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Be2

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Be2'),
			moves => {
				'd8c7' => 'r3k2r/1pqbbppp/p1nppn2/6B1/3NPP2/2N5/PPPQB1PP/2KR3R w kq - 3 11',
			},
		},
		'r2qk2r/1p1bbppp/p1nppn2/6B1/4PP2/2N2N2/PPPQ2PP/2KR1B1R b kq - 2 10' => {
			eco => 'B68',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3'),
			moves => {
				'b7b5' => 'r2qk2r/3bbppp/p1nppn2/1p4B1/4PP2/2N2N2/PPPQ2PP/2KR1B1R w kq b6 0 11',
			},
		},
		'r2qk2r/1p1n1ppp/2p1pn2/p4b2/PbBP4/1QN1PN2/1P3PPP/R1B2RK1 w kq a6 0 10' => {
			eco => 'D18j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Qb3 a5

			variation => N__('Slav: Dutch, 8...Nbd7 9. Qb3 a5'),
			moves => {
			},
		},
		'r2qk2r/1p1nbppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R w KQkq - 3 10' => {
			eco => 'B90p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 Nbd7

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 Nbd7'),
			moves => {
			},
		},
		'r2qk2r/1pp1bppp/p1np1n2/4p3/B3P1b1/2P2N1P/PP1P1PP1/RNBQR1K1 b kq - 0 8' => {
			eco => 'C87g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 Bg4 8. h3

			variation => N__('Spanish: Closed, Averbakh, 7. c3 Bg4 8. h3'),
			moves => {
			},
		},
		'r2qk2r/1pp1bppp/p1np1n2/4p3/B3P1b1/2P2N2/PP1P1PPP/RNBQR1K1 w kq - 1 8' => {
			eco => 'C87f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 Bg4

			variation => N__('Spanish: Closed, Averbakh, 7. c3 Bg4'),
			moves => {
				'd2d3' => 'r2qk2r/1pp1bppp/p1np1n2/4p3/B3P1b1/2PP1N2/PP3PPP/RNBQR1K1 b kq - 0 8',
				'h2h3' => 'r2qk2r/1pp1bppp/p1np1n2/4p3/B3P1b1/2P2N1P/PP1P1PP1/RNBQR1K1 b kq - 0 8',
			},
		},
		'r2qk2r/1pp1bppp/p1np1n2/4p3/B3P1b1/2PP1N2/PP3PPP/RNBQR1K1 b kq - 0 8' => {
			eco => 'C87h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 Bg4 8. d3

			variation => N__('Spanish: Closed, Averbakh, 7. c3 Bg4 8. d3'),
			moves => {
			},
		},
		'r2qk2r/1pp1bppp/p1p2n2/4p3/4P1b1/2N2N2/PPPP1PPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'C85e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. Nc3 Bg4

			variation => N__('Spanish: Closed, Exchange, 7. Nc3 Bg4'),
			moves => {
			},
		},
		'r2qk2r/1pp1bppp/p1p2n2/4p3/4P1b1/3P1N2/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C85k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Bxc6 dxc6 7. d3 Bg4

			variation => N__('Spanish: Closed, Exchange, 7. d3 Bg4'),
			moves => {
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/2P1n3/1B3N2/PP2QPPP/RNB2RK1 b kq c3 0 10' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. c4

			variation => N__('Spanish: Open, Keres, Adam Variation'),
			moves => {
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNB2RK1 w kq - 3 10' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7

			variation => N__('Spanish: Open, Keres Attack, 9... Be7'),
			moves => {
				'c2c4' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/2P1n3/1B3N2/PP2QPPP/RNB2RK1 b kq c3 0 10',
				'f1d1' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNBR2K1 b kq - 4 10',
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNBR2K1 b kq - 4 10' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. Rd1

			variation => N__('Spanish: Open, Keres, 10. Rd1'),
			moves => {
				'e8g8' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNBR2K1 w - - 5 11',
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP1BN2/PP3PPP/RN1Q1RK1 b kq - 2 10' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Be3

			variation => N__('Spanish: Open, Classical, 10. Be3'),
			moves => {
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 b kq - 2 10' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2

			variation => N__('Spanish: Open, Classical, 10.Nbd2'),
			moves => {
				'e4c5' => 'r2qk2r/2p1bppp/p1n1b3/1pnpP3/8/1BP2N2/PP1N1PPP/R1BQ1RK1 w kq - 3 11',
				'e8g8' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 w - - 3 11',
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQ1RK1 w kq - 1 10' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7

			variation => N__('Spanish: Open, Classical Defence'),
			moves => {
				'b1d2' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 b kq - 2 10',
				'c1e3' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP1BN2/PP3PPP/RN1Q1RK1 b kq - 2 10',
				'f1e1' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQR1K1 b kq - 2 10',
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQR1K1 b kq - 2 10' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Re1

			variation => N__('Spanish: Open, Classical, 10. Re1'),
			moves => {
				'e8g8' => 'r2q1rk1/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQR1K1 w - - 3 11',
			},
		},
		'r2qk2r/2p1bppp/p1n1b3/1pnpP3/8/1BP2N2/PP1N1PPP/R1BQ1RK1 w kq - 3 11' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Nbd2 Nc5

			variation => N__('Spanish: Open, Classical, 10.Nbd2 Nc5'),
			moves => {
			},
		},
		'r2qk2r/2p1bppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQ1RK1 w kq - 5 12' => {
			eco => 'C82e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4 11. Nbd2 Be7

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4 11.Nbd2 Be7'),
			moves => {
				'f1e1' => 'r2qk2r/2p1bppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQR1K1 b kq - 6 12',
			},
		},
		'r2qk2r/2p1bppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQR1K1 b kq - 6 12' => {
			eco => 'C82f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4 11. Nbd2 Be7 12. Re1

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4 11.Nbd2 Be7 12. Re1'),
			moves => {
				'd8d7' => 'r3k2r/2pqbppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQR1K1 w kq - 7 13',
				'e8g8' => 'r2q1rk1/2p1bppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQR1K1 w - - 7 13',
			},
		},
		'r2qk2r/2p1bppp/p1np1n2/1p2p3/3PP1b1/1BP2N2/PP2QPPP/RNB1K2R w KQkq - 1 9' => {
			eco => 'C77u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. Qe2 b5 6. Bb3 Be7 7. d4 d6 8. c3 Bg4

			variation => N__('Spanish: Wormald Attack, Grünfeld Variation'),
			moves => {
			},
		},
		'r2qk2r/2p1bppp/p1np1n2/1p2p3/4P1b1/1BP2N2/PP1P1PPP/RNBQR1K1 w kq - 1 9' => {
			eco => 'C88d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. c3 Bg4

			variation => N__('Spanish: Closed 7... d6 8. c3 Bg4'),
			moves => {
			},
		},
		'r2qk2r/2p1nppp/p3b3/1pbpP3/4n3/1BPQ1N2/PP3PPP/RNB2RK1 w kq - 3 11' => {
			eco => 'C82j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Qd3 Ne7

			variation => N__('Spanish: Open, Motzko Attack, Nenarokov Variation'),
			moves => {
			},
		},
		'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1B3N2/PPP1QPPP/RNB2RK1 w kq - 3 10' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Bc5

			variation => N__('Spanish: Open, Keres Attack, 9... Bc5'),
			moves => {
			},
		},
		'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1B3N2/PPPN1PPP/R1BQ1RK1 w kq - 3 10' => {
			eco => 'C80q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2 Bc5

			variation => N__('Spanish: Open, Bernstein, 9... Bc5'),
			moves => {
			},
		},
		'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 b kq - 2 10' => {
			eco => 'C82k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2

			variation => N__('Spanish: Open, St. Petersburg Variation'),
			moves => {
				'e8g8' => 'r2q1rk1/2p2ppp/p1n1b3/1pbpP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 w - - 3 11',
			},
		},
		'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1BP2N2/PP3PPP/RNBQ1RK1 w kq - 1 10' => {
			eco => 'C82i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5

			variation => N__('Spanish: Open, Italian Variation'),
			moves => {
				'b1d2' => 'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1BP2N2/PP1N1PPP/R1BQ1RK1 b kq - 2 10',
				'd1d3' => 'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1BPQ1N2/PP3PPP/RNB2RK1 b kq - 2 10',
			},
		},
		'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1BPQ1N2/PP3PPP/RNB2RK1 b kq - 2 10' => {
			eco => 'C82j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Qd3

			variation => N__('Spanish: Open, Motzko Attack'),
			moves => {
				'c6e7' => 'r2qk2r/2p1nppp/p3b3/1pbpP3/4n3/1BPQ1N2/PP3PPP/RNB2RK1 w kq - 3 11',
			},
		},
		'r2qk2r/3bbp1p/p1nppp2/1p3P2/4P3/2N2N2/PPPQ2PP/2KR1B1R b kq - 0 12' => {
			eco => 'B69',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3 b5 11. Bxf6 gxf6 12. f5

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3 b5 11.Bxf6 gxf6 12. f5'),
			moves => {
			},
		},
		'r2qk2r/3bbp1p/p1nppp2/1p6/4PP2/2N2N2/PPPQ2PP/1K1R1B1R b kq - 1 12' => {
			eco => 'B69',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3 b5 11. Bxf6 gxf6 12. Kb1

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3 b5 11.Bxf6 gxf6 12. Kb1'),
			moves => {
			},
		},
		'r2qk2r/3bbp1p/p1nppp2/1p6/4PP2/2N2N2/PPPQ2PP/2KR1B1R w kq - 0 12' => {
			eco => 'B69',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3 b5 11. Bxf6 gxf6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3 b5 11.Bxf6 gxf6'),
			moves => {
				'c1b1' => 'r2qk2r/3bbp1p/p1nppp2/1p6/4PP2/2N2N2/PPPQ2PP/1K1R1B1R b kq - 1 12',
				'f4f5' => 'r2qk2r/3bbp1p/p1nppp2/1p3P2/4P3/2N2N2/PPPQ2PP/2KR1B1R b kq - 0 12',
			},
		},
		'r2qk2r/3bbppp/p1nppB2/1p6/4PP2/2N2N2/PPPQ2PP/2KR1B1R b kq - 0 11' => {
			eco => 'B69',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3 b5 11. Bxf6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3 b5 11.Bxf6'),
			moves => {
				'g7f6' => 'r2qk2r/3bbp1p/p1nppp2/1p6/4PP2/2N2N2/PPPQ2PP/2KR1B1R w kq - 0 12',
			},
		},
		'r2qk2r/3bbppp/p1nppn2/1p2P1B1/5P2/2N2N2/PPPQ2PP/2KR1B1R b kq - 0 11' => {
			eco => 'B68',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3 b5 11. e5

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3 b5 11. e5'),
			moves => {
			},
		},
		'r2qk2r/3bbppp/p1nppn2/1p4B1/4PP2/2N2N2/PPPQ2PP/2KR1B1R w kq b6 0 11' => {
			eco => 'B68',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Nf3 b5

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Nf3 b5'),
			moves => {
				'e4e5' => 'r2qk2r/3bbppp/p1nppn2/1p2P1B1/5P2/2N2N2/PPPQ2PP/2KR1B1R b kq - 0 11',
				'g5f6' => 'r2qk2r/3bbppp/p1nppB2/1p6/4PP2/2N2N2/PPPQ2PP/2KR1B1R b kq - 0 11',
			},
		},
		'r2qk2r/3nppbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQ1K1R w kq - 1 11' => {
			eco => 'A59f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. Nf3 Bg7 10. h3 Nbd7

			variation => N__('Benko Gambit: 7. e4 Line, 9. Nf3 Bg7 10. h3 Nbd7'),
			moves => {
			},
		},
		'r2qk2r/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 4 10' => {
			eco => 'A58m',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Fianchetto, 9...Nbd7'),
			moves => {
				'd7b6' => 'r2qk2r/4ppbp/bn1p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 5 11',
			},
		},
		'r2qk2r/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 10' => {
			eco => 'A58m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 Nbd7

			variation => N__('Benko Gambit: Fianchetto, 9...Nbd7'),
			moves => {
				'e1g1' => 'r2qk2r/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 4 10',
			},
		},
		'r2qk2r/4ppbp/bn1p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 5 11' => {
			eco => 'A58n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 Nbd7 10. O-O Nb6

			variation => N__('Benko Gambit: Fianchetto, 9...Nbd7 10.O-O Nb6'),
			moves => {
			},
		},
		'r2qk2r/5pbp/p1np4/1p1Npb2/8/2P5/PPN2PPP/R2QKB1R b KQkq - 1 13' => {
			eco => 'B33v',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. c3 Bg7 12.exf5 Bxf5'),
			moves => {
				'e8g8' => 'r2q1rk1/5pbp/p1np4/1p1Npb2/8/2P5/PPN2PPP/R2QKB1R w KQ - 2 14',
			},
		},
		'r2qk2r/5pbp/p1np4/1p1Npb2/8/N1P5/PP3PPP/R2QKB1R w KQkq - 0 13' => {
			eco => 'B33v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. c3 Bg7 12. exf5 Bxf5

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. c3 Bg7 12.exf5 Bxf5'),
			moves => {
				'a3c2' => 'r2qk2r/5pbp/p1np4/1p1Npb2/8/2P5/PPN2PPP/R2QKB1R b KQkq - 1 13',
			},
		},
		'r2qk2r/p1pb1ppp/2p5/2bpP3/3Nn3/8/PPP2PPP/RNBQ1RK1 w kq - 2 10' => {
			eco => 'C55r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. e5 d5 6. Bb5 Ne4 7. Nxd4 Bd7 8. Bxc6 bxc6 9. O-O Bc5

			variation => N__('Two Knights: Modern, Main Line, 8.Bxc6 bxc6 9.O-O Bc5'),
			moves => {
			},
		},
		'r2qk2r/pb1nbppp/1p1ppn2/1N6/2PQ4/1P3NP1/P3PPBP/R1B2RK1 b kq - 2 10' => {
			eco => 'A30v',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. b3 Nbd7 10. Nb5

			variation => N__('English: Symmetrical, Hedgehog, 9. b3 Nbd7 10. Nb5'),
			moves => {
			},
		},
		'r2qk2r/pb1nbppp/1p1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1B2RK1 w kq - 1 10' => {
			eco => 'A30v',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. b3 Nbd7

			variation => N__('English: Symmetrical, Hedgehog, 9. b3 Nbd7'),
			moves => {
				'c3b5' => 'r2qk2r/pb1nbppp/1p1ppn2/1N6/2PQ4/1P3NP1/P3PPBP/R1B2RK1 b kq - 2 10',
			},
		},
		'r2qk2r/pb1nbppp/2p1pN2/8/1p1P4/3BPN2/PP3PPP/R1BQ1RK1 b kq - 0 11' => {
			eco => 'D47p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O b4 10. Ne4 Be7 11. Nxf6+

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O b4 10. Ne4 Be7 11.Nxf6+'),
			moves => {
				'd7f6' => 'r2qk2r/pb2bppp/2p1pn2/8/1p1P4/3BPN2/PP3PPP/R1BQ1RK1 w kq - 0 12',
			},
		},
		'r2qk2r/pb1nbppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQ1RK1 w kq - 2 11' => {
			eco => 'D47o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O b4 10. Ne4 Be7

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O b4 10. Ne4 Be7'),
			moves => {
				'e4f6' => 'r2qk2r/pb1nbppp/2p1pN2/8/1p1P4/3BPN2/PP3PPP/R1BQ1RK1 b kq - 0 11',
			},
		},
		'r2qk2r/pb1pbppp/1pn1pn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 w kq - 1 9' => {
			eco => 'A30t',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 Nc6

			variation => N__('English: Symmetrical, Hedgehog, 8.Qxd4 Nc6'),
			moves => {
			},
		},
		'r2qk2r/pb2bppp/2p1pn2/8/1p1P4/3BPN2/PP3PPP/R1BQ1RK1 w kq - 0 12' => {
			eco => 'D47p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O b4 10. Ne4 Be7 11. Nxf6+ Nxf6

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O b4 10. Ne4 Be7 11.Nxf6+ Nxf6'),
			moves => {
				'e3e4' => 'r2qk2r/pb2bppp/2p1pn2/8/1p1PP3/3B1N2/PP3PPP/R1BQ1RK1 b kq - 0 12',
			},
		},
		'r2qk2r/pb2bppp/2p1pn2/8/1p1PP3/3B1N2/PP3PPP/R1BQ1RK1 b kq - 0 12' => {
			eco => 'D47p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O b4 10. Ne4 Be7 11. Nxf6+ Nxf6 12. e4

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O b4 10. Ne4 Be7 11.Nxf6+ Nxf6 12. e4'),
			moves => {
			},
		},
		'r2qk2r/pbpn1pp1/1p1ppn1p/8/2PP3B/2P1P3/P2N1PPP/R2QKB1R w KQkq - 2 10' => {
			eco => 'E13k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8. bxc3 d6 9. Nd2 Nbd7

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8.bxc3 d6 9. Nd2 Nbd7'),
			moves => {
				'f2f3' => 'r2qk2r/pbpn1pp1/1p1ppn1p/8/2PP3B/2P1PP2/P2N2PP/R2QKB1R b KQkq - 0 10',
			},
		},
		'r2qk2r/pbpn1pp1/1p1ppn1p/8/2PP3B/2P1PP2/P2N2PP/R2QKB1R b KQkq - 0 10' => {
			eco => 'E13k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8. bxc3 d6 9. Nd2 Nbd7 10. f3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8.bxc3 d6 9. Nd2 Nbd7 10. f3'),
			moves => {
				'd8e7' => 'r3k2r/pbpnqpp1/1p1ppn1p/8/2PP3B/2P1PP2/P2N2PP/R2QKB1R w KQkq - 1 11',
			},
		},
		'r2qk2r/pp1b1ppp/2n2n2/2b5/8/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 10' => {
			eco => 'D32h',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'd8e7' => 'r3k2r/pp1bqppp/2n2n2/2b5/8/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 11',
			},
		},
		'r2qk2r/pp1b1ppp/2n2n2/2b5/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 5 10' => {
			eco => 'D32h',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'e2e3' => 'r2qk2r/pp1b1ppp/2n2n2/2b5/8/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 10',
			},
		},
		'r2qk2r/pp1bnppp/2n1p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'C19m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Bd7

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Bd7'),
			moves => {
			},
		},
		'r2qk2r/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b kq - 4 9' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 Nc6 8. Qd2 Bd7 9. O-O-O

			variation => N__('Sicilian: Dragon, Yugoslav, 7... Nc6 8. Qd2 Bd7 9.O-O-O'),
			moves => {
				'a8c8' => '2rqk2r/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R w k - 5 10',
			},
		},
		'r2qk2r/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 3 9' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 Nc6 8. Qd2 Bd7

			variation => N__('Sicilian: Dragon, Yugoslav, 7... Nc6 8. Qd2 Bd7'),
			moves => {
				'e1c1' => 'r2qk2r/pp1bppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b kq - 4 9',
			},
		},
		'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 b kq - 4 9' => {
			eco => 'D18g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Nh4

			variation => N__('Slav: Dutch, 8...Nbd7 9. Nh4'),
			moves => {
				'f5g6' => 'r2qk2r/pp1n1ppp/2p1pnb1/8/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 w kq - 5 10',
			},
		},
		'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP4/1QN1PN2/1P3PPP/R1B2RK1 b kq - 4 9' => {
			eco => 'D18i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Qb3

			variation => N__('Slav: Dutch, 8...Nbd7 9. Qb3'),
			moves => {
				'a7a5' => 'r2qk2r/1p1n1ppp/2p1pn2/p4b2/PbBP4/1QN1PN2/1P3PPP/R1B2RK1 w kq a6 0 10',
			},
		},
		'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 b kq - 4 9' => {
			eco => 'D18k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Qe2

			variation => N__('Slav: Dutch, 8...Nbd7 9. Qe2'),
			moves => {
				'f5g6' => 'r2qk2r/pp1n1ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 w kq - 5 10',
			},
		},
		'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQ1RK1 w kq - 3 9' => {
			eco => 'D18f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7

			variation => N__('Slav: Dutch, 8...Nbd7'),
			moves => {
				'd1b3' => 'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP4/1QN1PN2/1P3PPP/R1B2RK1 b kq - 4 9',
				'd1e2' => 'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 b kq - 4 9',
				'f3h4' => 'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 b kq - 4 9',
			},
		},
		'r2qk2r/pp1n1ppp/2p1pnb1/8/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 w kq - 5 10' => {
			eco => 'D18h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Nh4 Bg6

			variation => N__('Slav: Dutch, 8...Nbd7 9. Nh4 Bg6'),
			moves => {
			},
		},
		'r2qk2r/pp1n1ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 w kq - 5 10' => {
			eco => 'D18l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Qe2 Bg6

			variation => N__('Slav: Dutch, 8...Nbd7 9. Qe2 Bg6'),
			moves => {
				'e3e4' => 'r2qk2r/pp1n1ppp/2p1pnb1/8/PbBPP3/2N2N2/1P2QPPP/R1B2RK1 b kq - 0 10',
			},
		},
		'r2qk2r/pp1n1ppp/2p1pnb1/8/PbBPP3/2N2N2/1P2QPPP/R1B2RK1 b kq - 0 10' => {
			eco => 'D18m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O Nbd7 9. Qe2 Bg6 10. e4

			variation => N__('Slav: Dutch, 8...Nbd7 9. Qe2 Bg6 10. e4'),
			moves => {
			},
		},
		'r2qk2r/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/P3PPBP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3 Bd6

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'c1b2' => 'r2qk2r/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 b kq - 2 8',
			},
		},
		'r2qk2r/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 b kq - 2 8' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3 Bd6 8. Bb2

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'e8g8' => 'r2q1rk1/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 w - - 3 9',
			},
		},
		'r2qk2r/pp1n1ppp/2pbpn2/3p4/2PP2b1/5NP1/PP1NPPBP/R1BQ1RK1 w kq - 4 8' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. Nbd2 Bd6

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'r2qk2r/pp1nbppp/2p1bn2/3p2B1/3Q4/2NB1N2/PPP2PPP/2KRR3 w kq - 4 11' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'd4h4' => 'r2qk2r/pp1nbppp/2p1bn2/3p2B1/7Q/2NB1N2/PPP2PPP/2KRR3 b kq - 5 11',
			},
		},
		'r2qk2r/pp1nbppp/2p1bn2/3p2B1/7Q/2NB1N2/PPP2PPP/2KRR3 b kq - 5 11' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'c6c5' => 'r2qk2r/pp1nbppp/4bn2/2pp2B1/7Q/2NB1N2/PPP2PPP/2KRR3 w kq - 0 12',
				'd7c5' => 'r2qk2r/pp2bppp/2p1bn2/2np2B1/7Q/2NB1N2/PPP2PPP/2KRR3 w kq - 6 12',
			},
		},
		'r2qk2r/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/P3PPBP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3 Be7

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'c1b2' => 'r2qk2r/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 b kq - 2 8',
			},
		},
		'r2qk2r/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 b kq - 2 8' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3 Be7 8. Bb2

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'e8g8' => 'r2q1rk1/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/PB2PPBP/RN1Q1RK1 w - - 3 9',
			},
		},
		'r2qk2r/pp1nbppp/2p1pn2/3p4/2PP2b1/5NP1/PP1NPPBP/R1BQ1RK1 w kq - 4 8' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. Nbd2 Be7

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'r2qk2r/pp1nbppp/2p1pn2/5b2/2QP4/2N2NP1/PP2PPBP/R1B2RK1 b kq - 5 9' => {
			eco => 'D23o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7 8. O-O Be7 9. Nc3

			variation => N__('QGA: Mannheim, Main Line'),
			moves => {
				'e8g8' => 'r2q1rk1/pp1nbppp/2p1pn2/5b2/2QP4/2N2NP1/PP2PPBP/R1B2RK1 w - - 6 10',
			},
		},
		'r2qk2r/pp1nbppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB2RK1 w kq - 4 9' => {
			eco => 'D23n',
                        # TRANSLATORS:

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7'),
			moves => {
				'b1c3' => 'r2qk2r/pp1nbppp/2p1pn2/5b2/2QP4/2N2NP1/PP2PPBP/R1B2RK1 b kq - 5 9',
			},
		},
		'r2qk2r/pp1nbppp/4bn2/2pp2B1/7Q/2NB1N2/PPP2PPP/2KRR3 w kq - 0 12' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 c6 8. O-O-O d5 9. Rhe1 Be6 10. Bd3 Nbd7 11. Qh4 c5

			variation => N__('Bishop\'s Opening: Urusov Gambit, Karpov Variation'),
			moves => {
			},
		},
		'r2qk2r/pp1nbppp/5n2/2Pp4/8/5N2/PPPN1PPP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C08l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nf6 6. Bb5+ Bd7 7. Bxd7+ Nbxd7 8. O-O Be7 9. dxc5

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, Main Line'),
			moves => {
				'd7c5' => 'r2qk2r/pp2bppp/5n2/2np4/8/5N2/PPPN1PPP/R1BQ1RK1 w kq - 0 10',
			},
		},
		'r2qk2r/pp1nbppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'C08k',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, 7.Bxd7'),
			moves => {
				'd4c5' => 'r2qk2r/pp1nbppp/5n2/2Pp4/8/5N2/PPPN1PPP/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r2qk2r/pp2bppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KR3R w kq - 2 10' => {
			eco => 'B53t',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 7. Nc3 Nf6 8. Bg5 e6'),
			moves => {
				'd4d3' => 'r2qk2r/pp2bppp/2bppn2/6B1/4P3/2NQ1N2/PPP2PPP/2KR3R b kq - 3 10',
				'h1e1' => 'r2qk2r/pp2bppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KRR3 b kq - 3 10',
			},
		},
		'r2qk2r/pp2bppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KRR3 b kq - 3 10' => {
			eco => 'B53v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3 Nf6 8. Bg5 e6 9. O-O-O Be7 10. Rhe1

			variation => N__('Sicilian, Chekhover, Main Line, 10.Rhe1'),
			moves => {
				'e8g8' => 'r2q1rk1/pp2bppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KRR3 w - - 4 11',
			},
		},
		'r2qk2r/pp2bppp/2bppn2/6B1/4P3/2NQ1N2/PPP2PPP/2KR3R b kq - 3 10' => {
			eco => 'B53u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3 Nf6 8. Bg5 e6 9. O-O-O Be7 10. Qd3

			variation => N__('Sicilian, Chekhover, Main Line, 10. Qd3'),
			moves => {
			},
		},
		'r2qk2r/pp2bppp/2n1bn2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 5 9' => {
			eco => 'D34b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be7 8. O-O Be6

			variation => N__('QGD Tarrasch: 7. Bg2 Be7 8.O-O Be6'),
			moves => {
			},
		},
		'r2qk2r/pp2bppp/2p1bn2/2np2B1/3N3Q/2NB4/PPP2PPP/2KRR3 b kq - 7 12' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'f6g8' => 'r2qk1nr/pp2bppp/2p1b3/2np2B1/3N3Q/2NB4/PPP2PPP/2KRR3 w kq - 8 13',
			},
		},
		'r2qk2r/pp2bppp/2p1bn2/2np2B1/7Q/2NB1N2/PPP2PPP/2KRR3 w kq - 6 12' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'f3d4' => 'r2qk2r/pp2bppp/2p1bn2/2np2B1/3N3Q/2NB4/PPP2PPP/2KRR3 b kq - 7 12',
			},
		},
		'r2qk2r/pp2bppp/5n2/2np4/8/1N3N2/PPP2PPP/R1BQ1RK1 b kq - 1 10' => {
			eco => 'C08m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nf6 6. Bb5+ Bd7 7. Bxd7+ Nbxd7 8. O-O Be7 9. dxc5 Nxc5 10. Nb3

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, Main Line, 10. Nb3'),
			moves => {
			},
		},
		'r2qk2r/pp2bppp/5n2/2np4/8/5N2/PPPN1PPP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'C08l',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, Main Line'),
			moves => {
				'd2b3' => 'r2qk2r/pp2bppp/5n2/2np4/8/1N3N2/PPP2PPP/R1BQ1RK1 b kq - 1 10',
			},
		},
		'r2qk2r/pp3ppp/2n1pn2/3p1b2/1b1P1B2/1QN1PN2/PP3PPP/R3KB1R w KQkq - 2 9' => {
			eco => 'D14g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Qb3 Bb4

			variation => N__('Slav: Exchange, Trifunovic Variation'),
			moves => {
			},
		},
		'r2qk2r/pp3ppp/2nBpn2/3p4/3P4/2NQPN2/PP3PPP/R3K2R b KQkq - 0 10' => {
			eco => 'D14i',
                        # TRANSLATORS:

			variation => N__('Slav: Exchange, 8. Bd3 Bxd3 9.Qxd3 Bd6'),
			moves => {
				'd8d6' => 'r3k2r/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R3K2R w KQkq - 0 11',
			},
		},
		'r2qk2r/pp3ppp/2nbpn2/3p4/3P1B2/2NQPN2/PP3PPP/R3K2R w KQkq - 1 10' => {
			eco => 'D14i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3 Bxd3 9. Qxd3 Bd6

			variation => N__('Slav: Exchange, 8. Bd3 Bxd3 9.Qxd3 Bd6'),
			moves => {
				'f4d6' => 'r2qk2r/pp3ppp/2nBpn2/3p4/3P4/2NQPN2/PP3PPP/R3K2R b KQkq - 0 10',
			},
		},
		'r2qk2r/ppp1b1pp/2n5/3p1p2/2PPn1b1/3B1N2/PP3PPP/RNBQR1K1 b kq - 0 10' => {
			eco => 'C42x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bg4 9. c3 f5 10. c4

			variation => N__('Russian Game: Classical, Krause Variation'),
			moves => {
				'e7h4' => 'r2qk2r/ppp3pp/2n5/3p1p2/2PPn1bb/3B1N2/PP3PPP/RNBQR1K1 w kq - 1 11',
			},
		},
		'r2qk2r/ppp1b1pp/2n5/3p1p2/3Pn1b1/2PB1N2/PP1N1PPP/R1BQR1K1 b kq - 1 10' => {
			eco => 'C42x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bg4 9. c3 f5 10. Nbd2

			variation => N__('Russian Game: Classical, Berger Variation'),
			moves => {
			},
		},
		'r2qk2r/ppp1b1pp/2n5/3p1p2/3Pn1b1/2PB1N2/PP3PPP/RNBQR1K1 w kq f6 0 10' => {
			eco => 'C42x',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Chigorin, 8... Bg4 9. c3'),
			moves => {
				'b1d2' => 'r2qk2r/ppp1b1pp/2n5/3p1p2/3Pn1b1/2PB1N2/PP1N1PPP/R1BQR1K1 b kq - 1 10',
				'c3c4' => 'r2qk2r/ppp1b1pp/2n5/3p1p2/2PPn1b1/3B1N2/PP3PPP/RNBQR1K1 b kq - 0 10',
			},
		},
		'r2qk2r/ppp1bppp/2n5/3p1b2/3Pn3/3B1N2/PPP2PPP/RNBQR1K1 w kq - 6 9' => {
			eco => 'C42v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bf5

			variation => N__('Russian Game: Classical, Chigorin, 8... Bf5'),
			moves => {
			},
		},
		'r2qk2r/ppp1bppp/2n5/3p4/2PPn1b1/3B1N2/PP3PPP/RNBQR1K1 b kq c3 0 9' => {
			eco => 'C42w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bg4 9. c4

			variation => N__('Russian Game: Classical, Chigorin, 8... Bg4 9. c4'),
			moves => {
			},
		},
		'r2qk2r/ppp1bppp/2n5/3p4/3Pn1b1/2PB1N2/PP3PPP/RNBQR1K1 b kq - 0 9' => {
			eco => 'C42x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bg4 9. c3

			variation => N__('Russian Game: Classical, Chigorin, 8... Bg4 9. c3'),
			moves => {
				'f7f5' => 'r2qk2r/ppp1b1pp/2n5/3p1p2/3Pn1b1/2PB1N2/PP3PPP/RNBQR1K1 w kq f6 0 10',
			},
		},
		'r2qk2r/ppp1bppp/2n5/3p4/3Pn1b1/3B1N2/PPP2PPP/RNBQR1K1 w kq - 6 9' => {
			eco => 'C42w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bg4

			variation => N__('Russian Game: Classical, Chigorin, 8... Bg4'),
			moves => {
				'c2c3' => 'r2qk2r/ppp1bppp/2n5/3p4/3Pn1b1/2PB1N2/PP3PPP/RNBQR1K1 b kq - 0 9',
				'c2c4' => 'r2qk2r/ppp1bppp/2n5/3p4/2PPn1b1/3B1N2/PP3PPP/RNBQR1K1 b kq c3 0 9',
			},
		},
		'r2qk2r/ppp2p1p/2n1bPp1/2b3N1/2pp4/8/PPP2PPP/RNBQR1K1 w kq - 0 10' => {
			eco => 'C55x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 d5 7. exf6 dxc4 8. Re1+ Be6 9. Ng5 g6

			variation => N__('Two Knights: Max Lange, Loman Defence'),
			moves => {
			},
		},
		'r2qk2r/ppp2pPp/2n1b3/2b5/2pp4/5N2/PPP2PPP/RNBQR1K1 b kq - 0 9' => {
			eco => 'C55w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 d5 7. exf6 dxc4 8. Re1+ Be6 9. fxg7

			variation => N__('Two Knights: Max Lange, Schlechter Variation'),
			moves => {
			},
		},
		'r2qk2r/ppp2ppp/2n1bP2/2b3N1/2pp4/8/PPP2PPP/RNBQR1K1 b kq - 3 9' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'd8d5' => 'r3k2r/ppp2ppp/2n1bP2/2bq2N1/2pp4/8/PPP2PPP/RNBQR1K1 w kq - 4 10',
				'g7g6' => 'r2qk2r/ppp2p1p/2n1bPp1/2b3N1/2pp4/8/PPP2PPP/RNBQR1K1 w kq - 0 10',
			},
		},
		'r2qk2r/ppp2ppp/2n1bP2/2b5/2pp4/5N2/PPP2PPP/RNBQR1K1 w kq - 2 9' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'f3g5' => 'r2qk2r/ppp2ppp/2n1bP2/2b3N1/2pp4/8/PPP2PPP/RNBQR1K1 b kq - 3 9',
				'f6g7' => 'r2qk2r/ppp2pPp/2n1b3/2b5/2pp4/5N2/PPP2PPP/RNBQR1K1 b kq - 0 9',
			},
		},
		'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1Q1P/PPP3P1/R1B1K2R b KQkq - 0 8' => {
			eco => 'C30m',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'e5f4' => 'r2qk2r/ppp2ppp/2np1n2/2b5/2B1Pp2/2NP1Q1P/PPP3P1/R1B1K2R w KQkq - 0 9',
			},
		},
		'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1b1P/PPP3P1/R1BQK2R w KQkq - 0 8' => {
			eco => 'C30m',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'd1f3' => 'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1Q1P/PPP3P1/R1B1K2R b KQkq - 0 8',
			},
		},
		'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PPb1/2NP1N1P/PPP3P1/R1BQK2R b KQkq - 0 7' => {
			eco => 'C30m',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'g4f3' => 'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2NP1b1P/PPP3P1/R1BQK2R w KQkq - 0 8',
			},
		},
		'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PPb1/2NP1N2/PPP3PP/R1BQK2R w KQkq - 1 7' => {
			eco => 'C30m',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'h2h3' => 'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PPb1/2NP1N1P/PPP3P1/R1BQK2R b KQkq - 0 7',
			},
		},
		'r2qk2r/ppp2ppp/2np1n2/2b1p3/2B1PPb1/2NP1N2/PPP3PP/R1BQK2R w KQkq - 2 7' => {
			eco => 'C28k',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nc6 4. d3 Bc5 5. f4 d6 6. Nf3 Bg4

			variation => N__('Vienna: 3. Bc4 Nc6 4. d3 Bc5 5. f4 d6 6. Nf3 Bg4'),
			moves => {
			},
		},
		'r2qk2r/ppp2ppp/2np1n2/2b5/2B1Pp2/2NP1Q1P/PPP3P1/R1B1K2R w KQkq - 0 9' => {
			eco => 'C30m',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. Nc3 Nf6 5. Bc4 Nc6 6. d3 Bg4 7. h3 Bxf3 8. Qxf3 exf4

			variation => N__('KGD: Classical, Svenonius Variation'),
			moves => {
			},
		},
		'r2qk2r/ppp3pp/2n5/3p1p2/2PPn1bb/3B1N2/PP3PPP/RNBQR1K1 w kq - 1 11' => {
			eco => 'C42x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O Nc6 8. Re1 Bg4 9. c3 f5 10. c4 Bh4

			variation => N__('Russian Game: Classical, Maroczy Variation'),
			moves => {
			},
		},
		'r2qk2r/pppb1ppp/3b4/3p4/2PPn3/3B4/PP3PPP/RNBQ1RK1 b kq c3 0 8' => {
			eco => 'C43u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7 6. Nxd7 Bxd7 7. O-O Bd6 8. c4

			variation => N__('Russian Game: Modern Attack, 3...Nxe4, Main Line, 7... Bd6 8. c4'),
			moves => {
			},
		},
		'r2qk2r/pppb1ppp/3b4/3p4/3Pn3/3B4/PPP2PPP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'C43t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7 6. Nxd7 Bxd7 7. O-O Bd6

			variation => N__('Russian Game: Modern Attack, 3...Nxe4, Main Line, 7... Bd6'),
			moves => {
				'c2c4' => 'r2qk2r/pppb1ppp/3b4/3p4/2PPn3/3B4/PP3PPP/RNBQ1RK1 b kq c3 0 8',
			},
		},
		'r2qk2r/pppbbppp/2Bp1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Bd7 6. Nc3 Be7 7. Bxc6

			variation => N__('Spanish: Closed Berlin, Showalter Variation'),
			moves => {
			},
		},
		'r2qk2r/pppbbppp/2np1n2/1B2p1B1/3PP3/2N2N2/PPP2PPP/R2Q1RK1 b kq - 4 7' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Bd7 6. Nc3 Be7 7. Bg5

			variation => N__('Spanish: Closed Berlin, Bernstein Variation'),
			moves => {
			},
		},
		'r2qk2r/pppbbppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQ1RK1 w kq - 3 7' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Bd7 6. Nc3 Be7

			variation => N__('Spanish: Closed Berlin, Hedgehog Variation'),
			moves => {
				'b5c6' => 'r2qk2r/pppbbppp/2Bp1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 0 7',
				'c1g5' => 'r2qk2r/pppbbppp/2np1n2/1B2p1B1/3PP3/2N2N2/PPP2PPP/R2Q1RK1 b kq - 4 7',
				'f1e1' => 'r2qk2r/pppbbppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 4 7',
			},
		},
		'r2qk2r/pppbbppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQR1K1 b kq - 4 7' => {
			eco => 'C66',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed Berlin, Hedgehog Variation'),
			moves => {
				'e8g8' => 'r2q1rk1/pppbbppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQR1K1 w - - 5 8',
			},
		},
		'r2qk2r/pppn1ppp/3bpn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQ1RK1 w kq - 5 9' => {
			eco => 'D25r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 Nbd7 8. O-O Bd6

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 Nbd7 8.O-O Bd6'),
			moves => {
				'c4e2' => 'r2qk2r/pppn1ppp/3bpn2/7b/3P4/2N1PN1P/PP2BPP1/R1BQ1RK1 b kq - 6 9',
				'e3e4' => 'r2qk2r/pppn1ppp/3bpn2/7b/2BPP3/2N2N1P/PP3PP1/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r2qk2r/pppn1ppp/3bpn2/7b/2BPP3/2N2N1P/PP3PP1/R1BQ1RK1 b kq - 0 9' => {
			eco => 'D25s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 Nbd7 8. O-O Bd6 9. e4

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 Nbd7 8.O-O Bd6 9. e4'),
			moves => {
			},
		},
		'r2qk2r/pppn1ppp/3bpn2/7b/3P4/2N1PN1P/PP2BPP1/R1BQ1RK1 b kq - 6 9' => {
			eco => 'D25r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 Nbd7 8. O-O Bd6 9. Be2

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 Nbd7 8.O-O Bd6 9. Be2'),
			moves => {
			},
		},
		'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'D47g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Be2 Bb7 9. O-O a6

			variation => N__('Semi-Slav: Meran, Capablanca, 8... Bb7 9.O-O a6'),
			moves => {
			},
		},
		'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 2 10' => {
			eco => 'D48e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O Bb7

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7'),
			moves => {
				'e3e4' => 'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 10',
			},
		},
		'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 10' => {
			eco => 'D48f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O Bb7 10. e4

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7 10. e4'),
			moves => {
				'c6c5' => 'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3PP3/2NB1N2/PP3PPP/R1BQ1RK1 w kq - 0 11',
			},
		},
		'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 1 10' => {
			eco => 'D48i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. e4 Bb7

			variation => N__('Semi-Slav: Meran, 8... a6 9. e4 Bb7'),
			moves => {
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNBR2K1 w kq - 4 10' => {
			eco => 'D29g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7'),
			moves => {
				'a2a4' => 'r2qkb1r/1b1n1ppp/p3pn2/1pp5/P2P4/1B2PN2/1P2QPPP/RNBR2K1 b kq a3 0 10',
				'b1c3' => 'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 b kq - 5 10',
				'e3e4' => 'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3PP3/1B3N2/PP2QPPP/RNBR2K1 b kq - 0 10',
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 b kq - 5 10' => {
			eco => 'D29j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. Nc3

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. Nc3'),
			moves => {
				'd8b6' => 'r3kb1r/1b1n1ppp/pq2pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11',
				'd8b8' => 'rq2kb1r/1b1n1ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11',
				'd8c7' => 'r3kb1r/1bqn1ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11',
				'f8d6' => 'r2qk2r/1b1n1ppp/p2bpn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11',
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3PP3/1B3N2/PP2QPPP/RNBR2K1 b kq - 0 10' => {
			eco => 'D29g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. e4

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. e4'),
			moves => {
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3PP3/2NB1N2/PP3PPP/R1BQ1RK1 w kq - 0 11' => {
			eco => 'D48f',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7 10. e4'),
			moves => {
				'd4d5' => 'r2qkb1r/1b1n1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 11',
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1pp5/P2P4/1B2PN2/1P2QPPP/RNB2RK1 w kq - 1 10' => {
			eco => 'D29e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. a4 Nbd7

			variation => N__('QGA: Classical, 8... Bb7 9. a4 Nbd7'),
			moves => {
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1pp5/P2P4/1B2PN2/1P2QPPP/RNBR2K1 b kq a3 0 10' => {
			eco => 'D29h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. a4

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. a4'),
			moves => {
				'b5b4' => 'r2qkb1r/1b1n1ppp/p3pn2/2p5/Pp1P4/1B2PN2/1P2QPPP/RNBR2K1 w kq - 0 11',
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 11' => {
			eco => 'D48f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O Bb7 10. e4 c5 11. d5

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7 10. e4 c5 11. d5'),
			moves => {
				'd8c7' => 'r3kb1r/1bqn1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQ1RK1 w kq - 1 12',
			},
		},
		'r2qkb1r/1b1n1ppp/p3pn2/2p5/Pp1P4/1B2PN2/1P2QPPP/RNBR2K1 w kq - 0 11' => {
			eco => 'D29i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. a4 b4

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. a4 b4'),
			moves => {
			},
		},
		'r2qkb1r/1b3ppp/p1n1pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 w kq - 4 10' => {
			eco => 'D28m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Nc6 9. Nc3 Bb7

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3 Nc6 9. Nc3 Bb7'),
			moves => {
			},
		},
		'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'C78o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. d4

			variation => N__('Spanish: Archangelsk, 7. d4'),
			moves => {
			},
		},
		'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C78p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. d3

			variation => N__('Spanish: Archangelsk, 7. d3'),
			moves => {
				'f8e7' => 'r2qk2r/1bppbppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'C78n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7

			variation => N__('Spanish: Archangelsk Variation'),
			moves => {
				'c2c3' => 'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 7',
				'd2d3' => 'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1B1P1N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
				'd2d4' => 'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/3PP3/1B3N2/PPP2PPP/RNBQ1RK1 b kq d3 0 7',
				'f1e1' => 'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 b kq - 3 7',
			},
		},
		'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 b kq - 3 7' => {
			eco => 'C78t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. Re1

			variation => N__('Spanish: Archangelsk, 7. Re1'),
			moves => {
				'f8c5' => 'r2qk2r/1bpp1ppp/p1n2n2/1pb1p3/4P3/1B3N2/PPPP1PPP/RNBQR1K1 w kq - 4 8',
			},
		},
		'r2qkb1r/1bpp1ppp/p1n2n2/1p2p3/4P3/1BP2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C78r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. c3

			variation => N__('Spanish: Archangelsk, 7. c3'),
			moves => {
				'f6e4' => 'r2qkb1r/1bpp1ppp/p1n5/1p2p3/4n3/1BP2N2/PP1P1PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'r2qkb1r/1bpp1ppp/p1n5/1p2p3/4n3/1BP2N2/PP1P1PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'C78s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O b5 6. Bb3 Bb7 7. c3 Nxe4

			variation => N__('Spanish: Archangelsk, 7. c3 Nxe4'),
			moves => {
			},
		},
		'r2qkb1r/1p1b1p2/p1nppn1p/6p1/3NPP1B/2N5/PPPQ2PP/2KR1B1R w kq g6 0 11' => {
			eco => 'B67j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 h6 10. Bh4 g5

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 h6 10. Bh4 g5'),
			moves => {
			},
		},
		'r2qkb1r/1p1b1pp1/p1Nppn1p/8/4PB2/2N5/PPPQ1PPP/2KR1B1R b kq - 0 10' => {
			eco => 'B66g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Bf4'),
			moves => {
				'd7c6' => 'r2qkb1r/1p3pp1/p1bppn1p/8/4PB2/2N5/PPPQ1PPP/2KR1B1R w kq - 0 11',
			},
		},
		'r2qkb1r/1p1b1pp1/p1nppn1p/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq - 0 10' => {
			eco => 'B67h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 h6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 h6'),
			moves => {
				'g5h4' => 'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NPP1B/2N5/PPPQ2PP/2KR1B1R b kq - 1 10',
			},
		},
		'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NP3/2N1B3/PPPQ1PPP/2KR1B1R w kq - 2 10' => {
			eco => 'B66k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Bd7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Bd7'),
			moves => {
				'f2f3' => 'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b kq - 0 10',
			},
		},
		'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NP3/2N1BP2/PPPQ2PP/2KR1B1R b kq - 0 10' => {
			eco => 'B66l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Be3 Bd7 10. f3

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Be3 Bd7 10. f3'),
			moves => {
			},
		},
		'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NPB2/2N5/PPPQ1PPP/2KR1B1R w kq - 2 10' => {
			eco => 'B66g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Bf4'),
			moves => {
				'd4c6' => 'r2qkb1r/1p1b1pp1/p1Nppn1p/8/4PB2/2N5/PPPQ1PPP/2KR1B1R b kq - 0 10',
			},
		},
		'r2qkb1r/1p1b1pp1/p1nppn1p/8/3NPP1B/2N5/PPPQ2PP/2KR1B1R b kq - 1 10' => {
			eco => 'B67i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 h6 10. Bh4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 h6 10. Bh4'),
			moves => {
				'g7g5' => 'r2qkb1r/1p1b1p2/p1nppn1p/6p1/3NPP1B/2N5/PPPQ2PP/2KR1B1R w kq g6 0 11',
			},
		},
		'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R b kq - 0 9' => {
			eco => 'B67c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f3

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f3'),
			moves => {
				'a8c8' => '2rqkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R w k - 1 10',
				'f8e7' => 'r2qk2r/1p1bbppp/p1nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R w kq - 1 10',
			},
		},
		'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R w kq - 2 9' => {
			eco => 'B67a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O Bd7'),
			moves => {
				'f1e2' => 'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N5/PPPQBPPP/2KR3R b kq - 3 9',
				'f2f3' => 'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N2P2/PPPQ2PP/2KR1B1R b kq - 0 9',
				'f2f4' => 'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R b kq f3 0 9',
			},
		},
		'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NP3/2N5/PPPQBPPP/2KR3R b kq - 3 9' => {
			eco => 'B67b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. Be2

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. Be2'),
			moves => {
			},
		},
		'r2qkb1r/1p1b1ppp/p1nppn2/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R b kq f3 0 9' => {
			eco => 'B67g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4'),
			moves => {
				'b7b5' => 'r2qkb1r/3b1ppp/p1nppn2/1p4B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq b6 0 10',
				'f8e7' => 'r2qk2r/1p1bbppp/p1nppn2/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq - 1 10',
				'h7h6' => 'r2qkb1r/1p1b1pp1/p1nppn1p/6B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq - 0 10',
			},
		},
		'r2qkb1r/1p1b1ppp/p1nppn2/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'B80r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 Nc6 7. Bg2 a6 8. O-O Bd7

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
			},
		},
		'r2qkb1r/1p1bpppp/p1np1n2/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 w kq - 0 7' => {
			eco => 'B51r',
                        # TRANSLATORS:

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6 6. c3'),
			moves => {
				'b5a4' => 'r2qkb1r/1p1bpppp/p1np1n2/2p5/B3P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 1 7',
				'b5f1' => 'r2qkb1r/1p1bpppp/p1np1n2/2p5/4P3/2P2N2/PP1P1PPP/RNBQRBK1 b kq - 1 7',
			},
		},
		'r2qkb1r/1p1bpppp/p1np1n2/2p5/4P3/2P2N2/PP1P1PPP/RNBQRBK1 b kq - 1 7' => {
			eco => 'B51t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Bf1

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Bf1'),
			moves => {
				'd7g4' => 'r2qkb1r/1p2pppp/p1np1n2/2p5/4P1b1/2P2N2/PP1P1PPP/RNBQRBK1 w kq - 2 8',
			},
		},
		'r2qkb1r/1p1bpppp/p1np1n2/2p5/B3P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 1 7' => {
			eco => 'B51s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Ba4

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Ba4'),
			moves => {
			},
		},
		'r2qkb1r/1p1bpppp/p1np1n2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R w KQkq - 0 8' => {
			eco => 'B60j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Be2 a6

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Be2 a6'),
			moves => {
			},
		},
		'r2qkb1r/1p1bpppp/p1np1n2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8' => {
			eco => 'B61b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 a6

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 a6'),
			moves => {
			},
		},
		'r2qkb1r/1p1n1ppp/p2pbn2/4p3/4P3/1NN1BP2/PPP3PP/R2QKB1R w KQkq - 1 9' => {
			eco => 'B90s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Nbd7

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Nbd7'),
			moves => {
				'd1d2' => 'r2qkb1r/1p1n1ppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R b KQkq - 2 9',
			},
		},
		'r2qkb1r/1p1n1ppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R b KQkq - 2 9' => {
			eco => 'B90t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Nbd7 9. Qd2

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2'),
			moves => {
				'b7b5' => 'r2qkb1r/3n1ppp/p2pbn2/1p2p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R w KQkq b6 0 10',
			},
		},
		'r2qkb1r/1p2pppp/p1n2n2/3p1b2/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D14c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 a6

			variation => N__('Slav: Exchange, 6. Bf4 Bf5 7. e3 a6'),
			moves => {
			},
		},
		'r2qkb1r/1p2pppp/p1n2n2/3p4/3P1Bb1/2N1PN2/PP2BPPP/R2QK2R b KQkq - 2 8' => {
			eco => 'D13o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 a6 7. e3 Bg4 8. Be2

			variation => N__('Slav: Exchange, 6. Bf4 a6 7. e3 Bg4 8. Be2'),
			moves => {
			},
		},
		'r2qkb1r/1p2pppp/p1n2n2/3p4/3P1Bb1/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8' => {
			eco => 'D13n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 a6 7. e3 Bg4

			variation => N__('Slav: Exchange, 6. Bf4 a6 7. e3 Bg4'),
			moves => {
				'f1e2' => 'r2qkb1r/1p2pppp/p1n2n2/3p4/3P1Bb1/2N1PN2/PP2BPPP/R2QK2R b KQkq - 2 8',
			},
		},
		'r2qkb1r/1p2pppp/p1np1n2/2p5/4P1b1/2P2N1P/PP1P1PP1/RNBQRBK1 b kq - 0 8' => {
			eco => 'B51u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Bf1 Bg4 8. h3

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Bf1 Bg4 8. h3'),
			moves => {
			},
		},
		'r2qkb1r/1p2pppp/p1np1n2/2p5/4P1b1/2P2N2/PP1P1PPP/RNBQRBK1 w kq - 2 8' => {
			eco => 'B51t',
                        # TRANSLATORS:

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6 6. c3 a6 7. Bf1'),
			moves => {
				'h2h3' => 'r2qkb1r/1p2pppp/p1np1n2/2p5/4P1b1/2P2N1P/PP1P1PP1/RNBQRBK1 b kq - 0 8',
			},
		},
		'r2qkb1r/1p3pp1/p1bppn1p/8/4PB2/2N2P2/PPPQ2PP/2KR1B1R b kq - 0 11' => {
			eco => 'B66h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O h6 9. Bf4 Bd7 10. Nxc6 Bxc6 11. f3

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Bf4, 11. f3'),
			moves => {
			},
		},
		'r2qkb1r/1p3pp1/p1bppn1p/8/4PB2/2N5/PPPQ1PPP/2KR1B1R w kq - 0 11' => {
			eco => 'B66g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7... a6 8.O-O-O h6 9. Bf4'),
			moves => {
				'f2f3' => 'r2qkb1r/1p3pp1/p1bppn1p/8/4PB2/2N2P2/PPPQ2PP/2KR1B1R b kq - 0 11',
			},
		},
		'r2qkb1r/1p3ppp/p1npbn2/4p1B1/4P3/N1N5/PPP2PPP/R2QKB1R w KQkq - 2 9' => {
			eco => 'B33k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 Be6

			variation => N__('Sicilian: Pelikan, Bird Variation'),
			moves => {
			},
		},
		'r2qkb1r/1ppb1ppp/p1np1n2/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 Nf6

			variation => N__('Spanish: Modern Steinitz, 5. c3 Bd7 6. d4 Nf6'),
			moves => {
			},
		},
		'r2qkb1r/1ppbnppp/p1np4/4p3/3PP3/1BP2N2/PP3PPP/RNBQK2R b KQkq - 2 7' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 Nge7 7. Bb3

			variation => N__('Spanish: Modern Steinitz, Rubinstein, 7. Bb3'),
			moves => {
			},
		},
		'r2qkb1r/1ppbnppp/p1np4/4p3/B2PP3/2P1BN2/PP3PPP/RN1QK2R b KQkq - 2 7' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 Nge7 7. Be3

			variation => N__('Spanish: Modern Steinitz, Rubinstein, 7. Be3'),
			moves => {
			},
		},
		'r2qkb1r/1ppbnppp/p1np4/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 Nge7

			variation => N__('Spanish: Modern Steinitz, Rubinstein Variation'),
			moves => {
				'a4b3' => 'r2qkb1r/1ppbnppp/p1np4/4p3/3PP3/1BP2N2/PP3PPP/RNBQK2R b KQkq - 2 7',
				'c1e3' => 'r2qkb1r/1ppbnppp/p1np4/4p3/B2PP3/2P1BN2/PP3PPP/RN1QK2R b KQkq - 2 7',
			},
		},
		'r2qkb1r/2p2ppp/p1n1B3/1pn1P3/3p4/2P2N2/PP1N1PPP/R1BQ1RK1 b kq - 0 11' => {
			eco => 'C80u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2 Nc5 10. c3 d4 11. Bxe6

			variation => N__('Spanish: Open, Bernstein, 11.Bxe6'),
			moves => {
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B2BN2/PPP2PPP/RN1Q1RK1 b kq - 2 9' => {
			eco => 'C80o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Be3

			variation => N__('Spanish: Open, 8... Be6 9. Be3'),
			moves => {
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNB2RK1 b kq - 2 9' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2

			variation => N__('Spanish: Open, Keres Attack'),
			moves => {
				'f8c5' => 'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1B3N2/PPP1QPPP/RNB2RK1 w kq - 3 10',
				'f8e7' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNB2RK1 w kq - 3 10',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP2PPP/RNBQ1RK1 w kq - 1 9' => {
			eco => 'C80n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6

			variation => N__('Spanish: Open, 8... Be6'),
			moves => {
				'a2a4' => 'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/P3n3/1B3N2/1PP2PPP/RNBQ1RK1 b kq a3 0 9',
				'b1d2' => 'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B3N2/PPPN1PPP/R1BQ1RK1 b kq - 2 9',
				'c1e3' => 'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B2BN2/PPP2PPP/RN1Q1RK1 b kq - 2 9',
				'c2c3' => 'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQ1RK1 b kq - 0 9',
				'd1e2' => 'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B3N2/PPP1QPPP/RNB2RK1 b kq - 2 9',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1B3N2/PPPN1PPP/R1BQ1RK1 b kq - 2 9' => {
			eco => 'C80p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2

			variation => N__('Spanish: Open, Bernstein Variation'),
			moves => {
				'e4c5' => 'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/1B3N2/PPPN1PPP/R1BQ1RK1 w kq - 3 10',
				'f8c5' => 'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1B3N2/PPPN1PPP/R1BQ1RK1 w kq - 3 10',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQ1RK1 b kq - 0 9' => {
			eco => 'C82a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3

			variation => N__('Spanish: Open, 9. c3'),
			moves => {
				'e4c5' => 'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/1BP2N2/PP3PPP/RNBQ1RK1 w kq - 1 10',
				'f8c5' => 'r2qk2r/2p2ppp/p1n1b3/1pbpP3/4n3/1BP2N2/PP3PPP/RNBQ1RK1 w kq - 1 10',
				'f8e7' => 'r2qk2r/2p1bppp/p1n1b3/1p1pP3/4n3/1BP2N2/PP3PPP/RNBQ1RK1 w kq - 1 10',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1p1pP3/P3n3/1B3N2/1PP2PPP/RNBQ1RK1 b kq a3 0 9' => {
			eco => 'C80n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. a4

			variation => N__('Spanish: Open, 8... Be6 9. a4'),
			moves => {
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pn1P1N1/3p4/1BP5/PP1N1PPP/R1BQ1RK1 b kq - 1 11' => {
			eco => 'C80t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2 Nc5 10. c3 d4 11. Ng5

			variation => N__('Spanish: Open, Bernstein, Karpov Gambit'),
			moves => {
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pn1P3/3P4/1B3N2/PP1N1PPP/R1BQ1RK1 b kq - 0 11' => {
			eco => 'C80s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2 Nc5 10. c3 d4 11. cxd4

			variation => N__('Spanish: Open, Bernstein, 11.cxd4'),
			moves => {
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pn1P3/3p4/1BP2N2/PP1N1PPP/R1BQ1RK1 w kq - 0 11' => {
			eco => 'C80s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2 Nc5 10. c3 d4

			variation => N__('Spanish: Open, Bernstein, 9... Nc5 10. c3 d4'),
			moves => {
				'b3e6' => 'r2qkb1r/2p2ppp/p1n1B3/1pn1P3/3p4/2P2N2/PP1N1PPP/R1BQ1RK1 b kq - 0 11',
				'c3d4' => 'r2qkb1r/2p2ppp/p1n1b3/1pn1P3/3P4/1B3N2/PP1N1PPP/R1BQ1RK1 b kq - 0 11',
				'f3g5' => 'r2qkb1r/2p2ppp/p1n1b3/1pn1P1N1/3p4/1BP5/PP1N1PPP/R1BQ1RK1 b kq - 1 11',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/1B3N2/PPPN1PPP/R1BQ1RK1 w kq - 3 10' => {
			eco => 'C80r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Nbd2 Nc5

			variation => N__('Spanish: Open, Bernstein, 9... Nc5'),
			moves => {
				'c2c3' => 'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/1BP2N2/PP1N1PPP/R1BQ1RK1 b kq - 0 10',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/1BP2N2/PP1N1PPP/R1BQ1RK1 b kq - 0 10' => {
			eco => 'C80r',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Bernstein, 9... Nc5'),
			moves => {
				'd5d4' => 'r2qkb1r/2p2ppp/p1n1b3/1pn1P3/3p4/1BP2N2/PP1N1PPP/R1BQ1RK1 w kq - 0 11',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/1BP2N2/PP3PPP/RNBQ1RK1 w kq - 1 10' => {
			eco => 'C82b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5

			variation => N__('Spanish: Open, Berlin Variation'),
			moves => {
				'b3c2' => 'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/2P2N2/PPB2PPP/RNBQ1RK1 b kq - 2 10',
			},
		},
		'r2qkb1r/2p2ppp/p1n1b3/1pnpP3/8/2P2N2/PPB2PPP/RNBQ1RK1 b kq - 2 10' => {
			eco => 'C82b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2

			variation => N__('Spanish: Open, Berlin, 10. Bc2'),
			moves => {
				'e6g4' => 'r2qkb1r/2p2ppp/p1n5/1pnpP3/6b1/2P2N2/PPB2PPP/RNBQ1RK1 w kq - 3 11',
			},
		},
		'r2qkb1r/2p2ppp/p1n5/1pnpP3/6b1/2P2N2/PPB2PPP/RNBQ1RK1 w kq - 3 11' => {
			eco => 'C82c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4'),
			moves => {
				'b1d2' => 'r2qkb1r/2p2ppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQ1RK1 b kq - 4 11',
				'f1e1' => 'r2qkb1r/2p2ppp/p1n5/1pnpP3/6b1/2P2N2/PPB2PPP/RNBQR1K1 b kq - 4 11',
			},
		},
		'r2qkb1r/2p2ppp/p1n5/1pnpP3/6b1/2P2N2/PPB2PPP/RNBQR1K1 b kq - 4 11' => {
			eco => 'C82d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4 11. Re1

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4 11. Re1'),
			moves => {
			},
		},
		'r2qkb1r/2p2ppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQ1RK1 b kq - 4 11' => {
			eco => 'C82e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4 11. Nbd2

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4 11.Nbd2'),
			moves => {
				'f8e7' => 'r2qk2r/2p1bppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQ1RK1 w kq - 5 12',
			},
		},
		'r2qkb1r/2p2ppp/p1pp1n2/4p3/3PP1b1/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C79',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O d6 6. Bxc6+ bxc6 7. d4 Bg4

			variation => N__('Spanish: Steinitz Deferred, Lipnitsky Variation'),
			moves => {
			},
		},
		'r2qkb1r/3b1p1p/p1Nppp2/1p6/4PP2/2N5/PPPQ2PP/2KR1B1R b kq - 0 11' => {
			eco => 'B67o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6 11. Nxc6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11.Nxc6'),
			moves => {
				'd7c6' => 'r2qkb1r/5p1p/p1bppp2/1p6/4PP2/2N5/PPPQ2PP/2KR1B1R w kq - 0 12',
			},
		},
		'r2qkb1r/3b1p1p/p1nppp2/1p3P2/3NP3/2N5/PPPQ2PP/2KR1B1R b kq - 0 11' => {
			eco => 'B67n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6 11. f5

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11. f5'),
			moves => {
			},
		},
		'r2qkb1r/3b1p1p/p1nppp2/1p6/3NPP2/2N5/PPPQ2PP/1K1R1B1R b kq - 1 11' => {
			eco => 'B67q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6 11. Kb1

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11. Kb1'),
			moves => {
				'd8b6' => 'r3kb1r/3b1p1p/pqnppp2/1p6/3NPP2/2N5/PPPQ2PP/1K1R1B1R w kq - 2 12',
			},
		},
		'r2qkb1r/3b1p1p/p1nppp2/1p6/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq - 0 11' => {
			eco => 'B67m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6'),
			moves => {
				'c1b1' => 'r2qkb1r/3b1p1p/p1nppp2/1p6/3NPP2/2N5/PPPQ2PP/1K1R1B1R b kq - 1 11',
				'd4c6' => 'r2qkb1r/3b1p1p/p1Nppp2/1p6/4PP2/2N5/PPPQ2PP/2KR1B1R b kq - 0 11',
				'f4f5' => 'r2qkb1r/3b1p1p/p1nppp2/1p3P2/3NP3/2N5/PPPQ2PP/2KR1B1R b kq - 0 11',
			},
		},
		'r2qkb1r/3b1ppp/p1Nppn2/1p4B1/4PP2/2N5/PPPQ2PP/2KR1B1R b kq - 0 10' => {
			eco => 'B67l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Nxc6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Nxc6'),
			moves => {
			},
		},
		'r2qkb1r/3b1ppp/p1nppB2/1p6/3NPP2/2N5/PPPQ2PP/2KR1B1R b kq - 0 10' => {
			eco => 'B67m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6'),
			moves => {
				'g7f6' => 'r2qkb1r/3b1p1p/p1nppp2/1p6/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq - 0 11',
			},
		},
		'r2qkb1r/3b1ppp/p1nppn2/1p4B1/3NPP2/2N5/PPPQ2PP/2KR1B1R w kq b6 0 10' => {
			eco => 'B67k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5'),
			moves => {
				'd4c6' => 'r2qkb1r/3b1ppp/p1Nppn2/1p4B1/4PP2/2N5/PPPQ2PP/2KR1B1R b kq - 0 10',
				'g5f6' => 'r2qkb1r/3b1ppp/p1nppB2/1p6/3NPP2/2N5/PPPQ2PP/2KR1B1R b kq - 0 10',
			},
		},
		'r2qkb1r/3n1ppp/p2pbn2/1p2p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R w KQkq b6 0 10' => {
			eco => 'B90u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Nbd7 9. Qd2 b5

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 b5'),
			moves => {
			},
		},
		'r2qkb1r/5p1p/p1bppp2/1p6/4PP2/2N5/PPP3PP/2KRQB1R b kq - 1 12' => {
			eco => 'B67p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6 11. Nxc6 Bxc6 12. Qe1

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11.Nxc6 Bxc6 12. Qe1'),
			moves => {
			},
		},
		'r2qkb1r/5p1p/p1bppp2/1p6/4PP2/2N5/PPPQ2PP/2KR1B1R w kq - 0 12' => {
			eco => 'B67o',
                        # TRANSLATORS:

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11.Nxc6'),
			moves => {
				'd2e1' => 'r2qkb1r/5p1p/p1bppp2/1p6/4PP2/2N5/PPP3PP/2KRQB1R b kq - 1 12',
			},
		},
		'r2qkb1r/5p1p/p1npb3/1p1Npp2/4P3/N2B4/PPP2PPP/R2Q1RK1 b kq - 3 12' => {
			eco => 'B33y',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. Bd3 Be6 12. O-O

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. Bd3 Be6 12.O-O'),
			moves => {
			},
		},
		'r2qkb1r/5p1p/p1npb3/1p1Npp2/4P3/N2B4/PPP2PPP/R2QK2R w KQkq - 2 12' => {
			eco => 'B33x',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Ndb5 d6 7. Bg5 a6 8. Na3 b5 9. Bxf6 gxf6 10. Nd5 f5 11. Bd3 Be6

			variation => N__('Sicilian: Pelikan, Sveshnikov, 11. Bd3 Be6'),
			moves => {
				'e1g1' => 'r2qkb1r/5p1p/p1npb3/1p1Npp2/4P3/N2B4/PPP2PPP/R2Q1RK1 b kq - 3 12',
			},
		},
		'r2qkb1r/p1pb1ppp/2p5/3pP3/3Nn3/8/PPP2PPP/RNBQ1RK1 b kq - 1 9' => {
			eco => 'C55q',
                        # TRANSLATORS:

			variation => N__('Two Knights: Modern, Main Line'),
			moves => {
				'f8c5' => 'r2qk2r/p1pb1ppp/2p5/2bpP3/3Nn3/8/PPP2PPP/RNBQ1RK1 w kq - 2 10',
			},
		},
		'r2qkb1r/p1pb1ppp/2p5/3pP3/3Nn3/8/PPP2PPP/RNBQK2R w KQkq - 0 9' => {
			eco => 'C55q',
                        # TRANSLATORS:

			variation => N__('Two Knights: Modern, Main Line'),
			moves => {
				'e1g1' => 'r2qkb1r/p1pb1ppp/2p5/3pP3/3Nn3/8/PPP2PPP/RNBQ1RK1 b kq - 1 9',
			},
		},
		'r2qkb1r/p2b1ppp/2n1pn2/1pp5/2QP4/5NP1/PP2PPBP/RNB2RK1 w kq b6 0 9' => {
			eco => 'E04r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Qa4 Bd7 8. Qxc4 b5

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Qa4 Bd7 8.Qxc4 b5'),
			moves => {
			},
		},
		'r2qkb1r/pb1n1p2/2p1pP2/1p4B1/2pP4/2N3P1/PP3P1P/R2QKB1R b KQkq - 0 12' => {
			eco => 'D44p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. exf6 Bb7 12. g3

			variation => N__('Semi-Slav: Botvinnik, Main Line, 12. g3'),
			moves => {
				'c6c5' => 'r2qkb1r/pb1n1p2/4pP2/1pp3B1/2pP4/2N3P1/PP3P1P/R2QKB1R w KQkq - 0 13',
			},
		},
		'r2qkb1r/pb1n1p2/2p1pP2/1p4B1/2pP4/2N5/PP3PPP/R2QKB1R w KQkq - 1 12' => {
			eco => 'D44o',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, Main Line'),
			moves => {
				'g2g3' => 'r2qkb1r/pb1n1p2/2p1pP2/1p4B1/2pP4/2N3P1/PP3P1P/R2QKB1R b KQkq - 0 12',
			},
		},
		'r2qkb1r/pb1n1p2/2p1pn2/1p2P1B1/2pP4/2N3P1/PP3P1P/R2QKB1R w KQkq - 1 12' => {
			eco => 'D44m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. g3 Bb7

			variation => N__('Semi-Slav: Botvinnik, Lilienthal, 11... Bb7'),
			moves => {
			},
		},
		'r2qkb1r/pb1n1p2/4pP2/1pp3B1/2pP4/2N3P1/PP3P1P/R2QKB1R w KQkq - 0 13' => {
			eco => 'D44q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. exf6 Bb7 12. g3 c5

			variation => N__('Semi-Slav: Botvinnik, Main Line, 12. g3 c5'),
			moves => {
				'd4d5' => 'r2qkb1r/pb1n1p2/4pP2/1ppP2B1/2p5/2N3P1/PP3P1P/R2QKB1R b KQkq - 0 13',
			},
		},
		'r2qkb1r/pb1n1p2/4pP2/1ppP2B1/2p5/2N3P1/PP3P1P/R2QKB1R b KQkq - 0 13' => {
			eco => 'D44q',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, Main Line, 12. g3 c5'),
			moves => {
				'd8b6' => 'r3kb1r/pb1n1p2/1q2pP2/1ppP2B1/2p5/2N3P1/PP3P1P/R2QKB1R w KQkq - 1 14',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQ1RK1 b kq - 3 9' => {
			eco => 'D47f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Be2 Bb7 9. O-O

			variation => N__('Semi-Slav: Meran, Capablanca, 8... Bb7 9.O-O'),
			moves => {
				'a7a6' => 'r2qkb1r/1b1n1ppp/p1p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQ1RK1 w kq - 0 10',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQK2R w KQkq - 2 9' => {
			eco => 'D47f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Be2 Bb7

			variation => N__('Semi-Slav: Meran, Capablanca, 8... Bb7'),
			moves => {
				'e1g1' => 'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2N1PN2/PP2BPPP/R1BQ1RK1 b kq - 3 9',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 3 9' => {
			eco => 'D47n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O'),
			moves => {
				'b5b4' => 'r2qkb1r/pb1n1ppp/2p1pn2/8/1p1P4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 0 10',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQK2R w KQkq - 2 9' => {
			eco => 'D47m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7

			variation => N__('Semi-Slav: Meran, Wade Variation'),
			moves => {
				'a2a3' => 'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/P1NBPN2/1P3PPP/R1BQK2R b KQkq - 0 9',
				'e1g1' => 'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 3 9',
				'e3e4' => 'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 9',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3P4/P1NBPN2/1P3PPP/R1BQK2R b KQkq - 0 9' => {
			eco => 'D47m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. a3

			variation => N__('Semi-Slav: Meran, Wade, 9. a3'),
			moves => {
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/1p6/3PP3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 0 9' => {
			eco => 'D47q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. e4

			variation => N__('Semi-Slav: Meran, Wade, 9. e4'),
			moves => {
				'b5b4' => 'r2qkb1r/pb1n1ppp/2p1pn2/8/1p1PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 10',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/8/1p1P4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 0 10' => {
			eco => 'D47n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O b4

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O b4'),
			moves => {
				'c3e4' => 'r2qkb1r/pb1n1ppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQ1RK1 b kq - 1 10',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQ1RK1 b kq - 1 10' => {
			eco => 'D47o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. O-O b4 10. Ne4

			variation => N__('Semi-Slav: Meran, Wade, 9.O-O b4 10. Ne4'),
			moves => {
				'f8e7' => 'r2qk2r/pb1nbppp/2p1pn2/8/1p1PN3/3BPN2/PP3PPP/R1BQ1RK1 w kq - 2 11',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/8/1p1PP3/2NB1N2/PP3PPP/R1BQK2R w KQkq - 0 10' => {
			eco => 'D47q',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Wade, 9. e4'),
			moves => {
				'c3a4' => 'r2qkb1r/pb1n1ppp/2p1pn2/8/Np1PP3/3B1N2/PP3PPP/R1BQK2R b KQkq - 1 10',
			},
		},
		'r2qkb1r/pb1n1ppp/2p1pn2/8/Np1PP3/3B1N2/PP3PPP/R1BQK2R b KQkq - 1 10' => {
			eco => 'D47q',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Wade, 9. e4'),
			moves => {
				'c6c5' => 'r2qkb1r/pb1n1ppp/4pn2/2p5/Np1PP3/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 11',
			},
		},
		'r2qkb1r/pb1n1ppp/4p3/2NnP3/1p1P4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 12' => {
			eco => 'D47s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. e4 b4 10. Na4 c5 11. e5 Nd5 12. Nxc5

			variation => N__('Semi-Slav: Meran, Wade, Main Line, 12.Nxc5'),
			moves => {
			},
		},
		'r2qkb1r/pb1n1ppp/4p3/2pnP3/Np1P4/3B1N2/PP3PPP/R1BQ1RK1 b kq - 2 12' => {
			eco => 'D47t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. e4 b4 10. Na4 c5 11. e5 Nd5 12. O-O

			variation => N__('Semi-Slav: Meran, Wade, Main Line, 12.O-O'),
			moves => {
			},
		},
		'r2qkb1r/pb1n1ppp/4p3/2pnP3/Np1P4/3B1N2/PP3PPP/R1BQK2R w KQkq - 1 12' => {
			eco => 'D47r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 Bb7 9. e4 b4 10. Na4 c5 11. e5 Nd5

			variation => N__('Semi-Slav: Meran, Wade, Main Line'),
			moves => {
				'a4c5' => 'r2qkb1r/pb1n1ppp/4p3/2NnP3/1p1P4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 12',
				'e1g1' => 'r2qkb1r/pb1n1ppp/4p3/2pnP3/Np1P4/3B1N2/PP3PPP/R1BQ1RK1 b kq - 2 12',
			},
		},
		'r2qkb1r/pb1n1ppp/4pn2/2p1P3/Np1P4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 11' => {
			eco => 'D47q',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Wade, 9. e4'),
			moves => {
				'f6d5' => 'r2qkb1r/pb1n1ppp/4p3/2pnP3/Np1P4/3B1N2/PP3PPP/R1BQK2R w KQkq - 1 12',
			},
		},
		'r2qkb1r/pb1n1ppp/4pn2/2p5/Np1PP3/3B1N2/PP3PPP/R1BQK2R w KQkq - 0 11' => {
			eco => 'D47q',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Meran, Wade, 9. e4'),
			moves => {
				'e4e5' => 'r2qkb1r/pb1n1ppp/4pn2/2p1P3/Np1P4/3B1N2/PP3PPP/R1BQK2R b KQkq - 0 11',
			},
		},
		'r2qkb1r/pb1p1ppp/1pn1pn2/8/2PNP3/P1N5/1PQ2PPP/R1B1KB1R w KQkq - 1 9' => {
			eco => 'E12i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. Qc2 Bb7 6. Nc3 c5 7. e4 cxd4 8. Nxd4 Nc6

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7 6. Nc3 c5 7. e4 cd 8.Nxd4 Nc6'),
			moves => {
			},
		},
		'r2qkb1r/pp1b1ppp/1nn1p3/3pP3/3P4/3B4/PP1NNPPP/R1BQ1RK1 w kq - 3 10' => {
			eco => 'C06f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 Nc6 7. Ne2 cxd4 8. cxd4 Nb6 9. O-O Bd7

			variation => N__('French: Tarrasch, Closed, Leningrad, 9.O-O Bd7'),
			moves => {
			},
		},
		'r2qkb1r/pp1b1ppp/2Bppn2/6B1/3NP3/2N5/PPP2PPP/R2QK2R b KQkq - 0 8' => {
			eco => 'B62i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Bb5 Bd7 8. Bxc6

			variation => N__('Sicilian: Richter-Rauzer, Margate, 7... Bd7 8.Bxc6'),
			moves => {
			},
		},
		'r2qkb1r/pp1b1ppp/2n1pn2/2p1N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 w kq - 4 8' => {
			eco => 'E04m',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Ne5'),
			moves => {
				'b1a3' => 'r2qkb1r/pp1b1ppp/2n1pn2/2p1N3/2pP4/N5P1/PP2PPBP/R1BQ1RK1 b kq - 5 8',
			},
		},
		'r2qkb1r/pp1b1ppp/2n1pn2/2p1N3/2pP4/N5P1/PP2PPBP/R1BQ1RK1 b kq - 5 8' => {
			eco => 'E04n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Ne5 Bd7 8. Na3

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Ne5 Bd7 8. Na3'),
			moves => {
				'c5d4' => 'r2qkb1r/pp1b1ppp/2n1pn2/4N3/2pp4/N5P1/PP2PPBP/R1BQ1RK1 w kq - 0 9',
			},
		},
		'r2qkb1r/pp1b1ppp/2n1pn2/2p5/2QP4/5NP1/PP2PPBP/RNB2RK1 b kq - 0 8' => {
			eco => 'E04q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Qa4 Bd7 8. Qxc4

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Qa4 Bd7 8.Qxc4'),
			moves => {
				'b7b5' => 'r2qkb1r/p2b1ppp/2n1pn2/1pp5/2QP4/5NP1/PP2PPBP/RNB2RK1 w kq b6 0 9',
			},
		},
		'r2qkb1r/pp1b1ppp/2n1pn2/2p5/Q1pP4/5NP1/PP2PPBP/RNB2RK1 w kq - 4 8' => {
			eco => 'E04p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Qa4 Bd7

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Qa4 Bd7'),
			moves => {
				'a4c4' => 'r2qkb1r/pp1b1ppp/2n1pn2/2p5/2QP4/5NP1/PP2PPBP/RNB2RK1 b kq - 0 8',
			},
		},
		'r2qkb1r/pp1b1ppp/2n1pn2/4N3/2Np4/6P1/PP2PPBP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'E04n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O Nc6 7. Ne5 Bd7 8. Na3 cxd4 9. Naxc4

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Ne5 Bd7 8. Na3'),
			moves => {
			},
		},
		'r2qkb1r/pp1b1ppp/2n1pn2/4N3/2pp4/N5P1/PP2PPBP/R1BQ1RK1 w kq - 0 9' => {
			eco => 'E04n',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O Nc6 7. Ne5 Bd7 8. Na3'),
			moves => {
				'a3c4' => 'r2qkb1r/pp1b1ppp/2n1pn2/4N3/2Np4/6P1/PP2PPBP/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r2qkb1r/pp1b1ppp/2n2n2/3Q4/8/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 3 9' => {
			eco => 'D32h',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'd5d1' => 'r2qkb1r/pp1b1ppp/2n2n2/8/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 4 9',
			},
		},
		'r2qkb1r/pp1b1ppp/2n2n2/8/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 4 9' => {
			eco => 'D32h',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'f8c5' => 'r2qk2r/pp1b1ppp/2n2n2/2b5/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 5 10',
			},
		},
		'r2qkb1r/pp1b1ppp/2nppn2/1B4B1/3NP3/2N5/PPP2PPP/R2QK2R w KQkq - 2 8' => {
			eco => 'B62h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Bb5 Bd7

			variation => N__('Sicilian: Richter-Rauzer, Margate, 7... Bd7'),
			moves => {
				'b5c6' => 'r2qkb1r/pp1b1ppp/2Bppn2/6B1/3NP3/2N5/PPP2PPP/R2QK2R b KQkq - 0 8',
			},
		},
		'r2qkb1r/pp1bnppp/2n1p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R w KQkq - 5 7' => {
			eco => 'C02q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Bd7 6. Be2 Nge7

			variation => N__('French: Advance, Euwe, 6. Be2 Nge7'),
			moves => {
			},
		},
		'r2qkb1r/pp1bpppp/2np1B2/8/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B60h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Bxf6

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7.Bxf6'),
			moves => {
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 6' => {
			eco => 'B51r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1 Nf6 6. c3

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6 6. c3'),
			moves => {
				'a7a6' => 'r2qkb1r/1p1bpppp/p1np1n2/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 w kq - 0 7',
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 6 6' => {
			eco => 'B51q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1 Nf6

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 Nf6'),
			moves => {
				'c2c3' => 'r2qkb1r/pp1bpppp/2np1n2/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQR1K1 b kq - 0 6',
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R b KQkq - 6 7' => {
			eco => 'B60i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Be2

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Be2'),
			moves => {
				'a7a6' => 'r2qkb1r/1p1bpppp/p1np1n2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R w KQkq - 0 8',
				'd8a5' => 'r3kb1r/pp1bpppp/2np1n2/q5B1/3NP3/2N5/PPP1BPPP/R2QK2R w KQkq - 7 8',
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 5 7' => {
			eco => 'B60f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7

			variation => N__('Sicilian: Richter-Rauzer, Larsen Variation'),
			moves => {
				'd1d2' => 'r2qkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 6 7',
				'd4b3' => 'r2qkb1r/pp1bpppp/2np1n2/6B1/4P3/1NN5/PPP2PPP/R2QKB1R b KQkq - 6 7',
				'f1e2' => 'r2qkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R b KQkq - 6 7',
				'g5f6' => 'r2qkb1r/pp1bpppp/2np1B2/8/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 6 7' => {
			eco => 'B61a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2'),
			moves => {
				'a7a6' => 'r2qkb1r/1p1bpppp/p1np1n2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8',
				'a8c8' => '2rqkb1r/pp1bpppp/2np1n2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQk - 7 8',
				'c6d4' => 'r2qkb1r/pp1bpppp/3p1n2/6B1/3nP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8',
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/6B1/4P3/1NN5/PPP2PPP/R2QKB1R b KQkq - 6 7' => {
			eco => 'B60g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Nb3

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Nb3'),
			moves => {
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/8/2BNP3/2N5/PPP2PPP/R1BQ1RK1 b kq - 6 7' => {
			eco => 'B57c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Bd7 7. O-O

			variation => N__('Sicilian: Sozin, 6... Bd7 7.O-O'),
			moves => {
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 5 7' => {
			eco => 'B57b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bc4 Bd7

			variation => N__('Sicilian: Sozin, 6... Bd7'),
			moves => {
				'e1g1' => 'r2qkb1r/pp1bpppp/2np1n2/8/2BNP3/2N5/PPP2PPP/R1BQ1RK1 b kq - 6 7',
			},
		},
		'r2qkb1r/pp1bpppp/2np1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 5 7' => {
			eco => 'B58b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 Bd7

			variation => N__('Sicilian: Boleslavsky, 6... Bd7'),
			moves => {
			},
		},
		'r2qkb1r/pp1bpppp/3p1n2/6B1/3nP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 0 8' => {
			eco => 'B61c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Qd2 Nxd4

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Qd2 Nxd4'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/2n1p3/1B1p1b2/3P1B2/2N1PN2/PP3PPP/R2QK2R w KQkq - 2 9' => {
			eco => 'D14p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bb5 Nd7

			variation => N__('Slav: Exchange, 8. Bb5 Nd7'),
			moves => {
				'd1a4' => 'r2qkb1r/pp1n1ppp/2n1p3/1B1p1b2/Q2P1B2/2N1PN2/PP3PPP/R3K2R b KQkq - 3 9',
			},
		},
		'r2qkb1r/pp1n1ppp/2n1p3/1B1p1b2/Q2P1B2/2N1PN2/PP3PPP/R3K2R b KQkq - 3 9' => {
			eco => 'D14q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bb5 Nd7 9. Qa4

			variation => N__('Slav: Exchange, 8. Bb5 Nd7 9. Qa4'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'D11o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O Nbd7

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP2b1/1P3NP1/P3PPBP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'D11n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. b3

			variation => N__('Slav: Slav-Reti with b3'),
			moves => {
				'f8d6' => 'r2qk2r/pp1n1ppp/2pbpn2/3p4/2PP2b1/1P3NP1/P3PPBP/RNBQ1RK1 w kq - 1 8',
				'f8e7' => 'r2qk2r/pp1nbppp/2p1pn2/3p4/2PP2b1/1P3NP1/P3PPBP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP2b1/5NP1/PP1NPPBP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7 7. Nbd2

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'f8d6' => 'r2qk2r/pp1n1ppp/2pbpn2/3p4/2PP2b1/5NP1/PP1NPPBP/R1BQ1RK1 w kq - 4 8',
				'f8e7' => 'r2qk2r/pp1nbppp/2p1pn2/3p4/2PP2b1/5NP1/PP1NPPBP/R1BQ1RK1 w kq - 4 8',
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O Nbd7

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'b1d2' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP2b1/5NP1/PP1NPPBP/R1BQ1RK1 b kq - 3 7',
				'b2b3' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP2b1/1P3NP1/P3PPBP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2NQPN2/PP3PPP/R1B1K2R w KQkq - 1 8' => {
			eco => 'D12m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Bd3 Bxd3 7. Qxd3 Nbd7

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Bd3 Bxd3 7.Qxd3 Nbd7'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p4/3P2b1/5NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'D02t',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 c6 4. Bg2 Bg4 5. O-O Nbd7 6. Nbd2 e6

			variation => N__('Queen\'s Pawn: 3. g3 c6 4. Bg2 Bg4'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/3p4/6b1/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bg4 5. d3 Nbd7 6. Nbd2 e6

			variation => N__('Reti: KIA, Yugoslav, Main Line, 6.Nbd2 e6'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 2 8' => {
			eco => 'D23n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7'),
			moves => {
				'e1g1' => 'r2qkb1r/pp1n1ppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB2RK1 b kq - 3 8',
			},
		},
		'r2qkb1r/pp1n1ppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB2RK1 b kq - 3 8' => {
			eco => 'D23n',
                        # TRANSLATORS:

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. g3 e6 7. Bg2 Nbd7'),
			moves => {
				'f8e7' => 'r2qk2r/pp1nbppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB2RK1 w kq - 4 9',
			},
		},
		'r2qkb1r/pp1n1ppp/2p2n2/3pp3/6b1/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq e6 0 7' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bg4 5. d3 Nbd7 6. Nbd2 e5

			variation => N__('Reti: KIA, Yugoslav, Main Line, 6.Nbd2 e5'),
			moves => {
			},
		},
		'r2qkb1r/pp1n1ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'C08k',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, 7.Bxd7'),
			moves => {
				'f8e7' => 'r2qk2r/pp1nbppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'r2qkb1r/pp1n1ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'C08k',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, 7.Bxd7'),
			moves => {
				'e1g1' => 'r2qkb1r/pp1n1ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'r2qkb1r/pp1nppp1/2p2n1p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R w KQkq - 1 11' => {
			eco => 'B19j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Ngf6

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 Ngf6'),
			moves => {
			},
		},
		'r2qkb1r/pp1npppp/2p2n2/3p4/3P2b1/5NP1/PPP1PPBP/RNBQ1RK1 w kq - 4 6' => {
			eco => 'D02t',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 c6 4. Bg2 Bg4 5. O-O Nbd7

			variation => N__('Queen\'s Pawn: 3. g3 c6 4. Bg2 Bg4'),
			moves => {
				'b1d2' => 'r2qkb1r/pp1npppp/2p2n2/3p4/3P2b1/5NP1/PPPNPPBP/R1BQ1RK1 b kq - 5 6',
			},
		},
		'r2qkb1r/pp1npppp/2p2n2/3p4/3P2b1/5NP1/PPPNPPBP/R1BQ1RK1 b kq - 5 6' => {
			eco => 'D02t',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: 3. g3 c6 4. Bg2 Bg4'),
			moves => {
				'e7e6' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p4/3P2b1/5NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7',
			},
		},
		'r2qkb1r/pp1npppp/2p2n2/3p4/6b1/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bg4 5. d3 Nbd7

			variation => N__('Reti: KIA, Yugoslav, Main Line'),
			moves => {
				'b1d2' => 'r2qkb1r/pp1npppp/2p2n2/3p4/6b1/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6',
			},
		},
		'r2qkb1r/pp1npppp/2p2n2/3p4/6b1/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bg4 5. d3 Nbd7 6. Nbd2

			variation => N__('Reti: KIA, Yugoslav, Main Line, 6.Nbd2'),
			moves => {
				'e7e5' => 'r2qkb1r/pp1n1ppp/2p2n2/3pp3/6b1/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq e6 0 7',
				'e7e6' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p4/6b1/3P1NP1/PPPNPPBP/R1BQ1RK1 w kq - 0 7',
			},
		},
		'r2qkb1r/pp1npppp/2p2n2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'D17h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 Nbd7

			variation => N__('Slav: Central, 6. Ne5 Nbd7'),
			moves => {
				'e5c4' => 'r2qkb1r/pp1npppp/2p2n2/5b2/P1NP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r2qkb1r/pp1npppp/2p2n2/5b2/P1NP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D17h',
                        # TRANSLATORS:

			variation => N__('Slav: Central, 6. Ne5 Nbd7'),
			moves => {
				'd8c7' => 'r3kb1r/ppqnpppp/2p2n2/5b2/P1NP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 1 8',
			},
		},
		'r2qkb1r/pp1npppp/3p1n2/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'B52d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Nxd7 5. O-O Ngf6

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Nxd7 5.O-O Ngf6'),
			moves => {
				'd1e2' => 'r2qkb1r/pp1npppp/3p1n2/2p5/4P3/5N2/PPPPQPPP/RNB2RK1 b kq - 3 6',
			},
		},
		'r2qkb1r/pp1npppp/3p1n2/2p5/4P3/5N2/PPPPQPPP/RNB2RK1 b kq - 3 6' => {
			eco => 'B52e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Nxd7 5. O-O Ngf6 6. Qe2

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Nxd7 5.O-O Ngf6 6. Qe2'),
			moves => {
			},
		},
		'r2qkb1r/pp2pp1p/2bp1np1/8/2PQP3/2N2N2/PP3PPP/R1B1K2R w KQkq - 0 9' => {
			eco => 'B53q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. c4 Nf6 8. Nc3 g6

			variation => N__('Sicilian, Chekhover, 7. c4 Nf6 8. Nc3 g6'),
			moves => {
			},
		},
		'r2qkb1r/pp2pppp/1nn5/8/3P4/1QN2P2/PP3P1P/R1B1KB1R w KQkq - 1 10' => {
			eco => 'B13t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3 Bg4 7. cxd5 Nxd5 8. Qb3 Bxf3 9. gxf3 Nb6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4: 9.gxf3 Nb6'),
			moves => {
			},
		},
		'r2qkb1r/pp2pppp/2bp1n2/6B1/3QP3/2N2N2/PPP2PPP/R3K2R b KQkq - 3 8' => {
			eco => 'B53s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3 Nf6 8. Bg5

			variation => N__('Sicilian, Chekhover, 7. Nc3 Nf6 8. Bg5'),
			moves => {
				'e7e6' => 'r2qkb1r/pp3ppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/R3K2R w KQkq - 0 9',
			},
		},
		'r2qkb1r/pp2pppp/2bp1n2/8/2PQP3/2N2N2/PP3PPP/R1B1K2R b KQkq - 2 8' => {
			eco => 'B53p',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 7. c4'),
			moves => {
				'g7g6' => 'r2qkb1r/pp2pp1p/2bp1np1/8/2PQP3/2N2N2/PP3PPP/R1B1K2R w KQkq - 0 9',
			},
		},
		'r2qkb1r/pp2pppp/2bp1n2/8/2PQP3/5N2/PP3PPP/RNB1K2R w KQkq - 1 8' => {
			eco => 'B53p',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 7. c4'),
			moves => {
				'b1c3' => 'r2qkb1r/pp2pppp/2bp1n2/8/2PQP3/2N2N2/PP3PPP/R1B1K2R b KQkq - 2 8',
			},
		},
		'r2qkb1r/pp2pppp/2bp1n2/8/3QP3/2N2N2/PPP2PPP/R1B1K2R w KQkq - 2 8' => {
			eco => 'B53r',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 7. Nc3'),
			moves => {
				'c1g5' => 'r2qkb1r/pp2pppp/2bp1n2/6B1/3QP3/2N2N2/PPP2PPP/R3K2R b KQkq - 3 8',
			},
		},
		'r2qkb1r/pp2pppp/2n1bn2/3p2B1/2PP4/2N5/PP3PPP/R2QKBNR w KQkq - 5 7' => {
			eco => 'B13x',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Bg5 Be6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Bg5 Be6'),
			moves => {
			},
		},
		'r2qkb1r/pp2pppp/2n2n2/3P4/3P2b1/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B13s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3 Bg4 7. cxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4'),
			moves => {
				'f6d5' => 'r2qkb1r/pp2pppp/2n5/3n4/3P2b1/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'r2qkb1r/pp2pppp/2n2n2/3p1b2/3P1B2/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D14b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3

			variation => N__('Slav: Exchange, 6. Bf4 Bf5 7. e3'),
			moves => {
				'a7a6' => 'r2qkb1r/1p2pppp/p1n2n2/3p1b2/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 8',
				'e7e6' => 'r2qkb1r/pp3ppp/2n1pn2/3p1b2/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'r2qkb1r/pp2pppp/2n2n2/3p1b2/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 7' => {
			eco => 'D14a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5

			variation => N__('Slav: Exchange, 6. Bf4 Bf5'),
			moves => {
				'e2e3' => 'r2qkb1r/pp2pppp/2n2n2/3p1b2/3P1B2/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'r2qkb1r/pp2pppp/2n2n2/3p4/2PP2b1/2N2N2/PP3PPP/R1BQKB1R w KQkq - 5 7' => {
			eco => 'B13s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3 Bg4

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4'),
			moves => {
				'c4d5' => 'r2qkb1r/pp2pppp/2n2n2/3P4/3P2b1/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r2qkb1r/pp2pppp/2n2n2/3p4/3P1Bb1/1QPB4/PP3PPP/RN2K1NR b KQkq - 4 7' => {
			eco => 'B13k',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Exchange, Rubinstein, 6... Bg4'),
			moves => {
				'd8c8' => 'r1q1kb1r/pp2pppp/2n2n2/3p4/3P1Bb1/1QPB4/PP3PPP/RN2K1NR w KQkq - 5 8',
				'd8d7' => 'r3kb1r/pp1qpppp/2n2n2/3p4/3P1Bb1/1QPB4/PP3PPP/RN2K1NR w KQkq - 5 8',
			},
		},
		'r2qkb1r/pp2pppp/2n2n2/3p4/3P1Bb1/2PB4/PP3PPP/RN1QK1NR w KQkq - 3 7' => {
			eco => 'B13k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 Nf6 6. Bf4 Bg4

			variation => N__('Caro-Kann: Exchange, Rubinstein, 6... Bg4'),
			moves => {
				'd1b3' => 'r2qkb1r/pp2pppp/2n2n2/3p4/3P1Bb1/1QPB4/PP3PPP/RN2K1NR b KQkq - 4 7',
			},
		},
		'r2qkb1r/pp2pppp/2n5/3n4/3P2b1/1QN2N2/PP3PPP/R1B1KB1R b KQkq - 1 8' => {
			eco => 'B13t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3 Bg4 7. cxd5 Nxd5 8. Qb3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4: 8. Qb3'),
			moves => {
				'g4f3' => 'r2qkb1r/pp2pppp/2n5/3n4/3P4/1QN2b2/PP3PPP/R1B1KB1R w KQkq - 0 9',
			},
		},
		'r2qkb1r/pp2pppp/2n5/3n4/3P2b1/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B13s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3 Bg4 7. cxd5 Nxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4'),
			moves => {
				'd1b3' => 'r2qkb1r/pp2pppp/2n5/3n4/3P2b1/1QN2N2/PP3PPP/R1B1KB1R b KQkq - 1 8',
			},
		},
		'r2qkb1r/pp2pppp/2n5/3n4/3P4/1QN2P2/PP3P1P/R1B1KB1R b KQkq - 0 9' => {
			eco => 'B13t',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4: 8. Qb3'),
			moves => {
				'd5b6' => 'r2qkb1r/pp2pppp/1nn5/8/3P4/1QN2P2/PP3P1P/R1B1KB1R w KQkq - 1 10',
				'e7e6' => 'r2qkb1r/pp3ppp/2n1p3/3n4/3P4/1QN2P2/PP3P1P/R1B1KB1R w KQkq - 0 10',
			},
		},
		'r2qkb1r/pp2pppp/2n5/3n4/3P4/1QN2b2/PP3PPP/R1B1KB1R w KQkq - 0 9' => {
			eco => 'B13t',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4: 8. Qb3'),
			moves => {
				'g2f3' => 'r2qkb1r/pp2pppp/2n5/3n4/3P4/1QN2P2/PP3P1P/R1B1KB1R b KQkq - 0 9',
			},
		},
		'r2qkb1r/pp2pppp/n1p2n2/4Nb2/P1pP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D17g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 Na6 7. f3

			variation => N__('Slav: Central, 6. Ne5 Na6 7. f3'),
			moves => {
			},
		},
		'r2qkb1r/pp2pppp/n1p2n2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'D17g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 Na6

			variation => N__('Slav: Central, 6. Ne5 Na6'),
			moves => {
				'f2f3' => 'r2qkb1r/pp2pppp/n1p2n2/4Nb2/P1pP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r2qkb1r/pp2pppp/n1p2n2/5b2/P1pP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D18b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 Na6

			variation => N__('Slav: Dutch, Lasker Variation'),
			moves => {
			},
		},
		'r2qkb1r/pp2pppp/n1p2n2/8/P1BPP1b1/2N2N2/1P3PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'D16j',
                        # TRANSLATORS:

			variation => N__('Slav: Smyslov, 6. e4 Bg4'),
			moves => {
				'e7e6' => 'r2qkb1r/pp3ppp/n1p1pn2/8/P1BPP1b1/2N2N2/1P3PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r2qkb1r/pp2pppp/n1p2n2/8/P1pP2b1/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D16h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6 6. e3 Bg4

			variation => N__('Slav: Smyslov, 6. e3 Bg4'),
			moves => {
			},
		},
		'r2qkb1r/pp2pppp/n1p2n2/8/P1pPP1b1/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D16j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6 6. e4 Bg4

			variation => N__('Slav: Smyslov, 6. e4 Bg4'),
			moves => {
				'f1c4' => 'r2qkb1r/pp2pppp/n1p2n2/8/P1BPP1b1/2N2N2/1P3PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'r2qkb1r/pp3ppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KR3R b kq - 1 9' => {
			eco => 'B53t',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 7. Nc3 Nf6 8. Bg5 e6'),
			moves => {
				'f8e7' => 'r2qk2r/pp2bppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KR3R w kq - 2 10',
			},
		},
		'r2qkb1r/pp3ppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/R3K2R w KQkq - 0 9' => {
			eco => 'B53t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3 Nf6 8. Bg5 e6

			variation => N__('Sicilian, Chekhover, 7. Nc3 Nf6 8. Bg5 e6'),
			moves => {
				'e1c1' => 'r2qkb1r/pp3ppp/2bppn2/6B1/3QP3/2N2N2/PPP2PPP/2KR3R b kq - 1 9',
			},
		},
		'r2qkb1r/pp3ppp/2n1bn2/2pp4/3P4/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8' => {
			eco => 'D33m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Be6

			variation => N__('QGD Tarrasch: 7. Bg2 Be6'),
			moves => {
			},
		},
		'r2qkb1r/pp3ppp/2n1p3/3n4/3P4/1QN2P2/PP3P1P/R1B1KB1R w KQkq - 0 10' => {
			eco => 'B13u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 Nc6 6. Nf3 Bg4 7. cxd5 Nxd5 8. Qb3 Bxf3 9. gxf3 e6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... Nc6 6. Nf3 Bg4: Main Line'),
			moves => {
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/1B1p1b2/3P1B2/2N1PN2/PP3PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'D14o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bb5

			variation => N__('Slav: Exchange, 8. Bb5'),
			moves => {
				'f6d7' => 'r2qkb1r/pp1n1ppp/2n1p3/1B1p1b2/3P1B2/2N1PN2/PP3PPP/R2QK2R w KQkq - 2 9',
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/3p1b2/3P1B2/1QN1PN2/PP3PPP/R3KB1R b KQkq - 1 8' => {
			eco => 'D14f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Qb3

			variation => N__('Slav: Exchange, 8. Qb3'),
			moves => {
				'f8b4' => 'r2qk2r/pp3ppp/2n1pn2/3p1b2/1b1P1B2/1QN1PN2/PP3PPP/R3KB1R w KQkq - 2 9',
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/3p1b2/3P1B2/2N1PN2/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D14d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6

			variation => N__('Slav: Exchange, 6. Bf4 Bf5 7. e3 e6'),
			moves => {
				'd1b3' => 'r2qkb1r/pp3ppp/2n1pn2/3p1b2/3P1B2/1QN1PN2/PP3PPP/R3KB1R b KQkq - 1 8',
				'f1b5' => 'r2qkb1r/pp3ppp/2n1pn2/1B1p1b2/3P1B2/2N1PN2/PP3PPP/R2QK2R b KQkq - 1 8',
				'f1d3' => 'r2qkb1r/pp3ppp/2n1pn2/3p1b2/3P1B2/2NBPN2/PP3PPP/R2QK2R b KQkq - 1 8',
				'f3e5' => 'r2qkb1r/pp3ppp/2n1pn2/3pNb2/3P1B2/2N1P3/PP3PPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/3p1b2/3P1B2/2NBPN2/PP3PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'D14h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3

			variation => N__('Slav: Exchange, 8. Bd3'),
			moves => {
				'f5d3' => 'r2qkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2NbPN2/PP3PPP/R2QK2R w KQkq - 0 9',
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2NQPN2/PP3PPP/R3K2R b KQkq - 0 9' => {
			eco => 'D14h',
                        # TRANSLATORS:

			variation => N__('Slav: Exchange, 8. Bd3'),
			moves => {
				'f8d6' => 'r2qk2r/pp3ppp/2nbpn2/3p4/3P1B2/2NQPN2/PP3PPP/R3K2R w KQkq - 1 10',
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2NbPN2/PP3PPP/R2QK2R w KQkq - 0 9' => {
			eco => 'D14h',
                        # TRANSLATORS:

			variation => N__('Slav: Exchange, 8. Bd3'),
			moves => {
				'd1d3' => 'r2qkb1r/pp3ppp/2n1pn2/3p4/3P1B2/2NQPN2/PP3PPP/R3K2R b KQkq - 0 9',
			},
		},
		'r2qkb1r/pp3ppp/2n1pn2/3pNb2/3P1B2/2N1P3/PP3PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'D14e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Ne5

			variation => N__('Slav: Exchange, 8. Ne5'),
			moves => {
			},
		},
		'r2qkb1r/pp3ppp/2n2n2/2pp4/3P2b1/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 8' => {
			eco => 'D33n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3 Nc6 6. g3 Nf6 7. Bg2 Bg4

			variation => N__('QGD Tarrasch: Wagner Variation'),
			moves => {
			},
		},
		'r2qkb1r/pp3ppp/2npbn2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B59b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nb3 Be6

			variation => N__('Sicilian: Boleslavsky, 7. Nb3 Be6'),
			moves => {
			},
		},
		'r2qkb1r/pp3ppp/n1p1pn2/8/P1BPP1b1/2N2N2/1P3PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'D16k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Na6 6. e4 Bg4 7. Bxc4 e6

			variation => N__('Slav: Smyslov, 6. e4 Bg4 7.Bxc4 e6'),
			moves => {
			},
		},
		'r2qkb1r/ppp1p1pp/1nn1p3/2P5/3P2b1/5N2/PP4PP/RNBQKB1R b KQkq - 0 9' => {
			eco => 'B03m',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 Nc6 7. Nf3 Bg4 8. e6 fxe6 9. c5

			variation => N__('Alekhine: Four Pawns Attack, Ilyin-Zhenevsky Variation'),
			moves => {
			},
		},
		'r2qkb1r/ppp1p1pp/1nn1p3/8/2PP2b1/5N2/PP4PP/RNBQKB1R w KQkq - 0 9' => {
			eco => 'B03m',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Nc6'),
			moves => {
				'c4c5' => 'r2qkb1r/ppp1p1pp/1nn1p3/2P5/3P2b1/5N2/PP4PP/RNBQKB1R b KQkq - 0 9',
			},
		},
		'r2qkb1r/ppp1pppp/1nn1P3/8/2PP2b1/5N2/PP4PP/RNBQKB1R b KQkq - 0 8' => {
			eco => 'B03m',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Nc6'),
			moves => {
				'f7e6' => 'r2qkb1r/ppp1p1pp/1nn1p3/8/2PP2b1/5N2/PP4PP/RNBQKB1R w KQkq - 0 9',
			},
		},
		'r2qkb1r/ppp1pppp/1nn5/4P3/2PP2b1/5N2/PP4PP/RNBQKB1R w KQkq - 3 8' => {
			eco => 'B03m',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Nc6'),
			moves => {
				'e5e6' => 'r2qkb1r/ppp1pppp/1nn1P3/8/2PP2b1/5N2/PP4PP/RNBQKB1R b KQkq - 0 8',
			},
		},
		'r2qkb1r/ppp1pppp/2n2n2/3p4/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 5 5' => {
			eco => 'D07r',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nc3 Nf6 4. Nf3 Bg4

			variation => N__('QGD: Chigorin, 3. Nc3 Nf6 4. Nf3 Bg4'),
			moves => {
			},
		},
		'r2qkb1r/ppp1pppp/2np1n2/1B6/3PP1b1/2N2N2/PPP2PPP/R1BQK2R b KQkq - 4 5' => {
			eco => 'B00r',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. Bb5

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. Bb5'),
			moves => {
			},
		},
		'r2qkb1r/ppp1pppp/2np1n2/3P4/4P1b1/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B00r',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. d5

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. d5'),
			moves => {
			},
		},
		'r2qkb1r/ppp1pppp/2np1n2/8/3PP1b1/2N1BN2/PPP2PPP/R2QKB1R b KQkq - 4 5' => {
			eco => 'B00s',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. Be3

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. Be3'),
			moves => {
			},
		},
		'r2qkb1r/ppp1pppp/2np1n2/8/3PP1b1/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 4 5' => {
			eco => 'B00r',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. Be2

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4 5. Be2'),
			moves => {
			},
		},
		'r2qkb1r/ppp1pppp/2np1n2/8/3PP1b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 5' => {
			eco => 'B00r',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Nf6 4. Nc3 Bg4'),
			moves => {
				'c1e3' => 'r2qkb1r/ppp1pppp/2np1n2/8/3PP1b1/2N1BN2/PPP2PPP/R2QKB1R b KQkq - 4 5',
				'd4d5' => 'r2qkb1r/ppp1pppp/2np1n2/3P4/4P1b1/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5',
				'f1b5' => 'r2qkb1r/ppp1pppp/2np1n2/1B6/3PP1b1/2N2N2/PPP2PPP/R1BQK2R b KQkq - 4 5',
				'f1e2' => 'r2qkb1r/ppp1pppp/2np1n2/8/3PP1b1/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 4 5',
			},
		},
		'r2qkb1r/ppp1pppp/2np4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 4 6' => {
			eco => 'B05g',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 Nc6

			variation => N__('Alekhine: Modern, 5. Be2 Nc6'),
			moves => {
			},
		},
		'r2qkb1r/ppp2ppp/2n1b3/3np3/8/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 2 7' => {
			eco => 'A29',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. Nf3 Nf6 4. g3 d5 5. cxd5 Nxd5 6. Bg2 Be6

			variation => N__('English: Four Knights, 4. g3 d5 5.cxd5 Nxd5 6. Bg2 Be6'),
			moves => {
			},
		},
		'r2qkb1r/ppp2ppp/2nn4/5b2/5Q2/2N2N2/PPP2PPP/R1B1KB1R w KQkq - 4 9' => {
			eco => 'C43e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4 5. Qxd4 d5 6. exd6 Nxd6 7. Nc3 Nc6 8. Qf4 Bf5

			variation => N__('Russian Game: Modern Attack, Pillsbury Variation'),
			moves => {
			},
		},
		'r2qkb1r/pppb1ppp/2B5/3pP3/3Nn3/8/PPP2PPP/RNBQK2R b KQkq - 0 8' => {
			eco => 'C55q',
                        # TRANSLATORS:

			variation => N__('Two Knights: Modern, Main Line'),
			moves => {
				'b7c6' => 'r2qkb1r/p1pb1ppp/2p5/3pP3/3Nn3/8/PPP2PPP/RNBQK2R w KQkq - 0 9',
			},
		},
		'r2qkb1r/pppb1ppp/2Bp1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 0 6' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 Bd7 5. Nc3 Nf6 6. Bxc6

			variation => N__('Spanish: Old Steinitz, Nimzowitsch Attack'),
			moves => {
			},
		},
		'r2qkb1r/pppb1ppp/2n5/1B1pP3/3Nn3/8/PPP2PPP/RNBQK2R w KQkq - 1 8' => {
			eco => 'C55q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. e5 d5 6. Bb5 Ne4 7. Nxd4 Bd7

			variation => N__('Two Knights: Modern, Main Line'),
			moves => {
				'b5c6' => 'r2qkb1r/pppb1ppp/2B5/3pP3/3Nn3/8/PPP2PPP/RNBQK2R b KQkq - 0 8',
			},
		},
		'r2qkb1r/pppb1ppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 2 6' => {
			eco => 'C66',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed Berlin, 5. d4 Bd7'),
			moves => {
				'e5d4' => 'r2qkb1r/pppb1ppp/2np1n2/1B6/3pP3/2N2N2/PPP2PPP/R1BQ1RK1 w kq - 0 7',
				'f8e7' => 'r2qk2r/pppbbppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQ1RK1 w kq - 3 7',
			},
		},
		'r2qkb1r/pppb1ppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 3 6' => {
			eco => 'C62',
                        # TRANSLATORS:

			variation => N__('Spanish: Old Steinitz, 4. d4 Bd7 5. Nc3'),
			moves => {
				'b5c6' => 'r2qkb1r/pppb1ppp/2Bp1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 0 6',
			},
		},
		'r2qkb1r/pppb1ppp/2np1n2/1B2p3/3PP3/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Bd7

			variation => N__('Spanish: Closed Berlin, 5. d4 Bd7'),
			moves => {
				'b1c3' => 'r2qkb1r/pppb1ppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQ1RK1 b kq - 2 6',
			},
		},
		'r2qkb1r/pppb1ppp/2np1n2/1B6/3pP3/2N2N2/PPP2PPP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'C66',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 Nf6 4. O-O d6 5. d4 Bd7 6. Nc3 exd4

			variation => N__('Spanish: Closed Berlin, Wolf Variation'),
			moves => {
			},
		},
		'r2qkb1r/pppb1ppp/5n2/nB1Pp1N1/8/8/PPPP1PPP/RNBQK2R w KQkq - 3 7' => {
			eco => 'C58d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Na5 6. Bb5+ Bd7

			variation => N__('Two Knights: Morphy, Polerio, 6... Bd7'),
			moves => {
			},
		},
		'r2qkb1r/pppb1ppp/8/3p4/3Pn3/3B4/PPP2PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C43r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7 6. Nxd7 Bxd7 7. O-O

			variation => N__('Russian Game: Modern Attack, 3...Nxe4, Main Line'),
			moves => {
				'd8h4' => 'r3kb1r/pppb1ppp/8/3p4/3Pn2q/3B4/PPP2PPP/RNBQ1RK1 w kq - 2 8',
				'f8d6' => 'r2qk2r/pppb1ppp/3b4/3p4/3Pn3/3B4/PPP2PPP/RNBQ1RK1 w kq - 2 8',
			},
		},
		'r2qkb1r/pppb1ppp/8/3p4/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C43q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Nd7 6.Nxd7'),
			moves => {
				'e1g1' => 'r2qkb1r/pppb1ppp/8/3p4/3Pn3/3B4/PPP2PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r2qkb1r/pppn1ppp/2b1pn2/8/3P4/3B1NN1/PPP2PPP/R1BQ1RK1 b kq - 8 8' => {
			eco => 'C10x',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7 5. Nf3 Bc6 6. Bd3 Nd7 7. O-O Ngf6 8. Ng3

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3 Bc6 6. Bd3 Nd7 7.O-O Ngf6 8. Ng3'),
			moves => {
			},
		},
		'r2qkb1r/pppn1ppp/2b1pn2/8/3PN3/3B1N2/PPP2PPP/R1BQ1RK1 w kq - 7 8' => {
			eco => 'C10w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7 5. Nf3 Bc6 6. Bd3 Nd7 7. O-O Ngf6

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3 Bc6 6. Bd3 Nd7 7.O-O Ngf6'),
			moves => {
				'e4g3' => 'r2qkb1r/pppn1ppp/2b1pn2/8/3P4/3B1NN1/PPP2PPP/R1BQ1RK1 b kq - 8 8',
			},
		},
		'r2qkb1r/pppn1ppp/4pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQ1RK1 b kq - 4 8' => {
			eco => 'D25q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 Nbd7 8. O-O

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 Nbd7 8.O-O'),
			moves => {
				'f8d6' => 'r2qk2r/pppn1ppp/3bpn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQ1RK1 w kq - 5 9',
			},
		},
		'r2qkb1r/pppn1ppp/4pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQK2R w KQkq - 3 8' => {
			eco => 'D25p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 Nbd7

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 Nbd7'),
			moves => {
				'e1g1' => 'r2qkb1r/pppn1ppp/4pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQ1RK1 b kq - 4 8',
			},
		},
		'r2qkb1r/pppnpppp/5n2/1B1P1b2/2PP4/5P2/PP4PP/RNBQK1NR b KQkq c3 0 6' => {
			eco => 'B01v',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. f3 Bf5 5. Bb5+ Nbd7 6. c4

			variation => N__('Scandinavian: Portuguese, 4. f3 Bf5 5. Bb5+ Nbd7 6. c4'),
			moves => {
			},
		},
		'r2qkb1r/pppnpppp/5n2/1B1P1b2/3P4/5P2/PPP3PP/RNBQK1NR w KQkq - 3 6' => {
			eco => 'B01v',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. f3 Bf5 5. Bb5+ Nbd7

			variation => N__('Scandinavian: Portuguese, 4. f3 Bf5 5. Bb5+ Nbd7'),
			moves => {
				'c2c4' => 'r2qkb1r/pppnpppp/5n2/1B1P1b2/2PP4/5P2/PP4PP/RNBQK1NR b KQkq c3 0 6',
			},
		},
		'r2qkbnr/1p1bpppp/p1np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 6' => {
			eco => 'B51p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1 a6

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1 a6'),
			moves => {
			},
		},
		'r2qkbnr/1pp2pp1/p1p5/4p2p/4P1b1/3P1N1P/PPP2PP1/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C68o',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Alapin Gambit'),
			moves => {
				'd8f6' => 'r3kbnr/1pp2pp1/p1p2q2/4p2p/4P1b1/3P1N1P/PPP2PP1/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r2qkbnr/1pp2pp1/p1p5/4p2p/4P1b1/5N1P/PPPP1PP1/RNBQ1RK1 w kq h6 0 7' => {
			eco => 'C68o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Bg4 6. h3 h5

			variation => N__('Spanish: Exchange, Alapin Gambit'),
			moves => {
				'd2d3' => 'r2qkbnr/1pp2pp1/p1p5/4p2p/4P1b1/3P1N1P/PPP2PP1/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r2qkbnr/1pp2ppp/p1np4/4p3/B3P1b1/5N1P/PPPP1PP1/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C72',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. O-O Bg4 6. h3

			variation => N__('Spanish: Modern Steinitz, 5.O-O Bg4 6. h3'),
			moves => {
			},
		},
		'r2qkbnr/1pp2ppp/p1np4/4p3/B3P1b1/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C72',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. O-O Bg4

			variation => N__('Spanish: Modern Steinitz, 5.O-O Bg4'),
			moves => {
				'h2h3' => 'r2qkbnr/1pp2ppp/p1np4/4p3/B3P1b1/5N1P/PPPP1PP1/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r2qkbnr/1pp2ppp/p1p5/4p3/4P1b1/5N1P/PPPP1PP1/RNBQ1RK1 b kq - 0 6' => {
			eco => 'C68n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Bg4 6. h3

			variation => N__('Spanish: Exchange, 5.O-O Bg4 6. h3'),
			moves => {
				'h7h5' => 'r2qkbnr/1pp2pp1/p1p5/4p2p/4P1b1/5N1P/PPPP1PP1/RNBQ1RK1 w kq h6 0 7',
			},
		},
		'r2qkbnr/1pp2ppp/p1p5/4p3/4P1b1/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C68n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Bg4

			variation => N__('Spanish: Exchange, 5.O-O Bg4'),
			moves => {
				'h2h3' => 'r2qkbnr/1pp2ppp/p1p5/4p3/4P1b1/5N1P/PPPP1PP1/RNBQ1RK1 b kq - 0 6',
			},
		},
		'r2qkbnr/1pp3pp/p1np4/4p3/B7/2Pb1N2/PP1P1PPP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'C74',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev Variation'),
			moves => {
				'f1e1' => 'r2qkbnr/1pp3pp/p1np4/4p3/B7/2Pb1N2/PP1P1PPP/RNBQR1K1 b kq - 3 8',
			},
		},
		'r2qkbnr/1pp3pp/p1np4/4p3/B7/2Pb1N2/PP1P1PPP/RNBQR1K1 b kq - 3 8' => {
			eco => 'C74',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev Variation'),
			moves => {
				'f8e7' => 'r2qk1nr/1pp1b1pp/p1np4/4p3/B7/2Pb1N2/PP1P1PPP/RNBQR1K1 w kq - 4 9',
			},
		},
		'r2qkbnr/1pp3pp/p1np4/4pb2/B7/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C74',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 f5 6. exf5 Bxf5 7. O-O

			variation => N__('Spanish: Modern Steinitz, Siesta, Kopayev Variation'),
			moves => {
				'f5d3' => 'r2qkbnr/1pp3pp/p1np4/4p3/B7/2Pb1N2/PP1P1PPP/RNBQ1RK1 w kq - 2 8',
			},
		},
		'r2qkbnr/1pp3pp/p1np4/4pb2/B7/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C74',
                        # TRANSLATORS:

			variation => N__('Spanish: Modern Steinitz, Siesta, 6.exf5'),
			moves => {
				'e1g1' => 'r2qkbnr/1pp3pp/p1np4/4pb2/B7/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r2qkbnr/1pp3pp/p1p2p2/4P3/4P1b1/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C69e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 Bg4 7. dxe5

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 7.dxe5'),
			moves => {
				'd8d1' => 'r3kbnr/1pp3pp/p1p2p2/4P3/4P1b1/5N2/PPP2PPP/RNBq1RK1 w kq - 0 8',
			},
		},
		'r2qkbnr/1pp3pp/p1p2p2/4p3/3PP1b1/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'C69c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 Bg4 7. c3

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 7. c3'),
			moves => {
				'f8d6' => 'r2qk1nr/1pp3pp/p1pb1p2/4p3/3PP1b1/2P2N2/PP3PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'r2qkbnr/1pp3pp/p1p2p2/4p3/3PP1b1/5N2/PPP2PPP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'C69b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 Bg4

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4'),
			moves => {
				'c2c3' => 'r2qkbnr/1pp3pp/p1p2p2/4p3/3PP1b1/2P2N2/PP3PPP/RNBQ1RK1 b kq - 0 7',
				'd4e5' => 'r2qkbnr/1pp3pp/p1p2p2/4P3/4P1b1/5N2/PPP2PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'r2qkbnr/1ppb1p1p/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6 7. O-O

			variation => N__('Spanish: Modern Steinitz, Bronstein, 7.O-O'),
			moves => {
				'f8g7' => 'r2qk1nr/1ppb1pbp/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQ1RK1 w kq - 2 8',
			},
		},
		'r2qkbnr/1ppb1p1p/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C76',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4 g6

			variation => N__('Spanish: Modern Steinitz, Bronstein Variation'),
			moves => {
				'e1g1' => 'r2qkbnr/1ppb1p1p/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'r2qkbnr/1ppb1ppp/p1np4/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. d4

			variation => N__('Spanish: Modern Steinitz, 5. c3 Bd7 6. d4'),
			moves => {
				'g7g6' => 'r2qkbnr/1ppb1p1p/p1np2p1/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 0 7',
				'g8e7' => 'r2qkb1r/1ppbnppp/p1np4/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7',
				'g8f6' => 'r2qkb1r/1ppb1ppp/p1np1n2/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'r2qkbnr/1ppb1ppp/p1np4/4p3/B2PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6' => {
			eco => 'C72',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. O-O Bd7 6. d4

			variation => N__('Spanish: Modern Steinitz, 5.O-O Bd7 6. d4'),
			moves => {
			},
		},
		'r2qkbnr/1ppb1ppp/p1np4/4p3/B3P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7 6. O-O

			variation => N__('Spanish: Modern Steinitz, 5. c3 Bd7 6.O-O'),
			moves => {
			},
		},
		'r2qkbnr/1ppb1ppp/p1np4/4p3/B3P3/2P2N2/PP1P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C75',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. c3 Bd7

			variation => N__('Spanish: Modern Steinitz, 5. c3 Bd7'),
			moves => {
				'd2d4' => 'r2qkbnr/1ppb1ppp/p1np4/4p3/B2PP3/2P2N2/PP3PPP/RNBQK2R b KQkq d3 0 6',
				'e1g1' => 'r2qkbnr/1ppb1ppp/p1np4/4p3/B3P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'r2qkbnr/1ppb1ppp/p1np4/4p3/B3P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C72',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 d6 5. O-O Bd7

			variation => N__('Spanish: Modern Steinitz, 5.O-O Bd7'),
			moves => {
				'd2d4' => 'r2qkbnr/1ppb1ppp/p1np4/4p3/B2PP3/5N2/PPP2PPP/RNBQ1RK1 b kq d3 0 6',
			},
		},
		'r2qkbnr/pp1b1ppp/2n1p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6' => {
			eco => 'C02p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Bd7 6. Be2

			variation => N__('French: Advance, Euwe, 6. Be2'),
			moves => {
				'g8e7' => 'r2qkb1r/pp1bnppp/2n1p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R w KQkq - 5 7',
			},
		},
		'r2qkbnr/pp1b1ppp/2n1p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C02n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Bd7

			variation => N__('French: Advance, Euwe'),
			moves => {
				'a2a3' => 'r2qkbnr/pp1b1ppp/2n1p3/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R b KQkq - 0 6',
				'f1e2' => 'r2qkbnr/pp1b1ppp/2n1p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6',
			},
		},
		'r2qkbnr/pp1b1ppp/2n1p3/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C02o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Bd7 6. a3

			variation => N__('French: Advance, Euwe, 6. a3'),
			moves => {
			},
		},
		'r2qkbnr/pp1b1ppp/2n5/3Q2B1/8/2N5/PP2PPPP/R3KBNR b KQkq - 2 8' => {
			eco => 'D32g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qa4+ Bd7 6. Qxd4 exd5 7. Qxd5 Nc6 8. Bg5

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Bg5'),
			moves => {
			},
		},
		'r2qkbnr/pp1b1ppp/2n5/3Q4/8/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 2 8' => {
			eco => 'D32h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qa4+ Bd7 6. Qxd4 exd5 7. Qxd5 Nc6 8. Nf3

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'g8f6' => 'r2qkb1r/pp1b1ppp/2n2n2/3Q4/8/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 3 9',
			},
		},
		'r2qkbnr/pp1b1ppp/2n5/3Q4/8/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 8' => {
			eco => 'D32f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qa4+ Bd7 6. Qxd4 exd5 7. Qxd5 Nc6

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6'),
			moves => {
				'c1g5' => 'r2qkbnr/pp1b1ppp/2n5/3Q2B1/8/2N5/PP2PPPP/R3KBNR b KQkq - 2 8',
				'g1f3' => 'r2qkbnr/pp1b1ppp/2n5/3Q4/8/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 2 8',
			},
		},
		'r2qkbnr/pp1bpppp/2Bp4/8/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 6' => {
			eco => 'B53n',
                        # TRANSLATORS:

			variation => N__('Sicilian, Chekhover, 4... Nc6 5. Bb5 Bd7'),
			moves => {
				'd7c6' => 'r2qkbnr/pp2pppp/2bp4/8/3QP3/5N2/PPP2PPP/RNB1K2R w KQkq - 0 7',
			},
		},
		'r2qkbnr/pp1bpppp/2np4/1B6/3QP3/5N2/PPP2PPP/RNB1K2R w KQkq - 3 6' => {
			eco => 'B53n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7

			variation => N__('Sicilian, Chekhover, 4... Nc6 5. Bb5 Bd7'),
			moves => {
				'b5c6' => 'r2qkbnr/pp1bpppp/2Bp4/8/3QP3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 6',
			},
		},
		'r2qkbnr/pp1bpppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 4 5' => {
			eco => 'B51n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7'),
			moves => {
				'f1e1' => 'r2qkbnr/pp1bpppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 5 5',
			},
		},
		'r2qkbnr/pp1bpppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 5 5' => {
			eco => 'B51o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Nc6 4. O-O Bd7 5. Re1

			variation => N__('Sicilian: 3. Bb5+ Nc6 4.O-O Bd7 5. Re1'),
			moves => {
				'a7a6' => 'r2qkbnr/1p1bpppp/p1np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 0 6',
				'g8f6' => 'r2qkb1r/pp1bpppp/2np1n2/1Bp5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 6 6',
			},
		},
		'r2qkbnr/pp1n1pp1/2p1p2p/7P/3P1B2/3Q1NN1/PPP2PP1/R3K2R b KQkq - 1 11' => {
			eco => 'B19l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 e6 11. Bf4

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 e6 11. Bf4'),
			moves => {
			},
		},
		'r2qkbnr/pp1n1pp1/2p1p2p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R w KQkq - 0 11' => {
			eco => 'B19k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 e6

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 e6'),
			moves => {
				'c1f4' => 'r2qkbnr/pp1n1pp1/2p1p2p/7P/3P1B2/3Q1NN1/PPP2PP1/R3K2R b KQkq - 1 11',
			},
		},
		'r2qkbnr/pp1n1ppp/2p1p1b1/8/3P4/3B1NN1/PPP2PPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B18o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Nf3 Nd7 7. Bd3 e6

			variation => N__('Caro-Kann: Classical, 6. Nf3 Nd7 7. Bd3 e6'),
			moves => {
			},
		},
		'r2qkbnr/pp1n1ppp/2p1p3/3p4/4P3/2NP1Q1P/PPP2PP1/R1B1KB1R w KQkq - 1 7' => {
			eco => 'B11k',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. d3 Nd7

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. d3 Nd7'),
			moves => {
			},
		},
		'r2qkbnr/pp1n1ppp/2p1p3/3pPb2/3P4/5N2/PPP1BPPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B12r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 Nd7

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 Nd7'),
			moves => {
			},
		},
		'r2qkbnr/pp1n1ppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1Q1RK1 b kq - 3 7' => {
			eco => 'B12u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. Be3 Nd7 7. O-O

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6. Be3'),
			moves => {
			},
		},
		'r2qkbnr/pp1n1ppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1QK2R w KQkq - 2 7' => {
			eco => 'B12u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. Be3 Nd7

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6. Be3'),
			moves => {
				'e1g1' => 'r2qkbnr/pp1n1ppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1Q1RK1 b kq - 3 7',
			},
		},
		'r2qkbnr/pp1n1ppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'B12t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. O-O Nd7

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6.O-O'),
			moves => {
			},
		},
		'r2qkbnr/pp1n1ppp/8/3pP3/8/8/PPP2PPP/RNBQK1NR b KQkq - 0 7' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'd7e5' => 'r2qkbnr/pp3ppp/8/3pn3/8/8/PPP2PPP/RNBQK1NR w KQkq - 0 8',
			},
		},
		'r2qkbnr/pp1n1ppp/8/3pp3/3P4/8/PPP2PPP/RNBQK1NR w KQkq - 0 7' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'd4e5' => 'r2qkbnr/pp1n1ppp/8/3pP3/8/8/PPP2PPP/RNBQK1NR b KQkq - 0 7',
			},
		},
		'r2qkbnr/pp1nppp1/2p3bp/7P/3P4/5NN1/PPP2PP1/R1BQKB1R b KQkq - 0 8' => {
			eco => 'B19h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5

			variation => N__('Caro-Kann: Classical, Spassky Variation'),
			moves => {
				'g6h7' => 'r2qkbnr/pp1npppb/2p4p/7P/3P4/5NN1/PPP2PP1/R1BQKB1R w KQkq - 1 9',
			},
		},
		'r2qkbnr/pp1nppp1/2p3bp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B19g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nd7'),
			moves => {
				'h4h5' => 'r2qkbnr/pp1nppp1/2p3bp/7P/3P4/5NN1/PPP2PP1/R1BQKB1R b KQkq - 0 8',
			},
		},
		'r2qkbnr/pp1nppp1/2p4p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R b KQkq - 0 10' => {
			eco => 'B19i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3'),
			moves => {
				'd8c7' => 'r3kbnr/ppqnppp1/2p4p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R w KQkq - 1 11',
				'e7e6' => 'r2qkbnr/pp1n1pp1/2p1p2p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R w KQkq - 0 11',
				'g8f6' => 'r2qkb1r/pp1nppp1/2p2n1p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R w KQkq - 1 11',
			},
		},
		'r2qkbnr/pp1nppp1/2p4p/7P/3P4/3b1NN1/PPP2PP1/R1BQK2R w KQkq - 0 10' => {
			eco => 'B19h',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Classical, Spassky Variation'),
			moves => {
				'd1d3' => 'r2qkbnr/pp1nppp1/2p4p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R b KQkq - 0 10',
			},
		},
		'r2qkbnr/pp1npppb/2p4p/7P/3P4/3B1NN1/PPP2PP1/R1BQK2R b KQkq - 2 9' => {
			eco => 'B19h',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Classical, Spassky Variation'),
			moves => {
				'h7d3' => 'r2qkbnr/pp1nppp1/2p4p/7P/3P4/3b1NN1/PPP2PP1/R1BQK2R w KQkq - 0 10',
			},
		},
		'r2qkbnr/pp1npppb/2p4p/7P/3P4/5NN1/PPP2PP1/R1BQKB1R w KQkq - 1 9' => {
			eco => 'B19h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7

			variation => N__('Caro-Kann: Classical, Spassky Variation'),
			moves => {
				'f1d3' => 'r2qkbnr/pp1npppb/2p4p/7P/3P4/3B1NN1/PPP2PP1/R1BQK2R b KQkq - 2 9',
			},
		},
		'r2qkbnr/pp1npppp/2p3b1/8/3P4/3B1NN1/PPP2PPP/R1BQK2R b KQkq - 6 7' => {
			eco => 'B18n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Nf3 Nd7 7. Bd3

			variation => N__('Caro-Kann: Classical, 6. Nf3 Nd7 7. Bd3'),
			moves => {
				'e7e6' => 'r2qkbnr/pp1n1ppp/2p1p1b1/8/3P4/3B1NN1/PPP2PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'r2qkbnr/pp1npppp/2p3b1/8/3P4/5NN1/PPP2PPP/R1BQKB1R w KQkq - 5 7' => {
			eco => 'B18m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Nf3 Nd7

			variation => N__('Caro-Kann: Classical, 6. Nf3 Nd7'),
			moves => {
				'f1d3' => 'r2qkbnr/pp1npppp/2p3b1/8/3P4/3B1NN1/PPP2PPP/R1BQK2R b KQkq - 6 7',
			},
		},
		'r2qkbnr/pp1npppp/2p5/3p4/6b1/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c6 3. Bg2 Bg4 4. O-O Nd7 5. d3

			variation => N__('Reti: KIA, Yugoslav'),
			moves => {
			},
		},
		'r2qkbnr/pp1npppp/2p5/3p4/6b1/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 4 5' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c6 3. Bg2 Bg4 4. O-O Nd7

			variation => N__('Reti: KIA, Yugoslav'),
			moves => {
				'd2d3' => 'r2qkbnr/pp1npppp/2p5/3p4/6b1/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'r2qkbnr/pp1npppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'B52c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Nxd7 5. O-O

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Nxd7 5.O-O'),
			moves => {
				'g8f6' => 'r2qkb1r/pp1npppp/3p1n2/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'r2qkbnr/pp1npppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'B52b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Nxd7

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Nxd7'),
			moves => {
				'e1g1' => 'r2qkbnr/pp1npppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'r2qkbnr/pp2pppp/2bp4/8/2PQP3/5N2/PP3PPP/RNB1K2R b KQkq c3 0 7' => {
			eco => 'B53p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. c4

			variation => N__('Sicilian, Chekhover, 7. c4'),
			moves => {
				'g8f6' => 'r2qkb1r/pp2pppp/2bp1n2/8/2PQP3/5N2/PP3PPP/RNB1K2R w KQkq - 1 8',
			},
		},
		'r2qkbnr/pp2pppp/2bp4/8/3QP3/2N2N2/PPP2PPP/R1B1K2R b KQkq - 1 7' => {
			eco => 'B53r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6 7. Nc3

			variation => N__('Sicilian, Chekhover, 7. Nc3'),
			moves => {
				'g8f6' => 'r2qkb1r/pp2pppp/2bp1n2/8/3QP3/2N2N2/PPP2PPP/R1B1K2R w KQkq - 2 8',
			},
		},
		'r2qkbnr/pp2pppp/2bp4/8/3QP3/5N2/PPP2PPP/RNB1K2R w KQkq - 0 7' => {
			eco => 'B53o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nc6 5. Bb5 Bd7 6. Bxc6 Bxc6

			variation => N__('Sicilian, Chekhover, 4... Nc6 5. Bb5 Bd7 6.Bxc6 Bxc6'),
			moves => {
				'b1c3' => 'r2qkbnr/pp2pppp/2bp4/8/3QP3/2N2N2/PPP2PPP/R1B1K2R b KQkq - 1 7',
				'c2c4' => 'r2qkbnr/pp2pppp/2bp4/8/2PQP3/5N2/PP3PPP/RNB1K2R b KQkq c3 0 7',
			},
		},
		'r2qkbnr/pp2pppp/2n3b1/1B1p4/6P1/2N2N1P/PPPP1P2/R1BQK2R w KQkq - 1 8' => {
			eco => 'B11c',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bh5 5. exd5 cxd5 6. Bb5+ Nc6 7. g4 Bg6

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5, 7. g4 Bg6'),
			moves => {
			},
		},
		'r2qkbnr/pp2pppp/2n5/1B1p3b/6P1/2N2N1P/PPPP1P2/R1BQK2R b KQkq g3 0 7' => {
			eco => 'B11b',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5'),
			moves => {
				'h5g6' => 'r2qkbnr/pp2pppp/2n3b1/1B1p4/6P1/2N2N1P/PPPP1P2/R1BQK2R w KQkq - 1 8',
			},
		},
		'r2qkbnr/pp2pppp/2n5/1B1p3b/8/2N2N1P/PPPP1PP1/R1BQK2R w KQkq - 2 7' => {
			eco => 'B11b',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5'),
			moves => {
				'g2g4' => 'r2qkbnr/pp2pppp/2n5/1B1p3b/6P1/2N2N1P/PPPP1P2/R1BQK2R b KQkq g3 0 7',
			},
		},
		'r2qkbnr/pp3ppp/8/3pn3/8/8/PPP1NPPP/RNBQK2R b KQkq - 1 8' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 c6 3. d4 d5 4. exd5 cxd5 5. Bb5+ Bd7 6. Bxd7+ Nxd7 7. dxe5 Nxe5 8. Ne2

			variation => N__('Bishop\'s Opening: Lisitsin Variation'),
			moves => {
			},
		},
		'r2qkbnr/pp3ppp/8/3pn3/8/8/PPP2PPP/RNBQK1NR w KQkq - 0 8' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'g1e2' => 'r2qkbnr/pp3ppp/8/3pn3/8/8/PPP1NPPP/RNBQK2R b KQkq - 1 8',
			},
		},
		'r2qkbnr/ppp1pppp/2P5/8/3P4/5b2/PP2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D07l',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 Bg4 4. cxd5 Bxf3 5. dxc6

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4 4.cxd5 Bxf3 5.dxc6'),
			moves => {
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3P4/3P2b1/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D07k',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 Bg4 4. cxd5

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4 4.cxd5'),
			moves => {
				'g4f3' => 'r2qkbnr/ppp1pppp/2n5/3P4/3P4/5b2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3P4/3P4/5P2/PP2PP1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D07m',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 Bg4 4. cxd5 Bxf3 5. gxf3

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4 4.cxd5 Bxf3 5.gxf3'),
			moves => {
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3P4/3P4/5b2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D07k',
                        # TRANSLATORS:

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4 4.cxd5'),
			moves => {
				'd5c6' => 'r2qkbnr/ppp1pppp/2P5/8/3P4/5b2/PP2PPPP/RNBQKB1R b KQkq - 0 5',
				'g2f3' => 'r2qkbnr/ppp1pppp/2n5/3P4/3P4/5P2/PP2PP1P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3p4/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 4 4' => {
			eco => 'D07j',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 Bg4 4. Nc3

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4 4. Nc3'),
			moves => {
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3p4/2PP2b1/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D07i',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 Bg4 4. e3

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4 4. e3'),
			moves => {
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3p4/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 4' => {
			eco => 'D07h',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nc6 3. Nf3 Bg4

			variation => N__('QGD: Chigorin, 3. Nf3 Bg4'),
			moves => {
				'b1c3' => 'r2qkbnr/ppp1pppp/2n5/3p4/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 4 4',
				'c4d5' => 'r2qkbnr/ppp1pppp/2n5/3P4/3P2b1/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
				'e2e3' => 'r2qkbnr/ppp1pppp/2n5/3p4/2PP2b1/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3p4/3P2b1/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'D02g',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nc6 3. g3 Bg4

			variation => N__('Queen\'s Pawn: 2. Nf3 Nc6 3. g3 Bg4'),
			moves => {
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3pPb2/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B00x',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. e5 Bf5 4. c3

			variation => N__('Nimzowitsch Defence: 2. d4 d5 3. e5 Bf5 4. c3'),
			moves => {
			},
		},
		'r2qkbnr/ppp1pppp/2n5/3pPb2/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B00x',
                        # TRANSLATORS: 1. e4 Nc6 2. d4 d5 3. e5 Bf5

			variation => N__('Nimzowitsch Defence: 2. d4 d5 3. e5 Bf5'),
			moves => {
				'c2c3' => 'r2qkbnr/ppp1pppp/2n5/3pPb2/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'r2qkbnr/ppp1pppp/2np4/8/3PP1b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B00o',
                        # TRANSLATORS: 1. e4 Nc6 2. Nf3 d6 3. d4 Bg4

			variation => N__('Nimzowitsch Defence: 2. Nf3 d6 3. d4 Bg4'),
			moves => {
			},
		},
		'r2qkbnr/ppp2p1p/2P5/8/2BP1pb1/2N2p2/PPP3PP/R1BQ1RK1 b kq - 0 9' => {
			eco => 'C25r',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nc6 3. f4 exf4 4. Nf3 g5 5. d4 g4 6. Bc4 gxf3 7. O-O d5 8. exd5 Bg4 9. dxc6

			variation => N__('Vienna: Pierce Gambit, Rushmere Attack'),
			moves => {
			},
		},
		'r2qkbnr/ppp2p1p/2n5/3P4/2BP1pb1/2N2p2/PPP3PP/R1BQ1RK1 w kq - 1 9' => {
			eco => 'C25r',
                        # TRANSLATORS:

			variation => N__('Vienna: Pierce Gambit'),
			moves => {
				'd5c6' => 'r2qkbnr/ppp2p1p/2P5/8/2BP1pb1/2N2p2/PPP3PP/R1BQ1RK1 b kq - 0 9',
			},
		},
		'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. b3

			variation => N__('QGD: Albin, 5. g3 Be6 6. b3'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPP1P/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. Nbd2

			variation => N__('QGD: Albin, 5. g3 Be6 6.Nbd2'),
			moves => {
				'd8d7' => 'r3kbnr/pppq1ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPP1P/R1BQKB1R w KQkq - 3 7',
			},
		},
		'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6

			variation => N__('QGD: Albin, 5. g3 Be6'),
			moves => {
				'b1d2' => 'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPP1P/R1BQKB1R b KQkq - 2 6',
				'b2b3' => 'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 6',
				'f1g2' => 'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. Bg2

			variation => N__('QGD: Albin, 5. g3 Be6 6. Bg2'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2n1b3/4P3/2Pp4/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. a3 Be6

			variation => N__('QGD: Albin, 4. Nf3 Nc6 5. a3 Be6'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5N1P/PP1NPPP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, Alapin, 5... Bg4'),
			moves => {
				'g4f3' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp4/5b1P/PP1NPPP1/R1BQKB1R w KQkq - 0 7',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5N2/PP1NPPPP/R1BQKB1R w KQkq - 4 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. Nbd2 Bg4

			variation => N__('QGD: Albin, Alapin, 5... Bg4'),
			moves => {
				'h2h3' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5N1P/PP1NPPP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Bg4

			variation => N__('QGD: Albin, 5. g3 Bg4'),
			moves => {
				'f1g2' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'D09',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, 5. g3 Bg4'),
			moves => {
				'd8d7' => 'r3kbnr/pppq1ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 7',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/P4N2/1P1NPPPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. a3 Bg4 6. Nbd2

			variation => N__('QGD: Albin, 4. Nf3 Nc6 5. a3 Bg4 6.Nbd2'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. a3 Bg4

			variation => N__('QGD: Albin, 4. Nf3 Nc6 5. a3 Bg4'),
			moves => {
				'b1d2' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp2b1/P4N2/1P1NPPPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp4/5N1P/PP2PPP1/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, Alapin, 5... Bg4'),
			moves => {
				'f8b4' => 'r2qk1nr/ppp2ppp/2n5/4P3/1bPp4/5N1P/PP2PPP1/R1BQKB1R w KQkq - 1 8',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4P3/2Pp4/5b1P/PP1NPPP1/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, Alapin, 5... Bg4'),
			moves => {
				'd2f3' => 'r2qkbnr/ppp2ppp/2n5/4P3/2Pp4/5N1P/PP2PPP1/R1BQKB1R b KQkq - 0 7',
			},
		},
		'r2qkbnr/ppp2ppp/2n5/4Pb2/2Pp4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Bf5

			variation => N__('QGD: Albin, 5. g3 Bf5'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2np4/4P3/2P3b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'A41g',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 e5 4. dxe5 Nc6

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 e5 4.dxe5 Nc6 Gambit'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2np4/4p3/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'A41g',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 e5 4. Nc3 Nc6

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 e5 4. Nc3 Nc6'),
			moves => {
			},
		},
		'r2qkbnr/ppp2ppp/2npb3/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 d6 4. Bg2 Be6

			variation => N__('English: Closed, Tröger Defence'),
			moves => {
				'd2d3' => 'r2qkbnr/ppp2ppp/2npb3/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'r2qkbnr/ppp2ppp/2npb3/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A25',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nc6 3. g3 d6 4. Bg2 Be6 5. d3

			variation => N__('English: Closed, Tröger, 5. d3'),
			moves => {
			},
		},
		'r2qkbnr/pppb1ppp/2n5/3pp3/Q3P3/2P2N2/PP1P1PPP/RNB1KB1R w KQkq - 2 5' => {
			eco => 'C44l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 d5 4. Qa4 Bd7

			variation => N__('Ponziani: Caro Variation'),
			moves => {
			},
		},
		'r2qkbnr/pppb1ppp/2np4/1B2p3/2PPP3/5N2/PP3PPP/RNBQK2R b KQkq c3 0 5' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 Bd7 5. c4

			variation => N__('Spanish: Old Steinitz, Semi-Duras Variation'),
			moves => {
			},
		},
		'r2qkbnr/pppb1ppp/2np4/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 Bd7 5. Nc3

			variation => N__('Spanish: Old Steinitz, 4. d4 Bd7 5. Nc3'),
			moves => {
				'g8f6' => 'r2qkb1r/pppb1ppp/2np1n2/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 3 6',
			},
		},
		'r2qkbnr/pppb1ppp/2np4/1B2p3/3PP3/5N2/PPP2PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C62',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 d6 4. d4 Bd7

			variation => N__('Spanish: Old Steinitz, 4. d4 Bd7'),
			moves => {
				'b1c3' => 'r2qkbnr/pppb1ppp/2np4/1B2p3/3PP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 2 5',
				'c2c4' => 'r2qkbnr/pppb1ppp/2np4/1B2p3/2PPP3/5N2/PP3PPP/RNBQK2R b KQkq c3 0 5',
			},
		},
		'r2qkbnr/pppn1ppp/2b1p3/8/3PN3/3B1N2/PPP2PPP/R1BQ1RK1 b kq - 6 7' => {
			eco => 'C10v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7 5. Nf3 Bc6 6. Bd3 Nd7 7. O-O

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3 Bc6 6. Bd3 Nd7 7.O-O'),
			moves => {
				'g8f6' => 'r2qkb1r/pppn1ppp/2b1pn2/8/3PN3/3B1N2/PPP2PPP/R1BQ1RK1 w kq - 7 8',
			},
		},
		'r2qkbnr/pppn1ppp/2b1p3/8/3PN3/3B1N2/PPP2PPP/R1BQK2R w KQkq - 5 7' => {
			eco => 'C10u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7 5. Nf3 Bc6 6. Bd3 Nd7

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3 Bc6 6. Bd3 Nd7'),
			moves => {
				'e1g1' => 'r2qkbnr/pppn1ppp/2b1p3/8/3PN3/3B1N2/PPP2PPP/R1BQ1RK1 b kq - 6 7',
			},
		},
		'r2qkbnr/pppn1ppp/3p4/4p3/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq e6 0 5' => {
			eco => 'A41h',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 Nd7 4. Nc3 e5

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 Nd7 4. Nc3 e5'),
			moves => {
			},
		},
		'r2qkbnr/pppnpppp/3p4/8/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 2 4' => {
			eco => 'A41h',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 Nd7 4. Nc3

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 Nd7 4. Nc3'),
			moves => {
				'e7e5' => 'r2qkbnr/pppn1ppp/3p4/4p3/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq e6 0 5',
			},
		},
		'r2qkbnr/pppnpppp/3p4/8/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A41h',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 Nd7

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 Nd7'),
			moves => {
				'b1c3' => 'r2qkbnr/pppnpppp/3p4/8/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 2 4',
			},
		},
		'r2qkbnr/pppnpppp/3p4/8/3P2b1/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A41f',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. e3 Nd7

			variation => N__('Neo-Old Indian: Wade Defence, 3. e3 Nd7'),
			moves => {
			},
		},
		'r2qkbnr/pppnpppp/8/3p4/6b1/1P2PN2/PBPP1PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Bg4 3. Bb2 Nd7 4. e3

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Bg4'),
			moves => {
			},
		},
		'r2qkbnr/pppnpppp/8/3p4/6b1/1P3N2/PBPPPPPP/RN1QKB1R w KQkq - 3 4' => {
			eco => 'A06',
                        # TRANSLATORS:

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Bg4'),
			moves => {
				'e2e3' => 'r2qkbnr/pppnpppp/8/3p4/6b1/1P2PN2/PBPP1PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'r2qkbnr/pppnpppp/8/3p4/6b1/5NP1/PPPPPPBP/RNBQK2R w KQkq - 3 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Bg4 3. Bg2 Nd7

			variation => N__('Reti: KIA, Keres Variation'),
			moves => {
			},
		},
		'r2qr1k1/1bp1bppp/p1np1n2/1p2p1N1/3PP3/1BP4P/PP3PP1/RNBQR1K1 b - - 2 11' => {
			eco => 'C92l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Ng5

			variation => N__('Spanish: Closed, Zaitsev, 11. Ng5'),
			moves => {
			},
		},
		'r2qr1k1/1bp1bppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 2 11' => {
			eco => 'C92m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2

			variation => N__('Spanish: Closed, Zaitsev, 11.Nbd2'),
			moves => {
				'e7f8' => 'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 3 12',
			},
		},
		'r2qr1k1/1bp1bppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11' => {
			eco => 'C92k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8

			variation => N__('Spanish: Closed, Zaitsev, 10. d4 Re8'),
			moves => {
				'b1d2' => 'r2qr1k1/1bp1bppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 2 11',
				'f3g5' => 'r2qr1k1/1bp1bppp/p1np1n2/1p2p1N1/3PP3/1BP4P/PP3PP1/RNBQR1K1 b - - 2 11',
			},
		},
		'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/1P1PP3/2P2N1P/P1BN1PP1/R1BQR1K1 b - b3 0 13' => {
			eco => 'C95i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. b4

			variation => N__('Spanish: Closed, Breyer, Main Line, 13. b4'),
			moves => {
			},
		},
		'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 6 13' => {
			eco => 'C95k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1

			variation => N__('Spanish: Closed, Breyer, Main Line, 13. Nf1'),
			moves => {
				'e7f8' => 'r2qrbk1/1bpn1ppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 7 14',
			},
		},
		'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 5 13' => {
			eco => 'C95h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8

			variation => N__('Spanish: Closed, Breyer, Main Line'),
			moves => {
				'a2a4' => 'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/P2PP3/2P2N1P/1PBN1PP1/R1BQR1K1 b - a3 0 13',
				'b2b4' => 'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/1P1PP3/2P2N1P/P1BN1PP1/R1BQR1K1 b - b3 0 13',
				'd2f1' => 'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 6 13',
			},
		},
		'r2qr1k1/1bpnbppp/p2p1n2/1p2p3/P2PP3/2P2N1P/1PBN1PP1/R1BQR1K1 b - a3 0 13' => {
			eco => 'C95j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. a4

			variation => N__('Spanish: Closed, Breyer, Main Line, 13. a4'),
			moves => {
			},
		},
		'r2qr1k1/1ppbbppp/p1np1n2/4p3/B2PP3/2P2N2/PP1N1PPP/R1BQR1K1 w - - 3 10' => {
			eco => 'C87m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 d6 7. c3 O-O 8. d4 Bd7 9. Nbd2 Re8

			variation => N__('Spanish: Closed, Averbakh, 7. c3 O-O 8. d4 Bd7 9.Nbd2 Re8'),
			moves => {
			},
		},
		'r2qr1k1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2B1P/PP4P1/R1BQ1RK1 w - - 1 13' => {
			eco => 'A68m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4 10. O-O Nbd7 11. h3 Bxf3 12. Bxf3 Re8

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O Nbd7 11. h3 Bxf3 12.Bxf3 Re8'),
			moves => {
				'f1e1' => 'r2qr1k1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2B1P/PP4P1/R1BQR1K1 b - - 2 13',
			},
		},
		'r2qr1k1/pp1n1pbp/3p1np1/2pP4/4PP2/2N2B1P/PP4P1/R1BQR1K1 b - - 2 13' => {
			eco => 'A68n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4 10. O-O Nbd7 11. h3 Bxf3 12. Bxf3 Re8 13. Re1

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O Nbd7 11. h3 Bxf3 12.Bxf3 Re8 13. Re1'),
			moves => {
			},
		},
		'r2qr1k1/pp1n1ppp/2p1pnb1/8/Pb1PP3/2NB1N2/1P2QPPP/R1B2RK1 w - - 3 12' => {
			eco => 'D19m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 Re8

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Nbd7 10. e4 Bg6 11. Bd3 Re8'),
			moves => {
			},
		},
		'r2qr1k1/pp3pbp/n2p1np1/2pP4/4P1b1/2N2N2/PPQ1BPPP/R1B1R1K1 w - - 10 12' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Qc2 Na6 11. Re1 Bg4

			variation => N__('Benoni: Classical, Main Line, 10. Qc2 Na6 11. Re1 Bg4'),
			moves => {
			},
		},
		'r2qrbk1/1b1n1p1p/p2p1np1/1p1Pp3/P1p1P3/2P2NNP/1PB2PP1/R1BQR1K1 w - - 0 17' => {
			eco => 'C95o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3 g6 15. a4 c5 16. d5 c4

			variation => N__('Spanish: Closed, Breyer, Main Line, 15. a4 c5 16. d5 c4'),
			moves => {
			},
		},
		'r2qrbk1/1b1n1p1p/p2p1np1/1pp1p3/P2PP3/2P2NNP/1PB2PP1/R1BQR1K1 w - c6 0 16' => {
			eco => 'C95n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3 g6 15. a4 c5

			variation => N__('Spanish: Closed, Breyer, Main Line, 15. a4 c5'),
			moves => {
				'd4d5' => 'r2qrbk1/1b1n1p1p/p2p1np1/1ppPp3/P3P3/2P2NNP/1PB2PP1/R1BQR1K1 b - - 0 16',
			},
		},
		'r2qrbk1/1b1n1p1p/p2p1np1/1ppPp3/P3P3/2P2NNP/1PB2PP1/R1BQR1K1 b - - 0 16' => {
			eco => 'C95n',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Breyer, Main Line, 15. a4 c5'),
			moves => {
				'c5c4' => 'r2qrbk1/1b1n1p1p/p2p1np1/1p1Pp3/P1p1P3/2P2NNP/1PB2PP1/R1BQR1K1 w - - 0 17',
			},
		},
		'r2qrbk1/1b1n1ppp/p2p1n2/1pp1p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 w - c6 0 15' => {
			eco => 'C95k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3 c5

			variation => N__('Spanish: Closed, Breyer, Main Line, 14. Ng3 c5'),
			moves => {
			},
		},
		'r2qrbk1/1b3pp1/p2p1n1p/1pp5/Pn1PP3/5N1P/1P1N1PP1/RBBQR1K1 w - c6 0 16' => {
			eco => 'C92u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. a4 h6 13. Bc2 exd4 14. cxd4 Nb4 15. Bb1 c5

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6 13. Bc2 exd4 14.cxd4 Nb4 15. Bb1 c5'),
			moves => {
			},
		},
		'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/R1BQRNK1 w - - 5 13' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. Nf1 Bb7

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bb7'),
			moves => {
				'f1g3' => 'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 b - - 6 13',
			},
		},
		'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 b - - 6 13' => {
			eco => 'C93',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bb7'),
			moves => {
				'c6a5' => 'r2qrbk1/1bp2pp1/p2p1n1p/np2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 w - - 7 14',
			},
		},
		'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/P2PP3/1BP2N1P/1P1N1PP1/R1BQR1K1 w - - 0 13' => {
			eco => 'C92r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. a4 h6

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6'),
			moves => {
				'b3c2' => 'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/P2PP3/2P2N1P/1PBN1PP1/R1BQR1K1 b - - 1 13',
			},
		},
		'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/P2PP3/2P2N1P/1PBN1PP1/R1BQR1K1 b - - 1 13' => {
			eco => 'C92s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. a4 h6 13. Bc2

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6 13. Bc2'),
			moves => {
				'e5d4' => 'r2qrbk1/1bp2pp1/p1np1n1p/1p6/P2pP3/2P2N1P/1PBN1PP1/R1BQR1K1 w - - 0 14',
			},
		},
		'r2qrbk1/1bp2pp1/p1np1n1p/1p6/P2PP3/5N1P/1PBN1PP1/R1BQR1K1 b - - 0 14' => {
			eco => 'C92t',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6 13. Bc2 exd4'),
			moves => {
				'c6b4' => 'r2qrbk1/1bp2pp1/p2p1n1p/1p6/Pn1PP3/5N1P/1PBN1PP1/R1BQR1K1 w - - 1 15',
			},
		},
		'r2qrbk1/1bp2pp1/p1np1n1p/1p6/P2pP3/2P2N1P/1PBN1PP1/R1BQR1K1 w - - 0 14' => {
			eco => 'C92t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. a4 h6 13. Bc2 exd4

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6 13. Bc2 exd4'),
			moves => {
				'c3d4' => 'r2qrbk1/1bp2pp1/p1np1n1p/1p6/P2PP3/5N1P/1PBN1PP1/R1BQR1K1 b - - 0 14',
			},
		},
		'r2qrbk1/1bp2pp1/p2p1n1p/1p2p3/2nPP3/2P2NNP/PPB2PP1/R1BQR1K1 w - - 9 15' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. Nf1 Bb7 13. Ng3 Na5 14. Bc2 Nc4

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bb7 13. Ng3 Na5 14. Bc2 Nc4'),
			moves => {
			},
		},
		'r2qrbk1/1bp2pp1/p2p1n1p/1p6/Pn1PP3/5N1P/1P1N1PP1/RBBQR1K1 b - - 2 15' => {
			eco => 'C92t',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6 13. Bc2 exd4'),
			moves => {
				'c7c5' => 'r2qrbk1/1b3pp1/p2p1n1p/1pp5/Pn1PP3/5N1P/1P1N1PP1/RBBQR1K1 w - c6 0 16',
			},
		},
		'r2qrbk1/1bp2pp1/p2p1n1p/1p6/Pn1PP3/5N1P/1PBN1PP1/R1BQR1K1 w - - 1 15' => {
			eco => 'C92t',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Zaitsev, 12. a4 h6 13. Bc2 exd4'),
			moves => {
				'c2b1' => 'r2qrbk1/1bp2pp1/p2p1n1p/1p6/Pn1PP3/5N1P/1P1N1PP1/RBBQR1K1 b - - 2 15',
			},
		},
		'r2qrbk1/1bp2pp1/p2p1n1p/np2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 w - - 7 14' => {
			eco => 'C93',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bb7'),
			moves => {
				'b3c2' => 'r2qrbk1/1bp2pp1/p2p1n1p/np2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 b - - 8 14',
			},
		},
		'r2qrbk1/1bp2pp1/p2p1n1p/np2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 b - - 8 14' => {
			eco => 'C93',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bb7'),
			moves => {
				'a5c4' => 'r2qrbk1/1bp2pp1/p2p1n1p/1p2p3/2nPP3/2P2NNP/PPB2PP1/R1BQR1K1 w - - 9 15',
			},
		},
		'r2qrbk1/1bp2ppp/p1np1n2/1p1Pp3/4P3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 0 12' => {
			eco => 'C92m',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Zaitsev, 11.Nbd2'),
			moves => {
				'c6b8' => 'rn1qrbk1/1bp2ppp/p2p1n2/1p1Pp3/4P3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 1 13',
			},
		},
		'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/3PP3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 3 12' => {
			eco => 'C92m',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Zaitsev, 11.Nbd2'),
			moves => {
				'a2a3' => 'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/3PP3/PBP2N1P/1P1N1PP1/R1BQR1K1 b - - 0 12',
				'a2a4' => 'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/P2PP3/1BP2N1P/1P1N1PP1/R1BQR1K1 b - a3 0 12',
				'b3c2' => 'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 4 12',
				'd4d5' => 'r2qrbk1/1bp2ppp/p1np1n2/1p1Pp3/4P3/1BP2N1P/PP1N1PP1/R1BQR1K1 b - - 0 12',
			},
		},
		'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 b - - 4 12' => {
			eco => 'C92o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. Bc2

			variation => N__('Spanish: Closed, Zaitsev, 12. Bc2'),
			moves => {
			},
		},
		'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/3PP3/PBP2N1P/1P1N1PP1/R1BQR1K1 b - - 0 12' => {
			eco => 'C92p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. a3

			variation => N__('Spanish: Closed, Zaitsev, 12. a3'),
			moves => {
			},
		},
		'r2qrbk1/1bp2ppp/p1np1n2/1p2p3/P2PP3/1BP2N1P/1P1N1PP1/R1BQR1K1 b - a3 0 12' => {
			eco => 'C92q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. a4

			variation => N__('Spanish: Closed, Zaitsev, 12. a4'),
			moves => {
				'h7h6' => 'r2qrbk1/1bp2pp1/p1np1n1p/1p2p3/P2PP3/1BP2N1P/1P1N1PP1/R1BQR1K1 w - - 0 13',
			},
		},
		'r2qrbk1/1bpn1p1p/p2p1np1/1p2p3/3PP3/1PP2NNP/P1B2PP1/R1BQR1K1 b - - 0 15' => {
			eco => 'C95l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3 g6 15. b3

			variation => N__('Spanish: Closed, Breyer, Main Line, 14. Ng3 g6 15. b3'),
			moves => {
			},
		},
		'r2qrbk1/1bpn1p1p/p2p1np1/1p2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 w - - 0 15' => {
			eco => 'C95l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3 g6

			variation => N__('Spanish: Closed, Breyer, Main Line, 14. Ng3 g6'),
			moves => {
				'a2a4' => 'r2qrbk1/1bpn1p1p/p2p1np1/1p2p3/P2PP3/2P2NNP/1PB2PP1/R1BQR1K1 b - a3 0 15',
				'b2b3' => 'r2qrbk1/1bpn1p1p/p2p1np1/1p2p3/3PP3/1PP2NNP/P1B2PP1/R1BQR1K1 b - - 0 15',
			},
		},
		'r2qrbk1/1bpn1p1p/p2p1np1/1p2p3/P2PP3/2P2NNP/1PB2PP1/R1BQR1K1 b - a3 0 15' => {
			eco => 'C95m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3 g6 15. a4

			variation => N__('Spanish: Closed, Breyer, Main Line, 14. Ng3 g6 15. a4'),
			moves => {
				'c7c5' => 'r2qrbk1/1b1n1p1p/p2p1np1/1pp1p3/P2PP3/2P2NNP/1PB2PP1/R1BQR1K1 w - c6 0 16',
			},
		},
		'r2qrbk1/1bpn1ppp/p2p1n2/1p2p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 7 14' => {
			eco => 'C95k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8

			variation => N__('Spanish: Closed, Breyer, Main Line, 13. Nf1 Bf8'),
			moves => {
				'f1g3' => 'r2qrbk1/1bpn1ppp/p2p1n2/1p2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 b - - 8 14',
			},
		},
		'r2qrbk1/1bpn1ppp/p2p1n2/1p2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 b - - 8 14' => {
			eco => 'C95k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Nbd7 11. Nbd2 Bb7 12. Bc2 Re8 13. Nf1 Bf8 14. Ng3

			variation => N__('Spanish: Closed, Breyer, Main Line, 13. Nf1 Bf8 14. Ng3'),
			moves => {
				'c7c5' => 'r2qrbk1/1b1n1ppp/p2p1n2/1pp1p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 w - c6 0 15',
				'g7g6' => 'r2qrbk1/1bpn1p1p/p2p1np1/1p2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 w - - 0 15',
			},
		},
		'r2qrbk1/2pb1pp1/p1np1n1p/1p2p3/3PP3/1BP2N1P/PP3PP1/R1BQRNK1 w - - 5 13' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. Nf1 Bd7

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bd7'),
			moves => {
				'f1g3' => 'r2qrbk1/2pb1pp1/p1np1n1p/1p2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 b - - 6 13',
			},
		},
		'r2qrbk1/2pb1pp1/p1np1n1p/1p2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 b - - 6 13' => {
			eco => 'C93',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bd7'),
			moves => {
				'c6a5' => 'r2qrbk1/2pb1pp1/p2p1n1p/np2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 w - - 7 14',
			},
		},
		'r2qrbk1/2pb1pp1/p2p1n1p/np2p3/3PP3/1BP2NNP/PP3PP1/R1BQR1K1 w - - 7 14' => {
			eco => 'C93',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bd7'),
			moves => {
				'b3c2' => 'r2qrbk1/2pb1pp1/p2p1n1p/np2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 b - - 8 14',
			},
		},
		'r2qrbk1/2pb1pp1/p2p1n1p/np2p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 b - - 8 14' => {
			eco => 'C93',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bd7'),
			moves => {
				'c7c5' => 'r2qrbk1/3b1pp1/p2p1n1p/npp1p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 w - c6 0 15',
			},
		},
		'r2qrbk1/3b1pp1/p2p1n1p/npp1p3/3PP3/2P2NNP/PPB2PP1/R1BQR1K1 w - c6 0 15' => {
			eco => 'C93',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 h6 10. d4 Re8 11. Nbd2 Bf8 12. Nf1 Bd7 13. Ng3 Na5 14. Bc2 c5

			variation => N__('Spanish: Closed, Smyslov, 12. Nf1 Bd7 13. Ng3 Na5 14. Bc2 c5'),
			moves => {
			},
		},
		'r3k1nr/1pp3pp/p1pb4/4p3/4P1b1/3R1N2/PPP2PPP/RNB3K1 w kq - 2 10' => {
			eco => 'C69g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 Bg4 7. dxe5 Qxd1 8. Rxd1 fxe5 9. Rd3 Bd6

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 Queenswap, 9. Rd3 Bd6'),
			moves => {
			},
		},
		'r3k1nr/ppp1qppp/2n5/4P3/1bPp4/5N1P/PP1BPPP1/R2QKB1R w KQkq - 3 9' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. Nbd2 Bg4 6. h3 Bxf3 7. Nxf3 Bb4+ 8. Bd2 Qe7

			variation => N__('QGD: Albin, Krenosz Variation'),
			moves => {
			},
		},
		'r3k1r1/ppqbnp1Q/2n1p3/3pP3/3p1P2/P1P5/2P1N1PP/R1B1KB1R w KQq - 1 12' => {
			eco => 'C18t',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, Konstantinopolsky Variation'),
			moves => {
				'h7d3' => 'r3k1r1/ppqbnp2/2n1p3/3pP3/3p1P2/P1PQ4/2P1N1PP/R1B1KB1R b KQq - 2 12',
			},
		},
		'r3k1r1/ppqbnp2/2n1p3/3pP3/3p1P2/P1PQ4/2P1N1PP/R1B1KB1R b KQq - 2 12' => {
			eco => 'C18t',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, Konstantinopolsky Variation'),
			moves => {
				'd4c3' => 'r3k1r1/ppqbnp2/2n1p3/3pP3/5P2/P1pQ4/2P1N1PP/R1B1KB1R w KQq - 0 13',
			},
		},
		'r3k1r1/ppqbnp2/2n1p3/3pP3/5P2/P1NQ4/2P3PP/R1B1KB1R b KQq - 0 13' => {
			eco => 'C18v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7 8. Qxg7 Rg8 9. Qxh7 cxd4 10. Ne2 Nbc6 11. f4 Bd7 12. Qd3 dxc3 13. Nxc3

			variation => N__('French: Winawer, Poisoned Pawn, Main Line, 13.Nxc3'),
			moves => {
			},
		},
		'r3k1r1/ppqbnp2/2n1p3/3pP3/5P2/P1pQ4/2P1N1PP/R1B1KB1R w KQq - 0 13' => {
			eco => 'C18u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7 8. Qxg7 Rg8 9. Qxh7 cxd4 10. Ne2 Nbc6 11. f4 Bd7 12. Qd3 dxc3

			variation => N__('French: Winawer, Poisoned Pawn, Main Line'),
			moves => {
				'e2c3' => 'r3k1r1/ppqbnp2/2n1p3/3pP3/5P2/P1NQ4/2P3PP/R1B1KB1R b KQq - 0 13',
			},
		},
		'r3k2N/ppp1q1pp/2n2n2/3Pp3/2B3b1/8/PPPP1bPP/RNBQ1K1R w q - 1 9' => {
			eco => 'C57c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 Bc5 5. Nxf7 Bxf2+ 6. Kf1 Qe7 7. Nxh8 d5 8. exd5 Bg4

			variation => N__('Two Knights: Traxler, 5.Nxf7 & 6. Kf1, Beyer\'s 8... Bg4'),
			moves => {
			},
		},
		'r3k2r/1bq1bppp/p2ppP2/1p6/3N1P2/2N2Q2/PPP1B1PP/2KR3R b kq - 0 13' => {
			eco => 'B99d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Be2 b5 11. Bxf6 Nxf6 12. e5 Bb7 13. exf6

			variation => N__('Sicilian: Najdorf, Main Line, Keres, Queen Sacrifice Line'),
			moves => {
			},
		},
		'r3k2r/1bq1bppp/p2ppn2/1p2P3/3N1P2/2N2Q2/PPP1B1PP/2KR3R w kq - 1 13' => {
			eco => 'B99c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Be2 b5 11. Bxf6 Nxf6 12. e5 Bb7

			variation => N__('Sicilian: Najdorf, Main Line, Keres 10... b5 11.Bxf6 Nxf6 12. e5 Bb7'),
			moves => {
				'e5f6' => 'r3k2r/1bq1bppp/p2ppP2/1p6/3N1P2/2N2Q2/PPP1B1PP/2KR3R b kq - 0 13',
				'f3g3' => 'r3k2r/1bq1bppp/p2ppn2/1p2P3/3N1P2/2N3Q1/PPP1B1PP/2KR3R b kq - 2 13',
			},
		},
		'r3k2r/1bq1bppp/p2ppn2/1p2P3/3N1P2/2N3Q1/PPP1B1PP/2KR3R b kq - 2 13' => {
			eco => 'B99e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Be2 b5 11. Bxf6 Nxf6 12. e5 Bb7 13. Qg3

			variation => N__('Sicilian: Najdorf, Main Line, Keres, 13. Qg3'),
			moves => {
				'd6e5' => 'r3k2r/1bq1bppp/p3pn2/1p2p3/3N1P2/2N3Q1/PPP1B1PP/2KR3R w kq - 0 14',
			},
		},
		'r3k2r/1bq1bppp/p3pn2/1p2P3/3N4/2N3Q1/PPP1B1PP/2KR3R b kq - 0 14' => {
			eco => 'B99e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres, 13. Qg3'),
			moves => {
				'f6d7' => 'r3k2r/1bqnbppp/p3p3/1p2P3/3N4/2N3Q1/PPP1B1PP/2KR3R w kq - 1 15',
			},
		},
		'r3k2r/1bq1bppp/p3pn2/1p2p3/3N1P2/2N3Q1/PPP1B1PP/2KR3R w kq - 0 14' => {
			eco => 'B99e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres, 13. Qg3'),
			moves => {
				'f4e5' => 'r3k2r/1bq1bppp/p3pn2/1p2P3/3N4/2N3Q1/PPP1B1PP/2KR3R b kq - 0 14',
			},
		},
		'r3k2r/1bqnbppp/p2ppn2/1p1N2B1/3NPP2/3B1Q2/PPP3PP/2KRR3 b kq - 3 12' => {
			eco => 'B99k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 b5 11. Rhe1 Bb7 12. Nd5

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 b5 11.Rhe1 Bb7 12. Nd5'),
			moves => {
			},
		},
		'r3k2r/1bqnbppp/p2ppn2/1p4B1/3NPP2/2NB1Q2/PPP3PP/2KRR3 w kq - 2 12' => {
			eco => 'B99j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 b5 11. Rhe1 Bb7

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 b5 11.Rhe1 Bb7'),
			moves => {
				'c3d5' => 'r3k2r/1bqnbppp/p2ppn2/1p1N2B1/3NPP2/3B1Q2/PPP3PP/2KRR3 b kq - 3 12',
				'f3g3' => 'r3k2r/1bqnbppp/p2ppn2/1p4B1/3NPP2/2NB2Q1/PPP3PP/2KRR3 b kq - 3 12',
			},
		},
		'r3k2r/1bqnbppp/p2ppn2/1p4B1/3NPP2/2NB2Q1/PPP3PP/2KRR3 b kq - 3 12' => {
			eco => 'B99l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Bd3 b5 11. Rhe1 Bb7 12. Qg3

			variation => N__('Sicilian: Najdorf, Main Line, 10. Bd3 b5 11.Rhe1 Bb7 12. Qg3'),
			moves => {
			},
		},
		'r3k2r/1bqnbppp/p2ppn2/1p4B1/3NPPP1/2NB1Q2/PPP4P/2KR3R w kq - 2 12' => {
			eco => 'B99n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. g4 b5 11. Bd3 Bb7

			variation => N__('Sicilian: Najdorf, Main Line, 10. g4 b5 11. Bd3 Bb7'),
			moves => {
			},
		},
		'r3k2r/1bqnbppp/p3p3/1B2P3/3N4/2N3Q1/PPP3PP/2KR3R b kq - 0 15' => {
			eco => 'B99e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O Nbd7 10. Be2 b5 11. Bxf6 Nxf6 12. e5 Bb7 13. Qg3 dxe5 14. fxe5 Nd7 15. Bxb5

			variation => N__('Sicilian: Najdorf, Main Line, Keres, 13. Qg3 dxe5 14.fxe5 Nd7 15.Bxb5'),
			moves => {
			},
		},
		'r3k2r/1bqnbppp/p3p3/1p2P3/3N4/2N3Q1/PPP1B1PP/2KR3R w kq - 1 15' => {
			eco => 'B99e',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Main Line, Keres, 13. Qg3'),
			moves => {
				'e2b5' => 'r3k2r/1bqnbppp/p3p3/1B2P3/3N4/2N3Q1/PPP3PP/2KR3R b kq - 0 15',
			},
		},
		'r3k2r/1bqnbppp/pp1ppn2/8/2PQP3/1PN2NP1/P4PBP/R1BR2K1 w kq - 1 12' => {
			eco => 'A30y',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. Rd1 a6 10. b3 Nbd7 11. e4 Qc7

			variation => N__('English: Symmetrical, Hedgehog, Flexible Formation, 11. e4 Qc7'),
			moves => {
			},
		},
		'r3k2r/1pqbbppp/p1nppn2/6B1/3NPP2/2N5/PPPQB1PP/2KR3R w kq - 3 11' => {
			eco => 'B68',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 Be7 10. Be2 Qc7

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 Be7 10. Be2 Qc7'),
			moves => {
			},
		},
		'r3k2r/2pqbppp/p1n5/1pnpP3/6b1/2P2N2/PPBN1PPP/R1BQR1K1 w kq - 7 13' => {
			eco => 'C82g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Nc5 10. Bc2 Bg4 11. Nbd2 Be7 12. Re1 Qd7

			variation => N__('Spanish: Open, Berlin, 10. Bc2 Bg4 11.Nbd2 Be7 12. Re1 Qd7'),
			moves => {
			},
		},
		'r3k2r/pbpnqpp1/1p1ppn1p/8/2PP3B/2P1PP2/P2N2PP/R2QKB1R w KQkq - 1 11' => {
			eco => 'E13l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8. bxc3 d6 9. Nd2 Nbd7 10. f3 Qe7

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8.bxc3 d6 9. Nd2 Nbd7 10. f3 Qe7'),
			moves => {
			},
		},
		'r3k2r/pp1bnppp/2n1p3/q1ppP3/P2P4/2P2N2/2PB1PPP/R2QKB1R w KQkq - 3 10' => {
			eco => 'C19q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Bd2 Bd7

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Qa5+ 9. Bd2 Bd7'),
			moves => {
				'f1b5' => 'r3k2r/pp1bnppp/2n1p3/qBppP3/P2P4/2P2N2/2PB1PPP/R2QK2R b KQkq - 4 10',
			},
		},
		'r3k2r/pp1bnppp/2n1p3/q1ppP3/P2P4/2P2N2/2PQ1PPP/R1B1KB1R w KQkq - 3 10' => {
			eco => 'C19o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Qd2 Bd7

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Qd2 Bd7'),
			moves => {
			},
		},
		'r3k2r/pp1bnppp/2n1p3/qBppP3/P2P4/2P2N2/2PB1PPP/R2QK2R b KQkq - 4 10' => {
			eco => 'C19r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Nbc6 8. a4 Qa5 9. Bd2 Bd7 10. Bb5

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Nbc6 8. a4 Qa5+ 9. Bd2 Bd7 10. Bb5'),
			moves => {
			},
		},
		'r3k2r/pp1bqppp/2n2n2/2b5/8/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 2 11' => {
			eco => 'D32h',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'e8c8' => '2kr3r/pp1bqppp/2n2n2/2b5/8/2N1PN2/PP2BPPP/R1BQK2R w KQ - 3 12',
			},
		},
		'r3k2r/pp1bqppp/2n2n2/2b5/8/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 11' => {
			eco => 'D32h',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 7.Qxd5 Nc6 8. Nf3'),
			moves => {
				'f1e2' => 'r3k2r/pp1bqppp/2n2n2/2b5/8/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 2 11',
			},
		},
		'r3k2r/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R3K2R w KQkq - 0 11' => {
			eco => 'D14j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3 Bxd3 9. Qxd3 Bd6 10. Bxd6 Qxd6

			variation => N__('Slav: Exchange, 8. Bd3 Bxd3 9.Qxd3 Bd6 10.Bxd6 Qxd6'),
			moves => {
				'e1g1' => 'r3k2r/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R4RK1 b kq - 1 11',
			},
		},
		'r3k2r/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R4RK1 b kq - 1 11' => {
			eco => 'D14j',
                        # TRANSLATORS:

			variation => N__('Slav: Exchange, 8. Bd3 Bxd3 9.Qxd3 Bd6 10.Bxd6 Qxd6'),
			moves => {
				'e8g8' => 'r4rk1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R4RK1 w - - 2 12',
			},
		},
		'r3k2r/pp3ppp/n1p1p3/3q4/P1N5/8/1PK1Q1PP/R4B1R b kq - 0 15' => {
			eco => 'D17r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. e4 Bxe4 9. fxe4 Nxe4 10. Bd2 Qxd4 11. Nxe4 Qxe4+ 12. Qe2 Bxd2+ 13. Kxd2 Qd5+ 14. Kc2 Na6 15. Nxc4

			variation => N__('Slav: Central, Main Line, 15.Nxc4'),
			moves => {
				'e8c8' => '2kr3r/pp3ppp/n1p1p3/3q4/P1N5/8/1PK1Q1PP/R4B1R w - - 1 16',
				'e8g8' => 'r4rk1/pp3ppp/n1p1p3/3q4/P1N5/8/1PK1Q1PP/R4B1R w - - 1 16',
			},
		},
		'r3k2r/pp3ppp/n1p1p3/3qN3/P1p5/8/1PK1Q1PP/R4B1R w kq - 3 15' => {
			eco => 'D17q',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Main Line'),
			moves => {
				'e5c4' => 'r3k2r/pp3ppp/n1p1p3/3q4/P1N5/8/1PK1Q1PP/R4B1R b kq - 0 15',
			},
		},
		'r3k2r/ppp2ppp/1bn1bPq1/6N1/2ppN1P1/8/PPP2P1P/R1BQR1K1 w kq - 3 13' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'f2f4' => 'r3k2r/ppp2ppp/1bn1bPq1/6N1/2ppNPP1/8/PPP4P/R1BQR1K1 b kq f3 0 13',
			},
		},
		'r3k2r/ppp2ppp/1bn1bPq1/6N1/2ppNPP1/8/PPP4P/R1BQR1K1 b kq f3 0 13' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'e8c8' => '2kr3r/ppp2ppp/1bn1bPq1/6N1/2ppNPP1/8/PPP4P/R1BQR1K1 w - - 1 14',
			},
		},
		'r3k2r/ppp2ppp/2n1b3/q7/1b1pN3/5N2/PPPB1PPP/R2QR1K1 w kq - 3 11' => {
			eco => 'C56o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Bd2 Bb4

			variation => N__('Two Knights: Classical, Moller, 10. Bd2 Bb4'),
			moves => {
			},
		},
		'r3k2r/ppp2ppp/2n1bP2/2b2qN1/2pp2P1/2N5/PPP2P1P/R1BQR1K1 b kq g3 0 11' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'f5g6' => 'r3k2r/ppp2ppp/2n1bPq1/2b3N1/2pp2P1/2N5/PPP2P1P/R1BQR1K1 w kq - 1 12',
			},
		},
		'r3k2r/ppp2ppp/2n1bP2/2b2qN1/2pp4/2N5/PPP2PPP/R1BQR1K1 w kq - 6 11' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'c3e4' => 'r3k2r/ppp2ppp/2n1bP2/2b2qN1/2ppN3/8/PPP2PPP/R1BQR1K1 b kq - 7 11',
				'g2g4' => 'r3k2r/ppp2ppp/2n1bP2/2b2qN1/2pp2P1/2N5/PPP2P1P/R1BQR1K1 b kq g3 0 11',
			},
		},
		'r3k2r/ppp2ppp/2n1bP2/2b2qN1/2ppN3/8/PPP2PPP/R1BQR1K1 b kq - 7 11' => {
			eco => 'C55x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 d5 7. exf6 dxc4 8. Re1+ Be6 9. Ng5 Qd5 10. Nc3 Qf5 11. Nce4

			variation => N__('Two Knights: Max Lange, Marshall Variation'),
			moves => {
				'c5f8' => 'r3kb1r/ppp2ppp/2n1bP2/5qN1/2ppN3/8/PPP2PPP/R1BQR1K1 w kq - 8 12',
			},
		},
		'r3k2r/ppp2ppp/2n1bP2/2bq2N1/2pp4/2N5/PPP2PPP/R1BQR1K1 b kq - 5 10' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'd5f5' => 'r3k2r/ppp2ppp/2n1bP2/2b2qN1/2pp4/2N5/PPP2PPP/R1BQR1K1 w kq - 6 11',
			},
		},
		'r3k2r/ppp2ppp/2n1bP2/2bq2N1/2pp4/8/PPP2PPP/RNBQR1K1 w kq - 4 10' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'b1c3' => 'r3k2r/ppp2ppp/2n1bP2/2bq2N1/2pp4/2N5/PPP2PPP/R1BQR1K1 b kq - 5 10',
			},
		},
		'r3k2r/ppp2ppp/2n1bPq1/2b3N1/2pp2P1/2N5/PPP2P1P/R1BQR1K1 w kq - 1 12' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'c3e4' => 'r3k2r/ppp2ppp/2n1bPq1/2b3N1/2ppN1P1/8/PPP2P1P/R1BQR1K1 b kq - 2 12',
			},
		},
		'r3k2r/ppp2ppp/2n1bPq1/2b3N1/2ppN1P1/8/PPP2P1P/R1BQR1K1 b kq - 2 12' => {
			eco => 'C55w',
                        # TRANSLATORS:

			variation => N__('Two Knights: Max Lange, 6. e5 d5'),
			moves => {
				'c5b6' => 'r3k2r/ppp2ppp/1bn1bPq1/6N1/2ppN1P1/8/PPP2P1P/R1BQR1K1 w kq - 3 13',
			},
		},
		'r3k2r/ppp2ppp/2nbb3/6Bq/3pN3/5N2/PPP2PPP/R2QR1K1 w kq - 3 11' => {
			eco => 'C56g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qh5 9. Nxe4 Be6 10. Bg5 Bd6

			variation => N__('Two Knights: Classical, 8. Nc3 Qh5 9.Nxe4 Be6 10. Bg5 Bd6'),
			moves => {
			},
		},
		'r3kb1r/1b1n1ppp/pq2pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11' => {
			eco => 'D29n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. Nc3 Qb6

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. Nc3 Qb6'),
			moves => {
			},
		},
		'r3kb1r/1bqn1ppp/p3Pn2/1pp5/4P3/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 12' => {
			eco => 'D48g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O Bb7 10. e4 c5 11. d5 Qc7 12. dxe6

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7 10. e4 c5 11. d5 Qc7 12.dxe6'),
			moves => {
				'f7e6' => 'r3kb1r/1bqn2pp/p3pn2/1pp5/4P3/2NB1N2/PP3PPP/R1BQ1RK1 w kq - 0 13',
			},
		},
		'r3kb1r/1bqn1ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11' => {
			eco => 'D29l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. Nc3 Qc7

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. Nc3 Qc7'),
			moves => {
			},
		},
		'r3kb1r/1bqn1ppp/p3pn2/1ppP4/4P3/2NB1N2/PP3PPP/R1BQ1RK1 w kq - 1 12' => {
			eco => 'D48g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O Bb7 10. e4 c5 11. d5 Qc7

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7 10. e4 c5 11. d5 Qc7'),
			moves => {
				'd5e6' => 'r3kb1r/1bqn1ppp/p3Pn2/1pp5/4P3/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 12',
			},
		},
		'r3kb1r/1bqn2pp/p3pn2/1pp5/4P3/2NB1N2/PP3PPP/R1BQ1RK1 w kq - 0 13' => {
			eco => 'D48g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Nbd7 6. Bd3 dxc4 7. Bxc4 b5 8. Bd3 a6 9. O-O Bb7 10. e4 c5 11. d5 Qc7 12. dxe6 fxe6

			variation => N__('Semi-Slav: Meran, 8... a6 9.O-O Bb7 10. e4 c5 11. d5 Qc7 12.dxe6 fxe6'),
			moves => {
			},
		},
		'r3kb1r/1pp2ppp/p1p1b3/3q4/3PN3/8/PPP2PPP/R1BQR1K1 w kq - 1 11' => {
			eco => 'C48a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Nc3 Nf6 4. Bb5 a6 5. Bxc6 dxc6 6. Nxe5 Nxe4 7. Nxe4 Qd4 8. O-O Qxe5 9. Re1 Be6 10. d4 Qd5

			variation => N__('Four Knights: Spanish, Spielmann Variation'),
			moves => {
			},
		},
		'r3kb1r/1pp2ppp/p1p1b3/4q3/3PN3/8/PPP2PPP/R1BQR1K1 b kq d3 0 10' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'e5d5' => 'r3kb1r/1pp2ppp/p1p1b3/3q4/3PN3/8/PPP2PPP/R1BQR1K1 w kq - 1 11',
			},
		},
		'r3kb1r/1pp2ppp/p1p1b3/4q3/4N3/8/PPPP1PPP/R1BQR1K1 w kq - 2 10' => {
			eco => 'C48a',
                        # TRANSLATORS:

			variation => N__('Four Knights: Spanish, Ranken Variation'),
			moves => {
				'd2d4' => 'r3kb1r/1pp2ppp/p1p1b3/4q3/3PN3/8/PPP2PPP/R1BQR1K1 b kq d3 0 10',
			},
		},
		'r3kb1r/3b1p1p/pqNppp2/1p6/4PP2/2N5/PPPQ2PP/1K1R1B1R b kq - 0 12' => {
			eco => 'B67s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6 11. Kb1 Qb6 12. Nxc6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11. Kb1 Qb6 12.Nxc6'),
			moves => {
			},
		},
		'r3kb1r/3b1p1p/pqnppp2/1p6/3NPP2/2N5/PPPQ2PP/1K1R1B1R w kq - 2 12' => {
			eco => 'B67r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 a6 8. O-O-O Bd7 9. f4 b5 10. Bxf6 gxf6 11. Kb1 Qb6

			variation => N__('Sicilian: Richter-Rauzer, 7... a6, 9. f4 b5 10.Bxf6 gxf6 11. Kb1 Qb6'),
			moves => {
				'd4c6' => 'r3kb1r/3b1p1p/pqNppp2/1p6/4PP2/2N5/PPPQ2PP/1K1R1B1R b kq - 0 12',
			},
		},
		'r3kb1r/p1p2ppp/8/1p1n4/3np1bq/2P2PN1/PP1P2PP/RNBQKB1R w KQkq - 0 11' => {
			eco => 'C57m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5 8. Ne4 Qh4 9. Ng3 Bg4 10. f3 e4

			variation => N__('Two Knights: Fritz, Berliner Variation'),
			moves => {
			},
		},
		'r3kb1r/p1p2ppp/8/1p1np3/3n2bq/2P2PN1/PP1P2PP/RNBQKB1R b KQkq - 0 10' => {
			eco => 'C57k',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz, Gruber Variation (8. Ne4)'),
			moves => {
				'e5e4' => 'r3kb1r/p1p2ppp/8/1p1n4/3np1bq/2P2PN1/PP1P2PP/RNBQKB1R w KQkq - 0 11',
			},
		},
		'r3kb1r/p1p2ppp/8/1p1np3/3n2bq/2P3N1/PP1P1PPP/RNBQKB1R w KQkq - 4 10' => {
			eco => 'C57k',
                        # TRANSLATORS:

			variation => N__('Two Knights: Fritz, Gruber Variation (8. Ne4)'),
			moves => {
				'f2f3' => 'r3kb1r/p1p2ppp/8/1p1np3/3n2bq/2P2PN1/PP1P2PP/RNBQKB1R b KQkq - 0 10',
			},
		},
		'r3kb1r/p1ppqppp/b1p5/3nP3/2P5/1P6/P3QPPP/RNB1KB1R b KQkq - 0 9' => {
			eco => 'C45m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. e5 Qe7 7. Qe2 Nd5 8. c4 Ba6 9. b3

			variation => N__('Scotch: Mieses, 8. c4 Ba6 9. b3'),
			moves => {
			},
		},
		'r3kb1r/p1ppqppp/b1p5/3nP3/2P5/8/PP2QPPP/RNB1KB1R w KQkq - 1 9' => {
			eco => 'C45l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. d4 exd4 4. Nxd4 Nf6 5. Nxc6 bxc6 6. e5 Qe7 7. Qe2 Nd5 8. c4 Ba6

			variation => N__('Scotch: Mieses, 8. c4 Ba6'),
			moves => {
				'b2b3' => 'r3kb1r/p1ppqppp/b1p5/3nP3/2P5/1P6/P3QPPP/RNB1KB1R b KQkq - 0 9',
			},
		},
		'r3kb1r/pb1n1p2/1q2pP2/1ppP2B1/2p5/2N3P1/PP3P1P/R2QKB1R w KQkq - 1 14' => {
			eco => 'D44r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5 10. Bxg5 Nbd7 11. exf6 Bb7 12. g3 c5 13. d5 Qb6

			variation => N__('Semi-Slav: Botvinnik, Main Line, 12. g3 c5 13. d5 Qb6'),
			moves => {
			},
		},
		'r3kb1r/pbp2ppp/8/1p1np3/3n3q/2P3N1/PP1P1PPP/RNBQKB1R w KQkq - 4 10' => {
			eco => 'C57l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. Ng5 d5 5. exd5 Nd4 6. c3 b5 7. Bf1 Nxd5 8. Ne4 Qh4 9. Ng3 Bb7

			variation => N__('Two Knights: Fritz, Radchenko Variation'),
			moves => {
			},
		},
		'r3kb1r/pp1bpppp/2np1n2/q5B1/3NP3/2N5/PPP1BPPP/R2QK2R w KQkq - 7 8' => {
			eco => 'B60k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 Bd7 7. Be2 Qa5

			variation => N__('Sicilian: Richter-Rauzer, Larsen, 7. Be2 Qa5'),
			moves => {
			},
		},
		'r3kb1r/pp1qpppp/2n2n2/3p4/3P1Bb1/1QPB4/PP3PPP/RN2K1NR w KQkq - 5 8' => {
			eco => 'B13m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3 Nc6 5. c3 Nf6 6. Bf4 Bg4 7. Qb3 Qd7

			variation => N__('Caro-Kann: Exchange, Rubinstein, 7. Qb3 Qd7'),
			moves => {
			},
		},
		'r3kb1r/pp1qpppp/2np1n2/2p5/2P1P3/2N2N2/PP1P1PPP/R1BQ1RK1 b kq - 4 7' => {
			eco => 'B52m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4 Nc6 6. Nc3 Nf6 7. O-O

			variation => N__('Sicilian: 3. Bb5+, Sokolsky, 5... Nc6 6. Nc3 Nf6 7.O-O'),
			moves => {
			},
		},
		'r3kb1r/pp1qpppp/2np1n2/2p5/2P1P3/2N2N2/PP1P1PPP/R1BQK2R w KQkq - 3 7' => {
			eco => 'B52l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4 Nc6 6. Nc3 Nf6

			variation => N__('Sicilian: 3. Bb5+, Sokolsky, 5... Nc6 6. Nc3 Nf6'),
			moves => {
				'e1g1' => 'r3kb1r/pp1qpppp/2np1n2/2p5/2P1P3/2N2N2/PP1P1PPP/R1BQ1RK1 b kq - 4 7',
			},
		},
		'r3kb1r/pp1qpppp/2np1n2/2p5/2P1P3/5N2/PP1P1PPP/RNBQ1RK1 w kq - 3 7' => {
			eco => 'B52j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4 Nc6 6. O-O Nf6

			variation => N__('Sicilian: 3. Bb5+, Sokolsky, 5... Nc6 6.O-O Nf6'),
			moves => {
			},
		},
		'r3kb1r/pp1qpppp/2np1n2/2p5/3PP3/2P2N2/PP3PPP/RNBQ1RK1 b kq d3 0 7' => {
			eco => 'B52t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O Nc6 6. c3 Nf6 7. d4

			variation => N__('Sicilian: 3. Bb5+, Bronstein Gambit'),
			moves => {
			},
		},
		'r3kb1r/pp1qpppp/2np1n2/2p5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'B52s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O Nc6 6. c3 Nf6

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7 5.O-O Nc6 6. c3 Nf6'),
			moves => {
				'd2d4' => 'r3kb1r/pp1qpppp/2np1n2/2p5/3PP3/2P2N2/PP3PPP/RNBQ1RK1 b kq d3 0 7',
			},
		},
		'r3kb1r/pp1qpppp/2np1n2/2p5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 4 7' => {
			eco => 'B52q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O Nc6 6. Re1 Nf6

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7 5.O-O Nc6 6. Re1 Nf6'),
			moves => {
			},
		},
		'r3kb1r/ppp1pppp/2n2n2/3q4/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Nf3 Qxd5 5. Be2 Nc6

			variation => N__('Scandinavian: Portuguese, 4. Nf3 Qxd5 5. Be2 Nc6'),
			moves => {
			},
		},
		'r3kb1r/ppp2p2/2n1b2p/q5p1/3pN2B/5N2/PPP2PPP/R2QR1K1 w kq g6 0 12' => {
			eco => 'C56i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10. Bg5'),
			moves => {
				'e4f6' => 'r3kb1r/ppp2p2/2n1bN1p/q5p1/3p3B/5N2/PPP2PPP/R2QR1K1 b kq - 1 12',
			},
		},
		'r3kb1r/ppp2p2/2n1bN1p/q5p1/3p3B/5N2/PPP2PPP/R2QR1K1 b kq - 1 12' => {
			eco => 'C56i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10. Bg5'),
			moves => {
				'e8e7' => 'r4b1r/ppp1kp2/2n1bN1p/q5p1/3p3B/5N2/PPP2PPP/R2QR1K1 w - - 2 13',
			},
		},
		'r3kb1r/ppp2pp1/2n1b2p/q5B1/3pN3/5N2/PPP2PPP/R2QR1K1 w kq - 0 11' => {
			eco => 'C56i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10. Bg5'),
			moves => {
				'g5h4' => 'r3kb1r/ppp2pp1/2n1b2p/q7/3pN2B/5N2/PPP2PPP/R2QR1K1 b kq - 1 11',
			},
		},
		'r3kb1r/ppp2pp1/2n1b2p/q7/3pN2B/5N2/PPP2PPP/R2QR1K1 b kq - 1 11' => {
			eco => 'C56i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10. Bg5'),
			moves => {
				'g7g5' => 'r3kb1r/ppp2p2/2n1b2p/q5p1/3pN2B/5N2/PPP2PPP/R2QR1K1 w kq g6 0 12',
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/3q4/3pN3/5N2/PPPB1PPP/R2QR1K1 w kq - 3 11' => {
			eco => 'C56n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Bd2 Qd5

			variation => N__('Two Knights: Classical, Moller, 10. Bd2 Qd5'),
			moves => {
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/5q2/3pN3/5N2/PPPB1PPP/R2QR1K1 w kq - 3 11' => {
			eco => 'C56m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Bd2 Qf5

			variation => N__('Two Knights: Classical, Moller, 10. Bd2 Qf5'),
			moves => {
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/6Bq/3pN3/5N2/PPP2PPP/R2QR1K1 b kq - 2 10' => {
			eco => 'C56f',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, 8. Nc3 Qh5'),
			moves => {
				'f8d6' => 'r3k2r/ppp2ppp/2nbb3/6Bq/3pN3/5N2/PPP2PPP/R2QR1K1 w kq - 3 11',
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/7q/3pN3/5N2/PPP2PPP/R1BQR1K1 w kq - 1 10' => {
			eco => 'C56f',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, 8. Nc3 Qh5'),
			moves => {
				'c1g5' => 'r3kb1r/ppp2ppp/2n1b3/6Bq/3pN3/5N2/PPP2PPP/R2QR1K1 b kq - 2 10',
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/q5B1/3pN3/5N2/PPP2PPP/R2QR1K1 b kq - 2 10' => {
			eco => 'C56i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Bg5

			variation => N__('Two Knights: Classical, Moller, 10. Bg5'),
			moves => {
				'h7h6' => 'r3kb1r/ppp2pp1/2n1b2p/q5B1/3pN3/5N2/PPP2PPP/R2QR1K1 w kq - 0 11',
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/q5N1/3p4/5N2/PPP2PPP/R1BQR1K1 b kq - 2 10' => {
			eco => 'C56j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Neg5

			variation => N__('Two Knights: Classical, Moller, 10.Neg5'),
			moves => {
				'e8c8' => '2kr1b1r/ppp2ppp/2n1b3/q5N1/3p4/5N2/PPP2PPP/R1BQR1K1 w - - 3 11',
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/q7/3pN3/5N2/PPP2PPP/R1BQR1K1 w kq - 1 10' => {
			eco => 'C56h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6

			variation => N__('Two Knights: Classical, Moller, 9.Nxe4 Be6'),
			moves => {
				'c1d2' => 'r3kb1r/ppp2ppp/2n1b3/q7/3pN3/5N2/PPPB1PPP/R2QR1K1 b kq - 2 10',
				'c1g5' => 'r3kb1r/ppp2ppp/2n1b3/q5B1/3pN3/5N2/PPP2PPP/R2QR1K1 b kq - 2 10',
				'e4g5' => 'r3kb1r/ppp2ppp/2n1b3/q5N1/3p4/5N2/PPP2PPP/R1BQR1K1 b kq - 2 10',
			},
		},
		'r3kb1r/ppp2ppp/2n1b3/q7/3pN3/5N2/PPPB1PPP/R2QR1K1 b kq - 2 10' => {
			eco => 'C56l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Bd2

			variation => N__('Two Knights: Classical, Moller, 10. Bd2'),
			moves => {
				'a5d5' => 'r3kb1r/ppp2ppp/2n1b3/3q4/3pN3/5N2/PPPB1PPP/R2QR1K1 w kq - 3 11',
				'a5f5' => 'r3kb1r/ppp2ppp/2n1b3/5q2/3pN3/5N2/PPPB1PPP/R2QR1K1 w kq - 3 11',
				'f8b4' => 'r3k2r/ppp2ppp/2n1b3/q7/1b1pN3/5N2/PPPB1PPP/R2QR1K1 w kq - 3 11',
			},
		},
		'r3kb1r/ppp2ppp/2n1bP2/5qN1/2ppN3/8/PPP2PPP/R1BQR1K1 w kq - 8 12' => {
			eco => 'C55x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Bc5 6. e5 d5 7. exf6 dxc4 8. Re1+ Be6 9. Ng5 Qd5 10. Nc3 Qf5 11. Nce4 Bf8

			variation => N__('Two Knights: Max Lange, Rubinstein Variation'),
			moves => {
			},
		},
		'r3kb1r/pppb1ppp/8/3p4/3Pn2q/3B4/PPP2PPP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'C43s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Nd7 6. Nxd7 Bxd7 7. O-O Qh4

			variation => N__('Russian Game: Modern Attack, 3...Nxe4, Main Line, 7... Qh4'),
			moves => {
			},
		},
		'r3kb1r/pppqnppp/2n1b3/4P3/2Pp4/5NP1/PP1NPPBP/R1BQK2R w KQkq - 5 8' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. Nbd2 Qd7 7. Bg2 Nge7

			variation => N__('QGD: Albin, 5. g3 Be6 6.Nbd2 Qd7 7. Bg2 Nge7'),
			moves => {
			},
		},
		'r3kb1r/ppq2ppp/2p2n2/4nb2/P1N2B2/2N3P1/1P2PP1P/R2QKB1R b KQkq - 1 10' => {
			eco => 'D17i',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Carlsbad Variation'),
			moves => {
				'f6d7' => 'r3kb1r/ppqn1ppp/2p5/4nb2/P1N2B2/2N3P1/1P2PP1P/R2QKB1R w KQkq - 2 11',
			},
		},
		'r3kb1r/ppq2ppp/2p2n2/4nb2/P1N5/2N3P1/1P2PP1P/R1BQKB1R w KQkq - 0 10' => {
			eco => 'D17i',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Carlsbad Variation'),
			moves => {
				'c1f4' => 'r3kb1r/ppq2ppp/2p2n2/4nb2/P1N2B2/2N3P1/1P2PP1P/R2QKB1R b KQkq - 1 10',
			},
		},
		'r3kb1r/ppqn1pp1/2p1pn1p/7P/3P4/3Q1NN1/PPPB1PP1/2KR3R w kq - 2 13' => {
			eco => 'B19q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2 e6 12. O-O-O Ngf6

			variation => N__('Caro-Kann: Classical, Spassky, Main Line'),
			moves => {
				'g3e4' => 'r3kb1r/ppqn1pp1/2p1pn1p/7P/3PN3/3Q1N2/PPPB1PP1/2KR3R b kq - 3 13',
			},
		},
		'r3kb1r/ppqn1pp1/2p1pn1p/7P/3PN3/3Q1N2/PPPB1PP1/2KR3R b kq - 3 13' => {
			eco => 'B19r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2 e6 12. O-O-O Ngf6 13. Ne4

			variation => N__('Caro-Kann: Classical, Spassky, Main Line, 13. Ne4'),
			moves => {
				'e8c8' => '2kr1b1r/ppqn1pp1/2p1pn1p/7P/3PN3/3Q1N2/PPPB1PP1/2KR3R w - - 4 14',
			},
		},
		'r3kb1r/ppqn1ppp/2p2n2/4Pb2/P1N5/2N3P1/1P2PP1P/R1BQKB1R b KQkq - 0 9' => {
			eco => 'D17i',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Carlsbad Variation'),
			moves => {
				'd7e5' => 'r3kb1r/ppq2ppp/2p2n2/4nb2/P1N5/2N3P1/1P2PP1P/R1BQKB1R w KQkq - 0 10',
			},
		},
		'r3kb1r/ppqn1ppp/2p2n2/4pb2/P1NP4/2N3P1/1P2PP1P/R1BQKB1R w KQkq e6 0 9' => {
			eco => 'D17i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 Nbd7 7. Nxc4 Qc7 8. g3 e5

			variation => N__('Slav: Central, Carlsbad Variation'),
			moves => {
				'd4e5' => 'r3kb1r/ppqn1ppp/2p2n2/4Pb2/P1N5/2N3P1/1P2PP1P/R1BQKB1R b KQkq - 0 9',
			},
		},
		'r3kb1r/ppqn1ppp/2p5/4nb2/P1N2B2/2N3P1/1P2PP1P/R2QKB1R w KQkq - 2 11' => {
			eco => 'D17i',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Carlsbad Variation'),
			moves => {
				'f1g2' => 'r3kb1r/ppqn1ppp/2p5/4nb2/P1N2B2/2N3P1/1P2PPBP/R2QK2R b KQkq - 3 11',
			},
		},
		'r3kb1r/ppqn1ppp/2p5/4nb2/P1N2B2/2N3P1/1P2PPBP/R2QK2R b KQkq - 3 11' => {
			eco => 'D17i',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Carlsbad Variation'),
			moves => {
				'f7f6' => 'r3kb1r/ppqn2pp/2p2p2/4nb2/P1N2B2/2N3P1/1P2PPBP/R2QK2R w KQkq - 0 12',
			},
		},
		'r3kb1r/ppqn2pp/2p2p2/4nb2/P1N2B2/2N3P1/1P2PPBP/R2QK2R w KQkq - 0 12' => {
			eco => 'D17j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 Nbd7 7. Nxc4 Qc7 8. g3 e5 9. dxe5 Nxe5 10. Bf4 Nfd7 11. Bg2 f6

			variation => N__('Slav: Central, Carlsbad, Main Line'),
			moves => {
			},
		},
		'r3kb1r/ppqnpppp/2p2n2/5b2/P1NP4/2N3P1/1P2PP1P/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D17h',
                        # TRANSLATORS:

			variation => N__('Slav: Central, 6. Ne5 Nbd7'),
			moves => {
				'e7e5' => 'r3kb1r/ppqn1ppp/2p2n2/4pb2/P1NP4/2N3P1/1P2PP1P/R1BQKB1R w KQkq e6 0 9',
			},
		},
		'r3kb1r/ppqnpppp/2p2n2/5b2/P1NP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D17h',
                        # TRANSLATORS:

			variation => N__('Slav: Central, 6. Ne5 Nbd7'),
			moves => {
				'g2g3' => 'r3kb1r/ppqnpppp/2p2n2/5b2/P1NP4/2N3P1/1P2PP1P/R1BQKB1R b KQkq - 0 8',
			},
		},
		'r3kbnr/1pp2pp1/p1p2q2/4p2p/4P1b1/3P1N1P/PPP2PP1/RNBQ1RK1 w kq - 1 8' => {
			eco => 'C68o',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Alapin Gambit'),
			moves => {
				'b1d2' => 'r3kbnr/1pp2pp1/p1p2q2/4p2p/4P1b1/3P1N1P/PPPN1PP1/R1BQ1RK1 b kq - 2 8',
			},
		},
		'r3kbnr/1pp2pp1/p1p2q2/4p2p/4P1b1/3P1N1P/PPPN1PP1/R1BQ1RK1 b kq - 2 8' => {
			eco => 'C68p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Bg4 6. h3 h5 7. d3 Qf6 8. Nbd2

			variation => N__('Spanish: Exchange, Alapin Gambit, 7. d3 Qf6 8.Nbd2'),
			moves => {
			},
		},
		'r3kbnr/1pp2ppp/p1pqb3/4p3/4P3/N4N2/PPPP1PPP/R1BQ1RK1 w kq - 4 7' => {
			eco => 'C68u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O Qd6 6. Na3 Be6

			variation => N__('Spanish: Exchange, Bronstein, 6. Na3 Be6'),
			moves => {
			},
		},
		'r3kbnr/1pp3pp/p1p2p2/4P3/4P1b1/5N2/PPP2PPP/RNBR2K1 b kq - 0 8' => {
			eco => 'C69e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 7.dxe5'),
			moves => {
				'f6e5' => 'r3kbnr/1pp3pp/p1p5/4p3/4P1b1/5N2/PPP2PPP/RNBR2K1 w kq - 0 9',
			},
		},
		'r3kbnr/1pp3pp/p1p2p2/4P3/4P1b1/5N2/PPP2PPP/RNBq1RK1 w kq - 0 8' => {
			eco => 'C69e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 7.dxe5'),
			moves => {
				'f1d1' => 'r3kbnr/1pp3pp/p1p2p2/4P3/4P1b1/5N2/PPP2PPP/RNBR2K1 b kq - 0 8',
			},
		},
		'r3kbnr/1pp3pp/p1p5/4p3/4P1b1/3R1N2/PPP2PPP/RNB3K1 b kq - 1 9' => {
			eco => 'C69f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 Bg4 7. dxe5 Qxd1 8. Rxd1 fxe5 9. Rd3

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 Queenswap, 9. Rd3'),
			moves => {
				'f8d6' => 'r3k1nr/1pp3pp/p1pb4/4p3/4P1b1/3R1N2/PPP2PPP/RNB3K1 w kq - 2 10',
			},
		},
		'r3kbnr/1pp3pp/p1p5/4p3/4P1b1/5N2/PPP2PPP/RNBR2K1 w kq - 0 9' => {
			eco => 'C69e',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Gligoric, 6. d4 Bg4 7.dxe5'),
			moves => {
				'd1d3' => 'r3kbnr/1pp3pp/p1p5/4p3/4P1b1/3R1N2/PPP2PPP/RNB3K1 b kq - 1 9',
			},
		},
		'r3kbnr/1pp3pp/p3bp2/2p5/4P3/1N3P2/PPP3PP/RNBR2K1 w kq - 1 11' => {
			eco => 'C69n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5 8. Nb3 Qxd1 9. Rxd1 Bg4 10. f3 Be6

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3 Queenswap, 9... Bg4 10. f3 Be6'),
			moves => {
			},
		},
		'r3kbnr/1pp3pp/p4p2/2p5/4P1b1/1N3P2/PPP3PP/RNBR2K1 b kq - 0 10' => {
			eco => 'C69m',
                        # TRANSLATORS:

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3 Queenswap, 9... Bg4'),
			moves => {
				'g4e6' => 'r3kbnr/1pp3pp/p3bp2/2p5/4P3/1N3P2/PPP3PP/RNBR2K1 w kq - 1 11',
			},
		},
		'r3kbnr/1pp3pp/p4p2/2p5/4P1b1/1N6/PPP2PPP/RNBR2K1 w kq - 1 10' => {
			eco => 'C69m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5 8. Nb3 Qxd1 9. Rxd1 Bg4

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3 Queenswap, 9... Bg4'),
			moves => {
				'f2f3' => 'r3kbnr/1pp3pp/p4p2/2p5/4P1b1/1N3P2/PPP3PP/RNBR2K1 b kq - 0 10',
			},
		},
		'r3kbnr/1ppb1ppp/p1p5/8/3NP3/8/PPP2PPP/RNB1K2R w KQkq - 1 8' => {
			eco => 'C68f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. d4 exd4 6. Qxd4 Qxd4 7. Nxd4 Bd7

			variation => N__('Spanish: Exchange, Alekhine Variation'),
			moves => {
			},
		},
		'r3kbnr/1ppb2pp/p4p2/2p5/4P3/1N6/PPP2PPP/RNBR2K1 w kq - 1 10' => {
			eco => 'C69l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Bxc6 dxc6 5. O-O f6 6. d4 exd4 7. Nxd4 c5 8. Nb3 Qxd1 9. Rxd1 Bd7

			variation => N__('Spanish: Exchange, Gligoric, 8. Nb3 Queenswap, 9... Bd7'),
			moves => {
			},
		},
		'r3kbnr/1pqb1ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'B47j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 d6 8. O-O Bd7

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 d6 8.O-O Bd7'),
			moves => {
				'f1e1' => 'r3kbnr/1pqb1ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 b kq - 3 9',
			},
		},
		'r3kbnr/1pqb1ppp/p1npp3/8/3NP3/2N3P1/PPP2PBP/R1BQR1K1 b kq - 3 9' => {
			eco => 'B47k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nc6 5. Nc3 Qc7 6. g3 a6 7. Bg2 d6 8. O-O Bd7 9. Re1

			variation => N__('Sicilian: Taimanov, 6. g3 a6 7. Bg2 d6 8.O-O Bd7 9. Re1'),
			moves => {
			},
		},
		'r3kbnr/pp1b1ppp/1qn1p3/2ppP3/3P4/P1P2N2/1P3PPP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'C02w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Nc6 5. Nf3 Qb6 6. a3 Bd7

			variation => N__('French: Advance, 5. Nf3 Qb6 6. a3 Bd7'),
			moves => {
			},
		},
		'r3kbnr/pp1qpppp/2np4/2p5/2P1P3/2N2N2/PP1P1PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'B52k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4 Nc6 6. Nc3

			variation => N__('Sicilian: 3. Bb5+, Sokolsky, 5... Nc6 6. Nc3'),
			moves => {
				'g8f6' => 'r3kb1r/pp1qpppp/2np1n2/2p5/2P1P3/2N2N2/PP1P1PPP/R1BQK2R w KQkq - 3 7',
			},
		},
		'r3kbnr/pp1qpppp/2np4/2p5/2P1P3/5N2/PP1P1PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'B52i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4 Nc6 6. O-O

			variation => N__('Sicilian: 3. Bb5+, Sokolsky, 5... Nc6 6.O-O'),
			moves => {
				'g8f6' => 'r3kb1r/pp1qpppp/2np1n2/2p5/2P1P3/5N2/PP1P1PPP/RNBQ1RK1 w kq - 3 7',
			},
		},
		'r3kbnr/pp1qpppp/2np4/2p5/2P1P3/5N2/PP1P1PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'B52h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4 Nc6

			variation => N__('Sicilian: 3. Bb5+, Sokolsky, 5... Nc6'),
			moves => {
				'b1c3' => 'r3kbnr/pp1qpppp/2np4/2p5/2P1P3/2N2N2/PP1P1PPP/R1BQK2R b KQkq - 2 6',
				'e1g1' => 'r3kbnr/pp1qpppp/2np4/2p5/2P1P3/5N2/PP1P1PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'r3kbnr/pp1qpppp/2np4/2p5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'B52r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O Nc6 6. c3

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7 5.O-O Nc6 6. c3'),
			moves => {
				'g8f6' => 'r3kb1r/pp1qpppp/2np1n2/2p5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 w kq - 1 7',
			},
		},
		'r3kbnr/pp1qpppp/2np4/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'B52o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O Nc6

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7 5.O-O Nc6'),
			moves => {
				'c2c3' => 'r3kbnr/pp1qpppp/2np4/2p5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 0 6',
				'f1e1' => 'r3kbnr/pp1qpppp/2np4/2p5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 3 6',
			},
		},
		'r3kbnr/pp1qpppp/2np4/2p5/4P3/5N2/PPPP1PPP/RNBQR1K1 b kq - 3 6' => {
			eco => 'B52p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O Nc6 6. Re1

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7 5.O-O Nc6 6. Re1'),
			moves => {
				'g8f6' => 'r3kb1r/pp1qpppp/2np1n2/2p5/4P3/5N2/PPPP1PPP/RNBQR1K1 w kq - 4 7',
			},
		},
		'r3kbnr/pp2pppp/2n5/2pq4/3P2b1/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'B22i',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nc6 5. Nf3 Bg4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nc6 5. Nf3 Bg4'),
			moves => {
			},
		},
		'r3kbnr/pppq1ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPP1P/R1BQKB1R w KQkq - 3 7' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. Nbd2 Qd7

			variation => N__('QGD: Albin, 5. g3 Be6 6.Nbd2 Qd7'),
			moves => {
				'f1g2' => 'r3kbnr/pppq1ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPPBP/R1BQK2R b KQkq - 4 7',
			},
		},
		'r3kbnr/pppq1ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPPBP/R1BQK2R b KQkq - 4 7' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Be6 6. Nbd2 Qd7 7. Bg2

			variation => N__('QGD: Albin, 5. g3 Be6 6.Nbd2 Qd7 7. Bg2'),
			moves => {
				'e8c8' => '2kr1bnr/pppq1ppp/2n1b3/4P3/2Pp4/5NP1/PP1NPPBP/R1BQK2R w KQ - 5 8',
				'g8e7' => 'r3kb1r/pppqnppp/2n1b3/4P3/2Pp4/5NP1/PP1NPPBP/R1BQK2R w KQkq - 5 8',
			},
		},
		'r3kbnr/pppq1ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 7' => {
			eco => 'D09',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, 5. g3 Bg4 6. Bg2 Qd7'),
			moves => {
				'e8c8' => '2kr1bnr/pppq1ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQ1RK1 w - - 5 8',
			},
		},
		'r3kbnr/pppq1ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 7' => {
			eco => 'D09',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3 Nc6 5. g3 Bg4 6. Bg2 Qd7

			variation => N__('QGD: Albin, 5. g3 Bg4 6. Bg2 Qd7'),
			moves => {
				'e1g1' => 'r3kbnr/pppq1ppp/2n5/4P3/2Pp2b1/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 7',
			},
		},
		'r3kbnr/ppqn1pp1/2p1p2p/7P/3P4/3Q1NN1/PPPB1PP1/2KR3R b kq - 1 12' => {
			eco => 'B19p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2 e6 12. O-O-O

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 Qc7 11. Bd2 e6 12.O-O-O'),
			moves => {
				'g8f6' => 'r3kb1r/ppqn1pp1/2p1pn1p/7P/3P4/3Q1NN1/PPPB1PP1/2KR3R w kq - 2 13',
			},
		},
		'r3kbnr/ppqn1pp1/2p1p2p/7P/3P4/3Q1NN1/PPPB1PP1/R3K2R w KQkq - 0 12' => {
			eco => 'B19o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2 e6

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 Qc7 11. Bd2 e6'),
			moves => {
				'e1c1' => 'r3kbnr/ppqn1pp1/2p1p2p/7P/3P4/3Q1NN1/PPPB1PP1/2KR3R b kq - 1 12',
			},
		},
		'r3kbnr/ppqnppp1/2p4p/7P/3P4/3Q1NN1/PPP2PP1/R1B1K2R w KQkq - 1 11' => {
			eco => 'B19m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 Qc7'),
			moves => {
				'c1d2' => 'r3kbnr/ppqnppp1/2p4p/7P/3P4/3Q1NN1/PPPB1PP1/R3K2R b KQkq - 2 11',
			},
		},
		'r3kbnr/ppqnppp1/2p4p/7P/3P4/3Q1NN1/PPPB1PP1/R3K2R b KQkq - 2 11' => {
			eco => 'B19n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nd7 8. h5 Bh7 9. Bd3 Bxd3 10. Qxd3 Qc7 11. Bd2

			variation => N__('Caro-Kann: Classical, Spassky, 10.Qxd3 Qc7 11. Bd2'),
			moves => {
				'e7e6' => 'r3kbnr/ppqn1pp1/2p1p2p/7P/3P4/3Q1NN1/PPPB1PP1/R3K2R w KQkq - 0 12',
			},
		},
		'r3qrk1/1ppb1pb1/n2p1npp/p2Pp3/2P1P2B/2N5/PP1NBPPP/R2Q1RK1 w - - 6 12' => {
			eco => 'E92w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6 9. Bh4 Na6 10. Nd2 Qe8 11. O-O Bd7

			variation => N__('King\'s Indian: Petrosian, Stein, Main Line, 10... Qe8 11.O-O Bd7'),
			moves => {
			},
		},
		'r3r1k1/2qbbp1p/p2p1np1/npp1p3/3PP3/2P1NN1P/PPB2PP1/R1BQR1K1 w - - 0 15' => {
			eco => 'C97l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Bd7 13. Nf1 Rfe8 14. Ne3 g6

			variation => N__('Spanish: Closed, Chigorin, Yugoslav System'),
			moves => {
			},
		},
		'r3r1k1/2qbbppp/p2p1n2/npp1p3/3PP3/2P1NN1P/PPB2PP1/R1BQR1K1 b - - 6 14' => {
			eco => 'C97k',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed, Chigorin, 12... Bd7 13. Nf1 Rfe8'),
			moves => {
				'g7g6' => 'r3r1k1/2qbbp1p/p2p1np1/npp1p3/3PP3/2P1NN1P/PPB2PP1/R1BQR1K1 w - - 0 15',
			},
		},
		'r3r1k1/2qbbppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 5 14' => {
			eco => 'C97k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Bd7 13. Nf1 Rfe8

			variation => N__('Spanish: Closed, Chigorin, 12... Bd7 13. Nf1 Rfe8'),
			moves => {
				'f1e3' => 'r3r1k1/2qbbppp/p2p1n2/npp1p3/3PP3/2P1NN1P/PPB2PP1/R1BQR1K1 b - - 6 14',
			},
		},
		'r4b1r/ppp1kp2/2n1bN1p/q5p1/1P1p3B/5N2/P1P2PPP/R2QR1K1 b - b3 0 13' => {
			eco => 'C56i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Nf6 4. d4 exd4 5. O-O Nxe4 6. Re1 d5 7. Bxd5 Qxd5 8. Nc3 Qa5 9. Nxe4 Be6 10. Bg5 h6 11. Bh4 g5 12. Nf6+ Ke7 13. b4

			variation => N__('Two Knights: Classical, Yurdansky Attack'),
			moves => {
			},
		},
		'r4b1r/ppp1kp2/2n1bN1p/q5p1/3p3B/5N2/PPP2PPP/R2QR1K1 w - - 2 13' => {
			eco => 'C56i',
                        # TRANSLATORS:

			variation => N__('Two Knights: Classical, Moller, 10. Bg5'),
			moves => {
				'b2b4' => 'r4b1r/ppp1kp2/2n1bN1p/q5p1/1P1p3B/5N2/P1P2PPP/R2QR1K1 b - b3 0 13',
			},
		},
		'r4rk1/1bq1bppp/p2p1n2/np1Pp3/4P3/5N1P/PPBN1PP1/R1BQR1K1 b - - 0 14' => {
			eco => 'C99g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bb7 14. d5

			variation => N__('Spanish: Closed, Chigorin, 13... Bb7 14. d5'),
			moves => {
				'a8c8' => '2r2rk1/1bq1bppp/p2p1n2/np1Pp3/4P3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 15',
			},
		},
		'r4rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 b - - 2 14' => {
			eco => 'C99e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bb7 14. Nf1

			variation => N__('Spanish: Closed, Chigorin, 13... Bb7 14. Nf1'),
			moves => {
				'a8c8' => '2r2rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 w - - 3 15',
			},
		},
		'r4rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14' => {
			eco => 'C99e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bb7

			variation => N__('Spanish: Closed, Chigorin, 13... Bb7'),
			moves => {
				'd2f1' => 'r4rk1/1bq1bppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 b - - 2 14',
				'd4d5' => 'r4rk1/1bq1bppp/p2p1n2/np1Pp3/4P3/5N1P/PPBN1PP1/R1BQR1K1 b - - 0 14',
			},
		},
		'r4rk1/1bq1bppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13' => {
			eco => 'C97e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Bb7

			variation => N__('Spanish: Closed, Chigorin, 12... Bb7'),
			moves => {
			},
		},
		'r4rk1/1p1nqpbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2QR1K1 w - - 3 14' => {
			eco => 'A75',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12.Bxf3 Qe7 13. Re1'),
			moves => {
				'a4a5' => 'r4rk1/1p1nqpbp/p2p1np1/P1pP4/4PB2/2N2B2/1P3PPP/R2QR1K1 b - - 0 14',
			},
		},
		'r4rk1/1p1nqpbp/p2p1np1/P1pP4/4PB2/2N2B2/1P3PPP/R2QR1K1 b - - 0 14' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12. Bxf3 Qe7 13. Re1 Nbd7 14. a5

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12.Bxf3 Qe7 with 14. a5'),
			moves => {
			},
		},
		'r4rk1/2p3pp/p1n1bq2/1p1p4/8/2P2N2/PPB2KPP/R1BQ1N2 b - - 1 15' => {
			eco => 'C82q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Nxf2 12. Rxf2 f6 13. exf6 Bxf2+ 14. Kxf2 Qxf6 15. Nf1

			variation => N__('Spanish: Open, Dilworth, 14.Kxf2 Qxf6 15. Nf1'),
			moves => {
			},
		},
		'r4rk1/2p3pp/p1n1bq2/1p1p4/8/2P2N2/PPBN1KPP/R1BQ4 w - - 0 15' => {
			eco => 'C82p',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Dilworth, 14.Kxf2'),
			moves => {
				'd2f1' => 'r4rk1/2p3pp/p1n1bq2/1p1p4/8/2P2N2/PPB2KPP/R1BQ1N2 b - - 1 15',
				'f2g1' => 'r4rk1/2p3pp/p1n1bq2/1p1p4/8/2P2N2/PPBN2PP/R1BQ2K1 b - - 1 15',
			},
		},
		'r4rk1/2p3pp/p1n1bq2/1p1p4/8/2P2N2/PPBN2PP/R1BQ2K1 b - - 1 15' => {
			eco => 'C82r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Nxf2 12. Rxf2 f6 13. exf6 Bxf2+ 14. Kxf2 Qxf6 15. Kg1

			variation => N__('Spanish: Open, Dilworth, 14.Kxf2 Qxf6 15. Kg1'),
			moves => {
				'g7g5' => 'r4rk1/2p4p/p1n1bq2/1p1p2p1/8/2P2N2/PPBN2PP/R1BQ2K1 w - g6 0 16',
			},
		},
		'r4rk1/2p4p/p1n1bq2/1p1p2p1/8/2P2N2/PPBN2PP/R1BQ2K1 w - g6 0 16' => {
			eco => 'C82s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Bc5 10. Nbd2 O-O 11. Bc2 Nxf2 12. Rxf2 f6 13. exf6 Bxf2+ 14. Kxf2 Qxf6 15. Kg1 g5

			variation => N__('Spanish: Open, Dilworth, 14.Kxf2 Qxf6 15. Kg1 g5'),
			moves => {
			},
		},
		'r4rk1/2pqb1pp/p1n1p3/1p1pP3/4R3/1BP5/PP3PPP/RNBQ2K1 b - - 0 13' => {
			eco => 'C83',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. c3 Be7 10. Re1 O-O 11. Nd4 Qd7 12. Nxe6 fxe6 13. Rxe4

			variation => N__('Spanish: Open, Classical, Tarrasch Trap'),
			moves => {
			},
		},
		'r4rk1/2pqb1pp/p1n1p3/1p1pP3/4n3/1BP5/PP3PPP/RNBQR1K1 w - - 0 13' => {
			eco => 'C83',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Classical, 10. Re1'),
			moves => {
				'e1e4' => 'r4rk1/2pqb1pp/p1n1p3/1p1pP3/4R3/1BP5/PP3PPP/RNBQ2K1 b - - 0 13',
			},
		},
		'r4rk1/2pqbppp/p1n1N3/1p1pP3/4n3/1BP5/PP3PPP/RNBQR1K1 b - - 0 12' => {
			eco => 'C83',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Classical, 10. Re1'),
			moves => {
				'f7e6' => 'r4rk1/2pqb1pp/p1n1p3/1p1pP3/4n3/1BP5/PP3PPP/RNBQR1K1 w - - 0 13',
			},
		},
		'r4rk1/2pqbppp/p1n1b3/1p1pP3/3Nn3/1BP5/PP3PPP/RNBQR1K1 w - - 5 12' => {
			eco => 'C83',
                        # TRANSLATORS:

			variation => N__('Spanish: Open, Classical, 10. Re1'),
			moves => {
				'd4e6' => 'r4rk1/2pqbppp/p1n1N3/1p1pP3/4n3/1BP5/PP3PPP/RNBQR1K1 b - - 0 12',
			},
		},
		'r4rk1/2pqbppp/p1n1b3/3pP3/2B1n3/5N2/PP2QPPP/RNBR2K1 w - - 1 13' => {
			eco => 'C81',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Nxe4 6. d4 b5 7. Bb3 d5 8. dxe5 Be6 9. Qe2 Be7 10. Rd1 O-O 11. c4 bxc4 12. Bxc4 Qd7

			variation => N__('Spanish: Open, Keres, Ekstrom Variation'),
			moves => {
			},
		},
		'r4rk1/2q1bppp/p1n1bn2/1pp1p3/4P3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 2 15' => {
			eco => 'C98d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Nc6 13. dxc5 dxc5 14. Nf1 Be6

			variation => N__('Spanish: Closed, Chigorin, Rauzer, 14. Nf1 Be6'),
			moves => {
			},
		},
		'r4rk1/2qbbppp/2np1n2/1p2p3/p2PP3/4BN1P/PPBN1PP1/R2QR1K1 w - - 2 17' => {
			eco => 'C99o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Nc6 14. Nb3 a5 15. Be3 a4 16. Nbd2 Bd7

			variation => N__('Spanish: Closed, Chigorin, 13... Nc6 14. Nb3 a5 15. Be3 a4 16.Nbd2 Bd7'),
			moves => {
			},
		},
		'r4rk1/2qbbppp/p2p1n2/1pp1p3/2nPP3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 5 14' => {
			eco => 'C97j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Bd7 13. Nf1 Nc4

			variation => N__('Spanish: Closed, Chigorin, 12... Bd7 13. Nf1 Nc4'),
			moves => {
			},
		},
		'r4rk1/2qbbppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 b - - 2 14' => {
			eco => 'C99c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bd7 14. Nf1

			variation => N__('Spanish: Closed, Chigorin, 13... Bd7 14. Nf1'),
			moves => {
				'a8c8' => '2r2rk1/2qbbppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 w - - 3 15',
			},
		},
		'r4rk1/2qbbppp/p2p1n2/np2p3/3PP3/5N1P/PPBN1PP1/R1BQR1K1 w - - 1 14' => {
			eco => 'C99c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 cxd4 13. cxd4 Bd7

			variation => N__('Spanish: Closed, Chigorin, 13... Bd7'),
			moves => {
				'd2f1' => 'r4rk1/2qbbppp/p2p1n2/np2p3/3PP3/5N1P/PPB2PP1/R1BQRNK1 b - - 2 14',
			},
		},
		'r4rk1/2qbbppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 4 13' => {
			eco => 'C97i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Bd7 13. Nf1

			variation => N__('Spanish: Closed, Chigorin, 12... Bd7 13. Nf1'),
			moves => {
				'a5c4' => 'r4rk1/2qbbppp/p2p1n2/1pp1p3/2nPP3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 5 14',
				'f8e8' => 'r3r1k1/2qbbppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 w - - 5 14',
			},
		},
		'r4rk1/2qbbppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPBN1PP1/R1BQR1K1 w - - 3 13' => {
			eco => 'C97h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Na5 10. Bc2 c5 11. d4 Qc7 12. Nbd2 Bd7

			variation => N__('Spanish: Closed, Chigorin, 12... Bd7'),
			moves => {
				'd2f1' => 'r4rk1/2qbbppp/p2p1n2/npp1p3/3PP3/2P2N1P/PPB2PP1/R1BQRNK1 b - - 4 13',
			},
		},
		'r4rk1/3nppbp/1q1p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQ3R w - - 1 13' => {
			eco => 'A59p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3 Qb6

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3 Qb6'),
			moves => {
				'h1e1' => 'r4rk1/3nppbp/1q1p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQR3 b - - 2 13',
			},
		},
		'r4rk1/3nppbp/1q1p1np1/2pP4/4P3/2N2NPP/PP3PK1/R1BQR3 b - - 2 13' => {
			eco => 'A59p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3 Qb6 13. Re1

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3 Qb6 13. Re1'),
			moves => {
			},
		},
		'r4rk1/3nppbp/3p1np1/q1pP4/4P3/2N2NP1/PP3PKP/R1BQR3 w - - 7 13' => {
			eco => 'A59l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. Re1 Qa5

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. Re1 Qa5'),
			moves => {
			},
		},
		'r4rk1/3nppbp/3p1np1/q1pP4/4P3/2N2NPP/PP3PK1/R1BQ3R w - - 1 13' => {
			eco => 'A59q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3 Qa5

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3 Qa5'),
			moves => {
				'h1e1' => 'r4rk1/3nppbp/3p1np1/q1pP4/4P3/2N2NPP/PP3PK1/R1BQR3 b - - 2 13',
			},
		},
		'r4rk1/3nppbp/3p1np1/q1pP4/4P3/2N2NPP/PP3PK1/R1BQR3 b - - 2 13' => {
			eco => 'A59q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3 Nbd7 12. h3 Qa5 13. Re1

			variation => N__('Benko Gambit: 7. e4, Main Line, 11...Nbd7 12. h3 Qa5 13. Re1'),
			moves => {
			},
		},
		'r4rk1/3nppbp/bq1p1np1/2pP4/8/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 7 12' => {
			eco => 'A58t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 O-O 10. O-O Nbd7 11. Qc2 Qb6

			variation => N__('Benko Gambit: Fianchetto, Main Line, 11. Qc2 Qb6'),
			moves => {
			},
		},
		'r4rk1/5ppp/p1pb4/1p1n4/3P2b1/1BP1B1Pq/PP3P1P/RN1QR1K1 w - - 3 16' => {
			eco => 'C89p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4

			variation => N__('Spanish: Marshall, Main Line, 15. Be3 Bg4'),
			moves => {
				'd1d3' => 'r4rk1/5ppp/p1pb4/1p1n4/3P2b1/1BPQB1Pq/PP3P1P/RN2R1K1 b - - 4 16',
			},
		},
		'r4rk1/5ppp/p1pb4/1p1n4/3P2b1/1BPQB1Pq/PP3P1P/RN2R1K1 b - - 4 16' => {
			eco => 'C89p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d5 9. exd5 Nxd5 10. Nxe5 Nxe5 11. Rxe5 c6 12. d4 Bd6 13. Re1 Qh4 14. g3 Qh3 15. Be3 Bg4 16. Qd3

			variation => N__('Spanish: Marshall, Main Line, 15. Be3 Bg4 16. Qd3'),
			moves => {
				'a8e8' => '4rrk1/5ppp/p1pb4/1p1n4/3P2b1/1BPQB1Pq/PP3P1P/RN2R1K1 w - - 5 17',
			},
		},
		'r4rk1/pp1bbppp/3ppn2/q3P1B1/2BQ1P2/2N5/PPP3PP/2KR3R b - - 0 12' => {
			eco => 'B65g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4 Qa5 11. Bc4 Bd7 12. e5

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4 10.Qxd4 Qa5 11. Bc4 Bd7 12. e5'),
			moves => {
			},
		},
		'r4rk1/pp1bbppp/3ppn2/q5B1/2BQPP2/2N5/PPP3PP/2KR3R w - - 3 12' => {
			eco => 'B65f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O O-O 9. f4 Nxd4 10. Qxd4 Qa5 11. Bc4 Bd7

			variation => N__('Sicilian: Richter-Rauzer, 7... Be7, 9. f4 Nxd4 10.Qxd4 Qa5 11. Bc4 Bd7'),
			moves => {
				'e4e5' => 'r4rk1/pp1bbppp/3ppn2/q3P1B1/2BQ1P2/2N5/PPP3PP/2KR3R b - - 0 12',
			},
		},
		'r4rk1/pp1bppbp/2np1np1/q7/2BNP3/2N1BP2/PPPQ2PP/2KR3R w - - 7 11' => {
			eco => 'B78r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. Bc4 Bd7 10. O-O-O Qa5

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line'),
			moves => {
				'c4b3' => 'r4rk1/pp1bppbp/2np1np1/q7/3NP3/1BN1BP2/PPPQ2PP/2KR3R b - - 8 11',
			},
		},
		'r4rk1/pp1bppbp/2np1np1/q7/3NP3/1BN1B2P/PPP2PP1/R2Q1RK1 w - - 1 11' => {
			eco => 'B35d',
                        # TRANSLATORS:

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5 8.O-O O-O 9. Bb3'),
			moves => {
				'f2f4' => 'r4rk1/pp1bppbp/2np1np1/q7/3NPP2/1BN1B2P/PPP3P1/R2Q1RK1 b - f3 0 11',
			},
		},
		'r4rk1/pp1bppbp/2np1np1/q7/3NP3/1BN1BP2/PPPQ2PP/2KR3R b - - 8 11' => {
			eco => 'B78r',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Yugoslav, Old Main Line'),
			moves => {
				'a8c8' => '2r2rk1/pp1bppbp/2np1np1/q7/3NP3/1BN1BP2/PPPQ2PP/2KR3R w - - 9 12',
				'f8c8' => 'r1r3k1/pp1bppbp/2np1np1/q7/3NP3/1BN1BP2/PPPQ2PP/2KR3R w - - 9 12',
			},
		},
		'r4rk1/pp1bppbp/2np1np1/q7/3NPP2/1BN1B2P/PPP3P1/R2Q1RK1 b - f3 0 11' => {
			eco => 'B35e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. Be3 Nf6 7. Bc4 Qa5 8. O-O O-O 9. Bb3 d6 10. h3 Bd7 11. f4

			variation => N__('Sicilian: Accelerated Fianchetto, Modern, 7. Bc4 Qa5 8.O-O O-O 9. Bb3 d6 10. h3 Bd7 11. f4'),
			moves => {
			},
		},
		'r4rk1/pp2bpp1/2npbn1p/q3p3/4P3/2N2N1P/PPP2PP1/R1BQRBK1 w - - 3 12' => {
			eco => 'B58q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6 11. Bf1 Qa5

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6 11. Bf1 Qa5'),
			moves => {
			},
		},
		'r4rk1/pp2ppbp/3p1np1/q4P2/4P1P1/2N1B3/PPP1Q2P/R4RK1 b - g3 0 14' => {
			eco => 'B74',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 Nc6 8. O-O O-O 9. Nb3 Be6 10. f4 Na5 11. f5 Bc4 12. Nxa5 Bxe2 13. Qxe2 Qxa5 14. g4

			variation => N__('Sicilian: Dragon, Classical, Stockholm Attack'),
			moves => {
			},
		},
		'r4rk1/pp2ppbp/3p1np1/q4P2/4P3/2N1B3/PPP1Q1PP/R4RK1 w - - 0 14' => {
			eco => 'B74',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, Classical, Maroczy Variation'),
			moves => {
				'g2g4' => 'r4rk1/pp2ppbp/3p1np1/q4P2/4P1P1/2N1B3/PPP1Q2P/R4RK1 b - g3 0 14',
			},
		},
		'r4rk1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/2R2RK1 b - - 3 12' => {
			eco => 'D14l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3 Bxd3 9. Qxd3 Bd6 10. Bxd6 Qxd6 11. O-O O-O 12. Rac1

			variation => N__('Slav: Exchange, 8. Bd3 Main Line, 12.Rac1'),
			moves => {
			},
		},
		'r4rk1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R1R3K1 b - - 3 12' => {
			eco => 'D14m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3 Bxd3 9. Qxd3 Bd6 10. Bxd6 Qxd6 11. O-O O-O 12. Rfc1

			variation => N__('Slav: Exchange, 8. Bd3 Main Line, 12.Rfc1'),
			moves => {
				'f8c8' => 'r1r3k1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R1R3K1 w - - 4 13',
			},
		},
		'r4rk1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R4RK1 w - - 2 12' => {
			eco => 'D14k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Nc6 6. Bf4 Bf5 7. e3 e6 8. Bd3 Bxd3 9. Qxd3 Bd6 10. Bxd6 Qxd6 11. O-O O-O

			variation => N__('Slav: Exchange, 8. Bd3 Main Line'),
			moves => {
				'a1c1' => 'r4rk1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/2R2RK1 b - - 3 12',
				'f1c1' => 'r4rk1/pp3ppp/2nqpn2/3p4/3P4/2NQPN2/PP3PPP/R1R3K1 b - - 3 12',
			},
		},
		'r4rk1/pp3ppp/n1p1p3/3q4/P1N5/8/1PK1Q1PP/R4B1R w - - 1 16' => {
			eco => 'D17s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. e4 Bxe4 9. fxe4 Nxe4 10. Bd2 Qxd4 11. Nxe4 Qxe4+ 12. Qe2 Bxd2+ 13. Kxd2 Qd5+ 14. Kc2 Na6 15. Nxc4 O-O

			variation => N__('Slav: Central, Main Line, 15.Nxc4 O-O'),
			moves => {
			},
		},
		'r4rk1/ppB1ppbp/2n3p1/3q4/3P2b1/4P3/PP2NPPP/R2QKB1R w KQ - 3 10' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 Nc6 9. Ne2 Bg4

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Nc6 9. Ne2 Bg4'),
			moves => {
			},
		},
		'r4rk1/ppq1ppbp/3pbnp1/8/3BP1P1/2N2P2/PPPQ3P/1K1R1B1R b - g3 0 12' => {
			eco => 'B76n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Nxd4 10. Bxd4 Be6 11. Kb1 Qc7 12. g4

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Nxd4: 11. Kb1 Qc7 12. g4'),
			moves => {
			},
		},
		'r4rk1/ppq1ppbp/3pbnp1/8/3BP3/2N2P2/PPPQ2PP/1K1R1B1R w - - 3 12' => {
			eco => 'B76m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2 Nc6 9. O-O-O Nxd4 10. Bxd4 Be6 11. Kb1 Qc7

			variation => N__('Sicilian: Dragon, Yugoslav, 9.O-O-O Nxd4: 11. Kb1 Qc7'),
			moves => {
				'g2g4' => 'r4rk1/ppq1ppbp/3pbnp1/8/3BP1P1/2N2P2/PPPQ3P/1K1R1B1R b - g3 0 12',
			},
		},
		'rn1q1rk1/1bp1bppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11' => {
			eco => 'C95a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4 Bb7

			variation => N__('Spanish: Closed, Breyer, 10. d4 Bb7'),
			moves => {
			},
		},
		'rn1q1rk1/1bp1bppp/p3pn2/1p4B1/3P4/5NP1/PPQ1PPBP/RN3RK1 b - - 3 10' => {
			eco => 'E05r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7 10. Bg5

			variation => N__('Catalan: Open, Classical, 8.Qxc4, 10. Bg5'),
			moves => {
			},
		},
		'rn1q1rk1/1bp1bppp/p3pn2/1p6/3P1B2/5NP1/PPQ1PPBP/RN3RK1 b - - 3 10' => {
			eco => 'E05u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7 10. Bf4

			variation => N__('Catalan: Open, Classical, 8.Qxc4, 10. Bf4'),
			moves => {
				'b8c6' => 'r2q1rk1/1bp1bppp/p1n1pn2/1p6/3P1B2/5NP1/PPQ1PPBP/RN3RK1 w - - 4 11',
			},
		},
		'rn1q1rk1/1bp1bppp/p3pn2/1p6/3P4/5NP1/PPQ1PPBP/RNB2RK1 w - - 2 10' => {
			eco => 'E05q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7

			variation => N__('Catalan: Open, Classical, 8.Qxc4 b5 9. Qc2 Bb7'),
			moves => {
				'c1d2' => 'rn1q1rk1/1bp1bppp/p3pn2/1p6/3P4/5NP1/PPQBPPBP/RN3RK1 b - - 3 10',
				'c1f4' => 'rn1q1rk1/1bp1bppp/p3pn2/1p6/3P1B2/5NP1/PPQ1PPBP/RN3RK1 b - - 3 10',
				'c1g5' => 'rn1q1rk1/1bp1bppp/p3pn2/1p4B1/3P4/5NP1/PPQ1PPBP/RN3RK1 b - - 3 10',
			},
		},
		'rn1q1rk1/1bp1bppp/p3pn2/1p6/3P4/5NP1/PPQBPPBP/RN3RK1 b - - 3 10' => {
			eco => 'E05s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7 10. Bd2

			variation => N__('Catalan: Open, Classical, 8.Qxc4, 10. Bd2'),
			moves => {
				'b7e4' => 'rn1q1rk1/2p1bppp/p3pn2/1p6/3Pb3/5NP1/PPQBPPBP/RN3RK1 w - - 4 11',
			},
		},
		'rn1q1rk1/1bpp1ppp/1p2pn2/p7/1bPP4/1P3NP1/P2BPPBP/RN1Q1RK1 w - - 2 9' => {
			eco => 'E15n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb7 6. Bg2 Bb4+ 7. Bd2 a5 8. O-O O-O

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7, 7... a5 8.O-O O-O'),
			moves => {
			},
		},
		'rn1q1rk1/1bpp1ppp/1p2pn2/p7/1bPP4/5NP1/PP1BPPBP/RN1Q1RK1 w - - 2 8' => {
			eco => 'E16g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 a5 7. O-O O-O

			variation => N__('Queen\'s Indian: Capablanca, Yates, 7.O-O O-O'),
			moves => {
			},
		},
		'rn1q1rk1/1p2bpp1/p2pbn1p/4p3/4P3/1PN2N1P/P1P2PP1/R1BQRBK1 w - - 0 13' => {
			eco => 'B58u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8 12. b3 a6

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8 12. b3 a6'),
			moves => {
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/3Np3/4P3/1N2B3/PPP1BPPP/R2Q1RK1 b - - 7 10' => {
			eco => 'B92m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 Be6 10. Nd5

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Be3 Be6 10. Nd5'),
			moves => {
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 6 10' => {
			eco => 'B92k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 Be6

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Be3 Be6'),
			moves => {
				'a2a4' => 'rn1q1rk1/1p2bppp/p2pbn2/4p3/P3P3/1NN1B3/1PP1BPPP/R2Q1RK1 b - a3 0 10',
				'c3d5' => 'rn1q1rk1/1p2bppp/p2pbn2/3Np3/4P3/1N2B3/PPP1BPPP/R2Q1RK1 b - - 7 10',
				'd1d2' => 'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1B3/PPPQBPPP/R4RK1 b - - 7 10',
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1B3/PPPQBPPP/R4RK1 b - - 7 10' => {
			eco => 'B92n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 Be6 10. Qd2

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Be3 Be6 10. Qd2'),
			moves => {
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/2KR1B1R b - - 4 10' => {
			eco => 'B90q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 O-O 10. O-O-O

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 O-O 10.O-O-O'),
			moves => {
				'b8d7' => 'r2q1rk1/1p1nbppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/2KR1B1R w - - 5 11',
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R w KQ - 3 10' => {
			eco => 'B90q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 O-O

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2 O-O'),
			moves => {
				'e1c1' => 'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/2KR1B1R b - - 4 10',
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K w - - 6 10' => {
			eco => 'B92p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Kh1 Be6

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Kh1 Be6'),
			moves => {
			},
		},
		'rn1q1rk1/1p2bppp/p2pbn2/4p3/P3P3/1NN1B3/1PP1BPPP/R2Q1RK1 b - a3 0 10' => {
			eco => 'B92l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3 Be6 10. a4

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Be3 Be6 10. a4'),
			moves => {
			},
		},
		'rn1q1rk1/1p3pb1/p2p1npp/2pP2B1/P3P1b1/2N2N2/1P2BPPP/R2Q1RK1 w - - 0 12' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bg5 h6

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bg5 h6'),
			moves => {
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP2B1/P3P1b1/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bg5

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bg5'),
			moves => {
				'h7h6' => 'rn1q1rk1/1p3pb1/p2p1npp/2pP2B1/P3P1b1/2N2N2/1P2BPPP/R2Q1RK1 w - - 0 12',
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3P1b1/2N2N1P/1P2BPP1/R1BQ1RK1 b - - 0 11' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. h3

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. h3'),
			moves => {
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3P1b1/2N2N2/1P2BPPP/R1BQ1RK1 w - - 1 11' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4'),
			moves => {
				'c1f4' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3PBb1/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11',
				'c1g5' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP2B1/P3P1b1/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11',
				'f3d2' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3P1b1/2N5/1P1NBPPP/R1BQ1RK1 b - - 2 11',
				'h2h3' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3P1b1/2N2N1P/1P2BPP1/R1BQ1RK1 b - - 0 11',
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3P1b1/2N5/1P1NBPPP/R1BQ1RK1 b - - 2 11' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Nd2

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Nd2'),
			moves => {
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2Q1RK1 b - - 0 12' => {
			eco => 'A75',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Bxf3'),
			moves => {
				'd8e7' => 'rn3rk1/1p2qpbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2Q1RK1 w - - 1 13',
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3PB2/2N2b2/1P2BPPP/R2Q1RK1 w - - 0 12' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Bxf3

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Bxf3'),
			moves => {
				'e2f3' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2Q1RK1 b - - 0 12',
			},
		},
		'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3PBb1/2N2N2/1P2BPPP/R2Q1RK1 b - - 2 11' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4'),
			moves => {
				'd8e7' => 'rn3rk1/1p2qpbp/p2p1np1/2pP4/P3PBb1/2N2N2/1P2BPPP/R2Q1RK1 w - - 3 12',
				'f8e8' => 'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PBb1/2N2N2/1P2BPPP/R2Q1RK1 w - - 3 12',
				'g4f3' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3PB2/2N2b2/1P2BPPP/R2Q1RK1 w - - 0 12',
			},
		},
		'rn1q1rk1/1pp1bppp/2b1pn2/p5B1/P1QP4/5NP1/1P2PPBP/RN3RK1 w - - 0 11' => {
			eco => 'E05o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. a4 Bd7 9. Qxc4 Bc6 10. Bg5 a5

			variation => N__('Catalan: Open, Classical, 8. a4, 10. Bg5 a5'),
			moves => {
			},
		},
		'rn1q1rk1/1pp1bppp/p1b1pn2/6B1/P1QP4/5NP1/1P2PPBP/RN3RK1 b - - 2 10' => {
			eco => 'E05m',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, Classical, 8. a4 Bd7 9.Qxc4'),
			moves => {
				'a6a5' => 'rn1q1rk1/1pp1bppp/2b1pn2/p5B1/P1QP4/5NP1/1P2PPBP/RN3RK1 w - - 0 11',
				'c6d5' => 'rn1q1rk1/1pp1bppp/p3pn2/3b2B1/P1QP4/5NP1/1P2PPBP/RN3RK1 w - - 3 11',
			},
		},
		'rn1q1rk1/1pp1bppp/p1b1pn2/8/P1QP4/5NP1/1P2PPBP/RNB2RK1 w - - 1 10' => {
			eco => 'E05m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. a4 Bd7 9. Qxc4 Bc6

			variation => N__('Catalan: Open, Classical, 8. a4 Bd7 9.Qxc4'),
			moves => {
				'c1g5' => 'rn1q1rk1/1pp1bppp/p1b1pn2/6B1/P1QP4/5NP1/1P2PPBP/RN3RK1 b - - 2 10',
			},
		},
		'rn1q1rk1/1pp1bppp/p3pn2/3b2B1/P1QP4/5NP1/1P2PPBP/RN3RK1 w - - 3 11' => {
			eco => 'E05n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. a4 Bd7 9. Qxc4 Bc6 10. Bg5 Bd5

			variation => N__('Catalan: Open, Classical, 8. a4, 10. Bg5 Bd5'),
			moves => {
			},
		},
		'rn1q1rk1/1ppbbppp/p3pn2/8/P1QP4/5NP1/1P2PPBP/RNB2RK1 b - - 0 9' => {
			eco => 'E05m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. a4 Bd7 9. Qxc4

			variation => N__('Catalan: Open, Classical, 8. a4 Bd7 9.Qxc4'),
			moves => {
				'd7c6' => 'rn1q1rk1/1pp1bppp/p1b1pn2/8/P1QP4/5NP1/1P2PPBP/RNB2RK1 w - - 1 10',
			},
		},
		'rn1q1rk1/1ppbbppp/p3pn2/8/P1pP4/5NP1/1PQ1PPBP/RNB2RK1 w - - 1 9' => {
			eco => 'E05l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. a4 Bd7

			variation => N__('Catalan: Open, Classical, 8. a4 Bd7'),
			moves => {
				'c2c4' => 'rn1q1rk1/1ppbbppp/p3pn2/8/P1QP4/5NP1/1P2PPBP/RNB2RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/2p1bppp/p3pn2/1p6/3Pb3/5NP1/PPQBPPBP/RN3RK1 w - - 4 11' => {
			eco => 'E05t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4 b5 9. Qc2 Bb7 10. Bd2 Be4

			variation => N__('Catalan: Open, Classical, 8.Qxc4, 10. Bd2 Be4'),
			moves => {
			},
		},
		'rn1q1rk1/4ppbp/3p1np1/2pP4/4P3/2N2NP1/PP3PKP/R1BQ3R b - - 4 11' => {
			eco => 'A59i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3 Bg7 10. Kg2 O-O 11. Nf3

			variation => N__('Benko Gambit: 7. e4, Main Line'),
			moves => {
				'b8d7' => 'r2q1rk1/3nppbp/3p1np1/2pP4/4P3/2N2NP1/PP3PKP/R1BQ3R w - - 5 12',
			},
		},
		'rn1q1rk1/4ppbp/3p1np1/2pP4/4P3/2N3P1/PP3PKP/R1BQ2NR w - - 3 11' => {
			eco => 'A59h',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 7. e4 Line, 9. g3'),
			moves => {
				'g1f3' => 'rn1q1rk1/4ppbp/3p1np1/2pP4/4P3/2N2NP1/PP3PKP/R1BQ3R b - - 4 11',
			},
		},
		'rn1q1rk1/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 10' => {
			eco => 'A58o',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Fianchetto, 9...O-O'),
			moves => {
				'b8d7' => 'r2q1rk1/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 11',
			},
		},
		'rn1q1rk1/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 10' => {
			eco => 'A58o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2 O-O

			variation => N__('Benko Gambit: Fianchetto, 9...O-O'),
			moves => {
				'e1g1' => 'rn1q1rk1/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 10',
			},
		},
		'rn1q1rk1/p1p1bppp/1p2p3/3p4/2PP4/6P1/PP2PPNP/R1BQ1RK1 w - d6 0 11' => {
			eco => 'E18u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Ne1 Bxg2 10. Nxg2 d5

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1 Bxg2 10.Nxg2 d5'),
			moves => {
				'd1a4' => 'rn1q1rk1/p1p1bppp/1p2p3/3p4/Q1PP4/6P1/PP2PPNP/R1B2RK1 b - - 1 11',
			},
		},
		'rn1q1rk1/p1p1bppp/1p2p3/3p4/Q1PP4/6P1/PP2PPNP/R1B2RK1 b - - 1 11' => {
			eco => 'E18v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Ne1 Bxg2 10. Nxg2 d5 11. Qa4

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1 Bxg2 10.Nxg2 d5 11. Qa4'),
			moves => {
				'd5c4' => 'rn1q1rk1/p1p1bppp/1p2p3/8/Q1pP4/6P1/PP2PPNP/R1B2RK1 w - - 0 12',
				'd8d7' => 'rn3rk1/p1pqbppp/1p2p3/3p4/Q1PP4/6P1/PP2PPNP/R1B2RK1 w - - 2 12',
			},
		},
		'rn1q1rk1/p1p1bppp/1p2p3/8/Q1pP4/6P1/PP2PPNP/R1B2RK1 w - - 0 12' => {
			eco => 'E18x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Ne1 Bxg2 10. Nxg2 d5 11. Qa4 dxc4

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1 Bxg2 10.Nxg2 d5 11. Qa4 dxc4'),
			moves => {
			},
		},
		'rn1q1rk1/p1p1bppp/bp3n2/3p4/3P4/1P3NP1/P2BPPBP/RN1Q1RK1 w - - 2 10' => {
			eco => 'E15p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5 8. cxd5 exd5 9. O-O O-O

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5'),
			moves => {
			},
		},
		'rn1q1rk1/p1pp1ppp/bp2pn2/6B1/2PP4/P1Q5/1P2PPPP/R3KBNR w KQ - 2 8' => {
			eco => 'E32o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Ba6

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Ba6'),
			moves => {
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/3P4/2P1b3/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E18s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. d5

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. d5'),
			moves => {
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PP4/6P1/PP2PPNP/R1BQ1RK1 b - - 0 10' => {
			eco => 'E18u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Ne1 Bxg2 10. Nxg2

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1 Bxg2 10.Nxg2'),
			moves => {
				'd7d5' => 'rn1q1rk1/p1p1bppp/1p2p3/3p4/2PP4/6P1/PP2PPNP/R1BQ1RK1 w - d6 0 11',
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PP4/6P1/PP2PPbP/R1BQNRK1 w - - 0 10' => {
			eco => 'E18t',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1'),
			moves => {
				'e1g2' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PP4/6P1/PP2PPNP/R1BQ1RK1 b - - 0 10',
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb2N/6P1/PP2PPBP/R1BQ1RK1 b - - 1 9' => {
			eco => 'E18s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Nh4

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Nh4'),
			moves => {
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/2Q2NP1/PP2PPBP/R1B2RK1 w - - 1 10' => {
			eco => 'E19d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 Be4

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 Be4'),
			moves => {
				'f3e1' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/2Q3P1/PP2PPBP/R1B1NRK1 b - - 2 10',
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/2Q3P1/PP2PPBP/R1B1NRK1 b - - 2 10' => {
			eco => 'E19e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 Be4 10. Ne1

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 Be4 10. Ne1'),
			moves => {
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/5NP1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E18s',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4'),
			moves => {
				'c1f4' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PPbB2/5NP1/PP2PPBP/R2Q1RK1 b - - 1 9',
				'd4d5' => 'rn1q1rk1/p1ppbppp/1p2p3/3P4/2P1b3/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 9',
				'f3e1' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/6P1/PP2PPBP/R1BQNRK1 b - - 1 9',
				'f3h4' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb2N/6P1/PP2PPBP/R1BQ1RK1 b - - 1 9',
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/6P1/PP2PPBP/R1BQNRK1 b - - 1 9' => {
			eco => 'E18t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Ne1

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1'),
			moves => {
				'e4g2' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PP4/6P1/PP2PPbP/R1BQNRK1 w - - 0 10',
			},
		},
		'rn1q1rk1/p1ppbppp/1p2p3/8/2PPbB2/5NP1/PP2PPBP/R2Q1RK1 b - - 1 9' => {
			eco => 'E18s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Bf4

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Bf4'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2p3/2p5/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 b - - 0 10' => {
			eco => 'E19k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. b3

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. b3'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2p3/2p5/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 w - c6 0 10' => {
			eco => 'E19j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5'),
			moves => {
				'b2b3' => 'rn1q1rk1/pb1pbppp/1p2p3/2p5/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 b - - 0 10',
				'f1d1' => 'rn1q1rk1/pb1pbppp/1p2p3/2p5/2PP4/2Q2NP1/PP2PPBP/R1BR2K1 b - - 1 10',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2p3/2p5/2PP4/2Q2NP1/PP2PPBP/R1BR2K1 b - - 1 10' => {
			eco => 'E19l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. Rd1

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. Rd1'),
			moves => {
				'd7d6' => 'rn1q1rk1/pb2bppp/1p1pp3/2p5/2PP4/2Q2NP1/PP2PPBP/R1BR2K1 w - - 0 11',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 w - - 1 8' => {
			eco => 'A30p',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. b3 O-O

			variation => N__('English: Symmetrical, Hedgehog, 7. b3'),
			moves => {
				'c1b2' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 b - - 2 8',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 b - - 2 8' => {
			eco => 'A30p',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Hedgehog, 7. b3'),
			moves => {
				'd7d6' => 'rn1q1rk1/pb2bppp/1p1ppn2/2p5/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/1P1BPN2/P4PPP/RNBQ1RK1 w - - 1 8' => {
			eco => 'E14u',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3'),
			moves => {
				'c1b2' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/1P1BPN2/PB3PPP/RN1Q1RK1 b - - 2 8',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/1P1BPN2/PB3PPP/RN1Q1RK1 b - - 2 8' => {
			eco => 'E14u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3 O-O 8. Bb2

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3'),
			moves => {
				'c5d4' => 'rn1q1rk1/pb1pbppp/1p2pn2/8/2Pp4/1P1BPN2/PB3PPP/RN1Q1RK1 w - - 0 9',
				'd7d5' => 'rn1q1rk1/pb2bppp/1p2pn2/2pp4/2PP4/1P1BPN2/PB3PPP/RN1Q1RK1 w - d6 0 9',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - c6 0 8' => {
			eco => 'E17s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. b3 c5

			variation => N__('Queen\'s Indian: Euwe, 7... c5'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - c6 0 8' => {
			eco => 'E18b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 c5

			variation => N__('Queen\'s Indian: 7. Nc3 c5'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/2N2NP1/PPQ1PPBP/R1B1K2R w KQ c6 0 8' => {
			eco => 'E17e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 O-O 7. Qc2 c5

			variation => N__('Queen\'s Indian: 6. Nc3 O-O 7. Qc2 c5'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 4 8' => {
			eco => 'E14x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. Nc3 O-O

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. Nc3 O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/8/2PN4/1P1BP3/PB3PPP/RN1Q1RK1 b - - 0 9' => {
			eco => 'E14w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3 O-O 8. Bb2 cxd4 9. Nxd4

			variation => N__('Queen\'s Indian: Averbakh Variation'),
			moves => {
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/8/2PP4/1P1B1N2/PB3PPP/RN1Q1RK1 b - - 0 9' => {
			eco => 'E14w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3 O-O 8. Bb2 cxd4 9. exd4

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3 O-O 8. Bb2 cxd4 9.exd4'),
			moves => {
				'd7d5' => 'rn1q1rk1/pb2bppp/1p2pn2/3p4/2PP4/1P1B1N2/PB3PPP/RN1Q1RK1 w - d6 0 10',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 w - - 1 9' => {
			eco => 'A30s',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 O-O

			variation => N__('English: Symmetrical, Hedgehog, 8.Qxd4 O-O'),
			moves => {
				'f1d1' => 'rn1q1rk1/pb1pbppp/1p2pn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 b - - 2 9',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 b - - 2 9' => {
			eco => 'A30s',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Hedgehog, 8.Qxd4 O-O'),
			moves => {
				'b8c6' => 'r2q1rk1/pb1pbppp/1pn1pn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 w - - 3 10',
			},
		},
		'rn1q1rk1/pb1pbppp/1p2pn2/8/2Pp4/1P1BPN2/PB3PPP/RN1Q1RK1 w - - 0 9' => {
			eco => 'E14w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3 O-O 8. Bb2 cxd4

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3 O-O 8. Bb2 cxd4'),
			moves => {
				'e3d4' => 'rn1q1rk1/pb1pbppp/1p2pn2/8/2PP4/1P1B1N2/PB3PPP/RN1Q1RK1 b - - 0 9',
				'f3d4' => 'rn1q1rk1/pb1pbppp/1p2pn2/8/2PN4/1P1BP3/PB3PPP/RN1Q1RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/pb1pbppp/1pp2n2/3P4/7N/6P1/PP2PPBP/RNBQ1RK1 b - - 0 9' => {
			eco => 'E17w',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky, 8... c6'),
			moves => {
				'f6d5' => 'rn1q1rk1/pb1pbppp/1pp5/3n4/7N/6P1/PP2PPBP/RNBQ1RK1 w - - 0 10',
			},
		},
		'rn1q1rk1/pb1pbppp/1pp2n2/3p4/2P4N/6P1/PP2PPBP/RNBQ1RK1 w - - 0 9' => {
			eco => 'E17w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5 8. Nh4 c6

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky, 8... c6'),
			moves => {
				'c4d5' => 'rn1q1rk1/pb1pbppp/1pp2n2/3P4/7N/6P1/PP2PPBP/RNBQ1RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/pb1pbppp/1pp5/3n1N2/8/6P1/PP2PPBP/RNBQ1RK1 b - - 1 10' => {
			eco => 'E17x',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky, 8... c6 9.cxd5 Nxd5'),
			moves => {
				'd5c7' => 'rn1q1rk1/pbnpbppp/1pp5/5N2/8/6P1/PP2PPBP/RNBQ1RK1 w - - 2 11',
			},
		},
		'rn1q1rk1/pb1pbppp/1pp5/3n4/7N/6P1/PP2PPBP/RNBQ1RK1 w - - 0 10' => {
			eco => 'E17x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5 8. Nh4 c6 9. cxd5 Nxd5

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky, 8... c6 9.cxd5 Nxd5'),
			moves => {
				'h4f5' => 'rn1q1rk1/pb1pbppp/1pp5/3n1N2/8/6P1/PP2PPBP/RNBQ1RK1 b - - 1 10',
			},
		},
		'rn1q1rk1/pb1pppbp/1p3np1/2p5/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 w - - 3 8' => {
			eco => 'A30g',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O g6 6. b3 Bg7 7. Bb2 O-O

			variation => N__('English: Symmetrical, Mutual Double Fianchetto'),
			moves => {
				'b1c3' => 'rn1q1rk1/pb1pppbp/1p3np1/2p5/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 b - - 4 8',
			},
		},
		'rn1q1rk1/pb1pppbp/1p3np1/2p5/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 b - - 4 8' => {
			eco => 'A30g',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O g6 6. b3 Bg7 7. Bb2 O-O 8. Nc3

			variation => N__('English: Symmetrical, Mutual Double Fianchetto'),
			moves => {
				'd7d5' => 'rn1q1rk1/pb2ppbp/1p3np1/2pp4/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 w - d6 0 9',
			},
		},
		'rn1q1rk1/pb2bppp/1p1pp3/2p5/2PP4/1PQ2NP1/P3PPBP/R1BR2K1 b - - 0 11' => {
			eco => 'E19n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. Rd1 d6 11. b3

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. Rd1 d6 11. b3'),
			moves => {
				'e7f6' => 'rn1q1rk1/pb3ppp/1p1ppb2/2p5/2PP4/1PQ2NP1/P3PPBP/R1BR2K1 w - - 1 12',
			},
		},
		'rn1q1rk1/pb2bppp/1p1pp3/2p5/2PP4/2Q2NP1/PP2PPBP/R1BR2K1 w - - 0 11' => {
			eco => 'E19m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. Rd1 d6

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. Rd1 d6'),
			moves => {
				'b2b3' => 'rn1q1rk1/pb2bppp/1p1pp3/2p5/2PP4/1PQ2NP1/P3PPBP/R1BR2K1 b - - 0 11',
			},
		},
		'rn1q1rk1/pb2bppp/1p1ppn2/2p5/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 w - - 0 9' => {
			eco => 'A30p',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. b3 O-O 8. Bb2 d6

			variation => N__('English: Symmetrical, Hedgehog, 7. b3'),
			moves => {
			},
		},
		'rn1q1rk1/pb2bppp/1p2pn2/2pp4/2PP4/1P1BPN2/PB3PPP/RN1Q1RK1 w - d6 0 9' => {
			eco => 'E14v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3 O-O 8. Bb2 d5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3 O-O 8. Bb2 d5'),
			moves => {
			},
		},
		'rn1q1rk1/pb2bppp/1p2pn2/3p4/2PP4/1P1B1N2/PB3PPP/RN1Q1RK1 w - d6 0 10' => {
			eco => 'E14w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3 O-O 8. Bb2 cxd4 9. exd4 d5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3 O-O 8. Bb2 cxd4 9.exd4 d5'),
			moves => {
			},
		},
		'rn1q1rk1/pb2bppp/1pp1pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E18h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. Ne5 c6

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8. Ne5 c6'),
			moves => {
			},
		},
		'rn1q1rk1/pb2ppbp/1p1p1np1/2p5/2PPP3/2NBBP2/PP2N1PP/R2QK2R w KQ c6 0 9' => {
			eco => 'E82g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 Bb7 8. Nge2 c5

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 Bb7 8.Nge2 c5'),
			moves => {
				'd4d5' => 'rn1q1rk1/pb2ppbp/1p1p1np1/2pP4/2P1P3/2NBBP2/PP2N1PP/R2QK2R b KQ - 0 9',
			},
		},
		'rn1q1rk1/pb2ppbp/1p1p1np1/2pP4/2P1P3/2NBBP2/PP2N1PP/R2QK2R b KQ - 0 9' => {
			eco => 'E82g',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 Bb7 8.Nge2 c5'),
			moves => {
				'e7e6' => 'rn1q1rk1/pb3pbp/1p1ppnp1/2pP4/2P1P3/2NBBP2/PP2N1PP/R2QK2R w KQ - 0 10',
			},
		},
		'rn1q1rk1/pb2ppbp/1p3np1/2pp4/2P5/1PN2NP1/PB1PPPBP/R2Q1RK1 w - d6 0 9' => {
			eco => 'A30g',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O g6 6. b3 Bg7 7. Bb2 O-O 8. Nc3 d5

			variation => N__('English: Symmetrical, Mutual Double Fianchetto with ...d5'),
			moves => {
			},
		},
		'rn1q1rk1/pb3pbp/1p1ppnp1/2pP4/2P1P3/2NBBP2/PP2N1PP/R2QK2R w KQ - 0 10' => {
			eco => 'E82g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 Bb7 8. Nge2 c5 9. d5 e6

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 Bb7 8.Nge2 c5 9. d5 e6'),
			moves => {
			},
		},
		'rn1q1rk1/pb3pp1/1p3b1p/2pp4/1P1P4/2N1PN2/P3BPPP/R2QK2R w KQ c6 0 12' => {
			eco => 'D58n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5 11. b4 c5

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5 11. b4 c5'),
			moves => {
			},
		},
		'rn1q1rk1/pb3pp1/1pp2b1p/3p4/1P1P4/2N1PN2/P3BPPP/R2Q1RK1 b - - 1 12' => {
			eco => 'D58o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5 11. b4 c6 12. O-O

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5 11. b4 c6 12.O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pb3pp1/1pp2b1p/3p4/1P1P4/2N1PN2/P3BPPP/R2QK2R w KQ - 0 12' => {
			eco => 'D58o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5 11. b4 c6

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5 11. b4 c6'),
			moves => {
				'e1g1' => 'rn1q1rk1/pb3pp1/1pp2b1p/3p4/1P1P4/2N1PN2/P3BPPP/R2Q1RK1 b - - 1 12',
			},
		},
		'rn1q1rk1/pb3ppp/1p1ppb2/2p5/2PP4/1PQ2NP1/P3PPBP/R1BR2K1 w - - 1 12' => {
			eco => 'E19o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. Rd1 d6 11. b3 Bf6

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. Rd1 d6 11. b3 Bf6'),
			moves => {
				'c1b2' => 'rn1q1rk1/pb3ppp/1p1ppb2/2p5/2PP4/1PQ2NP1/PB2PPBP/R2R2K1 b - - 2 12',
			},
		},
		'rn1q1rk1/pb3ppp/1p1ppb2/2p5/2PP4/1PQ2NP1/PB2PPBP/R2R2K1 b - - 2 12' => {
			eco => 'E19o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. Rd1 d6 11. b3 Bf6 12. Bb2

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. Rd1 d6 11. b3 Bf6'),
			moves => {
				'd8e7' => 'rn3rk1/pb2qppp/1p1ppb2/2p5/2PP4/1PQ2NP1/PB2PPBP/R2R2K1 w - - 3 13',
			},
		},
		'rn1q1rk1/pb3ppp/1p2pn2/4N1B1/1bBP4/2N5/PP3PPP/R2Q1RK1 b - - 3 11' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6 10. Bg5 Bb7 11. Ne5

			variation => N__('Nimzo-Indian: Main Line, Karpov, 10. Bg5 Bb7 11. Ne5'),
			moves => {
			},
		},
		'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP2QPPP/R4RK1 b - - 3 11' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6 10. Bg5 Bb7 11. Qe2

			variation => N__('Nimzo-Indian: Main Line, Karpov, 10. Bg5 Bb7 11. Qe2'),
			moves => {
			},
		},
		'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/2RQ1RK1 b - - 3 11' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6 10. Bg5 Bb7 11. Rc1

			variation => N__('Nimzo-Indian: Main Line, Karpov, 10. Bg5 Bb7 11. Rc1'),
			moves => {
			},
		},
		'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/R2Q1RK1 w - - 2 11' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6 10. Bg5 Bb7

			variation => N__('Nimzo-Indian: Main Line, Karpov, 10. Bg5 Bb7'),
			moves => {
				'a1c1' => 'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/2RQ1RK1 b - - 3 11',
				'd1e2' => 'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP2QPPP/R4RK1 b - - 3 11',
				'f1e1' => 'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/R2QR1K1 b - - 3 11',
				'f3e5' => 'rn1q1rk1/pb3ppp/1p2pn2/4N1B1/1bBP4/2N5/PP3PPP/R2Q1RK1 b - - 3 11',
			},
		},
		'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/R2QR1K1 b - - 3 11' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6 10. Bg5 Bb7 11. Re1

			variation => N__('Nimzo-Indian: Main Line, Karpov, 10. Bg5 Bb7 11. Re1'),
			moves => {
			},
		},
		'rn1q1rk1/pbnpbppp/1pp5/5N2/4P3/6P1/PP3PBP/RNBQ1RK1 b - e3 0 11' => {
			eco => 'E17z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5 8. Nh4 c6 9. cxd5 Nxd5 10. Nf5 Nc7 11. e4

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky, 8... c6 9.cxd5 Nxd5 10. Nf5 Nc7 11. e4'),
			moves => {
			},
		},
		'rn1q1rk1/pbnpbppp/1pp5/5N2/8/6P1/PP2PPBP/RNBQ1RK1 w - - 2 11' => {
			eco => 'E17y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5 8. Nh4 c6 9. cxd5 Nxd5 10. Nf5 Nc7

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky, 8... c6 9.cxd5 Nxd5 10. Nf5 Nc7'),
			moves => {
				'e2e4' => 'rn1q1rk1/pbnpbppp/1pp5/5N2/4P3/6P1/PP3PBP/RNBQ1RK1 b - e3 0 11',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pB1p/3p4/2PP4/2N1PN2/PP2BPPP/R2QK2R b KQ - 0 9' => {
			eco => 'D58k',
                        # TRANSLATORS:

			variation => N__('QGD: Tartakower, 8. Be2 Bb7'),
			moves => {
				'e7f6' => 'rn1q1rk1/pbp2pp1/1p2pb1p/3p4/2PP4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 10',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pB1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R b K - 0 9' => {
			eco => 'D58r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Rc1 Bb7 9. Bxf6

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7 9.Bxf6'),
			moves => {
				'e7f6' => 'rn1q1rk1/pbp2pp1/1p2pb1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 10',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pn1p/3P4/3P3B/2N1PN2/PP3PPP/2RQKB1R b K - 0 9' => {
			eco => 'D58q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Rc1 Bb7 9. cxd5

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7 9.cxd5'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbp1bpp1/1p3n1p/3p4/3P3B/2N1PN2/PP3PPP/2RQKB1R w K - 0 10',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP2BPPP/R2QK2R w KQ - 2 9' => {
			eco => 'D58k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7

			variation => N__('QGD: Tartakower, 8. Be2 Bb7'),
			moves => {
				'h4f6' => 'rn1q1rk1/pbp1bpp1/1p2pB1p/3p4/2PP4/2N1PN2/PP2BPPP/R2QK2R b KQ - 0 9',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP3PPP/2RQKB1R w K - 2 9' => {
			eco => 'D58p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Rc1 Bb7

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7'),
			moves => {
				'c4d5' => 'rn1q1rk1/pbp1bpp1/1p2pn1p/3P4/3P3B/2N1PN2/PP3PPP/2RQKB1R b K - 0 9',
				'h4f6' => 'rn1q1rk1/pbp1bpp1/1p2pB1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R b K - 0 9',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2Q1RK1 b - - 3 9' => {
			eco => 'D58g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7 9. O-O

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7 9.O-O'),
			moves => {
				'b8d7' => 'r2q1rk1/pbpnbpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2Q1RK1 w - - 4 10',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2QK2R w KQ - 2 9' => {
			eco => 'D58f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3 Bb7

			variation => N__('QGD: Tartakower, 8. Bd3 Bb7'),
			moves => {
				'e1g1' => 'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2Q1RK1 b - - 3 9',
			},
		},
		'rn1q1rk1/pbp1bpp1/1p3n1p/3p4/3P3B/2N1PN2/PP3PPP/2RQKB1R w K - 0 10' => {
			eco => 'D58q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Rc1 Bb7 9. cxd5 exd5

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7 9.cxd5 exd5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p1pp3/8/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 b - - 0 10' => {
			eco => 'E19c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 d6 10. b3

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 d6 10. b3'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p1pp3/8/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 w - - 0 10' => {
			eco => 'E19b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 d6

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 d6'),
			moves => {
				'b2b3' => 'rn1q1rk1/pbp1bppp/1p1pp3/8/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 b - - 0 10',
			},
		},
		'rn1q1rk1/pbp1bppp/1p1ppn2/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E18c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d6

			variation => N__('Queen\'s Indian: 7. Nc3 d6'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2p3/3P4/3Pn3/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 0 9' => {
			eco => 'E18n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 d5'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbp1bppp/1p6/3p4/3Pn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - - 0 10',
			},
		},
		'rn1q1rk1/pbp1bppp/1p2p3/3p4/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - d6 0 9' => {
			eco => 'E18n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 d5

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 d5'),
			moves => {
				'c4d5' => 'rn1q1rk1/pbp1bppp/1p2p3/3P4/3Pn3/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3P4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E18e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. cxd5

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8.cxd5'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbp1bppp/1p3n2/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/1PPP4/P2BPN2/3N1PPP/R1BQ1RK1 b - b3 0 9' => {
			eco => 'E14i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2 O-O 7. O-O d5 8. a3 Be7 9. b4

			variation => N__('Queen\'s Indian: Dreev Variation'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2P5/1P2PNP1/PB1P1PBP/RN1Q1RK1 b - - 0 8' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 b6 7. Bb2 Bb7 8. e3

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 b6 7. Bb2 Bb7 8. e3'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 w - - 2 8' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 b6 7. Bb2 Bb7

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 b6 7. Bb2 Bb7'),
			moves => {
				'e2e3' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2P5/1P2PNP1/PB1P1PBP/RN1Q1RK1 b - - 0 8',
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - d6 0 8' => {
			eco => 'E17t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. b3 d5

			variation => N__('Queen\'s Indian: Euwe, 7... d5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PP1BPPBP/R2Q1RK1 w - d6 0 9' => {
			eco => 'E16n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Be7 7. Nc3 O-O 8. O-O d5

			variation => N__('Queen\'s Indian: Capablanca, Riumin, 7. Nc3 O-O 8.O-O d5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - d6 0 8' => {
			eco => 'E18e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5

			variation => N__('Queen\'s Indian: 7. Nc3 d5'),
			moves => {
				'c4d5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3P4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'f3e5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 8',
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PPQ1PPBP/R1B1K2R w KQ d6 0 8' => {
			eco => 'E17f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 O-O 7. Qc2 d5

			variation => N__('Queen\'s Indian: 6. Nc3 O-O 7. Qc2 d5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQR1K1 w - d6 0 8' => {
			eco => 'E17q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Re1 d5

			variation => N__('Queen\'s Indian: 6.O-O O-O 7. Re1 d5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/P2BPN2/1P1N1PPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E14i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2 O-O 7. O-O d5 8. a3 Be7

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2 O-O 7.O-O d5 8. a3 Be7'),
			moves => {
				'b2b4' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/1PPP4/P2BPN2/3N1PPP/R1BQ1RK1 b - b3 0 9',
			},
		},
		'rn1q1rk1/pbp1bppp/1p2pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 8' => {
			eco => 'E18g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. Ne5

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8. Ne5'),
			moves => {
				'b8a6' => 'r2q1rk1/pbp1bppp/np2pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 2 9',
				'b8d7' => 'r2q1rk1/pbpnbppp/1p2pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 2 9',
				'c7c6' => 'rn1q1rk1/pb2bppp/1pp1pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pbp1bppp/1p3n2/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E18f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 d5 8. cxd5 exd5

			variation => N__('Queen\'s Indian: 7. Nc3 d5 8.cxd5 exd5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1bppp/1p6/3p4/3Pn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - - 0 10' => {
			eco => 'E18n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 d5 9. cxd5 exd5

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 d5 9.cxd5 exd5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp1ppbp/1p1p1np1/8/2PPP3/2NBBP2/PP2N1PP/R2QK2R b KQ - 3 8' => {
			eco => 'E82g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 Bb7 8. Nge2

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 Bb7 8.Nge2'),
			moves => {
				'c7c5' => 'rn1q1rk1/pb2ppbp/1p1p1np1/2p5/2PPP3/2NBBP2/PP2N1PP/R2QK2R w KQ c6 0 9',
			},
		},
		'rn1q1rk1/pbp1ppbp/1p1p1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 2 8' => {
			eco => 'E82f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 Bb7

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 Bb7'),
			moves => {
				'g1e2' => 'rn1q1rk1/pbp1ppbp/1p1p1np1/8/2PPP3/2NBBP2/PP2N1PP/R2QK2R b KQ - 3 8',
			},
		},
		'rn1q1rk1/pbp2pp1/1p2pb1p/3P4/3P4/2N1PN2/PP2BPPP/R2QK2R b KQ - 0 10' => {
			eco => 'D58l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbp2pp1/1p3b1p/3p4/3P4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 11',
			},
		},
		'rn1q1rk1/pbp2pp1/1p2pb1p/3P4/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 10' => {
			eco => 'D58r',
                        # TRANSLATORS:

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7 9.Bxf6'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbp2pp1/1p3b1p/3p4/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 11',
			},
		},
		'rn1q1rk1/pbp2pp1/1p2pb1p/3p4/2PP4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 10' => {
			eco => 'D58k',
                        # TRANSLATORS:

			variation => N__('QGD: Tartakower, 8. Be2 Bb7'),
			moves => {
				'c4d5' => 'rn1q1rk1/pbp2pp1/1p2pb1p/3P4/3P4/2N1PN2/PP2BPPP/R2QK2R b KQ - 0 10',
			},
		},
		'rn1q1rk1/pbp2pp1/1p2pb1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 10' => {
			eco => 'D58r',
                        # TRANSLATORS:

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7 9.Bxf6'),
			moves => {
				'c4d5' => 'rn1q1rk1/pbp2pp1/1p2pb1p/3P4/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 0 10',
			},
		},
		'rn1q1rk1/pbp2pp1/1p3b1p/3p4/1P1P4/2N1PN2/P3BPPP/R2QK2R b KQ b3 0 11' => {
			eco => 'D58n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5 11. b4

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5 11. b4'),
			moves => {
				'c7c5' => 'rn1q1rk1/pb3pp1/1p3b1p/2pp4/1P1P4/2N1PN2/P3BPPP/R2QK2R w KQ c6 0 12',
				'c7c6' => 'rn1q1rk1/pb3pp1/1pp2b1p/3p4/1P1P4/2N1PN2/P3BPPP/R2QK2R w KQ - 0 12',
			},
		},
		'rn1q1rk1/pbp2pp1/1p3b1p/3p4/3P4/2N1PN2/PP2BPPP/R2Q1RK1 b - - 1 11' => {
			eco => 'D58m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5 11. O-O

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5 11.O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pbp2pp1/1p3b1p/3p4/3P4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 11' => {
			eco => 'D58l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5

			variation => N__('QGD: Tartakower, 8. Be2 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5'),
			moves => {
				'b2b4' => 'rn1q1rk1/pbp2pp1/1p3b1p/3p4/1P1P4/2N1PN2/P3BPPP/R2QK2R b KQ b3 0 11',
				'e1g1' => 'rn1q1rk1/pbp2pp1/1p3b1p/3p4/3P4/2N1PN2/PP2BPPP/R2Q1RK1 b - - 1 11',
			},
		},
		'rn1q1rk1/pbp2pp1/1p3b1p/3p4/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 0 11' => {
			eco => 'D58r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Rc1 Bb7 9. Bxf6 Bxf6 10. cxd5 exd5

			variation => N__('QGD: Tartakower, 8. Rc1 Bb7 9.Bxf6 Bxf6 10.cxd5 exd5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp2ppp/1p1b1n2/3p4/3P4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 1 10' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. cxd5 exd5 9. a3 Bd6

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8.cxd5 exd5 9. a3 Bd6'),
			moves => {
			},
		},
		'rn1q1rk1/pbp2ppp/1p1bpn2/3p4/2PP4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. a3 Bd6

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8. a3 Bd6'),
			moves => {
			},
		},
		'rn1q1rk1/pbp2ppp/1p1ppn2/6B1/2PP4/P1Q1P3/1P3PPP/R3KBNR w KQ - 0 9' => {
			eco => 'E32t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Bb7 8. e3 d6

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Bb7 8. e3 d6'),
			moves => {
			},
		},
		'rn1q1rk1/pbp2ppp/1p2pn2/3P4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. cxd5

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8.cxd5'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbp2ppp/1p3n2/3p4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7'),
			moves => {
				'a2a3' => 'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 8',
				'c4d5' => 'rn1q1rk1/pbp2ppp/1p2pn2/3P4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/3BPN2/PP1N1PPP/R1BQ1RK1 w - d6 0 8' => {
			eco => 'E14i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2 O-O 7. O-O d5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2 O-O 7.O-O d5'),
			moves => {
				'a2a3' => 'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/P2BPN2/1P1N1PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/5NP1/PP1NPPBP/R1BQ1RK1 w - d6 0 8' => {
			eco => 'E16d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Nbd2 O-O 7. O-O d5

			variation => N__('Queen\'s Indian: Capablanca, 6.Nbd2 O-O 7.O-O d5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. a3

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8. a3'),
			moves => {
				'b4d6' => 'rn1q1rk1/pbp2ppp/1p1bpn2/3p4/2PP4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/P2BPN2/1P1N1PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E14i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2 O-O 7. O-O d5 8. a3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2 O-O 7.O-O d5 8. a3'),
			moves => {
				'b4e7' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/P2BPN2/1P1N1PPP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rn1q1rk1/pbp2ppp/1p3n2/3p4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. cxd5 exd5

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8.cxd5 exd5'),
			moves => {
				'a2a3' => 'rn1q1rk1/pbp2ppp/1p3n2/3p4/1b1P4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 9',
				'f3e5' => 'rn1q1rk1/pbp2ppp/1p3n2/3pN3/1b1P4/2NBP3/PP3PPP/R1BQ1RK1 b - - 1 9',
			},
		},
		'rn1q1rk1/pbp2ppp/1p3n2/3p4/1b1P4/P1NBPN2/1P3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. cxd5 exd5 9. a3

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8.cxd5 exd5 9. a3'),
			moves => {
				'b4d6' => 'rn1q1rk1/pbp2ppp/1p1b1n2/3p4/3P4/P1NBPN2/1P3PPP/R1BQ1RK1 w - - 1 10',
			},
		},
		'rn1q1rk1/pbp2ppp/1p3n2/3pN3/1b1P4/2NBP3/PP3PPP/R1BQ1RK1 b - - 1 9' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O Bb7 8. cxd5 exd5 9. Ne5

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O Bb7 8.cxd5 exd5 9. Ne5'),
			moves => {
			},
		},
		'rn1q1rk1/pbp3pp/1p1ppb2/5p2/2PP4/1PQ2NP1/PB2PPBP/R4RK1 w - - 0 12' => {
			eco => 'E19i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 f5 10. b3 Bf6 11. Bb2 d6

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 f5 10. b3 Bf6 11. Bb2 d6'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp1pp1/1p2pn1p/6B1/2PP4/P1Q2P2/1P2P1PP/R3KBNR w KQ - 0 9' => {
			eco => 'E32r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Bb7 8. f3 h6

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Bb7 8. f3 h6'),
			moves => {
				'g5h4' => 'rn1q1rk1/pbpp1pp1/1p2pn1p/8/2PP3B/P1Q2P2/1P2P1PP/R3KBNR b KQ - 1 9',
			},
		},
		'rn1q1rk1/pbpp1pp1/1p2pn1p/8/2PP3B/P1Q2P2/1P2P1PP/R3KBNR b KQ - 1 9' => {
			eco => 'E32r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Bb7 8. f3 h6 9. Bh4

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Bb7 8. f3 h6 9. Bh4'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pb2/8/2PPn3/2N2NP1/PP1BPPBP/2RQ1RK1 b - - 10 9' => {
			eco => 'E18r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 Bf6 9. Rc1

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 Bf6 9. Rc1'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pb2/8/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - - 9 9' => {
			eco => 'E18q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 Bf6

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 Bf6'),
			moves => {
				'a1c1' => 'rn1q1rk1/pbpp1ppp/1p2pb2/8/2PPn3/2N2NP1/PP1BPPBP/2RQ1RK1 b - - 10 9',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/6B1/2PP4/P1Q1P3/1P3PPP/R3KBNR b KQ - 0 8' => {
			eco => 'E32s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Bb7 8. e3

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Bb7 8. e3'),
			moves => {
				'd7d6' => 'rn1q1rk1/pbp2ppp/1p1ppn2/6B1/2PP4/P1Q1P3/1P3PPP/R3KBNR w KQ - 0 9',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/6B1/2PP4/P1Q2P2/1P2P1PP/R3KBNR b KQ - 0 8' => {
			eco => 'E32q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Bb7 8. f3

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Bb7 8. f3'),
			moves => {
				'h7h6' => 'rn1q1rk1/pbpp1pp1/1p2pn1p/6B1/2PP4/P1Q2P2/1P2P1PP/R3KBNR w KQ - 0 9',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/6B1/2PP4/P1Q5/1P2PPPP/R3KBNR w KQ - 2 8' => {
			eco => 'E32p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5 Bb7

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5 Bb7'),
			moves => {
				'e2e3' => 'rn1q1rk1/pbpp1ppp/1p2pn2/6B1/2PP4/P1Q1P3/1P3PPP/R3KBNR b KQ - 0 8',
				'f2f3' => 'rn1q1rk1/pbpp1ppp/1p2pn2/6B1/2PP4/P1Q2P2/1P2P1PP/R3KBNR b KQ - 0 8',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 5 7' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7 6. Bd3 O-O 7. O-O

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3 Bb7 6. Bd3 O-O 7.O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 4 7' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7 6. Bd3 O-O

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3 Bb7 6. Bd3 O-O'),
			moves => {
				'e1g1' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 5 7',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP1N1PPP/R1BQ1RK1 b - - 6 7' => {
			eco => 'E14h',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2 O-O'),
			moves => {
				'd7d5' => 'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/3BPN2/PP1N1PPP/R1BQ1RK1 w - d6 0 8',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP1N1PPP/R1BQK2R w KQ - 5 7' => {
			eco => 'E14h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2 O-O

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2 O-O'),
			moves => {
				'e1g1' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP1N1PPP/R1BQ1RK1 b - - 6 7',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 6 7' => {
			eco => 'E16c',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Capablanca, 6.Nbd2 O-O'),
			moves => {
				'd7d5' => 'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/5NP1/PP1NPPBP/R1BQ1RK1 w - d6 0 8',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1NPPBP/R1BQK2R w KQ - 5 7' => {
			eco => 'E16c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Nbd2 O-O

			variation => N__('Queen\'s Indian: Capablanca, 6.Nbd2 O-O'),
			moves => {
				'e1g1' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 6 7',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/2PP4/2N2NP1/PP1QPPBP/R3K2R b KQ - 2 8' => {
			eco => 'E16r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Bxd2+ 7. Qxd2 O-O 8. Nc3

			variation => N__('Queen\'s Indian: Capablanca, 6...Bxd2+ 7.Qxd2 O-O 8. Nc3'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN2K2R w KQ - 1 8' => {
			eco => 'E16q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Bxd2+ 7. Qxd2 O-O

			variation => N__('Queen\'s Indian: Capablanca, 6...Bxd2+ 7.Qxd2 O-O'),
			moves => {
				'b1c3' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/2PP4/2N2NP1/PP1QPPBP/R3K2R b KQ - 2 8',
				'e1g1' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN3RK1 b - - 2 8',
			},
		},
		'rn1q1rk1/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN3RK1 b - - 2 8' => {
			eco => 'E16q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Bxd2+ 7. Qxd2 O-O 8. O-O

			variation => N__('Queen\'s Indian: Capablanca, 6...Bxd2+ 7.Qxd2 O-O 8.O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp2pp/1p2pb2/3P1p2/2P1n3/2N2NP1/PP1BPPBP/2RQ1RK1 b - - 2 10' => {
			eco => 'E18p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 f5 9. d5 Bf6 10. Rc1

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 f5 9. d5 Bf6 10. Rc1'),
			moves => {
			},
		},
		'rn1q1rk1/pbpp2pp/1p2pb2/3P1p2/2P1n3/2N2NP1/PP1BPPBP/R2Q1RK1 w - - 1 10' => {
			eco => 'E18p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 f5 9. d5 Bf6

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 f5 9. d5 Bf6'),
			moves => {
				'a1c1' => 'rn1q1rk1/pbpp2pp/1p2pb2/3P1p2/2P1n3/2N2NP1/PP1BPPBP/2RQ1RK1 b - - 2 10',
			},
		},
		'rn1q1rk1/pbpp2pp/1p2pb2/5p2/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 w - - 1 11' => {
			eco => 'E19h',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 f5 10. b3'),
			moves => {
				'c1b2' => 'rn1q1rk1/pbpp2pp/1p2pb2/5p2/2PP4/1PQ2NP1/PB2PPBP/R4RK1 b - - 2 11',
			},
		},
		'rn1q1rk1/pbpp2pp/1p2pb2/5p2/2PP4/1PQ2NP1/PB2PPBP/R4RK1 b - - 2 11' => {
			eco => 'E19h',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 f5 10. b3'),
			moves => {
				'd7d6' => 'rn1q1rk1/pbp3pp/1p1ppb2/5p2/2PP4/1PQ2NP1/PB2PPBP/R4RK1 w - - 0 12',
			},
		},
		'rn1q1rk1/pbppb1pp/1p2p3/3P1p2/2P1n3/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 0 9' => {
			eco => 'E18p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 f5 9. d5

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 f5 9. d5'),
			moves => {
				'e7f6' => 'rn1q1rk1/pbpp2pp/1p2pb2/3P1p2/2P1n3/2N2NP1/PP1BPPBP/R2Q1RK1 w - - 1 10',
			},
		},
		'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 b - - 0 10' => {
			eco => 'E19h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 f5 10. b3

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 f5 10. b3'),
			moves => {
				'e7f6' => 'rn1q1rk1/pbpp2pp/1p2pb2/5p2/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 w - - 1 11',
			},
		},
		'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 w - f6 0 10' => {
			eco => 'E19f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 f5

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 f5'),
			moves => {
				'b2b3' => 'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PP4/1PQ2NP1/P3PPBP/R1B2RK1 b - - 0 10',
				'f1d1' => 'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PP4/2Q2NP1/PP2PPBP/R1BR2K1 b - - 1 10',
			},
		},
		'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PP4/2Q2NP1/PP2PPBP/R1BR2K1 b - - 1 10' => {
			eco => 'E19g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 f5 10. Rd1

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 f5 10. Rd1'),
			moves => {
			},
		},
		'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - f6 0 9' => {
			eco => 'E18o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2 f5

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2 f5'),
			moves => {
				'd4d5' => 'rn1q1rk1/pbppb1pp/1p2p3/3P1p2/2P1n3/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PP4/2P2NP1/P1Q1PPBP/R1B2RK1 b - - 0 9' => {
			eco => 'E18z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. bxc3

			variation => N__('Queen\'s Indian: Old Main Line, 8. Qc2 Nxc3 9.bxc3'),
			moves => {
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 b - - 0 9' => {
			eco => 'E19a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3'),
			moves => {
				'b7e4' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/2Q2NP1/PP2PPBP/R1B2RK1 w - - 1 10',
				'c7c5' => 'rn1q1rk1/pb1pbppp/1p2p3/2p5/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 w - c6 0 10',
				'd7d6' => 'rn1q1rk1/pbp1bppp/1p1pp3/8/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 w - - 0 10',
				'f7f5' => 'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 w - f6 0 10',
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PP4/2n2NP1/PPQ1PPBP/R1B2RK1 w - - 0 9' => {
			eco => 'E18y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3

			variation => N__('Queen\'s Indian: Old Main Line, 8. Qc2 Nxc3'),
			moves => {
				'b2c3' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PP4/2P2NP1/P1Q1PPBP/R1B2RK1 b - - 0 9',
				'c2c3' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PP4/2Q2NP1/PP2PPBP/R1B2RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PPN3/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E18s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4'),
			moves => {
				'b7e4' => 'rn1q1rk1/p1ppbppp/1p2p3/8/2PPb3/5NP1/PP2PPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 8 8' => {
			eco => 'E18m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Bd2

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8. Bd2'),
			moves => {
				'd7d5' => 'rn1q1rk1/pbp1bppp/1p2p3/3p4/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - d6 0 9',
				'e7f6' => 'rn1q1rk1/pbpp1ppp/1p2pb2/8/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - - 9 9',
				'f7f5' => 'rn1q1rk1/pbppb1pp/1p2p3/5p2/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 w - f6 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP1BPPBP/R2QK2R w KQ - 7 8' => {
			eco => 'E17i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 Ne4 7. Bd2 O-O

			variation => N__('Queen\'s Indian: Opovcensky, 7...O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 7 8' => {
			eco => 'E18l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4'),
			moves => {
				'c1d2' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 8 8',
				'c3e4' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PPN3/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'd1c2' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 8 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 8 8' => {
			eco => 'E18y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2

			variation => N__('Queen\'s Indian: Old Main Line, 8. Qc2'),
			moves => {
				'e4c3' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PP4/2n2NP1/PPQ1PPBP/R1B2RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/3P4/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'E17u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5

			variation => N__('Queen\'s Indian: Pomar Variation'),
			moves => {
				'e6d5' => 'rn1q1rk1/pbppbppp/1p3n2/3p4/2P5/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 5 7' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
				'f1e1' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQR1K1 b - - 6 7',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQR1K1 b - - 6 7' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Re1

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'E17r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. b3

			variation => N__('Queen\'s Indian: Euwe Variation'),
			moves => {
				'c7c5' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - c6 0 8',
				'd7d5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - d6 0 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 8 8' => {
			eco => 'E16m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Be7 7. Nc3 O-O 8. O-O

			variation => N__('Queen\'s Indian: Capablanca, Riumin, 7. Nc3 O-O 8.O-O'),
			moves => {
				'd7d5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PP1BPPBP/R2Q1RK1 w - d6 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP1BPPBP/R2QK2R w KQ - 7 8' => {
			eco => 'E16l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Be7 7. Nc3 O-O

			variation => N__('Queen\'s Indian: Capablanca, Riumin, 7. Nc3 O-O'),
			moves => {
				'e1g1' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP1BPPBP/R2Q1RK1 b - - 8 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 6 7' => {
			eco => 'E18a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3

			variation => N__('Queen\'s Indian: 7. Nc3'),
			moves => {
				'b8a6' => 'r2q1rk1/pbppbppp/np2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 7 8',
				'c7c5' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - c6 0 8',
				'd7d5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - d6 0 8',
				'd7d6' => 'rn1q1rk1/pbp1bppp/1p1ppn2/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'f6e4' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 7 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 5 7' => {
			eco => 'E17c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 O-O

			variation => N__('Queen\'s Indian: 6. Nc3 O-O'),
			moves => {
				'd1c2' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PPQ1PPBP/R1B1K2R b KQ - 6 7',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PPQ1PPBP/R1B1K2R b KQ - 6 7' => {
			eco => 'E17d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 O-O 7. Qc2

			variation => N__('Queen\'s Indian: 6. Nc3 O-O 7. Qc2'),
			moves => {
				'c7c5' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/2N2NP1/PPQ1PPBP/R1B1K2R w KQ c6 0 8',
				'd7d5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PPQ1PPBP/R1B1K2R w KQ d6 0 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 6 7' => {
			eco => 'E14l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Be7 6. O-O O-O 7. Nc3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Be7 6.O-O O-O 7. Nc3'),
			moves => {
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQ1RK1 w - - 5 7' => {
			eco => 'E14k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Be7 6. O-O O-O

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Be7 6.O-O O-O'),
			moves => {
				'b1c3' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 6 7',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 5 7' => {
			eco => 'E17n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O

			variation => N__('Queen\'s Indian: 6.O-O O-O'),
			moves => {
				'b1c3' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 6 7',
				'b2b3' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
				'd1c2' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 6 7',
				'd4d5' => 'rn1q1rk1/pbppbppp/1p2pn2/3P4/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
				'f1e1' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQR1K1 b - - 6 7',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQR1K1 b - - 6 7' => {
			eco => 'E17p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Re1

			variation => N__('Queen\'s Indian: 6.O-O O-O 7. Re1'),
			moves => {
				'd7d5' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQR1K1 w - d6 0 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 6 7' => {
			eco => 'E17o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Qc2

			variation => N__('Queen\'s Indian: 6.O-O O-O 7. Qc2'),
			moves => {
			},
		},
		'rn1q1rk1/pbppbppp/1p3n2/3p4/2P4N/6P1/PP2PPBP/RNBQ1RK1 b - - 1 8' => {
			eco => 'E17w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5 8. Nh4

			variation => N__('Queen\'s Indian: Pomar, Polugaevsky Variation'),
			moves => {
				'c7c6' => 'rn1q1rk1/pb1pbppp/1pp2n2/3p4/2P4N/6P1/PP2PPBP/RNBQ1RK1 w - - 0 9',
			},
		},
		'rn1q1rk1/pbppbppp/1p3n2/3p4/2P5/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'E17u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5

			variation => N__('Queen\'s Indian: Pomar, 7...exd5'),
			moves => {
				'f3d4' => 'rn1q1rk1/pbppbppp/1p3n2/3p4/2PN4/6P1/PP2PPBP/RNBQ1RK1 b - - 1 8',
				'f3h4' => 'rn1q1rk1/pbppbppp/1p3n2/3p4/2P4N/6P1/PP2PPBP/RNBQ1RK1 b - - 1 8',
			},
		},
		'rn1q1rk1/pbppbppp/1p3n2/3p4/2PN4/6P1/PP2PPBP/RNBQ1RK1 b - - 1 8' => {
			eco => 'E17v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. d5 exd5 8. Nd4

			variation => N__('Queen\'s Indian: Pomar, Taimanov Variation'),
			moves => {
			},
		},
		'rn1q1rk1/pp1b1pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 3 10' => {
			eco => 'A70l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 Bd7

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 Bd7'),
			moves => {
			},
		},
		'rn1q1rk1/pp1b1pbp/3p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 1 10' => {
			eco => 'E89c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 Bd7

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 9... Bd7'),
			moves => {
			},
		},
		'rn1q1rk1/pp1b1ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O Bd7

			variation => N__('Nimzo-Indian: Main Line, 7... Bd7'),
			moves => {
			},
		},
		'rn1q1rk1/pp1bb1pp/2p1pn2/3p1p2/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 w - - 2 9' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Bd7

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Bd7'),
			moves => {
			},
		},
		'rn1q1rk1/pp1bp1bp/2pp1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 Bd7

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 Bd7'),
			moves => {
			},
		},
		'rn1q1rk1/pp1nppbp/6p1/2p5/3PP1b1/1QN1BN2/PP3PPP/R3KB1R w KQ c6 0 10' => {
			eco => 'D99b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3 c5

			variation => N__('Grünfeld: Russian, Smyslov, Yugoslav'),
			moves => {
			},
		},
		'rn1q1rk1/pp2bpp1/3pbn1p/4p3/4P3/1PN2N1P/P1P2PP1/R1BQRBK1 b - - 0 12' => {
			eco => 'B58t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8 12. b3

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8 12. b3'),
			moves => {
				'a7a6' => 'rn1q1rk1/1p2bpp1/p2pbn1p/4p3/4P3/1PN2N1P/P1P2PP1/R1BQRBK1 w - - 0 13',
			},
		},
		'rn1q1rk1/pp2bpp1/3pbn1p/4p3/4P3/2N2N1P/PPP2PP1/R1BQRBK1 w - - 3 12' => {
			eco => 'B58r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8'),
			moves => {
				'a2a4' => 'rn1q1rk1/pp2bpp1/3pbn1p/4p3/P3P3/2N2N1P/1PP2PP1/R1BQRBK1 b - a3 0 12',
				'b2b3' => 'rn1q1rk1/pp2bpp1/3pbn1p/4p3/4P3/1PN2N1P/P1P2PP1/R1BQRBK1 b - - 0 12',
			},
		},
		'rn1q1rk1/pp2bpp1/3pbn1p/4p3/P3P3/2N2N1P/1PP2PP1/R1BQRBK1 b - a3 0 12' => {
			eco => 'B58s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Nc6 6. Be2 e5 7. Nf3 h6 8. O-O Be7 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8 12. a4

			variation => N__('Sicilian: Boleslavsky, 7. Nf3, 9. Re1 O-O 10. h3 Be6 11. Bf1 Nb8 12. a4'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/2p2np1/3p1b2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8' => {
			eco => 'D78p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. b3 Bf5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. b3 Bf5'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/2p2np1/3p1b2/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Bd3 c6 7. O-O Bf5

			variation => N__('Grünfeld: Flohr Defence'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/2p2np1/3p1b2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D78n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Nbd2 Bf5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.Nbd2 Bf5'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/2p2np1/3p4/2PP2b1/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Bd3 c6 7. O-O Bg4

			variation => N__('Grünfeld: Smyslov Defence'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/2pp1np1/5b2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E62g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O Bf5 8. b3

			variation => N__('King\'s Indian: Fianchetto, Larsen System, 8. b3'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/2pp1np1/5b2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E62f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O Bf5

			variation => N__('King\'s Indian: Fianchetto, Larsen System'),
			moves => {
				'b2b3' => 'rn1q1rk1/pp2ppbp/2pp1np1/5b2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rn1q1rk1/pp2ppbp/3p1np1/2p5/2PPPPb1/2N2N2/PP2B1PP/R1BQK2R w KQ - 2 8' => {
			eco => 'E78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 Bg4

			variation => N__('King\'s Indian: Four Pawns Attack, 7. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1q1rk1/pp2ppbp/4bnp1/2Pp4/2P2B2/2N1P3/PP3PPP/2RQKBNR w K - 1 8' => {
			eco => 'D83',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. Rc1 c5 7. dxc5 Be6

			variation => N__('Grünfeld: Grünfeld Gambit, Botvinnik Variation'),
			moves => {
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pP2B1/4P1b1/2N2N2/PP2BPPP/R2Q1RK1 b - - 7 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Bg4 10. Bg5

			variation => N__('Benoni: Classical, 9.O-O Bg4 10. Bg5'),
			moves => {
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pP4/4P1b1/2N2N1P/PP2BPP1/R1BQ1RK1 b - - 0 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Bg4 10. h3

			variation => N__('Benoni: Classical, 9.O-O Bg4 10. h3'),
			moves => {
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pP4/4P1b1/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Bg4

			variation => N__('Benoni: Classical, 9.O-O Bg4'),
			moves => {
				'c1f4' => 'rn1q1rk1/pp3pbp/3p1np1/2pP4/4PBb1/2N2N2/PP2BPPP/R2Q1RK1 b - - 7 10',
				'c1g5' => 'rn1q1rk1/pp3pbp/3p1np1/2pP2B1/4P1b1/2N2N2/PP2BPPP/R2Q1RK1 b - - 7 10',
				'h2h3' => 'rn1q1rk1/pp3pbp/3p1np1/2pP4/4P1b1/2N2N1P/PP2BPP1/R1BQ1RK1 b - - 0 10',
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pP4/4PBb1/2N2N2/PP2BPPP/R2Q1RK1 b - - 7 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Bg4 10. Bf4

			variation => N__('Benoni: Classical, 9.O-O Bg4 10. Bf4'),
			moves => {
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pP4/4PPb1/2N2N2/PP2B1PP/R1BQ1RK1 b - - 6 10' => {
			eco => 'A68j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4 10. O-O

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10.O-O'),
			moves => {
				'b8d7' => 'r2q1rk1/pp1n1pbp/3p1np1/2pP4/4PPb1/2N2N2/PP2B1PP/R1BQ1RK1 w - - 7 11',
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pP4/4PPb1/2N2N2/PP2B1PP/R1BQK2R w KQ - 5 10' => {
			eco => 'A68h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4'),
			moves => {
				'e1g1' => 'rn1q1rk1/pp3pbp/3p1np1/2pP4/4PPb1/2N2N2/PP2B1PP/R1BQ1RK1 b - - 6 10',
				'e4e5' => 'rn1q1rk1/pp3pbp/3p1np1/2pPP3/5Pb1/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 10',
			},
		},
		'rn1q1rk1/pp3pbp/3p1np1/2pPP3/5Pb1/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 10' => {
			eco => 'A68i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Bg4 10. e5

			variation => N__('Benoni: Four Pawns, 9. Be2 Bg4 10. e5'),
			moves => {
			},
		},
		'rn1q1rk1/pp3ppp/2p1p3/5b2/PbBPn1P1/2N1PN2/1P2QP1P/R1B2RK1 b - g3 0 10' => {
			eco => 'D19g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Ne4 10. g4

			variation => N__('Slav: Dutch, Sämisch Variation'),
			moves => {
			},
		},
		'rn1q1rk1/pp3ppp/2p1p3/5b2/PbBPn3/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10' => {
			eco => 'D19g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Ne4

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Ne4'),
			moves => {
				'g2g4' => 'rn1q1rk1/pp3ppp/2p1p3/5b2/PbBPn1P1/2N1PN2/1P2QP1P/R1B2RK1 b - g3 0 10',
			},
		},
		'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 b - - 4 9' => {
			eco => 'D18p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4'),
			moves => {
				'b8d7' => 'r2q1rk1/pp1n1ppp/2p1pn2/5b2/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 w - - 5 10',
				'f5g4' => 'rn1q1rk1/pp3ppp/2p1pn2/8/PbBP2bN/2N1P3/1P3PPP/R1BQ1RK1 w - - 5 10',
				'f5g6' => 'rn1q1rk1/pp3ppp/2p1pnb1/8/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 w - - 5 10',
			},
		},
		'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP4/1QN1PN2/1P3PPP/R1B2RK1 b - - 4 9' => {
			eco => 'D18o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qb3

			variation => N__('Slav: Dutch, 8...O-O 9. Qb3'),
			moves => {
			},
		},
		'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 b - - 4 9' => {
			eco => 'D19a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2'),
			moves => {
				'b8d7' => 'r2q1rk1/pp1n1ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10',
				'f5g4' => 'rn1q1rk1/pp3ppp/2p1pn2/8/PbBP2b1/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10',
				'f5g6' => 'rn1q1rk1/pp3ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10',
				'f6e4' => 'rn1q1rk1/pp3ppp/2p1p3/5b2/PbBPn3/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10',
			},
		},
		'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQ1RK1 w - - 3 9' => {
			eco => 'D18n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O

			variation => N__('Slav: Dutch, 8...O-O'),
			moves => {
				'd1b3' => 'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP4/1QN1PN2/1P3PPP/R1B2RK1 b - - 4 9',
				'd1e2' => 'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 b - - 4 9',
				'f3h4' => 'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 b - - 4 9',
			},
		},
		'rn1q1rk1/pp3ppp/2p1pn2/8/PbBP2b1/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10' => {
			eco => 'D19b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg4

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg4'),
			moves => {
			},
		},
		'rn1q1rk1/pp3ppp/2p1pn2/8/PbBP2bN/2N1P3/1P3PPP/R1BQ1RK1 w - - 5 10' => {
			eco => 'D18q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4 Bg4

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4 Bg4'),
			moves => {
			},
		},
		'rn1q1rk1/pp3ppp/2p1pnb1/4N3/PbBP4/2N1P3/1P2QPPP/R1B2RK1 b - - 6 10' => {
			eco => 'D19e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg6 10. Ne5

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Ne5'),
			moves => {
				'b8d7' => 'r2q1rk1/pp1n1ppp/2p1pnb1/4N3/PbBP4/2N1P3/1P2QPPP/R1B2RK1 w - - 7 11',
			},
		},
		'rn1q1rk1/pp3ppp/2p1pnb1/8/PbBP3N/2N1P3/1P3PPP/R1BQ1RK1 w - - 5 10' => {
			eco => 'D18p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Nh4 Bg6

			variation => N__('Slav: Dutch, 8...O-O 9. Nh4 Bg6'),
			moves => {
			},
		},
		'rn1q1rk1/pp3ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1B2RK1 w - - 5 10' => {
			eco => 'D19c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg6

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6'),
			moves => {
				'f1d1' => 'rn1q1rk1/pp3ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1BR2K1 b - - 6 10',
				'f3e5' => 'rn1q1rk1/pp3ppp/2p1pnb1/4N3/PbBP4/2N1P3/1P2QPPP/R1B2RK1 b - - 6 10',
			},
		},
		'rn1q1rk1/pp3ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1BR2K1 b - - 6 10' => {
			eco => 'D19d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O O-O 9. Qe2 Bg6 10. Rd1

			variation => N__('Slav: Dutch, 8...O-O 9. Qe2 Bg6 10. Rd1'),
			moves => {
				'b8d7' => 'r2q1rk1/pp1n1ppp/2p1pnb1/8/PbBP4/2N1PN2/1P2QPPP/R1BR2K1 w - - 7 11',
			},
		},
		'rn1q1rk1/pp4bp/2ppbnp1/5p2/2P5/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 10' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 e5 9. dxe6 Bxe6 10. b3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 e5 9.dxe6 Bxe6 10. b3'),
			moves => {
			},
		},
		'rn1q1rk1/pp4bp/2ppbnp1/5p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A88',
                        # TRANSLATORS:

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 e5'),
			moves => {
				'b2b3' => 'rn1q1rk1/pp4bp/2ppbnp1/5p2/2P5/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 10',
				'd1d3' => 'rn1q1rk1/pp4bp/2ppbnp1/5p2/2P5/2NQ1NP1/PP2PPBP/R1B2RK1 b - - 1 10',
			},
		},
		'rn1q1rk1/pp4bp/2ppbnp1/5p2/2P5/2NQ1NP1/PP2PPBP/R1B2RK1 b - - 1 10' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 e5 9. dxe6 Bxe6 10. Qd3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 e5 9.dxe6 Bxe6 10. Qd3'),
			moves => {
			},
		},
		'rn1q1rk1/ppp1b1pp/1n2pp2/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQ1RK1 w - - 0 11' => {
			eco => 'B03l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 Bf5 7. Nc3 e6 8. Nf3 Be7 9. Be2 O-O 10. O-O f6

			variation => N__('Alekhine: Four Pawns Attack, Korchnoi Variation'),
			moves => {
			},
		},
		'rn1q1rk1/ppp1bppp/1n1pp3/4P2b/2PP4/2N1BN1P/PP2BPP1/R2Q1RK1 b - - 4 10' => {
			eco => 'B05o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. h3 Bh5 9. Nc3 O-O 10. Be3

			variation => N__('Alekhine: Modern, Main Line, 8. h3 Bh5 9. Nc3 O-O 10. Be3'),
			moves => {
				'd6d5' => 'rn1q1rk1/ppp1bppp/1n2p3/3pP2b/2PP4/2N1BN1P/PP2BPP1/R2Q1RK1 w - - 0 11',
			},
		},
		'rn1q1rk1/ppp1bppp/1n1pp3/4P2b/2PP4/2N2N1P/PP2BPP1/R1BQ1RK1 w - - 3 10' => {
			eco => 'B05o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. h3 Bh5 9. Nc3 O-O

			variation => N__('Alekhine: Modern, Main Line, 8. h3 Bh5 9. Nc3 O-O'),
			moves => {
				'c1e3' => 'rn1q1rk1/ppp1bppp/1n1pp3/4P2b/2PP4/2N1BN1P/PP2BPP1/R2Q1RK1 b - - 4 10',
			},
		},
		'rn1q1rk1/ppp1bppp/1n1pp3/4P3/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 b - - 4 9' => {
			eco => 'B05s',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. Nc3 O-O 9. Be3

			variation => N__('Alekhine: Modern, Main Line, 8. Nc3 O-O 9. Be3'),
			moves => {
				'b8c6' => 'r2q1rk1/ppp1bppp/1nnpp3/4P3/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 w - - 5 10',
			},
		},
		'rn1q1rk1/ppp1bppp/1n1pp3/4P3/2PP2b1/2N2N2/PP2BPPP/R1BQ1RK1 w - - 3 9' => {
			eco => 'B05r',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, Main Line, 8. Nc3'),
			moves => {
				'c1e3' => 'rn1q1rk1/ppp1bppp/1n1pp3/4P3/2PP2b1/2N1BN2/PP2BPPP/R2Q1RK1 b - - 4 9',
			},
		},
		'rn1q1rk1/ppp1bppp/1n2p3/3pP2b/2PP4/2N1BN1P/PP2BPP1/R2Q1RK1 w - - 0 11' => {
			eco => 'B05p',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. h3 Bh5 9. Nc3 O-O 10. Be3 d5

			variation => N__('Alekhine: Modern, Main Line, 8. h3 Bh5 9. Nc3 O-O 10. Be3 d5'),
			moves => {
			},
		},
		'rn1q1rk1/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQ1RK1 b - - 5 10' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'f7f6' => 'rn1q1rk1/ppp1b1pp/1n2pp2/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQ1RK1 w - - 0 11',
			},
		},
		'rn1q1rk1/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQK2R w KQ - 4 10' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'e1g1' => 'rn1q1rk1/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQ1RK1 b - - 5 10',
			},
		},
		'rn1q1rk1/ppp1ppbp/1n4p1/8/2QPP1b1/2N1BN2/PP2BPPP/R3K2R w KQ - 5 10' => {
			eco => 'D98g',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9. Be2'),
			moves => {
				'c4d3' => 'rn1q1rk1/ppp1ppbp/1n4p1/8/3PP1b1/2NQBN2/PP2BPPP/R3K2R b KQ - 6 10',
			},
		},
		'rn1q1rk1/ppp1ppbp/1n4p1/8/3PP1b1/1QN1BN2/PP3PPP/3RKB1R b K - 6 10' => {
			eco => 'D99d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3 Nb6 10. Rd1

			variation => N__('Grünfeld: Russian, Smyslov, Main Line, 9... Nb6 10. Rd1'),
			moves => {
				'b8c6' => 'r2q1rk1/ppp1ppbp/1nn3p1/8/3PP1b1/1QN1BN2/PP3PPP/3RKB1R w K - 7 11',
				'e7e6' => 'rn1q1rk1/ppp2pbp/1n2p1p1/8/3PP1b1/1QN1BN2/PP3PPP/3RKB1R w K - 0 11',
			},
		},
		'rn1q1rk1/ppp1ppbp/1n4p1/8/3PP1b1/1QN1BN2/PP3PPP/R3KB1R w KQ - 5 10' => {
			eco => 'D99c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3 Nb6

			variation => N__('Grünfeld: Russian, Smyslov, Main Line, 9... Nb6'),
			moves => {
				'a1d1' => 'rn1q1rk1/ppp1ppbp/1n4p1/8/3PP1b1/1QN1BN2/PP3PPP/3RKB1R b K - 6 10',
			},
		},
		'rn1q1rk1/ppp1ppbp/1n4p1/8/3PP1b1/2NQBN2/PP2BPPP/R3K2R b KQ - 6 10' => {
			eco => 'D98g',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9. Be2'),
			moves => {
				'b8c6' => 'r2q1rk1/ppp1ppbp/1nn3p1/8/3PP1b1/2NQBN2/PP2BPPP/R3K2R w KQ - 7 11',
			},
		},
		'rn1q1rk1/ppp1ppbp/3p1np1/8/2PPP1b1/2N1BN2/PP2BPPP/R2QK2R b KQ - 5 7' => {
			eco => 'E91v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Bg4 7. Be3

			variation => N__('King\'s Indian: 6. Be2 Bg4 7. Be3'),
			moves => {
				'f6d7' => 'rn1q1rk1/pppnppbp/3p2p1/8/2PPP1b1/2N1BN2/PP2BPPP/R2QK2R w KQ - 6 8',
			},
		},
		'rn1q1rk1/ppp1ppbp/3p1np1/8/2PPP1b1/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7' => {
			eco => 'E91u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Bg4

			variation => N__('King\'s Indian: 6. Be2 Bg4'),
			moves => {
				'c1e3' => 'rn1q1rk1/ppp1ppbp/3p1np1/8/2PPP1b1/2N1BN2/PP2BPPP/R2QK2R b KQ - 5 7',
			},
		},
		'rn1q1rk1/ppp1ppbp/3p1np1/8/2PPPPb1/2N2N2/PP4PP/R1BQKB1R w KQ - 3 7' => {
			eco => 'E76e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 Bg4

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1q1rk1/ppp1ppbp/3p1np1/8/3PP1b1/2N1BN2/PPP1BPPP/R2Q1RK1 b - - 7 7' => {
			eco => 'B08t',
                        # TRANSLATORS:

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O Bg4'),
			moves => {
				'b8c6' => 'r2q1rk1/ppp1ppbp/2np1np1/8/3PP1b1/2N1BN2/PPP1BPPP/R2Q1RK1 w - - 8 8',
			},
		},
		'rn1q1rk1/ppp1ppbp/3p1np1/8/3PP1b1/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 6 7' => {
			eco => 'B08t',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O Bg4

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O Bg4'),
			moves => {
				'c1e3' => 'rn1q1rk1/ppp1ppbp/3p1np1/8/3PP1b1/2N1BN2/PPP1BPPP/R2Q1RK1 b - - 7 7',
			},
		},
		'rn1q1rk1/ppp1ppbp/5np1/8/2QPP1b1/2N1BN2/PP3PPP/R3KB1R b KQ - 2 8' => {
			eco => 'D98e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3'),
			moves => {
				'f6d7' => 'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP3PPP/R3KB1R w KQ - 3 9',
			},
		},
		'rn1q1rk1/ppp1ppbp/5np1/8/2QPP1b1/2N2N2/PP2BPPP/R1B1K2R b KQ - 2 8' => {
			eco => 'D98b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be2

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be2'),
			moves => {
				'b8c6' => 'r2q1rk1/ppp1ppbp/2n2np1/8/2QPP1b1/2N2N2/PP2BPPP/R1B1K2R w KQ - 3 9',
			},
		},
		'rn1q1rk1/ppp1ppbp/5np1/8/2QPP1b1/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8' => {
			eco => 'D98a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4

			variation => N__('Grünfeld: Russian, Smyslov Variation'),
			moves => {
				'c1e3' => 'rn1q1rk1/ppp1ppbp/5np1/8/2QPP1b1/2N1BN2/PP3PPP/R3KB1R b KQ - 2 8',
				'f1e2' => 'rn1q1rk1/ppp1ppbp/5np1/8/2QPP1b1/2N2N2/PP2BPPP/R1B1K2R b KQ - 2 8',
			},
		},
		'rn1q1rk1/ppp2pbp/1n2p1p1/8/3PP1b1/1QN1BN2/PP3PPP/3RKB1R w K - 0 11' => {
			eco => 'D99e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3 Nb6 10. Rd1 e6

			variation => N__('Grünfeld: Russian, Smyslov, Main Line, 9... Nb6 10. Rd1 e6'),
			moves => {
			},
		},
		'rn1q1rk1/ppp2ppp/3b4/3P4/3Pn1b1/3B1N2/PP3PPP/RNBQ1RK1 b - - 0 9' => {
			eco => 'C42q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Tarrasch Variation'),
			moves => {
				'f7f5' => 'rn1q1rk1/ppp3pp/3b4/3P1p2/3Pn1b1/3B1N2/PP3PPP/RNBQ1RK1 w - f6 0 10',
			},
		},
		'rn1q1rk1/ppp2ppp/3b4/3p4/2PPn1b1/3B1N2/PP3PPP/RNBQ1RK1 w - - 1 9' => {
			eco => 'C42q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Bd6 7. O-O O-O 8. c4 Bg4

			variation => N__('Russian Game: Classical, Tarrasch Variation'),
			moves => {
				'c4d5' => 'rn1q1rk1/ppp2ppp/3b4/3P4/3Pn1b1/3B1N2/PP3PPP/RNBQ1RK1 b - - 0 9',
			},
		},
		'rn1q1rk1/ppp2ppp/8/2bP1b2/4nPP1/5N2/PPP1Q2P/RNB1KB1R w KQ - 1 9' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Nf3 Bc5 7. Qe2 Bf5 8. g4 O-O

			variation => N__('KGD: Falkbeer, Tarrasch Variation'),
			moves => {
			},
		},
		'rn1q1rk1/ppp3pp/3b4/3P1p2/3Pn1b1/3B1N2/PP3PPP/RNBQ1RK1 w - f6 0 10' => {
			eco => 'C42q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Tarrasch Variation'),
			moves => {
				'f1e1' => 'rn1q1rk1/ppp3pp/3b4/3P1p2/3Pn1b1/3B1N2/PP3PPP/RNBQR1K1 b - - 1 10',
			},
		},
		'rn1q1rk1/ppp3pp/3b4/3P1p2/3Pn1b1/3B1N2/PP3PPP/RNBQR1K1 b - - 1 10' => {
			eco => 'C42q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Tarrasch Variation'),
			moves => {
				'd6h2' => 'rn1q1rk1/ppp3pp/8/3P1p2/3Pn1b1/3B1N2/PP3PPb/RNBQR1K1 w - - 0 11',
			},
		},
		'rn1q1rk1/ppp3pp/8/3P1p2/3Pn1b1/3B1N2/PP3PPb/RNBQR1K1 w - - 0 11' => {
			eco => 'C42q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Bd6 7. O-O O-O 8. c4 Bg4 9. cxd5 f5 10. Re1 Bxh2+

			variation => N__('Russian Game: Classical, Marshall Trap'),
			moves => {
			},
		},
		'rn1q1rk1/pppnppbp/3p2p1/8/2PPP1b1/2N1B3/PP2BPPP/R2QK1NR b KQ - 7 8' => {
			eco => 'E91y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Bg4 7. Be3 Nfd7 8. Ng1

			variation => N__('King\'s Indian: 6. Be2 Bg4 7. Be3 Nfd7 8. Ng1'),
			moves => {
			},
		},
		'rn1q1rk1/pppnppbp/3p2p1/8/2PPP1b1/2N1BN2/PP2BPPP/2RQK2R b K - 7 8' => {
			eco => 'E91x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Bg4 7. Be3 Nfd7 8. Rc1

			variation => N__('King\'s Indian: 6. Be2 Bg4 7. Be3 Nfd7 8. Rc1'),
			moves => {
			},
		},
		'rn1q1rk1/pppnppbp/3p2p1/8/2PPP1b1/2N1BN2/PP2BPPP/R2QK2R w KQ - 6 8' => {
			eco => 'E91w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 Bg4 7. Be3 Nfd7

			variation => N__('King\'s Indian: 6. Be2 Bg4 7. Be3 Nfd7'),
			moves => {
				'a1c1' => 'rn1q1rk1/pppnppbp/3p2p1/8/2PPP1b1/2N1BN2/PP2BPPP/2RQK2R b K - 7 8',
				'f3g1' => 'rn1q1rk1/pppnppbp/3p2p1/8/2PPP1b1/2N1B3/PP2BPPP/R2QK1NR b KQ - 7 8',
			},
		},
		'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP2BPPP/R3K2R b KQ - 4 9' => {
			eco => 'D98g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Be2

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9. Be2'),
			moves => {
				'd7b6' => 'rn1q1rk1/ppp1ppbp/1n4p1/8/2QPP1b1/2N1BN2/PP2BPPP/R3K2R w KQ - 5 10',
			},
		},
		'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP3PPP/2KR1B1R b - - 4 9' => {
			eco => 'D98i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. O-O-O

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9.O-O-O'),
			moves => {
			},
		},
		'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP3PPP/3RKB1R b K - 4 9' => {
			eco => 'D98j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Rd1

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7 9. Rd1'),
			moves => {
				'b8c6' => 'r2q1rk1/pppnppbp/2n3p1/8/2QPP1b1/2N1BN2/PP3PPP/3RKB1R w K - 5 10',
			},
		},
		'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP3PPP/R3KB1R w KQ - 3 9' => {
			eco => 'D98f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7

			variation => N__('Grünfeld: Russian, Smyslov, 8. Be3 Nfd7'),
			moves => {
				'a1d1' => 'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP3PPP/3RKB1R b K - 4 9',
				'c4b3' => 'rn1q1rk1/pppnppbp/6p1/8/3PP1b1/1QN1BN2/PP3PPP/R3KB1R b KQ - 4 9',
				'e1c1' => 'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP3PPP/2KR1B1R b - - 4 9',
				'f1e2' => 'rn1q1rk1/pppnppbp/6p1/8/2QPP1b1/2N1BN2/PP2BPPP/R3K2R b KQ - 4 9',
			},
		},
		'rn1q1rk1/pppnppbp/6p1/8/3PP1b1/1QN1BN2/PP3PPP/R3KB1R b KQ - 4 9' => {
			eco => 'D99a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Bg4 8. Be3 Nfd7 9. Qb3

			variation => N__('Grünfeld: Russian, Smyslov, Main Line'),
			moves => {
				'c7c5' => 'rn1q1rk1/pp1nppbp/6p1/2p5/3PP1b1/1QN1BN2/PP3PPP/R3KB1R w KQ c6 0 10',
				'd7b6' => 'rn1q1rk1/ppp1ppbp/1n4p1/8/3PP1b1/1QN1BN2/PP3PPP/R3KB1R w KQ - 5 10',
			},
		},
		'rn1qk1nr/1b3ppp/4p3/p7/1pPP4/2B1PN2/5PPP/R2QKB1R w KQkq - 0 12' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'c3b2' => 'rn1qk1nr/1b3ppp/4p3/p7/1pPP4/4PN2/1B3PPP/R2QKB1R b KQkq - 1 12',
			},
		},
		'rn1qk1nr/1b3ppp/4p3/p7/1pPP4/4PN2/1B3PPP/R2QKB1R b KQkq - 1 12' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'g8f6' => 'rn1qk2r/1b3ppp/4pn2/p7/1pPP4/4PN2/1B3PPP/R2QKB1R w KQkq - 2 13',
			},
		},
		'rn1qk1nr/1b3ppp/4p3/pp6/2PP4/2B1PN2/5PPP/R2QKB1R b KQkq - 0 11' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'b5b4' => 'rn1qk1nr/1b3ppp/4p3/p7/1pPP4/2B1PN2/5PPP/R2QKB1R w KQkq - 0 12',
			},
		},
		'rn1qk1nr/1b3ppp/4p3/pp6/2pP4/1PB1PN2/5PPP/R2QKB1R w KQkq - 1 11' => {
			eco => 'D31u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4 Bb4 6. e3 b5 7. Bd2 a5 8. axb5 Bxc3 9. Bxc3 cxb5 10. b3 Bb7

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'b3c4' => 'rn1qk1nr/1b3ppp/4p3/pp6/2PP4/2B1PN2/5PPP/R2QKB1R b KQkq - 0 11',
			},
		},
		'rn1qk1nr/pbpp1ppp/1p2p3/8/2PP4/2N5/PP1QPPPP/R3KBNR w KQkq - 2 6' => {
			eco => 'A40q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 Bxd2+'),
			moves => {
				'e2e4' => 'rn1qk1nr/pbpp1ppp/1p2p3/8/2PPP3/2N5/PP1Q1PPP/R3KBNR b KQkq e3 0 6',
			},
		},
		'rn1qk1nr/pbpp1ppp/1p2p3/8/2PPP3/2N5/PP1Q1PPP/R3KBNR b KQkq e3 0 6' => {
			eco => 'A40q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 Bxd2+'),
			moves => {
				'g8h6' => 'rn1qk2r/pbpp1ppp/1p2p2n/8/2PPP3/2N5/PP1Q1PPP/R3KBNR w KQkq - 1 7',
			},
		},
		'rn1qk1nr/pp1bbppp/8/1Bpp4/3P4/8/PPPNQPPP/R1B1K1NR w KQkq - 4 7' => {
			eco => 'C08f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Bb5+ Bd7 6. Qe2+ Be7

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5. Bb5+ Bd7 6. Qe2+ Be7'),
			moves => {
			},
		},
		'rn1qk1nr/pp2bppp/2p5/3p1b2/3P1B2/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D31g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. cxd5 exd5 5. Bf4 c6 6. e3 Bf5

			variation => N__('QGD: Alatortsev, 5. Bf4 c6 6. e3 Bf5'),
			moves => {
				'g2g4' => 'rn1qk1nr/pp2bppp/2p5/3p1b2/3P1BP1/2N1P3/PP3P1P/R2QKBNR b KQkq g3 0 7',
			},
		},
		'rn1qk1nr/pp2bppp/2p5/3p1b2/3P1BP1/2N1P3/PP3P1P/R2QKBNR b KQkq g3 0 7' => {
			eco => 'D31h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. cxd5 exd5 5. Bf4 c6 6. e3 Bf5 7. g4

			variation => N__('QGD: Alatortsev, 5. Bf4 c6 6. e3 Bf5 7. g4'),
			moves => {
			},
		},
		'rn1qk1nr/pp2bppp/3p4/2pPp3/4P1b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 4 6' => {
			eco => 'A44y',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 Be7 5. Nf3 Bg4

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 Be7 5. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1qk1nr/ppbb1ppp/4p3/1N1pP3/1P1p1P2/P7/2P3PP/R1BQKBNR w KQkq - 1 9' => {
			eco => 'C17s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5 6. b4 cxd4 7. Nb5 Bc7 8. f4 Bd7

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Nb5 Bc7 8. f4 Bd7'),
			moves => {
			},
		},
		'rn1qk1nr/ppp1ppbp/3p2p1/8/2PP2b1/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A41w',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. Nf3 Bg4 5. e3

			variation => N__('Neo-Old Indian: Modern, 3. Nc3 Bg7 4. Nf3 Bf4 5. e3'),
			moves => {
				'b8c6' => 'r2qk1nr/ppp1ppbp/2np2p1/8/2PP2b1/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rn1qk1nr/ppp1ppbp/3p2p1/8/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'A41v',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. Nf3 Bg4

			variation => N__('Neo-Old Indian: Modern, 3. Nc3 Bg7 4. Nf3 Bf4'),
			moves => {
				'e2e3' => 'rn1qk1nr/ppp1ppbp/3p2p1/8/2PP2b1/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rn1qk1nr/ppp1ppbp/3p2p1/8/2PPP1b1/5N2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'A41s',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nf3 Bg7 4. e4 Bg4

			variation => N__('Neo-Old Indian: Modern, Rossolimo Variation'),
			moves => {
			},
		},
		'rn1qk1nr/ppp2ppp/3p4/2b1P3/4P1b1/2P2N2/PP1P2PP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C30n',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 4. c3'),
			moves => {
				'd6e5' => 'rn1qk1nr/ppp2ppp/8/2b1p3/4P1b1/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rn1qk1nr/ppp2ppp/3p4/2b1p3/4PPb1/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C30n',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 4. c3'),
			moves => {
				'f4e5' => 'rn1qk1nr/ppp2ppp/3p4/2b1P3/4P1b1/2P2N2/PP1P2PP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rn1qk1nr/ppp2ppp/8/2b1p3/4P1b1/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C30n',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 4. c3'),
			moves => {
				'd1a4' => 'rn1qk1nr/ppp2ppp/8/2b1p3/Q3P1b1/2P2N2/PP1P2PP/RNB1KB1R b KQkq - 1 6',
			},
		},
		'rn1qk1nr/ppp2ppp/8/2b1p3/Q3P1b1/2P2N2/PP1P2PP/RNB1KB1R b KQkq - 1 6' => {
			eco => 'C30n',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 Bg4 5. fxe5 dxe5 6. Qa4+

			variation => N__('KGD: Classical, Marshall Attack'),
			moves => {
			},
		},
		'rn1qk2r/1b2bppp/pp1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1BR2K1 b kq - 0 10' => {
			eco => 'A30w',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Hedgehog, 9. Rd1'),
			moves => {
				'b8d7' => 'r2qk2r/1b1nbppp/pp1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1BR2K1 w kq - 1 11',
			},
		},
		'rn1qk2r/1b2bppp/pp1ppn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 w kq - 0 10' => {
			eco => 'A30w',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Hedgehog, 9. Rd1'),
			moves => {
				'b2b3' => 'rn1qk2r/1b2bppp/pp1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1BR2K1 b kq - 0 10',
			},
		},
		'rn1qk2r/1b3ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2QK2R b KQkq - 3 13' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'b8d7' => 'r2qk2r/1b1n1ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2QK2R w KQkq - 4 14',
			},
		},
		'rn1qk2r/1b3ppp/4pn2/p7/1pPP4/4PN2/1B3PPP/R2QKB1R w KQkq - 2 13' => {
			eco => 'D31u',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams, Main Line'),
			moves => {
				'f1d3' => 'rn1qk2r/1b3ppp/4pn2/p7/1pPP4/3BPN2/1B3PPP/R2QK2R b KQkq - 3 13',
			},
		},
		'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/1P3NP1/P2BPPBP/RN1Q1RK1 b kq - 1 8' => {
			eco => 'E15n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7, 7... a5'),
			moves => {
				'e8g8' => 'rn1q1rk1/1bpp1ppp/1p2pn2/p7/1bPP4/1P3NP1/P2BPPBP/RN1Q1RK1 w - - 2 9',
			},
		},
		'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/1P3NP1/P2BPPBP/RN1QK2R w KQkq a6 0 8' => {
			eco => 'E15n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb7 6. Bg2 Bb4+ 7. Bd2 a5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7, 7... a5'),
			moves => {
				'e1g1' => 'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/1P3NP1/P2BPPBP/RN1Q1RK1 b kq - 1 8',
			},
		},
		'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/5NP1/PP1BPPBP/RN1Q1RK1 b kq - 1 7' => {
			eco => 'E16f',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Capablanca, Yates Variation'),
			moves => {
				'e8g8' => 'rn1q1rk1/1bpp1ppp/1p2pn2/p7/1bPP4/5NP1/PP1BPPBP/RN1Q1RK1 w - - 2 8',
			},
		},
		'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/5NP1/PP1BPPBP/RN1QK2R w KQkq a6 0 7' => {
			eco => 'E16f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 a5

			variation => N__('Queen\'s Indian: Capablanca, Yates Variation'),
			moves => {
				'e1g1' => 'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/5NP1/PP1BPPBP/RN1Q1RK1 b kq - 1 7',
			},
		},
		'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN1B3/PPP1BPPP/R2QK2R w KQkq - 4 9' => {
			eco => 'B92g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. Be3 Be6

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8. Be3 Be6'),
			moves => {
			},
		},
		'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPP3PP/R2QKB1R w KQkq - 1 9' => {
			eco => 'B90o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Be7

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7'),
			moves => {
				'd1d2' => 'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R b KQkq - 2 9',
			},
		},
		'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R b KQkq - 2 9' => {
			eco => 'B90o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3 Be7 9. Qd2'),
			moves => {
				'b8d7' => 'r2qk2r/1p1nbppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R w KQkq - 3 10',
				'e8g8' => 'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPPQ2PP/R3KB1R w KQ - 3 10',
			},
		},
		'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w kq - 4 9' => {
			eco => 'B92h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O Be6

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O Be6'),
			moves => {
				'f2f4' => 'rn1qk2r/1p2bppp/p2pbn2/4p3/4PP2/1NN5/PPP1B1PP/R1BQ1RK1 b kq f3 0 9',
			},
		},
		'rn1qk2r/1p2bppp/p2pbn2/4p3/4PP2/1NN5/PPP1B1PP/R1BQ1RK1 b kq f3 0 9' => {
			eco => 'B92i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O Be6 9. f4

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O Be6 9. f4'),
			moves => {
			},
		},
		'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQ1K1R b kq - 0 10' => {
			eco => 'A59e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. Nf3 Bg7 10. h3

			variation => N__('Benko Gambit: 7. e4 Line, 9. Nf3 Bg7 10. h3'),
			moves => {
				'b8d7' => 'r2qk2r/3nppbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQ1K1R w kq - 1 11',
			},
		},
		'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQ1K1R w kq - 2 10' => {
			eco => 'A59d',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 7. e4 Line, 9. Nf3'),
			moves => {
				'h2h3' => 'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQ1K1R b kq - 0 10',
			},
		},
		'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N3P1/PP3P1P/R1BQ1KNR w kq - 1 10' => {
			eco => 'A59h',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 7. e4 Line, 9. g3'),
			moves => {
				'f1g2' => 'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N3P1/PP3PKP/R1BQ2NR b kq - 2 10',
			},
		},
		'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N3P1/PP3PKP/R1BQ2NR b kq - 2 10' => {
			eco => 'A59h',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 7. e4 Line, 9. g3'),
			moves => {
				'e8g8' => 'rn1q1rk1/4ppbp/3p1np1/2pP4/4P3/2N3P1/PP3PKP/R1BQ2NR w - - 3 11',
			},
		},
		'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 9' => {
			eco => 'A58k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7

			variation => N__('Benko Gambit: Fianchetto Variation'),
			moves => {
				'f1g2' => 'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 9',
				'f1h3' => 'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N2NPB/PP2PP1P/R1BQK2R b KQkq - 2 9',
			},
		},
		'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 9' => {
			eco => 'A58m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bg2

			variation => N__('Benko Gambit: Fianchetto, 9. Bg2'),
			moves => {
				'b8d7' => 'r2qk2r/3nppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 3 10',
				'e8g8' => 'rn1q1rk1/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 10',
			},
		},
		'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N2NPB/PP2PP1P/R1BQK2R b KQkq - 2 9' => {
			eco => 'A58l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3 Bg7 9. Bh3

			variation => N__('Benko Gambit: Fianchetto, 9. Bh3'),
			moves => {
			},
		},
		'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'A58j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. Nd2 Bg7

			variation => N__('Benko Gambit: Accepted, 7. Nf3 g6 8. Nd2 Bg7'),
			moves => {
			},
		},
		'rn1qk2r/p1p1bppp/bp2pn2/3P4/3P4/1P3NP1/P2BPPBP/RN1QK2R b KQkq - 0 8' => {
			eco => 'E15p',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5'),
			moves => {
				'e6d5' => 'rn1qk2r/p1p1bppp/bp3n2/3p4/3P4/1P3NP1/P2BPPBP/RN1QK2R w KQkq - 0 9',
			},
		},
		'rn1qk2r/p1p1bppp/bp2pn2/3p4/2PP4/1P3NP1/P2BPPBP/RN1QK2R w KQkq d6 0 8' => {
			eco => 'E15p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5'),
			moves => {
				'c4d5' => 'rn1qk2r/p1p1bppp/bp2pn2/3P4/3P4/1P3NP1/P2BPPBP/RN1QK2R b KQkq - 0 8',
			},
		},
		'rn1qk2r/p1p1bppp/bp3n2/3p4/3P4/1P3NP1/P2BPPBP/RN1Q1RK1 b kq - 1 9' => {
			eco => 'E15p',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5'),
			moves => {
				'e8g8' => 'rn1q1rk1/p1p1bppp/bp3n2/3p4/3P4/1P3NP1/P2BPPBP/RN1Q1RK1 w - - 2 10',
			},
		},
		'rn1qk2r/p1p1bppp/bp3n2/3p4/3P4/1P3NP1/P2BPPBP/RN1QK2R w KQkq - 0 9' => {
			eco => 'E15p',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 d5'),
			moves => {
				'e1g1' => 'rn1qk2r/p1p1bppp/bp3n2/3p4/3P4/1P3NP1/P2BPPBP/RN1Q1RK1 b kq - 1 9',
			},
		},
		'rn1qk2r/p1pp1pp1/bp2pn2/7p/1bPP4/2N1P1N1/PP3PPP/R1BQKB1R w KQkq h6 0 7' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6 6. Ng3 h5

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6 6. Ng3 h5'),
			moves => {
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/1P3NP1/P2BPP1P/RN1QKB1R b KQkq - 2 6' => {
			eco => 'E15o',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+'),
			moves => {
				'b4e7' => 'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/1P3NP1/P2BPP1P/RN1QKB1R w KQkq - 3 7',
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'E15o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+'),
			moves => {
				'c1d2' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/1P3NP1/P2BPP1P/RN1QKB1R b KQkq - 2 6',
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/2N1P1N1/PP3PPP/R1BQKB1R b KQkq - 3 6' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6 6. Ng3

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6 6. Ng3'),
			moves => {
				'b4c3' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/2PP4/2b1P1N1/PP3PPP/R1BQKB1R w KQkq - 0 7',
				'h7h5' => 'rn1qk2r/p1pp1pp1/bp2pn2/7p/1bPP4/2N1P1N1/PP3PPP/R1BQKB1R w KQkq h6 0 7',
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6'),
			moves => {
				'a2a3' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/P1N1P3/1P2NPPP/R1BQKB1R b KQkq - 0 6',
				'e2g3' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/2N1P1N1/PP3PPP/R1BQKB1R b KQkq - 3 6',
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/5NP1/PP1NPP1P/R1BQKB1R w KQkq - 3 6' => {
			eco => 'E15g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Nbd2 Bb4

			variation => N__('Queen\'s Indian: Nimzowitsch, 5.Nbd2 Bb4'),
			moves => {
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/P1N1P3/1P2NPPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6 6. a3

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6 6. a3'),
			moves => {
				'b4c3' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/2PP4/P1b1P3/1P2NPPP/R1BQKB1R w KQkq - 0 7',
				'b4e7' => 'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/P1N1P3/1P2NPPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/2PP4/2b1P1N1/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6 6. Ng3 Bxc3+

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6 6. Ng3 Bxc3+'),
			moves => {
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PP1QPP1P/RN2KB1R w KQkq - 1 7' => {
			eco => 'E15c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb4+ 5. Bd2 Bxd2+ 6. Qxd2 Ba6

			variation => N__('Queen\'s Indian: 4. g3 Bb4+ 5. Bd2 Bxd2+ 6.Qxd2 Ba6'),
			moves => {
			},
		},
		'rn1qk2r/p1pp1ppp/bp2pn2/8/2PP4/P1b1P3/1P2NPPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6 6. a3 Bxc3+

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6 6. a3 Bxc3+'),
			moves => {
			},
		},
		'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/1P3NP1/P2BPP1P/RN1QKB1R w KQkq - 3 7' => {
			eco => 'E15o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7'),
			moves => {
				'b1c3' => 'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/1PN2NP1/P2BPP1P/R2QKB1R b KQkq - 4 7',
				'f1g2' => 'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/1P3NP1/P2BPPBP/RN1QK2R b KQkq - 4 7',
			},
		},
		'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/1P3NP1/P2BPPBP/RN1QK2R b KQkq - 4 7' => {
			eco => 'E15p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2'),
			moves => {
				'c7c6' => 'rn1qk2r/p2pbppp/bpp1pn2/8/2PP4/1P3NP1/P2BPPBP/RN1QK2R w KQkq - 0 8',
				'd7d5' => 'rn1qk2r/p1p1bppp/bp2pn2/3p4/2PP4/1P3NP1/P2BPPBP/RN1QK2R w KQkq d6 0 8',
			},
		},
		'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/1PN2NP1/P2BPP1P/R2QKB1R b KQkq - 4 7' => {
			eco => 'E15o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Nc3

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Nc3'),
			moves => {
			},
		},
		'rn1qk2r/p1ppbppp/bp2pn2/8/2PP4/P1N1P3/1P2NPPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'E45',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ba6 6. a3 Be7

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ba6 6. a3 Be7'),
			moves => {
			},
		},
		'rn1qk2r/p1ppbppp/bp2pn2/8/Q1PP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 3 6' => {
			eco => 'E15h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 Be7

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 Be7'),
			moves => {
			},
		},
		'rn1qk2r/p2nbppp/bpp1p3/3pN3/2PP4/1PB3P1/P3PPBP/RN1QK2R w KQkq - 2 10' => {
			eco => 'E15s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8. Bc3 d5 9. Ne5 Nfd7

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+, Main Line, 9. Ne5 Nfd7'),
			moves => {
			},
		},
		'rn1qk2r/p2pbppp/bpp1pn2/8/2PP4/1P3NP1/P2BPPBP/RN1Q1RK1 b kq - 1 8' => {
			eco => 'E15q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8. O-O

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8.O-O'),
			moves => {
				'd7d5' => 'rn1qk2r/p3bppp/bpp1pn2/3p4/2PP4/1P3NP1/P2BPPBP/RN1Q1RK1 w kq d6 0 9',
			},
		},
		'rn1qk2r/p2pbppp/bpp1pn2/8/2PP4/1P3NP1/P2BPPBP/RN1QK2R w KQkq - 0 8' => {
			eco => 'E15q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6'),
			moves => {
				'd2c3' => 'rn1qk2r/p2pbppp/bpp1pn2/8/2PP4/1PB2NP1/P3PPBP/RN1QK2R b KQkq - 1 8',
				'e1g1' => 'rn1qk2r/p2pbppp/bpp1pn2/8/2PP4/1P3NP1/P2BPPBP/RN1Q1RK1 b kq - 1 8',
			},
		},
		'rn1qk2r/p2pbppp/bpp1pn2/8/2PP4/1PB2NP1/P3PPBP/RN1QK2R b KQkq - 1 8' => {
			eco => 'E15r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8. Bc3

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+, Main Line'),
			moves => {
				'd7d5' => 'rn1qk2r/p3bppp/bpp1pn2/3p4/2PP4/1PB2NP1/P3PPBP/RN1QK2R w KQkq d6 0 9',
			},
		},
		'rn1qk2r/p3bppp/bpp1pn2/3p4/2PP4/1P3NP1/P2BPPBP/RN1Q1RK1 w kq d6 0 9' => {
			eco => 'E15q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8. O-O d5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8.O-O d5'),
			moves => {
			},
		},
		'rn1qk2r/p3bppp/bpp1pn2/3p4/2PP4/1PB2NP1/P3PPBP/RN1QK2R w KQkq d6 0 9' => {
			eco => 'E15r',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+, Main Line'),
			moves => {
				'f3e5' => 'rn1qk2r/p3bppp/bpp1pn2/3pN3/2PP4/1PB3P1/P3PPBP/RN1QK2R b KQkq - 1 9',
			},
		},
		'rn1qk2r/p3bppp/bpp1pn2/3pN3/2PP4/1PB3P1/P3PPBP/RN1QK2R b KQkq - 1 9' => {
			eco => 'E15s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb4+ 6. Bd2 Be7 7. Bg2 c6 8. Bc3 d5 9. Ne5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb4+, Main Line, 9. Ne5'),
			moves => {
				'f6d7' => 'rn1qk2r/p2nbppp/bpp1p3/3pN3/2PP4/1PB3P1/P3PPBP/RN1QK2R w KQkq - 2 10',
			},
		},
		'rn1qk2r/pb1p1pp1/1p2pn1p/2p5/1bPP3B/2N1PN2/PP3PPP/R2QKB1R w KQkq c6 0 8' => {
			eco => 'E13g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 c5

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 c5'),
			moves => {
			},
		},
		'rn1qk2r/pb1p1ppp/1p2pn2/2p5/1bPP4/3BPN2/PP1N1PPP/R1BQK2R w KQkq c6 0 7' => {
			eco => 'E14g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2 c5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2 c5'),
			moves => {
			},
		},
		'rn1qk2r/pb1p1ppp/1p2pn2/2p5/1bPP4/5NP1/PP1BPPBP/RN1QK2R w KQkq c6 0 7' => {
			eco => 'E16e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 c5

			variation => N__('Queen\'s Indian: Capablanca, 6. Bd2 c5'),
			moves => {
			},
		},
		'rn1qk2r/pb1pbpp1/1p2pn1p/2p5/2PP3B/2N1PN2/PP3PPP/R2QKB1R w KQkq c6 0 8' => {
			eco => 'E12x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Be7 7. e3 c5

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Be7 7. e3 c5'),
			moves => {
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A30p',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. b3

			variation => N__('English: Symmetrical, Hedgehog, 7. b3'),
			moves => {
				'e8g8' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 w - - 1 8',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7' => {
			eco => 'A30m',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7

			variation => N__('English: Symmetrical, Hedgehog System'),
			moves => {
				'b2b3' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 b kq - 0 7',
				'd2d4' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq d3 0 7',
				'f1e1' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQR1K1 b kq - 3 7',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQR1K1 b kq - 3 7' => {
			eco => 'A30n',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. Re1

			variation => N__('English: Symmetrical, Hedgehog, 7. Re1'),
			moves => {
				'd7d5' => 'rn1qk2r/pb2bppp/1p2pn2/2pp4/2P5/2N2NP1/PP1PPPBP/R1BQR1K1 w kq d6 0 8',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/1P1BPN2/P4PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'E14u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. b3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. b3'),
			moves => {
				'e8g8' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/1P1BPN2/P4PPP/RNBQ1RK1 w - - 1 8',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq d3 0 7' => {
			eco => 'A30q',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4

			variation => N__('English: Symmetrical, Hedgehog, 7. d4'),
			moves => {
				'c5d4' => 'rn1qk2r/pb1pbppp/1p2pn2/8/2Pp4/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 0 8',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'E14x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. Nc3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. Nc3'),
			moves => {
				'c5d4' => 'rn1qk2r/pb1pbppp/1p2pn2/8/2Pp4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 0 8',
				'e8g8' => 'rn1q1rk1/pb1pbppp/1p2pn2/2p5/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 4 8',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/3BPN2/PP1N1PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'E14t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. Nbd2

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7.Nbd2'),
			moves => {
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'E14t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7'),
			moves => {
				'b1c3' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 3 7',
				'b1d2' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/3BPN2/PP1N1PPP/R1BQ1RK1 b kq - 3 7',
				'b2b3' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/1P1BPN2/P4PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/8/2PP4/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 8' => {
			eco => 'E14y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. Nc3 cxd4 8. exd4

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. Nc3 cxd4 8.exd4'),
			moves => {
				'd7d5' => 'rn1qk2r/pb2bppp/1p2pn2/3p4/2PP4/2NB1N2/PP3PPP/R1BQ1RK1 w kq d6 0 9',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 b kq - 0 8' => {
			eco => 'A30r',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4

			variation => N__('English: Symmetrical, Hedgehog, 8.Qxd4'),
			moves => {
				'b8c6' => 'r2qk2r/pb1pbppp/1pn1pn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 w kq - 1 9',
				'd7d6' => 'rn1qk2r/pb2bppp/1p1ppn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 w kq - 0 9',
				'e8g8' => 'rn1q1rk1/pb1pbppp/1p2pn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 w - - 1 9',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/8/2Pp4/2N2NP1/PP2PPBP/R1BQ1RK1 w kq - 0 8' => {
			eco => 'A30q',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Hedgehog, 7. d4'),
			moves => {
				'd1d4' => 'rn1qk2r/pb1pbppp/1p2pn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 b kq - 0 8',
			},
		},
		'rn1qk2r/pb1pbppp/1p2pn2/8/2Pp4/2NBPN2/PP3PPP/R1BQ1RK1 w kq - 0 8' => {
			eco => 'E14x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. Nc3 cxd4

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. Nc3 cxd4'),
			moves => {
				'e3d4' => 'rn1qk2r/pb1pbppp/1p2pn2/8/2PP4/2NB1N2/PP3PPP/R1BQ1RK1 b kq - 0 8',
			},
		},
		'rn1qk2r/pb1pppbp/1p3np1/2p5/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 w kq - 1 7' => {
			eco => 'A30g',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Mutual Double Fianchetto'),
			moves => {
				'c1b2' => 'rn1qk2r/pb1pppbp/1p3np1/2p5/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 b kq - 2 7',
			},
		},
		'rn1qk2r/pb1pppbp/1p3np1/2p5/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 b kq - 2 7' => {
			eco => 'A30g',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Mutual Double Fianchetto'),
			moves => {
				'e8g8' => 'rn1q1rk1/pb1pppbp/1p3np1/2p5/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 w - - 3 8',
			},
		},
		'rn1qk2r/pb1pppbp/1p3np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7' => {
			eco => 'A30f',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O g6 6. Nc3 Bg7

			variation => N__('English: Symmetrical, Double Fianchetto'),
			moves => {
			},
		},
		'rn1qk2r/pb2bppp/1p1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1B2RK1 b kq - 0 9' => {
			eco => 'A30v',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. b3

			variation => N__('English: Symmetrical, Hedgehog, 9. b3'),
			moves => {
				'b8d7' => 'r2qk2r/pb1nbppp/1p1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1B2RK1 w kq - 1 10',
			},
		},
		'rn1qk2r/pb2bppp/1p1ppn2/8/2PQ4/2N2NP1/PP2PPBP/R1B2RK1 w kq - 0 9' => {
			eco => 'A30u',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6

			variation => N__('English: Symmetrical, Hedgehog, 8.Qxd4 d6'),
			moves => {
				'b2b3' => 'rn1qk2r/pb2bppp/1p1ppn2/8/2PQ4/1PN2NP1/P3PPBP/R1B2RK1 b kq - 0 9',
				'f1d1' => 'rn1qk2r/pb2bppp/1p1ppn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 b kq - 1 9',
			},
		},
		'rn1qk2r/pb2bppp/1p1ppn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 b kq - 1 9' => {
			eco => 'A30w',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. d4 cxd4 8. Qxd4 d6 9. Rd1

			variation => N__('English: Symmetrical, Hedgehog, 9. Rd1'),
			moves => {
				'a7a6' => 'rn1qk2r/1b2bppp/pp1ppn2/8/2PQ4/2N2NP1/PP2PPBP/R1BR2K1 w kq - 0 10',
			},
		},
		'rn1qk2r/pb2bppp/1p2pn2/2pp4/2P5/2N2NP1/PP1PPPBP/R1BQR1K1 w kq d6 0 8' => {
			eco => 'A30o',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 Be7 7. Re1 d5

			variation => N__('English: Symmetrical, Hedgehog, 7. Re1 d5'),
			moves => {
			},
		},
		'rn1qk2r/pb2bppp/1p2pn2/3p4/2PP4/2NB1N2/PP3PPP/R1BQ1RK1 w kq d6 0 9' => {
			eco => 'E14z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O Be7 7. Nc3 cxd4 8. exd4 d5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O Be7 7. Nc3 cxd4 8.exd4 d5'),
			moves => {
			},
		},
		'rn1qk2r/pb2bppp/1pp1pB2/3p4/Q1PP4/P1N2N2/1P2PPPP/R3KB1R b KQkq - 0 8' => {
			eco => 'E12o',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6'),
			moves => {
				'e7f6' => 'rn1qk2r/pb3ppp/1pp1pb2/3p4/Q1PP4/P1N2N2/1P2PPPP/R3KB1R w KQkq - 0 9',
			},
		},
		'rn1qk2r/pb2bppp/1pp1pn2/3p2B1/Q1PP4/P1N2N2/1P2PPPP/R3KB1R w KQkq - 0 8' => {
			eco => 'E12o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6'),
			moves => {
				'g5f6' => 'rn1qk2r/pb2bppp/1pp1pB2/3p4/Q1PP4/P1N2N2/1P2PPPP/R3KB1R b KQkq - 0 8',
			},
		},
		'rn1qk2r/pb3ppp/1pp1pb2/3P4/Q2P4/P1N2N2/1P2PPPP/R3KB1R b KQkq - 0 9' => {
			eco => 'E12o',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6'),
			moves => {
				'e6d5' => 'rn1qk2r/pb3ppp/1pp2b2/3p4/Q2P4/P1N2N2/1P2PPPP/R3KB1R w KQkq - 0 10',
			},
		},
		'rn1qk2r/pb3ppp/1pp1pb2/3p4/Q1PP4/P1N2N2/1P2PPPP/R3KB1R w KQkq - 0 9' => {
			eco => 'E12o',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6'),
			moves => {
				'c4d5' => 'rn1qk2r/pb3ppp/1pp1pb2/3P4/Q2P4/P1N2N2/1P2PPPP/R3KB1R b KQkq - 0 9',
			},
		},
		'rn1qk2r/pb3ppp/1pp2b2/3p4/Q2P4/P1N2N2/1P2PPPP/R3KB1R w KQkq - 0 10' => {
			eco => 'E12o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6 8. Bxf6 Bxf6 9. cxd5 exd5

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7 7. Qa4+ c6 8.Bxf6 Bxf6 9.cxd5 exd5'),
			moves => {
			},
		},
		'rn1qk2r/pbp1bppp/1p2p3/3n4/3P4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'E12s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. e3 Be7

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 Nxd5 7. e3 Be7'),
			moves => {
			},
		},
		'rn1qk2r/pbp1bppp/1p2p3/3n4/3P4/P1N2N2/1PQ1PPPP/R1B1KB1R w KQkq - 2 8' => {
			eco => 'E12t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Be7

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7... Be7'),
			moves => {
			},
		},
		'rn1qk2r/pbp1bppp/1p2p3/8/3P4/P1P2N2/2Q1PPPP/R1B1KB1R w KQkq - 1 9' => {
			eco => 'E12u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Nxc3 8. bxc3 Be7

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7...Nxc3 8.bxc3 Be7'),
			moves => {
				'e2e4' => 'rn1qk2r/pbp1bppp/1p2p3/8/3PP3/P1P2N2/2Q2PPP/R1B1KB1R b KQkq e3 0 9',
			},
		},
		'rn1qk2r/pbp1bppp/1p2p3/8/3PP3/P1P2N2/2Q2PPP/R1B1KB1R b KQkq e3 0 9' => {
			eco => 'E12u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Nxc3 8. bxc3 Be7 9. e4

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7...Nxc3 8.bxc3 Be7 9. e4'),
			moves => {
			},
		},
		'rn1qk2r/pbp1bppp/1p2pn2/3p2B1/2PP4/P1N2N2/1P2PPPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'E12n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. Bg5 Be7

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7'),
			moves => {
				'd1a4' => 'rn1qk2r/pbp1bppp/1p2pn2/3p2B1/Q1PP4/P1N2N2/1P2PPPP/R3KB1R b KQkq - 3 7',
			},
		},
		'rn1qk2r/pbp1bppp/1p2pn2/3p2B1/Q1PP4/P1N2N2/1P2PPPP/R3KB1R b KQkq - 3 7' => {
			eco => 'E12o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. Bg5 Be7 7. Qa4+

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5 Be7 7. Qa4+'),
			moves => {
				'c7c6' => 'rn1qk2r/pb2bppp/1pp1pn2/3p2B1/Q1PP4/P1N2N2/1P2PPPP/R3KB1R w KQkq - 0 8',
			},
		},
		'rn1qk2r/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQkq d6 0 7' => {
			eco => 'E17b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 d5

			variation => N__('Queen\'s Indian: 6. Nc3 d5'),
			moves => {
			},
		},
		'rn1qk2r/pbp1bppp/1p2pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq d6 0 7' => {
			eco => 'E17m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O d5

			variation => N__('Queen\'s Indian: 6.O-O d5'),
			moves => {
			},
		},
		'rn1qk2r/pbp2p2/1p1pp2p/6p1/2PPn3/2P1PNB1/P1Q2PPP/R3KB1R w KQkq - 0 11' => {
			eco => 'E13p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10. bxc3 d6

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10.bxc3 d6'),
			moves => {
				'f1d3' => 'rn1qk2r/pbp2p2/1p1pp2p/6p1/2PPn3/2PBPNB1/P1Q2PPP/R3K2R b KQkq - 1 11',
			},
		},
		'rn1qk2r/pbp2p2/1p1pp2p/6p1/2PPn3/2PBPNB1/P1Q2PPP/R3K2R b KQkq - 1 11' => {
			eco => 'E13p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10. bxc3 d6 11. Bd3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10.bxc3 d6 11. Bd3'),
			moves => {
				'f7f5' => 'rn1qk2r/pbp5/1p1pp2p/5pp1/2PPn3/2PBPNB1/P1Q2PPP/R3K2R w KQkq f6 0 12',
			},
		},
		'rn1qk2r/pbp2pp1/1p1ppn1p/8/2PP3B/2P1P3/P2N1PPP/R2QKB1R b KQkq - 1 9' => {
			eco => 'E13j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8. bxc3 d6 9. Nd2

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8.bxc3 d6 9. Nd2'),
			moves => {
				'b8d7' => 'r2qk2r/pbpn1pp1/1p1ppn1p/8/2PP3B/2P1P3/P2N1PPP/R2QKB1R w KQkq - 2 10',
			},
		},
		'rn1qk2r/pbp2pp1/1p1ppn1p/8/2PP3B/2P1PN2/P4PPP/R2QKB1R w KQkq - 0 9' => {
			eco => 'E13i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8. bxc3 d6

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8.bxc3 d6'),
			moves => {
				'f3d2' => 'rn1qk2r/pbp2pp1/1p1ppn1p/8/2PP3B/2P1P3/P2N1PPP/R2QKB1R b KQkq - 1 9',
			},
		},
		'rn1qk2r/pbp2ppp/1p1bpn2/3p4/2PP4/1P1BPN2/P4PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'E14o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 d5 6. O-O Bd6 7. b3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 d5 6.O-O Bd6 7. b3'),
			moves => {
			},
		},
		'rn1qk2r/pbp2ppp/1p1bpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'E14p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 d5 6. O-O Bd6 7. Nc3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 d5 6.O-O Bd6 7. Nc3'),
			moves => {
			},
		},
		'rn1qk2r/pbp2ppp/1p1bpn2/3p4/2PP4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'E14n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 d5 6. O-O Bd6

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 d5 6.O-O Bd6'),
			moves => {
				'b1c3' => 'rn1qk2r/pbp2ppp/1p1bpn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 3 7',
				'b2b3' => 'rn1qk2r/pbp2ppp/1p1bpn2/3p4/2PP4/1P1BPN2/P4PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rn1qk2r/pbp5/1p1pp2p/5pp1/2PPn3/2PBPNB1/P1Q2PPP/R3K2R w KQkq f6 0 12' => {
			eco => 'E13q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10. bxc3 d6 11. Bd3 f5

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10.bxc3 d6 11. Bd3 f5'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1p2/1p2p2p/6p1/1bPPn3/2N1PNB1/PP3PPP/R2QKB1R w KQkq - 2 9' => {
			eco => 'E13m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4'),
			moves => {
				'd1c2' => 'rn1qk2r/pbpp1p2/1p2p2p/6p1/1bPPn3/2N1PNB1/PPQ2PPP/R3KB1R b KQkq - 3 9',
			},
		},
		'rn1qk2r/pbpp1p2/1p2p2p/6p1/1bPPn3/2N1PNB1/PPQ2PPP/R3KB1R b KQkq - 3 9' => {
			eco => 'E13n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2'),
			moves => {
				'b4c3' => 'rn1qk2r/pbpp1p2/1p2p2p/6p1/2PPn3/2b1PNB1/PPQ2PPP/R3KB1R w KQkq - 0 10',
			},
		},
		'rn1qk2r/pbpp1p2/1p2p2p/6p1/2PPn3/2P1PNB1/P1Q2PPP/R3KB1R b KQkq - 0 10' => {
			eco => 'E13o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+ 10. bxc3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+'),
			moves => {
				'd7d6' => 'rn1qk2r/pbp2p2/1p1pp2p/6p1/2PPn3/2P1PNB1/P1Q2PPP/R3KB1R w KQkq - 0 11',
			},
		},
		'rn1qk2r/pbpp1p2/1p2p2p/6p1/2PPn3/2b1PNB1/PPQ2PPP/R3KB1R w KQkq - 0 10' => {
			eco => 'E13o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3 Ne4 9. Qc2 Bxc3+'),
			moves => {
				'b2c3' => 'rn1qk2r/pbpp1p2/1p2p2p/6p1/2PPn3/2P1PNB1/P1Q2PPP/R3KB1R b KQkq - 0 10',
			},
		},
		'rn1qk2r/pbpp1p2/1p2pn1p/6p1/1bPP3B/2N1PN2/PP3PPP/R2QKB1R w KQkq g6 0 8' => {
			eco => 'E13m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5'),
			moves => {
				'h4g3' => 'rn1qk2r/pbpp1p2/1p2pn1p/6p1/1bPP4/2N1PNB1/PP3PPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rn1qk2r/pbpp1p2/1p2pn1p/6p1/1bPP3B/2N2N2/PPQ1PPPP/R3KB1R w KQkq g6 0 8' => {
			eco => 'E13c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. Qc2 g5

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. Qc2 g5'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1p2/1p2pn1p/6p1/1bPP4/2N1PNB1/PP3PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'E13m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5 8. Bg3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 g5'),
			moves => {
				'f6e4' => 'rn1qk2r/pbpp1p2/1p2p2p/6p1/1bPPn3/2N1PNB1/PP3PPP/R2QKB1R w KQkq - 2 9',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'E13f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3'),
			moves => {
				'b4c3' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2b1PN2/PP3PPP/R2QKB1R w KQkq - 0 8',
				'c7c5' => 'rn1qk2r/pb1p1pp1/1p2pn1p/2p5/1bPP3B/2N1PN2/PP3PPP/R2QKB1R w KQkq c6 0 8',
				'g7g5' => 'rn1qk2r/pbpp1p2/1p2pn1p/6p1/1bPP3B/2N1PN2/PP3PPP/R2QKB1R w KQkq g6 0 8',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'E13a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4'),
			moves => {
				'd1c2' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 3 7',
				'e2e3' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
				'f3d2' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N5/PP1NPPPP/R2QKB1R b KQkq - 3 7',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 3 7' => {
			eco => 'E13b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. Qc2

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. Qc2'),
			moves => {
				'g7g5' => 'rn1qk2r/pbpp1p2/1p2pn1p/6p1/1bPP3B/2N2N2/PPQ1PPPP/R3KB1R w KQkq g6 0 8',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N5/PP1NPPPP/R2QKB1R b KQkq - 3 7' => {
			eco => 'E13d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. Nd2

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. Nd2'),
			moves => {
				'b4c3' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2b5/PP1NPPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2P1PN2/P4PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'E13h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8. bxc3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+ 8.bxc3'),
			moves => {
				'd7d6' => 'rn1qk2r/pbp2pp1/1p1ppn1p/8/2PP3B/2P1PN2/P4PPP/R2QKB1R w KQkq - 0 9',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2P5/P2NPPPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'E13e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. Nd2 Bxc3 8. bxc3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. Nd2 Bxc3 8.bxc3'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2b1PN2/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'E13h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. e3 Bxc3+'),
			moves => {
				'b2c3' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2P1PN2/P4PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2b5/PP1NPPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'E13e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Bb4 7. Nd2 Bxc3

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Bb4 7. Nd2 Bxc3'),
			moves => {
				'b2c3' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/2PP3B/2P5/P2NPPPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2p2n/8/2PPP3/2N5/PP1Q1PPP/R3KBNR w KQkq - 1 7' => {
			eco => 'A40q',
                        # TRANSLATORS: 1. d4 e6 2. c4 Bb4+ 3. Bd2 Bxd2+ 4. Qxd2 b6 5. Nc3 Bb7 6. e4 Nh6

			variation => N__('Queen\'s Pawn: Keres Defence, Kangaroo Variation'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2p3/8/1bPPn3/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 5 7' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7 6. Bd3 Ne4 7. O-O

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3 Bb7 6. Bd3 Ne4 7.O-O'),
			moves => {
				'f7f5' => 'rn1qk2r/pbpp2pp/1p2p3/5p2/1bPPn3/2NBPN2/PP3PPP/R1BQ1RK1 w kq f6 0 8',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2p3/8/1bPPn3/2NBPN2/PP3PPP/R1BQK2R w KQkq - 4 7' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7 6. Bd3 Ne4

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3 Bb7 6. Bd3 Ne4'),
			moves => {
				'e1g1' => 'rn1qk2r/pbpp1ppp/1p2p3/8/1bPPn3/2NBPN2/PP3PPP/R1BQ1RK1 b kq - 5 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pb2/8/2PPn3/2N2NP1/PP1BPPBP/R2QK2R w KQkq - 7 8' => {
			eco => 'E17k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 Ne4 7. Bd2 Bf6

			variation => N__('Queen\'s Indian: Opovcensky, 7... Bf6'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/6B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 6' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Bg5 Bb7

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Bg5 Bb7'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP1B2/4P3/PP1N1PPP/RN1QKB1R b KQkq - 2 6' => {
			eco => 'E12c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bf4 Bb7 5. e3 Bb4+ 6. Nfd2

			variation => N__('Queen\'s Indian: Miles, 5. e3 Bb4+ 6.Nfd2'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP1B2/4PN2/PP1N1PPP/R2QKB1R b KQkq - 2 6' => {
			eco => 'E12c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bf4 Bb7 5. e3 Bb4+ 6. Nbd2

			variation => N__('Queen\'s Indian: Miles, 5. e3 Bb4+ 6.Nbd2'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP1B2/4PN2/PP3PPP/RN1QKB1R w KQkq - 1 6' => {
			eco => 'E12c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bf4 Bb7 5. e3 Bb4+

			variation => N__('Queen\'s Indian: Miles, 5. e3 Bb4+'),
			moves => {
				'b1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP1B2/4PN2/PP1N1PPP/R2QKB1R b KQkq - 2 6',
				'f3d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP1B2/4P3/PP1N1PPP/RN1QKB1R b KQkq - 2 6',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/1P3NP1/P2BPPBP/RN1QK2R b KQkq - 4 7' => {
			eco => 'E15n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7 6. Bg2'),
			moves => {
				'a7a5' => 'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/1P3NP1/P2BPPBP/RN1QK2R w KQkq a6 0 8',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/1P3NP1/P3PPBP/RNBQK2R w KQkq - 3 7' => {
			eco => 'E15n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7 6. Bg2'),
			moves => {
				'c1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/1P3NP1/P2BPPBP/RN1QK2R b KQkq - 4 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E44',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Bb7

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Bb7'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3 Bb7'),
			moves => {
				'f1d3' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 2 6' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Qc2 Bb7

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Qc2 Bb7'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7 6. Bd3

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3 Bb7 6. Bd3'),
			moves => {
				'e8g8' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 4 7',
				'f6e4' => 'rn1qk2r/pbpp1ppp/1p2p3/8/1bPPn3/2NBPN2/PP3PPP/R1BQK2R w KQkq - 4 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP1N1PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'E14g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+ 6. Nbd2

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+ 6.Nbd2'),
			moves => {
				'c7c5' => 'rn1qk2r/pb1p1ppp/1p2pn2/2p5/1bPP4/3BPN2/PP1N1PPP/R1BQK2R w KQkq c6 0 7',
				'e8g8' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP1N1PPP/R1BQK2R w KQ - 5 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP3PPP/RNBQK2R w KQkq - 3 6' => {
			eco => 'E14f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Bb4+

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Bb4+'),
			moves => {
				'b1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP1N1PPP/R1BQK2R b KQkq - 4 6',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R b KQkq - 4 6' => {
			eco => 'E16e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2

			variation => N__('Queen\'s Indian: Capablanca, 6. Bd2'),
			moves => {
				'a7a5' => 'rn1qk2r/1bpp1ppp/1p2pn2/p7/1bPP4/5NP1/PP1BPPBP/RN1QK2R w KQkq a6 0 7',
				'b4d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1bPPBP/RN1QK2R w KQkq - 0 7',
				'b4e7' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/5NP1/PP1BPPBP/RN1QK2R w KQkq - 5 7',
				'c7c5' => 'rn1qk2r/pb1p1ppp/1p2pn2/2p5/1bPP4/5NP1/PP1BPPBP/RN1QK2R w KQkq c6 0 7',
				'd8e7' => 'rn2k2r/pbppqppp/1p2pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R w KQkq - 5 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1NPPBP/R1BQK2R b KQkq - 4 6' => {
			eco => 'E16b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Nbd2

			variation => N__('Queen\'s Indian: Capablanca, 6.Nbd2'),
			moves => {
				'e8g8' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1NPPBP/R1BQK2R w KQ - 5 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 6' => {
			eco => 'E16a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+

			variation => N__('Queen\'s Indian: Capablanca Variation'),
			moves => {
				'b1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1NPPBP/R1BQK2R b KQkq - 4 6',
				'c1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R b KQkq - 4 6',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1NPPBP/R2QK2R b KQkq - 0 7' => {
			eco => 'E16o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Bxd2+ 7. Nbxd2

			variation => N__('Queen\'s Indian: Capablanca, 6...Bxd2+ 7.Nbxd2'),
			moves => {
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN2K2R b KQkq - 0 7' => {
			eco => 'E16p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Bxd2+ 7. Qxd2

			variation => N__('Queen\'s Indian: Capablanca, 6...Bxd2+ 7.Qxd2'),
			moves => {
				'e8g8' => 'rn1q1rk1/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN2K2R w KQ - 1 8',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1bPPBP/RN1QK2R w KQkq - 0 7' => {
			eco => 'E16o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Bxd2+

			variation => N__('Queen\'s Indian: Capablanca, 6...Bxd2+'),
			moves => {
				'b1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1NPPBP/R2QK2R b KQkq - 0 7',
				'd1d2' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN2K2R b KQkq - 0 7',
			},
		},
		'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/P4N2/1P1BPPPP/R2QKB1R w KQkq - 1 7' => {
			eco => 'E11d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 b6 5. a3 Bxd2+ 6. Bxd2 Bb7

			variation => N__('Bogo-Indian: 4.Nbd2 b6 5. a3 Bxd2+ 6.Bxd2 Bb7'),
			moves => {
			},
		},
		'rn1qk2r/pbpp2pp/1p2p3/5p2/1bPPn3/2NBPN2/PP3PPP/R1BQ1RK1 w kq f6 0 8' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3 Bb7 6. Bd3 Ne4 7. O-O f5

			variation => N__('Nimzo-Indian: Nimzowitsch, Keene Variation'),
			moves => {
			},
		},
		'rn1qk2r/pbppb1pp/1p2p3/5p2/2PPn3/2N2NP1/PP1BPPBP/R2QK2R w KQkq f6 0 8' => {
			eco => 'E17j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 Ne4 7. Bd2 f5

			variation => N__('Queen\'s Indian: Opovcensky, 7... f5'),
			moves => {
			},
		},
		'rn1qk2r/pbppbpp1/1p2p2p/8/2PPn2B/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8' => {
			eco => 'E12x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Be7 7. e3 Ne4

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Be7 7. e3 Ne4'),
			moves => {
			},
		},
		'rn1qk2r/pbppbpp1/1p2pn1p/8/2PP3B/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'E12x',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Be7'),
			moves => {
				'c7c5' => 'rn1qk2r/pb1pbpp1/1p2pn1p/2p5/2PP3B/2N1PN2/PP3PPP/R2QKB1R w KQkq c6 0 8',
				'f6e4' => 'rn1qk2r/pbppbpp1/1p2p2p/8/2PPn2B/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8',
			},
		},
		'rn1qk2r/pbppbpp1/1p2pn1p/8/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'E12x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 Be7

			variation => N__('Queen\'s Indian: 5. Bg5 h6 6. Bh4 Be7'),
			moves => {
				'e2e3' => 'rn1qk2r/pbppbpp1/1p2pn1p/8/2PP3B/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP1BPPBP/R2QK2R b KQkq - 6 7' => {
			eco => 'E17h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 Ne4 7. Bd2

			variation => N__('Queen\'s Indian: Opovcensky Variation'),
			moves => {
				'e7f6' => 'rn1qk2r/pbpp1ppp/1p2pb2/8/2PPn3/2N2NP1/PP1BPPBP/R2QK2R w KQkq - 7 8',
				'e8g8' => 'rn1q1rk1/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP1BPPBP/R2QK2R w KQ - 7 8',
				'f7f5' => 'rn1qk2r/pbppb1pp/1p2p3/5p2/2PPn3/2N2NP1/PP1BPPBP/R2QK2R w KQkq f6 0 8',
			},
		},
		'rn1qk2r/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 5 7' => {
			eco => 'E17g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 Ne4

			variation => N__('Queen\'s Indian: 6. Nc3 Ne4'),
			moves => {
				'c1d2' => 'rn1qk2r/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP1BPPBP/R2QK2R b KQkq - 6 7',
				'd1c2' => 'rn1qk2r/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PPQ1PPBP/R1B1K2R b KQkq - 6 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PPQ1PPBP/R1B1K2R b KQkq - 6 7' => {
			eco => 'E17g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3 Ne4 7. Qc2

			variation => N__('Queen\'s Indian: 6. Nc3 Ne4 7. Qc2'),
			moves => {
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 4 6' => {
			eco => 'A17',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
				'e8g8' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 5 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 3 6' => {
			eco => 'A17',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
				'e1g1' => 'rn1qk2r/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 4 6',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP1B2/4PN1P/PP3PP1/RN1QKB1R b KQkq - 0 6' => {
			eco => 'E12b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bf4 Bb7 5. e3 Be7 6. h3

			variation => N__('Queen\'s Indian: Miles, 5. e3 Be7 6. h3'),
			moves => {
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP1B2/4PN2/PP3PPP/RN1QKB1R w KQkq - 1 6' => {
			eco => 'E12b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bf4 Bb7 5. e3 Be7

			variation => N__('Queen\'s Indian: Miles, 5. e3 Be7'),
			moves => {
				'h2h3' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP1B2/4PN1P/PP3PP1/RN1QKB1R b KQkq - 0 6',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP1BPPBP/R2QK2R b KQkq - 6 7' => {
			eco => 'E16k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Be7 7. Nc3

			variation => N__('Queen\'s Indian: Capablanca, Riumin, 7. Nc3'),
			moves => {
				'e8g8' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP1BPPBP/R2QK2R w KQ - 7 8',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 4 6' => {
			eco => 'E17b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. Nc3

			variation => N__('Queen\'s Indian: 6. Nc3'),
			moves => {
				'd7d5' => 'rn1qk2r/pbp1bppp/1p2pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQkq d6 0 7',
				'e8g8' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 5 7',
				'f6e4' => 'rn1qk2r/pbppbppp/1p2p3/8/2PPn3/2N2NP1/PP2PPBP/R1BQK2R w KQkq - 5 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 4 6' => {
			eco => 'E14k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Be7 6. O-O

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Be7 6.O-O'),
			moves => {
				'e8g8' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQ1RK1 w - - 5 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQK2R w KQkq - 3 6' => {
			eco => 'E14j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 Be7

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 Be7'),
			moves => {
				'e1g1' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 4 6',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/5NP1/PP1BPPBP/RN1QK2R w KQkq - 5 7' => {
			eco => 'E16j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Be7

			variation => N__('Queen\'s Indian: Capablanca, Riumin Variation'),
			moves => {
				'b1c3' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP1BPPBP/R2QK2R b KQkq - 6 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 6' => {
			eco => 'E17l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O

			variation => N__('Queen\'s Indian: 6.O-O'),
			moves => {
				'd7d5' => 'rn1qk2r/pbp1bppp/1p2pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq d6 0 7',
				'e8g8' => 'rn1q1rk1/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 5 7',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 6' => {
			eco => 'E17a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7

			variation => N__('Queen\'s Indian: 5. Bg2 Be7'),
			moves => {
				'b1c3' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 4 6',
				'e1g1' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 6',
			},
		},
		'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'E12j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 Be7

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 Be7'),
			moves => {
			},
		},
		'rn1qk2r/pp1Bppbp/3p2p1/2p1P3/3P1Pn1/2N2N2/PPP3PP/R1BQK2R b KQkq - 0 8' => {
			eco => 'B09v',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7 7. e5 Ng4 8. Bxd7+

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+ Bd7 7. e5 Ng4 8.Bxd7+'),
			moves => {
			},
		},
		'rn1qk2r/pp1b1pbp/3p1np1/2pP4/4P3/1QN2N2/PP3PPP/R1B1KB1R b KQkq - 5 9' => {
			eco => 'A70f',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Qa4+'),
			moves => {
				'd8c7' => 'rn2k2r/ppqb1pbp/3p1np1/2pP4/4P3/1QN2N2/PP3PPP/R1B1KB1R w KQkq - 6 10',
			},
		},
		'rn1qk2r/pp1b1pbp/3p1np1/2pP4/Q3P3/2N2N2/PP3PPP/R1B1KB1R w KQkq - 4 9' => {
			eco => 'A70f',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Qa4+'),
			moves => {
				'a4b3' => 'rn1qk2r/pp1b1pbp/3p1np1/2pP4/4P3/1QN2N2/PP3PPP/R1B1KB1R b KQkq - 5 9',
			},
		},
		'rn1qk2r/pp1bnppp/4p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8' => {
			eco => 'C19j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Bd7

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Bd7'),
			moves => {
				'a3a4' => 'rn1qk2r/pp1bnppp/4p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rn1qk2r/pp1bnppp/4p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'C19k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Bd7 8. a4

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Bd7 8. a4'),
			moves => {
			},
		},
		'rn1qk2r/pp1bp1bp/3pp1p1/1Bp5/3P1Pn1/2N2N2/PPP3PP/R1BQK2R w KQkq - 0 9' => {
			eco => 'B09w',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7 7. e5 Ng4 8. e6 fxe6

			variation => N__('Pirc: Austrian, 5... c5, Seirawan Variation'),
			moves => {
			},
		},
		'rn1qk2r/pp1bppbp/3p1np1/1Bp1P3/3P1P2/2N2N2/PPP3PP/R1BQK2R b KQkq - 0 7' => {
			eco => 'B09t',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7 7. e5

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+ Bd7 7. e5'),
			moves => {
				'f6g4' => 'rn1qk2r/pp1bppbp/3p2p1/1Bp1P3/3P1Pn1/2N2N2/PPP3PP/R1BQK2R w KQkq - 1 8',
			},
		},
		'rn1qk2r/pp1bppbp/3p1np1/1Bp5/3PPP2/2N2N2/PPP3PP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B09s',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+'),
			moves => {
				'e4e5' => 'rn1qk2r/pp1bppbp/3p1np1/1Bp1P3/3P1P2/2N2N2/PPP3PP/R1BQK2R b KQkq - 0 7',
			},
		},
		'rn1qk2r/pp1bppbp/3p2p1/1Bp1P3/3P1Pn1/2N2N2/PPP3PP/R1BQK2R w KQkq - 1 8' => {
			eco => 'B09u',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7 7. e5 Ng4

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+ Bd7 7. e5 Ng4'),
			moves => {
				'b5d7' => 'rn1qk2r/pp1Bppbp/3p2p1/2p1P3/3P1Pn1/2N2N2/PPP3PP/R1BQK2R b KQkq - 0 8',
				'e5e6' => 'rn1qk2r/pp1bppbp/3pP1p1/1Bp5/3P1Pn1/2N2N2/PPP3PP/R1BQK2R b KQkq - 0 8',
			},
		},
		'rn1qk2r/pp1bppbp/3pP1p1/1Bp5/3P1Pn1/2N2N2/PPP3PP/R1BQK2R b KQkq - 0 8' => {
			eco => 'B09w',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7 7. e5 Ng4 8. e6

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+ Bd7 7. e5 Ng4 8. e6'),
			moves => {
				'd7b5' => 'rn1qk2r/pp2ppbp/3pP1p1/1bp5/3P1Pn1/2N2N2/PPP3PP/R1BQK2R w KQkq - 0 9',
				'f7e6' => 'rn1qk2r/pp1bp1bp/3pp1p1/1Bp5/3P1Pn1/2N2N2/PPP3PP/R1BQK2R w KQkq - 0 9',
			},
		},
		'rn1qk2r/pp2bppp/2p1bn2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KRR3 w kq - 2 10' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'c4d3' => 'rn1qk2r/pp2bppp/2p1bn2/3p2B1/3Q4/2NB1N2/PPP2PPP/2KRR3 b kq - 3 10',
			},
		},
		'rn1qk2r/pp2bppp/2p1bn2/3p2B1/3Q4/2NB1N2/PPP2PPP/2KRR3 b kq - 3 10' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'b8d7' => 'r2qk2r/pp1nbppp/2p1bn2/3p2B1/3Q4/2NB1N2/PPP2PPP/2KRR3 w kq - 4 11',
			},
		},
		'rn1qk2r/pp2bppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'D11o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O Be7

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'rn1qk2r/pp2bppp/2p2n2/3p1bB1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 3 8' => {
			eco => 'D35p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Nf3 Bf5

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Nf3 Bf5'),
			moves => {
				'f1d3' => 'rn1qk2r/pp2bppp/2p2n2/3p1bB1/3P4/2NBPN2/PP3PPP/R2QK2R b KQkq - 4 8',
			},
		},
		'rn1qk2r/pp2bppp/2p2n2/3p1bB1/3P4/2NBPN2/PP3PPP/R2QK2R b KQkq - 4 8' => {
			eco => 'D35q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Nf3 Bf5 8. Bd3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Nf3 Bf5 8. Bd3'),
			moves => {
			},
		},
		'rn1qk2r/pp2ppbp/3pP1p1/1bp5/3P1Pn1/2N2N2/PPP3PP/R1BQK2R w KQkq - 0 9' => {
			eco => 'B09x',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+ Bd7 7. e5 Ng4 8. e6 Bxb5

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+ Bd7 7. e5 Ng4 8. e6 Bxb5'),
			moves => {
			},
		},
		'rn1qk2r/pp2ppbp/6p1/2p5/3PP1b1/2P1BN2/P4PPP/R2QKB1R w KQkq - 2 9' => {
			eco => 'D85t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 Bg4

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 Bg4'),
			moves => {
			},
		},
		'rn1qk2r/pp3ppp/2p1p3/4N3/PbpPn3/2N5/1P1B2PP/R2QKB1R b KQkq - 1 10' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'd8d4' => 'rn2k2r/pp3ppp/2p1p3/4N3/Pbpqn3/2N5/1P1B2PP/R2QKB1R w KQkq - 0 11',
			},
		},
		'rn1qk2r/pp3ppp/2p1p3/4N3/PbpPn3/2N5/1P4PP/R1BQKB1R w KQkq - 0 10' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'c1d2' => 'rn1qk2r/pp3ppp/2p1p3/4N3/PbpPn3/2N5/1P1B2PP/R2QKB1R b KQkq - 1 10',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/4N3/PbpPP3/2N5/1P4PP/R1BQKB1R b KQkq - 0 9' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'f6e4' => 'rn1qk2r/pp3ppp/2p1p3/4N3/PbpPn3/2N5/1P4PP/R1BQKB1R w KQkq - 0 10',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/4N3/PbpPb3/2N2P2/1P4PP/R1BQKB1R w KQkq - 0 9' => {
			eco => 'D17p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. e4 Bxe4

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'f3e4' => 'rn1qk2r/pp3ppp/2p1pn2/4N3/PbpPP3/2N5/1P4PP/R1BQKB1R b KQkq - 0 9',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/4Nb2/PbpP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D17m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4

			variation => N__('Slav: Central, 7. f3 Bb4'),
			moves => {
				'c1g5' => 'rn1qk2r/pp3ppp/2p1pn2/4NbB1/PbpP4/2N2P2/1P2P1PP/R2QKB1R b KQkq - 2 8',
				'e2e4' => 'rn1qk2r/pp3ppp/2p1pn2/4Nb2/PbpPP3/2N2P2/1P4PP/R1BQKB1R b KQkq e3 0 8',
				'e5c4' => 'rn1qk2r/pp3ppp/2p1pn2/5b2/PbNP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/4Nb2/PbpPP3/2N2P2/1P4PP/R1BQKB1R b KQkq e3 0 8' => {
			eco => 'D17o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. e4

			variation => N__('Slav: Central, 7. f3 Bb4 8. e4'),
			moves => {
				'f5e4' => 'rn1qk2r/pp3ppp/2p1pn2/4N3/PbpPb3/2N2P2/1P4PP/R1BQKB1R w KQkq - 0 9',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/4NbB1/PbpP4/2N2P2/1P2P1PP/R2QKB1R b KQkq - 2 8' => {
			eco => 'D17m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. Bg5

			variation => N__('Slav: Central, 7. f3 Bb4 8. Bg5'),
			moves => {
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQ1RK1 b kq - 2 8' => {
			eco => 'D18e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4 8. O-O

			variation => N__('Slav: Dutch, 8.O-O'),
			moves => {
				'b8d7' => 'r2qk2r/pp1n1ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQ1RK1 w kq - 3 9',
				'e8g8' => 'rn1q1rk1/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQ1RK1 w - - 3 9',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQK2R w KQkq - 1 8' => {
			eco => 'D18d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6 7. Bxc4 Bb4

			variation => N__('Slav: Dutch, 6... e6 7.Bxc4 Bb4'),
			moves => {
				'e1g1' => 'rn1qk2r/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQ1RK1 b kq - 2 8',
			},
		},
		'rn1qk2r/pp3ppp/2p1pn2/5b2/PbNP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D17n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. Nxc4

			variation => N__('Slav: Central, 7. f3 Bb4 8.Nxc4'),
			moves => {
			},
		},
		'rn1qk2r/ppp1bppp/1n1Pp3/8/2PP2b1/5N2/PP2BPPP/RNBQ1RK1 b kq - 0 8' => {
			eco => 'B05q',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. exd6

			variation => N__('Alekhine: Modern, Main Line, 8.exd6'),
			moves => {
			},
		},
		'rn1qk2r/ppp1bppp/1n1pp3/4P2b/2PP4/2N2N1P/PP2BPP1/R1BQ1RK1 b kq - 2 9' => {
			eco => 'B05o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. h3 Bh5 9. Nc3

			variation => N__('Alekhine: Modern, Main Line, 8. h3 Bh5 9. Nc3'),
			moves => {
				'e8g8' => 'rn1q1rk1/ppp1bppp/1n1pp3/4P2b/2PP4/2N2N1P/PP2BPP1/R1BQ1RK1 w - - 3 10',
			},
		},
		'rn1qk2r/ppp1bppp/1n1pp3/4P2b/2PP4/5N1P/PP2BPP1/RNBQ1RK1 w kq - 1 9' => {
			eco => 'B05n',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. h3 Bh5

			variation => N__('Alekhine: Modern, Main Line, 8. h3 Bh5'),
			moves => {
				'b1c3' => 'rn1qk2r/ppp1bppp/1n1pp3/4P2b/2PP4/2N2N1P/PP2BPP1/R1BQ1RK1 b kq - 2 9',
			},
		},
		'rn1qk2r/ppp1bppp/1n1pp3/4P3/2PP2b1/2N2N2/PP2BPPP/R1BQ1RK1 b kq - 2 8' => {
			eco => 'B05r',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. Nc3

			variation => N__('Alekhine: Modern, Main Line, 8. Nc3'),
			moves => {
				'e8g8' => 'rn1q1rk1/ppp1bppp/1n1pp3/4P3/2PP2b1/2N2N2/PP2BPPP/R1BQ1RK1 w - - 3 9',
			},
		},
		'rn1qk2r/ppp1bppp/1n1pp3/4P3/2PP2b1/5N1P/PP2BPP1/RNBQ1RK1 b kq - 0 8' => {
			eco => 'B05n',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4 Nb6 8. h3

			variation => N__('Alekhine: Modern, Main Line, 8. h3'),
			moves => {
				'g4h5' => 'rn1qk2r/ppp1bppp/1n1pp3/4P2b/2PP4/5N1P/PP2BPP1/RNBQ1RK1 w kq - 1 9',
			},
		},
		'rn1qk2r/ppp1bppp/1n1pp3/4P3/2PP2b1/5N2/PP2BPPP/RNBQ1RK1 w kq - 1 8' => {
			eco => 'B05m',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, Main Line'),
			moves => {
				'b1c3' => 'rn1qk2r/ppp1bppp/1n1pp3/4P3/2PP2b1/2N2N2/PP2BPPP/R1BQ1RK1 b kq - 2 8',
				'e5d6' => 'rn1qk2r/ppp1bppp/1n1Pp3/8/2PP2b1/5N2/PP2BPPP/RNBQ1RK1 b kq - 0 8',
				'h2h3' => 'rn1qk2r/ppp1bppp/1n1pp3/4P3/2PP2b1/5N1P/PP2BPP1/RNBQ1RK1 b kq - 0 8',
			},
		},
		'rn1qk2r/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQK2R b KQkq - 3 9' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'e8g8' => 'rn1q1rk1/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQK2R w KQ - 4 10',
			},
		},
		'rn1qk2r/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP4PP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'f1e2' => 'rn1qk2r/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP2B1PP/R1BQK2R b KQkq - 3 9',
			},
		},
		'rn1qk2r/ppp1bppp/3pp3/3nP3/2PP2b1/5N2/PP2BPPP/RNBQ1RK1 b kq c3 0 7' => {
			eco => 'B05m',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O Be7 7. c4

			variation => N__('Alekhine: Modern, Main Line'),
			moves => {
				'd5b6' => 'rn1qk2r/ppp1bppp/1n1pp3/4P3/2PP2b1/5N2/PP2BPPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'rn1qk2r/ppp1bppp/3pp3/3nP3/3P2b1/5N2/PPP1BPPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'B05l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, 5. Be2 e6 6.O-O'),
			moves => {
				'c2c4' => 'rn1qk2r/ppp1bppp/3pp3/3nP3/2PP2b1/5N2/PP2BPPP/RNBQ1RK1 b kq c3 0 7',
			},
		},
		'rn1qk2r/ppp1ppbp/3p1np1/8/2PPP1b1/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E90b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 Bg4

			variation => N__('King\'s Indian: 5. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1qk2r/ppp2ppp/1n2p3/4Pb2/1bPP4/2N2N2/PP4PP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'f1d3' => 'rn1qk2r/ppp2ppp/1n2p3/4Pb2/1bPP4/2NB1N2/PP4PP/R1BQK2R b KQkq - 3 9',
			},
		},
		'rn1qk2r/ppp2ppp/1n2p3/4Pb2/1bPP4/2NB1N2/PP4PP/R1BQK2R b KQkq - 3 9' => {
			eco => 'B03l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 Bf5 7. Nc3 e6 8. Nf3 Bb4 9. Bd3

			variation => N__('Alekhine: Four Pawns Attack, Fahrni Variation'),
			moves => {
			},
		},
		'rn1qk2r/ppp2ppp/8/2bP1b2/4nP2/2N2N2/PPP1Q1PP/R1B1KB1R b KQkq - 5 8' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Nf3 Bc5 7. Qe2 Bf5 8. Nc3

			variation => N__('KGD: Falkbeer, Main Line, 7... Bf5 8. Nc3'),
			moves => {
			},
		},
		'rn1qk2r/ppp2ppp/8/2bP1b2/4nP2/5N2/PPP1Q1PP/RNB1KB1R w KQkq - 4 8' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Nf3 Bc5 7. Qe2 Bf5

			variation => N__('KGD: Falkbeer, Main Line, 7... Bf5'),
			moves => {
				'b1c3' => 'rn1qk2r/ppp2ppp/8/2bP1b2/4nP2/2N2N2/PPP1Q1PP/R1B1KB1R b KQkq - 5 8',
				'g2g4' => 'rn1qk2r/ppp2ppp/8/2bP1b2/4nPP1/5N2/PPP1Q2P/RNB1KB1R b KQkq g3 0 8',
			},
		},
		'rn1qk2r/ppp2ppp/8/2bP1b2/4nPP1/5N2/PPP1Q2P/RNB1KB1R b KQkq g3 0 8' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, Main Line, 7... Bf5'),
			moves => {
				'e8g8' => 'rn1q1rk1/ppp2ppp/8/2bP1b2/4nPP1/5N2/PPP1Q2P/RNB1KB1R w KQ - 1 9',
			},
		},
		'rn1qkb1r/1b1p1ppp/pp2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A30i',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 a6

			variation => N__('English: Symmetrical, Hedgehog, 6... a6'),
			moves => {
			},
		},
		'rn1qkb1r/1b3ppp/p2ppn2/1p6/3NP1P1/2N1BP2/PPPQ3P/R3KB1R b KQkq g3 0 9' => {
			eco => 'B80l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 b5 8. Qd2 Bb7 9. g4

			variation => N__('Sicilian: Scheveningen, English, 7... b5 8. Qd2 Bb7'),
			moves => {
			},
		},
		'rn1qkb1r/1b3ppp/p2ppn2/1p6/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 2 9' => {
			eco => 'B80l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 b5 8. Qd2 Bb7

			variation => N__('Sicilian: Scheveningen, English, 7... b5, 8. Qd2 Bb7'),
			moves => {
				'g2g4' => 'rn1qkb1r/1b3ppp/p2ppn2/1p6/3NP1P1/2N1BP2/PPPQ3P/R3KB1R b KQkq g3 0 9',
			},
		},
		'rn1qkb1r/1b3ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNB2RK1 w kq - 2 9' => {
			eco => 'D29a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7

			variation => N__('QGA: Classical, 8... Bb7'),
			moves => {
				'a2a4' => 'rn1qkb1r/1b3ppp/p3pn2/1pp5/P2P4/1B2PN2/1P2QPPP/RNB2RK1 b kq a3 0 9',
				'b1c3' => 'rn1qkb1r/1b3ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 b kq - 3 9',
				'f1d1' => 'rn1qkb1r/1b3ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNBR2K1 b kq - 3 9',
			},
		},
		'rn1qkb1r/1b3ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNBR2K1 b kq - 3 9' => {
			eco => 'D29f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1'),
			moves => {
				'b8d7' => 'r2qkb1r/1b1n1ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNBR2K1 w kq - 4 10',
			},
		},
		'rn1qkb1r/1b3ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1B2RK1 b kq - 3 9' => {
			eco => 'D29b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Nc3

			variation => N__('QGA: Classical, 8... Bb7 9. Nc3'),
			moves => {
			},
		},
		'rn1qkb1r/1b3ppp/p3pn2/1pp5/P2P4/1B2PN2/1P2QPPP/RNB2RK1 b kq a3 0 9' => {
			eco => 'D29c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. a4

			variation => N__('QGA: Classical, 8... Bb7 9. a4'),
			moves => {
				'b5b4' => 'rn1qkb1r/1b3ppp/p3pn2/2p5/Pp1P4/1B2PN2/1P2QPPP/RNB2RK1 w kq - 0 10',
				'b8d7' => 'r2qkb1r/1b1n1ppp/p3pn2/1pp5/P2P4/1B2PN2/1P2QPPP/RNB2RK1 w kq - 1 10',
			},
		},
		'rn1qkb1r/1b3ppp/p3pn2/2p5/Pp1P4/1B2PN2/1P2QPPP/RNB2RK1 w kq - 0 10' => {
			eco => 'D29d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. a4 b4

			variation => N__('QGA: Classical, 8... Bb7 9. a4 b4'),
			moves => {
			},
		},
		'rn1qkb1r/1bp1pppp/p7/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D24o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 Bb7

			variation => N__('QGA: Bogoljubow, 7. a4 Bb7'),
			moves => {
			},
		},
		'rn1qkb1r/1bp1pppp/p7/1p2P3/P1pP4/2P2N2/5PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'D24r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 Nxc3 8. bxc3 Bb7

			variation => N__('QGA: Bogoljubow, 7. a4 Nxc3'),
			moves => {
			},
		},
		'rn1qkb1r/1p1b1ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B96e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Bd7

			variation => N__('Sicilian: Najdorf, 7. f4 Bd7'),
			moves => {
			},
		},
		'rn1qkb1r/1p2pppp/p2p1n2/8/3NP1b1/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B91e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 Bg4

			variation => N__('Sicilian: Najdorf, 6. g3 Bg4'),
			moves => {
			},
		},
		'rn1qkb1r/1p3p1p/p2p1np1/2pP4/P3P1b1/2N2N2/1P2BPPP/R1BQK2R b KQkq - 2 9' => {
			eco => 'A70c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 a6 8. a4 Bg4 9. Be2

			variation => N__('Benoni: Classical, 7... a6 8. a4 Bg4 9. Be2'),
			moves => {
			},
		},
		'rn1qkb1r/1p3p1p/p2p1np1/2pP4/P3P1b1/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'A70b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 a6 8. a4 Bg4

			variation => N__('Benoni: Classical, 7... a6 8. a4 Bg4'),
			moves => {
				'f1e2' => 'rn1qkb1r/1p3p1p/p2p1np1/2pP4/P3P1b1/2N2N2/1P2BPPP/R1BQK2R b KQkq - 2 9',
			},
		},
		'rn1qkb1r/1p3ppp/p2pbn2/4p3/4P3/1NN1B3/PPP2PPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'B90l',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3'),
			moves => {
				'd1d2' => 'rn1qkb1r/1p3ppp/p2pbn2/4p3/4P3/1NN1B3/PPPQ1PPP/R3KB1R b KQkq - 3 8',
				'f2f3' => 'rn1qkb1r/1p3ppp/p2pbn2/4p3/4P3/1NN1BP2/PPP3PP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/1p3ppp/p2pbn2/4p3/4P3/1NN1B3/PPPQ1PPP/R3KB1R b KQkq - 3 8' => {
			eco => 'B90m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. Qd2

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. Qd2'),
			moves => {
			},
		},
		'rn1qkb1r/1p3ppp/p2pbn2/4p3/4P3/1NN1BP2/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B90n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3 Be6 8. f3

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3 Be6 8. f3'),
			moves => {
				'b8d7' => 'r2qkb1r/1p1n1ppp/p2pbn2/4p3/4P3/1NN1BP2/PPP3PP/R2QKB1R w KQkq - 1 9',
				'f8e7' => 'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN1BP2/PPP3PP/R2QKB1R w KQkq - 1 9',
			},
		},
		'rn1qkb1r/1pp2ppp/p3pn2/7b/2BP2P1/2N1PN1P/PP3P2/R1BQK2R b KQkq g3 0 8' => {
			eco => 'D25o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 a6 8. g4

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 a6 8. g4'),
			moves => {
			},
		},
		'rn1qkb1r/1pp2ppp/p3pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQK2R w KQkq - 0 8' => {
			eco => 'D25n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3 a6

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3 a6'),
			moves => {
				'g2g4' => 'rn1qkb1r/1pp2ppp/p3pn2/7b/2BP2P1/2N1PN1P/PP3P2/R1BQK2R b KQkq g3 0 8',
			},
		},
		'rn1qkb1r/3ppp1p/b4np1/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'A58f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 g6

			variation => N__('Benko Gambit: Accepted, 6. Nc3 g6'),
			moves => {
			},
		},
		'rn1qkb1r/3ppppp/b4n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 6' => {
			eco => 'A58e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3

			variation => N__('Benko Gambit: Accepted, 6. Nc3'),
			moves => {
				'd7d6' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 7',
				'g7g6' => 'rn1qkb1r/3ppp1p/b4np1/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rn1qkb1r/3ppppp/b4n2/2pP4/8/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 6' => {
			eco => 'A58c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. g3

			variation => N__('Benko Gambit: Accepted, 6. g3'),
			moves => {
				'd7d6' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/8/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 7',
			},
		},
		'rn1qkb1r/3ppppp/b4n2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A58c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6

			variation => N__('Benko Gambit: 5.bxa6 Bxa6'),
			moves => {
				'b1c3' => 'rn1qkb1r/3ppppp/b4n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 6',
				'g2g3' => 'rn1qkb1r/3ppppp/b4n2/2pP4/8/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 6',
			},
		},
		'rn1qkb1r/4pp1p/3p1np1/2pP4/4P1P1/2N5/PP3P1P/R1BQ1KNR b kq g3 0 9' => {
			eco => 'A59g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g4

			variation => N__('Benko Gambit: 7. e4 Line, 9. g4'),
			moves => {
			},
		},
		'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQ1K1R b kq - 1 9' => {
			eco => 'A59d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. Nf3

			variation => N__('Benko Gambit: 7. e4 Line, 9. Nf3'),
			moves => {
				'f8g7' => 'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQ1K1R w kq - 2 10',
			},
		},
		'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N3P1/PP3P1P/R1BQ1KNR b kq - 0 9' => {
			eco => 'A59h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. g3

			variation => N__('Benko Gambit: 7. e4 Line, 9. g3'),
			moves => {
				'f8g7' => 'rn1qk2r/4ppbp/3p1np1/2pP4/4P3/2N3P1/PP3P1P/R1BQ1KNR w kq - 1 10',
			},
		},
		'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N5/PP2NPPP/R1BQ1K1R b kq - 1 9' => {
			eco => 'A59c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6 9. Nge2

			variation => N__('Benko Gambit: 7. e4 Line, 9.Nge2'),
			moves => {
			},
		},
		'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N5/PP3PPP/R1BQ1KNR w kq - 0 9' => {
			eco => 'A59b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4 Bxf1 8. Kxf1 g6

			variation => N__('Benko Gambit: 7. e4 Bxf1 8.Kxf1 g6'),
			moves => {
				'g1e2' => 'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N5/PP2NPPP/R1BQ1K1R b kq - 1 9',
				'g1f3' => 'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQ1K1R b kq - 1 9',
				'g2g3' => 'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N3P1/PP3P1P/R1BQ1KNR b kq - 0 9',
				'g2g4' => 'rn1qkb1r/4pp1p/3p1np1/2pP4/4P1P1/2N5/PP3P1P/R1BQ1KNR b kq g3 0 9',
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/5P2/2N5/PP2P1PP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'A58g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. f4 g6

			variation => N__('Benko Gambit: Accepted, 7. f4'),
			moves => {
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/1P4P1/P3PPBP/RNBQK1NR b KQkq - 0 8' => {
			eco => 'A58d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. g3 d6 7. Bg2 g6 8. b3

			variation => N__('Benko Gambit: Accepted, 6. g3 d6 7. Bg2 g6 8. b3'),
			moves => {
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'A58i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6

			variation => N__('Benko Gambit: Accepted, 7. Nf3 g6'),
			moves => {
				'f3d2' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R b KQkq - 1 8',
				'g2g3' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 8' => {
			eco => 'A58k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. g3

			variation => N__('Benko Gambit: Fianchetto Variation'),
			moves => {
				'f8g7' => 'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 9',
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N3P1/PP2PP1P/R1BQKBNR w KQkq - 0 8' => {
			eco => 'A58h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. g3 g6

			variation => N__('Benko Gambit: Accepted, 7. g3'),
			moves => {
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'A58j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. Nd2

			variation => N__('Benko Gambit: Accepted, 7. Nf3 g6 8. Nd2'),
			moves => {
				'd8a5' => 'rn2kb1r/4pp1p/b2p1np1/q1pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 9',
				'f8g7' => 'rn1qk2r/4ppbp/b2p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 9',
			},
		},
		'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 8' => {
			eco => 'A58d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. g3 d6 7. Bg2 g6

			variation => N__('Benko Gambit: Accepted, 6. g3 d6 7. Bg2 g6'),
			moves => {
				'b2b3' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/1P4P1/P3PPBP/RNBQK1NR b KQkq - 0 8',
			},
		},
		'rn1qkb1r/4pppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQ1KNR b kq - 0 8' => {
			eco => 'A59a',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 7. e4'),
			moves => {
				'g7g6' => 'rn1qkb1r/4pp1p/3p1np1/2pP4/4P3/2N5/PP3PPP/R1BQ1KNR w kq - 0 9',
			},
		},
		'rn1qkb1r/4pppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKbNR w KQkq - 0 8' => {
			eco => 'A59a',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 7. e4'),
			moves => {
				'e1f1' => 'rn1qkb1r/4pppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQ1KNR b kq - 0 8',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 7' => {
			eco => 'A59a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. e4

			variation => N__('Benko Gambit: 7. e4'),
			moves => {
				'a6f1' => 'rn1qkb1r/4pppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKbNR w KQkq - 0 8',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/5P2/2N5/PP2P1PP/R1BQKBNR b KQkq f3 0 7' => {
			eco => 'A58g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. f4

			variation => N__('Benko Gambit: Accepted, 7. f4'),
			moves => {
				'g7g6' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/5P2/2N5/PP2P1PP/R1BQKBNR w KQkq - 0 8',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'A58i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3

			variation => N__('Benko Gambit: Accepted, 7. Nf3'),
			moves => {
				'g7g6' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/8/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 7' => {
			eco => 'A58h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. g3

			variation => N__('Benko Gambit: Accepted, 7. g3'),
			moves => {
				'g7g6' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/2N3P1/PP2PP1P/R1BQKBNR w KQkq - 0 8',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'A58g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6

			variation => N__('Benko Gambit: Accepted, 6. Nc3 d6'),
			moves => {
				'e2e4' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 7',
				'f2f4' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/5P2/2N5/PP2P1PP/R1BQKBNR b KQkq f3 0 7',
				'g1f3' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 7',
				'g2g3' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/8/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 7',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/8/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 7' => {
			eco => 'A58c',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Accepted, 6. g3'),
			moves => {
				'f1g2' => 'rn1qkb1r/4pppp/b2p1n2/2pP4/8/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 7',
			},
		},
		'rn1qkb1r/4pppp/b2p1n2/2pP4/8/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 7' => {
			eco => 'A58c',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Accepted, 6. g3'),
			moves => {
				'g7g6' => 'rn1qkb1r/4pp1p/b2p1np1/2pP4/8/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 8',
			},
		},
		'rn1qkb1r/p1p2ppp/bp2pn2/3P4/3P4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 6' => {
			eco => 'E15m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 d5 6. cxd5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 d5 6.cxd5'),
			moves => {
			},
		},
		'rn1qkb1r/p1p2ppp/bp2pn2/3p4/2PP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq d6 0 6' => {
			eco => 'E15m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 d5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 d5'),
			moves => {
				'c4d5' => 'rn1qkb1r/p1p2ppp/bp2pn2/3P4/3P4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 6',
				'f1g2' => 'rn1qkb1r/p1p2ppp/bp2pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/p1p2ppp/bp2pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQK2R b KQkq - 1 6' => {
			eco => 'E15m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 d5 6. Bg2

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 d5 6. Bg2'),
			moves => {
			},
		},
		'rn1qkb1r/p1pp1ppp/1p2pn2/8/2PP4/P1N2b2/1P2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'E12j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 Bxf3

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 Bxf3'),
			moves => {
			},
		},
		'rn1qkb1r/p1pp1ppp/b3pn2/1P6/3P4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 6' => {
			eco => 'E15l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 b5 6. cxb5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 b5 6.cxb5'),
			moves => {
			},
		},
		'rn1qkb1r/p1pp1ppp/b3pn2/1p6/2PP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq - 0 6' => {
			eco => 'E15l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 b5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 b5'),
			moves => {
				'c4b5' => 'rn1qkb1r/p1pp1ppp/b3pn2/1P6/3P4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'E15l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 '),
			moves => {
				'a6b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq - 1 6',
				'b6b5' => 'rn1qkb1r/p1pp1ppp/b3pn2/1p6/2PP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq - 0 6',
				'd7d5' => 'rn1qkb1r/p1p2ppp/bp2pn2/3p4/2PP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq d6 0 6',
				'f8b4' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/1Q3NP1/PP2PP1P/RNB1KB1R b KQkq - 2 5' => {
			eco => 'E15d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qb3

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qb3'),
			moves => {
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PP1NPP1P/R1BQKB1R b KQkq - 2 5' => {
			eco => 'E15e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Nbd2

			variation => N__('Queen\'s Indian: Nimzowitsch, 5.Nbd2'),
			moves => {
				'a6b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1NPP1P/R1BQKB1R w KQkq - 3 6',
				'c7c5' => 'rn1qkb1r/p2p1ppp/bp2pn2/2p5/2PP4/5NP1/PP1NPP1P/R1BQKB1R w KQkq c6 0 6',
				'f8b4' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/5NP1/PP1NPP1P/R1BQKB1R w KQkq - 3 6',
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E15d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6

			variation => N__('Queen\'s Indian: Nimzowitsch Variation (4. g3 Ba6)'),
			moves => {
				'b1d2' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PP1NPP1P/R1BQKB1R b KQkq - 2 5',
				'b2b3' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/1P3NP1/P3PP1P/RNBQKB1R b KQkq - 0 5',
				'd1a4' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/Q1PP4/5NP1/PP2PP1P/RNB1KB1R b KQkq - 2 5',
				'd1b3' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/1Q3NP1/PP2PP1P/RNB1KB1R b KQkq - 2 5',
				'd1c2' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PPQ1PP1P/RNB1KB1R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PPQ1PP1P/RNB1KB1R b KQkq - 2 5' => {
			eco => 'E15d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qc2

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qc2'),
			moves => {
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/P3PN2/1P3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'E12g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. e3

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. e3'),
			moves => {
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E12g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6'),
			moves => {
				'd1c2' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/P4N2/1PQ1PPPP/RNB1KB1R b KQkq - 2 5',
				'e2e3' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/P3PN2/1P3PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/P4N2/1PQ1PPPP/RNB1KB1R b KQkq - 2 5' => {
			eco => 'E12g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. Qc2

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2'),
			moves => {
				'a6b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P4N2/1PQ1PPPP/RNB1KB1R w KQkq - 3 6',
				'c7c5' => 'rn1qkb1r/p2p1ppp/bp2pn2/2p5/2PP4/P4N2/1PQ1PPPP/RNB1KB1R w KQkq c6 0 6',
			},
		},
		'rn1qkb1r/p1pp1ppp/bp2pn2/8/Q1PP4/5NP1/PP2PP1P/RNB1KB1R b KQkq - 2 5' => {
			eco => 'E15h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4'),
			moves => {
				'c7c5' => 'rn1qkb1r/p2p1ppp/bp2pn2/2p5/Q1PP4/5NP1/PP2PP1P/RNB1KB1R w KQkq c6 0 6',
				'c7c6' => 'rn1qkb1r/p2p1ppp/bpp1pn2/8/Q1PP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 0 6',
				'f8e7' => 'rn1qk2r/p1ppbppp/bp2pn2/8/Q1PP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 3 6',
			},
		},
		'rn1qkb1r/p2p1p1p/bp3np1/2pP4/8/P4N2/1PQ1PPPP/RNB1KB1R w KQkq - 0 8' => {
			eco => 'E12f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 c5 5. d5 Ba6 6. Qc2 exd5 7. cxd5 g6

			variation => N__('Queen\'s Indian: Petrosian, 4... c5 5. d5 Ba6 6. Qc2 exd5 6.cxd5 g6'),
			moves => {
			},
		},
		'rn1qkb1r/p2p1ppp/b1p1pn2/1p6/Q1PP4/2N2NP1/PP2PP1P/R1B1KB1R w KQkq - 0 7' => {
			eco => 'E15i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 c6 6. Nc3 b5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c6 6. Nc3 b5'),
			moves => {
			},
		},
		'rn1qkb1r/p2p1ppp/bp2pn2/2p5/2PP4/5NP1/PP1NPP1P/R1BQKB1R w KQkq c6 0 6' => {
			eco => 'E15e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Nbd2 c5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5.Nbd2 c5'),
			moves => {
			},
		},
		'rn1qkb1r/p2p1ppp/bp2pn2/2p5/2PP4/P4N2/1PQ1PPPP/RNB1KB1R w KQkq c6 0 6' => {
			eco => 'E12g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. Qc2 c5

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 c5'),
			moves => {
			},
		},
		'rn1qkb1r/p2p1ppp/bp2pn2/2p5/Q1PP4/5NP1/PP2PP1P/RNB1KB1R w KQkq c6 0 6' => {
			eco => 'E15j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 c5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c5'),
			moves => {
				'f1g2' => 'rn1qkb1r/p2p1ppp/bp2pn2/2p5/Q1PP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/p2p1ppp/bp2pn2/2p5/Q1PP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 6' => {
			eco => 'E15j',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c5'),
			moves => {
				'a6b7' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/Q1PP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 2 7',
			},
		},
		'rn1qkb1r/p2p1ppp/bp2pn2/2pP4/2P5/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'E12e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 c5 5. d5 Ba6

			variation => N__('Queen\'s Indian: Petrosian, 4... c5 5. d5 Ba6'),
			moves => {
				'd1c2' => 'rn1qkb1r/p2p1ppp/bp2pn2/2pP4/2P5/P4N2/1PQ1PPPP/RNB1KB1R b KQkq - 2 6',
			},
		},
		'rn1qkb1r/p2p1ppp/bp2pn2/2pP4/2P5/P4N2/1PQ1PPPP/RNB1KB1R b KQkq - 2 6' => {
			eco => 'E12f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 c5 5. d5 Ba6 6. Qc2

			variation => N__('Queen\'s Indian: Petrosian, 4... c5 5. d5 Ba6 6. Qc2'),
			moves => {
				'e6d5' => 'rn1qkb1r/p2p1ppp/bp3n2/2pp4/2P5/P4N2/1PQ1PPPP/RNB1KB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/p2p1ppp/bp3n2/2pP4/8/P4N2/1PQ1PPPP/RNB1KB1R b KQkq - 0 7' => {
			eco => 'E12f',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 4... c5 5. d5 Ba6 6. Qc2'),
			moves => {
				'g7g6' => 'rn1qkb1r/p2p1p1p/bp3np1/2pP4/8/P4N2/1PQ1PPPP/RNB1KB1R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/p2p1ppp/bp3n2/2pp4/2P5/P4N2/1PQ1PPPP/RNB1KB1R w KQkq - 0 7' => {
			eco => 'E12f',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 4... c5 5. d5 Ba6 6. Qc2'),
			moves => {
				'c4d5' => 'rn1qkb1r/p2p1ppp/bp3n2/2pP4/8/P4N2/1PQ1PPPP/RNB1KB1R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/p2p1ppp/bpp1pn2/8/Q1PP4/2N2NP1/PP2PP1P/R1B1KB1R b KQkq - 1 6' => {
			eco => 'E15i',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c6'),
			moves => {
				'b6b5' => 'rn1qkb1r/p2p1ppp/b1p1pn2/1p6/Q1PP4/2N2NP1/PP2PP1P/R1B1KB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/p2p1ppp/bpp1pn2/8/Q1PP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 0 6' => {
			eco => 'E15i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 c6

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c6'),
			moves => {
				'b1c3' => 'rn1qkb1r/p2p1ppp/bpp1pn2/8/Q1PP4/2N2NP1/PP2PP1P/R1B1KB1R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2P5/Q1P5/5NP1/PP2PPBP/RNB1K2R b KQkq - 0 7' => {
			eco => 'E15j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 c5 6. Bg2 Bb7 7. dxc5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c5 6. Bg2 Bb7 7.dxc5'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A30h',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3

			variation => N__('English: Symmetrical, Hedgehog System'),
			moves => {
				'a7a6' => 'rn1qkb1r/1b1p1ppp/pp2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7',
				'd7d6' => 'rn1qkb1r/pb3ppp/1p1ppn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7',
				'f8e7' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A30h',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6

			variation => N__('English: Symmetrical, Hedgehog System'),
			moves => {
				'b1c3' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'E14q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. Nc3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6. Nc3'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'E14r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O'),
			moves => {
				'c5d4' => 'rn1qkb1r/pb1p1ppp/1p2pn2/8/2Pp4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 0 7',
				'f8e7' => 'rn1qk2r/pb1pbppp/1p2pn2/2p5/2PP4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/3BPN2/PP3PPP/RNBQK2R w KQkq c6 0 6' => {
			eco => 'E14q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5'),
			moves => {
				'b1c3' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 6',
				'e1g1' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/5NP1/PP1NPPBP/R1BQK2R w KQkq c6 0 7' => {
			eco => 'E15f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Nbd2 Bb7 6. Bg2 c5

			variation => N__('Queen\'s Indian: Nimzowitsch, 5.Nbd2 Bb7 6. Bg2 c5'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq c6 0 6' => {
			eco => 'E15u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 c5

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 c5'),
			moves => {
				'd4d5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2pP4/2P5/5NP1/PP2PPBP/RNBQK2R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/P1N2N2/1PQ1PPPP/R1B1KB1R w KQkq c6 0 7' => {
			eco => 'E12h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. Qc2 Bb7 6. Nc3 c5

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7 6. Nc3 c5'),
			moves => {
				'e2e4' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PPP3/P1N2N2/1PQ2PPP/R1B1KB1R b KQkq e3 0 7',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PPP3/P1N2N2/1PQ2PPP/R1B1KB1R b KQkq e3 0 7' => {
			eco => 'E12i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. Qc2 Bb7 6. Nc3 c5 7. e4

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7 6. Nc3 c5 7. e4'),
			moves => {
				'c5d4' => 'rn1qkb1r/pb1p1ppp/1p2pn2/8/2PpP3/P1N2N2/1PQ2PPP/R1B1KB1R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/3P4/3BPN2/PPP2PPP/RNBQ1RK1 w kq c6 0 6' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 Bb7 4. Bd3 e6 5. O-O c5

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/Q1PP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 2 7' => {
			eco => 'E15j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 c5 6. Bg2 Bb7

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c5 6. Bg2 Bb7'),
			moves => {
				'd4c5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2P5/Q1P5/5NP1/PP2PPBP/RNB1K2R b KQkq - 0 7',
				'e1g1' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/Q1PP4/5NP1/PP2PPBP/RNB2RK1 b kq - 3 7',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/Q1PP4/5NP1/PP2PPBP/RNB2RK1 b kq - 3 7' => {
			eco => 'E15k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Qa4 c5 6. Bg2 Bb7 7. O-O

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. Qa4 c5 6. Bg2 Bb7 7.O-O'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/2pP4/2P5/5NP1/PP2PPBP/RNBQK2R b KQkq - 0 6' => {
			eco => 'E15u',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 c5'),
			moves => {
				'e6d5' => 'rn1qkb1r/pb1p1ppp/1p3n2/2pp4/2P5/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/8/2PNP3/P1N5/1PQ2PPP/R1B1KB1R b KQkq - 0 8' => {
			eco => 'E12i',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7 6. Nc3 c5 7. e4'),
			moves => {
				'b8c6' => 'r2qkb1r/pb1p1ppp/1pn1pn2/8/2PNP3/P1N5/1PQ2PPP/R1B1KB1R w KQkq - 1 9',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/8/2PP4/3B1N2/PP3PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'E14s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O cxd4 7. exd4

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O cxd4 7.exd4'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/8/2Pp4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'E14s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 c5 6. O-O cxd4

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 c5 6.O-O cxd4'),
			moves => {
				'e3d4' => 'rn1qkb1r/pb1p1ppp/1p2pn2/8/2PP4/3B1N2/PP3PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p2pn2/8/2PpP3/P1N2N2/1PQ2PPP/R1B1KB1R w KQkq - 0 8' => {
			eco => 'E12i',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7 6. Nc3 c5 7. e4'),
			moves => {
				'f3d4' => 'rn1qkb1r/pb1p1ppp/1p2pn2/8/2PNP3/P1N5/1PQ2PPP/R1B1KB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/pb1p1ppp/1p3n2/2pp2N1/2P5/6P1/PP2PPBP/RNBQK2R b KQkq - 1 7' => {
			eco => 'E15u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 c5 6. d5 exd5 7. Ng5

			variation => N__('Queen\'s Indian: Buerger Variation'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p3n2/2pp4/2P4N/6P1/PP2PPBP/RNBQK2R b KQkq - 1 7' => {
			eco => 'E15v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 c5 6. d5 exd5 7. Nh4

			variation => N__('Queen\'s Indian: Rubinstein Variation'),
			moves => {
			},
		},
		'rn1qkb1r/pb1p1ppp/1p3n2/2pp4/2P5/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 7' => {
			eco => 'E15u',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 c5'),
			moves => {
				'f3g5' => 'rn1qkb1r/pb1p1ppp/1p3n2/2pp2N1/2P5/6P1/PP2PPBP/RNBQK2R b KQkq - 1 7',
				'f3h4' => 'rn1qkb1r/pb1p1ppp/1p3n2/2pp4/2P4N/6P1/PP2PPBP/RNBQK2R b KQkq - 1 7',
			},
		},
		'rn1qkb1r/pb1ppp1p/1p3np1/2p5/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'A30g',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O g6 6. b3

			variation => N__('English: Symmetrical, Mutual Double Fianchetto'),
			moves => {
				'f8g7' => 'rn1qk2r/pb1pppbp/1p3np1/2p5/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 w kq - 1 7',
			},
		},
		'rn1qkb1r/pb1ppp1p/1p3np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A30f',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Double Fianchetto'),
			moves => {
				'f8g7' => 'rn1qk2r/pb1pppbp/1p3np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkb1r/pb1ppp1p/1p3np1/2p5/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A30f',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O g6

			variation => N__('English: Symmetrical, Double Fianchetto'),
			moves => {
				'b1c3' => 'rn1qkb1r/pb1ppp1p/1p3np1/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
				'b2b3' => 'rn1qkb1r/pb1ppp1p/1p3np1/2p5/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5' => {
			eco => 'A30e',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. Nc3

			variation => N__('English: Symmetrical, b6 System'),
			moves => {
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/2p5/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'A30e',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O

			variation => N__('English: Symmetrical, b6 System'),
			moves => {
				'e7e6' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6',
				'g7g6' => 'rn1qkb1r/pb1ppp1p/1p3np1/2p5/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/2p5/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A30e',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7

			variation => N__('English: Symmetrical, b6 System'),
			moves => {
				'b1c3' => 'rn1qkb1r/pb1ppppp/1p3n2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5',
				'e1g1' => 'rn1qkb1r/pb1ppppp/1p3n2/2p5/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/2p5/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq c3 0 5' => {
			eco => 'A47',
                        # TRANSLATORS:

			variation => N__('Neo-Queen\'s Indian: Marienbad System'),
			moves => {
				'c5d4' => 'rn1qkb1r/pb1ppppp/1p3n2/8/2Pp4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq c6 0 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. g3 Bb7 4. Bg2 c5

			variation => N__('Neo-Queen\'s Indian: Marienbad System'),
			moves => {
				'c2c4' => 'rn1qkb1r/pb1ppppp/1p3n2/2p5/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq c3 0 5',
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 b6 5. Nc3 Bb7

			variation => N__('English: Symmetrical, Two Knights, 4... b6'),
			moves => {
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/8/2PQ4/5NP1/PP2PPBP/RNB1K2R b KQkq - 0 6' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. g3 Bb7 4. Bg2 c5 5. c4 cxd4 6. Qxd4

			variation => N__('Neo-Queen\'s Indian: Marienbad System, Berg Variation'),
			moves => {
			},
		},
		'rn1qkb1r/pb1ppppp/1p3n2/8/2Pp4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'A47',
                        # TRANSLATORS:

			variation => N__('Neo-Queen\'s Indian: Marienbad System'),
			moves => {
				'd1d4' => 'rn1qkb1r/pb1ppppp/1p3n2/8/2PQ4/5NP1/PP2PPBP/RNB1K2R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/pb1ppppp/5n2/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'A57e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3 Bb7

			variation => N__('Benko Gambit: 4. Nf3 Bb7'),
			moves => {
				'a2a4' => 'rn1qkb1r/pb1ppppp/5n2/1ppP4/P1P5/5N2/1P2PPPP/RNBQKB1R b KQkq a3 0 5',
			},
		},
		'rn1qkb1r/pb1ppppp/5n2/1ppP4/P1P5/5N2/1P2PPPP/RNBQKB1R b KQkq a3 0 5' => {
			eco => 'A57e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3 Bb7 5. a4

			variation => N__('Benko Gambit: 4. Nf3 Bb7 5. a4'),
			moves => {
			},
		},
		'rn1qkb1r/pb3p2/2p1pn1p/1p4p1/2pPP3/2N2NB1/PP2BPPP/R2QK2R w KQkq - 2 10' => {
			eco => 'D43n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4 dxc4 7. e4 g5 8. Bg3 b5 9. Be2 Bb7

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit, Main Line, 9... Bb7'),
			moves => {
			},
		},
		'rn1qkb1r/pb3ppp/1p1ppn2/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 b kq - 0 7' => {
			eco => 'A30k',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 d6 7. b3

			variation => N__('English: Symmetrical, Hedgehog, 6... d6 7. b3'),
			moves => {
			},
		},
		'rn1qkb1r/pb3ppp/1p1ppn2/2p5/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w kq - 0 7' => {
			eco => 'A30j',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 d6

			variation => N__('English: Symmetrical, Hedgehog, 6... d6'),
			moves => {
				'b2b3' => 'rn1qkb1r/pb3ppp/1p1ppn2/2p5/2P5/1PN2NP1/P2PPPBP/R1BQ1RK1 b kq - 0 7',
				'd2d4' => 'rn1qkb1r/pb3ppp/1p1ppn2/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq d3 0 7',
			},
		},
		'rn1qkb1r/pb3ppp/1p1ppn2/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq d3 0 7' => {
			eco => 'A30l',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6 4. Bg2 Bb7 5. O-O e6 6. Nc3 d6 7. d4

			variation => N__('English: Symmetrical, Hedgehog, 6... d6 7. d4'),
			moves => {
			},
		},
		'rn1qkb1r/pb3ppp/1p2p3/2p5/3P4/P1P2N2/2Q1PPPP/R1B1KB1R w KQkq c6 0 9' => {
			eco => 'E12u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Nxc3 8. bxc3 c5

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7...Nxc3 8.bxc3 c5'),
			moves => {
				'e2e4' => 'rn1qkb1r/pb3ppp/1p2p3/2p5/3PP3/P1P2N2/2Q2PPP/R1B1KB1R b KQkq e3 0 9',
			},
		},
		'rn1qkb1r/pb3ppp/1p2p3/2p5/3PP3/P1P2N2/2Q2PPP/R1B1KB1R b KQkq e3 0 9' => {
			eco => 'E12u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Nxc3 8. bxc3 c5 9. e4

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7...Nxc3 8.bxc3 c5 9. e4'),
			moves => {
			},
		},
		'rn1qkb1r/pb3ppp/1p2p3/2pn4/3P4/P1N2N2/1PQ1PPPP/R1B1KB1R w KQkq c6 0 8' => {
			eco => 'E12t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 c5

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7... c5'),
			moves => {
			},
		},
		'rn1qkb1r/pbp2p1p/1p2p1p1/3n4/3P4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'E12s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. e3 g6

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 Nxd5 7. e3 g6'),
			moves => {
			},
		},
		'rn1qkb1r/pbp2ppp/1p2p3/3n4/3P4/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'E12s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. e3

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 Nxd5 7. e3'),
			moves => {
				'f8e7' => 'rn1qk2r/pbp1bppp/1p2p3/3n4/3P4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 8',
				'g7g6' => 'rn1qkb1r/pbp2p1p/1p2p1p1/3n4/3P4/P1N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2p3/3n4/3P4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'E12r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 Nxd5'),
			moves => {
				'd1c2' => 'rn1qkb1r/pbp2ppp/1p2p3/3n4/3P4/P1N2N2/1PQ1PPPP/R1B1KB1R b KQkq - 1 7',
				'e2e3' => 'rn1qkb1r/pbp2ppp/1p2p3/3n4/3P4/P1N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 7',
				'e2e4' => 'rn1qkb1r/pbp2ppp/1p2p3/3n4/3PP3/P1N2N2/1P3PPP/R1BQKB1R b KQkq e3 0 7',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2p3/3n4/3P4/P1N2N2/1PQ1PPPP/R1B1KB1R b KQkq - 1 7' => {
			eco => 'E12t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2

			variation => N__('Queen\'s Indian: Petrosian, Kasparov Variation'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb3ppp/1p2p3/2pn4/3P4/P1N2N2/1PQ1PPPP/R1B1KB1R w KQkq c6 0 8',
				'd5c3' => 'rn1qkb1r/pbp2ppp/1p2p3/8/3P4/P1n2N2/1PQ1PPPP/R1B1KB1R w KQkq - 0 8',
				'f8e7' => 'rn1qk2r/pbp1bppp/1p2p3/3n4/3P4/P1N2N2/1PQ1PPPP/R1B1KB1R w KQkq - 2 8',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2p3/3n4/3PP3/P1N2N2/1P3PPP/R1BQKB1R b KQkq e3 0 7' => {
			eco => 'E12r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. e4

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 Nxd5 7. e4'),
			moves => {
			},
		},
		'rn1qkb1r/pbp2ppp/1p2p3/8/3P4/P1P2N2/2Q1PPPP/R1B1KB1R b KQkq - 0 8' => {
			eco => 'E12u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Nxc3 8. bxc3

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7...Nxc3 8.bxc3'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb3ppp/1p2p3/2p5/3P4/P1P2N2/2Q1PPPP/R1B1KB1R w KQkq c6 0 9',
				'f8e7' => 'rn1qk2r/pbp1bppp/1p2p3/8/3P4/P1P2N2/2Q1PPPP/R1B1KB1R w KQkq - 1 9',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2p3/8/3P4/P1n2N2/1PQ1PPPP/R1B1KB1R w KQkq - 0 8' => {
			eco => 'E12t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 Nxd5 7. Qc2 Nxc3

			variation => N__('Queen\'s Indian: Petrosian, Kasparov, 7...Nxc3'),
			moves => {
				'b2c3' => 'rn1qkb1r/pbp2ppp/1p2p3/8/3P4/P1P2N2/2Q1PPPP/R1B1KB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2pn2/3P4/3P4/P1N2N2/1P2PPPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'E12m',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5'),
			moves => {
				'e6d5' => 'rn1qkb1r/pbp2ppp/1p3n2/3p4/3P4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 7',
				'f6d5' => 'rn1qkb1r/pbp2ppp/1p2p3/3n4/3P4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2pn2/3p2B1/2PP4/P1N2N2/1P2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'E12n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. Bg5

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5 6. Bg5'),
			moves => {
				'f8e7' => 'rn1qk2r/pbp1bppp/1p2pn2/3p2B1/2PP4/P1N2N2/1P2PPPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq d6 0 6' => {
			eco => 'E14d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Nc3 d5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Nc3 d5'),
			moves => {
			},
		},
		'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'E14n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 d5 6. O-O

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 d5 6.O-O'),
			moves => {
				'f8d6' => 'rn1qk2r/pbp2ppp/1p1bpn2/3p4/2PP4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/3BPN2/PP3PPP/RNBQK2R w KQkq d6 0 6' => {
			eco => 'E14m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3 d5

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3 d5'),
			moves => {
				'e1g1' => 'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq d6 0 6' => {
			eco => 'E12m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 d5'),
			moves => {
				'c1g5' => 'rn1qkb1r/pbp2ppp/1p2pn2/3p2B1/2PP4/P1N2N2/1P2PPPP/R2QKB1R b KQkq - 1 6',
				'c4d5' => 'rn1qkb1r/pbp2ppp/1p2pn2/3P4/3P4/P1N2N2/1P2PPPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/pbp2ppp/1p3n2/3p2B1/3P4/P1N2N2/1P2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'E12q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 exd5 7. Bg5

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 exd5 7. Bg5'),
			moves => {
			},
		},
		'rn1qkb1r/pbp2ppp/1p3n2/3p4/3P4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'E12p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 exd5

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 exd5'),
			moves => {
				'c1g5' => 'rn1qkb1r/pbp2ppp/1p3n2/3p2B1/3P4/P1N2N2/1P2PPPP/R2QKB1R b KQkq - 1 7',
				'g2g3' => 'rn1qkb1r/pbp2ppp/1p3n2/3p4/3P4/P1N2NP1/1P2PP1P/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pbp2ppp/1p3n2/3p4/3P4/P1N2NP1/1P2PP1P/R1BQKB1R b KQkq - 0 7' => {
			eco => 'E12p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 d5 6. cxd5 exd5 7. g3

			variation => N__('Queen\'s Indian: Petrosian, 6.cxd5 exd5 7. g3'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1p1p/1p2pnp1/8/2PP4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'E12l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 g6

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 g6'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1p2/1p2p2p/6pn/2PP4/2N1PNB1/PP3PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'E12y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 g5 7. Bg3 Nh5 8. e3

			variation => N__('Queen\'s Indian: Botvinnik, 8. e3'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1p2/1p2p2p/6pn/2PP4/2N2NB1/PP2PPPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'E12y',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Botvinnik Variation'),
			moves => {
				'd1c2' => 'rn1qkb1r/pbpp1p2/1p2p2p/6pn/2PP4/2N2NB1/PPQ1PPPP/R3KB1R b KQkq - 3 8',
				'e2e3' => 'rn1qkb1r/pbpp1p2/1p2p2p/6pn/2PP4/2N1PNB1/PP3PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/pbpp1p2/1p2p2p/6pn/2PP4/2N2NB1/PPQ1PPPP/R3KB1R b KQkq - 3 8' => {
			eco => 'E12y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 g5 7. Bg3 Nh5 8. Qc2

			variation => N__('Queen\'s Indian: Botvinnik, 8. Qc2'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1p2/1p2pn1p/6p1/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq g6 0 7' => {
			eco => 'E12y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5 h6 6. Bh4 g5

			variation => N__('Queen\'s Indian: Botvinnik Variation'),
			moves => {
				'h4g3' => 'rn1qkb1r/pbpp1p2/1p2pn1p/6p1/2PP4/2N2NB1/PP2PPPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rn1qkb1r/pbpp1p2/1p2pn1p/6p1/2PP4/2N2NB1/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'E12y',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Botvinnik Variation'),
			moves => {
				'f6h5' => 'rn1qkb1r/pbpp1p2/1p2p2p/6pn/2PP4/2N2NB1/PP2PPPP/R2QKB1R w KQkq - 2 8',
			},
		},
		'rn1qkb1r/pbpp1pp1/1p2pn1p/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'E12w',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. Nc3 Bb7 5. Bg5'),
			moves => {
				'g5h4' => 'rn1qkb1r/pbpp1pp1/1p2pn1p/8/2PP3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/pbpp1pp1/1p2pn1p/8/2PP3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'E12w',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. Nc3 Bb7 5. Bg5'),
			moves => {
				'f8b4' => 'rn1qk2r/pbpp1pp1/1p2pn1p/8/1bPP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 7',
				'f8e7' => 'rn1qk2r/pbppbpp1/1p2pn1p/8/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 7',
				'g7g5' => 'rn1qkb1r/pbpp1p2/1p2pn1p/6p1/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq g6 0 7',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2p3/8/2PPn3/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'E12k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3 Ne4

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3 Ne4'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'E12w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7 5. Bg5

			variation => N__('Queen\'s Indian: 4. Nc3 Bb7 5. Bg5'),
			moves => {
				'h7h6' => 'rn1qkb1r/pbpp1pp1/1p2pn1p/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P1P3/2N2N2/PP1P1PPP/R1BQKB1R w KQkq - 1 5' => {
			eco => 'A17',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Queen\'s Indian, 4. e4'),
			moves => {
				'f1d3' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P1P3/2NB1N2/PP1P1PPP/R1BQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P1P3/2NB1N2/PP1P1PPP/R1BQK2R b KQkq - 2 5' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6 4. e4 Bb7 5. Bd3

			variation => N__('English: Anglo-Queen\'s Indian, Romanishin'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 5' => {
			eco => 'A17',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
				'f1g2' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 5' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
				'f8e7' => 'rn1qk2r/pbppbppp/1p2pn2/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 3 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP1B2/4PN2/PP3PPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'E12b',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Miles Variation'),
			moves => {
				'f8b4' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP1B2/4PN2/PP3PPP/RN1QKB1R w KQkq - 1 6',
				'f8e7' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP1B2/4PN2/PP3PPP/RN1QKB1R w KQkq - 1 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP1B2/5N2/PP2PPPP/RN1QKB1R w KQkq - 2 5' => {
			eco => 'E12b',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Miles Variation'),
			moves => {
				'e2e3' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP1B2/4PN2/PP3PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/1P3NP1/P3PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'E15n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb7

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7'),
			moves => {
				'f1g2' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/1P3NP1/P3PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/1P3NP1/P3PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'E15n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. b3 Bb7 6. Bg2

			variation => N__('Queen\'s Indian: Nimzowitsch, 5. b3 Bb7 6. Bg2'),
			moves => {
				'f8b4' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/1P3NP1/P3PPBP/RNBQK2R w KQkq - 3 7',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'E14c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Nc3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Nc3'),
			moves => {
				'd7d5' => 'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq d6 0 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'E12w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3 Bb7

			variation => N__('Queen\'s Indian: 4. Nc3 Bb7'),
			moves => {
				'c1g5' => 'rn1qkb1r/pbpp1ppp/1p2pn2/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'E14e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7 5. Bd3

			variation => N__('Queen\'s Indian: 4. e3 Bb7 5. Bd3'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/3BPN2/PP3PPP/RNBQK2R w KQkq c6 0 6',
				'd7d5' => 'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/3BPN2/PP3PPP/RNBQK2R w KQkq d6 0 6',
				'f8b4' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/3BPN2/PP3PPP/RNBQK2R w KQkq - 3 6',
				'f8e7' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQK2R w KQkq - 3 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E14b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb7

			variation => N__('Queen\'s Indian: 4. e3 Bb7'),
			moves => {
				'b1c3' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5',
				'f1d3' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/3BPN2/PP3PPP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1NPP1P/R1BQKB1R w KQkq - 3 6' => {
			eco => 'E15f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Nbd2 Bb7

			variation => N__('Queen\'s Indian: Nimzowitsch, 5.Nbd2 Bb7'),
			moves => {
				'f1g2' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1NPPBP/R1BQK2R b KQkq - 4 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP1NPPBP/R1BQK2R b KQkq - 4 6' => {
			eco => 'E15f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Ba6 5. Nbd2 Bb7 6. Bg2

			variation => N__('Queen\'s Indian: Nimzowitsch, 5.Nbd2 Bb7 6. Bg2'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/5NP1/PP1NPPBP/R1BQK2R w KQkq c6 0 7',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E15t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7

			variation => N__('Queen\'s Indian: 4. g3 Bb7'),
			moves => {
				'f1g2' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 5' => {
			eco => 'E15t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2

			variation => N__('Queen\'s Indian: 4. g3 Bb7'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq c6 0 6',
				'd8c8' => 'rnq1kb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 6',
				'f8b4' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 6',
				'f8e7' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P1N2N2/1P2PPPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'E12j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7 5. Nc3

			variation => N__('Queen\'s Indian: Petrosian, 5. Nc3'),
			moves => {
				'b7f3' => 'rn1qkb1r/p1pp1ppp/1p2pn2/8/2PP4/P1N2b2/1P2PPPP/R1BQKB1R w KQkq - 0 6',
				'd7d5' => 'rn1qkb1r/pbp2ppp/1p2pn2/3p4/2PP4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq d6 0 6',
				'f6e4' => 'rn1qkb1r/pbpp1ppp/1p2p3/8/2PPn3/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 3 6',
				'f8e7' => 'rn1qk2r/pbppbppp/1p2pn2/8/2PP4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 3 6',
				'g7g6' => 'rn1qkb1r/pbpp1p1p/1p2pnp1/8/2PP4/P1N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P1N2N2/1PQ1PPPP/R1B1KB1R b KQkq - 4 6' => {
			eco => 'E12h',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/2PP4/P1N2N2/1PQ1PPPP/R1B1KB1R w KQkq c6 0 7',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E12j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Bb7

			variation => N__('Queen\'s Indian: Petrosian, 4... Bb7'),
			moves => {
				'b1c3' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P1N2N2/1P2PPPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P4N2/1PQ1PPPP/RNB1KB1R w KQkq - 3 6' => {
			eco => 'E12h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 Ba6 5. Qc2 Bb7

			variation => N__('Queen\'s Indian: Petrosian, 4... Ba6 5. Qc2 Bb7'),
			moves => {
				'b1c3' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P1N2N2/1PQ1PPPP/R1B1KB1R b KQkq - 4 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4 Bb7 4. e3 e6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P4/3BPN2/PPP2PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 Bb7 4. Bd3 e6 5. O-O

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb1p1ppp/1p2pn2/2p5/3P4/3BPN2/PPP2PPP/RNBQ1RK1 w kq c6 0 6',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 Bb7 4. Bd3 e6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'e1g1' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P4/3BPN2/PPP2PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. g3 Bb7 4. Bg2 e6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1ppp/4pn2/1p6/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'A46n',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 b5 4. Bg2 Bb7 5. O-O

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 b5 4. Bg2 Bb7 5.O-O'),
			moves => {
			},
		},
		'rn1qkb1r/pbpp1ppp/4pn2/1p6/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A46m',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 b5 4. Bg2 Bb7

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 b5 4. Bg2 Bb7'),
			moves => {
				'e1g1' => 'rn1qkb1r/pbpp1ppp/4pn2/1p6/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bg5 Bb7

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A50m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 b6 3. Nc3 Bb7

			variation => N__('Indian: Queen\'s Indian Accelerated'),
			moves => {
				'd1c2' => 'rn1qkb1r/pbpppppp/1p3n2/8/2PP4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 3 4',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/2PP4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 3 4' => {
			eco => 'A50n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 b6 3. Nc3 Bb7 4. Qc2

			variation => N__('Indian: Queen\'s Indian Accelerated'),
			moves => {
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4 Bb7 4. e3

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'e7e6' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4 Bb7

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'e2e3' => 'rn1qkb1r/pbpppppp/1p3n2/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 Bb7 4. Bd3

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'e7e6' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 Bb7

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'f1d3' => 'rn1qkb1r/pbpppppp/1p3n2/8/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A47',
                        # TRANSLATORS:

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'f1g2' => 'rn1qkb1r/pbpppppp/1p3n2/8/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. g3 Bb7 4. Bg2

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c7c5' => 'rn1qkb1r/pb1ppppp/1p3n2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq c6 0 5',
				'e7e6' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq - 3 4' => {
			eco => 'B00h',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3 Nf6

			variation => N__('Owen Defence: 3. Bd3 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/pbpppppp/1p3n2/8/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 b6 3. Bg2 Bb7

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'rn1qkb1r/pp1B1ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'C08k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nf6 6. Bb5+ Bd7 7. Bxd7+

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6, 7.Bxd7'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1n1ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp1b1ppp/3p1n2/1B2p3/3NP3/5P2/PPP3PP/RNBQK2R w KQkq - 2 7' => {
			eco => 'B55',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Bb5+ Bd7

			variation => N__('Sicilian: Prins, Venice Attack, 6... Bd7'),
			moves => {
			},
		},
		'rn1qkb1r/pp1b1ppp/3ppn2/1B6/3NP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B80n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bb5+ Bd7

			variation => N__('Sicilian: Scheveningen, Vitolins, 6... Bd7'),
			moves => {
			},
		},
		'rn1qkb1r/pp1b1ppp/5n2/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R w KQkq - 4 7' => {
			eco => 'C08j',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6'),
			moves => {
				'b5d7' => 'rn1qkb1r/pp1B1ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp1bpppp/3p1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 4 6' => {
			eco => 'B56g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Bd7 6. Bg5

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Bd7 6. Bg5'),
			moves => {
			},
		},
		'rn1qkb1r/pp1bpppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B56f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Bd7

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 Bd7'),
			moves => {
				'c1g5' => 'rn1qkb1r/pp1bpppp/3p1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 4 6',
			},
		},
		'rn1qkb1r/pp1n1ppp/2p5/4p2b/P1NP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq e6 0 9' => {
			eco => 'D16r',
                        # TRANSLATORS:

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. f3 Nfd7'),
			moves => {
				'c3e4' => 'rn1qkb1r/pp1n1ppp/2p5/4p2b/P1NPN3/5P2/1P2P1PP/R1BQKB1R b KQkq - 1 9',
			},
		},
		'rn1qkb1r/pp1n1ppp/2p5/4p2b/P1NPN3/5P2/1P2P1PP/R1BQKB1R b KQkq - 1 9' => {
			eco => 'D16s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5 Bh5 7. f3 Nfd7 8. Nxc4 e5 9. Ne4

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. f3 Nfd7 8.Nxc4 e5 9. Ne4'),
			moves => {
			},
		},
		'rn1qkb1r/pp1n1ppp/4p3/3pNb2/3P4/2N1P3/PP3PPP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'D12f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. cxd5 cxd5 6. Nc3 e6 7. Ne5 Nfd7

			variation => N__('Slav: 4. e3 Bf5 5.cxd5 cxd5 6. Nc3, Amsterdam Variation'),
			moves => {
			},
		},
		'rn1qkb1r/pp1npppp/2p5/4N2b/P1pP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D16r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5 Bh5 7. f3 Nfd7

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. f3 Nfd7'),
			moves => {
				'e5c4' => 'rn1qkb1r/pp1npppp/2p5/7b/P1NP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp1npppp/2p5/7b/P1NP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D16r',
                        # TRANSLATORS:

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. f3 Nfd7'),
			moves => {
				'e7e5' => 'rn1qkb1r/pp1n1ppp/2p5/4p2b/P1NP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq e6 0 9',
			},
		},
		'rn1qkb1r/pp2nppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1QK2R w KQkq - 2 7' => {
			eco => 'B12u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. Be3 Ne7

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6. Be3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2nppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'B12t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. O-O Ne7

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6.O-O'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/2P2N2/PP3PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B16l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. c3 Bf5 7. Nf3

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. c3 Bf5 7. Nf3'),
			moves => {
				'd8c7' => 'rn2kb1r/ppq1pp1p/2p2p2/5b2/3P4/2P2N2/PP3PPP/R1BQKB1R w KQkq - 3 8',
				'e7e6' => 'rn1qkb1r/pp3p1p/2p1pp2/5b2/3P4/2P2N2/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/2P5/PP2NPPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B16k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. c3 Bf5 7. Ne2

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. c3 Bf5 7. Ne2'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/2P5/PP3PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'B16j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. c3 Bf5

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. c3 Bf5'),
			moves => {
				'g1e2' => 'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/2P5/PP2NPPP/R1BQKB1R b KQkq - 2 7',
				'g1f3' => 'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/2P2N2/PP3PPP/R1BQKB1R b KQkq - 2 7',
			},
		},
		'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B16f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Nf3 Bf5

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Nf3 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pp1p/2p2p2/8/3P2b1/5N2/PPP1BPPP/R1BQK2R b KQkq - 3 7' => {
			eco => 'B16g',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Nf3 Bg4'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3p1p/2p1pp2/8/3P2b1/5N2/PPP1BPPP/R1BQK2R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp2pp1p/2p2p2/8/3P2b1/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B16g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Nf3 Bg4

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Nf3 Bg4'),
			moves => {
				'f1e2' => 'rn1qkb1r/pp2pp1p/2p2p2/8/3P2b1/5N2/PPP1BPPP/R1BQK2R b KQkq - 3 7',
			},
		},
		'rn1qkb1r/pp2ppp1/2p2nbp/4N3/3P3P/6N1/PPP2PP1/R1BQKB1R b KQkq - 3 8' => {
			eco => 'B19e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nf6 8. Ne5

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nf6 8. Ne5'),
			moves => {
				'g6h7' => 'rn1qkb1r/pp2pppb/2p2n1p/4N3/3P3P/6N1/PPP2PP1/R1BQKB1R w KQkq - 4 9',
			},
		},
		'rn1qkb1r/pp2ppp1/2p2nbp/7P/3P4/5NN1/PPP2PP1/R1BQKB1R b KQkq - 0 8' => {
			eco => 'B19d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nf6 8. h5

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nf6 8. h5'),
			moves => {
				'g6h7' => 'rn1qkb1r/pp2pppb/2p2n1p/7P/3P4/5NN1/PPP2PP1/R1BQKB1R w KQkq - 1 9',
			},
		},
		'rn1qkb1r/pp2ppp1/2p2nbp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B19c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nf6

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nf6'),
			moves => {
				'f3e5' => 'rn1qkb1r/pp2ppp1/2p2nbp/4N3/3P3P/6N1/PPP2PP1/R1BQKB1R b KQkq - 3 8',
				'h4h5' => 'rn1qkb1r/pp2ppp1/2p2nbp/7P/3P4/5NN1/PPP2PP1/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp2pppb/2p2n1p/4N3/3P3P/3B2N1/PPP2PP1/R1BQK2R b KQkq - 5 9' => {
			eco => 'B19f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nf6 8. Ne5 Bh7 9. Bd3

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nf6 8. Ne5 Bh7 9. Bd3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppb/2p2n1p/4N3/3P3P/6N1/PPP2PP1/R1BQKB1R w KQkq - 4 9' => {
			eco => 'B19e',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nf6 8. Ne5'),
			moves => {
				'f1d3' => 'rn1qkb1r/pp2pppb/2p2n1p/4N3/3P3P/3B2N1/PPP2PP1/R1BQK2R b KQkq - 5 9',
			},
		},
		'rn1qkb1r/pp2pppb/2p2n1p/7P/3P4/5NN1/PPP2PP1/R1BQKB1R w KQkq - 1 9' => {
			eco => 'B19d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 Nf6 8. h5 Bh7

			variation => N__('Caro-Kann: Classical, 7. Nf3 Nf6 8. h5'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/1n1p4/8/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B05d',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. c4 Nb6 6. exd6 cxd6

			variation => N__('Alekhine: Modern, Alekhine Variation, 6.exd6 cxd6'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p1bn2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D25f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Be6 5. Nc3 c6

			variation => N__('QGA: Flohr, 5. Nc3 c6'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3P1b2/3P4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D12c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. cxd5

			variation => N__('Slav: 4. e3 Bf5 5.cxd5'),
			moves => {
				'c6d5' => 'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/1P3N2/PB1PPPPP/RN1QKB1R w KQkq - 3 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. Bb2 Bf5

			variation => N__('English: New York/London Defence'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/1P3NP1/P2PPP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bf5

			variation => N__('English: London Defence'),
			moves => {
				'f1g2' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/1P3NP1/P2PPPBP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/1P3NP1/P2PPPBP/RNBQK2R b KQkq - 2 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bf5 5. Bg2

			variation => N__('English: London Defence'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/1P3NP1/P2PPPBP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5 4. Nf3 Bf5 5. O-O

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5 4. Nf3 Bf5

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'e1g1' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/1Q2PN2/PP3PPP/RNB1KB1R b KQkq - 2 5' => {
			eco => 'D12b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Qb3

			variation => N__('Slav: 4. e3 Bf5 5. Qb3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5' => {
			eco => 'D15b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 Bf5 5. Qb3

			variation => N__('Slav: 4. Nc3 Bf5 5. Qb3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'D10v',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. e3 Bf5

			variation => N__('Slav: 3. Nc3 Nf6 4. e3 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'D12i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3

			variation => N__('Slav: 4. e3 Bf5 5. Nc3'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'D15b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 Bf5

			variation => N__('Slav: 4. Nc3 Bf5'),
			moves => {
				'd1b3' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/3BPN2/PP3PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'D12g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Bd3

			variation => N__('Slav: 4. e3 Bf5 5. Bd3'),
			moves => {
				'f5d3' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP4/3bPN2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D12a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5

			variation => N__('Slav: 4. e3 Bf5'),
			moves => {
				'b1c3' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5',
				'c4d5' => 'rn1qkb1r/pp2pppp/2p2n2/3P1b2/3P4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 5',
				'd1b3' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/1Q2PN2/PP3PPP/RNB1KB1R b KQkq - 2 5',
				'f1d3' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/3BPN2/PP3PPP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D11o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'f1g2' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 5' => {
			eco => 'D11o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p1b2/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'd2d3' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/1P3N2/PB1PPPPP/RN1QKB1R w KQkq - 3 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. Bb2 Bg4

			variation => N__('English: Capablanca'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/1P3NP1/P2PPP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bg4

			variation => N__('English: Torre Defence'),
			moves => {
				'f1g2' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/1P3NP1/P2PPPBP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/1P3NP1/P2PPPBP/RNBQK2R b KQkq - 2 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bg4 5. Bg2

			variation => N__('English: Torre Defence'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2P3b1/1P3NP1/P2PPPBP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5 4. Nf3 Bg4 5. O-O

			variation => N__('English: Caro-Kann Defence'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5 4. Nf3 Bg4

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'e1g1' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'D11t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bg4 5. Nc3

			variation => N__('Slav: 4. e3 Bg4'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D11t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bg4

			variation => N__('Slav: 4. e3 Bg4'),
			moves => {
				'b1c3' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'f1g2' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 5' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP4/3bPN2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D12h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Bd3 Bxd3

			variation => N__('Slav: 4. e3 Bf5 5. Bd3 Bxd3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/3P2b1/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'D02t',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 c6 4. Bg2 Bg4 5. O-O

			variation => N__('Queen\'s Pawn: 3. g3 c6 4. Bg2 Bg4'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1npppp/2p2n2/3p4/3P2b1/5NP1/PPP1PPBP/RNBQ1RK1 w kq - 4 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/3P2b1/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'D02t',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 c6 4. Bg2 Bg4

			variation => N__('Queen\'s Pawn: 3. g3 c6 4. Bg2 Bg4'),
			moves => {
				'e1g1' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/3P2b1/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/4P3/2N2Q1P/PPPP1PP1/R1B1KB1R w KQkq - 1 6' => {
			eco => 'B11f',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 Nf6

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/6b1/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bg4 5. d3

			variation => N__('Reti: KIA, Yugoslav'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1npppp/2p2n2/3p4/6b1/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/3p4/6b1/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bg4

			variation => N__('Reti: KIA, Yugoslav'),
			moves => {
				'd2d3' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/6b1/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D16q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5 Bh5 7. f3

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. f3'),
			moves => {
				'f6d7' => 'rn1qkb1r/pp1npppp/2p5/4N2b/P1pP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq - 1 8',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N3P1/1P2PP1P/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D16p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5 Bh5 7. g3

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. g3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N4P/1P2PPP1/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D16o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5 Bh5 7. h3

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5 7. h3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'D16n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5 Bh5

			variation => N__('Slav: Bronstein, 6. Ne5 Bh5'),
			moves => {
				'f2f3' => 'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 7',
				'g2g3' => 'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N3P1/1P2PP1P/R1BQKB1R b KQkq - 0 7',
				'h2h3' => 'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N4P/1P2PPP1/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/4N3/P1pP2b1/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D16m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4 6. Ne5

			variation => N__('Slav: Bronstein, 6. Ne5'),
			moves => {
				'g4h5' => 'rn1qkb1r/pp2pppp/2p2n2/4N2b/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D17f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5

			variation => N__('Slav: Central Variation'),
			moves => {
				'b8a6' => 'r2qkb1r/pp2pppp/n1p2n2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7',
				'b8d7' => 'r2qkb1r/pp1npppp/2p2n2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7',
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/5b2/2QP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 2 6' => {
			eco => 'D23k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. Nc3

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. Nc3'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/5b2/2QP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 1 6' => {
			eco => 'D23j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5'),
			moves => {
				'b1c3' => 'rn1qkb1r/pp2pppp/2p2n2/5b2/2QP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 2 6',
				'g2g3' => 'rn1qkb1r/pp2pppp/2p2n2/5b2/2QP4/5NP1/PP2PP1P/RNB1KB1R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/5b2/2QP4/5NP1/PP2PP1P/RNB1KB1R b KQkq - 0 6' => {
			eco => 'D23l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. g3'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/5b2/2QP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/5b2/P1pP3N/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'D17b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Nh4

			variation => N__('Slav: Czech, 6. Nh4'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/5b2/P1pP3N/2N5/1P2PPPP/R1BQKB1R w KQkq - 0 7',
				'f5c8' => 'rnbqkb1r/pp2pppp/2p2n2/8/P1pP3N/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/5b2/P1pP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D18a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3

			variation => N__('Slav: Dutch Variation'),
			moves => {
				'b8a6' => 'r2qkb1r/pp2pppp/n1p2n2/5b2/P1pP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 1 7',
				'e7e6' => 'rn1qkb1r/pp3ppp/2p1pn2/5b2/P1pP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/5b2/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D17a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5

			variation => N__('Slav: Czech Defence'),
			moves => {
				'e2e3' => 'rn1qkb1r/pp2pppp/2p2n2/5b2/P1pP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6',
				'f3e5' => 'rn1qkb1r/pp2pppp/2p2n2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6',
				'f3h4' => 'rn1qkb1r/pp2pppp/2p2n2/5b2/P1pP3N/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'rn1qkb1r/pp2pppp/2p2n2/8/P1pP2b1/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D16l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bg4

			variation => N__('Slav: Bronstein (Steiner) Variation'),
			moves => {
				'f3e5' => 'rn1qkb1r/pp2pppp/2p2n2/4N3/P1pP2b1/2N5/1P2PPPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'rn1qkb1r/pp2pppp/2pp4/3nP3/2PP2b1/5N2/PP2BPPP/RNBQK2R b KQkq c3 0 6' => {
			eco => 'B05i',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 c6 6. c4

			variation => N__('Alekhine: Modern, Flohr, 6. c4'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2pp4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'B05j',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 c6 6. O-O

			variation => N__('Alekhine: Modern, Flohr, 6.O-O'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/2pp4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'B05h',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 c6

			variation => N__('Alekhine: Modern, Flohr Variation'),
			moves => {
				'c2c4' => 'rn1qkb1r/pp2pppp/2pp4/3nP3/2PP2b1/5N2/PP2BPPP/RNBQK2R b KQkq c3 0 6',
				'e1g1' => 'rn1qkb1r/pp2pppp/2pp4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/1Q2PN2/PP3PPP/RNB1KB1R b KQkq - 1 6' => {
			eco => 'D12d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. cxd5 cxd5 6. Qb3

			variation => N__('Slav: 4. e3 Bf5 5.cxd5 cxd5 6. Qb3'),
			moves => {
				'd8c7' => 'rn2kb1r/ppq1pppp/5n2/3p1b2/3P4/1Q2PN2/PP3PPP/RNB1KB1R w KQkq - 2 7',
			},
		},
		'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'D12f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. cxd5 cxd5 6. Nc3

			variation => N__('Slav: 4. e3 Bf5 5.cxd5 cxd5 6. Nc3'),
			moves => {
				'e7e6' => 'rn1qkb1r/pp3ppp/4pn2/3p1b2/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D13d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 Bf5

			variation => N__('Slav: Exchange, 5. Nc3 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'D12c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. cxd5 cxd5

			variation => N__('Slav: 4. e3 Bf5 5.cxd5'),
			moves => {
				'b1c3' => 'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 1 6',
				'd1b3' => 'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/1Q2PN2/PP3PPP/RNB1KB1R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/pp3p1p/2p1pp2/5b2/3P4/2P2N2/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B16n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. c3 Bf5 7. Nf3 e6

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. c3 Bf5 7. Nf3 e6'),
			moves => {
			},
		},
		'rn1qkb1r/pp3p1p/2p1pp2/8/3P2b1/5N2/PPP1BPPP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B16h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Nf3 Bg4 7. Be2 e6

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Nf3 Bg4 7. Be2 e6'),
			moves => {
			},
		},
		'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'D11p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O h6 7. b3

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'D11p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O h6 7. Nc3

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'D11p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O h6 7. Nbd2

			variation => N__('Slav: Slav-Reti System'),
			moves => {
			},
		},
		'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'D11p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O h6

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'b1c3' => 'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b kq - 1 7',
				'b1d2' => 'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b kq - 1 7',
				'b2b3' => 'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/1P3NP1/P2PPPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bf5 5. Bg2 e6

			variation => N__('English: London Defence'),
			moves => {
				'c1b2' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/1P3NP1/PB1PPPBP/RN1QK2R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/1P3NP1/PB1PPPBP/RN1QK2R b KQkq - 1 6' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bf5 5. Bg2 e6 6. Bb2

			variation => N__('English: London Defence'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/3P1NP1/PP2PPBP/RNBQ1RK1 b kq - 0 6' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5 4. Nf3 Bf5 5. O-O e6 6. d3

			variation => N__('English: Caro-Kann Defence'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A11',
                        # TRANSLATORS:

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'd2d3' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2P5/3P1NP1/PP2PPBP/RNBQ1RK1 b kq - 0 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP3N/2N1P3/PP3PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'D12n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Nh4

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Nh4'),
			moves => {
				'f5e4' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PPb2N/2N1P3/PP3PPP/R1BQKB1R w KQkq - 2 7',
				'f5g4' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP2bN/2N1P3/PP3PPP/R1BQKB1R w KQkq - 2 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D12j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Be2

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Be2'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D12i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6'),
			moves => {
				'f1d3' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 6',
				'f1e2' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 1 6',
				'f3h4' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP3N/2N1P3/PP3PPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D12k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Bd3

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Bd3'),
			moves => {
				'f5d3' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP4/2NbPN2/PP3PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'D11o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6 6. O-O

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7',
				'f8e7' => 'rn1qk2r/pp2bppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7',
				'h7h6' => 'rn1qkb1r/pp3pp1/2p1pn1p/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D11o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bf5 5. Bg2 e6

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'e1g1' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'b1d2' => 'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p1b2/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6 4. O-O Bf5 5. d3 e6 6. Nbd2

			variation => N__('Reti: KIA, Petrosian Variation'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2P3b1/1P3NP1/P2PPPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bg4 5. Bg2 e6

			variation => N__('English: Torre Defence'),
			moves => {
				'c1b2' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2P3b1/1P3NP1/PB1PPPBP/RN1QK2R b KQkq - 1 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2P3b1/1P3NP1/PB1PPPBP/RN1QK2R b KQkq - 1 6' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. g3 Bg4 5. Bg2 e6 6. Bb2

			variation => N__('English: Torre Defence'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6 6. O-O

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D11m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3 Bg4 5. Bg2 e6

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'e1g1' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP2b1/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP2bN/2N1P3/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'D12o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Nh4 Bg4

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Nh4 Bg4'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP4/2NQPN2/PP3PPP/R1B1K2R b KQkq - 0 7' => {
			eco => 'D12l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Bd3 Bxd3 7. Qxd3

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Bd3 Bxd3'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2NQPN2/PP3PPP/R1B1K2R w KQkq - 1 8',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP4/2NbPN2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'D12l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Bd3 Bxd3

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Bd3 Bxd3'),
			moves => {
				'd1d3' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PP4/2NQPN2/PP3PPP/R1B1K2R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/2PPb2N/2N1P3/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'D12p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. Nc3 e6 6. Nh4 Be4

			variation => N__('Slav: 4. e3 Bf5 5. Nc3 e6 6. Nh4 Be4'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/2N2QPP/PPPP1P2/R1B1KB1R w KQkq - 1 7' => {
			eco => 'B11i',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. g3 Nf6

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. g3 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/2NP1Q1P/PPP2PP1/R1B1KB1R w KQkq - 1 7' => {
			eco => 'B11l',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. d3 Nf6

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. d3 Nf6'),
			moves => {
				'a2a3' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/P1NP1Q1P/1PP2PP1/R1B1KB1R b KQkq - 0 7',
				'c1d2' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/2NP1Q1P/PPPB1PP1/R3KB1R b KQkq - 2 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/2NP1Q1P/PPPB1PP1/R3KB1R b KQkq - 2 7' => {
			eco => 'B11n',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. d3 Nf6 7. Bd2

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. d3 Nf6 7. Bd2'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/P1NP1Q1P/1PP2PP1/R1B1KB1R b KQkq - 0 7' => {
			eco => 'B11m',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. d3 Nf6 7. a3

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. d3 Nf6 7. a3'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/4Nb2/P1pP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D17k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3

			variation => N__('Slav: Central, 6. Ne5 e6 7. f3'),
			moves => {
				'c6c5' => 'rn1qkb1r/pp3ppp/4pn2/2p1Nb2/P1pP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq - 0 8',
				'f8b4' => 'rn1qk2r/pp3ppp/2p1pn2/4Nb2/PbpP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq - 1 8',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/4Nb2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D17k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6

			variation => N__('Slav: Central, 6. Ne5 e6'),
			moves => {
				'f2f3' => 'rn1qkb1r/pp3ppp/2p1pn2/4Nb2/P1pP4/2N2P2/1P2P1PP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/5N2/P1pP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D17c',
                        # TRANSLATORS:

			variation => N__('Slav: Czech, 6. Nh4 e6'),
			moves => {
				'e6f5' => 'rn1qkb1r/pp3ppp/2p2n2/5p2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/5b2/2QP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 0 7' => {
			eco => 'D23m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. g3 e6'),
			moves => {
				'f1g2' => 'rn1qkb1r/pp3ppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB1K2R b KQkq - 1 7' => {
			eco => 'D23m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4 Bf5 6. g3 e6 7. Bg2

			variation => N__('QGA: Mannheim, 4... c6, 5.Qxc4 Bf5 6. g3 e6 7. Bg2'),
			moves => {
				'b8d7' => 'r2qkb1r/pp1n1ppp/2p1pn2/5b2/2QP4/5NP1/PP2PPBP/RNB1K2R w KQkq - 2 8',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/5b2/P1BP4/2N1PN2/1P3PPP/R1BQK2R b KQkq - 0 7' => {
			eco => 'D18c',
                        # TRANSLATORS:

			variation => N__('Slav: Dutch, 6... e6'),
			moves => {
				'f8b4' => 'rn1qk2r/pp3ppp/2p1pn2/5b2/PbBP4/2N1PN2/1P3PPP/R1BQK2R w KQkq - 1 8',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/5b2/P1pP3N/2N5/1P2PPPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D17c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Nh4 e6

			variation => N__('Slav: Czech, 6. Nh4 e6'),
			moves => {
				'h4f5' => 'rn1qkb1r/pp3ppp/2p1pn2/5N2/P1pP4/2N5/1P2PPPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pn2/5b2/P1pP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D18c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. e3 e6

			variation => N__('Slav: Dutch, 6... e6'),
			moves => {
				'f1c4' => 'rn1qkb1r/pp3ppp/2p1pn2/5b2/P1BP4/2N1PN2/1P3PPP/R1BQK2R b KQkq - 0 7',
			},
		},
		'rn1qkb1r/pp3ppp/2p1pnb1/8/2BP4/6N1/PPP1NPPP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B18k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Bc4 e6 7. N1e2 Nf6

			variation => N__('Caro-Kann: Classical, 6. Bc4 e6 7.N1e2 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p2n2/5p2/P1pP4/2N1P3/1P3PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D17d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Nh4 e6 7. Nxf5 exf5 8. e3

			variation => N__('Slav: Czech, 6. Nh4 e6 7.Nxf5 exf5 8. e3'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/2p2n2/5p2/P1pP4/2N5/1P2PPPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D17c',
                        # TRANSLATORS:

			variation => N__('Slav: Czech, 6. Nh4 e6'),
			moves => {
				'e2e3' => 'rn1qkb1r/pp3ppp/2p2n2/5p2/P1pP4/2N1P3/1P3PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rn1qkb1r/pp3ppp/3pbn2/4p3/4P3/1N3P2/PPP3PP/RNBQKB1R w KQkq - 2 7' => {
			eco => 'B54n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Nb3 Be6

			variation => N__('Sicilian: Prins (Moscow), 5... e5 6. Nb3 Be6'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/4pn2/2p1Nb2/P1pP4/2N2P2/1P2P1PP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D17l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 c5

			variation => N__('Slav: Central, Hübner (7... c5)'),
			moves => {
			},
		},
		'rn1qkb1r/pp3ppp/4pn2/3p1b2/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D12f',
                        # TRANSLATORS:

			variation => N__('Slav: 4. e3 Bf5 5.cxd5 cxd5 6. Nc3'),
			moves => {
				'f3e5' => 'rn1qkb1r/pp3ppp/4pn2/3pNb2/3P4/2N1P3/PP3PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'rn1qkb1r/pp3ppp/4pn2/3pNb2/3P4/2N1P3/PP3PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'D12f',
                        # TRANSLATORS:

			variation => N__('Slav: 4. e3 Bf5 5.cxd5 cxd5 6. Nc3'),
			moves => {
				'f6d7' => 'rn1qkb1r/pp1n1ppp/4p3/3pNb2/3P4/2N1P3/PP3PPP/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rn1qkb1r/ppp1pp1p/4bnp1/1Q6/3P4/2N5/PP2PPPP/R1B1KBNR b KQkq - 2 6' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4 Be6 6. Qb5+

			variation => N__('Grünfeld: Early Russian, 5... Be6 6. Qb5+'),
			moves => {
				'e6d7' => 'rn1qkb1r/pppbpp1p/5np1/1Q6/3P4/2N5/PP2PPPP/R1B1KBNR w KQkq - 3 7',
			},
		},
		'rn1qkb1r/ppp1pp1p/4bnp1/8/2QP4/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4 Be6

			variation => N__('Grünfeld: Early Russian, 5... Be6'),
			moves => {
				'c4b5' => 'rn1qkb1r/ppp1pp1p/4bnp1/1Q6/3P4/2N5/PP2PPPP/R1B1KBNR b KQkq - 2 6',
			},
		},
		'rn1qkb1r/ppp1pppp/1n1P4/8/2PP2b1/5N2/PP3PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'B05d',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. c4 Nb6 6. exd6

			variation => N__('Alekhine: Modern, Alekhine Variation, 6.exd6'),
			moves => {
				'c7d6' => 'rn1qkb1r/pp2pppp/1n1p4/8/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7',
				'e7d6' => 'rn1qkb1r/ppp2ppp/1n1p4/8/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/ppp1pppp/1n1p4/4P3/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B05c',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. c4 Nb6

			variation => N__('Alekhine: Modern, Alekhine Variation'),
			moves => {
				'e5d6' => 'rn1qkb1r/ppp1pppp/1n1P4/8/2PP2b1/5N2/PP3PPP/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/ppp1pppp/1n1p4/4Pb2/2PP1P2/8/PP4PP/RNBQKBNR w KQkq - 1 6' => {
			eco => 'B03h',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 Bf5

			variation => N__('Alekhine: Four Pawns Attack, Trifunovic Variation'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/1n6/2P5/3P2b1/5N2/PP3PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 Bg4 6. c5

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 Bg4 6. c5'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/1n6/4Pb2/2PP4/2N5/PP4PP/R1BQKBNR b KQkq - 2 7' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'e7e6' => 'rn1qkb1r/ppp2ppp/1n2p3/4Pb2/2PP4/2N5/PP4PP/R1BQKBNR w KQkq - 0 8',
			},
		},
		'rn1qkb1r/ppp1pppp/1n6/4Pb2/2PP4/8/PP4PP/RNBQKBNR w KQkq - 1 7' => {
			eco => 'B03l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 Bf5

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'b1c3' => 'rn1qkb1r/ppp1pppp/1n6/4Pb2/2PP4/2N5/PP4PP/R1BQKBNR b KQkq - 2 7',
			},
		},
		'rn1qkb1r/ppp1pppp/1n6/8/2PP2b1/5N2/PP2BPPP/RNBQK2R b KQkq - 4 6' => {
			eco => 'B01z',
                        # TRANSLATORS:

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 Bg4'),
			moves => {
				'e7e6' => 'rn1qkb1r/ppp2ppp/1n2p3/8/2PP2b1/5N2/PP2BPPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rn1qkb1r/ppp1pppp/1n6/8/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 Bg4

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 Bg4'),
			moves => {
				'c4c5' => 'rn1qkb1r/ppp1pppp/1n6/2P5/3P2b1/5N2/PP3PPP/RNBQKB1R b KQkq - 0 6',
				'f1e2' => 'rn1qkb1r/ppp1pppp/1n6/8/2PP2b1/5N2/PP2BPPP/RNBQK2R b KQkq - 4 6',
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/5b2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 Bf5

			variation => N__('Old Indian: Janowski'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/5b2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3 Bf5

			variation => N__('Old Indian: 3. Nf3 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/8/2PP2b1/1Q3N2/PP2PPPP/RNB1KB1R b KQkq - 3 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3 Bg4 4. Qb3

			variation => N__('Old Indian: 3. Nf3 Bg4 4. Qb3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/8/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3 Bg4 4. Nc3

			variation => N__('Old Indian: 3. Nf3 Bg4 4. Nc3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/8/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3 Bg4

			variation => N__('Old Indian: 3. Nf3 Bg4'),
			moves => {
				'b1c3' => 'rn1qkb1r/ppp1pppp/3p1n2/8/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
				'd1b3' => 'rn1qkb1r/ppp1pppp/3p1n2/8/2PP2b1/1Q3N2/PP2PPPP/RNB1KB1R b KQkq - 3 4',
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/8/3P2b1/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A41f',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. e3 Nf6

			variation => N__('Neo-Old Indian: Wade Defence, 3. e3 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p1n2/8/3PP1b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A41k',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. e4 Nf6

			variation => N__('Neo-Old Indian: Wade Defence, 3. e4 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p4/3nP3/2PP2b1/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'B05c',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. c4

			variation => N__('Alekhine: Modern, Alekhine Variation'),
			moves => {
				'd5b6' => 'rn1qkb1r/ppp1pppp/1n1p4/4P3/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rn1qkb1r/ppp1pppp/3p4/3nP3/3P2b1/5N1P/PPP2PP1/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B05b',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. h3

			variation => N__('Alekhine: Modern, Panov Variation'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/3p4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R b KQkq - 3 5' => {
			eco => 'B05f',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2

			variation => N__('Alekhine: Modern, 5. Be2'),
			moves => {
				'b8c6' => 'r2qkb1r/ppp1pppp/2np4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 4 6',
				'c7c6' => 'rn1qkb1r/pp2pppp/2pp4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6',
				'e7e6' => 'rn1qkb1r/ppp2ppp/3pp3/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/ppp1pppp/3p4/3nP3/3P2b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'B05a',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4

			variation => N__('Alekhine: Modern, 4... Bg4'),
			moves => {
				'c2c4' => 'rn1qkb1r/ppp1pppp/3p4/3nP3/2PP2b1/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 5',
				'f1e2' => 'rn1qkb1r/ppp1pppp/3p4/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R b KQkq - 3 5',
				'h2h3' => 'rn1qkb1r/ppp1pppp/3p4/3nP3/3P2b1/5N1P/PPP2PP1/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rn1qkb1r/ppp1pppp/4bn2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'D25f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Be6 5. Nc3

			variation => N__('QGA: Flohr, 5. Nc3'),
			moves => {
				'c7c6' => 'rn1qkb1r/pp2pppp/2p1bn2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/ppp1pppp/4bn2/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D25f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Be6

			variation => N__('QGA: Flohr Variation'),
			moves => {
				'b1c3' => 'rn1qkb1r/ppp1pppp/4bn2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5B2/3p1b2/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 0 4' => {
			eco => 'D01k',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Bf5 4. Bxf6

			variation => N__('Richter-Veresov: 3... Bf5 4.Bxf6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/1B1P1b2/3P4/5P2/PPP3PP/RNBQK1NR b KQkq - 2 5' => {
			eco => 'B01v',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. f3 Bf5 5. Bb5+

			variation => N__('Scandinavian: Portuguese, 4. f3 Bf5 5. Bb5+'),
			moves => {
				'b8d7' => 'r2qkb1r/pppnpppp/5n2/1B1P1b2/3P4/5P2/PPP3PP/RNBQK1NR w KQkq - 3 6',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/1B1P4/3P2b1/8/PPP2PPP/RNBQK1NR b KQkq - 2 4' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Bb5+

			variation => N__('Scandinavian: Portuguese, 4. Bb5+'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P1b2/2PP4/5P2/PP4PP/RNBQKBNR b KQkq c3 0 5' => {
			eco => 'B01v',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. f3 Bf5 5. c4

			variation => N__('Scandinavian: Portuguese, 4. f3 Bf5 5. c4'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P1b2/3P4/5P2/PPP3PP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B01v',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. f3 Bf5

			variation => N__('Scandinavian: Portuguese, 4. f3 Bf5'),
			moves => {
				'c2c4' => 'rn1qkb1r/ppp1pppp/5n2/3P1b2/2PP4/5P2/PP4PP/RNBQKBNR b KQkq c3 0 5',
				'f1b5' => 'rn1qkb1r/ppp1pppp/5n2/1B1P1b2/3P4/5P2/PPP3PP/RNBQK1NR b KQkq - 2 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P4/2B3b1/8/PPPP1PPP/RNBQK1NR w KQkq - 5 5' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+ Bd7 4. Bc4 Bg4

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7 4. Bc4 Bg4'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Nf3

			variation => N__('Scandinavian: Portuguese, 4. Nf3'),
			moves => {
				'd8d5' => 'rn2kb1r/ppp1pppp/5n2/3q4/3P2b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/5P2/PPP3PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B01v',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. f3

			variation => N__('Scandinavian: Portuguese, 4. f3'),
			moves => {
				'g4f5' => 'rn1qkb1r/ppp1pppp/5n2/3P1b2/3P4/5P2/PPP3PP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/8/PPP1BPPP/RNBQK1NR b KQkq - 2 4' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Be2

			variation => N__('Scandinavian: Portuguese, 4. Be2'),
			moves => {
				'g4e2' => 'rn1qkb1r/ppp1pppp/5n2/3P4/3P4/8/PPP1bPPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4

			variation => N__('Scandinavian: Portuguese Variation'),
			moves => {
				'f1b5' => 'rn1qkb1r/ppp1pppp/5n2/1B1P4/3P2b1/8/PPP2PPP/RNBQK1NR b KQkq - 2 4',
				'f1e2' => 'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/8/PPP1BPPP/RNBQK1NR b KQkq - 2 4',
				'f2f3' => 'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/5P2/PPP3PP/RNBQKBNR b KQkq - 0 4',
				'g1f3' => 'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3P4/3P4/8/PPP1bPPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Be2 Bxe2

			variation => N__('Scandinavian: Portuguese, 4. Be2 Bxe2'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 4 4' => {
			eco => 'D02m',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4 Bf5

			variation => N__('Queen\'s Pawn: London'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1b2/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'D04c',
                        # TRANSLATORS:

			variation => N__('Colle: 3... Bf5'),
			moves => {
				'e7e6' => 'rn1qkb1r/ppp2ppp/4pn2/3p1b2/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1b2/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'D04c',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 Bf5

			variation => N__('Colle: 3... Bf5'),
			moves => {
				'f1d3' => 'rn1qkb1r/ppp1pppp/5n2/3p1b2/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1b2/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 3 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 Bf5

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1bB1/3P4/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 5 4' => {
			eco => 'D01j',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Bf5 4. Nf3

			variation => N__('Richter-Veresov: 3... Bf5 4. Nf3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1bB1/3P4/2N2P2/PPP1P1PP/R2QKBNR b KQkq - 0 4' => {
			eco => 'D01i',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Bf5 4. f3

			variation => N__('Richter-Veresov: 3... Bf5 4. f3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p1bB1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 4 4' => {
			eco => 'D01h',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Bf5

			variation => N__('Richter-Veresov: 3... Bf5'),
			moves => {
				'f2f3' => 'rn1qkb1r/ppp1pppp/5n2/3p1bB1/3P4/2N2P2/PPP1P1PP/R2QKBNR b KQkq - 0 4',
				'g1f3' => 'rn1qkb1r/ppp1pppp/5n2/3p1bB1/3P4/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 5 4',
				'g5f6' => 'rn1qkb1r/ppp1pppp/5B2/3p1b2/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 0 4',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p4/3P2b1/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'D04b',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 Bg4

			variation => N__('Colle: 3... Bg4'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/3p4/5Pb1/4PN2/PPPP2PP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. e3 Bg4

			variation => N__('Bird: Lasker, 3... Bg4'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/5b2/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D00w',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Nxf3 Bf5

			variation => N__('Blackmar-Diemer: Gunderam Defence'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/5b2/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 Bf5

			variation => N__('Blackmar-Diemer: Vienna Defence'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/8/2BP2b1/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D25g',
                        # TRANSLATORS:

			variation => N__('QGA: Janowski-Larsen Variation'),
			moves => {
				'e7e6' => 'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/4PN2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/8/2pP2b1/4PN1P/PP3PP1/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D25g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. h3

			variation => N__('QGA: Janowski-Larsen, 5. h3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/8/2pP2b1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D25g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4

			variation => N__('QGA: Janowski-Larsen Variation'),
			moves => {
				'f1c4' => 'rn1qkb1r/ppp1pppp/5n2/8/2BP2b1/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5',
				'h2h3' => 'rn1qkb1r/ppp1pppp/5n2/8/2pP2b1/4PN1P/PP3PP1/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rn1qkb1r/ppp1pppp/5n2/8/3P2b1/2N2N2/PPP3PP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D00w',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Nxf3 Bg4

			variation => N__('Blackmar-Diemer: Tartakower Defence'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/8/3n1b2/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'B01w',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 Bf5

			variation => N__('Scandinavian: Marshall, 4. Nf3 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/8/3n1b2/3P4/8/PPP1BPPP/RNBQK1NR w KQkq - 1 6' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+ Bd7 4. Be2 Nxd5 5. d4 Bf5

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7 4. Be2 Nxd5 5. d4 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/ppp1pppp/8/3n4/3P2b1/5N2/PPP1BPPP/RNBQK2R b KQkq - 3 5' => {
			eco => 'B01w',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 Bg4 5. Be2

			variation => N__('Scandinavian: Marshall, 4. Nf3 Bg4 5. Be2'),
			moves => {
				'e7e6' => 'rn1qkb1r/ppp2ppp/4p3/3n4/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkb1r/ppp1pppp/8/3n4/3P2b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'B01w',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 Bg4

			variation => N__('Scandinavian: Marshall, 4. Nf3 Bg4'),
			moves => {
				'f1e2' => 'rn1qkb1r/ppp1pppp/8/3n4/3P2b1/5N2/PPP1BPPP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rn1qkb1r/ppp2ppp/1n1p4/8/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B05e',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. c4 Nb6 6. exd6 exd6

			variation => N__('Alekhine: Modern, Alekhine Variation, 6.exd6 exd6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/1n2p3/4Pb2/2PP4/2N2N2/PP4PP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'f8b4' => 'rn1qk2r/ppp2ppp/1n2p3/4Pb2/1bPP4/2N2N2/PP4PP/R1BQKB1R w KQkq - 2 9',
				'f8e7' => 'rn1qk2r/ppp1bppp/1n2p3/4Pb2/2PP4/2N2N2/PP4PP/R1BQKB1R w KQkq - 2 9',
			},
		},
		'rn1qkb1r/ppp2ppp/1n2p3/4Pb2/2PP4/2N5/PP4PP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'B03l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Four Pawns Attack, 6... Bf5'),
			moves => {
				'g1f3' => 'rn1qkb1r/ppp2ppp/1n2p3/4Pb2/2PP4/2N2N2/PP4PP/R1BQKB1R b KQkq - 1 8',
			},
		},
		'rn1qkb1r/ppp2ppp/1n2p3/8/2PP2b1/5N2/PP2BPPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 Bg4 6. Be2 e6

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 Bg4 6. Be2 e6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/3pp3/3nP3/3P2b1/5N2/PPP1BPPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'B05l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6 6. O-O

			variation => N__('Alekhine: Modern, 5. Be2 e6 6.O-O'),
			moves => {
				'f8e7' => 'rn1qk2r/ppp1bppp/3pp3/3nP3/3P2b1/5N2/PPP1BPPP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkb1r/ppp2ppp/3pp3/3nP3/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'B05k',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Bg4 5. Be2 e6

			variation => N__('Alekhine: Modern, 5. Be2 e6'),
			moves => {
				'e1g1' => 'rn1qkb1r/ppp2ppp/3pp3/3nP3/3P2b1/5N2/PPP1BPPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkb1r/ppp2ppp/4bn2/8/2P5/5N2/PP1P1PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 e6 4. dxe6 Bxe6 5. Nf3

			variation => N__('Scandinavian: Icelandic Gambit, 5. Nf3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4bn2/8/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 e6 4. dxe6 Bxe6

			variation => N__('Scandinavian: Icelandic Gambit Accepted'),
			moves => {
				'd2d4' => 'rn1qkb1r/ppp2ppp/4bn2/8/2PP4/8/PP3PPP/RNBQKBNR b KQkq d3 0 5',
				'g1f3' => 'rn1qkb1r/ppp2ppp/4bn2/8/2P5/5N2/PP1P1PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rn1qkb1r/ppp2ppp/4bn2/8/2PP4/8/PP3PPP/RNBQKBNR b KQkq d3 0 5' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 e6 4. dxe6 Bxe6 5. d4

			variation => N__('Scandinavian: Icelandic Gambit, 5. d4'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4bp2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'D00k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 exf6 4. e3 Be6

			variation => N__('Trompowsky: 2... d5 3.Bxf6 exf6 4. e3 Be6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4p3/3n4/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'B01w',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 Bg4 5. Be2 e6

			variation => N__('Scandinavian: Marshall, 4. Nf3 Bg4 5. Be2 e6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'D06q',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nf3 e6 4. Nc3 Nf6

			variation => N__('QGD: 2... Bf5 3. Nf3 e6 4. Nc3 Nf6'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/3p1b2/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'D04d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 Bf5 4. Bd3 e6

			variation => N__('Colle: 3... Bf5, Alekhine Variation'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQK2R b KQkq - 2 7' => {
			eco => 'D25m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3 Bh5 7. Nc3

			variation => N__('QGA: Janowski-Larsen, 6. h3 Bh5 7. Nc3'),
			moves => {
				'a7a6' => 'rn1qkb1r/1pp2ppp/p3pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQK2R w KQkq - 0 8',
				'b8d7' => 'r2qkb1r/pppn1ppp/4pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQK2R w KQkq - 3 8',
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/7b/2BP4/4PN1P/PP3PP1/RNBQK2R w KQkq - 1 7' => {
			eco => 'D25l',
                        # TRANSLATORS:

			variation => N__('QGA: Janowski-Larsen, 6. h3'),
			moves => {
				'b1c3' => 'rn1qkb1r/ppp2ppp/4pn2/7b/2BP4/2N1PN1P/PP3PP1/R1BQK2R b KQkq - 2 7',
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/1Q2PN2/PP3PPP/RNB1K2R b KQkq - 1 6' => {
			eco => 'D25i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. Qb3

			variation => N__('QGA: Janowski-Larsen, 6. Qb3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/2N1PN2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D25k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. Nc3

			variation => N__('QGA: Janowski-Larsen, 6. Nc3'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/4PN1P/PP3PP1/RNBQK2R b KQkq - 0 6' => {
			eco => 'D25l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. h3

			variation => N__('QGA: Janowski-Larsen, 6. h3'),
			moves => {
				'g4h5' => 'rn1qkb1r/ppp2ppp/4pn2/7b/2BP4/4PN1P/PP3PP1/RNBQK2R w KQkq - 1 7',
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/4PN2/PP3PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'D25j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6 6. O-O

			variation => N__('QGA: Janowski-Larsen, 6.O-O'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/4PN2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D25h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 Bg4 5. Bxc4 e6

			variation => N__('QGA: Janowski-Larsen, 5.Bxc4 e6'),
			moves => {
				'b1c3' => 'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/2N1PN2/PP3PPP/R1BQK2R b KQkq - 1 6',
				'd1b3' => 'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/1Q2PN2/PP3PPP/RNB1K2R b KQkq - 1 6',
				'e1g1' => 'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/4PN2/PP3PPP/RNBQ1RK1 b kq - 1 6',
				'h2h3' => 'rn1qkb1r/ppp2ppp/4pn2/8/2BP2b1/4PN1P/PP3PP1/RNBQK2R b KQkq - 0 6',
			},
		},
		'rn1qkb1r/ppp2ppp/5p2/3p1b2/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'D00k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 exf6 4. e3 Bf5

			variation => N__('Trompowsky: 2... d5 3.Bxf6 exf6 4. e3 Bf5'),
			moves => {
			},
		},
		'rn1qkb1r/ppp2ppp/8/3pP3/4n1b1/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C29o',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Bg4

			variation => N__('Vienna Gambit: 5. Nf3 Bg4'),
			moves => {
				'd1e2' => 'rn1qkb1r/ppp2ppp/8/3pP3/4n1b1/2N2N2/PPPPQ1PP/R1B1KB1R b KQkq - 3 6',
			},
		},
		'rn1qkb1r/ppp2ppp/8/3pP3/4n1b1/2N2N2/PPPPQ1PP/R1B1KB1R b KQkq - 3 6' => {
			eco => 'C29o',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Bg4 6. Qe2

			variation => N__('Vienna Gambit: Kaufmann Variation'),
			moves => {
			},
		},
		'rn1qkb1r/pppb1ppp/4pn2/8/2QP4/6P1/PP2PPBP/RNB1K1NR b KQkq - 0 6' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Bd7 6. Qxc4

			variation => N__('Catalan: Open, 5. Qa4+ Bd7 6.Qxc4'),
			moves => {
			},
		},
		'rn1qkb1r/pppb1ppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6' => {
			eco => 'E04a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Bd7

			variation => N__('Catalan: Open, 5. Nf3 Bd7'),
			moves => {
			},
		},
		'rn1qkb1r/pppb1ppp/4pn2/8/Q1pP4/6P1/PP2PPBP/RNB1K1NR w KQkq - 2 6' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+ Bd7

			variation => N__('Catalan: Open, 5. Qa4+ Bd7'),
			moves => {
				'a4c4' => 'rn1qkb1r/pppb1ppp/4pn2/8/2QP4/6P1/PP2PPBP/RNB1K1NR b KQkq - 0 6',
			},
		},
		'rn1qkb1r/pppbpp1p/5np1/1Q6/3P4/2N5/PP2PPPP/R1B1KBNR w KQkq - 3 7' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4 Be6 6. Qb5+ Bd7

			variation => N__('Grünfeld: Early Russian, Adorjan Gambit'),
			moves => {
			},
		},
		'rn1qkb1r/pppbpppp/5n2/1B1P4/8/8/PPPP1PPP/RNBQK1NR w KQkq - 3 4' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+ Bd7

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7'),
			moves => {
				'b5c4' => 'rn1qkb1r/pppbpppp/5n2/3P4/2B5/8/PPPP1PPP/RNBQK1NR b KQkq - 4 4',
				'b5e2' => 'rn1qkb1r/pppbpppp/5n2/3P4/8/8/PPPPBPPP/RNBQK1NR b KQkq - 4 4',
			},
		},
		'rn1qkb1r/pppbpppp/5n2/3P4/2B5/8/PPPP1PPP/RNBQK1NR b KQkq - 4 4' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+ Bd7 4. Bc4

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7 4. Bc4'),
			moves => {
				'd7g4' => 'rn1qkb1r/ppp1pppp/5n2/3P4/2B3b1/8/PPPP1PPP/RNBQK1NR w KQkq - 5 5',
			},
		},
		'rn1qkb1r/pppbpppp/5n2/3P4/8/8/PPPPBPPP/RNBQK1NR b KQkq - 4 4' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+ Bd7 4. Be2

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7 4. Be2'),
			moves => {
				'f6d5' => 'rn1qkb1r/pppbpppp/8/3n4/8/8/PPPPBPPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rn1qkb1r/pppbpppp/8/3n4/3P4/8/PPP1BPPP/RNBQK1NR b KQkq d3 0 5' => {
			eco => 'B01s',
                        # TRANSLATORS:

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7 4. Be2'),
			moves => {
				'd7f5' => 'rn1qkb1r/ppp1pppp/8/3n1b2/3P4/8/PPP1BPPP/RNBQK1NR w KQkq - 1 6',
			},
		},
		'rn1qkb1r/pppbpppp/8/3n4/8/8/PPPPBPPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'B01s',
                        # TRANSLATORS:

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+ Bd7 4. Be2'),
			moves => {
				'd2d4' => 'rn1qkb1r/pppbpppp/8/3n4/3P4/8/PPP1BPPP/RNBQK1NR b KQkq d3 0 5',
			},
		},
		'rn1qkbnr/1pp1pppp/p7/8/2BP2b1/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D22g',
                        # TRANSLATORS:

			variation => N__('QGA: Alekhine, 4. e3 Bg4'),
			moves => {
				'e7e6' => 'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/4PN2/PP3PPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkbnr/1pp1pppp/p7/8/2pP2b1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D22g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 Bg4

			variation => N__('QGA: Alekhine, 4. e3 Bg4'),
			moves => {
				'f1c4' => 'rn1qkbnr/1pp1pppp/p7/8/2BP2b1/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rn1qkbnr/1pp2ppp/p3p3/3P4/2B3b1/4PN2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'D22i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 Bg4 5. Bxc4 e6 6. d5

			variation => N__('QGA: Alekhine, Alatortsev Variation'),
			moves => {
			},
		},
		'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/1Q2PN2/PP3PPP/RNB1K2R b KQkq - 1 6' => {
			eco => 'D22j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 Bg4 5. Bxc4 e6 6. Qb3

			variation => N__('QGA: Alekhine, 4. e3 Bg4 5.Bxc4 e6 6. Qb3'),
			moves => {
			},
		},
		'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/2N1PN2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D22k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 Bg4 5. Bxc4 e6 6. Nc3

			variation => N__('QGA: Alekhine, 4. e3 Bg4 5.Bxc4 e6 6. Nc3'),
			moves => {
			},
		},
		'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/4PN1P/PP3PP1/RNBQK2R b KQkq - 0 6' => {
			eco => 'D22l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 Bg4 5. Bxc4 e6 6. h3

			variation => N__('QGA: Alekhine, 4. e3 Bg4 5.Bxc4 e6 6. h3'),
			moves => {
			},
		},
		'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/4PN2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D22h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 Bg4 5. Bxc4 e6

			variation => N__('QGA: Alekhine, 4. e3 Bg4 5.Bxc4 e6'),
			moves => {
				'b1c3' => 'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/2N1PN2/PP3PPP/R1BQK2R b KQkq - 1 6',
				'd1b3' => 'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/1Q2PN2/PP3PPP/RNB1K2R b KQkq - 1 6',
				'd4d5' => 'rn1qkbnr/1pp2ppp/p3p3/3P4/2B3b1/4PN2/PP3PPP/RNBQK2R b KQkq - 0 6',
				'h2h3' => 'rn1qkbnr/1pp2ppp/p3p3/8/2BP2b1/4PN1P/PP3PP1/RNBQK2R b KQkq - 0 6',
			},
		},
		'rn1qkbnr/p3pppp/1pp5/2N2b2/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B18c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Nc5 b6

			variation => N__('Caro-Kann: Classical, 5. Nc5 b6'),
			moves => {
			},
		},
		'rn1qkbnr/pb1p1ppp/1p2p3/2p5/3PP3/2PB1N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'B00k',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3 e6 4. Nf3 c5 5. c3

			variation => N__('Owen Defence: 3. Bd3 e6 4. Nf3 c5 5. c3'),
			moves => {
			},
		},
		'rn1qkbnr/pb1p1ppp/1p2p3/2p5/3PP3/3B1N2/PPP2PPP/RNBQK2R w KQkq c6 0 5' => {
			eco => 'B00k',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3 e6 4. Nf3 c5

			variation => N__('Owen Defence: 3. Bd3 e6 4. Nf3 c5'),
			moves => {
				'c2c3' => 'rn1qkbnr/pb1p1ppp/1p2p3/2p5/3PP3/2PB1N2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rn1qkbnr/pb1ppppp/1p6/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 2 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 b6 3. Nf3 Bb7

			variation => N__('English: Symmetrical, 2... b6'),
			moves => {
			},
		},
		'rn1qkbnr/pb1ppppp/1p6/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A30b',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 2. Nf3 b6 3. g3'),
			moves => {
				'f1g2' => 'rn1qkbnr/pb1ppppp/1p6/2p5/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rn1qkbnr/pb1ppppp/1p6/2p5/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A30b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 b6 3. g3 Bb7 4. Bg2

			variation => N__('English: Symmetrical, 2. Nf3 b6 3. g3 Bb7 4. Bg2'),
			moves => {
			},
		},
		'rn1qkbnr/pb1ppppp/1p6/2p5/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 b6 3. Bg2 Bb7

			variation => N__('Reti: 1... c5'),
			moves => {
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nc3 Bb7 3. e4 e6

			variation => N__('English: 1... b6 2. Nc3 Bb7 3. e4 e6'),
			moves => {
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A40d',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: English Defence, 2. c4 Bb7'),
			moves => {
				'e2e4' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'A40d',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: English Defence, 2. c4 Bb7'),
			moves => {
				'f7f5' => 'rn1qkbnr/pbpp2pp/1p2p3/5p2/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq f6 0 5',
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/2PPP3/5P2/PP4PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'A40g',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: English Defence, 3. e4'),
			moves => {
				'f7f5' => 'rn1qkbnr/pbpp2pp/1p2p3/5p2/2PPP3/5P2/PP4PP/RNBQKBNR w KQkq f6 0 5',
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/2PPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A40g',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: English Defence, 3. e4'),
			moves => {
				'f2f3' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/2PPP3/5P2/PP4PP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/3PP3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'B00j',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3 e6 4. Nf3

			variation => N__('Owen Defence: 3. Bd3 e6 4. Nf3'),
			moves => {
				'c7c5' => 'rn1qkbnr/pb1p1ppp/1p2p3/2p5/3PP3/3B1N2/PPP2PPP/RNBQK2R w KQkq c6 0 5',
			},
		},
		'rn1qkbnr/pbpp1ppp/1p2p3/8/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'B00i',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3 e6

			variation => N__('Owen Defence: 3. Bd3 e6'),
			moves => {
				'g1f3' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/3PP3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rn1qkbnr/pbpp2pp/1p2p3/5p2/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq f6 0 5' => {
			eco => 'A40g',
                        # TRANSLATORS: 1. d4 b6 2. c4 Bb7 3. Nc3 e6 4. e4 f5

			variation => N__('Queen\'s Pawn: English Defence, Hartlaub Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pbpp2pp/1p2p3/5p2/2PPP3/5P2/PP4PP/RNBQKBNR w KQkq f6 0 5' => {
			eco => 'A40g',
                        # TRANSLATORS: 1. d4 b6 2. c4 e6 3. e4 Bb7 4. f3 f5

			variation => N__('Queen\'s Pawn: English Defence, Polli Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pbppp1pp/1p6/5p2/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq f6 0 4' => {
			eco => 'B00g',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3 f5

			variation => N__('Owen Defence: Matinovsky Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pbpppppp/1p6/6B1/3PP3/8/PPP2PPP/RN1QKBNR b KQkq - 2 3' => {
			eco => 'B00f',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bg5

			variation => N__('Owen Defence: Naselwaus Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nc3 Bb7 3. e4

			variation => N__('English: 1... b6 2. Nc3 Bb7 3. e4'),
			moves => {
				'e7e6' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nc3 Bb7

			variation => N__('English: 1... b6 2. Nc3 Bb7'),
			moves => {
				'e2e4' => 'rn1qkbnr/pbpppppp/1p6/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3',
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nf3 Bb7

			variation => N__('English: 1... b6 2. Nf3 Bb7'),
			moves => {
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'A40d',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: English Defence, 2. c4 Bb7'),
			moves => {
				'e7e6' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A40d',
                        # TRANSLATORS: 1. d4 b6 2. c4 Bb7

			variation => N__('Queen\'s Pawn: English Defence, 2. c4 Bb7'),
			moves => {
				'b1c3' => 'rn1qkbnr/pbpppppp/1p6/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/3PP3/3B4/PPP2PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'B00g',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Bd3

			variation => N__('Owen Defence: 3. Bd3'),
			moves => {
				'e7e6' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq - 0 4',
				'f7f5' => 'rn1qkbnr/pbppp1pp/1p6/5p2/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq f6 0 4',
				'g8f6' => 'rn1qkb1r/pbpppppp/1p3n2/8/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq - 3 4',
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B00f',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7 3. Nf3

			variation => N__('Owen Defence: Smith Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pbpppppp/1p6/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B00f',
                        # TRANSLATORS: 1. e4 b6 2. d4 Bb7

			variation => N__('Owen Defence: 2. d4 Bb7'),
			moves => {
				'c1g5' => 'rn1qkbnr/pbpppppp/1p6/6B1/3PP3/8/PPP2PPP/RN1QKBNR b KQkq - 2 3',
				'f1d3' => 'rn1qkbnr/pbpppppp/1p6/8/3PP3/3B4/PPP2PPP/RNBQK1NR b KQkq - 2 3',
				'g1f3' => 'rn1qkbnr/pbpppppp/1p6/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rn1qkbnr/pbpppppp/8/1B6/3PP3/8/PPP2PPP/RNBQK1NR b KQkq - 0 3' => {
			eco => 'A40c',
                        # TRANSLATORS: 1. d4 b5 2. e4 Bb7 3. Bxb5

			variation => N__('Queen\'s Pawn: Polish Defence, Spassky Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pbpppppp/8/1p6/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A40c',
                        # TRANSLATORS: 1. d4 b5 2. e4 Bb7

			variation => N__('Queen\'s Pawn: Polish Defence'),
			moves => {
				'f1b5' => 'rn1qkbnr/pbpppppp/8/1B6/3PP3/8/PPP2PPP/RNBQK1NR b KQkq - 0 3',
			},
		},
		'rn1qkbnr/pp1B1ppp/8/3pp3/3P4/8/PPP2PPP/RNBQK1NR b KQkq - 0 6' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1n1ppp/8/3pp3/3P4/8/PPP2PPP/RNBQK1NR w KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp1Bpppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B52a',
                        # TRANSLATORS:

			variation => N__('Sicilian: 3. Bb5+ Bd7'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1npppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
				'd8d7' => 'rn2kbnr/pp1qpppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rn1qkbnr/pp1b1ppp/4p3/3P4/3Q4/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'D32f',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5. Qa4+'),
			moves => {
				'e6d5' => 'rn1qkbnr/pp1b1ppp/8/3p4/3Q4/2N5/PP2PPPP/R1B1KBNR w KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp1b1ppp/4p3/3P4/Q2p4/2N5/PP2PPPP/R1B1KBNR w KQkq - 2 6' => {
			eco => 'D32f',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5. Qa4+'),
			moves => {
				'a4d4' => 'rn1qkbnr/pp1b1ppp/4p3/3P4/3Q4/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp1b1ppp/8/1B1pp3/3P4/8/PPP2PPP/RNBQK1NR w KQkq - 2 6' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'b5d7' => 'rn1qkbnr/pp1B1ppp/8/3pp3/3P4/8/PPP2PPP/RNBQK1NR b KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp1b1ppp/8/1Bpp4/3P4/8/PPPN1PPP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'C08d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Bb5+ Bd7

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5. Bb5+ Bd7'),
			moves => {
				'd1e2' => 'rn1qkbnr/pp1b1ppp/8/1Bpp4/3P4/8/PPPNQPPP/R1B1K1NR b KQkq - 3 6',
			},
		},
		'rn1qkbnr/pp1b1ppp/8/1Bpp4/3P4/8/PPPNQPPP/R1B1K1NR b KQkq - 3 6' => {
			eco => 'C08e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Bb5+ Bd7 6. Qe2+

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5. Bb5+ Bd7 6. Qe2+'),
			moves => {
				'f8e7' => 'rn1qk1nr/pp1bbppp/8/1Bpp4/3P4/8/PPPNQPPP/R1B1K1NR w KQkq - 4 7',
			},
		},
		'rn1qkbnr/pp1b1ppp/8/3Q4/8/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 7' => {
			eco => 'D32f',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5. Qa4+'),
			moves => {
				'b8c6' => 'r2qkbnr/pp1b1ppp/2n5/3Q4/8/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 8',
			},
		},
		'rn1qkbnr/pp1b1ppp/8/3p4/3Q4/2N5/PP2PPPP/R1B1KBNR w KQkq - 0 7' => {
			eco => 'D32f',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5. Qa4+'),
			moves => {
				'd4d5' => 'rn1qkbnr/pp1b1ppp/8/3Q4/8/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp1bpppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4' => {
			eco => 'B52a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7

			variation => N__('Sicilian: 3. Bb5+ Bd7'),
			moves => {
				'b5d7' => 'rn1qkbnr/pp1Bpppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'rn1qkbnr/pp1bpppp/3p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5' => {
			eco => 'B53e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Bd7

			variation => N__('Sicilian, Chekhover, 4... Bd7'),
			moves => {
			},
		},
		'rn1qkbnr/pp2ppp1/2p3bp/7P/3P4/6N1/PPP2PP1/R1BQKBNR b KQkq - 0 7' => {
			eco => 'B18s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. h5

			variation => N__('Caro-Kann: Classical, 6. h4 h6 7. h5'),
			moves => {
			},
		},
		'rn1qkbnr/pp2ppp1/2p3bp/8/3P1P1P/6N1/PPP3P1/R1BQKBNR b KQkq f3 0 7' => {
			eco => 'B18r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. f4

			variation => N__('Caro-Kann: Classical, 6. h4 h6 7. f4'),
			moves => {
			},
		},
		'rn1qkbnr/pp2ppp1/2p3bp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B19a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3

			variation => N__('Caro-Kann: Classical, 7. Nf3'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1nppp1/2p3bp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R w KQkq - 2 8',
				'e7e6' => 'rn1qkbnr/pp3pp1/2p1p1bp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R w KQkq - 0 8',
				'g8f6' => 'rn1qkb1r/pp2ppp1/2p2nbp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rn1qkbnr/pp2ppp1/2p3bp/8/3P3P/6N1/PPP2PP1/R1BQKBNR w KQkq - 0 7' => {
			eco => 'B18p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6

			variation => N__('Caro-Kann: Classical, 6. h4 h6'),
			moves => {
				'f2f4' => 'rn1qkbnr/pp2ppp1/2p3bp/8/3P1P1P/6N1/PPP3P1/R1BQKBNR b KQkq f3 0 7',
				'g1f3' => 'rn1qkbnr/pp2ppp1/2p3bp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R b KQkq - 1 7',
				'g1h3' => 'rn1qkbnr/pp2ppp1/2p3bp/8/3P3P/6NN/PPP2PP1/R1BQKB1R b KQkq - 1 7',
				'h4h5' => 'rn1qkbnr/pp2ppp1/2p3bp/7P/3P4/6N1/PPP2PP1/R1BQKBNR b KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp2ppp1/2p3bp/8/3P3P/6NN/PPP2PP1/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B18q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nh3

			variation => N__('Caro-Kann: Classical, 6. h4 h6 7. Nh3'),
			moves => {
			},
		},
		'rn1qkbnr/pp2ppp1/2p5/3pPb1p/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq h6 0 5' => {
			eco => 'B12v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3 h5

			variation => N__('Caro-Kann: Advance, 4. Nc3 h5'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p1b3/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D11e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4 4. e3 Be6

			variation => N__('Slav: 3. Nf3 dxc4 4. e3 Be6'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/2BP4/6N1/PPP2PPP/R1BQK1NR b KQkq - 4 6' => {
			eco => 'B18i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Bc4

			variation => N__('Caro-Kann: Classical, 6. Bc4'),
			moves => {
				'e7e6' => 'rn1qkbnr/pp3ppp/2p1p1b1/8/2BP4/6N1/PPP2PPP/R1BQK1NR w KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/3P1P2/6N1/PPP3PP/R1BQKBNR b KQkq f3 0 6' => {
			eco => 'B18f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. f4

			variation => N__('Caro-Kann: Classical, Maroczy Attack'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/3P3P/6N1/PPP2PP1/R1BQKBNR b KQkq h3 0 6' => {
			eco => 'B18p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4

			variation => N__('Caro-Kann: Classical, 6. h4'),
			moves => {
				'h7h6' => 'rn1qkbnr/pp2ppp1/2p3bp/8/3P3P/6N1/PPP2PP1/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/3P4/5NN1/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B18l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Nf3

			variation => N__('Caro-Kann: Classical, 6. Nf3'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1npppp/2p3b1/8/3P4/5NN1/PPP2PPP/R1BQKB1R w KQkq - 5 7',
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/3P4/6N1/PPP1NPPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B18h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. N1e2

			variation => N__('Caro-Kann: Classical, 6.N1e2'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/3P4/6N1/PPP2PPP/R1BQKBNR w KQkq - 3 6' => {
			eco => 'B18e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6

			variation => N__('Caro-Kann: Classical, 5. Ng3 Bg6'),
			moves => {
				'f1c4' => 'rn1qkbnr/pp2pppp/2p3b1/8/2BP4/6N1/PPP2PPP/R1BQK1NR b KQkq - 4 6',
				'f2f4' => 'rn1qkbnr/pp2pppp/2p3b1/8/3P1P2/6N1/PPP3PP/R1BQKBNR b KQkq f3 0 6',
				'g1e2' => 'rn1qkbnr/pp2pppp/2p3b1/8/3P4/6N1/PPP1NPPP/R1BQKB1R b KQkq - 4 6',
				'g1f3' => 'rn1qkbnr/pp2pppp/2p3b1/8/3P4/5NN1/PPP2PPP/R1BQKB1R b KQkq - 4 6',
				'g1h3' => 'rn1qkbnr/pp2pppp/2p3b1/8/3P4/6NN/PPP2PPP/R1BQKB1R b KQkq - 4 6',
				'h2h4' => 'rn1qkbnr/pp2pppp/2p3b1/8/3P3P/6N1/PPP2PP1/R1BQKBNR b KQkq h3 0 6',
			},
		},
		'rn1qkbnr/pp2pppp/2p3b1/8/3P4/6NN/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B18g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Nh3

			variation => N__('Caro-Kann: Classical, Flohr Variation'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/2N2b2/3P4/8/PPP2PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B18b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Nc5

			variation => N__('Caro-Kann: Classical, 5. Nc5'),
			moves => {
				'b7b6' => 'rn1qkbnr/p3pppp/1pp5/2N2b2/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3P3b/8/2N2N1P/PPPP1PP1/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B11b',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5'),
			moves => {
				'c6d5' => 'rn1qkbnr/pp2pppp/8/3p3b/8/2N2N1P/PPPP1PP1/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3P4/Q2P4/8/PP2PPPP/1RB1KBNR b Kkq - 0 5' => {
			eco => 'D06u',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. cxd5 Bxb1 4. Qa4+ c6 5. Rxb1

			variation => N__('QGD: 2... Bf5 3.cxd5 Bxb1 4. Qa4+ c6 5.Rxb1'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3P4/Q2P4/8/PP2PPPP/RbB1KBNR w KQkq - 0 5' => {
			eco => 'D06t',
                        # TRANSLATORS:

			variation => N__('QGD: 2... Bf5 3.cxd5 Bxb1 4. Qa4+'),
			moves => {
				'a1b1' => 'rn1qkbnr/pp2pppp/2p5/3P4/Q2P4/8/PP2PPPP/1RB1KBNR b Kkq - 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'D11c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Bf5 4. Nc3

			variation => N__('Slav: 3. Nf3 Bf5 4. Nc3'),
			moves => {
				'e7e6' => 'rn1qkbnr/pp3ppp/2p1p3/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p1b2/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'D10h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. e3 Bf5

			variation => N__('Slav: 3. e3 Bf5'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p1b2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D11c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Bf5

			variation => N__('Slav: 3. Nf3 Bf5'),
			moves => {
				'b1c3' => 'rn1qkbnr/pp2pppp/2p5/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'D02d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bf5 3. Bf4 c6

			variation => N__('Queen\'s Pawn: 2. Nf3 Bf5 3. Bf4 c6'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p1b2/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D02d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bf5 3. e3 c6

			variation => N__('Queen\'s Pawn: 2. Nf3 Bf5 3. e3 c6'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p3b/4P3/2N2N1P/PPPP1PP1/R1BQKB1R w KQkq - 1 5' => {
			eco => 'B11b',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bh5

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5'),
			moves => {
				'e4d5' => 'rn1qkbnr/pp2pppp/2p5/3P3b/8/2N2N1P/PPPP1PP1/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/2P3b1/1P3N2/P2PPPPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Bg4

			variation => N__('English: Bogoljubow Variation'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/2P3b1/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. g3 Bg4

			variation => N__('English: Caro-Kann Defence, 3. g3 Bg4'),
			moves => {
				'f1g2' => 'rn1qkbnr/pp2pppp/2p5/3p4/2P3b1/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/2P3b1/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. g3 Bg4 4. Bg2

			variation => N__('English: Caro-Kann Defence, 3. g3 Bg4'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D11b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Bg4

			variation => N__('Slav: 3. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/4P1b1/2N2N1P/PPPP1PP1/R1BQKB1R b KQkq - 0 4' => {
			eco => 'B11b',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3'),
			moves => {
				'g4f3' => 'rn1qkbnr/pp2pppp/2p5/3p4/4P3/2N2b1P/PPPP1PP1/R1BQKB1R w KQkq - 0 5',
				'g4h5' => 'rn1qkbnr/pp2pppp/2p5/3p3b/4P3/2N2N1P/PPPP1PP1/R1BQKB1R w KQkq - 1 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/4P1b1/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 2 4' => {
			eco => 'B11a',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4

			variation => N__('Caro-Kann: Two Knights, 3... Bg4'),
			moves => {
				'h2h3' => 'rn1qkbnr/pp2pppp/2p5/3p4/4P1b1/2N2N1P/PPPP1PP1/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/4P3/2N2Q1P/PPPP1PP1/R1B1KB1R b KQkq - 0 5' => {
			eco => 'B11d',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3'),
			moves => {
				'd5e4' => 'rn1qkbnr/pp2pppp/2p5/8/4p3/2N2Q1P/PPPP1PP1/R1B1KB1R w KQkq - 0 6',
				'e7e6' => 'rn1qkbnr/pp3ppp/2p1p3/3p4/4P3/2N2Q1P/PPPP1PP1/R1B1KB1R w KQkq - 0 6',
				'g8f6' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/4P3/2N2Q1P/PPPP1PP1/R1B1KB1R w KQkq - 1 6',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/4P3/2N2b1P/PPPP1PP1/R1BQKB1R w KQkq - 0 5' => {
			eco => 'B11d',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3'),
			moves => {
				'd1f3' => 'rn1qkbnr/pp2pppp/2p5/3p4/4P3/2N2Q1P/PPPP1PP1/R1B1KB1R b KQkq - 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/6b1/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c6 3. Bg2 Bg4 4. O-O

			variation => N__('Reti: KIA, Yugoslav'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1npppp/2p5/3p4/6b1/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 4 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3p4/6b1/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c6 3. Bg2 Bg4

			variation => N__('Reti: KIA, Yugoslav'),
			moves => {
				'e1g1' => 'rn1qkbnr/pp2pppp/2p5/3p4/6b1/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P2P1/8/PPP2P1P/RNBQKBNR b KQkq g3 0 4' => {
			eco => 'B12l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. g4

			variation => N__('Caro-Kann: Advance, Bayonet Variation'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P3P/8/PPP2PP1/RNBQKBNR b KQkq h3 0 4' => {
			eco => 'B12m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. h4

			variation => N__('Caro-Kann: Advance, 4. h4'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 4' => {
			eco => 'B12v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3

			variation => N__('Caro-Kann: Advance, 4. Nc3'),
			moves => {
				'd8b6' => 'rn2kbnr/pp2pppp/1qp5/3pPb2/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 3 5',
				'e7e6' => 'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
				'h7h5' => 'rn1qkbnr/pp2ppp1/2p5/3pPb1p/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq h6 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B12n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. c3

			variation => N__('Caro-Kann: Advance, 4. c3'),
			moves => {
				'e7e6' => 'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/3B4/PPP2PPP/RNBQK1NR b KQkq - 2 4' => {
			eco => 'B12o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Bd3

			variation => N__('Caro-Kann: Advance, 4. Bd3'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'B12p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3

			variation => N__('Caro-Kann: Advance, 4. Nf3'),
			moves => {
				'e7e6' => 'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B12l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5

			variation => N__('Caro-Kann: Advance, 3... Bf5'),
			moves => {
				'b1c3' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 4',
				'c2c3' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4',
				'f1d3' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/3B4/PPP2PPP/RNBQK1NR b KQkq - 2 4',
				'g1f3' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 4',
				'g2g4' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P2P1/8/PPP2P1P/RNBQKBNR b KQkq g3 0 4',
				'h2h4' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P3P/8/PPP2PP1/RNBQKBNR b KQkq h3 0 4',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/5b2/3P4/6N1/PPP2PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B18d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3

			variation => N__('Caro-Kann: Classical, 5. Ng3'),
			moves => {
				'f5g6' => 'rn1qkbnr/pp2pppp/2p3b1/8/3P4/6N1/PPP2PPP/R1BQKBNR w KQkq - 3 6',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/5b2/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B18a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5

			variation => N__('Caro-Kann: Classical'),
			moves => {
				'e4c5' => 'rn1qkbnr/pp2pppp/2p5/2N2b2/3P4/8/PPP2PPP/R1BQKBNR b KQkq - 2 5',
				'e4g3' => 'rn1qkbnr/pp2pppp/2p5/5b2/3P4/6N1/PPP2PPP/R1BQKBNR b KQkq - 2 5',
			},
		},
		'rn1qkbnr/pp2pppp/2p5/8/4N3/5Q1P/PPPP1PP1/R1B1KB1R b KQkq - 0 6' => {
			eco => 'B11e',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 dxe4 6. Nxe4

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 dxe4'),
			moves => {
			},
		},
		'rn1qkbnr/pp2pppp/2p5/8/4p3/2N2Q1P/PPPP1PP1/R1B1KB1R w KQkq - 0 6' => {
			eco => 'B11e',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 dxe4

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 dxe4'),
			moves => {
				'c3e4' => 'rn1qkbnr/pp2pppp/2p5/8/4N3/5Q1P/PPPP1PP1/R1B1KB1R b KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp2pppp/8/1B1p3b/8/2N2N1P/PPPP1PP1/R1BQK2R b KQkq - 1 6' => {
			eco => 'B11b',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5'),
			moves => {
				'b8c6' => 'r2qkbnr/pp2pppp/2n5/1B1p3b/8/2N2N1P/PPPP1PP1/R1BQK2R w KQkq - 2 7',
			},
		},
		'rn1qkbnr/pp2pppp/8/3p3b/8/2N2N1P/PPPP1PP1/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B11b',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bh5'),
			moves => {
				'f1b5' => 'rn1qkbnr/pp2pppp/8/1B1p3b/8/2N2N1P/PPPP1PP1/R1BQK2R b KQkq - 1 6',
			},
		},
		'rn1qkbnr/pp3pp1/2p1p1bp/8/3P3P/5NN1/PPP2PP1/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B19b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. h4 h6 7. Nf3 e6

			variation => N__('Caro-Kann: Classical, 7. Nf3 e6'),
			moves => {
			},
		},
		'rn1qkbnr/pp3ppp/2p1p1b1/3pP3/3P2P1/2N5/PPP1NP1P/R1BQKB1R b KQkq - 2 6' => {
			eco => 'B12w',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3 e6 5. g4 Bg6 6. Nge2

			variation => N__('Caro-Kann: Advance, Main Line, 6.Nge2'),
			moves => {
				'c6c5' => 'rn1qkbnr/pp3ppp/4p1b1/2ppP3/3P2P1/2N5/PPP1NP1P/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p1b1/3pP3/3P2P1/2N5/PPP2P1P/R1BQKBNR w KQkq - 1 6' => {
			eco => 'B12w',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Advance, Main Line'),
			moves => {
				'g1e2' => 'rn1qkbnr/pp3ppp/2p1p1b1/3pP3/3P2P1/2N5/PPP1NP1P/R1BQKB1R b KQkq - 2 6',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p1b1/8/2BP4/6N1/PPP1NPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B18j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bf5 5. Ng3 Bg6 6. Bc4 e6 7. N1e2

			variation => N__('Caro-Kann: Classical, 6. Bc4 e6 7.N1e2'),
			moves => {
				'g8f6' => 'rn1qkb1r/pp3ppp/2p1pnb1/8/2BP4/6N1/PPP1NPPP/R1BQK2R w KQkq - 2 8',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p1b1/8/2BP4/6N1/PPP2PPP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'B18i',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Classical, 6. Bc4'),
			moves => {
				'g1e2' => 'rn1qkbnr/pp3ppp/2p1p1b1/8/2BP4/6N1/PPP1NPPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3p1b2/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'D11d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Bf5 4. Nc3 e6 5. Qb3

			variation => N__('Slav: 3. Nf3 Bf5 4. Nc3 e6 5. Qb3'),
			moves => {
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D11c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Bf5 4. Nc3 e6

			variation => N__('Slav: 3. Nf3 Bf5 4. Nc3 e6'),
			moves => {
				'd1b3' => 'rn1qkbnr/pp3ppp/2p1p3/3p1b2/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3p4/3PP3/2N2Q1P/PPP2PP1/R1B1KB1R b KQkq d3 0 6' => {
			eco => 'B11h',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. d4

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. d4'),
			moves => {
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3p4/4P3/2N2Q1P/PPPP1PP1/R1B1KB1R w KQkq - 0 6' => {
			eco => 'B11g',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6'),
			moves => {
				'd2d3' => 'rn1qkbnr/pp3ppp/2p1p3/3p4/4P3/2NP1Q1P/PPP2PP1/R1B1KB1R b KQkq - 0 6',
				'd2d4' => 'rn1qkbnr/pp3ppp/2p1p3/3p4/3PP3/2N2Q1P/PPP2PP1/R1B1KB1R b KQkq d3 0 6',
				'g2g3' => 'rn1qkbnr/pp3ppp/2p1p3/3p4/4P3/2N2QPP/PPPP1P2/R1B1KB1R b KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3p4/4P3/2N2QPP/PPPP1P2/R1B1KB1R b KQkq - 0 6' => {
			eco => 'B11i',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. g3

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. g3'),
			moves => {
				'g8f6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/2N2QPP/PPPP1P2/R1B1KB1R w KQkq - 1 7',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3p4/4P3/2NP1Q1P/PPP2PP1/R1B1KB1R b KQkq - 0 6' => {
			eco => 'B11j',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 Bg4 4. h3 Bxf3 5. Qxf3 e6 6. d3

			variation => N__('Caro-Kann: Two Knights, 3... Bg4 4. h3 Bxf3 5.Qxf3 e6 6. d3'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1n1ppp/2p1p3/3p4/4P3/2NP1Q1P/PPP2PP1/R1B1KB1R w KQkq - 1 7',
				'g8f6' => 'rn1qkb1r/pp3ppp/2p1pn2/3p4/4P3/2NP1Q1P/PPP2PP1/R1B1KB1R w KQkq - 1 7',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P2P1/2N5/PPP2P1P/R1BQKBNR b KQkq g3 0 5' => {
			eco => 'B12w',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3 e6 5. g4

			variation => N__('Caro-Kann: Advance, Main Line'),
			moves => {
				'f5g6' => 'rn1qkbnr/pp3ppp/2p1p1b1/3pP3/3P2P1/2N5/PPP2P1P/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B12v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3 e6

			variation => N__('Caro-Kann: Advance, 4. Nc3 e6'),
			moves => {
				'g2g4' => 'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P2P1/2N5/PPP2P1P/R1BQKBNR b KQkq g3 0 5',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/2P5/PP2BPPP/RNBQK1NR b KQkq - 1 5' => {
			eco => 'B12n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. c3 e6 5. Be2

			variation => N__('Caro-Kann: Advance, Short Variation'),
			moves => {
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B12n',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Advance, 4. c3'),
			moves => {
				'f1e2' => 'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/2P5/PP2BPPP/RNBQK1NR b KQkq - 1 5',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/5N2/PPP1BPPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B12q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1n1ppp/2p1p3/3pPb2/3P4/5N2/PPP1BPPP/RNBQK2R w KQkq - 2 6',
				'c6c5' => 'rn1qkbnr/pp3ppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B12p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6'),
			moves => {
				'f1e2' => 'rn1qkbnr/pp3ppp/2p1p3/3pPb2/3P4/5N2/PPP1BPPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rn1qkbnr/pp3ppp/2p5/3p1b2/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D31l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. cxd5 exd5 5. Nf3 Bf5

			variation => N__('Semi-Slav: 4.cxd5 exd5 5. Nf3 Bf5'),
			moves => {
			},
		},
		'rn1qkbnr/pp3ppp/4p1b1/2ppP3/3P2P1/2N5/PPP1NP1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B12x',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3 e6 5. g4 Bg6 6. Nge2 c5

			variation => N__('Caro-Kann: Advance, Main Line, 6.Nge2 c5'),
			moves => {
			},
		},
		'rn1qkbnr/pp3ppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1QK2R b KQkq - 1 6' => {
			eco => 'B12u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. Be3

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6. Be3'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1n1ppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1QK2R w KQkq - 2 7',
				'g8e7' => 'rn1qkb1r/pp2nppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1QK2R w KQkq - 2 7',
			},
		},
		'rn1qkbnr/pp3ppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'B12t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5 6. O-O

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5 6.O-O'),
			moves => {
				'b8d7' => 'r2qkbnr/pp1n1ppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQ1RK1 w kq - 2 7',
				'g8e7' => 'rn1qkb1r/pp2nppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rn1qkbnr/pp3ppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'B12s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nf3 e6 5. Be2 c5

			variation => N__('Caro-Kann: Advance, 4. Nf3 e6 5. Be2 c5'),
			moves => {
				'c1e3' => 'rn1qkbnr/pp3ppp/4p3/2ppPb2/3P4/4BN2/PPP1BPPP/RN1QK2R b KQkq - 1 6',
				'e1g1' => 'rn1qkbnr/pp3ppp/4p3/2ppPb2/3P4/5N2/PPP1BPPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rn1qkbnr/ppp1pppp/3p4/8/2PP2b1/5N2/PP2PPPP/RNBQKB1R b KQkq c3 0 3' => {
			eco => 'A41g',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4'),
			moves => {
				'b8d7' => 'r2qkbnr/pppnpppp/3p4/8/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 1 4',
				'e7e5' => 'rn1qkbnr/ppp2ppp/3p4/4p3/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq e6 0 4',
				'g4f3' => 'rn1qkbnr/ppp1pppp/3p4/8/2PP4/5b2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp1pppp/3p4/8/2PP4/5b2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A41i',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 Bxf3

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 Bxf3'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/3p4/8/3P2b1/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A41f',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. e3

			variation => N__('Neo-Old Indian: Wade Defence, 3. e3'),
			moves => {
				'b8d7' => 'r2qkbnr/pppnpppp/3p4/8/3P2b1/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
				'g8f6' => 'rn1qkb1r/ppp1pppp/3p1n2/8/3P2b1/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rn1qkbnr/ppp1pppp/3p4/8/3P2b1/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A41f',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4

			variation => N__('Neo-Old Indian: Wade Defence'),
			moves => {
				'c2c4' => 'rn1qkbnr/ppp1pppp/3p4/8/2PP2b1/5N2/PP2PPPP/RNBQKB1R b KQkq c3 0 3',
				'e2e3' => 'rn1qkbnr/ppp1pppp/3p4/8/3P2b1/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'e2e4' => 'rn1qkbnr/ppp1pppp/3p4/8/3PP1b1/5N2/PPP2PPP/RNBQKB1R b KQkq e3 0 3',
			},
		},
		'rn1qkbnr/ppp1pppp/3p4/8/3PP1b1/5N2/PPP2PPP/RNBQKB1R b KQkq e3 0 3' => {
			eco => 'A41j',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. e4

			variation => N__('Neo-Old Indian: Wade Defence, 3. e4'),
			moves => {
				'g8f6' => 'rn1qkb1r/ppp1pppp/3p1n2/8/3PP1b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rn1qkbnr/ppp1pppp/3p4/8/4P1b1/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'B07b',
                        # TRANSLATORS: 1. e4 d6 2. Nf3 Bg4

			variation => N__('Pirc/Reti: Wade Defence'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/4b3/8/2p5/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A09s',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. e3 Be6

			variation => N__('Reti: Accepted, Keres Variation'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3P1b2/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D06r',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. cxd5

			variation => N__('QGD: 2... Bf5 3.cxd5'),
			moves => {
				'f5b1' => 'rn1qkbnr/ppp1pppp/8/3P4/3P4/8/PP2PPPP/RbBQKBNR w KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3P4/3P4/8/PP2PPPP/1RBQKBNR b Kkq - 0 4' => {
			eco => 'D06s',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. cxd5 Bxb1 4. Rxb1

			variation => N__('QGD: 2... Bf5 3.cxd5 Bxb1 4.Rxb1'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3P4/3P4/8/PP2PPPP/RbBQKBNR w KQkq - 0 4' => {
			eco => 'D06r',
                        # TRANSLATORS:

			variation => N__('QGD: 2... Bf5 3.cxd5'),
			moves => {
				'a1b1' => 'rn1qkbnr/ppp1pppp/8/3P4/3P4/8/PP2PPPP/1RBQKBNR b Kkq - 0 4',
				'd1a4' => 'rn1qkbnr/ppp1pppp/8/3P4/Q2P4/8/PP2PPPP/RbB1KBNR b KQkq - 1 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3P4/Q2P4/8/PP2PPPP/RbB1KBNR b KQkq - 1 4' => {
			eco => 'D06t',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. cxd5 Bxb1 4. Qa4+

			variation => N__('QGD: 2... Bf5 3.cxd5 Bxb1 4. Qa4+'),
			moves => {
				'c7c6' => 'rn1qkbnr/pp2pppp/2p5/3P4/Q2P4/8/PP2PPPP/RbB1KBNR w KQkq - 0 5',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 2 3' => {
			eco => 'A00q',
                        # TRANSLATORS: 1. b4 d5 2. Bb2 Bf5

			variation => N__('Polish: 1... d5 2. Bb2 Bf5'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/1Q6/PP2PPPP/RNB1KBNR b KQkq - 2 3' => {
			eco => 'D06l',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Qb3

			variation => N__('QGD: 2... Bf5 3. Qb3'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'D06m',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nc3

			variation => N__('QGD: 2... Bf5 3. Nc3'),
			moves => {
				'e7e6' => 'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'D06n',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nf3

			variation => N__('QGD: 2... Bf5 3. Nf3'),
			moves => {
				'e7e6' => 'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'D06l',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5

			variation => N__('QGD: 2... Bf5'),
			moves => {
				'b1c3' => 'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
				'c4d5' => 'rn1qkbnr/ppp1pppp/8/3P1b2/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'd1b3' => 'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/1Q6/PP2PPPP/RNB1KBNR b KQkq - 2 3',
				'g1f3' => 'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3' => {
			eco => 'D02d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bf5 3. Bf4

			variation => N__('Queen\'s Pawn: 2. Nf3 Bf5 3. Bf4'),
			moves => {
				'c7c6' => 'rn1qkbnr/pp2pppp/2p5/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
				'e7e6' => 'rn1qkbnr/ppp2ppp/4p3/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'D00n',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Bf5

			variation => N__('Queen\'s Pawn: Veresov Attack'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bf5 3. e3

			variation => N__('Queen\'s Pawn: 2. Nf3 Bf5 3. e3'),
			moves => {
				'c7c6' => 'rn1qkbnr/pp2pppp/2p5/3p1b2/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p1b2/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'D02d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bf5

			variation => N__('Queen\'s Pawn: 2. Nf3 Bf5'),
			moves => {
				'c1f4' => 'rn1qkbnr/ppp1pppp/8/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3',
				'e2e3' => 'rn1qkbnr/ppp1pppp/8/3p1b2/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p2B1/3P2b1/8/PPP1PPPP/RN1QKBNR w KQkq - 2 3' => {
			eco => 'D00f',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 Bg4

			variation => N__('Hodgson Attack: Welling Variation'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/2P3b1/8/PP1PPPBP/RNBQK1NR b KQkq c3 0 3' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2 Bxg4 3. c4

			variation => N__('Grob Gambit Accepted: Fritz Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/3P2b1/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'D02a',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bg4

			variation => N__('Queen\'s Pawn: 2. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/6b1/1P3N2/P1PPPPPP/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Bg4

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Bg4'),
			moves => {
				'c1b2' => 'rn1qkbnr/ppp1pppp/8/3p4/6b1/1P3N2/PBPPPPPP/RN1QKB1R b KQkq - 2 3',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/6b1/1P3N2/PBPPPPPP/RN1QKB1R b KQkq - 2 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Bg4 3. Bb2

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Bg4'),
			moves => {
				'b8d7' => 'r2qkbnr/pppnpppp/8/3p4/6b1/1P3N2/PBPPPPPP/RN1QKB1R w KQkq - 3 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/6b1/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Bg4

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'rn1qkbnr/ppp1pppp/8/3p4/6b1/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/6b1/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'b8d7' => 'r2qkbnr/pppnpppp/8/3p4/6b1/5NP1/PPPPPPBP/RNBQK2R w KQkq - 3 4',
			},
		},
		'rn1qkbnr/ppp1pppp/8/3p4/6b1/8/PPPPPPBP/RNBQK1NR w KQkq - 0 3' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2 Bxg4

			variation => N__('Grob Gambit Accepted'),
			moves => {
				'c2c4' => 'rn1qkbnr/ppp1pppp/8/3p4/2P3b1/8/PP1PPPBP/RNBQK1NR b KQkq c3 0 3',
			},
		},
		'rn1qkbnr/ppp1pppp/8/5b2/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3 Bf5

			variation => N__('Blackmar-Diemer: Zeller Defence'),
			moves => {
			},
		},
		'rn1qkbnr/ppp1pppp/8/8/2pP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D21c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Bg4

			variation => N__('QGA: 3. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1qkbnr/ppp2ppp/2b1p3/8/3PN3/3B1N2/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'C10t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7 5. Nf3 Bc6 6. Bd3

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3 Bc6 6. Bd3'),
			moves => {
				'b8d7' => 'r2qkbnr/pppn1ppp/2b1p3/8/3PN3/3B1N2/PPP2PPP/R1BQK2R w KQkq - 5 7',
			},
		},
		'rn1qkbnr/ppp2ppp/2b1p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'C10s',
                        # TRANSLATORS:

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3'),
			moves => {
				'f1d3' => 'rn1qkbnr/ppp2ppp/2b1p3/8/3PN3/3B1N2/PPP2PPP/R1BQK2R b KQkq - 4 6',
			},
		},
		'rn1qkbnr/ppp2ppp/3p4/4P3/2P3b1/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A41g',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 e5 4. dxe5

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 e5 4.dxe5'),
			moves => {
				'b8c6' => 'r2qkbnr/ppp2ppp/2np4/4P3/2P3b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rn1qkbnr/ppp2ppp/3p4/4p3/2P3b1/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 2 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 Bg4

			variation => N__('English: Smyslov Defence'),
			moves => {
			},
		},
		'rn1qkbnr/ppp2ppp/3p4/4p3/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A41g',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 e5 4. Nc3

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 e5 4. Nc3'),
			moves => {
				'b8c6' => 'r2qkbnr/ppp2ppp/2np4/4p3/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rn1qkbnr/ppp2ppp/3p4/4p3/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq e6 0 4' => {
			eco => 'A41g',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 Bg4 3. c4 e5

			variation => N__('Neo-Old Indian: Wade Defence, 3. c4 e5'),
			moves => {
				'b1c3' => 'rn1qkbnr/ppp2ppp/3p4/4p3/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'd4e5' => 'rn1qkbnr/ppp2ppp/3p4/4P3/2P3b1/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/1Q3N2/PP2PPPP/RNB1KB1R b KQkq - 1 4' => {
			eco => 'D06o',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nf3 e6 4. Qb3

			variation => N__('QGD: 2... Bf5 3. Nf3 e6 4. Qb3'),
			moves => {
			},
		},
		'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D06p',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nf3 e6 4. Nc3

			variation => N__('QGD: 2... Bf5 3. Nf3 e6 4. Nc3'),
			moves => {
				'g8f6' => 'rn1qkb1r/ppp2ppp/4pn2/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D06m',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nc3 e6

			variation => N__('QGD: 2... Bf5 3. Nc3 e6'),
			moves => {
			},
		},
		'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D06n',
                        # TRANSLATORS: 1. d4 d5 2. c4 Bf5 3. Nf3 e6

			variation => N__('QGD: 2... Bf5 3. Nf3 e6'),
			moves => {
				'b1c3' => 'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'd1b3' => 'rn1qkbnr/ppp2ppp/4p3/3p1b2/2PP4/1Q3N2/PP2PPPP/RNB1KB1R b KQkq - 1 4',
			},
		},
		'rn1qkbnr/ppp2ppp/4p3/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'D02d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Bf5 3. Bf4 e6

			variation => N__('Queen\'s Pawn: 2. Nf3 Bf5 3. Bf4 e6'),
			moves => {
			},
		},
		'rn1qkbnr/pppb1ppp/3p4/4P3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 d6 3. dxe5 Bd7

			variation => N__('Centre Game: Maroczy Defence, Philidor Gambit'),
			moves => {
			},
		},
		'rn1qkbnr/pppb1ppp/4p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'C10s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7 5. Nf3

			variation => N__('French: Rubinstein, 4... Bd7 5. Nf3'),
			moves => {
				'd7c6' => 'rn1qkbnr/ppp2ppp/2b1p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'rn1qkbnr/pppb1ppp/4p3/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C10r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Bd7

			variation => N__('French: Rubinstein, 4... Bd7'),
			moves => {
				'g1f3' => 'rn1qkbnr/pppb1ppp/4p3/8/3PN3/5N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rn1qkbnr/pppbpppp/8/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3 Bd7

			variation => N__('Blackmar-Diemer: Grosshans Defence'),
			moves => {
			},
		},
		'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PB2/2N5/1P1NQPPP/R4RK1 b - - 0 13' => {
			eco => 'A75',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Re8'),
			moves => {
				'f6h5' => 'rn1qr1k1/1p3pbp/p2p2p1/2pP3n/P3PB2/2N5/1P1NQPPP/R4RK1 w - - 1 14',
			},
		},
		'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PB2/2N5/1P1NbPPP/R2Q1RK1 w - - 0 13' => {
			eco => 'A75',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Re8'),
			moves => {
				'd1e2' => 'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PB2/2N5/1P1NQPPP/R4RK1 b - - 0 13',
			},
		},
		'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PBb1/2N2N2/1P2BPPP/R2Q1RK1 w - - 3 12' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Re8

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Re8'),
			moves => {
				'f3d2' => 'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PBb1/2N5/1P1NBPPP/R2Q1RK1 b - - 4 12',
			},
		},
		'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PBb1/2N5/1P1NBPPP/R2Q1RK1 b - - 4 12' => {
			eco => 'A75',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Re8'),
			moves => {
				'g4e2' => 'rn1qr1k1/1p3pbp/p2p1np1/2pP4/P3PB2/2N5/1P1NbPPP/R2Q1RK1 w - - 0 13',
			},
		},
		'rn1qr1k1/1p3pbp/p2p2p1/2pP3n/P3PB2/2N5/1P1NQPPP/R4RK1 w - - 1 14' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Re8 12. Nd2 Bxe2 13. Qxe2 Nh5

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Re8 12. Nd2 Bxe2 13.Qxe2 Nh5'),
			moves => {
			},
		},
		'rn1qr1k1/pp3pbp/3p1np1/2pP2B1/2P3b1/2N2N2/PP1QBPPP/R3K2R w KQ - 3 11' => {
			eco => 'E75j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2 exd5 9. exd5 Re8 10. Nf3 Bg4

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2 exd5 9.exd5 Re8 10. Nf3 Bg4'),
			moves => {
			},
		},
		'rn1qr1k1/pp3pbp/3p1np1/2pP4/4P1b1/2N2N2/PPQ1BPPP/R1B2RK1 w - - 8 11' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Qc2 Bg4

			variation => N__('Benoni: Classical, Main Line, 10. Qc2 Bg4'),
			moves => {
			},
		},
		'rn1qrbk1/1bp2ppp/p2p1n2/1p1Pp3/4P3/1BP2N1P/PP1N1PP1/R1BQR1K1 w - - 1 13' => {
			eco => 'C92n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Bb7 10. d4 Re8 11. Nbd2 Bf8 12. d5 Nb8

			variation => N__('Spanish: Closed, Zaitsev, 11.Nbd2 Bf8 12. d5 Nb8'),
			moves => {
			},
		},
		'rn2k2r/1p1bbppp/p2ppn2/q3P1B1/3N1P2/2N2Q2/PPP3PP/2KR1B1R b kq - 0 10' => {
			eco => 'B98e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qa5 9. O-O-O Bd7 10. e5

			variation => N__('Sicilian: Najdorf, Danner, 9.O-O-O Bd7 10. e5'),
			moves => {
			},
		},
		'rn2k2r/1p1bbppp/p2ppn2/q5B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq - 5 10' => {
			eco => 'B98d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qa5 9. O-O-O Bd7

			variation => N__('Sicilian: Najdorf, Danner, 9.O-O-O Bd7'),
			moves => {
				'e4e5' => 'rn2k2r/1p1bbppp/p2ppn2/q3P1B1/3N1P2/2N2Q2/PPP3PP/2KR1B1R b kq - 0 10',
			},
		},
		'rn2k2r/pbppqppp/1p2pn2/8/1bPP4/5NP1/PP1BPPBP/RN1Q1RK1 b kq - 6 7' => {
			eco => 'E16h',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Capablanca, Nimzowitsch Variation'),
			moves => {
				'b4d2' => 'rn2k2r/pbppqppp/1p2pn2/8/2PP4/5NP1/PP1bPPBP/RN1Q1RK1 w kq - 0 8',
			},
		},
		'rn2k2r/pbppqppp/1p2pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R w KQkq - 5 7' => {
			eco => 'E16h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Qe7

			variation => N__('Queen\'s Indian: Capablanca, Nimzowitsch Variation'),
			moves => {
				'e1g1' => 'rn2k2r/pbppqppp/1p2pn2/8/1bPP4/5NP1/PP1BPPBP/RN1Q1RK1 b kq - 6 7',
			},
		},
		'rn2k2r/pbppqppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN3RK1 b kq - 0 8' => {
			eco => 'E16i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Bb4+ 6. Bd2 Qe7 7. O-O Bxd2 8. Qxd2

			variation => N__('Queen\'s Indian: Capablanca, Nimzowitsch, 7.O-O Bxd2 8.Qxd2'),
			moves => {
			},
		},
		'rn2k2r/pbppqppp/1p2pn2/8/2PP4/5NP1/PP1bPPBP/RN1Q1RK1 w kq - 0 8' => {
			eco => 'E16h',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: Capablanca, Nimzowitsch Variation'),
			moves => {
				'd1d2' => 'rn2k2r/pbppqppp/1p2pn2/8/2PP4/5NP1/PP1QPPBP/RN3RK1 b kq - 0 8',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/3qN3/P1p5/8/1P1KQ1PP/R4B1R w kq - 1 14' => {
			eco => 'D17q',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Main Line'),
			moves => {
				'd2c2' => 'rn2k2r/pp3ppp/2p1p3/3qN3/P1p5/8/1PK1Q1PP/R4B1R b kq - 2 14',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/3qN3/P1p5/8/1PK1Q1PP/R4B1R b kq - 2 14' => {
			eco => 'D17q',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Main Line'),
			moves => {
				'b8a6' => 'r3k2r/pp3ppp/n1p1p3/3qN3/P1p5/8/1PK1Q1PP/R4B1R w kq - 3 15',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/4N3/P1p1q3/8/1P1KQ1PP/R4B1R b kq - 0 13' => {
			eco => 'D17q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Ne5 e6 7. f3 Bb4 8. e4 Bxe4 9. fxe4 Nxe4 10. Bd2 Qxd4 11. Nxe4 Qxe4+ 12. Qe2 Bxd2+ 13. Kxd2

			variation => N__('Slav: Central, Main Line'),
			moves => {
				'e4d5' => 'rn2k2r/pp3ppp/2p1p3/3qN3/P1p5/8/1P1KQ1PP/R4B1R w kq - 1 14',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/4N3/P1p1q3/8/1P1bQ1PP/R3KB1R w KQkq - 0 13' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'e1d2' => 'rn2k2r/pp3ppp/2p1p3/4N3/P1p1q3/8/1P1KQ1PP/R4B1R b kq - 0 13',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/4N3/Pbp1q3/8/1P1B2PP/R2QKB1R w KQkq - 0 12' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'd1e2' => 'rn2k2r/pp3ppp/2p1p3/4N3/Pbp1q3/8/1P1BQ1PP/R3KB1R b KQkq - 1 12',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/4N3/Pbp1q3/8/1P1BQ1PP/R3KB1R b KQkq - 1 12' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'b4d2' => 'rn2k2r/pp3ppp/2p1p3/4N3/P1p1q3/8/1P1bQ1PP/R3KB1R w KQkq - 0 13',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/4N3/PbpqN3/8/1P1B2PP/R2QKB1R b KQkq - 0 11' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'd4e4' => 'rn2k2r/pp3ppp/2p1p3/4N3/Pbp1q3/8/1P1B2PP/R2QKB1R w KQkq - 0 12',
			},
		},
		'rn2k2r/pp3ppp/2p1p3/4N3/Pbpqn3/2N5/1P1B2PP/R2QKB1R w KQkq - 0 11' => {
			eco => 'D17p',
                        # TRANSLATORS:

			variation => N__('Slav: Central, Piece-Sac Line'),
			moves => {
				'c3e4' => 'rn2k2r/pp3ppp/2p1p3/4N3/PbpqN3/8/1P1B2PP/R2QKB1R b KQkq - 0 11',
			},
		},
		'rn2k2r/pp3ppp/2p1pn2/q4b2/1bBP4/2N2N2/PPPBQPPP/2KR3R b kq - 3 9' => {
			eco => 'B01q',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2 e6 8. Qe2 Bb4 9. O-O-O

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 8. Qe2 Bb4 9.O-O-O'),
			moves => {
			},
		},
		'rn2k2r/pp3ppp/2p1pn2/q4b2/1bBP4/2N2N2/PPPBQPPP/R3K2R w KQkq - 2 9' => {
			eco => 'B01q',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2 e6 8. Qe2 Bb4

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 8. Qe2 Bb4'),
			moves => {
				'a2a3' => 'rn2k2r/pp3ppp/2p1pn2/q4b2/1bBP4/P1N2N2/1PPBQPPP/R3K2R b KQkq - 0 9',
				'e1c1' => 'rn2k2r/pp3ppp/2p1pn2/q4b2/1bBP4/2N2N2/PPPBQPPP/2KR3R b kq - 3 9',
			},
		},
		'rn2k2r/pp3ppp/2p1pn2/q4b2/1bBP4/P1N2N2/1PPBQPPP/R3K2R b KQkq - 0 9' => {
			eco => 'B01q',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2 e6 8. Qe2 Bb4 9. a3

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 8. Qe2 Bb4 9. a3'),
			moves => {
			},
		},
		'rn2k2r/ppqb1pbp/3p1np1/2pP4/4P3/1QN2N2/PP3PPP/R1B1KB1R w KQkq - 6 10' => {
			eco => 'A70f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Qa4+ Bd7 9. Qb3 Qc7

			variation => N__('Benoni: Classical, 8. Qa4+ Bd7 9. Qb3 Qc7'),
			moves => {
			},
		},
		'rn2kb1r/1b1n1ppp/p3p3/1p2q1B1/3N2Q1/2N5/PPP3PP/2KR1B1R w kq - 0 13' => {
			eco => 'B96w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Qe2 Nfd7 11. O-O-O Bb7 12. Qg4 Qxe5

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2 Nfd7 11.O-O-O Bb7 12. Qg4 Qxe5'),
			moves => {
			},
		},
		'rn2kb1r/1bp1pppp/p7/1p1qP3/P1pP4/2P2NP1/5P1P/R1BQKB1R w KQkq - 1 10' => {
			eco => 'D24t',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 Nxc3 8. bxc3 Qd5 9. g3 Bb7

			variation => N__('QGA: Bogoljubow, 7. a4 Nxc3'),
			moves => {
			},
		},
		'rn2kb1r/1bqn1ppp/p3p3/1p2P1B1/3N2Q1/2N5/PPP3PP/2KR1B1R b kq - 6 12' => {
			eco => 'B96w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Qe2 Nfd7 11. O-O-O Bb7 12. Qg4

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2 Nfd7 11.O-O-O Bb7 12. Qg4'),
			moves => {
				'c7e5' => 'rn2kb1r/1b1n1ppp/p3p3/1p2q1B1/3N2Q1/2N5/PPP3PP/2KR1B1R w kq - 0 13',
			},
		},
		'rn2kb1r/1bqn1ppp/p3p3/1p2P1B1/3N4/2N5/PPP1Q1PP/2KR1B1R w kq - 5 12' => {
			eco => 'B96v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Qe2 Nfd7 11. O-O-O Bb7

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2 Nfd7 11.O-O-O Bb7'),
			moves => {
				'e2g4' => 'rn2kb1r/1bqn1ppp/p3p3/1p2P1B1/3N2Q1/2N5/PPP3PP/2KR1B1R b kq - 6 12',
			},
		},
		'rn2kb1r/4pp1p/b2p1np1/q1pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 9' => {
			eco => 'A58j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 Bxa6 6. Nc3 d6 7. Nf3 g6 8. Nd2 Qa5

			variation => N__('Benko Gambit: Accepted, 7. Nf3 g6 8. Nd2 Qa5'),
			moves => {
			},
		},
		'rn2kb1r/pp2pppp/2p2n2/q3Nb2/3P2P1/2N5/PPP2P1P/R1BQKB1R b KQkq g3 0 7' => {
			eco => 'B01l',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 Bf5 6. Ne5 c6 7. g4

			variation => N__('Scandinavian: Grünfeld Variation'),
			moves => {
			},
		},
		'rn2kb1r/pp2pppp/2p2n2/q3Nb2/3P4/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B01l',
                        # TRANSLATORS:

			variation => N__('Scandinavian: 2...Qxd5, 5. Nf3 Bf5'),
			moves => {
				'g2g4' => 'rn2kb1r/pp2pppp/2p2n2/q3Nb2/3P2P1/2N5/PPP2P1P/R1BQKB1R b KQkq g3 0 7',
			},
		},
		'rn2kb1r/pp2pppp/2p2n2/q4b2/2BP4/2N2N2/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B01o',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 6. Bc4 Bf5'),
			moves => {
				'c1d2' => 'rn2kb1r/pp2pppp/2p2n2/q4b2/2BP4/2N2N2/PPPB1PPP/R2QK2R b KQkq - 3 7',
			},
		},
		'rn2kb1r/pp2pppp/2p2n2/q4b2/2BP4/2N2N2/PPPB1PPP/R2QK2R b KQkq - 3 7' => {
			eco => 'B01p',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 6. Bc4 Bf5 7. Bd2'),
			moves => {
				'e7e6' => 'rn2kb1r/pp3ppp/2p1pn2/q4b2/2BP4/2N2N2/PPPB1PPP/R2QK2R w KQkq - 0 8',
			},
		},
		'rn2kb1r/pp2pppp/2p2n2/q7/2BP2b1/2N2N2/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B01n',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bg4

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 6. Bc4 Bg4'),
			moves => {
			},
		},
		'rn2kb1r/pp2pppp/5n2/2pq4/3P2b1/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6' => {
			eco => 'B22k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3 Bg4'),
			moves => {
				'e7e6' => 'rn2kb1r/pp3ppp/4pn2/2pq4/3P2b1/2P2N2/PP2BPPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rn2kb1r/pp2pppp/5n2/2pq4/3P2b1/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'B22k',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nf6 5. Nf3 Bg4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3 Bg4'),
			moves => {
				'f1e2' => 'rn2kb1r/pp2pppp/5n2/2pq4/3P2b1/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6',
			},
		},
		'rn2kb1r/pp3ppp/2p1pn2/q2N1b2/2BP4/5N2/PPPB1PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'B01p',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2 e6 8. Nd5

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 8. Nd5'),
			moves => {
			},
		},
		'rn2kb1r/pp3ppp/2p1pn2/q4b2/2BP4/2N2N2/PPPB1PPP/R2QK2R w KQkq - 0 8' => {
			eco => 'B01p',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2 e6

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 6. Bc4 Bf5 7. Bd2 e6'),
			moves => {
				'c3d5' => 'rn2kb1r/pp3ppp/2p1pn2/q2N1b2/2BP4/5N2/PPPB1PPP/R2QK2R b KQkq - 1 8',
				'd1e2' => 'rn2kb1r/pp3ppp/2p1pn2/q4b2/2BP4/2N2N2/PPPBQPPP/R3K2R b KQkq - 1 8',
			},
		},
		'rn2kb1r/pp3ppp/2p1pn2/q4b2/2BP4/2N2N2/PPPBQPPP/R3K2R b KQkq - 1 8' => {
			eco => 'B01q',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4 Bf5 7. Bd2 e6 8. Qe2

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 8. Qe2'),
			moves => {
				'f8b4' => 'rn2k2r/pp3ppp/2p1pn2/q4b2/1bBP4/2N2N2/PPPBQPPP/R3K2R w KQkq - 2 9',
			},
		},
		'rn2kb1r/pp3ppp/4pn2/2pq4/3P2b1/2P2N2/PP2BPPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'B22l',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nf6 5. Nf3 Bg4 6. Be2 e6

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3 Bg4 6. Be2 e6'),
			moves => {
			},
		},
		'rn2kb1r/ppp1pppp/5n2/3q4/3P2b1/5N2/PPP1BPPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Nf3 Qxd5 5. Be2

			variation => N__('Scandinavian: Portuguese, 4. Nf3 Qxd5 5. Be2'),
			moves => {
				'b8c6' => 'r3kb1r/ppp1pppp/2n2n2/3q4/3P2b1/5N2/PPP1BPPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rn2kb1r/ppp1pppp/5n2/3q4/3P2b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Bg4 4. Nf3 Qxd5

			variation => N__('Scandinavian: Portuguese, 4. Nf3 Qxd5'),
			moves => {
				'f1e2' => 'rn2kb1r/ppp1pppp/5n2/3q4/3P2b1/5N2/PPP1BPPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rn2kb1r/ppp1pppp/5n2/q3Nb2/3P4/2N5/PPP2PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'B01l',
                        # TRANSLATORS:

			variation => N__('Scandinavian: 2...Qxd5, 5. Nf3 Bf5'),
			moves => {
				'c7c6' => 'rn2kb1r/pp2pppp/2p2n2/q3Nb2/3P4/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rn2kb1r/ppp1pppp/5n2/q4b2/2BP4/2N2N2/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B01l',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 Bf5 6. Bc4

			variation => N__('Scandinavian: 2...Qxd5, 5. Nf3 Bf5 6. Bc4'),
			moves => {
			},
		},
		'rn2kb1r/ppp1pppp/5n2/q4b2/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B01l',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 Bf5

			variation => N__('Scandinavian: 2...Qxd5, 5. Nf3 Bf5'),
			moves => {
				'f1c4' => 'rn2kb1r/ppp1pppp/5n2/q4b2/2BP4/2N2N2/PPP2PPP/R1BQK2R b KQkq - 4 6',
				'f3e5' => 'rn2kb1r/ppp1pppp/5n2/q3Nb2/3P4/2N5/PPP2PPP/R1BQKB1R b KQkq - 4 6',
			},
		},
		'rn2kb1r/ppp1pppp/5n2/q7/3P2b1/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B01k',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 Bg4 6. h3

			variation => N__('Scandinavian: Lasker Variation'),
			moves => {
			},
		},
		'rn2kb1r/ppp1pppp/5n2/q7/3P2b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B01j',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 Bg4

			variation => N__('Scandinavian: 2...Qxd5, 5. Nf3 Bg4'),
			moves => {
				'h2h3' => 'rn2kb1r/ppp1pppp/5n2/q7/3P2b1/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rn2kb1r/ppq1pp1p/2p2p2/5b2/3P4/2P2N2/PP3PPP/R1BQKB1R w KQkq - 3 8' => {
			eco => 'B16m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. c3 Bf5 7. Nf3 Qc7

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. c3 Bf5 7. Nf3 Qc7'),
			moves => {
			},
		},
		'rn2kb1r/ppq1pppp/5n2/3p1b2/3P4/1Q2PN2/PP3PPP/RNB1KB1R w KQkq - 2 7' => {
			eco => 'D12e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 Bf5 5. cxd5 cxd5 6. Qb3 Qc7

			variation => N__('Slav: 4. e3 Bf5 5.cxd5 cxd5 6. Qb3 Qc7'),
			moves => {
			},
		},
		'rn2kbnr/pp1b1ppp/1q2p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6' => {
			eco => 'C02j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Qb6 5. Nf3 Bd7 6. Be2

			variation => N__('French: Advance, Wade, 6. Be2'),
			moves => {
			},
		},
		'rn2kbnr/pp1b1ppp/1q2p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6' => {
			eco => 'C02i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Qb6 5. Nf3 Bd7

			variation => N__('French: Advance, Wade'),
			moves => {
				'f1e2' => 'rn2kbnr/pp1b1ppp/1q2p3/2ppP3/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 4 6',
			},
		},
		'rn2kbnr/pp1qpppp/3p4/2p5/2P1P3/5N2/PP1P1PPP/RNBQK2R b KQkq c3 0 5' => {
			eco => 'B52g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. c4

			variation => N__('Sicilian: 3. Bb5+, Sokolsky Variation'),
			moves => {
				'b8c6' => 'r3kbnr/pp1qpppp/2np4/2p5/2P1P3/5N2/PP1P1PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rn2kbnr/pp1qpppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'B52n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7 5. O-O

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7 5.O-O'),
			moves => {
				'b8c6' => 'r3kbnr/pp1qpppp/2np4/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'rn2kbnr/pp1qpppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'B52f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+ Bd7 4. Bxd7+ Qxd7

			variation => N__('Sicilian: 3. Bb5+ Bd7 4.Bxd7+ Qxd7'),
			moves => {
				'c2c4' => 'rn2kbnr/pp1qpppp/3p4/2p5/2P1P3/5N2/PP1P1PPP/RNBQK2R b KQkq c3 0 5',
				'e1g1' => 'rn2kbnr/pp1qpppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'rn2kbnr/pp2pppp/1qp5/3pPb2/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 3 5' => {
			eco => 'B12v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 Bf5 4. Nc3 Qb6

			variation => N__('Caro-Kann: Advance, 4. Nc3 Qb6'),
			moves => {
			},
		},
		'rn2kbnr/ppp2ppp/8/q3p3/3P2b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'B01g',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 e5 5. Nf3 Bg4

			variation => N__('Scandinavian: Anderssen Counterattack, Collijn Variation'),
			moves => {
			},
		},
		'rn3rk1/1p2qpbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2Q1RK1 w - - 1 13' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12. Bxf3 Qe7

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12.Bxf3 Qe7'),
			moves => {
				'f1e1' => 'rn3rk1/1p2qpbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2QR1K1 b - - 2 13',
			},
		},
		'rn3rk1/1p2qpbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2QR1K1 b - - 2 13' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12. Bxf3 Qe7 13. Re1

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Bxf3 12.Bxf3 Qe7 13. Re1'),
			moves => {
				'b8d7' => 'r4rk1/1p1nqpbp/p2p1np1/2pP4/P3PB2/2N2B2/1P3PPP/R2QR1K1 w - - 3 14',
			},
		},
		'rn3rk1/1p2qpbp/p2p1np1/2pP4/P3PBb1/2N2N2/1P2BPPP/R2Q1RK1 w - - 3 12' => {
			eco => 'A75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4 Bg4 11. Bf4 Qe7

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4 Bg4 11. Bf4 Qe7'),
			moves => {
			},
		},
		'rn3rk1/p1p1qpp1/1p2b2p/3p4/3P4/3BPN2/PP3PPP/2RQK2R b K - 3 12' => {
			eco => 'D59m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Rc1 Be6 12. Bd3

			variation => N__('QGD: Tartakower, Main Line, 12. Bd3'),
			moves => {
			},
		},
		'rn3rk1/p1p1qpp1/1p2b2p/3p4/3P4/4PN2/PP3PPP/2RQKB1R w K - 2 12' => {
			eco => 'D59l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Rc1 Be6

			variation => N__('QGD: Tartakower, Main Line, 11. Rc1 Be6'),
			moves => {
				'd1a4' => 'rn3rk1/p1p1qpp1/1p2b2p/3p4/Q2P4/4PN2/PP3PPP/2R1KB1R b K - 3 12',
				'f1d3' => 'rn3rk1/p1p1qpp1/1p2b2p/3p4/3P4/3BPN2/PP3PPP/2RQK2R b K - 3 12',
			},
		},
		'rn3rk1/p1p1qpp1/1p2b2p/3p4/Q2P4/4PN2/PP3PPP/2R1KB1R b K - 3 12' => {
			eco => 'D59n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Rc1 Be6 12. Qa4

			variation => N__('QGD: Tartakower, Main Line, 12. Qa4'),
			moves => {
				'c7c5' => 'rn3rk1/p3qpp1/1p2b2p/2pp4/Q2P4/4PN2/PP3PPP/2R1KB1R w K c6 0 13',
			},
		},
		'rn3rk1/p1p1qpp1/1p2p2p/3b4/3P4/4PN2/PP3PPP/2RQKB1R w K - 0 12' => {
			eco => 'D59f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Rc1 Bb7 11. Nxd5 Bxd5

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10. Rc1 Bb7 11.Nxd5 Bxd5'),
			moves => {
			},
		},
		'rn3rk1/p1pqbppp/1p2p3/3p4/Q1PP4/6P1/PP2PPNP/R1B2RK1 w - - 2 12' => {
			eco => 'E18w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Nxe4 Bxe4 9. Ne1 Bxg2 10. Nxg2 d5 11. Qa4 Qd7

			variation => N__('Queen\'s Indian: 7. Nc3 Ne4 8.Nxe4 Bxe4 9. Ne1 Bxg2 10.Nxg2 d5 11. Qa4 Qd7'),
			moves => {
			},
		},
		'rn3rk1/p3qpp1/1p2b2p/2pp4/Q2P4/4PN2/PP3PPP/2R1KB1R w K c6 0 13' => {
			eco => 'D59n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Rc1 Be6 12. Qa4 c5

			variation => N__('QGD: Tartakower, Main Line, 12. Qa4 c5'),
			moves => {
			},
		},
		'rn3rk1/pb2qppp/1p1ppb2/2p5/2PP4/1PQ2NP1/PB2PPBP/R2R2K1 w - - 3 13' => {
			eco => 'E19p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Be7 6. O-O O-O 7. Nc3 Ne4 8. Qc2 Nxc3 9. Qxc3 c5 10. Rd1 d6 11. b3 Bf6 12. Bb2 Qe7

			variation => N__('Queen\'s Indian: Old Main Line, 9.Qxc3 c5 10. Rd1 d6 11. b3 Bf6 12. Bb2 Qe7'),
			moves => {
			},
		},
		'rn3rk1/pbp1qpp1/1p2p2p/3N4/3P4/4PN2/PP3PPP/2RQKB1R b K - 0 11' => {
			eco => 'D59f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Rc1 Bb7 11. Nxd5

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10. Rc1 Bb7 11.Nxd5'),
			moves => {
				'b7d5' => 'rn3rk1/p1p1qpp1/1p2p2p/3b4/3P4/4PN2/PP3PPP/2RQKB1R w K - 0 12',
			},
		},
		'rn3rk1/pbp1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP2BPPP/2RQK2R b K - 3 11' => {
			eco => 'D59h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Rc1 Bb7 11. Be2

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10. Rc1 Bb7 11. Be2'),
			moves => {
			},
		},
		'rn3rk1/pbp1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 2 11' => {
			eco => 'D59e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Rc1 Bb7

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10. Rc1 Bb7'),
			moves => {
				'c3d5' => 'rn3rk1/pbp1qpp1/1p2p2p/3N4/3P4/4PN2/PP3PPP/2RQKB1R b K - 0 11',
				'f1d3' => 'rn3rk1/pbp1qpp1/1p2p2p/3n4/3P4/2NBPN2/PP3PPP/2RQK2R b K - 3 11',
				'f1e2' => 'rn3rk1/pbp1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP2BPPP/2RQK2R b K - 3 11',
			},
		},
		'rn3rk1/pbp1qpp1/1p2p2p/3n4/3P4/2NBPN2/PP3PPP/2RQK2R b K - 3 11' => {
			eco => 'D59g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Rc1 Bb7 11. Bd3

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10. Rc1 Bb7 11. Bd3'),
			moves => {
			},
		},
		'rn3rk1/pp2ppbp/2pp1np1/q7/2PPP1b1/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E62k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O Qa5 8. e4 Bg4

			variation => N__('King\'s Indian: Fianchetto, Kavalek/Bronstein, 8. e4 Bg4'),
			moves => {
			},
		},
		'rn3rk1/pp2ppbp/3p1np1/2q5/2P1P1b1/2N2N2/PP1BBPPP/R2QK2R w KQ - 2 10' => {
			eco => 'E74c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. dxc5 Qa5 8. Bd2 Qxc5 9. Nf3 Bg4

			variation => N__('King\'s Indian: Averbakh, 6... c5 7.dxc5 Qa5 8. Bd2 Qxc5 9. Nf3 Bg4'),
			moves => {
			},
		},
		'rn3rk1/pp2ppbp/6p1/6B1/3PP1b1/5N2/q3BPPP/1R1Q1RK1 b - - 3 13' => {
			eco => 'D85s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4 10. cxd4 Qa5+ 11. Bd2 Qxa2 12. O-O Bg4 13. Bg5

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1, 10... Qa5+ 11. Bd2 Qxa2 12.O-O Bg4 13. Bg5'),
			moves => {
			},
		},
		'rn3rk1/pp2ppbp/6p1/8/3PP1b1/5N2/q2BBPPP/1R1Q1RK1 w - - 2 13' => {
			eco => 'D85s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4 10. cxd4 Qa5+ 11. Bd2 Qxa2 12. O-O Bg4

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1, 10... Qa5+ 11. Bd2 Qxa2 12.O-O Bg4'),
			moves => {
				'd2g5' => 'rn3rk1/pp2ppbp/6p1/6B1/3PP1b1/5N2/q3BPPP/1R1Q1RK1 b - - 3 13',
			},
		},
		'rn3rk1/ppB1ppbp/6p1/3q1b2/3P4/4P3/PP3PPP/R2QKBNR w KQ - 1 9' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 Bf5

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... Bf5'),
			moves => {
			},
		},
		'rnNq3r/pp1nkpbp/6p1/2pPp3/5P2/8/PP4PP/R1BQKBNR b KQ - 0 11' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7 9. Nb5 dxe5 10. Nd6+ Ke7 11. Nxc8+

			variation => N__('Benoni: Mikenas Attack, 9. Nb5 dxe5 10. Nd6+ Ke7 11.Nxc8+'),
			moves => {
				'd8c8' => 'rnq4r/pp1nkpbp/6p1/2pPp3/5P2/8/PP4PP/R1BQKBNR w KQ - 0 12',
			},
		},
		'rnb1k1nr/p1pq1ppp/1p2p3/3pP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 1 7' => {
			eco => 'C16i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Qd7 5. a3 Bxc3+ 6. bxc3 b6 7. Qg4

			variation => N__('French: Winawer, Petrosian, 5. a3 Bxc3+ 6.bxc3 b6 7. Qg4'),
			moves => {
			},
		},
		'rnb1k1nr/p1pq1ppp/1p2p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C16i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Qd7 5. a3 Bxc3+ 6. bxc3 b6

			variation => N__('French: Winawer, Petrosian, 5. a3 Bxc3+ 6.bxc3 b6'),
			moves => {
				'd1g4' => 'rnb1k1nr/p1pq1ppp/1p2p3/3pP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 1 7',
			},
		},
		'rnb1k1nr/p3qppp/2p1p3/1p6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'D31s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4 Bb4 6. e3 b5 7. Bd2 Qe7

			variation => N__('Semi-Slav: Noteboom, Koomen Variation'),
			moves => {
			},
		},
		'rnb1k1nr/p4ppp/1qp1p3/1p6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'D31s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4 Bb4 6. e3 b5 7. Bd2 Qb6

			variation => N__('Semi-Slav: Noteboom, Junge Variation'),
			moves => {
			},
		},
		'rnb1k1nr/pp1pppbp/6p1/2q5/4P3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B27r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. dxc5 Qa5+ 5. c3 Qxc5

			variation => N__('Sicilian: Accelerated Pterodactyl, 4.dxc5 Qa5+ 5. c3'),
			moves => {
			},
		},
		'rnb1k1nr/pp1pppbp/6p1/q1P5/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B27p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. dxc5 Qa5+ 5. Nc3

			variation => N__('Sicilian: Accelerated Pterodactyl, 4.dxc5 Qa5+ 5. Nc3'),
			moves => {
			},
		},
		'rnb1k1nr/pp1pppbp/6p1/q1P5/4P3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B27q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. dxc5 Qa5+ 5. c3

			variation => N__('Sicilian: Accelerated Pterodactyl, 4.dxc5 Qa5+ 5. c3'),
			moves => {
				'a5c5' => 'rnb1k1nr/pp1pppbp/6p1/2q5/4P3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnb1k1nr/pp1pppbp/6p1/q1P5/4P3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B27o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. dxc5 Qa5+

			variation => N__('Sicilian: Accelerated Pterodactyl, 4.dxc5 Qa5+'),
			moves => {
				'b1c3' => 'rnb1k1nr/pp1pppbp/6p1/q1P5/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
				'c2c3' => 'rnb1k1nr/pp1pppbp/6p1/q1P5/4P3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnb1k1nr/pp1pppbp/6p1/q1p5/2PPP3/5N2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B27t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. c4 Qa5+

			variation => N__('Sicilian: Accelerated Pterodactyl, 4. c4 Qa5+'),
			moves => {
			},
		},
		'rnb1k1nr/pp2ppbp/1qpp2p1/6B1/3PP3/2N2N2/PPP2PPP/R2QKB1R w KQkq - 2 6' => {
			eco => 'B06r',
                        # TRANSLATORS:

			variation => N__('Modern: Two Knights, Suttles Variation'),
			moves => {
				'd1d2' => 'rnb1k1nr/pp2ppbp/1qpp2p1/6B1/3PP3/2N2N2/PPPQ1PPP/R3KB1R b KQkq - 3 6',
			},
		},
		'rnb1k1nr/pp2ppbp/1qpp2p1/6B1/3PP3/2N2N2/PPPQ1PPP/R3KB1R b KQkq - 3 6' => {
			eco => 'B06r',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Nf3 c6 5. Bg5 Qb6 6. Qd2

			variation => N__('Modern: Two Knights, Suttles, Tal Gambit'),
			moves => {
			},
		},
		'rnb1k1nr/pp2ppbp/3p2p1/q1p5/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A42c',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 c5 5. Nf3 Qa5

			variation => N__('Modern: c4 Pterodactyl'),
			moves => {
			},
		},
		'rnb1k1nr/pp3ppp/2p1p3/8/1BP1q3/8/PP2BPPP/R2QK1NR b KQkq - 1 8' => {
			eco => 'D31y',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Bb4+ 6. Bd2 Qxd4 7. Bxb4 Qxe4+ 8. Be2

			variation => N__('Semi-Slav: Marshall Gambit, 8. Be2'),
			moves => {
				'b8a6' => 'r1b1k1nr/pp3ppp/n1p1p3/8/1BP1q3/8/PP2BPPP/R2QK1NR w KQkq - 2 9',
			},
		},
		'rnb1k1nr/pp3ppp/2p1p3/8/1BP1q3/8/PP2NPPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'D31x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Bb4+ 6. Bd2 Qxd4 7. Bxb4 Qxe4+ 8. Ne2

			variation => N__('Semi-Slav: Marshall Gambit, 8. Ne2'),
			moves => {
			},
		},
		'rnb1k1nr/pp3ppp/2p1p3/8/1BP1q3/8/PP3PPP/R2QKBNR w KQkq - 0 8' => {
			eco => 'D31x',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Bb4+ 6. Bd2'),
			moves => {
				'f1e2' => 'rnb1k1nr/pp3ppp/2p1p3/8/1BP1q3/8/PP2BPPP/R2QK1NR b KQkq - 1 8',
				'g1e2' => 'rnb1k1nr/pp3ppp/2p1p3/8/1BP1q3/8/PP2NPPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnb1k1nr/pp3ppp/2p1p3/8/1BPqN3/8/PP3PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'D31x',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Bb4+ 6. Bd2'),
			moves => {
				'd4e4' => 'rnb1k1nr/pp3ppp/2p1p3/8/1BP1q3/8/PP3PPP/R2QKBNR w KQkq - 0 8',
			},
		},
		'rnb1k1nr/pp3ppp/2p1p3/8/1bPqN3/8/PP1B1PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'D31x',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Bb4+ 6. Bd2'),
			moves => {
				'd2b4' => 'rnb1k1nr/pp3ppp/2p1p3/8/1BPqN3/8/PP3PPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnb1k1nr/pp3ppp/3b4/2pN4/2p5/5N2/PP2PPPP/R1B1KB1R w KQkq - 1 8' => {
			eco => 'D21q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6. Qxd5 Qxd5 7. Nxd5 Bd6

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6.Qxd5 Qxd5 7.Nxd5 Bd6'),
			moves => {
				'f3d2' => 'rnb1k1nr/pp3ppp/3b4/2pN4/2p5/8/PP1NPPPP/R1B1KB1R b KQkq - 2 8',
			},
		},
		'rnb1k1nr/pp3ppp/3b4/2pN4/2p5/8/PP1NPPPP/R1B1KB1R b KQkq - 2 8' => {
			eco => 'D21r',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6. Qxd5 Qxd5 7. Nxd5 Bd6 8. Nd2

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6.Qxd5 Qxd5 7.Nxd5 Bd6 8. Nd2'),
			moves => {
				'g8e7' => 'rnb1k2r/pp2nppp/3b4/2pN4/2p5/8/PP1NPPPP/R1B1KB1R w KQkq - 3 9',
			},
		},
		'rnb1k1nr/pp3ppp/4p3/2pq4/1b1P4/2NB4/PPP2PPP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'C15f',
                        # TRANSLATORS:

			variation => N__('French: Winawer, 4. Bd3 c5'),
			moves => {
				'c1d2' => 'rnb1k1nr/pp3ppp/4p3/2pq4/1b1P4/2NB4/PPPB1PPP/R2QK1NR b KQkq - 1 6',
			},
		},
		'rnb1k1nr/pp3ppp/4p3/2pq4/1b1P4/2NB4/PPPB1PPP/R2QK1NR b KQkq - 1 6' => {
			eco => 'C15f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd3 c5 5. exd5 Qxd5 6. Bd2

			variation => N__('French: Winawer, Kondratiyev Variation'),
			moves => {
			},
		},
		'rnb1k1nr/pp3ppp/4p3/q1ppP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'C18b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Qa5

			variation => N__('French: Winawer, 6... Qa5'),
			moves => {
			},
		},
		'rnb1k1nr/ppp2p1p/8/4P3/6p1/8/PPP1P1PP/RNBqKB1R w KQkq - 0 8' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Bxd6 4. Nf3 g5 5. d4 g4 6. Ne5 Bxe5 7. dxe5 Qxd1+

			variation => N__('Bird: From Gambit, Lasker, Queenswap Line'),
			moves => {
			},
		},
		'rnb1k1nr/ppp2pbp/3p4/4P1p1/2BP1p1q/2N5/PPP3PP/R1BQ1KNR b kq - 0 7' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5 5. Nc3 Bg7 6. d4 d6 7. e5

			variation => N__('KGA: Bishop\'s Gambit, Grimm Attack'),
			moves => {
			},
		},
		'rnb1k1nr/ppp2pbp/3p4/6p1/2BPPp1q/2N5/PPP3PP/R1BQ1KNR w kq - 0 7' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence'),
			moves => {
				'e4e5' => 'rnb1k1nr/ppp2pbp/3p4/4P1p1/2BP1p1q/2N5/PPP3PP/R1BQ1KNR b kq - 0 7',
			},
		},
		'rnb1k1nr/ppp2ppp/3b4/3B4/4Pp1q/8/PPPP2PP/RNBQ1KNR w kq - 3 6' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 d5 4. Bxd5 Qh4+ 5. Kf1 Bd6

			variation => N__('KGA: Bishop\'s Gambit, Boren-Svenonius Variation'),
			moves => {
			},
		},
		'rnb1k1nr/ppp2ppp/4p3/3q4/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C15b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. exd5 Qxd5

			variation => N__('French: Winawer, 4.exd5 Qxd5'),
			moves => {
			},
		},
		'rnb1k1nr/ppp2ppp/4p3/8/1b1qp1Q1/2N5/PPPB1PPP/R3KBNR w KQkq - 0 6' => {
			eco => 'C15i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd2 dxe4 5. Qg4 Qxd4

			variation => N__('French: Winawer, Müller-Zhuravlev Gambit, Kunin Variation'),
			moves => {
			},
		},
		'rnb1k1nr/pppp1p1p/7b/4q3/2B2p2/2NP1Q2/PPP3PP/R1B2RK1 b kq - 2 9' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'g8e7' => 'rnb1k2r/ppppnp1p/7b/4q3/2B2p2/2NP1Q2/PPP3PP/R1B2RK1 w kq - 3 10',
			},
		},
		'rnb1k1nr/pppp1p1p/7b/4q3/2B2p2/3P1Q2/PPP3PP/RNB2RK1 w kq - 1 9' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'b1c3' => 'rnb1k1nr/pppp1p1p/7b/4q3/2B2p2/2NP1Q2/PPP3PP/R1B2RK1 b kq - 2 9',
			},
		},
		'rnb1k1nr/pppp1pbp/8/6p1/2B1P2q/2N2Qp1/PPPP3P/R1B2KNR b kq - 1 7' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5 5. Nc3 Bg7 6. g3 fxg3 7. Qf3

			variation => N__('KGA: Bishop\'s Gambit, Fraser Variation'),
			moves => {
			},
		},
		'rnb1k1nr/pppp1pbp/8/6p1/2B1P2q/2N3p1/PPPP3P/R1BQ1KNR w kq - 0 7' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: bishop\'s Gambit, McDonnell Attack'),
			moves => {
				'd1f3' => 'rnb1k1nr/pppp1pbp/8/6p1/2B1P2q/2N2Qp1/PPPP3P/R1B2KNR b kq - 1 7',
			},
		},
		'rnb1k1nr/pppp1pbp/8/6p1/2B1Pp1q/2N3P1/PPPP3P/R1BQ1KNR b kq - 0 6' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5 5. Nc3 Bg7 6. g3

			variation => N__('KGA: bishop\'s Gambit, McDonnell Attack'),
			moves => {
				'f4g3' => 'rnb1k1nr/pppp1pbp/8/6p1/2B1P2q/2N3p1/PPPP3P/R1BQ1KNR w kq - 0 7',
			},
		},
		'rnb1k1nr/pppp1pbp/8/6p1/2B1Pp1q/2N5/PPPP2PP/R1BQ1KNR w kq - 2 6' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence'),
			moves => {
				'd2d4' => 'rnb1k1nr/pppp1pbp/8/6p1/2BPPp1q/2N5/PPP3PP/R1BQ1KNR b kq d3 0 6',
				'g2g3' => 'rnb1k1nr/pppp1pbp/8/6p1/2B1Pp1q/2N3P1/PPPP3P/R1BQ1KNR b kq - 0 6',
			},
		},
		'rnb1k1nr/pppp1pbp/8/6p1/2BPPp1q/2N5/PPP3PP/R1BQ1KNR b kq d3 0 6' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence'),
			moves => {
				'd7d6' => 'rnb1k1nr/ppp2pbp/3p4/6p1/2BPPp1q/2N5/PPP3PP/R1BQ1KNR w kq - 0 7',
				'g8e7' => 'rnb1k2r/ppppnpbp/8/6p1/2BPPp1q/2N5/PPP3PP/R1BQ1KNR w kq - 1 7',
			},
		},
		'rnb1k1nr/pppp1ppp/8/2b1p1q1/2B1P3/2P5/PP1P1PPP/RNBQK1NR w KQkq - 1 4' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. c3 Qg5

			variation => N__('Bishop\'s Opening: del Rio Variation'),
			moves => {
			},
		},
		'rnb1k1nr/pppp1ppp/8/2b5/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq - 4 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 Bc5

			variation => N__('KGA: Bishop\'s Gambit, Greco Variation'),
			moves => {
			},
		},
		'rnb1k1nr/pppp1ppp/8/4p3/1b2Pq2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'C30e',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'f1c4' => 'rnb1k1nr/pppp1ppp/8/4p3/1bB1Pq2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5',
			},
		},
		'rnb1k1nr/pppp1ppp/8/4p3/1bB1Pq2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5' => {
			eco => 'C30e',
                        # TRANSLATORS: 1. e4 e5 2. f4 Qf6 3. Nf3 Qxf4 4. Nc3 Bb4 5. Bc4

			variation => N__('King\'s Gambit: Norwald Variation, Bücker Gambit'),
			moves => {
			},
		},
		'rnb1k1nr/ppppqppp/1b1P4/4p3/2B1P3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 7' => {
			eco => 'C53',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bc4 Bc5 4. c3 Qe7 5. d4 Bb6 6. d5 Nb8 7. d6

			variation => N__('Giuoco Piano: Eisinger Variation'),
			moves => {
			},
		},
		'rnb1k1nr/ppppqppp/1b6/3Pp3/2B1P3/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C53',
                        # TRANSLATORS:

			variation => N__('Giuoco Piano: Centre-Holding Variation'),
			moves => {
				'd5d6' => 'rnb1k1nr/ppppqppp/1b1P4/4p3/2B1P3/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 7',
			},
		},
		'rnb1k1nr/pppq1ppp/4p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C16f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Qd7

			variation => N__('French: Winawer, Petrosian Variation'),
			moves => {
				'a2a3' => 'rnb1k1nr/pppq1ppp/4p3/3pP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5',
				'c1d2' => 'rnb1k1nr/pppq1ppp/4p3/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 2 5',
			},
		},
		'rnb1k1nr/pppq1ppp/4p3/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 2 5' => {
			eco => 'C16g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Qd7 5. Bd2

			variation => N__('French: Winawer, Petrosian, 5. Bd2'),
			moves => {
			},
		},
		'rnb1k1nr/pppq1ppp/4p3/3pP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C16h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Qd7 5. a3

			variation => N__('French: Winawer, Petrosian, 5. a3'),
			moves => {
				'b4c3' => 'rnb1k1nr/pppq1ppp/4p3/3pP3/3P4/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnb1k1nr/pppq1ppp/4p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C16h',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Petrosian, 5. a3'),
			moves => {
				'b7b6' => 'rnb1k1nr/p1pq1ppp/1p2p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnb1k1nr/pppq1ppp/4p3/3pP3/3P4/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C16h',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Petrosian, 5. a3'),
			moves => {
				'b2c3' => 'rnb1k1nr/pppq1ppp/4p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnb1k1nr/ppq2ppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 2 7' => {
			eco => 'C18d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Qc7 7. Qg4

			variation => N__('French: Winawer, 6... Qc7 7. Qg4'),
			moves => {
				'f7f5' => 'rnb1k1nr/ppq3pp/4p3/2ppPp2/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq f6 0 8',
				'f7f6' => 'rnb1k1nr/ppq3pp/4pp2/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 0 8',
			},
		},
		'rnb1k1nr/ppq2ppp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'C18c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Qc7

			variation => N__('French: Winawer, 6... Qc7'),
			moves => {
				'd1g4' => 'rnb1k1nr/ppq2ppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 2 7',
			},
		},
		'rnb1k1nr/ppq3pp/4p3/2ppPp2/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq f6 0 8' => {
			eco => 'C18e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Qc7 7. Qg4 f5

			variation => N__('French: Winawer, 6... Qc7 7. Qg4 f5'),
			moves => {
			},
		},
		'rnb1k1nr/ppq3pp/4pp2/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 0 8' => {
			eco => 'C18d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Qc7 7. Qg4 f6

			variation => N__('French: Winawer, 6... Qc7 7. Qg4 f6'),
			moves => {
			},
		},
		'rnb1k1r1/ppq1np1Q/4p3/2ppP3/3P4/P1P5/2P2PPP/R1B1KBNR b KQq - 0 9' => {
			eco => 'C18r',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, 8.Qxg7'),
			moves => {
				'c5d4' => 'rnb1k1r1/ppq1np1Q/4p3/3pP3/3p4/P1P5/2P2PPP/R1B1KBNR w KQq - 0 10',
			},
		},
		'rnb1k1r1/ppq1np1Q/4p3/3pP3/3p4/P1P5/2P1NPPP/R1B1KB1R b KQq - 1 10' => {
			eco => 'C18t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7 8. Qxg7 Rg8 9. Qxh7 cxd4 10. Ne2

			variation => N__('French: Winawer, Poisoned Pawn, Konstantinopolsky Variation'),
			moves => {
				'b8c6' => 'r1b1k1r1/ppq1np1Q/2n1p3/3pP3/3p4/P1P5/2P1NPPP/R1B1KB1R w KQq - 2 11',
			},
		},
		'rnb1k1r1/ppq1np1Q/4p3/3pP3/3p4/P1P5/2P2PPP/R1B1KBNR w KQq - 0 10' => {
			eco => 'C18r',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, 8.Qxg7'),
			moves => {
				'e1d1' => 'rnb1k1r1/ppq1np1Q/4p3/3pP3/3p4/P1P5/2P2PPP/R1BK1BNR b q - 1 10',
				'g1e2' => 'rnb1k1r1/ppq1np1Q/4p3/3pP3/3p4/P1P5/2P1NPPP/R1B1KB1R b KQq - 1 10',
				'h7d3' => 'rnb1k1r1/ppq1np2/4p3/3pP3/3p4/P1PQ4/2P2PPP/R1B1KBNR b KQq - 1 10',
			},
		},
		'rnb1k1r1/ppq1np1Q/4p3/3pP3/3p4/P1P5/2P2PPP/R1BK1BNR b q - 1 10' => {
			eco => 'C18s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7 8. Qxg7 Rg8 9. Qxh7 cxd4 10. Kd1

			variation => N__('French: Winawer, Poisoned Pawn, Euwe-Gligoric Variation'),
			moves => {
			},
		},
		'rnb1k1r1/ppq1np2/4p3/3pP3/3p4/P1PQ4/2P2PPP/R1B1KBNR b KQq - 1 10' => {
			eco => 'C18r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7 8. Qxg7 Rg8 9. Qxh7 cxd4 10. Qd3

			variation => N__('French: Winawer, Poisoned Pawn, 10. Qd3'),
			moves => {
			},
		},
		'rnb1k1r1/ppq1npQp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1B1KBNR w KQq - 1 9' => {
			eco => 'C18r',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Poisoned Pawn, 8.Qxg7'),
			moves => {
				'g7h7' => 'rnb1k1r1/ppq1np1Q/4p3/2ppP3/3P4/P1P5/2P2PPP/R1B1KBNR b KQq - 0 9',
			},
		},
		'rnb1k2r/1p1n1ppp/p3p3/4P1B1/1bBN4/q1N5/P1PQ2PP/1R2K2R w Kkq - 3 13' => {
			eco => 'B97p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. e5 dxe5 11. fxe5 Nfd7 12. Bc4 Bb4

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. e5 dxe5 11.fxe5 Nfd7 12. Bc4 Bb4'),
			moves => {
			},
		},
		'rnb1k2r/1p1nqppp/p3p3/2PpP3/5P2/2N2N2/PPP3PP/R2QKB1R b KQkq - 0 9' => {
			eco => 'C14i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 a6 8. Nf3 c5 9. dxc5

			variation => N__('French: Classical, Steinitz, 7... a6 8. Nf3 c5'),
			moves => {
			},
		},
		'rnb1k2r/1p1nqppp/p3p3/2ppP3/3P1P2/2N2N2/PPP3PP/R2QKB1R w KQkq c6 0 9' => {
			eco => 'C14h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 a6 8. Nf3 c5

			variation => N__('French: Classical, Steinitz, 7... a6 8. Nf3 c5'),
			moves => {
				'd4c5' => 'rnb1k2r/1p1nqppp/p3p3/2PpP3/5P2/2N2N2/PPP3PP/R2QKB1R b KQkq - 0 9',
			},
		},
		'rnb1k2r/1p2bppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R2K2R w Kkq - 4 11' => {
			eco => 'B97n',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. Be2'),
			moves => {
				'e1g1' => 'rnb1k2r/1p2bppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R3RK1 b kq - 5 11',
			},
		},
		'rnb1k2r/1p2bppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R3RK1 b kq - 5 11' => {
			eco => 'B97n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. Be2 Be7 11. O-O

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. Be2 Be7 11.O-O'),
			moves => {
				'b8d7' => 'r1b1k2r/1p1nbppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R3RK1 w kq - 6 12',
			},
		},
		'rnb1k2r/1p2bppp/p2ppn2/q5B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 4 9' => {
			eco => 'B98c',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Danner Variation'),
			moves => {
				'c8d7' => 'rn2k2r/1p1bbppp/p2ppn2/q5B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq - 5 10',
			},
		},
		'rnb1k2r/1p2bppp/p2ppn2/q5B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 3 9' => {
			eco => 'B98c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qa5

			variation => N__('Sicilian: Najdorf, Danner Variation'),
			moves => {
				'e1c1' => 'rnb1k2r/1p2bppp/p2ppn2/q5B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 4 9',
			},
		},
		'rnb1k2r/1p2bppp/pq1ppn2/6B1/4PP2/1NN5/PPP3PP/R2QKB1R w KQkq - 3 9' => {
			eco => 'B97f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Nb3 Be7

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Nb3 Be7'),
			moves => {
			},
		},
		'rnb1k2r/1p3pbp/p2ppp2/8/3NPP2/q1N5/P1PQB1PP/1R2K2R w Kkq - 2 12' => {
			eco => 'B97o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. Bxf6 gxf6 11. Be2 Bg7

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10.Bxf6 gxf6 11. Be2 Bg7'),
			moves => {
			},
		},
		'rnb1k2r/1p3ppp/p3pn2/2b5/2B5/1P2PN2/P4PPP/RNBR2K1 b kq - 0 9' => {
			eco => 'D27o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. dxc5 Qxd1 8. Rxd1 Bxc5 9. b3

			variation => N__('QGA: Classical, Spassky, 9. b3'),
			moves => {
			},
		},
		'rnb1k2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP1N1PPP/R1BR2K1 b kq - 1 9' => {
			eco => 'D27n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. dxc5 Qxd1 8. Rxd1 Bxc5 9. Nbd2

			variation => N__('QGA: Classical, Spassky, 9.Nbd2'),
			moves => {
			},
		},
		'rnb1k2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP3PPP/RNBR2K1 w kq - 0 9' => {
			eco => 'D27m',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Spassky, 7...Qxd1'),
			moves => {
				'b1d2' => 'rnb1k2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP1N1PPP/R1BR2K1 b kq - 1 9',
				'b2b3' => 'rnb1k2r/1p3ppp/p3pn2/2b5/2B5/1P2PN2/P4PPP/RNBR2K1 b kq - 0 9',
			},
		},
		'rnb1k2r/1pp1qppp/3p1n2/p2Pp3/2P1P3/5NP1/PP1N1PBP/R2Q1RK1 w kq - 1 11' => {
			eco => 'E11w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 Nc6 6. Bg2 Bxd2+ 7. Nbxd2 d6 8. O-O a5 9. e4 e5 10. d5 Nb8

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 Nc6 Main Line, 8... a5 9. e4 e5 10. d5 Nb8'),
			moves => {
			},
		},
		'rnb1k2r/1ppnqppp/p3p3/3pP3/3P1P2/2N2N2/PPP3PP/R2QKB1R b KQkq - 1 8' => {
			eco => 'C14g',
                        # TRANSLATORS:

			variation => N__('French: Classical, Steinitz, 7... a6'),
			moves => {
				'c7c5' => 'rnb1k2r/1p1nqppp/p3p3/2ppP3/3P1P2/2N2N2/PPP3PP/R2QKB1R w KQkq c6 0 9',
			},
		},
		'rnb1k2r/1ppnqppp/p3p3/3pP3/3P1P2/2N5/PPP3PP/R2QKBNR w KQkq - 0 8' => {
			eco => 'C14g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 a6

			variation => N__('French: Classical, Steinitz, 7... a6'),
			moves => {
				'g1f3' => 'rnb1k2r/1ppnqppp/p3p3/3pP3/3P1P2/2N2N2/PPP3PP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnb1k2r/1pq1bpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/2KR1B1R b kq - 3 10' => {
			eco => 'B98g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Browne Variation'),
			moves => {
				'b8d7' => 'r1b1k2r/1pqnbpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/2KR1B1R w kq - 4 11',
			},
		},
		'rnb1k2r/1pq1bpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/R3KB1R w KQkq - 2 10' => {
			eco => 'B98g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 Qc7

			variation => N__('Sicilian: Najdorf, Browne Variation'),
			moves => {
				'e1c1' => 'rnb1k2r/1pq1bpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/2KR1B1R b kq - 3 10',
			},
		},
		'rnb1k2r/1pq1bppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 4 9' => {
			eco => 'B98n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O

			variation => N__('Sicilian: Najdorf, 8... Qc7 9.O-O-O'),
			moves => {
				'b7b5' => 'rnb1k2r/2q1bppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq b6 0 10',
				'b8c6' => 'r1b1k2r/1pq1bppp/p1nppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq - 5 10',
				'b8d7' => 'r1b1k2r/1pqnbppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq - 5 10',
			},
		},
		'rnb1k2r/1pq1bppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 3 9' => {
			eco => 'B98n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7

			variation => N__('Sicilian: Najdorf, 8... Qc7'),
			moves => {
				'e1c1' => 'rnb1k2r/1pq1bppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 4 9',
			},
		},
		'rnb1k2r/1pq1bppp/p2ppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b kq - 2 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 Qc7 9. Be3

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7 8. f4 Qc7 9. Be3'),
			moves => {
				'b8c6' => 'r1b1k2r/1pq1bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w kq - 3 10',
			},
		},
		'rnb1k2r/1pq1bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1R1K b kq - 2 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 Qc7 9. Kh1

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7 8. f4 Qc7 9. Kh1'),
			moves => {
			},
		},
		'rnb1k2r/1pq1bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 w kq - 1 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 Qc7

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7 8. f4 Qc7'),
			moves => {
				'c1e3' => 'rnb1k2r/1pq1bppp/p2ppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b kq - 2 9',
				'g1h1' => 'rnb1k2r/1pq1bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1R1K b kq - 2 9',
			},
		},
		'rnb1k2r/2q1bp1p/p2p1n2/1ppPp1p1/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w kq g6 0 14' => {
			eco => 'C88e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 d6 8. c3 Na5 9. Bc2 c5 10. d4 Qc7 11. h3 Nc6 12. d5 Nb8 13. Nbd2 g5

			variation => N__('Spanish: Closed, Leonhardt Variation'),
			moves => {
			},
		},
		'rnb1k2r/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPB2PP1/RNBQR1K1 w kq - 1 13' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'b1d2' => 'rnb1k2r/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 b kq - 2 13',
			},
		},
		'rnb1k2r/2q1bppp/p2p1n2/1ppPp3/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 b kq - 2 13' => {
			eco => 'C88e',
                        # TRANSLATORS:

			variation => N__('Spanish: Closed 7... d6 8. c3 Na5'),
			moves => {
				'g7g5' => 'rnb1k2r/2q1bp1p/p2p1n2/1ppPp1p1/4P3/2P2N1P/PPBN1PP1/R1BQR1K1 w kq g6 0 14',
			},
		},
		'rnb1k2r/2q1bppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R w kq b6 0 10' => {
			eco => 'B98o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 Qc7 9. O-O-O b5

			variation => N__('Sicilian: Najdorf, 8... Qc7 9.O-O-O b5'),
			moves => {
			},
		},
		'rnb1k2r/p1q1nppp/1p2p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R w KQkq - 0 9' => {
			eco => 'C19f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Qc7 8. a4 b6

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Qc7 8. a4 b6'),
			moves => {
			},
		},
		'rnb1k2r/p3ppbp/1p4p1/q1p5/3PP3/2P1B3/P2Q1PPP/1R2KBNR w Kkq - 0 10' => {
			eco => 'D85h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2 Qa5 9. Rb1 b6

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2 Qa5 9. Rb1 b6'),
			moves => {
			},
		},
		'rnb1k2r/pp1n1pbp/3p2p1/1BpP4/P3PP1q/2N5/1P4PP/R1BQK1NR w KQkq - 1 10' => {
			eco => 'A67l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 Qh4+

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 Qh4+'),
			moves => {
			},
		},
		'rnb1k2r/pp1p1ppp/1q2pn2/2b5/2P5/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 2 7' => {
			eco => 'E38f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Bxc5 6. Nf3 Qb6

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Bxc5 6. Nf3 Qb6'),
			moves => {
			},
		},
		'rnb1k2r/pp1p1ppp/4pn2/q1p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'E26',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. e3 Qa5

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. e3 Qa5'),
			moves => {
			},
		},
		'rnb1k2r/pp1pppbp/1q6/2pP1p2/8/4P3/PPP2PPP/RNQ1KBNR w KQkq - 1 7' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. e3 Bg7

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. e3 Bg7'),
			moves => {
			},
		},
		'rnb1k2r/pp2nppp/3b4/2pN4/2p5/8/PP1NPPPP/R1B1KB1R w KQkq - 3 9' => {
			eco => 'D21s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6. Qxd5 Qxd5 7. Nxd5 Bd6 8. Nd2 Ne7

			variation => N__('QGA: Seirawan Variation'),
			moves => {
			},
		},
		'rnb1k2r/pp2nppp/4p3/q1ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8' => {
			eco => 'C19g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Qa5

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Qa5'),
			moves => {
			},
		},
		'rnb1k2r/pp2nppp/4p3/q1ppP3/P2P4/2P5/2P2PPP/R1BQKBNR w KQkq - 1 8' => {
			eco => 'C19b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. a4 Qa5

			variation => N__('French: Winawer, Smyslov, 7... Qa5'),
			moves => {
			},
		},
		'rnb1k2r/pp2ppbp/3p1np1/q1P5/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B09p',
                        # TRANSLATORS:

			variation => N__('Pirc: Austrian, 5... c5 6.dxc5'),
			moves => {
				'f1d3' => 'rnb1k2r/pp2ppbp/3p1np1/q1P5/4PP2/2NB1N2/PPP3PP/R1BQK2R b KQkq - 2 7',
			},
		},
		'rnb1k2r/pp2ppbp/3p1np1/q1P5/4PP2/2NB1N2/PPP3PP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B09q',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. dxc5 Qa5 7. Bd3

			variation => N__('Pirc: Austrian, 5... c5 6.dxc5 Qa5 7. Bd3'),
			moves => {
			},
		},
		'rnb1k2r/pp2ppbp/5np1/q1Pp4/2P2B2/2N1P3/PP3PPP/2RQKBNR b Kkq - 2 7' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 c5 6. dxc5 Qa5 7. Rc1

			variation => N__('Grünfeld: 4. Bf4 Bg7 5. e3 c5 6.dxc5 Qa5 7. Rc1'),
			moves => {
			},
		},
		'rnb1k2r/pp2ppbp/5np1/q1Pp4/2P2B2/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 c5 6. dxc5 Qa5

			variation => N__('Grünfeld: 4. Bf4 Bg7 5. e3 c5 6.dxc5 Qa5'),
			moves => {
				'a1c1' => 'rnb1k2r/pp2ppbp/5np1/q1Pp4/2P2B2/2N1P3/PP3PPP/2RQKBNR b Kkq - 2 7',
				'd1a4' => 'rnb1k2r/pp2ppbp/5np1/q1Pp4/Q1P2B2/2N1P3/PP3PPP/R3KBNR b KQkq - 2 7',
			},
		},
		'rnb1k2r/pp2ppbp/5np1/q1Pp4/Q1P2B2/2N1P3/PP3PPP/R3KBNR b KQkq - 2 7' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 c5 6. dxc5 Qa5 7. Qa4+

			variation => N__('Grünfeld: 4. Bf4 Bg7 5. e3 c5 6.dxc5 Qa5 7. Qa4+'),
			moves => {
			},
		},
		'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1B3/P2Q1PPP/1R2KBNR b Kkq - 3 9' => {
			eco => 'D85h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2 Qa5 9. Rb1

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2 Qa5 9. Rb1'),
			moves => {
				'b7b6' => 'rnb1k2r/p3ppbp/1p4p1/q1p5/3PP3/2P1B3/P2Q1PPP/1R2KBNR w Kkq - 0 10',
			},
		},
		'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1B3/P2Q1PPP/2R1KBNR b Kkq - 3 9' => {
			eco => 'D85h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2 Qa5 9. Rc1

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2 Qa5 9. Rc1'),
			moves => {
			},
		},
		'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1B3/P2Q1PPP/R3KBNR w KQkq - 2 9' => {
			eco => 'D85h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2 Qa5

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2 Qa5'),
			moves => {
				'a1b1' => 'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1B3/P2Q1PPP/1R2KBNR b Kkq - 3 9',
				'a1c1' => 'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1B3/P2Q1PPP/2R1KBNR b Kkq - 3 9',
			},
		},
		'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P2Q1PPP/R3KB1R b KQkq - 3 9' => {
			eco => 'D85u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 Qa5 9. Qd2

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 Qa5 9. Qd2'),
			moves => {
				'b8c6' => 'r1b1k2r/pp2ppbp/2n3p1/q1p5/3PP3/2P1BN2/P2Q1PPP/R3KB1R w KQkq - 4 10',
			},
		},
		'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P4PPP/R2QKB1R w KQkq - 2 9' => {
			eco => 'D85u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 Qa5

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 Qa5'),
			moves => {
				'd1d2' => 'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P2Q1PPP/R3KB1R b KQkq - 3 9',
			},
		},
		'rnb1k2r/pp2qpp1/3ppn1p/2pP4/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 8' => {
			eco => 'E31f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Qe7

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 Qe7'),
			moves => {
			},
		},
		'rnb1k2r/pp2qpp1/3ppn1p/2pP4/2P4B/2P1P3/P4PPP/R2QKBNR w KQkq - 1 9' => {
			eco => 'E31i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 Qe7

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... Qe7'),
			moves => {
			},
		},
		'rnb1k2r/pp3ppp/4p3/q7/1b1PP3/5N2/P2B1PPP/R2QKB1R w KQkq - 3 10' => {
			eco => 'D41m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+ 9. Bd2 Qa5

			variation => N__('QGD: Semi-Tarrasch, 6. e4, San Sebastian Variation'),
			moves => {
			},
		},
		'rnb1k2r/pp3ppp/4pn2/2pq4/1b1P4/2N1P3/PPQ2PPP/R1B1KBNR w KQkq c6 0 7' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. e3 c5

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5 6. e3 c5'),
			moves => {
			},
		},
		'rnb1k2r/pp3ppp/4pn2/2pq4/1b1P4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq c6 0 7' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. Nf3 c5

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5 6. Nf3 c5'),
			moves => {
			},
		},
		'rnb1k2r/pp3ppp/4pn2/q5B1/1bBNP3/2N5/PP3PPP/R2QK2R w KQkq - 1 9' => {
			eco => 'D39i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4 cxd4 8. Nxd4 Qa5

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4, 8... Qa5'),
			moves => {
			},
		},
		'rnb1k2r/pp3ppp/4pn2/q5B1/2BNP3/2P5/P4PPP/R2QK2R w KQkq - 1 10' => {
			eco => 'D39j',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4, 8...Bxc3+'),
			moves => {
				'c4b5' => 'rnb1k2r/pp3ppp/4pn2/qB4B1/3NP3/2P5/P4PPP/R2QK2R b KQkq - 2 10',
			},
		},
		'rnb1k2r/pp3ppp/4pn2/qB4B1/3NP3/2P5/P4PPP/R2QK2R b KQkq - 2 10' => {
			eco => 'D39k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4 cxd4 8. Nxd4 Bxc3+ 9. bxc3 Qa5 10. Bb5+

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4, 8...Bxc3+, 10. Bb5+'),
			moves => {
			},
		},
		'rnb1k2r/ppp1qppp/5n2/3P4/1bBp4/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 d5 5. exd5 Bb4+ 6. c3 Qe7+

			variation => N__('Bishop\'s Opening: Urusov Gambit, Panov Variation'),
			moves => {
			},
		},
		'rnb1k2r/ppp2p1p/4pp2/3q4/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C12c',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, 5.exd5 Qxd5'),
			moves => {
				'd1d2' => 'rnb1k2r/ppp2p1p/4pp2/3q4/1b1P4/2N5/PPPQ1PPP/R3KBNR b KQkq - 1 7',
			},
		},
		'rnb1k2r/ppp2p1p/4pp2/3q4/1b1P4/2N5/PPPQ1PPP/R3KBNR b KQkq - 1 7' => {
			eco => 'C12c',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, 5.exd5 Qxd5'),
			moves => {
				'd5a5' => 'rnb1k2r/ppp2p1p/4pp2/q7/1b1P4/2N5/PPPQ1PPP/R3KBNR w KQkq - 2 8',
			},
		},
		'rnb1k2r/ppp2p1p/4pp2/q7/1b1P4/2N5/PPPQ1PPP/R3KBNR w KQkq - 2 8' => {
			eco => 'C12c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. exd5 Qxd5 6. Bxf6 gxf6 7. Qd2 Qa5

			variation => N__('French: MacCutcheon, Bogoljubow Variation'),
			moves => {
			},
		},
		'rnb1k2r/ppp2p1p/5n2/3qN3/1b1P1ppP/2N5/PPP2KP1/R1BQ1B1R b kq - 3 9' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 d5 6. d4 Nf6 7. exd5 Qxd5 8. Nc3 Bb4 9. Kf2

			variation => N__('KGA: Kieseritsky, Brentano Defence, Kaplanek Variation'),
			moves => {
			},
		},
		'rnb1k2r/ppp2p1p/5n2/3qN3/1b1P1ppP/2N5/PPP3P1/R1BQKB1R w KQkq - 2 9' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'e1f2' => 'rnb1k2r/ppp2p1p/5n2/3qN3/1b1P1ppP/2N5/PPP2KP1/R1BQ1B1R b kq - 3 9',
			},
		},
		'rnb1k2r/ppp2pp1/4pq1p/3P4/1b1P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D38n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 h6 6. Bxf6 Qxf6 7. cxd5

			variation => N__('QGD: Ragozin, 5. Bg5 h6 6.Bxf6 Qxf6 7.cxd5'),
			moves => {
				'e6d5' => 'rnb1k2r/ppp2pp1/5q1p/3p4/1b1P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnb1k2r/ppp2pp1/4pq1p/3p4/1bPP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D38o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 h6 6. Bxf6 Qxf6 7. e3

			variation => N__('QGD: Ragozin, 5. Bg5 h6 6.Bxf6 Qxf6 7. e3'),
			moves => {
				'e8g8' => 'rnb2rk1/ppp2pp1/4pq1p/3p4/1bPP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 8',
			},
		},
		'rnb1k2r/ppp2pp1/4pq1p/3p4/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D38m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 h6 6. Bxf6 Qxf6

			variation => N__('QGD: Ragozin, 5. Bg5 h6 6.Bxf6 Qxf6'),
			moves => {
				'c4d5' => 'rnb1k2r/ppp2pp1/4pq1p/3P4/1b1P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 7',
				'e2e3' => 'rnb1k2r/ppp2pp1/4pq1p/3p4/1bPP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnb1k2r/ppp2pp1/5q1p/3p4/1b1P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D38n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 h6 6. Bxf6 Qxf6 7. cxd5 exd5

			variation => N__('QGD: Ragozin, 5. Bg5 h6 6.Bxf6 Qxf6 7.cxd5'),
			moves => {
			},
		},
		'rnb1k2r/ppp2ppp/4pB2/3q4/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C12c',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, 5.exd5 Qxd5'),
			moves => {
				'g7f6' => 'rnb1k2r/ppp2p1p/4pp2/3q4/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/3q2B1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'C12c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. exd5 Qxd5

			variation => N__('French: MacCutcheon, 5.exd5 Qxd5'),
			moves => {
				'g5f6' => 'rnb1k2r/ppp2ppp/4pB2/3q4/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/3q4/1b1P4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. e3

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5 6. e3'),
			moves => {
				'c7c5' => 'rnb1k2r/pp3ppp/4pn2/2pq4/1b1P4/2N1P3/PPQ2PPP/R1B1KBNR w KQkq c6 0 7',
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/3q4/1b1P4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 6' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. Nf3

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5 6. Nf3'),
			moves => {
				'c7c5' => 'rnb1k2r/pp3ppp/4pn2/2pq4/1b1P4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq c6 0 7',
				'd5f5' => 'rnb1k2r/ppp2ppp/4pn2/5q2/1b1P4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 2 7',
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/3q4/1b1P4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 6' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5'),
			moves => {
				'e2e3' => 'rnb1k2r/ppp2ppp/4pn2/3q4/1b1P4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 0 6',
				'g1f3' => 'rnb1k2r/ppp2ppp/4pn2/3q4/1b1P4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 6',
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/5Q2/1b1P4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. Nf3 Qf5 7. Qxf5

			variation => N__('Nimzo-Indian: Classical, Noa, Queenswap'),
			moves => {
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/5q2/1b1P4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 3 7' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. Nf3 Qf5 7. Qb3

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5 6. Nf3 Qf5 7. Qb3'),
			moves => {
			},
		},
		'rnb1k2r/ppp2ppp/4pn2/5q2/1b1P4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 2 7' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 Qxd5 6. Nf3 Qf5

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5 Qxd5 6. Nf3 Qf5'),
			moves => {
				'c2b3' => 'rnb1k2r/ppp2ppp/4pn2/5q2/1b1P4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 3 7',
				'c2f5' => 'rnb1k2r/ppp2ppp/4pn2/5Q2/1b1P4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 7',
			},
		},
		'rnb1k2r/ppp2ppp/8/3pP3/1b2P2q/2N5/PPP1K1PP/R1BQ1BNR b kq - 2 7' => {
			eco => 'C29h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. d3 Bb4 6. dxe4 Qh4+ 7. Ke2

			variation => N__('Vienna Gambit: Oxford, 5... Bb4 6.dxe4 Qh4+ 7. Ke2'),
			moves => {
			},
		},
		'rnb1k2r/ppp2ppp/8/3pP3/1b2P2q/2N5/PPP3PP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'C29h',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford, 5... Bb4'),
			moves => {
				'e1e2' => 'rnb1k2r/ppp2ppp/8/3pP3/1b2P2q/2N5/PPP1K1PP/R1BQ1BNR b kq - 2 7',
			},
		},
		'rnb1k2r/ppp2ppp/8/3q4/4nP2/5N2/PPP1QbPP/RNBK1B1R w kq - 0 9' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Nf3 Bc5 7. Qe2'),
			moves => {
				'f3d2' => 'rnb1k2r/ppp2ppp/8/3q4/4nP2/8/PPPNQbPP/RNBK1B1R b kq - 1 9',
			},
		},
		'rnb1k2r/ppp2ppp/8/3q4/4nP2/8/PPPNQbPP/RNBK1B1R b kq - 1 9' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Nf3 Bc5 7. Qe2 Bf2+ 8. Kd1 Qxd5+ 9. Nfd2

			variation => N__('KGD: Falkbeer, Alapin Variation'),
			moves => {
			},
		},
		'rnb1k2r/pppn1ppp/4p3/3pP1q1/3P4/2N4N/PPP2PP1/R2QKB1R b KQkq - 1 8' => {
			eco => 'C13l',
                        # TRANSLATORS:

			variation => N__('French: Chatard-Alekhine, 6...Bxg5 7.hxg5 Qxg5'),
			moves => {
				'g5e7' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2N4N/PPP2PP1/R2QKB1R w KQkq - 2 9',
			},
		},
		'rnb1k2r/pppn1ppp/4p3/3pP1q1/3P4/2N5/PPP2PP1/R2QKBNR w KQkq - 0 8' => {
			eco => 'C13l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 Bxg5 7. hxg5 Qxg5

			variation => N__('French: Chatard-Alekhine, 6...Bxg5 7.hxg5 Qxg5'),
			moves => {
				'g1h3' => 'rnb1k2r/pppn1ppp/4p3/3pP1q1/3P4/2N4N/PPP2PP1/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnb1k2r/pppnqppp/4p3/1N1pP3/3P4/8/PPP2PPP/R2QKBNR b KQkq - 1 7' => {
			eco => 'C14d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. Nb5

			variation => N__('French: Classical, Alapin Variation'),
			moves => {
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P1N2/2N5/PPP2PP1/R2QKB1R b KQkq - 3 9' => {
			eco => 'C13m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 Bxg5 7. hxg5 Qxg5 8. Nh3 Qe7 9. Nf4

			variation => N__('French: Chatard-Alekhine, 6...Bxg5 7.hxg5 Qxg5 8. Nh3 Qe7 9. Nf4'),
			moves => {
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P1P2/2N5/PPP3PP/R2QKBNR b KQkq f3 0 7' => {
			eco => 'C14f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4

			variation => N__('French: Classical, Steinitz Variation'),
			moves => {
				'a7a6' => 'rnb1k2r/1ppnqppp/p3p3/3pP3/3P1P2/2N5/PPP3PP/R2QKBNR w KQkq - 0 8',
				'e8g8' => 'rnb2rk1/pppnqppp/4p3/3pP3/3P1P2/2N5/PPP3PP/R2QKBNR w KQ - 1 8',
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P2Q1/2N5/PPP2PPP/R3KBNR b KQkq - 1 7' => {
			eco => 'C14b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. Qg4

			variation => N__('French: Classical, Pollock Variation'),
			moves => {
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2N4N/PPP2PP1/R2QKB1R w KQkq - 2 9' => {
			eco => 'C13l',
                        # TRANSLATORS:

			variation => N__('French: Chatard-Alekhine, 6...Bxg5 7.hxg5 Qxg5'),
			moves => {
				'h3f4' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P1N2/2N5/PPP2PP1/R2QKB1R b KQkq - 3 9',
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C14a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7

			variation => N__('French: Classical, 6.Bxe7 Qxe7'),
			moves => {
				'c3b5' => 'rnb1k2r/pppnqppp/4p3/1N1pP3/3P4/8/PPP2PPP/R2QKBNR b KQkq - 1 7',
				'd1d2' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2N5/PPPQ1PPP/R3KBNR b KQkq - 1 7',
				'd1g4' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P2Q1/2N5/PPP2PPP/R3KBNR b KQkq - 1 7',
				'f1d3' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2NB4/PPP2PPP/R2QK1NR b KQkq - 1 7',
				'f2f4' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P1P2/2N5/PPP3PP/R2QKBNR b KQkq f3 0 7',
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2N5/PPPQ1PPP/R3KBNR b KQkq - 1 7' => {
			eco => 'C14e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. Qd2

			variation => N__('French: Classical, Rubinstein Variation'),
			moves => {
			},
		},
		'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2NB4/PPP2PPP/R2QK1NR b KQkq - 1 7' => {
			eco => 'C14c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. Bd3

			variation => N__('French: Classical, Tarrasch Variation'),
			moves => {
			},
		},
		'rnb1k2r/pppp1pp1/4pq1p/8/1b1PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 2 6' => {
			eco => 'A45o',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6 5. Nc3 Bb4

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6 5. Nc3 Bb4'),
			moves => {
			},
		},
		'rnb1k2r/ppppnp1p/7b/4q3/2B2p2/2NP1Q2/PPP3PP/R1B2RK1 w kq - 3 10' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'c1d2' => 'rnb1k2r/ppppnp1p/7b/4q3/2B2p2/2NP1Q2/PPPB2PP/R4RK1 b kq - 4 10',
			},
		},
		'rnb1k2r/ppppnp1p/7b/4q3/2B2p2/2NP1Q2/PPPB2PP/R4RK1 b kq - 4 10' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'b8c6' => 'r1b1k2r/ppppnp1p/2n4b/4q3/2B2p2/2NP1Q2/PPPB2PP/R4RK1 w kq - 5 11',
			},
		},
		'rnb1k2r/ppppnpbp/8/6p1/2BPPp1q/2N3P1/PPP4P/R1BQ1KNR b kq - 0 7' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5 5. Nc3 Bg7 6. d4 Ne7 7. g3

			variation => N__('KGA: Bishop\'s Gambit, McDonnell Attack'),
			moves => {
			},
		},
		'rnb1k2r/ppppnpbp/8/6p1/2BPPp1q/2N5/PPP3PP/R1BQ1KNR w kq - 1 7' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5 5. Nc3 Bg7 6. d4 Ne7

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence'),
			moves => {
				'g2g3' => 'rnb1k2r/ppppnpbp/8/6p1/2BPPp1q/2N3P1/PPP4P/R1BQ1KNR b kq - 0 7',
			},
		},
		'rnb1k2r/ppppqppp/4pn2/8/1bPP4/2N2N2/PP1BPPPP/R2QKB1R b KQkq - 5 5' => {
			eco => 'E11o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. Nc3

			variation => N__('Bogo-Indian: Nimzowitsch, 5. Nc3'),
			moves => {
			},
		},
		'rnb1k2r/ppppqppp/4pn2/8/1bPP4/5N2/PP1BPPPP/RN1QKB1R w KQkq - 4 5' => {
			eco => 'E11n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7

			variation => N__('Bogo-Indian: Nimzowitsch Variation'),
			moves => {
				'b1c3' => 'rnb1k2r/ppppqppp/4pn2/8/1bPP4/2N2N2/PP1BPPPP/R2QKB1R b KQkq - 5 5',
				'g2g3' => 'rnb1k2r/ppppqppp/4pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnb1k2r/ppppqppp/4pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 0 5' => {
			eco => 'E11p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3'),
			moves => {
				'b8c6' => 'r1b1k2r/ppppqppp/2n1pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R w KQkq - 1 6',
				'e8g8' => 'rnb2rk1/ppppqppp/4pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R w KQ - 1 6',
			},
		},
		'rnb1k2r/ppppqppp/4pn2/8/1bPP4/6P1/PP1BPP1P/RN1QKBNR w KQkq - 3 5' => {
			eco => 'E00m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+ 4. Bd2 Qe7

			variation => N__('Catalan: 3... Bb4+ 4. Bd2 Qe7'),
			moves => {
				'f1g2' => 'rnb1k2r/ppppqppp/4pn2/8/1bPP4/6P1/PP1BPPBP/RN1QK1NR b KQkq - 4 5',
			},
		},
		'rnb1k2r/ppppqppp/4pn2/8/1bPP4/6P1/PP1BPPBP/RN1QK1NR b KQkq - 4 5' => {
			eco => 'E00m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+ 4. Bd2 Qe7 5. Bg2

			variation => N__('Catalan: 3... Bb4+ 4. Bd2 Qe7'),
			moves => {
			},
		},
		'rnb1k2r/ppq1npQp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1B1KBNR b KQkq - 0 8' => {
			eco => 'C18r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7 8. Qxg7

			variation => N__('French: Winawer, Poisoned Pawn, 8.Qxg7'),
			moves => {
				'h8g8' => 'rnb1k1r1/ppq1npQp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1B1KBNR w KQq - 1 9',
			},
		},
		'rnb1k2r/ppq1nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 3 8' => {
			eco => 'C18r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Qc7

			variation => N__('French: Winawer, Poisoned Pawn'),
			moves => {
				'g4g7' => 'rnb1k2r/ppq1npQp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1B1KBNR b KQkq - 0 8',
			},
		},
		'rnb1k2r/ppq1nppp/4p3/2ppP3/3P3P/P1P2N2/2P2PP1/R1BQKB1R b KQkq - 2 8' => {
			eco => 'C18h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. h4 Qc7 8. Nf3

			variation => N__('French: Winawer, 6... Ne7 7. h4 Qc7'),
			moves => {
			},
		},
		'rnb1k2r/ppq1nppp/4p3/2ppP3/3P3P/P1P5/2P2PP1/R1BQKBNR w KQkq - 1 8' => {
			eco => 'C18h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. h4 Qc7

			variation => N__('French: Winawer, 6... Ne7 7. h4 Qc7'),
			moves => {
				'g1f3' => 'rnb1k2r/ppq1nppp/4p3/2ppP3/3P3P/P1P2N2/2P2PP1/R1BQKB1R b KQkq - 2 8',
			},
		},
		'rnb1k2r/ppq1nppp/4p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8' => {
			eco => 'C19d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Qc7

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Qc7'),
			moves => {
				'a3a4' => 'rnb1k2r/ppq1nppp/4p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rnb1k2r/ppq1nppp/4p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'C19e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 Qc7 8. a4

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 Qc7 8. a4'),
			moves => {
				'b7b6' => 'rnb1k2r/p1q1nppp/1p2p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R w KQkq - 0 9',
			},
		},
		'rnb1k2r/ppqp1ppp/4pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'E38g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Qc7

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Qc7'),
			moves => {
			},
		},
		'rnb1kb1r/1p1n1ppp/p3p3/4P1B1/2BN4/q1N5/P1PQ2PP/1R2K2R b Kkq - 2 12' => {
			eco => 'B97p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. e5 dxe5 11. fxe5 Nfd7 12. Bc4

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. e5 dxe5 11.fxe5 Nfd7 12. Bc4'),
			moves => {
				'f8b4' => 'rnb1k2r/1p1n1ppp/p3p3/4P1B1/1bBN4/q1N5/P1PQ2PP/1R2K2R w Kkq - 3 13',
			},
		},
		'rnb1kb1r/1p1n1ppp/p3p3/4P1B1/3N4/q1N5/P1PQ2PP/1R2KB1R w Kkq - 1 12' => {
			eco => 'B97p',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. e5 dxe5'),
			moves => {
				'f1c4' => 'rnb1kb1r/1p1n1ppp/p3p3/4P1B1/2BN4/q1N5/P1PQ2PP/1R2K2R b Kkq - 2 12',
			},
		},
		'rnb1kb1r/1p3p1p/p2ppp2/8/3NPP2/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 11' => {
			eco => 'B97o',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10.Bxf6'),
			moves => {
				'f1e2' => 'rnb1kb1r/1p3p1p/p2ppp2/8/3NPP2/q1N5/P1PQB1PP/1R2K2R b Kkq - 1 11',
			},
		},
		'rnb1kb1r/1p3p1p/p2ppp2/8/3NPP2/q1N5/P1PQB1PP/1R2K2R b Kkq - 1 11' => {
			eco => 'B97o',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10.Bxf6'),
			moves => {
				'b8c6' => 'r1b1kb1r/1p3p1p/p1nppp2/8/3NPP2/q1N5/P1PQB1PP/1R2K2R w Kkq - 2 12',
				'f8g7' => 'rnb1k2r/1p3pbp/p2ppp2/8/3NPP2/q1N5/P1PQB1PP/1R2K2R w Kkq - 2 12',
			},
		},
		'rnb1kb1r/1p3pp1/pq1ppn1p/8/3NPP1B/2N5/PPP3PP/R2QKB1R w KQkq - 2 9' => {
			eco => 'B96b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 h6 8. Bh4 Qb6

			variation => N__('Sicilian: Najdorf, Delayed Poisoned Pawn'),
			moves => {
			},
		},
		'rnb1kb1r/1p3ppp/p2ppB2/8/3NPP2/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 10' => {
			eco => 'B97o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. Bxf6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10.Bxf6'),
			moves => {
				'g7f6' => 'rnb1kb1r/1p3p1p/p2ppp2/8/3NPP2/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 11',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppB2/8/4PP2/qNN5/P1PQ2PP/R3KB1R b KQkq - 0 10' => {
			eco => 'B97l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Nb3 Qa3 10. Bxf6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 9. Nb3 Qa3 10.Bxf6'),
			moves => {
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/4P1B1/3N1P2/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 10' => {
			eco => 'B97p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. e5

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. e5'),
			moves => {
				'd6e5' => 'rnb1kb1r/1p3ppp/p3pn2/4p1B1/3N1P2/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 11',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/5PB1/3NP3/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 10' => {
			eco => 'B97q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. f5

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. f5'),
			moves => {
				'b8c6' => 'r1b1kb1r/1p3ppp/p1nppn2/5PB1/3NP3/q1N5/P1PQ2PP/1R2KB1R w Kkq - 1 11',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/2N5/PqPQ2PP/1R2KB1R b Kkq - 1 9' => {
			eco => 'B97m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 9. Rb1'),
			moves => {
				'b2a3' => 'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/q1N5/P1PQ2PP/1R2KB1R w Kkq - 2 10',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/2N5/PqPQ2PP/R3KB1R w KQkq - 0 9' => {
			eco => 'B97i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2

			variation => N__('Sicilian: Najdorf, Poisoned Pawn Accepted'),
			moves => {
				'a1b1' => 'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/2N5/PqPQ2PP/1R2KB1R b Kkq - 1 9',
				'd4b3' => 'rnb1kb1r/1p3ppp/p2ppn2/6B1/4PP2/1NN5/PqPQ2PP/R3KB1R b KQkq - 1 9',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/q1N5/P1PQ2PP/1R2KB1R w Kkq - 2 10' => {
			eco => 'B97m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 9. Rb1 Qa3'),
			moves => {
				'e4e5' => 'rnb1kb1r/1p3ppp/p2ppn2/4P1B1/3N1P2/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 10',
				'f1e2' => 'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R2K2R b Kkq - 3 10',
				'f4f5' => 'rnb1kb1r/1p3ppp/p2ppn2/5PB1/3NP3/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 10',
				'g5f6' => 'rnb1kb1r/1p3ppp/p2ppB2/8/3NPP2/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 10',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R2K2R b Kkq - 3 10' => {
			eco => 'B97n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. Be2

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. Be2'),
			moves => {
				'f8e7' => 'rnb1k2r/1p2bppp/p2ppn2/6B1/3NPP2/q1N5/P1PQB1PP/1R2K2R w Kkq - 4 11',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/6B1/4PP2/1NN5/PqPQ2PP/R3KB1R b KQkq - 1 9' => {
			eco => 'B97j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Nb3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 9. Nb3'),
			moves => {
				'b2a3' => 'rnb1kb1r/1p3ppp/p2ppn2/6B1/4PP2/qNN5/P1PQ2PP/R3KB1R w KQkq - 2 10',
				'b8c6' => 'r1b1kb1r/1p3ppp/p1nppn2/6B1/4PP2/1NN5/PqPQ2PP/R3KB1R w KQkq - 2 10',
			},
		},
		'rnb1kb1r/1p3ppp/p2ppn2/6B1/4PP2/qNN5/P1PQ2PP/R3KB1R w KQkq - 2 10' => {
			eco => 'B97k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Nb3 Qa3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 9. Nb3 Qa3'),
			moves => {
				'g5f6' => 'rnb1kb1r/1p3ppp/p2ppB2/8/4PP2/qNN5/P1PQ2PP/R3KB1R b KQkq - 0 10',
			},
		},
		'rnb1kb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP3PPP/RNBR2K1 b kq - 0 8' => {
			eco => 'D27m',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Spassky, 7...Qxd1'),
			moves => {
				'f8c5' => 'rnb1k2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP3PPP/RNBR2K1 w kq - 0 9',
			},
		},
		'rnb1kb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP3PPP/RNBq1RK1 w kq - 0 8' => {
			eco => 'D27m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. dxc5 Qxd1

			variation => N__('QGA: Classical, Spassky, 7...Qxd1'),
			moves => {
				'f1d1' => 'rnb1kb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP3PPP/RNBR2K1 b kq - 0 8',
			},
		},
		'rnb1kb1r/1p3ppp/p3pn2/4P1B1/3N4/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 11' => {
			eco => 'B97p',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. e5 dxe5'),
			moves => {
				'f6d7' => 'rnb1kb1r/1p1n1ppp/p3p3/4P1B1/3N4/q1N5/P1PQ2PP/1R2KB1R w Kkq - 1 12',
			},
		},
		'rnb1kb1r/1p3ppp/p3pn2/4p1B1/3N1P2/q1N5/P1PQ2PP/1R2KB1R w Kkq - 0 11' => {
			eco => 'B97p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2 Qxb2 9. Rb1 Qa3 10. e5 dxe5

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 10. e5 dxe5'),
			moves => {
				'f4e5' => 'rnb1kb1r/1p3ppp/p3pn2/4P1B1/3N4/q1N5/P1PQ2PP/1R2KB1R b Kkq - 0 11',
			},
		},
		'rnb1kb1r/1p3ppp/pq1ppB2/8/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B97b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Bxf6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8.Bxf6'),
			moves => {
			},
		},
		'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B97a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6

			variation => N__('Sicilian: Najdorf, Poisoned Pawn'),
			moves => {
				'a2a3' => 'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/P1N5/1PP3PP/R2QKB1R b KQkq - 0 8',
				'd1d2' => 'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/2N5/PPPQ2PP/R3KB1R b KQkq - 2 8',
				'd1d3' => 'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/2NQ4/PPP3PP/R3KB1R b KQkq - 2 8',
				'd4b3' => 'rnb1kb1r/1p3ppp/pq1ppn2/6B1/4PP2/1NN5/PPP3PP/R2QKB1R b KQkq - 2 8',
				'g5f6' => 'rnb1kb1r/1p3ppp/pq1ppB2/8/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/2N5/PPPQ2PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B97i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd2

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Qd2'),
			moves => {
				'b6b2' => 'rnb1kb1r/1p3ppp/p2ppn2/6B1/3NPP2/2N5/PqPQ2PP/R3KB1R w KQkq - 0 9',
			},
		},
		'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/2NQ4/PPP3PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B97d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Qd3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Qd3'),
			moves => {
			},
		},
		'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/P1N5/1PP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B97c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. a3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. a3'),
			moves => {
			},
		},
		'rnb1kb1r/1p3ppp/pq1ppn2/6B1/4PP2/1NN5/PPP3PP/R2QKB1R b KQkq - 2 8' => {
			eco => 'B97e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qb6 8. Nb3

			variation => N__('Sicilian: Najdorf, Poisoned Pawn, 8. Nb3'),
			moves => {
				'b8d7' => 'r1b1kb1r/1p1n1ppp/pq1ppn2/6B1/4PP2/1NN5/PPP3PP/R2QKB1R w KQkq - 3 9',
				'f8e7' => 'rnb1k2r/1p2bppp/pq1ppn2/6B1/4PP2/1NN5/PPP3PP/R2QKB1R w KQkq - 3 9',
			},
		},
		'rnb1kb1r/1pp1pppp/p2q1n2/8/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B01d',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qd6 4. d4 Nf6 5. Nf3 a6

			variation => N__('Scandinavian: Schiller, Bronstein Variation'),
			moves => {
			},
		},
		'rnb1kb1r/1pq1pp1p/p2p1np1/8/3NPP2/2NB4/PPP3PP/R1BQK2R w KQkq - 0 8' => {
			eco => 'B93g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Qc7 7. Bd3 g6

			variation => N__('Sicilian: Najdorf, 6. f4 Qc7 7. Bd3 g6'),
			moves => {
			},
		},
		'rnb1kb1r/1pq1pppp/p2p1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B93e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Qc7

			variation => N__('Sicilian: Najdorf, 6. f4 Qc7'),
			moves => {
				'd4f3' => 'rnb1kb1r/1pq1pppp/p2p1n2/8/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 2 7',
				'f1d3' => 'rnb1kb1r/1pq1pppp/p2p1n2/8/3NPP2/2NB4/PPP3PP/R1BQK2R b KQkq - 2 7',
			},
		},
		'rnb1kb1r/1pq1pppp/p2p1n2/8/3NPP2/2NB4/PPP3PP/R1BQK2R b KQkq - 2 7' => {
			eco => 'B93f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Qc7 7. Bd3

			variation => N__('Sicilian: Najdorf, 6. f4 Qc7 7. Bd3'),
			moves => {
				'g7g6' => 'rnb1kb1r/1pq1pp1p/p2p1np1/8/3NPP2/2NB4/PPP3PP/R1BQK2R w KQkq - 0 8',
			},
		},
		'rnb1kb1r/1pq1pppp/p2p1n2/8/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'B93h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 Qc7 7. Nf3

			variation => N__('Sicilian: Najdorf, 6. f4 Qc7 7. Nf3'),
			moves => {
				'b8d7' => 'r1b1kb1r/1pqnpppp/p2p1n2/8/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 3 8',
			},
		},
		'rnb1kb1r/1pq2ppp/p2p1n2/4p3/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B93l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Qc7

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Qc7'),
			moves => {
				'a2a4' => 'rnb1kb1r/1pq2ppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R b KQkq a3 0 8',
			},
		},
		'rnb1kb1r/1pq2ppp/p2p1n2/4p3/P3PP2/2N2N2/1PP3PP/R1BQKB1R b KQkq a3 0 8' => {
			eco => 'B93m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3 Qc7 8. a4

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3 Qc7 8. a4'),
			moves => {
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppB2/8/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B96f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7 8. Bxf6

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7 8.Bxf6'),
			moves => {
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B96g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7 8. Qf3

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7 8. Qf3'),
			moves => {
				'b7b5' => 'rnb1kb1r/2q2ppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq b6 0 9',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B96f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7'),
			moves => {
				'd1f3' => 'rnb1kb1r/1pq2ppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 2 8',
				'g5f6' => 'rnb1kb1r/1pq2ppp/p2ppB2/8/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/2PNP3/3B4/PP2QPPP/RNB2RK1 b kq c3 0 8' => {
			eco => 'B42o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O Qc7 7. Qe2 d6 8. c4

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O Qc7 7. Qe2 d6 8. c4'),
			moves => {
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 3 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. Be3

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. Be3'),
			moves => {
				'b8c6' => 'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 w kq - 4 9',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N2B2/PPP2PPP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. Bf3

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. Bf3'),
			moves => {
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K b kq - 3 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. Kh1

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. Kh1'),
			moves => {
				'b8c6' => 'r1b1kb1r/1pq2ppp/p1nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K w kq - 4 9',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7'),
			moves => {
				'a2a4' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/P2NP3/2N5/1PP1BPPP/R1BQ1RK1 b kq a3 0 8',
				'c1e3' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b kq - 3 8',
				'e2f3' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N2B2/PPP2PPP/R1BQ1RK1 b kq - 3 8',
				'f2f4' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 b kq f3 0 8',
				'g1h1' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1R1K b kq - 3 8',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/3B4/PPP1QPPP/RNB2RK1 w kq - 0 8' => {
			eco => 'B42n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O Qc7 7. Qe2 d6

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O Qc7 7. Qe2'),
			moves => {
				'c2c4' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/2PNP3/3B4/PP2QPPP/RNB2RK1 b kq c3 0 8',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 b kq f3 0 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. f4

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. f4'),
			moves => {
				'b8c6' => 'r1b1kb1r/1pq2ppp/p1nppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 w kq - 1 9',
			},
		},
		'rnb1kb1r/1pq2ppp/p2ppn2/8/P2NP3/2N5/1PP1BPPP/R1BQ1RK1 b kq a3 0 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Qc7 8. a4

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Qc7 8. a4'),
			moves => {
				'b8c6' => 'r1b1kb1r/1pq2ppp/p1nppn2/8/P2NP3/2N5/1PP1BPPP/R1BQ1RK1 w kq - 1 9',
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/2PNP3/2N5/PP2BPPP/R1BQK2R b KQkq - 4 7' => {
			eco => 'B41m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6 6. Nc3 Qc7 7. Be2

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3 Qc7 7. Be2'),
			moves => {
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'B41l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6 6. Nc3 Qc7

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3 Qc7'),
			moves => {
				'f1e2' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/2PNP3/2N5/PP2BPPP/R1BQK2R b KQkq - 4 7',
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/2PNP3/3B4/PP3PPP/RNBQ1RK1 b kq c3 0 7' => {
			eco => 'B42m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O Qc7 7. c4

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O Qc7 7. c4'),
			moves => {
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B43i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. g3 Nf6

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. g3 Nf6'),
			moves => {
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 5 7' => {
			eco => 'B43l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. Be2 Nf6 7. O-O

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. Be2 Nf6 7.O-O'),
			moves => {
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 4 7' => {
			eco => 'B43k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. Be2 Nf6

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. Be2 Nf6'),
			moves => {
				'e1g1' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 5 7',
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2NB4/PPP2PPP/R1BQ1RK1 b kq - 5 7' => {
			eco => 'B43o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. Bd3 Nf6 7. O-O

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. Bd3 Nf6 7.O-O'),
			moves => {
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2NB4/PPP2PPP/R1BQK2R w KQkq - 4 7' => {
			eco => 'B43n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. Bd3 Nf6

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. Bd3 Nf6'),
			moves => {
				'e1g1' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2NB4/PPP2PPP/R1BQ1RK1 b kq - 5 7',
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/3B4/PPP1QPPP/RNB2RK1 b kq - 5 7' => {
			eco => 'B42n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O Qc7 7. Qe2

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O Qc7 7. Qe2'),
			moves => {
				'd7d6' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/3B4/PPP1QPPP/RNB2RK1 w kq - 0 8',
			},
		},
		'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/3B4/PPP2PPP/RNBQ1RK1 w kq - 4 7' => {
			eco => 'B42l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O Qc7

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O Qc7'),
			moves => {
				'c2c4' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/2PNP3/3B4/PP3PPP/RNBQ1RK1 b kq c3 0 7',
				'd1e2' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/3B4/PPP1QPPP/RNB2RK1 b kq - 5 7',
			},
		},
		'rnb1kb1r/2p1pppp/p7/1p1qP3/P1pP4/2P2N2/5PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'D24s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 Nxc3 8. bxc3 Qd5

			variation => N__('QGA: Bogoljubow, 7. a4 Nxc3'),
			moves => {
				'g2g3' => 'rnb1kb1r/2p1pppp/p7/1p1qP3/P1pP4/2P2NP1/5P1P/R1BQKB1R b KQkq - 0 9',
			},
		},
		'rnb1kb1r/2p1pppp/p7/1p1qP3/P1pP4/2P2NP1/5P1P/R1BQKB1R b KQkq - 0 9' => {
			eco => 'D24s',
                        # TRANSLATORS:

			variation => N__('QGA: Bogoljubow, 7. a4 Nxc3'),
			moves => {
				'c8b7' => 'rn2kb1r/1bp1pppp/p7/1p1qP3/P1pP4/2P2NP1/5P1P/R1BQKB1R w KQkq - 1 10',
			},
		},
		'rnb1kb1r/2q2ppp/p2ppB2/1p6/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 9' => {
			eco => 'B96h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7 8. Qf3 b5 9. Bxf6

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7 8. Qf3 b5 9.Bxf6'),
			moves => {
			},
		},
		'rnb1kb1r/2q2ppp/p2ppn2/1p3PB1/3NP3/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 9' => {
			eco => 'B96h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7 8. Qf3 b5 9. f5

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7 8. Qf3 b5 9. f5'),
			moves => {
			},
		},
		'rnb1kb1r/2q2ppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 1 9' => {
			eco => 'B96h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7 8. Qf3 b5 9. O-O-O

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7 8. Qf3 b5 9.O-O-O'),
			moves => {
			},
		},
		'rnb1kb1r/2q2ppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq b6 0 9' => {
			eco => 'B96h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Qc7 8. Qf3 b5

			variation => N__('Sicilian: Najdorf, 7. f4 Qc7 8. Qf3 b5'),
			moves => {
				'e1c1' => 'rnb1kb1r/2q2ppp/p2ppn2/1p4B1/3NPP2/2N2Q2/PPP3PP/2KR1B1R b kq - 1 9',
				'f4f5' => 'rnb1kb1r/2q2ppp/p2ppn2/1p3PB1/3NP3/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 9',
				'g5f6' => 'rnb1kb1r/2q2ppp/p2ppB2/1p6/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 9',
			},
		},
		'rnb1kb1r/2q2ppp/p3pP2/1p4B1/3N4/2N5/PPP3PP/R2QKB1R b KQkq - 0 10' => {
			eco => 'B96q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. exf6

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10.exf6'),
			moves => {
				'c7e5' => 'rnb1kb1r/5ppp/p3pP2/1p2q1B1/3N4/2N5/PPP3PP/R2QKB1R w KQkq - 1 11',
			},
		},
		'rnb1kb1r/2q2ppp/p3pn2/1B2P1B1/3N4/2N5/PPP3PP/R2QK2R b KQkq - 0 10' => {
			eco => 'B96p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Bxb5+

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10.Bxb5+'),
			moves => {
			},
		},
		'rnb1kb1r/2q2ppp/p3pn2/1p2P1B1/3N4/2N5/PPP1Q1PP/R3KB1R b KQkq - 2 10' => {
			eco => 'B96t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Qe2

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2'),
			moves => {
				'f6d7' => 'rnb1kb1r/2qn1ppp/p3p3/1p2P1B1/3N4/2N5/PPP1Q1PP/R3KB1R w KQkq - 3 11',
			},
		},
		'rnb1kb1r/2q2ppp/p3pn2/1p2P1B1/3N4/2N5/PPP3PP/R2QKB1R w KQkq - 1 10' => {
			eco => 'B96n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7

			variation => N__('Sicilian: Najdorf, Polugaevsky, 9.fxe5 Qc7'),
			moves => {
				'd1e2' => 'rnb1kb1r/2q2ppp/p3pn2/1p2P1B1/3N4/2N5/PPP1Q1PP/R3KB1R b KQkq - 2 10',
				'd4f3' => 'rnb1kb1r/2q2ppp/p3pn2/1p2P1B1/8/2N2N2/PPP3PP/R2QKB1R b KQkq - 2 10',
				'e5f6' => 'rnb1kb1r/2q2ppp/p3pP2/1p4B1/3N4/2N5/PPP3PP/R2QKB1R b KQkq - 0 10',
				'f1b5' => 'rnb1kb1r/2q2ppp/p3pn2/1B2P1B1/3N4/2N5/PPP3PP/R2QK2R b KQkq - 0 10',
			},
		},
		'rnb1kb1r/2q2ppp/p3pn2/1p2P1B1/8/2N2N2/PPP3PP/R2QKB1R b KQkq - 2 10' => {
			eco => 'B96o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. Nf3

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Nf3'),
			moves => {
			},
		},
		'rnb1kb1r/2qn1ppp/p3p3/1p2P1B1/3N4/2N5/PPP1Q1PP/2KR1B1R b kq - 4 11' => {
			eco => 'B96t',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2'),
			moves => {
				'b8c6' => 'r1b1kb1r/2qn1ppp/p1n1p3/1p2P1B1/3N4/2N5/PPP1Q1PP/2KR1B1R w kq - 5 12',
				'c8b7' => 'rn2kb1r/1bqn1ppp/p3p3/1p2P1B1/3N4/2N5/PPP1Q1PP/2KR1B1R w kq - 5 12',
			},
		},
		'rnb1kb1r/2qn1ppp/p3p3/1p2P1B1/3N4/2N5/PPP1Q1PP/R3KB1R w KQkq - 3 11' => {
			eco => 'B96t',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10. Qe2'),
			moves => {
				'e1c1' => 'rnb1kb1r/2qn1ppp/p3p3/1p2P1B1/3N4/2N5/PPP1Q1PP/2KR1B1R b kq - 4 11',
			},
		},
		'rnb1kb1r/3ppppp/p4n2/qPpP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 6' => {
			eco => 'A57r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 Qa5

			variation => N__('Benko Gambit: Zaitsev, 5... Qa5'),
			moves => {
			},
		},
		'rnb1kb1r/3ppppp/pq3n2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A57j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. b6 Qxb6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. b6 Qxb6'),
			moves => {
			},
		},
		'rnb1kb1r/5ppp/p3pP2/1p2q1B1/3N4/2N5/PPP1B1PP/R2QK2R b KQkq - 2 11' => {
			eco => 'B96r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. exf6 Qe5+ 11. Be2

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10.exf6 Qe5+ 11. Be2'),
			moves => {
				'e5g5' => 'rnb1kb1r/5ppp/p3pP2/1p4q1/3N4/2N5/PPP1B1PP/R2QK2R w KQkq - 0 12',
			},
		},
		'rnb1kb1r/5ppp/p3pP2/1p2q1B1/3N4/2N5/PPP3PP/R2QKB1R w KQkq - 1 11' => {
			eco => 'B96q',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10.exf6'),
			moves => {
				'f1e2' => 'rnb1kb1r/5ppp/p3pP2/1p2q1B1/3N4/2N5/PPP1B1PP/R2QK2R b KQkq - 2 11',
			},
		},
		'rnb1kb1r/5ppp/p3pP2/1p4q1/3N4/2N5/PPP1B1PP/R2QK2R w KQkq - 0 12' => {
			eco => 'B96r',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10.exf6 Qe5+ 11. Be2'),
			moves => {
				'd1d3' => 'rnb1kb1r/5ppp/p3pP2/1p4q1/3N4/2NQ4/PPP1B1PP/R3K2R b KQkq - 1 12',
			},
		},
		'rnb1kb1r/5ppp/p3pP2/1p4q1/3N4/2NQ4/PPP1B1PP/R3K2R b KQkq - 1 12' => {
			eco => 'B96s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5 dxe5 9. fxe5 Qc7 10. exf6 Qe5+ 11. Be2 Qxg5 12. Qd3

			variation => N__('Sicilian: Najdorf, Polugaevsky, 10.exf6 Qe5+ 11. Be2 Qxg5 12. Qd3'),
			moves => {
			},
		},
		'rnb1kb1r/p1pp1pp1/1p2pq1p/8/3PP3/5N2/PPP2PPP/RN1QKB1R w KQkq - 0 6' => {
			eco => 'A46y',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 b6

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 b6'),
			moves => {
			},
		},
		'rnb1kb1r/p2p1ppp/4pn2/qppP2B1/2P5/5N2/PP2PPPP/RN1QKB1R w KQkq - 2 6' => {
			eco => 'E10o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. Bg5 Qa5+

			variation => N__('Blumenfeld: 5. Bg5 Qa5+'),
			moves => {
			},
		},
		'rnb1kb1r/pp1p1pp1/4pq1p/2p5/3PP3/5N2/PPP2PPP/RN1QKB1R w KQkq c6 0 6' => {
			eco => 'A46y',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 c5

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 c5'),
			moves => {
			},
		},
		'rnb1kb1r/pp1p1ppp/1q2pn2/2p3B1/3P4/2P2N2/PP2PPPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'A46t',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. c3 Qb6

			variation => N__('Torre Attack: 3... c5 4. c3 Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp1p1ppp/1q2pn2/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'A46u',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e3 Qb6

			variation => N__('Torre Attack: 3... c5 4. e3 Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp1p1ppp/1q2pn2/2p5/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'A46p',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bf4 c5 4. e3 Qb6

			variation => N__('Indian: London, 3... c5 4. e3 Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp1p1ppp/1q2pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 Qb6

			variation => N__('English: Symmetrical, Two Knights, 5. g3 Qb6'),
			moves => {
				'f1g2' => 'rnb1kb1r/pp1p1ppp/1q2pn2/8/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'rnb1kb1r/pp1p1ppp/1q2pn2/8/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 Qb6 6. Bg2

			variation => N__('English: Symmetrical, Two Knights, 5. g3 Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp1p1ppp/4pn2/q1p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A46o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 Qa5+

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 Qa5+'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppp1p/1q3p2/2pP4/8/8/PPP1PPPP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6'),
			moves => {
				'd1c1' => 'rnb1kb1r/pp1ppp1p/1q3p2/2pP4/8/8/PPP1PPPP/RNQ1KBNR b KQkq - 2 5',
			},
		},
		'rnb1kb1r/pp1ppp1p/1q3p2/2pP4/8/8/PPP1PPPP/RNQ1KBNR b KQkq - 2 5' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6 5. Qc1

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6 5. Qc1'),
			moves => {
				'f6f5' => 'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/8/8/PPP1PPPP/RNQ1KBNR w KQkq - 0 6',
			},
		},
		'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/2P5/8/PP2PPPP/RNQ1KBNR b KQkq c3 0 6' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. c4

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. c4'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/8/4P3/PPP2PPP/RNQ1KBNR b KQkq - 0 6' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. e3

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. e3'),
			moves => {
				'f8g7' => 'rnb1k2r/pp1pppbp/1q6/2pP1p2/8/4P3/PPP2PPP/RNQ1KBNR w KQkq - 1 7',
			},
		},
		'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/8/6P1/PPP1PP1P/RNQ1KBNR b KQkq - 0 6' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. g3

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5 6. g3'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/8/8/PPP1PPPP/RNQ1KBNR w KQkq - 0 6' => {
			eco => 'A45r',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5 Qb6 5. Qc1 f5'),
			moves => {
				'c2c4' => 'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/2P5/8/PP2PPPP/RNQ1KBNR b KQkq c3 0 6',
				'e2e3' => 'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/8/4P3/PPP2PPP/RNQ1KBNR b KQkq - 0 6',
				'g2g3' => 'rnb1kb1r/pp1ppp1p/1q6/2pP1p2/8/6P1/PPP1PP1P/RNQ1KBNR b KQkq - 0 6',
			},
		},
		'rnb1kb1r/pp1ppppp/1q3n2/2pP2B1/8/2N5/PPP1PPPP/R2QKBNR b KQkq - 2 4' => {
			eco => 'A45p',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. d5 Qb6 4. Nc3

			variation => N__('Trompowsky: 2... c5 3. d5 Qb6 4. Nc3'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppppp/1q3n2/2pP2B1/8/8/PPP1PPPP/RN1QKBNR w KQkq - 1 4' => {
			eco => 'A45p',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. d5 Qb6

			variation => N__('Trompowsky: 2... c5 3. d5 Qb6'),
			moves => {
				'b1c3' => 'rnb1kb1r/pp1ppppp/1q3n2/2pP2B1/8/2N5/PPP1PPPP/R2QKBNR b KQkq - 2 4',
			},
		},
		'rnb1kb1r/pp1ppppp/1q6/2pP4/4nB2/8/PPP1PPPP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'A45x',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5 4. d5 Qb6

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. d5 Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppppp/5n2/q1p5/3P1B2/2P2P2/PP1NP1PP/R2QKBNR b KQkq - 2 6' => {
			eco => 'A45z',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5 4. f3 Qa5+ 5. c3 Nf6 6. Nd2

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. f3 Qa5+ 5. c3 Nf6 6. Nd2'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppppp/5n2/q1p5/3P1B2/2P2P2/PP2P1PP/RN1QKBNR w KQkq - 1 6' => {
			eco => 'A45y',
                        # TRANSLATORS:

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. f3 Qa5+'),
			moves => {
				'b1d2' => 'rnb1kb1r/pp1ppppp/5n2/q1p5/3P1B2/2P2P2/PP1NP1PP/R2QKBNR b KQkq - 2 6',
				'd4d5' => 'rnb1kb1r/pp1ppppp/5n2/q1pP4/5B2/2P2P2/PP2P1PP/RN1QKBNR b KQkq - 0 6',
			},
		},
		'rnb1kb1r/pp1ppppp/5n2/q1pP4/5B2/2P2P2/PP2P1PP/RN1QKBNR b KQkq - 0 6' => {
			eco => 'A45y',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5 4. f3 Qa5+ 5. c3 Nf6 6. d5

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. f3 Qa5+ 5. c3 Nf6 6. d5'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppppp/5n2/q1pP4/8/2N5/PPP1PPPP/R1BQKBNR w KQkq - 3 4' => {
			eco => 'A43k',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nc3 Qa5

			variation => N__('Old Benoni: Woozle'),
			moves => {
			},
		},
		'rnb1kb1r/pp1ppppp/8/q1p5/3PnB2/2P2P2/PP2P1PP/RN1QKBNR b KQkq - 0 5' => {
			eco => 'A45y',
                        # TRANSLATORS:

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. f3 Qa5+'),
			moves => {
				'e4f6' => 'rnb1kb1r/pp1ppppp/5n2/q1p5/3P1B2/2P2P2/PP2P1PP/RN1QKBNR w KQkq - 1 6',
			},
		},
		'rnb1kb1r/pp1ppppp/8/q1p5/3PnB2/5P2/PPP1P1PP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'A45y',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5 4. f3 Qa5+

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. f3 Qa5+'),
			moves => {
				'c2c3' => 'rnb1kb1r/pp1ppppp/8/q1p5/3PnB2/2P2P2/PP2P1PP/RN1QKBNR b KQkq - 0 5',
			},
		},
		'rnb1kb1r/pp2pppp/1qp2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'D15c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 Qb6

			variation => N__('Slav: Süchting Variation'),
			moves => {
			},
		},
		'rnb1kb1r/pp2pppp/2p2n2/q7/2BP4/2N2N2/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B01n',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6 6. Bc4

			variation => N__('Scandinavian: 2...Qxd5, Main Line, 6. Bc4'),
			moves => {
				'c8f5' => 'rn2kb1r/pp2pppp/2p2n2/q4b2/2BP4/2N2N2/PPP2PPP/R1BQK2R w KQkq - 2 7',
				'c8g4' => 'rn2kb1r/pp2pppp/2p2n2/q7/2BP2b1/2N2N2/PPP2PPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnb1kb1r/pp2pppp/2p2n2/q7/2BP4/2N5/PPP2PPP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'B01i',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Bc4 c6

			variation => N__('Scandinavian: 2...Qxd5, 5. Bc4 c6'),
			moves => {
			},
		},
		'rnb1kb1r/pp2pppp/2p2n2/q7/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B01m',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3 c6

			variation => N__('Scandinavian: 2...Qxd5, Main Line'),
			moves => {
				'f1c4' => 'rnb1kb1r/pp2pppp/2p2n2/q7/2BP4/2N2N2/PPP2PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnb1kb1r/pp2pppp/2p2n2/q7/3P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'B01i',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Bd2 c6

			variation => N__('Scandinavian: 2...Qxd5, 5. Bd2 c6'),
			moves => {
			},
		},
		'rnb1kb1r/pp2pppp/2pp1n2/q7/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B07p',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 c6 4. f4 Qa5

			variation => N__('Pirc: Czech, 4. f4 Qa5'),
			moves => {
			},
		},
		'rnb1kb1r/pp2pppp/5n2/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'B22j',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nf6 5. Nf3

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3'),
			moves => {
				'c8g4' => 'rn2kb1r/pp2pppp/5n2/2pq4/3P2b1/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
				'e7e6' => 'rnb1kb1r/pp3ppp/4pn2/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnb1kb1r/pp2pppp/5n2/2pq4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B22j',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nf6

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6'),
			moves => {
				'g1f3' => 'rnb1kb1r/pp2pppp/5n2/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'rnb1kb1r/pp2pppp/8/2p5/4P3/2P2N2/PP3PPP/R1BK1B1R b kq - 0 7' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. dxc3 Qxd1+ 7. Kxd1

			variation => N__('English: Symmetrical, 3 Knights, Queenswap'),
			moves => {
			},
		},
		'rnb1kb1r/pp2pppp/8/2p5/4P3/2P2N2/PP3PPP/R1BqKB1R w KQkq - 0 7' => {
			eco => 'A34',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 3 Knights, 5. e4 Nxc3'),
			moves => {
				'e1d1' => 'rnb1kb1r/pp2pppp/8/2p5/4P3/2P2N2/PP3PPP/R1BK1B1R b kq - 0 7',
			},
		},
		'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/1QN2N2/PP2PPPP/R3KB1R b KQkq - 1 7' => {
			eco => 'D43q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. Qb3

			variation => N__('Semi-Slav: Moscow, Hastings Variation'),
			moves => {
			},
		},
		'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D43s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. e3

			variation => N__('Semi-Slav: Moscow, 7. e3'),
			moves => {
				'b8d7' => 'r1b1kb1r/pp1n1pp1/2p1pq1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 8',
			},
		},
		'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D43o',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Moscow, 6.Bxf6'),
			moves => {
				'd1b3' => 'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/1QN2N2/PP2PPPP/R3KB1R b KQkq - 1 7',
				'd1c2' => 'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 1 7',
				'e2e3' => 'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
				'g2g3' => 'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N2NP1/PP2PP1P/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 1 7' => {
			eco => 'D43r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. Qc2

			variation => N__('Semi-Slav: Moscow, 7. Qc2'),
			moves => {
			},
		},
		'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N2NP1/PP2PP1P/R2QKB1R b KQkq - 0 7' => {
			eco => 'D43p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6 Qxf6 7. g3

			variation => N__('Semi-Slav: Moscow, 7. g3'),
			moves => {
			},
		},
		'rnb1kb1r/pp3ppp/1q1ppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B86',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 Qb6

			variation => N__('Sicilian: Sozin-Scheveningen, 6... Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp3ppp/1q1ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Qb6

			variation => N__('Sicilian: Scheveningen, 6. f4 Qb6'),
			moves => {
			},
		},
		'rnb1kb1r/pp3ppp/1q2pn2/2pP2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. cxd5 Qb6

			variation => N__('QGD: Dutch-Peruvian, Canal (Venice) Variation'),
			moves => {
			},
		},
		'rnb1kb1r/pp3ppp/3qpn2/8/2Bp4/1N3N2/PPP2PPP/R1BQ1RK1 b kq - 5 8' => {
			eco => 'C07o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'b8c6' => 'r1b1kb1r/pp3ppp/2nqpn2/8/2Bp4/1N3N2/PPP2PPP/R1BQ1RK1 w kq - 6 9',
			},
		},
		'rnb1kb1r/pp3ppp/3qpn2/8/2Bp4/5N2/PPPN1PPP/R1BQ1RK1 w kq - 4 8' => {
			eco => 'C07o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'd2b3' => 'rnb1kb1r/pp3ppp/3qpn2/8/2Bp4/1N3N2/PPP2PPP/R1BQ1RK1 b kq - 5 8',
			},
		},
		'rnb1kb1r/pp3ppp/4pn2/2pq4/3P4/2P1BN2/PP3PPP/RN1QKB1R b KQkq - 1 6' => {
			eco => 'B22m',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nf6 5. Nf3 e6 6. Be3

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3 e6 6. Be3'),
			moves => {
			},
		},
		'rnb1kb1r/pp3ppp/4pn2/2pq4/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'B22n',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 Nf6 5. Nf3 e6 6. Be2

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3 e6 6. Be2'),
			moves => {
			},
		},
		'rnb1kb1r/pp3ppp/4pn2/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B22j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 Nf6 5. Nf3'),
			moves => {
				'c1e3' => 'rnb1kb1r/pp3ppp/4pn2/2pq4/3P4/2P1BN2/PP3PPP/RN1QKB1R b KQkq - 1 6',
				'f1e2' => 'rnb1kb1r/pp3ppp/4pn2/2pq4/3P4/2P2N2/PP2BPPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnb1kb1r/pp3ppp/4pq2/2p5/2PP4/2P5/P4PPP/R1BQKBNR w KQkq c6 0 8' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5 d4 5. exf6 dxc3 6. bxc3 Qxf6 7. d4 c5

			variation => N__('English: Mikenas, Flohr, 7. d4 c5'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1pp1p/6p1/8/4P3/2P2N2/PP3PPP/R1BK1B1R b kq - 0 7' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6 5. e4 Nxc3 6. dxc3 Qxd1+ 7. Kxd1

			variation => N__('English: Anglo-Grünfeld, Andersson-Böök, Queenswap'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1pp1p/6p1/8/4P3/2P2N2/PP3PPP/R1BqKB1R w KQkq - 0 7' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, Andersson-Böök Variation'),
			moves => {
				'e1d1' => 'rnb1kb1r/ppp1pp1p/6p1/8/4P3/2P2N2/PP3PPP/R1BK1B1R b kq - 0 7',
			},
		},
		'rnb1kb1r/ppp1pppp/3q1n2/8/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B01d',
                        # TRANSLATORS:

			variation => N__('Scandinavian: Schiller, 4. d4 Nf6'),
			moves => {
				'a7a6' => 'rnb1kb1r/1pp1pppp/p2q1n2/8/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnb1kb1r/ppp1pppp/3q1n2/8/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B01d',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qd6 4. d4 Nf6

			variation => N__('Scandinavian: Schiller, 4. d4 Nf6'),
			moves => {
				'g1f3' => 'rnb1kb1r/ppp1pppp/3q1n2/8/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. d4 Nf6

			variation => N__('Scandinavian: 2...Qxd5 3. d4 Nf6'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/3q4/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nf3 Nf6

			variation => N__('Scandinavian: 2...Qxd5 3. Nf3 Nf6'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/2B5/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 5' => {
			eco => 'B01e',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. Bc4 Nf6

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. Bc4 Nf6'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/2BP4/2N5/PPP2PPP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'B01i',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Bc4

			variation => N__('Scandinavian: 2...Qxd5, 5. Bc4'),
			moves => {
				'c7c6' => 'rnb1kb1r/pp2pppp/2p2n2/q7/2BP4/2N5/PPP2PPP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B01j',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Nf3

			variation => N__('Scandinavian: 2...Qxd5, 5. Nf3'),
			moves => {
				'c7c6' => 'rnb1kb1r/pp2pppp/2p2n2/q7/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'c8f5' => 'rn2kb1r/ppp1pppp/5n2/q4b2/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'c8g4' => 'rn2kb1r/ppp1pppp/5n2/q7/3P2b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B01i',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. d4 Nf6'),
			moves => {
				'c1d2' => 'rnb1kb1r/ppp1pppp/5n2/q7/3P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 2 5',
				'f1c4' => 'rnb1kb1r/ppp1pppp/5n2/q7/2BP4/2N5/PPP2PPP/R1BQK1NR b KQkq - 2 5',
				'g1f3' => 'rnb1kb1r/ppp1pppp/5n2/q7/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/3P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 2 5' => {
			eco => 'B01i',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 Nf6 5. Bd2

			variation => N__('Scandinavian: 2...Qxd5, 5. Bd2'),
			moves => {
				'c7c6' => 'rnb1kb1r/pp2pppp/2p2n2/q7/3P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/8/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'B01f',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. Nf3 Nf6

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. Nf3 Nf6'),
			moves => {
				'f1e2' => 'rnb1kb1r/ppp1pppp/5n2/q7/8/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 5 5',
			},
		},
		'rnb1kb1r/ppp1pppp/5n2/q7/8/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 5 5' => {
			eco => 'B01f',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. Nf3 Nf6 5. Be2

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. Nf3 Nf6 5. Be2'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1pppp/8/3q4/8/8/PPPP1PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B02h',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5 4. Nxd5 Qxd5

			variation => N__('Alekhine: Scandinavian, Exchange, 4.Nxd5'),
			moves => {
			},
		},
		'rnb1kb1r/ppp1qppp/3p1n2/6B1/8/3P1N2/PPP1QPPP/RN2KB1R b KQkq - 2 7' => {
			eco => 'C42n',
                        # TRANSLATORS:

			variation => N__('Russian Game: Cozio/Lasker'),
			moves => {
				'b8d7' => 'r1b1kb1r/pppnqppp/3p1n2/6B1/8/3P1N2/PPP1QPPP/RN2KB1R w KQkq - 3 8',
				'e7e2' => 'rnb1kb1r/ppp2ppp/3p1n2/6B1/8/3P1N2/PPP1qPPP/RN2KB1R w KQkq - 0 8',
			},
		},
		'rnb1kb1r/ppp1qppp/3p1n2/8/8/3P1N2/PPP1QPPP/RNB1KB1R w KQkq - 1 7' => {
			eco => 'C42n',
                        # TRANSLATORS:

			variation => N__('Russian Game: Cozio/Lasker'),
			moves => {
				'c1g5' => 'rnb1kb1r/ppp1qppp/3p1n2/6B1/8/3P1N2/PPP1QPPP/RN2KB1R b KQkq - 2 7',
			},
		},
		'rnb1kb1r/ppp1qppp/3p4/8/4n3/3P1N2/PPP1QPPP/RNB1KB1R b KQkq - 0 6' => {
			eco => 'C42n',
                        # TRANSLATORS:

			variation => N__('Russian Game: Cozio/Lasker'),
			moves => {
				'e4f6' => 'rnb1kb1r/ppp1qppp/3p1n2/8/8/3P1N2/PPP1QPPP/RNB1KB1R w KQkq - 1 7',
			},
		},
		'rnb1kb1r/ppp1qppp/3p4/8/4n3/5N2/PPPPQPPP/RNB1KB1R w KQkq - 2 6' => {
			eco => 'C42n',
                        # TRANSLATORS:

			variation => N__('Russian Game: Cozio/Lasker'),
			moves => {
				'd2d3' => 'rnb1kb1r/ppp1qppp/3p4/8/4n3/3P1N2/PPP1QPPP/RNB1KB1R b KQkq - 0 6',
			},
		},
		'rnb1kb1r/ppp2p1p/3p3n/4N3/2BPPppq/8/PPP3PP/RNBQ1K1R w kq - 0 8' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Ne5 Qh4+ 6. Kf1 Nh6 7. d4 d6

			variation => N__('KGA: Salvio, Anderssen Counterattack'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2p1p/5n2/3qN3/3P1ppP/2N5/PPP3P1/R1BQKB1R b KQkq - 1 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'f8b4' => 'rnb1k2r/ppp2p1p/5n2/3qN3/1b1P1ppP/2N5/PPP3P1/R1BQKB1R w KQkq - 2 9',
			},
		},
		'rnb1kb1r/ppp2p1p/5n2/3qN3/3P1ppP/8/PPP3P1/RNBQKB1R w KQkq - 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'b1c3' => 'rnb1kb1r/ppp2p1p/5n2/3qN3/3P1ppP/2N5/PPP3P1/R1BQKB1R b KQkq - 1 8',
			},
		},
		'rnb1kb1r/ppp2p2/3ppq1p/6p1/3PP3/2N5/PPPQ1PPP/R3KBNR w KQkq g6 0 7' => {
			eco => 'A45o',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6 5. Nc3 d6 6. Qd2 g5

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6 5. Nc3 d6 6. Qd2 g5'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2p2/3ppqpp/8/3PP3/2N2N2/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'A46z',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 d6 6. Nc3 g6

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 d6 6. Nc3 g6'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/2N2N2/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'A46z',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 d6 6. Nc3

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 d6 6. Nc3'),
			moves => {
				'b8d7' => 'r1b1kb1r/pppn1pp1/3ppq1p/8/3PP3/2N2N2/PPP2PPP/R2QKB1R w KQkq - 2 7',
				'g7g6' => 'rnb1kb1r/ppp2p2/3ppqpp/8/3PP3/2N2N2/PPP2PPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'A45o',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6 5. Nc3 d6

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6 5. Nc3 d6'),
			moves => {
				'd1d2' => 'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/2N5/PPPQ1PPP/R3KBNR b KQkq - 1 6',
			},
		},
		'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/2N5/PPPQ1PPP/R3KBNR b KQkq - 1 6' => {
			eco => 'A45o',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6 5. Nc3 d6 6. Qd2

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6 5. Nc3 d6 6. Qd2'),
			moves => {
				'g7g5' => 'rnb1kb1r/ppp2p2/3ppq1p/6p1/3PP3/2N5/PPPQ1PPP/R3KBNR w KQkq g6 0 7',
			},
		},
		'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/5N2/PPP2PPP/RN1QKB1R w KQkq - 0 6' => {
			eco => 'A46z',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4 d6

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4 d6'),
			moves => {
				'b1c3' => 'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/2N2N2/PPP2PPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnb1kb1r/ppp2ppp/3p1n2/6B1/8/3P1N2/PPP1qPPP/RN2KB1R w KQkq - 0 8' => {
			eco => 'C42o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. Qe2 Qe7 6. d3 Nf6 7. Bg5 Qxe2+

			variation => N__('Russian Game: 5. Qe2, Queenswap'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2ppp/4pq2/8/2P5/2P2N2/P2P1PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5 d4 5. exf6 dxc3 6. bxc3 Qxf6 7. Nf3

			variation => N__('English: Mikenas, Flohr, 7. Nf3'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2ppp/4pq2/8/2P5/2P5/P2P1PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'A18',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, Flohr, 4... d4'),
			moves => {
				'd2d4' => 'rnb1kb1r/ppp2ppp/4pq2/8/2PP4/2P5/P4PPP/R1BQKBNR b KQkq d3 0 7',
				'g1f3' => 'rnb1kb1r/ppp2ppp/4pq2/8/2P5/2P2N2/P2P1PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'rnb1kb1r/ppp2ppp/4pq2/8/2PP4/2P5/P4PPP/R1BQKBNR b KQkq d3 0 7' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5 d4 5. exf6 dxc3 6. bxc3 Qxf6 7. d4

			variation => N__('English: Mikenas, Flohr, 7. d4'),
			moves => {
				'c7c5' => 'rnb1kb1r/pp3ppp/4pq2/2p5/2PP4/2P5/P4PPP/R1BQKBNR w KQkq c6 0 8',
				'e6e5' => 'rnb1kb1r/ppp2ppp/5q2/4p3/2PP4/2P5/P4PPP/R1BQKBNR w KQkq - 0 8',
			},
		},
		'rnb1kb1r/ppp2ppp/5q2/4p3/2PP4/2P5/P4PPP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5 d4 5. exf6 dxc3 6. bxc3 Qxf6 7. d4 e5

			variation => N__('English: Mikenas, Flohr, 7. d4 e5'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2ppp/8/3NP2q/8/3P1Nn1/PPP4P/R1BQKB1R b KQkq - 0 8' => {
			eco => 'C29g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. d3 Qh4+ 6. g3 Nxg3 7. Nf3 Qh5 8. Nxd5

			variation => N__('Vienna Gambit: Wurzburger Trap'),
			moves => {
			},
		},
		'rnb1kb1r/ppp2ppp/8/3pP2q/8/2NP1Nn1/PPP4P/R1BQKB1R w KQkq - 2 8' => {
			eco => 'C29g',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford Variation'),
			moves => {
				'c3d5' => 'rnb1kb1r/ppp2ppp/8/3NP2q/8/3P1Nn1/PPP4P/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rnb1kb1r/ppp2ppp/8/3pP3/4n2q/2NP2P1/PPP4P/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C29g',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford Variation'),
			moves => {
				'e4g3' => 'rnb1kb1r/ppp2ppp/8/3pP3/7q/2NP2n1/PPP4P/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnb1kb1r/ppp2ppp/8/3pP3/4n2q/2NP4/PPP3PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'C29g',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford Variation'),
			moves => {
				'g2g3' => 'rnb1kb1r/ppp2ppp/8/3pP3/4n2q/2NP2P1/PPP4P/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnb1kb1r/ppp2ppp/8/3pP3/7q/2NP1Nn1/PPP4P/R1BQKB1R b KQkq - 1 7' => {
			eco => 'C29g',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford Variation'),
			moves => {
				'h4h5' => 'rnb1kb1r/ppp2ppp/8/3pP2q/8/2NP1Nn1/PPP4P/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rnb1kb1r/ppp2ppp/8/3pP3/7q/2NP2n1/PPP4P/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C29g',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford Variation'),
			moves => {
				'g1f3' => 'rnb1kb1r/ppp2ppp/8/3pP3/7q/2NP1Nn1/PPP4P/R1BQKB1R b KQkq - 1 7',
			},
		},
		'rnb1kb1r/pppp1p1p/7n/4N3/2B1Pppq/8/PPPP2PP/RNBQ1K1R w kq - 4 7' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Salvio Gambit'),
			moves => {
				'd2d4' => 'rnb1kb1r/pppp1p1p/7n/4N3/2BPPppq/8/PPP3PP/RNBQ1K1R b kq d3 0 7',
			},
		},
		'rnb1kb1r/pppp1p1p/7n/4N3/2BPP1pq/5p2/PPP3PP/RNBQ1K1R w kq - 0 8' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Ne5 Qh4+ 6. Kf1 Nh6 7. d4 f3

			variation => N__('KGA: Salvio, Silberschmidt Gambit'),
			moves => {
			},
		},
		'rnb1kb1r/pppp1p1p/7n/4N3/2BPPppq/8/PPP3PP/RNBQ1K1R b kq d3 0 7' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Salvio Gambit'),
			moves => {
				'd7d6' => 'rnb1kb1r/ppp2p1p/3p3n/4N3/2BPPppq/8/PPP3PP/RNBQ1K1R w kq - 0 8',
				'f4f3' => 'rnb1kb1r/pppp1p1p/7n/4N3/2BPP1pq/5p2/PPP3PP/RNBQ1K1R w kq - 0 8',
			},
		},
		'rnb1kb1r/pppp1pp1/4pq1p/8/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A46y',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... h6 4.Bxf6'),
			moves => {
				'e2e4' => 'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/5N2/PPP2PPP/RN1QKB1R b KQkq e3 0 5',
			},
		},
		'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'A45o',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6 5. Nc3

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6 5. Nc3'),
			moves => {
				'd7d6' => 'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6',
				'f8b4' => 'rnb1k2r/pppp1pp1/4pq1p/8/1b1PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 2 6',
			},
		},
		'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/2P5/PP3PPP/RN1QKBNR b KQkq - 0 5' => {
			eco => 'A45n',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6 5. c3

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6 5. c3'),
			moves => {
			},
		},
		'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/5N2/PPP2PPP/RN1QKB1R b KQkq e3 0 5' => {
			eco => 'A46y',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6 Qxf6 5. e4

			variation => N__('Torre Attack: 3... h6 4.Bxf6 Qxf6 5. e4'),
			moves => {
				'b7b6' => 'rnb1kb1r/p1pp1pp1/1p2pq1p/8/3PP3/5N2/PPP2PPP/RN1QKB1R w KQkq - 0 6',
				'c7c5' => 'rnb1kb1r/pp1p1pp1/4pq1p/2p5/3PP3/5N2/PPP2PPP/RN1QKB1R w KQkq c6 0 6',
				'd7d6' => 'rnb1kb1r/ppp2pp1/3ppq1p/8/3PP3/5N2/PPP2PPP/RN1QKB1R w KQkq - 0 6',
			},
		},
		'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/8/PPP2PPP/RN1QKBNR w KQkq - 0 5' => {
			eco => 'A45n',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6 Qxf6

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6 Qxf6'),
			moves => {
				'b1c3' => 'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 1 5',
				'c2c3' => 'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/2P5/PP3PPP/RN1QKBNR b KQkq - 0 5',
			},
		},
		'rnb1kb1r/ppq1p1pp/2pp1n2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 d6 4. Bg2 c6 5. Nc3 Qc7

			variation => N__('Dutch: Hort-Antoshin System'),
			moves => {
			},
		},
		'rnb1kb1r/ppqp1ppp/4pn2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A46o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 Qc7

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 Qc7'),
			moves => {
			},
		},
		'rnb1kb1r/ppqp1ppp/4pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 Qc7

			variation => N__('English: Symmetrical, Two Knights, 5. g3 Qc7'),
			moves => {
			},
		},
		'rnb1kbnr/1p1p1ppp/pq2p3/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B42c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Qb6

			variation => N__('Sicilian: Kan, 5. Bd3 Qb6'),
			moves => {
			},
		},
		'rnb1kbnr/1pqp1ppp/p3p3/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B41g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Qc7

			variation => N__('Sicilian: Kan, 5. c4 Qc7'),
			moves => {
			},
		},
		'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B43h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. g3

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. g3'),
			moves => {
				'g8f6' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'B43j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. Be2

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. Be2'),
			moves => {
				'g8f6' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 4 7',
			},
		},
		'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'B43g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7'),
			moves => {
				'f1d3' => 'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2NB4/PPP2PPP/R1BQK2R b KQkq - 3 6',
				'f1e2' => 'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 3 6',
				'g2g3' => 'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2NB4/PPP2PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'B43m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 Qc7 6. Bd3

			variation => N__('Sicilian: Kan, 5. Nc3 Qc7 6. Bd3'),
			moves => {
				'g8f6' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/2NB4/PPP2PPP/R1BQK2R w KQkq - 4 7',
			},
		},
		'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B42d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Qc7

			variation => N__('Sicilian: Kan, 5. Bd3 Qc7'),
			moves => {
			},
		},
		'rnb1kbnr/3p1ppp/pq2p3/1p6/3NP3/2NB4/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B43e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 b5 6. Bd3 Qb6

			variation => N__('Sicilian: Kan, 5. Nc3 b5 6. Bd3 Qb6'),
			moves => {
				'd4b3' => 'rnb1kbnr/3p1ppp/pq2p3/1p6/4P3/1NNB4/PPP2PPP/R1BQK2R b KQkq - 3 7',
			},
		},
		'rnb1kbnr/3p1ppp/pq2p3/1p6/4P3/1NNB4/PPP2PPP/R1BQK2R b KQkq - 3 7' => {
			eco => 'B43f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 b5 6. Bd3 Qb6 7. Nb3

			variation => N__('Sicilian: Kan, 5. Nc3 b5 6. Bd3 Qb6 7. Nb3'),
			moves => {
			},
		},
		'rnb1kbnr/p1pp1ppp/8/1p6/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq b6 0 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 b5

			variation => N__('KGA: Bishop\'s Gambit, Bryan Countergambit'),
			moves => {
			},
		},
		'rnb1kbnr/p1pq1ppp/1p2p3/3pP3/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'C02b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 b6 4. c3 Qd7

			variation => N__('French: Advance, 3... b6'),
			moves => {
			},
		},
		'rnb1kbnr/pp1p1ppp/1q2p3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B40n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Qb6

			variation => N__('Sicilian: Kveinys Variation'),
			moves => {
			},
		},
		'rnb1kbnr/pp1ppppp/8/q1p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'B27b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Qa5

			variation => N__('Sicilian: Mongoose Variation'),
			moves => {
			},
		},
		'rnb1kbnr/pp2ppp1/1qp4p/3p4/3P3B/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'D00h',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 h6 3. Bh4 c6 4. e3 Qb6

			variation => N__('Hodgson Attack: 2... h6 3. Bh4 c6 4. e3 Qb6'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/1qp5/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'D02c',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c6 3. Bf4 Qb6

			variation => N__('Queen\'s Pawn: London, Alapin Variation'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/1qp5/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'B12g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 Qb6

			variation => N__('Caro-Kann: Edinburgh Variation'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/2p5/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B13a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 Qxd5

			variation => N__('Caro-Kann: Exchange, 3...Qxd5'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/2p5/q7/2BP4/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'B01h',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 c6 5. Bc4

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. d4 c6 5. Bc4'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/2p5/q7/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'B01h',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 c6 5. Nf3

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. d4 c6 5. Nf3'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/2p5/q7/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B01h',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 c6

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. d4 c6'),
			moves => {
				'f1c4' => 'rnb1kbnr/pp2pppp/2p5/q7/2BP4/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 5',
				'g1f3' => 'rnb1kbnr/pp2pppp/2p5/q7/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnb1kbnr/pp2pppp/8/2pq4/3P4/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 4' => {
			eco => 'B22f',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4'),
			moves => {
				'b8c6' => 'r1b1kbnr/pp2pppp/2n5/2pq4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5',
				'c5d4' => 'rnb1kbnr/pp2pppp/8/3q4/3p4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5',
				'e7e6' => 'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5',
				'g8f6' => 'rnb1kb1r/pp2pppp/5n2/2pq4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnb1kbnr/pp2pppp/8/2pq4/5P2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B21j',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5 3. exd5 Qxd5

			variation => N__('Sicilian: Grand Prix, Tal Defence, 3.exd5 Qxd5'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/8/2pq4/8/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B22f',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5'),
			moves => {
				'd2d4' => 'rnb1kbnr/pp2pppp/8/2pq4/3P4/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 4',
			},
		},
		'rnb1kbnr/pp2pppp/8/3q4/1p6/P7/1BPP1PPP/RN1QKBNR b KQkq - 1 5' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b4 cxb4 3. a3 d5 4. exd5 Qxd5 5. Bb2

			variation => N__('Sicilian: Wing Gambit, Marienbad Variation'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/8/3q4/1p6/P7/2PP1PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B20',
                        # TRANSLATORS:

			variation => N__('Sicilian: Wing Gambit, Marshall Variation'),
			moves => {
				'c1b2' => 'rnb1kbnr/pp2pppp/8/3q4/1p6/P7/1BPP1PPP/RN1QKBNR b KQkq - 1 5',
			},
		},
		'rnb1kbnr/pp2pppp/8/3q4/3P4/8/PP3PPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'B22f',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 cxd4 5. cxd4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 cxd4 5.cxd4'),
			moves => {
			},
		},
		'rnb1kbnr/pp2pppp/8/3q4/3p4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B22f',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 cxd4

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 cxd4'),
			moves => {
				'c3d4' => 'rnb1kbnr/pp2pppp/8/3q4/3P4/8/PP3PPP/RNBQKBNR b KQkq - 0 5',
			},
		},
		'rnb1kbnr/pp3ppp/1q2p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'C02h',
                        # TRANSLATORS:

			variation => N__('French: Advance, 4. c3 Qb6'),
			moves => {
				'c8d7' => 'rn2kbnr/pp1b1ppp/1q2p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
			},
		},
		'rnb1kbnr/pp3ppp/1q2p3/2ppP3/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'C02h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3 Qb6

			variation => N__('French: Advance, 4. c3 Qb6'),
			moves => {
				'g1f3' => 'rnb1kbnr/pp3ppp/1q2p3/2ppP3/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'rnb1kbnr/pp3ppp/2p5/q3P3/2Pp4/8/PP1NPPPP/R1BQKBNR b KQkq - 3 6' => {
			eco => 'D10m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5 4. dxe5 d4 5. Ne4 Qa5+ 6. Nd2

			variation => N__('Slav: Winawer Countergambit Accepted, 6. Nd2'),
			moves => {
			},
		},
		'rnb1kbnr/pp3ppp/2p5/q3P3/2PpN3/8/PP1BPPPP/R2QKBNR b KQkq - 3 6' => {
			eco => 'D10n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5 4. dxe5 d4 5. Ne4 Qa5+ 6. Bd2

			variation => N__('Slav: Winawer Countergambit Accepted, 6. Bd2'),
			moves => {
			},
		},
		'rnb1kbnr/pp3ppp/2p5/q3P3/2PpN3/8/PP2PPPP/R1BQKBNR w KQkq - 2 6' => {
			eco => 'D10m',
                        # TRANSLATORS:

			variation => N__('Slav: Winawer Countergambit Accepted'),
			moves => {
				'c1d2' => 'rnb1kbnr/pp3ppp/2p5/q3P3/2PpN3/8/PP1BPPPP/R2QKBNR b KQkq - 3 6',
				'e4d2' => 'rnb1kbnr/pp3ppp/2p5/q3P3/2Pp4/8/PP1NPPPP/R1BQKBNR b KQkq - 3 6',
			},
		},
		'rnb1kbnr/pp3ppp/3qp3/8/2Bp4/5N2/PPPN1PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'C07o',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'g8f6' => 'rnb1kb1r/pp3ppp/3qpn2/8/2Bp4/5N2/PPPN1PPP/R1BQ1RK1 w kq - 4 8',
			},
		},
		'rnb1kbnr/pp3ppp/3qp3/8/2Bp4/5N2/PPPN1PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'C07o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd6

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4 Qd6'),
			moves => {
				'e1g1' => 'rnb1kbnr/pp3ppp/3qp3/8/2Bp4/5N2/PPPN1PPP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'B22g',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 e6 5. Nf3

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 e6 5. Nf3'),
			moves => {
			},
		},
		'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B22g',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5 Qxd5 4. d4 e6

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5 Qxd5 4. d4 e6'),
			moves => {
				'g1f3' => 'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C07k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3'),
			moves => {
				'c5d4' => 'rnb1kbnr/pp3ppp/4p3/3q4/3p4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C07j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5'),
			moves => {
				'g1f3' => 'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnb1kbnr/pp3ppp/4p3/3q4/2Bp4/5N2/PPPN1PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C07m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4 6. Bc4'),
			moves => {
				'd5d6' => 'rnb1kbnr/pp3ppp/3qp3/8/2Bp4/5N2/PPPN1PPP/R1BQK2R w KQkq - 2 7',
				'd5d8' => 'rnbqkbnr/pp3ppp/4p3/8/2Bp4/5N2/PPPN1PPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnb1kbnr/pp3ppp/4p3/3q4/3p4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C07l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4

			variation => N__('French: Tarrasch, Open, 4.exd5 Qxd5 5.Ngf3 cxd4'),
			moves => {
				'f1c4' => 'rnb1kbnr/pp3ppp/4p3/3q4/2Bp4/5N2/PPPN1PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnb1kbnr/pp3ppp/8/2pN4/2p5/5N2/PP2PPPP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'D21p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6. Qxd5 Qxd5 7. Nxd5

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3 exd5 6.Qxd5 Qxd5 7.Nxd5'),
			moves => {
				'f8d6' => 'rnb1k1nr/pp3ppp/3b4/2pN4/2p5/5N2/PP2PPPP/R1B1KB1R w KQkq - 1 8',
			},
		},
		'rnb1kbnr/pp3ppp/8/2pq4/2p5/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'D21o',
                        # TRANSLATORS:

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3 exd5'),
			moves => {
				'c3d5' => 'rnb1kbnr/pp3ppp/8/2pN4/2p5/5N2/PP2PPPP/R1B1KB1R b KQkq - 0 7',
			},
		},
		'rnb1kbnr/ppp1pppp/3q4/8/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq d3 0 4' => {
			eco => 'B01d',
                        # TRANSLATORS:

			variation => N__('Scandinavian: Schiller Defence'),
			moves => {
				'g8f6' => 'rnb1kb1r/ppp1pppp/3q1n2/8/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnb1kbnr/ppp1pppp/3q4/8/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'B01d',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qd6

			variation => N__('Scandinavian: Schiller Defence'),
			moves => {
				'd2d4' => 'rnb1kbnr/ppp1pppp/3q4/8/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq d3 0 4',
			},
		},
		'rnb1kbnr/ppp1pppp/8/3q4/3P4/8/PPP2PPP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. d4

			variation => N__('Scandinavian: 2...Qxd5 3. d4'),
			moves => {
				'e7e5' => 'rnb1kbnr/ppp2ppp/8/3qp3/3P4/8/PPP2PPP/RNBQKBNR w KQkq e6 0 4',
				'g8f6' => 'rnb1kb1r/ppp1pppp/5n2/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnb1kbnr/ppp1pppp/8/3q4/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3'),
			moves => {
				'd5a5' => 'rnb1kbnr/ppp1pppp/8/q7/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 4',
				'd5d6' => 'rnb1kbnr/ppp1pppp/3q4/8/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 4',
				'd5d8' => 'rnbqkbnr/ppp1pppp/8/8/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnb1kbnr/ppp1pppp/8/3q4/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nf3

			variation => N__('Scandinavian: 2...Qxd5 3. Nf3'),
			moves => {
				'g8f6' => 'rnb1kb1r/ppp1pppp/5n2/3q4/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnb1kbnr/ppp1pppp/8/3q4/8/8/PPPP1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5

			variation => N__('Scandinavian: 2...Qxd5'),
			moves => {
				'b1c3' => 'rnb1kbnr/ppp1pppp/8/3q4/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 3',
				'd2d4' => 'rnb1kbnr/ppp1pppp/8/3q4/3P4/8/PPP2PPP/RNBQKBNR b KQkq d3 0 3',
				'g1f3' => 'rnb1kbnr/ppp1pppp/8/3q4/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnb1kbnr/ppp1pppp/8/q7/1P6/2N5/P1PP1PPP/R1BQKBNR b KQkq b3 0 4' => {
			eco => 'B01e',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. b4

			variation => N__('Scandinavian, Mieses Gambit'),
			moves => {
			},
		},
		'rnb1kbnr/ppp1pppp/8/q7/2B5/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 4' => {
			eco => 'B01e',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. Bc4

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. Bc4'),
			moves => {
				'g8f6' => 'rnb1kb1r/ppp1pppp/5n2/q7/2B5/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 5',
			},
		},
		'rnb1kbnr/ppp1pppp/8/q7/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq d3 0 4' => {
			eco => 'B01g',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. d4'),
			moves => {
				'c7c6' => 'rnb1kbnr/pp2pppp/2p5/q7/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
				'e7e5' => 'rnb1kbnr/ppp2ppp/8/q3p3/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 5',
				'g8f6' => 'rnb1kb1r/ppp1pppp/5n2/q7/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnb1kbnr/ppp1pppp/8/q7/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'B01f',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. Nf3

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. Nf3'),
			moves => {
				'g8f6' => 'rnb1kb1r/ppp1pppp/5n2/q7/8/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnb1kbnr/ppp1pppp/8/q7/8/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B01e',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. g3

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5 4. g3'),
			moves => {
			},
		},
		'rnb1kbnr/ppp1pppp/8/q7/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'B01e',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qa5'),
			moves => {
				'b2b4' => 'rnb1kbnr/ppp1pppp/8/q7/1P6/2N5/P1PP1PPP/R1BQKBNR b KQkq b3 0 4',
				'd2d4' => 'rnb1kbnr/ppp1pppp/8/q7/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq d3 0 4',
				'f1c4' => 'rnb1kbnr/ppp1pppp/8/q7/2B5/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 4',
				'g1f3' => 'rnb1kbnr/ppp1pppp/8/q7/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 4',
				'g2g3' => 'rnb1kbnr/ppp1pppp/8/q7/8/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnb1kbnr/ppp2p1p/8/3B2p1/4Pp1q/6P1/PPPP3P/RNBQ1KNR b kq - 0 6' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 d5 5. Bxd5 g5 6. g3

			variation => N__('KGA: Bishop\'s Gambit, Chigorin Attack'),
			moves => {
			},
		},
		'rnb1kbnr/ppp2p1p/8/3B2p1/4Pp1q/8/PPPP2PP/RNBQ1KNR w kq g6 0 6' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit'),
			moves => {
				'g2g3' => 'rnb1kbnr/ppp2p1p/8/3B2p1/4Pp1q/6P1/PPPP3P/RNBQ1KNR b kq - 0 6',
			},
		},
		'rnb1kbnr/ppp2ppp/4p3/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C01a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 Qxd5

			variation => N__('French: Exchange, 3...Qxd5'),
			moves => {
			},
		},
		'rnb1kbnr/ppp2ppp/4p3/3q4/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C10f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Qd5

			variation => N__('French: Rubinstein, Frere (Becker) Variation'),
			moves => {
			},
		},
		'rnb1kbnr/ppp2ppp/8/3B4/4Pp1q/8/PPPP2PP/RNBQ1KNR b kq - 0 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit'),
			moves => {
				'g7g5' => 'rnb1kbnr/ppp2p1p/8/3B2p1/4Pp1q/8/PPPP2PP/RNBQ1KNR w kq g6 0 6',
			},
		},
		'rnb1kbnr/ppp2ppp/8/3B4/4Pp1q/8/PPPP2PP/RNBQ1KNR b kq - 2 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Bledow, 4.Bxd5'),
			moves => {
				'f8d6' => 'rnb1k1nr/ppp2ppp/3b4/3B4/4Pp1q/8/PPPP2PP/RNBQ1KNR w kq - 3 6',
			},
		},
		'rnb1kbnr/ppp2ppp/8/3B4/4Pp1q/8/PPPP2PP/RNBQK1NR w KQkq - 1 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Bledow, 4.Bxd5'),
			moves => {
				'e1f1' => 'rnb1kbnr/ppp2ppp/8/3B4/4Pp1q/8/PPPP2PP/RNBQ1KNR b kq - 2 5',
			},
		},
		'rnb1kbnr/ppp2ppp/8/3p4/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq d6 0 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit'),
			moves => {
				'c4d5' => 'rnb1kbnr/ppp2ppp/8/3B4/4Pp1q/8/PPPP2PP/RNBQ1KNR b kq - 0 5',
			},
		},
		'rnb1kbnr/ppp2ppp/8/3qp3/3P4/8/PPP2PPP/RNBQKBNR w KQkq e6 0 4' => {
			eco => 'B01b',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. d4 e5

			variation => N__('Scandinavian: 2...Qxd5 3. d4 e5'),
			moves => {
			},
		},
		'rnb1kbnr/ppp2ppp/8/q3P3/8/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'B01g',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 e5 5. dxe5

			variation => N__('Scandinavian: Anderssen Counterattack, 5.dxe5'),
			moves => {
			},
		},
		'rnb1kbnr/ppp2ppp/8/q3p3/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'B01g',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 e5 5. Nf3

			variation => N__('Scandinavian: Anderssen Counterattack, Goteborg Variation'),
			moves => {
				'c8g4' => 'rn2kbnr/ppp2ppp/8/q3p3/3P2b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnb1kbnr/ppp2ppp/8/q3p3/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 5' => {
			eco => 'B01g',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qa5 4. d4 e5

			variation => N__('Scandinavian: Anderssen Counterattack'),
			moves => {
				'd4e5' => 'rnb1kbnr/ppp2ppp/8/q3P3/8/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 5',
				'g1f3' => 'rnb1kbnr/ppp2ppp/8/q3p3/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnb1kbnr/ppp3pp/3p1q2/4Np2/3PP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C40u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4 d6

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. d4 d6'),
			moves => {
				'e5c4' => 'rnb1kbnr/ppp3pp/3p1q2/5p2/2NPP3/8/PPP2PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnb1kbnr/ppp3pp/3p1q2/5p2/2NPP3/8/PPP2PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'C40u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4 d6 5. Nc4

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. d4 d6 5. Nc4'),
			moves => {
				'f5e4' => 'rnb1kbnr/ppp3pp/3p1q2/8/2NPp3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnb1kbnr/ppp3pp/3p1q2/8/2NPp3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'C40x',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4 6. Nc3

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4 6. Nc3'),
			moves => {
			},
		},
		'rnb1kbnr/ppp3pp/3p1q2/8/2NPp3/8/PPP1BPPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'C40v',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4 6. Be2

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4 6. Be2'),
			moves => {
			},
		},
		'rnb1kbnr/ppp3pp/3p1q2/8/2NPp3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C40u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4'),
			moves => {
				'b1c3' => 'rnb1kbnr/ppp3pp/3p1q2/8/2NPp3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6',
				'c4e3' => 'rnb1kbnr/ppp3pp/3p1q2/8/3Pp3/4N3/PPP2PPP/RNBQKB1R b KQkq - 1 6',
				'f1e2' => 'rnb1kbnr/ppp3pp/3p1q2/8/2NPp3/8/PPP1BPPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnb1kbnr/ppp3pp/3p1q2/8/3Pp3/4N3/PPP2PPP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'C40w',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4 d6 5. Nc4 fxe4 6. Ne3

			variation => N__('Latvian Gambit: Nimzowitsch Variation'),
			moves => {
			},
		},
		'rnb1kbnr/pppp1B1p/6p1/4N2Q/3Pp3/8/PPP2PqP/RNB1K2R b KQkq - 1 7' => {
			eco => 'C40q',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Poisoned Pawn Variation'),
			moves => {
				'e8d8' => 'rnbk1bnr/pppp1B1p/6p1/4N2Q/3Pp3/8/PPP2PqP/RNB1K2R w KQ - 2 8',
			},
		},
		'rnb1kbnr/pppp1B1p/8/4q3/5p2/5Q2/PPPP2PP/RNB2RK1 b kq - 0 8' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O gxf3 6. Qxf3 Qf6 7. e5 Qxe5 8. Bxf7+

			variation => N__('KGA: Double Muzio Gambit'),
			moves => {
			},
		},
		'rnb1kbnr/pppp1p1p/5q2/4P3/2B2p2/5Q2/PPPP2PP/RNB2RK1 b kq - 0 7' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'f6e5' => 'rnb1kbnr/pppp1p1p/8/4q3/2B2p2/5Q2/PPPP2PP/RNB2RK1 w kq - 0 8',
			},
		},
		'rnb1kbnr/pppp1p1p/5q2/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 w kq - 1 7' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'e4e5' => 'rnb1kbnr/pppp1p1p/5q2/4P3/2B2p2/5Q2/PPPP2PP/RNB2RK1 b kq - 0 7',
			},
		},
		'rnb1kbnr/pppp1p1p/8/4N3/2B1P1pq/5p2/PPPP2PP/RNBQ1K1R w kq - 0 7' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Ne5 Qh4+ 6. Kf1 f3

			variation => N__('KGA: Salvio, Cochrane Gambit'),
			moves => {
			},
		},
		'rnb1kbnr/pppp1p1p/8/4N3/2B1Pppq/8/PPPP2PP/RNBQ1K1R b kq - 3 6' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Salvio Gambit'),
			moves => {
				'b8c6' => 'r1b1kbnr/pppp1p1p/2n5/4N3/2B1Pppq/8/PPPP2PP/RNBQ1K1R w kq - 4 7',
				'f4f3' => 'rnb1kbnr/pppp1p1p/8/4N3/2B1P1pq/5p2/PPPP2PP/RNBQ1K1R w kq - 0 7',
				'g8h6' => 'rnb1kb1r/pppp1p1p/7n/4N3/2B1Pppq/8/PPPP2PP/RNBQ1K1R w kq - 4 7',
			},
		},
		'rnb1kbnr/pppp1p1p/8/4N3/2B1Pppq/8/PPPP2PP/RNBQK2R w KQkq - 2 6' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Salvio Gambit'),
			moves => {
				'e1f1' => 'rnb1kbnr/pppp1p1p/8/4N3/2B1Pppq/8/PPPP2PP/RNBQ1K1R b kq - 3 6',
			},
		},
		'rnb1kbnr/pppp1p1p/8/4q3/2B2p2/3P1Q2/PPP3PP/RNB2RK1 b kq - 0 8' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'f8h6' => 'rnb1k1nr/pppp1p1p/7b/4q3/2B2p2/3P1Q2/PPP3PP/RNB2RK1 w kq - 1 9',
			},
		},
		'rnb1kbnr/pppp1p1p/8/4q3/2B2p2/5Q2/PPPP2PP/RNB2RK1 w kq - 0 8' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'c4f7' => 'rnb1kbnr/pppp1B1p/8/4q3/5p2/5Q2/PPPP2PP/RNB2RK1 b kq - 0 8',
				'd2d3' => 'rnb1kbnr/pppp1p1p/8/4q3/2B2p2/3P1Q2/PPP3PP/RNB2RK1 b kq - 0 8',
			},
		},
		'rnb1kbnr/pppp1p1p/8/6p1/2B1Pp1q/2N5/PPPP2PP/R1BQ1KNR b kq - 1 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence'),
			moves => {
				'f8g7' => 'rnb1k1nr/pppp1pbp/8/6p1/2B1Pp1q/2N5/PPPP2PP/R1BQ1KNR w kq - 2 6',
			},
		},
		'rnb1kbnr/pppp1p1p/8/6p1/2B1Pp1q/5Q2/PPPP2PP/RNB2KNR b kq - 1 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5 5. Qf3

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence, Cozio Attack'),
			moves => {
			},
		},
		'rnb1kbnr/pppp1p1p/8/6p1/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq g6 0 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Qh4+ 4. Kf1 g5

			variation => N__('KGA: Bishop\'s Gambit, Classical Defence'),
			moves => {
				'b1c3' => 'rnb1kbnr/pppp1p1p/8/6p1/2B1Pp1q/2N5/PPPP2PP/R1BQ1KNR b kq - 1 5',
				'd1f3' => 'rnb1kbnr/pppp1p1p/8/6p1/2B1Pp1q/5Q2/PPPP2PP/RNB2KNR b kq - 1 5',
			},
		},
		'rnb1kbnr/pppp1ppp/5q2/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'C40b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Qf6

			variation => N__('Open Game: Greco Defence'),
			moves => {
			},
		},
		'rnb1kbnr/pppp1ppp/5q2/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'C30e',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'f6f4' => 'rnb1kbnr/pppp1ppp/8/4p3/4Pq2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnb1kbnr/pppp1ppp/5q2/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'C30e',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'f6f4' => 'rnb1kbnr/pppp1ppp/8/4p3/4Pq2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnb1kbnr/pppp1ppp/5q2/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'C30e',
                        # TRANSLATORS: 1. e4 e5 2. f4 Qf6

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'b1c3' => 'rnb1kbnr/pppp1ppp/5q2/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 2 3',
				'g1f3' => 'rnb1kbnr/pppp1ppp/5q2/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnb1kbnr/pppp1ppp/8/4p3/3PPq2/2N5/PPP3PP/R1BQKBNR b KQkq d3 0 4' => {
			eco => 'C30e',
                        # TRANSLATORS: 1. e4 e5 2. f4 Qf6 3. Nc3 Qxf4 4. d4

			variation => N__('King\'s Gambit: Norwald Variation, Schubert line'),
			moves => {
			},
		},
		'rnb1kbnr/pppp1ppp/8/4p3/4PP1q/6P1/PPPP3P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C30f',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Keene Defence'),
			moves => {
				'h4e7' => 'rnb1kbnr/ppppqppp/8/4p3/4PP2/6P1/PPPP3P/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnb1kbnr/pppp1ppp/8/4p3/4PP1q/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'C30f',
                        # TRANSLATORS: 1. e4 e5 2. f4 Qh4+

			variation => N__('King\'s Gambit: Keene Defence'),
			moves => {
				'g2g3' => 'rnb1kbnr/pppp1ppp/8/4p3/4PP1q/6P1/PPPP3P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnb1kbnr/pppp1ppp/8/4p3/4Pq2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C30e',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'f8b4' => 'rnb1k1nr/pppp1ppp/8/4p3/1b2Pq2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnb1kbnr/pppp1ppp/8/4p3/4Pq2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C30e',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'd2d4' => 'rnb1kbnr/pppp1ppp/8/4p3/3PPq2/2N5/PPP3PP/R1BQKBNR b KQkq d3 0 4',
			},
		},
		'rnb1kbnr/pppp1ppp/8/4p3/4Pq2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C30e',
                        # TRANSLATORS:

			variation => N__('King\'s Gambit: Norwald Variation'),
			moves => {
				'b1c3' => 'rnb1kbnr/pppp1ppp/8/4p3/4Pq2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnb1kbnr/pppp1ppp/8/8/2B1Pp1q/8/PPPP2PP/RNBQ1KNR b kq - 3 4' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit'),
			moves => {
				'b7b5' => 'rnb1kbnr/p1pp1ppp/8/1p6/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq b6 0 5',
				'b8c6' => 'r1b1kbnr/pppp1ppp/2n5/8/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq - 4 5',
				'd7d5' => 'rnb1kbnr/ppp2ppp/8/3p4/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq d6 0 5',
				'f8c5' => 'rnb1k1nr/pppp1ppp/8/2b5/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq - 4 5',
				'g7g5' => 'rnb1kbnr/pppp1p1p/8/6p1/2B1Pp1q/8/PPPP2PP/RNBQ1KNR w kq g6 0 5',
			},
		},
		'rnb1kbnr/pppp1ppp/8/8/2B1Pp1q/8/PPPP2PP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit'),
			moves => {
				'e1f1' => 'rnb1kbnr/pppp1ppp/8/8/2B1Pp1q/8/PPPP2PP/RNBQ1KNR b kq - 3 4',
			},
		},
		'rnb1kbnr/pppp2pp/5q2/4Np2/3PP3/8/PPP2PPP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'C40u',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. d4

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. d4'),
			moves => {
				'd7d6' => 'rnb1kbnr/ppp3pp/3p1q2/4Np2/3PP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnb1kbnr/pppp2pp/5q2/4Np2/4P3/8/PPPP1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C40t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6'),
			moves => {
				'd2d4' => 'rnb1kbnr/pppp2pp/5q2/4Np2/3PP3/8/PPP2PPP/RNBQKB1R b KQkq d3 0 4',
				'e5c4' => 'rnb1kbnr/pppp2pp/5q2/5p2/2N1P3/8/PPPP1PPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'rnb1kbnr/pppp2pp/5q2/5p2/2N1P3/8/PPPP1PPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'C40t',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5 Qf6 4. Nc4

			variation => N__('Latvian Gambit: 3.Nxe5 Qf6 4. Nc4'),
			moves => {
			},
		},
		'rnb1kbnr/pppp2pp/8/4N1q1/2B1p3/8/PPPP1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C40q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4 4. Nxe5 Qg5

			variation => N__('Latvian Gambit: Poisoned Pawn Variation'),
			moves => {
				'd2d4' => 'rnb1kbnr/pppp2pp/8/4N1q1/2BPp3/8/PPP2PPP/RNBQK2R b KQkq d3 0 5',
			},
		},
		'rnb1kbnr/pppp2pp/8/4N1q1/2BPp3/8/PPP2PPP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C40q',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Poisoned Pawn Variation'),
			moves => {
				'g5g2' => 'rnb1kbnr/pppp2pp/8/4N3/2BPp3/8/PPP2PqP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rnb1kbnr/pppp2pp/8/4N2Q/2BPp3/8/PPP2PqP/RNB1K2R b KQkq - 1 6' => {
			eco => 'C40q',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Poisoned Pawn Variation'),
			moves => {
				'g7g6' => 'rnb1kbnr/pppp3p/6p1/4N2Q/2BPp3/8/PPP2PqP/RNB1K2R w KQkq - 0 7',
			},
		},
		'rnb1kbnr/pppp2pp/8/4N3/2BPp3/8/PPP2PqP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C40q',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Poisoned Pawn Variation'),
			moves => {
				'd1h5' => 'rnb1kbnr/pppp2pp/8/4N2Q/2BPp3/8/PPP2PqP/RNB1K2R b KQkq - 1 6',
			},
		},
		'rnb1kbnr/pppp2pp/8/5p2/2B1Pp1q/8/PPPPQ1PP/RNB1K1NR w KQkq - 2 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit'),
			moves => {
				'e1d1' => 'rnb1kbnr/pppp2pp/8/5p2/2B1Pp1q/8/PPPPQ1PP/RNBK2NR b kq - 3 5',
			},
		},
		'rnb1kbnr/pppp2pp/8/5p2/2B1Pp1q/8/PPPPQ1PP/RNBK2NR b kq - 3 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit'),
			moves => {
				'f5e4' => 'rnb1kbnr/pppp2pp/8/8/2B1pp1q/8/PPPPQ1PP/RNBK2NR w kq - 0 6',
			},
		},
		'rnb1kbnr/pppp2pp/8/8/2B1pp1q/2N5/PPPPQ1PP/R1BK2NR b kq - 1 6' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit'),
			moves => {
				'e8d8' => 'rnbk1bnr/pppp2pp/8/8/2B1pp1q/2N5/PPPPQ1PP/R1BK2NR w - - 2 7',
			},
		},
		'rnb1kbnr/pppp2pp/8/8/2B1pp1q/8/PPPPQ1PP/RNBK2NR w kq - 0 6' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit'),
			moves => {
				'b1c3' => 'rnb1kbnr/pppp2pp/8/8/2B1pp1q/2N5/PPPPQ1PP/R1BK2NR b kq - 1 6',
			},
		},
		'rnb1kbnr/pppp3p/6p1/4N2Q/2BPp3/8/PPP2PqP/RNB1K2R w KQkq - 0 7' => {
			eco => 'C40q',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Poisoned Pawn Variation'),
			moves => {
				'c4f7' => 'rnb1kbnr/pppp1B1p/6p1/4N2Q/3Pp3/8/PPP2PqP/RNB1K2R b KQkq - 1 7',
			},
		},
		'rnb1kbnr/ppppq2p/8/4Np2/2BPPppP/8/PPP3P1/RNBQK2R b KQkq - 1 7' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Qe7 6. d4 f5 7. Bc4

			variation => N__('KGA: Kieseritsky, Salvio Defence, Cozio Variation'),
			moves => {
			},
		},
		'rnb1kbnr/ppppq2p/8/4Np2/3PPppP/8/PPP3P1/RNBQKB1R w KQkq f6 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Salvio (Rosenthal) Defence'),
			moves => {
				'f1c4' => 'rnb1kbnr/ppppq2p/8/4Np2/2BPPppP/8/PPP3P1/RNBQK2R b KQkq - 1 7',
			},
		},
		'rnb1kbnr/ppppqp1p/8/4N3/3PPppP/8/PPP3P1/RNBQKB1R b KQkq d3 0 6' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Salvio (Rosenthal) Defence'),
			moves => {
				'f7f5' => 'rnb1kbnr/ppppq2p/8/4Np2/3PPppP/8/PPP3P1/RNBQKB1R w KQkq f6 0 7',
			},
		},
		'rnb1kbnr/ppppqp1p/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Qe7

			variation => N__('KGA: Kieseritsky, Salvio (Rosenthal) Defence'),
			moves => {
				'd2d4' => 'rnb1kbnr/ppppqp1p/8/4N3/3PPppP/8/PPP3P1/RNBQKB1R b KQkq d3 0 6',
			},
		},
		'rnb1kbnr/ppppqp1p/8/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 w kq - 1 7' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O gxf3 6. Qxf3 Qe7

			variation => N__('KGA: Muzio Gambit, From Defence'),
			moves => {
			},
		},
		'rnb1kbnr/ppppqp1p/8/8/2B1Ppp1/5N2/PPPP2PP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O Qe7

			variation => N__('KGA: Muzio Gambit, Kling and Horwitz Counterattack'),
			moves => {
			},
		},
		'rnb1kbnr/ppppqppp/8/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'C40d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Qe7 3. Nc3

			variation => N__('Open Game: Gunderam Defence, 3. Nc3'),
			moves => {
			},
		},
		'rnb1kbnr/ppppqppp/8/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'C40c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Qe7

			variation => N__('Open Game: Gunderam Defence'),
			moves => {
				'b1c3' => 'rnb1kbnr/ppppqppp/8/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
			},
		},
		'rnb1kbnr/ppppqppp/8/4p3/4PP2/6P1/PPPP3P/RNBQKBNR w KQkq - 1 4' => {
			eco => 'C30f',
                        # TRANSLATORS: 1. e4 e5 2. f4 Qh4+ 3. g3 Qe7

			variation => N__('King\'s Gambit: Keene Defence, 3. g3 Qe7'),
			moves => {
			},
		},
		'rnb1kbnr/ppppqppp/8/8/2B1P3/8/PB3PPP/RN1QK1NR w KQkq - 1 6' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3 4. Bc4 cxb2 5. Bxb2 Qe7

			variation => N__('Danish Gambit: Chigorin Defence'),
			moves => {
			},
		},
		'rnb1kbnr/ppqppppp/8/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'B27c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Qc7

			variation => N__('Sicilian: Quinteros Variation'),
			moves => {
			},
		},
		'rnb1qrk1/1p2b1pp/2pppn2/p4p2/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 w - - 0 10' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Bb2 c6

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Bb2 c6'),
			moves => {
			},
		},
		'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/P1Q1PPBP/R1B2RK1 b - - 1 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Qc2

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Qc2'),
			moves => {
			},
		},
		'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - a6 0 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5'),
			moves => {
				'c1a3' => 'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/BPN2NP1/P3PPBP/R2Q1RK1 b - - 1 9',
				'c1b2' => 'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 b - - 1 9',
				'd1c2' => 'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/P1Q1PPBP/R1B2RK1 b - - 1 9',
			},
		},
		'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 b - - 1 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Bb2

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Bb2'),
			moves => {
				'b8a6' => 'r1b1qrk1/1pp1b1pp/n2ppn2/p4p2/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 w - - 2 10',
				'c7c6' => 'rnb1qrk1/1p2b1pp/2pppn2/p4p2/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 w - - 0 10',
				'e8h5' => 'rnb2rk1/1pp1b1pp/3ppn2/p4p1q/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 w - - 2 10',
			},
		},
		'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/BPN2NP1/P3PPBP/R2Q1RK1 b - - 1 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Ba3

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Ba3'),
			moves => {
			},
		},
		'rnb1qrk1/1pp1p1bp/3p1np1/p2P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - a6 0 9' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. d5 a5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. d5 a5'),
			moves => {
			},
		},
		'rnb1qrk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. b3 Qe8

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. b3 Qe8'),
			moves => {
			},
		},
		'rnb1qrk1/pp2b1pp/2p1pn2/3p1p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 2 9' => {
			eco => 'A95',
                        # TRANSLATORS:

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. Qc2'),
			moves => {
				'c1g5' => 'rnb1qrk1/pp2b1pp/2p1pn2/3p1pB1/2PP4/2N2NP1/PPQ1PPBP/R4RK1 b - - 3 9',
			},
		},
		'rnb1qrk1/pp2b1pp/2p1pn2/3p1pB1/2PP4/2N2NP1/PPQ1PPBP/R4RK1 b - - 3 9' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. Qc2 Qe8 9. Bg5

			variation => N__('Dutch: Stonewall: Chekhover Variation'),
			moves => {
			},
		},
		'rnb1qrk1/pp2b1pp/2pppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 c6

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 c6'),
			moves => {
			},
		},
		'rnb1qrk1/pp2p1bp/2pp1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 Qe8

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 Qe8'),
			moves => {
			},
		},
		'rnb1qrk1/pp2p1bp/2pp1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. b3 Qe8

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. b3 Qe8'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1b1pp/3pp3/5p2/2PPn3/2N2NP1/PP2PPBP/R1BQR1K1 w - - 4 9' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1 Ne4

			variation => N__('Dutch: Ilyin-Zhenevsky, Winter, 8... Ne4'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/1PPP4/2N2NP1/P3PPBP/R1BQ1RK1 b - b3 0 8' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b4

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b4'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. b3 Qe8

			variation => N__('Dutch: Classical, 7. b3 Qe8'),
			moves => {
				'c1b2' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 2 8',
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 2 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. b3 Qe8 8. Bb2

			variation => N__('Dutch: Classical, 7. b3 Qe8 8. Bb2'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3'),
			moves => {
				'a7a5' => 'rnb1qrk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - a6 0 9',
				'b8c6' => 'r1b1qrk1/ppp1b1pp/2nppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
				'b8d7' => 'r1b1qrk1/pppnb1pp/3ppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
				'c7c6' => 'rnb1qrk1/pp2b1pp/2pppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 0 9',
				'e8h5' => 'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8

			variation => N__('Dutch: Ilyin-Zhenevsky Variation'),
			moves => {
				'b2b3' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'b2b4' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/1PPP4/2N2NP1/P3PPBP/R1BQ1RK1 b - b3 0 8',
				'd1c2' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 3 8',
				'f1e1' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 3 8',
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 3 8' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1

			variation => N__('Dutch: Ilyin-Zhenevsky, Winter Variation'),
			moves => {
				'e8g6' => 'rnb2rk1/ppp1b1pp/3ppnq1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 w - - 4 9',
				'e8h5' => 'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 w - - 4 9',
				'f6e4' => 'rnb1qrk1/ppp1b1pp/3pp3/5p2/2PPn3/2N2NP1/PP2PPBP/R1BQR1K1 w - - 4 9',
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 3 8' => {
			eco => 'A98',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Qc2

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. Qc2'),
			moves => {
				'b8c6' => 'r1b1qrk1/ppp1b1pp/2nppn2/5p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 4 9',
				'e8h5' => 'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 4 9',
			},
		},
		'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N3PN/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6. O-O d6 7. Nc3 Qe8

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6.O-O d6 7. Nc3 Qe8'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1p1bp/3p1np1/3N1p2/2PP4/5NP1/PP2PPBP/R1BQ1RK1 b - - 3 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Nd5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. Nd5'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1p1bp/3p1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. d5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. d5'),
			moves => {
				'a7a5' => 'rnb1qrk1/1pp1p1bp/3p1np1/p2P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - a6 0 9',
				'b8a6' => 'r1b1qrk1/ppp1p1bp/n2p1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnb1qrk1/ppp1p1bp/3p1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. b3'),
			moves => {
			},
		},
		'rnb1qrk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8'),
			moves => {
				'b2b3' => 'rnb1qrk1/ppp1p1bp/3p1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'c3d5' => 'rnb1qrk1/ppp1p1bp/3p1np1/3N1p2/2PP4/5NP1/PP2PPBP/R1BQ1RK1 b - - 3 8',
				'd4d5' => 'rnb1qrk1/ppp1p1bp/3p1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'f1e1' => 'rnb1qrk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 3 8',
			},
		},
		'rnb1qrk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 3 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 Qe8 8. Re1'),
			moves => {
			},
		},
		'rnb1r1k1/pp3pbp/1q1P2p1/2p1n1B1/8/2N2N2/PP2B1PP/R2Q1RK1 b - - 0 14' => {
			eco => 'A69m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. Bg5 Qb6 13. O-O Nxe5 14. d6

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5 Qb6 13.O-O Nxe5 14. d6'),
			moves => {
			},
		},
		'rnb1r1k1/pp3pbp/1q4p1/2pPN1B1/8/2N5/PP2B1PP/R2Q1RK1 b - - 0 14' => {
			eco => 'A69l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. Bg5 Qb6 13. O-O Nxe5 14. Nxe5

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5 Qb6 13.O-O Nxe5 14.Nxe5'),
			moves => {
			},
		},
		'rnb1r1k1/pp3pbp/1q4p1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2Q1RK1 b - - 4 13' => {
			eco => 'A69k',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5 Qb6'),
			moves => {
				'g4e5' => 'rnb1r1k1/pp3pbp/1q4p1/2pPn1B1/8/2N2N2/PP2B1PP/R2Q1RK1 w - - 0 14',
			},
		},
		'rnb1r1k1/pp3pbp/1q4p1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R w KQ - 3 13' => {
			eco => 'A69k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. Bg5 Qb6

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5 Qb6'),
			moves => {
				'e1g1' => 'rnb1r1k1/pp3pbp/1q4p1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2Q1RK1 b - - 4 13',
			},
		},
		'rnb1r1k1/pp3pbp/1q4p1/2pPn1B1/8/2N2N2/PP2B1PP/R2Q1RK1 w - - 0 14' => {
			eco => 'A69k',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5 Qb6'),
			moves => {
				'd5d6' => 'rnb1r1k1/pp3pbp/1q1P2p1/2p1n1B1/8/2N2N2/PP2B1PP/R2Q1RK1 b - - 0 14',
				'f3e5' => 'rnb1r1k1/pp3pbp/1q4p1/2pPN1B1/8/2N5/PP2B1PP/R2Q1RK1 b - - 0 14',
			},
		},
		'rnb1r1k1/ppp2pbp/5np1/3Np1B1/2P1P3/5N2/PP2BPPP/R3K2R b KQ - 3 10' => {
			eco => 'E92f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. dxe5 dxe5 8. Qxd8 Rxd8 9. Bg5 Re8 10. Nd5

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap, 9. Bg5 Re8 10. Nd5'),
			moves => {
			},
		},
		'rnb1r1k1/ppp2pbp/5np1/4p1B1/2P1P3/2N2N2/PP2BPPP/R3K2R w KQ - 2 10' => {
			eco => 'E92e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. dxe5 dxe5 8. Qxd8 Rxd8 9. Bg5 Re8

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap, 9. Bg5 Re8'),
			moves => {
				'c3d5' => 'rnb1r1k1/ppp2pbp/5np1/3Np1B1/2P1P3/5N2/PP2BPPP/R3K2R b KQ - 3 10',
			},
		},
		'rnb2bnr/pppp1k1p/5q2/8/3PPp2/5Q2/PPP3PP/RNB2RK1 b - d3 0 8' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'f6d4' => 'rnb2bnr/pppp1k1p/8/8/3qPp2/5Q2/PPP3PP/RNB2RK1 w - - 0 9',
			},
		},
		'rnb2bnr/pppp1k1p/5q2/8/4Pp2/2N1BQ2/PPP3PP/R4RK1 b - - 3 10' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Bxf7+ Kxf7 6. O-O gxf3 7. Qxf3 Qf6 8. d4 Qxd4+ 9. Be3 Qf6 10. Nc3

			variation => N__('KGA: Lolli Gambit, Young Variation'),
			moves => {
			},
		},
		'rnb2bnr/pppp1k1p/5q2/8/4Pp2/4BQ2/PPP3PP/RN3RK1 w - - 2 10' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'b1c3' => 'rnb2bnr/pppp1k1p/5q2/8/4Pp2/2N1BQ2/PPP3PP/R4RK1 b - - 3 10',
			},
		},
		'rnb2bnr/pppp1k1p/5q2/8/4Pp2/5Q2/PPPP2PP/RNB2RK1 w - - 1 8' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'd2d4' => 'rnb2bnr/pppp1k1p/5q2/8/3PPp2/5Q2/PPP3PP/RNB2RK1 b - d3 0 8',
			},
		},
		'rnb2bnr/pppp1k1p/8/8/3qPp2/4BQ2/PPP3PP/RN3RK1 b - - 1 9' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'd4f6' => 'rnb2bnr/pppp1k1p/5q2/8/4Pp2/4BQ2/PPP3PP/RN3RK1 w - - 2 10',
			},
		},
		'rnb2bnr/pppp1k1p/8/8/3qPp2/5Q2/PPP3PP/RNB2RK1 w - - 0 9' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'c1e3' => 'rnb2bnr/pppp1k1p/8/8/3qPp2/4BQ2/PPP3PP/RN3RK1 b - - 1 9',
			},
		},
		'rnb2rk1/1p2ppbp/p2p1np1/q1pP2B1/P1P1P3/2N5/1P2BPPP/R2QK1NR w KQ - 1 9' => {
			eco => 'E74i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 a6 8. a4 Qa5

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 a6 8. a4 Qa5'),
			moves => {
			},
		},
		'rnb2rk1/1pp1b1pp/3ppn2/p4p1q/2PP4/1PN1PNP1/PB3PBP/R2Q1RK1 b - - 0 10' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Bb2 Qh5 10. e3

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Bb2 Qh5 10. e3'),
			moves => {
			},
		},
		'rnb2rk1/1pp1b1pp/3ppn2/p4p1q/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 w - - 2 10' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 a5 9. Bb2 Qh5

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 a5 9. Bb2 Qh5'),
			moves => {
				'e2e3' => 'rnb2rk1/1pp1b1pp/3ppn2/p4p1q/2PP4/1PN1PNP1/PB3PBP/R2Q1RK1 b - - 0 10',
			},
		},
		'rnb2rk1/1pq1bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K w - - 6 10' => {
			eco => 'B92r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Kh1 Qc7

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Kh1 Qc7'),
			moves => {
			},
		},
		'rnb2rk1/1pq1bppp/p2ppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 3 10' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3 Qc7

			variation => N__('Sicilian: Scheveningen, Classical, Main Line, 9... Qc7'),
			moves => {
			},
		},
		'rnb2rk1/p1B1ppbp/1p4p1/3q4/3P4/4P3/PP3PPP/R2QKBNR w KQ - 0 9' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7 b6

			variation => N__('Grünfeld: Grünfeld Gambit Accepted, 8... b6'),
			moves => {
			},
		},
		'rnb2rk1/p1p1qpp1/1p2p2p/3N4/3P4/4PN2/PP3PPP/R2QKB1R b KQ - 0 10' => {
			eco => 'D59i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10.Nxd5'),
			moves => {
				'e6d5' => 'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4PN2/PP3PPP/R2QKB1R w KQ - 0 11',
			},
		},
		'rnb2rk1/p1p1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 1 10' => {
			eco => 'D59e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Rc1

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10. Rc1'),
			moves => {
				'c8b7' => 'rn3rk1/pbp1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/2RQKB1R w K - 2 11',
			},
		},
		'rnb2rk1/p1p1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 10' => {
			eco => 'D59d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7'),
			moves => {
				'a1c1' => 'rnb2rk1/p1p1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/2RQKB1R b K - 1 10',
				'c3d5' => 'rnb2rk1/p1p1qpp1/1p2p2p/3N4/3P4/4PN2/PP3PPP/R2QKB1R b KQ - 0 10',
			},
		},
		'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4P3/PP2BPPP/2RQK1NR b K - 1 11' => {
			eco => 'D54e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Rc1 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Nxd5 exd5 10. Bxe7 Qxe7 11. Be2

			variation => N__('QGD: Anti-Neo-Orthodox, Main Line, 11. Be2'),
			moves => {
			},
		},
		'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4P3/PP3PPP/2RQKBNR w K - 0 11' => {
			eco => 'D54d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Rc1 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Nxd5 exd5 10. Bxe7 Qxe7

			variation => N__('QGD: Anti-Neo-Orthodox, Main Line'),
			moves => {
				'f1e2' => 'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4P3/PP2BPPP/2RQK1NR b K - 1 11',
			},
		},
		'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4PN2/PP2BPPP/R2QK2R b KQ - 1 11' => {
			eco => 'D59j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Be2

			variation => N__('QGD: Tartakower, Main Line, 11. Rc1'),
			moves => {
			},
		},
		'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4PN2/PP3PPP/2RQKB1R b K - 1 11' => {
			eco => 'D59k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7 Qxe7 10. Nxd5 exd5 11. Rc1

			variation => N__('QGD: Tartakower, Main Line, 11. Rc1'),
			moves => {
				'c8e6' => 'rn3rk1/p1p1qpp1/1p2b2p/3p4/3P4/4PN2/PP3PPP/2RQKB1R w K - 2 12',
			},
		},
		'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4PN2/PP3PPP/R2QKB1R w KQ - 0 11' => {
			eco => 'D59i',
                        # TRANSLATORS:

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7 Qxe7 10.Nxd5'),
			moves => {
				'a1c1' => 'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4PN2/PP3PPP/2RQKB1R b K - 1 11',
				'f1e2' => 'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4PN2/PP2BPPP/R2QK2R b KQ - 1 11',
			},
		},
		'rnb2rk1/p1pqppbp/1p4p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 Qd7 9. O-O b6

			variation => N__('Grünfeld: Classical Exchange, Larsen Variation'),
			moves => {
			},
		},
		'rnb2rk1/pp1nqppp/4p3/2PpP3/5P2/2N2N2/PPP3PP/R2QKB1R b KQ - 0 9' => {
			eco => 'C14l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O 8. Nf3 c5 9. dxc5

			variation => N__('French: Classical, Steinitz, 8. Nf3 c5 9.dxc5'),
			moves => {
			},
		},
		'rnb2rk1/pp1nqppp/4p3/2ppP3/3P1P2/2N2N2/PPP3PP/R2QKB1R w KQ c6 0 9' => {
			eco => 'C14k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O 8. Nf3 c5

			variation => N__('French: Classical, Steinitz, 7...O-O 8. Nf3 c5'),
			moves => {
				'd1d2' => 'rnb2rk1/pp1nqppp/4p3/2ppP3/3P1P2/2N2N2/PPPQ2PP/R3KB1R b KQ - 1 9',
				'd4c5' => 'rnb2rk1/pp1nqppp/4p3/2PpP3/5P2/2N2N2/PPP3PP/R2QKB1R b KQ - 0 9',
			},
		},
		'rnb2rk1/pp1nqppp/4p3/2ppP3/3P1P2/2N2N2/PPPQ2PP/R3KB1R b KQ - 1 9' => {
			eco => 'C14m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O 8. Nf3 c5 9. Qd2

			variation => N__('French: Classical, Steinitz, 9. Qd2'),
			moves => {
				'b8c6' => 'r1b2rk1/pp1nqppp/2n1p3/2ppP3/3P1P2/2N2N2/PPPQ2PP/R3KB1R w KQ - 2 10',
			},
		},
		'rnb2rk1/pp2p1bp/2pp1np1/q2P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 Qa5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 Qa5'),
			moves => {
			},
		},
		'rnb2rk1/pp2p1bp/2pp1np1/q4p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. b3 Qa5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. b3 Qa5'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/1qp2np1/3p4/2PP4/1Q3NP1/PP2PPBP/RNB2RK1 w - - 2 8' => {
			eco => 'D78h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Qb3 Qb6

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Qb3 Qb6'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/2pp1np1/q7/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'E62h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O Qa5

			variation => N__('King\'s Indian: Fianchetto, Kavalek/Bronstein Variation'),
			moves => {
				'e2e4' => 'rnb2rk1/pp2ppbp/2pp1np1/q7/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8',
				'h2h3' => 'rnb2rk1/pp2ppbp/2pp1np1/q7/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnb2rk1/pp2ppbp/2pp1np1/q7/2PP4/2N2NPP/PP2PPB1/R1BQ1RK1 b - - 0 8' => {
			eco => 'E62i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O Qa5 8. h3

			variation => N__('King\'s Indian: Fianchetto, Kavalek/Bronstein, 8. h3'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/2pp1np1/q7/2PPP3/2N2NP1/PP3PBP/R1BQ1RK1 b - e3 0 8' => {
			eco => 'E62j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O Qa5 8. e4

			variation => N__('King\'s Indian: Fianchetto, Kavalek/Bronstein, 8. e4'),
			moves => {
				'c8g4' => 'rn3rk1/pp2ppbp/2pp1np1/q7/2PPP1b1/2N2NP1/PP3PBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1P3/2N2N2/PP1BBPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E74b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Averbakh, 6... c5 7.dxc5 Qa5'),
			moves => {
				'c8g4' => 'rn3rk1/pp2ppbp/3p1np1/2q5/2P1P1b1/2N2N2/PP1BBPPP/R2QK2R w KQ - 2 10',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1P3/2N5/PP1BBPPP/R2QK1NR w KQ - 0 9' => {
			eco => 'E74b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Averbakh, 6... c5 7.dxc5 Qa5'),
			moves => {
				'g1f3' => 'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1P3/2N2N2/PP1BBPPP/R2QK2R b KQ - 1 9',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1PP2/2NB1N2/PP2Q1PP/R1B1K2R b KQ - 1 9' => {
			eco => 'E76l',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7.dxc5'),
			moves => {
				'b8c6' => 'r1b2rk1/pp2ppbp/2np1np1/2q5/2P1PP2/2NB1N2/PP2Q1PP/R1B1K2R w KQ - 2 10',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1PP2/2NB1N2/PP4PP/R1BQK2R w KQ - 0 9' => {
			eco => 'E76l',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7.dxc5'),
			moves => {
				'd1e2' => 'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1PP2/2NB1N2/PP2Q1PP/R1B1K2R b KQ - 1 9',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/q1P3B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 1 8' => {
			eco => 'E74b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. dxc5 Qa5

			variation => N__('King\'s Indian: Averbakh, 6... c5 7.dxc5 Qa5'),
			moves => {
				'g5d2' => 'rnb2rk1/pp2ppbp/3p1np1/q1P5/2P1P3/2N5/PP1BBPPP/R2QK1NR b KQ - 2 8',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/q1P5/2P1P3/2N5/PP1BBPPP/R2QK1NR b KQ - 2 8' => {
			eco => 'E74b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Averbakh, 6... c5 7.dxc5 Qa5'),
			moves => {
				'a5c5' => 'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1P3/2N5/PP1BBPPP/R2QK1NR w KQ - 0 9',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/q1P5/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 1 8' => {
			eco => 'E76l',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7.dxc5'),
			moves => {
				'f1d3' => 'rnb2rk1/pp2ppbp/3p1np1/q1P5/2P1PP2/2NB1N2/PP4PP/R1BQK2R b KQ - 2 8',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/q1P5/2P1PP2/2NB1N2/PP4PP/R1BQK2R b KQ - 2 8' => {
			eco => 'E76l',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7.dxc5'),
			moves => {
				'a5c5' => 'rnb2rk1/pp2ppbp/3p1np1/2q5/2P1PP2/2NB1N2/PP4PP/R1BQK2R w KQ - 0 9',
			},
		},
		'rnb2rk1/pp2ppbp/3p1np1/q1pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 1 8' => {
			eco => 'E74e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 Qa5

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 Qa5'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/5np1/q1P5/2B2B2/2N1PN2/PP3PPP/2RQK2R b K - 0 9' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c5 7. dxc5 Qa5 8. Rc1 dxc4 9. Bxc4

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c5 7.dxc5 Qa5 8. Rc1 dxc4 9.Bxc4'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/5np1/q1P5/2p2B2/2N1PN2/PP3PPP/2RQKB1R w K - 0 9' => {
			eco => 'D93',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c5 7.dxc5 Qa5 8. Rc1'),
			moves => {
				'f1c4' => 'rnb2rk1/pp2ppbp/5np1/q1P5/2B2B2/2N1PN2/PP3PPP/2RQK2R b K - 0 9',
			},
		},
		'rnb2rk1/pp2ppbp/5np1/q1Pp4/2P2B2/2N1PN2/PP3PPP/2RQKB1R b K - 2 8' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c5 7. dxc5 Qa5 8. Rc1

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c5 7.dxc5 Qa5 8. Rc1'),
			moves => {
				'd5c4' => 'rnb2rk1/pp2ppbp/5np1/q1P5/2p2B2/2N1PN2/PP3PPP/2RQKB1R w K - 0 9',
			},
		},
		'rnb2rk1/pp2ppbp/5np1/q1Pp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 8' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c5 7. dxc5 Qa5

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c5 7.dxc5 Qa5'),
			moves => {
				'a1c1' => 'rnb2rk1/pp2ppbp/5np1/q1Pp4/2P2B2/2N1PN2/PP3PPP/2RQKB1R b K - 2 8',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/8/3PP3/4B3/P2N1PPP/2R1KB1R b K - 0 12' => {
			eco => 'D85y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 O-O 9. Rc1 Qa5 10. Qd2 cxd4 11. cxd4 Qxd2+ 12. Nxd2

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5 Queenswap, 12.Nxd2'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/6p1/8/3PP3/4BN2/P2K1PPP/2R2B1R b - - 0 12' => {
			eco => 'D85x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 O-O 9. Rc1 Qa5 10. Qd2 cxd4 11. cxd4 Qxd2+ 12. Kxd2

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5 Queenswap, 12.Kxd2'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/6p1/8/3PP3/4BN2/P2q1PPP/2R1KB1R w K - 0 12' => {
			eco => 'D85x',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5 10. Qd2'),
			moves => {
				'e1d2' => 'rnb2rk1/pp2ppbp/6p1/8/3PP3/4BN2/P2K1PPP/2R2B1R b - - 0 12',
				'f3d2' => 'rnb2rk1/pp2ppbp/6p1/8/3PP3/4B3/P2N1PPP/2R1KB1R b K - 0 12',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/8/3PP3/5N2/q2BBPPP/1R1Q1RK1 b - - 1 12' => {
			eco => 'D85r',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd 10.cd Qa5+ 11. Bd2'),
			moves => {
				'c8g4' => 'rn3rk1/pp2ppbp/6p1/8/3PP1b1/5N2/q2BBPPP/1R1Q1RK1 w - - 2 13',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/8/3PP3/5N2/q2BBPPP/1R1QK2R w K - 0 12' => {
			eco => 'D85r',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd 10.cd Qa5+ 11. Bd2'),
			moves => {
				'e1g1' => 'rnb2rk1/pp2ppbp/6p1/8/3PP3/5N2/q2BBPPP/1R1Q1RK1 b - - 1 12',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 b - - 3 10' => {
			eco => 'D87c',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, 8... c5 9. Be3 Qa5'),
			moves => {
				'b8d7' => 'r1b2rk1/pp1nppbp/6p1/q1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 w - - 4 11',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q1p5/2BPP3/2P1B3/P3NPPP/R2QK2R w KQ - 2 10' => {
			eco => 'D87c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. Be3 Qa5

			variation => N__('Grünfeld: Classical Exchange, 8... c5 9. Be3 Qa5'),
			moves => {
				'e1g1' => 'rnb2rk1/pp2ppbp/6p1/q1p5/2BPP3/2P1B3/P3NPPP/R2Q1RK1 b - - 3 10',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P2Q1PPP/2R1KB1R b K - 5 10' => {
			eco => 'D85x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 O-O 9. Rc1 Qa5 10. Qd2

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5 10. Qd2'),
			moves => {
				'c5d4' => 'rnb2rk1/pp2ppbp/6p1/q7/3pP3/2P1BN2/P2Q1PPP/2R1KB1R w K - 0 11',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P4PPP/2RQKB1R w K - 4 10' => {
			eco => 'D85x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 O-O 9. Rc1 Qa5

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5'),
			moves => {
				'd1d2' => 'rnb2rk1/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P2Q1PPP/2R1KB1R b K - 5 10',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q1p5/3PP3/2P2N2/P3BPPP/1RBQK2R w K - 4 10' => {
			eco => 'D85m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 Qa5

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 Qa5'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q7/3PP3/4BN2/P2Q1PPP/2R1KB1R b K - 0 11' => {
			eco => 'D85x',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5 10. Qd2'),
			moves => {
				'a5d2' => 'rnb2rk1/pp2ppbp/6p1/8/3PP3/4BN2/P2q1PPP/2R1KB1R w K - 0 12',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q7/3PP3/5N2/P2BBPPP/1R1QK2R b K - 2 11' => {
			eco => 'D85r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4 10. cxd4 Qa5+ 11. Bd2

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd 10.cd Qa5+ 11. Bd2'),
			moves => {
				'a5a2' => 'rnb2rk1/pp2ppbp/6p1/8/3PP3/5N2/q2BBPPP/1R1QK2R w K - 0 12',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q7/3PP3/5N2/P2QBPPP/1RB1K2R b K - 2 11' => {
			eco => 'D85q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4 10. cxd4 Qa5+ 11. Qd2

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd 10.cd Qa5+ 11. Qd2'),
			moves => {
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q7/3PP3/5N2/P3BPPP/1RBQK2R w K - 1 11' => {
			eco => 'D85q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4 10. cxd4 Qa5+

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd 10.cd Qa5+'),
			moves => {
				'c1d2' => 'rnb2rk1/pp2ppbp/6p1/q7/3PP3/5N2/P2BBPPP/1R1QK2R b K - 2 11',
				'd1d2' => 'rnb2rk1/pp2ppbp/6p1/q7/3PP3/5N2/P2QBPPP/1RB1K2R b K - 2 11',
			},
		},
		'rnb2rk1/pp2ppbp/6p1/q7/3pP3/2P1BN2/P2Q1PPP/2R1KB1R w K - 0 11' => {
			eco => 'D85x',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O 9. Rc1 Qa5 10. Qd2'),
			moves => {
				'c3d4' => 'rnb2rk1/pp2ppbp/6p1/q7/3PP3/4BN2/P2Q1PPP/2R1KB1R b K - 0 11',
			},
		},
		'rnb2rk1/pp2q1pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R1Q2RK1 b - - 2 10' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Bxa3 9. Nxa3 Qe7 10. Qc1

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Bxa3 9.Nxa3 Qe7 10. Qc1'),
			moves => {
			},
		},
		'rnb2rk1/pp2q1pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R2Q1RK1 w - - 1 10' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Bxa3 9. Nxa3 Qe7

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Bxa3 9.Nxa3 Qe7'),
			moves => {
				'd1c1' => 'rnb2rk1/pp2q1pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R1Q2RK1 b - - 2 10',
			},
		},
		'rnb2rk1/pp2qpp1/2p1p2p/3p4/2PPn3/2N1PN2/PP3PPP/2RQKB1R w K - 0 10' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Rc1 c6

			variation => N__('QGD: Lasker Defence, 9. Rc1 c6'),
			moves => {
				'f1d3' => 'rnb2rk1/pp2qpp1/2p1p2p/3p4/2PPn3/2NBPN2/PP3PPP/2RQK2R b K - 1 10',
			},
		},
		'rnb2rk1/pp2qpp1/2p1p2p/3p4/2PPn3/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 10' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Qc2 c6

			variation => N__('QGD: Lasker Defence, Teichmann, 9... c6'),
			moves => {
			},
		},
		'rnb2rk1/pp2qpp1/2p1p2p/3p4/2PPn3/2NBPN2/PP3PPP/2RQK2R b K - 1 10' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Rc1 c6 10. Bd3

			variation => N__('QGD: Lasker Defence, 9. Rc1 c6 10. Bd3'),
			moves => {
			},
		},
		'rnb2rk1/pp2qppp/4pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 Qe7

			variation => N__('Nimzo-Indian: Main Line, Smyslov Variation'),
			moves => {
			},
		},
		'rnb2rk1/ppB1ppbp/6p1/3q4/3P4/4P3/PP3PPP/R2QKBNR b KQ - 0 8' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Nxd5 Qxd5 8. Bxc7

			variation => N__('Grünfeld: Grünfeld Gambit Accepted'),
			moves => {
				'b7b6' => 'rnb2rk1/p1B1ppbp/1p4p1/3q4/3P4/4P3/PP3PPP/R2QKBNR w KQ - 0 9',
				'b8a6' => 'r1b2rk1/ppB1ppbp/n5p1/3q4/3P4/4P3/PP3PPP/R2QKBNR w KQ - 1 9',
				'b8c6' => 'r1b2rk1/ppB1ppbp/2n3p1/3q4/3P4/4P3/PP3PPP/R2QKBNR w KQ - 1 9',
				'c8f5' => 'rn3rk1/ppB1ppbp/6p1/3q1b2/3P4/4P3/PP3PPP/R2QKBNR w KQ - 1 9',
			},
		},
		'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 Qh5

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 Qh5'),
			moves => {
				'c1b2' => 'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 b - - 2 9',
			},
		},
		'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/1PN2NP1/PB2PPBP/R2Q1RK1 b - - 2 9' => {
			eco => 'A99',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. b3 Qh5 9. Bb2

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. b3 Qh5 9. Bb2'),
			moves => {
			},
		},
		'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 w - - 4 9' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1 Qh5

			variation => N__('Dutch: Ilyin-Zhenevsky, Winter, 8... Qh5'),
			moves => {
			},
		},
		'rnb2rk1/ppp1b1pp/3ppn2/5p1q/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 4 9' => {
			eco => 'A98',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Qc2 Qh5

			variation => N__('Dutch: Ilyin-Zhenevsky, 8. Qc2 Qh5'),
			moves => {
			},
		},
		'rnb2rk1/ppp1b1pp/3ppnq1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 w - - 4 9' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1 Qg6

			variation => N__('Dutch: Ilyin-Zhenevsky, Winter, 8... Qg6'),
			moves => {
				'e2e4' => 'rnb2rk1/ppp1b1pp/3ppnq1/5p2/2PPP3/2N2NP1/PP3PBP/R1BQR1K1 b - e3 0 9',
			},
		},
		'rnb2rk1/ppp1b1pp/3ppnq1/5p2/2PPP3/2N2NP1/PP3PBP/R1BQR1K1 b - e3 0 9' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1 Qg6 9. e4

			variation => N__('Dutch: Ilyin-Zhenevsky, Winter, 8... Qg6 9. e4'),
			moves => {
				'f5e4' => 'rnb2rk1/ppp1b1pp/3ppnq1/8/2PPp3/2N2NP1/PP3PBP/R1BQR1K1 w - - 0 10',
			},
		},
		'rnb2rk1/ppp1b1pp/3ppnq1/8/2PPp3/2N2NP1/PP3PBP/R1BQR1K1 w - - 0 10' => {
			eco => 'A97',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 Qe8 8. Re1 Qg6 9. e4 fxe4

			variation => N__('Dutch: Ilyin-Zhenevsky, Winter, 8... Qg6 9. e4 fxe4'),
			moves => {
			},
		},
		'rnb2rk1/ppp1ppbp/6p1/3q4/3P1B2/4P3/PP3PPP/R2QKBNR w KQ - 0 8' => {
			eco => 'D84',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Grünfeld Gambit Accepted'),
			moves => {
				'f4c7' => 'rnb2rk1/ppB1ppbp/6p1/3q4/3P4/4P3/PP3PPP/R2QKBNR b KQ - 0 8',
			},
		},
		'rnb2rk1/ppp1qpbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2QK2R w KQ - 2 8' => {
			eco => 'E92j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 Qe7

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... Qe7'),
			moves => {
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3P4/3P4/2P1PN2/P4PPP/R2QKB1R b KQ - 0 10' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3 10. bxc3

			variation => N__('QGD: Lasker Defence, Main Line'),
			moves => {
				'e6d5' => 'rnb2rk1/ppp1qpp1/7p/3p4/3P4/2P1PN2/P4PPP/R2QKB1R w KQ - 0 11',
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3P4/3P4/2n1PN2/PP3PPP/R2QKB1R w KQ - 0 10' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3

			variation => N__('QGD: Lasker Defence, Main Line'),
			moves => {
				'b2c3' => 'rnb2rk1/ppp1qpp1/4p2p/3P4/3P4/2P1PN2/P4PPP/R2QKB1R b KQ - 0 10',
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3P4/3Pn3/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 9' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5

			variation => N__('QGD: Lasker Defence, Main Line'),
			moves => {
				'e4c3' => 'rnb2rk1/ppp1qpp1/4p2p/3P4/3P4/2n1PN2/PP3PPP/R2QKB1R w KQ - 0 10',
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3p4/2PP4/2n1PN2/PPQ2PPP/R3KB1R w KQ - 0 10' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Qc2 Nxc3

			variation => N__('QGD: Lasker Defence, Teichmann, 9...Nxc3'),
			moves => {
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPN3/4PN2/PP3PPP/R2QKB1R b KQ - 0 9' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Nxe4

			variation => N__('QGD: Lasker Defence, 9.Nxe4'),
			moves => {
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPn3/2N1PN2/PP3PPP/2RQKB1R b K - 1 9' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Rc1

			variation => N__('QGD: Lasker Defence, 9. Rc1'),
			moves => {
				'c7c6' => 'rnb2rk1/pp2qpp1/2p1p2p/3p4/2PPn3/2N1PN2/PP3PPP/2RQKB1R w K - 0 10',
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPn3/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 9' => {
			eco => 'D56',
                        # TRANSLATORS:

			variation => N__('QGD: Lasker Defence'),
			moves => {
				'a1c1' => 'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPn3/2N1PN2/PP3PPP/2RQKB1R b K - 1 9',
				'c3e4' => 'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPN3/4PN2/PP3PPP/R2QKB1R b KQ - 0 9',
				'c4d5' => 'rnb2rk1/ppp1qpp1/4p2p/3P4/3Pn3/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 9',
				'd1c2' => 'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPn3/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 9',
			},
		},
		'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPn3/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 9' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. Qc2

			variation => N__('QGD: Lasker Defence, Teichmann Variation'),
			moves => {
				'c7c6' => 'rnb2rk1/pp2qpp1/2p1p2p/3p4/2PPn3/2N1PN2/PPQ2PPP/R3KB1R w KQ - 0 10',
				'e4c3' => 'rnb2rk1/ppp1qpp1/4p2p/3p4/2PP4/2n1PN2/PPQ2PPP/R3KB1R w KQ - 0 10',
			},
		},
		'rnb2rk1/ppp1qpp1/7p/3p4/3P4/1QP1PN2/P4PPP/R3KB1R b KQ - 1 11' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3 10. bxc3 exd5 11. Qb3

			variation => N__('QGD: Lasker Defence, Main Line, 11. Qb3'),
			moves => {
				'e7d6' => 'rnb2rk1/ppp2pp1/3q3p/3p4/3P4/1QP1PN2/P4PPP/R3KB1R w KQ - 2 12',
				'f8d8' => 'rnbr2k1/ppp1qpp1/7p/3p4/3P4/1QP1PN2/P4PPP/R3KB1R w KQ - 2 12',
			},
		},
		'rnb2rk1/ppp1qpp1/7p/3p4/3P4/2P1PN2/P4PPP/R2QKB1R w KQ - 0 11' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3 10. bxc3 exd5

			variation => N__('QGD: Lasker Defence, Main Line'),
			moves => {
				'd1b3' => 'rnb2rk1/ppp1qpp1/7p/3p4/3P4/1QP1PN2/P4PPP/R3KB1R b KQ - 1 11',
			},
		},
		'rnb2rk1/ppp2pbp/3p2p1/3Pp2n/2P1P2q/2N1BP2/PP1Q2PP/R3KBNR w KQ - 3 9' => {
			eco => 'E87g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 Qh4+

			variation => N__('King\'s Indian: Sämisch, Orthodox, Bronstein Variation'),
			moves => {
				'e3f2' => 'rnb2rk1/ppp2pbp/3p2p1/3Pp2n/2P1P2q/2N2P2/PP1Q1BPP/R3KBNR b KQ - 4 9',
				'g2g3' => 'rnb2rk1/ppp2pbp/3p2p1/3Pp2n/2P1P2q/2N1BPP1/PP1Q3P/R3KBNR b KQ - 0 9',
			},
		},
		'rnb2rk1/ppp2pbp/3p2p1/3Pp2n/2P1P2q/2N1BPP1/PP1Q3P/R3KBNR b KQ - 0 9' => {
			eco => 'E87h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 Qh4+ 9. g3

			variation => N__('King\'s Indian: Sämisch, Orthodox, Bronstein, 9. g3'),
			moves => {
			},
		},
		'rnb2rk1/ppp2pbp/3p2p1/3Pp2n/2P1P2q/2N2P2/PP1Q1BPP/R3KBNR b KQ - 4 9' => {
			eco => 'E87i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 Qh4+ 9. Bf2

			variation => N__('King\'s Indian: Sämisch, Orthodox, Bronstein, 9. Bf2'),
			moves => {
			},
		},
		'rnb2rk1/ppp2pp1/3q3p/3p4/2PP4/1Q2PN2/P4PPP/R3KB1R b KQ - 0 12' => {
			eco => 'D57',
                        # TRANSLATORS:

			variation => N__('QGD: Lasker Defence, Main Line, Bernstein Variation'),
			moves => {
				'd5c4' => 'rnb2rk1/ppp2pp1/3q3p/8/2pP4/1Q2PN2/P4PPP/R3KB1R w KQ - 0 13',
			},
		},
		'rnb2rk1/ppp2pp1/3q3p/3p4/3P4/1QP1PN2/P4PPP/R3KB1R w KQ - 2 12' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3 10. bxc3 exd5 11. Qb3 Qd6

			variation => N__('QGD: Lasker Defence, Main Line, Bernstein Variation'),
			moves => {
				'c3c4' => 'rnb2rk1/ppp2pp1/3q3p/3p4/2PP4/1Q2PN2/P4PPP/R3KB1R b KQ - 0 12',
			},
		},
		'rnb2rk1/ppp2pp1/3q3p/8/2BP4/1Q2PN2/P4PPP/R3K2R b KQ - 0 13' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3 10. bxc3 exd5 11. Qb3 Qd6 12. c4 dxc4 13. Bxc4

			variation => N__('QGD: Lasker Defence, Main Line, Bernstein, 12. c4 dxc4 13.Bxc4'),
			moves => {
			},
		},
		'rnb2rk1/ppp2pp1/3q3p/8/2pP4/1Q2PN2/P4PPP/R3KB1R w KQ - 0 13' => {
			eco => 'D57',
                        # TRANSLATORS:

			variation => N__('QGD: Lasker Defence, Main Line, Bernstein Variation'),
			moves => {
				'f1c4' => 'rnb2rk1/ppp2pp1/3q3p/8/2BP4/1Q2PN2/P4PPP/R3K2R b KQ - 0 13',
			},
		},
		'rnb2rk1/ppp2pp1/4pq1p/3p4/1bPP4/2N1PN2/PP3PPP/2RQKB1R b K - 2 8' => {
			eco => 'D38p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 h6 6. Bxf6 Qxf6 7. e3 O-O 8. Rc1

			variation => N__('QGD: Ragozin, 5. Bg5 h6 6.Bxf6 Qxf6 7. e3 O-O 8. Rc1'),
			moves => {
			},
		},
		'rnb2rk1/ppp2pp1/4pq1p/3p4/1bPP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 8' => {
			eco => 'D38o',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, 5. Bg5 h6 6.Bxf6 Qxf6 7. e3'),
			moves => {
				'a1c1' => 'rnb2rk1/ppp2pp1/4pq1p/3p4/1bPP4/2N1PN2/PP3PPP/2RQKB1R b K - 2 8',
			},
		},
		'rnb2rk1/pppnqppp/4p3/3pP3/3P1P2/2N2N2/PPP3PP/R2QKB1R b KQ - 2 8' => {
			eco => 'C14j',
                        # TRANSLATORS:

			variation => N__('French: Classical, Steinitz, 7...O-O'),
			moves => {
				'c7c5' => 'rnb2rk1/pp1nqppp/4p3/2ppP3/3P1P2/2N2N2/PPP3PP/R2QKB1R w KQ c6 0 9',
			},
		},
		'rnb2rk1/pppnqppp/4p3/3pP3/3P1P2/2N5/PPP3PP/R2QKBNR w KQ - 1 8' => {
			eco => 'C14j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. Bxe7 Qxe7 7. f4 O-O

			variation => N__('French: Classical, Steinitz, 7...O-O'),
			moves => {
				'g1f3' => 'rnb2rk1/pppnqppp/4p3/3pP3/3P1P2/2N2N2/PPP3PP/R2QKB1R b KQ - 2 8',
			},
		},
		'rnb2rk1/ppppqppp/4pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R w KQ - 1 6' => {
			eco => 'E11q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 O-O

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 O-O'),
			moves => {
				'f1g2' => 'rnb2rk1/ppppqppp/4pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R b KQ - 2 6',
			},
		},
		'rnb2rk1/ppppqppp/4pn2/8/1bPP4/5NP1/PP1BPPBP/RN1QK2R b KQ - 2 6' => {
			eco => 'E11q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Qe7 5. g3 O-O 6. Bg2

			variation => N__('Bogo-Indian: Nimzowitsch, 5. g3 O-O'),
			moves => {
			},
		},
		'rnb2rk1/pppqppbp/6p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 6 9' => {
			eco => 'D86',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, 8. Ne2'),
			moves => {
				'b7b6' => 'rnb2rk1/p1pqppbp/1p4p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'rnb2rk1/pppqppbp/6p1/8/2BPP3/2P5/P3NPPP/R1BQK2R w KQ - 5 9' => {
			eco => 'D86',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Classical Exchange, 8. Ne2'),
			moves => {
				'e1g1' => 'rnb2rk1/pppqppbp/6p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 6 9',
			},
		},
		'rnbQ1rk1/pp2ppbp/5np1/2p5/2P1P3/2N1BP2/PP4PP/R3KBNR b KQ - 0 8' => {
			eco => 'E81n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5 7. dxc5 dxc5 8. Qxd8

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 Queenswap'),
			moves => {
			},
		},
		'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BN2/PP3PPP/R3KB1R b KQ - 0 8' => {
			eco => 'E90j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be3 e5 7. dxe5 dxe5 8. Qxd8

			variation => N__('King\'s Indian: Larsen, 6... e5, Queenswap'),
			moves => {
			},
		},
		'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BP2/PP4PP/R3KBNR b KQ - 0 8' => {
			eco => 'E85c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. dxe5 dxe5 8. Qxd8

			variation => N__('King\'s Indian: Sämisch, Orthodox, Queenswap'),
			moves => {
			},
		},
		'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N1P/PP3PP1/R1B1KB1R b KQ - 0 8' => {
			eco => 'E90o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. dxe5 dxe5 8. Qxd8

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 Queenswap'),
			moves => {
			},
		},
		'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N2/PP2BPPP/R1B1K2R b KQ - 0 8' => {
			eco => 'E92b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap'),
			moves => {
				'f8d8' => 'rnbr2k1/ppp2pbp/5np1/4p3/2P1P3/2N2N2/PP2BPPP/R1B1K2R w KQ - 0 9',
			},
		},
		'rnbQk1nr/ppp2pbp/6p1/4p3/2P1P3/2N5/PP3PPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'A42v',
                        # TRANSLATORS:

			variation => N__('Modern: Averbakh, 4... e5 5.dxe5 dxe5'),
			moves => {
				'e8d8' => 'rnbk2nr/ppp2pbp/6p1/4p3/2P1P3/2N5/PP3PPP/R1B1KBNR w KQ - 0 7',
			},
		},
		'rnbQk2r/pp2ppbp/5np1/2p5/2P1P3/2N2P2/PP4PP/R1B1KBNR b KQkq - 0 7' => {
			eco => 'E80h',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, 5... c5'),
			moves => {
				'e8d8' => 'rnbk3r/pp2ppbp/5np1/2p5/2P1P3/2N2P2/PP4PP/R1B1KBNR w KQ - 0 8',
			},
		},
		'rnbQkb1r/ppp2ppp/5n2/4p3/2P5/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'A53',
                        # TRANSLATORS:

			variation => N__('Old Indian: Ukrainian, 4.dxe5'),
			moves => {
				'e8d8' => 'rnbk1b1r/ppp2ppp/5n2/4p3/2P5/2N5/PP2PPPP/R1B1KBNR w KQ - 0 6',
			},
		},
		'rnbQkb1r/ppp2ppp/5n2/4p3/4P3/2N5/PPP2PPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'B07m',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 e5 4. dxe5 dxe5 5. Qxd8+

			variation => N__('Pirc: 3. Nc3 e5 Queenswap'),
			moves => {
			},
		},
		'rnbQkb1r/ppp2ppp/5n2/4p3/4P3/5P2/PPP3PP/RNB1KBNR b KQkq - 0 5' => {
			eco => 'B07f',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. f3 e5 4. dxe5 dxe5 5. Qxd8+

			variation => N__('Pirc: 3. f3 e5 Queenswap'),
			moves => {
			},
		},
		'rnbQkbnr/ppp2ppp/8/4p3/2P5/8/PP2PPPP/RNB1KBNR b KQkq - 0 4' => {
			eco => 'A41o',
                        # TRANSLATORS:

			variation => N__('Neo-Old Indian: 2. c4 e5 3.dxe5'),
			moves => {
				'e8d8' => 'rnbk1bnr/ppp2ppp/8/4p3/2P5/8/PP2PPPP/RNB1KBNR w KQ - 0 5',
			},
		},
		'rnbQkbnr/ppp2ppp/8/4p3/4P3/8/PPP2PPP/RNB1KBNR b KQkq - 0 4' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 d6 3. dxe5 dxe5 4. Qxd8+

			variation => N__('Centre Game: Queenswap line'),
			moves => {
			},
		},
		'rnbk1b1r/ppp2ppp/5n2/4p3/2P5/2N2N2/PP2PPPP/R1B1KB1R b KQ - 1 6' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. dxe5 dxe5 5. Qxd8+ Kxd8 6. Nf3

			variation => N__('Old Indian: Ukrainian, Queenswap, 6. Nf3'),
			moves => {
				'f6d7' => 'rnbk1b1r/pppn1ppp/8/4p3/2P5/2N2N2/PP2PPPP/R1B1KB1R w KQ - 2 7',
			},
		},
		'rnbk1b1r/ppp2ppp/5n2/4p3/2P5/2N5/PP2PPPP/R1B1KBNR w KQ - 0 6' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. dxe5 dxe5 5. Qxd8+ Kxd8

			variation => N__('Old Indian: Ukrainian, Queenswap'),
			moves => {
				'g1f3' => 'rnbk1b1r/ppp2ppp/5n2/4p3/2P5/2N2N2/PP2PPPP/R1B1KB1R b KQ - 1 6',
			},
		},
		'rnbk1b1r/pppn1ppp/8/4p3/2P5/2N2N2/PP2PPPP/R1B1KB1R w KQ - 2 7' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. dxe5 dxe5 5. Qxd8+ Kxd8 6. Nf3 Nfd7

			variation => N__('Old Indian: Ukrainian, Queenswap, 6. Nf3 Nfd7'),
			moves => {
			},
		},
		'rnbk1bnr/ppp2ppp/8/4p3/2P5/8/PP2PPPP/RNB1KBNR w KQ - 0 5' => {
			eco => 'A41o',
                        # TRANSLATORS: 1. d4 d6 2. c4 e5 3. dxe5 dxe5 4. Qxd8+ Kxd8

			variation => N__('Neo-Old Indian: Queenswap'),
			moves => {
			},
		},
		'rnbk1bnr/pppp1B1p/6p1/4N2Q/3Pp3/8/PPP2PqP/RNB1K2R w KQ - 2 8' => {
			eco => 'C40q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4 4. Nxe5 Qg5 5. d4 Qxg2 6. Qh5+ g6 7. Bf7+ Kd8

			variation => N__('Latvian Gambit: Poisoned Pawn, Main Line'),
			moves => {
			},
		},
		'rnbk1bnr/pppp2pp/8/8/2B1pp1q/2N5/PPPPQ1PP/R1BK2NR w - - 2 7' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 f5 4. Qe2 Qh4+ 5. Kd1 fxe4 6. Nc3 Kd8

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit, Hein Variation'),
			moves => {
			},
		},
		'rnbk2nr/ppp2pbp/6p1/4p3/2P1P3/2N5/PP3PPP/R1B1KBNR w KQ - 0 7' => {
			eco => 'A42v',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. dxe5 dxe5 6. Qxd8+ Kxd8

			variation => N__('Modern: Averbakh, Queenswap'),
			moves => {
				'f2f4' => 'rnbk2nr/ppp2pbp/6p1/4p3/2P1PP2/2N5/PP4PP/R1B1KBNR b KQ f3 0 7',
			},
		},
		'rnbk2nr/ppp2pbp/6p1/4p3/2P1PP2/2N5/PP4PP/R1B1KBNR b KQ f3 0 7' => {
			eco => 'A42w',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. dxe5 dxe5 6. Qxd8+ Kxd8 7. f4

			variation => N__('Modern: Averbakh, Queenswap, 7. f4'),
			moves => {
			},
		},
		'rnbk3r/pp2ppbp/5np1/2p5/2P1P3/2N2P2/PP4PP/R1B1KBNR w KQ - 0 8' => {
			eco => 'E80i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c5 6. dxc5 dxc5 7. Qxd8+ Kxd8

			variation => N__('King\'s Indian: Sämisch, 5... c5 Queenswap'),
			moves => {
			},
		},
		'rnbq1b1r/ppp2kpp/3p1n2/8/3PP3/8/PPP2PPP/RNBQKB1R b KQ d3 0 5' => {
			eco => 'C42h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nxf7 Kxf7 5. d4

			variation => N__('Russian Game: Cochrane Gambit, 5. d4'),
			moves => {
			},
		},
		'rnbq1b1r/ppp2kpp/3p1n2/8/4P3/8/PPPP1PPP/RNBQKB1R w KQ - 0 5' => {
			eco => 'C42h',
                        # TRANSLATORS:

			variation => N__('Russian Game: Cochrane Gambit'),
			moves => {
				'd2d4' => 'rnbq1b1r/ppp2kpp/3p1n2/8/3PP3/8/PPP2PPP/RNBQKB1R b KQ d3 0 5',
			},
		},
		'rnbq1b1r/pppp1k2/5n1p/8/4PQ1P/8/PPPP2P1/RNB1KB1R b KQ - 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'f8d6' => 'rnbq3r/pppp1k2/3b1n1p/8/4PQ1P/8/PPPP2P1/RNB1KB1R w KQ - 1 9',
			},
		},
		'rnbq1b1r/pppp1k2/5n1p/8/4PpQP/8/PPPP2P1/RNB1KB1R w KQ - 1 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'g4f4' => 'rnbq1b1r/pppp1k2/5n1p/8/4PQ1P/8/PPPP2P1/RNB1KB1R b KQ - 0 8',
			},
		},
		'rnbq1bnr/ppp2k2/7p/3B4/4PppP/8/PPPP2P1/RNBQK2R b KQ - 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Walker Attack'),
			moves => {
				'f7g7' => 'rnbq1bnr/ppp3k1/7p/3B4/4PppP/8/PPPP2P1/RNBQK2R w KQ - 1 9',
			},
		},
		'rnbq1bnr/ppp2k2/7p/3p4/2B1PppP/8/PPPP2P1/RNBQK2R w KQ d6 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Walker Attack'),
			moves => {
				'c4d5' => 'rnbq1bnr/ppp2k2/7p/3B4/4PppP/8/PPPP2P1/RNBQK2R b KQ - 0 8',
			},
		},
		'rnbq1bnr/ppp2k2/7p/3p4/3PPBpP/8/PPP3P1/RN1QKB1R b KQ - 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Thorold Variation'),
			moves => {
				'd5e4' => 'rnbq1bnr/ppp2k2/7p/8/3PpBpP/8/PPP3P1/RN1QKB1R w KQ - 0 9',
			},
		},
		'rnbq1bnr/ppp2k2/7p/3p4/3PPppP/8/PPP3P1/RNBQKB1R w KQ d6 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Thorold Variation'),
			moves => {
				'c1f4' => 'rnbq1bnr/ppp2k2/7p/3p4/3PPBpP/8/PPP3P1/RN1QKB1R b KQ - 0 8',
			},
		},
		'rnbq1bnr/ppp2k2/7p/8/2BPpBpP/8/PPP3P1/RN1QK2R b KQ - 1 9' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Thorold Variation'),
			moves => {
				'f7g7' => 'rnbq1bnr/ppp3k1/7p/8/2BPpBpP/8/PPP3P1/RN1QK2R w KQ - 2 10',
			},
		},
		'rnbq1bnr/ppp2k2/7p/8/3PpBpP/8/PPP3P1/RN1QKB1R w KQ - 0 9' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Thorold Variation'),
			moves => {
				'f1c4' => 'rnbq1bnr/ppp2k2/7p/8/2BPpBpP/8/PPP3P1/RN1QK2R b KQ - 1 9',
			},
		},
		'rnbq1bnr/ppp3k1/7p/3B4/3PPppP/8/PPP3P1/RNBQK2R b KQ d3 0 9' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 h6 6. Nxf7 Kxf7 7. Bc4+ d5 8. Bxd5+ Kg7 9. d4

			variation => N__('KGA: Allgaier, Urusov Attack'),
			moves => {
			},
		},
		'rnbq1bnr/ppp3k1/7p/3B4/4PppP/8/PPPP2P1/RNBQK2R w KQ - 1 9' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Walker Attack'),
			moves => {
				'd2d4' => 'rnbq1bnr/ppp3k1/7p/3B4/3PPppP/8/PPP3P1/RNBQK2R b KQ d3 0 9',
			},
		},
		'rnbq1bnr/ppp3k1/7p/4B3/2BPp1pP/8/PPP3P1/RN1QK2R b KQ - 3 10' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 h6 6. Nxf7 Kxf7 7. d4 d5 8. Bxf4 dxe4 9. Bc4+ Kg7 10. Be5+

			variation => N__('KGA: Allgaier, Cook Variation'),
			moves => {
			},
		},
		'rnbq1bnr/ppp3k1/7p/8/2BPpBpP/8/PPP3P1/RN1QK2R w KQ - 2 10' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier, Thorold Variation'),
			moves => {
				'f4e5' => 'rnbq1bnr/ppp3k1/7p/4B3/2BPp1pP/8/PPP3P1/RN1QK2R b KQ - 3 10',
			},
		},
		'rnbq1bnr/pppp1k1p/8/8/4Pp2/5Q2/PPPP2PP/RNB2RK1 b - - 0 7' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'd8f6' => 'rnb2bnr/pppp1k1p/5q2/8/4Pp2/5Q2/PPPP2PP/RNB2RK1 w - - 1 8',
			},
		},
		'rnbq1bnr/pppp1k1p/8/8/4Pp2/5p2/PPPP2PP/RNBQ1RK1 w - - 0 7' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'd1f3' => 'rnbq1bnr/pppp1k1p/8/8/4Pp2/5Q2/PPPP2PP/RNB2RK1 b - - 0 7',
			},
		},
		'rnbq1bnr/pppp1k1p/8/8/4Ppp1/5N2/PPPP2PP/RNBQ1RK1 b - - 1 6' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'g4f3' => 'rnbq1bnr/pppp1k1p/8/8/4Pp2/5p2/PPPP2PP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1bnr/pppp1k1p/8/8/4Ppp1/5N2/PPPP2PP/RNBQK2R w KQ - 0 6' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'e1g1' => 'rnbq1bnr/pppp1k1p/8/8/4Ppp1/5N2/PPPP2PP/RNBQ1RK1 b - - 1 6',
			},
		},
		'rnbq1bnr/pppp1k2/7p/8/2B1PppP/8/PPPP2P1/RNBQK2R b KQ - 1 7' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 h6 6. Nxf7 Kxf7 7. Bc4+

			variation => N__('KGA: Allgaier, Walker Attack'),
			moves => {
				'd7d5' => 'rnbq1bnr/ppp2k2/7p/3p4/2B1PppP/8/PPPP2P1/RNBQK2R w KQ d6 0 8',
			},
		},
		'rnbq1bnr/pppp1k2/7p/8/3PPppP/8/PPP3P1/RNBQKB1R b KQ d3 0 7' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 h6 6. Nxf7 Kxf7 7. d4

			variation => N__('KGA: Allgaier, Thorold Variation'),
			moves => {
				'd7d5' => 'rnbq1bnr/ppp2k2/7p/3p4/3PPppP/8/PPP3P1/RNBQKB1R w KQ d6 0 8',
			},
		},
		'rnbq1bnr/pppp1k2/7p/8/4PpQP/8/PPPP2P1/RNB1KB1R b KQ - 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'g8f6' => 'rnbq1b1r/pppp1k2/5n1p/8/4PpQP/8/PPPP2P1/RNB1KB1R w KQ - 1 8',
			},
		},
		'rnbq1bnr/pppp1k2/7p/8/4PppP/2N5/PPPP2P1/R1BQKB1R b KQ - 1 7' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 h6 6. Nxf7 Kxf7 7. Nc3

			variation => N__('KGA: Allgaier, Blackburne Gambit'),
			moves => {
			},
		},
		'rnbq1bnr/pppp1k2/7p/8/4PppP/8/PPPP2P1/RNBQKB1R w KQ - 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'b1c3' => 'rnbq1bnr/pppp1k2/7p/8/4PppP/2N5/PPPP2P1/R1BQKB1R b KQ - 1 7',
				'd1g4' => 'rnbq1bnr/pppp1k2/7p/8/4PpQP/8/PPPP2P1/RNB1KB1R b KQ - 0 7',
				'd2d4' => 'rnbq1bnr/pppp1k2/7p/8/3PPppP/8/PPP3P1/RNBQKB1R b KQ d3 0 7',
				'f1c4' => 'rnbq1bnr/pppp1k2/7p/8/2B1PppP/8/PPPP2P1/RNBQK2R b KQ - 1 7',
			},
		},
		'rnbq1k1r/pp2nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQ - 3 8' => {
			eco => 'C18l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 Kf8

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 Kf8'),
			moves => {
			},
		},
		'rnbq1k1r/ppp2p1p/5bp1/3p2N1/3P1Q1P/2N5/PP2PPP1/R3KB1R b KQ h3 0 11' => {
			eco => 'D91m',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5'),
			moves => {
				'h7h6' => 'rnbq1k1r/ppp2p2/5bpp/3p2N1/3P1Q1P/2N5/PP2PPP1/R3KB1R w KQ - 0 12',
			},
		},
		'rnbq1k1r/ppp2p1p/5bp1/3p2N1/3P1Q2/2N5/PP2PPPP/R3KB1R w KQ - 4 11' => {
			eco => 'D91m',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5'),
			moves => {
				'h2h4' => 'rnbq1k1r/ppp2p1p/5bp1/3p2N1/3P1Q1P/2N5/PP2PPP1/R3KB1R b KQ h3 0 11',
			},
		},
		'rnbq1k1r/ppp2p2/5bpp/3p2N1/3P1Q1P/2N5/PP2PPP1/R3KB1R w KQ - 0 12' => {
			eco => 'D91n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Qd2 exd5 9. Qe3+ Kf8 10. Qf4 Bf6 11. h4 h6

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5: 11. h4 h6'),
			moves => {
			},
		},
		'rnbq1k1r/ppp2pbp/6p1/3p2N1/3P1Q2/2N5/PP2PPPP/R3KB1R b KQ - 3 10' => {
			eco => 'D91m',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5'),
			moves => {
				'g7f6' => 'rnbq1k1r/ppp2p1p/5bp1/3p2N1/3P1Q2/2N5/PP2PPPP/R3KB1R w KQ - 4 11',
			},
		},
		'rnbq1k1r/ppp2pbp/6p1/3p2N1/3P4/2N1Q3/PP2PPPP/R3KB1R w KQ - 2 10' => {
			eco => 'D91m',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5'),
			moves => {
				'e3f4' => 'rnbq1k1r/ppp2pbp/6p1/3p2N1/3P1Q2/2N5/PP2PPPP/R3KB1R b KQ - 3 10',
			},
		},
		'rnbq1k1r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2P5/P1P2PPP/R1B1KBNR b KQ - 4 9' => {
			eco => 'C12l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 Kf8 9. Bc1

			variation => N__('French: MacCutcheon, Duras Variation'),
			moves => {
			},
		},
		'rnbq1k1r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2P5/P1PB1PPP/R3KBNR w KQ - 3 9' => {
			eco => 'C12l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 Kf8

			variation => N__('French: MacCutcheon, 8. Qg4 Kf8'),
			moves => {
				'd2c1' => 'rnbq1k1r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2P5/P1P2PPP/R1B1KBNR b KQ - 4 9',
				'f1d3' => 'rnbq1k1r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2PB4/P1PB1PPP/R3K1NR b KQ - 4 9',
			},
		},
		'rnbq1k1r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2PB4/P1PB1PPP/R3K1NR b KQ - 4 9' => {
			eco => 'C12m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 Kf8 9. Bd3

			variation => N__('French: MacCutcheon, 8. Qg4 Kf8 9. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/1p1n1pbp/p2p2p1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R w KQ - 0 11' => {
			eco => 'A67h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. Bd3 O-O 10. Nf3 a6

			variation => N__('Benoni: Four Pawns, Taimanov, 9. Bd3 O-O 10. Nf3 a6'),
			moves => {
				'a2a4' => 'rnbq1rk1/1p1n1pbp/p2p2p1/2pP4/P3PP2/2NB1N2/1P4PP/R1BQK2R b KQ a3 0 11',
			},
		},
		'rnbq1rk1/1p1n1pbp/p2p2p1/2pP4/P3PP2/2NB1N2/1P4PP/R1BQK2R b KQ a3 0 11' => {
			eco => 'A67i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. Bd3 O-O 10. Nf3 a6 11. a4

			variation => N__('Benoni: Four Pawns, Taimanov, 9. Bd3 O-O 10. Nf3 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p1pb1pp/2p1pn2/p4p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - a6 0 8' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. b3 a5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6.O-O c6 7. b3 a5'),
			moves => {
			},
		},
		'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 b - - 5 9' => {
			eco => 'B92k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Be3

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Be3'),
			moves => {
				'c8e6' => 'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 w - - 6 10',
			},
		},
		'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K b - - 5 9' => {
			eco => 'B92o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O 9. Kh1

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O 9. Kh1'),
			moves => {
				'b8c6' => 'r1bq1rk1/1p2bppp/p1np1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K w - - 6 10',
				'c8e6' => 'rn1q1rk1/1p2bppp/p2pbn2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K w - - 6 10',
				'd8c7' => 'rnb2rk1/1pq1bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K w - - 6 10',
			},
		},
		'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w - - 4 9' => {
			eco => 'B92j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O O-O

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2Q1RK1 b - - 5 9',
				'g1h1' => 'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1R1K b - - 5 9',
			},
		},
		'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/2N3P1/PPP1NPBP/R1BQK2R w KQ - 4 9' => {
			eco => 'B91n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nde2 Be7 8. Bg2 O-O

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2 Be7 8. Bg2 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/1p2bppp/p2ppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b - - 2 9' => {
			eco => 'B85',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O 9. Be3

			variation => N__('Sicilian: Scheveningen, Classical, Main Line'),
			moves => {
				'b8c6' => 'r1bq1rk1/1p2bppp/p1nppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 3 10',
				'd8c7' => 'rnb2rk1/1pq1bppp/p2ppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 w - - 3 10',
			},
		},
		'rnbq1rk1/1p2bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 w - - 1 9' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4 O-O

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7 8. f4 O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/1p2bppp/p2ppn2/8/3NPP2/2N1B3/PPP1B1PP/R2Q1RK1 b - - 2 9',
			},
		},
		'rnbq1rk1/1p2ppbp/p1pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E62e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O a6

			variation => N__('King\'s Indian: Fianchetto, 6... c6 7.O-O a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p2ppbp/p1pp1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 0 8' => {
			eco => 'E81l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c6 7. Bd3 a6

			variation => N__('King\'s Indian: Sämisch, Byrne Variation'),
			moves => {
			},
		},
		'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR w KQ - 0 8' => {
			eco => 'E81h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 c5 7. d5 a6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 c5 7. d5 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 1 8' => {
			eco => 'E74g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 a6 8. Qd2

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 a6 8. Qd2'),
			moves => {
			},
		},
		'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 8' => {
			eco => 'E74g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 a6

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 a6'),
			moves => {
				'a2a4' => 'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/P1P1P3/2N5/1P2BPPP/R2QK1NR b KQ a3 0 8',
				'd1d2' => 'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 1 8',
			},
		},
		'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/P1P1P3/2N5/1P2BPPP/R2QK1NR b KQ a3 0 8' => {
			eco => 'E74h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 a6 8. a4

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 a6 8. a4'),
			moves => {
				'd8a5' => 'rnb2rk1/1p2ppbp/p2p1np1/q1pP2B1/P1P1P3/2N5/1P2BPPP/R2QK1NR w KQ - 1 9',
			},
		},
		'rnbq1rk1/1p2ppbp/p2p1np1/2pP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 0 8' => {
			eco => 'E76o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. d5 a6

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pb1/p2p1np1/2pP3p/P3P3/2N1BP2/1P1Q2PP/R3KBNR w KQ - 0 12' => {
			eco => 'A65w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 h5

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 h5'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pb1/p2p1npp/2pP4/4P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 11' => {
			eco => 'A65v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6'),
			moves => {
				'a2a4' => 'rnbq1rk1/1p3pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1Q2PP/R3KBNR b KQ a3 0 11',
			},
		},
		'rnbq1rk1/1p3pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1Q2PP/R3KBNR b KQ a3 0 11' => {
			eco => 'A65v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4'),
			moves => {
				'b8d7' => 'r1bq1rk1/1p1n1pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1Q2PP/R3KBNR w KQ - 1 12',
				'f8e8' => 'rnbqr1k1/1p3pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1Q2PP/R3KBNR w KQ - 1 12',
				'h6h5' => 'rnbq1rk1/1p3pb1/p2p1np1/2pP3p/P3P3/2N1BP2/1P1Q2PP/R3KBNR w KQ - 0 12',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP2B1/2P5/2N5/PP1QBPPP/R3K1NR w KQ - 0 10' => {
			eco => 'E75h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2 exd5 9. exd5 a6

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2 exd5 9.exd5 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP2B1/4P3/2N2P2/PP1Q2PP/R3KBNR w KQ - 0 10' => {
			eco => 'A65q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 O-O 9. Qd2 a6

			variation => N__('Benoni: Sämisch, 8. Bg5 O-O 9. Qd2 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6

			variation => N__('Benoni: Classical, 9.O-O a6'),
			moves => {
				'a2a4' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/P3P3/2N2N2/1P2BPPP/R1BQ1RK1 b - a3 0 10',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQ1RK1 b - - 1 10' => {
			eco => 'A70p',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6'),
			moves => {
				'b7b5' => 'rnbq1rk1/5pbp/p2p1np1/1ppP4/4P3/2NB1N1P/PP3PP1/R1BQ1RK1 w - b6 0 11',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 0 10' => {
			eco => 'A70p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 a6

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6'),
			moves => {
				'a2a4' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQK2R b KQ a3 0 10',
				'e1g1' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQ1RK1 b - - 1 10',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A65j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. Nge2 O-O 9. O-O a6

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8.Nge2 O-O 9.O-O a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4PB2/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. Bf4 a6

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bf4 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A62',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O a6

			variation => N__('Benoni: Fianchetto, 9.O-O a6'),
			moves => {
				'a2a4' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/P7/2N2NP1/1P2PPBP/R1BQ1RK1 b - a3 0 10',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/P3P3/2N2N2/1P2BPPP/R1BQ1RK1 b - a3 0 10' => {
			eco => 'A74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O a6 10. a4

			variation => N__('Benoni: Classical, 9.O-O a6 10. a4'),
			moves => {
				'b8d7' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2N2N2/1P2BPPP/R1BQ1RK1 w - - 1 11',
				'c8g4' => 'rn1q1rk1/1p3pbp/p2p1np1/2pP4/P3P1b1/2N2N2/1P2BPPP/R1BQ1RK1 w - - 1 11',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQK2R b KQ a3 0 10' => {
			eco => 'A70r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 a6 10. a4

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6 10. a4'),
			moves => {
				'b8d7' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P3P3/2NB1N1P/1P3PP1/R1BQK2R w KQ - 1 11',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/2pP4/P7/2N2NP1/1P2PPBP/R1BQ1RK1 b - a3 0 10' => {
			eco => 'A62',
                        # TRANSLATORS:

			variation => N__('Benoni: Fianchetto, 9.O-O a6'),
			moves => {
				'b8d7' => 'r1bq1rk1/1p1n1pbp/p2p1np1/2pP4/P7/2N2NP1/1P2PPBP/R1BQ1RK1 w - - 1 11',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/3Pp3/4P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 10' => {
			eco => 'E88d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Qd2 cxd5 9. cxd5 a6

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6 8. Qd2 cxd5 9.cxd5 a6'),
			moves => {
				'f1d3' => 'rnbq1rk1/1p3pbp/p2p1np1/3Pp3/4P3/2NBBP2/PP1Q2PP/R3K1NR b KQ - 1 10',
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 10' => {
			eco => 'E89b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5 9. cxd5 a6

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line, 9... a6'),
			moves => {
			},
		},
		'rnbq1rk1/1p3pbp/p2p1np1/3Pp3/4P3/2NBBP2/PP1Q2PP/R3K1NR b KQ - 1 10' => {
			eco => 'E88e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Qd2 cxd5 9. cxd5 a6 10. Bd3

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6 8. Qd2 cxd5 9.cxd5 a6 10. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/1p3ppp/p3pn2/6B1/1bBP4/2N2N2/PP3PPP/R2Q1RK1 b - - 1 10' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 a6 10. Bg5

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4 8.Bxc4 cxd4 9.exd4 a6 10. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/1p3ppp/p3pn2/8/1bBP4/2N2N2/PP3PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 a6

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4 8.Bxc4 cxd4 9.exd4 a6'),
			moves => {
				'c1g5' => 'rnbq1rk1/1p3ppp/p3pn2/6B1/1bBP4/2N2N2/PP3PPP/R2Q1RK1 b - - 1 10',
			},
		},
		'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - a6 0 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. b3 a5

			variation => N__('Dutch: Classical, 7. b3 a5'),
			moves => {
				'c1b2' => 'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 1 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. b3 a5 8. Bb2

			variation => N__('Dutch: Classical, 7. b3 a5 8. Bb2'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 a5 8. b3

			variation => N__('Dutch: Classical, 7. Nc3 a5 8. b3'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - a6 0 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 a5

			variation => N__('Dutch: Classical, 7. Nc3 a5'),
			moves => {
				'b2b3' => 'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'd1c2' => 'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8',
				'f1e1' => 'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 1 8',
			},
		},
		'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 1 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 a5 8. Re1

			variation => N__('Dutch: Classical, 7. Nc3 a5 8. Re1'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3 a5 8. Qc2

			variation => N__('Dutch: Classical, 7. Nc3 a5 8. Qc2'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1bppp/p3pn2/8/2QP4/5NP1/PP2PPBP/RNB2RK1 b - - 0 8' => {
			eco => 'E05p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. Qxc4

			variation => N__('Catalan: Open, Classical, 8.Qxc4'),
			moves => {
				'b7b5' => 'rnbq1rk1/2p1bppp/p3pn2/1p6/2QP4/5NP1/PP2PPBP/RNB2RK1 w - b6 0 9',
			},
		},
		'rnbq1rk1/1pp1bppp/p3pn2/8/2pP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8' => {
			eco => 'E05j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6

			variation => N__('Catalan: Open, Classical Main Line'),
			moves => {
				'a2a4' => 'rnbq1rk1/1pp1bppp/p3pn2/8/P1pP4/5NP1/1PQ1PPBP/RNB2RK1 b - a3 0 8',
				'c2c4' => 'rnbq1rk1/1pp1bppp/p3pn2/8/2QP4/5NP1/PP2PPBP/RNB2RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/1pp1bppp/p3pn2/8/P1pP4/5NP1/1PQ1PPBP/RNB2RK1 b - a3 0 8' => {
			eco => 'E05k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2 a6 8. a4

			variation => N__('Catalan: Open, Classical, 8. a4'),
			moves => {
				'c8d7' => 'rn1q1rk1/1ppbbppp/p3pn2/8/P1pP4/5NP1/1PQ1PPBP/RNB2RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/1pp1ppbp/3p1np1/p7/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ a6 0 7' => {
			eco => 'E91b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 a5

			variation => N__('King\'s Indian: 6. Be2 a5'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1ppbp/p2p1np1/6B1/2PPP3/2N2P2/PP4PP/R2QKBNR w KQ - 0 7' => {
			eco => 'E81g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 a6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1ppbp/p2p1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 1 7' => {
			eco => 'E81k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 a6 7. Qd2

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 a6 7. Qd2'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1ppbp/p2p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 7' => {
			eco => 'E81k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 a6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 a6'),
			moves => {
				'd1d2' => 'rnbq1rk1/1pp1ppbp/p2p1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 1 7',
			},
		},
		'rnbq1rk1/1pp1ppbp/p2p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 7' => {
			eco => 'E91c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 a6

			variation => N__('King\'s Indian: 6. Be2 a6'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1ppbp/p2p1np1/8/3PP3/2N1BN1P/PPP2PP1/R2QKB1R w KQ - 0 7' => {
			eco => 'B08d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3 O-O 6. Be3 a6

			variation => N__('Pirc: Classical, Spassky System, 6... a6'),
			moves => {
			},
		},
		'rnbq1rk1/1pp1ppbp/p4np1/4P3/2QP4/2N2N2/PP3PPP/R1B1KB1R b KQ - 0 8' => {
			eco => 'D97h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. e5

			variation => N__('Grünfeld: Russian, Alekhine, 8. e5'),
			moves => {
				'b7b5' => 'rnbq1rk1/2p1ppbp/p4np1/1p2P3/2QP4/2N2N2/PP3PPP/R1B1KB1R w KQ b6 0 9',
			},
		},
		'rnbq1rk1/1pp1ppbp/p4np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 1 8' => {
			eco => 'D97e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. Be2

			variation => N__('Grünfeld: Russian, Alekhine, 8. Be2'),
			moves => {
				'b7b5' => 'rnbq1rk1/2p1ppbp/p4np1/1p6/2QPP3/2N2N2/PP2BPPP/R1B1K2R w KQ b6 0 9',
			},
		},
		'rnbq1rk1/1pp1ppbp/p4np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 0 8' => {
			eco => 'D97d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6

			variation => N__('Grünfeld: Russian, Alekhine (Hungarian) Variation'),
			moves => {
				'c4b3' => 'rnbq1rk1/1pp1ppbp/p4np1/8/3PP3/1QN2N2/PP3PPP/R1B1KB1R b KQ - 1 8',
				'e4e5' => 'rnbq1rk1/1pp1ppbp/p4np1/4P3/2QP4/2N2N2/PP3PPP/R1B1KB1R b KQ - 0 8',
				'f1e2' => 'rnbq1rk1/1pp1ppbp/p4np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/1pp1ppbp/p4np1/8/3PP3/1QN2N2/PP3PPP/R1B1KB1R b KQ - 1 8' => {
			eco => 'D97g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. Qb3

			variation => N__('Grünfeld: Russian, Alekhine, 8. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/1pp2pb1/3p1npp/p2Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'E92s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5 h6

			variation => N__('King\'s Indian: Petrosian, Stein, 8. Bg5 h6'),
			moves => {
				'g5h4' => 'rnbq1rk1/1pp2pb1/3p1npp/p2Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/1pp2pb1/3p1npp/p2Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E92s',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Petrosian, Stein, 8. Bg5 h6'),
			moves => {
				'b8a6' => 'r1bq1rk1/1pp2pb1/n2p1npp/p2Pp3/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 2 10',
			},
		},
		'rnbq1rk1/1pp2pbp/3p1np1/p2Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 1 8' => {
			eco => 'E92s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. Bg5

			variation => N__('King\'s Indian: Petrosian, Stein, 8. Bg5'),
			moves => {
				'h7h6' => 'rnbq1rk1/1pp2pb1/3p1npp/p2Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9',
			},
		},
		'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ a6 0 8' => {
			eco => 'E87b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 a5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 a5'),
			moves => {
			},
		},
		'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N2N1P/PP2BPP1/R1BQK2R b KQ - 0 8' => {
			eco => 'E92r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5 8. h3

			variation => N__('King\'s Indian: Petrosian, Stein, 8. h3'),
			moves => {
			},
		},
		'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ a6 0 8' => {
			eco => 'E90q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5 a5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5 a5'),
			moves => {
			},
		},
		'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ a6 0 8' => {
			eco => 'E92q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 a5

			variation => N__('King\'s Indian: Petrosian, Stein Variation'),
			moves => {
				'c1g5' => 'rnbq1rk1/1pp2pbp/3p1np1/p2Pp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 1 8',
				'h2h3' => 'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N2N1P/PP2BPP1/R1BQK2R b KQ - 0 8',
			},
		},
		'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10' => {
			eco => 'C95a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d4

			variation => N__('Spanish: Closed, Breyer, 10. d4'),
			moves => {
				'b8d7' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11',
				'c8b7' => 'rn1q1rk1/1bp1bppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 w - - 1 11',
			},
		},
		'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/4P3/1BP2N1P/PP1P1PP1/RNBQR1K1 w - - 1 10' => {
			eco => 'C94a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8

			variation => N__('Spanish: Closed, Breyer Defence'),
			moves => {
				'a2a4' => 'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/P3P3/1BP2N1P/1P1P1PP1/RNBQR1K1 b - a3 0 10',
				'd2d3' => 'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP3PP1/RNBQR1K1 b - - 0 10',
				'd2d4' => 'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/3PP3/1BP2N1P/PP3PP1/RNBQR1K1 b - d3 0 10',
			},
		},
		'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP3PP1/RNBQR1K1 b - - 0 10' => {
			eco => 'C94c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. d3

			variation => N__('Spanish: Closed, Breyer, 10. d3'),
			moves => {
				'b8d7' => 'r1bq1rk1/2pnbppp/p2p1n2/1p2p3/4P3/1BPP1N1P/PP3PP1/RNBQR1K1 w - - 1 11',
			},
		},
		'rnbq1rk1/2p1bppp/p2p1n2/1p2p3/P3P3/1BP2N1P/1P1P1PP1/RNBQR1K1 b - a3 0 10' => {
			eco => 'C94b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. Bb5 a6 4. Ba4 Nf6 5. O-O Be7 6. Re1 b5 7. Bb3 O-O 8. c3 d6 9. h3 Nb8 10. a4

			variation => N__('Spanish: Closed, Breyer, Matulovic Variation'),
			moves => {
			},
		},
		'rnbq1rk1/2p1bppp/p3pn2/1p6/2QP4/5NP1/PP2PPBP/RNB2RK1 w - b6 0 9' => {
			eco => 'E05p',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, Classical, 8.Qxc4'),
			moves => {
				'c4c2' => 'rnbq1rk1/2p1bppp/p3pn2/1p6/3P4/5NP1/PPQ1PPBP/RNB2RK1 b - - 1 9',
			},
		},
		'rnbq1rk1/2p1bppp/p3pn2/1p6/3P4/5NP1/PPQ1PPBP/RNB2RK1 b - - 1 9' => {
			eco => 'E05p',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, Classical, 8.Qxc4'),
			moves => {
				'c8b7' => 'rn1q1rk1/1bp1bppp/p3pn2/1p6/3P4/5NP1/PPQ1PPBP/RNB2RK1 w - - 2 10',
			},
		},
		'rnbq1rk1/2p1ppbp/p4np1/1p2P3/2QP4/2N2N2/PP3PPP/R1B1KB1R w KQ b6 0 9' => {
			eco => 'D97h',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Alekhine, 8. e5'),
			moves => {
				'c4b3' => 'rnbq1rk1/2p1ppbp/p4np1/1p2P3/3P4/1QN2N2/PP3PPP/R1B1KB1R b KQ - 1 9',
			},
		},
		'rnbq1rk1/2p1ppbp/p4np1/1p2P3/3P4/1QN2N2/PP3PPP/R1B1KB1R b KQ - 1 9' => {
			eco => 'D97i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. e5 b5 9. Qb3

			variation => N__('Grünfeld: Russian, Alekhine, 8. e5 b5 9. Qb3'),
			moves => {
				'f6d7' => 'rnbq1rk1/2pnppbp/p5p1/1p2P3/3P4/1QN2N2/PP3PPP/R1B1KB1R w KQ - 2 10',
			},
		},
		'rnbq1rk1/2p1ppbp/p4np1/1p6/2QPP3/2N2N2/PP2BPPP/R1B1K2R w KQ b6 0 9' => {
			eco => 'D97e',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, Alekhine, 8. Be2'),
			moves => {
				'c4b3' => 'rnbq1rk1/2p1ppbp/p4np1/1p6/3PP3/1QN2N2/PP2BPPP/R1B1K2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/2p1ppbp/p4np1/1p6/3PP3/1QN2N2/PP2BPPP/R1B1K2R b KQ - 1 9' => {
			eco => 'D97f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. Be2 b5 9. Qb3

			variation => N__('Grünfeld: Russian, Alekhine, 8. Be2 b5 9. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/2p1ppbp/pp1p1np1/8/2PPP3/2NBBP2/PP2N1PP/R2QK2R b KQ - 1 8' => {
			eco => 'E82i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 a6 8. Nge2

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 a6 8.Nge2'),
			moves => {
				'c7c5' => 'rnbq1rk1/4ppbp/pp1p1np1/2p5/2PPP3/2NBBP2/PP2N1PP/R2QK2R w KQ c6 0 9',
			},
		},
		'rnbq1rk1/2p1ppbp/pp1p1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 0 8' => {
			eco => 'E82h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 a6

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 a6'),
			moves => {
				'g1e2' => 'rnbq1rk1/2p1ppbp/pp1p1np1/8/2PPP3/2NBBP2/PP2N1PP/R2QK2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/2pnppbp/p5p1/1p2P3/3P4/1QN1BN2/PP3PPP/R3KB1R b KQ - 3 10' => {
			eco => 'D97k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. e5 b5 9. Qb3 Nfd7 10. Be3

			variation => N__('Grünfeld: Russian, Alekhine, 8. e5 b5 9. Qb3 Nfd7 10. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/2pnppbp/p5p1/1p2P3/3P4/1QN2N2/PP3PPP/R1B1KB1R w KQ - 2 10' => {
			eco => 'D97j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 a6 8. e5 b5 9. Qb3 Nfd7

			variation => N__('Grünfeld: Russian, Alekhine, 8. e5 b5 9. Qb3 Nfd7'),
			moves => {
				'c1e3' => 'rnbq1rk1/2pnppbp/p5p1/1p2P3/3P4/1QN1BN2/PP3PPP/R3KB1R b KQ - 3 10',
			},
		},
		'rnbq1rk1/4ppbp/p2p1np1/1PpP2B1/4P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 9' => {
			eco => 'E74f',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 b5'),
			moves => {
				'a2a4' => 'rnbq1rk1/4ppbp/p2p1np1/1PpP2B1/P3P3/2N5/1P2BPPP/R2QK1NR b KQ a3 0 9',
			},
		},
		'rnbq1rk1/4ppbp/p2p1np1/1PpP2B1/P3P3/2N5/1P2BPPP/R2QK1NR b KQ a3 0 9' => {
			eco => 'E74f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 b5 8. cxb5 a6 9. a4

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 b5 8.cxb5 a6 9. a4'),
			moves => {
			},
		},
		'rnbq1rk1/4ppbp/p2p1np1/1PpP4/4PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 0 9' => {
			eco => 'E76p',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 b5'),
			moves => {
				'a2a4' => 'rnbq1rk1/4ppbp/p2p1np1/1PpP4/P3PP2/2N2N2/1P4PP/R1BQKB1R b KQ a3 0 9',
			},
		},
		'rnbq1rk1/4ppbp/p2p1np1/1PpP4/P3PP2/2N2N2/1P4PP/R1BQKB1R b KQ a3 0 9' => {
			eco => 'E76q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. d5 b5 8. cxb5 a6 9. a4

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 b5 8.cxb5 a6 9. a4'),
			moves => {
			},
		},
		'rnbq1rk1/4ppbp/pp1p1np1/2p1P3/2PP4/2NBBP2/PP2N1PP/R2QK2R b KQ - 0 9' => {
			eco => 'E82k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 a6 8. Nge2 c5 9. e5

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 a6 8.Nge2 c5 9. e5'),
			moves => {
			},
		},
		'rnbq1rk1/4ppbp/pp1p1np1/2p5/2PPP3/2NBBP2/PP2N1PP/R2QK2R w KQ c6 0 9' => {
			eco => 'E82i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 a6 8. Nge2 c5

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 a6 8.Nge2 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/4ppbp/pp1p1np1/2pP4/2P1P3/2NBBP2/PP2N1PP/R2QK2R b KQ - 0 9',
				'e4e5' => 'rnbq1rk1/4ppbp/pp1p1np1/2p1P3/2PP4/2NBBP2/PP2N1PP/R2QK2R b KQ - 0 9',
			},
		},
		'rnbq1rk1/4ppbp/pp1p1np1/2pP4/2P1P3/2NBBP2/PP2N1PP/R2QK2R b KQ - 0 9' => {
			eco => 'E82j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 a6 8. Nge2 c5 9. d5

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 a6 8.Nge2 c5 9. d5'),
			moves => {
			},
		},
		'rnbq1rk1/5pbp/p2p1np1/1ppP4/4P3/2NB1N1P/PP3PP1/R1BQ1RK1 w - b6 0 11' => {
			eco => 'A70q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 a6 10. O-O b5

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 a6 10.O-O b5'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1Bpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 9' => {
			eco => 'D59c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Bxe7

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Bxe7'),
			moves => {
				'd8e7' => 'rnb2rk1/p1p1qpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 10',
			},
		},
		'rnbq1rk1/p1p1Bpp1/1p5p/3p4/3P4/4P3/PP3PPP/2RQKBNR b K - 0 10' => {
			eco => 'D54c',
                        # TRANSLATORS:

			variation => N__('QGD: Anti-Neo-Orthodox, 8.cxd5 Nxd5 9.Nxd5'),
			moves => {
				'd8e7' => 'rnb2rk1/p1p1qpp1/1p5p/3p4/3P4/4P3/PP3PPP/2RQKBNR w K - 0 11',
			},
		},
		'rnbq1rk1/p1p1b1pp/1p2pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 b6

			variation => N__('Dutch: Stonewall, Botvinnik, 7... b6'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2p2p/3N4/3P3B/4P3/PP3PPP/2RQKBNR b K - 0 9' => {
			eco => 'D54c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Rc1 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Nxd5

			variation => N__('QGD: Anti-Neo-Orthodox, 8.cxd5 Nxd5 9.Nxd5'),
			moves => {
				'e6d5' => 'rnbq1rk1/p1p1bpp1/1p5p/3p4/3P3B/4P3/PP3PPP/2RQKBNR w K - 0 10',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2p2p/3N4/3P3B/4PN2/PP3PPP/R2QKB1R b KQ - 0 9' => {
			eco => 'D59b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5 9. Nxd5

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5 9.Nxd5'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2p2p/3n4/3P3B/2N1P3/PP3PPP/2RQKBNR w K - 0 9' => {
			eco => 'D54c',
                        # TRANSLATORS:

			variation => N__('QGD: Anti-Neo-Orthodox, 7. Bh4 b6'),
			moves => {
				'c3d5' => 'rnbq1rk1/p1p1bpp1/1p2p2p/3N4/3P3B/4P3/PP3PPP/2RQKBNR b K - 0 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2p2p/3n4/3P3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 9' => {
			eco => 'D59a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 Nxd5

			variation => N__('QGD: Tartakower, 8.cxd5 Nxd5'),
			moves => {
				'c3d5' => 'rnbq1rk1/p1p1bpp1/1p2p2p/3N4/3P3B/4PN2/PP3PPP/R2QKB1R b KQ - 0 9',
				'h4e7' => 'rnbq1rk1/p1p1Bpp1/1p2p2p/3n4/3P4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pB1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8' => {
			eco => 'D58d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bxf6

			variation => N__('QGD: Tartakower, 8.Bxf6'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3P4/3P3B/2N1P3/PP3PPP/2RQKBNR b K - 0 8' => {
			eco => 'D54c',
                        # TRANSLATORS:

			variation => N__('QGD: Anti-Neo-Orthodox, 7. Bh4 b6'),
			moves => {
				'f6d5' => 'rnbq1rk1/p1p1bpp1/1p2p2p/3n4/3P3B/2N1P3/PP3PPP/2RQKBNR w K - 0 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3P4/3P3B/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8' => {
			eco => 'D58s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5

			variation => N__('QGD: Tartakower, 8.cxd5'),
			moves => {
				'e6d5' => 'rnbq1rk1/p1p1bpp1/1p3n1p/3p4/3P3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 9',
				'f6d5' => 'rnbq1rk1/p1p1bpp1/1p2p2p/3n4/3P3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/1QN1PN2/PP3PPP/R3KB1R b KQ - 1 8' => {
			eco => 'D58b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Qb3

			variation => N__('QGD: Tartakower, 8. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1P3/PP3PPP/2RQKBNR w K - 0 8' => {
			eco => 'D54c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Rc1 h6 7. Bh4 b6

			variation => N__('QGD: Anti-Neo-Orthodox, 7. Bh4 b6'),
			moves => {
				'c4d5' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3P4/3P3B/2N1P3/PP3PPP/2RQKBNR b K - 0 8',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP2BPPP/R2QK2R b KQ - 1 8' => {
			eco => 'D58k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Be2

			variation => N__('QGD: Tartakower, 8. Be2'),
			moves => {
				'c8b7' => 'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP2BPPP/R2QK2R w KQ - 2 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP3PPP/2RQKB1R b K - 1 8' => {
			eco => 'D58p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Rc1

			variation => N__('QGD: Tartakower, 8. Rc1'),
			moves => {
				'c8b7' => 'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP3PPP/2RQKB1R w K - 2 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'D58a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6

			variation => N__('QGD: Tartakower System'),
			moves => {
				'a1c1' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP3PPP/2RQKB1R b K - 1 8',
				'c4d5' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3P4/3P3B/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8',
				'd1b3' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/1QN1PN2/PP3PPP/R3KB1R b KQ - 1 8',
				'd1c2' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 8',
				'f1d3' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2QK2R b KQ - 1 8',
				'f1e2' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP2BPPP/R2QK2R b KQ - 1 8',
				'h4f6' => 'rnbq1rk1/p1p1bpp1/1p2pB1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 8' => {
			eco => 'D58c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Qc2

			variation => N__('QGD: Tartakower, 8. Qc2'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2QK2R b KQ - 1 8' => {
			eco => 'D58e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. Bd3

			variation => N__('QGD: Tartakower, 8. Bd3'),
			moves => {
				'c8b7' => 'rn1q1rk1/pbp1bpp1/1p2pn1p/3p4/2PP3B/2NBPN2/PP3PPP/R2QK2R w KQ - 2 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p3n1p/3p4/3P3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 9' => {
			eco => 'D58s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 exd5

			variation => N__('QGD: Tartakower, 8.cxd5 exd5'),
			moves => {
				'f1d3' => 'rnbq1rk1/p1p1bpp1/1p3n1p/3p4/3P3B/2NBPN2/PP3PPP/R2QK2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/p1p1bpp1/1p3n1p/3p4/3P3B/2NBPN2/PP3PPP/R2QK2R b KQ - 1 9' => {
			eco => 'D58t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 b6 8. cxd5 exd5 9. Bd3

			variation => N__('QGD: Tartakower, 8.cxd5 exd5 9. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bpp1/1p5p/3p4/3P3B/4P3/PP3PPP/2RQKBNR w K - 0 10' => {
			eco => 'D54c',
                        # TRANSLATORS:

			variation => N__('QGD: Anti-Neo-Orthodox, 8.cxd5 Nxd5 9.Nxd5'),
			moves => {
				'h4e7' => 'rnbq1rk1/p1p1Bpp1/1p5p/3p4/3P4/4P3/PP3PPP/2RQKBNR b K - 0 10',
			},
		},
		'rnbq1rk1/p1p1bppp/1p2p3/3n2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'D55f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 b6 7. cxd5 Nxd5

			variation => N__('QGD: 6. Nf3 b6 7.cxd5 Nxd5'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bppp/1p2pn2/3P2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7' => {
			eco => 'D55c',
                        # TRANSLATORS:

			variation => N__('QGD: 6. Nf3 b6'),
			moves => {
				'e6d5' => 'rnbq1rk1/p1p1bppp/1p3n2/3p2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8',
				'f6d5' => 'rnbq1rk1/p1p1bppp/1p2p3/3n2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/p1p1bppp/1p2pn2/3p2B1/2PP4/2N1PN2/PP2BPPP/R2QK2R b KQ - 1 7' => {
			eco => 'D55d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 b6 7. Be2

			variation => N__('QGD: 6. Nf3 b6 7. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bppp/1p2pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'D55c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 b6

			variation => N__('QGD: 6. Nf3 b6'),
			moves => {
				'c4d5' => 'rnbq1rk1/p1p1bppp/1p2pn2/3P2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7',
				'f1e2' => 'rnbq1rk1/p1p1bppp/1p2pn2/3p2B1/2PP4/2N1PN2/PP2BPPP/R2QK2R b KQ - 1 7',
			},
		},
		'rnbq1rk1/p1p1bppp/1p2pn2/3p4/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 b6

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 b6'),
			moves => {
				'c1b2' => 'rnbq1rk1/p1p1bppp/1p2pn2/3p4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/p1p1bppp/1p2pn2/3p4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 b - - 1 7' => {
			eco => 'A14',
                        # TRANSLATORS:

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 b6'),
			moves => {
				'c8b7' => 'rn1q1rk1/pbp1bppp/1p2pn2/3p4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/p1p1bppp/1p2pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'D37k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 b6

			variation => N__('QGD: Classical, 5...O-O 6. e3 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1bppp/1p3n2/3p2B1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'D55e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 b6 7. cxd5 exd5

			variation => N__('QGD: 6. Nf3 b6 7.cxd5 exd5'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1ppbp/1p1p1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 1 7' => {
			eco => 'E82b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Qd2

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Qd2'),
			moves => {
				'c7c5' => 'rnbq1rk1/p3ppbp/1p1p1np1/2p5/2PPP3/2N1BP2/PP1Q2PP/R3KBNR w KQ c6 0 8',
			},
		},
		'rnbq1rk1/p1p1ppbp/1p1p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 7' => {
			eco => 'E82a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6

			variation => N__('King\'s Indian: Sämisch, Fianchetto'),
			moves => {
				'd1d2' => 'rnbq1rk1/p1p1ppbp/1p1p1np1/8/2PPP3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 1 7',
				'f1d3' => 'rnbq1rk1/p1p1ppbp/1p1p1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR b KQ - 1 7',
			},
		},
		'rnbq1rk1/p1p1ppbp/1p1p1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR b KQ - 1 7' => {
			eco => 'E82d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3

			variation => N__('King\'s Indian: Sämisch, Fianchetto, Bronstein Variation'),
			moves => {
				'a7a6' => 'rnbq1rk1/2p1ppbp/pp1p1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 0 8',
				'c8b7' => 'rn1q1rk1/pbp1ppbp/1p1p1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 2 8',
				'f6d7' => 'rnbq1rk1/p1pnppbp/1p1p2p1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 2 8',
			},
		},
		'rnbq1rk1/p1p1ppbp/1p3np1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7' => {
			eco => 'D95b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 b6

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1ppbp/1p3np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 0 8' => {
			eco => 'D97b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 b6

			variation => N__('Grünfeld: Russian, Levenfish Variation'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1ppbp/1p4p1/8/2BPP2P/2P5/P3NPP1/R1BQK2R b KQ h3 0 9' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 b6 9. h4

			variation => N__('Grünfeld: Classical Exchange, Simagin, 9. h4'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1ppbp/1p4p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 1 9' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 b6 9. O-O

			variation => N__('Grünfeld: Classical Exchange, Simagin, 9.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/p1p1ppbp/1p4p1/8/2BPP3/2P5/P3NPPP/R1BQK2R w KQ - 0 9' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 b6

			variation => N__('Grünfeld: Classical Exchange, Simagin'),
			moves => {
				'e1g1' => 'rnbq1rk1/p1p1ppbp/1p4p1/8/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 1 9',
				'h2h4' => 'rnbq1rk1/p1p1ppbp/1p4p1/8/2BPP2P/2P5/P3NPP1/R1BQK2R b KQ h3 0 9',
			},
		},
		'rnbq1rk1/p1p2ppp/1p2pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6 7. O-O

			variation => N__('Nimzo-Indian: Main Line, 6... b6 7.O-O'),
			moves => {
				'c8b7' => 'rn1q1rk1/pbp2ppp/1p2pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/p1p2ppp/1p2pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 0 7' => {
			eco => 'E52',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 b6

			variation => N__('Nimzo-Indian: Main Line, 6... b6'),
			moves => {
				'e1g1' => 'rnbq1rk1/p1p2ppp/1p2pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/p1pnppbp/1p1p2p1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 2 8' => {
			eco => 'E82e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Bd3 Nfd7

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Bd3 Nfd7'),
			moves => {
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/6B1/2PP4/P1Q5/1P2PPPP/R3KBNR b KQ - 1 7' => {
			eco => 'E32n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Bg5

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Bg5'),
			moves => {
				'c8a6' => 'rn1q1rk1/p1pp1ppp/bp2pn2/6B1/2PP4/P1Q5/1P2PPPP/R3KBNR w KQ - 2 8',
				'c8b7' => 'rn1q1rk1/pbpp1ppp/1p2pn2/6B1/2PP4/P1Q5/1P2PPPP/R3KBNR w KQ - 2 8',
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQ - 0 6' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 b6

			variation => N__('Nimzo-Indian: Reshevsky Variation'),
			moves => {
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 0 6' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 b6

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ - 0 6' => {
			eco => 'E47',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 b6

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/8/2PP4/P1P1P3/5PPP/R1BQKBNR w KQ - 0 7' => {
			eco => 'E28',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 b6

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3'),
			moves => {
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/8/2PP4/P1Q2N2/1P2PPPP/R1B1KB1R b KQ - 1 7' => {
			eco => 'E32m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6 7. Nf3

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6 7. Nf3'),
			moves => {
			},
		},
		'rnbq1rk1/p1pp1ppp/1p2pn2/8/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ - 0 7' => {
			eco => 'E32l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b6

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... b6'),
			moves => {
				'c1g5' => 'rnbq1rk1/p1pp1ppp/1p2pn2/6B1/2PP4/P1Q5/1P2PPPP/R3KBNR b KQ - 1 7',
				'g1f3' => 'rnbq1rk1/p1pp1ppp/1p2pn2/8/2PP4/P1Q2N2/1P2PPPP/R1B1KB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/p1pp1ppp/4pn2/1p6/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ b6 0 7' => {
			eco => 'E32i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 b5

			variation => N__('Nimzo-Indian: Classical, Adorjan Gambit'),
			moves => {
			},
		},
		'rnbq1rk1/p2p1ppp/1p2pn2/2b3B1/2P5/P1N2N2/1PQ1PPPP/R3KB1R b KQ - 1 8' => {
			eco => 'E39m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. a3 Bxc5 7. Nf3 b6 8. Bg5

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. a3 Bxc5 7. Nf3 b6 8. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/p2p1ppp/1p2pn2/2b5/2P5/P1N2N2/1PQ1PPPP/R1B1KB1R w KQ - 0 8' => {
			eco => 'E39l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. a3 Bxc5 7. Nf3 b6

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. a3 Bxc5 7. Nf3 b6'),
			moves => {
				'c1g5' => 'rnbq1rk1/p2p1ppp/1p2pn2/2b3B1/2P5/P1N2N2/1PQ1PPPP/R3KB1R b KQ - 1 8',
			},
		},
		'rnbq1rk1/p2p1ppp/1p2pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR w KQ - 0 8' => {
			eco => 'E28',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3 b6

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 b - - 1 9' => {
			eco => 'E11i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7. O-O c6 8. Qc2 b6 9. Bf4

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7.O-O c6 8. Qc2 b6 9. Bf4'),
			moves => {
			},
		},
		'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. Nc3 b6

			variation => N__('Catalan: Closed, 6.O-O c6 7. Nc3 b6'),
			moves => {
				'f3e5' => 'rnbq1rk1/p3bppp/1pp1pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. Qc2 b6

			variation => N__('Catalan: Closed, 6.O-O c6 7. Qc2 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP4/5NP1/PPQBPPBP/RN3RK1 w - - 0 9' => {
			eco => 'E11i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7. O-O c6 8. Qc2 b6

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7.O-O c6 8. Qc2 b6'),
			moves => {
				'd2f4' => 'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP1B2/5NP1/PPQ1PPBP/RN3RK1 b - - 1 9',
			},
		},
		'rnbq1rk1/p3bppp/1pp1pn2/3pN3/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 8' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. Nc3 b6 8. Ne5

			variation => N__('Catalan: Closed, 6.O-O c6 7. Nc3 b6 8. Ne5'),
			moves => {
			},
		},
		'rnbq1rk1/p3ppbp/1p1p1np1/2p5/2PPP3/2N1BP2/PP1Q2PP/R3KBNR w KQ c6 0 8' => {
			eco => 'E82b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Qd2'),
			moves => {
				'd4d5' => 'rnbq1rk1/p3ppbp/1p1p1np1/2pP4/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 0 8',
			},
		},
		'rnbq1rk1/p3ppbp/1p1p1np1/2pP4/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 0 8' => {
			eco => 'E82c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 b6 7. Qd2 c5 8. d5

			variation => N__('King\'s Indian: Sämisch, Fianchetto, 7. Qd2 c5 8. d5'),
			moves => {
			},
		},
		'rnbq1rk1/p3ppbp/1p4p1/2p5/3PP3/2P2N2/P3BPPP/1RBQK2R w K - 0 10' => {
			eco => 'D85n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 b6

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 b6'),
			moves => {
			},
		},
		'rnbq1rk1/p3ppbp/1p4p1/2p5/3PP3/2P2N2/P3BPPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D85l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be2 O-O 9. O-O b6

			variation => N__('Grünfeld: Modern Exchange, 8. Be2 O-O 9.O-O b6'),
			moves => {
			},
		},
		'rnbq1rk1/p3ppbp/3p1np1/1PpP2B1/4P3/2N5/PP2BPPP/R2QK1NR b KQ - 0 8' => {
			eco => 'E74f',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 b5'),
			moves => {
				'a7a6' => 'rnbq1rk1/4ppbp/p2p1np1/1PpP2B1/4P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/p3ppbp/3p1np1/1PpP4/4PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 8' => {
			eco => 'E76p',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 b5'),
			moves => {
				'a7a6' => 'rnbq1rk1/4ppbp/p2p1np1/1PpP4/4PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 0 9',
			},
		},
		'rnbq1rk1/p3ppbp/3p1np1/1ppP2B1/2P1P3/2N4P/PP3PP1/R2QKBNR w KQ b6 0 8' => {
			eco => 'E71n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 c5 7. d5 b5

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 c5 7. d5 b5'),
			moves => {
			},
		},
		'rnbq1rk1/p3ppbp/3p1np1/1ppP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ b6 0 8' => {
			eco => 'E74f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 b5

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 b5'),
			moves => {
				'c4b5' => 'rnbq1rk1/p3ppbp/3p1np1/1PpP2B1/4P3/2N5/PP2BPPP/R2QK1NR b KQ - 0 8',
			},
		},
		'rnbq1rk1/p3ppbp/3p1np1/1ppP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ b6 0 8' => {
			eco => 'E76p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. d5 b5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 b5'),
			moves => {
				'c4b5' => 'rnbq1rk1/p3ppbp/3p1np1/1PpP4/4PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/p4pbp/1p1p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'A65h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. Nge2 O-O 9. O-O b6

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8.Nge2 O-O 9.O-O b6'),
			moves => {
			},
		},
		'rnbq1rk1/p4pbp/2pp1np1/1p1Pp3/2P1P3/2NBBP2/PP4PP/R2QK1NR w KQ b6 0 9' => {
			eco => 'E88g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Bd3 b5

			variation => N__('King\'s Indian: Sämisch, Orthodox, Polugayevsky, 8... b5'),
			moves => {
			},
		},
		'rnbq1rk1/p4pbp/3p1np1/1BpP4/4P3/2N2N1P/PP3PP1/R1BQK2R b KQ - 0 10' => {
			eco => 'A70v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 b5 10. Bxb5

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5 10.Bxb5'),
			moves => {
			},
		},
		'rnbq1rk1/p4pbp/3p1np1/1NpP4/4P3/3B1N1P/PP3PP1/R1BQK2R b KQ - 0 10' => {
			eco => 'A70w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 b5 10. Nxb5

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5 10.Nxb5'),
			moves => {
				'f6e4' => 'rnbq1rk1/p4pbp/3p2p1/1NpP4/4n3/3B1N1P/PP3PP1/R1BQK2R w KQ - 0 11',
				'f8e8' => 'rnbqr1k1/p4pbp/3p1np1/1NpP4/4P3/3B1N1P/PP3PP1/R1BQK2R w KQ - 1 11',
			},
		},
		'rnbq1rk1/p4pbp/3p1np1/1ppP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ b6 0 10' => {
			eco => 'A70u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 b5

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5'),
			moves => {
				'c3b5' => 'rnbq1rk1/p4pbp/3p1np1/1NpP4/4P3/3B1N1P/PP3PP1/R1BQK2R b KQ - 0 10',
				'd3b5' => 'rnbq1rk1/p4pbp/3p1np1/1BpP4/4P3/2N2N1P/PP3PP1/R1BQK2R b KQ - 0 10',
			},
		},
		'rnbq1rk1/p4pbp/3p1np1/1ppP4/4PB2/2N2N2/PP2BPPP/R2QK2R w KQ b6 0 10' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. Bf4 b5

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bf4 b5'),
			moves => {
			},
		},
		'rnbq1rk1/p4pbp/3p1np1/1ppP4/4PP2/2N2N2/PP2B1PP/R1BQK2R w KQ b6 0 10' => {
			eco => 'A68d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 b5

			variation => N__('Benoni: Four Pawns, 9. Be2 b5'),
			moves => {
				'e4e5' => 'rnbq1rk1/p4pbp/3p1np1/1ppPP3/5P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 10',
			},
		},
		'rnbq1rk1/p4pbp/3p1np1/1ppPP3/5P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 10' => {
			eco => 'A68e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 b5 10. e5

			variation => N__('Benoni: Four Pawns, 9. Be2 b5 10. e5'),
			moves => {
				'd6e5' => 'rnbq1rk1/p4pbp/5np1/1ppPp3/5P2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 11',
			},
		},
		'rnbq1rk1/p4pbp/3p2p1/1NpP4/4n3/3B1N1P/PP3PP1/R1BQK2R w KQ - 0 11' => {
			eco => 'A70w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 b5 10. Nxb5 Nxe4

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5 10.Nxb5 Nxe4'),
			moves => {
			},
		},
		'rnbq1rk1/p4pbp/5np1/1ppPP3/8/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 11' => {
			eco => 'A68f',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, 9. Be2 b5 10. e5 dxe5'),
			moves => {
				'f6g4' => 'rnbq1rk1/p4pbp/6p1/1ppPP3/6n1/2N2N2/PP2B1PP/R1BQK2R w KQ - 1 12',
			},
		},
		'rnbq1rk1/p4pbp/5np1/1ppPp3/5P2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 11' => {
			eco => 'A68f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 b5 10. e5 dxe5

			variation => N__('Benoni: Four Pawns, 9. Be2 b5 10. e5 dxe5'),
			moves => {
				'f4e5' => 'rnbq1rk1/p4pbp/5np1/1ppPP3/8/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 11',
			},
		},
		'rnbq1rk1/p4pbp/6p1/1ppPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R b KQ - 2 12' => {
			eco => 'A68g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 b5 10. e5 dxe5 11. fxe5 Ng4 12. Bg5

			variation => N__('Benoni: Four Pawns, 9. Be2 b5 10. e5 dxe5 11.fxe5 Ng4 12. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/p4pbp/6p1/1ppPP3/6n1/2N2N2/PP2B1PP/R1BQK2R w KQ - 1 12' => {
			eco => 'A68f',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, 9. Be2 b5 10. e5 dxe5'),
			moves => {
				'c1g5' => 'rnbq1rk1/p4pbp/6p1/1ppPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R b KQ - 2 12',
			},
		},
		'rnbq1rk1/p4ppp/1p2pn2/2pP4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E53',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Main Line, Keres Variation'),
			moves => {
				'e6d5' => 'rnbq1rk1/p4ppp/1p3n2/2pp4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 9',
			},
		},
		'rnbq1rk1/p4ppp/1p2pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O b6

			variation => N__('Nimzo-Indian: Main Line, Keres Variation'),
			moves => {
				'c4d5' => 'rnbq1rk1/p4ppp/1p2pn2/2pP4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/p4ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/R2Q1RK1 b - - 1 10' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6 10. Bg5

			variation => N__('Nimzo-Indian: Main Line, Karpov, 10. Bg5'),
			moves => {
				'c8b7' => 'rn1q1rk1/pb3ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/R2Q1RK1 w - - 2 11',
			},
		},
		'rnbq1rk1/p4ppp/1p2pn2/8/1bBP4/2N2N2/PP3PPP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4 b6

			variation => N__('Nimzo-Indian: Main Line, Karpov Variation'),
			moves => {
				'c1g5' => 'rnbq1rk1/p4ppp/1p2pn2/6B1/1bBP4/2N2N2/PP3PPP/R2Q1RK1 b - - 1 10',
			},
		},
		'rnbq1rk1/p4ppp/1p3n2/2pp4/1b1P4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O b6 8. cxd5 exd5

			variation => N__('Nimzo-Indian: Main Line, Keres, 8.cxd5 exd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp1n1pbp/3p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQK2R b KQ - 2 10' => {
			eco => 'A67q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 O-O 10. Nf3

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 O-O 10. Nf3'),
			moves => {
				'b8a6' => 'r1bq1rk1/pp1n1pbp/n2p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQK2R w KQ - 3 11',
			},
		},
		'rnbq1rk1/pp1n1pbp/3p2p1/1BpP4/P3PP2/2N5/1P4PP/R1BQK1NR w KQ - 1 10' => {
			eco => 'A67p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 O-O

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 O-O'),
			moves => {
				'g1f3' => 'rnbq1rk1/pp1n1pbp/3p2p1/1BpP4/P3PP2/2N2N2/1P4PP/R1BQK2R b KQ - 2 10',
			},
		},
		'rnbq1rk1/pp1n1pbp/3p2p1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R b KQ - 6 10' => {
			eco => 'A67g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. Bd3 O-O 10. Nf3

			variation => N__('Benoni: Four Pawns, Taimanov, 9. Bd3 O-O 10. Nf3'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p1n1pbp/p2p2p1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R w KQ - 0 11',
				'b8a6' => 'r1bq1rk1/pp1n1pbp/n2p2p1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R w KQ - 7 11',
			},
		},
		'rnbq1rk1/pp1n1pbp/3p2p1/2pP4/4PP2/2NB4/PP4PP/R1BQK1NR w KQ - 5 10' => {
			eco => 'A67f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. Bd3 O-O

			variation => N__('Benoni: Four Pawns, Taimanov, 9. Bd3 O-O'),
			moves => {
				'g1f3' => 'rnbq1rk1/pp1n1pbp/3p2p1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R b KQ - 6 10',
			},
		},
		'rnbq1rk1/pp1n1pbp/4p1p1/3pN3/3P1P2/2N3P1/PP2P1BP/R1BQ1RK1 b - f3 0 10' => {
			eco => 'D79k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3 Nfd7 10. f4

			variation => N__('Neo-Grünfeld, Main Line, 10. f4'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp1n1pbp/2n1p1p1/3pN3/3P1P2/2N3P1/PP2P1BP/R1BQ1RK1 w - - 1 11',
			},
		},
		'rnbq1rk1/pp1n1pbp/4p1p1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 2 10' => {
			eco => 'D79j',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, Main Line'),
			moves => {
				'f2f4' => 'rnbq1rk1/pp1n1pbp/4p1p1/3pN3/3P1P2/2N3P1/PP2P1BP/R1BQ1RK1 b - f3 0 10',
			},
		},
		'rnbq1rk1/pp1p1ppp/3b1n2/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R b KQ - 2 7' => {
			eco => 'A60n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 Bd6 6. e4 O-O 7. Nf3

			variation => N__('Benoni: Snake, 6. e4 O-O 7. Nf3'),
			moves => {
			},
		},
		'rnbq1rk1/pp1p1ppp/3b1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQ - 1 7' => {
			eco => 'A60m',
                        # TRANSLATORS:

			variation => N__('Benoni: Snake, 6. e4'),
			moves => {
				'g1f3' => 'rnbq1rk1/pp1p1ppp/3b1n2/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R b KQ - 2 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP2B2/2N5/PPQ1PPPP/R3KBNR b KQ - 2 6' => {
			eco => 'E39b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Bf4

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Bf4'),
			moves => {
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP5/2N2N2/PPQ1PPPP/R1B1KB1R b KQ - 2 6' => {
			eco => 'E39c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Nf3

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Nf3'),
			moves => {
				'b4c5' => 'rnbq1rk1/pp1p1ppp/4pn2/2b5/2P5/2N2N2/PPQ1PPPP/R1B1KB1R w KQ - 0 7',
				'b8a6' => 'r1bq1rk1/pp1p1ppp/n3pn2/2P5/1bP5/2N2N2/PPQ1PPPP/R1B1KB1R w KQ - 3 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQ - 1 6' => {
			eco => 'E39a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O

			variation => N__('Nimzo-Indian: Classical, Pirc Variation'),
			moves => {
				'a2a3' => 'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP5/P1N5/1PQ1PPPP/R1B1KBNR b KQ - 0 6',
				'c1f4' => 'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP2B2/2N5/PPQ1PPPP/R3KBNR b KQ - 2 6',
				'g1f3' => 'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP5/2N2N2/PPQ1PPPP/R1B1KB1R b KQ - 2 6',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP5/P1N5/1PQ1PPPP/R1B1KBNR b KQ - 0 6' => {
			eco => 'E39i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. a3

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. a3'),
			moves => {
				'b4c5' => 'rnbq1rk1/pp1p1ppp/4pn2/2b5/2P5/P1N5/1PQ1PPPP/R1B1KBNR w KQ - 0 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2b5/2P5/2N2N2/PPQ1PPPP/R1B1KB1R w KQ - 0 7' => {
			eco => 'E39d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. Nf3 Bxc5

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. Nf3 Bxc5'),
			moves => {
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2b5/2P5/P1N2N2/1PQ1PPPP/R1B1KB1R b KQ - 1 7' => {
			eco => 'E39j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 O-O 6. a3 Bxc5 7. Nf3

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. a3 Bxc5 7. Nf3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p2p1ppp/1p2pn2/2b5/2P5/P1N2N2/1PQ1PPPP/R1B1KB1R w KQ - 0 8',
				'b8c6' => 'r1bq1rk1/pp1p1ppp/2n1pn2/2b5/2P5/P1N2N2/1PQ1PPPP/R1B1KB1R w KQ - 2 8',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2b5/2P5/P1N5/1PQ1PPPP/R1B1KBNR w KQ - 0 7' => {
			eco => 'E39i',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, Pirc, 6. a3'),
			moves => {
				'g1f3' => 'rnbq1rk1/pp1p1ppp/4pn2/2b5/2P5/P1N2N2/1PQ1PPPP/R1B1KB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 c5 6. Be2

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 c5 6. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQ c6 0 6' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 c5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 c5'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 1 6',
				'f1e2' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 1 6',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2N2NP1/PP2PP1P/R1BQKB1R w KQ - 2 6' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 O-O

			variation => N__('Nimzo-Indian: Romanishin, 4... c5 5. Nf3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2NBP3/PP2NPPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E47',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 c5 6. Ne2

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 c5 6. Ne2'),
			moves => {
				'c5d4' => 'rnbq1rk1/pp1p1ppp/4pn2/8/1bPp4/2NBP3/PP2NPPP/R1BQK2R w KQ - 0 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ c6 0 6' => {
			eco => 'E47',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 c5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 c5'),
			moves => {
				'g1e2' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2NBP3/PP2NPPP/R1BQK2R b KQ - 1 6',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 c5 6. Bd3

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 c5 6. Bd3'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 2 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/2PP4/P1P1P3/4NPPP/R1BQKB1R b KQ - 1 7' => {
			eco => 'E28',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Ne2

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Ne2'),
			moves => {
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQ c6 0 7' => {
			eco => 'E28',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR b KQ - 1 7',
				'g1e2' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/2PP4/P1P1P3/4NPPP/R1BQKB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR b KQ - 1 7' => {
			eco => 'E28',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3 c5 7. Bd3

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3 c5 7. Bd3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p2p1ppp/1p2pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR w KQ - 0 8',
				'b8c6' => 'r1bq1rk1/pp1p1ppp/2n1pn2/2p5/2PP4/P1PBP3/5PPP/R1BQK1NR w KQ - 2 8',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/8/1bPN4/2N3P1/PP2PP1P/R1BQKB1R w KQ - 1 7' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 cxd4 6. Nxd4 O-O

			variation => N__('Nimzo-Indian: Romanishin, 6.Nxd4 O-O'),
			moves => {
				'f1g2' => 'rnbq1rk1/pp1p1ppp/4pn2/8/1bPN4/2N3P1/PP2PPBP/R1BQK2R b KQ - 2 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/8/1bPN4/2N3P1/PP2PPBP/R1BQK2R b KQ - 2 7' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 cxd4 6. Nxd4 O-O 7. Bg2

			variation => N__('Nimzo-Indian: Romanishin, 6.Nxd4 O-O 7. Bg2'),
			moves => {
				'd7d5' => 'rnbq1rk1/pp3ppp/4pn2/3p4/1bPN4/2N3P1/PP2PPBP/R1BQK2R w KQ d6 0 8',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/8/1bPP4/2N5/PP2NPPP/R1BQKB1R w KQ - 1 7' => {
			eco => 'E42',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Ne2 cxd4 6. exd4 O-O

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2 cxd4 6.exd4 O-O'),
			moves => {
				'a2a3' => 'rnbq1rk1/pp1p1ppp/4pn2/8/1bPP4/P1N5/1P2NPPP/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/8/1bPP4/P1N5/1P2NPPP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E42',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2 cxd4 6.exd4 O-O'),
			moves => {
				'b4e7' => 'rnbq1rk1/pp1pbppp/4pn2/8/2PP4/P1N5/1P2NPPP/R1BQKB1R w KQ - 1 8',
			},
		},
		'rnbq1rk1/pp1p1ppp/4pn2/8/1bPp4/2NBP3/PP2NPPP/R1BQK2R w KQ - 0 7' => {
			eco => 'E47',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 c5 6. Ne2 cxd4

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 c5 6. Ne2 cxd4'),
			moves => {
			},
		},
		'rnbq1rk1/pp1pb1pp/2p1pn2/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. b3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6.O-O c6 7. b3'),
			moves => {
				'a7a5' => 'rnbq1rk1/1p1pb1pp/2p1pn2/p4p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - a6 0 8',
			},
		},
		'rnbq1rk1/pp1pb1pp/2p1pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6.O-O c6'),
			moves => {
				'b2b3' => 'rnbq1rk1/pp1pb1pp/2p1pn2/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/pp1pbppp/4pn2/8/2PP4/P1N5/1P2NPPP/R1BQKB1R w KQ - 1 8' => {
			eco => 'E42',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Ne2 cxd4 6. exd4 O-O 7. a3 Be7

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2 cxd4 6.exd4 O-O 7. a3 Be7'),
			moves => {
			},
		},
		'rnbq1rk1/pp1pp1bp/2p2np1/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6

			variation => N__('Dutch: Leningrad, Main Line, 6... c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp1pppbp/5np1/2p3B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6' => {
			eco => 'E61t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 c5 6. e3

			variation => N__('King\'s Indian: Smyslov System, 5... c5 6. e3'),
			moves => {
			},
		},
		'rnbq1rk1/pp1pppbp/5np1/2p3B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQ c6 0 6' => {
			eco => 'E61t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 c5

			variation => N__('King\'s Indian: Smyslov System, 5... c5'),
			moves => {
				'e2e3' => 'rnbq1rk1/pp1pppbp/5np1/2p3B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/pp1pppbp/5np1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w - c6 0 6' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2 Bg7 4. O-O O-O 5. d3 c5

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1p3/3p1p2/2PPn3/1P3NP1/P1Q1PPBP/RNB2RK1 w - - 2 9' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Qc2 Ne4

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Qc2 Ne4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP1B2/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 8' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. Bf4

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. Bf4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 b - - 1 8' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Qc2

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Qc2'),
			moves => {
				'f6e4' => 'rnbq1rk1/pp2b1pp/2p1p3/3p1p2/2PPn3/1P3NP1/P1Q1PPBP/RNB2RK1 w - - 2 9',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6

			variation => N__('Dutch: Stonewall, Botvinnik, 7... c6'),
			moves => {
				'c1a3' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 b - - 1 8',
				'c1b2' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 1 8',
				'd1c2' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1P3NP1/P1Q1PPBP/RNB2RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1P3NP1/PB2PPBP/RN1Q1RK1 b - - 1 8' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Bb2

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Bb2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. b3

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. b3'),
			moves => {
				'd8e8' => 'rnb1qrk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6

			variation => N__('Dutch: Stonewall, 7. Nc3 c6'),
			moves => {
				'b2b3' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'c1f4' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP1B2/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 8',
				'c1g5' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1pB1/2PP4/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 8',
				'd1c2' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8',
				'f3e5' => 'rnbq1rk1/pp2b1pp/2p1pn2/3pNp2/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. Qc2

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. Qc2'),
			moves => {
				'd8e8' => 'rnb1qrk1/pp2b1pp/2p1pn2/3p1p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 w - - 2 9',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nbd2 c6

			variation => N__('Dutch: Stonewall, 7.Nbd2 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Qc2 c6

			variation => N__('Dutch: Stonewall, 7. Qc2 c6'),
			moves => {
				'b1d2' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PPQNPPBP/R1B2RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PPQNPPBP/R1B2RK1 b - - 1 8' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Qc2 c6 8. Nbd2

			variation => N__('Dutch: Stonewall, 7. Qc2 c6 8.Nbd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 b - - 1 8' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1nb1pp/2p1pn2/3p1p2/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 w - - 2 9',
				'c8d7' => 'rn1q1rk1/pp1bb1pp/2p1pn2/3p1p2/2PP4/BP3NP1/P3PPBP/RN1Q1RK1 w - - 2 9',
				'e7a3' => 'rnbq1rk1/pp4pp/2p1pn2/3p1p2/2PP4/bP3NP1/P3PPBP/RN1Q1RK1 w - - 0 9',
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3p1pB1/2PP4/2N2NP1/PP2PPBP/R2Q1RK1 b - - 1 8' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. Bg5

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2p1pn2/3pNp2/2PP4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 8' => {
			eco => 'A95',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3 c6 8. Ne5

			variation => N__('Dutch: Stonewall, 7. Nc3 c6 8. Ne5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2b1pp/2pppn2/5p2/2PP4/2N3PN/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6. O-O d6 7. Nc3 c6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6.O-O d6 7. Nc3 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bpp1/2p1pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 8' => {
			eco => 'D43k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4 Be7 7. e3 O-O

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Be7 7. e3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bpp1/2p2n1p/3p4/3P3B/2N1P3/PPQ2PPP/R3KBNR w KQ - 2 9' => {
			eco => 'D36g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 h6 8. Bh4 O-O

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 h6'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp2bpp1/2p2n1p/3p4/3P3B/2NBP3/PPQ2PPP/R3K1NR b KQ - 3 9',
			},
		},
		'rnbq1rk1/pp2bpp1/2p2n1p/3p4/3P3B/2NBP3/PPQ2PPP/R3K1NR b KQ - 3 9' => {
			eco => 'D36g',
                        # TRANSLATORS:

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 h6'),
			moves => {
				'f8e8' => 'rnbqr1k1/pp2bpp1/2p2n1p/3p4/3P3B/2NBP3/PPQ2PPP/R3K1NR w KQ - 4 10',
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7' => {
			eco => 'D43h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 Be7 6. e3 O-O

			variation => N__('Semi-Slav: Botvinnik, 5... Be7 6. e3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'D37l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c6

			variation => N__('QGD: Classical, 5...O-O 6. e3 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. b3

			variation => N__('Catalan: Closed, 6.O-O c6 7. b3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. Nc3

			variation => N__('Catalan: Closed, 6.O-O c6 7. Nc3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1Q1RK1 w - - 0 8' => {
			eco => 'E11h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7. O-O c6

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7.O-O c6'),
			moves => {
				'd1c2' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PPQBPPBP/RN3RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. Nbd2

			variation => N__('Catalan: Closed, 6.O-O c6 7.Nbd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6

			variation => N__('Catalan: Closed, 6.O-O c6'),
			moves => {
				'b1c3' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
				'b1d2' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 1 7',
				'b2b3' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
				'd1c2' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 1 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c6 7. Qc2

			variation => N__('Catalan: Closed, 6.O-O c6 7. Qc2'),
			moves => {
				'b7b6' => 'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PPQBPPBP/RN3RK1 b - - 1 8' => {
			eco => 'E11i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7. O-O c6 8. Qc2

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O 7.O-O c6 8. Qc2'),
			moves => {
				'b7b6' => 'rnbq1rk1/p3bppp/1pp1pn2/3p4/2PP4/5NP1/PPQBPPBP/RN3RK1 w - - 0 9',
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/2B4Q/2N2N2/PPP2PPP/2KRR3 b - - 3 10' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 c6 8. O-O-O d5 9. Rhe1 O-O 10. Qh4

			variation => N__('Bishop\'s Opening: Urusov Gambit, Larsen Variation'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KRR3 w - - 2 10' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'd4h4' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/2B4Q/2N2N2/PPP2PPP/2KRR3 b - - 3 10',
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR w KQ - 1 8' => {
			eco => 'D36f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 O-O

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 O-O'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR b KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R w KQ - 4 8' => {
			eco => 'D36c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. Nf3 O-O

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. Nf3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PP2NPPP/R2QK2R b KQ - 4 8' => {
			eco => 'D35v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 O-O 8. Nge2

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 O-O 8.Nge2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PP3PPP/R2QK1NR w KQ - 3 8' => {
			eco => 'D35v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 O-O

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 O-O'),
			moves => {
				'g1e2' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PP2NPPP/R2QK2R b KQ - 4 8',
				'g1f3' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PP3PPP/R2QK2R b KQ - 4 8',
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PPQ2PPP/R3K1NR b KQ - 2 8' => {
			eco => 'D36f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 O-O 8. Bd3

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 O-O 8. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBPN2/PP3PPP/R2QK2R b KQ - 4 8' => {
			eco => 'D35v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 O-O 8. Nf3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 O-O 8. Nf3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2p5/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w - c6 0 8' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'd4f3' => 'rnbq1rk1/pp2bppp/3p1n2/2p5/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2p5/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 b - - 1 8' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2bppp/2np1n2/2p5/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 2 9',
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7' => {
			eco => 'A56w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. Nf3 O-O 7. h3

			variation => N__('Benoni: Czech, 5. e4 Be7 6. Nf3 O-O 7. h3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 4 7' => {
			eco => 'A56x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. Nf3 O-O 7. Be2

			variation => N__('Benoni: Czech, 5. e4 Be7 6. Nf3 O-O 7. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N2/PP3PPP/R1BQKB1R w KQ - 3 7' => {
			eco => 'A56w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. Nf3 O-O

			variation => N__('Benoni: Czech, 5. e4 Be7 6. Nf3 O-O'),
			moves => {
				'f1e2' => 'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 4 7',
				'h2h3' => 'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N3P1/PP3P1P/R1BQKBNR w KQ - 1 7' => {
			eco => 'A56s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. g3 O-O

			variation => N__('Benoni: Czech, 5. e4 Be7 6. g3 O-O'),
			moves => {
				'f1g2' => 'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N3P1/PP3PBP/R1BQK1NR b KQ - 2 7',
			},
		},
		'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N3P1/PP3PBP/R1BQK1NR b KQ - 2 7' => {
			eco => 'A56s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. g3 O-O 7. Bg2

			variation => N__('Benoni: Czech, 5. e4 Be7 6. g3 O-O 7. Bg2'),
			moves => {
				'f6e8' => 'rnbqnrk1/pp2bppp/3p4/2pPp3/2P1P3/2N3P1/PP3PBP/R1BQK1NR w KQ - 3 8',
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2Pp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7' => {
			eco => 'D37n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5'),
			moves => {
				'e7c5' => 'rnbq1rk1/pp3ppp/4pn2/2bp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pP4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c5 7. cxd5

			variation => N__('Catalan: Closed, 6.O-O c5 7.cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pp2B1/2PP4/2N1P3/PPQ2PPP/R3KBNR w KQ c6 0 7' => {
			eco => 'D53v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Qc2 c5

			variation => N__('QGD: 4. Bg5 Be7 5. e3 O-O 6. Qc2 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pp2B1/2PP4/2N2N2/PPQ1PPPP/R3KB1R w KQ c6 0 7' => {
			eco => 'D53o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 O-O 6. Qc2 c5

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 O-O 6. Qc2 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pp4/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 w - c6 0 7' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 c5

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 c5'),
			moves => {
				'c1b2' => 'rnbq1rk1/pp2bppp/4pn2/2pp4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pp4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 b - - 1 7' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3 c5 7. Bb2

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3 c5 7. Bb2'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2bppp/2n1pn2/2pp4/2P5/1P3NP1/PB1PPPBP/RN1Q1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pp4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ c6 0 7' => {
			eco => 'D37m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2bppp/4pn2/2Pp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2bppp/4pn2/2pp4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 7' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O c5

			variation => N__('Catalan: Closed, 6.O-O c5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp2bppp/4pn2/2pP4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/pp2nppp/4p3/2ppP3/3P2Q1/P1P2N2/2P2PPP/R1B1KB1R b KQ - 4 8' => {
			eco => 'C18q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 O-O 8. Nf3

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 O-O 8. Nf3'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2nppp/2n1p3/2ppP3/3P2Q1/P1P2N2/2P2PPP/R1B1KB1R w KQ - 5 9',
			},
		},
		'rnbq1rk1/pp2nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQ - 3 8' => {
			eco => 'C18n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 O-O

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 O-O'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp2nppp/4p3/2ppP3/3P2Q1/P1PB4/2P2PPP/R1B1K1NR b KQ - 4 8',
				'g1f3' => 'rnbq1rk1/pp2nppp/4p3/2ppP3/3P2Q1/P1P2N2/2P2PPP/R1B1KB1R b KQ - 4 8',
			},
		},
		'rnbq1rk1/pp2nppp/4p3/2ppP3/3P2Q1/P1PB4/2P2PPP/R1B1K1NR b KQ - 4 8' => {
			eco => 'C18o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 O-O 8. Bd3

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 O-O 8. Bd3'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2nppp/2n1p3/2ppP3/3P2Q1/P1PB4/2P2PPP/R1B1K1NR w KQ - 5 9',
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5'),
			moves => {
				'c6d5' => 'rnbq1rk1/pp2p1bp/3p1np1/3p1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9',
				'c8d7' => 'rn1q1rk1/pp1bp1bp/2pp1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
				'd8a5' => 'rnb2rk1/pp2p1bp/2pp1np1/q2P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
				'd8e8' => 'rnb1qrk1/pp2p1bp/2pp1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 1 9',
				'e7e5' => 'rnbq1rk1/pp4bp/2pp1np1/3Ppp2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - e6 0 9',
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/3P1p2/2P5/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. d5 c6

			variation => N__('Dutch: Leningrad, Main Line, 7. d5 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. b3 c6

			variation => N__('Dutch: Leningrad, Main Line, 7. b3 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. b3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. b3'),
			moves => {
				'b8a6' => 'r1bq1rk1/pp2p1bp/n1pp1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
				'd8a5' => 'rnb2rk1/pp2p1bp/2pp1np1/q4p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
				'd8e8' => 'rnb1qrk1/pp2p1bp/2pp1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6'),
			moves => {
				'b2b3' => 'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/1PN2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
				'd1c2' => 'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8',
				'd4d5' => 'rnbq1rk1/pp2p1bp/2pp1np1/3P1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 8',
				'f1e1' => 'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQR1K1 b - - 1 8' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. Re1

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. Re1'),
			moves => {
			},
		},
		'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/2N2NP1/PPQ1PPBP/R1B2RK1 b - - 1 8' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. Qc2

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. Qc2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2p1bp/3p1np1/2pP1p2/2P5/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. d5 c5

			variation => N__('Dutch: Leningrad, Main Line, 7. d5 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2p1bp/3p1np1/3p1p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 cxd5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppb1/3p1npp/2p5/2PPP3/2N1B3/PP2BPPP/R2QK1NR w KQ c6 0 8' => {
			eco => 'E73x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6 7. Be3 c5

			variation => N__('King\'s Indian: Averbakh, 6... h6 7. Be3 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppb1/3p1npp/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR w KQ - 0 8' => {
			eco => 'E81h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 c5 7. d5 h6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 c5 7. d5 h6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppb1/3p1npp/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 8' => {
			eco => 'E74j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6'),
			moves => {
				'g5e3' => 'rnbq1rk1/pp2ppb1/3p1npp/2pP4/2P1P3/2N1B3/PP2BPPP/R2QK1NR b KQ - 1 8',
				'g5f4' => 'rnbq1rk1/pp2ppb1/3p1npp/2pP4/2P1PB2/2N5/PP2BPPP/R2QK1NR b KQ - 1 8',
				'g5h4' => 'rnbq1rk1/pp2ppb1/3p1npp/2pP4/2P1P2B/2N5/PP2BPPP/R2QK1NR b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp2ppb1/3p1npp/2pP4/2P1P2B/2N5/PP2BPPP/R2QK1NR b KQ - 1 8' => {
			eco => 'E74j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6 8. Bh4

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6 8. Bh4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppb1/3p1npp/2pP4/2P1P3/2N1B3/PP2BPPP/R2QK1NR b KQ - 1 8' => {
			eco => 'E74k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6 8. Be3

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6 8. Be3'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N1B3/PP2BPPP/R2QK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp2ppb1/3p1npp/2pP4/2P1PB2/2N5/PP2BPPP/R2QK1NR b KQ - 1 8' => {
			eco => 'E74n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6 8. Bf4

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6 8. Bf4'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1PB2/2N5/PP2BPPP/R2QK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3P4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'D79a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5'),
			moves => {
				'c6d5' => 'rnbq1rk1/pp2ppbp/5np1/3p4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP1B2/1QN1PN2/PP3PPP/R3KB1R b KQ - 1 7' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c6 7. Qb3

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c6 7. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP1B2/2N1PN2/PP3PPP/2RQKB1R b K - 1 7' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c6 7. Rc1

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c6 7. Rc1'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c6

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c6'),
			moves => {
				'a1c1' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP1B2/2N1PN2/PP3PPP/2RQKB1R b K - 1 7',
				'd1b3' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP1B2/1QN1PN2/PP3PPP/R3KB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'D78o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. b3

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. b3'),
			moves => {
				'c8f5' => 'rn1q1rk1/pp2ppbp/2p2np1/3p1b2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8',
				'f6e4' => 'rnbq1rk1/pp2ppbp/2p3p1/3p4/2PPn3/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1Q3NP1/PP2PPBP/RNB2RK1 b - - 1 7' => {
			eco => 'D78g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Qb3

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Qb3'),
			moves => {
				'd5c4' => 'rnbq1rk1/pp2ppbp/2p2np1/8/2pP4/1Q3NP1/PP2PPBP/RNB2RK1 w - - 0 8',
				'd8b6' => 'rnb2rk1/pp2ppbp/1qp2np1/3p4/2PP4/1Q3NP1/PP2PPBP/RNB2RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1QN1PN2/PP1B1PPP/R3KB1R b KQ - 1 7' => {
			eco => 'D95d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 c6 7. Bd2

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 c6 7. Bd2'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pbp/2p1pnp1/3p4/2PP4/1QN1PN2/PP1B1PPP/R3KB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7' => {
			eco => 'D95c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 c6

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 c6'),
			moves => {
				'c1d2' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1QN1PN2/PP1B1PPP/R3KB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQ1RK1 b - - 3 7' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 c6 6. Be2 O-O 7. O-O

			variation => N__('Grünfeld: Slav/Schlecter, 6. Be2 O-O 7.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQK2R w KQ - 2 7' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 c6 6. Be2 O-O

			variation => N__('Grünfeld: Slav/Schlecter, 6. Be2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQ1RK1 b - - 3 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'D78e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Nc3

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Nc3'),
			moves => {
				'd5c4' => 'rnbq1rk1/pp2ppbp/2p2np1/8/2pP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'D94',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. e3 O-O 6. Bd3 c6'),
			moves => {
				'c8f5' => 'rn1q1rk1/pp2ppbp/2p2np1/3p1b2/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8',
				'c8g4' => 'rn1q1rk1/pp2ppbp/2p2np1/3p4/2PP2b1/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 0 7' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Bd3 c6

			variation => N__('Grünfeld: 5. e3 O-O 6. Bd3 c6'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'D78k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Nbd2

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.Nbd2'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1nppbp/2p2np1/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 w - - 2 8',
				'c8f5' => 'rn1q1rk1/pp2ppbp/2p2np1/3p1b2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 w - - 2 8',
				'f6e4' => 'rnbq1rk1/pp2ppbp/2p3p1/3p4/2PPn3/5NP1/PP1NPPBP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'D78a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6

			variation => N__('Neo-Grünfeld, 6.O-O c6'),
			moves => {
				'b1a3' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/N4NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
				'b1c3' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
				'b1d2' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 1 7',
				'b2b3' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
				'c4d5' => 'rnbq1rk1/pp2ppbp/2p2np1/3P4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
				'd1a4' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/Q1PP4/5NP1/PP2PPBP/RNB2RK1 b - - 1 7',
				'd1b3' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1Q3NP1/PP2PPBP/RNB2RK1 b - - 1 7',
				'f3e5' => 'rnbq1rk1/pp2ppbp/2p2np1/3pN3/2PP4/6P1/PP2PPBP/RNBQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/N4NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'D78b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Na3

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Na3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/5P2/3P1NP1/PPP1P1BP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. O-O O-O 6. d3 c6

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6: 6. d3 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3p4/Q1PP4/5NP1/PP2PPBP/RNB2RK1 b - - 1 7' => {
			eco => 'D78d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Qa4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Qa4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/3pN3/2PP4/6P1/PP2PPBP/RNBQ1RK1 b - - 1 7' => {
			eco => 'D78c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Ne5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Ne5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/8/2QP1B2/2N2N2/PP2PPPP/R3KB1R w KQ - 0 8' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. Bf4 c6

			variation => N__('Grünfeld: Russian, 7. Bf4 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/8/2QP4/5NP1/PP2PPBP/RNB2RK1 b - - 0 8' => {
			eco => 'D78j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Qb3 dxc4 8. Qxc4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Qb3 dxc4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 1 8' => {
			eco => 'D97c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 c6 8. Be2

			variation => N__('Grünfeld: Russian, Szabo, 8. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 0 8' => {
			eco => 'D97c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 c6

			variation => N__('Grünfeld: Russian, Szabo Variation'),
			moves => {
				'f1e2' => 'rnbq1rk1/pp2ppbp/2p2np1/8/2QPP3/2N2N2/PP2BPPP/R1B1K2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/8/2pP4/1Q3NP1/PP2PPBP/RNB2RK1 w - - 0 8' => {
			eco => 'D78i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Qb3 dxc4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Qb3 dxc4'),
			moves => {
				'b3c4' => 'rnbq1rk1/pp2ppbp/2p2np1/8/2QP4/5NP1/PP2PPBP/RNB2RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2p2np1/8/2pP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'D78f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Nc3 dxc4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. Nc3 dxc4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p3p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p3p1/3p4/2PPn3/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8' => {
			eco => 'D78q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. b3 Ne4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7. b3 Ne4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p3p1/3p4/2PPn3/5NP1/PP1NPPBP/R1BQ1RK1 w - - 2 8' => {
			eco => 'D78m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. Nbd2 Ne4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.Nbd2 Ne4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2p3p1/8/B2PP3/2P5/P4PPP/R1BQK1NR w KQ - 2 9' => {
			eco => 'D85f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bb5+ c6 8. Ba4 O-O

			variation => N__('Grünfeld: Exchange, 7. Bb5+ c6 8. Ba4 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'E61w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3 c6

			variation => N__('King\'s Indian: Smyslov System, 6. e3 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 0 7' => {
			eco => 'E71h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 c6

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR b KQ - 1 7' => {
			eco => 'E73k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c6 7. Qd2

			variation => N__('King\'s Indian: Averbakh, 6... c6 7. Qd2'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1nppbp/2pp1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR w KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 0 7' => {
			eco => 'E73j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c6

			variation => N__('King\'s Indian: Averbakh, 6... c6'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PPP3/2N5/PP1QBPPP/R3K1NR b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E62d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6 7. O-O

			variation => N__('King\'s Indian: Fianchetto, 6... c6 7.O-O'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p2ppbp/p1pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'c8f5' => 'rn1q1rk1/pp2ppbp/2pp1np1/5b2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'd8a5' => 'rnb2rk1/pp2ppbp/2pp1np1/q7/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 7' => {
			eco => 'E62c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c6

			variation => N__('King\'s Indian: Fianchetto, 6... c6'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'E60m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O c6

			variation => N__('King\'s Indian: Fianchetto without Nc3, 6... c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 7' => {
			eco => 'E81l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c6'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E91e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c6 7. O-O

			variation => N__('King\'s Indian: 6. Be2 c6 7.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 7' => {
			eco => 'E91d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c6

			variation => N__('King\'s Indian: 6. Be2 c6'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR b KQ - 1 7' => {
			eco => 'E81l',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c6'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p2ppbp/p1pp1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/3PP3/2N1BN1P/PPP2PP1/R2QKB1R w KQ - 0 7' => {
			eco => 'B08e',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3 O-O 6. Be3 c6

			variation => N__('Pirc: Classical, Spassky System, 6... c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/3PP3/2N2N1P/PPP1BPP1/R1BQ1RK1 b - - 0 7' => {
			eco => 'B08q',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O c6 7. h3

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O c6 7. h3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 0 7' => {
			eco => 'B08p',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O c6

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O c6'),
			moves => {
				'a2a4' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/P2PP3/2N2N2/1PP1BPPP/R1BQ1RK1 b - a3 0 7',
				'h2h3' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/3PP3/2N2N1P/PPP1BPP1/R1BQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/2pp1np1/8/P2PP3/2N2N2/1PP1BPPP/R1BQ1RK1 b - a3 0 7' => {
			eco => 'B08r',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O c6 7. a4

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O c6 7. a4'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1nppbp/2pp1np1/8/P2PP3/2N2N2/1PP1BPPP/R1BQ1RK1 w - - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2P3B1/2P1P3/2N5/PP2BPPP/R2QK1NR b KQ - 0 7' => {
			eco => 'E74b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. dxc5

			variation => N__('King\'s Indian: Averbakh, 6... c5 7.dxc5'),
			moves => {
				'd8a5' => 'rnb2rk1/pp2ppbp/3p1np1/q1P3B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7' => {
			eco => 'E81n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5 7. dxc5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 7.dxc5'),
			moves => {
				'd6c5' => 'rnbq1rk1/pp2ppbp/5np1/2p5/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P1PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E76l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. dxc5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7.dxc5'),
			moves => {
				'd8a5' => 'rnb2rk1/pp2ppbp/3p1np1/q1P5/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P5/2N2NP1/PP2PPBP/R1BQK2R b KQ - 0 7' => {
			eco => 'E64b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. dxc5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, Early Exchange'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'E60l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O c5 7. dxc5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav without Nc3, 7.dxc5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ c6 0 7' => {
			eco => 'E61x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3 c5

			variation => N__('King\'s Indian: Smyslov System, 6. e3 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PPP3/2N2P2/PP4PP/R2QKBNR w KQ c6 0 7' => {
			eco => 'E81h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 c5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ c6 0 7' => {
			eco => 'E71m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 c5

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP2B1/2P1P3/2N4P/PP3PP1/R2QKBNR b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ c6 0 7' => {
			eco => 'E74a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5

			variation => N__('King\'s Indian: Averbakh, 6... c5'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2P3B1/2P1P3/2N5/PP2BPPP/R2QK1NR b KQ - 0 7',
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/3PP3/2P2N2/PP1N1PPP/R2QKB1R w KQ c6 0 7' => {
			eco => 'A48x',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. c3 d6 6. e4 c5

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/1P3NP1/PB2PP1P/RN1QKB1R w KQ c6 0 7' => {
			eco => 'E60i',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: b3+g3 System'),
			moves => {
				'f1g2' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/1P3NP1/PB2PPBP/RN1QK2R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/1P3NP1/PB2PPBP/RN1QK2R b KQ - 1 7' => {
			eco => 'E60i',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: b3+g3 System'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/1P3NP1/PB2PPBP/RN1QK2R w KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N1PN2/PP2BPPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E61o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. e3 d6 6. Be2 c5 7. O-O

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. e3 d6 6. Be2 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N1PN2/PP2BPPP/R1BQK2R w KQ c6 0 7' => {
			eco => 'E61o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. e3 d6 6. Be2 c5

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. e3 d6 6. Be2 c5'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N1PN2/PP2BPPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E65a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7.O-O'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'b8d7' => 'r1bq1rk1/pp1nppbp/3p1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'c5d4' => 'rnbq1rk1/pp2ppbp/3p1np1/8/2Pp4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ c6 0 7' => {
			eco => 'E64a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav System'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P5/2N2NP1/PP2PPBP/R1BQK2R b KQ - 0 7',
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQK2R b KQ - 0 7',
				'e1g1' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 7' => {
			eco => 'E60l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O c5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav without Nc3'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 1 7' => {
			eco => 'E90t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. Be3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BN2/PP3PPP/R2QKB1R w KQ c6 0 7' => {
			eco => 'E90h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be3 c5

			variation => N__('King\'s Indian: Larsen, 6... c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 1 7' => {
			eco => 'E81o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5 7. Nge2

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 7.Nge2'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ c6 0 7' => {
			eco => 'E81m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7',
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7',
				'g1e2' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ c6 0 7' => {
			eco => 'E90t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5'),
			moves => {
				'c1e3' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 1 7',
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E91j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. O-O

			variation => N__('King\'s Indian: 6. Be2 c5 7.O-O'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ c6 0 7' => {
			eco => 'E91f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5

			variation => N__('King\'s Indian: 6. Be2 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7',
				'e1g1' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2P2/PP2N1PP/R1BQKB1R w KQ c6 0 7' => {
			eco => 'E81d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Nge2 c5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6.Nge2 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2P2/PP2N1PP/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ c6 0 7' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2 c5

			variation => N__('King\'s Indian: 4. e4 d6 5. g3 O-O 6. Bg2 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2NB4/PP2NPPP/R1BQK2R w KQ c6 0 7' => {
			eco => 'E70o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 c5

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2NB4/PP2NPPP/R1BQK2R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPPP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 1 7' => {
			eco => 'E78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3

			variation => N__('King\'s Indian: Four Pawns Attack, 7. Nf3'),
			moves => {
				'c5d4' => 'rnbq1rk1/pp2ppbp/3p1np1/8/2PpPP2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 8',
				'c8g4' => 'rn1q1rk1/pp2ppbp/3p1np1/2p5/2PPPPb1/2N2N2/PP2B1PP/R1BQK2R w KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPPP2/2N2N2/PP4PP/R1BQKB1R w KQ c6 0 7' => {
			eco => 'E76k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2P5/2P1PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 7',
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPPP2/2N5/PP2B1PP/R1BQK1NR w KQ c6 0 7' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5'),
			moves => {
				'd4d5' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1PP2/2N5/PP2B1PP/R1BQK1NR b KQ - 0 7',
				'g1f3' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPPP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 1 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR b KQ - 0 7' => {
			eco => 'E81h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 c5 7. d5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 c5 7. d5'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR w KQ - 0 8',
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR w KQ - 0 8',
				'h7h6' => 'rnbq1rk1/pp2ppb1/3p1npp/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP2B1/2P1P3/2N4P/PP3PP1/R2QKBNR b KQ - 0 7' => {
			eco => 'E71m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 c5 7. d5

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 c5 7. d5'),
			moves => {
				'b7b5' => 'rnbq1rk1/p3ppbp/3p1np1/1ppP2B1/2P1P3/2N4P/PP3PP1/R2QKBNR w KQ b6 0 8',
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N4P/PP3PP1/R2QKBNR w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR b KQ - 0 7' => {
			eco => 'E74d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p2ppbp/p2p1np1/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 8',
				'b7b5' => 'rnbq1rk1/p3ppbp/3p1np1/1ppP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ b6 0 8',
				'd8a5' => 'rnb2rk1/pp2ppbp/3p1np1/q1pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 1 8',
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 8',
				'h7h6' => 'rnbq1rk1/pp2ppb1/3p1npp/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7' => {
			eco => 'E81m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 c5 7. d5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 7. d5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7' => {
			eco => 'E90u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E91g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. d5

			variation => N__('King\'s Indian: 6. Be2 c5 7. d5'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N2/PP3PPP/R1BQKB1R w KQ - 1 7' => {
			eco => 'E90d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 c5 6. d5 O-O

			variation => N__('King\'s Indian: 5. Nf3 c5 6. d5 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2P2/PP2N1PP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E81d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Nge2 c5 7. d5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6.Nge2 c5 7. d5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2NB4/PP2NPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E70p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 c5 7. d5

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 c5 7. d5'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2NB4/PP2NPPP/R1BQK2R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E76n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. d5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p2ppbp/p2p1np1/2pP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 0 8',
				'b7b5' => 'rnbq1rk1/p3ppbp/3p1np1/1ppP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ b6 0 8',
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1PP2/2N5/PP2B1PP/R1BQK1NR b KQ - 0 7' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. d5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1PP2/2N5/PP2B1PP/R1BQK1NR w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQK2R b KQ - 0 7' => {
			eco => 'E64c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. d5

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7. d5'),
			moves => {
				'b8a6' => 'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQK2R w KQ - 1 8',
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/4P3/2N2N1P/PPP2PP1/R1BQKB1R w KQ - 1 7' => {
			eco => 'A43v',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. h3 O-O

			variation => N__('Old Benoni: Schmidt, 6. h3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 b - - 5 7' => {
			eco => 'A43x',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Be2 O-O 7. O-O

			variation => N__('Old Benoni: Schmidt, 6. Be2 O-O 7.O-O'),
			moves => {
				'b8a6' => 'r1bq1rk1/pp2ppbp/n2p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 6 8',
				'e7e6' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQK2R w KQ - 4 7' => {
			eco => 'A43x',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Be2 O-O

			variation => N__('Old Benoni: Schmidt, 6. Be2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 b - - 5 7',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/6B1/3NP3/2N5/PPP1BPPP/R2Q1RK1 b - - 5 8' => {
			eco => 'B70s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Bg5

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Bg5'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/6B1/3NP3/2N5/PPP1BPPP/R2Q1RK1 w - - 6 9',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQ - 4 8' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Bc4 O-O

			variation => N__('Sicilian: Dragon, 6. Be3 Bg7 7. Bc4 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/2PNPP2/2N5/PP2B1PP/R1BQK2R b KQ - 0 8' => {
			eco => 'E78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4 8. Nxd4

			variation => N__('King\'s Indian: Four Pawns Attack, 7. Nf3 cxd4'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/8/2PNPP2/2N5/PP2B1PP/R1BQK2R w KQ - 1 9',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/2Pp4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E65b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. O-O cxd4

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7.O-O cxd4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/2PpPP2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 8' => {
			eco => 'E78',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. Nf3 cxd4

			variation => N__('King\'s Indian: Four Pawns Attack, 7. Nf3 cxd4'),
			moves => {
				'f3d4' => 'rnbq1rk1/pp2ppbp/3p1np1/8/2PNPP2/2N5/PP2B1PP/R1BQK2R b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b - - 5 8' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 O-O 8. O-O

			variation => N__('Sicilian: Dragon, Classical Attack'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQ - 4 8' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2 O-O

			variation => N__('Sicilian: Dragon, Classical Attack'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP1BPPP/R2Q1RK1 b - - 5 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQ - 1 8' => {
			eco => 'B76a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O

			variation => N__('Sicilian: Dragon, Yugoslav, 7. f3 O-O'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQ - 2 8' => {
			eco => 'B76b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 O-O 8. Qd2

			variation => N__('Sicilian: Dragon, Yugoslav, 7. f3 O-O 8. Qd2'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQ - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w - - 4 8' => {
			eco => 'B70o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O'),
			moves => {
				'c1g5' => 'rnbq1rk1/pp2ppbp/3p1np1/6B1/3NP3/2N5/PPP1BPPP/R2Q1RK1 b - - 5 8',
				'f1e1' => 'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQR1K1 b - - 5 8',
			},
		},
		'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQR1K1 b - - 5 8' => {
			eco => 'B70p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O O-O 8. Re1

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O O-O 8. Re1'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2np1np1/8/3NP3/2N5/PPP1BPPP/R1BQR1K1 w - - 6 9',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2P5/2p5/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'D77c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c5 7. dxc5 dxc4

			variation => N__('Neo-Grünfeld, 6.O-O c5 7.dxc5 dxc4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2Pp4/2P2B2/2N1P3/PP3PPP/2RQKBNR b K - 0 7' => {
			eco => 'D83',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Grünfeld Gambit, Capablanca Variation'),
			moves => {
				'c8e6' => 'rn1q1rk1/pp2ppbp/4bnp1/2Pp4/2P2B2/2N1P3/PP3PPP/2RQKBNR w K - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2Pp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c5 7. dxc5

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c5 7.dxc5'),
			moves => {
				'd8a5' => 'rnb2rk1/pp2ppbp/5np1/q1Pp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2Pp4/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'D77c',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, 6.O-O c5'),
			moves => {
				'd5c4' => 'rnbq1rk1/pp2ppbp/5np1/2P5/2p5/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2p1P3/3P4/5NP1/PP3PBP/RNBQ1RK1 b - - 0 9' => {
			eco => 'D74',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. e4 Nf6'),
			moves => {
				'f6d5' => 'rnbq1rk1/pp2ppbp/6p1/2pnP3/3P4/5NP1/PP3PBP/RNBQ1RK1 w - - 1 10',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2p5/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 8' => {
			eco => 'E81n',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3 c5 7.dxc5'),
			moves => {
				'd1d8' => 'rnbQ1rk1/pp2ppbp/5np1/2p5/2P1P3/2N1BP2/PP4PP/R3KBNR b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2p5/3P4/N1P1PNP1/P4PBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D77i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 c3 8. bxc3 c5 9. e3

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 c3 8.bxc3 c5 9. e3'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2p5/3P4/N1P2NP1/P3PPBP/R1BQ1RK1 w - c6 0 9' => {
			eco => 'D77h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 c3 8. bxc3 c5

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 c3 8.bxc3 c5'),
			moves => {
				'e2e3' => 'rnbq1rk1/pp2ppbp/5np1/2p5/3P4/N1P1PNP1/P4PBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2p5/3PP3/5NP1/PP3PBP/RNBQ1RK1 w - - 1 9' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. e4 Nf6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. e4 Nf6'),
			moves => {
				'e4e5' => 'rnbq1rk1/pp2ppbp/5np1/2p1P3/3P4/5NP1/PP3PBP/RNBQ1RK1 b - - 0 9',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp2B1/3P4/2PBPN2/PP1N1PPP/R2QK2R b KQ - 0 7' => {
			eco => 'D03t',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7 5. e3 O-O 6. Bd3 c5 7. c3

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. Bd3 c5'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1nppbp/5np1/2pp2B1/3P4/2PBPN2/PP1N1PPP/R2QK2R w KQ - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp2B1/3P4/3BPN2/PPPN1PPP/R2QK2R w KQ c6 0 7' => {
			eco => 'D03t',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7 5. e3 O-O 6. Bd3 c5

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. Bd3 c5'),
			moves => {
				'c2c3' => 'rnbq1rk1/pp2ppbp/5np1/2pp2B1/3P4/2PBPN2/PP1N1PPP/R2QK2R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP1B2/2N1P3/PP3PPP/2RQKBNR w K c6 0 7' => {
			eco => 'D83',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. Rc1 c5

			variation => N__('Grünfeld: Grünfeld Gambit, Capablanca Variation'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/5np1/2Pp4/2P2B2/2N1P3/PP3PPP/2RQKBNR b K - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ c6 0 7' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3 c5

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3 c5'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/5np1/2Pp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP1B2/2N2N2/PP2PPPP/2RQKB1R w K c6 0 7' => {
			eco => 'D92',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. Rc1 c5

			variation => N__('Grünfeld: 5. Bf4 O-O 6. Rc1 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP4/2N1PN2/PP2BPPP/R1BQK2R w KQ c6 0 7' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Be2 c5

			variation => N__('Grünfeld: 5. e3 O-O 6. Be2 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 7' => {
			eco => 'D77c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c5

			variation => N__('Neo-Grünfeld, 6.O-O c5'),
			moves => {
				'd4c5' => 'rnbq1rk1/pp2ppbp/5np1/2Pp4/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/5P2/3P1NP1/PPP1P1BP/RNBQ1RK1 w - c6 0 7' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. O-O O-O 6. d3 c5

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6: 6. d3 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w - - 3 7' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 g6 4. O-O Bg7 5. d3 Nf6 6. Nbd2 O-O

			variation => N__('Reti: KIA, 2... c5 + 3... g6'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/3P4/3P4/1QN5/PP2BPPP/R1B1K1NR b KQ - 2 8' => {
			eco => 'B14g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. Qb3 Bg7 7. cxd5 O-O 8. Be2

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6, Main Line, 8. Be2'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1nppbp/5np1/3P4/3P4/1QN5/PP2BPPP/R1B1K1NR w KQ - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/3P4/3P4/1QN5/PP3PPP/R1B1KBNR w KQ - 1 8' => {
			eco => 'B14f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. Qb3 Bg7 7. cxd5 O-O

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6, Main Line'),
			moves => {
				'f1e2' => 'rnbq1rk1/pp2ppbp/5np1/3P4/3P4/1QN5/PP2BPPP/R1B1K1NR b KQ - 2 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 8' => {
			eco => 'D79b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2n2np1/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 9',
				'e7e6' => 'rnbq1rk1/pp3pbp/4pnp1/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9',
				'f6e4' => 'rnbq1rk1/pp2ppbp/6p1/3p4/3Pn3/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 9',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/3p4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8' => {
			eco => 'D79a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5'),
			moves => {
				'b1c3' => 'rnbq1rk1/pp2ppbp/5np1/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 8',
				'f3e5' => 'rnbq1rk1/pp2ppbp/5np1/3pN3/3P4/6P1/PP2PPBP/RNBQ1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp2ppbp/5np1/3pN3/3P4/6P1/PP2PPBP/RNBQ1RK1 b - - 1 8' => {
			eco => 'D79i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Ne5'),
			moves => {
				'e7e6' => 'rnbq1rk1/pp3pbp/4pnp1/3pN3/3P4/6P1/PP2PPBP/RNBQ1RK1 w - - 0 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2Pn4/8/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 8' => {
			eco => 'D75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. dxc5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8.dxc5'),
			moves => {
				'b8a6' => 'r1bq1rk1/pp2ppbp/n5p1/2Pn4/8/5NP1/PP2PPBP/RNBQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/2BPP3/2P1B3/P3NPPP/R2QK2R b KQ - 1 9' => {
			eco => 'D87b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. Be3

			variation => N__('Grünfeld: Classical Exchange, 8... c5 9. Be3'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2n3p1/2p5/2BPP3/2P1B3/P3NPPP/R2QK2R w KQ - 2 10',
				'd8a5' => 'rnb2rk1/pp2ppbp/6p1/q1p5/2BPP3/2P1B3/P3NPPP/R2QK2R w KQ - 2 10',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 1 9' => {
			eco => 'D87e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5 9. O-O

			variation => N__('Grünfeld: Classical Exchange, 9.O-O'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp2ppbp/2n3p1/2p5/2BPP3/2P5/P3NPPP/R1BQ1RK1 w - - 2 10',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/2BPP3/2P5/P3NPPP/R1BQK2R w KQ c6 0 9' => {
			eco => 'D87a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2 c5

			variation => N__('Grünfeld: Classical Exchange, 8... c5'),
			moves => {
				'c1e3' => 'rnbq1rk1/pp2ppbp/6p1/2p5/2BPP3/2P1B3/P3NPPP/R2QK2R b KQ - 1 9',
				'e1g1' => 'rnbq1rk1/pp2ppbp/6p1/2p5/2BPP3/2P5/P3NPPP/R1BQ1RK1 b - - 1 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3P4/2P2NP1/P3PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D75',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. Nc3 Nxc3'),
			moves => {
				'c5d4' => 'rnbq1rk1/pp2ppbp/6p1/8/3p4/2P2NP1/P3PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3P4/2n2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'D75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. Nc3 Nxc3

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. Nc3 Nxc3'),
			moves => {
				'b2c3' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3P4/2P2NP1/P3PPBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1B3/P2Q1PPP/2R1KBNR b K - 3 9' => {
			eco => 'D85g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2 O-O 9. Rc1

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2 O-O 9. Rc1'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1B3/P2Q1PPP/R3KBNR w KQ - 2 9' => {
			eco => 'D85g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2 O-O

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2 O-O'),
			moves => {
				'a1c1' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1B3/P2Q1PPP/2R1KBNR b K - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1BN2/P4PPP/2RQKB1R b K - 3 9' => {
			eco => 'D85w',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O'),
			moves => {
				'd8a5' => 'rnb2rk1/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P4PPP/2RQKB1R w K - 4 10',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1BN2/P4PPP/R2QKB1R w KQ - 2 9' => {
			eco => 'D85w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3 O-O

			variation => N__('Grünfeld: Modern Exchange, 8. Be3 O-O'),
			moves => {
				'a1c1' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1BN2/P4PPP/2RQKB1R b K - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/1RBQK2R b K - 3 9' => {
			eco => 'D85m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2'),
			moves => {
				'b7b6' => 'rnbq1rk1/p3ppbp/1p4p1/2p5/3PP3/2P2N2/P3BPPP/1RBQK2R w K - 0 10',
				'b8c6' => 'r1bq1rk1/pp2ppbp/2n3p1/2p5/3PP3/2P2N2/P3BPPP/1RBQK2R w K - 4 10',
				'c5d4' => 'rnbq1rk1/pp2ppbp/6p1/8/3pP3/2P2N2/P3BPPP/1RBQK2R w K - 0 10',
				'd8a5' => 'rnb2rk1/pp2ppbp/6p1/q1p5/3PP3/2P2N2/P3BPPP/1RBQK2R w K - 4 10',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/R1BQ1RK1 b - - 3 9' => {
			eco => 'D85l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be2 O-O 9. O-O

			variation => N__('Grünfeld: Modern Exchange, 8. Be2 O-O 9.O-O'),
			moves => {
				'b7b6' => 'rnbq1rk1/p3ppbp/1p4p1/2p5/3PP3/2P2N2/P3BPPP/R1BQ1RK1 w - - 0 10',
				'b8c6' => 'r1bq1rk1/pp2ppbp/2n3p1/2p5/3PP3/2P2N2/P3BPPP/R1BQ1RK1 w - - 4 10',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/R1BQK2R w KQ - 2 9' => {
			eco => 'D85l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be2 O-O

			variation => N__('Grünfeld: Modern Exchange, 8. Be2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/R1BQ1RK1 b - - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P4PPP/1RBQKB1R w K - 2 9' => {
			eco => 'D85m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O'),
			moves => {
				'f1e2' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/1RBQK2R b K - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2pn4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 8' => {
			eco => 'D75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. Nc3

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. Nc3'),
			moves => {
				'd5c3' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3P4/2n2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2pn4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 8' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5'),
			moves => {
				'b1c3' => 'rnbq1rk1/pp2ppbp/6p1/2pn4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 8',
				'd4c5' => 'rnbq1rk1/pp2ppbp/6p1/2Pn4/8/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 8',
				'e2e4' => 'rnbq1rk1/pp2ppbp/6p1/2pn4/3PP3/5NP1/PP3PBP/RNBQ1RK1 b - e3 0 8',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2pn4/3PP3/5NP1/PP3PBP/RNBQ1RK1 b - e3 0 8' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. e4

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. e4'),
			moves => {
				'd5f6' => 'rnbq1rk1/pp2ppbp/5np1/2p5/3PP3/5NP1/PP3PBP/RNBQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/2pnP3/3P4/5NP1/PP3PBP/RNBQ1RK1 w - - 1 10' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. e4 Nf6 9. e5 Nd5

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. e4 Nf6 9. e5 Nd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/3p4/3PN3/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'D79e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 Ne4 9. Nxe4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 Ne4 9.Nxe4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/3p4/3Pn3/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 9' => {
			eco => 'D79d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 Ne4

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 Ne4'),
			moves => {
				'c3e4' => 'rnbq1rk1/pp2ppbp/6p1/3p4/3PN3/5NP1/PP2PPBP/R1BQ1RK1 b - - 0 9',
				'f3e5' => 'rnbq1rk1/pp2ppbp/6p1/3pN3/3Pn3/2N3P1/PP2PPBP/R1BQ1RK1 b - - 3 9',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/3pN3/3Pn3/2N3P1/PP2PPBP/R1BQ1RK1 b - - 3 9' => {
			eco => 'D79d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 Ne4 9. Ne5

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 Ne4 9. Ne5'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/8/3PP3/5N2/P3BPPP/1RBQK2R b K - 0 10' => {
			eco => 'D85q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4 10. cxd4

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd 10.cd'),
			moves => {
				'd8a5' => 'rnb2rk1/pp2ppbp/6p1/q7/3PP3/5N2/P3BPPP/1RBQK2R w K - 1 11',
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/8/3p4/2P2NP1/P3PPBP/R1BQ1RK1 w - - 0 10' => {
			eco => 'D75',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O c5 8. Nc3 Nxc3 9. bxc3 cxd4

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O c5 8. Nc3 Nxc3 9.bxc3 cxd4'),
			moves => {
			},
		},
		'rnbq1rk1/pp2ppbp/6p1/8/3pP3/2P2N2/P3BPPP/1RBQK2R w K - 0 10' => {
			eco => 'D85q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1 O-O 9. Be2 cxd4

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1 O-O 9. Be2 cd'),
			moves => {
				'c3d4' => 'rnbq1rk1/pp2ppbp/6p1/8/3PP3/5N2/P3BPPP/1RBQK2R b K - 0 10',
			},
		},
		'rnbq1rk1/pp3pb1/2pp1npp/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QK1NR w KQ - 0 9' => {
			eco => 'E73y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6 7. Be3 e5 8. d5 c6

			variation => N__('King\'s Indian: Averbakh, 6... h6 7. Be3 e5 8. d5 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3p1n1p/2pP2p1/4P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 11' => {
			eco => 'A72',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5'),
			moves => {
				'h4g3' => 'rnbq1rk1/pp3pb1/3p1n1p/2pP2p1/4P3/2N2NB1/PP2BPPP/R2QK2R b KQ - 1 11',
			},
		},
		'rnbq1rk1/pp3pb1/3p1n1p/2pP2p1/4P3/2N2NB1/PP2BPPP/R2QK2R b KQ - 1 11' => {
			eco => 'A72',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5'),
			moves => {
				'f6h5' => 'rnbq1rk1/pp3pb1/3p3p/2pP2pn/4P3/2N2NB1/PP2BPPP/R2QK2R w KQ - 2 12',
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pP2B1/4P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10' => {
			eco => 'A72',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5'),
			moves => {
				'g5h4' => 'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 10',
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 0 10' => {
			eco => 'E75e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 h6 9. Bh4 exd5 10. cxd5

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6 9. Bh4 exd5 10.cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 10' => {
			eco => 'A72',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5'),
			moves => {
				'g6g5' => 'rnbq1rk1/pp3pb1/3p1n1p/2pP2p1/4P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 11',
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 3 10' => {
			eco => 'A65t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pb1/p2p1npp/2pP4/4P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 11',
				'f8e8' => 'rnbqr1k1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 4 11',
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 3 10' => {
			eco => 'A65s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Nge2

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10.Nge2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP4PP/R2QKBNR w KQ - 2 10' => {
			eco => 'A65r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 3 10',
				'g1e2' => 'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 3 10',
			},
		},
		'rnbq1rk1/pp3pb1/3p1npp/2pp4/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10' => {
			eco => 'E75d',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6 9. Bh4'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 0 10',
			},
		},
		'rnbq1rk1/pp3pb1/3p3p/2pP2pn/4P3/2N2NB1/PP2BPPP/R2QK2R w KQ - 2 12' => {
			eco => 'A72',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5'),
			moves => {
				'f3d2' => 'rnbq1rk1/pp3pb1/3p3p/2pP2pn/4P3/2N3B1/PP1NBPPP/R2QK2R b KQ - 3 12',
			},
		},
		'rnbq1rk1/pp3pb1/3p3p/2pP2pn/4P3/2N3B1/PP1NBPPP/R2QK2R b KQ - 3 12' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. Bg5 h6 10. Bh4 g5 11. Bg3 Nh5 12. Nd2

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5 h6 10. Bh4 g5 11. Bg3 Nh5 12. Nd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP2B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'E75c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 h6

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6'),
			moves => {
				'g5d2' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N2N2/PP1BBPPP/R2QK2R b KQ - 1 9',
				'g5e3' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N1BN2/PP2BPPP/R2QK2R b KQ - 1 9',
				'g5f4' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1PB2/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9',
				'g5h4' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P2B/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E75d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 h6 9. Bh4

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6 9. Bh4'),
			moves => {
				'e6d5' => 'rnbq1rk1/pp3pb1/3p1npp/2pp4/2P1P2B/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10',
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N1B3/PP1QBPPP/R3K1NR b KQ - 1 9' => {
			eco => 'E74m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6 8. Be3 e6 9. Qd2

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6 8. Be3 e6 9. Qd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N1B3/PP2BPPP/R2QK1NR w KQ - 0 9' => {
			eco => 'E74l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6 8. Be3 e6

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6 8. Be3 e6'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N1B3/PP1QBPPP/R3K1NR b KQ - 1 9',
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N1BN2/PP2BPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E75c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 h6 9. Be3

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6 9. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1P3/2N2N2/PP1BBPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E75c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 h6 9. Bd2

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6 9. Bd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1PB2/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E75c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 h6 9. Bf4

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 h6 9. Bf4'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pb1/3ppnpp/2pP4/2P1PB2/2N5/PP2BPPP/R2QK1NR w KQ - 0 9' => {
			eco => 'E74o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 h6 8. Bf4 e6

			variation => N__('King\'s Indian: Averbakh, 6... c5 7. d5 h6 8. Bf4 e6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2p1pnp1/3p4/2PP4/1QN1PN2/PP1B1PPP/R3KB1R w KQ - 0 8' => {
			eco => 'D95e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 c6 7. Bd2 e6

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 c6 7. Bd2 e6'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp3pbp/2p1pnp1/3p4/2PP4/1QNBPN2/PP1B1PPP/R3K2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/2p1pnp1/3p4/2PP4/1QNBPN2/PP1B1PPP/R3K2R b KQ - 1 8' => {
			eco => 'D95f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 c6 7. Bd2 e6 8. Bd3

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 c6 7. Bd2 e6 8. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2p3p1/3p4/1P1P4/2N1PN2/P4PPP/R2QKB1R w KQ - 0 11' => {
			eco => 'D91t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. b4 c6

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. b4 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2p3p1/3p4/3P4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 11' => {
			eco => 'D91s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Be2 c6

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2p3p1/3p4/3P4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 11' => {
			eco => 'D91r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Bd3 c6

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 1 8' => {
			eco => 'E88b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Qd2

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6 8. Qd2'),
			moves => {
				'c6d5' => 'rnbq1rk1/pp3pbp/3p1np1/3pp3/2P1P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 8' => {
			eco => 'E86b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. d5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. d5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 1 8' => {
			eco => 'E88a',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6'),
			moves => {
				'c6d5' => 'rnbq1rk1/pp3pbp/3p1np1/3pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 8' => {
			eco => 'E88a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 1 8',
				'f1d3' => 'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2NBBP2/PP4PP/R2QK1NR b KQ - 1 8',
				'g1e2' => 'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E94g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O c6 8. d5

			variation => N__('King\'s Indian: Donner, 8. d5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2NBBP2/PP4PP/R2QK1NR b KQ - 1 8' => {
			eco => 'E88f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Bd3

			variation => N__('King\'s Indian: Sämisch, Orthodox, Polugayevsky'),
			moves => {
				'b7b5' => 'rnbq1rk1/p4pbp/2pp1np1/1p1Pp3/2P1P3/2NBBP2/PP4PP/R2QK1NR w KQ b6 0 9',
				'c6d5' => 'rnbq1rk1/pp3pbp/3p1np1/3pp3/2P1P3/2NBBP2/PP4PP/R2QK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. d3 O-O 7. O-O c6

			variation => N__('English: Bremen, 3... g6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/1QN1BP2/PP2N1PP/R3KB1R b KQ - 1 8' => {
			eco => 'E86c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qb3

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 1 8' => {
			eco => 'E94h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O c6 8. Be3

			variation => N__('King\'s Indian: Donner, 8. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2QK2R w KQ - 0 8' => {
			eco => 'E92h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 c6

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 1 8' => {
			eco => 'E86d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6 8. Qd2

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6 8. Qd2'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1n1pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/R3KB1R w KQ - 2 9',
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 8' => {
			eco => 'E86a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 c6

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 c6'),
			moves => {
				'd1b3' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/1QN1BP2/PP2N1PP/R3KB1R b KQ - 1 8',
				'd1d2' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP1QN1PP/R3KB1R b KQ - 1 8',
				'd4d5' => 'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E94f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O c6

			variation => N__('King\'s Indian: Donner Variation'),
			moves => {
				'c1e3' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2Q1RK1 b - - 1 8',
				'd4d5' => 'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/2N5/2N5/PP2PPPP/R2QKB1R b KQ - 5 9' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Nd2 Bg7 8. Nc4 O-O 9. Bg5

			variation => N__('Benoni: Nimzowitsch, 7... Bg7 8. Nc4 O-O 9. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/2P5/2N2N2/PP2BPPP/R2QK2R b KQ - 0 9' => {
			eco => 'E75b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 exd5 9. exd5

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 exd5 9.exd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/2P5/2N5/PP1QBPPP/R3K1NR b KQ - 0 9' => {
			eco => 'E75g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2 exd5 9. exd5

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2 exd5 9.exd5'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP2B1/2P5/2N5/PP1QBPPP/R3K1NR w KQ - 0 10',
				'f8e8' => 'rnbqr1k1/pp3pbp/3p1np1/2pP2B1/2P5/2N5/PP1QBPPP/R3K1NR w KQ - 1 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2N2/PP2BPPP/R2QK2R b KQ - 5 9' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. Bg5

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bg5'),
			moves => {
				'h7h6' => 'rnbq1rk1/pp3pb1/3p1npp/2pP2B1/4P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP1Q2PP/R3KBNR b KQ - 4 9' => {
			eco => 'A65p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 O-O 9. Qd2

			variation => N__('Benoni: Sämisch, 8. Bg5 O-O 9. Qd2'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP2B1/4P3/2N2P2/PP1Q2PP/R3KBNR w KQ - 0 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP2N1PP/R2QKB1R b KQ - 4 9' => {
			eco => 'A65o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 O-O 9. Nge2

			variation => N__('Benoni: Sämisch, 8. Bg5 O-O 9.Nge2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP4PP/R2QKBNR w KQ - 3 9' => {
			eco => 'A65n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 O-O

			variation => N__('Benoni: Sämisch, 8. Bg5 O-O'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP1Q2PP/R3KBNR b KQ - 4 9',
				'g1e2' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP2N1PP/R2QKB1R b KQ - 4 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/2N2B2/2N5/PP2PPPP/R2QKB1R b KQ - 5 9' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Nd2 Bg7 8. Nc4 O-O 9. Bf4

			variation => N__('Benoni: Nimzowitsch, 7... Bg7 8. Nc4 O-O 9. Bf4'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/2N5/2N5/PP2PPPP/R1BQKB1R w KQ - 4 9' => {
			eco => 'A61',
                        # TRANSLATORS:

			variation => N__('Benoni: Nimzowitsch, 7... Bg7'),
			moves => {
				'c1f4' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/2N2B2/2N5/PP2PPPP/R2QKB1R b KQ - 5 9',
				'c1g5' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/2N5/2N5/PP2PPPP/R2QKB1R b KQ - 5 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/2P2P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 9' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. d5 e6 8. Nf3 exd5 9. exd5

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5 e6 8. Nf3 exd5 9.exd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/2P5/2NB1N1P/PP3PP1/R1BQK2R b KQ - 0 9' => {
			eco => 'E90x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 exd5 9. exd5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 exd5 9.exd5'),
			moves => {
				'f8e8' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/2P5/2NB1N1P/PP3PP1/R1BQK2R w KQ - 1 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 9' => {
			eco => 'A70j',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. h3'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R b KQ - 2 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 5 9' => {
			eco => 'A73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O

			variation => N__('Benoni: Classical, 9.O-O'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 10',
				'b8a6' => 'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10',
				'b8d7' => 'r1bq1rk1/pp1n1pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10',
				'c8g4' => 'rn1q1rk1/pp3pbp/3p1np1/2pP4/4P1b1/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10',
				'f8e8' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 9' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O

			variation => N__('Benoni: Classical, 8. Be2 O-O'),
			moves => {
				'c1f4' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PB2/2N2N2/PP2BPPP/R2QK2R b KQ - 5 9',
				'c1g5' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2N2/PP2BPPP/R2QK2R b KQ - 5 9',
				'e1g1' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 5 9',
				'f3d2' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQK2R b KQ - 5 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQK2R b KQ - 5 9' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. Nd2

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Nd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R b KQ - 2 9' => {
			eco => 'A70k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 0 10',
				'b7b5' => 'rnbq1rk1/p4pbp/3p1np1/1ppP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ b6 0 10',
				'b8a6' => 'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 3 10',
				'c8d7' => 'rn1q1rk1/pp1b1pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 3 10',
				'f8e8' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 3 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB1N2/PP3PPP/R1BQ1RK1 b - - 5 9' => {
			eco => 'A70i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bd3 O-O 9. O-O

			variation => N__('Benoni: Classical, 8. Bd3 O-O 9.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB1N2/PP3PPP/R1BQK2R w KQ - 4 9' => {
			eco => 'A70h',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Bd3'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB1N2/PP3PPP/R1BQ1RK1 b - - 5 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 b - - 5 9' => {
			eco => 'A65g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. Nge2 O-O 9. O-O

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8.Nge2 O-O 9.O-O'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 w - - 0 10',
				'b7b6' => 'rnbq1rk1/p4pbp/1p1p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 w - - 0 10',
				'b8a6' => 'r1bq1rk1/pp3pbp/n2p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 w - - 6 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQK2R w KQ - 4 9' => {
			eco => 'A65g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. Nge2 O-O

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8.Nge2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQ1RK1 b - - 5 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PB2/2N2N2/PP2BPPP/R2QK2R b KQ - 5 9' => {
			eco => 'A72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. Bf4

			variation => N__('Benoni: Classical, 8. Be2 O-O 9. Bf4'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/4PB2/2N2N2/PP2BPPP/R2QK2R w KQ - 0 10',
				'b7b5' => 'rnbq1rk1/p4pbp/3p1np1/1ppP4/4PB2/2N2N2/PP2BPPP/R2QK2R w KQ b6 0 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PB2/2N2N2/PP3PPP/R2QKB1R w KQ - 4 9' => {
			eco => 'A70g',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Bf4'),
			moves => {
				'f3d2' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PB2/2N5/PP1N1PPP/R2QKB1R b KQ - 5 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PB2/2N5/PP1N1PPP/R2QKB1R b KQ - 5 9' => {
			eco => 'A70g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bf4 O-O 9. Nd2

			variation => N__('Benoni: Classical, 8. Bf4 O-O 9. Nd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 4 9' => {
			eco => 'A68c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2

			variation => N__('Benoni: Four Pawns, 9. Be2'),
			moves => {
				'b7b5' => 'rnbq1rk1/p4pbp/3p1np1/1ppP4/4PP2/2N2N2/PP2B1PP/R1BQK2R w KQ b6 0 10',
				'c8g4' => 'rn1q1rk1/pp3pbp/3p1np1/2pP4/4PPb1/2N2N2/PP2B1PP/R1BQK2R w KQ - 5 10',
				'f8e8' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP2B1PP/R1BQK2R w KQ - 5 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 3 9' => {
			eco => 'A68a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O

			variation => N__('Benoni: Four Pawns, 8. Nf3 O-O'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R b KQ - 4 9',
				'f1e2' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 4 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PP2/2NB1N2/PP4PP/R1BQK2R b KQ - 4 9' => {
			eco => 'A68b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Bd3

			variation => N__('Benoni: Four Pawns, 9. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 9' => {
			eco => 'A62',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O

			variation => N__('Benoni: Fianchetto, 9.O-O'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10',
				'b8a6' => 'r1bq1rk1/pp3pbp/n2p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 10',
				'b8d7' => 'r1bq1rk1/pp1n1pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 10',
				'f8e8' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 9' => {
			eco => 'A62',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O

			variation => N__('Benoni: Fianchetto, 8. Bg2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 4 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pPp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 1 8' => {
			eco => 'E92p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 c5 8. Bg5

			variation => N__('King\'s Indian: Petrosian, 7... c5 8. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pPp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ c6 0 8' => {
			eco => 'E87c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pPp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ c6 0 8' => {
			eco => 'E92o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5 c5

			variation => N__('King\'s Indian: Petrosian, 7... c5'),
			moves => {
				'c1g5' => 'rnbq1rk1/pp3pbp/3p1np1/2pPp1B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pp2B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'E75b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3 exd5

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3 exd5'),
			moves => {
				'e4d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/2P5/2N2N2/PP2BPPP/R2QK2R b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pp2B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 0 9' => {
			eco => 'E75g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2 exd5

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2 exd5'),
			moves => {
				'e4d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/2P5/2N5/PP1QBPPP/R3K1NR b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pp4/2P1P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 0 9' => {
			eco => 'E90x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 exd5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 exd5'),
			moves => {
				'e4d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/2P5/2NB1N1P/PP3PP1/R1BQK2R b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2pp4/2P1PP2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 9' => {
			eco => 'E77',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5 e6 8. Nf3'),
			moves => {
				'e4d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/2P2P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 9',
				'e4e5' => 'rnbq1rk1/pp3pbp/3p1np1/2ppP3/2P2P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/2ppP3/2P2P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 9' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. d5 e6 8. Nf3 exd5 9. e5

			variation => N__('King\'s Indian: Four Pawns Attack, Florentine Gambit'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/3Pp3/4P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 0 9' => {
			eco => 'E88c',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6 8. Qd2 cxd5'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/3Pp3/4P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 9' => {
			eco => 'E89a',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3pbp/p2p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 10',
				'b8d7' => 'r1bq1rk1/pp1n1pbp/3p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 1 10',
				'c8d7' => 'rn1q1rk1/pp1b1pbp/3p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 1 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/3Pp3/4P3/2NBBP2/PP4PP/R2QK1NR b KQ - 0 9' => {
			eco => 'E88h',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Orthodox, Polugayevsky, 8...cxd5'),
			moves => {
				'f6h5' => 'rnbq1rk1/pp3pbp/3p2p1/3Pp2n/4P3/2NBBP2/PP4PP/R2QK1NR w KQ - 1 10',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/3pp3/2P1P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 0 9' => {
			eco => 'E88c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Qd2 cxd5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 c6 8. Qd2 cxd5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3pbp/3p1np1/3Pp3/4P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/3pp3/2P1P3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 9' => {
			eco => 'E89a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Nge2 cxd5

			variation => N__('King\'s Indian: Sämisch, Orthodox Main Line'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3pbp/3p1np1/3Pp3/4P3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p1np1/3pp3/2P1P3/2NBBP2/PP4PP/R2QK1NR w KQ - 0 9' => {
			eco => 'E88h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Bd3 cxd5

			variation => N__('King\'s Indian: Sämisch, Orthodox, Polugayevsky, 8...cxd5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3pbp/3p1np1/3Pp3/4P3/2NBBP2/PP4PP/R2QK1NR b KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3p2p1/3Pp2n/4P3/2NBBP2/PP4PP/R2QK1NR w KQ - 1 10' => {
			eco => 'E88i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 c6 8. Bd3 cxd5 9. cxd5 Nh5

			variation => N__('King\'s Indian: Sämisch, Orthodox, Polugayevsky, 8...cxd5 9.cxd5 Nh5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3pPnp1/2p5/2P1PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 8' => {
			eco => 'E76r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. d5 e6 8. dxe6

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 e6 8.dxe6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3pPnp1/2p5/2P1PP2/2N5/PP2B1PP/R1BQK1NR b KQ - 0 8' => {
			eco => 'E77',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5'),
			moves => {
				'f7e6' => 'rnbq1rk1/pp4bp/3ppnp1/2p5/2P1PP2/2N5/PP2B1PP/R1BQK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 1 8' => {
			eco => 'E75b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Nf3

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Nf3'),
			moves => {
				'e6d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pp2B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9',
				'h7h6' => 'rnbq1rk1/pp3pb1/3ppnpp/2pP2B1/2P1P3/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N2P2/PP1Q2PP/R3KBNR b KQ - 1 8' => {
			eco => 'E81i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 c5 7. d5 e6 8. Qd2

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 c5 7. d5 e6 8. Qd2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N2P2/PP4PP/R2QKBNR w KQ - 0 8' => {
			eco => 'E81i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5 c5 7. d5 e6

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5 c5 7. d5 e6'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N2P2/PP1Q2PP/R3KBNR b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N4P/PP3PP1/R2QKBNR w KQ - 0 8' => {
			eco => 'E71o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 c5 7. d5 e6

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 c5 7. d5 e6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 1 8' => {
			eco => 'E75f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2'),
			moves => {
				'e6d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pp2B1/2P1P3/2N5/PP1QBPPP/R3K1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N5/PP2BPPP/R2QK1NR w KQ - 0 8' => {
			eco => 'E75a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6'),
			moves => {
				'd1d2' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N5/PP1QBPPP/R3K1NR b KQ - 1 8',
				'g1f3' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP2B1/2P1P3/2N2N2/PP2BPPP/R2QK2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 0 8' => {
			eco => 'E90v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQK2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 8' => {
			eco => 'E91h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. d5 e6 8. O-O

			variation => N__('King\'s Indian: 6. Be2 c5 7. d5 e6 8.O-O'),
			moves => {
				'f8e8' => 'rnbqr1k1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 9',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 8' => {
			eco => 'E91h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. d5 e6

			variation => N__('King\'s Indian: 6. Be2 c5 7. d5 e6'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQK2R b KQ - 1 8' => {
			eco => 'E90v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3'),
			moves => {
				'b8a6' => 'r1bq1rk1/pp3pbp/n2ppnp1/2pP4/2P1P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 2 9',
				'e6d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pp4/2P1P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2NB4/PP2NPPP/R1BQ1RK1 b - - 1 8' => {
			eco => 'E70p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 c5 7. d5 e6 8. O-O

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 c5 7. d5 e6 8.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2NB4/PP2NPPP/R1BQK2R w KQ - 0 8' => {
			eco => 'E70p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 c5 7. d5 e6

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 c5 7. d5 e6'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1P3/2NB4/PP2NPPP/R1BQ1RK1 b - - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1PP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 1 8' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2 c5 7. d5 e6 8. Nf3

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5 e6 8. Nf3'),
			moves => {
				'e6d5' => 'rnbq1rk1/pp3pbp/3p1np1/2pp4/2P1PP2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 0 8' => {
			eco => 'E76r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3 c5 7. d5 e6

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Nf3 c5 7. d5 e6'),
			moves => {
				'd5e6' => 'rnbq1rk1/pp3pbp/3pPnp1/2p5/2P1PP2/2N2N2/PP4PP/R1BQKB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1PP2/2N5/PP2B1PP/R1BQK1NR w KQ - 0 8' => {
			eco => 'E77',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5'),
			moves => {
				'd5e6' => 'rnbq1rk1/pp3pbp/3pPnp1/2p5/2P1PP2/2N5/PP2B1PP/R1BQK1NR b KQ - 0 8',
				'g1f3' => 'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P1PP2/2N2N2/PP2B1PP/R1BQK2R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/2P5/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 8' => {
			eco => 'E64d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2 c5 7. d5 e6

			variation => N__('King\'s Indian: Fianchetto, Yugoslav, 7. d5 e6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/3ppnp1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A43x',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Be2 O-O 7. O-O e6

			variation => N__('Old Benoni: Schmidt, 6. Be2 O-O 7.O-O e6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/4pnp1/3p4/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 9' => {
			eco => 'D79c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Nc3 e6

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Nc3 e6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3pbp/4pnp1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 9' => {
			eco => 'D79j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6 9. Nc3

			variation => N__('Neo-Grünfeld, Main Line'),
			moves => {
				'f6d7' => 'rnbq1rk1/pp1n1pbp/4p1p1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 w - - 2 10',
			},
		},
		'rnbq1rk1/pp3pbp/4pnp1/3pN3/3P4/6P1/PP2PPBP/RNBQ1RK1 w - - 0 9' => {
			eco => 'D79i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O c6 7. cxd5 cxd5 8. Ne5 e6

			variation => N__('Neo-Grünfeld, 6.O-O c6 7.cxd5 cxd5 8. Ne5 e6'),
			moves => {
				'b1c3' => 'rnbq1rk1/pp3pbp/4pnp1/3pN3/3P4/2N3P1/PP2PPBP/R1BQ1RK1 b - - 1 9',
			},
		},
		'rnbq1rk1/pp3pp1/2p1pb1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9' => {
			eco => 'D55j',
                        # TRANSLATORS:

			variation => N__('QGD: Neo-Orthodox, Gligoric Variation'),
			moves => {
				'f1d3' => 'rnbq1rk1/pp3pp1/2p1pb1p/3p4/2PP4/2NBPN2/PP3PPP/2RQK2R b K - 1 9',
			},
		},
		'rnbq1rk1/pp3pp1/2p1pb1p/3p4/2PP4/2NBPN2/PP3PPP/2RQK2R b K - 1 9' => {
			eco => 'D55j',
                        # TRANSLATORS:

			variation => N__('QGD: Neo-Orthodox, Gligoric Variation'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1n1pp1/2p1pb1p/3p4/2PP4/2NBPN2/PP3PPP/2RQK2R w K - 2 10',
			},
		},
		'rnbq1rk1/pp3pp1/3ppn1p/2pP4/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQ - 1 8' => {
			eco => 'E31g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 O-O

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/2pb4/3p4/2PPn3/3B1N2/PP3PPP/RNBQ1RK1 w - - 0 9' => {
			eco => 'C42r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Bd6 7. O-O O-O 8. c4 c6

			variation => N__('Russian Game: Classical, Marshall, 8. c4 c6'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4p3/1B6/3PP3/5N2/P2Q1PPP/R3K2R b KQ - 2 11' => {
			eco => 'D41m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+ 9. Bd2 Bxd2+ 10. Qxd2 O-O 11. Bb5

			variation => N__('QGD: Semi-Tarrasch, 6. e4, Kmoch Variation'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4p3/2pn4/3P4/P1P1PP2/6PP/R1BQKBNR w KQ - 0 9' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. e3 O-O 8. cxd5 Nxd5

			variation => N__('Nimzo-Indian: Sämisch, Botvinnik Variation'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4p3/3n4/1b1P4/2NB4/PP2NPPP/R1BQ1RK1 b - - 1 9' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7. cxd5 cxd4 8. exd4 Nxd5 9. O-O

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.cxd5 cxd4 8.exd4 Nxd5 9.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4p3/3n4/1b1P4/2NB4/PP2NPPP/R1BQK2R w KQ - 0 9' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7. cxd5 cxd4 8. exd4 Nxd5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.cxd5 cxd4 8.exd4 Nxd5'),
			moves => {
				'e1g1' => 'rnbq1rk1/pp3ppp/4p3/3n4/1b1P4/2NB4/PP2NPPP/R1BQ1RK1 b - - 1 9',
			},
		},
		'rnbq1rk1/pp3ppp/4p3/8/2BPP3/5N2/P2Q1PPP/R3K2R b KQ - 2 11' => {
			eco => 'D41n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+ 9. Bd2 Bxd2+ 10. Qxd2 O-O 11. Bc4

			variation => N__('QGD: Semi-Tarrasch, 6. e4, Main Line'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp3ppp/2n1p3/8/2BPP3/5N2/P2Q1PPP/R3K2R w KQ - 3 12',
			},
		},
		'rnbq1rk1/pp3ppp/4p3/8/3PP3/5N2/P2Q1PPP/R3KB1R w KQ - 1 11' => {
			eco => 'D41m',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4 Bb4+'),
			moves => {
				'f1b5' => 'rnbq1rk1/pp3ppp/4p3/1B6/3PP3/5N2/P2Q1PPP/R3K2R b KQ - 2 11',
				'f1c4' => 'rnbq1rk1/pp3ppp/4p3/8/2BPP3/5N2/P2Q1PPP/R3K2R b KQ - 2 11',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2bP4/5B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8' => {
			eco => 'D37q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. cxd5

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8.cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2bp4/2P2B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'D37o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5'),
			moves => {
				'a2a3' => 'rnbq1rk1/pp3ppp/4pn2/2bp4/2P2B2/P1N1PN2/1P3PPP/R2QKB1R b KQ - 0 8',
				'c4d5' => 'rnbq1rk1/pp3ppp/4pn2/2bP4/5B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8',
				'd1c2' => 'rnbq1rk1/pp3ppp/4pn2/2bp4/2P2B2/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2bp4/2P2B2/2N1PN2/PPQ2PPP/R3KB1R b KQ - 1 8' => {
			eco => 'D37r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. Qc2

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. Qc2'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp3ppp/2n1pn2/2bp4/2P2B2/2N1PN2/PPQ2PPP/R3KB1R w KQ - 2 9',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2bp4/2P2B2/P1N1PN2/1P3PPP/R2QKB1R b KQ - 0 8' => {
			eco => 'D37p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O 6. e3 c5 7. dxc5 Bxc5 8. a3

			variation => N__('QGD: Classical, 5...O-O 6. e3 c5 7.dxc5 Bxc5 8. a3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4 8.Bxc4'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp3ppp/2n1pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 1 9',
				'b8d7' => 'r1bq1rk1/pp1n1ppp/4pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 1 9',
				'c5d4' => 'rnbq1rk1/pp3ppp/4pn2/8/1bBp4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 0 9',
				'd8e7' => 'rnb2rk1/pp2qppp/4pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2p5/1bpP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4'),
			moves => {
				'd3c4' => 'rnbq1rk1/pp3ppp/4pn2/2p5/1bBP4/2N1PN2/PP3PPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2p5/2BP4/P1P1P3/4NPPP/R1BQK2R b KQ - 1 9' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 dxc4 8. Bxc4 c5 9. Ne2

			variation => N__('Nimzo-Indian: Botvinnik, 7...dxc4 8.Bxc4 c5 9. Ne2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2p5/2BP4/P1P1P3/5PPP/R1BQK1NR w KQ c6 0 9' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 dxc4 8. Bxc4 c5

			variation => N__('Nimzo-Indian: Botvinnik, 7...dxc4 8.Bxc4 c5'),
			moves => {
				'g1e2' => 'rnbq1rk1/pp3ppp/4pn2/2p5/2BP4/P1P1P3/4NPPP/R1BQK2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pP4/1b1P4/2NBP3/PP2NPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7. cxd5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.cxd5'),
			moves => {
				'c5d4' => 'rnbq1rk1/pp3ppp/4pn2/3P4/1b1p4/2NBP3/PP2NPPP/R1BQK2R w KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pP4/3P4/P1P1PP2/6PP/R1BQKBNR b KQ - 0 8' => {
			eco => 'E24',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5'),
			moves => {
				'f6d5' => 'rnbq1rk1/pp3ppp/4p3/2pn4/3P4/P1P1PP2/6PP/R1BQKBNR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pP4/3P4/P1PBP3/5PPP/R1BQK1NR b KQ - 0 8' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 c5 8. cxd5

			variation => N__('Nimzo-Indian: Botvinnik, 7... c5 8.cxd5'),
			moves => {
				'e6d5' => 'rnbq1rk1/pp3ppp/5n2/2pp4/3P4/P1PBP3/5PPP/R1BQK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBP3/PP2NPPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7. O-O

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.O-O'),
			moves => {
				'c5d4' => 'rnbq1rk1/pp3ppp/4pn2/3p4/1bPp4/2NBP3/PP2NPPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBP3/PP2NPPP/R1BQK2R w KQ c6 0 7' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3ppp/4pn2/2pP4/1b1P4/2NBP3/PP2NPPP/R1BQK2R b KQ - 0 7',
				'e1g1' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBP3/PP2NPPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O

			variation => N__('Nimzo-Indian: Main Line, 6... c5 7.O-O'),
			moves => {
				'b7b6' => 'rnbq1rk1/p4ppp/1p2pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8',
				'b8c6' => 'r1bq1rk1/pp3ppp/2n1pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8',
				'b8d7' => 'r1bq1rk1/pp1n1ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8',
				'c8d7' => 'rn1q1rk1/pp1b1ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 2 8',
				'd5c4' => 'rnbq1rk1/pp3ppp/4pn2/2p5/1bpP4/2NBPN2/PP3PPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ c6 0 7' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5

			variation => N__('Nimzo-Indian: Main Line, 6... c5'),
			moves => {
				'a2a3' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/P1NBPN2/1P3PPP/R1BQK2R b KQ - 0 7',
				'e1g1' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/P1NBPN2/1P3PPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. a3

			variation => N__('Nimzo-Indian: Main Line, 6... c5 7. a3'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/2PP4/P1P1PP2/6PP/R1BQKBNR w KQ - 1 8' => {
			eco => 'E24',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3ppp/4pn2/2pP4/3P4/P1P1PP2/6PP/R1BQKBNR b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/2pp4/2PP4/P1PBP3/5PPP/R1BQK1NR w KQ c6 0 8' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 c5

			variation => N__('Nimzo-Indian: Botvinnik, 7... c5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3ppp/4pn2/2pP4/3P4/P1PBP3/5PPP/R1BQK1NR b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/3P4/1b1N4/2N3P1/PP2PPBP/R1BQK2R b KQ - 0 8' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 cxd4 6. Nxd4 O-O 7. Bg2 d5 8. cxd5

			variation => N__('Nimzo-Indian: Romanishin, 8.cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/3P4/1b1P4/2NB4/PP2NPPP/R1BQK2R b KQ - 0 8' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7. cxd5 cxd4 8. exd4

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.cxd5 cxd4 8.exd4'),
			moves => {
				'f6d5' => 'rnbq1rk1/pp3ppp/4p3/3n4/1b1P4/2NB4/PP2NPPP/R1BQK2R w KQ - 0 9',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/3P4/1b1p4/2NBP3/PP2NPPP/R1BQK2R w KQ - 0 8' => {
			eco => 'E48',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.cxd5'),
			moves => {
				'e3d4' => 'rnbq1rk1/pp3ppp/4pn2/3P4/1b1P4/2NB4/PP2NPPP/R1BQK2R b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/3p4/1bPN4/2N3P1/PP2PPBP/R1BQK2R w KQ d6 0 8' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 cxd4 6. Nxd4 O-O 7. Bg2 d5

			variation => N__('Nimzo-Indian: Romanishin, 6.Nxd4 O-O 7. Bg2 d5'),
			moves => {
				'c4d5' => 'rnbq1rk1/pp3ppp/4pn2/3P4/1b1N4/2N3P1/PP2PPBP/R1BQK2R b KQ - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/3p4/1bPP4/2NB4/PP2NPPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7. O-O cxd4 8. exd4

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.O-O cxd4 8.exd4'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/3p4/1bPp4/2NBP3/PP2NPPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E48',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2 c5 7.O-O'),
			moves => {
				'e3d4' => 'rnbq1rk1/pp3ppp/4pn2/3p4/1bPP4/2NB4/PP2NPPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/8/1bBP4/2N2N2/PP3PPP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4 9. exd4

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4 8.Bxc4 cxd4 9.exd4'),
			moves => {
				'a7a6' => 'rnbq1rk1/1p3ppp/p3pn2/8/1bBP4/2N2N2/PP3PPP/R1BQ1RK1 w - - 0 10',
				'b7b6' => 'rnbq1rk1/p4ppp/1p2pn2/8/1bBP4/2N2N2/PP3PPP/R1BQ1RK1 w - - 0 10',
			},
		},
		'rnbq1rk1/pp3ppp/4pn2/8/1bBp4/2N1PN2/PP3PPP/R1BQ1RK1 w - - 0 9' => {
			eco => 'E54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3 c5 7. O-O dxc4 8. Bxc4 cxd4

			variation => N__('Nimzo-Indian: Main Line, 7...dxc4 8.Bxc4 cxd4'),
			moves => {
				'e3d4' => 'rnbq1rk1/pp3ppp/4pn2/8/1bBP4/2N2N2/PP3PPP/R1BQ1RK1 b - - 0 9',
			},
		},
		'rnbq1rk1/pp3ppp/5n2/2pp4/3P4/P1PBP3/4NPPP/R1BQK2R b KQ - 1 9' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 c5 8. cxd5 exd5 9. Ne2

			variation => N__('Nimzo-Indian: Botvinnik, 7... c5 8.cxd5 exd5 9. Ne2'),
			moves => {
			},
		},
		'rnbq1rk1/pp3ppp/5n2/2pp4/3P4/P1PBP3/5PPP/R1BQK1NR w KQ - 0 9' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 c5 8. cxd5 exd5

			variation => N__('Nimzo-Indian: Botvinnik, 7... c5 8.cxd5 exd5'),
			moves => {
				'g1e2' => 'rnbq1rk1/pp3ppp/5n2/2pp4/3P4/P1PBP3/4NPPP/R1BQK2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/pp4bp/2pp1np1/3Ppp2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - e6 0 9' => {
			eco => 'A88',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 c6 8. d5 e5

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 e5'),
			moves => {
				'd5e6' => 'rnbq1rk1/pp4bp/2ppPnp1/5p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 9',
			},
		},
		'rnbq1rk1/pp4bp/2ppPnp1/5p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 0 9' => {
			eco => 'A88',
                        # TRANSLATORS:

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 c6 8. d5 e5'),
			moves => {
				'c8e6' => 'rn1q1rk1/pp4bp/2ppbnp1/5p2/2P5/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 10',
			},
		},
		'rnbq1rk1/pp4bp/3ppnp1/2p5/2P1PP2/2N5/PP2B1PP/R1BQK1NR w KQ - 0 9' => {
			eco => 'E77',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5'),
			moves => {
				'g2g4' => 'rnbq1rk1/pp4bp/3ppnp1/2p5/2P1PPP1/2N5/PP2B2P/R1BQK1NR b KQ g3 0 9',
			},
		},
		'rnbq1rk1/pp4bp/3ppnp1/2p5/2P1PPP1/2N5/PP2B2P/R1BQK1NR b KQ g3 0 9' => {
			eco => 'E77',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2 c5 7. d5'),
			moves => {
				'b8c6' => 'r1bq1rk1/pp4bp/2nppnp1/2p5/2P1PPP1/2N5/PP2B2P/R1BQK1NR w KQ - 1 10',
			},
		},
		'rnbq1rk1/pp4pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R2Q1RK1 b - - 0 9' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Bxa3 9. Nxa3

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Bxa3 9.Nxa3'),
			moves => {
				'b8d7' => 'r1bq1rk1/pp1n2pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R2Q1RK1 w - - 1 10',
				'd8e7' => 'rnb2rk1/pp2q1pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R2Q1RK1 w - - 1 10',
			},
		},
		'rnbq1rk1/pp4pp/2p1pn2/3p1p2/2PP4/bP3NP1/P3PPBP/RN1Q1RK1 w - - 0 9' => {
			eco => 'A94',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3 c6 8. Ba3 Bxa3

			variation => N__('Dutch: Stonewall, Botvinnik, 8. Ba3 Bxa3'),
			moves => {
				'b1a3' => 'rnbq1rk1/pp4pp/2p1pn2/3p1p2/2PP4/NP3NP1/P3PPBP/R2Q1RK1 b - - 0 9',
			},
		},
		'rnbq1rk1/ppp1Bpp1/4p2p/3p4/2PPn3/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8' => {
			eco => 'D56',
                        # TRANSLATORS:

			variation => N__('QGD: Lasker Defence'),
			moves => {
				'd8e7' => 'rnb2rk1/ppp1qpp1/4p2p/3p4/2PPn3/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 9',
			},
		},
		'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. b3

			variation => N__('Dutch: Classical, 7. b3'),
			moves => {
				'a7a5' => 'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - a6 0 8',
				'd8e8' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8',
			},
		},
		'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6 7. Nc3

			variation => N__('Dutch: Classical, 7. Nc3'),
			moves => {
				'a7a5' => 'rnbq1rk1/1pp1b1pp/3ppn2/p4p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - a6 0 8',
				'd8e8' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. Nc3 d6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. Nc3 d6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/2N3PN/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6. O-O d6 7. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6.O-O d6 7. Nc3'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2b1pp/2pppn2/5p2/2PP4/2N3PN/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'd8e8' => 'rnb1qrk1/ppp1b1pp/3ppn2/5p2/2PP4/2N3PN/PP2PPBP/R1BQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A96',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d6

			variation => N__('Dutch: Classical'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
				'b2b3' => 'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/6PN/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6. O-O d6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6.O-O d6'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/2N3PN/PP2PPBP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'A93',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. b3

			variation => N__('Dutch: Stonewall, Botvinnik Variation'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1b1pp/1p2pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8',
				'b8c6' => 'r1bq1rk1/ppp1b1pp/2n1pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 1 8',
				'c7c6' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nc3

			variation => N__('Dutch: Stonewall, 7. Nc3'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Nbd2

			variation => N__('Dutch: Stonewall, 7.Nbd2'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - d6 0 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5

			variation => N__('Dutch: Stonewall Variation'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
				'b1d2' => 'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP1NPPBP/R1BQ1RK1 b - - 1 7',
				'b2b3' => 'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
				'd1c2' => 'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 1 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O d5 7. Qc2

			variation => N__('Dutch: Stonewall, 7. Qc2'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1bpp1/4p2p/3p4/2PPn2B/2N1PN2/PP3PPP/R2QKB1R w KQ - 2 8' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4

			variation => N__('QGD: Lasker Defence'),
			moves => {
				'h4e7' => 'rnbq1rk1/ppp1Bpp1/4p2p/3p4/2PPn3/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pB1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7' => {
			eco => 'D55h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bxf6

			variation => N__('QGD: Neo-Orthodox, 7.Bxf6'),
			moves => {
				'e7f6' => 'rnbq1rk1/ppp2pp1/4pb1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p2B1/2PP4/2N1P3/PP3PPP/2RQKBNR w K - 0 7' => {
			eco => 'D54a',
                        # TRANSLATORS:

			variation => N__('QGD: Anti-Neo-Orthodox Variation'),
			moves => {
				'g5h4' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N1P3/PP3PPP/2RQKBNR b K - 1 7',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'D55g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6

			variation => N__('QGD: Neo-Orthodox Variation'),
			moves => {
				'g5f6' => 'rnbq1rk1/ppp1bpp1/4pB1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 7',
				'g5h4' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N1P3/PP3PPP/2RQKBNR b K - 1 7' => {
			eco => 'D54b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Rc1 h6 7. Bh4

			variation => N__('QGD: Anti-Neo-Orthodox Variation'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1P3/PP3PPP/2RQKBNR w K - 0 8',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R b KQ - 1 7' => {
			eco => 'D56',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4

			variation => N__('QGD: Neo-Orthodox, 7. Bh4'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1bpp1/1p2pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8',
				'f6e4' => 'rnbq1rk1/ppp1bpp1/4p2p/3p4/2PPn2B/2N1PN2/PP3PPP/R2QKB1R w KQ - 2 8',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PP2PPPP/2RQKB1R b K - 3 7' => {
			eco => 'D53k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bh4 O-O 7. Rc1

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6. Bh4 O-O 7. Rc1'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQ - 2 7' => {
			eco => 'D53i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bh4 O-O

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6. Bh4 O-O'),
			moves => {
				'a1c1' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PP2PPPP/2RQKB1R b K - 3 7',
				'd1c2' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PPQ1PPPP/R3KB1R b KQ - 3 7',
			},
		},
		'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PPQ1PPPP/R3KB1R b KQ - 3 7' => {
			eco => 'D53j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bh4 O-O 7. Qc2

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6. Bh4 O-O 7. Qc2'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/3p1n2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b - - 6 7' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2bppp/3p1n2/2p5/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w - c6 0 8',
			},
		},
		'rnbq1rk1/ppp1bppp/3p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQ - 5 7' => {
			eco => 'C41m',
                        # TRANSLATORS:

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp1bppp/3p1n2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b - - 6 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4p3/3p2B1/2PPn3/2N1PN2/PP3PPP/R2QKB1R w KQ - 3 7' => {
			eco => 'D55b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 Ne4

			variation => N__('QGD: Lasker without ...h6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3P4/3P4/P1N1P3/1P2NPPP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 d5 6. a3 Be7 7. cxd5

			variation => N__('Nimzo-Indian: Reshevsky, 6. a3 Be7 7.cxd5'),
			moves => {
				'e6d5' => 'rnbq1rk1/ppp1bppp/5n2/3p4/3P4/P1N1P3/1P2NPPP/R1BQKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/2RQKBNR b K - 2 6' => {
			eco => 'D54a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Rc1

			variation => N__('QGD: Anti-Neo-Orthodox Variation'),
			moves => {
				'h7h6' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p2B1/2PP4/2N1P3/PP3PPP/2RQKBNR w K - 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQ - 1 6' => {
			eco => 'D53u',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O

			variation => N__('QGD: 4. Bg5 Be7 5. e3 O-O'),
			moves => {
				'a1c1' => 'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/2RQKBNR b K - 2 6',
				'd1c2' => 'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PPQ2PPP/R3KBNR b KQ - 2 6',
				'g1f3' => 'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PPQ2PPP/R3KBNR b KQ - 2 6' => {
			eco => 'D53v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Qc2

			variation => N__('QGD: 4. Bg5 Be7 5. e3 O-O 6. Qc2'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2bppp/4pn2/2pp2B1/2PP4/2N1P3/PPQ2PPP/R3KBNR w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 2 6' => {
			eco => 'D55a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3

			variation => N__('QGD: 6. Nf3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1bppp/1p2pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7',
				'b8d7' => 'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 3 7',
				'f6e4' => 'rnbq1rk1/ppp1bppp/4p3/3p2B1/2PPn3/2N1PN2/PP3PPP/R2QKB1R w KQ - 3 7',
				'h7h6' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQ - 6 6' => {
			eco => 'D53l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 O-O

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 O-O'),
			moves => {
				'd1c2' => 'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N2N2/PPQ1PPPP/R3KB1R b KQ - 7 6',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N2N2/PPQ1PPPP/R3KB1R b KQ - 7 6' => {
			eco => 'D53m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 O-O 6. Qc2

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 O-O 6. Qc2'),
			moves => {
				'b8d7' => 'r1bq1rk1/pppnbppp/4pn2/3p2B1/2PP4/2N2N2/PPQ1PPPP/R3KB1R w KQ - 8 7',
				'c7c5' => 'rnbq1rk1/pp2bppp/4pn2/2pp2B1/2PP4/2N2N2/PPQ1PPPP/R3KB1R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 b - - 0 6' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O 6. b3

			variation => N__('English: Neo-Catalan Declined, 5...O-O 6. b3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1bppp/1p2pn2/3p4/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 w - - 0 7',
				'c7c5' => 'rnbq1rk1/pp2bppp/4pn2/2pp4/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 w - c6 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w - - 5 6' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O O-O

			variation => N__('English: Neo-Catalan Declined, 5...O-O'),
			moves => {
				'b2b3' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2P5/1P3NP1/P2PPPBP/RNBQ1RK1 b - - 0 6',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6' => {
			eco => 'D37i',
                        # TRANSLATORS:

			variation => N__('QGD: Classical, 5...O-O'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1bppp/1p2pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7',
				'b8d7' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7',
				'c7c5' => 'rnbq1rk1/pp2bppp/4pn2/2pp4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQ - 6 6' => {
			eco => 'D37i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4 O-O

			variation => N__('QGD: Classical, 5...O-O'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/1PN1PN2/P4PPP/R1BQKB1R b KQ - 0 6' => {
			eco => 'D37f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. e3 O-O 6. b3

			variation => N__('QGD: 4. Nf3 Be7 5. e3 O-O 6. b3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 1 6' => {
			eco => 'D37f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. e3 O-O

			variation => N__('QGD: 4. Nf3 Be7 5. e3 O-O'),
			moves => {
				'b2b3' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/1PN1PN2/P4PPP/R1BQKB1R b KQ - 0 6',
				'f1d3' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 2 6' => {
			eco => 'D37g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. e3 O-O 6. Bd3

			variation => N__('QGD: 4. Nf3 Be7 5. e3 O-O 6. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1Q1RK1 b - - 3 7' => {
			eco => 'E11h',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1Q1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1QK2R w KQ - 2 7' => {
			eco => 'E11h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1Q1RK1 b - - 3 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. O-O

			variation => N__('Catalan: Closed, 6.O-O'),
			moves => {
				'b8d7' => 'r1bq1rk1/pppnbppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 6 7',
				'c7c5' => 'rnbq1rk1/pp2bppp/4pn2/2pp4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2bppp/2p1pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O

			variation => N__('Catalan: Closed, 5. Nf3 O-O'),
			moves => {
				'd1c2' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB1K2R b KQ - 5 6',
				'e1g1' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PPQ1PPBP/RNB1K2R b KQ - 5 6' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3 O-O 6. Qc2

			variation => N__('Catalan: Closed, 6. Qc2'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/P1N1P3/1P2NPPP/R1BQKB1R w KQ - 1 7' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 d5 6. a3 Be7

			variation => N__('Nimzo-Indian: Reshevsky, 6. a3 Be7'),
			moves => {
				'c4d5' => 'rnbq1rk1/ppp1bppp/4pn2/3P4/3P4/P1N1P3/1P2NPPP/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/3p4/2PPP3/P4N2/1P1N1PPP/R1BQKB1R w KQ d6 0 7' => {
			eco => 'E11f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 O-O 5. a3 Be7 6. e4 d5

			variation => N__('Bogo-Indian: Grünfeld Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/4N3/2pP4/2N3P1/PP2PPBP/R1BQK2R b KQ - 5 7' => {
			eco => 'E05d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. Nc3 O-O 7. Ne5

			variation => N__('Catalan: Open, Classical, 6. Nc3 O-O 7. Ne5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/4N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 b - - 5 7' => {
			eco => 'E05g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Ne5

			variation => N__('Catalan: Open, Classical, 7. Ne5'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1bppp/2n1pn2/4N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 w - - 6 8',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 4 7' => {
			eco => 'E05c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. Nc3 O-O

			variation => N__('Catalan: Open, Classical, 6. Nc3 O-O'),
			moves => {
				'f3e5' => 'rnbq1rk1/ppp1bppp/4pn2/4N3/2pP4/2N3P1/PP2PPBP/R1BQK2R b KQ - 5 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 4 7' => {
			eco => 'E05f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O

			variation => N__('Catalan: Open, Classical, 6.O-O O-O'),
			moves => {
				'd1c2' => 'rnbq1rk1/ppp1bppp/4pn2/8/2pP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 5 7',
				'f3e5' => 'rnbq1rk1/ppp1bppp/4pn2/4N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 b - - 5 7',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/8/2pP4/5NP1/PPQ1PPBP/RNB2RK1 b - - 5 7' => {
			eco => 'E05i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O O-O 7. Qc2

			variation => N__('Catalan: Open, Classical, 7. Qc2'),
			moves => {
				'a7a6' => 'rnbq1rk1/1pp1bppp/p3pn2/8/2pP4/5NP1/PPQ1PPBP/RNB2RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/8/3PN3/P5N1/1PP1BPPP/R1BQK2R b KQ - 4 8' => {
			eco => 'C15u',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7 6.Nxe4 Nf6'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1bppp/2n1pn2/8/3PN3/P5N1/1PP1BPPP/R1BQK2R w KQ - 5 9',
			},
		},
		'rnbq1rk1/ppp1bppp/4pn2/8/3PN3/P5N1/1PP2PPP/R1BQKB1R w KQ - 3 8' => {
			eco => 'C15u',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7 6.Nxe4 Nf6'),
			moves => {
				'f1e2' => 'rnbq1rk1/ppp1bppp/4pn2/8/3PN3/P5N1/1PP1BPPP/R1BQK2R b KQ - 4 8',
			},
		},
		'rnbq1rk1/ppp1bppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQ - 1 7' => {
			eco => 'D35k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Be7 6. e3 O-O

			variation => N__('QGD: Exchange, 5. Bg5 Be7 6. e3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/5n2/3p4/3P4/P1N1P3/1P2NPPP/R1BQKB1R w KQ - 0 8' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 d5 6. a3 Be7 7. cxd5 exd5

			variation => N__('Nimzo-Indian: Reshevsky, 6. a3 Be7 7.cxd5 exd5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1bppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 w - - 4 8' => {
			eco => 'C42s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7 7. O-O O-O

			variation => N__('Russian Game: Classical, Mason Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1p1bp/3p1np1/3P1p2/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. d5

			variation => N__('Dutch: Leningrad, Main Line, 7. d5'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2p1bp/3p1np1/2pP1p2/2P5/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 8',
				'c7c6' => 'rnbq1rk1/pp2p1bp/2pp1np1/3P1p2/2P5/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. b3

			variation => N__('Dutch: Leningrad, Main Line, 7. b3'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1p1bp/n2p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'b8c6' => 'r1bq1rk1/ppp1p1bp/2np1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'c7c6' => 'rnbq1rk1/pp2p1bp/2pp1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
				'd8e8' => 'rnb1qrk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'e7e6' => 'rnbq1rk1/ppp3bp/3ppnp1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 7' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. Nc3 d6

			variation => N__('Dutch: Leningrad, Main Line, 6. Nc3 d6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6

			variation => N__('Dutch: Leningrad, Main Line, 6... d6'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
				'b2b3' => 'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b - - 0 7',
				'd4d5' => 'rnbq1rk1/ppp1p1bp/3p1np1/3P1p2/2P5/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppb1/3p1npp/6B1/2PPP3/2N2N2/PP3PPP/R2QKB1R w KQ - 0 7' => {
			eco => 'E90g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Bg5 h6

			variation => N__('King\'s Indian: Zinnowitz, 6... h6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppb1/3p1npp/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 0 7' => {
			eco => 'E71i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5 h6

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5 h6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppb1/3p1npp/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 0 7' => {
			eco => 'E73v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6

			variation => N__('King\'s Indian: Averbakh, 6... h6'),
			moves => {
				'g5e3' => 'rnbq1rk1/ppp1ppb1/3p1npp/8/2PPP3/2N1B3/PP2BPPP/R2QK1NR b KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp1ppb1/3p1npp/8/2PPP3/2N1B3/PP2BPPP/R2QK1NR b KQ - 1 7' => {
			eco => 'E73w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6 7. Be3

			variation => N__('King\'s Indian: Averbakh, 6... h6 7. Be3'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppb1/3p1npp/2p5/2PPP3/2N1B3/PP2BPPP/R2QK1NR w KQ c6 0 8',
				'e7e5' => 'rnbq1rk1/ppp2pb1/3p1npp/4p3/2PPP3/2N1B3/PP2BPPP/R2QK1NR w KQ e6 0 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/1n4p1/8/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 3 8' => {
			eco => 'D76b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6 8. Nc3

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6 8. Nc3'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/1nn3p1/8/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 4 9',
			},
		},
		'rnbq1rk1/ppp1ppbp/1n4p1/8/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 8' => {
			eco => 'D76a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O Nb6

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O Nb6'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1ppbp/1n4p1/8/3P4/2N2NP1/PP2PPBP/R1BQ1RK1 b - - 3 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/1n4p1/8/3PP3/2N1BP2/PP4PP/R2QKBNR w KQ - 5 8' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. f3 d5 4. cxd5 Nxd5 5. e4 Nb6 6. Nc3 Bg7 7. Be3 O-O

			variation => N__('Neo-Grünfeld: Alekhine\'s, 7. Be3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/1n4p1/8/3PP3/6P1/PP2NPBP/RNBQ1RK1 b - - 4 8' => {
			eco => 'D72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6 7. Ne2 O-O 8. O-O

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6 7. Ne2 O-O 8.O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/1n4p1/8/3PP3/6P1/PP2NPBP/RNBQK2R w KQ - 3 8' => {
			eco => 'D72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6 7. Ne2 O-O

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6 7. Ne2 O-O'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp1ppbp/1n4p1/8/3PP3/6P1/PP2NPBP/RNBQ1RK1 b - - 4 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/4P3/3P1P2/2N2N2/PPP3PP/R1BQKB1R b KQ - 0 6' => {
			eco => 'B09f',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. e5

			variation => N__('Pirc: Austrian, 5...O-O 6. e5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQ - 0 6' => {
			eco => 'E61e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Bg5 O-O 5. e3 d6

			variation => N__('King\'s Indian: 4. Bg5 O-O 5. e3 d6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6' => {
			eco => 'E61v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6 6. e3

			variation => N__('King\'s Indian: Smyslov System, 6. e3'),
			moves => {
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQ - 0 6' => {
			eco => 'E61u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5 d6

			variation => N__('King\'s Indian: Smyslov System, 5... d6'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N2N2/PP3PPP/R2QKB1R b KQ - 3 6' => {
			eco => 'E90g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Bg5

			variation => N__('King\'s Indian: Zinnowitz Variation'),
			moves => {
				'h7h6' => 'rnbq1rk1/ppp1ppb1/3p1npp/6B1/2PPP3/2N2N2/PP3PPP/R2QKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N2P2/PP4PP/R2QKBNR b KQ - 2 6' => {
			eco => 'E81e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Bg5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Bg5'),
			moves => {
				'a7a6' => 'rnbq1rk1/1pp1ppbp/p2p1np1/6B1/2PPP3/2N2P2/PP4PP/R2QKBNR w KQ - 0 7',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/6B1/2PPP3/2N2P2/PP4PP/R2QKBNR w KQ - 3 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PPP3/2N2P2/PP4PP/R2QKBNR w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR b KQ - 2 6' => {
			eco => 'E71g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Bg5

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Bg5'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 3 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 3 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 0 7',
				'h7h6' => 'rnbq1rk1/ppp1ppb1/3p1npp/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP1Q1PPP/R3KBNR b KQ - 3 6' => {
			eco => 'E70v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bg5 O-O 6. Qd2

			variation => N__('King\'s Indian: Accelerated Averbakh, 5...O-O 6. Qd2'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR b KQ - 3 6' => {
			eco => 'E73i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5

			variation => N__('King\'s Indian: Averbakh'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n2p1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 4 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 4 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 0 7',
				'h7h6' => 'rnbq1rk1/ppp1ppb1/3p1npp/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP3PPP/R2QKBNR w KQ - 2 6' => {
			eco => 'E70u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bg5 O-O

			variation => N__('King\'s Indian: Accelerated Averbakh, 5...O-O'),
			moves => {
				'd1d2' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP1Q1PPP/R3KBNR b KQ - 3 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/3P4/2P2N2/PP1NPPPP/R2QKB1R w KQ - 0 6' => {
			eco => 'A48v',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. c3 d6

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'e2e4' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/3PP3/2P2N2/PP1N1PPP/R2QKB1R b KQ e3 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/3PP3/2P2N2/PP1N1PPP/R2QKB1R b KQ e3 0 6' => {
			eco => 'A48w',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. c3 d6 6. e4

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p3B1/3PP3/2P2N2/PP1N1PPP/R2QKB1R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/6B1/3PP3/5N2/PPPN1PPP/R2QKB1R w KQ - 0 6' => {
			eco => 'A48t',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. e4 d6

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b - - 1 6' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3 Bg7 4. Bg2 O-O 5. Nf3 d6 6. O-O

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQ - 0 6' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b - - 1 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP1B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6' => {
			eco => 'E61r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bf4 d6 6. e3

			variation => N__('King\'s Indian: 4. Nf3 d6 5. Bf4 d6 6. e3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP1B2/2N2N1P/PP2PPP1/R2QKB1R b KQ - 0 6' => {
			eco => 'E61q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bf4 d6 6. h3

			variation => N__('King\'s Indian: 4. Nf3 d6 5. Bf4 d6 6. h3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQ - 0 6' => {
			eco => 'E61q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bf4 d6

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. Bf4 d6'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP1B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6',
				'h2h3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP1B2/2N2N1P/PP2PPP1/R2QKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/1P2PN2/PB3PPP/RN1QKB1R b KQ - 0 6' => {
			eco => 'E60h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3 d6 5. Bb2 O-O 6. e3

			variation => N__('King\'s Indian: b3 System'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/1P3N2/PB2PPPP/RN1QKB1R w KQ - 2 6' => {
			eco => 'E60h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3 d6 5. Bb2 O-O

			variation => N__('King\'s Indian: b3 System'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/1P2PN2/PB3PPP/RN1QKB1R b KQ - 0 6',
				'g2g3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/1P3NP1/PB2PP1P/RN1QKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/1P3NP1/PB2PP1P/RN1QKB1R b KQ - 0 6' => {
			eco => 'E60i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3 d6 5. Bb2 O-O 6. g3

			variation => N__('King\'s Indian: b3+g3 System'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/1P3NP1/PB2PP1P/RN1QKB1R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N1P1P1/PP3PBP/R1BQK1NR b KQ - 0 6' => {
			eco => 'E61h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3 O-O 5. Bg2 d6 6. e3

			variation => N__('King\'s Indian: Fianchetto, Flohr Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E61n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. e3 d6 6. Be2

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. e3 d6 6. Be2'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N1PN2/PP2BPPP/R1BQK2R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 0 6' => {
			eco => 'E61m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. e3 d6

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. e3 d6'),
			moves => {
				'f1e2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 1 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQ - 1 6' => {
			eco => 'E62a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O

			variation => N__('King\'s Indian: Fianchetto Variation'),
			moves => {
				'f1g2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 2 6' => {
			eco => 'E62b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3 O-O 6. Bg2

			variation => N__('King\'s Indian: Fianchetto Variation'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQ - 0 6' => {
			eco => 'E61g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3 O-O 5. Bg2 d6

			variation => N__('King\'s Indian: 4. g3 O-O 5. Bg2 d6'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N1P1P1/PP3PBP/R1BQK1NR b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 1 6' => {
			eco => 'E60k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6 6. O-O

			variation => N__('King\'s Indian: Fianchetto without Nc3'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 0 6' => {
			eco => 'E60k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2 d6

			variation => N__('King\'s Indian: Fianchetto without Nc3'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 1 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP1P1/2N5/PP2BP1P/R1BQK1NR b KQ g3 0 6' => {
			eco => 'E73g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. g4

			variation => N__('King\'s Indian: 5. Be2 O-O 6. g4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1B2P/PP3PP1/R2QKBNR b KQ - 2 6' => {
			eco => 'E71e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Be3

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Be3'),
			moves => {
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1B2P/PP3PP1/R2QKBNR w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1B3/PP2BPPP/R2QK1NR b KQ - 3 6' => {
			eco => 'E73h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Be3

			variation => N__('King\'s Indian: Semi-Averbakh System'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1BN2/PP3PPP/R2QKB1R b KQ - 3 6' => {
			eco => 'E90h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be3

			variation => N__('King\'s Indian: Larsen Variation'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BN2/PP3PPP/R2QKB1R w KQ c6 0 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BN2/PP3PPP/R2QKB1R w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR b KQ - 2 6' => {
			eco => 'E81j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6. Be3'),
			moves => {
				'a7a6' => 'rnbq1rk1/1pp1ppbp/p2p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 7',
				'b7b6' => 'rnbq1rk1/p1p1ppbp/1p1p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 7',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 3 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 3 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 6' => {
			eco => 'E90k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ c6 0 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQ - 3 6' => {
			eco => 'E91a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2

			variation => N__('King\'s Indian: 6. Be2'),
			moves => {
				'a7a5' => 'rnbq1rk1/1pp1ppbp/3p1np1/p7/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ a6 0 7',
				'a7a6' => 'rnbq1rk1/1pp1ppbp/p2p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 7',
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 7',
				'c8g4' => 'rn1q1rk1/ppp1ppbp/3p1np1/8/2PPP1b1/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQ - 2 6' => {
			eco => 'E90e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O

			variation => N__('King\'s Indian: 5. Nf3 O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1BN2/PP3PPP/R2QKB1R b KQ - 3 6',
				'c1g5' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N2N2/PP3PPP/R2QKB1R b KQ - 3 6',
				'f1d3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2NB1N2/PP3PPP/R1BQK2R b KQ - 3 6',
				'f1e2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2N2/PP2BPPP/R1BQK2R b KQ - 3 6',
				'h2h3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2P2/PP2N1PP/R1BQKB1R b KQ - 2 6' => {
			eco => 'E81b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Nge2

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6.Nge2'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N2P2/PP2N1PP/R1BQKB1R w KQ c6 0 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2P2/PP2N1PP/R1BQKB1R w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQ - 1 6' => {
			eco => 'E81a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O

			variation => N__('King\'s Indian: Sämisch, 5...O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR b KQ - 2 6',
				'c1g5' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N2P2/PP4PP/R2QKBNR b KQ - 2 6',
				'g1e2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2P2/PP2N1PP/R1BQKB1R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N3N1/PP3PPP/R1BQKB1R b KQ - 3 6' => {
			eco => 'E70h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nge2 O-O 6. Ng3

			variation => N__('King\'s Indian: Kramer, 5...O-O 6. Ng3'),
			moves => {
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N3N1/PP3PPP/R1BQKB1R w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N3P1/PP3P1P/R1BQKBNR w KQ - 1 6' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O

			variation => N__('King\'s Indian: 4. e4 d6 5. g3 O-O'),
			moves => {
				'f1g2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N3P1/PP3PBP/R1BQK1NR b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N3P1/PP3PBP/R1BQK1NR b KQ - 2 6' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2

			variation => N__('King\'s Indian: 4. e4 d6 5. g3 O-O 6. Bg2'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ - 3 7',
				'b8d7' => 'r1bq1rk1/pppnppbp/3p1np1/8/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ - 3 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ c6 0 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N4P/PP3PP1/R1BQKBNR w KQ - 1 6' => {
			eco => 'E71d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O

			variation => N__('King\'s Indian: Makagonov, 5...O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1B2P/PP3PP1/R2QKBNR b KQ - 2 6',
				'c1g5' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N4P/PP3PP1/R2QKBNR b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQ - 2 6' => {
			eco => 'E73f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O

			variation => N__('King\'s Indian: 5. Be2 O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N1B3/PP2BPPP/R2QK1NR b KQ - 3 6',
				'c1g5' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP2BPPP/R2QK1NR b KQ - 3 6',
				'g2g4' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP1P1/2N5/PP2BP1P/R1BQK1NR b KQ g3 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2NPPP/R1BQKB1R w KQ - 2 6' => {
			eco => 'E70g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nge2 O-O

			variation => N__('King\'s Indian: Kramer, 5...O-O'),
			moves => {
				'e2g3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N3N1/PP3PPP/R1BQKB1R b KQ - 3 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2NB1N2/PP3PPP/R1BQK2R b KQ - 3 6' => {
			eco => 'E90f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Bd3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2NB4/PP2NPPP/R1BQK2R b KQ - 3 6' => {
			eco => 'E70m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/2PPP3/2NB4/PP2NPPP/R1BQK2R w KQ - 4 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPP3/2NB4/PP2NPPP/R1BQK2R w KQ c6 0 7',
				'e7e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2NB4/PP2NPPP/R1BQK2R w KQ e6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2NB4/PP3PPP/R1BQK1NR w KQ - 2 6' => {
			eco => 'E70l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O'),
			moves => {
				'g1e2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2NB4/PP2NPPP/R1BQK2R b KQ - 3 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPPP2/2N2N2/PP4PP/R1BQKB1R b KQ - 2 6' => {
			eco => 'E76e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Nf3

			variation => N__('King\'s Indian: Four Pawns Attack, 5...O-O 6. Nf3'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/2PPPP2/2N2N2/PP4PP/R1BQKB1R w KQ - 3 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPPP2/2N2N2/PP4PP/R1BQKB1R w KQ c6 0 7',
				'c8g4' => 'rn1q1rk1/ppp1ppbp/3p1np1/8/2PPPPb1/2N2N2/PP4PP/R1BQKB1R w KQ - 3 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPPP2/2N5/PP2B1PP/R1BQK1NR b KQ - 2 6' => {
			eco => 'E77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O 6. Be2

			variation => N__('King\'s Indian: Four Pawns Attack, 6. Be2'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/3p1np1/2p5/2PPPP2/2N5/PP2B1PP/R1BQK1NR w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPPP2/2N5/PP4PP/R1BQKBNR w KQ - 1 6' => {
			eco => 'E76d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 O-O

			variation => N__('King\'s Indian: Four Pawns Attack, 5...O-O'),
			moves => {
				'f1e2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPPP2/2N5/PP2B1PP/R1BQK1NR b KQ - 2 6',
				'g1f3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPPP2/2N2N2/PP4PP/R1BQKB1R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3P1B2/4PN1P/PPP1BPP1/RN1QK2R b KQ - 0 6' => {
			eco => 'A48l',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. e3 O-O 5. Be2 d6 6. h3

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3P1B2/4PN2/PPP1BPPP/RN1QK2R w KQ - 0 6' => {
			eco => 'A48k',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. e3 O-O 5. Be2 d6

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
				'h2h3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3P1B2/4PN1P/PPP1BPP1/RN1QK2R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3P4/1P3NP1/P1P1PPBP/RNBQK2R w KQ - 0 6' => {
			eco => 'A49',
                        # TRANSLATORS:

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'c1b2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3P4/1P3NP1/PBP1PPBP/RN1QK2R b KQ - 1 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3P4/1P3NP1/PBP1PPBP/RN1QK2R b KQ - 1 6' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2 O-O 5. b3 d6 6. Bb2

			variation => N__('Neo-King\'s Indian: Double Fianchetto System'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3P4/5NP1/PPP1PPBP/RNBQ1RK1 w - - 0 6' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2 O-O 5. O-O d6

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N1BN1P/PPP2PP1/R2QKB1R b KQ - 2 6' => {
			eco => 'B08c',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3 O-O 6. Be3

			variation => N__('Pirc: Classical, Spassky System'),
			moves => {
				'a7a6' => 'rnbq1rk1/1pp1ppbp/p2p1np1/8/3PP3/2N1BN1P/PPP2PP1/R2QKB1R w KQ - 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/3PP3/2N1BN1P/PPP2PP1/R2QKB1R w KQ - 0 7',
				'd6d5' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/3PP3/2N1BN1P/PPP2PP1/R2QKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N1BN2/PPP1BPPP/R2QK2R b KQ - 5 6' => {
			eco => 'B08m',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. Be3

			variation => N__('Pirc: Classical, 5. Be2 O-O 6. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N2N1P/PPP2PP1/R1BQKB1R w KQ - 1 6' => {
			eco => 'B08b',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3 O-O

			variation => N__('Pirc: Classical, 5. h3'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N1BN1P/PPP2PP1/R2QKB1R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQ1RK1 b - - 5 6' => {
			eco => 'B08n',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O 6. O-O

			variation => N__('Pirc: Classical, 5. Be2 O-O 6.O-O'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 6 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2pp1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 0 7',
				'c8g4' => 'rn1q1rk1/ppp1ppbp/3p1np1/8/3PP1b1/2N2N2/PPP1BPPP/R1BQ1RK1 w - - 6 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R w KQ - 4 6' => {
			eco => 'B08l',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 O-O

			variation => N__('Pirc: Classical, 5. Be2 O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N1BN2/PPP1BPPP/R2QK2R b KQ - 5 6',
				'e1g1' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2N1BN2/PPP3PP/R2QKB1R b KQ - 4 6' => {
			eco => 'B09g',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Be3

			variation => N__('Pirc: Austrian, 5...O-O 6. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2N2N2/PPP1B1PP/R1BQK2R b KQ - 4 6' => {
			eco => 'B09h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Be2

			variation => N__('Pirc: Austrian, 5...O-O 6. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2N2N2/PPP3PP/R1BQKB1R w KQ - 3 6' => {
			eco => 'B09e',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O

			variation => N__('Pirc: Austrian, 5...O-O'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2N1BN2/PPP3PP/R2QKB1R b KQ - 4 6',
				'e4e5' => 'rnbq1rk1/ppp1ppbp/3p1np1/4P3/3P1P2/2N2N2/PPP3PP/R1BQKB1R b KQ - 0 6',
				'f1d3' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQK2R b KQ - 4 6',
				'f1e2' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2N2N2/PPP1B1PP/R1BQK2R b KQ - 4 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQK2R b KQ - 4 6' => {
			eco => 'B09i',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 O-O 6. Bd3

			variation => N__('Pirc: Austrian, 5...O-O 6. Bd3'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n2p1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQK2R w KQ - 5 7',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2np1np1/8/3PPP2/2NB1N2/PPP3PP/R1BQK2R w KQ - 5 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/3p1np1/8/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w - - 0 6' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2 Bg7 4. O-O O-O 5. d3 d6

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3P4/3P1B2/2N1P3/PP3PPP/R2QKBNR b KQ - 0 6' => {
			eco => 'D84',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. cxd5

			variation => N__('Grünfeld: Grünfeld Gambit Accepted'),
			moves => {
				'f6d5' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3P1B2/2N1P3/PP3PPP/R2QKBNR w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3P4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQ - 0 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. cxd5

			variation => N__('Grünfeld: 5. e3 O-O 6.cxd5'),
			moves => {
				'f6d5' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3P4/3P4/5NP1/PP2PPBP/RNBQK2R b KQ - 0 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5

			variation => N__('Neo-Grünfeld, 6.cxd5'),
			moves => {
				'f6d5' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/5NP1/PP2PPBP/RNBQK2R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R b KQ - 0 6' => {
			eco => 'D03p',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7 5. e3 O-O 6. c3

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. c3'),
			moves => {
				'b8d7' => 'r1bq1rk1/pppnppbp/5np1/3p2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R w KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/3BPN2/PPPN1PPP/R2QK2R b KQ - 2 6' => {
			eco => 'D03l',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp2B1/3P4/3BPN2/PPPN1PPP/R2QK2R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/3BPN2/PPPN1PPP/R2QK2R b KQ - 4 6' => {
			eco => 'D03s',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. e3 Bg7 5. Nbd2 O-O 6. Bd3

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O 6. Bd3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQ - 1 6' => {
			eco => 'D03l',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'c2c3' => 'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R b KQ - 0 6',
				'f1d3' => 'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/3BPN2/PPPN1PPP/R2QK2R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQ - 3 6' => {
			eco => 'D03o',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. e3 Bg7 5. Nbd2 O-O

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2 O-O'),
			moves => {
				'f1d3' => 'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/3BPN2/PPPN1PPP/R2QK2R b KQ - 4 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3PP3/5N2/PPPN1PPP/R2QKB1R w KQ d6 0 6' => {
			eco => 'A48u',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. e4 d5

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/1PPP4/2N1PN2/P4PPP/R1BQKB1R b KQ b3 0 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. b4

			variation => N__('Grünfeld: Makogonov Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/1QN1P3/PP3PPP/R3KBNR b KQ - 2 6' => {
			eco => 'D83',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. Qb3

			variation => N__('Grünfeld: Grünfeld Gambit, 6. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N1P3/PP3PPP/2RQKBNR b K - 2 6' => {
			eco => 'D83',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O 6. Rc1

			variation => N__('Grünfeld: Grünfeld Gambit, Capablanca Variation'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP1B2/2N1P3/PP3PPP/2RQKBNR w K c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N1P3/PP3PPP/R2QKBNR w KQ - 1 6' => {
			eco => 'D83',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 O-O

			variation => N__('Grünfeld: Grünfeld Gambit'),
			moves => {
				'a1c1' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N1P3/PP3PPP/2RQKBNR b K - 2 6',
				'c4d5' => 'rnbq1rk1/ppp1ppbp/5np1/3P4/3P1B2/2N1P3/PP3PPP/R2QKBNR b KQ - 0 6',
				'd1b3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/1QN1P3/PP3PPP/R3KBNR b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6' => {
			eco => 'D93',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. e3

			variation => N__('Grünfeld: 5. Bf4 O-O 6. e3'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N2N2/PP2PPPP/2RQKB1R b K - 5 6' => {
			eco => 'D92',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. Rc1

			variation => N__('Grünfeld: 5. Bf4 O-O 6. Rc1'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP1B2/2N2N2/PP2PPPP/2RQKB1R w K c6 0 7',
				'd5c4' => 'rnbq1rk1/ppp1ppbp/5np1/8/2pP1B2/2N2N2/PP2PPPP/2RQKB1R w K - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQ - 4 6' => {
			eco => 'D92',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O

			variation => N__('Grünfeld: 5. Bf4 O-O'),
			moves => {
				'a1c1' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N2N2/PP2PPPP/2RQKB1R b K - 5 6',
				'e2e3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N1PN2/PP3PPP/R2QKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/1Q3NP1/PP2PPBP/RNB1K2R b KQ - 5 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. Qb3

			variation => N__('Neo-Grünfeld, 6. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R b KQ - 2 6' => {
			eco => 'D95a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1ppbp/1p3np1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7',
				'd5c4' => 'rnbq1rk1/ppp1ppbp/5np1/8/2pP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7',
				'e7e6' => 'rnbq1rk1/ppp2pbp/4pnp1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP1B1PPP/R2QKB1R b KQ - 2 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Bd2

			variation => N__('Grünfeld: Opovcensky Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 2 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Be2

			variation => N__('Grünfeld: 5. e3 O-O 6. Be2'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP4/2N1PN2/PP2BPPP/R1BQK2R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 1 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O

			variation => N__('Grünfeld: 5. e3 O-O'),
			moves => {
				'b2b4' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/1PPP4/2N1PN2/P4PPP/R1BQKB1R b KQ b3 0 6',
				'c1d2' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP1B1PPP/R2QKB1R b KQ - 2 6',
				'c4d5' => 'rnbq1rk1/ppp1ppbp/5np1/3P4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQ - 0 6',
				'd1b3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R b KQ - 2 6',
				'f1d3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 2 6',
				'f1e2' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 5 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. Nc3

			variation => N__('Neo-Grünfeld, 6. Nc3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 2 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Bd3

			variation => N__('Grünfeld: 5. e3 O-O 6. Bd3'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6' => {
			eco => 'D77a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O

			variation => N__('Neo-Grünfeld, 6.O-O'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2n2np1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 6 7',
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
				'd5c4' => 'rnbq1rk1/ppp1ppbp/5np1/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
				'e7e6' => 'rnbq1rk1/ppp2pbp/4pnp1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O

			variation => N__('Neo-Grünfeld, 5. Nf3 O-O'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 5 6',
				'c4d5' => 'rnbq1rk1/ppp1ppbp/5np1/3P4/3P4/5NP1/PP2PPBP/RNBQK2R b KQ - 0 6',
				'd1b3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/1Q3NP1/PP2PPBP/RNB1K2R b KQ - 5 6',
				'e1g1' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/3P1B2/2N1PN2/PPP1BPPP/R2QK2R b KQ - 2 6' => {
			eco => 'D00r',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Nf3 g6 4. Bf4 Bg7 5. e3 O-O 6. Be2

			variation => N__('Queen\'s Pawn: Anti-King\'s Indian, Main Line'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/3P1B2/2N1PN2/PPP2PPP/R2QKB1R w KQ - 1 6' => {
			eco => 'D00q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Anti-King\'s Indian'),
			moves => {
				'f1e2' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/3P1B2/2N1PN2/PPP1BPPP/R2QK2R b KQ - 2 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQ1RK1 w - - 4 6' => {
			eco => 'D02u',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. O-O O-O

			variation => N__('Queen\'s Pawn: 3. g3 g6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/3PP3/2N1BN1P/PPP2PP1/R2QKB1R w KQ - 0 7' => {
			eco => 'B08c',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3 O-O 6. Be3 d5

			variation => N__('Pirc: Classical, Spassky System, 6... d5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/5P2/3P1NP1/PPP1P1BP/RNBQ1RK1 b - - 0 6' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. O-O O-O 6. d3

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6: 5.O-O O-O 6. d3'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/5P2/3P1NP1/PPP1P1BP/RNBQ1RK1 w - c6 0 7',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/5P2/3P1NP1/PPP1P1BP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQ1RK1 w - - 4 6' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. O-O O-O

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6: 5.O-O O-O'),
			moves => {
				'd2d3' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/5P2/3P1NP1/PPP1P1BP/RNBQ1RK1 b - - 0 6',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2BP4/1QN1PN2/PP3PPP/R1B1K2R b KQ - 0 7' => {
			eco => 'D95i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 dxc4 7. Bxc4

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 dxc4 7.Bxc4'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2n2np1/8/2BP4/1QN1PN2/PP3PPP/R1B1K2R w KQ - 1 8',
				'b8d7' => 'r1bq1rk1/pppnppbp/5np1/8/2BP4/1QN1PN2/PP3PPP/R1B1K2R w KQ - 1 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2BP4/2N1PN2/PP3PPP/R1BQ1RK1 b - - 4 7' => {
			eco => 'D25e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 g6 5. Bxc4 Bg7 6. O-O O-O 7. Nc3

			variation => N__('QGA: Smyslov, 5.Bxc4 Bg7 6.O-O O-O 7. Nc3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2BP4/4PN2/PP3PPP/RNBQ1RK1 w - - 3 7' => {
			eco => 'D25e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 g6 5. Bxc4 Bg7 6. O-O O-O

			variation => N__('QGA: Smyslov, 5.Bxc4 Bg7 6.O-O O-O'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppp1ppbp/5np1/8/2BP4/2N1PN2/PP3PPP/R1BQ1RK1 b - - 4 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2QP1B2/2N2N2/PP2PPPP/R3KB1R b KQ - 2 7' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. Bf4

			variation => N__('Grünfeld: Russian, 7. Bf4'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/8/2QP1B2/2N2N2/PP2PPPP/R3KB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R w KQ - 1 7' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O

			variation => N__('Grünfeld: Russian, 6...O-O'),
			moves => {
				'c1f4' => 'rnbq1rk1/ppp1ppbp/5np1/8/2QP1B2/2N2N2/PP2PPPP/R3KB1R b KQ - 2 7',
				'e2e4' => 'rnbq1rk1/ppp1ppbp/5np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R b KQ e3 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R b KQ e3 0 7' => {
			eco => 'D97a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4

			variation => N__('Grünfeld: Russian, 7. e4'),
			moves => {
				'a7a6' => 'rnbq1rk1/1pp1ppbp/p4np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 0 8',
				'b7b6' => 'rnbq1rk1/p1p1ppbp/1p3np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 0 8',
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n4np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2n2np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p2np1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 0 8',
				'c8g4' => 'rn1q1rk1/ppp1ppbp/5np1/8/2QPP1b1/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8',
				'f6d7' => 'rnbq1rk1/pppnppbp/6p1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2QPP3/2N5/PP3PPP/R1B1KBNR w KQ - 1 7' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4 Bg7 6. e4 O-O

			variation => N__('Grünfeld: Early Russian, 5... Bg7 6. e4 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2pP1B2/2N2N2/PP2PPPP/2RQKB1R w K - 0 7' => {
			eco => 'D92',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 O-O 6. Rc1 dxc4

			variation => N__('Grünfeld: 5. Bf4 O-O 6. Rc1 dxc4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2pP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7' => {
			eco => 'D95i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 dxc4

			variation => N__('Grünfeld: 5. e3 O-O 6. Qb3 dxc4'),
			moves => {
				'f1c4' => 'rnbq1rk1/ppp1ppbp/5np1/8/2BP4/1QN1PN2/PP3PPP/R1B1K2R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'D77e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4

			variation => N__('Neo-Grünfeld, 6.O-O dxc4'),
			moves => {
				'b1a3' => 'rnbq1rk1/ppp1ppbp/5np1/8/2pP4/N4NP1/PP2PPBP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/2pP4/N4NP1/PP2PPBP/R1BQ1RK1 b - - 1 7' => {
			eco => 'D77f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n4np1/8/2pP4/N4NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2n2np1/8/2pP4/N4NP1/PP2PPBP/R1BQ1RK1 w - - 2 8',
				'c4c3' => 'rnbq1rk1/ppp1ppbp/5np1/8/3P4/N1p2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/3P4/N1P2NP1/P3PPBP/R1BQ1RK1 b - - 0 8' => {
			eco => 'D77h',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 c3'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp2ppbp/5np1/2p5/3P4/N1P2NP1/P3PPBP/R1BQ1RK1 w - c6 0 9',
			},
		},
		'rnbq1rk1/ppp1ppbp/5np1/8/3P4/N1p2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'D77h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O dxc4 7. Na3 c3

			variation => N__('Neo-Grünfeld, 6.O-O dxc4 7. Na3 c3'),
			moves => {
				'b2c3' => 'rnbq1rk1/ppp1ppbp/5np1/8/3P4/N1P2NP1/P3PPBP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3N4/3P1B2/4P3/PP3PPP/R2QKBNR b KQ - 0 7' => {
			eco => 'D84',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Grünfeld Gambit Accepted'),
			moves => {
				'd8d5' => 'rnb2rk1/ppp1ppbp/6p1/3q4/3P1B2/4P3/PP3PPP/R2QKBNR w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQ - 1 7' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. cxd5 Nxd5 7. Bc4

			variation => N__('Grünfeld: 5. e3 O-O 6.cxd5 Nxd5 7. Bc4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3P1B2/2N1P3/PP3PPP/R2QKBNR w KQ - 0 7' => {
			eco => 'D84',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Grünfeld Gambit Accepted'),
			moves => {
				'c3d5' => 'rnbq1rk1/ppp1ppbp/6p1/3N4/3P1B2/4P3/PP3PPP/R2QKBNR b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 0 7' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. cxd5 Nxd5

			variation => N__('Grünfeld: 5. e3 O-O 6.cxd5'),
			moves => {
				'f1c4' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP1BPPPP/2RQKB1R b K - 3 7' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. cxd5 Nxd5 6. Bd2 O-O 7. Rc1

			variation => N__('Grünfeld: Three Knights Exchange, 6. Bd2 O-O 7. Rc1'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP1BPPPP/R2QKB1R w KQ - 2 7' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. cxd5 Nxd5 6. Bd2 O-O

			variation => N__('Grünfeld: Three Knights Exchange, 6. Bd2 O-O'),
			moves => {
				'a1c1' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP1BPPPP/2RQKB1R b K - 3 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b - - 1 7' => {
			eco => 'D74',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. O-O

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7.O-O'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp1ppbp/n5p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 8',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2n3p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 8',
				'c7c5' => 'rnbq1rk1/pp2ppbp/6p1/2pn4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - c6 0 8',
				'c7c6' => 'rnbq1rk1/pp2ppbp/2p3p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 8',
				'd5b6' => 'rnbq1rk1/ppp1ppbp/1n4p1/8/3P4/5NP1/PP2PPBP/RNBQ1RK1 w - - 2 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/5NP1/PP2PPBP/RNBQK2R w KQ - 0 7' => {
			eco => 'D73',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, 6.cxd5'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/5NP1/PP2PPBP/RNBQ1RK1 b - - 1 7',
				'e2e4' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3PP3/5NP1/PP3PBP/RNBQK2R b KQ e3 0 7',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/3n4/3PP3/5NP1/PP3PBP/RNBQK2R b KQ e3 0 7' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. cxd5 Nxd5 7. e4

			variation => N__('Neo-Grünfeld, 6.cxd5 Nxd5 7. e4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/8/2BPP3/2P5/P3NPPP/R1BQK2R b KQ - 4 8' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O 8. Ne2

			variation => N__('Grünfeld: Classical Exchange, 8. Ne2'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p1ppbp/1p4p1/8/2BPP3/2P5/P3NPPP/R1BQK2R w KQ - 0 9',
				'b8c6' => 'r1bq1rk1/ppp1ppbp/2n3p1/8/2BPP3/2P5/P3NPPP/R1BQK2R w KQ - 5 9',
				'c7c5' => 'rnbq1rk1/pp2ppbp/6p1/2p5/2BPP3/2P5/P3NPPP/R1BQK2R w KQ c6 0 9',
				'd8d7' => 'rnb2rk1/pppqppbp/6p1/8/2BPP3/2P5/P3NPPP/R1BQK2R w KQ - 5 9',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/8/2BPP3/2P5/P4PPP/R1BQK1NR w KQ - 3 8' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 O-O

			variation => N__('Grünfeld: Classical Exchange, 7...O-O'),
			moves => {
				'g1e2' => 'rnbq1rk1/ppp1ppbp/6p1/8/2BPP3/2P5/P3NPPP/R1BQK2R b KQ - 4 8',
			},
		},
		'rnbq1rk1/ppp1ppbp/6p1/8/3PP3/2P2N2/P4PPP/R1BQKB1R w KQ - 3 8' => {
			eco => 'D85i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 O-O

			variation => N__('Grünfeld: Modern Exchange, 7...O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pb1/3p1npp/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QK1NR b KQ - 0 8' => {
			eco => 'E73y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6 7. Be3 e5 8. d5

			variation => N__('King\'s Indian: Averbakh, 6... h6 7. Be3 e5 8. d5'),
			moves => {
				'b8d7' => 'r1bq1rk1/pppn1pb1/3p1npp/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QK1NR w KQ - 1 9',
				'c7c6' => 'rnbq1rk1/pp3pb1/2pp1npp/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QK1NR w KQ - 0 9',
			},
		},
		'rnbq1rk1/ppp2pb1/3p1npp/4p3/2PPP3/2N1B3/PP2BPPP/R2QK1NR w KQ e6 0 8' => {
			eco => 'E73y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 h6 7. Be3 e5

			variation => N__('King\'s Indian: Averbakh, 6... h6 7. Be3 e5'),
			moves => {
				'd4d5' => 'rnbq1rk1/ppp2pb1/3p1npp/3Pp3/2P1P3/2N1B3/PP2BPPP/R2QK1NR b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7' => {
			eco => 'E87a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5'),
			moves => {
				'a7a5' => 'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ a6 0 8',
				'b8d7' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 1 8',
				'c7c5' => 'rnbq1rk1/pp3pbp/3p1np1/2pPp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ c6 0 8',
				'c7c6' => 'rnbq1rk1/pp3pbp/2pp1np1/3Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 8',
				'f6h5' => 'rnbq1rk1/ppp2pbp/3p2p1/3Pp2n/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 1 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7' => {
			eco => 'E90p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. d5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. d5'),
			moves => {
				'a7a5' => 'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ a6 0 8',
				'b8a6' => 'r1bq1rk1/ppp2pbp/n2p1np1/3Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 8',
				'b8d7' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E92n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. d5

			variation => N__('King\'s Indian: Petrosian System'),
			moves => {
				'a7a5' => 'rnbq1rk1/1pp2pbp/3p1np1/p2Pp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ a6 0 8',
				'b8d7' => 'r1bq1rk1/pppn1pbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 1 8',
				'c7c5' => 'rnbq1rk1/pp3pbp/3p1np1/2pPp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ c6 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N3N1/PP3PPP/R1BQKB1R b KQ - 0 7' => {
			eco => 'E70i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nge2 O-O 6. Ng3 e5 7. d5

			variation => N__('King\'s Indian: Kramer, 5...O-O 6. Ng3 e5 7. d5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2NB4/PP2NPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E70r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 e5 7. d5

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 e5 7. d5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N1BN2/PP3PPP/R2QKB1R b KQ - 0 7' => {
			eco => 'E90i',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Larsen, 6... e5'),
			moves => {
				'd6e5' => 'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BN2/PP3PPP/R2QKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7' => {
			eco => 'E85b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. dxe5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.dxe5'),
			moves => {
				'd6e5' => 'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7' => {
			eco => 'E90n',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5'),
			moves => {
				'd6e5' => 'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7' => {
			eco => 'E92b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. dxe5

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap'),
			moves => {
				'd6e5' => 'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P1P3/2N3P1/PP1PNPBP/R1BQK2R w KQ - 2 7' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 O-O

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'd2d3' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P1P3/2NP2P1/PP2NPBP/R1BQK2R b KQ - 0 7' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nge2 O-O 7. d3

			variation => N__('English: Bremen, 3... g6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQ - 2 7' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e3 d6 6. Nge2 O-O

			variation => N__('English: Bremen, 3... g6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 2 7' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. O-O O-O

			variation => N__('English: Bremen, 3... g6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b - - 2 7' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. d3 O-O 7. O-O

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R w KQ - 1 7' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQ1RK1 b - - 2 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1B2P/PP3PP1/R2QKBNR w KQ e6 0 7' => {
			eco => 'E71f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 O-O 6. Be3 e5

			variation => N__('King\'s Indian: Makagonov, 5...O-O 6. Be3 e5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 1 7' => {
			eco => 'E90n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5 7. Be3

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5 7. Be3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2QK2R b KQ - 1 7' => {
			eco => 'E92g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3

			variation => N__('King\'s Indian: Gligoric-Taimanov System'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2QK2R w KQ - 0 8',
				'd8e7' => 'rnb2rk1/ppp1qpbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2QK2R w KQ - 2 8',
				'e5d4' => 'rnbq1rk1/ppp2pbp/3p1np1/8/2PpP3/2N1BN2/PP2BPPP/R2QK2R w KQ - 0 8',
				'f6g4' => 'rnbq1rk1/ppp2pbp/3p2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2QK2R w KQ - 2 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BN2/PP3PPP/R2QKB1R w KQ e6 0 7' => {
			eco => 'E90i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be3 e5

			variation => N__('King\'s Indian: Larsen, 6... e5'),
			moves => {
				'd4e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N1BN2/PP3PPP/R2QKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 1 7' => {
			eco => 'E85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 2 8',
				'b8d7' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 2 8',
				'c7c6' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 8',
				'e5d4' => 'rnbq1rk1/ppp2pbp/3p1np1/8/2PpP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQ e6 0 7' => {
			eco => 'E85a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5

			variation => N__('King\'s Indian: Sämisch, Orthodox Variation'),
			moves => {
				'd4d5' => 'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7',
				'd4e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N1BP2/PP4PP/R2QKBNR b KQ - 0 7',
				'g1e2' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BP2/PP2N1PP/R2QKB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2N1P/PP3PP1/R1BQKB1R w KQ e6 0 7' => {
			eco => 'E90n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 e5

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BN1P/PP3PP1/R2QKB1R b KQ - 1 7',
				'd4d5' => 'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7',
				'd4e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 7' => {
			eco => 'E94a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O

			variation => N__('King\'s Indian: 7.O-O'),
			moves => {
				'b8a6' => 'r1bq1rk1/ppp2pbp/n2p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8',
				'b8c6' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8',
				'b8d7' => 'r1bq1rk1/pppn1pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 8',
				'c7c6' => 'rnbq1rk1/pp3pbp/2pp1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8',
				'e5d4' => 'rnbq1rk1/ppp2pbp/3p1np1/8/2PpP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQK2R w KQ e6 0 7' => {
			eco => 'E92a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5

			variation => N__('King\'s Indian: 6. Be2 e5'),
			moves => {
				'c1e3' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N1BN2/PP2BPPP/R2QK2R b KQ - 1 7',
				'd4d5' => 'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7',
				'd4e5' => 'rnbq1rk1/ppp2pbp/3p1np1/4P3/2P1P3/2N2N2/PP2BPPP/R1BQK2R b KQ - 0 7',
				'e1g1' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2N2/PP2BPPP/R1BQ1RK1 b - - 1 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N2P2/PP2N1PP/R1BQKB1R w KQ e6 0 7' => {
			eco => 'E81c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Nge2 e5

			variation => N__('King\'s Indian: Sämisch, 5...O-O 6.Nge2 e5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N3N1/PP3PPP/R1BQKB1R w KQ e6 0 7' => {
			eco => 'E70i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nge2 O-O 6. Ng3 e5

			variation => N__('King\'s Indian: Kramer, 5...O-O 6. Ng3 e5'),
			moves => {
				'd4d5' => 'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2N3N1/PP3PPP/R1BQKB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N3P1/PP2NPBP/R1BQK2R b KQ - 1 7' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2 e5 7. Nge2

			variation => N__('King\'s Indian: Pomar System'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp2pbp/2np1np1/4p3/2PPP3/2N3P1/PP2NPBP/R1BQK2R w KQ - 2 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N3P1/PP3PBP/R1BQK1NR w KQ e6 0 7' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3 O-O 6. Bg2 e5

			variation => N__('King\'s Indian: 4. e4 d6 5. g3 O-O 6. Bg2 e5'),
			moves => {
				'g1e2' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2N3P1/PP2NPBP/R1BQK2R b KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/4p3/2PPP3/2NB4/PP2NPPP/R1BQK2R w KQ e6 0 7' => {
			eco => 'E70q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 O-O 6. Nge2 e5

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 O-O 6.Nge2 e5'),
			moves => {
				'd4d5' => 'rnbq1rk1/ppp2pbp/3p1np1/3Pp3/2P1P3/2NB4/PP2NPPP/R1BQK2R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/8/2PNP3/2N5/PP2BPPP/R1BQ1RK1 b - - 0 8' => {
			eco => 'E94b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 7.O-O exd4'),
			moves => {
				'f8e8' => 'rnbqr1k1/ppp2pbp/3p1np1/8/2PNP3/2N5/PP2BPPP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/8/2PpP3/2N1BN2/PP2BPPP/R2QK2R w KQ - 0 8' => {
			eco => 'E92i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 exd4

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7...exd4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/8/2PpP3/2N1BP2/PP2N1PP/R2QKB1R w KQ - 0 8' => {
			eco => 'E85e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. Nge2 exd4

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.Nge2 exd4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/3p1np1/8/2PpP3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 0 8' => {
			eco => 'E94b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4

			variation => N__('King\'s Indian: 7.O-O exd4'),
			moves => {
				'f3d4' => 'rnbq1rk1/ppp2pbp/3p1np1/8/2PNP3/2N5/PP2BPPP/R1BQ1RK1 b - - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p2p1/3Pp2n/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 2 8' => {
			eco => 'E87f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5 8. Qd2'),
			moves => {
				'd8h4' => 'rnb2rk1/ppp2pbp/3p2p1/3Pp2n/2P1P2q/2N1BP2/PP1Q2PP/R3KBNR w KQ - 3 9',
				'f7f5' => 'rnbq1rk1/ppp3bp/3p2p1/3Ppp1n/2P1P3/2N1BP2/PP1Q2PP/R3KBNR w KQ f6 0 9',
			},
		},
		'rnbq1rk1/ppp2pbp/3p2p1/3Pp2n/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 1 8' => {
			eco => 'E87e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5'),
			moves => {
				'd1d2' => 'rnbq1rk1/ppp2pbp/3p2p1/3Pp2n/2P1P3/2N1BP2/PP1Q2PP/R3KBNR b KQ - 2 8',
			},
		},
		'rnbq1rk1/ppp2pbp/3p2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2QK2R b KQ - 3 8' => {
			eco => 'E92k',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... Ng4'),
			moves => {
				'f7f6' => 'rnbq1rk1/ppp3bp/3p1pp1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9',
			},
		},
		'rnbq1rk1/ppp2pbp/3p2p1/4p3/2PPP1n1/2N1BN2/PP2BPPP/R2QK2R w KQ - 2 8' => {
			eco => 'E92k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 Ng4

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... Ng4'),
			moves => {
				'e3g5' => 'rnbq1rk1/ppp2pbp/3p2p1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2QK2R b KQ - 3 8',
			},
		},
		'rnbq1rk1/ppp2pbp/4pnp1/3p4/2PP4/1QN1PN2/PP1B1PPP/R3KB1R b KQ - 1 7' => {
			eco => 'D95h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 e6 7. Bd2

			variation => N__('Grünfeld: Botvinnik, 7. Bd2'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/4pnp1/3p4/2PP4/1QN1PN2/PP3PPP/R1B1KB1R w KQ - 0 7' => {
			eco => 'D95g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 O-O 6. Qb3 e6

			variation => N__('Grünfeld: Botvinnik Variation'),
			moves => {
				'c1d2' => 'rnbq1rk1/ppp2pbp/4pnp1/3p4/2PP4/1QN1PN2/PP1B1PPP/R3KB1R b KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp2pbp/4pnp1/3p4/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7' => {
			eco => 'D77b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 O-O 6. O-O e6

			variation => N__('Neo-Grünfeld, 6.O-O e6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'E90i',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Larsen, 6... e5'),
			moves => {
				'd1d8' => 'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BN2/PP3PPP/R3KB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BP2/PP4PP/R2QKBNR w KQ - 0 8' => {
			eco => 'E85b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7.dxe5'),
			moves => {
				'd1d8' => 'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N1BP2/PP4PP/R3KBNR b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 0 8' => {
			eco => 'E90n',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 e5'),
			moves => {
				'd1d8' => 'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N1P/PP3PP1/R1B1KB1R b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 0 8' => {
			eco => 'E92b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. dxe5 dxe5

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap'),
			moves => {
				'd1d8' => 'rnbQ1rk1/ppp2pbp/5np1/4p3/2P1P3/2N2N2/PP2BPPP/R1B1K2R b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2pbp/6p1/3p4/1P1P4/2N1PN2/P4PPP/R2QKB1R b KQ b3 0 10' => {
			eco => 'D91t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. b4

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. b4'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3pbp/2p3p1/3p4/1P1P4/2N1PN2/P4PPP/R2QKB1R w KQ - 0 11',
			},
		},
		'rnbq1rk1/ppp2pbp/6p1/3p4/3P4/2N1PN2/PP2BPPP/R2QK2R b KQ - 2 10' => {
			eco => 'D91s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Be2

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Be2'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3pbp/2p3p1/3p4/3P4/2N1PN2/PP2BPPP/R2QK2R w KQ - 0 11',
			},
		},
		'rnbq1rk1/ppp2pbp/6p1/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 10' => {
			eco => 'D91q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O'),
			moves => {
				'b2b4' => 'rnbq1rk1/ppp2pbp/6p1/3p4/1P1P4/2N1PN2/P4PPP/R2QKB1R b KQ b3 0 10',
				'f1d3' => 'rnbq1rk1/ppp2pbp/6p1/3p4/3P4/2NBPN2/PP3PPP/R2QK2R b KQ - 2 10',
				'f1e2' => 'rnbq1rk1/ppp2pbp/6p1/3p4/3P4/2N1PN2/PP2BPPP/R2QK2R b KQ - 2 10',
			},
		},
		'rnbq1rk1/ppp2pbp/6p1/3p4/3P4/2NBPN2/PP3PPP/R2QK2R b KQ - 2 10' => {
			eco => 'D91r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Bd3

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3 O-O 10. Bd3'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3pbp/2p3p1/3p4/3P4/2NBPN2/PP3PPP/R2QK2R w KQ - 0 11',
			},
		},
		'rnbq1rk1/ppp2pp1/4pb1p/3p4/2PP4/1QN1PN2/PP3PPP/R3KB1R b KQ - 1 8' => {
			eco => 'D55i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bxf6 Bxf6 8. Qb3

			variation => N__('QGD: Neo-Orthodox, 7.Bxf6 Bxf6 8. Qb3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2pp1/4pb1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R b K - 1 8' => {
			eco => 'D55j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bxf6 Bxf6 8. Rc1

			variation => N__('QGD: Neo-Orthodox, Gligoric Variation'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3pp1/2p1pb1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R w K - 0 9',
			},
		},
		'rnbq1rk1/ppp2pp1/4pb1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 0 8' => {
			eco => 'D55h',
                        # TRANSLATORS:

			variation => N__('QGD: Neo-Orthodox, 7.Bxf6'),
			moves => {
				'a1c1' => 'rnbq1rk1/ppp2pp1/4pb1p/3p4/2PP4/2N1PN2/PP3PPP/2RQKB1R b K - 1 8',
				'd1b3' => 'rnbq1rk1/ppp2pp1/4pb1p/3p4/2PP4/1QN1PN2/PP3PPP/R3KB1R b KQ - 1 8',
			},
		},
		'rnbq1rk1/ppp2ppp/3b4/3p4/2PPn3/3B1N2/PP3PPP/RNBQ1RK1 b - c3 0 8' => {
			eco => 'C42q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Marshall Variation'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp3ppp/2pb4/3p4/2PPn3/3B1N2/PP3PPP/RNBQ1RK1 w - - 0 9',
				'c8g4' => 'rn1q1rk1/ppp2ppp/3b4/3p4/2PPn1b1/3B1N2/PP3PPP/RNBQ1RK1 w - - 1 9',
			},
		},
		'rnbq1rk1/ppp2ppp/3b4/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 w - - 4 8' => {
			eco => 'C42q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Marshall Variation'),
			moves => {
				'c2c4' => 'rnbq1rk1/ppp2ppp/3b4/3p4/2PPn3/3B1N2/PP3PPP/RNBQ1RK1 b - c3 0 8',
			},
		},
		'rnbq1rk1/ppp2ppp/3b4/3pN3/2PPn3/3B4/PP3PPP/RNBQ1RK1 b - c3 0 7' => {
			eco => 'C43m',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Bd6'),
			moves => {
				'd6e5' => 'rnbq1rk1/ppp2ppp/8/3pb3/2PPn3/3B4/PP3PPP/RNBQ1RK1 w - - 0 8',
			},
		},
		'rnbq1rk1/ppp2ppp/3b4/3pN3/3Pn3/3B4/PPP2PPP/RNBQ1RK1 w - - 3 7' => {
			eco => 'C43m',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Bd6'),
			moves => {
				'c2c4' => 'rnbq1rk1/ppp2ppp/3b4/3pN3/2PPn3/3B4/PP3PPP/RNBQ1RK1 b - c3 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/3bpn2/3p4/2PP4/P1N1P3/1P2NPPP/R1BQKB1R w KQ - 1 7' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 d5 6. a3 Bd6

			variation => N__('Nimzo-Indian: Reshevsky, Simagin Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/3ppn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ - 0 6' => {
			eco => 'E47',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d6

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/3ppn2/8/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ - 0 7' => {
			eco => 'E32j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 d6

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... d6'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3P4/1b1P4/2NBP3/PP3PPP/R1BQK1NR b KQ - 0 6' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. cxd5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6.cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3P4/3P4/P1P2P2/4P1PP/R1BQKBNR b KQ - 0 7' => {
			eco => 'E27',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. f3 d5 7. cxd5

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. f3 d5 7.cxd5'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQ d6 0 6' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 d5

			variation => N__('Nimzo-Indian: Reshevsky Variation'),
			moves => {
				'a2a3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/P1N1P3/1P2NPPP/R1BQKB1R b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1P3/PPQ2PPP/R1B1KBNR w KQ - 1 6' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. e3 O-O

			variation => N__('Nimzo-Indian: Classical, Noa, 5. e3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Be2

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. Be2'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQ d6 0 6' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5'),
			moves => {
				'a2a3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/P1N1PN2/1P3PPP/R1BQKB1R b KQ - 0 6',
				'f1d3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 1 6',
				'f1e2' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1PN2/PP2BPPP/R1BQK2R b KQ - 1 6',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2NBP3/PP2NPPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. Ne2

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. Ne2'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBP3/PP2NPPP/R1BQK2R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ d6 0 6' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5'),
			moves => {
				'a2a3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/P1NBP3/1P3PPP/R1BQK1NR b KQ - 0 6',
				'c4d5' => 'rnbq1rk1/ppp2ppp/4pn2/3P4/1b1P4/2NBP3/PP3PPP/R1BQK1NR b KQ - 0 6',
				'g1e2' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2NBP3/PP2NPPP/R1BQK2R b KQ - 1 6',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R b KQ - 1 6' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. Bd3

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. Bd3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1p2ppp/1p2pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 0 7',
				'b8c6' => 'r1bq1rk1/ppp2ppp/2n1pn2/3p4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ - 2 7',
				'c7c5' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/1bPP4/2NBPN2/PP3PPP/R1BQK2R w KQ c6 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP1NPPBP/R1BQK1NR w KQ - 4 6' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Bb4+ 5. Nd2 O-O

			variation => N__('Catalan: 4... Bb4+ 5. Nd2 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/P1N1P3/1P2NPPP/R1BQKB1R b KQ - 0 6' => {
			eco => 'E46',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Reshevsky Variation'),
			moves => {
				'b4d6' => 'rnbq1rk1/ppp2ppp/3bpn2/3p4/2PP4/P1N1P3/1P2NPPP/R1BQKB1R w KQ - 1 7',
				'b4e7' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/P1N1P3/1P2NPPP/R1BQKB1R w KQ - 1 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/P1N1PN2/1P3PPP/R1BQKB1R b KQ - 0 6' => {
			eco => 'E51',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 d5 6. a3

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 d5 6. a3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/P1NBP3/1P3PPP/R1BQK1NR b KQ - 0 6' => {
			eco => 'E48',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. a3'),
			moves => {
				'b4c3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1bBP3/1P3PPP/R1BQK1NR w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/2N1PN2/PP1Q1PPP/R3KB1R b KQ - 0 7' => {
			eco => 'E11z',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+ 5. Qxd2 O-O 6. Nc3 d5 7. e3

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+ 5.Qxd2 O-O 6. Nc3 d5 7. e3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/2N2N2/PP1QPPPP/R3KB1R w KQ d6 0 7' => {
			eco => 'E11y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+ 5. Qxd2 O-O 6. Nc3 d5

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+ 5.Qxd2 O-O 6. Nc3 d5'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/2N1PN2/PP1Q1PPP/R3KB1R b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1P1PP2/6PP/R1BQKBNR b KQ - 0 7' => {
			eco => 'E27',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. f3 d5 7. e3

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. f3 d5 7. e3'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1P2P2/4P1PP/R1BQKBNR w KQ d6 0 7' => {
			eco => 'E27',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. f3 d5

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. f3 d5'),
			moves => {
				'c4d5' => 'rnbq1rk1/ppp2ppp/4pn2/3P4/3P4/P1P2P2/4P1PP/R1BQKBNR b KQ - 0 7',
				'e2e3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1P1PP2/6PP/R1BQKBNR b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1PBP3/5PPP/R1BQK1NR b KQ - 0 7' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3

			variation => N__('Nimzo-Indian: Botvinnik System'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/2PP4/P1PBP3/5PPP/R1BQK1NR w KQ c6 0 8',
				'd5c4' => 'rnbq1rk1/ppp2ppp/4pn2/8/2pP4/P1PBP3/5PPP/R1BQK1NR w KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ - 1 7' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 O-O

			variation => N__('Nimzo-Indian: Classical, Noa, 5. a3 Bxc3+ 6.Qxc3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1bBP3/1P3PPP/R1BQK1NR w KQ - 0 7' => {
			eco => 'E48',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3 d5 6. a3'),
			moves => {
				'b2c3' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1PBP3/5PPP/R1BQK1NR b KQ - 0 7',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/8/2BP4/P1P1P3/5PPP/R1BQK1NR b KQ - 0 8' => {
			eco => 'E49',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Botvinnik, 7...dxc4'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp3ppp/4pn2/2p5/2BP4/P1P1P3/5PPP/R1BQK1NR w KQ c6 0 9',
			},
		},
		'rnbq1rk1/ppp2ppp/4pn2/8/2pP4/P1PBP3/5PPP/R1BQK1NR w KQ - 0 8' => {
			eco => 'E49',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3 d5 6. a3 Bxc3+ 7. bxc3 dxc4

			variation => N__('Nimzo-Indian: Botvinnik, 7...dxc4'),
			moves => {
				'd3c4' => 'rnbq1rk1/ppp2ppp/4pn2/8/2BP4/P1P1P3/5PPP/R1BQK1NR b KQ - 0 8',
			},
		},
		'rnbq1rk1/ppp2ppp/5P2/2b4Q/2pp4/2P5/PP3PPP/RNB1K1NR w KQ - 2 8' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. c3 Nf6 4. d4 exd4 5. e5 d5 6. exf6 dxc4 7. Qh5 O-O

			variation => N__('Bishop\'s Opening: Pratt Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp2ppp/8/3pb3/2PPn3/3B4/PP3PPP/RNBQ1RK1 w - - 0 8' => {
			eco => 'C43n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Bd6 6. O-O O-O 7. c4 Bxe5

			variation => N__('Russian Game: Modern Attack, Trifunovic Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppp3bp/3p1pp1/4p1B1/2PPP1n1/2N2N2/PP2BPPP/R2QK2R w KQ - 0 9' => {
			eco => 'E92k',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... Ng4'),
			moves => {
				'g5c1' => 'rnbq1rk1/ppp3bp/3p1pp1/4p3/2PPP1n1/2N2N2/PP2BPPP/R1BQK2R b KQ - 1 9',
				'g5h4' => 'rnbq1rk1/ppp3bp/3p1pp1/4p3/2PPP1nB/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9',
			},
		},
		'rnbq1rk1/ppp3bp/3p1pp1/4p3/2PPP1n1/2N2N2/PP2BPPP/R1BQK2R b KQ - 1 9' => {
			eco => 'E92k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 Ng4 8. Bg5 f6 9. Bc1

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... Ng4 8. Bg5 f6 9. Bc1'),
			moves => {
			},
		},
		'rnbq1rk1/ppp3bp/3p1pp1/4p3/2PPP1nB/2N2N2/PP2BPPP/R2QK2R b KQ - 1 9' => {
			eco => 'E92l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. Be3 Ng4 8. Bg5 f6 9. Bh4

			variation => N__('King\'s Indian: Gligoric-Taimanov, 7... Ng4 8. Bg5 f6 9. Bh4'),
			moves => {
				'b8c6' => 'r1bq1rk1/ppp3bp/2np1pp1/4p3/2PPP1nB/2N2N2/PP2BPPP/R2QK2R w KQ - 2 10',
			},
		},
		'rnbq1rk1/ppp3bp/3p2p1/3Pp2n/2P1Pp2/2N1BP2/PP1Q2PP/2KR1BNR w - - 0 10' => {
			eco => 'E87l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 f5 9. O-O-O f4

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5 8. Qd2 f5 9.O-O-O f4'),
			moves => {
			},
		},
		'rnbq1rk1/ppp3bp/3p2p1/3Ppp1n/2P1P3/2N1BP2/PP1Q2PP/2KR1BNR b - - 1 9' => {
			eco => 'E87k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 f5 9. O-O-O

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5 8. Qd2 f5 9.O-O-O'),
			moves => {
				'b8d7' => 'r1bq1rk1/pppn2bp/3p2p1/3Ppp1n/2P1P3/2N1BP2/PP1Q2PP/2KR1BNR w - - 2 10',
				'f5f4' => 'rnbq1rk1/ppp3bp/3p2p1/3Pp2n/2P1Pp2/2N1BP2/PP1Q2PP/2KR1BNR w - - 0 10',
			},
		},
		'rnbq1rk1/ppp3bp/3p2p1/3Ppp1n/2P1P3/2N1BP2/PP1Q2PP/R3KBNR w KQ f6 0 9' => {
			eco => 'E87j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 O-O 6. Be3 e5 7. d5 Nh5 8. Qd2 f5

			variation => N__('King\'s Indian: Sämisch, Orthodox, 7. d5 Nh5 8. Qd2 f5'),
			moves => {
				'e1c1' => 'rnbq1rk1/ppp3bp/3p2p1/3Ppp1n/2P1P3/2N1BP2/PP1Q2PP/2KR1BNR b - - 1 9',
			},
		},
		'rnbq1rk1/ppp3bp/3ppnp1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 0 8' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O d6 7. Nc3 e6

			variation => N__('Dutch: Leningrad, Main Line, 7. Nc3 e6'),
			moves => {
			},
		},
		'rnbq1rk1/pppnbppp/4p3/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQ - 1 7' => {
			eco => 'C13k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 O-O

			variation => N__('French: Chatard-Alekhine, Spielmann Variation'),
			moves => {
			},
		},
		'rnbq1rk1/pppnppbp/6p1/8/2QPP3/2N2N2/PP3PPP/R1B1KB1R w KQ - 1 8' => {
			eco => 'D97a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 O-O 7. e4 Nfd7

			variation => N__('Grünfeld: Russian, 7. e4 Nfd7'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4p3/8/1bPPn3/2N1PN2/PP3PPP/R1BQKB1R w KQ - 3 6' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3 Ne4

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3 Ne4'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4p3/8/2PPn3/P1Q5/1P2PPPP/R1B1KBNR w KQ - 1 7' => {
			eco => 'E32k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3 Ne4

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3, 6... Ne4'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/6B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQ - 4 5' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 O-O

			variation => N__('Nimzo-Indian: Leningrad, 4...O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/6B1/1bPP4/2N5/PPQ1PPPP/R3KBNR b KQ - 5 5' => {
			eco => 'E32e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. Bg5

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. Bg5'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bP5/2N2N2/PPQPPPPP/R1B1KB1R w KQ - 4 5' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 Bb4 4. Qc2 O-O

			variation => N__('English: Nimzo-English, 4. Qc2 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQ - 2 5' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2

			variation => N__('Nimzo-Indian: Reshevsky Variation'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1pp1ppp/1p2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQ - 0 6',
				'd7d5' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQ d6 0 6',
				'f8e8' => 'rnbqr1k1/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQ - 3 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQ - 1 5' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O

			variation => N__('Nimzo-Indian: 4. e3 O-O'),
			moves => {
				'f1d3' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQ - 2 5',
				'g1e2' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQ - 2 5',
				'g1f3' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R b KQ - 2 5',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R b KQ - 2 5' => {
			eco => 'E50',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Nf3

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Nf3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1pp1ppp/1p2pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 0 6',
				'c7c5' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQ c6 0 6',
				'd7d5' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQ d6 0 6',
				'f6e4' => 'rnbq1rk1/pppp1ppp/4p3/8/1bPPn3/2N1PN2/PP3PPP/R1BQKB1R w KQ - 3 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N2N2/PP1BPPPP/R2QKB1R b KQ - 5 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bd2 O-O 5. Nf3

			variation => N__('Nimzo-Indian: 4. Bd2 O-O 5. Nf3'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQ - 4 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 O-O

			variation => N__('Nimzo-Indian: Three Knights, 4...O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQ - 5 5' => {
			eco => 'E32g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. Nf3

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. Nf3'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N5/PP1BPPPP/R2QKBNR w KQ - 4 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bd2 O-O

			variation => N__('Nimzo-Indian: 4. Bd2 O-O'),
			moves => {
				'g1f3' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N2N2/PP1BPPPP/R2QKB1R b KQ - 5 5',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQ - 4 5' => {
			eco => 'E32d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O

			variation => N__('Nimzo-Indian: Classical, 4...O-O'),
			moves => {
				'a2a3' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/P1N5/1PQ1PPPP/R1B1KBNR b KQ - 0 5',
				'c1g5' => 'rnbq1rk1/pppp1ppp/4pn2/6B1/1bPP4/2N5/PPQ1PPPP/R3KBNR b KQ - 5 5',
				'e2e4' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPPP3/2N5/PPQ2PPP/R1B1KBNR b KQ e3 0 5',
				'g1f3' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQ - 5 5',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQ - 2 5' => {
			eco => 'E47',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Bd3

			variation => N__('Nimzo-Indian: 4. e3 O-O 5. Bd3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1pp1ppp/1p2pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ - 0 6',
				'c7c5' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ c6 0 6',
				'd7d5' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ d6 0 6',
				'd7d6' => 'rnbq1rk1/ppp2ppp/3ppn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQ - 0 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/5N2/PP1NPPPP/R1BQKB1R w KQ - 4 5' => {
			eco => 'E11e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 O-O

			variation => N__('Bogo-Indian: 4.Nbd2 O-O'),
			moves => {
				'a2a3' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/P4N2/1P1NPPPP/R1BQKB1R b KQ - 0 5',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/P1N5/1PQ1PPPP/R1B1KBNR b KQ - 0 5' => {
			eco => 'E32h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3'),
			moves => {
				'b4c3' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1b5/1PQ1PPPP/R1B1KBNR w KQ - 0 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/P4N2/1P1NPPPP/R1BQKB1R b KQ - 0 5' => {
			eco => 'E11f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 O-O 5. a3

			variation => N__('Bogo-Indian: 4.Nbd2 O-O 5. a3'),
			moves => {
				'b4d2' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P4N2/1P1bPPPP/R1BQKB1R w KQ - 0 6',
				'b4e7' => 'rnbq1rk1/ppppbppp/4pn2/8/2PP4/P4N2/1P1NPPPP/R1BQKB1R w KQ - 1 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/1bPPP3/2N5/PPQ2PPP/R1B1KBNR b KQ e3 0 5' => {
			eco => 'E32f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. e4

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. e4'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/2N2N2/PP1QPPPP/R3KB1R b KQ - 2 6' => {
			eco => 'E11y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+ 5. Qxd2 O-O 6. Nc3

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+ 5.Qxd2 O-O 6. Nc3'),
			moves => {
				'd7d5' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/2N2N2/PP1QPPPP/R3KB1R w KQ d6 0 7',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/5N2/PP1QPPPP/RN2KB1R w KQ - 1 6' => {
			eco => 'E11y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+ 5. Qxd2 O-O

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+ 5.Qxd2 O-O'),
			moves => {
				'b1c3' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/2N2N2/PP1QPPPP/R3KB1R b KQ - 2 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1P1P3/5PPP/R1BQKBNR b KQ - 0 6' => {
			eco => 'E28',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. e3

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. e3'),
			moves => {
				'b7b6' => 'rnbq1rk1/p1pp1ppp/1p2pn2/8/2PP4/P1P1P3/5PPP/R1BQKBNR w KQ - 0 7',
				'c7c5' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQ c6 0 7',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQ - 0 6' => {
			eco => 'E27',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O 6. f3

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O 6. f3'),
			moves => {
				'd7d5' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1P2P2/4P1PP/R1BQKBNR w KQ d6 0 7',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1P5/4PPPP/R1BQKBNR w KQ - 1 6' => {
			eco => 'E27',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 O-O

			variation => N__('Nimzo-Indian: Sämisch, 5...O-O'),
			moves => {
				'e2e3' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1P1P3/5PPP/R1BQKBNR b KQ - 0 6',
				'f2f3' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQ - 0 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1Q5/1P2PPPP/R1B1KBNR b KQ - 0 6' => {
			eco => 'E32h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+ 6. Qxc3

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3'),
			moves => {
				'b7b5' => 'rnbq1rk1/p1pp1ppp/4pn2/1p6/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ b6 0 7',
				'b7b6' => 'rnbq1rk1/p1pp1ppp/1p2pn2/8/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ - 0 7',
				'd7d6' => 'rnbq1rk1/ppp2ppp/3ppn2/8/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ - 0 7',
				'f6e4' => 'rnbq1rk1/pppp1ppp/4p3/8/2PPn3/P1Q5/1P2PPPP/R1B1KBNR w KQ - 1 7',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1b5/1PQ1PPPP/R1B1KBNR w KQ - 0 6' => {
			eco => 'E32h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 O-O 5. a3 Bxc3+

			variation => N__('Nimzo-Indian: Classical, 4...O-O 5. a3'),
			moves => {
				'c2c3' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1Q5/1P2PPPP/R1B1KBNR b KQ - 0 6',
			},
		},
		'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P4N2/1P1bPPPP/R1BQKB1R w KQ - 0 6' => {
			eco => 'E11f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 O-O 5. a3 Bxd2+

			variation => N__('Bogo-Indian: 4.Nbd2 O-O 5. a3 Bxd2+'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/5n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQ - 3 5' => {
			eco => 'C26g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Bc5 4. Bg2 O-O

			variation => N__('Vienna: Smyslov, 3... Bc5 4. Bg2 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/5n2/4p3/1bB1P3/2N2N2/PPPP1PPP/R1BQK2R w KQ - 6 5' => {
			eco => 'C26o',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bb4 4. Nf3 O-O

			variation => N__('Vienna: 3. Bc4 Bb4 4. Nf3 O-O'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/5n2/4p3/1bP1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQ e3 0 5' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 Bb4 4. Bg2 O-O 5. e4

			variation => N__('English: Bremen, Smyslov, 4. Bg2 O-O 5. e4'),
			moves => {
			},
		},
		'rnbq1rk1/pppp1ppp/5n2/4p3/1bP5/2N3P1/PP1PPPBP/R1BQK1NR w KQ - 3 5' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 Bb4 4. Bg2 O-O

			variation => N__('English: Bremen, Smyslov, 4. Bg2 O-O'),
			moves => {
				'e2e4' => 'rnbq1rk1/pppp1ppp/5n2/4p3/1bP1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQ e3 0 5',
			},
		},
		'rnbq1rk1/ppppb1pp/4p3/3P1p2/2P1n3/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O Ne4 7. d5

			variation => N__('Dutch: Alekhine, 7. d5'),
			moves => {
			},
		},
		'rnbq1rk1/ppppb1pp/4p3/5p2/2PPn3/5NP1/PP1NPPBP/R1BQ1RK1 b - - 7 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O Ne4 7. Nbd2

			variation => N__('Dutch: Alekhine, 7.Nbd2'),
			moves => {
			},
		},
		'rnbq1rk1/ppppb1pp/4p3/5p2/2PPn3/5NP1/PP2PPBP/RNBQ1RK1 w - - 6 7' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O Ne4

			variation => N__('Dutch: Alekhine Variation'),
			moves => {
				'b1d2' => 'rnbq1rk1/ppppb1pp/4p3/5p2/2PPn3/5NP1/PP1NPPBP/R1BQ1RK1 b - - 7 7',
				'd4d5' => 'rnbq1rk1/ppppb1pp/4p3/3P1p2/2P1n3/5NP1/PP2PPBP/RNBQ1RK1 b - - 0 7',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/3P1p2/2P5/5NP1/PP2PPBP/RNBQK2R b KQ - 0 6' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. d5'),
			moves => {
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b - - 5 6' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6 4. Bg2 Be7 5. O-O O-O 6. Nc3

			variation => N__('English: Anglo-Dutch'),
			moves => {
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w - - 4 6' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6 4. Bg2 Be7 5. O-O O-O

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/2N1P1P1/PP3PBP/R1BQK1NR b KQ - 0 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3 O-O 6. e3

			variation => N__('Dutch: Botvinnik-Bronstein Variation'),
			moves => {
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 5 6' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. Nc3'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQ - 4 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3 O-O

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3 O-O'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/2N1P1P1/PP3PBP/R1BQK1NR b KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6. O-O

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O 6.O-O'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp1pb1pp/2p1pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
				'd7d5' => 'rnbq1rk1/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - d6 0 7',
				'd7d6' => 'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
				'f6e4' => 'rnbq1rk1/ppppb1pp/4p3/5p2/2PPn3/5NP1/PP2PPBP/RNBQ1RK1 w - - 6 7',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6' => {
			eco => 'A92',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 O-O'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 5 6',
				'd4d5' => 'rnbq1rk1/ppppb1pp/4pn2/3P1p2/2P5/5NP1/PP2PPBP/RNBQK2R b KQ - 0 6',
				'e1g1' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/6PN/PP2PPBP/RNBQ1RK1 b - - 5 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6. O-O

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3 O-O 6.O-O'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1b1pp/3ppn2/5p2/2PP4/6PN/PP2PPBP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/6PN/PP2PPBP/RNBQK2R w KQ - 4 6' => {
			eco => 'A91',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3'),
			moves => {
				'e1g1' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/6PN/PP2PPBP/RNBQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppppbppp/4pn2/8/2PP4/P4N2/1P1NPPPP/R1BQKB1R w KQ - 1 6' => {
			eco => 'E11f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 O-O 5. a3 Be7

			variation => N__('Bogo-Indian: 4.Nbd2 O-O 5. a3 Be7'),
			moves => {
				'e2e4' => 'rnbq1rk1/ppppbppp/4pn2/8/2PPP3/P4N2/1P1N1PPP/R1BQKB1R b KQ e3 0 6',
			},
		},
		'rnbq1rk1/ppppbppp/4pn2/8/2PPP3/P4N2/1P1N1PPP/R1BQKB1R b KQ e3 0 6' => {
			eco => 'E11f',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: 4.Nbd2 O-O 5. a3 Be7'),
			moves => {
				'd7d5' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PPP3/P4N2/1P1N1PPP/R1BQKB1R w KQ d6 0 7',
			},
		},
		'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 5 6' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. Nc3

			variation => N__('Dutch: Leningrad, Main Line, 6. Nc3'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 0 7',
			},
		},
		'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQ - 4 6' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nc3 O-O

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nc3 O-O'),
			moves => {
				'g1h3' => 'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/2N3PN/PP2PPBP/R1BQK2R b KQ - 5 6',
			},
		},
		'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/2N3PN/PP2PPBP/R1BQK2R b KQ - 5 6' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nc3 O-O 6. Nh3

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nc3 O-O 6. Nh3'),
			moves => {
			},
		},
		'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O 6. O-O

			variation => N__('Dutch: Leningrad, Main Line, 6.O-O'),
			moves => {
				'c7c6' => 'rnbq1rk1/pp1pp1bp/2p2np1/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
				'd7d6' => 'rnbq1rk1/ppp1p1bp/3p1np1/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 0 7',
			},
		},
		'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 O-O

			variation => N__('Dutch: Leningrad, Main Line, 5...O-O'),
			moves => {
				'b1c3' => 'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQ - 5 6',
				'e1g1' => 'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b - - 5 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/4P3/2PP4/2N5/PP3PPP/R1BQKBNR b KQ - 0 5' => {
			eco => 'E70c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 O-O 5. e5

			variation => N__('King\'s Indian: 4. e4 O-O 5. e5'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/6B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQ - 0 5' => {
			eco => 'E61d',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 4. Bg5 O-O'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQ - 5 5' => {
			eco => 'E61s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bg5

			variation => N__('King\'s Indian: Smyslov System'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp1pppbp/5np1/2p3B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQ c6 0 6',
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/6B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQ - 4 5' => {
			eco => 'E61d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Bg5 O-O

			variation => N__('King\'s Indian: 4. Bg5 O-O'),
			moves => {
				'e2e3' => 'rnbq1rk1/ppppppbp/5np1/6B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQ - 0 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/6B1/3P4/2P2N2/PP1NPPPP/R2QKB1R b KQ - 0 5' => {
			eco => 'A48v',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. c3

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/3P4/2P2N2/PP1NPPPP/R2QKB1R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/6B1/3P4/5N2/PPPNPPPP/R2QKB1R w KQ - 4 5' => {
			eco => 'A48r',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'c2c3' => 'rnbq1rk1/ppppppbp/5np1/6B1/3P4/2P2N2/PP1NPPPP/R2QKB1R b KQ - 0 5',
				'e2e4' => 'rnbq1rk1/ppppppbp/5np1/6B1/3PP3/5N2/PPPN1PPP/R2QKB1R b KQ e3 0 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/6B1/3PP3/5N2/PPPN1PPP/R2QKB1R b KQ e3 0 5' => {
			eco => 'A48s',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2 O-O 5. e4

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'd7d5' => 'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3PP3/5N2/PPPN1PPP/R2QKB1R w KQ d6 0 6',
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/3PP3/5N2/PPPN1PPP/R2QKB1R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQ e3 0 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3 Bg7 4. Bg2 O-O 5. e4

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQ - 4 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3 Bg7 4. Bg2 O-O 5. Nf3

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQ - 3 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3 Bg7 4. Bg2 O-O

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
				'e2e4' => 'rnbq1rk1/ppppppbp/5np1/8/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQ e3 0 5',
				'g1f3' => 'rnbq1rk1/ppppppbp/5np1/8/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQ - 4 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2P5/5NP1/PP1PPPBP/RNBQK2R w KQ - 3 5' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2 O-O

			variation => N__('English: Anglo-Indian, 2. Nf3 g6'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQ - 5 5' => {
			eco => 'E61p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. Bf4

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. Bf4'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/1P3N2/P3PPPP/RNBQKB1R w KQ - 1 5' => {
			eco => 'E60h',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: b3 System'),
			moves => {
				'c1b2' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/1P3N2/PB2PPPP/RN1QKB1R b KQ - 2 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/1P3N2/PB2PPPP/RN1QKB1R b KQ - 2 5' => {
			eco => 'E60h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3 O-O 5. Bb2

			variation => N__('King\'s Indian: b3 System'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQ - 0 5' => {
			eco => 'E61m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. e3

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. e3'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQ - 4 5' => {
			eco => 'E61k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O

			variation => N__('King\'s Indian: 4. Nf3 O-O'),
			moves => {
				'c1f4' => 'rnbq1rk1/ppppppbp/5np1/8/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQ - 5 5',
				'c1g5' => 'rnbq1rk1/ppppppbp/5np1/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQ - 5 5',
				'e2e3' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQ - 0 5',
				'g2g3' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQ - 0 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQ - 0 5' => {
			eco => 'E61l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 O-O 5. g3

			variation => N__('King\'s Indian: 4. Nf3 O-O 5. g3'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQ - 1 5' => {
			eco => 'E61g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3 O-O

			variation => N__('King\'s Indian: 4. g3 O-O'),
			moves => {
				'f1g2' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQ - 2 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQ - 2 5' => {
			eco => 'E61g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3 O-O 5. Bg2

			variation => N__('King\'s Indian: 4. g3 O-O 5. Bg2'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQ - 1 5' => {
			eco => 'E60j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O

			variation => N__('King\'s Indian: Fianchetto'),
			moves => {
				'f1g2' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/5NP1/PP2PPBP/RNBQK2R b KQ - 2 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/5NP1/PP2PPBP/RNBQK2R b KQ - 2 5' => {
			eco => 'E60k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3 O-O 5. Bg2

			variation => N__('King\'s Indian: Fianchetto without Nc3'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PP4/6P1/PP2PPBP/RNBQK1NR w KQ - 3 5' => {
			eco => 'E60e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 Bg7 4. Bg2 O-O

			variation => N__('King\'s Indian: 3. g3'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQ - 2 5' => {
			eco => 'E70d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 O-O 5. Nf3

			variation => N__('King\'s Indian: 4. e4 O-O 5. Nf3'),
			moves => {
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQ - 1 5' => {
			eco => 'E70b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 O-O

			variation => N__('King\'s Indian: 4. e4 O-O'),
			moves => {
				'e4e5' => 'rnbq1rk1/ppppppbp/5np1/4P3/2PP4/2N5/PP3PPP/R1BQKBNR b KQ - 0 5',
				'g1f3' => 'rnbq1rk1/ppppppbp/5np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQ - 2 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/3P1B2/4PN2/PPP1BPPP/RN1QK2R b KQ - 2 5' => {
			eco => 'A48k',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. e3 O-O 5. Be2

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3P1B2/4PN2/PPP1BPPP/RN1QK2R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQ - 1 5' => {
			eco => 'A48j',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. e3 O-O

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
				'f1e2' => 'rnbq1rk1/ppppppbp/5np1/8/3P1B2/4PN2/PPP1BPPP/RN1QK2R b KQ - 2 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/3P4/1P3NP1/P1P1PPBP/RNBQK2R b KQ - 0 5' => {
			eco => 'A49',
                        # TRANSLATORS:

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3P4/1P3NP1/P1P1PPBP/RNBQK2R w KQ - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b - - 4 5' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2 O-O 5. O-O

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3P4/5NP1/PPP1PPBP/RNBQ1RK1 w - - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/3P4/5NP1/PPP1PPBP/RNBQK2R w KQ - 3 5' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2 O-O

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'b2b3' => 'rnbq1rk1/ppppppbp/5np1/8/3P4/1P3NP1/P1P1PPBP/RNBQK2R b KQ - 0 5',
				'e1g1' => 'rnbq1rk1/ppppppbp/5np1/8/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b - - 4 5',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b - - 0 5' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2 Bg7 4. O-O O-O 5. d3

			variation => N__('Reti: KIA'),
			moves => {
				'c7c5' => 'rnbq1rk1/pp1pppbp/5np1/2p5/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w - c6 0 6',
				'd7d6' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w - - 0 6',
			},
		},
		'rnbq1rk1/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQ1RK1 w - - 4 5' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2 Bg7 4. O-O O-O

			variation => N__('Reti: KIA'),
			moves => {
				'd2d3' => 'rnbq1rk1/ppppppbp/5np1/8/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b - - 0 5',
			},
		},
		'rnbq3r/pp1nkpbp/3N2p1/2pPp3/5P2/8/PP4PP/R1BQKBNR w KQ - 2 11' => {
			eco => 'A66',
                        # TRANSLATORS:

			variation => N__('Benoni: Mikenas Attack, 9. Nb5 dxe5 10. Nd6+'),
			moves => {
				'd6c8' => 'rnNq3r/pp1nkpbp/6p1/2pPp3/5P2/8/PP4PP/R1BQKBNR b KQ - 0 11',
			},
		},
		'rnbq3r/pp1nkppp/4p3/2ppP3/3P3P/2N5/PPP2PP1/R2QKBNR w KQ - 0 8' => {
			eco => 'C13r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 c5 7. Bxe7 Kxe7

			variation => N__('French: Chatard-Alekhine, Breyer, 7.Bxe7 Kxe7'),
			moves => {
			},
		},
		'rnbq3r/pppp1k2/3b1n1p/8/4PQ1P/8/PPPP2P1/RNB1KB1R w KQ - 1 9' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 h6 6. Nxf7 Kxf7 7. Qxg4 Nf6 8. Qxf4 Bd6

			variation => N__('KGA: Allgaier, Horny Defence'),
			moves => {
			},
		},
		'rnbqk1n1/pppp1p1r/7b/4N2p/2BPPppP/2N5/PPP3P1/R1BQK2R b KQq - 2 8' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 h5 6. Bc4 Rh7 7. d4 Bh6 8. Nc3

			variation => N__('KGA: Kieseritsky, Long Whip Defence, Jänisch Variation'),
			moves => {
			},
		},
		'rnbqk1n1/pppp1p1r/7b/4N2p/2BPPppP/8/PPP3P1/RNBQK2R w KQq - 1 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Long Whip Defence'),
			moves => {
				'b1c3' => 'rnbqk1n1/pppp1p1r/7b/4N2p/2BPPppP/2N5/PPP3P1/R1BQK2R b KQq - 2 8',
			},
		},
		'rnbqk1nR/pp3pb1/2pp4/6p1/2BPPp2/2N2N2/PPP3P1/R1BQK3 b Qq - 0 9' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'g7h8' => 'rnbqk1nb/pp3p2/2pp4/6p1/2BPPp2/2N2N2/PPP3P1/R1BQK3 w Qq - 0 10',
			},
		},
		'rnbqk1nb/pp3p2/2pp4/4N1p1/2BPPp2/2N5/PPP3P1/R1BQK3 b Qq - 1 10' => {
			eco => 'C38',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Bg7 5. h4 h6 6. d4 d6 7. Nc3 c6 8. hxg5 hxg5 9. Rxh8 Bxh8 10. Ne5

			variation => N__('KGA: Greco Gambit'),
			moves => {
			},
		},
		'rnbqk1nb/pp3p2/2pp4/6p1/2BPPp2/2N2N2/PPP3P1/R1BQK3 w Qq - 0 10' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'f3e5' => 'rnbqk1nb/pp3p2/2pp4/4N1p1/2BPPp2/2N5/PPP3P1/R1BQK3 b Qq - 1 10',
			},
		},
		'rnbqk1nr/1p1p1ppp/p3p3/2b5/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B42f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Bc5

			variation => N__('Sicilian: Kan, Polugaevsky Variation'),
			moves => {
				'd4b3' => 'rnbqk1nr/1p1p1ppp/p3p3/2b5/4P3/1N1B4/PPP2PPP/RNBQK2R b KQkq - 3 6',
			},
		},
		'rnbqk1nr/1p1p1ppp/p3p3/2b5/4P3/1N1B4/PPP2PPP/RNBQK2R b KQkq - 3 6' => {
			eco => 'B42f',
                        # TRANSLATORS:

			variation => N__('Sicilian: Kan, Polugaevsky Variation'),
			moves => {
				'c5a7' => 'rnbqk1nr/bp1p1ppp/p3p3/8/4P3/1N1B4/PPP2PPP/RNBQK2R w KQkq - 4 7',
			},
		},
		'rnbqk1nr/1p2bppp/p2p4/2pPp3/P3P3/2N5/1PP2PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'A44u',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 a6 5. a4 Be7

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 a6 5. a4 Be7'),
			moves => {
			},
		},
		'rnbqk1nr/1pp1ppbp/p2p2p1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'B06p',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Be3 a6

			variation => N__('Modern: 3. Nc3 d6 4. Be3 a6'),
			moves => {
			},
		},
		'rnbqk1nr/1ppp1ppp/4p3/p7/1bPP4/8/PP1BPPPP/RN1QKBNR w KQkq a6 0 4' => {
			eco => 'A40r',
                        # TRANSLATORS: 1. d4 e6 2. c4 Bb4+ 3. Bd2 a5

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 a5'),
			moves => {
			},
		},
		'rnbqk1nr/1ppp1ppp/8/p2Np3/1bP5/8/PP1PPPPP/R1BQKBNR w KQkq a6 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. Nd5 a5

			variation => N__('English: Kramnik-Shirov, 3. Nd5 a5'),
			moves => {
			},
		},
		'rnbqk1nr/5ppp/2p1p3/pP6/1bpP4/2N1PN2/1P1B1PPP/R2QKB1R b KQkq - 0 8' => {
			eco => 'D31t',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams Variation'),
			moves => {
				'b4c3' => 'rnbqk1nr/5ppp/2p1p3/pP6/2pP4/2b1PN2/1P1B1PPP/R2QKB1R w KQkq - 0 9',
			},
		},
		'rnbqk1nr/5ppp/2p1p3/pP6/2pP4/2B1PN2/1P3PPP/R2QKB1R b KQkq - 0 9' => {
			eco => 'D31t',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams Variation'),
			moves => {
				'c6b5' => 'rnbqk1nr/5ppp/4p3/pp6/2pP4/2B1PN2/1P3PPP/R2QKB1R w KQkq - 0 10',
			},
		},
		'rnbqk1nr/5ppp/2p1p3/pP6/2pP4/2b1PN2/1P1B1PPP/R2QKB1R w KQkq - 0 9' => {
			eco => 'D31t',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams Variation'),
			moves => {
				'd2c3' => 'rnbqk1nr/5ppp/2p1p3/pP6/2pP4/2B1PN2/1P3PPP/R2QKB1R b KQkq - 0 9',
			},
		},
		'rnbqk1nr/5ppp/2p1p3/pp6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R w KQkq a6 0 8' => {
			eco => 'D31t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4 Bb4 6. e3 b5 7. Bd2 a5

			variation => N__('Semi-Slav: Noteboom, Abrahams Variation'),
			moves => {
				'a4b5' => 'rnbqk1nr/5ppp/2p1p3/pP6/1bpP4/2N1PN2/1P1B1PPP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rnbqk1nr/5ppp/4p3/pp6/2pP4/1PB1PN2/5PPP/R2QKB1R b KQkq - 0 10' => {
			eco => 'D31t',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams Variation'),
			moves => {
				'c8b7' => 'rn1qk1nr/1b3ppp/4p3/pp6/2pP4/1PB1PN2/5PPP/R2QKB1R w KQkq - 1 11',
			},
		},
		'rnbqk1nr/5ppp/4p3/pp6/2pP4/2B1PN2/1P3PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'D31t',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, Abrahams Variation'),
			moves => {
				'b2b3' => 'rnbqk1nr/5ppp/4p3/pp6/2pP4/1PB1PN2/5PPP/R2QKB1R b KQkq - 0 10',
			},
		},
		'rnbqk1nr/bp1p1ppp/p3p3/8/4P3/1N1B4/PPP2PPP/RNBQK2R w KQkq - 4 7' => {
			eco => 'B42g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Bc5 6. Nb3 Ba7

			variation => N__('Sicilian: Kan, Polugaevsky, 6. Nb3 Ba7'),
			moves => {
			},
		},
		'rnbqk1nr/p1p1bppp/1p2p3/3pP3/3P4/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C13e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Ng8 6. Be3 b6

			variation => N__('French: Classical, Frankfurt Variation'),
			moves => {
			},
		},
		'rnbqk1nr/p1p2ppp/1p2p3/3pP3/1b1P2Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 1 5' => {
			eco => 'C16c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 b6 5. Qg4

			variation => N__('French: Winawer, 4. e5 b6 5. Qg4'),
			moves => {
				'b4f8' => 'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P2Q1/2N5/PPP2PPP/R1B1KBNR w KQkq - 2 6',
			},
		},
		'rnbqk1nr/p1p2ppp/1p2p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C16b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 b6

			variation => N__('French: Winawer, 4. e5 b6'),
			moves => {
				'a2a3' => 'rnbqk1nr/p1p2ppp/1p2p3/3pP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5',
				'd1g4' => 'rnbqk1nr/p1p2ppp/1p2p3/3pP3/1b1P2Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 1 5',
			},
		},
		'rnbqk1nr/p1p2ppp/1p2p3/3pP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C16e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 b6 5. a3

			variation => N__('French: Winawer, 4. e5 b6 5. a3'),
			moves => {
				'b4f8' => 'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P4/P1N5/1PP2PPP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqk1nr/p1pp1ppp/1p2p3/8/2PP4/2N5/PP1QPPPP/R3KBNR b KQkq - 1 5' => {
			eco => 'A40q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 Bxd2+'),
			moves => {
				'c8b7' => 'rn1qk1nr/pbpp1ppp/1p2p3/8/2PP4/2N5/PP1QPPPP/R3KBNR w KQkq - 2 6',
			},
		},
		'rnbqk1nr/p1pp1ppp/1p2p3/8/2PP4/8/PP1QPPPP/RN2KBNR w KQkq - 0 5' => {
			eco => 'A40q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 Bxd2+'),
			moves => {
				'b1c3' => 'rnbqk1nr/p1pp1ppp/1p2p3/8/2PP4/2N5/PP1QPPPP/R3KBNR b KQkq - 1 5',
			},
		},
		'rnbqk1nr/p4ppp/2p1p3/1p6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'D31s',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, 5. a4 Bb4 6. e3 b5'),
			moves => {
				'a7a5' => 'rnbqk1nr/5ppp/2p1p3/pp6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R w KQkq a6 0 8',
				'd8b6' => 'rnb1k1nr/p4ppp/1qp1p3/1p6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R w KQkq - 2 8',
				'd8e7' => 'rnb1k1nr/p3qppp/2p1p3/1p6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R w KQkq - 2 8',
			},
		},
		'rnbqk1nr/p4ppp/2p1p3/1p6/PbpP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq b6 0 7' => {
			eco => 'D31s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4 Bb4 6. e3 b5

			variation => N__('Semi-Slav: Noteboom, 5. a4 Bb4 6. e3 b5'),
			moves => {
				'c1d2' => 'rnbqk1nr/p4ppp/2p1p3/1p6/PbpP4/2N1PN2/1P1B1PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqk1nr/pp1p1ppp/4p3/2b5/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B40m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Bc5

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 Bc5'),
			moves => {
			},
		},
		'rnbqk1nr/pp1pp1bp/2p3p1/5p2/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'A81',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. g3'),
			moves => {
				'g8h6' => 'rnbqk2r/pp1pp1bp/2p3pn/5p2/3P4/5NP1/PPP1PPBP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'rnbqk1nr/pp1pp1bp/2p3p1/5p2/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A81',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. g3'),
			moves => {
				'e1g1' => 'rnbqk1nr/pp1pp1bp/2p3p1/5p2/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'rnbqk1nr/pp1pp2p/6p1/2pP1p2/2P5/2P5/P3PPPP/R1BQKBNR w KQkq f6 0 6' => {
			eco => 'A40y',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. Nc3 c5 4. d5 Bxc3+ 5. bxc3 f5

			variation => N__('Queen\'s Pawn: Modern, Beefeater Defence'),
			moves => {
			},
		},
		'rnbqk1nr/pp1ppp1p/6p1/2pP4/2P5/2P5/P3PPPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A40x',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'f7f5' => 'rnbqk1nr/pp1pp2p/6p1/2pP1p2/2P5/2P5/P3PPPP/R1BQKBNR w KQkq f6 0 6',
			},
		},
		'rnbqk1nr/pp1ppp1p/6p1/2pP4/2P5/2b5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A40x',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'b2c3' => 'rnbqk1nr/pp1ppp1p/6p1/2pP4/2P5/2P5/P3PPPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2P5/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B27n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. dxc5

			variation => N__('Sicilian: Accelerated Pterodactyl, 4.dxc5'),
			moves => {
				'd8a5' => 'rnb1k1nr/pp1pppbp/6p1/q1P5/4P3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A34',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 2... g6'),
			moves => {
				'f1g2' => 'rnbqk1nr/pp1pppbp/6p1/2p5/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 g6 3. g3 Bg7 4. Bg2

			variation => N__('English: Symmetrical, 2... g6'),
			moves => {
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'A40x',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. Nc3 c5

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'd4d5' => 'rnbqk1nr/pp1pppbp/6p1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A40w',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. Nf3 c5

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/2PPP3/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 4' => {
			eco => 'B27s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. c4

			variation => N__('Sicilian: Accelerated Pterodactyl, 4. c4'),
			moves => {
				'd8a5' => 'rnb1k1nr/pp1pppbp/6p1/q1p5/2PPP3/5N2/PP3PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 4' => {
			eco => 'B27t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7 4. Nc3

			variation => N__('Sicilian: Accelerated Pterodactyl, 4. Nc3'),
			moves => {
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'B06l',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 c5

			variation => N__('Modern: Nc3 Pterodactyl'),
			moves => {
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'B27g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Hungarian, 3. c3'),
			moves => {
				'c5d4' => 'rnbqk1nr/pp1pppbp/6p1/8/3pP3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B27m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 Bg7

			variation => N__('Sicilian: Accelerated Pterodactyl'),
			moves => {
				'b1c3' => 'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 4',
				'c2c4' => 'rnbqk1nr/pp1pppbp/6p1/2p5/2PPP3/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 4',
				'd4c5' => 'rnbqk1nr/pp1pppbp/6p1/2P5/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B27g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Hungarian, 3. c3'),
			moves => {
				'd2d4' => 'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq d3 0 4',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/4P3/6P1/PPPP1PBP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. g3 g6 3. Bg2 Bg7

			variation => N__('Sicilian: 2. g3'),
			moves => {
				'f2f4' => 'rnbqk1nr/pp1pppbp/6p1/2p5/4PP2/6P1/PPPP2BP/RNBQK1NR b KQkq f3 0 4',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/4PP2/6P1/PPPP2BP/RNBQK1NR b KQkq f3 0 4' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. g3 g6 3. Bg2 Bg7 4. f4

			variation => N__('Sicilian: 2. g3'),
			moves => {
				'd7d6' => 'rnbqk1nr/pp2ppbp/3p2p1/2p5/4PP2/6P1/PPPP2BP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: 1... c5'),
			moves => {
				'b8c6' => 'r1bqk1nr/pp1pppbp/2n3p1/2p5/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 4 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2p5/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6 3. Bg2 Bg7

			variation => N__('Reti: 1... c5'),
			moves => {
				'e1g1' => 'rnbqk1nr/pp1pppbp/6p1/2p5/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A40x',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. Nc3 c5 4. d5

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'g7c3' => 'rnbqk1nr/pp1ppp1p/6p1/2pP4/2P5/2b5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/8/3PP3/5N2/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B27h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. c3 Bg7 4. d4 cxd4 5. cxd4

			variation => N__('Sicilian: Hungarian, 3. c3 Bg7 4. d4 cxd4 5.cxd4'),
			moves => {
				'd7d5' => 'rnbqk1nr/pp2ppbp/6p1/3p4/3PP3/5N2/PP3PPP/RNBQKB1R w KQkq d6 0 6',
			},
		},
		'rnbqk1nr/pp1pppbp/6p1/8/3pP3/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B27g',
                        # TRANSLATORS:

			variation => N__('Sicilian: Hungarian, 3. c3'),
			moves => {
				'c3d4' => 'rnbqk1nr/pp1pppbp/6p1/8/3PP3/5N2/PP3PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp2bppp/2p5/3p4/3P1B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D31f',
                        # TRANSLATORS:

			variation => N__('QGD: Alatortsev, 5. Bf4 c6'),
			moves => {
				'c8f5' => 'rn1qk1nr/pp2bppp/2p5/3p1b2/3P1B2/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'rnbqk1nr/pp2bppp/2p5/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D31f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. cxd5 exd5 5. Bf4 c6

			variation => N__('QGD: Alatortsev, 5. Bf4 c6'),
			moves => {
				'e2e3' => 'rnbqk1nr/pp2bppp/2p5/3p4/3P1B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp2bppp/3p4/1BpPp3/4P3/2N5/PPP2PPP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'A44w',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 Be7 5. Bb5+

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 Be7 5. Bb5+'),
			moves => {
			},
		},
		'rnbqk1nr/pp2bppp/3p4/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'A44g',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 Be7 5. Nc3

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 Be7 5. Nc3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2bppp/3p4/2pPp3/2P1P3/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'A44f',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 Be7

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 Be7'),
			moves => {
				'b1c3' => 'rnbqk1nr/pp2bppp/3p4/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5',
			},
		},
		'rnbqk1nr/pp2bppp/3p4/2pPp3/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 3 5' => {
			eco => 'A44x',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 Be7 5. Nf3

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 Be7 5. Nf3'),
			moves => {
				'c8g4' => 'rn1qk1nr/pp2bppp/3p4/2pPp3/4P1b1/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 4 6',
			},
		},
		'rnbqk1nr/pp2bppp/3p4/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'A44v',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 Be7

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 Be7'),
			moves => {
				'f1b5' => 'rnbqk1nr/pp2bppp/3p4/1BpPp3/4P3/2N5/PPP2PPP/R1BQK1NR b KQkq - 3 5',
				'g1f3' => 'rnbqk1nr/pp2bppp/3p4/2pPp3/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 3 5',
			},
		},
		'rnbqk1nr/pp2bppp/4p3/2Pp4/4P3/3B4/PPPN1PPP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'C03h',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, 3... Be7 4. Bd3 c5'),
			moves => {
				'g8f6' => 'rnbqk2r/pp2bppp/4pn2/2Pp4/4P3/3B4/PPPN1PPP/R1BQK1NR w KQkq - 1 6',
			},
		},
		'rnbqk1nr/pp2bppp/4p3/2pp4/3PP3/3B4/PPPN1PPP/R1BQK1NR w KQkq c6 0 5' => {
			eco => 'C03h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7 4. Bd3 c5

			variation => N__('French: Tarrasch, 3... Be7 4. Bd3 c5'),
			moves => {
				'd4c5' => 'rnbqk1nr/pp2bppp/4p3/2Pp4/4P3/3B4/PPPN1PPP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp2ppb1/2p3p1/3pP2p/3P1P2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B15i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. e5 Bg7 5. f4 h5 6. Nf3

			variation => N__('Caro-Kann: Gurgenidze: 4. e5 Bg7 5. f4 h5 6. Nf3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppb1/2p3p1/3pP2p/3P1P2/2N5/PPP3PP/R1BQKBNR w KQkq h6 0 6' => {
			eco => 'B15h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. e5 Bg7 5. f4 h5

			variation => N__('Caro-Kann: Gurgenidze: 4. e5 Bg7 5. f4 h5'),
			moves => {
				'g1f3' => 'rnbqk1nr/pp2ppb1/2p3p1/3pP2p/3P1P2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3P4/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B15k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. exd5

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5.exd5'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 2 5' => {
			eco => 'B15d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Be3 Bg7

			variation => N__('Caro-Kann: Gurgenidze: 4. Be3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B15l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. h3

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5. h3'),
			moves => {
				'd5e4' => 'rnbqk1nr/pp2ppbp/2p3p1/8/3Pp3/2N2N1P/PPP2PP1/R1BQKB1R w KQkq - 0 6',
				'g8f6' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/3PP3/2N2N1P/PPP2PP1/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B15k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7'),
			moves => {
				'e4d5' => 'rnbqk1nr/pp2ppbp/2p3p1/3P4/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5',
				'e4e5' => 'rnbqk1nr/pp2ppbp/2p3p1/3pP3/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5',
				'h2h3' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N4P/PPP2PP1/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B15e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. h3 Bg7

			variation => N__('Caro-Kann: Gurgenidze: 4. h3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2P2N2/PP1N1PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B12h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 g6 4. Ngf3 Bg7 5. c3

			variation => N__('Caro-Kann: Gurgenidze/Modern: 3. Nd2 g6 4.Ngf3 Bg7 5. c3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/5N1P/PPPN1PP1/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B12h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 g6 4. Ngf3 Bg7 5. h3

			variation => N__('Caro-Kann: Gurgenidze/Modern: 3. Nd2 g6 4.Ngf3 Bg7 5. h3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B12h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 g6 4. Ngf3 Bg7

			variation => N__('Caro-Kann: Gurgenidze/Modern: 3. Nd2 g6 4.Ngf3 Bg7'),
			moves => {
				'c2c3' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2P2N2/PP1N1PPP/R1BQKB1R b KQkq - 0 5',
				'h2h3' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/5N1P/PPPN1PP1/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3pP3/3P1P2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 5' => {
			eco => 'B15g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. e5 Bg7 5. f4

			variation => N__('Caro-Kann: Gurgenidze: 4. e5 Bg7 5. f4'),
			moves => {
				'g8h6' => 'rnbqk2r/pp2ppbp/2p3pn/3pP3/3P1P2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 6',
				'h7h5' => 'rnbqk1nr/pp2ppb1/2p3p1/3pP2p/3P1P2/2N5/PPP3PP/R1BQKBNR w KQkq h6 0 6',
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3pP3/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B15k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. e5

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5. e5'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B15f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. e5 Bg7

			variation => N__('Caro-Kann: Gurgenidze: 4. e5'),
			moves => {
				'f2f4' => 'rnbqk1nr/pp2ppbp/2p3p1/3pP3/3P1P2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 5',
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/8/3PN3/5N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B15m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. h3 dxe4 6. Nxe4

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5. h3 dxe4'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2p3p1/8/3Pp3/2N2N1P/PPP2PP1/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B15m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. h3 dxe4

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5. h3 dxe4'),
			moves => {
				'c3e4' => 'rnbqk1nr/pp2ppbp/2p3p1/8/3PN3/5N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/6B1/3PP3/2N2N2/PPP2PPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'B06r',
                        # TRANSLATORS:

			variation => N__('Modern: Two Knights, Suttles Variation'),
			moves => {
				'd8b6' => 'rnb1k1nr/pp2ppbp/1qpp2p1/6B1/3PP3/2N2N2/PPP2PPP/R2QKB1R w KQkq - 2 6',
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/2BPP3/2N2Q2/PPP2PPP/R1B1K1NR b KQkq - 1 5' => {
			eco => 'B06n',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Bc4 c6 5. Qf3

			variation => N__('Modern: 3. Nc3 d6 4. Bc4'),
			moves => {
				'e7e6' => 'rnbqk1nr/pp3pbp/2ppp1p1/8/2BPP3/2N2Q2/PPP2PPP/R1B1K1NR w KQkq - 0 6',
				'g8f6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/2BPP3/2N2Q2/PPP2PPP/R1B1K1NR w KQkq - 2 6',
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'B06n',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Bc4 c6

			variation => N__('Modern: 3. Nc3 d6 4. Bc4'),
			moves => {
				'd1f3' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/2BPP3/2N2Q2/PPP2PPP/R1B1K1NR b KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/2PPP3/2N1B3/PP3PPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'A42d',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 c6 5. Be3

			variation => N__('Modern: Averbakh, 4... c6 5. Be3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A42e',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 c6 5. Nf3

			variation => N__('Modern: Averbakh, 4... c6 5. Nf3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A42d',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 c6

			variation => N__('Modern: Averbakh, 4... c6'),
			moves => {
				'c1e3' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/2PPP3/2N1B3/PP3PPP/R2QKBNR b KQkq - 1 5',
				'g1f3' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'B06q',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Be3 c6

			variation => N__('Modern: 3. Nc3 d6 4. Be3 c6'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'B06r',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Nf3 c6

			variation => N__('Modern: Two Knights, Suttles Variation'),
			moves => {
				'c1g5' => 'rnbqk1nr/pp2ppbp/2pp2p1/6B1/3PP3/2N2N2/PPP2PPP/R2QKB1R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/3PPP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'B06t',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. f4 c6 5. Nf3

			variation => N__('Modern: Pseudo-Austrian Attack'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/2pp2p1/8/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B06s',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. f4 c6

			variation => N__('Modern: Pseudo-Austrian Attack'),
			moves => {
				'g1f3' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/3PPP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2p5/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A42c',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 c5 5. Nf3

			variation => N__('Modern: c4 Pterodactyl'),
			moves => {
				'd8a5' => 'rnb1k1nr/pp2ppbp/3p2p1/q1p5/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2p5/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq c6 0 5' => {
			eco => 'A42c',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 c5

			variation => N__('Modern: c4 Pterodactyl'),
			moves => {
				'g1f3' => 'rnbqk1nr/pp2ppbp/3p2p1/2p5/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2p5/3PPP2/2P5/PP4PP/RNBQKBNR w KQkq c6 0 5' => {
			eco => 'B06g',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. c3 d6 4. f4 c5

			variation => N__('Modern: c3 Pterodactyl'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2p5/4PP2/6P1/PPPP2BP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. g3 g6 3. Bg2 Bg7 4. f4 d6

			variation => N__('Sicilian: 2. g3'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'A43t',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 g6 4. Nc3 Bg7

			variation => N__('Old Benoni: Schmidt, 3. e4 g6'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2pP4/4P3/5N2/PPP1BPPP/RNBQK2R b KQkq - 3 5' => {
			eco => 'A43t',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 g6 4. Nf3 Bg7 5. Be2

			variation => N__('Old Benoni: Schmidt, 3. e4 g6'),
			moves => {
			},
		},
		'rnbqk1nr/pp2ppbp/3p2p1/2pP4/4P3/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'A43t',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 g6 4. Nf3 Bg7

			variation => N__('Old Benoni: Schmidt, 3. e4 g6'),
			moves => {
				'f1e2' => 'rnbqk1nr/pp2ppbp/3p2p1/2pP4/4P3/5N2/PPP1BPPP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqk1nr/pp2ppbp/6p1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 g6 4. O-O Bg7 5. d3

			variation => N__('Reti: KIA, 2... c5 + 3... g6'),
			moves => {
				'g8f6' => 'rnbqk2r/pp2ppbp/5np1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'rnbqk1nr/pp2ppbp/6p1/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 g6 4. O-O Bg7

			variation => N__('Reti: KIA, 2... c5 + 3... g6'),
			moves => {
				'd2d3' => 'rnbqk1nr/pp2ppbp/6p1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'rnbqk1nr/pp2ppbp/6p1/3p4/3PP3/5N2/PP3PPP/RNBQKB1R w KQkq d6 0 6' => {
			eco => 'B27i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. c3 Bg7 4. d4 cxd4 5. cxd4 d5

			variation => N__('Sicilian: Hungarian, 3. c3 Bg7 4. d4 cxd4 5.cxd4 d5'),
			moves => {
			},
		},
		'rnbqk1nr/pp3pb1/2pp3p/6P1/2BPPp2/2N2N2/PPP3P1/R1BQK2R b KQkq - 0 8' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'h6g5' => 'rnbqk1nr/pp3pb1/2pp4/6p1/2BPPp2/2N2N2/PPP3P1/R1BQK2R w KQkq - 0 9',
			},
		},
		'rnbqk1nr/pp3pb1/2pp3p/6p1/2BPPp1P/2N2N2/PPP3P1/R1BQK2R w KQkq - 0 8' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'h4g5' => 'rnbqk1nr/pp3pb1/2pp3p/6P1/2BPPp2/2N2N2/PPP3P1/R1BQK2R b KQkq - 0 8',
			},
		},
		'rnbqk1nr/pp3pb1/2pp4/6p1/2BPPp2/2N2N2/PPP3P1/R1BQK2R w KQkq - 0 9' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'h1h8' => 'rnbqk1nR/pp3pb1/2pp4/6p1/2BPPp2/2N2N2/PPP3P1/R1BQK3 b Qq - 0 9',
			},
		},
		'rnbqk1nr/pp3pbp/2ppp1p1/8/2BPP3/2N2Q2/PPP2PPP/R1B1K1NR w KQkq - 0 6' => {
			eco => 'B06n',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Bc4 c6 5. Qf3 e6

			variation => N__('Modern: 3. Nc3 d6 4. Bc4'),
			moves => {
			},
		},
		'rnbqk1nr/pp3pbp/3p2p1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'A43i',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4 exd5 4. cxd5 d6 5. Nc3 g6 6. e4 Bg7 7. Nf3

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
			},
		},
		'rnbqk1nr/pp3pbp/3p2p1/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'A43i',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4 exd5 4. cxd5 d6 5. Nc3 g6 6. e4 Bg7

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'g1f3' => 'rnbqk1nr/pp3pbp/3p2p1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 7',
			},
		},
		'rnbqk1nr/pp3pbp/3p2p1/2pPp3/2P1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 3 6' => {
			eco => 'A44j',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 g6 5. Nc3 Bg7 6. Nf3

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 g6 5. Nc3 Bg7 6. Nf3'),
			moves => {
			},
		},
		'rnbqk1nr/pp3pbp/3p2p1/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 2 6' => {
			eco => 'A44i',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 g6 5. Nc3 Bg7

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 g6 5. Nc3 Bg7'),
			moves => {
				'f1d3' => 'rnbqk1nr/pp3pbp/3p2p1/2pPp3/2P1P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 3 6',
				'g1f3' => 'rnbqk1nr/pp3pbp/3p2p1/2pPp3/2P1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 3 6',
			},
		},
		'rnbqk1nr/pp3pbp/3p2p1/2pPp3/2P1P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 3 6' => {
			eco => 'A44k',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 g6 5. Nc3 Bg7 6. Bd3

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 g6 5. Nc3 Bg7 6. Bd3'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/2p1p3/3p4/1bPPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'D31w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 Bb4

			variation => N__('Semi-Slav: Marshall Gambit, 4... Bb4'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/2p1p3/8/1bPP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 6' => {
			eco => 'D31x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Bb4+ 6. Nc3

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Bb4+ 6. Nc3'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/2p1p3/8/1bPPN3/8/PP1B1PPP/R2QKBNR b KQkq - 2 6' => {
			eco => 'D31x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Bb4+ 6. Bd2

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Bb4+ 6. Bd2'),
			moves => {
				'd8d4' => 'rnb1k1nr/pp3ppp/2p1p3/8/1bPqN3/8/PP1B1PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk1nr/pp3ppp/2p1p3/8/1bPPN3/8/PP3PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'D31x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Bb4+

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Bb4+'),
			moves => {
				'c1d2' => 'rnbqk1nr/pp3ppp/2p1p3/8/1bPPN3/8/PP1B1PPP/R2QKBNR b KQkq - 2 6',
				'e4c3' => 'rnbqk1nr/pp3ppp/2p1p3/8/1bPP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 6',
			},
		},
		'rnbqk1nr/pp3ppp/2p1p3/8/PbpP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D31r',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, 5. a4'),
			moves => {
				'b7b5' => 'rnbqk1nr/p4ppp/2p1p3/1p6/PbpP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq b6 0 7',
			},
		},
		'rnbqk1nr/pp3ppp/2p1p3/8/PbpP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D31r',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Noteboom, 5. a4'),
			moves => {
				'e2e3' => 'rnbqk1nr/pp3ppp/2p1p3/8/PbpP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp3ppp/2pb4/3pp3/4P3/3P1N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B10e',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Breyer, 3. Nd2 e5'),
			moves => {
				'g2g3' => 'rnbqk1nr/pp3ppp/2pb4/3pp3/4P3/3P1NP1/PPPN1P1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp3ppp/2pb4/3pp3/4P3/3P1NP1/PPPN1P1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B10f',
                        # TRANSLATORS: 1. e4 c6 2. d3 d5 3. Nd2 e5 4. Ngf3 Bd6 5. g3

			variation => N__('Caro-Kann: Breyer, Main Line'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2PpP3/1b6/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C17b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. dxc5

			variation => N__('French: Winawer, Advance, 5.dxc5'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2p5/3Pp3/P1P2P2/2P3PP/R1BQKBNR w KQkq c6 0 7' => {
			eco => 'C15l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 dxe4 6. f3 c5

			variation => N__('French: Winawer, Winkelmann-Reimer, 6... c5'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2pP4/1b1P4/2NB4/PPP2PPP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'C15f',
                        # TRANSLATORS:

			variation => N__('French: Winawer, 4. Bd3 c5'),
			moves => {
				'd8d5' => 'rnb1k1nr/pp3ppp/4p3/2pq4/1b1P4/2NB4/PPP2PPP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2pp4/1b1PP3/2NB4/PPP2PPP/R1BQK1NR w KQkq c6 0 5' => {
			eco => 'C15f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd3 c5

			variation => N__('French: Winawer, 4. Bd3 c5'),
			moves => {
				'e4d5' => 'rnbqk1nr/pp3ppp/4p3/2pP4/1b1P4/2NB4/PPP2PPP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P2Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 1 5' => {
			eco => 'C17c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Qg4

			variation => N__('French: Winawer, Russian Variation'),
			moves => {
				'g8e7' => 'rnbqk2r/pp2nppp/4p3/2ppP3/1b1P2Q1/2N5/PPP2PPP/R1B1KBNR w KQkq - 2 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq c6 0 5' => {
			eco => 'C17a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5

			variation => N__('French: Winawer, Advance, 4... c5'),
			moves => {
				'a2a3' => 'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5',
				'c1d2' => 'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 1 5',
				'd1g4' => 'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P2Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 1 5',
				'd4c5' => 'rnbqk1nr/pp3ppp/4p3/2PpP3/1b6/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'C17e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Bd2

			variation => N__('French: Winawer, Bogoljubow Variation'),
			moves => {
				'c5d4' => 'rnbqk1nr/pp3ppp/4p3/3pP3/1b1p4/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 6',
				'g8e7' => 'rnbqk2r/pp2nppp/4p3/2ppP3/1b1P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 2 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C17k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3

			variation => N__('French: Winawer, 5. a3'),
			moves => {
				'b4a5' => 'rnbqk1nr/pp3ppp/4p3/b1ppP3/3P4/P1N5/1PP2PPP/R1BQKBNR w KQkq - 1 6',
				'b4c3' => 'rnbqk1nr/pp3ppp/4p3/2ppP3/3P4/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 6',
				'c5d4' => 'rnbqk1nr/pp3ppp/4p3/3pP3/1b1p4/P1N5/1PP2PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C18a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3

			variation => N__('French: Winawer, 5...Bxc3+ 6.bxc3'),
			moves => {
				'd8a5' => 'rnb1k1nr/pp3ppp/4p3/q1ppP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 1 7',
				'd8c7' => 'rnb1k1nr/ppq2ppp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 1 7',
				'g8e7' => 'rnbqk2r/pp2nppp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/2ppP3/3P4/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C18a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+

			variation => N__('French: Winawer, 5...Bxc3+'),
			moves => {
				'b2c3' => 'rnbqk1nr/pp3ppp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/3pP3/1P1p4/2N5/1PP2PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C17l',
                        # TRANSLATORS:

			variation => N__('French: Winawer, 5. a3 cxd4'),
			moves => {
				'd4c3' => 'rnbqk1nr/pp3ppp/4p3/3pP3/1P6/2p5/1PP2PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/3pP3/1P6/2p2N2/1PP2PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'C17l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 cxd4 6. axb4 dxc3 7. Nf3

			variation => N__('French: Winawer, Rauzer Variation'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/4p3/3pP3/1P6/2p5/1PP2PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C17l',
                        # TRANSLATORS:

			variation => N__('French: Winawer, 5. a3 cxd4'),
			moves => {
				'g1f3' => 'rnbqk1nr/pp3ppp/4p3/3pP3/1P6/2p2N2/1PP2PPP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/3pP3/1b1p4/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'C17f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Bd2 cxd4

			variation => N__('French: Winawer, Bogoljubow, 5...cxd4'),
			moves => {
			},
		},
		'rnbqk1nr/pp3ppp/4p3/3pP3/1b1p4/P1N5/1PP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C17l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 cxd4

			variation => N__('French: Winawer, 5. a3 cxd4'),
			moves => {
				'a3b4' => 'rnbqk1nr/pp3ppp/4p3/3pP3/1P1p4/2N5/1PP2PPP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/b1ppP3/1P1P4/P1N5/2P2PPP/R1BQKBNR b KQkq b3 0 6' => {
			eco => 'C17n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5 6. b4

			variation => N__('French: Winawer, Swiss, 6. b4'),
			moves => {
				'c5d4' => 'rnbqk1nr/pp3ppp/4p3/b2pP3/1P1p4/P1N5/2P2PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/b1ppP3/3P4/P1N5/1PP2PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'C17m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5

			variation => N__('French: Winawer, Swiss Variation'),
			moves => {
				'b2b4' => 'rnbqk1nr/pp3ppp/4p3/b1ppP3/1P1P4/P1N5/2P2PPP/R1BQKBNR b KQkq b3 0 6',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/b2pP3/1P1p2Q1/P1N5/2P2PPP/R1B1KBNR b KQkq - 1 7' => {
			eco => 'C17o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5 6. b4 cxd4 7. Qg4

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Qg4'),
			moves => {
				'g8e7' => 'rnbqk2r/pp2nppp/4p3/b2pP3/1P1p2Q1/P1N5/2P2PPP/R1B1KBNR w KQkq - 2 8',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/b2pP3/1P1p4/P1N5/2P2PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C17n',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Swiss, 6. b4'),
			moves => {
				'c3b5' => 'rnbqk1nr/pp3ppp/4p3/bN1pP3/1P1p4/P7/2P2PPP/R1BQKBNR b KQkq - 1 7',
				'd1g4' => 'rnbqk1nr/pp3ppp/4p3/b2pP3/1P1p2Q1/P1N5/2P2PPP/R1B1KBNR b KQkq - 1 7',
			},
		},
		'rnbqk1nr/pp3ppp/4p3/bN1pP3/1P1p4/P7/2P2PPP/R1BQKBNR b KQkq - 1 7' => {
			eco => 'C17q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5 6. b4 cxd4 7. Nb5

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Nb5'),
			moves => {
				'a5c7' => 'rnbqk1nr/ppb2ppp/4p3/1N1pP3/1P1p4/P7/2P2PPP/R1BQKBNR w KQkq - 2 8',
			},
		},
		'rnbqk1nr/ppb2ppp/4p3/1N1pP3/1P1p1P2/P7/2P3PP/R1BQKBNR b KQkq f3 0 8' => {
			eco => 'C17r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5 6. b4 cxd4 7. Nb5 Bc7 8. f4

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Nb5 Bc7 8. f4'),
			moves => {
				'c8d7' => 'rn1qk1nr/ppbb1ppp/4p3/1N1pP3/1P1p1P2/P7/2P3PP/R1BQKBNR w KQkq - 1 9',
			},
		},
		'rnbqk1nr/ppb2ppp/4p3/1N1pP3/1P1p4/P7/2P2PPP/R1BQKBNR w KQkq - 2 8' => {
			eco => 'C17q',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Nb5'),
			moves => {
				'f2f4' => 'rnbqk1nr/ppb2ppp/4p3/1N1pP3/1P1p1P2/P7/2P3PP/R1BQKBNR b KQkq f3 0 8',
			},
		},
		'rnbqk1nr/ppp1b1pp/3p4/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4 5. Ng5 Be7

			variation => N__('English: Lukin, 5. Ng5 Be7'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1bppp/3p4/4p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C41b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. Bc4 Be7 4. c3

			variation => N__('Philidor: Steinitz Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1bppp/3p4/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4' => {
			eco => 'C41b',
                        # TRANSLATORS:

			variation => N__('Philidor: 3. Bc4'),
			moves => {
				'c2c3' => 'rnbqk1nr/ppp1bppp/3p4/4p3/2B1P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D31e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. cxd5

			variation => N__('QGD: Alatortsev, Exchange'),
			moves => {
				'e6d5' => 'rnbqk1nr/ppp1bppp/8/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'D31d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. Bf4

			variation => N__('QGD: Alatortsev, 4. Bf4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'D31d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. Nf3

			variation => N__('QGD: Alatortsev, 4. Nf3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D31d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7

			variation => N__('QGD: Alatortsev Variation'),
			moves => {
				'c1f4' => 'rnbqk1nr/ppp1bppp/4p3/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'c4d5' => 'rnbqk1nr/ppp1bppp/4p3/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqk1nr/ppp1bppp/4p3/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C10e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Be7

			variation => N__('French: 3. Nc3 Be7'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/3B4/PPPN1PPP/R1BQK1NR b KQkq - 3 4' => {
			eco => 'C03g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7 4. Bd3

			variation => N__('French: Tarrasch, 3... Be7 4. Bd3'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp2bppp/4p3/2pp4/3PP3/3B4/PPPN1PPP/R1BQK1NR w KQkq c6 0 5',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'C03j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7 4. Ngf3

			variation => N__('French: Tarrasch, 3... Be7 4.Ngf3'),
			moves => {
				'g8f6' => 'rnbqk2r/ppp1bppp/4pn2/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C03f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7

			variation => N__('French: Tarrasch, 3... Be7'),
			moves => {
				'f1d3' => 'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/3B4/PPPN1PPP/R1BQK1NR b KQkq - 3 4',
				'g1f3' => 'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3pP1B1/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'C13d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Ng8

			variation => N__('French: Classical, Nimzowitsch Variation'),
			moves => {
				'g5e3' => 'rnbqk1nr/ppp1bppp/4p3/3pP3/3P4/2N1B3/PPP2PPP/R2QKBNR b KQkq - 2 6',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/3pP3/3P4/2N1B3/PPP2PPP/R2QKBNR b KQkq - 2 6' => {
			eco => 'C13d',
                        # TRANSLATORS:

			variation => N__('French: Classical, Nimzowitsch Variation'),
			moves => {
				'b7b6' => 'rnbqk1nr/p1p1bppp/1p2p3/3pP3/3P4/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/8/3PN3/P7/1PP1NPPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'C15t',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7'),
			moves => {
				'g8f6' => 'rnbqk2r/ppp1bppp/4pn2/8/3PN3/P7/1PP1NPPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqk1nr/ppp1bppp/4p3/8/3Pp3/P1N5/1PP1NPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'C15t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Be7

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7'),
			moves => {
				'c3e4' => 'rnbqk1nr/ppp1bppp/4p3/8/3PN3/P7/1PP1NPPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp1bppp/8/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'D31e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. cxd5 exd5 5. Bf4

			variation => N__('QGD: Alatortsev, 5. Bf4'),
			moves => {
				'c7c6' => 'rnbqk1nr/pp2bppp/2p5/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp1bppp/8/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D31e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Be7 4. cxd5 exd5

			variation => N__('QGD: Alatortsev, Exchange'),
			moves => {
				'c1f4' => 'rnbqk1nr/ppp1bppp/8/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp1p1bp/3p2p1/5p2/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A42b',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 f5 5. Nf3

			variation => N__('Modern: Averbakh, Randspringer, 5. Nf3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1p1bp/3p2p1/5p2/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq f6 0 5' => {
			eco => 'A42b',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 f5

			variation => N__('Modern: Averbakh, Randspringer Variation'),
			moves => {
				'g1f3' => 'rnbqk1nr/ppp1p1bp/3p2p1/5p2/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'B06n',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Bc4

			variation => N__('Modern: 3. Nc3 d6 4. Bc4'),
			moves => {
				'c7c6' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'B06k',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nf3 d6 4. Bc4

			variation => N__('Modern: 3. Nf3 d6 4. Bc4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'A41u',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. Nf3

			variation => N__('Neo-Old Indian: Modern, 3. Nc3 Bg7 4. Nf3'),
			moves => {
				'c8g4' => 'rn1qk1nr/ppp1ppbp/3p2p1/8/2PP2b1/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A41t',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7

			variation => N__('Neo-Old Indian: Modern, 3. Nc3 Bg7'),
			moves => {
				'e2e4' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A41r',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nf3 Bg7

			variation => N__('Neo-Old Indian: Modern, 3. Nf3 Bg7'),
			moves => {
				'e2e4' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PPP3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 4',
				'g2g3' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A41r',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nf3 Bg7 4. g3

			variation => N__('Neo-Old Indian: Modern, 3. Nf3 Bg7 4. g3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'A42a',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4

			variation => N__('Modern: Averbakh'),
			moves => {
				'b8c6' => 'r1bqk1nr/ppp1ppbp/2np2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5',
				'b8d7' => 'r1bqk1nr/pppnppbp/3p2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5',
				'c7c5' => 'rnbqk1nr/pp2ppbp/3p2p1/2p5/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq c6 0 5',
				'c7c6' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5',
				'e7e5' => 'rnbqk1nr/ppp2pbp/3p2p1/4p3/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq e6 0 5',
				'f7f5' => 'rnbqk1nr/ppp1p1bp/3p2p1/5p2/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq f6 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PPP3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 4' => {
			eco => 'A41s',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nf3 Bg7 4. e4

			variation => N__('Neo-Old Indian: Modern, 3. Nf3 Bg7 4. e4'),
			moves => {
				'c8g4' => 'rn1qk1nr/ppp1ppbp/3p2p1/8/2PPP1b1/5N2/PP3PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/2PPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A41q',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. e4 Bg7

			variation => N__('Neo-Old Indian: Modern, 3. e4 Bg7'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A41e',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 g6 3. Bf4 Bg7

			variation => N__('Neo-Old Indian / Modern: 3. Bf4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A41e',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 g6 3. g3 Bg7

			variation => N__('Neo-Old Indian / Modern: 3. g3'),
			moves => {
				'f1g2' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A41e',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 g6 3. g3 Bg7 4. Bg2

			variation => N__('Neo-Old Indian / Modern: 3. g3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'B06o',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Be3

			variation => N__('Modern: 3. Nc3 d6 4. Be3'),
			moves => {
				'a7a6' => 'rnbqk1nr/1pp1ppbp/p2p2p1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 5',
				'c7c6' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B06r',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Nf3

			variation => N__('Modern: Two Knights Variation'),
			moves => {
				'c7c6' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B06m',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6

			variation => N__('Modern: 3. Nc3'),
			moves => {
				'c1e3' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 4',
				'f1c4' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 4',
				'f2f4' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PPP2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 4',
				'g1f3' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'B06h',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. c3 d6 4. Nf3

			variation => N__('Modern: Geller System'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2P5/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B06g',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. c3 d6

			variation => N__('Modern: 3. c3'),
			moves => {
				'f2f4' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PPP2/2P5/PP4PP/RNBQKBNR b KQkq f3 0 4',
				'g1f3' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B06j',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nf3 d6

			variation => N__('Modern: 3. Nf3 d6'),
			moves => {
				'f1c4' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PPP2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 4' => {
			eco => 'B06s',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. f4

			variation => N__('Modern: Pseudo-Austrian Attack'),
			moves => {
				'b8c6' => 'r1bqk1nr/ppp1ppbp/2np2p1/8/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5',
				'c7c6' => 'rnbqk1nr/pp2ppbp/2pp2p1/8/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/3PPP2/2P5/PP4PP/RNBQKBNR b KQkq f3 0 4' => {
			eco => 'B06g',
                        # TRANSLATORS:

			variation => N__('Modern: 3. c3'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp2ppbp/3p2p1/2p5/3PPP2/2P5/PP4PP/RNBQKBNR w KQkq c6 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B06c',
                        # TRANSLATORS: 1. e4 g6 2. Nc3 Bg7 3. f4 d6

			variation => N__('Modern'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/3p2p1/8/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B06c',
                        # TRANSLATORS: 1. e4 g6 2. f4 Bg7 3. Nf3 d6

			variation => N__('Modern'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/6p1/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'B06l',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d5

			variation => N__('Modern: Mittenberger Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/6p1/3p4/5P2/5NP1/PPPPP1BP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 g6 3. g3 Bg7 4. Bg2

			variation => N__('Bird: 1... d5 2. Nf3 g6 3. g3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp1ppbp/6p1/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A03',
                        # TRANSLATORS:

			variation => N__('Bird: 1... d5 2. Nf3 g6 3. g3'),
			moves => {
				'f1g2' => 'rnbqk1nr/ppp1ppbp/6p1/3p4/5P2/5NP1/PPPPP1BP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/6p1/3p4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'e7e5' => 'rnbqk1nr/ppp2pbp/6p1/3pp3/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq e6 0 5',
			},
		},
		'rnbqk1nr/ppp1ppbp/6p1/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'e1g1' => 'rnbqk1nr/ppp1ppbp/6p1/3p4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'rnbqk1nr/ppp1ppbp/6p1/8/2Pp4/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A09i',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 g6 4. Bg2 Bg7

			variation => N__('Reti: Advance, 3. g3 g6 4. Bg2 Bg7'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2p1p/3b4/4N3/3P2p1/8/PPP1P1PP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: From Gambit, Lasker, 5. d4'),
			moves => {
				'd6e5' => 'rnbqk1nr/ppp2p1p/8/4b3/3P2p1/8/PPP1P1PP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'rnbqk1nr/ppp2p1p/3b4/6p1/3P4/5N2/PPP1P1PP/RNBQKB1R b KQkq d3 0 5' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Bxd6 4. Nf3 g5 5. d4

			variation => N__('Bird: From Gambit, Lasker, 5. d4'),
			moves => {
				'g5g4' => 'rnbqk1nr/ppp2p1p/3b4/8/3P2p1/5N2/PPP1P1PP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2p1p/3b4/6p1/8/5N2/PPPPP1PP/RNBQKB1R w KQkq g6 0 5' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Bxd6 4. Nf3 g5

			variation => N__('Bird: From Gambit, Lasker Variation'),
			moves => {
				'd2d4' => 'rnbqk1nr/ppp2p1p/3b4/6p1/3P4/5N2/PPP1P1PP/RNBQKB1R b KQkq d3 0 5',
				'g2g3' => 'rnbqk1nr/ppp2p1p/3b4/6p1/8/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2p1p/3b4/6p1/8/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Bxd6 4. Nf3 g5 5. g3

			variation => N__('Bird: From Gambit, Lasker, 5. g3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2p1p/3b4/8/3P2p1/5N2/PPP1P1PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: From Gambit, Lasker, 5. d4'),
			moves => {
				'f3e5' => 'rnbqk1nr/ppp2p1p/3b4/4N3/3P2p1/8/PPP1P1PP/RNBQKB1R b KQkq - 1 6',
			},
		},
		'rnbqk1nr/ppp2p1p/8/4P3/6p1/8/PPP1P1PP/RNBQKB1R b KQkq - 0 7' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: From Gambit, Lasker, 5. d4'),
			moves => {
				'd8d1' => 'rnb1k1nr/ppp2p1p/8/4P3/6p1/8/PPP1P1PP/RNBqKB1R w KQkq - 0 8',
			},
		},
		'rnbqk1nr/ppp2p1p/8/4b3/3P2p1/8/PPP1P1PP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: From Gambit, Lasker, 5. d4'),
			moves => {
				'd4e5' => 'rnbqk1nr/ppp2p1p/8/4P3/6p1/8/PPP1P1PP/RNBQKB1R b KQkq - 0 7',
			},
		},
		'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp1P/2N2N2/PPP3P1/R1BQK2R b KQkq - 1 7' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'c7c6' => 'rnbqk1nr/pp3pb1/2pp3p/6p1/2BPPp1P/2N2N2/PPP3P1/R1BQK2R w KQkq - 0 8',
			},
		},
		'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp1P/3Q1N2/PPP3P1/RNB1K2R b KQkq - 1 7' => {
			eco => 'C38',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Bg7 5. h4 h6 6. d4 d6 7. Qd3

			variation => N__('KGA: Philidor Gambit, Schultz Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp1P/5N2/PPP3P1/RNBQK2R w KQkq - 0 7' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'b1c3' => 'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp1P/2N2N2/PPP3P1/R1BQK2R b KQkq - 1 7',
				'd1d3' => 'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp1P/3Q1N2/PPP3P1/RNB1K2R b KQkq - 1 7',
			},
		},
		'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp2/5N2/PPP3PP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C38',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Bg7 5. O-O h6 6. d4 d6

			variation => N__('KGA: Hanstein Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2pbp/3p2p1/3Pp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A42t',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. d5

			variation => N__('Modern: Averbakh, 4... e5 5. d5'),
			moves => {
				'b8d7' => 'r1bqk1nr/pppn1pbp/3p2p1/3Pp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqk1nr/ppp2pbp/3p2p1/4P3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A42v',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. dxe5

			variation => N__('Modern: Averbakh, 4... e5 5.dxe5'),
			moves => {
				'd6e5' => 'rnbqk1nr/ppp2pbp/6p1/4p3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2pbp/3p2p1/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A42p',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. Nf3

			variation => N__('Modern: Averbakh, 4... e5 5. Nf3'),
			moves => {
				'b8d7' => 'r1bqk1nr/pppn1pbp/3p2p1/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqk1nr/ppp2pbp/3p2p1/4p3/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A42o',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. Nge2

			variation => N__('Modern: Averbakh, 4... e5 5.Nge2'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2pbp/3p2p1/4p3/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq e6 0 5' => {
			eco => 'A42n',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5

			variation => N__('Modern: Averbakh, 4... e5'),
			moves => {
				'd4d5' => 'rnbqk1nr/ppp2pbp/3p2p1/3Pp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5',
				'd4e5' => 'rnbqk1nr/ppp2pbp/3p2p1/4P3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 5',
				'g1e2' => 'rnbqk1nr/ppp2pbp/3p2p1/4p3/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 1 5',
				'g1f3' => 'rnbqk1nr/ppp2pbp/3p2p1/4p3/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp2pbp/6p1/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'g8e7' => 'rnbqk2r/ppp1npbp/6p1/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6',
			},
		},
		'rnbqk1nr/ppp2pbp/6p1/3pp3/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq e6 0 5' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'd2d3' => 'rnbqk1nr/ppp2pbp/6p1/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 b kq - 0 5',
			},
		},
		'rnbqk1nr/ppp2pbp/6p1/4p3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A42v',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3 Bg7 4. e4 e5 5. dxe5 dxe5

			variation => N__('Modern: Averbakh, 4... e5 5.dxe5 dxe5'),
			moves => {
				'd1d8' => 'rnbQk1nr/ppp2pbp/6p1/4p3/2P1P3/2N5/PP3PPP/R1B1KBNR b KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2ppp/1b1p4/4p3/4PP2/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C30o',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 Bb6

			variation => N__('KGD: Classical, 4. c3 Bb6'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/3P4/5p2/5N2/PPPP2PP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Bd6

			variation => N__('KGA: Scandinavian, 4.exd5 Bd6'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/3Pp3/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C40h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d5 3. exd5 Bd6

			variation => N__('Elephant Gambit: Maroczy'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/3p4/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'C01q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nf3 Bd6 5. c4

			variation => N__('French: Exchange, 4. Nf3 Bd6 5. c4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/3p4/3P4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 3 5' => {
			eco => 'C01o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Bd3 Bd6 5. Nf3

			variation => N__('French: Exchange, 4. Nf3 Bd6 5. Bd3'),
			moves => {
				'g8f6' => 'rnbqk2r/ppp2ppp/3b1n2/3p4/3P4/3B1N2/PPP2PPP/RNBQK2R w KQkq - 4 6',
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/3p4/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'C01j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Bd3 Bd6

			variation => N__('French: Exchange, 4. Bd3 Bd6'),
			moves => {
				'g1f3' => 'rnbqk1nr/ppp2ppp/3b4/3p4/3P4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/3p4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'C01n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nf3 Bd6

			variation => N__('French: Exchange, 4. Nf3 Bd6'),
			moves => {
				'c2c4' => 'rnbqk1nr/ppp2ppp/3b4/3p4/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/8/8/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: From Gambit, 3...Bxd6'),
			moves => {
				'g7g5' => 'rnbqk1nr/ppp2p1p/3b4/6p1/8/5N2/PPPPP1PP/RNBQKB1R w KQkq g6 0 5',
				'g8h6' => 'rnbqk2r/ppp2ppp/3b3n/8/8/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqk1nr/ppp2ppp/3b4/8/8/8/PPPPP1PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Bxd6

			variation => N__('Bird: From Gambit, 3...Bxd6'),
			moves => {
				'g1f3' => 'rnbqk1nr/ppp2ppp/3b4/8/8/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqk1nr/ppp2ppp/3p4/2b1P3/4P3/5N2/PPPP2PP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C30k',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. fxe5

			variation => N__('KGD: Classical, Soldatenkov Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/3p4/2b1p3/1P2PP2/5N2/P1PP2PP/RNBQKB1R b KQkq b3 0 4' => {
			eco => 'C30k',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. b4

			variation => N__('KGD: Classical, Heath Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/3p4/2b1p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C30l',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. Nc3

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3'),
			moves => {
				'b8d7' => 'r1bqk1nr/pppn1ppp/3p4/2b1p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5',
				'g8f6' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqk1nr/ppp2ppp/3p4/2b1p3/4PP2/2P2N2/PP1P2PP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C30n',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3

			variation => N__('KGD: Classical, 4. c3'),
			moves => {
				'c5b6' => 'rnbqk1nr/ppp2ppp/1b1p4/4p3/4PP2/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 1 5',
				'c8g4' => 'rn1qk1nr/ppp2ppp/3p4/2b1p3/4PPb1/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 1 5',
				'f7f5' => 'rnbqk1nr/ppp3pp/3p4/2b1pp2/4PP2/2P2N2/PP1P2PP/RNBQKB1R w KQkq f6 0 5',
				'g8f6' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/4PP2/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp2ppp/3p4/2b1p3/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C30k',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6

			variation => N__('KGD: Classical, 3. Nf3 d6'),
			moves => {
				'b1c3' => 'rnbqk1nr/ppp2ppp/3p4/2b1p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
				'b2b4' => 'rnbqk1nr/ppp2ppp/3p4/2b1p3/1P2PP2/5N2/P1PP2PP/RNBQKB1R b KQkq b3 0 4',
				'c2c3' => 'rnbqk1nr/ppp2ppp/3p4/2b1p3/4PP2/2P2N2/PP1P2PP/RNBQKB1R b KQkq - 0 4',
				'f4e5' => 'rnbqk1nr/ppp2ppp/3p4/2b1P3/4P3/5N2/PPPP2PP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3P4/1b1P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C15b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. exd5

			variation => N__('French: Winawer, 4.exd5'),
			moves => {
				'd8d5' => 'rnb1k1nr/ppp2ppp/4p3/3q4/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP1Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 3 4' => {
			eco => 'C15c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Qg4

			variation => N__('French: Winawer, 4. Qg4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'C15n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2

			variation => N__('French: Winawer, Alekhine Gambit'),
			moves => {
				'b8c6' => 'r1bqk1nr/ppp2ppp/2n1p3/3p4/1b1PP3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 4 5',
				'd5e4' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 0 5',
				'g8f6' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1b1PP3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C15a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4

			variation => N__('French: Winawer'),
			moves => {
				'a2a3' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 4',
				'c1d2' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2N5/PPPB1PPP/R2QKBNR b KQkq - 3 4',
				'd1d3' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2NQ4/PPP2PPP/R1B1KBNR b KQkq - 3 4',
				'd1g4' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP1Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 3 4',
				'e4d5' => 'rnbqk1nr/ppp2ppp/4p3/3P4/1b1P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
				'e4e5' => 'rnbqk1nr/ppp2ppp/4p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
				'f1d3' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2NB4/PPP2PPP/R1BQK1NR b KQkq - 3 4',
				'g1e2' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2N5/PPPB1PPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'C15h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd2

			variation => N__('French: Winawer, Müller-Zhuravlev Gambit'),
			moves => {
				'd5e4' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2NB4/PPP2PPP/R1BQK1NR b KQkq - 3 4' => {
			eco => 'C15e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd3

			variation => N__('French: Winawer, 4. Bd3'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp3ppp/4p3/2pp4/1b1PP3/2NB4/PPP2PPP/R1BQK1NR w KQkq c6 0 5',
				'd5e4' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2NB4/PPP2PPP/R1BQK1NR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2NQ4/PPP2PPP/R1B1KBNR b KQkq - 3 4' => {
			eco => 'C15d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Qd3

			variation => N__('French: Winawer, 4. Qd3'),
			moves => {
				'd5e4' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2NQ4/PPP2PPP/R1B1KBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C15k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3

			variation => N__('French: Winawer, 4. a3'),
			moves => {
				'b4c3' => 'rnbqk1nr/ppp2ppp/4p3/3p4/3PP3/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1bPP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D31c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Bb4

			variation => N__('QGD: 3. Nc3 Bb4'),
			moves => {
				'a2a3' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1bPP4/P1N5/1P2PPPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/1bPP4/P1N5/1P2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D31c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Bb4 4. a3

			variation => N__('QGD: 3. Nc3 Bb4 4. a3'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/3PP3/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C15k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3

			variation => N__('French: Winawer, 4. a3 Bxc3+'),
			moves => {
				'd5e4' => 'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1P5/2P2PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3p4/3PP3/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C15k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+

			variation => N__('French: Winawer, 4. a3 Bxc3+'),
			moves => {
				'b2c3' => 'rnbqk1nr/ppp2ppp/4p3/3p4/3PP3/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C16a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5

			variation => N__('French: Winawer, Advance Variation'),
			moves => {
				'b7b6' => 'rnbqk1nr/p1p2ppp/1p2p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
				'c7c5' => 'rnbqk1nr/pp3ppp/4p3/2ppP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq c6 0 5',
				'd8d7' => 'rnb1k1nr/pppq1ppp/4p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5',
				'g8e7' => 'rnbqk2r/ppp1nppp/4p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1PB3/2N5/PPP2PPP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'C15g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd3 dxe4 5. Bxe4

			variation => N__('French: Winawer, 4. Bd3 dxe4 5.Bxe4'),
			moves => {
				'g8f6' => 'rnbqk2r/ppp2ppp/4pn2/8/1b1PB3/2N5/PPP2PPP/R1BQK1NR w KQkq - 1 6',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp1Q1/2N5/PPPB1PPP/R3KBNR b KQkq - 1 5' => {
			eco => 'C15i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd2 dxe4 5. Qg4

			variation => N__('French: Winawer, Müller-Zhuravlev Gambit, 5. Qg4'),
			moves => {
				'd8d4' => 'rnb1k1nr/ppp2ppp/4p3/8/1b1qp1Q1/2N5/PPPB1PPP/R3KBNR w KQkq - 0 6',
				'g8f6' => 'rnbqk2r/ppp2ppp/4pn2/8/1b1Pp1Q1/2N5/PPPB1PPP/R3KBNR w KQkq - 2 6',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C15q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4

			variation => N__('French: Winawer, Alekhine Gambit Accepted'),
			moves => {
				'a2a3' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/P1N5/1PP1NPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'C15h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd2 dxe4

			variation => N__('French: Winawer, Müller-Zhuravlev Gambit'),
			moves => {
				'd1g4' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp1Q1/2N5/PPPB1PPP/R3KBNR b KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2NB4/PPP2PPP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'C15g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd3 dxe4

			variation => N__('French: Winawer, 4. Bd3 dxe4'),
			moves => {
				'd3e4' => 'rnbqk1nr/ppp2ppp/4p3/8/1b1PB3/2N5/PPP2PPP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/2NQ4/PPP2PPP/R1B1KBNR w KQkq - 0 5' => {
			eco => 'C15d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Qd3 dxe4

			variation => N__('French: Winawer, 4. Qd3 dxe4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/1b1Pp3/P1N5/1PP1NPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'C15q',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Alekhine Gambit Accepted'),
			moves => {
				'b4c3' => 'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1b5/1PP1NPPP/R1BQKB1R w KQkq - 0 6',
				'b4e7' => 'rnbqk1nr/ppp1bppp/4p3/8/3Pp3/P1N5/1PP1NPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/3P4/P1P2p2/2P3PP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C15l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 dxe4 6. f3 exf3

			variation => N__('French: Winawer, Winkelmann-Reimer Gambit Accepted'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/3Pp1Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 1 6' => {
			eco => 'C15m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 dxe4 6. Qg4

			variation => N__('French: Winawer, 4. a3 Bxc3+ 5.bxc3 dxe4 6. Qg4'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'C15q',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Bxc3+'),
			moves => {
				'b8c6' => 'r1bqk1nr/ppp2ppp/2n1p3/8/3Pp3/P1N5/1PP2PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1P2P2/2P3PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C15l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 dxe4 6. f3

			variation => N__('French: Winawer, Winkelmann-Reimer Gambit (WRG)'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp3ppp/4p3/2p5/3Pp3/P1P2P2/2P3PP/R1BQKBNR w KQkq c6 0 7',
				'e4f3' => 'rnbqk1nr/ppp2ppp/4p3/8/3P4/P1P2p2/2P3PP/R1BQKBNR w KQkq - 0 7',
				'e6e5' => 'rnbqk1nr/ppp2ppp/8/4p3/3Pp3/P1P2P2/2P3PP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1P5/2P2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C15k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 dxe4

			variation => N__('French: Winawer, 4. a3 Bxc3+ 5.bxc3 dxe4'),
			moves => {
				'd1g4' => 'rnbqk1nr/ppp2ppp/4p3/8/3Pp1Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 1 6',
				'f2f3' => 'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1P2P2/2P3PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1b5/1PP1NPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C15q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Bxc3+

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Bxc3+'),
			moves => {
				'e2c3' => 'rnbqk1nr/ppp2ppp/4p3/8/3Pp3/P1N5/1PP2PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2ppp/8/2bBp3/4P3/2P5/PP1P1PPP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Lewis Countergambit'),
			moves => {
				'g8f6' => 'rnbqk2r/ppp2ppp/5n2/2bBp3/4P3/2P5/PP1P1PPP/RNBQK1NR w KQkq - 1 5',
			},
		},
		'rnbqk1nr/ppp2ppp/8/2bpp3/2B1P3/2P5/PP1P1PPP/RNBQK1NR w KQkq d6 0 4' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. c3 d5

			variation => N__('Bishop\'s Opening: Lewis Countergambit'),
			moves => {
				'c4d5' => 'rnbqk1nr/ppp2ppp/8/2bBp3/4P3/2P5/PP1P1PPP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppp2ppp/8/3p4/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'C01u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Bb4

			variation => N__('French: Exchange Winawer'),
			moves => {
				'f1d3' => 'rnbqk1nr/ppp2ppp/8/3p4/1b1P4/2NB4/PPP2PPP/R1BQK1NR b KQkq - 3 5',
			},
		},
		'rnbqk1nr/ppp2ppp/8/3p4/1b1P4/2NB4/PPP2PPP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'C01v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Bb4 5. Bd3

			variation => N__('French: Exchange Winawer, 5. Bd3'),
			moves => {
				'b8c6' => 'r1bqk1nr/ppp2ppp/2n5/3p4/1b1P4/2NB4/PPP2PPP/R1BQK1NR w KQkq - 4 6',
				'g8e7' => 'rnbqk2r/ppp1nppp/8/3p4/1b1P4/2NB4/PPP2PPP/R1BQK1NR w KQkq - 4 6',
			},
		},
		'rnbqk1nr/ppp2ppp/8/4P3/1bP5/4p3/PP1B1PPP/RN1QKBNR w KQkq - 0 6' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. e3 Bb4+ 5. Bd2 dxe3

			variation => N__('QGD: Albin, Lasker Trap'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/8/4P3/1bPp4/4P3/PP1B1PPP/RN1QKBNR b KQkq - 2 5' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, 3.dxe5 d4'),
			moves => {
				'd4e3' => 'rnbqk1nr/ppp2ppp/8/4P3/1bP5/4p3/PP1B1PPP/RN1QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp2ppp/8/4P3/1bPp4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, 3.dxe5 d4'),
			moves => {
				'c1d2' => 'rnbqk1nr/ppp2ppp/8/4P3/1bPp4/4P3/PP1B1PPP/RN1QKBNR b KQkq - 2 5',
			},
		},
		'rnbqk1nr/ppp2ppp/8/4p3/1bpPP3/5N2/PP3PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'D20w',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 e5 4. Nf3 Bb4+

			variation => N__('QGA: 3. e4 e5 4. Nf3 Bb4+'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/8/4p3/3Pp3/P1P2P2/2P3PP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C15l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 dxe4 6. f3 e5

			variation => N__('French: Winawer, Winkelmann-Reimer, Hübner Defence'),
			moves => {
			},
		},
		'rnbqk1nr/ppp2ppp/8/8/1bBP4/2N5/PP3PPP/R1BQK1NR b KQkq - 2 6' => {
			eco => 'D20f',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e3 e5'),
			moves => {
				'g8f6' => 'rnbqk2r/ppp2ppp/5n2/8/1bBP4/2N5/PP3PPP/R1BQK1NR w KQkq - 3 7',
			},
		},
		'rnbqk1nr/ppp2ppp/8/8/1bBP4/8/PP3PPP/RNBQK1NR w KQkq - 1 6' => {
			eco => 'D20f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e5 4. Bxc4 exd4 5. exd4 Bb4+

			variation => N__('QGA: 3. e3 e5'),
			moves => {
				'b1c3' => 'rnbqk1nr/ppp2ppp/8/8/1bBP4/2N5/PP3PPP/R1BQK1NR b KQkq - 2 6',
			},
		},
		'rnbqk1nr/ppp3pp/3p4/2b1Pp2/4P3/2P2N2/PP1P2PP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C30r',
                        # TRANSLATORS:

			variation => N__('KGD: Classical Countergambit'),
			moves => {
				'd6e5' => 'rnbqk1nr/ppp3pp/8/2b1pp2/4P3/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk1nr/ppp3pp/3p4/2b1pp2/4PP2/2P2N2/PP1P2PP/RNBQKB1R w KQkq f6 0 5' => {
			eco => 'C30r',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 f5

			variation => N__('KGD: Classical Countergambit'),
			moves => {
				'f4e5' => 'rnbqk1nr/ppp3pp/3p4/2b1Pp2/4P3/2P2N2/PP1P2PP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppp3pp/4P3/2bp2N1/4p3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'C41e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 f5 4. dxe5 fxe4 5. Ng5 d5 6. e6 Bc5 7. Nc3

			variation => N__('Philidor: Philidor Countergambit, Berger Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppp3pp/4P3/2bp2N1/4p3/8/PPP2PPP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'C41e',
                        # TRANSLATORS:

			variation => N__('Philidor: Philidor Countergambit, del Rio Attack'),
			moves => {
				'b1c3' => 'rnbqk1nr/ppp3pp/4P3/2bp2N1/4p3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 7',
			},
		},
		'rnbqk1nr/ppp3pp/8/2b1pp2/3PP3/2P2N2/PP4PP/RNBQKB1R b KQkq d3 0 6' => {
			eco => 'C30r',
                        # TRANSLATORS:

			variation => N__('KGD: Classical Countergambit'),
			moves => {
				'e5d4' => 'rnbqk1nr/ppp3pp/8/2b2p2/3pP3/2P2N2/PP4PP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'rnbqk1nr/ppp3pp/8/2b1pp2/4P3/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C30r',
                        # TRANSLATORS:

			variation => N__('KGD: Classical Countergambit'),
			moves => {
				'd2d4' => 'rnbqk1nr/ppp3pp/8/2b1pp2/3PP3/2P2N2/PP4PP/RNBQKB1R b KQkq d3 0 6',
			},
		},
		'rnbqk1nr/ppp3pp/8/2b2p2/2BpP3/2P2N2/PP4PP/RNBQK2R b KQkq - 1 7' => {
			eco => 'C30r',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 f5 5. fxe5 dxe5 6. d4 exd4 7. Bc4

			variation => N__('KGD: Classical, Reti Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppp3pp/8/2b2p2/3pP3/2P2N2/PP4PP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'C30r',
                        # TRANSLATORS:

			variation => N__('KGD: Classical Countergambit'),
			moves => {
				'f1c4' => 'rnbqk1nr/ppp3pp/8/2b2p2/2BpP3/2P2N2/PP4PP/RNBQK2R b KQkq - 1 7',
			},
		},
		'rnbqk1nr/pppp1p1p/8/2b1p1p1/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq g6 0 4' => {
			eco => 'C30j',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 g5

			variation => N__('KGD: Classical, Senechaud Countergambit'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1pb1/7p/6p1/2B1Pp1P/5N2/PPPP2P1/RNBQK2R w KQkq - 0 6' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'd2d4' => 'rnbqk1nr/pppp1pb1/7p/6p1/2BPPp1P/5N2/PPP3P1/RNBQK2R b KQkq d3 0 6',
			},
		},
		'rnbqk1nr/pppp1pb1/7p/6p1/2B1Pp2/5N2/PPPP2PP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: 3. Nf3 g5 4. Bc4 Bg7'),
			moves => {
				'd2d4' => 'rnbqk1nr/pppp1pb1/7p/6p1/2BPPp2/5N2/PPP3PP/RNBQ1RK1 b kq d3 0 6',
			},
		},
		'rnbqk1nr/pppp1pb1/7p/6p1/2BPPp1P/5N2/PPP3P1/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp1P/5N2/PPP3P1/RNBQK2R w KQkq - 0 7',
			},
		},
		'rnbqk1nr/pppp1pb1/7p/6p1/2BPPp2/5N2/PPP3PP/RNBQ1RK1 b kq d3 0 6' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: 3. Nf3 g5 4. Bc4 Bg7'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp2pb1/3p3p/6p1/2BPPp2/5N2/PPP3PP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'rnbqk1nr/pppp1pbp/6p1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A21',
                        # TRANSLATORS:

			variation => N__('English: King\'s, 2. Nc3 g6'),
			moves => {
				'f1g2' => 'rnbqk1nr/pppp1pbp/6p1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'rnbqk1nr/pppp1pbp/6p1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 g6 3. g3 Bg7 4. Bg2

			variation => N__('English: King\'s, 2. Nc3 g6'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1pbp/6p1/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 g6 3. Bg2 Bg7

			variation => N__('English: King\'s, 2. g3 g6'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1pbp/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Bg7

			variation => N__('KGA: Kieseritsky, Paulsen Defence'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1pbp/8/6p1/2B1Pp1P/5N2/PPPP2P1/RNBQK2R b KQkq h3 0 5' => {
			eco => 'C38',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Bg7 5. h4

			variation => N__('KGA: Philidor Gambit'),
			moves => {
				'h7h6' => 'rnbqk1nr/pppp1pb1/7p/6p1/2B1Pp1P/5N2/PPPP2P1/RNBQK2R w KQkq - 0 6',
			},
		},
		'rnbqk1nr/pppp1pbp/8/6p1/2B1Pp2/5N2/PPPP2PP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'C38',
                        # TRANSLATORS:

			variation => N__('KGA: 3. Nf3 g5 4. Bc4 Bg7'),
			moves => {
				'h7h6' => 'rnbqk1nr/pppp1pb1/7p/6p1/2B1Pp2/5N2/PPPP2PP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'rnbqk1nr/pppp1pbp/8/6p1/2B1Pp2/5N2/PPPP2PP/RNBQK2R w KQkq - 2 5' => {
			eco => 'C38',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Bg7

			variation => N__('KGA: 3. Nf3 g5 4. Bc4 Bg7'),
			moves => {
				'd2d4' => 'rnbqk1nr/pppp1pbp/8/6p1/2BPPp2/5N2/PPP3PP/RNBQK2R b KQkq d3 0 5',
				'e1g1' => 'rnbqk1nr/pppp1pbp/8/6p1/2B1Pp2/5N2/PPPP2PP/RNBQ1RK1 b kq - 3 5',
				'h2h4' => 'rnbqk1nr/pppp1pbp/8/6p1/2B1Pp1P/5N2/PPPP2P1/RNBQK2R b KQkq h3 0 5',
			},
		},
		'rnbqk1nr/pppp1pbp/8/6p1/2BPPp2/5N2/PPP3PP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C38',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 Bg7 5. d4

			variation => N__('KGA: Hanstein Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/4p3/8/1bP5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A13f',
                        # TRANSLATORS: 1. c4 e6 2. Nc3 Bb4

			variation => N__('English: 1... e6 2. Nc3 Bb4'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/4p3/8/1bPP4/8/PP1BPPPP/RN1QKBNR b KQkq - 2 3' => {
			eco => 'A40q',
                        # TRANSLATORS: 1. d4 e6 2. c4 Bb4+ 3. Bd2

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2'),
			moves => {
				'a7a5' => 'rnbqk1nr/1ppp1ppp/4p3/p7/1bPP4/8/PP1BPPPP/RN1QKBNR w KQkq a6 0 4',
				'b4d2' => 'rnbqk1nr/pppp1ppp/4p3/8/2PP4/8/PP1bPPPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/4p3/8/1bPP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A40p',
                        # TRANSLATORS: 1. d4 e6 2. c4 Bb4+

			variation => N__('Queen\'s Pawn: Keres Defence'),
			moves => {
				'c1d2' => 'rnbqk1nr/pppp1ppp/4p3/8/1bPP4/8/PP1BPPPP/RN1QKBNR b KQkq - 2 3',
			},
		},
		'rnbqk1nr/pppp1ppp/4p3/8/2PP4/8/PP1QPPPP/RN2KBNR b KQkq - 0 4' => {
			eco => 'A40q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 Bxd2+'),
			moves => {
				'b7b6' => 'rnbqk1nr/p1pp1ppp/1p2p3/8/2PP4/8/PP1QPPPP/RN2KBNR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/pppp1ppp/4p3/8/2PP4/8/PP1bPPPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'A40q',
                        # TRANSLATORS: 1. d4 e6 2. c4 Bb4+ 3. Bd2 Bxd2+

			variation => N__('Queen\'s Pawn: Keres Defence, 3. Bd2 Bxd2+'),
			moves => {
				'd1d2' => 'rnbqk1nr/pppp1ppp/4p3/8/2PP4/8/PP1QPPPP/RN2KBNR b KQkq - 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/1PB1P3/8/P1PP1PPP/RNBQK1NR b KQkq b3 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. b4

			variation => N__('Bishop\'s Opening: MacDonnell Gambit'),
			moves => {
				'c5b4' => 'rnbqk1nr/pppp1ppp/8/4p3/1bB1P3/8/P1PP1PPP/RNBQK1NR w KQkq - 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3' => {
			eco => 'C25e',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Bc5 3. Bc4

			variation => N__('Vienna: 2... Bc5 3. Bc4'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/2P5/PP1P1PPP/RNBQK1NR b KQkq - 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. c3

			variation => N__('Bishop\'s Opening: Philidor Variation'),
			moves => {
				'd7d5' => 'rnbqk1nr/ppp2ppp/8/2bpp3/2B1P3/2P5/PP1P1PPP/RNBQK1NR w KQkq d6 0 4',
				'd8g5' => 'rnb1k1nr/pppp1ppp/8/2b1p1q1/2B1P3/2P5/PP1P1PPP/RNBQK1NR w KQkq - 1 4',
				'g8f6' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2P5/PP1P1PPP/RNBQK1NR w KQkq - 1 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5

			variation => N__('Bishop\'s Opening: Classical Variation'),
			moves => {
				'b2b4' => 'rnbqk1nr/pppp1ppp/8/2b1p3/1PB1P3/8/P1PP1PPP/RNBQK1NR b KQkq b3 0 3',
				'c2c3' => 'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/2P5/PP1P1PPP/RNBQK1NR b KQkq - 0 3',
				'd1e2' => 'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/8/PPPPQPPP/RNB1K1NR b KQkq - 3 3',
				'd2d4' => 'rnbqk1nr/pppp1ppp/8/2b1p3/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/8/PPPPQPPP/RNB1K1NR b KQkq - 3 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. Qe2

			variation => N__('Bishop\'s Opening: 2... Bc5 3. Qe2'),
			moves => {
				'b8c6' => 'r1bqk1nr/pppp1ppp/2n5/2b1p3/2B1P3/8/PPPPQPPP/RNB1K1NR w KQkq - 4 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/2B1PP2/8/PPPP2PP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'C30j',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Bc4

			variation => N__('KGD: Classical, 3. Bc4'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. d4

			variation => N__('Bishop\'s Opening: Lewis Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'C25f',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Bc5 3. Nf3

			variation => N__('Vienna: 2... Bc5 3. Nf3'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'C25d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Bc5

			variation => N__('Vienna: 2... Bc5'),
			moves => {
				'c3a4' => 'rnbqk1nr/pppp1ppp/8/2b1p3/N3P3/8/PPPP1PPP/R1BQKBNR b KQkq - 3 3',
				'f1c4' => 'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3',
				'g1f3' => 'rnbqk1nr/pppp1ppp/8/2b1p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'C30j',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3

			variation => N__('KGD: Classical, 3. Nf3'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp2ppp/3p4/2b1p3/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
				'g7g5' => 'rnbqk1nr/pppp1p1p/8/2b1p1p1/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq g6 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'C30j',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5

			variation => N__('King\'s Gambit: Classical KGD'),
			moves => {
				'f1c4' => 'rnbqk1nr/pppp1ppp/8/2b1p3/2B1PP2/8/PPPP2PP/RNBQK1NR b KQkq - 2 3',
				'g1f3' => 'rnbqk1nr/pppp1ppp/8/2b1p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqk1nr/pppp1ppp/8/2b1p3/N3P3/8/PPPP1PPP/R1BQKBNR b KQkq - 3 3' => {
			eco => 'C25d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Bc5 3. Na4

			variation => N__('Vienna: Hammpe-Meitner'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/2bNp3/2P5/8/PP1PPPPP/R1BQKBNR w KQkq - 4 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. Nd5 Bc5

			variation => N__('English: Kramnik-Shirov, 3. Nd5 Bc5'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/3Np3/1bP5/8/PP1PPPPP/R1BQKBNR b KQkq - 3 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. Nd5

			variation => N__('English: Kramnik-Shirov, 3. Nd5'),
			moves => {
				'a7a5' => 'rnbqk1nr/1ppp1ppp/8/p2Np3/1bP5/8/PP1PPPPP/R1BQKBNR w KQkq a6 0 4',
				'b4a5' => 'rnbqk1nr/pppp1ppp/8/b2Np3/2P5/8/PP1PPPPP/R1BQKBNR w KQkq - 4 4',
				'b4c5' => 'rnbqk1nr/pppp1ppp/8/2bNp3/2P5/8/PP1PPPPP/R1BQKBNR w KQkq - 4 4',
				'b4e7' => 'rnbqk1nr/ppppbppp/8/3Np3/2P5/8/PP1PPPPP/R1BQKBNR w KQkq - 4 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4B3/1b6/8/P1PPPPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A00s',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 Bxb4 3. Bxe5

			variation => N__('Polish: 2...Bxb4 3.Bxe5'),
			moves => {
				'g8f6' => 'rnbqk2r/pppp1ppp/5n2/4B3/1b6/8/P1PPPPPP/RN1QKBNR w KQkq - 1 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1b2P1Q1/2N5/PPPP1PPP/R1B1KBNR b KQkq - 3 3' => {
			eco => 'C25b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Bb4 3. Qg4

			variation => N__('Vienna: Zhuravlev'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1b2P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'C25b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Bb4

			variation => N__('Vienna: 2... Bb4'),
			moves => {
				'd1g4' => 'rnbqk1nr/pppp1ppp/8/4p3/1b2P1Q1/2N5/PPPP1PPP/R1B1KBNR b KQkq - 3 3',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1b6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A00s',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 Bxb4

			variation => N__('Polish: 2...Bxb4'),
			moves => {
				'b2e5' => 'rnbqk1nr/pppp1ppp/8/4B3/1b6/8/P1PPPPPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1bB1P3/8/P1PP1PPP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Gambit'),
			moves => {
				'f2f4' => 'rnbqk1nr/pppp1ppp/8/4p3/1bB1PP2/8/P1PP2PP/RNBQK1NR b KQkq f3 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1bB1PP2/8/P1PP2PP/RNBQK1NR b KQkq f3 0 4' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. b4 Bxb4 4. f4

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'e5f4' => 'rnbqk1nr/pppp1ppp/8/8/1bB1Pp2/8/P1PP2PP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1bP5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. g3

			variation => N__('English: Kramnik-Shirov, 3. g3'),
			moves => {
				'b4c3' => 'rnbqk1nr/pppp1ppp/8/4p3/2P5/2b3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/1bP5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4

			variation => N__('English: Kramnik-Shirov Counterattack'),
			moves => {
				'c3d5' => 'rnbqk1nr/pppp1ppp/8/3Np3/1bP5/8/PP1PPPPP/R1BQKBNR b KQkq - 3 3',
				'g2g3' => 'rnbqk1nr/pppp1ppp/8/4p3/1bP5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/2P5/2P3P1/P2PPP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. g3 Bxc3 4. bxc3

			variation => N__('English: Kramnik-Shirov, 3. g3 Bxc3 4.bxc3'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/4p3/2P5/2b3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. g3 Bxc3

			variation => N__('English: Kramnik-Shirov, 3. g3 Bxc3'),
			moves => {
				'b2c3' => 'rnbqk1nr/pppp1ppp/8/4p3/2P5/2P3P1/P2PPP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/1bB1P3/8/PB3PPP/RN1QK1NR w KQkq - 1 6' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3 4. Bc4 cxb2 5. Bxb2 Bb4+

			variation => N__('Danish Gambit: Copenhagen Defence'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/1bB1Pp2/5N2/P1PP2PP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'b4e7' => 'rnbqk1nr/ppppbppp/8/8/2B1Pp2/5N2/P1PP2PP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/1bB1Pp2/8/P1PP2PP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'g1f3' => 'rnbqk1nr/pppp1ppp/8/8/1bB1Pp2/5N2/P1PP2PP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5N2/PPPP3p/RNBQ1R1K b kq - 1 7' => {
			eco => 'C35',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 Be7 4. Bc4 Bh4+ 5. g3 fxg3 6. O-O gxh2+ 7. Kh1

			variation => N__('KGA: Cunningham, Three Pawns Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5N2/PPPP3p/RNBQ1RK1 w kq - 0 7' => {
			eco => 'C35',
                        # TRANSLATORS:

			variation => N__('KGA: Cunningham, Bertin Gambit'),
			moves => {
				'g1h1' => 'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5N2/PPPP3p/RNBQ1R1K b kq - 1 7',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5Np1/PPPP3P/RNBQ1RK1 b kq - 1 6' => {
			eco => 'C35',
                        # TRANSLATORS:

			variation => N__('KGA: Cunningham, Bertin Gambit'),
			moves => {
				'g3h2' => 'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5N2/PPPP3p/RNBQ1RK1 w kq - 0 7',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5Np1/PPPP3P/RNBQK2R w KQkq - 0 6' => {
			eco => 'C35',
                        # TRANSLATORS:

			variation => N__('KGA: Cunningham, Bertin Gambit'),
			moves => {
				'e1g1' => 'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5Np1/PPPP3P/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2B1Pp1b/5N2/PPPP2PP/RNBQK2R w KQkq - 4 5' => {
			eco => 'C35',
                        # TRANSLATORS:

			variation => N__('KGA: Cunningham Defence'),
			moves => {
				'g2g3' => 'rnbqk1nr/pppp1ppp/8/8/2B1Pp1b/5NP1/PPPP3P/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2B1Pp1b/5NP1/PPPP3P/RNBQK2R b KQkq - 0 5' => {
			eco => 'C35',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 Be7 4. Bc4 Bh4+ 5. g3

			variation => N__('KGA: Cunningham, Bertin Gambit'),
			moves => {
				'f4g3' => 'rnbqk1nr/pppp1ppp/8/8/2B1P2b/5Np1/PPPP3P/RNBQK2R w KQkq - 0 6',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5N2/P1P4p/RNBQ1R1K b kq - 1 9' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. b4 Bxb4 4. f4 exf4 5. Nf3 Be7 6. d4 Bh4+ 7. g3 fxg3 8. O-O gxh2+ 9. Kh1

			variation => N__('Bishop\'s Opening: MacDonnell, Four Pawns Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5N2/P1P4p/RNBQ1RK1 w kq - 0 9' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'g1h1' => 'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5N2/P1P4p/RNBQ1R1K b kq - 1 9',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5Np1/P1P4P/RNBQ1RK1 b kq - 1 8' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'g3h2' => 'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5N2/P1P4p/RNBQ1RK1 w kq - 0 9',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5Np1/P1P4P/RNBQK2R w KQkq - 0 8' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'e1g1' => 'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5Np1/P1P4P/RNBQ1RK1 b kq - 1 8',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2BPPp1b/5N2/P1P3PP/RNBQK2R w KQkq - 1 7' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'g2g3' => 'rnbqk1nr/pppp1ppp/8/8/2BPPp1b/5NP1/P1P4P/RNBQK2R b KQkq - 0 7',
			},
		},
		'rnbqk1nr/pppp1ppp/8/8/2BPPp1b/5NP1/P1P4P/RNBQK2R b KQkq - 0 7' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'f4g3' => 'rnbqk1nr/pppp1ppp/8/8/2BPP2b/5Np1/P1P4P/RNBQK2R w KQkq - 0 8',
			},
		},
		'rnbqk1nr/pppp1ppp/8/b2Np3/2P5/8/PP1PPPPP/R1BQKBNR w KQkq - 4 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. Nd5 Ba5

			variation => N__('English: Kramnik-Shirov, 3. Nd5 Ba5'),
			moves => {
			},
		},
		'rnbqk1nr/pppp2pp/5p2/4p3/1b2P3/8/PBPP1PPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 f6 3. e4 Bxb4

			variation => N__('Polish: Tartakower Gambit'),
			moves => {
			},
		},
		'rnbqk1nr/ppppbp1p/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Be7

			variation => N__('KGA: Kieseritsky, Polerio Defence'),
			moves => {
			},
		},
		'rnbqk1nr/ppppbppp/8/3Np3/2P5/8/PP1PPPPP/R1BQKBNR w KQkq - 4 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. Nd5 Be7

			variation => N__('English: Kramnik-Shirov, 3. Nd5 Be7'),
			moves => {
				'd2d4' => 'rnbqk1nr/ppppbppp/8/3Np3/2PP4/8/PP2PPPP/R1BQKBNR b KQkq d3 0 4',
			},
		},
		'rnbqk1nr/ppppbppp/8/3Np3/2PP4/8/PP2PPPP/R1BQKBNR b KQkq d3 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Bb4 3. Nd5 Be7 4. d4

			variation => N__('English: Kramnik-Shirov, 3. Nd5 Be7 4. d4'),
			moves => {
			},
		},
		'rnbqk1nr/ppppbppp/8/8/2B1Pp2/5N2/P1PP2PP/RNBQK2R w KQkq - 2 6' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'd2d4' => 'rnbqk1nr/ppppbppp/8/8/2BPPp2/5N2/P1P3PP/RNBQK2R b KQkq d3 0 6',
			},
		},
		'rnbqk1nr/ppppbppp/8/8/2B1Pp2/5N2/PPPP2PP/RNBQK2R b KQkq - 3 4' => {
			eco => 'C35',
                        # TRANSLATORS:

			variation => N__('KGA: Cunningham Defence'),
			moves => {
				'e7h4' => 'rnbqk1nr/pppp1ppp/8/8/2B1Pp1b/5N2/PPPP2PP/RNBQK2R w KQkq - 4 5',
				'g8f6' => 'rnbqk2r/ppppbppp/5n2/8/2B1Pp2/5N2/PPPP2PP/RNBQK2R w KQkq - 4 5',
			},
		},
		'rnbqk1nr/ppppbppp/8/8/2BPPp2/5N2/P1P3PP/RNBQK2R b KQkq d3 0 6' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: MacDonnell Double Gambit'),
			moves => {
				'e7h4' => 'rnbqk1nr/pppp1ppp/8/8/2BPPp1b/5N2/P1P3PP/RNBQK2R w KQkq - 1 7',
			},
		},
		'rnbqk1nr/ppppbppp/8/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'C35',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 Be7

			variation => N__('KGA: Cunningham Defence'),
			moves => {
				'f1c4' => 'rnbqk1nr/ppppbppp/8/8/2B1Pp2/5N2/PPPP2PP/RNBQK2R b KQkq - 3 4',
			},
		},
		'rnbqk1nr/ppppp1bp/6p1/5p2/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 3 4' => {
			eco => 'A81',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. g3'),
			moves => {
				'c7c6' => 'rnbqk1nr/pp1pp1bp/2p3p1/5p2/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqk1nr/ppppp1bp/6p1/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'A81',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. g3'),
			moves => {
				'g1f3' => 'rnbqk1nr/ppppp1bp/6p1/5p2/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 3 4',
				'g1h3' => 'rnbqk1nr/ppppp1bp/6p1/5p2/3P4/6PN/PPP1PPBP/RNBQK2R b KQkq - 3 4',
			},
		},
		'rnbqk1nr/ppppp1bp/6p1/5p2/3P4/6PN/PPP1PPBP/RNBQK2R b KQkq - 3 4' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 g6 3. Bg2 Bg7 4. Nh3

			variation => N__('Dutch: Leningrad, Carlsbad Variation'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'B06f',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Bc4

			variation => N__('Modern: 3. Bc4'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. Nc3 Bg7 3. g3

			variation => N__('English: 1... g6 2. Nc3 Bg7 3. g3'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. Nc3 Bg7

			variation => N__('English: 1... g6 2. Nc3 Bg7'),
			moves => {
				'g2g3' => 'rnbqk1nr/ppppppbp/6p1/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. Nf3 Bg7

			variation => N__('English: 1... g6'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'A40x',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. Nc3

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp1pppbp/6p1/2p5/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'A40w',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. Nf3

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp1pppbp/6p1/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A40u',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'b1c3' => 'rnbqk1nr/ppppppbp/6p1/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
				'e2e4' => 'rnbqk1nr/ppppppbp/6p1/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqk1nr/ppppppbp/6p1/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A40v',
                        # TRANSLATORS: 1. d4 g6 2. c4 Bg7 3. e4

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'B06l',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3

			variation => N__('Modern: 3. Nc3'),
			moves => {
				'c7c5' => 'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq c6 0 4',
				'd7d5' => 'rnbqk1nr/ppp1ppbp/6p1/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq d6 0 4',
				'd7d6' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B06g',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. c3

			variation => N__('Modern: 3. c3'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/2P5/PP3PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B06i',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nf3

			variation => N__('Modern: 3. Nf3'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B06f',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7

			variation => N__('Modern'),
			moves => {
				'b1c3' => 'rnbqk1nr/ppppppbp/6p1/8/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 3',
				'c2c3' => 'rnbqk1nr/ppppppbp/6p1/8/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'f1c4' => 'rnbqk1nr/ppppppbp/6p1/8/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq - 2 3',
				'f2f4' => 'rnbqk1nr/ppppppbp/6p1/8/3PPP2/8/PPP3PP/RNBQKBNR b KQkq f3 0 3',
				'g1f3' => 'rnbqk1nr/ppppppbp/6p1/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/3PPP2/8/PPP3PP/RNBQKBNR b KQkq f3 0 3' => {
			eco => 'B06u',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. f4

			variation => N__('Modern: Three Pawns Attack'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'B06b',
                        # TRANSLATORS: 1. e4 g6 2. Nc3 Bg7

			variation => N__('Modern'),
			moves => {
				'f2f4' => 'rnbqk1nr/ppppppbp/6p1/8/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'B06b',
                        # TRANSLATORS:

			variation => N__('Modern'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B06c',
                        # TRANSLATORS: 1. e4 g6 2. f4 Bg7 3. Nf3

			variation => N__('Modern'),
			moves => {
				'd7d6' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B06c',
                        # TRANSLATORS:

			variation => N__('Modern'),
			moves => {
				'g1f3' => 'rnbqk1nr/ppppppbp/6p1/8/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 g6 2. Nf3 Bg7 3. e3

			variation => N__('Bird: 1..g6'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: 1..g6'),
			moves => {
				'e2e3' => 'rnbqk1nr/ppppppbp/6p1/8/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqk1nr/ppppppbp/6p1/8/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 g6 2. Nf3 Bg7 3. g3

			variation => N__('Bird: 1..g6'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 g6 2. g3 Bg7

			variation => N__('Reti: 1... g6'),
			moves => {
				'f1g2' => 'rnbqk1nr/ppppppbp/6p1/8/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3',
			},
		},
		'rnbqk1nr/ppppppbp/6p1/8/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 g6 2. g3 Bg7 3. Bg2

			variation => N__('Reti: 1... g6'),
			moves => {
			},
		},
		'rnbqk1nr/ppppppbp/8/6p1/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g5 2. d4 Bg7

			variation => N__('English: Myers Gambit'),
			moves => {
			},
		},
		'rnbqk1r1/ppp2pP1/4p3/3p2p1/1b1P3P/2N5/PPP2PP1/R2QKBNR b KQq h3 0 8' => {
			eco => 'C12e',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, Chigorin Variation'),
			moves => {
				'g5h4' => 'rnbqk1r1/ppp2pP1/4p3/3p4/1b1P3p/2N5/PPP2PP1/R2QKBNR w KQq - 0 9',
			},
		},
		'rnbqk1r1/ppp2pP1/4p3/3p2p1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQq - 1 8' => {
			eco => 'C12e',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, Chigorin Variation'),
			moves => {
				'h2h4' => 'rnbqk1r1/ppp2pP1/4p3/3p2p1/1b1P3P/2N5/PPP2PP1/R2QKBNR b KQq h3 0 8',
			},
		},
		'rnbqk1r1/ppp2pP1/4p3/3p4/1b1P2Qp/2N5/PPP2PP1/R3KBNR b KQq - 1 9' => {
			eco => 'C12e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. exf6 hxg5 7. fxg7 Rg8 8. h4 gxh4 9. Qg4

			variation => N__('French: MacCutcheon, Grigoriev Variation'),
			moves => {
			},
		},
		'rnbqk1r1/ppp2pP1/4p3/3p4/1b1P3p/2N5/PPP2PP1/R2QKBNR w KQq - 0 9' => {
			eco => 'C12e',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, Chigorin Variation'),
			moves => {
				'd1g4' => 'rnbqk1r1/ppp2pP1/4p3/3p4/1b1P2Qp/2N5/PPP2PP1/R3KBNR b KQq - 1 9',
			},
		},
		'rnbqk2r/1p1n1pbp/p2p2p1/1BpP4/P3PP2/2N5/1P4PP/R1BQK1NR w KQkq - 0 10' => {
			eco => 'A67m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 a6

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 a6'),
			moves => {
				'b5d3' => 'rnbqk2r/1p1n1pbp/p2p2p1/2pP4/P3PP2/2NB4/1P4PP/R1BQK1NR b KQkq - 1 10',
				'b5e2' => 'rnbqk2r/1p1n1pbp/p2p2p1/2pP4/P3PP2/2N5/1P2B1PP/R1BQK1NR b KQkq - 1 10',
			},
		},
		'rnbqk2r/1p1n1pbp/p2p2p1/2pP4/P3PP2/2N5/1P2B1PP/R1BQK1NR b KQkq - 1 10' => {
			eco => 'A67o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 a6 10. Be2

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 a6 10. Be2'),
			moves => {
			},
		},
		'rnbqk2r/1p1n1pbp/p2p2p1/2pP4/P3PP2/2NB4/1P4PP/R1BQK1NR b KQkq - 1 10' => {
			eco => 'A67n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4 a6 10. Bd3

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 a6 10. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/1p1nbp2/p2pN2p/6P1/4P2B/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 11' => {
			eco => 'B98l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 g5 10. fxg5 Nfd7 11. Nxe6

			variation => N__('Sicilian: Najdorf, Gothenburg, 11.Nxe6'),
			moves => {
			},
		},
		'rnbqk2r/1p1nbp2/p2pp2p/6P1/3NP2B/2N2Q2/PPP3PP/R3KB1R w KQkq - 1 11' => {
			eco => 'B98k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Gothenburg (Argentine) Variation'),
			moves => {
				'd4e6' => 'rnbqk2r/1p1nbp2/p2pN2p/6P1/4P2B/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 11',
				'f3h5' => 'rnbqk2r/1p1nbp2/p2pp2p/6PQ/3NP2B/2N5/PPP3PP/R3KB1R b KQkq - 2 11',
			},
		},
		'rnbqk2r/1p1nbp2/p2pp2p/6PQ/3NP2B/2N5/PPP3PP/R3KB1R b KQkq - 2 11' => {
			eco => 'B98m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 g5 10. fxg5 Nfd7 11. Qh5

			variation => N__('Sicilian: Najdorf, Gothenburg, 11. Qh5'),
			moves => {
			},
		},
		'rnbqk2r/1p1p1ppp/p3pn2/8/1bPNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'B41k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6 6. Nc3 Bb4

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3 Bb4'),
			moves => {
				'f1d3' => 'rnbqk2r/1p1p1ppp/p3pn2/8/1bPNP3/2NB4/PP3PPP/R1BQK2R b KQkq - 4 7',
			},
		},
		'rnbqk2r/1p1p1ppp/p3pn2/8/1bPNP3/2NB4/PP3PPP/R1BQK2R b KQkq - 4 7' => {
			eco => 'B41k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3 Bb4'),
			moves => {
				'b8c6' => 'r1bqk2r/1p1p1ppp/p1n1pn2/8/1bPNP3/2NB4/PP3PPP/R1BQK2R w KQkq - 5 8',
			},
		},
		'rnbqk2r/1p2bp2/p2ppn1p/6P1/3NP2B/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 10' => {
			eco => 'B98k',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Gothenburg (Argentine) Variation'),
			moves => {
				'f6d7' => 'rnbqk2r/1p1nbp2/p2pp2p/6P1/3NP2B/2N2Q2/PPP3PP/R3KB1R w KQkq - 1 11',
			},
		},
		'rnbqk2r/1p2bp2/p2ppn1p/6p1/3NPP1B/2N2Q2/PPP3PP/R3KB1R w KQkq g6 0 10' => {
			eco => 'B98k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6 9. Bh4 g5

			variation => N__('Sicilian: Najdorf, Gothenburg (Argentine) Variation'),
			moves => {
				'f4g5' => 'rnbqk2r/1p2bp2/p2ppn1p/6P1/3NP2B/2N2Q2/PPP3PP/R3KB1R b KQkq - 0 10',
			},
		},
		'rnbqk2r/1p2bpp1/p2ppn1p/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 0 9' => {
			eco => 'B98f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3 h6

			variation => N__('Sicilian: Najdorf, 7... Be7 8. Qf3 h6'),
			moves => {
				'g5h4' => 'rnbqk2r/1p2bpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/R3KB1R b KQkq - 1 9',
			},
		},
		'rnbqk2r/1p2bpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/R3KB1R b KQkq - 1 9' => {
			eco => 'B98f',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 7... Be7 8. Qf3 h6'),
			moves => {
				'd8c7' => 'rnb1k2r/1pq1bpp1/p2ppn1p/8/3NPP1B/2N2Q2/PPP3PP/R3KB1R w KQkq - 2 10',
				'g7g5' => 'rnbqk2r/1p2bp2/p2ppn1p/6p1/3NPP1B/2N2Q2/PPP3PP/R3KB1R w KQkq g6 0 10',
			},
		},
		'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2QK2R b KQkq - 3 8' => {
			eco => 'B92g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. Be3

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8. Be3'),
			moves => {
				'c8e6' => 'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN1B3/PPP1BPPP/R2QK2R w KQkq - 4 9',
			},
		},
		'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN3P1/PPP2P1P/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B91h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nb3 Be7

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7. Nb3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 b kq - 3 8' => {
			eco => 'B92h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7 8. O-O

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7 8.O-O'),
			moves => {
				'c8e6' => 'rn1qk2r/1p2bppp/p2pbn2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w kq - 4 9',
				'e8g8' => 'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 w - - 4 9',
			},
		},
		'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B92f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3 Be7

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3 Be7'),
			moves => {
				'c1e3' => 'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN1B3/PPP1BPPP/R2QK2R b KQkq - 3 8',
				'e1g1' => 'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQ1RK1 b kq - 3 8',
			},
		},
		'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R w KQkq - 2 8' => {
			eco => 'B91l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nde2 Be7

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2 Be7'),
			moves => {
				'f1g2' => 'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/2N3P1/PPP1NPBP/R1BQK2R b KQkq - 3 8',
			},
		},
		'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/2N3P1/PPP1NPBP/R1BQK2R b KQkq - 3 8' => {
			eco => 'B91l',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2 Be7'),
			moves => {
				'b8d7' => 'r1bqk2r/1p1nbppp/p2p1n2/4p3/4P3/2N3P1/PPP1NPBP/R1BQK2R w KQkq - 4 9',
				'e8g8' => 'rnbq1rk1/1p2bppp/p2p1n2/4p3/4P3/2N3P1/PPP1NPBP/R1BQK2R w KQ - 4 9',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/2KR1B1R b kq - 3 8' => {
			eco => 'B95l',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qf3 Be7'),
			moves => {
				'b8d7' => 'r1bqk2r/1p1nbppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/2KR1B1R w kq - 4 9',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B95l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qf3 Be7

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qf3 Be7'),
			moves => {
				'e1c1' => 'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/2KR1B1R b kq - 3 8',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 3 8' => {
			eco => 'B95g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qd2 Be7 8. O-O-O

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qd2 Be7 8.O-O-O'),
			moves => {
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 2 8' => {
			eco => 'B95f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qd2 Be7

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qd2 Be7'),
			moves => {
				'e1c1' => 'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N5/PPPQ1PPP/2KR1B1R b kq - 3 8',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 2 8' => {
			eco => 'B98b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7 8. Qf3

			variation => N__('Sicilian: Najdorf, 7... Be7 8. Qf3'),
			moves => {
				'd8a5' => 'rnb1k2r/1p2bppp/p2ppn2/q5B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 3 9',
				'd8c7' => 'rnb1k2r/1pq1bppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 3 9',
				'h7h6' => 'rnbqk2r/1p2bpp1/p2ppn1p/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R w KQkq - 0 9',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8' => {
			eco => 'B98a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 Be7

			variation => N__('Sicilian: Najdorf, 7... Be7'),
			moves => {
				'd1f3' => 'rnbqk2r/1p2bppp/p2ppn2/6B1/3NPP2/2N2Q2/PPP3PP/R3KB1R b KQkq - 2 8',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R w KQkq - 2 8' => {
			eco => 'B86',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3 Be7

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7'),
			moves => {
				'f2f4' => 'rnbqk2r/1p2bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 b kq f3 0 8',
			},
		},
		'rnbqk2r/1p2bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 b kq f3 0 8' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O Be7 8. f4

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O Be7 8. f4'),
			moves => {
				'd8c7' => 'rnb1k2r/1pq1bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 w kq - 1 9',
				'e8g8' => 'rnbq1rk1/1p2bppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQ1RK1 w - - 1 9',
			},
		},
		'rnbqk2r/1p2ppb1/p2p3p/6p1/3NP1n1/2N3B1/PPP1BPPP/R2QK2R b KQkq - 3 10' => {
			eco => 'B90j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 Ng4 7. Bg5 h6 8. Bh4 g5 9. Bg3 Bg7 10. Be2

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5 h6 8. Bh4 g5 9. Bg3 Bg7 10. Be2'),
			moves => {
				'h6h5' => 'rnbqk2r/1p2ppb1/p2p4/6pp/3NP1n1/2N3B1/PPP1BPPP/R2QK2R w KQkq - 0 11',
			},
		},
		'rnbqk2r/1p2ppb1/p2p3p/6p1/3NP1n1/2N3B1/PPP2PPP/R2QKB1R w KQkq - 2 10' => {
			eco => 'B90i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 Ng4 7. Bg5 h6 8. Bh4 g5 9. Bg3 Bg7

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5 h6 8. Bh4 g5 9. Bg3 Bg7'),
			moves => {
				'f1e2' => 'rnbqk2r/1p2ppb1/p2p3p/6p1/3NP1n1/2N3B1/PPP1BPPP/R2QK2R b KQkq - 3 10',
			},
		},
		'rnbqk2r/1p2ppb1/p2p4/6pp/3NP1n1/2N3B1/PPP1BPPP/R2QK2R w KQkq - 0 11' => {
			eco => 'B90j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 Ng4 7. Bg5 h6 8. Bh4 g5 9. Bg3 Bg7 10. Be2 h5

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5 h6 8. Bh4 g5 9. Bg3 Bg7 10. Be2 h5'),
			moves => {
			},
		},
		'rnbqk2r/1p2ppbp/p1pp1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQkq - 0 7' => {
			eco => 'E80n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c6 6. Be3 a6

			variation => N__('King\'s Indian: Sämisch, 5... c6 6. Be3 a6'),
			moves => {
				'a2a4' => 'rnbqk2r/1p2ppbp/p1pp1np1/8/P1PPP3/2N1BP2/1P4PP/R2QKBNR b KQkq a3 0 7',
				'f1d3' => 'rnbqk2r/1p2ppbp/p1pp1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR b KQkq - 1 7',
			},
		},
		'rnbqk2r/1p2ppbp/p1pp1np1/8/2PPP3/2NBBP2/PP4PP/R2QK1NR b KQkq - 1 7' => {
			eco => 'E80p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c6 6. Be3 a6 7. Bd3

			variation => N__('King\'s Indian: Sämisch, 5... c6 6. Be3 a6 7. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/1p2ppbp/p1pp1np1/8/P1PPP3/2N1BP2/1P4PP/R2QKBNR b KQkq a3 0 7' => {
			eco => 'E80o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c6 6. Be3 a6 7. a4

			variation => N__('King\'s Indian: Sämisch, 5... c6 6. Be3 a6 7. a4'),
			moves => {
			},
		},
		'rnbqk2r/1p2ppbp/p2p1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq - 0 8' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3 a6

			variation => N__('Sicilian: Dragon, Yugoslav, 7... a6'),
			moves => {
			},
		},
		'rnbqk2r/1p2ppbp/p2p1np1/8/P2NP3/2N5/1PP1BPPP/R1BQK2R w KQkq - 1 8' => {
			eco => 'B70j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 a6 7. a4 Bg7

			variation => N__('Sicilian: Dragon, 6. Be2 a6 7. a4 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/1p3pb1/p2p1npp/2pP4/4P2B/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'A71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bg5 h6 9. Bh4 a6

			variation => N__('Benoni: Classical, 8. Bg5 h6 9. Bh4 a6'),
			moves => {
				'f3d2' => 'rnbqk2r/1p3pb1/p2p1npp/2pP4/4P2B/2N5/PP1N1PPP/R2QKB1R b KQkq - 1 10',
			},
		},
		'rnbqk2r/1p3pb1/p2p1npp/2pP4/4P2B/2N5/PP1N1PPP/R2QKB1R b KQkq - 1 10' => {
			eco => 'A71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bg5 h6 9. Bh4 a6 10. Nd2

			variation => N__('Benoni: Classical, 8. Bg5 h6 9. Bh4 a6 10. Nd2'),
			moves => {
				'b7b5' => 'rnbqk2r/5pb1/p2p1npp/1ppP4/4P2B/2N5/PP1N1PPP/R2QKB1R w KQkq b6 0 11',
			},
		},
		'rnbqk2r/1p3pbp/p2p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQK2R w KQkq - 0 9' => {
			eco => 'A70e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 a6

			variation => N__('Benoni: Classical, 8. Be2 a6'),
			moves => {
			},
		},
		'rnbqk2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP2QPPP/RNB1K2R w KQkq - 0 8' => {
			eco => 'D26l',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Furman, 6... a6'),
			moves => {
				'e1g1' => 'rnbqk2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP2QPPP/RNB2RK1 b kq - 1 8',
			},
		},
		'rnbqk2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP2QPPP/RNB2RK1 b kq - 1 8' => {
			eco => 'D26m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Qe2 a6 7. dxc5 Bxc5 8. O-O

			variation => N__('QGA: Classical, Furman, 6... a6 7.dxc5 Bxc5 8.O-O'),
			moves => {
				'b7b5' => 'rnbqk2r/5ppp/p3pn2/1pb5/2B5/4PN2/PP2QPPP/RNB2RK1 w kq b6 0 9',
				'b8c6' => 'r1bqk2r/1p3ppp/p1n1pn2/2b5/2B5/4PN2/PP2QPPP/RNB2RK1 w kq - 2 9',
			},
		},
		'rnbqk2r/1pp1ppbp/p2p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR b KQkq - 1 6' => {
			eco => 'E80k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 a6 6. Be3

			variation => N__('King\'s Indian: Sämisch, 5... a6 6. Be3'),
			moves => {
			},
		},
		'rnbqk2r/1pp1ppbp/p2p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'E80j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 a6

			variation => N__('King\'s Indian: Sämisch, 5... a6'),
			moves => {
				'c1e3' => 'rnbqk2r/1pp1ppbp/p2p1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR b KQkq - 1 6',
			},
		},
		'rnbqk2r/1pp1ppbp/p2p1np1/8/3PP3/2N1BN2/PPP2PPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'B08h',
                        # TRANSLATORS:

			variation => N__('Pirc: Classical, 5. Be3'),
			moves => {
				'a2a4' => 'rnbqk2r/1pp1ppbp/p2p1np1/8/P2PP3/2N1BN2/1PP2PPP/R2QKB1R b KQkq a3 0 6',
			},
		},
		'rnbqk2r/1pp1ppbp/p2p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'B08j',
                        # TRANSLATORS:

			variation => N__('Pirc: Classical, 5. Be2'),
			moves => {
				'a2a4' => 'rnbqk2r/1pp1ppbp/p2p1np1/8/P2PP3/2N2N2/1PP1BPPP/R1BQK2R b KQkq a3 0 6',
			},
		},
		'rnbqk2r/1pp1ppbp/p2p1np1/8/P2PP3/2N1BN2/1PP2PPP/R2QKB1R b KQkq a3 0 6' => {
			eco => 'B08h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be3 a6 6. a4

			variation => N__('Pirc: Classical, 5. Be3 a6 6. a4'),
			moves => {
			},
		},
		'rnbqk2r/1pp1ppbp/p2p1np1/8/P2PP3/2N2N2/1PP1BPPP/R1BQK2R b KQkq a3 0 6' => {
			eco => 'B08j',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 a6 6. a4

			variation => N__('Pirc: Classical, 5. Be2 a6 6. a4'),
			moves => {
			},
		},
		'rnbqk2r/1pp1ppbp/p4np1/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4 a6

			variation => N__('Grünfeld: Russian, 6... a6'),
			moves => {
			},
		},
		'rnbqk2r/1ppn1ppp/p3p3/3pP1b1/3P2QP/2N5/PPP2PP1/R3KBNR w KQkq - 0 8' => {
			eco => 'C13o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 a6 7. Qg4 Bxg5

			variation => N__('French: Chatard-Alekhine, Maroczy, 7. Qg4 Bxg5'),
			moves => {
			},
		},
		'rnbqk2r/1ppnbppp/p3p3/3pP1B1/3P2QP/2N5/PPP2PP1/R3KBNR b KQkq - 1 7' => {
			eco => 'C13n',
                        # TRANSLATORS:

			variation => N__('French: Chatard-Alekhine, Maroczy Variation'),
			moves => {
				'e7g5' => 'rnbqk2r/1ppn1ppp/p3p3/3pP1b1/3P2QP/2N5/PPP2PP1/R3KBNR w KQkq - 0 8',
			},
		},
		'rnbqk2r/1ppnbppp/p3p3/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq - 0 7' => {
			eco => 'C13n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 a6

			variation => N__('French: Chatard-Alekhine, Maroczy Variation'),
			moves => {
				'd1g4' => 'rnbqk2r/1ppnbppp/p3p3/3pP1B1/3P2QP/2N5/PPP2PP1/R3KBNR b KQkq - 1 7',
			},
		},
		'rnbqk2r/1ppp1ppp/4pn2/p7/1bPP4/2N2N2/PP1BPPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'E11l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 a5 5. Nc3

			variation => N__('Bogo-Indian: 4. Bd2 a5 5. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/1ppp1ppp/4pn2/p7/1bPP4/5N2/PP1BPPPP/RN1QKB1R w KQkq a6 0 5' => {
			eco => 'E11l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 a5

			variation => N__('Bogo-Indian: 4. Bd2 a5'),
			moves => {
				'b1c3' => 'rnbqk2r/1ppp1ppp/4pn2/p7/1bPP4/2N2N2/PP1BPPPP/R2QKB1R b KQkq - 1 5',
				'g2g3' => 'rnbqk2r/1ppp1ppp/4pn2/p7/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/1ppp1ppp/4pn2/p7/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 0 5' => {
			eco => 'E11m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 a5 5. g3

			variation => N__('Bogo-Indian: 4. Bd2 a5 5. g3'),
			moves => {
			},
		},
		'rnbqk2r/2pp1ppp/1p2pn2/p7/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq a6 0 6' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Qb3 a5

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Qb3 a5'),
			moves => {
			},
		},
		'rnbqk2r/3pppbp/p4np1/1PpP4/8/2N1P3/PP3PPP/R1BQKBNR w KQkq - 2 7' => {
			eco => 'A57n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. e3 g6 6. Nc3 Bg7

			variation => N__('Benko Gambit: 4.cxb5 a6 5. e3 g6 6. Nc3 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/4bppp/p2ppn2/1p6/3NP3/1BN2Q2/PPP2PPP/R1B2RK1 b kq - 3 9' => {
			eco => 'B87',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3 b5 8. O-O Be7 9. Qf3

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3 b5 8.O-O Be7 9. Qf3'),
			moves => {
			},
		},
		'rnbqk2r/4bppp/p2ppn2/1p6/3NP3/1BN5/PPP2PPP/R1BQ1RK1 w kq - 2 9' => {
			eco => 'B87',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3 b5 8. O-O Be7

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3 b5 8.O-O Be7'),
			moves => {
				'd1f3' => 'rnbqk2r/4bppp/p2ppn2/1p6/3NP3/1BN2Q2/PPP2PPP/R1B2RK1 b kq - 3 9',
				'f2f4' => 'rnbqk2r/4bppp/p2ppn2/1p6/3NPP2/1BN5/PPP3PP/R1BQ1RK1 b kq f3 0 9',
			},
		},
		'rnbqk2r/4bppp/p2ppn2/1p6/3NPP2/1BN5/PPP3PP/R1BQ1RK1 b kq f3 0 9' => {
			eco => 'B87',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3 b5 8. O-O Be7 9. f4

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3 b5 8.O-O Be7 9. f4'),
			moves => {
			},
		},
		'rnbqk2r/5pb1/p2p1npp/1ppP4/4P2B/2N5/PP1N1PPP/R2QKB1R w KQkq b6 0 11' => {
			eco => 'A71',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Bg5 h6 9. Bh4 a6 10. Nd2'),
			moves => {
				'f1e2' => 'rnbqk2r/5pb1/p2p1npp/1ppP4/4P2B/2N5/PP1NBPPP/R2QK2R b KQkq - 1 11',
			},
		},
		'rnbqk2r/5pb1/p2p1npp/1ppP4/4P2B/2N5/PP1NBPPP/R2QK2R b KQkq - 1 11' => {
			eco => 'A71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bg5 h6 9. Bh4 a6 10. Nd2 b5 11. Be2

			variation => N__('Benoni: Classical, 8. Bg5 h6 9. Bh4 a6 10. Nd2 b5 11. Be2'),
			moves => {
			},
		},
		'rnbqk2r/5ppp/p3pn2/1pb5/2B5/4PN2/PP2QPPP/RNB2RK1 w kq b6 0 9' => {
			eco => 'D26o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Qe2 a6 7. dxc5 Bxc5 8. O-O b5

			variation => N__('QGA: Classical, Furman, 6... a6 7.dxc5 Bxc5 8.O-O b5'),
			moves => {
			},
		},
		'rnbqk2r/p1p1bp1p/1p2pp2/8/2BPN3/5N2/PPP2PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'C11w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 gxf6 7. Nf3 b6 8. Bc4

			variation => N__('French: Burn, 6.Bxf6 gxf6 7. Nf3 b6 8. Bc4'),
			moves => {
			},
		},
		'rnbqk2r/p1p1bp1p/1p2pp2/8/3PN3/3B1N2/PPP2PPP/R2QK2R b KQkq - 1 8' => {
			eco => 'C11w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 gxf6 7. Nf3 b6 8. Bd3

			variation => N__('French: Burn, 6.Bxf6 gxf6 7. Nf3 b6 8. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/p1p1bp1p/1p2pp2/8/3PN3/5N2/PPP2PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'C11w',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 gxf6 7. Nf3 b6

			variation => N__('French: Burn, 6.Bxf6 gxf6 7. Nf3 b6'),
			moves => {
				'f1c4' => 'rnbqk2r/p1p1bp1p/1p2pp2/8/2BPN3/5N2/PPP2PPP/R2QK2R b KQkq - 1 8',
				'f1d3' => 'rnbqk2r/p1p1bp1p/1p2pp2/8/3PN3/3B1N2/PPP2PPP/R2QK2R b KQkq - 1 8',
			},
		},
		'rnbqk2r/p1p1nppp/1p2p3/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'C16l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Ne7 5. Bd2 b6

			variation => N__('French: Winawer, Advance, 4... Ne7 5. Bd2 b6'),
			moves => {
			},
		},
		'rnbqk2r/p1p1nppp/1p2p3/3pP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 1 7' => {
			eco => 'C16o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Ne7 5. a3 Bxc3+ 6. bxc3 b6 7. Qg4

			variation => N__('French: Winawer, Advance, 4... Ne7 5. a3 Bxc3+ 6.bxc3 b6 7. Qg4'),
			moves => {
			},
		},
		'rnbqk2r/p1p1nppp/1p2p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C16n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Ne7 5. a3 Bxc3+ 6. bxc3 b6

			variation => N__('French: Winawer, Advance, 4... Ne7 5. a3 Bxc3+ 6.bxc3 b6'),
			moves => {
				'd1g4' => 'rnbqk2r/p1p1nppp/1p2p3/3pP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 1 7',
			},
		},
		'rnbqk2r/p1p1ppbp/1p4p1/8/2BPP3/2P5/P4PPP/R1BQK1NR w KQkq - 0 8' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 b6

			variation => N__('Grünfeld: Classical Exchange, 7... b6'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2p3/8/1bPPn3/2N1P3/PP1BNPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'E44',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ne4 6. Bd2

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ne4 6. Bd2'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2p3/8/1bPPn3/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E44',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ne4

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ne4'),
			moves => {
				'c1d2' => 'rnbqk2r/p1pp1ppp/1p2p3/8/1bPPn3/2N1P3/PP1BNPPP/R2QKB1R b KQkq - 3 6',
				'd1c2' => 'rnbqk2r/p1pp1ppp/1p2p3/8/1bPPn3/2N1P3/PPQ1NPPP/R1B1KB1R b KQkq - 3 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2p3/8/1bPPn3/2N1P3/PPQ1NPPP/R1B1KB1R b KQkq - 3 6' => {
			eco => 'E44',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2 Ne4 6. Qc2

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2 Ne4 6. Qc2'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/6B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Bg5

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Bg5'),
			moves => {
				'c8b7' => 'rn1qk2r/pbpp1ppp/1p2pn2/6B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Qb3

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Qb3'),
			moves => {
				'a7a5' => 'rnbqk2r/2pp1ppp/1p2pn2/p7/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq a6 0 6',
				'c7c5' => 'rnbqk2r/p2p1ppp/1p2pn2/2p5/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq c6 0 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E44',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Ne2

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Ne2'),
			moves => {
				'c8a6' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 2 6',
				'c8b7' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 2 6',
				'f6e4' => 'rnbqk2r/p1pp1ppp/1p2p3/8/1bPPn3/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6

			variation => N__('Nimzo-Indian: Nimzowitsch (Fischer) Variation'),
			moves => {
				'f1d3' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 1 5',
				'g1e2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQkq - 1 5',
				'g1f3' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Nf3

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Nf3'),
			moves => {
				'c8b7' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid'),
			moves => {
				'c1g5' => 'rnbqk2r/p1pp1ppp/1p2pn2/6B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5',
				'd1b3' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5',
				'd1c2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Qc2

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Qc2'),
			moves => {
				'c8b7' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 2 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 5' => {
			eco => 'E32b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 b6

			variation => N__('Nimzo-Indian: Classical, 4... b6'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'E43',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 b6 5. Bd3

			variation => N__('Nimzo-Indian: Nimzowitsch, 5. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E14a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3 Bb4+

			variation => N__('Queen\'s Indian: 4. e3 Bb4+'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/5N2/PP1NPPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'E11c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 b6

			variation => N__('Bogo-Indian: 4.Nbd2 b6'),
			moves => {
				'a2a3' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/P4N2/1P1NPPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 2 5' => {
			eco => 'E15b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb4+ 5. Bd2

			variation => N__('Queen\'s Indian: 4. g3 Bb4+'),
			moves => {
				'b4d2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/5NP1/PP1bPP1P/RN1QKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5' => {
			eco => 'E15b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb4+

			variation => N__('Queen\'s Indian: 4. g3 Bb4+'),
			moves => {
				'c1d2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 2 5',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/P4N2/1P1NPPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'E11c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 b6 5. a3

			variation => N__('Bogo-Indian: 4.Nbd2 b6 5. a3'),
			moves => {
				'b4d2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P4N2/1P1bPPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPP1P/RN2KB1R b KQkq - 0 6' => {
			eco => 'E15c',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. g3 Bb4+ 5. Bd2 Bxd2+'),
			moves => {
				'c8a6' => 'rn1qk2r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PP1QPP1P/RN2KB1R w KQkq - 1 7',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/5NP1/PP1bPP1P/RN1QKB1R w KQkq - 0 6' => {
			eco => 'E15c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb4+ 5. Bd2 Bxd2+

			variation => N__('Queen\'s Indian: 4. g3 Bb4+ 5. Bd2 Bxd2+'),
			moves => {
				'd1d2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/5NP1/PP1QPP1P/RN2KB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 b6 6. f3

			variation => N__('Nimzo-Indian: Sämisch, 5... b6 6. f3'),
			moves => {
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P1P5/4PPPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 b6

			variation => N__('Nimzo-Indian: Sämisch, 5... b6'),
			moves => {
				'f2f3' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P4N2/1P1BPPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'E11d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 b6 5. a3 Bxd2+ 6. Bxd2

			variation => N__('Bogo-Indian: 4.Nbd2 b6 5. a3 Bxd2+ 6.Bxd2'),
			moves => {
				'c8b7' => 'rn1qk2r/pbpp1ppp/1p2pn2/8/2PP4/P4N2/1P1BPPPP/R2QKB1R w KQkq - 1 7',
			},
		},
		'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P4N2/1P1bPPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'E11c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 b6 5. a3 Bxd2+

			variation => N__('Bogo-Indian: 4.Nbd2 b6 5. a3 Bxd2+'),
			moves => {
				'c1d2' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P4N2/1P1BPPPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/p2p1pp1/4pn1p/1ppP4/1bP4B/2N5/PP2PPPP/R2QKBNR w KQkq b6 0 7' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 b5

			variation => N__('Nimzo-Indian: Leningrad, 6. d5 b5 Gambit'),
			moves => {
			},
		},
		'rnbqk2r/p2p1ppp/1p2pn2/2p5/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq c6 0 6' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 b6 5. Qb3 c5

			variation => N__('Nimzo-Indian: Nimzo-Queen\'s Hybrid, 5. Qb3 c5'),
			moves => {
			},
		},
		'rnbqk2r/p2p1ppp/1p2pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'E26',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. e3 b6

			variation => N__('Nimzo-Indian: Sämisch, O\'Kelly Variation'),
			moves => {
			},
		},
		'rnbqk2r/p3nppp/1p2p3/1BppP3/3P4/P1P2N2/2P2PPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'C19i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 b6 8. Bb5+

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 b6 8. Bb5+'),
			moves => {
			},
		},
		'rnbqk2r/p3nppp/1p2p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'C19h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 b6

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 b6'),
			moves => {
				'a3a4' => 'rnbqk2r/p3nppp/1p2p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8',
				'f1b5' => 'rnbqk2r/p3nppp/1p2p3/1BppP3/3P4/P1P2N2/2P2PPP/R1BQK2R b KQkq - 1 8',
			},
		},
		'rnbqk2r/p3nppp/1p2p3/2ppP3/P2P4/2P2N2/2P2PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'C19h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3 b6 8. a4

			variation => N__('French: Winawer, 6... Ne7 7. Nf3 b6 8. a4'),
			moves => {
			},
		},
		'rnbqk2r/p3ppbp/2pp1np1/1p6/3PP3/2N1BP2/PPPQ2PP/R3KBNR w KQkq b6 0 7' => {
			eco => 'B07v',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be3 Bg7 5. Qd2 c6 6. f3 b5

			variation => N__('Pirc: 4. Be3, 150 Attack'),
			moves => {
			},
		},
		'rnbqk2r/pp1n1pbp/3N2p1/2pPp3/5P2/8/PP4PP/R1BQKBNR b KQkq - 1 10' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7 9. Nb5 dxe5 10. Nd6+

			variation => N__('Benoni: Mikenas Attack, 9. Nb5 dxe5 10. Nd6+'),
			moves => {
				'e8e7' => 'rnbq3r/pp1nkpbp/3N2p1/2pPp3/5P2/8/PP4PP/R1BQKBNR w KQ - 2 11',
			},
		},
		'rnbqk2r/pp1n1pbp/3p2p1/1BpP4/4PP2/2N5/PP4PP/R1BQK1NR w KQkq - 3 9' => {
			eco => 'A67d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7

			variation => N__('Benoni: Four Pawns, Taimanov, 8...Nfd7'),
			moves => {
				'a2a4' => 'rnbqk2r/pp1n1pbp/3p2p1/1BpP4/P3PP2/2N5/1P4PP/R1BQK1NR b KQkq a3 0 9',
				'b5d3' => 'rnbqk2r/pp1n1pbp/3p2p1/2pP4/4PP2/2NB4/PP4PP/R1BQK1NR b KQkq - 4 9',
			},
		},
		'rnbqk2r/pp1n1pbp/3p2p1/1BpP4/P3PP2/2N5/1P4PP/R1BQK1NR b KQkq a3 0 9' => {
			eco => 'A67k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. a4

			variation => N__('Benoni: Four Pawns, Taimanov, 9. a4 (Zaitsev)'),
			moves => {
				'a7a6' => 'rnbqk2r/1p1n1pbp/p2p2p1/1BpP4/P3PP2/2N5/1P4PP/R1BQK1NR w KQkq - 0 10',
				'd8h4' => 'rnb1k2r/pp1n1pbp/3p2p1/1BpP4/P3PP1q/2N5/1P4PP/R1BQK1NR w KQkq - 1 10',
				'e8g8' => 'rnbq1rk1/pp1n1pbp/3p2p1/1BpP4/P3PP2/2N5/1P4PP/R1BQK1NR w KQ - 1 10',
			},
		},
		'rnbqk2r/pp1n1pbp/3p2p1/1NpPP3/5P2/8/PP4PP/R1BQKBNR b KQkq - 2 9' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7 9. Nb5

			variation => N__('Benoni: Mikenas Attack, 8...Nfd7 9. Nb5'),
			moves => {
				'd6e5' => 'rnbqk2r/pp1n1pbp/6p1/1NpPp3/5P2/8/PP4PP/R1BQKBNR w KQkq - 0 10',
			},
		},
		'rnbqk2r/pp1n1pbp/3p2p1/2pP4/4PP2/2NB4/PP4PP/R1BQK1NR b KQkq - 4 9' => {
			eco => 'A67e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+ Nfd7 9. Bd3

			variation => N__('Benoni: Four Pawns, Taimanov, 9. Bd3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp1n1pbp/3p2p1/2pP4/4PP2/2NB4/PP4PP/R1BQK1NR w KQ - 5 10',
			},
		},
		'rnbqk2r/pp1n1pbp/3p2p1/2pPP3/5P2/2N2N2/PP4PP/R1BQKB1R b KQkq - 2 9' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7 9. Nf3

			variation => N__('Benoni: Mikenas Attack, 8...Nfd7 9. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pp1n1pbp/3p2p1/2pPP3/5P2/2N5/PP4PP/R1BQKBNR w KQkq - 1 9' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7

			variation => N__('Benoni: Mikenas Attack, 8...Nfd7'),
			moves => {
				'c3b5' => 'rnbqk2r/pp1n1pbp/3p2p1/1NpPP3/5P2/8/PP4PP/R1BQKBNR b KQkq - 2 9',
				'e5e6' => 'rnbqk2r/pp1n1pbp/3pP1p1/2pP4/5P2/2N5/PP4PP/R1BQKBNR b KQkq - 0 9',
				'g1f3' => 'rnbqk2r/pp1n1pbp/3p2p1/2pPP3/5P2/2N2N2/PP4PP/R1BQKB1R b KQkq - 2 9',
			},
		},
		'rnbqk2r/pp1n1pbp/3pP1p1/2pP4/5P2/2N5/PP4PP/R1BQKBNR b KQkq - 0 9' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7 9. e6

			variation => N__('Benoni: Mikenas Attack, 8...Nfd7 9. e6'),
			moves => {
			},
		},
		'rnbqk2r/pp1n1pbp/6p1/1NpPp3/5P2/8/PP4PP/R1BQKBNR w KQkq - 0 10' => {
			eco => 'A66',
                        # TRANSLATORS:

			variation => N__('Benoni: Mikenas Attack, 8...Nfd7 9. Nb5'),
			moves => {
				'b5d6' => 'rnbqk2r/pp1n1pbp/3N2p1/2pPp3/5P2/8/PP4PP/R1BQKBNR b KQkq - 1 10',
			},
		},
		'rnbqk2r/pp1n1ppp/4p3/2bpP3/5P2/2N5/PPP3PP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C11g',
                        # TRANSLATORS:

			variation => N__('French: Steinitz, 6.dxc5'),
			moves => {
				'd1g4' => 'rnbqk2r/pp1n1ppp/4p3/2bpP3/5PQ1/2N5/PPP3PP/R1B1KBNR b KQkq - 1 7',
			},
		},
		'rnbqk2r/pp1n1ppp/4p3/2bpP3/5PQ1/2N5/PPP3PP/R1B1KBNR b KQkq - 1 7' => {
			eco => 'C11g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. dxc5 Bxc5 7. Qg4

			variation => N__('French: Steinitz, Bradford Attack'),
			moves => {
			},
		},
		'rnbqk2r/pp1nBppp/4p3/2ppP3/3P3P/2N5/PPP2PP1/R2QKBNR b KQkq - 0 7' => {
			eco => 'C13q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 c5 7. Bxe7

			variation => N__('French: Chatard-Alekhine, Breyer, 7.Bxe7'),
			moves => {
				'e8e7' => 'rnbq3r/pp1nkppp/4p3/2ppP3/3P3P/2N5/PPP2PP1/R2QKBNR w KQ - 0 8',
			},
		},
		'rnbqk2r/pp1nbppp/4p3/2ppP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq c6 0 7' => {
			eco => 'C13p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 c5

			variation => N__('French: Chatard-Alekhine, Breyer Variation'),
			moves => {
				'g5e7' => 'rnbqk2r/pp1nBppp/4p3/2ppP3/3P3P/2N5/PPP2PP1/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp1nbppp/8/2ppP1B1/2p5/2N2N2/PP3PPP/R2QKB1R w KQkq - 3 9' => {
			eco => 'D24g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 c5 5. d5 e6 6. e4 exd5 7. e5 Nfd7 8. Bg5 Be7

			variation => N__('QGA: 4. Nc3 c5 5. d5 e6 6. e4'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1pp1/4pn1p/2p5/1bPP3B/2N5/PP2PPPP/R2QKBNR w KQkq c6 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5

			variation => N__('Nimzo-Indian: Leningrad, 4... h6 5. Bh4 c5'),
			moves => {
				'd4d5' => 'rnbqk2r/pp1p1pp1/4pn1p/2pP4/1bP4B/2N5/PP2PPPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1pp1/4pn1p/2pP2B1/1bP5/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5 5. d5 h6

			variation => N__('Nimzo-Indian: Leningrad, 4... c5 5. d5 h6'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1pp1/4pn1p/2pP4/1bP4B/2N5/PP2PPPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5

			variation => N__('Nimzo-Indian: Leningrad, 6. d5'),
			moves => {
				'b4c3' => 'rnbqk2r/pp1p1pp1/4pn1p/2pP4/2P4B/2b5/PP2PPPP/R2QKBNR w KQkq - 0 7',
				'b7b5' => 'rnbqk2r/p2p1pp1/4pn1p/1ppP4/1bP4B/2N5/PP2PPPP/R2QKBNR w KQkq b6 0 7',
				'd7d6' => 'rnbqk2r/pp3pp1/3ppn1p/2pP4/1bP4B/2N5/PP2PPPP/R2QKBNR w KQkq - 0 7',
				'e6d5' => 'rnbqk2r/pp1p1pp1/5n1p/2pp4/1bP4B/2N5/PP2PPPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/pp1p1pp1/4pn1p/2pP4/2P4B/2b5/PP2PPPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 Bxc3+

			variation => N__('Nimzo-Indian: Leningrad, 6. d5 Bxc3+'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1pp1/5n1p/2pP4/1b5B/2N5/PP2PPPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 exd5 7. cxd5

			variation => N__('Nimzo-Indian: Leningrad, 6. d5 exd5'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1pp1/5n1p/2pp4/1bP4B/2N5/PP2PPPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 exd5

			variation => N__('Nimzo-Indian: Leningrad, 6. d5 exd5'),
			moves => {
				'c4d5' => 'rnbqk2r/pp1p1pp1/5n1p/2pP4/1b5B/2N5/PP2PPPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp1p1ppp/3b1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6' => {
			eco => 'A60m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 Bd6 6. e4

			variation => N__('Benoni: Snake, 6. e4'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp1p1ppp/3b1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQ - 1 7',
			},
		},
		'rnbqk2r/pp1p1ppp/3b1n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'A60k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 Bd6 6. Nf3

			variation => N__('Benoni: Snake, 6. Nf3'),
			moves => {
				'd6c7' => 'rnbqk2r/ppbp1ppp/5n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 3 7',
			},
		},
		'rnbqk2r/pp1p1ppp/3b1n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'A60j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 Bd6

			variation => N__('Benoni: Snake Variation'),
			moves => {
				'e2e4' => 'rnbqk2r/pp1p1ppp/3b1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6',
				'g1f3' => 'rnbqk2r/pp1p1ppp/3b1n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'rnbqk2r/pp1p1ppp/3b1n2/2pP4/8/5N2/PP2PPPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'A60d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nf3 exd5 5. cxd5 Bd6

			variation => N__('Benoni: Snake Variation vs. 4. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4p3/2p5/1bPPn3/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 2 6' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 Ne4

			variation => N__('Nimzo-Indian: Romanishin, 4... c5 5. Nf3 Ne4'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4p3/2pP4/1bP1n3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 c5 5. d5 Ne4

			variation => N__('Nimzo-Indian: Three Knights, Euwe Variation'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/1N6/1bP5/2N5/PP2PPPP/R1BQKB1R b KQkq - 3 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Bb4 6. Nb5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Bb4 6. Nb5'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2P5/1bP5/1QN5/PP2PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. dxc5

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5.dxc5'),
			moves => {
				'b8a6' => 'r1bqk2r/pp1p1ppp/n3pn2/2P5/1bP5/1QN5/PP2PPPP/R1B1KBNR w KQkq - 1 6',
				'b8c6' => 'r1bqk2r/pp1p1ppp/2n1pn2/2P5/1bP5/1QN5/PP2PPPP/R1B1KBNR w KQkq - 1 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E38d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5'),
			moves => {
				'b4c5' => 'rnbqk2r/pp1p1ppp/4pn2/2b5/2P5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 6',
				'b8a6' => 'r1bqk2r/pp1p1ppp/n3pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 1 6',
				'b8c6' => 'r1bqk2r/pp1p1ppp/2n1pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 1 6',
				'd8c7' => 'rnb1k2r/ppqp1ppp/4pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 1 6',
				'e8g8' => 'rnbq1rk1/pp1p1ppp/4pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2b5/2P5/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 6' => {
			eco => 'E38e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Bxc5 6. Nf3

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Bxc5 6. Nf3'),
			moves => {
				'd8b6' => 'rnb1k2r/pp1p1ppp/1q2pn2/2b5/2P5/2N2N2/PPQ1PPPP/R1B1KB1R w KQkq - 2 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2b5/2P5/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 6' => {
			eco => 'E38e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. dxc5 Bxc5

			variation => N__('Nimzo-Indian: Classical, 4... c5 5.dxc5 Bxc5'),
			moves => {
				'g1f3' => 'rnbqk2r/pp1p1ppp/4pn2/2b5/2P5/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p3B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQkq c6 0 5' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5

			variation => N__('Nimzo-Indian: Leningrad, 4... c5'),
			moves => {
				'd4d5' => 'rnbqk2r/pp1p1ppp/4pn2/2pP2B1/1bP5/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1BPP4/5N2/PP2PPPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'E11j',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: Vitolins Variation'),
			moves => {
				'c5b4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1pPP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/1QN1P3/PP3PPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. e3

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5. e3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5 5. Nf3

			variation => N__('Nimzo-Indian: Spielmann, 4... c5 5. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/1QN5/PP2PPPP/R1B1KBNR w KQkq c6 0 5' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3 c5

			variation => N__('Nimzo-Indian: Spielmann, 4... c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp1p1ppp/4pn2/2P5/1bP5/1QN5/PP2PPPP/R1B1KBNR b KQkq - 0 5',
				'e2e3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/1QN1P3/PP3PPP/R1B1KBNR b KQkq - 0 5',
				'g1f3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E42',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Ne2

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2'),
			moves => {
				'c5d4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPp4/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 0 6',
				'd7d5' => 'rnbqk2r/pp3ppp/4pn2/2pp4/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq d6 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq c6 0 5' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5

			variation => N__('Nimzo-Indian: 4. e3 c5'),
			moves => {
				'f1d3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 1 5',
				'g1e2' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1P3/PP2NPPP/R1BQKB1R b KQkq - 1 5',
				'g1f3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E38c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. e3

			variation => N__('Nimzo-Indian: Classical, 4... c5 5. e3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Nf3

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 c5

			variation => N__('Nimzo-Indian: Three Knights, 4... c5'),
			moves => {
				'd4d5' => 'rnbqk2r/pp1p1ppp/4pn2/2pP4/1bP5/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'E38b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5 5. Nf3

			variation => N__('Nimzo-Indian: Classical, 4... c5 5. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3

			variation => N__('Nimzo-Indian: Romanishin, 4... c5 5. Nf3'),
			moves => {
				'b8c6' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 2 6',
				'c5d4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPp4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/pp1p1ppp/4pn2/2p5/1bPP4/2N2NP1/PP2PP1P/R1BQKB1R w KQ - 2 6',
				'f6e4' => 'rnbqk2r/pp1p1ppp/4p3/2p5/1bPPn3/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2P2/PP2P1PP/R1BQKBNR w KQkq c6 0 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. f3 c5

			variation => N__('Nimzo-Indian: 4. f3 c5'),
			moves => {
				'd4d5' => 'rnbqk2r/pp1p1ppp/4pn2/2pP4/1bP5/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq c6 0 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5

			variation => N__('Nimzo-Indian: Romanishin, 4... c5'),
			moves => {
				'd4d5' => 'rnbqk2r/pp1p1ppp/4pn2/2pP4/1bP5/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 5',
				'g1f3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq c6 0 5' => {
			eco => 'E38a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 c5

			variation => N__('Nimzo-Indian: Classical, 4... c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp1p1ppp/4pn2/2P5/1bP5/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 0 5',
				'e2e3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 0 5',
				'g1f3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2N2/PPQ1PPPP/R1B1KB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'E41',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Bd3

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Bd3'),
			moves => {
				'b8c6' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/1bPP4/2NBP3/PP3PPP/R1BQK1NR w KQkq - 2 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/5N2/PP1BPPPP/RN1QKB1R w KQkq c6 0 5' => {
			eco => 'E11j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 c5

			variation => N__('Bogo-Indian: Vitolins Variation'),
			moves => {
				'd2b4' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1BPP4/5N2/PP2PPPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E26',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. e3

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. e3'),
			moves => {
				'b7b6' => 'rnbqk2r/p2p1ppp/1p2pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 0 7',
				'b8c6' => 'r1bqk2r/pp1p1ppp/2n1pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 1 7',
				'd7d6' => 'rnbqk2r/pp3ppp/3ppn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 0 7',
				'd8a5' => 'rnb1k2r/pp1p1ppp/4pn2/q1p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3'),
			moves => {
				'd7d5' => 'rnbqk2r/pp3ppp/4pn2/2pp4/2PP4/P1P2P2/4P1PP/R1BQKBNR w KQkq d6 0 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2p5/2PP4/P1P5/4PPPP/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5

			variation => N__('Nimzo-Indian: Sämisch, 5... c5'),
			moves => {
				'e2e3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR b KQkq - 0 6',
				'f2f3' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2pP2B1/1bP5/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5 5. d5

			variation => N__('Nimzo-Indian: Leningrad, 4... c5 5. d5'),
			moves => {
				'b4c3' => 'rnbqk2r/pp1p1ppp/4pn2/2pP2B1/2P5/2b5/PP2PPPP/R2QKBNR w KQkq - 0 6',
				'd7d6' => 'rnbqk2r/pp3ppp/3ppn2/2pP2B1/1bP5/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
				'e6d5' => 'rnbqk2r/pp1p1ppp/5n2/2pp2B1/1bP5/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
				'h7h6' => 'rnbqk2r/pp1p1pp1/4pn1p/2pP2B1/1bP5/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2pP2B1/2P5/2b5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5 5. d5 Bxc3+

			variation => N__('Nimzo-Indian: Leningrad, 4... c5 5. d5 Bxc3+'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2pP4/1bP5/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'E21',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Three Knights, 4... c5'),
			moves => {
				'f6e4' => 'rnbqk2r/pp1p1ppp/4p3/2pP4/1bP1n3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2pP4/1bP5/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. f3 c5 5. d5

			variation => N__('Nimzo-Indian: 4. f3 c5 5. d5'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/2pP4/1bP5/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. d5

			variation => N__('Nimzo-Indian: Romanishin, 4... c5 5. d5'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/4P3/1b1N4/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B40u',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Bb4 6. e5

			variation => N__('Sicilian: Pin, Koch Variation'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1b1NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'B40s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Bb4

			variation => N__('Sicilian: Pin Variation'),
			moves => {
				'e4e5' => 'rnbqk2r/pp1p1ppp/4pn2/4P3/1b1N4/2N5/PPP2PPP/R1BQKB1R b KQkq - 0 6',
				'f1d3' => 'rnbqk2r/pp1p1ppp/4pn2/8/1b1NP3/2NB4/PPP2PPP/R1BQK2R b KQkq - 4 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1b1NP3/2NB4/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'B40s',
                        # TRANSLATORS:

			variation => N__('Sicilian: Pin Variation'),
			moves => {
				'e6e5' => 'rnbqk2r/pp1p1ppp/5n2/4p3/1b1NP3/2NB4/PPP2PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/1QN5/PP2PPPP/R1B1KB1R b KQkq - 3 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Bb4 6. Qb3

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Bb4 6. Qb3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/2N3P1/PP2PP1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 cxd4 6. Nxd4

			variation => N__('Nimzo-Indian: Romanishin, 6.Nxd4'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp1p1ppp/4pn2/8/1bPN4/2N3P1/PP2PP1P/R1BQKB1R w KQ - 1 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/2N5/PP1BPPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Bb4 6. Bd2

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Bb4 6. Bd2'),
			moves => {
				'b8c6' => 'r1bqk2r/pp1p1ppp/2n1pn2/8/1bPN4/2N5/PP1BPPPP/R2QKB1R w KQkq - 4 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 Bb4

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 Bb4'),
			moves => {
				'c1d2' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/2N5/PP1BPPPP/R2QKB1R b KQkq - 3 6',
				'd1b3' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/1QN5/PP2PPPP/R1B1KB1R b KQkq - 3 6',
				'd4b5' => 'rnbqk2r/pp1p1ppp/4pn2/1N6/1bP5/2N5/PP2PPPP/R1BQKB1R b KQkq - 3 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 Bb4+

			variation => N__('English: Symmetrical, Two Knights, 5. g3 Bb4+'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPP4/2N5/PP2NPPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'E42',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Ne2 cxd4 6. exd4

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2 cxd4 6.exd4'),
			moves => {
				'd7d5' => 'rnbqk2r/pp3ppp/4pn2/3p4/1bPP4/2N5/PP2NPPP/R1BQKB1R w KQkq d6 0 7',
				'e8g8' => 'rnbq1rk1/pp1p1ppp/4pn2/8/1bPP4/2N5/PP2NPPP/R1BQKB1R w KQ - 1 7',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPp4/2N1P3/PP2NPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'E42',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2'),
			moves => {
				'e3d4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPP4/2N5/PP2NPPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1bPp4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 0 6' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3 c5 5. Nf3 cxd4

			variation => N__('Nimzo-Indian: Romanishin, 4... c5 5. Nf3 cxd4'),
			moves => {
				'f3d4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/2N3P1/PP2PP1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1pPP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 6' => {
			eco => 'E11j',
                        # TRANSLATORS:

			variation => N__('Bogo-Indian: Vitolins Variation'),
			moves => {
				'g2g3' => 'rnbqk2r/pp1p1ppp/4pn2/8/1pPP4/5NP1/PP2PP1P/RN1QKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp1p1ppp/4pn2/8/1pPP4/5NP1/PP2PP1P/RN1QKB1R b KQkq - 0 6' => {
			eco => 'E11k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 c5 5. Bxb4 cxb4 6. g3

			variation => N__('Bogo-Indian: Vitolins, 6. g3'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/5n2/2pp2B1/1bP5/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5 5. d5 exd5

			variation => N__('Nimzo-Indian: Leningrad, 4... c5 5. d5 exd5'),
			moves => {
			},
		},
		'rnbqk2r/pp1p1ppp/5n2/4p3/1b1NP3/2NB4/PPP2PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B40t',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 Bb4 6. Bd3 e5

			variation => N__('Sicilian: Pin, Jaffe Variation'),
			moves => {
			},
		},
		'rnbqk2r/pp1pbppp/2p2n2/6B1/2BQ4/2N2N2/PPP2PPP/2KR3R b kq - 1 8' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'd7d5' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KR3R w kq d6 0 9',
			},
		},
		'rnbqk2r/pp1pbppp/2p2n2/6B1/2BQ4/2N2N2/PPP2PPP/R3K2R w KQkq - 0 8' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3 Nxe4 5. Qxd4 Nf6 6. Bg5 Be7 7. Nc3 c6

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'e1c1' => 'rnbqk2r/pp1pbppp/2p2n2/6B1/2BQ4/2N2N2/PPP2PPP/2KR3R b kq - 1 8',
			},
		},
		'rnbqk2r/pp1pbppp/4pn2/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'A46v',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e3 Be7

			variation => N__('Torre Attack: 3... c5 4. e3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/pp1pp1bp/2p3pn/5p2/3P4/5NP1/PPP1PPBP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 g6 3. Bg2 Bg7 4. Nf3 c6 5. O-O Nh6

			variation => N__('Dutch: Leningrad, Basman Variation'),
			moves => {
			},
		},
		'rnbqk2r/pp1pppbp/5np1/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7

			variation => N__('English: Symmetrical, Two Knights, 4... g6'),
			moves => {
				'e2e4' => 'rnbqk2r/pp1pppbp/5np1/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq e3 0 6',
			},
		},
		'rnbqk2r/pp1pppbp/5np1/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq e3 0 6' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 g6 5. Nc3 Bg7 6. e4

			variation => N__('English: Symmetrical, Two Knights, 4... g6'),
			moves => {
			},
		},
		'rnbqk2r/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d5 6. O-O c6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d5 6.O-O c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2bp1p/2p2np1/3p2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQkq - 0 8' => {
			eco => 'D36e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. Nf3 g6 8. e3

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. Nf3 g6 8. e3'),
			moves => {
			},
		},
		'rnbqk2r/pp2bp1p/2p2np1/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R w KQkq - 0 8' => {
			eco => 'D36d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. Nf3 g6

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. Nf3 g6'),
			moves => {
				'e2e3' => 'rnbqk2r/pp2bp1p/2p2np1/3p2B1/3P4/2N1PN2/PPQ2PPP/R3KB1R b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp2bpp1/2p1pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D43k',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Be7'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2bpp1/2p1pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 8',
			},
		},
		'rnbqk2r/pp2bpp1/2p1pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'D43k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4 Be7

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Be7'),
			moves => {
				'e2e3' => 'rnbqk2r/pp2bpp1/2p1pn1p/3p4/2PP3B/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2bpp1/2p2n1p/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR w KQkq - 0 8' => {
			eco => 'D36g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 h6

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 h6'),
			moves => {
				'g5h4' => 'rnbqk2r/pp2bpp1/2p2n1p/3p4/3P3B/2N1P3/PPQ2PPP/R3KBNR b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2bpp1/2p2n1p/3p4/3P3B/2N1P3/PPQ2PPP/R3KBNR b KQkq - 1 8' => {
			eco => 'D36g',
                        # TRANSLATORS:

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 h6'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2bpp1/2p2n1p/3p4/3P3B/2N1P3/PPQ2PPP/R3KBNR w KQ - 2 9',
			},
		},
		'rnbqk2r/pp2bpp1/3ppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKB1R w KQkq - 1 8' => {
			eco => 'B81k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h4 Be7

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h4 Be7'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D43g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 Be7 6. e3

			variation => N__('Semi-Slav: Botvinnik, 5... Be7 6. e3'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1nbppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 1 7',
				'e8g8' => 'rnbq1rk1/pp2bppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 7',
			},
		},
		'rnbqk2r/pp2bppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 6' => {
			eco => 'D43f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 Be7

			variation => N__('Semi-Slav: Botvinnik, 5... Be7'),
			moves => {
				'e2e3' => 'rnbqk2r/pp2bppp/2p1pn2/3p2B1/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2bppp/2p1pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O c6

			variation => N__('English: Neo-Catalan Declined, 5... c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/2p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D45c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Be7

			variation => N__('Semi-Slav: 5. e3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KR3R w kq d6 0 9' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'h1e1' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KRR3 b kq - 1 9',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KRR3 b kq - 1 9' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit, 7... c6 Defence'),
			moves => {
				'c8e6' => 'rn1qk2r/pp2bppp/2p1bn2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KRR3 w kq - 2 10',
				'e8g8' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/2BQ4/2N2N2/PPP2PPP/2KRR3 w - - 2 10',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D35o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7'),
			moves => {
				'f1d3' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PP3PPP/R2QK1NR b KQkq - 2 7',
				'g1f3' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 2 7',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR b KQkq - 0 7' => {
			eco => 'D36f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR w KQkq - 1 8',
				'e8g8' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR w KQ - 1 8',
				'h7h6' => 'rnbqk2r/pp2bpp1/2p2n1p/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 2 7' => {
			eco => 'D35o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Nf3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Nf3'),
			moves => {
				'c8f5' => 'rn1qk2r/pp2bppp/2p2n2/3p1bB1/3P4/2N1PN2/PP3PPP/R2QKB1R w KQkq - 3 8',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 3 7' => {
			eco => 'D36c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. Nf3

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. Nf3'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R w KQkq - 4 8',
				'e8g8' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R w KQ - 4 8',
				'g7g6' => 'rnbqk2r/pp2bp1p/2p2np1/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 2 7' => {
			eco => 'D36c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7

			variation => N__('QGD: Exchange, 6. Qc2 Be7'),
			moves => {
				'e2e3' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N1P3/PPQ2PPP/R3KBNR b KQkq - 0 7',
				'g1f3' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 3 7',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PP3PPP/R2QK1NR b KQkq - 2 7' => {
			eco => 'D35r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1nbppp/2p2n2/3p2B1/3P4/2NBP3/PP3PPP/R2QK1NR w KQkq - 3 8',
				'e8g8' => 'rnbq1rk1/pp2bppp/2p2n2/3p2B1/3P4/2NBP3/PP3PPP/R2QK1NR w KQ - 3 8',
				'f6e4' => 'rnbqk2r/pp2bppp/2p5/3p2B1/3Pn3/2NBP3/PP3PPP/R2QK1NR w KQkq - 3 8',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D35h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Be7 6. Bf4 c6

			variation => N__('QGD: Exchange, 5. Nf3 Be7 6. Bf4 c6'),
			moves => {
				'd1c2' => 'rnbqk2r/pp2bppp/2p2n2/3p4/3P1B2/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 1 7',
			},
		},
		'rnbqk2r/pp2bppp/2p2n2/3p4/3P1B2/2N2N2/PPQ1PPPP/R3KB1R b KQkq - 1 7' => {
			eco => 'D35h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Be7 6. Bf4 c6 7. Qc2

			variation => N__('QGD: Exchange, 5. Nf3 Be7 6. Bf4 c6 7. Qc2'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1nbppp/2p2n2/3p4/3P1B2/2N2N2/PPQ1PPPP/R3KB1R w KQkq - 2 8',
			},
		},
		'rnbqk2r/pp2bppp/2p5/3p2B1/3Pn3/2NBP3/PP3PPP/R2QK1NR w KQkq - 3 8' => {
			eco => 'D35s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3 Be7 7. Bd3 Ne4

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3 Be7 7. Bd3 Ne4'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'A65a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 Be7

			variation => N__('Benoni: 6. e4 Be7'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'A56v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. Nf3

			variation => N__('Benoni: Czech, 5. e4 Be7 6. Nf3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N2/PP3PPP/R1BQKB1R w KQ - 3 7',
			},
		},
		'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2N3P1/PP3P1P/R1BQKBNR b KQkq - 0 6' => {
			eco => 'A56s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. g3

			variation => N__('Benoni: Czech, 5. e4 Be7 6. g3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2bppp/3p1n2/2pPp3/2P1P3/2N3P1/PP3P1P/R1BQKBNR w KQ - 1 7',
			},
		},
		'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'A56r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7

			variation => N__('Benoni: Czech, 5. e4 Be7'),
			moves => {
				'f1d3' => 'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 2 6',
				'g1f3' => 'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 2 6',
				'g2g3' => 'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2N3P1/PP3P1P/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 2 6' => {
			eco => 'A56u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. Bd3

			variation => N__('Benoni: Czech, 5. e4 Be7 6. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/3ppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 Be7

			variation => N__('Sicilian: Scheveningen, 6. Be2'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/3ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 Be7

			variation => N__('Sicilian: Scheveningen, 6. f4 Be7'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/4p3/3n4/3P4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B14n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Be7 7. cxd5 Nxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Be7 7.cxd5 Nxd5'),
			moves => {
				'f1d3' => 'rnbqk2r/pp2bppp/4p3/3n4/3P4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2bppp/4p3/3n4/3P4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'B14o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Be7 7. cxd5 Nxd5 8. Bd3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Be7 7.cxd5 Nxd5 8. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/4pn2/2Pp4/4P3/3B4/PPPN1PPP/R1BQK1NR w KQkq - 1 6' => {
			eco => 'C03i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7 4. Bd3 c5 5. dxc5 Nf6

			variation => N__('French: Tarrasch, 3... Be7 4. Bd3 c5 5.dxc5 Nf6'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/4pn2/2pp2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D03i',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... e6 4. e3 c5'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1nbppp/4pn2/2pp2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R w KQkq - 1 7',
			},
		},
		'rnbqk2r/pp2bppp/4pn2/2pp2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQkq - 2 6' => {
			eco => 'D03i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3 c5 5. Nbd2 Be7

			variation => N__('Torre Attack: 3... e6 4. e3 c5'),
			moves => {
				'c2c3' => 'rnbqk2r/pp2bppp/4pn2/2pp2B1/3P4/2P1PN2/PP1N1PPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2bppp/4pn2/2pp4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq c6 0 6' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O c5

			variation => N__('English: Neo-Catalan Declined, 5... c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/4pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D40b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 Be7

			variation => N__('QGD: Semi-Tarrasch, 5. e3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/pp2bppp/4pn2/3P2B1/3Q4/2N5/PP2PPPP/R3KBNR w KQkq - 1 7' => {
			eco => 'D50',
                        # TRANSLATORS:

			variation => N__('QGD: Dutch-Peruvian, Prins Variation'),
			moves => {
				'e2e4' => 'rnbqk2r/pp2bppp/4pn2/3P2B1/3QP3/2N5/PP3PPP/R3KBNR b KQkq e3 0 7',
			},
		},
		'rnbqk2r/pp2bppp/4pn2/3P2B1/3QP3/2N5/PP3PPP/R3KBNR b KQkq e3 0 7' => {
			eco => 'D50',
                        # TRANSLATORS:

			variation => N__('QGD: Dutch-Peruvian, Prins Variation'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2bppp/2n1pn2/3P2B1/3QP3/2N5/PP3PPP/R3KBNR w KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2bppp/4pn2/3P4/3P4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B14m',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Be7 7. cxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Be7 7.cxd5'),
			moves => {
				'f6d5' => 'rnbqk2r/pp2bppp/4p3/3n4/3P4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp2bppp/4pn2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B14k',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Be7

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Be7'),
			moves => {
				'c4d5' => 'rnbqk2r/pp2bppp/4pn2/3P4/3P4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7',
				'f1d3' => 'rnbqk2r/pp2bppp/4pn2/3p4/2PP4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 3 7',
			},
		},
		'rnbqk2r/pp2bppp/4pn2/3p4/2PP4/2NB1N2/PP3PPP/R1BQK2R b KQkq - 3 7' => {
			eco => 'B14l',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Be7 7. Bd3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Be7 7. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/1NppP3/1b1P4/8/PPPB1PPP/R2QKBNR b KQkq - 3 6' => {
			eco => 'C17j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Bd2 Ne7 6. Nb5

			variation => N__('French: Winawer, Bogoljubow, 5... Ne7 6. Nb5'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/2PpP3/1b4Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'C17d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Qg4 Ne7 6. dxc5

			variation => N__('French: Winawer, Russian, 6.dxc5'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/2PpP3/1b6/2N5/PPPB1PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C17i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Bd2 Ne7 6. dxc5

			variation => N__('French: Winawer, Bogoljubow, 5... Ne7 6.dxc5'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/1b1P2Q1/2N5/PPP2PPP/R1B1KBNR w KQkq - 2 6' => {
			eco => 'C17c',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Russian Variation'),
			moves => {
				'd4c5' => 'rnbqk2r/pp2nppp/4p3/2PpP3/1b4Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/1b1P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 2 6' => {
			eco => 'C17g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Bd2 Ne7

			variation => N__('French: Winawer, Bogoljubow, 5... Ne7'),
			moves => {
				'a2a3' => 'rnbqk2r/pp2nppp/4p3/2ppP3/1b1P4/P1N5/1PPB1PPP/R2QKBNR b KQkq - 0 6',
				'c3b5' => 'rnbqk2r/pp2nppp/4p3/1NppP3/1b1P4/8/PPPB1PPP/R2QKBNR b KQkq - 3 6',
				'd4c5' => 'rnbqk2r/pp2nppp/4p3/2PpP3/1b6/2N5/PPPB1PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/1b1P4/P1N5/1PPB1PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C17h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. Bd2 Ne7 6. a3

			variation => N__('French: Winawer, Bogoljubow, 5... Ne7 6. a3'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 2 7' => {
			eco => 'C18j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4

			variation => N__('French: Winawer, 6... Ne7 7. Qg4'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 3 8',
				'c5d4' => 'rnbqk2r/pp2nppp/4p3/3pP3/3p2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 0 8',
				'd8c7' => 'rnb1k2r/ppq1nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 3 8',
				'e8f8' => 'rnbq1k1r/pp2nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQ - 3 8',
				'e8g8' => 'rnbq1rk1/pp2nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR w KQ - 3 8',
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/3P3P/P1P5/2P2PP1/R1BQKBNR b KQkq h3 0 7' => {
			eco => 'C18g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. h4

			variation => N__('French: Winawer, 6... Ne7 7. h4'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P3P/P1P5/2P2PP1/R1BQKBNR w KQkq - 1 8',
				'd8c7' => 'rnb1k2r/ppq1nppp/4p3/2ppP3/3P3P/P1P5/2P2PP1/R1BQKBNR w KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'C19c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Nf3

			variation => N__('French: Winawer, 6... Ne7 7. Nf3'),
			moves => {
				'b7b6' => 'rnbqk2r/p3nppp/1p2p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 0 8',
				'b8c6' => 'r1bqk2r/pp2nppp/2n1p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8',
				'c8d7' => 'rn1qk2r/pp1bnppp/4p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8',
				'd8a5' => 'rnb1k2r/pp2nppp/4p3/q1ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8',
				'd8c7' => 'rnb1k2r/ppq1nppp/4p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R w KQkq - 3 8',
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'C18f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7

			variation => N__('French: Winawer, 6... Ne7'),
			moves => {
				'a3a4' => 'rnbqk2r/pp2nppp/4p3/2ppP3/P2P4/2P5/2P2PPP/R1BQKBNR b KQkq - 0 7',
				'd1g4' => 'rnbqk2r/pp2nppp/4p3/2ppP3/3P2Q1/P1P5/2P2PPP/R1B1KBNR b KQkq - 2 7',
				'g1f3' => 'rnbqk2r/pp2nppp/4p3/2ppP3/3P4/P1P2N2/2P2PPP/R1BQKB1R b KQkq - 2 7',
				'h2h4' => 'rnbqk2r/pp2nppp/4p3/2ppP3/3P3P/P1P5/2P2PP1/R1BQKBNR b KQkq h3 0 7',
			},
		},
		'rnbqk2r/pp2nppp/4p3/2ppP3/P2P4/2P5/2P2PPP/R1BQKBNR b KQkq - 0 7' => {
			eco => 'C19a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. a4

			variation => N__('French: Winawer, Smyslov Variation'),
			moves => {
				'd8a5' => 'rnb1k2r/pp2nppp/4p3/q1ppP3/P2P4/2P5/2P2PPP/R1BQKBNR w KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2nppp/4p3/3pP3/3p2Q1/P1P5/2P2PPP/R1B1KBNR w KQkq - 0 8' => {
			eco => 'C18m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Bxc3+ 6. bxc3 Ne7 7. Qg4 cxd4

			variation => N__('French: Winawer, 6... Ne7 7. Qg4 cxd4'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/P2pP3/3p2Q1/P1N5/2P2PPP/R1B1KBNR b KQkq - 0 8' => {
			eco => 'C17p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 c5 5. a3 Ba5 6. b4 cxd4 7. Qg4 Ne7 8. bxa5

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Qg4 Ne7 8.bxa5'),
			moves => {
			},
		},
		'rnbqk2r/pp2nppp/4p3/b2pP3/1P1p2Q1/P1N5/2P2PPP/R1B1KBNR w KQkq - 2 8' => {
			eco => 'C17o',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Swiss, 6. b4 cxd4 7. Qg4'),
			moves => {
				'b4a5' => 'rnbqk2r/pp2nppp/4p3/P2pP3/3p2Q1/P1N5/2P2PPP/R1B1KBNR b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp2ppbp/1n4p1/2p5/3PP3/6P1/PP2NPBP/RNBQK2R w KQkq c6 0 8' => {
			eco => 'D72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6 7. Ne2 c5

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6 7. Ne2 c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3P4/3P4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6' => {
			eco => 'D96',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Russian, 5... c6'),
			moves => {
				'c6d5' => 'rnbqk2r/pp2ppbp/5np1/3p4/3P4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7',
				'f6d5' => 'rnbqk2r/pp2ppbp/2p3p1/3n4/3P4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3P4/3P4/5NP1/PP2PPBP/RNBQK2R b KQkq - 0 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 c6 6. cxd5

			variation => N__('Neo-Grünfeld, 5. Nf3 c6 6.cxd5'),
			moves => {
				'c6d5' => 'rnbqk2r/pp2ppbp/5np1/3p4/3P4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D91a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 c6

			variation => N__('Grünfeld: 5. Bg5 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R w KQkq - 1 6' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. Bb2 g6 5. e3 Bg7

			variation => N__('English: Bled Variation'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP1B2/1QN2N2/PP2PPPP/R3KB1R b KQkq - 1 6' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 c6 6. Bf4

			variation => N__('Grünfeld: Russian, 5... c6 6. Bf4'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D92',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4 c6

			variation => N__('Grünfeld: 5. Bf4 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 c6

			variation => N__('Grünfeld: Russian, 5... c6'),
			moves => {
				'c1f4' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP1B2/1QN2N2/PP2PPPP/R3KB1R b KQkq - 1 6',
				'c4d5' => 'rnbqk2r/pp2ppbp/2p2np1/3P4/3P4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 c6 6. Be2

			variation => N__('Grünfeld: Slav/Schlecter, 6. Be2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQK2R w KQ - 2 7',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 c6

			variation => N__('Grünfeld: Slav/Schlecter'),
			moves => {
				'f1d3' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 6',
				'f1e2' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP2BPPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/2NBPN2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3 c6 6. Bd3

			variation => N__('Grünfeld: Slav/Schlecter, 6. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 c6

			variation => N__('Neo-Grünfeld, 5. Nf3 c6'),
			moves => {
				'c4d5' => 'rnbqk2r/pp2ppbp/2p2np1/3P4/3P4/5NP1/PP2PPBP/RNBQK2R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3p4/3PP3/2N2N1P/PPP2PP1/R1BQKB1R w KQkq - 1 6' => {
			eco => 'B15n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. h3 Nf6

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5. h3 Nf6'),
			moves => {
				'e4e5' => 'rnbqk2r/pp2ppbp/2p2np1/3pP3/3P4/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/2p2np1/3pP3/3P4/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B15o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3 Bg7 5. h3 Nf6 6. e5

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3 Bg7 5. h3 Nf6 6. e5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p3p1/1B6/3PP3/2P5/P4PPP/R1BQK1NR w KQkq - 0 8' => {
			eco => 'D85e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bb5+ c6

			variation => N__('Grünfeld: Exchange, 7. Bb5+ c6'),
			moves => {
				'b5a4' => 'rnbqk2r/pp2ppbp/2p3p1/8/B2PP3/2P5/P4PPP/R1BQK1NR b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2ppbp/2p3p1/3P2N1/3P4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D91j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 c6

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p3p1/3n4/3P4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 c6 6. cxd5 Nxd5

			variation => N__('Grünfeld: Russian, 5... c6 6.cxd5 Nxd5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2p3p1/8/B2PP3/2P5/P4PPP/R1BQK1NR b KQkq - 1 8' => {
			eco => 'D85e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bb5+ c6 8. Ba4

			variation => N__('Grünfeld: Exchange, 7. Bb5+ c6 8. Ba4'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/2p3p1/8/B2PP3/2P5/P4PPP/R1BQK1NR w KQ - 2 9',
			},
		},
		'rnbqk2r/pp2ppbp/2p3pn/3pP3/3P1P2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'B15g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. e5 Bg7 5. f4 Nh6

			variation => N__('Caro-Kann: Gurgenidze: 4. e5 Bg7 5. f4 Nh6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/2BPP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'B08g',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Bc4 c6

			variation => N__('Pirc: Classical, 5. Bc4 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/2BPP3/2N2Q2/PPP2PPP/R1B1K1NR w KQkq - 2 6' => {
			eco => 'B06n',
                        # TRANSLATORS: 1. e4 g6 2. d4 Bg7 3. Nc3 d6 4. Bc4 c6 5. Qf3 Nf6

			variation => N__('Modern: 3. Nc3 d6 4. Bc4'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR b KQkq - 1 6' => {
			eco => 'E80m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c6 6. Be3

			variation => N__('King\'s Indian: Sämisch, 5... c6 6. Be3'),
			moves => {
				'a7a6' => 'rnbqk2r/1p2ppbp/p1pp1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'E80l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c6

			variation => N__('King\'s Indian: Sämisch, 5... c6'),
			moves => {
				'c1e3' => 'rnbqk2r/pp2ppbp/2pp1np1/8/2PPP3/2N1BP2/PP4PP/R2QKBNR b KQkq - 1 6',
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N1B3/PPPQ1PPP/R3KBNR w KQkq - 0 6' => {
			eco => 'B07v',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be3 Bg7 5. Qd2 c6

			variation => N__('Pirc: 4. Be3, 150 Attack'),
			moves => {
				'f2f3' => 'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N1BP2/PPPQ2PP/R3KBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N1BN2/PPP2PPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'B08i',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be3 c6

			variation => N__('Pirc: Classical, 5. Be3 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N1BP2/PPPQ2PP/R3KBNR b KQkq - 0 6' => {
			eco => 'B07v',
                        # TRANSLATORS:

			variation => N__('Pirc: 4. Be3, 150 Attack'),
			moves => {
				'b7b5' => 'rnbqk2r/p3ppbp/2pp1np1/1p6/3PP3/2N1BP2/PPPQ2PP/R3KBNR w KQkq b6 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N2N1P/PPP2PP1/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B08b',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3 c6

			variation => N__('Pirc: Classical, 5. h3'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R w KQkq - 0 6' => {
			eco => 'B08k',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2 c6

			variation => N__('Pirc: Classical, 5. Be2 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N3P1/PPP2PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'B07r',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. g3 Bg7 5. Bg2 c6

			variation => N__('Pirc: Sveshnikov (4. g3)'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/2pp1np1/8/P2PP3/2N2N2/1PP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B08f',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. a4 c6

			variation => N__('Pirc: Classical, 5. a4 c6'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/1Bp5/3PPP2/2N2N2/PPP3PP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B09r',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. Bb5+

			variation => N__('Pirc: Austrian, 5... c5 6. Bb5+'),
			moves => {
				'c8d7' => 'rn1qk2r/pp1bppbp/3p1np1/1Bp5/3PPP2/2N2N2/PPP3PP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 3 6' => {
			eco => 'B50w',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Be2 g6 5. O-O Bg7 6. Bb5+

			variation => N__('Sicilian: 2. Nf3 d6 3. c3, Torre Variation'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/1BpP4/4P3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'A43w',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Bb5+

			variation => N__('Old Benoni: Schmidt, 6. Bb5+'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2P5/2P1P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E80h',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, 5... c5'),
			moves => {
				'd6c5' => 'rnbqk2r/pp2ppbp/5np1/2p5/2P1P3/2N2P2/PP4PP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2P5/2P1PP2/2N5/PP4PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E76b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 c5 6. dxc5

			variation => N__('King\'s Indian: Four Pawns Attack, Exchange'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2P5/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B09p',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5 6. dxc5

			variation => N__('Pirc: Austrian, 5... c5 6.dxc5'),
			moves => {
				'd8a5' => 'rnb1k2r/pp2ppbp/3p1np1/q1P5/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq c6 0 6' => {
			eco => 'E90c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 c5

			variation => N__('King\'s Indian: 5. Nf3 c5'),
			moves => {
				'd4d5' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'E80h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 c5

			variation => N__('King\'s Indian: Sämisch, 5... c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp2ppbp/3p1np1/2P5/2P1P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N4P/PP3PP1/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'E71b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3 c5

			variation => N__('King\'s Indian: Makagonov, 5... c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQkq c6 0 6' => {
			eco => 'E73b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 c5

			variation => N__('King\'s Indian: 5. Be2 c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPPP2/2N5/PP4PP/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'E76b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 c5

			variation => N__('King\'s Indian: Four Pawns Attack, 5... c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp2ppbp/3p1np1/2P5/2P1PP2/2N5/PP4PP/R1BQKBNR b KQkq - 0 6',
				'd4d5' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1PP2/2N5/PP4PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/3PPP2/2N2N2/PPP3PP/R1BQKB1R w KQkq c6 0 6' => {
			eco => 'B09o',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3 c5

			variation => N__('Pirc: Austrian, 5... c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp2ppbp/3p1np1/2P5/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 0 6',
				'f1b5' => 'rnbqk2r/pp2ppbp/3p1np1/1Bp5/3PPP2/2N2N2/PPP3PP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2p5/4P3/2P2N2/PP1PBPPP/RNBQ1RK1 w kq - 2 6' => {
			eco => 'B50v',
                        # TRANSLATORS:

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Be2 g6'),
			moves => {
				'e2b5' => 'rnbqk2r/pp2ppbp/3p1np1/1Bp5/4P3/2P2N2/PP1P1PPP/RNBQ1RK1 b kq - 3 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'E90c',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 5. Nf3 c5'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/2P1P3/2N2N2/PP3PPP/R1BQKB1R w KQ - 1 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'A56l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 d6 4. Nc3 g6 5. e4 Bg7 6. f3

			variation => N__('Benoni: 3. d5 d6 4. Nc3 g6 5. e4 Bg7 6. f3'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'A56k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 d6 4. Nc3 g6 5. e4 Bg7

			variation => N__('Benoni: 3. d5 d6 4. Nc3 g6 5. e4 Bg7'),
			moves => {
				'f1d3' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 2 6',
				'f2f3' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 2 6' => {
			eco => 'A56m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 d6 4. Nc3 g6 5. e4 Bg7 6. Bd3

			variation => N__('Benoni: 3. d5 d6 4. Nc3 g6 5. e4 Bg7 6. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1PP2/2N5/PP4PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E76c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4 c5 6. d5

			variation => N__('King\'s Indian: Four Pawns Attack, 5... c5 6. d5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/4P3/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A43v',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. h3

			variation => N__('Old Benoni: Schmidt, 6. h3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/4P3/2N2N1P/PPP2PP1/R1BQKB1R w KQ - 1 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'A43x',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7 6. Be2

			variation => N__('Old Benoni: Schmidt, 6. Be2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQK2R w KQ - 4 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A43u',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3 Bg7

			variation => N__('Old Benoni: Schmidt, 3. e4 Nf6'),
			moves => {
				'f1b5' => 'rnbqk2r/pp2ppbp/3p1np1/1BpP4/4P3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 3 6',
				'f1e2' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 3 6',
				'h2h3' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/4P3/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/4P3/3N1P2/2N5/PPP3PP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Bg7 7. e5

			variation => N__('Sicilian: Dragon, Levenfish, 6... Bg7 7. e5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B70e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Bg5 Bg7

			variation => N__('Sicilian: Dragon, 6. Bg5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R b KQkq - 3 7' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Bc4

			variation => N__('Sicilian: Dragon, 6. Be3 Bg7 7. Bc4'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2ppbp/2np1np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQkq - 4 8',
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R w KQ - 4 8',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N4P/PPP2PP1/R1BQK2R b KQkq - 0 7' => {
			eco => 'B70g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Bc4 Bg7 7. h3

			variation => N__('Sicilian: Dragon, 6. Bc4 Bg7 7. h3'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N5/PPP2PPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'B70h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Bc4 Bg7 7. O-O

			variation => N__('Sicilian: Dragon, 6. Bc4 Bg7 7.O-O'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B70f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Bc4 Bg7

			variation => N__('Sicilian: Dragon, 6. Bc4 Bg7'),
			moves => {
				'e1g1' => 'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N5/PPP2PPP/R1BQ1RK1 b kq - 3 7',
				'h2h3' => 'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N4P/PPP2PP1/R1BQK2R b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 3 7' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Be2

			variation => N__('Sicilian: Dragon, Classical Attack'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQkq - 4 8',
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R w KQ - 4 8',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7

			variation => N__('Sicilian: Dragon, 6. Be3 Bg7'),
			moves => {
				'd1d2' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 3 7',
				'f1c4' => 'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N1B3/PPP2PPP/R2QK2R b KQkq - 3 7',
				'f1e2' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 3 7',
				'f2f3' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 3 7' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. Qd2

			variation => N__('Sicilian: Dragon, 6. Be3 Bg7 7. Qd2'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B75',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3 Bg7 7. f3

			variation => N__('Sicilian: Dragon, Yugoslav Attack'),
			moves => {
				'a7a6' => 'rnbqk2r/1p2ppbp/p2p1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq - 0 8',
				'b8c6' => 'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq - 1 8',
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQ - 1 8',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 3 7' => {
			eco => 'B70l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7 7. O-O

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7 7.O-O'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2ppbp/2np1np1/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 4 8',
				'e8g8' => 'rnbq1rk1/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w - - 4 8',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'B70k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 Bg7

			variation => N__('Sicilian: Dragon, 6. Be2 Bg7'),
			moves => {
				'e1g1' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 3 7',
			},
		},
		'rnbqk2r/pp2ppbp/3p1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4 Bg7

			variation => N__('Sicilian: Dragon, Levenfish, 6... Bg7'),
			moves => {
				'e4e5' => 'rnbqk2r/pp2ppbp/3p1np1/4P3/3N1P2/2N5/PPP3PP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/2Pp4/2P2B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D82',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 4. Bf4 Bg7 5. e3 c5'),
			moves => {
				'd8a5' => 'rnb1k2r/pp2ppbp/5np1/q1Pp4/2P2B2/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/2p5/2P1P3/2N2P2/PP4PP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'E80h',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: Sämisch, 5... c5'),
			moves => {
				'd1d8' => 'rnbQk2r/pp2ppbp/5np1/2p5/2P1P3/2N2P2/PP4PP/R1B1KBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/2pp4/2PP1B2/2N1P3/PP3PPP/R2QKBNR w KQkq c6 0 6' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3 c5

			variation => N__('Grünfeld: 4. Bf4 Bg7 5. e3 c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp2ppbp/5np1/2Pp4/2P2B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/2pp4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq c6 0 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 c5

			variation => N__('Neo-Grünfeld, 5. Nf3 c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/5np1/2pp4/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 g6 4. O-O Bg7 5. d3 Nf6

			variation => N__('Reti: KIA, 2... c5 + 3... g6'),
			moves => {
				'b1d2' => 'rnbqk2r/pp2ppbp/5np1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 b kq - 2 6' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5 + 3... g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/5np1/2pp4/8/3P1NP1/PPPNPPBP/R1BQ1RK1 w - - 3 7',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/3P4/3P4/1QN5/PP3PPP/R1B1KBNR b KQkq - 0 7' => {
			eco => 'B14f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. Qb3 Bg7 7. cxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6. Qb3 Bg7 7.cxd5'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/5np1/3P4/3P4/1QN5/PP3PPP/R1B1KBNR w KQ - 1 8',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/3P4/3P4/2N5/PP3PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'B14e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. cxd5 Bg7

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6.cxd5 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/5np1/3p4/2PP4/1QN5/PP3PPP/R1B1KBNR w KQkq - 2 7' => {
			eco => 'B14f',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6. Qb3'),
			moves => {
				'c4d5' => 'rnbqk2r/pp2ppbp/5np1/3P4/3P4/1QN5/PP3PPP/R1B1KBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp2ppbp/5np1/3p4/3P4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 c6 6. cxd5 cxd5

			variation => N__('Grünfeld: Russian, 5... c6 6.cxd5 cxd5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/5np1/3p4/3P4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 7' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 c6 6. cxd5 cxd5 7. Nc3

			variation => N__('Neo-Grünfeld, 5. Nf3 c6 6.cxd5 cxd5 7. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/5np1/3p4/3P4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 7' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 c6 6. cxd5 cxd5

			variation => N__('Neo-Grünfeld, 5. Nf3 c6 6.cxd5 cxd5'),
			moves => {
				'b1c3' => 'rnbqk2r/pp2ppbp/5np1/3p4/3P4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 7',
			},
		},
		'rnbqk2r/pp2ppbp/6p1/1Bp5/3PP3/2P2N2/P4PPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'D85k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Bb5+

			variation => N__('Grünfeld: Modern Exchange, 8. Bb5+'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/2BPP3/2P5/P4PPP/R1BQK1NR w KQkq c6 0 8' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4 c5

			variation => N__('Grünfeld: Classical Exchange, 7... c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3P4/2P3P1/P3PPBP/R1BQK1NR w KQkq c6 0 8' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. Nc3 Nxc3 7. bxc3 c5

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. Nc3 Nxc3 7.bxc3 c5'),
			moves => {
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P1B3/P2Q1PPP/R3KBNR b KQkq - 1 8' => {
			eco => 'D85g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5 8. Qd2

			variation => N__('Grünfeld: Exchange, 7. Be3 c5 8. Qd2'),
			moves => {
				'd8a5' => 'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1B3/P2Q1PPP/R3KBNR w KQkq - 2 9',
				'e8g8' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1B3/P2Q1PPP/R3KBNR w KQ - 2 9',
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P1B3/P4PPP/R2QKBNR w KQkq c6 0 8' => {
			eco => 'D85g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3 c5

			variation => N__('Grünfeld: Exchange, 7. Be3 c5'),
			moves => {
				'd1d2' => 'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P1B3/P2Q1PPP/R3KBNR b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P1BN2/P4PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'D85t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be3

			variation => N__('Grünfeld: Modern Exchange, 8. Be3'),
			moves => {
				'b8c6' => 'r1bqk2r/pp2ppbp/2n3p1/2p5/3PP3/2P1BN2/P4PPP/R2QKB1R w KQkq - 2 9',
				'c8g4' => 'rn1qk2r/pp2ppbp/6p1/2p5/3PP1b1/2P1BN2/P4PPP/R2QKB1R w KQkq - 2 9',
				'd8a5' => 'rnb1k2r/pp2ppbp/6p1/q1p5/3PP3/2P1BN2/P4PPP/R2QKB1R w KQkq - 2 9',
				'e8g8' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P1BN2/P4PPP/R2QKB1R w KQ - 2 9',
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/R1BQK2R b KQkq - 1 8' => {
			eco => 'D85l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Be2

			variation => N__('Grünfeld: Modern Exchange, 8. Be2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/R1BQK2R w KQ - 2 9',
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P2N2/P4PPP/1RBQKB1R b Kkq - 1 8' => {
			eco => 'D85m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5 8. Rb1

			variation => N__('Grünfeld: Modern Exchange, 8. Rb1'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp2ppbp/6p1/2p5/3PP3/2P2N2/P4PPP/1RBQKB1R w K - 2 9',
			},
		},
		'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P2N2/P4PPP/R1BQKB1R w KQkq c6 0 8' => {
			eco => 'D85j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3 c5

			variation => N__('Grünfeld: Modern Exchange, 7... c5'),
			moves => {
				'a1b1' => 'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P2N2/P4PPP/1RBQKB1R b Kkq - 1 8',
				'c1e3' => 'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P1BN2/P4PPP/R2QKB1R b KQkq - 1 8',
				'f1b5' => 'rnbqk2r/pp2ppbp/6p1/1Bp5/3PP3/2P2N2/P4PPP/R1BQK2R b KQkq - 1 8',
				'f1e2' => 'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P2N2/P3BPPP/R1BQK2R b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp3p2/3ppn1p/2pP2p1/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq g6 0 8' => {
			eco => 'E31c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 g5

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 g5'),
			moves => {
			},
		},
		'rnbqk2r/pp3p2/4p1pp/2ppP3/3P2Q1/2PB1N2/P1PK1PPP/R6R b kq - 1 11' => {
			eco => 'C12q',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 g6 9. Bd3 Nxd2 10. Kxd2 c5 11. Nf3

			variation => N__('French: MacCutcheon, Main Line, 11. Nf3'),
			moves => {
				'b8c6' => 'r1bqk2r/pp3p2/2n1p1pp/2ppP3/3P2Q1/2PB1N2/P1PK1PPP/R6R w kq - 2 12',
			},
		},
		'rnbqk2r/pp3p2/4p1pp/2ppP3/3P2Q1/2PB4/P1PK1PPP/R5NR w kq c6 0 11' => {
			eco => 'C12p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 g6 9. Bd3 Nxd2 10. Kxd2 c5

			variation => N__('French: MacCutcheon, Main Line'),
			moves => {
				'g1f3' => 'rnbqk2r/pp3p2/4p1pp/2ppP3/3P2Q1/2PB1N2/P1PK1PPP/R6R b kq - 1 11',
			},
		},
		'rnbqk2r/pp3pb1/3p1n1p/2pP2p1/4P2B/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'A71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bg5 h6 9. Bh4 g5

			variation => N__('Benoni: Classical, 8. Bg5 h6 9. Bh4 g5'),
			moves => {
			},
		},
		'rnbqk2r/pp3pb1/3p1npp/2pP2B1/4P3/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 9' => {
			eco => 'A71',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Bg5'),
			moves => {
				'g5h4' => 'rnbqk2r/pp3pb1/3p1npp/2pP4/4P2B/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 9',
			},
		},
		'rnbqk2r/pp3pb1/3p1npp/2pP2B1/4P3/2N2P2/PP4PP/R2QKBNR w KQkq - 0 9' => {
			eco => 'A65r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6

			variation => N__('Benoni: Sämisch, 8. Bg5 h6'),
			moves => {
				'g5e3' => 'rnbqk2r/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP4PP/R2QKBNR b KQkq - 1 9',
			},
		},
		'rnbqk2r/pp3pb1/3p1npp/2pP4/4P2B/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 9' => {
			eco => 'A71',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. Bg5'),
			moves => {
				'a7a6' => 'rnbqk2r/1p3pb1/p2p1npp/2pP4/4P2B/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 10',
				'g6g5' => 'rnbqk2r/pp3pb1/3p1n1p/2pP2p1/4P2B/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 10',
			},
		},
		'rnbqk2r/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP4PP/R2QKBNR b KQkq - 1 9' => {
			eco => 'A65r',
                        # TRANSLATORS:

			variation => N__('Benoni: Sämisch, 8. Bg5 h6'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP4PP/R2QKBNR w KQ - 2 10',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/1BpP4/4PP2/2N5/PP4PP/R1BQK1NR b KQkq - 2 8' => {
			eco => 'A67a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Bb5+

			variation => N__('Benoni: Four Pawns, Taimanov (Alatortsev) Variation'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1n1pbp/3p1np1/1BpP4/4PP2/2N5/PP4PP/R1BQK1NR w KQkq - 3 9',
				'f6d7' => 'rnbqk2r/pp1n1pbp/3p2p1/1BpP4/4PP2/2N5/PP4PP/R1BQK1NR w KQkq - 3 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP2B1/4P3/2N2N2/PP3PPP/R2QKB1R b KQkq - 3 8' => {
			eco => 'A71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bg5

			variation => N__('Benoni: Classical, 8. Bg5'),
			moves => {
				'h7h6' => 'rnbqk2r/pp3pb1/3p1npp/2pP2B1/4P3/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP4PP/R2QKBNR b KQkq - 2 8' => {
			eco => 'A65n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5

			variation => N__('Benoni: Sämisch, 8. Bg5'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP4PP/R2QKBNR w KQ - 3 9',
				'h7h6' => 'rnbqk2r/pp3pb1/3p1npp/2pP2B1/4P3/2N2P2/PP4PP/R2QKBNR w KQkq - 0 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP2B1/8/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bg5 Bg7

			variation => N__('Benoni: Uhlmann, 7... Bg7'),
			moves => {
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/2N5/2N5/PP2PPPP/R1BQKB1R b KQkq - 3 8' => {
			eco => 'A61',
                        # TRANSLATORS:

			variation => N__('Benoni: Nimzowitsch, 7... Bg7'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/2N5/2N5/PP2PPPP/R1BQKB1R w KQ - 4 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N1BP2/PP4PP/R2QKBNR b KQkq - 2 8' => {
			eco => 'A65m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Be3

			variation => N__('Benoni: Sämisch, 8. Be3'),
			moves => {
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQKB1R b KQkq - 0 8' => {
			eco => 'A70j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3

			variation => N__('Benoni: Classical, 8. h3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQKB1R w KQ - 1 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 3 8' => {
			eco => 'A70e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2

			variation => N__('Benoni: Classical, 8. Be2'),
			moves => {
				'a7a6' => 'rnbqk2r/1p3pbp/p2p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQK2R w KQkq - 0 9',
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQK2R w KQ - 4 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'A70d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7

			variation => N__('Benoni: Classical, 7... Bg7'),
			moves => {
				'c1f4' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4PB2/2N2N2/PP3PPP/R2QKB1R b KQkq - 3 8',
				'c1g5' => 'rnbqk2r/pp3pbp/3p1np1/2pP2B1/4P3/2N2N2/PP3PPP/R2QKB1R b KQkq - 3 8',
				'd1a4' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/Q3P3/2N2N2/PP3PPP/R1B1KB1R b KQkq - 3 8',
				'f1d3' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 3 8',
				'f1e2' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQK2R b KQkq - 3 8',
				'h2h3' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2N1P/PP3PP1/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2P2/PP2N1PP/R1BQKB1R b KQkq - 2 8' => {
			eco => 'A65l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Nge2

			variation => N__('Benoni: Sämisch, 8.Nge2'),
			moves => {
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 8' => {
			eco => 'A65k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7

			variation => N__('Benoni: Sämisch, 7... Bg7'),
			moves => {
				'c1e3' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N1BP2/PP4PP/R2QKBNR b KQkq - 2 8',
				'c1g5' => 'rnbqk2r/pp3pbp/3p1np1/2pP2B1/4P3/2N2P2/PP4PP/R2QKBNR b KQkq - 2 8',
				'g1e2' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2P2/PP2N1PP/R1BQKB1R b KQkq - 2 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB1N2/PP3PPP/R1BQK2R b KQkq - 3 8' => {
			eco => 'A70h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bd3

			variation => N__('Benoni: Classical, 8. Bd3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB1N2/PP3PPP/R1BQK2R w KQ - 4 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB3P/PP3PP1/R1BQK1NR b KQkq - 0 8' => {
			eco => 'A65f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. h3

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8. h3'),
			moves => {
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQK2R b KQkq - 3 8' => {
			eco => 'A65g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3 Bg7 8. Nge2

			variation => N__('Benoni: 6. e4 g6 7. Bd3 Bg7 8.Nge2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQK2R w KQ - 4 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP3PPP/R1BQK1NR w KQkq - 2 8' => {
			eco => 'A65f',
                        # TRANSLATORS:

			variation => N__('Benoni: 6. e4 g6 7. Bd3'),
			moves => {
				'g1e2' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP2NPPP/R1BQK2R b KQkq - 3 8',
				'h2h3' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB3P/PP3PP1/R1BQK1NR b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4PB2/2N2N2/PP3PPP/R2QKB1R b KQkq - 3 8' => {
			eco => 'A70g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Bf4

			variation => N__('Benoni: Classical, 8. Bf4'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PB2/2N2N2/PP3PPP/R2QKB1R w KQ - 4 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP4PP/R1BQKB1R b KQkq - 2 8' => {
			eco => 'A68a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3

			variation => N__('Benoni: Four Pawns, 8. Nf3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP4PP/R1BQKB1R w KQ - 3 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/4PP2/2N5/PP4PP/R1BQKBNR w KQkq - 1 8' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7

			variation => N__('Benoni: Four Pawns Attack'),
			moves => {
				'e4e5' => 'rnbqk2r/pp3pbp/3p1np1/2pPP3/5P2/2N5/PP4PP/R1BQKBNR b KQkq - 0 8',
				'f1b5' => 'rnbqk2r/pp3pbp/3p1np1/1BpP4/4PP2/2N5/PP4PP/R1BQK1NR b KQkq - 2 8',
				'g1f3' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP4PP/R1BQKB1R b KQkq - 2 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/5B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bf4 Bg7

			variation => N__('Benoni: 6. Nf3 g6 7. Bf4 Bg7'),
			moves => {
				'd1a4' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/Q4B2/2N2N2/PP2PPPP/R3KB1R b KQkq - 3 8',
				'f3d2' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/5B2/2N5/PP1NPPPP/R2QKB1R b KQkq - 3 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/5B2/2N5/PP1NPPPP/R2QKB1R b KQkq - 3 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bf4 Bg7 8. Nd2

			variation => N__('Benoni: 6. Nf3 g6 7. Bf4 Bg7 8. Nd2'),
			moves => {
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 8' => {
			eco => 'A61',
                        # TRANSLATORS:

			variation => N__('Benoni: Fianchetto Variation'),
			moves => {
				'f1g2' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 2 8' => {
			eco => 'A61',
                        # TRANSLATORS:

			variation => N__('Benoni: Fianchetto Variation'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQK2R w KQ - 3 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Nd2 Bg7

			variation => N__('Benoni: Nimzowitsch, 7... Bg7'),
			moves => {
				'd2c4' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/2N5/2N5/PP2PPPP/R1BQKB1R b KQkq - 3 8',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/Q3P3/2N2N2/PP3PPP/R1B1KB1R b KQkq - 3 8' => {
			eco => 'A70f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Qa4+

			variation => N__('Benoni: Classical, 8. Qa4+'),
			moves => {
				'c8d7' => 'rn1qk2r/pp1b1pbp/3p1np1/2pP4/Q3P3/2N2N2/PP3PPP/R1B1KB1R w KQkq - 4 9',
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pP4/Q4B2/2N2N2/PP2PPPP/R3KB1R b KQkq - 3 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bf4 Bg7 8. Qa4+

			variation => N__('Benoni: 6. Nf3 g6 7. Bf4 Bg7 8. Qa4+'),
			moves => {
			},
		},
		'rnbqk2r/pp3pbp/3p1np1/2pPP3/5P2/2N5/PP4PP/R1BQKBNR b KQkq - 0 8' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5

			variation => N__('Benoni: Four Pawns, Mikenas Attack'),
			moves => {
				'd6e5' => 'rnbqk2r/pp3pbp/5np1/2pPp3/5P2/2N5/PP4PP/R1BQKBNR w KQkq - 0 9',
				'f6d7' => 'rnbqk2r/pp1n1pbp/3p2p1/2pPP3/5P2/2N5/PP4PP/R1BQKBNR w KQkq - 1 9',
			},
		},
		'rnbqk2r/pp3pbp/5np1/2pPp3/5P2/2N5/PP4PP/R1BQKBNR w KQkq - 0 9' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 dxe5

			variation => N__('Benoni: Mikenas Attack, 8...dxe5'),
			moves => {
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pPp3/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 8' => {
			eco => 'E31e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 e5

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 e5'),
			moves => {
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1P3/P1Q2PPP/R3KBNR b KQkq - 1 9' => {
			eco => 'E31l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 e5 9. Qc2

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... e5 9. Qc2'),
			moves => {
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1P3/P4PPP/R2QKBNR w KQkq - 0 9' => {
			eco => 'E31j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 e5

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... e5'),
			moves => {
				'd1c2' => 'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1P3/P1Q2PPP/R3KBNR b KQkq - 1 9',
				'f1d3' => 'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2PBP3/P4PPP/R2QK1NR b KQkq - 1 9',
				'f2f3' => 'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1PP2/P5PP/R2QKBNR b KQkq - 0 9',
				'g1f3' => 'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1PN2/P4PPP/R2QKB1R b KQkq - 1 9',
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1PN2/P4PPP/R2QKB1R b KQkq - 1 9' => {
			eco => 'E31k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 e5 9. Nf3

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... e5 9. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1PP2/P5PP/R2QKBNR b KQkq - 0 9' => {
			eco => 'E31n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 e5 9. f3

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... e5 9. f3'),
			moves => {
				'b8d7' => 'r1bqk2r/pp1n1pp1/3p1n1p/2pPp3/2P4B/2P1PP2/P5PP/R2QKBNR w KQkq - 1 10',
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2PBP3/P4PPP/R2QK1NR b KQkq - 1 9' => {
			eco => 'E31m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3 e5 9. Bd3

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 8... e5 9. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/pp3pp1/3p1n1p/2pp4/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 8' => {
			eco => 'E31d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 exd5

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 exd5'),
			moves => {
			},
		},
		'rnbqk2r/pp3pp1/3ppn1p/2pP4/1bP4B/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'E31b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3'),
			moves => {
				'b4c3' => 'rnbqk2r/pp3pp1/3ppn1p/2pP4/2P4B/2b1P3/PP3PPP/R2QKBNR w KQkq - 0 8',
				'd8e7' => 'rnb1k2r/pp2qpp1/3ppn1p/2pP4/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 8',
				'e6d5' => 'rnbqk2r/pp3pp1/3p1n1p/2pp4/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 8',
				'e6e5' => 'rnbqk2r/pp3pp1/3p1n1p/2pPp3/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 8',
				'e8g8' => 'rnbq1rk1/pp3pp1/3ppn1p/2pP4/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQ - 1 8',
				'g7g5' => 'rnbqk2r/pp3p2/3ppn1p/2pP2p1/1bP4B/2N1P3/PP3PPP/R2QKBNR w KQkq g6 0 8',
			},
		},
		'rnbqk2r/pp3pp1/3ppn1p/2pP4/1bP4B/2N5/PP2PPPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'E31a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6

			variation => N__('Nimzo-Indian: Leningrad, Main Line'),
			moves => {
				'e2e3' => 'rnbqk2r/pp3pp1/3ppn1p/2pP4/1bP4B/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp3pp1/3ppn1p/2pP4/2P4B/2P1P3/P4PPP/R2QKBNR b KQkq - 0 8' => {
			eco => 'E31h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+ 8. bxc3

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 Bxc3+'),
			moves => {
				'd8e7' => 'rnb1k2r/pp2qpp1/3ppn1p/2pP4/2P4B/2P1P3/P4PPP/R2QKBNR w KQkq - 1 9',
				'e6e5' => 'rnbqk2r/pp3pp1/3p1n1p/2pPp3/2P4B/2P1P3/P4PPP/R2QKBNR w KQkq - 0 9',
			},
		},
		'rnbqk2r/pp3pp1/3ppn1p/2pP4/2P4B/2b1P3/PP3PPP/R2QKBNR w KQkq - 0 8' => {
			eco => 'E31h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4 c5 6. d5 d6 7. e3 Bxc3+

			variation => N__('Nimzo-Indian: Leningrad, Main Line, 7. e3 Bxc3+'),
			moves => {
				'b2c3' => 'rnbqk2r/pp3pp1/3ppn1p/2pP4/2P4B/2P1P3/P4PPP/R2QKBNR b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3pp1/5n1p/2pp4/1b1P3B/2N5/PPQ1PPPP/R3KBNR w KQkq c6 0 8' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. Bg5 h6 7. Bh4 c5

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. Bg5 h6 7. Bh4 c5'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/2pbpn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D45b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Bd6

			variation => N__('Semi-Slav: 5. e3 Bd6'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/3ppn2/2p5/2PP4/P1P1P3/5PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'E26',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. e3 d6

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. e3 d6'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/3ppn2/2pP2B1/1bP5/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5 5. d5 d6 6. e3

			variation => N__('Nimzo-Indian: Leningrad, 4... c5 5. d5 d6 6. e3'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/3ppn2/2pP2B1/1bP5/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 c5 5. d5 d6

			variation => N__('Nimzo-Indian: Leningrad, 4... c5 5. d5 d6'),
			moves => {
				'e2e3' => 'rnbqk2r/pp3ppp/3ppn2/2pP2B1/1bP5/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pp3ppp/4p3/2Pn4/8/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 8' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5 Nxd5 8. dxc5

			variation => N__('Nimzo-Indian: Sämisch, Keres Variation'),
			moves => {
				'f7f5' => 'rnbqk2r/pp4pp/4p3/2Pn1p2/8/P1P2P2/4P1PP/R1BQKBNR w KQkq f6 0 9',
			},
		},
		'rnbqk2r/pp3ppp/4p3/2Pp4/2P1n3/P7/1PQ1PPPP/R1B1KBNR b KQkq - 0 8' => {
			eco => 'E37',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line, 7. Qc2 c5'),
			moves => {
				'b8c6' => 'r1bqk2r/pp3ppp/2n1p3/2Pp4/2P1n3/P7/1PQ1PPPP/R1B1KBNR w KQkq - 1 9',
			},
		},
		'rnbqk2r/pp3ppp/4p3/2pn4/3P4/P1P2P2/4P1PP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5 Nxd5

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5 7.cxd5 Nxd5'),
			moves => {
				'd1d3' => 'rnbqk2r/pp3ppp/4p3/2pn4/3P4/P1PQ1P2/4P1PP/R1B1KBNR b KQkq - 1 8',
				'd4c5' => 'rnbqk2r/pp3ppp/4p3/2Pn4/8/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4p3/2pn4/3P4/P1PQ1P2/4P1PP/R1B1KBNR b KQkq - 1 8' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5 Nxd5 8. Qd3

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5 7.cxd5 Nxd5 8. Qd3'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/4p3/2pp4/2PPn3/P7/1PQ1PPPP/R1B1KBNR w KQkq c6 0 8' => {
			eco => 'E37',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Ne4 7. Qc2 c5

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line, 7. Qc2 c5'),
			moves => {
				'd4c5' => 'rnbqk2r/pp3ppp/4p3/2Pp4/2P1n3/P7/1PQ1PPPP/R1B1KBNR b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4p3/3n4/1b1P4/2N2N2/PP1B1PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'B14v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5 8. Bd2

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Bd2'),
			moves => {
				'b8c6' => 'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2N2N2/PP1B1PPP/R2QKB1R w KQkq - 2 9',
			},
		},
		'rnbqk2r/pp3ppp/4p3/3n4/1b1P4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B14r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5'),
			moves => {
				'c1d2' => 'rnbqk2r/pp3ppp/4p3/3n4/1b1P4/2N2N2/PP1B1PPP/R2QKB1R b KQkq - 1 8',
				'd1c2' => 'rnbqk2r/pp3ppp/4p3/3n4/1b1P4/2N2N2/PPQ2PPP/R1B1KB1R b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp3ppp/4p3/3n4/1b1P4/2N2N2/PPQ2PPP/R1B1KB1R b KQkq - 1 8' => {
			eco => 'B14s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 Nxd5 8. Qc2

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 Nxd5 8. Qc2'),
			moves => {
				'b8c6' => 'r1bqk2r/pp3ppp/2n1p3/3n4/1b1P4/2N2N2/PPQ2PPP/R1B1KB1R w KQkq - 2 9',
			},
		},
		'rnbqk2r/pp3ppp/4p3/8/1b1PP3/5N2/P2B1PPP/R2QKB1R b KQkq - 2 9' => {
			eco => 'D41m',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4 Bb4+'),
			moves => {
				'b4d2' => 'rnbqk2r/pp3ppp/4p3/8/3PP3/5N2/P2b1PPP/R2QKB1R w KQkq - 0 10',
				'd8a5' => 'rnb1k2r/pp3ppp/4p3/q7/1b1PP3/5N2/P2B1PPP/R2QKB1R w KQkq - 3 10',
			},
		},
		'rnbqk2r/pp3ppp/4p3/8/1b1PP3/5N2/P4PPP/R1BQKB1R w KQkq - 1 9' => {
			eco => 'D41m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4 Bb4+

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4 Bb4+'),
			moves => {
				'c1d2' => 'rnbqk2r/pp3ppp/4p3/8/1b1PP3/5N2/P2B1PPP/R2QKB1R b KQkq - 2 9',
			},
		},
		'rnbqk2r/pp3ppp/4p3/8/3PP3/5N2/P2Q1PPP/R3KB1R b KQkq - 0 10' => {
			eco => 'D41m',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4 Bb4+'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3ppp/4p3/8/3PP3/5N2/P2Q1PPP/R3KB1R w KQ - 1 11',
			},
		},
		'rnbqk2r/pp3ppp/4p3/8/3PP3/5N2/P2b1PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'D41m',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4 Bb4+'),
			moves => {
				'd1d2' => 'rnbqk2r/pp3ppp/4p3/8/3PP3/5N2/P2Q1PPP/R3KB1R b KQkq - 0 10',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2p1P1B1/1bpP4/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D39e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. e5

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7. e5'),
			moves => {
				'c5d4' => 'rnbqk2r/pp3ppp/4pn2/4P1B1/1bpp4/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2p3B1/1bBPP3/2N2N2/PP3PPP/R2QK2R b KQkq - 0 7' => {
			eco => 'D39h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4'),
			moves => {
				'c5d4' => 'rnbqk2r/pp3ppp/4pn2/6B1/1bBpP3/2N2N2/PP3PPP/R2QK2R w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2p3B1/1bpPP3/2N2N2/PP3PPP/R2QKB1R w KQkq c6 0 7' => {
			eco => 'D39d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5'),
			moves => {
				'e4e5' => 'rnbqk2r/pp3ppp/4pn2/2p1P1B1/1bpP4/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 7',
				'f1c4' => 'rnbqk2r/pp3ppp/4pn2/2p3B1/1bBPP3/2N2N2/PP3PPP/R2QK2R b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2pP4/3P4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 7' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5 7.cxd5'),
			moves => {
				'f6d5' => 'rnbqk2r/pp3ppp/4p3/2pn4/3P4/P1P2P2/4P1PP/R1BQKBNR w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2pp4/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQkq d6 0 6' => {
			eco => 'E42',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Ne2 d5

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2 d5'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2pp4/2PP4/P1P1PP2/6PP/R1BQKBNR b KQkq - 0 7' => {
			eco => 'E24',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5'),
			moves => {
				'e8g8' => 'rnbq1rk1/pp3ppp/4pn2/2pp4/2PP4/P1P1PP2/6PP/R1BQKBNR w KQ - 1 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/2pp4/2PP4/P1P2P2/4P1PP/R1BQKBNR w KQkq d6 0 7' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5

			variation => N__('Nimzo-Indian: Sämisch, 5... c5 6. f3 d5'),
			moves => {
				'c4d5' => 'rnbqk2r/pp3ppp/4pn2/2pP4/3P4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 7',
				'e2e3' => 'rnbqk2r/pp3ppp/4pn2/2pp4/2PP4/P1P1PP2/6PP/R1BQKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/3P4/1b1P4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B14p',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4'),
			moves => {
				'e6d5' => 'rnbqk2r/pp3ppp/5n2/3p4/1b1P4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8',
				'f6d5' => 'rnbqk2r/pp3ppp/4p3/3n4/1b1P4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/3p4/1bPP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B14p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4'),
			moves => {
				'c4d5' => 'rnbqk2r/pp3ppp/4pn2/3P4/1b1P4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/3p4/1bPP4/2N5/PP2NPPP/R1BQKB1R w KQkq d6 0 7' => {
			eco => 'E42',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 c5 5. Ne2 cxd4 6. exd4 d5

			variation => N__('Nimzo-Indian: 4. e3 c5 5. Ne2 cxd4 6.exd4 d5'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/4pn2/4P1B1/1bpp4/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D39f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. e5 cxd4

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7. e5 cxd4'),
			moves => {
				'd1a4' => 'rnbqk2r/pp3ppp/4pn2/4P1B1/Qbpp4/2N2N2/PP3PPP/R3KB1R b KQkq - 1 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/4P1B1/Qbpp4/2N2N2/PP3PPP/R3KB1R b KQkq - 1 8' => {
			eco => 'D39f',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7. e5 cxd4'),
			moves => {
				'b8c6' => 'r1bqk2r/pp3ppp/2n1pn2/4P1B1/Qbpp4/2N2N2/PP3PPP/R3KB1R w KQkq - 2 9',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/6B1/1bBNP3/2N5/PP3PPP/R2QK2R b KQkq - 0 8' => {
			eco => 'D39h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4 cxd4 8. Nxd4

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4'),
			moves => {
				'b4c3' => 'rnbqk2r/pp3ppp/4pn2/6B1/2BNP3/2b5/PP3PPP/R2QK2R w KQkq - 0 9',
				'd8a5' => 'rnb1k2r/pp3ppp/4pn2/q5B1/1bBNP3/2N5/PP3PPP/R2QK2R w KQkq - 1 9',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/6B1/1bBpP3/2N2N2/PP3PPP/R2QK2R w KQkq - 0 8' => {
			eco => 'D39h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4 cxd4

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4'),
			moves => {
				'f3d4' => 'rnbqk2r/pp3ppp/4pn2/6B1/1bBNP3/2N5/PP3PPP/R2QK2R b KQkq - 0 8',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/6B1/2BNP3/2P5/P4PPP/R2QK2R b KQkq - 0 9' => {
			eco => 'D39j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4 cxd4 8. Nxd4 Bxc3+ 9. bxc3

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4, 8...Bxc3+'),
			moves => {
				'd8a5' => 'rnb1k2r/pp3ppp/4pn2/q5B1/2BNP3/2P5/P4PPP/R2QK2R w KQkq - 1 10',
			},
		},
		'rnbqk2r/pp3ppp/4pn2/6B1/2BNP3/2b5/PP3PPP/R2QK2R w KQkq - 0 9' => {
			eco => 'D39j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4 c5 7. Bxc4 cxd4 8. Nxd4 Bxc3+

			variation => N__('QGD: Ragozin, Vienna, 6. e4 c5 7.Bxc4, 8...Bxc3+'),
			moves => {
				'b2c3' => 'rnbqk2r/pp3ppp/4pn2/6B1/2BNP3/2P5/P4PPP/R2QK2R b KQkq - 0 9',
			},
		},
		'rnbqk2r/pp3ppp/5n2/2pp2B1/1b1P4/2N5/PPQ1PPPP/R3KBNR w KQkq c6 0 7' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. Bg5 c5

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. Bg5 c5'),
			moves => {
			},
		},
		'rnbqk2r/pp3ppp/5n2/3p4/1b1P4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B14q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3 Bb4 7. cxd5 exd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3 Bb4 7.cxd5 exd5'),
			moves => {
			},
		},
		'rnbqk2r/pp4pp/2pbpn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5 6. O-O Bd6 7. b3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 5. Nf3 d5 6.O-O Bd6 7. b3'),
			moves => {
			},
		},
		'rnbqk2r/pp4pp/2pbpn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5 6. O-O Bd6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 5. Nf3 d5 6.O-O Bd6'),
			moves => {
				'b2b3' => 'rnbqk2r/pp4pp/2pbpn2/3p1p2/2PP4/1P3NP1/P3PPBP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rnbqk2r/pp4pp/2pbpn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB1K2R w KQkq - 2 7' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5 6. Qc2 Bd6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 5. Nf3 d5 6. Qc2 Bd6'),
			moves => {
			},
		},
		'rnbqk2r/pp4pp/4p3/2Pn1p2/4P3/P1P2P2/6PP/R1BQKBNR b KQkq e3 0 9' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5 Nxd5 8. dxc5 f5 9. e4

			variation => N__('Nimzo-Indian: Sämisch, Keres, Romanovsky, 9. e4'),
			moves => {
			},
		},
		'rnbqk2r/pp4pp/4p3/2Pn1p2/8/P1P2P1N/4P1PP/R1BQKB1R b KQkq - 1 9' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5 Nxd5 8. dxc5 f5 9. Nh3

			variation => N__('Nimzo-Indian: Sämisch, Keres, Romanovsky, 9. Nh3'),
			moves => {
			},
		},
		'rnbqk2r/pp4pp/4p3/2Pn1p2/8/P1P2P2/4P1PP/R1BQKBNR w KQkq f6 0 9' => {
			eco => 'E25',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 c5 6. f3 d5 7. cxd5 Nxd5 8. dxc5 f5

			variation => N__('Nimzo-Indian: Sämisch, Keres, Romanovsky Variation'),
			moves => {
				'e2e4' => 'rnbqk2r/pp4pp/4p3/2Pn1p2/4P3/P1P2P2/6PP/R1BQKBNR b KQkq e3 0 9',
				'g1h3' => 'rnbqk2r/pp4pp/4p3/2Pn1p2/8/P1P2P1N/4P1PP/R1BQKB1R b KQkq - 1 9',
			},
		},
		'rnbqk2r/ppbp1ppp/5n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'A60l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 Bd6 6. Nf3 Bc7

			variation => N__('Benoni: Snake, 6. Nf3 Bc7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1Bppp/4p3/3pP3/3Pn3/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C13g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Ne4 6. Bxe7

			variation => N__('French: Classical, Tartakower, 6.Bxe7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1b1pp/3p1n2/3Pp3/4N3/2PB4/PP3PPP/RNBQK2R w KQkq - 3 9' => {
			eco => 'C44j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nc6 3. c3 f5 4. d4 d6 5. d5 fxe4 6. Ng5 Nb8 7. Nxe4 Nf6 8. Bd3 Be7

			variation => N__('Ponziani Countergambit, Cordel Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d6 6. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d6 6. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1b1pp/3ppn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d6'),
			moves => {
				'b1c3' => 'rnbqk2r/ppp1b1pp/3ppn2/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1b1pp/4pn2/3p1p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQkq d6 0 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3 d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3 d5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d5 6. O-O

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d5 6.O-O'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2b1pp/2p1pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'rnbqk2r/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq d6 0 6' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3 d5'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqk2r/ppp1bp1p/4p3/5p2/3PN3/5N2/PPP2PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'C11x',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 gxf6 7. Nf3 f5

			variation => N__('French: Burn, 6.Bxf6 gxf6 7. Nf3 f5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bp1p/4pp2/8/3PN3/5N2/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'C11v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 gxf6 7. Nf3

			variation => N__('French: Burn, 6.Bxf6 gxf6 7. Nf3'),
			moves => {
				'b7b6' => 'rnbqk2r/p1p1bp1p/1p2pp2/8/3PN3/5N2/PPP2PPP/R2QKB1R w KQkq - 0 8',
				'f6f5' => 'rnbqk2r/ppp1bp1p/4p3/5p2/3PN3/5N2/PPP2PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/ppp1bp1p/4pp2/8/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C11u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 gxf6

			variation => N__('French: Burn, 6.Bxf6 gxf6'),
			moves => {
				'g1f3' => 'rnbqk2r/ppp1bp1p/4pp2/8/3PN3/5N2/PPP2PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp1bpp1/4pB1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D53e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bxf6

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6.Bxf6'),
			moves => {
				'e7f6' => 'rnbqk2r/ppp2pp1/4pb1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1bpp1/4pn1p/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D53t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 h6

			variation => N__('QGD: 4. Bg5 Be7 5. e3 h6'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bpp1/4pn1p/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D53d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6'),
			moves => {
				'g5f6' => 'rnbqk2r/ppp1bpp1/4pB1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6',
				'g5h4' => 'rnbqk2r/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D53h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bh4

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6. Bh4'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bpp1/4pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQ - 2 7',
			},
		},
		'rnbqk2r/ppp1bpp1/5n1p/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'D35j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Be7 6. e3 h6

			variation => N__('QGD: Exchange, 5. Bg5 Be7 6. e3 h6'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/1n1p4/8/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 2 7' => {
			eco => 'B03q',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6 exd6 6. Nc3 Be7

			variation => N__('Alekhine: Exchange, 5...exd6 6. Nc3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/3p1n2/3Pp3/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. d5 Be7

			variation => N__('Old Indian: Ukrainian, 4. d5 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/3p1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'C41l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 Nf6 5. Nc3 Be7 6. Bc4

			variation => N__('Philidor: 3...exd4 4.Nxd4 Nf6 5. Nc3 Be7 6. Bc4'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/3p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6' => {
			eco => 'C41m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 Nf6 5. Nc3 Be7 6. Be2

			variation => N__('Philidor: Antoshin Variation'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/3p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQ - 5 7',
			},
		},
		'rnbqk2r/ppp1bppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6' => {
			eco => 'C41k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 Nf6 5. Nc3 Be7

			variation => N__('Philidor: 3...exd4 4.Nxd4 Nf6 5. Nc3 Be7'),
			moves => {
				'f1c4' => 'rnbqk2r/ppp1bppp/3p1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 4 6',
				'f1e2' => 'rnbqk2r/ppp1bppp/3p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 4 6',
			},
		},
		'rnbqk2r/ppp1bppp/3ppn2/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d6 4. d4 Be7

			variation => N__('English: Mikenas, 3... d6 4. d4 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4p3/3n2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D53b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. cxd5 Nxd5

			variation => N__('QGD: 4. Bg5 Be7 5.cxd5 Nxd5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4p3/3n4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D37d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. cxd5 Nxd5

			variation => N__('QGD: 4. Nf3 Be7 5.cxd5 Nxd5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4p3/3p2B1/2PPn3/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'D53q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 Ne4

			variation => N__('QGD: Early Lasker Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4p3/3pP1B1/3Pn3/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'C13f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Ne4

			variation => N__('French: Classical, Tartakower Variation'),
			moves => {
				'g5e7' => 'rnbqk2r/ppp1Bppp/4p3/3pP3/3Pn3/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4p3/3pP3/3P2Q1/2N5/PPP2PPP/R3KBNR b KQkq - 2 7' => {
			eco => 'C13c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. Bxf6 Bxf6 6. e5 Be7 7. Qg4

			variation => N__('French: Classical, Anderssen-Richter Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4p3/3pP3/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'C13b',
                        # TRANSLATORS:

			variation => N__('French: Classical, Anderssen Variation'),
			moves => {
				'd1g4' => 'rnbqk2r/ppp1bppp/4p3/3pP3/3P2Q1/2N5/PPP2PPP/R3KBNR b KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp1bppp/4pB2/3p4/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'C13b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. Bxf6

			variation => N__('French: Classical, Anderssen Variation'),
			moves => {
				'e7f6' => 'rnbqk2r/ppp2ppp/4pb2/3p4/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pB2/8/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C11s',
                        # TRANSLATORS:

			variation => N__('French: Burn, 5.Nxe4 Be7'),
			moves => {
				'e7f6' => 'rnbqk2r/ppp2ppp/4pb2/8/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 0 7',
				'g7f6' => 'rnbqk2r/ppp1bp1p/4pp2/8/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3P2B1/3P4/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D53a',
                        # TRANSLATORS:

			variation => N__('QGD: 4. Bg5 Be7'),
			moves => {
				'f6d5' => 'rnbqk2r/ppp1bppp/4p3/3n2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3P4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D37d',
                        # TRANSLATORS:

			variation => N__('QGD: 4. Nf3 Be7'),
			moves => {
				'f6d5' => 'rnbqk2r/ppp1bppp/4p3/3n4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D53p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3

			variation => N__('QGD: 4. Bg5 Be7 5. e3'),
			moves => {
				'b8d7' => 'r1bqk2r/pppnbppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 6',
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQ - 1 6',
				'f6e4' => 'rnbqk2r/ppp1bppp/4p3/3p2B1/2PPn3/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 6',
				'h7h6' => 'rnbqk2r/ppp1bpp1/4pn1p/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5' => {
			eco => 'D53c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQ - 6 6',
				'h7h6' => 'rnbqk2r/ppp1bpp1/4pn1p/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq - 4 5' => {
			eco => 'D53a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7

			variation => N__('QGD: 4. Bg5 Be7'),
			moves => {
				'c4d5' => 'rnbqk2r/ppp1bppp/4pn2/3P2B1/3P4/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5',
				'e2e3' => 'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5',
				'g1f3' => 'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/4PN2/PP3PPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'D30x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 Be7 5. e3

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5 Be7 5. e3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 4 5' => {
			eco => 'D30x',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 Be7

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5 Be7'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/4PN2/PP3PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p2B1/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 4 5' => {
			eco => 'C13a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7

			variation => N__('French: Classical'),
			moves => {
				'e4e5' => 'rnbqk2r/ppp1bppp/4pn2/3pP1B1/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5',
				'g5f6' => 'rnbqk2r/ppp1bppp/4pB2/3p4/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 4 5' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7 5. O-O

			variation => N__('English: Neo-Catalan Declined'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2bppp/4pn2/2pp4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq c6 0 6',
				'c7c6' => 'rnbqk2r/pp2bppp/2p1pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq - 0 6',
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w - - 5 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 3 5' => {
			eco => 'A14',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 Be7

			variation => N__('English: Neo-Catalan Declined'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 4 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5' => {
			eco => 'D37h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. Bf4

			variation => N__('QGD: Classical Variation'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQ - 6 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D37e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7 5. e3

			variation => N__('QGD: 4. Nf3 Be7 5. e3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'D37d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Be7

			variation => N__('QGD: 4. Nf3 Be7'),
			moves => {
				'c1f4' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5',
				'c4d5' => 'rnbqk2r/ppp1bppp/4pn2/3P4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
				'e2e3' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPP1P/RN1QKB1R w KQkq d6 0 6' => {
			eco => 'E11g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5'),
			moves => {
				'f1g2' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1QK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1QK2R b KQkq - 1 6' => {
			eco => 'E11g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3 d5 6. Bg2

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3 d5 6. Bg2'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPPBP/RN1QK2R w KQ - 2 7',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 2 5' => {
			eco => 'E00s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 Be7

			variation => N__('Catalan: 4. Nf3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7 5. Nf3

			variation => N__('Catalan: Closed, 5. Nf3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'E06',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Be7

			variation => N__('Catalan: Closed'),
			moves => {
				'g1f3' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/P1N2P2/1P2P1PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. f3 d5 5. a3 Be7

			variation => N__('Nimzo-Indian: 4. f3 d5 5. a3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/P1N5/1PQ1PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Be7

			variation => N__('Nimzo-Indian: Classical, Noa, 5. a3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/3PP3/3B1N2/PPPN1PPP/R1BQK2R b KQkq - 5 5' => {
			eco => 'C03k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7 4. Ngf3 Nf6 5. Bd3

			variation => N__('French: Tarrasch, 3... Be7 4.Ngf3 Nf6 5. Bd3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'C03j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Be7 4. Ngf3 Nf6

			variation => N__('French: Tarrasch, 3... Be7 4.Ngf3 Nf6'),
			moves => {
				'f1d3' => 'rnbqk2r/ppp1bppp/4pn2/3p4/3PP3/3B1N2/PPPN1PPP/R1BQK2R b KQkq - 5 5',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/3pP1B1/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'C13d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5

			variation => N__('French: Classical, 5. e5'),
			moves => {
				'f6d7' => 'rnbqk2r/pppnbppp/4p3/3pP1B1/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6',
				'f6e4' => 'rnbqk2r/ppp1bppp/4p3/3pP1B1/3Pn3/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6',
				'f6g8' => 'rnbqk1nr/ppp1bppp/4p3/3pP1B1/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'C11s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7

			variation => N__('French: Burn, 5.Nxe4 Be7'),
			moves => {
				'g5f6' => 'rnbqk2r/ppp1bppp/4pB2/8/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 3 6' => {
			eco => 'E05b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. Nc3

			variation => N__('Catalan: Open, Classical, 6. Nc3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQK2R w KQ - 4 7',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 3 6' => {
			eco => 'E05e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7 6. O-O

			variation => N__('Catalan: Open, Classical, 6.O-O'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w - - 4 7',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6' => {
			eco => 'E05a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Be7

			variation => N__('Catalan: Open, Classical'),
			moves => {
				'b1c3' => 'rnbqk2r/ppp1bppp/4pn2/8/2pP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 3 6',
				'e1g1' => 'rnbqk2r/ppp1bppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 3 6',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/8/3PN3/P2Q4/1PP1NPPP/R1B1KB1R b KQkq - 2 7' => {
			eco => 'C15v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Be7 6. Nxe4 Nf6 7. Qd3

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7 6.Nxe4 Nf6 7. Qd3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/8/3PN3/P5N1/1PP2PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'C15u',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7 6.Nxe4 Nf6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1bppp/4pn2/8/3PN3/P5N1/1PP2PPP/R1BQKB1R w KQ - 3 8',
			},
		},
		'rnbqk2r/ppp1bppp/4pn2/8/3PN3/P7/1PP1NPPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'C15u',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 dxe4 5. a3 Be7 6. Nxe4 Nf6

			variation => N__('French: Winawer, Alekhine Gambit, 5. a3 Be7 6.Nxe4 Nf6'),
			moves => {
				'd1d3' => 'rnbqk2r/ppp1bppp/4pn2/8/3PN3/P2Q4/1PP1NPPP/R1B1KB1R b KQkq - 2 7',
				'e2g3' => 'rnbqk2r/ppp1bppp/4pn2/8/3PN3/P5N1/1PP2PPP/R1BQKB1R b KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D35j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Be7 6. e3

			variation => N__('QGD: Exchange, 5. Bg5 Be7 6. e3'),
			moves => {
				'b8d7' => 'r1bqk2r/pppnbppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7',
				'e8g8' => 'rnbq1rk1/ppp1bppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQ - 1 7',
				'h7h6' => 'rnbqk2r/ppp1bpp1/5n1p/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D35i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Be7 6. Nf3

			variation => N__('QGD: Exchange, 5. Bg5 Be7 6. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 2 6' => {
			eco => 'D35i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 Be7

			variation => N__('QGD: Exchange, 5. Bg5 Be7'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp1bppp/5n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
				'g1f3' => 'rnbqk2r/ppp1bppp/5n2/3p2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'C01f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nc3 Be7 6. Nf3

			variation => N__('French: Exchange, 4. c4 Nf6 5. Nc3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 3 6' => {
			eco => 'C01f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nc3 Be7

			variation => N__('French: Exchange, 4. c4 Nf6 5. Nc3 Be7'),
			moves => {
				'g1f3' => 'rnbqk2r/ppp1bppp/5n2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 4 6',
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6' => {
			eco => 'D35g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Be7 6. Bf4

			variation => N__('QGD: Exchange, 5. Nf3 Be7 6. Bf4'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2bppp/2p2n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1bppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D35g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 Be7

			variation => N__('QGD: Exchange, 5. Nf3 Be7'),
			moves => {
				'c1f4' => 'rnbqk2r/ppp1bppp/5n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 6',
			},
		},
		'rnbqk2r/ppp1bppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 b kq - 3 7' => {
			eco => 'C42s',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, 6. Bd3 Be7'),
			moves => {
				'b8c6' => 'r1bqk2r/ppp1bppp/2n5/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 w kq - 4 8',
				'e8g8' => 'rnbq1rk1/ppp1bppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 w - - 4 8',
			},
		},
		'rnbqk2r/ppp1bppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 7' => {
			eco => 'C42s',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Be7

			variation => N__('Russian Game: Classical, 6. Bd3 Be7'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp1bppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 b kq - 3 7',
			},
		},
		'rnbqk2r/ppp1bppp/8/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C43l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Be7

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/8/3pP3/4n3/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C29r',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Be7

			variation => N__('Vienna Gambit: Breyer Variation'),
			moves => {
				'd1e2' => 'rnbqk2r/ppp1bppp/8/3pP3/4n3/2N2N2/PPPPQ1PP/R1B1KB1R b KQkq - 3 6',
			},
		},
		'rnbqk2r/ppp1bppp/8/3pP3/4n3/2N2N2/PPPPQ1PP/R1B1KB1R b KQkq - 3 6' => {
			eco => 'C29s',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Be7 6. Qe2

			variation => N__('Vienna Gambit: Breyer, 6. Qe2'),
			moves => {
				'e4c3' => 'rnbqk2r/ppp1bppp/8/3pP3/8/2n2N2/PPPPQ1PP/R1B1KB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1bppp/8/3pP3/8/2P2N2/PPP1Q1PP/R1B1KB1R b KQkq - 0 7' => {
			eco => 'C29t',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Be7 6. Qe2 Nxc3 7. dxc3

			variation => N__('Vienna Gambit: Breyer, 6. Qe2 Nxc3 7.dxc3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1bppp/8/3pP3/8/2n2N2/PPPPQ1PP/R1B1KB1R w KQkq - 0 7' => {
			eco => 'C29t',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Be7 6. Qe2 Nxc3

			variation => N__('Vienna Gambit: Breyer, 6. Qe2 Nxc3'),
			moves => {
				'd2c3' => 'rnbqk2r/ppp1bppp/8/3pP3/8/2P2N2/PPP1Q1PP/R1B1KB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1npbp/6p1/3pp3/8/3P1NP1/PPP1PPBP/RNBQ1RK1 w kq - 1 6' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 g6 3. Bg2 Bg7 4. O-O e5 5. d3 Ne7

			variation => N__('Reti: KIA, Pachman'),
			moves => {
			},
		},
		'rnbqk2r/ppp1nppp/4p3/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C16j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Ne7

			variation => N__('French: Winawer, Advance, 4... Ne7'),
			moves => {
				'a2a3' => 'rnbqk2r/ppp1nppp/4p3/3pP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5',
				'c1d2' => 'rnbqk2r/ppp1nppp/4p3/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 2 5',
			},
		},
		'rnbqk2r/ppp1nppp/4p3/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 2 5' => {
			eco => 'C16k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Ne7 5. Bd2

			variation => N__('French: Winawer, Advance, 4... Ne7 5. Bd2'),
			moves => {
				'b7b6' => 'rnbqk2r/p1p1nppp/1p2p3/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1nppp/4p3/3pP3/1b1P4/P1N5/1PP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C16m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 Ne7 5. a3

			variation => N__('French: Winawer, Advance, 4... Ne7 5. a3'),
			moves => {
				'b4c3' => 'rnbqk2r/ppp1nppp/4p3/3pP3/3P4/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1nppp/4p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C16m',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Advance, 4... Ne7 5. a3'),
			moves => {
				'b7b6' => 'rnbqk2r/p1p1nppp/1p2p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1nppp/4p3/3pP3/3P4/P1b5/1PP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C16m',
                        # TRANSLATORS:

			variation => N__('French: Winawer, Advance, 4... Ne7 5. a3'),
			moves => {
				'b2c3' => 'rnbqk2r/ppp1nppp/4p3/3pP3/3P4/P1P5/2P2PPP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1nppp/8/3p3Q/1b1P4/2NB4/PPP2PPP/R1B1K1NR b KQkq - 5 6' => {
			eco => 'C01v',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Bb4 5. Bd3 Ne7 6. Qh5

			variation => N__('French: Exchange, Canal Attack'),
			moves => {
			},
		},
		'rnbqk2r/ppp1nppp/8/3p4/1b1P4/2NB4/PPP2PPP/R1BQK1NR w KQkq - 4 6' => {
			eco => 'C01v',
                        # TRANSLATORS:

			variation => N__('French: Exchange Winawer, 5. Bd3'),
			moves => {
				'd1h5' => 'rnbqk2r/ppp1nppp/8/3p3Q/1b1P4/2NB4/PPP2PPP/R1B1K1NR b KQkq - 5 6',
			},
		},
		'rnbqk2r/ppp1p1bp/3p1np1/3P1p2/2P5/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nc3 d6 6. d5

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nc3 d6 6. d5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6 6. Nc3

			variation => N__('Dutch: Leningrad, Main Line, 5... d6 6. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nc3 d6

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nc3 d6'),
			moves => {
				'd4d5' => 'rnbqk2r/ppp1p1bp/3p1np1/3P1p2/2P5/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 0 6',
				'g1h3' => 'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/2N3PN/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/2N3PN/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nc3 d6 6. Nh3

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nc3 d6 6. Nh3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3 d6

			variation => N__('Dutch: Leningrad, Main Line, 5... d6'),
			moves => {
				'b1c3' => 'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1ppb1/3p1npp/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'E61j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. Bg5 h6

			variation => N__('King\'s Indian: Smyslov System'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppb1/3p1npp/6B1/2PPP3/2N5/PP3PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'E70t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bg5 h6

			variation => N__('King\'s Indian: Accelerated Averbakh, 5... h6'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n1p2p1/4P1N1/3P4/1B6/PPP2PPP/RNBQK2R b KQkq - 5 7' => {
			eco => 'B04o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 g6 5. Bc4 Nb6 6. Bb3 Bg7 7. Ng5

			variation => N__('Alekhine: Modern, Fianchetto, 5. Bc4 Nb6 6. Bb3 Bg7 7. Ng5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n1p2p1/4P3/3P4/1B3N2/PPP2PPP/RNBQK2R w KQkq - 4 7' => {
			eco => 'B04m',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 g6 5. Bc4 Nb6 6. Bb3 Bg7

			variation => N__('Alekhine: Modern, Fianchetto, 5. Bc4 Nb6 6. Bb3 Bg7'),
			moves => {
				'a2a4' => 'rnbqk2r/ppp1ppbp/1n1p2p1/4P3/P2P4/1B3N2/1PP2PPP/RNBQK2R b KQkq a3 0 7',
				'f3g5' => 'rnbqk2r/ppp1ppbp/1n1p2p1/4P1N1/3P4/1B6/PPP2PPP/RNBQK2R b KQkq - 5 7',
			},
		},
		'rnbqk2r/ppp1ppbp/1n1p2p1/4P3/P2P4/1B3N2/1PP2PPP/RNBQK2R b KQkq a3 0 7' => {
			eco => 'B04n',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 g6 5. Bc4 Nb6 6. Bb3 Bg7 7. a4

			variation => N__('Alekhine: Modern, Fianchetto, Keres Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/2PP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6 6. Nc3 Bg7

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6 6. Nc3 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/2PP4/5N1P/PP3PP1/RNBQKB1R w KQkq - 1 7' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6 6. h3 Bg7

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6 6. h3 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/2PP4/5N2/PP2BPPP/RNBQK2R w KQkq - 2 7' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6 6. Be2 Bg7

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6 6. Be2 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3P4/2N3P1/PP2PPBP/R1BQK1NR w KQkq - 2 7' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. Nc3 Nb6

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. Nc3 Nb6'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N1B3/PP3PPP/R2QKBNR b KQkq - 2 7' => {
			eco => 'D85c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. Bd2 Bg7 6. e4 Nb6 7. Be3

			variation => N__('Grünfeld: Exchange, 5. Bd2 Bg7 6. e4 Nb6 7. Be3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N1BP2/PP4PP/R2QKBNR b KQkq - 4 7' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: Alekhine\'s, 5. e4 Nb6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/1n4p1/8/3PP3/2N1BP2/PP4PP/R2QKBNR w KQ - 5 8',
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 3 7' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: Alekhine\'s, 5. e4 Nb6'),
			moves => {
				'c1e3' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N1BP2/PP4PP/R2QKBNR b KQkq - 4 7',
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N5/PP1B1PPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D85c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. Bd2 Bg7 6. e4 Nb6

			variation => N__('Grünfeld: Exchange, 5. Bd2 Bg7 6. e4 Nb6'),
			moves => {
				'd2e3' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N1B3/PP3PPP/R2QKBNR b KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/6P1/PP2NPBP/RNBQK2R b KQkq - 2 7' => {
			eco => 'D72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6 7. Ne2

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6 7. Ne2'),
			moves => {
				'b8c6' => 'r1bqk2r/ppp1ppbp/1nn3p1/8/3PP3/6P1/PP2NPBP/RNBQK2R w KQkq - 3 8',
				'c7c5' => 'rnbqk2r/pp2ppbp/1n4p1/2p5/3PP3/6P1/PP2NPBP/RNBQK2R w KQkq c6 0 8',
				'e7e5' => 'rnbqk2r/ppp2pbp/1n4p1/4p3/3PP3/6P1/PP2NPBP/RNBQK2R w KQkq e6 0 8',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/1n4p1/8/3PP3/6P1/PP2NPBP/RNBQK2R w KQ - 3 8',
			},
		},
		'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/6P1/PP3PBP/RNBQK1NR w KQkq - 1 7' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6'),
			moves => {
				'g1e2' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/6P1/PP2NPBP/RNBQK2R b KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'E61j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. Bg5

			variation => N__('King\'s Indian: Smyslov System'),
			moves => {
				'h7h6' => 'rnbqk2r/ppp1ppb1/3p1npp/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP3PPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'E70s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bg5

			variation => N__('King\'s Indian: Accelerated Averbakh System'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP3PPP/R2QKBNR w KQ - 2 6',
				'h7h6' => 'rnbqk2r/ppp1ppb1/3p1npp/6B1/2PPP3/2N5/PP3PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/6B1/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 2 5' => {
			eco => 'B07x',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Bg5 Bg7

			variation => N__('Pirc: 4. Bg5 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2BPP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 3 5' => {
			eco => 'B08g',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Bc4

			variation => N__('Pirc: Classical, 5. Bc4'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/2BPP3/2N2N2/PPP2PPP/R1BQK2R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'B07s',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Bc4 Bg7

			variation => N__('Pirc: Holmov (4. Bc4)'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2BPPP2/2N5/PPP3PP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'B09c',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Bc4

			variation => N__('Pirc: Austrian, Ljubojevic Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/1P3N2/P3PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'E60h',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: b3 System'),
			moves => {
				'c1b2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/1P3N2/PB2PPPP/RN1QKB1R b KQkq - 1 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/1P3N2/PB2PPPP/RN1QKB1R b KQkq - 1 5' => {
			eco => 'E60h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3 d6 5. Bb2

			variation => N__('King\'s Indian: b3 System'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/1P3N2/PB2PPPP/RN1QKB1R w KQ - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'E61i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. e3

			variation => N__('King\'s Indian: 4. Nf3 d6 5. e3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'E61i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6

			variation => N__('King\'s Indian: 4. Nf3 d6'),
			moves => {
				'c1g5' => 'rnbqk2r/ppp1ppbp/3p1np1/6B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5',
				'e2e3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'g2g3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'E62a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3 d6 5. g3

			variation => N__('King\'s Indian: Fianchetto Variation'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PP4/2N2NP1/PP2PP1P/R1BQKB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq - 0 5' => {
			eco => 'E61f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3 d6

			variation => N__('King\'s Indian: 4. g3 d6'),
			moves => {
				'f1g2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'E61f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3 d6 5. Bg2

			variation => N__('King\'s Indian: 4. g3 d6 5. Bg2'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E90a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3

			variation => N__('King\'s Indian: 5. Nf3'),
			moves => {
				'b8d7' => 'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6',
				'c7c5' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq c6 0 6',
				'c8g4' => 'rn1qk2r/ppp1ppbp/3p1np1/8/2PPP1b1/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R w KQ - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E80a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3

			variation => N__('King\'s Indian: Sämisch Variation'),
			moves => {
				'a7a6' => 'rnbqk2r/1pp1ppbp/p2p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 0 6',
				'b8a6' => 'r1bqk2r/ppp1ppbp/n2p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6',
				'b8c6' => 'r1bqk2r/ppp1ppbp/2np1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6',
				'b8d7' => 'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6',
				'c7c5' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq c6 0 6',
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 0 6',
				'e7e5' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq e6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N3P1/PP3P1P/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. g3

			variation => N__('King\'s Indian: 4. e4 d6 5. g3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N3P1/PP3P1P/R1BQKBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N4P/PP3PP1/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E71a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. h3

			variation => N__('King\'s Indian: Makagonov System'),
			moves => {
				'b8d7' => 'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N4P/PP3PP1/R1BQKBNR w KQkq - 1 6',
				'c7c5' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N4P/PP3PP1/R1BQKBNR w KQkq c6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N4P/PP3PP1/R1BQKBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2BPPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'E73a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2

			variation => N__('King\'s Indian: 5. Be2'),
			moves => {
				'b8d7' => 'r1bqk2r/pppnppbp/3p1np1/8/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQkq - 2 6',
				'c7c5' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQkq c6 0 6',
				'e7e5' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQkq e6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQ - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'E70f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nge2

			variation => N__('King\'s Indian: Kramer'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2NPPP/R1BQKB1R w KQ - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'E70e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6

			variation => N__('King\'s Indian: 4. e4 d6'),
			moves => {
				'c1g5' => 'rnbqk2r/ppp1ppbp/3p1np1/6B1/2PPP3/2N5/PP3PPP/R2QKBNR b KQkq - 1 5',
				'f1d3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2NB4/PP3PPP/R1BQK1NR b KQkq - 1 5',
				'f1e2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2BPPP/R1BQK1NR b KQkq - 1 5',
				'f2f3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 5',
				'f2f4' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPPP2/2N5/PP4PP/R1BQKBNR b KQkq f3 0 5',
				'g1e2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP2NPPP/R1BQKB1R b KQkq - 1 5',
				'g1f3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5',
				'g2g3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N3P1/PP3P1P/R1BQKBNR b KQkq - 0 5',
				'h2h3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N4P/PP3PP1/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2NB4/PP3PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'E70j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3'),
			moves => {
				'e7e5' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2NB4/PP3PPP/R1BQK1NR w KQkq e6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPP3/2NB4/PP3PPP/R1BQK1NR w KQ - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/2PPPP2/2N5/PP4PP/R1BQKBNR b KQkq f3 0 5' => {
			eco => 'E76a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f4

			variation => N__('King\'s Indian: Four Pawns Attack'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/2PPPP2/2N5/PP4PP/R1BQKBNR w KQkq c6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/2PPPP2/2N5/PP4PP/R1BQKBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP1P1/2N5/PPP1BP1P/R1BQK1NR b KQkq g3 0 5' => {
			eco => 'B07y',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be2 Bg7 5. g4

			variation => N__('Pirc: Chinese Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP2P/2N5/PPP1BPP1/R1BQK1NR b KQkq h3 0 5' => {
			eco => 'B07z',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be2 Bg7 5. h4

			variation => N__('Pirc: Bayonet/Mariotti Attack'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 2 5' => {
			eco => 'B07u',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be3 Bg7

			variation => N__('Pirc: 4. Be3 Bg7'),
			moves => {
				'd1d2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N1B3/PPPQ1PPP/R3KBNR b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N1B3/PPPQ1PPP/R3KBNR b KQkq - 3 5' => {
			eco => 'B07u',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be3 Bg7 5. Qd2

			variation => N__('Pirc: 4. Be3, 150 Attack'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N1B3/PPPQ1PPP/R3KBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N1BN2/PPP2PPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'B08h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be3

			variation => N__('Pirc: Classical, 5. Be3'),
			moves => {
				'a7a6' => 'rnbqk2r/1pp1ppbp/p2p1np1/8/3PP3/2N1BN2/PPP2PPP/R2QKB1R w KQkq - 0 6',
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N1BN2/PPP2PPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B08b',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. h3

			variation => N__('Pirc: Classical, 5. h3'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N2N1P/PPP2PP1/R1BQKB1R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N2N1P/PPP2PP1/R1BQKB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 3 5' => {
			eco => 'B08j',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Be2

			variation => N__('Pirc: Classical, 5. Be2'),
			moves => {
				'a7a6' => 'rnbqk2r/1pp1ppbp/p2p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R w KQkq - 0 6',
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B08a',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7

			variation => N__('Pirc: Classical'),
			moves => {
				'a2a4' => 'rnbqk2r/ppp1ppbp/3p1np1/8/P2PP3/2N2N2/1PP2PPP/R1BQKB1R b KQkq a3 0 5',
				'c1e3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N1BN2/PPP2PPP/R2QKB1R b KQkq - 3 5',
				'f1c4' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2BPP3/2N2N2/PPP2PPP/R1BQK2R b KQkq - 3 5',
				'f1e2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 3 5',
				'h2h3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N2N1P/PPP2PP1/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N3P1/PPP2P1P/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B07r',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. g3 Bg7

			variation => N__('Pirc: Sveshnikov (4. g3)'),
			moves => {
				'f1g2' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N3P1/PPP2PBP/R1BQK1NR b KQkq - 2 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N3P1/PPP2PBP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'B07r',
                        # TRANSLATORS:

			variation => N__('Pirc: Sveshnikov (4. g3)'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/3PP3/2N3P1/PPP2PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N5/PPP1BPPP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'B07y',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be2 Bg7

			variation => N__('Pirc: 4. Be2'),
			moves => {
				'g2g4' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP1P1/2N5/PPP1BP1P/R1BQK1NR b KQkq g3 0 5',
				'h2h4' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP2P/2N5/PPP1BPP1/R1BQK1NR b KQkq h3 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B07q',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nge2 Bg7

			variation => N__('Pirc: 4.Nge2'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2P5/PP1N1PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nd2 g6 4. c3 Bg7

			variation => N__('Pirc: 3.Nbd2 g6'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2PB1N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'B07j',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 g6 4. Nf3 Bg7 5. c3

			variation => N__('Pirc: 3. Bd3 g6 4. Nf3 Bg7 5. c3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 5' => {
			eco => 'B07i',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 g6 4. Nf3 Bg7

			variation => N__('Pirc: 3. Bd3 g6'),
			moves => {
				'c2c3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2PB1N2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PPP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B09d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7 5. Nf3

			variation => N__('Pirc: Austrian, 5. Nf3'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/3PPP2/2N2N2/PPP3PP/R1BQKB1R w KQkq c6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/3p1np1/8/3PPP2/2N2N2/PPP3PP/R1BQKB1R w KQ - 3 6',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B09b',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4 Bg7

			variation => N__('Pirc: Austrian Attack'),
			moves => {
				'f1c4' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2BPPP2/2N5/PPP3PP/R1BQK1NR b KQkq - 2 5',
				'g1f3' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PPP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'B07c',
                        # TRANSLATORS: 1. e4 d6 2. f4 Nf6 3. Nc3 g6 4. Nf3 Bg7

			variation => N__('Pirc: 2. f4 Nf6 3. Nc3 g6 4. Nf3 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/5P2/5NP1/PPPPP1BP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2 d6

			variation => N__('Bird: 1..Nf6 2. Nf3 g6 3. g3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/3p1np1/8/P2PP3/2N2N2/1PP2PPP/R1BQKB1R b KQkq a3 0 5' => {
			eco => 'B08f',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. a4

			variation => N__('Pirc: Classical, 5. a4'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2pp1np1/8/P2PP3/2N2N2/1PP2PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3P4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. cxd5

			variation => N__('Grünfeld: Three Knights Exchange'),
			moves => {
				'f6d5' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3P4/3P4/6P1/PP2PPBP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: 4. Bg2 Bg7'),
			moves => {
				'f6d5' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'D91a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5

			variation => N__('Grünfeld: 5. Bg5'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2p2np1/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
				'd5c4' => 'rnbqk2r/ppp1ppbp/5np1/6B1/2pP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
				'f6e4' => 'rnbqk2r/ppp1ppbp/6p1/3p2B1/2PPn3/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/2P2N2/PP2PPPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'D03k',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. c3 Bg7

			variation => N__('Torre Attack: 3... g6'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5' => {
			eco => 'D03m',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. e3 Bg7

			variation => N__('Torre Attack: 3... g6 4. e3'),
			moves => {
				'b1d2' => 'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 2 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 0 5' => {
			eco => 'D03l',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 2 5' => {
			eco => 'D03n',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. e3 Bg7 5. Nbd2

			variation => N__('Torre Attack: 3... g6 4. e3 Bg7 5.Nbd2'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQ - 3 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/5N2/PPPNPPPP/R2QKB1R w KQkq - 2 5' => {
			eco => 'D03l',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2 Bg7

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP1B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7 5. e3

			variation => N__('Grünfeld: 4. Bf4 Bg7 5. e3'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/5np1/2pp4/2PP1B2/2N1P3/PP3PPP/R2QKBNR w KQkq c6 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N1P3/PP3PPP/R2QKBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'D92',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bf4

			variation => N__('Grünfeld: 5. Bf4'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR w KQkq - 2 5' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4 Bg7

			variation => N__('Grünfeld: 4. Bf4 Bg7'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP1B2/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/1QN1P3/PP3PPP/R1B1KBNR b KQkq - 2 5' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. e3 Bg7 5. Qb3

			variation => N__('Grünfeld: 4. e3 Bg7 5. Qb3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 3 5' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3

			variation => N__('Grünfeld: Russian Variation'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6',
				'd5c4' => 'rnbqk2r/ppp1ppbp/5np1/8/2pP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. e3 Bg7

			variation => N__('Grünfeld: 4. e3 Bg7'),
			moves => {
				'd1b3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/1QN1P3/PP3PPP/R1B1KBNR b KQkq - 2 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D94',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. e3

			variation => N__('Grünfeld: 5. e3'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7

			variation => N__('Grünfeld: Three Knights Variation'),
			moves => {
				'c1f4' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5',
				'c1g5' => 'rnbqk2r/ppp1ppbp/5np1/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5',
				'c4d5' => 'rnbqk2r/ppp1ppbp/5np1/3P4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
				'd1a4' => 'rnbqk2r/ppp1ppbp/5np1/3p4/Q1PP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 3 5',
				'd1b3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 3 5',
				'e2e3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'g2g3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. g3

			variation => N__('Grünfeld: Three Knights, 5. g3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3

			variation => N__('Neo-Grünfeld, 5. Nf3'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/5np1/2pp4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq c6 0 6',
				'c7c6' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
				'd5c4' => 'rnbqk2r/ppp1ppbp/5np1/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7

			variation => N__('Neo-Grünfeld: 4. Bg2 Bg7'),
			moves => {
				'c4d5' => 'rnbqk2r/ppp1ppbp/5np1/3P4/3P4/6P1/PP2PPBP/RNBQK1NR b KQkq - 0 5',
				'g1f3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/3P1B2/2N1PN2/PPP2PPP/R2QKB1R b KQkq - 0 5' => {
			eco => 'D00q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Anti-King\'s Indian'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/3P1B2/2N1PN2/PPP2PPP/R2QKB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/3P1B2/2N2N2/PPP1PPPP/R2QKB1R w KQkq - 2 5' => {
			eco => 'D00q',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: Anti-King\'s Indian'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/3P1B2/2N1PN2/PPP2PPP/R2QKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'D02u',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: 3. g3 g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQ1RK1 w - - 4 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'D02u',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7

			variation => N__('Queen\'s Pawn: 3. g3 g6'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp1ppbp/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/5P2/3P1NP1/PPP1P1BP/RNBQK2R b KQkq - 0 5' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. d3

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6: 5. d3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7 5. O-O

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6: 5.O-O'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQ1RK1 w - - 4 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6 4. Bg2 Bg7

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6'),
			moves => {
				'd2d3' => 'rnbqk2r/ppp1ppbp/5np1/3p4/5P2/3P1NP1/PPP1P1BP/RNBQK2R b KQkq - 0 5',
				'e1g1' => 'rnbqk2r/ppp1ppbp/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/3p4/Q1PP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 3 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qa4+

			variation => N__('Grünfeld: Flohr Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/6B1/2pP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D91b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 dxc4

			variation => N__('Grünfeld: 5. Bg5 dxc4'),
			moves => {
				'e2e4' => 'rnbqk2r/ppp1ppbp/5np1/6B1/2pPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/6B1/2pPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6' => {
			eco => 'D91c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 dxc4 6. e4

			variation => N__('Grünfeld: 5. Bg5 dxc4 6. e4'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'D25d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 g6 5. Bxc4 Bg7 6. Nc3

			variation => N__('QGA: Smyslov, 5.Bxc4 Bg7 6. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2BP4/4PN2/PP3PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'D25e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 g6 5. Bxc4 Bg7 6. O-O

			variation => N__('QGA: Smyslov, 5.Bxc4 Bg7 6.O-O'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/8/2BP4/4PN2/PP3PPP/RNBQ1RK1 w - - 3 7',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'D25d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 g6 5. Bxc4 Bg7

			variation => N__('QGA: Smyslov, 5.Bxc4 Bg7'),
			moves => {
				'b1c3' => 'rnbqk2r/ppp1ppbp/5np1/8/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 2 6',
				'e1g1' => 'rnbqk2r/ppp1ppbp/5np1/8/2BP4/4PN2/PP3PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4 6. Qxc4

			variation => N__('Grünfeld: Russian, 5...dxc4'),
			moves => {
				'a7a6' => 'rnbqk2r/1pp1ppbp/p4np1/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R w KQ - 1 7',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2QP4/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 6' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4 Bg7

			variation => N__('Grünfeld: Early Russian, 5... Bg7'),
			moves => {
				'e2e4' => 'rnbqk2r/ppp1ppbp/5np1/8/2QPP3/2N5/PP3PPP/R1B1KBNR b KQkq e3 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2QPP3/2N5/PP3PPP/R1B1KBNR b KQkq e3 0 6' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4 Bg7 6. e4

			variation => N__('Grünfeld: Early Russian, 5... Bg7 6. e4'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/5np1/8/2QPP3/2N5/PP3PPP/R1B1KBNR w KQ - 1 7',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2pP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6' => {
			eco => 'D96',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Qb3 dxc4

			variation => N__('Grünfeld: Russian, 5...dxc4'),
			moves => {
				'b3c4' => 'rnbqk2r/ppp1ppbp/5np1/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 dxc4

			variation => N__('Neo-Grünfeld, 5. Nf3 dxc4'),
			moves => {
				'b1a3' => 'rnbqk2r/ppp1ppbp/5np1/8/2pP4/N4NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/5np1/8/2pP4/N4NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D73',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. Nf3 dxc4 6. Na3

			variation => N__('Neo-Grünfeld, 5. Nf3 dxc4 6. Na3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/1B6/3PP3/2P5/P4PPP/R1BQK1NR b KQkq - 2 7' => {
			eco => 'D85e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bb5+

			variation => N__('Grünfeld: Exchange, 7. Bb5+'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2p3p1/1B6/3PP3/2P5/P4PPP/R1BQK1NR w KQkq - 0 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3P2B1/3Pn3/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D91h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5'),
			moves => {
				'e4g5' => 'rnbqk2r/ppp1ppbp/6p1/3P2n1/3P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3P2N1/3P4/2N5/PP2PPPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D91i',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5'),
			moves => {
				'c7c6' => 'rnbqk2r/pp2ppbp/2p3p1/3P2N1/3P4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8',
				'e7e6' => 'rnbqk2r/ppp2pbp/4p1p1/3P2N1/3P4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3P2n1/3P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D91i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5'),
			moves => {
				'f3g5' => 'rnbqk2r/ppp1ppbp/6p1/3P2N1/3P4/2N5/PP2PPPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3P4/1n2P3/6P1/PP3PBP/RNBQK1NR b KQkq - 0 7' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb4 7. d5

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb4 7. d5'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 6' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. cxd5 Nxd5 6. Qb3

			variation => N__('Grünfeld: Three Knights Exchange, Romanishin Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP1BPPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. cxd5 Nxd5 6. Bd2

			variation => N__('Grünfeld: Three Knights Exchange, 6. Bd2'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP1BPPPP/R2QKB1R w KQ - 2 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. cxd5 Nxd5

			variation => N__('Grünfeld: Three Knights Exchange'),
			moves => {
				'c1d2' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N2N2/PP1BPPPP/R2QKB1R b KQkq - 1 6',
				'd1b3' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 1 6' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. Nc3

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. Nc3'),
			moves => {
				'd5b6' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3P4/2N3P1/PP2PPBP/R1BQK1NR w KQkq - 2 7',
				'd5c3' => 'rnbqk2r/ppp1ppbp/6p1/8/3P4/2n3P1/PP2PPBP/R1BQK1NR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N5/PP1BPPPP/R2QKBNR w KQkq - 2 6' => {
			eco => 'D85b',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Exchange, 5. Bd2'),
			moves => {
				'e2e4' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3PP3/2N5/PP1B1PPP/R2QKBNR b KQkq e3 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/5N2/PPP1BPPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B01x',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 g6 5. Be2 Bg7

			variation => N__('Scandinavian: Marshall, 4. Nf3 g6 5. Be2 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 6' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5'),
			moves => {
				'b1c3' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 1 6',
				'e2e4' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3PP3/6P1/PP3PBP/RNBQK1NR b KQkq e3 0 6',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3PP3/2N5/PP1B1PPP/R2QKBNR b KQkq e3 0 6' => {
			eco => 'D85b',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Exchange, 5. Bd2'),
			moves => {
				'd5b6' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N5/PP1B1PPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/3PP3/6P1/PP3PBP/RNBQK1NR b KQkq e3 0 6' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4'),
			moves => {
				'd5b4' => 'rnbqk2r/ppp1ppbp/6p1/8/1n1PP3/6P1/PP3PBP/RNBQK1NR w KQkq - 1 7',
				'd5b6' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/6P1/PP3PBP/RNBQK1NR w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/8/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 6' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6 5. g3 Bg7

			variation => N__('English: Anglo-Grünfeld, 4. Nf3 g6 5. g3'),
			moves => {
				'f1g2' => 'rnbqk2r/ppp1ppbp/6p1/3n4/8/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 6',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3n4/8/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 2 6' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, 4. Nf3 g6 5. g3'),
			moves => {
				'e7e5' => 'rnbqk2r/ppp2pbp/6p1/3np3/8/2N2NP1/PP1PPPBP/R1BQK2R w KQkq e6 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3p2B1/2PPn3/2N2N2/PP2PPPP/R2QKB1R w KQkq - 4 6' => {
			eco => 'D91d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4

			variation => N__('Grünfeld: 5. Bg5 Ne4'),
			moves => {
				'c4d5' => 'rnbqk2r/ppp1ppbp/6p1/3P2B1/3Pn3/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6',
				'g5f4' => 'rnbqk2r/ppp1ppbp/6p1/3p4/2PPnB2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 6',
				'g5h4' => 'rnbqk2r/ppp1ppbp/6p1/3p4/2PPn2B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 6',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3p4/2PP3B/2P2N2/P3PPPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D91f',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6. Bh4 Nxc3'),
			moves => {
				'd5c4' => 'rnbqk2r/ppp1ppbp/6p1/8/2pP3B/2P2N2/P3PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3p4/2PP3B/2P5/P3PPPP/R2QKBNR w KQkq - 1 7' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4 5. Bh4 Nxc3 6. bxc3 Bg7

			variation => N__('Grünfeld: Stockholm, Taimanov, 5...Nxc3 6.bxc3 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3p4/2PP3B/2n2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D91f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. Bh4 Nxc3

			variation => N__('Grünfeld: 5. Bg5 Ne4 6. Bh4 Nxc3'),
			moves => {
				'b2c3' => 'rnbqk2r/ppp1ppbp/6p1/3p4/2PP3B/2P2N2/P3PPPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3p4/2PPn2B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 6' => {
			eco => 'D91e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. Bh4

			variation => N__('Grünfeld: 5. Bg5 Ne4 6. Bh4'),
			moves => {
				'e4c3' => 'rnbqk2r/ppp1ppbp/6p1/3p4/2PP3B/2n2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/3p4/2PPnB2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 6' => {
			eco => 'D91d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. Bf4

			variation => N__('Grünfeld: 5. Bg5 Ne4 6. Bf4'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/1n1PP3/6P1/PP3PBP/RNBQK1NR w KQkq - 1 7' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb4

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb4'),
			moves => {
				'd4d5' => 'rnbqk2r/ppp1ppbp/6p1/3P4/1n2P3/6P1/PP3PBP/RNBQK1NR b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/2BPP3/2P5/P4PPP/R1BQK1NR b KQkq - 2 7' => {
			eco => 'D86',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Bc4

			variation => N__('Grünfeld: Classical Exchange'),
			moves => {
				'b7b6' => 'rnbqk2r/p1p1ppbp/1p4p1/8/2BPP3/2P5/P4PPP/R1BQK1NR w KQkq - 0 8',
				'c7c5' => 'rnbqk2r/pp2ppbp/6p1/2p5/2BPP3/2P5/P4PPP/R1BQK1NR w KQkq c6 0 8',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/6p1/8/2BPP3/2P5/P4PPP/R1BQK1NR w KQ - 3 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/2pP3B/2P2N2/P3PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D91g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. Bh4 Nxc3 7. bxc3 dxc4

			variation => N__('Grünfeld: 5. Bg5 Ne4 6. Bh4 Nxc3 7.bxc3 dxc4'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/3P4/2P3P1/P3PPBP/R1BQK1NR b KQkq - 0 7' => {
			eco => 'D71',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. Nc3 Nxc3'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/6p1/2p5/3P4/2P3P1/P3PPBP/R1BQK1NR w KQkq c6 0 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/3P4/2n3P1/PP2PPBP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'D71',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. Nc3 Nxc3

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. Nc3 Nxc3'),
			moves => {
				'b2c3' => 'rnbqk2r/ppp1ppbp/6p1/8/3P4/2P3P1/P3PPBP/R1BQK1NR b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/3PP3/2P1B3/P4PPP/R2QKBNR b KQkq - 2 7' => {
			eco => 'D85g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Be3

			variation => N__('Grünfeld: Exchange, 7. Be3'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P1B3/P4PPP/R2QKBNR w KQkq c6 0 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/3PP3/2P2N2/P4PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'D85i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Nf3

			variation => N__('Grünfeld: Modern Exchange'),
			moves => {
				'c7c5' => 'rnbqk2r/pp2ppbp/6p1/2p5/3PP3/2P2N2/P4PPP/R1BQKB1R w KQkq c6 0 8',
				'e8g8' => 'rnbq1rk1/ppp1ppbp/6p1/8/3PP3/2P2N2/P4PPP/R1BQKB1R w KQ - 3 8',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/3PP3/2P5/P4PPP/R1BQKBNR w KQkq - 1 7' => {
			eco => 'D85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7

			variation => N__('Grünfeld: Exchange, 5. e4 Nxc3 6.bxc3 Bg7'),
			moves => {
				'c1a3' => 'rnbqk2r/ppp1ppbp/6p1/8/3PP3/B1P5/P4PPP/R2QKBNR b KQkq - 2 7',
				'c1e3' => 'rnbqk2r/ppp1ppbp/6p1/8/3PP3/2P1B3/P4PPP/R2QKBNR b KQkq - 2 7',
				'f1b5' => 'rnbqk2r/ppp1ppbp/6p1/1B6/3PP3/2P5/P4PPP/R1BQK1NR b KQkq - 2 7',
				'f1c4' => 'rnbqk2r/ppp1ppbp/6p1/8/2BPP3/2P5/P4PPP/R1BQK1NR b KQkq - 2 7',
				'g1f3' => 'rnbqk2r/ppp1ppbp/6p1/8/3PP3/2P2N2/P4PPP/R1BQKB1R b KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/3PP3/B1P5/P4PPP/R2QKBNR b KQkq - 2 7' => {
			eco => 'D85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 Bg7 7. Ba3

			variation => N__('Grünfeld: Exchange, 7. Ba3'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/8/2P3P1/P2PPPBP/1RBQK1NR b Kkq - 2 7' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. g3 g6 5. Bg2 Nxc3 6. bxc3 Bg7 7. Rb1

			variation => N__('English: Anglo-Grünfeld, Smyslov, 7. Rb1'),
			moves => {
			},
		},
		'rnbqk2r/ppp1ppbp/6p1/8/8/2P3P1/P2PPPBP/R1BQK1NR w KQkq - 1 7' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. g3 g6 5. Bg2 Nxc3 6. bxc3 Bg7

			variation => N__('English: Anglo-Grünfeld, Smyslov Variation'),
			moves => {
				'a1b1' => 'rnbqk2r/ppp1ppbp/6p1/8/8/2P3P1/P2PPPBP/1RBQK1NR b Kkq - 2 7',
			},
		},
		'rnbqk2r/ppp2p1p/3b1n2/3PN3/2B2ppP/8/PPPP2P1/RNBQ1RK1 b kq - 2 8' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Nf6 6. Bc4 d5 7. exd5 Bd6 8. O-O

			variation => N__('KGA: Kieseritsky, Rice Gambit'),
			moves => {
			},
		},
		'rnbqk2r/ppp2p1p/3b1n2/3PN3/2B2ppP/8/PPPP2P1/RNBQK2R w KQkq - 1 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Berlin Defence, 6. Bc4'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp2p1p/3b1n2/3PN3/2B2ppP/8/PPPP2P1/RNBQ1RK1 b kq - 2 8',
			},
		},
		'rnbqk2r/ppp2p2/4p1pp/3pP3/3P2Q1/2PB4/P1PK1PPP/R5NR b kq - 0 10' => {
			eco => 'C12o',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, 8. Qg4 g6 9. Bd3'),
			moves => {
				'c7c5' => 'rnbqk2r/pp3p2/4p1pp/2ppP3/3P2Q1/2PB4/P1PK1PPP/R5NR w kq c6 0 11',
			},
		},
		'rnbqk2r/ppp2p2/4p1pp/3pP3/3P2Q1/2PB4/P1Pn1PPP/R3K1NR w KQkq - 0 10' => {
			eco => 'C12o',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, 8. Qg4 g6 9. Bd3'),
			moves => {
				'e1d2' => 'rnbqk2r/ppp2p2/4p1pp/3pP3/3P2Q1/2PB4/P1PK1PPP/R5NR b kq - 0 10',
			},
		},
		'rnbqk2r/ppp2p2/4p1pp/3pP3/3Pn1Q1/2P5/P1PB1PPP/R3KBNR w KQkq - 0 9' => {
			eco => 'C12n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 g6

			variation => N__('French: MacCutcheon, 8. Qg4 g6'),
			moves => {
				'f1d3' => 'rnbqk2r/ppp2p2/4p1pp/3pP3/3Pn1Q1/2PB4/P1PB1PPP/R3K1NR b KQkq - 1 9',
			},
		},
		'rnbqk2r/ppp2p2/4p1pp/3pP3/3Pn1Q1/2PB4/P1PB1PPP/R3K1NR b KQkq - 1 9' => {
			eco => 'C12o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4 g6 9. Bd3

			variation => N__('French: MacCutcheon, 8. Qg4 g6 9. Bd3'),
			moves => {
				'e4d2' => 'rnbqk2r/ppp2p2/4p1pp/3pP3/3P2Q1/2PB4/P1Pn1PPP/R3K1NR w KQkq - 0 10',
			},
		},
		'rnbqk2r/ppp2pP1/4p3/3p2p1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'C12e',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, Chigorin Variation'),
			moves => {
				'h8g8' => 'rnbqk1r1/ppp2pP1/4p3/3p2p1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQq - 1 8',
			},
		},
		'rnbqk2r/ppp2pbp/1n4p1/4p3/3PP3/6P1/PP2NPBP/RNBQK2R w KQkq e6 0 8' => {
			eco => 'D72',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 Bg7 5. cxd5 Nxd5 6. e4 Nb6 7. Ne2 e5

			variation => N__('Neo-Grünfeld, 5.cxd5 Nxd5 6. e4 Nb6 7. Ne2 e5'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/3Pp3/2P1P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E80g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 e5 6. d5

			variation => N__('King\'s Indian: Sämisch, 5... e5 6. d5'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'E73d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 e5 6. d5

			variation => N__('King\'s Indian: 5. Be2 e5 6. d5'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P1P3/2N2NP1/PP1P1PBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e4 d6 6. Nf3

			variation => N__('English: Bremen, 3... g6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e4 d6

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'g1e2' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 1 6',
				'g1f3' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P1P3/2N2NP1/PP1P1PBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P1P3/2N3P1/PP1PNPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P1P3/2N3P1/PP1PNPBP/R1BQK2R w KQ - 2 7',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'g1e2' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2N1P1P1/PP1PNPBP/R1BQK2R w KQ - 2 7',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 w - - 2 7',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'd2d3' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6',
				'e1g1' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQ1RK1 b kq - 1 6',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R b KQkq - 0 6' => {
			eco => 'A24',
                        # TRANSLATORS:

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2pbp/3p1np1/4p3/2P5/2NP1NP1/PP2PPBP/R1BQK2R w KQ - 1 7',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2N2P2/PP2N1PP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'E80f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 e5 6. Nge2

			variation => N__('King\'s Indian: Sämisch, 5... e5 6.Nge2'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2N2P2/PP4PP/R1BQKBNR w KQkq e6 0 6' => {
			eco => 'E80e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. f3 e5

			variation => N__('King\'s Indian: Sämisch, 5... e5'),
			moves => {
				'd4d5' => 'rnbqk2r/ppp2pbp/3p1np1/3Pp3/2P1P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 6',
				'g1e2' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2N2P2/PP2N1PP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2N5/PP2BPPP/R1BQK1NR w KQkq e6 0 6' => {
			eco => 'E73c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 e5

			variation => N__('King\'s Indian: 5. Be2 e5'),
			moves => {
				'd4d5' => 'rnbqk2r/ppp2pbp/3p1np1/3Pp3/2P1P3/2N5/PP2BPPP/R1BQK1NR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2pbp/3p1np1/4p3/2PPP3/2NB4/PP3PPP/R1BQK1NR w KQkq e6 0 6' => {
			eco => 'E70k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Bd3 e5

			variation => N__('King\'s Indian: 4. e4 d6 5. Bd3 e5'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/4p1p1/3P2N1/3P4/2N5/PP1QPPPP/R3KB1R b KQkq - 1 8' => {
			eco => 'D91l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Qd2

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2'),
			moves => {
				'e6d5' => 'rnbqk2r/ppp2pbp/6p1/3p2N1/3P4/2N5/PP1QPPPP/R3KB1R w KQkq - 0 9',
			},
		},
		'rnbqk2r/ppp2pbp/4p1p1/3P2N1/3P4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D91k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6'),
			moves => {
				'd1d2' => 'rnbqk2r/ppp2pbp/4p1p1/3P2N1/3P4/2N5/PP1QPPPP/R3KB1R b KQkq - 1 8',
				'g5f3' => 'rnbqk2r/ppp2pbp/4p1p1/3P4/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqk2r/ppp2pbp/4p1p1/3P4/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'D91o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3'),
			moves => {
				'e6d5' => 'rnbqk2r/ppp2pbp/6p1/3p4/3P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 9',
			},
		},
		'rnbqk2r/ppp2pbp/6p1/3np3/8/2N2NP1/PP1PPPBP/R1BQK2R w KQkq e6 0 7' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6 5. g3 Bg7 6. Bg2 e5

			variation => N__('English: Anglo-Grünfeld, Korchnoi Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pbp/6p1/3p2N1/3P4/2N1Q3/PP2PPPP/R3KB1R b KQkq - 1 9' => {
			eco => 'D91m',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5'),
			moves => {
				'e8f8' => 'rnbq1k1r/ppp2pbp/6p1/3p2N1/3P4/2N1Q3/PP2PPPP/R3KB1R w KQ - 2 10',
			},
		},
		'rnbqk2r/ppp2pbp/6p1/3p2N1/3P4/2N5/PP1QPPPP/R3KB1R w KQkq - 0 9' => {
			eco => 'D91m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Qd2 exd5

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Qd2 exd5'),
			moves => {
				'd2e3' => 'rnbqk2r/ppp2pbp/6p1/3p2N1/3P4/2N1Q3/PP2PPPP/R3KB1R b KQkq - 1 9',
			},
		},
		'rnbqk2r/ppp2pbp/6p1/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 9' => {
			eco => 'D91p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 Bg7 5. Bg5 Ne4 6. cxd5 Nxg5 7. Nxg5 e6 8. Nf3 exd5 9. e3

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3 exd5 9. e3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2pbp/6p1/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R w KQ - 1 10',
			},
		},
		'rnbqk2r/ppp2pbp/6p1/3p4/3P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 9' => {
			eco => 'D91o',
                        # TRANSLATORS:

			variation => N__('Grünfeld: 5. Bg5 Ne4 6.cxd5 Nxg5 7.Nxg5 e6 8. Nf3'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp2pbp/6p1/3p4/3P4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 9',
			},
		},
		'rnbqk2r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2P5/P1PB1PPP/R3KBNR b KQkq - 2 8' => {
			eco => 'C12l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3 Ne4 8. Qg4

			variation => N__('French: MacCutcheon, 8. Qg4'),
			moves => {
				'e8f8' => 'rnbq1k1r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2P5/P1PB1PPP/R3KBNR w KQ - 3 9',
				'g7g6' => 'rnbqk2r/ppp2p2/4p1pp/3pP3/3Pn1Q1/2P5/P1PB1PPP/R3KBNR w KQkq - 0 9',
			},
		},
		'rnbqk2r/ppp2pp1/4p2p/3pP3/3Pn3/2P5/P1PB1PPP/R2QKBNR w KQkq - 1 8' => {
			eco => 'C12k',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, Lasker, 7.bxc3'),
			moves => {
				'd1g4' => 'rnbqk2r/ppp2pp1/4p2p/3pP3/3Pn1Q1/2P5/P1PB1PPP/R3KBNR b KQkq - 2 8',
			},
		},
		'rnbqk2r/ppp2pp1/4pB1p/3p4/1bPP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D38l',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, 5. Bg5 h6'),
			moves => {
				'd8f6' => 'rnb1k2r/ppp2pp1/4pq1p/3p4/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2pp1/4pP1p/3p2B1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C12e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. exf6

			variation => N__('French: MacCutcheon, Chigorin Variation'),
			moves => {
				'h6g5' => 'rnbqk2r/ppp2pp1/4pP2/3p2p1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2pp1/4pP2/3p2p1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C12e',
                        # TRANSLATORS:

			variation => N__('French: MacCutcheon, Chigorin Variation'),
			moves => {
				'f6g7' => 'rnbqk2r/ppp2pP1/4p3/3p2p1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2pp1/4pb1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D53g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bxf6 Bxf6 7. e3

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6.Bxf6 Bxf6 7. e3'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pp1/4pb1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D53f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. Nf3 h6 6. Bxf6 Bxf6

			variation => N__('QGD: 4. Bg5 Be7 5. Nf3 h6 6.Bxf6 Bxf6'),
			moves => {
				'e2e3' => 'rnbqk2r/ppp2pp1/4pb1p/3p4/2PP4/2N1PN2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3p2B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D38l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 h6

			variation => N__('QGD: Ragozin, 5. Bg5 h6'),
			moves => {
				'g5f6' => 'rnbqk2r/ppp2pp1/4pB1p/3p4/1bPP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP1B1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'C12d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6

			variation => N__('French: MacCutcheon, Advance Variation'),
			moves => {
				'e5f6' => 'rnbqk2r/ppp2pp1/4pP1p/3p2B1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6',
				'g5c1' => 'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 6',
				'g5d2' => 'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 1 6',
				'g5e3' => 'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P4/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 6',
				'g5h4' => 'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P3B/2N5/PPP2PPP/R2QKBNR b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P3B/2N5/PPP2PPP/R2QKBNR b KQkq - 1 6' => {
			eco => 'C12g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bh4

			variation => N__('French: MacCutcheon, Bernstein Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P4/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 6' => {
			eco => 'C12h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Be3

			variation => N__('French: MacCutcheon, Janowski Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 6' => {
			eco => 'C12f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bc1

			variation => N__('French: MacCutcheon, Olland (Dutch) Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR b KQkq - 1 6' => {
			eco => 'C12i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2

			variation => N__('French: MacCutcheon, 6. Bd2'),
			moves => {
				'b4c3' => 'rnbqk2r/ppp2pp1/4pn1p/3pP3/3P4/2b5/PPPB1PPP/R2QKBNR w KQkq - 0 7',
				'f6d7' => 'rnbqk2r/pppn1pp1/4p2p/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP3/3P4/2P5/P1PB1PPP/R2QKBNR b KQkq - 0 7' => {
			eco => 'C12k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3 7. bxc3

			variation => N__('French: MacCutcheon, Lasker, 7.bxc3'),
			moves => {
				'f6e4' => 'rnbqk2r/ppp2pp1/4p2p/3pP3/3Pn3/2P5/P1PB1PPP/R2QKBNR w KQkq - 1 8',
			},
		},
		'rnbqk2r/ppp2pp1/4pn1p/3pP3/3P4/2b5/PPPB1PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C12j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Bxc3

			variation => N__('French: MacCutcheon, Lasker Variation'),
			moves => {
				'b2c3' => 'rnbqk2r/ppp2pp1/4pn1p/3pP3/3P4/2P5/P1PB1PPP/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2pp1/5B1p/3p4/1b1P4/2N5/PPQ1PPPP/R3KBNR b KQkq - 0 7' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. Bg5 h6 7. Bxf6

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. Bg5 h6 7.Bxf6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pp1/5n1p/3p2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D38g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. cxd5 exd5 6. Bg5 h6

			variation => N__('QGD: Ragozin, 5.cxd5 exd5 6. Bg5 h6'),
			moves => {
				'g5h4' => 'rnbqk2r/ppp2pp1/5n1p/3p4/1b1P3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2pp1/5n1p/3p2B1/1b1P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 0 7' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. Bg5 h6

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. Bg5 h6'),
			moves => {
				'g5f6' => 'rnbqk2r/ppp2pp1/5B1p/3p4/1b1P4/2N5/PPQ1PPPP/R3KBNR b KQkq - 0 7',
				'g5h4' => 'rnbqk2r/ppp2pp1/5n1p/3p4/1b1P3B/2N5/PPQ1PPPP/R3KBNR b KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2pp1/5n1p/3p4/1b1P3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'D38g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. cxd5 exd5 6. Bg5 h6 7. Bh4

			variation => N__('QGD: Ragozin, 5.cxd5 exd5 6. Bg5 h6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2pp1/5n1p/3p4/1b1P3B/2N5/PPQ1PPPP/R3KBNR b KQkq - 1 7' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. Bg5 h6 7. Bh4

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. Bg5 h6 7. Bh4'),
			moves => {
				'c7c5' => 'rnbqk2r/pp3pp1/5n1p/2pp4/1b1P3B/2N5/PPQ1PPPP/R3KBNR w KQkq c6 0 8',
			},
		},
		'rnbqk2r/ppp2ppp/1b1p1n2/4p3/3PPP2/2P2N2/PP4PP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'C30q',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 Nf6 5. d4 Bb6

			variation => N__('KGD: Classical, 4. c3 Nf6 5. d4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/1b1p1n2/8/3PPP2/5N2/PP4PP/RNBQKB1R w KQkq - 1 7' => {
			eco => 'C30q',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 Nf6 5. d4 exd4 6. cxd4 Bb6

			variation => N__('KGD: Classical, 4. c3 Nf6 5. d4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3b1n2/3p4/3P4/3B1N2/PPP2PPP/RNBQK2R w KQkq - 4 6' => {
			eco => 'C01p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Bd3 Bd6 5. Nf3 Nf6

			variation => N__('French: Exchange, 4. Nf3 Bd6 5. Bd3 Nf6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3b1p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'D00k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 exf6 4. e3 Bd6

			variation => N__('Trompowsky: 2... d5 3.Bxf6 exf6 4. e3 Bd6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3b3n/8/3P4/5N2/PPP1P1PP/RNBQKB1R b KQkq d3 0 5' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Bxd6 4. Nf3 Nh6 5. d4

			variation => N__('Bird: From Gambit, Lipke'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3b3n/8/8/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: From Gambit, 3...Bxd6'),
			moves => {
				'd2d4' => 'rnbqk2r/ppp2ppp/3b3n/8/3P4/5N2/PPP1P1PP/RNBQKB1R b KQkq d3 0 5',
			},
		},
		'rnbqk2r/ppp2ppp/3b4/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 b kq - 3 7' => {
			eco => 'C42q',
                        # TRANSLATORS:

			variation => N__('Russian Game: Classical, Marshall Variation'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2ppp/3b4/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 w - - 4 8',
			},
		},
		'rnbqk2r/ppp2ppp/3b4/3p4/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 7' => {
			eco => 'C42q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3 Bd6

			variation => N__('Russian Game: Classical, Marshall Variation'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp2ppp/3b4/3p4/3Pn3/3B1N2/PPP2PPP/RNBQ1RK1 b kq - 3 7',
			},
		},
		'rnbqk2r/ppp2ppp/3b4/3pN3/3Pn3/3B4/PPP2PPP/RNBQ1RK1 b kq - 2 6' => {
			eco => 'C43m',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Bd6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2ppp/3b4/3pN3/3Pn3/3B4/PPP2PPP/RNBQ1RK1 w - - 3 7',
			},
		},
		'rnbqk2r/ppp2ppp/3b4/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C43m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5 Bd6

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5 Bd6'),
			moves => {
				'e1g1' => 'rnbqk2r/ppp2ppp/3b4/3pN3/3Pn3/3B4/PPP2PPP/RNBQ1RK1 b kq - 2 6',
			},
		},
		'rnbqk2r/ppp2ppp/3b4/4p3/8/2P3P1/P1PP1PBP/R1BQK1NR w KQkq - 1 7' => {
			eco => 'C26k',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 d5 4. exd5 Nxd5 5. Bg2 Nxc3 6. bxc3 Bd6

			variation => N__('Vienna: Smyslov, 3... d5, 5...Nxc3 6.bxc3 Bd6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'C26s',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5 4. d3 d6

			variation => N__('Vienna: 3. Bc4 Bc5 4. d3 d6'),
			moves => {
				'c3a4' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/N1B1P3/3P4/PPP2PPP/R1BQK1NR b KQkq - 1 5',
				'f2f4' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/2B1PP2/2NP4/PPP3PP/R1BQK1NR b KQkq f3 0 5',
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/2B1PP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5' => {
			eco => 'C30m',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. Nc3 Nf6 5. Bc4

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3 Nf6 5. Bc4'),
			moves => {
				'b8c6' => 'r1bqk2r/ppp2ppp/2np1n2/2b1p3/2B1PP2/2N2N2/PPPP2PP/R1BQK2R w KQkq - 4 6',
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/2B1PP2/2NP4/PPP3PP/R1BQK1NR b KQkq f3 0 5' => {
			eco => 'C26u',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5 4. d3 d6 5. f4

			variation => N__('Vienna: 3. Bc4 Bc5 4. d3 d6 5. f4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/3PPP2/2P2N2/PP4PP/RNBQKB1R b KQkq d3 0 5' => {
			eco => 'C30q',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 Nf6 5. d4

			variation => N__('KGD: Classical, 4. c3 Nf6 5. d4'),
			moves => {
				'c5b6' => 'rnbqk2r/ppp2ppp/1b1p1n2/4p3/3PPP2/2P2N2/PP4PP/RNBQKB1R w KQkq - 1 6',
				'e5d4' => 'rnbqk2r/ppp2ppp/3p1n2/2b5/3pPP2/2P2N2/PP4PP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'C26f',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Bc5 4. Bg2 d6

			variation => N__('Vienna: Smyslov, 3... Bc5 4. Bg2 d6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'C30l',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 3. Nf3 d6 4. Nc3'),
			moves => {
				'f1c4' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/2B1PP2/2N2N2/PPPP2PP/R1BQK2R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/4PP2/2P2N2/PP1P2PP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C30p',
                        # TRANSLATORS: 1. e4 e5 2. f4 Bc5 3. Nf3 d6 4. c3 Nf6

			variation => N__('KGD: Classical, 4. c3 Nf6'),
			moves => {
				'd2d4' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/3PPP2/2P2N2/PP4PP/RNBQKB1R b KQkq d3 0 5',
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b1p3/N1B1P3/3P4/PPP2PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'C26t',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5 4. d3 d6 5. Na4

			variation => N__('Vienna: 3. Bc4 Bc5 4. d3 d6 5. Na4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b5/3PPP2/5N2/PP4PP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C30q',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 4. c3 Nf6 5. d4'),
			moves => {
				'c5b6' => 'rnbqk2r/ppp2ppp/1b1p1n2/8/3PPP2/5N2/PP4PP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2ppp/3p1n2/2b5/3pPP2/2P2N2/PP4PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C30q',
                        # TRANSLATORS:

			variation => N__('KGD: Classical, 4. c3 Nf6 5. d4'),
			moves => {
				'c3d4' => 'rnbqk2r/ppp2ppp/3p1n2/2b5/3PPP2/5N2/PP4PP/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/3ppn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 5' => {
			eco => 'E32c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d6

			variation => N__('Nimzo-Indian: Classical, 4... d6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4p3/3p4/2PPn3/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 1 7' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Ne4

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line'),
			moves => {
				'c3c2' => 'rnbqk2r/ppp2ppp/4p3/3p4/2PPn3/P7/1PQ1PPPP/R1B1KBNR b KQkq - 2 7',
			},
		},
		'rnbqk2r/ppp2ppp/4p3/3p4/2PPn3/P7/1PQ1PPPP/R1B1KBNR b KQkq - 2 7' => {
			eco => 'E37',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 Ne4 7. Qc2

			variation => N__('Nimzo-Indian: Classical, Noa, Main Line, 7. Qc2'),
			moves => {
				'b8c6' => 'r1bqk2r/ppp2ppp/2n1p3/3p4/2PPn3/P7/1PQ1PPPP/R1B1KBNR w KQkq - 3 8',
				'c7c5' => 'rnbqk2r/pp3ppp/4p3/2pp4/2PPn3/P7/1PQ1PPPP/R1B1KBNR w KQkq c6 0 8',
			},
		},
		'rnbqk2r/ppp2ppp/4pb2/3p4/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'C13b',
                        # TRANSLATORS:

			variation => N__('French: Classical, Anderssen Variation'),
			moves => {
				'e4e5' => 'rnbqk2r/ppp2ppp/4pb2/3pP3/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pb2/3pP3/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C13b',
                        # TRANSLATORS:

			variation => N__('French: Classical, Anderssen Variation'),
			moves => {
				'f6e7' => 'rnbqk2r/ppp1bppp/4p3/3pP3/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2ppp/4pb2/8/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C11t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4 Be7 6. Bxf6 Bxf6

			variation => N__('French: Burn, 6.Bxf6 Bxf6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3P2B1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'C12b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. exd5

			variation => N__('French: MacCutcheon, 5.exd5'),
			moves => {
				'd8d5' => 'rnb1k2r/ppp2ppp/4pn2/3q2B1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3P4/1b1P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D38e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. cxd5

			variation => N__('QGD: Ragozin, 5.cxd5'),
			moves => {
				'e6d5' => 'rnbqk2r/ppp2ppp/5n2/3p4/1b1P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3P4/1b1P4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5

			variation => N__('Nimzo-Indian: Classical, Noa, 5.cxd5'),
			moves => {
				'd8d5' => 'rnb1k2r/ppp2ppp/4pn2/3q4/1b1P4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 6',
				'e6d5' => 'rnbqk2r/ppp2ppp/5n2/3p4/1b1P4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p2B1/1b1PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 4 5' => {
			eco => 'C12a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4

			variation => N__('French: MacCutcheon'),
			moves => {
				'e4d5' => 'rnbqk2r/ppp2ppp/4pn2/3P2B1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5',
				'e4e5' => 'rnbqk2r/ppp2ppp/4pn2/3pP1B1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p2B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5' => {
			eco => 'D38h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5

			variation => N__('QGD: Ragozin, 5. Bg5'),
			moves => {
				'b8d7' => 'r1bqk2r/pppn1ppp/4pn2/3p2B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 6 6',
				'd5c4' => 'rnbqk2r/ppp2ppp/4pn2/6B1/1bpP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
				'h7h6' => 'rnbqk2r/ppp2pp1/4pn1p/3p2B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p2B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQkq - 4 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Bb4

			variation => N__('QGD: 4. Bg5 Bb4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p2B1/1bPP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 4 5' => {
			eco => 'D30v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 Bb4+

			variation => N__('QGD: Vienna Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1b1PP3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'C15o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Ne2 Nf6

			variation => N__('French: Winawer, Alekhine Gambit, 4... Nf6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. e3

			variation => N__('Nimzo-Indian: Classical, Noa, 5. e3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/2N1P3/PPQ2PPP/R1B1KBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'D38a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4

			variation => N__('QGD: Ragozin'),
			moves => {
				'c1g5' => 'rnbqk2r/ppp2ppp/4pn2/3p2B1/1bPP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 5 5',
				'c4d5' => 'rnbqk2r/ppp2ppp/4pn2/3P4/1b1P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
				'd1a4' => 'rnbqk2r/ppp2ppp/4pn2/3p4/QbPP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N2P2/PP2P1PP/R1BQKBNR w KQkq d6 0 5' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. f3 d5

			variation => N__('Nimzo-Indian: 4. f3 d5'),
			moves => {
				'a2a3' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/P1N2P2/1P2P1PP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq d6 0 5' => {
			eco => 'E34',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5

			variation => N__('Nimzo-Indian: Classical, Noa Variation'),
			moves => {
				'a2a3' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/P1N5/1PQ1PPPP/R1B1KBNR b KQkq - 0 5',
				'c4d5' => 'rnbqk2r/ppp2ppp/4pn2/3P4/1b1P4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 0 5',
				'e2e3' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/5N2/PP1NPPPP/R1BQKB1R w KQkq d6 0 5' => {
			eco => 'E11b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2 d5

			variation => N__('Bogo-Indian: 4.Nbd2 d5'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 3 5' => {
			eco => 'E00r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 Bb4+ 5. Bd2

			variation => N__('Catalan: 4. Nf3 Bb4+ 5. Bd2'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 2 5' => {
			eco => 'E00r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 Bb4+

			variation => N__('Catalan: 4. Nf3 Bb4+'),
			moves => {
				'c1d2' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP1BPPBP/RN1QK1NR b KQkq - 3 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Bb4+ 5. Bd2

			variation => N__('Catalan: 4... Bb4+ 5. Bd2'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP1NPPBP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Bb4+ 5. Nd2

			variation => N__('Catalan: 4... Bb4+ 5. Nd2'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP1NPPBP/R1BQK1NR w KQ - 4 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 Bb4+

			variation => N__('Catalan: 4... Bb4+'),
			moves => {
				'b1d2' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP1NPPBP/R1BQK1NR b KQkq - 3 5',
				'c1d2' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP1BPPBP/RN1QK1NR b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/P1N2P2/1P2P1PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E20',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: 4. f3 d5'),
			moves => {
				'b4e7' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/P1N2P2/1P2P1PP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/P1N5/1PQ1PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3

			variation => N__('Nimzo-Indian: Classical, Noa, 5. a3'),
			moves => {
				'b4c3' => 'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1b5/1PQ1PPPP/R1B1KBNR w KQkq - 0 6',
				'b4e7' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/P1N5/1PQ1PPPP/R1B1KBNR w KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 d5 6. f3

			variation => N__('Nimzo-Indian: Sämisch, 5... d5 6. f3'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1P5/4PPPP/R1BQKBNR w KQkq d6 0 6' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3 d5

			variation => N__('Nimzo-Indian: Sämisch, 5... d5'),
			moves => {
				'f2f3' => 'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1P2P2/4P1PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1Q5/1P2PPPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3

			variation => N__('Nimzo-Indian: Classical, Noa, 5. a3 Bxc3+'),
			moves => {
				'b8c6' => 'r1bqk2r/ppp2ppp/2n1pn2/3p4/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 1 7',
				'd5c4' => 'rnbqk2r/ppp2ppp/4pn2/8/2pP4/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 0 7',
				'e8g8' => 'rnbq1rk1/ppp2ppp/4pn2/3p4/2PP4/P1Q5/1P2PPPP/R1B1KBNR w KQ - 1 7',
				'f6e4' => 'rnbqk2r/ppp2ppp/4p3/3p4/2PPn3/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1b5/1PQ1PPPP/R1B1KBNR w KQkq - 0 6' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+

			variation => N__('Nimzo-Indian: Classical, Noa, 5. a3 Bxc3+'),
			moves => {
				'c2c3' => 'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1Q5/1P2PPPP/R1B1KBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3p4/QbPP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 5 5' => {
			eco => 'D38b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Qa4+

			variation => N__('QGD: Ragozin, 5. Qa4+'),
			moves => {
				'b8c6' => 'r1bqk2r/ppp2ppp/2n1pn2/3p4/QbPP4/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 6 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/3pP1B1/1b1P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'C12d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5

			variation => N__('French: MacCutcheon, Advance Variation'),
			moves => {
				'h7h6' => 'rnbqk2r/ppp2pp1/4pn1p/3pP1B1/1b1P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/6B1/1bpP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D39a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4

			variation => N__('QGD: Ragozin, Vienna Variation'),
			moves => {
				'd1a4' => 'rnbqk2r/ppp2ppp/4pn2/6B1/QbpP4/2N2N2/PP2PPPP/R3KB1R b KQkq - 1 6',
				'e2e4' => 'rnbqk2r/ppp2ppp/4pn2/6B1/1bpPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/6B1/1bpPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6' => {
			eco => 'D39c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. e4

			variation => N__('QGD: Ragozin, Vienna, 6. e4'),
			moves => {
				'c7c5' => 'rnbqk2r/pp3ppp/4pn2/2p3B1/1bpPP3/2N2N2/PP3PPP/R2QKB1R w KQkq c6 0 7',
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/6B1/QbpP4/2N2N2/PP2PPPP/R3KB1R b KQkq - 1 6' => {
			eco => 'D39b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. Bg5 dxc4 6. Qa4+

			variation => N__('QGD: Ragozin, Vienna, 6. Qa4+'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/8/1b1PB3/2N5/PPP2PPP/R1BQK1NR w KQkq - 1 6' => {
			eco => 'C15g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd3 dxe4 5. Bxe4 Nf6

			variation => N__('French: Winawer, 4. Bd3 dxe4 5.Bxe4 Nf6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/8/1b1Pp1Q1/2N5/PPPB1PPP/R3KBNR w KQkq - 2 6' => {
			eco => 'C15j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. Bd2 dxe4 5. Qg4 Nf6

			variation => N__('French: Winawer, Müller-Zhuravlev Gambit, 5. Qg4 Nf6'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/8/1bPPp3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 Nf6 5. f3 Bb4

			variation => N__('Diemer-Duhm Gambit (DDG): Main Line'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/8/1bpP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6' => {
			eco => 'E04b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 Bb4+

			variation => N__('Catalan: Open, 5. Nf3 Bb4+'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/4pn2/8/2pP4/P1Q5/1P2PPPP/R1B1KBNR w KQkq - 0 7' => {
			eco => 'E36',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. a3 Bxc3+ 6. Qxc3 dxc4

			variation => N__('Nimzo-Indian: Classical, Noa, 5. a3 Bxc3+ 6.Qxc3 dxc4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/5P2/2b4Q/2pp4/2P5/PP3PPP/RNB1K1NR b KQkq - 1 7' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppp2ppp/5P2/2b4Q/2pp4/2P5/PP3PPP/RNB1K1NR w KQ - 2 8',
			},
		},
		'rnbqk2r/ppp2ppp/5P2/2b5/2pp4/2P5/PP3PPP/RNBQK1NR w KQkq - 0 7' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'd1h5' => 'rnbqk2r/ppp2ppp/5P2/2b4Q/2pp4/2P5/PP3PPP/RNB1K1NR b KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2ppp/5P2/2bp4/2Bp4/2P5/PP3PPP/RNBQK1NR b KQkq - 0 6' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'd5c4' => 'rnbqk2r/ppp2ppp/5P2/2b5/2pp4/2P5/PP3PPP/RNBQK1NR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/2bBp3/4P3/2P2Q2/PP1P1PPP/RNB1K1NR b KQkq - 2 5' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. c3 d5 4. Bxd5 Nf6 5. Qf3

			variation => N__('Bishop\'s Opening: Lewis Countergambit, Jänisch'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/5n2/2bBp3/4P3/2P5/PP1P1PPP/RNBQK1NR w KQkq - 1 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Lewis Countergambit'),
			moves => {
				'd1f3' => 'rnbqk2r/ppp2ppp/5n2/2bBp3/4P3/2P2Q2/PP1P1PPP/RNB1K1NR b KQkq - 2 5',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/2bpP3/2Bp4/2P5/PP3PPP/RNBQK1NR w KQkq d6 0 6' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'e5f6' => 'rnbqk2r/ppp2ppp/5P2/2bp4/2Bp4/2P5/PP3PPP/RNBQK1NR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3P4/1b2pP2/2NP4/PPP3PP/R1BQKBNR w KQkq - 3 6' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. Nc3 Bb4

			variation => N__('KGD: Falkbeer, 4. d3 Nf6 5. Nc3 Bb4'),
			moves => {
				'c1d2' => 'rnbqk2r/ppp2ppp/5n2/3P4/1b2pP2/2NP4/PPPB2PP/R2QKBNR b KQkq - 4 6',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3P4/1b2pP2/2NP4/PPPB2PP/R2QKBNR b KQkq - 4 6' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 4. d3 Nf6 5. Nc3 Bb4'),
			moves => {
				'e4e3' => 'rnbqk2r/ppp2ppp/5n2/3P4/1b3P2/2NPp3/PPPB2PP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3P4/1b3P2/2NPp3/PPPB2PP/R2QKBNR w KQkq - 0 7' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. Nc3 Bb4 6. Bd2 e3

			variation => N__('KGD: Falkbeer, Morphy Gambit'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3P4/1bBp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'd8e7' => 'rnb1k2r/ppp1qppp/5n2/3P4/1bBp4/2P2N2/PP3PPP/RNBQK2R w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3P4/1bBp4/5N2/PPP2PPP/RNBQK2R w KQkq - 1 6' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'c2c3' => 'rnbqk2r/ppp2ppp/5n2/3P4/1bBp4/2P2N2/PP3PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D38f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 Bb4 5. cxd5 exd5 6. Bg5

			variation => N__('QGD: Ragozin, 5.cxd5 exd5 6. Bg5'),
			moves => {
				'h7h6' => 'rnbqk2r/ppp2pp1/5n1p/3p2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p2B1/1b1P4/2N5/PPQ1PPPP/R3KBNR b KQkq - 1 6' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. Bg5

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. Bg5'),
			moves => {
				'c7c5' => 'rnbqk2r/pp3ppp/5n2/2pp2B1/1b1P4/2N5/PPQ1PPPP/R3KBNR w KQkq c6 0 7',
				'h7h6' => 'rnbqk2r/ppp2pp1/5n1p/3p2B1/1b1P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p4/1b1P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D38e',
                        # TRANSLATORS:

			variation => N__('QGD: Ragozin, 5.cxd5'),
			moves => {
				'c1g5' => 'rnbqk2r/ppp2ppp/5n2/3p2B1/1b1P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p4/1b1P4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 6' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange'),
			moves => {
				'a2a3' => 'rnbqk2r/ppp2ppp/5n2/3p4/1b1P4/P1N5/1PQ1PPPP/R1B1KBNR b KQkq - 0 6',
				'c1g5' => 'rnbqk2r/ppp2ppp/5n2/3p2B1/1b1P4/2N5/PPQ1PPPP/R3KBNR b KQkq - 1 6',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p4/1b1P4/P1N5/1PQ1PPPP/R1B1KBNR b KQkq - 0 6' => {
			eco => 'E35',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2 d5 5. cxd5 exd5 6. a3

			variation => N__('Nimzo-Indian: Classical, Noa, Exchange, 6. a3'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p4/1bPP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 4 6' => {
			eco => 'C01e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nc3 Bb4 6. Nf3

			variation => N__('French: Exchange, 4. c4 Nf6 5. Nc3 Bb4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/5n2/3p4/1bPP4/2N5/PP3PPP/R1BQKBNR w KQkq - 3 6' => {
			eco => 'C01e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nc3 Bb4

			variation => N__('French: Exchange, 4. c4 Nf6 5. Nc3 Bb4'),
			moves => {
				'g1f3' => 'rnbqk2r/ppp2ppp/5n2/3p4/1bPP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 4 6',
			},
		},
		'rnbqk2r/ppp2ppp/5n2/8/1bBP4/2N5/PP3PPP/R1BQK1NR w KQkq - 3 7' => {
			eco => 'D20f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e5 4. Bxc4 exd4 5. exd4 Bb4+ 6. Nc3 Nf6

			variation => N__('QGA: 3. e3 e5'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/8/2bP4/4nP2/5N2/PPP1Q1PP/RNB1KB1R b KQkq - 3 7' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Nf3 Bc5 7. Qe2

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Nf3 Bc5 7. Qe2'),
			moves => {
				'c5f2' => 'rnbqk2r/ppp2ppp/8/3P4/4nP2/5N2/PPP1QbPP/RNB1KB1R w KQkq - 4 8',
				'c8f5' => 'rn1qk2r/ppp2ppp/8/2bP1b2/4nP2/5N2/PPP1Q1PP/RNB1KB1R w KQkq - 4 8',
			},
		},
		'rnbqk2r/ppp2ppp/8/2bP4/4nP2/5N2/PPP3PP/RNBQKB1R w KQkq - 2 7' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Nf3'),
			moves => {
				'd1e2' => 'rnbqk2r/ppp2ppp/8/2bP4/4nP2/5N2/PPP1Q1PP/RNB1KB1R b KQkq - 3 7',
			},
		},
		'rnbqk2r/ppp2ppp/8/3P4/4nP2/5N2/PPP1QbPP/RNB1KB1R w KQkq - 4 8' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Nf3 Bc5 7. Qe2'),
			moves => {
				'e1d1' => 'rnbqk2r/ppp2ppp/8/3P4/4nP2/5N2/PPP1QbPP/RNBK1B1R b kq - 5 8',
			},
		},
		'rnbqk2r/ppp2ppp/8/3P4/4nP2/5N2/PPP1QbPP/RNBK1B1R b kq - 5 8' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Nf3 Bc5 7. Qe2'),
			moves => {
				'd8d5' => 'rnb1k2r/ppp2ppp/8/3q4/4nP2/5N2/PPP1QbPP/RNBK1B1R w kq - 0 9',
			},
		},
		'rnbqk2r/ppp2ppp/8/3pP3/1b2P3/2N5/PPP3PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C29h',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford, 5... Bb4'),
			moves => {
				'd8h4' => 'rnb1k2r/ppp2ppp/8/3pP3/1b2P2q/2N5/PPP3PP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqk2r/ppp2ppp/8/3pP3/1b2n3/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C29p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3 Bb4

			variation => N__('Vienna Gambit: 5. Nf3 Bb4'),
			moves => {
			},
		},
		'rnbqk2r/ppp2ppp/8/3pP3/1b2n3/2NP4/PPP3PP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'C29h',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. d3 Bb4

			variation => N__('Vienna Gambit: Oxford, 5... Bb4'),
			moves => {
				'd3e4' => 'rnbqk2r/ppp2ppp/8/3pP3/1b2P3/2N5/PPP3PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/pppn1pp1/4p2p/3pP3/1b1P4/2N5/PPPB1PPP/R2QKBNR w KQkq - 2 7' => {
			eco => 'C12i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Bb4 5. e5 h6 6. Bd2 Nfd7

			variation => N__('French: MacCutcheon, Tartakower Variation'),
			moves => {
			},
		},
		'rnbqk2r/pppn1ppp/4p3/3pP1P1/3P4/2N5/PPP2PP1/R2QKBNR b KQkq - 0 7' => {
			eco => 'C13l',
                        # TRANSLATORS:

			variation => N__('French: Chatard-Alekhine, 6...Bxg5'),
			moves => {
				'd8g5' => 'rnb1k2r/pppn1ppp/4p3/3pP1q1/3P4/2N5/PPP2PP1/R2QKBNR w KQkq - 0 8',
			},
		},
		'rnbqk2r/pppn1ppp/4p3/3pP1b1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq - 0 7' => {
			eco => 'C13l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 Bxg5

			variation => N__('French: Chatard-Alekhine, 6...Bxg5'),
			moves => {
				'h4g5' => 'rnbqk2r/pppn1ppp/4p3/3pP1P1/3P4/2N5/PPP2PP1/R2QKBNR b KQkq - 0 7',
			},
		},
		'rnbqk2r/pppnBppp/4p3/3pP3/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'C13h',
                        # TRANSLATORS:

			variation => N__('French: Classical, 5. e5 Nfd7'),
			moves => {
				'd8e7' => 'rnb1k2r/pppnqppp/4p3/3pP3/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/pppnb1pp/4pp2/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq - 0 7' => {
			eco => 'C13j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4 f6

			variation => N__('French: Chatard-Alekhine, Teichmann Variation'),
			moves => {
			},
		},
		'rnbqk2r/pppnbppp/4p3/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR b KQkq h3 0 6' => {
			eco => 'C13i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7 6. h4

			variation => N__('French: Chatard-Alekhine Attack'),
			moves => {
				'a7a6' => 'rnbqk2r/1ppnbppp/p3p3/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq - 0 7',
				'c7c5' => 'rnbqk2r/pp1nbppp/4p3/2ppP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq c6 0 7',
				'e7g5' => 'rnbqk2r/pppn1ppp/4p3/3pP1b1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq - 0 7',
				'e8g8' => 'rnbq1rk1/pppnbppp/4p3/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQ - 1 7',
				'f7f6' => 'rnbqk2r/pppnb1pp/4pp2/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/pppnbppp/4p3/3pP1B1/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'C13h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 Be7 5. e5 Nfd7

			variation => N__('French: Classical, 5. e5 Nfd7'),
			moves => {
				'g5e7' => 'rnbqk2r/pppnBppp/4p3/3pP3/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 0 6',
				'h2h4' => 'rnbqk2r/pppnbppp/4p3/3pP1B1/3P3P/2N5/PPP2PP1/R2QKBNR b KQkq h3 0 6',
			},
		},
		'rnbqk2r/pppp1pbp/5np1/4p3/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQkq e3 0 5' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e4

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. e3

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp2pbp/3p1np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R w KQkq - 0 6',
			},
		},
		'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'd2d3' => 'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5',
				'e2e3' => 'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2N1P1P1/PP1P1PBP/R1BQK1NR b KQkq - 0 5',
				'e2e4' => 'rnbqk2r/pppp1pbp/5np1/4p3/2P1P3/2N3P1/PP1P1PBP/R1BQK1NR b KQkq e3 0 5',
				'g1f3' => 'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 3 5',
			},
		},
		'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2NP2P1/PP2PPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2 Bg7 5. d3

			variation => N__('English: Bremen, 3... g6'),
			moves => {
			},
		},
		'rnbqk2r/pppp1pp1/4pB1p/8/1bPP4/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bxf6

			variation => N__('Nimzo-Indian: Leningrad, 4... h6 5.Bxf6'),
			moves => {
			},
		},
		'rnbqk2r/pppp1pp1/4pn1p/6B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6

			variation => N__('Nimzo-Indian: Leningrad, 4... h6'),
			moves => {
				'g5f6' => 'rnbqk2r/pppp1pp1/4pB1p/8/1bPP4/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5',
				'g5h4' => 'rnbqk2r/pppp1pp1/4pn1p/8/1bPP3B/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5',
			},
		},
		'rnbqk2r/pppp1pp1/4pn1p/8/1bPP3B/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5 h6 5. Bh4

			variation => N__('Nimzo-Indian: Leningrad, 4... h6 5. Bh4'),
			moves => {
				'c7c5' => 'rnbqk2r/pp1p1pp1/4pn1p/2p5/1bPP3B/2N5/PP2PPPP/R2QKBNR w KQkq c6 0 6',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/6B1/1bPP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'E30',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bg5

			variation => N__('Nimzo-Indian: Leningrad'),
			moves => {
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p3B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQkq c6 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/6B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQ - 4 5',
				'h7h6' => 'rnbqk2r/pppp1pp1/4pn1p/6B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bP5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 2 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 Bb4

			variation => N__('English: Nimzo-English'),
			moves => {
				'd1c2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bP5/2N2N2/PPQPPPPP/R1B1KB1R b KQkq - 3 4',
				'g2g3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bP5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bP5/2N2N2/PPQPPPPP/R1B1KB1R b KQkq - 3 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 Bb4 4. Qc2

			variation => N__('English: Nimzo-English, 4. Qc2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bP5/2N2N2/PPQPPPPP/R1B1KB1R w KQ - 4 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bP5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 Bb4 4. g3

			variation => N__('English: Nimzo-English, 4. g3'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/1QN5/PP2PPPP/R1B1KBNR b KQkq - 3 4' => {
			eco => 'E22',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qb3

			variation => N__('Nimzo-Indian: Spielmann Variation'),
			moves => {
				'b8c6' => 'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/1QN5/PP2PPPP/R1B1KBNR w KQkq - 4 5',
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/1QN5/PP2PPPP/R1B1KBNR w KQkq c6 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'E40',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3

			variation => N__('Nimzo-Indian: Rubinstein'),
			moves => {
				'b7b6' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 5',
				'b8c6' => 'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5',
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq c6 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQ - 1 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3

			variation => N__('Nimzo-Indian: Three Knights'),
			moves => {
				'b4c3' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/2b2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'b7b6' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQ - 4 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. f3

			variation => N__('Nimzo-Indian: 4. f3 (Kmoch)'),
			moves => {
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N2P2/PP2P1PP/R1BQKBNR w KQkq c6 0 5',
				'd7d5' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N2P2/PP2P1PP/R1BQKBNR w KQkq d6 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. g3

			variation => N__('Nimzo-Indian: Romanishin'),
			moves => {
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq c6 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N5/PP1BPPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Bd2

			variation => N__('Nimzo-Indian: 4. Bd2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N5/PP1BPPPP/R2QKBNR w KQ - 4 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4

			variation => N__('Nimzo-Indian Defence'),
			moves => {
				'a2a3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/P1N5/1P2PPPP/R1BQKBNR b KQkq - 0 4',
				'c1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N5/PP1BPPPP/R2QKBNR b KQkq - 3 4',
				'c1g5' => 'rnbqk2r/pppp1ppp/4pn2/6B1/1bPP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'd1b3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/1QN5/PP2PPPP/R1B1KBNR b KQkq - 3 4',
				'd1c2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 3 4',
				'd1d3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2NQ4/PP2PPPP/R1B1KBNR b KQkq - 3 4',
				'e2e3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'f2f3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
				'g2g3' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 3 4' => {
			eco => 'E32a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qc2

			variation => N__('Nimzo-Indian: Classical Variation'),
			moves => {
				'b7b6' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 5',
				'b8c6' => 'r1bqk2r/pppp1ppp/2n1pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 4 5',
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq c6 0 5',
				'd7d5' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq d6 0 5',
				'd7d6' => 'rnbqk2r/ppp2ppp/3ppn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/2N5/PPQ1PPPP/R1B1KBNR w KQ - 4 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2NQ4/PP2PPPP/R1B1KBNR b KQkq - 3 4' => {
			eco => 'E20',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Qd3

			variation => N__('Nimzo-Indian: Mikenas Attack'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/5N2/PP1BPPPP/RN1QKB1R b KQkq - 3 4' => {
			eco => 'E11g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2

			variation => N__('Bogo-Indian: 4. Bd2'),
			moves => {
				'a7a5' => 'rnbqk2r/1ppp1ppp/4pn2/p7/1bPP4/5N2/PP1BPPPP/RN1QKB1R w KQkq a6 0 5',
				'b4d2' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/5N2/PP1bPPPP/RN1QKB1R w KQkq - 0 5',
				'b4e7' => 'rnbqk2r/ppppbppp/4pn2/8/2PP4/5N2/PP1BPPPP/RN1QKB1R w KQkq - 4 5',
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/1bPP4/5N2/PP1BPPPP/RN1QKB1R w KQkq c6 0 5',
				'd8e7' => 'rnb1k2r/ppppqppp/4pn2/8/1bPP4/5N2/PP1BPPPP/RN1QKB1R w KQkq - 4 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'E11a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Nbd2

			variation => N__('Bogo-Indian: 4.Nbd2'),
			moves => {
				'b7b6' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/5N2/PP1NPPPP/R1BQKB1R w KQkq - 0 5',
				'd7d5' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/5N2/PP1NPPPP/R1BQKB1R w KQkq d6 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/1bPP4/5N2/PP1NPPPP/R1BQKB1R w KQ - 4 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'E11a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+

			variation => N__('Bogo-Indian'),
			moves => {
				'b1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 3 4',
				'c1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/5N2/PP1BPPPP/RN1QKB1R b KQkq - 3 4',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/6P1/PP1BPP1P/RN1QKBNR b KQkq - 2 4' => {
			eco => 'E00j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+ 4. Bd2

			variation => N__('Catalan: 3... Bb4+ 4. Bd2'),
			moves => {
				'b4d2' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/6P1/PP1bPP1P/RN1QKBNR w KQkq - 0 5',
				'b4e7' => 'rnbqk2r/ppppbppp/4pn2/8/2PP4/6P1/PP1BPP1P/RN1QKBNR w KQkq - 3 5',
				'd8e7' => 'rnb1k2r/ppppqppp/4pn2/8/1bPP4/6P1/PP1BPP1P/RN1QKBNR w KQkq - 3 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/6P1/PP1NPP1P/R1BQKBNR b KQkq - 2 4' => {
			eco => 'E00i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+ 4. Nd2

			variation => N__('Catalan: 3... Bb4+ 4. Nd2'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 1 4' => {
			eco => 'E00i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+

			variation => N__('Catalan: 3... Bb4+'),
			moves => {
				'b1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/6P1/PP1NPP1P/R1BQKBNR b KQkq - 2 4',
				'c1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/6P1/PP1BPP1P/RN1QKBNR b KQkq - 2 4',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/P1N5/1P2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3

			variation => N__('Nimzo-Indian: Sämisch'),
			moves => {
				'b4c3' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/P1b5/1P2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/2b2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'E21',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. Nf3 Bxc3+

			variation => N__('Nimzo-Indian: Three Knights, 4...Bxc3+'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/5N2/PP1NPPPP/R2QKB1R b KQkq - 0 5' => {
			eco => 'E11x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+ 5. Nbxd2

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+ 5.Nbxd2'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/5N2/PP1QPPPP/RN2KB1R b KQkq - 0 5' => {
			eco => 'E11y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+ 5. Qxd2

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+ 5.Qxd2'),
			moves => {
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/5N2/PP1QPPPP/RN2KB1R w KQ - 1 6',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/5N2/PP1bPPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'E11x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Bxd2+

			variation => N__('Bogo-Indian: 4. Bd2 Bxd2+'),
			moves => {
				'b1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/5N2/PP1NPPPP/R2QKB1R b KQkq - 0 5',
				'd1d2' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/5N2/PP1QPPPP/RN2KB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/6P1/PP1bPP1P/RN1QKBNR w KQkq - 0 5' => {
			eco => 'E00k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+ 4. Bd2 Bxd2+

			variation => N__('Catalan: 3... Bb4+ 4. Bd2 Bxd2+'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/P1P5/4PPPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'E24',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. a3 Bxc3+ 5. bxc3

			variation => N__('Nimzo-Indian: Sämisch'),
			moves => {
				'b7b6' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/2PP4/P1P5/4PPPP/R1BQKBNR w KQkq - 0 6',
				'c7c5' => 'rnbqk2r/pp1p1ppp/4pn2/2p5/2PP4/P1P5/4PPPP/R1BQKBNR w KQkq c6 0 6',
				'd7d5' => 'rnbqk2r/ppp2ppp/4pn2/3p4/2PP4/P1P5/4PPPP/R1BQKBNR w KQkq d6 0 6',
				'e8g8' => 'rnbq1rk1/pppp1ppp/4pn2/8/2PP4/P1P5/4PPPP/R1BQKBNR w KQ - 1 6',
			},
		},
		'rnbqk2r/pppp1ppp/4pn2/8/2PP4/P1b5/1P2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'E24',
                        # TRANSLATORS:

			variation => N__('Nimzo-Indian: Sämisch'),
			moves => {
				'b2c3' => 'rnbqk2r/pppp1ppp/4pn2/8/2PP4/P1P5/4PPPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1P3/2Bp4/2P5/PP3PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'd7d5' => 'rnbqk2r/ppp2ppp/5n2/2bpP3/2Bp4/2P5/PP3PPP/RNBQK1NR w KQkq d6 0 6',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C26q',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5 4. Nf3

			variation => N__('Vienna: 3. Bc4 Bc5 4. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4' => {
			eco => 'C26p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5

			variation => N__('Vienna: 3. Bc4 Bc5'),
			moves => {
				'd2d3' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR b KQkq - 0 4',
				'f2f4' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1PP2/2N5/PPPP2PP/R1BQK1NR b KQkq f3 0 4',
				'g1f3' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR b KQkq - 0 4' => {
			eco => 'C26r',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5 4. d3

			variation => N__('Vienna: 3. Bc4 Bc5 4. d3'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/2B1P3/2NP4/PPP2PPP/R1BQK1NR w KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2P5/PP1P1PPP/RNBQK1NR w KQkq - 1 4' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Bc5 3. c3 Nf6

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'd2d4' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2BPP3/2P5/PP3PPP/RNBQK1NR b KQkq d3 0 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1PP2/2N5/PPPP2PP/R1BQK1NR b KQkq f3 0 4' => {
			eco => 'C26p',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bc5 4. f4

			variation => N__('Vienna: 3. Bc4 Bc5 4. f4'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2BPP3/2P5/PP3PPP/RNBQK1NR b KQkq d3 0 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'e5d4' => 'rnbqk2r/pppp1ppp/5n2/2b5/2BpP3/2P5/PP3PPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 Bc5

			variation => N__('English: Bremen, 3... Bc5'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 3 4' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nf6 3. Bg2 Bc5

			variation => N__('English: King\'s, 2. g3 Nf6 3. Bg2 Bc5'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'C26e',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Bc5

			variation => N__('Vienna: Smyslov, 3... Bc5'),
			moves => {
				'f1g2' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'C26e',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 Bc5 4. Bg2

			variation => N__('Vienna: Smyslov, 3... Bc5'),
			moves => {
				'b8c6' => 'r1bqk2r/pppp1ppp/2n2n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQkq - 3 5',
				'd7d6' => 'rnbqk2r/ppp2ppp/3p1n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/5n2/2b1p3/4P3/2N3P1/PPPP1PBP/R1BQK1NR w KQ - 3 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/2b5/2BpP3/2P5/PP3PPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 2... Bc5 3. c3 Nf6'),
			moves => {
				'e4e5' => 'rnbqk2r/pppp1ppp/5n2/2b1P3/2Bp4/2P5/PP3PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4B3/1b6/5N2/P1PPPPPP/RN1QKB1R b KQkq - 2 4' => {
			eco => 'A00s',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 Bxb4 3. Bxe5 Nf6 4. Nf3

			variation => N__('Polish: 2...Bxb4 3.Bxe5 Nf6 4. Nf3'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4B3/1b6/8/P1PPPPPP/RN1QKBNR w KQkq - 1 4' => {
			eco => 'A00s',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 Bxb4 3. Bxe5 Nf6

			variation => N__('Polish: 2...Bxb4 3.Bxe5 Nf6'),
			moves => {
				'c2c4' => 'rnbqk2r/pppp1ppp/5n2/4B3/1bP5/8/P2PPPPP/RN1QKBNR b KQkq c3 0 4',
				'g1f3' => 'rnbqk2r/pppp1ppp/5n2/4B3/1b6/5N2/P1PPPPPP/RN1QKB1R b KQkq - 2 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4B3/1bP5/8/P2PPPPP/RN1QKBNR b KQkq c3 0 4' => {
			eco => 'A00s',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 Bxb4 3. Bxe5 Nf6 4. c4

			variation => N__('Polish: 2...Bxb4 3.Bxe5 Nf6 4. c4'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4N3/1b2P3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'C42d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nc3 Bb4 4. Nxe5

			variation => N__('Russian-Three Knights Game, 4.Nxe5'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4P3/1bB2p2/2N5/PPPP2PP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Nf6 4. Nc3 Bb4 5. e5

			variation => N__('KGA: Bishop\'s Gambit, Paulsen Attack'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/1b2P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4' => {
			eco => 'C42c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nc3 Bb4

			variation => N__('Russian-Three Knights Game'),
			moves => {
				'f3e5' => 'rnbqk2r/pppp1ppp/5n2/4N3/1b2P3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/1bB1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4' => {
			eco => 'C26n',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bb4 4. Nf3

			variation => N__('Vienna: 3. Bc4 Bb4 4. Nf3'),
			moves => {
				'e8g8' => 'rnbq1rk1/pppp1ppp/5n2/4p3/1bB1P3/2N2N2/PPPP1PPP/R1BQK2R w KQ - 6 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/1bB1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4' => {
			eco => 'C26m',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Bb4

			variation => N__('Vienna: 3. Bc4 Bb4'),
			moves => {
				'g1f3' => 'rnbqk2r/pppp1ppp/5n2/4p3/1bB1P3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 5 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/1bP5/2N1P3/PP1P1PPP/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. e3 Bb4

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. e3 Bb4'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/1bP5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 Bb4

			variation => N__('English: Bremen, Smyslov System'),
			moves => {
				'f1g2' => 'rnbqk2r/pppp1ppp/5n2/4p3/1bP5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/1bP5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 Bb4 4. Bg2

			variation => N__('English: Bremen, Smyslov, 4. Bg2'),
			moves => {
				'b4c3' => 'rnbqk2r/pppp1ppp/5n2/4p3/2P5/2b3P1/PP1PPPBP/R1BQK1NR w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/pppp1ppp/5n2/4p3/1bP5/2N3P1/PP1PPPBP/R1BQK1NR w KQ - 3 5',
			},
		},
		'rnbqk2r/pppp1ppp/5n2/4p3/2P5/2b3P1/PP1PPPBP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 Bb4 4. Bg2 Bxc3

			variation => N__('English: Bremen, Smyslov, 4. Bg2 Bxc3'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/5n2/8/1bB1Pp2/2N5/PPPP2PP/R1BQK1NR w KQkq - 4 5' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Bogoljubow Variation'),
			moves => {
				'e4e5' => 'rnbqk2r/pppp1ppp/5n2/4P3/1bB2p2/2N5/PPPP2PP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/8/2b1P3/2P3n1/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A52f',
                        # TRANSLATORS:

			variation => N__('Budapest: Adler, 4... Bc5'),
			moves => {
				'b8c6' => 'r1bqk2r/pppp1ppp/2n5/2b1P3/2P3n1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqk2r/pppp1ppp/8/2b1P3/2P3n1/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 5' => {
			eco => 'A52f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Nf3 Bc5

			variation => N__('Budapest: Adler, 4... Bc5'),
			moves => {
				'e2e3' => 'rnbqk2r/pppp1ppp/8/2b1P3/2P3n1/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/pppp1ppp/8/4P3/1bP1n3/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 5' => {
			eco => 'A51i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. Nf3 Bb4+

			variation => N__('Budapest: Fajarowicz, 4. Nf3 Bb4+'),
			moves => {
			},
		},
		'rnbqk2r/pppp1ppp/8/4P3/1bP2Bn1/8/PP2PPPP/RN1QKBNR w KQkq - 3 5' => {
			eco => 'A52i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Bf4 Bb4+

			variation => N__('Budapest: Rubinstein, 4... Bb4+'),
			moves => {
			},
		},
		'rnbqk2r/pppp2pp/4pb2/8/3PN3/5N2/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'A83i',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 e6 5. Nxe4 Be7 6. Bxf6 Bxf6 7. Nf3

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 e6 5.Nxe4'),
			moves => {
			},
		},
		'rnbqk2r/pppp2pp/4pb2/8/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 0 7' => {
			eco => 'A83h',
                        # TRANSLATORS:

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 e6 5.Nxe4'),
			moves => {
				'g1f3' => 'rnbqk2r/pppp2pp/4pb2/8/3PN3/5N2/PPP2PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. e3 Bb4

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. e3 Bb4'),
			moves => {
			},
		},
		'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Bb4+ 5. Nc3

			variation => N__('Dutch: Dutch-Indian, 5. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/6P1/PP1BPPBP/RN1QK1NR b KQkq - 3 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Bb4+ 5. Bd2

			variation => N__('Dutch: Dutch-Indian, 5. Bd2'),
			moves => {
				'b4e7' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/6P1/PP1BPPBP/RN1QK1NR w KQkq - 4 6',
			},
		},
		'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/6P1/PP1NPPBP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Bb4+ 5. Nd2

			variation => N__('Dutch: Dutch-Indian, 5. Nd2'),
			moves => {
			},
		},
		'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Bb4+

			variation => N__('Dutch: Dutch-Indian (Nimzo-Dutch) Variation'),
			moves => {
				'b1c3' => 'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 3 5',
				'b1d2' => 'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/6P1/PP1NPPBP/R1BQK1NR b KQkq - 3 5',
				'c1d2' => 'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/6P1/PP1BPPBP/RN1QK1NR b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppppb1pp/4pB2/8/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'A83h',
                        # TRANSLATORS:

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 e6 5.Nxe4'),
			moves => {
				'e7f6' => 'rnbqk2r/pppp2pp/4pb2/8/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 0 7',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6 4. Bg2 Be7 5. O-O

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w - - 4 6',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6 4. Bg2 Be7

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'e1g1' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 3 5',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nc3'),
			moves => {
				'd7d5' => 'rnbqk2r/ppp1b1pp/4pn2/3p1p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQkq d6 0 6',
				'e8g8' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQ - 4 6',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nf3'),
			moves => {
				'd7d5' => 'rnbqk2r/ppp1b1pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq d6 0 6',
				'd7d6' => 'rnbqk2r/ppp1b1pp/3ppn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/6P1/PP1BPPBP/RN1QK1NR w KQkq - 4 6' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Bb4+ 5. Bd2 Be7

			variation => N__('Dutch: Dutch-Indian, Alekhine Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7'),
			moves => {
				'b1c3' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 3 5',
				'g1f3' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5',
				'g1h3' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 3 5' => {
			eco => 'A91',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 Be7 5. Nh3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppb1pp/4pn2/5p2/2PP4/6PN/PP2PPBP/RNBQK2R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppppb1pp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 1 6' => {
			eco => 'A83h',
                        # TRANSLATORS:

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 e6 5.Nxe4'),
			moves => {
				'g5f6' => 'rnbqk2r/ppppb1pp/4pB2/8/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqk2r/ppppbppp/3n4/4p2Q/8/1BN2N2/PPPP1PPP/R1B1K2R b KQkq - 5 6' => {
			eco => 'C27f',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Be7'),
			moves => {
				'b8c6' => 'r1bqk2r/ppppbppp/2nn4/4p2Q/8/1BN2N2/PPPP1PPP/R1B1K2R w KQkq - 6 7',
			},
		},
		'rnbqk2r/ppppbppp/3n4/4p2Q/8/1BN5/PPPP1PPP/R1B1K1NR w KQkq - 4 6' => {
			eco => 'C27f',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Be7

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3 Be7'),
			moves => {
				'g1f3' => 'rnbqk2r/ppppbppp/3n4/4p2Q/8/1BN2N2/PPPP1PPP/R1B1K2R b KQkq - 5 6',
			},
		},
		'rnbqk2r/ppppbppp/4pn2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A46r',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 Be7

			variation => N__('Torre Attack: 3... Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppppbppp/4pn2/8/2PP4/5N2/PP1BPPPP/RN1QKB1R w KQkq - 4 5' => {
			eco => 'E11g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7

			variation => N__('Bogo-Indian: 4. Bd2 Be7'),
			moves => {
				'g2g3' => 'rnbqk2r/ppppbppp/4pn2/8/2PP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppppbppp/4pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'E10c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Be7

			variation => N__('Neo-Indian: 3. Nf3 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppppbppp/4pn2/8/2PP4/5NP1/PP1BPP1P/RN1QKB1R b KQkq - 0 5' => {
			eco => 'E11g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Bb4+ 4. Bd2 Be7 5. g3

			variation => N__('Bogo-Indian: 4. Bd2 Be7 5. g3'),
			moves => {
				'd7d5' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP1BPP1P/RN1QKB1R w KQkq d6 0 6',
			},
		},
		'rnbqk2r/ppppbppp/4pn2/8/2PP4/6P1/PP1BPP1P/RN1QKBNR w KQkq - 3 5' => {
			eco => 'E00l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 Bb4+ 4. Bd2 Be7

			variation => N__('Catalan: 3... Bb4+ 4. Bd2 Be7'),
			moves => {
			},
		},
		'rnbqk2r/ppppbppp/5n2/6B1/2BQ4/2N2N2/PPP2PPP/R3K2R b KQkq - 4 7' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'b8c6' => 'r1bqk2r/ppppbppp/2n2n2/6B1/2BQ4/2N2N2/PPP2PPP/R3K2R w KQkq - 5 8',
				'c7c6' => 'rnbqk2r/pp1pbppp/2p2n2/6B1/2BQ4/2N2N2/PPP2PPP/R3K2R w KQkq - 0 8',
			},
		},
		'rnbqk2r/ppppbppp/5n2/6B1/2BQ4/5N2/PPP2PPP/RN2K2R w KQkq - 3 7' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'b1c3' => 'rnbqk2r/ppppbppp/5n2/6B1/2BQ4/2N2N2/PPP2PPP/R3K2R b KQkq - 4 7',
			},
		},
		'rnbqk2r/ppppbppp/5n2/8/2B1Pp2/5N2/PPPP2PP/RNBQK2R w KQkq - 4 5' => {
			eco => 'C35',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 Be7 4. Bc4 Nf6

			variation => N__('KGA: Cunningham, Euwe Defence'),
			moves => {
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Bf4

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Bf4'),
			moves => {
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. e3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. e3'),
			moves => {
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7'),
			moves => {
				'c1f4' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5',
				'c1g5' => 'rnbqk2r/ppppp1bp/5np1/5pB1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5',
				'e2e3' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 3 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Nf3 Bg7 5. Nc3

			variation => N__('Dutch: Leningrad, 4. Nf3 Bg7 5. Nc3'),
			moves => {
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq - 2 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Nc3 Bg7

			variation => N__('Dutch: Leningrad, 4. Nc3 Bg7'),
			moves => {
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 3 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nc3

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nc3'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQ - 4 6',
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 2 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Nf3 Bg7

			variation => N__('Dutch: Leningrad, 4. Nf3 Bg7'),
			moves => {
				'b1c3' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5' => {
			eco => 'A87',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nf3

			variation => N__('Dutch: Leningrad, Main Line'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1p1bp/3p1np1/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
				'e8g8' => 'rnbq1rk1/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQ - 4 6',
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7'),
			moves => {
				'b1c3' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 3 5',
				'g1f3' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 3 5',
				'g1h3' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 3 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 Bg7 5. Nh3

			variation => N__('Dutch: Leningrad, 4. Bg2 Bg7 5. Nh3'),
			moves => {
			},
		},
		'rnbqk2r/ppppp1bp/5np1/5pB1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 3 5' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Bg5

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. Nf3 Bg7 5. Bg5'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/6B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'E61c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Bg5

			variation => N__('King\'s Indian: 4. Bg5'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/6B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQ - 4 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/6B1/3P4/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 3 4' => {
			eco => 'A48p',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nc3

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/6B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A48o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. c3

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/6B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A48o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. e3

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A48o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'b1c3' => 'rnbqk2r/ppppppbp/5np1/6B1/3P4/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 3 4',
				'b1d2' => 'rnbqk2r/ppppppbp/5np1/6B1/3P4/5N2/PPPNPPPP/R2QKB1R b KQkq - 3 4',
				'c2c3' => 'rnbqk2r/ppppppbp/5np1/6B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqk2r/ppppppbp/5np1/6B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/6B1/3P4/5N2/PPPNPPPP/R2QKB1R b KQkq - 3 4' => {
			eco => 'A48q',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Bg7 4. Nbd2

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/6B1/3P4/5N2/PPPNPPPP/R2QKB1R w KQ - 4 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/1P6/5NP1/P1PPPP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A05',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, Reti-Smyslov Variation'),
			moves => {
				'c1b2' => 'rnbqk2r/ppppppbp/5np1/8/1P6/5NP1/PBPPPP1P/RN1QKB1R b KQkq - 2 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/1P6/5NP1/PBPPPP1P/RN1QKB1R b KQkq - 2 4' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. b4 Bg7 4. Bb2

			variation => N__('Reti: KIA, Reti-Smyslov Variation'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3 Bg7

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
				'f1g2' => 'rnbqk2r/ppppppbp/5np1/8/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3 Bg7 4. Bg2

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQ - 3 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A15',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Indian, 2. Nf3 g6'),
			moves => {
				'f1g2' => 'rnbqk2r/ppppppbp/5np1/8/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2

			variation => N__('English: Anglo-Indian, 2. Nf3 g6'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2P5/5NP1/PP1PPPBP/RNBQK2R w KQ - 3 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. g3 g6 3. Bg2 Bg7

			variation => N__('English: Anglo-Indian, 2. g3 g6'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'E61b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Bf4

			variation => N__('King\'s Indian: 4. Bf4'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/1P3N2/P3PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E60h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. b3

			variation => N__('King\'s Indian: b3 System'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/1P3N2/P3PPPP/RNBQKB1R w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/1P3N2/P3PPPP/RNBQKB1R w KQ - 1 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'E61i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. Nf3

			variation => N__('King\'s Indian: 4. Nf3'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQ - 4 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'E61f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. g3

			variation => N__('King\'s Indian: 4. g3'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQ - 1 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'E61b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7

			variation => N__('King\'s Indian: 3. Nc3 Bg7'),
			moves => {
				'c1f4' => 'rnbqk2r/ppppppbp/5np1/8/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'c1g5' => 'rnbqk2r/ppppppbp/5np1/6B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'e2e4' => 'rnbqk2r/ppppppbp/5np1/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
				'g2g3' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'E60g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7

			variation => N__('King\'s Indian: 3. Nf3 Bg7'),
			moves => {
				'b2b3' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/1P3N2/P3PPPP/RNBQKB1R b KQkq - 0 4',
				'g2g3' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E60j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 Bg7 4. g3

			variation => N__('King\'s Indian: Fianchetto'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQ - 1 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 1 4' => {
			eco => 'E60e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 Bg7

			variation => N__('King\'s Indian: 3. g3'),
			moves => {
				'f1g2' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 2 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 2 4' => {
			eco => 'E60e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 Bg7 4. Bg2

			variation => N__('King\'s Indian: 3. g3'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2PP4/6P1/PP2PPBP/RNBQK1NR w KQ - 3 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'E70a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4

			variation => N__('King\'s Indian: 4. e4'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5',
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQ - 1 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P1B2/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A48i',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. c3

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A48j',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. e3

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQ - 1 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A48g',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
				'b1d2' => 'rnbqk2r/ppppppbp/5np1/8/3P1B2/5N2/PPPNPPPP/R2QKB1R b KQkq - 3 4',
				'c2c3' => 'rnbqk2r/ppppppbp/5np1/8/3P1B2/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqk2r/ppppppbp/5np1/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P1B2/5N2/PPPNPPPP/R2QKB1R b KQkq - 3 4' => {
			eco => 'A48h',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4 Bg7 4. Nbd2

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A48b',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. c3 Bg7

			variation => N__('Neo-King\'s Indian'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A48c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. e3 Bg7

			variation => N__('Neo-King\'s Indian'),
			moves => {
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3 Bg7

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'f1g2' => 'rnbqk2r/ppppppbp/5np1/8/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/3P4/5NP1/PPP1PPBP/RNBQK2R w KQ - 3 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/5P2/5NP1/PPPPP1BP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 g6 3. g3 Bg7 4. Bg2

			variation => N__('Bird: 1..Nf6 2. Nf3 g6 3. g3'),
			moves => {
				'd7d6' => 'rnbqk2r/ppp1ppbp/3p1np1/8/5P2/5NP1/PPPPP1BP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/5P2/5NP1/PPPPP2P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: 1..Nf6 2. Nf3 g6 3. g3'),
			moves => {
				'f1g2' => 'rnbqk2r/ppppppbp/5np1/8/5P2/5NP1/PPPPP1BP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2 Bg7 4. O-O

			variation => N__('Reti: KIA'),
			moves => {
				'e8g8' => 'rnbq1rk1/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQ1RK1 w - - 4 5',
			},
		},
		'rnbqk2r/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2 Bg7

			variation => N__('Reti: KIA'),
			moves => {
				'e1g1' => 'rnbqk2r/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/2p5/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5' => {
			eco => 'E10f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 a6 4. Nc3 c5

			variation => N__('Neo-Indian: 3. Nf3 a6 4. Nc3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 a6 6. g3

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 a6 6. g3'),
			moves => {
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 a6

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 a6'),
			moves => {
				'g2g3' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/2N3P1/PP2PP1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 0 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 a6

			variation => N__('English: Symmetrical, Two Knights, 5. g3 a6'),
			moves => {
				'f1g2' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 1 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 a6 6. Bg2

			variation => N__('English: Symmetrical, Two Knights, 5. g3 a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 6' => {
			eco => 'B41i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6 6. Nc3

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3'),
			moves => {
				'd7d6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 7',
				'd8c7' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 3 7',
				'f8b4' => 'rnbqk2r/1p1p1ppp/p3pn2/8/1bPNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 3 7',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PNP3/3B4/PP3PPP/RNBQK2R b KQkq c3 0 6' => {
			eco => 'B42h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. c4

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6. c4'),
			moves => {
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'B41h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6

			variation => N__('Sicilian: Kan, 5. c4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R b KQkq - 2 6',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/3NP3/3B4/PPP2PPP/RNBQ1RK1 b kq - 3 6' => {
			eco => 'B42h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O'),
			moves => {
				'd7d6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/3B4/PPP2PPP/RNBQ1RK1 w kq - 0 7',
				'd8c7' => 'rnb1kb1r/1pqp1ppp/p3pn2/8/3NP3/3B4/PPP2PPP/RNBQ1RK1 w kq - 4 7',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B42h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6'),
			moves => {
				'c2c4' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PNP3/3B4/PP3PPP/RNBQK2R b KQkq c3 0 6',
				'e1g1' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/3NP3/3B4/PPP2PPP/RNBQ1RK1 b kq - 3 6',
			},
		},
		'rnbqkb1r/1p1p1ppp/p3pn2/8/3NP3/8/PPP1BPPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B41e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Be2 Nf6

			variation => N__('Sicilian: Kan, 5. Be2 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/1p1ppppp/p4n2/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 a6

			variation => N__('English: Symmetrical, Two Knights, 3... a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p1ppppp/p4n2/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A56g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 a6

			variation => N__('Benoni: 3. d5 a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p1ppppp/p4n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B28e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. d4 cxd4 4. Nxd4 Nf6

			variation => N__('Sicilian: O\'Kelly, 3. d4 cxd4 4.Nxd4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pp1p/p2p1np1/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B91c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 g6

			variation => N__('Sicilian: Najdorf, 6. g3 g6'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pp1p/p2p1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B70j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2 a6

			variation => N__('Sicilian: Dragon, 6. Be2 a6'),
			moves => {
				'a2a4' => 'rnbqkb1r/1p2pp1p/p2p1np1/8/P2NP3/2N5/1PP1BPPP/R1BQK2R b KQkq a3 0 7',
			},
		},
		'rnbqkb1r/1p2pp1p/p2p1np1/8/P2NP3/2N5/1PP1BPPP/R1BQK2R b KQkq a3 0 7' => {
			eco => 'B70j',
                        # TRANSLATORS:

			variation => N__('Sicilian: Dragon, 6. Be2 a6'),
			moves => {
				'f8g7' => 'rnbqk2r/1p2ppbp/p2p1np1/8/P2NP3/2N5/1PP1BPPP/R1BQK2R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p2pp2/p2p3p/6p1/3NP1n1/2N3B1/PPP2PPP/R2QKB1R b KQkq - 1 9' => {
			eco => 'B90h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5'),
			moves => {
				'f8g7' => 'rnbqk2r/1p2ppb1/p2p3p/6p1/3NP1n1/2N3B1/PPP2PPP/R2QKB1R w KQkq - 2 10',
			},
		},
		'rnbqkb1r/1p2pp2/p2p3p/6p1/3NP1nB/2N5/PPP2PPP/R2QKB1R w KQkq g6 0 9' => {
			eco => 'B90h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5'),
			moves => {
				'h4g3' => 'rnbqkb1r/1p2pp2/p2p3p/6p1/3NP1n1/2N3B1/PPP2PPP/R2QKB1R b KQkq - 1 9',
			},
		},
		'rnbqkb1r/1p2ppp1/p2p3p/6B1/3NP1n1/2N5/PPP2PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'B90h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5'),
			moves => {
				'g5h4' => 'rnbqkb1r/1p2ppp1/p2p3p/8/3NP1nB/2N5/PPP2PPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p2ppp1/p2p3p/8/3NP1nB/2N5/PPP2PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'B90h',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5'),
			moves => {
				'g7g5' => 'rnbqkb1r/1p2pp2/p2p3p/6p1/3NP1nB/2N5/PPP2PPP/R2QKB1R w KQkq g6 0 9',
			},
		},
		'rnbqkb1r/1p2pppp/2p2n2/p7/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq a6 0 6' => {
			eco => 'D16d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 a5

			variation => N__('Slav: Murey Variation'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/2Pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D15j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. c5

			variation => N__('Slav: Chameleon, 5. c5'),
			moves => {
				'b8d7' => 'r1bqkb1r/1p1npppp/p1p2n2/2Pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'D15e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. Bg5

			variation => N__('Slav: Chameleon, 5. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D10w',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. e3 a6

			variation => N__('Slav: 3. Nc3 Nf6 4. e3 a6'),
			moves => {
				'd1c2' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 1 5',
				'f1d3' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N1P3/PPQ2PPP/R1B1KBNR b KQkq - 1 5' => {
			eco => 'D10x',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. e3 a6 5. Qc2

			variation => N__('Slav: 3. Nc3 Nf6 4. e3 a6 5. Qc2'),
			moves => {
				'b7b5' => 'rnbqkb1r/4pppp/p1p2n2/1p1p4/2PP4/2N1P3/PPQ2PPP/R1B1KBNR w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D15g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. e3

			variation => N__('Slav: Chameleon, 5. e3'),
			moves => {
				'b7b5' => 'rnbqkb1r/4pppp/p1p2n2/1p1p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D15d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6

			variation => N__('Slav: Chameleon Variation'),
			moves => {
				'a2a4' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/P1PP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5',
				'c1g5' => 'rnbqkb1r/1p2pppp/p1p2n2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5',
				'c4c5' => 'rnbqkb1r/1p2pppp/p1p2n2/2Pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
				'e2e3' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'f3e5' => 'rnbqkb1r/1p2pppp/p1p2n2/3pN3/2PP4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2NBP3/PP3PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'D10w',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. e3 a6 5. Bd3

			variation => N__('Slav: 3. Nc3 Nf6 4. e3 a6 5. Bd3'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D11s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 a6

			variation => N__('Slav: 4. e3 a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3p4/P1PP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5' => {
			eco => 'D15l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. a4

			variation => N__('Slav: Chameleon, 5. a4'),
			moves => {
				'e7e6' => 'rnbqkb1r/1p3ppp/p1p1pn2/3p4/P1PP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/1p2pppp/p1p2n2/3pN3/2PP4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D15f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. Ne5

			variation => N__('Slav: Chameleon, 5. Ne5'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'B94a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5

			variation => N__('Sicilian: Najdorf, 6. Bg5'),
			moves => {
				'b7b5' => 'rnbqkb1r/4pppp/p2p1n2/1p4B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq b6 0 7',
				'b8d7' => 'r1bqkb1r/1p1npppp/p2p1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 2 7',
				'e7e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p1B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq e6 0 7',
				'e7e6' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B90f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bc4

			variation => N__('Sicilian: Najdorf, Fischer-Sozin Attack'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'B90g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3

			variation => N__('Sicilian: Najdorf, 6. Be3'),
			moves => {
				'e7e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq e6 0 7',
				'f6g4' => 'rnbqkb1r/1p2pppp/p2p4/8/3NP1n1/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N2P2/PPP3PP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B90d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f3

			variation => N__('Sicilian: Najdorf, 6. f3'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B91a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3

			variation => N__('Sicilian: Najdorf, 6. g3'),
			moves => {
				'b7b5' => 'rnbqkb1r/4pppp/p2p1n2/1p6/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq b6 0 7',
				'b8c6' => 'r1bqkb1r/1p2pppp/p1np1n2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7',
				'c8g4' => 'rn1qkb1r/1p2pppp/p2p1n2/8/3NP1b1/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7',
				'e7e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq e6 0 7',
				'g7g6' => 'rnbqkb1r/1p2pp1p/p2p1np1/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N4P/PPP2PP1/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B90e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. h3

			variation => N__('Sicilian: Najdorf, Adams Attack'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B92a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2

			variation => N__('Sicilian: Najdorf, 6. Be2'),
			moves => {
				'b8d7' => 'r1bqkb1r/1p1npppp/p2p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
				'e7e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq e6 0 7',
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B90a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6

			variation => N__('Sicilian: Najdorf'),
			moves => {
				'a2a4' => 'rnbqkb1r/1p2pppp/p2p1n2/8/P2NP3/2N5/1PP2PPP/R1BQKB1R b KQkq a3 0 6',
				'c1e3' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6',
				'c1g5' => 'rnbqkb1r/1p2pppp/p2p1n2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 1 6',
				'f1c4' => 'rnbqkb1r/1p2pppp/p2p1n2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6',
				'f1e2' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6',
				'f2f3' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N2P2/PPP3PP/R1BQKB1R b KQkq - 0 6',
				'f2f4' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6',
				'g2g3' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
				'h2h3' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N4P/PPP2PP1/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B54h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 a6

			variation => N__('Sicilian: Prins (Moscow), 5... a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6' => {
			eco => 'B93a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4

			variation => N__('Sicilian: Najdorf, 6. f4'),
			moves => {
				'b8c6' => 'r1bqkb1r/1p2pppp/p1np1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'b8d7' => 'r1bqkb1r/1p1npppp/p2p1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'd8c7' => 'rnb1kb1r/1pq1pppp/p2p1n2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'e7e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq e6 0 7',
			},
		},
		'rnbqkb1r/1p2pppp/p2p1n2/8/P2NP3/2N5/1PP2PPP/R1BQKB1R b KQkq a3 0 6' => {
			eco => 'B90b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. a4

			variation => N__('Sicilian: Najdorf, 6. a4'),
			moves => {
				'e7e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/P2NP3/2N5/1PP2PPP/R1BQKB1R w KQkq e6 0 7',
			},
		},
		'rnbqkb1r/1p2pppp/p2p4/6B1/3NP1n1/2N5/PPP2PPP/R2QKB1R b KQkq - 3 7' => {
			eco => 'B90h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 Ng4 7. Bg5

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4 7. Bg5'),
			moves => {
				'h7h6' => 'rnbqkb1r/1p2ppp1/p2p3p/6B1/3NP1n1/2N5/PPP2PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/1p2pppp/p2p4/8/3NP1n1/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7' => {
			eco => 'B90h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 Ng4

			variation => N__('Sicilian: Najdorf, 6. Be3 Ng4'),
			moves => {
				'e3g5' => 'rnbqkb1r/1p2pppp/p2p4/6B1/3NP1n1/2N5/PPP2PPP/R2QKB1R b KQkq - 3 7',
			},
		},
		'rnbqkb1r/1p2pppp/p4n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D13g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 a6 6. Bf4

			variation => N__('Slav: Exchange, 5. Nc3 a6 6. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/1p2pppp/p4n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D13e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 a6

			variation => N__('Slav: Exchange, 5. Nc3 a6'),
			moves => {
				'c1f4' => 'rnbqkb1r/1p2pppp/p4n2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
				'f3e5' => 'rnbqkb1r/1p2pppp/p4n2/3pN3/3P4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/1p2pppp/p4n2/3pN3/3P4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'D13f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 a6 6. Ne5

			variation => N__('Slav: Exchange, 5. Nc3 a6 6. Ne5'),
			moves => {
			},
		},
		'rnbqkb1r/1p3p1p/p2p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'A70b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 a6

			variation => N__('Benoni: Classical, 7... a6'),
			moves => {
				'a2a4' => 'rnbqkb1r/1p3p1p/p2p1np1/2pP4/P3P3/2N2N2/1P3PPP/R1BQKB1R b KQkq a3 0 8',
			},
		},
		'rnbqkb1r/1p3p1p/p2p1np1/2pP4/4PB2/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'A65d',
                        # TRANSLATORS:

			variation => N__('Benoni: 6. e4 g6 7. Bf4 a6'),
			moves => {
				'b7b5' => 'rnbqkb1r/5p1p/p2p1np1/1ppP4/4PB2/2N2N2/PP3PPP/R2QKB1R w KQkq b6 0 9',
			},
		},
		'rnbqkb1r/1p3p1p/p2p1np1/2pP4/4PB2/2N5/PP3PPP/R2QKBNR w KQkq - 0 8' => {
			eco => 'A65d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bf4 a6

			variation => N__('Benoni: 6. e4 g6 7. Bf4 a6'),
			moves => {
				'g1f3' => 'rnbqkb1r/1p3p1p/p2p1np1/2pP4/4PB2/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p3p1p/p2p1np1/2pP4/5B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bf4 a6

			variation => N__('Benoni: 6. Nf3 g6 7. Bf4 a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p3p1p/p2p1np1/2pP4/P3P3/2N2N2/1P3PPP/R1BQKB1R b KQkq a3 0 8' => {
			eco => 'A70b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 a6 8. a4

			variation => N__('Benoni: Classical, 7... a6 8. a4'),
			moves => {
				'c8g4' => 'rn1qkb1r/1p3p1p/p2p1np1/2pP4/P3P1b1/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 9',
			},
		},
		'rnbqkb1r/1p3p1p/p2ppnp1/8/2PNP3/3B4/PP3PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'B42k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O d6 7. c4 g6

			variation => N__('Sicilian: Kan, Gipslis Variation'),
			moves => {
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R w KQkq - 0 8' => {
			eco => 'B95n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qf3 h6

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qf3 h6'),
			moves => {
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 0 8' => {
			eco => 'B96b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 h6

			variation => N__('Sicilian: Najdorf, 7. f4 h6'),
			moves => {
				'g5h4' => 'rnbqkb1r/1p3pp1/p2ppn1p/8/3NPP1B/2N5/PPP3PP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/8/3NP1P1/2N1B3/PPP2P1P/R2QKB1R w KQkq - 0 8' => {
			eco => 'B81f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 a6 7. Be3 h6

			variation => N__('Sicilian: Scheveningen, Keres, Perenyi Attack, 7... h6'),
			moves => {
				'f2f4' => 'rnbqkb1r/1p3pp1/p2ppn1p/8/3NPPP1/2N1B3/PPP4P/R2QKB1R b KQkq f3 0 8',
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/8/3NP1P1/2N4P/PPP2P2/R1BQKB1R w KQkq - 0 8' => {
			eco => 'B81i',
                        # TRANSLATORS:

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h3'),
			moves => {
				'f1g2' => 'rnbqkb1r/1p3pp1/p2ppn1p/8/3NP1P1/2N4P/PPP2PB1/R1BQK2R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/8/3NP1P1/2N4P/PPP2PB1/R1BQK2R b KQkq - 1 8' => {
			eco => 'B81i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h3 a6 8. Bg2

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/8/3NPP1B/2N5/PPP3PP/R2QKB1R b KQkq - 1 8' => {
			eco => 'B96b',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, 7. f4 h6'),
			moves => {
				'd8b6' => 'rnb1kb1r/1p3pp1/pq1ppn1p/8/3NPP1B/2N5/PPP3PP/R2QKB1R w KQkq - 2 9',
			},
		},
		'rnbqkb1r/1p3pp1/p2ppn1p/8/3NPPP1/2N1B3/PPP4P/R2QKB1R b KQkq f3 0 8' => {
			eco => 'B81g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 a6 7. Be3 h6 8. f4

			variation => N__('Sicilian: Scheveningen, Keres, Perenyi Attack, 7... h6 8. f4'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p1p1pn2/3p2B1/P1PP4/2N2N2/1P2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D15n',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. a4 e6 6. Bg5

			variation => N__('Slav: Chameleon, 5. a4 e6 6. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p1p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D45e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 a6

			variation => N__('Semi-Slav: Accelerated Meran (Alekhine Variation)'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p1p1pn2/3p4/P1PP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D15m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. a4 e6

			variation => N__('Slav: Chameleon, 5. a4 e6'),
			moves => {
				'c1g5' => 'rnbqkb1r/1p3ppp/p1p1pn2/3p2B1/P1PP4/2N2N2/1P2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p1B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq e6 0 7' => {
			eco => 'B94c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e5

			variation => N__('Sicilian: Najdorf, 6. Bg5 e5'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP1P1/2N1B3/PPP2P1P/R2QKB1R w KQkq - 0 8' => {
			eco => 'B81e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 a6 7. Be3 e5

			variation => N__('Sicilian: Scheveningen, Keres, Perenyi Attack, 7... e5'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq e6 0 7' => {
			eco => 'B90k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5

			variation => N__('Sicilian: Najdorf, 6. Be3 e5'),
			moves => {
				'd4b3' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/1NN1B3/PPP2PPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq e6 0 7' => {
			eco => 'B91f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5

			variation => N__('Sicilian: Najdorf, 6. g3 e5'),
			moves => {
				'd4b3' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/1NN3P1/PPP2P1P/R1BQKB1R b KQkq - 1 7',
				'd4e2' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq e6 0 7' => {
			eco => 'B92c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5

			variation => N__('Sicilian: Najdorf, 6. Be2 e5'),
			moves => {
				'd4b3' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R b KQkq - 1 7',
				'd4f3' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq e6 0 7' => {
			eco => 'B93k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5

			variation => N__('Sicilian: Najdorf, 6. f4 e5'),
			moves => {
				'd4f3' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/1NN1B3/PPP2PPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B90l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be3 e5 7. Nb3

			variation => N__('Sicilian: Najdorf, 6. Be3 e5 7. Nb3'),
			moves => {
				'c8e6' => 'rn1qkb1r/1p3ppp/p2pbn2/4p3/4P3/1NN1B3/PPP2PPP/R2QKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/1NN3P1/PPP2P1P/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B91g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nb3

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7. Nb3'),
			moves => {
				'f8e7' => 'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN3P1/PPP2P1P/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B92e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nb3

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nb3'),
			moves => {
				'f8e7' => 'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/1NN5/PPP1BPPP/R1BQK2R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/2N2N2/PPP1BPPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B92d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Be2 e5 7. Nf3

			variation => N__('Sicilian: Najdorf, 6. Be2 e5 7. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B91i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nde2

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p2p1n2/1p2p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R w KQkq b6 0 8',
				'b8d7' => 'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R w KQkq - 2 8',
				'f8e7' => 'rnbqk2r/1p2bppp/p2p1n2/4p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/4PP2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'B93k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. f4 e5 7. Nf3

			variation => N__('Sicilian: Najdorf, 6. f4 e5 7. Nf3'),
			moves => {
				'b8d7' => 'r1bqkb1r/1p1n1ppp/p2p1n2/4p3/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 2 8',
				'd8c7' => 'rnb1kb1r/1pq2ppp/p2p1n2/4p3/4PP2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2p1n2/4p3/P2NP3/2N5/1PP2PPP/R1BQKB1R w KQkq e6 0 7' => {
			eco => 'B90c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. a4 e5

			variation => N__('Sicilian: Najdorf, 6. a4 e5'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B95j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qf3

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qf3'),
			moves => {
				'b8d7' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R w KQkq - 2 8',
				'f8e7' => 'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R w KQkq - 2 8',
				'h7h6' => 'rnbqkb1r/1p3pp1/p2ppn1p/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B95c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Be2

			variation => N__('Sicilian: Najdorf, 6... e6 7. Be2'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPP1QPPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B95d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qe2

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qe2'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B95a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6

			variation => N__('Sicilian: Najdorf, 6... e6'),
			moves => {
				'd1d2' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 1 7',
				'd1d3' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2NQ4/PPP2PPP/R3KB1R b KQkq - 1 7',
				'd1e2' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPP1QPPP/R3KB1R b KQkq - 1 7',
				'd1f3' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N2Q2/PPP2PPP/R3KB1R b KQkq - 1 7',
				'f1d3' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2NB4/PPP2PPP/R2QK2R b KQkq - 1 7',
				'f1e2' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPP1BPPP/R2QK2R b KQkq - 1 7',
				'f2f4' => 'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq f3 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B95e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qd2

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qd2'),
			moves => {
				'f8e7' => 'rnbqk2r/1p2bppp/p2ppn2/6B1/3NP3/2N5/PPPQ1PPP/R3KB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2NB4/PPP2PPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B95b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Bd3

			variation => N__('Sicilian: Najdorf, 6... e6 7. Bd3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NP3/2NQ4/PPP2PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B95h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. Qd3

			variation => N__('Sicilian: Najdorf, 6... e6 7. Qd3'),
			moves => {
				'b8c6' => 'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NP3/2NQ4/PPP2PPP/R3KB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R b KQkq f3 0 7' => {
			eco => 'B96a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4

			variation => N__('Sicilian: Najdorf, 7. f4'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p2ppn2/1p4B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq b6 0 8',
				'b8c6' => 'r1bqkb1r/1p3ppp/p1nppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8',
				'b8d7' => 'r1bqkb1r/1p1n1ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8',
				'c8d7' => 'rn1qkb1r/1p1b1ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8',
				'd8b6' => 'rnb1kb1r/1p3ppp/pq1ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8',
				'd8c7' => 'rnb1kb1r/1pq2ppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8',
				'f8e7' => 'rnbqk2r/1p2bppp/p2ppn2/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 1 8',
				'h7h6' => 'rnbqkb1r/1p3pp1/p2ppn1p/6B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B86',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6

			variation => N__('Sicilian: Sozin-Najdorf'),
			moves => {
				'c4b3' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/2PNP3/2N5/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B41j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4 Nf6 6. Nc3 d6

			variation => N__('Sicilian: Kan, 5. c4 Nf6 6. Nc3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/2PNP3/3B4/PP3PPP/RNBQ1RK1 b kq c3 0 7' => {
			eco => 'B42j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O d6 7. c4

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O d6 7. c4'),
			moves => {
				'g7g6' => 'rnbqkb1r/1p3p1p/p2ppnp1/8/2PNP3/3B4/PP3PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP1P1/2N1B3/PPP2P1P/R2QKB1R b KQkq - 1 7' => {
			eco => 'B81d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 a6 7. Be3

			variation => N__('Sicilian: Scheveningen, Keres, Perenyi Attack'),
			moves => {
				'e6e5' => 'rnbqkb1r/1p3ppp/p2p1n2/4p3/3NP1P1/2N1B3/PPP2P1P/R2QKB1R w KQkq - 0 8',
				'h7h6' => 'rnbqkb1r/1p3pp1/p2ppn1p/8/3NP1P1/2N1B3/PPP2P1P/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B81c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 a6

			variation => N__('Sicilian: Scheveningen, Keres, 6... a6'),
			moves => {
				'c1e3' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP1P1/2N1B3/PPP2P1P/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R b KQkq - 1 7' => {
			eco => 'B86',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/1BN5/PPP2PPP/R1BQK2R w KQkq b6 0 8',
				'f8e7' => 'rnbqk2r/1p2bppp/p2ppn2/8/3NP3/1BN5/PPP2PPP/R1BQK2R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 1 7' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. Be3

			variation => N__('Sicilian: Scheveningen, Classical, 7. Be3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'B80e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6

			variation => N__('Sicilian: Scheveningen, 6. Be3 a6'),
			moves => {
				'd1d2' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 1 7',
				'f2f3' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1B3/PPPQ1PPP/R3KB1R b KQkq - 1 7' => {
			eco => 'B80f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. Qd2

			variation => N__('Sicilian: Scheveningen, 7. Qd2'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1BP2/PPP3PP/R2QKB1R b KQkq - 0 7' => {
			eco => 'B80g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3

			variation => N__('Sicilian: Scheveningen, English Attack'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq b6 0 8',
				'b8c6' => 'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B80p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3 a6

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. O-O

			variation => N__('Sicilian: Scheveningen, Classical, 7.O-O'),
			moves => {
				'b8c6' => 'r1bqkb1r/1p3ppp/p1nppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8',
				'b8d7' => 'r1bqkb1r/1p1n1ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8',
				'd8c7' => 'rnb1kb1r/1pq2ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8',
				'f8e7' => 'rnbqk2r/1p2bppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 w kq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6

			variation => N__('Sicilian: Scheveningen, Classical'),
			moves => {
				'a2a4' => 'rnbqkb1r/1p3ppp/p2ppn2/8/P2NP3/2N5/1PP1BPPP/R1BQK2R b KQkq a3 0 7',
				'c1e3' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1B3/PPP1BPPP/R2QK2R b KQkq - 1 7',
				'e1g1' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQ1RK1 b kq - 1 7',
				'f2f4' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQK2R b KQkq f3 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/3B4/PPP2PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'B42i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 Nf6 6. O-O d6

			variation => N__('Sicilian: Kan, 5. Bd3 Nf6 6.O-O d6'),
			moves => {
				'c2c4' => 'rnbqkb1r/1p3ppp/p2ppn2/8/2PNP3/3B4/PP3PPP/RNBQ1RK1 b kq c3 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R b KQkq - 1 7' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 a6 7. Be3

			variation => N__('Sicilian: Scheveningen, 6. f4 a6 7. Be3'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p2ppn2/1p6/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq b6 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NPP2/2N5/PPP1B1PP/R1BQK2R b KQkq f3 0 7' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. f4

			variation => N__('Sicilian: Scheveningen, Classical, 7. f4'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 a6

			variation => N__('Sicilian: Scheveningen, 6. f4 a6'),
			moves => {
				'c1e3' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NPP2/2N1B3/PPP3PP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/1p3ppp/p2ppn2/8/P2NP3/2N5/1PP1BPPP/R1BQK2R b KQkq a3 0 7' => {
			eco => 'B84',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2 a6 7. a4

			variation => N__('Sicilian: Scheveningen, Classical, 7. a4'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP2QPPP/RNB1K2R b KQkq - 0 7' => {
			eco => 'D26l',
                        # TRANSLATORS:

			variation => N__('QGA: Classical, Furman, 6... a6'),
			moves => {
				'f8c5' => 'rnbqk2r/1p3ppp/p3pn2/2b5/2B5/4PN2/PP2QPPP/RNB1K2R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP3PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'D27l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. dxc5

			variation => N__('QGA: Classical, Spassky Variation'),
			moves => {
				'd8d1' => 'rnb1kb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP3PPP/RNBq1RK1 w kq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/1P2PN2/P4PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'D27b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. b3

			variation => N__('QGA: Classical, 6... a6 7. b3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQ1RK1 b kq - 1 7' => {
			eco => 'D27c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Nc3

			variation => N__('QGA: Classical, 6... a6 7. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQK2R w KQkq - 0 7' => {
			eco => 'D26i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Nc3 a6

			variation => N__('QGA: Classical, 6. Nc3 a6'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/4PN2/PP2QPPP/RNB1K2R w KQkq - 0 7' => {
			eco => 'D26l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Qe2 a6

			variation => N__('QGA: Classical, Furman, 6... a6'),
			moves => {
				'd4c5' => 'rnbqkb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP2QPPP/RNB1K2R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/4PN2/PP2QPPP/RNB2RK1 b kq - 1 7' => {
			eco => 'D28a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2

			variation => N__('QGA: Classical, 7. Qe2'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p3pn2/1pp5/2BP4/4PN2/PP2QPPP/RNB2RK1 w kq b6 0 8',
				'b8c6' => 'r1bqkb1r/1p3ppp/p1n1pn2/2p5/2BP4/4PN2/PP2QPPP/RNB2RK1 w kq - 2 8',
				'c5d4' => 'rnbqkb1r/1p3ppp/p3pn2/8/2Bp4/4PN2/PP2QPPP/RNB2RK1 w kq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/4PN2/PP3PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'D27a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6

			variation => N__('QGA: Classical, 6... a6'),
			moves => {
				'a2a3' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/P3PN2/1P3PPP/RNBQ1RK1 b kq - 0 7',
				'a2a4' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/P1BP4/4PN2/1P3PPP/RNBQ1RK1 b kq a3 0 7',
				'b1c3' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQ1RK1 b kq - 1 7',
				'b2b3' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/1P2PN2/P4PPP/RNBQ1RK1 b kq - 0 7',
				'c4b3' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/3P4/1B2PN2/PP3PPP/RNBQ1RK1 b kq - 1 7',
				'c4d3' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/3P4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 1 7',
				'd1e2' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/4PN2/PP2QPPP/RNB2RK1 b kq - 1 7',
				'd4c5' => 'rnbqkb1r/1p3ppp/p3pn2/2P5/2B5/4PN2/PP3PPP/RNBQ1RK1 b kq - 0 7',
				'e3e4' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BPP3/5N2/PP3PPP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/P3PN2/1P3PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'D27a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a3

			variation => N__('QGA: Classical, 6... a6 7. a3'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/2BPP3/5N2/PP3PPP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'D27k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. e4

			variation => N__('QGA: Classical, Geller Variation'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/3P4/1B2PN2/PP3PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'D27f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bb3

			variation => N__('QGA: Classical, 6... a6 7. Bb3'),
			moves => {
				'b7b5' => 'rnbqkb1r/5ppp/p3pn2/1pp5/3P4/1B2PN2/PP3PPP/RNBQ1RK1 w kq b6 0 8',
				'b8c6' => 'r1bqkb1r/1p3ppp/p1n1pn2/2p5/3P4/1B2PN2/PP3PPP/RNBQ1RK1 w kq - 2 8',
				'c5d4' => 'rnbqkb1r/1p3ppp/p3pn2/8/3p4/1B2PN2/PP3PPP/RNBQ1RK1 w kq - 0 8',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/3P4/3BPN2/PP3PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'D27d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bd3

			variation => N__('QGA: Classical, 6... a6 7. Bd3'),
			moves => {
				'b8d7' => 'r1bqkb1r/1p1n1ppp/p3pn2/2p5/3P4/3BPN2/PP3PPP/RNBQ1RK1 w kq - 2 8',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2p5/P1BP4/4PN2/1P3PPP/RNBQ1RK1 b kq a3 0 7' => {
			eco => 'D27p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. a4

			variation => N__('QGA: Classical, Rubinstein Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/1p3ppp/p1n1pn2/2p5/P1BP4/4PN2/1P3PPP/RNBQ1RK1 w kq - 1 8',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2pP4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D40e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 a6 6. cxd5

			variation => N__('QGD: Semi-Tarrasch, 5. e3 a6 6.cxd5'),
			moves => {
				'e6d5' => 'rnbqkb1r/1p3ppp/p4n2/2pp4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D40d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 a6

			variation => N__('QGD: Semi-Tarrasch, 5. e3 a6'),
			moves => {
				'c4d5' => 'rnbqkb1r/1p3ppp/p3pn2/2pP4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/8/2Bp4/4PN2/PP2QPPP/RNB2RK1 w kq - 0 8' => {
			eco => 'D28b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 cxd4

			variation => N__('QGA: Classical, 7. Qe2 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p3pn2/8/3p4/1B2PN2/PP3PPP/RNBQ1RK1 w kq - 0 8' => {
			eco => 'D27h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bb3 cxd4

			variation => N__('QGA: Classical, 6... a6 7. Bb3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p4n2/1N1pp1B1/2P5/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D50',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, Pillsbury, 6.Nxd4 e5'),
			moves => {
				'd1a4' => 'rnbqkb1r/1p3ppp/p4n2/1N1pp1B1/Q1P5/2N5/PP2PPPP/R3KB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/1p3ppp/p4n2/1N1pp1B1/Q1P5/2N5/PP2PPPP/R3KB1R b KQkq - 1 8' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. Nf3 cxd4 6. Nxd4 e5 7. Ndb5 a6 8. Qa4

			variation => N__('QGD: Semi-Tarrasch, Pillsbury, Krause Variation'),
			moves => {
			},
		},
		'rnbqkb1r/1p3ppp/p4n2/2pp4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D40e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 a6 6. cxd5 exd5

			variation => N__('QGD: Semi-Tarrasch, 5. e3 a6 6.cxd5 exd5'),
			moves => {
			},
		},
		'rnbqkb1r/1pp1pppp/p4n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D24h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6

			variation => N__('QGA: 4. Nc3 a6'),
			moves => {
				'a2a4' => 'rnbqkb1r/1pp1pppp/p4n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5',
				'e2e4' => 'rnbqkb1r/1pp1pppp/p4n2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5',
			},
		},
		'rnbqkb1r/1pp1pppp/p4n2/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D25c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 a6

			variation => N__('QGA: 4. e3 a6'),
			moves => {
			},
		},
		'rnbqkb1r/1pp1pppp/p4n2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'D24k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4

			variation => N__('QGA: Bogoljubow'),
			moves => {
				'b7b5' => 'rnbqkb1r/2p1pppp/p4n2/1p6/2pPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/1pp1pppp/p4n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5' => {
			eco => 'D24i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. a4

			variation => N__('QGA: 4. Nc3 a6 5. a4'),
			moves => {
				'b8c6' => 'r1bqkb1r/1pp1pppp/p1n2n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/1pp2ppp/p3pn2/8/2BP4/4PN2/PP3PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'D26d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 a6 6. O-O

			variation => N__('QGA: 4. e3 e6 5.Bxc4 a6 6.O-O'),
			moves => {
				'b7b5' => 'rnbqkb1r/2p2ppp/p3pn2/1p6/2BP4/4PN2/PP3PPP/RNBQ1RK1 w kq b6 0 7',
			},
		},
		'rnbqkb1r/1pp2ppp/p3pn2/8/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D26c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 a6

			variation => N__('QGA: 4. e3 e6 5.Bxc4 a6'),
			moves => {
				'e1g1' => 'rnbqkb1r/1pp2ppp/p3pn2/8/2BP4/4PN2/PP3PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqkb1r/1pp2ppp/p3pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'E04f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6 6. O-O

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O'),
			moves => {
				'b7b5' => 'rnbqkb1r/2p2ppp/p3pn2/1p6/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w kq b6 0 7',
				'b8c6' => 'r1bqkb1r/1pp2ppp/p1n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rnbqkb1r/1pp2ppp/p3pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'E04e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6

			variation => N__('Catalan: Open, 5. Nf3 a6'),
			moves => {
				'e1g1' => 'rnbqkb1r/1pp2ppp/p3pn2/8/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqkb1r/1ppp1ppp/p3pn2/8/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A13j',
                        # TRANSLATORS:

			variation => N__('English: 1... e6 2. Nf3 Nf6 3. g3'),
			moves => {
				'f1g2' => 'rnbqkb1r/1ppp1ppp/p3pn2/8/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/1ppp1ppp/p3pn2/8/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A13j',
                        # TRANSLATORS:

			variation => N__('English: 1... e6 2. Nf3 Nf6 3. g3'),
			moves => {
				'b7b5' => 'rnbqkb1r/2pp1ppp/p3pn2/1p6/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq b6 0 5',
			},
		},
		'rnbqkb1r/1ppp1ppp/p3pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'E10e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 a6 4. Nc3

			variation => N__('Neo-Indian: 3. Nf3 a6 4. Nc3'),
			moves => {
				'c7c5' => 'rnbqkb1r/1p1p1ppp/p3pn2/2p5/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/1ppp1ppp/p3pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'E10d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 a6

			variation => N__('Neo-Indian: 3. Nf3 a6'),
			moves => {
				'b1c3' => 'rnbqkb1r/1ppp1ppp/p3pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/1ppppppp/p4n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A50k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 a6

			variation => N__('Indian: 2. c4 a6'),
			moves => {
			},
		},
		'rnbqkb1r/2p1pppp/p4n2/1p2P3/2pP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D24k',
                        # TRANSLATORS:

			variation => N__('QGA: Bogoljubow'),
			moves => {
				'f6d5' => 'rnbqkb1r/2p1pppp/p7/1p1nP3/2pP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/2p1pppp/p4n2/1p6/2pPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq b6 0 6' => {
			eco => 'D24k',
                        # TRANSLATORS:

			variation => N__('QGA: Bogoljubow'),
			moves => {
				'e4e5' => 'rnbqkb1r/2p1pppp/p4n2/1p2P3/2pP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/2p1pppp/p7/1p1nP3/2pP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D24k',
                        # TRANSLATORS:

			variation => N__('QGA: Bogoljubow'),
			moves => {
				'a2a4' => 'rnbqkb1r/2p1pppp/p7/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R b KQkq a3 0 7',
			},
		},
		'rnbqkb1r/2p1pppp/p7/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R b KQkq a3 0 7' => {
			eco => 'D24l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4

			variation => N__('QGA: Bogoljubow, 7. a4'),
			moves => {
				'c7c6' => 'rnbqkb1r/4pppp/p1p5/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 0 8',
				'c8b7' => 'rn1qkb1r/1bp1pppp/p7/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 8',
				'd5b4' => 'rnbqkb1r/2p1pppp/p7/1p2P3/PnpP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 8',
				'd5c3' => 'rnbqkb1r/2p1pppp/p7/1p2P3/P1pP4/2n2N2/1P3PPP/R1BQKB1R w KQkq - 0 8',
				'e7e6' => 'rnbqkb1r/2p2ppp/p3p3/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/2p1pppp/p7/1p2P3/P1pP4/2P2N2/5PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D24q',
                        # TRANSLATORS:

			variation => N__('QGA: Bogoljubow, 7. a4 Nxc3'),
			moves => {
				'c8b7' => 'rn1qkb1r/1bp1pppp/p7/1p2P3/P1pP4/2P2N2/5PPP/R1BQKB1R w KQkq - 1 9',
				'd8d5' => 'rnb1kb1r/2p1pppp/p7/1p1qP3/P1pP4/2P2N2/5PPP/R1BQKB1R w KQkq - 1 9',
			},
		},
		'rnbqkb1r/2p1pppp/p7/1p2P3/P1pP4/2n2N2/1P3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D24q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 Nxc3

			variation => N__('QGA: Bogoljubow, 7. a4 Nxc3'),
			moves => {
				'b2c3' => 'rnbqkb1r/2p1pppp/p7/1p2P3/P1pP4/2P2N2/5PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rnbqkb1r/2p1pppp/p7/1p2P3/PnpP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D24p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 Nb4

			variation => N__('QGA: Bogoljubow, 7. a4 Nb4'),
			moves => {
			},
		},
		'rnbqkb1r/2p2ppp/p3p3/1p1nN3/2pP4/6P1/PP2PPBP/RNBQ1RK1 w kq - 2 8' => {
			eco => 'E04g',
                        # TRANSLATORS:

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O b5 7. Ne5'),
			moves => {
				'a2a4' => 'rnbqkb1r/2p2ppp/p3p3/1p1nN3/P1pP4/6P1/1P2PPBP/RNBQ1RK1 b kq a3 0 8',
			},
		},
		'rnbqkb1r/2p2ppp/p3p3/1p1nN3/P1pP4/6P1/1P2PPBP/RNBQ1RK1 b kq a3 0 8' => {
			eco => 'E04h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6 6. O-O b5 7. Ne5 Nd5 8. a4

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O b5 7. Ne5 Nd5 8. a4'),
			moves => {
			},
		},
		'rnbqkb1r/2p2ppp/p3p3/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D24m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 e6

			variation => N__('QGA: Bogoljubow, 7. a4 e6'),
			moves => {
			},
		},
		'rnbqkb1r/2p2ppp/p3pn2/1p2N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'E04g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6 6. O-O b5 7. Ne5

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O b5 7. Ne5'),
			moves => {
				'f6d5' => 'rnbqkb1r/2p2ppp/p3p3/1p1nN3/2pP4/6P1/PP2PPBP/RNBQ1RK1 w kq - 2 8',
			},
		},
		'rnbqkb1r/2p2ppp/p3pn2/1p6/2BP4/4PN2/PP3PPP/RNBQ1RK1 w kq b6 0 7' => {
			eco => 'D26e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 a6 6. O-O b5

			variation => N__('QGA: 4. e3 e6 5.Bxc4 a6 6.O-O b5'),
			moves => {
			},
		},
		'rnbqkb1r/2p2ppp/p3pn2/1p6/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w kq b6 0 7' => {
			eco => 'E04g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 a6 6. O-O b5

			variation => N__('Catalan: Open, 5. Nf3 a6 6.O-O b5'),
			moves => {
				'f3e5' => 'rnbqkb1r/2p2ppp/p3pn2/1p2N3/2pP4/6P1/PP2PPBP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'rnbqkb1r/2pp1ppp/p3pn2/1p6/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq b6 0 5' => {
			eco => 'A13k',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 Nf6 3. g3 a6 4. Bg2 b5

			variation => N__('English: Romanishin Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/3p1ppp/p3pn2/1PpP4/8/5P2/PP2P1PP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A57p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. f3 e6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. f3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/3p1ppp/pP2pn2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A57l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. b6 e6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. b6 e6'),
			moves => {
			},
		},
		'rnbqkb1r/3ppp1p/P4np1/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A58b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6 g6

			variation => N__('Benko Gambit: 5.bxa6 g6'),
			moves => {
			},
		},
		'rnbqkb1r/3ppp1p/p4np1/1PpP4/8/2N1P3/PP3PPP/R1BQKBNR b KQkq - 1 6' => {
			eco => 'A57n',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: 4.cxb5 a6 5. e3 g6'),
			moves => {
				'd7d6' => 'rnbqkb1r/4pp1p/p2p1np1/1PpP4/8/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 7',
				'f8g7' => 'rnbqk2r/3pppbp/p4np1/1PpP4/8/2N1P3/PP3PPP/R1BQKBNR w KQkq - 2 7',
			},
		},
		'rnbqkb1r/3ppp1p/p4np1/1PpP4/8/4P3/PP3PPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A57n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. e3 g6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. e3 g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/3ppp1p/p4np1/1PpP4/8/2N1P3/PP3PPP/R1BQKBNR b KQkq - 1 6',
			},
		},
		'rnbqkb1r/3ppp1p/p4np1/1PpP4/8/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'A57g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3 g6 5. cxb5 a6

			variation => N__('Benko Gambit: 4. Nf3 g6 5.cxb5 a6'),
			moves => {
			},
		},
		'rnbqkb1r/3ppppp/5n2/1NpP4/1p2P3/8/PP3PPP/R1BQKBNR b KQkq - 1 7' => {
			eco => 'A57s',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Zaitsev, 5...axb5 6. e4 b4'),
			moves => {
				'd7d6' => 'rnbqkb1r/4pppp/3p1n2/1NpP4/1p2P3/8/PP3PPP/R1BQKBNR w KQkq - 0 8',
			},
		},
		'rnbqkb1r/3ppppp/5n2/1ppP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6' => {
			eco => 'A57s',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Zaitsev, 5...axb5'),
			moves => {
				'b5b4' => 'rnbqkb1r/3ppppp/5n2/2pP4/1p2P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/3ppppp/5n2/1ppP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A57s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 axb5

			variation => N__('Benko Gambit: Zaitsev, 5...axb5'),
			moves => {
				'e2e4' => 'rnbqkb1r/3ppppp/5n2/1ppP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6',
			},
		},
		'rnbqkb1r/3ppppp/5n2/1ppP4/8/5P2/PP2P1PP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A57q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. f3 axb5

			variation => N__('Benko Gambit: 4.cxb5 a6 5. f3 axb5'),
			moves => {
			},
		},
		'rnbqkb1r/3ppppp/5n2/2pP4/1p2P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'A57s',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 axb5 6. e4 b4

			variation => N__('Benko Gambit: Zaitsev, 5...axb5 6. e4 b4'),
			moves => {
				'c3b5' => 'rnbqkb1r/3ppppp/5n2/1NpP4/1p2P3/8/PP3PPP/R1BQKBNR b KQkq - 1 7',
			},
		},
		'rnbqkb1r/3ppppp/P4n2/2pP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'A58a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. bxa6

			variation => N__('Benko Gambit: 5.bxa6'),
			moves => {
				'c8a6' => 'rn1qkb1r/3ppppp/b4n2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6',
				'g7g6' => 'rnbqkb1r/3ppp1p/P4np1/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/3ppppp/p4n2/1PpP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 5' => {
			eco => 'A57r',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3

			variation => N__('Benko Gambit: Zaitsev Variation'),
			moves => {
				'a6b5' => 'rnbqkb1r/3ppppp/5n2/1ppP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6',
				'd8a5' => 'rnb1kb1r/3ppppp/p4n2/qPpP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/3ppppp/p4n2/1PpP4/8/4P3/PP3PPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'A57m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. e3

			variation => N__('Benko Gambit: 4.cxb5 a6 5. e3'),
			moves => {
				'g7g6' => 'rnbqkb1r/3ppp1p/p4np1/1PpP4/8/4P3/PP3PPP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/3ppppp/p4n2/1PpP4/8/5P2/PP2P1PP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'A57o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. f3

			variation => N__('Benko Gambit: 4.cxb5 a6 5. f3'),
			moves => {
				'a6b5' => 'rnbqkb1r/3ppppp/5n2/1ppP4/8/5P2/PP2P1PP/RNBQKBNR w KQkq - 0 6',
				'e7e6' => 'rnbqkb1r/3p1ppp/p3pn2/1PpP4/8/5P2/PP2P1PP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/3ppppp/p4n2/1PpP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'A57h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6

			variation => N__('Benko Gambit: 4.cxb5 a6'),
			moves => {
				'b1c3' => 'rnbqkb1r/3ppppp/p4n2/1PpP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 5',
				'b5a6' => 'rnbqkb1r/3ppppp/P4n2/2pP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 5',
				'b5b6' => 'rnbqkb1r/3ppppp/pP3n2/2pP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 5',
				'e2e3' => 'rnbqkb1r/3ppppp/p4n2/1PpP4/8/4P3/PP3PPP/RNBQKBNR b KQkq - 0 5',
				'f2f3' => 'rnbqkb1r/3ppppp/p4n2/1PpP4/8/5P2/PP2P1PP/RNBQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/3ppppp/pP3n2/2pP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'A57i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. b6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. b6'),
			moves => {
				'd7d6' => 'rnbqkb1r/4pppp/pP1p1n2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6',
				'd8b6' => 'rnb1kb1r/3ppppp/pq3n2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6',
				'e7e6' => 'rnbqkb1r/3p1ppp/pP2pn2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/4pp1p/3p1n2/1NpP2p1/1p2PB2/8/PP3PPP/R2QKBNR w KQkq g6 0 9' => {
			eco => 'A57w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 axb5 6. e4 b4 7. Nb5 d6 8. Bf4 g5

			variation => N__('Benko Gambit: Zaitsev, 8. Bf4 g5'),
			moves => {
			},
		},
		'rnbqkb1r/4pp1p/p2p1np1/1PpP4/8/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'A57n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. e3 g6 6. Nc3 d6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. e3 g6 6. Nc3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/3p1n2/1NpP4/1p2P3/5N2/PP3PPP/R1BQKB1R b KQkq - 1 8' => {
			eco => 'A57t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 axb5 6. e4 b4 7. Nb5 d6 8. Nf3

			variation => N__('Benko Gambit: Zaitsev, 8. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/3p1n2/1NpP4/1p2P3/8/PP3PPP/R1BQKBNR w KQkq - 0 8' => {
			eco => 'A57s',
                        # TRANSLATORS:

			variation => N__('Benko Gambit: Zaitsev, 5...axb5 6. e4 b4'),
			moves => {
				'c1f4' => 'rnbqkb1r/4pppp/3p1n2/1NpP4/1p2PB2/8/PP3PPP/R2QKBNR b KQkq - 1 8',
				'f1c4' => 'rnbqkb1r/4pppp/3p1n2/1NpP4/1pB1P3/8/PP3PPP/R1BQK1NR b KQkq - 1 8',
				'g1f3' => 'rnbqkb1r/4pppp/3p1n2/1NpP4/1p2P3/5N2/PP3PPP/R1BQKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/4pppp/3p1n2/1NpP4/1p2PB2/8/PP3PPP/R2QKBNR b KQkq - 1 8' => {
			eco => 'A57v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 axb5 6. e4 b4 7. Nb5 d6 8. Bf4

			variation => N__('Benko Gambit: Zaitsev, 8. Bf4'),
			moves => {
				'g7g5' => 'rnbqkb1r/4pp1p/3p1n2/1NpP2p1/1p2PB2/8/PP3PPP/R2QKBNR w KQkq g6 0 9',
			},
		},
		'rnbqkb1r/4pppp/3p1n2/1NpP4/1pB1P3/8/PP3PPP/R1BQK1NR b KQkq - 1 8' => {
			eco => 'A57u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. Nc3 axb5 6. e4 b4 7. Nb5 d6 8. Bc4

			variation => N__('Benko Gambit: Zaitsev, Nescafe Frappe Attack'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/p1p2n2/1p1p4/2PP4/1PN1PN2/P4PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D15i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. e3 b5 6. b3

			variation => N__('Slav: Chameleon, 5. e3 b5 6. b3'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/p1p2n2/1p1p4/2PP4/2N1P3/PPQ2PPP/R1B1KBNR w KQkq b6 0 6' => {
			eco => 'D10y',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. e3 a6 5. Qc2 b5

			variation => N__('Slav: 3. Nc3 Nf6 4. e3 a6 5. Qc2 b5'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/p1p2n2/1p1p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq b6 0 6' => {
			eco => 'D15h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 a6 5. e3 b5

			variation => N__('Slav: Chameleon, 5. e3 b5'),
			moves => {
				'b2b3' => 'rnbqkb1r/4pppp/p1p2n2/1p1p4/2PP4/1PN1PN2/P4PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/4pppp/p1p5/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D24n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 a6 5. e4 b5 6. e5 Nd5 7. a4 c6

			variation => N__('QGA: Bogoljubow, 7. a4 c6'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/p2p1n2/1p4B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq b6 0 7' => {
			eco => 'B94b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 b5

			variation => N__('Sicilian: Najdorf, 6. Bg5 b5'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/p2p1n2/1p6/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq b6 0 7' => {
			eco => 'B91b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 b5

			variation => N__('Sicilian: Najdorf, 6. g3 b5'),
			moves => {
			},
		},
		'rnbqkb1r/4pppp/pP1p1n2/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'A57k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5 a6 5. b6 d6

			variation => N__('Benko Gambit: 4.cxb5 a6 5. b6 d6'),
			moves => {
			},
		},
		'rnbqkb1r/5p1p/p2p1np1/1ppP4/4PB2/2N2N2/PP2QPPP/R3KB1R b KQkq - 1 9' => {
			eco => 'A65e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bf4 a6 8. Nf3 b5 9. Qe2

			variation => N__('Benoni: 6. e4 g6 7. Bf4 a6 8. Nf3 b5 9. Qe2'),
			moves => {
			},
		},
		'rnbqkb1r/5p1p/p2p1np1/1ppP4/4PB2/2N2N2/PP3PPP/R2QKB1R w KQkq b6 0 9' => {
			eco => 'A65e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bf4 a6 8. Nf3 b5

			variation => N__('Benoni: 6. e4 g6 7. Bf4 a6 8. Nf3 b5'),
			moves => {
				'd1e2' => 'rnbqkb1r/5p1p/p2p1np1/1ppP4/4PB2/2N2N2/PP2QPPP/R3KB1R b KQkq - 1 9',
			},
		},
		'rnbqkb1r/5ppp/p2p1n2/1p2p3/4P3/2N3P1/PPP1NP1P/R1BQKB1R w KQkq b6 0 8' => {
			eco => 'B91j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. g3 e5 7. Nde2 b5

			variation => N__('Sicilian: Najdorf, 6. g3 e5 7.Nde2 b5'),
			moves => {
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p2P1B1/3N1P2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8' => {
			eco => 'B96n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5 8. e5

			variation => N__('Sicilian: Najdorf, Polugaevsky, 8. e5'),
			moves => {
				'd6e5' => 'rnbqkb1r/5ppp/p3pn2/1p2p1B1/3N1P2/2N5/PPP3PP/R2QKB1R w KQkq - 0 9',
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p4B1/3NPP2/2N5/PPP3PP/R2QKB1R w KQkq b6 0 8' => {
			eco => 'B96n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 a6 6. Bg5 e6 7. f4 b5

			variation => N__('Sicilian: Najdorf, Polugaevsky Variation'),
			moves => {
				'e4e5' => 'rnbqkb1r/5ppp/p2ppn2/1p2P1B1/3N1P2/2N5/PPP3PP/R2QKB1R b KQkq - 0 8',
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/1BN5/PPP2PPP/R1BQ1RK1 b kq - 1 8' => {
			eco => 'B87',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3 b5 8. O-O

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3 b5 8.O-O'),
			moves => {
				'f8e7' => 'rnbqk2r/4bppp/p2ppn2/1p6/3NP3/1BN5/PPP2PPP/R1BQ1RK1 w kq - 2 9',
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/1BN5/PPP2PPP/R1BQK2R w KQkq b6 0 8' => {
			eco => 'B87',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4 a6 7. Bb3 b5

			variation => N__('Sicilian: Sozin-Najdorf, 7. Bb3 b5'),
			moves => {
				'e1g1' => 'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/1BN5/PPP2PPP/R1BQ1RK1 b kq - 1 8',
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/2N1BP2/PPP3PP/R2QKB1R w KQkq b6 0 8' => {
			eco => 'B80j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 b5

			variation => N__('Sicilian: Scheveningen, English Attack, 7... b5'),
			moves => {
				'd1d2' => 'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p6/3NP3/2N1BP2/PPPQ2PP/R3KB1R b KQkq - 1 8' => {
			eco => 'B80k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3 a6 7. f3 b5 8. Qd2

			variation => N__('Sicilian: Scheveningen, English, 7... b5 8. Qd2'),
			moves => {
				'b8d7' => 'r1bqkb1r/3n1ppp/p2ppn2/1p6/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 2 9',
				'c8b7' => 'rn1qkb1r/1b3ppp/p2ppn2/1p6/3NP3/2N1BP2/PPPQ2PP/R3KB1R w KQkq - 2 9',
			},
		},
		'rnbqkb1r/5ppp/p2ppn2/1p6/3NPP2/2N1B3/PPP3PP/R2QKB1R w KQkq b6 0 8' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4 a6 7. Be3 b5

			variation => N__('Sicilian: Scheveningen, 6. f4 a6 7. Be3 b5'),
			moves => {
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1p2P1B1/3N4/2N5/PPP3PP/R2QKB1R b KQkq - 0 9' => {
			eco => 'B96n',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Polugaevsky, 8. e5'),
			moves => {
				'd8c7' => 'rnb1kb1r/2q2ppp/p3pn2/1p2P1B1/3N4/2N5/PPP3PP/R2QKB1R w KQkq - 1 10',
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1p2p1B1/3N1P2/2N5/PPP3PP/R2QKB1R w KQkq - 0 9' => {
			eco => 'B96n',
                        # TRANSLATORS:

			variation => N__('Sicilian: Najdorf, Polugaevsky, 8. e5'),
			moves => {
				'f4e5' => 'rnbqkb1r/5ppp/p3pn2/1p2P1B1/3N4/2N5/PPP3PP/R2QKB1R b KQkq - 0 9',
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1p6/3P4/3B1N2/PP2QPPP/RNB2RK1 b kq - 0 9' => {
			eco => 'D28g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bd3 cxd4 9. exd4

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bd3 cxd4 9.exd4'),
			moves => {
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1p6/3p4/3BPN2/PP2QPPP/RNB2RK1 w kq - 0 9' => {
			eco => 'D28f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bd3 cxd4

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bd3 cxd4'),
			moves => {
				'e3d4' => 'rnbqkb1r/5ppp/p3pn2/1p6/3P4/3B1N2/PP2QPPP/RNB2RK1 b kq - 0 9',
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1pp5/2BP4/4PN2/PP2QPPP/RNB2RK1 w kq b6 0 8' => {
			eco => 'D28d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5

			variation => N__('QGA: Classical, 7. Qe2 b5'),
			moves => {
				'c4b3' => 'rnbqkb1r/5ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNB2RK1 b kq - 1 8',
				'c4d3' => 'rnbqkb1r/5ppp/p3pn2/1pp5/3P4/3BPN2/PP2QPPP/RNB2RK1 b kq - 1 8',
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNB2RK1 b kq - 1 8' => {
			eco => 'D28h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bb3'),
			moves => {
				'b8c6' => 'r1bqkb1r/5ppp/p1n1pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNB2RK1 w kq - 2 9',
				'c8b7' => 'rn1qkb1r/1b3ppp/p3pn2/1pp5/3P4/1B2PN2/PP2QPPP/RNB2RK1 w kq - 2 9',
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1pp5/3P4/1B2PN2/PP3PPP/RNBQ1RK1 w kq b6 0 8' => {
			eco => 'D27g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Bb3 b5

			variation => N__('QGA: Classical, 6... a6 7. Bb3 b5'),
			moves => {
			},
		},
		'rnbqkb1r/5ppp/p3pn2/1pp5/3P4/3BPN2/PP2QPPP/RNB2RK1 b kq - 1 8' => {
			eco => 'D28e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bd3

			variation => N__('QGA: Classical, 7. Qe2 b5 8. Bd3'),
			moves => {
				'c5d4' => 'rnbqkb1r/5ppp/p3pn2/1p6/3p4/3BPN2/PP2QPPP/RNB2RK1 w kq - 0 9',
			},
		},
		'rnbqkb1r/p1p2ppp/1p2pn2/3p4/4P3/3P1N2/PPPN1PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C00t',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2 Nf6 4. Ngf3 b6

			variation => N__('French: KIA, Reversed Philidor, 4... b6'),
			moves => {
			},
		},
		'rnbqkb1r/p1p2ppp/4pn2/1p6/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq b6 0 6' => {
			eco => 'E04d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 b5

			variation => N__('Catalan: Open, 5. Nf3 b5'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/6B1/2PP4/5N2/PP2PPPP/RN1QKB1R b KQkq - 1 4' => {
			eco => 'E12v',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bg5

			variation => N__('Queen\'s Indian: 4. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/6B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 1 4' => {
			eco => 'A46j',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. c3 b6 4. Bg5

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. c3 b6 4. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'A46r',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 b6

			variation => N__('Torre Attack: 3... b6'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2P1P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6 4. e4

			variation => N__('English: Anglo-Queen\'s Indian, 4. e4'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P1P3/2N2N2/PP1P1PPP/R1BQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6

			variation => N__('English: Anglo-Queen\'s Indian'),
			moves => {
				'e2e4' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2P1P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 4',
				'g2g3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 b6 4. g3

			variation => N__('English: Anglo-Queen\'s Indian, 4. g3'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP1B2/5N2/PP2PPPP/RN1QKB1R b KQkq - 1 4' => {
			eco => 'E12b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Bf4

			variation => N__('Queen\'s Indian: Miles Variation'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP1B2/5N2/PP2PPPP/RN1QKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'E12w',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. Nc3

			variation => N__('Queen\'s Indian: 4. Nc3'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'E00e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 b6

			variation => N__('Queen\'s Pawn: Neo-Indian, 3... b6'),
			moves => {
				'e2e4' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E14a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. e3

			variation => N__('Queen\'s Indian: 4. e3'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'f8b4' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'E12a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6

			variation => N__('Queen\'s Indian'),
			moves => {
				'a2a3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 4',
				'b1c3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'c1f4' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP1B2/5N2/PP2PPPP/RN1QKB1R b KQkq - 1 4',
				'c1g5' => 'rnbqkb1r/p1pp1ppp/1p2pn2/6B1/2PP4/5N2/PP2PPPP/RN1QKB1R b KQkq - 1 4',
				'e2e3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'g2g3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E15a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3

			variation => N__('Queen\'s Indian: 4. g3'),
			moves => {
				'c8a6' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5',
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5',
				'f8b4' => 'rnbqk2r/p1pp1ppp/1p2pn2/8/1bPP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E12d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3

			variation => N__('Queen\'s Indian: Petrosian'),
			moves => {
				'c7c5' => 'rnbqkb1r/p2p1ppp/1p2pn2/2p5/2PP4/P4N2/1P2PPPP/RNBQKB1R w KQkq c6 0 5',
				'c8a6' => 'rn1qkb1r/p1pp1ppp/bp2pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 5',
				'c8b7' => 'rn1qkb1r/pbpp1ppp/1p2pn2/8/2PP4/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'E00e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 b6 4. e4

			variation => N__('Queen\'s Pawn: Neo-Indian, 3... b6'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4 e6 4. e3

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c7c5' => 'rnbqkb1r/p2p1ppp/1p2pn2/2p5/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4 e6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'e2e3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46j',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. c3 b6

			variation => N__('Indian: 2. Nf3 e6 3. c3 b6'),
			moves => {
				'c1g5' => 'rnbqkb1r/p1pp1ppp/1p2pn2/6B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 e6 4. Bd3

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c7c5' => 'rnbqkb1r/p2p1ppp/1p2pn2/2p5/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 e6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'f1d3' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. g3 e6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/4pn2/1p6/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq b6 0 4' => {
			eco => 'A46m',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 b5

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 b5'),
			moves => {
				'f1g2' => 'rnbqkb1r/p1pp1ppp/4pn2/1p6/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/p1pp1ppp/4pn2/1p6/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A46m',
                        # TRANSLATORS:

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 b5'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpp1ppp/4pn2/1p6/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/p1pp1ppp/5n2/1p2p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq b6 0 4' => {
			eco => 'C26l',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 b5

			variation => N__('Vienna: Horwitz Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/p1pp1ppp/5n2/1p4N1/2P1p3/2N5/PP1PPPPP/R1BQKB1R w KQkq b6 0 5' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. Nf3 e4 4. Ng5 b5

			variation => N__('English: Bellon Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/p1ppp1pp/1p3n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'A83b',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 b6

			variation => N__('Dutch: Staunton Gambit, Nimzowitsch Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p1pppp1p/5np1/1p1P4/2P5/8/PP2PPPP/RNBQKBNR w KQkq b6 0 4' => {
			eco => 'E60c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. d5 b5

			variation => N__('King\'s Indian: 3. d5, Danube/Adorjan Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bg5

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpppppp/1p3n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A50l',
                        # TRANSLATORS:

			variation => N__('Indian: Queen\'s Indian Accelerated'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpppppp/1p3n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A50l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 b6

			variation => N__('Indian: Queen\'s Indian Accelerated'),
			moves => {
				'b1c3' => 'rnbqkb1r/p1pppppp/1p3n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpppppp/1p3n2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
				'e7e6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpppppp/1p3n2/8/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
				'e7e6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c1f4' => 'rnbqkb1r/p1pppppp/1p3n2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'c1g5' => 'rnbqkb1r/p1pppppp/1p3n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'e2e3' => 'rnbqkb1r/p1pppppp/1p3n2/8/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkb1r/p1pppppp/1p3n2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. g3

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpppppp/1p3n2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4',
				'e7e6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 b6

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'rnbqkb1r/p1pppppp/1p3n2/8/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkb1r/p1pppppp/1p3n2/8/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 b6 3. Bg2

			variation => N__('Reti: KIA'),
			moves => {
				'c8b7' => 'rn1qkb1r/pbpppppp/1p3n2/8/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/p1pppppp/5n2/1p6/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq b6 0 3' => {
			eco => 'A46b',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b5

			variation => N__('Indian: 2. Nf3 b5'),
			moves => {
				'g2g3' => 'rnbqkb1r/p1pppppp/5n2/1p6/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/p1pppppp/5n2/1p6/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46b',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b5 3. g3

			variation => N__('Indian: 2. Nf3 b5 3. g3'),
			moves => {
			},
		},
		'rnbqkb1r/p1pppppp/5n2/1p6/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq b6 0 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 b5

			variation => N__('Reti: KIA, Spassky'),
			moves => {
				'f1g2' => 'rnbqkb1r/p1pppppp/5n2/1p6/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkb1r/p1pppppp/5n2/1p6/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 b5 3. Bg2

			variation => N__('Reti: KIA, Spassky'),
			moves => {
			},
		},
		'rnbqkb1r/p1pppppp/8/1p1nP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq b6 0 4' => {
			eco => 'B03a',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 b5

			variation => N__('Alekhine: O\'Sullivan Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/p2n1ppp/1p2p3/2ppP3/3P4/2PB4/PP1N1PPP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'C05o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3 c5 6. c3 b6

			variation => N__('French: Tarrasch, Botvinnik'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1pp1/4pn1p/1ppP2B1/2P5/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 6' => {
			eco => 'E10n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. Bg5 h6

			variation => N__('Blumenfeld: 5. Bg5 h6'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1pp1/5n1p/1ppP2B1/8/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 7' => {
			eco => 'E10q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. Bg5 exd5 6. cxd5 h6

			variation => N__('Blumenfeld: Spielmann Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1ppp/1p2p3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B22x',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 e6 6. Nf3 b6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 e6 6. Nf3 b6'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1ppp/1p2pn2/2p5/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'E10i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. e3 b6

			variation => N__('Neo-Indian: Blumenfeld/Benoni, 4. e3 b6'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1ppp/1p2pn2/2p5/2PP4/P4N2/1P2PPPP/RNBQKB1R w KQkq c6 0 5' => {
			eco => 'E12e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 c5

			variation => N__('Queen\'s Indian: Petrosian, 4... c5'),
			moves => {
				'd4d5' => 'rnbqkb1r/p2p1ppp/1p2pn2/2pP4/2P5/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/p2p1ppp/1p2pn2/2p5/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq c6 0 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. Bf4 e6 4. e3 c5

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1ppp/1p2pn2/2p5/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq c6 0 5' => {
			eco => 'A47',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 b6 3. e3 e6 4. Bd3 c5

			variation => N__('Neo-Queen\'s Indian'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1ppp/1p2pn2/2pP4/2P5/P4N2/1P2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'E12e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. a3 c5 5. d5

			variation => N__('Queen\'s Indian: Petrosian, 4... c5 5. d5'),
			moves => {
				'c8a6' => 'rn1qkb1r/p2p1ppp/bp2pn2/2pP4/2P5/P4N2/1P2PPPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/p2p1ppp/4Pn2/1pp5/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'E10k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. dxe6

			variation => N__('Blumenfeld: 5.dxe6'),
			moves => {
				'f7e6' => 'rnbqkb1r/p2p2pp/4pn2/1pp5/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/p2p1ppp/4pn2/1ppP2B1/2P5/5N2/PP2PPPP/RN1QKB1R b KQkq - 1 5' => {
			eco => 'E10m',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. Bg5

			variation => N__('Blumenfeld: 5. Bg5'),
			moves => {
				'd8a5' => 'rnb1kb1r/p2p1ppp/4pn2/qppP2B1/2P5/5N2/PP2PPPP/RN1QKB1R w KQkq - 2 6',
				'e6d5' => 'rnbqkb1r/p2p1ppp/5n2/1ppp2B1/2P5/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 6',
				'h7h6' => 'rnbqkb1r/p2p1pp1/4pn1p/1ppP2B1/2P5/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/p2p1ppp/4pn2/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq b6 0 5' => {
			eco => 'E10k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5

			variation => N__('Blumenfeld Countergambit'),
			moves => {
				'c1g5' => 'rnbqkb1r/p2p1ppp/4pn2/1ppP2B1/2P5/5N2/PP2PPPP/RN1QKB1R b KQkq - 1 5',
				'd5e6' => 'rnbqkb1r/p2p1ppp/4Pn2/1pp5/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/p2p1ppp/5n2/1ppP2B1/8/5N2/PP2PPPP/RN1QKB1R b KQkq - 0 6' => {
			eco => 'E10p',
                        # TRANSLATORS:

			variation => N__('Blumenfeld: 5. Bg5 exd5'),
			moves => {
				'h7h6' => 'rnbqkb1r/p2p1pp1/5n1p/1ppP2B1/8/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/p2p1ppp/5n2/1ppP4/8/6P1/PP2PP1P/RNBQKBNR w KQkq b6 0 6' => {
			eco => 'A60f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. g3 exd5 5. cxd5 b5

			variation => N__('Benoni: 4. g3 exd5 5.cxd5 b5'),
			moves => {
			},
		},
		'rnbqkb1r/p2p1ppp/5n2/1ppp2B1/2P5/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 6' => {
			eco => 'E10p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. Bg5 exd5

			variation => N__('Blumenfeld: 5. Bg5 exd5'),
			moves => {
				'c4d5' => 'rnbqkb1r/p2p1ppp/5n2/1ppP2B1/8/5N2/PP2PPPP/RN1QKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/p2p2pp/4pn2/1Pp5/8/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'E10k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. dxe6 fxe6 6. cxb5

			variation => N__('Blumenfeld: 5.dxe6 fxe6 6.cxb5'),
			moves => {
				'd7d5' => 'rnbqkb1r/p5pp/4pn2/1Ppp4/8/5N2/PP2PPPP/RNBQKB1R w KQkq d6 0 7',
			},
		},
		'rnbqkb1r/p2p2pp/4pn2/1pp5/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'E10k',
                        # TRANSLATORS:

			variation => N__('Blumenfeld: 5.dxe6'),
			moves => {
				'c4b5' => 'rnbqkb1r/p2p2pp/4pn2/1Pp5/8/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/p2ppp1p/5np1/1PpP4/8/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A57g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3 g6 5. cxb5

			variation => N__('Benko Gambit: 4. Nf3 g6 5.cxb5'),
			moves => {
				'a7a6' => 'rnbqkb1r/3ppp1p/p4np1/1PpP4/8/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/p2ppp1p/5np1/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A57f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3 g6

			variation => N__('Benko Gambit: 4. Nf3 g6'),
			moves => {
				'c4b5' => 'rnbqkb1r/p2ppp1p/5np1/1PpP4/8/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/p2ppppp/1p3n2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 b6

			variation => N__('English: Symmetrical, 3 Knights, 3... b6'),
			moves => {
			},
		},
		'rnbqkb1r/p2ppppp/1p3n2/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A30e',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3 b6

			variation => N__('English: Symmetrical, b6 System'),
			moves => {
				'f1g2' => 'rnbqkb1r/p2ppppp/1p3n2/2p5/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/p2ppppp/1p3n2/2p5/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A30e',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, b6 System'),
			moves => {
				'c8b7' => 'rn1qkb1r/pb1ppppp/1p3n2/2p5/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/p2ppppp/1p3n2/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46d',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. c3 b6

			variation => N__('Neo-Benoni: 3. c3 b6'),
			moves => {
			},
		},
		'rnbqkb1r/p2ppppp/1p3n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A31',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 4... b6'),
			moves => {
				'c8b7' => 'rn1qkb1r/pb1ppppp/1p3n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/p2ppppp/1p3n2/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 b6

			variation => N__('English: Symmetrical, Two Knights, 4... b6'),
			moves => {
				'b1c3' => 'rnbqkb1r/p2ppppp/1p3n2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1PpP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'A57h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. cxb5

			variation => N__('Benko Gambit: 4.cxb5'),
			moves => {
				'a7a6' => 'rnbqkb1r/3ppppp/p4n2/1PpP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP2B1/8/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 4' => {
			eco => 'A43o',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 b5 4. Bg5

			variation => N__('Old Benoni: Neo-Benko, 4. Bg5'),
			moves => {
				'd7d6' => 'rnbqkb1r/p3pppp/3p1n2/1ppP2B1/8/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 5',
				'f6e4' => 'rnbqkb1r/p2ppppp/8/1ppP2B1/4n3/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A57d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3

			variation => N__('Benko Gambit: 4. Nf3'),
			moves => {
				'b5c4' => 'rnbqkb1r/p2ppppp/5n2/2pP4/2p5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
				'c8b7' => 'rn1qkb1r/pb1ppppp/5n2/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 5',
				'g7g6' => 'rnbqkb1r/p2ppp1p/5np1/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP4/2P5/8/PP1NPPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A57c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nd2

			variation => N__('Benko Gambit: 4. Nd2'),
			moves => {
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq b6 0 4' => {
			eco => 'A57a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5

			variation => N__('Benko Gambit'),
			moves => {
				'a2a4' => 'rnbqkb1r/p2ppppp/5n2/1ppP4/P1P5/8/1P2PPPP/RNBQKBNR b KQkq a3 0 4',
				'b1d2' => 'rnbqkb1r/p2ppppp/5n2/1ppP4/2P5/8/PP1NPPPP/R1BQKBNR b KQkq - 1 4',
				'c4b5' => 'rnbqkb1r/p2ppppp/5n2/1PpP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkb1r/p2ppppp/5n2/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq b6 0 4' => {
			eco => 'A43n',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 b5

			variation => N__('Old Benoni: Neo-Benko'),
			moves => {
				'c1g5' => 'rnbqkb1r/p2ppppp/5n2/1ppP2B1/8/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP4/8/6P1/PPP1PP1P/RNBQKBNR w KQkq b6 0 4' => {
			eco => 'A45i',
                        # TRANSLATORS: 1. d4 Nf6 2. g3 c5 3. d5 b5

			variation => N__('Indian: 2. g3 c5 3. d5 b5'),
			moves => {
			},
		},
		'rnbqkb1r/p2ppppp/5n2/1ppP4/P1P5/8/1P2PPPP/RNBQKBNR b KQkq a3 0 4' => {
			eco => 'A57b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. a4

			variation => N__('Benko Gambit: 4. a4'),
			moves => {
			},
		},
		'rnbqkb1r/p2ppppp/5n2/2pP4/2p5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A57d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 b5 4. Nf3 bxc4

			variation => N__('Benko Gambit: 4. Nf3 bxc4'),
			moves => {
			},
		},
		'rnbqkb1r/p2ppppp/8/1ppP2B1/4n3/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 5' => {
			eco => 'A43q',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 b5 4. Bg5 Ne4

			variation => N__('Old Benoni: Neo-Benko, 4. Bg5 Ne4'),
			moves => {
			},
		},
		'rnbqkb1r/p3pp1p/3p1np1/1ppP4/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq b6 0 6' => {
			eco => 'A56j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 g6 4. Nc3 d6 5. e4 b5

			variation => N__('Benoni: Bronstein\'s Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/1p2P3/2pP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D15v',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4 b5 6. e5

			variation => N__('Slav: Geller Gambit, 6. e5'),
			moves => {
				'f6d5' => 'rnbqkb1r/p3pppp/2p5/1p1nP3/2pP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/1p6/2pP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq b6 0 6' => {
			eco => 'D15q',
                        # TRANSLATORS:

			variation => N__('Slav: Accepted, Alekhine'),
			moves => {
				'a2a4' => 'rnbqkb1r/p3pppp/2p2n2/1p6/P1pP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq a3 0 6',
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/1p6/2pPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq b6 0 6' => {
			eco => 'D15t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4 b5

			variation => N__('Slav: Geller Gambit'),
			moves => {
				'd1c2' => 'rnbqkb1r/p3pppp/2p2n2/1p6/2pPP3/2N2N2/PPQ2PPP/R1B1KB1R b KQkq - 1 6',
				'e4e5' => 'rnbqkb1r/p3pppp/2p2n2/1p2P3/2pP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/1p6/2pPP3/2N2N2/PPQ2PPP/R1B1KB1R b KQkq - 1 6' => {
			eco => 'D15u',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4 b5 6. Qc2

			variation => N__('Slav: Geller Gambit, Spassky Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/1p6/P1pP4/2N1PN2/1P3PPP/R1BQKB1R b KQkq a3 0 6' => {
			eco => 'D15q',
                        # TRANSLATORS:

			variation => N__('Slav: Accepted, Alekhine'),
			moves => {
				'b5b4' => 'rnbqkb1r/p3pppp/2p2n2/8/PppP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/8/PppP4/2N1PN2/1P3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D15r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e3 b5 6. a4 b4

			variation => N__('Slav: Accepted, Alekhine: 5... b5 6. a4 b4'),
			moves => {
				'c3b1' => 'rnbqkb1r/p3pppp/2p2n2/8/PppP4/4PN2/1P3PPP/RNBQKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/p3pppp/2p2n2/8/PppP4/4PN2/1P3PPP/RNBQKB1R b KQkq - 1 7' => {
			eco => 'D15s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e3 b5 6. a4 b4 7. Nb1

			variation => N__('Slav: Accepted, Alekhine: 5... b5 6. a4 b4 7. Nb1'),
			moves => {
			},
		},
		'rnbqkb1r/p3pppp/2p5/1p1nP1N1/2pP4/2N5/PP3PPP/R1BQKB1R b KQkq - 2 7' => {
			eco => 'D15w',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4 b5 6. e5 Nd5 7. Ng5

			variation => N__('Slav: Geller Gambit, 6. e5 Nd5 7. Ng5'),
			moves => {
			},
		},
		'rnbqkb1r/p3pppp/2p5/1p1nP3/2pP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 7' => {
			eco => 'D15v',
                        # TRANSLATORS:

			variation => N__('Slav: Geller Gambit, 6. e5'),
			moves => {
				'a2a4' => 'rnbqkb1r/p3pppp/2p5/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R b KQkq a3 0 7',
				'f3g5' => 'rnbqkb1r/p3pppp/2p5/1p1nP1N1/2pP4/2N5/PP3PPP/R1BQKB1R b KQkq - 2 7',
			},
		},
		'rnbqkb1r/p3pppp/2p5/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R b KQkq a3 0 7' => {
			eco => 'D15x',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4 b5 6. e5 Nd5 7. a4

			variation => N__('Slav: Geller Gambit, 6. e5 Nd5 7. a4'),
			moves => {
				'e7e6' => 'rnbqkb1r/p4ppp/2p1p3/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/p3pppp/3p1n2/1ppP2B1/8/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A43p',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 b5 4. Bg5 d6

			variation => N__('Old Benoni: Neo-Benko, 4. Bg5 d6'),
			moves => {
			},
		},
		'rnbqkb1r/p3pppp/5n2/1ppP4/2p1P3/2N5/PP3PPP/R1BQKBNR w KQkq b6 0 6' => {
			eco => 'D20u',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 c5 4. d5 Nf6 5. Nc3 b5

			variation => N__('QGA: Linares Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p4p2/2p1p2p/1p1nP1N1/2pP3B/2N5/PP3PPP/R2QKB1R w KQkq - 1 10' => {
			eco => 'D44i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 Nd5

			variation => N__('Semi-Slav: Botvinnik, Alatortsev Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p4p2/2p1pP1p/1p2N3/2pP3p/2N5/PP3PPP/R2QKB1R b KQkq - 1 10' => {
			eco => 'D44g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. exf6 gxh4 10. Ne5

			variation => N__('Semi-Slav: Botvinnik, Ekstrom Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p4p2/2p1pP1p/1p4p1/2pP3B/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 9' => {
			eco => 'D44f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. exf6

			variation => N__('Semi-Slav: Botvinnik, 9.exf6'),
			moves => {
				'g5h4' => 'rnbqkb1r/p4p2/2p1pP1p/1p6/2pP3p/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 10',
			},
		},
		'rnbqkb1r/p4p2/2p1pP1p/1p6/2pP3p/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'D44f',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, 9.exf6'),
			moves => {
				'f3e5' => 'rnbqkb1r/p4p2/2p1pP1p/1p2N3/2pP3p/2N5/PP3PPP/R2QKB1R b KQkq - 1 10',
			},
		},
		'rnbqkb1r/p4p2/2p1pn1p/1p2P1N1/2pP3B/2N5/PP3PPP/R2QKB1R b KQkq - 0 9' => {
			eco => 'D44h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5

			variation => N__('Semi-Slav: Botvinnik, 9.Nxg5'),
			moves => {
				'f6d5' => 'rnbqkb1r/p4p2/2p1p2p/1p1nP1N1/2pP3B/2N5/PP3PPP/R2QKB1R w KQkq - 1 10',
				'h6g5' => 'rnbqkb1r/p4p2/2p1pn2/1p2P1p1/2pP3B/2N5/PP3PPP/R2QKB1R w KQkq - 0 10',
			},
		},
		'rnbqkb1r/p4p2/2p1pn1p/1p2P1p1/2pP3B/2N2N2/PP3PPP/R2QKB1R w KQkq g6 0 9' => {
			eco => 'D44e',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, 7. e5'),
			moves => {
				'e5f6' => 'rnbqkb1r/p4p2/2p1pP1p/1p4p1/2pP3B/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 9',
				'f3g5' => 'rnbqkb1r/p4p2/2p1pn1p/1p2P1N1/2pP3B/2N5/PP3PPP/R2QKB1R b KQkq - 0 9',
			},
		},
		'rnbqkb1r/p4p2/2p1pn1p/1p4p1/2pPP3/2N2NB1/PP2BPPP/R2QK2R b KQkq - 1 9' => {
			eco => 'D43m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4 dxc4 7. e4 g5 8. Bg3 b5 9. Be2

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit, Main Line'),
			moves => {
				'c8b7' => 'rn1qkb1r/pb3p2/2p1pn1p/1p4p1/2pPP3/2N2NB1/PP2BPPP/R2QK2R w KQkq - 2 10',
			},
		},
		'rnbqkb1r/p4p2/2p1pn1p/1p4p1/2pPP3/2N2NB1/PP3PPP/R2QKB1R w KQkq b6 0 9' => {
			eco => 'D43l',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit, 7. e4'),
			moves => {
				'f1e2' => 'rnbqkb1r/p4p2/2p1pn1p/1p4p1/2pPP3/2N2NB1/PP2BPPP/R2QK2R b KQkq - 1 9',
			},
		},
		'rnbqkb1r/p4p2/2p1pn2/1p2P1B1/2pP4/2N5/PP3PPP/R2QKB1R b KQkq - 0 10' => {
			eco => 'D44j',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, 9.Nxg5'),
			moves => {
				'b8d7' => 'r1bqkb1r/p2n1p2/2p1pn2/1p2P1B1/2pP4/2N5/PP3PPP/R2QKB1R w KQkq - 1 11',
			},
		},
		'rnbqkb1r/p4p2/2p1pn2/1p2P1p1/2pP3B/2N5/PP3PPP/R2QKB1R w KQkq - 0 10' => {
			eco => 'D44j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5 h6 8. Bh4 g5 9. Nxg5 hxg5

			variation => N__('Semi-Slav: Botvinnik, 9.Nxg5'),
			moves => {
				'h4g5' => 'rnbqkb1r/p4p2/2p1pn2/1p2P1B1/2pP4/2N5/PP3PPP/R2QKB1R b KQkq - 0 10',
			},
		},
		'rnbqkb1r/p4pp1/2p1pn1p/1p2P1B1/2pP4/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'D44e',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, 7. e5'),
			moves => {
				'g5h4' => 'rnbqkb1r/p4pp1/2p1pn1p/1p2P3/2pP3B/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/p4pp1/2p1pn1p/1p2P3/2pP3B/2N2N2/PP3PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'D44e',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Botvinnik, 7. e5'),
			moves => {
				'g7g5' => 'rnbqkb1r/p4p2/2p1pn1p/1p2P1p1/2pP3B/2N2N2/PP3PPP/R2QKB1R w KQkq g6 0 9',
			},
		},
		'rnbqkb1r/p4ppp/2p1p3/1p1nP3/P1pP4/2N2N2/1P3PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D15y',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4 b5 6. e5 Nd5 7. a4 e6

			variation => N__('Slav: Geller Gambit, 6. e5 Nd5 7. a4 e6'),
			moves => {
			},
		},
		'rnbqkb1r/p4ppp/2p1pn2/1p2P1B1/2pP4/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 7' => {
			eco => 'D44e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. e5

			variation => N__('Semi-Slav: Botvinnik, 7. e5'),
			moves => {
				'h7h6' => 'rnbqkb1r/p4pp1/2p1pn1p/1p2P1B1/2pP4/2N2N2/PP3PPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/p4ppp/2p1pn2/1p4B1/2pPP3/2N2N2/PP3PPP/R2QKB1R w KQkq b6 0 7' => {
			eco => 'D44c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5

			variation => N__('Semi-Slav: Botvinnik, 6. e4'),
			moves => {
				'a2a4' => 'rnbqkb1r/p4ppp/2p1pn2/1p4B1/P1pPP3/2N2N2/1P3PPP/R2QKB1R b KQkq a3 0 7',
				'e4e5' => 'rnbqkb1r/p4ppp/2p1pn2/1p2P1B1/2pP4/2N2N2/PP3PPP/R2QKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/p4ppp/2p1pn2/1p4B1/P1pPP3/2N2N2/1P3PPP/R2QKB1R b KQkq a3 0 7' => {
			eco => 'D44d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4 b5 7. a4

			variation => N__('Semi-Slav: Botvinnik, 7. a4'),
			moves => {
			},
		},
		'rnbqkb1r/p4ppp/2p2n2/1B6/5p2/5N2/PPPP2PP/RNBQK2R w KQkq - 0 7' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Bb5+ c6 6. dxc6 bxc6

			variation => N__('KGA: Scandinavian, Modern, 5. Bb5+ c6 6.dxc6 bxc6'),
			moves => {
				'b5c4' => 'rnbqkb1r/p4ppp/2p2n2/8/2B2p2/5N2/PPPP2PP/RNBQK2R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/p4ppp/2p2n2/8/2B2p2/5N2/PPPP2PP/RNBQK2R b KQkq - 1 7' => {
			eco => 'C36',
                        # TRANSLATORS:

			variation => N__('KGA: Scandinavian, Modern, 5. Bb5+ c6 6.dxc6 bxc6'),
			moves => {
				'f6d5' => 'rnbqkb1r/p4ppp/2p5/3n4/2B2p2/5N2/PPPP2PP/RNBQK2R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/p4ppp/2p5/3n4/2B2p2/5N2/PPPP2PP/RNBQK2R w KQkq - 2 8' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Bb5+ c6 6. dxc6 bxc6 7. Bc4 Nd5

			variation => N__('KGA: Scandinavian, Botvinnik Variation'),
			moves => {
			},
		},
		'rnbqkb1r/p5pp/4pn2/1Ppp4/8/5N2/PP2PPPP/RNBQKB1R w KQkq d6 0 7' => {
			eco => 'E10l',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. d5 b5 5. dxe6 fxe6 6. cxb5 d5

			variation => N__('Blumenfeld: 5.dxe6 fxe6 6.cxb5 d5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2PpP3/5P2/2N5/PPP3PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C11g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. dxc5

			variation => N__('French: Steinitz, 6.dxc5'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1n1ppp/2n1p3/2PpP3/5P2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 7',
				'f8c5' => 'rnbqk2r/pp1n1ppp/4p3/2bpP3/5P2/2N5/PPP3PP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2PpP3/8/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'C11e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. Nf3 c5 6. dxc5

			variation => N__('French: Steinitz, 5. Nf3 c5 6.dxc5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'C11h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5 6. Nf3

			variation => N__('French: Steinitz, 6. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2N2N2/PPP3PP/R1BQKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/2N5/PPP3PP/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'C11f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4 c5

			variation => N__('French: Steinitz, 5. f4 c5'),
			moves => {
				'd4c5' => 'rnbqkb1r/pp1n1ppp/4p3/2PpP3/5P2/2N5/PPP3PP/R1BQKBNR b KQkq - 0 6',
				'g1f3' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/2N2N2/PPP3PP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/2P5/PP1N2PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C05e',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 5. f4'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P1P2/2P5/PP1N2PP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/8/PPPN2PP/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'C05e',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Closed, 5. f4'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/2P5/PP1N2PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq c6 0 6' => {
			eco => 'C11d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. Nf3 c5

			variation => N__('French: Steinitz, 5. Nf3 c5'),
			moves => {
				'd4c5' => 'rnbqkb1r/pp1n1ppp/4p3/2PpP3/8/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P4/2PB4/PP1N1PPP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'C05n',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, 5. Bd3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2n1ppp/1p2p3/2ppP3/3P4/2PB4/PP1N1PPP/R1BQK1NR w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp1n1ppp/2n1p3/2ppP3/3P4/2PB4/PP1N1PPP/R1BQK1NR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P4/3B4/PPPN1PPP/R1BQK1NR w KQkq c6 0 6' => {
			eco => 'C05n',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, 5. Bd3'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P4/2PB4/PP1N1PPP/R1BQK1NR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1n1ppp/8/2ppP1B1/2p5/2N2N2/PP3PPP/R2QKB1R b KQkq - 2 8' => {
			eco => 'D24g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 c5 5. d5 e6 6. e4 exd5 7. e5 Nfd7 8. Bg5

			variation => N__('QGA: 4. Nc3 c5 5. d5 e6 6. e4'),
			moves => {
				'f8e7' => 'rnbqk2r/pp1nbppp/8/2ppP1B1/2p5/2N2N2/PP3PPP/R2QKB1R w KQkq - 3 9',
			},
		},
		'rnbqkb1r/pp1n1ppp/8/2ppP3/2p5/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 8' => {
			eco => 'D24g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 c5 5. d5 e6 6. e4 exd5 7. e5 Nfd7

			variation => N__('QGA: 4. Nc3 c5 5. d5 e6 6. e4'),
			moves => {
				'c1g5' => 'rnbqkb1r/pp1n1ppp/8/2ppP1B1/2p5/2N2N2/PP3PPP/R2QKB1R b KQkq - 2 8',
			},
		},
		'rnbqkb1r/pp1p1p1p/5np1/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A60o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 g6

			variation => N__('Benoni: 4. Nc3 exd5 5.cxd5 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1pp1/4pn1p/2p3B1/3P4/2P2N2/PP2PPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A46t',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. c3 h6

			variation => N__('Torre Attack: 3... c5 4. c3 h6'),
			moves => {
				'g5h4' => 'rnbqkb1r/pp1p1pp1/4pn1p/2p5/3P3B/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1p1pp1/4pn1p/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A46w',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e3 h6

			variation => N__('Torre Attack: 3... c5 4. e3 h6'),
			moves => {
				'g5h4' => 'rnbqkb1r/pp1p1pp1/4pn1p/2p5/3P3B/4PN2/PPP2PPP/RN1QKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1p1pp1/4pn1p/2p5/3P3B/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 1 5' => {
			eco => 'A46t',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. c3 h6 5. Bh4

			variation => N__('Torre Attack: 3... c5 4. c3 h6 5. Bh4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1pp1/4pn1p/2p5/3P3B/4PN2/PPP2PPP/RN1QKB1R b KQkq - 1 5' => {
			eco => 'A46w',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e3 h6 5. Bh4

			variation => N__('Torre Attack: 3... c5 4. e3 h6 5. Bh4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/2p1pn2/8/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4' => {
			eco => 'E00g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 c6

			variation => N__('Catalan: 3... c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d3 c6 4. Nf3

			variation => N__('Bishop\'s Opening: Paulsen, 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2B1P3/3P4/PPP2PPP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d3 c6

			variation => N__('Bishop\'s Opening: Paulsen Defence'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 c6 3. e4 e5

			variation => N__('English: Anglo-Indian, 2. Nc3 c6 3. e4 e5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Nf3

			variation => N__('English: Bremen, Keres, 4. Nf3'),
			moves => {
				'd7d6' => 'rnbqkb1r/pp3ppp/2pp1n2/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 5',
				'e5e4' => 'rnbqkb1r/pp1p1ppp/2p2n2/8/2P1p3/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6

			variation => N__('English: Bremen, Keres System'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Bg2

			variation => N__('English: Bremen, Keres, 4. Bg2'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp3ppp/2p2n2/3pp3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq d6 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nf6 3. Bg2 c6

			variation => N__('English: King\'s, 2. g3 Nf6 3. Bg2 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/8/2B1Pp2/2N5/PPPP2PP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Nf6 4. Nc3 c6

			variation => N__('KGA: Bishop\'s Gambit, Jänisch Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/2p2n2/8/2P1p3/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Nf3 e4

			variation => N__('English: Bremen, Keres, 4. Nf3 e4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4p3/2pNP3/8/5N2/PPPP1PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'B29l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. Nc3 e6 5. Nxd5

			variation => N__('Sicilian: Nimzowitsch, 4. Nc3 e6 5.Nxd5'),
			moves => {
				'e6d5' => 'rnbqkb1r/pp1p1ppp/8/2ppP3/8/5N2/PPPP1PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/4p3/2pnP3/8/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'B29k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. Nc3 e6

			variation => N__('Sicilian: Nimzowitsch, 4. Nc3 e6'),
			moves => {
				'c3d5' => 'rnbqkb1r/pp1p1ppp/4p3/2pNP3/8/5N2/PPPP1PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4p3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'B22v',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 e6 6. Nf3

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 e6 6. Nf3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2p1ppp/1p2p3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp1p1ppp/2n1p3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 2 7',
				'd7d6' => 'rnbqkb1r/pp3ppp/3pp3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp1p1ppp/4p3/3nP3/3P4/8/PP3PPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'B22v',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 e6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 e6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp1p1ppp/4p3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/4p3/3nP3/3p4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B22r',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3 e6

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p1P3/2P5/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5

			variation => N__('English: Mikenas, Sicilian, 4. e5'),
			moves => {
				'f6g8' => 'rnbqkbnr/pp1p1ppp/4p3/2p1P3/2P5/2N5/PP1P1PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A46t',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. c3

			variation => N__('Torre Attack: 3... c5 4. c3'),
			moves => {
				'd8b6' => 'rnb1kb1r/pp1p1ppp/1q2pn2/2p3B1/3P4/2P2N2/PP2PPPP/RN1QKB1R w KQkq - 1 5',
				'h7h6' => 'rnbqkb1r/pp1p1pp1/4pn1p/2p3B1/3P4/2P2N2/PP2PPPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A46u',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e3

			variation => N__('Torre Attack: 3... c5 4. e3'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp1p1ppp/4pn2/6B1/3p4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5',
				'd8b6' => 'rnb1kb1r/pp1p1ppp/1q2pn2/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5',
				'f8e7' => 'rnbqk2r/pp1pbppp/4pn2/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5',
				'h7h6' => 'rnbqkb1r/pp1p1pp1/4pn1p/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq c6 0 4' => {
			eco => 'A46s',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5

			variation => N__('Torre Attack: 3... c5'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
				'e2e4' => 'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3PP3/5N2/PPP2PPP/RN1QKB1R b KQkq e3 0 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3PP3/5N2/PPP2PPP/RN1QKB1R b KQkq e3 0 4' => {
			eco => 'A46s',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e4

			variation => N__('Torre Attack: Wagner Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5

			variation => N__('English: Mikenas, Sicilian Variation'),
			moves => {
				'e4e5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p1P3/2P5/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 e6

			variation => N__('English: Symmetrical, 3 Knights, 3... e6'),
			moves => {
				'g2g3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 e6 4. g3

			variation => N__('English: Symmetrical, 3 Knights, 3... e6 4. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'E00f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 c5

			variation => N__('Queen\'s Pawn: Neo-Indian, 3... c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A56b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. e3 e6

			variation => N__('Benoni: 2... c5 3. e3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E10h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. e3

			variation => N__('Neo-Indian: Blumenfeld/Benoni, 4. e3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2p1ppp/1p2pn2/2p5/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'c5d4' => 'rnbqkb1r/pp1p1ppp/4pn2/8/2Pp4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'E10g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5

			variation => N__('Neo-Indian: Blumenfeld/Benoni'),
			moves => {
				'd4d5' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4' => {
			eco => 'E00h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 c5 4. Nf3

			variation => N__('Catalan: 3... c5 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq c6 0 4' => {
			eco => 'E00h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 c5

			variation => N__('Catalan: 3... c5'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P1B2/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A46q',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bf4 c5 4. c3

			variation => N__('Indian: London, 3... c5 4. c3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A46p',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bf4 c5 4. e3

			variation => N__('Indian: London, 3... c5 4. e3'),
			moves => {
				'd8b6' => 'rnb1kb1r/pp1p1ppp/1q2pn2/2p5/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq c6 0 4' => {
			eco => 'A46p',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bf4 c5

			variation => N__('Indian: London, 3... c5'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P1B2/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46e',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. c3 e6

			variation => N__('Neo-Benoni: 3. c3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A46l',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. e3 c5 4. Bd3

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. e3 c5 4. Bd3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A46k',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. e3 c5

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. e3 c5'),
			moves => {
				'f1d3' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A46o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 c5

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 c5'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A46o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5',
				'c5d4' => 'rnbqkb1r/pp1p1ppp/4pn2/8/3p4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5',
				'd8a5' => 'rnb1kb1r/pp1p1ppp/4pn2/q1p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5',
				'd8c7' => 'rnb1kb1r/ppqp1ppp/4pn2/2p5/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A60g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3

			variation => N__('Benoni: 4. Nc3'),
			moves => {
				'e6d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pp4/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'E10g',
                        # TRANSLATORS:

			variation => N__('Neo-Indian: Blumenfeld/Benoni'),
			moves => {
				'b7b5' => 'rnbqkb1r/p2p1ppp/4pn2/1ppP4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq b6 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A60b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nf3

			variation => N__('Benoni: 4. Nf3'),
			moves => {
				'e6d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pp4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 4' => {
			eco => 'A60e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. g3

			variation => N__('Benoni: 4. g3'),
			moves => {
				'e6d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pp4/2P5/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A60a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6

			variation => N__('Benoni: 3. d5 e6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4',
				'g2g3' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/8/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A43l',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 e6 4. Nc3

			variation => N__('Old Benoni: 2. d5 Nf6 3. Nf3 e6 4. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/2pP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A43l',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 e6

			variation => N__('Old Benoni: 2. d5 Nf6 3. Nf3 e6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/8/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/6B1/3p4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A46u',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 c5 4. e3 cxd4

			variation => N__('Torre Attack: 3... c5 4. e3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6',
				'd7d5' => 'rnbqkb1r/pp3ppp/4pn2/3p4/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq d6 0 6',
				'f8b4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/4P3/PP3PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. e3

			variation => N__('English: Symmetrical, Two Knights, 5. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3

			variation => N__('English: Symmetrical, Two Knights, 5. g3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'd7d5' => 'rnbqkb1r/pp3ppp/4pn2/3p4/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq d6 0 6',
				'd8b6' => 'rnb1kb1r/pp1p1ppp/1q2pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'd8c7' => 'rnb1kb1r/ppqp1ppp/4pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'f8b4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1bPN4/6P1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6

			variation => N__('English: Symmetrical, Two Knights, 4... e6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
				'e2e3' => 'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/4P3/PP3PPP/RNBQKB1R b KQkq - 0 5',
				'g2g3' => 'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/6P1/PP2PP1P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/2Pp4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'E10j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 c5 4. e3 cxd4

			variation => N__('Neo-Indian: Blumenfeld/Benoni, 4. e3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B40r',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 Nf6 5. Nc3'),
			moves => {
				'f8b4' => 'rnbqk2r/pp1p1ppp/4pn2/8/1b1NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/3NP3/3B4/PPP2PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'B40p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6 5. Bd3

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 Nf6 5. Bd3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1p1ppp/2n1pn2/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 3 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B40o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 Nf6

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1p1ppp/4pn2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5',
				'f1d3' => 'rnbqkb1r/pp1p1ppp/4pn2/8/3NP3/3B4/PPP2PPP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/4pn2/8/3p4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A46o',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 cxd4

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3 c5 4. Bg2 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/1N2p3/2P5/8/PP2PPPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e5 5. Nb5

			variation => N__('English: Symmetrical, Two Knights, 4... e5 5. Nb5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pN4/2P5/8/PP2PPPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A60h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. Nxd5

			variation => N__('Benoni: 4. Nc3 exd5 5.Nxd5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A60i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5

			variation => N__('Benoni: 4. Nc3 exd5 5.cxd5'),
			moves => {
				'd7d6' => 'rnbqkb1r/pp3ppp/3p1n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6',
				'f8d6' => 'rnbqk2r/pp1p1ppp/3b1n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 1 6',
				'g7g6' => 'rnbqkb1r/pp1p1p1p/5np1/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pP4/8/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'A60c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nf3 exd5 5. cxd5

			variation => N__('Benoni: 4. Nf3 exd5 5.cxd5'),
			moves => {
				'f8d6' => 'rnbqk2r/pp1p1ppp/3b1n2/2pP4/8/5N2/PP2PPPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pP4/8/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 5' => {
			eco => 'A60e',
                        # TRANSLATORS:

			variation => N__('Benoni: 4. g3'),
			moves => {
				'b7b5' => 'rnbqkb1r/p2p1ppp/5n2/1ppP4/8/6P1/PP2PP1P/RNBQKBNR w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pPp3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A56o',
                        # TRANSLATORS:

			variation => N__('Benoni: Czech'),
			moves => {
				'd7d6' => 'rnbqkb1r/pp3ppp/3p1n2/2pPp3/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pPp3/2P5/8/PP2PPPP/RNBQKBNR w KQkq e6 0 4' => {
			eco => 'A56o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5

			variation => N__('Benoni: Czech'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1p1ppp/5n2/2pPp3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pp4/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A60g',
                        # TRANSLATORS:

			variation => N__('Benoni: 4. Nc3'),
			moves => {
				'c3d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pN4/2P5/8/PP2PPPP/R1BQKBNR b KQkq - 0 5',
				'c4d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pp4/2P5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A60b',
                        # TRANSLATORS:

			variation => N__('Benoni: 4. Nf3'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pP4/8/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/2pp4/2P5/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 5' => {
			eco => 'A60e',
                        # TRANSLATORS:

			variation => N__('Benoni: 4. g3'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp1p1ppp/5n2/2pP4/8/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/5n2/4p3/2PN4/8/PP2PPPP/RNBQKB1R w KQkq e6 0 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e5

			variation => N__('English: Symmetrical, Two Knights, 4... e5'),
			moves => {
				'd4b5' => 'rnbqkb1r/pp1p1ppp/5n2/1N2p3/2P5/8/PP2PPPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1p1ppp/8/2ppP3/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq d3 0 6' => {
			eco => 'B29l',
                        # TRANSLATORS:

			variation => N__('Sicilian: Nimzowitsch, 4. Nc3 e6 5.Nxd5'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1p1ppp/2n5/2ppP3/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp1p1ppp/8/2ppP3/8/5N2/PPPP1PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B29l',
                        # TRANSLATORS:

			variation => N__('Sicilian: Nimzowitsch, 4. Nc3 e6 5.Nxd5'),
			moves => {
				'd2d4' => 'rnbqkb1r/pp1p1ppp/8/2ppP3/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq d3 0 6',
			},
		},
		'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'A10',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq d6 0 6',
			},
		},
		'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A10',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq d6 0 6',
			},
		},
		'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 c6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5',
				'g1h3' => 'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nh3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nh3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1pp1pp/2p2n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'A83c',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 c6

			variation => N__('Dutch: Staunton Gambit, Chigorin Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/2p2np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'E61a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 c6

			variation => N__('King\'s Indian: 3. Nc3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 g6

			variation => N__('English: Symmetrical, 3 Knights, 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A56d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. e3 g6 4. Nc3

			variation => N__('Benoni: 2... c5 3. e3 g6 4. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 g6 4. Nc3

			variation => N__('English: Symmetrical, Two Knights, 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'E61a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 c5

			variation => N__('King\'s Indian: 3. Nc3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A56c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. e3 g6

			variation => N__('Benoni: 2... c5 3. e3 g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 g6

			variation => N__('English: Symmetrical, Two Knights, 3... g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'd4d5' => 'rnbqkb1r/pp1ppp1p/5np1/2pP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46d',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. c3 g6

			variation => N__('Neo-Benoni: 3. c3 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2p5/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A48d',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. e3 c5

			variation => N__('Neo-King\'s Indian'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A56h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 g6 4. Nc3

			variation => N__('Benoni: 3. d5 g6'),
			moves => {
				'd7d6' => 'rnbqkb1r/pp2pp1p/3p1np1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2pP4/2P5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 g6 4. d5

			variation => N__('English: Symmetrical, Two Knights, 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A56h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 g6

			variation => N__('Benoni: 3. d5 g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppp1p/5np1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2pP4/8/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A43m',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 g6 4. Nc3

			variation => N__('Old Benoni: 2. d5 Nf6 3. Nf3 g6 4. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/2pP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A43m',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 g6

			variation => N__('Old Benoni: 2. d5 Nf6 3. Nf3 g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppp1p/5np1/2pP4/8/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 g6 5. Nc3

			variation => N__('English: Symmetrical, Two Knights, 4... g6'),
			moves => {
				'f8g7' => 'rnbqk2r/pp1pppbp/5np1/8/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp1ppp1p/5np1/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 g6

			variation => N__('English: Symmetrical, Two Knights, 4... g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppp1p/5np1/8/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1ppp1p/5p2/2p5/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'A45q',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6'),
			moves => {
				'd4d5' => 'rnbqkb1r/pp1ppp1p/5p2/2pP4/8/8/PPP1PPPP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppp1p/5p2/2pP4/8/8/PPP1PPPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45q',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6 gxf6 4. d5

			variation => N__('Trompowsky: 2... c5 3.Bxf6 gxf6 4. d5'),
			moves => {
				'd8b6' => 'rnb1kb1r/pp1ppp1p/1q3p2/2pP4/8/8/PPP1PPPP/RN1QKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1ppppp/1n6/2p1P3/8/1B1P4/PPP2PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'B02n',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. Bc4 Nb6 4. Bb3 c5 5. d3

			variation => N__('Alekhine: Kmoch Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/1n6/2p1P3/8/1B6/PPPP1PPP/RNBQK1NR w KQkq c6 0 5' => {
			eco => 'B02n',
                        # TRANSLATORS:

			variation => N__('Alekhine: 3. Bc4'),
			moves => {
				'd2d3' => 'rnbqkb1r/pp1ppppp/1n6/2p1P3/8/1B1P4/PPP2PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 c6 3. e4

			variation => N__('English: Anglo-Indian, 2. Nc3 c6 3. e4'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq d6 0 4',
				'e7e5' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq e6 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 c6

			variation => N__('English: Anglo-Indian, 2. Nc3 c6'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 Nf6

			variation => N__('English: Caro-Kann Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'A11',
                        # TRANSLATORS:

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A50j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c6 3. Nc3

			variation => N__('Indian: Slav-Indian, 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A50i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c6 3. Nf3

			variation => N__('Indian: Slav-Indian, 3. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A50h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c6

			variation => N__('Indian: Slav-Indian'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'g1f3' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pp1ppppp/2p2n2/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B12a',
                        # TRANSLATORS: 1. e4 c6 2. d4 Nf6

			variation => N__('Caro-Kann: Masi Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5B2/2p5/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A45q',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Bxf6

			variation => N__('Trompowsky: 2... c5 3.Bxf6'),
			moves => {
				'g7f6' => 'rnbqkb1r/pp1ppp1p/5p2/2p5/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2P3B1/8/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A45p',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. dxc5

			variation => N__('Trompowsky: 2... c5 3.dxc5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2P5/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A56e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. dxc5

			variation => N__('Benoni: 3.dxc5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2P5/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. dxc5

			variation => N__('Neo-Benoni 3.dxc5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p1P3/8/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B22o',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5'),
			moves => {
				'f6d5' => 'rnbqkb1r/pp1ppppp/8/2pnP3/8/2P5/PP1P1PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p1P3/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B29f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5

			variation => N__('Sicilian: Nimzowitsch, 3. e5'),
			moves => {
				'f6d5' => 'rnbqkb1r/pp1ppppp/8/2pnP3/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p3B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A45p',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. Nc3

			variation => N__('Trompowsky: 2... c5 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p3B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 3' => {
			eco => 'A45p',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5

			variation => N__('Trompowsky: 2... c5'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppppp/5n2/2p3B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3',
				'd4c5' => 'rnbqkb1r/pp1ppppp/5n2/2P3B1/8/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3',
				'd4d5' => 'rnbqkb1r/pp1ppppp/5n2/2pP2B1/8/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3',
				'g5f6' => 'rnbqkb1r/pp1ppppp/5B2/2p5/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3

			variation => N__('English: Symmetrical, 3 Knights'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2ppppp/1p3n2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/pp2pppp/5n2/2pp4/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq d6 0 4',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. g3

			variation => N__('English: Symmetrical, 2... Nf6 3. g3'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp2pppp/5n2/2pp4/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6

			variation => N__('English: Symmetrical, 2... Nf6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3',
				'g2g3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A30c',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6

			variation => N__('English: Symmetrical, 2. Nf3 Nf6'),
			moves => {
				'd2d4' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq d3 0 3',
				'g2g3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A30d',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. g3

			variation => N__('English: Symmetrical, 2. Nf3 Nf6 3. g3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2ppppp/1p3n2/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A56b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. e3

			variation => N__('Benoni: 2... c5 3. e3'),
			moves => {
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4

			variation => N__('English: Symmetrical, Two Knights'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p1ppppp/p4n2/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'c5d4' => 'rnbqkb1r/pp1ppppp/5n2/8/2Pp4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/2PP4/8/PP2PPPP/RNBQKBNR w KQkq c6 0 3' => {
			eco => 'A56a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5

			variation => N__('Benoni: 2... c5'),
			moves => {
				'd4c5' => 'rnbqkb1r/pp1ppppp/5n2/2P5/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'd4d5' => 'rnbqkb1r/pp1ppppp/5n2/2pP4/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46d',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. c3

			variation => N__('Neo-Benoni: 3. c3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2ppppp/1p3n2/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'c5d4' => 'rnbqkb1r/pp1ppppp/5n2/8/3p4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. e3

			variation => N__('Neo-Benoni: 3. e3'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp1ppppp/5n2/8/3p4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A46c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5

			variation => N__('Neo-Benoni'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3',
				'd4c5' => 'rnbqkb1r/pp1ppppp/5n2/2P5/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46f',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. g3

			variation => N__('Neo-Benoni: 3. g3'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp1ppppp/5n2/8/3p4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq c6 0 3' => {
			eco => 'A45i',
                        # TRANSLATORS: 1. d4 Nf6 2. g3 c5

			variation => N__('Indian: 2. g3 c5'),
			moves => {
				'd4d5' => 'rnbqkb1r/pp1ppppp/5n2/2pP4/8/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'B29c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. Nc3

			variation => N__('Sicilian: Nimzowitsch, 3. Nc3'),
			moves => {
				'd7d5' => 'rnbqkb1r/pp2pppp/5n2/2pp4/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B22o',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6

			variation => N__('Sicilian: Alapin, 2... Nf6'),
			moves => {
				'e4e5' => 'rnbqkb1r/pp1ppppp/5n2/2p1P3/8/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B29b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. d3

			variation => N__('Sicilian: Nimzowitsch, 3. d3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'B29a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6

			variation => N__('Sicilian: Nimzowitsch'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
				'd2d3' => 'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'e4e5' => 'rnbqkb1r/pp1ppppp/5n2/2p1P3/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 c5

			variation => N__('Bird: 1..Nf6 2. Nf3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 c5

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp1ppppp/5n2/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 c5 3. Bg2

			variation => N__('Reti: KIA'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1ppppp/2n2n2/2p5/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2pP2B1/8/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A45p',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 c5 3. d5

			variation => N__('Trompowsky: 2... c5 3. d5'),
			moves => {
				'd8b6' => 'rnb1kb1r/pp1ppppp/1q3n2/2pP2B1/8/8/PPP1PPPP/RN1QKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2pP4/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A56f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5

			variation => N__('Benoni: 3. d5'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p1ppppp/p4n2/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
				'b7b5' => 'rnbqkb1r/p2ppppp/5n2/1ppP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq b6 0 4',
				'd7d6' => 'rnbqkb1r/pp2pppp/3p1n2/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
				'e7e5' => 'rnbqkb1r/pp1p1ppp/5n2/2pPp3/2P5/8/PP2PPPP/RNBQKBNR w KQkq e6 0 4',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
				'f6e4' => 'rnbqkb1r/pp1ppppp/8/2pP4/2P1n3/8/PP2PPPP/RNBQKBNR w KQkq - 1 4',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2pP4/8/2N5/PPP1PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'A43k',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nc3

			variation => N__('Old Benoni: 2. d5 Nf6 3. Nc3'),
			moves => {
				'd8a5' => 'rnb1kb1r/pp1ppppp/5n2/q1pP4/8/2N5/PPP1PPPP/R1BQKBNR w KQkq - 3 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2pP4/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'A43l',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3

			variation => N__('Old Benoni: 2. d5 Nf6 3. Nf3'),
			moves => {
				'b7b5' => 'rnbqkb1r/p2ppppp/5n2/1ppP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq b6 0 4',
				'c5c4' => 'rnbqkb1r/pp1ppppp/5n2/3P4/2p5/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4pn2/2pP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/2pP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2pP4/8/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A45i',
                        # TRANSLATORS:

			variation => N__('Indian: 2. g3 c5'),
			moves => {
				'b7b5' => 'rnbqkb1r/p2ppppp/5n2/1ppP4/8/6P1/PPP1PP1P/RNBQKBNR w KQkq b6 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/2pP4/8/8/PPP1PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A43k',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6

			variation => N__('Old Benoni: 2. d5 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppppp/5n2/2pP4/8/2N5/PPP1PPPP/R1BQKBNR b KQkq - 2 3',
				'g1f3' => 'rnbqkb1r/pp1ppppp/5n2/2pP4/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/3P4/2p1P3/5N2/PPP2PPP/RNBQKB1R b KQkq e3 0 4' => {
			eco => 'A43r',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 c4 4. e4

			variation => N__('Old Benoni: Hawk, 4. e4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/3P4/2p5/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A43r',
                        # TRANSLATORS: 1. d4 c5 2. d5 Nf6 3. Nf3 c4

			variation => N__('Old Benoni: Hawk'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp1ppppp/5n2/3P4/2p1P3/5N2/PPP2PPP/RNBQKB1R b KQkq e3 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/2PN4/8/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4

			variation => N__('English: Symmetrical, Two Knights'),
			moves => {
				'b7b6' => 'rnbqkb1r/p2ppppp/1p3n2/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 0 5',
				'b8c6' => 'r1bqkb1r/pp1ppppp/2n2n2/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 1 5',
				'e7e5' => 'rnbqkb1r/pp1p1ppp/5n2/4p3/2PN4/8/PP2PPPP/RNBQKB1R w KQkq e6 0 5',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4pn2/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 0 5',
				'g7g6' => 'rnbqkb1r/pp1ppp1p/5np1/8/2PN4/8/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/2Pp4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A31',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4

			variation => N__('English: Symmetrical, Two Knights'),
			moves => {
				'f3d4' => 'rnbqkb1r/pp1ppppp/5n2/8/2PN4/8/PP2PPPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/3N4/6P1/PPP1PP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A46f',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. g3 cxd4 4. Nxd4

			variation => N__('Neo-Benoni: 3. g3 cxd4 4.Nxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/3p4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46d',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. c3 cxd4

			variation => N__('Neo-Benoni: 3. c3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/3p4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. e3 cxd4

			variation => N__('Neo-Benoni: 3. e3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/3p4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A46f',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 c5 3. g3 cxd4

			variation => N__('Neo-Benoni: 3. g3 cxd4'),
			moves => {
				'f3d4' => 'rnbqkb1r/pp1ppppp/5n2/8/3N4/6P1/PPP1PP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/5n2/8/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B21p',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 Nf6

			variation => N__('Sicilian: Smith-Morra, 3. c3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2P3B1/4n2P/8/PPP1PPP1/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45s',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. h4 c5 4. dxc5

			variation => N__('Trompowsky: 2... Ne4 3. h4 c5 4.dxc5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2p1P3/8/2n2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'B29j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. Nc3 Nxc3

			variation => N__('Sicilian: Nimzowitsch, 4. Nc3 Nxc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2p3B1/3Pn2P/8/PPP1PPP1/RN1QKBNR w KQkq c6 0 4' => {
			eco => 'A45s',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. h4 c5

			variation => N__('Trompowsky: 2... Ne4 3. h4 c5'),
			moves => {
				'd4c5' => 'rnbqkb1r/pp1ppppp/8/2P3B1/4n2P/8/PPP1PPP1/RN1QKBNR b KQkq - 0 4',
				'd4d5' => 'rnbqkb1r/pp1ppppp/8/2pP2B1/4n2P/8/PPP1PPP1/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2p5/3Pn2B/5P2/PPP1P1PP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45t',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bh4 c5 4. f3

			variation => N__('Trompowsky: 2... Ne4 3. Bh4 c5 4. f3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2p5/3Pn2B/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 4' => {
			eco => 'A45t',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bh4 c5

			variation => N__('Trompowsky: 2... Ne4 3. Bh4 c5'),
			moves => {
				'f2f3' => 'rnbqkb1r/pp1ppppp/8/2p5/3Pn2B/5P2/PPP1P1PP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2p5/3PnB2/5P2/PPP1P1PP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45y',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5 4. f3

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. f3'),
			moves => {
				'd8a5' => 'rnb1kb1r/pp1ppppp/8/q1p5/3PnB2/5P2/PPP1P1PP/RN1QKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2p5/3PnB2/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 4' => {
			eco => 'A45x',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5'),
			moves => {
				'd4d5' => 'rnbqkb1r/pp1ppppp/8/2pP4/4nB2/8/PPP1PPPP/RN1QKBNR b KQkq - 0 4',
				'f2f3' => 'rnbqkb1r/pp1ppppp/8/2p5/3PnB2/5P2/PPP1P1PP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pP2B1/4n2P/8/PPP1PPP1/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45s',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. h4 c5 4. d5

			variation => N__('Trompowsky: 2... Ne4 3. h4 c5 4. d5'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pP4/2P1n3/8/PP2PPPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A56n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 Ne4

			variation => N__('Benoni: Vulture'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pP4/4nB2/8/PPP1PPPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45x',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 c5 4. d5

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 c5 4. d5'),
			moves => {
				'd8b6' => 'rnb1kb1r/pp1ppppp/1q6/2pP4/4nB2/8/PPP1PPPP/RN1QKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/2P5/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 4' => {
			eco => 'B29g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. c4

			variation => N__('Sicilian: Nimzowitsch, 4. c4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/3P4/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 4' => {
			eco => 'B22p',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. d4'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp1ppppp/8/3nP3/3p4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'B29h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. d4

			variation => N__('Sicilian: Nimzowitsch, 4. d4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 2 4' => {
			eco => 'B29i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5 4. Nc3

			variation => N__('Sicilian: Nimzowitsch, 4. Nc3'),
			moves => {
				'd5c3' => 'rnbqkb1r/pp1ppppp/8/2p1P3/8/2n2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 5',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4p3/2pnP3/8/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/8/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'B22p',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. Nf3

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1ppppp/2n5/2pnP3/8/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 3 5',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/8/2P3P1/PP1P1P1P/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B22o',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. g3

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/8/2P5/PP1P1PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B22o',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5'),
			moves => {
				'd2d4' => 'rnbqkb1r/pp1ppppp/8/2pnP3/3P4/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 4',
				'g1f3' => 'rnbqkb1r/pp1ppppp/8/2pnP3/8/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 2 4',
				'g2g3' => 'rnbqkb1r/pp1ppppp/8/2pnP3/8/2P3P1/PP1P1P1P/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/8/2pnP3/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B29f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. e5 Nd5

			variation => N__('Sicilian: Nimzowitsch, 3. e5 Nd5'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp1ppppp/8/2pnP3/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 2 4',
				'c2c4' => 'rnbqkb1r/pp1ppppp/8/2pnP3/2P5/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 4',
				'd2d4' => 'rnbqkb1r/pp1ppppp/8/2pnP3/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 4',
			},
		},
		'rnbqkb1r/pp1ppppp/8/3nP3/2Bp4/2P5/PP3PPP/RNBQK1NR b KQkq - 1 5' => {
			eco => 'B22q',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. Bc4

			variation => N__('Sicilian: Alapin, 2... Nf6, 5. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/3nP3/3P4/8/PP3PPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'B22t',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1ppppp/2n5/3nP3/3P4/8/PP3PPP/RNBQKBNR w KQkq - 1 6',
				'd7d6' => 'rnbqkb1r/pp2pppp/3p4/3nP3/3P4/8/PP3PPP/RNBQKBNR w KQkq - 0 6',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4p3/3nP3/3P4/8/PP3PPP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1ppppp/8/3nP3/3Q4/2P5/PP3PPP/RNB1KBNR b KQkq - 0 5' => {
			eco => 'B22q',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. Qxd4

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.Qxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp1ppppp/8/3nP3/3p4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'B22r',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. d4 cxd4 5. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp1ppppp/2n5/3nP3/3p4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 2 6',
				'e7e6' => 'rnbqkb1r/pp1p1ppp/4p3/3nP3/3p4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp1ppppp/8/3nP3/3p4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B22q',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4

			variation => N__('Sicilian: Alapin, 2... Nf6 3. e5 Nd5 4. d4 cxd4'),
			moves => {
				'c3d4' => 'rnbqkb1r/pp1ppppp/8/3nP3/3P4/8/PP3PPP/RNBQKBNR b KQkq - 0 5',
				'd1d4' => 'rnbqkb1r/pp1ppppp/8/3nP3/3Q4/2P5/PP3PPP/RNB1KBNR b KQkq - 0 5',
				'f1c4' => 'rnbqkb1r/pp1ppppp/8/3nP3/2Bp4/2P5/PP3PPP/RNBQK1NR b KQkq - 1 5',
				'g1f3' => 'rnbqkb1r/pp1ppppp/8/3nP3/3p4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2p1pp/2pp1n2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'A86',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. c4 Nf6 3. g3 d6'),
			moves => {
				'd8c7' => 'rnb1kb1r/ppq1p1pp/2pp1n2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp2p1pp/2pp1n2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'A86',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. c4 Nf6 3. g3 d6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2p1pp/2pp1n2/5p2/2PP4/2N3P1/PP2PPBP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pp1p/1n1p2p1/8/2PP4/2N1B3/PP3PPP/R2QKBNR b KQkq - 1 7' => {
			eco => 'B03t',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6 cxd6 6. Nc3 g6 7. Be3

			variation => N__('Alekhine: Exchange, 5...cxd6 6. Nc3 g6 7. Be3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/1n1p2p1/8/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'B03s',
                        # TRANSLATORS:

			variation => N__('Alekhine: Exchange, 5...cxd6 6. Nc3'),
			moves => {
				'c1e3' => 'rnbqkb1r/pp2pp1p/1n1p2p1/8/2PP4/2N1B3/PP3PPP/R2QKBNR b KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3P4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 c6 5. cxd5

			variation => N__('Grünfeld: Schlechter, Exchange'),
			moves => {
				'c6d5' => 'rnbqkb1r/pp2pp1p/5np1/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'A12',
                        # TRANSLATORS:

			variation => N__('English: Bled Variation'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/2p2np1/3p4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2P5/1P3N2/PB1PPPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. Bb2 g6

			variation => N__('English: Bled Variation'),
			moves => {
				'e2e3' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP1B2/5N2/PPQ1PPPP/RN2KB1R b KQkq - 1 5' => {
			eco => 'D11j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Qc2 g6 5. Bf4

			variation => N__('Slav: 4. Qc2 g6 5. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 c6 5. Qb3

			variation => N__('Grünfeld: Schlechter, 5. Qb3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 c6

			variation => N__('Grünfeld: Schlechter Variation'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp2pp1p/2p2np1/3P4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
				'd1b3' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D11r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3 g6

			variation => N__('Slav: 4. e3 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R w KQkq - 0 5' => {
			eco => 'D11i',
                        # TRANSLATORS:

			variation => N__('Slav: 4. Qc2'),
			moves => {
				'c1f4' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP1B2/5N2/PPQ1PPPP/RN2KB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2 c6

			variation => N__('Neo-Grünfeld: 4. Bg2 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2p2/8/2BP4/8/PPP2PPP/R1BQK1NR b KQkq - 1 6' => {
			eco => 'B16d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Bc4

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/2P5/PP3PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'B16i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. c3

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. c3'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/2P5/PP3PPP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/3Q4/PPP2PPP/R1B1KBNR b KQkq - 1 6' => {
			eco => 'B16b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Qd3

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Qd3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B16e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Nf3

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Nf3'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pp1p/2p2p2/5b2/3P4/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 7',
				'c8g4' => 'rn1qkb1r/pp2pp1p/2p2p2/8/3P2b1/5N2/PPP2PPP/R1BQKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/8/PPP1NPPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B16c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6 6. Ne2

			variation => N__('Caro-Kann: Bronstein-Larsen, 6. Ne2'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B16a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ gxf6

			variation => N__('Caro-Kann: Bronstein-Larsen'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/2P5/PP3PPP/R1BQKBNR b KQkq - 0 6',
				'd1d3' => 'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/3Q4/PPP2PPP/R1B1KBNR b KQkq - 1 6',
				'f1c4' => 'rnbqkb1r/pp2pp1p/2p2p2/8/2BP4/8/PPP2PPP/R1BQK1NR b KQkq - 1 6',
				'g1e2' => 'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/8/PPP1NPPP/R1BQKB1R b KQkq - 1 6',
				'g1f3' => 'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/5N2/PPP2PPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pp1p/2p3p1/3nN3/2BP4/8/PPP2PPP/RNBQ1RK1 b kq - 1 7' => {
			eco => 'B04i',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 dxe5 5. Nxe5 g6 6. Bc4 c6 7. O-O

			variation => N__('Alekhine: Modern, Larsen, 5.Nxe5 g6 6. Bc4 c6 7.O-O'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/2p3p1/3nN3/2BP4/8/PPP2PPP/RNBQK2R w KQkq - 0 7' => {
			eco => 'B04h',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, Larsen, 5.Nxe5 g6 6. Bc4'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp2pp1p/2p3p1/3nN3/2BP4/8/PPP2PPP/RNBQ1RK1 b kq - 1 7',
			},
		},
		'rnbqkb1r/pp2pp1p/2pp1np1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'B07t',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be3 c6

			variation => N__('Pirc: 4. Be3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/2p5/4P3/2P2N2/PP1PBPPP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'B50v',
                        # TRANSLATORS:

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Be2 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/2p5/4P3/2P2N2/PP1PBPPP/RNBQ1RK1 w kq - 2 6',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'B50v',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Be2 g6

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Be2 g6'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp2pp1p/3p1np1/2p5/4P3/2P2N2/PP1PBPPP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/2pP4/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5' => {
			eco => 'A56j',
                        # TRANSLATORS:

			variation => N__('Benoni: 3. d5 d6 4. Nc3 g6'),
			moves => {
				'b7b5' => 'rnbqkb1r/p3pp1p/3p1np1/1ppP4/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq b6 0 6',
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A56j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 d6 4. Nc3 g6

			variation => N__('Benoni: 3. d5 d6 4. Nc3 g6'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp2pp1p/3p1np1/2pP4/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/2pP4/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A43u',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6 5. Nf3

			variation => N__('Old Benoni: Schmidt, 3. e4 Nf6'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/2pP4/4P3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A43u',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3 g6

			variation => N__('Old Benoni: Schmidt, 3. e4 Nf6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp2pp1p/3p1np1/2pP4/4P3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'B70e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Bg5

			variation => N__('Sicilian: Dragon, 6. Bg5'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/6B1/3NP3/2N5/PPP2PPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B70f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Bc4

			variation => N__('Sicilian: Dragon, 6. Bc4'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'B72',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be3

			variation => N__('Sicilian: Dragon, 6. Be3'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N2P2/PPP3PP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B70a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f3

			variation => N__('Sicilian: Dragon, 6. f3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B70b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. g3

			variation => N__('Sicilian: Dragon, 6. g3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pp1p/2np1np1/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B70i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. Be2

			variation => N__('Sicilian: Dragon, 6. Be2'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pp1p/p2p1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp2pp1p/2np1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B70a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6

			variation => N__('Sicilian: Dragon'),
			moves => {
				'c1e3' => 'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6',
				'c1g5' => 'rnbqkb1r/pp2pp1p/3p1np1/6B1/3NP3/2N5/PPP2PPP/R2QKB1R b KQkq - 1 6',
				'f1c4' => 'rnbqkb1r/pp2pp1p/3p1np1/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6',
				'f1e2' => 'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6',
				'f2f3' => 'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N2P2/PPP3PP/R1BQKB1R b KQkq - 0 6',
				'f2f4' => 'rnbqkb1r/pp2pp1p/3p1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6',
				'g2g3' => 'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pp1p/3p1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6' => {
			eco => 'B71',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 g6 6. f4

			variation => N__('Sicilian: Dragon, Levenfish Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pp1p/2np1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'b8d7' => 'r1bqkb1r/pp1npp1p/3p1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'f8g7' => 'rnbqk2r/pp2ppbp/3p1np1/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp2pp1p/5np1/3P4/3P4/2N5/PP3PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'B14b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. cxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6.cxd5'),
			moves => {
				'f6d5' => 'rnbqkb1r/pp2pp1p/6p1/3n4/3P4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7',
				'f8g7' => 'rnbqk2r/pp2ppbp/5np1/3P4/3P4/2N5/PP3PPP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp2pp1p/5np1/3p4/2PP4/1QN5/PP3PPP/R1B1KBNR b KQkq - 1 6' => {
			eco => 'B14f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. Qb3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6. Qb3'),
			moves => {
				'f8g7' => 'rnbqk2r/pp2ppbp/5np1/3p4/2PP4/1QN5/PP3PPP/R1B1KBNR w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pp1p/5np1/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B14a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. Nf3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/5np1/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B14a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp2pp1p/5np1/3P4/3P4/2N5/PP3PPP/R1BQKBNR b KQkq - 0 6',
				'd1b3' => 'rnbqkb1r/pp2pp1p/5np1/3p4/2PP4/1QN5/PP3PPP/R1B1KBNR b KQkq - 1 6',
				'g1f3' => 'rnbqkb1r/pp2pp1p/5np1/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pp1p/5np1/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3 c6 5. cxd5 cxd5

			variation => N__('Grünfeld: Schlechter, Exchange'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/5p2/2pp4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq c6 0 5' => {
			eco => 'D00j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 gxf6 4. e3 c5

			variation => N__('Trompowsky: 2... d5 3.Bxf6 gxf6 4. e3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/6p1/2p3B1/2PPp3/8/PP1QPPPP/R3KBNR w KQkq c6 0 7' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4 5. Nxe4 dxe4 6. Qd2 c5

			variation => N__('Grünfeld: Stockholm, Lundin Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/6p1/2p5/3PP3/2P5/P4PPP/R1BQKBNR w KQkq c6 0 7' => {
			eco => 'D85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3 6. bxc3 c5

			variation => N__('Grünfeld: Exchange, 5. e4 Nxc3 6.bxc3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/6p1/3n4/2BP4/2N5/PP3PPP/R1BQK1NR b KQkq - 1 7' => {
			eco => 'B14d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. cxd5 Nxd5 7. Bc4

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6.cxd5 Nxd5 7. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pp1p/6p1/3n4/3P4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'B14c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 g6 6. cxd5 Nxd5

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... g6 6.cxd5 Nxd5'),
			moves => {
				'f1c4' => 'rnbqkb1r/pp2pp1p/6p1/3n4/2BP4/2N5/PP3PPP/R1BQK1NR b KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp2pppp/1n1p4/8/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 1 6' => {
			eco => 'B03s',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6 cxd6 6. Nc3

			variation => N__('Alekhine: Exchange, 5...cxd6 6. Nc3'),
			moves => {
				'g7g6' => 'rnbqkb1r/pp2pp1p/1n1p2p1/8/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp2pppp/1n1p4/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'B03r',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6 cxd6

			variation => N__('Alekhine: Exchange, 5...cxd6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/1n1p4/8/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pppp/1n6/2p1P3/2PP4/8/PP4PP/RNBQKBNR w KQkq c6 0 7' => {
			eco => 'B03k',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5 c5

			variation => N__('Alekhine: Four Pawns Attack, 6... c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2P2n2/8/2P5/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 c6 4. dxc6

			variation => N__('Scandinavian: Scandinavian Gambit Accepted'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2N2/8/3P4/8/PPP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'B15t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+

			variation => N__('Caro-Kann: 4.Nxe4 Nf6 5.Nxf6+'),
			moves => {
				'e7f6' => 'rnbqkb1r/pp3ppp/2p2p2/8/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 0 6',
				'g7f6' => 'rnbqkb1r/pp2pp1p/2p2p2/8/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3P4/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 c6

			variation => N__('Scandinavian: Scandinavian Gambit'),
			moves => {
				'd5c6' => 'rnbqkb1r/pp2pppp/2P2n2/8/2P5/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3P4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D13a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5

			variation => N__('Slav: Exchange'),
			moves => {
				'c6d5' => 'rnbqkb1r/pp2pppp/5n2/3p4/3P4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3P4/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B02g',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 c6

			variation => N__('Alekhine: Scandinavian, Geschev Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'D10s',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. Bg5

			variation => N__('Slav: 3. Nc3 Nf6 4. Bg5'),
			moves => {
				'd5c4' => 'rnbqkb1r/pp2pppp/2p2n2/6B1/2pP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4' => {
			eco => 'D01f',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 c6

			variation => N__('Richter-Veresov: 3... c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 c6 3. e4 d5

			variation => N__('English: Anglo-Indian, 2. Nc3 c6 3. e4 d5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A12',
                        # TRANSLATORS:

			variation => N__('English: Caro-Kann Defence, 3. b3'),
			moves => {
				'c1b2' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/1P3N2/PB1PPPPP/RN1QKB1R b KQkq - 2 4',
				'g2g3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/1P3NP1/P2PPP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/1P3N2/PB1PPPPP/RN1QKB1R b KQkq - 2 4' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3 Nf6 4. Bb2

			variation => N__('English: Caro-Kann Defence, 3. b3'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/1P3N2/PB1PPPPP/RN1QKB1R w KQkq - 3 5',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/1P3N2/PB1PPPPP/RN1QKB1R w KQkq - 3 5',
				'g7g6' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2P5/1P3N2/PB1PPPPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/1P3NP1/P2PPP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A12',
                        # TRANSLATORS:

			variation => N__('English: Caro-Kann Defence, 3. b3'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/1P3NP1/P2PPP1P/RNBQKB1R w KQkq - 1 5',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/1P3NP1/P2PPP1P/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 2 4' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. e3 Nf6 4. Nc3

			variation => N__('English: Caro-Kann Defence, 3. e3 Nf6 4. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. e3 Nf6

			variation => N__('English: Caro-Kann Defence, 3. e3 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/2N1PN2/PP1P1PPP/R1BQKB1R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5 4. Nf3

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2P3b1/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq d6 0 4' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3 Nf6 3. Bg2 d5

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/1Q3N2/PP2PPPP/RNB1KB1R b KQkq - 3 4' => {
			eco => 'D11h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Qb3

			variation => N__('Slav: 4. Qb3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D10u',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. e3

			variation => N__('Slav: 3. Nc3 Nf6 4. e3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 5',
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'D15a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3

			variation => N__('Slav: 4. Nc3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
				'd5c4' => 'rnbqkb1r/pp2pppp/2p2n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'd8b6' => 'rnb1kb1r/pp2pppp/1qp2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D10r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6

			variation => N__('Slav: 3. Nc3 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/pp2pppp/2p2n2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'e2e3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'D10h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. e3 Nf6

			variation => N__('Slav: 3. e3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D11q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. e3

			variation => N__('Slav: 4. e3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pppp/p1p2n2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'g7g6' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'D11k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nbd2

			variation => N__('Slav: Breyer Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D11g',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6

			variation => N__('Slav: 3. Nf3 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
				'b1d2' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 3 4',
				'c4d5' => 'rnbqkb1r/pp2pppp/2p2n2/3P4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
				'd1b3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/1Q3N2/PP2PPPP/RNB1KB1R b KQkq - 3 4',
				'd1c2' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R b KQkq - 3 4',
				'e2e3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'g2g3' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R b KQkq - 3 4' => {
			eco => 'D11i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Qc2

			variation => N__('Slav: 4. Qc2'),
			moves => {
				'g7g6' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D11l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. g3

			variation => N__('Slav: Slav-Reti System'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/2PP2b1/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D04f',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 c6

			variation => N__('Colle: 3... c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D02s',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 c6

			variation => N__('Queen\'s Pawn: 3. g3 c6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'D02s',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 c6 4. Bg2

			variation => N__('Queen\'s Pawn: 3. g3 c6'),
			moves => {
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/3P2b1/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 1 4' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/3p1b2/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/3p4/6b1/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 c6

			variation => N__('Reti: KIA'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 1 4',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/4N3/2pP4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D15p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. Ne5

			variation => N__('Slav: Accepted, 5. Ne5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/6B1/2pP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'D10t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 Nf6 4. Bg5 dxc4

			variation => N__('Slav: 3. Nc3 Nf6 4. Bg5 dxc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/2QP4/5N2/PP2PPPP/RNB1KB1R b KQkq - 0 5' => {
			eco => 'D23i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6 5. Qxc4

			variation => N__('QGA: Mannheim, 4... c6 5.Qxc4'),
			moves => {
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/5b2/2QP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D15q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e3

			variation => N__('Slav: Accepted, Alekhine'),
			moves => {
				'b7b5' => 'rnbqkb1r/p3pppp/2p2n2/1p6/2pP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D15o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4

			variation => N__('Slav: Accepted'),
			moves => {
				'a2a4' => 'rnbqkb1r/pp2pppp/2p2n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5',
				'e2e3' => 'rnbqkb1r/pp2pppp/2p2n2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'e2e4' => 'rnbqkb1r/pp2pppp/2p2n2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5',
				'f3e5' => 'rnbqkb1r/pp2pppp/2p2n2/4N3/2pP4/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'D15t',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. e4

			variation => N__('Slav: Geller (Tolush) Gambit'),
			moves => {
				'b7b5' => 'rnbqkb1r/p3pppp/2p2n2/1p6/2pPP3/2N2N2/PP3PPP/R1BQKB1R w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D00w',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Nxf3 c6

			variation => N__('Blackmar-Diemer: Ziegler Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/3P4/6N1/PPP2PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B15s',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Ng3

			variation => N__('Caro-Kann: 4.Nxe4 Nf6 5. Ng3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/3PN3/3B4/PPP2PPP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'B15r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Bd3

			variation => N__('Caro-Kann: Alekhine Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/3PN3/5P2/PPP3PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'B15r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. f3

			variation => N__('Caro-Kann: 4.Nxe4 Nf6 5. f3 Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'B15r',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6

			variation => N__('Caro-Kann: 4.Nxe4 Nf6'),
			moves => {
				'e4f6' => 'rnbqkb1r/pp2pppp/2p2N2/8/3P4/8/PPP2PPP/R1BQKBNR b KQkq - 0 5',
				'e4g3' => 'rnbqkb1r/pp2pppp/2p2n2/8/3P4/6N1/PPP2PPP/R1BQKBNR b KQkq - 2 5',
				'f1d3' => 'rnbqkb1r/pp2pppp/2p2n2/8/3PN3/3B4/PPP2PPP/R1BQK1NR b KQkq - 2 5',
				'f2f3' => 'rnbqkb1r/pp2pppp/2p2n2/8/3PN3/5P2/PPP3PP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 c6

			variation => N__('Blackmar-Diemer: O\'Kelly Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/4N3/5N2/PPPP1PPP/R1BQKB1R w KQkq - 1 5' => {
			eco => 'B10v',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 dxe4 4. Nxe4 Nf6

			variation => N__('Caro-Kann: Two Knights, 3...dxe4 4.Nxe4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/P1pP3N/2N1P3/1P3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D17e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Nh4 Bc8 7. e3

			variation => N__('Slav: Czech, 6. Nh4 Bc8 7. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/P1pP3N/2N5/1P2PPPP/R1BQKB1R w KQkq - 3 7' => {
			eco => 'D17e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Bf5 6. Nh4 Bc8

			variation => N__('Slav: Czech, 6. Nh4 Bc8'),
			moves => {
				'e2e3' => 'rnbqkb1r/pp2pppp/2p2n2/8/P1pP3N/2N1P3/1P3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5' => {
			eco => 'D16a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4

			variation => N__('Slav: Alapin'),
			moves => {
				'a7a5' => 'rnbqkb1r/1p2pppp/2p2n2/p7/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq a6 0 6',
				'b8a6' => 'r1bqkb1r/pp2pppp/n1p2n2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6',
				'c8f5' => 'rn1qkb1r/pp2pppp/2p2n2/5b2/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6',
				'c8g4' => 'rn1qkb1r/pp2pppp/2p2n2/8/P1pP2b1/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6',
				'e7e6' => 'rnbqkb1r/pp3ppp/2p1pn2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 6',
				'f6d5' => 'rnbqkb1r/pp2pppp/2p5/3n4/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pppp/2p2n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 0 5' => {
			eco => 'D23i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+ c6

			variation => N__('QGA: Mannheim, 4... c6'),
			moves => {
				'a4c4' => 'rnbqkb1r/pp2pppp/2p2n2/8/2QP4/5N2/PP2PPPP/RNB1KB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/2p5/3n4/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D16b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 Nd5

			variation => N__('Slav: 5. a4 Nd5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2pp1n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 c6

			variation => N__('Old Indian: 3. Nc3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2pp1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3 c6

			variation => N__('Old Indian: 3. Nf3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2pp1n2/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B07o',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 c6 4. Nf3

			variation => N__('Pirc: Czech, 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2pp1n2/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B07n',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 c6

			variation => N__('Pirc: Pytel/Czech'),
			moves => {
				'f2f4' => 'rnbqkb1r/pp2pppp/2pp1n2/8/3PPP2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 4',
				'g1f3' => 'rnbqkb1r/pp2pppp/2pp1n2/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp2pppp/2pp1n2/8/3PPP2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 4' => {
			eco => 'B07p',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 c6 4. f4

			variation => N__('Pirc: Czech, 4. f4'),
			moves => {
				'd8a5' => 'rnb1kb1r/pp2pppp/2pp1n2/q7/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/2pp4/3nP3/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'B04d',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 c6 5. c4

			variation => N__('Alekhine: Modern, 4... c6 5. c4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/2pp4/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B04c',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 c6

			variation => N__('Alekhine: Modern, 4... c6'),
			moves => {
				'c2c4' => 'rnbqkb1r/pp2pppp/2pp4/3nP3/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq c3 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/2B1P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'B50p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Bc4

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'B50h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bc4 Nf6 4. d3

			variation => N__('Sicilian: 2. Nf3 d6 3. Bc4 Nf6 4. d3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4' => {
			eco => 'B50g',
                        # TRANSLATORS:

			variation => N__('Sicilian: 2. Nf3 d6 3. Bc4'),
			moves => {
				'd2d3' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/2B1P3/3P1N2/PPP2PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B53b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 Nf6

			variation => N__('Sicilian: 2... d6 3. d4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 2 4' => {
			eco => 'B50k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Nc3 Nf6

			variation => N__('Sicilian: 2. Nf3 d6 3. Nc3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2P2N1P/PP1P1PP1/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B50n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. h3

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. h3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2np1n2/2p5/4P3/2P2N1P/PP1P1PP1/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'B50m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6'),
			moves => {
				'f1c4' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/2B1P3/2P2N2/PP1P1PPP/RNBQK2R b KQkq - 2 4',
				'f1d3' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2PB1N2/PP1P1PPP/RNBQK2R b KQkq - 2 4',
				'f1e2' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R b KQkq - 2 4',
				'h2h3' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2P2N1P/PP1P1PP1/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'B50s',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Be2

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Be2'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2np1n2/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R w KQkq - 3 5',
				'b8d7' => 'r1bqkb1r/pp1npppp/3p1n2/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R w KQkq - 3 5',
				'g7g6' => 'rnbqkb1r/pp2pp1p/3p1np1/2p5/4P3/2P2N2/PP1PBPPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2PB1N2/PP1P1PPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'B50q',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3 Nf6 4. Bd3

			variation => N__('Sicilian: 2. Nf3 d6 3. c3 Nf6 4. Bd3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2np1n2/2p5/4P3/2PB1N2/PP1P1PPP/RNBQK2R w KQkq - 3 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A56i',
                        # TRANSLATORS:

			variation => N__('Benoni: 3. d5 d6'),
			moves => {
				'g7g6' => 'rnbqkb1r/pp2pp1p/3p1np1/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2pP4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A56i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 d6

			variation => N__('Benoni: 3. d5 d6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/3p1n2/2pP4/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 4' => {
			eco => 'A43u',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6 4. Nc3

			variation => N__('Old Benoni: Schmidt, 3. e4 Nf6'),
			moves => {
				'g7g6' => 'rnbqkb1r/pp2pp1p/3p1np1/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2pP4/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A43u',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 Nf6

			variation => N__('Old Benoni: Schmidt, 3. e4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/3p1n2/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2pP4/8/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. Nc3 Nf6

			variation => N__('Old Benoni: Schmidt, 3. Nc3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/2pP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. Nf3 Nf6

			variation => N__('Old Benoni: Schmidt, 3. Nf3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'B56a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'b8d7' => 'r1bqkb1r/pp1npppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'c8d7' => 'rn1qkb1r/pp1bpppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
				'e7e5' => 'rnbqkb1r/pp3ppp/3p1n2/4p3/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq e6 0 6',
				'e7e6' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'g7g6' => 'rnbqkb1r/pp2pp1p/3p1np1/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/3B4/PPP2PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'B54e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Bd3

			variation => N__('Sicilian: Open, 2... d6, 4... Nf6 5. Bd3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/5P2/PPP3PP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B54f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3

			variation => N__('Sicilian: Prins (Moscow) Variation'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pppp/p2p1n2/8/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkb1r/pp2pppp/2np1n2/8/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq - 1 6',
				'e7e5' => 'rnbqkb1r/pp3ppp/3p1n2/4p3/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq e6 0 6',
				'e7e6' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'B54d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6

			variation => N__('Sicilian: Open, 2... d6, 4... Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5',
				'f1d3' => 'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/3B4/PPP2PPP/RNBQK2R b KQkq - 2 5',
				'f2f3' => 'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/5P2/PPP3PP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/8/3QP3/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 2 5' => {
			eco => 'B53k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nf6 5. Nc3

			variation => N__('Sicilian, Chekhover, 4... Nf6 5. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/3p1n2/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5' => {
			eco => 'B53k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 Nf6

			variation => N__('Sicilian, Chekhover, 4... Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/3p1n2/8/3QP3/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp2pppp/3p4/3nP3/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'B22t',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 d6 6. Nf3

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 d6'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2np4/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp2pppp/3p4/3nP3/3P4/8/PP3PPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'B22t',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 d6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 d6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp2pppp/3p4/3nP3/3P4/5N2/PP3PPP/RNBQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/1B1P4/8/8/PP1P1PPP/RNBQK1NR b KQkq - 2 5' => {
			eco => 'B10l',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5 cxd5 4. cxd5 Nf6 5. Bb5+

			variation => N__('Caro-Kann: English, Exchange, 4... Nf6 5. Bb5+'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/1BpP4/5P2/8/PPPP2PP/RNBQK1NR b KQkq - 2 4' => {
			eco => 'B21l',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5 3. exd5 Nf6 4. Bb5+

			variation => N__('Sicilian: Grand Prix, Tal Gambit, 4. Bb5+'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2PP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D06b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c5 3. cxd5 Nf6 4. dxc5

			variation => N__('QGD: Austrian, Rubinstein, 4.dxc5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2Pp4/3P4/8/PP3PPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'B13o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. c5

			variation => N__('Caro-Kann: Panov-Botvinnik, Gunderam Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2p5/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5' => {
			eco => 'D24d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 c5

			variation => N__('QGA: 4. Nc3 c5'),
			moves => {
				'd4d5' => 'rnbqkb1r/pp2pppp/5n2/2pP4/2p5/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2p5/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq c6 0 5' => {
			eco => 'D25b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 c5

			variation => N__('QGA: 4. e3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/2p1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'D20t',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e4 c5 4. d5'),
			moves => {
				'b7b5' => 'rnbqkb1r/p3pppp/5n2/1ppP4/2p1P3/2N5/PP3PPP/R1BQKBNR w KQkq b6 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/2p1P3/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'D20t',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e4 c5 4. d5'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/5n2/2pP4/2p1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/2p5/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D24e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 c5 5. d5

			variation => N__('QGA: 4. Nc3 c5 5. d5'),
			moves => {
				'e7e6' => 'rnbqkb1r/pp3ppp/4pn2/2pP4/2p5/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'D06c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c5 3. cxd5 Nf6 4. Nf3

			variation => N__('QGD: Austrian, Rubinstein, 4. Nf3'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp2pppp/5n2/3P4/3p4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'D06b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c5 3. cxd5 Nf6

			variation => N__('QGD: Austrian, Rubinstein Variation'),
			moves => {
				'd4c5' => 'rnbqkb1r/pp2pppp/5n2/2PP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkb1r/pp2pppp/5n2/2pP4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/5P2/8/PPPP2PP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B21k',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5 3. exd5 Nf6

			variation => N__('Sicilian: Grand Prix, Tal Gambit'),
			moves => {
				'f1b5' => 'rnbqkb1r/pp2pppp/5n2/1BpP4/5P2/8/PPPP2PP/RNBQK1NR b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/8/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A34',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 3 Knights'),
			moves => {
				'f6d5' => 'rnbqkb1r/pp2pppp/8/2pn4/8/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'B29e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. Nc3 d5 4. exd5

			variation => N__('Sicilian: Nimzowitsch, 3. Nc3 d5 4.exd5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pP4/8/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A34',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 2... Nf6'),
			moves => {
				'f6d5' => 'rnbqkb1r/pp2pppp/8/2pn4/8/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq c6 0 4' => {
			eco => 'D01g',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 c5

			variation => N__('Richter-Veresov: 3... c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq d6 0 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5

			variation => N__('English: Symmetrical, 3 Knights'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp2pppp/5n2/2pP4/8/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. g3 d5

			variation => N__('English: Symmetrical, 2... Nf6'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp2pppp/5n2/2pP4/8/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'D02o',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'e7e6' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq c6 0 4' => {
			eco => 'D02o',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4 c5

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'e2e3' => 'rnbqkb1r/pp2pppp/5n2/2pp4/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/3P4/2P1PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D04h',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 c5 4. c3

			variation => N__('Colle: 3... c5 4. c3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2n2n2/2pp4/3P4/2P1PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'b8d7' => 'r1bqkb1r/pp1npppp/5n2/2pp4/3P4/2P1PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'D04g',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 c5

			variation => N__('Colle: 3... c5'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp2pppp/5n2/2pp4/3P4/2P1PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq d6 0 4' => {
			eco => 'B29d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nf6 3. Nc3 d5

			variation => N__('Sicilian: Nimzowitsch, 3. Nc3 d5'),
			moves => {
				'e4d5' => 'rnbqkb1r/pp2pppp/5n2/2pP4/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/5P2/1P2PN2/P1PP2PP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. e3 c5 4. b3

			variation => N__('Bird: Lasker, 3... c5 4. b3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/5P2/4PN2/PPPP2PP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. e3 c5

			variation => N__('Bird: Lasker, 3... c5'),
			moves => {
				'b2b3' => 'rnbqkb1r/pp2pppp/5n2/2pp4/5P2/1P2PN2/P1PP2PP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nf6 4. O-O

			variation => N__('Reti: KIA, 2... c5 3. Bg2 Nf6 4.O-O'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2n2n2/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 4 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/2pp4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2 Nf6

			variation => N__('Reti: KIA, 2... c5 3. Bg2 Nf6'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp2pppp/5n2/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 3 4',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3P4/3p4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D06d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c5 3. cxd5 Nf6 4. Nf3 cxd4

			variation => N__('QGD: Austrian, Rubinstein, 4. Nf3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3P4/8/2N5/PP1P1PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B10m',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5 cxd5 4. cxd5 Nf6 5. Nc3

			variation => N__('Caro-Kann: English, Exchange, 4... Nf6 5. Nc3'),
			moves => {
				'f6d5' => 'rnbqkb1r/pp2pppp/8/3n4/8/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3P4/8/8/PP1P1PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B10k',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5 cxd5 4. cxd5 Nf6

			variation => N__('Caro-Kann: English, Exchange, 4... Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/5n2/3P4/8/2N5/PP1P1PPP/R1BQKBNR b KQkq - 2 5',
				'f1b5' => 'rnbqkb1r/pp2pppp/5n2/1B1P4/8/8/PP1P1PPP/RNBQK1NR b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B13p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5. Nc3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 3 6',
				'd5c4' => 'rnbqkb1r/pp2pppp/5n2/8/2pP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
				'e7e6' => 'rnbqkb1r/pp3ppp/4pn2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
				'g7g6' => 'rnbqkb1r/pp2pp1p/5np1/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B13o',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6

			variation => N__('Caro-Kann: Panov-Botvinnik, 4... Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/5n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5',
				'c4c5' => 'rnbqkb1r/pp2pppp/5n2/2Pp4/3P4/8/PP3PPP/RNBQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 5' => {
			eco => 'D10e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nc3 Nf6 5. Bf4

			variation => N__('Slav: Exchange, 4. Nc3 Nf6 5. Bf4'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR w KQkq - 4 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D13c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3

			variation => N__('Slav: Exchange, 5. Nc3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p2pppp/p4n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkb1r/pp2pppp/2n2n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6',
				'c8f5' => 'rn1qkb1r/pp2pppp/5n2/3p1b2/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6',
				'e7e6' => 'rnbqkb1r/pp3ppp/4pn2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'D10e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nc3 Nf6

			variation => N__('Slav: Exchange, 4. Nc3 Nf6'),
			moves => {
				'c1f4' => 'rnbqkb1r/pp2pppp/5n2/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/3P4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D13b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5

			variation => N__('Slav: Exchange'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp2pppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/5n2/3p4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'B13d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Nf3 Nf6

			variation => N__('Caro-Kann: Exchange, 4. Nf3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/8/2BP4/2N5/PP3PPP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'B13p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 dxc4 6. Bxc4

			variation => N__('Caro-Kann: Panov-Botvinnik, 5...dxc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/5n2/8/2pP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B13p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 dxc4

			variation => N__('Caro-Kann: Panov-Botvinnik, 5...dxc4'),
			moves => {
				'f1c4' => 'rnbqkb1r/pp2pppp/5n2/8/2BP4/2N5/PP3PPP/R1BQK1NR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/8/1Bp5/1n2P3/2N2N2/PP1P1PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. e4 Nb4 6. Bb5+

			variation => N__('English: Symmetrical, 3 Knights, 5. e4 Nb4 6. Bb5+'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/8/2p5/1n2P3/2N2N2/PP1P1PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. e4 Nb4

			variation => N__('English: Symmetrical, 3 Knights, 5. e4 Nb4'),
			moves => {
				'f1b5' => 'rnbqkb1r/pp2pppp/8/1Bp5/1n2P3/2N2N2/PP1P1PPP/R1BQK2R b KQkq - 2 6',
				'f1c4' => 'rnbqkb1r/pp2pppp/8/2p5/1nB1P3/2N2N2/PP1P1PPP/R1BQK2R b KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp2pppp/8/2p5/1nB1P3/2N2N2/PP1P1PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. e4 Nb4 6. Bc4

			variation => N__('English: Symmetrical, 3 Knights, 5. e4 Nb4 6. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/8/2p5/3P4/2n2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. d4 Nxc3

			variation => N__('English: Symmetrical, 3 Knights, 5. d4 Nxc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/8/2p5/4P3/2P2N2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A34',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 3 Knights, 5. e4 Nxc3'),
			moves => {
				'd8d1' => 'rnb1kb1r/pp2pppp/8/2p5/4P3/2P2N2/PP3PPP/R1BqKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp2pppp/8/2p5/4P3/2n2N2/PP1P1PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. e4 Nxc3

			variation => N__('English: Symmetrical, 3 Knights, 5. e4 Nxc3'),
			moves => {
				'd2c3' => 'rnbqkb1r/pp2pppp/8/2p5/4P3/2P2N2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/8/2pn4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 5' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. d4

			variation => N__('English: Symmetrical, 3 Knights, 5. d4'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp2pppp/8/3n4/3p4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
				'd5c3' => 'rnbqkb1r/pp2pppp/8/2p5/3P4/2n2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/8/2pn4/4P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. e4

			variation => N__('English: Symmetrical, 3 Knights, 5. e4'),
			moves => {
				'd5b4' => 'rnbqkb1r/pp2pppp/8/2p5/1n2P3/2N2N2/PP1P1PPP/R1BQKB1R w KQkq - 1 6',
				'd5c3' => 'rnbqkb1r/pp2pppp/8/2p5/4P3/2n2N2/PP1P1PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp2pppp/8/2pn4/8/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5

			variation => N__('English: Symmetrical, 3 Knights, Exchange'),
			moves => {
				'd2d4' => 'rnbqkb1r/pp2pppp/8/2pn4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 5',
				'e2e4' => 'rnbqkb1r/pp2pppp/8/2pn4/4P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 5',
				'g2g3' => 'rnbqkb1r/pp2pppp/8/2pn4/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp2pppp/8/2pn4/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. g3

			variation => N__('English: Symmetrical, 3 Knights, 5. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/8/2pn4/8/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A34',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, 2... Nf6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp2pppp/8/2pn4/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp2pppp/8/2pn4/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. g3 d5 4. cxd5 Nxd5 5. Bg2

			variation => N__('English: Symmetrical, 2... Nf6 3. g3'),
			moves => {
				'd5c7' => 'rnbqkb1r/ppn1pppp/8/2p5/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp2pppp/8/2pp4/3PnB2/4P3/PPP2PPP/RN1QKBNR w KQkq c6 0 5' => {
			eco => 'A45w',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 d5 4. e3 c5

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 d5 4. e3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/8/3n4/3p4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. Nf3 d5 4. cxd5 Nxd5 5. d4 cxd4

			variation => N__('English: Symmetrical, 3 Knights, 5. d4 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp2pppp/8/3n4/8/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B10n',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5 cxd5 4. cxd5 Nf6 5. Nc3 Nxd5

			variation => N__('Caro-Kann: English, Exchange, 4... Nf6 5. Nc3 Nxd5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3p1p/2p1pnp1/3p4/2PP4/4PN2/PP1N1PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D30t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6 5. Nbd2 g6

			variation => N__('QGD: Spielmann Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp3p1p/2p2np1/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 0 7' => {
			eco => 'D36a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 g6

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. Qc2 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP2B1/8/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bg5

			variation => N__('Benoni: Uhlmann Variation'),
			moves => {
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP2B1/8/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 8',
				'h7h6' => 'rnbqkb1r/pp3p2/3p1npp/2pP2B1/8/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'A70a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3

			variation => N__('Benoni: Classical'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3p1p/p2p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 8',
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 7' => {
			eco => 'A65k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3

			variation => N__('Benoni: Sämisch (6. e4 g6 7. f3)'),
			moves => {
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N4P/PP3PP1/R1BQKBNR b KQkq - 0 7' => {
			eco => 'A65b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. h3

			variation => N__('Benoni: 6. e4 g6 7. h3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'A65b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6

			variation => N__('Benoni: 6. e4 g6'),
			moves => {
				'c1f4' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4PB2/2N5/PP3PPP/R2QKBNR b KQkq - 1 7',
				'f1d3' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 1 7',
				'f2f3' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 7',
				'f2f4' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4PP2/2N5/PP4PP/R1BQKBNR b KQkq f3 0 7',
				'g1f3' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 7',
				'h2h3' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N4P/PP3PP1/R1BQKBNR b KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2NB4/PP3PPP/R1BQK1NR b KQkq - 1 7' => {
			eco => 'A65f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bd3

			variation => N__('Benoni: 6. e4 g6 7. Bd3'),
			moves => {
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4P3/2NB4/PP3PPP/R1BQK1NR w KQkq - 2 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4PB2/2N5/PP3PPP/R2QKBNR b KQkq - 1 7' => {
			eco => 'A65c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Bf4

			variation => N__('Benoni: 6. e4 g6 7. Bf4'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3p1p/p2p1np1/2pP4/4PB2/2N5/PP3PPP/R2QKBNR w KQkq - 0 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/4PP2/2N5/PP4PP/R1BQKBNR b KQkq f3 0 7' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4

			variation => N__('Benoni: Four Pawns Attack'),
			moves => {
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/4PP2/2N5/PP4PP/R1BQKBNR w KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/5B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bf4

			variation => N__('Benoni: 6. Nf3 g6 7. Bf4'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3p1p/p2p1np1/2pP4/5B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 8',
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/5B2/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N2N1P/PP2PPP1/R1BQKB1R b KQkq - 0 7' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. h3

			variation => N__('Benoni: 6. Nf3 g6 7. h3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6

			variation => N__('Benoni: 6. Nf3 g6'),
			moves => {
				'c1f4' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/5B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 7',
				'c1g5' => 'rnbqkb1r/pp3p1p/3p1np1/2pP2B1/8/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 7',
				'f3d2' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R b KQkq - 1 7',
				'g2g3' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 7',
				'h2h3' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N2N1P/PP2PPP1/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 7' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3

			variation => N__('Benoni: Fianchetto Variation'),
			moves => {
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R b KQkq - 1 7' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Nd2

			variation => N__('Benoni: Nimzowitsch Variation'),
			moves => {
				'b8d7' => 'r1bqkb1r/pp1n1p1p/3p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 8',
				'f8g7' => 'rnbqk2r/pp3pbp/3p1np1/2pP4/8/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/pp3p1p/3p1np1/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A56q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 g6

			variation => N__('Benoni: Czech, 5. e4 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3p2/2p1pn1p/6p1/2pPP2B/2N2N2/PP3PPP/R2QKB1R w KQkq g6 0 8' => {
			eco => 'D43l',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit, 7. e4'),
			moves => {
				'h4g3' => 'rnbqkb1r/pp3p2/2p1pn1p/6p1/2pPP3/2N2NB1/PP3PPP/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3p2/2p1pn1p/6p1/2pPP3/2N2NB1/PP3PPP/R2QKB1R b KQkq - 1 8' => {
			eco => 'D43l',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit, 7. e4'),
			moves => {
				'b7b5' => 'rnbqkb1r/p4p2/2p1pn1p/1p4p1/2pPP3/2N2NB1/PP3PPP/R2QKB1R w KQkq b6 0 9',
			},
		},
		'rnbqkb1r/pp3p2/3p1npp/2pP2B1/8/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 8' => {
			eco => 'A61',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. Bg5 h6

			variation => N__('Benoni: Uhlmann, 7... h6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3pp1/2p1pB1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D43o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bxf6

			variation => N__('Semi-Slav: Moscow, 6.Bxf6'),
			moves => {
				'd8f6' => 'rnb1kb1r/pp3pp1/2p1pq1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3pp1/2p1pn1p/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D43k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6

			variation => N__('Semi-Slav: Moscow Variation'),
			moves => {
				'g5f6' => 'rnbqkb1r/pp3pp1/2p1pB1p/3p4/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 0 6',
				'g5h4' => 'rnbqkb1r/pp3pp1/2p1pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3pp1/2p1pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D43k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit'),
			moves => {
				'd5c4' => 'rnbqkb1r/pp3pp1/2p1pn1p/8/2pP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7',
				'f8e7' => 'rnbqk2r/pp2bpp1/2p1pn1p/3p4/2PP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3pp1/2p1pn1p/8/2pP3B/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D43k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4 dxc4

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit Accepted'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp3pp1/2p1pn1p/8/2pPP2B/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 7',
			},
		},
		'rnbqkb1r/pp3pp1/2p1pn1p/8/2pPP2B/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 7' => {
			eco => 'D43l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 h6 6. Bh4 dxc4 7. e4

			variation => N__('Semi-Slav: Moscow, 6. Bh4 Gambit, 7. e4'),
			moves => {
				'g7g5' => 'rnbqkb1r/pp3p2/2p1pn1p/6p1/2pPP2B/2N2N2/PP3PPP/R2QKB1R w KQkq g6 0 8',
			},
		},
		'rnbqkb1r/pp3pp1/3ppn1p/6P1/3NP3/2N5/PPP2P1P/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B81o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. g5

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. g5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1P1/2N4P/PPP2P2/R1BQKB1R b KQkq - 0 7' => {
			eco => 'B81i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h3

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3pp1/p2ppn1p/8/3NP1P1/2N4P/PPP2P2/R1BQKB1R w KQkq - 0 8',
				'b8c6' => 'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1P1/2N4P/PPP2P2/R1BQKB1R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B81h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6'),
			moves => {
				'g4g5' => 'rnbqkb1r/pp3pp1/3ppn1p/6P1/3NP3/2N5/PPP2P1P/R1BQKB1R b KQkq - 0 7',
				'h1g1' => 'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1P1/2N5/PPP2P1P/R1BQKBR1 b Qkq - 1 7',
				'h2h3' => 'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1P1/2N4P/PPP2P2/R1BQKB1R b KQkq - 0 7',
				'h2h4' => 'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKB1R b KQkq h3 0 7',
			},
		},
		'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1P1/2N5/PPP2P1P/R1BQKBR1 b Qkq - 1 7' => {
			eco => 'B81p',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. Rg1

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. Rg1'),
			moves => {
			},
		},
		'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKB1R b KQkq h3 0 7' => {
			eco => 'B81j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 h6 7. h4

			variation => N__('Sicilian: Scheveningen, Keres, 6... h6 7. h4'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3pp1/2nppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKB1R w KQkq - 1 8',
				'f8e7' => 'rnbqk2r/pp2bpp1/3ppn1p/8/3NP1PP/2N5/PPP2P2/R1BQKB1R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3ppp/2P2n2/1B6/5p2/5N2/PPPP2PP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C36',
                        # TRANSLATORS:

			variation => N__('KGA: Scandinavian, Modern, 5. Bb5+ c6'),
			moves => {
				'b7c6' => 'rnbqkb1r/p4ppp/2p2n2/1B6/5p2/5N2/PPPP2PP/RNBQK2R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2n2n2/1B6/5p2/5N2/PPPP2PP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6' => {
			eco => 'D45d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Ne4

			variation => N__('Semi-Slav: 5. e3 Ne4'),
			moves => {
				'f1d3' => 'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/2NBPN2/PP3PPP/R1BQK2R b KQkq - 2 6' => {
			eco => 'D45d',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: 5. e3 Ne4'),
			moves => {
				'f7f5' => 'rnbqkb1r/pp4pp/2p1p3/3p1p2/2PPn3/2NBPN2/PP3PPP/R1BQK2R w KQkq f6 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/3BPN2/PP1N1PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'D30t',
                        # TRANSLATORS:

			variation => N__('QGD: 3. Nf3 Nf6 4. e3 c6 5.Nbd2'),
			moves => {
				'f7f5' => 'rnbqkb1r/pp4pp/2p1p3/3p1p2/2PPn3/3BPN2/PP1N1PPP/R1BQK2R w KQkq f6 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/4PN2/PP1N1PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'D30t',
                        # TRANSLATORS:

			variation => N__('QGD: 3. Nf3 Nf6 4. e3 c6 5.Nbd2'),
			moves => {
				'f1d3' => 'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/3BPN2/PP1N1PPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'D43f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5

			variation => N__('Semi-Slav: Botvinnik (Anti-Meran)'),
			moves => {
				'd5c4' => 'rnbqkb1r/pp3ppp/2p1pn2/6B1/2pP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
				'f8e7' => 'rnbqk2r/pp2bppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 2 6',
				'h7h6' => 'rnbqkb1r/pp3pp1/2p1pn1p/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c6

			variation => N__('QGD: 4. Bg5 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p2B1/2PP4/5N2/PPQ1PPPP/RN2KB1R b KQkq - 3 5' => {
			eco => 'D30o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6 4. Qc2 Nf6 5. Bg5

			variation => N__('QGD: 3. Nf3 c6 4. Qc2 Nf6 5. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2P5/1P3NP1/P2PPPBP/RNBQK2R b KQkq - 0 5' => {
			eco => 'A13s',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 c6 5. b3

			variation => N__('English: Neo-Catalan, 4... c6 5. b3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A13s',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 c6

			variation => N__('English: Neo-Catalan, 4... c6'),
			moves => {
				'b2b3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2P5/1P3NP1/P2PPPBP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'D43d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Qb3

			variation => N__('Semi-Slav: 5. Qb3'),
			moves => {
				'd5c4' => 'rnbqkb1r/pp3ppp/2p1pn2/8/2pP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'D31o',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e3 Nf6

			variation => N__('Semi-Slav: 4. e3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D45a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3

			variation => N__('Semi-Slav: 5. e3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p1p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6',
				'b8d7' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
				'f6e4' => 'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
				'f8d6' => 'rnbqk2r/pp3ppp/2pbpn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
				'f8e7' => 'rnbqk2r/pp2bppp/2p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D43a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6

			variation => N__('Semi-Slav'),
			moves => {
				'c1g5' => 'rnbqkb1r/pp3ppp/2p1pn2/3p2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5',
				'd1b3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/1QN2N2/PP2PPPP/R1B1KB1R b KQkq - 1 5',
				'd1d3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2NQ1N2/PP2PPPP/R1B1KB1R b KQkq - 1 5',
				'e2e3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'g2g3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D43b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. g3

			variation => N__('Semi-Slav: 5. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2NQ1N2/PP2PPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'D43c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Qd3

			variation => N__('Semi-Slav: 5. Qd3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/4PN2/PP1N1PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D30t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6 5. Nbd2

			variation => N__('QGD: 3. Nf3 Nf6 4. e3 c6 5.Nbd2'),
			moves => {
				'b8d7' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/4PN2/PP1N1PPP/R1BQKB1R w KQkq - 2 6',
				'f6e4' => 'rnbqkb1r/pp3ppp/2p1p3/3p4/2PPn3/4PN2/PP1N1PPP/R1BQKB1R w KQkq - 2 6',
				'g7g6' => 'rnbqkb1r/pp3p1p/2p1pnp1/3p4/2PP4/4PN2/PP1N1PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D30s',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6

			variation => N__('QGD: 3. Nf3 Nf6 4. e3 c6'),
			moves => {
				'b1d2' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/4PN2/PP1N1PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R w KQkq - 2 5' => {
			eco => 'D30n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6 4. Qc2 Nf6

			variation => N__('QGD: 3. Nf3 c6 4. Qc2 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/pp3ppp/2p1pn2/3p2B1/2PP4/5N2/PPQ1PPPP/RN2KB1R b KQkq - 3 5',
				'g2g3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PP1P/RNB1KB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 0 5' => {
			eco => 'E00p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 c6

			variation => N__('Catalan: 4. Nf3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 c6 5. Nf3

			variation => N__('Catalan: 4... c6 5. Nf3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pp1n1ppp/2p1pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5NP1/PPQ1PP1P/RNB1KB1R b KQkq - 0 5' => {
			eco => 'D30p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6 4. Qc2 Nf6 5. g3

			variation => N__('QGD: 3. Nf3 c6 4. Qc2 Nf6 5. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 c6

			variation => N__('Catalan: 4... c6'),
			moves => {
				'd1c2' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/6P1/PPQ1PPBP/RNB1K1NR b KQkq - 1 5',
				'g1f3' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/6P1/PPQ1PPBP/RNB1K1NR b KQkq - 1 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 c6 5. Qc2

			variation => N__('Catalan: 4... c6 5. Qc2'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/6B1/2pP4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D44a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4

			variation => N__('Semi-Slav: Botvinnik Accepted'),
			moves => {
				'a2a4' => 'rnbqkb1r/pp3ppp/2p1pn2/6B1/P1pP4/2N2N2/1P2PPPP/R2QKB1R b KQkq a3 0 6',
				'e2e4' => 'rnbqkb1r/pp3ppp/2p1pn2/6B1/2pPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/6B1/2pPP3/2N2N2/PP3PPP/R2QKB1R b KQkq e3 0 6' => {
			eco => 'D44c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. e4

			variation => N__('Semi-Slav: Botvinnik, 6. e4'),
			moves => {
				'b7b5' => 'rnbqkb1r/p4ppp/2p1pn2/1p4B1/2pPP3/2N2N2/PP3PPP/R2QKB1R w KQkq b6 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/6B1/P1pP4/2N2N2/1P2PPPP/R2QKB1R b KQkq a3 0 6' => {
			eco => 'D44b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Bg5 dxc4 6. a4

			variation => N__('Semi-Slav: Botvinnik, 6. a4'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/8/2PPN3/8/PP3PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'D31w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4 Nf6

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6' => {
			eco => 'D43e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. Qb3 dxc4 6. Qxc4

			variation => N__('Semi-Slav: 5. Qb3 dxc4'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/8/2pP4/1QN2N2/PP2PPPP/R1B1KB1R w KQkq - 0 6' => {
			eco => 'D43d',
                        # TRANSLATORS:

			variation => N__('Semi-Slav: 5. Qb3'),
			moves => {
				'b3c4' => 'rnbqkb1r/pp3ppp/2p1pn2/8/2QP4/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p1pn2/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D16c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. Nc3 dxc4 5. a4 e6

			variation => N__('Slav: Soultanbeieff Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/1B1P4/5p2/5N2/PPPP2PP/RNBQK2R w KQkq - 0 6' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Bb5+ c6

			variation => N__('KGA: Scandinavian, Modern, 5. Bb5+ c6'),
			moves => {
				'd5c6' => 'rnbqkb1r/pp3ppp/2P2n2/1B6/5p2/5N2/PPPP2PP/RNBQK2R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3Pp3/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 0 5' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Bg2 d5 5. cxd5

			variation => N__('English: Bremen, Keres, 4. Bg2 d5 5.cxd5'),
			moves => {
				'c6d5' => 'rnbqkb1r/pp3ppp/5n2/3pp3/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D35m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. e3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. e3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pp1n1ppp/2p2n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7',
				'f8e7' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D35l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Nf3

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D35l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6

			variation => N__('QGD: Exchange, 5. Bg5 c6'),
			moves => {
				'd1c2' => 'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR b KQkq - 1 6',
				'e2e3' => 'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 6',
				'g1f3' => 'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR b KQkq - 1 6' => {
			eco => 'D36a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2

			variation => N__('QGD: Exchange, 5. Bg5 c6 6. Qc2'),
			moves => {
				'b8a6' => 'r1bqkb1r/pp3ppp/n1p2n2/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 2 7',
				'f8e7' => 'rnbqk2r/pp2bppp/2p2n2/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 2 7',
				'g7g6' => 'rnbqkb1r/pp3p1p/2p2np1/3p2B1/3P4/2N5/PPQ1PPPP/R3KBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C01e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nc3 c6

			variation => N__('French: Exchange, 4. c4 Nf6 5. Nc3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D35d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 c6 6. e3

			variation => N__('QGD: Exchange, 5. Nf3 c6 6. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D35d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3 c6

			variation => N__('QGD: Exchange, 5. Nf3 c6'),
			moves => {
				'e2e3' => 'rnbqkb1r/pp3ppp/2p2n2/3p4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/2p2n2/3pp3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq d6 0 5' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Bg2 d5

			variation => N__('English: Bremen, Keres, 4. Bg2 d5'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp3ppp/2p2n2/3Pp3/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/2p2p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 0 5' => {
			eco => 'D00k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 exf6 4. e3 c6

			variation => N__('Trompowsky: 2... d5 3.Bxf6 exf6 4. e3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2p2/8/2BP4/8/PPP2PPP/R1BQK1NR b KQkq - 1 6' => {
			eco => 'B15u',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ exf6 6. Bc4

			variation => N__('Caro-Kann: Tartakower, Forgacs Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2p2/8/3P4/2P5/PP3PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'B15v',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ exf6 6. c3

			variation => N__('Caro-Kann: Tartakower, 6. c3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/2p2p2/8/3P4/8/PPP2PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B15t',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6 5. Nxf6+ exf6

			variation => N__('Caro-Kann: Tartakower (Nimzowitsch) Variation'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp3ppp/2p2p2/8/3P4/2P5/PP3PPP/R1BQKBNR b KQkq - 0 6',
				'f1c4' => 'rnbqkb1r/pp3ppp/2p2p2/8/2BP4/8/PPP2PPP/R1BQK1NR b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/2pp1n2/4p3/2P5/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Nf3 d6

			variation => N__('English: Bremen, Keres, 4. Nf3 d6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp3ppp/2pp1n2/4p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/2pp1n2/4p3/2P5/2N2NP1/PP1PPPBP/R1BQK2R b KQkq - 1 5' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Nf3 d6 5. Bg2

			variation => N__('English: Bremen, Keres, 4. Nf3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/1B2p3/3NP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B56c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5 6. Bb5+

			variation => N__('Sicilian: Venice Attack'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/1B2p3/3NP3/5P2/PPP3PP/RNBQK2R b KQkq - 1 6' => {
			eco => 'B55',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Bb5+

			variation => N__('Sicilian: Prins, Venice Attack'),
			moves => {
				'b8d7' => 'r1bqkb1r/pp1n1ppp/3p1n2/1B2p3/3NP3/5P2/PPP3PP/RNBQK2R w KQkq - 2 7',
				'c8d7' => 'rn1qkb1r/pp1b1ppp/3p1n2/1B2p3/3NP3/5P2/PPP3PP/RNBQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6' => {
			eco => 'A65a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4

			variation => N__('Benoni: 6. e4'),
			moves => {
				'f8e7' => 'rnbqk2r/pp2bppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 7',
				'g7g6' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'A60q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3

			variation => N__('Benoni: 4. Nc3 exd5 5.cxd5 d6 6. Nf3'),
			moves => {
				'g7g6' => 'rnbqkb1r/pp3p1p/3p1np1/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A60p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6

			variation => N__('Benoni: 4. Nc3 exd5 5.cxd5 d6'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp3ppp/3p1n2/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6',
				'g1f3' => 'rnbqkb1r/pp3ppp/3p1n2/2pP4/8/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5' => {
			eco => 'A56p',
                        # TRANSLATORS:

			variation => N__('Benoni: Czech, 4. Nc3 d6'),
			moves => {
				'f8e7' => 'rnbqk2r/pp2bppp/3p1n2/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6',
				'g7g6' => 'rnbqkb1r/pp3p1p/3p1np1/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/2pPp3/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A56p',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6

			variation => N__('Benoni: Czech, 4. Nc3 d6'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp3ppp/3p1n2/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'A44r',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 Nf6

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/4p3/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 0 7' => {
			eco => 'B81b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4 e5

			variation => N__('Sicilian: Scheveningen, Keres, 6... e5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/4p3/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq e6 0 6' => {
			eco => 'B56b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e5

			variation => N__('Sicilian: Open, 2... d6, 5. Nc3 e5'),
			moves => {
				'f1b5' => 'rnbqkb1r/pp3ppp/3p1n2/1B2p3/3NP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/4p3/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq e6 0 6' => {
			eco => 'B54l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5

			variation => N__('Sicilian: Prins (Moscow), 5... e5'),
			moves => {
				'd4b3' => 'rnbqkb1r/pp3ppp/3p1n2/4p3/4P3/1N3P2/PPP3PP/RNBQKB1R b KQkq - 1 6',
				'f1b5' => 'rnbqkb1r/pp3ppp/3p1n2/1B2p3/3NP3/5P2/PPP3PP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/3p1n2/4p3/4P3/1N3P2/PPP3PP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'B54m',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Nb3

			variation => N__('Sicilian: Prins (Moscow), 5... e5 6. Nb3'),
			moves => {
				'c8e6' => 'rn1qkb1r/pp3ppp/3pbn2/4p3/4P3/1N3P2/PPP3PP/RNBQKB1R w KQkq - 2 7',
				'd6d5' => 'rnbqkb1r/pp3ppp/5n2/3pp3/4P3/1N3P2/PPP3PP/RNBQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/3pp3/3nP3/3P4/5N2/PP3PPP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B22w',
                        # TRANSLATORS: 1. e4 c5 2. c3 Nf6 3. e5 Nd5 4. d4 cxd4 5. cxd4 e6 6. Nf3 d6

			variation => N__('Sicilian: Alapin, 2... Nf6, 5.cxd4 e6 6. Nf3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/1B6/3NP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B80n',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bb5+

			variation => N__('Sicilian: Scheveningen, Vitolins Variation'),
			moves => {
				'c8d7' => 'rn1qkb1r/pp1b1ppp/3ppn2/1B6/3NP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B86',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Bc4

			variation => N__('Sicilian: Sozin-Scheveningen'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2nppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7',
				'd8b6' => 'rnb1kb1r/pp3ppp/1q1ppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R b KQkq g3 0 6' => {
			eco => 'B81a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g4

			variation => N__('Sicilian: Scheveningen, Keres Attack'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 1 7',
				'e6e5' => 'rnbqkb1r/pp3ppp/3p1n2/4p3/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 0 7',
				'h7h6' => 'rnbqkb1r/pp3pp1/3ppn1p/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'B80b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be3

			variation => N__('Sicilian: Scheveningen, 6. Be3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'B80o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. g3

			variation => N__('Sicilian: Scheveningen, Fianchetto Variation'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B83',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. Be2

			variation => N__('Sicilian: Scheveningen, 6. Be2'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
				'f8e7' => 'rnbqk2r/pp2bppp/3ppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B80a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6

			variation => N__('Sicilian: Scheveningen'),
			moves => {
				'c1e3' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N1B3/PPP2PPP/R2QKB1R b KQkq - 1 6',
				'f1b5' => 'rnbqkb1r/pp3ppp/3ppn2/1B6/3NP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6',
				'f1c4' => 'rnbqkb1r/pp3ppp/3ppn2/8/2BNP3/2N5/PPP2PPP/R1BQK2R b KQkq - 1 6',
				'f1e2' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N5/PPP1BPPP/R1BQK2R b KQkq - 1 6',
				'f2f4' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6',
				'g2g3' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/2N3P1/PPP2P1P/R1BQKB1R b KQkq - 0 6',
				'g2g4' => 'rnbqkb1r/pp3ppp/3ppn2/8/3NP1P1/2N5/PPP2P1P/R1BQKB1R b KQkq g3 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NP3/5P2/PPP3PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B54g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e6

			variation => N__('Sicilian: Prins (Moscow), 5... e6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/3ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R b KQkq f3 0 6' => {
			eco => 'B82',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. Nc3 e6 6. f4

			variation => N__('Sicilian: Scheveningen, 6. f4'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p2ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2nppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'd8b6' => 'rnb1kb1r/pp3ppp/1q1ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
				'f8e7' => 'rnbqk2r/pp2bppp/3ppn2/8/3NPP2/2N5/PPP3PP/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/2p5/3PP3/2P2N2/P4PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D41k',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4 Nxc3'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp3ppp/4p3/8/3pP3/2P2N2/P4PPP/R1BQKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/2p5/3PP3/2n2N2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D41k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3

			variation => N__('QGD: Semi-Tarrasch, 6. e4 Nxc3'),
			moves => {
				'b2c3' => 'rnbqkb1r/pp3ppp/4p3/2p5/3PP3/2P2N2/P4PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/2pn4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D41p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3

			variation => N__('QGD: Semi-Tarrasch, 6. e3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 7',
				'c5d4' => 'rnbqkb1r/pp3ppp/4p3/3n4/3p4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/2pn4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D41c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5

			variation => N__('QGD: Semi-Tarrasch, 5.cxd5 Nxd5'),
			moves => {
				'e2e3' => 'rnbqkb1r/pp3ppp/4p3/2pn4/3P4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 6',
				'e2e4' => 'rnbqkb1r/pp3ppp/4p3/2pn4/3PP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 6',
				'g2g3' => 'rnbqkb1r/pp3ppp/4p3/2pn4/3P4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/2pn4/3P4/2N2NP1/PP2PP1P/R1BQKB1R b KQkq - 0 6' => {
			eco => 'D41c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. g3

			variation => N__('QGD: Semi-Tarrasch, 6. g3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1p3/2pn4/3P4/2N2NP1/PP2PP1P/R1BQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/2pn4/3PP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 6' => {
			eco => 'D41k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4

			variation => N__('QGD: Semi-Tarrasch, 6. e4'),
			moves => {
				'd5c3' => 'rnbqkb1r/pp3ppp/4p3/2p5/3PP3/2n2N2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/3n4/3P4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D41p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 cxd4 7. exd4

			variation => N__('QGD: Semi-Tarrasch, 6. e3 cxd4 7.exd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4p3/3n4/3p4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D41p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e3 cxd4

			variation => N__('QGD: Semi-Tarrasch, 6. e3 cxd4'),
			moves => {
				'e3d4' => 'rnbqkb1r/pp3ppp/4p3/3n4/3P4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/8/3PP3/5N2/P4PPP/R1BQKB1R b KQkq - 0 8' => {
			eco => 'D41k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 Nxd5 6. e4 Nxc3 7. bxc3 cxd4 8. cxd4

			variation => N__('QGD: Semi-Tarrasch, 6. e4, 8.cxd4'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1p3/8/3PP3/5N2/P4PPP/R1BQKB1R w KQkq - 1 9',
				'f8b4' => 'rnbqk2r/pp3ppp/4p3/8/1b1PP3/5N2/P4PPP/R1BQKB1R w KQkq - 1 9',
			},
		},
		'rnbqkb1r/pp3ppp/4p3/8/3pP3/2P2N2/P4PPP/R1BQKB1R w KQkq - 0 8' => {
			eco => 'D41k',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, 6. e4 Nxc3'),
			moves => {
				'c3d4' => 'rnbqkb1r/pp3ppp/4p3/8/3PP3/5N2/P4PPP/R1BQKB1R b KQkq - 0 8',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2P5/2B5/4PN2/PP3PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'D26g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. dxc5

			variation => N__('QGA: Classical, Early Queenswap'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'D26h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Nc3

			variation => N__('QGA: Classical, 6. Nc3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQK2R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/4PN2/PP2QPPP/RNB1K2R b KQkq - 1 6' => {
			eco => 'D26j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Qe2

			variation => N__('QGA: Classical, Furman Variation'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/4PN2/PP2QPPP/RNB1K2R w KQkq - 0 7',
				'c5d4' => 'rnbqkb1r/pp3ppp/4pn2/8/2Bp4/4PN2/PP2QPPP/RNB1K2R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/4PN2/PP3PPP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'D26p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O

			variation => N__('QGA: Classical, 6.O-O'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p3pn2/2p5/2BP4/4PN2/PP3PPP/RNBQ1RK1 w kq - 0 7',
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2p5/2BP4/4PN2/PP3PPP/RNBQ1RK1 w kq - 2 7',
				'c5d4' => 'rnbqkb1r/pp3ppp/4pn2/8/2Bp4/4PN2/PP3PPP/RNBQ1RK1 w kq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq c6 0 6' => {
			eco => 'D26f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5

			variation => N__('QGA: Classical Variation'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/2N1PN2/PP3PPP/R1BQK2R b KQkq - 1 6',
				'd1e2' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/4PN2/PP2QPPP/RNB1K2R b KQkq - 1 6',
				'd4c5' => 'rnbqkb1r/pp3ppp/4pn2/2P5/2B5/4PN2/PP3PPP/RNBQK2R b KQkq - 0 6',
				'e1g1' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/4PN2/PP3PPP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2PPp3/2N2P2/PP4PP/R1BQKBNR w KQkq c6 0 6' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 Nf6 5. f3 c5

			variation => N__('Diemer-Duhm Gambit (DDG): Keres Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'E04k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5 6. O-O

			variation => N__('Catalan: Open, 5. Nf3 c5 6.O-O'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2p5/2pP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2p5/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq c6 0 6' => {
			eco => 'E04k',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3 c5

			variation => N__('Catalan: Open, 5. Nf3 c5'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2pP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pP2B1/3P4/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. cxd5

			variation => N__('QGD: Dutch-Peruvian, 5.cxd5'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp3ppp/4pn2/3P2B1/3p4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
				'd8b6' => 'rnb1kb1r/pp3ppp/1q2pn2/2pP2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pP4/2p1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 6' => {
			eco => 'D24f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 c5 5. d5 e6 6. e4

			variation => N__('QGA: 4. Nc3 c5 5. d5 e6 6. e4'),
			moves => {
				'e6d5' => 'rnbqkb1r/pp3ppp/5n2/2pp4/2p1P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pP4/2p5/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D24e',
                        # TRANSLATORS:

			variation => N__('QGA: 4. Nc3 c5 5. d5'),
			moves => {
				'e2e4' => 'rnbqkb1r/pp3ppp/4pn2/2pP4/2p1P3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pP4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D41a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5

			variation => N__('QGD: Semi-Tarrasch, 5.cxd5'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp3ppp/4pn2/3P4/3p4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
				'f6d5' => 'rnbqkb1r/pp3ppp/4p3/2pn4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pP4/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C07i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Nf6

			variation => N__('French: Tarrasch, Shaposhnikov Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp2B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. e3

			variation => N__('QGD: Dutch-Peruvian, 5. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. Nf3

			variation => N__('QGD: Semi-Tarrasch, Pillsbury Variation'),
			moves => {
				'c5d4' => 'rnbqkb1r/pp3ppp/4pn2/3p2B1/2Pp4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq c6 0 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5

			variation => N__('QGD: Dutch-Peruvian Gambit'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp3ppp/4pn2/2pP2B1/3P4/2N5/PP2PPPP/R2QKBNR b KQkq - 0 5',
				'e2e3' => 'rnbqkb1r/pp3ppp/4pn2/2pp2B1/2PP4/2N1P3/PP3PPP/R2QKBNR b KQkq - 0 5',
				'g1f3' => 'rnbqkb1r/pp3ppp/4pn2/2pp2B1/2PP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp2B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq c6 0 5' => {
			eco => 'D03g',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3 c5

			variation => N__('Torre Attack: 3... e6 4. e3 c5'),
			moves => {
				'b1d2' => 'rnbqkb1r/pp3ppp/4pn2/2pp2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp2B1/3P4/4PN2/PPPN1PPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'D03g',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... e6 4. e3 c5'),
			moves => {
				'b8d7' => 'r1bqkb1r/pp1n1ppp/4pn2/2pp2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQkq - 2 6',
				'f8e7' => 'rnbqk2r/pp2bppp/4pn2/2pp2B1/3P4/4PN2/PPPN1PPP/R2QKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'A13n',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. b3 Nf6 4. Bb2 c5 5. e3

			variation => N__('English: Wimpey System'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/1P3N2/PB1PPPPP/RN1QKB1R w KQkq c6 0 5' => {
			eco => 'A13m',
                        # TRANSLATORS:

			variation => N__('English: Agincourt Variation'),
			moves => {
				'e2e3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/1P2PN2/PB1P1PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'A13t',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 c5 5. O-O

			variation => N__('English: Neo-Catalan, 4... c5 5.O-O'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq c6 0 5' => {
			eco => 'A13t',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 c5

			variation => N__('English: Neo-Catalan, 4... c5'),
			moves => {
				'e1g1' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/5NP1/PP1PPPBP/RNBQ1RK1 b kq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D40a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3

			variation => N__('QGD: Semi-Tarrasch, 5. e3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p3pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
				'c5d4' => 'rnbqkb1r/pp3ppp/4pn2/3p4/2Pp4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6',
				'f8e7' => 'rnbqk2r/pp2bppp/4pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5' => {
			eco => 'D40a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5

			variation => N__('QGD: Semi-Tarrasch'),
			moves => {
				'c4d5' => 'rnbqkb1r/pp3ppp/4pn2/2pP4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 0 5',
				'e2e3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'D30h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. e3 Nf6

			variation => N__('QGD: Tarrasch without Nc3: 4. e3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq c6 0 5' => {
			eco => 'E00q',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 c5

			variation => N__('Catalan: 4. Nf3 c5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 c5 5. Nf3

			variation => N__('Catalan: 4... c5 5. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq c6 0 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 c5

			variation => N__('Catalan: 4... c5'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/2P1PN2/PP3PPP/RN1QKB1R b KQkq - 0 5' => {
			eco => 'D02n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P1B2/2P1PN2/PP3PPP/RN1QKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'D02p',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4 c5 4. e3 e6

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/2P1PN2/PP3PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq c6 0 5' => {
			eco => 'D02n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'c2c3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/2P1PN2/PP3PPP/RN1QKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/1P1BPN2/P1P2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D05f',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. b3

			variation => N__('Colle: Rubinstein\'s Attack'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P4/1P1BPN2/P1P2PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/1P2PN2/P1PN1PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D05c',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Nbd2 c5 5. b3

			variation => N__('Colle: Zukertort Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/2PBPN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D05h',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5 5. c3

			variation => N__('Colle: 5. c3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/2pp4/3P4/2PBPN2/PP3PPP/RNBQK2R w KQkq - 1 6',
				'b8d7' => 'r1bqkb1r/pp1n1ppp/4pn2/2pp4/3P4/2PBPN2/PP3PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq c6 0 5' => {
			eco => 'D05e',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3 c5

			variation => N__('Colle: 3... e6 4. Bd3 c5'),
			moves => {
				'b2b3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/1P1BPN2/P1P2PPP/RNBQK2R b KQkq - 0 5',
				'c2c3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/2PBPN2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/4PN2/PPPN1PPP/R1BQKB1R w KQkq c6 0 5' => {
			eco => 'D05b',
                        # TRANSLATORS:

			variation => N__('Colle: 3... e6 4.Nbd2'),
			moves => {
				'b2b3' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/1P2PN2/P1PN1PPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5' => {
			eco => 'C07f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. Ngf3 Nf6

			variation => N__('French: Tarrasch, Open, 4.Ngf3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3P2B1/3Q4/2N5/PP2PPPP/R3KBNR b KQkq - 0 6' => {
			eco => 'D50',
                        # TRANSLATORS:

			variation => N__('QGD: Dutch-Peruvian, Prins Variation'),
			moves => {
				'f8e7' => 'rnbqk2r/pp2bppp/4pn2/3P2B1/3Q4/2N5/PP2PPPP/R3KBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3P2B1/3p4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. cxd5 cxd4

			variation => N__('QGD: Dutch-Peruvian, Prins Variation'),
			moves => {
				'd1d4' => 'rnbqkb1r/pp3ppp/4pn2/3P2B1/3Q4/2N5/PP2PPPP/R3KBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3P4/3p4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D41b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. cxd5 cxd4

			variation => N__('QGD: Semi-Tarrasch, Keres Counterattack'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p2B1/2PN4/2N5/PP2PPPP/R2QKB1R b KQkq - 0 6' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. Nf3 cxd4 6. Nxd4

			variation => N__('QGD: Semi-Tarrasch, Pillsbury, 6.Nxd4'),
			moves => {
				'e6e5' => 'rnbqkb1r/pp3ppp/5n2/3pp1B1/2PN4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p2B1/2PQ4/2N2N2/PP2PPPP/R3KB1R b KQkq - 0 6' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. Nf3 cxd4 6. Qxd4

			variation => N__('QGD: Semi-Tarrasch, Old Pillsbury Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p2B1/2Pp4/2N2N2/PP2PPPP/R2QKB1R w KQkq - 0 6' => {
			eco => 'D50',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, Pillsbury Variation'),
			moves => {
				'd1d4' => 'rnbqkb1r/pp3ppp/4pn2/3p2B1/2PQ4/2N2N2/PP2PPPP/R3KB1R b KQkq - 0 6',
				'f3d4' => 'rnbqkb1r/pp3ppp/4pn2/3p2B1/2PN4/2N5/PP2PPPP/R2QKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq d6 0 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. Nc3 d5

			variation => N__('English: Symmetrical, Two Knights, 5. Nc3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/2PN4/6P1/PP2PP1P/RNBQKB1R w KQkq d6 0 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 d5

			variation => N__('English: Symmetrical, Two Knights, 5. g3 d5'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp3ppp/4pn2/3p4/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/2PN4/6P1/PP2PPBP/RNBQK2R b KQkq - 1 6' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 d5 6. Bg2

			variation => N__('English: Symmetrical, Two Knights, 5. g3 d5 6. Bg2'),
			moves => {
				'e6e5' => 'rnbqkb1r/pp3ppp/5n2/3pp3/2PN4/6P1/PP2PPBP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B14j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6 6. Nf3

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6 6. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pp3ppp/2n1pn2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7',
				'f8b4' => 'rnbqk2r/pp3ppp/4pn2/3p4/1bPP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7',
				'f8e7' => 'rnbqk2r/pp2bppp/4pn2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'B14i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 Nf6 5. Nc3 e6

			variation => N__('Caro-Kann: Panov-Botvinnik, 5... e6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pp3ppp/4pn2/3p4/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/2Pp4/2N1PN2/PP3PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D40c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c5 5. e3 cxd4

			variation => N__('QGD: Semi-Tarrasch, 5. e3 cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6' => {
			eco => 'D13i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 e6 6. Bf4

			variation => N__('Slav: Exchange, 5. Nc3 e6 6. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D13h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 Nf6 4. cxd5 cxd5 5. Nc3 e6

			variation => N__('Slav: Exchange, 5. Nc3 e6'),
			moves => {
				'c1f4' => 'rnbqkb1r/pp3ppp/4pn2/3p4/3P1B2/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/8/2Bp4/4PN2/PP2QPPP/RNB1K2R w KQkq - 0 7' => {
			eco => 'D26k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. Qe2 cxd4

			variation => N__('QGA: Classical, Furman, 6...cxd4'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/4pn2/8/2Bp4/4PN2/PP3PPP/RNBQ1RK1 w kq - 0 7' => {
			eco => 'D26q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O cxd4

			variation => N__('QGA: Classical, Steinitz Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/5n2/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 3 6' => {
			eco => 'C08j',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6'),
			moves => {
				'c8d7' => 'rn1qkb1r/pp1b1ppp/5n2/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R w KQkq - 4 7',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/1N1pp1B1/2P5/2N5/PP2PPPP/R2QKB1R b KQkq - 1 7' => {
			eco => 'D50',
                        # TRANSLATORS:

			variation => N__('QGD: Semi-Tarrasch, Pillsbury, 6.Nxd4 e5'),
			moves => {
				'a7a6' => 'rnbqkb1r/1p3ppp/p4n2/1N1pp1B1/2P5/2N5/PP2PPPP/R2QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/2pp4/2p1P3/2N2N2/PP3PPP/R1BQKB1R w KQkq - 0 7' => {
			eco => 'D24f',
                        # TRANSLATORS:

			variation => N__('QGA: 4. Nc3 c5 5. d5 e6 6. e4'),
			moves => {
				'e4e5' => 'rnbqkb1r/pp3ppp/5n2/2ppP3/2p5/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'C08j',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 Nf6

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 Nf6'),
			moves => {
				'f1b5' => 'rnbqkb1r/pp3ppp/5n2/1Bpp4/3P4/5N2/PPPN1PPP/R1BQK2R b KQkq - 3 6',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/2pp4/3P4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nf6

			variation => N__('QGD: Tarrasch without Nc3: 5. g3'),
			moves => {
				'f1g2' => 'rnbqkb1r/pp3ppp/5n2/2pp4/3P4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/2pp4/3P4/5NP1/PP2PPBP/RNBQK2R b KQkq - 2 6' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3 Nf6 6. Bg2

			variation => N__('QGD: Tarrasch without Nc3: 5. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/5n2/2ppP3/2p5/2N2N2/PP3PPP/R1BQKB1R b KQkq - 0 7' => {
			eco => 'D24f',
                        # TRANSLATORS:

			variation => N__('QGA: 4. Nc3 c5 5. d5 e6 6. e4'),
			moves => {
				'f6d7' => 'rnbqkb1r/pp1n1ppp/8/2ppP3/2p5/2N2N2/PP3PPP/R1BQKB1R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/3pp1B1/2PN4/2N5/PP2PPPP/R2QKB1R w KQkq - 0 7' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 c5 5. Nf3 cxd4 6. Nxd4 e5

			variation => N__('QGD: Semi-Tarrasch, Pillsbury, 6.Nxd4 e5'),
			moves => {
				'd4b5' => 'rnbqkb1r/pp3ppp/5n2/1N1pp1B1/2P5/2N5/PP2PPPP/R2QKB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/3pp3/2P5/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 7' => {
			eco => 'A32',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 Nf6 3. d4 cxd4 4. Nxd4 e6 5. g3 d5 6. Bg2 e5 7. Nf3

			variation => N__('English: Symmetrical, Two Knights, 5. g3 d5 6. Bg2 e5 7. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/5n2/3pp3/2PN4/6P1/PP2PPBP/RNBQK2R w KQkq - 0 7' => {
			eco => 'A32',
                        # TRANSLATORS:

			variation => N__('English: Symmetrical, Two Knights, 5. g3 d5 6. Bg2'),
			moves => {
				'd4f3' => 'rnbqkb1r/pp3ppp/5n2/3pp3/2P5/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/pp3ppp/5n2/3pp3/4P3/1N3P2/PPP3PP/RNBQKB1R w KQkq - 0 7' => {
			eco => 'B54o',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 Nf6 5. f3 e5 6. Nb3 d5

			variation => N__('Sicilian: Prins (Moscow), 5... e5 6. Nb3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/pp3ppp/5n2/3pp3/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A23',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 c6 4. Bg2 d5 5. cxd5 cxd5

			variation => N__('English: Bremen, Keres, 4. Bg2 d5 5.cxd5 cxd5'),
			moves => {
			},
		},
		'rnbqkb1r/pp4pp/2p1p3/3p1p2/2PPn3/2NBPN2/PP3PPP/R1BQK2R w KQkq f6 0 7' => {
			eco => 'D45d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3 c6 5. e3 Ne4 6. Bd3 f5

			variation => N__('Semi-Slav: Stonewall Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pp4pp/2p1p3/3p1p2/2PPn3/3BPN2/PP1N1PPP/R1BQK2R w KQkq f6 0 7' => {
			eco => 'D30t',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3 c6 5. Nbd2 Ne4 6. Bd3 f5

			variation => N__('QGD: Stonewall Formation'),
			moves => {
			},
		},
		'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2P5/5NP1/PP1PPPBP/RNBQ1RK1 w kq d6 0 6' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6 4. Bg2 c6 5. O-O d5

			variation => N__('English: Anglo-Dutch'),
			moves => {
			},
		},
		'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5 6. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 5. Nf3 d5 6. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5 6. O-O

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 5. Nf3 d5 6.O-O'),
			moves => {
				'f8d6' => 'rnbqk2r/pp4pp/2pbpn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq - 2 7',
			},
		},
		'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq d6 0 6' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5'),
			moves => {
				'b1c3' => 'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/2N2NP1/PP2PPBP/R1BQK2R b KQkq - 1 6',
				'd1c2' => 'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB1K2R b KQkq - 1 6',
				'e1g1' => 'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 1 6',
			},
		},
		'rnbqkb1r/pp4pp/2p1pn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB1K2R b KQkq - 1 6' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 c6 5. Nf3 d5 6. Qc2

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 5. Nf3 d5 6. Qc2'),
			moves => {
				'f8d6' => 'rnbqk2r/pp4pp/2pbpn2/3p1p2/2PP4/5NP1/PPQ1PPBP/RNB1K2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/pp4pp/2p2p2/3p1p2/3P4/2N1P3/PPP2PPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'A80p',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. Bg5 d5 4. Bxf6 exf6 5. e3 c6

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5 d5 4.Bxf6 exf6 5. e3 c6'),
			moves => {
			},
		},
		'rnbqkb1r/ppn1pppp/8/2p5/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 Nf6 3. g3 d5 4. cxd5 Nxd5 5. Bg2 Nc7

			variation => N__('English: Symmetrical, Rubinstein System'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1nppp/4p3/3pP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'C02a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 Ne7

			variation => N__('French: Advance, 3... Ne7'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1p1pp/3p1n2/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 d6

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1p1pp/3p1n2/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 d6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 d6'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp1p1pp/3p1n2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1p1pp/3p1n2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'A86',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. c4 Nf6 3. g3 d6'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2p1pp/2pp1n2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1p1pp/5B2/3p1p2/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 0 4' => {
			eco => 'A80o',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5 d5'),
			moves => {
				'e7f6' => 'rnbqkb1r/ppp3pp/5p2/3p1p2/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1p1pp/5n2/3p1pB1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq d6 0 4' => {
			eco => 'A80o',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. Bg5 d5

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5 d5'),
			moves => {
				'g5f6' => 'rnbqkb1r/ppp1p1pp/5B2/3p1p2/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1p1pp/5n2/3p4/3PP3/2N5/PPP3PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3 d5 5. fxe4

			variation => N__('Dutch: Staunton Gambit, 4. f3 d5 5.fxe4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1p1pp/5n2/3p4/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq d6 0 5' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3 d5

			variation => N__('Dutch: Staunton Gambit, 4. f3 d5'),
			moves => {
				'f3e4' => 'rnbqkb1r/ppp1p1pp/5n2/3p4/3PP3/2N5/PPP3PP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1p2p/3p1np1/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2 d6

			variation => N__('Dutch: Leningrad, 4. Bg2 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/1n1p2p1/4P3/2BP4/5N2/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'B04l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 g6 5. Bc4 Nb6

			variation => N__('Alekhine: Modern, Fianchetto, 5. Bc4 Nb6'),
			moves => {
				'c4b3' => 'rnbqkb1r/ppp1pp1p/1n1p2p1/4P3/3P4/1B3N2/PPP2PPP/RNBQK2R b KQkq - 3 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n1p2p1/4P3/2PP1P2/8/PP4PP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'B03i',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 g6

			variation => N__('Alekhine: Four Pawns Attack, Fianchetto Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/1n1p2p1/4P3/3P4/1B3N2/PPP2PPP/RNBQK2R b KQkq - 3 6' => {
			eco => 'B04l',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, Fianchetto, 5. Bc4 Nb6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/1n1p2p1/4P3/3P4/1B3N2/PPP2PPP/RNBQK2R w KQkq - 4 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n1p4/4P1p1/2PP1P2/8/PP4PP/RNBQKBNR w KQkq g6 0 6' => {
			eco => 'B03g',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 g5

			variation => N__('Alekhine: Four Pawns Attack, Planinc Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6 6. Nc3

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6 6. Nc3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/1n4p1/8/2PP4/2N2N2/PP3PPP/R1BQKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/5N1P/PP3PP1/RNBQKB1R b KQkq - 0 6' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6 6. h3

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6 6. h3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/1n4p1/8/2PP4/5N1P/PP3PP1/RNBQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/5N2/PP2BPPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6 6. Be2

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6 6. Be2'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/1n4p1/8/2PP4/5N2/PP2BPPP/RNBQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/5N2/PP3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3 g6

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3 g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 6',
				'f1e2' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/5N2/PP2BPPP/RNBQK2R b KQkq - 1 6',
				'h2h3' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/5N1P/PP3PP1/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/3PP3/2N2P2/PP4PP/R1BQKBNR b KQkq - 2 6' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: Alekhine\'s, 5. e4 Nb6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/1n4p1/8/3PP3/2N2P2/PP4PP/R1BQKBNR w KQkq - 3 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/3PP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'D85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nb6

			variation => N__('Grünfeld: Exchange, 5. e4 Nb6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/3PP3/5P2/PP4PP/RNBQKBNR w KQkq - 1 6' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. f3 d5 4. cxd5 Nxd5 5. e4 Nb6

			variation => N__('Neo-Grünfeld: Alekhine\'s, 5. e4 Nb6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/3PP3/2N2P2/PP4PP/R1BQKBNR b KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/1n4p1/8/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. g3 g6 5. Bg2 Nb6

			variation => N__('English: Anglo-Grünfeld, Czech Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/6B1/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'B07w',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Bg5

			variation => N__('Pirc: Byrne 4. Bg5'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/6B1/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'B07s',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Bc4

			variation => N__('Pirc: Holmov (4. Bc4)'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'E61a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d6

			variation => N__('King\'s Indian: 3. Nc3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'E60f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 d6

			variation => N__('King\'s Indian: 3. Nf3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP2P/2N5/PPP2PP1/R1BQKBNR b KQkq h3 0 4' => {
			eco => 'B07q',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. h4

			variation => N__('Pirc: 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'B07t',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be3

			variation => N__('Pirc: 4. Be3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pp1p/2pp1np1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 0 5',
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B08a',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nf3

			variation => N__('Pirc: Classical'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N3P1/PPP2P1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B07r',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. g3

			variation => N__('Pirc: Sveshnikov (4. g3)'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N3P1/PPP2P1P/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N5/PPP1BPPP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'B07y',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Be2

			variation => N__('Pirc: 4. Be2'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N5/PPP1BPPP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B07q',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. Nge2

			variation => N__('Pirc: 4.Nge2'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2N5/PPP1NPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B07q',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6

			variation => N__('Pirc: 3... g6'),
			moves => {
				'c1e3' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 4',
				'c1g5' => 'rnbqkb1r/ppp1pp1p/3p1np1/6B1/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 1 4',
				'f1c4' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/2BPP3/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 4',
				'f1e2' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N5/PPP1BPPP/R1BQK1NR b KQkq - 1 4',
				'f2f4' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PPP2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 4',
				'g1e2' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4',
				'g2g3' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N3P1/PPP2P1P/R1BQKBNR b KQkq - 0 4',
				'h2h4' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP2P/2N5/PPP2PP1/R1BQKBNR b KQkq h3 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B07d',
                        # TRANSLATORS:

			variation => N__('Pirc: 3.Nbd2 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/2P5/PP1N1PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'B07i',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 g6 4. Nf3

			variation => N__('Pirc: 3. Bd3 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PP3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'B07i',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 g6

			variation => N__('Pirc: 3. Bd3 g6'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B07g',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. f3 g6

			variation => N__('Pirc: 3. f3 g6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nd2 g6

			variation => N__('Pirc: 3.Nbd2 g6'),
			moves => {
				'c2c3' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/3PPP2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 4' => {
			eco => 'B09a',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 g6 4. f4

			variation => N__('Pirc: Austrian Attack'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/3PPP2/2N5/PPP3PP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B07c',
                        # TRANSLATORS:

			variation => N__('Pirc: 2. f4 Nf6 3. Nc3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/3p1np1/8/4PP2/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p1np1/8/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B07c',
                        # TRANSLATORS:

			variation => N__('Pirc: 2. f4 Nf6 3. Nc3'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p2p1/3nP3/2BP4/5N2/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B04k',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 g6 5. Bc4

			variation => N__('Alekhine: Modern, Fianchetto, 5. Bc4'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pp1p/1n1p2p1/4P3/2BP4/5N2/PPP2PPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/3p2p1/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B04j',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 g6

			variation => N__('Alekhine: Modern, Fianchetto Variation'),
			moves => {
				'f1c4' => 'rnbqkb1r/ppp1pp1p/3p2p1/3nP3/2BP4/5N2/PPP2PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D85a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5

			variation => N__('Grünfeld: Exchange Variation'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/5P2/PP2P1PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: Alekhine\'s Anti-Grünfeld'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/5P2/PP2P1PP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. cxd5

			variation => N__('Neo-Grünfeld: 4.cxd5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 g6

			variation => N__('Scandinavian: Richter Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5

			variation => N__('Grünfeld: Stockholm Variation'),
			moves => {
				'f6e4' => 'rnbqkb1r/ppp1pp1p/6p1/3p2B1/2PPn3/2N5/PP2PPPP/R2QKBNR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4' => {
			eco => 'D01e',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 g6

			variation => N__('Richter-Veresov: 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'D03k',
                        # TRANSLATORS:

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/2P2N2/PP2PPPP/RN1QKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'D03m',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. e3

			variation => N__('Torre Attack: 3... g6 4. e3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'D03k',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'b1d2' => 'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/5N2/PPPNPPPP/R2QKB1R b KQkq - 1 4',
				'c2c3' => 'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/2P2N2/PP2PPPP/RN1QKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/5N2/PPPNPPPP/R2QKB1R b KQkq - 1 4' => {
			eco => 'D03l',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 g6 4. Nbd2

			variation => N__('Torre Attack: 3... g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p2B1/3P4/5N2/PPPNPPPP/R2QKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. Nf3 g6

			variation => N__('English: Anglo-Grünfeld'),
			moves => {
				'd1a4' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/Q1P5/2N2N2/PP1PPPPP/R1B1KB1R b KQkq - 1 4',
				'g2g3' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2P5/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. Nf3 g6 4. g3

			variation => N__('English: Anglo-Grünfeld'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'D82',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bf4

			variation => N__('Grünfeld: 4. Bf4'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP2P1/2N5/PP2PP1P/R1BQKBNR b KQkq g3 0 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. g4

			variation => N__('Grünfeld: Spike/Gibbon Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP3P/2N5/PP2PPP1/R1BQKBNR b KQkq h3 0 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. h4

			variation => N__('Grünfeld: 4. h4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/1QN5/PP2PPPP/R1B1KBNR b KQkq - 1 4' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3

			variation => N__('Grünfeld: Early Russian Variation'),
			moves => {
				'd5c4' => 'rnbqkb1r/ppp1pp1p/5np1/8/2pP4/1QN5/PP2PPPP/R1B1KBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. e3

			variation => N__('Grünfeld: 4. e3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D90',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Nf3

			variation => N__('Grünfeld: Three Knights Variation'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. f3

			variation => N__('Grünfeld: 4. f3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. g3

			variation => N__('Grünfeld: 4. g3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5

			variation => N__('Grünfeld Defence'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4',
				'c1g5' => 'rnbqkb1r/ppp1pp1p/5np1/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4',
				'c4d5' => 'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'd1b3' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/1QN5/PP2PPPP/R1B1KBNR b KQkq - 1 4',
				'e2e3' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'f2f3' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'g2g3' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4',
				'g2g4' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP2P1/2N5/PP2PP1P/R1BQKBNR b KQkq g3 0 4',
				'h2h4' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP3P/2N5/PP2PPP1/R1BQKBNR b KQkq h3 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3 d5

			variation => N__('Neo-Grünfeld: 3. Nf3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/5P2/PP2P1PP/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. f3 d5

			variation => N__('Neo-Grünfeld: Alekhine\'s Anti-Grünfeld'),
			moves => {
				'c4d5' => 'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/5P2/PP2P1PP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5

			variation => N__('Neo-Grünfeld: 3. g3 d5'),
			moves => {
				'c4d5' => 'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 4',
				'f1g2' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'D70',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3 d5 4. Bg2

			variation => N__('Neo-Grünfeld: 4. Bg2'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pp1p/2p2np1/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5',
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/3P1B2/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 1 4' => {
			eco => 'D00q',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Nf3 g6 4. Bf4

			variation => N__('Queen\'s Pawn: Anti-King\'s Indian'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/3P1B2/2N2N2/PPP1PPPP/R2QKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/2N2N2/PPP1PPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'D00p',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Nf3 g6

			variation => N__('Queen\'s Pawn: Veresov, 3. Nf3 g6'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/3P1B2/2N2N2/PPP1PPPP/R2QKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D04e',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 g6

			variation => N__('Colle: 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D02u',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3 g6

			variation => N__('Queen\'s Pawn: 3. g3 g6'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'D02u',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: 3. g3 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/3P4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. e3 g6

			variation => N__('Bird: Lasker, 3... g6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A03',
                        # TRANSLATORS:

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3 g6

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3 g6'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/5P2/5NP1/PPPPP1BP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 g6

			variation => N__('Reti: KIA, Neo-Grünfeld'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/3p4/Q1P5/2N2N2/PP1PPPPP/R1B1KB1R b KQkq - 1 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. Nf3 g6 4. Qa4+

			variation => N__('English: Anglo-Grünfeld'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/8/2BP4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D25d',
                        # TRANSLATORS:

			variation => N__('QGA: Smyslov Variation'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/8/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/8/2QP4/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'D81',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Qb3 dxc4 5. Qxc4

			variation => N__('Grünfeld: Early Russian Variation'),
			moves => {
				'c8e6' => 'rn1qkb1r/ppp1pp1p/4bnp1/8/2QP4/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 6',
				'f8g7' => 'rnbqk2r/ppp1ppbp/5np1/8/2QP4/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/8/2pP4/1QN5/PP2PPPP/R1B1KBNR w KQkq - 0 5' => {
			eco => 'D81',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Early Russian Variation'),
			moves => {
				'b3c4' => 'rnbqkb1r/ppp1pp1p/5np1/8/2QP4/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D25d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 g6

			variation => N__('QGA: Smyslov Variation'),
			moves => {
				'f1c4' => 'rnbqkb1r/ppp1pp1p/5np1/8/2BP4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5np1/8/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D00y',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Nxf3 g6

			variation => N__('Blackmar-Diemer: Bogoljubow Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/5p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'D00j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 gxf6 4. e3

			variation => N__('Trompowsky: 2... d5 3.Bxf6 gxf6 4. e3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pp1p/5p2/2pp4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/5p2/3p4/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'D00j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 gxf6

			variation => N__('Trompowsky: 2... d5 3.Bxf6 gxf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp1pp1p/5p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D85a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. g3

			variation => N__('Grünfeld: Exchange, 5. g3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/2N5/PP1BPPPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'D85b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. Bd2

			variation => N__('Grünfeld: Exchange, 5. Bd2'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/2N5/PP1BPPPP/R2QKBNR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D85a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5

			variation => N__('Grünfeld: Exchange Variation'),
			moves => {
				'c1d2' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/2N5/PP1BPPPP/R2QKBNR b KQkq - 1 5',
				'c3a4' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/N2P4/8/PP2PPPP/R1BQKBNR b KQkq - 1 5',
				'e2e4' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3PP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5',
				'g2g3' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/5N2/PPP1BPPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B01x',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 g6 5. Be2

			variation => N__('Scandinavian: Marshall, 4. Nf3 g6 5. Be2'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/6p1/3n4/3P4/5N2/PPP1BPPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B01x',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3 g6

			variation => N__('Scandinavian: Marshall, 4. Nf3 g6'),
			moves => {
				'f1e2' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/5N2/PPP1BPPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/5P2/PP2P1PP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: Alekhine\'s Anti-Grünfeld'),
			moves => {
				'e2e4' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3PP3/5P2/PP4PP/RNBQKBNR b KQkq e3 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3PP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5' => {
			eco => 'D85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4

			variation => N__('Grünfeld: Exchange, 5. e4'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/3PP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 6',
				'd5c3' => 'rnbqkb1r/ppp1pp1p/6p1/8/3PP3/2n5/PP3PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/3PP3/5P2/PP4PP/RNBQKBNR b KQkq e3 0 5' => {
			eco => 'D70',
                        # TRANSLATORS:

			variation => N__('Neo-Grünfeld: Alekhine\'s Anti-Grünfeld'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/3PP3/5P2/PP4PP/RNBQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/4P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6 5. e4

			variation => N__('English: Anglo-Grünfeld, Andersson-Böök Variation'),
			moves => {
				'd5c3' => 'rnbqkb1r/ppp1pp1p/6p1/8/4P3/2n2N2/PP1P1PPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6

			variation => N__('English: Anglo-Grünfeld, 4. Nf3 g6'),
			moves => {
				'd1a4' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/Q7/2N2N2/PP1PPPPP/R1B1KB1R b KQkq - 1 5',
				'e2e4' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/4P3/2N2N2/PP1P1PPP/R1BQKB1R b KQkq e3 0 5',
				'g2g3' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N2NP1/PP1PPP1P/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6 5. g3

			variation => N__('English: Anglo-Grünfeld, 4. Nf3 g6 5. g3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/6p1/3n4/8/2N2NP1/PP1PPP1P/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, 3.cxd5'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, 3.cxd5'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 6',
				'd5c3' => 'rnbqkb1r/ppp1pp1p/6p1/8/8/2n3P1/PP1PPPBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/N2P4/8/PP2PPPP/R1BQKBNR b KQkq - 1 5' => {
			eco => 'D85a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. Na4

			variation => N__('Grünfeld: Exchange, 5. Na4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3n4/Q7/2N2N2/PP1PPPPP/R1B1KB1R b KQkq - 1 5' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3 g6 5. Qa4+

			variation => N__('English: Anglo-Grünfeld, 4. Nf3 g6 5. Qa4+'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3nN3/2BP4/8/PPP2PPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'B04h',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 dxe5 5. Nxe5 g6 6. Bc4

			variation => N__('Alekhine: Modern, Larsen, 5.Nxe5 g6 6. Bc4'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pp1p/2p3p1/3nN3/2BP4/8/PPP2PPP/RNBQK2R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3nN3/3P4/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B04g',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 dxe5 5. Nxe5 g6

			variation => N__('Alekhine: Modern, Larsen, 5.Nxe5 g6'),
			moves => {
				'f1c4' => 'rnbqkb1r/ppp1pp1p/6p1/3nN3/2BP4/8/PPP2PPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3p2B1/2PPN3/8/PP2PPPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4 5. Nxe4

			variation => N__('Grünfeld: Stockholm, 4... Ne4 5.Nxe4'),
			moves => {
				'd5e4' => 'rnbqkb1r/ppp1pp1p/6p1/6B1/2PPp3/8/PP2PPPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3p2B1/2PPn3/2N5/PP2PPPP/R2QKBNR w KQkq - 2 5' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4

			variation => N__('Grünfeld: Stockholm, 4... Ne4'),
			moves => {
				'c3e4' => 'rnbqkb1r/ppp1pp1p/6p1/3p2B1/2PPN3/8/PP2PPPP/R2QKBNR b KQkq - 0 5',
				'g5f4' => 'rnbqkb1r/ppp1pp1p/6p1/3p4/2PPnB2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 5',
				'g5h4' => 'rnbqkb1r/ppp1pp1p/6p1/3p4/2PPn2B/2N5/PP2PPPP/R2QKBNR b KQkq - 3 5',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3p4/2PP3B/2P5/P3PPPP/R2QKBNR b KQkq - 0 6' => {
			eco => 'D80',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Stockholm, Taimanov, 5...Nxc3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/6p1/3p4/2PP3B/2P5/P3PPPP/R2QKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3p4/2PP3B/2n5/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4 5. Bh4 Nxc3

			variation => N__('Grünfeld: Stockholm, Taimanov, 5...Nxc3'),
			moves => {
				'b2c3' => 'rnbqkb1r/ppp1pp1p/6p1/3p4/2PP3B/2P5/P3PPPP/R2QKBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3p4/2PPn2B/2N5/PP2PPPP/R2QKBNR b KQkq - 3 5' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4 5. Bh4

			variation => N__('Grünfeld: Stockholm, Taimanov Variation'),
			moves => {
				'e4c3' => 'rnbqkb1r/ppp1pp1p/6p1/3p4/2PP3B/2n5/PP2PPPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/3p4/2PPnB2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 5' => {
			eco => 'D80',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. Bg5 Ne4 5. Bf4

			variation => N__('Grünfeld: Stockholm, 4... Ne4 5. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/6B1/2PPp3/8/PP1QPPPP/R3KBNR b KQkq - 1 6' => {
			eco => 'D80',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Stockholm, 4... Ne4 5.Nxe4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pp1p/6p1/2p3B1/2PPp3/8/PP1QPPPP/R3KBNR w KQkq c6 0 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/6B1/2PPp3/8/PP2PPPP/R2QKBNR w KQkq - 0 6' => {
			eco => 'D80',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Stockholm, 4... Ne4 5.Nxe4'),
			moves => {
				'd1d2' => 'rnbqkb1r/ppp1pp1p/6p1/6B1/2PPp3/8/PP1QPPPP/R3KBNR b KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/8/3PP3/2P5/P4PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'D85d',
                        # TRANSLATORS:

			variation => N__('Grünfeld: Exchange, 5. e4 Nxc3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pp1p/6p1/2p5/3PP3/2P5/P4PPP/R1BQKBNR w KQkq c6 0 7',
				'f8g7' => 'rnbqk2r/ppp1ppbp/6p1/8/3PP3/2P5/P4PPP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/8/3PP3/2n5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'D85d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 d5 4. cxd5 Nxd5 5. e4 Nxc3

			variation => N__('Grünfeld: Exchange, 5. e4 Nxc3'),
			moves => {
				'b2c3' => 'rnbqkb1r/ppp1pp1p/6p1/8/3PP3/2P5/P4PPP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/8/4P3/2P2N2/PP3PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, Andersson-Böök Variation'),
			moves => {
				'd8d1' => 'rnb1kb1r/ppp1pp1p/6p1/8/4P3/2P2N2/PP3PPP/R1BqKB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/8/4P3/2n2N2/PP1P1PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, Andersson-Böök Variation'),
			moves => {
				'd2c3' => 'rnbqkb1r/ppp1pp1p/6p1/8/4P3/2P2N2/PP3PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/8/8/2P3P1/P2PPPBP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. g3 g6 5. Bg2 Nxc3 6. bxc3

			variation => N__('English: Anglo-Grünfeld, Smyslov Variation'),
			moves => {
				'f8g7' => 'rnbqk2r/ppp1ppbp/6p1/8/8/2P3P1/P2PPPBP/R1BQK1NR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp1pp1p/6p1/8/8/2n3P1/PP1PPPBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. g3 g6 5. Bg2 Nxc3

			variation => N__('English: Anglo-Grünfeld, Smyslov Variation'),
			moves => {
				'b2c3' => 'rnbqkb1r/ppp1pp1p/6p1/8/8/2P3P1/P2PPPBP/R1BQK1NR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1ppp1/5n1p/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4' => {
			eco => 'D01d',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 h6

			variation => N__('Richter-Veresov: 3... h6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/1n1P4/8/2PP4/8/PP3PPP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'B03o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6

			variation => N__('Alekhine: Exchange Variation'),
			moves => {
				'c7d6' => 'rnbqkb1r/pp2pppp/1n1p4/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 6',
				'e7d6' => 'rnbqkb1r/ppp2ppp/1n1p4/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/1n1p4/4P3/2PP1P2/8/PP4PP/RNBQKBNR b KQkq f3 0 5' => {
			eco => 'B03g',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4

			variation => N__('Alekhine: Four Pawns Attack'),
			moves => {
				'c8f5' => 'rn1qkb1r/ppp1pppp/1n1p4/4Pb2/2PP1P2/8/PP4PP/RNBQKBNR w KQkq - 1 6',
				'd6e5' => 'rnbqkb1r/ppp1pppp/1n6/4p3/2PP1P2/8/PP4PP/RNBQKBNR w KQkq - 0 6',
				'g7g5' => 'rnbqkb1r/ppp1pp1p/1n1p4/4P1p1/2PP1P2/8/PP4PP/RNBQKBNR w KQkq g6 0 6',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/1n1p2p1/4P3/2PP1P2/8/PP4PP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/1n1p4/4P3/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'B03f',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. Nf3

			variation => N__('Alekhine: 4. c4 Nb6 5. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/1n1p4/4P3/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B03e',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6

			variation => N__('Alekhine: 4. c4 Nb6'),
			moves => {
				'e5d6' => 'rnbqkb1r/ppp1pppp/1n1P4/8/2PP4/8/PP3PPP/RNBQKBNR b KQkq - 0 5',
				'f2f4' => 'rnbqkb1r/ppp1pppp/1n1p4/4P3/2PP1P2/8/PP4PP/RNBQKBNR b KQkq f3 0 5',
				'g1f3' => 'rnbqkb1r/ppp1pppp/1n1p4/4P3/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pppp/1n1p4/4P3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'B04e',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 Nb6

			variation => N__('Alekhine: Modern, Schmid Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/4P3/2BP4/8/PP3PPP/RNBQK1NR w KQkq - 1 6' => {
			eco => 'D20p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5 5. Bxc4 Nb6

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5 5.Bxc4 Nb6'),
			moves => {
				'c4b3' => 'rnbqkb1r/ppp1pppp/1n6/4P3/3P4/1B6/PP3PPP/RNBQK1NR b KQkq - 2 6',
				'c4d3' => 'rnbqkb1r/ppp1pppp/1n6/4P3/3P4/3B4/PP3PPP/RNBQK1NR b KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/4P3/2PP4/8/PP4PP/RNBQKBNR b KQkq - 0 6' => {
			eco => 'B03j',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5 6. fxe5

			variation => N__('Alekhine: Four Pawns Attack, 5...dxe5'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp1pppp/1nn5/4P3/2PP4/8/PP4PP/RNBQKBNR w KQkq - 1 7',
				'c7c5' => 'rnbqkb1r/pp2pppp/1n6/2p1P3/2PP4/8/PP4PP/RNBQKBNR w KQkq c6 0 7',
				'c8f5' => 'rn1qkb1r/ppp1pppp/1n6/4Pb2/2PP4/8/PP4PP/RNBQKBNR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/4P3/3P4/1B6/PP3PPP/RNBQK1NR b KQkq - 2 6' => {
			eco => 'D20q',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5 5. Bxc4 Nb6 6. Bb3

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5 5.Bxc4 Nb6 6. Bb3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp1pppp/1nn5/4P3/3P4/1B6/PP3PPP/RNBQK1NR w KQkq - 3 7',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/4P3/3P4/3B4/PP3PPP/RNBQK1NR b KQkq - 2 6' => {
			eco => 'D20p',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5 5. Bxc4 Nb6 6. Bd3

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5 5.Bxc4 Nb6 6. Bd3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp1pppp/1nn5/4P3/3P4/3B4/PP3PPP/RNBQK1NR w KQkq - 3 7',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/4p3/2PP1P2/8/PP4PP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'B03j',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. f4 dxe5

			variation => N__('Alekhine: Four Pawns Attack, 5...dxe5'),
			moves => {
				'f4e5' => 'rnbqkb1r/ppp1pppp/1n6/4P3/2PP4/8/PP4PP/RNBQKBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/8/2B5/2N5/PPPP1PPP/R1BQK1NR w KQkq - 2 5' => {
			eco => 'B02k',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5 4. Bc4 Nb6

			variation => N__('Alekhine: Scandinavian, Exchange, 4. Bc4 Nb6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/8/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B01y',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nc3

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nc3'),
			moves => {
				'e7e5' => 'rnbqkb1r/ppp2ppp/1n6/4p3/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq e6 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/8/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'B01z',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nf3

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nf3'),
			moves => {
				'c8g4' => 'rn1qkb1r/ppp1pppp/1n6/8/2PP2b1/5N2/PP3PPP/RNBQKB1R w KQkq - 3 6',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/1n4p1/8/2PP4/5N2/PP3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/1n6/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B01y',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/1n6/8/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5',
				'g1f3' => 'rnbqkb1r/ppp1pppp/1n6/8/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp1pppp/3P4/3n4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B03c',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. exd6

			variation => N__('Alekhine: 3. d4 d6 4.exd6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1B2/8/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A45j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d6 3. Bxf6

			variation => N__('Trompowsky 2... d6 3.Bxf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/6B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A45j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d6 3. Nc3

			variation => N__('Trompowsky 2... d6 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A46i',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 d6 3. Bg5

			variation => N__('Indian: 2. Nf3 d6 3. Bg5'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppnpppp/3p1n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/6B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A45j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d6

			variation => N__('Trompowsky 2... d6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/3p1n2/6B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3',
				'g5f6' => 'rnbqkb1r/ppp1pppp/3p1B2/8/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d6

			variation => N__('English: Anglo-Indian, 2. Nc3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3

			variation => N__('Old Indian: 3. Nc3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppnpppp/3p1n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
				'c7c6' => 'rnbqkb1r/pp2pppp/2pp1n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'c8f5' => 'rn1qkb1r/ppp1pppp/3p1n2/5b2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
				'e7e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nf3

			variation => N__('Old Indian: 3. Nf3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppnpppp/3p1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'c7c6' => 'rnbqkb1r/pp2pppp/2pp1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'c8f5' => 'rn1qkb1r/ppp1pppp/3p1n2/5b2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'c8g4' => 'rn1qkb1r/ppp1pppp/3p1n2/8/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. g3

			variation => N__('Old Indian: 3. g3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6

			variation => N__('Old Indian'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'g1f3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A46g',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 d6

			variation => N__('Indian: 2. Nf3 d6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppp1pppp/3p1n2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46h',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 d6 3. g3

			variation => N__('Indian: 2. Nf3 d6 3. g3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'B07k',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3

			variation => N__('Pirc: 3. Nc3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppnpppp/3p1n2/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 3 4',
				'c7c6' => 'rnbqkb1r/pp2pppp/2pp1n2/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'e7e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/3B4/PPP2PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'B07h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3

			variation => N__('Pirc: 3. Bd3'),
			moves => {
				'e7e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq e6 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nf3

			variation => N__('Pirc: Roscher Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B07e',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. f3

			variation => N__('Pirc: 3. f3'),
			moves => {
				'e7e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/5P2/PPP3PP/RNBQKBNR w KQkq e6 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6

			variation => N__('Pirc: 2. d4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 2 3',
				'b1d2' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/8/PPPN1PPP/R1BQKBNR b KQkq - 2 3',
				'f1d3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/3B4/PPP2PPP/RNBQK1NR b KQkq - 2 3',
				'f2f3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/8/PPPN1PPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nd2

			variation => N__('Pirc: 3.Nbd2'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'B07b',
                        # TRANSLATORS: 1. e4 d6 2. Nf3 Nf6 3. Nc3

			variation => N__('Pirc: 2. Nf3 Nf6 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'B07a',
                        # TRANSLATORS: 1. e4 d6 2. Nc3 Nf6

			variation => N__('Pirc: 2. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'B07b',
                        # TRANSLATORS: 1. e4 d6 2. Nf3 Nf6

			variation => N__('Pirc: 2. Nf3 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'B07c',
                        # TRANSLATORS: 1. e4 d6 2. f4 Nf6 3. Nc3

			variation => N__('Pirc: 2. f4 Nf6 3. Nc3'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B07c',
                        # TRANSLATORS: 1. e4 d6 2. f4 Nf6

			variation => N__('Pirc: 2. f4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/3p1n2/8/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/3p1n2/8/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 d6

			variation => N__('Bird: 1..Nf6 2. Nf3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p4/3nP3/2BP4/8/PPP2PPP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'B03d',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Bc4

			variation => N__('Alekhine: Balogh Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/3p4/3nP3/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4' => {
			eco => 'B03e',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4

			variation => N__('Alekhine: 4. c4'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pppp/1n1p4/4P3/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pppp/3p4/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'B04a',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3

			variation => N__('Alekhine: Modern Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp1pppp/2np4/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
				'c7c6' => 'rnbqkb1r/pp2pppp/2pp4/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'c8g4' => 'rn1qkb1r/ppp1pppp/3p4/3nP3/3P2b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
				'd5b6' => 'rnbqkb1r/ppp1pppp/1n1p4/4P3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
				'd6e5' => 'rnbqkb1r/ppp1pppp/8/3np3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/3p2p1/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/3p4/3nP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B03b',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6

			variation => N__('Alekhine: 3. d4 d6'),
			moves => {
				'c2c4' => 'rnbqkb1r/ppp1pppp/3p4/3nP3/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4',
				'e5d6' => 'rnbqkb1r/ppp1pppp/3P4/3n4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 4',
				'f1c4' => 'rnbqkb1r/ppp1pppp/3p4/3nP3/2BP4/8/PPP2PPP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/ppp1pppp/3p4/3nP3/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5B2/3p4/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'D00j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6

			variation => N__('Trompowsky: 2... d5 3.Bxf6'),
			moves => {
				'e7f6' => 'rnbqkb1r/ppp2ppp/5p2/3p4/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4',
				'g7f6' => 'rnbqkb1r/ppp1pp1p/5p2/3p4/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/1B1P4/8/8/PPPP1PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'B01s',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bb5+

			variation => N__('Scandinavian: 2... Nf6 3. Bb5+'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppnpppp/5n2/1B1P4/8/8/PPPP1PPP/RNBQK1NR w KQkq - 3 4',
				'c8d7' => 'rn1qkb1r/pppbpppp/5n2/1B1P4/8/8/PPPP1PPP/RNBQK1NR w KQkq - 3 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/2B5/8/PPPP1PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'B01r',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Bc4

			variation => N__('Scandinavian: 2... Nf6 3. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/2P5/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4

			variation => N__('Scandinavian: 2... Nf6 3. c4'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/3P4/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3P4/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D06h',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6 3. cxd5

			variation => N__('QGD: Marshall Defence, 3.cxd5'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3n4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'B01u',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4

			variation => N__('Scandinavian: 2... Nf6 3. d4'),
			moves => {
				'c8g4' => 'rn1qkb1r/ppp1pppp/5n2/3P4/3P2b1/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3n4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3P4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/8/2N5/PP1PPPPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5

			variation => N__('English: Anglo-Grünfeld, 3.cxd5'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3n4/8/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'B02g',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5

			variation => N__('Alekhine: Scandinavian, Exchange'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/3P4/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 4',
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3n4/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'B01r',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Nf3

			variation => N__('Scandinavian: 2... Nf6 3. Nf3'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3n4/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3P4/8/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B01r',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6

			variation => N__('Scandinavian: 2... Nf6'),
			moves => {
				'c2c4' => 'rnbqkb1r/ppp1pppp/5n2/3P4/2P5/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 3',
				'd2d4' => 'rnbqkb1r/ppp1pppp/5n2/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq d3 0 3',
				'f1b5' => 'rnbqkb1r/ppp1pppp/5n2/1B1P4/8/8/PPPP1PPP/RNBQK1NR b KQkq - 2 3',
				'f1c4' => 'rnbqkb1r/ppp1pppp/5n2/3P4/2B5/8/PPPP1PPP/RNBQK1NR b KQkq - 2 3',
				'g1f3' => 'rnbqkb1r/ppp1pppp/5n2/3P4/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 3 3' => {
			eco => 'D01a',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5

			variation => N__('Richter-Veresov Attack'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppnpppp/5n2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 4 4',
				'c7c5' => 'rnbqkb1r/pp2pppp/5n2/2pp2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq c6 0 4',
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4',
				'c8f5' => 'rn1qkb1r/ppp1pppp/5n2/3p1bB1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 4 4',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4',
				'f6e4' => 'rnbqkb1r/ppp1pppp/8/3p2B1/3Pn3/2N5/PPP1PPPP/R2QKBNR w KQkq - 4 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4',
				'h7h6' => 'rnbqkb1r/ppp1ppp1/5n1p/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/2P5/PP2PPPP/RN1QKBNR b KQkq - 2 3' => {
			eco => 'D00b',
                        # TRANSLATORS: 1. d4 d5 2. c3 Nf6 3. Bg5

			variation => N__('Queen\'s Pawn: 2. c3 Nf6 3. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'D00i',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. e3

			variation => N__('Trompowsky: 2... d5 3. e3'),
			moves => {
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3' => {
			eco => 'D03a',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5

			variation => N__('Torre Attack (Tartakower)'),
			moves => {
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
				'f6e4' => 'rnbqkb1r/ppp1pppp/8/3p2B1/3Pn3/5N2/PPP1PPPP/RN1QKB1R w KQkq - 4 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p2B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 3' => {
			eco => 'D00i',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5

			variation => N__('Trompowsky: 2... d5'),
			moves => {
				'b1d2' => 'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/8/PPPNPPPP/R2QKBNR b KQkq - 1 3',
				'e2e3' => 'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 3',
				'g5f6' => 'rnbqkb1r/ppp1pppp/5B2/3p4/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/8/PPPNPPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'D00i',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Nd2

			variation => N__('Trompowsky: 2... d5 3. Nd2'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/1P6/4P3/PBPP1PPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A00q',
                        # TRANSLATORS: 1. b4 d5 2. Bb2 Nf6 3. e3

			variation => N__('Polish: 1... d5 2. Bb2 Nf6 3. e3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/1P6/5N2/P1PPPPPP/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b4 Nf6

			variation => N__('Reti: Santasiere\'s folly'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 2 3' => {
			eco => 'A00q',
                        # TRANSLATORS: 1. b4 d5 2. Bb2 Nf6

			variation => N__('Polish: 1... d5 2. Bb2 Nf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/1P6/4P3/PBPP1PPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. Nf3

			variation => N__('English: Anglo-Grünfeld'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5

			variation => N__('English: Anglo-Grünfeld'),
			moves => {
				'c4d5' => 'rnbqkb1r/ppp1pppp/5n2/3P4/8/2N5/PP1PPPPP/R1BQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'D06f',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6 3. Nc3

			variation => N__('QGD: Marshall Defence, 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'D06g',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6 3. Nf3

			variation => N__('QGD: Marshall Defence, 3. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'D06e',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6

			variation => N__('QGD: Marshall Defence'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
				'c4d5' => 'rnbqkb1r/ppp1pppp/5n2/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/2N5/PPP1PPPP/R2QKBNR b KQkq - 3 3' => {
			eco => 'D00o',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bf4

			variation => N__('Queen\'s Pawn: Veresov, 3. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/2P5/PP2PPPP/RN1QKBNR b KQkq - 2 3' => {
			eco => 'D00b',
                        # TRANSLATORS: 1. d4 d5 2. c3 Nf6 3. Bf4

			variation => N__('Queen\'s Pawn: 2. c3 Nf6 3. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3' => {
			eco => 'D02l',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pppp/5n2/2pp4/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq c6 0 4',
				'c8f5' => 'rn1qkb1r/ppp1pppp/5n2/3p1b2/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 4 4',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/5P2/PPP1P1PP/RN1QKBNR w KQkq - 1 5' => {
			eco => 'A45v',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 d5 4. f3 Nf6

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 d5 4. f3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P2P1/5P2/PPP1P2P/RNBQKBNR b KQkq g3 0 3' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. f3 d5 3. g4

			variation => N__('Indian: Gedult Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'D00p',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Nf3

			variation => N__('Queen\'s Pawn: Veresov, 3. Nf3'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/2N2N2/PPP1PPPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'D00n',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6

			variation => N__('Queen\'s Pawn: Veresov Attack'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/2N5/PPP1PPPP/R2QKBNR b KQkq - 3 3',
				'c1g5' => 'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 3 3',
				'e2e4' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 3 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02k',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. c3

			variation => N__('Queen\'s Pawn: 3. c3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2P5/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'D00b',
                        # TRANSLATORS: 1. d4 d5 2. c3 Nf6

			variation => N__('Queen\'s Pawn: 2. c3 Nf6'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/2P5/PP2PPPP/RN1QKBNR b KQkq - 2 3',
				'c1g5' => 'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/2P5/PP2PPPP/RN1QKBNR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/3BP3/PPP2PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'D00e',
                        # TRANSLATORS: 1. d4 d5 2. e3 Nf6 3. Bd3

			variation => N__('Queen\'s Pawn: Stonewall Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/4P3/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'D00d',
                        # TRANSLATORS: 1. d4 d5 2. e3 Nf6

			variation => N__('Queen\'s Pawn: 2. e3 Nf6'),
			moves => {
				'f1d3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/3BP3/PPP2PPP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D04a',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3

			variation => N__('Queen\'s Pawn: Colle'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pppp/5n2/2pp4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq c6 0 4',
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'c8f5' => 'rn1qkb1r/ppp1pppp/5n2/3p1b2/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
				'c8g4' => 'rn1qkb1r/ppp1pppp/5n2/3p4/3P2b1/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'D02k',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6

			variation => N__('Queen\'s Pawn: 2. Nf3 Nf6'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3',
				'c1g5' => 'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3',
				'c2c3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02r',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. g3

			variation => N__('Queen\'s Pawn: 3. g3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/5P2/PPP1P1PP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'A45a',
                        # TRANSLATORS:

			variation => N__('Indian: Paleface Attack'),
			moves => {
				'e2e4' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3PP3/5P2/PPP3PP/RNBQKBNR b KQkq e3 0 3',
				'g2g4' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P2P1/5P2/PPP1P2P/RNBQKBNR b KQkq g3 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'D00s',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4

			variation => N__('Blackmar-Diemer Gambit (BDG)'),
			moves => {
				'd5e4' => 'rnbqkb1r/ppp1pppp/5n2/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'f6e4' => 'rnbqkb1r/ppp1pppp/8/3p4/3Pn3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/3PP3/5P2/PPP3PP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. f3 d5 3. e4

			variation => N__('Indian: Blackmar-Diemer Gambit (without Nc3)'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'B02c',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5

			variation => N__('Alekhine: Scandinavian Variation'),
			moves => {
				'e4d5' => 'rnbqkb1r/ppp1pppp/5n2/3P4/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 0 3',
				'e4e5' => 'rnbqkb1r/ppp1pppp/5n2/3pP3/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/1P3N2/P1PPP1PP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. b3

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. b3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/1P6/P1PPP1PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. b3 Nf6

			variation => N__('Bird: 1... d5 2. b3'),
			moves => {
				'c1b2' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/1P6/PBPPP1PP/RN1QKBNR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/1P6/PBPPP1PP/RN1QKBNR b KQkq - 2 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. b3 Nf6 3. Bb2

			variation => N__('Bird: 1... d5 2. b3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/4P3/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. e3 Nf6

			variation => N__('Bird: Lasker Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. e3

			variation => N__('Bird: Lasker Variation'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pppp/5n2/2pp4/5P2/4PN2/PPPP2PP/RNBQKB1R w KQkq c6 0 4',
				'c8g4' => 'rn1qkb1r/ppp1pppp/5n2/3p4/5Pb1/4PN2/PPPP2PP/RNBQKB1R w KQkq - 1 4',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6

			variation => N__('Bird: 1... d5 2. Nf3 Nf6'),
			moves => {
				'b2b3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/1P3N2/P1PPP1PP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. g3

			variation => N__('Bird: 1... d5 2. Nf3 Nf6 3. g3'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/6P1/PPPPP1BP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. g3 Nf6 3. Bg2

			variation => N__('Bird: 1... d5 2. g3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/6P1/PPPPP2P/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. g3 Nf6

			variation => N__('Bird: 1... d5 2. g3'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/6P1/PPPPP1BP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/1P3N2/P1PPPPPP/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Nf6

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Nf6'),
			moves => {
				'c1b2' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/1P3N2/PBPPPPPP/RN1QKB1R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/1P3N2/PBPPPPPP/RN1QKB1R b KQkq - 2 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Nf6 3. Bb2

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Nf6'),
			moves => {
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/8/1P3N2/PBPPPPPP/RN1QKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/2N2N2/PPPPPPPP/R1BQKB1R w KQkq - 2 3' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5 2. Nf3 Nf6

			variation => N__('Van Geet: 1... d5 2. Nf3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/3P1N2/PPP1PPPP/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. d3 Nf6

			variation => N__('Reti: Old Indian Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 2 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2

			variation => N__('Reti: KIA'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4',
				'c8f5' => 'rn1qkb1r/ppp1pppp/5n2/3p1b2/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 3 4',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3p4/8/6P1/PPPPPPBP/RNBQK1NR w KQkq - 2 3' => {
			eco => 'A00v',
                        # TRANSLATORS: 1. g3 d5 2. Bg2 Nf6

			variation => N__('Benko Opening'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/3pP3/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'B02d',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. e5

			variation => N__('Alekhine: Scandinavian, 3. e5'),
			moves => {
				'f6d7' => 'rnbqkb1r/pppnpppp/8/3pP3/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 1 4',
				'f6e4' => 'rnbqkb1r/ppp1pppp/8/3pP3/4n3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/4P3/2pP4/8/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D20o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5

			variation => N__('QGA: 3. e4 Nf6 4. e5'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3nP3/2pP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'D00t',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. Bg5

			variation => N__('Blackmar-Diemer: von Popiel Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2B5/4PN2/PP1P1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'A09t',
                        # TRANSLATORS:

			variation => N__('Reti: Accepted, 3. e3 Nf6'),
			moves => {
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/8/2B5/4PN2/PP1P1PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B01y',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nf6

			variation => N__('Scandinavian: Marshall, 4. c4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2p5/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A09t',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. e3 Nf6

			variation => N__('Reti: Accepted, 3. e3 Nf6'),
			moves => {
				'f1c4' => 'rnbqkb1r/ppp1pppp/5n2/8/2B5/4PN2/PP1P1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'D24a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3

			variation => N__('QGA: 4. Nc3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1pp1pppp/p4n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'c7c5' => 'rnbqkb1r/pp2pppp/5n2/2p5/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'f6d5' => 'rnbqkb1r/ppp1pppp/8/3n4/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D20i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 Nf6

			variation => N__('QGA: 3. Nc3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'D20h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 Nf6

			variation => N__('QGA: 3. e3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D25a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3

			variation => N__('QGA: 4. e3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1pp1pppp/p4n2/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'c7c5' => 'rnbqkb1r/pp2pppp/5n2/2p5/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq c6 0 5',
				'c8e6' => 'rn1qkb1r/ppp1pppp/4bn2/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'c8g4' => 'rn1qkb1r/ppp1pppp/5n2/8/2pP2b1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D23a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6

			variation => N__('QGA: 3. Nf3 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/5n2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
				'd1a4' => 'rnbqkb1r/ppp1pppp/5n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R b KQkq - 3 4',
				'e2e3' => 'rnbqkb1r/ppp1pppp/5n2/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'g2g3' => 'rnbqkb1r/ppp1pppp/5n2/8/2pP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D23b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. g3

			variation => N__('QGA: 3. Nf3 Nf6 4. g3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pPP3/2N5/PP3PPP/R1BQKBNR b KQkq - 2 4' => {
			eco => 'D20o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. Nc3

			variation => N__('QGA: 3. e4 Nf6 4. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/2pPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'D20o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6

			variation => N__('QGA: 3. e4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp1pppp/5n2/8/2pPP3/2N5/PP3PPP/R1BQKBNR b KQkq - 2 4',
				'e4e5' => 'rnbqkb1r/ppp1pppp/5n2/4P3/2pP4/8/PP3PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N1pP2/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 e3

			variation => N__('Blackmar-Diemer: Langeheinecke Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N2N2/PPP3PP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D00w',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Nxf3

			variation => N__('Blackmar-Diemer: 4. f3 exf3 5.Nxf3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/8/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6',
				'c8f5' => 'rn1qkb1r/ppp1pppp/5n2/5b2/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 1 6',
				'c8g4' => 'rn1qkb1r/ppp1pppp/5n2/8/3P2b1/2N2N2/PPP3PP/R1BQKB1R w KQkq - 1 6',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/8/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/5np1/8/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N2Q2/PPP3PP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'D00v',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Qxf3

			variation => N__('Blackmar-Diemer: Ryder Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N2p2/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D00v',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3

			variation => N__('Blackmar-Diemer: Accepted'),
			moves => {
				'd1f3' => 'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N2Q2/PPP3PP/R1B1KBNR b KQkq - 0 5',
				'g1f3' => 'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N2N2/PPP3PP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3

			variation => N__('Blackmar-Diemer: 4. f3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp1pppp/2n2n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 1 5',
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 0 5',
				'c8f5' => 'rn1qkb1r/ppp1pppp/5n2/5b2/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 1 5',
				'e4e3' => 'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N1pP2/PPP3PP/R1BQKBNR w KQkq - 0 5',
				'e4f3' => 'rnbqkb1r/ppp1pppp/5n2/8/3P4/2N2p2/PPP3PP/R1BQKBNR w KQkq - 0 5',
				'e7e5' => 'rnbqkb1r/ppp2ppp/5n2/4p3/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq e6 0 5',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4pn2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D00t',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4

			variation => N__('Blackmar-Diemer: 3...dxe4'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppp1pppp/5n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR b KQkq - 1 4',
				'f2f3' => 'rnbqkb1r/ppp1pppp/5n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/5n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R b KQkq - 3 4' => {
			eco => 'D23c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Qa4+

			variation => N__('QGA: Mannheim Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp1pppp/2n2n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 4 5',
				'b8d7' => 'r1bqkb1r/pppnpppp/5n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 4 5',
				'c7c6' => 'rnbqkb1r/pp2pppp/2p2n2/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3N4/8/8/PPPP1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B02h',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5 4. Nxd5

			variation => N__('Alekhine: Scandinavian, Exchange, 4.Nxd5'),
			moves => {
				'd8d5' => 'rnb1kb1r/ppp1pppp/8/3q4/8/8/PPPP1PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/2B5/2N5/PPPP1PPP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'B02i',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5 4. Bc4

			variation => N__('Alekhine: Scandinavian, Exchange, 4. Bc4'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pppp/1n6/8/2B5/2N5/PPPP1PPP/R1BQK1NR w KQkq - 2 5',
				'e7e6' => 'rnbqkb1r/ppp2ppp/4p3/3n4/2B5/2N5/PPPP1PPP/R1BQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4' => {
			eco => 'B01y',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4

			variation => N__('Scandinavian: Marshall, 4. c4'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pppp/1n6/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
				'd5f6' => 'rnbqkb1r/ppp1pppp/5n2/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5' => {
			eco => 'D24a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 Nd5

			variation => N__('QGA: 4. Nc3 Nd5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'D06j',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6 3. cxd5 Nxd5 4. Nf3

			variation => N__('QGD: Marshall Defence, 3.cxd5 Nxd5 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'B01w',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. Nf3

			variation => N__('Scandinavian: Marshall, 4. Nf3'),
			moves => {
				'c8f5' => 'rn1qkb1r/ppp1pppp/8/3n1b2/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
				'c8g4' => 'rn1qkb1r/ppp1pppp/8/3n4/3P2b1/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
				'g7g6' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D06i',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6 3. cxd5 Nxd5

			variation => N__('QGD: Marshall Defence, 3.cxd5 Nxd5'),
			moves => {
				'e2e4' => 'rnbqkb1r/ppp1pppp/8/3n4/3PP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqkb1r/ppp1pppp/8/3n4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B01w',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5

			variation => N__('Scandinavian: Marshall Variation'),
			moves => {
				'c2c4' => 'rnbqkb1r/ppp1pppp/8/3n4/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4',
				'g1f3' => 'rnbqkb1r/ppp1pppp/8/3n4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/3PP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 4' => {
			eco => 'D06k',
                        # TRANSLATORS: 1. d4 d5 2. c4 Nf6 3. cxd5 Nxd5 4. e4

			variation => N__('QGD: Marshall Defence, 3.cxd5 Nxd5 4. e4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/8/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 d5 3. cxd5 Nxd5 4. Nf3

			variation => N__('English: Anglo-Grünfeld, 4. Nf3'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B02g',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5 4. Nf3

			variation => N__('Alekhine: Scandinavian, Exchange, 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/8/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, 3.cxd5'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/6p1/3n4/8/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/8/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A16',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Grünfeld, 3.cxd5'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppp1pppp/8/3n4/8/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 4',
				'g2g3' => 'rnbqkb1r/ppp1pppp/8/3n4/8/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B02g',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5

			variation => N__('Alekhine: Scandinavian, Exchange'),
			moves => {
				'c3d5' => 'rnbqkb1r/ppp1pppp/8/3N4/8/8/PPPP1PPP/R1BQKBNR b KQkq - 0 4',
				'f1c4' => 'rnbqkb1r/ppp1pppp/8/3n4/2B5/2N5/PPPP1PPP/R1BQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/ppp1pppp/8/3n4/8/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3n4/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B01r',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. Nf3 Nxd5

			variation => N__('Scandinavian: 2... Nf6 3. Nf3 Nxd5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3nN3/3P4/8/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B04f',
                        # TRANSLATORS:

			variation => N__('Alekhine: Modern, Larsen Variation'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppp1pp1p/6p1/3nN3/3P4/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3nP3/2BP4/8/PP3PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'D20o',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp1pppp/1n6/4P3/2BP4/8/PP3PPP/RNBQK1NR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3nP3/2pP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'D20o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 Nf6 4. e5 Nd5

			variation => N__('QGA: 3. e4 Nf6 4. e5 Nd5'),
			moves => {
				'f1c4' => 'rnbqkb1r/ppp1pppp/8/3nP3/2BP4/8/PP3PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3np3/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B04f',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. Nf3 dxe5

			variation => N__('Alekhine: Modern, Larsen Variation'),
			moves => {
				'f3e5' => 'rnbqkb1r/ppp1pppp/8/3nN3/3P4/8/PPP2PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p2B1/3Pn2P/8/PPP1PPP1/RN1QKBNR w KQkq d6 0 4' => {
			eco => 'A45s',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. h4 d5

			variation => N__('Trompowsky: 2... Ne4 3. h4 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p2B1/3Pn3/2N5/PPP1PPPP/R2QKBNR w KQkq - 4 4' => {
			eco => 'D01b',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 Ne4

			variation => N__('Richter-Veresov: 3... Ne4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p2B1/3Pn3/5N2/PPP1PPPP/RN1QKB1R w KQkq - 4 4' => {
			eco => 'D03b',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 Ne4

			variation => N__('Torre Attack: 3... Ne4'),
			moves => {
				'g5f4' => 'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 5 4',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3Pn2B/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 4' => {
			eco => 'A45t',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bh4 d5

			variation => N__('Trompowsky: 2... Ne4 3. Bh4 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3Pn3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D00s',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 Nxe4

			variation => N__('Blackmar-Diemer: Hubsch Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45w',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 d5 4. e3

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 d5 4. e3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp2pppp/8/2pp4/3PnB2/4P3/PPP2PPP/RN1QKBNR w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 5 4' => {
			eco => 'D03c',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 Ne4 4. Bf4

			variation => N__('Torre Attack: 3... Ne4 4. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/5P2/PPP1P1PP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45v',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 d5 4. f3

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 d5 4. f3'),
			moves => {
				'e4f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P1B2/5P2/PPP1P1PP/RN1QKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 4' => {
			eco => 'A45u',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 d5

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 d5'),
			moves => {
				'b1d2' => 'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/8/PPPNPPPP/R2QKBNR b KQkq - 1 4',
				'e2e3' => 'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4',
				'f2f3' => 'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/5P2/PPP1P1PP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/8/PPPNPPPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'A45u',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 d5 4. Nd2

			variation => N__('Trompowsky: 2... Ne4 3. Bf4 d5 4. Nd2'),
			moves => {
			},
		},
		'rnbqkb1r/ppp1pppp/8/3pP3/4n3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 1 4' => {
			eco => 'B02e',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. e5 Ne4

			variation => N__('Alekhine: Scandinavian, 3. e5 Ne4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2Npp/3p1n2/8/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C42h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nxf7

			variation => N__('Russian Game: Cochrane Gambit'),
			moves => {
				'e8f7' => 'rnbq1b1r/ppp2kpp/3p1n2/8/4P3/8/PPPP1PPP/RNBQKB1R w KQ - 0 5',
			},
		},
		'rnbqkb1r/ppp2p1p/5n2/3PN3/2B2ppP/8/PPPP2P1/RNBQK2R b KQkq - 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Berlin Defence, 6. Bc4'),
			moves => {
				'f8d6' => 'rnbqk2r/ppp2p1p/3b1n2/3PN3/2B2ppP/8/PPPP2P1/RNBQK2R w KQkq - 1 8',
			},
		},
		'rnbqkb1r/ppp2p1p/5n2/3PN3/3P1ppP/8/PPP3P1/RNBQKB1R b KQkq - 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'd8d5' => 'rnb1kb1r/ppp2p1p/5n2/3qN3/3P1ppP/8/PPP3P1/RNBQKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/ppp2p1p/5n2/3p4/4PpNP/8/PPPP2P1/RNBQKB1R w KQkq d6 0 7' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Nf6 6. Nxg4 d5

			variation => N__('KGA: Kieseritsky, Berlin Defence, Riviere Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2p1p/5n2/3pN3/2B1PppP/8/PPPP2P1/RNBQK2R w KQkq d6 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Berlin Defence, 6. Bc4'),
			moves => {
				'e4d5' => 'rnbqkb1r/ppp2p1p/5n2/3PN3/2B2ppP/8/PPPP2P1/RNBQK2R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp2p1p/5n2/3pN3/3PPBpP/8/PPP3P1/RN1QKB1R b KQkq - 0 7' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 d5 6. d4 Nf6 7. Bxf4

			variation => N__('KGA: Kieseritsky, Brentano Defence'),
			moves => {
				'f6e4' => 'rnbqkb1r/ppp2p1p/8/3pN3/3PnBpP/8/PPP3P1/RN1QKB1R w KQkq - 0 8',
			},
		},
		'rnbqkb1r/ppp2p1p/5n2/3pN3/3PPppP/8/PPP3P1/RNBQKB1R w KQkq - 1 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp2p1p/5n2/3pN3/3PPBpP/8/PPP3P1/RN1QKB1R b KQkq - 0 7',
				'e4d5' => 'rnbqkb1r/ppp2p1p/5n2/3PN3/3P1ppP/8/PPP3P1/RNBQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp2p1p/8/3pN3/3PnBpP/8/PPP3P1/RN1QKB1R w KQkq - 0 8' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano Defence'),
			moves => {
				'b1d2' => 'rnbqkb1r/ppp2p1p/8/3pN3/3PnBpP/8/PPPN2P1/R2QKB1R b KQkq - 1 8',
			},
		},
		'rnbqkb1r/ppp2p1p/8/3pN3/3PnBpP/8/PPPN2P1/R2QKB1R b KQkq - 1 8' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 d5 6. d4 Nf6 7. Bxf4 Nxe4 8. Nd2

			variation => N__('KGA: Kieseritsky, Brentano Defence, Caro Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2pp1/4pn1p/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'D30w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 h6

			variation => N__('QGD: Capablanca-Duras Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/1n1p4/8/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 1 6' => {
			eco => 'B03p',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6 exd6 6. Nc3

			variation => N__('Alekhine: Exchange, 5...exd6 6. Nc3'),
			moves => {
				'f8e7' => 'rnbqk2r/ppp1bppp/1n1p4/8/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 2 7',
			},
		},
		'rnbqkb1r/ppp2ppp/1n1p4/8/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'B03o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4 d6 4. c4 Nb6 5. exd6 exd6

			variation => N__('Alekhine: Exchange, 5...exd6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/1n1p4/8/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/1n6/4p3/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq e6 0 6' => {
			eco => 'B01y',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. d4 Nxd5 4. c4 Nb6 5. Nc3 e5

			variation => N__('Scandinavian: Marshall, 4. c4 Nb6 5. Nc3 e5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/1n6/4p3/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 6' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 d5 4. cxd5 Nxd5 5. Bg2 Nb6

			variation => N__('English: Bremen, Reverse Dragon, 4.cxd5 Nxd5 5. Bg2 Nb6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3P1n2/8/8/8/PPPPP1PP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6 Nf6

			variation => N__('Bird: From Gambit, Langheld Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3P4/8/3Qn3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 6' => {
			eco => 'C43d',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line'),
			moves => {
				'e4d6' => 'rnbqkb1r/ppp2ppp/3n4/8/3Q4/5N2/PPP2PPP/RNB1KB1R w KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp2ppp/3n4/8/3Q4/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 1 7' => {
			eco => 'C43e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4 5. Qxd4 d5 6. exd6 Nxd6 7. Nc3

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line, 7. Nc3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp2ppp/2nn4/8/3Q4/2N2N2/PPP2PPP/R1B1KB1R w KQkq - 2 8',
			},
		},
		'rnbqkb1r/ppp2ppp/3n4/8/3Q4/5N2/PPP2PPP/RNB1KB1R w KQkq - 0 7' => {
			eco => 'C43d',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/3n4/8/3Q4/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/3Pp3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. d5

			variation => N__('Old Indian: Ukrainian, 4. d5'),
			moves => {
				'f8e7' => 'rnbqk2r/ppp1bppp/3p1n2/3Pp3/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/3Pp3/4P3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B07f',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. f3 e5 4. d5

			variation => N__('Pirc: 3. f3 e5 4. d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4N3/4P3/8/PPPP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C42g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6

			variation => N__('Russian Game: 3.Nxe5 d6'),
			moves => {
				'e5c4' => 'rnbqkb1r/ppp2ppp/3p1n2/8/2N1P3/8/PPPP1PPP/RNBQKB1R b KQkq - 1 4',
				'e5f3' => 'rnbqkb1r/ppp2ppp/3p1n2/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 4',
				'e5f7' => 'rnbqkb1r/ppp2Npp/3p1n2/8/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4P3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. dxe5

			variation => N__('Old Indian: Ukrainian, 4.dxe5'),
			moves => {
				'd6e5' => 'rnbqkb1r/ppp2ppp/5n2/4p3/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4P3/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B07m',
                        # TRANSLATORS:

			variation => N__('Pirc: 3. Nc3 e5'),
			moves => {
				'd6e5' => 'rnbqkb1r/ppp2ppp/5n2/4p3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4P3/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C41p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. dxe5

			variation => N__('Philidor: Exchange Variation'),
			moves => {
				'f6e4' => 'rnbqkb1r/ppp2ppp/3p4/4P3/4n3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4P3/4P3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B07f',
                        # TRANSLATORS:

			variation => N__('Pirc: 3. f3 e5'),
			moves => {
				'd6e5' => 'rnbqkb1r/ppp2ppp/5n2/4p3/4P3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p1N1/3PP3/8/PPP2PPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'C41o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Ng5

			variation => N__('Philidor: Nimzowitsch, Locock Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq - 2 4' => {
			eco => 'C41o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Bc4

			variation => N__('Philidor: Nimzowitsch, Klein Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. Nf3 d6

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. Nf3 d6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. e3

			variation => N__('Old Indian: Ukranian, 4. e3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3

			variation => N__('Old Indian: 4. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp2ppp/2np1n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
				'b8d7' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
				'e5d4' => 'rnbqkb1r/ppp2ppp/3p1n2/8/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'e5e4' => 'rnbqkb1r/ppp2ppp/3p1n2/8/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5

			variation => N__('Old Indian: Ukrainian'),
			moves => {
				'd4d5' => 'rnbqkb1r/ppp2ppp/3p1n2/3Pp3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'd4e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4P3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'e2e3' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'e2e4' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'A53',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. e4

			variation => N__('Old Indian: Ukranian, 4. e4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 4' => {
			eco => 'C41q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. Nc3

			variation => N__('Philidor: Nimzowitsch Variation'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/3p1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 3 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'B07m',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Nc3 e5

			variation => N__('Pirc: 3. Nc3 e5'),
			moves => {
				'd4e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4P3/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/2PB4/PP3PPP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'B07h',
                        # TRANSLATORS:

			variation => N__('Pirc: 3. Bd3 e5'),
			moves => {
				'd6d5' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/3PP3/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/3B4/PPP2PPP/RNBQK1NR w KQkq e6 0 4' => {
			eco => 'B07h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 e5

			variation => N__('Pirc: 3. Bd3 e5'),
			moves => {
				'c2c3' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/2PB4/PP3PPP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'C41o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6

			variation => N__('Philidor: Nimzowitsch Variation'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 4',
				'd4e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4P3/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4',
				'f1c4' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq - 2 4',
				'f3g5' => 'rnbqkb1r/ppp2ppp/3p1n2/4p1N1/3PP3/8/PPP2PPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/5P2/PPP3PP/RNBQKBNR w KQkq e6 0 4' => {
			eco => 'B07f',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. f3 e5

			variation => N__('Pirc: 3. f3 e5'),
			moves => {
				'd4d5' => 'rnbqkb1r/ppp2ppp/3p1n2/3Pp3/4P3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 4',
				'd4e5' => 'rnbqkb1r/ppp2ppp/3p1n2/4P3/4P3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C29b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d6 4. Nf3

			variation => N__('Vienna Gambit: 3... d6 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C29b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d6

			variation => N__('Vienna Gambit: 3... d6'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/4PP2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/6N1/2PPp3/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 e4 5. Ng5

			variation => N__('Old Indian: 4. Nf3 e4 5. Ng5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/2N1P3/8/PPPP1PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'C42i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nc4

			variation => N__('Russian Game: Paulsen Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 e4

			variation => N__('Old Indian: 4. Nf3 e4'),
			moves => {
				'f3g5' => 'rnbqkb1r/ppp2ppp/3p1n2/6N1/2PPp3/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A54',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 d6 3. Nc3 e5 4. Nf3 exd4

			variation => N__('Old Indian: 4. Nf3 exd4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'C41k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 Nf6 5. Nc3

			variation => N__('Philidor: 3...exd4 4.Nxd4 Nf6 5. Nc3'),
			moves => {
				'f8e7' => 'rnbqk2r/ppp1bppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C41k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 Nf6

			variation => N__('Philidor: 3...exd4 4.Nxd4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/3p1n2/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 6' => {
			eco => 'C42p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 Nf6

			variation => N__('Russian Game: Classical, Close Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/3QP3/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 2 5' => {
			eco => 'C41i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Qxd4 Nf6 5. Nc3

			variation => N__('Philidor: Morphy, 4... Nf6 5. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5' => {
			eco => 'C41i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Qxd4 Nf6

			variation => N__('Philidor: Morphy, 4... Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/3p1n2/8/3QP3/2N2N2/PPP2PPP/R1B1KB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p1n2/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'C42j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3

			variation => N__('Russian Game: 3.Nxe5 d6 4. Nf3'),
			moves => {
				'f6e4' => 'rnbqkb1r/ppp2ppp/3p4/8/4n3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/3QP3/4n3/5N2/PPP2PPP/RNB1KB1R b KQkq - 1 5' => {
			eco => 'C41p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. dxe5 Nxe4 5. Qd5

			variation => N__('Philidor: Exchange, Rellstab Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/4P3/2P1P1n1/8/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'A52c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. e4 d6

			variation => N__('Budapest: Alekhine, Balogh Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/4P3/4n3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C41p',
                        # TRANSLATORS:

			variation => N__('Philidor: Exchange Variation'),
			moves => {
				'b1d2' => 'rnbqkb1r/ppp2ppp/3p4/4P3/4n3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 5',
				'd1d5' => 'rnbqkb1r/ppp2ppp/3p4/3QP3/4n3/5N2/PPP2PPP/RNB1KB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/4P3/4n3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C41p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 Nf6 4. dxe5 Nxe4 5. Nbd2

			variation => N__('Philidor: Exchange, Sokolsky Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/8/2P1n3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'C42l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. c4

			variation => N__('Russian Game: Vienna/Kaufmann'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/8/3Pn3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 5' => {
			eco => 'C42p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4

			variation => N__('Russian Game: Classical'),
			moves => {
				'd6d5' => 'rnbqkb1r/ppp2ppp/8/3p4/3Pn3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 6',
				'e4f6' => 'rnbqkb1r/ppp2ppp/3p1n2/8/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/8/4n3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C42m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. Nc3

			variation => N__('Russian Game: Nimzowitsch Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/8/4n3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C42k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d3

			variation => N__('Russian Game: French Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/8/4n3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C42j',
                        # TRANSLATORS:

			variation => N__('Russian Game: 3.Nxe5 d6 4. Nf3'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/3p4/8/4n3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 5',
				'c2c4' => 'rnbqkb1r/ppp2ppp/3p4/8/2P1n3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 5',
				'd1e2' => 'rnbqkb1r/ppp2ppp/3p4/8/4n3/5N2/PPPPQPPP/RNB1KB1R b KQkq - 1 5',
				'd2d3' => 'rnbqkb1r/ppp2ppp/3p4/8/4n3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 5',
				'd2d4' => 'rnbqkb1r/ppp2ppp/3p4/8/3Pn3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/3p4/8/4n3/5N2/PPPPQPPP/RNB1KB1R b KQkq - 1 5' => {
			eco => 'C42n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. Qe2

			variation => N__('Russian Game: Cozio/Lasker'),
			moves => {
				'd8e7' => 'rnb1kb1r/ppp1qppp/3p4/8/4n3/5N2/PPPPQPPP/RNB1KB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/3pp3/2PnP3/2B5/2N5/PP1P1PPP/R1BQK1NR w KQkq - 0 7' => {
			eco => 'B02u',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4 Nb6 4. c5 Nd5 5. Bc4 e6 6. Nc3 d6

			variation => N__('Alekhine: Two Pawns Attack, Mikenas Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/3ppn2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d6

			variation => N__('English: Mikenas, 3... d6'),
			moves => {
				'd2d4' => 'rnbqkb1r/ppp2ppp/3ppn2/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq d3 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/3ppn2/8/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq d3 0 4' => {
			eco => 'A18',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, 3... d6'),
			moves => {
				'f8e7' => 'rnbqk2r/ppp1bppp/3ppn2/8/2PPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4Pn2/8/2P5/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 e6 4. dxe6

			variation => N__('Scandinavian: Icelandic Gambit Accepted'),
			moves => {
				'c8e6' => 'rn1qkb1r/ppp2ppp/4bn2/8/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4p3/3n4/2B5/2N5/PPPP1PPP/R1BQK1NR w KQkq - 0 5' => {
			eco => 'B02j',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. exd5 Nxd5 4. Bc4 e6

			variation => N__('Alekhine: Scandinavian, Exchange, 4. Bc4 e6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4p3/3n4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D35b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 Nxd5

			variation => N__('QGD: Exchange, 4...Nxd5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4p3/3pP3/2P1n3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5 Ne4

			variation => N__('English: Mikenas, Flohr, 4... Ne4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4p3/3pP3/3PN3/8/PPP2PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C05c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Ne4 5. Nxe4

			variation => N__('French: Tarrasch, Closed, 4... Ne4 5.Nxe4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4p3/3pP3/3Pn3/8/PPPN1PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C05c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Ne4

			variation => N__('French: Tarrasch, Closed, 4... Ne4'),
			moves => {
				'd2e4' => 'rnbqkb1r/ppp2ppp/4p3/3pP3/3PN3/8/PPP2PPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pP2/8/2P5/2P5/P2P1PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'A18',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, Flohr, 4... d4'),
			moves => {
				'd8f6' => 'rnb1kb1r/ppp2ppp/4pq2/8/2P5/2P5/P2P1PPP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp2ppp/4pP2/8/2P5/2p5/PP1P1PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A18',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, Flohr, 4... d4'),
			moves => {
				'b2c3' => 'rnbqkb1r/ppp2ppp/4pP2/8/2P5/2P5/P2P1PPP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pP2/8/2Pp4/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'A18',
                        # TRANSLATORS:

			variation => N__('English: Mikenas, Flohr, 4... d4'),
			moves => {
				'd4c3' => 'rnbqkb1r/ppp2ppp/4pP2/8/2P5/2p5/PP1P1PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3P4/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B01t',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Nf6 3. c4 e6

			variation => N__('Scandinavian: Icelandic Gambit'),
			moves => {
				'd5e6' => 'rnbqkb1r/ppp2ppp/4Pn2/8/2P5/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D35b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5

			variation => N__('QGD: Exchange'),
			moves => {
				'e6d5' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
				'f6d5' => 'rnbqkb1r/ppp2ppp/4p3/3n4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3P4/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C11a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. exd5

			variation => N__('French: 3. Nc3 Nf6 4.exd5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3P4/4P3/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. cxd5

			variation => N__('English: Mikenas, French, 4.cxd5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5

			variation => N__('QGD: 4. Bg5'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq - 4 5',
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq c6 0 5',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5',
				'd5c4' => 'rnbqkb1r/ppp2ppp/4pn2/6B1/2pP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/3p2B1/1bPP4/2N5/PP2PPPP/R2QKBNR w KQkq - 4 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR w KQkq - 4 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R b KQkq - 3 4' => {
			eco => 'D30v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 4 5',
				'd5c4' => 'rnbqkb1r/ppp2ppp/4pn2/6B1/2pP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/3p2B1/1bPP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 4 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 4 5',
				'h7h6' => 'rnbqkb1r/ppp2pp1/4pn1p/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4' => {
			eco => 'D01c',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. Bg5 e6

			variation => N__('Richter-Veresov: 3... e6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'D00i',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. e3 e6

			variation => N__('Trompowsky: 2... d5 3. e3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'D03e',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6 4. e3

			variation => N__('Torre Attack: 3... e6 4. e3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq - 1 5',
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp2B1/3P4/4PN2/PPP2PPP/RN1QKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'D03d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bg5 e6

			variation => N__('Torre Attack: 3... e6'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3P4/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'C11o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5

			variation => N__('French: 3. Nc3 Nf6 4. Bg5'),
			moves => {
				'd5e4' => 'rnbqkb1r/ppp2ppp/4pn2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/3p2B1/1b1PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 4 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p2B1/3PP3/2N5/PPP2PPP/R2QKBNR w KQkq - 4 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5

			variation => N__('English: Mikenas, French Variation'),
			moves => {
				'c4d5' => 'rnbqkb1r/ppp2ppp/4pn2/3P4/4P3/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 4',
				'e4e5' => 'rnbqkb1r/ppp2ppp/4pn2/3pP3/2P5/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A13m',
                        # TRANSLATORS:

			variation => N__('English: Agincourt Variation'),
			moves => {
				'c1b2' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/1P3N2/PB1PPPPP/RN1QKB1R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/1P3N2/PB1PPPPP/RN1QKB1R b KQkq - 2 4' => {
			eco => 'A13m',
                        # TRANSLATORS:

			variation => N__('English: Agincourt Variation'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/1P3N2/PB1PPPPP/RN1QKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq d6 0 4' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3 d5

			variation => N__('English: Anglo-Indian, 2. Nc3 e6 3. Nf3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4' => {
			eco => 'A13r',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6

			variation => N__('English: Neo-Catalan'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 2 4' => {
			eco => 'A13s',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2

			variation => N__('English: Neo-Catalan'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq c6 0 5',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 0 5',
				'd5c4' => 'rnbqkb1r/ppp2ppp/4pn2/8/2p5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 0 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 3 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'D35a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bf4

			variation => N__('QGD: Harrwitz Attack'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D35a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. e3

			variation => N__('QGD: 3. Nc3 Nf6 4. e3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'D37a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Nf3

			variation => N__('QGD: 4. Nf3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq c6 0 5',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D35a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. g3

			variation => N__('QGD: Catalan without Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'D35a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6

			variation => N__('QGD: 3. Nc3 Nf6'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'c1g5' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 3 4',
				'c4d5' => 'rnbqkb1r/ppp2ppp/4pn2/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'e2e3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 3 4',
				'g2g3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D30r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. e3

			variation => N__('QGD: 3. Nf3 Nf6 4. e3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'D30q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6

			variation => N__('QGD: 3. Nf3 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/2PP4/5N2/PP2PPPP/RN1QKB1R b KQkq - 3 4',
				'e2e3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4' => {
			eco => 'E00o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3

			variation => N__('Catalan: 4. Nf3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq c6 0 5',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 0 5',
				'd5c4' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 2 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'E00n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5

			variation => N__('Catalan: 3... d5'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2

			variation => N__('Catalan: 4. Bg2'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq c6 0 5',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5',
				'd5c4' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/3p4/1bPP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/3p4/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/P7/1P2PPPP/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'E00b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. a3 d5

			variation => N__('Queen\'s Pawn: Anti-Nimzo-Indian, 3... d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'D02n',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P1B2/4PN2/PPP2PPP/RN1QKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'D02n',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. Bf4 e6

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3P1B2/4PN2/PPP2PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'D05d',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Bd3

			variation => N__('Colle: 3... e6 4. Bd3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/3BPN2/PPP2PPP/RNBQK2R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D05a',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6

			variation => N__('Colle: 3... e6'),
			moves => {
				'b1d2' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3P4/4PN2/PPPN1PPP/R1BQKB1R b KQkq - 1 4',
				'f1d3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3P4/3BPN2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3P4/4PN2/PPPN1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D05b',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 Nf6 3. e3 e6 4. Nbd2

			variation => N__('Colle: 3... e6 4.Nbd2'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3P4/4PN2/PPPN1PPP/R1BQKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'C11a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Be3

			variation => N__('French: Henneberger Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C11a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6

			variation => N__('French: 3. Nc3 Nf6'),
			moves => {
				'c1e3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 3 4',
				'c1g5' => 'rnbqkb1r/ppp2ppp/4pn2/3p2B1/3PP3/2N5/PPP2PPP/R2QKBNR b KQkq - 3 4',
				'e4d5' => 'rnbqkb1r/ppp2ppp/4pn2/3P4/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
				'e4e5' => 'rnbqkb1r/ppp2ppp/4pn2/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
				'f1d3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/2NB4/PPP2PPP/R1BQK1NR b KQkq - 3 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/2NB4/PPP2PPP/R1BQK1NR b KQkq - 3 4' => {
			eco => 'C11b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bd3

			variation => N__('French: Swiss Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C05a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6

			variation => N__('French: Tarrasch, Closed'),
			moves => {
				'e4e5' => 'rnbqkb1r/ppp2ppp/4pn2/3pP3/3P4/8/PPPN1PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/4P3/3P1N2/PPPN1PPP/R1BQKB1R b KQkq - 3 4' => {
			eco => 'C00s',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2 Nf6 4. Ngf3

			variation => N__('French: KIA, Reversed Philidor'),
			moves => {
				'b7b6' => 'rnbqkb1r/p1p2ppp/1p2pn2/3p4/4P3/3P1N2/PPPN1PPP/R1BQKB1R w KQkq - 0 5',
				'b8c6' => 'r1bqkb1r/ppp2ppp/2n1pn2/3p4/4P3/3P1N2/PPPN1PPP/R1BQKB1R w KQkq - 4 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/4P3/3P4/PPP1QPPP/RNB1KBNR w KQkq - 2 4' => {
			eco => 'C00o',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Qe2 Nf6

			variation => N__('French: KIA 2. d3 d5 3. Qe2 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'C00r',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2 Nf6

			variation => N__('French: KIA 2. d3 d5 3. Nd2 Nf6'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/4P3/3P1N2/PPPN1PPP/R1BQKB1R b KQkq - 3 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 Nf6 3. e3 e6

			variation => N__('Bird: Lasker, 3... e6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/8/1P2PN2/PBPP1PPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Nf6 3. Bb2 e6 4. e3

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/8/1P3N2/PBPPPPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 Nf6 3. Bb2 e6

			variation => N__('Reti: Nimzowitsch-Larsen, 2... Nf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/8/1P2PN2/PBPP1PPP/RN1QKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 Nf6 3. Bg2 e6

			variation => N__('Reti: KIA'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3pP3/2P5/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5

			variation => N__('English: Mikenas, Flohr Variation'),
			moves => {
				'd5d4' => 'rnbqkb1r/ppp2ppp/4pn2/4P3/2Pp4/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 5',
				'f6e4' => 'rnbqkb1r/ppp2ppp/4p3/3pP3/2P1n3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C11c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5

			variation => N__('French: Steinitz'),
			moves => {
				'f6d7' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/3pP3/3P4/8/PPPN1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C05b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5

			variation => N__('French: Tarrasch, Closed, 4. e5'),
			moves => {
				'f6d7' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 1 5',
				'f6e4' => 'rnbqkb1r/ppp2ppp/4p3/3pP3/3Pn3/8/PPPN1PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/4P3/2Pp4/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 d5 4. e5 d4

			variation => N__('English: Mikenas, Flohr, 4... d4'),
			moves => {
				'e5f6' => 'rnbqkb1r/ppp2ppp/4pP2/8/2Pp4/2N5/PP1P1PPP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/6B1/2pP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5' => {
			eco => 'D24b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 e6 5. Bg5

			variation => N__('QGA: 4. Nc3 e6 5. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/6B1/2pP4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'D50',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 dxc4

			variation => N__('QGD: 4. Bg5 dxc4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/6B1/2pP4/5N2/PP2PPPP/RN1QKB1R w KQkq - 0 5' => {
			eco => 'D30v',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 Nf6 4. Bg5 dxc4

			variation => N__('QGD: 3. Nf3 Nf6 4. Bg5 dxc4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'C11p',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4 5. Nxe4

			variation => N__('French: Burn, 5.Nxe4'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 1 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'C11o',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. Bg5 dxe4

			variation => N__('French: Burn Variation'),
			moves => {
				'c3e4' => 'rnbqkb1r/ppp2ppp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2B5/4PN2/PP1P1PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A09u',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. e3 Nf6 4. Bxc4 e6

			variation => N__('Reti: Accepted, 3. e3 Nf6 4.Bxc4 e6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2BP4/4P3/PP3PPP/RNBQK1NR w KQkq - 1 5' => {
			eco => 'D20d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e6 4. Bxc4 Nf6

			variation => N__('QGA: 3. e3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2BP4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D26b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4

			variation => N__('QGA: 4. e3 e6 5.Bxc4'),
			moves => {
				'a7a6' => 'rnbqkb1r/1pp2ppp/p3pn2/8/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq - 0 6',
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq c6 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2PP4/2N2p2/PP4PP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 Nf6 5. f3 exf3

			variation => N__('Diemer-Duhm Gambit (DDG): Exchange Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/2N1B3/PP3PPP/R2QKBNR b KQkq - 3 5' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 Nf6 5. Be3

			variation => N__('Diemer-Duhm Gambit (DDG): Alapin Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 Nf6 5. f3

			variation => N__('Diemer-Duhm Gambit (DDG): 5. f3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2PPp3/2N2P2/PP4PP/R1BQKBNR w KQkq c6 0 6',
				'e4f3' => 'rnbqkb1r/ppp2ppp/4pn2/8/2PP4/2N2p2/PP4PP/R1BQKBNR w KQkq - 0 6',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/8/1bPPp3/2N2P2/PP4PP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/2N5/PP3PPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'D30b',
                        # TRANSLATORS:

			variation => N__('Diemer-Duhm Gambit (DDG) Accepted'),
			moves => {
				'a2a3' => 'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/P1N5/1P3PPP/R1BQKBNR b KQkq - 0 5',
				'c1e3' => 'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/2N1B3/PP3PPP/R2QKBNR b KQkq - 3 5',
				'f2f3' => 'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/P1N5/1P3PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 Nf6 5. a3

			variation => N__('Diemer-Duhm Gambit (DDG): Duhm Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2p5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'A13u',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4

			variation => N__('English: Neo-Catalan Accepted'),
			moves => {
				'd1a4' => 'rnbqkb1r/ppp2ppp/4pn2/8/Q1p5/5NP1/PP1PPPBP/RNB1K2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D24c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 e6 5. e3

			variation => N__('QGA: 4. Nc3 e6 5. e3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D24b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 e6

			variation => N__('QGA: 4. Nc3 e6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppp2ppp/4pn2/6B1/2pP4/2N2N2/PP2PPPP/R2QKB1R b KQkq - 1 5',
				'e2e3' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/2N1PN2/PP3PPP/R1BQKB1R b KQkq - 0 5',
				'e2e4' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D26a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6

			variation => N__('QGA: 4. e3 e6'),
			moves => {
				'f1c4' => 'rnbqkb1r/ppp2ppp/4pn2/8/2BP4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 0 5' => {
			eco => 'E00t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 dxc4

			variation => N__('Catalan: 4. Nf3 dxc4'),
			moves => {
				'd1a4' => 'rnbqkb1r/ppp2ppp/4pn2/8/Q1pP4/5NP1/PP2PP1P/RNB1KB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'E04a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Nf3

			variation => N__('Catalan: Open, 5. Nf3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1pp2ppp/p3pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 0 6',
				'b7b5' => 'rnbqkb1r/p1p2ppp/4pn2/1p6/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq b6 0 6',
				'b8c6' => 'r1bqkb1r/ppp2ppp/2n1pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6',
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6',
				'c7c5' => 'rnbqkb1r/pp3ppp/4pn2/2p5/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq c6 0 6',
				'c8d7' => 'rn1qkb1r/pppb1ppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6',
				'f8b4' => 'rnbqk2r/ppp2ppp/4pn2/8/1bpP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'E01',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4

			variation => N__('Catalan: Open'),
			moves => {
				'd1a4' => 'rnbqkb1r/ppp2ppp/4pn2/8/Q1pP4/6P1/PP2PPBP/RNB1K1NR b KQkq - 1 5',
				'g1f3' => 'rnbqkb1r/ppp2ppp/4pn2/8/2pP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/2pPP3/2N2N2/PP3PPP/R1BQKB1R b KQkq e3 0 5' => {
			eco => 'D24b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. Nc3 e6 5. e4

			variation => N__('QGA: 4. Nc3 e6 5. e4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/3P4/2N2N2/PPP3PP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D00x',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 exf3 5. Nxf3 e6

			variation => N__('Blackmar-Diemer: Euwe Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C10g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 Nf6

			variation => N__('French: Rubinstein, 4... Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 e6

			variation => N__('Blackmar-Diemer: Weinspach Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/Q1p5/5NP1/PP1PPPBP/RNB1K2R b KQkq - 1 5' => {
			eco => 'A13v',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 Nf6 4. Bg2 dxc4 5. Qa4+

			variation => N__('English: Neo-Catalan Accepted, 5. Qa4+'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1p5/5NP1/PP1PPPBP/RNB1K2R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/Q1pP4/5NP1/PP2PP1P/RNB1KB1R b KQkq - 1 5' => {
			eco => 'E00u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Nf3 dxc4 5. Qa4+

			variation => N__('Catalan: 4. Nf3 dxc4 5. Qa4+'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/5NP1/PP2PP1P/RNB1KB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/4pn2/8/Q1pP4/6P1/PP2PPBP/RNB1K1NR b KQkq - 1 5' => {
			eco => 'E02',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 d5 4. Bg2 dxc4 5. Qa4+

			variation => N__('Catalan: Open, 5. Qa4+'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/4pn2/8/Q1pP4/6P1/PP2PPBP/RNB1K1NR w KQkq - 2 6',
				'c8d7' => 'rn1qkb1r/pppb1ppp/4pn2/8/Q1pP4/6P1/PP2PPBP/RNB1K1NR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/1B1P4/5p2/5N2/PPPP2PP/RNBQK2R b KQkq - 2 5' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Bb5+

			variation => N__('KGA: Scandinavian, Modern, 5. Bb5+ '),
			moves => {
				'c7c6' => 'rnbqkb1r/pp3ppp/2p2n2/1B1P4/5p2/5N2/PPPP2PP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3B4/4Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 1 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 d5 4. Bxd5 Nf6

			variation => N__('KGA: Bishop\'s Gambit, Morphy Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/2B2p2/5N2/PPPP2PP/RNBQK2R b KQkq - 2 5' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Bc4

			variation => N__('KGA: Scandinavian, Modern, 5. Bc4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/2Bp4/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'f8b4' => 'rnbqk2r/ppp2ppp/5n2/3P4/1bBp4/5N2/PPP2PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4PP2/8/PPP3PP/RNBQKBNR b KQkq - 0 5' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4

			variation => N__('KGD: Falkbeer, 5.dxe4'),
			moves => {
				'f6e4' => 'rnbqkb1r/ppp2ppp/8/3P4/4nP2/8/PPP3PP/RNBQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'C31',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 3.exd5 e4'),
			moves => {
				'd1e2' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/2N5/PPPPQ1PP/R1B1KBNR b KQkq - 3 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/2N5/PPPPQ1PP/R1B1KBNR b KQkq - 3 5' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. Nc3 Nf6 5. Qe2

			variation => N__('KGD: Falkbeer, Rubinstein Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/2NP4/PPP3PP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. Nc3

			variation => N__('KGD: Falkbeer, 4. d3 Nf6 5. Nc3'),
			moves => {
				'f8b4' => 'rnbqk2r/ppp2ppp/5n2/3P4/1b2pP2/2NP4/PPP3PP/R1BQKBNR w KQkq - 3 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/3P4/PPP1Q1PP/RNB1KBNR b KQkq - 2 5' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. Qe2

			variation => N__('KGD: Falkbeer, Reti Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/3P4/PPP3PP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6

			variation => N__('KGD: Falkbeer, 4. d3 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/2NP4/PPP3PP/R1BQKBNR b KQkq - 2 5',
				'b1d2' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/3P4/PPPN2PP/R1BQKBNR b KQkq - 2 5',
				'd1e2' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/3P4/PPP1Q1PP/RNB1KBNR b KQkq - 2 5',
				'd3e4' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4PP2/8/PPP3PP/RNBQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/3P4/PPPN2PP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. Nd2

			variation => N__('KGD: Falkbeer, Keres Variation'),
			moves => {
				'e4d3' => 'rnbqkb1r/ppp2ppp/5n2/3P4/5P2/3p4/PPPN2PP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/5P2/3p4/PPPN2PP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. Nd2 exd3

			variation => N__('KGD: Falkbeer, Keres, 5...exd3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/5p2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6 5. Nc3

			variation => N__('KGA: Scandinavian, Modern, 5. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/5p2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C29d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. exd5 exf4

			variation => N__('Vienna Gambit: 3... d5 4.exd5 exf4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3P4/5p2/5N2/PPPP2PP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5 Nf6

			variation => N__('KGA: Scandinavian, Modern Variation'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/5n2/3P4/5p2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 2 5',
				'f1b5' => 'rnbqkb1r/ppp2ppp/5n2/1B1P4/5p2/5N2/PPPP2PP/RNBQK2R b KQkq - 2 5',
				'f1c4' => 'rnbqkb1r/ppp2ppp/5n2/3P4/2B2p2/5N2/PPPP2PP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3Pp3/5P2/2N5/PPPP2PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C29d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. exd5

			variation => N__('Vienna Gambit: 3... d5 4.exd5'),
			moves => {
				'e5f4' => 'rnbqkb1r/ppp2ppp/5n2/3P4/5p2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3Pp3/8/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 d5 4. cxd5

			variation => N__('English: Bremen, Reverse Dragon, 4.cxd5'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3Pp3/8/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C26i',
                        # TRANSLATORS:

			variation => N__('Vienna: Smyslov, 3... d5'),
			moves => {
				'f6d5' => 'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'D35i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5

			variation => N__('QGD: Exchange, 5. Bg5'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/5n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 2 6',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p2n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 0 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/5n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p2B1/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 3 5' => {
			eco => 'C01t',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Nf6 5. Bg5

			variation => N__('French: Exchange, Svenonius Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp2ppp/2n2n2/3p2B1/3P4/2N5/PPP2PPP/R2QKBNR w KQkq - 4 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq d6 0 5' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'e4d5' => 'rnbqkb1r/ppp2ppp/5n2/3P4/2Bp4/5N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'C01e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nc3

			variation => N__('French: Exchange, 4. c4 Nf6 5. Nc3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp3ppp/2p2n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
				'f8b4' => 'rnbqk2r/ppp2ppp/5n2/3p4/1bPP4/2N5/PP3PPP/R1BQKBNR w KQkq - 3 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/5n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR w KQkq - 3 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 5' => {
			eco => 'C01d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6 5. Nf3

			variation => N__('French: Exchange, 4. c4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'C01d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 Nf6

			variation => N__('French: Exchange, 4. c4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppp2ppp/5n2/3p4/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq - 2 5',
				'g1f3' => 'rnbqkb1r/ppp2ppp/5n2/3p4/2PP4/5N2/PP3PPP/RNBQKB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'D35c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bf4

			variation => N__('QGD: Exchange, Sämisch Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D35d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Nf3

			variation => N__('QGD: Exchange, 5. Nf3'),
			moves => {
				'b8d7' => 'r1bqkb1r/pppn1ppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6',
				'c7c6' => 'rnbqkb1r/pp3ppp/2p2n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 3 5' => {
			eco => 'C01s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Nf6 5. Nf3

			variation => N__('French: Exchange, 4. Nc3 Nf6 5. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D35b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5

			variation => N__('QGD: Exchange'),
			moves => {
				'c1f4' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5',
				'c1g5' => 'rnbqkb1r/ppp2ppp/5n2/3p2B1/3P4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5',
				'g1f3' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5' => {
			eco => 'C01s',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3 Nf6

			variation => N__('French: Exchange, 4. Nc3 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppp2ppp/5n2/3p2B1/3P4/2N5/PPP2PPP/R2QKBNR b KQkq - 3 5',
				'g1f3' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 3 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 3 5' => {
			eco => 'C01m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nf3 Nf6 5. Bd3

			variation => N__('French: Exchange, 4. Nf3 Nf6 5. Bd3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5' => {
			eco => 'C01l',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nf3 Nf6

			variation => N__('French: Exchange, 4. Nf3 Nf6'),
			moves => {
				'f1d3' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 3 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pP3/4P3/2N5/PPPP2PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C29f',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5

			variation => N__('Vienna Gambit: 4.fxe5'),
			moves => {
				'f6e4' => 'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pP3/4P3/2PB4/PP3PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'B07h',
                        # TRANSLATORS:

			variation => N__('Pirc: 3. Bd3 e5 4. c3 d5'),
			moves => {
				'd5e4' => 'rnbqkb1r/ppp2ppp/5n2/4P3/4p3/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 6',
				'f6e4' => 'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/2B1P3/3P4/PPP2PPP/RNBQK1NR w KQkq d6 0 4' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d3 d5

			variation => N__('Bishop\'s Opening: 3. d3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 d5

			variation => N__('English: Bremen, Reverse Dragon'),
			moves => {
				'c4d5' => 'rnbqkb1r/ppp2ppp/5n2/3Pp3/8/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq d6 0 4' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nf6 3. Bg2 d5

			variation => N__('English: King\'s, 2. g3 Nf6 3. Bg2 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/3PP3/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'B07h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 e5 4. c3 d5

			variation => N__('Pirc: 3. Bd3 e5 4. c3 d5'),
			moves => {
				'd4e5' => 'rnbqkb1r/ppp2ppp/5n2/3pP3/4P3/2PB4/PP3PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'C26i',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 d5

			variation => N__('Vienna: Smyslov, 3... d5'),
			moves => {
				'e4d5' => 'rnbqkb1r/ppp2ppp/5n2/3Pp3/8/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'C29c',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5

			variation => N__('Vienna Gambit: 3... d5'),
			moves => {
				'd2d3' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/4PP2/2NP4/PPP3PP/R1BQKBNR b KQkq - 0 4',
				'e4d5' => 'rnbqkb1r/ppp2ppp/5n2/3Pp3/5P2/2N5/PPPP2PP/R1BQKBNR b KQkq - 0 4',
				'f4e5' => 'rnbqkb1r/ppp2ppp/5n2/3pP3/4P3/2N5/PPPP2PP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/3pp3/4PP2/2NP4/PPP3PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C29e',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. d3

			variation => N__('Vienna Gambit: Steinitz Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/4P3/4p3/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 6' => {
			eco => 'B07h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 e5 4. c3 d5 5. dxe5 dxe4

			variation => N__('Pirc: 3. Bd3 e5 4. c3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/4p3/2P5/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A53',
                        # TRANSLATORS:

			variation => N__('Old Indian: Ukrainian, 4.dxe5'),
			moves => {
				'd1d8' => 'rnbQkb1r/ppp2ppp/5n2/4p3/2P5/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/4p3/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq e6 0 5' => {
			eco => 'D00u',
                        # TRANSLATORS: 1. d4 d5 2. Nc3 Nf6 3. e4 dxe4 4. f3 e5

			variation => N__('Blackmar-Diemer: Elbert Countergambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/4p3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B07m',
                        # TRANSLATORS:

			variation => N__('Pirc: 3. Nc3 e5'),
			moves => {
				'd1d8' => 'rnbQkb1r/ppp2ppp/5n2/4p3/4P3/2N5/PPP2PPP/R1B1KBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/4p3/4P3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B07f',
                        # TRANSLATORS:

			variation => N__('Pirc: 3. f3 e5'),
			moves => {
				'd1d8' => 'rnbQkb1r/ppp2ppp/5n2/4p3/4P3/5P2/PPP3PP/RNB1KBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5n2/8/2BP4/8/PP3PPP/RNBQK1NR w KQkq - 1 6' => {
			eco => 'D20g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e5 4. Bxc4 exd4 5. exd4 Nf6

			variation => N__('QGA: 3. e3 e5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/5p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'D00k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 exf6 4. e3

			variation => N__('Trompowsky: 2... d5 3.Bxf6 exf6 4. e3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp3ppp/2p2p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 0 5',
				'c8e6' => 'rn1qkb1r/ppp2ppp/4bp2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5',
				'c8f5' => 'rn1qkb1r/ppp2ppp/5p2/3p1b2/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5',
				'f8d6' => 'rnbqk2r/ppp2ppp/3b1p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/5p2/3p4/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'D00k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 d5 3. Bxf6 exf6

			variation => N__('Trompowsky: 2... d5 3.Bxf6 exf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp2ppp/5p2/3p4/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3P4/4nP2/4B3/PPP3PP/RN1QKBNR b KQkq - 1 6' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Be3

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Be3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/8/3P4/4nP2/5N2/PPP3PP/RNBQKB1R b KQkq - 1 6' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Nf3

			variation => N__('KGD: Falkbeer, 5.dxe4 Nxe4 6. Nf3'),
			moves => {
				'f8c5' => 'rnbqk2r/ppp2ppp/8/2bP4/4nP2/5N2/PPP3PP/RNBQKB1R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3P4/4nP2/8/PPP1Q1PP/RNB1KBNR b KQkq - 1 6' => {
			eco => 'C32',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3 Nf6 5. dxe4 Nxe4 6. Qe2

			variation => N__('KGD: Falkbeer, Charousek Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/8/3P4/4nP2/8/PPP3PP/RNBQKBNR w KQkq - 0 6' => {
			eco => 'C32',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 5.dxe4'),
			moves => {
				'c1e3' => 'rnbqkb1r/ppp2ppp/8/3P4/4nP2/4B3/PPP3PP/RN1QKBNR b KQkq - 1 6',
				'd1e2' => 'rnbqkb1r/ppp2ppp/8/3P4/4nP2/8/PPP1Q1PP/RNB1KBNR b KQkq - 1 6',
				'g1f3' => 'rnbqkb1r/ppp2ppp/8/3P4/4nP2/5N2/PPP3PP/RNBQKB1R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 d5 4. cxd5 Nxd5

			variation => N__('English: Bremen, Reverse Dragon, 4.cxd5 Nxd5'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 d5 4. cxd5 Nxd5 5. Bg2

			variation => N__('English: Bremen, Reverse Dragon, 4.cxd5 Nxd5 5. Bg2'),
			moves => {
				'd5b6' => 'rnbqkb1r/ppp2ppp/1n6/4p3/8/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C26i',
                        # TRANSLATORS:

			variation => N__('Vienna: Smyslov, 3... d5'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3np3/8/2N3P1/PPPP1PBP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'C26i',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 d5 4. exd5 Nxd5 5. Bg2

			variation => N__('Vienna: Smyslov, 3... d5'),
			moves => {
				'd5c3' => 'rnbqkb1r/ppp2ppp/8/4p3/8/2n3P1/PPPP1PBP/R1BQK1NR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 6' => {
			eco => 'C42q',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5 6. Bd3

			variation => N__('Russian Game: Classical, 6. Bd3'),
			moves => {
				'f8d6' => 'rnbqk2r/ppp2ppp/3b4/3p4/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 7',
				'f8e7' => 'rnbqk2r/ppp1bppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 7',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3p4/3Pn3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C42p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 d6 4. Nf3 Nxe4 5. d4 d5

			variation => N__('Russian Game: Classical, 5... d5'),
			moves => {
				'f1d3' => 'rnbqkb1r/ppp2ppp/8/3p4/3Pn3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C43j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. Nxe5

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.Nxe5'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp2ppp/2n5/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6',
				'b8d7' => 'r1bqkb1r/pppn1ppp/8/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6',
				'f8d6' => 'rnbqk2r/ppp2ppp/3b4/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/8/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/3Qn3/5N2/PPP2PPP/RNB1KB1R w KQkq d6 0 6' => {
			eco => 'C43d',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line'),
			moves => {
				'e5d6' => 'rnbqkb1r/ppp2ppp/3P4/8/3Qn3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C29n',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Nf3

			variation => N__('Vienna Gambit: 5. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp2ppp/2n5/3pP3/4n3/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6',
				'c8g4' => 'rn1qkb1r/ppp2ppp/8/3pP3/4n1b1/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6',
				'f8b4' => 'rnbqk2r/ppp2ppp/8/3pP3/1b2n3/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6',
				'f8e7' => 'rnbqk2r/ppp1bppp/8/3pP3/4n3/2N2N2/PPPP2PP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2N2Q2/PPPP2PP/R1B1KBNR b KQkq - 1 5' => {
			eco => 'C29k',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Qf3

			variation => N__('Vienna Gambit: Paulsen Attack'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppp2ppp/2n5/3pP3/4n3/2N2Q2/PPPP2PP/R1B1KBNR w KQkq - 2 6',
				'e4c3' => 'rnbqkb1r/ppp2ppp/8/3pP3/8/2n2Q2/PPPP2PP/R1B1KBNR w KQkq - 0 6',
				'f7f5' => 'rnbqkb1r/ppp3pp/8/3pPp2/4n3/2N2Q2/PPPP2PP/R1B1KBNR w KQkq f6 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C29f',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4

			variation => N__('Vienna Gambit: 4.fxe5'),
			moves => {
				'd1f3' => 'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2N2Q2/PPPP2PP/R1B1KBNR b KQkq - 1 5',
				'd2d3' => 'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2NP4/PPP3PP/R1BQKBNR b KQkq - 0 5',
				'g1f3' => 'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2NP4/PPP3PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C29g',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. d3

			variation => N__('Vienna Gambit: Oxford Variation'),
			moves => {
				'd8h4' => 'rnb1kb1r/ppp2ppp/8/3pP3/4n2q/2NP4/PPP3PP/R1BQKBNR w KQkq - 1 6',
				'e4c3' => 'rnbqkb1r/ppp2ppp/8/3pP3/8/2nP4/PPP3PP/R1BQKBNR w KQkq - 0 6',
				'f8b4' => 'rnbqk2r/ppp2ppp/8/3pP3/1b2n3/2NP4/PPP3PP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/4n3/2PB4/PP3PPP/RNBQK1NR w KQkq - 0 6' => {
			eco => 'B07h',
                        # TRANSLATORS: 1. e4 d6 2. d4 Nf6 3. Bd3 e5 4. c3 d5 5. dxe5 Nxe4

			variation => N__('Pirc: 3. Bd3 e5 4. c3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/4n3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C43i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5 5. dxe5

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5 5.dxe5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/8/2PP4/P1P3PP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'C29i',
                        # TRANSLATORS:

			variation => N__('Vienna Gambit: Oxford, 5...Nxc3'),
			moves => {
				'd5d4' => 'rnbqkb1r/ppp2ppp/8/4P3/3p4/2PP4/P1P3PP/R1BQKBNR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/8/2n2Q2/PPPP2PP/R1B1KBNR w KQkq - 0 6' => {
			eco => 'C29m',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Qf3 Nxc3

			variation => N__('Vienna Gambit: Paulsen Attack, 5...Nxc3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pP3/8/2nP4/PPP3PP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'C29i',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. d3 Nxc3

			variation => N__('Vienna Gambit: Oxford, 5...Nxc3'),
			moves => {
				'b2c3' => 'rnbqkb1r/ppp2ppp/8/3pP3/8/2PP4/P1P3PP/R1BQKBNR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp2ppp/8/3pp3/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq d6 0 5' => {
			eco => 'C43i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3 d5

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3 d5'),
			moves => {
				'd4e5' => 'rnbqkb1r/ppp2ppp/8/3pP3/4n3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 0 5',
				'f3e5' => 'rnbqkb1r/ppp2ppp/8/3pN3/3Pn3/3B4/PPP2PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp2ppp/8/4P3/3p4/2PP4/P1P3PP/R1BQKBNR w KQkq - 0 7' => {
			eco => 'C29j',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. d3 Nxc3 6. bxc3 d4

			variation => N__('Vienna Gambit: Oxford, 5...Nxc3 6.bxc3 d4'),
			moves => {
			},
		},
		'rnbqkb1r/ppp2ppp/8/4p3/8/2P3P1/P1PP1PBP/R1BQK1NR b KQkq - 0 6' => {
			eco => 'C26j',
                        # TRANSLATORS:

			variation => N__('Vienna: Smyslov, 3... d5, 5...Nxc3'),
			moves => {
				'f8d6' => 'rnbqk2r/ppp2ppp/3b4/4p3/8/2P3P1/P1PP1PBP/R1BQK1NR w KQkq - 1 7',
			},
		},
		'rnbqkb1r/ppp2ppp/8/4p3/8/2n3P1/PPPP1PBP/R1BQK1NR w KQkq - 0 6' => {
			eco => 'C26j',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3 d5 4. exd5 Nxd5 5. Bg2 Nxc3

			variation => N__('Vienna: Smyslov, 3... d5, 5...Nxc3'),
			moves => {
				'b2c3' => 'rnbqkb1r/ppp2ppp/8/4p3/8/2P3P1/P1PP1PBP/R1BQK1NR b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp3pN/3p3n/5p2/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 0 6' => {
			eco => 'C41b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. Bc4 f5 4. d4 exd4 5. Ng5 Nh6 6. Nxh7

			variation => N__('Philidor: Lopez Countergambit, Jänisch Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/3p1n2/3Pp3/4N3/2P5/PP3PPP/RNBQKB1R w KQkq - 1 8' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'f1d3' => 'rnbqkb1r/ppp3pp/3p1n2/3Pp3/4N3/2PB4/PP3PPP/RNBQK2R b KQkq - 2 8',
			},
		},
		'rnbqkb1r/ppp3pp/3p1n2/3Pp3/4N3/2PB4/PP3PPP/RNBQK2R b KQkq - 2 8' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'f8e7' => 'rnbqk2r/ppp1b1pp/3p1n2/3Pp3/4N3/2PB4/PP3PPP/RNBQK2R w KQkq - 3 9',
			},
		},
		'rnbqkb1r/ppp3pp/3p1n2/5p2/2PPp3/2N1P3/PP1N1PPP/R1BQKB1R b KQkq - 0 6' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4 5. Nd2 Nf6 6. e3

			variation => N__('English: Lukin, 5. Nd2 Nf6 6. e3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/3p1n2/5p2/2PPp3/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A21',
                        # TRANSLATORS:

			variation => N__('English: Lukin, 5. Nd2'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp3pp/3p1n2/5p2/2PPp3/2N1P3/PP1N1PPP/R1BQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp3pp/3p1n2/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4 5. Ng5 Nf6

			variation => N__('English: Lukin, 5. Ng5 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/3p3n/5pN1/2BpP3/8/PPP2PPP/RNBQK2R w KQkq - 2 6' => {
			eco => 'C41b',
                        # TRANSLATORS:

			variation => N__('Philidor: Lopez Countergambit'),
			moves => {
				'g5h7' => 'rnbqkb1r/ppp3pN/3p3n/5p2/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq d6 0 5' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. e3 d5

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. e3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq d6 0 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Nc3 d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Nc3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq d6 0 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Nf3 d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Nf3 d5'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 d5 5. Nf3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 d5 5. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq d6 0 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 d5

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 d5'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/5NP1/PP2PPBP/RNBQK2R b KQkq - 1 5',
				'g1h3' => 'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/6PN/PP2PPBP/RNBQK2R b KQkq - 1 5' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2 d5 5. Nh3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2 d5 5. Nh3'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/5p2/3p1p2/3P4/2N1P3/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'A80p',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. Bg5 d5 4. Bxf6 exf6 5. e3

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5 d5 4.Bxf6 exf6 5. e3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp4pp/2p2p2/3p1p2/3P4/2N1P3/PPP2PPP/R2QKBNR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/ppp3pp/5p2/3p1p2/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'A80p',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. Bg5 d5 4. Bxf6 exf6

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5 d5 4.Bxf6 exf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/ppp3pp/5p2/3p1p2/3P4/2N1P3/PPP2PPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppp3pp/8/3pPp2/3Pn3/2N2Q2/PPP3PP/R1B1KBNR b KQkq d3 0 6' => {
			eco => 'C29l',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Qf3 f5 6. d4

			variation => N__('Vienna Gambit: Paulsen Attack, Heyde Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppp3pp/8/3pPp2/4n3/2N2Q2/PPPP2PP/R1B1KBNR w KQkq f6 0 6' => {
			eco => 'C29l',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 d5 4. fxe5 Nxe4 5. Qf3 f5

			variation => N__('Vienna Gambit: Paulsen Attack, Bardeleben Variation'),
			moves => {
				'd2d4' => 'rnbqkb1r/ppp3pp/8/3pPp2/3Pn3/2N2Q2/PPP3PP/R1B1KBNR b KQkq d3 0 6',
			},
		},
		'rnbqkb1r/ppp4p/5nN1/3p3Q/2B1p3/8/PPPP1PPP/RNB1K2R w KQkq - 1 7' => {
			eco => 'C40p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4 4. Nxe5 d5 5. Qh5+ g6 6. Nxg6 Nf6

			variation => N__('Latvian Gambit: Svedenborg, 6.Nxg6 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P1P2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 5' => {
			eco => 'C11f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. f4

			variation => N__('French: Steinitz, 5. f4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/2N5/PPP3PP/R1BQKBNR w KQkq c6 0 6',
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P1P2/8/PPPN2PP/R1BQKBNR b KQkq f3 0 5' => {
			eco => 'C05e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. f4

			variation => N__('French: Tarrasch, Closed, 5. f4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P1P2/8/PPPN2PP/R1BQKBNR w KQkq c6 0 6',
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P2Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 2 5' => {
			eco => 'C11c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. Qg4

			variation => N__('French: Steinitz, Gledhill Attack'),
			moves => {
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'C11d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Nf6 4. e5 Nfd7 5. Nf3

			variation => N__('French: Steinitz, 5. Nf3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P4/2N2N2/PPP2PPP/R1BQKB1R w KQkq c6 0 6',
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C11c',
                        # TRANSLATORS:

			variation => N__('French: Steinitz'),
			moves => {
				'd1g4' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P2Q1/2N5/PPP2PPP/R1B1KBNR b KQkq - 2 5',
				'f2f4' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P1P2/2N5/PPP3PP/R1BQKBNR b KQkq f3 0 5',
				'g1f3' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'C05m',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. c3

			variation => N__('French: Tarrasch, Closed, 5. c3'),
			moves => {
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/3B4/PPPN1PPP/R1BQK1NR b KQkq - 2 5' => {
			eco => 'C05n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7 5. Bd3

			variation => N__('French: Tarrasch, 5. Bd3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1n1ppp/4p3/2ppP3/3P4/3B4/PPPN1PPP/R1BQK1NR w KQkq c6 0 6',
			},
		},
		'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'C05d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 Nf6 4. e5 Nfd7

			variation => N__('French: Tarrasch, Closed, 4...Nfd7'),
			moves => {
				'c2c3' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 5',
				'f1d3' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P4/3B4/PPPN1PPP/R1BQK1NR b KQkq - 2 5',
				'f2f4' => 'rnbqkb1r/pppn1ppp/4p3/3pP3/3P1P2/8/PPPN2PP/R1BQKBNR b KQkq f3 0 5',
			},
		},
		'rnbqkb1r/pppnpppp/4P3/3p4/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B02f',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. e5 Nfd7 4. e6

			variation => N__('Alekhine: Spielmann Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppnpppp/8/3pP3/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 1 4' => {
			eco => 'B02f',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3 d5 3. e5 Nfd7

			variation => N__('Alekhine: Scandinavian, 3. e5 Nfd7'),
			moves => {
				'e5e6' => 'rnbqkb1r/pppnpppp/4P3/3p4/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1p1p/5n2/4N3/2B1PppP/8/PPPP2P1/RNBQK2R b KQkq - 3 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Nf6 6. Bc4

			variation => N__('KGA: Kieseritsky, Berlin Defence, 6. Bc4'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp2p1p/5n2/3pN3/2B1PppP/8/PPPP2P1/RNBQK2R w KQkq d6 0 7',
			},
		},
		'rnbqkb1r/pppp1p1p/5n2/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 Nf6

			variation => N__('KGA: Kieseritsky, Berlin Defence'),
			moves => {
				'e5g4' => 'rnbqkb1r/pppp1p1p/5n2/8/4PpNP/8/PPPP2P1/RNBQKB1R b KQkq - 0 6',
				'f1c4' => 'rnbqkb1r/pppp1p1p/5n2/4N3/2B1PppP/8/PPPP2P1/RNBQK2R b KQkq - 3 6',
			},
		},
		'rnbqkb1r/pppp1p1p/5n2/6N1/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5 Nf6

			variation => N__('KGA: Allgaier, Schlechter Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1p1p/5n2/8/4PpNP/8/PPPP2P1/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Berlin Defence'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp2p1p/5n2/3p4/4PpNP/8/PPPP2P1/RNBQKB1R w KQkq d6 0 7',
			},
		},
		'rnbqkb1r/pppp1p1p/5np1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pppp1p1p/5np1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1p1p/5np1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A24',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3 g6 4. Bg2

			variation => N__('English: Bremen, 3... g6'),
			moves => {
				'f8g7' => 'rnbqk2r/pppp1pbp/5np1/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppp1p1p/5pp1/8/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45l',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 g6 3. Bxf6 exf6 4. e3

			variation => N__('Trompowsky 2... g6 3.Bxf6 exf6 4. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1p1p/5pp1/8/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'A45k',
                        # TRANSLATORS:

			variation => N__('Trompowsky 2... g6 3.Bxf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/pppp1p1p/5pp1/8/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1p2/4pn1p/6p1/3P3B/5N2/PPP1PPPP/RN1QKB1R w KQkq g6 0 5' => {
			eco => 'A46x',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bh4 g5

			variation => N__('Torre Attack: 3... h6 4. Bh4 g5'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1pp1/4pB1p/8/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 0 4' => {
			eco => 'A46y',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bxf6

			variation => N__('Torre Attack: 3... h6 4.Bxf6'),
			moves => {
				'd8f6' => 'rnb1kb1r/pppp1pp1/4pq1p/8/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp1pp1/4pB1p/8/3PP3/8/PPP2PPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'A45n',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6 4. Bxf6

			variation => N__('Trompowsky: 2... e6 3. e4 h6 4.Bxf6'),
			moves => {
				'd8f6' => 'rnb1kb1r/pppp1pp1/4pq1p/8/3PP3/8/PPP2PPP/RN1QKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp1pp1/4pn1p/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4' => {
			eco => 'A46x',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6

			variation => N__('Torre Attack: 3... h6'),
			moves => {
				'g5f6' => 'rnbqkb1r/pppp1pp1/4pB1p/8/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 0 4',
				'g5h4' => 'rnbqkb1r/pppp1pp1/4pn1p/8/3P3B/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1pp1/4pn1p/6B1/3PP3/8/PPP2PPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'A45n',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4 h6

			variation => N__('Trompowsky: 2... e6 3. e4 h6'),
			moves => {
				'g5f6' => 'rnbqkb1r/pppp1pp1/4pB1p/8/3PP3/8/PPP2PPP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1pp1/4pn1p/8/3P3B/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 4' => {
			eco => 'A46x',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5 h6 4. Bh4

			variation => N__('Torre Attack: 3... h6 4. Bh4'),
			moves => {
				'g7g5' => 'rnbqkb1r/pppp1p2/4pn1p/6p1/3P3B/5N2/PPP1PPPP/RN1QKB1R w KQkq g6 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/2n5/8/2P1PP2/8/PP4PP/RNBQKBNR w KQkq - 1 6' => {
			eco => 'A52d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. e4 Nxe5 5. f4 Nec6

			variation => N__('Budapest: Alekhine, Abonyi Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/3n4/4Q3/2B5/2N5/PPPP1PPP/R1B1K1NR b KQkq - 0 5' => {
			eco => 'C27d',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Qxe5+

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5.Qxe5+'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/3n4/4p2Q/2B5/2N5/PPPP1PPP/R1B1K1NR w KQkq - 2 5' => {
			eco => 'C27c',
                        # TRANSLATORS:

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5'),
			moves => {
				'c4b3' => 'rnbqkb1r/pppp1ppp/3n4/4p2Q/8/1BN5/PPPP1PPP/R1B1K1NR b KQkq - 3 5',
				'h5e5' => 'rnbqkb1r/pppp1ppp/3n4/4Q3/2B5/2N5/PPPP1PPP/R1B1K1NR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/3n4/4p2Q/8/1BN5/PPPP1PPP/R1B1K1NR b KQkq - 3 5' => {
			eco => 'C27e',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5 Nd6 5. Bb3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2nn4/4p2Q/8/1BN5/PPPP1PPP/R1B1K1NR w KQkq - 4 6',
				'f8e7' => 'rnbqk2r/ppppbppp/3n4/4p2Q/8/1BN5/PPPP1PPP/R1B1K1NR w KQkq - 4 6',
			},
		},
		'rnbqkb1r/pppp1ppp/4p3/2PnP3/2B5/2N5/PP1P1PPP/R1BQK1NR b KQkq - 1 6' => {
			eco => 'B02u',
                        # TRANSLATORS:

			variation => N__('Alekhine: Two Pawns Attack, 5. Bc4'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp2ppp/3pp3/2PnP3/2B5/2N5/PP1P1PPP/R1BQK1NR w KQkq - 0 7',
			},
		},
		'rnbqkb1r/pppp1ppp/4p3/2PnP3/2B5/8/PP1P1PPP/RNBQK1NR w KQkq - 0 6' => {
			eco => 'B02u',
                        # TRANSLATORS:

			variation => N__('Alekhine: Two Pawns Attack, 5. Bc4'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp1ppp/4p3/2PnP3/2B5/2N5/PP1P1PPP/R1BQK1NR b KQkq - 1 6',
			},
		},
		'rnbqkb1r/pppp1ppp/4p3/8/2PPn3/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'E10b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 Ne4

			variation => N__('Neo-Indian: Döry Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/2PP4/8/PP2PPPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'E00c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Bg5

			variation => N__('Neo-Indian (Seirawan) Attack'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A45m',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. Nc3

			variation => N__('Trompowsky: 2... e6 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A45m',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e3

			variation => N__('Trompowsky: 2... e6 3. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A46r',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bg5

			variation => N__('Torre Attack'),
			moves => {
				'b7b6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p3B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq c6 0 4',
				'f8e7' => 'rnbqk2r/ppppbppp/4pn2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
				'h7h6' => 'rnbqkb1r/pppp1pp1/4pn1p/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A45m',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6

			variation => N__('Trompowsky: 2... e6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3',
				'b1d2' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/8/PPPNPPPP/R2QKBNR b KQkq - 1 3',
				'e2e3' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 3',
				'e2e4' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/3PP3/8/PPP2PPP/RN1QKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/8/PPPNPPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A45m',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. Nd2

			variation => N__('Trompowsky: 2... e6 3. Nd2'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/6B1/3PP3/8/PPP2PPP/RN1QKBNR b KQkq e3 0 3' => {
			eco => 'A45n',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 e6 3. e4

			variation => N__('Trompowsky: 2... e6 3. e4'),
			moves => {
				'h7h6' => 'rnbqkb1r/pppp1pp1/4pn1p/6B1/3PP3/8/PPP2PPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4 Nf6 2. Bb2 e6

			variation => N__('Polish: 1... Nf6 2. Bb2 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A18',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4

			variation => N__('English: Mikenas'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n1pn2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 1 4',
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq c6 0 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq d6 0 4',
				'd7d6' => 'rnbqkb1r/ppp2ppp/3ppn2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. Nf3

			variation => N__('English: Anglo-Indian, 2. Nc3 e6 3. Nf3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq d6 0 4',
				'f8b4' => 'rnbqk2r/pppp1ppp/4pn2/8/1bP5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. g3

			variation => N__('English: Anglo-Indian, 2. Nc3 e6 3. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A17',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6

			variation => N__('English: Anglo-Indian, 2. Nc3 e6'),
			moves => {
				'e2e4' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A13i',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 Nf6

			variation => N__('English: 1... e6 2. Nf3 Nf6'),
			moves => {
				'g2g3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A13j',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 Nf6 3. g3

			variation => N__('English: 1... e6 2. Nf3 Nf6 3. g3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1ppp1ppp/p3pn2/8/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. g3 e6

			variation => N__('English: Anglo-Indian, 2. g3 e6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. g3 e6 3. Bg2

			variation => N__('English: Anglo-Indian, 2. g3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2PP2P1/8/PP2PP1P/RNBQKBNR b KQkq g3 0 3' => {
			eco => 'E00a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g4

			variation => N__('Queen\'s Pawn: Neo-Indian, Devin Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'E00d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3

			variation => N__('Queen\'s Pawn: Neo-Indian'),
			moves => {
				'b7b6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4',
				'f8b4' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'E10a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3

			variation => N__('Neo-Indian: 3. Nf3'),
			moves => {
				'a7a6' => 'rnbqkb1r/1ppp1ppp/p3pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'b7b6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4',
				'f6e4' => 'rnbqkb1r/pppp1ppp/4p3/8/2PPn3/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'f8b4' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'f8e7' => 'rnbqk2r/ppppbppp/4pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'E00g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3

			variation => N__('Catalan'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq c6 0 4',
				'c7c6' => 'rnbqkb1r/pp1p1ppp/2p1pn2/8/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq d6 0 4',
				'e6e5' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4',
				'f8b4' => 'rnbqk2r/pppp1ppp/4pn2/8/1bPP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'E00a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6

			variation => N__('Queen\'s Pawn: Neo-Indian'),
			moves => {
				'a2a3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/P7/1P2PPPP/RNBQKBNR b KQkq - 0 3',
				'b1c3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'c1g5' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/2PP4/8/PP2PPPP/RN1QKBNR b KQkq - 1 3',
				'g1f3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3',
				'g2g4' => 'rnbqkb1r/pppp1ppp/4pn2/8/2PP2P1/8/PP2PP1P/RNBQKBNR b KQkq g3 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/P7/1P2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'E00b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. a3

			variation => N__('Queen\'s Pawn: Anti-Nimzo-Indian'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/P7/1P2PPPP/RNBQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A46p',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. Bf4

			variation => N__('Indian: London System'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46j',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. c3

			variation => N__('Indian: 2. Nf3 e6 3. c3'),
			moves => {
				'b7b6' => 'rnbqkb1r/p1pp1ppp/1p2pn2/8/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/3P4/4P3/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A45d',
                        # TRANSLATORS: 1. d4 Nf6 2. e3 e6

			variation => N__('Indian: 2. e3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46k',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. e3

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. e3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A46j',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6

			variation => N__('Indian: 2. Nf3 e6'),
			moves => {
				'c1f4' => 'rnbqkb1r/pppp1ppp/4pn2/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'c1g5' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'c2c3' => 'rnbqkb1r/pppp1ppp/4pn2/8/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/pppp1ppp/4pn2/8/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkb1r/pppp1ppp/4pn2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/4pn2/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A46m',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 e6 3. g3

			variation => N__('Indian: 1. d4 Nf6 2. Nf3 e6 3. g3'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pp1ppp/4pn2/1p6/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq b6 0 4',
				'c7c5' => 'rnbqkb1r/pp1p1ppp/4pn2/2p5/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/1B2p3/3PP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Bb5 Nf6 3. d4

			variation => N__('Open Game: Portuguese Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Bb5 Nf6

			variation => N__('Open Game: Portuguese, 2... Nf6'),
			moves => {
				'd2d4' => 'rnbqkb1r/pppp1ppp/5n2/1B2p3/3PP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/3Pp3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A51c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. d5

			variation => N__('Budapest: 3. d5'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4N3/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C42g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5

			variation => N__('Russian Game: 3.Nxe5'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp2ppp/3p1n2/4N3/4P3/8/PPPP1PPP/RNBQKB1R w KQkq - 0 4',
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/4N3/4n3/8/PPPP1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4P3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A51d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5

			variation => N__('Budapest: 3.dxe5'),
			moves => {
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/8/PP2PPPP/RNBQKBNR w KQkq - 1 4',
				'f6g4' => 'rnbqkb1r/pppp1ppp/8/4P3/2P3n1/8/PP2PPPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4P3/3p4/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C43b',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack'),
			moves => {
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/4P3/3pn3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3' => {
			eco => 'C26l',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4

			variation => N__('Vienna: 3. Bc4'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pp1ppp/5n2/1p2p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq b6 0 4',
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4',
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/4p3/2B1n3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 0 4',
				'f8b4' => 'rnbqk2r/pppp1ppp/5n2/4p3/1bB1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4',
				'f8c5' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 4 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/3P4/PPP2PPP/RNBQK1NR b KQkq - 0 3' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d3

			variation => N__('Bishop\'s Opening: 3. d3'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2B1P3/3P4/PPP2PPP/RNBQK1NR w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/2B1P3/3P4/PPP2PPP/RNBQK1NR w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3' => {
			eco => 'C42e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Bc4

			variation => N__('Russian Game: Italian Variation'),
			moves => {
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/4p3/2B1n3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6

			variation => N__('Bishop\'s Opening: Berlin Defence'),
			moves => {
				'd2d3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/3P4/PPP2PPP/RNBQK1NR b KQkq - 0 3',
				'd2d4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3',
				'f2f4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2B1PP2/8/PPPP2PP/RNBQK1NR b KQkq f3 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2B1PP2/8/PPPP2PP/RNBQK1NR b KQkq f3 0 3' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. f4

			variation => N__('Bishop\'s Opening: Greco Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4

			variation => N__('Bishop\'s Opening: 3. d4'),
			moves => {
				'e5d4' => 'rnbqkb1r/pppp1ppp/5n2/8/2BpP3/8/PPP2PPP/RNBQK1NR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. e4

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. e4'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N1P3/PP1P1PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. e3

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. e3'),
			moves => {
				'f8b4' => 'rnbqk2r/pppp1ppp/5n2/4p3/1bP5/2N1P3/PP1P1PPP/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. Nf3

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. Nf3'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'e5e4' => 'rnbqkb1r/pppp1ppp/5n2/8/2P1p3/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. g3

			variation => N__('English: Bremen System'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq d6 0 4',
				'f8b4' => 'rnbqk2r/pppp1ppp/5n2/4p3/1bP5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
				'f8c5' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
				'g7g6' => 'rnbqkb1r/pppp1p1p/5np1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6

			variation => N__('English: King\'s, 2. Nc3 Nf6'),
			moves => {
				'e2e3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N1P3/PP1P1PPP/R1BQKBNR b KQkq - 0 3',
				'e2e4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3',
				'g2g3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nf6

			variation => N__('English: King\'s, 2. g3 Nf6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 Nf6 3. Bg2

			variation => N__('English: King\'s, 2. g3 Nf6 3. Bg2'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 3 4',
				'c7c6' => 'rnbqkb1r/pp1p1ppp/2p2n2/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq d6 0 4',
				'f8c5' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 3 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A51b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. e3

			variation => N__('Budapest: 3. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4' => {
			eco => 'E00g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. g3 e5

			variation => N__('Catalan: Hungarian Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'A51a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5

			variation => N__('Budapest'),
			moves => {
				'd4d5' => 'rnbqkb1r/pppp1ppp/5n2/3Pp3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'd4e5' => 'rnbqkb1r/pppp1ppp/5n2/4P3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/3P2P1/8/PPP1PP1P/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. g4 e5

			variation => N__('Indian: Gibbins Gambit, Oshima Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/3P4/8/PPPNPPPP/R1BQKBNR w KQkq e6 0 3' => {
			eco => 'A45c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nd2 e5

			variation => N__('Indian: Lazard Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'C43a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4

			variation => N__('Russian Game: Modern (Steinitz) Attack'),
			moves => {
				'e5d4' => 'rnbqkb1r/pppp1ppp/5n2/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/4p3/3Pn3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'C42c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nc3

			variation => N__('Russian-Three Knights Game'),
			moves => {
				'f8b4' => 'rnbqk2r/pppp1ppp/5n2/4p3/1b2P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 4 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'C26c',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. g3

			variation => N__('Vienna: Smyslov Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n2n2/4p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 1 4',
				'd7d5' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq d6 0 4',
				'f8c5' => 'rnbqk2r/pppp1ppp/5n2/2b1p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'C26a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6

			variation => N__('Vienna: 2... Nf6'),
			moves => {
				'a2a3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/P1N5/1PPP1PPP/R1BQKBNR b KQkq - 0 3',
				'd2d3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2NP4/PPP2PPP/R1BQKBNR b KQkq - 0 3',
				'f1c4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 3 3',
				'f2f4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
				'g2g3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2NP4/PPP2PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'C26b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. d3

			variation => N__('Vienna: 2... Nf6 3. d3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C42b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d3

			variation => N__('Russian Game: 3. d3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'C42a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6

			variation => N__('Russian Game (Petroff Defence)'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 3 3',
				'd2d3' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'd2d4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'f1c4' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 3 3',
				'f3e5' => 'rnbqkb1r/pppp1ppp/5n2/4N3/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/8/PPPPNPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Ne2 Nf6

			variation => N__('Open Game: Alapin Opening'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/P1N5/1PPP1PPP/R1BQKBNR b KQkq - 0 3' => {
			eco => 'C26a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. a3

			variation => N__('Vienna: Mengarini Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'C29a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4

			variation => N__('Vienna Gambit'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp2ppp/5n2/3pp3/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq d6 0 4',
				'd7d6' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/4PP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
				'e5f4' => 'rnbqkb1r/pppp1ppp/5n2/8/4Pp2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'C30d',
                        # TRANSLATORS: 1. e4 e5 2. f4 Nf6

			variation => N__('King\'s Gambit: Wade Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/6B1/2BQ4/5N2/PPP2PPP/RN2K2R b KQkq - 2 6' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n2n2/6B1/2BQ4/5N2/PPP2PPP/RN2K2R w KQkq - 3 7',
				'f8e7' => 'rnbqk2r/ppppbppp/5n2/6B1/2BQ4/5N2/PPP2PPP/RN2K2R w KQkq - 3 7',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/6N1/2P1p3/2N5/PP1PPPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A22',
                        # TRANSLATORS:

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. Nf3 e4'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pp1ppp/5n2/1p4N1/2P1p3/2N5/PP1PPPPP/R1BQKB1R w KQkq b6 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2B1P3/8/PB3PPP/RN1QK1NR w KQkq - 1 6' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3 4. Bc4 cxb2 5. Bxb2 Nf6

			variation => N__('Danish Gambit: Classical Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2B1Pp2/2N5/PPPP2PP/R1BQK1NR b KQkq - 3 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Nf6 4. Nc3

			variation => N__('KGA: Bishop\'s Gambit, Bogoljubow Variation'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp1p1ppp/2p2n2/8/2B1Pp2/2N5/PPPP2PP/R1BQK1NR w KQkq - 0 5',
				'f8b4' => 'rnbqk2r/pppp1ppp/5n2/8/1bB1Pp2/2N5/PPPP2PP/R1BQK1NR w KQkq - 4 5',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Nf6

			variation => N__('KGA: Bishop\'s Gambit, Cozio Defence'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp1ppp/5n2/8/2B1Pp2/2N5/PPPP2PP/R1BQK1NR b KQkq - 3 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2BQ4/2N2N2/PPP2PPP/R1B1K2R b KQkq - 2 6' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n2n2/8/2BQ4/2N2N2/PPP2PPP/R1B1K2R w KQkq - 3 7',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2BQ4/5N2/PPP2PPP/RNB1K2R w KQkq - 1 6' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp1ppp/5n2/8/2BQ4/2N2N2/PPP2PPP/R1B1K2R b KQkq - 2 6',
				'c1g5' => 'rnbqkb1r/pppp1ppp/5n2/6B1/2BQ4/5N2/PPP2PPP/RN2K2R b KQkq - 2 6',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2BpP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C24',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 Nf6 3. d4 exd4 4. Nf3

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp2ppp/5n2/3p4/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq d6 0 5',
				'f6e4' => 'rnbqkb1r/pppp1ppp/8/8/2Bpn3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2BpP3/8/PPP2PPP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: 3. d4'),
			moves => {
				'g1f3' => 'rnbqkb1r/pppp1ppp/5n2/8/2BpP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/2P1p3/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A22',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 Nf6 3. Nf3 e4

			variation => N__('English: King\'s, 2. Nc3 Nf6 3. Nf3 e4'),
			moves => {
				'f3g5' => 'rnbqkb1r/pppp1ppp/5n2/6N1/2P1p3/2N5/PP1PPPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C43b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4

			variation => N__('Russian Game: Modern Attack'),
			moves => {
				'e4e5' => 'rnbqkb1r/pppp1ppp/5n2/4P3/3p4/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/4Pp2/2N5/PPPP2PP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C29a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. f4 exf4

			variation => N__('Vienna Gambit: 3...exf4'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/5n2/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 Nf6

			variation => N__('KGA: Schallop Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/1B2P3/3pn3/5N2/PPP2PPP/RNBQK2R b KQkq - 2 5' => {
			eco => 'C43c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4 5. Bb5

			variation => N__('Russian Game: Modern Attack, Tal Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/2n1P3/3N4/8/PPP1QPPP/RNB1KB1R b KQkq - 0 6' => {
			eco => 'C43c',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, Steinitz Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n5/2n1P3/3N4/8/PPP1QPPP/RNB1KB1R w KQkq - 1 7',
			},
		},
		'rnbqkb1r/pppp1ppp/8/2n1P3/3p4/5N2/PPP1QPPP/RNB1KB1R w KQkq - 3 6' => {
			eco => 'C43c',
                        # TRANSLATORS:

			variation => N__('Russian Game: Modern Attack, Steinitz Variation'),
			moves => {
				'f3d4' => 'rnbqkb1r/pppp1ppp/8/2n1P3/3N4/8/PPP1QPPP/RNB1KB1R b KQkq - 0 6',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4N3/4n3/8/PPPP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C42g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Nxe5 Nxe4

			variation => N__('Russian Game: Damiano Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P1P1n1/8/PP3PPP/RNBQKBNR b KQkq e3 0 4' => {
			eco => 'A52c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. e4

			variation => N__('Budapest: Alekhine Variation'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp2ppp/3p4/4P3/2P1P1n1/8/PP3PPP/RNBQKBNR w KQkq - 0 5',
				'g4e5' => 'rnbqkb1r/pppp1ppp/8/4n3/2P1P3/8/PP3PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'A51h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. Nf3

			variation => N__('Budapest: Fajarowicz, 4. Nf3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n5/4P3/2P1n3/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 5',
				'f8b4' => 'rnbqk2r/pppp1ppp/8/4P3/1bP1n3/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/8/PP1NPPPP/R1BQKBNR b KQkq - 2 4' => {
			eco => 'A51f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. Nd2

			variation => N__('Budapest: Fajarowicz, 4. Nd2'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/8/PP2PPPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A51d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4

			variation => N__('Budapest: Fajarowicz'),
			moves => {
				'a2a3' => 'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/P7/1P2PPPP/RNBQKBNR b KQkq - 0 4',
				'b1d2' => 'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/8/PP1NPPPP/R1BQKBNR b KQkq - 2 4',
				'd1c2' => 'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/8/PPQ1PPPP/RNB1KBNR b KQkq - 2 4',
				'g1f3' => 'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/8/PPQ1PPPP/RNB1KBNR b KQkq - 2 4' => {
			eco => 'A51e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. Qc2

			variation => N__('Budapest: Fajarowicz, Steiner Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P1n3/P7/1P2PPPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'A51g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ne4 4. a3

			variation => N__('Budapest: Fajarowicz, 4. a3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P2Bn1/8/PP2PPPP/RN1QKBNR b KQkq - 2 4' => {
			eco => 'A52h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Bf4

			variation => N__('Budapest: Rubinstein Variation'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n5/4P3/2P2Bn1/8/PP2PPPP/RN1QKBNR w KQkq - 3 5',
				'f8b4' => 'rnbqk2r/pppp1ppp/8/4P3/1bP2Bn1/8/PP2PPPP/RN1QKBNR w KQkq - 3 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P3n1/4P3/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'A52b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. e3

			variation => N__('Budapest: 3... Ng4 4. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P3n1/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 4' => {
			eco => 'A52e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4 4. Nf3

			variation => N__('Budapest: Adler Variation'),
			moves => {
				'f8c5' => 'rnbqk2r/pppp1ppp/8/2b1P3/2P3n1/5N2/PP2PPPP/RNBQKB1R w KQkq - 3 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/2P3n1/8/PP2PPPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'A52a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e5 3. dxe5 Ng4

			variation => N__('Budapest: 3... Ng4'),
			moves => {
				'c1f4' => 'rnbqkb1r/pppp1ppp/8/4P3/2P2Bn1/8/PP2PPPP/RN1QKBNR b KQkq - 2 4',
				'e2e3' => 'rnbqkb1r/pppp1ppp/8/4P3/2P3n1/4P3/PP3PPP/RNBQKBNR b KQkq - 0 4',
				'e2e4' => 'rnbqkb1r/pppp1ppp/8/4P3/2P1P1n1/8/PP3PPP/RNBQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqkb1r/pppp1ppp/8/4P3/2P3n1/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/3Qn3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 5' => {
			eco => 'C43d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4 5. Qxd4

			variation => N__('Russian Game: Modern Attack, 3...exd4, Main Line'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp2ppp/8/3pP3/3Qn3/5N2/PPP2PPP/RNB1KB1R w KQkq d6 0 6',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/3pn3/5N2/PPP1QPPP/RNB1KB1R b KQkq - 2 5' => {
			eco => 'C43c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4 5. Qe2

			variation => N__('Russian Game: Modern Attack, Steinitz Variation'),
			moves => {
				'e4c5' => 'rnbqkb1r/pppp1ppp/8/2n1P3/3p4/5N2/PPP1QPPP/RNB1KB1R w KQkq - 3 6',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4P3/3pn3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C43c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 exd4 4. e5 Ne4

			variation => N__('Russian Game: Modern Attack'),
			moves => {
				'd1d4' => 'rnbqkb1r/pppp1ppp/8/4P3/3Qn3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 5',
				'd1e2' => 'rnbqkb1r/pppp1ppp/8/4P3/3pn3/5N2/PPP1QPPP/RNB1KB1R b KQkq - 2 5',
				'f1b5' => 'rnbqkb1r/pppp1ppp/8/1B2P3/3pn3/5N2/PPP2PPP/RNBQK2R b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4n3/2P1P3/8/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'A52c',
                        # TRANSLATORS:

			variation => N__('Budapest: Alekhine Variation'),
			moves => {
				'f2f4' => 'rnbqkb1r/pppp1ppp/8/4n3/2P1PP2/8/PP4PP/RNBQKBNR b KQkq f3 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4n3/2P1PP2/8/PP4PP/RNBQKBNR b KQkq f3 0 5' => {
			eco => 'A52c',
                        # TRANSLATORS:

			variation => N__('Budapest: Alekhine Variation'),
			moves => {
				'e5c6' => 'rnbqkb1r/pppp1ppp/2n5/8/2P1PP2/8/PP4PP/RNBQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p2Q/2B1n3/2N5/PPPP1PPP/R1B1K1NR b KQkq - 1 4' => {
			eco => 'C27c',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Qh5

			variation => N__('Vienna: 3. Bc4 Nxe4 4. Qh5'),
			moves => {
				'e4d6' => 'rnbqkb1r/pppp1ppp/3n4/4p2Q/2B5/2N5/PPPP1PPP/R1B1K1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/2B1N3/8/PPPP1PPP/R1BQK1NR b KQkq - 0 4' => {
			eco => 'C27b',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4 4. Nxe4

			variation => N__('Vienna: 3. Bc4 Nxe4 4.Nxe4'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/2B1n3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4' => {
			eco => 'C42f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Bc4 Nxe4 4. Nc3

			variation => N__('Russian Game: Boden-Kieseritsky Gambit'),
			moves => {
				'e4c3' => 'rnbqkb1r/pppp1ppp/8/4p3/2B5/2n2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/2B1n3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 0 4' => {
			eco => 'C27a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 Nf6 3. Bc4 Nxe4

			variation => N__('Vienna: 3. Bc4 Nxe4'),
			moves => {
				'c3e4' => 'rnbqkb1r/pppp1ppp/8/4p3/2B1N3/8/PPPP1PPP/R1BQK1NR b KQkq - 0 4',
				'd1h5' => 'rnbqkb1r/pppp1ppp/8/4p2Q/2B1n3/2N5/PPPP1PPP/R1B1K1NR b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/2B1n3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C42e',
                        # TRANSLATORS:

			variation => N__('Russian Game: Italian Variation'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp1ppp/8/4p3/2B1n3/2N2N2/PPPP1PPP/R1BQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/2B5/2n2N2/PPPP1PPP/R1BQK2R w KQkq - 0 5' => {
			eco => 'C42f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. Bc4 Nxe4 4. Nc3 Nxc3

			variation => N__('Russian Game: Boden-Kieseritsky Gambit Accepted'),
			moves => {
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/3Pn3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C43g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4 4. Bd3

			variation => N__('Russian Game: Modern Attack, 3...Nxe4 4. Bd3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppp1ppp/2n5/4p3/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq - 2 5',
				'd7d5' => 'rnbqkb1r/ppp2ppp/8/3pp3/3Pn3/3B1N2/PPP2PPP/RNBQK2R w KQkq d6 0 5',
			},
		},
		'rnbqkb1r/pppp1ppp/8/4p3/3Pn3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C43f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 Nf6 3. d4 Nxe4

			variation => N__('Russian Game: Modern Attack, 3...Nxe4'),
			moves => {
				'f1d3' => 'rnbqkb1r/pppp1ppp/8/4p3/3Pn3/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp1ppp/8/8/2BQn3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 5' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'e4f6' => 'rnbqkb1r/pppp1ppp/5n2/8/2BQ4/5N2/PPP2PPP/RNB1K2R w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pppp1ppp/8/8/2Bpn3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C24',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Urusov Gambit'),
			moves => {
				'd1d4' => 'rnbqkb1r/pppp1ppp/8/8/2BQn3/5N2/PPP2PPP/RNB1K2R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'f1g2' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3 e6 4. Bg2

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 0 5',
				'f8e7' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2P5/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. e3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. e3'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq d6 0 5',
				'f8b4' => 'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. Nf3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Nc3'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq d6 0 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6'),
			moves => {
				'a2a3' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/P1N5/1P2PPPP/R1BQKBNR b KQkq - 0 4',
				'c1g5' => 'rnbqkb1r/pppp2pp/4pn2/5pB1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4',
				'd1c2' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 1 4',
				'e2e3' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N5/PPQ1PPPP/R1B1KBNR b KQkq - 1 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. Qc2

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. Qc2'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6 3. Nf3 Nf6

			variation => N__('Dutch: 2. c4 e6 3. Nf3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Nf3

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Nf3'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq d6 0 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 1 4',
				'f1g2' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'A90',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 e6 4. Bg2

			variation => N__('Dutch: 2. c4 Nf6 3. g3 e6 4. Bg2'),
			moves => {
				'c7c6' => 'rnbqkb1r/pp1p2pp/2p1pn2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5',
				'd7d5' => 'rnbqkb1r/ppp3pp/4pn2/3p1p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq d6 0 5',
				'f8b4' => 'rnbqk2r/pppp2pp/4pn2/5p2/1bPP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5',
				'f8e7' => 'rnbqk2r/ppppb1pp/4pn2/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/P1N5/1P2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. a3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. a3'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 e6 3. Nf3 Nf6

			variation => N__('Dutch: 2. g3 e6 3. Nf3 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6 3. Bg2 e6

			variation => N__('Dutch: 2. g3 Nf6 3. Bg2 e6'),
			moves => {
				'g1h3' => 'rnbqkb1r/pppp2pp/4pn2/5p2/3P4/6PN/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5p2/3P4/6PN/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6 3. Bg2 e6 4. Nh3

			variation => N__('Dutch: Blackburne Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5pB1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 e6 4. Bg5

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 e6 4. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/5pB1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4' => {
			eco => 'A80n',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. Bg5 e6

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 5' => {
			eco => 'A83h',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 e6 5. Nxe4

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 e6 5.Nxe4'),
			moves => {
				'f8e7' => 'rnbqk2r/ppppb1pp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR w KQkq - 1 6',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'A83g',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 e6

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 e6'),
			moves => {
				'c3e4' => 'rnbqkb1r/pppp2pp/4pn2/6B1/3PN3/8/PPP2PPP/R2QKBNR b KQkq - 0 5',
			},
		},
		'rnbqkb1r/pppp2pp/4pn2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3 e6

			variation => N__('Dutch: Staunton Gambit, 4. f3 e6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/5n2/4N3/2B1p3/8/PPPP1PPP/RNBQK2R w KQkq - 1 5' => {
			eco => 'C40p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4 4. Nxe5 Nf6

			variation => N__('Latvian: Blackburne Variation (Corkscrew Countergambit)'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/5n2/4N3/3Pp3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5' => {
			eco => 'C40m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. d4 fxe4 4. Nxe5 Nf6

			variation => N__('Latvian Gambit: 3. d4 fxe4 5.Nxe5 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/5n2/4p1N1/4p3/3P4/PPP2PPP/RNBQKB1R w KQkq e6 0 5' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. e4 fxe4 3. Ng5 Nf6 4. d3 e5

			variation => N__('Reti: Lisitsin: 3. Ng5 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/5n2/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4' => {
			eco => 'C40o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 Nf6

			variation => N__('Latvian Gambit: Morgado Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppp2pp/5n2/4pp2/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 f5 3. g3 Nf6

			variation => N__('English: King\'s, 2. Nc3 f5'),
			moves => {
				'f1g2' => 'rnbqkb1r/pppp2pp/5n2/4pp2/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppp2pp/5n2/4pp2/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 2 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 f5 3. g3 Nf6 4. Bg2

			variation => N__('English: King\'s, 2. Nc3 f5'),
			moves => {
			},
		},
		'rnbqkb1r/ppppnppp/8/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 Ne7

			variation => N__('KGA: Bishop\'s Gambit, Steinitz Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppppnppp/8/8/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 Ne7

			variation => N__('Danish Gambit: Svenonius Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppppnppp/8/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 Ne7

			variation => N__('KGA: Bonsch-Osmolovsky Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. Nc3

			variation => N__('English: Anglo-Dutch'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nc3 Nf6 3. g3

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'g7g6' => 'rnbqkb1r/ppppp2p/5np1/5p2/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nc3 Nf6

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'g2g3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 3 3',
				'g2g3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 Nf6 3. g3

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. g3 Nf6

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. g3 Nf6 3. Bg2

			variation => N__('English: Anglo-Dutch'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp1p1pp/3p1n2/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nf3

			variation => N__('Dutch: 2. c4 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3

			variation => N__('Dutch: 2. c4 Nf6 3. g3'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp1p1pp/3p1n2/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4',
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6

			variation => N__('Dutch: 2. c4 Nf6'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 2 3',
				'g1f3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 2 3',
				'g2g3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P2P1/2N5/PPP1PP1P/R1BQKBNR b KQkq g3 0 3' => {
			eco => 'A80m',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. g4

			variation => N__('Dutch: Spielmann Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P2P1/7P/PPP1PP2/RNBQKBNR b KQkq g3 0 3' => {
			eco => 'A80c',
                        # TRANSLATORS: 1. d4 f5 2. h3 Nf6 3. g4

			variation => N__('Dutch: Korchnoi, Janzen Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 3' => {
			eco => 'A80m',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6

			variation => N__('Dutch: 2. Nc3 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppppp1pp/5n2/5pB1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 3 3',
				'g2g4' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P2P1/2N5/PPP1PP1P/R1BQKBNR b KQkq g3 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A80u',
                        # TRANSLATORS: 1. d4 f5 2. Nf3 Nf6 3. c3

			variation => N__('Dutch: Barcza System'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3' => {
			eco => 'A80t',
                        # TRANSLATORS: 1. d4 f5 2. Nf3 Nf6

			variation => N__('Dutch: 2. Nf3 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppppp1pp/5n2/5pB1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3',
				'c2c3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 2 3' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6 3. Nf3

			variation => N__('Dutch: 2. g3 Nf6 3. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6

			variation => N__('Dutch: 2. g3 Nf6'),
			moves => {
				'f1g2' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR b KQkq - 2 3',
				'g1f3' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 2 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6 3. Bg2

			variation => N__('Dutch: 2. g3 Nf6 3. Bg2'),
			moves => {
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR w KQkq - 0 4',
				'g7g6' => 'rnbqkb1r/ppppp2p/5np1/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/7P/PPP1PPP1/RNBQKBNR w KQkq - 1 3' => {
			eco => 'A80c',
                        # TRANSLATORS:

			variation => N__('Dutch: Korchnoi Attack'),
			moves => {
				'g2g4' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P2P1/7P/PPP1PP2/RNBQKBNR b KQkq g3 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq e3 0 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. d3 Nf6 3. e4

			variation => N__('Reti: Lisitsin Deferred'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5p2/8/3P1N2/PPP1PPPP/RNBQKB1R w KQkq - 1 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. d3 Nf6

			variation => N__('Reti: 1... f5 2. d3 Nf6'),
			moves => {
				'e2e4' => 'rnbqkb1r/ppppp1pp/5n2/5p2/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq e3 0 3',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5pB1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 3 3' => {
			eco => 'A80n',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 Nf6 3. Bg5

			variation => N__('Dutch: 2. Nc3 Nf6 3. Bg5'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp1p1pp/5n2/3p1pB1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq d6 0 4',
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/5pB1/3P4/2N5/PPP1PPPP/R2QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5pB1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 3 3' => {
			eco => 'A80v',
                        # TRANSLATORS: 1. d4 f5 2. Nf3 Nf6 3. Bg5

			variation => N__('Dutch: 2. Nf3 Nf6 3. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 2 3' => {
			eco => 'A80e',
                        # TRANSLATORS: 1. d4 f5 2. Bg5 Nf6

			variation => N__('Dutch: 2. Bg5 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR b KQkq - 3 4' => {
			eco => 'A83a',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5

			variation => N__('Dutch: Staunton Gambit, Staunton Variation'),
			moves => {
				'b7b6' => 'rnbqkb1r/p1ppp1pp/1p3n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5',
				'b8c6' => 'r1bqkb1r/ppppp1pp/2n2n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 4 5',
				'c7c6' => 'rnbqkb1r/pp1pp1pp/2p2n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5',
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5',
				'g7g6' => 'rnbqkb1r/ppppp2p/5np1/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/6N1/4p3/3P4/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: Lisitsin: 3. Ng5 Nf6'),
			moves => {
				'e4e3' => 'rnbqkb1r/ppppp1pp/5n2/6N1/8/3Pp3/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'e7e5' => 'rnbqkb1r/pppp2pp/5n2/4p1N1/4p3/3P4/PPP2PPP/RNBQKB1R w KQkq e6 0 5',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/6N1/4p3/8/PPPP1PPP/RNBQKB1R w KQkq - 2 4' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. e4 fxe4 3. Ng5 Nf6

			variation => N__('Reti: Lisitsin: 3. Ng5 Nf6'),
			moves => {
				'd2d3' => 'rnbqkb1r/ppppp1pp/5n2/6N1/4p3/3P4/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/6N1/8/3Pp3/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. e4 fxe4 3. Ng5 Nf6 4. d3 e3

			variation => N__('Reti: Lisitsin: 3. Ng5 Nf6'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/3P4/2N2N2/PPP3PP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3 exf3 5. Nxf3

			variation => N__('Dutch: Staunton Gambit, 4. f3 exf3 5.Nxf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/3P4/2N2p2/PPP3PP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3 exf3

			variation => N__('Dutch: Staunton Gambit, 4. f3 exf3'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppppp1pp/5n2/8/3P4/2N2N2/PPP3PP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/3Pp1P1/2N5/PPP2P1P/R1BQKBNR b KQkq g3 0 4' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. g4

			variation => N__('Dutch: Staunton Gambit, Tartakower Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. f3

			variation => N__('Dutch: Staunton Gambit, 4. f3'),
			moves => {
				'b8c6' => 'r1bqkb1r/ppppp1pp/2n2n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 1 5',
				'd7d5' => 'rnbqkb1r/ppp1p1pp/5n2/3p4/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq d6 0 5',
				'e4f3' => 'rnbqkb1r/ppppp1pp/5n2/8/3P4/2N2p2/PPP3PP/R1BQKBNR w KQkq - 0 5',
				'e7e6' => 'rnbqkb1r/pppp2pp/4pn2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6

			variation => N__('Dutch: Staunton Gambit, 3. Nc3 Nf6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppppp1pp/5n2/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR b KQkq - 3 4',
				'f2f3' => 'rnbqkb1r/ppppp1pp/5n2/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR b KQkq - 0 4',
				'g2g4' => 'rnbqkb1r/ppppp1pp/5n2/8/3Pp1P1/2N5/PPP2P1P/R1BQKBNR b KQkq g3 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/4pP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: Swiss Gambit'),
			moves => {
				'g2g4' => 'rnbqkb1r/ppppp1pp/5n2/8/4pPP1/2N5/PPPP3P/R1BQKBNR b KQkq g3 0 4',
			},
		},
		'rnbqkb1r/ppppp1pp/5n2/8/4pPP1/2N5/PPPP3P/R1BQKBNR b KQkq g3 0 4' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 f5 2. e4 fxe4 3. Nc3 Nf6 4. g4

			variation => N__('Bird: Swiss Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nc3 Nf6 3. g3 g6

			variation => N__('English: Anglo-Dutch'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. Nf3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. Nf3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. f3

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. f3'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Nc3

			variation => N__('Dutch: Leningrad, 4. Nc3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6'),
			moves => {
				'c1g5' => 'rnbqkb1r/ppppp2p/5np1/5pB1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4',
				'f2f3' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N2P2/PP2P1PP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Nf3

			variation => N__('Dutch: Leningrad, 4. Nf3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6

			variation => N__('Dutch: Leningrad Variation'),
			moves => {
				'b1c3' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 1 4',
				'f1g2' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'A86',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. g3 g6 4. Bg2

			variation => N__('Dutch: Leningrad, 4. Bg2'),
			moves => {
				'd7d6' => 'rnbqkb1r/ppp1p2p/3p1np1/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 0 5',
				'f8g7' => 'rnbqk2r/ppppp1bp/5np1/5p2/2PP4/6P1/PP2PPBP/RNBQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6 3. Bg2 g6 4. Nf3

			variation => N__('Dutch: 2. g3 Nf6 3. Bg2 g6 4. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 Nf6 3. Bg2 g6

			variation => N__('Dutch: 2. g3 Nf6 3. Bg2 g6'),
			moves => {
				'g1f3' => 'rnbqkb1r/ppppp2p/5np1/5p2/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkb1r/ppppp2p/5np1/5pB1/2PP4/2N5/PP2PPPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'A85',
                        # TRANSLATORS: 1. d4 f5 2. c4 Nf6 3. Nc3 g6 4. Bg5

			variation => N__('Dutch: 2. c4 Nf6 3. Nc3 g6 4. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/6B1/3Pp2P/2N5/PPP2PP1/R2QKBNR b KQkq h3 0 5' => {
			eco => 'A83e',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 g6 5. h4

			variation => N__('Dutch: Staunton Gambit, Alekhine Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/6B1/3Pp3/2N2P2/PPP3PP/R2QKBNR b KQkq - 0 5' => {
			eco => 'A83f',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 g6 5. f3

			variation => N__('Dutch: Staunton Gambit, Lasker Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/5np1/6B1/3Pp3/2N5/PPP2PPP/R2QKBNR w KQkq - 0 5' => {
			eco => 'A83d',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 Nf6 4. Bg5 g6

			variation => N__('Dutch: Staunton Gambit, 4. Bg5 g6'),
			moves => {
				'f2f3' => 'rnbqkb1r/ppppp2p/5np1/6B1/3Pp3/2N2P2/PPP3PP/R2QKBNR b KQkq - 0 5',
				'h2h4' => 'rnbqkb1r/ppppp2p/5np1/6B1/3Pp2P/2N5/PPP2PP1/R2QKBNR b KQkq h3 0 5',
			},
		},
		'rnbqkb1r/ppppp2p/5ppn/8/2PPP3/8/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B00a',
                        # TRANSLATORS: 1. e4 Nh6 2. d4 g6 3. c4 f6

			variation => N__('King\'s Pawn: Hippopotamus Defence'),
			moves => {
			},
		},
		'rnbqkb1r/ppppp2p/6pn/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 g6 3. Nc3 Nh6

			variation => N__('Dutch: Bladel Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5Bp1/8/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A45k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 g6 3. Bxf6

			variation => N__('Trompowsky 2... g6 3.Bxf6'),
			moves => {
				'e7f6' => 'rnbqkb1r/pppp1p1p/5pp1/8/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/3P4/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'E60c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. d5

			variation => N__('King\'s Indian: 3. d5'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pppp1p/5np1/1p1P4/2P5/8/PP2PPPP/RNBQKBNR w KQkq b6 0 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/4P3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B06d',
                        # TRANSLATORS:

			variation => N__('Modern: Norwegian Defence'),
			moves => {
				'f6h5' => 'rnbqkb1r/pppppp1p/6p1/4P2n/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/6B1/2PP4/8/PP2PPPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'E60b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Bg5

			variation => N__('King\'s Indian: 3. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/6B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A45k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 g6 3. Nc3

			variation => N__('Trompowsky 2... g6 3. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/6B1/3P4/2P5/PP2PPPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'A45e',
                        # TRANSLATORS: 1. d4 Nf6 2. c3 g6 3. Bg5

			variation => N__('Indian: 2. c3 g6 3. Bg5'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A48m',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
				'f6e4' => 'rnbqkb1r/pppppp1p/6p1/6B1/3Pn3/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/6B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A45k',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 g6

			variation => N__('Trompowsky 2... g6'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppp1p/5np1/6B1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3',
				'g5f6' => 'rnbqkb1r/pppppp1p/5Bp1/8/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/1P3P2/5N2/P1PPP1PP/RNBQKB1R b KQkq b3 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 g6 3. b4

			variation => N__('Bird: Batavo Polish Attack'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/1P6/5NP1/P1PPPP1P/RNBQKB1R b KQkq b3 0 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. b4

			variation => N__('Reti: KIA, Reti-Smyslov Variation'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/1P6/5NP1/P1PPPP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. e4

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. e4'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6 3. g3

			variation => N__('English: Anglo-Indian, 2. Nc3 g6 3. g3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 g6

			variation => N__('English: Anglo-Indian, 2. Nc3 g6'),
			moves => {
				'e2e4' => 'rnbqkb1r/pppppp1p/5np1/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3',
				'g2g3' => 'rnbqkb1r/pppppp1p/5np1/8/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. Nf3 g6

			variation => N__('English: Anglo-Indian, 2. Nf3 g6'),
			moves => {
				'g2g3' => 'rnbqkb1r/pppppp1p/5np1/8/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A15',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Indian, 2. Nf3 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. g3 g6

			variation => N__('English: Anglo-Indian, 2. g3 g6'),
			moves => {
				'f1g2' => 'rnbqkb1r/pppppp1p/5np1/8/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A15',
                        # TRANSLATORS:

			variation => N__('English: Anglo-Indian, 2. g3 g6'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'E61a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3

			variation => N__('King\'s Indian: 3. Nc3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4',
				'c7c6' => 'rnbqkb1r/pp1ppp1p/2p2np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'd7d5' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq d6 0 4',
				'd7d6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'E60f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nf3

			variation => N__('King\'s Indian: 3. Nf3'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq d6 0 4',
				'd7d6' => 'rnbqkb1r/ppp1pp1p/3p1np1/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2PP4/5P2/PP2P1PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'E60d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. f3

			variation => N__('King\'s Indian: 3. f3'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/5P2/PP2P1PP/RNBQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'E60e',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. g3

			variation => N__('King\'s Indian: 3. g3'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp1pp1p/5np1/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq d6 0 4',
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/2PP4/6P1/PP2PP1P/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'E60a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6

			variation => N__('King\'s Indian'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppp1p/5np1/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'c1g5' => 'rnbqkb1r/pppppp1p/5np1/6B1/2PP4/8/PP2PPPP/RN1QKBNR b KQkq - 1 3',
				'd1c2' => 'rnbqkb1r/pppppp1p/5np1/8/2PP4/8/PPQ1PPPP/RNB1KBNR b KQkq - 1 3',
				'd4d5' => 'rnbqkb1r/pppppp1p/5np1/3P4/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'f2f3' => 'rnbqkb1r/pppppp1p/5np1/8/2PP4/5P2/PP2P1PP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkb1r/pppppp1p/5np1/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkb1r/pppppp1p/5np1/8/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/2PP4/8/PPQ1PPPP/RNB1KBNR b KQkq - 1 3' => {
			eco => 'E60a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Qc2

			variation => N__('King\'s Indian: Mengarini Attack'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A48f',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bf4

			variation => N__('Neo-King\'s Indian: London System'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A48e',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Nc3

			variation => N__('Neo-King\'s Indian'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A48b',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. c3

			variation => N__('Neo-King\'s Indian'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/3P4/2P2N2/PP2PPPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/2P5/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A45e',
                        # TRANSLATORS: 1. d4 Nf6 2. c3 g6

			variation => N__('Indian: 2. c3 g6'),
			moves => {
				'c1g5' => 'rnbqkb1r/pppppp1p/5np1/6B1/3P4/2P5/PP2PPPP/RN1QKBNR b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/4P3/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A45d',
                        # TRANSLATORS: 1. d4 Nf6 2. e3 g6

			variation => N__('Indian: 2. e3 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A48c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. e3

			variation => N__('Neo-King\'s Indian'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppp1p/5np1/2p5/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq c6 0 4',
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/3P4/4PN2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A48a',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6

			variation => N__('Neo-King\'s Indian'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/2N2N2/PPP1PPPP/R1BQKB1R b KQkq - 1 3',
				'b1d2' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/5N2/PPPNPPPP/R1BQKB1R b KQkq - 1 3',
				'c1f4' => 'rnbqkb1r/pppppp1p/5np1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'c1g5' => 'rnbqkb1r/pppppp1p/5np1/6B1/3P4/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'c2c3' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/5N2/PPPNPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A48a',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Nbd2

			variation => N__('Neo-King\'s Indian'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A49',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. g3

			variation => N__('Neo-King\'s Indian: Fianchetto System'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A45h',
                        # TRANSLATORS: 1. d4 Nf6 2. g3 g6

			variation => N__('Indian: 2. g3 g6'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B06d',
                        # TRANSLATORS: 1. e4 g6 2. d4 Nf6

			variation => N__('Modern: Norwegian Defence'),
			moves => {
				'e4e5' => 'rnbqkb1r/pppppp1p/5np1/4P3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 g6

			variation => N__('Bird: 1..Nf6 2. Nf3 g6'),
			moves => {
				'b2b4' => 'rnbqkb1r/pppppp1p/5np1/8/1P3P2/5N2/P1PPP1PP/RNBQKB1R b KQkq b3 0 3',
				'g2g3' => 'rnbqkb1r/pppppp1p/5np1/8/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3 g6 3. g3

			variation => N__('Bird: 1..Nf6 2. Nf3 g6 3. g3'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/5P2/5NP1/PPPPP2P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/6P1/1P6/PBPPPP1P/RN1QKBNR b KQkq g3 0 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 Nf6 2. Bb2 g6 3. g4

			variation => N__('Nimzowitsch-Larsen: Spike Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 Nf6 2. Bb2 g6

			variation => N__('Nimzowitsch-Larsen: Indian Variation'),
			moves => {
				'g2g4' => 'rnbqkb1r/pppppp1p/5np1/8/6P1/1P6/PBPPPP1P/RN1QKBNR b KQkq g3 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6

			variation => N__('Reti: KIA'),
			moves => {
				'b2b4' => 'rnbqkb1r/pppppp1p/5np1/8/1P6/5NP1/P1PPPP1P/RNBQKB1R b KQkq b3 0 3',
				'f1g2' => 'rnbqkb1r/pppppp1p/5np1/8/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pppppp1p/5np1/8/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3 g6 3. Bg2

			variation => N__('Reti: KIA'),
			moves => {
				'f8g7' => 'rnbqk2r/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppp1p/6p1/4P2n/3P2P1/8/PPP2P1P/RNBQKBNR b KQkq g3 0 4' => {
			eco => 'B06d',
                        # TRANSLATORS:

			variation => N__('Modern: Norwegian Defence'),
			moves => {
				'h5g7' => 'rnbqkb1r/ppppppnp/6p1/4P3/3P2P1/8/PPP2P1P/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pppppp1p/6p1/4P2n/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B06d',
                        # TRANSLATORS:

			variation => N__('Modern: Norwegian Defence'),
			moves => {
				'g2g4' => 'rnbqkb1r/pppppp1p/6p1/4P2n/3P2P1/8/PPP2P1P/RNBQKBNR b KQkq g3 0 4',
			},
		},
		'rnbqkb1r/pppppp1p/6p1/6B1/3Pn3/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4' => {
			eco => 'A48n',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 g6 3. Bg5 Ne4

			variation => N__('Neo-King\'s Indian: Torre Attack'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/6pn/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'B00a',
                        # TRANSLATORS:

			variation => N__('King\'s Pawn'),
			moves => {
				'f7f6' => 'rnbqkb1r/ppppp2p/5ppn/8/2PPP3/8/PP3PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppppp1p/6pn/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B00a',
                        # TRANSLATORS:

			variation => N__('King\'s Pawn'),
			moves => {
				'c2c4' => 'rnbqkb1r/pppppp1p/6pn/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq c3 0 3',
			},
		},
		'rnbqkb1r/pppppp1p/8/6p1/3Pn2B/8/PPP1PPPP/RN1QKBNR w KQkq g6 0 4' => {
			eco => 'A45t',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bh4 g5

			variation => N__('Trompowsky: 2... Ne4 3. Bh4 g5'),
			moves => {
			},
		},
		'rnbqkb1r/pppppp1p/8/6p1/3PnB2/8/PPP1PPPP/RN1QKBNR w KQkq g6 0 4' => {
			eco => 'A45u',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4 g5

			variation => N__('Trompowsky: Borg Variation'),
			moves => {
			},
		},
		'rnbqkb1r/ppppppnp/6p1/4P3/3P2P1/8/PPP2P1P/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B06d',
                        # TRANSLATORS: 1. e4 g6 2. d4 Nf6 3. e5 Nh5 4. g4 Ng7

			variation => N__('Modern: Norwegian Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/1n6/2P1P3/8/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B02s',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4 Nb6 4. c5

			variation => N__('Alekhine: Two Pawns (Lasker) Attack'),
			moves => {
				'b6d5' => 'rnbqkb1r/pppppppp/8/2PnP3/8/8/PP1P1PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkb1r/pppppppp/1n6/4P3/2B5/8/PPPP1PPP/RNBQK1NR w KQkq - 3 4' => {
			eco => 'B02n',
                        # TRANSLATORS:

			variation => N__('Alekhine: 3. Bc4'),
			moves => {
				'c4b3' => 'rnbqkb1r/pppppppp/1n6/4P3/8/1B6/PPPP1PPP/RNBQK1NR b KQkq - 4 4',
			},
		},
		'rnbqkb1r/pppppppp/1n6/4P3/2P5/1P6/P2P1PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B02r',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4 Nb6 4. b3

			variation => N__('Alekhine: Chase, Steiner Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/1n6/4P3/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 1 4' => {
			eco => 'B02r',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4 Nb6

			variation => N__('Alekhine: Chase Variation'),
			moves => {
				'b2b3' => 'rnbqkb1r/pppppppp/1n6/4P3/2P5/1P6/P2P1PPP/RNBQKBNR b KQkq - 0 4',
				'c4c5' => 'rnbqkb1r/pppppppp/1n6/2P1P3/8/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppppppp/1n6/4P3/8/1B6/PPPP1PPP/RNBQK1NR b KQkq - 4 4' => {
			eco => 'B02n',
                        # TRANSLATORS:

			variation => N__('Alekhine: 3. Bc4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/1n6/2p1P3/8/1B6/PPPP1PPP/RNBQK1NR w KQkq c6 0 5',
			},
		},
		'rnbqkb1r/pppppppp/5n2/4P3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B02l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5

			variation => N__('Alekhine: 2. e5'),
			moves => {
				'f6d5' => 'rnbqkb1r/pppppppp/8/3nP3/8/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3',
				'f6e4' => 'rnbqkb1r/pppppppp/8/4P3/4n3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3',
				'f6g8' => 'rnbqkbnr/pppppppp/8/4P3/8/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/6B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 2 2' => {
			eco => 'A45j',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5

			variation => N__('Trompowsky Opening'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/5n2/2p3B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 3',
				'd7d5' => 'rnbqkb1r/ppp1pppp/5n2/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 3',
				'd7d6' => 'rnbqkb1r/ppp1pppp/3p1n2/6B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/6B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'f6e4' => 'rnbqkb1r/pppppppp/8/6B1/3Pn3/8/PPP1PPPP/RN1QKBNR w KQkq - 3 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/6B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/1P6/5N2/P1PPPPPP/RNBQKB1R b KQkq b3 0 2' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. b4

			variation => N__('Reti: Santasiere\'s folly'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/1P6/8/P1PPPPPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4 Nf6

			variation => N__('Polish: 1... Nf6'),
			moves => {
				'c1b2' => 'rnbqkb1r/pppppppp/5n2/8/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 2 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 2 2' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4 Nf6 2. Bb2

			variation => N__('Polish: 1... Nf6 2. Bb2'),
			moves => {
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/8/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/1PP5/8/P2PPPPP/RNBQKBNR b KQkq b3 0 2' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. b4

			variation => N__('English: Anglo-Indian, Polish'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/2B1P3/8/PPPP1PPP/RNBQK1NR b KQkq - 2 2' => {
			eco => 'B02a',
                        # TRANSLATORS: 1. e4 Nf6 2. Bc4

			variation => N__('Alekhine: Krejcik Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 2 2' => {
			eco => 'A16',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3

			variation => N__('English: Anglo-Indian, 2. Nc3'),
			moves => {
				'b8c6' => 'r1bqkb1r/pppppppp/2n2n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 3 3',
				'c7c6' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'd7d5' => 'rnbqkb1r/ppp1pppp/5n2/3p4/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq d6 0 3',
				'd7d6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 2 2' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. Nf3

			variation => N__('English: Anglo-Indian, 2. Nf3'),
			moves => {
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6 2. g3

			variation => N__('English: Anglo-Indian, 2. g3'),
			moves => {
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A15',
                        # TRANSLATORS: 1. c4 Nf6

			variation => N__('English: Anglo-Indian'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppppp/5n2/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 2 2',
				'b2b4' => 'rnbqkb1r/pppppppp/5n2/8/1PP5/8/P2PPPPP/RNBQKBNR b KQkq b3 0 2',
				'g1f3' => 'rnbqkb1r/pppppppp/5n2/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 2 2',
				'g2g3' => 'rnbqkb1r/pppppppp/5n2/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A50a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4

			variation => N__('Indian: 2. c4'),
			moves => {
				'a7a6' => 'rnbqkb1r/1ppppppp/p4n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'b7b6' => 'rnbqkb1r/p1pppppp/1p3n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'b8c6' => 'r1bqkb1r/pppppppp/2n2n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
				'c7c5' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2PP4/8/PP2PPPP/RNBQKBNR w KQkq c6 0 3',
				'c7c6' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'd7d6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3',
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P1B2/8/PPP1PPPP/RN1QKBNR b KQkq - 2 2' => {
			eco => 'A45g',
                        # TRANSLATORS: 1. d4 Nf6 2. Bf4

			variation => N__('Indian: 2. Bf4'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P1P2/8/PPP1P1PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'A45b',
                        # TRANSLATORS: 1. d4 Nf6 2. f4

			variation => N__('Indian: Canard Opening'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P2P1/8/PPP1PP1P/RNBQKBNR b KQkq g3 0 2' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. g4

			variation => N__('Indian: Gibbins Gambit'),
			moves => {
				'e7e5' => 'rnbqkb1r/pppp1ppp/5n2/4p3/3P2P1/8/PPP1PP1P/RNBQKBNR w KQkq e6 0 3',
				'f6g4' => 'rnbqkb1r/pppppppp/8/8/3P2n1/8/PPP1PP1P/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/2N5/PPP1PPPP/R1BQKBNR b KQkq - 2 2' => {
			eco => 'A45f',
                        # TRANSLATORS: 1. d4 Nf6 2. Nc3

			variation => N__('Indian: 2. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/2P5/PP2PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A45e',
                        # TRANSLATORS: 1. d4 Nf6 2. c3

			variation => N__('Indian: 2. c3'),
			moves => {
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/2P5/PP2PPPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 3 4' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. e4 Nxe4 3. Bd3 Nf6 4. Nf3

			variation => N__('Indian: Arafat Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 4' => {
			eco => 'A45a',
                        # TRANSLATORS:

			variation => N__('Indian: Omega Gambit'),
			moves => {
				'g1f3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 3 4',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/4P3/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A45d',
                        # TRANSLATORS: 1. d4 Nf6 2. e3

			variation => N__('Indian: 2. e3'),
			moves => {
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/8/3P4/4P3/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/4P3/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 2' => {
			eco => 'A46a',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3

			variation => N__('Indian: 2. Nf3'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pppppp/5n2/1p6/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq b6 0 3',
				'b7b6' => 'rnbqkb1r/p1pppppp/1p3n2/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'c7c5' => 'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq c6 0 3',
				'd7d6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'e7e6' => 'rnbqkb1r/pppp1ppp/4pn2/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'f6e4' => 'rnbqkb1r/pppppppp/8/8/3Pn3/5N2/PPP1PPPP/RNBQKB1R w KQkq - 3 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/5P2/PPP1P1PP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. f3

			variation => N__('Indian: Paleface Attack'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/5P2/PPP1P1PP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A45h',
                        # TRANSLATORS: 1. d4 Nf6 2. g3

			variation => N__('Indian: 2. g3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/5n2/2p5/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq c6 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6

			variation => N__('Queen\'s Pawn: Indian'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/2N5/PPP1PPPP/R1BQKBNR b KQkq - 2 2',
				'b1d2' => 'rnbqkb1r/pppppppp/5n2/8/3P4/8/PPPNPPPP/R1BQKBNR b KQkq - 2 2',
				'c1f4' => 'rnbqkb1r/pppppppp/5n2/8/3P1B2/8/PPP1PPPP/RN1QKBNR b KQkq - 2 2',
				'c1g5' => 'rnbqkb1r/pppppppp/5n2/6B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 2 2',
				'c2c3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/2P5/PP2PPPP/RNBQKBNR b KQkq - 0 2',
				'c2c4' => 'rnbqkb1r/pppppppp/5n2/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'e2e3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/4P3/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'e2e4' => 'rnbqkb1r/pppppppp/5n2/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2',
				'f2f3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/5P2/PPP1P1PP/RNBQKBNR b KQkq - 0 2',
				'f2f4' => 'rnbqkb1r/pppppppp/5n2/8/3P1P2/8/PPP1P1PP/RNBQKBNR b KQkq f3 0 2',
				'g1f3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 2 2',
				'g2g3' => 'rnbqkb1r/pppppppp/5n2/8/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2',
				'g2g4' => 'rnbqkb1r/pppppppp/5n2/8/3P2P1/8/PPP1PP1P/RNBQKBNR b KQkq g3 0 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3P4/8/PPPNPPPP/R1BQKBNR b KQkq - 2 2' => {
			eco => 'A45c',
                        # TRANSLATORS: 1. d4 Nf6 2. Nd2

			variation => N__('Indian: 2. Nd2'),
			moves => {
				'e7e5' => 'rnbqkb1r/pppp1ppp/5n2/4p3/3P4/8/PPPNPPPP/R1BQKBNR w KQkq e6 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. e4

			variation => N__('Indian: Omega Gambit'),
			moves => {
				'f6e4' => 'rnbqkb1r/pppppppp/8/8/3Pn3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 2 2' => {
			eco => 'B02c',
                        # TRANSLATORS: 1. e4 Nf6 2. Nc3

			variation => N__('Alekhine: Scandinavian Variation'),
			moves => {
				'd7d5' => 'rnbqkb1r/ppp1pppp/5n2/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B02b',
                        # TRANSLATORS: 1. e4 Nf6 2. d3

			variation => N__('Alekhine: Maroczy Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'B02a',
                        # TRANSLATORS: 1. e4 Nf6

			variation => N__('Alekhine Defence'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppppp/5n2/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 2 2',
				'd2d3' => 'rnbqkb1r/pppppppp/5n2/8/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'e4e5' => 'rnbqkb1r/pppppppp/5n2/4P3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2',
				'f1c4' => 'rnbqkb1r/pppppppp/5n2/8/2B1P3/8/PPPP1PPP/RNBQK1NR b KQkq - 2 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/5P2/1P6/P1PPP1PP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. b3

			variation => N__('Bird: 1..Nf6 2. b3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/5P2/4P3/PPPP2PP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. e3

			variation => N__('Bird: 1..Nf6 2. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 2 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. Nf3

			variation => N__('Bird: 1..Nf6 2. Nf3'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/5n2/2p5/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq c6 0 3',
				'd7d6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/5P2/6P1/PPPPP2P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6 2. g3

			variation => N__('Bird: 1..Nf6 2. g3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/5P2/8/PPPPP1PP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 Nf6

			variation => N__('Bird: 1..Nf6'),
			moves => {
				'b2b3' => 'rnbqkb1r/pppppppp/5n2/8/5P2/1P6/P1PPP1PP/RNBQKBNR b KQkq - 0 2',
				'e2e3' => 'rnbqkb1r/pppppppp/5n2/8/5P2/4P3/PPPP2PP/RNBQKBNR b KQkq - 0 2',
				'g1f3' => 'rnbqkb1r/pppppppp/5n2/8/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 2 2',
				'g2g3' => 'rnbqkb1r/pppppppp/5n2/8/5P2/6P1/PPPPP2P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/6P1/2N5/PPPPPP1P/R1BQKBNR b KQkq g3 0 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 Nf6 2. g4

			variation => N__('Van Geet: Tübingen Gambit'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/1P3N2/P1PPPPPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. b3

			variation => N__('Reti: 1... Nf6 2. b3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/1P6/P1PPPPPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 Nf6

			variation => N__('Nimzowitsch-Larsen: Indian Variation'),
			moves => {
				'c1b2' => 'rnbqkb1r/pppppppp/5n2/8/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 2 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 2 2' => {
			eco => 'A01',
                        # TRANSLATORS:

			variation => N__('Nimzowitsch-Larsen: Indian Variation'),
			moves => {
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 3 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 Nf6 2. Nf3

			variation => N__('Van Geet: 1... Nf6 2. Nf3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/2N5/PPPPPPPP/R1BQKBNR w KQkq - 2 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 Nf6

			variation => N__('Van Geet: 1... Nf6'),
			moves => {
				'g1f3' => 'rnbqkb1r/pppppppp/5n2/8/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 3 2',
				'g2g4' => 'rnbqkb1r/pppppppp/5n2/8/6P1/2N5/PPPPPP1P/R1BQKBNR b KQkq g3 0 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/4PN2/PPPP1PPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. e3

			variation => N__('Reti: 1... Nf6 2. e3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 2 2' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6

			variation => N__('Reti: 1... Nf6'),
			moves => {
				'b2b3' => 'rnbqkb1r/pppppppp/5n2/8/8/1P3N2/P1PPPPPP/RNBQKB1R b KQkq - 0 2',
				'b2b4' => 'rnbqkb1r/pppppppp/5n2/8/1P6/5N2/P1PPPPPP/RNBQKB1R b KQkq b3 0 2',
				'e2e3' => 'rnbqkb1r/pppppppp/5n2/8/8/4PN2/PPPP1PPP/RNBQKB1R b KQkq - 0 2',
				'g2g3' => 'rnbqkb1r/pppppppp/5n2/8/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A05',
                        # TRANSLATORS: 1. Nf3 Nf6 2. g3

			variation => N__('Reti: KIA'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pppppp/5n2/1p6/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq b6 0 3',
				'b7b6' => 'rnbqkb1r/p1pppppp/1p3n2/8/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3',
				'c7c5' => 'rnbqkb1r/pp1ppppp/5n2/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq c6 0 3',
				'g7g6' => 'rnbqkb1r/pppppp1p/5np1/8/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/5n2/8/8/6P1/PPPPPP1P/RNBQKBNR w KQkq - 1 2' => {
			eco => 'A00t',
                        # TRANSLATORS: 1. g3 Nf6

			variation => N__('Benko Opening'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/7n/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B00a',
                        # TRANSLATORS:

			variation => N__('King\'s Pawn'),
			moves => {
				'g7g6' => 'rnbqkb1r/pppppp1p/6pn/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkb1r/pppppppp/7n/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 2' => {
			eco => 'B00a',
                        # TRANSLATORS:

			variation => N__('King\'s Pawn'),
			moves => {
				'd2d4' => 'rnbqkb1r/pppppppp/7n/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
			},
		},
		'rnbqkb1r/pppppppp/8/2PnP3/2B5/8/PP1P1PPP/RNBQK1NR b KQkq - 2 5' => {
			eco => 'B02u',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4 Nb6 4. c5 Nd5 5. Bc4

			variation => N__('Alekhine: Two Pawns Attack, 5. Bc4'),
			moves => {
				'e7e6' => 'rnbqkb1r/pppp1ppp/4p3/2PnP3/2B5/8/PP1P1PPP/RNBQK1NR w KQkq - 0 6',
			},
		},
		'rnbqkb1r/pppppppp/8/2PnP3/8/2N5/PP1P1PPP/R1BQKBNR b KQkq - 2 5' => {
			eco => 'B02t',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4 Nb6 4. c5 Nd5 5. Nc3

			variation => N__('Alekhine: Two Pawns Attack, 5. Nc3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/8/2PnP3/8/8/PP1P1PPP/RNBQKBNR w KQkq - 1 5' => {
			eco => 'B02s',
                        # TRANSLATORS:

			variation => N__('Alekhine: Two Pawns (Lasker) Attack'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppppp/8/2PnP3/8/2N5/PP1P1PPP/R1BQKBNR b KQkq - 2 5',
				'f1c4' => 'rnbqkb1r/pppppppp/8/2PnP3/2B5/8/PP1P1PPP/RNBQK1NR b KQkq - 2 5',
			},
		},
		'rnbqkb1r/pppppppp/8/3nP3/2B5/8/PPPP1PPP/RNBQK1NR b KQkq - 2 3' => {
			eco => 'B02n',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. Bc4

			variation => N__('Alekhine: 3. Bc4'),
			moves => {
				'd5b6' => 'rnbqkb1r/pppppppp/1n6/4P3/2B5/8/PPPP1PPP/RNBQK1NR w KQkq - 3 4',
			},
		},
		'rnbqkb1r/pppppppp/8/3nP3/2P5/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'B02r',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. c4

			variation => N__('Alekhine: Chase Variation'),
			moves => {
				'd5b6' => 'rnbqkb1r/pppppppp/1n6/4P3/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkb1r/pppppppp/8/3nP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'B03a',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. d4

			variation => N__('Alekhine: 3. d4'),
			moves => {
				'b7b5' => 'rnbqkb1r/p1pppppp/8/1p1nP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq b6 0 4',
				'd7d6' => 'rnbqkb1r/ppp1pppp/3p4/3nP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppppppp/8/3nP3/8/1P6/P1PP1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B02m',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. b3

			variation => N__('Alekhine: Welling Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/8/3nP3/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 2 3' => {
			eco => 'B02o',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. Nc3

			variation => N__('Alekhine: Sämisch Attack'),
			moves => {
				'd5c3' => 'rnbqkb1r/pppppppp/8/4P3/8/2n5/PPPP1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppppppp/8/3nP3/8/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B02m',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5

			variation => N__('Alekhine: 2. e5 Nd5'),
			moves => {
				'b1c3' => 'rnbqkb1r/pppppppp/8/3nP3/8/2N5/PPPP1PPP/R1BQKBNR b KQkq - 2 3',
				'b2b3' => 'rnbqkb1r/pppppppp/8/3nP3/8/1P6/P1PP1PPP/RNBQKBNR b KQkq - 0 3',
				'c2c4' => 'rnbqkb1r/pppppppp/8/3nP3/2P5/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 3',
				'd2d4' => 'rnbqkb1r/pppppppp/8/3nP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq d3 0 3',
				'f1c4' => 'rnbqkb1r/pppppppp/8/3nP3/2B5/8/PPPP1PPP/RNBQK1NR b KQkq - 2 3',
			},
		},
		'rnbqkb1r/pppppppp/8/4P3/4n3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B02l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Ne4

			variation => N__('Alekhine: Mokele Mbembe (Bücker) Variation'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/8/4P3/8/2P5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B02q',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. Nc3 Nxc3 4. dxc3

			variation => N__('Alekhine: Sämisch Attack, 3...Nxc3 4.dxc3'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/8/4P3/8/2n5/PPPP1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B02p',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Nd5 3. Nc3 Nxc3

			variation => N__('Alekhine: Sämisch Attack, 3...Nxc3'),
			moves => {
				'd2c3' => 'rnbqkb1r/pppppppp/8/4P3/8/2P5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkb1r/pppppppp/8/6B1/3Pn2P/8/PPP1PPP1/RN1QKBNR b KQkq h3 0 3' => {
			eco => 'A45s',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. h4

			variation => N__('Trompowsky: 2... Ne4 3. h4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/8/2p3B1/3Pn2P/8/PPP1PPP1/RN1QKBNR w KQkq c6 0 4',
				'd7d5' => 'rnbqkb1r/ppp1pppp/8/3p2B1/3Pn2P/8/PPP1PPP1/RN1QKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkb1r/pppppppp/8/6B1/3Pn3/8/PPP1PPPP/RN1QKBNR w KQkq - 3 3' => {
			eco => 'A45s',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4

			variation => N__('Trompowsky: 2... Ne4'),
			moves => {
				'g5f4' => 'rnbqkb1r/pppppppp/8/8/3PnB2/8/PPP1PPPP/RN1QKBNR b KQkq - 4 3',
				'g5h4' => 'rnbqkb1r/pppppppp/8/8/3Pn2B/8/PPP1PPPP/RN1QKBNR b KQkq - 4 3',
				'h2h4' => 'rnbqkb1r/pppppppp/8/6B1/3Pn2P/8/PPP1PPP1/RN1QKBNR b KQkq h3 0 3',
			},
		},
		'rnbqkb1r/pppppppp/8/8/3P2n1/8/PPP1PP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A45a',
                        # TRANSLATORS: 1. d4 Nf6 2. g4 Nxg4

			variation => N__('Indian: Gibbins Gambit Accepted'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/8/8/3Pn2B/8/PPP1PPPP/RN1QKBNR b KQkq - 4 3' => {
			eco => 'A45t',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bh4

			variation => N__('Trompowsky: 2... Ne4 3. Bh4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/8/2p5/3Pn2B/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 4',
				'd7d5' => 'rnbqkb1r/ppp1pppp/8/3p4/3Pn2B/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 4',
				'g7g5' => 'rnbqkb1r/pppppp1p/8/6p1/3Pn2B/8/PPP1PPPP/RN1QKBNR w KQkq g6 0 4',
			},
		},
		'rnbqkb1r/pppppppp/8/8/3Pn3/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A45a',
                        # TRANSLATORS:

			variation => N__('Indian: Omega Gambit'),
			moves => {
				'e4f6' => 'rnbqkb1r/pppppppp/5n2/8/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 4',
			},
		},
		'rnbqkb1r/pppppppp/8/8/3Pn3/5N2/PPP1PPPP/RNBQKB1R w KQkq - 3 3' => {
			eco => 'A46a',
                        # TRANSLATORS: 1. d4 Nf6 2. Nf3 Ne4

			variation => N__('Indian: Döry Defence'),
			moves => {
			},
		},
		'rnbqkb1r/pppppppp/8/8/3Pn3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A45a',
                        # TRANSLATORS:

			variation => N__('Indian: Omega Gambit'),
			moves => {
				'f1d3' => 'rnbqkb1r/pppppppp/8/8/3Pn3/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkb1r/pppppppp/8/8/3PnB2/8/PPP1PPPP/RN1QKBNR b KQkq - 4 3' => {
			eco => 'A45u',
                        # TRANSLATORS: 1. d4 Nf6 2. Bg5 Ne4 3. Bf4

			variation => N__('Trompowsky: 2... Ne4 3. Bf4'),
			moves => {
				'c7c5' => 'rnbqkb1r/pp1ppppp/8/2p5/3PnB2/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 4',
				'd7d5' => 'rnbqkb1r/ppp1pppp/8/3p4/3PnB2/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 4',
				'g7g5' => 'rnbqkb1r/pppppp1p/8/6p1/3PnB2/8/PPP1PPPP/RN1QKBNR w KQkq g6 0 4',
			},
		},
		'rnbqkbn1/pppp1p1r/8/4N2p/2B1PppP/8/PPPP2P1/RNBQK2R w KQq - 2 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Long Whip Defence'),
			moves => {
				'd2d4' => 'rnbqkbn1/pppp1p1r/8/4N2p/2BPPppP/8/PPP3P1/RNBQK2R b KQq d3 0 7',
			},
		},
		'rnbqkbn1/pppp1p1r/8/4N2p/2BPPppP/8/PPP3P1/RNBQK2R b KQq d3 0 7' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Long Whip Defence'),
			moves => {
				'f8h6' => 'rnbqk1n1/pppp1p1r/7b/4N2p/2BPPppP/8/PPP3P1/RNBQK2R w KQq - 1 8',
			},
		},
		'rnbqkbnr/1p1p1p1p/p3p1p1/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 0 6' => {
			eco => 'B42b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3 g6

			variation => N__('Sicilian: Kan, Swiss Cheese Variation'),
			moves => {
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B28g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. c4 e6

			variation => N__('Sicilian: O\'Kelly, 3. c4 e6'),
			moves => {
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B28i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. c3 e6

			variation => N__('Sicilian: O\'Kelly, 3. c3 e6'),
			moves => {
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/2PNP3/8/PP3PPP/RNBQKB1R b KQkq c3 0 5' => {
			eco => 'B41f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. c4

			variation => N__('Sicilian: Kan, 5. c4'),
			moves => {
				'd8c7' => 'rnb1kbnr/1pqp1ppp/p3p3/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6',
				'g8f6' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/2PNP3/8/PP3PPP/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'B43a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3

			variation => N__('Sicilian: Kan, 5. Nc3'),
			moves => {
				'b7b5' => 'rnbqkbnr/3p1ppp/p3p3/1p6/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq b6 0 6',
				'd7d6' => 'rnbqkbnr/1p3ppp/p2pp3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6',
				'd8c7' => 'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/3B4/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B42a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Bd3

			variation => N__('Sicilian: Kan, 5. Bd3'),
			moves => {
				'b8c6' => 'r1bqkbnr/1p1p1ppp/p1n1p3/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6',
				'd8b6' => 'rnb1kbnr/1p1p1ppp/pq2p3/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6',
				'd8c7' => 'rnb1kbnr/1pqp1ppp/p3p3/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6',
				'f8c5' => 'rnbqk1nr/1p1p1ppp/p3p3/2b5/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6',
				'g7g6' => 'rnbqkbnr/1p1p1p1p/p3p1p1/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 0 6',
				'g8f6' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/3NP3/3B4/PPP2PPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 1 5' => {
			eco => 'B41c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Be3

			variation => N__('Sicilian: Kan, 5. Be3'),
			moves => {
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/6P1/PPP2P1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'B41b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. g3

			variation => N__('Sicilian: Kan, 5. g3'),
			moves => {
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/8/PPP1BPPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'B41d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Be2

			variation => N__('Sicilian: Kan, 5. Be2'),
			moves => {
				'g8f6' => 'rnbqkb1r/1p1p1ppp/p3pn2/8/3NP3/8/PPP1BPPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B41a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6

			variation => N__('Sicilian: Kan (Paulsen)'),
			moves => {
				'b1c3' => 'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 5',
				'c1e3' => 'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/4B3/PPP2PPP/RN1QKB1R b KQkq - 1 5',
				'c2c4' => 'rnbqkbnr/1p1p1ppp/p3p3/8/2PNP3/8/PP3PPP/RNBQKB1R b KQkq c3 0 5',
				'f1d3' => 'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/3B4/PPP2PPP/RNBQK2R b KQkq - 1 5',
				'f1e2' => 'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/8/PPP1BPPP/RNBQK2R b KQkq - 1 5',
				'g2g3' => 'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/6P1/PPP2P1P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/1p1ppppp/2p5/p7/1P6/8/PBPPPPPP/RN1QKBNR w KQkq a6 0 3' => {
			eco => 'A00p',
                        # TRANSLATORS:

			variation => N__('Polish: 1... c6'),
			moves => {
				'b4b5' => 'rnbqkbnr/1p1ppppp/2p5/pP6/8/8/PBPPPPPP/RN1QKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/1p1ppppp/2p5/pP6/8/8/PBPPPPPP/RN1QKBNR b KQkq - 0 3' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4 c6 2. Bb2 a5 3. b5

			variation => N__('Polish: Schühler Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/1p1ppppp/p7/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3' => {
			eco => 'B28f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. c4

			variation => N__('Sicilian: O\'Kelly, 3. c4'),
			moves => {
				'e7e6' => 'rnbqkbnr/1p1p1ppp/p3p3/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/1p1ppppp/p7/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'B28c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. d4

			variation => N__('Sicilian: O\'Kelly, 3. d4'),
			moves => {
				'c5d4' => 'rnbqkbnr/1p1ppppp/p7/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/1p1ppppp/p7/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'B28b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. Nc3

			variation => N__('Sicilian: O\'Kelly, 3. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/1p1ppppp/p7/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B28h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. c3

			variation => N__('Sicilian: O\'Kelly, 3. c3'),
			moves => {
				'd7d5' => 'rnbqkbnr/1p2pppp/p7/2pp4/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq d6 0 4',
				'e7e6' => 'rnbqkbnr/1p1p1ppp/p3p3/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/1p1ppppp/p7/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'B28a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6

			variation => N__('Sicilian: O\'Kelly Variation'),
			moves => {
				'b1c3' => 'rnbqkbnr/1p1ppppp/p7/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3',
				'c2c3' => 'rnbqkbnr/1p1ppppp/p7/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'c2c4' => 'rnbqkbnr/1p1ppppp/p7/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3',
				'd2d4' => 'rnbqkbnr/1p1ppppp/p7/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
			},
		},
		'rnbqkbnr/1p1ppppp/p7/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B28d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. d4 cxd4 4. Nxd4

			variation => N__('Sicilian: O\'Kelly, 3. d4 cxd4 4.Nxd4'),
			moves => {
				'g8f6' => 'rnbqkb1r/1p1ppppp/p4n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/1p1ppppp/p7/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B28c',
                        # TRANSLATORS:

			variation => N__('Sicilian: O\'Kelly, 3. d4'),
			moves => {
				'f3d4' => 'rnbqkbnr/1p1ppppp/p7/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/1p2pppp/p2p4/8/2PQP3/5N2/PP3PPP/RNB1KB1R b KQkq c3 0 5' => {
			eco => 'B53h',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 a6 5. c4

			variation => N__('Sicilian, Chekhover, 4... a6 5. c4'),
			moves => {
				'b8c6' => 'r1bqkbnr/1p2pppp/p1np4/8/2PQP3/5N2/PP3PPP/RNB1KB1R w KQkq - 1 6',
			},
		},
		'rnbqkbnr/1p2pppp/p2p4/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B54c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 a6

			variation => N__('Sicilian: Open, 2... d6, 4... a6'),
			moves => {
			},
		},
		'rnbqkbnr/1p2pppp/p2p4/8/3QP3/4BN2/PPP2PPP/RN2KB1R b KQkq - 1 5' => {
			eco => 'B53g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 a6 5. Be3

			variation => N__('Sicilian, Chekhover, 4... a6 5. Be3'),
			moves => {
			},
		},
		'rnbqkbnr/1p2pppp/p2p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 0 5' => {
			eco => 'B53f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4 a6

			variation => N__('Sicilian, Chekhover, 4... a6'),
			moves => {
				'c1e3' => 'rnbqkbnr/1p2pppp/p2p4/8/3QP3/4BN2/PPP2PPP/RN2KB1R b KQkq - 1 5',
				'c2c4' => 'rnbqkbnr/1p2pppp/p2p4/8/2PQP3/5N2/PP3PPP/RNB1KB1R b KQkq c3 0 5',
			},
		},
		'rnbqkbnr/1p2pppp/p7/2pp4/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'B28j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 a6 3. c3 d5

			variation => N__('Sicilian: O\'Kelly, 3. c3 d5'),
			moves => {
			},
		},
		'rnbqkbnr/1p3ppp/p2p4/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A44t',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 a6

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 a6'),
			moves => {
				'a2a4' => 'rnbqkbnr/1p3ppp/p2p4/2pPp3/P3P3/2N5/1PP2PPP/R1BQKBNR b KQkq a3 0 5',
			},
		},
		'rnbqkbnr/1p3ppp/p2p4/2pPp3/P3P3/2N5/1PP2PPP/R1BQKBNR b KQkq a3 0 5' => {
			eco => 'A44t',
                        # TRANSLATORS:

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 a6'),
			moves => {
				'f8e7' => 'rnbqk1nr/1p2bppp/p2p4/2pPp3/P3P3/2N5/1PP2PPP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkbnr/1p3ppp/p2pp3/8/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'B43b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 d6

			variation => N__('Sicilian: Kan, 5. Nc3 d6'),
			moves => {
			},
		},
		'rnbqkbnr/1p3ppp/p3p3/2p5/2BP4/4PN2/PP2QPPP/RNB1K2R b KQkq - 1 6' => {
			eco => 'D22f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 e6 5. Bxc4 c5 6. Qe2

			variation => N__('QGA: Alekhine, 4. e3 e6 5.Bxc4 c5 6. Qe2'),
			moves => {
			},
		},
		'rnbqkbnr/1p3ppp/p3p3/2p5/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq c6 0 6' => {
			eco => 'D22e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 e6 5. Bxc4 c5

			variation => N__('QGA: Alekhine, 4. e3 e6 5.Bxc4 c5'),
			moves => {
				'd1e2' => 'rnbqkbnr/1p3ppp/p3p3/2p5/2BP4/4PN2/PP2QPPP/RNB1K2R b KQkq - 1 6',
			},
		},
		'rnbqkbnr/1p3ppp/p3p3/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C07e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. Ngf3 a6

			variation => N__('French: Tarrasch, Open, 4.Ngf3 a6'),
			moves => {
			},
		},
		'rnbqkbnr/1p3ppp/p7/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'C08h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 a6

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 a6'),
			moves => {
				'f1e2' => 'rnbqkbnr/1p3ppp/p7/2pp4/3P4/5N2/PPPNBPPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqkbnr/1p3ppp/p7/2pp4/3P4/5N2/PPPNBPPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'C08i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3 a6 6. Be2

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3 a6 6. Be2'),
			moves => {
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/2p5/N4N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A09p',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. Na3 a6

			variation => N__('Reti: Accepted, 3. Na3 a6'),
			moves => {
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D20k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 a6

			variation => N__('QGA: 3. Nc3 a6'),
			moves => {
				'a2a4' => 'rnbqkbnr/1pp1pppp/p7/8/P1pP4/2N5/1P2PPPP/R1BQKBNR b KQkq a3 0 4',
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D22a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3

			variation => N__('QGA: Alekhine, 4. e3'),
			moves => {
				'b7b5' => 'rnbqkbnr/2p1pppp/p7/1p6/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq b6 0 5',
				'c8g4' => 'rn1qkbnr/1pp1pppp/p7/8/2pP2b1/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
				'e7e6' => 'rnbqkbnr/1pp2ppp/p3p3/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D21t',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6

			variation => N__('QGA: Alekhine Defence'),
			moves => {
				'a2a4' => 'rnbqkbnr/1pp1pppp/p7/8/P1pP4/5N2/1P2PPPP/RNBQKB1R b KQkq a3 0 4',
				'e2e3' => 'rnbqkbnr/1pp1pppp/p7/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'e2e4' => 'rnbqkbnr/1pp1pppp/p7/8/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 4',
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 4' => {
			eco => 'D21u',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e4

			variation => N__('QGA: Alekhine, Borisenko-Furman Variation'),
			moves => {
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/P1pP4/2N5/1P2PPPP/R1BQKBNR b KQkq a3 0 4' => {
			eco => 'D20k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 a6 4. a4

			variation => N__('QGA: 3. Nc3 a6 4. a4'),
			moves => {
			},
		},
		'rnbqkbnr/1pp1pppp/p7/8/P1pP4/5N2/1P2PPPP/RNBQKB1R b KQkq a3 0 4' => {
			eco => 'D21v',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. a4

			variation => N__('QGA: Alekhine, 4. a4'),
			moves => {
			},
		},
		'rnbqkbnr/1pp2ppp/p3p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D31b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 a6

			variation => N__('QGD: Janowski Variation'),
			moves => {
			},
		},
		'rnbqkbnr/1pp2ppp/p3p3/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C03e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 a6 4. Ngf3

			variation => N__('French: Tarrasch, 3... a6 4.Ngf3'),
			moves => {
			},
		},
		'rnbqkbnr/1pp2ppp/p3p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C03d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 a6

			variation => N__('French: Tarrasch, 3... a6'),
			moves => {
				'g1f3' => 'rnbqkbnr/1pp2ppp/p3p3/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/1pp2ppp/p3p3/8/2BP4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D22d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 e6 5. Bxc4

			variation => N__('QGA: Alekhine, 4. e3 e6 5.Bxc4'),
			moves => {
				'c7c5' => 'rnbqkbnr/1p3ppp/p3p3/2p5/2BP4/4PN2/PP3PPP/RNBQK2R w KQkq c6 0 6',
			},
		},
		'rnbqkbnr/1pp2ppp/p3p3/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D22c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 e6

			variation => N__('QGA: Alekhine, 4. e3 e6'),
			moves => {
				'f1c4' => 'rnbqkbnr/1pp2ppp/p3p3/8/2BP4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/1ppp1ppp/p3p3/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'C00w',
                        # TRANSLATORS: 1. e4 e6 2. d4 a6

			variation => N__('French: St. George'),
			moves => {
			},
		},
		'rnbqkbnr/1pppppp1/p6p/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B00c',
                        # TRANSLATORS: 1. e4 a6 2. d4 h6

			variation => N__('Basman\'s Creepy-Crawly System (as Black)'),
			moves => {
			},
		},
		'rnbqkbnr/1ppppppp/p7/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B00c',
                        # TRANSLATORS: 1. e4 a6 2. d4

			variation => N__('St. George Defence'),
			moves => {
				'h7h6' => 'rnbqkbnr/1pppppp1/p6p/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/1ppppppp/p7/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'B00c',
                        # TRANSLATORS: 1. e4 a6

			variation => N__('St. George Defence'),
			moves => {
				'd2d4' => 'rnbqkbnr/1ppppppp/p7/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
			},
		},
		'rnbqkbnr/2p1pppp/p7/1p6/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq b6 0 5' => {
			eco => 'D22b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 a6 4. e3 b5

			variation => N__('QGA: Alekhine, Haberditz Variation'),
			moves => {
			},
		},
		'rnbqkbnr/3p1ppp/p3p3/1p6/3NP3/2N5/PPP2PPP/R1BQKB1R w KQkq b6 0 6' => {
			eco => 'B43c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 b5

			variation => N__('Sicilian: Kan, 5. Nc3 b5'),
			moves => {
				'f1d3' => 'rnbqkbnr/3p1ppp/p3p3/1p6/3NP3/2NB4/PPP2PPP/R1BQK2R b KQkq - 1 6',
			},
		},
		'rnbqkbnr/3p1ppp/p3p3/1p6/3NP3/2NB4/PPP2PPP/R1BQK2R b KQkq - 1 6' => {
			eco => 'B43d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 a6 5. Nc3 b5 6. Bd3

			variation => N__('Sicilian: Kan, 5. Nc3 b5 6. Bd3'),
			moves => {
				'd8b6' => 'rnb1kbnr/3p1ppp/pq2p3/1p6/3NP3/2NB4/PPP2PPP/R1BQK2R w KQkq - 2 7',
			},
		},
		'rnbqkbnr/p1p1pppp/8/1p6/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq b6 0 4' => {
			eco => 'D21a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 b5

			variation => N__('QGA: Ericson Variation'),
			moves => {
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D31a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 b6

			variation => N__('QGD: Alapin Variation'),
			moves => {
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C03c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 b6

			variation => N__('French: Tarrasch, 3... b6'),
			moves => {
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3pP1B1/3P2Q1/2N5/PPP2PPP/R3KBNR b KQkq - 3 6' => {
			eco => 'C16d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 b6 5. Qg4 Bf8 6. Bg5

			variation => N__('French: Winawer, 4. e5 b6 5. Qg4 Bf8 6. Bg5'),
			moves => {
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P2Q1/2N5/PPP2PPP/R1B1KBNR w KQkq - 2 6' => {
			eco => 'C16c',
                        # TRANSLATORS:

			variation => N__('French: Winawer, 4. e5 b6 5. Qg4'),
			moves => {
				'c1g5' => 'rnbqkbnr/p1p2ppp/1p2p3/3pP1B1/3P2Q1/2N5/PPP2PPP/R3KBNR b KQkq - 3 6',
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'C02b',
                        # TRANSLATORS:

			variation => N__('French: Advance, 3... b6'),
			moves => {
				'd8d7' => 'rnb1kbnr/p1pq1ppp/1p2p3/3pP3/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C02b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 b6

			variation => N__('French: Advance, 3... b6'),
			moves => {
				'c2c3' => 'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P4/P1N5/1PP2PPP/R1BQKBNR w KQkq - 1 6' => {
			eco => 'C16e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 Bb4 4. e5 b6 5. a3 Bf8

			variation => N__('French: Winawer, 4. e5 b6 5. a3 Bf8'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nc3 e6 3. e4

			variation => N__('English: 1... b6 2. Nc3 e6 3. e4'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nc3 e6

			variation => N__('English: 1... b6 2. Nc3 e6'),
			moves => {
				'e2e4' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/2P1P3/2N5/PP1P1PPP/R1BQKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A40h',
                        # TRANSLATORS: 1. d4 b6 2. c4 e6 3. Nc3

			variation => N__('Queen\'s Pawn: English Defence, 3. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A40e',
                        # TRANSLATORS: 1. d4 b6 2. c4 e6

			variation => N__('Queen\'s Pawn: English Defence, 2. c4 e6'),
			moves => {
				'a2a3' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/2PP4/P7/1P2PPPP/RNBQKBNR b KQkq - 0 3',
				'b1c3' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'e2e4' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/2PP4/P7/1P2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A40f',
                        # TRANSLATORS: 1. d4 b6 2. c4 e6 3. a3

			variation => N__('Queen\'s Pawn: English Defence, 3. a3'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A40g',
                        # TRANSLATORS: 1. d4 b6 2. c4 e6 3. e4

			variation => N__('Queen\'s Pawn: English Defence, 3. e4'),
			moves => {
				'c8b7' => 'rn1qkbnr/pbpp1ppp/1p2p3/8/2PPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/p1pp1ppp/1p2p3/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B00e',
                        # TRANSLATORS: 1. e4 b6 2. d4 e6

			variation => N__('Owen Defence: French'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp1ppp/8/1p2p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq b6 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 b5

			variation => N__('Bishop\'s Opening: Anderssen Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp1ppp/8/1p6/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq b6 0 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 b5

			variation => N__('KGA: Bishop\'s Gambit, Bryan Countergambit'),
			moves => {
			},
		},
		'rnbqkbnr/p1pp2pp/8/1p2pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq b6 0 4' => {
			eco => 'C40o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 b5

			variation => N__('Latvian Gambit: Strautins Variation'),
			moves => {
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nc3

			variation => N__('English: 1... b6 2. Nc3'),
			moves => {
				'c8b7' => 'rn1qkbnr/pbpppppp/1p6/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
				'e7e6' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6 2. Nf3

			variation => N__('English: 1... b6 2. Nf3'),
			moves => {
				'c8b7' => 'rn1qkbnr/pbpppppp/1p6/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b6

			variation => N__('English: 1... b6'),
			moves => {
				'b1c3' => 'rnbqkbnr/p1pppppp/1p6/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/p1pppppp/1p6/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A40d',
                        # TRANSLATORS: 1. d4 b6 2. c4

			variation => N__('Queen\'s Pawn: English Defence, 2. c4'),
			moves => {
				'c8b7' => 'rn1qkbnr/pbpppppp/1p6/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
				'e7e6' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A40d',
                        # TRANSLATORS: 1. d4 b6

			variation => N__('Queen\'s Pawn: English Defence'),
			moves => {
				'c2c4' => 'rnbqkbnr/p1pppppp/1p6/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B00d',
                        # TRANSLATORS: 1. e4 b6 2. d4

			variation => N__('Owen Defence'),
			moves => {
				'c8b7' => 'rn1qkbnr/pbpppppp/1p6/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
				'e7e6' => 'rnbqkbnr/p1pp1ppp/1p2p3/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'B00d',
                        # TRANSLATORS: 1. e4 b6

			variation => N__('Owen Defence'),
			moves => {
				'd2d4' => 'rnbqkbnr/p1pppppp/1p6/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/8/1P6/P1PPPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 b6

			variation => N__('Nimzowitsch-Larsen: Symmetrical'),
			moves => {
			},
		},
		'rnbqkbnr/p1pppppp/1p6/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 b6

			variation => N__('Reti: 1... b6'),
			moves => {
			},
		},
		'rnbqkbnr/p1pppppp/8/1p6/2P5/8/PP1PPPPP/RNBQKBNR w KQkq b6 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 b5

			variation => N__('English: Jänisch Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/p1pppppp/8/1p6/3P4/8/PPP1PPPP/RNBQKBNR w KQkq b6 0 2' => {
			eco => 'A40c',
                        # TRANSLATORS: 1. d4 b5

			variation => N__('Queen\'s Pawn: Polish Defence'),
			moves => {
				'e2e4' => 'rnbqkbnr/p1pppppp/8/1p6/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2',
			},
		},
		'rnbqkbnr/p1pppppp/8/1p6/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'A40c',
                        # TRANSLATORS: 1. d4 b5 2. e4

			variation => N__('Queen\'s Pawn: Polish Defence'),
			moves => {
				'c8b7' => 'rn1qkbnr/pbpppppp/8/1p6/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/p1pppppp/8/1p6/8/1P6/P1PPPPPP/RNBQKBNR w KQkq b6 0 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 b5

			variation => N__('Nimzowitsch-Larsen: Polish Variation'),
			moves => {
			},
		},
		'rnbqkbnr/p1pppppp/8/1p6/8/5N2/PPPPPPPP/RNBQKB1R w KQkq b6 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 b5

			variation => N__('Reti: 1... b5'),
			moves => {
			},
		},
		'rnbqkbnr/p2p1ppp/8/1pp5/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq b6 0 5' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Nf3 c5 4. Bc4 b5

			variation => N__('Centre Game: Kieseritsky Variation'),
			moves => {
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2P5/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A43b',
                        # TRANSLATORS: 1. d4 c5 2. dxc5 b6

			variation => N__('Old Benoni: Cormorant Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 b6 3. Nf3

			variation => N__('English: Symmetrical, 2... b6'),
			moves => {
				'c8b7' => 'rn1qkbnr/pb1ppppp/1p6/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 b6

			variation => N__('English: Symmetrical, 2... b6'),
			moves => {
				'g1f3' => 'rnbqkbnr/p2ppppp/1p6/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A30b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 b6

			variation => N__('English: Symmetrical, 2. Nf3 b6'),
			moves => {
				'g2g3' => 'rnbqkbnr/p2ppppp/1p6/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A30b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 b6 3. g3

			variation => N__('English: Symmetrical, 2. Nf3 b6 3. g3'),
			moves => {
				'c8b7' => 'rn1qkbnr/pb1ppppp/1p6/2p5/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'B27d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 b6

			variation => N__('Sicilian: Katalimov Variation'),
			moves => {
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 b6

			variation => N__('Reti: 1... c5'),
			moves => {
				'f1g2' => 'rnbqkbnr/p2ppppp/1p6/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/p2ppppp/1p6/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: 1... c5'),
			moves => {
				'c8b7' => 'rn1qkbnr/pb1ppppp/1p6/2p5/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/p2ppppp/2p5/1p6/2PP4/8/PP2PPPP/RNBQKBNR w KQkq b6 0 3' => {
			eco => 'A40b',
                        # TRANSLATORS: 1. d4 c6 2. c4 b5

			variation => N__('Queen\'s Pawn: Jadoul'),
			moves => {
			},
		},
		'rnbqkbnr/p2ppppp/8/1ppP4/8/8/PPP1PPPP/RNBQKBNR w KQkq b6 0 3' => {
			eco => 'A43e',
                        # TRANSLATORS: 1. d4 c5 2. d5 b5

			variation => N__('Old Benoni: 2. d5 b5'),
			moves => {
			},
		},
		'rnbqkbnr/p3pppp/2p5/1p1p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq b6 0 4' => {
			eco => 'B15b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 b5

			variation => N__('Caro-Kann: Gurgenidze Counterattack'),
			moves => {
			},
		},
		'rnbqkbnr/p3pppp/2p5/1p6/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq b6 0 5' => {
			eco => 'D11f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4 4. e3 b5

			variation => N__('Slav: 3. Nf3 dxc4 4. e3 b5'),
			moves => {
				'a2a4' => 'rnbqkbnr/p3pppp/2p5/1p6/P1pP4/4PN2/1P3PPP/RNBQKB1R b KQkq a3 0 5',
			},
		},
		'rnbqkbnr/p3pppp/2p5/1p6/2pPP3/2N5/PP3PPP/R1BQKBNR w KQkq b6 0 5' => {
			eco => 'D10q',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 dxc4 4. e4 b5

			variation => N__('Slav: 3. Nc3 dxc4 4. e4 b5'),
			moves => {
				'a2a4' => 'rnbqkbnr/p3pppp/2p5/1p6/P1pPP3/2N5/1P3PPP/R1BQKBNR b KQkq a3 0 5',
			},
		},
		'rnbqkbnr/p3pppp/2p5/1p6/P1pP4/4PN2/1P3PPP/RNBQKB1R b KQkq a3 0 5' => {
			eco => 'D11f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4 4. e3 b5 5. a4

			variation => N__('Slav: 3. Nf3 dxc4 4. e3 b5 5. a4'),
			moves => {
				'e7e6' => 'rnbqkbnr/p4ppp/2p1p3/1p6/P1pP4/4PN2/1P3PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/p3pppp/2p5/1p6/P1pPP3/2N5/1P3PPP/R1BQKBNR b KQkq a3 0 5' => {
			eco => 'D10r',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 dxc4 4. e4 b5 5. a4

			variation => N__('Slav: 3. Nc3 dxc4 4. e4 b5 5. a4'),
			moves => {
			},
		},
		'rnbqkbnr/p4ppp/2p1p3/1p6/P1pP4/4PN2/1P3PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'D11f',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4 4. e3 b5 5. a4 e6

			variation => N__('Slav: 3. Nf3 dxc4 4. e3 b5 5. a4 e6'),
			moves => {
			},
		},
		'rnbqkbnr/p4ppp/8/1pP5/N2p4/8/PP2PPPP/R1BQKBNR w KQkq b6 0 7' => {
			eco => 'D32k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. dxc5 d4 6. Na4 b5

			variation => N__('QGD Tarrasch: 5.dxc5, Tarrasch Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR w KQkq - 0 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Bb5 c6

			variation => N__('Open Game: Portuguese, 2... c6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/4p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq - 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 c6

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'd2d4' => 'rnbqkbnr/pp1p1ppp/2p5/4p3/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/4p3/2BPP3/8/PPP2PPP/RNBQK1NR b KQkq d3 0 3' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp3ppp/2p5/3pp3/2BPP3/8/PPP2PPP/RNBQK1NR w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/4p3/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'B10h',
                        # TRANSLATORS: 1. e4 c6 2. c4 e5

			variation => N__('Caro-Kann: English Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 c6

			variation => N__('English: King\'s, 2. g3 c6'),
			moves => {
				'd2d4' => 'rnbqkbnr/pp1p1ppp/2p5/4p3/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq d3 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/4p3/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 c6 3. d4

			variation => N__('English: King\'s, 2. g3 c6 3. d4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/2p5/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 0 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 c6

			variation => N__('KGA: Bishop\'s Gambit, Ruy Lopez Defence'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4P3/2p5/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A44b',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. dxe6

			variation => N__('Old Benoni: Czech, 3.dxe6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p1P3/2P5/2N2N2/PP1P1PPP/R1BQKB1R b KQkq - 2 5' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5 Ng8 5. Nf3

			variation => N__('English: Mikenas, 5. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p1P3/2P5/2N2N2/PP1P1PPP/R1BQKB1R w KQkq - 3 6',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p1P3/2P5/2N5/PP1P1PPP/R1BQKBNR w KQkq - 1 5' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5 Ng8

			variation => N__('English: Mikenas, Sicilian, 4. e5 Ng8'),
			moves => {
				'd2d4' => 'rnbqkbnr/pp1p1ppp/4p3/2p1P3/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq d3 0 5',
				'g1f3' => 'rnbqkbnr/pp1p1ppp/4p3/2p1P3/2P5/2N2N2/PP1P1PPP/R1BQKB1R b KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p1P3/2PP4/2N5/PP3PPP/R1BQKBNR b KQkq d3 0 5' => {
			eco => 'A19',
                        # TRANSLATORS: 1. c4 Nf6 2. Nc3 e6 3. e4 c5 4. e5 Ng8 5. d4

			variation => N__('English: Mikenas, Nei Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3' => {
			eco => 'B40b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. c4

			variation => N__('Sicilian: 2... e6 3. c4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 e6

			variation => N__('Sicilian: English, 2... e6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A40n',
                        # TRANSLATORS: 1. d4 e6 2. Nf3 c5

			variation => N__('Queen\'s Pawn: 1... e6 2. Nf3 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'B22d',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. d4

			variation => N__('Sicilian: Alapin, 2... e6 3. d4'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/2P5/PP3PPP/RNBQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'B40i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4

			variation => N__('Sicilian: 2... e6 3. d4'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp1p1ppp/4p3/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'd7d5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/3PP3/8/PPP2PPP/RNBQKBNR w KQkq c6 0 3' => {
			eco => 'C00w',
                        # TRANSLATORS: 1. e4 e6 2. d4 c5

			variation => N__('French: Franco-Benoni'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B40c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. b3

			variation => N__('Sicilian: 2... e6 3. b3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b3 e6

			variation => N__('Sicilian: Snyder, 2... e6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'B40g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. Nc3

			variation => N__('Sicilian: 2... e6 3. Nc3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1p1ppp/2n1p3/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'B23i',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 e6 3. g3

			variation => N__('Sicilian: Closed'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'B23f',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 e6

			variation => N__('Sicilian: Closed'),
			moves => {
				'f2f4' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
				'g1e2' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'B23h',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 e6 3. Nge2

			variation => N__('Sicilian: Closed'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'B22c',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. Nf3

			variation => N__('Sicilian: Alapin, 2... e6 3. Nf3'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B22c',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6

			variation => N__('Sicilian: Alapin, 2... e6'),
			moves => {
				'd2d4' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 3',
				'g1f3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B40f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d3

			variation => N__('Sicilian: 2... e6 3. d3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/3P2P1/PPP2P1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. d3 e6 3. g3

			variation => N__('Sicilian: 2. d3 e6 3. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. d3 e6

			variation => N__('Sicilian: 2. d3 e6'),
			moves => {
				'g2g3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/3P2P1/PPP2P1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'B40a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6

			variation => N__('Sicilian: 2... e6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3',
				'b2b3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R b KQkq - 0 3',
				'c2c4' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3',
				'd2d3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'd2d4' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'g2g3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/5N2/PPPPQPPP/RNB1KB1R b KQkq - 1 3' => {
			eco => 'C00i',
                        # TRANSLATORS: 1. e4 e6 2. Qe2 c5 3. Nf3

			variation => N__('French: Chigorin, 2... c5 3. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B40e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. g3

			variation => N__('Sicilian: 2... e6 3. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/8/PPPPQPPP/RNB1KBNR w KQkq c6 0 3' => {
			eco => 'C00h',
                        # TRANSLATORS: 1. e4 e6 2. Qe2 c5

			variation => N__('French: Chigorin, 2... c5'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/5N2/PPPPQPPP/RNB1KB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'B23g',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 e6 3. f4

			variation => N__('Sicilian: Closed'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'B21d',
                        # TRANSLATORS: 1. e4 c5 2. f4 e6 3. Nf3

			variation => N__('Sicilian: Grand Prix, 2... e6 3. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B21d',
                        # TRANSLATORS: 1. e4 c5 2. f4 e6

			variation => N__('Sicilian: Grand Prix, 2... e6'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2pP4/2P5/8/PP2PPPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'A43h',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'e6d5' => 'rnbqkbnr/pp1p1ppp/8/2pp4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2pP4/4P3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A43g',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. e4

			variation => N__('Old Benoni: Franco-Benoni'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/2pP4/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A43f',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6

			variation => N__('Old Benoni: 2. d5 e6'),
			moves => {
				'c2c4' => 'rnbqkbnr/pp1p1ppp/4p3/2pP4/2P5/8/PP2PPPP/RNBQKBNR b KQkq c3 0 3',
				'e2e4' => 'rnbqkbnr/pp1p1ppp/4p3/2pP4/4P3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B40l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p1p1ppp/p3p3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'b8c6' => 'r1bqkbnr/pp1p1ppp/2n1p3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'd7d6' => 'rnbqkbnr/pp3ppp/3pp3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'd8b6' => 'rnb1kbnr/pp1p1ppp/1q2p3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'f8c5' => 'rnbqk1nr/pp1p1ppp/4p3/2b5/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/pp1p1ppp/4pn2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B40k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4

			variation => N__('Sicilian: Open, 2... e6'),
			moves => {
				'f3d4' => 'rnbqkbnr/pp1p1ppp/4p3/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/8/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'B21v',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 e6 5. Nf3

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 e6 5. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/4p3/8/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B21u',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 e6

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 e6'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp1p1ppp/4p3/8/4P3/2N2N2/PP3PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2p1p3/1P6/8/PBPPPPPP/RN1QKBNR w KQkq c6 0 3' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 c5

			variation => N__('Polish: Wolfertz Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2p1p3/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'B22a',
                        # TRANSLATORS: 1. e4 c5 2. c3 e5

			variation => N__('Sicilian: Alapin, 2... e5'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2p1p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq c6 0 3' => {
			eco => 'C30b',
                        # TRANSLATORS: 1. e4 e5 2. f4 c5

			variation => N__('King\'s Gambit: Mafia Defence'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2p5/2BpP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C21',
                        # TRANSLATORS:

			variation => N__('Centre Game: 3. Nf3'),
			moves => {
				'b7b5' => 'rnbqkbnr/p2p1ppp/8/1pp5/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq b6 0 5',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2p5/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'C21',
                        # TRANSLATORS:

			variation => N__('Centre Game: 3. Nf3'),
			moves => {
				'f1c4' => 'rnbqkbnr/pp1p1ppp/8/2p5/2BpP3/5N2/PPP2PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2pP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'A43h',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4 exd5 4. cxd5

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'd7d6' => 'rnbqkbnr/pp3ppp/3p4/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2pPp3/2P5/8/PP2PPPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'A44c',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4

			variation => N__('Old Benoni: Czech, 3. c4'),
			moves => {
				'd7d6' => 'rnbqkbnr/pp3ppp/3p4/2pPp3/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2pPp3/4P3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A44l',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4

			variation => N__('Old Benoni: Czech, 3. e4'),
			moves => {
				'd7d6' => 'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2pPp3/8/8/PPP1PPPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'A44a',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5

			variation => N__('Old Benoni: Czech'),
			moves => {
				'c2c4' => 'rnbqkbnr/pp1p1ppp/8/2pPp3/2P5/8/PP2PPPP/RNBQKBNR b KQkq c3 0 3',
				'd5e6' => 'rnbqkbnr/pp1p1ppp/4P3/2p5/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e4' => 'rnbqkbnr/pp1p1ppp/8/2pPp3/4P3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/2pp4/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A43h',
                        # TRANSLATORS:

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp1p1ppp/8/2pP4/8/8/PP2PPPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1p1ppp/8/4p3/3pP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B21n',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. Nf3 e5 4. c3

			variation => N__('Sicilian: Smith-Morra, Andreaschek Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1p1ppp/8/4p3/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq e6 0 4' => {
			eco => 'B21n',
                        # TRANSLATORS:

			variation => N__('Sicilian: Smith-Morra, Morphy Gambit'),
			moves => {
				'c2c3' => 'rnbqkbnr/pp1p1ppp/8/4p3/3pP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1pp1pp/2p5/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A80d',
                        # TRANSLATORS: 1. d4 f5 2. Bg5 c6

			variation => N__('Dutch: 2. Bg5 c6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1pp1pp/8/2pP1p2/2P5/8/PP2PPPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'A43j',
                        # TRANSLATORS: 1. d4 c5 2. d5 f5 3. c4

			variation => N__('Old Benoni: Clarendon Court, 3. c4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1pp1pp/8/2pP1p2/8/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A43j',
                        # TRANSLATORS: 1. d4 c5 2. d5 f5 3. Nc3

			variation => N__('Old Benoni: Clarendon Court, 3. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1pp1pp/8/2pP1p2/8/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A43j',
                        # TRANSLATORS: 1. d4 c5 2. d5 f5 3. g3

			variation => N__('Old Benoni: Clarendon Court, 3. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1pp1pp/8/2pP1p2/8/8/PPP1PPPP/RNBQKBNR w KQkq f6 0 3' => {
			eco => 'A43j',
                        # TRANSLATORS: 1. d4 c5 2. d5 f5

			variation => N__('Old Benoni: Clarendon Court Defence'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp1pp1pp/8/2pP1p2/8/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 3',
				'c2c4' => 'rnbqkbnr/pp1pp1pp/8/2pP1p2/2P5/8/PP2PPPP/RNBQKBNR b KQkq c3 0 3',
				'g2g3' => 'rnbqkbnr/pp1pp1pp/8/2pP1p2/8/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppp1p/2p3p1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B06d',
                        # TRANSLATORS: 1. e4 g6 2. d4 c6

			variation => N__('Modern'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3' => {
			eco => 'B27f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. c4

			variation => N__('Sicilian: Hungarian, 3. c4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 g6 3. Nf3

			variation => N__('English: Symmetrical, 2... g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 g6 3. g3

			variation => N__('English: Symmetrical, 2... g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp1pppbp/6p1/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3 g6

			variation => N__('English: Symmetrical, 2... g6'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A30b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3 g6

			variation => N__('English: Symmetrical, 2. Nf3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A30a',
                        # TRANSLATORS: 1. c4 c5 2. g3 g6

			variation => N__('English: Symmetrical, 2. g3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'B22a',
                        # TRANSLATORS: 1. e4 c5 2. c3 g6 3. d4

			variation => N__('Sicilian: Alapin, 2... g6 3. d4'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp1ppp1p/6p1/8/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'B27j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4

			variation => N__('Sicilian: Hungarian, 3. d4'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp1ppp1p/6p1/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'f8g7' => 'rnbqk1nr/pp1pppbp/6p1/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'B23b',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 g6

			variation => N__('Sicilian: Closed'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B27g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. c3

			variation => N__('Sicilian: Hungarian, 3. c3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp1pppbp/6p1/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B22a',
                        # TRANSLATORS: 1. e4 c5 2. c3 g6

			variation => N__('Sicilian: Alapin, 2... g6'),
			moves => {
				'd2d4' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/3PP3/2P5/PP3PPP/RNBQKBNR b KQkq d3 0 3',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'B27e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6

			variation => N__('Sicilian: Hungarian Variation'),
			moves => {
				'c2c3' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'c2c4' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/2P1P3/5N2/PP1P1PPP/RNBQKB1R b KQkq c3 0 3',
				'd2d4' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/6P1/PPPP1P1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. g3 g6

			variation => N__('Sicilian: 2. g3'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/6P1/PPPP1PBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/6P1/PPPP1PBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'B20',
                        # TRANSLATORS:

			variation => N__('Sicilian: 2. g3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp1pppbp/6p1/2p5/4P3/6P1/PPPP1PBP/RNBQK1NR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B21b',
                        # TRANSLATORS: 1. e4 c5 2. f4 g6

			variation => N__('Sicilian: Grand Prix, 2... g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3 g6

			variation => N__('Reti: 1... c5'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/2p5/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: 1... c5'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp1pppbp/6p1/2p5/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B27l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 cxd4 4. Nxd4

			variation => N__('Sicilian: Hungarian, 3. d4 cxd4 4.Nxd4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/8/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B22a',
                        # TRANSLATORS: 1. e4 c5 2. c3 g6 3. d4 cxd4

			variation => N__('Sicilian: Alapin, 2... g6 3. d4 cxd4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppp1p/6p1/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B27k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 g6 3. d4 cxd4

			variation => N__('Sicilian: Hungarian, 3. d4 cxd4'),
			moves => {
				'f3d4' => 'rnbqkbnr/pp1ppp1p/6p1/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/1P6/8/P1PPPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4 c6

			variation => N__('Polish: 1... c6'),
			moves => {
				'c1b2' => 'rnbqkbnr/pp1ppppp/2p5/8/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A00p',
                        # TRANSLATORS:

			variation => N__('Polish: 1... c6'),
			moves => {
				'a7a5' => 'rnbqkbnr/1p1ppppp/2p5/p7/1P6/8/PBPPPPPP/RN1QKBNR w KQkq a6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'B10g',
                        # TRANSLATORS: 1. e4 c6 2. c4

			variation => N__('Caro-Kann: English Variation'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp2pppp/2p5/3p4/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq d6 0 3',
				'e7e5' => 'rnbqkbnr/pp1p1ppp/2p5/4p3/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq e6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nc3

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp2pppp/2p5/3p4/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp2pppp/2p5/3p4/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq d6 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. g3

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp1ppppp/2p2n2/8/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp1ppppp/2p5/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/pp1ppppp/2p5/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/pp1ppppp/2p5/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A40b',
                        # TRANSLATORS: 1. d4 c6 2. c4

			variation => N__('Queen\'s Pawn: 1... c6 2. c4'),
			moves => {
				'b7b5' => 'rnbqkbnr/p2ppppp/2p5/1p6/2PP4/8/PP2PPPP/RNBQKBNR w KQkq b6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A40b',
                        # TRANSLATORS: 1. d4 c6 2. Nf3

			variation => N__('Queen\'s Pawn: 1... c6 2. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A40b',
                        # TRANSLATORS: 1. d4 c6

			variation => N__('Queen\'s Pawn: 1... c6'),
			moves => {
				'c2c4' => 'rnbqkbnr/pp1ppppp/2p5/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'g1f3' => 'rnbqkbnr/pp1ppppp/2p5/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B12a',
                        # TRANSLATORS: 1. e4 c6 2. d4

			variation => N__('Caro-Kann: 2. d4'),
			moves => {
				'b8a6' => 'r1bqkbnr/pp1ppppp/n1p5/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
				'd7d5' => 'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/2p2n2/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'B10p',
                        # TRANSLATORS: 1. e4 c6 2. Nc3

			variation => N__('Caro-Kann: 2. Nc3'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp2pppp/2p5/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B10b',
                        # TRANSLATORS: 1. e4 c6 2. d3

			variation => N__('Caro-Kann: Breyer Variation'),
			moves => {
				'd7d5' => 'rnbqkbnr/pp2pppp/2p5/3p4/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'B10o',
                        # TRANSLATORS: 1. e4 c6 2. Nf3

			variation => N__('Caro-Kann: 2. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'B10a',
                        # TRANSLATORS: 1. e4 c6

			variation => N__('Caro-Kann'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp1ppppp/2p5/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2',
				'c2c4' => 'rnbqkbnr/pp1ppppp/2p5/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2',
				'd2d3' => 'rnbqkbnr/pp1ppppp/2p5/8/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'd2d4' => 'rnbqkbnr/pp1ppppp/2p5/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'f2f4' => 'rnbqkbnr/pp1ppppp/2p5/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2',
				'g1e2' => 'rnbqkbnr/pp1ppppp/2p5/8/4P3/8/PPPPNPPP/RNBQKB1R b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/pp1ppppp/2p5/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/4P3/8/PPPPNPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'B10a',
                        # TRANSLATORS: 1. e4 c6 2. Ne2

			variation => N__('Caro-Kann: 2. Ne2'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/2p5/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'B10a',
                        # TRANSLATORS: 1. e4 c6 2. f4

			variation => N__('Caro-Kann: 2. f4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2P5/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A43b',
                        # TRANSLATORS: 1. d4 c5 2. dxc5

			variation => N__('Old Benoni: 2.dxc5'),
			moves => {
				'b7b6' => 'rnbqkbnr/p2ppppp/1p6/2P5/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/1P1P4/8/P1P1PPPP/RNBQKBNR b KQkq b3 0 2' => {
			eco => 'A43a',
                        # TRANSLATORS: 1. d4 c5 2. b4

			variation => N__('Old Benoni: Nakamura Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/1P2P3/8/P1PP1PPP/RNBQKBNR b KQkq b3 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b4

			variation => N__('Sicilian: Wing Gambit'),
			moves => {
				'c5b4' => 'rnbqkbnr/pp1ppppp/8/8/1p2P3/8/P1PP1PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/1P6/8/P1PPPPPP/RNBQKBNR w KQkq c6 0 2' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4 c5

			variation => N__('Polish: Birmingham Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2B1P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. Bc4

			variation => N__('Sicilian: 2. Bc4'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4

			variation => N__('Sicilian: English (2. c4)'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq - 1 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2P5/1P6/P2PPPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A30a',
                        # TRANSLATORS: 1. c4 c5 2. b3

			variation => N__('English: Symmetrical, 2. b3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A34',
                        # TRANSLATORS: 1. c4 c5 2. Nc3

			variation => N__('English: Symmetrical'),
			moves => {
				'b7b6' => 'rnbqkbnr/p2ppppp/1p6/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A30b',
                        # TRANSLATORS: 1. c4 c5 2. Nf3

			variation => N__('English: Symmetrical, 2. Nf3'),
			moves => {
				'b7b6' => 'rnbqkbnr/p2ppppp/1p6/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/5n2/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A30a',
                        # TRANSLATORS: 1. c4 c5 2. g3

			variation => N__('English: Symmetrical, 2. g3'),
			moves => {
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/2P5/8/PP1PPPPP/RNBQKBNR w KQkq c6 0 2' => {
			eco => 'A30a',
                        # TRANSLATORS: 1. c4 c5

			variation => N__('English: Symmetrical'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp1ppppp/8/2p5/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'b2b3' => 'rnbqkbnr/pp1ppppp/8/2p5/2P5/1P6/P2PPPPP/RNBQKBNR b KQkq - 0 2',
				'g1f3' => 'rnbqkbnr/pp1ppppp/8/2p5/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/pp1ppppp/8/2p5/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/3P4/2P5/PP2PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A43c',
                        # TRANSLATORS: 1. d4 c5 2. c3

			variation => N__('Old Benoni: 2. c3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/3P4/4P3/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A43d',
                        # TRANSLATORS: 1. d4 c5 2. e3

			variation => N__('Old Benoni: 2. e3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/3P4/8/PPP1PPPP/RNBQKBNR w KQkq c6 0 2' => {
			eco => 'A43a',
                        # TRANSLATORS: 1. d4 c5

			variation => N__('Old Benoni'),
			moves => {
				'b2b4' => 'rnbqkbnr/pp1ppppp/8/2p5/1P1P4/8/P1P1PPPP/RNBQKBNR b KQkq b3 0 2',
				'c2c3' => 'rnbqkbnr/pp1ppppp/8/2p5/3P4/2P5/PP2PPPP/RNBQKBNR b KQkq - 0 2',
				'd4c5' => 'rnbqkbnr/pp1ppppp/8/2P5/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2',
				'd4d5' => 'rnbqkbnr/pp1ppppp/8/2pP4/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2',
				'e2e3' => 'rnbqkbnr/pp1ppppp/8/2p5/3P4/4P3/PPP2PPP/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B21m',
                        # TRANSLATORS: 1. e4 c5 2. d4

			variation => N__('Sicilian: Smith-Morra Gambit'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp1ppppp/8/8/3pP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'd7d5' => 'rnbqkbnr/pp2pppp/8/2pp4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b3

			variation => N__('Sicilian: Snyder'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR w KQkq - 1 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'B23a',
                        # TRANSLATORS: 1. e4 c5 2. Nc3

			variation => N__('Sicilian: Closed'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B22a',
                        # TRANSLATORS: 1. e4 c5 2. c3

			variation => N__('Sicilian: Alapin'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 1 3',
				'd7d5' => 'rnbqkbnr/pp2pppp/8/2pp4/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq d6 0 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/pp1p1ppp/8/2p1p3/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq e6 0 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. d3

			variation => N__('Sicilian: 2. d3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq - 1 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'B27a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3

			variation => N__('Sicilian: 2. Nf3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p1ppppp/p7/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'b7b6' => 'rnbqkbnr/p2ppppp/1p6/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'd8a5' => 'rnb1kbnr/pp1ppppp/8/q1p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'd8c7' => 'rnb1kbnr/ppqppppp/8/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/5n2/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/6P1/PPPP1P1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. g3

			variation => N__('Sicilian: 2. g3'),
			moves => {
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4P3/6P1/PPPP1P1P/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/8/PPPP1PPP/RNBQKBNR w KQkq c6 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5

			variation => N__('Sicilian Defence'),
			moves => {
				'a2a3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/P7/1PPP1PPP/RNBQKBNR b KQkq - 0 2',
				'b1c3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2',
				'b2b3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR b KQkq - 0 2',
				'b2b4' => 'rnbqkbnr/pp1ppppp/8/2p5/1P2P3/8/P1PP1PPP/RNBQKBNR b KQkq b3 0 2',
				'c2c3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 2',
				'c2c4' => 'rnbqkbnr/pp1ppppp/8/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2',
				'd2d3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'd2d4' => 'rnbqkbnr/pp1ppppp/8/2p5/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'f1c4' => 'rnbqkbnr/pp1ppppp/8/2p5/2B1P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2',
				'f2f4' => 'rnbqkbnr/pp1ppppp/8/2p5/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2',
				'g1e2' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/8/PPPPNPPP/RNBQKB1R b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/6P1/PPPP1P1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/8/PPPPNPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. Ne2

			variation => N__('Sicilian: Keres Variation (2. Ne2)'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4P3/P7/1PPP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. a3

			variation => N__('Sicilian: Mengarini'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'B21a',
                        # TRANSLATORS: 1. e4 c5 2. f4

			variation => N__('Sicilian: Grand Prix Attack'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
				'd7d5' => 'rnbqkbnr/pp2pppp/8/2pp4/4PP2/8/PPPP2PP/RNBQKBNR w KQkq d6 0 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: 1..c5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/5P2/8/PPPPP1PP/RNBQKBNR w KQkq c6 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 c5

			variation => N__('Bird: 1..c5'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp1ppppp/8/2p5/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/8/1P3N2/P1PPPPPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. b3

			variation => N__('Reti: 1... c5, Nimzowitsch-Larsen'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/8/1P6/P1PPPPPP/RNBQKBNR w KQkq c6 0 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 c5

			variation => N__('Nimzowitsch-Larsen: English Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 1 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 c5 2. Nf3

			variation => N__('Van Geet: Sicilian Variation, 2. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/8/2N2N2/PPPPPPPP/R1BQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/8/2N5/PPPPPPPP/R1BQKBNR w KQkq c6 0 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 c5

			variation => N__('Van Geet: Sicilian Variation'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp1ppppp/8/2p5/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/8/5N2/PPPPPPPP/RNBQKB1R w KQkq c6 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5

			variation => N__('Reti: 1... c5'),
			moves => {
				'b2b3' => 'rnbqkbnr/pp1ppppp/8/2p5/8/1P3N2/P1PPPPPP/RNBQKB1R b KQkq - 0 2',
				'g2g3' => 'rnbqkbnr/pp1ppppp/8/2p5/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2p5/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 c5 2. g3

			variation => N__('Reti: 1... c5'),
			moves => {
				'b7b6' => 'rnbqkbnr/p2ppppp/1p6/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3',
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3',
				'g7g6' => 'rnbqkbnr/pp1ppp1p/6p1/2p5/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/2pP4/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A43e',
                        # TRANSLATORS: 1. d4 c5 2. d5

			variation => N__('Old Benoni: 2. d5'),
			moves => {
				'b7b5' => 'rnbqkbnr/p2ppppp/8/1ppP4/8/8/PPP1PPPP/RNBQKBNR w KQkq b6 0 3',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/2pP4/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/pp1p1ppp/8/2pPp3/8/8/PPP1PPPP/RNBQKBNR w KQkq e6 0 3',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/2pP4/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3',
				'f7f5' => 'rnbqkbnr/pp1pp1pp/8/2pP1p2/8/8/PPP1PPPP/RNBQKBNR w KQkq f6 0 3',
				'g8f6' => 'rnbqkb1r/pp1ppppp/5n2/2pP4/8/8/PPP1PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/1p2P3/8/P1PP1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS:

			variation => N__('Sicilian: Wing Gambit'),
			moves => {
				'a2a3' => 'rnbqkbnr/pp1ppppp/8/8/1p2P3/P7/2PP1PPP/RNBQKBNR b KQkq - 0 3',
				'c2c4' => 'rnbqkbnr/pp1ppppp/8/8/1pP1P3/8/P2P1PPP/RNBQKBNR b KQkq c3 0 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/1p2P3/P7/2PP1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b4 cxb4 3. a3

			variation => N__('Sicilian: Wing Gambit, Marshall Variation'),
			moves => {
				'b4a3' => 'rnbqkbnr/pp1ppppp/8/8/4P3/p7/2PP1PPP/RNBQKBNR w KQkq - 0 4',
				'd7d5' => 'rnbqkbnr/pp2pppp/8/3p4/1p2P3/P7/2PP1PPP/RNBQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/1pP1P3/8/P2P1PPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b4 cxb4 3. c4

			variation => N__('Sicilian: Wing Gambit, Santasiere Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/3pP3/2P5/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B21o',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3

			variation => N__('Sicilian: Smith-Morra, 3. c3'),
			moves => {
				'd4c3' => 'rnbqkbnr/pp1ppppp/8/8/4P3/2p5/PP3PPP/RNBQKBNR w KQkq - 0 4',
				'd4d3' => 'rnbqkbnr/pp1ppppp/8/8/4P3/2Pp4/PP3PPP/RNBQKBNR w KQkq - 0 4',
				'd7d5' => 'rnbqkbnr/pp2pppp/8/3p4/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq d6 0 4',
				'g8f6' => 'rnbqkb1r/pp1ppppp/5n2/8/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/3pP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'B21n',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. Nf3

			variation => N__('Sicilian: Smith-Morra, Morphy Gambit'),
			moves => {
				'e7e5' => 'rnbqkbnr/pp1p1ppp/8/4p3/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq e6 0 4',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/3pP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B21m',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4

			variation => N__('Sicilian: Smith-Morra, 2...cxd4'),
			moves => {
				'c2c3' => 'rnbqkbnr/pp1ppppp/8/8/3pP3/2P5/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'f2f4' => 'rnbqkbnr/pp1ppppp/8/8/3pPP2/8/PPP3PP/RNBQKBNR b KQkq f3 0 3',
				'g1f3' => 'rnbqkbnr/pp1ppppp/8/8/3pP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/3pPP2/8/PPP3PP/RNBQKBNR b KQkq f3 0 3' => {
			eco => 'B21m',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. f4

			variation => N__('Sicilian: Halasz Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/4P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B21s',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3

			variation => N__('Sicilian: Smith-Morra Accepted'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp1ppppp/2n5/8/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5',
				'd7d6' => 'rnbqkbnr/pp2pppp/3p4/8/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5',
				'e7e6' => 'rnbqkbnr/pp1p1ppp/4p3/8/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/4P3/2Pp4/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B21r',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 d3

			variation => N__('Sicilian: Smith-Morra, 3. c3 d3'),
			moves => {
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/4P3/2p5/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B21s',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3

			variation => N__('Sicilian: Smith-Morra Accepted'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp1ppppp/8/8/4P3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp1ppppp/8/8/4P3/p7/2PP1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b4 cxb4 3. a3 bxa3

			variation => N__('Sicilian: Wing Gambit, Carlsbad Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 4' => {
			eco => 'B15d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Be3

			variation => N__('Caro-Kann: Gurgenidze: 4. Be3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N1B3/PPP2PPP/R2QKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B15j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. Nf3

			variation => N__('Caro-Kann: Gurgenidze: 4. Nf3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N4P/PPP2PP1/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B15e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. h3

			variation => N__('Caro-Kann: Gurgenidze: 4. h3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/2N4P/PPP2PP1/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B15c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6

			variation => N__('Caro-Kann: Gurgenidze/Modern'),
			moves => {
				'c1e3' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N1B3/PPP2PPP/R2QKBNR b KQkq - 1 4',
				'e4e5' => 'rnbqkbnr/pp2pp1p/2p3p1/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4',
				'h2h3' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N4P/PPP2PP1/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B12h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 g6 4. Ngf3

			variation => N__('Caro-Kann: Gurgenidze/Modern: 3. Nd2 g6 4.Ngf3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/2p3p1/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B12h',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 g6

			variation => N__('Caro-Kann: Gurgenidze/Modern: 3. Nd2 g6'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B10d',
                        # TRANSLATORS: 1. e4 c6 2. d3 d5 3. Nd2 g6

			variation => N__('Caro-Kann: Breyer, 3. Nd2 g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pp1p/2p3p1/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B15f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 g6 4. e5

			variation => N__('Caro-Kann: Gurgenidze: 4. e5'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/2p3p1/3pP3/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pp1p/3p2p1/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A43t',
                        # TRANSLATORS:

			variation => N__('Old Benoni: Schmidt, 3. e4 g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/3p2p1/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pp1p/3p2p1/2pP4/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A43t',
                        # TRANSLATORS:

			variation => N__('Old Benoni: Schmidt, 3. e4 g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp2ppbp/3p2p1/2pP4/4P3/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pp1p/3p2p1/2pP4/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A43t',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4 g6

			variation => N__('Old Benoni: Schmidt, 3. e4 g6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pp1p/3p2p1/2pP4/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4',
				'g1f3' => 'rnbqkbnr/pp2pp1p/3p2p1/2pP4/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pp1p/3p2p1/2pP4/8/2N5/PPP1PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. Nc3 g6

			variation => N__('Old Benoni: Schmidt, 3. Nc3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pp1p/6p1/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 1 4' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp2pp1p/2n3p1/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5',
				'f8g7' => 'rnbqk1nr/pp2ppbp/6p1/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 w kq - 2 5',
			},
		},
		'rnbqkbnr/pp2pp1p/6p1/2pp4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4' => {
			eco => 'A08',
                        # TRANSLATORS:

			variation => N__('Reti: KIA, 2... c5'),
			moves => {
				'e1g1' => 'rnbqkbnr/pp2pp1p/6p1/2pp4/8/5NP1/PPPPPPBP/RNBQ1RK1 b kq - 1 4',
			},
		},
		'rnbqkbnr/pp2ppp1/2p4p/3p4/3P3B/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4' => {
			eco => 'D00h',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 h6 3. Bh4 c6 4. e3

			variation => N__('Hodgson Attack: 2... h6 3. Bh4 c6 4. e3'),
			moves => {
				'd8b6' => 'rnb1kbnr/pp2ppp1/1qp4p/3p4/3P3B/4P3/PPP2PPP/RN1QKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2ppp1/2p4p/3p4/3P3B/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4' => {
			eco => 'D00h',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 h6 3. Bh4 c6

			variation => N__('Hodgson Attack: 2... h6 3. Bh4 c6'),
			moves => {
				'e2e3' => 'rnbqkbnr/pp2ppp1/2p4p/3p4/3P3B/4P3/PPP2PPP/RN1QKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3P4/2P5/8/PP1P1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B10i',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5

			variation => N__('Caro-Kann: English Variation'),
			moves => {
				'c6d5' => 'rnbqkbnr/pp2pppp/8/3p4/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D10b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5

			variation => N__('Slav: Exchange'),
			moves => {
				'c6d5' => 'rnbqkbnr/pp2pppp/8/3p4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B13a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5

			variation => N__('Caro-Kann: Exchange Variation'),
			moves => {
				'c6d5' => 'rnbqkbnr/pp2pppp/8/3p4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
				'd8d5' => 'rnb1kbnr/pp2pppp/2p5/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3P4/4P3/8/PP1P1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B10i',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. cxd5

			variation => N__('Caro-Kann: English Variation'),
			moves => {
				'c6d5' => 'rnbqkbnr/pp2pppp/8/3p4/4P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'D00g',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 c6

			variation => N__('Hodgson Attack: 2... c6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p2P1/8/8/PPPPPPBP/RNBQK1NR b KQkq - 0 3' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2 c6 3. g5

			variation => N__('Grob Gambit: Spike Attack'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B10i',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5

			variation => N__('Caro-Kann: English Variation'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp2pppp/2p5/3P4/4P3/8/PP1P1PPP/RNBQKBNR b KQkq - 0 3',
				'e4d5' => 'rnbqkbnr/pp2pppp/2p5/3P4/2P5/8/PP1P1PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A12',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. b3

			variation => N__('English: Caro-Kann Defence, 3. b3'),
			moves => {
				'c8g4' => 'rn1qkbnr/pp2pppp/2p5/3p4/2P3b1/1P3N2/P2PPPPP/RNBQKB1R w KQkq - 1 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nc3 d5

			variation => N__('English: Caro-Kann Defence'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2P5/4PN2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. e3

			variation => N__('English: Caro-Kann Defence, 3. e3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2P5/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq d6 0 3' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5

			variation => N__('English: Caro-Kann Defence'),
			moves => {
				'b2b3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2P5/4PN2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A11',
                        # TRANSLATORS: 1. c4 c6 2. Nf3 d5 3. g3

			variation => N__('English: Caro-Kann Defence, 3. g3'),
			moves => {
				'c8g4' => 'rn1qkbnr/pp2pppp/2p5/3p4/2P3b1/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PP1B2/8/PP2PPPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'D10a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Bf4

			variation => N__('Slav: 3. Bf4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'D10i',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3

			variation => N__('Slav: 3. Nc3'),
			moves => {
				'd5c4' => 'rnbqkbnr/pp2pppp/2p5/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'e7e5' => 'rnbqkbnr/pp3ppp/2p5/3pp3/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D10h',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. e3

			variation => N__('Slav: 3. e3'),
			moves => {
				'c8f5' => 'rn1qkbnr/pp2pppp/2p5/3p1b2/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'D11a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3

			variation => N__('Slav: 3. Nf3'),
			moves => {
				'c8f5' => 'rn1qkbnr/pp2pppp/2p5/3p1b2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'c8g4' => 'rn1qkbnr/pp2pppp/2p5/3p4/2PP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'd5c4' => 'rnbqkbnr/pp2pppp/2p5/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D10a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. g3

			variation => N__('Slav: 3. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'D10a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6

			variation => N__('Slav Defence'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'c1f4' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PP1B2/8/PP2PPPP/RN1QKBNR b KQkq - 1 3',
				'c4d5' => 'rnbqkbnr/pp2pppp/2p5/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'e2e4' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'D10a',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. e4

			variation => N__('Diemer-Duhm Gambit (DDG) vs. Slav/Caro-Kann'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'D02c',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c6 3. Bf4

			variation => N__('Queen\'s Pawn: London'),
			moves => {
				'd8b6' => 'rnb1kbnr/pp2pppp/1qp5/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'D02b',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c6

			variation => N__('Queen\'s Pawn: 2. Nf3 c6'),
			moves => {
				'c1f4' => 'rnbqkbnr/pp2pppp/2p5/3p4/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B15a',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3

			variation => N__('Caro-Kann: 3. Nc3'),
			moves => {
				'b7b5' => 'rnbqkbnr/p3pppp/2p5/1p1p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq b6 0 4',
				'd5e4' => 'rnbqkbnr/pp2pppp/2p5/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'g7g6' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/4B3/PPP2PPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'B12c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Be3

			variation => N__('Caro-Kann: Mieses Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'B12b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nf3

			variation => N__('Caro-Kann: Ulysses Gambit'),
			moves => {
				'd5e4' => 'rnbqkbnr/pp2pppp/2p5/8/3Pp3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B12d',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. f3

			variation => N__('Caro-Kann: Maroczy (Fantasy) Variation'),
			moves => {
				'd5e4' => 'rnbqkbnr/pp2pppp/2p5/8/3Pp3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 4',
				'e7e6' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/3PP3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B12b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5

			variation => N__('Caro-Kann: 2. d4 d5'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3',
				'b1d2' => 'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/8/PPPN1PPP/R1BQKBNR b KQkq - 1 3',
				'c1e3' => 'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/4B3/PPP2PPP/RN1QKBNR b KQkq - 1 3',
				'e4d5' => 'rnbqkbnr/pp2pppp/2p5/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
				'e4e5' => 'rnbqkbnr/pp2pppp/2p5/3pP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
				'f2f3' => 'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/3PP3/8/PPPN1PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B12g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2

			variation => N__('Caro-Kann: 3. Nd2'),
			moves => {
				'd5e4' => 'rnbqkbnr/pp2pppp/2p5/8/3Pp3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
				'd8b6' => 'rnb1kbnr/pp2pppp/1qp5/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4',
				'g7g6' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'B10s',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3

			variation => N__('Caro-Kann: Two Knights Variation'),
			moves => {
				'c8g4' => 'rn1qkbnr/pp2pppp/2p5/3p4/4P1b1/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 2 4',
				'd5e4' => 'rnbqkbnr/pp2pppp/2p5/8/4p3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/4P3/2N2Q2/PPPP1PPP/R1B1KBNR b KQkq - 1 3' => {
			eco => 'B10r',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Qf3

			variation => N__('Caro-Kann: Goldman/Spielmann'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'B10q',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5

			variation => N__('Caro-Kann: 2. Nc3 d5'),
			moves => {
				'd1f3' => 'rnbqkbnr/pp2pppp/2p5/3p4/4P3/2N2Q2/PPPP1PPP/R1B1KBNR b KQkq - 1 3',
				'g1f3' => 'rnbqkbnr/pp2pppp/2p5/3p4/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B10b',
                        # TRANSLATORS: 1. e4 c6 2. d3 d5

			variation => N__('Caro-Kann: Breyer Variation'),
			moves => {
				'b1d2' => 'rnbqkbnr/pp2pppp/2p5/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B10c',
                        # TRANSLATORS: 1. e4 c6 2. d3 d5 3. Nd2

			variation => N__('Caro-Kann: Breyer, 3. Nd2'),
			moves => {
				'e7e5' => 'rnbqkbnr/pp3ppp/2p5/3pp3/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq e6 0 4',
				'g7g6' => 'rnbqkbnr/pp2pp1p/2p3p1/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/6P1/8/PPPPPPBP/RNBQK1NR w KQkq - 0 3' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2 c6

			variation => N__('Grob Gambit: 2... c6'),
			moves => {
				'g4g5' => 'rnbqkbnr/pp2pppp/2p5/3p2P1/8/8/PPPPPPBP/RNBQK1NR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c6

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp2pppp/2p5/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A07',
                        # TRANSLATORS:

			variation => N__('Reti: KIA'),
			moves => {
				'c8g4' => 'rn1qkbnr/pp2pppp/2p5/3p4/6b1/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3p4/8/6P1/PPPPPPBP/RNBQK1NR w KQkq - 0 3' => {
			eco => 'A00v',
                        # TRANSLATORS: 1. g3 d5 2. Bg2 c6

			variation => N__('Benko Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/3pP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B12i',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5

			variation => N__('Caro-Kann: Advance Variation'),
			moves => {
				'c6c5' => 'rnbqkbnr/pp2pppp/8/2ppP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
				'c8f5' => 'rn1qkbnr/pp2pppp/2p5/3pPb2/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/6N1/3Pp3/8/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'B12b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nf3 dxe4 4. Ng5

			variation => N__('Caro-Kann: Ulysses Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/6N1/4p3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B10t',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 dxe4 4. Ng5

			variation => N__('Caro-Kann: Hector Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2BPp3/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'B15p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Bc4

			variation => N__('Caro-Kann: von Hennig Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2pP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D10o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 dxc4 4. e3

			variation => N__('Slav: 3. Nc3 dxc4 4. e3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D11e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4 4. Nc3

			variation => N__('Slav: 3. Nf3 dxc4 4. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D10o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 dxc4

			variation => N__('Slav: 3. Nc3 dxc4'),
			moves => {
				'a2a4' => 'rnbqkbnr/pp2pppp/2p5/8/P1pP4/2N5/1P2PPPP/R1BQKBNR b KQkq a3 0 4',
				'e2e3' => 'rnbqkbnr/pp2pppp/2p5/8/2pP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'e2e4' => 'rnbqkbnr/pp2pppp/2p5/8/2pPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D11e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4 4. e3

			variation => N__('Slav: 3. Nf3 dxc4 4. e3'),
			moves => {
				'b7b5' => 'rnbqkbnr/p3pppp/2p5/1p6/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq b6 0 5',
				'c8e6' => 'rn1qkbnr/pp2pppp/2p1b3/8/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D11e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nf3 dxc4

			variation => N__('Slav: 3. Nf3 dxc4'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/2p5/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'e2e3' => 'rnbqkbnr/pp2pppp/2p5/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/2pPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'D10p',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 dxc4 4. e4

			variation => N__('Slav: 3. Nc3 dxc4 4. e4'),
			moves => {
				'b7b5' => 'rnbqkbnr/p3pppp/2p5/1p6/2pPP3/2N5/PP3PPP/R1BQKBNR w KQkq b6 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3PN3/8/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B15q',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. Nxe4

			variation => N__('Caro-Kann: 4.Nxe4'),
			moves => {
				'b8d7' => 'r1bqkbnr/pp1npppp/2p5/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
				'c8f5' => 'rn1qkbnr/pp2pppp/2p5/5b2/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3PP3/8/PPP3PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B12f',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Maroczy (Fantasy), 3...dxe4'),
			moves => {
				'e7e5' => 'rnbqkbnr/pp3ppp/2p5/4p3/3PP3/8/PPP3PP/RNBQKBNR w KQkq e6 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'B15p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4 4. f3

			variation => N__('Caro-Kann: Rasa-Studier Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B15p',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nc3 dxe4

			variation => N__('Caro-Kann: 3. Nc3 dxe4'),
			moves => {
				'c3e4' => 'rnbqkbnr/pp2pppp/2p5/8/3PN3/8/PPP2PPP/R1BQKBNR b KQkq - 0 4',
				'f1c4' => 'rnbqkbnr/pp2pppp/2p5/8/2BPp3/2N5/PPP2PPP/R1BQK1NR b KQkq - 1 4',
				'f2f3' => 'rnbqkbnr/pp2pppp/2p5/8/3Pp3/2N2P2/PPP3PP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3Pp3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B12b',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Ulysses Gambit'),
			moves => {
				'f3g5' => 'rnbqkbnr/pp2pppp/2p5/6N1/3Pp3/8/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3Pp3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B12f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. f3 dxe4

			variation => N__('Caro-Kann: Maroczy (Fantasy), 3...dxe4'),
			moves => {
				'f3e4' => 'rnbqkbnr/pp2pppp/2p5/8/3PP3/8/PPP3PP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/3Pp3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'B12g',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. Nd2 dxe4

			variation => N__('Caro-Kann: 3. Nd2 dxe4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/4N3/5N2/PPPP1PPP/R1BQKB1R b KQkq - 0 4' => {
			eco => 'B10u',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 dxe4 4. Nxe4

			variation => N__('Caro-Kann: Two Knights, 3...dxe4 4.Nxe4'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/2p2n2/8/4N3/5N2/PPPP1PPP/R1BQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/4p3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'B10t',
                        # TRANSLATORS: 1. e4 c6 2. Nc3 d5 3. Nf3 dxe4

			variation => N__('Caro-Kann: Two Knights, 3...dxe4'),
			moves => {
				'c3e4' => 'rnbqkbnr/pp2pppp/2p5/8/4N3/5N2/PPPP1PPP/R1BQKB1R b KQkq - 0 4',
				'f3g5' => 'rnbqkbnr/pp2pppp/2p5/6N1/4p3/2N5/PPPP1PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/2p5/8/P1pP4/2N5/1P2PPPP/R1BQKBNR b KQkq a3 0 4' => {
			eco => 'D10o',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 dxc4 4. a4

			variation => N__('Slav: 3. Nc3 dxc4 4. a4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/2pp4/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 c6

			variation => N__('Pirc: 2. d4 c6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3' => {
			eco => 'B51a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bb5+

			variation => N__('Sicilian: 3. Bb5+'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp2pppp/2np4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4',
				'b8d7' => 'r1bqkbnr/pp1npppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4',
				'c8d7' => 'rn1qkbnr/pp1bpppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/1P2P3/5N2/P1PP1PPP/RNBQKB1R b KQkq b3 0 3' => {
			eco => 'B50b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. b4

			variation => N__('Sicilian: Wing Gambit deferred'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3' => {
			eco => 'B50g',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Bc4

			variation => N__('Sicilian: 2. Nf3 d6 3. Bc4'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. c4 d6

			variation => N__('Sicilian: English, 2... d6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'B53a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4

			variation => N__('Sicilian: 2... d6 3. d4'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp2pppp/3p4/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B50c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. b3

			variation => N__('Sicilian: 2. Nf3 d6 3. b3'),
			moves => {
				'e7e6' => 'rnbqkbnr/pp3ppp/3pp3/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/1P6/P1PP1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B20',
                        # TRANSLATORS: 1. e4 c5 2. b3 d6

			variation => N__('Sicilian: Snyder, 2... d6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'B50i',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Nc3

			variation => N__('Sicilian: 2. Nf3 d6 3. Nc3'),
			moves => {
				'e7e6' => 'rnbqkbnr/pp3ppp/3pp3/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'B23d',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 d6 3. g3

			variation => N__('Sicilian: Closed'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'B23c',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 d6

			variation => N__('Sicilian: Closed'),
			moves => {
				'f2f4' => 'rnbqkbnr/pp2pppp/3p4/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
				'g1e2' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N3P1/PPPP1P1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N5/PPPPNPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'B23c',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 d6 3. Nge2

			variation => N__('Sicilian: Closed'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B50l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. c3

			variation => N__('Sicilian: 2. Nf3 d6 3. c3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B22e',
                        # TRANSLATORS: 1. e4 c5 2. c3 d6

			variation => N__('Sicilian: Alapin, 2... d6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B50e',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d3

			variation => N__('Sicilian: 2. Nf3 d6 3. d3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'B50a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6

			variation => N__('Sicilian: 2. Nf3 d6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3',
				'b2b3' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R b KQkq - 0 3',
				'b2b4' => 'rnbqkbnr/pp2pppp/3p4/2p5/1P2P3/5N2/P1PP1PPP/RNBQKB1R b KQkq b3 0 3',
				'c2c3' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/2P2N2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'd2d3' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'd2d4' => 'rnbqkbnr/pp2pppp/3p4/2p5/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'f1b5' => 'rnbqkbnr/pp2pppp/3p4/1Bp5/4P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3',
				'f1c4' => 'rnbqkbnr/pp2pppp/3p4/2p5/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/pp2pppp/3p4/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4P3/5NP1/PPPP1P1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'B50f',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. g3

			variation => N__('Sicilian: 2. Nf3 d6 3. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'B23e',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 d6 3. f4

			variation => N__('Sicilian: Closed'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2p5/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B21c',
                        # TRANSLATORS: 1. e4 c5 2. f4 d6

			variation => N__('Sicilian: Grand Prix, 2... d6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2pP4/4P3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. e4

			variation => N__('Old Benoni: Schmidt, 3. e4'),
			moves => {
				'g7g6' => 'rnbqkbnr/pp2pp1p/3p2p1/2pP4/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2pP4/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2pP4/8/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. Nc3

			variation => N__('Old Benoni: Schmidt, 3. Nc3'),
			moves => {
				'g7g6' => 'rnbqkbnr/pp2pp1p/3p2p1/2pP4/8/2N5/PPP1PPPP/R1BQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2pP4/8/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2pP4/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6 3. Nf3

			variation => N__('Old Benoni: Schmidt, 3. Nf3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/2pP4/8/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/2pP4/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A43s',
                        # TRANSLATORS: 1. d4 c5 2. d5 d6

			variation => N__('Old Benoni: Schmidt'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/3p4/2pP4/8/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 3',
				'e2e4' => 'rnbqkbnr/pp2pppp/3p4/2pP4/4P3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkbnr/pp2pppp/3p4/2pP4/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'B54a',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4

			variation => N__('Sicilian: Open, 2... d6'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p2pppp/p2p4/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'e7e5' => 'rnbqkbnr/pp3ppp/3p4/4p3/3NP3/8/PPP2PPP/RNBQKB1R w KQkq e6 0 5',
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 4' => {
			eco => 'B53d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Qxd4

			variation => N__('Sicilian, Chekhover Variation'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p2pppp/p2p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 0 5',
				'b8c6' => 'r1bqkbnr/pp2pppp/2np4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5',
				'c8d7' => 'rn1qkbnr/pp1bpppp/3p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/pp2pppp/3p1n2/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B53c',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4

			variation => N__('Sicilian: 2... d6 3. d4 cxd4'),
			moves => {
				'd1d4' => 'rnbqkbnr/pp2pppp/3p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 4',
				'f3d4' => 'rnbqkbnr/pp2pppp/3p4/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/3p4/8/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'B21t',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 dxc3 4. Nxc3 d6

			variation => N__('Sicilian: Smith-Morra, 4.Nxc3 d6'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2Pp4/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c5 3. dxc5

			variation => N__('Queen\'s Pawn: Krause, Reversed QGA'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2PpP3/8/8/PPP2PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B12j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 c5 4. dxc5

			variation => N__('Caro-Kann: Advance, 3... c5 4.dxc5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp2pppp/2n5/2PpP3/8/8/PPP2PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2BP4/4P3/PP3PPP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'D20c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 c5 4. Bxc4

			variation => N__('QGA: 3. e3 c5 4.Bxc4'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp2pppp/8/8/2Bp4/4P3/PP3PPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2Pp4/4PN2/PP1P1PPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A09e',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. e3 c5

			variation => N__('Reti: Advance, 3. e3 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2Pp4/5NP1/PP1PPP1P/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'A09j',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 c5

			variation => N__('Reti: Advance, 3. g3 c5'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp2pppp/8/2p5/2Pp4/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2Pp4/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A09j',
                        # TRANSLATORS:

			variation => N__('Reti: Advance, 3. g3 c5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp2pppp/2n5/2p5/2Pp4/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2p5/N4N2/PP1PPPPP/R1BQKB1R w KQkq c6 0 4' => {
			eco => 'A09q',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. Na3 c5

			variation => N__('Reti: Accepted, 3. Na3 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D21i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. Nc3

			variation => N__('QGA: 3. Nf3 c5 4. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'D20i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 c5

			variation => N__('QGA: 3. Nc3 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq c6 0 4' => {
			eco => 'D20c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 c5

			variation => N__('QGA: 3. e3 c5'),
			moves => {
				'f1c4' => 'rnbqkbnr/pp2pppp/8/2p5/2BP4/4P3/PP3PPP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D21j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. e3

			variation => N__('QGA: 3. Nf3 c5 4. e3'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp2pppp/8/8/2pp4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'D21h',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5

			variation => N__('QGA: 3. Nf3 c5'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/8/2p5/2pP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
				'd4d5' => 'rnbqkbnr/pp2pppp/8/2pP4/2p5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqkbnr/pp2pppp/8/2p5/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'D20s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 c5 4. Nf3

			variation => N__('QGA: 3. e4 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2p5/2pPP3/8/PP3PPP/RNBQKBNR w KQkq c6 0 4' => {
			eco => 'D20s',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 c5

			variation => N__('QGA: 3. e4 c5'),
			moves => {
				'd4d5' => 'rnbqkbnr/pp2pppp/8/2pP4/2p1P3/8/PP3PPP/RNBQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkbnr/pp2pppp/8/2p5/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pP4/2p1P3/8/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D20t',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 c5 4. d5

			variation => N__('QGA: 3. e4 c5 4. d5'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/2pP4/2p1P3/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pP4/2p5/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D21l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5

			variation => N__('QGA: 3. Nf3 c5 4. d5'),
			moves => {
				'e7e6' => 'rnbqkbnr/pp3ppp/4p3/2pP4/2p5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pP4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D06b',
                        # TRANSLATORS:

			variation => N__('QGD: Austrian Defence'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/2pP4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pP4/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B21j',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5 3. exd5

			variation => N__('Sicilian: Grand Prix, Tal Defence, 3.exd5'),
			moves => {
				'd8d5' => 'rnb1kbnr/pp2pppp/8/2pq4/5P2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/2pP4/5P2/8/PPPP2PP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pP4/8/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B22f',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5 3. exd5

			variation => N__('Sicilian: Alapin, 2... d5 3.exd5'),
			moves => {
				'd8d5' => 'rnb1kbnr/pp2pppp/8/2pq4/8/2P5/PP1P1PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq c3 0 3' => {
			eco => 'D02j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c5 3. c4

			variation => N__('Queen\'s Pawn: Krause, 3. c4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq c6 0 3' => {
			eco => 'D06b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c5

			variation => N__('QGD: Austrian Defence'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp2pppp/8/2pP4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/3P1B2/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 3' => {
			eco => 'D00c',
                        # TRANSLATORS: 1. d4 d5 2. Bf4 c5

			variation => N__('Queen\'s Pawn, Mason, Steinitz Countergambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c5 3. c3

			variation => N__('Queen\'s Pawn: Krause, Reversed Slav'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c5 3. e3

			variation => N__('Queen\'s Pawn: Krause, Reversed QGD'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'D02j',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 c5

			variation => N__('Queen\'s Pawn: Krause Variation'),
			moves => {
				'c2c3' => 'rnbqkbnr/pp2pppp/8/2pp4/3P4/2P2N2/PP2PPPP/RNBQKB1R b KQkq - 0 3',
				'c2c4' => 'rnbqkbnr/pp2pppp/8/2pp4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq c3 0 3',
				'd4c5' => 'rnbqkbnr/pp2pppp/8/2Pp4/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 0 3',
				'e2e3' => 'rnbqkbnr/pp2pppp/8/2pp4/3P4/4PN2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B21m',
                        # TRANSLATORS: 1. e4 c5 2. d4 d5

			variation => N__('Sicilian: Smith-Morra, 2... d5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B22f',
                        # TRANSLATORS: 1. e4 c5 2. c3 d5

			variation => N__('Sicilian: Alapin, 2... d5'),
			moves => {
				'e4d5' => 'rnbqkbnr/pp2pppp/8/2pP4/8/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'B21i',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5 3. Nc3

			variation => N__('Sicilian: Grand Prix, Toilet Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq e3 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 c5 3. e4

			variation => N__('Bird: Batavo Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/4PP2/8/PPPP2PP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'B21i',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5

			variation => N__('Sicilian: Grand Prix, Tal Defence'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/8/2pp4/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3',
				'e4d5' => 'rnbqkbnr/pp2pppp/8/2pP4/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 3',
				'e4e5' => 'rnbqkbnr/pp2pppp/8/2ppP3/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 c5 3. e3

			variation => N__('Bird: 1... d5 2. Nf3 c5 3. e3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 c5

			variation => N__('Bird: 1... d5 2. Nf3 c5'),
			moves => {
				'e2e3' => 'rnbqkbnr/pp2pppp/8/2pp4/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3',
				'e2e4' => 'rnbqkbnr/pp2pppp/8/2pp4/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/8/1P3N2/P1PPPPPP/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3 c5

			variation => N__('Reti: Nimzowitsch-Larsen, 2... c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq c6 0 3' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5

			variation => N__('Reti: KIA, 2... c5'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp2pppp/8/2pp4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pp2pppp/8/2pp4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A08',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 c5 3. Bg2

			variation => N__('Reti: KIA, 2... c5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp2pppp/2n5/2pp4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
				'g7g6' => 'rnbqkbnr/pp2pp1p/6p1/2pp4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/2pp4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2ppP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B12j',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. e5 c5

			variation => N__('Caro-Kann: Advance, 3... c5'),
			moves => {
				'd4c5' => 'rnbqkbnr/pp2pppp/8/2PpP3/8/8/PPP2PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/2ppP3/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'B21i',
                        # TRANSLATORS: 1. e4 c5 2. f4 d5 3. e5

			variation => N__('Sicilian: Grand Prix, Tal Defence, 3. e5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/3P4/1p6/P7/2PP1PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B20',
                        # TRANSLATORS:

			variation => N__('Sicilian: Wing Gambit, Marshall Variation'),
			moves => {
				'd8d5' => 'rnb1kbnr/pp2pppp/8/3q4/1p6/P7/2PP1PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/3P4/8/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B10j',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5 cxd5 4. cxd5

			variation => N__('Caro-Kann: English, Exchange'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/3P4/8/8/PP1P1PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/1p2P3/P7/2PP1PPP/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'B20',
                        # TRANSLATORS:

			variation => N__('Sicilian: Wing Gambit, Marshall Variation'),
			moves => {
				'e4d5' => 'rnbqkbnr/pp2pppp/8/3P4/1p6/P7/2PP1PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/2P5/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B10i',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. exd5 cxd5

			variation => N__('Caro-Kann: English Variation'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp2pppp/8/3P4/8/8/PP1P1PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4' => {
			eco => 'B13n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4

			variation => N__('Caro-Kann: Panov-Botvinnik Attack'),
			moves => {
				'e7e6' => 'rnbqkbnr/pp3ppp/4p3/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P1B2/8/PP2PPPP/RN1QKBNR b KQkq - 1 4' => {
			eco => 'D10c',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Bf4

			variation => N__('Slav: Exchange, 4. Bf4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'D10e',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nc3

			variation => N__('Slav: Exchange, 4. Nc3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P4/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'B13e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Bd3

			variation => N__('Caro-Kann: Exchange, 4. Bd3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp2pppp/2n5/3p4/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'D10d',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5 4. Nf3

			variation => N__('Slav: Exchange, 4. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'B13c',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. Nf3

			variation => N__('Caro-Kann: Exchange, 4. Nf3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp2pppp/5n2/3p4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D10b',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. cxd5 cxd5

			variation => N__('Slav: Exchange'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp2pppp/8/3p4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
				'c1f4' => 'rnbqkbnr/pp2pppp/8/3p4/3P1B2/8/PP2PPPP/RN1QKBNR b KQkq - 1 4',
				'g1f3' => 'rnbqkbnr/pp2pppp/8/3p4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B13b',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5

			variation => N__('Caro-Kann: Exchange, 3...cxd5'),
			moves => {
				'c2c4' => 'rnbqkbnr/pp2pppp/8/3p4/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4',
				'f1d3' => 'rnbqkbnr/pp2pppp/8/3p4/3P4/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkbnr/pp2pppp/8/3p4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'B21q',
                        # TRANSLATORS: 1. e4 c5 2. d4 cxd4 3. c3 d5

			variation => N__('Sicilian: Smith-Morra, 3. c3 d5'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/3p4/4P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B10i',
                        # TRANSLATORS: 1. e4 c6 2. c4 d5 3. cxd5 cxd5

			variation => N__('Caro-Kann: English Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/8/2BP4/8/PP3PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'D20c',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 c5 4. Bxc4 cxd4 5. exd4

			variation => N__('QGA: 3. e3 c5 4.Bxc4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/8/2Bp4/4P3/PP3PPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'D20c',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e3 c5 4.Bxc4'),
			moves => {
				'e3d4' => 'rnbqkbnr/pp2pppp/8/8/2BP4/8/PP3PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp2pppp/8/8/2Bp4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D21k',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. e3 cxd4 5. Bxc4

			variation => N__('QGA: 3. Nf3 c5 4. e3 cxd4 5.Bxc4'),
			moves => {
			},
		},
		'rnbqkbnr/pp2pppp/8/8/2pp4/4PN2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D21j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. e3 cxd4

			variation => N__('QGA: 3. Nf3 c5 4. e3 cxd4'),
			moves => {
				'f1c4' => 'rnbqkbnr/pp2pppp/8/8/2Bp4/4PN2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3p1p/3p2p1/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6' => {
			eco => 'A43i',
                        # TRANSLATORS:

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp3pbp/3p2p1/2pP4/4P3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 7',
			},
		},
		'rnbqkbnr/pp3p1p/3p2p1/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'A43i',
                        # TRANSLATORS:

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'e2e4' => 'rnbqkbnr/pp3p1p/3p2p1/2pP4/4P3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 6',
			},
		},
		'rnbqkbnr/pp3p1p/3p2p1/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 1 5' => {
			eco => 'A44h',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 g6 5. Nc3

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 g6 5. Nc3'),
			moves => {
				'f8g7' => 'rnbqk1nr/pp3pbp/3p2p1/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3p1p/3p2p1/2pPp3/2P1P3/8/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'A44h',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4 g6

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4 g6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp3p1p/3p2p1/2pPp3/2P1P3/2N5/PP3PPP/R1BQKBNR b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3p1p/3p2p1/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A44s',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3 g6

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2P5/4p3/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 c6 4. dxc6

			variation => N__('KGD: Falkbeer, Marshall/Nimzowitsch, 4.dxc6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D31j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. cxd5

			variation => N__('Semi-Slav: 4.cxd5'),
			moves => {
				'e6d5' => 'rnbqkbnr/pp3ppp/2p5/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A13p',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 c6

			variation => N__('English: Kurajica Defence'),
			moves => {
				'd1c2' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2P5/5NP1/PPQPPP1P/RNB1KB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2P5/5NP1/PPQPPP1P/RNB1KB1R b KQkq - 1 4' => {
			eco => 'A13q',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3 c6 4. Qc2

			variation => N__('English: Kurajica Defence'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D31n',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e3

			variation => N__('Semi-Slav: 4. e3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D31p',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3

			variation => N__('Semi-Slav: 4. Nf3'),
			moves => {
				'd5c4' => 'rnbqkbnr/pp3ppp/2p1p3/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D31i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6

			variation => N__('Semi-Slav'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp3ppp/2p1p3/3P4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'e2e3' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'e2e4' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D30k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6 4. e3

			variation => N__('QGD: 3. Nf3 c6 4. e3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D30l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6 4. Nbd2

			variation => N__('QGD: 3. Nf3 c6 4.Nbd2'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D30j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6

			variation => N__('QGD: 3. Nf3 c6'),
			moves => {
				'b1d2' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/5N2/PP1NPPPP/R1BQKB1R b KQkq - 1 4',
				'd1c2' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R b KQkq - 1 4',
				'e2e3' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R b KQkq - 1 4' => {
			eco => 'D30m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c6 4. Qc2

			variation => N__('QGD: 3. Nf3 c6 4. Qc2'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp3ppp/2p1pn2/3p4/2PP4/5N2/PPQ1PPPP/RNB1KB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/2PPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'D31w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4

			variation => N__('Semi-Slav: Marshall Gambit'),
			moves => {
				'd5e4' => 'rnbqkbnr/pp3ppp/2p1p3/8/2PPp3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5',
				'f8b4' => 'rnbqk1nr/pp3ppp/2p1p3/3p4/1bPPP3/2N5/PP3PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/3p4/3PP3/5P2/PPP3PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'B12e',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. f3 e6

			variation => N__('Caro-Kann: Maroczy (Fantasy), 3... e6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/8/2PPN3/8/PP3PPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D31w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. Nxe4

			variation => N__('Semi-Slav: Marshall Gambit, 5.Nxe4'),
			moves => {
				'f8b4' => 'rnbqk1nr/pp3ppp/2p1p3/8/1bPPN3/8/PP3PPP/R1BQKBNR w KQkq - 1 6',
				'g8f6' => 'rnbqkb1r/pp3ppp/2p1pn2/8/2PPN3/8/PP3PPP/R1BQKBNR w KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/8/2PPp3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D31w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4 5. f3

			variation => N__('Semi-Slav: Marshall Gambit, Gunderam Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/8/2PPp3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D31w',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. e4 dxe4

			variation => N__('Semi-Slav: Marshall Gambit, 4...dxe4'),
			moves => {
				'c3e4' => 'rnbqkbnr/pp3ppp/2p1p3/8/2PPN3/8/PP3PPP/R1BQKBNR b KQkq - 0 5',
				'f2f3' => 'rnbqkbnr/pp3ppp/2p1p3/8/2PPp3/2N2P2/PP4PP/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/8/2pP4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D31q',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4

			variation => N__('Semi-Slav: Noteboom Variation'),
			moves => {
				'a2a4' => 'rnbqkbnr/pp3ppp/2p1p3/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p1p3/8/P1pP4/2N2N2/1P2PPPP/R1BQKB1R b KQkq a3 0 5' => {
			eco => 'D31r',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. Nf3 dxc4 5. a4

			variation => N__('Semi-Slav: Noteboom, 5. a4'),
			moves => {
				'f8b4' => 'rnbqk1nr/pp3ppp/2p1p3/8/PbpP4/2N2N2/1P2PPPP/R1BQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3B4/4Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 d5 4. Bxd5 c6

			variation => N__('KGA: Bishop\'s Gambit, Anderssen Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3Pp3/2BP4/8/PPP2PPP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'c6d5' => 'rnbqkbnr/pp3ppp/8/3pp3/2BP4/8/PPP2PPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3Pp3/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D10k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5 4. cxd5

			variation => N__('Slav: Winawer Countergambit, 4.cxd5'),
			moves => {
				'c6d5' => 'rnbqkbnr/pp3ppp/8/3pp3/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3Pp3/5P2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 c6 4. Nc3

			variation => N__('KGD: Falkbeer, Marshall/Nimzowitsch, 4. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3Pp3/5P2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 c6

			variation => N__('KGD: Falkbeer, Marshall/Nimzowitsch Countergambit'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp3ppp/2p5/3Pp3/5P2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 4',
				'd1e2' => 'rnbqkbnr/pp3ppp/2p5/3Pp3/5P2/8/PPPPQ1PP/RNB1KBNR b KQkq - 1 4',
				'd5c6' => 'rnbqkbnr/pp3ppp/2P5/4p3/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3Pp3/5P2/8/PPPPQ1PP/RNB1KBNR b KQkq - 1 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 c6 4. Qe2

			variation => N__('KGD: Falkbeer, Marshall/Nimzowitsch, 4. Qe2'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'C01c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4 c6

			variation => N__('French: Exchange, 4. c4 c6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5' => {
			eco => 'D31m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. cxd5 exd5 5. Bf4

			variation => N__('Semi-Slav: 4.cxd5 exd5 5. Bf4'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D31k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. cxd5 exd5 5. Nf3

			variation => N__('Semi-Slav: 4.cxd5 exd5 5. Nf3'),
			moves => {
				'c8f5' => 'rn1qkbnr/pp3ppp/2p5/3p1b2/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3p4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D31j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c6 4. cxd5 exd5

			variation => N__('Semi-Slav: 4.cxd5'),
			moves => {
				'c1f4' => 'rnbqkbnr/pp3ppp/2p5/3p4/3P1B2/2N5/PP2PPPP/R2QKBNR b KQkq - 1 5',
				'g1f3' => 'rnbqkbnr/pp3ppp/2p5/3p4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3pP3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D10m',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5 4. dxe5

			variation => N__('Slav: Winawer Countergambit Accepted'),
			moves => {
				'd5d4' => 'rnbqkbnr/pp3ppp/2p5/4P3/2Pp4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3pp3/2BPP3/8/PPP2PPP/RNBQK1NR w KQkq d6 0 4' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'e4d5' => 'rnbqkbnr/pp3ppp/2p5/3Pp3/2BP4/8/PPP2PPP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3pp3/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'D10j',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5

			variation => N__('Slav: Winawer Countergambit'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp3ppp/2p5/3Pp3/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'd4e5' => 'rnbqkbnr/pp3ppp/2p5/3pP3/2P5/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3pp3/4P3/3P1N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'B10e',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Breyer, 3. Nd2 e5'),
			moves => {
				'f8d6' => 'rnbqk1nr/pp3ppp/2pb4/3pp3/4P3/3P1N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/3pp3/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'B10e',
                        # TRANSLATORS: 1. e4 c6 2. d3 d5 3. Nd2 e5

			variation => N__('Caro-Kann: Breyer, 3. Nd2 e5'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp3ppp/2p5/3pp3/4P3/3P1N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/4P3/2Pp4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D10m',
                        # TRANSLATORS:

			variation => N__('Slav: Winawer Countergambit Accepted'),
			moves => {
				'c3e4' => 'rnbqkbnr/pp3ppp/2p5/4P3/2PpN3/8/PP2PPPP/R1BQKBNR b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/4P3/2PpN3/8/PP2PPPP/R1BQKBNR b KQkq - 1 5' => {
			eco => 'D10m',
                        # TRANSLATORS:

			variation => N__('Slav: Winawer Countergambit Accepted'),
			moves => {
				'd8a5' => 'rnb1kbnr/pp3ppp/2p5/q3P3/2PpN3/8/PP2PPPP/R1BQKBNR w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/4p3/3PP3/5N2/PPP3PP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'B12f',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Maroczy (Fantasy), 3...dxe4'),
			moves => {
				'e5d4' => 'rnbqkbnr/pp3ppp/2p5/8/3pP3/5N2/PPP3PP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/4p3/3PP3/8/PPP3PP/RNBQKBNR w KQkq e6 0 5' => {
			eco => 'B12f',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Maroczy (Fantasy), 3...dxe4'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp3ppp/2p5/4p3/3PP3/5N2/PPP3PP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/2p5/8/2BpP3/5N2/PPP3PP/RNBQK2R b KQkq - 1 6' => {
			eco => 'B12f',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. f3 dxe4 4. fxe4 e5 5. Nf3 exd4 6. Bc4

			variation => N__('Caro-Kann: Maroczy (Fantasy), Gambit line'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/2p5/8/3pP3/5N2/PPP3PP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'B12f',
                        # TRANSLATORS:

			variation => N__('Caro-Kann: Maroczy (Fantasy), 3...dxe4'),
			moves => {
				'f1c4' => 'rnbqkbnr/pp3ppp/2p5/8/2BpP3/5N2/PPP3PP/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/2pp4/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. g3 c6

			variation => N__('English: King\'s, Keres Variation'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp3ppp/2pp4/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/2pp4/4p3/2P5/2N3P1/PP1PPPBP/R1BQK1NR b KQkq - 1 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. g3 c6 4. Bg2

			variation => N__('English: King\'s, Keres Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3p4/1BpPp3/4P3/8/PPP2PPP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'A44o',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Bb5+

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Bb5+'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pP4/4P3/8/PP3PPP/RNBQKBNR b KQkq e3 0 5' => {
			eco => 'A43h',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4 exd5 4. cxd5 d6 5. e4

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 5' => {
			eco => 'A43i',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4 exd5 4. cxd5 d6 5. Nc3

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'g7g6' => 'rnbqkbnr/pp3p1p/3p2p1/2pP4/8/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pP4/8/8/PP2PPPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'A43h',
                        # TRANSLATORS: 1. d4 c5 2. d5 e6 3. c4 exd5 4. cxd5 d6

			variation => N__('Old Benoni: 2. d5 e6 3. c4'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp3ppp/3p4/2pP4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 5',
				'e2e4' => 'rnbqkbnr/pp3ppp/3p4/2pP4/4P3/8/PP3PPP/RNBQKBNR b KQkq e3 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pPp3/2P1P3/8/PP3PPP/RNBQKBNR b KQkq e3 0 4' => {
			eco => 'A44e',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6 4. e4

			variation => N__('Old Benoni: Czech, 3. c4 d6 4. e4'),
			moves => {
				'f8e7' => 'rnbqk1nr/pp2bppp/3p4/2pPp3/2P1P3/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
				'g7g6' => 'rnbqkbnr/pp3p1p/3p2p1/2pPp3/2P1P3/8/PP3PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pPp3/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A44d',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. c4 d6

			variation => N__('Old Benoni: Czech, 3. c4 d6'),
			moves => {
				'e2e4' => 'rnbqkbnr/pp3ppp/3p4/2pPp3/2P1P3/8/PP3PPP/RNBQKBNR b KQkq e3 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'A44q',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nc3

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nc3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p3ppp/p2p4/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
				'f8e7' => 'rnbqk1nr/pp2bppp/3p4/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5',
				'g7g6' => 'rnbqkbnr/pp3p1p/3p2p1/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/pp3ppp/3p1n2/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'A44p',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Bd3

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Bd3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'A44n',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6 4. Nf3

			variation => N__('Old Benoni: Czech, 3. e4 d6 4. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A44m',
                        # TRANSLATORS: 1. d4 c5 2. d5 e5 3. e4 d6

			variation => N__('Old Benoni: Czech, 3. e4 d6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4',
				'f1b5' => 'rnbqkbnr/pp3ppp/3p4/1BpPp3/4P3/8/PPP2PPP/RNBQK1NR b KQkq - 1 4',
				'f1d3' => 'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkbnr/pp3ppp/3p4/2pPp3/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/3p4/4p3/3NP3/8/PPP2PPP/RNBQKB1R w KQkq e6 0 5' => {
			eco => 'B54b',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. d4 cxd4 4. Nxd4 e5

			variation => N__('Sicilian: Open, 2... d6, 4... e5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3pp3/2p5/4P3/1P3N2/P1PP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'B50d',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. b3 e6

			variation => N__('Sicilian: 2. Nf3 d6 3. b3 e6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3pp3/2p5/4P3/2N2N2/PPPP1PPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'B50j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 d6 3. Nc3 e6

			variation => N__('Sicilian: 2. Nf3 d6 3. Nc3 e6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/3pp3/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'B40l',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 cxd4 4. Nxd4 d6

			variation => N__('Sicilian: Open, 2... e6, 4.Nxd4 d6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2Pp4/4P3/8/PPPN1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C07c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. dxc5

			variation => N__('French: Tarrasch, Open, 4.dxc5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2PpP3/8/8/PPP2PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'C02d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. dxc5

			variation => N__('French: Advance, Steinitz'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2p5/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq c6 0 5' => {
			eco => 'D21g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 e6 4. e3 c5

			variation => N__('QGA: 3. Nf3 e6 4. e3 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/2p1P3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 5' => {
			eco => 'D21m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. e4

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. e4'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/2p5/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D21n',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. Nc3

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3'),
			moves => {
				'e6d5' => 'rnbqkbnr/pp3ppp/8/2pp4/2p5/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/2p5/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D21m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pp3ppp/4p3/2pP4/2p5/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5',
				'e2e4' => 'rnbqkbnr/pp3ppp/4p3/2pP4/2p1P3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D32d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5

			variation => N__('QGD Tarrasch: 4.cxd5'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp3ppp/4p3/3P4/3p4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
				'e6d5' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'B22d',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. d4 d5 4. exd5

			variation => N__('Sicilian: Alapin, 2... e6 3. d4 d5 4.exd5'),
			moves => {
				'e6d5' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D30h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5

			variation => N__('QGD: Tarrasch without Nc3'),
			moves => {
				'e6d5' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/8/PPPN1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C07i',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5

			variation => N__('French: Tarrasch, Open, 4.exd5'),
			moves => {
				'd8d5' => 'rnb1kbnr/pp3ppp/4p3/2pq4/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 0 5',
				'e6d5' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/pp3ppp/4pn2/2pP4/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'D32a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. e3

			variation => N__('QGD Tarrasch: 4. e3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D32b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. Nf3

			variation => N__('QGD Tarrasch: 4. Nf3'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp3ppp/4p3/3p4/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'D32a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5

			variation => N__('QGD Tarrasch'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 4',
				'e2e3' => 'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D30h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. e3

			variation => N__('QGD: Tarrasch without Nc3: 4. e3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/2PP4/4PN2/PP3PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'D30h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5

			variation => N__('QGD: Tarrasch without Nc3'),
			moves => {
				'c4d5' => 'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/5N2/PP2PPPP/RNBQKB1R b KQkq - 0 4',
				'e2e3' => 'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'D02i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 e6 3. g3 c5

			variation => N__('Queen\'s Pawn: 2. Nf3 e6 3. g3 c5'),
			moves => {
				'f1g2' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3P4/5NP1/PPP1PPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'D02i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 e6 3. g3 c5 4. Bg2

			variation => N__('Queen\'s Pawn: 2. Nf3 e6 3. g3 c5'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp3ppp/4p3/3p4/3p4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'C10b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 c5

			variation => N__('French: Marshall Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C07b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. c3

			variation => N__('French: Tarrasch, Open, 4. c3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/2P5/PP3PPP/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'B22d',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. d4 d5

			variation => N__('Sicilian: Alapin, 2... e6 3. d4 d5'),
			moves => {
				'e4d5' => 'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'B40j',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 e6 3. d4 d5

			variation => N__('Sicilian: Marshall Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C07d',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. Ngf3

			variation => N__('French: Tarrasch, Open, 4.Ngf3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p3ppp/p3p3/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 5',
				'b8c6' => 'r1bqkbnr/pp3ppp/2n1p3/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5',
				'c5d4' => 'rnbqkbnr/pp3ppp/4p3/3p4/3pP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/pp3ppp/4pn2/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'C07a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5

			variation => N__('French: Tarrasch, Open'),
			moves => {
				'c2c3' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/2P5/PP1N1PPP/R1BQKBNR b KQkq - 0 4',
				'd4c5' => 'rnbqkbnr/pp3ppp/4p3/2Pp4/4P3/8/PPPN1PPP/R1BQKBNR b KQkq - 0 4',
				'e4d5' => 'rnbqkbnr/pp3ppp/4p3/2pP4/3P4/8/PPPN1PPP/R1BQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/4P3/2N3P1/PPPP1P1P/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'B23j',
                        # TRANSLATORS: 1. e4 c5 2. Nc3 e6 3. g3 d5

			variation => N__('Sicilian: Closed, Korchnoi Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/4P3/2P2N2/PP1P1PPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'B22c',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. Nf3 d5

			variation => N__('Sicilian: Alapin, 2... e6 3. Nf3 d5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2pp4/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq c6 0 4' => {
			eco => 'C00q',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2 c5

			variation => N__('French: KIA 2. d3 d5 3. Nd2 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/1P1P4/8/P1P2PPP/RNBQKBNR b KQkq b3 0 4' => {
			eco => 'C02c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. b4

			variation => N__('French: Advance, Wing Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/1P6/5N2/P1PP1PPP/RNBQKB1R b KQkq b3 0 4' => {
			eco => 'C00k',
                        # TRANSLATORS: 1. e4 e6 2. Nf3 d5 3. e5 c5 4. b4

			variation => N__('French: Wing Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/3P2Q1/8/PPP2PPP/RNB1KBNR b KQkq - 1 4' => {
			eco => 'C02e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. Qg4

			variation => N__('French: Advance, Nimzowitsch Attack'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp3ppp/4p3/3pP3/3p2Q1/8/PPP2PPP/RNB1KBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'C02g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. c3

			variation => N__('French: Advance, 4. c3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp3ppp/2n1p3/2ppP3/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5',
				'd8b6' => 'rnb1kbnr/pp3ppp/1q2p3/2ppP3/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'C02f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. Nf3

			variation => N__('French: Advance, 4. Nf3'),
			moves => {
				'c5d4' => 'rnbqkbnr/pp3ppp/4p3/3pP3/3p4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq c6 0 4' => {
			eco => 'C02c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5

			variation => N__('French: Advance, 3... c5'),
			moves => {
				'b2b4' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/1P1P4/8/P1P2PPP/RNBQKBNR b KQkq b3 0 4',
				'c2c3' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/3P4/2P5/PP3PPP/RNBQKBNR b KQkq - 0 4',
				'd1g4' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/3P2Q1/8/PPP2PPP/RNB1KBNR b KQkq - 1 4',
				'd4c5' => 'rnbqkbnr/pp3ppp/4p3/2PpP3/8/8/PPP2PPP/RNBQKBNR b KQkq - 0 4',
				'g1f3' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/2ppP3/8/5N2/PPPP1PPP/RNBQKB1R w KQkq c6 0 4' => {
			eco => 'C00j',
                        # TRANSLATORS: 1. e4 e6 2. Nf3 d5 3. e5 c5

			variation => N__('French: 2. Nf3 d5 3. e5 c5'),
			moves => {
				'b2b4' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/1P6/5N2/P1PP1PPP/RNBQKB1R b KQkq b3 0 4',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3P4/3Q4/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 5' => {
			eco => 'D32d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qxd4

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5.Qxd4'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp3ppp/2n1p3/3P4/3Q4/2N5/PP2PPPP/R1B1KBNR w KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3P4/3p4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D32d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit'),
			moves => {
				'd1a4' => 'rnbqkbnr/pp3ppp/4p3/3P4/Q2p4/2N5/PP2PPPP/R1B1KBNR b KQkq - 1 5',
				'd1d4' => 'rnbqkbnr/pp3ppp/4p3/3P4/3Q4/2N5/PP2PPPP/R1B1KBNR b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3P4/Q2p4/2N5/PP2PPPP/R1B1KBNR b KQkq - 1 5' => {
			eco => 'D32f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 cxd4 5. Qa4+

			variation => N__('QGD Tarrasch: von Hennig-Schara Gambit, 5. Qa4+'),
			moves => {
				'c8d7' => 'rn1qkbnr/pp1b1ppp/4p3/3P4/Q2p4/2N5/PP2PPPP/R1B1KBNR w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3p4/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 0 5' => {
			eco => 'D32b',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 4. Nf3'),
			moves => {
				'e6e5' => 'rnbqkbnr/pp3ppp/8/3pp3/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B13n',
                        # TRANSLATORS: 1. e4 c6 2. d4 d5 3. exd5 cxd5 4. c4 e6

			variation => N__('Caro-Kann: Panov-Botvinnik, 4... e6'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3p4/2Pp4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'D32b',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 4. Nf3'),
			moves => {
				'f3d4' => 'rnbqkbnr/pp3ppp/4p3/3p4/2PN4/2N5/PP2PPPP/R1BQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3p4/3N4/6P1/PPP1PPBP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D02i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 e6 3. g3 c5 4. Bg2 cxd4 5. Nxd4

			variation => N__('Queen\'s Pawn: 2. Nf3 e6 3. g3 c5'),
			moves => {
				'e6e5' => 'rnbqkbnr/pp3ppp/8/3pp3/3N4/6P1/PPP1PPBP/RNBQK2R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3p4/3p4/5NP1/PPP1PPBP/RNBQK2R w KQkq - 0 5' => {
			eco => 'D02i',
                        # TRANSLATORS:

			variation => N__('Queen\'s Pawn: 2. Nf3 e6 3. g3 c5'),
			moves => {
				'f3d4' => 'rnbqkbnr/pp3ppp/4p3/3p4/3N4/6P1/PPP1PPBP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3p4/3pP3/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'C07h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. Ngf3 cxd4

			variation => N__('French: Tarrasch, Open, 4.Ngf3 cxd4'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3pP3/3p2Q1/5N2/PPP2PPP/RNB1KB1R b KQkq - 1 5' => {
			eco => 'C02e',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. Qg4 cxd4 5. Nf3

			variation => N__('French: Advance, Nimzowitsch Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3pP3/3p2Q1/8/PPP2PPP/RNB1KBNR w KQkq - 0 5' => {
			eco => 'C02e',
                        # TRANSLATORS:

			variation => N__('French: Advance, Nimzowitsch Attack'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp3ppp/4p3/3pP3/3p2Q1/5N2/PPP2PPP/RNB1KB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3pP3/3p4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C02f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5 c5 4. Nf3 cxd4 5. Bd3

			variation => N__('French: Advance, Ruisdonk Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/4p3/3pP3/3p4/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C02f',
                        # TRANSLATORS:

			variation => N__('French: Advance, 4. Nf3'),
			moves => {
				'f1d3' => 'rnbqkbnr/pp3ppp/4p3/3pP3/3p4/3B1N2/PPP2PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/4p3/8/2Bp4/5N2/PPPN1PPP/R1BQK2R w KQkq - 2 7' => {
			eco => 'C07n',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 Qxd5 5. Ngf3 cxd4 6. Bc4 Qd8

			variation => N__('French: Tarrasch, Open, Eliskases Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/1B1pp3/3P4/8/PPP2PPP/RNBQK1NR b KQkq - 1 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'c8d7' => 'rn1qkbnr/pp1b1ppp/8/1B1pp3/3P4/8/PPP2PPP/RNBQK1NR w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/1Bpp4/3P4/8/PPPN1PPP/R1BQK1NR b KQkq - 1 5' => {
			eco => 'C08a',
                        # TRANSLATORS:

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp3ppp/2n5/1Bpp4/3P4/8/PPPN1PPP/R1BQK1NR w KQkq - 2 6',
				'c8d7' => 'rn1qkbnr/pp1b1ppp/8/1Bpp4/3P4/8/PPPN1PPP/R1BQK1NR w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2P5/3p4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'D32k',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 5.dxc5'),
			moves => {
				'c3a4' => 'rnbqkbnr/pp3ppp/8/2P5/N2p4/8/PP2PPPP/R1BQKBNR b KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2P5/N2p4/8/PP2PPPP/R1BQKBNR b KQkq - 1 6' => {
			eco => 'D32k',
                        # TRANSLATORS:

			variation => N__('QGD Tarrasch: 5.dxc5'),
			moves => {
				'b7b5' => 'rnbqkbnr/p4ppp/8/1pP5/N2p4/8/PP2PPPP/R1BQKBNR w KQkq b6 0 7',
			},
		},
		'rnbqkbnr/pp3ppp/8/2Pp4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D32k',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. dxc5

			variation => N__('QGD Tarrasch: 5.dxc5'),
			moves => {
				'd5d4' => 'rnbqkbnr/pp3ppp/8/2P5/3p4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2p5/2BPp3/2N5/PP3PPP/R1BQK1NR b KQkq - 1 6' => {
			eco => 'D32l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. e4 dxe4 6. Bc4

			variation => N__('QGD Tarrasch: Marshall Gambit, Modern 6. Bc4'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/2p5/3Pp3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6' => {
			eco => 'D32l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. e4 dxe4

			variation => N__('QGD Tarrasch: Marshall Gambit'),
			moves => {
				'd4d5' => 'rnbqkbnr/pp3ppp/8/2pP4/4p3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 6',
				'f1c4' => 'rnbqkbnr/pp3ppp/8/2p5/2BPp3/2N5/PP3PPP/R1BQK1NR b KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pP4/4p3/2N5/PP3PPP/R1BQKBNR b KQkq - 0 6' => {
			eco => 'D32l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. e4 dxe4 6. d5

			variation => N__('QGD Tarrasch: Marshall Gambit, 6. d5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/2pQ4/2p5/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6' => {
			eco => 'D21o',
                        # TRANSLATORS:

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3 exd5'),
			moves => {
				'd8d5' => 'rnb1kbnr/pp3ppp/8/2pq4/2p5/2N2N2/PP2PPPP/R1B1KB1R w KQkq - 0 7',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/2p5/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D21o',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 c5 4. d5 e6 5. Nc3 exd5

			variation => N__('QGA: 3. Nf3 c5 4. d5 e6 5. Nc3 exd5'),
			moves => {
				'd1d5' => 'rnbqkbnr/pp3ppp/8/2pQ4/2p5/2N2N2/PP2PPPP/R1B1KB1R b KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D32m',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. Nf3

			variation => N__('QGD Tarrasch: 5. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D32j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. g3

			variation => N__('QGD Tarrasch: 5. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D32j',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5

			variation => N__('QGD Tarrasch: 4.cxd5 exd5'),
			moves => {
				'd4c5' => 'rnbqkbnr/pp3ppp/8/2Pp4/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 5',
				'e2e4' => 'rnbqkbnr/pp3ppp/8/2pp4/3PP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5',
				'g1f3' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5',
				'g2g3' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/2N3P1/PP2PP1P/R1BQKBNR b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'B22d',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. d4 d5 4. exd5 exd5 5. Nf3

			variation => N__('Sicilian: Alapin, 2... e6 3. d4 d5 4.exd5 exd5 5. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/2P2N2/PP3PPP/RNBQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/2P5/PP3PPP/RNBQKBNR w KQkq - 0 5' => {
			eco => 'B22d',
                        # TRANSLATORS: 1. e4 c5 2. c3 e6 3. d4 d5 4. exd5 exd5

			variation => N__('Sicilian: Alapin, 2... e6 3. d4 d5 4.exd5 exd5'),
			moves => {
				'g1f3' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/2P2N2/PP3PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D30h',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5

			variation => N__('QGD: Tarrasch without Nc3'),
			moves => {
				'g2g3' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'C08g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5 5. Ngf3

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5 5.Ngf3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1p3ppp/p7/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 0 6',
				'b8c6' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 6',
				'g8f6' => 'rnbqkb1r/pp3ppp/5n2/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/5NP1/PP2PP1P/RNBQKB1R b KQkq - 0 5' => {
			eco => 'D30i',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3 c5 4. cxd5 exd5 5. g3

			variation => N__('QGD: Tarrasch without Nc3: 5. g3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pp3ppp/2n5/2pp4/3P4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
				'g8f6' => 'rnbqkb1r/pp3ppp/5n2/2pp4/3P4/5NP1/PP2PP1P/RNBQKB1R w KQkq - 1 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3P4/8/PPPN1PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C08a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 c5 4. exd5 exd5

			variation => N__('French: Tarrasch, Open, 4.exd5 exd5'),
			moves => {
				'f1b5' => 'rnbqkbnr/pp3ppp/8/1Bpp4/3P4/8/PPPN1PPP/R1BQK1NR b KQkq - 1 5',
				'g1f3' => 'rnbqkbnr/pp3ppp/8/2pp4/3P4/5N2/PPPN1PPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/8/2pp4/3PP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 5' => {
			eco => 'D32l',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. cxd5 exd5 5. e4

			variation => N__('QGD Tarrasch: Marshall Gambit'),
			moves => {
				'd5e4' => 'rnbqkbnr/pp3ppp/8/2p5/3Pp3/2N5/PP3PPP/R1BQKBNR w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pp3ppp/8/3pP3/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 5' => {
			eco => 'D10k',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5 4. cxd5 cxd5 5. dxe5

			variation => N__('Slav: Winawer Countergambit, 4.cxd5 cxd5 5.dxe5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/3pp3/2BP4/8/PPP2PPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'C23',
                        # TRANSLATORS:

			variation => N__('Bishop\'s Opening: Philidor Counterattack'),
			moves => {
				'c4b5' => 'rnbqkbnr/pp3ppp/8/1B1pp3/3P4/8/PPP2PPP/RNBQK1NR b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp3ppp/8/3pp3/2PN4/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'D32c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 c5 4. Nf3 cxd4 5. Nxd4 e5

			variation => N__('QGD Tarrasch: 4. Nf3 cxd4 5.Nxd4 e5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/3pp3/3N4/6P1/PPP1PPBP/RNBQK2R w KQkq - 0 6' => {
			eco => 'D02i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 e6 3. g3 c5 4. Bg2 cxd4 5. Nxd4 e5

			variation => N__('Queen\'s Pawn: 2. Nf3 e6 3. g3 c5'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/3pp3/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'D10l',
                        # TRANSLATORS: 1. d4 d5 2. c4 c6 3. Nc3 e5 4. cxd5 cxd5 5. Nf3

			variation => N__('Slav: Winawer Countergambit, 4.cxd5 cxd5 5. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pp3ppp/8/3pp3/3P4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'D10k',
                        # TRANSLATORS:

			variation => N__('Slav: Winawer Countergambit, 4.cxd5'),
			moves => {
				'd4e5' => 'rnbqkbnr/pp3ppp/8/3pP3/8/2N5/PP2PPPP/R1BQKBNR b KQkq - 0 5',
				'g1f3' => 'rnbqkbnr/pp3ppp/8/3pp3/3P4/2N2N2/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pp4pp/2p1p3/3p1p2/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6 3. Nc3 d5 4. e3 c6

			variation => N__('Dutch: Rubinstein, 3... d5 4. e3 c6'),
			moves => {
			},
		},
		'rnbqkbnr/pp4pp/2pp4/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4 5. Ng5 c6

			variation => N__('English: Lukin, 5. Ng5 c6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/3p4/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 d6

			variation => N__('Dutch: 2. c4 d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/3p4/5p2/3P2P1/3Q4/PPP1PP1P/RNB1KBNR b KQkq g3 0 3' => {
			eco => 'A80i',
                        # TRANSLATORS: 1. d4 f5 2. Qd3 d6 3. g4

			variation => N__('Dutch: Alapin, Manhattan Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/3p4/5p2/3P4/3Q4/PPP1PPPP/RNB1KBNR w KQkq - 0 3' => {
			eco => 'A80i',
                        # TRANSLATORS:

			variation => N__('Dutch: Alapin'),
			moves => {
				'g2g4' => 'rnbqkbnr/ppp1p1pp/3p4/5p2/3P2P1/3Q4/PPP1PP1P/RNB1KBNR b KQkq g3 0 3',
			},
		},
		'rnbqkbnr/ppp1p1pp/3p4/5p2/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 d6

			variation => N__('Dutch: Staunton Gambit, Balogh Defence'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/5p2/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'D00f',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 f6

			variation => N__('Hodgson Attack: 2... f6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/8/3p1p2/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'A80k',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 d5

			variation => N__('Dutch: 2. Nc3 d5'),
			moves => {
				'c1g5' => 'rnbqkbnr/ppp1p1pp/8/3p1pB1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3',
				'e2e4' => 'rnbqkbnr/ppp1p1pp/8/3p1p2/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/ppp1p1pp/8/3p1p2/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq e3 0 3' => {
			eco => 'A80k',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 d5 3. e4

			variation => N__('Dutch: 2. Nc3 d5, Euwe Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/8/3p1pB1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A80l',
                        # TRANSLATORS: 1. d4 f5 2. Nc3 d5 3. Bg5

			variation => N__('Dutch: 2. Nc3 d5 3. Bg5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/8/3p1pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 3' => {
			eco => 'A80d',
                        # TRANSLATORS: 1. d4 f5 2. Bg5 d5

			variation => N__('Dutch: 2. Bg5 d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/8/3p2N1/4p3/8/PPPP1PPP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. e4 fxe4 3. Ng5 d5

			variation => N__('Reti: Lisitsin: 3. Ng5 d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/8/5p2/2pPP3/8/PP3PPP/RNBQKBNR w KQkq f6 0 4' => {
			eco => 'D20l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 f5

			variation => N__('QGA: 3. e4, Schwartz Defence'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1p1pp/8/5p2/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq f6 0 4' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3 f5

			variation => N__('Blackmar-Diemer: Pohmlann Defence'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A41t',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nc3

			variation => N__('Neo-Old Indian: Modern, 3. Nc3'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A41r',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. Nf3

			variation => N__('Neo-Old Indian: Modern, 3. Nf3'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A41q',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6

			variation => N__('Neo-Old Indian: Modern'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'e2e4' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A41q',
                        # TRANSLATORS: 1. d4 d6 2. c4 g6 3. e4

			variation => N__('Neo-Old Indian: Modern, 3. e4'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/2PPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3' => {
			eco => 'A41e',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 g6 3. Bf4

			variation => N__('Neo-Old Indian / Modern: 3. Bf4'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A41e',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 g6

			variation => N__('Neo-Old Indian / Modern'),
			moves => {
				'c1f4' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/3P1B2/5N2/PPP1PPPP/RN1QKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A41e',
                        # TRANSLATORS: 1. d4 d6 2. Nf3 g6 3. g3

			variation => N__('Neo-Old Indian / Modern: 3. g3'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/3p2p1/8/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B06e',
                        # TRANSLATORS: 1. e4 g6 2. d4 d6

			variation => N__('Modern'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'B06b',
                        # TRANSLATORS: 1. e4 g6 2. Nc3 d6

			variation => N__('Modern'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/3p2p1/8/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B06c',
                        # TRANSLATORS: 1. e4 g6 2. f4 d6

			variation => N__('Modern'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'D00f',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 g6

			variation => N__('Hodgson Attack: 2... g6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'D02a',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 g6

			variation => N__('Queen\'s Pawn: 2. Nf3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 g6 3. e3

			variation => N__('Bird: 1... d5 2. Nf3 g6 3. e3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p4/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 g6

			variation => N__('Bird: 1... d5 2. Nf3 g6'),
			moves => {
				'e2e3' => 'rnbqkbnr/ppp1pp1p/6p1/3p4/5P2/4PN2/PPPP2PP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkbnr/ppp1pp1p/6p1/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3 g6 3. g3

			variation => N__('Bird: 1... d5 2. Nf3 g6 3. g3'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/6p1/3p4/5P2/5NP1/PPPPP2P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 g6

			variation => N__('Reti: KIA'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppp1pp1p/6p1/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/3p4/8/5NP1/PPPPPPBP/RNBQK2R b KQkq - 1 3' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3 g6 3. Bg2

			variation => N__('Reti: KIA'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/6p1/3p4/8/5NP1/PPPPPPBP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/8/1PPp4/5N2/P2PPPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A09c',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. b4 g6

			variation => N__('Reti: Advance, Anglo-Polish, 3... g6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/8/2Pp4/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A09i',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3 g6

			variation => N__('Reti: Advance, 3. g3 g6'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppp1pp1p/6p1/8/2Pp4/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pp1p/6p1/8/2Pp4/5NP1/PP1PPPBP/RNBQK2R b KQkq - 1 4' => {
			eco => 'A09i',
                        # TRANSLATORS:

			variation => N__('Reti: Advance, 3. g3 g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppp1ppbp/6p1/8/2Pp4/5NP1/PP1PPPBP/RNBQK2R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp1ppp1/7p/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'D00h',
                        # TRANSLATORS: 1. d4 d5 2. Bg5 h6

			variation => N__('Hodgson Attack, 2... h6'),
			moves => {
				'g5h4' => 'rnbqkbnr/ppp1ppp1/7p/3p4/3P3B/8/PPP1PPPP/RN1QKBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1ppp1/7p/3p4/3P3B/8/PPP1PPPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'D00h',
                        # TRANSLATORS:

			variation => N__('Hodgson Attack, 2... h6'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp2ppp1/2p4p/3p4/3P3B/8/PPP1PPPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/6B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A41b',
                        # TRANSLATORS: 1. d4 d6 2. Bg5

			variation => N__('Neo-Old Indian: 2. Bg5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'B07a',
                        # TRANSLATORS: 1. e4 d6 2. c4

			variation => N__('Pirc'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 d6 2. Nc3

			variation => N__('English: 1... d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 d6 2. Nf3

			variation => N__('English: 1... d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 d6

			variation => N__('English: 1... d6'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp1pppp/3p4/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/ppp1pppp/3p4/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A41l',
                        # TRANSLATORS: 1. d4 d6 2. c4

			variation => N__('Neo-Old Indian: 2. c4'),
			moves => {
				'e7e5' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A41d',
                        # TRANSLATORS: 1. d4 d6 2. Nf3

			variation => N__('Neo-Old Indian: 2. Nf3'),
			moves => {
				'c8g4' => 'rn1qkbnr/ppp1pppp/3p4/8/3P2b1/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A41c',
                        # TRANSLATORS: 1. d4 d6 2. g3

			variation => N__('Neo-Old Indian: 2. g3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A41a',
                        # TRANSLATORS: 1. d4 d6

			variation => N__('Neo-Old Indian'),
			moves => {
				'c1g5' => 'rnbqkbnr/ppp1pppp/3p4/6B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2',
				'c2c4' => 'rnbqkbnr/ppp1pppp/3p4/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'g1f3' => 'rnbqkbnr/ppp1pppp/3p4/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/ppp1pppp/3p4/8/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4

			variation => N__('Pirc: 2. d4'),
			moves => {
				'b8d7' => 'r1bqkbnr/pppnpppp/3p4/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
				'c7c6' => 'rnbqkbnr/pp2pppp/2pp4/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/ppp2ppp/3pp3/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'B07a',
                        # TRANSLATORS: 1. e4 d6 2. Nc3

			variation => N__('Pirc: 2. Nc3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'B07b',
                        # TRANSLATORS: 1. e4 d6 2. Nf3

			variation => N__('Pirc: 2. Nf3'),
			moves => {
				'c8g4' => 'rn1qkbnr/ppp1pppp/3p4/8/4P1b1/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/4P3/6P1/PPPP1P1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B07a',
                        # TRANSLATORS: 1. e4 d6 2. g3

			variation => N__('Pirc'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'B07a',
                        # TRANSLATORS: 1. e4 d6

			variation => N__('Pirc'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp1pppp/3p4/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2',
				'c2c4' => 'rnbqkbnr/ppp1pppp/3p4/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2',
				'd2d4' => 'rnbqkbnr/ppp1pppp/3p4/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'f2f4' => 'rnbqkbnr/ppp1pppp/3p4/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2',
				'g1f3' => 'rnbqkbnr/ppp1pppp/3p4/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/ppp1pppp/3p4/8/4P3/6P1/PPPP1P1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'B07c',
                        # TRANSLATORS: 1. e4 d6 2. f4

			variation => N__('Pirc: 2. f4'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/3p1n2/8/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/5P2/8/PPPPP1PP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 d6

			variation => N__('Bird: 1..d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/3p4/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 d6

			variation => N__('Reti: 1... d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3P4/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B01a',
                        # TRANSLATORS: 1. e4 d5 2. exd5

			variation => N__('Scandinavian: 2.exd5'),
			moves => {
				'd8d5' => 'rnb1kbnr/ppp1pppp/8/3q4/8/8/PPPP1PPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/3Pp3/8/8/PPPP1PPP/RNBQKBNR w KQkq e6 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3P4/8/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'D00f',
                        # TRANSLATORS: 1. d4 d5 2. Bg5

			variation => N__('Hodgson Attack (Trompowsky vs. 1... d5)'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp2pppp/2p5/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'c8g4' => 'rn1qkbnr/ppp1pppp/8/3p2B1/3P2b1/8/PPP1PPPP/RN1QKBNR w KQkq - 2 3',
				'f7f6' => 'rnbqkbnr/ppp1p1pp/5p2/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/6p1/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'h7h6' => 'rnbqkbnr/ppp1ppp1/7p/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/1P6/5N2/P1PPPPPP/RNBQKB1R b KQkq b3 0 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b4

			variation => N__('Reti: Santasiere\'s folly'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/1P6/5N2/P1PPPPPP/RNBQKB1R w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/1P6/8/P1PPPPPP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A00q',
                        # TRANSLATORS: 1. b4 d5

			variation => N__('Polish: 1... d5'),
			moves => {
				'c1b2' => 'rnbqkbnr/ppp1pppp/8/3p4/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A00q',
                        # TRANSLATORS: 1. b4 d5 2. Bb2

			variation => N__('Polish: 1... d5 2. Bb2'),
			moves => {
				'c8f5' => 'rn1qkbnr/ppp1pppp/8/3p1b2/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 2 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/2P2P2/8/PP1PP1PP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. c4

			variation => N__('Bird: Sturm Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq c3 0 2' => {
			eco => 'A09a',
                        # TRANSLATORS: 1. Nf3 d5 2. c4

			variation => N__('Reti: 2. c4'),
			moves => {
				'd5c4' => 'rnbqkbnr/ppp1pppp/8/8/2p5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
				'd5d4' => 'rnbqkbnr/ppp1pppp/8/8/2Pp4/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/2P5/8/PP1PPPPP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 d5

			variation => N__('English: Vector'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'D06a',
                        # TRANSLATORS: 1. d4 d5 2. c4

			variation => N__('Queen\'s Gambit'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp1pppp/2n5/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2pp4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq c6 0 3',
				'c7c6' => 'rnbqkbnr/pp2pppp/2p5/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'c8f5' => 'rn1qkbnr/ppp1pppp/8/3p1b2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
				'd5c4' => 'rnbqkbnr/ppp1pppp/8/8/2pP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/3pp3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3',
				'e7e6' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P1B2/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'D00c',
                        # TRANSLATORS: 1. d4 d5 2. Bf4

			variation => N__('Queen\'s Pawn: Mason Variation'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2pp4/3P1B2/8/PPP1PPPP/RN1QKBNR w KQkq c6 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P1P2/8/PPP1P1PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'D00a',
                        # TRANSLATORS: 1. d4 d5 2. f4

			variation => N__('Queen\'s Pawn: 2. f4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P4/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'D00n',
                        # TRANSLATORS: 1. d4 d5 2. Nc3

			variation => N__('Queen\'s Pawn: Veresov Attack'),
			moves => {
				'c8f5' => 'rn1qkbnr/ppp1pppp/8/3p1b2/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P4/2P5/PP2PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'D00b',
                        # TRANSLATORS: 1. d4 d5 2. c3

			variation => N__('Queen\'s Pawn: 2. c3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/2P5/PP2PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P4/4P3/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'D00d',
                        # TRANSLATORS: 1. d4 d5 2. e3

			variation => N__('Queen\'s Pawn: 2. e3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/4P3/PPP2PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'D02a',
                        # TRANSLATORS: 1. d4 d5 2. Nf3

			variation => N__('Queen\'s Pawn: 2. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp1pppp/2n5/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2pp4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq c6 0 3',
				'c7c6' => 'rnbqkbnr/pp2pppp/2p5/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'c8f5' => 'rn1qkbnr/ppp1pppp/8/3p1b2/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3',
				'c8g4' => 'rn1qkbnr/ppp1pppp/8/3p4/3P2b1/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3',
				'e7e6' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/6p1/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'D00a',
                        # TRANSLATORS: 1. d4 d5 2. g3

			variation => N__('Queen\'s Pawn: 2. g3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3P4/8/PPP1PPPP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'D00a',
                        # TRANSLATORS: 1. d4 d5

			variation => N__('Queen\'s Pawn Game'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp1pppp/8/3p4/3P4/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 2',
				'c1f4' => 'rnbqkbnr/ppp1pppp/8/3p4/3P1B2/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2',
				'c1g5' => 'rnbqkbnr/ppp1pppp/8/3p2B1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2',
				'c2c3' => 'rnbqkbnr/ppp1pppp/8/3p4/3P4/2P5/PP2PPPP/RNBQKBNR b KQkq - 0 2',
				'c2c4' => 'rnbqkbnr/ppp1pppp/8/3p4/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'e2e3' => 'rnbqkbnr/ppp1pppp/8/3p4/3P4/4P3/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'e2e4' => 'rnbqkbnr/ppp1pppp/8/3p4/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2',
				'f2f4' => 'rnbqkbnr/ppp1pppp/8/3p4/3P1P2/8/PPP1P1PP/RNBQKBNR b KQkq f3 0 2',
				'g1f3' => 'rnbqkbnr/ppp1pppp/8/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/ppp1pppp/8/3p4/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4

			variation => N__('Blackmar-Diemer Gambit (BDG): 2. e4'),
			moves => {
				'd5e4' => 'rnbqkbnr/ppp1pppp/8/8/3Pp3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/3pp3/3PP3/8/PPP2PPP/RNBQKBNR w KQkq e6 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq e3 0 2' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5 2. e4

			variation => N__('Van Geet: 1... d5 2. e4'),
			moves => {
				'd5d4' => 'rnbqkbnr/ppp1pppp/8/8/3pP3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
				'd5e4' => 'rnbqkbnr/ppp1pppp/8/8/4p3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B01a',
                        # TRANSLATORS: 1. e4 d5 2. d3

			variation => N__('Scandinavian: 2. d3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq e3 0 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. e4

			variation => N__('Reti: Tennison/Zukertort Gambit'),
			moves => {
				'd5e4' => 'rnbqkbnr/ppp1pppp/8/8/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/4P3/8/PPPP1PPP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'B01a',
                        # TRANSLATORS: 1. e4 d5

			variation => N__('Scandinavian (Centre Counter)'),
			moves => {
				'd2d3' => 'rnbqkbnr/ppp1pppp/8/3p4/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'e4d5' => 'rnbqkbnr/ppp1pppp/8/3P4/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2',
				'e4e5' => 'rnbqkbnr/ppp1pppp/8/3pP3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/4PP2/8/PPPP2PP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. e4

			variation => N__('Bird: Williams Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/5P2/1P6/P1PPP1PP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. b3

			variation => N__('Bird: 1... d5 2. b3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/1P6/P1PPP1PP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/5P2/4P3/PPPP2PP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. e3

			variation => N__('Bird: Lasker Variation'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/4P3/PPPP2PP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. Nf3

			variation => N__('Bird: 1... d5 2. Nf3'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2pp4/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq c6 0 3',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/6p1/3p4/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/5P2/6P1/PPPPP2P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. g3

			variation => N__('Bird: 1... d5 2. g3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/5P2/6P1/PPPPP2P/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/5P2/8/PPPPP1PP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5

			variation => N__('Bird: 1... d5'),
			moves => {
				'b2b3' => 'rnbqkbnr/ppp1pppp/8/3p4/5P2/1P6/P1PPP1PP/RNBQKBNR b KQkq - 0 2',
				'c2c4' => 'rnbqkbnr/ppp1pppp/8/3p4/2P2P2/8/PP1PP1PP/RNBQKBNR b KQkq c3 0 2',
				'e2e3' => 'rnbqkbnr/ppp1pppp/8/3p4/5P2/4P3/PPPP2PP/RNBQKBNR b KQkq - 0 2',
				'e2e4' => 'rnbqkbnr/ppp1pppp/8/3p4/4PP2/8/PPPP2PP/RNBQKBNR b KQkq e3 0 2',
				'g1f3' => 'rnbqkbnr/ppp1pppp/8/3p4/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/ppp1pppp/8/3p4/5P2/6P1/PPPPP2P/RNBQKBNR b KQkq - 0 2',
				'g2g4' => 'rnbqkbnr/ppp1pppp/8/3p4/5PP1/8/PPPPP2P/RNBQKBNR b KQkq g3 0 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/5PP1/8/PPPPP2P/RNBQKBNR b KQkq g3 0 2' => {
			eco => 'A03',
                        # TRANSLATORS: 1. f4 d5 2. g4

			variation => N__('Bird: Dudweiler Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/6P1/8/PPPPPP1P/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5

			variation => N__('Grob: 1... d5'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppp1pppp/8/3p4/6P1/8/PPPPPPBP/RNBQK1NR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/6P1/8/PPPPPPBP/RNBQK1NR b KQkq - 1 2' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2

			variation => N__('Grob Gambit'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp2pppp/2p5/3p4/6P1/8/PPPPPPBP/RNBQK1NR w KQkq - 0 3',
				'c8g4' => 'rn1qkbnr/ppp1pppp/8/3p4/6b1/8/PPPPPPBP/RNBQK1NR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/3pp3/6P1/8/PPPPPPBP/RNBQK1NR w KQkq e6 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/1P3N2/P1PPPPPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. b3

			variation => N__('Reti: Nimzowitsch-Larsen'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2pp4/8/1P3N2/P1PPPPPP/RNBQKB1R w KQkq c6 0 3',
				'c8g4' => 'rn1qkbnr/ppp1pppp/8/3p4/6b1/1P3N2/P1PPPPPP/RNBQKB1R w KQkq - 1 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/1P3N2/P1PPPPPP/RNBQKB1R w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/1P6/P1PPPPPP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 d5

			variation => N__('Nimzowitsch-Larsen: 1... d5'),
			moves => {
				'c1b2' => 'rnbqkbnr/ppp1pppp/8/3p4/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 d5 2. Bb2

			variation => N__('Nimzowitsch-Larsen: 1... d5 2. Bb2'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 1 2' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5 2. Nf3

			variation => N__('Van Geet: 1... d5 2. Nf3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/2N2N2/PPPPPPPP/R1BQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/2N5/PPPPPPPP/R1BQKBNR w KQkq d6 0 2' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5

			variation => N__('Van Geet: 1... d5'),
			moves => {
				'e2e4' => 'rnbqkbnr/ppp1pppp/8/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq e3 0 2',
				'g1f3' => 'rnbqkbnr/ppp1pppp/8/3p4/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/3P1N2/PPP1PPPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. d3

			variation => N__('Reti: Old Indian Attack'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/3P1N2/PPP1PPPP/RNBQKB1R w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/3P4/PPP1PPPP/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A00j',
                        # TRANSLATORS: 1. d3 d5

			variation => N__('Mieses: 1... d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/4PN2/PPPP1PPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. e3

			variation => N__('Reti: 1... d5 2. e3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/5N2/PPPPPPPP/RNBQKB1R w KQkq d6 0 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5

			variation => N__('Reti: 1... d5'),
			moves => {
				'b2b3' => 'rnbqkbnr/ppp1pppp/8/3p4/8/1P3N2/P1PPPPPP/RNBQKB1R b KQkq - 0 2',
				'b2b4' => 'rnbqkbnr/ppp1pppp/8/3p4/1P6/5N2/P1PPPPPP/RNBQKB1R b KQkq b3 0 2',
				'c2c4' => 'rnbqkbnr/ppp1pppp/8/3p4/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq c3 0 2',
				'd2d3' => 'rnbqkbnr/ppp1pppp/8/3p4/8/3P1N2/PPP1PPPP/RNBQKB1R b KQkq - 0 2',
				'e2e3' => 'rnbqkbnr/ppp1pppp/8/3p4/8/4PN2/PPPP1PPP/RNBQKB1R b KQkq - 0 2',
				'e2e4' => 'rnbqkbnr/ppp1pppp/8/3p4/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq e3 0 2',
				'g2g3' => 'rnbqkbnr/ppp1pppp/8/3p4/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
				'h1g1' => 'rnbqkbnr/ppp1pppp/8/3p4/8/5N2/PPPPPPPP/RNBQKBR1 b Qkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/5N2/PPPPPPPP/RNBQKBR1 b Qkq - 1 2' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. Rg1

			variation => N__('Reti: Ampel Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A07',
                        # TRANSLATORS: 1. Nf3 d5 2. g3

			variation => N__('Reti: KIA'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp1pppp/2n5/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2pp4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq c6 0 3',
				'c7c6' => 'rnbqkbnr/pp2pppp/2p5/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3',
				'c8g4' => 'rn1qkbnr/ppp1pppp/8/3p4/6b1/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/6p1/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/6P1/PPPPPP1P/RNBQKBNR w KQkq d6 0 2' => {
			eco => 'A00v',
                        # TRANSLATORS: 1. g3 d5

			variation => N__('Benko Opening'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppp1pppp/8/3p4/8/6P1/PPPPPPBP/RNBQK1NR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/6P1/PPPPPPBP/RNBQK1NR b KQkq - 1 2' => {
			eco => 'A00v',
                        # TRANSLATORS: 1. g3 d5 2. Bg2

			variation => N__('Benko Opening'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp2pppp/2p5/3p4/8/6P1/PPPPPPBP/RNBQK1NR w KQkq - 0 3',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/3pp3/8/6P1/PPPPPPBP/RNBQK1NR w KQkq e6 0 3',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/3p4/8/6P1/PPPPPPBP/RNBQK1NR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/6PN/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A00g',
                        # TRANSLATORS:

			variation => N__('Amar/Paris Opening'),
			moves => {
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/3pp3/8/6PN/PPPPPP1P/RNBQKB1R w KQkq e6 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3p4/8/7N/PPPPPPPP/RNBQKB1R w KQkq d6 0 2' => {
			eco => 'A00g',
                        # TRANSLATORS:

			variation => N__('Amar/Paris Opening'),
			moves => {
				'g2g3' => 'rnbqkbnr/ppp1pppp/8/3p4/8/6PN/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppp1pppp/8/3pP3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'B01a',
                        # TRANSLATORS: 1. e4 d5 2. e5

			variation => N__('Scandinavian: 2. e5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/1PPp4/5N2/P2PPPPP/RNBQKB1R b KQkq b3 0 3' => {
			eco => 'A09c',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. b4

			variation => N__('Reti: Advance, Anglo-Polish Attack'),
			moves => {
				'g7g6' => 'rnbqkbnr/ppp1pp1p/6p1/8/1PPp4/5N2/P2PPPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2B1p3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 1 3' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5 2. e4 dxe4 3. Bc4

			variation => N__('Van Geet: Hector Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2BPp3/8/PPP2PPP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Bc4

			variation => N__('Blackmar-Diemer: Fritz Attack'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2Pp4/4PN2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A09d',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. e3

			variation => N__('Reti: Advance, 3. e3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp1pppp/2n5/8/2Pp4/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2Pp4/4PN2/PP1P1PPP/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2Pp4/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A09b',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4

			variation => N__('Reti: Advance Variation'),
			moves => {
				'b2b4' => 'rnbqkbnr/ppp1pppp/8/8/1PPp4/5N2/P2PPPPP/RNBQKB1R b KQkq b3 0 3',
				'e2e3' => 'rnbqkbnr/ppp1pppp/8/8/2Pp4/4PN2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkbnr/ppp1pppp/8/8/2Pp4/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2Pp4/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A09g',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 d4 3. g3

			variation => N__('Reti: Advance, 3. g3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp1pppp/2n5/8/2Pp4/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2Pp4/5NP1/PP1PPP1P/RNBQKB1R w KQkq c6 0 4',
				'g7g6' => 'rnbqkbnr/ppp1pp1p/6p1/8/2Pp4/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2p5/4PN2/PP1P1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A09r',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. e3

			variation => N__('Reti: Accepted, 3. e3'),
			moves => {
				'c8e6' => 'rn1qkbnr/ppp1pppp/4b3/8/2p5/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/8/2p5/4PN2/PP1P1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2p5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A09k',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4

			variation => N__('Reti: Accepted'),
			moves => {
				'b1a3' => 'rnbqkbnr/ppp1pppp/8/8/2p5/N4N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3',
				'd1a4' => 'rnbqkbnr/ppp1pppp/8/8/Q1p5/5N2/PP1PPPPP/RNB1KB1R b KQkq - 1 3',
				'e2e3' => 'rnbqkbnr/ppp1pppp/8/8/2p5/4PN2/PP1P1PPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkbnr/ppp1pppp/8/8/2p5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2p5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A09l',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. g3

			variation => N__('Reti: Accepted, 3. g3'),
			moves => {
				'e7e6' => 'rnbqkbnr/ppp2ppp/4p3/8/2p5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2p5/N4N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A09o',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. Na3

			variation => N__('Reti: Accepted, 3. Na3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1pp1pppp/p7/8/2p5/N4N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2p5/N4N2/PP1PPPPP/R1BQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2pP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'D20i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3

			variation => N__('QGA: 3. Nc3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1pp1pppp/p7/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/4p3/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4',
				'e7e6' => 'rnbqkbnr/ppp2ppp/4p3/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2pP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D20b',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3

			variation => N__('QGA: 3. e3'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq c6 0 4',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/4p3/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq e6 0 4',
				'e7e6' => 'rnbqkbnr/ppp2ppp/4p3/8/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/8/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2pP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'D21a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3

			variation => N__('QGA: 3. Nf3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1pp1pppp/p7/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'b7b5' => 'rnbqkbnr/p1p1pppp/8/1p6/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq b6 0 4',
				'b8d7' => 'r1bqkbnr/pppnpppp/8/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4',
				'c8g4' => 'rn1qkbnr/ppp1pppp/8/8/2pP2b1/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'e7e6' => 'rnbqkbnr/ppp2ppp/4p3/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2pP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'D20a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4

			variation => N__('Queen\'s Gambit Accepted (QGA)'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp1pppp/8/8/2pP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'd1a4' => 'rnbqkbnr/ppp1pppp/8/8/Q1pP4/8/PP2PPPP/RNB1KBNR b KQkq - 1 3',
				'e2e3' => 'rnbqkbnr/ppp1pppp/8/8/2pP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'e2e4' => 'rnbqkbnr/ppp1pppp/8/8/2pPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkbnr/ppp1pppp/8/8/2pP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/2pPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'D20l',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4

			variation => N__('QGA: 3. e4'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp1pppp/2n5/8/2pPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4',
				'c7c5' => 'rnbqkbnr/pp2pppp/8/2p5/2pPP3/8/PP3PPP/RNBQKBNR w KQkq c6 0 4',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/4p3/2pPP3/8/PP3PPP/RNBQKBNR w KQkq e6 0 4',
				'f7f5' => 'rnbqkbnr/ppp1p1pp/8/5p2/2pPP3/8/PP3PPP/RNBQKBNR w KQkq f6 0 4',
				'g8f6' => 'rnbqkb1r/ppp1pppp/5n2/8/2pPP3/8/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/3Pp3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3

			variation => N__('Blackmar-Diemer: 2. e4 dxe4 3. Nc3'),
			moves => {
				'c8d7' => 'rn1qkbnr/pppbpppp/8/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
				'c8f5' => 'rn1qkbnr/ppp1pppp/8/5b2/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
				'e7e5' => 'rnbqkbnr/ppp2ppp/8/4p3/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 4',
				'f7f5' => 'rnbqkbnr/ppp1p1pp/8/5p2/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq f6 0 4',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/3Pp3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. f3

			variation => N__('Blackmar-Diemer: Gedult Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/3Pp3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4

			variation => N__('Blackmar-Diemer Gambit (BDG): 2. e4 dxe4'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp1pppp/8/8/3Pp3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3',
				'f1c4' => 'rnbqkbnr/ppp1pppp/8/8/2BPp3/8/PPP2PPP/RNBQK1NR b KQkq - 1 3',
				'f2f3' => 'rnbqkbnr/ppp1pppp/8/8/3Pp3/5P2/PPP3PP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/3pP3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5 2. e4 d4

			variation => N__('Van Geet: 1... d5 2. e4 d4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/4p3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A00m',
                        # TRANSLATORS: 1. Nc3 d5 2. e4 dxe4

			variation => N__('Van Geet: 1... d5 2. e4 dxe4'),
			moves => {
				'f1c4' => 'rnbqkbnr/ppp1pppp/8/8/2B1p3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A06',
                        # TRANSLATORS: 1. Nf3 d5 2. e4 dxe4

			variation => N__('Reti: Tennison Gambit Accepted'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/8/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 4' => {
			eco => 'B01c',
                        # TRANSLATORS: 1. e4 d5 2. exd5 Qxd5 3. Nc3 Qd8

			variation => N__('Scandinavian: 2...Qxd5 3. Nc3 Qd8'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/Q1p5/5N2/PP1PPPPP/RNB1KB1R b KQkq - 1 3' => {
			eco => 'A09n',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. Qa4+

			variation => N__('Reti: Accepted, 3. Qa4+'),
			moves => {
			},
		},
		'rnbqkbnr/ppp1pppp/8/8/Q1pP4/8/PP2PPPP/RNB1KBNR b KQkq - 1 3' => {
			eco => 'D20a',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Qa4+

			variation => N__('QGA: 3. Qa4+'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2p1p/3p2p1/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 g6

			variation => N__('English: King\'s, 2. Nc3 d6 3. Nf3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2p1p/3p2p1/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C41n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 g6

			variation => N__('Philidor: Larsen Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2p1p/3p4/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 d6

			variation => N__('KGA: Kieseritsky, Green Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2p1p/3p4/6p1/3PPp1P/5N2/PPP3P1/RNBQKB1R b KQkq h3 0 5' => {
			eco => 'C34',
                        # TRANSLATORS:

			variation => N__('KGA: Fischer, 4. d4'),
			moves => {
				'g5g4' => 'rnbqkbnr/ppp2p1p/3p4/8/3PPppP/5N2/PPP3P1/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/ppp2p1p/3p4/6p1/3PPp2/5N2/PPP3PP/RNBQKB1R w KQkq g6 0 5' => {
			eco => 'C34',
                        # TRANSLATORS:

			variation => N__('KGA: Fischer, 4. d4'),
			moves => {
				'h2h4' => 'rnbqkbnr/ppp2p1p/3p4/6p1/3PPp1P/5N2/PPP3P1/RNBQKB1R b KQkq h3 0 5',
			},
		},
		'rnbqkbnr/ppp2p1p/3p4/8/3PPppP/5N2/PPP3P1/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C34',
                        # TRANSLATORS:

			variation => N__('KGA: Fischer, 4. d4'),
			moves => {
				'f3g1' => 'rnbqkbnr/ppp2p1p/3p4/8/3PPppP/8/PPP3P1/RNBQKBNR b KQkq - 1 6',
			},
		},
		'rnbqkbnr/ppp2p1p/3p4/8/3PPppP/8/PPP3P1/RNBQKBNR b KQkq - 1 6' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d6 4. d4 g5 5. h4 g4 6. Ng1

			variation => N__('KGA: Fischer, Main Line'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2p1p/8/3p4/2B1Ppp1/5N2/PPPP2PP/RNBQ1RK1 w kq d6 0 6' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O d5

			variation => N__('KGA: Muzio Gambit, Brentano Defence'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2p1p/8/3pN3/3PPppP/8/PPP3P1/RNBQKB1R b KQkq d3 0 6' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp2p1p/5n2/3pN3/3PPppP/8/PPP3P1/RNBQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkbnr/ppp2p1p/8/3pN3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq d6 0 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 d5

			variation => N__('KGA: Kieseritsky, Brentano (Campbell) Defence'),
			moves => {
				'd2d4' => 'rnbqkbnr/ppp2p1p/8/3pN3/3PPppP/8/PPP3P1/RNBQKB1R b KQkq d3 0 6',
			},
		},
		'rnbqkbnr/ppp2ppp/3P4/8/8/8/PPPPP1PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6 3. exd6

			variation => N__('Bird: From Gambit, 3.exd6'),
			moves => {
				'f8d6' => 'rnbqk1nr/ppp2ppp/3b4/8/8/8/PPPPP1PP/RNBQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/3P1n2/8/8/8/PPPPP1PP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/3Pp3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A41n',
                        # TRANSLATORS: 1. d4 d6 2. c4 e5 3. d5

			variation => N__('Neo-Old Indian: 2. c4 e5 3. d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4P3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'A41o',
                        # TRANSLATORS: 1. d4 d6 2. c4 e5 3. dxe5

			variation => N__('Neo-Old Indian: 2. c4 e5 3.dxe5'),
			moves => {
				'd6e5' => 'rnbqkbnr/ppp2ppp/8/4p3/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4P3/4P3/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 d6 3. dxe5

			variation => N__('Centre Game: Maroczy Defence, 3.dxe5'),
			moves => {
				'c8d7' => 'rn1qkbnr/pppb1ppp/3p4/4P3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
				'd6e5' => 'rnbqkbnr/ppp2ppp/8/4p3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4P3/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 d6

			variation => N__('Englund Gambit: Hartlaub'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4P3/8/8/PPPPP1PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5 d6

			variation => N__('Bird: From Gambit, 2... d6'),
			moves => {
				'e5d6' => 'rnbqkbnr/ppp2ppp/3P4/8/8/8/PPPPP1PP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 d6

			variation => N__('Polish: 1... e5 2. Bb2 d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 1 3' => {
			eco => 'C25c',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 d6 3. Bc4

			variation => N__('Vienna: 2... d6 3. Bc4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3' => {
			eco => 'C41b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. Bc4

			variation => N__('Philidor: 3. Bc4'),
			moves => {
				'f7f5' => 'rnbqkbnr/ppp3pp/3p4/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 4',
				'f8e7' => 'rnbqk1nr/ppp1bppp/3p4/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3

			variation => N__('English: King\'s, 2. Nc3 d6 3. Nf3'),
			moves => {
				'c8g4' => 'rn1qkbnr/ppp2ppp/3p4/4p3/2P3b1/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 2 4',
				'f7f5' => 'rnbqkbnr/ppp3pp/3p4/4pp2/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq f6 0 4',
				'g7g6' => 'rnbqkbnr/ppp2p1p/3p2p1/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. g3

			variation => N__('English: King\'s, 2. Nc3 d6 3. g3'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp3ppp/2pp4/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6

			variation => N__('English: King\'s, 2. Nc3 d6'),
			moves => {
				'd2d4' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq d3 0 3',
				'g1f3' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/2N2N2/PP1PPPPP/R1BQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 d6

			variation => N__('English: King\'s, 2. g3 d6'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 d6 3. Bg2

			variation => N__('English: King\'s, 2. g3 d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq d3 0 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. d4

			variation => N__('English: King\'s, 2. Nc3 d6 3. d4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A41p',
                        # TRANSLATORS: 1. d4 d6 2. c4 e5 3. Nf3

			variation => N__('Neo-Old Indian: 2. c4 e5 3. Nf3'),
			moves => {
				'e5e4' => 'rnbqkbnr/ppp2ppp/3p4/8/2PPp3/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'A41m',
                        # TRANSLATORS: 1. d4 d6 2. c4 e5

			variation => N__('Neo-Old Indian: 2. c4 e5'),
			moves => {
				'd4d5' => 'rnbqkbnr/ppp2ppp/3p4/3Pp3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'd4e5' => 'rnbqkbnr/ppp2ppp/3p4/4P3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'C41c',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4

			variation => N__('Philidor: 3. d4'),
			moves => {
				'b8d7' => 'r1bqkbnr/pppn1ppp/3p4/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
				'e5d4' => 'rnbqkbnr/ppp2ppp/3p4/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
				'f7f5' => 'rnbqkbnr/ppp3pp/3p4/4pp2/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq f6 0 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/3p1n2/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 d6

			variation => N__('Centre Game: Maroczy Defence'),
			moves => {
				'd4e5' => 'rnbqkbnr/ppp2ppp/3p4/4P3/4P3/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'C25c',
                        # TRANSLATORS: 1. e4 e5 2. Nc3 d6

			variation => N__('Vienna: 2... d6'),
			moves => {
				'f1c4' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2B1P3/2N5/PPPP1PPP/R1BQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'C41a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6

			variation => N__('Philidor Defence'),
			moves => {
				'd2d4' => 'rnbqkbnr/ppp2ppp/3p4/4p3/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'f1c4' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'C30c',
                        # TRANSLATORS: 1. e4 e5 2. f4 d6 3. Nf3

			variation => N__('King\'s Gambit: 2... d6 3. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp2ppp/2np4/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'C30c',
                        # TRANSLATORS: 1. e4 e5 2. f4 d6

			variation => N__('King\'s Gambit: 2... d6'),
			moves => {
				'g1f3' => 'rnbqkbnr/ppp2ppp/3p4/4p3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/4p3/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 e5 2. Bb2 d6

			variation => N__('Nimzowitsch-Larsen: 1... e5 2. Bb2 d6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/2B1Pp2/5N2/PPPP2PP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d6 4. Bc4

			variation => N__('KGA: Fischer, 4. Bc4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/2PPp3/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A41p',
                        # TRANSLATORS: 1. d4 d6 2. c4 e5 3. Nf3 e4

			variation => N__('Neo-Old Indian: 2. c4 e5 3. Nf3 e4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C41j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4

			variation => N__('Philidor: 3...exd4 4.Nxd4'),
			moves => {
				'd6d5' => 'rnbqkbnr/ppp2ppp/8/3p4/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'g7g6' => 'rnbqkbnr/ppp2p1p/3p2p1/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/3p1n2/8/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/3PPp2/5N2/PPP3PP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d6 4. d4

			variation => N__('KGA: Fischer, 4. d4'),
			moves => {
				'g7g5' => 'rnbqkbnr/ppp2p1p/3p4/6p1/3PPp2/5N2/PPP3PP/RNBQKB1R w KQkq g6 0 5',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 4' => {
			eco => 'C41i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Qxd4

			variation => N__('Philidor: Morphy Variation'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp2ppp/2np4/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/3p1n2/8/3QP3/5N2/PPP2PPP/RNB1KB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/3pP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C41h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. c3

			variation => N__('Philidor: Bird Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C41h',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4

			variation => N__('Philidor: 3...exd4'),
			moves => {
				'c2c3' => 'rnbqkbnr/ppp2ppp/3p4/8/3pP3/2P2N2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'd1d4' => 'rnbqkbnr/ppp2ppp/3p4/8/3QP3/5N2/PPP2PPP/RNB1KB1R b KQkq - 0 4',
				'f3d4' => 'rnbqkbnr/ppp2ppp/3p4/8/3NP3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3p4/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d6

			variation => N__('KGA: Fischer Defence'),
			moves => {
				'd2d4' => 'rnbqkbnr/ppp2ppp/3p4/8/3PPp2/5N2/PPP3PP/RNBQKB1R b KQkq d3 0 4',
				'f1c4' => 'rnbqkbnr/ppp2ppp/3p4/8/2B1Pp2/5N2/PPPP2PP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/3pp3/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'B07d',
                        # TRANSLATORS: 1. e4 d6 2. d4 e6

			variation => N__('Pirc: Lengfellner System'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D30d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. cxd5

			variation => N__('QGD: 3.cxd5'),
			moves => {
				'e6d5' => 'rnbqkbnr/ppp2ppp/8/3p4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C01a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5

			variation => N__('French: Exchange'),
			moves => {
				'd8d5' => 'rnb1kbnr/ppp2ppp/4p3/3q4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
				'e6d5' => 'rnbqkbnr/ppp2ppp/8/3p4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3P4/4P3/8/PP1P1PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C00e',
                        # TRANSLATORS:

			variation => N__('French: Steiner (Anglo-French) Variation'),
			moves => {
				'e6d5' => 'rnbqkbnr/ppp2ppp/8/3p4/4P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'C00e',
                        # TRANSLATORS: 1. e4 e6 2. c4 d5

			variation => N__('French: Steiner (Anglo-French) Variation'),
			moves => {
				'c4d5' => 'rnbqkbnr/ppp2ppp/4p3/3P4/4P3/8/PP1P1PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A13m',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. b3

			variation => N__('English: Agincourt Variation'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'A13g',
                        # TRANSLATORS: 1. c4 e6 2. Nc3 d5

			variation => N__('English: 1... e6 2. Nc3 d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq d6 0 3' => {
			eco => 'A13l',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5

			variation => N__('English: 1... e6 2. Nf3 d5'),
			moves => {
				'b2b3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/1P3N2/P2PPPPP/RNBQKB1R b KQkq - 0 3',
				'g2g3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A13o',
                        # TRANSLATORS: 1. c4 e6 2. Nf3 d5 3. g3

			variation => N__('English: Agincourt Variation'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2P5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'A13c',
                        # TRANSLATORS: 1. c4 e6 2. g3 d5

			variation => N__('English: 1... e6 2. g3 d5'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A13d',
                        # TRANSLATORS: 1. c4 e6 2. g3 d5 3. Bg2

			variation => N__('English: 1... e6 2. g3 d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PP1B2/8/PP2PPPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'D30e',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Bf4

			variation => N__('QGD: 3. Bf4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'D31a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3

			variation => N__('QGD: 3. Nc3'),
			moves => {
				'a7a6' => 'rnbqkbnr/1pp2ppp/p3p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'b7b6' => 'rnbqkbnr/p1p2ppp/1p2p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'b8c6' => 'r1bqkbnr/ppp2ppp/2n1p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq c6 0 4',
				'c7c6' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4',
				'f8b4' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1bPP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
				'f8e7' => 'rnbqk1nr/ppp1bppp/4p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D30a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. e3

			variation => N__('QGD: 3. e3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'D30f',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nf3

			variation => N__('QGD: 3. Nf3'),
			moves => {
				'b8d7' => 'r1bqkbnr/pppn1ppp/4p3/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq c6 0 4',
				'c7c6' => 'rnbqkbnr/pp3ppp/2p1p3/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D30c',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. g3

			variation => N__('QGD: 3. g3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'D30a',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6

			variation => N__('Queen\'s Gambit Declined (QGD)'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'c1f4' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PP1B2/8/PP2PPPP/RN1QKBNR b KQkq - 1 3',
				'c4d5' => 'rnbqkbnr/ppp2ppp/4p3/3P4/3P4/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'e2e4' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
				'g2g3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PP4/6P1/PP2PP1P/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PPP3/8/PP3PPP/RNBQKBNR b KQkq c3 0 3' => {
			eco => 'C00x',
                        # TRANSLATORS:

			variation => N__('French: 2. d4 d5'),
			moves => {
				'd5e4' => 'rnbqkbnr/ppp2ppp/4p3/8/2PPp3/8/PP3PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. e4

			variation => N__('Diemer-Duhm Gambit (DDG)'),
			moves => {
				'd5e4' => 'rnbqkbnr/ppp2ppp/4p3/8/2PPp3/8/PP3PPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'D02h',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 e6

			variation => N__('Queen\'s Pawn: 2. Nf3 e6'),
			moves => {
				'g2g3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 0 3' => {
			eco => 'D02i',
                        # TRANSLATORS: 1. d4 d5 2. Nf3 e6 3. g3

			variation => N__('Queen\'s Pawn: 2. Nf3 e6 3. g3'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'C10a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3

			variation => N__('French: 3. Nc3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq c6 0 4',
				'd5e4' => 'rnbqkbnr/ppp2ppp/4p3/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'f8b4' => 'rnbqk1nr/ppp2ppp/4p3/3p4/1b1PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
				'f8e7' => 'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'C00y',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Bd3

			variation => N__('French: Schlechter'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/4B3/PPP2PPP/RN1QKBNR b KQkq - 1 3' => {
			eco => 'C00x',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Be3

			variation => N__('French: Alapin-Diemer Gambit (ADG)'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'C00x',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5

			variation => N__('French: 2. d4 d5'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3',
				'b1d2' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR b KQkq - 1 3',
				'c1e3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/4B3/PPP2PPP/RN1QKBNR b KQkq - 1 3',
				'c2c4' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2PPP3/8/PP3PPP/RNBQKBNR b KQkq c3 0 3',
				'e4d5' => 'rnbqkbnr/ppp2ppp/4p3/3P4/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
				'e4e5' => 'rnbqkbnr/ppp2ppp/4p3/3pP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3',
				'f1d3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'C03a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2

			variation => N__('French: Tarrasch'),
			moves => {
				'a7a6' => 'rnbqkbnr/1pp2ppp/p3p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
				'b7b6' => 'rnbqkbnr/p1p2ppp/1p2p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
				'b8c6' => 'r1bqkbnr/ppp2ppp/2n1p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4',
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq c6 0 4',
				'd5e4' => 'rnbqkbnr/ppp2ppp/4p3/8/3Pp3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4',
				'f7f5' => 'rnbqkbnr/ppp3pp/4p3/3p1p2/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq f6 0 4',
				'f8e7' => 'rnbqk1nr/ppp1bppp/4p3/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'C00l',
                        # TRANSLATORS: 1. e4 e6 2. Nf3 d5 3. Nc3

			variation => N__('French: Two Knights Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq d6 0 3' => {
			eco => 'C00f',
                        # TRANSLATORS: 1. e4 e6 2. Nc3 d5

			variation => N__('French: 2. Nc3 d5'),
			moves => {
				'f2f4' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/3P4/PPP1QPPP/RNB1KBNR b KQkq - 1 3' => {
			eco => 'C00n',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Qe2

			variation => N__('French: KIA 2. d3 d5 3. Qe2'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/4P3/3P4/PPP1QPPP/RNB1KBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'C00m',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5

			variation => N__('French: KIA 2. d3 d5'),
			moves => {
				'b1d2' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR b KQkq - 1 3',
				'd1e2' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/3P4/PPP1QPPP/RNB1KBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'C00p',
                        # TRANSLATORS: 1. e4 e6 2. d3 d5 3. Nd2

			variation => N__('French: KIA 2. d3 d5 3. Nd2'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2pp4/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq c6 0 4',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/3p4/4P3/3P4/PPPN1PPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq d6 0 3' => {
			eco => 'C00j',
                        # TRANSLATORS: 1. e4 e6 2. Nf3 d5

			variation => N__('French: 2. Nf3 d5'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3',
				'e4e5' => 'rnbqkbnr/ppp2ppp/4p3/3pP3/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3p4/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq f3 0 3' => {
			eco => 'C00f',
                        # TRANSLATORS: 1. e4 e6 2. Nc3 d5 3. f4

			variation => N__('French: Pelikan Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3pP3/3P4/8/PPP2PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C02a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. e5

			variation => N__('French: Advance'),
			moves => {
				'b7b6' => 'rnbqkbnr/p1p2ppp/1p2p3/3pP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4',
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq c6 0 4',
				'g8e7' => 'rnbqkb1r/ppp1nppp/4p3/3pP3/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/3pP3/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C00j',
                        # TRANSLATORS: 1. e4 e6 2. Nf3 d5 3. e5

			variation => N__('French: 2. Nf3 d5 3. e5'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2ppP3/8/5N2/PPPP1PPP/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2BP4/4P3/PP3PPP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'D20d',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e3 e6'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/8/2BP4/4P3/PP3PPP/RNBQK1NR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2PPp3/2N5/PP3PPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'D30b',
                        # TRANSLATORS:

			variation => N__('Diemer-Duhm Gambit (DDG) Accepted'),
			moves => {
				'f7f5' => 'rnbqkbnr/ppp3pp/4p3/5p2/2PPp3/2N5/PP3PPP/R1BQKBNR w KQkq f6 0 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/8/2PPp3/2N5/PP3PPP/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2PPp3/8/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. e4 dxe4

			variation => N__('Diemer-Duhm Gambit (DDG) Accepted'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/4p3/8/2PPp3/2N5/PP3PPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2p5/5NP1/PP1PPP1P/RNBQKB1R w KQkq - 0 4' => {
			eco => 'A09m',
                        # TRANSLATORS: 1. Nf3 d5 2. c4 dxc4 3. g3 e6

			variation => N__('Reti: Accepted, 3. g3 e6'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'D20j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 e6

			variation => N__('QGA: 3. Nc3 e6'),
			moves => {
				'e2e4' => 'rnbqkbnr/ppp2ppp/4p3/8/2pPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D20d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e6

			variation => N__('QGA: 3. e3 e6'),
			moves => {
				'f1c4' => 'rnbqkbnr/ppp2ppp/4p3/8/2BP4/4P3/PP3PPP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'D21g',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 e6 4. e3

			variation => N__('QGA: 3. Nf3 e6 4. e3'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp3ppp/4p3/2p5/2pP4/4PN2/PP3PPP/RNBQKB1R w KQkq c6 0 5',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2pP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'D21d',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 e6

			variation => N__('QGA: 3. Nf3 e6'),
			moves => {
				'd1a4' => 'rnbqkbnr/ppp2ppp/4p3/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R b KQkq - 1 4',
				'e2e3' => 'rnbqkbnr/ppp2ppp/4p3/8/2pP4/4PN2/PP3PPP/RNBQKB1R b KQkq - 0 4',
				'e2e4' => 'rnbqkbnr/ppp2ppp/4p3/8/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2pPP3/2N5/PP3PPP/R1BQKBNR b KQkq e3 0 4' => {
			eco => 'D20j',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 e6 4. e4

			variation => N__('QGA: 3. Nc3 e6 4. e4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq e3 0 4' => {
			eco => 'D21f',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 e6 4. e4

			variation => N__('QGA: 3. Nf3 e6 4. e4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/3PN3/8/PPP2PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'C10f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4

			variation => N__('French: Rubinstein'),
			moves => {
				'b8d7' => 'r1bqkbnr/pppn1ppp/4p3/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
				'c8d7' => 'rn1qkbnr/pppb1ppp/4p3/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
				'd8d5' => 'rnb1kbnr/ppp2ppp/4p3/3q4/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
				'e6e5' => 'rnbqkbnr/ppp2ppp/8/4p3/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/4pn2/8/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C10f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4

			variation => N__('French: Rubinstein'),
			moves => {
				'c3e4' => 'rnbqkbnr/ppp2ppp/4p3/8/3PN3/8/PPP2PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/3Pp3/8/PPPN1PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'C03a',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 dxe4

			variation => N__('French: Tarrasch, 3...dxe4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/4p3/8/Q1pP4/5N2/PP2PPPP/RNB1KB1R b KQkq - 1 4' => {
			eco => 'D21e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 e6 4. Qa4+

			variation => N__('QGA: 3. Nf3 e6 4. Qa4+'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/1B1P4/4pP2/8/PPPP2PP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. Bb5+

			variation => N__('KGD: Falkbeer, Nimzowitsch Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3B4/4Pp2/8/PPPP2PP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 d5 4. Bxd5

			variation => N__('KGA: Bishop\'s Gambit, Bledow, 4.Bxd5'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp3ppp/2p5/3B4/4Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 0 5',
				'd8h4' => 'rnb1kbnr/ppp2ppp/8/3B4/4Pp1q/8/PPPP2PP/RNBQK1NR w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3B4/4Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/2B2p2/8/PPPP2PP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 d5 4. exd5

			variation => N__('KGA: Bishop\'s Gambit, Bledow, 4.exd5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/3N4/8/PPP2PPP/RNBQKB1R b KQkq - 0 5' => {
			eco => 'C41j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 exd4 4. Nxd4 d5 5. exd5

			variation => N__('Philidor: Paulsen Attack'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C40i',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d5 3. exd5 e4

			variation => N__('Elephant Gambit: Paulsen'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/4pP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'C31',
                        # TRANSLATORS:

			variation => N__('KGD: Falkbeer, 3.exd5 e4'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/4pP2/3P4/PPP3PP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4 4. d3

			variation => N__('KGD: Falkbeer, 4. d3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3P4/4pP2/3P4/PPP3PP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/4pP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 e4

			variation => N__('KGD: Falkbeer, 3.exd5 e4'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/8/3P4/4pP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 4',
				'd2d3' => 'rnbqkbnr/ppp2ppp/8/3P4/4pP2/3P4/PPP3PP/RNBQKBNR b KQkq - 0 4',
				'f1b5' => 'rnbqkbnr/ppp2ppp/8/1B1P4/4pP2/8/PPPP2PP/RNBQK1NR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/5p2/5N2/PPPP2PP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5 4. exd5

			variation => N__('KGA: Scandinavian, 4.exd5'),
			moves => {
				'f8d6' => 'rnbqk1nr/ppp2ppp/3b4/3P4/5p2/5N2/PPPP2PP/RNBQKB1R w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3P4/5p2/5N2/PPPP2PP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3P4/5p2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5 exf4

			variation => N__('KGD: Falkbeer, 3.exd5 exf4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3Pp3/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. exd5

			variation => N__('KGD: Falkbeer, 3.exd5'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp3ppp/2p5/3Pp3/5P2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4',
				'e5e4' => 'rnbqkbnr/ppp2ppp/8/3P4/4pP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4',
				'e5f4' => 'rnbqkbnr/ppp2ppp/8/3P4/5p2/8/PPPP2PP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3Pp3/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C40g',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d5 3. exd5

			variation => N__('Elephant Gambit: 3.exd5'),
			moves => {
				'e5e4' => 'rnbqkbnr/ppp2ppp/8/3P4/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 4',
				'f8d6' => 'rnbqk1nr/ppp2ppp/3b4/3Pp3/8/5N2/PPPP1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3Pp3/8/8/PPPP1PPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'B01a',
                        # TRANSLATORS: 1. e4 d5 2. exd5 e5

			variation => N__('Scandinavian: Böhnke Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/2B1P3/8/PB3PPP/RN1QK1NR w KQkq d6 0 6' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3 4. Bc4 cxb2 5. Bxb2 d5

			variation => N__('Danish Gambit: Schlechter Defence'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq d6 0 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 d5

			variation => N__('KGA: Bishop\'s Gambit, Bledow Variation'),
			moves => {
				'c4d5' => 'rnbqkbnr/ppp2ppp/8/3B4/4Pp2/8/PPPP2PP/RNBQK1NR b KQkq - 0 4',
				'e4d5' => 'rnbqkbnr/ppp2ppp/8/3P4/2B2p2/8/PPPP2PP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4' => {
			eco => 'C01c',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. c4

			variation => N__('French: Exchange, 4. c4'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp3ppp/2p5/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 0 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3p4/2PP4/8/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3NP3/8/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C41j',
                        # TRANSLATORS:

			variation => N__('Philidor: 3...exd4 4.Nxd4'),
			moves => {
				'e4d5' => 'rnbqkbnr/ppp2ppp/8/3P4/3N4/8/PPP2PPP/RNBQKB1R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P1B2/8/PPP2PPP/RN1QKBNR b KQkq - 1 4' => {
			eco => 'C01g',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Bf4

			variation => N__('French: Exchange, 4. Bf4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'D30d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. cxd5 exd5 4. Nc3

			variation => N__('QGD: 3.cxd5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4' => {
			eco => 'C01r',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nc3

			variation => N__('French: Exchange, 4. Nc3'),
			moves => {
				'f8b4' => 'rnbqk1nr/ppp2ppp/8/3p4/1b1P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P4/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'C01h',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Bd3

			variation => N__('French: Exchange, 4. Bd3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp2ppp/2n5/3p4/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 5',
				'f8d6' => 'rnbqk1nr/ppp2ppp/3b4/3p4/3P4/3B4/PPP2PPP/RNBQK1NR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'C01k',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5 4. Nf3

			variation => N__('French: Exchange, 4. Nf3'),
			moves => {
				'f8d6' => 'rnbqk1nr/ppp2ppp/3b4/3p4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/3p4/3P4/5N2/PPP2PPP/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D30d',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. cxd5 exd5

			variation => N__('QGD: 3.cxd5'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/8/3p4/3P4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3P4/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C01b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. exd5 exd5

			variation => N__('French: Exchange'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/8/3p4/3P4/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 4',
				'c1f4' => 'rnbqkbnr/ppp2ppp/8/3p4/3P1B2/8/PPP2PPP/RN1QKBNR b KQkq - 1 4',
				'c2c4' => 'rnbqkbnr/ppp2ppp/8/3p4/2PP4/8/PP3PPP/RNBQKBNR b KQkq c3 0 4',
				'f1d3' => 'rnbqkbnr/ppp2ppp/8/3p4/3P4/3B4/PPP2PPP/RNBQK1NR b KQkq - 1 4',
				'g1f3' => 'rnbqkbnr/ppp2ppp/8/3p4/3P4/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq d6 0 4' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 d5

			variation => N__('Danish Gambit: Sörensen Defence'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/4P3/1Q6/PP1P1PPP/RNB1KBNR b KQkq - 1 4' => {
			eco => 'C00e',
                        # TRANSLATORS: 1. e4 e6 2. c4 d5 3. cxd5 exd5 4. Qb3

			variation => N__('French: Orthoschnapp Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/4P3/8/PP1P1PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C00e',
                        # TRANSLATORS:

			variation => N__('French: Steiner (Anglo-French) Variation'),
			moves => {
				'd1b3' => 'rnbqkbnr/ppp2ppp/8/3p4/4P3/1Q6/PP1P1PPP/RNB1KBNR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3p4/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq d6 0 4' => {
			eco => 'C36',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 d5

			variation => N__('KGA: Scandinavian (Abbazia) Variation'),
			moves => {
				'e4d5' => 'rnbqkbnr/ppp2ppp/8/3P4/5p2/5N2/PPPP2PP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pN3/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C40f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d5 3. Nxe5

			variation => N__('Elephant Gambit: 3.Nxe5'),
			moves => {
				'd5e4' => 'rnbqkbnr/ppp2ppp/8/4N3/4p3/8/PPPP1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pP3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5

			variation => N__('QGD: Albin, 3.dxe5'),
			moves => {
				'd5d4' => 'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/2P3P1/8/PP1PPPBP/RNBQK1NR b KQkq c3 0 3' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2 e5 3. c4

			variation => N__('Grob Gambit: Hurst Attack'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. e3

			variation => N__('QGD: Albin, 3. e3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/2PP4/8/PP2PPPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5

			variation => N__('QGD: Albin Countergambit'),
			moves => {
				'd4e5' => 'rnbqkbnr/ppp2ppp/8/3pP3/2P5/8/PP2PPPP/RNBQKBNR b KQkq - 0 3',
				'e2e3' => 'rnbqkbnr/ppp2ppp/8/3pp3/2PP4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/3PP3/8/PPP2PPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'D00l',
                        # TRANSLATORS: 1. d4 d5 2. e4 e5

			variation => N__('Blackmar-Diemer: Beyer Countergambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. d3 d5

			variation => N__('Open Game: 2. d3 d5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq d6 0 3' => {
			eco => 'C40e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d5

			variation => N__('Elephant Gambit'),
			moves => {
				'e4d5' => 'rnbqkbnr/ppp2ppp/8/3Pp3/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
				'f3e5' => 'rnbqkbnr/ppp2ppp/8/3pN3/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. Nc3

			variation => N__('KGD: Falkbeer, Milner-Barry Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5 3. Nf3

			variation => N__('KGD: Falkbeer, Tartakower Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'C31',
                        # TRANSLATORS: 1. e4 e5 2. f4 d5

			variation => N__('KGD: Falkbeer Countergambit'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp2ppp/8/3pp3/4PP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3',
				'e4d5' => 'rnbqkbnr/ppp2ppp/8/3Pp3/5P2/8/PPPP2PP/RNBQKBNR b KQkq - 0 3',
				'g1f3' => 'rnbqkbnr/ppp2ppp/8/3pp3/4PP2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/5P2/6PN/PPPPP2P/RNBQKB1R b KQkq f3 0 3' => {
			eco => 'A00g',
                        # TRANSLATORS: 1. Nh3 d5 2. g3 e5 3. f4

			variation => N__('Amar: Paris Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/6P1/8/PPPPPPBP/RNBQK1NR w KQkq e6 0 3' => {
			eco => 'A00o',
                        # TRANSLATORS: 1. g4 d5 2. Bg2 e5

			variation => N__('Grob Gambit: e5'),
			moves => {
				'c2c4' => 'rnbqkbnr/ppp2ppp/8/3pp3/2P3P1/8/PP1PPPBP/RNBQK1NR b KQkq c3 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/8/6P1/PPPPPPBP/RNBQK1NR w KQkq e6 0 3' => {
			eco => 'A00v',
                        # TRANSLATORS: 1. g3 d5 2. Bg2 e5

			variation => N__('Benko Opening'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/8/6PN/PPPPPP1P/RNBQKB1R w KQkq e6 0 3' => {
			eco => 'A00g',
                        # TRANSLATORS:

			variation => N__('Amar/Paris Opening'),
			moves => {
				'f2f4' => 'rnbqkbnr/ppp2ppp/8/3pp3/5P2/6PN/PPPPP2P/RNBQKB1R b KQkq f3 0 3',
			},
		},
		'rnbqkbnr/ppp2ppp/8/3pp3/8/P6P/1PPPPPP1/RNBQKBNR w KQkq d6 0 3' => {
			eco => 'A00f',
                        # TRANSLATORS: 1. a3 e5 2. h3 d5

			variation => N__('Creepy Crawly Opening (Basman)'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4N3/2B1p3/8/PPPP1PPP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C40f',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d5 3. Nxe5 dxe4 4. Bc4

			variation => N__('Elephant Gambit: 3.Nxe5 dxe4 4. Bc4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4N3/4p3/8/PPPP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C40f',
                        # TRANSLATORS:

			variation => N__('Elephant Gambit: 3.Nxe5'),
			moves => {
				'f1c4' => 'rnbqkbnr/ppp2ppp/8/4N3/2B1p3/8/PPPP1PPP/RNBQK2R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D08',
                        # TRANSLATORS:

			variation => N__('QGD: Albin, 3.dxe5 d4'),
			moves => {
				'f8b4' => 'rnbqk1nr/ppp2ppp/8/4P3/1bPp4/4P3/PP3PPP/RNBQKBNR w KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. Nf3

			variation => N__('QGD: Albin, 4. Nf3'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp2ppp/2n5/4P3/2Pp4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4

			variation => N__('QGD: Albin, 3.dxe5 d4'),
			moves => {
				'a2a3' => 'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/P7/1P2PPPP/RNBQKBNR b KQkq - 0 4',
				'e2e3' => 'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/4P3/PP3PPP/RNBQKBNR b KQkq - 0 4',
				'e2e4' => 'rnbqkbnr/ppp2ppp/8/4P3/2PpP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 4',
				'g1f3' => 'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4P3/2Pp4/P7/1P2PPPP/RNBQKBNR b KQkq - 0 4' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. a3

			variation => N__('QGD: Albin, 4. a3'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4P3/2PpP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 4' => {
			eco => 'D08',
                        # TRANSLATORS: 1. d4 d5 2. c4 e5 3. dxe5 d4 4. e4

			variation => N__('QGD: Albin, 4. e4'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p2Q/3Pp3/2N5/PPP2PPP/R1B1KBNR b KQkq - 1 4' => {
			eco => 'D00m',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3 e5 4. Qh5

			variation => N__('Blackmar-Diemer: Lemberger Countergambit, Sneider Attack'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/2BP4/4P3/PP3PPP/RNBQK1NR b KQkq - 0 4' => {
			eco => 'D20e',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e3 e5'),
			moves => {
				'e5d4' => 'rnbqkbnr/ppp2ppp/8/8/2Bp4/4P3/PP3PPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/2P5/8/PP2PPPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'A41o',
                        # TRANSLATORS:

			variation => N__('Neo-Old Indian: 2. c4 e5 3.dxe5'),
			moves => {
				'd1d8' => 'rnbQkbnr/ppp2ppp/8/4p3/2P5/8/PP2PPPP/RNB1KBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/2pP4/2N5/PP2PPPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'D20i',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nc3 e5

			variation => N__('QGA: 3. Nc3 e5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/2pP4/4P3/PP3PPP/RNBQKBNR w KQkq e6 0 4' => {
			eco => 'D20e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e5

			variation => N__('QGA: 3. e3 e5'),
			moves => {
				'f1c4' => 'rnbqkbnr/ppp2ppp/8/4p3/2BP4/4P3/PP3PPP/RNBQK1NR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq - 1 4' => {
			eco => 'D20v',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e4 e5'),
			moves => {
				'e5d4' => 'rnbqkbnr/ppp2ppp/8/8/2ppP3/5N2/PP3PPP/RNBQKB1R w KQkq - 0 5',
				'f8b4' => 'rnbqk1nr/ppp2ppp/8/4p3/1bpPP3/5N2/PP3PPP/RNBQKB1R w KQkq - 2 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/2pPP3/8/PP3PPP/RNBQKBNR w KQkq e6 0 4' => {
			eco => 'D20v',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 e5

			variation => N__('QGA: 3. e4 e5'),
			moves => {
				'g1f3' => 'rnbqkbnr/ppp2ppp/8/4p3/2pPP3/5N2/PP3PPP/RNBQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/3PN3/8/PPP2PPP/R1BQKBNR w KQkq - 0 5' => {
			eco => 'C10f',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nc3 dxe4 4. Nxe4 e5

			variation => N__('French: Rubinstein, Ellis Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/3Pp3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'D00m',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3 e5 4. Nge2

			variation => N__('Blackmar-Diemer: Lemberger Countergambit, Rassmussen Attack'),
			moves => {
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq e6 0 4' => {
			eco => 'D00m',
                        # TRANSLATORS: 1. d4 d5 2. e4 dxe4 3. Nc3 e5

			variation => N__('Blackmar-Diemer: Lemberger Countergambit'),
			moves => {
				'd1h5' => 'rnbqkbnr/ppp2ppp/8/4p2Q/3Pp3/2N5/PPP2PPP/R1B1KBNR b KQkq - 1 4',
				'g1e2' => 'rnbqkbnr/ppp2ppp/8/4p3/3Pp3/2N5/PPP1NPPP/R1BQKB1R b KQkq - 1 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/4p3/4P3/8/PPP2PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C21',
                        # TRANSLATORS:

			variation => N__('Centre Game: Maroczy Defence, 3.dxe5'),
			moves => {
				'd1d8' => 'rnbQkbnr/ppp2ppp/8/4p3/4P3/8/PPP2PPP/RNB1KBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp2ppp/8/8/2BP4/8/PP3PPP/RNBQK1NR b KQkq - 0 5' => {
			eco => 'D20e',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e3 e5 4. Bxc4 exd4 5. exd4

			variation => N__('QGA: 3. e3 e5'),
			moves => {
				'f8b4' => 'rnbqk1nr/ppp2ppp/8/8/1bBP4/8/PP3PPP/RNBQK1NR w KQkq - 1 6',
				'g8f6' => 'rnbqkb1r/ppp2ppp/5n2/8/2BP4/8/PP3PPP/RNBQK1NR w KQkq - 1 6',
			},
		},
		'rnbqkbnr/ppp2ppp/8/8/2Bp4/4P3/PP3PPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'D20e',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e3 e5'),
			moves => {
				'e3d4' => 'rnbqkbnr/ppp2ppp/8/8/2BP4/8/PP3PPP/RNBQK1NR b KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp2ppp/8/8/2BpP3/5N2/PP3PPP/RNBQK2R b KQkq - 0 5' => {
			eco => 'D20x',
                        # TRANSLATORS:

			variation => N__('QGA: 3. e4 e5 4. Nf3 exd4'),
			moves => {
				'b8c6' => 'r1bqkbnr/ppp2ppp/2n5/8/2BpP3/5N2/PP3PPP/RNBQK2R w KQkq - 1 6',
			},
		},
		'rnbqkbnr/ppp2ppp/8/8/2ppP3/5N2/PP3PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'D20x',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. e4 e5 4. Nf3 exd4

			variation => N__('QGA: 3. e4 e5 4. Nf3 exd4'),
			moves => {
				'f1c4' => 'rnbqkbnr/ppp2ppp/8/8/2BpP3/5N2/PP3PPP/RNBQK2R b KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/3Pp1N1/4p3/2P5/PP3PPP/RNBQKB1R w KQkq - 2 7' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'g5e4' => 'rnbqkbnr/ppp3pp/3p4/3Pp3/4N3/2P5/PP3PPP/RNBQKB1R b KQkq - 0 7',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/3Pp3/4N3/2P5/PP3PPP/RNBQKB1R b KQkq - 0 7' => {
			eco => 'C44j',
                        # TRANSLATORS:

			variation => N__('Ponziani Countergambit, Schmidt Attack'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp3pp/3p1n2/3Pp3/4N3/2P5/PP3PPP/RNBQKB1R w KQkq - 1 8',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4P1N1/4p3/8/PPP2PPP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'C41e',
                        # TRANSLATORS:

			variation => N__('Philidor: Philidor Countergambit, 4.dxe5'),
			moves => {
				'd6d5' => 'rnbqkbnr/ppp3pp/8/3pP1N1/4p3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4P3/4p3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C41e',
                        # TRANSLATORS:

			variation => N__('Philidor: Philidor Countergambit, 4.dxe5'),
			moves => {
				'f3g5' => 'rnbqkbnr/ppp3pp/3p4/4P1N1/4p3/8/PPP2PPP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4Pp2/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C41e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 f5 4. dxe5

			variation => N__('Philidor: Philidor Countergambit, 4.dxe5'),
			moves => {
				'f5e4' => 'rnbqkbnr/ppp3pp/3p4/4P3/4p3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq f6 0 4' => {
			eco => 'C41b',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. Bc4 f5

			variation => N__('Philidor: Lopez Countergambit'),
			moves => {
				'd2d4' => 'rnbqkbnr/ppp3pp/3p4/4pp2/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4pp2/2BPP3/5N2/PPP2PPP/RNBQK2R b KQkq d3 0 4' => {
			eco => 'C41b',
                        # TRANSLATORS:

			variation => N__('Philidor: Lopez Countergambit'),
			moves => {
				'e5d4' => 'rnbqkbnr/ppp3pp/3p4/5p2/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4pp2/2P5/2N2N2/PP1PPPPP/R1BQKB1R w KQkq f6 0 4' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5

			variation => N__('English: Lukin Variation'),
			moves => {
				'd2d4' => 'rnbqkbnr/ppp3pp/3p4/4pp2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4pp2/2PP4/2N2N2/PP2PPPP/R1BQKB1R b KQkq d3 0 4' => {
			eco => 'A21',
                        # TRANSLATORS:

			variation => N__('English: Lukin Variation'),
			moves => {
				'e5e4' => 'rnbqkbnr/ppp3pp/3p4/5p2/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4pp2/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C41d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 f5 4. Nc3

			variation => N__('Philidor: Philidor Countergambit, Zukertort Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp3pp/3p4/4pp2/3PP3/5N2/PPP2PPP/RNBQKB1R w KQkq f6 0 4' => {
			eco => 'C41d',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 f5

			variation => N__('Philidor: Philidor Countergambit'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp3pp/3p4/4pp2/3PP3/2N2N2/PPP2PPP/R1BQKB1R b KQkq - 1 4',
				'd4e5' => 'rnbqkbnr/ppp3pp/3p4/4Pp2/4P3/5N2/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/5p2/2BpP3/5N2/PPP2PPP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C41b',
                        # TRANSLATORS:

			variation => N__('Philidor: Lopez Countergambit'),
			moves => {
				'f3g5' => 'rnbqkbnr/ppp3pp/3p4/5pN1/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/5p2/2PPp3/2N2N2/PP2PPPP/R1BQKB1R w KQkq - 0 5' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4

			variation => N__('English: Lukin, 4. d4 e4'),
			moves => {
				'f3d2' => 'rnbqkbnr/ppp3pp/3p4/5p2/2PPp3/2N5/PP1NPPPP/R1BQKB1R b KQkq - 1 5',
				'f3g5' => 'rnbqkbnr/ppp3pp/3p4/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/5p2/2PPp3/2N5/PP1NPPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4 5. Nd2

			variation => N__('English: Lukin, 5. Nd2'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp3pp/3p1n2/5p2/2PPp3/2N5/PP1NPPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/5pN1/2BpP3/8/PPP2PPP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C41b',
                        # TRANSLATORS:

			variation => N__('Philidor: Lopez Countergambit'),
			moves => {
				'g8h6' => 'rnbqkb1r/ppp3pp/3p3n/5pN1/2BpP3/8/PPP2PPP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/ppp3pp/3p4/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R b KQkq - 1 5' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 d6 3. Nf3 f5 4. d4 e4 5. Ng5

			variation => N__('English: Lukin, 5. Ng5'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp4pp/2pp4/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R w KQkq - 0 6',
				'f8e7' => 'rnbqk1nr/ppp1b1pp/3p4/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6',
				'g8f6' => 'rnbqkb1r/ppp3pp/3p1n2/5pN1/2PPp3/2N5/PP2PPPP/R1BQKB1R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/ppp3pp/4P3/3p2N1/4p3/8/PPP2PPP/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C41e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 f5 4. dxe5 fxe4 5. Ng5 d5 6. e6

			variation => N__('Philidor: Philidor Countergambit, del Rio Attack'),
			moves => {
				'f8c5' => 'rnbqk1nr/ppp3pp/4P3/2bp2N1/4p3/8/PPP2PPP/RNBQKB1R w KQkq - 1 7',
			},
		},
		'rnbqkbnr/ppp3pp/4p3/3p1p2/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4' => {
			eco => 'A84',
                        # TRANSLATORS:

			variation => N__('Dutch: Rubinstein, 3... d5'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp4pp/2p1p3/3p1p2/2PP4/2N1P3/PP3PPP/R1BQKBNR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/ppp3pp/4p3/3p1p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq d6 0 4' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6 3. Nc3 d5

			variation => N__('Dutch: Rubinstein, 3... d5'),
			moves => {
				'e2e3' => 'rnbqkbnr/ppp3pp/4p3/3p1p2/2PP4/2N1P3/PP3PPP/R1BQKBNR b KQkq - 0 4',
			},
		},
		'rnbqkbnr/ppp3pp/4p3/3p1p2/3PP3/8/PPPN1PPP/R1BQKBNR w KQkq f6 0 4' => {
			eco => 'C03b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. Nd2 f5

			variation => N__('French: Tarrasch, Haberditz Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp3pp/4p3/5p2/2PPp3/2N5/PP3PPP/R1BQKBNR w KQkq f6 0 5' => {
			eco => 'D30b',
                        # TRANSLATORS: 1. e4 e6 2. d4 d5 3. c4 dxe4 4. Nc3 f5

			variation => N__('Diemer-Duhm Gambit (DDG): 4... f5'),
			moves => {
			},
		},
		'rnbqkbnr/ppp3pp/8/3pN2Q/2B1p3/8/PPPP1PPP/RNB1K2R b KQkq - 1 5' => {
			eco => 'C40p',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Svedenborg Variation'),
			moves => {
				'g7g6' => 'rnbqkbnr/ppp4p/6p1/3pN2Q/2B1p3/8/PPPP1PPP/RNB1K2R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/ppp3pp/8/3pN3/2B1p3/8/PPPP1PPP/RNBQK2R w KQkq d6 0 5' => {
			eco => 'C40p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4 4. Nxe5 d5

			variation => N__('Latvian Gambit: Svedenborg Variation'),
			moves => {
				'd1h5' => 'rnbqkbnr/ppp3pp/8/3pN2Q/2B1p3/8/PPPP1PPP/RNB1K2R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/ppp3pp/8/3pP1N1/4p3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6' => {
			eco => 'C41e',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 d6 3. d4 f5 4. dxe5 fxe4 5. Ng5 d5 6. Nc3

			variation => N__('Philidor: Philidor Countergambit, Steinitz Variation'),
			moves => {
			},
		},
		'rnbqkbnr/ppp3pp/8/3pP1N1/4p3/8/PPP2PPP/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C41e',
                        # TRANSLATORS:

			variation => N__('Philidor: Philidor Countergambit, 4.dxe5'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppp3pp/8/3pP1N1/4p3/2N5/PPP2PPP/R1BQKB1R b KQkq - 1 6',
				'e5e6' => 'rnbqkbnr/ppp3pp/4P3/3p2N1/4p3/8/PPP2PPP/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkbnr/ppp4p/6N1/3p3Q/2B1p3/8/PPPP1PPP/RNB1K2R b KQkq - 0 6' => {
			eco => 'C40p',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Svedenborg Variation'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppp4p/5nN1/3p3Q/2B1p3/8/PPPP1PPP/RNB1K2R w KQkq - 1 7',
				'h7g6' => 'rnbqkbnr/ppp5/6p1/3p3Q/2B1p3/8/PPPP1PPP/RNB1K2R w KQkq - 0 7',
			},
		},
		'rnbqkbnr/ppp4p/6p1/3pN2Q/2B1p3/8/PPPP1PPP/RNB1K2R w KQkq - 0 6' => {
			eco => 'C40p',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: Svedenborg Variation'),
			moves => {
				'e5g6' => 'rnbqkbnr/ppp4p/6N1/3p3Q/2B1p3/8/PPPP1PPP/RNB1K2R b KQkq - 0 6',
			},
		},
		'rnbqkbnr/ppp5/6p1/3p3Q/2B1p3/8/PPPP1PPP/RNB1K2R w KQkq - 0 7' => {
			eco => 'C40p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4 4. Nxe5 d5 5. Qh5+ g6 6. Nxg6 hxg6

			variation => N__('Latvian Gambit: Svedenborg, 6.Nxg6 hxg6'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1B1p/8/8/4Ppp1/5N2/PPPP2PP/RNBQK2R b KQkq - 0 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Bxf7+

			variation => N__('KGA: Lolli Gambit (Wild Muzio)'),
			moves => {
				'e8f7' => 'rnbq1bnr/pppp1k1p/8/8/4Ppp1/5N2/PPPP2PP/RNBQK2R w KQ - 0 6',
			},
		},
		'rnbqkbnr/pppp1N2/7p/8/4PppP/8/PPPP2P1/RNBQKB1R b KQkq - 0 6' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'e8f7' => 'rnbq1bnr/pppp1k2/7p/8/4PppP/8/PPPP2P1/RNBQKB1R w KQ - 0 7',
			},
		},
		'rnbqkbnr/pppp1p1p/6p1/4p3/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq e6 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. e4 e5

			variation => N__('English: Adorjan Defence'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A21',
                        # TRANSLATORS:

			variation => N__('English: King\'s, 2. Nc3 g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/pppp1pbp/6p1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 g6

			variation => N__('English: King\'s, 2. Nc3 g6'),
			moves => {
				'g2g3' => 'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 g6

			variation => N__('English: King\'s, 2. g3 g6'),
			moves => {
				'f1g2' => 'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 g6 3. Bg2

			variation => N__('English: King\'s, 2. g3 g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/pppp1pbp/6p1/4p3/2P5/6P1/PP1PPPBP/RNBQK1NR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pppp1p1p/8/4N3/2B1Ppp1/8/PPPP2PP/RNBQK2R b KQkq - 1 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Ne5

			variation => N__('KGA: Salvio Gambit'),
			moves => {
				'd8h4' => 'rnb1kbnr/pppp1p1p/8/4N3/2B1Pppq/8/PPPP2PP/RNBQK2R w KQkq - 2 6',
			},
		},
		'rnbqkbnr/pppp1p1p/8/4N3/3PPpp1/8/PPP3PP/RNBQKB1R b KQkq - 1 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. d4 g4 5. Ne5

			variation => N__('KGA: Sörensen Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1p1p/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R b KQkq - 1 5' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5

			variation => N__('KGA: Kieseritsky'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1p1p/2n5/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6',
				'd7d5' => 'rnbqkbnr/ppp2p1p/8/3pN3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq d6 0 6',
				'd7d6' => 'rnbqkbnr/ppp2p1p/3p4/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 0 6',
				'd8e7' => 'rnb1kbnr/ppppqp1p/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6',
				'f8e7' => 'rnbqk1nr/ppppbp1p/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6',
				'f8g7' => 'rnbqk1nr/pppp1pbp/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6',
				'g8f6' => 'rnbqkb1r/pppp1p1p/5n2/4N3/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6',
				'h7h5' => 'rnbqkbnr/pppp1p2/8/4N2p/4PppP/8/PPPP2P1/RNBQKB1R w KQkq h6 0 6',
			},
		},
		'rnbqkbnr/pppp1p1p/8/6N1/4PppP/8/PPPP2P1/RNBQKB1R b KQkq - 1 5' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ng5

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'g8f6' => 'rnbqkb1r/pppp1p1p/5n2/6N1/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 2 6',
				'h7h6' => 'rnbqkbnr/pppp1p2/7p/6N1/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 0 6',
			},
		},
		'rnbqkbnr/pppp1p1p/8/6p1/2B1Pp2/5N2/PPPP2PP/RNBQK2R b KQkq - 1 4' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4

			variation => N__('KGA: 3. Nf3 g5 4. Bc4'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1p1p/2n5/6p1/2B1Pp2/5N2/PPPP2PP/RNBQK2R w KQkq - 2 5',
				'f8g7' => 'rnbqk1nr/pppp1pbp/8/6p1/2B1Pp2/5N2/PPPP2PP/RNBQK2R w KQkq - 2 5',
				'g5g4' => 'rnbqkbnr/pppp1p1p/8/8/2B1Ppp1/5N2/PPPP2PP/RNBQK2R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pppp1p1p/8/6p1/3PPp2/5N2/PPP3PP/RNBQKB1R b KQkq d3 0 4' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. d4

			variation => N__('KGA: Rosentreter Gambit'),
			moves => {
				'g5g4' => 'rnbqkbnr/pppp1p1p/8/8/3PPpp1/5N2/PPP3PP/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pppp1p1p/8/6p1/4Pp1P/5N2/PPPP2P1/RNBQKB1R b KQkq h3 0 4' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4

			variation => N__('KGA: 3. Nf3 g5 4. h4'),
			moves => {
				'g5g4' => 'rnbqkbnr/pppp1p1p/8/8/4PppP/5N2/PPPP2P1/RNBQKB1R w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pppp1p1p/8/6p1/4Pp2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Nc3

			variation => N__('KGA: Quaade Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1p1p/8/6p1/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq g6 0 4' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5

			variation => N__('KGA: 3. Nf3 g5'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp1p1p/8/6p1/4Pp2/2N2N2/PPPP2PP/R1BQKB1R b KQkq - 1 4',
				'd2d4' => 'rnbqkbnr/pppp1p1p/8/6p1/3PPp2/5N2/PPP3PP/RNBQKB1R b KQkq d3 0 4',
				'f1c4' => 'rnbqkbnr/pppp1p1p/8/6p1/2B1Pp2/5N2/PPPP2PP/RNBQK2R b KQkq - 1 4',
				'h2h4' => 'rnbqkbnr/pppp1p1p/8/6p1/4Pp1P/5N2/PPPP2P1/RNBQKB1R b KQkq h3 0 4',
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 b kq - 0 6' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1p1p/2n5/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 w kq - 1 7',
				'd8e7' => 'rnb1kbnr/ppppqp1p/8/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 w kq - 1 7',
				'd8f6' => 'rnb1kbnr/pppp1p1p/5q2/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 w kq - 1 7',
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/2B1Pp2/5p2/PPPP2PP/RNBQ1RK1 w kq - 0 6' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'd1f3' => 'rnbqkbnr/pppp1p1p/8/8/2B1Pp2/5Q2/PPPP2PP/RNB2RK1 b kq - 0 6',
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/2B1Ppp1/2N2N2/PPPP2PP/R1BQK2R b KQkq - 1 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. Nc3

			variation => N__('KGA: MacDonnell Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/2B1Ppp1/5N2/PPPP2PP/RNBQ1RK1 b kq - 1 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. O-O

			variation => N__('KGA: Muzio Gambit'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2p1p/8/3p4/2B1Ppp1/5N2/PPPP2PP/RNBQ1RK1 w kq d6 0 6',
				'd8e7' => 'rnb1kbnr/ppppqp1p/8/8/2B1Ppp1/5N2/PPPP2PP/RNBQ1RK1 w kq - 2 6',
				'g4f3' => 'rnbqkbnr/pppp1p1p/8/8/2B1Pp2/5p2/PPPP2PP/RNBQ1RK1 w kq - 0 6',
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/2B1Ppp1/5N2/PPPP2PP/RNBQK2R w KQkq - 0 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4

			variation => N__('KGA: 3. Nf3 g5 4. Bc4 g4'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp1p1p/8/8/2B1Ppp1/2N2N2/PPPP2PP/R1BQK2R b KQkq - 1 5',
				'c4f7' => 'rnbqkbnr/pppp1B1p/8/8/4Ppp1/5N2/PPPP2PP/RNBQK2R b KQkq - 0 5',
				'd2d4' => 'rnbqkbnr/pppp1p1p/8/8/2BPPpp1/5N2/PPP3PP/RNBQK2R b KQkq d3 0 5',
				'e1g1' => 'rnbqkbnr/pppp1p1p/8/8/2B1Ppp1/5N2/PPPP2PP/RNBQ1RK1 b kq - 1 5',
				'f3e5' => 'rnbqkbnr/pppp1p1p/8/4N3/2B1Ppp1/8/PPPP2PP/RNBQK2R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/2BPPpp1/5N2/PPP3PP/RNBQK2R b KQkq d3 0 5' => {
			eco => 'C37',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. Bc4 g4 5. d4

			variation => N__('KGA: Ghulam-Kassim Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/3PPpp1/5N2/PPP3PP/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C37',
                        # TRANSLATORS:

			variation => N__('KGA: Rosentreter Gambit'),
			moves => {
				'f3e5' => 'rnbqkbnr/pppp1p1p/8/4N3/3PPpp1/8/PPP3PP/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pppp1p1p/8/8/4PppP/5N2/PPPP2P1/RNBQKB1R w KQkq - 0 5' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: 3. Nf3 g5 4. h4'),
			moves => {
				'f3e5' => 'rnbqkbnr/pppp1p1p/8/4N3/4PppP/8/PPPP2P1/RNBQKB1R b KQkq - 1 5',
				'f3g5' => 'rnbqkbnr/pppp1p1p/8/6N1/4PppP/8/PPPP2P1/RNBQKB1R b KQkq - 1 5',
			},
		},
		'rnbqkbnr/pppp1p2/7p/6N1/4PppP/8/PPPP2P1/RNBQKB1R w KQkq - 0 6' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Allgaier Gambit'),
			moves => {
				'g5f7' => 'rnbqkbnr/pppp1N2/7p/8/4PppP/8/PPPP2P1/RNBQKB1R b KQkq - 0 6',
			},
		},
		'rnbqkbnr/pppp1p2/8/4N2p/2B1PppP/8/PPPP2P1/RNBQK2R b KQkq - 1 6' => {
			eco => 'C39',
                        # TRANSLATORS:

			variation => N__('KGA: Kieseritsky, Long Whip Defence'),
			moves => {
				'h8h7' => 'rnbqkbn1/pppp1p1r/8/4N2p/2B1PppP/8/PPPP2P1/RNBQK2R w KQq - 2 7',
			},
		},
		'rnbqkbnr/pppp1p2/8/4N2p/4PppP/8/PPPP2P1/RNBQKB1R w KQkq h6 0 6' => {
			eco => 'C39',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 g5 4. h4 g4 5. Ne5 h5

			variation => N__('KGA: Kieseritsky, Long Whip Defence'),
			moves => {
				'f1c4' => 'rnbqkbnr/pppp1p2/8/4N2p/2B1PppP/8/PPPP2P1/RNBQK2R b KQkq - 1 6',
			},
		},
		'rnbqkbnr/pppp1pp1/7p/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 h6

			variation => N__('KGA: Becker Defence (Anti-Kieseritzky)'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/1B6/4P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2' => {
			eco => 'C00a',
                        # TRANSLATORS: 1. e4 e6 2. Bb5

			variation => N__('French: Bird Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/4P3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'C00d',
                        # TRANSLATORS: 1. e4 e6 2. e5

			variation => N__('French: Steinitz Attack'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'C00e',
                        # TRANSLATORS: 1. e4 e6 2. c4

			variation => N__('French: Steiner (Anglo-French) Variation'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A13e',
                        # TRANSLATORS: 1. c4 e6 2. Nc3

			variation => N__('English: 1... e6 2. Nc3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq d6 0 3',
				'f8b4' => 'rnbqk1nr/pppp1ppp/4p3/8/1bP5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A13h',
                        # TRANSLATORS: 1. c4 e6 2. Nf3

			variation => N__('English: 1... e6 2. Nf3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq d6 0 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/4pn2/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A13b',
                        # TRANSLATORS: 1. c4 e6 2. g3

			variation => N__('English: 1... e6 2. g3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A13a',
                        # TRANSLATORS: 1. c4 e6

			variation => N__('English: 1... e6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp1ppp/4p3/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/pppp1ppp/4p3/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/pppp1ppp/4p3/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A40o',
                        # TRANSLATORS: 1. d4 e6 2. c4

			variation => N__('Queen\'s Pawn: 1... e6 2. c4'),
			moves => {
				'f8b4' => 'rnbqk1nr/pppp1ppp/4p3/8/1bPP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A40n',
                        # TRANSLATORS: 1. d4 e6 2. Nf3

			variation => N__('Queen\'s Pawn: 1... e6 2. Nf3'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq c6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A40m',
                        # TRANSLATORS: 1. d4 e6

			variation => N__('Queen\'s Pawn: 1... e6'),
			moves => {
				'c2c4' => 'rnbqkbnr/pppp1ppp/4p3/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'g1f3' => 'rnbqkbnr/pppp1ppp/4p3/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'C00w',
                        # TRANSLATORS: 1. e4 e6 2. d4

			variation => N__('French: 2. d4'),
			moves => {
				'a7a6' => 'rnbqkbnr/1ppp1ppp/p3p3/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'c7c5' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/3PP3/8/PPP2PPP/RNBQKBNR w KQkq c6 0 3',
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/3PP3/8/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4P3/1P6/P1PP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'C00b',
                        # TRANSLATORS: 1. e4 e6 2. b3

			variation => N__('French: Reti (Spielmann) Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'C00f',
                        # TRANSLATORS: 1. e4 e6 2. Nc3

			variation => N__('French: 2. Nc3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'C00m',
                        # TRANSLATORS: 1. e4 e6 2. d3

			variation => N__('French: KIA 2. d3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'C00j',
                        # TRANSLATORS: 1. e4 e6 2. Nf3

			variation => N__('French: 2. Nf3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/4p3/3p4/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'C00a',
                        # TRANSLATORS: 1. e4 e6

			variation => N__('French'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp1ppp/4p3/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2',
				'b2b3' => 'rnbqkbnr/pppp1ppp/4p3/8/4P3/1P6/P1PP1PPP/RNBQKBNR b KQkq - 0 2',
				'c2c4' => 'rnbqkbnr/pppp1ppp/4p3/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2',
				'd1e2' => 'rnbqkbnr/pppp1ppp/4p3/8/4P3/8/PPPPQPPP/RNB1KBNR b KQkq - 1 2',
				'd2d3' => 'rnbqkbnr/pppp1ppp/4p3/8/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'd2d4' => 'rnbqkbnr/pppp1ppp/4p3/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'e4e5' => 'rnbqkbnr/pppp1ppp/4p3/4P3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2',
				'f1b5' => 'rnbqkbnr/pppp1ppp/4p3/1B6/4P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2',
				'f2f4' => 'rnbqkbnr/pppp1ppp/4p3/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2',
				'g1f3' => 'rnbqkbnr/pppp1ppp/4p3/8/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4P3/8/PPPPQPPP/RNB1KBNR b KQkq - 1 2' => {
			eco => 'C00g',
                        # TRANSLATORS: 1. e4 e6 2. Qe2

			variation => N__('French: Chigorin Variation'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1p1ppp/4p3/2p5/4P3/8/PPPPQPPP/RNB1KBNR w KQkq c6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'C00c',
                        # TRANSLATORS: 1. e4 e6 2. f4

			variation => N__('French: La Bourdonnais Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 e6

			variation => N__('Reti: 1... e6'),
			moves => {
				'g2g3' => 'rnbqkbnr/pppp1ppp/4p3/8/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppp1ppp/4p3/8/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 e6 2. g3

			variation => N__('Reti: 1... e6'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Bb5

			variation => N__('Open Game: Portuguese Opening'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3',
				'c7c6' => 'rnbqkbnr/pp1p1ppp/2p5/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4P3/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5

			variation => N__('Englund Gambit Accepted'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4P3/8/8/PPP1PPPP/RNBQKBNR w KQkq - 1 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4P3/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3',
				'f7f6' => 'rnbqkbnr/pppp2pp/5p2/4P3/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4P3/8/8/PPPPP1PP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5 2. fxe5

			variation => N__('Bird: From Gambit Accepted'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4P3/8/8/PPPPP1PP/RNBQKBNR w KQkq - 1 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4P3/8/8/PPPPP1PP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p2Q/4P3/8/PPPP1PPP/RNB1KBNR b KQkq - 1 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Qh5

			variation => N__('Open Game: Patzer/Parnham Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/1P6/8/P1PPPPPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5

			variation => N__('Polish: 1... e5'),
			moves => {
				'a2a3' => 'rnbqkbnr/pppp1ppp/8/4p3/1P6/P7/2PPPPPP/RNBQKBNR b KQkq - 0 2',
				'c1b2' => 'rnbqkbnr/pppp1ppp/8/4p3/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/1P6/8/PBPPPPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5 2. Bb2

			variation => N__('Polish: 1... e5 2. Bb2'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1p1ppp/8/2p1p3/1P6/8/PBPPPPPP/RN1QKBNR w KQkq c6 0 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
				'f7f6' => 'rnbqkbnr/pppp2pp/5p2/4p3/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
				'f8b4' => 'rnbqk1nr/pppp1ppp/8/4p3/1b6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/1P6/P7/2PPPPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5 2. a3

			variation => N__('Polish: Bugayev Attack'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2B1P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4

			variation => N__('Bishop\'s Opening'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pp1ppp/8/1p2p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq b6 0 3',
				'c7c6' => 'rnbqkbnr/pp1p1ppp/2p5/4p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq - 0 3',
				'f7f5' => 'rnbqkbnr/pppp2pp/8/4pp2/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq f6 0 3',
				'f8c5' => 'rnbqk1nr/pppp1ppp/8/2b1p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. c4

			variation => N__('Open Game: 2. c4'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3

			variation => N__('English: King\'s, 2. Nc3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'f7f5' => 'rnbqkbnr/pppp2pp/8/4pp2/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq f6 0 3',
				'f8b4' => 'rnbqk1nr/pppp1ppp/8/4p3/1bP5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
				'g7g6' => 'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P5/3P4/PP2PPPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. d3

			variation => N__('English: King\'s, 2. d3'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P5/4P3/PP1P1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. e3

			variation => N__('English: King\'s, 2. e3'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. Nf3

			variation => N__('English: King\'s, Nimzowitsch'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
				'e5e4' => 'rnbqkbnr/pppp1ppp/8/8/2P1p3/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3

			variation => N__('English: King\'s, 2. g3'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3',
				'c7c6' => 'rnbqkbnr/pp1p1ppp/2p5/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3',
				'f7f5' => 'rnbqkbnr/pppp2pp/8/4pp2/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq f6 0 3',
				'g7g6' => 'rnbqkbnr/pppp1p1p/6p1/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/2P5/8/PP1PPPPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5

			variation => N__('English: King\'s (1... e5)'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp1ppp/8/4p3/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'd2d3' => 'rnbqkbnr/pppp1ppp/8/4p3/2P5/3P4/PP2PPPP/RNBQKBNR b KQkq - 0 2',
				'e2e3' => 'rnbqkbnr/pppp1ppp/8/4p3/2P5/4P3/PP1P1PPP/RNBQKBNR b KQkq - 0 2',
				'g1f3' => 'rnbqkbnr/pppp1ppp/8/4p3/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/pppp1ppp/8/4p3/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/3P4/8/PPP1PPPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5

			variation => N__('Englund Gambit'),
			moves => {
				'd4e5' => 'rnbqkbnr/pppp1ppp/8/4P3/8/8/PPP1PPPP/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4

			variation => N__('Centre Game'),
			moves => {
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'e5d4' => 'rnbqkbnr/pppp1ppp/8/8/3pP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'C25a',
                        # TRANSLATORS: 1. e4 e5 2. Nc3

			variation => N__('Vienna Game'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
				'f8b4' => 'rnbqk1nr/pppp1ppp/8/4p3/1b2P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
				'f8c5' => 'rnbqk1nr/pppp1ppp/8/2b1p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. c3

			variation => N__('Open Game: Lopez/Mcleod Opening'),
			moves => {
				'f7f5' => 'rnbqkbnr/pppp2pp/8/4pp2/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq f6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. d3

			variation => N__('Open Game: 2. d3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3pp3/4P3/3P4/PPP2PPP/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'C40a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3

			variation => N__('Open Game'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3pp3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq d6 0 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'd8e7' => 'rnb1kbnr/ppppqppp/8/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'd8f6' => 'rnb1kbnr/pppp1ppp/5q2/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
				'f7f5' => 'rnbqkbnr/pppp2pp/8/4pp2/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq f6 0 3',
				'f7f6' => 'rnbqkbnr/pppp2pp/5p2/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/5Q2/PPPP1PPP/RNB1KBNR b KQkq - 1 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Qf3

			variation => N__('Open Game: Napoleon\'s Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/8/PPPP1PPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5

			variation => N__('Open Game'),
			moves => {
				'a2a3' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/P7/1PPP1PPP/RNBQKBNR b KQkq - 0 2',
				'b1c3' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2',
				'c2c3' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/2P5/PP1P1PPP/RNBQKBNR b KQkq - 0 2',
				'c2c4' => 'rnbqkbnr/pppp1ppp/8/4p3/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq c3 0 2',
				'd1f3' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/5Q2/PPPP1PPP/RNB1KBNR b KQkq - 1 2',
				'd1h5' => 'rnbqkbnr/pppp1ppp/8/4p2Q/4P3/8/PPPP1PPP/RNB1KBNR b KQkq - 1 2',
				'd2d3' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/3P4/PPP2PPP/RNBQKBNR b KQkq - 0 2',
				'd2d4' => 'rnbqkbnr/pppp1ppp/8/4p3/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'f1b5' => 'rnbqkbnr/pppp1ppp/8/1B2p3/4P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2',
				'f1c4' => 'rnbqkbnr/pppp1ppp/8/4p3/2B1P3/8/PPPP1PPP/RNBQK1NR b KQkq - 1 2',
				'f2f4' => 'rnbqkbnr/pppp1ppp/8/4p3/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2',
				'g1e2' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/8/PPPPNPPP/RNBQKB1R b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/8/PPPPNPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. Ne2

			variation => N__('Open Game: Alapin Opening'),
			moves => {
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4P3/8/PPPPNPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4P3/P7/1PPP1PPP/RNBQKBNR b KQkq - 0 2' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. a3

			variation => N__('Open Game: Mengarini Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'C30a',
                        # TRANSLATORS: 1. e4 e5 2. f4

			variation => N__('King\'s Gambit'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
				'c7c5' => 'rnbqkbnr/pp1p1ppp/8/2p1p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq c6 0 3',
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3pp3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq d6 0 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
				'd8f6' => 'rnb1kbnr/pppp1ppp/5q2/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
				'd8h4' => 'rnb1kbnr/pppp1ppp/8/4p3/4PP1q/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
				'e5f4' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
				'f8c5' => 'rnbqk1nr/pppp1ppp/8/2b1p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/4p3/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/5P2/8/PPPPP1PP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 e5

			variation => N__('Bird: From Gambit'),
			moves => {
				'f4e5' => 'rnbqkbnr/pppp1ppp/8/4P3/8/8/PPPPP1PP/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/6P1/8/PPPPPP1P/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A00n',
                        # TRANSLATORS: 1. g4 e5

			variation => N__('Grob: 1... e5'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/1P6/P1PPPPPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 e5

			variation => N__('Nimzowitsch-Larsen: 1... e5'),
			moves => {
				'c1b2' => 'rnbqkbnr/pppp1ppp/8/4p3/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 e5 2. Bb2

			variation => N__('Nimzowitsch-Larsen: 1... e5 2. Bb2'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/4p3/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 2 3',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/4p3/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 1 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 e5 2. Nf3

			variation => N__('Van Geet: 1... e5 2. Nf3'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/2N5/PPPPPPPP/R1BQKBNR w KQkq e6 0 2' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3 e5

			variation => N__('Van Geet: 1... e5'),
			moves => {
				'g1f3' => 'rnbqkbnr/pppp1ppp/8/4p3/8/2N2N2/PPPPPPPP/R1BQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/3P4/PPP1PPPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A00j',
                        # TRANSLATORS: 1. d3 e5

			variation => N__('Mieses: 1... e5'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/5P2/PPPPP1PP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A00b',
                        # TRANSLATORS:

			variation => N__('Barnes Opening'),
			moves => {
				'e1f2' => 'rnbqkbnr/pppp1ppp/8/4p3/8/5P2/PPPPPKPP/RNBQ1BNR b kq - 1 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/5P2/PPPPPKPP/RNBQ1BNR b kq - 1 2' => {
			eco => 'A00b',
                        # TRANSLATORS: 1. f3 e5 2. Kf2

			variation => N__('Fried fox'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/6P1/PPPPPP1P/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A00u',
                        # TRANSLATORS: 1. g3 e5

			variation => N__('Benko Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/P6P/1PPPPPP1/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A00f',
                        # TRANSLATORS:

			variation => N__('Anderssen Opening'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3pp3/8/P6P/1PPPPPP1/RNBQKBNR w KQkq d6 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/4p3/8/P7/1PPPPPPP/RNBQKBNR w KQkq e6 0 2' => {
			eco => 'A00f',
                        # TRANSLATORS:

			variation => N__('Anderssen Opening'),
			moves => {
				'h2h3' => 'rnbqkbnr/pppp1ppp/8/4p3/8/P6P/1PPPPPP1/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppp1ppp/8/7Q/4Pp2/8/PPPP2PP/RNB1KBNR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Qh5

			variation => N__('KGA: Carrera Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/2B1P3/2p5/PP3PPP/RNBQK1NR b KQkq - 1 4' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3 4. Bc4

			variation => N__('Danish Gambit: Accepted, 4. Bc4'),
			moves => {
				'c3b2' => 'rnbqkbnr/pppp1ppp/8/8/2B1P3/8/Pp3PPP/RNBQK1NR w KQkq - 0 5',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/2B1P3/8/PB3PPP/RN1QK1NR b KQkq - 0 5' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3 4. Bc4 cxb2 5. Bxb2

			variation => N__('Danish Gambit: Accepted, 4. Bc4 cxb2 5.Bxb2'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3p4/2B1P3/8/PB3PPP/RN1QK1NR w KQkq d6 0 6',
				'd8e7' => 'rnb1kbnr/ppppqppp/8/8/2B1P3/8/PB3PPP/RN1QK1NR w KQkq - 1 6',
				'f8b4' => 'rnbqk1nr/pppp1ppp/8/8/1bB1P3/8/PB3PPP/RN1QK1NR w KQkq - 1 6',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/8/2B1P3/8/PB3PPP/RN1QK1NR w KQkq - 1 6',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/2B1P3/8/Pp3PPP/RNBQK1NR w KQkq - 0 5' => {
			eco => 'C21',
                        # TRANSLATORS:

			variation => N__('Danish Gambit: Accepted, 4. Bc4'),
			moves => {
				'c1b2' => 'rnbqkbnr/pppp1ppp/8/8/2B1P3/8/PB3PPP/RN1QK1NR b KQkq - 0 5',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/2B1Pp2/8/PPPP2PP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4

			variation => N__('KGA: Bishop\'s Gambit'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pp1ppp/8/1p6/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq b6 0 4',
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 2 4',
				'c7c6' => 'rnbqkbnr/pp1p1ppp/2p5/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 0 4',
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3p4/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq d6 0 4',
				'd8h4' => 'rnb1kbnr/pppp1ppp/8/8/2B1Pp1q/8/PPPP2PP/RNBQK1NR w KQkq - 2 4',
				'f7f5' => 'rnbqkbnr/pppp2pp/8/5p2/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq f6 0 4',
				'g8e7' => 'rnbqkb1r/ppppnppp/8/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 2 4',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/8/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/2P1p3/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. Nf3 e4

			variation => N__('English: King\'s, Nimzowitsch, Flohr Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/3PPp2/8/PPP3PP/RNBQKBNR b KQkq d3 0 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. d4

			variation => N__('KGA: Villemson Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/3QP3/8/PPP2PPP/RNB1KBNR b KQkq - 0 3' => {
			eco => 'C22',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Qxd4

			variation => N__('Centre Game'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp1ppp/2n5/8/3QP3/8/PPP2PPP/RNB1KBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/3pP3/2P5/PP3PPP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3

			variation => N__('Danish Gambit'),
			moves => {
				'd4c3' => 'rnbqkbnr/pppp1ppp/8/8/4P3/2p5/PP3PPP/RNBQKBNR w KQkq - 0 4',
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3p4/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq d6 0 4',
				'g8e7' => 'rnbqkb1r/ppppnppp/8/8/3pP3/2P5/PP3PPP/RNBQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/3pP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. Nf3

			variation => N__('Centre Game: 3. Nf3'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1p1ppp/8/2p5/3pP3/5N2/PPP2PPP/RNBQKB1R w KQkq c6 0 4',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/3pP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4

			variation => N__('Centre Game'),
			moves => {
				'c2c3' => 'rnbqkbnr/pppp1ppp/8/8/3pP3/2P5/PP3PPP/RNBQKBNR b KQkq - 0 3',
				'd1d4' => 'rnbqkbnr/pppp1ppp/8/8/3QP3/8/PPP2PPP/RNB1KBNR b KQkq - 0 3',
				'f2f4' => 'rnbqkbnr/pppp1ppp/8/8/3pPP2/8/PPP3PP/RNBQKBNR b KQkq f3 0 3',
				'g1f3' => 'rnbqkbnr/pppp1ppp/8/8/3pP3/5N2/PPP2PPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/3pPP2/8/PPP3PP/RNBQKBNR b KQkq f3 0 3' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. f4

			variation => N__('Centre Game: Halasz Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4P3/2p5/PP3PPP/RNBQKBNR w KQkq - 0 4' => {
			eco => 'C21',
                        # TRANSLATORS: 1. e4 e5 2. d4 exd4 3. c3 dxc3

			variation => N__('Danish Gambit: Accepted'),
			moves => {
				'f1c4' => 'rnbqkbnr/pppp1ppp/8/8/2B1P3/2p5/PP3PPP/RNBQK1NR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp1P/8/PPPP2P1/RNBQKBNR b KQkq h3 0 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. h4

			variation => N__('KGA: Stamma (Leonardo) Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/1P6/P1PP2PP/RNBQKBNR b KQkq - 0 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. b3

			variation => N__('KGA: Orsini Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nc3

			variation => N__('KGA: Keres Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/3B4/PPPP2PP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bd3

			variation => N__('KGA: Schurig Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3

			variation => N__('KGA: King\'s Knight Gambit'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp2ppp/8/3p4/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq d6 0 4',
				'd7d6' => 'rnbqkbnr/ppp2ppp/3p4/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
				'f7f5' => 'rnbqkbnr/pppp2pp/8/5p2/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq f6 0 4',
				'f8e7' => 'rnbqk1nr/ppppbppp/8/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4',
				'g7g5' => 'rnbqkbnr/pppp1p1p/8/6p1/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq g6 0 4',
				'g8e7' => 'rnbqkb1r/ppppnppp/8/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4',
				'g8f6' => 'rnbqkb1r/pppp1ppp/5n2/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 2 4',
				'h7h6' => 'rnbqkbnr/pppp1pp1/7p/8/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/5Q2/PPPP2PP/RNB1KBNR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Qf3

			variation => N__('KGA: Breyer Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/7N/PPPP2PP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nh3

			variation => N__('KGA: Eisenberg Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPP1KPP/RNBQ1BNR b kq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Kf2

			variation => N__('KGA: Tumbleweed/Drunken King'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4

			variation => N__('King\'s Gambit Accepted (KGA)'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3',
				'b2b3' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/1P6/P1PP2PP/RNBQKBNR b KQkq - 0 3',
				'd1e2' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPPQ1PP/RNB1KBNR b KQkq - 1 3',
				'd1f3' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/5Q2/PPPP2PP/RNB1KBNR b KQkq - 1 3',
				'd1h5' => 'rnbqkbnr/pppp1ppp/8/7Q/4Pp2/8/PPPP2PP/RNB1KBNR b KQkq - 1 3',
				'd2d4' => 'rnbqkbnr/pppp1ppp/8/8/3PPp2/8/PPP3PP/RNBQKBNR b KQkq d3 0 3',
				'e1f2' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPP1KPP/RNBQ1BNR b kq - 1 3',
				'f1c4' => 'rnbqkbnr/pppp1ppp/8/8/2B1Pp2/8/PPPP2PP/RNBQK1NR b KQkq - 1 3',
				'f1d3' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/3B4/PPPP2PP/RNBQK1NR b KQkq - 1 3',
				'f1e2' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPPB1PP/RNBQK1NR b KQkq - 1 3',
				'g1e2' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPPN1PP/RNBQKB1R b KQkq - 1 3',
				'g1f3' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/5N2/PPPP2PP/RNBQKB1R b KQkq - 1 3',
				'g1h3' => 'rnbqkbnr/pppp1ppp/8/8/4Pp2/7N/PPPP2PP/RNBQKB1R b KQkq - 1 3',
				'h2h4' => 'rnbqkbnr/pppp1ppp/8/8/4Pp1P/8/PPPP2P1/RNBQKBNR b KQkq h3 0 3',
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPPB1PP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Be2

			variation => N__('KGA: Lesser Bishop\'s (Tartakower) Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPPN1PP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Ne2

			variation => N__('KGA: Eisenberg Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp1ppp/8/8/4Pp2/8/PPPPQ1PP/RNB1KBNR b KQkq - 1 3' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Qe2

			variation => N__('KGA: Basman Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/4p3/3P1p2/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'A80s',
                        # TRANSLATORS: 1. d4 f5 2. Nf3 e6 3. d5

			variation => N__('Dutch: 2. Nf3 e6 3. d5'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3 e6

			variation => N__('English: Anglo-Dutch'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6 3. Nc3

			variation => N__('Dutch: Rubinstein Variation'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp3pp/4p3/3p1p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq d6 0 4',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6 3. Nf3

			variation => N__('Dutch: 2. c4 e6 3. Nf3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pppp2pp/4pn2/5p2/2PP4/5N2/PP2PPPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6

			variation => N__('Dutch: 2. c4 e6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp2pp/4p3/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
				'e2e4' => 'rnbqkbnr/pppp2pp/4p3/5p2/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3',
				'g1f3' => 'rnbqkbnr/pppp2pp/4p3/5p2/2PP4/5N2/PP2PPPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/2PPP3/8/PP3PPP/RNBQKBNR b KQkq e3 0 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 e6 3. e4

			variation => N__('Dutch: Staunton Gambit Deferred'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/3P2P1/3Q4/PPP1PP1P/RNB1KBNR b KQkq g3 0 3' => {
			eco => 'A80i',
                        # TRANSLATORS: 1. d4 f5 2. Qd3 e6 3. g4

			variation => N__('Dutch: Von Pretzel Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/3P4/3Q4/PPP1PPPP/RNB1KBNR w KQkq - 0 3' => {
			eco => 'A80i',
                        # TRANSLATORS:

			variation => N__('Dutch: Alapin'),
			moves => {
				'g2g4' => 'rnbqkbnr/pppp2pp/4p3/5p2/3P2P1/3Q4/PPP1PP1P/RNB1KBNR b KQkq g3 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A80r',
                        # TRANSLATORS: 1. d4 f5 2. Nf3 e6

			variation => N__('Dutch: 2. Nf3 e6'),
			moves => {
				'd4d5' => 'rnbqkbnr/pppp2pp/4p3/3P1p2/8/5N2/PPP1PPPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 e6 3. Nf3

			variation => N__('Dutch: 2. g3 e6 3. Nf3'),
			moves => {
				'g8f6' => 'rnbqkb1r/pppp2pp/4pn2/5p2/3P4/5NP1/PPP1PP1P/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3 e6

			variation => N__('Dutch: 2. g3 e6'),
			moves => {
				'g1f3' => 'rnbqkbnr/pppp2pp/4p3/5p2/3P4/5NP1/PPP1PP1P/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/4P3/1P6/PBPP1PPP/RN1QKBNR b KQkq e3 0 3' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 f5 2. Bb2 e6 3. e4

			variation => N__('Nimzowitsch-Larsen: Ringelbach Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/4p3/5p2/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A01',
                        # TRANSLATORS:

			variation => N__('Nimzowitsch-Larsen: Dutch Variation'),
			moves => {
				'e2e4' => 'rnbqkbnr/pppp2pp/4p3/5p2/4P3/1P6/PBPP1PPP/RN1QKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/4p3/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 e6

			variation => N__('Dutch: Staunton Gambit, 3. Nc3 e6'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/5p2/4P3/8/8/PPP1PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A40i',
                        # TRANSLATORS: 1. d4 e5 2. dxe5 f6

			variation => N__('Englund Gambit: Soller'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/5p2/4p3/1P2P3/8/PBPP1PPP/RN1QKBNR b KQkq e3 0 3' => {
			eco => 'A00r',
                        # TRANSLATORS:

			variation => N__('Polish: 1... e5 2. Bb2 f6'),
			moves => {
				'f8b4' => 'rnbqk1nr/pppp2pp/5p2/4p3/1b2P3/8/PBPP1PPP/RN1QKBNR w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pppp2pp/5p2/4p3/1P6/8/PBPPPPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A00r',
                        # TRANSLATORS: 1. b4 e5 2. Bb2 f6

			variation => N__('Polish: 1... e5 2. Bb2 f6'),
			moves => {
				'e2e4' => 'rnbqkbnr/pppp2pp/5p2/4p3/1P2P3/8/PBPP1PPP/RN1QKBNR b KQkq e3 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/5p2/4p3/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'C40a',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f6

			variation => N__('Open Game: Damiano Defence'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4N3/2B1p3/8/PPPP1PPP/RNBQK2R b KQkq - 0 4' => {
			eco => 'C40p',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: 3. Bc4 fxe4'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp3pp/8/3pN3/2B1p3/8/PPPP1PPP/RNBQK2R w KQkq d6 0 5',
				'd8g5' => 'rnb1kbnr/pppp2pp/8/4N1q1/2B1p3/8/PPPP1PPP/RNBQK2R w KQkq - 1 5',
				'g8f6' => 'rnbqkb1r/pppp2pp/5n2/4N3/2B1p3/8/PPPP1PPP/RNBQK2R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pppp2pp/8/4N3/3Pp3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4' => {
			eco => 'C40m',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: 3. d4'),
			moves => {
				'g8f6' => 'rnbqkb1r/pppp2pp/5n2/4N3/3Pp3/8/PPP2PPP/RNBQKB1R w KQkq - 1 5',
			},
		},
		'rnbqkbnr/pppp2pp/8/4Np2/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C40r',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nxe5

			variation => N__('Latvian Gambit: 3.Nxe5'),
			moves => {
				'b8c6' => 'r1bqkbnr/pppp2pp/2n5/4Np2/4P3/8/PPPP1PPP/RNBQKB1R w KQkq - 1 4',
				'd8f6' => 'rnb1kbnr/pppp2pp/5q2/4Np2/4P3/8/PPPP1PPP/RNBQKB1R w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4p1N1/4p3/8/PPPP1PPP/RNBQKB1R w KQkq e6 0 4' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. e4 fxe4 3. Ng5 e5

			variation => N__('Reti: Lisitsin: 3. Ng5 e5'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4p3/2B1p3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4' => {
			eco => 'C40p',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4 fxe4

			variation => N__('Latvian Gambit: 3. Bc4 fxe4'),
			moves => {
				'f3e5' => 'rnbqkbnr/pppp2pp/8/4N3/2B1p3/8/PPPP1PPP/RNBQK2R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4p3/3Pp3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C40m',
                        # TRANSLATORS:

			variation => N__('Latvian Gambit: 3. d4'),
			moves => {
				'f3e5' => 'rnbqkbnr/pppp2pp/8/4N3/3Pp3/8/PPP2PPP/RNBQKB1R b KQkq - 0 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pP2/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C40n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. exf5

			variation => N__('Latvian Gambit: 3.exf5'),
			moves => {
				'e5e4' => 'rnbqkbnr/pppp2pp/8/5P2/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/2B1P3/3P4/PPP2PPP/RNBQK1NR b KQkq - 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 f5 3. d3

			variation => N__('Bishop\'s Opening: Calabrese Countergambit, Jänisch Variation'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3' => {
			eco => 'C40o',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Bc4

			variation => N__('Latvian Gambit: 3. Bc4'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pp2pp/8/1p2pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq b6 0 4',
				'f5e4' => 'rnbqkbnr/pppp2pp/8/4p3/2B1p3/5N2/PPPP1PPP/RNBQK2R w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/pppp2pp/5n2/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/2B1P3/8/PPPP1PPP/RNBQK1NR w KQkq f6 0 3' => {
			eco => 'C23',
                        # TRANSLATORS: 1. e4 e5 2. Bc4 f5

			variation => N__('Bishop\'s Opening: Calabrese Countergambit'),
			moves => {
				'd2d3' => 'rnbqkbnr/pppp2pp/8/4pp2/2B1P3/3P4/PPP2PPP/RNBQK1NR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3' => {
			eco => 'A21',
                        # TRANSLATORS:

			variation => N__('English: King\'s, 2. Nc3 f5'),
			moves => {
				'g8f6' => 'rnbqkb1r/pppp2pp/5n2/4pp2/2P5/2N3P1/PP1PPP1P/R1BQKBNR w KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq f6 0 3' => {
			eco => 'A21',
                        # TRANSLATORS: 1. c4 e5 2. Nc3 f5

			variation => N__('English: King\'s, 2. Nc3 f5'),
			moves => {
				'g2g3' => 'rnbqkbnr/pppp2pp/8/4pp2/2P5/2N3P1/PP1PPP1P/R1BQKBNR b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq f6 0 3' => {
			eco => 'A20',
                        # TRANSLATORS: 1. c4 e5 2. g3 f5

			variation => N__('English: King\'s, 2. g3 f5'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3' => {
			eco => 'C40m',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. d4

			variation => N__('Latvian Gambit: 3. d4'),
			moves => {
				'f5e4' => 'rnbqkbnr/pppp2pp/8/4p3/3Pp3/5N2/PPP2PPP/RNBQKB1R w KQkq - 0 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3' => {
			eco => 'C40l',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. Nc3

			variation => N__('Latvian Gambit: 3. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/4P3/2P5/PP1P1PPP/RNBQKBNR w KQkq f6 0 3' => {
			eco => 'C20',
                        # TRANSLATORS: 1. e4 e5 2. c3 f5

			variation => N__('Open Game: Lopez/Mcleod, Lasa Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3' => {
			eco => 'C40k',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. d3

			variation => N__('Latvian Gambit: 3. d3'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/4pp2/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq f6 0 3' => {
			eco => 'C40j',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5

			variation => N__('Latvian Gambit'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppp2pp/8/4pp2/4P3/2N2N2/PPPP1PPP/R1BQKB1R b KQkq - 1 3',
				'd2d3' => 'rnbqkbnr/pppp2pp/8/4pp2/4P3/3P1N2/PPP2PPP/RNBQKB1R b KQkq - 0 3',
				'd2d4' => 'rnbqkbnr/pppp2pp/8/4pp2/3PP3/5N2/PPP2PPP/RNBQKB1R b KQkq d3 0 3',
				'e4f5' => 'rnbqkbnr/pppp2pp/8/4pP2/8/5N2/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
				'f1c4' => 'rnbqkbnr/pppp2pp/8/4pp2/2B1P3/5N2/PPPP1PPP/RNBQK2R b KQkq - 1 3',
				'f3e5' => 'rnbqkbnr/pppp2pp/8/4Np2/4P3/8/PPPP1PPP/RNBQKB1R b KQkq - 0 3',
			},
		},
		'rnbqkbnr/pppp2pp/8/5P2/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 4' => {
			eco => 'C40n',
                        # TRANSLATORS: 1. e4 e5 2. Nf3 f5 3. exf5 e4

			variation => N__('Latvian Gambit: 3.exf5 e4'),
			moves => {
			},
		},
		'rnbqkbnr/pppp2pp/8/5p2/2B1Pp2/8/PPPP2PP/RNBQK1NR w KQkq f6 0 4' => {
			eco => 'C33',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Bc4 f5

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit'),
			moves => {
				'd1e2' => 'rnbqkbnr/pppp2pp/8/5p2/2B1Pp2/8/PPPPQ1PP/RNB1K1NR b KQkq - 1 4',
			},
		},
		'rnbqkbnr/pppp2pp/8/5p2/2B1Pp2/8/PPPPQ1PP/RNB1K1NR b KQkq - 1 4' => {
			eco => 'C33',
                        # TRANSLATORS:

			variation => N__('KGA: Bishop\'s Gambit, Lopez-Gianutio Countergambit'),
			moves => {
				'd8h4' => 'rnb1kbnr/pppp2pp/8/5p2/2B1Pp1q/8/PPPPQ1PP/RNB1K1NR w KQkq - 2 5',
			},
		},
		'rnbqkbnr/pppp2pp/8/5p2/4Pp2/5N2/PPPP2PP/RNBQKB1R w KQkq f6 0 4' => {
			eco => 'C34',
                        # TRANSLATORS: 1. e4 e5 2. f4 exf4 3. Nf3 f5

			variation => N__('KGA: Gianutio Countergambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1p1/7p/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A80f',
                        # TRANSLATORS: 1. d4 f5 2. Bg5 h6

			variation => N__('Dutch: 2. Bg5 h6'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/2P3P1/8/PP1PPP1P/RNBQKBNR b KQkq g3 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. g4

			variation => N__('English: Wade Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nc3

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. Nf3

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/5p2/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5 2. g3

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2P5/6P1/PP1PPP1P/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/2P5/8/PP1PPPPP/RNBQKBNR w KQkq f6 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 f5

			variation => N__('English: Anglo-Dutch'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppppp1pp/8/5p2/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'g1f3' => 'rnbqkbnr/ppppp1pp/8/5p2/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/ppppp1pp/8/5p2/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
				'g2g4' => 'rnbqkbnr/ppppp1pp/8/5p2/2P3P1/8/PP1PPP1P/RNBQKBNR b KQkq g3 0 2',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4

			variation => N__('Dutch: 2. c4'),
			moves => {
				'd7d6' => 'rnbqkbnr/ppp1p1pp/3p4/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/ppppp2p/6p1/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P2P1/8/PPP1PP1P/RNBQKBNR b KQkq g3 0 2' => {
			eco => 'A80b',
                        # TRANSLATORS: 1. d4 f5 2. g4

			variation => N__('Dutch: Krejcik Gambit'),
			moves => {
				'f5g4' => 'rnbqkbnr/ppppp1pp/8/8/3P2p1/8/PPP1PP1P/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P4/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A80j',
                        # TRANSLATORS: 1. d4 f5 2. Nc3

			variation => N__('Dutch: 2. Nc3'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp1p1pp/8/3p1p2/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq d6 0 3',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/2N5/PPP1PPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P4/3Q4/PPP1PPPP/RNB1KBNR b KQkq - 1 2' => {
			eco => 'A80i',
                        # TRANSLATORS: 1. d4 f5 2. Qd3

			variation => N__('Dutch: Alapin'),
			moves => {
				'd7d6' => 'rnbqkbnr/ppp1p1pp/3p4/5p2/3P4/3Q4/PPP1PPPP/RNB1KBNR w KQkq - 0 3',
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/5p2/3P4/3Q4/PPP1PPPP/RNB1KBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A80q',
                        # TRANSLATORS: 1. d4 f5 2. Nf3

			variation => N__('Dutch: 2. Nf3'),
			moves => {
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/5p2/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/5N2/PPP1PPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A81',
                        # TRANSLATORS: 1. d4 f5 2. g3

			variation => N__('Dutch: 2. g3'),
			moves => {
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 0 3',
				'g7g6' => 'rnbqkbnr/ppppp2p/6p1/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P4/7P/PPP1PPP1/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A80c',
                        # TRANSLATORS: 1. d4 f5 2. h3

			variation => N__('Dutch: Korchnoi Attack'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/3P4/7P/PPP1PPP1/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3P4/8/PPP1PPPP/RNBQKBNR w KQkq f6 0 2' => {
			eco => 'A80a',
                        # TRANSLATORS: 1. d4 f5

			variation => N__('Dutch'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppppp1pp/8/5p2/3P4/2N5/PPP1PPPP/R1BQKBNR b KQkq - 1 2',
				'c1g5' => 'rnbqkbnr/ppppp1pp/8/5pB1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2',
				'c2c4' => 'rnbqkbnr/ppppp1pp/8/5p2/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'd1d3' => 'rnbqkbnr/ppppp1pp/8/5p2/3P4/3Q4/PPP1PPPP/RNB1KBNR b KQkq - 1 2',
				'e2e4' => 'rnbqkbnr/ppppp1pp/8/5p2/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2',
				'g1f3' => 'rnbqkbnr/ppppp1pp/8/5p2/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/ppppp1pp/8/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR b KQkq - 0 2',
				'g2g4' => 'rnbqkbnr/ppppp1pp/8/5p2/3P2P1/8/PPP1PP1P/RNBQKBNR b KQkq g3 0 2',
				'h2h3' => 'rnbqkbnr/ppppp1pp/8/5p2/3P4/7P/PPP1PPP1/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/3PP3/8/PPP2PPP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4

			variation => N__('Dutch: Staunton Gambit'),
			moves => {
				'd7d6' => 'rnbqkbnr/ppp1p1pp/3p4/5p2/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'f5e4' => 'rnbqkbnr/ppppp1pp/8/8/3Pp3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq e3 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. e4

			variation => N__('Reti: Lisitsin'),
			moves => {
				'f5e4' => 'rnbqkbnr/ppppp1pp/8/8/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/4P3/8/PPPP1PPP/RNBQKBNR w KQkq f6 0 2' => {
			eco => 'B00a',
                        # TRANSLATORS: 1. e4 f5

			variation => N__('King\'s Pawn: Fred'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/4PP2/8/PPPP2PP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 f5 2. e4

			variation => N__('Bird: Swiss Gambit'),
			moves => {
				'f5e4' => 'rnbqkbnr/ppppp1pp/8/8/4pP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/5P2/8/PPPPP1PP/RNBQKBNR w KQkq f6 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 f5

			variation => N__('Bird: Symmetrical'),
			moves => {
				'e2e4' => 'rnbqkbnr/ppppp1pp/8/5p2/4PP2/8/PPPP2PP/RNBQKBNR b KQkq e3 0 2',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/6P1/8/PPPPPP1P/RNBQKBNR w KQkq f6 0 2' => {
			eco => 'A00n',
                        # TRANSLATORS: 1. g4 f5

			variation => N__('Grob: Alessi Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/8/1P6/P1PPPPPP/RNBQKBNR w KQkq f6 0 2' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3 f5

			variation => N__('Nimzowitsch-Larsen: Dutch Variation'),
			moves => {
				'c1b2' => 'rnbqkbnr/ppppp1pp/8/5p2/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 1 2',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/8/1P6/PBPPPPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A01',
                        # TRANSLATORS:

			variation => N__('Nimzowitsch-Larsen: Dutch Variation'),
			moves => {
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/5p2/8/1P6/PBPPPPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/8/3P1N2/PPP1PPPP/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. d3

			variation => N__('Reti: 1... f5 2. d3'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5p2/8/3P1N2/PPP1PPPP/RNBQKB1R w KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/8/5N2/PPPPPPPP/RNBQKB1R w KQkq f6 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5

			variation => N__('Reti: 1... f5'),
			moves => {
				'd2d3' => 'rnbqkbnr/ppppp1pp/8/5p2/8/3P1N2/PPP1PPPP/RNBQKB1R b KQkq - 0 2',
				'e2e4' => 'rnbqkbnr/ppppp1pp/8/5p2/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq e3 0 2',
				'g2g3' => 'rnbqkbnr/ppppp1pp/8/5p2/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
			},
		},
		'rnbqkbnr/ppppp1pp/8/5p2/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 f5 2. g3

			variation => N__('Reti: 1... f5 2. g3'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1pp/8/5pB1/3P4/8/PPP1PPPP/RN1QKBNR b KQkq - 1 2' => {
			eco => 'A80d',
                        # TRANSLATORS: 1. d4 f5 2. Bg5

			variation => N__('Dutch: 2. Bg5'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp1pp1pp/2p5/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'd7d5' => 'rnbqkbnr/ppp1p1pp/8/3p1pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq d6 0 3',
				'g7g6' => 'rnbqkbnr/ppppp2p/6p1/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 2 3',
				'h7h6' => 'rnbqkbnr/ppppp1p1/7p/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/6N1/4p3/8/PPPP1PPP/RNBQKB1R b KQkq - 1 3' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: Lisitsin'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp1p1pp/8/3p2N1/4p3/8/PPPP1PPP/RNBQKB1R w KQkq d6 0 4',
				'e7e5' => 'rnbqkbnr/pppp2pp/8/4p1N1/4p3/8/PPPP1PPP/RNBQKB1R w KQkq e6 0 4',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/6N1/4p3/8/PPPP1PPP/RNBQKB1R w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppppp1pp/8/8/3P2p1/8/PPP1PP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A80b',
                        # TRANSLATORS: 1. d4 f5 2. g4 fxg4

			variation => N__('Dutch: Krejcik Gambit Accepted'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp1pp/8/8/3Pp3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3

			variation => N__('Dutch: Staunton Gambit, 3. Nc3'),
			moves => {
				'e7e6' => 'rnbqkbnr/pppp2pp/4p3/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'g7g6' => 'rnbqkbnr/ppppp2p/6p1/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4',
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppppp1pp/8/8/3Pp3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4

			variation => N__('Dutch: Staunton Gambit Accepted'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppppp1pp/8/8/3Pp3/2N5/PPP2PPP/R1BQKBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/8/4p3/5N2/PPPP1PPP/RNBQKB1R w KQkq - 0 3' => {
			eco => 'A04',
                        # TRANSLATORS:

			variation => N__('Reti: Lisitsin'),
			moves => {
				'f3g5' => 'rnbqkbnr/ppppp1pp/8/6N1/4p3/8/PPPP1PPP/RNBQKB1R b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp1pp/8/8/4pP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: Swiss Gambit'),
			moves => {
				'g8f6' => 'rnbqkb1r/ppppp1pp/5n2/8/4pP2/2N5/PPPP2PP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppppp1pp/8/8/4pP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: Swiss Gambit'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppppp1pp/8/8/4pP2/2N5/PPPP2PP/R1BQKBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp2p/6p1/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3' => {
			eco => 'A84',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. c4 g6'),
			moves => {
				'g8h6' => 'rnbqkb1r/ppppp2p/6pn/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppppp2p/6p1/5p2/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A84',
                        # TRANSLATORS: 1. d4 f5 2. c4 g6

			variation => N__('Dutch: 2. c4 g6'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppppp2p/6p1/5p2/2PP4/2N5/PP2PPPP/R1BQKBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp2p/6p1/5p2/3P4/6P1/PPP1PP1P/RNBQKBNR w KQkq - 0 3' => {
			eco => 'A81',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. g3'),
			moves => {
				'f1g2' => 'rnbqkbnr/ppppp2p/6p1/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp2p/6p1/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR b KQkq - 1 3' => {
			eco => 'A81',
                        # TRANSLATORS:

			variation => N__('Dutch: 2. g3'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppp1bp/6p1/5p2/3P4/6P1/PPP1PPBP/RNBQK1NR w KQkq - 2 4',
			},
		},
		'rnbqkbnr/ppppp2p/6p1/5pB1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3' => {
			eco => 'A80h',
                        # TRANSLATORS: 1. d4 f5 2. Bg5 g6 3. Nc3

			variation => N__('Dutch: 2. Bg5 g6 3. Nc3'),
			moves => {
			},
		},
		'rnbqkbnr/ppppp2p/6p1/5pB1/3P4/8/PPP1PPPP/RN1QKBNR w KQkq - 0 3' => {
			eco => 'A80g',
                        # TRANSLATORS: 1. d4 f5 2. Bg5 g6

			variation => N__('Dutch: 2. Bg5 g6'),
			moves => {
				'b1c3' => 'rnbqkbnr/ppppp2p/6p1/5pB1/3P4/2N5/PPP1PPPP/R2QKBNR b KQkq - 1 3',
			},
		},
		'rnbqkbnr/ppppp2p/6p1/8/3Pp3/2N5/PPP2PPP/R1BQKBNR w KQkq - 0 4' => {
			eco => 'A82',
                        # TRANSLATORS: 1. d4 f5 2. e4 fxe4 3. Nc3 g6

			variation => N__('Dutch: Staunton Gambit, 3. Nc3 g6'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq e3 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. e4

			variation => N__('English: 1... g6 2. e4'),
			moves => {
				'e7e5' => 'rnbqkbnr/pppp1p1p/6p1/4p3/2P1P3/8/PP1P1PPP/RNBQKBNR w KQkq e6 0 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. Nc3

			variation => N__('English: 1... g6 2. Nc3'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/2P5/2N5/PP1PPPPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. Nf3

			variation => N__('English: 1... g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/2P5/5N2/PP1PPPPP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6 2. g3

			variation => N__('English: 1... g6 2. g3'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g6

			variation => N__('English: 1... g6'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppppp1p/6p1/8/2P5/2N5/PP1PPPPP/R1BQKBNR b KQkq - 1 2',
				'e2e4' => 'rnbqkbnr/pppppp1p/6p1/8/2P1P3/8/PP1P1PPP/RNBQKBNR b KQkq e3 0 2',
				'g1f3' => 'rnbqkbnr/pppppp1p/6p1/8/2P5/5N2/PP1PPPPP/RNBQKB1R b KQkq - 1 2',
				'g2g3' => 'rnbqkbnr/pppppp1p/6p1/8/2P5/6P1/PP1PPP1P/RNBQKBNR b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2' => {
			eco => 'A40u',
                        # TRANSLATORS: 1. d4 g6 2. c4

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A40t',
                        # TRANSLATORS: 1. d4 g6 2. Nf3

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A40s',
                        # TRANSLATORS: 1. d4 g6

			variation => N__('Queen\'s Pawn: Modern'),
			moves => {
				'c2c4' => 'rnbqkbnr/pppppp1p/6p1/8/2PP4/8/PP2PPPP/RNBQKBNR b KQkq c3 0 2',
				'g1f3' => 'rnbqkbnr/pppppp1p/6p1/8/3P4/5N2/PPP1PPPP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B06d',
                        # TRANSLATORS: 1. e4 g6 2. d4

			variation => N__('Modern'),
			moves => {
				'c7c6' => 'rnbqkbnr/pp1ppp1p/2p3p1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'd7d6' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 0 3',
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
				'g8f6' => 'rnbqkb1r/pppppp1p/5np1/8/3PP3/8/PPP2PPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2' => {
			eco => 'B06b',
                        # TRANSLATORS: 1. e4 g6 2. Nc3

			variation => N__('Modern'),
			moves => {
				'd7d6' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 0 3',
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/4P3/2N5/PPPP1PPP/R1BQKBNR w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'B06a',
                        # TRANSLATORS: 1. e4 g6

			variation => N__('Modern'),
			moves => {
				'b1c3' => 'rnbqkbnr/pppppp1p/6p1/8/4P3/2N5/PPPP1PPP/R1BQKBNR b KQkq - 1 2',
				'd2d4' => 'rnbqkbnr/pppppp1p/6p1/8/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
				'f2f4' => 'rnbqkbnr/pppppp1p/6p1/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/4PP2/8/PPPP2PP/RNBQKBNR b KQkq f3 0 2' => {
			eco => 'B06c',
                        # TRANSLATORS: 1. e4 g6 2. f4

			variation => N__('Modern'),
			moves => {
				'd7d6' => 'rnbqkbnr/ppp1pp1p/3p2p1/8/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 0 3',
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/4PP2/8/PPPP2PP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 2' => {
			eco => 'A02',
                        # TRANSLATORS:

			variation => N__('Bird: 1..g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/5P2/5N2/PPPPP1PP/RNBQKB1R w KQkq - 2 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/5P2/8/PPPPP1PP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 g6

			variation => N__('Bird: 1..g6'),
			moves => {
				'g1f3' => 'rnbqkbnr/pppppp1p/6p1/8/5P2/5N2/PPPPP1PP/RNBQKB1R b KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/6P1/3P4/PPP1PP1P/RNBQKBNR b KQkq g3 0 2' => {
			eco => 'A00j',
                        # TRANSLATORS: 1. d3 g6 2. g4

			variation => N__('Spike Deferred'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/8/3P4/PPP1PPPP/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A00j',
                        # TRANSLATORS:

			variation => N__('Mieses'),
			moves => {
				'g2g4' => 'rnbqkbnr/pppppp1p/6p1/8/6P1/3P4/PPP1PP1P/RNBQKBNR b KQkq g3 0 2',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 g6

			variation => N__('Reti: 1... g6'),
			moves => {
				'g2g3' => 'rnbqkbnr/pppppp1p/6p1/8/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/8/5NP1/PPPPPP1P/RNBQKB1R b KQkq - 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 g6 2. g3

			variation => N__('Reti: 1... g6'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppppbp/6p1/8/8/5NP1/PPPPPP1P/RNBQKB1R w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppppp1p/6p1/8/8/6P1/PPPPPP1P/RNBQKBNR w KQkq - 0 2' => {
			eco => 'A00t',
                        # TRANSLATORS: 1. g3 g6

			variation => N__('Benko Opening: Symmetrical'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/2P5/8/PP1PPPPP/RNBQKBNR w KQkq g6 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g5

			variation => N__('English: 1... g5'),
			moves => {
				'd2d4' => 'rnbqkbnr/pppppp1p/8/6p1/2PP4/8/PP2PPPP/RNBQKBNR b KQkq d3 0 2',
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/2PP4/8/PP2PPPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4 g5 2. d4

			variation => N__('English: 1... g5 2. d4'),
			moves => {
				'f8g7' => 'rnbqk1nr/ppppppbp/8/6p1/2PP4/8/PP2PPPP/RNBQKBNR w KQkq - 1 3',
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2' => {
			eco => 'B00b',
                        # TRANSLATORS: 1. e4 g5 2. d4

			variation => N__('Reversed Grob (Borg/Basman Defence)'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/4P3/8/PPPP1PPP/RNBQKBNR w KQkq g6 0 2' => {
			eco => 'B00b',
                        # TRANSLATORS: 1. e4 g5

			variation => N__('Reversed Grob (Borg/Basman Defence)'),
			moves => {
				'd2d4' => 'rnbqkbnr/pppppp1p/8/6p1/3PP3/8/PPP2PPP/RNBQKBNR b KQkq d3 0 2',
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/5P2/8/PPPPP1PP/RNBQKBNR w KQkq g6 0 2' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4 g5

			variation => N__('Bird: Hobbs Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/6P1/8/PPPPPP1P/RNBQKBNR w KQkq g6 0 2' => {
			eco => 'A00n',
                        # TRANSLATORS: 1. g4 g5

			variation => N__('Grob: Double Grob'),
			moves => {
			},
		},
		'rnbqkbnr/pppppp1p/8/6p1/8/5N2/PPPPPPPP/RNBQKB1R w KQkq g6 0 2' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3 g5

			variation => N__('Reti: Herrström Gambit'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/4P3/8/8/PPPP1PPP/RNBQKBNR w KQkq - 1 3' => {
			eco => 'B02l',
                        # TRANSLATORS: 1. e4 Nf6 2. e5 Ng8

			variation => N__('Alekhine: Brooklyn Defence (Retreat Variation)'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/8/1P6/8/P1PPPPPP/RNBQKBNR b KQkq b3 0 1' => {
			eco => 'A00p',
                        # TRANSLATORS: 1. b4

			variation => N__('Polish (Sokolsky; Orang-Utan)'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/1P6/8/P1PPPPPP/RNBQKBNR w KQkq c6 0 2',
				'c7c6' => 'rnbqkbnr/pp1ppppp/2p5/8/1P6/8/P1PPPPPP/RNBQKBNR w KQkq - 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/1P6/8/P1PPPPPP/RNBQKBNR w KQkq d6 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/1P6/8/P1PPPPPP/RNBQKBNR w KQkq e6 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/1P6/8/P1PPPPPP/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/2P5/8/PP1PPPPP/RNBQKBNR b KQkq c3 0 1' => {
			eco => 'A10',
                        # TRANSLATORS: 1. c4

			variation => N__('English'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pppppp/8/1p6/2P5/8/PP1PPPPP/RNBQKBNR w KQkq b6 0 2',
				'b7b6' => 'rnbqkbnr/p1pppppp/1p6/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2',
				'b8c6' => 'r1bqkbnr/pppppppp/2n5/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 1 2',
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/2P5/8/PP1PPPPP/RNBQKBNR w KQkq c6 0 2',
				'c7c6' => 'rnbqkbnr/pp1ppppp/2p5/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/2P5/8/PP1PPPPP/RNBQKBNR w KQkq d6 0 2',
				'd7d6' => 'rnbqkbnr/ppp1pppp/3p4/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/2P5/8/PP1PPPPP/RNBQKBNR w KQkq e6 0 2',
				'e7e6' => 'rnbqkbnr/pppp1ppp/4p3/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/2P5/8/PP1PPPPP/RNBQKBNR w KQkq f6 0 2',
				'g7g5' => 'rnbqkbnr/pppppp1p/8/6p1/2P5/8/PP1PPPPP/RNBQKBNR w KQkq g6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/2P5/8/PP1PPPPP/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/3P4/8/PPP1PPPP/RNBQKBNR b KQkq d3 0 1' => {
			eco => 'A40a',
                        # TRANSLATORS: 1. d4

			variation => N__('Queen\'s Pawn Game'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pppppp/8/1p6/3P4/8/PPP1PPPP/RNBQKBNR w KQkq b6 0 2',
				'b7b6' => 'rnbqkbnr/p1pppppp/1p6/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2',
				'b8c6' => 'r1bqkbnr/pppppppp/2n5/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 1 2',
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/3P4/8/PPP1PPPP/RNBQKBNR w KQkq c6 0 2',
				'c7c6' => 'rnbqkbnr/pp1ppppp/2p5/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/3P4/8/PPP1PPPP/RNBQKBNR w KQkq d6 0 2',
				'd7d6' => 'rnbqkbnr/ppp1pppp/3p4/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/3P4/8/PPP1PPPP/RNBQKBNR w KQkq e6 0 2',
				'e7e6' => 'rnbqkbnr/pppp1ppp/4p3/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/3P4/8/PPP1PPPP/RNBQKBNR w KQkq f6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/3P4/8/PPP1PPPP/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq e3 0 1' => {
			eco => 'B00a',
                        # TRANSLATORS: 1. e4

			variation => N__('King\'s Pawn'),
			moves => {
				'a7a6' => 'rnbqkbnr/1ppppppp/p7/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2',
				'b7b6' => 'rnbqkbnr/p1pppppp/1p6/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2',
				'b8c6' => 'r1bqkbnr/pppppppp/2n5/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 2',
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/4P3/8/PPPP1PPP/RNBQKBNR w KQkq c6 0 2',
				'c7c6' => 'rnbqkbnr/pp1ppppp/2p5/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/4P3/8/PPPP1PPP/RNBQKBNR w KQkq d6 0 2',
				'd7d6' => 'rnbqkbnr/ppp1pppp/3p4/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/4P3/8/PPPP1PPP/RNBQKBNR w KQkq e6 0 2',
				'e7e6' => 'rnbqkbnr/pppp1ppp/4p3/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/4P3/8/PPPP1PPP/RNBQKBNR w KQkq f6 0 2',
				'g7g5' => 'rnbqkbnr/pppppp1p/8/6p1/4P3/8/PPPP1PPP/RNBQKBNR w KQkq g6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 2',
				'g8h6' => 'rnbqkb1r/pppppppp/7n/8/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/5P2/8/PPPPP1PP/RNBQKBNR b KQkq f3 0 1' => {
			eco => 'A02',
                        # TRANSLATORS: 1. f4

			variation => N__('Bird'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/5P2/8/PPPPP1PP/RNBQKBNR w KQkq c6 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/5P2/8/PPPPP1PP/RNBQKBNR w KQkq d6 0 2',
				'd7d6' => 'rnbqkbnr/ppp1pppp/3p4/8/5P2/8/PPPPP1PP/RNBQKBNR w KQkq - 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/5P2/8/PPPPP1PP/RNBQKBNR w KQkq e6 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/5P2/8/PPPPP1PP/RNBQKBNR w KQkq f6 0 2',
				'g7g5' => 'rnbqkbnr/pppppp1p/8/6p1/5P2/8/PPPPP1PP/RNBQKBNR w KQkq g6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/5P2/8/PPPPP1PP/RNBQKBNR w KQkq - 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/5P2/8/PPPPP1PP/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/6P1/8/PPPPPP1P/RNBQKBNR b KQkq g3 0 1' => {
			eco => 'A00n',
                        # TRANSLATORS: 1. g4

			variation => N__('Grob'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/6P1/8/PPPPPP1P/RNBQKBNR w KQkq d6 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/6P1/8/PPPPPP1P/RNBQKBNR w KQkq e6 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/6P1/8/PPPPPP1P/RNBQKBNR w KQkq f6 0 2',
				'g7g5' => 'rnbqkbnr/pppppp1p/8/6p1/6P1/8/PPPPPP1P/RNBQKBNR w KQkq g6 0 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/7P/8/PPPPPPP1/RNBQKBNR b KQkq h3 0 1' => {
			eco => 'A00c',
                        # TRANSLATORS: 1. h4

			variation => N__('Kadas Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/1P6/P1PPPPPP/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A01',
                        # TRANSLATORS: 1. b3

			variation => N__('Nimzowitsch-Larsen Attack'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pppppp/8/1p6/8/1P6/P1PPPPPP/RNBQKBNR w KQkq b6 0 2',
				'b7b6' => 'rnbqkbnr/p1pppppp/1p6/8/8/1P6/P1PPPPPP/RNBQKBNR w KQkq - 0 2',
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/8/1P6/P1PPPPPP/RNBQKBNR w KQkq c6 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/8/1P6/P1PPPPPP/RNBQKBNR w KQkq d6 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/8/1P6/P1PPPPPP/RNBQKBNR w KQkq e6 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/8/1P6/P1PPPPPP/RNBQKBNR w KQkq f6 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/8/1P6/P1PPPPPP/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/2N5/PPPPPPPP/R1BQKBNR b KQkq - 1 1' => {
			eco => 'A00l',
                        # TRANSLATORS: 1. Nc3

			variation => N__('Van Geet (Dunst) Opening'),
			moves => {
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/8/2N5/PPPPPPPP/R1BQKBNR w KQkq c6 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/8/2N5/PPPPPPPP/R1BQKBNR w KQkq d6 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/8/2N5/PPPPPPPP/R1BQKBNR w KQkq e6 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/8/2N5/PPPPPPPP/R1BQKBNR w KQkq - 2 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/2P5/PP1PPPPP/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00i',
                        # TRANSLATORS: 1. c3

			variation => N__('Saragossa'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/3P4/PPP1PPPP/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00j',
                        # TRANSLATORS: 1. d3

			variation => N__('Mieses'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/8/3P4/PPP1PPPP/RNBQKBNR w KQkq d6 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/8/3P4/PPP1PPPP/RNBQKBNR w KQkq e6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/8/3P4/PPP1PPPP/RNBQKBNR w KQkq - 0 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/4P3/PPPP1PPP/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00k',
                        # TRANSLATORS: 1. e3

			variation => N__('Van Kruijs'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/5N2/PPPPPPPP/RNBQKB1R b KQkq - 1 1' => {
			eco => 'A04',
                        # TRANSLATORS: 1. Nf3

			variation => N__('Reti'),
			moves => {
				'b7b5' => 'rnbqkbnr/p1pppppp/8/1p6/8/5N2/PPPPPPPP/RNBQKB1R w KQkq b6 0 2',
				'b7b6' => 'rnbqkbnr/p1pppppp/1p6/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2',
				'b8c6' => 'r1bqkbnr/pppppppp/2n5/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 2 2',
				'c7c5' => 'rnbqkbnr/pp1ppppp/8/2p5/8/5N2/PPPPPPPP/RNBQKB1R w KQkq c6 0 2',
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/8/5N2/PPPPPPPP/RNBQKB1R w KQkq d6 0 2',
				'd7d6' => 'rnbqkbnr/ppp1pppp/3p4/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2',
				'e7e6' => 'rnbqkbnr/pppp1ppp/4p3/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2',
				'f7f5' => 'rnbqkbnr/ppppp1pp/8/5p2/8/5N2/PPPPPPPP/RNBQKB1R w KQkq f6 0 2',
				'g7g5' => 'rnbqkbnr/pppppp1p/8/6p1/8/5N2/PPPPPPPP/RNBQKB1R w KQkq g6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/8/5N2/PPPPPPPP/RNBQKB1R w KQkq - 2 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/5P2/PPPPP1PP/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00b',
                        # TRANSLATORS: 1. f3

			variation => N__('Barnes Opening'),
			moves => {
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/8/5P2/PPPPP1PP/RNBQKBNR w KQkq e6 0 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/6P1/PPPPPP1P/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00t',
                        # TRANSLATORS: 1. g3

			variation => N__('Benko Opening'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/8/6P1/PPPPPP1P/RNBQKBNR w KQkq d6 0 2',
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/8/6P1/PPPPPP1P/RNBQKBNR w KQkq e6 0 2',
				'g7g6' => 'rnbqkbnr/pppppp1p/6p1/8/8/6P1/PPPPPP1P/RNBQKBNR w KQkq - 0 2',
				'g8f6' => 'rnbqkb1r/pppppppp/5n2/8/8/6P1/PPPPPP1P/RNBQKBNR w KQkq - 1 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/7N/PPPPPPPP/RNBQKB1R b KQkq - 1 1' => {
			eco => 'A00g',
                        # TRANSLATORS: 1. Nh3

			variation => N__('Amar/Paris Opening'),
			moves => {
				'd7d5' => 'rnbqkbnr/ppp1pppp/8/3p4/8/7N/PPPPPPPP/RNBQKB1R w KQkq d6 0 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/7P/PPPPPPP1/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00d',
                        # TRANSLATORS: 1. h3

			variation => N__('Clemenz Opening'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1' => {
			eco => 'A00a',
                        # TRANSLATORS:

			variation => N__('Start'),
			moves => {
				'a2a3' => 'rnbqkbnr/pppppppp/8/8/8/P7/1PPPPPPP/RNBQKBNR b KQkq - 0 1',
				'a2a4' => 'rnbqkbnr/pppppppp/8/8/P7/8/1PPPPPPP/RNBQKBNR b KQkq a3 0 1',
				'b1a3' => 'rnbqkbnr/pppppppp/8/8/8/N7/PPPPPPPP/R1BQKBNR b KQkq - 1 1',
				'b1c3' => 'rnbqkbnr/pppppppp/8/8/8/2N5/PPPPPPPP/R1BQKBNR b KQkq - 1 1',
				'b2b3' => 'rnbqkbnr/pppppppp/8/8/8/1P6/P1PPPPPP/RNBQKBNR b KQkq - 0 1',
				'b2b4' => 'rnbqkbnr/pppppppp/8/8/1P6/8/P1PPPPPP/RNBQKBNR b KQkq b3 0 1',
				'c2c3' => 'rnbqkbnr/pppppppp/8/8/8/2P5/PP1PPPPP/RNBQKBNR b KQkq - 0 1',
				'c2c4' => 'rnbqkbnr/pppppppp/8/8/2P5/8/PP1PPPPP/RNBQKBNR b KQkq c3 0 1',
				'd2d3' => 'rnbqkbnr/pppppppp/8/8/8/3P4/PPP1PPPP/RNBQKBNR b KQkq - 0 1',
				'd2d4' => 'rnbqkbnr/pppppppp/8/8/3P4/8/PPP1PPPP/RNBQKBNR b KQkq d3 0 1',
				'e2e3' => 'rnbqkbnr/pppppppp/8/8/8/4P3/PPPP1PPP/RNBQKBNR b KQkq - 0 1',
				'e2e4' => 'rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq e3 0 1',
				'f2f3' => 'rnbqkbnr/pppppppp/8/8/8/5P2/PPPPP1PP/RNBQKBNR b KQkq - 0 1',
				'f2f4' => 'rnbqkbnr/pppppppp/8/8/5P2/8/PPPPP1PP/RNBQKBNR b KQkq f3 0 1',
				'g1f3' => 'rnbqkbnr/pppppppp/8/8/8/5N2/PPPPPPPP/RNBQKB1R b KQkq - 1 1',
				'g1h3' => 'rnbqkbnr/pppppppp/8/8/8/7N/PPPPPPPP/RNBQKB1R b KQkq - 1 1',
				'g2g3' => 'rnbqkbnr/pppppppp/8/8/8/6P1/PPPPPP1P/RNBQKBNR b KQkq - 0 1',
				'g2g4' => 'rnbqkbnr/pppppppp/8/8/6P1/8/PPPPPP1P/RNBQKBNR b KQkq g3 0 1',
				'h2h3' => 'rnbqkbnr/pppppppp/8/8/8/7P/PPPPPPP1/RNBQKBNR b KQkq - 0 1',
				'h2h4' => 'rnbqkbnr/pppppppp/8/8/7P/8/PPPPPPP1/RNBQKBNR b KQkq h3 0 1',
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/N7/PPPPPPPP/R1BQKBNR b KQkq - 1 1' => {
			eco => 'A00h',
                        # TRANSLATORS: 1. Na3

			variation => N__('Durkin'),
			moves => {
			},
		},
		'rnbqkbnr/pppppppp/8/8/8/P7/1PPPPPPP/RNBQKBNR b KQkq - 0 1' => {
			eco => 'A00f',
                        # TRANSLATORS: 1. a3

			variation => N__('Anderssen Opening'),
			moves => {
				'e7e5' => 'rnbqkbnr/pppp1ppp/8/4p3/8/P7/1PPPPPPP/RNBQKBNR w KQkq e6 0 2',
			},
		},
		'rnbqkbnr/pppppppp/8/8/P7/8/1PPPPPPP/RNBQKBNR b KQkq a3 0 1' => {
			eco => 'A00e',
                        # TRANSLATORS: 1. a4

			variation => N__('Ware Opening'),
			moves => {
			},
		},
		'rnbqnrk1/pp2bppp/3p4/2pPp3/2P1P3/2N3P1/PP3PBP/R1BQK1NR w KQ - 3 8' => {
			eco => 'A56t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e5 4. Nc3 d6 5. e4 Be7 6. g3 O-O 7. Bg2 Ne8

			variation => N__('Benoni: Czech, 5. e4 Be7 6. g3 O-O 7. Bg2 Ne8'),
			moves => {
			},
		},
		'rnbqr1k1/1p3pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1Q2PP/R3KBNR w KQ - 1 12' => {
			eco => 'A65y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Re8

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Re8'),
			moves => {
				'g1e2' => 'rnbqr1k1/1p3pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1QN1PP/R3KB1R b KQ - 2 12',
			},
		},
		'rnbqr1k1/1p3pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1QN1PP/R3KB1R b KQ - 2 12' => {
			eco => 'A65y',
                        # TRANSLATORS:

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 a6 11. a4 Re8'),
			moves => {
				'b8d7' => 'r1bqr1k1/1p1n1pb1/p2p1npp/2pP4/P3P3/2N1BP2/1P1QN1PP/R3KB1R w KQ - 3 13',
			},
		},
		'rnbqr1k1/1p3pbp/p2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 0 11' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2 a6

			variation => N__('Benoni: Classical, Main Line, 10. Nd2 a6'),
			moves => {
			},
		},
		'rnbqr1k1/1p3pbp/p2p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQK2R w KQ - 0 11' => {
			eco => 'A69d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. Nd2 a6

			variation => N__('Benoni: Four Pawns, Main Line, 10. Nd2 a6'),
			moves => {
			},
		},
		'rnbqr1k1/p4pbp/3p1np1/1NpP4/4P3/3B1N1P/PP3PP1/R1BQ1RK1 b - - 2 11' => {
			eco => 'A70x',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5 10.Nxb5 Re8'),
			moves => {
				'f6e4' => 'rnbqr1k1/p4pbp/3p2p1/1NpP4/4n3/3B1N1P/PP3PP1/R1BQ1RK1 w - - 0 12',
			},
		},
		'rnbqr1k1/p4pbp/3p1np1/1NpP4/4P3/3B1N1P/PP3PP1/R1BQK2R w KQ - 1 11' => {
			eco => 'A70x',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 b5 10. Nxb5 Re8

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5 10.Nxb5 Re8'),
			moves => {
				'e1g1' => 'rnbqr1k1/p4pbp/3p1np1/1NpP4/4P3/3B1N1P/PP3PP1/R1BQ1RK1 b - - 2 11',
			},
		},
		'rnbqr1k1/p4pbp/3p2p1/1NpP4/4n3/3B1N1P/PP3PP1/R1BQ1RK1 w - - 0 12' => {
			eco => 'A70y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 b5 10. Nxb5 Re8 11. O-O Nxe4

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 b5 10.Nxb5 Re8 11.O-O Nxe4'),
			moves => {
			},
		},
		'rnbqr1k1/pp2bpp1/2p2n1p/3p4/3P3B/2NBP3/PPQ2PPP/R3K1NR w KQ - 4 10' => {
			eco => 'D36g',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. cxd5 exd5 5. Bg5 c6 6. Qc2 Be7 7. e3 h6 8. Bh4 O-O 9. Bd3 Re8

			variation => N__('QGD: Exchange, 6. Qc2 Be7 7. e3 h6'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pb1/3p1npp/2pP4/4P3/2N1BP2/PP1Q2PP/R3KBNR w KQ - 4 11' => {
			eco => 'A65u',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f3 Bg7 8. Bg5 h6 9. Be3 O-O 10. Qd2 Re8

			variation => N__('Benoni: Sämisch, 8. Bg5 h6 9. Be3 O-O 10. Qd2 Re8'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/2pp1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1R1K b - - 1 10' => {
			eco => 'E94c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4 8. Nxd4 Re8 9. f3 c6 10. Kh1

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3 c6 10. Kh1'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/2pp1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1RK1 w - - 0 10' => {
			eco => 'E94c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4 8. Nxd4 Re8 9. f3 c6

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3 c6'),
			moves => {
				'g1h1' => 'rnbqr1k1/pp3pbp/2pp1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1R1K b - - 1 10',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP2B1/2P5/2N2N2/PP1QBPPP/R3K2R b KQ - 2 10' => {
			eco => 'E75i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2 exd5 9. exd5 Re8 10. Nf3

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2 exd5 9.exd5 Re8 10. Nf3'),
			moves => {
				'c8g4' => 'rn1qr1k1/pp3pbp/3p1np1/2pP2B1/2P3b1/2N2N2/PP1QBPPP/R3K2R w KQ - 3 11',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP2B1/2P5/2N5/PP1QBPPP/R3K1NR w KQ - 1 10' => {
			eco => 'E75i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Be2 O-O 6. Bg5 c5 7. d5 e6 8. Qd2 exd5 9. exd5 Re8

			variation => N__('King\'s Indian: Averbakh, 7. d5 e6 8. Qd2 exd5 9.exd5 Re8'),
			moves => {
				'g1f3' => 'rnbqr1k1/pp3pbp/3p1np1/2pP2B1/2P5/2N2N2/PP1QBPPP/R3K2R b KQ - 2 10',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/2P5/2NB1N1P/PP3PP1/R1BQK2R w KQ - 1 10' => {
			eco => 'E90y',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 exd5 9. exd5 Re8+

			variation => N__('King\'s Indian: 5. Nf3 O-O 6. h3 c5 7. d5 e6 8. Bd3 exd5 9.exd5 Re8'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 6 10' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8

			variation => N__('Benoni: Classical, Main Line'),
			moves => {
				'd1c2' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 7 10',
				'f3d2' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 7 10',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2N2N2/PPQ1BPPP/R1B2RK1 b - - 7 10' => {
			eco => 'A76',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Qc2

			variation => N__('Benoni: Classical, Main Line, 10. Qc2'),
			moves => {
				'b8a6' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N2N2/PPQ1BPPP/R1B2RK1 w - - 8 11',
				'c8g4' => 'rn1qr1k1/pp3pbp/3p1np1/2pP4/4P1b1/2N2N2/PPQ1BPPP/R1B2RK1 w - - 8 11',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 7 10' => {
			eco => 'A77',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. Be2 O-O 9. O-O Re8 10. Nd2

			variation => N__('Benoni: Classical, Main Line, 10. Nd2'),
			moves => {
				'a7a6' => 'rnbqr1k1/1p3pbp/p2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 0 11',
				'b8a6' => 'r1bqr1k1/pp3pbp/n2p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 8 11',
				'b8d7' => 'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/4P3/2N5/PP1NBPPP/R1BQ1RK1 w - - 8 11',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQ1RK1 b - - 4 10' => {
			eco => 'A70n',
                        # TRANSLATORS:

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 Re8'),
			moves => {
				'c5c4' => 'rnbqr1k1/pp3pbp/3p1np1/3P4/2p1P3/2NB1N1P/PP3PP1/R1BQ1RK1 w - - 0 11',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQK2R w KQ - 3 10' => {
			eco => 'A70n',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 Re8

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 Re8'),
			moves => {
				'e1g1' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4P3/2NB1N1P/PP3PP1/R1BQ1RK1 b - - 4 10',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP2B1PP/R1BQ1RK1 b - - 6 10' => {
			eco => 'A69b',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. O-O

			variation => N__('Benoni: Four Pawns, Main Line, 10.O-O'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP2B1PP/R1BQK2R w KQ - 5 10' => {
			eco => 'A69a',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8

			variation => N__('Benoni: Four Pawns, Main Line'),
			moves => {
				'e1g1' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4PP2/2N2N2/PP2B1PP/R1BQ1RK1 b - - 6 10',
				'e4e5' => 'rnbqr1k1/pp3pbp/3p1np1/2pPP3/5P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 10',
				'f3d2' => 'rnbqr1k1/pp3pbp/3p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQK2R b KQ - 6 10',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQK2R b KQ - 6 10' => {
			eco => 'A69c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. Nd2

			variation => N__('Benoni: Four Pawns, Main Line, 10. Nd2'),
			moves => {
				'a7a6' => 'rnbqr1k1/1p3pbp/p2p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQK2R w KQ - 0 11',
				'b8d7' => 'r1bqr1k1/pp1n1pbp/3p1np1/2pP4/4PP2/2N5/PP1NB1PP/R1BQK2R w KQ - 7 11',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pP4/8/2N2NP1/PP2PPBP/R1BQ1RK1 w - - 5 10' => {
			eco => 'A62',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. Nf3 g6 7. g3 Bg7 8. Bg2 O-O 9. O-O Re8

			variation => N__('Benoni: Fianchetto, 9.O-O Re8'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/2pPP3/5P2/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 10' => {
			eco => 'A69f',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5'),
			moves => {
				'd6e5' => 'rnbqr1k1/pp3pbp/5np1/2pPp3/5P2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 11',
			},
		},
		'rnbqr1k1/pp3pbp/3p1np1/3P4/2p1P3/2NB1N1P/PP3PP1/R1BQ1RK1 w - - 0 11' => {
			eco => 'A70o',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. Nf3 Bg7 8. h3 O-O 9. Bd3 Re8 10. O-O c4

			variation => N__('Benoni: Classical, 8. h3 O-O 9. Bd3 Re8 10.O-O c4'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/3ppnp1/2pP4/2P1P3/2N2N2/PP2BPPP/R1BQ1RK1 w - - 2 9' => {
			eco => 'E91i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. d5 e6 8. O-O Re8

			variation => N__('King\'s Indian: 6. Be2 c5 7. d5 e6 8.O-O Re8'),
			moves => {
				'f3d2' => 'rnbqr1k1/pp3pbp/3ppnp1/2pP4/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 3 9',
			},
		},
		'rnbqr1k1/pp3pbp/3ppnp1/2pP4/2P1P3/2N5/PP1NBPPP/R1BQ1RK1 b - - 3 9' => {
			eco => 'E91i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 c5 7. d5 e6 8. O-O Re8 9. Nd2

			variation => N__('King\'s Indian: 6. Be2 c5 7. d5 e6 8.O-O Re8 9. Nd2'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/4P1p1/2pP4/6n1/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 12' => {
			eco => 'A69h',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. e6

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. e6'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/5np1/2pPP3/8/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 11' => {
			eco => 'A69f',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5'),
			moves => {
				'f6g4' => 'rnbqr1k1/pp3pbp/6p1/2pPP3/6n1/2N2N2/PP2B1PP/R1BQK2R w KQ - 1 12',
			},
		},
		'rnbqr1k1/pp3pbp/5np1/2pPp3/5P2/2N2N2/PP2B1PP/R1BQK2R w KQ - 0 11' => {
			eco => 'A69f',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5'),
			moves => {
				'f4e5' => 'rnbqr1k1/pp3pbp/5np1/2pPP3/8/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 11',
			},
		},
		'rnbqr1k1/pp3pbp/6p1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R b KQ - 2 12' => {
			eco => 'A69i',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. Bg5

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5'),
			moves => {
				'd8b6' => 'rnb1r1k1/pp3pbp/1q4p1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R w KQ - 3 13',
				'f7f6' => 'rnbqr1k1/pp4bp/5pp1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R w KQ - 0 13',
			},
		},
		'rnbqr1k1/pp3pbp/6p1/2pPP3/6n1/2N2N2/PP2B1PP/R1BQ1RK1 b - - 2 12' => {
			eco => 'A69g',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. O-O

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12.O-O'),
			moves => {
			},
		},
		'rnbqr1k1/pp3pbp/6p1/2pPP3/6n1/2N2N2/PP2B1PP/R1BQK2R w KQ - 1 12' => {
			eco => 'A69f',
                        # TRANSLATORS:

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5'),
			moves => {
				'c1g5' => 'rnbqr1k1/pp3pbp/6p1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R b KQ - 2 12',
				'e1g1' => 'rnbqr1k1/pp3pbp/6p1/2pPP3/6n1/2N2N2/PP2B1PP/R1BQ1RK1 b - - 2 12',
				'e5e6' => 'rnbqr1k1/pp3pbp/4P1p1/2pP4/6n1/2N2N2/PP2B1PP/R1BQK2R b KQ - 0 12',
			},
		},
		'rnbqr1k1/pp4bp/5pp1/2pPP1B1/6n1/2N2N2/PP2B1PP/R2QK2R w KQ - 0 13' => {
			eco => 'A69j',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. Nf3 O-O 9. Be2 Re8 10. e5 dxe5 11. fxe5 Ng4 12. Bg5 f6

			variation => N__('Benoni: Four Pawns, Main Line, 10. e5: 12. Bg5 f6'),
			moves => {
			},
		},
		'rnbqr1k1/ppp2pbp/3p1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1RK1 b - - 0 9' => {
			eco => 'E94c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. O-O exd4 8. Nxd4 Re8 9. f3

			variation => N__('King\'s Indian: 7.O-O exd4 8.Nxd4 Re8 9. f3'),
			moves => {
				'b8c6' => 'r1bqr1k1/ppp2pbp/2np1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1RK1 w - - 1 10',
				'c7c6' => 'rnbqr1k1/pp3pbp/2pp1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1RK1 w - - 0 10',
			},
		},
		'rnbqr1k1/ppp2pbp/3p1np1/8/2PNP3/2N5/PP2BPPP/R1BQ1RK1 w - - 1 9' => {
			eco => 'E94b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 7.O-O exd4'),
			moves => {
				'f2f3' => 'rnbqr1k1/ppp2pbp/3p1np1/8/2PNP3/2N2P2/PP2B1PP/R1BQ1RK1 b - - 0 9',
			},
		},
		'rnbqr1k1/pppp1ppp/4pn2/8/1bPP4/2N1P3/PP2NPPP/R1BQKB1R w KQ - 3 6' => {
			eco => 'E46',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nc3 Bb4 4. e3 O-O 5. Ne2 Re8

			variation => N__('Nimzo-Indian: Reshevsky Variation'),
			moves => {
			},
		},
		'rnbr2k1/pp3pbp/2p2np1/4p1B1/2P1P3/2N2N2/PP2BPPP/R3K2R w KQ - 0 10' => {
			eco => 'E92d',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. dxe5 dxe5 8. Qxd8 Rxd8 9. Bg5 c6

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap, 9. Bg5 c6'),
			moves => {
			},
		},
		'rnbr2k1/ppp1qpp1/7p/3p4/3P4/1QP1PN2/P4PPP/R3KB1R w KQ - 2 12' => {
			eco => 'D57',
                        # TRANSLATORS: 1. d4 d5 2. c4 e6 3. Nc3 Nf6 4. Bg5 Be7 5. e3 O-O 6. Nf3 h6 7. Bh4 Ne4 8. Bxe7 Qxe7 9. cxd5 Nxc3 10. bxc3 exd5 11. Qb3 Rd8

			variation => N__('QGD: Lasker Defence, Main Line, 11. Qb3 Rd8'),
			moves => {
			},
		},
		'rnbr2k1/ppp2pbp/5np1/4p1B1/2P1P3/2N2N2/PP2BPPP/R3K2R b KQ - 1 9' => {
			eco => 'E92c',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 g6 3. Nc3 Bg7 4. e4 d6 5. Nf3 O-O 6. Be2 e5 7. dxe5 dxe5 8. Qxd8 Rxd8 9. Bg5

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap, 9. Bg5'),
			moves => {
				'c7c6' => 'rnbr2k1/pp3pbp/2p2np1/4p1B1/2P1P3/2N2N2/PP2BPPP/R3K2R w KQ - 0 10',
				'd8e8' => 'rnb1r1k1/ppp2pbp/5np1/4p1B1/2P1P3/2N2N2/PP2BPPP/R3K2R w KQ - 2 10',
			},
		},
		'rnbr2k1/ppp2pbp/5np1/4p3/2P1P3/2N2N2/PP2BPPP/R1B1K2R w KQ - 0 9' => {
			eco => 'E92b',
                        # TRANSLATORS:

			variation => N__('King\'s Indian: 6. Be2 e5 Queenswap'),
			moves => {
				'c1g5' => 'rnbr2k1/ppp2pbp/5np1/4p1B1/2P1P3/2N2N2/PP2BPPP/R3K2R b KQ - 1 9',
			},
		},
		'rnq1kb1r/pb1p1ppp/1p2pn2/2p5/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq c6 0 7' => {
			eco => 'E15t',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 Qc8'),
			moves => {
				'd4d5' => 'rnq1kb1r/pb1p1ppp/1p2pn2/2pP4/2P5/5NP1/PP2PPBP/RNBQ1RK1 b kq - 0 7',
			},
		},
		'rnq1kb1r/pb1p1ppp/1p2pn2/2pP4/2P5/5NP1/PP2PPBP/RNBQ1RK1 b kq - 0 7' => {
			eco => 'E15t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Qc8 6. O-O c5 7. d5

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 Qc8 6.O-O c5 7. d5'),
			moves => {
			},
		},
		'rnq1kb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 6' => {
			eco => 'E15t',
                        # TRANSLATORS:

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 Qc8'),
			moves => {
				'c7c5' => 'rnq1kb1r/pb1p1ppp/1p2pn2/2p5/2PP4/5NP1/PP2PPBP/RNBQ1RK1 w kq c6 0 7',
			},
		},
		'rnq1kb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQK2R w KQkq - 3 6' => {
			eco => 'E15t',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 e6 3. Nf3 b6 4. g3 Bb7 5. Bg2 Qc8

			variation => N__('Queen\'s Indian: 4. g3 Bb7 5. Bg2 Qc8'),
			moves => {
				'e1g1' => 'rnq1kb1r/pbpp1ppp/1p2pn2/8/2PP4/5NP1/PP2PPBP/RNBQ1RK1 b kq - 4 6',
			},
		},
		'rnq4r/pp1nkpbp/6p1/2pPp3/5P2/5N2/PP4PP/R1BQKB1R b KQ - 1 12' => {
			eco => 'A66',
                        # TRANSLATORS: 1. d4 Nf6 2. c4 c5 3. d5 e6 4. Nc3 exd5 5. cxd5 d6 6. e4 g6 7. f4 Bg7 8. e5 Nfd7 9. Nb5 dxe5 10. Nd6+ Ke7 11. Nxc8+ Qxc8 12. Nf3

			variation => N__('Benoni: Mikenas Attack, 9. Nb5 dxe5 10. Nd6+ Ke7 11.Nxc8+ Qxc8 12. Nf3'),
			moves => {
			},
		},
		'rnq4r/pp1nkpbp/6p1/2pPp3/5P2/8/PP4PP/R1BQKBNR w KQ - 0 12' => {
			eco => 'A66',
                        # TRANSLATORS:

			variation => N__('Benoni: Mikenas Attack, 9. Nb5 dxe5 10. Nd6+ Ke7 11.Nxc8+'),
			moves => {
				'g1f3' => 'rnq4r/pp1nkpbp/6p1/2pPp3/5P2/5N2/PP4PP/R1BQKB1R b KQ - 1 12',
			},
		},
		'rq2kb1r/1b1n1ppp/p3pn2/1pp5/3P4/1BN1PN2/PP2QPPP/R1BR2K1 w kq - 6 11' => {
			eco => 'D29m',
                        # TRANSLATORS: 1. d4 d5 2. c4 dxc4 3. Nf3 Nf6 4. e3 e6 5. Bxc4 c5 6. O-O a6 7. Qe2 b5 8. Bb3 Bb7 9. Rd1 Nbd7 10. Nc3 Qb8

			variation => N__('QGA: Classical, 8... Bb7 9. Rd1 Nbd7 10. Nc3 Qb8'),
			moves => {
			},
		},
		'rq3rk1/pb1pppbp/1pn2np1/8/2PNP3/2N1BP2/PP2B1PP/R2Q1RK1 w - - 1 11' => {
			eco => 'B38k',
                        # TRANSLATORS: 1. e4 c5 2. Nf3 Nc6 3. d4 cxd4 4. Nxd4 g6 5. c4 Bg7 6. Be3 Nf6 7. Nc3 O-O 8. Be2 b6 9. O-O Bb7 10. f3 Qb8

			variation => N__('Sicilian: Maroczy Bind, 7. Nc3 O-O 8. Be2 b6 9.O-O Bb7 10. f3 Qb8'),
			moves => {
			},
		},
	};
}

1;

=head1 NAME

Chess::Opening::ECO - ECO data

=head1 SYNOPSIS

	use Chess::Opening::ECO;

	$positions = Chess::Opening::ECO->positions;

=head1 DESCRIPTION

This module is a mere container for the ECO (Encyclopedia of Chess Openings)
data.  You should rather use one the high-level interface
L<Chess::Opening::Book::ECO> for accessing or browsing the data contained here.

=head1 METHODS

=over 4

=item B<positions>

This is the only method in this module.  You can call it as a class method
or simply as C<Chess::Opening::ECO::Data>.  It returns a reference to a hash
of opening positions.  The keys of this hash are the positions in
Forsyth-Edwards Notation (FEN).  The values have four items:

=over 8

=item B<eco>

The ECO code of the position.  Note that these codes contain the so-called
"Scid Extensions to ECO", see L<http://scidvspc.sourceforge.net/doc/ECO.htm>.
If you are only interested in the pure ECO code, strip off everything after
the third character.

=item B<variation>

The name of the variation (like "Sicilian", "El Greco", and so on).  Note that
this is always the English name of the variation.  Use L<Chess::Opening::ECO>
if you want to have them translated into the user's language.

=item B<parent>

The parent position as FEN or C<undef> for the starting position.

=item B<moves>

A reference to a possibly empty array of moves that lead from this position
to other opening positions.

=back

=back

=head1 COPYRIGHT

Copyright (C) 2019 Guido Flohr <guido.flohr@cantanea.com>,
all rights reserved.

=head1 SEE ALSO

L<Chess::Opening::Book::ECO>, L<Chess::Opening::Book::Entry>, perl(1)
